/*****************************************************************************
* Copyright (c) 2014 Intelligent Light. All rights reserved.
* Work partially supported by DOE Grant SC0007548.
* FieldView XDB Export is provided expressly for use within VisIt.
* All other uses are strictly forbidden.
*****************************************************************************/
#include <FieldViewXDBPluginInfo.h>
#include <avtFieldViewXDBFileFormat.h>
#include <avtMTMDFileFormatInterface.h>
#include <avtGenericDatabase.h>

// ****************************************************************************
//  Method:  FieldViewXDBCommonPluginInfo::GetDatabaseType
//
//  Purpose:
//    Returns the type of a FieldViewXDB database.
//
//  Programmer:  generated by xml2info
//  Creation:    omitted
//
// ****************************************************************************
DatabaseType
FieldViewXDBCommonPluginInfo::GetDatabaseType()
{
    return DB_TYPE_MTMD;
}

// ****************************************************************************
//  Method: FieldViewXDBCommonPluginInfo::SetupDatabase
//
//  Purpose:
//      Sets up a FieldViewXDB database.
//
//  Arguments:
//      list    A list of file names.
//      nList   The number of timesteps in list.
//      nBlocks The number of blocks in the list.
//
//  Returns:    A FieldViewXDB database from list.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************
avtDatabase *
FieldViewXDBCommonPluginInfo::SetupDatabase(const char *const *list,
                                   int nList, int nBlock)
{
    // ignore any nBlocks past 1
    int nTimestepGroups = nList / nBlock;
    avtMTMDFileFormat **ffl = new avtMTMDFileFormat*[nTimestepGroups];
    for (int i = 0 ; i < nTimestepGroups ; i++)
    {
        ffl[i] = new avtFieldViewXDBFileFormat(list[i*nBlock]);
    }
    avtMTMDFileFormatInterface *inter 
           = new avtMTMDFileFormatInterface(ffl, nTimestepGroups);
    return new avtGenericDatabase(inter);
}
// ****************************************************************************
//  Method: FieldViewXDBCommonPluginInfo::GetLicense
//
//  Purpose: Get the copyright for the library.
//
//  Programmer: Brad Whitlock
//  Creation:   Tue Apr  8 16:07:39 PDT 2014
//
// ****************************************************************************
#include <avtFieldViewXDBWriterInternal.h>

std::string
FieldViewXDBCommonPluginInfo::GetLicense() const
{
    return avtFieldViewXDBWriterInternal::GetCopyright();
}

