/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

// ************************************************************************* //
//                       avtWavefrontOBJWriter.C                             //
// ************************************************************************* //

#include <avtWavefrontOBJWriter.h>

#include <vector>

#include <vtkAlgorithm.h>
#include <vtkAppendPolyData.h>
#include <vtkDataSet.h>
#include <vtkGeometryFilter.h>
#include <vtkDataSetWriter.h>
#include <vtkPolyDataReader.h>
#include <vtkPolyDataWriter.h>
#include <vtkCharArray.h>
#include <vtkPolyData.h>
#include <vtkTriangleFilter.h>
#include <avtDatabaseMetaData.h>
#include <DBOptionsAttributes.h>
#include <vtkOBJWriter.h>

#include <avtDatabaseMetaData.h>
#include <DebugStream.h>
#include <avtParallel.h>
#ifdef PARALLEL
  #include <mpi.h>
#endif

using     std::string;
using     std::vector;


// ****************************************************************************
//  Method: avtWavefrontOBJWriter constructor
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 16 08:33:21 PDT 2013
//
// ****************************************************************************

avtWavefrontOBJWriter::avtWavefrontOBJWriter(void)
{
}

// ****************************************************************************
//  Method: avtWavefrontOBJWriter::OpenFile
//
//  Purpose:
//      Does no actual work.  Just records the stem name for the files.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 16 08:33:21 PDT 2013
//
// ****************************************************************************

void
avtWavefrontOBJWriter::OpenFile(const string &stemname, int numblocks)
{
    stem = stemname;
    pds.clear();
}


// ****************************************************************************
//  Method: avtWavefrontOBJWriter::WriteHeaders
//
//  Purpose:
//      Writes out a VisIt file to tie the WavefrontOBJ files together.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 16 08:33:21 PDT 2013
//
// ****************************************************************************

void
avtWavefrontOBJWriter::WriteHeaders(const avtDatabaseMetaData *md,
                           vector<string> &scalars, vector<string> &vectors,
                           vector<string> &materials)
{
    // WRITE OUT HEADER INFO
}


// ****************************************************************************
//  Method: avtWavefrontOBJWriter::WriteChunk
//
//  Purpose:
//      This writes out one chunk of an avtDataset.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 16 08:33:21 PDT 2013
//
// ****************************************************************************

void
avtWavefrontOBJWriter::WriteChunk(vtkDataSet *ds, int chunk)
{
    vtkPolyData *pd = vtkPolyData::SafeDownCast(ds);
    vtkTriangleFilter *tri = vtkTriangleFilter::New();
    if(pd == NULL)
    {
        vtkGeometryFilter *geom = vtkGeometryFilter::New();
        geom->SetInputData(ds);

        tri->SetInputConnection(geom->GetOutputPort());        
        tri->Update();
        pd = tri->GetOutput();
        pd->Register(NULL);
        geom->Delete();
    }
    else
    {
        tri->SetInputData(pd);        
        tri->Update();
        pd = tri->GetOutput();
        pd->Register(NULL);
    }

    tri->Delete();

    pds.push_back(pd);

}

//****************************************************************************
// Method:  avtWavefrontOBJWriter::SendPolyDataToRank0
//
// Purpose:
//   Move all the data to rank0
//
// Programmer:  Dave Pugmire
// Creation:    April 15, 2013
//
// Modifications:
//
//****************************************************************************

#ifdef PARALLEL
void
avtWavefrontOBJWriter::SendPolyDataToRank0()
{
    int *inA = new int[PAR_Size()], *outA = new int[PAR_Size()];
    for (int i = 0; i < PAR_Size(); i++)
        inA[i] = outA[i] = 0;

    if (PAR_Rank() != 0)
    {
        vtkPolyDataWriter *writer = NULL;
        int len = 0;
        
        if (pds.size() == 0)
            len = 0;
        else
        {
            writer = vtkPolyDataWriter::New();
            writer->WriteToOutputStringOn();
            writer->SetFileTypeToBinary();
            if (pds.size() == 1)
                writer->SetInputData(pds[0]);
            else
            {
                vtkAppendPolyData *f = vtkAppendPolyData::New();
                for(size_t i = 0; i < pds.size(); ++i)
                    f->AddInputData(pds[i]);
                
                writer->SetInputConnection(f->GetOutputPort());
            }
            
            writer->Write();
            len = writer->GetOutputStringLength();

            for(size_t i = 0; i < pds.size(); ++i)
                pds[i]->Delete();
            pds.clear();
        }
        
        inA[PAR_Rank()] = len;
        MPI_Reduce(inA, outA, PAR_Size(), MPI_INT, MPI_SUM, 0, VISIT_MPI_COMM);
        if (len > 0)
        {
            char *data = writer->GetOutputString();
            MPI_Send(data, len, MPI_CHAR, 0, 0, VISIT_MPI_COMM);
            writer->Delete();
        }
    }
    else
    {
        MPI_Reduce(inA, outA, PAR_Size(), MPI_INT, MPI_SUM, 0, VISIT_MPI_COMM);
        for (int i = 1; i < PAR_Size(); i++)
        {
            if (outA[i] > 0)
            {
                char *data = new char[outA[i]];
                MPI_Status stat;
                MPI_Recv(data, outA[i], MPI_CHAR, i, 0, VISIT_MPI_COMM, &stat);

                vtkPolyDataReader *rdr = vtkPolyDataReader::New();
                rdr->ReadFromInputStringOn();
                vtkCharArray *charArray = vtkCharArray::New();
                charArray->SetArray(data, outA[i], 1);
                rdr->SetInputArray(charArray);
                rdr->Update();
                vtkPolyData *pd = rdr->GetOutput();
                pd->Register(NULL);
                pds.push_back(pd);

                delete [] data;
                rdr->Delete();
                charArray->Delete();
            }
        }
    }

    delete [] inA;
    delete [] outA;
}
#endif

// ****************************************************************************
//  Method: avtWavefrontOBJWriter::CloseFile
//
//  Purpose:
//      Closes the file.  This does nothing in this case.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 16 08:33:21 PDT 2013
//
// ****************************************************************************

void
avtWavefrontOBJWriter::CloseFile(void)
{
#ifdef PARALLEL
    SendPolyDataToRank0();
    if (PAR_Rank() != 0)
        return;
#endif

    std::string filename(stem + ".obj");
    vtkOBJWriter *writer = vtkOBJWriter::New();
    
    writer->SetFileName(filename.c_str());

    if(pds.size() == 1)
    {
        writer->SetInputData(pds[0]);
        writer->Update();
    }
    else if(pds.size() > 1)
    {
        vtkAppendPolyData *f = vtkAppendPolyData::New();
        for(size_t i = 0; i < pds.size(); ++i)
            f->AddInputData(pds[i]);

        writer->SetInputConnection(f->GetOutputPort());
        writer->Update();

        f->Delete();
    }
    else
    {
        debug4 << "avtOBJWriter::CloseFile: No data to write" << endl;
    }

    for(size_t i = 0; i < pds.size(); ++i)
        pds[i]->Delete();
    pds.clear();
    writer->Delete();
}
