/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

// ************************************************************************* //
//  File: DataBinningViewerPluginInfo.C
// ************************************************************************* //

#include <DataBinningPluginInfo.h>
#include <QApplication>
#include <DataBinningAttributes.h>

// ****************************************************************************
//  Function:  GetViewerInfo
//
//  Purpose:
//    Return a new ViewerPluginInfo for the DataBinning operator.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************
extern "C" OP_EXPORT ViewerOperatorPluginInfo* DataBinning_GetViewerInfo()
{
    DataBinningViewerPluginInfo::InitializeGlobalObjects();
    return new DataBinningViewerPluginInfo;
}

//
// Storage for static data elements.
//
DataBinningAttributes *DataBinningViewerPluginInfo::clientAtts = NULL;
DataBinningAttributes *DataBinningViewerPluginInfo::defaultAtts = NULL;

// ****************************************************************************
//  Method:  DataBinningViewerPluginInfo::InitializeGlobalObjects
//
//  Purpose:
//    Initialize the operator atts.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************
void
DataBinningViewerPluginInfo::InitializeGlobalObjects()
{
    DataBinningViewerPluginInfo::clientAtts  = new DataBinningAttributes;
    DataBinningViewerPluginInfo::defaultAtts = new DataBinningAttributes;
}

// ****************************************************************************
//  Method: DataBinningViewerPluginInfo::GetClientAtts
//
//  Purpose:
//    Return a pointer to the viewer client attributes.
//
//  Returns:    A pointer to the viewer client attributes.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

AttributeSubject *
DataBinningViewerPluginInfo::GetClientAtts()
{
    return clientAtts;
}

// ****************************************************************************
//  Method: DataBinningViewerPluginInfo::GetDefaultAtts
//
//  Purpose:
//    Return a pointer to the viewer default attributes.
//
//  Returns:    A pointer to the viewer default attributes.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

AttributeSubject *
DataBinningViewerPluginInfo::GetDefaultAtts()
{
    return defaultAtts;
}

// ****************************************************************************
//  Method: DataBinningViewerPluginInfo::SetClientAtts
//
//  Purpose:
//    Set the viewer client attributes.
//
//  Arguments:
//    atts      A pointer to the new client attributes.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

void
DataBinningViewerPluginInfo::SetClientAtts(AttributeSubject *atts)
{
    *clientAtts = *(DataBinningAttributes *)atts;
    clientAtts->Notify();
}

// ****************************************************************************
//  Method: DataBinningViewerPluginInfo::GetClientAtts
//
//  Purpose:
//    Get the viewer client attributes.
//
//  Arguments:
//    atts      A pointer to return the client default attributes in.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

void
DataBinningViewerPluginInfo::GetClientAtts(AttributeSubject *atts)
{
    *(DataBinningAttributes *)atts = *clientAtts;
}

// ****************************************************************************
//  Method: DataBinningViewerPluginInfo::InitializeOperatorAtts
//
//  Purpose:
//    Initialize the operator attributes to the default attributes.
//
//  Arguments:
//    atts        The attribute subject to initialize.
//    plot        The viewer plot that owns the operator.
//    fromDefault True to initialize the attributes from the defaults. False
//                to initialize from the current attributes.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

void
DataBinningViewerPluginInfo::InitializeOperatorAtts(AttributeSubject *atts,
                                              const ViewerPlot *plot,
                                              const bool fromDefault)
{
    if (fromDefault)
        *(DataBinningAttributes*)atts = *defaultAtts;
    else
        *(DataBinningAttributes*)atts = *clientAtts;

    UpdateOperatorAtts(atts, plot);
}

// ****************************************************************************
//  Method: DataBinningViewerPluginInfo::UpdateOperatorAtts
//
//  Purpose:
//    Update the operator attributes to the default attributes.
//
//  Arguments:
//    atts        The attribute subject to update.
//    plot        The viewer plot that owns the operator.
//    fromDefault True to update the attributes from the defaults. False
//                to update from the current attributes.
//
//  Programmer: Brad Whitlock
//  Creation:   Wed Jun 27 10:10:44 PDT 2012
//
//  Modifications:
//    Brad Whitlock, Wed Jun 27 10:13:24 PDT 2012
//    I transplanted this code from the old version of 
//    DataBinningAttributes::CopyAttributes since we should just set the 
//    operator dimensions when we create the operator attributes.
//
// ****************************************************************************

#include <ViewerPlot.h>

void
DataBinningViewerPluginInfo::UpdateOperatorAtts(AttributeSubject *atts,
    const ViewerPlot *plot)
{
    DataBinningAttributes *binning = (DataBinningAttributes *)atts;

    // Set the number of dimensions based on the variable name.
    const char *varname = plot->GetVariableName().c_str();
    if (strstr(varname, "operators/DataBinning/1D") != NULL)
        binning->SetNumDimensions(DataBinningAttributes::One);
    else if (strstr(varname, "operators/DataBinning/2D") != NULL)
        binning->SetNumDimensions(DataBinningAttributes::Two);
    else if (strstr(varname, "operators/DataBinning/3D") != NULL)
        binning->SetNumDimensions(DataBinningAttributes::Three);
}

// ****************************************************************************
//  Method: DataBinningViewerPluginInfo::GetOperatorVarDescription
//
//  Purpose:
//    Return the operator  variable description.
//
//  Arguments:
//    atts        The current attributes.
//    plot        The viewer plot that owns the operator.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

std::string
DataBinningViewerPluginInfo::GetOperatorVarDescription(AttributeSubject *atts,
                                              const ViewerPlot *plot)
{
    return std::string("");
}

// ****************************************************************************
//  Method: DataBinningViewerPluginInfo::GetMenuName
//
//  Purpose:
//    Return a pointer to the name to use in the viewer menus.
//
//  Returns:    A pointer to the name to use in the viewer menus.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

QString *
DataBinningViewerPluginInfo::GetMenuName() const
{
    return new QString(qApp->translate("OperatorNames", "DataBinning"));
}

