/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include "QvisDelaunayWindow.h"

#include <DelaunayAttributes.h>
#include <ViewerProxy.h>

#include <QCheckBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QSpinBox>
#include <QButtonGroup>
#include <QRadioButton>
#include <QvisColorTableButton.h>
#include <QvisOpacitySlider.h>
#include <QvisColorButton.h>
#include <QvisLineStyleWidget.h>
#include <QvisLineWidthWidget.h>
#include <QvisVariableButton.h>

#include <stdio.h>
#include <string>

using std::string;

// ****************************************************************************
// Method: QvisDelaunayWindow::QvisDelaunayWindow
//
// Purpose: 
//   Constructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisDelaunayWindow::QvisDelaunayWindow(const int type,
                         DelaunayAttributes *subj,
                         const QString &caption,
                         const QString &shortName,
                         QvisNotepadArea *notepad)
    : QvisOperatorWindow(type,subj, caption, shortName, notepad)
{
    atts = subj;
}


// ****************************************************************************
// Method: QvisDelaunayWindow::~QvisDelaunayWindow
//
// Purpose: 
//   Destructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisDelaunayWindow::~QvisDelaunayWindow()
{
}


// ****************************************************************************
// Method: QvisDelaunayWindow::CreateWindowContents
//
// Purpose: 
//   Creates the widgets for the window.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisDelaunayWindow::CreateWindowContents()
{
    QGridLayout *mainLayout = new QGridLayout(0);
    topLayout->addLayout(mainLayout);

    dimensionLabel = new QLabel(tr("Dimension"), central);
    mainLayout->addWidget(dimensionLabel,0,0);
    dimension = new QWidget(central);
    dimensionButtonGroup= new QButtonGroup(dimension);
    QHBoxLayout *dimensionLayout = new QHBoxLayout(dimension);
    dimensionLayout->setMargin(0);
    dimensionLayout->setSpacing(10);
    QRadioButton *dimensionDimensionAutomatic = new QRadioButton(tr("Automatic"), dimension);
    dimensionButtonGroup->addButton(dimensionDimensionAutomatic,0);
    dimensionLayout->addWidget(dimensionDimensionAutomatic);
    QRadioButton *dimensionDimensionTriangulation = new QRadioButton(tr("Triangulation"), dimension);
    dimensionButtonGroup->addButton(dimensionDimensionTriangulation,1);
    dimensionLayout->addWidget(dimensionDimensionTriangulation);
    QRadioButton *dimensionDimensionTetrahedralization = new QRadioButton(tr("Tetrahedralization"), dimension);
    dimensionButtonGroup->addButton(dimensionDimensionTetrahedralization,2);
    dimensionLayout->addWidget(dimensionDimensionTetrahedralization);
    connect(dimensionButtonGroup, SIGNAL(buttonClicked(int)),
            this, SLOT(dimensionChanged(int)));
    mainLayout->addWidget(dimension, 0,1);

}


// ****************************************************************************
// Method: QvisDelaunayWindow::UpdateWindow
//
// Purpose: 
//   Updates the widgets in the window when the subject changes.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisDelaunayWindow::UpdateWindow(bool doAll)
{

    for(int i = 0; i < atts->NumAttributes(); ++i)
    {
        if(!doAll)
        {
            if(!atts->IsSelected(i))
            {
                continue;
            }
        }

        switch(i)
        {
          case DelaunayAttributes::ID_dimension:
            dimensionButtonGroup->blockSignals(true);
            if(dimensionButtonGroup->button((int)atts->GetDimension()) != 0)
                dimensionButtonGroup->button((int)atts->GetDimension())->setChecked(true);
            dimensionButtonGroup->blockSignals(false);
            break;
        }
    }
}


// ****************************************************************************
// Method: QvisDelaunayWindow::GetCurrentValues
//
// Purpose: 
//   Gets values from certain widgets and stores them in the subject.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisDelaunayWindow::GetCurrentValues(int which_widget)
{
}


//
// Qt Slot functions
//


void
QvisDelaunayWindow::dimensionChanged(int val)
{
    if(val != atts->GetDimension())
    {
        atts->SetDimension(DelaunayAttributes::Dimension(val));
        SetUpdate(false);
        Apply();
    }
}


