/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include "QvisExtractPointFunction2DWindow.h"

#include <ExtractPointFunction2DAttributes.h>
#include <ViewerProxy.h>

#include <QCheckBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QSpinBox>
#include <QButtonGroup>
#include <QRadioButton>
#include <QvisColorTableButton.h>
#include <QvisOpacitySlider.h>
#include <QvisColorButton.h>
#include <QvisLineStyleWidget.h>
#include <QvisLineWidthWidget.h>
#include <QvisVariableButton.h>

#include <stdio.h>
#include <string>

using std::string;

// ****************************************************************************
// Method: QvisExtractPointFunction2DWindow::QvisExtractPointFunction2DWindow
//
// Purpose: 
//   Constructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisExtractPointFunction2DWindow::QvisExtractPointFunction2DWindow(const int type,
                         ExtractPointFunction2DAttributes *subj,
                         const QString &caption,
                         const QString &shortName,
                         QvisNotepadArea *notepad)
    : QvisOperatorWindow(type,subj, caption, shortName, notepad)
{
    atts = subj;
}


// ****************************************************************************
// Method: QvisExtractPointFunction2DWindow::~QvisExtractPointFunction2DWindow
//
// Purpose: 
//   Destructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisExtractPointFunction2DWindow::~QvisExtractPointFunction2DWindow()
{
}


// ****************************************************************************
// Method: QvisExtractPointFunction2DWindow::CreateWindowContents
//
// Purpose: 
//   Creates the widgets for the window.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisExtractPointFunction2DWindow::CreateWindowContents()
{
    QGridLayout *mainLayout = new QGridLayout(0);
    topLayout->addLayout(mainLayout);

    ILabel = new QLabel(tr("I"), central);
    mainLayout->addWidget(ILabel,0,0);
    I = new QLineEdit(central);
    connect(I, SIGNAL(returnPressed()),
            this, SLOT(IProcessText()));
    mainLayout->addWidget(I, 0,1);

    JLabel = new QLabel(tr("J"), central);
    mainLayout->addWidget(JLabel,1,0);
    J = new QLineEdit(central);
    connect(J, SIGNAL(returnPressed()),
            this, SLOT(JProcessText()));
    mainLayout->addWidget(J, 1,1);

}


// ****************************************************************************
// Method: QvisExtractPointFunction2DWindow::UpdateWindow
//
// Purpose: 
//   Updates the widgets in the window when the subject changes.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisExtractPointFunction2DWindow::UpdateWindow(bool doAll)
{

    for(int i = 0; i < atts->NumAttributes(); ++i)
    {
        if(!doAll)
        {
            if(!atts->IsSelected(i))
            {
                continue;
            }
        }

        switch(i)
        {
          case ExtractPointFunction2DAttributes::ID_I:
            I->setText(IntToQString(atts->GetI()));
            break;
          case ExtractPointFunction2DAttributes::ID_J:
            J->setText(IntToQString(atts->GetJ()));
            break;
        }
    }
}


// ****************************************************************************
// Method: QvisExtractPointFunction2DWindow::GetCurrentValues
//
// Purpose: 
//   Gets values from certain widgets and stores them in the subject.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisExtractPointFunction2DWindow::GetCurrentValues(int which_widget)
{
    bool doAll = (which_widget == -1);

    // Do I
    if(which_widget == ExtractPointFunction2DAttributes::ID_I || doAll)
    {
        int val;
        if(LineEditGetInt(I, val))
            atts->SetI(val);
        else
        {
            ResettingError(tr("I"),
                IntToQString(atts->GetI()));
            atts->SetI(atts->GetI());
        }
    }

    // Do J
    if(which_widget == ExtractPointFunction2DAttributes::ID_J || doAll)
    {
        int val;
        if(LineEditGetInt(J, val))
            atts->SetJ(val);
        else
        {
            ResettingError(tr("J"),
                IntToQString(atts->GetJ()));
            atts->SetJ(atts->GetJ());
        }
    }

}


//
// Qt Slot functions
//


void
QvisExtractPointFunction2DWindow::IProcessText()
{
    GetCurrentValues(ExtractPointFunction2DAttributes::ID_I);
    Apply();
}


void
QvisExtractPointFunction2DWindow::JProcessText()
{
    GetCurrentValues(ExtractPointFunction2DAttributes::ID_J);
    Apply();
}


