/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include <PyExtremeValueAnalysisAttributes.h>
#include <ObserverToCallback.h>
#include <stdio.h>
#include <snprintf.h>

// ****************************************************************************
// Module: PyExtremeValueAnalysisAttributes
//
// Purpose: 
//   Attributes for ExtremeValueAnalysis operator
//
// Note:       Autogenerated by xml2python. Do not modify by hand!
//
// Programmer: xml2python
// Creation:   omitted
//
// ****************************************************************************

//
// This struct contains the Python type information and a ExtremeValueAnalysisAttributes.
//
struct ExtremeValueAnalysisAttributesObject
{
    PyObject_HEAD
    ExtremeValueAnalysisAttributes *data;
    bool        owns;
    PyObject   *parent;
};

//
// Internal prototypes
//
static PyObject *NewExtremeValueAnalysisAttributes(int);

std::string
PyExtremeValueAnalysisAttributes_ToString(const ExtremeValueAnalysisAttributes *atts, const char *prefix)
{
    std::string str;
    char tmpStr[1000];

    SNPRINTF(tmpStr, 1000, "%sdataYearBegin = %d\n", prefix, atts->GetDataYearBegin());
    str += tmpStr;
    if(atts->GetDataAnalysisYearRangeEnabled())
        SNPRINTF(tmpStr, 1000, "%sdataAnalysisYearRangeEnabled = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sdataAnalysisYearRangeEnabled = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sdataAnalysisYear1 = %d\n", prefix, atts->GetDataAnalysisYear1());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sdataAnalysisYear2 = %d\n", prefix, atts->GetDataAnalysisYear2());
    str += tmpStr;
    if(atts->GetEnsemble())
        SNPRINTF(tmpStr, 1000, "%sensemble = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sensemble = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%snumEnsembles = %d\n", prefix, atts->GetNumEnsembles());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sdataScaling = %g\n", prefix, atts->GetDataScaling());
    str += tmpStr;
    const char *extremeMethod_names = "MINIMA, MAXIMA";
    switch (atts->GetExtremeMethod())
    {
      case ExtremeValueAnalysisAttributes::MINIMA:
          SNPRINTF(tmpStr, 1000, "%sextremeMethod = %sMINIMA  # %s\n", prefix, prefix, extremeMethod_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::MAXIMA:
          SNPRINTF(tmpStr, 1000, "%sextremeMethod = %sMAXIMA  # %s\n", prefix, prefix, extremeMethod_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *optimizationMethod_names = "NELDER_MEAD, BFGS";
    switch (atts->GetOptimizationMethod())
    {
      case ExtremeValueAnalysisAttributes::NELDER_MEAD:
          SNPRINTF(tmpStr, 1000, "%soptimizationMethod = %sNELDER_MEAD  # %s\n", prefix, prefix, optimizationMethod_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::BFGS:
          SNPRINTF(tmpStr, 1000, "%soptimizationMethod = %sBFGS  # %s\n", prefix, prefix, optimizationMethod_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *aggregation_names = "ANNUAL, SEASONAL, MONTHLY";
    switch (atts->GetAggregation())
    {
      case ExtremeValueAnalysisAttributes::ANNUAL:
          SNPRINTF(tmpStr, 1000, "%saggregation = %sANNUAL  # %s\n", prefix, prefix, aggregation_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::SEASONAL:
          SNPRINTF(tmpStr, 1000, "%saggregation = %sSEASONAL  # %s\n", prefix, prefix, aggregation_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::MONTHLY:
          SNPRINTF(tmpStr, 1000, "%saggregation = %sMONTHLY  # %s\n", prefix, prefix, aggregation_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    if(atts->GetCovariateModelScale())
        SNPRINTF(tmpStr, 1000, "%scovariateModelScale = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scovariateModelScale = 0\n", prefix);
    str += tmpStr;
    if(atts->GetCovariateModelLocation())
        SNPRINTF(tmpStr, 1000, "%scovariateModelLocation = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scovariateModelLocation = 0\n", prefix);
    str += tmpStr;
    if(atts->GetCovariateModelShape())
        SNPRINTF(tmpStr, 1000, "%scovariateModelShape = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scovariateModelShape = 0\n", prefix);
    str += tmpStr;
    if(atts->GetComputeReturnValues())
        SNPRINTF(tmpStr, 1000, "%scomputeReturnValues = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scomputeReturnValues = 0\n", prefix);
    str += tmpStr;
    {   const intVector &returnValues = atts->GetReturnValues();
        SNPRINTF(tmpStr, 1000, "%sreturnValues = (", prefix);
        str += tmpStr;
        for(size_t i = 0; i < returnValues.size(); ++i)
        {
            SNPRINTF(tmpStr, 1000, "%d", returnValues[i]);
            str += tmpStr;
            if(i < returnValues.size() - 1)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    if(atts->GetComputeRVDifferences())
        SNPRINTF(tmpStr, 1000, "%scomputeRVDifferences = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scomputeRVDifferences = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%srvDifference1 = %d\n", prefix, atts->GetRvDifference1());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%srvDifference2 = %d\n", prefix, atts->GetRvDifference2());
    str += tmpStr;
    const char *displayMonth_names = "JANUARY, FEBRUARY, MARCH, APRIL, MAY, "
        "JUNE, JULY, AUGUST, SEPTEMBER, "
        "OCTOBER, NOVEMBER, DECEMBER";
    switch (atts->GetDisplayMonth())
    {
      case ExtremeValueAnalysisAttributes::JANUARY:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sJANUARY  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::FEBRUARY:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sFEBRUARY  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::MARCH:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sMARCH  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::APRIL:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sAPRIL  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::MAY:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sMAY  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::JUNE:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sJUNE  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::JULY:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sJULY  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::AUGUST:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sAUGUST  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::SEPTEMBER:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sSEPTEMBER  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::OCTOBER:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sOCTOBER  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::NOVEMBER:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sNOVEMBER  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::DECEMBER:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sDECEMBER  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *displaySeason_names = "WINTER, SPRING, SUMMER, FALL";
    switch (atts->GetDisplaySeason())
    {
      case ExtremeValueAnalysisAttributes::WINTER:
          SNPRINTF(tmpStr, 1000, "%sdisplaySeason = %sWINTER  # %s\n", prefix, prefix, displaySeason_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::SPRING:
          SNPRINTF(tmpStr, 1000, "%sdisplaySeason = %sSPRING  # %s\n", prefix, prefix, displaySeason_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::SUMMER:
          SNPRINTF(tmpStr, 1000, "%sdisplaySeason = %sSUMMER  # %s\n", prefix, prefix, displaySeason_names);
          str += tmpStr;
          break;
      case ExtremeValueAnalysisAttributes::FALL:
          SNPRINTF(tmpStr, 1000, "%sdisplaySeason = %sFALL  # %s\n", prefix, prefix, displaySeason_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    if(atts->GetComputeParamValues())
        SNPRINTF(tmpStr, 1000, "%scomputeParamValues = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scomputeParamValues = 0\n", prefix);
    str += tmpStr;
    if(atts->GetDumpData())
        SNPRINTF(tmpStr, 1000, "%sdumpData = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sdumpData = 0\n", prefix);
    str += tmpStr;
    if(atts->GetDumpDebug())
        SNPRINTF(tmpStr, 1000, "%sdumpDebug = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sdumpDebug = 0\n", prefix);
    str += tmpStr;
    return str;
}

static PyObject *
ExtremeValueAnalysisAttributes_Notify(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    obj->data->Notify();
    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetDataYearBegin(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dataYearBegin in the object.
    obj->data->SetDataYearBegin((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetDataYearBegin(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDataYearBegin()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetDataAnalysisYearRangeEnabled(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dataAnalysisYearRangeEnabled in the object.
    obj->data->SetDataAnalysisYearRangeEnabled(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetDataAnalysisYearRangeEnabled(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetDataAnalysisYearRangeEnabled()?1L:0L);
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetDataAnalysisYear1(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dataAnalysisYear1 in the object.
    obj->data->SetDataAnalysisYear1((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetDataAnalysisYear1(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDataAnalysisYear1()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetDataAnalysisYear2(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dataAnalysisYear2 in the object.
    obj->data->SetDataAnalysisYear2((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetDataAnalysisYear2(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDataAnalysisYear2()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetEnsemble(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the ensemble in the object.
    obj->data->SetEnsemble(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetEnsemble(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetEnsemble()?1L:0L);
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetNumEnsembles(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the numEnsembles in the object.
    obj->data->SetNumEnsembles((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetNumEnsembles(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetNumEnsembles()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetDataScaling(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the dataScaling in the object.
    obj->data->SetDataScaling(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetDataScaling(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetDataScaling());
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetExtremeMethod(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the extremeMethod in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetExtremeMethod(ExtremeValueAnalysisAttributes::ExtremeType(ival));
    else
    {
        fprintf(stderr, "An invalid extremeMethod value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "MINIMA, MAXIMA.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetExtremeMethod(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetExtremeMethod()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetOptimizationMethod(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the optimizationMethod in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetOptimizationMethod(ExtremeValueAnalysisAttributes::OptimizationType(ival));
    else
    {
        fprintf(stderr, "An invalid optimizationMethod value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "NELDER_MEAD, BFGS.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetOptimizationMethod(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetOptimizationMethod()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetAggregation(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the aggregation in the object.
    if(ival >= 0 && ival < 3)
        obj->data->SetAggregation(ExtremeValueAnalysisAttributes::AggregationType(ival));
    else
    {
        fprintf(stderr, "An invalid aggregation value was given. "
                        "Valid values are in the range of [0,2]. "
                        "You can also use the following names: "
                        "ANNUAL, SEASONAL, MONTHLY.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetAggregation(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetAggregation()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetCovariateModelScale(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the covariateModelScale in the object.
    obj->data->SetCovariateModelScale(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetCovariateModelScale(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetCovariateModelScale()?1L:0L);
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetCovariateModelLocation(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the covariateModelLocation in the object.
    obj->data->SetCovariateModelLocation(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetCovariateModelLocation(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetCovariateModelLocation()?1L:0L);
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetCovariateModelShape(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the covariateModelShape in the object.
    obj->data->SetCovariateModelShape(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetCovariateModelShape(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetCovariateModelShape()?1L:0L);
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetComputeReturnValues(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the computeReturnValues in the object.
    obj->data->SetComputeReturnValues(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetComputeReturnValues(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetComputeReturnValues()?1L:0L);
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetReturnValues(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    intVector  &vec = obj->data->GetReturnValues();
    PyObject   *tuple;
    if(!PyArg_ParseTuple(args, "O", &tuple))
        return NULL;

    if(PyTuple_Check(tuple))
    {
        vec.resize(PyTuple_Size(tuple));
        for(int i = 0; i < PyTuple_Size(tuple); ++i)
        {
            PyObject *item = PyTuple_GET_ITEM(tuple, i);
            if(PyFloat_Check(item))
                vec[i] = int(PyFloat_AS_DOUBLE(item));
            else if(PyInt_Check(item))
                vec[i] = int(PyInt_AS_LONG(item));
            else if(PyLong_Check(item))
                vec[i] = int(PyLong_AsLong(item));
            else
                vec[i] = 0;
        }
    }
    else if(PyFloat_Check(tuple))
    {
        vec.resize(1);
        vec[0] = int(PyFloat_AS_DOUBLE(tuple));
    }
    else if(PyInt_Check(tuple))
    {
        vec.resize(1);
        vec[0] = int(PyInt_AS_LONG(tuple));
    }
    else if(PyLong_Check(tuple))
    {
        vec.resize(1);
        vec[0] = int(PyLong_AsLong(tuple));
    }
    else
        return NULL;

    // Mark the returnValues in the object as modified.
    obj->data->SelectReturnValues();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetReturnValues(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the returnValues.
    const intVector &returnValues = obj->data->GetReturnValues();
    PyObject *retval = PyTuple_New(returnValues.size());
    for(size_t i = 0; i < returnValues.size(); ++i)
        PyTuple_SET_ITEM(retval, i, PyInt_FromLong(long(returnValues[i])));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetComputeRVDifferences(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the computeRVDifferences in the object.
    obj->data->SetComputeRVDifferences(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetComputeRVDifferences(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetComputeRVDifferences()?1L:0L);
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetRvDifference1(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the rvDifference1 in the object.
    obj->data->SetRvDifference1((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetRvDifference1(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetRvDifference1()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetRvDifference2(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the rvDifference2 in the object.
    obj->data->SetRvDifference2((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetRvDifference2(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetRvDifference2()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetDisplayMonth(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the displayMonth in the object.
    if(ival >= 0 && ival < 12)
        obj->data->SetDisplayMonth(ExtremeValueAnalysisAttributes::MonthType(ival));
    else
    {
        fprintf(stderr, "An invalid displayMonth value was given. "
                        "Valid values are in the range of [0,11]. "
                        "You can also use the following names: "
                        "JANUARY, FEBRUARY, MARCH, APRIL, MAY, "
                        "JUNE, JULY, AUGUST, SEPTEMBER, "
                        "OCTOBER, NOVEMBER, DECEMBER.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetDisplayMonth(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDisplayMonth()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetDisplaySeason(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the displaySeason in the object.
    if(ival >= 0 && ival < 4)
        obj->data->SetDisplaySeason(ExtremeValueAnalysisAttributes::SeasonType(ival));
    else
    {
        fprintf(stderr, "An invalid displaySeason value was given. "
                        "Valid values are in the range of [0,3]. "
                        "You can also use the following names: "
                        "WINTER, SPRING, SUMMER, FALL.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetDisplaySeason(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDisplaySeason()));
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetComputeParamValues(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the computeParamValues in the object.
    obj->data->SetComputeParamValues(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetComputeParamValues(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetComputeParamValues()?1L:0L);
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetDumpData(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dumpData in the object.
    obj->data->SetDumpData(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetDumpData(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetDumpData()?1L:0L);
    return retval;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_SetDumpDebug(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dumpDebug in the object.
    obj->data->SetDumpDebug(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
ExtremeValueAnalysisAttributes_GetDumpDebug(PyObject *self, PyObject *args)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetDumpDebug()?1L:0L);
    return retval;
}



PyMethodDef PyExtremeValueAnalysisAttributes_methods[EXTREMEVALUEANALYSISATTRIBUTES_NMETH] = {
    {"Notify", ExtremeValueAnalysisAttributes_Notify, METH_VARARGS},
    {"SetDataYearBegin", ExtremeValueAnalysisAttributes_SetDataYearBegin, METH_VARARGS},
    {"GetDataYearBegin", ExtremeValueAnalysisAttributes_GetDataYearBegin, METH_VARARGS},
    {"SetDataAnalysisYearRangeEnabled", ExtremeValueAnalysisAttributes_SetDataAnalysisYearRangeEnabled, METH_VARARGS},
    {"GetDataAnalysisYearRangeEnabled", ExtremeValueAnalysisAttributes_GetDataAnalysisYearRangeEnabled, METH_VARARGS},
    {"SetDataAnalysisYear1", ExtremeValueAnalysisAttributes_SetDataAnalysisYear1, METH_VARARGS},
    {"GetDataAnalysisYear1", ExtremeValueAnalysisAttributes_GetDataAnalysisYear1, METH_VARARGS},
    {"SetDataAnalysisYear2", ExtremeValueAnalysisAttributes_SetDataAnalysisYear2, METH_VARARGS},
    {"GetDataAnalysisYear2", ExtremeValueAnalysisAttributes_GetDataAnalysisYear2, METH_VARARGS},
    {"SetEnsemble", ExtremeValueAnalysisAttributes_SetEnsemble, METH_VARARGS},
    {"GetEnsemble", ExtremeValueAnalysisAttributes_GetEnsemble, METH_VARARGS},
    {"SetNumEnsembles", ExtremeValueAnalysisAttributes_SetNumEnsembles, METH_VARARGS},
    {"GetNumEnsembles", ExtremeValueAnalysisAttributes_GetNumEnsembles, METH_VARARGS},
    {"SetDataScaling", ExtremeValueAnalysisAttributes_SetDataScaling, METH_VARARGS},
    {"GetDataScaling", ExtremeValueAnalysisAttributes_GetDataScaling, METH_VARARGS},
    {"SetExtremeMethod", ExtremeValueAnalysisAttributes_SetExtremeMethod, METH_VARARGS},
    {"GetExtremeMethod", ExtremeValueAnalysisAttributes_GetExtremeMethod, METH_VARARGS},
    {"SetOptimizationMethod", ExtremeValueAnalysisAttributes_SetOptimizationMethod, METH_VARARGS},
    {"GetOptimizationMethod", ExtremeValueAnalysisAttributes_GetOptimizationMethod, METH_VARARGS},
    {"SetAggregation", ExtremeValueAnalysisAttributes_SetAggregation, METH_VARARGS},
    {"GetAggregation", ExtremeValueAnalysisAttributes_GetAggregation, METH_VARARGS},
    {"SetCovariateModelScale", ExtremeValueAnalysisAttributes_SetCovariateModelScale, METH_VARARGS},
    {"GetCovariateModelScale", ExtremeValueAnalysisAttributes_GetCovariateModelScale, METH_VARARGS},
    {"SetCovariateModelLocation", ExtremeValueAnalysisAttributes_SetCovariateModelLocation, METH_VARARGS},
    {"GetCovariateModelLocation", ExtremeValueAnalysisAttributes_GetCovariateModelLocation, METH_VARARGS},
    {"SetCovariateModelShape", ExtremeValueAnalysisAttributes_SetCovariateModelShape, METH_VARARGS},
    {"GetCovariateModelShape", ExtremeValueAnalysisAttributes_GetCovariateModelShape, METH_VARARGS},
    {"SetComputeReturnValues", ExtremeValueAnalysisAttributes_SetComputeReturnValues, METH_VARARGS},
    {"GetComputeReturnValues", ExtremeValueAnalysisAttributes_GetComputeReturnValues, METH_VARARGS},
    {"SetReturnValues", ExtremeValueAnalysisAttributes_SetReturnValues, METH_VARARGS},
    {"GetReturnValues", ExtremeValueAnalysisAttributes_GetReturnValues, METH_VARARGS},
    {"SetComputeRVDifferences", ExtremeValueAnalysisAttributes_SetComputeRVDifferences, METH_VARARGS},
    {"GetComputeRVDifferences", ExtremeValueAnalysisAttributes_GetComputeRVDifferences, METH_VARARGS},
    {"SetRvDifference1", ExtremeValueAnalysisAttributes_SetRvDifference1, METH_VARARGS},
    {"GetRvDifference1", ExtremeValueAnalysisAttributes_GetRvDifference1, METH_VARARGS},
    {"SetRvDifference2", ExtremeValueAnalysisAttributes_SetRvDifference2, METH_VARARGS},
    {"GetRvDifference2", ExtremeValueAnalysisAttributes_GetRvDifference2, METH_VARARGS},
    {"SetDisplayMonth", ExtremeValueAnalysisAttributes_SetDisplayMonth, METH_VARARGS},
    {"GetDisplayMonth", ExtremeValueAnalysisAttributes_GetDisplayMonth, METH_VARARGS},
    {"SetDisplaySeason", ExtremeValueAnalysisAttributes_SetDisplaySeason, METH_VARARGS},
    {"GetDisplaySeason", ExtremeValueAnalysisAttributes_GetDisplaySeason, METH_VARARGS},
    {"SetComputeParamValues", ExtremeValueAnalysisAttributes_SetComputeParamValues, METH_VARARGS},
    {"GetComputeParamValues", ExtremeValueAnalysisAttributes_GetComputeParamValues, METH_VARARGS},
    {"SetDumpData", ExtremeValueAnalysisAttributes_SetDumpData, METH_VARARGS},
    {"GetDumpData", ExtremeValueAnalysisAttributes_GetDumpData, METH_VARARGS},
    {"SetDumpDebug", ExtremeValueAnalysisAttributes_SetDumpDebug, METH_VARARGS},
    {"GetDumpDebug", ExtremeValueAnalysisAttributes_GetDumpDebug, METH_VARARGS},
    {NULL, NULL}
};

//
// Type functions
//

static void
ExtremeValueAnalysisAttributes_dealloc(PyObject *v)
{
   ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)v;
   if(obj->parent != 0)
       Py_DECREF(obj->parent);
   if(obj->owns)
       delete obj->data;
}

static int
ExtremeValueAnalysisAttributes_compare(PyObject *v, PyObject *w)
{
    ExtremeValueAnalysisAttributes *a = ((ExtremeValueAnalysisAttributesObject *)v)->data;
    ExtremeValueAnalysisAttributes *b = ((ExtremeValueAnalysisAttributesObject *)w)->data;
    return (*a == *b) ? 0 : -1;
}

PyObject *
PyExtremeValueAnalysisAttributes_getattr(PyObject *self, char *name)
{
    if(strcmp(name, "dataYearBegin") == 0)
        return ExtremeValueAnalysisAttributes_GetDataYearBegin(self, NULL);
    if(strcmp(name, "dataAnalysisYearRangeEnabled") == 0)
        return ExtremeValueAnalysisAttributes_GetDataAnalysisYearRangeEnabled(self, NULL);
    if(strcmp(name, "dataAnalysisYear1") == 0)
        return ExtremeValueAnalysisAttributes_GetDataAnalysisYear1(self, NULL);
    if(strcmp(name, "dataAnalysisYear2") == 0)
        return ExtremeValueAnalysisAttributes_GetDataAnalysisYear2(self, NULL);
    if(strcmp(name, "ensemble") == 0)
        return ExtremeValueAnalysisAttributes_GetEnsemble(self, NULL);
    if(strcmp(name, "numEnsembles") == 0)
        return ExtremeValueAnalysisAttributes_GetNumEnsembles(self, NULL);
    if(strcmp(name, "dataScaling") == 0)
        return ExtremeValueAnalysisAttributes_GetDataScaling(self, NULL);
    if(strcmp(name, "extremeMethod") == 0)
        return ExtremeValueAnalysisAttributes_GetExtremeMethod(self, NULL);
    if(strcmp(name, "MINIMA") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::MINIMA));
    if(strcmp(name, "MAXIMA") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::MAXIMA));

    if(strcmp(name, "optimizationMethod") == 0)
        return ExtremeValueAnalysisAttributes_GetOptimizationMethod(self, NULL);
    if(strcmp(name, "NELDER_MEAD") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::NELDER_MEAD));
    if(strcmp(name, "BFGS") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::BFGS));

    if(strcmp(name, "aggregation") == 0)
        return ExtremeValueAnalysisAttributes_GetAggregation(self, NULL);
    if(strcmp(name, "ANNUAL") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::ANNUAL));
    if(strcmp(name, "SEASONAL") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::SEASONAL));
    if(strcmp(name, "MONTHLY") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::MONTHLY));

    if(strcmp(name, "covariateModelScale") == 0)
        return ExtremeValueAnalysisAttributes_GetCovariateModelScale(self, NULL);
    if(strcmp(name, "covariateModelLocation") == 0)
        return ExtremeValueAnalysisAttributes_GetCovariateModelLocation(self, NULL);
    if(strcmp(name, "covariateModelShape") == 0)
        return ExtremeValueAnalysisAttributes_GetCovariateModelShape(self, NULL);
    if(strcmp(name, "computeReturnValues") == 0)
        return ExtremeValueAnalysisAttributes_GetComputeReturnValues(self, NULL);
    if(strcmp(name, "returnValues") == 0)
        return ExtremeValueAnalysisAttributes_GetReturnValues(self, NULL);
    if(strcmp(name, "computeRVDifferences") == 0)
        return ExtremeValueAnalysisAttributes_GetComputeRVDifferences(self, NULL);
    if(strcmp(name, "rvDifference1") == 0)
        return ExtremeValueAnalysisAttributes_GetRvDifference1(self, NULL);
    if(strcmp(name, "rvDifference2") == 0)
        return ExtremeValueAnalysisAttributes_GetRvDifference2(self, NULL);
    if(strcmp(name, "displayMonth") == 0)
        return ExtremeValueAnalysisAttributes_GetDisplayMonth(self, NULL);
    if(strcmp(name, "JANUARY") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::JANUARY));
    if(strcmp(name, "FEBRUARY") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::FEBRUARY));
    if(strcmp(name, "MARCH") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::MARCH));
    if(strcmp(name, "APRIL") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::APRIL));
    if(strcmp(name, "MAY") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::MAY));
    if(strcmp(name, "JUNE") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::JUNE));
    if(strcmp(name, "JULY") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::JULY));
    if(strcmp(name, "AUGUST") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::AUGUST));
    if(strcmp(name, "SEPTEMBER") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::SEPTEMBER));
    if(strcmp(name, "OCTOBER") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::OCTOBER));
    if(strcmp(name, "NOVEMBER") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::NOVEMBER));
    if(strcmp(name, "DECEMBER") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::DECEMBER));

    if(strcmp(name, "displaySeason") == 0)
        return ExtremeValueAnalysisAttributes_GetDisplaySeason(self, NULL);
    if(strcmp(name, "WINTER") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::WINTER));
    if(strcmp(name, "SPRING") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::SPRING));
    if(strcmp(name, "SUMMER") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::SUMMER));
    if(strcmp(name, "FALL") == 0)
        return PyInt_FromLong(long(ExtremeValueAnalysisAttributes::FALL));

    if(strcmp(name, "computeParamValues") == 0)
        return ExtremeValueAnalysisAttributes_GetComputeParamValues(self, NULL);
    if(strcmp(name, "dumpData") == 0)
        return ExtremeValueAnalysisAttributes_GetDumpData(self, NULL);
    if(strcmp(name, "dumpDebug") == 0)
        return ExtremeValueAnalysisAttributes_GetDumpDebug(self, NULL);

    return Py_FindMethod(PyExtremeValueAnalysisAttributes_methods, self, name);
}

int
PyExtremeValueAnalysisAttributes_setattr(PyObject *self, char *name, PyObject *args)
{
    // Create a tuple to contain the arguments since all of the Set
    // functions expect a tuple.
    PyObject *tuple = PyTuple_New(1);
    PyTuple_SET_ITEM(tuple, 0, args);
    Py_INCREF(args);
    PyObject *obj = NULL;

    if(strcmp(name, "dataYearBegin") == 0)
        obj = ExtremeValueAnalysisAttributes_SetDataYearBegin(self, tuple);
    else if(strcmp(name, "dataAnalysisYearRangeEnabled") == 0)
        obj = ExtremeValueAnalysisAttributes_SetDataAnalysisYearRangeEnabled(self, tuple);
    else if(strcmp(name, "dataAnalysisYear1") == 0)
        obj = ExtremeValueAnalysisAttributes_SetDataAnalysisYear1(self, tuple);
    else if(strcmp(name, "dataAnalysisYear2") == 0)
        obj = ExtremeValueAnalysisAttributes_SetDataAnalysisYear2(self, tuple);
    else if(strcmp(name, "ensemble") == 0)
        obj = ExtremeValueAnalysisAttributes_SetEnsemble(self, tuple);
    else if(strcmp(name, "numEnsembles") == 0)
        obj = ExtremeValueAnalysisAttributes_SetNumEnsembles(self, tuple);
    else if(strcmp(name, "dataScaling") == 0)
        obj = ExtremeValueAnalysisAttributes_SetDataScaling(self, tuple);
    else if(strcmp(name, "extremeMethod") == 0)
        obj = ExtremeValueAnalysisAttributes_SetExtremeMethod(self, tuple);
    else if(strcmp(name, "optimizationMethod") == 0)
        obj = ExtremeValueAnalysisAttributes_SetOptimizationMethod(self, tuple);
    else if(strcmp(name, "aggregation") == 0)
        obj = ExtremeValueAnalysisAttributes_SetAggregation(self, tuple);
    else if(strcmp(name, "covariateModelScale") == 0)
        obj = ExtremeValueAnalysisAttributes_SetCovariateModelScale(self, tuple);
    else if(strcmp(name, "covariateModelLocation") == 0)
        obj = ExtremeValueAnalysisAttributes_SetCovariateModelLocation(self, tuple);
    else if(strcmp(name, "covariateModelShape") == 0)
        obj = ExtremeValueAnalysisAttributes_SetCovariateModelShape(self, tuple);
    else if(strcmp(name, "computeReturnValues") == 0)
        obj = ExtremeValueAnalysisAttributes_SetComputeReturnValues(self, tuple);
    else if(strcmp(name, "returnValues") == 0)
        obj = ExtremeValueAnalysisAttributes_SetReturnValues(self, tuple);
    else if(strcmp(name, "computeRVDifferences") == 0)
        obj = ExtremeValueAnalysisAttributes_SetComputeRVDifferences(self, tuple);
    else if(strcmp(name, "rvDifference1") == 0)
        obj = ExtremeValueAnalysisAttributes_SetRvDifference1(self, tuple);
    else if(strcmp(name, "rvDifference2") == 0)
        obj = ExtremeValueAnalysisAttributes_SetRvDifference2(self, tuple);
    else if(strcmp(name, "displayMonth") == 0)
        obj = ExtremeValueAnalysisAttributes_SetDisplayMonth(self, tuple);
    else if(strcmp(name, "displaySeason") == 0)
        obj = ExtremeValueAnalysisAttributes_SetDisplaySeason(self, tuple);
    else if(strcmp(name, "computeParamValues") == 0)
        obj = ExtremeValueAnalysisAttributes_SetComputeParamValues(self, tuple);
    else if(strcmp(name, "dumpData") == 0)
        obj = ExtremeValueAnalysisAttributes_SetDumpData(self, tuple);
    else if(strcmp(name, "dumpDebug") == 0)
        obj = ExtremeValueAnalysisAttributes_SetDumpDebug(self, tuple);

    if(obj != NULL)
        Py_DECREF(obj);

    Py_DECREF(tuple);
    if( obj == NULL)
        PyErr_Format(PyExc_RuntimeError, "Unable to set unknown attribute: '%s'", name);
    return (obj != NULL) ? 0 : -1;
}

static int
ExtremeValueAnalysisAttributes_print(PyObject *v, FILE *fp, int flags)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)v;
    fprintf(fp, "%s", PyExtremeValueAnalysisAttributes_ToString(obj->data, "").c_str());
    return 0;
}

PyObject *
ExtremeValueAnalysisAttributes_str(PyObject *v)
{
    ExtremeValueAnalysisAttributesObject *obj = (ExtremeValueAnalysisAttributesObject *)v;
    return PyString_FromString(PyExtremeValueAnalysisAttributes_ToString(obj->data,"").c_str());
}

//
// The doc string for the class.
//
#if PY_MAJOR_VERSION > 2 || (PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION >= 5)
static const char *ExtremeValueAnalysisAttributes_Purpose = "Attributes for ExtremeValueAnalysis operator";
#else
static char *ExtremeValueAnalysisAttributes_Purpose = "Attributes for ExtremeValueAnalysis operator";
#endif

//
// The type description structure
//
static PyTypeObject ExtremeValueAnalysisAttributesType =
{
    //
    // Type header
    //
    PyObject_HEAD_INIT(&PyType_Type)
    0,                                   // ob_size
    "ExtremeValueAnalysisAttributes",                    // tp_name
    sizeof(ExtremeValueAnalysisAttributesObject),        // tp_basicsize
    0,                                   // tp_itemsize
    //
    // Standard methods
    //
    (destructor)ExtremeValueAnalysisAttributes_dealloc,  // tp_dealloc
    (printfunc)ExtremeValueAnalysisAttributes_print,     // tp_print
    (getattrfunc)PyExtremeValueAnalysisAttributes_getattr, // tp_getattr
    (setattrfunc)PyExtremeValueAnalysisAttributes_setattr, // tp_setattr
    (cmpfunc)ExtremeValueAnalysisAttributes_compare,     // tp_compare
    (reprfunc)0,                         // tp_repr
    //
    // Type categories
    //
    0,                                   // tp_as_number
    0,                                   // tp_as_sequence
    0,                                   // tp_as_mapping
    //
    // More methods
    //
    0,                                   // tp_hash
    0,                                   // tp_call
    (reprfunc)ExtremeValueAnalysisAttributes_str,        // tp_str
    0,                                   // tp_getattro
    0,                                   // tp_setattro
    0,                                   // tp_as_buffer
    Py_TPFLAGS_CHECKTYPES,               // tp_flags
    ExtremeValueAnalysisAttributes_Purpose,              // tp_doc
    0,                                   // tp_traverse
    0,                                   // tp_clear
    0,                                   // tp_richcompare
    0                                    // tp_weaklistoffset
};

//
// Helper functions for object allocation.
//

static ExtremeValueAnalysisAttributes *defaultAtts = 0;
static ExtremeValueAnalysisAttributes *currentAtts = 0;

static PyObject *
NewExtremeValueAnalysisAttributes(int useCurrent)
{
    ExtremeValueAnalysisAttributesObject *newObject;
    newObject = PyObject_NEW(ExtremeValueAnalysisAttributesObject, &ExtremeValueAnalysisAttributesType);
    if(newObject == NULL)
        return NULL;
    if(useCurrent && currentAtts != 0)
        newObject->data = new ExtremeValueAnalysisAttributes(*currentAtts);
    else if(defaultAtts != 0)
        newObject->data = new ExtremeValueAnalysisAttributes(*defaultAtts);
    else
        newObject->data = new ExtremeValueAnalysisAttributes;
    newObject->owns = true;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

static PyObject *
WrapExtremeValueAnalysisAttributes(const ExtremeValueAnalysisAttributes *attr)
{
    ExtremeValueAnalysisAttributesObject *newObject;
    newObject = PyObject_NEW(ExtremeValueAnalysisAttributesObject, &ExtremeValueAnalysisAttributesType);
    if(newObject == NULL)
        return NULL;
    newObject->data = (ExtremeValueAnalysisAttributes *)attr;
    newObject->owns = false;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

///////////////////////////////////////////////////////////////////////////////
//
// Interface that is exposed to the VisIt module.
//
///////////////////////////////////////////////////////////////////////////////

PyObject *
ExtremeValueAnalysisAttributes_new(PyObject *self, PyObject *args)
{
    int useCurrent = 0;
    if (!PyArg_ParseTuple(args, "i", &useCurrent))
    {
        if (!PyArg_ParseTuple(args, ""))
            return NULL;
        else
            PyErr_Clear();
    }

    return (PyObject *)NewExtremeValueAnalysisAttributes(useCurrent);
}

//
// Plugin method table. These methods are added to the visitmodule's methods.
//
static PyMethodDef ExtremeValueAnalysisAttributesMethods[] = {
    {"ExtremeValueAnalysisAttributes", ExtremeValueAnalysisAttributes_new, METH_VARARGS},
    {NULL,      NULL}        /* Sentinel */
};

static Observer *ExtremeValueAnalysisAttributesObserver = 0;

std::string
PyExtremeValueAnalysisAttributes_GetLogString()
{
    std::string s("ExtremeValueAnalysisAtts = ExtremeValueAnalysisAttributes()\n");
    if(currentAtts != 0)
        s += PyExtremeValueAnalysisAttributes_ToString(currentAtts, "ExtremeValueAnalysisAtts.");
    return s;
}

static void
PyExtremeValueAnalysisAttributes_CallLogRoutine(Subject *subj, void *data)
{
    typedef void (*logCallback)(const std::string &);
    logCallback cb = (logCallback)data;

    if(cb != 0)
    {
        std::string s("ExtremeValueAnalysisAtts = ExtremeValueAnalysisAttributes()\n");
        s += PyExtremeValueAnalysisAttributes_ToString(currentAtts, "ExtremeValueAnalysisAtts.");
        cb(s);
    }
}

void
PyExtremeValueAnalysisAttributes_StartUp(ExtremeValueAnalysisAttributes *subj, void *data)
{
    if(subj == 0)
        return;

    currentAtts = subj;
    PyExtremeValueAnalysisAttributes_SetDefaults(subj);

    //
    // Create the observer that will be notified when the attributes change.
    //
    if(ExtremeValueAnalysisAttributesObserver == 0)
    {
        ExtremeValueAnalysisAttributesObserver = new ObserverToCallback(subj,
            PyExtremeValueAnalysisAttributes_CallLogRoutine, (void *)data);
    }

}

void
PyExtremeValueAnalysisAttributes_CloseDown()
{
    delete defaultAtts;
    defaultAtts = 0;
    delete ExtremeValueAnalysisAttributesObserver;
    ExtremeValueAnalysisAttributesObserver = 0;
}

PyMethodDef *
PyExtremeValueAnalysisAttributes_GetMethodTable(int *nMethods)
{
    *nMethods = 1;
    return ExtremeValueAnalysisAttributesMethods;
}

bool
PyExtremeValueAnalysisAttributes_Check(PyObject *obj)
{
    return (obj->ob_type == &ExtremeValueAnalysisAttributesType);
}

ExtremeValueAnalysisAttributes *
PyExtremeValueAnalysisAttributes_FromPyObject(PyObject *obj)
{
    ExtremeValueAnalysisAttributesObject *obj2 = (ExtremeValueAnalysisAttributesObject *)obj;
    return obj2->data;
}

PyObject *
PyExtremeValueAnalysisAttributes_New()
{
    return NewExtremeValueAnalysisAttributes(0);
}

PyObject *
PyExtremeValueAnalysisAttributes_Wrap(const ExtremeValueAnalysisAttributes *attr)
{
    return WrapExtremeValueAnalysisAttributes(attr);
}

void
PyExtremeValueAnalysisAttributes_SetParent(PyObject *obj, PyObject *parent)
{
    ExtremeValueAnalysisAttributesObject *obj2 = (ExtremeValueAnalysisAttributesObject *)obj;
    obj2->parent = parent;
}

void
PyExtremeValueAnalysisAttributes_SetDefaults(const ExtremeValueAnalysisAttributes *atts)
{
    if(defaultAtts)
        delete defaultAtts;

    defaultAtts = new ExtremeValueAnalysisAttributes(*atts);
}

