/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include "QvisExtremeValueAnalysisWindow.h"

#include <ExtremeValueAnalysisAttributes.h>
#include <ViewerProxy.h>

#include <QCheckBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QSpinBox>
#include <QButtonGroup>
#include <QRadioButton>
#include <QGroupBox>
#include <QComboBox>
#include <QvisColorTableButton.h>
#include <QvisOpacitySlider.h>
#include <QvisColorButton.h>
#include <QvisLineStyleWidget.h>
#include <QvisLineWidthWidget.h>
#include <QvisVariableButton.h>

#include <stdio.h>
#include <string>

using std::string;

// ****************************************************************************
// Method: QvisExtremeValueAnalysisWindow::QvisExtremeValueAnalysisWindow
//
// Purpose: 
//   Constructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisExtremeValueAnalysisWindow::QvisExtremeValueAnalysisWindow(const int type,
                         ExtremeValueAnalysisAttributes *subj,
                         const QString &caption,
                         const QString &shortName,
                         QvisNotepadArea *notepad)
    : QvisOperatorWindow(type,subj, caption, shortName, notepad)
{
    atts = subj;
}


// ****************************************************************************
// Method: QvisExtremeValueAnalysisWindow::~QvisExtremeValueAnalysisWindow
//
// Purpose: 
//   Destructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisExtremeValueAnalysisWindow::~QvisExtremeValueAnalysisWindow()
{
}


// ****************************************************************************
// Method: QvisExtremeValueAnalysisWindow::CreateWindowContents
//
// Purpose: 
//   Creates the widgets for the window.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisExtremeValueAnalysisWindow::CreateWindowContents()
{
    int row = 0;
    //
    // Inputs group box
    //
    QGroupBox *inputGroup = new QGroupBox(central);
    inputGroup->setTitle(tr("Inputs"));
    topLayout->addWidget(inputGroup);
    QGridLayout *inputLayout = new QGridLayout(inputGroup);
    inputLayout->setMargin(5);
    inputLayout->setSpacing(10);

    inputLayout->addWidget(new QLabel(tr("Start year"), central), row, 0);
    dataYearBegin = new QLineEdit(central);
    connect(dataYearBegin, SIGNAL(returnPressed()),
            this, SLOT(dataYearBeginProcessText()));
    inputLayout->addWidget(dataYearBegin, row,1);

    row++;
    dataAnalysisYearRangeEnabled = new QCheckBox(tr("Analysis year range"), central);
    connect(dataAnalysisYearRangeEnabled, SIGNAL(toggled(bool)),
            this, SLOT(dataAnalysisYearRangeEnabledChanged(bool)));
    inputLayout->addWidget(dataAnalysisYearRangeEnabled, row,0);
    dataAnalysisYear1 = new QLineEdit(central);
    connect(dataAnalysisYear1, SIGNAL(returnPressed()),
            this, SLOT(dataAnalysisYear1ProcessText()));
    inputLayout->addWidget(dataAnalysisYear1, row,1);
    dataAnalysisYear2 = new QLineEdit(central);
    connect(dataAnalysisYear2, SIGNAL(returnPressed()),
            this, SLOT(dataAnalysisYear2ProcessText()));
    inputLayout->addWidget(dataAnalysisYear2, row,2);
    row++;

    ensemble = new QCheckBox(tr("Ensemble"), central);
    connect(ensemble, SIGNAL(toggled(bool)),
            this, SLOT(ensembleChanged(bool)));
    inputLayout->addWidget(ensemble, row,0);

    numEnsemblesLabel = new QLabel(tr("Number of members"), central);
    inputLayout->addWidget(numEnsemblesLabel,row,1);
    numEnsembles = new QLineEdit(central);
    connect(numEnsembles, SIGNAL(returnPressed()),
            this, SLOT(numEnsemblesProcessText()));
    inputLayout->addWidget(numEnsembles, row,2);
    row++;

    dataScalingLabel = new QLabel(tr("Data Scaling"), central);
    inputLayout->addWidget(dataScalingLabel,row,0);
    dataScaling = new QLineEdit(central);
    connect(dataScaling, SIGNAL(returnPressed()),
            this, SLOT(dataScalingProcessText()));
    inputLayout->addWidget(dataScaling, row,1);

    row = 0;

    //
    // Analysis group box
    //
    QGroupBox *analysisGroup = new QGroupBox(central);
    analysisGroup->setTitle(tr("Analysis"));
    topLayout->addWidget(analysisGroup);
    QGridLayout *analysisLayout = new QGridLayout(analysisGroup);
    analysisLayout->setMargin(5);
    analysisLayout->setSpacing(10);

    analysisLayout->addWidget(new QLabel(tr("Extremes"), central), row, 0);
    extremeMethod = new QWidget(central);
    extremeMethodButtonGroup= new QButtonGroup(extremeMethod);
    QHBoxLayout *extremeMethodLayout = new QHBoxLayout(extremeMethod);
    extremeMethodLayout->setMargin(0);
    extremeMethodLayout->setSpacing(10);
    QRadioButton *extremeMethodExtremeTypeMINIMA = new QRadioButton(tr("Minima"), extremeMethod);
    extremeMethodButtonGroup->addButton(extremeMethodExtremeTypeMINIMA,0);
    extremeMethodLayout->addWidget(extremeMethodExtremeTypeMINIMA);
    QRadioButton *extremeMethodExtremeTypeMAXIMA = new QRadioButton(tr("Maxima"), extremeMethod);
    extremeMethodButtonGroup->addButton(extremeMethodExtremeTypeMAXIMA,1);
    extremeMethodLayout->addWidget(extremeMethodExtremeTypeMAXIMA);
    connect(extremeMethodButtonGroup, SIGNAL(buttonClicked(int)),
            this, SLOT(extremeMethodChanged(int)));
    analysisLayout->addWidget(extremeMethod, row,1);
    row++;

    analysisLayout->addWidget(new QLabel(tr("Optimization"), central),row,0);
    optimizationSelect = new QComboBox(central);
    optimizationSelect->addItem(tr("Nelder-Mead"));
    optimizationSelect->addItem(tr("BFGS"));
    connect(optimizationSelect, SIGNAL(activated(int)),
            this, SLOT(optimizationChanged(int))); 
    analysisLayout->addWidget(optimizationSelect, row, 1);
    row++;

    analysisLayout->addWidget(new QLabel(tr("Aggregation"), central), row, 0);
    aggregationSelect = new QComboBox(central);
    aggregationSelect->addItem(tr("Annual"));
    aggregationSelect->addItem(tr("Seasonal"));
    aggregationSelect->addItem(tr("Monthly"));
    connect(aggregationSelect, SIGNAL(activated(int)),
            this, SLOT(aggregationChanged(int))); 
    analysisLayout->addWidget(aggregationSelect, row, 1);
    row++;

    //
    // Trend Modeling group box
    //
    QGroupBox *trendGroup = new QGroupBox(central);
    trendGroup->setTitle(tr("Trend Modeling"));
    analysisLayout->addWidget(trendGroup, row, 0);
    QGridLayout *trendLayout = new QGridLayout(trendGroup);
    trendLayout->setMargin(5);
    trendLayout->setSpacing(10);
    row = 0;

    covariateModelScale = new QCheckBox(tr("Linear trend in log scale parameter"), central);
    connect(covariateModelScale, SIGNAL(toggled(bool)),
            this, SLOT(covariateModelScaleChanged(bool)));
    trendLayout->addWidget(covariateModelScale, row,0);
    row++;

    covariateModelLocation = new QCheckBox(tr("Linear trend in location parameter"), central);
    connect(covariateModelLocation, SIGNAL(toggled(bool)),
            this, SLOT(covariateModelLocationChanged(bool)));
    trendLayout->addWidget(covariateModelLocation, row,0);
    row++;

    covariateModelShape = new QCheckBox(tr("Linear trend in shape parameter"), central);
    connect(covariateModelShape, SIGNAL(toggled(bool)),
            this, SLOT(covariateModelShapeChanged(bool)));
    trendLayout->addWidget(covariateModelShape, row,0);
    row++;


    //
    // Outputs group box
    //
    row = 0;
    QGroupBox *outputGroup = new QGroupBox(central);
    outputGroup->setTitle(tr("Outputs"));
    topLayout->addWidget(outputGroup);
    QGridLayout *outputLayout = new QGridLayout(outputGroup);
    outputLayout->setMargin(5);
    outputLayout->setSpacing(10);

    computeReturnValues = new QCheckBox(tr("Compute return value for years"), central);
    connect(computeReturnValues, SIGNAL(toggled(bool)),
            this, SLOT(computeReturnValuesChanged(bool)));
    outputLayout->addWidget(computeReturnValues, row,0);

    returnValues = new QLineEdit(central);
    connect(returnValues, SIGNAL(returnPressed()),
            this, SLOT(returnValuesProcessText()));
    outputLayout->addWidget(returnValues, row,1);
    row++;

    computeRVDifferences = new QCheckBox(tr("Compute return value differences for years"), central);
    connect(computeRVDifferences, SIGNAL(toggled(bool)),
            this, SLOT(computeRVDifferencesChanged(bool)));
    outputLayout->addWidget(computeRVDifferences, row,0);

    rvDifference1 = new QLineEdit(central);
    connect(rvDifference1, SIGNAL(returnPressed()),
            this, SLOT(rvDifference1ProcessText()));
    outputLayout->addWidget(rvDifference1, row,1);
    rvDifference2 = new QLineEdit(central);
    connect(rvDifference2, SIGNAL(returnPressed()),
            this, SLOT(rvDifference2ProcessText()));
    outputLayout->addWidget(rvDifference2, row,2);
    row++;

    computeParamValues = new QCheckBox(tr("Output parameter values"), central);
    connect(computeParamValues, SIGNAL(toggled(bool)),
            this, SLOT(computeParamValuesChanged(bool)));
    outputLayout->addWidget(computeParamValues, row,0);
    computeParamValues->hide();
    row++;

    dumpData = new QCheckBox(tr("Dump Data"), central);
    connect(dumpData, SIGNAL(toggled(bool)),
            this, SLOT(dumpDataChanged(bool)));
    outputLayout->addWidget(dumpData, row,0);
    row++;

    dumpDebug = new QCheckBox(tr("Dump Debug"), central);
    connect(dumpDebug, SIGNAL(toggled(bool)),
            this, SLOT(dumpDebugChanged(bool)));
    outputLayout->addWidget(dumpDebug, row,0);
    row++;

    displayLabel = new QLabel(tr("Display"), central);
    outputLayout->addWidget(displayLabel, row, 0);
    displaySeasonSelect = new QComboBox(central);
    displaySeasonSelect->addItem(tr("Winter"));
    displaySeasonSelect->addItem(tr("Spring"));
    displaySeasonSelect->addItem(tr("Summer"));
    displaySeasonSelect->addItem(tr("Fall"));
    connect(displaySeasonSelect, SIGNAL(activated(int)),
            this, SLOT(displaySeasonChanged(int)));
    outputLayout->addWidget(displaySeasonSelect, row, 1);

    displayMonthSelect = new QComboBox(central);
    displayMonthSelect->addItem(tr("January"));
    displayMonthSelect->addItem(tr("February"));
    displayMonthSelect->addItem(tr("March"));
    displayMonthSelect->addItem(tr("April"));
    displayMonthSelect->addItem(tr("May"));
    displayMonthSelect->addItem(tr("June"));
    displayMonthSelect->addItem(tr("July"));
    displayMonthSelect->addItem(tr("August"));
    displayMonthSelect->addItem(tr("September"));
    displayMonthSelect->addItem(tr("October"));
    displayMonthSelect->addItem(tr("November"));
    displayMonthSelect->addItem(tr("December"));
    connect(displayMonthSelect, SIGNAL(activated(int)),
            this, SLOT(displayMonthChanged(int))); 
    outputLayout->addWidget(displayMonthSelect, row, 1);
}


// ****************************************************************************
// Method: QvisExtremeValueAnalysisWindow::UpdateWindow
//
// Purpose: 
//   Updates the widgets in the window when the subject changes.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisExtremeValueAnalysisWindow::UpdateWindow(bool doAll)
{

    for(int i = 0; i < atts->NumAttributes(); ++i)
    {
        if(!doAll)
        {
            if(!atts->IsSelected(i))
            {
                continue;
            }
        }

        switch(i)
        {
          case ExtremeValueAnalysisAttributes::ID_dataYearBegin:
            dataYearBegin->setText(IntToQString(atts->GetDataYearBegin()));
            break;
          case ExtremeValueAnalysisAttributes::ID_dataAnalysisYearRangeEnabled:
            if (atts->GetDataAnalysisYearRangeEnabled() == true)
            {
                dataAnalysisYear1->setEnabled(true);
                dataAnalysisYear2->setEnabled(true);
            }
            else
            {
                dataAnalysisYear1->setEnabled(false);
                dataAnalysisYear2->setEnabled(false);
            }
            dataAnalysisYearRangeEnabled->blockSignals(true);
            dataAnalysisYearRangeEnabled->setChecked(atts->GetDataAnalysisYearRangeEnabled());
            dataAnalysisYearRangeEnabled->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_dataAnalysisYear1:
            dataAnalysisYear1->setText(IntToQString(atts->GetDataAnalysisYear1()));
            break;
          case ExtremeValueAnalysisAttributes::ID_dataAnalysisYear2:
            dataAnalysisYear2->setText(IntToQString(atts->GetDataAnalysisYear2()));
            break;
          case ExtremeValueAnalysisAttributes::ID_ensemble:
            if (atts->GetEnsemble() == true)
            {
                numEnsembles->setEnabled(true);
                if(numEnsemblesLabel)
                    numEnsemblesLabel->setEnabled(true);
            }
            else
            {
                numEnsembles->setEnabled(false);
                if(numEnsemblesLabel)
                    numEnsemblesLabel->setEnabled(false);
            }
            ensemble->blockSignals(true);
            ensemble->setChecked(atts->GetEnsemble());
            ensemble->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_numEnsembles:
            numEnsembles->setText(IntToQString(atts->GetNumEnsembles()));
            break;
          case ExtremeValueAnalysisAttributes::ID_dataScaling:
            dataScaling->setText(DoubleToQString(atts->GetDataScaling()));
            break;
          case ExtremeValueAnalysisAttributes::ID_extremeMethod:
            extremeMethodButtonGroup->blockSignals(true);
            if(extremeMethodButtonGroup->button((int)atts->GetExtremeMethod()) != 0)
                extremeMethodButtonGroup->button((int)atts->GetExtremeMethod())->setChecked(true);
            extremeMethodButtonGroup->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_optimizationMethod:
            optimizationSelect->blockSignals(true);
            optimizationSelect->setCurrentIndex(int(atts->GetOptimizationMethod()));
            optimizationSelect->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_aggregation:
            displayLabel->hide();
            displayLabel->setEnabled(false);
            displayMonthSelect->hide();
            displayMonthSelect->setEnabled(false);
            displaySeasonSelect->hide();
            displaySeasonSelect->setEnabled(false);

            if (atts->GetAggregation() == ExtremeValueAnalysisAttributes::MONTHLY)
            {
                displayLabel->show();
                displayLabel->setEnabled(true);
                displayMonthSelect->show();
                displayMonthSelect->setEnabled(true);
            }
            if (atts->GetAggregation() == ExtremeValueAnalysisAttributes::SEASONAL)
            {
                displayLabel->show();
                displayLabel->setEnabled(true);
                displaySeasonSelect->show();
                displaySeasonSelect->setEnabled(true);
            }
            aggregationSelect->blockSignals(true);
            aggregationSelect->setCurrentIndex(int(atts->GetAggregation()));
            aggregationSelect->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_covariateModelScale:
            covariateModelScale->blockSignals(true);
            covariateModelScale->setChecked(atts->GetCovariateModelScale());
            covariateModelScale->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_covariateModelLocation:
            covariateModelLocation->blockSignals(true);
            covariateModelLocation->setChecked(atts->GetCovariateModelLocation());
            covariateModelLocation->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_covariateModelShape:
            covariateModelShape->blockSignals(true);
            covariateModelShape->setChecked(atts->GetCovariateModelShape());
            covariateModelShape->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_computeReturnValues:
            if (atts->GetComputeReturnValues() == true)
                returnValues->setEnabled(true);
            else
                returnValues->setEnabled(false);
            computeReturnValues->blockSignals(true);
            computeReturnValues->setChecked(atts->GetComputeReturnValues());
            computeReturnValues->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_returnValues:
            returnValues->setText(IntsToQString(atts->GetReturnValues()));
            break;
          case ExtremeValueAnalysisAttributes::ID_computeRVDifferences:
            if (atts->GetComputeRVDifferences() == true)
                rvDifference1->setEnabled(true);
            else
                rvDifference1->setEnabled(false);
            if (atts->GetComputeRVDifferences() == true)
                rvDifference2->setEnabled(true);
            else
                rvDifference2->setEnabled(false);
            computeRVDifferences->blockSignals(true);
            computeRVDifferences->setChecked(atts->GetComputeRVDifferences());
            computeRVDifferences->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_rvDifference1:
            rvDifference1->setText(IntToQString(atts->GetRvDifference1()));
            break;
          case ExtremeValueAnalysisAttributes::ID_rvDifference2:
            rvDifference2->setText(IntToQString(atts->GetRvDifference2()));
            break;
          case ExtremeValueAnalysisAttributes::ID_displayMonth:
            displayMonthSelect->blockSignals(true);
            displayMonthSelect->setCurrentIndex(int(atts->GetDisplayMonth()));
            displayMonthSelect->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_displaySeason:
            displaySeasonSelect->blockSignals(true);
            displaySeasonSelect->setCurrentIndex(int(atts->GetDisplaySeason()));
            displaySeasonSelect->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_computeParamValues:
            computeParamValues->blockSignals(true);
            computeParamValues->setChecked(atts->GetComputeParamValues());
            computeParamValues->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_dumpData:
            dumpData->blockSignals(true);
            dumpData->setChecked(atts->GetDumpData());
            dumpData->blockSignals(false);
            break;
          case ExtremeValueAnalysisAttributes::ID_dumpDebug:
            dumpDebug->blockSignals(true);
            dumpDebug->setChecked(atts->GetDumpDebug());
            dumpDebug->blockSignals(false);
            break;
        }
    }
}


// ****************************************************************************
// Method: QvisExtremeValueAnalysisWindow::GetCurrentValues
//
// Purpose: 
//   Gets values from certain widgets and stores them in the subject.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisExtremeValueAnalysisWindow::GetCurrentValues(int which_widget)
{
    bool doAll = (which_widget == -1);

    // Do dataYearBegin
    if(which_widget == ExtremeValueAnalysisAttributes::ID_dataYearBegin || doAll)
    {
        int val;
        if(LineEditGetInt(dataYearBegin, val))
            atts->SetDataYearBegin(val);
        else
        {
            ResettingError(tr("dataYearBegin"),
                IntToQString(atts->GetDataYearBegin()));
            atts->SetDataYearBegin(atts->GetDataYearBegin());
        }
    }

    // Do dataAnalysisYear1
    if(which_widget == ExtremeValueAnalysisAttributes::ID_dataAnalysisYear1 || doAll)
    {
        int val;
        if(LineEditGetInt(dataAnalysisYear1, val))
            atts->SetDataAnalysisYear1(val);
        else
        {
            ResettingError(tr("dataAnalysisYear1"),
                IntToQString(atts->GetDataAnalysisYear1()));
            atts->SetDataAnalysisYear1(atts->GetDataAnalysisYear1());
        }
    }

    // Do dataAnalysisYear2
    if(which_widget == ExtremeValueAnalysisAttributes::ID_dataAnalysisYear2 || doAll)
    {
        int val;
        if(LineEditGetInt(dataAnalysisYear2, val))
            atts->SetDataAnalysisYear2(val);
        else
        {
            ResettingError(tr("dataAnalysisYear2"),
                IntToQString(atts->GetDataAnalysisYear2()));
            atts->SetDataAnalysisYear2(atts->GetDataAnalysisYear2());
        }
    }

    // Do numEnsembles
    if(which_widget == ExtremeValueAnalysisAttributes::ID_numEnsembles || doAll)
    {
        int val;
        if(LineEditGetInt(numEnsembles, val))
            atts->SetNumEnsembles(val);
        else
        {
            ResettingError(tr("numEnsembles"),
                IntToQString(atts->GetNumEnsembles()));
            atts->SetNumEnsembles(atts->GetNumEnsembles());
        }
    }

    // Do dataScaling
    if(which_widget == ExtremeValueAnalysisAttributes::ID_dataScaling || doAll)
    {
        double val;
        if(LineEditGetDouble(dataScaling, val))
            atts->SetDataScaling(val);
        else
        {
            ResettingError(tr("Data Scaling"),
                DoubleToQString(atts->GetDataScaling()));
            atts->SetDataScaling(atts->GetDataScaling());
        }
    }

    // Do returnValues
    if(which_widget == ExtremeValueAnalysisAttributes::ID_returnValues || doAll)
    {
        intVector val;
        if(LineEditGetInts(returnValues, val))
            atts->SetReturnValues(val);
        else
        {
            ResettingError(tr("returnValues"),
                           IntsToQString(atts->GetReturnValues()));
            atts->SetReturnValues(atts->GetReturnValues());
        }
    }

    // Do rvDifference1
    if(which_widget == ExtremeValueAnalysisAttributes::ID_rvDifference1 || doAll)
    {
        int val;
        if(LineEditGetInt(rvDifference1, val))
            atts->SetRvDifference1(val);
        else
        {
            ResettingError(tr("rvDifference1"),
                IntToQString(atts->GetRvDifference1()));
            atts->SetRvDifference1(atts->GetRvDifference1());
        }
    }

    // Do rvDifference2
    if(which_widget == ExtremeValueAnalysisAttributes::ID_rvDifference2 || doAll)
    {
        int val;
        if(LineEditGetInt(rvDifference2, val))
            atts->SetRvDifference2(val);
        else
        {
            ResettingError(tr("rvDifference2"),
                IntToQString(atts->GetRvDifference2()));
            atts->SetRvDifference2(atts->GetRvDifference2());
        }
    }

}


//
// Qt Slot functions
//


void
QvisExtremeValueAnalysisWindow::dataYearBeginProcessText()
{
    GetCurrentValues(ExtremeValueAnalysisAttributes::ID_dataYearBegin);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::dataAnalysisYearRangeEnabledChanged(bool val)
{
    atts->SetDataAnalysisYearRangeEnabled(val);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::dataAnalysisYear1ProcessText()
{
    GetCurrentValues(ExtremeValueAnalysisAttributes::ID_dataAnalysisYear1);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::dataAnalysisYear2ProcessText()
{
    GetCurrentValues(ExtremeValueAnalysisAttributes::ID_dataAnalysisYear2);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::ensembleChanged(bool val)
{
    atts->SetEnsemble(val);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::numEnsemblesProcessText()
{
    GetCurrentValues(ExtremeValueAnalysisAttributes::ID_numEnsembles);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::dataScalingProcessText()
{
    GetCurrentValues(ExtremeValueAnalysisAttributes::ID_dataScaling);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::extremeMethodChanged(int val)
{
    if(val != atts->GetExtremeMethod())
    {
        atts->SetExtremeMethod(ExtremeValueAnalysisAttributes::ExtremeType(val));
        SetUpdate(false);
        Apply();
    }
}


void
QvisExtremeValueAnalysisWindow::optimizationChanged(int val)
{
    if(val != atts->GetOptimizationMethod())
    {
        atts->SetOptimizationMethod(ExtremeValueAnalysisAttributes::OptimizationType(val));
        SetUpdate(false);
        Apply();
    }
}


void
QvisExtremeValueAnalysisWindow::aggregationChanged(int val)
{
    if(val != atts->GetAggregation())
    {
        atts->SetAggregation(ExtremeValueAnalysisAttributes::AggregationType(val));
        Apply();
    }
}


void
QvisExtremeValueAnalysisWindow::covariateModelScaleChanged(bool val)
{
    atts->SetCovariateModelScale(val);
    SetUpdate(false);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::covariateModelLocationChanged(bool val)
{
    atts->SetCovariateModelLocation(val);
    SetUpdate(false);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::covariateModelShapeChanged(bool val)
{
    atts->SetCovariateModelShape(val);
    SetUpdate(false);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::computeReturnValuesChanged(bool val)
{
    atts->SetComputeReturnValues(val);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::returnValuesProcessText()
{
    GetCurrentValues(ExtremeValueAnalysisAttributes::ID_returnValues);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::computeRVDifferencesChanged(bool val)
{
    atts->SetComputeRVDifferences(val);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::rvDifference1ProcessText()
{
    GetCurrentValues(ExtremeValueAnalysisAttributes::ID_rvDifference1);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::rvDifference2ProcessText()
{
    GetCurrentValues(ExtremeValueAnalysisAttributes::ID_rvDifference2);
    Apply();
}


void
QvisExtremeValueAnalysisWindow::displayMonthChanged(int val)
{
    if(val != atts->GetDisplayMonth())
    {
        atts->SetDisplayMonth(ExtremeValueAnalysisAttributes::MonthType(val));
        SetUpdate(false);
        Apply();
    }
}


void
QvisExtremeValueAnalysisWindow::displaySeasonChanged(int val)
{
    if(val != atts->GetDisplaySeason())
    {
        atts->SetDisplaySeason(ExtremeValueAnalysisAttributes::SeasonType(val));
        SetUpdate(false);
        Apply();
    }
}

void
QvisExtremeValueAnalysisWindow::computeParamValuesChanged(bool val)
{
    atts->SetComputeParamValues(val);
    SetUpdate(false);
    Apply();
}

void
QvisExtremeValueAnalysisWindow::dumpDataChanged(bool val)
{
    atts->SetDumpData(val);
    SetUpdate(false);
    Apply();
}

void
QvisExtremeValueAnalysisWindow::dumpDebugChanged(bool val)
{
    atts->SetDumpDebug(val);
    SetUpdate(false);
    Apply();
}


