/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

// ************************************************************************* //
//  File: avtExtremeValueAnalysisFilter.C
// ************************************************************************* //

#include <avtExtremeValueAnalysisFilter.h>
#include <avtRExtremesFilter.h>
#include <avtDatasetExaminer.h>
#include <avtContract.h>
#include <avtSourceFromAVTDataset.h>
#include <vtkDataSet.h>
#include <vtkPointData.h>
#include <vtkCellData.h>
#include <vtkFloatArray.h>
#include <string.h>
#include <DebugStream.h>
#include <InstallationFunctions.h>
#include <string>


// ****************************************************************************
//  Method: avtExtremeValueAnalysisFilter constructor
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
// ****************************************************************************

avtExtremeValueAnalysisFilter::avtExtremeValueAnalysisFilter()
{
}


// ****************************************************************************
//  Method: avtExtremeValueAnalysisFilter destructor
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
//  Modifications:
//
// ****************************************************************************

avtExtremeValueAnalysisFilter::~avtExtremeValueAnalysisFilter()
{
}


// ****************************************************************************
//  Method:  avtExtremeValueAnalysisFilter::Create
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
// ****************************************************************************

avtFilter *
avtExtremeValueAnalysisFilter::Create()
{
    return new avtExtremeValueAnalysisFilter();
}


// ****************************************************************************
//  Method:      avtExtremeValueAnalysisFilter::SetAtts
//
//  Purpose:
//      Sets the state of the filter based on the attribute object.
//
//  Arguments:
//      a        The attributes to use.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
// ****************************************************************************

void
avtExtremeValueAnalysisFilter::SetAtts(const AttributeGroup *a)
{
    atts = *(const ExtremeValueAnalysisAttributes*)a;
}


// ****************************************************************************
//  Method: avtExtremeValueAnalysisFilter::Equivalent
//
//  Purpose:
//      Returns true if creating a new avtExtremeValueAnalysisFilter with the given
//      parameters would result in an equivalent avtExtremeValueAnalysisFilter.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
// ****************************************************************************

bool
avtExtremeValueAnalysisFilter::Equivalent(const AttributeGroup *a)
{
    return (atts == *(ExtremeValueAnalysisAttributes*)a);
}


// ****************************************************************************
//  Method: avtExtremeValueAnalysisFilter::Execute
//
//  Purpose:
//      Sends the specified input and output through the ExtremeValueAnalysis filter.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
//  Modifications:
//
//   Dave Pugmire, Wed Apr 11 14:53:44 EDT 2012
//   Name the output scalar coming out of avtRExtremesFilter
//
//   Kathleen Biagas, Fri May 4 14:05:27 PDT 2012
//   Call GetVisItLibraryDirectory instead of GetVisItArchitectureDirectory.
//
// ****************************************************************************

void
avtExtremeValueAnalysisFilter::Execute()
{
    avtRExtremesFilter *f = new avtRExtremesFilter();
    f->atts = atts;

    std::string vlibdir = GetVisItLibraryDirectory() + VISIT_SLASH_CHAR + "r_support";
    std::string vlibrdir  = vlibdir  + VISIT_SLASH_CHAR + "Rscripts" + VISIT_SLASH_CHAR;
    f->codeDir = vlibrdir;

    f->SetInput(GetInput());
    
    avtContract_p spec = GetInput()->GetOriginatingSource()->GetGeneralContract();
    avtDataObject_p dob = f->GetOutput();

    dob->Update(spec);
    avtDataTree_p tree = f->GetTypedOutput()->GetDataTree();

    //Set the output variable properly.
    int nleaves;
    vtkDataSet **leaves = tree->GetAllLeaves(nleaves);
    for (int i = 0; i < nleaves; i++)
    {
        if (leaves[i]->GetPointData()->GetScalars())
            leaves[i]->GetPointData()->GetScalars()->SetName(pipelineVariable);
        else if (leaves[i]->GetCellData()->GetScalars())
            leaves[i]->GetCellData()->GetScalars()->SetName(pipelineVariable);
    }
    delete [] leaves;
            

    SetOutputDataTree(tree);
    delete f;
}


// ****************************************************************************
//  Method: avtExtremeValueAnalysisFilter::ModifyContract
//
//  Purpose:
//      Creates a contract the removes the operator-created-expression.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
// ****************************************************************************

avtContract_p
avtExtremeValueAnalysisFilter::ModifyContract(avtContract_p in_contract)
{
    avtContract_p rv;
    if (strncmp(pipelineVariable, "operators/ExtremeValueAnalysis", strlen("operators/ExtremeValueAnalysis")) == 0)
    {
        outVarName = pipelineVariable;
        const char *justTheVar = pipelineVariable + strlen("operators/ExtremeValueAnalysis/");
        avtDataRequest_p dr = new avtDataRequest(in_contract->GetDataRequest(), justTheVar);
        rv = new avtContract(in_contract, dr);
    }
    else
    {
        rv = new avtContract(in_contract);
    }
    return rv;
}


// ****************************************************************************
//  Method: avtExtremeValueAnalysisFilter::UpdateDataObjectInfo
//
//  Purpose:
//      Tells output that we have a new variable.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
// ****************************************************************************

void
avtExtremeValueAnalysisFilter::UpdateDataObjectInfo(void)
{
   if (outVarName != "")
   {
       avtDataAttributes &atts = GetOutput()->GetInfo().GetAttributes();
       if (! atts.ValidVariable(outVarName))
       {
           atts.AddVariable(outVarName.c_str());
           atts.SetActiveVariable(outVarName.c_str());
           atts.SetVariableDimension(1);
           atts.SetVariableType(AVT_SCALAR_VAR);
           atts.SetCentering(AVT_NODECENT);
           // you might want to set up extents too
       }
   }
}


