/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include <PyFTLEAttributes.h>
#include <ObserverToCallback.h>
#include <stdio.h>
#include <snprintf.h>

// ****************************************************************************
// Module: PyFTLEAttributes
//
// Purpose: 
//   Attributes for FTLE
//
// Note:       Autogenerated by xml2python. Do not modify by hand!
//
// Programmer: xml2python
// Creation:   omitted
//
// ****************************************************************************

//
// This struct contains the Python type information and a FTLEAttributes.
//
struct FTLEAttributesObject
{
    PyObject_HEAD
    FTLEAttributes *data;
    bool        owns;
    PyObject   *parent;
};

//
// Internal prototypes
//
static PyObject *NewFTLEAttributes(int);

std::string
PyFTLEAttributes_ToString(const FTLEAttributes *atts, const char *prefix)
{
    std::string str;
    char tmpStr[1000];

    const char *sourceType_names = "NativeMesh, RegularGrid";
    switch (atts->GetSourceType())
    {
      case FTLEAttributes::NativeMesh:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sNativeMesh  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::RegularGrid:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sRegularGrid  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    {   const int *Resolution = atts->GetResolution();
        SNPRINTF(tmpStr, 1000, "%sResolution = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%d", Resolution[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    const char *UseDataSetStart_names = "Full, Subset";
    switch (atts->GetUseDataSetStart())
    {
      case FTLEAttributes::Full:
          SNPRINTF(tmpStr, 1000, "%sUseDataSetStart = %sFull  # %s\n", prefix, prefix, UseDataSetStart_names);
          str += tmpStr;
          break;
      case FTLEAttributes::Subset:
          SNPRINTF(tmpStr, 1000, "%sUseDataSetStart = %sSubset  # %s\n", prefix, prefix, UseDataSetStart_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    {   const double *StartPosition = atts->GetStartPosition();
        SNPRINTF(tmpStr, 1000, "%sStartPosition = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", StartPosition[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    const char *UseDataSetEnd_names = "Full, Subset";
    switch (atts->GetUseDataSetEnd())
    {
      case FTLEAttributes::Full:
          SNPRINTF(tmpStr, 1000, "%sUseDataSetEnd = %sFull  # %s\n", prefix, prefix, UseDataSetEnd_names);
          str += tmpStr;
          break;
      case FTLEAttributes::Subset:
          SNPRINTF(tmpStr, 1000, "%sUseDataSetEnd = %sSubset  # %s\n", prefix, prefix, UseDataSetEnd_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    {   const double *EndPosition = atts->GetEndPosition();
        SNPRINTF(tmpStr, 1000, "%sEndPosition = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", EndPosition[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    const char *integrationDirection_names = "Forward, Backward, Both";
    switch (atts->GetIntegrationDirection())
    {
      case FTLEAttributes::Forward:
          SNPRINTF(tmpStr, 1000, "%sintegrationDirection = %sForward  # %s\n", prefix, prefix, integrationDirection_names);
          str += tmpStr;
          break;
      case FTLEAttributes::Backward:
          SNPRINTF(tmpStr, 1000, "%sintegrationDirection = %sBackward  # %s\n", prefix, prefix, integrationDirection_names);
          str += tmpStr;
          break;
      case FTLEAttributes::Both:
          SNPRINTF(tmpStr, 1000, "%sintegrationDirection = %sBoth  # %s\n", prefix, prefix, integrationDirection_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%smaxSteps = %d\n", prefix, atts->GetMaxSteps());
    str += tmpStr;
    const char *terminationType_names = "Time, Distance, Size";
    switch (atts->GetTerminationType())
    {
      case FTLEAttributes::Time:
          SNPRINTF(tmpStr, 1000, "%sterminationType = %sTime  # %s\n", prefix, prefix, terminationType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::Distance:
          SNPRINTF(tmpStr, 1000, "%sterminationType = %sDistance  # %s\n", prefix, prefix, terminationType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::Size:
          SNPRINTF(tmpStr, 1000, "%sterminationType = %sSize  # %s\n", prefix, prefix, terminationType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    if(atts->GetTerminateBySize())
        SNPRINTF(tmpStr, 1000, "%sterminateBySize = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sterminateBySize = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%stermSize = %g\n", prefix, atts->GetTermSize());
    str += tmpStr;
    if(atts->GetTerminateByDistance())
        SNPRINTF(tmpStr, 1000, "%sterminateByDistance = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sterminateByDistance = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%stermDistance = %g\n", prefix, atts->GetTermDistance());
    str += tmpStr;
    if(atts->GetTerminateByTime())
        SNPRINTF(tmpStr, 1000, "%sterminateByTime = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sterminateByTime = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%stermTime = %g\n", prefix, atts->GetTermTime());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%smaxStepLength = %g\n", prefix, atts->GetMaxStepLength());
    str += tmpStr;
    if(atts->GetLimitMaximumTimestep())
        SNPRINTF(tmpStr, 1000, "%slimitMaximumTimestep = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%slimitMaximumTimestep = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%smaxTimeStep = %g\n", prefix, atts->GetMaxTimeStep());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%srelTol = %g\n", prefix, atts->GetRelTol());
    str += tmpStr;
    const char *absTolSizeType_names = "Absolute, FractionOfBBox";
    switch (atts->GetAbsTolSizeType())
    {
      case FTLEAttributes::Absolute:
          SNPRINTF(tmpStr, 1000, "%sabsTolSizeType = %sAbsolute  # %s\n", prefix, prefix, absTolSizeType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::FractionOfBBox:
          SNPRINTF(tmpStr, 1000, "%sabsTolSizeType = %sFractionOfBBox  # %s\n", prefix, prefix, absTolSizeType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sabsTolAbsolute = %g\n", prefix, atts->GetAbsTolAbsolute());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sabsTolBBox = %g\n", prefix, atts->GetAbsTolBBox());
    str += tmpStr;
    const char *fieldType_names = "Default, FlashField, M3DC12DField, M3DC13DField, Nek5000Field, "
        "NIMRODField";
    switch (atts->GetFieldType())
    {
      case FTLEAttributes::Default:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sDefault  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::FlashField:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sFlashField  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::M3DC12DField:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sM3DC12DField  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::M3DC13DField:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sM3DC13DField  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::Nek5000Field:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sNek5000Field  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::NIMRODField:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sNIMRODField  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sfieldConstant = %g\n", prefix, atts->GetFieldConstant());
    str += tmpStr;
    {   const double *velocitySource = atts->GetVelocitySource();
        SNPRINTF(tmpStr, 1000, "%svelocitySource = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", velocitySource[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    const char *integrationType_names = "Euler, Leapfrog, DormandPrince, AdamsBashforth, RK4, "
        "M3DC12DIntegrator";
    switch (atts->GetIntegrationType())
    {
      case FTLEAttributes::Euler:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sEuler  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::Leapfrog:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sLeapfrog  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::DormandPrince:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sDormandPrince  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::AdamsBashforth:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sAdamsBashforth  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::RK4:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sRK4  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::M3DC12DIntegrator:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sM3DC12DIntegrator  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *parallelizationAlgorithmType_names = "LoadOnDemand, ParallelStaticDomains, MasterSlave, VisItSelects";
    switch (atts->GetParallelizationAlgorithmType())
    {
      case FTLEAttributes::LoadOnDemand:
          SNPRINTF(tmpStr, 1000, "%sparallelizationAlgorithmType = %sLoadOnDemand  # %s\n", prefix, prefix, parallelizationAlgorithmType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::ParallelStaticDomains:
          SNPRINTF(tmpStr, 1000, "%sparallelizationAlgorithmType = %sParallelStaticDomains  # %s\n", prefix, prefix, parallelizationAlgorithmType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::MasterSlave:
          SNPRINTF(tmpStr, 1000, "%sparallelizationAlgorithmType = %sMasterSlave  # %s\n", prefix, prefix, parallelizationAlgorithmType_names);
          str += tmpStr;
          break;
      case FTLEAttributes::VisItSelects:
          SNPRINTF(tmpStr, 1000, "%sparallelizationAlgorithmType = %sVisItSelects  # %s\n", prefix, prefix, parallelizationAlgorithmType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%smaxProcessCount = %d\n", prefix, atts->GetMaxProcessCount());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%smaxDomainCacheSize = %d\n", prefix, atts->GetMaxDomainCacheSize());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sworkGroupSize = %d\n", prefix, atts->GetWorkGroupSize());
    str += tmpStr;
    if(atts->GetPathlines())
        SNPRINTF(tmpStr, 1000, "%spathlines = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%spathlines = 0\n", prefix);
    str += tmpStr;
    if(atts->GetPathlinesOverrideStartingTimeFlag())
        SNPRINTF(tmpStr, 1000, "%spathlinesOverrideStartingTimeFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%spathlinesOverrideStartingTimeFlag = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%spathlinesOverrideStartingTime = %g\n", prefix, atts->GetPathlinesOverrideStartingTime());
    str += tmpStr;
    const char *pathlinesCMFE_names = "CONN_CMFE, POS_CMFE";
    switch (atts->GetPathlinesCMFE())
    {
      case FTLEAttributes::CONN_CMFE:
          SNPRINTF(tmpStr, 1000, "%spathlinesCMFE = %sCONN_CMFE  # %s\n", prefix, prefix, pathlinesCMFE_names);
          str += tmpStr;
          break;
      case FTLEAttributes::POS_CMFE:
          SNPRINTF(tmpStr, 1000, "%spathlinesCMFE = %sPOS_CMFE  # %s\n", prefix, prefix, pathlinesCMFE_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    if(atts->GetForceNodeCenteredData())
        SNPRINTF(tmpStr, 1000, "%sforceNodeCenteredData = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sforceNodeCenteredData = 0\n", prefix);
    str += tmpStr;
    if(atts->GetIssueTerminationWarnings())
        SNPRINTF(tmpStr, 1000, "%sissueTerminationWarnings = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sissueTerminationWarnings = 0\n", prefix);
    str += tmpStr;
    if(atts->GetIssueStiffnessWarnings())
        SNPRINTF(tmpStr, 1000, "%sissueStiffnessWarnings = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sissueStiffnessWarnings = 0\n", prefix);
    str += tmpStr;
    if(atts->GetIssueCriticalPointsWarnings())
        SNPRINTF(tmpStr, 1000, "%sissueCriticalPointsWarnings = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sissueCriticalPointsWarnings = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%scriticalPointThreshold = %g\n", prefix, atts->GetCriticalPointThreshold());
    str += tmpStr;
    return str;
}

static PyObject *
FTLEAttributes_Notify(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    obj->data->Notify();
    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_SetSourceType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the sourceType in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetSourceType(FTLEAttributes::SourceType(ival));
    else
    {
        fprintf(stderr, "An invalid sourceType value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "NativeMesh, RegularGrid.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetSourceType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetSourceType()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetResolution(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int *ivals = obj->data->GetResolution();
    if(!PyArg_ParseTuple(args, "iii", &ivals[0], &ivals[1], &ivals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    ivals[i] = int(PyFloat_AS_DOUBLE(item));
                else if(PyInt_Check(item))
                    ivals[i] = int(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    ivals[i] = int(PyLong_AsDouble(item));
                else
                    ivals[i] = 0;
            }
        }
        else
            return NULL;
    }

    // Mark the Resolution in the object as modified.
    obj->data->SelectResolution();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetResolution(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the Resolution.
    PyObject *retval = PyTuple_New(3);
    const int *Resolution = obj->data->GetResolution();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyInt_FromLong(long(Resolution[i])));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetUseDataSetStart(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the UseDataSetStart in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetUseDataSetStart(FTLEAttributes::Extents(ival));
    else
    {
        fprintf(stderr, "An invalid UseDataSetStart value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Full, Subset.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetUseDataSetStart(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetUseDataSetStart()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetStartPosition(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double *dvals = obj->data->GetStartPosition();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the StartPosition in the object as modified.
    obj->data->SelectStartPosition();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetStartPosition(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the StartPosition.
    PyObject *retval = PyTuple_New(3);
    const double *StartPosition = obj->data->GetStartPosition();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(StartPosition[i]));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetUseDataSetEnd(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the UseDataSetEnd in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetUseDataSetEnd(FTLEAttributes::Extents(ival));
    else
    {
        fprintf(stderr, "An invalid UseDataSetEnd value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Full, Subset.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetUseDataSetEnd(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetUseDataSetEnd()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetEndPosition(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double *dvals = obj->data->GetEndPosition();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the EndPosition in the object as modified.
    obj->data->SelectEndPosition();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetEndPosition(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the EndPosition.
    PyObject *retval = PyTuple_New(3);
    const double *EndPosition = obj->data->GetEndPosition();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(EndPosition[i]));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetIntegrationDirection(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the integrationDirection in the object.
    if(ival >= 0 && ival < 3)
        obj->data->SetIntegrationDirection(FTLEAttributes::IntegrationDirection(ival));
    else
    {
        fprintf(stderr, "An invalid integrationDirection value was given. "
                        "Valid values are in the range of [0,2]. "
                        "You can also use the following names: "
                        "Forward, Backward, Both.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetIntegrationDirection(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetIntegrationDirection()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetMaxSteps(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the maxSteps in the object.
    obj->data->SetMaxSteps((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetMaxSteps(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetMaxSteps()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetTerminationType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the terminationType in the object.
    if(ival >= 0 && ival < 3)
        obj->data->SetTerminationType(FTLEAttributes::TerminationType(ival));
    else
    {
        fprintf(stderr, "An invalid terminationType value was given. "
                        "Valid values are in the range of [0,2]. "
                        "You can also use the following names: "
                        "Time, Distance, Size.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetTerminationType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetTerminationType()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetTerminateBySize(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the terminateBySize in the object.
    obj->data->SetTerminateBySize(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetTerminateBySize(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetTerminateBySize()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetTermSize(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the termSize in the object.
    obj->data->SetTermSize(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetTermSize(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetTermSize());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetTerminateByDistance(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the terminateByDistance in the object.
    obj->data->SetTerminateByDistance(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetTerminateByDistance(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetTerminateByDistance()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetTermDistance(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the termDistance in the object.
    obj->data->SetTermDistance(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetTermDistance(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetTermDistance());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetTerminateByTime(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the terminateByTime in the object.
    obj->data->SetTerminateByTime(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetTerminateByTime(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetTerminateByTime()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetTermTime(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the termTime in the object.
    obj->data->SetTermTime(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetTermTime(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetTermTime());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetMaxStepLength(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the maxStepLength in the object.
    obj->data->SetMaxStepLength(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetMaxStepLength(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetMaxStepLength());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetLimitMaximumTimestep(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the limitMaximumTimestep in the object.
    obj->data->SetLimitMaximumTimestep(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetLimitMaximumTimestep(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetLimitMaximumTimestep()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetMaxTimeStep(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the maxTimeStep in the object.
    obj->data->SetMaxTimeStep(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetMaxTimeStep(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetMaxTimeStep());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetRelTol(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the relTol in the object.
    obj->data->SetRelTol(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetRelTol(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetRelTol());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetAbsTolSizeType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the absTolSizeType in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetAbsTolSizeType(FTLEAttributes::SizeType(ival));
    else
    {
        fprintf(stderr, "An invalid absTolSizeType value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Absolute, FractionOfBBox.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetAbsTolSizeType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetAbsTolSizeType()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetAbsTolAbsolute(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the absTolAbsolute in the object.
    obj->data->SetAbsTolAbsolute(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetAbsTolAbsolute(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetAbsTolAbsolute());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetAbsTolBBox(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the absTolBBox in the object.
    obj->data->SetAbsTolBBox(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetAbsTolBBox(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetAbsTolBBox());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetFieldType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the fieldType in the object.
    if(ival >= 0 && ival < 6)
        obj->data->SetFieldType(FTLEAttributes::FieldType(ival));
    else
    {
        fprintf(stderr, "An invalid fieldType value was given. "
                        "Valid values are in the range of [0,5]. "
                        "You can also use the following names: "
                        "Default, FlashField, M3DC12DField, M3DC13DField, Nek5000Field, "
                        "NIMRODField.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetFieldType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetFieldType()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetFieldConstant(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the fieldConstant in the object.
    obj->data->SetFieldConstant(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetFieldConstant(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetFieldConstant());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetVelocitySource(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double *dvals = obj->data->GetVelocitySource();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the velocitySource in the object as modified.
    obj->data->SelectVelocitySource();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetVelocitySource(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the velocitySource.
    PyObject *retval = PyTuple_New(3);
    const double *velocitySource = obj->data->GetVelocitySource();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(velocitySource[i]));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetIntegrationType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the integrationType in the object.
    if(ival >= 0 && ival < 6)
        obj->data->SetIntegrationType(FTLEAttributes::IntegrationType(ival));
    else
    {
        fprintf(stderr, "An invalid integrationType value was given. "
                        "Valid values are in the range of [0,5]. "
                        "You can also use the following names: "
                        "Euler, Leapfrog, DormandPrince, AdamsBashforth, RK4, "
                        "M3DC12DIntegrator.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetIntegrationType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetIntegrationType()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetParallelizationAlgorithmType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the parallelizationAlgorithmType in the object.
    if(ival >= 0 && ival < 4)
        obj->data->SetParallelizationAlgorithmType(FTLEAttributes::ParallelizationAlgorithmType(ival));
    else
    {
        fprintf(stderr, "An invalid parallelizationAlgorithmType value was given. "
                        "Valid values are in the range of [0,3]. "
                        "You can also use the following names: "
                        "LoadOnDemand, ParallelStaticDomains, MasterSlave, VisItSelects.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetParallelizationAlgorithmType(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetParallelizationAlgorithmType()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetMaxProcessCount(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the maxProcessCount in the object.
    obj->data->SetMaxProcessCount((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetMaxProcessCount(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetMaxProcessCount()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetMaxDomainCacheSize(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the maxDomainCacheSize in the object.
    obj->data->SetMaxDomainCacheSize((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetMaxDomainCacheSize(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetMaxDomainCacheSize()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetWorkGroupSize(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the workGroupSize in the object.
    obj->data->SetWorkGroupSize((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetWorkGroupSize(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetWorkGroupSize()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetPathlines(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the pathlines in the object.
    obj->data->SetPathlines(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetPathlines(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetPathlines()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetPathlinesOverrideStartingTimeFlag(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the pathlinesOverrideStartingTimeFlag in the object.
    obj->data->SetPathlinesOverrideStartingTimeFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetPathlinesOverrideStartingTimeFlag(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetPathlinesOverrideStartingTimeFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetPathlinesOverrideStartingTime(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the pathlinesOverrideStartingTime in the object.
    obj->data->SetPathlinesOverrideStartingTime(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetPathlinesOverrideStartingTime(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetPathlinesOverrideStartingTime());
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetPathlinesCMFE(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the pathlinesCMFE in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetPathlinesCMFE(FTLEAttributes::PathlinesCMFE(ival));
    else
    {
        fprintf(stderr, "An invalid pathlinesCMFE value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "CONN_CMFE, POS_CMFE.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetPathlinesCMFE(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetPathlinesCMFE()));
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetForceNodeCenteredData(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the forceNodeCenteredData in the object.
    obj->data->SetForceNodeCenteredData(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetForceNodeCenteredData(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetForceNodeCenteredData()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetIssueTerminationWarnings(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the issueTerminationWarnings in the object.
    obj->data->SetIssueTerminationWarnings(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetIssueTerminationWarnings(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetIssueTerminationWarnings()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetIssueStiffnessWarnings(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the issueStiffnessWarnings in the object.
    obj->data->SetIssueStiffnessWarnings(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetIssueStiffnessWarnings(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetIssueStiffnessWarnings()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetIssueCriticalPointsWarnings(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the issueCriticalPointsWarnings in the object.
    obj->data->SetIssueCriticalPointsWarnings(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetIssueCriticalPointsWarnings(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetIssueCriticalPointsWarnings()?1L:0L);
    return retval;
}

/*static*/ PyObject *
FTLEAttributes_SetCriticalPointThreshold(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the criticalPointThreshold in the object.
    obj->data->SetCriticalPointThreshold(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
FTLEAttributes_GetCriticalPointThreshold(PyObject *self, PyObject *args)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetCriticalPointThreshold());
    return retval;
}



PyMethodDef PyFTLEAttributes_methods[FTLEATTRIBUTES_NMETH] = {
    {"Notify", FTLEAttributes_Notify, METH_VARARGS},
    {"SetSourceType", FTLEAttributes_SetSourceType, METH_VARARGS},
    {"GetSourceType", FTLEAttributes_GetSourceType, METH_VARARGS},
    {"SetResolution", FTLEAttributes_SetResolution, METH_VARARGS},
    {"GetResolution", FTLEAttributes_GetResolution, METH_VARARGS},
    {"SetUseDataSetStart", FTLEAttributes_SetUseDataSetStart, METH_VARARGS},
    {"GetUseDataSetStart", FTLEAttributes_GetUseDataSetStart, METH_VARARGS},
    {"SetStartPosition", FTLEAttributes_SetStartPosition, METH_VARARGS},
    {"GetStartPosition", FTLEAttributes_GetStartPosition, METH_VARARGS},
    {"SetUseDataSetEnd", FTLEAttributes_SetUseDataSetEnd, METH_VARARGS},
    {"GetUseDataSetEnd", FTLEAttributes_GetUseDataSetEnd, METH_VARARGS},
    {"SetEndPosition", FTLEAttributes_SetEndPosition, METH_VARARGS},
    {"GetEndPosition", FTLEAttributes_GetEndPosition, METH_VARARGS},
    {"SetIntegrationDirection", FTLEAttributes_SetIntegrationDirection, METH_VARARGS},
    {"GetIntegrationDirection", FTLEAttributes_GetIntegrationDirection, METH_VARARGS},
    {"SetMaxSteps", FTLEAttributes_SetMaxSteps, METH_VARARGS},
    {"GetMaxSteps", FTLEAttributes_GetMaxSteps, METH_VARARGS},
    {"SetTerminationType", FTLEAttributes_SetTerminationType, METH_VARARGS},
    {"GetTerminationType", FTLEAttributes_GetTerminationType, METH_VARARGS},
    {"SetTerminateBySize", FTLEAttributes_SetTerminateBySize, METH_VARARGS},
    {"GetTerminateBySize", FTLEAttributes_GetTerminateBySize, METH_VARARGS},
    {"SetTermSize", FTLEAttributes_SetTermSize, METH_VARARGS},
    {"GetTermSize", FTLEAttributes_GetTermSize, METH_VARARGS},
    {"SetTerminateByDistance", FTLEAttributes_SetTerminateByDistance, METH_VARARGS},
    {"GetTerminateByDistance", FTLEAttributes_GetTerminateByDistance, METH_VARARGS},
    {"SetTermDistance", FTLEAttributes_SetTermDistance, METH_VARARGS},
    {"GetTermDistance", FTLEAttributes_GetTermDistance, METH_VARARGS},
    {"SetTerminateByTime", FTLEAttributes_SetTerminateByTime, METH_VARARGS},
    {"GetTerminateByTime", FTLEAttributes_GetTerminateByTime, METH_VARARGS},
    {"SetTermTime", FTLEAttributes_SetTermTime, METH_VARARGS},
    {"GetTermTime", FTLEAttributes_GetTermTime, METH_VARARGS},
    {"SetMaxStepLength", FTLEAttributes_SetMaxStepLength, METH_VARARGS},
    {"GetMaxStepLength", FTLEAttributes_GetMaxStepLength, METH_VARARGS},
    {"SetLimitMaximumTimestep", FTLEAttributes_SetLimitMaximumTimestep, METH_VARARGS},
    {"GetLimitMaximumTimestep", FTLEAttributes_GetLimitMaximumTimestep, METH_VARARGS},
    {"SetMaxTimeStep", FTLEAttributes_SetMaxTimeStep, METH_VARARGS},
    {"GetMaxTimeStep", FTLEAttributes_GetMaxTimeStep, METH_VARARGS},
    {"SetRelTol", FTLEAttributes_SetRelTol, METH_VARARGS},
    {"GetRelTol", FTLEAttributes_GetRelTol, METH_VARARGS},
    {"SetAbsTolSizeType", FTLEAttributes_SetAbsTolSizeType, METH_VARARGS},
    {"GetAbsTolSizeType", FTLEAttributes_GetAbsTolSizeType, METH_VARARGS},
    {"SetAbsTolAbsolute", FTLEAttributes_SetAbsTolAbsolute, METH_VARARGS},
    {"GetAbsTolAbsolute", FTLEAttributes_GetAbsTolAbsolute, METH_VARARGS},
    {"SetAbsTolBBox", FTLEAttributes_SetAbsTolBBox, METH_VARARGS},
    {"GetAbsTolBBox", FTLEAttributes_GetAbsTolBBox, METH_VARARGS},
    {"SetFieldType", FTLEAttributes_SetFieldType, METH_VARARGS},
    {"GetFieldType", FTLEAttributes_GetFieldType, METH_VARARGS},
    {"SetFieldConstant", FTLEAttributes_SetFieldConstant, METH_VARARGS},
    {"GetFieldConstant", FTLEAttributes_GetFieldConstant, METH_VARARGS},
    {"SetVelocitySource", FTLEAttributes_SetVelocitySource, METH_VARARGS},
    {"GetVelocitySource", FTLEAttributes_GetVelocitySource, METH_VARARGS},
    {"SetIntegrationType", FTLEAttributes_SetIntegrationType, METH_VARARGS},
    {"GetIntegrationType", FTLEAttributes_GetIntegrationType, METH_VARARGS},
    {"SetParallelizationAlgorithmType", FTLEAttributes_SetParallelizationAlgorithmType, METH_VARARGS},
    {"GetParallelizationAlgorithmType", FTLEAttributes_GetParallelizationAlgorithmType, METH_VARARGS},
    {"SetMaxProcessCount", FTLEAttributes_SetMaxProcessCount, METH_VARARGS},
    {"GetMaxProcessCount", FTLEAttributes_GetMaxProcessCount, METH_VARARGS},
    {"SetMaxDomainCacheSize", FTLEAttributes_SetMaxDomainCacheSize, METH_VARARGS},
    {"GetMaxDomainCacheSize", FTLEAttributes_GetMaxDomainCacheSize, METH_VARARGS},
    {"SetWorkGroupSize", FTLEAttributes_SetWorkGroupSize, METH_VARARGS},
    {"GetWorkGroupSize", FTLEAttributes_GetWorkGroupSize, METH_VARARGS},
    {"SetPathlines", FTLEAttributes_SetPathlines, METH_VARARGS},
    {"GetPathlines", FTLEAttributes_GetPathlines, METH_VARARGS},
    {"SetPathlinesOverrideStartingTimeFlag", FTLEAttributes_SetPathlinesOverrideStartingTimeFlag, METH_VARARGS},
    {"GetPathlinesOverrideStartingTimeFlag", FTLEAttributes_GetPathlinesOverrideStartingTimeFlag, METH_VARARGS},
    {"SetPathlinesOverrideStartingTime", FTLEAttributes_SetPathlinesOverrideStartingTime, METH_VARARGS},
    {"GetPathlinesOverrideStartingTime", FTLEAttributes_GetPathlinesOverrideStartingTime, METH_VARARGS},
    {"SetPathlinesCMFE", FTLEAttributes_SetPathlinesCMFE, METH_VARARGS},
    {"GetPathlinesCMFE", FTLEAttributes_GetPathlinesCMFE, METH_VARARGS},
    {"SetForceNodeCenteredData", FTLEAttributes_SetForceNodeCenteredData, METH_VARARGS},
    {"GetForceNodeCenteredData", FTLEAttributes_GetForceNodeCenteredData, METH_VARARGS},
    {"SetIssueTerminationWarnings", FTLEAttributes_SetIssueTerminationWarnings, METH_VARARGS},
    {"GetIssueTerminationWarnings", FTLEAttributes_GetIssueTerminationWarnings, METH_VARARGS},
    {"SetIssueStiffnessWarnings", FTLEAttributes_SetIssueStiffnessWarnings, METH_VARARGS},
    {"GetIssueStiffnessWarnings", FTLEAttributes_GetIssueStiffnessWarnings, METH_VARARGS},
    {"SetIssueCriticalPointsWarnings", FTLEAttributes_SetIssueCriticalPointsWarnings, METH_VARARGS},
    {"GetIssueCriticalPointsWarnings", FTLEAttributes_GetIssueCriticalPointsWarnings, METH_VARARGS},
    {"SetCriticalPointThreshold", FTLEAttributes_SetCriticalPointThreshold, METH_VARARGS},
    {"GetCriticalPointThreshold", FTLEAttributes_GetCriticalPointThreshold, METH_VARARGS},
    {NULL, NULL}
};

//
// Type functions
//

static void
FTLEAttributes_dealloc(PyObject *v)
{
   FTLEAttributesObject *obj = (FTLEAttributesObject *)v;
   if(obj->parent != 0)
       Py_DECREF(obj->parent);
   if(obj->owns)
       delete obj->data;
}

static int
FTLEAttributes_compare(PyObject *v, PyObject *w)
{
    FTLEAttributes *a = ((FTLEAttributesObject *)v)->data;
    FTLEAttributes *b = ((FTLEAttributesObject *)w)->data;
    return (*a == *b) ? 0 : -1;
}

PyObject *
PyFTLEAttributes_getattr(PyObject *self, char *name)
{
    if(strcmp(name, "sourceType") == 0)
        return FTLEAttributes_GetSourceType(self, NULL);
    if(strcmp(name, "NativeMesh") == 0)
        return PyInt_FromLong(long(FTLEAttributes::NativeMesh));
    if(strcmp(name, "RegularGrid") == 0)
        return PyInt_FromLong(long(FTLEAttributes::RegularGrid));

    if(strcmp(name, "Resolution") == 0)
        return FTLEAttributes_GetResolution(self, NULL);
    if(strcmp(name, "UseDataSetStart") == 0)
        return FTLEAttributes_GetUseDataSetStart(self, NULL);
    if(strcmp(name, "Full") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Full));
    if(strcmp(name, "Subset") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Subset));

    if(strcmp(name, "StartPosition") == 0)
        return FTLEAttributes_GetStartPosition(self, NULL);
    if(strcmp(name, "UseDataSetEnd") == 0)
        return FTLEAttributes_GetUseDataSetEnd(self, NULL);
    if(strcmp(name, "Full") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Full));
    if(strcmp(name, "Subset") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Subset));

    if(strcmp(name, "EndPosition") == 0)
        return FTLEAttributes_GetEndPosition(self, NULL);
    if(strcmp(name, "integrationDirection") == 0)
        return FTLEAttributes_GetIntegrationDirection(self, NULL);
    if(strcmp(name, "Forward") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Forward));
    if(strcmp(name, "Backward") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Backward));
    if(strcmp(name, "Both") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Both));

    if(strcmp(name, "maxSteps") == 0)
        return FTLEAttributes_GetMaxSteps(self, NULL);
    if(strcmp(name, "terminationType") == 0)
        return FTLEAttributes_GetTerminationType(self, NULL);
    if(strcmp(name, "Time") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Time));
    if(strcmp(name, "Distance") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Distance));
    if(strcmp(name, "Size") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Size));

    if(strcmp(name, "terminateBySize") == 0)
        return FTLEAttributes_GetTerminateBySize(self, NULL);
    if(strcmp(name, "termSize") == 0)
        return FTLEAttributes_GetTermSize(self, NULL);
    if(strcmp(name, "terminateByDistance") == 0)
        return FTLEAttributes_GetTerminateByDistance(self, NULL);
    if(strcmp(name, "termDistance") == 0)
        return FTLEAttributes_GetTermDistance(self, NULL);
    if(strcmp(name, "terminateByTime") == 0)
        return FTLEAttributes_GetTerminateByTime(self, NULL);
    if(strcmp(name, "termTime") == 0)
        return FTLEAttributes_GetTermTime(self, NULL);
    if(strcmp(name, "maxStepLength") == 0)
        return FTLEAttributes_GetMaxStepLength(self, NULL);
    if(strcmp(name, "limitMaximumTimestep") == 0)
        return FTLEAttributes_GetLimitMaximumTimestep(self, NULL);
    if(strcmp(name, "maxTimeStep") == 0)
        return FTLEAttributes_GetMaxTimeStep(self, NULL);
    if(strcmp(name, "relTol") == 0)
        return FTLEAttributes_GetRelTol(self, NULL);
    if(strcmp(name, "absTolSizeType") == 0)
        return FTLEAttributes_GetAbsTolSizeType(self, NULL);
    if(strcmp(name, "Absolute") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Absolute));
    if(strcmp(name, "FractionOfBBox") == 0)
        return PyInt_FromLong(long(FTLEAttributes::FractionOfBBox));

    if(strcmp(name, "absTolAbsolute") == 0)
        return FTLEAttributes_GetAbsTolAbsolute(self, NULL);
    if(strcmp(name, "absTolBBox") == 0)
        return FTLEAttributes_GetAbsTolBBox(self, NULL);
    if(strcmp(name, "fieldType") == 0)
        return FTLEAttributes_GetFieldType(self, NULL);
    if(strcmp(name, "Default") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Default));
    if(strcmp(name, "FlashField") == 0)
        return PyInt_FromLong(long(FTLEAttributes::FlashField));
    if(strcmp(name, "M3DC12DField") == 0)
        return PyInt_FromLong(long(FTLEAttributes::M3DC12DField));
    if(strcmp(name, "M3DC13DField") == 0)
        return PyInt_FromLong(long(FTLEAttributes::M3DC13DField));
    if(strcmp(name, "Nek5000Field") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Nek5000Field));
    if(strcmp(name, "NIMRODField") == 0)
        return PyInt_FromLong(long(FTLEAttributes::NIMRODField));

    if(strcmp(name, "fieldConstant") == 0)
        return FTLEAttributes_GetFieldConstant(self, NULL);
    if(strcmp(name, "velocitySource") == 0)
        return FTLEAttributes_GetVelocitySource(self, NULL);
    if(strcmp(name, "integrationType") == 0)
        return FTLEAttributes_GetIntegrationType(self, NULL);
    if(strcmp(name, "Euler") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Euler));
    if(strcmp(name, "Leapfrog") == 0)
        return PyInt_FromLong(long(FTLEAttributes::Leapfrog));
    if(strcmp(name, "DormandPrince") == 0)
        return PyInt_FromLong(long(FTLEAttributes::DormandPrince));
    if(strcmp(name, "AdamsBashforth") == 0)
        return PyInt_FromLong(long(FTLEAttributes::AdamsBashforth));
    if(strcmp(name, "RK4") == 0)
        return PyInt_FromLong(long(FTLEAttributes::RK4));
    if(strcmp(name, "M3DC12DIntegrator") == 0)
        return PyInt_FromLong(long(FTLEAttributes::M3DC12DIntegrator));

    if(strcmp(name, "parallelizationAlgorithmType") == 0)
        return FTLEAttributes_GetParallelizationAlgorithmType(self, NULL);
    if(strcmp(name, "LoadOnDemand") == 0)
        return PyInt_FromLong(long(FTLEAttributes::LoadOnDemand));
    if(strcmp(name, "ParallelStaticDomains") == 0)
        return PyInt_FromLong(long(FTLEAttributes::ParallelStaticDomains));
    if(strcmp(name, "MasterSlave") == 0)
        return PyInt_FromLong(long(FTLEAttributes::MasterSlave));
    if(strcmp(name, "VisItSelects") == 0)
        return PyInt_FromLong(long(FTLEAttributes::VisItSelects));

    if(strcmp(name, "maxProcessCount") == 0)
        return FTLEAttributes_GetMaxProcessCount(self, NULL);
    if(strcmp(name, "maxDomainCacheSize") == 0)
        return FTLEAttributes_GetMaxDomainCacheSize(self, NULL);
    if(strcmp(name, "workGroupSize") == 0)
        return FTLEAttributes_GetWorkGroupSize(self, NULL);
    if(strcmp(name, "pathlines") == 0)
        return FTLEAttributes_GetPathlines(self, NULL);
    if(strcmp(name, "pathlinesOverrideStartingTimeFlag") == 0)
        return FTLEAttributes_GetPathlinesOverrideStartingTimeFlag(self, NULL);
    if(strcmp(name, "pathlinesOverrideStartingTime") == 0)
        return FTLEAttributes_GetPathlinesOverrideStartingTime(self, NULL);
    if(strcmp(name, "pathlinesCMFE") == 0)
        return FTLEAttributes_GetPathlinesCMFE(self, NULL);
    if(strcmp(name, "CONN_CMFE") == 0)
        return PyInt_FromLong(long(FTLEAttributes::CONN_CMFE));
    if(strcmp(name, "POS_CMFE") == 0)
        return PyInt_FromLong(long(FTLEAttributes::POS_CMFE));

    if(strcmp(name, "forceNodeCenteredData") == 0)
        return FTLEAttributes_GetForceNodeCenteredData(self, NULL);
    if(strcmp(name, "issueTerminationWarnings") == 0)
        return FTLEAttributes_GetIssueTerminationWarnings(self, NULL);
    if(strcmp(name, "issueStiffnessWarnings") == 0)
        return FTLEAttributes_GetIssueStiffnessWarnings(self, NULL);
    if(strcmp(name, "issueCriticalPointsWarnings") == 0)
        return FTLEAttributes_GetIssueCriticalPointsWarnings(self, NULL);
    if(strcmp(name, "criticalPointThreshold") == 0)
        return FTLEAttributes_GetCriticalPointThreshold(self, NULL);

    return Py_FindMethod(PyFTLEAttributes_methods, self, name);
}

int
PyFTLEAttributes_setattr(PyObject *self, char *name, PyObject *args)
{
    // Create a tuple to contain the arguments since all of the Set
    // functions expect a tuple.
    PyObject *tuple = PyTuple_New(1);
    PyTuple_SET_ITEM(tuple, 0, args);
    Py_INCREF(args);
    PyObject *obj = NULL;

    if(strcmp(name, "sourceType") == 0)
        obj = FTLEAttributes_SetSourceType(self, tuple);
    else if(strcmp(name, "Resolution") == 0)
        obj = FTLEAttributes_SetResolution(self, tuple);
    else if(strcmp(name, "UseDataSetStart") == 0)
        obj = FTLEAttributes_SetUseDataSetStart(self, tuple);
    else if(strcmp(name, "StartPosition") == 0)
        obj = FTLEAttributes_SetStartPosition(self, tuple);
    else if(strcmp(name, "UseDataSetEnd") == 0)
        obj = FTLEAttributes_SetUseDataSetEnd(self, tuple);
    else if(strcmp(name, "EndPosition") == 0)
        obj = FTLEAttributes_SetEndPosition(self, tuple);
    else if(strcmp(name, "integrationDirection") == 0)
        obj = FTLEAttributes_SetIntegrationDirection(self, tuple);
    else if(strcmp(name, "maxSteps") == 0)
        obj = FTLEAttributes_SetMaxSteps(self, tuple);
    else if(strcmp(name, "terminationType") == 0)
        obj = FTLEAttributes_SetTerminationType(self, tuple);
    else if(strcmp(name, "terminateBySize") == 0)
        obj = FTLEAttributes_SetTerminateBySize(self, tuple);
    else if(strcmp(name, "termSize") == 0)
        obj = FTLEAttributes_SetTermSize(self, tuple);
    else if(strcmp(name, "terminateByDistance") == 0)
        obj = FTLEAttributes_SetTerminateByDistance(self, tuple);
    else if(strcmp(name, "termDistance") == 0)
        obj = FTLEAttributes_SetTermDistance(self, tuple);
    else if(strcmp(name, "terminateByTime") == 0)
        obj = FTLEAttributes_SetTerminateByTime(self, tuple);
    else if(strcmp(name, "termTime") == 0)
        obj = FTLEAttributes_SetTermTime(self, tuple);
    else if(strcmp(name, "maxStepLength") == 0)
        obj = FTLEAttributes_SetMaxStepLength(self, tuple);
    else if(strcmp(name, "limitMaximumTimestep") == 0)
        obj = FTLEAttributes_SetLimitMaximumTimestep(self, tuple);
    else if(strcmp(name, "maxTimeStep") == 0)
        obj = FTLEAttributes_SetMaxTimeStep(self, tuple);
    else if(strcmp(name, "relTol") == 0)
        obj = FTLEAttributes_SetRelTol(self, tuple);
    else if(strcmp(name, "absTolSizeType") == 0)
        obj = FTLEAttributes_SetAbsTolSizeType(self, tuple);
    else if(strcmp(name, "absTolAbsolute") == 0)
        obj = FTLEAttributes_SetAbsTolAbsolute(self, tuple);
    else if(strcmp(name, "absTolBBox") == 0)
        obj = FTLEAttributes_SetAbsTolBBox(self, tuple);
    else if(strcmp(name, "fieldType") == 0)
        obj = FTLEAttributes_SetFieldType(self, tuple);
    else if(strcmp(name, "fieldConstant") == 0)
        obj = FTLEAttributes_SetFieldConstant(self, tuple);
    else if(strcmp(name, "velocitySource") == 0)
        obj = FTLEAttributes_SetVelocitySource(self, tuple);
    else if(strcmp(name, "integrationType") == 0)
        obj = FTLEAttributes_SetIntegrationType(self, tuple);
    else if(strcmp(name, "parallelizationAlgorithmType") == 0)
        obj = FTLEAttributes_SetParallelizationAlgorithmType(self, tuple);
    else if(strcmp(name, "maxProcessCount") == 0)
        obj = FTLEAttributes_SetMaxProcessCount(self, tuple);
    else if(strcmp(name, "maxDomainCacheSize") == 0)
        obj = FTLEAttributes_SetMaxDomainCacheSize(self, tuple);
    else if(strcmp(name, "workGroupSize") == 0)
        obj = FTLEAttributes_SetWorkGroupSize(self, tuple);
    else if(strcmp(name, "pathlines") == 0)
        obj = FTLEAttributes_SetPathlines(self, tuple);
    else if(strcmp(name, "pathlinesOverrideStartingTimeFlag") == 0)
        obj = FTLEAttributes_SetPathlinesOverrideStartingTimeFlag(self, tuple);
    else if(strcmp(name, "pathlinesOverrideStartingTime") == 0)
        obj = FTLEAttributes_SetPathlinesOverrideStartingTime(self, tuple);
    else if(strcmp(name, "pathlinesCMFE") == 0)
        obj = FTLEAttributes_SetPathlinesCMFE(self, tuple);
    else if(strcmp(name, "forceNodeCenteredData") == 0)
        obj = FTLEAttributes_SetForceNodeCenteredData(self, tuple);
    else if(strcmp(name, "issueTerminationWarnings") == 0)
        obj = FTLEAttributes_SetIssueTerminationWarnings(self, tuple);
    else if(strcmp(name, "issueStiffnessWarnings") == 0)
        obj = FTLEAttributes_SetIssueStiffnessWarnings(self, tuple);
    else if(strcmp(name, "issueCriticalPointsWarnings") == 0)
        obj = FTLEAttributes_SetIssueCriticalPointsWarnings(self, tuple);
    else if(strcmp(name, "criticalPointThreshold") == 0)
        obj = FTLEAttributes_SetCriticalPointThreshold(self, tuple);

    if(obj != NULL)
        Py_DECREF(obj);

    Py_DECREF(tuple);
    if( obj == NULL)
        PyErr_Format(PyExc_RuntimeError, "Unable to set unknown attribute: '%s'", name);
    return (obj != NULL) ? 0 : -1;
}

static int
FTLEAttributes_print(PyObject *v, FILE *fp, int flags)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)v;
    fprintf(fp, "%s", PyFTLEAttributes_ToString(obj->data, "").c_str());
    return 0;
}

PyObject *
FTLEAttributes_str(PyObject *v)
{
    FTLEAttributesObject *obj = (FTLEAttributesObject *)v;
    return PyString_FromString(PyFTLEAttributes_ToString(obj->data,"").c_str());
}

//
// The doc string for the class.
//
#if PY_MAJOR_VERSION > 2 || (PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION >= 5)
static const char *FTLEAttributes_Purpose = "Attributes for FTLE";
#else
static char *FTLEAttributes_Purpose = "Attributes for FTLE";
#endif

//
// The type description structure
//
static PyTypeObject FTLEAttributesType =
{
    //
    // Type header
    //
    PyObject_HEAD_INIT(&PyType_Type)
    0,                                   // ob_size
    "FTLEAttributes",                    // tp_name
    sizeof(FTLEAttributesObject),        // tp_basicsize
    0,                                   // tp_itemsize
    //
    // Standard methods
    //
    (destructor)FTLEAttributes_dealloc,  // tp_dealloc
    (printfunc)FTLEAttributes_print,     // tp_print
    (getattrfunc)PyFTLEAttributes_getattr, // tp_getattr
    (setattrfunc)PyFTLEAttributes_setattr, // tp_setattr
    (cmpfunc)FTLEAttributes_compare,     // tp_compare
    (reprfunc)0,                         // tp_repr
    //
    // Type categories
    //
    0,                                   // tp_as_number
    0,                                   // tp_as_sequence
    0,                                   // tp_as_mapping
    //
    // More methods
    //
    0,                                   // tp_hash
    0,                                   // tp_call
    (reprfunc)FTLEAttributes_str,        // tp_str
    0,                                   // tp_getattro
    0,                                   // tp_setattro
    0,                                   // tp_as_buffer
    Py_TPFLAGS_CHECKTYPES,               // tp_flags
    FTLEAttributes_Purpose,              // tp_doc
    0,                                   // tp_traverse
    0,                                   // tp_clear
    0,                                   // tp_richcompare
    0                                    // tp_weaklistoffset
};

//
// Helper functions for object allocation.
//

static FTLEAttributes *defaultAtts = 0;
static FTLEAttributes *currentAtts = 0;

static PyObject *
NewFTLEAttributes(int useCurrent)
{
    FTLEAttributesObject *newObject;
    newObject = PyObject_NEW(FTLEAttributesObject, &FTLEAttributesType);
    if(newObject == NULL)
        return NULL;
    if(useCurrent && currentAtts != 0)
        newObject->data = new FTLEAttributes(*currentAtts);
    else if(defaultAtts != 0)
        newObject->data = new FTLEAttributes(*defaultAtts);
    else
        newObject->data = new FTLEAttributes;
    newObject->owns = true;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

static PyObject *
WrapFTLEAttributes(const FTLEAttributes *attr)
{
    FTLEAttributesObject *newObject;
    newObject = PyObject_NEW(FTLEAttributesObject, &FTLEAttributesType);
    if(newObject == NULL)
        return NULL;
    newObject->data = (FTLEAttributes *)attr;
    newObject->owns = false;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

///////////////////////////////////////////////////////////////////////////////
//
// Interface that is exposed to the VisIt module.
//
///////////////////////////////////////////////////////////////////////////////

PyObject *
FTLEAttributes_new(PyObject *self, PyObject *args)
{
    int useCurrent = 0;
    if (!PyArg_ParseTuple(args, "i", &useCurrent))
    {
        if (!PyArg_ParseTuple(args, ""))
            return NULL;
        else
            PyErr_Clear();
    }

    return (PyObject *)NewFTLEAttributes(useCurrent);
}

//
// Plugin method table. These methods are added to the visitmodule's methods.
//
static PyMethodDef FTLEAttributesMethods[] = {
    {"FTLEAttributes", FTLEAttributes_new, METH_VARARGS},
    {NULL,      NULL}        /* Sentinel */
};

static Observer *FTLEAttributesObserver = 0;

std::string
PyFTLEAttributes_GetLogString()
{
    std::string s("FTLEAtts = FTLEAttributes()\n");
    if(currentAtts != 0)
        s += PyFTLEAttributes_ToString(currentAtts, "FTLEAtts.");
    return s;
}

static void
PyFTLEAttributes_CallLogRoutine(Subject *subj, void *data)
{
    typedef void (*logCallback)(const std::string &);
    logCallback cb = (logCallback)data;

    if(cb != 0)
    {
        std::string s("FTLEAtts = FTLEAttributes()\n");
        s += PyFTLEAttributes_ToString(currentAtts, "FTLEAtts.");
        cb(s);
    }
}

void
PyFTLEAttributes_StartUp(FTLEAttributes *subj, void *data)
{
    if(subj == 0)
        return;

    currentAtts = subj;
    PyFTLEAttributes_SetDefaults(subj);

    //
    // Create the observer that will be notified when the attributes change.
    //
    if(FTLEAttributesObserver == 0)
    {
        FTLEAttributesObserver = new ObserverToCallback(subj,
            PyFTLEAttributes_CallLogRoutine, (void *)data);
    }

}

void
PyFTLEAttributes_CloseDown()
{
    delete defaultAtts;
    defaultAtts = 0;
    delete FTLEAttributesObserver;
    FTLEAttributesObserver = 0;
}

PyMethodDef *
PyFTLEAttributes_GetMethodTable(int *nMethods)
{
    *nMethods = 1;
    return FTLEAttributesMethods;
}

bool
PyFTLEAttributes_Check(PyObject *obj)
{
    return (obj->ob_type == &FTLEAttributesType);
}

FTLEAttributes *
PyFTLEAttributes_FromPyObject(PyObject *obj)
{
    FTLEAttributesObject *obj2 = (FTLEAttributesObject *)obj;
    return obj2->data;
}

PyObject *
PyFTLEAttributes_New()
{
    return NewFTLEAttributes(0);
}

PyObject *
PyFTLEAttributes_Wrap(const FTLEAttributes *attr)
{
    return WrapFTLEAttributes(attr);
}

void
PyFTLEAttributes_SetParent(PyObject *obj, PyObject *parent)
{
    FTLEAttributesObject *obj2 = (FTLEAttributesObject *)obj;
    obj2->parent = parent;
}

void
PyFTLEAttributes_SetDefaults(const FTLEAttributes *atts)
{
    if(defaultAtts)
        delete defaultAtts;

    defaultAtts = new FTLEAttributes(*atts);
}

