/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

// ************************************************************************* //
//  File: IntegralCurveViewerPluginInfo.C
// ************************************************************************* //

#include <IntegralCurvePluginInfo.h>
#include <QApplication>
#include <IntegralCurveAttributes.h>

// ****************************************************************************
//  Function:  GetViewerInfo
//
//  Purpose:
//    Return a new ViewerPluginInfo for the IntegralCurve operator.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************
extern "C" OP_EXPORT ViewerOperatorPluginInfo* IntegralCurve_GetViewerInfo()
{
    IntegralCurveViewerPluginInfo::InitializeGlobalObjects();
    return new IntegralCurveViewerPluginInfo;
}

//
// Storage for static data elements.
//
IntegralCurveAttributes *IntegralCurveViewerPluginInfo::clientAtts = NULL;
IntegralCurveAttributes *IntegralCurveViewerPluginInfo::defaultAtts = NULL;

// ****************************************************************************
//  Method:  IntegralCurveViewerPluginInfo::InitializeGlobalObjects
//
//  Purpose:
//    Initialize the operator atts.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************
void
IntegralCurveViewerPluginInfo::InitializeGlobalObjects()
{
    IntegralCurveViewerPluginInfo::clientAtts  = new IntegralCurveAttributes;
    IntegralCurveViewerPluginInfo::defaultAtts = new IntegralCurveAttributes;
}

// ****************************************************************************
//  Method: IntegralCurveViewerPluginInfo::GetClientAtts
//
//  Purpose:
//    Return a pointer to the viewer client attributes.
//
//  Returns:    A pointer to the viewer client attributes.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

AttributeSubject *
IntegralCurveViewerPluginInfo::GetClientAtts()
{
    return clientAtts;
}

// ****************************************************************************
//  Method: IntegralCurveViewerPluginInfo::GetDefaultAtts
//
//  Purpose:
//    Return a pointer to the viewer default attributes.
//
//  Returns:    A pointer to the viewer default attributes.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

AttributeSubject *
IntegralCurveViewerPluginInfo::GetDefaultAtts()
{
    return defaultAtts;
}

// ****************************************************************************
//  Method: IntegralCurveViewerPluginInfo::SetClientAtts
//
//  Purpose:
//    Set the viewer client attributes.
//
//  Arguments:
//    atts      A pointer to the new client attributes.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

void
IntegralCurveViewerPluginInfo::SetClientAtts(AttributeSubject *atts)
{
    *clientAtts = *(IntegralCurveAttributes *)atts;
    clientAtts->Notify();
}

// ****************************************************************************
//  Method: IntegralCurveViewerPluginInfo::GetClientAtts
//
//  Purpose:
//    Get the viewer client attributes.
//
//  Arguments:
//    atts      A pointer to return the client default attributes in.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

void
IntegralCurveViewerPluginInfo::GetClientAtts(AttributeSubject *atts)
{
    *(IntegralCurveAttributes *)atts = *clientAtts;
}

// ****************************************************************************
//  Method: IntegralCurveViewerPluginInfo::InitializeOperatorAtts
//
//  Purpose:
//    Initialize the operator attributes to the default attributes.
//
//  Arguments:
//    atts        The attribute subject to initialize.
//    plot        The viewer plot that owns the operator.
//    fromDefault True to initialize the attributes from the defaults. False
//                to initialize from the current attributes.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

void
IntegralCurveViewerPluginInfo::InitializeOperatorAtts(AttributeSubject *atts,
                                              const ViewerPlot *plot,
                                              const bool fromDefault)
{
    if (fromDefault)
        *(IntegralCurveAttributes*)atts = *defaultAtts;
    else
        *(IntegralCurveAttributes*)atts = *clientAtts;

    UpdateOperatorAtts(atts, plot);
}

// ****************************************************************************
//  Method: IntegralCurveViewerPluginInfo::UpdateOperatorAtts
//
//  Purpose:
//    Update the operator attributes when using operator expressions.
//
//  Arguments:
//    atts        The attribute subject to update.
//    plot        The viewer plot that owns the operator.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

void
IntegralCurveViewerPluginInfo::UpdateOperatorAtts(AttributeSubject *atts,
                                              const ViewerPlot *plot)
{
}

// ****************************************************************************
//  Method: IntegralCurveViewerPluginInfo::GetOperatorVarDescription
//
//  Purpose:
//    Return the operator variable description.
//
//  Arguments:
//    atts        The attribute subject to initialize.
//    plot        The viewer plot that owns the operator.
//
//  Programmer: Allen Sanderson
//  Creation:   19 August 2013
//
// ****************************************************************************

#include <ViewerPlot.h>

std::string
IntegralCurveViewerPluginInfo::GetOperatorVarDescription(AttributeSubject *atts,
                                                    const ViewerPlot *plot)
{
    IntegralCurveAttributes *atts_in = (IntegralCurveAttributes *)atts;

    const char *typeString[11] =
      { "None", "Seed Point ID", "Speed", "Vorticity", "Arc Length",
         "Absolute Time", "Relative Time",
         "Ave. dist. from seed", "Correlation Distance", "Difference",
         "Variable" };

    std::string var = plot->GetVariableName();

    if( strncmp(var.c_str(), "operators/IntegralCurve/",
                strlen("operators/IntegralCurve/")) == 0)
    {
      std::string justTheVar = var.substr(strlen("operators/IntegralCurve/"));

      var = justTheVar;
    }
    
    var += std::string(" - ") +
        std::string(typeString[atts_in->GetDataValue()]);

    if( atts_in->GetDataValue() == IntegralCurveAttributes::Variable ) 
    {
        var += std::string(": ") + atts_in->GetDataVariable();
    }

    return var;
}

// ****************************************************************************
//  Method: IntegralCurveViewerPluginInfo::GetMenuName
//
//  Purpose:
//    Return a pointer to the name to use in the viewer menus.
//
//  Returns:    A pointer to the name to use in the viewer menus.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

QString *
IntegralCurveViewerPluginInfo::GetMenuName() const
{
    return new QString(qApp->translate("OperatorNames", "IntegralCurve"));
}

// ****************************************************************************
//  Method: IntegralCurveViewerPluginInfo::XPMIconData
//
//  Purpose:
//    Return a pointer to the icon data.
//
//  Returns:    A pointer to the icon data.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

#include <IntegralCurve.xpm>
const char **
IntegralCurveViewerPluginInfo::XPMIconData() const
{
    return IntegralCurve_xpm;
}

