/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include <ModelBasedClusteringAttributes.h>
#include <DataNode.h>

//
// Enum conversion methods for ModelBasedClusteringAttributes::clusterAlgorithmType
//

static const char *clusterAlgorithmType_strings[] = {
"EM", "ACEM", "APECM", 
"APECMA", "K_MEANS"};

std::string
ModelBasedClusteringAttributes::clusterAlgorithmType_ToString(ModelBasedClusteringAttributes::clusterAlgorithmType t)
{
    int index = int(t);
    if(index < 0 || index >= 5) index = 0;
    return clusterAlgorithmType_strings[index];
}

std::string
ModelBasedClusteringAttributes::clusterAlgorithmType_ToString(int t)
{
    int index = (t < 0 || t >= 5) ? 0 : t;
    return clusterAlgorithmType_strings[index];
}

bool
ModelBasedClusteringAttributes::clusterAlgorithmType_FromString(const std::string &s, ModelBasedClusteringAttributes::clusterAlgorithmType &val)
{
    val = ModelBasedClusteringAttributes::EM;
    for(int i = 0; i < 5; ++i)
    {
        if(s == clusterAlgorithmType_strings[i])
        {
            val = (clusterAlgorithmType)i;
            return true;
        }
    }
    return false;
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::ModelBasedClusteringAttributes
//
// Purpose: 
//   Init utility for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void ModelBasedClusteringAttributes::Init()
{
    numClusters = 4;
    clusterMethod = EM;
    threshold = 1;
    percentile = 0.95;

    ModelBasedClusteringAttributes::SelectAll();
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::ModelBasedClusteringAttributes
//
// Purpose: 
//   Copy utility for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void ModelBasedClusteringAttributes::Copy(const ModelBasedClusteringAttributes &obj)
{
    numClusters = obj.numClusters;
    clusterMethod = obj.clusterMethod;
    threshold = obj.threshold;
    percentile = obj.percentile;
    var2 = obj.var2;
    var3 = obj.var3;

    ModelBasedClusteringAttributes::SelectAll();
}

// Type map format string
const char *ModelBasedClusteringAttributes::TypeMapFormatString = MODELBASEDCLUSTERINGATTRIBUTES_TMFS;
const AttributeGroup::private_tmfs_t ModelBasedClusteringAttributes::TmfsStruct = {MODELBASEDCLUSTERINGATTRIBUTES_TMFS};


// ****************************************************************************
// Method: ModelBasedClusteringAttributes::ModelBasedClusteringAttributes
//
// Purpose: 
//   Default constructor for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

ModelBasedClusteringAttributes::ModelBasedClusteringAttributes() : 
    AttributeSubject(ModelBasedClusteringAttributes::TypeMapFormatString),
    var2("default"), var3("default")
{
    ModelBasedClusteringAttributes::Init();
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::ModelBasedClusteringAttributes
//
// Purpose: 
//   Constructor for the derived classes of ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

ModelBasedClusteringAttributes::ModelBasedClusteringAttributes(private_tmfs_t tmfs) : 
    AttributeSubject(tmfs.tmfs),
    var2("default"), var3("default")
{
    ModelBasedClusteringAttributes::Init();
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::ModelBasedClusteringAttributes
//
// Purpose: 
//   Copy constructor for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

ModelBasedClusteringAttributes::ModelBasedClusteringAttributes(const ModelBasedClusteringAttributes &obj) : 
    AttributeSubject(ModelBasedClusteringAttributes::TypeMapFormatString)
{
    ModelBasedClusteringAttributes::Copy(obj);
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::ModelBasedClusteringAttributes
//
// Purpose: 
//   Copy constructor for derived classes of the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

ModelBasedClusteringAttributes::ModelBasedClusteringAttributes(const ModelBasedClusteringAttributes &obj, private_tmfs_t tmfs) : 
    AttributeSubject(tmfs.tmfs)
{
    ModelBasedClusteringAttributes::Copy(obj);
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::~ModelBasedClusteringAttributes
//
// Purpose: 
//   Destructor for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

ModelBasedClusteringAttributes::~ModelBasedClusteringAttributes()
{
    // nothing here
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::operator = 
//
// Purpose: 
//   Assignment operator for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

ModelBasedClusteringAttributes& 
ModelBasedClusteringAttributes::operator = (const ModelBasedClusteringAttributes &obj)
{
    if (this == &obj) return *this;

    ModelBasedClusteringAttributes::Copy(obj);

    return *this;
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::operator == 
//
// Purpose: 
//   Comparison operator == for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
ModelBasedClusteringAttributes::operator == (const ModelBasedClusteringAttributes &obj) const
{
    // Create the return value
    return ((numClusters == obj.numClusters) &&
            (clusterMethod == obj.clusterMethod) &&
            (threshold == obj.threshold) &&
            (percentile == obj.percentile) &&
            (var2 == obj.var2) &&
            (var3 == obj.var3));
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::operator != 
//
// Purpose: 
//   Comparison operator != for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
ModelBasedClusteringAttributes::operator != (const ModelBasedClusteringAttributes &obj) const
{
    return !(this->operator == (obj));
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::TypeName
//
// Purpose: 
//   Type name method for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

const std::string
ModelBasedClusteringAttributes::TypeName() const
{
    return "ModelBasedClusteringAttributes";
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::CopyAttributes
//
// Purpose: 
//   CopyAttributes method for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
ModelBasedClusteringAttributes::CopyAttributes(const AttributeGroup *atts)
{
    if(TypeName() != atts->TypeName())
        return false;

    // Call assignment operator.
    const ModelBasedClusteringAttributes *tmp = (const ModelBasedClusteringAttributes *)atts;
    *this = *tmp;

    return true;
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::CreateCompatible
//
// Purpose: 
//   CreateCompatible method for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

AttributeSubject *
ModelBasedClusteringAttributes::CreateCompatible(const std::string &tname) const
{
    AttributeSubject *retval = 0;
    if(TypeName() == tname)
        retval = new ModelBasedClusteringAttributes(*this);
    // Other cases could go here too. 

    return retval;
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::NewInstance
//
// Purpose: 
//   NewInstance method for the ModelBasedClusteringAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

AttributeSubject *
ModelBasedClusteringAttributes::NewInstance(bool copy) const
{
    AttributeSubject *retval = 0;
    if(copy)
        retval = new ModelBasedClusteringAttributes(*this);
    else
        retval = new ModelBasedClusteringAttributes;

    return retval;
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::SelectAll
//
// Purpose: 
//   Selects all attributes.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
ModelBasedClusteringAttributes::SelectAll()
{
    Select(ID_numClusters,   (void *)&numClusters);
    Select(ID_clusterMethod, (void *)&clusterMethod);
    Select(ID_threshold,     (void *)&threshold);
    Select(ID_percentile,    (void *)&percentile);
    Select(ID_var2,          (void *)&var2);
    Select(ID_var3,          (void *)&var3);
}

///////////////////////////////////////////////////////////////////////////////
// Persistence methods
///////////////////////////////////////////////////////////////////////////////

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::CreateNode
//
// Purpose: 
//   This method creates a DataNode representation of the object so it can be saved to a config file.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
ModelBasedClusteringAttributes::CreateNode(DataNode *parentNode, bool completeSave, bool forceAdd)
{
    if(parentNode == 0)
        return false;

    ModelBasedClusteringAttributes defaultObject;
    bool addToParent = false;
    // Create a node for ModelBasedClusteringAttributes.
    DataNode *node = new DataNode("ModelBasedClusteringAttributes");

    if(completeSave || !FieldsEqual(ID_numClusters, &defaultObject))
    {
        addToParent = true;
        node->AddNode(new DataNode("numClusters", numClusters));
    }

    if(completeSave || !FieldsEqual(ID_clusterMethod, &defaultObject))
    {
        addToParent = true;
        node->AddNode(new DataNode("clusterMethod", clusterAlgorithmType_ToString(clusterMethod)));
    }

    if(completeSave || !FieldsEqual(ID_threshold, &defaultObject))
    {
        addToParent = true;
        node->AddNode(new DataNode("threshold", threshold));
    }

    if(completeSave || !FieldsEqual(ID_percentile, &defaultObject))
    {
        addToParent = true;
        node->AddNode(new DataNode("percentile", percentile));
    }

    if(completeSave || !FieldsEqual(ID_var2, &defaultObject))
    {
        addToParent = true;
        node->AddNode(new DataNode("var2", var2));
    }

    if(completeSave || !FieldsEqual(ID_var3, &defaultObject))
    {
        addToParent = true;
        node->AddNode(new DataNode("var3", var3));
    }


    // Add the node to the parent node.
    if(addToParent || forceAdd)
        parentNode->AddNode(node);
    else
        delete node;

    return (addToParent || forceAdd);
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::SetFromNode
//
// Purpose: 
//   This method sets attributes in this object from values in a DataNode representation of the object.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
ModelBasedClusteringAttributes::SetFromNode(DataNode *parentNode)
{
    if(parentNode == 0)
        return;

    DataNode *searchNode = parentNode->GetNode("ModelBasedClusteringAttributes");
    if(searchNode == 0)
        return;

    DataNode *node;
    if((node = searchNode->GetNode("numClusters")) != 0)
        SetNumClusters(node->AsInt());
    if((node = searchNode->GetNode("clusterMethod")) != 0)
    {
        // Allow enums to be int or string in the config file
        if(node->GetNodeType() == INT_NODE)
        {
            int ival = node->AsInt();
            if(ival >= 0 && ival < 5)
                SetClusterMethod(clusterAlgorithmType(ival));
        }
        else if(node->GetNodeType() == STRING_NODE)
        {
            clusterAlgorithmType value;
            if(clusterAlgorithmType_FromString(node->AsString(), value))
                SetClusterMethod(value);
        }
    }
    if((node = searchNode->GetNode("threshold")) != 0)
        SetThreshold(node->AsDouble());
    if((node = searchNode->GetNode("percentile")) != 0)
        SetPercentile(node->AsDouble());
    if((node = searchNode->GetNode("var2")) != 0)
        SetVar2(node->AsString());
    if((node = searchNode->GetNode("var3")) != 0)
        SetVar3(node->AsString());
}

///////////////////////////////////////////////////////////////////////////////
// Set property methods
///////////////////////////////////////////////////////////////////////////////

void
ModelBasedClusteringAttributes::SetNumClusters(int numClusters_)
{
    numClusters = numClusters_;
    Select(ID_numClusters, (void *)&numClusters);
}

void
ModelBasedClusteringAttributes::SetClusterMethod(ModelBasedClusteringAttributes::clusterAlgorithmType clusterMethod_)
{
    clusterMethod = clusterMethod_;
    Select(ID_clusterMethod, (void *)&clusterMethod);
}

void
ModelBasedClusteringAttributes::SetThreshold(double threshold_)
{
    threshold = threshold_;
    Select(ID_threshold, (void *)&threshold);
}

void
ModelBasedClusteringAttributes::SetPercentile(double percentile_)
{
    percentile = percentile_;
    Select(ID_percentile, (void *)&percentile);
}

void
ModelBasedClusteringAttributes::SetVar2(const std::string &var2_)
{
    var2 = var2_;
    Select(ID_var2, (void *)&var2);
}

void
ModelBasedClusteringAttributes::SetVar3(const std::string &var3_)
{
    var3 = var3_;
    Select(ID_var3, (void *)&var3);
}

///////////////////////////////////////////////////////////////////////////////
// Get property methods
///////////////////////////////////////////////////////////////////////////////

int
ModelBasedClusteringAttributes::GetNumClusters() const
{
    return numClusters;
}

ModelBasedClusteringAttributes::clusterAlgorithmType
ModelBasedClusteringAttributes::GetClusterMethod() const
{
    return clusterAlgorithmType(clusterMethod);
}

double
ModelBasedClusteringAttributes::GetThreshold() const
{
    return threshold;
}

double
ModelBasedClusteringAttributes::GetPercentile() const
{
    return percentile;
}

const std::string &
ModelBasedClusteringAttributes::GetVar2() const
{
    return var2;
}

std::string &
ModelBasedClusteringAttributes::GetVar2()
{
    return var2;
}

const std::string &
ModelBasedClusteringAttributes::GetVar3() const
{
    return var3;
}

std::string &
ModelBasedClusteringAttributes::GetVar3()
{
    return var3;
}

///////////////////////////////////////////////////////////////////////////////
// Select property methods
///////////////////////////////////////////////////////////////////////////////

void
ModelBasedClusteringAttributes::SelectVar2()
{
    Select(ID_var2, (void *)&var2);
}

void
ModelBasedClusteringAttributes::SelectVar3()
{
    Select(ID_var3, (void *)&var3);
}

///////////////////////////////////////////////////////////////////////////////
// Keyframing methods
///////////////////////////////////////////////////////////////////////////////

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::GetFieldName
//
// Purpose: 
//   This method returns the name of a field given its index.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

std::string
ModelBasedClusteringAttributes::GetFieldName(int index) const
{
    switch (index)
    {
    case ID_numClusters:   return "numClusters";
    case ID_clusterMethod: return "clusterMethod";
    case ID_threshold:     return "threshold";
    case ID_percentile:    return "percentile";
    case ID_var2:          return "var2";
    case ID_var3:          return "var3";
    default:  return "invalid index";
    }
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::GetFieldType
//
// Purpose: 
//   This method returns the type of a field given its index.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

AttributeGroup::FieldType
ModelBasedClusteringAttributes::GetFieldType(int index) const
{
    switch (index)
    {
    case ID_numClusters:   return FieldType_int;
    case ID_clusterMethod: return FieldType_enum;
    case ID_threshold:     return FieldType_double;
    case ID_percentile:    return FieldType_double;
    case ID_var2:          return FieldType_variablename;
    case ID_var3:          return FieldType_variablename;
    default:  return FieldType_unknown;
    }
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::GetFieldTypeName
//
// Purpose: 
//   This method returns the name of a field type given its index.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

std::string
ModelBasedClusteringAttributes::GetFieldTypeName(int index) const
{
    switch (index)
    {
    case ID_numClusters:   return "int";
    case ID_clusterMethod: return "enum";
    case ID_threshold:     return "double";
    case ID_percentile:    return "double";
    case ID_var2:          return "variablename";
    case ID_var3:          return "variablename";
    default:  return "invalid index";
    }
}

// ****************************************************************************
// Method: ModelBasedClusteringAttributes::FieldsEqual
//
// Purpose: 
//   This method compares two fields and return true if they are equal.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
ModelBasedClusteringAttributes::FieldsEqual(int index_, const AttributeGroup *rhs) const
{
    const ModelBasedClusteringAttributes &obj = *((const ModelBasedClusteringAttributes*)rhs);
    bool retval = false;
    switch (index_)
    {
    case ID_numClusters:
        {  // new scope
        retval = (numClusters == obj.numClusters);
        }
        break;
    case ID_clusterMethod:
        {  // new scope
        retval = (clusterMethod == obj.clusterMethod);
        }
        break;
    case ID_threshold:
        {  // new scope
        retval = (threshold == obj.threshold);
        }
        break;
    case ID_percentile:
        {  // new scope
        retval = (percentile == obj.percentile);
        }
        break;
    case ID_var2:
        {  // new scope
        retval = (var2 == obj.var2);
        }
        break;
    case ID_var3:
        {  // new scope
        retval = (var3 == obj.var3);
        }
        break;
    default: retval = false;
    }

    return retval;
}

///////////////////////////////////////////////////////////////////////////////
// User-defined methods.
///////////////////////////////////////////////////////////////////////////////

