/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include "QvisModelBasedClusteringWindow.h"

#include <ModelBasedClusteringAttributes.h>
#include <ViewerProxy.h>

#include <QCheckBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QSpinBox>
#include <QButtonGroup>
#include <QRadioButton>
#include <QvisColorTableButton.h>
#include <QvisOpacitySlider.h>
#include <QvisColorButton.h>
#include <QvisLineStyleWidget.h>
#include <QvisLineWidthWidget.h>
#include <QvisVariableButton.h>

#include <stdio.h>
#include <string>

using std::string;

// ****************************************************************************
// Method: QvisModelBasedClusteringWindow::QvisModelBasedClusteringWindow
//
// Purpose: 
//   Constructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisModelBasedClusteringWindow::QvisModelBasedClusteringWindow(const int type,
                         ModelBasedClusteringAttributes *subj,
                         const QString &caption,
                         const QString &shortName,
                         QvisNotepadArea *notepad)
    : QvisOperatorWindow(type,subj, caption, shortName, notepad)
{
    atts = subj;
}


// ****************************************************************************
// Method: QvisModelBasedClusteringWindow::~QvisModelBasedClusteringWindow
//
// Purpose: 
//   Destructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisModelBasedClusteringWindow::~QvisModelBasedClusteringWindow()
{
}


// ****************************************************************************
// Method: QvisModelBasedClusteringWindow::CreateWindowContents
//
// Purpose: 
//   Creates the widgets for the window.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisModelBasedClusteringWindow::CreateWindowContents()
{
    QGridLayout *mainLayout = new QGridLayout(0);
    topLayout->addLayout(mainLayout);

    numClustersLabel = new QLabel(tr("Number of clusters"), central);
    mainLayout->addWidget(numClustersLabel,0,0);
    numClusters = new QLineEdit(central);
    connect(numClusters, SIGNAL(returnPressed()),
            this, SLOT(numClustersProcessText()));
    mainLayout->addWidget(numClusters, 0,1);

    clusterMethodLabel = new QLabel(tr("Clustering Method"), central);
    mainLayout->addWidget(clusterMethodLabel,1,0);
    clusterMethod = new QWidget(central);
    clusterMethodButtonGroup= new QButtonGroup(clusterMethod);
    QHBoxLayout *clusterMethodLayout = new QHBoxLayout(clusterMethod);
    clusterMethodLayout->setMargin(0);
    clusterMethodLayout->setSpacing(10);
    QRadioButton *clusterMethodclusterAlgorithmTypeEM = new QRadioButton(tr("EM"), clusterMethod);
    clusterMethodButtonGroup->addButton(clusterMethodclusterAlgorithmTypeEM,0);
    clusterMethodLayout->addWidget(clusterMethodclusterAlgorithmTypeEM);
    QRadioButton *clusterMethodclusterAlgorithmTypeACEM = new QRadioButton(tr("ACEM"), clusterMethod);
    clusterMethodButtonGroup->addButton(clusterMethodclusterAlgorithmTypeACEM,1);
    clusterMethodLayout->addWidget(clusterMethodclusterAlgorithmTypeACEM);
    QRadioButton *clusterMethodclusterAlgorithmTypeAPECM = new QRadioButton(tr("APECM"), clusterMethod);
    clusterMethodButtonGroup->addButton(clusterMethodclusterAlgorithmTypeAPECM,2);
    clusterMethodLayout->addWidget(clusterMethodclusterAlgorithmTypeAPECM);
    QRadioButton *clusterMethodclusterAlgorithmTypeAPECMA = new QRadioButton(tr("APECMA"), clusterMethod);
    clusterMethodButtonGroup->addButton(clusterMethodclusterAlgorithmTypeAPECMA,3);
    clusterMethodLayout->addWidget(clusterMethodclusterAlgorithmTypeAPECMA);
    QRadioButton *clusterMethodclusterAlgorithmTypeK_MEANS = new QRadioButton(tr("K_MEANS"), clusterMethod);
    clusterMethodButtonGroup->addButton(clusterMethodclusterAlgorithmTypeK_MEANS,4);
    clusterMethodLayout->addWidget(clusterMethodclusterAlgorithmTypeK_MEANS);
    connect(clusterMethodButtonGroup, SIGNAL(buttonClicked(int)),
            this, SLOT(clusterMethodChanged(int)));
    mainLayout->addWidget(clusterMethod, 1,1);

    thresholdLabel = new QLabel(tr("Threshold"), central);
    mainLayout->addWidget(thresholdLabel,2,0);
    threshold = new QLineEdit(central);
    connect(threshold, SIGNAL(returnPressed()),
            this, SLOT(thresholdProcessText()));
    mainLayout->addWidget(threshold, 2,1);

    percentileLabel = new QLabel(tr("Percentile"), central);
    mainLayout->addWidget(percentileLabel,3,0);
    percentile = new QLineEdit(central);
    connect(percentile, SIGNAL(returnPressed()),
            this, SLOT(percentileProcessText()));
    mainLayout->addWidget(percentile, 3,1);

    var2Label = new QLabel(tr("Variable 2"), central);
    mainLayout->addWidget(var2Label,4,0);
    int var2Mask = QvisVariableButton::Scalars;
    var2 = new QvisVariableButton(true, true, true, var2Mask, central);
    connect(var2, SIGNAL(activated(const QString&)),
            this, SLOT(var2Changed(const QString&)));
    mainLayout->addWidget(var2, 4,1);

    var3Label = new QLabel(tr("Variable 3"), central);
    mainLayout->addWidget(var3Label,5,0);
    int var3Mask = QvisVariableButton::Scalars;
    var3 = new QvisVariableButton(true, true, true, var3Mask, central);
    connect(var3, SIGNAL(activated(const QString&)),
            this, SLOT(var3Changed(const QString&)));
    mainLayout->addWidget(var3, 5,1);

}


// ****************************************************************************
// Method: QvisModelBasedClusteringWindow::UpdateWindow
//
// Purpose: 
//   Updates the widgets in the window when the subject changes.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisModelBasedClusteringWindow::UpdateWindow(bool doAll)
{

    for(int i = 0; i < atts->NumAttributes(); ++i)
    {
        if(!doAll)
        {
            if(!atts->IsSelected(i))
            {
                continue;
            }
        }

        switch(i)
        {
          case ModelBasedClusteringAttributes::ID_numClusters:
            numClusters->setText(IntToQString(atts->GetNumClusters()));
            break;
          case ModelBasedClusteringAttributes::ID_clusterMethod:
            clusterMethodButtonGroup->blockSignals(true);
            if(clusterMethodButtonGroup->button((int)atts->GetClusterMethod()) != 0)
                clusterMethodButtonGroup->button((int)atts->GetClusterMethod())->setChecked(true);
            clusterMethodButtonGroup->blockSignals(false);
            break;
          case ModelBasedClusteringAttributes::ID_threshold:
            threshold->setText(DoubleToQString(atts->GetThreshold()));
            break;
          case ModelBasedClusteringAttributes::ID_percentile:
            percentile->setText(DoubleToQString(atts->GetPercentile()));
            break;
          case ModelBasedClusteringAttributes::ID_var2:
            var2->blockSignals(true);
            var2->setText(QString(atts->GetVar2().c_str()));
            var2->blockSignals(false);
            break;
          case ModelBasedClusteringAttributes::ID_var3:
            var3->blockSignals(true);
            var3->setText(QString(atts->GetVar3().c_str()));
            var3->blockSignals(false);
            break;
        }
    }
}


// ****************************************************************************
// Method: QvisModelBasedClusteringWindow::GetCurrentValues
//
// Purpose: 
//   Gets values from certain widgets and stores them in the subject.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisModelBasedClusteringWindow::GetCurrentValues(int which_widget)
{
    bool doAll = (which_widget == -1);

    // Do numClusters
    if(which_widget == ModelBasedClusteringAttributes::ID_numClusters || doAll)
    {
        int val;
        if(LineEditGetInt(numClusters, val))
            atts->SetNumClusters(val);
        else
        {
            ResettingError(tr("Number of clusters"),
                IntToQString(atts->GetNumClusters()));
            atts->SetNumClusters(atts->GetNumClusters());
        }
    }

    // Do threshold
    if(which_widget == ModelBasedClusteringAttributes::ID_threshold || doAll)
    {
        double val;
        if(LineEditGetDouble(threshold, val))
            atts->SetThreshold(val);
        else
        {
            ResettingError(tr("Threshold"),
                DoubleToQString(atts->GetThreshold()));
            atts->SetThreshold(atts->GetThreshold());
        }
    }

    // Do percentile
    if(which_widget == ModelBasedClusteringAttributes::ID_percentile || doAll)
    {
        double val;
        if(LineEditGetDouble(percentile, val))
            atts->SetPercentile(val);
        else
        {
            ResettingError(tr("Percentile"),
                DoubleToQString(atts->GetPercentile()));
            atts->SetPercentile(atts->GetPercentile());
        }
    }

}


//
// Qt Slot functions
//


void
QvisModelBasedClusteringWindow::numClustersProcessText()
{
    GetCurrentValues(ModelBasedClusteringAttributes::ID_numClusters);
    Apply();
}


void
QvisModelBasedClusteringWindow::clusterMethodChanged(int val)
{
    if(val != atts->GetClusterMethod())
    {
        atts->SetClusterMethod(ModelBasedClusteringAttributes::clusterAlgorithmType(val));
        SetUpdate(false);
        Apply();
    }
}


void
QvisModelBasedClusteringWindow::thresholdProcessText()
{
    GetCurrentValues(ModelBasedClusteringAttributes::ID_threshold);
    Apply();
}


void
QvisModelBasedClusteringWindow::percentileProcessText()
{
    GetCurrentValues(ModelBasedClusteringAttributes::ID_percentile);
    Apply();
}


void
QvisModelBasedClusteringWindow::var2Changed(const QString &varName)
{
    atts->SetVar2(varName.toStdString());
    SetUpdate(false);
    Apply();
}


void
QvisModelBasedClusteringWindow::var3Changed(const QString &varName)
{
    atts->SetVar3(varName.toStdString());
    SetUpdate(false);
    Apply();
}


