/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include <PyPeaksOverThresholdAttributes.h>
#include <ObserverToCallback.h>
#include <stdio.h>
#include <snprintf.h>

// ****************************************************************************
// Module: PyPeaksOverThresholdAttributes
//
// Purpose: 
//   Attributes for PeaksOverThreshold operator
//
// Note:       Autogenerated by xml2python. Do not modify by hand!
//
// Programmer: xml2python
// Creation:   omitted
//
// ****************************************************************************

//
// This struct contains the Python type information and a PeaksOverThresholdAttributes.
//
struct PeaksOverThresholdAttributesObject
{
    PyObject_HEAD
    PeaksOverThresholdAttributes *data;
    bool        owns;
    PyObject   *parent;
};

//
// Internal prototypes
//
static PyObject *NewPeaksOverThresholdAttributes(int);

std::string
PyPeaksOverThresholdAttributes_ToString(const PeaksOverThresholdAttributes *atts, const char *prefix)
{
    std::string str;
    char tmpStr[1000];

    SNPRINTF(tmpStr, 1000, "%sdataYearBegin = %d\n", prefix, atts->GetDataYearBegin());
    str += tmpStr;
    if(atts->GetDataAnalysisYearRangeEnabled())
        SNPRINTF(tmpStr, 1000, "%sdataAnalysisYearRangeEnabled = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sdataAnalysisYearRangeEnabled = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sdataAnalysisYear1 = %d\n", prefix, atts->GetDataAnalysisYear1());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sdataAnalysisYear2 = %d\n", prefix, atts->GetDataAnalysisYear2());
    str += tmpStr;
    if(atts->GetEnsemble())
        SNPRINTF(tmpStr, 1000, "%sensemble = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sensemble = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%snumEnsembles = %d\n", prefix, atts->GetNumEnsembles());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%scutoff = %g\n", prefix, atts->GetCutoff());
    str += tmpStr;
    const char *cutoffMode_names = "UPPER_TAIL, LOWER_TAIL";
    switch (atts->GetCutoffMode())
    {
      case PeaksOverThresholdAttributes::UPPER_TAIL:
          SNPRINTF(tmpStr, 1000, "%scutoffMode = %sUPPER_TAIL  # %s\n", prefix, prefix, cutoffMode_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::LOWER_TAIL:
          SNPRINTF(tmpStr, 1000, "%scutoffMode = %sLOWER_TAIL  # %s\n", prefix, prefix, cutoffMode_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    if(atts->GetNoConsecutiveDay())
        SNPRINTF(tmpStr, 1000, "%snoConsecutiveDay = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%snoConsecutiveDay = 0\n", prefix);
    str += tmpStr;
    const char *optimizationMethod_names = "NELDER_MEAD, BFGS";
    switch (atts->GetOptimizationMethod())
    {
      case PeaksOverThresholdAttributes::NELDER_MEAD:
          SNPRINTF(tmpStr, 1000, "%soptimizationMethod = %sNELDER_MEAD  # %s\n", prefix, prefix, optimizationMethod_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::BFGS:
          SNPRINTF(tmpStr, 1000, "%soptimizationMethod = %sBFGS  # %s\n", prefix, prefix, optimizationMethod_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sdataScaling = %g\n", prefix, atts->GetDataScaling());
    str += tmpStr;
    const char *aggregation_names = "ANNUAL, SEASONAL, MONTHLY";
    switch (atts->GetAggregation())
    {
      case PeaksOverThresholdAttributes::ANNUAL:
          SNPRINTF(tmpStr, 1000, "%saggregation = %sANNUAL  # %s\n", prefix, prefix, aggregation_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::SEASONAL:
          SNPRINTF(tmpStr, 1000, "%saggregation = %sSEASONAL  # %s\n", prefix, prefix, aggregation_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::MONTHLY:
          SNPRINTF(tmpStr, 1000, "%saggregation = %sMONTHLY  # %s\n", prefix, prefix, aggregation_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sannualPercentile = %g\n", prefix, atts->GetAnnualPercentile());
    str += tmpStr;
    {   const double *seasonalPercentile = atts->GetSeasonalPercentile();
        SNPRINTF(tmpStr, 1000, "%sseasonalPercentile = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 4; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", seasonalPercentile[i]);
            str += tmpStr;
            if(i < 3)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    {   const double *monthlyPercentile = atts->GetMonthlyPercentile();
        SNPRINTF(tmpStr, 1000, "%smonthlyPercentile = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 12; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", monthlyPercentile[i]);
            str += tmpStr;
            if(i < 11)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    SNPRINTF(tmpStr, 1000, "%sdaysPerYear = %d\n", prefix, atts->GetDaysPerYear());
    str += tmpStr;
    {   const int *daysPerMonth = atts->GetDaysPerMonth();
        SNPRINTF(tmpStr, 1000, "%sdaysPerMonth = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 12; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%d", daysPerMonth[i]);
            str += tmpStr;
            if(i < 11)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    if(atts->GetCovariateModelScale())
        SNPRINTF(tmpStr, 1000, "%scovariateModelScale = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scovariateModelScale = 0\n", prefix);
    str += tmpStr;
    if(atts->GetCovariateModelLocation())
        SNPRINTF(tmpStr, 1000, "%scovariateModelLocation = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scovariateModelLocation = 0\n", prefix);
    str += tmpStr;
    if(atts->GetCovariateModelShape())
        SNPRINTF(tmpStr, 1000, "%scovariateModelShape = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scovariateModelShape = 0\n", prefix);
    str += tmpStr;
    if(atts->GetComputeCovariates())
        SNPRINTF(tmpStr, 1000, "%scomputeCovariates = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scomputeCovariates = 0\n", prefix);
    str += tmpStr;
    {   const intVector &covariateReturnYears = atts->GetCovariateReturnYears();
        SNPRINTF(tmpStr, 1000, "%scovariateReturnYears = (", prefix);
        str += tmpStr;
        for(size_t i = 0; i < covariateReturnYears.size(); ++i)
        {
            SNPRINTF(tmpStr, 1000, "%d", covariateReturnYears[i]);
            str += tmpStr;
            if(i < covariateReturnYears.size() - 1)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    if(atts->GetComputeRVDifferences())
        SNPRINTF(tmpStr, 1000, "%scomputeRVDifferences = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scomputeRVDifferences = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%srvDifference1 = %d\n", prefix, atts->GetRvDifference1());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%srvDifference2 = %d\n", prefix, atts->GetRvDifference2());
    str += tmpStr;
    if(atts->GetComputeParamValues())
        SNPRINTF(tmpStr, 1000, "%scomputeParamValues = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%scomputeParamValues = 0\n", prefix);
    str += tmpStr;
    const char *displaySeason_names = "WINTER, SPRING, SUMMER, FALL";
    switch (atts->GetDisplaySeason())
    {
      case PeaksOverThresholdAttributes::WINTER:
          SNPRINTF(tmpStr, 1000, "%sdisplaySeason = %sWINTER  # %s\n", prefix, prefix, displaySeason_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::SPRING:
          SNPRINTF(tmpStr, 1000, "%sdisplaySeason = %sSPRING  # %s\n", prefix, prefix, displaySeason_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::SUMMER:
          SNPRINTF(tmpStr, 1000, "%sdisplaySeason = %sSUMMER  # %s\n", prefix, prefix, displaySeason_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::FALL:
          SNPRINTF(tmpStr, 1000, "%sdisplaySeason = %sFALL  # %s\n", prefix, prefix, displaySeason_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *displayMonth_names = "JAN, FEB, MAR, APR, MAY, "
        "JUN, JUL, AUG, SEP, "
        "OCT, NOV, DEC";
    switch (atts->GetDisplayMonth())
    {
      case PeaksOverThresholdAttributes::JAN:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sJAN  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::FEB:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sFEB  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::MAR:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sMAR  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::APR:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sAPR  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::MAY:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sMAY  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::JUN:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sJUN  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::JUL:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sJUL  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::AUG:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sAUG  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::SEP:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sSEP  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::OCT:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sOCT  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::NOV:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sNOV  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      case PeaksOverThresholdAttributes::DEC:
          SNPRINTF(tmpStr, 1000, "%sdisplayMonth = %sDEC  # %s\n", prefix, prefix, displayMonth_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    if(atts->GetDumpData())
        SNPRINTF(tmpStr, 1000, "%sdumpData = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sdumpData = 0\n", prefix);
    str += tmpStr;
    if(atts->GetDumpDebug())
        SNPRINTF(tmpStr, 1000, "%sdumpDebug = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sdumpDebug = 0\n", prefix);
    str += tmpStr;
    return str;
}

static PyObject *
PeaksOverThresholdAttributes_Notify(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    obj->data->Notify();
    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDataYearBegin(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dataYearBegin in the object.
    obj->data->SetDataYearBegin((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDataYearBegin(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDataYearBegin()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDataAnalysisYearRangeEnabled(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dataAnalysisYearRangeEnabled in the object.
    obj->data->SetDataAnalysisYearRangeEnabled(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDataAnalysisYearRangeEnabled(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetDataAnalysisYearRangeEnabled()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDataAnalysisYear1(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dataAnalysisYear1 in the object.
    obj->data->SetDataAnalysisYear1((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDataAnalysisYear1(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDataAnalysisYear1()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDataAnalysisYear2(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dataAnalysisYear2 in the object.
    obj->data->SetDataAnalysisYear2((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDataAnalysisYear2(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDataAnalysisYear2()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetEnsemble(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the ensemble in the object.
    obj->data->SetEnsemble(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetEnsemble(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetEnsemble()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetNumEnsembles(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the numEnsembles in the object.
    obj->data->SetNumEnsembles((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetNumEnsembles(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetNumEnsembles()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetCutoff(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    float fval;
    if(!PyArg_ParseTuple(args, "f", &fval))
        return NULL;

    // Set the cutoff in the object.
    obj->data->SetCutoff(fval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetCutoff(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(double(obj->data->GetCutoff()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetCutoffMode(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the cutoffMode in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetCutoffMode(PeaksOverThresholdAttributes::CutoffModeType(ival));
    else
    {
        fprintf(stderr, "An invalid cutoffMode value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "UPPER_TAIL, LOWER_TAIL.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetCutoffMode(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetCutoffMode()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetNoConsecutiveDay(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the noConsecutiveDay in the object.
    obj->data->SetNoConsecutiveDay(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetNoConsecutiveDay(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetNoConsecutiveDay()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetOptimizationMethod(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the optimizationMethod in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetOptimizationMethod(PeaksOverThresholdAttributes::OptimizationType(ival));
    else
    {
        fprintf(stderr, "An invalid optimizationMethod value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "NELDER_MEAD, BFGS.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetOptimizationMethod(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetOptimizationMethod()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDataScaling(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the dataScaling in the object.
    obj->data->SetDataScaling(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDataScaling(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetDataScaling());
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetAggregation(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the aggregation in the object.
    if(ival >= 0 && ival < 3)
        obj->data->SetAggregation(PeaksOverThresholdAttributes::AggregationType(ival));
    else
    {
        fprintf(stderr, "An invalid aggregation value was given. "
                        "Valid values are in the range of [0,2]. "
                        "You can also use the following names: "
                        "ANNUAL, SEASONAL, MONTHLY.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetAggregation(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetAggregation()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetAnnualPercentile(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the annualPercentile in the object.
    obj->data->SetAnnualPercentile(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetAnnualPercentile(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetAnnualPercentile());
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetSeasonalPercentile(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    double *dvals = obj->data->GetSeasonalPercentile();
    if(!PyArg_ParseTuple(args, "dddd", &dvals[0], &dvals[1], &dvals[2], &dvals[3]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 4)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the seasonalPercentile in the object as modified.
    obj->data->SelectSeasonalPercentile();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetSeasonalPercentile(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the seasonalPercentile.
    PyObject *retval = PyTuple_New(4);
    const double *seasonalPercentile = obj->data->GetSeasonalPercentile();
    for(int i = 0; i < 4; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(seasonalPercentile[i]));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetMonthlyPercentile(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    double *dvals = obj->data->GetMonthlyPercentile();
    if(!PyArg_ParseTuple(args, "dddddddddddd", &dvals[0], &dvals[1], &dvals[2], &dvals[3], &dvals[4], &dvals[5], &dvals[6], &dvals[7], &dvals[8], &dvals[9], &dvals[10], &dvals[11]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 12)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the monthlyPercentile in the object as modified.
    obj->data->SelectMonthlyPercentile();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetMonthlyPercentile(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the monthlyPercentile.
    PyObject *retval = PyTuple_New(12);
    const double *monthlyPercentile = obj->data->GetMonthlyPercentile();
    for(int i = 0; i < 12; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(monthlyPercentile[i]));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDaysPerYear(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the daysPerYear in the object.
    obj->data->SetDaysPerYear((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDaysPerYear(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDaysPerYear()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDaysPerMonth(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int *ivals = obj->data->GetDaysPerMonth();
    if(!PyArg_ParseTuple(args, "iiiiiiiiiiii", &ivals[0], &ivals[1], &ivals[2], &ivals[3], &ivals[4], &ivals[5], &ivals[6], &ivals[7], &ivals[8], &ivals[9], &ivals[10], &ivals[11]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 12)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    ivals[i] = int(PyFloat_AS_DOUBLE(item));
                else if(PyInt_Check(item))
                    ivals[i] = int(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    ivals[i] = int(PyLong_AsDouble(item));
                else
                    ivals[i] = 0;
            }
        }
        else
            return NULL;
    }

    // Mark the daysPerMonth in the object as modified.
    obj->data->SelectDaysPerMonth();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDaysPerMonth(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the daysPerMonth.
    PyObject *retval = PyTuple_New(12);
    const int *daysPerMonth = obj->data->GetDaysPerMonth();
    for(int i = 0; i < 12; ++i)
        PyTuple_SET_ITEM(retval, i, PyInt_FromLong(long(daysPerMonth[i])));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetCovariateModelScale(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the covariateModelScale in the object.
    obj->data->SetCovariateModelScale(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetCovariateModelScale(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetCovariateModelScale()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetCovariateModelLocation(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the covariateModelLocation in the object.
    obj->data->SetCovariateModelLocation(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetCovariateModelLocation(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetCovariateModelLocation()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetCovariateModelShape(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the covariateModelShape in the object.
    obj->data->SetCovariateModelShape(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetCovariateModelShape(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetCovariateModelShape()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetComputeCovariates(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the computeCovariates in the object.
    obj->data->SetComputeCovariates(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetComputeCovariates(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetComputeCovariates()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetCovariateReturnYears(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    intVector  &vec = obj->data->GetCovariateReturnYears();
    PyObject   *tuple;
    if(!PyArg_ParseTuple(args, "O", &tuple))
        return NULL;

    if(PyTuple_Check(tuple))
    {
        vec.resize(PyTuple_Size(tuple));
        for(int i = 0; i < PyTuple_Size(tuple); ++i)
        {
            PyObject *item = PyTuple_GET_ITEM(tuple, i);
            if(PyFloat_Check(item))
                vec[i] = int(PyFloat_AS_DOUBLE(item));
            else if(PyInt_Check(item))
                vec[i] = int(PyInt_AS_LONG(item));
            else if(PyLong_Check(item))
                vec[i] = int(PyLong_AsLong(item));
            else
                vec[i] = 0;
        }
    }
    else if(PyFloat_Check(tuple))
    {
        vec.resize(1);
        vec[0] = int(PyFloat_AS_DOUBLE(tuple));
    }
    else if(PyInt_Check(tuple))
    {
        vec.resize(1);
        vec[0] = int(PyInt_AS_LONG(tuple));
    }
    else if(PyLong_Check(tuple))
    {
        vec.resize(1);
        vec[0] = int(PyLong_AsLong(tuple));
    }
    else
        return NULL;

    // Mark the covariateReturnYears in the object as modified.
    obj->data->SelectCovariateReturnYears();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetCovariateReturnYears(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the covariateReturnYears.
    const intVector &covariateReturnYears = obj->data->GetCovariateReturnYears();
    PyObject *retval = PyTuple_New(covariateReturnYears.size());
    for(size_t i = 0; i < covariateReturnYears.size(); ++i)
        PyTuple_SET_ITEM(retval, i, PyInt_FromLong(long(covariateReturnYears[i])));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetComputeRVDifferences(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the computeRVDifferences in the object.
    obj->data->SetComputeRVDifferences(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetComputeRVDifferences(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetComputeRVDifferences()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetRvDifference1(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the rvDifference1 in the object.
    obj->data->SetRvDifference1((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetRvDifference1(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetRvDifference1()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetRvDifference2(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the rvDifference2 in the object.
    obj->data->SetRvDifference2((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetRvDifference2(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetRvDifference2()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetComputeParamValues(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the computeParamValues in the object.
    obj->data->SetComputeParamValues(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetComputeParamValues(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetComputeParamValues()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDisplaySeason(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the displaySeason in the object.
    if(ival >= 0 && ival < 4)
        obj->data->SetDisplaySeason(PeaksOverThresholdAttributes::SeasonType(ival));
    else
    {
        fprintf(stderr, "An invalid displaySeason value was given. "
                        "Valid values are in the range of [0,3]. "
                        "You can also use the following names: "
                        "WINTER, SPRING, SUMMER, FALL.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDisplaySeason(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDisplaySeason()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDisplayMonth(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the displayMonth in the object.
    if(ival >= 0 && ival < 12)
        obj->data->SetDisplayMonth(PeaksOverThresholdAttributes::MonthType(ival));
    else
    {
        fprintf(stderr, "An invalid displayMonth value was given. "
                        "Valid values are in the range of [0,11]. "
                        "You can also use the following names: "
                        "JAN, FEB, MAR, APR, MAY, "
                        "JUN, JUL, AUG, SEP, "
                        "OCT, NOV, DEC.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDisplayMonth(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDisplayMonth()));
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDumpData(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dumpData in the object.
    obj->data->SetDumpData(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDumpData(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetDumpData()?1L:0L);
    return retval;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_SetDumpDebug(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the dumpDebug in the object.
    obj->data->SetDumpDebug(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
PeaksOverThresholdAttributes_GetDumpDebug(PyObject *self, PyObject *args)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetDumpDebug()?1L:0L);
    return retval;
}



PyMethodDef PyPeaksOverThresholdAttributes_methods[PEAKSOVERTHRESHOLDATTRIBUTES_NMETH] = {
    {"Notify", PeaksOverThresholdAttributes_Notify, METH_VARARGS},
    {"SetDataYearBegin", PeaksOverThresholdAttributes_SetDataYearBegin, METH_VARARGS},
    {"GetDataYearBegin", PeaksOverThresholdAttributes_GetDataYearBegin, METH_VARARGS},
    {"SetDataAnalysisYearRangeEnabled", PeaksOverThresholdAttributes_SetDataAnalysisYearRangeEnabled, METH_VARARGS},
    {"GetDataAnalysisYearRangeEnabled", PeaksOverThresholdAttributes_GetDataAnalysisYearRangeEnabled, METH_VARARGS},
    {"SetDataAnalysisYear1", PeaksOverThresholdAttributes_SetDataAnalysisYear1, METH_VARARGS},
    {"GetDataAnalysisYear1", PeaksOverThresholdAttributes_GetDataAnalysisYear1, METH_VARARGS},
    {"SetDataAnalysisYear2", PeaksOverThresholdAttributes_SetDataAnalysisYear2, METH_VARARGS},
    {"GetDataAnalysisYear2", PeaksOverThresholdAttributes_GetDataAnalysisYear2, METH_VARARGS},
    {"SetEnsemble", PeaksOverThresholdAttributes_SetEnsemble, METH_VARARGS},
    {"GetEnsemble", PeaksOverThresholdAttributes_GetEnsemble, METH_VARARGS},
    {"SetNumEnsembles", PeaksOverThresholdAttributes_SetNumEnsembles, METH_VARARGS},
    {"GetNumEnsembles", PeaksOverThresholdAttributes_GetNumEnsembles, METH_VARARGS},
    {"SetCutoff", PeaksOverThresholdAttributes_SetCutoff, METH_VARARGS},
    {"GetCutoff", PeaksOverThresholdAttributes_GetCutoff, METH_VARARGS},
    {"SetCutoffMode", PeaksOverThresholdAttributes_SetCutoffMode, METH_VARARGS},
    {"GetCutoffMode", PeaksOverThresholdAttributes_GetCutoffMode, METH_VARARGS},
    {"SetNoConsecutiveDay", PeaksOverThresholdAttributes_SetNoConsecutiveDay, METH_VARARGS},
    {"GetNoConsecutiveDay", PeaksOverThresholdAttributes_GetNoConsecutiveDay, METH_VARARGS},
    {"SetOptimizationMethod", PeaksOverThresholdAttributes_SetOptimizationMethod, METH_VARARGS},
    {"GetOptimizationMethod", PeaksOverThresholdAttributes_GetOptimizationMethod, METH_VARARGS},
    {"SetDataScaling", PeaksOverThresholdAttributes_SetDataScaling, METH_VARARGS},
    {"GetDataScaling", PeaksOverThresholdAttributes_GetDataScaling, METH_VARARGS},
    {"SetAggregation", PeaksOverThresholdAttributes_SetAggregation, METH_VARARGS},
    {"GetAggregation", PeaksOverThresholdAttributes_GetAggregation, METH_VARARGS},
    {"SetAnnualPercentile", PeaksOverThresholdAttributes_SetAnnualPercentile, METH_VARARGS},
    {"GetAnnualPercentile", PeaksOverThresholdAttributes_GetAnnualPercentile, METH_VARARGS},
    {"SetSeasonalPercentile", PeaksOverThresholdAttributes_SetSeasonalPercentile, METH_VARARGS},
    {"GetSeasonalPercentile", PeaksOverThresholdAttributes_GetSeasonalPercentile, METH_VARARGS},
    {"SetMonthlyPercentile", PeaksOverThresholdAttributes_SetMonthlyPercentile, METH_VARARGS},
    {"GetMonthlyPercentile", PeaksOverThresholdAttributes_GetMonthlyPercentile, METH_VARARGS},
    {"SetDaysPerYear", PeaksOverThresholdAttributes_SetDaysPerYear, METH_VARARGS},
    {"GetDaysPerYear", PeaksOverThresholdAttributes_GetDaysPerYear, METH_VARARGS},
    {"SetDaysPerMonth", PeaksOverThresholdAttributes_SetDaysPerMonth, METH_VARARGS},
    {"GetDaysPerMonth", PeaksOverThresholdAttributes_GetDaysPerMonth, METH_VARARGS},
    {"SetCovariateModelScale", PeaksOverThresholdAttributes_SetCovariateModelScale, METH_VARARGS},
    {"GetCovariateModelScale", PeaksOverThresholdAttributes_GetCovariateModelScale, METH_VARARGS},
    {"SetCovariateModelLocation", PeaksOverThresholdAttributes_SetCovariateModelLocation, METH_VARARGS},
    {"GetCovariateModelLocation", PeaksOverThresholdAttributes_GetCovariateModelLocation, METH_VARARGS},
    {"SetCovariateModelShape", PeaksOverThresholdAttributes_SetCovariateModelShape, METH_VARARGS},
    {"GetCovariateModelShape", PeaksOverThresholdAttributes_GetCovariateModelShape, METH_VARARGS},
    {"SetComputeCovariates", PeaksOverThresholdAttributes_SetComputeCovariates, METH_VARARGS},
    {"GetComputeCovariates", PeaksOverThresholdAttributes_GetComputeCovariates, METH_VARARGS},
    {"SetCovariateReturnYears", PeaksOverThresholdAttributes_SetCovariateReturnYears, METH_VARARGS},
    {"GetCovariateReturnYears", PeaksOverThresholdAttributes_GetCovariateReturnYears, METH_VARARGS},
    {"SetComputeRVDifferences", PeaksOverThresholdAttributes_SetComputeRVDifferences, METH_VARARGS},
    {"GetComputeRVDifferences", PeaksOverThresholdAttributes_GetComputeRVDifferences, METH_VARARGS},
    {"SetRvDifference1", PeaksOverThresholdAttributes_SetRvDifference1, METH_VARARGS},
    {"GetRvDifference1", PeaksOverThresholdAttributes_GetRvDifference1, METH_VARARGS},
    {"SetRvDifference2", PeaksOverThresholdAttributes_SetRvDifference2, METH_VARARGS},
    {"GetRvDifference2", PeaksOverThresholdAttributes_GetRvDifference2, METH_VARARGS},
    {"SetComputeParamValues", PeaksOverThresholdAttributes_SetComputeParamValues, METH_VARARGS},
    {"GetComputeParamValues", PeaksOverThresholdAttributes_GetComputeParamValues, METH_VARARGS},
    {"SetDisplaySeason", PeaksOverThresholdAttributes_SetDisplaySeason, METH_VARARGS},
    {"GetDisplaySeason", PeaksOverThresholdAttributes_GetDisplaySeason, METH_VARARGS},
    {"SetDisplayMonth", PeaksOverThresholdAttributes_SetDisplayMonth, METH_VARARGS},
    {"GetDisplayMonth", PeaksOverThresholdAttributes_GetDisplayMonth, METH_VARARGS},
    {"SetDumpData", PeaksOverThresholdAttributes_SetDumpData, METH_VARARGS},
    {"GetDumpData", PeaksOverThresholdAttributes_GetDumpData, METH_VARARGS},
    {"SetDumpDebug", PeaksOverThresholdAttributes_SetDumpDebug, METH_VARARGS},
    {"GetDumpDebug", PeaksOverThresholdAttributes_GetDumpDebug, METH_VARARGS},
    {NULL, NULL}
};

//
// Type functions
//

static void
PeaksOverThresholdAttributes_dealloc(PyObject *v)
{
   PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)v;
   if(obj->parent != 0)
       Py_DECREF(obj->parent);
   if(obj->owns)
       delete obj->data;
}

static int
PeaksOverThresholdAttributes_compare(PyObject *v, PyObject *w)
{
    PeaksOverThresholdAttributes *a = ((PeaksOverThresholdAttributesObject *)v)->data;
    PeaksOverThresholdAttributes *b = ((PeaksOverThresholdAttributesObject *)w)->data;
    return (*a == *b) ? 0 : -1;
}

PyObject *
PyPeaksOverThresholdAttributes_getattr(PyObject *self, char *name)
{
    if(strcmp(name, "dataYearBegin") == 0)
        return PeaksOverThresholdAttributes_GetDataYearBegin(self, NULL);
    if(strcmp(name, "dataAnalysisYearRangeEnabled") == 0)
        return PeaksOverThresholdAttributes_GetDataAnalysisYearRangeEnabled(self, NULL);
    if(strcmp(name, "dataAnalysisYear1") == 0)
        return PeaksOverThresholdAttributes_GetDataAnalysisYear1(self, NULL);
    if(strcmp(name, "dataAnalysisYear2") == 0)
        return PeaksOverThresholdAttributes_GetDataAnalysisYear2(self, NULL);
    if(strcmp(name, "ensemble") == 0)
        return PeaksOverThresholdAttributes_GetEnsemble(self, NULL);
    if(strcmp(name, "numEnsembles") == 0)
        return PeaksOverThresholdAttributes_GetNumEnsembles(self, NULL);
    if(strcmp(name, "cutoff") == 0)
        return PeaksOverThresholdAttributes_GetCutoff(self, NULL);
    if(strcmp(name, "cutoffMode") == 0)
        return PeaksOverThresholdAttributes_GetCutoffMode(self, NULL);
    if(strcmp(name, "UPPER_TAIL") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::UPPER_TAIL));
    if(strcmp(name, "LOWER_TAIL") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::LOWER_TAIL));

    if(strcmp(name, "noConsecutiveDay") == 0)
        return PeaksOverThresholdAttributes_GetNoConsecutiveDay(self, NULL);
    if(strcmp(name, "optimizationMethod") == 0)
        return PeaksOverThresholdAttributes_GetOptimizationMethod(self, NULL);
    if(strcmp(name, "NELDER_MEAD") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::NELDER_MEAD));
    if(strcmp(name, "BFGS") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::BFGS));

    if(strcmp(name, "dataScaling") == 0)
        return PeaksOverThresholdAttributes_GetDataScaling(self, NULL);
    if(strcmp(name, "aggregation") == 0)
        return PeaksOverThresholdAttributes_GetAggregation(self, NULL);
    if(strcmp(name, "ANNUAL") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::ANNUAL));
    if(strcmp(name, "SEASONAL") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::SEASONAL));
    if(strcmp(name, "MONTHLY") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::MONTHLY));

    if(strcmp(name, "annualPercentile") == 0)
        return PeaksOverThresholdAttributes_GetAnnualPercentile(self, NULL);
    if(strcmp(name, "seasonalPercentile") == 0)
        return PeaksOverThresholdAttributes_GetSeasonalPercentile(self, NULL);
    if(strcmp(name, "monthlyPercentile") == 0)
        return PeaksOverThresholdAttributes_GetMonthlyPercentile(self, NULL);
    if(strcmp(name, "daysPerYear") == 0)
        return PeaksOverThresholdAttributes_GetDaysPerYear(self, NULL);
    if(strcmp(name, "daysPerMonth") == 0)
        return PeaksOverThresholdAttributes_GetDaysPerMonth(self, NULL);
    if(strcmp(name, "covariateModelScale") == 0)
        return PeaksOverThresholdAttributes_GetCovariateModelScale(self, NULL);
    if(strcmp(name, "covariateModelLocation") == 0)
        return PeaksOverThresholdAttributes_GetCovariateModelLocation(self, NULL);
    if(strcmp(name, "covariateModelShape") == 0)
        return PeaksOverThresholdAttributes_GetCovariateModelShape(self, NULL);
    if(strcmp(name, "computeCovariates") == 0)
        return PeaksOverThresholdAttributes_GetComputeCovariates(self, NULL);
    if(strcmp(name, "covariateReturnYears") == 0)
        return PeaksOverThresholdAttributes_GetCovariateReturnYears(self, NULL);
    if(strcmp(name, "computeRVDifferences") == 0)
        return PeaksOverThresholdAttributes_GetComputeRVDifferences(self, NULL);
    if(strcmp(name, "rvDifference1") == 0)
        return PeaksOverThresholdAttributes_GetRvDifference1(self, NULL);
    if(strcmp(name, "rvDifference2") == 0)
        return PeaksOverThresholdAttributes_GetRvDifference2(self, NULL);
    if(strcmp(name, "computeParamValues") == 0)
        return PeaksOverThresholdAttributes_GetComputeParamValues(self, NULL);
    if(strcmp(name, "displaySeason") == 0)
        return PeaksOverThresholdAttributes_GetDisplaySeason(self, NULL);
    if(strcmp(name, "WINTER") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::WINTER));
    if(strcmp(name, "SPRING") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::SPRING));
    if(strcmp(name, "SUMMER") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::SUMMER));
    if(strcmp(name, "FALL") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::FALL));

    if(strcmp(name, "displayMonth") == 0)
        return PeaksOverThresholdAttributes_GetDisplayMonth(self, NULL);
    if(strcmp(name, "JAN") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::JAN));
    if(strcmp(name, "FEB") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::FEB));
    if(strcmp(name, "MAR") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::MAR));
    if(strcmp(name, "APR") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::APR));
    if(strcmp(name, "MAY") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::MAY));
    if(strcmp(name, "JUN") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::JUN));
    if(strcmp(name, "JUL") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::JUL));
    if(strcmp(name, "AUG") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::AUG));
    if(strcmp(name, "SEP") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::SEP));
    if(strcmp(name, "OCT") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::OCT));
    if(strcmp(name, "NOV") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::NOV));
    if(strcmp(name, "DEC") == 0)
        return PyInt_FromLong(long(PeaksOverThresholdAttributes::DEC));

    if(strcmp(name, "dumpData") == 0)
        return PeaksOverThresholdAttributes_GetDumpData(self, NULL);
    if(strcmp(name, "dumpDebug") == 0)
        return PeaksOverThresholdAttributes_GetDumpDebug(self, NULL);

    return Py_FindMethod(PyPeaksOverThresholdAttributes_methods, self, name);
}

int
PyPeaksOverThresholdAttributes_setattr(PyObject *self, char *name, PyObject *args)
{
    // Create a tuple to contain the arguments since all of the Set
    // functions expect a tuple.
    PyObject *tuple = PyTuple_New(1);
    PyTuple_SET_ITEM(tuple, 0, args);
    Py_INCREF(args);
    PyObject *obj = NULL;

    if(strcmp(name, "dataYearBegin") == 0)
        obj = PeaksOverThresholdAttributes_SetDataYearBegin(self, tuple);
    else if(strcmp(name, "dataAnalysisYearRangeEnabled") == 0)
        obj = PeaksOverThresholdAttributes_SetDataAnalysisYearRangeEnabled(self, tuple);
    else if(strcmp(name, "dataAnalysisYear1") == 0)
        obj = PeaksOverThresholdAttributes_SetDataAnalysisYear1(self, tuple);
    else if(strcmp(name, "dataAnalysisYear2") == 0)
        obj = PeaksOverThresholdAttributes_SetDataAnalysisYear2(self, tuple);
    else if(strcmp(name, "ensemble") == 0)
        obj = PeaksOverThresholdAttributes_SetEnsemble(self, tuple);
    else if(strcmp(name, "numEnsembles") == 0)
        obj = PeaksOverThresholdAttributes_SetNumEnsembles(self, tuple);
    else if(strcmp(name, "cutoff") == 0)
        obj = PeaksOverThresholdAttributes_SetCutoff(self, tuple);
    else if(strcmp(name, "cutoffMode") == 0)
        obj = PeaksOverThresholdAttributes_SetCutoffMode(self, tuple);
    else if(strcmp(name, "noConsecutiveDay") == 0)
        obj = PeaksOverThresholdAttributes_SetNoConsecutiveDay(self, tuple);
    else if(strcmp(name, "optimizationMethod") == 0)
        obj = PeaksOverThresholdAttributes_SetOptimizationMethod(self, tuple);
    else if(strcmp(name, "dataScaling") == 0)
        obj = PeaksOverThresholdAttributes_SetDataScaling(self, tuple);
    else if(strcmp(name, "aggregation") == 0)
        obj = PeaksOverThresholdAttributes_SetAggregation(self, tuple);
    else if(strcmp(name, "annualPercentile") == 0)
        obj = PeaksOverThresholdAttributes_SetAnnualPercentile(self, tuple);
    else if(strcmp(name, "seasonalPercentile") == 0)
        obj = PeaksOverThresholdAttributes_SetSeasonalPercentile(self, tuple);
    else if(strcmp(name, "monthlyPercentile") == 0)
        obj = PeaksOverThresholdAttributes_SetMonthlyPercentile(self, tuple);
    else if(strcmp(name, "daysPerYear") == 0)
        obj = PeaksOverThresholdAttributes_SetDaysPerYear(self, tuple);
    else if(strcmp(name, "daysPerMonth") == 0)
        obj = PeaksOverThresholdAttributes_SetDaysPerMonth(self, tuple);
    else if(strcmp(name, "covariateModelScale") == 0)
        obj = PeaksOverThresholdAttributes_SetCovariateModelScale(self, tuple);
    else if(strcmp(name, "covariateModelLocation") == 0)
        obj = PeaksOverThresholdAttributes_SetCovariateModelLocation(self, tuple);
    else if(strcmp(name, "covariateModelShape") == 0)
        obj = PeaksOverThresholdAttributes_SetCovariateModelShape(self, tuple);
    else if(strcmp(name, "computeCovariates") == 0)
        obj = PeaksOverThresholdAttributes_SetComputeCovariates(self, tuple);
    else if(strcmp(name, "covariateReturnYears") == 0)
        obj = PeaksOverThresholdAttributes_SetCovariateReturnYears(self, tuple);
    else if(strcmp(name, "computeRVDifferences") == 0)
        obj = PeaksOverThresholdAttributes_SetComputeRVDifferences(self, tuple);
    else if(strcmp(name, "rvDifference1") == 0)
        obj = PeaksOverThresholdAttributes_SetRvDifference1(self, tuple);
    else if(strcmp(name, "rvDifference2") == 0)
        obj = PeaksOverThresholdAttributes_SetRvDifference2(self, tuple);
    else if(strcmp(name, "computeParamValues") == 0)
        obj = PeaksOverThresholdAttributes_SetComputeParamValues(self, tuple);
    else if(strcmp(name, "displaySeason") == 0)
        obj = PeaksOverThresholdAttributes_SetDisplaySeason(self, tuple);
    else if(strcmp(name, "displayMonth") == 0)
        obj = PeaksOverThresholdAttributes_SetDisplayMonth(self, tuple);
    else if(strcmp(name, "dumpData") == 0)
        obj = PeaksOverThresholdAttributes_SetDumpData(self, tuple);
    else if(strcmp(name, "dumpDebug") == 0)
        obj = PeaksOverThresholdAttributes_SetDumpDebug(self, tuple);

    if(obj != NULL)
        Py_DECREF(obj);

    Py_DECREF(tuple);
    if( obj == NULL)
        PyErr_Format(PyExc_RuntimeError, "Unable to set unknown attribute: '%s'", name);
    return (obj != NULL) ? 0 : -1;
}

static int
PeaksOverThresholdAttributes_print(PyObject *v, FILE *fp, int flags)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)v;
    fprintf(fp, "%s", PyPeaksOverThresholdAttributes_ToString(obj->data, "").c_str());
    return 0;
}

PyObject *
PeaksOverThresholdAttributes_str(PyObject *v)
{
    PeaksOverThresholdAttributesObject *obj = (PeaksOverThresholdAttributesObject *)v;
    return PyString_FromString(PyPeaksOverThresholdAttributes_ToString(obj->data,"").c_str());
}

//
// The doc string for the class.
//
#if PY_MAJOR_VERSION > 2 || (PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION >= 5)
static const char *PeaksOverThresholdAttributes_Purpose = "Attributes for PeaksOverThreshold operator";
#else
static char *PeaksOverThresholdAttributes_Purpose = "Attributes for PeaksOverThreshold operator";
#endif

//
// The type description structure
//
static PyTypeObject PeaksOverThresholdAttributesType =
{
    //
    // Type header
    //
    PyObject_HEAD_INIT(&PyType_Type)
    0,                                   // ob_size
    "PeaksOverThresholdAttributes",                    // tp_name
    sizeof(PeaksOverThresholdAttributesObject),        // tp_basicsize
    0,                                   // tp_itemsize
    //
    // Standard methods
    //
    (destructor)PeaksOverThresholdAttributes_dealloc,  // tp_dealloc
    (printfunc)PeaksOverThresholdAttributes_print,     // tp_print
    (getattrfunc)PyPeaksOverThresholdAttributes_getattr, // tp_getattr
    (setattrfunc)PyPeaksOverThresholdAttributes_setattr, // tp_setattr
    (cmpfunc)PeaksOverThresholdAttributes_compare,     // tp_compare
    (reprfunc)0,                         // tp_repr
    //
    // Type categories
    //
    0,                                   // tp_as_number
    0,                                   // tp_as_sequence
    0,                                   // tp_as_mapping
    //
    // More methods
    //
    0,                                   // tp_hash
    0,                                   // tp_call
    (reprfunc)PeaksOverThresholdAttributes_str,        // tp_str
    0,                                   // tp_getattro
    0,                                   // tp_setattro
    0,                                   // tp_as_buffer
    Py_TPFLAGS_CHECKTYPES,               // tp_flags
    PeaksOverThresholdAttributes_Purpose,              // tp_doc
    0,                                   // tp_traverse
    0,                                   // tp_clear
    0,                                   // tp_richcompare
    0                                    // tp_weaklistoffset
};

//
// Helper functions for object allocation.
//

static PeaksOverThresholdAttributes *defaultAtts = 0;
static PeaksOverThresholdAttributes *currentAtts = 0;

static PyObject *
NewPeaksOverThresholdAttributes(int useCurrent)
{
    PeaksOverThresholdAttributesObject *newObject;
    newObject = PyObject_NEW(PeaksOverThresholdAttributesObject, &PeaksOverThresholdAttributesType);
    if(newObject == NULL)
        return NULL;
    if(useCurrent && currentAtts != 0)
        newObject->data = new PeaksOverThresholdAttributes(*currentAtts);
    else if(defaultAtts != 0)
        newObject->data = new PeaksOverThresholdAttributes(*defaultAtts);
    else
        newObject->data = new PeaksOverThresholdAttributes;
    newObject->owns = true;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

static PyObject *
WrapPeaksOverThresholdAttributes(const PeaksOverThresholdAttributes *attr)
{
    PeaksOverThresholdAttributesObject *newObject;
    newObject = PyObject_NEW(PeaksOverThresholdAttributesObject, &PeaksOverThresholdAttributesType);
    if(newObject == NULL)
        return NULL;
    newObject->data = (PeaksOverThresholdAttributes *)attr;
    newObject->owns = false;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

///////////////////////////////////////////////////////////////////////////////
//
// Interface that is exposed to the VisIt module.
//
///////////////////////////////////////////////////////////////////////////////

PyObject *
PeaksOverThresholdAttributes_new(PyObject *self, PyObject *args)
{
    int useCurrent = 0;
    if (!PyArg_ParseTuple(args, "i", &useCurrent))
    {
        if (!PyArg_ParseTuple(args, ""))
            return NULL;
        else
            PyErr_Clear();
    }

    return (PyObject *)NewPeaksOverThresholdAttributes(useCurrent);
}

//
// Plugin method table. These methods are added to the visitmodule's methods.
//
static PyMethodDef PeaksOverThresholdAttributesMethods[] = {
    {"PeaksOverThresholdAttributes", PeaksOverThresholdAttributes_new, METH_VARARGS},
    {NULL,      NULL}        /* Sentinel */
};

static Observer *PeaksOverThresholdAttributesObserver = 0;

std::string
PyPeaksOverThresholdAttributes_GetLogString()
{
    std::string s("PeaksOverThresholdAtts = PeaksOverThresholdAttributes()\n");
    if(currentAtts != 0)
        s += PyPeaksOverThresholdAttributes_ToString(currentAtts, "PeaksOverThresholdAtts.");
    return s;
}

static void
PyPeaksOverThresholdAttributes_CallLogRoutine(Subject *subj, void *data)
{
    typedef void (*logCallback)(const std::string &);
    logCallback cb = (logCallback)data;

    if(cb != 0)
    {
        std::string s("PeaksOverThresholdAtts = PeaksOverThresholdAttributes()\n");
        s += PyPeaksOverThresholdAttributes_ToString(currentAtts, "PeaksOverThresholdAtts.");
        cb(s);
    }
}

void
PyPeaksOverThresholdAttributes_StartUp(PeaksOverThresholdAttributes *subj, void *data)
{
    if(subj == 0)
        return;

    currentAtts = subj;
    PyPeaksOverThresholdAttributes_SetDefaults(subj);

    //
    // Create the observer that will be notified when the attributes change.
    //
    if(PeaksOverThresholdAttributesObserver == 0)
    {
        PeaksOverThresholdAttributesObserver = new ObserverToCallback(subj,
            PyPeaksOverThresholdAttributes_CallLogRoutine, (void *)data);
    }

}

void
PyPeaksOverThresholdAttributes_CloseDown()
{
    delete defaultAtts;
    defaultAtts = 0;
    delete PeaksOverThresholdAttributesObserver;
    PeaksOverThresholdAttributesObserver = 0;
}

PyMethodDef *
PyPeaksOverThresholdAttributes_GetMethodTable(int *nMethods)
{
    *nMethods = 1;
    return PeaksOverThresholdAttributesMethods;
}

bool
PyPeaksOverThresholdAttributes_Check(PyObject *obj)
{
    return (obj->ob_type == &PeaksOverThresholdAttributesType);
}

PeaksOverThresholdAttributes *
PyPeaksOverThresholdAttributes_FromPyObject(PyObject *obj)
{
    PeaksOverThresholdAttributesObject *obj2 = (PeaksOverThresholdAttributesObject *)obj;
    return obj2->data;
}

PyObject *
PyPeaksOverThresholdAttributes_New()
{
    return NewPeaksOverThresholdAttributes(0);
}

PyObject *
PyPeaksOverThresholdAttributes_Wrap(const PeaksOverThresholdAttributes *attr)
{
    return WrapPeaksOverThresholdAttributes(attr);
}

void
PyPeaksOverThresholdAttributes_SetParent(PyObject *obj, PyObject *parent)
{
    PeaksOverThresholdAttributesObject *obj2 = (PeaksOverThresholdAttributesObject *)obj;
    obj2->parent = parent;
}

void
PyPeaksOverThresholdAttributes_SetDefaults(const PeaksOverThresholdAttributes *atts)
{
    if(defaultAtts)
        delete defaultAtts;

    defaultAtts = new PeaksOverThresholdAttributes(*atts);
}

