/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

// ************************************************************************* //
//  File: avtPeaksOverThresholdFilter.C
// ************************************************************************* //

#include <avtPeaksOverThresholdFilter.h>
#include <avtRPOTFilter.h>
#include <avtDatasetExaminer.h>
#include <avtContract.h>
#include <avtSourceFromAVTDataset.h>
#include <vtkDataSet.h>
#include <vtkPointData.h>
#include <vtkCellData.h>
#include <vtkFloatArray.h>
#include <string.h>
#include <DebugStream.h>
#include <InstallationFunctions.h>
#include <string>
#include <InvalidFilesException.h>
#include <TimingsManager.h>

// ****************************************************************************
//  Method: avtPeaksOverThresholdFilter constructor
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Wed May 16 11:41:11 PDT 2012
//
// ****************************************************************************

avtPeaksOverThresholdFilter::avtPeaksOverThresholdFilter()
{
}


// ****************************************************************************
//  Method: avtPeaksOverThresholdFilter destructor
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Wed May 16 11:41:11 PDT 2012
//
//  Modifications:
//
// ****************************************************************************

avtPeaksOverThresholdFilter::~avtPeaksOverThresholdFilter()
{
}


// ****************************************************************************
//  Method:  avtPeaksOverThresholdFilter::Create
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Wed May 16 11:41:11 PDT 2012
//
// ****************************************************************************

avtFilter *
avtPeaksOverThresholdFilter::Create()
{
    return new avtPeaksOverThresholdFilter();
}


// ****************************************************************************
//  Method:      avtPeaksOverThresholdFilter::SetAtts
//
//  Purpose:
//      Sets the state of the filter based on the attribute object.
//
//  Arguments:
//      a        The attributes to use.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Wed May 16 11:41:11 PDT 2012
//
// ****************************************************************************

void
avtPeaksOverThresholdFilter::SetAtts(const AttributeGroup *a)
{
    atts = *(const PeaksOverThresholdAttributes*)a;
}


// ****************************************************************************
//  Method: avtPeaksOverThresholdFilter::Equivalent
//
//  Purpose:
//      Returns true if creating a new avtPeaksOverThresholdFilter with the given
//      parameters would result in an equivalent avtPeaksOverThresholdFilter.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Wed May 16 11:41:11 PDT 2012
//
// ****************************************************************************

bool
avtPeaksOverThresholdFilter::Equivalent(const AttributeGroup *a)
{
    return (atts == *(PeaksOverThresholdAttributes*)a);
}


// ****************************************************************************
//  Method: avtPeaksOverThresholdFilter::Execute
//
//  Purpose:
//      Sends the specified input and output through the PeaksOverThreshold filter.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Wed May 16 11:41:11 PDT 2012
//
// ****************************************************************************

void
avtPeaksOverThresholdFilter::Execute()
{
    if (atts.GetDataAnalysisYearRangeEnabled())
    {
        if (atts.GetDataYearBegin() < atts.GetDataAnalysisYear1() ||
            atts.GetDataYearBegin() > atts.GetDataAnalysisYear2())
        {
            EXCEPTION1(ImproperUseException, "Invalid data analysis begin year.");
        }
            
        if (atts.GetDataAnalysisYear1() >= atts.GetDataAnalysisYear2())
        {
            EXCEPTION1(ImproperUseException, "Invalid data analysis year range.");
        }
        if (atts.GetDataAnalysisYear1() < atts.GetDataYearBegin() ||
            atts.GetDataAnalysisYear2() < atts.GetDataYearBegin())
        {
            EXCEPTION1(ImproperUseException, "Invalid data analysis year range.");
        }
    }
    if (atts.GetEnsemble())
    {
        if (!atts.GetDataAnalysisYearRangeEnabled())
        {
            EXCEPTION1(ImproperUseException, "Ensemble usage requires year range to be set.");
        }
    }
    
    int t1 = visitTimer->StartTimer();
    avtRPOTFilter *f = new avtRPOTFilter();

    std::string vlibdir = GetVisItLibraryDirectory() + VISIT_SLASH_CHAR + "r_support";
    std::string vlibrdir  = vlibdir  + VISIT_SLASH_CHAR + "Rscripts" + VISIT_SLASH_CHAR;
    f->codeDir = vlibrdir;
    f->atts = atts;

    f->SetInput(GetInput());
    
    avtContract_p spec = GetInput()->GetOriginatingSource()->GetGeneralContract();
    avtDataObject_p dob = f->GetOutput();

    dob->Update(spec);
    avtDataTree_p tree = f->GetTypedOutput()->GetDataTree();

    visitTimer->StopTimer(t1, "avtPeaksOverThreshold: call avtRPOTFilter");

    //Set the output variable properly.
    int nleaves;
    vtkDataSet **leaves = tree->GetAllLeaves(nleaves);
    for (int i = 0; i < nleaves; i++)
    {
        if (leaves[i]->GetPointData()->GetScalars())
            leaves[i]->GetPointData()->GetScalars()->SetName(pipelineVariable);
        else if (leaves[i]->GetCellData()->GetScalars())
            leaves[i]->GetCellData()->GetScalars()->SetName(pipelineVariable);
    }
    delete [] leaves;

    SetOutputDataTree(tree);
    delete f;
}


// ****************************************************************************
//  Method: avtPeaksOverThresholdFilter::ModifyContract
//
//  Purpose:
//      Creates a contract the removes the operator-created-expression.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
// ****************************************************************************

avtContract_p
avtPeaksOverThresholdFilter::ModifyContract(avtContract_p in_contract)
{
    avtContract_p rv;
    if (strncmp(pipelineVariable, "operators/PeaksOverThreshold", strlen("operators/PeaksOverThreshold")) == 0)
    {
        outVarName = pipelineVariable;
        const char *justTheVar = pipelineVariable + strlen("operators/PeaksOverThreshold/");
        avtDataRequest_p dr = new avtDataRequest(in_contract->GetDataRequest(), justTheVar);
        rv = new avtContract(in_contract, dr);
    }
    else
    {
        rv = new avtContract(in_contract);
    }
    return rv;
}


// ****************************************************************************
//  Method: a avtPeaksOverThresholdFilter::UpdateDataObjectInfo
//
//  Purpose:
//      Tells output that we have a new variable.
//
//  Programmer: pugmire -- generated by xml2avt
//  Creation:   Tue Apr 3 09:21:07 PDT 2012
//
// ****************************************************************************

void
 avtPeaksOverThresholdFilter::UpdateDataObjectInfo(void)
{
   if (outVarName != "")
   {
       avtDataAttributes &atts = GetOutput()->GetInfo().GetAttributes();
       if (! atts.ValidVariable(outVarName))
       {
           atts.AddVariable(outVarName.c_str());
           atts.SetActiveVariable(outVarName.c_str());
           atts.SetVariableDimension(1);
           atts.SetVariableType(AVT_SCALAR_VAR);
           atts.SetCentering(AVT_NODECENT);
           // you might want to set up extents too
       }
   }
}
