/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include <PyRectilinearProject2DAttributes.h>
#include <ObserverToCallback.h>
#include <stdio.h>
#include <snprintf.h>

// ****************************************************************************
// Module: PyRectilinearProject2DAttributes
//
// Purpose: 
//   Attributes for RectilinearProject2D
//
// Note:       Autogenerated by xml2python. Do not modify by hand!
//
// Programmer: xml2python
// Creation:   omitted
//
// ****************************************************************************

//
// This struct contains the Python type information and a RectilinearProject2DAttributes.
//
struct RectilinearProject2DAttributesObject
{
    PyObject_HEAD
    RectilinearProject2DAttributes *data;
    bool        owns;
    PyObject   *parent;
};

//
// Internal prototypes
//
static PyObject *NewRectilinearProject2DAttributes(int);

std::string
PyRectilinearProject2DAttributes_ToString(const RectilinearProject2DAttributes *atts, const char *prefix)
{
    std::string str;
    char tmpStr[1000];

    const char *reductionOperator_names = "Sum, Average";
    switch (atts->GetReductionOperator())
    {
      case RectilinearProject2DAttributes::Sum:
          SNPRINTF(tmpStr, 1000, "%sreductionOperator = %sSum  # %s\n", prefix, prefix, reductionOperator_names);
          str += tmpStr;
          break;
      case RectilinearProject2DAttributes::Average:
          SNPRINTF(tmpStr, 1000, "%sreductionOperator = %sAverage  # %s\n", prefix, prefix, reductionOperator_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    if(atts->GetMapGrid())
        SNPRINTF(tmpStr, 1000, "%smapGrid = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%smapGrid = 0\n", prefix);
    str += tmpStr;
    return str;
}

static PyObject *
RectilinearProject2DAttributes_Notify(PyObject *self, PyObject *args)
{
    RectilinearProject2DAttributesObject *obj = (RectilinearProject2DAttributesObject *)self;
    obj->data->Notify();
    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
RectilinearProject2DAttributes_SetReductionOperator(PyObject *self, PyObject *args)
{
    RectilinearProject2DAttributesObject *obj = (RectilinearProject2DAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the reductionOperator in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetReductionOperator(RectilinearProject2DAttributes::ReductionOperator(ival));
    else
    {
        fprintf(stderr, "An invalid reductionOperator value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Sum, Average.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
RectilinearProject2DAttributes_GetReductionOperator(PyObject *self, PyObject *args)
{
    RectilinearProject2DAttributesObject *obj = (RectilinearProject2DAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetReductionOperator()));
    return retval;
}

/*static*/ PyObject *
RectilinearProject2DAttributes_SetMapGrid(PyObject *self, PyObject *args)
{
    RectilinearProject2DAttributesObject *obj = (RectilinearProject2DAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the mapGrid in the object.
    obj->data->SetMapGrid(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
RectilinearProject2DAttributes_GetMapGrid(PyObject *self, PyObject *args)
{
    RectilinearProject2DAttributesObject *obj = (RectilinearProject2DAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetMapGrid()?1L:0L);
    return retval;
}



PyMethodDef PyRectilinearProject2DAttributes_methods[RECTILINEARPROJECT2DATTRIBUTES_NMETH] = {
    {"Notify", RectilinearProject2DAttributes_Notify, METH_VARARGS},
    {"SetReductionOperator", RectilinearProject2DAttributes_SetReductionOperator, METH_VARARGS},
    {"GetReductionOperator", RectilinearProject2DAttributes_GetReductionOperator, METH_VARARGS},
    {"SetMapGrid", RectilinearProject2DAttributes_SetMapGrid, METH_VARARGS},
    {"GetMapGrid", RectilinearProject2DAttributes_GetMapGrid, METH_VARARGS},
    {NULL, NULL}
};

//
// Type functions
//

static void
RectilinearProject2DAttributes_dealloc(PyObject *v)
{
   RectilinearProject2DAttributesObject *obj = (RectilinearProject2DAttributesObject *)v;
   if(obj->parent != 0)
       Py_DECREF(obj->parent);
   if(obj->owns)
       delete obj->data;
}

static int
RectilinearProject2DAttributes_compare(PyObject *v, PyObject *w)
{
    RectilinearProject2DAttributes *a = ((RectilinearProject2DAttributesObject *)v)->data;
    RectilinearProject2DAttributes *b = ((RectilinearProject2DAttributesObject *)w)->data;
    return (*a == *b) ? 0 : -1;
}

PyObject *
PyRectilinearProject2DAttributes_getattr(PyObject *self, char *name)
{
    if(strcmp(name, "reductionOperator") == 0)
        return RectilinearProject2DAttributes_GetReductionOperator(self, NULL);
    if(strcmp(name, "Sum") == 0)
        return PyInt_FromLong(long(RectilinearProject2DAttributes::Sum));
    if(strcmp(name, "Average") == 0)
        return PyInt_FromLong(long(RectilinearProject2DAttributes::Average));

    if(strcmp(name, "mapGrid") == 0)
        return RectilinearProject2DAttributes_GetMapGrid(self, NULL);

    return Py_FindMethod(PyRectilinearProject2DAttributes_methods, self, name);
}

int
PyRectilinearProject2DAttributes_setattr(PyObject *self, char *name, PyObject *args)
{
    // Create a tuple to contain the arguments since all of the Set
    // functions expect a tuple.
    PyObject *tuple = PyTuple_New(1);
    PyTuple_SET_ITEM(tuple, 0, args);
    Py_INCREF(args);
    PyObject *obj = NULL;

    if(strcmp(name, "reductionOperator") == 0)
        obj = RectilinearProject2DAttributes_SetReductionOperator(self, tuple);
    else if(strcmp(name, "mapGrid") == 0)
        obj = RectilinearProject2DAttributes_SetMapGrid(self, tuple);

    if(obj != NULL)
        Py_DECREF(obj);

    Py_DECREF(tuple);
    if( obj == NULL)
        PyErr_Format(PyExc_RuntimeError, "Unable to set unknown attribute: '%s'", name);
    return (obj != NULL) ? 0 : -1;
}

static int
RectilinearProject2DAttributes_print(PyObject *v, FILE *fp, int flags)
{
    RectilinearProject2DAttributesObject *obj = (RectilinearProject2DAttributesObject *)v;
    fprintf(fp, "%s", PyRectilinearProject2DAttributes_ToString(obj->data, "").c_str());
    return 0;
}

PyObject *
RectilinearProject2DAttributes_str(PyObject *v)
{
    RectilinearProject2DAttributesObject *obj = (RectilinearProject2DAttributesObject *)v;
    return PyString_FromString(PyRectilinearProject2DAttributes_ToString(obj->data,"").c_str());
}

//
// The doc string for the class.
//
#if PY_MAJOR_VERSION > 2 || (PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION >= 5)
static const char *RectilinearProject2DAttributes_Purpose = "Attributes for RectilinearProject2D";
#else
static char *RectilinearProject2DAttributes_Purpose = "Attributes for RectilinearProject2D";
#endif

//
// The type description structure
//
static PyTypeObject RectilinearProject2DAttributesType =
{
    //
    // Type header
    //
    PyObject_HEAD_INIT(&PyType_Type)
    0,                                   // ob_size
    "RectilinearProject2DAttributes",                    // tp_name
    sizeof(RectilinearProject2DAttributesObject),        // tp_basicsize
    0,                                   // tp_itemsize
    //
    // Standard methods
    //
    (destructor)RectilinearProject2DAttributes_dealloc,  // tp_dealloc
    (printfunc)RectilinearProject2DAttributes_print,     // tp_print
    (getattrfunc)PyRectilinearProject2DAttributes_getattr, // tp_getattr
    (setattrfunc)PyRectilinearProject2DAttributes_setattr, // tp_setattr
    (cmpfunc)RectilinearProject2DAttributes_compare,     // tp_compare
    (reprfunc)0,                         // tp_repr
    //
    // Type categories
    //
    0,                                   // tp_as_number
    0,                                   // tp_as_sequence
    0,                                   // tp_as_mapping
    //
    // More methods
    //
    0,                                   // tp_hash
    0,                                   // tp_call
    (reprfunc)RectilinearProject2DAttributes_str,        // tp_str
    0,                                   // tp_getattro
    0,                                   // tp_setattro
    0,                                   // tp_as_buffer
    Py_TPFLAGS_CHECKTYPES,               // tp_flags
    RectilinearProject2DAttributes_Purpose,              // tp_doc
    0,                                   // tp_traverse
    0,                                   // tp_clear
    0,                                   // tp_richcompare
    0                                    // tp_weaklistoffset
};

//
// Helper functions for object allocation.
//

static RectilinearProject2DAttributes *defaultAtts = 0;
static RectilinearProject2DAttributes *currentAtts = 0;

static PyObject *
NewRectilinearProject2DAttributes(int useCurrent)
{
    RectilinearProject2DAttributesObject *newObject;
    newObject = PyObject_NEW(RectilinearProject2DAttributesObject, &RectilinearProject2DAttributesType);
    if(newObject == NULL)
        return NULL;
    if(useCurrent && currentAtts != 0)
        newObject->data = new RectilinearProject2DAttributes(*currentAtts);
    else if(defaultAtts != 0)
        newObject->data = new RectilinearProject2DAttributes(*defaultAtts);
    else
        newObject->data = new RectilinearProject2DAttributes;
    newObject->owns = true;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

static PyObject *
WrapRectilinearProject2DAttributes(const RectilinearProject2DAttributes *attr)
{
    RectilinearProject2DAttributesObject *newObject;
    newObject = PyObject_NEW(RectilinearProject2DAttributesObject, &RectilinearProject2DAttributesType);
    if(newObject == NULL)
        return NULL;
    newObject->data = (RectilinearProject2DAttributes *)attr;
    newObject->owns = false;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

///////////////////////////////////////////////////////////////////////////////
//
// Interface that is exposed to the VisIt module.
//
///////////////////////////////////////////////////////////////////////////////

PyObject *
RectilinearProject2DAttributes_new(PyObject *self, PyObject *args)
{
    int useCurrent = 0;
    if (!PyArg_ParseTuple(args, "i", &useCurrent))
    {
        if (!PyArg_ParseTuple(args, ""))
            return NULL;
        else
            PyErr_Clear();
    }

    return (PyObject *)NewRectilinearProject2DAttributes(useCurrent);
}

//
// Plugin method table. These methods are added to the visitmodule's methods.
//
static PyMethodDef RectilinearProject2DAttributesMethods[] = {
    {"RectilinearProject2DAttributes", RectilinearProject2DAttributes_new, METH_VARARGS},
    {NULL,      NULL}        /* Sentinel */
};

static Observer *RectilinearProject2DAttributesObserver = 0;

std::string
PyRectilinearProject2DAttributes_GetLogString()
{
    std::string s("RectilinearProject2DAtts = RectilinearProject2DAttributes()\n");
    if(currentAtts != 0)
        s += PyRectilinearProject2DAttributes_ToString(currentAtts, "RectilinearProject2DAtts.");
    return s;
}

static void
PyRectilinearProject2DAttributes_CallLogRoutine(Subject *subj, void *data)
{
    typedef void (*logCallback)(const std::string &);
    logCallback cb = (logCallback)data;

    if(cb != 0)
    {
        std::string s("RectilinearProject2DAtts = RectilinearProject2DAttributes()\n");
        s += PyRectilinearProject2DAttributes_ToString(currentAtts, "RectilinearProject2DAtts.");
        cb(s);
    }
}

void
PyRectilinearProject2DAttributes_StartUp(RectilinearProject2DAttributes *subj, void *data)
{
    if(subj == 0)
        return;

    currentAtts = subj;
    PyRectilinearProject2DAttributes_SetDefaults(subj);

    //
    // Create the observer that will be notified when the attributes change.
    //
    if(RectilinearProject2DAttributesObserver == 0)
    {
        RectilinearProject2DAttributesObserver = new ObserverToCallback(subj,
            PyRectilinearProject2DAttributes_CallLogRoutine, (void *)data);
    }

}

void
PyRectilinearProject2DAttributes_CloseDown()
{
    delete defaultAtts;
    defaultAtts = 0;
    delete RectilinearProject2DAttributesObserver;
    RectilinearProject2DAttributesObserver = 0;
}

PyMethodDef *
PyRectilinearProject2DAttributes_GetMethodTable(int *nMethods)
{
    *nMethods = 1;
    return RectilinearProject2DAttributesMethods;
}

bool
PyRectilinearProject2DAttributes_Check(PyObject *obj)
{
    return (obj->ob_type == &RectilinearProject2DAttributesType);
}

RectilinearProject2DAttributes *
PyRectilinearProject2DAttributes_FromPyObject(PyObject *obj)
{
    RectilinearProject2DAttributesObject *obj2 = (RectilinearProject2DAttributesObject *)obj;
    return obj2->data;
}

PyObject *
PyRectilinearProject2DAttributes_New()
{
    return NewRectilinearProject2DAttributes(0);
}

PyObject *
PyRectilinearProject2DAttributes_Wrap(const RectilinearProject2DAttributes *attr)
{
    return WrapRectilinearProject2DAttributes(attr);
}

void
PyRectilinearProject2DAttributes_SetParent(PyObject *obj, PyObject *parent)
{
    RectilinearProject2DAttributesObject *obj2 = (RectilinearProject2DAttributesObject *)obj;
    obj2->parent = parent;
}

void
PyRectilinearProject2DAttributes_SetDefaults(const RectilinearProject2DAttributes *atts)
{
    if(defaultAtts)
        delete defaultAtts;

    defaultAtts = new RectilinearProject2DAttributes(*atts);
}

