/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include "QvisRectilinearProject2DWindow.h"

#include <RectilinearProject2DAttributes.h>
#include <ViewerProxy.h>

#include <QCheckBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QSpinBox>
#include <QButtonGroup>
#include <QRadioButton>
#include <QvisColorTableButton.h>
#include <QvisOpacitySlider.h>
#include <QvisColorButton.h>
#include <QvisLineStyleWidget.h>
#include <QvisLineWidthWidget.h>
#include <QvisVariableButton.h>

#include <stdio.h>
#include <string>

using std::string;

// ****************************************************************************
// Method: QvisRectilinearProject2DWindow::QvisRectilinearProject2DWindow
//
// Purpose: 
//   Constructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisRectilinearProject2DWindow::QvisRectilinearProject2DWindow(const int type,
                         RectilinearProject2DAttributes *subj,
                         const QString &caption,
                         const QString &shortName,
                         QvisNotepadArea *notepad)
    : QvisOperatorWindow(type,subj, caption, shortName, notepad)
{
    atts = subj;
}


// ****************************************************************************
// Method: QvisRectilinearProject2DWindow::~QvisRectilinearProject2DWindow
//
// Purpose: 
//   Destructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisRectilinearProject2DWindow::~QvisRectilinearProject2DWindow()
{
}


// ****************************************************************************
// Method: QvisRectilinearProject2DWindow::CreateWindowContents
//
// Purpose: 
//   Creates the widgets for the window.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisRectilinearProject2DWindow::CreateWindowContents()
{
    QGridLayout *mainLayout = new QGridLayout(0);
    topLayout->addLayout(mainLayout);

    reductionOperatorLabel = new QLabel(tr("Reduction Operator"), central);
    mainLayout->addWidget(reductionOperatorLabel,0,0);
    reductionOperator = new QWidget(central);
    reductionOperatorButtonGroup= new QButtonGroup(reductionOperator);
    QHBoxLayout *reductionOperatorLayout = new QHBoxLayout(reductionOperator);
    reductionOperatorLayout->setMargin(0);
    reductionOperatorLayout->setSpacing(10);
    QRadioButton *reductionOperatorReductionOperatorSum = new QRadioButton(tr("Sum"), reductionOperator);
    reductionOperatorButtonGroup->addButton(reductionOperatorReductionOperatorSum,0);
    reductionOperatorLayout->addWidget(reductionOperatorReductionOperatorSum);
    QRadioButton *reductionOperatorReductionOperatorAverage = new QRadioButton(tr("Average"), reductionOperator);
    reductionOperatorButtonGroup->addButton(reductionOperatorReductionOperatorAverage,1);
    reductionOperatorLayout->addWidget(reductionOperatorReductionOperatorAverage);
    connect(reductionOperatorButtonGroup, SIGNAL(buttonClicked(int)),
            this, SLOT(reductionOperatorChanged(int)));
    mainLayout->addWidget(reductionOperator, 0,1);

    mapGrid = new QCheckBox(tr("Map Grid"), central);
    connect(mapGrid, SIGNAL(toggled(bool)),
            this, SLOT(mapGridChanged(bool)));
    mainLayout->addWidget(mapGrid, 1,0);

}


// ****************************************************************************
// Method: QvisRectilinearProject2DWindow::UpdateWindow
//
// Purpose: 
//   Updates the widgets in the window when the subject changes.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisRectilinearProject2DWindow::UpdateWindow(bool doAll)
{

    for(int i = 0; i < atts->NumAttributes(); ++i)
    {
        if(!doAll)
        {
            if(!atts->IsSelected(i))
            {
                continue;
            }
        }

        switch(i)
        {
          case RectilinearProject2DAttributes::ID_reductionOperator:
            reductionOperatorButtonGroup->blockSignals(true);
            if(reductionOperatorButtonGroup->button((int)atts->GetReductionOperator()) != 0)
                reductionOperatorButtonGroup->button((int)atts->GetReductionOperator())->setChecked(true);
            reductionOperatorButtonGroup->blockSignals(false);
            break;
          case RectilinearProject2DAttributes::ID_mapGrid:
            mapGrid->blockSignals(true);
            mapGrid->setChecked(atts->GetMapGrid());
            mapGrid->blockSignals(false);
            break;
        }
    }
}


// ****************************************************************************
// Method: QvisRectilinearProject2DWindow::GetCurrentValues
//
// Purpose: 
//   Gets values from certain widgets and stores them in the subject.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisRectilinearProject2DWindow::GetCurrentValues(int which_widget)
{
}


//
// Qt Slot functions
//


void
QvisRectilinearProject2DWindow::reductionOperatorChanged(int val)
{
    if(val != atts->GetReductionOperator())
    {
        atts->SetReductionOperator(RectilinearProject2DAttributes::ReductionOperator(val));
        SetUpdate(false);
        Apply();
    }
}


void
QvisRectilinearProject2DWindow::mapGridChanged(bool val)
{
    atts->SetMapGrid(val);
    SetUpdate(false);
    Apply();
}


