/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include <RectilinearProject2DAttributes.h>
#include <DataNode.h>

//
// Enum conversion methods for RectilinearProject2DAttributes::ReductionOperator
//

static const char *ReductionOperator_strings[] = {
"Sum", "Average"};

std::string
RectilinearProject2DAttributes::ReductionOperator_ToString(RectilinearProject2DAttributes::ReductionOperator t)
{
    int index = int(t);
    if(index < 0 || index >= 2) index = 0;
    return ReductionOperator_strings[index];
}

std::string
RectilinearProject2DAttributes::ReductionOperator_ToString(int t)
{
    int index = (t < 0 || t >= 2) ? 0 : t;
    return ReductionOperator_strings[index];
}

bool
RectilinearProject2DAttributes::ReductionOperator_FromString(const std::string &s, RectilinearProject2DAttributes::ReductionOperator &val)
{
    val = RectilinearProject2DAttributes::Sum;
    for(int i = 0; i < 2; ++i)
    {
        if(s == ReductionOperator_strings[i])
        {
            val = (ReductionOperator)i;
            return true;
        }
    }
    return false;
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::RectilinearProject2DAttributes
//
// Purpose: 
//   Init utility for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void RectilinearProject2DAttributes::Init()
{
    reductionOperator = Sum;
    mapGrid = true;

    RectilinearProject2DAttributes::SelectAll();
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::RectilinearProject2DAttributes
//
// Purpose: 
//   Copy utility for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void RectilinearProject2DAttributes::Copy(const RectilinearProject2DAttributes &obj)
{
    reductionOperator = obj.reductionOperator;
    mapGrid = obj.mapGrid;

    RectilinearProject2DAttributes::SelectAll();
}

// Type map format string
const char *RectilinearProject2DAttributes::TypeMapFormatString = RECTILINEARPROJECT2DATTRIBUTES_TMFS;
const AttributeGroup::private_tmfs_t RectilinearProject2DAttributes::TmfsStruct = {RECTILINEARPROJECT2DATTRIBUTES_TMFS};


// ****************************************************************************
// Method: RectilinearProject2DAttributes::RectilinearProject2DAttributes
//
// Purpose: 
//   Default constructor for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

RectilinearProject2DAttributes::RectilinearProject2DAttributes() : 
    AttributeSubject(RectilinearProject2DAttributes::TypeMapFormatString)
{
    RectilinearProject2DAttributes::Init();
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::RectilinearProject2DAttributes
//
// Purpose: 
//   Constructor for the derived classes of RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

RectilinearProject2DAttributes::RectilinearProject2DAttributes(private_tmfs_t tmfs) : 
    AttributeSubject(tmfs.tmfs)
{
    RectilinearProject2DAttributes::Init();
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::RectilinearProject2DAttributes
//
// Purpose: 
//   Copy constructor for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

RectilinearProject2DAttributes::RectilinearProject2DAttributes(const RectilinearProject2DAttributes &obj) : 
    AttributeSubject(RectilinearProject2DAttributes::TypeMapFormatString)
{
    RectilinearProject2DAttributes::Copy(obj);
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::RectilinearProject2DAttributes
//
// Purpose: 
//   Copy constructor for derived classes of the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

RectilinearProject2DAttributes::RectilinearProject2DAttributes(const RectilinearProject2DAttributes &obj, private_tmfs_t tmfs) : 
    AttributeSubject(tmfs.tmfs)
{
    RectilinearProject2DAttributes::Copy(obj);
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::~RectilinearProject2DAttributes
//
// Purpose: 
//   Destructor for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

RectilinearProject2DAttributes::~RectilinearProject2DAttributes()
{
    // nothing here
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::operator = 
//
// Purpose: 
//   Assignment operator for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

RectilinearProject2DAttributes& 
RectilinearProject2DAttributes::operator = (const RectilinearProject2DAttributes &obj)
{
    if (this == &obj) return *this;

    RectilinearProject2DAttributes::Copy(obj);

    return *this;
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::operator == 
//
// Purpose: 
//   Comparison operator == for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
RectilinearProject2DAttributes::operator == (const RectilinearProject2DAttributes &obj) const
{
    // Create the return value
    return ((reductionOperator == obj.reductionOperator) &&
            (mapGrid == obj.mapGrid));
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::operator != 
//
// Purpose: 
//   Comparison operator != for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
RectilinearProject2DAttributes::operator != (const RectilinearProject2DAttributes &obj) const
{
    return !(this->operator == (obj));
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::TypeName
//
// Purpose: 
//   Type name method for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

const std::string
RectilinearProject2DAttributes::TypeName() const
{
    return "RectilinearProject2DAttributes";
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::CopyAttributes
//
// Purpose: 
//   CopyAttributes method for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
RectilinearProject2DAttributes::CopyAttributes(const AttributeGroup *atts)
{
    if(TypeName() != atts->TypeName())
        return false;

    // Call assignment operator.
    const RectilinearProject2DAttributes *tmp = (const RectilinearProject2DAttributes *)atts;
    *this = *tmp;

    return true;
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::CreateCompatible
//
// Purpose: 
//   CreateCompatible method for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

AttributeSubject *
RectilinearProject2DAttributes::CreateCompatible(const std::string &tname) const
{
    AttributeSubject *retval = 0;
    if(TypeName() == tname)
        retval = new RectilinearProject2DAttributes(*this);
    // Other cases could go here too. 

    return retval;
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::NewInstance
//
// Purpose: 
//   NewInstance method for the RectilinearProject2DAttributes class.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

AttributeSubject *
RectilinearProject2DAttributes::NewInstance(bool copy) const
{
    AttributeSubject *retval = 0;
    if(copy)
        retval = new RectilinearProject2DAttributes(*this);
    else
        retval = new RectilinearProject2DAttributes;

    return retval;
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::SelectAll
//
// Purpose: 
//   Selects all attributes.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
RectilinearProject2DAttributes::SelectAll()
{
    Select(ID_reductionOperator, (void *)&reductionOperator);
    Select(ID_mapGrid,           (void *)&mapGrid);
}

///////////////////////////////////////////////////////////////////////////////
// Persistence methods
///////////////////////////////////////////////////////////////////////////////

// ****************************************************************************
// Method: RectilinearProject2DAttributes::CreateNode
//
// Purpose: 
//   This method creates a DataNode representation of the object so it can be saved to a config file.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
RectilinearProject2DAttributes::CreateNode(DataNode *parentNode, bool completeSave, bool forceAdd)
{
    if(parentNode == 0)
        return false;

    RectilinearProject2DAttributes defaultObject;
    bool addToParent = false;
    // Create a node for RectilinearProject2DAttributes.
    DataNode *node = new DataNode("RectilinearProject2DAttributes");

    if(completeSave || !FieldsEqual(ID_reductionOperator, &defaultObject))
    {
        addToParent = true;
        node->AddNode(new DataNode("reductionOperator", ReductionOperator_ToString(reductionOperator)));
    }

    if(completeSave || !FieldsEqual(ID_mapGrid, &defaultObject))
    {
        addToParent = true;
        node->AddNode(new DataNode("mapGrid", mapGrid));
    }


    // Add the node to the parent node.
    if(addToParent || forceAdd)
        parentNode->AddNode(node);
    else
        delete node;

    return (addToParent || forceAdd);
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::SetFromNode
//
// Purpose: 
//   This method sets attributes in this object from values in a DataNode representation of the object.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
RectilinearProject2DAttributes::SetFromNode(DataNode *parentNode)
{
    if(parentNode == 0)
        return;

    DataNode *searchNode = parentNode->GetNode("RectilinearProject2DAttributes");
    if(searchNode == 0)
        return;

    DataNode *node;
    if((node = searchNode->GetNode("reductionOperator")) != 0)
    {
        // Allow enums to be int or string in the config file
        if(node->GetNodeType() == INT_NODE)
        {
            int ival = node->AsInt();
            if(ival >= 0 && ival < 2)
                SetReductionOperator(ReductionOperator(ival));
        }
        else if(node->GetNodeType() == STRING_NODE)
        {
            ReductionOperator value;
            if(ReductionOperator_FromString(node->AsString(), value))
                SetReductionOperator(value);
        }
    }
    if((node = searchNode->GetNode("mapGrid")) != 0)
        SetMapGrid(node->AsBool());
}

///////////////////////////////////////////////////////////////////////////////
// Set property methods
///////////////////////////////////////////////////////////////////////////////

void
RectilinearProject2DAttributes::SetReductionOperator(RectilinearProject2DAttributes::ReductionOperator reductionOperator_)
{
    reductionOperator = reductionOperator_;
    Select(ID_reductionOperator, (void *)&reductionOperator);
}

void
RectilinearProject2DAttributes::SetMapGrid(bool mapGrid_)
{
    mapGrid = mapGrid_;
    Select(ID_mapGrid, (void *)&mapGrid);
}

///////////////////////////////////////////////////////////////////////////////
// Get property methods
///////////////////////////////////////////////////////////////////////////////

RectilinearProject2DAttributes::ReductionOperator
RectilinearProject2DAttributes::GetReductionOperator() const
{
    return ReductionOperator(reductionOperator);
}

bool
RectilinearProject2DAttributes::GetMapGrid() const
{
    return mapGrid;
}

///////////////////////////////////////////////////////////////////////////////
// Keyframing methods
///////////////////////////////////////////////////////////////////////////////

// ****************************************************************************
// Method: RectilinearProject2DAttributes::GetFieldName
//
// Purpose: 
//   This method returns the name of a field given its index.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

std::string
RectilinearProject2DAttributes::GetFieldName(int index) const
{
    switch (index)
    {
    case ID_reductionOperator: return "reductionOperator";
    case ID_mapGrid:           return "mapGrid";
    default:  return "invalid index";
    }
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::GetFieldType
//
// Purpose: 
//   This method returns the type of a field given its index.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

AttributeGroup::FieldType
RectilinearProject2DAttributes::GetFieldType(int index) const
{
    switch (index)
    {
    case ID_reductionOperator: return FieldType_enum;
    case ID_mapGrid:           return FieldType_bool;
    default:  return FieldType_unknown;
    }
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::GetFieldTypeName
//
// Purpose: 
//   This method returns the name of a field type given its index.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

std::string
RectilinearProject2DAttributes::GetFieldTypeName(int index) const
{
    switch (index)
    {
    case ID_reductionOperator: return "enum";
    case ID_mapGrid:           return "bool";
    default:  return "invalid index";
    }
}

// ****************************************************************************
// Method: RectilinearProject2DAttributes::FieldsEqual
//
// Purpose: 
//   This method compares two fields and return true if they are equal.
//
// Note:       Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

bool
RectilinearProject2DAttributes::FieldsEqual(int index_, const AttributeGroup *rhs) const
{
    const RectilinearProject2DAttributes &obj = *((const RectilinearProject2DAttributes*)rhs);
    bool retval = false;
    switch (index_)
    {
    case ID_reductionOperator:
        {  // new scope
        retval = (reductionOperator == obj.reductionOperator);
        }
        break;
    case ID_mapGrid:
        {  // new scope
        retval = (mapGrid == obj.mapGrid);
        }
        break;
    default: retval = false;
    }

    return retval;
}

///////////////////////////////////////////////////////////////////////////////
// User-defined methods.
///////////////////////////////////////////////////////////////////////////////

