/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include "QvisThreeSliceWindow.h"

#include <ThreeSliceAttributes.h>
#include <ViewerProxy.h>

#include <QCheckBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QSpinBox>
#include <QButtonGroup>
#include <QRadioButton>
#include <QvisColorTableButton.h>
#include <QvisOpacitySlider.h>
#include <QvisColorButton.h>
#include <QvisLineStyleWidget.h>
#include <QvisLineWidthWidget.h>
#include <QvisVariableButton.h>

#include <stdio.h>
#include <string>

using std::string;

// ****************************************************************************
// Method: QvisThreeSliceWindow::QvisThreeSliceWindow
//
// Purpose: 
//   Constructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisThreeSliceWindow::QvisThreeSliceWindow(const int type,
                         ThreeSliceAttributes *subj,
                         const QString &caption,
                         const QString &shortName,
                         QvisNotepadArea *notepad)
    : QvisOperatorWindow(type,subj, caption, shortName, notepad)
{
    atts = subj;
}


// ****************************************************************************
// Method: QvisThreeSliceWindow::~QvisThreeSliceWindow
//
// Purpose: 
//   Destructor
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

QvisThreeSliceWindow::~QvisThreeSliceWindow()
{
}


// ****************************************************************************
// Method: QvisThreeSliceWindow::CreateWindowContents
//
// Purpose: 
//   Creates the widgets for the window.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisThreeSliceWindow::CreateWindowContents()
{
    QGridLayout *mainLayout = new QGridLayout(0);
    topLayout->addLayout(mainLayout);

    xLabel = new QLabel(tr("X"), central);
    mainLayout->addWidget(xLabel,0,0);
    x = new QLineEdit(central);
    connect(x, SIGNAL(returnPressed()),
            this, SLOT(xProcessText()));
    mainLayout->addWidget(x, 0,1);

    yLabel = new QLabel(tr("Y"), central);
    mainLayout->addWidget(yLabel,1,0);
    y = new QLineEdit(central);
    connect(y, SIGNAL(returnPressed()),
            this, SLOT(yProcessText()));
    mainLayout->addWidget(y, 1,1);

    zLabel = new QLabel(tr("Z"), central);
    mainLayout->addWidget(zLabel,2,0);
    z = new QLineEdit(central);
    connect(z, SIGNAL(returnPressed()),
            this, SLOT(zProcessText()));
    mainLayout->addWidget(z, 2,1);

    interactive = new QCheckBox(tr("Interactive"), central);
    connect(interactive, SIGNAL(toggled(bool)),
            this, SLOT(interactiveChanged(bool)));
    mainLayout->addWidget(interactive, 3,0);

}


// ****************************************************************************
// Method: QvisThreeSliceWindow::UpdateWindow
//
// Purpose: 
//   Updates the widgets in the window when the subject changes.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisThreeSliceWindow::UpdateWindow(bool doAll)
{

    for(int i = 0; i < atts->NumAttributes(); ++i)
    {
        if(!doAll)
        {
            if(!atts->IsSelected(i))
            {
                continue;
            }
        }

        switch(i)
        {
          case ThreeSliceAttributes::ID_x:
            x->setText(FloatToQString(atts->GetX()));
            break;
          case ThreeSliceAttributes::ID_y:
            y->setText(FloatToQString(atts->GetY()));
            break;
          case ThreeSliceAttributes::ID_z:
            z->setText(FloatToQString(atts->GetZ()));
            break;
          case ThreeSliceAttributes::ID_interactive:
            interactive->blockSignals(true);
            interactive->setChecked(atts->GetInteractive());
            interactive->blockSignals(false);
            break;
        }
    }
}


// ****************************************************************************
// Method: QvisThreeSliceWindow::GetCurrentValues
//
// Purpose: 
//   Gets values from certain widgets and stores them in the subject.
//
// Note:       Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

void
QvisThreeSliceWindow::GetCurrentValues(int which_widget)
{
    bool doAll = (which_widget == -1);

    // Do x
    if(which_widget == ThreeSliceAttributes::ID_x || doAll)
    {
        float val;
        if(LineEditGetFloat(x, val))
            atts->SetX(val);
        else
        {
            ResettingError(tr("x"),
                FloatToQString(atts->GetX()));
            atts->SetX(atts->GetX());
        }
    }

    // Do y
    if(which_widget == ThreeSliceAttributes::ID_y || doAll)
    {
        float val;
        if(LineEditGetFloat(y, val))
            atts->SetY(val);
        else
        {
            ResettingError(tr("y"),
                FloatToQString(atts->GetY()));
            atts->SetY(atts->GetY());
        }
    }

    // Do z
    if(which_widget == ThreeSliceAttributes::ID_z || doAll)
    {
        float val;
        if(LineEditGetFloat(z, val))
            atts->SetZ(val);
        else
        {
            ResettingError(tr("z"),
                FloatToQString(atts->GetZ()));
            atts->SetZ(atts->GetZ());
        }
    }

}


//
// Qt Slot functions
//


void
QvisThreeSliceWindow::xProcessText()
{
    GetCurrentValues(ThreeSliceAttributes::ID_x);
    Apply();
}


void
QvisThreeSliceWindow::yProcessText()
{
    GetCurrentValues(ThreeSliceAttributes::ID_y);
    Apply();
}


void
QvisThreeSliceWindow::zProcessText()
{
    GetCurrentValues(ThreeSliceAttributes::ID_z);
    Apply();
}


void
QvisThreeSliceWindow::interactiveChanged(bool val)
{
    atts->SetInteractive(val);
    SetUpdate(false);
    Apply();
}


