/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include <PyStreamlineAttributes.h>
#include <ObserverToCallback.h>
#include <stdio.h>
#include <snprintf.h>
#include <ColorAttribute.h>

// ****************************************************************************
// Module: PyStreamlineAttributes
//
// Purpose: 
//   Attributes for the Streamline plot
//
// Note:       Autogenerated by xml2python. Do not modify by hand!
//
// Programmer: xml2python
// Creation:   omitted
//
// ****************************************************************************

//
// This struct contains the Python type information and a StreamlineAttributes.
//
struct StreamlineAttributesObject
{
    PyObject_HEAD
    StreamlineAttributes *data;
    bool        owns;
    PyObject   *parent;
};

//
// Internal prototypes
//
static PyObject *NewStreamlineAttributes(int);

std::string
PyStreamlineAttributes_ToString(const StreamlineAttributes *atts, const char *prefix)
{
    std::string str;
    char tmpStr[1000];

    const char *sourceType_names = "SpecifiedPoint, SpecifiedPointList, SpecifiedLine, SpecifiedCircle, SpecifiedPlane, "
        "SpecifiedSphere, SpecifiedBox, Selection";
    switch (atts->GetSourceType())
    {
      case StreamlineAttributes::SpecifiedPoint:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sSpecifiedPoint  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::SpecifiedPointList:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sSpecifiedPointList  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::SpecifiedLine:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sSpecifiedLine  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::SpecifiedCircle:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sSpecifiedCircle  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::SpecifiedPlane:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sSpecifiedPlane  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::SpecifiedSphere:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sSpecifiedSphere  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::SpecifiedBox:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sSpecifiedBox  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Selection:
          SNPRINTF(tmpStr, 1000, "%ssourceType = %sSelection  # %s\n", prefix, prefix, sourceType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    {   const double *pointSource = atts->GetPointSource();
        SNPRINTF(tmpStr, 1000, "%spointSource = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", pointSource[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    {   const double *lineStart = atts->GetLineStart();
        SNPRINTF(tmpStr, 1000, "%slineStart = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", lineStart[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    {   const double *lineEnd = atts->GetLineEnd();
        SNPRINTF(tmpStr, 1000, "%slineEnd = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", lineEnd[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    {   const double *planeOrigin = atts->GetPlaneOrigin();
        SNPRINTF(tmpStr, 1000, "%splaneOrigin = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", planeOrigin[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    {   const double *planeNormal = atts->GetPlaneNormal();
        SNPRINTF(tmpStr, 1000, "%splaneNormal = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", planeNormal[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    {   const double *planeUpAxis = atts->GetPlaneUpAxis();
        SNPRINTF(tmpStr, 1000, "%splaneUpAxis = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", planeUpAxis[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    SNPRINTF(tmpStr, 1000, "%sradius = %g\n", prefix, atts->GetRadius());
    str += tmpStr;
    {   const double *sphereOrigin = atts->GetSphereOrigin();
        SNPRINTF(tmpStr, 1000, "%ssphereOrigin = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", sphereOrigin[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    {   const double *boxExtents = atts->GetBoxExtents();
        SNPRINTF(tmpStr, 1000, "%sboxExtents = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 6; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", boxExtents[i]);
            str += tmpStr;
            if(i < 5)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    if(atts->GetUseWholeBox())
        SNPRINTF(tmpStr, 1000, "%suseWholeBox = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%suseWholeBox = 0\n", prefix);
    str += tmpStr;
    {   const doubleVector &pointList = atts->GetPointList();
        SNPRINTF(tmpStr, 1000, "%spointList = (", prefix);
        str += tmpStr;
        for(size_t i = 0; i < pointList.size(); ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", pointList[i]);
            str += tmpStr;
            if(i < pointList.size() - 1)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    SNPRINTF(tmpStr, 1000, "%ssampleDensity0 = %d\n", prefix, atts->GetSampleDensity0());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%ssampleDensity1 = %d\n", prefix, atts->GetSampleDensity1());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%ssampleDensity2 = %d\n", prefix, atts->GetSampleDensity2());
    str += tmpStr;
    const char *coloringMethod_names = "Solid, ColorBySpeed, ColorByVorticity, ColorByLength, ColorByTime, "
        "ColorBySeedPointID, ColorByVariable, ColorByCorrelationDistance";
    switch (atts->GetColoringMethod())
    {
      case StreamlineAttributes::Solid:
          SNPRINTF(tmpStr, 1000, "%scoloringMethod = %sSolid  # %s\n", prefix, prefix, coloringMethod_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::ColorBySpeed:
          SNPRINTF(tmpStr, 1000, "%scoloringMethod = %sColorBySpeed  # %s\n", prefix, prefix, coloringMethod_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::ColorByVorticity:
          SNPRINTF(tmpStr, 1000, "%scoloringMethod = %sColorByVorticity  # %s\n", prefix, prefix, coloringMethod_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::ColorByLength:
          SNPRINTF(tmpStr, 1000, "%scoloringMethod = %sColorByLength  # %s\n", prefix, prefix, coloringMethod_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::ColorByTime:
          SNPRINTF(tmpStr, 1000, "%scoloringMethod = %sColorByTime  # %s\n", prefix, prefix, coloringMethod_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::ColorBySeedPointID:
          SNPRINTF(tmpStr, 1000, "%scoloringMethod = %sColorBySeedPointID  # %s\n", prefix, prefix, coloringMethod_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::ColorByVariable:
          SNPRINTF(tmpStr, 1000, "%scoloringMethod = %sColorByVariable  # %s\n", prefix, prefix, coloringMethod_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::ColorByCorrelationDistance:
          SNPRINTF(tmpStr, 1000, "%scoloringMethod = %sColorByCorrelationDistance  # %s\n", prefix, prefix, coloringMethod_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%scolorTableName = \"%s\"\n", prefix, atts->GetColorTableName().c_str());
    str += tmpStr;
    const unsigned char *singleColor = atts->GetSingleColor().GetColor();
    SNPRINTF(tmpStr, 1000, "%ssingleColor = (%d, %d, %d, %d)\n", prefix, int(singleColor[0]), int(singleColor[1]), int(singleColor[2]), int(singleColor[3]));
    str += tmpStr;
    if(atts->GetLegendFlag())
        SNPRINTF(tmpStr, 1000, "%slegendFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%slegendFlag = 0\n", prefix);
    str += tmpStr;
    if(atts->GetLightingFlag())
        SNPRINTF(tmpStr, 1000, "%slightingFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%slightingFlag = 0\n", prefix);
    str += tmpStr;
    const char *integrationDirection_names = "Forward, Backward, Both";
    switch (atts->GetIntegrationDirection())
    {
      case StreamlineAttributes::Forward:
          SNPRINTF(tmpStr, 1000, "%sintegrationDirection = %sForward  # %s\n", prefix, prefix, integrationDirection_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Backward:
          SNPRINTF(tmpStr, 1000, "%sintegrationDirection = %sBackward  # %s\n", prefix, prefix, integrationDirection_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Both:
          SNPRINTF(tmpStr, 1000, "%sintegrationDirection = %sBoth  # %s\n", prefix, prefix, integrationDirection_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%smaxSteps = %d\n", prefix, atts->GetMaxSteps());
    str += tmpStr;
    if(atts->GetTerminateByDistance())
        SNPRINTF(tmpStr, 1000, "%sterminateByDistance = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sterminateByDistance = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%stermDistance = %g\n", prefix, atts->GetTermDistance());
    str += tmpStr;
    if(atts->GetTerminateByTime())
        SNPRINTF(tmpStr, 1000, "%sterminateByTime = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sterminateByTime = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%stermTime = %g\n", prefix, atts->GetTermTime());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%smaxStepLength = %g\n", prefix, atts->GetMaxStepLength());
    str += tmpStr;
    if(atts->GetLimitMaximumTimestep())
        SNPRINTF(tmpStr, 1000, "%slimitMaximumTimestep = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%slimitMaximumTimestep = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%smaxTimeStep = %g\n", prefix, atts->GetMaxTimeStep());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%srelTol = %g\n", prefix, atts->GetRelTol());
    str += tmpStr;
    const char *absTolSizeType_names = "Absolute, FractionOfBBox";
    switch (atts->GetAbsTolSizeType())
    {
      case StreamlineAttributes::Absolute:
          SNPRINTF(tmpStr, 1000, "%sabsTolSizeType = %sAbsolute  # %s\n", prefix, prefix, absTolSizeType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::FractionOfBBox:
          SNPRINTF(tmpStr, 1000, "%sabsTolSizeType = %sFractionOfBBox  # %s\n", prefix, prefix, absTolSizeType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sabsTolAbsolute = %g\n", prefix, atts->GetAbsTolAbsolute());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sabsTolBBox = %g\n", prefix, atts->GetAbsTolBBox());
    str += tmpStr;
    const char *fieldType_names = "Default, FlashField, M3DC12DField, M3DC13DField, Nek5000Field, "
        "NIMRODField";
    switch (atts->GetFieldType())
    {
      case StreamlineAttributes::Default:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sDefault  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::FlashField:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sFlashField  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::M3DC12DField:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sM3DC12DField  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::M3DC13DField:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sM3DC13DField  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Nek5000Field:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sNek5000Field  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::NIMRODField:
          SNPRINTF(tmpStr, 1000, "%sfieldType = %sNIMRODField  # %s\n", prefix, prefix, fieldType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sfieldConstant = %g\n", prefix, atts->GetFieldConstant());
    str += tmpStr;
    {   const double *velocitySource = atts->GetVelocitySource();
        SNPRINTF(tmpStr, 1000, "%svelocitySource = (", prefix);
        str += tmpStr;
        for(int i = 0; i < 3; ++i)
        {
            SNPRINTF(tmpStr, 1000, "%g", velocitySource[i]);
            str += tmpStr;
            if(i < 2)
            {
                SNPRINTF(tmpStr, 1000, ", ");
                str += tmpStr;
            }
        }
        SNPRINTF(tmpStr, 1000, ")\n");
        str += tmpStr;
    }
    const char *integrationType_names = "Euler, Leapfrog, DormandPrince, AdamsBashforth, RK4, "
        "M3DC12DIntegrator";
    switch (atts->GetIntegrationType())
    {
      case StreamlineAttributes::Euler:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sEuler  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Leapfrog:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sLeapfrog  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::DormandPrince:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sDormandPrince  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::AdamsBashforth:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sAdamsBashforth  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::RK4:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sRK4  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::M3DC12DIntegrator:
          SNPRINTF(tmpStr, 1000, "%sintegrationType = %sM3DC12DIntegrator  # %s\n", prefix, prefix, integrationType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *parallelizationAlgorithmType_names = "LoadOnDemand, ParallelStaticDomains, MasterSlave, VisItSelects";
    switch (atts->GetParallelizationAlgorithmType())
    {
      case StreamlineAttributes::LoadOnDemand:
          SNPRINTF(tmpStr, 1000, "%sparallelizationAlgorithmType = %sLoadOnDemand  # %s\n", prefix, prefix, parallelizationAlgorithmType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::ParallelStaticDomains:
          SNPRINTF(tmpStr, 1000, "%sparallelizationAlgorithmType = %sParallelStaticDomains  # %s\n", prefix, prefix, parallelizationAlgorithmType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::MasterSlave:
          SNPRINTF(tmpStr, 1000, "%sparallelizationAlgorithmType = %sMasterSlave  # %s\n", prefix, prefix, parallelizationAlgorithmType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::VisItSelects:
          SNPRINTF(tmpStr, 1000, "%sparallelizationAlgorithmType = %sVisItSelects  # %s\n", prefix, prefix, parallelizationAlgorithmType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%smaxProcessCount = %d\n", prefix, atts->GetMaxProcessCount());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%smaxDomainCacheSize = %d\n", prefix, atts->GetMaxDomainCacheSize());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sworkGroupSize = %d\n", prefix, atts->GetWorkGroupSize());
    str += tmpStr;
    if(atts->GetPathlines())
        SNPRINTF(tmpStr, 1000, "%spathlines = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%spathlines = 0\n", prefix);
    str += tmpStr;
    if(atts->GetPathlinesOverrideStartingTimeFlag())
        SNPRINTF(tmpStr, 1000, "%spathlinesOverrideStartingTimeFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%spathlinesOverrideStartingTimeFlag = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%spathlinesOverrideStartingTime = %g\n", prefix, atts->GetPathlinesOverrideStartingTime());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%spathlinesPeriod = %g\n", prefix, atts->GetPathlinesPeriod());
    str += tmpStr;
    const char *pathlinesCMFE_names = "CONN_CMFE, POS_CMFE";
    switch (atts->GetPathlinesCMFE())
    {
      case StreamlineAttributes::CONN_CMFE:
          SNPRINTF(tmpStr, 1000, "%spathlinesCMFE = %sCONN_CMFE  # %s\n", prefix, prefix, pathlinesCMFE_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::POS_CMFE:
          SNPRINTF(tmpStr, 1000, "%spathlinesCMFE = %sPOS_CMFE  # %s\n", prefix, prefix, pathlinesCMFE_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *coordinateSystem_names = "AsIs, CylindricalToCartesian, CartesianToCylindrical";
    switch (atts->GetCoordinateSystem())
    {
      case StreamlineAttributes::AsIs:
          SNPRINTF(tmpStr, 1000, "%scoordinateSystem = %sAsIs  # %s\n", prefix, prefix, coordinateSystem_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::CylindricalToCartesian:
          SNPRINTF(tmpStr, 1000, "%scoordinateSystem = %sCylindricalToCartesian  # %s\n", prefix, prefix, coordinateSystem_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::CartesianToCylindrical:
          SNPRINTF(tmpStr, 1000, "%scoordinateSystem = %sCartesianToCylindrical  # %s\n", prefix, prefix, coordinateSystem_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    if(atts->GetPhiScalingFlag())
        SNPRINTF(tmpStr, 1000, "%sphiScalingFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sphiScalingFlag = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sphiScaling = %g\n", prefix, atts->GetPhiScaling());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%scoloringVariable = \"%s\"\n", prefix, atts->GetColoringVariable().c_str());
    str += tmpStr;
    if(atts->GetLegendMinFlag())
        SNPRINTF(tmpStr, 1000, "%slegendMinFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%slegendMinFlag = 0\n", prefix);
    str += tmpStr;
    if(atts->GetLegendMaxFlag())
        SNPRINTF(tmpStr, 1000, "%slegendMaxFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%slegendMaxFlag = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%slegendMin = %g\n", prefix, atts->GetLegendMin());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%slegendMax = %g\n", prefix, atts->GetLegendMax());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sdisplayBegin = %g\n", prefix, atts->GetDisplayBegin());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sdisplayEnd = %g\n", prefix, atts->GetDisplayEnd());
    str += tmpStr;
    if(atts->GetDisplayBeginFlag())
        SNPRINTF(tmpStr, 1000, "%sdisplayBeginFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sdisplayBeginFlag = 0\n", prefix);
    str += tmpStr;
    if(atts->GetDisplayEndFlag())
        SNPRINTF(tmpStr, 1000, "%sdisplayEndFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sdisplayEndFlag = 0\n", prefix);
    str += tmpStr;
    const char *referenceTypeForDisplay_names = "Distance, Time, Step";
    switch (atts->GetReferenceTypeForDisplay())
    {
      case StreamlineAttributes::Distance:
          SNPRINTF(tmpStr, 1000, "%sreferenceTypeForDisplay = %sDistance  # %s\n", prefix, prefix, referenceTypeForDisplay_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Time:
          SNPRINTF(tmpStr, 1000, "%sreferenceTypeForDisplay = %sTime  # %s\n", prefix, prefix, referenceTypeForDisplay_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Step:
          SNPRINTF(tmpStr, 1000, "%sreferenceTypeForDisplay = %sStep  # %s\n", prefix, prefix, referenceTypeForDisplay_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *displayMethod_names = "Lines, Tubes, Ribbons";
    switch (atts->GetDisplayMethod())
    {
      case StreamlineAttributes::Lines:
          SNPRINTF(tmpStr, 1000, "%sdisplayMethod = %sLines  # %s\n", prefix, prefix, displayMethod_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Tubes:
          SNPRINTF(tmpStr, 1000, "%sdisplayMethod = %sTubes  # %s\n", prefix, prefix, displayMethod_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Ribbons:
          SNPRINTF(tmpStr, 1000, "%sdisplayMethod = %sRibbons  # %s\n", prefix, prefix, displayMethod_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *tubeSizeType_names = "Absolute, FractionOfBBox";
    switch (atts->GetTubeSizeType())
    {
      case StreamlineAttributes::Absolute:
          SNPRINTF(tmpStr, 1000, "%stubeSizeType = %sAbsolute  # %s\n", prefix, prefix, tubeSizeType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::FractionOfBBox:
          SNPRINTF(tmpStr, 1000, "%stubeSizeType = %sFractionOfBBox  # %s\n", prefix, prefix, tubeSizeType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%stubeRadiusAbsolute = %g\n", prefix, atts->GetTubeRadiusAbsolute());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%stubeRadiusBBox = %g\n", prefix, atts->GetTubeRadiusBBox());
    str += tmpStr;
    const char *ribbonWidthSizeType_names = "Absolute, FractionOfBBox";
    switch (atts->GetRibbonWidthSizeType())
    {
      case StreamlineAttributes::Absolute:
          SNPRINTF(tmpStr, 1000, "%sribbonWidthSizeType = %sAbsolute  # %s\n", prefix, prefix, ribbonWidthSizeType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::FractionOfBBox:
          SNPRINTF(tmpStr, 1000, "%sribbonWidthSizeType = %sFractionOfBBox  # %s\n", prefix, prefix, ribbonWidthSizeType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sribbonWidthAbsolute = %g\n", prefix, atts->GetRibbonWidthAbsolute());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sribbonWidthBBox = %g\n", prefix, atts->GetRibbonWidthBBox());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%slineWidth = %d\n", prefix, atts->GetLineWidth());
    str += tmpStr;
    if(atts->GetShowSeeds())
        SNPRINTF(tmpStr, 1000, "%sshowSeeds = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sshowSeeds = 0\n", prefix);
    str += tmpStr;
    const char *seedRadiusSizeType_names = "Absolute, FractionOfBBox";
    switch (atts->GetSeedRadiusSizeType())
    {
      case StreamlineAttributes::Absolute:
          SNPRINTF(tmpStr, 1000, "%sseedRadiusSizeType = %sAbsolute  # %s\n", prefix, prefix, seedRadiusSizeType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::FractionOfBBox:
          SNPRINTF(tmpStr, 1000, "%sseedRadiusSizeType = %sFractionOfBBox  # %s\n", prefix, prefix, seedRadiusSizeType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sseedRadiusAbsolute = %g\n", prefix, atts->GetSeedRadiusAbsolute());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sseedRadiusBBox = %g\n", prefix, atts->GetSeedRadiusBBox());
    str += tmpStr;
    if(atts->GetShowHeads())
        SNPRINTF(tmpStr, 1000, "%sshowHeads = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sshowHeads = 0\n", prefix);
    str += tmpStr;
    const char *headDisplayType_names = "Sphere, Cone";
    switch (atts->GetHeadDisplayType())
    {
      case StreamlineAttributes::Sphere:
          SNPRINTF(tmpStr, 1000, "%sheadDisplayType = %sSphere  # %s\n", prefix, prefix, headDisplayType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Cone:
          SNPRINTF(tmpStr, 1000, "%sheadDisplayType = %sCone  # %s\n", prefix, prefix, headDisplayType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    const char *headRadiusSizeType_names = "Absolute, FractionOfBBox";
    switch (atts->GetHeadRadiusSizeType())
    {
      case StreamlineAttributes::Absolute:
          SNPRINTF(tmpStr, 1000, "%sheadRadiusSizeType = %sAbsolute  # %s\n", prefix, prefix, headRadiusSizeType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::FractionOfBBox:
          SNPRINTF(tmpStr, 1000, "%sheadRadiusSizeType = %sFractionOfBBox  # %s\n", prefix, prefix, headRadiusSizeType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sheadRadiusAbsolute = %g\n", prefix, atts->GetHeadRadiusAbsolute());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sheadRadiusBBox = %g\n", prefix, atts->GetHeadRadiusBBox());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sheadHeightRatio = %g\n", prefix, atts->GetHeadHeightRatio());
    str += tmpStr;
    const char *opacityType_names = "FullyOpaque, Constant, Ramp, VariableRange";
    switch (atts->GetOpacityType())
    {
      case StreamlineAttributes::FullyOpaque:
          SNPRINTF(tmpStr, 1000, "%sopacityType = %sFullyOpaque  # %s\n", prefix, prefix, opacityType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Constant:
          SNPRINTF(tmpStr, 1000, "%sopacityType = %sConstant  # %s\n", prefix, prefix, opacityType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Ramp:
          SNPRINTF(tmpStr, 1000, "%sopacityType = %sRamp  # %s\n", prefix, prefix, opacityType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::VariableRange:
          SNPRINTF(tmpStr, 1000, "%sopacityType = %sVariableRange  # %s\n", prefix, prefix, opacityType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sopacityVariable = \"%s\"\n", prefix, atts->GetOpacityVariable().c_str());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sopacity = %g\n", prefix, atts->GetOpacity());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sopacityVarMin = %g\n", prefix, atts->GetOpacityVarMin());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%sopacityVarMax = %g\n", prefix, atts->GetOpacityVarMax());
    str += tmpStr;
    if(atts->GetOpacityVarMinFlag())
        SNPRINTF(tmpStr, 1000, "%sopacityVarMinFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sopacityVarMinFlag = 0\n", prefix);
    str += tmpStr;
    if(atts->GetOpacityVarMaxFlag())
        SNPRINTF(tmpStr, 1000, "%sopacityVarMaxFlag = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sopacityVarMaxFlag = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%stubeDisplayDensity = %d\n", prefix, atts->GetTubeDisplayDensity());
    str += tmpStr;
    const char *geomDisplayQuality_names = "Low, Medium, High, Super";
    switch (atts->GetGeomDisplayQuality())
    {
      case StreamlineAttributes::Low:
          SNPRINTF(tmpStr, 1000, "%sgeomDisplayQuality = %sLow  # %s\n", prefix, prefix, geomDisplayQuality_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Medium:
          SNPRINTF(tmpStr, 1000, "%sgeomDisplayQuality = %sMedium  # %s\n", prefix, prefix, geomDisplayQuality_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::High:
          SNPRINTF(tmpStr, 1000, "%sgeomDisplayQuality = %sHigh  # %s\n", prefix, prefix, geomDisplayQuality_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Super:
          SNPRINTF(tmpStr, 1000, "%sgeomDisplayQuality = %sSuper  # %s\n", prefix, prefix, geomDisplayQuality_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%ssampleDistance0 = %g\n", prefix, atts->GetSampleDistance0());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%ssampleDistance1 = %g\n", prefix, atts->GetSampleDistance1());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%ssampleDistance2 = %g\n", prefix, atts->GetSampleDistance2());
    str += tmpStr;
    if(atts->GetFillInterior())
        SNPRINTF(tmpStr, 1000, "%sfillInterior = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sfillInterior = 0\n", prefix);
    str += tmpStr;
    if(atts->GetRandomSamples())
        SNPRINTF(tmpStr, 1000, "%srandomSamples = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%srandomSamples = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%srandomSeed = %d\n", prefix, atts->GetRandomSeed());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%snumberOfRandomSamples = %d\n", prefix, atts->GetNumberOfRandomSamples());
    str += tmpStr;
    if(atts->GetForceNodeCenteredData())
        SNPRINTF(tmpStr, 1000, "%sforceNodeCenteredData = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sforceNodeCenteredData = 0\n", prefix);
    str += tmpStr;
    if(atts->GetIssueTerminationWarnings())
        SNPRINTF(tmpStr, 1000, "%sissueTerminationWarnings = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sissueTerminationWarnings = 0\n", prefix);
    str += tmpStr;
    if(atts->GetIssueStiffnessWarnings())
        SNPRINTF(tmpStr, 1000, "%sissueStiffnessWarnings = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sissueStiffnessWarnings = 0\n", prefix);
    str += tmpStr;
    if(atts->GetIssueCriticalPointsWarnings())
        SNPRINTF(tmpStr, 1000, "%sissueCriticalPointsWarnings = 1\n", prefix);
    else
        SNPRINTF(tmpStr, 1000, "%sissueCriticalPointsWarnings = 0\n", prefix);
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%scriticalPointThreshold = %g\n", prefix, atts->GetCriticalPointThreshold());
    str += tmpStr;
    const char *varyTubeRadius_names = "None, Scalar";
    switch (atts->GetVaryTubeRadius())
    {
      case StreamlineAttributes::None:
          SNPRINTF(tmpStr, 1000, "%svaryTubeRadius = %sNone  # %s\n", prefix, prefix, varyTubeRadius_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::Scalar:
          SNPRINTF(tmpStr, 1000, "%svaryTubeRadius = %sScalar  # %s\n", prefix, prefix, varyTubeRadius_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%svaryTubeRadiusFactor = %g\n", prefix, atts->GetVaryTubeRadiusFactor());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%svaryTubeRadiusVariable = \"%s\"\n", prefix, atts->GetVaryTubeRadiusVariable().c_str());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%scorrelationDistanceAngTol = %g\n", prefix, atts->GetCorrelationDistanceAngTol());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%scorrelationDistanceMinDistAbsolute = %g\n", prefix, atts->GetCorrelationDistanceMinDistAbsolute());
    str += tmpStr;
    SNPRINTF(tmpStr, 1000, "%scorrelationDistanceMinDistBBox = %g\n", prefix, atts->GetCorrelationDistanceMinDistBBox());
    str += tmpStr;
    const char *correlationDistanceMinDistType_names = "Absolute, FractionOfBBox";
    switch (atts->GetCorrelationDistanceMinDistType())
    {
      case StreamlineAttributes::Absolute:
          SNPRINTF(tmpStr, 1000, "%scorrelationDistanceMinDistType = %sAbsolute  # %s\n", prefix, prefix, correlationDistanceMinDistType_names);
          str += tmpStr;
          break;
      case StreamlineAttributes::FractionOfBBox:
          SNPRINTF(tmpStr, 1000, "%scorrelationDistanceMinDistType = %sFractionOfBBox  # %s\n", prefix, prefix, correlationDistanceMinDistType_names);
          str += tmpStr;
          break;
      default:
          break;
    }

    SNPRINTF(tmpStr, 1000, "%sselection = \"%s\"\n", prefix, atts->GetSelection().c_str());
    str += tmpStr;
    return str;
}

static PyObject *
StreamlineAttributes_Notify(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    obj->data->Notify();
    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_SetSourceType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the sourceType in the object.
    if(ival >= 0 && ival < 8)
        obj->data->SetSourceType(StreamlineAttributes::SourceType(ival));
    else
    {
        fprintf(stderr, "An invalid sourceType value was given. "
                        "Valid values are in the range of [0,7]. "
                        "You can also use the following names: "
                        "SpecifiedPoint, SpecifiedPointList, SpecifiedLine, SpecifiedCircle, SpecifiedPlane, "
                        "SpecifiedSphere, SpecifiedBox, Selection.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSourceType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetSourceType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPointSource(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double *dvals = obj->data->GetPointSource();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the pointSource in the object as modified.
    obj->data->SelectPointSource();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPointSource(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the pointSource.
    PyObject *retval = PyTuple_New(3);
    const double *pointSource = obj->data->GetPointSource();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(pointSource[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLineStart(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double *dvals = obj->data->GetLineStart();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the lineStart in the object as modified.
    obj->data->SelectLineStart();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLineStart(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the lineStart.
    PyObject *retval = PyTuple_New(3);
    const double *lineStart = obj->data->GetLineStart();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(lineStart[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLineEnd(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double *dvals = obj->data->GetLineEnd();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the lineEnd in the object as modified.
    obj->data->SelectLineEnd();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLineEnd(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the lineEnd.
    PyObject *retval = PyTuple_New(3);
    const double *lineEnd = obj->data->GetLineEnd();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(lineEnd[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPlaneOrigin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double *dvals = obj->data->GetPlaneOrigin();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the planeOrigin in the object as modified.
    obj->data->SelectPlaneOrigin();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPlaneOrigin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the planeOrigin.
    PyObject *retval = PyTuple_New(3);
    const double *planeOrigin = obj->data->GetPlaneOrigin();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(planeOrigin[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPlaneNormal(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double *dvals = obj->data->GetPlaneNormal();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the planeNormal in the object as modified.
    obj->data->SelectPlaneNormal();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPlaneNormal(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the planeNormal.
    PyObject *retval = PyTuple_New(3);
    const double *planeNormal = obj->data->GetPlaneNormal();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(planeNormal[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPlaneUpAxis(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double *dvals = obj->data->GetPlaneUpAxis();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the planeUpAxis in the object as modified.
    obj->data->SelectPlaneUpAxis();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPlaneUpAxis(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the planeUpAxis.
    PyObject *retval = PyTuple_New(3);
    const double *planeUpAxis = obj->data->GetPlaneUpAxis();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(planeUpAxis[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetRadius(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the radius in the object.
    obj->data->SetRadius(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetRadius(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetRadius());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSphereOrigin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double *dvals = obj->data->GetSphereOrigin();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the sphereOrigin in the object as modified.
    obj->data->SelectSphereOrigin();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSphereOrigin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the sphereOrigin.
    PyObject *retval = PyTuple_New(3);
    const double *sphereOrigin = obj->data->GetSphereOrigin();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(sphereOrigin[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetBoxExtents(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double *dvals = obj->data->GetBoxExtents();
    if(!PyArg_ParseTuple(args, "dddddd", &dvals[0], &dvals[1], &dvals[2], &dvals[3], &dvals[4], &dvals[5]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 6)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the boxExtents in the object as modified.
    obj->data->SelectBoxExtents();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetBoxExtents(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the boxExtents.
    PyObject *retval = PyTuple_New(6);
    const double *boxExtents = obj->data->GetBoxExtents();
    for(int i = 0; i < 6; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(boxExtents[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetUseWholeBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the useWholeBox in the object.
    obj->data->SetUseWholeBox(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetUseWholeBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetUseWholeBox()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPointList(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    doubleVector  &vec = obj->data->GetPointList();
    PyObject     *tuple;
    if(!PyArg_ParseTuple(args, "O", &tuple))
        return NULL;

    if(PyTuple_Check(tuple))
    {
        vec.resize(PyTuple_Size(tuple));
        for(int i = 0; i < PyTuple_Size(tuple); ++i)
        {
            PyObject *item = PyTuple_GET_ITEM(tuple, i);
            if(PyFloat_Check(item))
                vec[i] = PyFloat_AS_DOUBLE(item);
            else if(PyInt_Check(item))
                vec[i] = double(PyInt_AS_LONG(item));
            else if(PyLong_Check(item))
                vec[i] = PyLong_AsDouble(item);
            else
                vec[i] = 0.;
        }
    }
    else if(PyFloat_Check(tuple))
    {
        vec.resize(1);
        vec[0] = PyFloat_AS_DOUBLE(tuple);
    }
    else if(PyInt_Check(tuple))
    {
        vec.resize(1);
        vec[0] = double(PyInt_AS_LONG(tuple));
    }
    else if(PyLong_Check(tuple))
    {
        vec.resize(1);
        vec[0] = PyLong_AsDouble(tuple);
    }
    else
        return NULL;

    // Mark the pointList in the object as modified.
    obj->data->SelectPointList();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPointList(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the pointList.
    const doubleVector &pointList = obj->data->GetPointList();
    PyObject *retval = PyTuple_New(pointList.size());
    for(size_t i = 0; i < pointList.size(); ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(pointList[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSampleDensity0(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the sampleDensity0 in the object.
    obj->data->SetSampleDensity0((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSampleDensity0(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetSampleDensity0()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSampleDensity1(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the sampleDensity1 in the object.
    obj->data->SetSampleDensity1((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSampleDensity1(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetSampleDensity1()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSampleDensity2(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the sampleDensity2 in the object.
    obj->data->SetSampleDensity2((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSampleDensity2(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetSampleDensity2()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetColoringMethod(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the coloringMethod in the object.
    if(ival >= 0 && ival < 8)
        obj->data->SetColoringMethod(StreamlineAttributes::ColoringMethod(ival));
    else
    {
        fprintf(stderr, "An invalid coloringMethod value was given. "
                        "Valid values are in the range of [0,7]. "
                        "You can also use the following names: "
                        "Solid, ColorBySpeed, ColorByVorticity, ColorByLength, ColorByTime, "
                        "ColorBySeedPointID, ColorByVariable, ColorByCorrelationDistance.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetColoringMethod(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetColoringMethod()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetColorTableName(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    char *str;
    if(!PyArg_ParseTuple(args, "s", &str))
        return NULL;

    // Set the colorTableName in the object.
    obj->data->SetColorTableName(std::string(str));

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetColorTableName(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyString_FromString(obj->data->GetColorTableName().c_str());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSingleColor(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int c[4];
    if(!PyArg_ParseTuple(args, "iiii", &c[0], &c[1], &c[2], &c[3]))
    {
        c[3] = 255;
        if(!PyArg_ParseTuple(args, "iii", &c[0], &c[1], &c[2]))
        {
            double dr, dg, db, da;
            if(PyArg_ParseTuple(args, "dddd", &dr, &dg, &db, &da))
            {
                c[0] = int(dr);
                c[1] = int(dg);
                c[2] = int(db);
                c[3] = int(da);
            }
            else if(PyArg_ParseTuple(args, "ddd", &dr, &dg, &db))
            {
                c[0] = int(dr);
                c[1] = int(dg);
                c[2] = int(db);
                c[3] = 255;
            }
            else
            {
                PyObject *tuple = NULL;
                if(!PyArg_ParseTuple(args, "O", &tuple))
                    return NULL;

                if(!PyTuple_Check(tuple))
                    return NULL;

                // Make sure that the tuple is the right size.
                if(PyTuple_Size(tuple) < 3 || PyTuple_Size(tuple) > 4)
                    return NULL;

                // Make sure that all elements in the tuple are ints.
                for(int i = 0; i < PyTuple_Size(tuple); ++i)
                {
                    PyObject *item = PyTuple_GET_ITEM(tuple, i);
                    if(PyInt_Check(item))
                        c[i] = int(PyInt_AS_LONG(PyTuple_GET_ITEM(tuple, i)));
                    else if(PyFloat_Check(item))
                        c[i] = int(PyFloat_AS_DOUBLE(PyTuple_GET_ITEM(tuple, i)));
                    else
                        return NULL;
                }
            }
        }
        PyErr_Clear();
    }

    // Set the singleColor in the object.
    ColorAttribute ca(c[0], c[1], c[2], c[3]);
    obj->data->SetSingleColor(ca);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSingleColor(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the singleColor.
    PyObject *retval = PyTuple_New(4);
    const unsigned char *singleColor = obj->data->GetSingleColor().GetColor();
    PyTuple_SET_ITEM(retval, 0, PyInt_FromLong(long(singleColor[0])));
    PyTuple_SET_ITEM(retval, 1, PyInt_FromLong(long(singleColor[1])));
    PyTuple_SET_ITEM(retval, 2, PyInt_FromLong(long(singleColor[2])));
    PyTuple_SET_ITEM(retval, 3, PyInt_FromLong(long(singleColor[3])));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLegendFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the legendFlag in the object.
    obj->data->SetLegendFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLegendFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetLegendFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLightingFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the lightingFlag in the object.
    obj->data->SetLightingFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLightingFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetLightingFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetIntegrationDirection(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the integrationDirection in the object.
    if(ival >= 0 && ival < 3)
        obj->data->SetIntegrationDirection(StreamlineAttributes::IntegrationDirection(ival));
    else
    {
        fprintf(stderr, "An invalid integrationDirection value was given. "
                        "Valid values are in the range of [0,2]. "
                        "You can also use the following names: "
                        "Forward, Backward, Both.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetIntegrationDirection(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetIntegrationDirection()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetMaxSteps(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the maxSteps in the object.
    obj->data->SetMaxSteps((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetMaxSteps(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetMaxSteps()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetTerminateByDistance(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the terminateByDistance in the object.
    obj->data->SetTerminateByDistance(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetTerminateByDistance(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetTerminateByDistance()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetTermDistance(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the termDistance in the object.
    obj->data->SetTermDistance(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetTermDistance(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetTermDistance());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetTerminateByTime(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the terminateByTime in the object.
    obj->data->SetTerminateByTime(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetTerminateByTime(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetTerminateByTime()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetTermTime(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the termTime in the object.
    obj->data->SetTermTime(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetTermTime(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetTermTime());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetMaxStepLength(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the maxStepLength in the object.
    obj->data->SetMaxStepLength(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetMaxStepLength(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetMaxStepLength());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLimitMaximumTimestep(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the limitMaximumTimestep in the object.
    obj->data->SetLimitMaximumTimestep(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLimitMaximumTimestep(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetLimitMaximumTimestep()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetMaxTimeStep(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the maxTimeStep in the object.
    obj->data->SetMaxTimeStep(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetMaxTimeStep(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetMaxTimeStep());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetRelTol(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the relTol in the object.
    obj->data->SetRelTol(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetRelTol(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetRelTol());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetAbsTolSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the absTolSizeType in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetAbsTolSizeType(StreamlineAttributes::SizeType(ival));
    else
    {
        fprintf(stderr, "An invalid absTolSizeType value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Absolute, FractionOfBBox.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetAbsTolSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetAbsTolSizeType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetAbsTolAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the absTolAbsolute in the object.
    obj->data->SetAbsTolAbsolute(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetAbsTolAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetAbsTolAbsolute());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetAbsTolBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the absTolBBox in the object.
    obj->data->SetAbsTolBBox(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetAbsTolBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetAbsTolBBox());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetFieldType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the fieldType in the object.
    if(ival >= 0 && ival < 6)
        obj->data->SetFieldType(StreamlineAttributes::FieldType(ival));
    else
    {
        fprintf(stderr, "An invalid fieldType value was given. "
                        "Valid values are in the range of [0,5]. "
                        "You can also use the following names: "
                        "Default, FlashField, M3DC12DField, M3DC13DField, Nek5000Field, "
                        "NIMRODField.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetFieldType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetFieldType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetFieldConstant(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the fieldConstant in the object.
    obj->data->SetFieldConstant(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetFieldConstant(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetFieldConstant());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetVelocitySource(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double *dvals = obj->data->GetVelocitySource();
    if(!PyArg_ParseTuple(args, "ddd", &dvals[0], &dvals[1], &dvals[2]))
    {
        PyObject     *tuple;
        if(!PyArg_ParseTuple(args, "O", &tuple))
            return NULL;

        if(PyTuple_Check(tuple))
        {
            if(PyTuple_Size(tuple) != 3)
                return NULL;

            PyErr_Clear();
            for(int i = 0; i < PyTuple_Size(tuple); ++i)
            {
                PyObject *item = PyTuple_GET_ITEM(tuple, i);
                if(PyFloat_Check(item))
                    dvals[i] = PyFloat_AS_DOUBLE(item);
                else if(PyInt_Check(item))
                    dvals[i] = double(PyInt_AS_LONG(item));
                else if(PyLong_Check(item))
                    dvals[i] = PyLong_AsDouble(item);
                else
                    dvals[i] = 0.;
            }
        }
        else
            return NULL;
    }

    // Mark the velocitySource in the object as modified.
    obj->data->SelectVelocitySource();

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetVelocitySource(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    // Allocate a tuple the with enough entries to hold the velocitySource.
    PyObject *retval = PyTuple_New(3);
    const double *velocitySource = obj->data->GetVelocitySource();
    for(int i = 0; i < 3; ++i)
        PyTuple_SET_ITEM(retval, i, PyFloat_FromDouble(velocitySource[i]));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetIntegrationType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the integrationType in the object.
    if(ival >= 0 && ival < 6)
        obj->data->SetIntegrationType(StreamlineAttributes::IntegrationType(ival));
    else
    {
        fprintf(stderr, "An invalid integrationType value was given. "
                        "Valid values are in the range of [0,5]. "
                        "You can also use the following names: "
                        "Euler, Leapfrog, DormandPrince, AdamsBashforth, RK4, "
                        "M3DC12DIntegrator.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetIntegrationType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetIntegrationType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetParallelizationAlgorithmType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the parallelizationAlgorithmType in the object.
    if(ival >= 0 && ival < 4)
        obj->data->SetParallelizationAlgorithmType(StreamlineAttributes::ParallelizationAlgorithmType(ival));
    else
    {
        fprintf(stderr, "An invalid parallelizationAlgorithmType value was given. "
                        "Valid values are in the range of [0,3]. "
                        "You can also use the following names: "
                        "LoadOnDemand, ParallelStaticDomains, MasterSlave, VisItSelects.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetParallelizationAlgorithmType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetParallelizationAlgorithmType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetMaxProcessCount(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the maxProcessCount in the object.
    obj->data->SetMaxProcessCount((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetMaxProcessCount(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetMaxProcessCount()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetMaxDomainCacheSize(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the maxDomainCacheSize in the object.
    obj->data->SetMaxDomainCacheSize((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetMaxDomainCacheSize(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetMaxDomainCacheSize()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetWorkGroupSize(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the workGroupSize in the object.
    obj->data->SetWorkGroupSize((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetWorkGroupSize(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetWorkGroupSize()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPathlines(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the pathlines in the object.
    obj->data->SetPathlines(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPathlines(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetPathlines()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPathlinesOverrideStartingTimeFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the pathlinesOverrideStartingTimeFlag in the object.
    obj->data->SetPathlinesOverrideStartingTimeFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPathlinesOverrideStartingTimeFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetPathlinesOverrideStartingTimeFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPathlinesOverrideStartingTime(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the pathlinesOverrideStartingTime in the object.
    obj->data->SetPathlinesOverrideStartingTime(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPathlinesOverrideStartingTime(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetPathlinesOverrideStartingTime());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPathlinesPeriod(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the pathlinesPeriod in the object.
    obj->data->SetPathlinesPeriod(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPathlinesPeriod(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetPathlinesPeriod());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPathlinesCMFE(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the pathlinesCMFE in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetPathlinesCMFE(StreamlineAttributes::PathlinesCMFE(ival));
    else
    {
        fprintf(stderr, "An invalid pathlinesCMFE value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "CONN_CMFE, POS_CMFE.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPathlinesCMFE(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetPathlinesCMFE()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetCoordinateSystem(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the coordinateSystem in the object.
    if(ival >= 0 && ival < 3)
        obj->data->SetCoordinateSystem(StreamlineAttributes::CoordinateSystem(ival));
    else
    {
        fprintf(stderr, "An invalid coordinateSystem value was given. "
                        "Valid values are in the range of [0,2]. "
                        "You can also use the following names: "
                        "AsIs, CylindricalToCartesian, CartesianToCylindrical.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetCoordinateSystem(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetCoordinateSystem()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPhiScalingFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the phiScalingFlag in the object.
    obj->data->SetPhiScalingFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPhiScalingFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetPhiScalingFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetPhiScaling(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the phiScaling in the object.
    obj->data->SetPhiScaling(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetPhiScaling(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetPhiScaling());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetColoringVariable(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    char *str;
    if(!PyArg_ParseTuple(args, "s", &str))
        return NULL;

    // Set the coloringVariable in the object.
    obj->data->SetColoringVariable(std::string(str));

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetColoringVariable(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyString_FromString(obj->data->GetColoringVariable().c_str());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLegendMinFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the legendMinFlag in the object.
    obj->data->SetLegendMinFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLegendMinFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetLegendMinFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLegendMaxFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the legendMaxFlag in the object.
    obj->data->SetLegendMaxFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLegendMaxFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetLegendMaxFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLegendMin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the legendMin in the object.
    obj->data->SetLegendMin(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLegendMin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetLegendMin());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLegendMax(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the legendMax in the object.
    obj->data->SetLegendMax(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLegendMax(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetLegendMax());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetDisplayBegin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the displayBegin in the object.
    obj->data->SetDisplayBegin(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetDisplayBegin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetDisplayBegin());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetDisplayEnd(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the displayEnd in the object.
    obj->data->SetDisplayEnd(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetDisplayEnd(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetDisplayEnd());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetDisplayBeginFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the displayBeginFlag in the object.
    obj->data->SetDisplayBeginFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetDisplayBeginFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetDisplayBeginFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetDisplayEndFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the displayEndFlag in the object.
    obj->data->SetDisplayEndFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetDisplayEndFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetDisplayEndFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetReferenceTypeForDisplay(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the referenceTypeForDisplay in the object.
    if(ival >= 0 && ival < 3)
        obj->data->SetReferenceTypeForDisplay(StreamlineAttributes::ReferenceType(ival));
    else
    {
        fprintf(stderr, "An invalid referenceTypeForDisplay value was given. "
                        "Valid values are in the range of [0,2]. "
                        "You can also use the following names: "
                        "Distance, Time, Step.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetReferenceTypeForDisplay(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetReferenceTypeForDisplay()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetDisplayMethod(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the displayMethod in the object.
    if(ival >= 0 && ival < 3)
        obj->data->SetDisplayMethod(StreamlineAttributes::DisplayMethod(ival));
    else
    {
        fprintf(stderr, "An invalid displayMethod value was given. "
                        "Valid values are in the range of [0,2]. "
                        "You can also use the following names: "
                        "Lines, Tubes, Ribbons.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetDisplayMethod(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetDisplayMethod()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetTubeSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the tubeSizeType in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetTubeSizeType(StreamlineAttributes::SizeType(ival));
    else
    {
        fprintf(stderr, "An invalid tubeSizeType value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Absolute, FractionOfBBox.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetTubeSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetTubeSizeType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetTubeRadiusAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the tubeRadiusAbsolute in the object.
    obj->data->SetTubeRadiusAbsolute(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetTubeRadiusAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetTubeRadiusAbsolute());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetTubeRadiusBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the tubeRadiusBBox in the object.
    obj->data->SetTubeRadiusBBox(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetTubeRadiusBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetTubeRadiusBBox());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetRibbonWidthSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the ribbonWidthSizeType in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetRibbonWidthSizeType(StreamlineAttributes::SizeType(ival));
    else
    {
        fprintf(stderr, "An invalid ribbonWidthSizeType value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Absolute, FractionOfBBox.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetRibbonWidthSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetRibbonWidthSizeType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetRibbonWidthAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the ribbonWidthAbsolute in the object.
    obj->data->SetRibbonWidthAbsolute(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetRibbonWidthAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetRibbonWidthAbsolute());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetRibbonWidthBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the ribbonWidthBBox in the object.
    obj->data->SetRibbonWidthBBox(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetRibbonWidthBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetRibbonWidthBBox());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetLineWidth(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the lineWidth in the object.
    obj->data->SetLineWidth(ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetLineWidth(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetLineWidth()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetShowSeeds(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the showSeeds in the object.
    obj->data->SetShowSeeds(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetShowSeeds(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetShowSeeds()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSeedRadiusSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the seedRadiusSizeType in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetSeedRadiusSizeType(StreamlineAttributes::SizeType(ival));
    else
    {
        fprintf(stderr, "An invalid seedRadiusSizeType value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Absolute, FractionOfBBox.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSeedRadiusSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetSeedRadiusSizeType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSeedRadiusAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the seedRadiusAbsolute in the object.
    obj->data->SetSeedRadiusAbsolute(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSeedRadiusAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetSeedRadiusAbsolute());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSeedRadiusBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the seedRadiusBBox in the object.
    obj->data->SetSeedRadiusBBox(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSeedRadiusBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetSeedRadiusBBox());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetShowHeads(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the showHeads in the object.
    obj->data->SetShowHeads(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetShowHeads(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetShowHeads()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetHeadDisplayType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the headDisplayType in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetHeadDisplayType(StreamlineAttributes::GeomDisplayType(ival));
    else
    {
        fprintf(stderr, "An invalid headDisplayType value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Sphere, Cone.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetHeadDisplayType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetHeadDisplayType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetHeadRadiusSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the headRadiusSizeType in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetHeadRadiusSizeType(StreamlineAttributes::SizeType(ival));
    else
    {
        fprintf(stderr, "An invalid headRadiusSizeType value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Absolute, FractionOfBBox.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetHeadRadiusSizeType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetHeadRadiusSizeType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetHeadRadiusAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the headRadiusAbsolute in the object.
    obj->data->SetHeadRadiusAbsolute(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetHeadRadiusAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetHeadRadiusAbsolute());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetHeadRadiusBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the headRadiusBBox in the object.
    obj->data->SetHeadRadiusBBox(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetHeadRadiusBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetHeadRadiusBBox());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetHeadHeightRatio(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the headHeightRatio in the object.
    obj->data->SetHeadHeightRatio(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetHeadHeightRatio(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetHeadHeightRatio());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetOpacityType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the opacityType in the object.
    if(ival >= 0 && ival < 4)
        obj->data->SetOpacityType(StreamlineAttributes::OpacityType(ival));
    else
    {
        fprintf(stderr, "An invalid opacityType value was given. "
                        "Valid values are in the range of [0,3]. "
                        "You can also use the following names: "
                        "FullyOpaque, Constant, Ramp, VariableRange.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetOpacityType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetOpacityType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetOpacityVariable(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    char *str;
    if(!PyArg_ParseTuple(args, "s", &str))
        return NULL;

    // Set the opacityVariable in the object.
    obj->data->SetOpacityVariable(std::string(str));

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetOpacityVariable(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyString_FromString(obj->data->GetOpacityVariable().c_str());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetOpacity(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the opacity in the object.
    obj->data->SetOpacity(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetOpacity(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetOpacity());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetOpacityVarMin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the opacityVarMin in the object.
    obj->data->SetOpacityVarMin(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetOpacityVarMin(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetOpacityVarMin());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetOpacityVarMax(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the opacityVarMax in the object.
    obj->data->SetOpacityVarMax(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetOpacityVarMax(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetOpacityVarMax());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetOpacityVarMinFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the opacityVarMinFlag in the object.
    obj->data->SetOpacityVarMinFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetOpacityVarMinFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetOpacityVarMinFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetOpacityVarMaxFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the opacityVarMaxFlag in the object.
    obj->data->SetOpacityVarMaxFlag(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetOpacityVarMaxFlag(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetOpacityVarMaxFlag()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetTubeDisplayDensity(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the tubeDisplayDensity in the object.
    obj->data->SetTubeDisplayDensity((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetTubeDisplayDensity(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetTubeDisplayDensity()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetGeomDisplayQuality(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the geomDisplayQuality in the object.
    if(ival >= 0 && ival < 4)
        obj->data->SetGeomDisplayQuality(StreamlineAttributes::DisplayQuality(ival));
    else
    {
        fprintf(stderr, "An invalid geomDisplayQuality value was given. "
                        "Valid values are in the range of [0,3]. "
                        "You can also use the following names: "
                        "Low, Medium, High, Super.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetGeomDisplayQuality(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetGeomDisplayQuality()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSampleDistance0(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the sampleDistance0 in the object.
    obj->data->SetSampleDistance0(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSampleDistance0(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetSampleDistance0());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSampleDistance1(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the sampleDistance1 in the object.
    obj->data->SetSampleDistance1(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSampleDistance1(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetSampleDistance1());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSampleDistance2(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the sampleDistance2 in the object.
    obj->data->SetSampleDistance2(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSampleDistance2(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetSampleDistance2());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetFillInterior(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the fillInterior in the object.
    obj->data->SetFillInterior(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetFillInterior(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetFillInterior()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetRandomSamples(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the randomSamples in the object.
    obj->data->SetRandomSamples(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetRandomSamples(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetRandomSamples()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetRandomSeed(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the randomSeed in the object.
    obj->data->SetRandomSeed((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetRandomSeed(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetRandomSeed()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetNumberOfRandomSamples(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the numberOfRandomSamples in the object.
    obj->data->SetNumberOfRandomSamples((int)ival);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetNumberOfRandomSamples(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetNumberOfRandomSamples()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetForceNodeCenteredData(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the forceNodeCenteredData in the object.
    obj->data->SetForceNodeCenteredData(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetForceNodeCenteredData(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetForceNodeCenteredData()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetIssueTerminationWarnings(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the issueTerminationWarnings in the object.
    obj->data->SetIssueTerminationWarnings(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetIssueTerminationWarnings(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetIssueTerminationWarnings()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetIssueStiffnessWarnings(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the issueStiffnessWarnings in the object.
    obj->data->SetIssueStiffnessWarnings(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetIssueStiffnessWarnings(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetIssueStiffnessWarnings()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetIssueCriticalPointsWarnings(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the issueCriticalPointsWarnings in the object.
    obj->data->SetIssueCriticalPointsWarnings(ival != 0);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetIssueCriticalPointsWarnings(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(obj->data->GetIssueCriticalPointsWarnings()?1L:0L);
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetCriticalPointThreshold(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the criticalPointThreshold in the object.
    obj->data->SetCriticalPointThreshold(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetCriticalPointThreshold(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetCriticalPointThreshold());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetVaryTubeRadius(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the varyTubeRadius in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetVaryTubeRadius(StreamlineAttributes::VaryTubeRadiusType(ival));
    else
    {
        fprintf(stderr, "An invalid varyTubeRadius value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "None, Scalar.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetVaryTubeRadius(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetVaryTubeRadius()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetVaryTubeRadiusFactor(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the varyTubeRadiusFactor in the object.
    obj->data->SetVaryTubeRadiusFactor(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetVaryTubeRadiusFactor(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetVaryTubeRadiusFactor());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetVaryTubeRadiusVariable(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    char *str;
    if(!PyArg_ParseTuple(args, "s", &str))
        return NULL;

    // Set the varyTubeRadiusVariable in the object.
    obj->data->SetVaryTubeRadiusVariable(std::string(str));

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetVaryTubeRadiusVariable(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyString_FromString(obj->data->GetVaryTubeRadiusVariable().c_str());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetCorrelationDistanceAngTol(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the correlationDistanceAngTol in the object.
    obj->data->SetCorrelationDistanceAngTol(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetCorrelationDistanceAngTol(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetCorrelationDistanceAngTol());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetCorrelationDistanceMinDistAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the correlationDistanceMinDistAbsolute in the object.
    obj->data->SetCorrelationDistanceMinDistAbsolute(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetCorrelationDistanceMinDistAbsolute(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetCorrelationDistanceMinDistAbsolute());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetCorrelationDistanceMinDistBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    double dval;
    if(!PyArg_ParseTuple(args, "d", &dval))
        return NULL;

    // Set the correlationDistanceMinDistBBox in the object.
    obj->data->SetCorrelationDistanceMinDistBBox(dval);

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetCorrelationDistanceMinDistBBox(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyFloat_FromDouble(obj->data->GetCorrelationDistanceMinDistBBox());
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetCorrelationDistanceMinDistType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    int ival;
    if(!PyArg_ParseTuple(args, "i", &ival))
        return NULL;

    // Set the correlationDistanceMinDistType in the object.
    if(ival >= 0 && ival < 2)
        obj->data->SetCorrelationDistanceMinDistType(StreamlineAttributes::SizeType(ival));
    else
    {
        fprintf(stderr, "An invalid correlationDistanceMinDistType value was given. "
                        "Valid values are in the range of [0,1]. "
                        "You can also use the following names: "
                        "Absolute, FractionOfBBox.");
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetCorrelationDistanceMinDistType(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyInt_FromLong(long(obj->data->GetCorrelationDistanceMinDistType()));
    return retval;
}

/*static*/ PyObject *
StreamlineAttributes_SetSelection(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;

    char *str;
    if(!PyArg_ParseTuple(args, "s", &str))
        return NULL;

    // Set the selection in the object.
    obj->data->SetSelection(std::string(str));

    Py_INCREF(Py_None);
    return Py_None;
}

/*static*/ PyObject *
StreamlineAttributes_GetSelection(PyObject *self, PyObject *args)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)self;
    PyObject *retval = PyString_FromString(obj->data->GetSelection().c_str());
    return retval;
}



PyMethodDef PyStreamlineAttributes_methods[STREAMLINEATTRIBUTES_NMETH] = {
    {"Notify", StreamlineAttributes_Notify, METH_VARARGS},
    {"SetSourceType", StreamlineAttributes_SetSourceType, METH_VARARGS},
    {"GetSourceType", StreamlineAttributes_GetSourceType, METH_VARARGS},
    {"SetPointSource", StreamlineAttributes_SetPointSource, METH_VARARGS},
    {"GetPointSource", StreamlineAttributes_GetPointSource, METH_VARARGS},
    {"SetLineStart", StreamlineAttributes_SetLineStart, METH_VARARGS},
    {"GetLineStart", StreamlineAttributes_GetLineStart, METH_VARARGS},
    {"SetLineEnd", StreamlineAttributes_SetLineEnd, METH_VARARGS},
    {"GetLineEnd", StreamlineAttributes_GetLineEnd, METH_VARARGS},
    {"SetPlaneOrigin", StreamlineAttributes_SetPlaneOrigin, METH_VARARGS},
    {"GetPlaneOrigin", StreamlineAttributes_GetPlaneOrigin, METH_VARARGS},
    {"SetPlaneNormal", StreamlineAttributes_SetPlaneNormal, METH_VARARGS},
    {"GetPlaneNormal", StreamlineAttributes_GetPlaneNormal, METH_VARARGS},
    {"SetPlaneUpAxis", StreamlineAttributes_SetPlaneUpAxis, METH_VARARGS},
    {"GetPlaneUpAxis", StreamlineAttributes_GetPlaneUpAxis, METH_VARARGS},
    {"SetRadius", StreamlineAttributes_SetRadius, METH_VARARGS},
    {"GetRadius", StreamlineAttributes_GetRadius, METH_VARARGS},
    {"SetSphereOrigin", StreamlineAttributes_SetSphereOrigin, METH_VARARGS},
    {"GetSphereOrigin", StreamlineAttributes_GetSphereOrigin, METH_VARARGS},
    {"SetBoxExtents", StreamlineAttributes_SetBoxExtents, METH_VARARGS},
    {"GetBoxExtents", StreamlineAttributes_GetBoxExtents, METH_VARARGS},
    {"SetUseWholeBox", StreamlineAttributes_SetUseWholeBox, METH_VARARGS},
    {"GetUseWholeBox", StreamlineAttributes_GetUseWholeBox, METH_VARARGS},
    {"SetPointList", StreamlineAttributes_SetPointList, METH_VARARGS},
    {"GetPointList", StreamlineAttributes_GetPointList, METH_VARARGS},
    {"SetSampleDensity0", StreamlineAttributes_SetSampleDensity0, METH_VARARGS},
    {"GetSampleDensity0", StreamlineAttributes_GetSampleDensity0, METH_VARARGS},
    {"SetSampleDensity1", StreamlineAttributes_SetSampleDensity1, METH_VARARGS},
    {"GetSampleDensity1", StreamlineAttributes_GetSampleDensity1, METH_VARARGS},
    {"SetSampleDensity2", StreamlineAttributes_SetSampleDensity2, METH_VARARGS},
    {"GetSampleDensity2", StreamlineAttributes_GetSampleDensity2, METH_VARARGS},
    {"SetColoringMethod", StreamlineAttributes_SetColoringMethod, METH_VARARGS},
    {"GetColoringMethod", StreamlineAttributes_GetColoringMethod, METH_VARARGS},
    {"SetColorTableName", StreamlineAttributes_SetColorTableName, METH_VARARGS},
    {"GetColorTableName", StreamlineAttributes_GetColorTableName, METH_VARARGS},
    {"SetSingleColor", StreamlineAttributes_SetSingleColor, METH_VARARGS},
    {"GetSingleColor", StreamlineAttributes_GetSingleColor, METH_VARARGS},
    {"SetLegendFlag", StreamlineAttributes_SetLegendFlag, METH_VARARGS},
    {"GetLegendFlag", StreamlineAttributes_GetLegendFlag, METH_VARARGS},
    {"SetLightingFlag", StreamlineAttributes_SetLightingFlag, METH_VARARGS},
    {"GetLightingFlag", StreamlineAttributes_GetLightingFlag, METH_VARARGS},
    {"SetIntegrationDirection", StreamlineAttributes_SetIntegrationDirection, METH_VARARGS},
    {"GetIntegrationDirection", StreamlineAttributes_GetIntegrationDirection, METH_VARARGS},
    {"SetMaxSteps", StreamlineAttributes_SetMaxSteps, METH_VARARGS},
    {"GetMaxSteps", StreamlineAttributes_GetMaxSteps, METH_VARARGS},
    {"SetTerminateByDistance", StreamlineAttributes_SetTerminateByDistance, METH_VARARGS},
    {"GetTerminateByDistance", StreamlineAttributes_GetTerminateByDistance, METH_VARARGS},
    {"SetTermDistance", StreamlineAttributes_SetTermDistance, METH_VARARGS},
    {"GetTermDistance", StreamlineAttributes_GetTermDistance, METH_VARARGS},
    {"SetTerminateByTime", StreamlineAttributes_SetTerminateByTime, METH_VARARGS},
    {"GetTerminateByTime", StreamlineAttributes_GetTerminateByTime, METH_VARARGS},
    {"SetTermTime", StreamlineAttributes_SetTermTime, METH_VARARGS},
    {"GetTermTime", StreamlineAttributes_GetTermTime, METH_VARARGS},
    {"SetMaxStepLength", StreamlineAttributes_SetMaxStepLength, METH_VARARGS},
    {"GetMaxStepLength", StreamlineAttributes_GetMaxStepLength, METH_VARARGS},
    {"SetLimitMaximumTimestep", StreamlineAttributes_SetLimitMaximumTimestep, METH_VARARGS},
    {"GetLimitMaximumTimestep", StreamlineAttributes_GetLimitMaximumTimestep, METH_VARARGS},
    {"SetMaxTimeStep", StreamlineAttributes_SetMaxTimeStep, METH_VARARGS},
    {"GetMaxTimeStep", StreamlineAttributes_GetMaxTimeStep, METH_VARARGS},
    {"SetRelTol", StreamlineAttributes_SetRelTol, METH_VARARGS},
    {"GetRelTol", StreamlineAttributes_GetRelTol, METH_VARARGS},
    {"SetAbsTolSizeType", StreamlineAttributes_SetAbsTolSizeType, METH_VARARGS},
    {"GetAbsTolSizeType", StreamlineAttributes_GetAbsTolSizeType, METH_VARARGS},
    {"SetAbsTolAbsolute", StreamlineAttributes_SetAbsTolAbsolute, METH_VARARGS},
    {"GetAbsTolAbsolute", StreamlineAttributes_GetAbsTolAbsolute, METH_VARARGS},
    {"SetAbsTolBBox", StreamlineAttributes_SetAbsTolBBox, METH_VARARGS},
    {"GetAbsTolBBox", StreamlineAttributes_GetAbsTolBBox, METH_VARARGS},
    {"SetFieldType", StreamlineAttributes_SetFieldType, METH_VARARGS},
    {"GetFieldType", StreamlineAttributes_GetFieldType, METH_VARARGS},
    {"SetFieldConstant", StreamlineAttributes_SetFieldConstant, METH_VARARGS},
    {"GetFieldConstant", StreamlineAttributes_GetFieldConstant, METH_VARARGS},
    {"SetVelocitySource", StreamlineAttributes_SetVelocitySource, METH_VARARGS},
    {"GetVelocitySource", StreamlineAttributes_GetVelocitySource, METH_VARARGS},
    {"SetIntegrationType", StreamlineAttributes_SetIntegrationType, METH_VARARGS},
    {"GetIntegrationType", StreamlineAttributes_GetIntegrationType, METH_VARARGS},
    {"SetParallelizationAlgorithmType", StreamlineAttributes_SetParallelizationAlgorithmType, METH_VARARGS},
    {"GetParallelizationAlgorithmType", StreamlineAttributes_GetParallelizationAlgorithmType, METH_VARARGS},
    {"SetMaxProcessCount", StreamlineAttributes_SetMaxProcessCount, METH_VARARGS},
    {"GetMaxProcessCount", StreamlineAttributes_GetMaxProcessCount, METH_VARARGS},
    {"SetMaxDomainCacheSize", StreamlineAttributes_SetMaxDomainCacheSize, METH_VARARGS},
    {"GetMaxDomainCacheSize", StreamlineAttributes_GetMaxDomainCacheSize, METH_VARARGS},
    {"SetWorkGroupSize", StreamlineAttributes_SetWorkGroupSize, METH_VARARGS},
    {"GetWorkGroupSize", StreamlineAttributes_GetWorkGroupSize, METH_VARARGS},
    {"SetPathlines", StreamlineAttributes_SetPathlines, METH_VARARGS},
    {"GetPathlines", StreamlineAttributes_GetPathlines, METH_VARARGS},
    {"SetPathlinesOverrideStartingTimeFlag", StreamlineAttributes_SetPathlinesOverrideStartingTimeFlag, METH_VARARGS},
    {"GetPathlinesOverrideStartingTimeFlag", StreamlineAttributes_GetPathlinesOverrideStartingTimeFlag, METH_VARARGS},
    {"SetPathlinesOverrideStartingTime", StreamlineAttributes_SetPathlinesOverrideStartingTime, METH_VARARGS},
    {"GetPathlinesOverrideStartingTime", StreamlineAttributes_GetPathlinesOverrideStartingTime, METH_VARARGS},
    {"SetPathlinesPeriod", StreamlineAttributes_SetPathlinesPeriod, METH_VARARGS},
    {"GetPathlinesPeriod", StreamlineAttributes_GetPathlinesPeriod, METH_VARARGS},
    {"SetPathlinesCMFE", StreamlineAttributes_SetPathlinesCMFE, METH_VARARGS},
    {"GetPathlinesCMFE", StreamlineAttributes_GetPathlinesCMFE, METH_VARARGS},
    {"SetCoordinateSystem", StreamlineAttributes_SetCoordinateSystem, METH_VARARGS},
    {"GetCoordinateSystem", StreamlineAttributes_GetCoordinateSystem, METH_VARARGS},
    {"SetPhiScalingFlag", StreamlineAttributes_SetPhiScalingFlag, METH_VARARGS},
    {"GetPhiScalingFlag", StreamlineAttributes_GetPhiScalingFlag, METH_VARARGS},
    {"SetPhiScaling", StreamlineAttributes_SetPhiScaling, METH_VARARGS},
    {"GetPhiScaling", StreamlineAttributes_GetPhiScaling, METH_VARARGS},
    {"SetColoringVariable", StreamlineAttributes_SetColoringVariable, METH_VARARGS},
    {"GetColoringVariable", StreamlineAttributes_GetColoringVariable, METH_VARARGS},
    {"SetLegendMinFlag", StreamlineAttributes_SetLegendMinFlag, METH_VARARGS},
    {"GetLegendMinFlag", StreamlineAttributes_GetLegendMinFlag, METH_VARARGS},
    {"SetLegendMaxFlag", StreamlineAttributes_SetLegendMaxFlag, METH_VARARGS},
    {"GetLegendMaxFlag", StreamlineAttributes_GetLegendMaxFlag, METH_VARARGS},
    {"SetLegendMin", StreamlineAttributes_SetLegendMin, METH_VARARGS},
    {"GetLegendMin", StreamlineAttributes_GetLegendMin, METH_VARARGS},
    {"SetLegendMax", StreamlineAttributes_SetLegendMax, METH_VARARGS},
    {"GetLegendMax", StreamlineAttributes_GetLegendMax, METH_VARARGS},
    {"SetDisplayBegin", StreamlineAttributes_SetDisplayBegin, METH_VARARGS},
    {"GetDisplayBegin", StreamlineAttributes_GetDisplayBegin, METH_VARARGS},
    {"SetDisplayEnd", StreamlineAttributes_SetDisplayEnd, METH_VARARGS},
    {"GetDisplayEnd", StreamlineAttributes_GetDisplayEnd, METH_VARARGS},
    {"SetDisplayBeginFlag", StreamlineAttributes_SetDisplayBeginFlag, METH_VARARGS},
    {"GetDisplayBeginFlag", StreamlineAttributes_GetDisplayBeginFlag, METH_VARARGS},
    {"SetDisplayEndFlag", StreamlineAttributes_SetDisplayEndFlag, METH_VARARGS},
    {"GetDisplayEndFlag", StreamlineAttributes_GetDisplayEndFlag, METH_VARARGS},
    {"SetReferenceTypeForDisplay", StreamlineAttributes_SetReferenceTypeForDisplay, METH_VARARGS},
    {"GetReferenceTypeForDisplay", StreamlineAttributes_GetReferenceTypeForDisplay, METH_VARARGS},
    {"SetDisplayMethod", StreamlineAttributes_SetDisplayMethod, METH_VARARGS},
    {"GetDisplayMethod", StreamlineAttributes_GetDisplayMethod, METH_VARARGS},
    {"SetTubeSizeType", StreamlineAttributes_SetTubeSizeType, METH_VARARGS},
    {"GetTubeSizeType", StreamlineAttributes_GetTubeSizeType, METH_VARARGS},
    {"SetTubeRadiusAbsolute", StreamlineAttributes_SetTubeRadiusAbsolute, METH_VARARGS},
    {"GetTubeRadiusAbsolute", StreamlineAttributes_GetTubeRadiusAbsolute, METH_VARARGS},
    {"SetTubeRadiusBBox", StreamlineAttributes_SetTubeRadiusBBox, METH_VARARGS},
    {"GetTubeRadiusBBox", StreamlineAttributes_GetTubeRadiusBBox, METH_VARARGS},
    {"SetRibbonWidthSizeType", StreamlineAttributes_SetRibbonWidthSizeType, METH_VARARGS},
    {"GetRibbonWidthSizeType", StreamlineAttributes_GetRibbonWidthSizeType, METH_VARARGS},
    {"SetRibbonWidthAbsolute", StreamlineAttributes_SetRibbonWidthAbsolute, METH_VARARGS},
    {"GetRibbonWidthAbsolute", StreamlineAttributes_GetRibbonWidthAbsolute, METH_VARARGS},
    {"SetRibbonWidthBBox", StreamlineAttributes_SetRibbonWidthBBox, METH_VARARGS},
    {"GetRibbonWidthBBox", StreamlineAttributes_GetRibbonWidthBBox, METH_VARARGS},
    {"SetLineWidth", StreamlineAttributes_SetLineWidth, METH_VARARGS},
    {"GetLineWidth", StreamlineAttributes_GetLineWidth, METH_VARARGS},
    {"SetShowSeeds", StreamlineAttributes_SetShowSeeds, METH_VARARGS},
    {"GetShowSeeds", StreamlineAttributes_GetShowSeeds, METH_VARARGS},
    {"SetSeedRadiusSizeType", StreamlineAttributes_SetSeedRadiusSizeType, METH_VARARGS},
    {"GetSeedRadiusSizeType", StreamlineAttributes_GetSeedRadiusSizeType, METH_VARARGS},
    {"SetSeedRadiusAbsolute", StreamlineAttributes_SetSeedRadiusAbsolute, METH_VARARGS},
    {"GetSeedRadiusAbsolute", StreamlineAttributes_GetSeedRadiusAbsolute, METH_VARARGS},
    {"SetSeedRadiusBBox", StreamlineAttributes_SetSeedRadiusBBox, METH_VARARGS},
    {"GetSeedRadiusBBox", StreamlineAttributes_GetSeedRadiusBBox, METH_VARARGS},
    {"SetShowHeads", StreamlineAttributes_SetShowHeads, METH_VARARGS},
    {"GetShowHeads", StreamlineAttributes_GetShowHeads, METH_VARARGS},
    {"SetHeadDisplayType", StreamlineAttributes_SetHeadDisplayType, METH_VARARGS},
    {"GetHeadDisplayType", StreamlineAttributes_GetHeadDisplayType, METH_VARARGS},
    {"SetHeadRadiusSizeType", StreamlineAttributes_SetHeadRadiusSizeType, METH_VARARGS},
    {"GetHeadRadiusSizeType", StreamlineAttributes_GetHeadRadiusSizeType, METH_VARARGS},
    {"SetHeadRadiusAbsolute", StreamlineAttributes_SetHeadRadiusAbsolute, METH_VARARGS},
    {"GetHeadRadiusAbsolute", StreamlineAttributes_GetHeadRadiusAbsolute, METH_VARARGS},
    {"SetHeadRadiusBBox", StreamlineAttributes_SetHeadRadiusBBox, METH_VARARGS},
    {"GetHeadRadiusBBox", StreamlineAttributes_GetHeadRadiusBBox, METH_VARARGS},
    {"SetHeadHeightRatio", StreamlineAttributes_SetHeadHeightRatio, METH_VARARGS},
    {"GetHeadHeightRatio", StreamlineAttributes_GetHeadHeightRatio, METH_VARARGS},
    {"SetOpacityType", StreamlineAttributes_SetOpacityType, METH_VARARGS},
    {"GetOpacityType", StreamlineAttributes_GetOpacityType, METH_VARARGS},
    {"SetOpacityVariable", StreamlineAttributes_SetOpacityVariable, METH_VARARGS},
    {"GetOpacityVariable", StreamlineAttributes_GetOpacityVariable, METH_VARARGS},
    {"SetOpacity", StreamlineAttributes_SetOpacity, METH_VARARGS},
    {"GetOpacity", StreamlineAttributes_GetOpacity, METH_VARARGS},
    {"SetOpacityVarMin", StreamlineAttributes_SetOpacityVarMin, METH_VARARGS},
    {"GetOpacityVarMin", StreamlineAttributes_GetOpacityVarMin, METH_VARARGS},
    {"SetOpacityVarMax", StreamlineAttributes_SetOpacityVarMax, METH_VARARGS},
    {"GetOpacityVarMax", StreamlineAttributes_GetOpacityVarMax, METH_VARARGS},
    {"SetOpacityVarMinFlag", StreamlineAttributes_SetOpacityVarMinFlag, METH_VARARGS},
    {"GetOpacityVarMinFlag", StreamlineAttributes_GetOpacityVarMinFlag, METH_VARARGS},
    {"SetOpacityVarMaxFlag", StreamlineAttributes_SetOpacityVarMaxFlag, METH_VARARGS},
    {"GetOpacityVarMaxFlag", StreamlineAttributes_GetOpacityVarMaxFlag, METH_VARARGS},
    {"SetTubeDisplayDensity", StreamlineAttributes_SetTubeDisplayDensity, METH_VARARGS},
    {"GetTubeDisplayDensity", StreamlineAttributes_GetTubeDisplayDensity, METH_VARARGS},
    {"SetGeomDisplayQuality", StreamlineAttributes_SetGeomDisplayQuality, METH_VARARGS},
    {"GetGeomDisplayQuality", StreamlineAttributes_GetGeomDisplayQuality, METH_VARARGS},
    {"SetSampleDistance0", StreamlineAttributes_SetSampleDistance0, METH_VARARGS},
    {"GetSampleDistance0", StreamlineAttributes_GetSampleDistance0, METH_VARARGS},
    {"SetSampleDistance1", StreamlineAttributes_SetSampleDistance1, METH_VARARGS},
    {"GetSampleDistance1", StreamlineAttributes_GetSampleDistance1, METH_VARARGS},
    {"SetSampleDistance2", StreamlineAttributes_SetSampleDistance2, METH_VARARGS},
    {"GetSampleDistance2", StreamlineAttributes_GetSampleDistance2, METH_VARARGS},
    {"SetFillInterior", StreamlineAttributes_SetFillInterior, METH_VARARGS},
    {"GetFillInterior", StreamlineAttributes_GetFillInterior, METH_VARARGS},
    {"SetRandomSamples", StreamlineAttributes_SetRandomSamples, METH_VARARGS},
    {"GetRandomSamples", StreamlineAttributes_GetRandomSamples, METH_VARARGS},
    {"SetRandomSeed", StreamlineAttributes_SetRandomSeed, METH_VARARGS},
    {"GetRandomSeed", StreamlineAttributes_GetRandomSeed, METH_VARARGS},
    {"SetNumberOfRandomSamples", StreamlineAttributes_SetNumberOfRandomSamples, METH_VARARGS},
    {"GetNumberOfRandomSamples", StreamlineAttributes_GetNumberOfRandomSamples, METH_VARARGS},
    {"SetForceNodeCenteredData", StreamlineAttributes_SetForceNodeCenteredData, METH_VARARGS},
    {"GetForceNodeCenteredData", StreamlineAttributes_GetForceNodeCenteredData, METH_VARARGS},
    {"SetIssueTerminationWarnings", StreamlineAttributes_SetIssueTerminationWarnings, METH_VARARGS},
    {"GetIssueTerminationWarnings", StreamlineAttributes_GetIssueTerminationWarnings, METH_VARARGS},
    {"SetIssueStiffnessWarnings", StreamlineAttributes_SetIssueStiffnessWarnings, METH_VARARGS},
    {"GetIssueStiffnessWarnings", StreamlineAttributes_GetIssueStiffnessWarnings, METH_VARARGS},
    {"SetIssueCriticalPointsWarnings", StreamlineAttributes_SetIssueCriticalPointsWarnings, METH_VARARGS},
    {"GetIssueCriticalPointsWarnings", StreamlineAttributes_GetIssueCriticalPointsWarnings, METH_VARARGS},
    {"SetCriticalPointThreshold", StreamlineAttributes_SetCriticalPointThreshold, METH_VARARGS},
    {"GetCriticalPointThreshold", StreamlineAttributes_GetCriticalPointThreshold, METH_VARARGS},
    {"SetVaryTubeRadius", StreamlineAttributes_SetVaryTubeRadius, METH_VARARGS},
    {"GetVaryTubeRadius", StreamlineAttributes_GetVaryTubeRadius, METH_VARARGS},
    {"SetVaryTubeRadiusFactor", StreamlineAttributes_SetVaryTubeRadiusFactor, METH_VARARGS},
    {"GetVaryTubeRadiusFactor", StreamlineAttributes_GetVaryTubeRadiusFactor, METH_VARARGS},
    {"SetVaryTubeRadiusVariable", StreamlineAttributes_SetVaryTubeRadiusVariable, METH_VARARGS},
    {"GetVaryTubeRadiusVariable", StreamlineAttributes_GetVaryTubeRadiusVariable, METH_VARARGS},
    {"SetCorrelationDistanceAngTol", StreamlineAttributes_SetCorrelationDistanceAngTol, METH_VARARGS},
    {"GetCorrelationDistanceAngTol", StreamlineAttributes_GetCorrelationDistanceAngTol, METH_VARARGS},
    {"SetCorrelationDistanceMinDistAbsolute", StreamlineAttributes_SetCorrelationDistanceMinDistAbsolute, METH_VARARGS},
    {"GetCorrelationDistanceMinDistAbsolute", StreamlineAttributes_GetCorrelationDistanceMinDistAbsolute, METH_VARARGS},
    {"SetCorrelationDistanceMinDistBBox", StreamlineAttributes_SetCorrelationDistanceMinDistBBox, METH_VARARGS},
    {"GetCorrelationDistanceMinDistBBox", StreamlineAttributes_GetCorrelationDistanceMinDistBBox, METH_VARARGS},
    {"SetCorrelationDistanceMinDistType", StreamlineAttributes_SetCorrelationDistanceMinDistType, METH_VARARGS},
    {"GetCorrelationDistanceMinDistType", StreamlineAttributes_GetCorrelationDistanceMinDistType, METH_VARARGS},
    {"SetSelection", StreamlineAttributes_SetSelection, METH_VARARGS},
    {"GetSelection", StreamlineAttributes_GetSelection, METH_VARARGS},
    {NULL, NULL}
};

//
// Type functions
//

static void
StreamlineAttributes_dealloc(PyObject *v)
{
   StreamlineAttributesObject *obj = (StreamlineAttributesObject *)v;
   if(obj->parent != 0)
       Py_DECREF(obj->parent);
   if(obj->owns)
       delete obj->data;
}

static int
StreamlineAttributes_compare(PyObject *v, PyObject *w)
{
    StreamlineAttributes *a = ((StreamlineAttributesObject *)v)->data;
    StreamlineAttributes *b = ((StreamlineAttributesObject *)w)->data;
    return (*a == *b) ? 0 : -1;
}

PyObject *
PyStreamlineAttributes_getattr(PyObject *self, char *name)
{
    if(strcmp(name, "sourceType") == 0)
        return StreamlineAttributes_GetSourceType(self, NULL);
    if(strcmp(name, "SpecifiedPoint") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::SpecifiedPoint));
    if(strcmp(name, "SpecifiedPointList") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::SpecifiedPointList));
    if(strcmp(name, "SpecifiedLine") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::SpecifiedLine));
    if(strcmp(name, "SpecifiedCircle") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::SpecifiedCircle));
    if(strcmp(name, "SpecifiedPlane") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::SpecifiedPlane));
    if(strcmp(name, "SpecifiedSphere") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::SpecifiedSphere));
    if(strcmp(name, "SpecifiedBox") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::SpecifiedBox));
    if(strcmp(name, "Selection") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Selection));

    if(strcmp(name, "pointSource") == 0)
        return StreamlineAttributes_GetPointSource(self, NULL);
    if(strcmp(name, "lineStart") == 0)
        return StreamlineAttributes_GetLineStart(self, NULL);
    if(strcmp(name, "lineEnd") == 0)
        return StreamlineAttributes_GetLineEnd(self, NULL);
    if(strcmp(name, "planeOrigin") == 0)
        return StreamlineAttributes_GetPlaneOrigin(self, NULL);
    if(strcmp(name, "planeNormal") == 0)
        return StreamlineAttributes_GetPlaneNormal(self, NULL);
    if(strcmp(name, "planeUpAxis") == 0)
        return StreamlineAttributes_GetPlaneUpAxis(self, NULL);
    if(strcmp(name, "radius") == 0)
        return StreamlineAttributes_GetRadius(self, NULL);
    if(strcmp(name, "sphereOrigin") == 0)
        return StreamlineAttributes_GetSphereOrigin(self, NULL);
    if(strcmp(name, "boxExtents") == 0)
        return StreamlineAttributes_GetBoxExtents(self, NULL);
    if(strcmp(name, "useWholeBox") == 0)
        return StreamlineAttributes_GetUseWholeBox(self, NULL);
    if(strcmp(name, "pointList") == 0)
        return StreamlineAttributes_GetPointList(self, NULL);
    if(strcmp(name, "sampleDensity0") == 0)
        return StreamlineAttributes_GetSampleDensity0(self, NULL);
    if(strcmp(name, "sampleDensity1") == 0)
        return StreamlineAttributes_GetSampleDensity1(self, NULL);
    if(strcmp(name, "sampleDensity2") == 0)
        return StreamlineAttributes_GetSampleDensity2(self, NULL);
    if(strcmp(name, "coloringMethod") == 0)
        return StreamlineAttributes_GetColoringMethod(self, NULL);
    if(strcmp(name, "Solid") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Solid));
    if(strcmp(name, "ColorBySpeed") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::ColorBySpeed));
    if(strcmp(name, "ColorByVorticity") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::ColorByVorticity));
    if(strcmp(name, "ColorByLength") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::ColorByLength));
    if(strcmp(name, "ColorByTime") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::ColorByTime));
    if(strcmp(name, "ColorBySeedPointID") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::ColorBySeedPointID));
    if(strcmp(name, "ColorByVariable") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::ColorByVariable));
    if(strcmp(name, "ColorByCorrelationDistance") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::ColorByCorrelationDistance));

    if(strcmp(name, "colorTableName") == 0)
        return StreamlineAttributes_GetColorTableName(self, NULL);
    if(strcmp(name, "singleColor") == 0)
        return StreamlineAttributes_GetSingleColor(self, NULL);
    if(strcmp(name, "legendFlag") == 0)
        return StreamlineAttributes_GetLegendFlag(self, NULL);
    if(strcmp(name, "lightingFlag") == 0)
        return StreamlineAttributes_GetLightingFlag(self, NULL);
    if(strcmp(name, "integrationDirection") == 0)
        return StreamlineAttributes_GetIntegrationDirection(self, NULL);
    if(strcmp(name, "Forward") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Forward));
    if(strcmp(name, "Backward") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Backward));
    if(strcmp(name, "Both") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Both));

    if(strcmp(name, "maxSteps") == 0)
        return StreamlineAttributes_GetMaxSteps(self, NULL);
    if(strcmp(name, "terminateByDistance") == 0)
        return StreamlineAttributes_GetTerminateByDistance(self, NULL);
    if(strcmp(name, "termDistance") == 0)
        return StreamlineAttributes_GetTermDistance(self, NULL);
    if(strcmp(name, "terminateByTime") == 0)
        return StreamlineAttributes_GetTerminateByTime(self, NULL);
    if(strcmp(name, "termTime") == 0)
        return StreamlineAttributes_GetTermTime(self, NULL);
    if(strcmp(name, "maxStepLength") == 0)
        return StreamlineAttributes_GetMaxStepLength(self, NULL);
    if(strcmp(name, "limitMaximumTimestep") == 0)
        return StreamlineAttributes_GetLimitMaximumTimestep(self, NULL);
    if(strcmp(name, "maxTimeStep") == 0)
        return StreamlineAttributes_GetMaxTimeStep(self, NULL);
    if(strcmp(name, "relTol") == 0)
        return StreamlineAttributes_GetRelTol(self, NULL);
    if(strcmp(name, "absTolSizeType") == 0)
        return StreamlineAttributes_GetAbsTolSizeType(self, NULL);
    if(strcmp(name, "Absolute") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Absolute));
    if(strcmp(name, "FractionOfBBox") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::FractionOfBBox));

    if(strcmp(name, "absTolAbsolute") == 0)
        return StreamlineAttributes_GetAbsTolAbsolute(self, NULL);
    if(strcmp(name, "absTolBBox") == 0)
        return StreamlineAttributes_GetAbsTolBBox(self, NULL);
    if(strcmp(name, "fieldType") == 0)
        return StreamlineAttributes_GetFieldType(self, NULL);
    if(strcmp(name, "Default") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Default));
    if(strcmp(name, "FlashField") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::FlashField));
    if(strcmp(name, "M3DC12DField") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::M3DC12DField));
    if(strcmp(name, "M3DC13DField") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::M3DC13DField));
    if(strcmp(name, "Nek5000Field") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Nek5000Field));
    if(strcmp(name, "NIMRODField") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::NIMRODField));

    if(strcmp(name, "fieldConstant") == 0)
        return StreamlineAttributes_GetFieldConstant(self, NULL);
    if(strcmp(name, "velocitySource") == 0)
        return StreamlineAttributes_GetVelocitySource(self, NULL);
    if(strcmp(name, "integrationType") == 0)
        return StreamlineAttributes_GetIntegrationType(self, NULL);
    if(strcmp(name, "Euler") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Euler));
    if(strcmp(name, "Leapfrog") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Leapfrog));
    if(strcmp(name, "DormandPrince") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::DormandPrince));
    if(strcmp(name, "AdamsBashforth") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::AdamsBashforth));
    if(strcmp(name, "RK4") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::RK4));
    if(strcmp(name, "M3DC12DIntegrator") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::M3DC12DIntegrator));

    if(strcmp(name, "parallelizationAlgorithmType") == 0)
        return StreamlineAttributes_GetParallelizationAlgorithmType(self, NULL);
    if(strcmp(name, "LoadOnDemand") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::LoadOnDemand));
    if(strcmp(name, "ParallelStaticDomains") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::ParallelStaticDomains));
    if(strcmp(name, "MasterSlave") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::MasterSlave));
    if(strcmp(name, "VisItSelects") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::VisItSelects));

    if(strcmp(name, "maxProcessCount") == 0)
        return StreamlineAttributes_GetMaxProcessCount(self, NULL);
    if(strcmp(name, "maxDomainCacheSize") == 0)
        return StreamlineAttributes_GetMaxDomainCacheSize(self, NULL);
    if(strcmp(name, "workGroupSize") == 0)
        return StreamlineAttributes_GetWorkGroupSize(self, NULL);
    if(strcmp(name, "pathlines") == 0)
        return StreamlineAttributes_GetPathlines(self, NULL);
    if(strcmp(name, "pathlinesOverrideStartingTimeFlag") == 0)
        return StreamlineAttributes_GetPathlinesOverrideStartingTimeFlag(self, NULL);
    if(strcmp(name, "pathlinesOverrideStartingTime") == 0)
        return StreamlineAttributes_GetPathlinesOverrideStartingTime(self, NULL);
    if(strcmp(name, "pathlinesPeriod") == 0)
        return StreamlineAttributes_GetPathlinesPeriod(self, NULL);
    if(strcmp(name, "pathlinesCMFE") == 0)
        return StreamlineAttributes_GetPathlinesCMFE(self, NULL);
    if(strcmp(name, "CONN_CMFE") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::CONN_CMFE));
    if(strcmp(name, "POS_CMFE") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::POS_CMFE));

    if(strcmp(name, "coordinateSystem") == 0)
        return StreamlineAttributes_GetCoordinateSystem(self, NULL);
    if(strcmp(name, "AsIs") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::AsIs));
    if(strcmp(name, "CylindricalToCartesian") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::CylindricalToCartesian));
    if(strcmp(name, "CartesianToCylindrical") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::CartesianToCylindrical));

    if(strcmp(name, "phiScalingFlag") == 0)
        return StreamlineAttributes_GetPhiScalingFlag(self, NULL);
    if(strcmp(name, "phiScaling") == 0)
        return StreamlineAttributes_GetPhiScaling(self, NULL);
    if(strcmp(name, "coloringVariable") == 0)
        return StreamlineAttributes_GetColoringVariable(self, NULL);
    if(strcmp(name, "legendMinFlag") == 0)
        return StreamlineAttributes_GetLegendMinFlag(self, NULL);
    if(strcmp(name, "legendMaxFlag") == 0)
        return StreamlineAttributes_GetLegendMaxFlag(self, NULL);
    if(strcmp(name, "legendMin") == 0)
        return StreamlineAttributes_GetLegendMin(self, NULL);
    if(strcmp(name, "legendMax") == 0)
        return StreamlineAttributes_GetLegendMax(self, NULL);
    if(strcmp(name, "displayBegin") == 0)
        return StreamlineAttributes_GetDisplayBegin(self, NULL);
    if(strcmp(name, "displayEnd") == 0)
        return StreamlineAttributes_GetDisplayEnd(self, NULL);
    if(strcmp(name, "displayBeginFlag") == 0)
        return StreamlineAttributes_GetDisplayBeginFlag(self, NULL);
    if(strcmp(name, "displayEndFlag") == 0)
        return StreamlineAttributes_GetDisplayEndFlag(self, NULL);
    if(strcmp(name, "referenceTypeForDisplay") == 0)
        return StreamlineAttributes_GetReferenceTypeForDisplay(self, NULL);
    if(strcmp(name, "Distance") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Distance));
    if(strcmp(name, "Time") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Time));
    if(strcmp(name, "Step") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Step));

    if(strcmp(name, "displayMethod") == 0)
        return StreamlineAttributes_GetDisplayMethod(self, NULL);
    if(strcmp(name, "Lines") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Lines));
    if(strcmp(name, "Tubes") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Tubes));
    if(strcmp(name, "Ribbons") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Ribbons));

    if(strcmp(name, "tubeSizeType") == 0)
        return StreamlineAttributes_GetTubeSizeType(self, NULL);
    if(strcmp(name, "Absolute") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Absolute));
    if(strcmp(name, "FractionOfBBox") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::FractionOfBBox));

    if(strcmp(name, "tubeRadiusAbsolute") == 0)
        return StreamlineAttributes_GetTubeRadiusAbsolute(self, NULL);
    if(strcmp(name, "tubeRadiusBBox") == 0)
        return StreamlineAttributes_GetTubeRadiusBBox(self, NULL);
    if(strcmp(name, "ribbonWidthSizeType") == 0)
        return StreamlineAttributes_GetRibbonWidthSizeType(self, NULL);
    if(strcmp(name, "Absolute") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Absolute));
    if(strcmp(name, "FractionOfBBox") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::FractionOfBBox));

    if(strcmp(name, "ribbonWidthAbsolute") == 0)
        return StreamlineAttributes_GetRibbonWidthAbsolute(self, NULL);
    if(strcmp(name, "ribbonWidthBBox") == 0)
        return StreamlineAttributes_GetRibbonWidthBBox(self, NULL);
    if(strcmp(name, "lineWidth") == 0)
        return StreamlineAttributes_GetLineWidth(self, NULL);
    if(strcmp(name, "showSeeds") == 0)
        return StreamlineAttributes_GetShowSeeds(self, NULL);
    if(strcmp(name, "seedRadiusSizeType") == 0)
        return StreamlineAttributes_GetSeedRadiusSizeType(self, NULL);
    if(strcmp(name, "Absolute") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Absolute));
    if(strcmp(name, "FractionOfBBox") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::FractionOfBBox));

    if(strcmp(name, "seedRadiusAbsolute") == 0)
        return StreamlineAttributes_GetSeedRadiusAbsolute(self, NULL);
    if(strcmp(name, "seedRadiusBBox") == 0)
        return StreamlineAttributes_GetSeedRadiusBBox(self, NULL);
    if(strcmp(name, "showHeads") == 0)
        return StreamlineAttributes_GetShowHeads(self, NULL);
    if(strcmp(name, "headDisplayType") == 0)
        return StreamlineAttributes_GetHeadDisplayType(self, NULL);
    if(strcmp(name, "Sphere") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Sphere));
    if(strcmp(name, "Cone") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Cone));

    if(strcmp(name, "headRadiusSizeType") == 0)
        return StreamlineAttributes_GetHeadRadiusSizeType(self, NULL);
    if(strcmp(name, "Absolute") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Absolute));
    if(strcmp(name, "FractionOfBBox") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::FractionOfBBox));

    if(strcmp(name, "headRadiusAbsolute") == 0)
        return StreamlineAttributes_GetHeadRadiusAbsolute(self, NULL);
    if(strcmp(name, "headRadiusBBox") == 0)
        return StreamlineAttributes_GetHeadRadiusBBox(self, NULL);
    if(strcmp(name, "headHeightRatio") == 0)
        return StreamlineAttributes_GetHeadHeightRatio(self, NULL);
    if(strcmp(name, "opacityType") == 0)
        return StreamlineAttributes_GetOpacityType(self, NULL);
    if(strcmp(name, "FullyOpaque") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::FullyOpaque));
    if(strcmp(name, "Constant") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Constant));
    if(strcmp(name, "Ramp") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Ramp));
    if(strcmp(name, "VariableRange") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::VariableRange));

    if(strcmp(name, "opacityVariable") == 0)
        return StreamlineAttributes_GetOpacityVariable(self, NULL);
    if(strcmp(name, "opacity") == 0)
        return StreamlineAttributes_GetOpacity(self, NULL);
    if(strcmp(name, "opacityVarMin") == 0)
        return StreamlineAttributes_GetOpacityVarMin(self, NULL);
    if(strcmp(name, "opacityVarMax") == 0)
        return StreamlineAttributes_GetOpacityVarMax(self, NULL);
    if(strcmp(name, "opacityVarMinFlag") == 0)
        return StreamlineAttributes_GetOpacityVarMinFlag(self, NULL);
    if(strcmp(name, "opacityVarMaxFlag") == 0)
        return StreamlineAttributes_GetOpacityVarMaxFlag(self, NULL);
    if(strcmp(name, "tubeDisplayDensity") == 0)
        return StreamlineAttributes_GetTubeDisplayDensity(self, NULL);
    if(strcmp(name, "geomDisplayQuality") == 0)
        return StreamlineAttributes_GetGeomDisplayQuality(self, NULL);
    if(strcmp(name, "Low") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Low));
    if(strcmp(name, "Medium") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Medium));
    if(strcmp(name, "High") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::High));
    if(strcmp(name, "Super") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Super));

    if(strcmp(name, "sampleDistance0") == 0)
        return StreamlineAttributes_GetSampleDistance0(self, NULL);
    if(strcmp(name, "sampleDistance1") == 0)
        return StreamlineAttributes_GetSampleDistance1(self, NULL);
    if(strcmp(name, "sampleDistance2") == 0)
        return StreamlineAttributes_GetSampleDistance2(self, NULL);
    if(strcmp(name, "fillInterior") == 0)
        return StreamlineAttributes_GetFillInterior(self, NULL);
    if(strcmp(name, "randomSamples") == 0)
        return StreamlineAttributes_GetRandomSamples(self, NULL);
    if(strcmp(name, "randomSeed") == 0)
        return StreamlineAttributes_GetRandomSeed(self, NULL);
    if(strcmp(name, "numberOfRandomSamples") == 0)
        return StreamlineAttributes_GetNumberOfRandomSamples(self, NULL);
    if(strcmp(name, "forceNodeCenteredData") == 0)
        return StreamlineAttributes_GetForceNodeCenteredData(self, NULL);
    if(strcmp(name, "issueTerminationWarnings") == 0)
        return StreamlineAttributes_GetIssueTerminationWarnings(self, NULL);
    if(strcmp(name, "issueStiffnessWarnings") == 0)
        return StreamlineAttributes_GetIssueStiffnessWarnings(self, NULL);
    if(strcmp(name, "issueCriticalPointsWarnings") == 0)
        return StreamlineAttributes_GetIssueCriticalPointsWarnings(self, NULL);
    if(strcmp(name, "criticalPointThreshold") == 0)
        return StreamlineAttributes_GetCriticalPointThreshold(self, NULL);
    if(strcmp(name, "varyTubeRadius") == 0)
        return StreamlineAttributes_GetVaryTubeRadius(self, NULL);
    if(strcmp(name, "None") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::None));
    if(strcmp(name, "Scalar") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Scalar));

    if(strcmp(name, "varyTubeRadiusFactor") == 0)
        return StreamlineAttributes_GetVaryTubeRadiusFactor(self, NULL);
    if(strcmp(name, "varyTubeRadiusVariable") == 0)
        return StreamlineAttributes_GetVaryTubeRadiusVariable(self, NULL);
    if(strcmp(name, "correlationDistanceAngTol") == 0)
        return StreamlineAttributes_GetCorrelationDistanceAngTol(self, NULL);
    if(strcmp(name, "correlationDistanceMinDistAbsolute") == 0)
        return StreamlineAttributes_GetCorrelationDistanceMinDistAbsolute(self, NULL);
    if(strcmp(name, "correlationDistanceMinDistBBox") == 0)
        return StreamlineAttributes_GetCorrelationDistanceMinDistBBox(self, NULL);
    if(strcmp(name, "correlationDistanceMinDistType") == 0)
        return StreamlineAttributes_GetCorrelationDistanceMinDistType(self, NULL);
    if(strcmp(name, "Absolute") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::Absolute));
    if(strcmp(name, "FractionOfBBox") == 0)
        return PyInt_FromLong(long(StreamlineAttributes::FractionOfBBox));

    if(strcmp(name, "selection") == 0)
        return StreamlineAttributes_GetSelection(self, NULL);

    return Py_FindMethod(PyStreamlineAttributes_methods, self, name);
}

int
PyStreamlineAttributes_setattr(PyObject *self, char *name, PyObject *args)
{
    // Create a tuple to contain the arguments since all of the Set
    // functions expect a tuple.
    PyObject *tuple = PyTuple_New(1);
    PyTuple_SET_ITEM(tuple, 0, args);
    Py_INCREF(args);
    PyObject *obj = NULL;

    if(strcmp(name, "sourceType") == 0)
        obj = StreamlineAttributes_SetSourceType(self, tuple);
    else if(strcmp(name, "pointSource") == 0)
        obj = StreamlineAttributes_SetPointSource(self, tuple);
    else if(strcmp(name, "lineStart") == 0)
        obj = StreamlineAttributes_SetLineStart(self, tuple);
    else if(strcmp(name, "lineEnd") == 0)
        obj = StreamlineAttributes_SetLineEnd(self, tuple);
    else if(strcmp(name, "planeOrigin") == 0)
        obj = StreamlineAttributes_SetPlaneOrigin(self, tuple);
    else if(strcmp(name, "planeNormal") == 0)
        obj = StreamlineAttributes_SetPlaneNormal(self, tuple);
    else if(strcmp(name, "planeUpAxis") == 0)
        obj = StreamlineAttributes_SetPlaneUpAxis(self, tuple);
    else if(strcmp(name, "radius") == 0)
        obj = StreamlineAttributes_SetRadius(self, tuple);
    else if(strcmp(name, "sphereOrigin") == 0)
        obj = StreamlineAttributes_SetSphereOrigin(self, tuple);
    else if(strcmp(name, "boxExtents") == 0)
        obj = StreamlineAttributes_SetBoxExtents(self, tuple);
    else if(strcmp(name, "useWholeBox") == 0)
        obj = StreamlineAttributes_SetUseWholeBox(self, tuple);
    else if(strcmp(name, "pointList") == 0)
        obj = StreamlineAttributes_SetPointList(self, tuple);
    else if(strcmp(name, "sampleDensity0") == 0)
        obj = StreamlineAttributes_SetSampleDensity0(self, tuple);
    else if(strcmp(name, "sampleDensity1") == 0)
        obj = StreamlineAttributes_SetSampleDensity1(self, tuple);
    else if(strcmp(name, "sampleDensity2") == 0)
        obj = StreamlineAttributes_SetSampleDensity2(self, tuple);
    else if(strcmp(name, "coloringMethod") == 0)
        obj = StreamlineAttributes_SetColoringMethod(self, tuple);
    else if(strcmp(name, "colorTableName") == 0)
        obj = StreamlineAttributes_SetColorTableName(self, tuple);
    else if(strcmp(name, "singleColor") == 0)
        obj = StreamlineAttributes_SetSingleColor(self, tuple);
    else if(strcmp(name, "legendFlag") == 0)
        obj = StreamlineAttributes_SetLegendFlag(self, tuple);
    else if(strcmp(name, "lightingFlag") == 0)
        obj = StreamlineAttributes_SetLightingFlag(self, tuple);
    else if(strcmp(name, "integrationDirection") == 0)
        obj = StreamlineAttributes_SetIntegrationDirection(self, tuple);
    else if(strcmp(name, "maxSteps") == 0)
        obj = StreamlineAttributes_SetMaxSteps(self, tuple);
    else if(strcmp(name, "terminateByDistance") == 0)
        obj = StreamlineAttributes_SetTerminateByDistance(self, tuple);
    else if(strcmp(name, "termDistance") == 0)
        obj = StreamlineAttributes_SetTermDistance(self, tuple);
    else if(strcmp(name, "terminateByTime") == 0)
        obj = StreamlineAttributes_SetTerminateByTime(self, tuple);
    else if(strcmp(name, "termTime") == 0)
        obj = StreamlineAttributes_SetTermTime(self, tuple);
    else if(strcmp(name, "maxStepLength") == 0)
        obj = StreamlineAttributes_SetMaxStepLength(self, tuple);
    else if(strcmp(name, "limitMaximumTimestep") == 0)
        obj = StreamlineAttributes_SetLimitMaximumTimestep(self, tuple);
    else if(strcmp(name, "maxTimeStep") == 0)
        obj = StreamlineAttributes_SetMaxTimeStep(self, tuple);
    else if(strcmp(name, "relTol") == 0)
        obj = StreamlineAttributes_SetRelTol(self, tuple);
    else if(strcmp(name, "absTolSizeType") == 0)
        obj = StreamlineAttributes_SetAbsTolSizeType(self, tuple);
    else if(strcmp(name, "absTolAbsolute") == 0)
        obj = StreamlineAttributes_SetAbsTolAbsolute(self, tuple);
    else if(strcmp(name, "absTolBBox") == 0)
        obj = StreamlineAttributes_SetAbsTolBBox(self, tuple);
    else if(strcmp(name, "fieldType") == 0)
        obj = StreamlineAttributes_SetFieldType(self, tuple);
    else if(strcmp(name, "fieldConstant") == 0)
        obj = StreamlineAttributes_SetFieldConstant(self, tuple);
    else if(strcmp(name, "velocitySource") == 0)
        obj = StreamlineAttributes_SetVelocitySource(self, tuple);
    else if(strcmp(name, "integrationType") == 0)
        obj = StreamlineAttributes_SetIntegrationType(self, tuple);
    else if(strcmp(name, "parallelizationAlgorithmType") == 0)
        obj = StreamlineAttributes_SetParallelizationAlgorithmType(self, tuple);
    else if(strcmp(name, "maxProcessCount") == 0)
        obj = StreamlineAttributes_SetMaxProcessCount(self, tuple);
    else if(strcmp(name, "maxDomainCacheSize") == 0)
        obj = StreamlineAttributes_SetMaxDomainCacheSize(self, tuple);
    else if(strcmp(name, "workGroupSize") == 0)
        obj = StreamlineAttributes_SetWorkGroupSize(self, tuple);
    else if(strcmp(name, "pathlines") == 0)
        obj = StreamlineAttributes_SetPathlines(self, tuple);
    else if(strcmp(name, "pathlinesOverrideStartingTimeFlag") == 0)
        obj = StreamlineAttributes_SetPathlinesOverrideStartingTimeFlag(self, tuple);
    else if(strcmp(name, "pathlinesOverrideStartingTime") == 0)
        obj = StreamlineAttributes_SetPathlinesOverrideStartingTime(self, tuple);
    else if(strcmp(name, "pathlinesPeriod") == 0)
        obj = StreamlineAttributes_SetPathlinesPeriod(self, tuple);
    else if(strcmp(name, "pathlinesCMFE") == 0)
        obj = StreamlineAttributes_SetPathlinesCMFE(self, tuple);
    else if(strcmp(name, "coordinateSystem") == 0)
        obj = StreamlineAttributes_SetCoordinateSystem(self, tuple);
    else if(strcmp(name, "phiScalingFlag") == 0)
        obj = StreamlineAttributes_SetPhiScalingFlag(self, tuple);
    else if(strcmp(name, "phiScaling") == 0)
        obj = StreamlineAttributes_SetPhiScaling(self, tuple);
    else if(strcmp(name, "coloringVariable") == 0)
        obj = StreamlineAttributes_SetColoringVariable(self, tuple);
    else if(strcmp(name, "legendMinFlag") == 0)
        obj = StreamlineAttributes_SetLegendMinFlag(self, tuple);
    else if(strcmp(name, "legendMaxFlag") == 0)
        obj = StreamlineAttributes_SetLegendMaxFlag(self, tuple);
    else if(strcmp(name, "legendMin") == 0)
        obj = StreamlineAttributes_SetLegendMin(self, tuple);
    else if(strcmp(name, "legendMax") == 0)
        obj = StreamlineAttributes_SetLegendMax(self, tuple);
    else if(strcmp(name, "displayBegin") == 0)
        obj = StreamlineAttributes_SetDisplayBegin(self, tuple);
    else if(strcmp(name, "displayEnd") == 0)
        obj = StreamlineAttributes_SetDisplayEnd(self, tuple);
    else if(strcmp(name, "displayBeginFlag") == 0)
        obj = StreamlineAttributes_SetDisplayBeginFlag(self, tuple);
    else if(strcmp(name, "displayEndFlag") == 0)
        obj = StreamlineAttributes_SetDisplayEndFlag(self, tuple);
    else if(strcmp(name, "referenceTypeForDisplay") == 0)
        obj = StreamlineAttributes_SetReferenceTypeForDisplay(self, tuple);
    else if(strcmp(name, "displayMethod") == 0)
        obj = StreamlineAttributes_SetDisplayMethod(self, tuple);
    else if(strcmp(name, "tubeSizeType") == 0)
        obj = StreamlineAttributes_SetTubeSizeType(self, tuple);
    else if(strcmp(name, "tubeRadiusAbsolute") == 0)
        obj = StreamlineAttributes_SetTubeRadiusAbsolute(self, tuple);
    else if(strcmp(name, "tubeRadiusBBox") == 0)
        obj = StreamlineAttributes_SetTubeRadiusBBox(self, tuple);
    else if(strcmp(name, "ribbonWidthSizeType") == 0)
        obj = StreamlineAttributes_SetRibbonWidthSizeType(self, tuple);
    else if(strcmp(name, "ribbonWidthAbsolute") == 0)
        obj = StreamlineAttributes_SetRibbonWidthAbsolute(self, tuple);
    else if(strcmp(name, "ribbonWidthBBox") == 0)
        obj = StreamlineAttributes_SetRibbonWidthBBox(self, tuple);
    else if(strcmp(name, "lineWidth") == 0)
        obj = StreamlineAttributes_SetLineWidth(self, tuple);
    else if(strcmp(name, "showSeeds") == 0)
        obj = StreamlineAttributes_SetShowSeeds(self, tuple);
    else if(strcmp(name, "seedRadiusSizeType") == 0)
        obj = StreamlineAttributes_SetSeedRadiusSizeType(self, tuple);
    else if(strcmp(name, "seedRadiusAbsolute") == 0)
        obj = StreamlineAttributes_SetSeedRadiusAbsolute(self, tuple);
    else if(strcmp(name, "seedRadiusBBox") == 0)
        obj = StreamlineAttributes_SetSeedRadiusBBox(self, tuple);
    else if(strcmp(name, "showHeads") == 0)
        obj = StreamlineAttributes_SetShowHeads(self, tuple);
    else if(strcmp(name, "headDisplayType") == 0)
        obj = StreamlineAttributes_SetHeadDisplayType(self, tuple);
    else if(strcmp(name, "headRadiusSizeType") == 0)
        obj = StreamlineAttributes_SetHeadRadiusSizeType(self, tuple);
    else if(strcmp(name, "headRadiusAbsolute") == 0)
        obj = StreamlineAttributes_SetHeadRadiusAbsolute(self, tuple);
    else if(strcmp(name, "headRadiusBBox") == 0)
        obj = StreamlineAttributes_SetHeadRadiusBBox(self, tuple);
    else if(strcmp(name, "headHeightRatio") == 0)
        obj = StreamlineAttributes_SetHeadHeightRatio(self, tuple);
    else if(strcmp(name, "opacityType") == 0)
        obj = StreamlineAttributes_SetOpacityType(self, tuple);
    else if(strcmp(name, "opacityVariable") == 0)
        obj = StreamlineAttributes_SetOpacityVariable(self, tuple);
    else if(strcmp(name, "opacity") == 0)
        obj = StreamlineAttributes_SetOpacity(self, tuple);
    else if(strcmp(name, "opacityVarMin") == 0)
        obj = StreamlineAttributes_SetOpacityVarMin(self, tuple);
    else if(strcmp(name, "opacityVarMax") == 0)
        obj = StreamlineAttributes_SetOpacityVarMax(self, tuple);
    else if(strcmp(name, "opacityVarMinFlag") == 0)
        obj = StreamlineAttributes_SetOpacityVarMinFlag(self, tuple);
    else if(strcmp(name, "opacityVarMaxFlag") == 0)
        obj = StreamlineAttributes_SetOpacityVarMaxFlag(self, tuple);
    else if(strcmp(name, "tubeDisplayDensity") == 0)
        obj = StreamlineAttributes_SetTubeDisplayDensity(self, tuple);
    else if(strcmp(name, "geomDisplayQuality") == 0)
        obj = StreamlineAttributes_SetGeomDisplayQuality(self, tuple);
    else if(strcmp(name, "sampleDistance0") == 0)
        obj = StreamlineAttributes_SetSampleDistance0(self, tuple);
    else if(strcmp(name, "sampleDistance1") == 0)
        obj = StreamlineAttributes_SetSampleDistance1(self, tuple);
    else if(strcmp(name, "sampleDistance2") == 0)
        obj = StreamlineAttributes_SetSampleDistance2(self, tuple);
    else if(strcmp(name, "fillInterior") == 0)
        obj = StreamlineAttributes_SetFillInterior(self, tuple);
    else if(strcmp(name, "randomSamples") == 0)
        obj = StreamlineAttributes_SetRandomSamples(self, tuple);
    else if(strcmp(name, "randomSeed") == 0)
        obj = StreamlineAttributes_SetRandomSeed(self, tuple);
    else if(strcmp(name, "numberOfRandomSamples") == 0)
        obj = StreamlineAttributes_SetNumberOfRandomSamples(self, tuple);
    else if(strcmp(name, "forceNodeCenteredData") == 0)
        obj = StreamlineAttributes_SetForceNodeCenteredData(self, tuple);
    else if(strcmp(name, "issueTerminationWarnings") == 0)
        obj = StreamlineAttributes_SetIssueTerminationWarnings(self, tuple);
    else if(strcmp(name, "issueStiffnessWarnings") == 0)
        obj = StreamlineAttributes_SetIssueStiffnessWarnings(self, tuple);
    else if(strcmp(name, "issueCriticalPointsWarnings") == 0)
        obj = StreamlineAttributes_SetIssueCriticalPointsWarnings(self, tuple);
    else if(strcmp(name, "criticalPointThreshold") == 0)
        obj = StreamlineAttributes_SetCriticalPointThreshold(self, tuple);
    else if(strcmp(name, "varyTubeRadius") == 0)
        obj = StreamlineAttributes_SetVaryTubeRadius(self, tuple);
    else if(strcmp(name, "varyTubeRadiusFactor") == 0)
        obj = StreamlineAttributes_SetVaryTubeRadiusFactor(self, tuple);
    else if(strcmp(name, "varyTubeRadiusVariable") == 0)
        obj = StreamlineAttributes_SetVaryTubeRadiusVariable(self, tuple);
    else if(strcmp(name, "correlationDistanceAngTol") == 0)
        obj = StreamlineAttributes_SetCorrelationDistanceAngTol(self, tuple);
    else if(strcmp(name, "correlationDistanceMinDistAbsolute") == 0)
        obj = StreamlineAttributes_SetCorrelationDistanceMinDistAbsolute(self, tuple);
    else if(strcmp(name, "correlationDistanceMinDistBBox") == 0)
        obj = StreamlineAttributes_SetCorrelationDistanceMinDistBBox(self, tuple);
    else if(strcmp(name, "correlationDistanceMinDistType") == 0)
        obj = StreamlineAttributes_SetCorrelationDistanceMinDistType(self, tuple);
    else if(strcmp(name, "selection") == 0)
        obj = StreamlineAttributes_SetSelection(self, tuple);

    if(obj != NULL)
        Py_DECREF(obj);

    Py_DECREF(tuple);
    if( obj == NULL)
        PyErr_Format(PyExc_RuntimeError, "Unable to set unknown attribute: '%s'", name);
    return (obj != NULL) ? 0 : -1;
}

static int
StreamlineAttributes_print(PyObject *v, FILE *fp, int flags)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)v;
    fprintf(fp, "%s", PyStreamlineAttributes_ToString(obj->data, "").c_str());
    return 0;
}

PyObject *
StreamlineAttributes_str(PyObject *v)
{
    StreamlineAttributesObject *obj = (StreamlineAttributesObject *)v;
    return PyString_FromString(PyStreamlineAttributes_ToString(obj->data,"").c_str());
}

//
// The doc string for the class.
//
#if PY_MAJOR_VERSION > 2 || (PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION >= 5)
static const char *StreamlineAttributes_Purpose = "Attributes for the Streamline plot";
#else
static char *StreamlineAttributes_Purpose = "Attributes for the Streamline plot";
#endif

//
// The type description structure
//
static PyTypeObject StreamlineAttributesType =
{
    //
    // Type header
    //
    PyObject_HEAD_INIT(&PyType_Type)
    0,                                   // ob_size
    "StreamlineAttributes",                    // tp_name
    sizeof(StreamlineAttributesObject),        // tp_basicsize
    0,                                   // tp_itemsize
    //
    // Standard methods
    //
    (destructor)StreamlineAttributes_dealloc,  // tp_dealloc
    (printfunc)StreamlineAttributes_print,     // tp_print
    (getattrfunc)PyStreamlineAttributes_getattr, // tp_getattr
    (setattrfunc)PyStreamlineAttributes_setattr, // tp_setattr
    (cmpfunc)StreamlineAttributes_compare,     // tp_compare
    (reprfunc)0,                         // tp_repr
    //
    // Type categories
    //
    0,                                   // tp_as_number
    0,                                   // tp_as_sequence
    0,                                   // tp_as_mapping
    //
    // More methods
    //
    0,                                   // tp_hash
    0,                                   // tp_call
    (reprfunc)StreamlineAttributes_str,        // tp_str
    0,                                   // tp_getattro
    0,                                   // tp_setattro
    0,                                   // tp_as_buffer
    Py_TPFLAGS_CHECKTYPES,               // tp_flags
    StreamlineAttributes_Purpose,              // tp_doc
    0,                                   // tp_traverse
    0,                                   // tp_clear
    0,                                   // tp_richcompare
    0                                    // tp_weaklistoffset
};

//
// Helper functions for object allocation.
//

static StreamlineAttributes *defaultAtts = 0;
static StreamlineAttributes *currentAtts = 0;

static PyObject *
NewStreamlineAttributes(int useCurrent)
{
    StreamlineAttributesObject *newObject;
    newObject = PyObject_NEW(StreamlineAttributesObject, &StreamlineAttributesType);
    if(newObject == NULL)
        return NULL;
    if(useCurrent && currentAtts != 0)
        newObject->data = new StreamlineAttributes(*currentAtts);
    else if(defaultAtts != 0)
        newObject->data = new StreamlineAttributes(*defaultAtts);
    else
        newObject->data = new StreamlineAttributes;
    newObject->owns = true;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

static PyObject *
WrapStreamlineAttributes(const StreamlineAttributes *attr)
{
    StreamlineAttributesObject *newObject;
    newObject = PyObject_NEW(StreamlineAttributesObject, &StreamlineAttributesType);
    if(newObject == NULL)
        return NULL;
    newObject->data = (StreamlineAttributes *)attr;
    newObject->owns = false;
    newObject->parent = 0;
    return (PyObject *)newObject;
}

///////////////////////////////////////////////////////////////////////////////
//
// Interface that is exposed to the VisIt module.
//
///////////////////////////////////////////////////////////////////////////////

PyObject *
StreamlineAttributes_new(PyObject *self, PyObject *args)
{
    int useCurrent = 0;
    if (!PyArg_ParseTuple(args, "i", &useCurrent))
    {
        if (!PyArg_ParseTuple(args, ""))
            return NULL;
        else
            PyErr_Clear();
    }

    return (PyObject *)NewStreamlineAttributes(useCurrent);
}

//
// Plugin method table. These methods are added to the visitmodule's methods.
//
static PyMethodDef StreamlineAttributesMethods[] = {
    {"StreamlineAttributes", StreamlineAttributes_new, METH_VARARGS},
    {NULL,      NULL}        /* Sentinel */
};

static Observer *StreamlineAttributesObserver = 0;

std::string
PyStreamlineAttributes_GetLogString()
{
    std::string s("StreamlineAtts = StreamlineAttributes()\n");
    if(currentAtts != 0)
        s += PyStreamlineAttributes_ToString(currentAtts, "StreamlineAtts.");
    return s;
}

static void
PyStreamlineAttributes_CallLogRoutine(Subject *subj, void *data)
{
    typedef void (*logCallback)(const std::string &);
    logCallback cb = (logCallback)data;

    if(cb != 0)
    {
        std::string s("StreamlineAtts = StreamlineAttributes()\n");
        s += PyStreamlineAttributes_ToString(currentAtts, "StreamlineAtts.");
        cb(s);
    }
}

void
PyStreamlineAttributes_StartUp(StreamlineAttributes *subj, void *data)
{
    if(subj == 0)
        return;

    currentAtts = subj;
    PyStreamlineAttributes_SetDefaults(subj);

    //
    // Create the observer that will be notified when the attributes change.
    //
    if(StreamlineAttributesObserver == 0)
    {
        StreamlineAttributesObserver = new ObserverToCallback(subj,
            PyStreamlineAttributes_CallLogRoutine, (void *)data);
    }

}

void
PyStreamlineAttributes_CloseDown()
{
    delete defaultAtts;
    defaultAtts = 0;
    delete StreamlineAttributesObserver;
    StreamlineAttributesObserver = 0;
}

PyMethodDef *
PyStreamlineAttributes_GetMethodTable(int *nMethods)
{
    *nMethods = 1;
    return StreamlineAttributesMethods;
}

bool
PyStreamlineAttributes_Check(PyObject *obj)
{
    return (obj->ob_type == &StreamlineAttributesType);
}

StreamlineAttributes *
PyStreamlineAttributes_FromPyObject(PyObject *obj)
{
    StreamlineAttributesObject *obj2 = (StreamlineAttributesObject *)obj;
    return obj2->data;
}

PyObject *
PyStreamlineAttributes_New()
{
    return NewStreamlineAttributes(0);
}

PyObject *
PyStreamlineAttributes_Wrap(const StreamlineAttributes *attr)
{
    return WrapStreamlineAttributes(attr);
}

void
PyStreamlineAttributes_SetParent(PyObject *obj, PyObject *parent)
{
    StreamlineAttributesObject *obj2 = (StreamlineAttributesObject *)obj;
    obj2->parent = parent;
}

void
PyStreamlineAttributes_SetDefaults(const StreamlineAttributes *atts)
{
    if(defaultAtts)
        delete defaultAtts;

    defaultAtts = new StreamlineAttributes(*atts);
}

