/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#ifndef QVISTOPOLOGYPLOTWINDOW_H
#define QVISTOPOLOGYPLOTWINDOW_H

#include <QvisPostableWindowObserver.h>
#include <AttributeSubject.h>

class TopologyAttributes;
class QLabel;
class QCheckBox;
class QLineEdit;
class QSpinBox;
class QVBox;
class QButtonGroup;
class QvisColorTableButton;
class QvisOpacitySlider;
class QvisColorButton;
class QvisLineStyleWidget;
class QvisLineWidthWidget;
class QvisVariableButton;

// ****************************************************************************
// Class: QvisTopologyPlotWindow
//
// Purpose:
//    Defines QvisTopologyPlotWindow class.
//
// Notes:      Autogenerated by xml2window.
//
// Programmer: xml2window
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

class QvisTopologyPlotWindow : public QvisPostableWindowObserver
{
    Q_OBJECT
  public:
    QvisTopologyPlotWindow(const int type,
                         TopologyAttributes *subj,
                         const QString &caption = QString::null,
                         const QString &shortName = QString::null,
                         QvisNotepadArea *notepad = 0);
    virtual ~QvisTopologyPlotWindow();
    virtual void CreateWindowContents();
  public slots:
    virtual void apply();
    virtual void makeDefault();
    virtual void reset();
  protected:
    void UpdateWindow(bool doAll);
    void GetCurrentValues(int which_widget);
    void Apply(bool ignore = false);
  private slots:
    void lineWidthChanged(int style);
    void lineStyleChanged(int style);
    //writeHeaderCallback unknown for att (variable multiColor)
    void minOpacityChanged(int opacity, const void*);
    void minPlateauOpacityChanged(int opacity, const void*);
    void maxPlateauOpacityChanged(int opacity, const void*);
    void maxOpacityChanged(int opacity, const void*);
    void toleranceProcessText();
    void hitpercentProcessText();
  private:
    int plotType;
    QvisLineWidthWidget *lineWidth;
    QvisLineStyleWidget *lineStyle;
    //writeHeaderData unknown for att (variable multiColor)
    QvisOpacitySlider *minOpacity;
    QvisOpacitySlider *minPlateauOpacity;
    QvisOpacitySlider *maxPlateauOpacity;
    QvisOpacitySlider *maxOpacity;
    QLineEdit *tolerance;
    QLineEdit *hitpercent;
    QLabel *lineWidthLabel;
    QLabel *lineStyleLabel;
    QLabel *multiColorLabel;
    QLabel *minOpacityLabel;
    QLabel *minPlateauOpacityLabel;
    QLabel *maxPlateauOpacityLabel;
    QLabel *maxOpacityLabel;
    QLabel *toleranceLabel;
    QLabel *hitpercentLabel;

    TopologyAttributes *atts;
};



#endif
