/*****************************************************************************
*
* Copyright (c) 2000 - 2014, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#include <string.h>
#include <visitstream.h>
#include <silo.h>

// supress the following since silo uses char * in its API
#if defined(__clang__)
# pragma GCC diagnostic ignored "-Wdeprecated-writable-strings"
#elif defined(__GNUC__)
# pragma GCC diagnostic ignored "-Wwrite-strings"
#endif

#define NNODES 17890
#define NHEXES 7939
#define NBEAMS 4
#define NSHELLS 7060

struct node {
   int    nodeNumber;
   double x;
   double y;
   double z;
};

struct beam {
   int    elementNumber;
   int    materialNumber;
   int    nodes[3];
};

struct shell {
   int    elementNumber;
   int    materialNumber;
   int    nodes[4];
};

struct hex {
   int    elementNumber;
   int    materialNumber;
   int    nodes[8];
};

struct node nodes[NNODES] = {
 {       1, .8457714843750E+04, -.1336392089844E+04, .1137171875000E+04},
 {       2, .8453079101562E+04, -.1336392089844E+04, .1139248046875E+04},
 {       3, .8457714843750E+04, -.1322421997070E+04, .1137171875000E+04},
 {       4, .8453079101562E+04, -.1322421997070E+04, .1139248046875E+04},
 {       5, .8454361328125E+04, -.1336392089844E+04, .1124345092773E+04},
 {       6, .8449513671875E+04, -.1336392089844E+04, .1125612792969E+04},
 {       7, .8454361328125E+04, -.1322421997070E+04, .1124345092773E+04},
 {       8, .8449513671875E+04, -.1322421997070E+04, .1125612792969E+04},
 {       9, .8453173828125E+04, -.1336392089844E+04, .1119800903320E+04},
 {      10, .8448323242188E+04, -.1336392089844E+04, .1121068237305E+04},
 {      11, .8453173828125E+04, -.1322421997070E+04, .1119800903320E+04},
 {      12, .8448323242188E+04, -.1322421997070E+04, .1121068237305E+04},
 {      13, .8445935546875E+04, -.1336392089844E+04, .1092127197266E+04},
 {      14, .8441193359375E+04, -.1336392089844E+04, .1093798583984E+04},
 {      15, .8445935546875E+04, -.1322421997070E+04, .1092127197266E+04},
 {      16, .8441193359375E+04, -.1322421997070E+04, .1093798583984E+04},
 {      17, .8438700195312E+04, -.1336392089844E+04, .1064454101562E+04},
 {      18, .8434066406250E+04, -.1336392089844E+04, .1066529052734E+04},
 {      19, .8438700195312E+04, -.1322421997070E+04, .1064454101562E+04},
 {      20, .8434066406250E+04, -.1322421997070E+04, .1066529052734E+04},
 {      21, .8457714843750E+04, -.1316072021484E+04, .1137171875000E+04},
 {      22, .8453079101562E+04, -.1316072021484E+04, .1139248046875E+04},
 {      23, .8454361328125E+04, -.1316072021484E+04, .1124345092773E+04},
 {      24, .8449513671875E+04, -.1316072021484E+04, .1125612792969E+04},
 {      25, .8453173828125E+04, -.1316072021484E+04, .1119800903320E+04},
 {      26, .8448323242188E+04, -.1316072021484E+04, .1121068237305E+04},
 {      27, .8445935546875E+04, -.1316072021484E+04, .1092127197266E+04},
 {      28, .8441193359375E+04, -.1316072021484E+04, .1093798583984E+04},
 {      29, .8438700195312E+04, -.1316072021484E+04, .1064454101562E+04},
 {      30, .8434066406250E+04, -.1316072021484E+04, .1066529052734E+04},
 {      31, .8457714843750E+04, -.1300832031250E+04, .1137171875000E+04},
 {      32, .8453079101562E+04, -.1300832031250E+04, .1139248046875E+04},
 {      33, .8457714843750E+04, -.1285592041016E+04, .1137171875000E+04},
 {      34, .8453079101562E+04, -.1285592041016E+04, .1139248046875E+04},
 {      35, .8457714843750E+04, -.1270352050781E+04, .1137171875000E+04},
 {      36, .8453079101562E+04, -.1270352050781E+04, .1139248046875E+04},
 {      37, .8457714843750E+04, -.1255112060547E+04, .1137171875000E+04},
 {      38, .8453079101562E+04, -.1255112060547E+04, .1139248046875E+04},
 {      39, .8454361328125E+04, -.1300832031250E+04, .1124345092773E+04},
 {      40, .8449513671875E+04, -.1300832031250E+04, .1125612792969E+04},
 {      41, .8454361328125E+04, -.1285592041016E+04, .1124345092773E+04},
 {      42, .8449513671875E+04, -.1285592041016E+04, .1125612792969E+04},
 {      43, .8454361328125E+04, -.1270352050781E+04, .1124345092773E+04},
 {      44, .8449513671875E+04, -.1270352050781E+04, .1125612792969E+04},
 {      45, .8454361328125E+04, -.1255112060547E+04, .1124345092773E+04},
 {      46, .8449513671875E+04, -.1255112060547E+04, .1125612792969E+04},
 {      47, .8453173828125E+04, -.1300832031250E+04, .1119800903320E+04},
 {      48, .8448323242188E+04, -.1300832031250E+04, .1121068237305E+04},
 {      49, .8453173828125E+04, -.1285592041016E+04, .1119800903320E+04},
 {      50, .8448323242188E+04, -.1285592041016E+04, .1121068237305E+04},
 {      51, .8453173828125E+04, -.1270352050781E+04, .1119800903320E+04},
 {      52, .8448323242188E+04, -.1270352050781E+04, .1121068237305E+04},
 {      53, .8453173828125E+04, -.1255112060547E+04, .1119800659180E+04},
 {      54, .8448323242188E+04, -.1255112060547E+04, .1121068237305E+04},
 {      55, .8445935546875E+04, -.1300832031250E+04, .1092127197266E+04},
 {      56, .8441193359375E+04, -.1300832031250E+04, .1093798583984E+04},
 {      57, .8445935546875E+04, -.1285592041016E+04, .1092127197266E+04},
 {      58, .8441193359375E+04, -.1285592041016E+04, .1093798583984E+04},
 {      59, .8445935546875E+04, -.1270352050781E+04, .1092127197266E+04},
 {      60, .8441193359375E+04, -.1270352050781E+04, .1093798583984E+04},
 {      61, .8445935546875E+04, -.1255112060547E+04, .1092127197266E+04},
 {      62, .8441193359375E+04, -.1255112060547E+04, .1093798583984E+04},
 {      63, .8438700195312E+04, -.1300832031250E+04, .1064454101562E+04},
 {      64, .8434066406250E+04, -.1300832031250E+04, .1066529052734E+04},
 {      65, .8438700195312E+04, -.1285592041016E+04, .1064454101562E+04},
 {      66, .8434066406250E+04, -.1285592041016E+04, .1066529052734E+04},
 {      67, .8438700195312E+04, -.1270352050781E+04, .1064454101562E+04},
 {      68, .8434066406250E+04, -.1270352050781E+04, .1066529052734E+04},
 {      69, .8438700195312E+04, -.1255112060547E+04, .1064454101562E+04},
 {      70, .8434066406250E+04, -.1255112060547E+04, .1066529052734E+04},
 {      71, .8457714843750E+04, -.1248762084961E+04, .1137171875000E+04},
 {      72, .8453079101562E+04, -.1248762084961E+04, .1139248046875E+04},
 {      73, .8454361328125E+04, -.1248762084961E+04, .1124345092773E+04},
 {      74, .8449513671875E+04, -.1248762084961E+04, .1125612792969E+04},
 {      75, .8453173828125E+04, -.1248762084961E+04, .1119800659180E+04},
 {      76, .8448323242188E+04, -.1248762084961E+04, .1121068237305E+04},
 {      77, .8445935546875E+04, -.1248762084961E+04, .1092127197266E+04},
 {      78, .8441193359375E+04, -.1248762084961E+04, .1093798583984E+04},
 {      79, .8438700195312E+04, -.1248762084961E+04, .1064454101562E+04},
 {      80, .8434066406250E+04, -.1248762084961E+04, .1066529052734E+04},
 {      81, .8457714843750E+04, -.1234791992188E+04, .1137171875000E+04},
 {      82, .8453079101562E+04, -.1234791992188E+04, .1139248046875E+04},
 {      83, .8454361328125E+04, -.1234791992188E+04, .1124345092773E+04},
 {      84, .8449513671875E+04, -.1234791992188E+04, .1125612792969E+04},
 {      85, .8453173828125E+04, -.1234791992188E+04, .1119800659180E+04},
 {      86, .8448323242188E+04, -.1234791992188E+04, .1121068237305E+04},
 {      87, .8445935546875E+04, -.1234791992188E+04, .1092127197266E+04},
 {      88, .8441193359375E+04, -.1234791992188E+04, .1093798583984E+04},
 {      89, .8438700195312E+04, -.1234791992188E+04, .1064454101562E+04},
 {      90, .8434066406250E+04, -.1234791992188E+04, .1066529052734E+04},
 {      91, .8434968750000E+04, -.1336392089844E+04, .1129415771484E+04},
 {      92, .8434968750000E+04, -.1322421997070E+04, .1129415771484E+04},
 {      93, .8433782226562E+04, -.1336392089844E+04, .1124871093750E+04},
 {      94, .8433782226562E+04, -.1322421997070E+04, .1124871093750E+04},
 {      95, .8434968750000E+04, -.1316072021484E+04, .1129415771484E+04},
 {      96, .8433782226562E+04, -.1316072021484E+04, .1124871093750E+04},
 {      97, .8426966796875E+04, -.1322421997070E+04, .1098813476562E+04},
 {      98, .8426966796875E+04, -.1316072021484E+04, .1098813476562E+04},
 {      99, .8420154296875E+04, -.1322421997070E+04, .1072756347656E+04},
 {     100, .8420154296875E+04, -.1316072021484E+04, .1072756347656E+04},
 {     101, .8428875976562E+04, -.1322421997070E+04, .1054938110352E+04},
 {     102, .8414964843750E+04, -.1322421997070E+04, .1061164794922E+04},
 {     103, .8428875976562E+04, -.1316072021484E+04, .1054938110352E+04},
 {     104, .8414964843750E+04, -.1316072021484E+04, .1061164794922E+04},
 {     105, .8423685546875E+04, -.1322421997070E+04, .1043346679688E+04},
 {     106, .8409775390625E+04, -.1322421997070E+04, .1049573486328E+04},
 {     107, .8423685546875E+04, -.1316072021484E+04, .1043346679688E+04},
 {     108, .8409775390625E+04, -.1316072021484E+04, .1049573486328E+04},
 {     109, .8418497070312E+04, -.1322421997070E+04, .1031754394531E+04},
 {     110, .8404586914062E+04, -.1322421997070E+04, .1037981933594E+04},
 {     111, .8418497070312E+04, -.1316072021484E+04, .1031754394531E+04},
 {     112, .8404586914062E+04, -.1316072021484E+04, .1037981933594E+04},
 {     113, .8413306640625E+04, -.1322421997070E+04, .1020163574219E+04},
 {     114, .8399399414062E+04, -.1322421997070E+04, .1026390747070E+04},
 {     115, .8413306640625E+04, -.1316072021484E+04, .1020163574219E+04},
 {     116, .8399399414062E+04, -.1316072021484E+04, .1026390747070E+04},
 {     117, .8412009765625E+04, -.1322421997070E+04, .1017265747070E+04},
 {     118, .8398100585938E+04, -.1322421997070E+04, .1023492065430E+04},
 {     119, .8412009765625E+04, -.1316072021484E+04, .1017265747070E+04},
 {     120, .8398100585938E+04, -.1316072021484E+04, .1023492065430E+04},
 {     121, .8406821289062E+04, -.1322421997070E+04, .1005673706055E+04},
 {     122, .8392910156250E+04, -.1322421997070E+04, .1011901184082E+04},
 {     123, .8406821289062E+04, -.1316072021484E+04, .1005673706055E+04},
 {     124, .8392910156250E+04, -.1316072021484E+04, .1011901184082E+04},
 {     125, .8401631835938E+04, -.1322421997070E+04, .9940820312500E+03},
 {     126, .8387722656250E+04, -.1322421997070E+04, .1000309448242E+04},
 {     127, .8401631835938E+04, -.1316072021484E+04, .9940820312500E+03},
 {     128, .8387722656250E+04, -.1316072021484E+04, .1000309448242E+04},
 {     129, .8400334960938E+04, -.1322421997070E+04, .9911846923828E+03},
 {     130, .8386424804688E+04, -.1322421997070E+04, .9974113159180E+03},
 {     131, .8400334960938E+04, -.1316072021484E+04, .9911846923828E+03},
 {     132, .8386424804688E+04, -.1316072021484E+04, .9974113159180E+03},
 {     133, .8395144531250E+04, -.1322421997070E+04, .9795933837891E+03},
 {     134, .8381236328125E+04, -.1322421997070E+04, .9858200683594E+03},
 {     135, .8395144531250E+04, -.1316072021484E+04, .9795933837891E+03},
 {     136, .8381236328125E+04, -.1316072021484E+04, .9858200683594E+03},
 {     137, .8389957031250E+04, -.1322421997070E+04, .9680015869141E+03},
 {     138, .8376045898438E+04, -.1322421997070E+04, .9742283325195E+03},
 {     139, .8389957031250E+04, -.1316072021484E+04, .9680015869141E+03},
 {     140, .8376045898438E+04, -.1316072021484E+04, .9742283325195E+03},
 {     141, .8388660156250E+04, -.1322421997070E+04, .9651035156250E+03},
 {     142, .8374750000000E+04, -.1322421997070E+04, .9713304443359E+03},
 {     143, .8388660156250E+04, -.1316072021484E+04, .9651035156250E+03},
 {     144, .8374750000000E+04, -.1316072021484E+04, .9713304443359E+03},
 {     145, .8383468750000E+04, -.1322421997070E+04, .9535125122070E+03},
 {     146, .8369560546875E+04, -.1322421997070E+04, .9597390747070E+03},
 {     147, .8383468750000E+04, -.1316072021484E+04, .9535125122070E+03},
 {     148, .8369560546875E+04, -.1316072021484E+04, .9597390747070E+03},
 {     149, .8378281250000E+04, -.1322421997070E+04, .9419204711914E+03},
 {     150, .8364371093750E+04, -.1322421997070E+04, .9481473999023E+03},
 {     151, .8378281250000E+04, -.1316072021484E+04, .9419204711914E+03},
 {     152, .8364371093750E+04, -.1316072021484E+04, .9481473999023E+03},
 {     153, .8376984375000E+04, -.1322421997070E+04, .9390224609375E+03},
 {     154, .8363074218750E+04, -.1322421997070E+04, .9452495727539E+03},
 {     155, .8376984375000E+04, -.1316072021484E+04, .9390224609375E+03},
 {     156, .8363074218750E+04, -.1316072021484E+04, .9452495727539E+03},
 {     157, .8371793945312E+04, -.1322421997070E+04, .9274310302734E+03},
 {     158, .8357883789062E+04, -.1322421997070E+04, .9336578979492E+03},
 {     159, .8371793945312E+04, -.1316072021484E+04, .9274310302734E+03},
 {     160, .8357883789062E+04, -.1316072021484E+04, .9336578979492E+03},
 {     161, .8366604492188E+04, -.1322421997070E+04, .9158396606445E+03},
 {     162, .8352696289062E+04, -.1322421997070E+04, .9220664672852E+03},
 {     163, .8366604492188E+04, -.1316072021484E+04, .9158396606445E+03},
 {     164, .8352696289062E+04, -.1316072021484E+04, .9220664672852E+03},
 {     165, .8364531250000E+04, -.1322421997070E+04, .9112027587891E+03},
 {     166, .8350619140625E+04, -.1322421997070E+04, .9174299926758E+03},
 {     167, .8364531250000E+04, -.1316072021484E+04, .9112027587891E+03},
 {     168, .8350619140625E+04, -.1316072021484E+04, .9174299926758E+03},
 {     169, .8434968750000E+04, -.1300832031250E+04, .1129415771484E+04},
 {     170, .8434968750000E+04, -.1285592041016E+04, .1129415771484E+04},
 {     171, .8434968750000E+04, -.1270352050781E+04, .1129415771484E+04},
 {     172, .8434968750000E+04, -.1255112060547E+04, .1129415771484E+04},
 {     173, .8433782226562E+04, -.1300832031250E+04, .1124871093750E+04},
 {     174, .8433782226562E+04, -.1285592041016E+04, .1124871093750E+04},
 {     175, .8433782226562E+04, -.1270352050781E+04, .1124871093750E+04},
 {     176, .8433782226562E+04, -.1255112060547E+04, .1124871093750E+04},
 {     177, .8420154296875E+04, -.1300832031250E+04, .1072756347656E+04},
 {     178, .8420154296875E+04, -.1285592041016E+04, .1072756347656E+04},
 {     179, .8420154296875E+04, -.1270352050781E+04, .1072756347656E+04},
 {     180, .8420154296875E+04, -.1255112060547E+04, .1072756347656E+04},
 {     181, .8428875976562E+04, -.1300832031250E+04, .1054938110352E+04},
 {     182, .8414964843750E+04, -.1300832031250E+04, .1061164794922E+04},
 {     183, .8428875976562E+04, -.1285592041016E+04, .1054938110352E+04},
 {     184, .8414964843750E+04, -.1285592041016E+04, .1061164794922E+04},
 {     185, .8428875976562E+04, -.1270352050781E+04, .1054938110352E+04},
 {     186, .8414964843750E+04, -.1270352050781E+04, .1061164794922E+04},
 {     187, .8428875976562E+04, -.1255112060547E+04, .1054938110352E+04},
 {     188, .8414964843750E+04, -.1255112060547E+04, .1061164794922E+04},
 {     189, .8423685546875E+04, -.1300832031250E+04, .1043346679688E+04},
 {     190, .8409775390625E+04, -.1300832031250E+04, .1049573486328E+04},
 {     191, .8423685546875E+04, -.1285592041016E+04, .1043346679688E+04},
 {     192, .8409775390625E+04, -.1285592041016E+04, .1049573486328E+04},
 {     193, .8423685546875E+04, -.1270352050781E+04, .1043346679688E+04},
 {     194, .8409775390625E+04, -.1270352050781E+04, .1049573486328E+04},
 {     195, .8423685546875E+04, -.1255112060547E+04, .1043346679688E+04},
 {     196, .8409775390625E+04, -.1255112060547E+04, .1049573486328E+04},
 {     197, .8418497070312E+04, -.1300832031250E+04, .1031754394531E+04},
 {     198, .8404586914062E+04, -.1300832031250E+04, .1037981933594E+04},
 {     199, .8418497070312E+04, -.1285592041016E+04, .1031754394531E+04},
 {     200, .8404586914062E+04, -.1285592041016E+04, .1037981933594E+04},
 {     201, .8418497070312E+04, -.1270352050781E+04, .1031754394531E+04},
 {     202, .8404586914062E+04, -.1270352050781E+04, .1037981933594E+04},
 {     203, .8418497070312E+04, -.1255112060547E+04, .1031754394531E+04},
 {     204, .8404586914062E+04, -.1255112060547E+04, .1037981933594E+04},
 {     205, .8413306640625E+04, -.1300832031250E+04, .1020163574219E+04},
 {     206, .8399399414062E+04, -.1300832031250E+04, .1026390747070E+04},
 {     207, .8413306640625E+04, -.1285592041016E+04, .1020163574219E+04},
 {     208, .8399399414062E+04, -.1285592041016E+04, .1026390747070E+04},
 {     209, .8413306640625E+04, -.1270352050781E+04, .1020163574219E+04},
 {     210, .8399399414062E+04, -.1270352050781E+04, .1026390747070E+04},
 {     211, .8413306640625E+04, -.1255112060547E+04, .1020163574219E+04},
 {     212, .8399399414062E+04, -.1255112060547E+04, .1026390747070E+04},
 {     213, .8412009765625E+04, -.1300832031250E+04, .1017265747070E+04},
 {     214, .8398100585938E+04, -.1300832031250E+04, .1023492065430E+04},
 {     215, .8412009765625E+04, -.1285592041016E+04, .1017265747070E+04},
 {     216, .8398100585938E+04, -.1285592041016E+04, .1023492065430E+04},
 {     217, .8412009765625E+04, -.1270352050781E+04, .1017265747070E+04},
 {     218, .8398100585938E+04, -.1270352050781E+04, .1023492065430E+04},
 {     219, .8412009765625E+04, -.1255112060547E+04, .1017265747070E+04},
 {     220, .8398100585938E+04, -.1255112060547E+04, .1023492065430E+04},
 {     221, .8406821289062E+04, -.1300832031250E+04, .1005673706055E+04},
 {     222, .8392910156250E+04, -.1300832031250E+04, .1011901184082E+04},
 {     223, .8406821289062E+04, -.1285592041016E+04, .1005673706055E+04},
 {     224, .8392910156250E+04, -.1285592041016E+04, .1011901184082E+04},
 {     225, .8406821289062E+04, -.1270352050781E+04, .1005673706055E+04},
 {     226, .8392910156250E+04, -.1270352050781E+04, .1011901184082E+04},
 {     227, .8406821289062E+04, -.1255112060547E+04, .1005673706055E+04},
 {     228, .8392910156250E+04, -.1255112060547E+04, .1011901184082E+04},
 {     229, .8401631835938E+04, -.1300832031250E+04, .9940820312500E+03},
 {     230, .8387722656250E+04, -.1300832031250E+04, .1000309448242E+04},
 {     231, .8401631835938E+04, -.1285592041016E+04, .9940820312500E+03},
 {     232, .8387722656250E+04, -.1285592041016E+04, .1000309448242E+04},
 {     233, .8401631835938E+04, -.1270352050781E+04, .9940820312500E+03},
 {     234, .8387722656250E+04, -.1270352050781E+04, .1000309448242E+04},
 {     235, .8401631835938E+04, -.1255112060547E+04, .9940820312500E+03},
 {     236, .8387722656250E+04, -.1255112060547E+04, .1000309448242E+04},
 {     237, .8400334960938E+04, -.1300832031250E+04, .9911846923828E+03},
 {     238, .8386424804688E+04, -.1300832031250E+04, .9974113159180E+03},
 {     239, .8400334960938E+04, -.1285592041016E+04, .9911846923828E+03},
 {     240, .8386424804688E+04, -.1285592041016E+04, .9974113159180E+03},
 {     241, .8400334960938E+04, -.1270352050781E+04, .9911846923828E+03},
 {     242, .8386424804688E+04, -.1270352050781E+04, .9974113159180E+03},
 {     243, .8400334960938E+04, -.1255112060547E+04, .9911846923828E+03},
 {     244, .8386424804688E+04, -.1255112060547E+04, .9974113159180E+03},
 {     245, .8395144531250E+04, -.1300832031250E+04, .9795933837891E+03},
 {     246, .8381236328125E+04, -.1300832031250E+04, .9858200683594E+03},
 {     247, .8395144531250E+04, -.1285592041016E+04, .9795933837891E+03},
 {     248, .8381236328125E+04, -.1285592041016E+04, .9858200683594E+03},
 {     249, .8395144531250E+04, -.1270352050781E+04, .9795933837891E+03},
 {     250, .8381236328125E+04, -.1270352050781E+04, .9858200683594E+03},
 {     251, .8395144531250E+04, -.1255112060547E+04, .9795933837891E+03},
 {     252, .8381236328125E+04, -.1255112060547E+04, .9858200683594E+03},
 {     253, .8389957031250E+04, -.1300832031250E+04, .9680015869141E+03},
 {     254, .8376045898438E+04, -.1300832031250E+04, .9742283325195E+03},
 {     255, .8389957031250E+04, -.1285592041016E+04, .9680015869141E+03},
 {     256, .8376045898438E+04, -.1285592041016E+04, .9742283325195E+03},
 {     257, .8389957031250E+04, -.1270352050781E+04, .9680015869141E+03},
 {     258, .8376045898438E+04, -.1270352050781E+04, .9742283325195E+03},
 {     259, .8389957031250E+04, -.1255112060547E+04, .9680015869141E+03},
 {     260, .8376045898438E+04, -.1255112060547E+04, .9742283325195E+03},
 {     261, .8388660156250E+04, -.1300832031250E+04, .9651035156250E+03},
 {     262, .8374750000000E+04, -.1300832031250E+04, .9713304443359E+03},
 {     263, .8388660156250E+04, -.1285592041016E+04, .9651035156250E+03},
 {     264, .8374750000000E+04, -.1285592041016E+04, .9713304443359E+03},
 {     265, .8388660156250E+04, -.1270352050781E+04, .9651035156250E+03},
 {     266, .8374750000000E+04, -.1270352050781E+04, .9713304443359E+03},
 {     267, .8388660156250E+04, -.1255112060547E+04, .9651035156250E+03},
 {     268, .8374750000000E+04, -.1255112060547E+04, .9713304443359E+03},
 {     269, .8383468750000E+04, -.1300832031250E+04, .9535125122070E+03},
 {     270, .8369560546875E+04, -.1300832031250E+04, .9597390747070E+03},
 {     271, .8383468750000E+04, -.1285592041016E+04, .9535125122070E+03},
 {     272, .8369560546875E+04, -.1285592041016E+04, .9597390747070E+03},
 {     273, .8383468750000E+04, -.1270352050781E+04, .9535125122070E+03},
 {     274, .8369560546875E+04, -.1270352050781E+04, .9597390747070E+03},
 {     275, .8383468750000E+04, -.1255112060547E+04, .9535125122070E+03},
 {     276, .8369560546875E+04, -.1255112060547E+04, .9597390747070E+03},
 {     277, .8378281250000E+04, -.1300832031250E+04, .9419204711914E+03},
 {     278, .8364371093750E+04, -.1300832031250E+04, .9481473999023E+03},
 {     279, .8378281250000E+04, -.1285592041016E+04, .9419204711914E+03},
 {     280, .8364371093750E+04, -.1285592041016E+04, .9481473999023E+03},
 {     281, .8378281250000E+04, -.1270352050781E+04, .9419204711914E+03},
 {     282, .8364371093750E+04, -.1270352050781E+04, .9481473999023E+03},
 {     283, .8378281250000E+04, -.1255112060547E+04, .9419204711914E+03},
 {     284, .8364371093750E+04, -.1255112060547E+04, .9481473999023E+03},
 {     285, .8376984375000E+04, -.1300832031250E+04, .9390224609375E+03},
 {     286, .8363074218750E+04, -.1300832031250E+04, .9452495727539E+03},
 {     287, .8376984375000E+04, -.1285592041016E+04, .9390224609375E+03},
 {     288, .8363074218750E+04, -.1285592041016E+04, .9452495727539E+03},
 {     289, .8376984375000E+04, -.1270352050781E+04, .9390224609375E+03},
 {     290, .8363074218750E+04, -.1270352050781E+04, .9452495727539E+03},
 {     291, .8376984375000E+04, -.1255112060547E+04, .9390224609375E+03},
 {     292, .8363074218750E+04, -.1255112060547E+04, .9452495727539E+03},
 {     293, .8371793945312E+04, -.1300832031250E+04, .9274310302734E+03},
 {     294, .8357883789062E+04, -.1300832031250E+04, .9336578979492E+03},
 {     295, .8371793945312E+04, -.1285592041016E+04, .9274310302734E+03},
 {     296, .8357883789062E+04, -.1285592041016E+04, .9336578979492E+03},
 {     297, .8371793945312E+04, -.1270352050781E+04, .9274310302734E+03},
 {     298, .8357883789062E+04, -.1270352050781E+04, .9336578979492E+03},
 {     299, .8371793945312E+04, -.1255112060547E+04, .9274310302734E+03},
 {     300, .8357883789062E+04, -.1255112060547E+04, .9336578979492E+03},
 {     301, .8366604492188E+04, -.1300832031250E+04, .9158396606445E+03},
 {     302, .8352696289062E+04, -.1300832031250E+04, .9220664672852E+03},
 {     303, .8366604492188E+04, -.1285592041016E+04, .9158396606445E+03},
 {     304, .8352696289062E+04, -.1285592041016E+04, .9220664672852E+03},
 {     305, .8366604492188E+04, -.1270352050781E+04, .9158396606445E+03},
 {     306, .8352696289062E+04, -.1270352050781E+04, .9220664672852E+03},
 {     307, .8366604492188E+04, -.1255112060547E+04, .9158396606445E+03},
 {     308, .8352696289062E+04, -.1255112060547E+04, .9220664672852E+03},
 {     309, .8364531250000E+04, -.1300832031250E+04, .9112027587891E+03},
 {     310, .8350619140625E+04, -.1300832031250E+04, .9174299926758E+03},
 {     311, .8364531250000E+04, -.1285592041016E+04, .9112027587891E+03},
 {     312, .8350619140625E+04, -.1285592041016E+04, .9174299926758E+03},
 {     313, .8364531250000E+04, -.1270352050781E+04, .9112027587891E+03},
 {     314, .8350619140625E+04, -.1270352050781E+04, .9174299926758E+03},
 {     315, .8364531250000E+04, -.1255112060547E+04, .9112027587891E+03},
 {     316, .8350619140625E+04, -.1255112060547E+04, .9174299926758E+03},
 {     317, .8434968750000E+04, -.1248762084961E+04, .1129415771484E+04},
 {     318, .8433782226562E+04, -.1248762084961E+04, .1124871093750E+04},
 {     319, .8426966796875E+04, -.1255112060547E+04, .1098813476562E+04},
 {     320, .8426966796875E+04, -.1248762084961E+04, .1098813476562E+04},
 {     321, .8420154296875E+04, -.1248762084961E+04, .1072756347656E+04},
 {     322, .8428875976562E+04, -.1248762084961E+04, .1054938110352E+04},
 {     323, .8414964843750E+04, -.1248762084961E+04, .1061164794922E+04},
 {     324, .8423685546875E+04, -.1248762084961E+04, .1043346679688E+04},
 {     325, .8409775390625E+04, -.1248762084961E+04, .1049573486328E+04},
 {     326, .8418497070312E+04, -.1248762084961E+04, .1031754394531E+04},
 {     327, .8404586914062E+04, -.1248762084961E+04, .1037981933594E+04},
 {     328, .8413306640625E+04, -.1248762084961E+04, .1020163574219E+04},
 {     329, .8399399414062E+04, -.1248762084961E+04, .1026390747070E+04},
 {     330, .8412009765625E+04, -.1248762084961E+04, .1017265747070E+04},
 {     331, .8398100585938E+04, -.1248762084961E+04, .1023492065430E+04},
 {     332, .8406821289062E+04, -.1248762084961E+04, .1005673706055E+04},
 {     333, .8392910156250E+04, -.1248762084961E+04, .1011901184082E+04},
 {     334, .8401631835938E+04, -.1248762084961E+04, .9940820312500E+03},
 {     335, .8387722656250E+04, -.1248762084961E+04, .1000309448242E+04},
 {     336, .8400334960938E+04, -.1248762084961E+04, .9911846923828E+03},
 {     337, .8386424804688E+04, -.1248762084961E+04, .9974113159180E+03},
 {     338, .8395144531250E+04, -.1248762084961E+04, .9795933837891E+03},
 {     339, .8381236328125E+04, -.1248762084961E+04, .9858200683594E+03},
 {     340, .8389957031250E+04, -.1248762084961E+04, .9680015869141E+03},
 {     341, .8376045898438E+04, -.1248762084961E+04, .9742283325195E+03},
 {     342, .8388660156250E+04, -.1248762084961E+04, .9651035156250E+03},
 {     343, .8374750000000E+04, -.1248762084961E+04, .9713304443359E+03},
 {     344, .8383468750000E+04, -.1248762084961E+04, .9535125122070E+03},
 {     345, .8369560546875E+04, -.1248762084961E+04, .9597390747070E+03},
 {     346, .8378281250000E+04, -.1248762084961E+04, .9419204711914E+03},
 {     347, .8364371093750E+04, -.1248762084961E+04, .9481473999023E+03},
 {     348, .8376984375000E+04, -.1248762084961E+04, .9390224609375E+03},
 {     349, .8363074218750E+04, -.1248762084961E+04, .9452495727539E+03},
 {     350, .8371793945312E+04, -.1248762084961E+04, .9274310302734E+03},
 {     351, .8357883789062E+04, -.1248762084961E+04, .9336578979492E+03},
 {     352, .8366604492188E+04, -.1248762084961E+04, .9158396606445E+03},
 {     353, .8352696289062E+04, -.1248762084961E+04, .9220664672852E+03},
 {     354, .8364531250000E+04, -.1248762084961E+04, .9112027587891E+03},
 {     355, .8350619140625E+04, -.1248762084961E+04, .9174299926758E+03},
 {     356, .8434968750000E+04, -.1234791992188E+04, .1129415771484E+04},
 {     357, .8433782226562E+04, -.1234791992188E+04, .1124871093750E+04},
 {     358, .8416972656250E+04, -.1336392089844E+04, .1134121948242E+04},
 {     359, .8398976562500E+04, -.1336392089844E+04, .1138827880859E+04},
 {     360, .8416972656250E+04, -.1322421997070E+04, .1134121948242E+04},
 {     361, .8398976562500E+04, -.1322421997070E+04, .1138827880859E+04},
 {     362, .8415571289062E+04, -.1336392089844E+04, .1129632568359E+04},
 {     363, .8397360351562E+04, -.1336392089844E+04, .1134395141602E+04},
 {     364, .8415571289062E+04, -.1322421997070E+04, .1129632568359E+04},
 {     365, .8397360351562E+04, -.1322421997070E+04, .1134395141602E+04},
 {     366, .8416972656250E+04, -.1316072021484E+04, .1134121948242E+04},
 {     367, .8398976562500E+04, -.1316072021484E+04, .1138827880859E+04},
 {     368, .8415571289062E+04, -.1316072021484E+04, .1129632568359E+04},
 {     369, .8397360351562E+04, -.1316072021484E+04, .1134395141602E+04},
 {     370, .8408059570312E+04, -.1322421997070E+04, .1105346069336E+04},
 {     371, .8389152343750E+04, -.1322421997070E+04, .1111878662109E+04},
 {     372, .8408059570312E+04, -.1316072021484E+04, .1105346069336E+04},
 {     373, .8389152343750E+04, -.1316072021484E+04, .1111878662109E+04},
 {     374, .8400548828125E+04, -.1322421997070E+04, .1081059814453E+04},
 {     375, .8380944335938E+04, -.1322421997070E+04, .1089363159180E+04},
 {     376, .8400548828125E+04, -.1316072021484E+04, .1081059814453E+04},
 {     377, .8380944335938E+04, -.1316072021484E+04, .1089363159180E+04},
 {     378, .8395785156250E+04, -.1322421997070E+04, .1069313354492E+04},
 {     379, .8376604492188E+04, -.1322421997070E+04, .1077461914062E+04},
 {     380, .8395785156250E+04, -.1316072021484E+04, .1069313354492E+04},
 {     381, .8376604492188E+04, -.1316072021484E+04, .1077461914062E+04},
 {     382, .8391020507812E+04, -.1322421997070E+04, .1057566650391E+04},
 {     383, .8372265625000E+04, -.1322421997070E+04, .1065559936523E+04},
 {     384, .8391020507812E+04, -.1316072021484E+04, .1057566650391E+04},
 {     385, .8372265625000E+04, -.1316072021484E+04, .1065559936523E+04},
 {     386, .8386255859375E+04, -.1322421997070E+04, .1045820434570E+04},
 {     387, .8367925781250E+04, -.1322421997070E+04, .1053658691406E+04},
 {     388, .8386255859375E+04, -.1316072021484E+04, .1045820434570E+04},
 {     389, .8367925781250E+04, -.1316072021484E+04, .1053658691406E+04},
 {     390, .8381492187500E+04, -.1322421997070E+04, .1034073974609E+04},
 {     391, .8363587890625E+04, -.1322421997070E+04, .1041757446289E+04},
 {     392, .8381492187500E+04, -.1316072021484E+04, .1034073974609E+04},
 {     393, .8363587890625E+04, -.1316072021484E+04, .1041757446289E+04},
 {     394, .8380301757812E+04, -.1322421997070E+04, .1031137573242E+04},
 {     395, .8362500976562E+04, -.1322421997070E+04, .1038782226562E+04},
 {     396, .8380301757812E+04, -.1316072021484E+04, .1031137573242E+04},
 {     397, .8362500976562E+04, -.1316072021484E+04, .1038782226562E+04},
 {     398, .8375536132812E+04, -.1322421997070E+04, .1019390747070E+04},
 {     399, .8358162109375E+04, -.1322421997070E+04, .1026880493164E+04},
 {     400, .8375536132812E+04, -.1316072021484E+04, .1019390747070E+04},
 {     401, .8358162109375E+04, -.1316072021484E+04, .1026880493164E+04},
 {     402, .8370773437500E+04, -.1322421997070E+04, .1007644409180E+04},
 {     403, .8353825195312E+04, -.1322421997070E+04, .1014978881836E+04},
 {     404, .8370773437500E+04, -.1316072021484E+04, .1007644409180E+04},
 {     405, .8353825195312E+04, -.1316072021484E+04, .1014978881836E+04},
 {     406, .8369581054688E+04, -.1322421997070E+04, .1004707702637E+04},
 {     407, .8352739257812E+04, -.1322421997070E+04, .1012003540039E+04},
 {     408, .8369581054688E+04, -.1316072021484E+04, .1004707702637E+04},
 {     409, .8352739257812E+04, -.1316072021484E+04, .1012003540039E+04},
 {     410, .8364818359375E+04, -.1322421997070E+04, .9929609375000E+03},
 {     411, .8348402343750E+04, -.1322421997070E+04, .1000102050781E+04},
 {     412, .8364818359375E+04, -.1316072021484E+04, .9929609375000E+03},
 {     413, .8348402343750E+04, -.1316072021484E+04, .1000102050781E+04},
 {     414, .8360055664062E+04, -.1322421997070E+04, .9812144775391E+03},
 {     415, .8344062500000E+04, -.1322421997070E+04, .9882008666992E+03},
 {     416, .8360055664062E+04, -.1316072021484E+04, .9812144775391E+03},
 {     417, .8344062500000E+04, -.1316072021484E+04, .9882008666992E+03},
 {     418, .8358863281250E+04, -.1322421997070E+04, .9782779541016E+03},
 {     419, .8342976562500E+04, -.1322421997070E+04, .9852258300781E+03},
 {     420, .8358863281250E+04, -.1316072021484E+04, .9782779541016E+03},
 {     421, .8342976562500E+04, -.1316072021484E+04, .9852258300781E+03},
 {     422, .8354100585938E+04, -.1322421997070E+04, .9665313110352E+03},
 {     423, .8338638671875E+04, -.1322421997070E+04, .9733240966797E+03},
 {     424, .8354100585938E+04, -.1316072021484E+04, .9665313110352E+03},
 {     425, .8338638671875E+04, -.1316072021484E+04, .9733240966797E+03},
 {     426, .8349333984375E+04, -.1322421997070E+04, .9547850341797E+03},
 {     427, .8334298828125E+04, -.1322421997070E+04, .9614224853516E+03},
 {     428, .8349333984375E+04, -.1316072021484E+04, .9547850341797E+03},
 {     429, .8334298828125E+04, -.1316072021484E+04, .9614224853516E+03},
 {     430, .8348143554688E+04, -.1322421997070E+04, .9518484497070E+03},
 {     431, .8333214843750E+04, -.1322421997070E+04, .9584469604492E+03},
 {     432, .8348143554688E+04, -.1316072021484E+04, .9518484497070E+03},
 {     433, .8333214843750E+04, -.1316072021484E+04, .9584469604492E+03},
 {     434, .8343379882812E+04, -.1322421997070E+04, .9401017456055E+03},
 {     435, .8328873046875E+04, -.1322421997070E+04, .9465457763672E+03},
 {     436, .8343379882812E+04, -.1316072021484E+04, .9401017456055E+03},
 {     437, .8328873046875E+04, -.1316072021484E+04, .9465457763672E+03},
 {     438, .8338615234375E+04, -.1322421997070E+04, .9283553466797E+03},
 {     439, .8324534179688E+04, -.1322421997070E+04, .9346444091797E+03},
 {     440, .8338615234375E+04, -.1316072021484E+04, .9283553466797E+03},
 {     441, .8324534179688E+04, -.1316072021484E+04, .9346444091797E+03},
 {     442, .8336710937500E+04, -.1322421997070E+04, .9236568603516E+03},
 {     443, .8322800781250E+04, -.1322421997070E+04, .9298833618164E+03},
 {     444, .8336710937500E+04, -.1316072021484E+04, .9236568603516E+03},
 {     445, .8322800781250E+04, -.1316072021484E+04, .9298833618164E+03},
 {     446, .8416972656250E+04, -.1300832031250E+04, .1134121948242E+04},
 {     447, .8398976562500E+04, -.1300832031250E+04, .1138827880859E+04},
 {     448, .8416972656250E+04, -.1285592041016E+04, .1134121948242E+04},
 {     449, .8398976562500E+04, -.1285592041016E+04, .1138827880859E+04},
 {     450, .8416972656250E+04, -.1270352050781E+04, .1134121948242E+04},
 {     451, .8398976562500E+04, -.1270352050781E+04, .1138827880859E+04},
 {     452, .8416972656250E+04, -.1255112060547E+04, .1134121948242E+04},
 {     453, .8398976562500E+04, -.1255112060547E+04, .1138827880859E+04},
 {     454, .8415571289062E+04, -.1300832031250E+04, .1129632568359E+04},
 {     455, .8397360351562E+04, -.1300832031250E+04, .1134395141602E+04},
 {     456, .8415571289062E+04, -.1285592041016E+04, .1129632568359E+04},
 {     457, .8397360351562E+04, -.1285592041016E+04, .1134395141602E+04},
 {     458, .8415571289062E+04, -.1270352050781E+04, .1129632568359E+04},
 {     459, .8397360351562E+04, -.1270352050781E+04, .1134395141602E+04},
 {     460, .8415571289062E+04, -.1255112060547E+04, .1129632568359E+04},
 {     461, .8397360351562E+04, -.1255112060547E+04, .1134395141602E+04},
 {     462, .8338615234375E+04, -.1300832031250E+04, .9283553466797E+03},
 {     463, .8324534179688E+04, -.1300832031250E+04, .9346444091797E+03},
 {     464, .8338615234375E+04, -.1285592041016E+04, .9283553466797E+03},
 {     465, .8324534179688E+04, -.1285592041016E+04, .9346444091797E+03},
 {     466, .8338615234375E+04, -.1270352050781E+04, .9283553466797E+03},
 {     467, .8324534179688E+04, -.1270352050781E+04, .9346444091797E+03},
 {     468, .8338615234375E+04, -.1255112060547E+04, .9283553466797E+03},
 {     469, .8324534179688E+04, -.1255112060547E+04, .9346444091797E+03},
 {     470, .8336710937500E+04, -.1300832031250E+04, .9236568603516E+03},
 {     471, .8322800781250E+04, -.1300832031250E+04, .9298833618164E+03},
 {     472, .8336710937500E+04, -.1285592041016E+04, .9236568603516E+03},
 {     473, .8322800781250E+04, -.1285592041016E+04, .9298833618164E+03},
 {     474, .8336710937500E+04, -.1270352050781E+04, .9236568603516E+03},
 {     475, .8322800781250E+04, -.1270352050781E+04, .9298833618164E+03},
 {     476, .8336710937500E+04, -.1255112060547E+04, .9236568603516E+03},
 {     477, .8322800781250E+04, -.1255112060547E+04, .9298833618164E+03},
 {     478, .8416972656250E+04, -.1248762084961E+04, .1134121948242E+04},
 {     479, .8398976562500E+04, -.1248762084961E+04, .1138827880859E+04},
 {     480, .8415571289062E+04, -.1248762084961E+04, .1129632568359E+04},
 {     481, .8397360351562E+04, -.1248762084961E+04, .1134395141602E+04},
 {     482, .8408059570312E+04, -.1255112060547E+04, .1105346069336E+04},
 {     483, .8389152343750E+04, -.1255112060547E+04, .1111878662109E+04},
 {     484, .8408059570312E+04, -.1248762084961E+04, .1105346069336E+04},
 {     485, .8389152343750E+04, -.1248762084961E+04, .1111878662109E+04},
 {     486, .8400548828125E+04, -.1255112060547E+04, .1081059814453E+04},
 {     487, .8380944335938E+04, -.1255112060547E+04, .1089363159180E+04},
 {     488, .8400548828125E+04, -.1248762084961E+04, .1081059814453E+04},
 {     489, .8380944335938E+04, -.1248762084961E+04, .1089363159180E+04},
 {     490, .8395785156250E+04, -.1255112060547E+04, .1069313354492E+04},
 {     491, .8376604492188E+04, -.1255112060547E+04, .1077461914062E+04},
 {     492, .8395785156250E+04, -.1248762084961E+04, .1069313354492E+04},
 {     493, .8376604492188E+04, -.1248762084961E+04, .1077461914062E+04},
 {     494, .8391020507812E+04, -.1255112060547E+04, .1057566650391E+04},
 {     495, .8372265625000E+04, -.1255112060547E+04, .1065559936523E+04},
 {     496, .8391020507812E+04, -.1248762084961E+04, .1057566650391E+04},
 {     497, .8372265625000E+04, -.1248762084961E+04, .1065559936523E+04},
 {     498, .8386255859375E+04, -.1255112060547E+04, .1045820434570E+04},
 {     499, .8367925781250E+04, -.1255112060547E+04, .1053658691406E+04},
 {     500, .8386255859375E+04, -.1248762084961E+04, .1045820434570E+04},
 {     501, .8367925781250E+04, -.1248762084961E+04, .1053658691406E+04},
 {     502, .8381492187500E+04, -.1255112060547E+04, .1034073974609E+04},
 {     503, .8363587890625E+04, -.1255112060547E+04, .1041757446289E+04},
 {     504, .8381492187500E+04, -.1248762084961E+04, .1034073974609E+04},
 {     505, .8363587890625E+04, -.1248762084961E+04, .1041757446289E+04},
 {     506, .8380301757812E+04, -.1255112060547E+04, .1031137573242E+04},
 {     507, .8362500976562E+04, -.1255112060547E+04, .1038782226562E+04},
 {     508, .8380301757812E+04, -.1248762084961E+04, .1031137573242E+04},
 {     509, .8362500976562E+04, -.1248762084961E+04, .1038782226562E+04},
 {     510, .8375536132812E+04, -.1255112060547E+04, .1019390747070E+04},
 {     511, .8358162109375E+04, -.1255112060547E+04, .1026880493164E+04},
 {     512, .8375536132812E+04, -.1248762084961E+04, .1019390747070E+04},
 {     513, .8358162109375E+04, -.1248762084961E+04, .1026880493164E+04},
 {     514, .8370773437500E+04, -.1255112060547E+04, .1007644409180E+04},
 {     515, .8353825195312E+04, -.1255112060547E+04, .1014978881836E+04},
 {     516, .8370773437500E+04, -.1248762084961E+04, .1007644409180E+04},
 {     517, .8353825195312E+04, -.1248762084961E+04, .1014978881836E+04},
 {     518, .8369581054688E+04, -.1255112060547E+04, .1004707702637E+04},
 {     519, .8352739257812E+04, -.1255112060547E+04, .1012003540039E+04},
 {     520, .8369581054688E+04, -.1248762084961E+04, .1004707702637E+04},
 {     521, .8352739257812E+04, -.1248762084961E+04, .1012003540039E+04},
 {     522, .8364818359375E+04, -.1255112060547E+04, .9929609375000E+03},
 {     523, .8348402343750E+04, -.1255112060547E+04, .1000102050781E+04},
 {     524, .8364818359375E+04, -.1248762084961E+04, .9929609375000E+03},
 {     525, .8348402343750E+04, -.1248762084961E+04, .1000102050781E+04},
 {     526, .8360055664062E+04, -.1255112060547E+04, .9812144775391E+03},
 {     527, .8344062500000E+04, -.1255112060547E+04, .9882008666992E+03},
 {     528, .8360055664062E+04, -.1248762084961E+04, .9812144775391E+03},
 {     529, .8344062500000E+04, -.1248762084961E+04, .9882008666992E+03},
 {     530, .8358863281250E+04, -.1255112060547E+04, .9782779541016E+03},
 {     531, .8342976562500E+04, -.1255112060547E+04, .9852258300781E+03},
 {     532, .8358863281250E+04, -.1248762084961E+04, .9782779541016E+03},
 {     533, .8342976562500E+04, -.1248762084961E+04, .9852258300781E+03},
 {     534, .8354100585938E+04, -.1255112060547E+04, .9665313110352E+03},
 {     535, .8338638671875E+04, -.1255112060547E+04, .9733240966797E+03},
 {     536, .8354100585938E+04, -.1248762084961E+04, .9665313110352E+03},
 {     537, .8338638671875E+04, -.1248762084961E+04, .9733240966797E+03},
 {     538, .8349333984375E+04, -.1255112060547E+04, .9547850341797E+03},
 {     539, .8334298828125E+04, -.1255112060547E+04, .9614224853516E+03},
 {     540, .8349333984375E+04, -.1248762084961E+04, .9547850341797E+03},
 {     541, .8334298828125E+04, -.1248762084961E+04, .9614224853516E+03},
 {     542, .8348143554688E+04, -.1255112060547E+04, .9518484497070E+03},
 {     543, .8333214843750E+04, -.1255112060547E+04, .9584469604492E+03},
 {     544, .8348143554688E+04, -.1248762084961E+04, .9518484497070E+03},
 {     545, .8333214843750E+04, -.1248762084961E+04, .9584469604492E+03},
 {     546, .8343379882812E+04, -.1255112060547E+04, .9401017456055E+03},
 {     547, .8328873046875E+04, -.1255112060547E+04, .9465457763672E+03},
 {     548, .8343379882812E+04, -.1248762084961E+04, .9401017456055E+03},
 {     549, .8328873046875E+04, -.1248762084961E+04, .9465457763672E+03},
 {     550, .8338615234375E+04, -.1248762084961E+04, .9283553466797E+03},
 {     551, .8324534179688E+04, -.1248762084961E+04, .9346444091797E+03},
 {     552, .8336710937500E+04, -.1248762084961E+04, .9236568603516E+03},
 {     553, .8322800781250E+04, -.1248762084961E+04, .9298833618164E+03},
 {     554, .8416972656250E+04, -.1234791992188E+04, .1134121948242E+04},
 {     555, .8398976562500E+04, -.1234791992188E+04, .1138827880859E+04},
 {     556, .8415571289062E+04, -.1234791992188E+04, .1129632568359E+04},
 {     557, .8397360351562E+04, -.1234791992188E+04, .1134395141602E+04},
 {     558, .8380511718750E+04, -.1336392089844E+04, .1143656005859E+04},
 {     559, .8362046875000E+04, -.1336392089844E+04, .1148484252930E+04},
 {     560, .8380511718750E+04, -.1322421997070E+04, .1143656005859E+04},
 {     561, .8362046875000E+04, -.1322421997070E+04, .1148484252930E+04},
 {     562, .8378894531250E+04, -.1336392089844E+04, .1139223510742E+04},
 {     563, .8360430664062E+04, -.1336392089844E+04, .1144051879883E+04},
 {     564, .8378894531250E+04, -.1322421997070E+04, .1139223510742E+04},
 {     565, .8360430664062E+04, -.1322421997070E+04, .1144051879883E+04},
 {     566, .8380511718750E+04, -.1316072021484E+04, .1143656005859E+04},
 {     567, .8362046875000E+04, -.1316072021484E+04, .1148484252930E+04},
 {     568, .8378894531250E+04, -.1316072021484E+04, .1139223510742E+04},
 {     569, .8360430664062E+04, -.1316072021484E+04, .1144051879883E+04},
 {     570, .8371226562500E+04, -.1322421997070E+04, .1118185058594E+04},
 {     571, .8353299804688E+04, -.1322421997070E+04, .1124491210938E+04},
 {     572, .8371226562500E+04, -.1316072021484E+04, .1118185058594E+04},
 {     573, .8353299804688E+04, -.1316072021484E+04, .1124491210938E+04},
 {     574, .8363556640625E+04, -.1322421997070E+04, .1097146728516E+04},
 {     575, .8346169921875E+04, -.1322421997070E+04, .1104930053711E+04},
 {     576, .8363556640625E+04, -.1316072021484E+04, .1097146728516E+04},
 {     577, .8346169921875E+04, -.1316072021484E+04, .1104930053711E+04},
 {     578, .8359215820312E+04, -.1322421997070E+04, .1085245239258E+04},
 {     579, .8341830078125E+04, -.1322421997070E+04, .1093029174805E+04},
 {     580, .8359215820312E+04, -.1316072021484E+04, .1085245239258E+04},
 {     581, .8341830078125E+04, -.1316072021484E+04, .1093029174805E+04},
 {     582, .8354876953125E+04, -.1322421997070E+04, .1073343994141E+04},
 {     583, .8337489257812E+04, -.1322421997070E+04, .1081127685547E+04},
 {     584, .8354876953125E+04, -.1316072021484E+04, .1073343994141E+04},
 {     585, .8337489257812E+04, -.1316072021484E+04, .1081127685547E+04},
 {     586, .8350541015625E+04, -.1322421997070E+04, .1061442260742E+04},
 {     587, .8333151367188E+04, -.1322421997070E+04, .1069225830078E+04},
 {     588, .8350541015625E+04, -.1316072021484E+04, .1061442260742E+04},
 {     589, .8333151367188E+04, -.1316072021484E+04, .1069225830078E+04},
 {     590, .8346200195312E+04, -.1322421997070E+04, .1049540527344E+04},
 {     591, .8328813476562E+04, -.1322421997070E+04, .1057324462891E+04},
 {     592, .8346200195312E+04, -.1316072021484E+04, .1049540527344E+04},
 {     593, .8328813476562E+04, -.1316072021484E+04, .1057324462891E+04},
 {     594, .8345115234375E+04, -.1322421997070E+04, .1046565673828E+04},
 {     595, .8327726562500E+04, -.1322421997070E+04, .1054348999023E+04},
 {     596, .8345115234375E+04, -.1316072021484E+04, .1046565673828E+04},
 {     597, .8327726562500E+04, -.1316072021484E+04, .1054348999023E+04},
 {     598, .8340775390625E+04, -.1322421997070E+04, .1034663818359E+04},
 {     599, .8323387695312E+04, -.1322421997070E+04, .1042448120117E+04},
 {     600, .8340775390625E+04, -.1316072021484E+04, .1034663818359E+04},
 {     601, .8323387695312E+04, -.1316072021484E+04, .1042448120117E+04},
 {     602, .8336436523438E+04, -.1322421997070E+04, .1022763061523E+04},
 {     603, .8319047851562E+04, -.1322421997070E+04, .1030546630859E+04},
 {     604, .8336436523438E+04, -.1316072021484E+04, .1022763061523E+04},
 {     605, .8319047851562E+04, -.1316072021484E+04, .1030546630859E+04},
 {     606, .8335351562500E+04, -.1322421997070E+04, .1019787292480E+04},
 {     607, .8317963867188E+04, -.1322421997070E+04, .1027571166992E+04},
 {     608, .8335351562500E+04, -.1316072021484E+04, .1019787292480E+04},
 {     609, .8317963867188E+04, -.1316072021484E+04, .1027571166992E+04},
 {     610, .8331013671875E+04, -.1322421997070E+04, .1007885559082E+04},
 {     611, .8313628906250E+04, -.1322421997070E+04, .1015669189453E+04},
 {     612, .8331013671875E+04, -.1316072021484E+04, .1007885559082E+04},
 {     613, .8313628906250E+04, -.1316072021484E+04, .1015669189453E+04},
 {     614, .8326674804688E+04, -.1322421997070E+04, .9959844970703E+03},
 {     615, .8309287109375E+04, -.1322421997070E+04, .1003768188477E+04},
 {     616, .8326674804688E+04, -.1316072021484E+04, .9959842529297E+03},
 {     617, .8309287109375E+04, -.1316072021484E+04, .1003768188477E+04},
 {     618, .8325590820312E+04, -.1322421997070E+04, .9930088500977E+03},
 {     619, .8308203125000E+04, -.1322421997070E+04, .1000792480469E+04},
 {     620, .8325590820312E+04, -.1316072021484E+04, .9930088500977E+03},
 {     621, .8308203125000E+04, -.1316072021484E+04, .1000792480469E+04},
 {     622, .8321250976562E+04, -.1322421997070E+04, .9811076660156E+03},
 {     623, .8303865234375E+04, -.1322421997070E+04, .9888907470703E+03},
 {     624, .8321250976562E+04, -.1316072021484E+04, .9811076660156E+03},
 {     625, .8303865234375E+04, -.1316072021484E+04, .9888907470703E+03},
 {     626, .8316912109375E+04, -.1322421997070E+04, .9692061767578E+03},
 {     627, .8299524414062E+04, -.1322421997070E+04, .9769899291992E+03},
 {     628, .8316912109375E+04, -.1316072021484E+04, .9692061767578E+03},
 {     629, .8299524414062E+04, -.1316072021484E+04, .9769899291992E+03},
 {     630, .8315827148438E+04, -.1322421997070E+04, .9662307128906E+03},
 {     631, .8298438476562E+04, -.1322421997070E+04, .9740146484375E+03},
 {     632, .8315827148438E+04, -.1316072021484E+04, .9662307128906E+03},
 {     633, .8298438476562E+04, -.1316072021484E+04, .9740146484375E+03},
 {     634, .8311488281250E+04, -.1322421997070E+04, .9543291625977E+03},
 {     635, .8294100585938E+04, -.1322421997070E+04, .9621129150391E+03},
 {     636, .8311488281250E+04, -.1316072021484E+04, .9543291625977E+03},
 {     637, .8294100585938E+04, -.1316072021484E+04, .9621129150391E+03},
 {     638, .8307148437500E+04, -.1322421997070E+04, .9424277954102E+03},
 {     639, .8289761718750E+04, -.1322421997070E+04, .9502113647461E+03},
 {     640, .8307148437500E+04, -.1316072021484E+04, .9424277954102E+03},
 {     641, .8289761718750E+04, -.1316072021484E+04, .9502113647461E+03},
 {     642, .8305412109375E+04, -.1322421997070E+04, .9376674194336E+03},
 {     643, .8288026367188E+04, -.1322421997070E+04, .9454510498047E+03},
 {     644, .8305412109375E+04, -.1316072021484E+04, .9376674194336E+03},
 {     645, .8288026367188E+04, -.1316072021484E+04, .9454510498047E+03},
 {     646, .8380511718750E+04, -.1300832031250E+04, .1143656005859E+04},
 {     647, .8362046875000E+04, -.1300832031250E+04, .1148484252930E+04},
 {     648, .8380511718750E+04, -.1285592041016E+04, .1143656005859E+04},
 {     649, .8362046875000E+04, -.1285592041016E+04, .1148484252930E+04},
 {     650, .8380511718750E+04, -.1270352050781E+04, .1143656005859E+04},
 {     651, .8362046875000E+04, -.1270352050781E+04, .1148484252930E+04},
 {     652, .8380511718750E+04, -.1255112060547E+04, .1143656005859E+04},
 {     653, .8362046875000E+04, -.1255112060547E+04, .1148484252930E+04},
 {     654, .8378894531250E+04, -.1300832031250E+04, .1139223510742E+04},
 {     655, .8360430664062E+04, -.1300832031250E+04, .1144051879883E+04},
 {     656, .8378894531250E+04, -.1285592041016E+04, .1139223510742E+04},
 {     657, .8360430664062E+04, -.1285592041016E+04, .1144051879883E+04},
 {     658, .8378894531250E+04, -.1270352050781E+04, .1139223510742E+04},
 {     659, .8360430664062E+04, -.1270352050781E+04, .1144051879883E+04},
 {     660, .8378894531250E+04, -.1255112060547E+04, .1139223510742E+04},
 {     661, .8360430664062E+04, -.1255112060547E+04, .1144051879883E+04},
 {     662, .8389152343750E+04, -.1300832031250E+04, .1111878662109E+04},
 {     663, .8371226562500E+04, -.1300832031250E+04, .1118185058594E+04},
 {     664, .8353299804688E+04, -.1300832031250E+04, .1124491210938E+04},
 {     665, .8389152343750E+04, -.1285592041016E+04, .1111878662109E+04},
 {     666, .8371226562500E+04, -.1285592041016E+04, .1118185058594E+04},
 {     667, .8353299804688E+04, -.1285592041016E+04, .1124491210938E+04},
 {     668, .8389152343750E+04, -.1270352050781E+04, .1111878662109E+04},
 {     669, .8371226562500E+04, -.1270352050781E+04, .1118185058594E+04},
 {     670, .8353299804688E+04, -.1270352050781E+04, .1124491210938E+04},
 {     671, .8371226562500E+04, -.1255112060547E+04, .1118185058594E+04},
 {     672, .8353299804688E+04, -.1255112060547E+04, .1124491210938E+04},
 {     673, .8380944335938E+04, -.1300832031250E+04, .1089363159180E+04},
 {     674, .8363556640625E+04, -.1300832031250E+04, .1097146728516E+04},
 {     675, .8346169921875E+04, -.1300832031250E+04, .1104930053711E+04},
 {     676, .8380944335938E+04, -.1285592041016E+04, .1089363159180E+04},
 {     677, .8363556640625E+04, -.1285592041016E+04, .1097146728516E+04},
 {     678, .8346169921875E+04, -.1285592041016E+04, .1104930053711E+04},
 {     679, .8380944335938E+04, -.1270352050781E+04, .1089363159180E+04},
 {     680, .8363556640625E+04, -.1270352050781E+04, .1097146728516E+04},
 {     681, .8346169921875E+04, -.1270352050781E+04, .1104930053711E+04},
 {     682, .8363556640625E+04, -.1255112060547E+04, .1097146728516E+04},
 {     683, .8346169921875E+04, -.1255112060547E+04, .1104930053711E+04},
 {     684, .8376604492188E+04, -.1300832031250E+04, .1077461914062E+04},
 {     685, .8359215820312E+04, -.1300832031250E+04, .1085245239258E+04},
 {     686, .8341830078125E+04, -.1300832031250E+04, .1093029174805E+04},
 {     687, .8376604492188E+04, -.1285592041016E+04, .1077461914062E+04},
 {     688, .8359215820312E+04, -.1285592041016E+04, .1085245239258E+04},
 {     689, .8341830078125E+04, -.1285592041016E+04, .1093029174805E+04},
 {     690, .8376604492188E+04, -.1270352050781E+04, .1077461914062E+04},
 {     691, .8359215820312E+04, -.1270352050781E+04, .1085245239258E+04},
 {     692, .8341830078125E+04, -.1270352050781E+04, .1093029174805E+04},
 {     693, .8359215820312E+04, -.1255112060547E+04, .1085245239258E+04},
 {     694, .8341830078125E+04, -.1255112060547E+04, .1093029174805E+04},
 {     695, .8372265625000E+04, -.1300832031250E+04, .1065559936523E+04},
 {     696, .8354876953125E+04, -.1300832031250E+04, .1073343994141E+04},
 {     697, .8337489257812E+04, -.1300832031250E+04, .1081127685547E+04},
 {     698, .8372265625000E+04, -.1285592041016E+04, .1065559936523E+04},
 {     699, .8354876953125E+04, -.1285592041016E+04, .1073343994141E+04},
 {     700, .8337489257812E+04, -.1285592041016E+04, .1081127685547E+04},
 {     701, .8372265625000E+04, -.1270352050781E+04, .1065559936523E+04},
 {     702, .8354876953125E+04, -.1270352050781E+04, .1073343994141E+04},
 {     703, .8337489257812E+04, -.1270352050781E+04, .1081127685547E+04},
 {     704, .8354876953125E+04, -.1255112060547E+04, .1073343994141E+04},
 {     705, .8337489257812E+04, -.1255112060547E+04, .1081127685547E+04},
 {     706, .8367925781250E+04, -.1300832031250E+04, .1053658691406E+04},
 {     707, .8350541015625E+04, -.1300832031250E+04, .1061442260742E+04},
 {     708, .8333151367188E+04, -.1300832031250E+04, .1069225830078E+04},
 {     709, .8367925781250E+04, -.1285592041016E+04, .1053658691406E+04},
 {     710, .8350541015625E+04, -.1285592041016E+04, .1061442260742E+04},
 {     711, .8333151367188E+04, -.1285592041016E+04, .1069225830078E+04},
 {     712, .8367925781250E+04, -.1270352050781E+04, .1053658691406E+04},
 {     713, .8350541015625E+04, -.1270352050781E+04, .1061442260742E+04},
 {     714, .8333151367188E+04, -.1270352050781E+04, .1069225830078E+04},
 {     715, .8350541015625E+04, -.1255112060547E+04, .1061442260742E+04},
 {     716, .8333151367188E+04, -.1255112060547E+04, .1069225830078E+04},
 {     717, .8363587890625E+04, -.1300832031250E+04, .1041757446289E+04},
 {     718, .8346200195312E+04, -.1300832031250E+04, .1049540527344E+04},
 {     719, .8328813476562E+04, -.1300832031250E+04, .1057324462891E+04},
 {     720, .8363587890625E+04, -.1285592041016E+04, .1041757446289E+04},
 {     721, .8346200195312E+04, -.1285592041016E+04, .1049540527344E+04},
 {     722, .8328813476562E+04, -.1285592041016E+04, .1057324462891E+04},
 {     723, .8363587890625E+04, -.1270352050781E+04, .1041757446289E+04},
 {     724, .8346200195312E+04, -.1270352050781E+04, .1049540527344E+04},
 {     725, .8328813476562E+04, -.1270352050781E+04, .1057324462891E+04},
 {     726, .8346200195312E+04, -.1255112060547E+04, .1049540527344E+04},
 {     727, .8328813476562E+04, -.1255112060547E+04, .1057324462891E+04},
 {     728, .8362500976562E+04, -.1300832031250E+04, .1038782226562E+04},
 {     729, .8345115234375E+04, -.1300832031250E+04, .1046565673828E+04},
 {     730, .8327726562500E+04, -.1300832031250E+04, .1054348999023E+04},
 {     731, .8362500976562E+04, -.1285592041016E+04, .1038782226562E+04},
 {     732, .8345115234375E+04, -.1285592041016E+04, .1046565673828E+04},
 {     733, .8327726562500E+04, -.1285592041016E+04, .1054348999023E+04},
 {     734, .8362500976562E+04, -.1270352050781E+04, .1038782226562E+04},
 {     735, .8345115234375E+04, -.1270352050781E+04, .1046565673828E+04},
 {     736, .8327726562500E+04, -.1270352050781E+04, .1054348999023E+04},
 {     737, .8345115234375E+04, -.1255112060547E+04, .1046565673828E+04},
 {     738, .8327726562500E+04, -.1255112060547E+04, .1054348999023E+04},
 {     739, .8358162109375E+04, -.1300832031250E+04, .1026880493164E+04},
 {     740, .8340775390625E+04, -.1300832031250E+04, .1034663818359E+04},
 {     741, .8323387695312E+04, -.1300832031250E+04, .1042448120117E+04},
 {     742, .8358162109375E+04, -.1285592041016E+04, .1026880493164E+04},
 {     743, .8340775390625E+04, -.1285592041016E+04, .1034663818359E+04},
 {     744, .8323387695312E+04, -.1285592041016E+04, .1042448120117E+04},
 {     745, .8358162109375E+04, -.1270352050781E+04, .1026880493164E+04},
 {     746, .8340775390625E+04, -.1270352050781E+04, .1034663818359E+04},
 {     747, .8323387695312E+04, -.1270352050781E+04, .1042448120117E+04},
 {     748, .8340775390625E+04, -.1255112060547E+04, .1034663818359E+04},
 {     749, .8323387695312E+04, -.1255112060547E+04, .1042448120117E+04},
 {     750, .8353825195312E+04, -.1300832031250E+04, .1014978881836E+04},
 {     751, .8336436523438E+04, -.1300832031250E+04, .1022763061523E+04},
 {     752, .8319047851562E+04, -.1300832031250E+04, .1030546630859E+04},
 {     753, .8353825195312E+04, -.1285592041016E+04, .1014978881836E+04},
 {     754, .8336436523438E+04, -.1285592041016E+04, .1022763061523E+04},
 {     755, .8319047851562E+04, -.1285592041016E+04, .1030546630859E+04},
 {     756, .8353825195312E+04, -.1270352050781E+04, .1014978881836E+04},
 {     757, .8336436523438E+04, -.1270352050781E+04, .1022763061523E+04},
 {     758, .8319047851562E+04, -.1270352050781E+04, .1030546630859E+04},
 {     759, .8336436523438E+04, -.1255112060547E+04, .1022763061523E+04},
 {     760, .8319047851562E+04, -.1255112060547E+04, .1030546630859E+04},
 {     761, .8352739257812E+04, -.1300832031250E+04, .1012003540039E+04},
 {     762, .8335351562500E+04, -.1300832031250E+04, .1019787292480E+04},
 {     763, .8317963867188E+04, -.1300832031250E+04, .1027571166992E+04},
 {     764, .8352739257812E+04, -.1285592041016E+04, .1012003540039E+04},
 {     765, .8335351562500E+04, -.1285592041016E+04, .1019787292480E+04},
 {     766, .8317963867188E+04, -.1285592041016E+04, .1027571166992E+04},
 {     767, .8352739257812E+04, -.1270352050781E+04, .1012003540039E+04},
 {     768, .8335351562500E+04, -.1270352050781E+04, .1019787292480E+04},
 {     769, .8317963867188E+04, -.1270352050781E+04, .1027571166992E+04},
 {     770, .8335351562500E+04, -.1255112060547E+04, .1019787292480E+04},
 {     771, .8317963867188E+04, -.1255112060547E+04, .1027571166992E+04},
 {     772, .8348402343750E+04, -.1300832031250E+04, .1000102050781E+04},
 {     773, .8331013671875E+04, -.1300832031250E+04, .1007885559082E+04},
 {     774, .8313628906250E+04, -.1300832031250E+04, .1015669189453E+04},
 {     775, .8348402343750E+04, -.1285592041016E+04, .1000102050781E+04},
 {     776, .8331013671875E+04, -.1285592041016E+04, .1007885559082E+04},
 {     777, .8313628906250E+04, -.1285592041016E+04, .1015669189453E+04},
 {     778, .8348402343750E+04, -.1270352050781E+04, .1000102050781E+04},
 {     779, .8331013671875E+04, -.1270352050781E+04, .1007885559082E+04},
 {     780, .8313628906250E+04, -.1270352050781E+04, .1015669189453E+04},
 {     781, .8331013671875E+04, -.1255112060547E+04, .1007885559082E+04},
 {     782, .8313628906250E+04, -.1255112060547E+04, .1015669189453E+04},
 {     783, .8344062500000E+04, -.1300832031250E+04, .9882008666992E+03},
 {     784, .8326674804688E+04, -.1300832031250E+04, .9959842529297E+03},
 {     785, .8309287109375E+04, -.1300832031250E+04, .1003768188477E+04},
 {     786, .8344062500000E+04, -.1285592041016E+04, .9882008666992E+03},
 {     787, .8326674804688E+04, -.1285592041016E+04, .9959842529297E+03},
 {     788, .8309287109375E+04, -.1285592041016E+04, .1003768188477E+04},
 {     789, .8344062500000E+04, -.1270352050781E+04, .9882008666992E+03},
 {     790, .8326674804688E+04, -.1270352050781E+04, .9959842529297E+03},
 {     791, .8309287109375E+04, -.1270352050781E+04, .1003768188477E+04},
 {     792, .8326674804688E+04, -.1255112060547E+04, .9959842529297E+03},
 {     793, .8309287109375E+04, -.1255112060547E+04, .1003768188477E+04},
 {     794, .8342976562500E+04, -.1300832031250E+04, .9852258300781E+03},
 {     795, .8325590820312E+04, -.1300832031250E+04, .9930088500977E+03},
 {     796, .8308203125000E+04, -.1300832031250E+04, .1000792480469E+04},
 {     797, .8342976562500E+04, -.1285592041016E+04, .9852258300781E+03},
 {     798, .8325590820312E+04, -.1285592041016E+04, .9930088500977E+03},
 {     799, .8308203125000E+04, -.1285592041016E+04, .1000792480469E+04},
 {     800, .8342976562500E+04, -.1270352050781E+04, .9852258300781E+03},
 {     801, .8325590820312E+04, -.1270352050781E+04, .9930088500977E+03},
 {     802, .8308203125000E+04, -.1270352050781E+04, .1000792480469E+04},
 {     803, .8325590820312E+04, -.1255112060547E+04, .9930088500977E+03},
 {     804, .8308203125000E+04, -.1255112060547E+04, .1000792480469E+04},
 {     805, .8338638671875E+04, -.1300832031250E+04, .9733240966797E+03},
 {     806, .8321250976562E+04, -.1300832031250E+04, .9811076660156E+03},
 {     807, .8303865234375E+04, -.1300832031250E+04, .9888907470703E+03},
 {     808, .8338638671875E+04, -.1285592041016E+04, .9733240966797E+03},
 {     809, .8321250976562E+04, -.1285592041016E+04, .9811076660156E+03},
 {     810, .8303865234375E+04, -.1285592041016E+04, .9888907470703E+03},
 {     811, .8338638671875E+04, -.1270352050781E+04, .9733240966797E+03},
 {     812, .8321250976562E+04, -.1270352050781E+04, .9811076660156E+03},
 {     813, .8303865234375E+04, -.1270352050781E+04, .9888907470703E+03},
 {     814, .8321250976562E+04, -.1255112060547E+04, .9811076660156E+03},
 {     815, .8303865234375E+04, -.1255112060547E+04, .9888907470703E+03},
 {     816, .8334298828125E+04, -.1300832031250E+04, .9614224853516E+03},
 {     817, .8316912109375E+04, -.1300832031250E+04, .9692061767578E+03},
 {     818, .8299524414062E+04, -.1300832031250E+04, .9769899291992E+03},
 {     819, .8334298828125E+04, -.1285592041016E+04, .9614224853516E+03},
 {     820, .8316912109375E+04, -.1285592041016E+04, .9692061767578E+03},
 {     821, .8299524414062E+04, -.1285592041016E+04, .9769899291992E+03},
 {     822, .8334298828125E+04, -.1270352050781E+04, .9614224853516E+03},
 {     823, .8316912109375E+04, -.1270352050781E+04, .9692061767578E+03},
 {     824, .8299524414062E+04, -.1270352050781E+04, .9769899291992E+03},
 {     825, .8316912109375E+04, -.1255112060547E+04, .9692061767578E+03},
 {     826, .8299524414062E+04, -.1255112060547E+04, .9769899291992E+03},
 {     827, .8333214843750E+04, -.1300832031250E+04, .9584469604492E+03},
 {     828, .8315827148438E+04, -.1300832031250E+04, .9662307128906E+03},
 {     829, .8298438476562E+04, -.1300832031250E+04, .9740146484375E+03},
 {     830, .8333214843750E+04, -.1285592041016E+04, .9584469604492E+03},
 {     831, .8315827148438E+04, -.1285592041016E+04, .9662307128906E+03},
 {     832, .8298438476562E+04, -.1285592041016E+04, .9740146484375E+03},
 {     833, .8333214843750E+04, -.1270352050781E+04, .9584469604492E+03},
 {     834, .8315827148438E+04, -.1270352050781E+04, .9662307128906E+03},
 {     835, .8298438476562E+04, -.1270352050781E+04, .9740146484375E+03},
 {     836, .8315827148438E+04, -.1255112060547E+04, .9662307128906E+03},
 {     837, .8298438476562E+04, -.1255112060547E+04, .9740146484375E+03},
 {     838, .8328873046875E+04, -.1300832031250E+04, .9465457763672E+03},
 {     839, .8311488281250E+04, -.1300832031250E+04, .9543291625977E+03},
 {     840, .8294100585938E+04, -.1300832031250E+04, .9621129150391E+03},
 {     841, .8328873046875E+04, -.1285592041016E+04, .9465457763672E+03},
 {     842, .8311488281250E+04, -.1285592041016E+04, .9543291625977E+03},
 {     843, .8294100585938E+04, -.1285592041016E+04, .9621129150391E+03},
 {     844, .8328873046875E+04, -.1270352050781E+04, .9465457763672E+03},
 {     845, .8311488281250E+04, -.1270352050781E+04, .9543291625977E+03},
 {     846, .8294100585938E+04, -.1270352050781E+04, .9621129150391E+03},
 {     847, .8311488281250E+04, -.1255112060547E+04, .9543291625977E+03},
 {     848, .8294100585938E+04, -.1255112060547E+04, .9621129150391E+03},
 {     849, .8307148437500E+04, -.1300832031250E+04, .9424277954102E+03},
 {     850, .8289761718750E+04, -.1300832031250E+04, .9502113647461E+03},
 {     851, .8307148437500E+04, -.1285592041016E+04, .9424277954102E+03},
 {     852, .8289761718750E+04, -.1285592041016E+04, .9502113647461E+03},
 {     853, .8307148437500E+04, -.1270352050781E+04, .9424277954102E+03},
 {     854, .8289761718750E+04, -.1270352050781E+04, .9502113647461E+03},
 {     855, .8307148437500E+04, -.1255112060547E+04, .9424277954102E+03},
 {     856, .8289761718750E+04, -.1255112060547E+04, .9502113647461E+03},
 {     857, .8305412109375E+04, -.1300832031250E+04, .9376674194336E+03},
 {     858, .8288026367188E+04, -.1300832031250E+04, .9454510498047E+03},
 {     859, .8305412109375E+04, -.1285592041016E+04, .9376674194336E+03},
 {     860, .8288026367188E+04, -.1285592041016E+04, .9454510498047E+03},
 {     861, .8305412109375E+04, -.1270352050781E+04, .9376674194336E+03},
 {     862, .8288026367188E+04, -.1270352050781E+04, .9454510498047E+03},
 {     863, .8305412109375E+04, -.1255112060547E+04, .9376674194336E+03},
 {     864, .8288026367188E+04, -.1255112060547E+04, .9454510498047E+03},
 {     865, .8380511718750E+04, -.1248762084961E+04, .1143656005859E+04},
 {     866, .8362046875000E+04, -.1248762084961E+04, .1148484252930E+04},
 {     867, .8378894531250E+04, -.1248762084961E+04, .1139223510742E+04},
 {     868, .8360430664062E+04, -.1248762084961E+04, .1144051879883E+04},
 {     869, .8371226562500E+04, -.1248762084961E+04, .1118185058594E+04},
 {     870, .8353299804688E+04, -.1248762084961E+04, .1124491210938E+04},
 {     871, .8363556640625E+04, -.1248762084961E+04, .1097146728516E+04},
 {     872, .8346169921875E+04, -.1248762084961E+04, .1104930053711E+04},
 {     873, .8359215820312E+04, -.1248762084961E+04, .1085245239258E+04},
 {     874, .8341830078125E+04, -.1248762084961E+04, .1093029174805E+04},
 {     875, .8354876953125E+04, -.1248762084961E+04, .1073343994141E+04},
 {     876, .8337489257812E+04, -.1248762084961E+04, .1081127685547E+04},
 {     877, .8350541015625E+04, -.1248762084961E+04, .1061442260742E+04},
 {     878, .8333151367188E+04, -.1248762084961E+04, .1069225830078E+04},
 {     879, .8346200195312E+04, -.1248762084961E+04, .1049540527344E+04},
 {     880, .8328813476562E+04, -.1248762084961E+04, .1057324462891E+04},
 {     881, .8345115234375E+04, -.1248762084961E+04, .1046565673828E+04},
 {     882, .8327726562500E+04, -.1248762084961E+04, .1054348999023E+04},
 {     883, .8340775390625E+04, -.1248762084961E+04, .1034663818359E+04},
 {     884, .8323387695312E+04, -.1248762084961E+04, .1042448120117E+04},
 {     885, .8336436523438E+04, -.1248762084961E+04, .1022763061523E+04},
 {     886, .8319047851562E+04, -.1248762084961E+04, .1030546630859E+04},
 {     887, .8335351562500E+04, -.1248762084961E+04, .1019787292480E+04},
 {     888, .8317963867188E+04, -.1248762084961E+04, .1027571166992E+04},
 {     889, .8331013671875E+04, -.1248762084961E+04, .1007885559082E+04},
 {     890, .8313628906250E+04, -.1248762084961E+04, .1015669189453E+04},
 {     891, .8326674804688E+04, -.1248762084961E+04, .9959842529297E+03},
 {     892, .8309287109375E+04, -.1248762084961E+04, .1003768188477E+04},
 {     893, .8325590820312E+04, -.1248762084961E+04, .9930088500977E+03},
 {     894, .8308203125000E+04, -.1248762084961E+04, .1000792480469E+04},
 {     895, .8321250976562E+04, -.1248762084961E+04, .9811076660156E+03},
 {     896, .8303865234375E+04, -.1248762084961E+04, .9888907470703E+03},
 {     897, .8316912109375E+04, -.1248762084961E+04, .9692061767578E+03},
 {     898, .8299524414062E+04, -.1248762084961E+04, .9769899291992E+03},
 {     899, .8315827148438E+04, -.1248762084961E+04, .9662307128906E+03},
 {     900, .8298438476562E+04, -.1248762084961E+04, .9740146484375E+03},
 {     901, .8311488281250E+04, -.1248762084961E+04, .9543291625977E+03},
 {     902, .8294100585938E+04, -.1248762084961E+04, .9621129150391E+03},
 {     903, .8307148437500E+04, -.1248762084961E+04, .9424277954102E+03},
 {     904, .8289761718750E+04, -.1248762084961E+04, .9502113647461E+03},
 {     905, .8305412109375E+04, -.1248762084961E+04, .9376674194336E+03},
 {     906, .8288026367188E+04, -.1248762084961E+04, .9454510498047E+03},
 {     907, .8380511718750E+04, -.1234791992188E+04, .1143656005859E+04},
 {     908, .8362046875000E+04, -.1234791992188E+04, .1148484252930E+04},
 {     909, .8378894531250E+04, -.1234791992188E+04, .1139223510742E+04},
 {     910, .8360430664062E+04, -.1234791992188E+04, .1144051879883E+04},
 {     911, .8384388671875E+04, -.1248762084961E+04, .1031717529297E+04},
 {     912, .8370479492188E+04, -.1248762084961E+04, .1037944335938E+04},
 {     913, .8384388671875E+04, -.1243682006836E+04, .1031717529297E+04},
 {     914, .8370479492188E+04, -.1243682006836E+04, .1037944335938E+04},
 {     915, .8382312500000E+04, -.1248762084961E+04, .1027081298828E+04},
 {     916, .8368404296875E+04, -.1248762084961E+04, .1033307861328E+04},
 {     917, .8382312500000E+04, -.1243682006836E+04, .1027081298828E+04},
 {     918, .8368404296875E+04, -.1243682006836E+04, .1033307861328E+04},
 {     919, .8366747070312E+04, -.1248762084961E+04, .9923063354492E+03},
 {     920, .8352836914062E+04, -.1248762084961E+04, .9985332031250E+03},
 {     921, .8366747070312E+04, -.1243682006836E+04, .9923063354492E+03},
 {     922, .8352836914062E+04, -.1243682006836E+04, .9985332031250E+03},
 {     923, .8351178710938E+04, -.1248762084961E+04, .9575316162109E+03},
 {     924, .8337269531250E+04, -.1248762084961E+04, .9637586669922E+03},
 {     925, .8351178710938E+04, -.1243682006836E+04, .9575316162109E+03},
 {     926, .8337269531250E+04, -.1243682006836E+04, .9637586669922E+03},
 {     927, .8349103515625E+04, -.1248762084961E+04, .9528950195312E+03},
 {     928, .8335194335938E+04, -.1248762084961E+04, .9591218261719E+03},
 {     929, .8349103515625E+04, -.1243682006836E+04, .9528950195312E+03},
 {     930, .8335194335938E+04, -.1243682006836E+04, .9591218261719E+03},
 {     931, .8373957031250E+04, -.1232252075195E+04, .1036387451172E+04},
 {     932, .8365261718750E+04, -.1234156982422E+04, .1040279541016E+04},
 {     933, .8363524414062E+04, -.1220822021484E+04, .1041057495117E+04},
 {     934, .8360046875000E+04, -.1224632080078E+04, .1042614624023E+04},
 {     935, .8371880859375E+04, -.1232252075195E+04, .1031751464844E+04},
 {     936, .8363188476562E+04, -.1234156982422E+04, .1035642456055E+04},
 {     937, .8361448242188E+04, -.1220822021484E+04, .1036421264648E+04},
 {     938, .8357970703125E+04, -.1224632080078E+04, .1037977783203E+04},
 {     939, .8340746093750E+04, -.1232252075195E+04, .9622017822266E+03},
 {     940, .8332054687500E+04, -.1234157104492E+04, .9660935668945E+03},
 {     941, .8330314453125E+04, -.1220822021484E+04, .9668717651367E+03},
 {     942, .8326837890625E+04, -.1224632080078E+04, .9684285888672E+03},
 {     943, .8338672851562E+04, -.1232252075195E+04, .9575651855469E+03},
 {     944, .8329978515625E+04, -.1234157104492E+04, .9614570312500E+03},
 {     945, .8328240234375E+04, -.1220822021484E+04, .9622355957031E+03},
 {     946, .8324761718750E+04, -.1224632080078E+04, .9637919311523E+03},
 {     947, .8356571289062E+04, -.1248762084961E+04, .1044170898438E+04},
 {     948, .8342660156250E+04, -.1248762084961E+04, .1050398315430E+04},
 {     949, .8356571289062E+04, -.1243682006836E+04, .1044170898438E+04},
 {     950, .8342660156250E+04, -.1243682006836E+04, .1050398315430E+04},
 {     951, .8354495117188E+04, -.1248762084961E+04, .1039534545898E+04},
 {     952, .8340585937500E+04, -.1248762084961E+04, .1045761230469E+04},
 {     953, .8354495117188E+04, -.1243682006836E+04, .1039534545898E+04},
 {     954, .8340585937500E+04, -.1243682006836E+04, .1045761230469E+04},
 {     955, .8338926757812E+04, -.1248762084961E+04, .1004759948730E+04},
 {     956, .8325017578125E+04, -.1248762084961E+04, .1010986938477E+04},
 {     957, .8338926757812E+04, -.1243682006836E+04, .1004759948730E+04},
 {     958, .8325017578125E+04, -.1243682006836E+04, .1010986938477E+04},
 {     959, .8323359375000E+04, -.1248762084961E+04, .9699854736328E+03},
 {     960, .8309452148438E+04, -.1248762084961E+04, .9762120361328E+03},
 {     961, .8323359375000E+04, -.1243682006836E+04, .9699854736328E+03},
 {     962, .8309452148438E+04, -.1243682006836E+04, .9762120361328E+03},
 {     963, .8321284179688E+04, -.1248762084961E+04, .9653489379883E+03},
 {     964, .8307374023438E+04, -.1248762084961E+04, .9715754394531E+03},
 {     965, .8321284179688E+04, -.1243682006836E+04, .9653489379883E+03},
 {     966, .8307374023438E+04, -.1243682006836E+04, .9715754394531E+03},
 {     967, .8353959960938E+04, -.1231299560547E+04, .1045338745117E+04},
 {     968, .8342660156250E+04, -.1228442016602E+04, .1050398315430E+04},
 {     969, .8351354492188E+04, -.1218916992188E+04, .1046506103516E+04},
 {     970, .8342660156250E+04, -.1213202026367E+04, .1050398315430E+04},
 {     971, .8351885742188E+04, -.1231299560547E+04, .1040701782227E+04},
 {     972, .8340585937500E+04, -.1228442016602E+04, .1045761230469E+04},
 {     973, .8349277343750E+04, -.1218916992188E+04, .1041869873047E+04},
 {     974, .8340585937500E+04, -.1213202026367E+04, .1045761230469E+04},
 {     975, .8320751953125E+04, -.1231299560547E+04, .9711528320312E+03},
 {     976, .8309452148438E+04, -.1228442016602E+04, .9762120361328E+03},
 {     977, .8318144531250E+04, -.1218916992188E+04, .9723205566406E+03},
 {     978, .8309452148438E+04, -.1213202026367E+04, .9762120361328E+03},
 {     979, .8318675781250E+04, -.1231299560547E+04, .9665163574219E+03},
 {     980, .8307374023438E+04, -.1228442016602E+04, .9715754394531E+03},
 {     981, .8316069335938E+04, -.1218916992188E+04, .9676834716797E+03},
 {     982, .8307374023438E+04, -.1213202026367E+04, .9715754394531E+03},
 {     983, .8353090820312E+04, -.1209392089844E+04, .1045728027344E+04},
 {     984, .8342660156250E+04, -.1197962036133E+04, .1050398315430E+04},
 {     985, .8351016601562E+04, -.1209392089844E+04, .1041091796875E+04},
 {     986, .8340585937500E+04, -.1197962036133E+04, .1045761230469E+04},
 {     987, .8319881835938E+04, -.1209392089844E+04, .9715419921875E+03},
 {     988, .8309452148438E+04, -.1197962036133E+04, .9762120361328E+03},
 {     989, .8317806640625E+04, -.1209392089844E+04, .9669058837891E+03},
 {     990, .8307374023438E+04, -.1197962036133E+04, .9715754394531E+03},
 {     991, .8338023437500E+04, -.1248762084961E+04, .1052473632812E+04},
 {     992, .8338023437500E+04, -.1243682006836E+04, .1052473632812E+04},
 {     993, .8335948242188E+04, -.1248762084961E+04, .1047837280273E+04},
 {     994, .8335948242188E+04, -.1243682006836E+04, .1047837280273E+04},
 {     995, .8320379882812E+04, -.1248762084961E+04, .1013062622070E+04},
 {     996, .8320379882812E+04, -.1243682006836E+04, .1013062622070E+04},
 {     997, .8304811523438E+04, -.1248762084961E+04, .9782882080078E+03},
 {     998, .8304811523438E+04, -.1243682006836E+04, .9782882080078E+03},
 {     999, .8302738281250E+04, -.1248762084961E+04, .9736513061523E+03},
 {    1000, .8302738281250E+04, -.1243682006836E+04, .9736513061523E+03},
 {    1001, .8338023437500E+04, -.1228442016602E+04, .1052473632812E+04},
 {    1002, .8338023437500E+04, -.1213202026367E+04, .1052473632812E+04},
 {    1003, .8335948242188E+04, -.1228442016602E+04, .1047837280273E+04},
 {    1004, .8335948242188E+04, -.1213202026367E+04, .1047837280273E+04},
 {    1005, .8325017578125E+04, -.1228442016602E+04, .1010986938477E+04},
 {    1006, .8320379882812E+04, -.1228442016602E+04, .1013062622070E+04},
 {    1007, .8325017578125E+04, -.1213202026367E+04, .1010986938477E+04},
 {    1008, .8320379882812E+04, -.1213202026367E+04, .1013062622070E+04},
 {    1009, .8304811523438E+04, -.1228442016602E+04, .9782882080078E+03},
 {    1010, .8304811523438E+04, -.1213202026367E+04, .9782882080078E+03},
 {    1011, .8302738281250E+04, -.1228442016602E+04, .9736513061523E+03},
 {    1012, .8302738281250E+04, -.1213202026367E+04, .9736513061523E+03},
 {    1013, .8338023437500E+04, -.1197962036133E+04, .1052473632812E+04},
 {    1014, .8335948242188E+04, -.1197962036133E+04, .1047837280273E+04},
 {    1015, .8325017578125E+04, -.1197962036133E+04, .1010986938477E+04},
 {    1016, .8320379882812E+04, -.1197962036133E+04, .1013062622070E+04},
 {    1017, .8304811523438E+04, -.1197962036133E+04, .9782882080078E+03},
 {    1018, .8302738281250E+04, -.1197962036133E+04, .9736513061523E+03},
 {    1019, .8326053710938E+04, -.1213202026367E+04, .1013305480957E+04},
 {    1020, .8328127929688E+04, -.1213202026367E+04, .1017941894531E+04},
 {    1021, .8325664062500E+04, -.1211216186523E+04, .1012432006836E+04},
 {    1022, .8326958984375E+04, -.1207244506836E+04, .1015323242188E+04},
 {    1023, .8324786132812E+04, -.1210725708008E+04, .1010471191406E+04},
 {    1024, .8324324218750E+04, -.1205773071289E+04, .1009438903809E+04},
 {    1025, .8324083007812E+04, -.1212099975586E+04, .1008897949219E+04},
 {    1026, .8322211914062E+04, -.1209895874023E+04, .1004720581055E+04},
 {    1027, .8324083007812E+04, -.1214304077148E+04, .1008897949219E+04},
 {    1028, .8322211914062E+04, -.1216508300781E+04, .1004720581055E+04},
 {    1029, .8324786132812E+04, -.1215678344727E+04, .1010471191406E+04},
 {    1030, .8324324218750E+04, -.1220630981445E+04, .1009438903809E+04},
 {    1031, .8325664062500E+04, -.1215187866211E+04, .1012432006836E+04},
 {    1032, .8326958984375E+04, -.1219159545898E+04, .1015323242188E+04},
 {    1033, .8321416992188E+04, -.1213202026367E+04, .1015381103516E+04},
 {    1034, .8323493164062E+04, -.1213202026367E+04, .1020017456055E+04},
 {    1035, .8321026367188E+04, -.1211216186523E+04, .1014508178711E+04},
 {    1036, .8322321289062E+04, -.1207244506836E+04, .1017398864746E+04},
 {    1037, .8320150390625E+04, -.1210725708008E+04, .1012546752930E+04},
 {    1038, .8319685546875E+04, -.1205773071289E+04, .1011514892578E+04},
 {    1039, .8319444335938E+04, -.1212099975586E+04, .1010974121094E+04},
 {    1040, .8317574218750E+04, -.1209895874023E+04, .1006796630859E+04},
 {    1041, .8319444335938E+04, -.1214304077148E+04, .1010974121094E+04},
 {    1042, .8317574218750E+04, -.1216508300781E+04, .1006796630859E+04},
 {    1043, .8320150390625E+04, -.1215678344727E+04, .1012546752930E+04},
 {    1044, .8319685546875E+04, -.1220630981445E+04, .1011514892578E+04},
 {    1045, .8321026367188E+04, -.1215187866211E+04, .1014508178711E+04},
 {    1046, .8322321289062E+04, -.1219159545898E+04, .1017398864746E+04},
 {    1047, .8319098632812E+04, -.1213202026367E+04, .1016419250488E+04},
 {    1048, .8321175781250E+04, -.1213202026367E+04, .1021055053711E+04},
 {    1049, .8318708007812E+04, -.1211216186523E+04, .1015545959473E+04},
 {    1050, .8320002929688E+04, -.1207244506836E+04, .1018436767578E+04},
 {    1051, .8317830078125E+04, -.1210725708008E+04, .1013584838867E+04},
 {    1052, .8317369140625E+04, -.1205773071289E+04, .1012552368164E+04},
 {    1053, .8317125000000E+04, -.1212099975586E+04, .1012011840820E+04},
 {    1054, .8315255859375E+04, -.1209895874023E+04, .1007834228516E+04},
 {    1055, .8317125000000E+04, -.1214304077148E+04, .1012011840820E+04},
 {    1056, .8315255859375E+04, -.1216508300781E+04, .1007834228516E+04},
 {    1057, .8317830078125E+04, -.1215678344727E+04, .1013584838867E+04},
 {    1058, .8317369140625E+04, -.1220630981445E+04, .1012552368164E+04},
 {    1059, .8318708007812E+04, -.1215187866211E+04, .1015545959473E+04},
 {    1060, .8320002929688E+04, -.1219159545898E+04, .1018437011719E+04},
 {    1061, .8314462890625E+04, -.1213202026367E+04, .1018493774414E+04},
 {    1062, .8316538085938E+04, -.1213202026367E+04, .1023131286621E+04},
 {    1063, .8314072265625E+04, -.1211216186523E+04, .1017621398926E+04},
 {    1064, .8315366210938E+04, -.1207244506836E+04, .1020512695312E+04},
 {    1065, .8313195312500E+04, -.1210725708008E+04, .1015660217285E+04},
 {    1066, .8312730468750E+04, -.1205773071289E+04, .1014628417969E+04},
 {    1067, .8312488281250E+04, -.1212099975586E+04, .1014087524414E+04},
 {    1068, .8310619140625E+04, -.1209895874023E+04, .1009909912109E+04},
 {    1069, .8312488281250E+04, -.1214304077148E+04, .1014087524414E+04},
 {    1070, .8310619140625E+04, -.1216508300781E+04, .1009909912109E+04},
 {    1071, .8313195312500E+04, -.1215678344727E+04, .1015660217285E+04},
 {    1072, .8312730468750E+04, -.1220630981445E+04, .1014628417969E+04},
 {    1073, .8314072265625E+04, -.1215187866211E+04, .1017621398926E+04},
 {    1074, .8315366210938E+04, -.1219159545898E+04, .1020512695312E+04},
 {    1075, .8330514648438E+04, -.1238602050781E+04, .1041920043945E+04},
 {    1076, .8325879882812E+04, -.1238602050781E+04, .1043995605469E+04},
 {    1077, .8330514648438E+04, -.1225902099609E+04, .1041920043945E+04},
 {    1078, .8325879882812E+04, -.1225902099609E+04, .1043995605469E+04},
 {    1079, .8330514648438E+04, -.1213202026367E+04, .1041920043945E+04},
 {    1080, .8325879882812E+04, -.1213202026367E+04, .1043995605469E+04},
 {    1081, .8328440429688E+04, -.1238602050781E+04, .1037283203125E+04},
 {    1082, .8323803710938E+04, -.1238602050781E+04, .1039358886719E+04},
 {    1083, .8328440429688E+04, -.1225902099609E+04, .1037283203125E+04},
 {    1084, .8323803710938E+04, -.1225902099609E+04, .1039358886719E+04},
 {    1085, .8328440429688E+04, -.1213202026367E+04, .1037283203125E+04},
 {    1086, .8323803710938E+04, -.1213202026367E+04, .1039358886719E+04},
 {    1087, .8318061523438E+04, -.1238602050781E+04, .1014100219727E+04},
 {    1088, .8313423828125E+04, -.1238602050781E+04, .1016175903320E+04},
 {    1089, .8318061523438E+04, -.1225902099609E+04, .1014100219727E+04},
 {    1090, .8313423828125E+04, -.1225902099609E+04, .1016175903320E+04},
 {    1091, .8318061523438E+04, -.1213202026367E+04, .1014100219727E+04},
 {    1092, .8313423828125E+04, -.1213202026367E+04, .1016175903320E+04},
 {    1093, .8307683593750E+04, -.1238602050781E+04, .9909177246094E+03},
 {    1094, .8303047851562E+04, -.1238602050781E+04, .9929933471680E+03},
 {    1095, .8307683593750E+04, -.1225902099609E+04, .9909177246094E+03},
 {    1096, .8303047851562E+04, -.1225902099609E+04, .9929933471680E+03},
 {    1097, .8307683593750E+04, -.1213202026367E+04, .9909177246094E+03},
 {    1098, .8303047851562E+04, -.1213202026367E+04, .9929933471680E+03},
 {    1099, .8305609375000E+04, -.1238602050781E+04, .9862807617188E+03},
 {    1100, .8300971679688E+04, -.1238602050781E+04, .9883564453125E+03},
 {    1101, .8305609375000E+04, -.1225902099609E+04, .9862807617188E+03},
 {    1102, .8300971679688E+04, -.1225902099609E+04, .9883564453125E+03},
 {    1103, .8305609375000E+04, -.1213202026367E+04, .9862807617188E+03},
 {    1104, .8300971679688E+04, -.1213202026367E+04, .9883564453125E+03},
 {    1105, .8330514648438E+04, -.1200502075195E+04, .1041920043945E+04},
 {    1106, .8325879882812E+04, -.1200502075195E+04, .1043995605469E+04},
 {    1107, .8328440429688E+04, -.1200502075195E+04, .1037283203125E+04},
 {    1108, .8323803710938E+04, -.1200502075195E+04, .1039358886719E+04},
 {    1109, .8318061523438E+04, -.1200502075195E+04, .1014100219727E+04},
 {    1110, .8313423828125E+04, -.1200502075195E+04, .1016175903320E+04},
 {    1111, .8307683593750E+04, -.1200502075195E+04, .9909177246094E+03},
 {    1112, .8303047851562E+04, -.1200502075195E+04, .9929933471680E+03},
 {    1113, .8305609375000E+04, -.1200502075195E+04, .9862807617188E+03},
 {    1114, .8300971679688E+04, -.1200502075195E+04, .9883564453125E+03},
 {    1115, .8330514648438E+04, -.1187802001953E+04, .1041920043945E+04},
 {    1116, .8325879882812E+04, -.1187802001953E+04, .1043995605469E+04},
 {    1117, .8328440429688E+04, -.1187802001953E+04, .1037283203125E+04},
 {    1118, .8323803710938E+04, -.1187802001953E+04, .1039358886719E+04},
 {    1119, .8318061523438E+04, -.1187802001953E+04, .1014100219727E+04},
 {    1120, .8313423828125E+04, -.1187802001953E+04, .1016175903320E+04},
 {    1121, .8307683593750E+04, -.1187802001953E+04, .9909177246094E+03},
 {    1122, .8303047851562E+04, -.1187802001953E+04, .9929933471680E+03},
 {    1123, .8305609375000E+04, -.1187802001953E+04, .9862807617188E+03},
 {    1124, .8300971679688E+04, -.1187802001953E+04, .9883564453125E+03},
 {    1125, .8314288085938E+04, -.1238602050781E+04, .1049184570312E+04},
 {    1126, .8302695312500E+04, -.1238602050781E+04, .1054373779297E+04},
 {    1127, .8314288085938E+04, -.1225902099609E+04, .1049184570312E+04},
 {    1128, .8302695312500E+04, -.1225902099609E+04, .1054373779297E+04},
 {    1129, .8314288085938E+04, -.1213202026367E+04, .1049184570312E+04},
 {    1130, .8302695312500E+04, -.1213202026367E+04, .1054373779297E+04},
 {    1131, .8312212890625E+04, -.1238602050781E+04, .1044547851562E+04},
 {    1132, .8300619140625E+04, -.1238602050781E+04, .1049737304688E+04},
 {    1133, .8312212890625E+04, -.1225902099609E+04, .1044547851562E+04},
 {    1134, .8300619140625E+04, -.1225902099609E+04, .1049737304688E+04},
 {    1135, .8312212890625E+04, -.1213202026367E+04, .1044547851562E+04},
 {    1136, .8300619140625E+04, -.1213202026367E+04, .1049737304688E+04},
 {    1137, .8291455078125E+04, -.1238602050781E+04, .9981821289062E+03},
 {    1138, .8279864257812E+04, -.1238602050781E+04, .1003370788574E+04},
 {    1139, .8291455078125E+04, -.1225902099609E+04, .9981821289062E+03},
 {    1140, .8279864257812E+04, -.1225902099609E+04, .1003370788574E+04},
 {    1141, .8291455078125E+04, -.1213202026367E+04, .9981821289062E+03},
 {    1142, .8279864257812E+04, -.1213202026367E+04, .1003370788574E+04},
 {    1143, .8289379882812E+04, -.1238602050781E+04, .9935454711914E+03},
 {    1144, .8277790039062E+04, -.1238602050781E+04, .9987342529297E+03},
 {    1145, .8289379882812E+04, -.1225902099609E+04, .9935454711914E+03},
 {    1146, .8277790039062E+04, -.1225902099609E+04, .9987342529297E+03},
 {    1147, .8289379882812E+04, -.1213202026367E+04, .9935454711914E+03},
 {    1148, .8277790039062E+04, -.1213202026367E+04, .9987342529297E+03},
 {    1149, .8314288085938E+04, -.1200502075195E+04, .1049184570312E+04},
 {    1150, .8302695312500E+04, -.1200502075195E+04, .1054373779297E+04},
 {    1151, .8312212890625E+04, -.1200502075195E+04, .1044547851562E+04},
 {    1152, .8300619140625E+04, -.1200502075195E+04, .1049737304688E+04},
 {    1153, .8291455078125E+04, -.1200502075195E+04, .9981821289062E+03},
 {    1154, .8279864257812E+04, -.1200502075195E+04, .1003370788574E+04},
 {    1155, .8289379882812E+04, -.1200502075195E+04, .9935454711914E+03},
 {    1156, .8277790039062E+04, -.1200502075195E+04, .9987342529297E+03},
 {    1157, .8314288085938E+04, -.1187802001953E+04, .1049184570312E+04},
 {    1158, .8302695312500E+04, -.1187802001953E+04, .1054373779297E+04},
 {    1159, .8312212890625E+04, -.1187802001953E+04, .1044547851562E+04},
 {    1160, .8300619140625E+04, -.1187802001953E+04, .1049737304688E+04},
 {    1161, .8291455078125E+04, -.1187802001953E+04, .9981821289062E+03},
 {    1162, .8279864257812E+04, -.1187802001953E+04, .1003370788574E+04},
 {    1163, .8289379882812E+04, -.1187802001953E+04, .9935454711914E+03},
 {    1164, .8277790039062E+04, -.1187802001953E+04, .9987342529297E+03},
 {    1165, .8291105468750E+04, -.1238602050781E+04, .1059562866211E+04},
 {    1166, .8291105468750E+04, -.1225902099609E+04, .1059562866211E+04},
 {    1167, .8291105468750E+04, -.1213202026367E+04, .1059562866211E+04},
 {    1168, .8289028320312E+04, -.1238602050781E+04, .1054926269531E+04},
 {    1169, .8289028320312E+04, -.1225902099609E+04, .1054926269531E+04},
 {    1170, .8289028320312E+04, -.1213202026367E+04, .1054926269531E+04},
 {    1171, .8268272460938E+04, -.1238602050781E+04, .1008559814453E+04},
 {    1172, .8268272460938E+04, -.1225902099609E+04, .1008559814453E+04},
 {    1173, .8268272460938E+04, -.1213202026367E+04, .1008559814453E+04},
 {    1174, .8266196289062E+04, -.1238602050781E+04, .1003923828125E+04},
 {    1175, .8266196289062E+04, -.1225902099609E+04, .1003923828125E+04},
 {    1176, .8266196289062E+04, -.1213202026367E+04, .1003923828125E+04},
 {    1177, .8291105468750E+04, -.1200502075195E+04, .1059562866211E+04},
 {    1178, .8289028320312E+04, -.1200502075195E+04, .1054926269531E+04},
 {    1179, .8268272460938E+04, -.1200502075195E+04, .1008559814453E+04},
 {    1180, .8266196289062E+04, -.1200502075195E+04, .1003923828125E+04},
 {    1181, .8291105468750E+04, -.1187802001953E+04, .1059562866211E+04},
 {    1182, .8289028320312E+04, -.1187802001953E+04, .1054926269531E+04},
 {    1183, .8268272460938E+04, -.1187802001953E+04, .1008559814453E+04},
 {    1184, .8266196289062E+04, -.1187802001953E+04, .1003923828125E+04},
 {    1185, .8395313476562E+04, -.1322421997070E+04, .1042133056641E+04},
 {    1186, .8372129882812E+04, -.1322421997070E+04, .1052511230469E+04},
 {    1187, .8348947265625E+04, -.1322421997070E+04, .1062889038086E+04},
 {    1188, .8395313476562E+04, -.1331947021484E+04, .1042133056641E+04},
 {    1189, .8372129882812E+04, -.1331947021484E+04, .1052511230469E+04},
 {    1190, .8348947265625E+04, -.1331947021484E+04, .1062889038086E+04},
 {    1191, .8383637695312E+04, -.1322421997070E+04, .1016052368164E+04},
 {    1192, .8360456054688E+04, -.1322421997070E+04, .1026430297852E+04},
 {    1193, .8337271484375E+04, -.1322421997070E+04, .1036808105469E+04},
 {    1194, .8383637695312E+04, -.1331947021484E+04, .1016052368164E+04},
 {    1195, .8360456054688E+04, -.1331947021484E+04, .1026430297852E+04},
 {    1196, .8337271484375E+04, -.1331947021484E+04, .1036808105469E+04},
 {    1197, .8371962890625E+04, -.1322421997070E+04, .9899710693359E+03},
 {    1198, .8348777343750E+04, -.1322421997070E+04, .1000349548340E+04},
 {    1199, .8325596679688E+04, -.1322421997070E+04, .1010727416992E+04},
 {    1200, .8371962890625E+04, -.1331947021484E+04, .9899710693359E+03},
 {    1201, .8348777343750E+04, -.1331947021484E+04, .1000349548340E+04},
 {    1202, .8325596679688E+04, -.1331947021484E+04, .1010727416992E+04},
 {    1203, .8360286132812E+04, -.1322421997070E+04, .9638902587891E+03},
 {    1204, .8337104492188E+04, -.1322421997070E+04, .9742684326172E+03},
 {    1205, .8313920898438E+04, -.1322421997070E+04, .9846463623047E+03},
 {    1206, .8360286132812E+04, -.1331947021484E+04, .9638902587891E+03},
 {    1207, .8337104492188E+04, -.1331947021484E+04, .9742684326172E+03},
 {    1208, .8313920898438E+04, -.1331947021484E+04, .9846463623047E+03},
 {    1209, .8348611328125E+04, -.1322421997070E+04, .9378092651367E+03},
 {    1210, .8325427734375E+04, -.1322421997070E+04, .9481873168945E+03},
 {    1211, .8302246093750E+04, -.1322421997070E+04, .9585657348633E+03},
 {    1212, .8348611328125E+04, -.1331947021484E+04, .9378092651367E+03},
 {    1213, .8325427734375E+04, -.1331947021484E+04, .9481873168945E+03},
 {    1214, .8302246093750E+04, -.1331947021484E+04, .9585657348633E+03},
 {    1215, .8316424804688E+04, -.1285592041016E+04, .9327375488281E+03},
 {    1216, .8303964843750E+04, -.1285592041016E+04, .9383157958984E+03},
 {    1217, .8291503906250E+04, -.1285592041016E+04, .9438941040039E+03},
 {    1218, .8317589843750E+04, -.1280829589844E+04, .9322163085938E+03},
 {    1219, .8306798828125E+04, -.1274003295898E+04, .9370472412109E+03},
 {    1220, .8296005859375E+04, -.1267177001953E+04, .9418781127930E+03},
 {    1221, .8320771484375E+04, -.1277343139648E+04, .9307918701172E+03},
 {    1222, .8314541992188E+04, -.1265519775391E+04, .9335808105469E+03},
 {    1223, .8308311523438E+04, -.1253696289062E+04, .9363698730469E+03},
 {    1224, .8325119140625E+04, -.1276067016602E+04, .9288457031250E+03},
 {    1225, .8325119140625E+04, -.1262414550781E+04, .9288457031250E+03},
 {    1226, .8325119140625E+04, -.1248762084961E+04, .9288457031250E+03},
 {    1227, .8329465820312E+04, -.1277343139648E+04, .9268997802734E+03},
 {    1228, .8335697265625E+04, -.1265519775391E+04, .9241107177734E+03},
 {    1229, .8341927734375E+04, -.1253696289062E+04, .9213214721680E+03},
 {    1230, .8332648437500E+04, -.1280829589844E+04, .9254752197266E+03},
 {    1231, .8343438476562E+04, -.1274003295898E+04, .9206443481445E+03},
 {    1232, .8354230468750E+04, -.1267177001953E+04, .9158136596680E+03},
 {    1233, .8333813476562E+04, -.1285592041016E+04, .9249539794922E+03},
 {    1234, .8346272460938E+04, -.1285592041016E+04, .9193757324219E+03},
 {    1235, .8358735351562E+04, -.1285592041016E+04, .9137974853516E+03},
 {    1236, .8332648437500E+04, -.1290354492188E+04, .9254752197266E+03},
 {    1237, .8343438476562E+04, -.1297180786133E+04, .9206443481445E+03},
 {    1238, .8354230468750E+04, -.1304007080078E+04, .9158136596680E+03},
 {    1239, .8329465820312E+04, -.1293840942383E+04, .9268997802734E+03},
 {    1240, .8335697265625E+04, -.1305664306641E+04, .9241107177734E+03},
 {    1241, .8341927734375E+04, -.1317487792969E+04, .9213214721680E+03},
 {    1242, .8325119140625E+04, -.1295117065430E+04, .9288457031250E+03},
 {    1243, .8325119140625E+04, -.1308769531250E+04, .9288457031250E+03},
 {    1244, .8325119140625E+04, -.1322421997070E+04, .9288457031250E+03},
 {    1245, .8320771484375E+04, -.1293840942383E+04, .9307918701172E+03},
 {    1246, .8314541992188E+04, -.1305664306641E+04, .9335808105469E+03},
 {    1247, .8308311523438E+04, -.1317487792969E+04, .9363698730469E+03},
 {    1248, .8317589843750E+04, -.1290354492188E+04, .9322163085938E+03},
 {    1249, .8306798828125E+04, -.1297180786133E+04, .9370472412109E+03},
 {    1250, .8296005859375E+04, -.1304007080078E+04, .9418781127930E+03},
 {    1251, .8314349609375E+04, -.1285592041016E+04, .9281008300781E+03},
 {    1252, .8301887695312E+04, -.1285592041016E+04, .9336794433594E+03},
 {    1253, .8289425781250E+04, -.1285592041016E+04, .9392575683594E+03},
 {    1254, .8315515625000E+04, -.1280829589844E+04, .9275798339844E+03},
 {    1255, .8304723632812E+04, -.1274003295898E+04, .9324104003906E+03},
 {    1256, .8293932617188E+04, -.1267177001953E+04, .9372412719727E+03},
 {    1257, .8318698242188E+04, -.1277343139648E+04, .9261550292969E+03},
 {    1258, .8312467773438E+04, -.1265519775391E+04, .9289443359375E+03},
 {    1259, .8306236328125E+04, -.1253696289062E+04, .9317331542969E+03},
 {    1260, .8323044921875E+04, -.1276067016602E+04, .9242091064453E+03},
 {    1261, .8323044921875E+04, -.1262414550781E+04, .9242091064453E+03},
 {    1262, .8323044921875E+04, -.1248762084961E+04, .9242091064453E+03},
 {    1263, .8327389648438E+04, -.1277343139648E+04, .9222633056641E+03},
 {    1264, .8333620117188E+04, -.1265519775391E+04, .9194742431641E+03},
 {    1265, .8339851562500E+04, -.1253696289062E+04, .9166852416992E+03},
 {    1266, .8330573242188E+04, -.1280829589844E+04, .9208387451172E+03},
 {    1267, .8341364257812E+04, -.1274003295898E+04, .9160078125000E+03},
 {    1268, .8352155273438E+04, -.1267177001953E+04, .9111768188477E+03},
 {    1269, .8331736328125E+04, -.1285592041016E+04, .9203173828125E+03},
 {    1270, .8344197265625E+04, -.1285592041016E+04, .9147390747070E+03},
 {    1271, .8356658203125E+04, -.1285592041016E+04, .9091607055664E+03},
 {    1272, .8330573242188E+04, -.1290354492188E+04, .9208387451172E+03},
 {    1273, .8341364257812E+04, -.1297180786133E+04, .9160078125000E+03},
 {    1274, .8352155273438E+04, -.1304007080078E+04, .9111768188477E+03},
 {    1275, .8327389648438E+04, -.1293840942383E+04, .9222633056641E+03},
 {    1276, .8333620117188E+04, -.1305664306641E+04, .9194742431641E+03},
 {    1277, .8339851562500E+04, -.1317487792969E+04, .9166852416992E+03},
 {    1278, .8323044921875E+04, -.1295117065430E+04, .9242091064453E+03},
 {    1279, .8323044921875E+04, -.1308769531250E+04, .9242091064453E+03},
 {    1280, .8323044921875E+04, -.1322421997070E+04, .9242091064453E+03},
 {    1281, .8318698242188E+04, -.1293840942383E+04, .9261550292969E+03},
 {    1282, .8312467773438E+04, -.1305664306641E+04, .9289443359375E+03},
 {    1283, .8306236328125E+04, -.1317487792969E+04, .9317331542969E+03},
 {    1284, .8315515625000E+04, -.1290354492188E+04, .9275798339844E+03},
 {    1285, .8304723632812E+04, -.1297180786133E+04, .9324104003906E+03},
 {    1286, .8293932617188E+04, -.1304007080078E+04, .9372412719727E+03},
 {    1287, .8302931640625E+04, -.1285592041016E+04, .9025998535156E+03},
 {    1288, .8290471679688E+04, -.1285592041016E+04, .9081780395508E+03},
 {    1289, .8278011718750E+04, -.1285592041016E+04, .9137561645508E+03},
 {    1290, .8304096679688E+04, -.1280829589844E+04, .9020783691406E+03},
 {    1291, .8293306640625E+04, -.1274003295898E+04, .9069093017578E+03},
 {    1292, .8282514648438E+04, -.1267177001953E+04, .9117402343750E+03},
 {    1293, .8307281250000E+04, -.1277343139648E+04, .9006535644531E+03},
 {    1294, .8301050781250E+04, -.1265519775391E+04, .9034426269531E+03},
 {    1295, .8294818359375E+04, -.1253696289062E+04, .9062321777344E+03},
 {    1296, .8311625976562E+04, -.1276067016602E+04, .8987078857422E+03},
 {    1297, .8311625976562E+04, -.1262414550781E+04, .8987078857422E+03},
 {    1298, .8311625976562E+04, -.1248762084961E+04, .8987078857422E+03},
 {    1299, .8315971679688E+04, -.1277343139648E+04, .8967618408203E+03},
 {    1300, .8322202148438E+04, -.1265519775391E+04, .8939728393555E+03},
 {    1301, .8328432617188E+04, -.1253696289062E+04, .8911837158203E+03},
 {    1302, .8319153320312E+04, -.1280829589844E+04, .8953376464844E+03},
 {    1303, .8329945312500E+04, -.1274003295898E+04, .8905066528320E+03},
 {    1304, .8340739257812E+04, -.1267177001953E+04, .8856756591797E+03},
 {    1305, .8320319335938E+04, -.1285592041016E+04, .8948159790039E+03},
 {    1306, .8332782226562E+04, -.1285592041016E+04, .8892377929688E+03},
 {    1307, .8345244140625E+04, -.1285592041016E+04, .8836594238281E+03},
 {    1308, .8319153320312E+04, -.1290354492188E+04, .8953376464844E+03},
 {    1309, .8329945312500E+04, -.1297180786133E+04, .8905066528320E+03},
 {    1310, .8340739257812E+04, -.1304007080078E+04, .8856757812500E+03},
 {    1311, .8315971679688E+04, -.1293840942383E+04, .8967618408203E+03},
 {    1312, .8322202148438E+04, -.1305664306641E+04, .8939728393555E+03},
 {    1313, .8328432617188E+04, -.1317487792969E+04, .8911837158203E+03},
 {    1314, .8311625976562E+04, -.1295117065430E+04, .8987078857422E+03},
 {    1315, .8311625976562E+04, -.1308769531250E+04, .8987078857422E+03},
 {    1316, .8311625976562E+04, -.1322421997070E+04, .8987078857422E+03},
 {    1317, .8307281250000E+04, -.1293840942383E+04, .9006535644531E+03},
 {    1318, .8301050781250E+04, -.1305664306641E+04, .9034426269531E+03},
 {    1319, .8294818359375E+04, -.1317487792969E+04, .9062321777344E+03},
 {    1320, .8304096679688E+04, -.1290354492188E+04, .9020783691406E+03},
 {    1321, .8293306640625E+04, -.1297180786133E+04, .9069093017578E+03},
 {    1322, .8282514648438E+04, -.1304007080078E+04, .9117402343750E+03},
 {    1323, .8300858398438E+04, -.1285592041016E+04, .8979628906250E+03},
 {    1324, .8288397460938E+04, -.1285592041016E+04, .9035413208008E+03},
 {    1325, .8275935546875E+04, -.1285592041016E+04, .9091195678711E+03},
 {    1326, .8302022460938E+04, -.1280829589844E+04, .8974417724609E+03},
 {    1327, .8291231445312E+04, -.1274003295898E+04, .9022724609375E+03},
 {    1328, .8280438476562E+04, -.1267177001953E+04, .9071033935547E+03},
 {    1329, .8305205078125E+04, -.1277343139648E+04, .8960172119141E+03},
 {    1330, .8298973632812E+04, -.1265519775391E+04, .8988064575195E+03},
 {    1331, .8292743164062E+04, -.1253696289062E+04, .9015954589844E+03},
 {    1332, .8309549804688E+04, -.1276067016602E+04, .8940714111328E+03},
 {    1333, .8309549804688E+04, -.1262414550781E+04, .8940714111328E+03},
 {    1334, .8309549804688E+04, -.1248762084961E+04, .8940714111328E+03},
 {    1335, .8313899414062E+04, -.1277343139648E+04, .8921250610352E+03},
 {    1336, .8320127929688E+04, -.1265519775391E+04, .8893361816406E+03},
 {    1337, .8326359375000E+04, -.1253696289062E+04, .8865470581055E+03},
 {    1338, .8317080078125E+04, -.1280829589844E+04, .8907007446289E+03},
 {    1339, .8327872070312E+04, -.1274003295898E+04, .8858696899414E+03},
 {    1340, .8338664062500E+04, -.1267177001953E+04, .8810389404297E+03},
 {    1341, .8318246093750E+04, -.1285592041016E+04, .8901795043945E+03},
 {    1342, .8330706054688E+04, -.1285592041016E+04, .8846011962891E+03},
 {    1343, .8343166015625E+04, -.1285592041016E+04, .8790230712891E+03},
 {    1344, .8317080078125E+04, -.1290354492188E+04, .8907007446289E+03},
 {    1345, .8327872070312E+04, -.1297180786133E+04, .8858696899414E+03},
 {    1346, .8338664062500E+04, -.1304007080078E+04, .8810389404297E+03},
 {    1347, .8313899414062E+04, -.1293840942383E+04, .8921250610352E+03},
 {    1348, .8320127929688E+04, -.1305664306641E+04, .8893361816406E+03},
 {    1349, .8326359375000E+04, -.1317487792969E+04, .8865470581055E+03},
 {    1350, .8309549804688E+04, -.1295117065430E+04, .8940714111328E+03},
 {    1351, .8309549804688E+04, -.1308769531250E+04, .8940714111328E+03},
 {    1352, .8309549804688E+04, -.1322421997070E+04, .8940714111328E+03},
 {    1353, .8305205078125E+04, -.1293840942383E+04, .8960172119141E+03},
 {    1354, .8298973632812E+04, -.1305664306641E+04, .8988064575195E+03},
 {    1355, .8292743164062E+04, -.1317487792969E+04, .9015954589844E+03},
 {    1356, .8302022460938E+04, -.1290354492188E+04, .8974417724609E+03},
 {    1357, .8291231445312E+04, -.1297180786133E+04, .9022724609375E+03},
 {    1358, .8280438476562E+04, -.1304007080078E+04, .9071033935547E+03},
 {    1359, .8289183593750E+04, -.1285592041016E+04, .9449321289062E+03},
 {    1360, .8294000000000E+04, -.1265907104492E+04, .9427765502930E+03},
 {    1361, .8307153320312E+04, -.1251496582031E+04, .9368885498047E+03},
 {    1362, .8325119140625E+04, -.1246222045898E+04, .9288457031250E+03},
 {    1363, .8343084960938E+04, -.1251496582031E+04, .9208028564453E+03},
 {    1364, .8356238281250E+04, -.1265907104492E+04, .9149147338867E+03},
 {    1365, .8361052734375E+04, -.1285592041016E+04, .9127595214844E+03},
 {    1366, .8356238281250E+04, -.1305277099609E+04, .9149147338867E+03},
 {    1367, .8343084960938E+04, -.1319687500000E+04, .9208028564453E+03},
 {    1368, .8325119140625E+04, -.1324962036133E+04, .9288457031250E+03},
 {    1369, .8307153320312E+04, -.1319687500000E+04, .9368885498047E+03},
 {    1370, .8294000000000E+04, -.1305277099609E+04, .9427765502930E+03},
 {    1371, .8287109375000E+04, -.1285592041016E+04, .9402951049805E+03},
 {    1372, .8291923828125E+04, -.1265907104492E+04, .9381401977539E+03},
 {    1373, .8305075195312E+04, -.1251496582031E+04, .9322524414062E+03},
 {    1374, .8323044921875E+04, -.1246222045898E+04, .9242091064453E+03},
 {    1375, .8341009765625E+04, -.1251496582031E+04, .9161661987305E+03},
 {    1376, .8354164062500E+04, -.1265907104492E+04, .9102781982422E+03},
 {    1377, .8358976562500E+04, -.1285592041016E+04, .9081231079102E+03},
 {    1378, .8354164062500E+04, -.1305277099609E+04, .9102781982422E+03},
 {    1379, .8341009765625E+04, -.1319687500000E+04, .9161661987305E+03},
 {    1380, .8323044921875E+04, -.1324962036133E+04, .9242091064453E+03},
 {    1381, .8305076171875E+04, -.1319687500000E+04, .9322521972656E+03},
 {    1382, .8291923828125E+04, -.1305277099609E+04, .9381401977539E+03},
 {    1383, .8275694335938E+04, -.1285592041016E+04, .9147940063477E+03},
 {    1384, .8280505859375E+04, -.1265907104492E+04, .9126388549805E+03},
 {    1385, .8293659179688E+04, -.1251496582031E+04, .9067509765625E+03},
 {    1386, .8311625976562E+04, -.1246222045898E+04, .8987078857422E+03},
 {    1387, .8329594726562E+04, -.1251496582031E+04, .8906648559570E+03},
 {    1388, .8342746093750E+04, -.1265907104492E+04, .8847769775391E+03},
 {    1389, .8347560546875E+04, -.1285592041016E+04, .8826217041016E+03},
 {    1390, .8342746093750E+04, -.1305277099609E+04, .8847769775391E+03},
 {    1391, .8329594726562E+04, -.1319687500000E+04, .8906648559570E+03},
 {    1392, .8311625976562E+04, -.1324962036133E+04, .8987078857422E+03},
 {    1393, .8293659179688E+04, -.1319687500000E+04, .9067510986328E+03},
 {    1394, .8280505859375E+04, -.1305277099609E+04, .9126388549805E+03},
 {    1395, .8273618164062E+04, -.1285592041016E+04, .9101577758789E+03},
 {    1396, .8278432617188E+04, -.1265907104492E+04, .9080020751953E+03},
 {    1397, .8291584960938E+04, -.1251496582031E+04, .9021141357422E+03},
 {    1398, .8309549804688E+04, -.1246222045898E+04, .8940714111328E+03},
 {    1399, .8327519531250E+04, -.1251496582031E+04, .8860280761719E+03},
 {    1400, .8340671875000E+04, -.1265907104492E+04, .8801399536133E+03},
 {    1401, .8345485351562E+04, -.1285592041016E+04, .8779854125977E+03},
 {    1402, .8340671875000E+04, -.1305277099609E+04, .8801399536133E+03},
 {    1403, .8327519531250E+04, -.1319687500000E+04, .8860280761719E+03},
 {    1404, .8309549804688E+04, -.1324962036133E+04, .8940714111328E+03},
 {    1405, .8291584960938E+04, -.1319687500000E+04, .9021141357422E+03},
 {    1406, .8278432617188E+04, -.1305277099609E+04, .9080020751953E+03},
 {    1407, .8424918945312E+04, -.1336392089844E+04, .1132043945312E+04},
 {    1408, .8400327148438E+04, -.1336392089844E+04, .1138474121094E+04},
 {    1409, .8375732421875E+04, -.1336392089844E+04, .1144905151367E+04},
 {    1410, .8449513671875E+04, -.1323692016602E+04, .1125612792969E+04},
 {    1411, .8424918945312E+04, -.1323692016602E+04, .1132043945312E+04},
 {    1412, .8400327148438E+04, -.1323692016602E+04, .1138474121094E+04},
 {    1413, .8375732421875E+04, -.1323692016602E+04, .1144905151367E+04},
 {    1414, .8451096679688E+04, -.1336392089844E+04, .1131673217773E+04},
 {    1415, .8426840820312E+04, -.1336392089844E+04, .1138015747070E+04},
 {    1416, .8402583984375E+04, -.1336392089844E+04, .1144358398438E+04},
 {    1417, .8378328125000E+04, -.1336392089844E+04, .1150701171875E+04},
 {    1418, .8451096679688E+04, -.1323692016602E+04, .1131673217773E+04},
 {    1419, .8426840820312E+04, -.1323692016602E+04, .1138015747070E+04},
 {    1420, .8402583984375E+04, -.1323692016602E+04, .1144358398438E+04},
 {    1421, .8378328125000E+04, -.1323692016602E+04, .1150701171875E+04},
 {    1422, .8449513671875E+04, -.1317342041016E+04, .1125612792969E+04},
 {    1423, .8424918945312E+04, -.1317342041016E+04, .1132043945312E+04},
 {    1424, .8400327148438E+04, -.1317342041016E+04, .1138474121094E+04},
 {    1425, .8375732421875E+04, -.1317342041016E+04, .1144905151367E+04},
 {    1426, .8451096679688E+04, -.1317342041016E+04, .1131673217773E+04},
 {    1427, .8426840820312E+04, -.1317342041016E+04, .1138015747070E+04},
 {    1428, .8402583984375E+04, -.1317342041016E+04, .1144358398438E+04},
 {    1429, .8378328125000E+04, -.1317342041016E+04, .1150701171875E+04},
 {    1430, .8449513671875E+04, -.1310992065430E+04, .1125612792969E+04},
 {    1431, .8424918945312E+04, -.1310992065430E+04, .1132043945312E+04},
 {    1432, .8400327148438E+04, -.1310992065430E+04, .1138474121094E+04},
 {    1433, .8375732421875E+04, -.1310992065430E+04, .1144905151367E+04},
 {    1434, .8451096679688E+04, -.1310992065430E+04, .1131673217773E+04},
 {    1435, .8426840820312E+04, -.1310992065430E+04, .1138015747070E+04},
 {    1436, .8402583984375E+04, -.1310992065430E+04, .1144358398438E+04},
 {    1437, .8378328125000E+04, -.1310992065430E+04, .1150701171875E+04},
 {    1438, .8453606445312E+04, -.1317342041016E+04, .1141267700195E+04},
 {    1439, .8429772460938E+04, -.1317342041016E+04, .1147500366211E+04},
 {    1440, .8405935546875E+04, -.1317342041016E+04, .1153733276367E+04},
 {    1441, .8382102539062E+04, -.1317342041016E+04, .1159964965820E+04},
 {    1442, .8453606445312E+04, -.1310992065430E+04, .1141267700195E+04},
 {    1443, .8429772460938E+04, -.1310992065430E+04, .1147500366211E+04},
 {    1444, .8405935546875E+04, -.1310992065430E+04, .1153733276367E+04},
 {    1445, .8382102539062E+04, -.1310992065430E+04, .1159964965820E+04},
 {    1446, .8443737304688E+04, -.1317342041016E+04, .1157407470703E+04},
 {    1447, .8425268554688E+04, -.1317342041016E+04, .1163724609375E+04},
 {    1448, .8406796875000E+04, -.1317342041016E+04, .1170040039062E+04},
 {    1449, .8388329101562E+04, -.1317342041016E+04, .1176356689453E+04},
 {    1450, .8443737304688E+04, -.1310992065430E+04, .1157407470703E+04},
 {    1451, .8425268554688E+04, -.1310992065430E+04, .1163724609375E+04},
 {    1452, .8406796875000E+04, -.1310992065430E+04, .1170040039062E+04},
 {    1453, .8388329101562E+04, -.1310992065430E+04, .1176356689453E+04},
 {    1454, .8433868164062E+04, -.1317342041016E+04, .1173547485352E+04},
 {    1455, .8420765625000E+04, -.1317342041016E+04, .1179947509766E+04},
 {    1456, .8407661132812E+04, -.1317342041016E+04, .1186348022461E+04},
 {    1457, .8394555664062E+04, -.1317342041016E+04, .1192748901367E+04},
 {    1458, .8433868164062E+04, -.1310992065430E+04, .1173547485352E+04},
 {    1459, .8420765625000E+04, -.1310992065430E+04, .1179947509766E+04},
 {    1460, .8407661132812E+04, -.1310992065430E+04, .1186348022461E+04},
 {    1461, .8394555664062E+04, -.1310992065430E+04, .1192748901367E+04},
 {    1462, .8423999023438E+04, -.1317342041016E+04, .1189687988281E+04},
 {    1463, .8415592773438E+04, -.1317342041016E+04, .1193002929688E+04},
 {    1464, .8407188476562E+04, -.1317342041016E+04, .1196318725586E+04},
 {    1465, .8398785156250E+04, -.1317342041016E+04, .1199633544922E+04},
 {    1466, .8423999023438E+04, -.1310992065430E+04, .1189687988281E+04},
 {    1467, .8415592773438E+04, -.1310992065430E+04, .1193002929688E+04},
 {    1468, .8407188476562E+04, -.1310992065430E+04, .1196318725586E+04},
 {    1469, .8398785156250E+04, -.1310992065430E+04, .1199633544922E+04},
 {    1470, .8414131835938E+04, -.1317342041016E+04, .1205827880859E+04},
 {    1471, .8410420898438E+04, -.1317342041016E+04, .1206058105469E+04},
 {    1472, .8406715820312E+04, -.1317342041016E+04, .1206288696289E+04},
 {    1473, .8403009765625E+04, -.1317342041016E+04, .1206519165039E+04},
 {    1474, .8414131835938E+04, -.1310992065430E+04, .1205827880859E+04},
 {    1475, .8410420898438E+04, -.1310992065430E+04, .1206058105469E+04},
 {    1476, .8406715820312E+04, -.1310992065430E+04, .1206288696289E+04},
 {    1477, .8403009765625E+04, -.1310992065430E+04, .1206519165039E+04},
 {    1478, .8449513671875E+04, -.1295117065430E+04, .1125612792969E+04},
 {    1479, .8424918945312E+04, -.1295117065430E+04, .1132043945312E+04},
 {    1480, .8400327148438E+04, -.1295117065430E+04, .1138474121094E+04},
 {    1481, .8375732421875E+04, -.1295117065430E+04, .1144905151367E+04},
 {    1482, .8451096679688E+04, -.1295117065430E+04, .1131673217773E+04},
 {    1483, .8426840820312E+04, -.1295117065430E+04, .1138015747070E+04},
 {    1484, .8402583984375E+04, -.1295117065430E+04, .1144358398438E+04},
 {    1485, .8378328125000E+04, -.1295117065430E+04, .1150701171875E+04},
 {    1486, .8449513671875E+04, -.1276067016602E+04, .1125612792969E+04},
 {    1487, .8424918945312E+04, -.1276067016602E+04, .1132043945312E+04},
 {    1488, .8400327148438E+04, -.1276067016602E+04, .1138474121094E+04},
 {    1489, .8375732421875E+04, -.1276067016602E+04, .1144905151367E+04},
 {    1490, .8451096679688E+04, -.1276067016602E+04, .1131673217773E+04},
 {    1491, .8426840820312E+04, -.1276067016602E+04, .1138015747070E+04},
 {    1492, .8402583984375E+04, -.1276067016602E+04, .1144358398438E+04},
 {    1493, .8378328125000E+04, -.1276067016602E+04, .1150701171875E+04},
 {    1494, .8453606445312E+04, -.1295117065430E+04, .1141267700195E+04},
 {    1495, .8429772460938E+04, -.1295117065430E+04, .1147500366211E+04},
 {    1496, .8405935546875E+04, -.1295117065430E+04, .1153733276367E+04},
 {    1497, .8382102539062E+04, -.1295117065430E+04, .1159964965820E+04},
 {    1498, .8453606445312E+04, -.1276067016602E+04, .1141267700195E+04},
 {    1499, .8429772460938E+04, -.1276067016602E+04, .1147500366211E+04},
 {    1500, .8405935546875E+04, -.1276067016602E+04, .1153733276367E+04},
 {    1501, .8382102539062E+04, -.1276067016602E+04, .1159964965820E+04},
 {    1502, .8443737304688E+04, -.1295117065430E+04, .1157407470703E+04},
 {    1503, .8425268554688E+04, -.1295117065430E+04, .1163724365234E+04},
 {    1504, .8406796875000E+04, -.1295117065430E+04, .1170040039062E+04},
 {    1505, .8388329101562E+04, -.1295117065430E+04, .1176356689453E+04},
 {    1506, .8443737304688E+04, -.1276067016602E+04, .1157407470703E+04},
 {    1507, .8425268554688E+04, -.1276067016602E+04, .1163724365234E+04},
 {    1508, .8406796875000E+04, -.1276067016602E+04, .1170040039062E+04},
 {    1509, .8388329101562E+04, -.1276067016602E+04, .1176356689453E+04},
 {    1510, .8433868164062E+04, -.1295117065430E+04, .1173547485352E+04},
 {    1511, .8420765625000E+04, -.1295117065430E+04, .1179947509766E+04},
 {    1512, .8407661132812E+04, -.1295117065430E+04, .1186348022461E+04},
 {    1513, .8394555664062E+04, -.1295117065430E+04, .1192748901367E+04},
 {    1514, .8433868164062E+04, -.1276067016602E+04, .1173547485352E+04},
 {    1515, .8420765625000E+04, -.1276067016602E+04, .1179947509766E+04},
 {    1516, .8407661132812E+04, -.1276067016602E+04, .1186348022461E+04},
 {    1517, .8394555664062E+04, -.1276067016602E+04, .1192748901367E+04},
 {    1518, .8423999023438E+04, -.1295117065430E+04, .1189687988281E+04},
 {    1519, .8415592773438E+04, -.1295117065430E+04, .1193002929688E+04},
 {    1520, .8407188476562E+04, -.1295117065430E+04, .1196318725586E+04},
 {    1521, .8398785156250E+04, -.1295117065430E+04, .1199633544922E+04},
 {    1522, .8423999023438E+04, -.1276067016602E+04, .1189687988281E+04},
 {    1523, .8415592773438E+04, -.1276067016602E+04, .1193002929688E+04},
 {    1524, .8407188476562E+04, -.1276067016602E+04, .1196318725586E+04},
 {    1525, .8398785156250E+04, -.1276067016602E+04, .1199633544922E+04},
 {    1526, .8414131835938E+04, -.1295117065430E+04, .1205827880859E+04},
 {    1527, .8410420898438E+04, -.1295117065430E+04, .1206058105469E+04},
 {    1528, .8406715820312E+04, -.1295117065430E+04, .1206288696289E+04},
 {    1529, .8403009765625E+04, -.1295117065430E+04, .1206519165039E+04},
 {    1530, .8414131835938E+04, -.1276067016602E+04, .1205827880859E+04},
 {    1531, .8410420898438E+04, -.1276067016602E+04, .1206058105469E+04},
 {    1532, .8406715820312E+04, -.1276067016602E+04, .1206288696289E+04},
 {    1533, .8403009765625E+04, -.1276067016602E+04, .1206519165039E+04},
 {    1534, .8449513671875E+04, -.1260192016602E+04, .1125612792969E+04},
 {    1535, .8424918945312E+04, -.1260192016602E+04, .1132043945312E+04},
 {    1536, .8400327148438E+04, -.1260192016602E+04, .1138474121094E+04},
 {    1537, .8375732421875E+04, -.1260192016602E+04, .1144905151367E+04},
 {    1538, .8451096679688E+04, -.1260192016602E+04, .1131673217773E+04},
 {    1539, .8426840820312E+04, -.1260192016602E+04, .1138015747070E+04},
 {    1540, .8402583984375E+04, -.1260192016602E+04, .1144358398438E+04},
 {    1541, .8378328125000E+04, -.1260192016602E+04, .1150701171875E+04},
 {    1542, .8449513671875E+04, -.1253842041016E+04, .1125612792969E+04},
 {    1543, .8424918945312E+04, -.1253842041016E+04, .1132043945312E+04},
 {    1544, .8400327148438E+04, -.1253842041016E+04, .1138474121094E+04},
 {    1545, .8375732421875E+04, -.1253842041016E+04, .1144905151367E+04},
 {    1546, .8451096679688E+04, -.1253842041016E+04, .1131673217773E+04},
 {    1547, .8426840820312E+04, -.1253842041016E+04, .1138015747070E+04},
 {    1548, .8402583984375E+04, -.1253842041016E+04, .1144358398438E+04},
 {    1549, .8378328125000E+04, -.1253842041016E+04, .1150701171875E+04},
 {    1550, .8453606445312E+04, -.1260192016602E+04, .1141267700195E+04},
 {    1551, .8429772460938E+04, -.1260192016602E+04, .1147500366211E+04},
 {    1552, .8405935546875E+04, -.1260192016602E+04, .1153733276367E+04},
 {    1553, .8382102539062E+04, -.1260192016602E+04, .1159964965820E+04},
 {    1554, .8453606445312E+04, -.1253842041016E+04, .1141267700195E+04},
 {    1555, .8429772460938E+04, -.1253842041016E+04, .1147500366211E+04},
 {    1556, .8405935546875E+04, -.1253842041016E+04, .1153733276367E+04},
 {    1557, .8382102539062E+04, -.1253842041016E+04, .1159964965820E+04},
 {    1558, .8443737304688E+04, -.1260192016602E+04, .1157407470703E+04},
 {    1559, .8425268554688E+04, -.1260192016602E+04, .1163724365234E+04},
 {    1560, .8406796875000E+04, -.1260192016602E+04, .1170040039062E+04},
 {    1561, .8388329101562E+04, -.1260192016602E+04, .1176356689453E+04},
 {    1562, .8443737304688E+04, -.1253842041016E+04, .1157407470703E+04},
 {    1563, .8425268554688E+04, -.1253842041016E+04, .1163724365234E+04},
 {    1564, .8406796875000E+04, -.1253842041016E+04, .1170040039062E+04},
 {    1565, .8388329101562E+04, -.1253842041016E+04, .1176356689453E+04},
 {    1566, .8433868164062E+04, -.1260192016602E+04, .1173547485352E+04},
 {    1567, .8420765625000E+04, -.1260192016602E+04, .1179947509766E+04},
 {    1568, .8407661132812E+04, -.1260192016602E+04, .1186348022461E+04},
 {    1569, .8394555664062E+04, -.1260192016602E+04, .1192748901367E+04},
 {    1570, .8433868164062E+04, -.1253842041016E+04, .1173547485352E+04},
 {    1571, .8420765625000E+04, -.1253842041016E+04, .1179947509766E+04},
 {    1572, .8407661132812E+04, -.1253842041016E+04, .1186348022461E+04},
 {    1573, .8394555664062E+04, -.1253842041016E+04, .1192748901367E+04},
 {    1574, .8423999023438E+04, -.1260192016602E+04, .1189687988281E+04},
 {    1575, .8415592773438E+04, -.1260192016602E+04, .1193002929688E+04},
 {    1576, .8407188476562E+04, -.1260192016602E+04, .1196318725586E+04},
 {    1577, .8398785156250E+04, -.1260192016602E+04, .1199633544922E+04},
 {    1578, .8423999023438E+04, -.1253842041016E+04, .1189687988281E+04},
 {    1579, .8415592773438E+04, -.1253842041016E+04, .1193002929688E+04},
 {    1580, .8407188476562E+04, -.1253842041016E+04, .1196318725586E+04},
 {    1581, .8398785156250E+04, -.1253842041016E+04, .1199633544922E+04},
 {    1582, .8414131835938E+04, -.1260192016602E+04, .1205827880859E+04},
 {    1583, .8410420898438E+04, -.1260192016602E+04, .1206058105469E+04},
 {    1584, .8406715820312E+04, -.1260192016602E+04, .1206288696289E+04},
 {    1585, .8403009765625E+04, -.1260192016602E+04, .1206519165039E+04},
 {    1586, .8414131835938E+04, -.1253842041016E+04, .1205827880859E+04},
 {    1587, .8410420898438E+04, -.1253842041016E+04, .1206058105469E+04},
 {    1588, .8406715820312E+04, -.1253842041016E+04, .1206288696289E+04},
 {    1589, .8403009765625E+04, -.1253842041016E+04, .1206519165039E+04},
 {    1590, .8449513671875E+04, -.1247492065430E+04, .1125612792969E+04},
 {    1591, .8424918945312E+04, -.1247492065430E+04, .1132043945312E+04},
 {    1592, .8400327148438E+04, -.1247492065430E+04, .1138474121094E+04},
 {    1593, .8375732421875E+04, -.1247492065430E+04, .1144905151367E+04},
 {    1594, .8451096679688E+04, -.1247492065430E+04, .1131673217773E+04},
 {    1595, .8426840820312E+04, -.1247492065430E+04, .1138015747070E+04},
 {    1596, .8402583984375E+04, -.1247492065430E+04, .1144358398438E+04},
 {    1597, .8378328125000E+04, -.1247492065430E+04, .1150701171875E+04},
 {    1598, .8424918945312E+04, -.1234791992188E+04, .1132043945312E+04},
 {    1599, .8400327148438E+04, -.1234791992188E+04, .1138474121094E+04},
 {    1600, .8375732421875E+04, -.1234791992188E+04, .1144905151367E+04},
 {    1601, .8451096679688E+04, -.1234791992188E+04, .1131673217773E+04},
 {    1602, .8426840820312E+04, -.1234791992188E+04, .1138015747070E+04},
 {    1603, .8402583984375E+04, -.1234791992188E+04, .1144358398438E+04},
 {    1604, .8378328125000E+04, -.1234791992188E+04, .1150701171875E+04},
 {    1605, .8369673828125E+04, -.1336392089844E+04, .1146489746094E+04},
 {    1606, .8369673828125E+04, -.1323692016602E+04, .1146489746094E+04},
 {    1607, .8372268554688E+04, -.1336392089844E+04, .1152285888672E+04},
 {    1608, .8372268554688E+04, -.1323692016602E+04, .1152285888672E+04},
 {    1609, .8369673828125E+04, -.1317342041016E+04, .1146489746094E+04},
 {    1610, .8372268554688E+04, -.1317342041016E+04, .1152285888672E+04},
 {    1611, .8403009765625E+04, -.1323692016602E+04, .1206519165039E+04},
 {    1612, .8397076171875E+04, -.1323692016602E+04, .1206888183594E+04},
 {    1613, .8397076171875E+04, -.1317342041016E+04, .1206888183594E+04},
 {    1614, .8403379882812E+04, -.1323692016602E+04, .1212453979492E+04},
 {    1615, .8397443359375E+04, -.1323692016602E+04, .1212823120117E+04},
 {    1616, .8403379882812E+04, -.1317342041016E+04, .1212453979492E+04},
 {    1617, .8397443359375E+04, -.1317342041016E+04, .1212823120117E+04},
 {    1618, .8369673828125E+04, -.1310992065430E+04, .1146489746094E+04},
 {    1619, .8372268554688E+04, -.1310992065430E+04, .1152285888672E+04},
 {    1620, .8376040039062E+04, -.1317342041016E+04, .1161549804688E+04},
 {    1621, .8376040039062E+04, -.1310992065430E+04, .1161549804688E+04},
 {    1622, .8382333007812E+04, -.1317342041016E+04, .1177333740234E+04},
 {    1623, .8382333007812E+04, -.1310992065430E+04, .1177333740234E+04},
 {    1624, .8388622070312E+04, -.1317342041016E+04, .1193117187500E+04},
 {    1625, .8388622070312E+04, -.1310992065430E+04, .1193117187500E+04},
 {    1626, .8392849609375E+04, -.1317342041016E+04, .1200002929688E+04},
 {    1627, .8392849609375E+04, -.1310992065430E+04, .1200002929688E+04},
 {    1628, .8397076171875E+04, -.1310992065430E+04, .1206888183594E+04},
 {    1629, .8403379882812E+04, -.1310992065430E+04, .1212453979492E+04},
 {    1630, .8397443359375E+04, -.1310992065430E+04, .1212823120117E+04},
 {    1631, .8369673828125E+04, -.1295117065430E+04, .1146489746094E+04},
 {    1632, .8372268554688E+04, -.1295117065430E+04, .1152285888672E+04},
 {    1633, .8397076171875E+04, -.1295117065430E+04, .1206888183594E+04},
 {    1634, .8403379882812E+04, -.1295117065430E+04, .1212453979492E+04},
 {    1635, .8397443359375E+04, -.1295117065430E+04, .1212823120117E+04},
 {    1636, .8369673828125E+04, -.1276067016602E+04, .1146489746094E+04},
 {    1637, .8372268554688E+04, -.1276067016602E+04, .1152285888672E+04},
 {    1638, .8376040039062E+04, -.1295117065430E+04, .1161549804688E+04},
 {    1639, .8376040039062E+04, -.1276067016602E+04, .1161549804688E+04},
 {    1640, .8382333007812E+04, -.1295117065430E+04, .1177333740234E+04},
 {    1641, .8382333007812E+04, -.1276067016602E+04, .1177333740234E+04},
 {    1642, .8388622070312E+04, -.1295117065430E+04, .1193117187500E+04},
 {    1643, .8388622070312E+04, -.1276067016602E+04, .1193117187500E+04},
 {    1644, .8392849609375E+04, -.1295117065430E+04, .1200002929688E+04},
 {    1645, .8392849609375E+04, -.1276067016602E+04, .1200002929688E+04},
 {    1646, .8397076171875E+04, -.1276067016602E+04, .1206888183594E+04},
 {    1647, .8403379882812E+04, -.1276067016602E+04, .1212453979492E+04},
 {    1648, .8397443359375E+04, -.1276067016602E+04, .1212823120117E+04},
 {    1649, .8369673828125E+04, -.1260192016602E+04, .1146489746094E+04},
 {    1650, .8372268554688E+04, -.1260192016602E+04, .1152285888672E+04},
 {    1651, .8397076171875E+04, -.1260192016602E+04, .1206888183594E+04},
 {    1652, .8403379882812E+04, -.1260192016602E+04, .1212453979492E+04},
 {    1653, .8397443359375E+04, -.1260192016602E+04, .1212823120117E+04},
 {    1654, .8369673828125E+04, -.1253842041016E+04, .1146489746094E+04},
 {    1655, .8372268554688E+04, -.1253842041016E+04, .1152285888672E+04},
 {    1656, .8376040039062E+04, -.1260192016602E+04, .1161549804688E+04},
 {    1657, .8376040039062E+04, -.1253842041016E+04, .1161549804688E+04},
 {    1658, .8382333007812E+04, -.1260192016602E+04, .1177333740234E+04},
 {    1659, .8382333007812E+04, -.1253842041016E+04, .1177333740234E+04},
 {    1660, .8388622070312E+04, -.1260192016602E+04, .1193117187500E+04},
 {    1661, .8388622070312E+04, -.1253842041016E+04, .1193117187500E+04},
 {    1662, .8392849609375E+04, -.1260192016602E+04, .1200002929688E+04},
 {    1663, .8392849609375E+04, -.1253842041016E+04, .1200002929688E+04},
 {    1664, .8397076171875E+04, -.1253842041016E+04, .1206888183594E+04},
 {    1665, .8403379882812E+04, -.1253842041016E+04, .1212453979492E+04},
 {    1666, .8397443359375E+04, -.1253842041016E+04, .1212823120117E+04},
 {    1667, .8369673828125E+04, -.1247492065430E+04, .1146489746094E+04},
 {    1668, .8372268554688E+04, -.1247492065430E+04, .1152285888672E+04},
 {    1669, .8403009765625E+04, -.1247492065430E+04, .1206519165039E+04},
 {    1670, .8397076171875E+04, -.1247492065430E+04, .1206888183594E+04},
 {    1671, .8403379882812E+04, -.1247492065430E+04, .1212453979492E+04},
 {    1672, .8397443359375E+04, -.1247492065430E+04, .1212823120117E+04},
 {    1673, .8369673828125E+04, -.1234791992188E+04, .1146489746094E+04},
 {    1674, .8372268554688E+04, -.1234791992188E+04, .1152285888672E+04},
 {    1675, .8381201171875E+04, -.1323692016602E+04, .1207875854492E+04},
 {    1676, .8381201171875E+04, -.1317342041016E+04, .1207875854492E+04},
 {    1677, .8381570312500E+04, -.1323692016602E+04, .1213810791016E+04},
 {    1678, .8381570312500E+04, -.1317342041016E+04, .1213810791016E+04},
 {    1679, .8380314453125E+04, -.1317342041016E+04, .1193634155273E+04},
 {    1680, .8380314453125E+04, -.1310992065430E+04, .1193634155273E+04},
 {    1681, .8380758789062E+04, -.1317342041016E+04, .1200755126953E+04},
 {    1682, .8380758789062E+04, -.1310992065430E+04, .1200755126953E+04},
 {    1683, .8381201171875E+04, -.1310992065430E+04, .1207875854492E+04},
 {    1684, .8381570312500E+04, -.1310992065430E+04, .1213810791016E+04},
 {    1685, .8381201171875E+04, -.1295117065430E+04, .1207875854492E+04},
 {    1686, .8381570312500E+04, -.1295117065430E+04, .1213810791016E+04},
 {    1687, .8380314453125E+04, -.1295117065430E+04, .1193634155273E+04},
 {    1688, .8380314453125E+04, -.1276067016602E+04, .1193634155273E+04},
 {    1689, .8380758789062E+04, -.1295117065430E+04, .1200755126953E+04},
 {    1690, .8380758789062E+04, -.1276067016602E+04, .1200755126953E+04},
 {    1691, .8381201171875E+04, -.1276067016602E+04, .1207875854492E+04},
 {    1692, .8381570312500E+04, -.1276067016602E+04, .1213810791016E+04},
 {    1693, .8381201171875E+04, -.1260192016602E+04, .1207875854492E+04},
 {    1694, .8381570312500E+04, -.1260192016602E+04, .1213810791016E+04},
 {    1695, .8380314453125E+04, -.1260192016602E+04, .1193634155273E+04},
 {    1696, .8380314453125E+04, -.1253842041016E+04, .1193634155273E+04},
 {    1697, .8380758789062E+04, -.1260192016602E+04, .1200755126953E+04},
 {    1698, .8380758789062E+04, -.1253842041016E+04, .1200755126953E+04},
 {    1699, .8381201171875E+04, -.1253842041016E+04, .1207875854492E+04},
 {    1700, .8381570312500E+04, -.1253842041016E+04, .1213810791016E+04},
 {    1701, .8381201171875E+04, -.1247492065430E+04, .1207875854492E+04},
 {    1702, .8381570312500E+04, -.1247492065430E+04, .1213810791016E+04},
 {    1703, .8365672851562E+04, -.1323692016602E+04, .1208842529297E+04},
 {    1704, .8350144531250E+04, -.1323692016602E+04, .1209808349609E+04},
 {    1705, .8334618164062E+04, -.1323692016602E+04, .1210774780273E+04},
 {    1706, .8365672851562E+04, -.1317342041016E+04, .1208842529297E+04},
 {    1707, .8350144531250E+04, -.1317342041016E+04, .1209808349609E+04},
 {    1708, .8334618164062E+04, -.1317342041016E+04, .1210774780273E+04},
 {    1709, .8366041015625E+04, -.1323692016602E+04, .1214777465820E+04},
 {    1710, .8350513671875E+04, -.1323692016602E+04, .1215742187500E+04},
 {    1711, .8334985351562E+04, -.1323692016602E+04, .1216708740234E+04},
 {    1712, .8366041015625E+04, -.1317342041016E+04, .1214777465820E+04},
 {    1713, .8350513671875E+04, -.1317342041016E+04, .1215742187500E+04},
 {    1714, .8334985351562E+04, -.1317342041016E+04, .1216708740234E+04},
 {    1715, .8365672851562E+04, -.1310992065430E+04, .1208842529297E+04},
 {    1716, .8350144531250E+04, -.1310992065430E+04, .1209808349609E+04},
 {    1717, .8334618164062E+04, -.1310992065430E+04, .1210774780273E+04},
 {    1718, .8366041015625E+04, -.1310992065430E+04, .1214777465820E+04},
 {    1719, .8350513671875E+04, -.1310992065430E+04, .1215742187500E+04},
 {    1720, .8334985351562E+04, -.1310992065430E+04, .1216708740234E+04},
 {    1721, .8365672851562E+04, -.1295117065430E+04, .1208842529297E+04},
 {    1722, .8350144531250E+04, -.1295117065430E+04, .1209808349609E+04},
 {    1723, .8334618164062E+04, -.1295117065430E+04, .1210774780273E+04},
 {    1724, .8366041015625E+04, -.1295117065430E+04, .1214777465820E+04},
 {    1725, .8350513671875E+04, -.1295117065430E+04, .1215742187500E+04},
 {    1726, .8334985351562E+04, -.1295117065430E+04, .1216708740234E+04},
 {    1727, .8364786132812E+04, -.1295117065430E+04, .1194600219727E+04},
 {    1728, .8349257812500E+04, -.1295117065430E+04, .1195566162109E+04},
 {    1729, .8333730468750E+04, -.1295117065430E+04, .1196532348633E+04},
 {    1730, .8364786132812E+04, -.1276067016602E+04, .1194600219727E+04},
 {    1731, .8349257812500E+04, -.1276067016602E+04, .1195566162109E+04},
 {    1732, .8333730468750E+04, -.1276067016602E+04, .1196532348633E+04},
 {    1733, .8365230468750E+04, -.1295117065430E+04, .1201721191406E+04},
 {    1734, .8349703125000E+04, -.1295117065430E+04, .1202687500000E+04},
 {    1735, .8334175781250E+04, -.1295117065430E+04, .1203653442383E+04},
 {    1736, .8365230468750E+04, -.1276067016602E+04, .1201721191406E+04},
 {    1737, .8349703125000E+04, -.1276067016602E+04, .1202687500000E+04},
 {    1738, .8334175781250E+04, -.1276067016602E+04, .1203653442383E+04},
 {    1739, .8365672851562E+04, -.1276067016602E+04, .1208842529297E+04},
 {    1740, .8350144531250E+04, -.1276067016602E+04, .1209808349609E+04},
 {    1741, .8334618164062E+04, -.1276067016602E+04, .1210774780273E+04},
 {    1742, .8366041015625E+04, -.1276067016602E+04, .1214777465820E+04},
 {    1743, .8350513671875E+04, -.1276067016602E+04, .1215742187500E+04},
 {    1744, .8334985351562E+04, -.1276067016602E+04, .1216708740234E+04},
 {    1745, .8365672851562E+04, -.1260192016602E+04, .1208842529297E+04},
 {    1746, .8350144531250E+04, -.1260192016602E+04, .1209808349609E+04},
 {    1747, .8334618164062E+04, -.1260192016602E+04, .1210774780273E+04},
 {    1748, .8366041015625E+04, -.1260192016602E+04, .1214777465820E+04},
 {    1749, .8350513671875E+04, -.1260192016602E+04, .1215742187500E+04},
 {    1750, .8334985351562E+04, -.1260192016602E+04, .1216708740234E+04},
 {    1751, .8365672851562E+04, -.1253842041016E+04, .1208842529297E+04},
 {    1752, .8350144531250E+04, -.1253842041016E+04, .1209808349609E+04},
 {    1753, .8334618164062E+04, -.1253842041016E+04, .1210774780273E+04},
 {    1754, .8366041015625E+04, -.1253842041016E+04, .1214777465820E+04},
 {    1755, .8350513671875E+04, -.1253842041016E+04, .1215742187500E+04},
 {    1756, .8334985351562E+04, -.1253842041016E+04, .1216708740234E+04},
 {    1757, .8365672851562E+04, -.1247492065430E+04, .1208842529297E+04},
 {    1758, .8350144531250E+04, -.1247492065430E+04, .1209808349609E+04},
 {    1759, .8334618164062E+04, -.1247492065430E+04, .1210774780273E+04},
 {    1760, .8366041015625E+04, -.1247492065430E+04, .1214777465820E+04},
 {    1761, .8350513671875E+04, -.1247492065430E+04, .1215742187500E+04},
 {    1762, .8334985351562E+04, -.1247492065430E+04, .1216708740234E+04},
 {    1763, .8285606445312E+04, -.1186532104492E+04, .1028630371094E+04},
 {    1764, .8272855468750E+04, -.1186532104492E+04, .1034337890625E+04},
 {    1765, .8285188476562E+04, -.1186532104492E+04, .1026033569336E+04},
 {    1766, .8271292968750E+04, -.1186532104492E+04, .1024601318359E+04},
 {    1767, .8286127929688E+04, -.1186532104492E+04, .1023576171875E+04},
 {    1768, .8274807617188E+04, -.1186532104492E+04, .1015388061523E+04},
 {    1769, .8288166015625E+04, -.1186532104492E+04, .1021918090820E+04},
 {    1770, .8282458984375E+04, -.1186532104492E+04, .1009166748047E+04},
 {    1771, .8285606445312E+04, -.1190342041016E+04, .1028630371094E+04},
 {    1772, .8272855468750E+04, -.1190342041016E+04, .1034337890625E+04},
 {    1773, .8285188476562E+04, -.1190342041016E+04, .1026033569336E+04},
 {    1774, .8271292968750E+04, -.1190342041016E+04, .1024601318359E+04},
 {    1775, .8286127929688E+04, -.1190342041016E+04, .1023576171875E+04},
 {    1776, .8274807617188E+04, -.1190342041016E+04, .1015388061523E+04},
 {    1777, .8288166015625E+04, -.1190342041016E+04, .1021918090820E+04},
 {    1778, .8282458984375E+04, -.1190342041016E+04, .1009166748047E+04},
 {    1779, .8285606445312E+04, -.1200502075195E+04, .1028630371094E+04},
 {    1780, .8272855468750E+04, -.1200502075195E+04, .1034337890625E+04},
 {    1781, .8285188476562E+04, -.1200502075195E+04, .1026033569336E+04},
 {    1782, .8271292968750E+04, -.1200502075195E+04, .1024601318359E+04},
 {    1783, .8286127929688E+04, -.1200502075195E+04, .1023576171875E+04},
 {    1784, .8274807617188E+04, -.1200502075195E+04, .1015388061523E+04},
 {    1785, .8288166015625E+04, -.1200502075195E+04, .1021918090820E+04},
 {    1786, .8282458984375E+04, -.1200502075195E+04, .1009166748047E+04},
 {    1787, .8285606445312E+04, -.1223679565430E+04, .1028630371094E+04},
 {    1788, .8272855468750E+04, -.1223679565430E+04, .1034337890625E+04},
 {    1789, .8285188476562E+04, -.1223679565430E+04, .1026033569336E+04},
 {    1790, .8271292968750E+04, -.1223679565430E+04, .1024601318359E+04},
 {    1791, .8286127929688E+04, -.1223679565430E+04, .1023576171875E+04},
 {    1792, .8274807617188E+04, -.1223679565430E+04, .1015388061523E+04},
 {    1793, .8288166015625E+04, -.1223679565430E+04, .1021918090820E+04},
 {    1794, .8282458984375E+04, -.1223679565430E+04, .1009166748047E+04},
 {    1795, .8285606445312E+04, -.1246857055664E+04, .1028630371094E+04},
 {    1796, .8272855468750E+04, -.1246857055664E+04, .1034337890625E+04},
 {    1797, .8285188476562E+04, -.1246857055664E+04, .1026033569336E+04},
 {    1798, .8271292968750E+04, -.1246857055664E+04, .1024601318359E+04},
 {    1799, .8286127929688E+04, -.1246857055664E+04, .1023576171875E+04},
 {    1800, .8274807617188E+04, -.1246857055664E+04, .1015388061523E+04},
 {    1801, .8288166015625E+04, -.1246857055664E+04, .1021918090820E+04},
 {    1802, .8282458984375E+04, -.1246857055664E+04, .1009166748047E+04},
 {    1803, .8285606445312E+04, -.1270034545898E+04, .1028630371094E+04},
 {    1804, .8272855468750E+04, -.1270034545898E+04, .1034337890625E+04},
 {    1805, .8285188476562E+04, -.1270034545898E+04, .1026033569336E+04},
 {    1806, .8271292968750E+04, -.1270034545898E+04, .1024601318359E+04},
 {    1807, .8286127929688E+04, -.1270034545898E+04, .1023576171875E+04},
 {    1808, .8274807617188E+04, -.1270034545898E+04, .1015388061523E+04},
 {    1809, .8288166015625E+04, -.1270034545898E+04, .1021918090820E+04},
 {    1810, .8282458984375E+04, -.1270034545898E+04, .1009166748047E+04},
 {    1811, .8285606445312E+04, -.1293212036133E+04, .1028630371094E+04},
 {    1812, .8272855468750E+04, -.1293212036133E+04, .1034337890625E+04},
 {    1813, .8285188476562E+04, -.1293212036133E+04, .1026033569336E+04},
 {    1814, .8271292968750E+04, -.1293212036133E+04, .1024601318359E+04},
 {    1815, .8286127929688E+04, -.1293212036133E+04, .1023576171875E+04},
 {    1816, .8274807617188E+04, -.1293212036133E+04, .1015388061523E+04},
 {    1817, .8288166015625E+04, -.1293212036133E+04, .1021918090820E+04},
 {    1818, .8282458984375E+04, -.1293212036133E+04, .1009166748047E+04},
 {    1819, .8290762695312E+04, -.1186532104492E+04, .1021500854492E+04},
 {    1820, .8292194335938E+04, -.1186532104492E+04, .1007604736328E+04},
 {    1821, .8293221679688E+04, -.1186532104492E+04, .1022438232422E+04},
 {    1822, .8301407226562E+04, -.1186532104492E+04, .1011119628906E+04},
 {    1823, .8294878906250E+04, -.1186532104492E+04, .1024478881836E+04},
 {    1824, .8307628906250E+04, -.1186532104492E+04, .1018770141602E+04},
 {    1825, .8295294921875E+04, -.1186532104492E+04, .1027074951172E+04},
 {    1826, .8309192382812E+04, -.1186532104492E+04, .1028507568359E+04},
 {    1827, .8294360351562E+04, -.1186532104492E+04, .1029531738281E+04},
 {    1828, .8305677734375E+04, -.1186532104492E+04, .1037719970703E+04},
 {    1829, .8292318359375E+04, -.1186532104492E+04, .1031190917969E+04},
 {    1830, .8298025390625E+04, -.1186532104492E+04, .1043941162109E+04},
 {    1831, .8290762695312E+04, -.1190342041016E+04, .1021500854492E+04},
 {    1832, .8292194335938E+04, -.1190342041016E+04, .1007604736328E+04},
 {    1833, .8293221679688E+04, -.1190342041016E+04, .1022438232422E+04},
 {    1834, .8301407226562E+04, -.1190342041016E+04, .1011119628906E+04},
 {    1835, .8294878906250E+04, -.1190342041016E+04, .1024478881836E+04},
 {    1836, .8307628906250E+04, -.1190342041016E+04, .1018770141602E+04},
 {    1837, .8295294921875E+04, -.1190342041016E+04, .1027074951172E+04},
 {    1838, .8309192382812E+04, -.1190342041016E+04, .1028507568359E+04},
 {    1839, .8294360351562E+04, -.1190342041016E+04, .1029531738281E+04},
 {    1840, .8305677734375E+04, -.1190342041016E+04, .1037719970703E+04},
 {    1841, .8292318359375E+04, -.1190342041016E+04, .1031190917969E+04},
 {    1842, .8298025390625E+04, -.1190342041016E+04, .1043941162109E+04},
 {    1843, .8290762695312E+04, -.1200502075195E+04, .1021500854492E+04},
 {    1844, .8292194335938E+04, -.1200502075195E+04, .1007604736328E+04},
 {    1845, .8293221679688E+04, -.1200502075195E+04, .1022438232422E+04},
 {    1846, .8301407226562E+04, -.1200502075195E+04, .1011119628906E+04},
 {    1847, .8294878906250E+04, -.1200502075195E+04, .1024478881836E+04},
 {    1848, .8307628906250E+04, -.1200502075195E+04, .1018770141602E+04},
 {    1849, .8295294921875E+04, -.1200502075195E+04, .1027074951172E+04},
 {    1850, .8309192382812E+04, -.1200502075195E+04, .1028507568359E+04},
 {    1851, .8294360351562E+04, -.1200502075195E+04, .1029531738281E+04},
 {    1852, .8305677734375E+04, -.1200502075195E+04, .1037719970703E+04},
 {    1853, .8292318359375E+04, -.1200502075195E+04, .1031190917969E+04},
 {    1854, .8298025390625E+04, -.1200502075195E+04, .1043941162109E+04},
 {    1855, .8290762695312E+04, -.1223679565430E+04, .1021500854492E+04},
 {    1856, .8292194335938E+04, -.1223679565430E+04, .1007604736328E+04},
 {    1857, .8293221679688E+04, -.1223679565430E+04, .1022438232422E+04},
 {    1858, .8301407226562E+04, -.1223679565430E+04, .1011119628906E+04},
 {    1859, .8294878906250E+04, -.1223679565430E+04, .1024478881836E+04},
 {    1860, .8307628906250E+04, -.1223679565430E+04, .1018770141602E+04},
 {    1861, .8295294921875E+04, -.1223679565430E+04, .1027074951172E+04},
 {    1862, .8309192382812E+04, -.1223679565430E+04, .1028507568359E+04},
 {    1863, .8294360351562E+04, -.1223679565430E+04, .1029531738281E+04},
 {    1864, .8305677734375E+04, -.1223679565430E+04, .1037719970703E+04},
 {    1865, .8292318359375E+04, -.1223679565430E+04, .1031190917969E+04},
 {    1866, .8298025390625E+04, -.1223679565430E+04, .1043941162109E+04},
 {    1867, .8290762695312E+04, -.1246857055664E+04, .1021500854492E+04},
 {    1868, .8292194335938E+04, -.1246857055664E+04, .1007604736328E+04},
 {    1869, .8293221679688E+04, -.1246857055664E+04, .1022438232422E+04},
 {    1870, .8301407226562E+04, -.1246857055664E+04, .1011119628906E+04},
 {    1871, .8294878906250E+04, -.1246857055664E+04, .1024478881836E+04},
 {    1872, .8307628906250E+04, -.1246857055664E+04, .1018770141602E+04},
 {    1873, .8295294921875E+04, -.1246857055664E+04, .1027074951172E+04},
 {    1874, .8309192382812E+04, -.1246857055664E+04, .1028507568359E+04},
 {    1875, .8294360351562E+04, -.1246857055664E+04, .1029531738281E+04},
 {    1876, .8305677734375E+04, -.1246857055664E+04, .1037719970703E+04},
 {    1877, .8292318359375E+04, -.1246857055664E+04, .1031190917969E+04},
 {    1878, .8298025390625E+04, -.1246857055664E+04, .1043941162109E+04},
 {    1879, .8290762695312E+04, -.1270034545898E+04, .1021500854492E+04},
 {    1880, .8292194335938E+04, -.1270034545898E+04, .1007604736328E+04},
 {    1881, .8293221679688E+04, -.1270034545898E+04, .1022438232422E+04},
 {    1882, .8301407226562E+04, -.1270034545898E+04, .1011119628906E+04},
 {    1883, .8294878906250E+04, -.1270034545898E+04, .1024478881836E+04},
 {    1884, .8307628906250E+04, -.1270034545898E+04, .1018770141602E+04},
 {    1885, .8295294921875E+04, -.1270034545898E+04, .1027074951172E+04},
 {    1886, .8309192382812E+04, -.1270034545898E+04, .1028507568359E+04},
 {    1887, .8294360351562E+04, -.1270034545898E+04, .1029531738281E+04},
 {    1888, .8305677734375E+04, -.1270034545898E+04, .1037719970703E+04},
 {    1889, .8292318359375E+04, -.1270034545898E+04, .1031190917969E+04},
 {    1890, .8298025390625E+04, -.1270034545898E+04, .1043941162109E+04},
 {    1891, .8290762695312E+04, -.1293212036133E+04, .1021500854492E+04},
 {    1892, .8292194335938E+04, -.1293212036133E+04, .1007604736328E+04},
 {    1893, .8293221679688E+04, -.1293212036133E+04, .1022438232422E+04},
 {    1894, .8301407226562E+04, -.1293212036133E+04, .1011119628906E+04},
 {    1895, .8294878906250E+04, -.1293212036133E+04, .1024478881836E+04},
 {    1896, .8307628906250E+04, -.1293212036133E+04, .1018770141602E+04},
 {    1897, .8295294921875E+04, -.1293212036133E+04, .1027074951172E+04},
 {    1898, .8309192382812E+04, -.1293212036133E+04, .1028507568359E+04},
 {    1899, .8294360351562E+04, -.1293212036133E+04, .1029531738281E+04},
 {    1900, .8305677734375E+04, -.1293212036133E+04, .1037719970703E+04},
 {    1901, .8292318359375E+04, -.1293212036133E+04, .1031190917969E+04},
 {    1902, .8298025390625E+04, -.1293212036133E+04, .1043941162109E+04},
 {    1903, .8289721679688E+04, -.1186532104492E+04, .1031607421875E+04},
 {    1904, .8288289062500E+04, -.1186532104492E+04, .1045503784180E+04},
 {    1905, .8287264648438E+04, -.1186532104492E+04, .1030670043945E+04},
 {    1906, .8279077148438E+04, -.1186532104492E+04, .1041988647461E+04},
 {    1907, .8289721679688E+04, -.1190342041016E+04, .1031607421875E+04},
 {    1908, .8288289062500E+04, -.1190342041016E+04, .1045503784180E+04},
 {    1909, .8287264648438E+04, -.1190342041016E+04, .1030670043945E+04},
 {    1910, .8279077148438E+04, -.1190342041016E+04, .1041988647461E+04},
 {    1911, .8289721679688E+04, -.1200502075195E+04, .1031607421875E+04},
 {    1912, .8288289062500E+04, -.1200502075195E+04, .1045503784180E+04},
 {    1913, .8287264648438E+04, -.1200502075195E+04, .1030670043945E+04},
 {    1914, .8279077148438E+04, -.1200502075195E+04, .1041988647461E+04},
 {    1915, .8289721679688E+04, -.1223679565430E+04, .1031607421875E+04},
 {    1916, .8288289062500E+04, -.1223679565430E+04, .1045503784180E+04},
 {    1917, .8287264648438E+04, -.1223679565430E+04, .1030670043945E+04},
 {    1918, .8279077148438E+04, -.1223679565430E+04, .1041988647461E+04},
 {    1919, .8289721679688E+04, -.1246857055664E+04, .1031607421875E+04},
 {    1920, .8288289062500E+04, -.1246857055664E+04, .1045503784180E+04},
 {    1921, .8287264648438E+04, -.1246857055664E+04, .1030670043945E+04},
 {    1922, .8279077148438E+04, -.1246857055664E+04, .1041988647461E+04},
 {    1923, .8289721679688E+04, -.1270034545898E+04, .1031607421875E+04},
 {    1924, .8288289062500E+04, -.1270034545898E+04, .1045503784180E+04},
 {    1925, .8287264648438E+04, -.1270034545898E+04, .1030670043945E+04},
 {    1926, .8279077148438E+04, -.1270034545898E+04, .1041988647461E+04},
 {    1927, .8289721679688E+04, -.1293212036133E+04, .1031607421875E+04},
 {    1928, .8288289062500E+04, -.1293212036133E+04, .1045503784180E+04},
 {    1929, .8287264648438E+04, -.1293212036133E+04, .1030670043945E+04},
 {    1930, .8279077148438E+04, -.1293212036133E+04, .1041988647461E+04},
 {    1931, .8267059570312E+04, -.1190342041016E+04, .1036932617188E+04},
 {    1932, .8264975585938E+04, -.1190342041016E+04, .1023950683594E+04},
 {    1933, .8269664062500E+04, -.1190342041016E+04, .1011665832520E+04},
 {    1934, .8279864257812E+04, -.1190342041016E+04, .1003370788574E+04},
 {    1935, .8267059570312E+04, -.1200502075195E+04, .1036932617188E+04},
 {    1936, .8264975585938E+04, -.1200502075195E+04, .1023950683594E+04},
 {    1937, .8269664062500E+04, -.1200502075195E+04, .1011665832520E+04},
 {    1938, .8279864257812E+04, -.1200502075195E+04, .1003370788574E+04},
 {    1939, .8292847656250E+04, -.1190342041016E+04, .1001287841797E+04},
 {    1940, .8305130859375E+04, -.1190342041016E+04, .1005974731445E+04},
 {    1941, .8313423828125E+04, -.1190342041016E+04, .1016175903320E+04},
 {    1942, .8315507812500E+04, -.1190342041016E+04, .1029157592773E+04},
 {    1943, .8310823242188E+04, -.1190342041016E+04, .1041442260742E+04},
 {    1944, .8300619140625E+04, -.1190342041016E+04, .1049737304688E+04},
 {    1945, .8292847656250E+04, -.1200502075195E+04, .1001287841797E+04},
 {    1946, .8305130859375E+04, -.1200502075195E+04, .1005974731445E+04},
 {    1947, .8313423828125E+04, -.1200502075195E+04, .1016175903320E+04},
 {    1948, .8315507812500E+04, -.1200502075195E+04, .1029157592773E+04},
 {    1949, .8310823242188E+04, -.1200502075195E+04, .1041442260742E+04},
 {    1950, .8300619140625E+04, -.1200502075195E+04, .1049737304688E+04},
 {    1951, .8287638671875E+04, -.1190342041016E+04, .1051820434570E+04},
 {    1952, .8275354492188E+04, -.1190342041016E+04, .1047132934570E+04},
 {    1953, .8287638671875E+04, -.1200502075195E+04, .1051820434570E+04},
 {    1954, .8275354492188E+04, -.1200502075195E+04, .1047132934570E+04},
 {    1955, .8290245117188E+04, -.1323692016602E+04, .1026553833008E+04},
 {    1956, .8290245117188E+04, -.1248127075195E+04, .1026553833008E+04},
 {    1957, .8287923828125E+04, -.1295752075195E+04, .1027591796875E+04},
 {    1958, .8285606445312E+04, -.1295752075195E+04, .1028630371094E+04},
 {    1959, .8287716796875E+04, -.1295752075195E+04, .1026293579102E+04},
 {    1960, .8285188476562E+04, -.1295752075195E+04, .1026033569336E+04},
 {    1961, .8288183593750E+04, -.1295752075195E+04, .1025065673828E+04},
 {    1962, .8286127929688E+04, -.1295752075195E+04, .1023576171875E+04},
 {    1963, .8289204101562E+04, -.1295752075195E+04, .1024235595703E+04},
 {    1964, .8288166015625E+04, -.1295752075195E+04, .1021918090820E+04},
 {    1965, .8287923828125E+04, -.1310357055664E+04, .1027591796875E+04},
 {    1966, .8285606445312E+04, -.1310357055664E+04, .1028630371094E+04},
 {    1967, .8287716796875E+04, -.1310357055664E+04, .1026293579102E+04},
 {    1968, .8285188476562E+04, -.1310357055664E+04, .1026033569336E+04},
 {    1969, .8288184570312E+04, -.1310357055664E+04, .1025065185547E+04},
 {    1970, .8286127929688E+04, -.1310357055664E+04, .1023576171875E+04},
 {    1971, .8289204101562E+04, -.1310357055664E+04, .1024235595703E+04},
 {    1972, .8288166015625E+04, -.1310357055664E+04, .1021918090820E+04},
 {    1973, .8287923828125E+04, -.1324962036133E+04, .1027591796875E+04},
 {    1974, .8285606445312E+04, -.1324962036133E+04, .1028630371094E+04},
 {    1975, .8287716796875E+04, -.1324962036133E+04, .1026293579102E+04},
 {    1976, .8285188476562E+04, -.1324962036133E+04, .1026033569336E+04},
 {    1977, .8288184570312E+04, -.1324962036133E+04, .1025065185547E+04},
 {    1978, .8286127929688E+04, -.1324962036133E+04, .1023576171875E+04},
 {    1979, .8289204101562E+04, -.1324962036133E+04, .1024235595703E+04},
 {    1980, .8288166015625E+04, -.1324962036133E+04, .1021918090820E+04},
 {    1981, .8287923828125E+04, -.1339567016602E+04, .1027591796875E+04},
 {    1982, .8285606445312E+04, -.1339567016602E+04, .1028630371094E+04},
 {    1983, .8287716796875E+04, -.1339567016602E+04, .1026293579102E+04},
 {    1984, .8285188476562E+04, -.1339567016602E+04, .1026033569336E+04},
 {    1985, .8288184570312E+04, -.1339567016602E+04, .1025065185547E+04},
 {    1986, .8286127929688E+04, -.1339567016602E+04, .1023576171875E+04},
 {    1987, .8289204101562E+04, -.1339567016602E+04, .1024235595703E+04},
 {    1988, .8288166015625E+04, -.1339567016602E+04, .1021918090820E+04},
 {    1989, .8287923828125E+04, -.1354171997070E+04, .1027591796875E+04},
 {    1990, .8285606445312E+04, -.1354171997070E+04, .1028630371094E+04},
 {    1991, .8287716796875E+04, -.1354171997070E+04, .1026293579102E+04},
 {    1992, .8285188476562E+04, -.1354171997070E+04, .1026033569336E+04},
 {    1993, .8288184570312E+04, -.1354171997070E+04, .1025065185547E+04},
 {    1994, .8286127929688E+04, -.1354171997070E+04, .1023576171875E+04},
 {    1995, .8289204101562E+04, -.1354171997070E+04, .1024235595703E+04},
 {    1996, .8288166015625E+04, -.1354171997070E+04, .1021918090820E+04},
 {    1997, .8287923828125E+04, -.1364332031250E+04, .1027591796875E+04},
 {    1998, .8285606445312E+04, -.1364332031250E+04, .1028630371094E+04},
 {    1999, .8287716796875E+04, -.1364332031250E+04, .1026293579102E+04},
 {    2000, .8285188476562E+04, -.1364332031250E+04, .1026033569336E+04},
 {    2001, .8288184570312E+04, -.1364332031250E+04, .1025065185547E+04},
 {    2002, .8286127929688E+04, -.1364332031250E+04, .1023576171875E+04},
 {    2003, .8289204101562E+04, -.1364332031250E+04, .1024235595703E+04},
 {    2004, .8288166015625E+04, -.1364332031250E+04, .1021918090820E+04},
 {    2005, .8287923828125E+04, -.1374492065430E+04, .1027591796875E+04},
 {    2006, .8285606445312E+04, -.1374492065430E+04, .1028630371094E+04},
 {    2007, .8287716796875E+04, -.1374492065430E+04, .1026293579102E+04},
 {    2008, .8285188476562E+04, -.1374492065430E+04, .1026033569336E+04},
 {    2009, .8288184570312E+04, -.1374492065430E+04, .1025065185547E+04},
 {    2010, .8286127929688E+04, -.1374492065430E+04, .1023576171875E+04},
 {    2011, .8289204101562E+04, -.1374492065430E+04, .1024235595703E+04},
 {    2012, .8288166015625E+04, -.1374492065430E+04, .1021918090820E+04},
 {    2013, .8289083984375E+04, -.1384652099609E+04, .1027073120117E+04},
 {    2014, .8285606445312E+04, -.1384652099609E+04, .1028630371094E+04},
 {    2015, .8287716796875E+04, -.1384652099609E+04, .1026293579102E+04},
 {    2016, .8285188476562E+04, -.1384652099609E+04, .1026033569336E+04},
 {    2017, .8288184570312E+04, -.1384652099609E+04, .1025065185547E+04},
 {    2018, .8286127929688E+04, -.1384652099609E+04, .1023576171875E+04},
 {    2019, .8289204101562E+04, -.1384652099609E+04, .1024235595703E+04},
 {    2020, .8288166015625E+04, -.1384652099609E+04, .1021918090820E+04},
 {    2021, .8290504882812E+04, -.1295752075195E+04, .1024027343750E+04},
 {    2022, .8290762695312E+04, -.1295752075195E+04, .1021500854492E+04},
 {    2023, .8291729492188E+04, -.1295752075195E+04, .1024496582031E+04},
 {    2024, .8293221679688E+04, -.1295752075195E+04, .1022438232422E+04},
 {    2025, .8292560546875E+04, -.1295752075195E+04, .1025516601562E+04},
 {    2026, .8294878906250E+04, -.1295752075195E+04, .1024478881836E+04},
 {    2027, .8292769531250E+04, -.1295752075195E+04, .1026814575195E+04},
 {    2028, .8295294921875E+04, -.1295752075195E+04, .1027074951172E+04},
 {    2029, .8292300781250E+04, -.1295752075195E+04, .1028042968750E+04},
 {    2030, .8294360351562E+04, -.1295752075195E+04, .1029531738281E+04},
 {    2031, .8291280273438E+04, -.1295752075195E+04, .1028872314453E+04},
 {    2032, .8292318359375E+04, -.1295752075195E+04, .1031190917969E+04},
 {    2033, .8290504882812E+04, -.1310357055664E+04, .1024027343750E+04},
 {    2034, .8290762695312E+04, -.1310357055664E+04, .1021500854492E+04},
 {    2035, .8291729492188E+04, -.1310357055664E+04, .1024496582031E+04},
 {    2036, .8293221679688E+04, -.1310357055664E+04, .1022438232422E+04},
 {    2037, .8292560546875E+04, -.1310357055664E+04, .1025516601562E+04},
 {    2038, .8294878906250E+04, -.1310357055664E+04, .1024478881836E+04},
 {    2039, .8292769531250E+04, -.1310357055664E+04, .1026814575195E+04},
 {    2040, .8295294921875E+04, -.1310357055664E+04, .1027074951172E+04},
 {    2041, .8292300781250E+04, -.1310357055664E+04, .1028042968750E+04},
 {    2042, .8294360351562E+04, -.1310357055664E+04, .1029531738281E+04},
 {    2043, .8291280273438E+04, -.1310357055664E+04, .1028872314453E+04},
 {    2044, .8292318359375E+04, -.1310357055664E+04, .1031190917969E+04},
 {    2045, .8290504882812E+04, -.1324962036133E+04, .1024027343750E+04},
 {    2046, .8290762695312E+04, -.1324962036133E+04, .1021500854492E+04},
 {    2047, .8291729492188E+04, -.1324962036133E+04, .1024496582031E+04},
 {    2048, .8293221679688E+04, -.1324962036133E+04, .1022438232422E+04},
 {    2049, .8292560546875E+04, -.1324962036133E+04, .1025516601562E+04},
 {    2050, .8294878906250E+04, -.1324962036133E+04, .1024478881836E+04},
 {    2051, .8292769531250E+04, -.1324962036133E+04, .1026814575195E+04},
 {    2052, .8295294921875E+04, -.1324962036133E+04, .1027074951172E+04},
 {    2053, .8292300781250E+04, -.1324962036133E+04, .1028042968750E+04},
 {    2054, .8294360351562E+04, -.1324962036133E+04, .1029531738281E+04},
 {    2055, .8291280273438E+04, -.1324962036133E+04, .1028872314453E+04},
 {    2056, .8292318359375E+04, -.1324962036133E+04, .1031190917969E+04},
 {    2057, .8290504882812E+04, -.1339567016602E+04, .1024027343750E+04},
 {    2058, .8290762695312E+04, -.1339567016602E+04, .1021500854492E+04},
 {    2059, .8291729492188E+04, -.1339567016602E+04, .1024496582031E+04},
 {    2060, .8293221679688E+04, -.1339567016602E+04, .1022438232422E+04},
 {    2061, .8292560546875E+04, -.1339567016602E+04, .1025516601562E+04},
 {    2062, .8294878906250E+04, -.1339567016602E+04, .1024478881836E+04},
 {    2063, .8292769531250E+04, -.1339567016602E+04, .1026814575195E+04},
 {    2064, .8295294921875E+04, -.1339567016602E+04, .1027074951172E+04},
 {    2065, .8292300781250E+04, -.1339567016602E+04, .1028042968750E+04},
 {    2066, .8294360351562E+04, -.1339567016602E+04, .1029531738281E+04},
 {    2067, .8291280273438E+04, -.1339567016602E+04, .1028872314453E+04},
 {    2068, .8292318359375E+04, -.1339567016602E+04, .1031190917969E+04},
 {    2069, .8290504882812E+04, -.1354171997070E+04, .1024027343750E+04},
 {    2070, .8290762695312E+04, -.1354171997070E+04, .1021501098633E+04},
 {    2071, .8291729492188E+04, -.1354171997070E+04, .1024496582031E+04},
 {    2072, .8293221679688E+04, -.1354171997070E+04, .1022438232422E+04},
 {    2073, .8292560546875E+04, -.1354171997070E+04, .1025516601562E+04},
 {    2074, .8294878906250E+04, -.1354171997070E+04, .1024478881836E+04},
 {    2075, .8292769531250E+04, -.1354171997070E+04, .1026814575195E+04},
 {    2076, .8295294921875E+04, -.1354171997070E+04, .1027074951172E+04},
 {    2077, .8292300781250E+04, -.1354171997070E+04, .1028042968750E+04},
 {    2078, .8294360351562E+04, -.1354171997070E+04, .1029531738281E+04},
 {    2079, .8291280273438E+04, -.1354171997070E+04, .1028872314453E+04},
 {    2080, .8292318359375E+04, -.1354171997070E+04, .1031190917969E+04},
 {    2081, .8290504882812E+04, -.1364332031250E+04, .1024027343750E+04},
 {    2082, .8290762695312E+04, -.1364332031250E+04, .1021501098633E+04},
 {    2083, .8291729492188E+04, -.1364332031250E+04, .1024496582031E+04},
 {    2084, .8293221679688E+04, -.1364332031250E+04, .1022438232422E+04},
 {    2085, .8292560546875E+04, -.1364332031250E+04, .1025516601562E+04},
 {    2086, .8294878906250E+04, -.1364332031250E+04, .1024478881836E+04},
 {    2087, .8292769531250E+04, -.1364332031250E+04, .1026814575195E+04},
 {    2088, .8295294921875E+04, -.1364332031250E+04, .1027074951172E+04},
 {    2089, .8292300781250E+04, -.1364332031250E+04, .1028042968750E+04},
 {    2090, .8294360351562E+04, -.1364332031250E+04, .1029531738281E+04},
 {    2091, .8291280273438E+04, -.1364332031250E+04, .1028872314453E+04},
 {    2092, .8292318359375E+04, -.1364332031250E+04, .1031190917969E+04},
 {    2093, .8290504882812E+04, -.1374492065430E+04, .1024027343750E+04},
 {    2094, .8290762695312E+04, -.1374492065430E+04, .1021501098633E+04},
 {    2095, .8291729492188E+04, -.1374492065430E+04, .1024496582031E+04},
 {    2096, .8293221679688E+04, -.1374492065430E+04, .1022438232422E+04},
 {    2097, .8292560546875E+04, -.1374492065430E+04, .1025516601562E+04},
 {    2098, .8294878906250E+04, -.1374492065430E+04, .1024478881836E+04},
 {    2099, .8292769531250E+04, -.1374492065430E+04, .1026814575195E+04},
 {    2100, .8295294921875E+04, -.1374492065430E+04, .1027074951172E+04},
 {    2101, .8292300781250E+04, -.1374492065430E+04, .1028042968750E+04},
 {    2102, .8294360351562E+04, -.1374492065430E+04, .1029531738281E+04},
 {    2103, .8291280273438E+04, -.1374492065430E+04, .1028872314453E+04},
 {    2104, .8292318359375E+04, -.1374492065430E+04, .1031190917969E+04},
 {    2105, .8290504882812E+04, -.1384652099609E+04, .1024027343750E+04},
 {    2106, .8290762695312E+04, -.1384652099609E+04, .1021501098633E+04},
 {    2107, .8291729492188E+04, -.1384652099609E+04, .1024496582031E+04},
 {    2108, .8293221679688E+04, -.1384652099609E+04, .1022438232422E+04},
 {    2109, .8292560546875E+04, -.1384652099609E+04, .1025516601562E+04},
 {    2110, .8294878906250E+04, -.1384652099609E+04, .1024478881836E+04},
 {    2111, .8292769531250E+04, -.1384652099609E+04, .1026814575195E+04},
 {    2112, .8295294921875E+04, -.1384652099609E+04, .1027074951172E+04},
 {    2113, .8292300781250E+04, -.1384652099609E+04, .1028042968750E+04},
 {    2114, .8294360351562E+04, -.1384652099609E+04, .1029531738281E+04},
 {    2115, .8291280273438E+04, -.1384652099609E+04, .1028872314453E+04},
 {    2116, .8292318359375E+04, -.1384652099609E+04, .1031190917969E+04},
 {    2117, .8289981445312E+04, -.1295752075195E+04, .1029081054688E+04},
 {    2118, .8289721679688E+04, -.1295752075195E+04, .1031607421875E+04},
 {    2119, .8288752929688E+04, -.1295752075195E+04, .1028612060547E+04},
 {    2120, .8287264648438E+04, -.1295752075195E+04, .1030670043945E+04},
 {    2121, .8289981445312E+04, -.1310357055664E+04, .1029081054688E+04},
 {    2122, .8289721679688E+04, -.1310357055664E+04, .1031607421875E+04},
 {    2123, .8288752929688E+04, -.1310357055664E+04, .1028612060547E+04},
 {    2124, .8287264648438E+04, -.1310357055664E+04, .1030670043945E+04},
 {    2125, .8289981445312E+04, -.1324962036133E+04, .1029081054688E+04},
 {    2126, .8289721679688E+04, -.1324962036133E+04, .1031607421875E+04},
 {    2127, .8288752929688E+04, -.1324962036133E+04, .1028612060547E+04},
 {    2128, .8287264648438E+04, -.1324962036133E+04, .1030670043945E+04},
 {    2129, .8289981445312E+04, -.1339567016602E+04, .1029081054688E+04},
 {    2130, .8289721679688E+04, -.1339567016602E+04, .1031607421875E+04},
 {    2131, .8288752929688E+04, -.1339567016602E+04, .1028612060547E+04},
 {    2132, .8287264648438E+04, -.1339567016602E+04, .1030670043945E+04},
 {    2133, .8289981445312E+04, -.1354171997070E+04, .1029081054688E+04},
 {    2134, .8289721679688E+04, -.1354171997070E+04, .1031607421875E+04},
 {    2135, .8288752929688E+04, -.1354171997070E+04, .1028612060547E+04},
 {    2136, .8287264648438E+04, -.1354171997070E+04, .1030670043945E+04},
 {    2137, .8289981445312E+04, -.1364332031250E+04, .1029081054688E+04},
 {    2138, .8289721679688E+04, -.1364332031250E+04, .1031607421875E+04},
 {    2139, .8288752929688E+04, -.1364332031250E+04, .1028612060547E+04},
 {    2140, .8287264648438E+04, -.1364332031250E+04, .1030670043945E+04},
 {    2141, .8289981445312E+04, -.1374492065430E+04, .1029081054688E+04},
 {    2142, .8289721679688E+04, -.1374492065430E+04, .1031607421875E+04},
 {    2143, .8288752929688E+04, -.1374492065430E+04, .1028612060547E+04},
 {    2144, .8287264648438E+04, -.1374492065430E+04, .1030670043945E+04},
 {    2145, .8289981445312E+04, -.1384652099609E+04, .1029081054688E+04},
 {    2146, .8289721679688E+04, -.1384652099609E+04, .1031607421875E+04},
 {    2147, .8288752929688E+04, -.1384652099609E+04, .1028612060547E+04},
 {    2148, .8287264648438E+04, -.1384652099609E+04, .1030670043945E+04},
 {    2149, .8279115234375E+04, -.1354171997070E+04, .1031535522461E+04},
 {    2150, .8278114257812E+04, -.1354171997070E+04, .1025304443359E+04},
 {    2151, .8280364257812E+04, -.1354171997070E+04, .1019407958984E+04},
 {    2152, .8285260742188E+04, -.1354171997070E+04, .1015426330566E+04},
 {    2153, .8279115234375E+04, -.1364332031250E+04, .1031535522461E+04},
 {    2154, .8278114257812E+04, -.1364332031250E+04, .1025304443359E+04},
 {    2155, .8280364257812E+04, -.1364332031250E+04, .1019407958984E+04},
 {    2156, .8285260742188E+04, -.1364332031250E+04, .1015426330566E+04},
 {    2157, .8279115234375E+04, -.1374492065430E+04, .1031535522461E+04},
 {    2158, .8278114257812E+04, -.1374492065430E+04, .1025304443359E+04},
 {    2159, .8280364257812E+04, -.1374492065430E+04, .1019407958984E+04},
 {    2160, .8285260742188E+04, -.1374492065430E+04, .1015426330566E+04},
 {    2161, .8279115234375E+04, -.1384652099609E+04, .1031535522461E+04},
 {    2162, .8278114257812E+04, -.1384652099609E+04, .1025304443359E+04},
 {    2163, .8280364257812E+04, -.1384652099609E+04, .1019407958984E+04},
 {    2164, .8285260742188E+04, -.1384652099609E+04, .1015426330566E+04},
 {    2165, .8291492187500E+04, -.1354171997070E+04, .1014426391602E+04},
 {    2166, .8297388671875E+04, -.1354171997070E+04, .1016676269531E+04},
 {    2167, .8301370117188E+04, -.1354171997070E+04, .1021572326660E+04},
 {    2168, .8302370117188E+04, -.1354171997070E+04, .1027804077148E+04},
 {    2169, .8300122070312E+04, -.1354171997070E+04, .1033700195312E+04},
 {    2170, .8295223632812E+04, -.1354171997070E+04, .1037682006836E+04},
 {    2171, .8291492187500E+04, -.1364332031250E+04, .1014426391602E+04},
 {    2172, .8297388671875E+04, -.1364332031250E+04, .1016676269531E+04},
 {    2173, .8301370117188E+04, -.1364332031250E+04, .1021572326660E+04},
 {    2174, .8302370117188E+04, -.1364332031250E+04, .1027804077148E+04},
 {    2175, .8300122070312E+04, -.1364332031250E+04, .1033700195312E+04},
 {    2176, .8295223632812E+04, -.1364332031250E+04, .1037682006836E+04},
 {    2177, .8291492187500E+04, -.1374492065430E+04, .1014426391602E+04},
 {    2178, .8297388671875E+04, -.1374492065430E+04, .1016676269531E+04},
 {    2179, .8301370117188E+04, -.1374492065430E+04, .1021572326660E+04},
 {    2180, .8302370117188E+04, -.1374492065430E+04, .1027804077148E+04},
 {    2181, .8300122070312E+04, -.1374492065430E+04, .1033700195312E+04},
 {    2182, .8295223632812E+04, -.1374492065430E+04, .1037682006836E+04},
 {    2183, .8291492187500E+04, -.1384652099609E+04, .1014426391602E+04},
 {    2184, .8297388671875E+04, -.1384652099609E+04, .1016676269531E+04},
 {    2185, .8301370117188E+04, -.1384652099609E+04, .1021572326660E+04},
 {    2186, .8302370117188E+04, -.1384652099609E+04, .1027804077148E+04},
 {    2187, .8300122070312E+04, -.1384652099609E+04, .1033700195312E+04},
 {    2188, .8295223632812E+04, -.1384652099609E+04, .1037682006836E+04},
 {    2189, .8288993164062E+04, -.1354171997070E+04, .1038681884766E+04},
 {    2190, .8283096679688E+04, -.1354171997070E+04, .1036432006836E+04},
 {    2191, .8288993164062E+04, -.1364332031250E+04, .1038681884766E+04},
 {    2192, .8283096679688E+04, -.1364332031250E+04, .1036432006836E+04},
 {    2193, .8288993164062E+04, -.1374492065430E+04, .1038681884766E+04},
 {    2194, .8283096679688E+04, -.1374492065430E+04, .1036432006836E+04},
 {    2195, .8288993164062E+04, -.1384652099609E+04, .1038681884766E+04},
 {    2196, .8283096679688E+04, -.1384652099609E+04, .1036432006836E+04},
 {    2197, .8290245117188E+04, -.1323692016602E+04, .1026553833008E+04},
 {    2198, .8290245117188E+04, -.1248127075195E+04, .1026553833008E+04},
 {    2876, .8395313476562E+04, -.1343059570312E+04, .1042133056641E+04},
 {    2877, .8372129882812E+04, -.1348615844727E+04, .1052511230469E+04},
 {    2878, .8348947265625E+04, -.1354171997070E+04, .1062889038086E+04},
 {    2879, .8395313476562E+04, -.1354171997070E+04, .1042133056641E+04},
 {    2880, .8372129882812E+04, -.1365284545898E+04, .1052511230469E+04},
 {    2881, .8348947265625E+04, -.1376397094727E+04, .1062889038086E+04},
 {    2882, .8383637695312E+04, -.1343059570312E+04, .1016052368164E+04},
 {    2883, .8360456054688E+04, -.1348615844727E+04, .1026430297852E+04},
 {    2884, .8337271484375E+04, -.1354171997070E+04, .1036808105469E+04},
 {    2885, .8383637695312E+04, -.1354171997070E+04, .1016052368164E+04},
 {    2886, .8360456054688E+04, -.1365284545898E+04, .1026430297852E+04},
 {    2887, .8337271484375E+04, -.1376397094727E+04, .1036808105469E+04},
 {    2888, .8371962890625E+04, -.1343059570312E+04, .9899710693359E+03},
 {    2889, .8348777343750E+04, -.1348615844727E+04, .1000349548340E+04},
 {    2890, .8325596679688E+04, -.1354171997070E+04, .1010727416992E+04},
 {    2891, .8371962890625E+04, -.1354171997070E+04, .9899710693359E+03},
 {    2892, .8348777343750E+04, -.1365284545898E+04, .1000349548340E+04},
 {    2893, .8325596679688E+04, -.1376397094727E+04, .1010727416992E+04},
 {    2894, .8360286132812E+04, -.1343059570312E+04, .9638902587891E+03},
 {    2895, .8337104492188E+04, -.1348615844727E+04, .9742684326172E+03},
 {    2896, .8313920898438E+04, -.1354171997070E+04, .9846463623047E+03},
 {    2897, .8360286132812E+04, -.1354171997070E+04, .9638902587891E+03},
 {    2898, .8337104492188E+04, -.1365284545898E+04, .9742684326172E+03},
 {    2899, .8313920898438E+04, -.1376397094727E+04, .9846463623047E+03},
 {    2900, .8348611328125E+04, -.1343059570312E+04, .9378092651367E+03},
 {    2901, .8325427734375E+04, -.1348615844727E+04, .9481873168945E+03},
 {    2902, .8302246093750E+04, -.1354172119141E+04, .9585657348633E+03},
 {    2903, .8348611328125E+04, -.1354171997070E+04, .9378092651367E+03},
 {    2904, .8325427734375E+04, -.1365284545898E+04, .9481873168945E+03},
 {    2905, .8302246093750E+04, -.1376397094727E+04, .9585657348633E+03},
 {    2906, .8356454101562E+04, -.1153512084961E+04, .1073442993164E+04},
 {    2907, .8344863281250E+04, -.1153512084961E+04, .1078632080078E+04},
 {    2908, .8356520507812E+04, -.1150775634766E+04, .1073413452148E+04},
 {    2909, .8344863281250E+04, -.1149702026367E+04, .1078632080078E+04},
 {    2910, .8351264648438E+04, -.1153512084961E+04, .1061851684570E+04},
 {    2911, .8339673828125E+04, -.1153512084961E+04, .1067040649414E+04},
 {    2912, .8351333007812E+04, -.1150775634766E+04, .1061821899414E+04},
 {    2913, .8339673828125E+04, -.1149702026367E+04, .1067040649414E+04},
 {    2914, .8346075195312E+04, -.1153512084961E+04, .1050260009766E+04},
 {    2915, .8334484375000E+04, -.1153512084961E+04, .1055449218750E+04},
 {    2916, .8346143554688E+04, -.1150775634766E+04, .1050230224609E+04},
 {    2917, .8334484375000E+04, -.1149702026367E+04, .1055449218750E+04},
 {    2918, .8344782226562E+04, -.1153512084961E+04, .1047361816406E+04},
 {    2919, .8333187500000E+04, -.1153512084961E+04, .1052551025391E+04},
 {    2920, .8344845703125E+04, -.1150775634766E+04, .1047332397461E+04},
 {    2921, .8333187500000E+04, -.1149702026367E+04, .1052551025391E+04},
 {    2922, .8339590820312E+04, -.1153512084961E+04, .1035770751953E+04},
 {    2923, .8327998046875E+04, -.1153512084961E+04, .1040959594727E+04},
 {    2924, .8339656250000E+04, -.1150775634766E+04, .1035740966797E+04},
 {    2925, .8327998046875E+04, -.1149702026367E+04, .1040959594727E+04},
 {    2926, .8334400390625E+04, -.1153512084961E+04, .1024179321289E+04},
 {    2927, .8322810546875E+04, -.1153512084961E+04, .1029367919922E+04},
 {    2928, .8334469726562E+04, -.1150775634766E+04, .1024149536133E+04},
 {    2929, .8322810546875E+04, -.1149702026367E+04, .1029367919922E+04},
 {    2930, .8333103515625E+04, -.1153512084961E+04, .1021281066895E+04},
 {    2931, .8321513671875E+04, -.1153512084961E+04, .1026470092773E+04},
 {    2932, .8333168945312E+04, -.1150775634766E+04, .1021251586914E+04},
 {    2933, .8321513671875E+04, -.1149702026367E+04, .1026470092773E+04},
 {    2934, .8327915039062E+04, -.1153512084961E+04, .1009689575195E+04},
 {    2935, .8316325195312E+04, -.1153512084961E+04, .1014878845215E+04},
 {    2936, .8327980468750E+04, -.1150775634766E+04, .1009660278320E+04},
 {    2937, .8316325195312E+04, -.1149702026367E+04, .1014878845215E+04},
 {    2938, .8322724609375E+04, -.1153512084961E+04, .9980984497070E+03},
 {    2939, .8311132812500E+04, -.1153512084961E+04, .1003287475586E+04},
 {    2940, .8322793945312E+04, -.1150775634766E+04, .9980681152344E+03},
 {    2941, .8311132812500E+04, -.1149702026367E+04, .1003287475586E+04},
 {    2942, .8321427734375E+04, -.1153512084961E+04, .9951999511719E+03},
 {    2943, .8309836914062E+04, -.1153512084961E+04, .1000389099121E+04},
 {    2944, .8321496093750E+04, -.1150775634766E+04, .9951699218750E+03},
 {    2945, .8309836914062E+04, -.1149702026367E+04, .1000389099121E+04},
 {    2946, .8316240234375E+04, -.1153512084961E+04, .9836082763672E+03},
 {    2947, .8304648437500E+04, -.1153512084961E+04, .9887977294922E+03},
 {    2948, .8316306640625E+04, -.1150775634766E+04, .9835786132812E+03},
 {    2949, .8304648437500E+04, -.1149702026367E+04, .9887977294922E+03},
 {    2950, .8311048828125E+04, -.1153512084961E+04, .9720169677734E+03},
 {    2951, .8299458007812E+04, -.1153512084961E+04, .9772065429688E+03},
 {    2952, .8311118164062E+04, -.1150775634766E+04, .9719872436523E+03},
 {    2953, .8299458007812E+04, -.1149702026367E+04, .9772065429688E+03},
 {    2954, .8309751953125E+04, -.1153512084961E+04, .9691195068359E+03},
 {    2955, .8298160156250E+04, -.1153512084961E+04, .9743084106445E+03},
 {    2956, .8309819335938E+04, -.1150775634766E+04, .9690899047852E+03},
 {    2957, .8298160156250E+04, -.1149702026367E+04, .9743084106445E+03},
 {    2958, .8304563476562E+04, -.1153512084961E+04, .9575277099609E+03},
 {    2959, .8292974609375E+04, -.1153512084961E+04, .9627166748047E+03},
 {    2960, .8304630859375E+04, -.1150775634766E+04, .9574979248047E+03},
 {    2961, .8292974609375E+04, -.1149702026367E+04, .9627166748047E+03},
 {    2962, .8299374023438E+04, -.1153512084961E+04, .9459361572266E+03},
 {    2963, .8287781250000E+04, -.1153512084961E+04, .9511254882812E+03},
 {    2964, .8299442382812E+04, -.1150775634766E+04, .9459061279297E+03},
 {    2965, .8287781250000E+04, -.1149702026367E+04, .9511254882812E+03},
 {    2966, .8333272460938E+04, -.1153512084961E+04, .1083821044922E+04},
 {    2967, .8321680664062E+04, -.1153512084961E+04, .1089010131836E+04},
 {    2968, .8333272460938E+04, -.1149702026367E+04, .1083821044922E+04},
 {    2969, .8321680664062E+04, -.1149702026367E+04, .1089010131836E+04},
 {    2970, .8328083007812E+04, -.1153512084961E+04, .1072229858398E+04},
 {    2971, .8316491210938E+04, -.1153512084961E+04, .1077418945312E+04},
 {    2972, .8328083007812E+04, -.1149702026367E+04, .1072229858398E+04},
 {    2973, .8316491210938E+04, -.1149702026367E+04, .1077418945312E+04},
 {    2974, .8322892578125E+04, -.1153512084961E+04, .1060638183594E+04},
 {    2975, .8311302734375E+04, -.1153512084961E+04, .1065827270508E+04},
 {    2976, .8322892578125E+04, -.1149702026367E+04, .1060638183594E+04},
 {    2977, .8311302734375E+04, -.1149702026367E+04, .1065827270508E+04},
 {    2978, .8321595703125E+04, -.1153512084961E+04, .1057740356445E+04},
 {    2979, .8310006835938E+04, -.1153512084961E+04, .1062929321289E+04},
 {    2980, .8321595703125E+04, -.1149702026367E+04, .1057740356445E+04},
 {    2981, .8310006835938E+04, -.1149702026367E+04, .1062929321289E+04},
 {    2982, .8316409179688E+04, -.1153512084961E+04, .1046148437500E+04},
 {    2983, .8304816406250E+04, -.1153512084961E+04, .1051337646484E+04},
 {    2984, .8316409179688E+04, -.1149702026367E+04, .1046148437500E+04},
 {    2985, .8304816406250E+04, -.1149702026367E+04, .1051337646484E+04},
 {    2986, .8311216796875E+04, -.1153512084961E+04, .1034557128906E+04},
 {    2987, .8299626953125E+04, -.1153512084961E+04, .1039746093750E+04},
 {    2988, .8311216796875E+04, -.1149702026367E+04, .1034557128906E+04},
 {    2989, .8299626953125E+04, -.1149702026367E+04, .1039746093750E+04},
 {    2990, .8309919921875E+04, -.1153512084961E+04, .1031659179688E+04},
 {    2991, .8298330078125E+04, -.1153512084961E+04, .1036848388672E+04},
 {    2992, .8309919921875E+04, -.1149702026367E+04, .1031659179688E+04},
 {    2993, .8298330078125E+04, -.1149702026367E+04, .1036848388672E+04},
 {    2994, .8304731445312E+04, -.1153512084961E+04, .1020067871094E+04},
 {    2995, .8293141601562E+04, -.1153512084961E+04, .1025256958008E+04},
 {    2996, .8304731445312E+04, -.1149702026367E+04, .1020067871094E+04},
 {    2997, .8293141601562E+04, -.1149702026367E+04, .1025256958008E+04},
 {    2998, .8299541992188E+04, -.1153512084961E+04, .1008476074219E+04},
 {    2999, .8287951171875E+04, -.1153512084961E+04, .1013665771484E+04},
 {    3000, .8299541992188E+04, -.1149702026367E+04, .1008476074219E+04},
 {    3001, .8287951171875E+04, -.1149702026367E+04, .1013665771484E+04},
 {    3002, .8298244140625E+04, -.1153512084961E+04, .1005578613281E+04},
 {    3003, .8286653320312E+04, -.1153512084961E+04, .1010767395020E+04},
 {    3004, .8298244140625E+04, -.1149702026367E+04, .1005578613281E+04},
 {    3005, .8286653320312E+04, -.1149702026367E+04, .1010767395020E+04},
 {    3006, .8293055664062E+04, -.1153512084961E+04, .9939868164062E+03},
 {    3007, .8281464843750E+04, -.1153512084961E+04, .9991757812500E+03},
 {    3008, .8293055664062E+04, -.1149702026367E+04, .9939868164062E+03},
 {    3009, .8281464843750E+04, -.1149702026367E+04, .9991757812500E+03},
 {    3010, .8287866210938E+04, -.1153512084961E+04, .9823952636719E+03},
 {    3011, .8276275390625E+04, -.1153512084961E+04, .9875843505859E+03},
 {    3012, .8287866210938E+04, -.1149702026367E+04, .9823952636719E+03},
 {    3013, .8276275390625E+04, -.1149702026367E+04, .9875843505859E+03},
 {    3014, .8286571289062E+04, -.1153512084961E+04, .9794973144531E+03},
 {    3015, .8274979492188E+04, -.1153512084961E+04, .9846864624023E+03},
 {    3016, .8286571289062E+04, -.1149702026367E+04, .9794973144531E+03},
 {    3017, .8274979492188E+04, -.1149702026367E+04, .9846864624023E+03},
 {    3018, .8281380859375E+04, -.1153512084961E+04, .9679062500000E+03},
 {    3019, .8269791015625E+04, -.1153512084961E+04, .9730949707031E+03},
 {    3020, .8281380859375E+04, -.1149702026367E+04, .9679062500000E+03},
 {    3021, .8269791015625E+04, -.1149702026367E+04, .9730949707031E+03},
 {    3022, .8276192382812E+04, -.1153512084961E+04, .9563143310547E+03},
 {    3023, .8264600585938E+04, -.1153512084961E+04, .9615036621094E+03},
 {    3024, .8276192382812E+04, -.1149702026367E+04, .9563143310547E+03},
 {    3025, .8264600585938E+04, -.1149702026367E+04, .9615036621094E+03},
 {    3026, .8310089843750E+04, -.1153512084961E+04, .1094199707031E+04},
 {    3027, .8309917968750E+04, -.1150696289062E+04, .1094276123047E+04},
 {    3028, .8304899414062E+04, -.1153512084961E+04, .1082608154297E+04},
 {    3029, .8304728515625E+04, -.1150696289062E+04, .1082684814453E+04},
 {    3030, .8299710937500E+04, -.1153512084961E+04, .1071016357422E+04},
 {    3031, .8299537109375E+04, -.1150696289062E+04, .1071093017578E+04},
 {    3032, .8298413085938E+04, -.1153512084961E+04, .1068118286133E+04},
 {    3033, .8298241210938E+04, -.1150696289062E+04, .1068195068359E+04},
 {    3034, .8293222656250E+04, -.1153512084961E+04, .1056526855469E+04},
 {    3035, .8293050781250E+04, -.1150696289062E+04, .1056603759766E+04},
 {    3036, .8288036132812E+04, -.1153512084961E+04, .1044935058594E+04},
 {    3037, .8287862304688E+04, -.1150696289062E+04, .1045012207031E+04},
 {    3038, .8286740234375E+04, -.1153512084961E+04, .1042037475586E+04},
 {    3039, .8286567382812E+04, -.1150696289062E+04, .1042114135742E+04},
 {    3040, .8281547851562E+04, -.1153512084961E+04, .1030446166992E+04},
 {    3041, .8281377929688E+04, -.1150696289062E+04, .1030522705078E+04},
 {    3042, .8276358398438E+04, -.1153512084961E+04, .1018854675293E+04},
 {    3043, .8276186523438E+04, -.1150696289062E+04, .1018931396484E+04},
 {    3044, .8275063476562E+04, -.1153512084961E+04, .1015956665039E+04},
 {    3045, .8274890625000E+04, -.1150696289062E+04, .1016033325195E+04},
 {    3046, .8269874023438E+04, -.1153512084961E+04, .1004364807129E+04},
 {    3047, .8269702148438E+04, -.1150696289062E+04, .1004441650391E+04},
 {    3048, .8264683593750E+04, -.1153512084961E+04, .9927739257812E+03},
 {    3049, .8264510742188E+04, -.1150696289062E+04, .9928507080078E+03},
 {    3050, .8263387695312E+04, -.1153512084961E+04, .9898752441406E+03},
 {    3051, .8263213867188E+04, -.1150696289062E+04, .9899526367188E+03},
 {    3052, .8258200195312E+04, -.1153512084961E+04, .9782839355469E+03},
 {    3053, .8258027343750E+04, -.1150696289062E+04, .9783605957031E+03},
 {    3054, .8253008789062E+04, -.1153512084961E+04, .9666925048828E+03},
 {    3055, .8252834960938E+04, -.1150696289062E+04, .9667696533203E+03},
 {    3056, .8310089843750E+04, -.1417671997070E+04, .1094199707031E+04},
 {    3057, .8321680664062E+04, -.1417671997070E+04, .1089010131836E+04},
 {    3058, .8310121093750E+04, -.1420615112305E+04, .1094184814453E+04},
 {    3059, .8321680664062E+04, -.1421482055664E+04, .1089010131836E+04},
 {    3060, .8304899414062E+04, -.1417671997070E+04, .1082608154297E+04},
 {    3061, .8316491210938E+04, -.1417671997070E+04, .1077418945312E+04},
 {    3062, .8304932617188E+04, -.1420615112305E+04, .1082593261719E+04},
 {    3063, .8316491210938E+04, -.1421482055664E+04, .1077418945312E+04},
 {    3064, .8299710937500E+04, -.1417671997070E+04, .1071016357422E+04},
 {    3065, .8311302734375E+04, -.1417671997070E+04, .1065827270508E+04},
 {    3066, .8299743164062E+04, -.1420615112305E+04, .1071002075195E+04},
 {    3067, .8311302734375E+04, -.1421482055664E+04, .1065827270508E+04},
 {    3068, .8298413085938E+04, -.1417671997070E+04, .1068118286133E+04},
 {    3069, .8310006835938E+04, -.1417671997070E+04, .1062929321289E+04},
 {    3070, .8298444335938E+04, -.1420615112305E+04, .1068104248047E+04},
 {    3071, .8310006835938E+04, -.1421482055664E+04, .1062929321289E+04},
 {    3072, .8293222656250E+04, -.1417671997070E+04, .1056526855469E+04},
 {    3073, .8304816406250E+04, -.1417671997070E+04, .1051337890625E+04},
 {    3074, .8293255859375E+04, -.1420615112305E+04, .1056512939453E+04},
 {    3075, .8304816406250E+04, -.1421482055664E+04, .1051337890625E+04},
 {    3076, .8288036132812E+04, -.1417671997070E+04, .1044935058594E+04},
 {    3077, .8299626953125E+04, -.1417671997070E+04, .1039746093750E+04},
 {    3078, .8288067382812E+04, -.1420615112305E+04, .1044921020508E+04},
 {    3079, .8299626953125E+04, -.1421482055664E+04, .1039746093750E+04},
 {    3080, .8286740234375E+04, -.1417671997070E+04, .1042037475586E+04},
 {    3081, .8298330078125E+04, -.1417671997070E+04, .1036848388672E+04},
 {    3082, .8286768554688E+04, -.1420615112305E+04, .1042023193359E+04},
 {    3083, .8298330078125E+04, -.1421482055664E+04, .1036848388672E+04},
 {    3084, .8281547851562E+04, -.1417671997070E+04, .1030446166992E+04},
 {    3085, .8293141601562E+04, -.1417671997070E+04, .1025256958008E+04},
 {    3086, .8281580078125E+04, -.1420615112305E+04, .1030432128906E+04},
 {    3087, .8293141601562E+04, -.1421482055664E+04, .1025256958008E+04},
 {    3088, .8276358398438E+04, -.1417671997070E+04, .1018854675293E+04},
 {    3089, .8287951171875E+04, -.1417671997070E+04, .1013665771484E+04},
 {    3090, .8276393554688E+04, -.1420615112305E+04, .1018839904785E+04},
 {    3091, .8287951171875E+04, -.1421482055664E+04, .1013665771484E+04},
 {    3092, .8275063476562E+04, -.1417671997070E+04, .1015956665039E+04},
 {    3093, .8286653320312E+04, -.1417671997070E+04, .1010767395020E+04},
 {    3094, .8275093750000E+04, -.1420615112305E+04, .1015942138672E+04},
 {    3095, .8286653320312E+04, -.1421482055664E+04, .1010767395020E+04},
 {    3096, .8269874023438E+04, -.1417671997070E+04, .1004364807129E+04},
 {    3097, .8281464843750E+04, -.1417671997070E+04, .9991757812500E+03},
 {    3098, .8269904296875E+04, -.1420615112305E+04, .1004350830078E+04},
 {    3099, .8281464843750E+04, -.1421482055664E+04, .9991757812500E+03},
 {    3100, .8264683593750E+04, -.1417671997070E+04, .9927739257812E+03},
 {    3101, .8276275390625E+04, -.1417671997070E+04, .9875843505859E+03},
 {    3102, .8264715820312E+04, -.1420615112305E+04, .9927590942383E+03},
 {    3103, .8276275390625E+04, -.1421482055664E+04, .9875843505859E+03},
 {    3104, .8263387695312E+04, -.1417671997070E+04, .9898752441406E+03},
 {    3105, .8274979492188E+04, -.1417671997070E+04, .9846864624023E+03},
 {    3106, .8263417968750E+04, -.1420615112305E+04, .9898618164062E+03},
 {    3107, .8274979492188E+04, -.1421482055664E+04, .9846864624023E+03},
 {    3108, .8258200195312E+04, -.1417671997070E+04, .9782839355469E+03},
 {    3109, .8269791015625E+04, -.1417671997070E+04, .9730949707031E+03},
 {    3110, .8258229492188E+04, -.1420615112305E+04, .9782700195312E+03},
 {    3111, .8269791015625E+04, -.1421482055664E+04, .9730949707031E+03},
 {    3112, .8253008789062E+04, -.1417671997070E+04, .9666925048828E+03},
 {    3113, .8264600585938E+04, -.1417671997070E+04, .9615036621094E+03},
 {    3114, .8253041015625E+04, -.1420615112305E+04, .9666783447266E+03},
 {    3115, .8264600585938E+04, -.1421482055664E+04, .9615036621094E+03},
 {    3116, .8333272460938E+04, -.1417671997070E+04, .1083821044922E+04},
 {    3117, .8344863281250E+04, -.1417671997070E+04, .1078632080078E+04},
 {    3118, .8333272460938E+04, -.1421482055664E+04, .1083821044922E+04},
 {    3119, .8344863281250E+04, -.1421482055664E+04, .1078632080078E+04},
 {    3120, .8328083007812E+04, -.1417671997070E+04, .1072229858398E+04},
 {    3121, .8339673828125E+04, -.1417671997070E+04, .1067040893555E+04},
 {    3122, .8328083007812E+04, -.1421482055664E+04, .1072229858398E+04},
 {    3123, .8339673828125E+04, -.1421482055664E+04, .1067040893555E+04},
 {    3124, .8322892578125E+04, -.1417671997070E+04, .1060638183594E+04},
 {    3125, .8334484375000E+04, -.1417671997070E+04, .1055449218750E+04},
 {    3126, .8322892578125E+04, -.1421482055664E+04, .1060638183594E+04},
 {    3127, .8334484375000E+04, -.1421482055664E+04, .1055449218750E+04},
 {    3128, .8321595703125E+04, -.1417671997070E+04, .1057740356445E+04},
 {    3129, .8333187500000E+04, -.1417671997070E+04, .1052551025391E+04},
 {    3130, .8321595703125E+04, -.1421482055664E+04, .1057740356445E+04},
 {    3131, .8333187500000E+04, -.1421482055664E+04, .1052551025391E+04},
 {    3132, .8316409179688E+04, -.1417671997070E+04, .1046148437500E+04},
 {    3133, .8327998046875E+04, -.1417671997070E+04, .1040959594727E+04},
 {    3134, .8316409179688E+04, -.1421482055664E+04, .1046148437500E+04},
 {    3135, .8327998046875E+04, -.1421482055664E+04, .1040959594727E+04},
 {    3136, .8311216796875E+04, -.1417671997070E+04, .1034557128906E+04},
 {    3137, .8322810546875E+04, -.1417671997070E+04, .1029367919922E+04},
 {    3138, .8311216796875E+04, -.1421482055664E+04, .1034557128906E+04},
 {    3139, .8322810546875E+04, -.1421482055664E+04, .1029367919922E+04},
 {    3140, .8309919921875E+04, -.1417671997070E+04, .1031659423828E+04},
 {    3141, .8321513671875E+04, -.1417671997070E+04, .1026470336914E+04},
 {    3142, .8309919921875E+04, -.1421482055664E+04, .1031659423828E+04},
 {    3143, .8321513671875E+04, -.1421482055664E+04, .1026470336914E+04},
 {    3144, .8304731445312E+04, -.1417671997070E+04, .1020067871094E+04},
 {    3145, .8316325195312E+04, -.1417671997070E+04, .1014878845215E+04},
 {    3146, .8304731445312E+04, -.1421482055664E+04, .1020067871094E+04},
 {    3147, .8316325195312E+04, -.1421482055664E+04, .1014878845215E+04},
 {    3148, .8299541992188E+04, -.1417671997070E+04, .1008476074219E+04},
 {    3149, .8311132812500E+04, -.1417671997070E+04, .1003287475586E+04},
 {    3150, .8299541992188E+04, -.1421482055664E+04, .1008476074219E+04},
 {    3151, .8311132812500E+04, -.1421482055664E+04, .1003287475586E+04},
 {    3152, .8298244140625E+04, -.1417671997070E+04, .1005578613281E+04},
 {    3153, .8309836914062E+04, -.1417671997070E+04, .1000389099121E+04},
 {    3154, .8298244140625E+04, -.1421482055664E+04, .1005578613281E+04},
 {    3155, .8309836914062E+04, -.1421482055664E+04, .1000389099121E+04},
 {    3156, .8293055664062E+04, -.1417671997070E+04, .9939868164062E+03},
 {    3157, .8304648437500E+04, -.1417671997070E+04, .9887977294922E+03},
 {    3158, .8293055664062E+04, -.1421482055664E+04, .9939868164062E+03},
 {    3159, .8304648437500E+04, -.1421482055664E+04, .9887977294922E+03},
 {    3160, .8287866210938E+04, -.1417671997070E+04, .9823955078125E+03},
 {    3161, .8299458007812E+04, -.1417671997070E+04, .9772065429688E+03},
 {    3162, .8287866210938E+04, -.1421482055664E+04, .9823955078125E+03},
 {    3163, .8299458007812E+04, -.1421482055664E+04, .9772065429688E+03},
 {    3164, .8286571289062E+04, -.1417671997070E+04, .9794973144531E+03},
 {    3165, .8298160156250E+04, -.1417671997070E+04, .9743084106445E+03},
 {    3166, .8286571289062E+04, -.1421482055664E+04, .9794973144531E+03},
 {    3167, .8298160156250E+04, -.1421482055664E+04, .9743084106445E+03},
 {    3168, .8281380859375E+04, -.1417671997070E+04, .9679062500000E+03},
 {    3169, .8292974609375E+04, -.1417671997070E+04, .9627166748047E+03},
 {    3170, .8281380859375E+04, -.1421482055664E+04, .9679062500000E+03},
 {    3171, .8292974609375E+04, -.1421482055664E+04, .9627166748047E+03},
 {    3172, .8276192382812E+04, -.1417671997070E+04, .9563143310547E+03},
 {    3173, .8287782226562E+04, -.1417671997070E+04, .9511254882812E+03},
 {    3174, .8276192382812E+04, -.1421482055664E+04, .9563143310547E+03},
 {    3175, .8287782226562E+04, -.1421482055664E+04, .9511254882812E+03},
 {    3176, .8356454101562E+04, -.1417671997070E+04, .1073442993164E+04},
 {    3177, .8356752929688E+04, -.1420387207031E+04, .1073309326172E+04},
 {    3178, .8351264648438E+04, -.1417671997070E+04, .1061851684570E+04},
 {    3179, .8351563476562E+04, -.1420387207031E+04, .1061718261719E+04},
 {    3180, .8346075195312E+04, -.1417671997070E+04, .1050260009766E+04},
 {    3181, .8346373046875E+04, -.1420387207031E+04, .1050126708984E+04},
 {    3182, .8344782226562E+04, -.1417671997070E+04, .1047361816406E+04},
 {    3183, .8345077148438E+04, -.1420387207031E+04, .1047229003906E+04},
 {    3184, .8339590820312E+04, -.1417671997070E+04, .1035770751953E+04},
 {    3185, .8339888671875E+04, -.1420387207031E+04, .1035637451172E+04},
 {    3186, .8334400390625E+04, -.1417671997070E+04, .1024179321289E+04},
 {    3187, .8334699218750E+04, -.1420387207031E+04, .1024045654297E+04},
 {    3188, .8333103515625E+04, -.1417671997070E+04, .1021281066895E+04},
 {    3189, .8333401367188E+04, -.1420387207031E+04, .1021148071289E+04},
 {    3190, .8327915039062E+04, -.1417671997070E+04, .1009689575195E+04},
 {    3191, .8328213867188E+04, -.1420387207031E+04, .1009556274414E+04},
 {    3192, .8322724609375E+04, -.1417671997070E+04, .9980984497070E+03},
 {    3193, .8323023437500E+04, -.1420387207031E+04, .9979649658203E+03},
 {    3194, .8321426757812E+04, -.1417671997070E+04, .9952003173828E+03},
 {    3195, .8321725585938E+04, -.1420387207031E+04, .9950671386719E+03},
 {    3196, .8316240234375E+04, -.1417671997070E+04, .9836082763672E+03},
 {    3197, .8316538085938E+04, -.1420387207031E+04, .9834755859375E+03},
 {    3198, .8311048828125E+04, -.1417671997070E+04, .9720169677734E+03},
 {    3199, .8311348632812E+04, -.1420387207031E+04, .9718837280273E+03},
 {    3200, .8309751953125E+04, -.1417671997070E+04, .9691195068359E+03},
 {    3201, .8310051757812E+04, -.1420387207031E+04, .9689860839844E+03},
 {    3202, .8304563476562E+04, -.1417671997070E+04, .9575277099609E+03},
 {    3203, .8304861328125E+04, -.1420387207031E+04, .9573945312500E+03},
 {    3204, .8299374023438E+04, -.1417671997070E+04, .9459361572266E+03},
 {    3205, .8299671875000E+04, -.1420387207031E+04, .9458029785156E+03},
 {    3206, .8414109375000E+04, -.1196710449219E+04, .1047633666992E+04},
 {    3207, .8404593750000E+04, -.1181935180664E+04, .1051893066406E+04},
 {    3208, .8394711914062E+04, -.1171106323242E+04, .1056317016602E+04},
 {    3209, .8383438476562E+04, -.1163107421875E+04, .1061363769531E+04},
 {    3210, .8415550781250E+04, -.1193971191406E+04, .1046988159180E+04},
 {    3211, .8406066406250E+04, -.1179587768555E+04, .1051234130859E+04},
 {    3212, .8396022460938E+04, -.1169240722656E+04, .1055729980469E+04},
 {    3213, .8384544921875E+04, -.1160875366211E+04, .1060868408203E+04},
 {    3214, .8408918945312E+04, -.1196710449219E+04, .1036042602539E+04},
 {    3215, .8399406250000E+04, -.1181935180664E+04, .1040301025391E+04},
 {    3216, .8389521484375E+04, -.1171106323242E+04, .1044725463867E+04},
 {    3217, .8378250000000E+04, -.1163107421875E+04, .1049771972656E+04},
 {    3218, .8410363281250E+04, -.1193971191406E+04, .1035395629883E+04},
 {    3219, .8400876953125E+04, -.1179587768555E+04, .1039642456055E+04},
 {    3220, .8390833007812E+04, -.1169240722656E+04, .1044138427734E+04},
 {    3221, .8379357421875E+04, -.1160875366211E+04, .1049276977539E+04},
 {    3222, .8403730468750E+04, -.1196710449219E+04, .1024450805664E+04},
 {    3223, .8394214843750E+04, -.1181935180664E+04, .1028709594727E+04},
 {    3224, .8384333007812E+04, -.1171106323242E+04, .1033134155273E+04},
 {    3225, .8373059570312E+04, -.1163107421875E+04, .1038180786133E+04},
 {    3226, .8405173828125E+04, -.1193971191406E+04, .1023804321289E+04},
 {    3227, .8395687500000E+04, -.1179587768555E+04, .1028051269531E+04},
 {    3228, .8385645507812E+04, -.1169240722656E+04, .1032546875000E+04},
 {    3229, .8374166992188E+04, -.1160875366211E+04, .1037685302734E+04},
 {    3230, .8402432617188E+04, -.1196710449219E+04, .1021553100586E+04},
 {    3231, .8392916992188E+04, -.1181935180664E+04, .1025811889648E+04},
 {    3232, .8383035156250E+04, -.1171106323242E+04, .1030236694336E+04},
 {    3233, .8371762695312E+04, -.1163107421875E+04, .1035282348633E+04},
 {    3234, .8403874023438E+04, -.1193971191406E+04, .1020906982422E+04},
 {    3235, .8394391601562E+04, -.1179587768555E+04, .1025153076172E+04},
 {    3236, .8384346679688E+04, -.1169240722656E+04, .1029648803711E+04},
 {    3237, .8372871093750E+04, -.1160875366211E+04, .1034787597656E+04},
 {    3238, .8397244140625E+04, -.1196710449219E+04, .1009961181641E+04},
 {    3239, .8387728515625E+04, -.1181935180664E+04, .1014220214844E+04},
 {    3240, .8377846679688E+04, -.1171106323242E+04, .1018644836426E+04},
 {    3241, .8366573242188E+04, -.1163107421875E+04, .1023691284180E+04},
 {    3242, .8398685546875E+04, -.1193971191406E+04, .1009315063477E+04},
 {    3243, .8389201171875E+04, -.1179587768555E+04, .1013561523438E+04},
 {    3244, .8379159179688E+04, -.1169240722656E+04, .1018057434082E+04},
 {    3245, .8367679687500E+04, -.1160875366211E+04, .1023195800781E+04},
 {    3246, .8392054687500E+04, -.1196710449219E+04, .9983700561523E+03},
 {    3247, .8382541015625E+04, -.1181935180664E+04, .1002628540039E+04},
 {    3248, .8372656250000E+04, -.1171106323242E+04, .1007053344727E+04},
 {    3249, .8361383789062E+04, -.1163107421875E+04, .1012099426270E+04},
 {    3250, .8393499023438E+04, -.1193971191406E+04, .9977236328125E+03},
 {    3251, .8384011718750E+04, -.1179587768555E+04, .1001970092773E+04},
 {    3252, .8373969726562E+04, -.1169240722656E+04, .1006465942383E+04},
 {    3253, .8362491210938E+04, -.1160875366211E+04, .1011604553223E+04},
 {    3254, .8390756835938E+04, -.1196710449219E+04, .9954720458984E+03},
 {    3255, .8381243164062E+04, -.1181935180664E+04, .9997308349609E+03},
 {    3256, .8371360351562E+04, -.1171106323242E+04, .1004155212402E+04},
 {    3257, .8360087890625E+04, -.1163107421875E+04, .1009201477051E+04},
 {    3258, .8392202148438E+04, -.1193971191406E+04, .9948258056641E+03},
 {    3259, .8382714843750E+04, -.1179587768555E+04, .9990728149414E+03},
 {    3260, .8372671875000E+04, -.1169240722656E+04, .1003568115234E+04},
 {    3261, .8361193359375E+04, -.1160875366211E+04, .1008706420898E+04},
 {    3262, .8385566406250E+04, -.1196710449219E+04, .9838812255859E+03},
 {    3263, .8376055664062E+04, -.1181935180664E+04, .9881390991211E+03},
 {    3264, .8366171875000E+04, -.1171106323242E+04, .9925639648438E+03},
 {    3265, .8354898437500E+04, -.1163107421875E+04, .9976102294922E+03},
 {    3266, .8387011718750E+04, -.1193971191406E+04, .9832340087891E+03},
 {    3267, .8377526367188E+04, -.1179587768555E+04, .9874804687500E+03},
 {    3268, .8367482421875E+04, -.1169240722656E+04, .9919765014648E+03},
 {    3269, .8356003906250E+04, -.1160875366211E+04, .9971148071289E+03},
 {    3270, .8380380859375E+04, -.1196710449219E+04, .9722885742188E+03},
 {    3271, .8370866210938E+04, -.1181935180664E+04, .9765481567383E+03},
 {    3272, .8360981445312E+04, -.1171106323242E+04, .9809725341797E+03},
 {    3273, .8349708984375E+04, -.1163107421875E+04, .9860187988281E+03},
 {    3274, .8381824218750E+04, -.1193971191406E+04, .9716420898438E+03},
 {    3275, .8372334960938E+04, -.1179587890625E+04, .9758894042969E+03},
 {    3276, .8362293945312E+04, -.1169240722656E+04, .9803850097656E+03},
 {    3277, .8350816406250E+04, -.1160875366211E+04, .9855231323242E+03},
 {    3278, .8379081054688E+04, -.1196710449219E+04, .9693909912109E+03},
 {    3279, .8369567382812E+04, -.1181935180664E+04, .9736498413086E+03},
 {    3280, .8359683593750E+04, -.1171106323242E+04, .9780749511719E+03},
 {    3281, .8348412109375E+04, -.1163107421875E+04, .9831212158203E+03},
 {    3282, .8380524414062E+04, -.1193971191406E+04, .9687448730469E+03},
 {    3283, .8371040039062E+04, -.1179587890625E+04, .9729910888672E+03},
 {    3284, .8360996093750E+04, -.1169240722656E+04, .9774871826172E+03},
 {    3285, .8349520507812E+04, -.1160875366211E+04, .9826252441406E+03},
 {    3286, .8373892578125E+04, -.1196710449219E+04, .9577994384766E+03},
 {    3287, .8364378906250E+04, -.1181935180664E+04, .9620584106445E+03},
 {    3288, .8354495117188E+04, -.1171106323242E+04, .9664827880859E+03},
 {    3289, .8343223632812E+04, -.1163107421875E+04, .9715290527344E+03},
 {    3290, .8375336914062E+04, -.1193971191406E+04, .9571532592773E+03},
 {    3291, .8365851562500E+04, -.1179587890625E+04, .9613994140625E+03},
 {    3292, .8355807617188E+04, -.1169240722656E+04, .9658959350586E+03},
 {    3293, .8344329101562E+04, -.1160875366211E+04, .9710339965820E+03},
 {    3294, .8368704101562E+04, -.1196710449219E+04, .9462077636719E+03},
 {    3295, .8359191406250E+04, -.1181935180664E+04, .9504667968750E+03},
 {    3296, .8349305664062E+04, -.1171106323242E+04, .9548912963867E+03},
 {    3297, .8338034179688E+04, -.1163107421875E+04, .9599376220703E+03},
 {    3298, .8370147460938E+04, -.1193971191406E+04, .9455616455078E+03},
 {    3299, .8360660156250E+04, -.1179587890625E+04, .9498082885742E+03},
 {    3300, .8350618164062E+04, -.1169240722656E+04, .9543040161133E+03},
 {    3301, .8339141601562E+04, -.1160875366211E+04, .9594423828125E+03},
 {    3302, .8367408203125E+04, -.1196710449219E+04, .9433099365234E+03},
 {    3303, .8357893554688E+04, -.1181935180664E+04, .9475689697266E+03},
 {    3304, .8348008789062E+04, -.1171106323242E+04, .9519936523438E+03},
 {    3305, .8336736328125E+04, -.1163107421875E+04, .9570397949219E+03},
 {    3306, .8368849609375E+04, -.1193971191406E+04, .9426639404297E+03},
 {    3307, .8359364257812E+04, -.1179587890625E+04, .9469100952148E+03},
 {    3308, .8349319335938E+04, -.1169240722656E+04, .9514063110352E+03},
 {    3309, .8337843750000E+04, -.1160875366211E+04, .9565447998047E+03},
 {    3310, .8362215820312E+04, -.1196710449219E+04, .9317186279297E+03},
 {    3311, .8352705078125E+04, -.1181935180664E+04, .9359776611328E+03},
 {    3312, .8342818359375E+04, -.1171106323242E+04, .9404022216797E+03},
 {    3313, .8331548828125E+04, -.1163107421875E+04, .9454482421875E+03},
 {    3314, .8363661132812E+04, -.1193971191406E+04, .9310722656250E+03},
 {    3315, .8354176757812E+04, -.1179587890625E+04, .9353189086914E+03},
 {    3316, .8344132812500E+04, -.1169240722656E+04, .9398144531250E+03},
 {    3317, .8332654296875E+04, -.1160875366211E+04, .9449530029297E+03},
 {    3318, .8357027343750E+04, -.1196710449219E+04, .9201270751953E+03},
 {    3319, .8347515625000E+04, -.1181935180664E+04, .9243858642578E+03},
 {    3320, .8337629882812E+04, -.1171106323242E+04, .9288106689453E+03},
 {    3321, .8326359375000E+04, -.1163107421875E+04, .9338565673828E+03},
 {    3322, .8358473632812E+04, -.1193971191406E+04, .9194806518555E+03},
 {    3323, .8348985351562E+04, -.1179587890625E+04, .9237270507812E+03},
 {    3324, .8338943359375E+04, -.1169240722656E+04, .9282229614258E+03},
 {    3325, .8327464843750E+04, -.1160875366211E+04, .9333614501953E+03},
 {    3326, .8383438476562E+04, -.1408076660156E+04, .1061363769531E+04},
 {    3327, .8394711914062E+04, -.1400077758789E+04, .1056317016602E+04},
 {    3328, .8404347656250E+04, -.1389518310547E+04, .1052002807617E+04},
 {    3329, .8414927734375E+04, -.1373575683594E+04, .1047267456055E+04},
 {    3330, .8384544921875E+04, -.1410308715820E+04, .1060868408203E+04},
 {    3331, .8396022460938E+04, -.1401943359375E+04, .1055729980469E+04},
 {    3332, .8406066406250E+04, -.1391596191406E+04, .1051234130859E+04},
 {    3333, .8416159179688E+04, -.1375984741211E+04, .1046715576172E+04},
 {    3334, .8378250000000E+04, -.1408076660156E+04, .1049771972656E+04},
 {    3335, .8389521484375E+04, -.1400077758789E+04, .1044725463867E+04},
 {    3336, .8399160156250E+04, -.1389518310547E+04, .1040411010742E+04},
 {    3337, .8409739257812E+04, -.1373575683594E+04, .1035675537109E+04},
 {    3338, .8379357421875E+04, -.1410308715820E+04, .1049276977539E+04},
 {    3339, .8390834960938E+04, -.1401943359375E+04, .1044138305664E+04},
 {    3340, .8400876953125E+04, -.1391596191406E+04, .1039642456055E+04},
 {    3341, .8410970703125E+04, -.1375984741211E+04, .1035124267578E+04},
 {    3342, .8373059570312E+04, -.1408076660156E+04, .1038180786133E+04},
 {    3343, .8384333007812E+04, -.1400077758789E+04, .1033134155273E+04},
 {    3344, .8393970703125E+04, -.1389518310547E+04, .1028820068359E+04},
 {    3345, .8404549804688E+04, -.1373575683594E+04, .1024083496094E+04},
 {    3346, .8374166992188E+04, -.1410308715820E+04, .1037685302734E+04},
 {    3347, .8385645507812E+04, -.1401943359375E+04, .1032546875000E+04},
 {    3348, .8395687500000E+04, -.1391596191406E+04, .1028051269531E+04},
 {    3349, .8405781250000E+04, -.1375984741211E+04, .1023532714844E+04},
 {    3350, .8371762695312E+04, -.1408076660156E+04, .1035282348633E+04},
 {    3351, .8383035156250E+04, -.1400077758789E+04, .1030236694336E+04},
 {    3352, .8392673828125E+04, -.1389518310547E+04, .1025921142578E+04},
 {    3353, .8403252929688E+04, -.1373575683594E+04, .1021185668945E+04},
 {    3354, .8372871093750E+04, -.1410308715820E+04, .1034787597656E+04},
 {    3355, .8384346679688E+04, -.1401943359375E+04, .1029648803711E+04},
 {    3356, .8394391601562E+04, -.1391596191406E+04, .1025153076172E+04},
 {    3357, .8404485351562E+04, -.1375984741211E+04, .1020634521484E+04},
 {    3358, .8366573242188E+04, -.1408076660156E+04, .1023691284180E+04},
 {    3359, .8377846679688E+04, -.1400077758789E+04, .1018644836426E+04},
 {    3360, .8387482421875E+04, -.1389518310547E+04, .1014330566406E+04},
 {    3361, .8398064453125E+04, -.1373575683594E+04, .1009594238281E+04},
 {    3362, .8367679687500E+04, -.1410308715820E+04, .1023195800781E+04},
 {    3363, .8379159179688E+04, -.1401943359375E+04, .1018057434082E+04},
 {    3364, .8389201171875E+04, -.1391596191406E+04, .1013561523438E+04},
 {    3365, .8399293945312E+04, -.1375984741211E+04, .1009042358398E+04},
 {    3366, .8361383789062E+04, -.1408076660156E+04, .1012099426270E+04},
 {    3367, .8372656250000E+04, -.1400077758789E+04, .1007053344727E+04},
 {    3368, .8382294921875E+04, -.1389518310547E+04, .1002739135742E+04},
 {    3369, .8392873046875E+04, -.1373575683594E+04, .9980029296875E+03},
 {    3370, .8362491210938E+04, -.1410308715820E+04, .1011604553223E+04},
 {    3371, .8373969726562E+04, -.1401943359375E+04, .1006465942383E+04},
 {    3372, .8384011718750E+04, -.1391596191406E+04, .1001970092773E+04},
 {    3373, .8394105468750E+04, -.1375984741211E+04, .9974515991211E+03},
 {    3374, .8360087890625E+04, -.1408076660156E+04, .1009201477051E+04},
 {    3375, .8371360351562E+04, -.1400077758789E+04, .1004155456543E+04},
 {    3376, .8380996093750E+04, -.1389518310547E+04, .9998408813477E+03},
 {    3377, .8391576171875E+04, -.1373575683594E+04, .9951052246094E+03},
 {    3378, .8361193359375E+04, -.1410308715820E+04, .1008706420898E+04},
 {    3379, .8372671875000E+04, -.1401943359375E+04, .1003568115234E+04},
 {    3380, .8382714843750E+04, -.1391596191406E+04, .9990728149414E+03},
 {    3381, .8392807617188E+04, -.1375984741211E+04, .9945540161133E+03},
 {    3382, .8354898437500E+04, -.1408076660156E+04, .9976102294922E+03},
 {    3383, .8366171875000E+04, -.1400077758789E+04, .9925642089844E+03},
 {    3384, .8375808593750E+04, -.1389518310547E+04, .9882491455078E+03},
 {    3385, .8386387695312E+04, -.1373575683594E+04, .9835132446289E+03},
 {    3386, .8356005859375E+04, -.1410308715820E+04, .9971148681641E+03},
 {    3387, .8367482421875E+04, -.1401943359375E+04, .9919765014648E+03},
 {    3388, .8377526367188E+04, -.1391596191406E+04, .9874804687500E+03},
 {    3389, .8387620117188E+04, -.1375984741211E+04, .9829624023438E+03},
 {    3390, .8349708984375E+04, -.1408076660156E+04, .9860187988281E+03},
 {    3391, .8360981445312E+04, -.1400077880859E+04, .9809725341797E+03},
 {    3392, .8370618164062E+04, -.1389518310547E+04, .9766577148438E+03},
 {    3393, .8381199218750E+04, -.1373575683594E+04, .9719219970703E+03},
 {    3394, .8350816406250E+04, -.1410308715820E+04, .9855231323242E+03},
 {    3395, .8362293945312E+04, -.1401943359375E+04, .9803850097656E+03},
 {    3396, .8372334960938E+04, -.1391596313477E+04, .9758894042969E+03},
 {    3397, .8382429687500E+04, -.1375984741211E+04, .9713704833984E+03},
 {    3398, .8348412109375E+04, -.1408076660156E+04, .9831212158203E+03},
 {    3399, .8359683593750E+04, -.1400077880859E+04, .9780749511719E+03},
 {    3400, .8369321289062E+04, -.1389518310547E+04, .9737595214844E+03},
 {    3401, .8379900390625E+04, -.1373575683594E+04, .9690239257812E+03},
 {    3402, .8349520507812E+04, -.1410308715820E+04, .9826252441406E+03},
 {    3403, .8360996093750E+04, -.1401943359375E+04, .9774871826172E+03},
 {    3404, .8371040039062E+04, -.1391596313477E+04, .9729910888672E+03},
 {    3405, .8381133789062E+04, -.1375984741211E+04, .9684728393555E+03},
 {    3406, .8343223632812E+04, -.1408076660156E+04, .9715290527344E+03},
 {    3407, .8354495117188E+04, -.1400077880859E+04, .9664827880859E+03},
 {    3408, .8364132812500E+04, -.1389518310547E+04, .9621687011719E+03},
 {    3409, .8374713867188E+04, -.1373575683594E+04, .9574324951172E+03},
 {    3410, .8344329101562E+04, -.1410308715820E+04, .9710339965820E+03},
 {    3411, .8355807617188E+04, -.1401943359375E+04, .9658959350586E+03},
 {    3412, .8365851562500E+04, -.1391596313477E+04, .9613994140625E+03},
 {    3413, .8375942382812E+04, -.1375984741211E+04, .9568811645508E+03},
 {    3414, .8338034179688E+04, -.1408076660156E+04, .9599376220703E+03},
 {    3415, .8349305664062E+04, -.1400077880859E+04, .9548915405273E+03},
 {    3416, .8358945312500E+04, -.1389518310547E+04, .9505769653320E+03},
 {    3417, .8369522460938E+04, -.1373575683594E+04, .9458411865234E+03},
 {    3418, .8339141601562E+04, -.1410308715820E+04, .9594423828125E+03},
 {    3419, .8350618164062E+04, -.1401943359375E+04, .9543040161133E+03},
 {    3420, .8360660156250E+04, -.1391596313477E+04, .9498082885742E+03},
 {    3421, .8370754882812E+04, -.1375984741211E+04, .9452899169922E+03},
 {    3422, .8336736328125E+04, -.1408076660156E+04, .9570397949219E+03},
 {    3423, .8348008789062E+04, -.1400077880859E+04, .9519936523438E+03},
 {    3424, .8357646484375E+04, -.1389518310547E+04, .9476791992188E+03},
 {    3425, .8368225585938E+04, -.1373575683594E+04, .9429432983398E+03},
 {    3426, .8337843750000E+04, -.1410308715820E+04, .9565447998047E+03},
 {    3427, .8349319335938E+04, -.1401943359375E+04, .9514063110352E+03},
 {    3428, .8359364257812E+04, -.1391596313477E+04, .9469100952148E+03},
 {    3429, .8369458007812E+04, -.1375984741211E+04, .9423917846680E+03},
 {    3430, .8331548828125E+04, -.1408076660156E+04, .9454482421875E+03},
 {    3431, .8342818359375E+04, -.1400077880859E+04, .9404022216797E+03},
 {    3432, .8352458007812E+04, -.1389518310547E+04, .9360877075195E+03},
 {    3433, .8363039062500E+04, -.1373575683594E+04, .9313516235352E+03},
 {    3434, .8332654296875E+04, -.1410308715820E+04, .9449530029297E+03},
 {    3435, .8344132812500E+04, -.1401943359375E+04, .9398144531250E+03},
 {    3436, .8354176757812E+04, -.1391596313477E+04, .9353189086914E+03},
 {    3437, .8364268554688E+04, -.1375984741211E+04, .9308005371094E+03},
 {    3438, .8326359375000E+04, -.1408076660156E+04, .9338565673828E+03},
 {    3439, .8337629882812E+04, -.1400077880859E+04, .9288106689453E+03},
 {    3440, .8347267578125E+04, -.1389518310547E+04, .9244961547852E+03},
 {    3441, .8357849609375E+04, -.1373575683594E+04, .9197601928711E+03},
 {    3442, .8327464843750E+04, -.1410308715820E+04, .9333614501953E+03},
 {    3443, .8338943359375E+04, -.1401943359375E+04, .9282229614258E+03},
 {    3444, .8348985351562E+04, -.1391596313477E+04, .9237270507812E+03},
 {    3445, .8359079101562E+04, -.1375984741211E+04, .9192086791992E+03},
 {    3446, .8251616210938E+04, -.1373575683594E+04, .1120375976562E+04},
 {    3447, .8260720703125E+04, -.1387901855469E+04, .1116300048828E+04},
 {    3448, .8273143554688E+04, -.1401514648438E+04, .1110738403320E+04},
 {    3449, .8283105468750E+04, -.1408076660156E+04, .1106278564453E+04},
 {    3450, .8249857421875E+04, -.1375029052734E+04, .1121162475586E+04},
 {    3451, .8259072265625E+04, -.1389839477539E+04, .1117037597656E+04},
 {    3452, .8271983398438E+04, -.1403186767578E+04, .1111258056641E+04},
 {    3453, .8282237304688E+04, -.1409924194336E+04, .1106667724609E+04},
 {    3454, .8246426757812E+04, -.1373575683594E+04, .1108784057617E+04},
 {    3455, .8255530273438E+04, -.1387901855469E+04, .1104708251953E+04},
 {    3456, .8267955078125E+04, -.1401514648438E+04, .1099146484375E+04},
 {    3457, .8277916992188E+04, -.1408076660156E+04, .1094687500000E+04},
 {    3458, .8244669921875E+04, -.1375029052734E+04, .1109570678711E+04},
 {    3459, .8253881835938E+04, -.1389839477539E+04, .1105445800781E+04},
 {    3460, .8266795898438E+04, -.1403186767578E+04, .1099666015625E+04},
 {    3461, .8277047851562E+04, -.1409924194336E+04, .1095076293945E+04},
 {    3462, .8241239257812E+04, -.1373575683594E+04, .1097191772461E+04},
 {    3463, .8250342773438E+04, -.1387901855469E+04, .1093116943359E+04},
 {    3464, .8262765625000E+04, -.1401514648438E+04, .1087555175781E+04},
 {    3465, .8272727539062E+04, -.1408076660156E+04, .1083095947266E+04},
 {    3466, .8239479492188E+04, -.1375029052734E+04, .1097979003906E+04},
 {    3467, .8248697265625E+04, -.1389839477539E+04, .1093853881836E+04},
 {    3468, .8261606445312E+04, -.1403186767578E+04, .1088074462891E+04},
 {    3469, .8271857421875E+04, -.1409924194336E+04, .1083484375000E+04},
 {    3470, .8239940429688E+04, -.1373575683594E+04, .1094294677734E+04},
 {    3471, .8249044921875E+04, -.1387901855469E+04, .1090219360352E+04},
 {    3472, .8261469726562E+04, -.1401514648438E+04, .1084656982422E+04},
 {    3473, .8271429687500E+04, -.1408076660156E+04, .1080197875977E+04},
 {    3474, .8238182617188E+04, -.1375029052734E+04, .1095081542969E+04},
 {    3475, .8247399414062E+04, -.1389839477539E+04, .1090955932617E+04},
 {    3476, .8260309570312E+04, -.1403186767578E+04, .1085176269531E+04},
 {    3477, .8270560546875E+04, -.1409924194336E+04, .1080586669922E+04},
 {    3478, .8234751953125E+04, -.1373575683594E+04, .1082702880859E+04},
 {    3479, .8243856445312E+04, -.1387901855469E+04, .1078627441406E+04},
 {    3480, .8256280273438E+04, -.1401514648438E+04, .1073065429688E+04},
 {    3481, .8266241210938E+04, -.1408076660156E+04, .1068606323242E+04},
 {    3482, .8232994140625E+04, -.1375029052734E+04, .1083489624023E+04},
 {    3483, .8242206054688E+04, -.1389839477539E+04, .1079364624023E+04},
 {    3484, .8255119140625E+04, -.1403186767578E+04, .1073584960938E+04},
 {    3485, .8265372070312E+04, -.1409924194336E+04, .1068995361328E+04},
 {    3486, .8229562500000E+04, -.1373575683594E+04, .1071111083984E+04},
 {    3487, .8238665039062E+04, -.1387901855469E+04, .1067035644531E+04},
 {    3488, .8251092773438E+04, -.1401514648438E+04, .1061473754883E+04},
 {    3489, .8261052734375E+04, -.1408076660156E+04, .1057014770508E+04},
 {    3490, .8227803710938E+04, -.1375029052734E+04, .1071897705078E+04},
 {    3491, .8237017578125E+04, -.1389839477539E+04, .1067773803711E+04},
 {    3492, .8249930664062E+04, -.1403186767578E+04, .1061993408203E+04},
 {    3493, .8260183593750E+04, -.1409924194336E+04, .1057403442383E+04},
 {    3494, .8228262695312E+04, -.1373575683594E+04, .1068213745117E+04},
 {    3495, .8237367187500E+04, -.1387901855469E+04, .1064138183594E+04},
 {    3496, .8249794921875E+04, -.1401514648438E+04, .1058575927734E+04},
 {    3497, .8259752929688E+04, -.1408076660156E+04, .1054117675781E+04},
 {    3498, .8226509765625E+04, -.1375029052734E+04, .1069000610352E+04},
 {    3499, .8235722656250E+04, -.1389839477539E+04, .1064875000000E+04},
 {    3500, .8248633789062E+04, -.1403186767578E+04, .1059095458984E+04},
 {    3501, .8258886718750E+04, -.1409924194336E+04, .1054505371094E+04},
 {    3502, .8223076171875E+04, -.1373575683594E+04, .1056622314453E+04},
 {    3503, .8232178710938E+04, -.1387901855469E+04, .1052546630859E+04},
 {    3504, .8244603515625E+04, -.1401514648438E+04, .1046984497070E+04},
 {    3505, .8254567382812E+04, -.1408076660156E+04, .1042525634766E+04},
 {    3506, .8221319335938E+04, -.1375029052734E+04, .1057408935547E+04},
 {    3507, .8230530273438E+04, -.1389839477539E+04, .1053283935547E+04},
 {    3508, .8243444335938E+04, -.1403186767578E+04, .1047503906250E+04},
 {    3509, .8253695312500E+04, -.1409924194336E+04, .1042914794922E+04},
 {    3510, .8217887695312E+04, -.1373575683594E+04, .1045030639648E+04},
 {    3511, .8226990234375E+04, -.1387901855469E+04, .1040955322266E+04},
 {    3512, .8239416015625E+04, -.1401514648438E+04, .1035392456055E+04},
 {    3513, .8249376953125E+04, -.1408076660156E+04, .1030933715820E+04},
 {    3514, .8216129882812E+04, -.1375029052734E+04, .1045817260742E+04},
 {    3515, .8225341796875E+04, -.1389839477539E+04, .1041692260742E+04},
 {    3516, .8238255859375E+04, -.1403186767578E+04, .1035912841797E+04},
 {    3517, .8248505859375E+04, -.1409924194336E+04, .1031323242188E+04},
 {    3518, .8216587890625E+04, -.1373575683594E+04, .1042132934570E+04},
 {    3519, .8225691406250E+04, -.1387901855469E+04, .1038057495117E+04},
 {    3520, .8238119140625E+04, -.1401514648438E+04, .1032494873047E+04},
 {    3521, .8248080078125E+04, -.1408076660156E+04, .1028035766602E+04},
 {    3522, .8214831054688E+04, -.1375029052734E+04, .1042919433594E+04},
 {    3523, .8224047851562E+04, -.1389839477539E+04, .1038794189453E+04},
 {    3524, .8236958984375E+04, -.1403186767578E+04, .1033014404297E+04},
 {    3525, .8247210937500E+04, -.1409924194336E+04, .1028424560547E+04},
 {    3526, .8211399414062E+04, -.1373575683594E+04, .1030541015625E+04},
 {    3527, .8220503906250E+04, -.1387901855469E+04, .1026465820312E+04},
 {    3528, .8232926757812E+04, -.1401514648438E+04, .1020904113770E+04},
 {    3529, .8242891601562E+04, -.1408076660156E+04, .1016444213867E+04},
 {    3530, .8209641601562E+04, -.1375029052734E+04, .1031328125000E+04},
 {    3531, .8218858398438E+04, -.1389839477539E+04, .1027202636719E+04},
 {    3532, .8231766601562E+04, -.1403186767578E+04, .1021423583984E+04},
 {    3533, .8242021484375E+04, -.1409924194336E+04, .1016833374023E+04},
 {    3534, .8206210937500E+04, -.1373575683594E+04, .1018949707031E+04},
 {    3535, .8215314453125E+04, -.1387901855469E+04, .1014873901367E+04},
 {    3536, .8227737304688E+04, -.1401514648438E+04, .1009312683105E+04},
 {    3537, .8237700195312E+04, -.1408076660156E+04, .1004853393555E+04},
 {    3538, .8204453125000E+04, -.1375029052734E+04, .1019736328125E+04},
 {    3539, .8213670898438E+04, -.1389839477539E+04, .1015610961914E+04},
 {    3540, .8226579101562E+04, -.1403186767578E+04, .1009831542969E+04},
 {    3541, .8236829101562E+04, -.1409924194336E+04, .1005242065430E+04},
 {    3542, .8204914062500E+04, -.1373575683594E+04, .1016051696777E+04},
 {    3543, .8214018554688E+04, -.1387901855469E+04, .1011976379395E+04},
 {    3544, .8226443359375E+04, -.1401514648438E+04, .1006413940430E+04},
 {    3545, .8236402343750E+04, -.1408076660156E+04, .1001955200195E+04},
 {    3546, .8203156250000E+04, -.1375029052734E+04, .1016838378906E+04},
 {    3547, .8212370117188E+04, -.1389839477539E+04, .1012713623047E+04},
 {    3548, .8225283203125E+04, -.1403186767578E+04, .1006933715820E+04},
 {    3549, .8235535156250E+04, -.1409924194336E+04, .1002343627930E+04},
 {    3550, .8199724609375E+04, -.1373575683594E+04, .1004460632324E+04},
 {    3551, .8208828125000E+04, -.1387901855469E+04, .1000385009766E+04},
 {    3552, .8221254882812E+04, -.1401514648438E+04, .9948225097656E+03},
 {    3553, .8231215820312E+04, -.1408076660156E+04, .9903632812500E+03},
 {    3554, .8197966796875E+04, -.1375029052734E+04, .1005246948242E+04},
 {    3555, .8207180664062E+04, -.1389839477539E+04, .1001122314453E+04},
 {    3556, .8220092773438E+04, -.1403186767578E+04, .9953419799805E+03},
 {    3557, .8230346679688E+04, -.1409924194336E+04, .9907523193359E+03},
 {    3558, .8194537109375E+04, -.1373575683594E+04, .9928685302734E+03},
 {    3559, .8203639648438E+04, -.1387901855469E+04, .9887930297852E+03},
 {    3560, .8216064453125E+04, -.1401514648438E+04, .9832310791016E+03},
 {    3561, .8226023437500E+04, -.1408076660156E+04, .9787721557617E+03},
 {    3562, .8192779296875E+04, -.1375029052734E+04, .9936555175781E+03},
 {    3563, .8201994140625E+04, -.1389839477539E+04, .9895301513672E+03},
 {    3564, .8214904296875E+04, -.1403186767578E+04, .9837504882812E+03},
 {    3565, .8225159179688E+04, -.1409924194336E+04, .9791605834961E+03},
 {    3566, .8283105468750E+04, -.1163107421875E+04, .1106278564453E+04},
 {    3567, .8273306640625E+04, -.1169489868164E+04, .1110664794922E+04},
 {    3568, .8260720703125E+04, -.1183282226562E+04, .1116299804688E+04},
 {    3569, .8251616210938E+04, -.1197608398438E+04, .1120375976562E+04},
 {    3570, .8282037109375E+04, -.1161390625000E+04, .1106756591797E+04},
 {    3571, .8271983398438E+04, -.1167997314453E+04, .1111258056641E+04},
 {    3572, .8259072265625E+04, -.1181344604492E+04, .1117037597656E+04},
 {    3573, .8249857421875E+04, -.1196155029297E+04, .1121162475586E+04},
 {    3574, .8277916992188E+04, -.1163107421875E+04, .1094687500000E+04},
 {    3575, .8268120117188E+04, -.1169489868164E+04, .1099072753906E+04},
 {    3576, .8255530273438E+04, -.1183282226562E+04, .1104708251953E+04},
 {    3577, .8246426757812E+04, -.1197608398438E+04, .1108784057617E+04},
 {    3578, .8276848632812E+04, -.1161390625000E+04, .1095165283203E+04},
 {    3579, .8266795898438E+04, -.1167997314453E+04, .1099665771484E+04},
 {    3580, .8253881835938E+04, -.1181344604492E+04, .1105445800781E+04},
 {    3581, .8244669921875E+04, -.1196155029297E+04, .1109570678711E+04},
 {    3582, .8272727539062E+04, -.1163107421875E+04, .1083095947266E+04},
 {    3583, .8262929687500E+04, -.1169489868164E+04, .1087481689453E+04},
 {    3584, .8250341796875E+04, -.1183282226562E+04, .1093116943359E+04},
 {    3585, .8241239257812E+04, -.1197608398438E+04, .1097191772461E+04},
 {    3586, .8271660156250E+04, -.1161390625000E+04, .1083572753906E+04},
 {    3587, .8261606445312E+04, -.1167997314453E+04, .1088074462891E+04},
 {    3588, .8248697265625E+04, -.1181344604492E+04, .1093853637695E+04},
 {    3589, .8239479492188E+04, -.1196155029297E+04, .1097979003906E+04},
 {    3590, .8271429687500E+04, -.1163107421875E+04, .1080197875977E+04},
 {    3591, .8261630859375E+04, -.1169489868164E+04, .1084583862305E+04},
 {    3592, .8249044921875E+04, -.1183282226562E+04, .1090219360352E+04},
 {    3593, .8239940429688E+04, -.1197608398438E+04, .1094294677734E+04},
 {    3594, .8270363281250E+04, -.1161390625000E+04, .1080675781250E+04},
 {    3595, .8260309570312E+04, -.1167997314453E+04, .1085176269531E+04},
 {    3596, .8247399414062E+04, -.1181344604492E+04, .1090955932617E+04},
 {    3597, .8238182617188E+04, -.1196155029297E+04, .1095081542969E+04},
 {    3598, .8266241210938E+04, -.1163107421875E+04, .1068606323242E+04},
 {    3599, .8256441406250E+04, -.1169489868164E+04, .1072992431641E+04},
 {    3600, .8243856445312E+04, -.1183282226562E+04, .1078627441406E+04},
 {    3601, .8234751953125E+04, -.1197608398438E+04, .1082702880859E+04},
 {    3602, .8265173828125E+04, -.1161390625000E+04, .1069084472656E+04},
 {    3603, .8255119140625E+04, -.1167997314453E+04, .1073584960938E+04},
 {    3604, .8242206054688E+04, -.1181344604492E+04, .1079364624023E+04},
 {    3605, .8232994140625E+04, -.1196155029297E+04, .1083489624023E+04},
 {    3606, .8261052734375E+04, -.1163107421875E+04, .1057014770508E+04},
 {    3607, .8251253906250E+04, -.1169489868164E+04, .1061400756836E+04},
 {    3608, .8238665039062E+04, -.1183282226562E+04, .1067035644531E+04},
 {    3609, .8229562500000E+04, -.1197608398438E+04, .1071111083984E+04},
 {    3610, .8259985351562E+04, -.1161390625000E+04, .1057492675781E+04},
 {    3611, .8249930664062E+04, -.1167997314453E+04, .1061993408203E+04},
 {    3612, .8237017578125E+04, -.1181344604492E+04, .1067773803711E+04},
 {    3613, .8227803710938E+04, -.1196155029297E+04, .1071897705078E+04},
 {    3614, .8259752929688E+04, -.1163107421875E+04, .1054117675781E+04},
 {    3615, .8249957031250E+04, -.1169489868164E+04, .1058503173828E+04},
 {    3616, .8237367187500E+04, -.1183282226562E+04, .1064138183594E+04},
 {    3617, .8228262695312E+04, -.1197608398438E+04, .1068213745117E+04},
 {    3618, .8258687500000E+04, -.1161390625000E+04, .1054594482422E+04},
 {    3619, .8248633789062E+04, -.1167997314453E+04, .1059095214844E+04},
 {    3620, .8235722656250E+04, -.1181344604492E+04, .1064875000000E+04},
 {    3621, .8226509765625E+04, -.1196155029297E+04, .1069000610352E+04},
 {    3622, .8254567382812E+04, -.1163107421875E+04, .1042525634766E+04},
 {    3623, .8244766601562E+04, -.1169489868164E+04, .1046911376953E+04},
 {    3624, .8232178710938E+04, -.1183282226562E+04, .1052546630859E+04},
 {    3625, .8223076171875E+04, -.1197608398438E+04, .1056622314453E+04},
 {    3626, .8253498046875E+04, -.1161390625000E+04, .1043003173828E+04},
 {    3627, .8243443359375E+04, -.1167997314453E+04, .1047503662109E+04},
 {    3628, .8230530273438E+04, -.1181344604492E+04, .1053283935547E+04},
 {    3629, .8221319335938E+04, -.1196155029297E+04, .1057408691406E+04},
 {    3630, .8249376953125E+04, -.1163107421875E+04, .1030933715820E+04},
 {    3631, .8239580078125E+04, -.1169489868164E+04, .1035319335938E+04},
 {    3632, .8226990234375E+04, -.1183282226562E+04, .1040955322266E+04},
 {    3633, .8217887695312E+04, -.1197608398438E+04, .1045030639648E+04},
 {    3634, .8248308593750E+04, -.1161390625000E+04, .1031411376953E+04},
 {    3635, .8238255859375E+04, -.1167997314453E+04, .1035912597656E+04},
 {    3636, .8225341796875E+04, -.1181344604492E+04, .1041692260742E+04},
 {    3637, .8216129882812E+04, -.1196155029297E+04, .1045817260742E+04},
 {    3638, .8248080078125E+04, -.1163107421875E+04, .1028035766602E+04},
 {    3639, .8238281250000E+04, -.1169489868164E+04, .1032421875000E+04},
 {    3640, .8225690429688E+04, -.1183282226562E+04, .1038057861328E+04},
 {    3641, .8216587890625E+04, -.1197608398438E+04, .1042132934570E+04},
 {    3642, .8247010742188E+04, -.1161390625000E+04, .1028513671875E+04},
 {    3643, .8236958984375E+04, -.1167997314453E+04, .1033014404297E+04},
 {    3644, .8224047851562E+04, -.1181344604492E+04, .1038794189453E+04},
 {    3645, .8214831054688E+04, -.1196155029297E+04, .1042919433594E+04},
 {    3646, .8242891601562E+04, -.1163107421875E+04, .1016444213867E+04},
 {    3647, .8233091796875E+04, -.1169489868164E+04, .1020830566406E+04},
 {    3648, .8220503906250E+04, -.1183282226562E+04, .1026465820312E+04},
 {    3649, .8211399414062E+04, -.1197608398438E+04, .1030541015625E+04},
 {    3650, .8241822265625E+04, -.1161390625000E+04, .1016922241211E+04},
 {    3651, .8231766601562E+04, -.1167997314453E+04, .1021423583984E+04},
 {    3652, .8218858398438E+04, -.1181344604492E+04, .1027202636719E+04},
 {    3653, .8209641601562E+04, -.1196155029297E+04, .1031328125000E+04},
 {    3654, .8237700195312E+04, -.1163107421875E+04, .1004853393555E+04},
 {    3655, .8227902343750E+04, -.1169489868164E+04, .1009238891602E+04},
 {    3656, .8215314453125E+04, -.1183282226562E+04, .1014873901367E+04},
 {    3657, .8206210937500E+04, -.1197608398438E+04, .1018949707031E+04},
 {    3658, .8236632812500E+04, -.1161390625000E+04, .1005330688477E+04},
 {    3659, .8226579101562E+04, -.1167997314453E+04, .1009831298828E+04},
 {    3660, .8213670898438E+04, -.1181344604492E+04, .1015610961914E+04},
 {    3661, .8204453125000E+04, -.1196155029297E+04, .1019736328125E+04},
 {    3662, .8236402343750E+04, -.1163107421875E+04, .1001955200195E+04},
 {    3663, .8226605468750E+04, -.1169489868164E+04, .1006340820312E+04},
 {    3664, .8214018554688E+04, -.1183282226562E+04, .1011976379395E+04},
 {    3665, .8204914062500E+04, -.1197608398438E+04, .1016051696777E+04},
 {    3666, .8235334960938E+04, -.1161390625000E+04, .1002432739258E+04},
 {    3667, .8225283203125E+04, -.1167997314453E+04, .1006933715820E+04},
 {    3668, .8212370117188E+04, -.1181344604492E+04, .1012713623047E+04},
 {    3669, .8203156250000E+04, -.1196155029297E+04, .1016838378906E+04},
 {    3670, .8231215820312E+04, -.1163107421875E+04, .9903632812500E+03},
 {    3671, .8221416992188E+04, -.1169489868164E+04, .9947493896484E+03},
 {    3672, .8208828125000E+04, -.1183282226562E+04, .1000384765625E+04},
 {    3673, .8199724609375E+04, -.1197608398438E+04, .1004460632324E+04},
 {    3674, .8230148437500E+04, -.1161390625000E+04, .9908409423828E+03},
 {    3675, .8220092773438E+04, -.1167997314453E+04, .9953419799805E+03},
 {    3676, .8207181640625E+04, -.1181344604492E+04, .1001121643066E+04},
 {    3677, .8197966796875E+04, -.1196155029297E+04, .1005246948242E+04},
 {    3678, .8226023437500E+04, -.1163107421875E+04, .9787721557617E+03},
 {    3679, .8216228515625E+04, -.1169489868164E+04, .9831574707031E+03},
 {    3680, .8203639648438E+04, -.1183282226562E+04, .9887927856445E+03},
 {    3681, .8194537109375E+04, -.1197608398438E+04, .9928685302734E+03},
 {    3682, .8224958007812E+04, -.1161390625000E+04, .9792498168945E+03},
 {    3683, .8214904296875E+04, -.1167997314453E+04, .9837504882812E+03},
 {    3684, .8201994140625E+04, -.1181344604492E+04, .9895301513672E+03},
 {    3685, .8192779296875E+04, -.1196155029297E+04, .9936555175781E+03},
 {    3686, .8245729492188E+04, -.1336392089844E+04, .1136925170898E+04},
 {    3687, .8248625000000E+04, -.1336392089844E+04, .1135628295898E+04},
 {    3688, .8245729492188E+04, -.1323692016602E+04, .1136925170898E+04},
 {    3689, .8248625000000E+04, -.1323692016602E+04, .1135628295898E+04},
 {    3690, .8240538085938E+04, -.1336392089844E+04, .1125334472656E+04},
 {    3691, .8243438476562E+04, -.1336392089844E+04, .1124036499023E+04},
 {    3692, .8240538085938E+04, -.1323692016602E+04, .1125334472656E+04},
 {    3693, .8243438476562E+04, -.1323692016602E+04, .1124036499023E+04},
 {    3694, .8235350585938E+04, -.1336392089844E+04, .1113742431641E+04},
 {    3695, .8238247070312E+04, -.1336392089844E+04, .1112445312500E+04},
 {    3696, .8235350585938E+04, -.1323692016602E+04, .1113742431641E+04},
 {    3697, .8238247070312E+04, -.1323692016602E+04, .1112445312500E+04},
 {    3698, .8230163085938E+04, -.1336392089844E+04, .1102150634766E+04},
 {    3699, .8233059570312E+04, -.1336392089844E+04, .1100853637695E+04},
 {    3700, .8230163085938E+04, -.1323692016602E+04, .1102150634766E+04},
 {    3701, .8233059570312E+04, -.1323692016602E+04, .1100853637695E+04},
 {    3702, .8228862304688E+04, -.1336392089844E+04, .1099253173828E+04},
 {    3703, .8231761718750E+04, -.1336392089844E+04, .1097955688477E+04},
 {    3704, .8228862304688E+04, -.1323692016602E+04, .1099253173828E+04},
 {    3705, .8231761718750E+04, -.1323692016602E+04, .1097955688477E+04},
 {    3706, .8223675781250E+04, -.1336392089844E+04, .1087661376953E+04},
 {    3707, .8226575195312E+04, -.1336392089844E+04, .1086364501953E+04},
 {    3708, .8223675781250E+04, -.1323692016602E+04, .1087661376953E+04},
 {    3709, .8226575195312E+04, -.1323692016602E+04, .1086364501953E+04},
 {    3710, .8218486328125E+04, -.1336392089844E+04, .1076069580078E+04},
 {    3711, .8221383789062E+04, -.1336392089844E+04, .1074772827148E+04},
 {    3712, .8218486328125E+04, -.1323692016602E+04, .1076069580078E+04},
 {    3713, .8221383789062E+04, -.1323692016602E+04, .1074772827148E+04},
 {    3714, .8217188476562E+04, -.1336392089844E+04, .1073171875000E+04},
 {    3715, .8220084960938E+04, -.1336392089844E+04, .1071875122070E+04},
 {    3716, .8217188476562E+04, -.1323692016602E+04, .1073171875000E+04},
 {    3717, .8220084960938E+04, -.1323692016602E+04, .1071875122070E+04},
 {    3718, .8212000000000E+04, -.1336392089844E+04, .1061580444336E+04},
 {    3719, .8214895507812E+04, -.1336392089844E+04, .1060283203125E+04},
 {    3720, .8212000000000E+04, -.1323692016602E+04, .1061580444336E+04},
 {    3721, .8214895507812E+04, -.1323692016602E+04, .1060283203125E+04},
 {    3722, .8206809570312E+04, -.1336392089844E+04, .1049989257812E+04},
 {    3723, .8209708984375E+04, -.1336392089844E+04, .1048691894531E+04},
 {    3724, .8206809570312E+04, -.1323692016602E+04, .1049989257812E+04},
 {    3725, .8209708984375E+04, -.1323692016602E+04, .1048691894531E+04},
 {    3726, .8205513671875E+04, -.1336392089844E+04, .1047091064453E+04},
 {    3727, .8208410156250E+04, -.1336392089844E+04, .1045794067383E+04},
 {    3728, .8205513671875E+04, -.1323692016602E+04, .1047091064453E+04},
 {    3729, .8208410156250E+04, -.1323692016602E+04, .1045794067383E+04},
 {    3730, .8200321289062E+04, -.1336392089844E+04, .1035499633789E+04},
 {    3731, .8203223632812E+04, -.1336392089844E+04, .1034202270508E+04},
 {    3732, .8200321289062E+04, -.1323692016602E+04, .1035499633789E+04},
 {    3733, .8203223632812E+04, -.1323692016602E+04, .1034202270508E+04},
 {    3734, .8195135742188E+04, -.1336392089844E+04, .1023908081055E+04},
 {    3735, .8198030273438E+04, -.1336392089844E+04, .1022611267090E+04},
 {    3736, .8195135742188E+04, -.1323692016602E+04, .1023908081055E+04},
 {    3737, .8198030273438E+04, -.1323692016602E+04, .1022611267090E+04},
 {    3738, .8193837890625E+04, -.1336392089844E+04, .1021010009766E+04},
 {    3739, .8196735351562E+04, -.1336392089844E+04, .1019712768555E+04},
 {    3740, .8193837890625E+04, -.1323692016602E+04, .1021010009766E+04},
 {    3741, .8196735351562E+04, -.1323692016602E+04, .1019712768555E+04},
 {    3742, .8188649414062E+04, -.1336392089844E+04, .1009418762207E+04},
 {    3743, .8191546875000E+04, -.1336392089844E+04, .1008121154785E+04},
 {    3744, .8188649414062E+04, -.1323692016602E+04, .1009418762207E+04},
 {    3745, .8191546875000E+04, -.1323692016602E+04, .1008121154785E+04},
 {    3746, .8183458007812E+04, -.1336392089844E+04, .9978272094727E+03},
 {    3747, .8186355957031E+04, -.1336392089844E+04, .9965302734375E+03},
 {    3748, .8183458007812E+04, -.1323692016602E+04, .9978272094727E+03},
 {    3749, .8186355957031E+04, -.1323692016602E+04, .9965302734375E+03},
 {    3750, .8178270019531E+04, -.1336392089844E+04, .9862356567383E+03},
 {    3751, .8181168945312E+04, -.1336392089844E+04, .9849380493164E+03},
 {    3752, .8178270019531E+04, -.1323692016602E+04, .9862356567383E+03},
 {    3753, .8181168945312E+04, -.1323692016602E+04, .9849380493164E+03},
 {    3754, .8245729492188E+04, -.1305911987305E+04, .1136925170898E+04},
 {    3755, .8248625000000E+04, -.1305911987305E+04, .1135628295898E+04},
 {    3756, .8245729492188E+04, -.1288132080078E+04, .1136925170898E+04},
 {    3757, .8248625000000E+04, -.1288132080078E+04, .1135628295898E+04},
 {    3758, .8240538085938E+04, -.1305911987305E+04, .1125334472656E+04},
 {    3759, .8243438476562E+04, -.1305911987305E+04, .1124036499023E+04},
 {    3760, .8240538085938E+04, -.1288132080078E+04, .1125334472656E+04},
 {    3761, .8243438476562E+04, -.1288132080078E+04, .1124036499023E+04},
 {    3762, .8235350585938E+04, -.1305911987305E+04, .1113742431641E+04},
 {    3763, .8238247070312E+04, -.1305911987305E+04, .1112445312500E+04},
 {    3764, .8235350585938E+04, -.1288132080078E+04, .1113742431641E+04},
 {    3765, .8238247070312E+04, -.1288132080078E+04, .1112445312500E+04},
 {    3766, .8230163085938E+04, -.1305911987305E+04, .1102150634766E+04},
 {    3767, .8233059570312E+04, -.1305911987305E+04, .1100853637695E+04},
 {    3768, .8230163085938E+04, -.1288132080078E+04, .1102150634766E+04},
 {    3769, .8233059570312E+04, -.1288132080078E+04, .1100853637695E+04},
 {    3770, .8228862304688E+04, -.1305911987305E+04, .1099253173828E+04},
 {    3771, .8231761718750E+04, -.1305911987305E+04, .1097955688477E+04},
 {    3772, .8228862304688E+04, -.1288132080078E+04, .1099253173828E+04},
 {    3773, .8231761718750E+04, -.1288132080078E+04, .1097955688477E+04},
 {    3774, .8223675781250E+04, -.1305911987305E+04, .1087661376953E+04},
 {    3775, .8226575195312E+04, -.1305911987305E+04, .1086364501953E+04},
 {    3776, .8223675781250E+04, -.1288132080078E+04, .1087661376953E+04},
 {    3777, .8226575195312E+04, -.1288132080078E+04, .1086364501953E+04},
 {    3778, .8218486328125E+04, -.1305911987305E+04, .1076069580078E+04},
 {    3779, .8221383789062E+04, -.1305911987305E+04, .1074772827148E+04},
 {    3780, .8218486328125E+04, -.1288132080078E+04, .1076069580078E+04},
 {    3781, .8221383789062E+04, -.1288132080078E+04, .1074772827148E+04},
 {    3782, .8217188476562E+04, -.1305911987305E+04, .1073171875000E+04},
 {    3783, .8220084960938E+04, -.1305911987305E+04, .1071875122070E+04},
 {    3784, .8217188476562E+04, -.1288132080078E+04, .1073171875000E+04},
 {    3785, .8220084960938E+04, -.1288132080078E+04, .1071875122070E+04},
 {    3786, .8212000000000E+04, -.1305911987305E+04, .1061580444336E+04},
 {    3787, .8214895507812E+04, -.1305911987305E+04, .1060283203125E+04},
 {    3788, .8212000000000E+04, -.1288132080078E+04, .1061580444336E+04},
 {    3789, .8214895507812E+04, -.1288132080078E+04, .1060283203125E+04},
 {    3790, .8206809570312E+04, -.1305911987305E+04, .1049989257812E+04},
 {    3791, .8209708984375E+04, -.1305911987305E+04, .1048691894531E+04},
 {    3792, .8206809570312E+04, -.1288132080078E+04, .1049989257812E+04},
 {    3793, .8209708984375E+04, -.1288132080078E+04, .1048691894531E+04},
 {    3794, .8205513671875E+04, -.1305911987305E+04, .1047091064453E+04},
 {    3795, .8208410156250E+04, -.1305911987305E+04, .1045794067383E+04},
 {    3796, .8205513671875E+04, -.1288132080078E+04, .1047091064453E+04},
 {    3797, .8208410156250E+04, -.1288132080078E+04, .1045794067383E+04},
 {    3798, .8200321289062E+04, -.1305912109375E+04, .1035499633789E+04},
 {    3799, .8203223632812E+04, -.1305912109375E+04, .1034202270508E+04},
 {    3800, .8200321289062E+04, -.1288132080078E+04, .1035499633789E+04},
 {    3801, .8203223632812E+04, -.1288132080078E+04, .1034202270508E+04},
 {    3802, .8195135742188E+04, -.1305912109375E+04, .1023908081055E+04},
 {    3803, .8198030273438E+04, -.1305912109375E+04, .1022611267090E+04},
 {    3804, .8195135742188E+04, -.1288132080078E+04, .1023908081055E+04},
 {    3805, .8198030273438E+04, -.1288132080078E+04, .1022611267090E+04},
 {    3806, .8193837890625E+04, -.1305912109375E+04, .1021010009766E+04},
 {    3807, .8196735351562E+04, -.1305912109375E+04, .1019712768555E+04},
 {    3808, .8193837890625E+04, -.1288132080078E+04, .1021010009766E+04},
 {    3809, .8196735351562E+04, -.1288132080078E+04, .1019712768555E+04},
 {    3810, .8188649414062E+04, -.1305912109375E+04, .1009418762207E+04},
 {    3811, .8191546875000E+04, -.1305912109375E+04, .1008121154785E+04},
 {    3812, .8188649414062E+04, -.1288132080078E+04, .1009418762207E+04},
 {    3813, .8191546875000E+04, -.1288132080078E+04, .1008121154785E+04},
 {    3814, .8183458007812E+04, -.1305912109375E+04, .9978272094727E+03},
 {    3815, .8186355957031E+04, -.1305912109375E+04, .9965302734375E+03},
 {    3816, .8183458007812E+04, -.1288132080078E+04, .9978272094727E+03},
 {    3817, .8186355957031E+04, -.1288132080078E+04, .9965302734375E+03},
 {    3818, .8178270019531E+04, -.1305912109375E+04, .9862356567383E+03},
 {    3819, .8181168945312E+04, -.1305912109375E+04, .9849380493164E+03},
 {    3820, .8178270019531E+04, -.1288132080078E+04, .9862356567383E+03},
 {    3821, .8181168945312E+04, -.1288132080078E+04, .9849380493164E+03},
 {    3822, .8245729492188E+04, -.1283052001953E+04, .1136925170898E+04},
 {    3823, .8248625000000E+04, -.1283052001953E+04, .1135628295898E+04},
 {    3824, .8240538085938E+04, -.1283052001953E+04, .1125334472656E+04},
 {    3825, .8243438476562E+04, -.1283052001953E+04, .1124036499023E+04},
 {    3826, .8235350585938E+04, -.1283052001953E+04, .1113742431641E+04},
 {    3827, .8238247070312E+04, -.1283052001953E+04, .1112445312500E+04},
 {    3828, .8230163085938E+04, -.1283052001953E+04, .1102150634766E+04},
 {    3829, .8233059570312E+04, -.1283052001953E+04, .1100853637695E+04},
 {    3830, .8228862304688E+04, -.1283052001953E+04, .1099253173828E+04},
 {    3831, .8231761718750E+04, -.1283052001953E+04, .1097955688477E+04},
 {    3832, .8223675781250E+04, -.1283052001953E+04, .1087661376953E+04},
 {    3833, .8226575195312E+04, -.1283052001953E+04, .1086364501953E+04},
 {    3834, .8218486328125E+04, -.1283052001953E+04, .1076069580078E+04},
 {    3835, .8221383789062E+04, -.1283052001953E+04, .1074772827148E+04},
 {    3836, .8217188476562E+04, -.1283052001953E+04, .1073171875000E+04},
 {    3837, .8220084960938E+04, -.1283052001953E+04, .1071875122070E+04},
 {    3838, .8212000000000E+04, -.1283052001953E+04, .1061580444336E+04},
 {    3839, .8214895507812E+04, -.1283052001953E+04, .1060283203125E+04},
 {    3840, .8206809570312E+04, -.1283052001953E+04, .1049989257812E+04},
 {    3841, .8209708984375E+04, -.1283052001953E+04, .1048691894531E+04},
 {    3842, .8205513671875E+04, -.1283052001953E+04, .1047091064453E+04},
 {    3843, .8208410156250E+04, -.1283052001953E+04, .1045794067383E+04},
 {    3844, .8200321289062E+04, -.1283052001953E+04, .1035499633789E+04},
 {    3845, .8203223632812E+04, -.1283052001953E+04, .1034202270508E+04},
 {    3846, .8195135742188E+04, -.1283052001953E+04, .1023908081055E+04},
 {    3847, .8198030273438E+04, -.1283052001953E+04, .1022611267090E+04},
 {    3848, .8193837890625E+04, -.1283052001953E+04, .1021010009766E+04},
 {    3849, .8196735351562E+04, -.1283052001953E+04, .1019712768555E+04},
 {    3850, .8188649414062E+04, -.1283052001953E+04, .1009418762207E+04},
 {    3851, .8191546875000E+04, -.1283052001953E+04, .1008121154785E+04},
 {    3852, .8183458007812E+04, -.1283052001953E+04, .9978272094727E+03},
 {    3853, .8186355957031E+04, -.1283052001953E+04, .9965302734375E+03},
 {    3854, .8178270019531E+04, -.1283052001953E+04, .9862356567383E+03},
 {    3855, .8181168945312E+04, -.1283052001953E+04, .9849380493164E+03},
 {    3856, .8245729492188E+04, -.1265272094727E+04, .1136925170898E+04},
 {    3857, .8248625000000E+04, -.1265272094727E+04, .1135628295898E+04},
 {    3858, .8245729492188E+04, -.1247492065430E+04, .1136925170898E+04},
 {    3859, .8248625000000E+04, -.1247492065430E+04, .1135628295898E+04},
 {    3860, .8240538085938E+04, -.1265272094727E+04, .1125334472656E+04},
 {    3861, .8243438476562E+04, -.1265272094727E+04, .1124036499023E+04},
 {    3862, .8240538085938E+04, -.1247492065430E+04, .1125334472656E+04},
 {    3863, .8243438476562E+04, -.1247492065430E+04, .1124036499023E+04},
 {    3864, .8235350585938E+04, -.1265272094727E+04, .1113742431641E+04},
 {    3865, .8238247070312E+04, -.1265272094727E+04, .1112445312500E+04},
 {    3866, .8235350585938E+04, -.1247492065430E+04, .1113742431641E+04},
 {    3867, .8238247070312E+04, -.1247492065430E+04, .1112445312500E+04},
 {    3868, .8230163085938E+04, -.1265272094727E+04, .1102150634766E+04},
 {    3869, .8233059570312E+04, -.1265272094727E+04, .1100853637695E+04},
 {    3870, .8230163085938E+04, -.1247492065430E+04, .1102150634766E+04},
 {    3871, .8233059570312E+04, -.1247492065430E+04, .1100853637695E+04},
 {    3872, .8228862304688E+04, -.1265272094727E+04, .1099253173828E+04},
 {    3873, .8231761718750E+04, -.1265272094727E+04, .1097955688477E+04},
 {    3874, .8228862304688E+04, -.1247492065430E+04, .1099253173828E+04},
 {    3875, .8231761718750E+04, -.1247492065430E+04, .1097955688477E+04},
 {    3876, .8223675781250E+04, -.1265272094727E+04, .1087661376953E+04},
 {    3877, .8226575195312E+04, -.1265272094727E+04, .1086364501953E+04},
 {    3878, .8223675781250E+04, -.1247492065430E+04, .1087661376953E+04},
 {    3879, .8226575195312E+04, -.1247492065430E+04, .1086364501953E+04},
 {    3880, .8218486328125E+04, -.1265272094727E+04, .1076069580078E+04},
 {    3881, .8221383789062E+04, -.1265272094727E+04, .1074772827148E+04},
 {    3882, .8218486328125E+04, -.1247492065430E+04, .1076069580078E+04},
 {    3883, .8221383789062E+04, -.1247492065430E+04, .1074772827148E+04},
 {    3884, .8217188476562E+04, -.1265272094727E+04, .1073171875000E+04},
 {    3885, .8220084960938E+04, -.1265272094727E+04, .1071875122070E+04},
 {    3886, .8217188476562E+04, -.1247492065430E+04, .1073171875000E+04},
 {    3887, .8220084960938E+04, -.1247492065430E+04, .1071875122070E+04},
 {    3888, .8212000000000E+04, -.1265272094727E+04, .1061580444336E+04},
 {    3889, .8214895507812E+04, -.1265272094727E+04, .1060283203125E+04},
 {    3890, .8212000000000E+04, -.1247492065430E+04, .1061580444336E+04},
 {    3891, .8214895507812E+04, -.1247492065430E+04, .1060283203125E+04},
 {    3892, .8206809570312E+04, -.1265272094727E+04, .1049989257812E+04},
 {    3893, .8209708984375E+04, -.1265272094727E+04, .1048691894531E+04},
 {    3894, .8206809570312E+04, -.1247492065430E+04, .1049989257812E+04},
 {    3895, .8209708984375E+04, -.1247492065430E+04, .1048691894531E+04},
 {    3896, .8205513671875E+04, -.1265272094727E+04, .1047091064453E+04},
 {    3897, .8208410156250E+04, -.1265272094727E+04, .1045794067383E+04},
 {    3898, .8205513671875E+04, -.1247492065430E+04, .1047091064453E+04},
 {    3899, .8208410156250E+04, -.1247492065430E+04, .1045794067383E+04},
 {    3900, .8200321289062E+04, -.1265272094727E+04, .1035499633789E+04},
 {    3901, .8203223632812E+04, -.1265272094727E+04, .1034202270508E+04},
 {    3902, .8200321289062E+04, -.1247492065430E+04, .1035499633789E+04},
 {    3903, .8203223632812E+04, -.1247492065430E+04, .1034202270508E+04},
 {    3904, .8195135742188E+04, -.1265272094727E+04, .1023908081055E+04},
 {    3905, .8198030273438E+04, -.1265272094727E+04, .1022611267090E+04},
 {    3906, .8195135742188E+04, -.1247492065430E+04, .1023908081055E+04},
 {    3907, .8198030273438E+04, -.1247492065430E+04, .1022611267090E+04},
 {    3908, .8193837890625E+04, -.1265272094727E+04, .1021010009766E+04},
 {    3909, .8196735351562E+04, -.1265272094727E+04, .1019712768555E+04},
 {    3910, .8193837890625E+04, -.1247492065430E+04, .1021010009766E+04},
 {    3911, .8196735351562E+04, -.1247492065430E+04, .1019712768555E+04},
 {    3912, .8188649414062E+04, -.1265272094727E+04, .1009418762207E+04},
 {    3913, .8191546875000E+04, -.1265272094727E+04, .1008121154785E+04},
 {    3914, .8188649414062E+04, -.1247492065430E+04, .1009418762207E+04},
 {    3915, .8191546875000E+04, -.1247492065430E+04, .1008121154785E+04},
 {    3916, .8183458007812E+04, -.1265272094727E+04, .9978272094727E+03},
 {    3917, .8186355957031E+04, -.1265272094727E+04, .9965302734375E+03},
 {    3918, .8183458007812E+04, -.1247492065430E+04, .9978272094727E+03},
 {    3919, .8186355957031E+04, -.1247492065430E+04, .9965302734375E+03},
 {    3920, .8178270019531E+04, -.1265272094727E+04, .9862356567383E+03},
 {    3921, .8181168945312E+04, -.1265272094727E+04, .9849380493164E+03},
 {    3922, .8178270019531E+04, -.1247492065430E+04, .9862356567383E+03},
 {    3923, .8181168945312E+04, -.1247492065430E+04, .9849380493164E+03},
 {    3924, .8245729492188E+04, -.1234791992188E+04, .1136925170898E+04},
 {    3925, .8248625000000E+04, -.1234791992188E+04, .1135628295898E+04},
 {    3926, .8240538085938E+04, -.1234791992188E+04, .1125334472656E+04},
 {    3927, .8243438476562E+04, -.1234791992188E+04, .1124036499023E+04},
 {    3928, .8235350585938E+04, -.1234791992188E+04, .1113742431641E+04},
 {    3929, .8238247070312E+04, -.1234791992188E+04, .1112445312500E+04},
 {    3930, .8230163085938E+04, -.1234791992188E+04, .1102150634766E+04},
 {    3931, .8233059570312E+04, -.1234791992188E+04, .1100853637695E+04},
 {    3932, .8228862304688E+04, -.1234791992188E+04, .1099253173828E+04},
 {    3933, .8231761718750E+04, -.1234791992188E+04, .1097955688477E+04},
 {    3934, .8223675781250E+04, -.1234791992188E+04, .1087661376953E+04},
 {    3935, .8226575195312E+04, -.1234791992188E+04, .1086364501953E+04},
 {    3936, .8218486328125E+04, -.1234791992188E+04, .1076069580078E+04},
 {    3937, .8221383789062E+04, -.1234791992188E+04, .1074772827148E+04},
 {    3938, .8217188476562E+04, -.1234791992188E+04, .1073171875000E+04},
 {    3939, .8220084960938E+04, -.1234791992188E+04, .1071875122070E+04},
 {    3940, .8212000000000E+04, -.1234791992188E+04, .1061580444336E+04},
 {    3941, .8214895507812E+04, -.1234791992188E+04, .1060283203125E+04},
 {    3942, .8206809570312E+04, -.1234791992188E+04, .1049989257812E+04},
 {    3943, .8209708984375E+04, -.1234791992188E+04, .1048691894531E+04},
 {    3944, .8205513671875E+04, -.1234791992188E+04, .1047091064453E+04},
 {    3945, .8208410156250E+04, -.1234791992188E+04, .1045794067383E+04},
 {    3946, .8200321289062E+04, -.1234791992188E+04, .1035499633789E+04},
 {    3947, .8203223632812E+04, -.1234791992188E+04, .1034202270508E+04},
 {    3948, .8195135742188E+04, -.1234791992188E+04, .1023908081055E+04},
 {    3949, .8198030273438E+04, -.1234791992188E+04, .1022611267090E+04},
 {    3950, .8193837890625E+04, -.1234791992188E+04, .1021010009766E+04},
 {    3951, .8196735351562E+04, -.1234791992188E+04, .1019712768555E+04},
 {    3952, .8188649414062E+04, -.1234791992188E+04, .1009418762207E+04},
 {    3953, .8191546875000E+04, -.1234791992188E+04, .1008121154785E+04},
 {    3954, .8183458007812E+04, -.1234791992188E+04, .9978272094727E+03},
 {    3955, .8186355957031E+04, -.1234791992188E+04, .9965302734375E+03},
 {    3956, .8178270019531E+04, -.1234791992188E+04, .9862356567383E+03},
 {    3957, .8181168945312E+04, -.1234791992188E+04, .9849380493164E+03},
 {    3958, .8290064453125E+04, -.1285592041016E+04, .8738526611328E+03},
 {    3959, .8277605468750E+04, -.1285592041016E+04, .8794305419922E+03},
 {    3960, .8265143554688E+04, -.1285592041016E+04, .8850091552734E+03},
 {    3961, .8291229492188E+04, -.1280829589844E+04, .8733309326172E+03},
 {    3962, .8280437500000E+04, -.1274003295898E+04, .8781619873047E+03},
 {    3963, .8269646484375E+04, -.1267177001953E+04, .8829931030273E+03},
 {    3964, .8294411132812E+04, -.1277343139648E+04, .8719064941406E+03},
 {    3965, .8288180664062E+04, -.1265519775391E+04, .8746956787109E+03},
 {    3966, .8281950195312E+04, -.1253696289062E+04, .8774849853516E+03},
 {    3967, .8298756835938E+04, -.1276067016602E+04, .8699608764648E+03},
 {    3968, .8298756835938E+04, -.1262414550781E+04, .8699608764648E+03},
 {    3969, .8298756835938E+04, -.1248762084961E+04, .8699608764648E+03},
 {    3970, .8303106445312E+04, -.1277343139648E+04, .8680151367188E+03},
 {    3971, .8309336914062E+04, -.1265519775391E+04, .8652255859375E+03},
 {    3972, .8315567382812E+04, -.1253696289062E+04, .8624365844727E+03},
 {    3973, .8306288085938E+04, -.1280829589844E+04, .8665902709961E+03},
 {    3974, .8317078125000E+04, -.1274003295898E+04, .8617597656250E+03},
 {    3975, .8327869140625E+04, -.1267177001953E+04, .8569287719727E+03},
 {    3976, .8307453125000E+04, -.1285592041016E+04, .8660687255859E+03},
 {    3977, .8319914062500E+04, -.1285592041016E+04, .8604906616211E+03},
 {    3978, .8332374023438E+04, -.1285592041016E+04, .8549126586914E+03},
 {    3979, .8306288085938E+04, -.1290354614258E+04, .8665902709961E+03},
 {    3980, .8317078125000E+04, -.1297180786133E+04, .8617597656250E+03},
 {    3981, .8327869140625E+04, -.1304007080078E+04, .8569287719727E+03},
 {    3982, .8303106445312E+04, -.1293840942383E+04, .8680151367188E+03},
 {    3983, .8309336914062E+04, -.1305664306641E+04, .8652255859375E+03},
 {    3984, .8315567382812E+04, -.1317487792969E+04, .8624367065430E+03},
 {    3985, .8298756835938E+04, -.1295117065430E+04, .8699608764648E+03},
 {    3986, .8298756835938E+04, -.1308769531250E+04, .8699608764648E+03},
 {    3987, .8298756835938E+04, -.1322421997070E+04, .8699608764648E+03},
 {    3988, .8294411132812E+04, -.1293840942383E+04, .8719064941406E+03},
 {    3989, .8288180664062E+04, -.1305664306641E+04, .8746956787109E+03},
 {    3990, .8281950195312E+04, -.1317487792969E+04, .8774849853516E+03},
 {    3991, .8291229492188E+04, -.1290354614258E+04, .8733309326172E+03},
 {    3992, .8280437500000E+04, -.1297180786133E+04, .8781619873047E+03},
 {    3993, .8269646484375E+04, -.1304007080078E+04, .8829931030273E+03},
 {    3994, .8279271484375E+04, -.1285592041016E+04, .8497422485352E+03},
 {    3995, .8266810546875E+04, -.1285592041016E+04, .8553202514648E+03},
 {    3996, .8254350585938E+04, -.1285592041016E+04, .8608986816406E+03},
 {    3997, .8280435546875E+04, -.1280829589844E+04, .8492209472656E+03},
 {    3998, .8269645507812E+04, -.1274003295898E+04, .8540517578125E+03},
 {    3999, .8258852539062E+04, -.1267177001953E+04, .8588827514648E+03},
 {    4000, .8283619140625E+04, -.1277343139648E+04, .8477963867188E+03},
 {    4001, .8277387695312E+04, -.1265519775391E+04, .8505855102539E+03},
 {    4002, .8271157226562E+04, -.1253696289062E+04, .8533746337891E+03},
 {    4003, .8287965820312E+04, -.1276067016602E+04, .8458502807617E+03},
 {    4004, .8287965820312E+04, -.1262414550781E+04, .8458502807617E+03},
 {    4005, .8287965820312E+04, -.1248762084961E+04, .8458502807617E+03},
 {    4006, .8292312500000E+04, -.1277343139648E+04, .8439044189453E+03},
 {    4007, .8298541992188E+04, -.1265519775391E+04, .8411154174805E+03},
 {    4008, .8304772460938E+04, -.1253696289062E+04, .8383262329102E+03},
 {    4009, .8295494140625E+04, -.1280829589844E+04, .8424800415039E+03},
 {    4010, .8306285156250E+04, -.1274003295898E+04, .8376492919922E+03},
 {    4011, .8317077148438E+04, -.1267177001953E+04, .8328181762695E+03},
 {    4012, .8296659179688E+04, -.1285592041016E+04, .8419584350586E+03},
 {    4013, .8309119140625E+04, -.1285592041016E+04, .8363804931641E+03},
 {    4014, .8321579101562E+04, -.1285592041016E+04, .8308020019531E+03},
 {    4015, .8295494140625E+04, -.1290354614258E+04, .8424800415039E+03},
 {    4016, .8306285156250E+04, -.1297180786133E+04, .8376492919922E+03},
 {    4017, .8317077148438E+04, -.1304007080078E+04, .8328181762695E+03},
 {    4018, .8292312500000E+04, -.1293840942383E+04, .8439045410156E+03},
 {    4019, .8298541992188E+04, -.1305664306641E+04, .8411154174805E+03},
 {    4020, .8304772460938E+04, -.1317487792969E+04, .8383262329102E+03},
 {    4021, .8287965820312E+04, -.1295117065430E+04, .8458502807617E+03},
 {    4022, .8287965820312E+04, -.1308769531250E+04, .8458502807617E+03},
 {    4023, .8287965820312E+04, -.1322422119141E+04, .8458502807617E+03},
 {    4024, .8283619140625E+04, -.1293840942383E+04, .8477963867188E+03},
 {    4025, .8277387695312E+04, -.1305664306641E+04, .8505855102539E+03},
 {    4026, .8271157226562E+04, -.1317487792969E+04, .8533746337891E+03},
 {    4027, .8280435546875E+04, -.1290354614258E+04, .8492209472656E+03},
 {    4028, .8269645507812E+04, -.1297180786133E+04, .8540517578125E+03},
 {    4029, .8258852539062E+04, -.1304007080078E+04, .8588827514648E+03},
 {    4030, .8268477539062E+04, -.1285592041016E+04, .8256318969727E+03},
 {    4031, .8256016601562E+04, -.1285592041016E+04, .8312100219727E+03},
 {    4032, .8243555664062E+04, -.1285592041016E+04, .8367886962891E+03},
 {    4033, .8269641601562E+04, -.1280829589844E+04, .8251106567383E+03},
 {    4034, .8258852539062E+04, -.1274003295898E+04, .8299414062500E+03},
 {    4035, .8248061523438E+04, -.1267177001953E+04, .8347722778320E+03},
 {    4036, .8272824218750E+04, -.1277343139648E+04, .8236859741211E+03},
 {    4037, .8266593750000E+04, -.1265519775391E+04, .8264750976562E+03},
 {    4038, .8260362304688E+04, -.1253696289062E+04, .8292642822266E+03},
 {    4039, .8277172851562E+04, -.1276067016602E+04, .8217399902344E+03},
 {    4040, .8277172851562E+04, -.1262414550781E+04, .8217399902344E+03},
 {    4041, .8277172851562E+04, -.1248762084961E+04, .8217399902344E+03},
 {    4042, .8281519531250E+04, -.1277343139648E+04, .8197940673828E+03},
 {    4043, .8287749023438E+04, -.1265519775391E+04, .8170050048828E+03},
 {    4044, .8293979492188E+04, -.1253696289062E+04, .8142157592773E+03},
 {    4045, .8284702148438E+04, -.1280829589844E+04, .8183694458008E+03},
 {    4046, .8295492187500E+04, -.1274003295898E+04, .8135386352539E+03},
 {    4047, .8306285156250E+04, -.1267177001953E+04, .8087073974609E+03},
 {    4048, .8285865234375E+04, -.1285592041016E+04, .8178482055664E+03},
 {    4049, .8298325195312E+04, -.1285592041016E+04, .8122700195312E+03},
 {    4050, .8310787109375E+04, -.1285592041016E+04, .8066917114258E+03},
 {    4051, .8284702148438E+04, -.1290354614258E+04, .8183694458008E+03},
 {    4052, .8295492187500E+04, -.1297180786133E+04, .8135386352539E+03},
 {    4053, .8306285156250E+04, -.1304007080078E+04, .8087073974609E+03},
 {    4054, .8281519531250E+04, -.1293840942383E+04, .8197941894531E+03},
 {    4055, .8287749023438E+04, -.1305664306641E+04, .8170050048828E+03},
 {    4056, .8293979492188E+04, -.1317487792969E+04, .8142158813477E+03},
 {    4057, .8277172851562E+04, -.1295117065430E+04, .8217399902344E+03},
 {    4058, .8277172851562E+04, -.1308769531250E+04, .8217399902344E+03},
 {    4059, .8277172851562E+04, -.1322422119141E+04, .8217399902344E+03},
 {    4060, .8272824218750E+04, -.1293840942383E+04, .8236859741211E+03},
 {    4061, .8266593750000E+04, -.1305664306641E+04, .8264750976562E+03},
 {    4062, .8260362304688E+04, -.1317487792969E+04, .8292642822266E+03},
 {    4063, .8269641601562E+04, -.1290354614258E+04, .8251106567383E+03},
 {    4064, .8258852539062E+04, -.1297180786133E+04, .8299414062500E+03},
 {    4065, .8248061523438E+04, -.1304007080078E+04, .8347722778320E+03},
 {    4066, .8257684570312E+04, -.1285592041016E+04, .8015216064453E+03},
 {    4067, .8245223632812E+04, -.1285592041016E+04, .8070999755859E+03},
 {    4068, .8232763671875E+04, -.1285592041016E+04, .8126779174805E+03},
 {    4069, .8258849609375E+04, -.1280829589844E+04, .8010001831055E+03},
 {    4070, .8248057617188E+04, -.1274003295898E+04, .8058309936523E+03},
 {    4071, .8237266601562E+04, -.1267177001953E+04, .8106618652344E+03},
 {    4072, .8262032226562E+04, -.1277343139648E+04, .7995756835938E+03},
 {    4073, .8255801757812E+04, -.1265519775391E+04, .8023648071289E+03},
 {    4074, .8249571289062E+04, -.1253696289062E+04, .8051539916992E+03},
 {    4075, .8266379882812E+04, -.1276067016602E+04, .7976295776367E+03},
 {    4076, .8266379882812E+04, -.1262414550781E+04, .7976295776367E+03},
 {    4077, .8266379882812E+04, -.1248762084961E+04, .7976295776367E+03},
 {    4078, .8270726562500E+04, -.1277343139648E+04, .7956835937500E+03},
 {    4079, .8276956054688E+04, -.1265519775391E+04, .7928944702148E+03},
 {    4080, .8283185546875E+04, -.1253696289062E+04, .7901054687500E+03},
 {    4081, .8273907226562E+04, -.1280829589844E+04, .7942592773438E+03},
 {    4082, .8284698242188E+04, -.1274003295898E+04, .7894284057617E+03},
 {    4083, .8295490234375E+04, -.1267177124023E+04, .7845974121094E+03},
 {    4084, .8275071289062E+04, -.1285592041016E+04, .7937378540039E+03},
 {    4085, .8287533203125E+04, -.1285592041016E+04, .7881596069336E+03},
 {    4086, .8299994140625E+04, -.1285592041016E+04, .7825811767578E+03},
 {    4087, .8273907226562E+04, -.1290354614258E+04, .7942592773438E+03},
 {    4088, .8284698242188E+04, -.1297180786133E+04, .7894284057617E+03},
 {    4089, .8295490234375E+04, -.1304007080078E+04, .7845974121094E+03},
 {    4090, .8270726562500E+04, -.1293840942383E+04, .7956835937500E+03},
 {    4091, .8276956054688E+04, -.1305664306641E+04, .7928944702148E+03},
 {    4092, .8283185546875E+04, -.1317487792969E+04, .7901054687500E+03},
 {    4093, .8266379882812E+04, -.1295117065430E+04, .7976295776367E+03},
 {    4094, .8266379882812E+04, -.1308769531250E+04, .7976295776367E+03},
 {    4095, .8266379882812E+04, -.1322422119141E+04, .7976295776367E+03},
 {    4096, .8262032226562E+04, -.1293840942383E+04, .7995756835938E+03},
 {    4097, .8255801757812E+04, -.1305664306641E+04, .8023648071289E+03},
 {    4098, .8249571289062E+04, -.1317487792969E+04, .8051539916992E+03},
 {    4099, .8258849609375E+04, -.1290354614258E+04, .8010001831055E+03},
 {    4100, .8248057617188E+04, -.1297180786133E+04, .8058309936523E+03},
 {    4101, .8237266601562E+04, -.1304007080078E+04, .8106618652344E+03},
 {    4102, .8246890625000E+04, -.1285592041016E+04, .7774111328125E+03},
 {    4103, .8234429687500E+04, -.1285592041016E+04, .7829895019531E+03},
 {    4104, .8221969726562E+04, -.1285592041016E+04, .7885675659180E+03},
 {    4105, .8248054687500E+04, -.1280829589844E+04, .7768898315430E+03},
 {    4106, .8237263671875E+04, -.1274003295898E+04, .7817205810547E+03},
 {    4107, .8226472656250E+04, -.1267177124023E+04, .7865515136719E+03},
 {    4108, .8251237304688E+04, -.1277343139648E+04, .7754653930664E+03},
 {    4109, .8245006835938E+04, -.1265519775391E+04, .7782544555664E+03},
 {    4110, .8238776367188E+04, -.1253696289062E+04, .7810434570312E+03},
 {    4111, .8255583984375E+04, -.1276067016602E+04, .7735194091797E+03},
 {    4112, .8255583984375E+04, -.1262414550781E+04, .7735193481445E+03},
 {    4113, .8255583984375E+04, -.1248762084961E+04, .7735193481445E+03},
 {    4114, .8259932617188E+04, -.1277343139648E+04, .7715731201172E+03},
 {    4115, .8266160156250E+04, -.1265519775391E+04, .7687843627930E+03},
 {    4116, .8272391601562E+04, -.1253696411133E+04, .7659952392578E+03},
 {    4117, .8263112304688E+04, -.1280829589844E+04, .7701489868164E+03},
 {    4118, .8273905273438E+04, -.1274003295898E+04, .7653179931641E+03},
 {    4119, .8284696289062E+04, -.1267177124023E+04, .7604870605469E+03},
 {    4120, .8264279296875E+04, -.1285592041016E+04, .7696273803711E+03},
 {    4121, .8276740234375E+04, -.1285592041016E+04, .7640490722656E+03},
 {    4122, .8289199218750E+04, -.1285592041016E+04, .7584710083008E+03},
 {    4123, .8263112304688E+04, -.1290354614258E+04, .7701489868164E+03},
 {    4124, .8273905273438E+04, -.1297180786133E+04, .7653179931641E+03},
 {    4125, .8284696289062E+04, -.1304007080078E+04, .7604870605469E+03},
 {    4126, .8259932617188E+04, -.1293840942383E+04, .7715732421875E+03},
 {    4127, .8266160156250E+04, -.1305664306641E+04, .7687843627930E+03},
 {    4128, .8272391601562E+04, -.1317487792969E+04, .7659952392578E+03},
 {    4129, .8255583984375E+04, -.1295117065430E+04, .7735194091797E+03},
 {    4130, .8255583984375E+04, -.1308769531250E+04, .7735194091797E+03},
 {    4131, .8255583984375E+04, -.1322422119141E+04, .7735194091797E+03},
 {    4132, .8251237304688E+04, -.1293840942383E+04, .7754653930664E+03},
 {    4133, .8245006835938E+04, -.1305664306641E+04, .7782544555664E+03},
 {    4134, .8238776367188E+04, -.1317487792969E+04, .7810434570312E+03},
 {    4135, .8248054687500E+04, -.1290354614258E+04, .7768898315430E+03},
 {    4136, .8237263671875E+04, -.1297180786133E+04, .7817205810547E+03},
 {    4137, .8226472656250E+04, -.1304007080078E+04, .7865515136719E+03},
 {    4138, .8360027343750E+04, -.1285592041016E+04, .1215151489258E+04},
 {    4139, .8346401367188E+04, -.1285592041016E+04, .1215998535156E+04},
 {    4140, .8332776367188E+04, -.1285592041016E+04, .1216846923828E+04},
 {    4141, .8361301757812E+04, -.1280829589844E+04, .1215072021484E+04},
 {    4142, .8349500976562E+04, -.1274003295898E+04, .1215806030273E+04},
 {    4143, .8337700195312E+04, -.1267177001953E+04, .1216540649414E+04},
 {    4144, .8364781250000E+04, -.1277343139648E+04, .1214855224609E+04},
 {    4145, .8357966796875E+04, -.1265519775391E+04, .1215278442383E+04},
 {    4146, .8351154296875E+04, -.1253696289062E+04, .1215702880859E+04},
 {    4147, .8369535156250E+04, -.1276067016602E+04, .1214559692383E+04},
 {    4148, .8369535156250E+04, -.1262414550781E+04, .1214559692383E+04},
 {    4149, .8369535156250E+04, -.1248762084961E+04, .1214559692383E+04},
 {    4150, .8374288085938E+04, -.1277343139648E+04, .1214264038086E+04},
 {    4151, .8381101562500E+04, -.1265519775391E+04, .1213840087891E+04},
 {    4152, .8387914062500E+04, -.1253696289062E+04, .1213415649414E+04},
 {    4153, .8377767578125E+04, -.1280829589844E+04, .1214047119141E+04},
 {    4154, .8389568359375E+04, -.1274003295898E+04, .1213312744141E+04},
 {    4155, .8401369140625E+04, -.1267177001953E+04, .1212578369141E+04},
 {    4156, .8379041015625E+04, -.1285592041016E+04, .1213967285156E+04},
 {    4157, .8392666992188E+04, -.1285592041016E+04, .1213119384766E+04},
 {    4158, .8406292968750E+04, -.1285592041016E+04, .1212271972656E+04},
 {    4159, .8377767578125E+04, -.1290354492188E+04, .1214047119141E+04},
 {    4160, .8389568359375E+04, -.1297180786133E+04, .1213312744141E+04},
 {    4161, .8401369140625E+04, -.1304007080078E+04, .1212578369141E+04},
 {    4162, .8374288085938E+04, -.1293840942383E+04, .1214264038086E+04},
 {    4163, .8381101562500E+04, -.1305664306641E+04, .1213840087891E+04},
 {    4164, .8387914062500E+04, -.1317487792969E+04, .1213415649414E+04},
 {    4165, .8369535156250E+04, -.1295117065430E+04, .1214559692383E+04},
 {    4166, .8369535156250E+04, -.1308769531250E+04, .1214559692383E+04},
 {    4167, .8369535156250E+04, -.1322421997070E+04, .1214559692383E+04},
 {    4168, .8364781250000E+04, -.1293840942383E+04, .1214855224609E+04},
 {    4169, .8357966796875E+04, -.1305664306641E+04, .1215278442383E+04},
 {    4170, .8351154296875E+04, -.1317487792969E+04, .1215702880859E+04},
 {    4171, .8361301757812E+04, -.1290354492188E+04, .1215072021484E+04},
 {    4172, .8349500976562E+04, -.1297180786133E+04, .1215806030273E+04},
 {    4173, .8337700195312E+04, -.1304007080078E+04, .1216540649414E+04},
 {    4174, .8361603515625E+04, -.1285592041016E+04, .1240502197266E+04},
 {    4175, .8347981445312E+04, -.1285592041016E+04, .1241349609375E+04},
 {    4176, .8334351562500E+04, -.1285592041016E+04, .1242197998047E+04},
 {    4177, .8362879882812E+04, -.1280829589844E+04, .1240422485352E+04},
 {    4178, .8351080078125E+04, -.1274003295898E+04, .1241156250000E+04},
 {    4179, .8339278320312E+04, -.1267177001953E+04, .1241891113281E+04},
 {    4180, .8366360351562E+04, -.1277343139648E+04, .1240206176758E+04},
 {    4181, .8359545898438E+04, -.1265519775391E+04, .1240629394531E+04},
 {    4182, .8352732421875E+04, -.1253696289062E+04, .1241053588867E+04},
 {    4183, .8371112304688E+04, -.1276067016602E+04, .1239910400391E+04},
 {    4184, .8371112304688E+04, -.1262414550781E+04, .1239910400391E+04},
 {    4185, .8371112304688E+04, -.1248762084961E+04, .1239910400391E+04},
 {    4186, .8375867187500E+04, -.1277343139648E+04, .1239614624023E+04},
 {    4187, .8382679687500E+04, -.1265519775391E+04, .1239191162109E+04},
 {    4188, .8389491210938E+04, -.1253696289062E+04, .1238766845703E+04},
 {    4189, .8379345703125E+04, -.1280829589844E+04, .1239397827148E+04},
 {    4190, .8391144531250E+04, -.1274003295898E+04, .1238663940430E+04},
 {    4191, .8402946289062E+04, -.1267177001953E+04, .1237929931641E+04},
 {    4192, .8380620117188E+04, -.1285592041016E+04, .1239318847656E+04},
 {    4193, .8394244140625E+04, -.1285592041016E+04, .1238471435547E+04},
 {    4194, .8407872070312E+04, -.1285592041016E+04, .1237623413086E+04},
 {    4195, .8379345703125E+04, -.1290354492188E+04, .1239397827148E+04},
 {    4196, .8391144531250E+04, -.1297180786133E+04, .1238663940430E+04},
 {    4197, .8402946289062E+04, -.1304007080078E+04, .1237929931641E+04},
 {    4198, .8375867187500E+04, -.1293840942383E+04, .1239614624023E+04},
 {    4199, .8382679687500E+04, -.1305664306641E+04, .1239191162109E+04},
 {    4200, .8389491210938E+04, -.1317487792969E+04, .1238766845703E+04},
 {    4201, .8371112304688E+04, -.1295117065430E+04, .1239910400391E+04},
 {    4202, .8371112304688E+04, -.1308769531250E+04, .1239910400391E+04},
 {    4203, .8371112304688E+04, -.1322421997070E+04, .1239910400391E+04},
 {    4204, .8366360351562E+04, -.1293840942383E+04, .1240206176758E+04},
 {    4205, .8359545898438E+04, -.1305664306641E+04, .1240629638672E+04},
 {    4206, .8352732421875E+04, -.1317487792969E+04, .1241053588867E+04},
 {    4207, .8362879882812E+04, -.1290354492188E+04, .1240422485352E+04},
 {    4208, .8351080078125E+04, -.1297180786133E+04, .1241156250000E+04},
 {    4209, .8339278320312E+04, -.1304007080078E+04, .1241891113281E+04},
 {    4210, .8363184570312E+04, -.1285592041016E+04, .1265853027344E+04},
 {    4211, .8349556640625E+04, -.1285592041016E+04, .1266700805664E+04},
 {    4212, .8335931640625E+04, -.1285592041016E+04, .1267548339844E+04},
 {    4213, .8364458984375E+04, -.1280829589844E+04, .1265773193359E+04},
 {    4214, .8352655273438E+04, -.1274003295898E+04, .1266507568359E+04},
 {    4215, .8340855468750E+04, -.1267177001953E+04, .1267241943359E+04},
 {    4216, .8367935546875E+04, -.1277343139648E+04, .1265556884766E+04},
 {    4217, .8361125000000E+04, -.1265519775391E+04, .1265981445312E+04},
 {    4218, .8354309570312E+04, -.1253696289062E+04, .1266405273438E+04},
 {    4219, .8372689453125E+04, -.1276067016602E+04, .1265261230469E+04},
 {    4220, .8372689453125E+04, -.1262414550781E+04, .1265261230469E+04},
 {    4221, .8372689453125E+04, -.1248762084961E+04, .1265261230469E+04},
 {    4222, .8377443359375E+04, -.1277343139648E+04, .1264965820312E+04},
 {    4223, .8384255859375E+04, -.1265519775391E+04, .1264541625977E+04},
 {    4224, .8391068359375E+04, -.1253696289062E+04, .1264117431641E+04},
 {    4225, .8380922851562E+04, -.1280829589844E+04, .1264749023438E+04},
 {    4226, .8392721679688E+04, -.1274003295898E+04, .1264015380859E+04},
 {    4227, .8404526367188E+04, -.1267177001953E+04, .1263280273438E+04},
 {    4228, .8382197265625E+04, -.1285592041016E+04, .1264669921875E+04},
 {    4229, .8395821289062E+04, -.1285592041016E+04, .1263822265625E+04},
 {    4230, .8409449218750E+04, -.1285592041016E+04, .1262974243164E+04},
 {    4231, .8380922851562E+04, -.1290354492188E+04, .1264749023438E+04},
 {    4232, .8392721679688E+04, -.1297180786133E+04, .1264015380859E+04},
 {    4233, .8404526367188E+04, -.1304007080078E+04, .1263280273438E+04},
 {    4234, .8377443359375E+04, -.1293840942383E+04, .1264965820312E+04},
 {    4235, .8384255859375E+04, -.1305664306641E+04, .1264541625977E+04},
 {    4236, .8391068359375E+04, -.1317487792969E+04, .1264117431641E+04},
 {    4237, .8372689453125E+04, -.1295117065430E+04, .1265261230469E+04},
 {    4238, .8372689453125E+04, -.1308769531250E+04, .1265261230469E+04},
 {    4239, .8372689453125E+04, -.1322421997070E+04, .1265261230469E+04},
 {    4240, .8367935546875E+04, -.1293840942383E+04, .1265556884766E+04},
 {    4241, .8361125000000E+04, -.1305664306641E+04, .1265981445312E+04},
 {    4242, .8354309570312E+04, -.1317487792969E+04, .1266405273438E+04},
 {    4243, .8364458984375E+04, -.1290354492188E+04, .1265773193359E+04},
 {    4244, .8352655273438E+04, -.1297180786133E+04, .1266507568359E+04},
 {    4245, .8340855468750E+04, -.1304007080078E+04, .1267241943359E+04},
 {    4246, .8364759765625E+04, -.1285592041016E+04, .1291203613281E+04},
 {    4247, .8351133789062E+04, -.1285592041016E+04, .1292051757812E+04},
 {    4248, .8337506835938E+04, -.1285592041016E+04, .1292899536133E+04},
 {    4249, .8366034179688E+04, -.1280829467773E+04, .1291124267578E+04},
 {    4250, .8354232421875E+04, -.1274003295898E+04, .1291859130859E+04},
 {    4251, .8342430664062E+04, -.1267177001953E+04, .1292593261719E+04},
 {    4252, .8369512695312E+04, -.1277343139648E+04, .1290907470703E+04},
 {    4253, .8362700195312E+04, -.1265519775391E+04, .1291331787109E+04},
 {    4254, .8355888671875E+04, -.1253696289062E+04, .1291756103516E+04},
 {    4255, .8374268554688E+04, -.1276067016602E+04, .1290612792969E+04},
 {    4256, .8374268554688E+04, -.1262414550781E+04, .1290612792969E+04},
 {    4257, .8374268554688E+04, -.1248762084961E+04, .1290612792969E+04},
 {    4258, .8379019531250E+04, -.1277343139648E+04, .1290316284180E+04},
 {    4259, .8385833984375E+04, -.1265519775391E+04, .1289892456055E+04},
 {    4260, .8392646484375E+04, -.1253696289062E+04, .1289468994141E+04},
 {    4261, .8382500976562E+04, -.1280829467773E+04, .1290101074219E+04},
 {    4262, .8394299804688E+04, -.1274003295898E+04, .1289366699219E+04},
 {    4263, .8406099609375E+04, -.1267177001953E+04, .1288631835938E+04},
 {    4264, .8383772460938E+04, -.1285592041016E+04, .1290021118164E+04},
 {    4265, .8397399414062E+04, -.1285592041016E+04, .1289173217773E+04},
 {    4266, .8411025390625E+04, -.1285592041016E+04, .1288325927734E+04},
 {    4267, .8382500976562E+04, -.1290354492188E+04, .1290101074219E+04},
 {    4268, .8394299804688E+04, -.1297180786133E+04, .1289366699219E+04},
 {    4269, .8406099609375E+04, -.1304007080078E+04, .1288631835938E+04},
 {    4270, .8379019531250E+04, -.1293840942383E+04, .1290316284180E+04},
 {    4271, .8385833984375E+04, -.1305664306641E+04, .1289892456055E+04},
 {    4272, .8392646484375E+04, -.1317487792969E+04, .1289468994141E+04},
 {    4273, .8374268554688E+04, -.1295117065430E+04, .1290612792969E+04},
 {    4274, .8374268554688E+04, -.1308769531250E+04, .1290612792969E+04},
 {    4275, .8374268554688E+04, -.1322421997070E+04, .1290612792969E+04},
 {    4276, .8369512695312E+04, -.1293840942383E+04, .1290907470703E+04},
 {    4277, .8362700195312E+04, -.1305664306641E+04, .1291331787109E+04},
 {    4278, .8355888671875E+04, -.1317487792969E+04, .1291756103516E+04},
 {    4279, .8366034179688E+04, -.1290354492188E+04, .1291124267578E+04},
 {    4280, .8354232421875E+04, -.1297180786133E+04, .1291859130859E+04},
 {    4281, .8342430664062E+04, -.1304007080078E+04, .1292593261719E+04},
 {    4282, .8366336914062E+04, -.1285592041016E+04, .1316554931641E+04},
 {    4283, .8352710937500E+04, -.1285592041016E+04, .1317402832031E+04},
 {    4284, .8339083984375E+04, -.1285592041016E+04, .1318249755859E+04},
 {    4285, .8367611328125E+04, -.1280829467773E+04, .1316475830078E+04},
 {    4286, .8355811523438E+04, -.1274003295898E+04, .1317209472656E+04},
 {    4287, .8344010742188E+04, -.1267177001953E+04, .1317943847656E+04},
 {    4288, .8371091796875E+04, -.1277343139648E+04, .1316259277344E+04},
 {    4289, .8364277343750E+04, -.1265519775391E+04, .1316682739258E+04},
 {    4290, .8357464843750E+04, -.1253696289062E+04, .1317106811523E+04},
 {    4291, .8375843750000E+04, -.1276067016602E+04, .1315962890625E+04},
 {    4292, .8375843750000E+04, -.1262414550781E+04, .1315962890625E+04},
 {    4293, .8375843750000E+04, -.1248762084961E+04, .1315962890625E+04},
 {    4294, .8380600585938E+04, -.1277343139648E+04, .1315667480469E+04},
 {    4295, .8387410156250E+04, -.1265519775391E+04, .1315244384766E+04},
 {    4296, .8394223632812E+04, -.1253696289062E+04, .1314819213867E+04},
 {    4297, .8384076171875E+04, -.1280829467773E+04, .1315450683594E+04},
 {    4298, .8395876953125E+04, -.1274003295898E+04, .1314717041016E+04},
 {    4299, .8407678710938E+04, -.1267177001953E+04, .1313982666016E+04},
 {    4300, .8385350585938E+04, -.1285592041016E+04, .1315371826172E+04},
 {    4301, .8398976562500E+04, -.1285592041016E+04, .1314524658203E+04},
 {    4302, .8412604492188E+04, -.1285592041016E+04, .1313676269531E+04},
 {    4303, .8384076171875E+04, -.1290354492188E+04, .1315450683594E+04},
 {    4304, .8395876953125E+04, -.1297180786133E+04, .1314717041016E+04},
 {    4305, .8407678710938E+04, -.1304007080078E+04, .1313982666016E+04},
 {    4306, .8380600585938E+04, -.1293840942383E+04, .1315667480469E+04},
 {    4307, .8387410156250E+04, -.1305664306641E+04, .1315244384766E+04},
 {    4308, .8394223632812E+04, -.1317487792969E+04, .1314819213867E+04},
 {    4309, .8375843750000E+04, -.1295117065430E+04, .1315962890625E+04},
 {    4310, .8375843750000E+04, -.1308769531250E+04, .1315962890625E+04},
 {    4311, .8375843750000E+04, -.1322421997070E+04, .1315962890625E+04},
 {    4312, .8371091796875E+04, -.1293840942383E+04, .1316259277344E+04},
 {    4313, .8364277343750E+04, -.1305664306641E+04, .1316682739258E+04},
 {    4314, .8357464843750E+04, -.1317487792969E+04, .1317106811523E+04},
 {    4315, .8367611328125E+04, -.1290354492188E+04, .1316475830078E+04},
 {    4316, .8355811523438E+04, -.1297180786133E+04, .1317209472656E+04},
 {    4317, .8344010742188E+04, -.1304007080078E+04, .1317943847656E+04},
 {    4318, .8367916015625E+04, -.1285592041016E+04, .1341906250000E+04},
 {    4319, .8354288085938E+04, -.1285592041016E+04, .1342754028320E+04},
 {    4320, .8340663085938E+04, -.1285592041016E+04, .1343600708008E+04},
 {    4321, .8369189453125E+04, -.1280829467773E+04, .1341826538086E+04},
 {    4322, .8357387695312E+04, -.1274003295898E+04, .1342560791016E+04},
 {    4323, .8345588867188E+04, -.1267177001953E+04, .1343294677734E+04},
 {    4324, .8372667968750E+04, -.1277343139648E+04, .1341609985352E+04},
 {    4325, .8365855468750E+04, -.1265519775391E+04, .1342033447266E+04},
 {    4326, .8359041992188E+04, -.1253696289062E+04, .1342457031250E+04},
 {    4327, .8377421875000E+04, -.1276067016602E+04, .1341314453125E+04},
 {    4328, .8377421875000E+04, -.1262414550781E+04, .1341314453125E+04},
 {    4329, .8377421875000E+04, -.1248762084961E+04, .1341314453125E+04},
 {    4330, .8382174804688E+04, -.1277343139648E+04, .1341018554688E+04},
 {    4331, .8388986328125E+04, -.1265519775391E+04, .1340594970703E+04},
 {    4332, .8395800781250E+04, -.1253696289062E+04, .1340171142578E+04},
 {    4333, .8385653320312E+04, -.1280829467773E+04, .1340802246094E+04},
 {    4334, .8397456054688E+04, -.1274003295898E+04, .1340067626953E+04},
 {    4335, .8409255859375E+04, -.1267177001953E+04, .1339333618164E+04},
 {    4336, .8386927734375E+04, -.1285592041016E+04, .1340722656250E+04},
 {    4337, .8400553710938E+04, -.1285592041016E+04, .1339874511719E+04},
 {    4338, .8414179687500E+04, -.1285592041016E+04, .1339027832031E+04},
 {    4339, .8385653320312E+04, -.1290354492188E+04, .1340802246094E+04},
 {    4340, .8397456054688E+04, -.1297180786133E+04, .1340067626953E+04},
 {    4341, .8409255859375E+04, -.1304007080078E+04, .1339333618164E+04},
 {    4342, .8382174804688E+04, -.1293840942383E+04, .1341018554688E+04},
 {    4343, .8388986328125E+04, -.1305664306641E+04, .1340594970703E+04},
 {    4344, .8395800781250E+04, -.1317487792969E+04, .1340171142578E+04},
 {    4345, .8377421875000E+04, -.1295117065430E+04, .1341314453125E+04},
 {    4346, .8377421875000E+04, -.1308769531250E+04, .1341314453125E+04},
 {    4347, .8377421875000E+04, -.1322421997070E+04, .1341314453125E+04},
 {    4348, .8372667968750E+04, -.1293840942383E+04, .1341609985352E+04},
 {    4349, .8365855468750E+04, -.1305664306641E+04, .1342033447266E+04},
 {    4350, .8359041992188E+04, -.1317487792969E+04, .1342457031250E+04},
 {    4351, .8369189453125E+04, -.1290354492188E+04, .1341826538086E+04},
 {    4352, .8357387695312E+04, -.1297180786133E+04, .1342560791016E+04},
 {    4353, .8345588867188E+04, -.1304007080078E+04, .1343294677734E+04},
 {    4354, .8285778320312E+04, -.1336392089844E+04, .7321741943359E+03},
 {    4355, .8268388671875E+04, -.1336392089844E+04, .7399581909180E+03},
 {    4356, .8285778320312E+04, -.1317342041016E+04, .7321741943359E+03},
 {    4357, .8273025390625E+04, -.1317342041016E+04, .7378823852539E+03},
 {    4358, .8285778320312E+04, -.1298292114258E+04, .7321741943359E+03},
 {    4359, .8277663085938E+04, -.1298292114258E+04, .7358067016602E+03},
 {    4360, .8275400390625E+04, -.1336392089844E+04, .7089911499023E+03},
 {    4361, .8258010742188E+04, -.1336392089844E+04, .7167747802734E+03},
 {    4362, .8275400390625E+04, -.1317342041016E+04, .7089911499023E+03},
 {    4363, .8262648437500E+04, -.1317342041016E+04, .7146995239258E+03},
 {    4364, .8275400390625E+04, -.1298292114258E+04, .7089911499023E+03},
 {    4365, .8267286132812E+04, -.1298292114258E+04, .7126237182617E+03},
 {    4366, .8285778320312E+04, -.1272892089844E+04, .7321741943359E+03},
 {    4367, .8277663085938E+04, -.1272892089844E+04, .7358067016602E+03},
 {    4368, .8275400390625E+04, -.1272892089844E+04, .7089911499023E+03},
 {    4369, .8267286132812E+04, -.1272892089844E+04, .7126237182617E+03},
 {    4370, .8285778320312E+04, -.1253842041016E+04, .7321741943359E+03},
 {    4371, .8273025390625E+04, -.1253842041016E+04, .7378823852539E+03},
 {    4372, .8285778320312E+04, -.1234792114258E+04, .7321741943359E+03},
 {    4373, .8268388671875E+04, -.1234792114258E+04, .7399581909180E+03},
 {    4374, .8275400390625E+04, -.1253842041016E+04, .7089911499023E+03},
 {    4375, .8262648437500E+04, -.1253842041016E+04, .7146995239258E+03},
 {    4376, .8275400390625E+04, -.1234792114258E+04, .7089911499023E+03},
 {    4377, .8258010742188E+04, -.1234792114258E+04, .7167747802734E+03},
 {    4378, .8278768554688E+04, -.1336392089844E+04, .7631411132812E+03},
 {    4379, .8263314453125E+04, -.1336392089844E+04, .7700597534180E+03},
 {    4380, .8247856445312E+04, -.1336392089844E+04, .7769786987305E+03},
 {    4381, .8232401367188E+04, -.1336392089844E+04, .7838974609375E+03},
 {    4382, .8283405273438E+04, -.1317342041016E+04, .7610656127930E+03},
 {    4383, .8264858398438E+04, -.1317342041016E+04, .7693679199219E+03},
 {    4384, .8246311523438E+04, -.1317342041016E+04, .7776707763672E+03},
 {    4385, .8227764648438E+04, -.1317342041016E+04, .7859730224609E+03},
 {    4386, .8288041015625E+04, -.1298292114258E+04, .7589898681641E+03},
 {    4387, .8266404296875E+04, -.1298292114258E+04, .7686760253906E+03},
 {    4388, .8244767578125E+04, -.1298292114258E+04, .7783623657227E+03},
 {    4389, .8223128906250E+04, -.1298292114258E+04, .7880487060547E+03},
 {    4390, .8252934570312E+04, -.1336392089844E+04, .7468768920898E+03},
 {    4391, .8237478515625E+04, -.1336392089844E+04, .7537955932617E+03},
 {    4392, .8222026367188E+04, -.1336392089844E+04, .7607140502930E+03},
 {    4393, .8254480468750E+04, -.1317342041016E+04, .7461848144531E+03},
 {    4394, .8235935546875E+04, -.1317342041016E+04, .7544873657227E+03},
 {    4395, .8217388671875E+04, -.1317342041016E+04, .7627896728516E+03},
 {    4396, .8256026367188E+04, -.1298292114258E+04, .7454929199219E+03},
 {    4397, .8234388671875E+04, -.1298292114258E+04, .7551792602539E+03},
 {    4398, .8212750976562E+04, -.1298292114258E+04, .7648654174805E+03},
 {    4399, .8242556640625E+04, -.1336392089844E+04, .7236938476562E+03},
 {    4400, .8227099609375E+04, -.1336392089844E+04, .7306127319336E+03},
 {    4401, .8211645507812E+04, -.1336392089844E+04, .7375313110352E+03},
 {    4402, .8244101562500E+04, -.1317342041016E+04, .7230019531250E+03},
 {    4403, .8225555664062E+04, -.1317342041016E+04, .7313045043945E+03},
 {    4404, .8207010742188E+04, -.1317342041016E+04, .7396068115234E+03},
 {    4405, .8245647460938E+04, -.1298292114258E+04, .7223100585938E+03},
 {    4406, .8224010742188E+04, -.1298292114258E+04, .7319962158203E+03},
 {    4407, .8202372070312E+04, -.1298292114258E+04, .7416827392578E+03},
 {    4408, .8288041015625E+04, -.1272892089844E+04, .7589898681641E+03},
 {    4409, .8266404296875E+04, -.1272892089844E+04, .7686760253906E+03},
 {    4410, .8244767578125E+04, -.1272892089844E+04, .7783623657227E+03},
 {    4411, .8223128906250E+04, -.1272892089844E+04, .7880487060547E+03},
 {    4412, .8256026367188E+04, -.1272892089844E+04, .7454929199219E+03},
 {    4413, .8234388671875E+04, -.1272892089844E+04, .7551792602539E+03},
 {    4414, .8212750976562E+04, -.1272892089844E+04, .7648654174805E+03},
 {    4415, .8245647460938E+04, -.1272892089844E+04, .7223100585938E+03},
 {    4416, .8224010742188E+04, -.1272892089844E+04, .7319962158203E+03},
 {    4417, .8202372070312E+04, -.1272892089844E+04, .7416827392578E+03},
 {    4418, .8283405273438E+04, -.1253842041016E+04, .7610656127930E+03},
 {    4419, .8264858398438E+04, -.1253842041016E+04, .7693679199219E+03},
 {    4420, .8246311523438E+04, -.1253842041016E+04, .7776707763672E+03},
 {    4421, .8227764648438E+04, -.1253842041016E+04, .7859730224609E+03},
 {    4422, .8278768554688E+04, -.1234792114258E+04, .7631411132812E+03},
 {    4423, .8263314453125E+04, -.1234792114258E+04, .7700597534180E+03},
 {    4424, .8247856445312E+04, -.1234792114258E+04, .7769786987305E+03},
 {    4425, .8232401367188E+04, -.1234792114258E+04, .7838974609375E+03},
 {    4426, .8254480468750E+04, -.1253842041016E+04, .7461848144531E+03},
 {    4427, .8235935546875E+04, -.1253842041016E+04, .7544873657227E+03},
 {    4428, .8217388671875E+04, -.1253842041016E+04, .7627896728516E+03},
 {    4429, .8252934570312E+04, -.1234792114258E+04, .7468768920898E+03},
 {    4430, .8237478515625E+04, -.1234792114258E+04, .7537954711914E+03},
 {    4431, .8222026367188E+04, -.1234792114258E+04, .7607140502930E+03},
 {    4432, .8244101562500E+04, -.1253842041016E+04, .7230019531250E+03},
 {    4433, .8225555664062E+04, -.1253842041016E+04, .7313045043945E+03},
 {    4434, .8207010742188E+04, -.1253842041016E+04, .7396068115234E+03},
 {    4435, .8242556640625E+04, -.1234792114258E+04, .7236937255859E+03},
 {    4436, .8227099609375E+04, -.1234792114258E+04, .7306126708984E+03},
 {    4437, .8211645507812E+04, -.1234792114258E+04, .7375313110352E+03},
 {    4438, .8204635742188E+04, -.1336392089844E+04, .7684982910156E+03},
 {    4439, .8204635742188E+04, -.1317342041016E+04, .7684982910156E+03},
 {    4440, .8204635742188E+04, -.1298292114258E+04, .7684982910156E+03},
 {    4441, .8194258789062E+04, -.1336392089844E+04, .7453149414062E+03},
 {    4442, .8194258789062E+04, -.1317342041016E+04, .7453149414062E+03},
 {    4443, .8194258789062E+04, -.1298292114258E+04, .7453149414062E+03},
 {    4444, .8204635742188E+04, -.1272892089844E+04, .7684982910156E+03},
 {    4445, .8194258789062E+04, -.1272892089844E+04, .7453149414062E+03},
 {    4446, .8204635742188E+04, -.1253842041016E+04, .7684982910156E+03},
 {    4447, .8204635742188E+04, -.1234792114258E+04, .7684982910156E+03},
 {    4448, .8194258789062E+04, -.1253842041016E+04, .7453149414062E+03},
 {    4449, .8194258789062E+04, -.1234792114258E+04, .7453149414062E+03},
 {    4450, .8441018554688E+04, -.1316072021484E+04, .1063415771484E+04},
 {    4451, .8446815429688E+04, -.1316072021484E+04, .1060821289062E+04},
 {    4452, .8452613281250E+04, -.1316072021484E+04, .1058226318359E+04},
 {    4453, .8441018554688E+04, -.1300832031250E+04, .1063415771484E+04},
 {    4454, .8446815429688E+04, -.1300832031250E+04, .1060821289062E+04},
 {    4455, .8452613281250E+04, -.1300832031250E+04, .1058226318359E+04},
 {    4456, .8441018554688E+04, -.1285592041016E+04, .1063415771484E+04},
 {    4457, .8446815429688E+04, -.1285592041016E+04, .1060821289062E+04},
 {    4458, .8452613281250E+04, -.1285592041016E+04, .1058226318359E+04},
 {    4459, .8441018554688E+04, -.1270352050781E+04, .1063415771484E+04},
 {    4460, .8446815429688E+04, -.1270352050781E+04, .1060821289062E+04},
 {    4461, .8452613281250E+04, -.1270352050781E+04, .1058226318359E+04},
 {    4462, .8441018554688E+04, -.1255112060547E+04, .1063415771484E+04},
 {    4463, .8446815429688E+04, -.1255112060547E+04, .1060821289062E+04},
 {    4464, .8452613281250E+04, -.1255112060547E+04, .1058226318359E+04},
 {    4465, .8435829101562E+04, -.1316072021484E+04, .1051824462891E+04},
 {    4466, .8441626953125E+04, -.1316072021484E+04, .1049229736328E+04},
 {    4467, .8447420898438E+04, -.1316072021484E+04, .1046635253906E+04},
 {    4468, .8435829101562E+04, -.1300832031250E+04, .1051824462891E+04},
 {    4469, .8441626953125E+04, -.1300832031250E+04, .1049229736328E+04},
 {    4470, .8447420898438E+04, -.1300832031250E+04, .1046635253906E+04},
 {    4471, .8435829101562E+04, -.1285592041016E+04, .1051824462891E+04},
 {    4472, .8441626953125E+04, -.1285592041016E+04, .1049229736328E+04},
 {    4473, .8447420898438E+04, -.1285592041016E+04, .1046635253906E+04},
 {    4474, .8435829101562E+04, -.1270352050781E+04, .1051824462891E+04},
 {    4475, .8441626953125E+04, -.1270352050781E+04, .1049229736328E+04},
 {    4476, .8447420898438E+04, -.1270352050781E+04, .1046635253906E+04},
 {    4477, .8435829101562E+04, -.1255112060547E+04, .1051824462891E+04},
 {    4478, .8441626953125E+04, -.1255112060547E+04, .1049229736328E+04},
 {    4479, .8447420898438E+04, -.1255112060547E+04, .1046635253906E+04},
 {    4480, .8430640625000E+04, -.1316072021484E+04, .1040232788086E+04},
 {    4481, .8436436523438E+04, -.1316072021484E+04, .1037638305664E+04},
 {    4482, .8442232421875E+04, -.1316072021484E+04, .1035043945312E+04},
 {    4483, .8430640625000E+04, -.1300832031250E+04, .1040232788086E+04},
 {    4484, .8436436523438E+04, -.1300832031250E+04, .1037638305664E+04},
 {    4485, .8442232421875E+04, -.1300832031250E+04, .1035043945312E+04},
 {    4486, .8430640625000E+04, -.1285592041016E+04, .1040232788086E+04},
 {    4487, .8436436523438E+04, -.1285592041016E+04, .1037638305664E+04},
 {    4488, .8442232421875E+04, -.1285592041016E+04, .1035043945312E+04},
 {    4489, .8430640625000E+04, -.1270352050781E+04, .1040232788086E+04},
 {    4490, .8436436523438E+04, -.1270352050781E+04, .1037638305664E+04},
 {    4491, .8442232421875E+04, -.1270352050781E+04, .1035043945312E+04},
 {    4492, .8430640625000E+04, -.1255112060547E+04, .1040232788086E+04},
 {    4493, .8436436523438E+04, -.1255112060547E+04, .1037638305664E+04},
 {    4494, .8442232421875E+04, -.1255112060547E+04, .1035043945312E+04},
 {    4495, .8425451171875E+04, -.1316072021484E+04, .1028641723633E+04},
 {    4496, .8431246093750E+04, -.1316072021484E+04, .1026047363281E+04},
 {    4497, .8437041992188E+04, -.1316072021484E+04, .1023452514648E+04},
 {    4498, .8425451171875E+04, -.1300832031250E+04, .1028641723633E+04},
 {    4499, .8431246093750E+04, -.1300832031250E+04, .1026047363281E+04},
 {    4500, .8437041992188E+04, -.1300832031250E+04, .1023452514648E+04},
 {    4501, .8425451171875E+04, -.1285592041016E+04, .1028641723633E+04},
 {    4502, .8431246093750E+04, -.1285592041016E+04, .1026047363281E+04},
 {    4503, .8437041992188E+04, -.1285592041016E+04, .1023452514648E+04},
 {    4504, .8425451171875E+04, -.1270352050781E+04, .1028641723633E+04},
 {    4505, .8431246093750E+04, -.1270352050781E+04, .1026047363281E+04},
 {    4506, .8437041992188E+04, -.1270352050781E+04, .1023452514648E+04},
 {    4507, .8425451171875E+04, -.1255112060547E+04, .1028641723633E+04},
 {    4508, .8431246093750E+04, -.1255112060547E+04, .1026047363281E+04},
 {    4509, .8437041992188E+04, -.1255112060547E+04, .1023452514648E+04},
 {    4510, .8420261718750E+04, -.1316072021484E+04, .1017049621582E+04},
 {    4511, .8426059570312E+04, -.1316072021484E+04, .1014454711914E+04},
 {    4512, .8431854492188E+04, -.1316072021484E+04, .1011860656738E+04},
 {    4513, .8420261718750E+04, -.1300832031250E+04, .1017049621582E+04},
 {    4514, .8426059570312E+04, -.1300832031250E+04, .1014454711914E+04},
 {    4515, .8431854492188E+04, -.1300832031250E+04, .1011860656738E+04},
 {    4516, .8420261718750E+04, -.1285592041016E+04, .1017049621582E+04},
 {    4517, .8426059570312E+04, -.1285592041016E+04, .1014454711914E+04},
 {    4518, .8431854492188E+04, -.1285592041016E+04, .1011860656738E+04},
 {    4519, .8420261718750E+04, -.1270352050781E+04, .1017049621582E+04},
 {    4520, .8426059570312E+04, -.1270352050781E+04, .1014454711914E+04},
 {    4521, .8431854492188E+04, -.1270352050781E+04, .1011860656738E+04},
 {    4522, .8420261718750E+04, -.1255112060547E+04, .1017049621582E+04},
 {    4523, .8426059570312E+04, -.1255112060547E+04, .1014454711914E+04},
 {    4524, .8431854492188E+04, -.1255112060547E+04, .1011860656738E+04},
 {    4525, .8418965820312E+04, -.1316072021484E+04, .1014152038574E+04},
 {    4526, .8424761718750E+04, -.1316072021484E+04, .1011557006836E+04},
 {    4527, .8430557617188E+04, -.1316072021484E+04, .1008962524414E+04},
 {    4528, .8418965820312E+04, -.1300832031250E+04, .1014152038574E+04},
 {    4529, .8424761718750E+04, -.1300832031250E+04, .1011557006836E+04},
 {    4530, .8430557617188E+04, -.1300832031250E+04, .1008962524414E+04},
 {    4531, .8418965820312E+04, -.1285592041016E+04, .1014152038574E+04},
 {    4532, .8424761718750E+04, -.1285592041016E+04, .1011557006836E+04},
 {    4533, .8430557617188E+04, -.1285592041016E+04, .1008962524414E+04},
 {    4534, .8418965820312E+04, -.1270352050781E+04, .1014152038574E+04},
 {    4535, .8424761718750E+04, -.1270352050781E+04, .1011557006836E+04},
 {    4536, .8430557617188E+04, -.1270352050781E+04, .1008962524414E+04},
 {    4537, .8418965820312E+04, -.1255112060547E+04, .1014152038574E+04},
 {    4538, .8424761718750E+04, -.1255112060547E+04, .1011557006836E+04},
 {    4539, .8430557617188E+04, -.1255112060547E+04, .1008962524414E+04},
 {    4540, .8413776367188E+04, -.1316072021484E+04, .1002560485840E+04},
 {    4541, .8419572265625E+04, -.1316072021484E+04, .9999659423828E+03},
 {    4542, .8425366210938E+04, -.1316072021484E+04, .9973714599609E+03},
 {    4543, .8413776367188E+04, -.1300832031250E+04, .1002560485840E+04},
 {    4544, .8419572265625E+04, -.1300832031250E+04, .9999659423828E+03},
 {    4545, .8425366210938E+04, -.1300832031250E+04, .9973714599609E+03},
 {    4546, .8413776367188E+04, -.1285592041016E+04, .1002560485840E+04},
 {    4547, .8419572265625E+04, -.1285592041016E+04, .9999659423828E+03},
 {    4548, .8425366210938E+04, -.1285592041016E+04, .9973714599609E+03},
 {    4549, .8413776367188E+04, -.1270352050781E+04, .1002560485840E+04},
 {    4550, .8419572265625E+04, -.1270352050781E+04, .9999659423828E+03},
 {    4551, .8425366210938E+04, -.1270352050781E+04, .9973714599609E+03},
 {    4552, .8413776367188E+04, -.1255112060547E+04, .1002560485840E+04},
 {    4553, .8419572265625E+04, -.1255112060547E+04, .9999659423828E+03},
 {    4554, .8425366210938E+04, -.1255112060547E+04, .9973714599609E+03},
 {    4555, .8408587890625E+04, -.1316072021484E+04, .9909688720703E+03},
 {    4556, .8414383789062E+04, -.1316072021484E+04, .9883745117188E+03},
 {    4557, .8420177734375E+04, -.1316072021484E+04, .9857799072266E+03},
 {    4558, .8408587890625E+04, -.1300832031250E+04, .9909688720703E+03},
 {    4559, .8414383789062E+04, -.1300832031250E+04, .9883745117188E+03},
 {    4560, .8420177734375E+04, -.1300832031250E+04, .9857799072266E+03},
 {    4561, .8408587890625E+04, -.1285592041016E+04, .9909688720703E+03},
 {    4562, .8414383789062E+04, -.1285592041016E+04, .9883745117188E+03},
 {    4563, .8420177734375E+04, -.1285592041016E+04, .9857799072266E+03},
 {    4564, .8408587890625E+04, -.1270352050781E+04, .9909688720703E+03},
 {    4565, .8414383789062E+04, -.1270352050781E+04, .9883745117188E+03},
 {    4566, .8420177734375E+04, -.1270352050781E+04, .9857799072266E+03},
 {    4567, .8408587890625E+04, -.1255112060547E+04, .9909688720703E+03},
 {    4568, .8414383789062E+04, -.1255112060547E+04, .9883745117188E+03},
 {    4569, .8420177734375E+04, -.1255112060547E+04, .9857799072266E+03},
 {    4570, .8407289062500E+04, -.1316072021484E+04, .9880708007812E+03},
 {    4571, .8413086914062E+04, -.1316072021484E+04, .9854764404297E+03},
 {    4572, .8418881835938E+04, -.1316072021484E+04, .9828820800781E+03},
 {    4573, .8407289062500E+04, -.1300832031250E+04, .9880708007812E+03},
 {    4574, .8413086914062E+04, -.1300832031250E+04, .9854764404297E+03},
 {    4575, .8418881835938E+04, -.1300832031250E+04, .9828820800781E+03},
 {    4576, .8407289062500E+04, -.1285592041016E+04, .9880708007812E+03},
 {    4577, .8413086914062E+04, -.1285592041016E+04, .9854764404297E+03},
 {    4578, .8418881835938E+04, -.1285592041016E+04, .9828820800781E+03},
 {    4579, .8407289062500E+04, -.1270352050781E+04, .9880708007812E+03},
 {    4580, .8413086914062E+04, -.1270352050781E+04, .9854764404297E+03},
 {    4581, .8418881835938E+04, -.1270352050781E+04, .9828820800781E+03},
 {    4582, .8407289062500E+04, -.1255112060547E+04, .9880708007812E+03},
 {    4583, .8413086914062E+04, -.1255112060547E+04, .9854764404297E+03},
 {    4584, .8418881835938E+04, -.1255112060547E+04, .9828820800781E+03},
 {    4585, .8402100585938E+04, -.1316072021484E+04, .9764794921875E+03},
 {    4586, .8407896484375E+04, -.1316072021484E+04, .9738850097656E+03},
 {    4587, .8413689453125E+04, -.1316072021484E+04, .9712902832031E+03},
 {    4588, .8402100585938E+04, -.1300832031250E+04, .9764794921875E+03},
 {    4589, .8407896484375E+04, -.1300832031250E+04, .9738850097656E+03},
 {    4590, .8413689453125E+04, -.1300832031250E+04, .9712902832031E+03},
 {    4591, .8402100585938E+04, -.1285592041016E+04, .9764794921875E+03},
 {    4592, .8407896484375E+04, -.1285592041016E+04, .9738850097656E+03},
 {    4593, .8413689453125E+04, -.1285592041016E+04, .9712902832031E+03},
 {    4594, .8402100585938E+04, -.1270352050781E+04, .9764794921875E+03},
 {    4595, .8407896484375E+04, -.1270352050781E+04, .9738850097656E+03},
 {    4596, .8413689453125E+04, -.1270352050781E+04, .9712902832031E+03},
 {    4597, .8402100585938E+04, -.1255112060547E+04, .9764794921875E+03},
 {    4598, .8407896484375E+04, -.1255112060547E+04, .9738850097656E+03},
 {    4599, .8413689453125E+04, -.1255112060547E+04, .9712902832031E+03},
 {    4600, .8396912109375E+04, -.1316072021484E+04, .9648877563477E+03},
 {    4601, .8402708007812E+04, -.1316072021484E+04, .9622937011719E+03},
 {    4602, .8408500976562E+04, -.1316072021484E+04, .9596988525391E+03},
 {    4603, .8396912109375E+04, -.1300832031250E+04, .9648877563477E+03},
 {    4604, .8402708007812E+04, -.1300832031250E+04, .9622937011719E+03},
 {    4605, .8408500976562E+04, -.1300832031250E+04, .9596988525391E+03},
 {    4606, .8396912109375E+04, -.1285592041016E+04, .9648877563477E+03},
 {    4607, .8402708007812E+04, -.1285592041016E+04, .9622937011719E+03},
 {    4608, .8408500976562E+04, -.1285592041016E+04, .9596988525391E+03},
 {    4609, .8396912109375E+04, -.1270352050781E+04, .9648877563477E+03},
 {    4610, .8402708007812E+04, -.1270352050781E+04, .9622937011719E+03},
 {    4611, .8408500976562E+04, -.1270352050781E+04, .9596988525391E+03},
 {    4612, .8396912109375E+04, -.1255112060547E+04, .9648877563477E+03},
 {    4613, .8402708007812E+04, -.1255112060547E+04, .9622937011719E+03},
 {    4614, .8408500976562E+04, -.1255112060547E+04, .9596988525391E+03},
 {    4615, .8395613281250E+04, -.1316072021484E+04, .9619902343750E+03},
 {    4616, .8401409179688E+04, -.1316072021484E+04, .9593955078125E+03},
 {    4617, .8407206054688E+04, -.1316072021484E+04, .9568010253906E+03},
 {    4618, .8395613281250E+04, -.1300832031250E+04, .9619902343750E+03},
 {    4619, .8401409179688E+04, -.1300832031250E+04, .9593955078125E+03},
 {    4620, .8407206054688E+04, -.1300832031250E+04, .9568010253906E+03},
 {    4621, .8395613281250E+04, -.1285592041016E+04, .9619902343750E+03},
 {    4622, .8401409179688E+04, -.1285592041016E+04, .9593955078125E+03},
 {    4623, .8407206054688E+04, -.1285592041016E+04, .9568010253906E+03},
 {    4624, .8395613281250E+04, -.1270352050781E+04, .9619902343750E+03},
 {    4625, .8401409179688E+04, -.1270352050781E+04, .9593955078125E+03},
 {    4626, .8407206054688E+04, -.1270352050781E+04, .9568010253906E+03},
 {    4627, .8395613281250E+04, -.1255112060547E+04, .9619902343750E+03},
 {    4628, .8401409179688E+04, -.1255112060547E+04, .9593955078125E+03},
 {    4629, .8407206054688E+04, -.1255112060547E+04, .9568010253906E+03},
 {    4630, .8390424804688E+04, -.1316072021484E+04, .9503986206055E+03},
 {    4631, .8396221679688E+04, -.1316072021484E+04, .9478041992188E+03},
 {    4632, .8402015625000E+04, -.1316072021484E+04, .9452095947266E+03},
 {    4633, .8390424804688E+04, -.1300832031250E+04, .9503986206055E+03},
 {    4634, .8396221679688E+04, -.1300832031250E+04, .9478041992188E+03},
 {    4635, .8402015625000E+04, -.1300832031250E+04, .9452095947266E+03},
 {    4636, .8390424804688E+04, -.1285592041016E+04, .9503986206055E+03},
 {    4637, .8396221679688E+04, -.1285592041016E+04, .9478041992188E+03},
 {    4638, .8402015625000E+04, -.1285592041016E+04, .9452095947266E+03},
 {    4639, .8390424804688E+04, -.1270352050781E+04, .9503986206055E+03},
 {    4640, .8396221679688E+04, -.1270352050781E+04, .9478041992188E+03},
 {    4641, .8402015625000E+04, -.1270352050781E+04, .9452095947266E+03},
 {    4642, .8390424804688E+04, -.1255112060547E+04, .9503986206055E+03},
 {    4643, .8396221679688E+04, -.1255112060547E+04, .9478041992188E+03},
 {    4644, .8402015625000E+04, -.1255112060547E+04, .9452095947266E+03},
 {    4645, .8385236328125E+04, -.1316072021484E+04, .9388069458008E+03},
 {    4646, .8391031250000E+04, -.1316072021484E+04, .9362123413086E+03},
 {    4647, .8396826171875E+04, -.1316072021484E+04, .9336179199219E+03},
 {    4648, .8385236328125E+04, -.1300832031250E+04, .9388069458008E+03},
 {    4649, .8391031250000E+04, -.1300832031250E+04, .9362123413086E+03},
 {    4650, .8396826171875E+04, -.1300832031250E+04, .9336179199219E+03},
 {    4651, .8385236328125E+04, -.1285592041016E+04, .9388069458008E+03},
 {    4652, .8391031250000E+04, -.1285592041016E+04, .9362123413086E+03},
 {    4653, .8396826171875E+04, -.1285592041016E+04, .9336179199219E+03},
 {    4654, .8385236328125E+04, -.1270352050781E+04, .9388069458008E+03},
 {    4655, .8391031250000E+04, -.1270352050781E+04, .9362123413086E+03},
 {    4656, .8396826171875E+04, -.1270352050781E+04, .9336179199219E+03},
 {    4657, .8385236328125E+04, -.1255112060547E+04, .9388069458008E+03},
 {    4658, .8391031250000E+04, -.1255112060547E+04, .9362123413086E+03},
 {    4659, .8396826171875E+04, -.1255112060547E+04, .9336179199219E+03},
 {    4660, .8383940429688E+04, -.1316072021484E+04, .9359089965820E+03},
 {    4661, .8389734375000E+04, -.1316072021484E+04, .9333146362305E+03},
 {    4662, .8395529296875E+04, -.1316072021484E+04, .9307202758789E+03},
 {    4663, .8383940429688E+04, -.1300832031250E+04, .9359089965820E+03},
 {    4664, .8389734375000E+04, -.1300832031250E+04, .9333146362305E+03},
 {    4665, .8395529296875E+04, -.1300832031250E+04, .9307202758789E+03},
 {    4666, .8383940429688E+04, -.1285592041016E+04, .9359089965820E+03},
 {    4667, .8389734375000E+04, -.1285592041016E+04, .9333146362305E+03},
 {    4668, .8395529296875E+04, -.1285592041016E+04, .9307202758789E+03},
 {    4669, .8383940429688E+04, -.1270352050781E+04, .9359089965820E+03},
 {    4670, .8389734375000E+04, -.1270352050781E+04, .9333146362305E+03},
 {    4671, .8395529296875E+04, -.1270352050781E+04, .9307202758789E+03},
 {    4672, .8383940429688E+04, -.1255112060547E+04, .9359089965820E+03},
 {    4673, .8389734375000E+04, -.1255112060547E+04, .9333146362305E+03},
 {    4674, .8395529296875E+04, -.1255112060547E+04, .9307202758789E+03},
 {    4675, .8378748046875E+04, -.1316072021484E+04, .9243178710938E+03},
 {    4676, .8384543945312E+04, -.1316072021484E+04, .9217230224609E+03},
 {    4677, .8390341796875E+04, -.1316072021484E+04, .9191286621094E+03},
 {    4678, .8378748046875E+04, -.1300832031250E+04, .9243178710938E+03},
 {    4679, .8384543945312E+04, -.1300832031250E+04, .9217230224609E+03},
 {    4680, .8390341796875E+04, -.1300832031250E+04, .9191286621094E+03},
 {    4681, .8378748046875E+04, -.1285592041016E+04, .9243178710938E+03},
 {    4682, .8384543945312E+04, -.1285592041016E+04, .9217230224609E+03},
 {    4683, .8390341796875E+04, -.1285592041016E+04, .9191286621094E+03},
 {    4684, .8378748046875E+04, -.1270352050781E+04, .9243178710938E+03},
 {    4685, .8384543945312E+04, -.1270352050781E+04, .9217230224609E+03},
 {    4686, .8390341796875E+04, -.1270352050781E+04, .9191286621094E+03},
 {    4687, .8378748046875E+04, -.1255112060547E+04, .9243178710938E+03},
 {    4688, .8384543945312E+04, -.1255112060547E+04, .9217230224609E+03},
 {    4689, .8390341796875E+04, -.1255112060547E+04, .9191286621094E+03},
 {    4690, .8373558593750E+04, -.1316072021484E+04, .9127261962891E+03},
 {    4691, .8379356445312E+04, -.1316072021484E+04, .9101314697266E+03},
 {    4692, .8385152343750E+04, -.1316072021484E+04, .9075369873047E+03},
 {    4693, .8373558593750E+04, -.1300832031250E+04, .9127261962891E+03},
 {    4694, .8379356445312E+04, -.1300832031250E+04, .9101314697266E+03},
 {    4695, .8385152343750E+04, -.1300832031250E+04, .9075369873047E+03},
 {    4696, .8373558593750E+04, -.1285592041016E+04, .9127261962891E+03},
 {    4697, .8379356445312E+04, -.1285592041016E+04, .9101314697266E+03},
 {    4698, .8385152343750E+04, -.1285592041016E+04, .9075369873047E+03},
 {    4699, .8373558593750E+04, -.1270352050781E+04, .9127261962891E+03},
 {    4700, .8379356445312E+04, -.1270352050781E+04, .9101314697266E+03},
 {    4701, .8385152343750E+04, -.1270352050781E+04, .9075369873047E+03},
 {    4702, .8373558593750E+04, -.1255112060547E+04, .9127261962891E+03},
 {    4703, .8379356445312E+04, -.1255112060547E+04, .9101314697266E+03},
 {    4704, .8385152343750E+04, -.1255112060547E+04, .9075369873047E+03},
 {    4705, .8432866210938E+04, -.1346320190430E+04, .1067066284180E+04},
 {    4706, .8436389648438E+04, -.1346337890625E+04, .1065488525391E+04},
 {    4707, .8439912109375E+04, -.1346355468750E+04, .1063911376953E+04},
 {    4708, .8434066406250E+04, -.1331312011719E+04, .1066529052734E+04},
 {    4709, .8437540039062E+04, -.1331312011719E+04, .1064972778320E+04},
 {    4710, .8441018554688E+04, -.1331312011719E+04, .1063415771484E+04},
 {    4711, .8427674804688E+04, -.1346320190430E+04, .1055474365234E+04},
 {    4712, .8431199218750E+04, -.1346337890625E+04, .1053896972656E+04},
 {    4713, .8434723632812E+04, -.1346355468750E+04, .1052319702148E+04},
 {    4714, .8428875976562E+04, -.1331312011719E+04, .1054938110352E+04},
 {    4715, .8432352539062E+04, -.1331312011719E+04, .1053380981445E+04},
 {    4716, .8435829101562E+04, -.1331312011719E+04, .1051824462891E+04},
 {    4717, .8422486328125E+04, -.1346320190430E+04, .1043883300781E+04},
 {    4718, .8426010742188E+04, -.1346337890625E+04, .1042305541992E+04},
 {    4719, .8429534179688E+04, -.1346355468750E+04, .1040727661133E+04},
 {    4720, .8423685546875E+04, -.1331312011719E+04, .1043346679688E+04},
 {    4721, .8427162109375E+04, -.1331312011719E+04, .1041789916992E+04},
 {    4722, .8430640625000E+04, -.1331312011719E+04, .1040232788086E+04},
 {    4723, .8417297851562E+04, -.1346320190430E+04, .1032291748047E+04},
 {    4724, .8420820312500E+04, -.1346337890625E+04, .1030714355469E+04},
 {    4725, .8424345703125E+04, -.1346355468750E+04, .1029136962891E+04},
 {    4726, .8418497070312E+04, -.1331312011719E+04, .1031754394531E+04},
 {    4727, .8421972656250E+04, -.1331312011719E+04, .1030198242188E+04},
 {    4728, .8425451171875E+04, -.1331312011719E+04, .1028641723633E+04},
 {    4729, .8412106445312E+04, -.1346320190430E+04, .1020699951172E+04},
 {    4730, .8415631835938E+04, -.1346337890625E+04, .1019122680664E+04},
 {    4731, .8419157226562E+04, -.1346355468750E+04, .1017545288086E+04},
 {    4732, .8413306640625E+04, -.1331312011719E+04, .1020163574219E+04},
 {    4733, .8416784179688E+04, -.1331312011719E+04, .1018606567383E+04},
 {    4734, .8420261718750E+04, -.1331312011719E+04, .1017049621582E+04},
 {    4735, .8410809570312E+04, -.1346320190430E+04, .1017802490234E+04},
 {    4736, .8414334960938E+04, -.1346337890625E+04, .1016224731445E+04},
 {    4737, .8417860351562E+04, -.1346355468750E+04, .1014647338867E+04},
 {    4738, .8412009765625E+04, -.1331312011719E+04, .1017265747070E+04},
 {    4739, .8415489257812E+04, -.1331312011719E+04, .1015708374023E+04},
 {    4740, .8418965820312E+04, -.1331312011719E+04, .1014152038574E+04},
 {    4741, .8405622070312E+04, -.1346320190430E+04, .1006210815430E+04},
 {    4742, .8409144531250E+04, -.1346337890625E+04, .1004633666992E+04},
 {    4743, .8412669921875E+04, -.1346355468750E+04, .1003055786133E+04},
 {    4744, .8406821289062E+04, -.1331312011719E+04, .1005673706055E+04},
 {    4745, .8410297851562E+04, -.1331312011719E+04, .1004116821289E+04},
 {    4746, .8413776367188E+04, -.1331312011719E+04, .1002560485840E+04},
 {    4747, .8400432617188E+04, -.1346320190430E+04, .9946193237305E+03},
 {    4748, .8403958007812E+04, -.1346337890625E+04, .9930415039062E+03},
 {    4749, .8407480468750E+04, -.1346355468750E+04, .9914648437500E+03},
 {    4750, .8401631835938E+04, -.1331312011719E+04, .9940820312500E+03},
 {    4751, .8405110351562E+04, -.1331312011719E+04, .9925257568359E+03},
 {    4752, .8408587890625E+04, -.1331312011719E+04, .9909688720703E+03},
 {    4753, .8399136718750E+04, -.1346320190430E+04, .9917211303711E+03},
 {    4754, .8402660156250E+04, -.1346337890625E+04, .9901436767578E+03},
 {    4755, .8406183593750E+04, -.1346355468750E+04, .9885661010742E+03},
 {    4756, .8400334960938E+04, -.1331312011719E+04, .9911846923828E+03},
 {    4757, .8403811523438E+04, -.1331312011719E+04, .9896278076172E+03},
 {    4758, .8407289062500E+04, -.1331312011719E+04, .9880708007812E+03},
 {    4759, .8393945312500E+04, -.1346320190430E+04, .9801302490234E+03},
 {    4760, .8397470703125E+04, -.1346337890625E+04, .9785526123047E+03},
 {    4761, .8400995117188E+04, -.1346355468750E+04, .9769745483398E+03},
 {    4762, .8395144531250E+04, -.1331312011719E+04, .9795933837891E+03},
 {    4763, .8398622070312E+04, -.1331312011719E+04, .9780361328125E+03},
 {    4764, .8402100585938E+04, -.1331312011719E+04, .9764794921875E+03},
 {    4765, .8388756835938E+04, -.1346320190430E+04, .9685386962891E+03},
 {    4766, .8392280273438E+04, -.1346337890625E+04, .9669611206055E+03},
 {    4767, .8395807617188E+04, -.1346355468750E+04, .9653829345703E+03},
 {    4768, .8389957031250E+04, -.1331312011719E+04, .9680015869141E+03},
 {    4769, .8393433593750E+04, -.1331312011719E+04, .9664445190430E+03},
 {    4770, .8396912109375E+04, -.1331312011719E+04, .9648877563477E+03},
 {    4771, .8387458984375E+04, -.1346320190430E+04, .9656406250000E+03},
 {    4772, .8390984375000E+04, -.1346337890625E+04, .9640627441406E+03},
 {    4773, .8394507812500E+04, -.1346355468750E+04, .9624852905273E+03},
 {    4774, .8388660156250E+04, -.1331312011719E+04, .9651035156250E+03},
 {    4775, .8392136718750E+04, -.1331312011719E+04, .9635468139648E+03},
 {    4776, .8395613281250E+04, -.1331312011719E+04, .9619902343750E+03},
 {    4777, .8382271484375E+04, -.1346320190430E+04, .9540488891602E+03},
 {    4778, .8385794921875E+04, -.1346337890625E+04, .9524712524414E+03},
 {    4779, .8389321289062E+04, -.1346355468750E+04, .9508933105469E+03},
 {    4780, .8383468750000E+04, -.1331312011719E+04, .9535125122070E+03},
 {    4781, .8386948242188E+04, -.1331312011719E+04, .9519552612305E+03},
 {    4782, .8390424804688E+04, -.1331312011719E+04, .9503986206055E+03},
 {    4783, .8377081054688E+04, -.1346320190430E+04, .9424575195312E+03},
 {    4784, .8380605468750E+04, -.1346337890625E+04, .9408801269531E+03},
 {    4785, .8384129882812E+04, -.1346355468750E+04, .9393023681641E+03},
 {    4786, .8378281250000E+04, -.1331312011719E+04, .9419204711914E+03},
 {    4787, .8381758789062E+04, -.1331312011719E+04, .9403638305664E+03},
 {    4788, .8385236328125E+04, -.1331312011719E+04, .9388069458008E+03},
 {    4789, .8375783203125E+04, -.1346320190430E+04, .9395596923828E+03},
 {    4790, .8379309570312E+04, -.1346337890625E+04, .9379819335938E+03},
 {    4791, .8382831054688E+04, -.1346355468750E+04, .9364044189453E+03},
 {    4792, .8376984375000E+04, -.1331312011719E+04, .9390224609375E+03},
 {    4793, .8380461914062E+04, -.1331312011719E+04, .9374660034180E+03},
 {    4794, .8383940429688E+04, -.1331312011719E+04, .9359089965820E+03},
 {    4795, .8370595703125E+04, -.1346320190430E+04, .9279678344727E+03},
 {    4796, .8374120117188E+04, -.1346337890625E+04, .9263905029297E+03},
 {    4797, .8377643554688E+04, -.1346355590820E+04, .9248126831055E+03},
 {    4798, .8371793945312E+04, -.1331312011719E+04, .9274310302734E+03},
 {    4799, .8375271484375E+04, -.1331312011719E+04, .9258743896484E+03},
 {    4800, .8378748046875E+04, -.1331312011719E+04, .9243178710938E+03},
 {    4801, .8365405273438E+04, -.1346320190430E+04, .9163764648438E+03},
 {    4802, .8368930664062E+04, -.1346337890625E+04, .9147989501953E+03},
 {    4803, .8372456054688E+04, -.1346355590820E+04, .9132211914062E+03},
 {    4804, .8366604492188E+04, -.1331312011719E+04, .9158396606445E+03},
 {    4805, .8370084960938E+04, -.1331312011719E+04, .9142827148438E+03},
 {    4806, .8373558593750E+04, -.1331312011719E+04, .9127261962891E+03},
 {    4807, .8358142578125E+04, -.1346320190430E+04, .9001481933594E+03},
 {    4808, .8361666992188E+04, -.1346337890625E+04, .8985707397461E+03},
 {    4809, .8365190429688E+04, -.1346355590820E+04, .8969929199219E+03},
 {    4810, .8359339843750E+04, -.1331312011719E+04, .8996115722656E+03},
 {    4811, .8362818359375E+04, -.1331312011719E+04, .8980548706055E+03},
 {    4812, .8366295898438E+04, -.1331312011719E+04, .8964980468750E+03},
 {    4813, .8434066406250E+04, -.1326232055664E+04, .1066529052734E+04},
 {    4814, .8437540039062E+04, -.1326232055664E+04, .1064972778320E+04},
 {    4815, .8441018554688E+04, -.1326232055664E+04, .1063415771484E+04},
 {    4816, .8434066406250E+04, -.1321152099609E+04, .1066529052734E+04},
 {    4817, .8437540039062E+04, -.1321152099609E+04, .1064972778320E+04},
 {    4818, .8441018554688E+04, -.1321152099609E+04, .1063415771484E+04},
 {    4819, .8437540039062E+04, -.1316072021484E+04, .1064972778320E+04},
 {    4820, .8428875976562E+04, -.1326232055664E+04, .1054938110352E+04},
 {    4821, .8432352539062E+04, -.1326232055664E+04, .1053380981445E+04},
 {    4822, .8435829101562E+04, -.1326232055664E+04, .1051824462891E+04},
 {    4823, .8428875976562E+04, -.1321152099609E+04, .1054938110352E+04},
 {    4824, .8432352539062E+04, -.1321152099609E+04, .1053380981445E+04},
 {    4825, .8435829101562E+04, -.1321152099609E+04, .1051824462891E+04},
 {    4826, .8432352539062E+04, -.1316072021484E+04, .1053380981445E+04},
 {    4827, .8423685546875E+04, -.1326232055664E+04, .1043346679688E+04},
 {    4828, .8427162109375E+04, -.1326232055664E+04, .1041789916992E+04},
 {    4829, .8430640625000E+04, -.1326232055664E+04, .1040232788086E+04},
 {    4830, .8423685546875E+04, -.1321152099609E+04, .1043346679688E+04},
 {    4831, .8427162109375E+04, -.1321152099609E+04, .1041789916992E+04},
 {    4832, .8430640625000E+04, -.1321152099609E+04, .1040232788086E+04},
 {    4833, .8427162109375E+04, -.1316072021484E+04, .1041789916992E+04},
 {    4834, .8418497070312E+04, -.1326232055664E+04, .1031754394531E+04},
 {    4835, .8421972656250E+04, -.1326232055664E+04, .1030198242188E+04},
 {    4836, .8425451171875E+04, -.1326232055664E+04, .1028641723633E+04},
 {    4837, .8418497070312E+04, -.1321152099609E+04, .1031754394531E+04},
 {    4838, .8421972656250E+04, -.1321152099609E+04, .1030198242188E+04},
 {    4839, .8425451171875E+04, -.1321152099609E+04, .1028641723633E+04},
 {    4840, .8421972656250E+04, -.1316072021484E+04, .1030198242188E+04},
 {    4841, .8413306640625E+04, -.1326232055664E+04, .1020163574219E+04},
 {    4842, .8416784179688E+04, -.1326232055664E+04, .1018606567383E+04},
 {    4843, .8420261718750E+04, -.1326232055664E+04, .1017049621582E+04},
 {    4844, .8413306640625E+04, -.1321152099609E+04, .1020163574219E+04},
 {    4845, .8416784179688E+04, -.1321152099609E+04, .1018606567383E+04},
 {    4846, .8420261718750E+04, -.1321152099609E+04, .1017049621582E+04},
 {    4847, .8416784179688E+04, -.1316072021484E+04, .1018606567383E+04},
 {    4848, .8412009765625E+04, -.1326232055664E+04, .1017265747070E+04},
 {    4849, .8415489257812E+04, -.1326232055664E+04, .1015708374023E+04},
 {    4850, .8418965820312E+04, -.1326232055664E+04, .1014152038574E+04},
 {    4851, .8412009765625E+04, -.1321152099609E+04, .1017265747070E+04},
 {    4852, .8415489257812E+04, -.1321152099609E+04, .1015708374023E+04},
 {    4853, .8418965820312E+04, -.1321152099609E+04, .1014152038574E+04},
 {    4854, .8415489257812E+04, -.1316072021484E+04, .1015708374023E+04},
 {    4855, .8406821289062E+04, -.1326232055664E+04, .1005673706055E+04},
 {    4856, .8410297851562E+04, -.1326232055664E+04, .1004116821289E+04},
 {    4857, .8413776367188E+04, -.1326232055664E+04, .1002560485840E+04},
 {    4858, .8406821289062E+04, -.1321152099609E+04, .1005673706055E+04},
 {    4859, .8410297851562E+04, -.1321152099609E+04, .1004116821289E+04},
 {    4860, .8413776367188E+04, -.1321152099609E+04, .1002560485840E+04},
 {    4861, .8410297851562E+04, -.1316072021484E+04, .1004116821289E+04},
 {    4862, .8401631835938E+04, -.1326232055664E+04, .9940820312500E+03},
 {    4863, .8405110351562E+04, -.1326232055664E+04, .9925257568359E+03},
 {    4864, .8408587890625E+04, -.1326232055664E+04, .9909688720703E+03},
 {    4865, .8401631835938E+04, -.1321152099609E+04, .9940820312500E+03},
 {    4866, .8405110351562E+04, -.1321152099609E+04, .9925257568359E+03},
 {    4867, .8408587890625E+04, -.1321152099609E+04, .9909688720703E+03},
 {    4868, .8405110351562E+04, -.1316072021484E+04, .9925257568359E+03},
 {    4869, .8400334960938E+04, -.1326232055664E+04, .9911846923828E+03},
 {    4870, .8403811523438E+04, -.1326232055664E+04, .9896278076172E+03},
 {    4871, .8407289062500E+04, -.1326232055664E+04, .9880708007812E+03},
 {    4872, .8400334960938E+04, -.1321152099609E+04, .9911846923828E+03},
 {    4873, .8403811523438E+04, -.1321152099609E+04, .9896278076172E+03},
 {    4874, .8407289062500E+04, -.1321152099609E+04, .9880708007812E+03},
 {    4875, .8403811523438E+04, -.1316072021484E+04, .9896278076172E+03},
 {    4876, .8395144531250E+04, -.1326232055664E+04, .9795933837891E+03},
 {    4877, .8398622070312E+04, -.1326232055664E+04, .9780361328125E+03},
 {    4878, .8402100585938E+04, -.1326232055664E+04, .9764794921875E+03},
 {    4879, .8395144531250E+04, -.1321152099609E+04, .9795933837891E+03},
 {    4880, .8398622070312E+04, -.1321152099609E+04, .9780361328125E+03},
 {    4881, .8402100585938E+04, -.1321152099609E+04, .9764794921875E+03},
 {    4882, .8398622070312E+04, -.1316072021484E+04, .9780361328125E+03},
 {    4883, .8389957031250E+04, -.1326232055664E+04, .9680015869141E+03},
 {    4884, .8393433593750E+04, -.1326232055664E+04, .9664445190430E+03},
 {    4885, .8396912109375E+04, -.1326232055664E+04, .9648877563477E+03},
 {    4886, .8389957031250E+04, -.1321152099609E+04, .9680015869141E+03},
 {    4887, .8393433593750E+04, -.1321152099609E+04, .9664445190430E+03},
 {    4888, .8396912109375E+04, -.1321152099609E+04, .9648877563477E+03},
 {    4889, .8393433593750E+04, -.1316072021484E+04, .9664445190430E+03},
 {    4890, .8388660156250E+04, -.1326232055664E+04, .9651035156250E+03},
 {    4891, .8392136718750E+04, -.1326232055664E+04, .9635468139648E+03},
 {    4892, .8395613281250E+04, -.1326232055664E+04, .9619902343750E+03},
 {    4893, .8388660156250E+04, -.1321152099609E+04, .9651035156250E+03},
 {    4894, .8392136718750E+04, -.1321152099609E+04, .9635468139648E+03},
 {    4895, .8395613281250E+04, -.1321152099609E+04, .9619902343750E+03},
 {    4896, .8392136718750E+04, -.1316072021484E+04, .9635468139648E+03},
 {    4897, .8383468750000E+04, -.1326232055664E+04, .9535125122070E+03},
 {    4898, .8386948242188E+04, -.1326232055664E+04, .9519552612305E+03},
 {    4899, .8390424804688E+04, -.1326232055664E+04, .9503986206055E+03},
 {    4900, .8383468750000E+04, -.1321152099609E+04, .9535125122070E+03},
 {    4901, .8386948242188E+04, -.1321152099609E+04, .9519552612305E+03},
 {    4902, .8390424804688E+04, -.1321152099609E+04, .9503986206055E+03},
 {    4903, .8386948242188E+04, -.1316072021484E+04, .9519552612305E+03},
 {    4904, .8378281250000E+04, -.1326232055664E+04, .9419204711914E+03},
 {    4905, .8381758789062E+04, -.1326232055664E+04, .9403638305664E+03},
 {    4906, .8385236328125E+04, -.1326232055664E+04, .9388069458008E+03},
 {    4907, .8378281250000E+04, -.1321152099609E+04, .9419205932617E+03},
 {    4908, .8381758789062E+04, -.1321152099609E+04, .9403638305664E+03},
 {    4909, .8385236328125E+04, -.1321152099609E+04, .9388069458008E+03},
 {    4910, .8381758789062E+04, -.1316072021484E+04, .9403638305664E+03},
 {    4911, .8376984375000E+04, -.1326232055664E+04, .9390224609375E+03},
 {    4912, .8380461914062E+04, -.1326232055664E+04, .9374660034180E+03},
 {    4913, .8383940429688E+04, -.1326232055664E+04, .9359089965820E+03},
 {    4914, .8376984375000E+04, -.1321152099609E+04, .9390224609375E+03},
 {    4915, .8380461914062E+04, -.1321152099609E+04, .9374660034180E+03},
 {    4916, .8383940429688E+04, -.1321152099609E+04, .9359089965820E+03},
 {    4917, .8380461914062E+04, -.1316072021484E+04, .9374660034180E+03},
 {    4918, .8371793945312E+04, -.1326232055664E+04, .9274310302734E+03},
 {    4919, .8375271484375E+04, -.1326232055664E+04, .9258743896484E+03},
 {    4920, .8378748046875E+04, -.1326232055664E+04, .9243178710938E+03},
 {    4921, .8371793945312E+04, -.1321152099609E+04, .9274310302734E+03},
 {    4922, .8375271484375E+04, -.1321152099609E+04, .9258743896484E+03},
 {    4923, .8378748046875E+04, -.1321152099609E+04, .9243178710938E+03},
 {    4924, .8375271484375E+04, -.1316072021484E+04, .9258743896484E+03},
 {    4925, .8366604492188E+04, -.1326232055664E+04, .9158396606445E+03},
 {    4926, .8370084960938E+04, -.1326232055664E+04, .9142827148438E+03},
 {    4927, .8373558593750E+04, -.1326232055664E+04, .9127261962891E+03},
 {    4928, .8366604492188E+04, -.1321152099609E+04, .9158396606445E+03},
 {    4929, .8370084960938E+04, -.1321152099609E+04, .9142827148438E+03},
 {    4930, .8373558593750E+04, -.1321152099609E+04, .9127261962891E+03},
 {    4931, .8370084960938E+04, -.1316072021484E+04, .9142827148438E+03},
 {    4932, .8437540039062E+04, -.1300832031250E+04, .1064972778320E+04},
 {    4933, .8437540039062E+04, -.1285592041016E+04, .1064972778320E+04},
 {    4934, .8437540039062E+04, -.1270352050781E+04, .1064972778320E+04},
 {    4935, .8437540039062E+04, -.1255112060547E+04, .1064972778320E+04},
 {    4936, .8432352539062E+04, -.1300832031250E+04, .1053380981445E+04},
 {    4937, .8432352539062E+04, -.1285592041016E+04, .1053380981445E+04},
 {    4938, .8432352539062E+04, -.1270352050781E+04, .1053380981445E+04},
 {    4939, .8432352539062E+04, -.1255112060547E+04, .1053380981445E+04},
 {    4940, .8427162109375E+04, -.1300832031250E+04, .1041789916992E+04},
 {    4941, .8427162109375E+04, -.1285592041016E+04, .1041789916992E+04},
 {    4942, .8427162109375E+04, -.1270352050781E+04, .1041789916992E+04},
 {    4943, .8427162109375E+04, -.1255112060547E+04, .1041789916992E+04},
 {    4944, .8421972656250E+04, -.1300832031250E+04, .1030198242188E+04},
 {    4945, .8421972656250E+04, -.1285592041016E+04, .1030198242188E+04},
 {    4946, .8421972656250E+04, -.1270352050781E+04, .1030198242188E+04},
 {    4947, .8421972656250E+04, -.1255112060547E+04, .1030198242188E+04},
 {    4948, .8416784179688E+04, -.1300832031250E+04, .1018606567383E+04},
 {    4949, .8416784179688E+04, -.1285592041016E+04, .1018606567383E+04},
 {    4950, .8416784179688E+04, -.1270352050781E+04, .1018606567383E+04},
 {    4951, .8416784179688E+04, -.1255112060547E+04, .1018606567383E+04},
 {    4952, .8415489257812E+04, -.1300832031250E+04, .1015708374023E+04},
 {    4953, .8415489257812E+04, -.1285592041016E+04, .1015708374023E+04},
 {    4954, .8415489257812E+04, -.1270352050781E+04, .1015708374023E+04},
 {    4955, .8415489257812E+04, -.1255112060547E+04, .1015708374023E+04},
 {    4956, .8410297851562E+04, -.1300832031250E+04, .1004116821289E+04},
 {    4957, .8410297851562E+04, -.1285592041016E+04, .1004116821289E+04},
 {    4958, .8410297851562E+04, -.1270352050781E+04, .1004116821289E+04},
 {    4959, .8410297851562E+04, -.1255112060547E+04, .1004116821289E+04},
 {    4960, .8405110351562E+04, -.1300832031250E+04, .9925257568359E+03},
 {    4961, .8405110351562E+04, -.1285592041016E+04, .9925257568359E+03},
 {    4962, .8405110351562E+04, -.1270352050781E+04, .9925257568359E+03},
 {    4963, .8405110351562E+04, -.1255112060547E+04, .9925257568359E+03},
 {    4964, .8403811523438E+04, -.1300832031250E+04, .9896278076172E+03},
 {    4965, .8403811523438E+04, -.1285592041016E+04, .9896278076172E+03},
 {    4966, .8403811523438E+04, -.1270352050781E+04, .9896278076172E+03},
 {    4967, .8403811523438E+04, -.1255112060547E+04, .9896278076172E+03},
 {    4968, .8398622070312E+04, -.1300832031250E+04, .9780361328125E+03},
 {    4969, .8398622070312E+04, -.1285592041016E+04, .9780361328125E+03},
 {    4970, .8398622070312E+04, -.1270352050781E+04, .9780361328125E+03},
 {    4971, .8398622070312E+04, -.1255112060547E+04, .9780361328125E+03},
 {    4972, .8393433593750E+04, -.1300832031250E+04, .9664445190430E+03},
 {    4973, .8393433593750E+04, -.1285592041016E+04, .9664445190430E+03},
 {    4974, .8393433593750E+04, -.1270352050781E+04, .9664445190430E+03},
 {    4975, .8393433593750E+04, -.1255112060547E+04, .9664445190430E+03},
 {    4976, .8392136718750E+04, -.1300832031250E+04, .9635468139648E+03},
 {    4977, .8392136718750E+04, -.1285592041016E+04, .9635468139648E+03},
 {    4978, .8392136718750E+04, -.1270352050781E+04, .9635468139648E+03},
 {    4979, .8392136718750E+04, -.1255112060547E+04, .9635468139648E+03},
 {    4980, .8386948242188E+04, -.1300832031250E+04, .9519552612305E+03},
 {    4981, .8386948242188E+04, -.1285592041016E+04, .9519552612305E+03},
 {    4982, .8386948242188E+04, -.1270352050781E+04, .9519552612305E+03},
 {    4983, .8386948242188E+04, -.1255112060547E+04, .9519552612305E+03},
 {    4984, .8381758789062E+04, -.1300832031250E+04, .9403638305664E+03},
 {    4985, .8381758789062E+04, -.1285592041016E+04, .9403638305664E+03},
 {    4986, .8381758789062E+04, -.1270352050781E+04, .9403638305664E+03},
 {    4987, .8381758789062E+04, -.1255112060547E+04, .9403638305664E+03},
 {    4988, .8380461914062E+04, -.1300832031250E+04, .9374660034180E+03},
 {    4989, .8380461914062E+04, -.1285592041016E+04, .9374660034180E+03},
 {    4990, .8380461914062E+04, -.1270352050781E+04, .9374660034180E+03},
 {    4991, .8380461914062E+04, -.1255112060547E+04, .9374660034180E+03},
 {    4992, .8375271484375E+04, -.1300832031250E+04, .9258743896484E+03},
 {    4993, .8375271484375E+04, -.1285592041016E+04, .9258743896484E+03},
 {    4994, .8375271484375E+04, -.1270352050781E+04, .9258743896484E+03},
 {    4995, .8375271484375E+04, -.1255112060547E+04, .9258743896484E+03},
 {    4996, .8370084960938E+04, -.1300832031250E+04, .9142827148438E+03},
 {    4997, .8370084960938E+04, -.1285592041016E+04, .9142827148438E+03},
 {    4998, .8370084960938E+04, -.1270352050781E+04, .9142827148438E+03},
 {    4999, .8370084960938E+04, -.1255112060547E+04, .9142827148438E+03},
 {    5000, .8434066406250E+04, -.1250031982422E+04, .1066529052734E+04},
 {    5001, .8437540039062E+04, -.1250031982422E+04, .1064972778320E+04},
 {    5002, .8441018554688E+04, -.1250031982422E+04, .1063415771484E+04},
 {    5003, .8434066406250E+04, -.1244952026367E+04, .1066529052734E+04},
 {    5004, .8437540039062E+04, -.1244952026367E+04, .1064972778320E+04},
 {    5005, .8441018554688E+04, -.1244952026367E+04, .1063415771484E+04},
 {    5006, .8434066406250E+04, -.1239872070312E+04, .1066529052734E+04},
 {    5007, .8437540039062E+04, -.1239872070312E+04, .1064972778320E+04},
 {    5008, .8441018554688E+04, -.1239872070312E+04, .1063415771484E+04},
 {    5009, .8428875976562E+04, -.1250031982422E+04, .1054938110352E+04},
 {    5010, .8432352539062E+04, -.1250031982422E+04, .1053380981445E+04},
 {    5011, .8435829101562E+04, -.1250031982422E+04, .1051824462891E+04},
 {    5012, .8428875976562E+04, -.1244952026367E+04, .1054938110352E+04},
 {    5013, .8432352539062E+04, -.1244952026367E+04, .1053380981445E+04},
 {    5014, .8435829101562E+04, -.1244952026367E+04, .1051824462891E+04},
 {    5015, .8428875976562E+04, -.1239872070312E+04, .1054938110352E+04},
 {    5016, .8432352539062E+04, -.1239872070312E+04, .1053380981445E+04},
 {    5017, .8435829101562E+04, -.1239872070312E+04, .1051824462891E+04},
 {    5018, .8423685546875E+04, -.1250031982422E+04, .1043346679688E+04},
 {    5019, .8427162109375E+04, -.1250031982422E+04, .1041789916992E+04},
 {    5020, .8430640625000E+04, -.1250031982422E+04, .1040232788086E+04},
 {    5021, .8423685546875E+04, -.1244952026367E+04, .1043346679688E+04},
 {    5022, .8427162109375E+04, -.1244952026367E+04, .1041789916992E+04},
 {    5023, .8430640625000E+04, -.1244952026367E+04, .1040232788086E+04},
 {    5024, .8423685546875E+04, -.1239872070312E+04, .1043346679688E+04},
 {    5025, .8427162109375E+04, -.1239872070312E+04, .1041789916992E+04},
 {    5026, .8430640625000E+04, -.1239872070312E+04, .1040232788086E+04},
 {    5027, .8418497070312E+04, -.1250031982422E+04, .1031754394531E+04},
 {    5028, .8421972656250E+04, -.1250031982422E+04, .1030198242188E+04},
 {    5029, .8425451171875E+04, -.1250031982422E+04, .1028641723633E+04},
 {    5030, .8418497070312E+04, -.1244952026367E+04, .1031754394531E+04},
 {    5031, .8421972656250E+04, -.1244952026367E+04, .1030198242188E+04},
 {    5032, .8425451171875E+04, -.1244952026367E+04, .1028641723633E+04},
 {    5033, .8418497070312E+04, -.1239872070312E+04, .1031754394531E+04},
 {    5034, .8421972656250E+04, -.1239872070312E+04, .1030198242188E+04},
 {    5035, .8425451171875E+04, -.1239872070312E+04, .1028641723633E+04},
 {    5036, .8413306640625E+04, -.1250031982422E+04, .1020163574219E+04},
 {    5037, .8416784179688E+04, -.1250031982422E+04, .1018606567383E+04},
 {    5038, .8420261718750E+04, -.1250031982422E+04, .1017049621582E+04},
 {    5039, .8413306640625E+04, -.1244952026367E+04, .1020163574219E+04},
 {    5040, .8416784179688E+04, -.1244952026367E+04, .1018606567383E+04},
 {    5041, .8420261718750E+04, -.1244952026367E+04, .1017049621582E+04},
 {    5042, .8413306640625E+04, -.1239872070312E+04, .1020163574219E+04},
 {    5043, .8416784179688E+04, -.1239872070312E+04, .1018606567383E+04},
 {    5044, .8420261718750E+04, -.1239872070312E+04, .1017049621582E+04},
 {    5045, .8412009765625E+04, -.1250031982422E+04, .1017265747070E+04},
 {    5046, .8415489257812E+04, -.1250031982422E+04, .1015708374023E+04},
 {    5047, .8418965820312E+04, -.1250031982422E+04, .1014152038574E+04},
 {    5048, .8412009765625E+04, -.1244952026367E+04, .1017265747070E+04},
 {    5049, .8415489257812E+04, -.1244952026367E+04, .1015708374023E+04},
 {    5050, .8418965820312E+04, -.1244952026367E+04, .1014152038574E+04},
 {    5051, .8412009765625E+04, -.1239872070312E+04, .1017265747070E+04},
 {    5052, .8415489257812E+04, -.1239872070312E+04, .1015708374023E+04},
 {    5053, .8418965820312E+04, -.1239872070312E+04, .1014152038574E+04},
 {    5054, .8406821289062E+04, -.1250031982422E+04, .1005673706055E+04},
 {    5055, .8410297851562E+04, -.1250031982422E+04, .1004116821289E+04},
 {    5056, .8413776367188E+04, -.1250031982422E+04, .1002560485840E+04},
 {    5057, .8406821289062E+04, -.1244952026367E+04, .1005673706055E+04},
 {    5058, .8410297851562E+04, -.1244952026367E+04, .1004116821289E+04},
 {    5059, .8413776367188E+04, -.1244952026367E+04, .1002560485840E+04},
 {    5060, .8406821289062E+04, -.1239872070312E+04, .1005673706055E+04},
 {    5061, .8410297851562E+04, -.1239872070312E+04, .1004116821289E+04},
 {    5062, .8413776367188E+04, -.1239872070312E+04, .1002560485840E+04},
 {    5063, .8401631835938E+04, -.1250031982422E+04, .9940820312500E+03},
 {    5064, .8405110351562E+04, -.1250031982422E+04, .9925257568359E+03},
 {    5065, .8408587890625E+04, -.1250031982422E+04, .9909688720703E+03},
 {    5066, .8401631835938E+04, -.1244952026367E+04, .9940820312500E+03},
 {    5067, .8405110351562E+04, -.1244952026367E+04, .9925257568359E+03},
 {    5068, .8408587890625E+04, -.1244952026367E+04, .9909688720703E+03},
 {    5069, .8401631835938E+04, -.1239872070312E+04, .9940820312500E+03},
 {    5070, .8405110351562E+04, -.1239872070312E+04, .9925257568359E+03},
 {    5071, .8408587890625E+04, -.1239872070312E+04, .9909688720703E+03},
 {    5072, .8400334960938E+04, -.1250031982422E+04, .9911846923828E+03},
 {    5073, .8403811523438E+04, -.1250031982422E+04, .9896278076172E+03},
 {    5074, .8407289062500E+04, -.1250031982422E+04, .9880708007812E+03},
 {    5075, .8400334960938E+04, -.1244952026367E+04, .9911846923828E+03},
 {    5076, .8403811523438E+04, -.1244952026367E+04, .9896278076172E+03},
 {    5077, .8407289062500E+04, -.1244952026367E+04, .9880708007812E+03},
 {    5078, .8400334960938E+04, -.1239872070312E+04, .9911846923828E+03},
 {    5079, .8403811523438E+04, -.1239872070312E+04, .9896278076172E+03},
 {    5080, .8407289062500E+04, -.1239872070312E+04, .9880708007812E+03},
 {    5081, .8395144531250E+04, -.1250031982422E+04, .9795933837891E+03},
 {    5082, .8398622070312E+04, -.1250031982422E+04, .9780361328125E+03},
 {    5083, .8402100585938E+04, -.1250031982422E+04, .9764794921875E+03},
 {    5084, .8395144531250E+04, -.1244952026367E+04, .9795933837891E+03},
 {    5085, .8398622070312E+04, -.1244952026367E+04, .9780361328125E+03},
 {    5086, .8402100585938E+04, -.1244952026367E+04, .9764794921875E+03},
 {    5087, .8395144531250E+04, -.1239872070312E+04, .9795933837891E+03},
 {    5088, .8398622070312E+04, -.1239872070312E+04, .9780361328125E+03},
 {    5089, .8402100585938E+04, -.1239872070312E+04, .9764794921875E+03},
 {    5090, .8389957031250E+04, -.1250031982422E+04, .9680015869141E+03},
 {    5091, .8393433593750E+04, -.1250031982422E+04, .9664445190430E+03},
 {    5092, .8396912109375E+04, -.1250031982422E+04, .9648877563477E+03},
 {    5093, .8389957031250E+04, -.1244952026367E+04, .9680015869141E+03},
 {    5094, .8393433593750E+04, -.1244952026367E+04, .9664445190430E+03},
 {    5095, .8396912109375E+04, -.1244952026367E+04, .9648877563477E+03},
 {    5096, .8389957031250E+04, -.1239872070312E+04, .9680015869141E+03},
 {    5097, .8393433593750E+04, -.1239872070312E+04, .9664445190430E+03},
 {    5098, .8396912109375E+04, -.1239872070312E+04, .9648877563477E+03},
 {    5099, .8388660156250E+04, -.1250031982422E+04, .9651035156250E+03},
 {    5100, .8392136718750E+04, -.1250031982422E+04, .9635468139648E+03},
 {    5101, .8395613281250E+04, -.1250031982422E+04, .9619902343750E+03},
 {    5102, .8388660156250E+04, -.1244952026367E+04, .9651035156250E+03},
 {    5103, .8392136718750E+04, -.1244952026367E+04, .9635468139648E+03},
 {    5104, .8395613281250E+04, -.1244952026367E+04, .9619902343750E+03},
 {    5105, .8388660156250E+04, -.1239872070312E+04, .9651035156250E+03},
 {    5106, .8392136718750E+04, -.1239872070312E+04, .9635468139648E+03},
 {    5107, .8395613281250E+04, -.1239872070312E+04, .9619902343750E+03},
 {    5108, .8383468750000E+04, -.1250031982422E+04, .9535125122070E+03},
 {    5109, .8386948242188E+04, -.1250031982422E+04, .9519552612305E+03},
 {    5110, .8390424804688E+04, -.1250031982422E+04, .9503986206055E+03},
 {    5111, .8383468750000E+04, -.1244952026367E+04, .9535125122070E+03},
 {    5112, .8386948242188E+04, -.1244952026367E+04, .9519552612305E+03},
 {    5113, .8390424804688E+04, -.1244952026367E+04, .9503986206055E+03},
 {    5114, .8383468750000E+04, -.1239872070312E+04, .9535125122070E+03},
 {    5115, .8386948242188E+04, -.1239872070312E+04, .9519552612305E+03},
 {    5116, .8390424804688E+04, -.1239872070312E+04, .9503986206055E+03},
 {    5117, .8378281250000E+04, -.1250031982422E+04, .9419205932617E+03},
 {    5118, .8381758789062E+04, -.1250031982422E+04, .9403638305664E+03},
 {    5119, .8385236328125E+04, -.1250031982422E+04, .9388069458008E+03},
 {    5120, .8378281250000E+04, -.1244952026367E+04, .9419204711914E+03},
 {    5121, .8381758789062E+04, -.1244952026367E+04, .9403638305664E+03},
 {    5122, .8385236328125E+04, -.1244952026367E+04, .9388069458008E+03},
 {    5123, .8378281250000E+04, -.1239872070312E+04, .9419204711914E+03},
 {    5124, .8381758789062E+04, -.1239872070312E+04, .9403638305664E+03},
 {    5125, .8385236328125E+04, -.1239872070312E+04, .9388069458008E+03},
 {    5126, .8376984375000E+04, -.1250031982422E+04, .9390224609375E+03},
 {    5127, .8380461914062E+04, -.1250031982422E+04, .9374660034180E+03},
 {    5128, .8383940429688E+04, -.1250031982422E+04, .9359089965820E+03},
 {    5129, .8376984375000E+04, -.1244952026367E+04, .9390224609375E+03},
 {    5130, .8380461914062E+04, -.1244952026367E+04, .9374660034180E+03},
 {    5131, .8383940429688E+04, -.1244952026367E+04, .9359089965820E+03},
 {    5132, .8376984375000E+04, -.1239872070312E+04, .9390224609375E+03},
 {    5133, .8380461914062E+04, -.1239872070312E+04, .9374660034180E+03},
 {    5134, .8383940429688E+04, -.1239872070312E+04, .9359089965820E+03},
 {    5135, .8371793945312E+04, -.1250031982422E+04, .9274310302734E+03},
 {    5136, .8375271484375E+04, -.1250031982422E+04, .9258743896484E+03},
 {    5137, .8378748046875E+04, -.1250031982422E+04, .9243178710938E+03},
 {    5138, .8371793945312E+04, -.1244952026367E+04, .9274310302734E+03},
 {    5139, .8375271484375E+04, -.1244952026367E+04, .9258743896484E+03},
 {    5140, .8378748046875E+04, -.1244952026367E+04, .9243178710938E+03},
 {    5141, .8371793945312E+04, -.1239872070312E+04, .9274310302734E+03},
 {    5142, .8375271484375E+04, -.1239872070312E+04, .9258743896484E+03},
 {    5143, .8378748046875E+04, -.1239872070312E+04, .9243178710938E+03},
 {    5144, .8366604492188E+04, -.1250031982422E+04, .9158396606445E+03},
 {    5145, .8370084960938E+04, -.1250031982422E+04, .9142827148438E+03},
 {    5146, .8373558593750E+04, -.1250031982422E+04, .9127261962891E+03},
 {    5147, .8366604492188E+04, -.1244952026367E+04, .9158396606445E+03},
 {    5148, .8370084960938E+04, -.1244952026367E+04, .9142827148438E+03},
 {    5149, .8373558593750E+04, -.1244952026367E+04, .9127261962891E+03},
 {    5150, .8366604492188E+04, -.1239872070312E+04, .9158396606445E+03},
 {    5151, .8370084960938E+04, -.1239872070312E+04, .9142827148438E+03},
 {    5152, .8373558593750E+04, -.1239872070312E+04, .9127261962891E+03},
 {    5153, .8432866210938E+04, -.1224863891602E+04, .1067066284180E+04},
 {    5154, .8436389648438E+04, -.1224846191406E+04, .1065488525391E+04},
 {    5155, .8439912109375E+04, -.1224828613281E+04, .1063911132812E+04},
 {    5156, .8427674804688E+04, -.1224863891602E+04, .1055474365234E+04},
 {    5157, .8431199218750E+04, -.1224846191406E+04, .1053896972656E+04},
 {    5158, .8434723632812E+04, -.1224828613281E+04, .1052319702148E+04},
 {    5159, .8422486328125E+04, -.1224863891602E+04, .1043883300781E+04},
 {    5160, .8426010742188E+04, -.1224846191406E+04, .1042305541992E+04},
 {    5161, .8429534179688E+04, -.1224828613281E+04, .1040727661133E+04},
 {    5162, .8417297851562E+04, -.1224863891602E+04, .1032291748047E+04},
 {    5163, .8420820312500E+04, -.1224846191406E+04, .1030714355469E+04},
 {    5164, .8424345703125E+04, -.1224828613281E+04, .1029136962891E+04},
 {    5165, .8412106445312E+04, -.1224863891602E+04, .1020699951172E+04},
 {    5166, .8415631835938E+04, -.1224846191406E+04, .1019122680664E+04},
 {    5167, .8419157226562E+04, -.1224828613281E+04, .1017545288086E+04},
 {    5168, .8410809570312E+04, -.1224863891602E+04, .1017802490234E+04},
 {    5169, .8414334960938E+04, -.1224846191406E+04, .1016224731445E+04},
 {    5170, .8417860351562E+04, -.1224828613281E+04, .1014647338867E+04},
 {    5171, .8405622070312E+04, -.1224863891602E+04, .1006210815430E+04},
 {    5172, .8409144531250E+04, -.1224846191406E+04, .1004633422852E+04},
 {    5173, .8412669921875E+04, -.1224828613281E+04, .1003055786133E+04},
 {    5174, .8400432617188E+04, -.1224863891602E+04, .9946193237305E+03},
 {    5175, .8403958007812E+04, -.1224846191406E+04, .9930415039062E+03},
 {    5176, .8407480468750E+04, -.1224828613281E+04, .9914648437500E+03},
 {    5177, .8399136718750E+04, -.1224863891602E+04, .9917211303711E+03},
 {    5178, .8402660156250E+04, -.1224846191406E+04, .9901436767578E+03},
 {    5179, .8406183593750E+04, -.1224828613281E+04, .9885661010742E+03},
 {    5180, .8393945312500E+04, -.1224863891602E+04, .9801302490234E+03},
 {    5181, .8397470703125E+04, -.1224846191406E+04, .9785526123047E+03},
 {    5182, .8400995117188E+04, -.1224828613281E+04, .9769745483398E+03},
 {    5183, .8388756835938E+04, -.1224863891602E+04, .9685386962891E+03},
 {    5184, .8392280273438E+04, -.1224846191406E+04, .9669611206055E+03},
 {    5185, .8395807617188E+04, -.1224828613281E+04, .9653829345703E+03},
 {    5186, .8387458984375E+04, -.1224863891602E+04, .9656406250000E+03},
 {    5187, .8390984375000E+04, -.1224846191406E+04, .9640627441406E+03},
 {    5188, .8394507812500E+04, -.1224828613281E+04, .9624852905273E+03},
 {    5189, .8382271484375E+04, -.1224863891602E+04, .9540488891602E+03},
 {    5190, .8385794921875E+04, -.1224846191406E+04, .9524712524414E+03},
 {    5191, .8389321289062E+04, -.1224828613281E+04, .9508933105469E+03},
 {    5192, .8377081054688E+04, -.1224863891602E+04, .9424575195312E+03},
 {    5193, .8380605468750E+04, -.1224846191406E+04, .9408801269531E+03},
 {    5194, .8384129882812E+04, -.1224828613281E+04, .9393023681641E+03},
 {    5195, .8375783203125E+04, -.1224863891602E+04, .9395596923828E+03},
 {    5196, .8379309570312E+04, -.1224846191406E+04, .9379819335938E+03},
 {    5197, .8382831054688E+04, -.1224828613281E+04, .9364044189453E+03},
 {    5198, .8370595703125E+04, -.1224863891602E+04, .9279678344727E+03},
 {    5199, .8374120117188E+04, -.1224846313477E+04, .9263905029297E+03},
 {    5200, .8377643554688E+04, -.1224828613281E+04, .9248126831055E+03},
 {    5201, .8365405273438E+04, -.1224863891602E+04, .9163764648438E+03},
 {    5202, .8368930664062E+04, -.1224846313477E+04, .9147989501953E+03},
 {    5203, .8372456054688E+04, -.1224828613281E+04, .9132211914062E+03},
 {    5204, .8359339843750E+04, -.1239872070312E+04, .8996115722656E+03},
 {    5205, .8362818359375E+04, -.1239872070312E+04, .8980548706055E+03},
 {    5206, .8366295898438E+04, -.1239872070312E+04, .8964980468750E+03},
 {    5207, .8358142578125E+04, -.1224863891602E+04, .9001481933594E+03},
 {    5208, .8361666992188E+04, -.1224846313477E+04, .8985707397461E+03},
 {    5209, .8365190429688E+04, -.1224828613281E+04, .8969929199219E+03},
 {    5210, .8417889648438E+04, -.1239872070312E+04, .1045940917969E+04},
 {    5211, .8416601562500E+04, -.1224901245117E+04, .1046518066406E+04},
 {    5212, .8412700195312E+04, -.1239872070312E+04, .1034349243164E+04},
 {    5213, .8411412109375E+04, -.1224901245117E+04, .1034926757812E+04},
 {    5214, .8407511718750E+04, -.1239872070312E+04, .1022758117676E+04},
 {    5215, .8406222656250E+04, -.1224901245117E+04, .1023335388184E+04},
 {    5216, .8406216796875E+04, -.1239872070312E+04, .1019859497070E+04},
 {    5217, .8404924804688E+04, -.1224901245117E+04, .1020437500000E+04},
 {    5218, .8401024414062E+04, -.1239872070312E+04, .1008268310547E+04},
 {    5219, .8399735351562E+04, -.1224901245117E+04, .1008845825195E+04},
 {    5220, .8395836914062E+04, -.1239872070312E+04, .9966770019531E+03},
 {    5221, .8394545898438E+04, -.1224901245117E+04, .9972546386719E+03},
 {    5222, .8394540039062E+04, -.1239872070312E+04, .9937788085938E+03},
 {    5223, .8393249023438E+04, -.1224901245117E+04, .9943563232422E+03},
 {    5224, .8389348632812E+04, -.1239872070312E+04, .9821878051758E+03},
 {    5225, .8388059570312E+04, -.1224901245117E+04, .9827647094727E+03},
 {    5226, .8384160156250E+04, -.1239872070312E+04, .9705960693359E+03},
 {    5227, .8382872070312E+04, -.1224901245117E+04, .9711729736328E+03},
 {    5228, .8382863281250E+04, -.1239872070312E+04, .9676982421875E+03},
 {    5229, .8381574218750E+04, -.1224901245117E+04, .9682755126953E+03},
 {    5230, .8377673828125E+04, -.1239872070312E+04, .9561064453125E+03},
 {    5231, .8376383789062E+04, -.1224901245117E+04, .9566840820312E+03},
 {    5232, .8372485351562E+04, -.1239872070312E+04, .9445149536133E+03},
 {    5233, .8371196289062E+04, -.1224901245117E+04, .9450922241211E+03},
 {    5234, .8371188476562E+04, -.1239872070312E+04, .9416169433594E+03},
 {    5235, .8369897460938E+04, -.1224901245117E+04, .9421945190430E+03},
 {    5236, .8365998046875E+04, -.1239872070312E+04, .9300255126953E+03},
 {    5237, .8364712890625E+04, -.1224901245117E+04, .9306029052734E+03},
 {    5238, .8360809570312E+04, -.1239872070312E+04, .9184338378906E+03},
 {    5239, .8359521484375E+04, -.1224901245117E+04, .9190114135742E+03},
 {    5240, .8417889648438E+04, -.1331312011719E+04, .1045940917969E+04},
 {    5241, .8416601562500E+04, -.1346282836914E+04, .1046518066406E+04},
 {    5242, .8412700195312E+04, -.1331312011719E+04, .1034349487305E+04},
 {    5243, .8411412109375E+04, -.1346282836914E+04, .1034926757812E+04},
 {    5244, .8407511718750E+04, -.1331312011719E+04, .1022758117676E+04},
 {    5245, .8406222656250E+04, -.1346282836914E+04, .1023335388184E+04},
 {    5246, .8406216796875E+04, -.1331312011719E+04, .1019859497070E+04},
 {    5247, .8404924804688E+04, -.1346282836914E+04, .1020437744141E+04},
 {    5248, .8401024414062E+04, -.1331312011719E+04, .1008268310547E+04},
 {    5249, .8399735351562E+04, -.1346282836914E+04, .1008845825195E+04},
 {    5250, .8395836914062E+04, -.1331312011719E+04, .9966770019531E+03},
 {    5251, .8394545898438E+04, -.1346282836914E+04, .9972546386719E+03},
 {    5252, .8394540039062E+04, -.1331312011719E+04, .9937788085938E+03},
 {    5253, .8393249023438E+04, -.1346282836914E+04, .9943565673828E+03},
 {    5254, .8389348632812E+04, -.1331312011719E+04, .9821878051758E+03},
 {    5255, .8388059570312E+04, -.1346282836914E+04, .9827649536133E+03},
 {    5256, .8384160156250E+04, -.1331312011719E+04, .9705960693359E+03},
 {    5257, .8382872070312E+04, -.1346282836914E+04, .9711729736328E+03},
 {    5258, .8382863281250E+04, -.1331312011719E+04, .9676982421875E+03},
 {    5259, .8381574218750E+04, -.1346282836914E+04, .9682755126953E+03},
 {    5260, .8377673828125E+04, -.1331312011719E+04, .9561064453125E+03},
 {    5261, .8376383789062E+04, -.1346282836914E+04, .9566840820312E+03},
 {    5262, .8372485351562E+04, -.1331312011719E+04, .9445149536133E+03},
 {    5263, .8371196289062E+04, -.1346282836914E+04, .9450922241211E+03},
 {    5264, .8371188476562E+04, -.1331312011719E+04, .9416169433594E+03},
 {    5265, .8369897460938E+04, -.1346282836914E+04, .9421945190430E+03},
 {    5266, .8365998046875E+04, -.1331312011719E+04, .9300255126953E+03},
 {    5267, .8364712890625E+04, -.1346282958984E+04, .9306029052734E+03},
 {    5268, .8360809570312E+04, -.1331312011719E+04, .9184338378906E+03},
 {    5269, .8359521484375E+04, -.1346282958984E+04, .9190114135742E+03},
 {    5270, .8358530273438E+04, -.1159862060547E+04, .1078079589844E+04},
 {    5271, .8352735351562E+04, -.1159862060547E+04, .1080674316406E+04},
 {    5272, .8346940429688E+04, -.1159862060547E+04, .1083268554688E+04},
 {    5273, .8341141601562E+04, -.1159862060547E+04, .1085863403320E+04},
 {    5274, .8335346679688E+04, -.1159862060547E+04, .1088457763672E+04},
 {    5275, .8329552734375E+04, -.1159862060547E+04, .1091052490234E+04},
 {    5276, .8323755859375E+04, -.1159862060547E+04, .1093646728516E+04},
 {    5277, .8317958984375E+04, -.1159862060547E+04, .1096241699219E+04},
 {    5278, .8312165039062E+04, -.1159862060547E+04, .1098835937500E+04},
 {    5279, .8358530273438E+04, -.1153512084961E+04, .1078079589844E+04},
 {    5280, .8352735351562E+04, -.1153512084961E+04, .1080674316406E+04},
 {    5281, .8346940429688E+04, -.1153512084961E+04, .1083268554688E+04},
 {    5282, .8341141601562E+04, -.1153512084961E+04, .1085863403320E+04},
 {    5283, .8335346679688E+04, -.1153512084961E+04, .1088457763672E+04},
 {    5284, .8329552734375E+04, -.1153512084961E+04, .1091052490234E+04},
 {    5285, .8323755859375E+04, -.1153512084961E+04, .1093646728516E+04},
 {    5286, .8317958984375E+04, -.1153512084961E+04, .1096241699219E+04},
 {    5287, .8312165039062E+04, -.1153512084961E+04, .1098835937500E+04},
 {    5288, .8356454101562E+04, -.1159862060547E+04, .1073442993164E+04},
 {    5289, .8350658203125E+04, -.1159862060547E+04, .1076037597656E+04},
 {    5290, .8344863281250E+04, -.1159862060547E+04, .1078632080078E+04},
 {    5291, .8339066406250E+04, -.1159862060547E+04, .1081226562500E+04},
 {    5292, .8333272460938E+04, -.1159862060547E+04, .1083821044922E+04},
 {    5293, .8327475585938E+04, -.1159862060547E+04, .1086415771484E+04},
 {    5294, .8321680664062E+04, -.1159862060547E+04, .1089010131836E+04},
 {    5295, .8315884765625E+04, -.1159862060547E+04, .1091604980469E+04},
 {    5296, .8310089843750E+04, -.1159862060547E+04, .1094199707031E+04},
 {    5297, .8350658203125E+04, -.1153512084961E+04, .1076037597656E+04},
 {    5298, .8339066406250E+04, -.1153512084961E+04, .1081226562500E+04},
 {    5299, .8327475585938E+04, -.1153512084961E+04, .1086415771484E+04},
 {    5300, .8315884765625E+04, -.1153512084961E+04, .1091604980469E+04},
 {    5301, .8351264648438E+04, -.1159862060547E+04, .1061851684570E+04},
 {    5302, .8345468750000E+04, -.1159862060547E+04, .1064446044922E+04},
 {    5303, .8339673828125E+04, -.1159862060547E+04, .1067040649414E+04},
 {    5304, .8333877929688E+04, -.1159862060547E+04, .1069635131836E+04},
 {    5305, .8328083007812E+04, -.1159862060547E+04, .1072229858398E+04},
 {    5306, .8322289062500E+04, -.1159862060547E+04, .1074823974609E+04},
 {    5307, .8316491210938E+04, -.1159862060547E+04, .1077418945312E+04},
 {    5308, .8310696289062E+04, -.1159862060547E+04, .1080013427734E+04},
 {    5309, .8304899414062E+04, -.1159862060547E+04, .1082608154297E+04},
 {    5310, .8345468750000E+04, -.1153512084961E+04, .1064446044922E+04},
 {    5311, .8333877929688E+04, -.1153512084961E+04, .1069635131836E+04},
 {    5312, .8322289062500E+04, -.1153512084961E+04, .1074823974609E+04},
 {    5313, .8310696289062E+04, -.1153512084961E+04, .1080013427734E+04},
 {    5314, .8346075195312E+04, -.1159862060547E+04, .1050260009766E+04},
 {    5315, .8340279296875E+04, -.1159862060547E+04, .1052854492188E+04},
 {    5316, .8334484375000E+04, -.1159862060547E+04, .1055449218750E+04},
 {    5317, .8328690429688E+04, -.1159862060547E+04, .1058043334961E+04},
 {    5318, .8322892578125E+04, -.1159862060547E+04, .1060638183594E+04},
 {    5319, .8317096679688E+04, -.1159862060547E+04, .1063233032227E+04},
 {    5320, .8311302734375E+04, -.1159862060547E+04, .1065827270508E+04},
 {    5321, .8305505859375E+04, -.1159862060547E+04, .1068421630859E+04},
 {    5322, .8299710937500E+04, -.1159862060547E+04, .1071016357422E+04},
 {    5323, .8340279296875E+04, -.1153512084961E+04, .1052854492188E+04},
 {    5324, .8328690429688E+04, -.1153512084961E+04, .1058043334961E+04},
 {    5325, .8317096679688E+04, -.1153512084961E+04, .1063233032227E+04},
 {    5326, .8305505859375E+04, -.1153512084961E+04, .1068421630859E+04},
 {    5327, .8344782226562E+04, -.1159862060547E+04, .1047361816406E+04},
 {    5328, .8338984375000E+04, -.1159862060547E+04, .1049956420898E+04},
 {    5329, .8333187500000E+04, -.1159862060547E+04, .1052551025391E+04},
 {    5330, .8327390625000E+04, -.1159862060547E+04, .1055145629883E+04},
 {    5331, .8321595703125E+04, -.1159862060547E+04, .1057740356445E+04},
 {    5332, .8315798828125E+04, -.1159862060547E+04, .1060335449219E+04},
 {    5333, .8310006835938E+04, -.1159862060547E+04, .1062929321289E+04},
 {    5334, .8304208007812E+04, -.1159862060547E+04, .1065523925781E+04},
 {    5335, .8298413085938E+04, -.1159862060547E+04, .1068118286133E+04},
 {    5336, .8338984375000E+04, -.1153512084961E+04, .1049956420898E+04},
 {    5337, .8327390625000E+04, -.1153512084961E+04, .1055145629883E+04},
 {    5338, .8315798828125E+04, -.1153512084961E+04, .1060335449219E+04},
 {    5339, .8304208007812E+04, -.1153512084961E+04, .1065523925781E+04},
 {    5340, .8339590820312E+04, -.1159862060547E+04, .1035770751953E+04},
 {    5341, .8333795898438E+04, -.1159862060547E+04, .1038365356445E+04},
 {    5342, .8327998046875E+04, -.1159862060547E+04, .1040959594727E+04},
 {    5343, .8322202148438E+04, -.1159862060547E+04, .1043554321289E+04},
 {    5344, .8316409179688E+04, -.1159862060547E+04, .1046148437500E+04},
 {    5345, .8310611328125E+04, -.1159862060547E+04, .1048743408203E+04},
 {    5346, .8304816406250E+04, -.1159862060547E+04, .1051337646484E+04},
 {    5347, .8299019531250E+04, -.1159862060547E+04, .1053931884766E+04},
 {    5348, .8293222656250E+04, -.1159862060547E+04, .1056526855469E+04},
 {    5349, .8333795898438E+04, -.1153512084961E+04, .1038365356445E+04},
 {    5350, .8322202148438E+04, -.1153512084961E+04, .1043554321289E+04},
 {    5351, .8310611328125E+04, -.1153512084961E+04, .1048743408203E+04},
 {    5352, .8299019531250E+04, -.1153512084961E+04, .1053931884766E+04},
 {    5353, .8334400390625E+04, -.1159862060547E+04, .1024179321289E+04},
 {    5354, .8328605468750E+04, -.1159862060547E+04, .1026773681641E+04},
 {    5355, .8322810546875E+04, -.1159862060547E+04, .1029367919922E+04},
 {    5356, .8317013671875E+04, -.1159862060547E+04, .1031962402344E+04},
 {    5357, .8311216796875E+04, -.1159862060547E+04, .1034557128906E+04},
 {    5358, .8305422851562E+04, -.1159862060547E+04, .1037151611328E+04},
 {    5359, .8299626953125E+04, -.1159862060547E+04, .1039746093750E+04},
 {    5360, .8293831054688E+04, -.1159862060547E+04, .1042341308594E+04},
 {    5361, .8288036132812E+04, -.1159862060547E+04, .1044935058594E+04},
 {    5362, .8328605468750E+04, -.1153512084961E+04, .1026773681641E+04},
 {    5363, .8317013671875E+04, -.1153512084961E+04, .1031962402344E+04},
 {    5364, .8305422851562E+04, -.1153512084961E+04, .1037151611328E+04},
 {    5365, .8293831054688E+04, -.1153512084961E+04, .1042341308594E+04},
 {    5366, .8333103515625E+04, -.1159862060547E+04, .1021281066895E+04},
 {    5367, .8327308593750E+04, -.1159862060547E+04, .1023875549316E+04},
 {    5368, .8321513671875E+04, -.1159862060547E+04, .1026470092773E+04},
 {    5369, .8315716796875E+04, -.1159862060547E+04, .1029065063477E+04},
 {    5370, .8309919921875E+04, -.1159862060547E+04, .1031659179688E+04},
 {    5371, .8304125000000E+04, -.1159862060547E+04, .1034253662109E+04},
 {    5372, .8298330078125E+04, -.1159862060547E+04, .1036848388672E+04},
 {    5373, .8292533203125E+04, -.1159862060547E+04, .1039442871094E+04},
 {    5374, .8286740234375E+04, -.1159862060547E+04, .1042037475586E+04},
 {    5375, .8327308593750E+04, -.1153512084961E+04, .1023875549316E+04},
 {    5376, .8315716796875E+04, -.1153512084961E+04, .1029065063477E+04},
 {    5377, .8304125000000E+04, -.1153512084961E+04, .1034253662109E+04},
 {    5378, .8292533203125E+04, -.1153512084961E+04, .1039442871094E+04},
 {    5379, .8327915039062E+04, -.1159862060547E+04, .1009689575195E+04},
 {    5380, .8322119140625E+04, -.1159862060547E+04, .1012283935547E+04},
 {    5381, .8316325195312E+04, -.1159862060547E+04, .1014878845215E+04},
 {    5382, .8310527343750E+04, -.1159862060547E+04, .1017473144531E+04},
 {    5383, .8304731445312E+04, -.1159862060547E+04, .1020067871094E+04},
 {    5384, .8298936523438E+04, -.1159862060547E+04, .1022662475586E+04},
 {    5385, .8293141601562E+04, -.1159862060547E+04, .1025256958008E+04},
 {    5386, .8287344726562E+04, -.1159862060547E+04, .1027851440430E+04},
 {    5387, .8281547851562E+04, -.1159862060547E+04, .1030446166992E+04},
 {    5388, .8322119140625E+04, -.1153512084961E+04, .1012283935547E+04},
 {    5389, .8310527343750E+04, -.1153512084961E+04, .1017473144531E+04},
 {    5390, .8298936523438E+04, -.1153512084961E+04, .1022662231445E+04},
 {    5391, .8287344726562E+04, -.1153512084961E+04, .1027851440430E+04},
 {    5392, .8322724609375E+04, -.1159862060547E+04, .9980984497070E+03},
 {    5393, .8316929687500E+04, -.1159862060547E+04, .1000692810059E+04},
 {    5394, .8311132812500E+04, -.1159862060547E+04, .1003287475586E+04},
 {    5395, .8305338867188E+04, -.1159862060547E+04, .1005881713867E+04},
 {    5396, .8299541992188E+04, -.1159862060547E+04, .1008476074219E+04},
 {    5397, .8293746093750E+04, -.1159862060547E+04, .1011070922852E+04},
 {    5398, .8287951171875E+04, -.1159862060547E+04, .1013665771484E+04},
 {    5399, .8282156250000E+04, -.1159862060547E+04, .1016260009766E+04},
 {    5400, .8276358398438E+04, -.1159862060547E+04, .1018854675293E+04},
 {    5401, .8316929687500E+04, -.1153512084961E+04, .1000692810059E+04},
 {    5402, .8305338867188E+04, -.1153512084961E+04, .1005881713867E+04},
 {    5403, .8293746093750E+04, -.1153512084961E+04, .1011070922852E+04},
 {    5404, .8282156250000E+04, -.1153512084961E+04, .1016260009766E+04},
 {    5405, .8321427734375E+04, -.1159862060547E+04, .9951999511719E+03},
 {    5406, .8315631835938E+04, -.1159862060547E+04, .9977947998047E+03},
 {    5407, .8309836914062E+04, -.1159862060547E+04, .1000389099121E+04},
 {    5408, .8304040039062E+04, -.1159862060547E+04, .1002984252930E+04},
 {    5409, .8298244140625E+04, -.1159862060547E+04, .1005578613281E+04},
 {    5410, .8292449218750E+04, -.1159862060547E+04, .1008172973633E+04},
 {    5411, .8286653320312E+04, -.1159862060547E+04, .1010767395020E+04},
 {    5412, .8280858398438E+04, -.1159862060547E+04, .1013362121582E+04},
 {    5413, .8275063476562E+04, -.1159862060547E+04, .1015956665039E+04},
 {    5414, .8315631835938E+04, -.1153512084961E+04, .9977947998047E+03},
 {    5415, .8304040039062E+04, -.1153512084961E+04, .1002984252930E+04},
 {    5416, .8292449218750E+04, -.1153512084961E+04, .1008172973633E+04},
 {    5417, .8280858398438E+04, -.1153512084961E+04, .1013362121582E+04},
 {    5418, .8316240234375E+04, -.1159862060547E+04, .9836082763672E+03},
 {    5419, .8310444335938E+04, -.1159862060547E+04, .9862033691406E+03},
 {    5420, .8304648437500E+04, -.1159862060547E+04, .9887977294922E+03},
 {    5421, .8298852539062E+04, -.1159862060547E+04, .9913925781250E+03},
 {    5422, .8293055664062E+04, -.1159862060547E+04, .9939868164062E+03},
 {    5423, .8287260742188E+04, -.1159862060547E+04, .9965816650391E+03},
 {    5424, .8281464843750E+04, -.1159862060547E+04, .9991757812500E+03},
 {    5425, .8275667968750E+04, -.1159862060547E+04, .1001770507812E+04},
 {    5426, .8269874023438E+04, -.1159862060547E+04, .1004364807129E+04},
 {    5427, .8310444335938E+04, -.1153512084961E+04, .9862033691406E+03},
 {    5428, .8298852539062E+04, -.1153512084961E+04, .9913925781250E+03},
 {    5429, .8287260742188E+04, -.1153512084961E+04, .9965816650391E+03},
 {    5430, .8275667968750E+04, -.1153512084961E+04, .1001770507812E+04},
 {    5431, .8311048828125E+04, -.1159862060547E+04, .9720169677734E+03},
 {    5432, .8305254882812E+04, -.1159862060547E+04, .9746119384766E+03},
 {    5433, .8299458007812E+04, -.1159862060547E+04, .9772065429688E+03},
 {    5434, .8293663085938E+04, -.1159862060547E+04, .9798010253906E+03},
 {    5435, .8287866210938E+04, -.1159862060547E+04, .9823952636719E+03},
 {    5436, .8282072265625E+04, -.1159862060547E+04, .9849899291992E+03},
 {    5437, .8276275390625E+04, -.1159862060547E+04, .9875843505859E+03},
 {    5438, .8270480468750E+04, -.1159862060547E+04, .9901790161133E+03},
 {    5439, .8264683593750E+04, -.1159862060547E+04, .9927739257812E+03},
 {    5440, .8305254882812E+04, -.1153512084961E+04, .9746119384766E+03},
 {    5441, .8293663085938E+04, -.1153512084961E+04, .9798010253906E+03},
 {    5442, .8282072265625E+04, -.1153512084961E+04, .9849899291992E+03},
 {    5443, .8270480468750E+04, -.1153512084961E+04, .9901790161133E+03},
 {    5444, .8309751953125E+04, -.1159862060547E+04, .9691195068359E+03},
 {    5445, .8303958007812E+04, -.1159862060547E+04, .9717136230469E+03},
 {    5446, .8298160156250E+04, -.1159862060547E+04, .9743084106445E+03},
 {    5447, .8292365234375E+04, -.1159862060547E+04, .9769032592773E+03},
 {    5448, .8286571289062E+04, -.1159862060547E+04, .9794973144531E+03},
 {    5449, .8280773437500E+04, -.1159862060547E+04, .9820920410156E+03},
 {    5450, .8274979492188E+04, -.1159862060547E+04, .9846864624023E+03},
 {    5451, .8269183593750E+04, -.1159862060547E+04, .9872810058594E+03},
 {    5452, .8263387695312E+04, -.1159862060547E+04, .9898752441406E+03},
 {    5453, .8303958007812E+04, -.1153512084961E+04, .9717136230469E+03},
 {    5454, .8292365234375E+04, -.1153512084961E+04, .9769032592773E+03},
 {    5455, .8280773437500E+04, -.1153512084961E+04, .9820920410156E+03},
 {    5456, .8269183593750E+04, -.1153512084961E+04, .9872810058594E+03},
 {    5457, .8304563476562E+04, -.1159862060547E+04, .9575277099609E+03},
 {    5458, .8298767578125E+04, -.1159862060547E+04, .9601221313477E+03},
 {    5459, .8292974609375E+04, -.1159862060547E+04, .9627166748047E+03},
 {    5460, .8287175781250E+04, -.1159862060547E+04, .9653112792969E+03},
 {    5461, .8281380859375E+04, -.1159862060547E+04, .9679062500000E+03},
 {    5462, .8275585937500E+04, -.1159862060547E+04, .9705003662109E+03},
 {    5463, .8269791015625E+04, -.1159862060547E+04, .9730949707031E+03},
 {    5464, .8263994140625E+04, -.1159862060547E+04, .9756896362305E+03},
 {    5465, .8258200195312E+04, -.1159862060547E+04, .9782839355469E+03},
 {    5466, .8298767578125E+04, -.1153512084961E+04, .9601221313477E+03},
 {    5467, .8287175781250E+04, -.1153512084961E+04, .9653112792969E+03},
 {    5468, .8275585937500E+04, -.1153512084961E+04, .9705003662109E+03},
 {    5469, .8263994140625E+04, -.1153512084961E+04, .9756896362305E+03},
 {    5470, .8299374023438E+04, -.1159862060547E+04, .9459361572266E+03},
 {    5471, .8293578125000E+04, -.1159862060547E+04, .9485307617188E+03},
 {    5472, .8287781250000E+04, -.1159862060547E+04, .9511254882812E+03},
 {    5473, .8281987304688E+04, -.1159862060547E+04, .9537197875977E+03},
 {    5474, .8276192382812E+04, -.1159862060547E+04, .9563143310547E+03},
 {    5475, .8270395507812E+04, -.1159862060547E+04, .9589091796875E+03},
 {    5476, .8264600585938E+04, -.1159862060547E+04, .9615036621094E+03},
 {    5477, .8258804687500E+04, -.1159862060547E+04, .9640978393555E+03},
 {    5478, .8253008789062E+04, -.1159862060547E+04, .9666925048828E+03},
 {    5479, .8293578125000E+04, -.1153512084961E+04, .9485307617188E+03},
 {    5480, .8281987304688E+04, -.1153512084961E+04, .9537197875977E+03},
 {    5481, .8270395507812E+04, -.1153512084961E+04, .9589091796875E+03},
 {    5482, .8258804687500E+04, -.1153512084961E+04, .9640978393555E+03},
 {    5483, .8297298828125E+04, -.1159862060547E+04, .9412995605469E+03},
 {    5484, .8291503906250E+04, -.1159862060547E+04, .9438941040039E+03},
 {    5485, .8285707031250E+04, -.1159862060547E+04, .9464887084961E+03},
 {    5486, .8279911132812E+04, -.1159862060547E+04, .9490833740234E+03},
 {    5487, .8274116210938E+04, -.1159862060547E+04, .9516777343750E+03},
 {    5488, .8268319335938E+04, -.1159862060547E+04, .9542723388672E+03},
 {    5489, .8262524414062E+04, -.1159862060547E+04, .9568668823242E+03},
 {    5490, .8256730468750E+04, -.1159862060547E+04, .9594612426758E+03},
 {    5491, .8250930664062E+04, -.1159862060547E+04, .9620561523438E+03},
 {    5492, .8297298828125E+04, -.1153512084961E+04, .9412995605469E+03},
 {    5493, .8291503906250E+04, -.1153512084961E+04, .9438941040039E+03},
 {    5494, .8285707031250E+04, -.1153512084961E+04, .9464887084961E+03},
 {    5495, .8279911132812E+04, -.1153512084961E+04, .9490833740234E+03},
 {    5496, .8274116210938E+04, -.1153512084961E+04, .9516777343750E+03},
 {    5497, .8268319335938E+04, -.1153512084961E+04, .9542723388672E+03},
 {    5498, .8262524414062E+04, -.1153512084961E+04, .9568668823242E+03},
 {    5499, .8256730468750E+04, -.1153512084961E+04, .9594612426758E+03},
 {    5500, .8250930664062E+04, -.1153512084961E+04, .9620561523438E+03},
 {    5501, .8346075195312E+04, -.1171291992188E+04, .1050260009766E+04},
 {    5502, .8334484375000E+04, -.1171291992188E+04, .1055449218750E+04},
 {    5503, .8322892578125E+04, -.1171291992188E+04, .1060638183594E+04},
 {    5504, .8344782226562E+04, -.1171291992188E+04, .1047361816406E+04},
 {    5505, .8333187500000E+04, -.1171291992188E+04, .1052551025391E+04},
 {    5506, .8321595703125E+04, -.1171291992188E+04, .1057740356445E+04},
 {    5507, .8339590820312E+04, -.1171291992188E+04, .1035770751953E+04},
 {    5508, .8327998046875E+04, -.1171291992188E+04, .1040959594727E+04},
 {    5509, .8316409179688E+04, -.1171291992188E+04, .1046148437500E+04},
 {    5510, .8334400390625E+04, -.1171291992188E+04, .1024179321289E+04},
 {    5511, .8322810546875E+04, -.1171291992188E+04, .1029367919922E+04},
 {    5512, .8311216796875E+04, -.1171291992188E+04, .1034557128906E+04},
 {    5513, .8333103515625E+04, -.1171291992188E+04, .1021281066895E+04},
 {    5514, .8321513671875E+04, -.1171291992188E+04, .1026470092773E+04},
 {    5515, .8309919921875E+04, -.1171291992188E+04, .1031659179688E+04},
 {    5516, .8327915039062E+04, -.1171291992188E+04, .1009689575195E+04},
 {    5517, .8316325195312E+04, -.1171291992188E+04, .1014878845215E+04},
 {    5518, .8304731445312E+04, -.1171291992188E+04, .1020067871094E+04},
 {    5519, .8322724609375E+04, -.1171291992188E+04, .9980984497070E+03},
 {    5520, .8311132812500E+04, -.1171291992188E+04, .1003287475586E+04},
 {    5521, .8299541992188E+04, -.1171291992188E+04, .1008476074219E+04},
 {    5522, .8321427734375E+04, -.1171291992188E+04, .9951999511719E+03},
 {    5523, .8309836914062E+04, -.1171291992188E+04, .1000389099121E+04},
 {    5524, .8298244140625E+04, -.1171291992188E+04, .1005578613281E+04},
 {    5525, .8316240234375E+04, -.1171291992188E+04, .9836082763672E+03},
 {    5526, .8304648437500E+04, -.1171291992188E+04, .9887977294922E+03},
 {    5527, .8293055664062E+04, -.1171291992188E+04, .9939868164062E+03},
 {    5528, .8311048828125E+04, -.1171291992188E+04, .9720169677734E+03},
 {    5529, .8299458007812E+04, -.1171291992188E+04, .9772065429688E+03},
 {    5530, .8287866210938E+04, -.1171291992188E+04, .9823952636719E+03},
 {    5531, .8309751953125E+04, -.1171291992188E+04, .9691195068359E+03},
 {    5532, .8298160156250E+04, -.1171291992188E+04, .9743084106445E+03},
 {    5533, .8286571289062E+04, -.1171291992188E+04, .9794973144531E+03},
 {    5534, .8311302734375E+04, -.1171291992188E+04, .1065827270508E+04},
 {    5535, .8299710937500E+04, -.1171291992188E+04, .1071016357422E+04},
 {    5536, .8310006835938E+04, -.1171291992188E+04, .1062929321289E+04},
 {    5537, .8298413085938E+04, -.1171291992188E+04, .1068118286133E+04},
 {    5538, .8304816406250E+04, -.1171291992188E+04, .1051337646484E+04},
 {    5539, .8293222656250E+04, -.1171291992188E+04, .1056526855469E+04},
 {    5540, .8299626953125E+04, -.1171291992188E+04, .1039746093750E+04},
 {    5541, .8288036132812E+04, -.1171291992188E+04, .1044935058594E+04},
 {    5542, .8286653320312E+04, -.1171291992188E+04, .1010767395020E+04},
 {    5543, .8275063476562E+04, -.1171291992188E+04, .1015956665039E+04},
 {    5544, .8281464843750E+04, -.1171291992188E+04, .9991757812500E+03},
 {    5545, .8269874023438E+04, -.1171291992188E+04, .1004364807129E+04},
 {    5546, .8276275390625E+04, -.1171291992188E+04, .9875843505859E+03},
 {    5547, .8264683593750E+04, -.1171291992188E+04, .9927739257812E+03},
 {    5548, .8274979492188E+04, -.1171291992188E+04, .9846864624023E+03},
 {    5549, .8263387695312E+04, -.1171291992188E+04, .9898752441406E+03},
 {    5550, .8358530273438E+04, -.1411322021484E+04, .1078079589844E+04},
 {    5551, .8352735351562E+04, -.1411322021484E+04, .1080674316406E+04},
 {    5552, .8346940429688E+04, -.1411322021484E+04, .1083268798828E+04},
 {    5553, .8341141601562E+04, -.1411322021484E+04, .1085863403320E+04},
 {    5554, .8335346679688E+04, -.1411322021484E+04, .1088457763672E+04},
 {    5555, .8329552734375E+04, -.1411322021484E+04, .1091052490234E+04},
 {    5556, .8323755859375E+04, -.1411322021484E+04, .1093646728516E+04},
 {    5557, .8317958984375E+04, -.1411322021484E+04, .1096241699219E+04},
 {    5558, .8312165039062E+04, -.1411322021484E+04, .1098835937500E+04},
 {    5559, .8358530273438E+04, -.1417671997070E+04, .1078079589844E+04},
 {    5560, .8352735351562E+04, -.1417671997070E+04, .1080674316406E+04},
 {    5561, .8346940429688E+04, -.1417671997070E+04, .1083268798828E+04},
 {    5562, .8341141601562E+04, -.1417671997070E+04, .1085863403320E+04},
 {    5563, .8335346679688E+04, -.1417671997070E+04, .1088457763672E+04},
 {    5564, .8329552734375E+04, -.1417671997070E+04, .1091052490234E+04},
 {    5565, .8323755859375E+04, -.1417671997070E+04, .1093646728516E+04},
 {    5566, .8317958984375E+04, -.1417671997070E+04, .1096241699219E+04},
 {    5567, .8312165039062E+04, -.1417671997070E+04, .1098835937500E+04},
 {    5568, .8356454101562E+04, -.1411322021484E+04, .1073442993164E+04},
 {    5569, .8350658203125E+04, -.1411322021484E+04, .1076037597656E+04},
 {    5570, .8344863281250E+04, -.1411322021484E+04, .1078632080078E+04},
 {    5571, .8339066406250E+04, -.1411322021484E+04, .1081226562500E+04},
 {    5572, .8333272460938E+04, -.1411322021484E+04, .1083821044922E+04},
 {    5573, .8327475585938E+04, -.1411322021484E+04, .1086415771484E+04},
 {    5574, .8321680664062E+04, -.1411322021484E+04, .1089010131836E+04},
 {    5575, .8315884765625E+04, -.1411322021484E+04, .1091604980469E+04},
 {    5576, .8310089843750E+04, -.1411322021484E+04, .1094199707031E+04},
 {    5577, .8350658203125E+04, -.1417671997070E+04, .1076037597656E+04},
 {    5578, .8339066406250E+04, -.1417671997070E+04, .1081226562500E+04},
 {    5579, .8327475585938E+04, -.1417671997070E+04, .1086415771484E+04},
 {    5580, .8315884765625E+04, -.1417671997070E+04, .1091604980469E+04},
 {    5581, .8351264648438E+04, -.1411322021484E+04, .1061851684570E+04},
 {    5582, .8345468750000E+04, -.1411322021484E+04, .1064446044922E+04},
 {    5583, .8339673828125E+04, -.1411322021484E+04, .1067040649414E+04},
 {    5584, .8333877929688E+04, -.1411322021484E+04, .1069635131836E+04},
 {    5585, .8328083007812E+04, -.1411322021484E+04, .1072229858398E+04},
 {    5586, .8322289062500E+04, -.1411322021484E+04, .1074823974609E+04},
 {    5587, .8316491210938E+04, -.1411322021484E+04, .1077418945312E+04},
 {    5588, .8310696289062E+04, -.1411322021484E+04, .1080013427734E+04},
 {    5589, .8304899414062E+04, -.1411322021484E+04, .1082608154297E+04},
 {    5590, .8345468750000E+04, -.1417671997070E+04, .1064446044922E+04},
 {    5591, .8333877929688E+04, -.1417671997070E+04, .1069635131836E+04},
 {    5592, .8322289062500E+04, -.1417671997070E+04, .1074823974609E+04},
 {    5593, .8310696289062E+04, -.1417671997070E+04, .1080013427734E+04},
 {    5594, .8346075195312E+04, -.1411322021484E+04, .1050260009766E+04},
 {    5595, .8340279296875E+04, -.1411322021484E+04, .1052854492188E+04},
 {    5596, .8334484375000E+04, -.1411322021484E+04, .1055449218750E+04},
 {    5597, .8328690429688E+04, -.1411322021484E+04, .1058043579102E+04},
 {    5598, .8322892578125E+04, -.1411322021484E+04, .1060638183594E+04},
 {    5599, .8317096679688E+04, -.1411322021484E+04, .1063233032227E+04},
 {    5600, .8311302734375E+04, -.1411322021484E+04, .1065827270508E+04},
 {    5601, .8305505859375E+04, -.1411322021484E+04, .1068421630859E+04},
 {    5602, .8299710937500E+04, -.1411322021484E+04, .1071016357422E+04},
 {    5603, .8340279296875E+04, -.1417671997070E+04, .1052854492188E+04},
 {    5604, .8328690429688E+04, -.1417671997070E+04, .1058043579102E+04},
 {    5605, .8317096679688E+04, -.1417671997070E+04, .1063233032227E+04},
 {    5606, .8305505859375E+04, -.1417671997070E+04, .1068421630859E+04},
 {    5607, .8344782226562E+04, -.1411322021484E+04, .1047361816406E+04},
 {    5608, .8338984375000E+04, -.1411322021484E+04, .1049956420898E+04},
 {    5609, .8333187500000E+04, -.1411322021484E+04, .1052551025391E+04},
 {    5610, .8327390625000E+04, -.1411322021484E+04, .1055145629883E+04},
 {    5611, .8321595703125E+04, -.1411322021484E+04, .1057740356445E+04},
 {    5612, .8315798828125E+04, -.1411322021484E+04, .1060335449219E+04},
 {    5613, .8310006835938E+04, -.1411322021484E+04, .1062929321289E+04},
 {    5614, .8304208007812E+04, -.1411322021484E+04, .1065523925781E+04},
 {    5615, .8298413085938E+04, -.1411322021484E+04, .1068118286133E+04},
 {    5616, .8338984375000E+04, -.1417671997070E+04, .1049956420898E+04},
 {    5617, .8327390625000E+04, -.1417671997070E+04, .1055145629883E+04},
 {    5618, .8315798828125E+04, -.1417671997070E+04, .1060335449219E+04},
 {    5619, .8304208007812E+04, -.1417671997070E+04, .1065523925781E+04},
 {    5620, .8339590820312E+04, -.1411322021484E+04, .1035770751953E+04},
 {    5621, .8333795898438E+04, -.1411322021484E+04, .1038365356445E+04},
 {    5622, .8327998046875E+04, -.1411322021484E+04, .1040959594727E+04},
 {    5623, .8322202148438E+04, -.1411322021484E+04, .1043554565430E+04},
 {    5624, .8316409179688E+04, -.1411322021484E+04, .1046148437500E+04},
 {    5625, .8310612304688E+04, -.1411322021484E+04, .1048743408203E+04},
 {    5626, .8304816406250E+04, -.1411322021484E+04, .1051337646484E+04},
 {    5627, .8299019531250E+04, -.1411322021484E+04, .1053931884766E+04},
 {    5628, .8293222656250E+04, -.1411322021484E+04, .1056526855469E+04},
 {    5629, .8333795898438E+04, -.1417671997070E+04, .1038365356445E+04},
 {    5630, .8322202148438E+04, -.1417671997070E+04, .1043554565430E+04},
 {    5631, .8310612304688E+04, -.1417671997070E+04, .1048743408203E+04},
 {    5632, .8299019531250E+04, -.1417671997070E+04, .1053931884766E+04},
 {    5633, .8334400390625E+04, -.1411322021484E+04, .1024179321289E+04},
 {    5634, .8328605468750E+04, -.1411322021484E+04, .1026773681641E+04},
 {    5635, .8322810546875E+04, -.1411322021484E+04, .1029367919922E+04},
 {    5636, .8317013671875E+04, -.1411322021484E+04, .1031962402344E+04},
 {    5637, .8311216796875E+04, -.1411322021484E+04, .1034557128906E+04},
 {    5638, .8305422851562E+04, -.1411322021484E+04, .1037151611328E+04},
 {    5639, .8299626953125E+04, -.1411322021484E+04, .1039746093750E+04},
 {    5640, .8293831054688E+04, -.1411322021484E+04, .1042341308594E+04},
 {    5641, .8288036132812E+04, -.1411322021484E+04, .1044935058594E+04},
 {    5642, .8328605468750E+04, -.1417671997070E+04, .1026773681641E+04},
 {    5643, .8317013671875E+04, -.1417671997070E+04, .1031962402344E+04},
 {    5644, .8305422851562E+04, -.1417671997070E+04, .1037151611328E+04},
 {    5645, .8293831054688E+04, -.1417671997070E+04, .1042341308594E+04},
 {    5646, .8333103515625E+04, -.1411322021484E+04, .1021281066895E+04},
 {    5647, .8327308593750E+04, -.1411322021484E+04, .1023875549316E+04},
 {    5648, .8321513671875E+04, -.1411322021484E+04, .1026470336914E+04},
 {    5649, .8315716796875E+04, -.1411322021484E+04, .1029065063477E+04},
 {    5650, .8309919921875E+04, -.1411322021484E+04, .1031659423828E+04},
 {    5651, .8304125000000E+04, -.1411322021484E+04, .1034253662109E+04},
 {    5652, .8298330078125E+04, -.1411322021484E+04, .1036848388672E+04},
 {    5653, .8292533203125E+04, -.1411322021484E+04, .1039442871094E+04},
 {    5654, .8286740234375E+04, -.1411322021484E+04, .1042037475586E+04},
 {    5655, .8327308593750E+04, -.1417671997070E+04, .1023875549316E+04},
 {    5656, .8315716796875E+04, -.1417671997070E+04, .1029065063477E+04},
 {    5657, .8304125000000E+04, -.1417671997070E+04, .1034253662109E+04},
 {    5658, .8292533203125E+04, -.1417671997070E+04, .1039442871094E+04},
 {    5659, .8327915039062E+04, -.1411322021484E+04, .1009689575195E+04},
 {    5660, .8322119140625E+04, -.1411322021484E+04, .1012283935547E+04},
 {    5661, .8316325195312E+04, -.1411322021484E+04, .1014878845215E+04},
 {    5662, .8310527343750E+04, -.1411322021484E+04, .1017473144531E+04},
 {    5663, .8304731445312E+04, -.1411322021484E+04, .1020067871094E+04},
 {    5664, .8298936523438E+04, -.1411322021484E+04, .1022662475586E+04},
 {    5665, .8293141601562E+04, -.1411322021484E+04, .1025256958008E+04},
 {    5666, .8287344726562E+04, -.1411322021484E+04, .1027851440430E+04},
 {    5667, .8281547851562E+04, -.1411322021484E+04, .1030446166992E+04},
 {    5668, .8322119140625E+04, -.1417671997070E+04, .1012283935547E+04},
 {    5669, .8310527343750E+04, -.1417671997070E+04, .1017473144531E+04},
 {    5670, .8298936523438E+04, -.1417671997070E+04, .1022662475586E+04},
 {    5671, .8287344726562E+04, -.1417671997070E+04, .1027851440430E+04},
 {    5672, .8322724609375E+04, -.1411322021484E+04, .9980984497070E+03},
 {    5673, .8316929687500E+04, -.1411322021484E+04, .1000692810059E+04},
 {    5674, .8311132812500E+04, -.1411322021484E+04, .1003287475586E+04},
 {    5675, .8305338867188E+04, -.1411322021484E+04, .1005881713867E+04},
 {    5676, .8299541992188E+04, -.1411322021484E+04, .1008476074219E+04},
 {    5677, .8293746093750E+04, -.1411322021484E+04, .1011070922852E+04},
 {    5678, .8287951171875E+04, -.1411322021484E+04, .1013665771484E+04},
 {    5679, .8282156250000E+04, -.1411322021484E+04, .1016260253906E+04},
 {    5680, .8276358398438E+04, -.1411322021484E+04, .1018854675293E+04},
 {    5681, .8316929687500E+04, -.1417671997070E+04, .1000692810059E+04},
 {    5682, .8305338867188E+04, -.1417671997070E+04, .1005881713867E+04},
 {    5683, .8293746093750E+04, -.1417671997070E+04, .1011070922852E+04},
 {    5684, .8282156250000E+04, -.1417671997070E+04, .1016260253906E+04},
 {    5685, .8321427734375E+04, -.1411322021484E+04, .9951999511719E+03},
 {    5686, .8315631835938E+04, -.1411322021484E+04, .9977950439453E+03},
 {    5687, .8309836914062E+04, -.1411322021484E+04, .1000389099121E+04},
 {    5688, .8304040039062E+04, -.1411322021484E+04, .1002984252930E+04},
 {    5689, .8298244140625E+04, -.1411322021484E+04, .1005578613281E+04},
 {    5690, .8292449218750E+04, -.1411322021484E+04, .1008172973633E+04},
 {    5691, .8286653320312E+04, -.1411322021484E+04, .1010767395020E+04},
 {    5692, .8280858398438E+04, -.1411322021484E+04, .1013362121582E+04},
 {    5693, .8275063476562E+04, -.1411322021484E+04, .1015956665039E+04},
 {    5694, .8315631835938E+04, -.1417671997070E+04, .9977950439453E+03},
 {    5695, .8304040039062E+04, -.1417671997070E+04, .1002984252930E+04},
 {    5696, .8292449218750E+04, -.1417671997070E+04, .1008172973633E+04},
 {    5697, .8280858398438E+04, -.1417671997070E+04, .1013362121582E+04},
 {    5698, .8316240234375E+04, -.1411322021484E+04, .9836082763672E+03},
 {    5699, .8310444335938E+04, -.1411322021484E+04, .9862033691406E+03},
 {    5700, .8304648437500E+04, -.1411322021484E+04, .9887977294922E+03},
 {    5701, .8298852539062E+04, -.1411322021484E+04, .9913925781250E+03},
 {    5702, .8293055664062E+04, -.1411322021484E+04, .9939868164062E+03},
 {    5703, .8287260742188E+04, -.1411322021484E+04, .9965816650391E+03},
 {    5704, .8281464843750E+04, -.1411322021484E+04, .9991757812500E+03},
 {    5705, .8275667968750E+04, -.1411322021484E+04, .1001770507812E+04},
 {    5706, .8269874023438E+04, -.1411322021484E+04, .1004364807129E+04},
 {    5707, .8310444335938E+04, -.1417671997070E+04, .9862033691406E+03},
 {    5708, .8298852539062E+04, -.1417671997070E+04, .9913928222656E+03},
 {    5709, .8287260742188E+04, -.1417671997070E+04, .9965816650391E+03},
 {    5710, .8275667968750E+04, -.1417671997070E+04, .1001770507812E+04},
 {    5711, .8311048828125E+04, -.1411322021484E+04, .9720169677734E+03},
 {    5712, .8305254882812E+04, -.1411322021484E+04, .9746121826172E+03},
 {    5713, .8299458007812E+04, -.1411322021484E+04, .9772065429688E+03},
 {    5714, .8293663085938E+04, -.1411322021484E+04, .9798010253906E+03},
 {    5715, .8287866210938E+04, -.1411322021484E+04, .9823955078125E+03},
 {    5716, .8282072265625E+04, -.1411322021484E+04, .9849899291992E+03},
 {    5717, .8276275390625E+04, -.1411322021484E+04, .9875843505859E+03},
 {    5718, .8270480468750E+04, -.1411322021484E+04, .9901790161133E+03},
 {    5719, .8264683593750E+04, -.1411322021484E+04, .9927739257812E+03},
 {    5720, .8305254882812E+04, -.1417671997070E+04, .9746121826172E+03},
 {    5721, .8293663085938E+04, -.1417671997070E+04, .9798010253906E+03},
 {    5722, .8282072265625E+04, -.1417671997070E+04, .9849899291992E+03},
 {    5723, .8270480468750E+04, -.1417671997070E+04, .9901790161133E+03},
 {    5724, .8309751953125E+04, -.1411322021484E+04, .9691195068359E+03},
 {    5725, .8303958007812E+04, -.1411322021484E+04, .9717136230469E+03},
 {    5726, .8298160156250E+04, -.1411322021484E+04, .9743084106445E+03},
 {    5727, .8292365234375E+04, -.1411322021484E+04, .9769032592773E+03},
 {    5728, .8286571289062E+04, -.1411322021484E+04, .9794973144531E+03},
 {    5729, .8280773437500E+04, -.1411322021484E+04, .9820920410156E+03},
 {    5730, .8274979492188E+04, -.1411322021484E+04, .9846864624023E+03},
 {    5731, .8269183593750E+04, -.1411322021484E+04, .9872812500000E+03},
 {    5732, .8263387695312E+04, -.1411322021484E+04, .9898752441406E+03},
 {    5733, .8303958007812E+04, -.1417671997070E+04, .9717136230469E+03},
 {    5734, .8292365234375E+04, -.1417671997070E+04, .9769032592773E+03},
 {    5735, .8280773437500E+04, -.1417671997070E+04, .9820920410156E+03},
 {    5736, .8269183593750E+04, -.1417671997070E+04, .9872812500000E+03},
 {    5737, .8304563476562E+04, -.1411322021484E+04, .9575277099609E+03},
 {    5738, .8298767578125E+04, -.1411322021484E+04, .9601221313477E+03},
 {    5739, .8292974609375E+04, -.1411322021484E+04, .9627166748047E+03},
 {    5740, .8287175781250E+04, -.1411322021484E+04, .9653112792969E+03},
 {    5741, .8281380859375E+04, -.1411322021484E+04, .9679062500000E+03},
 {    5742, .8275585937500E+04, -.1411322021484E+04, .9705003662109E+03},
 {    5743, .8269791015625E+04, -.1411322021484E+04, .9730949707031E+03},
 {    5744, .8263994140625E+04, -.1411322021484E+04, .9756896362305E+03},
 {    5745, .8258200195312E+04, -.1411322021484E+04, .9782839355469E+03},
 {    5746, .8298767578125E+04, -.1417671997070E+04, .9601221313477E+03},
 {    5747, .8287175781250E+04, -.1417671997070E+04, .9653112792969E+03},
 {    5748, .8275585937500E+04, -.1417671997070E+04, .9705003662109E+03},
 {    5749, .8263994140625E+04, -.1417671997070E+04, .9756896362305E+03},
 {    5750, .8299374023438E+04, -.1411322021484E+04, .9459361572266E+03},
 {    5751, .8293578125000E+04, -.1411322021484E+04, .9485307617188E+03},
 {    5752, .8287782226562E+04, -.1411322021484E+04, .9511254882812E+03},
 {    5753, .8281987304688E+04, -.1411322021484E+04, .9537197875977E+03},
 {    5754, .8276192382812E+04, -.1411322021484E+04, .9563143310547E+03},
 {    5755, .8270395507812E+04, -.1411322021484E+04, .9589091796875E+03},
 {    5756, .8264600585938E+04, -.1411322021484E+04, .9615036621094E+03},
 {    5757, .8258804687500E+04, -.1411322021484E+04, .9640978393555E+03},
 {    5758, .8253008789062E+04, -.1411322021484E+04, .9666925048828E+03},
 {    5759, .8293578125000E+04, -.1417671997070E+04, .9485307617188E+03},
 {    5760, .8281987304688E+04, -.1417671997070E+04, .9537197875977E+03},
 {    5761, .8270395507812E+04, -.1417671997070E+04, .9589091796875E+03},
 {    5762, .8258804687500E+04, -.1417671997070E+04, .9640978393555E+03},
 {    5763, .8297298828125E+04, -.1411322021484E+04, .9412995605469E+03},
 {    5764, .8291503906250E+04, -.1411322021484E+04, .9438941040039E+03},
 {    5765, .8285707031250E+04, -.1411322021484E+04, .9464887084961E+03},
 {    5766, .8279911132812E+04, -.1411322021484E+04, .9490833740234E+03},
 {    5767, .8274116210938E+04, -.1411322021484E+04, .9516777343750E+03},
 {    5768, .8268319335938E+04, -.1411322021484E+04, .9542723388672E+03},
 {    5769, .8262524414062E+04, -.1411322021484E+04, .9568668823242E+03},
 {    5770, .8256730468750E+04, -.1411322021484E+04, .9594612426758E+03},
 {    5771, .8250930664062E+04, -.1411322021484E+04, .9620561523438E+03},
 {    5772, .8297298828125E+04, -.1417671997070E+04, .9412995605469E+03},
 {    5773, .8291503906250E+04, -.1417671997070E+04, .9438941040039E+03},
 {    5774, .8285707031250E+04, -.1417671997070E+04, .9464887084961E+03},
 {    5775, .8279911132812E+04, -.1417671997070E+04, .9490833740234E+03},
 {    5776, .8274116210938E+04, -.1417671997070E+04, .9516777343750E+03},
 {    5777, .8268319335938E+04, -.1417671997070E+04, .9542723388672E+03},
 {    5778, .8262524414062E+04, -.1417671997070E+04, .9568668823242E+03},
 {    5779, .8256730468750E+04, -.1417671997070E+04, .9594612426758E+03},
 {    5780, .8250930664062E+04, -.1417671997070E+04, .9620561523438E+03},
 {    5781, .8346075195312E+04, -.1399892089844E+04, .1050260009766E+04},
 {    5782, .8334484375000E+04, -.1399892089844E+04, .1055449218750E+04},
 {    5783, .8322892578125E+04, -.1399892089844E+04, .1060638183594E+04},
 {    5784, .8344782226562E+04, -.1399892089844E+04, .1047361816406E+04},
 {    5785, .8333187500000E+04, -.1399892089844E+04, .1052551025391E+04},
 {    5786, .8321595703125E+04, -.1399892089844E+04, .1057740356445E+04},
 {    5787, .8339590820312E+04, -.1399892089844E+04, .1035770751953E+04},
 {    5788, .8327998046875E+04, -.1399892089844E+04, .1040959594727E+04},
 {    5789, .8316409179688E+04, -.1399892089844E+04, .1046148437500E+04},
 {    5790, .8334400390625E+04, -.1399892089844E+04, .1024179321289E+04},
 {    5791, .8322810546875E+04, -.1399892089844E+04, .1029367919922E+04},
 {    5792, .8311216796875E+04, -.1399892089844E+04, .1034557128906E+04},
 {    5793, .8333103515625E+04, -.1399892089844E+04, .1021281066895E+04},
 {    5794, .8321513671875E+04, -.1399892089844E+04, .1026470336914E+04},
 {    5795, .8309919921875E+04, -.1399892089844E+04, .1031659179688E+04},
 {    5796, .8327915039062E+04, -.1399892089844E+04, .1009689575195E+04},
 {    5797, .8316325195312E+04, -.1399892089844E+04, .1014878845215E+04},
 {    5798, .8304731445312E+04, -.1399892089844E+04, .1020067871094E+04},
 {    5799, .8322724609375E+04, -.1399892089844E+04, .9980984497070E+03},
 {    5800, .8311132812500E+04, -.1399892089844E+04, .1003287475586E+04},
 {    5801, .8299541992188E+04, -.1399892089844E+04, .1008476074219E+04},
 {    5802, .8321427734375E+04, -.1399892089844E+04, .9951999511719E+03},
 {    5803, .8309836914062E+04, -.1399892089844E+04, .1000389099121E+04},
 {    5804, .8298244140625E+04, -.1399892089844E+04, .1005578613281E+04},
 {    5805, .8316240234375E+04, -.1399892089844E+04, .9836082763672E+03},
 {    5806, .8304648437500E+04, -.1399892089844E+04, .9887977294922E+03},
 {    5807, .8293055664062E+04, -.1399892089844E+04, .9939868164062E+03},
 {    5808, .8311048828125E+04, -.1399892089844E+04, .9720169677734E+03},
 {    5809, .8299458007812E+04, -.1399892089844E+04, .9772065429688E+03},
 {    5810, .8287866210938E+04, -.1399892089844E+04, .9823955078125E+03},
 {    5811, .8309751953125E+04, -.1399892089844E+04, .9691195068359E+03},
 {    5812, .8298160156250E+04, -.1399892089844E+04, .9743084106445E+03},
 {    5813, .8286571289062E+04, -.1399892089844E+04, .9794973144531E+03},
 {    5814, .8311302734375E+04, -.1399892089844E+04, .1065827270508E+04},
 {    5815, .8299710937500E+04, -.1399892089844E+04, .1071016357422E+04},
 {    5816, .8310006835938E+04, -.1399892089844E+04, .1062929321289E+04},
 {    5817, .8298413085938E+04, -.1399892089844E+04, .1068118286133E+04},
 {    5818, .8304816406250E+04, -.1399892089844E+04, .1051337646484E+04},
 {    5819, .8293222656250E+04, -.1399892089844E+04, .1056526855469E+04},
 {    5820, .8299626953125E+04, -.1399892089844E+04, .1039746093750E+04},
 {    5821, .8288036132812E+04, -.1399892089844E+04, .1044935058594E+04},
 {    5822, .8286653320312E+04, -.1399892089844E+04, .1010767395020E+04},
 {    5823, .8275063476562E+04, -.1399892089844E+04, .1015956665039E+04},
 {    5824, .8281464843750E+04, -.1399892089844E+04, .9991757812500E+03},
 {    5825, .8269874023438E+04, -.1399892089844E+04, .1004364807129E+04},
 {    5826, .8276275390625E+04, -.1399892089844E+04, .9875843505859E+03},
 {    5827, .8264683593750E+04, -.1399892089844E+04, .9927739257812E+03},
 {    5828, .8274979492188E+04, -.1399892089844E+04, .9846864624023E+03},
 {    5829, .8263387695312E+04, -.1399892089844E+04, .9898752441406E+03},
 {    5830, .8305422851562E+04, -.1412592041016E+04, .1037151611328E+04},
 {    5831, .8296728515625E+04, -.1412592041016E+04, .1041043579102E+04},
 {    5832, .8305422851562E+04, -.1404972045898E+04, .1037151611328E+04},
 {    5833, .8296728515625E+04, -.1404972045898E+04, .1041043579102E+04},
 {    5834, .8304125000000E+04, -.1412592041016E+04, .1034253662109E+04},
 {    5835, .8295432617188E+04, -.1412592041016E+04, .1038145751953E+04},
 {    5836, .8304125000000E+04, -.1404972045898E+04, .1034253662109E+04},
 {    5837, .8295432617188E+04, -.1404972045898E+04, .1038145751953E+04},
 {    5838, .8298936523438E+04, -.1412592041016E+04, .1022662475586E+04},
 {    5839, .8290245117188E+04, -.1412592041016E+04, .1026553833008E+04},
 {    5840, .8298936523438E+04, -.1404972045898E+04, .1022662475586E+04},
 {    5841, .8290245117188E+04, -.1404972045898E+04, .1026553833008E+04},
 {    5842, .8293746093750E+04, -.1412592041016E+04, .1011070922852E+04},
 {    5843, .8285053710938E+04, -.1412592041016E+04, .1014962463379E+04},
 {    5844, .8293746093750E+04, -.1404972045898E+04, .1011070922852E+04},
 {    5845, .8285053710938E+04, -.1404972045898E+04, .1014962463379E+04},
 {    5846, .8292449218750E+04, -.1412592041016E+04, .1008172973633E+04},
 {    5847, .8283754882812E+04, -.1412592041016E+04, .1012065063477E+04},
 {    5848, .8292449218750E+04, -.1404972045898E+04, .1008172973633E+04},
 {    5849, .8283754882812E+04, -.1404972045898E+04, .1012065063477E+04},
 {    5850, .8305422851562E+04, -.1394812011719E+04, .1037151611328E+04},
 {    5851, .8296728515625E+04, -.1394812011719E+04, .1041043579102E+04},
 {    5852, .8305422851562E+04, -.1384652099609E+04, .1037151611328E+04},
 {    5853, .8296728515625E+04, -.1384652099609E+04, .1041043579102E+04},
 {    5854, .8304125000000E+04, -.1394812011719E+04, .1034253662109E+04},
 {    5855, .8295432617188E+04, -.1394812011719E+04, .1038145751953E+04},
 {    5856, .8304125000000E+04, -.1384652099609E+04, .1034253662109E+04},
 {    5857, .8295432617188E+04, -.1384652099609E+04, .1038145751953E+04},
 {    5858, .8293746093750E+04, -.1394812011719E+04, .1011070922852E+04},
 {    5859, .8285053710938E+04, -.1394812011719E+04, .1014962463379E+04},
 {    5860, .8293746093750E+04, -.1384652099609E+04, .1011070922852E+04},
 {    5861, .8285053710938E+04, -.1384652099609E+04, .1014962463379E+04},
 {    5862, .8292449218750E+04, -.1394812011719E+04, .1008172973633E+04},
 {    5863, .8283754882812E+04, -.1394812011719E+04, .1012065063477E+04},
 {    5864, .8292449218750E+04, -.1384652099609E+04, .1008172973633E+04},
 {    5865, .8283754882812E+04, -.1384652099609E+04, .1012065063477E+04},
 {    5866, .8305422851562E+04, -.1374492065430E+04, .1037151611328E+04},
 {    5867, .8296728515625E+04, -.1374492065430E+04, .1041043579102E+04},
 {    5868, .8304125000000E+04, -.1374492065430E+04, .1034253662109E+04},
 {    5869, .8295432617188E+04, -.1374492065430E+04, .1038145751953E+04},
 {    5870, .8298936523438E+04, -.1384652099609E+04, .1022662475586E+04},
 {    5871, .8289083984375E+04, -.1384652099609E+04, .1027073120117E+04},
 {    5872, .8298936523438E+04, -.1374492065430E+04, .1022662475586E+04},
 {    5873, .8290245117188E+04, -.1374492065430E+04, .1026553833008E+04},
 {    5874, .8293746093750E+04, -.1374492065430E+04, .1011070922852E+04},
 {    5875, .8285053710938E+04, -.1374492065430E+04, .1014962463379E+04},
 {    5876, .8292449218750E+04, -.1374492065430E+04, .1008172973633E+04},
 {    5877, .8283754882812E+04, -.1374492065430E+04, .1012065063477E+04},
 {    5878, .8288036132812E+04, -.1412592041016E+04, .1044935058594E+04},
 {    5879, .8288036132812E+04, -.1404972045898E+04, .1044935058594E+04},
 {    5880, .8286740234375E+04, -.1412592041016E+04, .1042037475586E+04},
 {    5881, .8286740234375E+04, -.1404972045898E+04, .1042037475586E+04},
 {    5882, .8281547851562E+04, -.1412592041016E+04, .1030446166992E+04},
 {    5883, .8281547851562E+04, -.1404972045898E+04, .1030446166992E+04},
 {    5884, .8276358398438E+04, -.1412592041016E+04, .1018854675293E+04},
 {    5885, .8276358398438E+04, -.1404972045898E+04, .1018854675293E+04},
 {    5886, .8275063476562E+04, -.1412592041016E+04, .1015956665039E+04},
 {    5887, .8275063476562E+04, -.1404972045898E+04, .1015956665039E+04},
 {    5888, .8288036132812E+04, -.1394812011719E+04, .1044935058594E+04},
 {    5889, .8288036132812E+04, -.1384652099609E+04, .1044935058594E+04},
 {    5890, .8286740234375E+04, -.1394812011719E+04, .1042037475586E+04},
 {    5891, .8286740234375E+04, -.1384652099609E+04, .1042037475586E+04},
 {    5892, .8276358398438E+04, -.1394812011719E+04, .1018854675293E+04},
 {    5893, .8276358398438E+04, -.1384652099609E+04, .1018854675293E+04},
 {    5894, .8275063476562E+04, -.1394812011719E+04, .1015956665039E+04},
 {    5895, .8275063476562E+04, -.1384652099609E+04, .1015956665039E+04},
 {    5896, .8288036132812E+04, -.1374492065430E+04, .1044935058594E+04},
 {    5897, .8286740234375E+04, -.1374492065430E+04, .1042037475586E+04},
 {    5898, .8281547851562E+04, -.1384652099609E+04, .1030446166992E+04},
 {    5899, .8281547851562E+04, -.1374492065430E+04, .1030446166992E+04},
 {    5900, .8276358398438E+04, -.1374492065430E+04, .1018854675293E+04},
 {    5901, .8275063476562E+04, -.1374492065430E+04, .1015956665039E+04},
 {    5902, .8410052734375E+04, -.1204849121094E+04, .1055015747070E+04},
 {    5903, .8398058593750E+04, -.1191707885742E+04, .1060385131836E+04},
 {    5904, .8387867187500E+04, -.1180542480469E+04, .1064946655273E+04},
 {    5905, .8378083984375E+04, -.1169826049805E+04, .1069325927734E+04},
 {    5906, .8413117187500E+04, -.1200779663086E+04, .1053643310547E+04},
 {    5907, .8402361328125E+04, -.1186821533203E+04, .1058457641602E+04},
 {    5908, .8392325195312E+04, -.1175824340820E+04, .1062950561523E+04},
 {    5909, .8381799804688E+04, -.1166466796875E+04, .1067663085938E+04},
 {    5910, .8416183593750E+04, -.1196710449219E+04, .1052270507812E+04},
 {    5911, .8406669921875E+04, -.1181935180664E+04, .1056529174805E+04},
 {    5912, .8396785156250E+04, -.1171106323242E+04, .1060954101562E+04},
 {    5913, .8385511718750E+04, -.1163107421875E+04, .1066000122070E+04},
 {    5914, .8407976562500E+04, -.1204849121094E+04, .1050379394531E+04},
 {    5915, .8395981445312E+04, -.1191707885742E+04, .1055748291016E+04},
 {    5916, .8385790039062E+04, -.1180542480469E+04, .1060310546875E+04},
 {    5917, .8376007812500E+04, -.1169826049805E+04, .1064689086914E+04},
 {    5918, .8411041992188E+04, -.1200779663086E+04, .1049006225586E+04},
 {    5919, .8400289062500E+04, -.1186821533203E+04, .1053820312500E+04},
 {    5920, .8390250000000E+04, -.1175824340820E+04, .1058313964844E+04},
 {    5921, .8379723632812E+04, -.1166466796875E+04, .1063026611328E+04},
 {    5922, .8402789062500E+04, -.1204849121094E+04, .1038786865234E+04},
 {    5923, .8390792968750E+04, -.1191707885742E+04, .1044156738281E+04},
 {    5924, .8380602539062E+04, -.1180542480469E+04, .1048718627930E+04},
 {    5925, .8370820312500E+04, -.1169826049805E+04, .1053097412109E+04},
 {    5926, .8405852539062E+04, -.1200779663086E+04, .1037415039062E+04},
 {    5927, .8395097656250E+04, -.1186821533203E+04, .1042229248047E+04},
 {    5928, .8385062500000E+04, -.1175824340820E+04, .1046722534180E+04},
 {    5929, .8374534179688E+04, -.1166466796875E+04, .1051435180664E+04},
 {    5930, .8397598632812E+04, -.1204849121094E+04, .1027196533203E+04},
 {    5931, .8385602539062E+04, -.1191707885742E+04, .1032565795898E+04},
 {    5932, .8375414062500E+04, -.1180542480469E+04, .1037127929688E+04},
 {    5933, .8365631835938E+04, -.1169826049805E+04, .1041506103516E+04},
 {    5934, .8400664062500E+04, -.1200779663086E+04, .1025823120117E+04},
 {    5935, .8389908203125E+04, -.1186821533203E+04, .1030637817383E+04},
 {    5936, .8379871093750E+04, -.1175824340820E+04, .1035131347656E+04},
 {    5937, .8369345703125E+04, -.1166466796875E+04, .1039843139648E+04},
 {    5938, .8396299804688E+04, -.1204849121094E+04, .1024298095703E+04},
 {    5939, .8384306640625E+04, -.1191707885742E+04, .1029667236328E+04},
 {    5940, .8374114257812E+04, -.1180542480469E+04, .1034229248047E+04},
 {    5941, .8364333984375E+04, -.1169826049805E+04, .1038607910156E+04},
 {    5942, .8399366210938E+04, -.1200779663086E+04, .1022925659180E+04},
 {    5943, .8388612304688E+04, -.1186821533203E+04, .1027739746094E+04},
 {    5944, .8378574218750E+04, -.1175824340820E+04, .1032232910156E+04},
 {    5945, .8368048828125E+04, -.1166466796875E+04, .1036945556641E+04},
 {    5946, .8391111328125E+04, -.1204849121094E+04, .1012706970215E+04},
 {    5947, .8379118164062E+04, -.1191708007812E+04, .1018075927734E+04},
 {    5948, .8368925781250E+04, -.1180542480469E+04, .1022638305664E+04},
 {    5949, .8359146484375E+04, -.1169826049805E+04, .1027016357422E+04},
 {    5950, .8394177734375E+04, -.1200779785156E+04, .1011333801270E+04},
 {    5951, .8383423828125E+04, -.1186821533203E+04, .1016148437500E+04},
 {    5952, .8373385742188E+04, -.1175824340820E+04, .1020641235352E+04},
 {    5953, .8362860351562E+04, -.1166466796875E+04, .1025353881836E+04},
 {    5954, .8385921875000E+04, -.1204849121094E+04, .1001115112305E+04},
 {    5955, .8373929687500E+04, -.1191708007812E+04, .1006484985352E+04},
 {    5956, .8363735351562E+04, -.1180542480469E+04, .1011046630859E+04},
 {    5957, .8353957031250E+04, -.1169826049805E+04, .1015425292969E+04},
 {    5958, .8388987304688E+04, -.1200779785156E+04, .9997425537109E+03},
 {    5959, .8378234375000E+04, -.1186821533203E+04, .1004556457520E+04},
 {    5960, .8368197265625E+04, -.1175824340820E+04, .1009049743652E+04},
 {    5961, .8357671875000E+04, -.1166466796875E+04, .1013762207031E+04},
 {    5962, .8384625000000E+04, -.1204849121094E+04, .9982170410156E+03},
 {    5963, .8372631835938E+04, -.1191708007812E+04, .1003586181641E+04},
 {    5964, .8362438476562E+04, -.1180542480469E+04, .1008148193359E+04},
 {    5965, .8352659179688E+04, -.1169826049805E+04, .1012526977539E+04},
 {    5966, .8387691406250E+04, -.1200779785156E+04, .9968448486328E+03},
 {    5967, .8376937500000E+04, -.1186821533203E+04, .1001658691406E+04},
 {    5968, .8366899414062E+04, -.1175824340820E+04, .1006151855469E+04},
 {    5969, .8356373046875E+04, -.1166466796875E+04, .1010864379883E+04},
 {    5970, .8379433593750E+04, -.1204849121094E+04, .9866258544922E+03},
 {    5971, .8367441406250E+04, -.1191708007812E+04, .9919949340820E+03},
 {    5972, .8357250976562E+04, -.1180542480469E+04, .9965565185547E+03},
 {    5973, .8347470703125E+04, -.1169826049805E+04, .1000935791016E+04},
 {    5974, .8382500976562E+04, -.1200779785156E+04, .9852527465820E+03},
 {    5975, .8371749023438E+04, -.1186821533203E+04, .9900670776367E+03},
 {    5976, .8361709960938E+04, -.1175824340820E+04, .9945605468750E+03},
 {    5977, .8351184570312E+04, -.1166466796875E+04, .9992730712891E+03},
 {    5978, .8374247070312E+04, -.1204849121094E+04, .9750340576172E+03},
 {    5979, .8362250976562E+04, -.1191708007812E+04, .9804036254883E+03},
 {    5980, .8352061523438E+04, -.1180542480469E+04, .9849656982422E+03},
 {    5981, .8342281250000E+04, -.1169826049805E+04, .9893442382812E+03},
 {    5982, .8377312500000E+04, -.1200779785156E+04, .9736614990234E+03},
 {    5983, .8366558593750E+04, -.1186821533203E+04, .9784755859375E+03},
 {    5984, .8356519531250E+04, -.1175824340820E+04, .9829691162109E+03},
 {    5985, .8345996093750E+04, -.1166466796875E+04, .9876814575195E+03},
 {    5986, .8372952148438E+04, -.1204849121094E+04, .9721361083984E+03},
 {    5987, .8360955078125E+04, -.1191708007812E+04, .9775054321289E+03},
 {    5988, .8350763671875E+04, -.1180542480469E+04, .9820677490234E+03},
 {    5989, .8340982421875E+04, -.1169826049805E+04, .9864465332031E+03},
 {    5990, .8376017578125E+04, -.1200779785156E+04, .9707634277344E+03},
 {    5991, .8365260742188E+04, -.1186821533203E+04, .9755781250000E+03},
 {    5992, .8355223632812E+04, -.1175824340820E+04, .9800710449219E+03},
 {    5993, .8344697265625E+04, -.1166466796875E+04, .9847835083008E+03},
 {    5994, .8367760742188E+04, -.1204849121094E+04, .9605444335938E+03},
 {    5995, .8355766601562E+04, -.1191708007812E+04, .9659140625000E+03},
 {    5996, .8345575195312E+04, -.1180542480469E+04, .9704758300781E+03},
 {    5997, .8335793945312E+04, -.1169826049805E+04, .9748542480469E+03},
 {    5998, .8370823242188E+04, -.1200779785156E+04, .9591724243164E+03},
 {    5999, .8360072265625E+04, -.1186821533203E+04, .9639863281250E+03},
 {    6000, .8350035156250E+04, -.1175824340820E+04, .9684794311523E+03},
 {    6001, .8339507812500E+04, -.1166466796875E+04, .9731922607422E+03},
 {    6002, .8362571289062E+04, -.1204849121094E+04, .9489532470703E+03},
 {    6003, .8350576171875E+04, -.1191708007812E+04, .9543225097656E+03},
 {    6004, .8340387695312E+04, -.1180542480469E+04, .9588843383789E+03},
 {    6005, .8330605468750E+04, -.1169826049805E+04, .9632632446289E+03},
 {    6006, .8365636718750E+04, -.1200779785156E+04, .9475803833008E+03},
 {    6007, .8354884765625E+04, -.1186821533203E+04, .9523945312500E+03},
 {    6008, .8344844726562E+04, -.1175824340820E+04, .9568877563477E+03},
 {    6009, .8334320312500E+04, -.1166466796875E+04, .9616002197266E+03},
 {    6010, .8361274414062E+04, -.1204849121094E+04, .9460551757812E+03},
 {    6011, .8349280273438E+04, -.1191708007812E+04, .9514244995117E+03},
 {    6012, .8339087890625E+04, -.1180542480469E+04, .9559864501953E+03},
 {    6013, .8329307617188E+04, -.1169826049805E+04, .9603653564453E+03},
 {    6014, .8364340820312E+04, -.1200779785156E+04, .9446827392578E+03},
 {    6015, .8353587890625E+04, -.1186821533203E+04, .9494965820312E+03},
 {    6016, .8343549804688E+04, -.1175824340820E+04, .9539900512695E+03},
 {    6017, .8333022460938E+04, -.1166466796875E+04, .9587027587891E+03},
 {    6018, .8356086914062E+04, -.1204849121094E+04, .9344635620117E+03},
 {    6019, .8344090820312E+04, -.1191708007812E+04, .9398328857422E+03},
 {    6020, .8333899414062E+04, -.1180542480469E+04, .9443953247070E+03},
 {    6021, .8324117187500E+04, -.1169826049805E+04, .9487739868164E+03},
 {    6022, .8359151367188E+04, -.1200779785156E+04, .9330910644531E+03},
 {    6023, .8348396484375E+04, -.1186821533203E+04, .9379051513672E+03},
 {    6024, .8338360351562E+04, -.1175824340820E+04, .9423988037109E+03},
 {    6025, .8327832031250E+04, -.1166466796875E+04, .9471110839844E+03},
 {    6026, .8350895507812E+04, -.1204849121094E+04, .9228720703125E+03},
 {    6027, .8338900390625E+04, -.1191708007812E+04, .9282416992188E+03},
 {    6028, .8328709960938E+04, -.1180542480469E+04, .9328036499023E+03},
 {    6029, .8318929687500E+04, -.1169826049805E+04, .9371823730469E+03},
 {    6030, .8353960937500E+04, -.1200779785156E+04, .9214998779297E+03},
 {    6031, .8343207031250E+04, -.1186821533203E+04, .9263137207031E+03},
 {    6032, .8333169921875E+04, -.1175824340820E+04, .9308072509766E+03},
 {    6033, .8322645507812E+04, -.1166466796875E+04, .9355192871094E+03},
 {    6034, .8348819335938E+04, -.1204849121094E+04, .9182355346680E+03},
 {    6035, .8336826171875E+04, -.1191708007812E+04, .9236049194336E+03},
 {    6036, .8326634765625E+04, -.1180542480469E+04, .9281667480469E+03},
 {    6037, .8316853515625E+04, -.1169826049805E+04, .9325457763672E+03},
 {    6038, .8351886718750E+04, -.1200779785156E+04, .9168629760742E+03},
 {    6039, .8341131835938E+04, -.1186821533203E+04, .9216771850586E+03},
 {    6040, .8331093750000E+04, -.1175824340820E+04, .9261706542969E+03},
 {    6041, .8320569335938E+04, -.1166466796875E+04, .9308826904297E+03},
 {    6042, .8354953125000E+04, -.1196710449219E+04, .9154902343750E+03},
 {    6043, .8345439453125E+04, -.1181935180664E+04, .9197494506836E+03},
 {    6044, .8335553710938E+04, -.1171106323242E+04, .9241740722656E+03},
 {    6045, .8324280273438E+04, -.1163107421875E+04, .9292204589844E+03},
 {    6046, .8378083984375E+04, -.1401358032227E+04, .1069325927734E+04},
 {    6047, .8387867187500E+04, -.1390641601562E+04, .1064946655273E+04},
 {    6048, .8397811523438E+04, -.1379745483398E+04, .1060495117188E+04},
 {    6049, .8410871093750E+04, -.1365437011719E+04, .1054648315430E+04},
 {    6050, .8381799804688E+04, -.1404717285156E+04, .1067663085938E+04},
 {    6051, .8392325195312E+04, -.1395359741211E+04, .1062950561523E+04},
 {    6052, .8402117187500E+04, -.1384631958008E+04, .1058567626953E+04},
 {    6053, .8413935546875E+04, -.1369506347656E+04, .1053276611328E+04},
 {    6054, .8385511718750E+04, -.1408076660156E+04, .1066000122070E+04},
 {    6055, .8396785156250E+04, -.1400077758789E+04, .1060954101562E+04},
 {    6056, .8406422851562E+04, -.1389518310547E+04, .1056639404297E+04},
 {    6057, .8417002929688E+04, -.1373575683594E+04, .1051903442383E+04},
 {    6058, .8376007812500E+04, -.1401358032227E+04, .1064689086914E+04},
 {    6059, .8385790039062E+04, -.1390641601562E+04, .1060310546875E+04},
 {    6060, .8395737304688E+04, -.1379745483398E+04, .1055858520508E+04},
 {    6061, .8408795898438E+04, -.1365437011719E+04, .1050012207031E+04},
 {    6062, .8379723632812E+04, -.1404717285156E+04, .1063026611328E+04},
 {    6063, .8390250000000E+04, -.1395359741211E+04, .1058313964844E+04},
 {    6064, .8400041992188E+04, -.1384631958008E+04, .1053930786133E+04},
 {    6065, .8411861328125E+04, -.1369506347656E+04, .1048639648438E+04},
 {    6066, .8370820312500E+04, -.1401358032227E+04, .1053097412109E+04},
 {    6067, .8380602539062E+04, -.1390641601562E+04, .1048718627930E+04},
 {    6068, .8390545898438E+04, -.1379745483398E+04, .1044266967773E+04},
 {    6069, .8403606445312E+04, -.1365437011719E+04, .1038420898438E+04},
 {    6070, .8374534179688E+04, -.1404717285156E+04, .1051435180664E+04},
 {    6071, .8385062500000E+04, -.1395359741211E+04, .1046722534180E+04},
 {    6072, .8394854492188E+04, -.1384631835938E+04, .1042339111328E+04},
 {    6073, .8406672851562E+04, -.1369506347656E+04, .1037047851562E+04},
 {    6074, .8365631835938E+04, -.1401358032227E+04, .1041506103516E+04},
 {    6075, .8375414062500E+04, -.1390641601562E+04, .1037127929688E+04},
 {    6076, .8385357421875E+04, -.1379745483398E+04, .1032675170898E+04},
 {    6077, .8398416015625E+04, -.1365437011719E+04, .1026828979492E+04},
 {    6078, .8369345703125E+04, -.1404717285156E+04, .1039843139648E+04},
 {    6079, .8379871093750E+04, -.1395359741211E+04, .1035131347656E+04},
 {    6080, .8389663085938E+04, -.1384631958008E+04, .1030748046875E+04},
 {    6081, .8401482421875E+04, -.1369506347656E+04, .1025456542969E+04},
 {    6082, .8364333984375E+04, -.1401358032227E+04, .1038607910156E+04},
 {    6083, .8374114257812E+04, -.1390641601562E+04, .1034229248047E+04},
 {    6084, .8384060546875E+04, -.1379745483398E+04, .1029777954102E+04},
 {    6085, .8397119140625E+04, -.1365437011719E+04, .1023931396484E+04},
 {    6086, .8368048828125E+04, -.1404717285156E+04, .1036945556641E+04},
 {    6087, .8378574218750E+04, -.1395359741211E+04, .1032232910156E+04},
 {    6088, .8388365234375E+04, -.1384631958008E+04, .1027849853516E+04},
 {    6089, .8400184570312E+04, -.1369506347656E+04, .1022558410645E+04},
 {    6090, .8359146484375E+04, -.1401358032227E+04, .1027016357422E+04},
 {    6091, .8368925781250E+04, -.1390641601562E+04, .1022638305664E+04},
 {    6092, .8378872070312E+04, -.1379745483398E+04, .1018186157227E+04},
 {    6093, .8391929687500E+04, -.1365437011719E+04, .1012339599609E+04},
 {    6094, .8362860351562E+04, -.1404717285156E+04, .1025353881836E+04},
 {    6095, .8373385742188E+04, -.1395359741211E+04, .1020641235352E+04},
 {    6096, .8383177734375E+04, -.1384631958008E+04, .1016257934570E+04},
 {    6097, .8394997070312E+04, -.1369506347656E+04, .1010966796875E+04},
 {    6098, .8353957031250E+04, -.1401358032227E+04, .1015425292969E+04},
 {    6099, .8363735351562E+04, -.1390641601562E+04, .1011046630859E+04},
 {    6100, .8373681640625E+04, -.1379745483398E+04, .1006594238281E+04},
 {    6101, .8386742187500E+04, -.1365437011719E+04, .1000747802734E+04},
 {    6102, .8357671875000E+04, -.1404717285156E+04, .1013762207031E+04},
 {    6103, .8368197265625E+04, -.1395359741211E+04, .1009049743652E+04},
 {    6104, .8377989257812E+04, -.1384631958008E+04, .1004666870117E+04},
 {    6105, .8389808593750E+04, -.1369506347656E+04, .9993753662109E+03},
 {    6106, .8352659179688E+04, -.1401358032227E+04, .1012526977539E+04},
 {    6107, .8362438476562E+04, -.1390641601562E+04, .1008148193359E+04},
 {    6108, .8372386718750E+04, -.1379745483398E+04, .1003696166992E+04},
 {    6109, .8385445312500E+04, -.1365437011719E+04, .9978502197266E+03},
 {    6110, .8356373046875E+04, -.1404717285156E+04, .1010864379883E+04},
 {    6111, .8366899414062E+04, -.1395359741211E+04, .1006151855469E+04},
 {    6112, .8376692382812E+04, -.1384631958008E+04, .1001768737793E+04},
 {    6113, .8388509765625E+04, -.1369506347656E+04, .9964776611328E+03},
 {    6114, .8347470703125E+04, -.1401358032227E+04, .1000935791016E+04},
 {    6115, .8357250976562E+04, -.1390641601562E+04, .9965565185547E+03},
 {    6116, .8367198242188E+04, -.1379745483398E+04, .9921044921875E+03},
 {    6117, .8380254882812E+04, -.1365437011719E+04, .9862587890625E+03},
 {    6118, .8351184570312E+04, -.1404717285156E+04, .9992730712891E+03},
 {    6119, .8361709960938E+04, -.1395359741211E+04, .9945605468750E+03},
 {    6120, .8371502929688E+04, -.1384631958008E+04, .9901776123047E+03},
 {    6121, .8383321289062E+04, -.1369506347656E+04, .9848861083984E+03},
 {    6122, .8342281250000E+04, -.1401358032227E+04, .9893442382812E+03},
 {    6123, .8352061523438E+04, -.1390641601562E+04, .9849656982422E+03},
 {    6124, .8362006835938E+04, -.1379745483398E+04, .9805135498047E+03},
 {    6125, .8375066406250E+04, -.1365437011719E+04, .9746672363281E+03},
 {    6126, .8345996093750E+04, -.1404717285156E+04, .9876814575195E+03},
 {    6127, .8356519531250E+04, -.1395359741211E+04, .9829691162109E+03},
 {    6128, .8366313476562E+04, -.1384631958008E+04, .9785855712891E+03},
 {    6129, .8378132812500E+04, -.1369506347656E+04, .9732945556641E+03},
 {    6130, .8340982421875E+04, -.1401358032227E+04, .9864465332031E+03},
 {    6131, .8350763671875E+04, -.1390641601562E+04, .9820677490234E+03},
 {    6132, .8360708984375E+04, -.1379745483398E+04, .9776159057617E+03},
 {    6133, .8373769531250E+04, -.1365437011719E+04, .9717693481445E+03},
 {    6134, .8344697265625E+04, -.1404717285156E+04, .9847835083008E+03},
 {    6135, .8355223632812E+04, -.1395359741211E+04, .9800710449219E+03},
 {    6136, .8365015625000E+04, -.1384631958008E+04, .9756875000000E+03},
 {    6137, .8376834960938E+04, -.1369506347656E+04, .9703967285156E+03},
 {    6138, .8335793945312E+04, -.1401358032227E+04, .9748542480469E+03},
 {    6139, .8345575195312E+04, -.1390641601562E+04, .9704758300781E+03},
 {    6140, .8355520507812E+04, -.1379745483398E+04, .9660242919922E+03},
 {    6141, .8368581054688E+04, -.1365437011719E+04, .9601776123047E+03},
 {    6142, .8339507812500E+04, -.1404717285156E+04, .9731923828125E+03},
 {    6143, .8350035156250E+04, -.1395359741211E+04, .9684794311523E+03},
 {    6144, .8359826171875E+04, -.1384631958008E+04, .9640964355469E+03},
 {    6145, .8371645507812E+04, -.1369506347656E+04, .9588054199219E+03},
 {    6146, .8330605468750E+04, -.1401358032227E+04, .9632632446289E+03},
 {    6147, .8340387695312E+04, -.1390641601562E+04, .9588843383789E+03},
 {    6148, .8350333007812E+04, -.1379745483398E+04, .9544322509766E+03},
 {    6149, .8363390625000E+04, -.1365437011719E+04, .9485863037109E+03},
 {    6150, .8334320312500E+04, -.1404717285156E+04, .9616002197266E+03},
 {    6151, .8344844726562E+04, -.1395359741211E+04, .9568877563477E+03},
 {    6152, .8354637695312E+04, -.1384631958008E+04, .9525048828125E+03},
 {    6153, .8366458007812E+04, -.1369506347656E+04, .9472134399414E+03},
 {    6154, .8329307617188E+04, -.1401358032227E+04, .9603653564453E+03},
 {    6155, .8339087890625E+04, -.1390641601562E+04, .9559864501953E+03},
 {    6156, .8349034179688E+04, -.1379745483398E+04, .9515345458984E+03},
 {    6157, .8362093750000E+04, -.1365437011719E+04, .9456883544922E+03},
 {    6158, .8333022460938E+04, -.1404717285156E+04, .9587027587891E+03},
 {    6159, .8343549804688E+04, -.1395359741211E+04, .9539900512695E+03},
 {    6160, .8353340820312E+04, -.1384631958008E+04, .9496066894531E+03},
 {    6161, .8365160156250E+04, -.1369506347656E+04, .9443159179688E+03},
 {    6162, .8324117187500E+04, -.1401358032227E+04, .9487739868164E+03},
 {    6163, .8333899414062E+04, -.1390641601562E+04, .9443954467773E+03},
 {    6164, .8343845703125E+04, -.1379745605469E+04, .9399429321289E+03},
 {    6165, .8356906250000E+04, -.1365437011719E+04, .9340969238281E+03},
 {    6166, .8327832031250E+04, -.1404717285156E+04, .9471110839844E+03},
 {    6167, .8338360351562E+04, -.1395359741211E+04, .9423988037109E+03},
 {    6168, .8348150390625E+04, -.1384631958008E+04, .9380154418945E+03},
 {    6169, .8359971679688E+04, -.1369506347656E+04, .9327243041992E+03},
 {    6170, .8318929687500E+04, -.1401358032227E+04, .9371823730469E+03},
 {    6171, .8328709960938E+04, -.1390641601562E+04, .9328036499023E+03},
 {    6172, .8338656250000E+04, -.1379745605469E+04, .9283515625000E+03},
 {    6173, .8351715820312E+04, -.1365437011719E+04, .9225053100586E+03},
 {    6174, .8322645507812E+04, -.1404717285156E+04, .9355192871094E+03},
 {    6175, .8333169921875E+04, -.1395359741211E+04, .9308072509766E+03},
 {    6176, .8342962890625E+04, -.1384631958008E+04, .9264238281250E+03},
 {    6177, .8354781250000E+04, -.1369506347656E+04, .9211328735352E+03},
 {    6178, .8316854492188E+04, -.1401358032227E+04, .9325457763672E+03},
 {    6179, .8326634765625E+04, -.1390641601562E+04, .9281667480469E+03},
 {    6180, .8336580078125E+04, -.1379745605469E+04, .9237149047852E+03},
 {    6181, .8349639648438E+04, -.1365437011719E+04, .9178687744141E+03},
 {    6182, .8320570312500E+04, -.1404717285156E+04, .9308826904297E+03},
 {    6183, .8331093750000E+04, -.1395359741211E+04, .9261706542969E+03},
 {    6184, .8340886718750E+04, -.1384631958008E+04, .9217869873047E+03},
 {    6185, .8352706054688E+04, -.1369506347656E+04, .9164960327148E+03},
 {    6186, .8324280273438E+04, -.1408076660156E+04, .9292204589844E+03},
 {    6187, .8335553710938E+04, -.1400077880859E+04, .9241740722656E+03},
 {    6188, .8345192382812E+04, -.1389518310547E+04, .9198593750000E+03},
 {    6189, .8355771484375E+04, -.1373575683594E+04, .9151234130859E+03},
 {    6190, .8259824218750E+04, -.1365437011719E+04, .1122266845703E+04},
 {    6191, .8271407226562E+04, -.1378129150391E+04, .1117081298828E+04},
 {    6192, .8284141601562E+04, -.1392078369141E+04, .1111381469727E+04},
 {    6193, .8292609375000E+04, -.1401358032227E+04, .1107589843750E+04},
 {    6194, .8256755859375E+04, -.1369506347656E+04, .1123639770508E+04},
 {    6195, .8267101562500E+04, -.1383015502930E+04, .1119008911133E+04},
 {    6196, .8279680664062E+04, -.1396796508789E+04, .1113378051758E+04},
 {    6197, .8288894531250E+04, -.1404717285156E+04, .1109252685547E+04},
 {    6198, .8253691406250E+04, -.1373575683594E+04, .1125011962891E+04},
 {    6199, .8262795898438E+04, -.1387901855469E+04, .1120936523438E+04},
 {    6200, .8275220703125E+04, -.1401514648438E+04, .1115374755859E+04},
 {    6201, .8285181640625E+04, -.1408076660156E+04, .1110915771484E+04},
 {    6202, .8257747070312E+04, -.1365437011719E+04, .1117630615234E+04},
 {    6203, .8269332031250E+04, -.1378129150391E+04, .1112445068359E+04},
 {    6204, .8282063476562E+04, -.1392078369141E+04, .1106745361328E+04},
 {    6205, .8290533203125E+04, -.1401358032227E+04, .1102953247070E+04},
 {    6206, .8254681640625E+04, -.1369506347656E+04, .1119002807617E+04},
 {    6207, .8265026367188E+04, -.1383015502930E+04, .1114372558594E+04},
 {    6208, .8277602539062E+04, -.1396796508789E+04, .1108741577148E+04},
 {    6209, .8286819335938E+04, -.1404717285156E+04, .1104615844727E+04},
 {    6210, .8252558593750E+04, -.1365437011719E+04, .1106039184570E+04},
 {    6211, .8264143554688E+04, -.1378129150391E+04, .1100852661133E+04},
 {    6212, .8276875000000E+04, -.1392078369141E+04, .1095153198242E+04},
 {    6213, .8285345703125E+04, -.1401358032227E+04, .1091361694336E+04},
 {    6214, .8249492187500E+04, -.1369506347656E+04, .1107411010742E+04},
 {    6215, .8259836914062E+04, -.1383015502930E+04, .1102780883789E+04},
 {    6216, .8272414062500E+04, -.1396796508789E+04, .1097149780273E+04},
 {    6217, .8281630859375E+04, -.1404717285156E+04, .1093024414062E+04},
 {    6218, .8247369140625E+04, -.1365437011719E+04, .1094447021484E+04},
 {    6219, .8258953125000E+04, -.1378129150391E+04, .1089261840820E+04},
 {    6220, .8271684570312E+04, -.1392078369141E+04, .1083561523438E+04},
 {    6221, .8280155273438E+04, -.1401358032227E+04, .1079770263672E+04},
 {    6222, .8244302734375E+04, -.1369506347656E+04, .1095820068359E+04},
 {    6223, .8254646484375E+04, -.1383015502930E+04, .1091188964844E+04},
 {    6224, .8267224609375E+04, -.1396796508789E+04, .1085559082031E+04},
 {    6225, .8276440429688E+04, -.1404717285156E+04, .1081433471680E+04},
 {    6226, .8246073242188E+04, -.1365437011719E+04, .1091549804688E+04},
 {    6227, .8257654296875E+04, -.1378129150391E+04, .1086363769531E+04},
 {    6228, .8270387695312E+04, -.1392078369141E+04, .1080664306641E+04},
 {    6229, .8278858398438E+04, -.1401358032227E+04, .1076872558594E+04},
 {    6230, .8243006835938E+04, -.1369506347656E+04, .1092921997070E+04},
 {    6231, .8253350585938E+04, -.1383015502930E+04, .1088291259766E+04},
 {    6232, .8265929687500E+04, -.1396796508789E+04, .1082660522461E+04},
 {    6233, .8275143554688E+04, -.1404717285156E+04, .1078535522461E+04},
 {    6234, .8240881835938E+04, -.1365437011719E+04, .1079958007812E+04},
 {    6235, .8252468750000E+04, -.1378129150391E+04, .1074772094727E+04},
 {    6236, .8265198242188E+04, -.1392078369141E+04, .1069072631836E+04},
 {    6237, .8273668945312E+04, -.1401358032227E+04, .1065280883789E+04},
 {    6238, .8237816406250E+04, -.1369506347656E+04, .1081330688477E+04},
 {    6239, .8248161132812E+04, -.1383015502930E+04, .1076699707031E+04},
 {    6240, .8260738281250E+04, -.1396796508789E+04, .1071069458008E+04},
 {    6241, .8269955078125E+04, -.1404717285156E+04, .1066943603516E+04},
 {    6242, .8235694335938E+04, -.1365437011719E+04, .1068366699219E+04},
 {    6243, .8247277343750E+04, -.1378129150391E+04, .1063180419922E+04},
 {    6244, .8260009765625E+04, -.1392078369141E+04, .1057480957031E+04},
 {    6245, .8268481445312E+04, -.1401358032227E+04, .1053689697266E+04},
 {    6246, .8232630859375E+04, -.1369506347656E+04, .1069739013672E+04},
 {    6247, .8242972656250E+04, -.1383015502930E+04, .1065108398438E+04},
 {    6248, .8255550781250E+04, -.1396796508789E+04, .1059477172852E+04},
 {    6249, .8264765625000E+04, -.1404717285156E+04, .1055351928711E+04},
 {    6250, .8234397460938E+04, -.1365437011719E+04, .1065468017578E+04},
 {    6251, .8245982421875E+04, -.1378129150391E+04, .1060282226562E+04},
 {    6252, .8258715820312E+04, -.1392078369141E+04, .1054582763672E+04},
 {    6253, .8267181640625E+04, -.1401358032227E+04, .1050791503906E+04},
 {    6254, .8231331054688E+04, -.1369506347656E+04, .1066841552734E+04},
 {    6255, .8241674804688E+04, -.1383015502930E+04, .1062209960938E+04},
 {    6256, .8254253906250E+04, -.1396796508789E+04, .1056579345703E+04},
 {    6257, .8263467773438E+04, -.1404717285156E+04, .1052454345703E+04},
 {    6258, .8229207031250E+04, -.1365437011719E+04, .1053877197266E+04},
 {    6259, .8240790039062E+04, -.1378129150391E+04, .1048691650391E+04},
 {    6260, .8253524414062E+04, -.1392078369141E+04, .1042991577148E+04},
 {    6261, .8261993164062E+04, -.1401358032227E+04, .1039200195312E+04},
 {    6262, .8226141601562E+04, -.1369506347656E+04, .1055249145508E+04},
 {    6263, .8236485351562E+04, -.1383015502930E+04, .1050618774414E+04},
 {    6264, .8249064453125E+04, -.1396796508789E+04, .1044987915039E+04},
 {    6265, .8258279296875E+04, -.1404717285156E+04, .1040862915039E+04},
 {    6266, .8224018554688E+04, -.1365437011719E+04, .1042285156250E+04},
 {    6267, .8235604492188E+04, -.1378129150391E+04, .1037099365234E+04},
 {    6268, .8248335937500E+04, -.1392078369141E+04, .1031399658203E+04},
 {    6269, .8256803710938E+04, -.1401358032227E+04, .1027608764648E+04},
 {    6270, .8220954101562E+04, -.1369506347656E+04, .1043657470703E+04},
 {    6271, .8231294921875E+04, -.1383015502930E+04, .1039027343750E+04},
 {    6272, .8243876953125E+04, -.1396796508789E+04, .1033396606445E+04},
 {    6273, .8253089843750E+04, -.1404717407227E+04, .1029271362305E+04},
 {    6274, .8222724609375E+04, -.1365437011719E+04, .1039386840820E+04},
 {    6275, .8234306640625E+04, -.1378129150391E+04, .1034201538086E+04},
 {    6276, .8247039062500E+04, -.1392078369141E+04, .1028501953125E+04},
 {    6277, .8255507812500E+04, -.1401358032227E+04, .1024711059570E+04},
 {    6278, .8219654296875E+04, -.1369506347656E+04, .1040760253906E+04},
 {    6279, .8229999023438E+04, -.1383015502930E+04, .1036129638672E+04},
 {    6280, .8242576171875E+04, -.1396796508789E+04, .1030498901367E+04},
 {    6281, .8251792968750E+04, -.1404717407227E+04, .1026373046875E+04},
 {    6282, .8217531250000E+04, -.1365437011719E+04, .1027796142578E+04},
 {    6283, .8229119140625E+04, -.1378129150391E+04, .1022610229492E+04},
 {    6284, .8241848632812E+04, -.1392078369141E+04, .1016910461426E+04},
 {    6285, .8250316406250E+04, -.1401358032227E+04, .1013119323730E+04},
 {    6286, .8214467773438E+04, -.1369506347656E+04, .1029168579102E+04},
 {    6287, .8224809570312E+04, -.1383015502930E+04, .1024538085938E+04},
 {    6288, .8237389648438E+04, -.1396796508789E+04, .1018907348633E+04},
 {    6289, .8246600585938E+04, -.1404717407227E+04, .1014782470703E+04},
 {    6290, .8212344726562E+04, -.1365437011719E+04, .1016204467773E+04},
 {    6291, .8223927734375E+04, -.1378129150391E+04, .1011018676758E+04},
 {    6292, .8236660156250E+04, -.1392078369141E+04, .1005318481445E+04},
 {    6293, .8245130859375E+04, -.1401358032227E+04, .1001527038574E+04},
 {    6294, .8209277343750E+04, -.1369506347656E+04, .1017576904297E+04},
 {    6295, .8219621093750E+04, -.1383015502930E+04, .1012946533203E+04},
 {    6296, .8232198242188E+04, -.1396796508789E+04, .1007316162109E+04},
 {    6297, .8241415039062E+04, -.1404717407227E+04, .1003190063477E+04},
 {    6298, .8211044921875E+04, -.1365437011719E+04, .1013306823730E+04},
 {    6299, .8222627929688E+04, -.1378129150391E+04, .1008120849609E+04},
 {    6300, .8235363281250E+04, -.1392078369141E+04, .1002420471191E+04},
 {    6301, .8243831054688E+04, -.1401358032227E+04, .9986295776367E+03},
 {    6302, .8207980468750E+04, -.1369506347656E+04, .1014679565430E+04},
 {    6303, .8218324218750E+04, -.1383015502930E+04, .1010048034668E+04},
 {    6304, .8230904296875E+04, -.1396796508789E+04, .1004417358398E+04},
 {    6305, .8240119140625E+04, -.1404717407227E+04, .1000292236328E+04},
 {    6306, .8205858398438E+04, -.1365437011719E+04, .1001715087891E+04},
 {    6307, .8217441406250E+04, -.1378129150391E+04, .9965289306641E+03},
 {    6308, .8230173828125E+04, -.1392078369141E+04, .9908295898438E+03},
 {    6309, .8238644531250E+04, -.1401358032227E+04, .9870379028320E+03},
 {    6310, .8202791015625E+04, -.1369506347656E+04, .1003087646484E+04},
 {    6311, .8213133789062E+04, -.1383015502930E+04, .9984571533203E+03},
 {    6312, .8225712890625E+04, -.1396796508789E+04, .9928261108398E+03},
 {    6313, .8234931640625E+04, -.1404717407227E+04, .9887004394531E+03},
 {    6314, .8200668945312E+04, -.1365437011719E+04, .9901234130859E+03},
 {    6315, .8212250976562E+04, -.1378129150391E+04, .9849379272461E+03},
 {    6316, .8224984375000E+04, -.1392078369141E+04, .9792382812500E+03},
 {    6317, .8233452148438E+04, -.1401358032227E+04, .9754467773438E+03},
 {    6318, .8197600585938E+04, -.1369506347656E+04, .9914962158203E+03},
 {    6319, .8207945312500E+04, -.1383015502930E+04, .9868653564453E+03},
 {    6320, .8220524414062E+04, -.1396796508789E+04, .9812348632812E+03},
 {    6321, .8229739257812E+04, -.1404717407227E+04, .9771093750000E+03},
 {    6322, .8198592773438E+04, -.1365437011719E+04, .9854865722656E+03},
 {    6323, .8210175781250E+04, -.1378129150391E+04, .9803011474609E+03},
 {    6324, .8222911132812E+04, -.1392078369141E+04, .9746013183594E+03},
 {    6325, .8231378906250E+04, -.1401358032227E+04, .9708096923828E+03},
 {    6326, .8195523437500E+04, -.1369506347656E+04, .9868599243164E+03},
 {    6327, .8205870117188E+04, -.1383015502930E+04, .9822286987305E+03},
 {    6328, .8218449218750E+04, -.1396796508789E+04, .9765979003906E+03},
 {    6329, .8227663085938E+04, -.1404717407227E+04, .9724725341797E+03},
 {    6330, .8192460937500E+04, -.1373575683594E+04, .9882319335938E+03},
 {    6331, .8201565429688E+04, -.1387901855469E+04, .9841562500000E+03},
 {    6332, .8213990234375E+04, -.1401514648438E+04, .9785944824219E+03},
 {    6333, .8223949218750E+04, -.1408076660156E+04, .9741352539062E+03},
 {    6334, .8292609375000E+04, -.1169826049805E+04, .1107589843750E+04},
 {    6335, .8284302734375E+04, -.1178926025391E+04, .1111308227539E+04},
 {    6336, .8271407226562E+04, -.1193054931641E+04, .1117081298828E+04},
 {    6337, .8259824218750E+04, -.1205747070312E+04, .1122266845703E+04},
 {    6338, .8288894531250E+04, -.1166466796875E+04, .1109252685547E+04},
 {    6339, .8279843750000E+04, -.1174208007812E+04, .1113304687500E+04},
 {    6340, .8267101562500E+04, -.1188168579102E+04, .1119008911133E+04},
 {    6341, .8256755859375E+04, -.1201677734375E+04, .1123639770508E+04},
 {    6342, .8285181640625E+04, -.1163107421875E+04, .1110915527344E+04},
 {    6343, .8275382812500E+04, -.1169489868164E+04, .1115301147461E+04},
 {    6344, .8262795898438E+04, -.1183282226562E+04, .1120936523438E+04},
 {    6345, .8253691406250E+04, -.1197608398438E+04, .1125011962891E+04},
 {    6346, .8290533203125E+04, -.1169826049805E+04, .1102953247070E+04},
 {    6347, .8282228515625E+04, -.1178926025391E+04, .1106671508789E+04},
 {    6348, .8269332031250E+04, -.1193054931641E+04, .1112445068359E+04},
 {    6349, .8257747070312E+04, -.1205747070312E+04, .1117630615234E+04},
 {    6350, .8286819335938E+04, -.1166466796875E+04, .1104615844727E+04},
 {    6351, .8277768554688E+04, -.1174208007812E+04, .1108668212891E+04},
 {    6352, .8265026367188E+04, -.1188168579102E+04, .1114372558594E+04},
 {    6353, .8254681640625E+04, -.1201677734375E+04, .1119002807617E+04},
 {    6354, .8285345703125E+04, -.1169826049805E+04, .1091361694336E+04},
 {    6355, .8277039062500E+04, -.1178926025391E+04, .1095079833984E+04},
 {    6356, .8264143554688E+04, -.1193054931641E+04, .1100852661133E+04},
 {    6357, .8252558593750E+04, -.1205747070312E+04, .1106039184570E+04},
 {    6358, .8281630859375E+04, -.1166466796875E+04, .1093024414062E+04},
 {    6359, .8272578125000E+04, -.1174208007812E+04, .1097076660156E+04},
 {    6360, .8259836914062E+04, -.1188168579102E+04, .1102780883789E+04},
 {    6361, .8249492187500E+04, -.1201677734375E+04, .1107411010742E+04},
 {    6362, .8280155273438E+04, -.1169826049805E+04, .1079770263672E+04},
 {    6363, .8271850585938E+04, -.1178926025391E+04, .1083488525391E+04},
 {    6364, .8258953125000E+04, -.1193054931641E+04, .1089261840820E+04},
 {    6365, .8247369140625E+04, -.1205747070312E+04, .1094447021484E+04},
 {    6366, .8276440429688E+04, -.1166466796875E+04, .1081433471680E+04},
 {    6367, .8267388671875E+04, -.1174208007812E+04, .1085485229492E+04},
 {    6368, .8254646484375E+04, -.1188168579102E+04, .1091188964844E+04},
 {    6369, .8244302734375E+04, -.1201677734375E+04, .1095820068359E+04},
 {    6370, .8278858398438E+04, -.1169826049805E+04, .1076872558594E+04},
 {    6371, .8270551757812E+04, -.1178926025391E+04, .1080590698242E+04},
 {    6372, .8257654296875E+04, -.1193054931641E+04, .1086363769531E+04},
 {    6373, .8246073242188E+04, -.1205747070312E+04, .1091549804688E+04},
 {    6374, .8275143554688E+04, -.1166466796875E+04, .1078535522461E+04},
 {    6375, .8266089843750E+04, -.1174208007812E+04, .1082587646484E+04},
 {    6376, .8253350585938E+04, -.1188168579102E+04, .1088291259766E+04},
 {    6377, .8243006835938E+04, -.1201677734375E+04, .1092921997070E+04},
 {    6378, .8273668945312E+04, -.1169826049805E+04, .1065280883789E+04},
 {    6379, .8265365234375E+04, -.1178926025391E+04, .1068999267578E+04},
 {    6380, .8252468750000E+04, -.1193054931641E+04, .1074772094727E+04},
 {    6381, .8240881835938E+04, -.1205747070312E+04, .1079958007812E+04},
 {    6382, .8269955078125E+04, -.1166466796875E+04, .1066943603516E+04},
 {    6383, .8260902343750E+04, -.1174208007812E+04, .1070995483398E+04},
 {    6384, .8248161132812E+04, -.1188168579102E+04, .1076699707031E+04},
 {    6385, .8237816406250E+04, -.1201677734375E+04, .1081330688477E+04},
 {    6386, .8268481445312E+04, -.1169826049805E+04, .1053689697266E+04},
 {    6387, .8260173828125E+04, -.1178926025391E+04, .1057406982422E+04},
 {    6388, .8247277343750E+04, -.1193054931641E+04, .1063180419922E+04},
 {    6389, .8235694335938E+04, -.1205747070312E+04, .1068366699219E+04},
 {    6390, .8264765625000E+04, -.1166466796875E+04, .1055351928711E+04},
 {    6391, .8255714843750E+04, -.1174208007812E+04, .1059404296875E+04},
 {    6392, .8242972656250E+04, -.1188168579102E+04, .1065108398438E+04},
 {    6393, .8232630859375E+04, -.1201677734375E+04, .1069739013672E+04},
 {    6394, .8267181640625E+04, -.1169826049805E+04, .1050791503906E+04},
 {    6395, .8258877929688E+04, -.1178926025391E+04, .1054509521484E+04},
 {    6396, .8245982421875E+04, -.1193054931641E+04, .1060282226562E+04},
 {    6397, .8234397460938E+04, -.1205747070312E+04, .1065468017578E+04},
 {    6398, .8263467773438E+04, -.1166466796875E+04, .1052454345703E+04},
 {    6399, .8254417968750E+04, -.1174208007812E+04, .1056506103516E+04},
 {    6400, .8241674804688E+04, -.1188168579102E+04, .1062209960938E+04},
 {    6401, .8231331054688E+04, -.1201677734375E+04, .1066841552734E+04},
 {    6402, .8261993164062E+04, -.1169826049805E+04, .1039200195312E+04},
 {    6403, .8253686523438E+04, -.1178926025391E+04, .1042918701172E+04},
 {    6404, .8240790039062E+04, -.1193054931641E+04, .1048691650391E+04},
 {    6405, .8229207031250E+04, -.1205747070312E+04, .1053877197266E+04},
 {    6406, .8258279296875E+04, -.1166466796875E+04, .1040862915039E+04},
 {    6407, .8249225585938E+04, -.1174208007812E+04, .1044915283203E+04},
 {    6408, .8236485351562E+04, -.1188168579102E+04, .1050618774414E+04},
 {    6409, .8226141601562E+04, -.1201677734375E+04, .1055249145508E+04},
 {    6410, .8256803710938E+04, -.1169826049805E+04, .1027608764648E+04},
 {    6411, .8248498046875E+04, -.1178926025391E+04, .1031326782227E+04},
 {    6412, .8235604492188E+04, -.1193054931641E+04, .1037099365234E+04},
 {    6413, .8224018554688E+04, -.1205747070312E+04, .1042285156250E+04},
 {    6414, .8253089843750E+04, -.1166466796875E+04, .1029271362305E+04},
 {    6415, .8244037109375E+04, -.1174208007812E+04, .1033322998047E+04},
 {    6416, .8231294921875E+04, -.1188168579102E+04, .1039027343750E+04},
 {    6417, .8220954101562E+04, -.1201677734375E+04, .1043657470703E+04},
 {    6418, .8255507812500E+04, -.1169826049805E+04, .1024711059570E+04},
 {    6419, .8247202148438E+04, -.1178926025391E+04, .1028429077148E+04},
 {    6420, .8234306640625E+04, -.1193054931641E+04, .1034201538086E+04},
 {    6421, .8222724609375E+04, -.1205747070312E+04, .1039386840820E+04},
 {    6422, .8251792968750E+04, -.1166466796875E+04, .1026373046875E+04},
 {    6423, .8242741210938E+04, -.1174208007812E+04, .1030425170898E+04},
 {    6424, .8229999023438E+04, -.1188168579102E+04, .1036129638672E+04},
 {    6425, .8219654296875E+04, -.1201677734375E+04, .1040760253906E+04},
 {    6426, .8250316406250E+04, -.1169826049805E+04, .1013119323730E+04},
 {    6427, .8242013671875E+04, -.1178926025391E+04, .1016836730957E+04},
 {    6428, .8229119140625E+04, -.1193054931641E+04, .1022610229492E+04},
 {    6429, .8217531250000E+04, -.1205747070312E+04, .1027796142578E+04},
 {    6430, .8246600585938E+04, -.1166466796875E+04, .1014782470703E+04},
 {    6431, .8237552734375E+04, -.1174208007812E+04, .1018833374023E+04},
 {    6432, .8224809570312E+04, -.1188168579102E+04, .1024538085938E+04},
 {    6433, .8214467773438E+04, -.1201677734375E+04, .1029168579102E+04},
 {    6434, .8245130859375E+04, -.1169826049805E+04, .1001527038574E+04},
 {    6435, .8236822265625E+04, -.1178926025391E+04, .1005246093750E+04},
 {    6436, .8223927734375E+04, -.1193054931641E+04, .1011018432617E+04},
 {    6437, .8212344726562E+04, -.1205747070312E+04, .1016204467773E+04},
 {    6438, .8241415039062E+04, -.1166466796875E+04, .1003190063477E+04},
 {    6439, .8232363281250E+04, -.1174208007812E+04, .1007242065430E+04},
 {    6440, .8219621093750E+04, -.1188168579102E+04, .1012946533203E+04},
 {    6441, .8209277343750E+04, -.1201677734375E+04, .1017576904297E+04},
 {    6442, .8243831054688E+04, -.1169826049805E+04, .9986295776367E+03},
 {    6443, .8235526367188E+04, -.1178926025391E+04, .1002347900391E+04},
 {    6444, .8222627929688E+04, -.1193054931641E+04, .1008120849609E+04},
 {    6445, .8211044921875E+04, -.1205747070312E+04, .1013306823730E+04},
 {    6446, .8240119140625E+04, -.1166466796875E+04, .1000292236328E+04},
 {    6447, .8231068359375E+04, -.1174208007812E+04, .1004343811035E+04},
 {    6448, .8218324218750E+04, -.1188168579102E+04, .1010048034668E+04},
 {    6449, .8207980468750E+04, -.1201677734375E+04, .1014679565430E+04},
 {    6450, .8238644531250E+04, -.1169826049805E+04, .9870379028320E+03},
 {    6451, .8230337890625E+04, -.1178926025391E+04, .9907561035156E+03},
 {    6452, .8217441406250E+04, -.1193054931641E+04, .9965289306641E+03},
 {    6453, .8205858398438E+04, -.1205747070312E+04, .1001715087891E+04},
 {    6454, .8234931640625E+04, -.1166466796875E+04, .9887003173828E+03},
 {    6455, .8225876953125E+04, -.1174208007812E+04, .9927526855469E+03},
 {    6456, .8213133789062E+04, -.1188168579102E+04, .9984571533203E+03},
 {    6457, .8202791015625E+04, -.1201677734375E+04, .1003087402344E+04},
 {    6458, .8233452148438E+04, -.1169826049805E+04, .9754467773438E+03},
 {    6459, .8225150390625E+04, -.1178926025391E+04, .9791641845703E+03},
 {    6460, .8212250976562E+04, -.1193054931641E+04, .9849378051758E+03},
 {    6461, .8200668945312E+04, -.1205747070312E+04, .9901234130859E+03},
 {    6462, .8229739257812E+04, -.1166466796875E+04, .9771093750000E+03},
 {    6463, .8220689453125E+04, -.1174208007812E+04, .9811609497070E+03},
 {    6464, .8207945312500E+04, -.1188168579102E+04, .9868653564453E+03},
 {    6465, .8197600585938E+04, -.1201677734375E+04, .9914962158203E+03},
 {    6466, .8231378906250E+04, -.1169826049805E+04, .9708096923828E+03},
 {    6467, .8223073242188E+04, -.1178926025391E+04, .9745277099609E+03},
 {    6468, .8210175781250E+04, -.1193054931641E+04, .9803011474609E+03},
 {    6469, .8198592773438E+04, -.1205747070312E+04, .9854865722656E+03},
 {    6470, .8227663085938E+04, -.1166466796875E+04, .9724725341797E+03},
 {    6471, .8218613281250E+04, -.1174208007812E+04, .9765247192383E+03},
 {    6472, .8205870117188E+04, -.1188168579102E+04, .9822286987305E+03},
 {    6473, .8195523437500E+04, -.1201677734375E+04, .9868599243164E+03},
 {    6474, .8223949218750E+04, -.1163107421875E+04, .9741352539062E+03},
 {    6475, .8214153320312E+04, -.1169489868164E+04, .9785209350586E+03},
 {    6476, .8201565429688E+04, -.1183282226562E+04, .9841562500000E+03},
 {    6477, .8192460937500E+04, -.1197608398438E+04, .9882318115234E+03},
 {    6478, .8235901367188E+04, -.1323692016602E+04, .1127409667969E+04},
 {    6479, .8235901367188E+04, -.1305911987305E+04, .1127409667969E+04},
 {    6480, .8235901367188E+04, -.1288132080078E+04, .1127409667969E+04},
 {    6481, .8230715820312E+04, -.1323692016602E+04, .1115818359375E+04},
 {    6482, .8230715820312E+04, -.1305911987305E+04, .1115818359375E+04},
 {    6483, .8230715820312E+04, -.1288132080078E+04, .1115818359375E+04},
 {    6484, .8225525390625E+04, -.1323692016602E+04, .1104226562500E+04},
 {    6485, .8225525390625E+04, -.1305911987305E+04, .1104226562500E+04},
 {    6486, .8225525390625E+04, -.1288132080078E+04, .1104226562500E+04},
 {    6487, .8224225585938E+04, -.1323692016602E+04, .1101328613281E+04},
 {    6488, .8224225585938E+04, -.1305911987305E+04, .1101328613281E+04},
 {    6489, .8224225585938E+04, -.1288132080078E+04, .1101328613281E+04},
 {    6490, .8219039062500E+04, -.1323692016602E+04, .1089737060547E+04},
 {    6491, .8219039062500E+04, -.1305911987305E+04, .1089737060547E+04},
 {    6492, .8219039062500E+04, -.1288132080078E+04, .1089737060547E+04},
 {    6493, .8213849609375E+04, -.1323692016602E+04, .1078145507812E+04},
 {    6494, .8213849609375E+04, -.1305911987305E+04, .1078145507812E+04},
 {    6495, .8213849609375E+04, -.1288132080078E+04, .1078145507812E+04},
 {    6496, .8212553710938E+04, -.1323692016602E+04, .1075247558594E+04},
 {    6497, .8212553710938E+04, -.1305911987305E+04, .1075247558594E+04},
 {    6498, .8212553710938E+04, -.1288132080078E+04, .1075247558594E+04},
 {    6499, .8207362304688E+04, -.1323692016602E+04, .1063656616211E+04},
 {    6500, .8207362304688E+04, -.1305911987305E+04, .1063656616211E+04},
 {    6501, .8207362304688E+04, -.1288132080078E+04, .1063656616211E+04},
 {    6502, .8202173828125E+04, -.1323692016602E+04, .1052065063477E+04},
 {    6503, .8202173828125E+04, -.1305911987305E+04, .1052065063477E+04},
 {    6504, .8202173828125E+04, -.1288132080078E+04, .1052065063477E+04},
 {    6505, .8200876953125E+04, -.1323692016602E+04, .1049166992188E+04},
 {    6506, .8200876953125E+04, -.1305911987305E+04, .1049166992188E+04},
 {    6507, .8200876953125E+04, -.1288132080078E+04, .1049166992188E+04},
 {    6508, .8195686523438E+04, -.1323692016602E+04, .1037575439453E+04},
 {    6509, .8195686523438E+04, -.1305912109375E+04, .1037575439453E+04},
 {    6510, .8195686523438E+04, -.1288132080078E+04, .1037575439453E+04},
 {    6511, .8190498535156E+04, -.1323692016602E+04, .1025983886719E+04},
 {    6512, .8190498535156E+04, -.1305912109375E+04, .1025983886719E+04},
 {    6513, .8190498535156E+04, -.1288132080078E+04, .1025983886719E+04},
 {    6514, .8189201660156E+04, -.1323692016602E+04, .1023085693359E+04},
 {    6515, .8189201660156E+04, -.1305912109375E+04, .1023085693359E+04},
 {    6516, .8189201660156E+04, -.1288132080078E+04, .1023085693359E+04},
 {    6517, .8184013183594E+04, -.1323692016602E+04, .1011494079590E+04},
 {    6518, .8184013183594E+04, -.1305912109375E+04, .1011494079590E+04},
 {    6519, .8184013183594E+04, -.1288132080078E+04, .1011494079590E+04},
 {    6520, .8178822265625E+04, -.1323692016602E+04, .9999027709961E+03},
 {    6521, .8178822265625E+04, -.1305912109375E+04, .9999027709961E+03},
 {    6522, .8178822265625E+04, -.1288132080078E+04, .9999027709961E+03},
 {    6523, .8235901367188E+04, -.1283052001953E+04, .1127409667969E+04},
 {    6524, .8230715820312E+04, -.1283052001953E+04, .1115818359375E+04},
 {    6525, .8225525390625E+04, -.1283052001953E+04, .1104226562500E+04},
 {    6526, .8224225585938E+04, -.1283052001953E+04, .1101328613281E+04},
 {    6527, .8219039062500E+04, -.1283052001953E+04, .1089737060547E+04},
 {    6528, .8213849609375E+04, -.1283052001953E+04, .1078145507812E+04},
 {    6529, .8212553710938E+04, -.1283052001953E+04, .1075247558594E+04},
 {    6530, .8207362304688E+04, -.1283052001953E+04, .1063656616211E+04},
 {    6531, .8202173828125E+04, -.1283052001953E+04, .1052065063477E+04},
 {    6532, .8200876953125E+04, -.1283052001953E+04, .1049166992188E+04},
 {    6533, .8195686523438E+04, -.1283052001953E+04, .1037575439453E+04},
 {    6534, .8190498535156E+04, -.1283052001953E+04, .1025983886719E+04},
 {    6535, .8189201660156E+04, -.1283052001953E+04, .1023085693359E+04},
 {    6536, .8184013183594E+04, -.1283052001953E+04, .1011494079590E+04},
 {    6537, .8178822265625E+04, -.1283052001953E+04, .9999027709961E+03},
 {    6538, .8235901367188E+04, -.1265272094727E+04, .1127409667969E+04},
 {    6539, .8235901367188E+04, -.1247492065430E+04, .1127409667969E+04},
 {    6540, .8230715820312E+04, -.1265272094727E+04, .1115818359375E+04},
 {    6541, .8230715820312E+04, -.1247492065430E+04, .1115818359375E+04},
 {    6542, .8225525390625E+04, -.1265272094727E+04, .1104226562500E+04},
 {    6543, .8225525390625E+04, -.1247492065430E+04, .1104226562500E+04},
 {    6544, .8224225585938E+04, -.1265272094727E+04, .1101328613281E+04},
 {    6545, .8224225585938E+04, -.1247492065430E+04, .1101328613281E+04},
 {    6546, .8219039062500E+04, -.1265272094727E+04, .1089737060547E+04},
 {    6547, .8219039062500E+04, -.1247492065430E+04, .1089737060547E+04},
 {    6548, .8213849609375E+04, -.1265272094727E+04, .1078145507812E+04},
 {    6549, .8213849609375E+04, -.1247492065430E+04, .1078145507812E+04},
 {    6550, .8212553710938E+04, -.1265272094727E+04, .1075247558594E+04},
 {    6551, .8212553710938E+04, -.1247492065430E+04, .1075247558594E+04},
 {    6552, .8207362304688E+04, -.1265272094727E+04, .1063656616211E+04},
 {    6553, .8207362304688E+04, -.1247492065430E+04, .1063656616211E+04},
 {    6554, .8202173828125E+04, -.1265272094727E+04, .1052065063477E+04},
 {    6555, .8202173828125E+04, -.1247492065430E+04, .1052065063477E+04},
 {    6556, .8200876953125E+04, -.1265272094727E+04, .1049166992188E+04},
 {    6557, .8200876953125E+04, -.1247492065430E+04, .1049166992188E+04},
 {    6558, .8195686523438E+04, -.1265272094727E+04, .1037575439453E+04},
 {    6559, .8195686523438E+04, -.1247492065430E+04, .1037575439453E+04},
 {    6560, .8190498535156E+04, -.1265272094727E+04, .1025983886719E+04},
 {    6561, .8190498535156E+04, -.1247492065430E+04, .1025983886719E+04},
 {    6562, .8189201660156E+04, -.1265272094727E+04, .1023085693359E+04},
 {    6563, .8189201660156E+04, -.1247492065430E+04, .1023085693359E+04},
 {    6564, .8184013183594E+04, -.1265272094727E+04, .1011494079590E+04},
 {    6565, .8184013183594E+04, -.1247492065430E+04, .1011494079590E+04},
 {    6566, .8178822265625E+04, -.1265272094727E+04, .9999027709961E+03},
 {    6567, .8178822265625E+04, -.1247492065430E+04, .9999027709961E+03},
 {    6568, .8254424804688E+04, -.1336392089844E+04, .1133033447266E+04},
 {    6569, .8260219726562E+04, -.1336392089844E+04, .1130439453125E+04},
 {    6570, .8254424804688E+04, -.1323692016602E+04, .1133033447266E+04},
 {    6571, .8260219726562E+04, -.1323692016602E+04, .1130439453125E+04},
 {    6572, .8249233398438E+04, -.1336392089844E+04, .1121441894531E+04},
 {    6573, .8255029296875E+04, -.1336392089844E+04, .1118847412109E+04},
 {    6574, .8249233398438E+04, -.1323692016602E+04, .1121441894531E+04},
 {    6575, .8255029296875E+04, -.1323692016602E+04, .1118847412109E+04},
 {    6576, .8244044921875E+04, -.1336392089844E+04, .1109850585938E+04},
 {    6577, .8249837890625E+04, -.1336392089844E+04, .1107256591797E+04},
 {    6578, .8244044921875E+04, -.1323692016602E+04, .1109850585938E+04},
 {    6579, .8249837890625E+04, -.1323692016602E+04, .1107256591797E+04},
 {    6580, .8238854492188E+04, -.1336392089844E+04, .1098259033203E+04},
 {    6581, .8244649414062E+04, -.1336392089844E+04, .1095664794922E+04},
 {    6582, .8238854492188E+04, -.1323692016602E+04, .1098259033203E+04},
 {    6583, .8244649414062E+04, -.1323692016602E+04, .1095664794922E+04},
 {    6584, .8237558593750E+04, -.1336392089844E+04, .1095361206055E+04},
 {    6585, .8243356445312E+04, -.1336392089844E+04, .1092766601562E+04},
 {    6586, .8237558593750E+04, -.1323692016602E+04, .1095361206055E+04},
 {    6587, .8243356445312E+04, -.1323692016602E+04, .1092766601562E+04},
 {    6588, .8232369140625E+04, -.1336392089844E+04, .1083769287109E+04},
 {    6589, .8238166015625E+04, -.1336392089844E+04, .1081174926758E+04},
 {    6590, .8232369140625E+04, -.1323692016602E+04, .1083769287109E+04},
 {    6591, .8238166015625E+04, -.1323692016602E+04, .1081174926758E+04},
 {    6592, .8227179687500E+04, -.1336392089844E+04, .1072178100586E+04},
 {    6593, .8232976562500E+04, -.1336392089844E+04, .1069583129883E+04},
 {    6594, .8227179687500E+04, -.1323692016602E+04, .1072178100586E+04},
 {    6595, .8232976562500E+04, -.1323692016602E+04, .1069583129883E+04},
 {    6596, .8225882812500E+04, -.1336392089844E+04, .1069280273438E+04},
 {    6597, .8231680664062E+04, -.1336392089844E+04, .1066685424805E+04},
 {    6598, .8225882812500E+04, -.1323692016602E+04, .1069280273438E+04},
 {    6599, .8231680664062E+04, -.1323692016602E+04, .1066685424805E+04},
 {    6600, .8220692382812E+04, -.1336392089844E+04, .1057688842773E+04},
 {    6601, .8226489257812E+04, -.1336392089844E+04, .1055094482422E+04},
 {    6602, .8220692382812E+04, -.1323692016602E+04, .1057688842773E+04},
 {    6603, .8226489257812E+04, -.1323692016602E+04, .1055094482422E+04},
 {    6604, .8215502929688E+04, -.1336392089844E+04, .1046097167969E+04},
 {    6605, .8221301757812E+04, -.1336392089844E+04, .1043502807617E+04},
 {    6606, .8215502929688E+04, -.1323692016602E+04, .1046097167969E+04},
 {    6607, .8221301757812E+04, -.1323692016602E+04, .1043502807617E+04},
 {    6608, .8214208007812E+04, -.1336392089844E+04, .1043198730469E+04},
 {    6609, .8220001953125E+04, -.1336392089844E+04, .1040605102539E+04},
 {    6610, .8214208007812E+04, -.1323692016602E+04, .1043198730469E+04},
 {    6611, .8220001953125E+04, -.1323692016602E+04, .1040605102539E+04},
 {    6612, .8209017578125E+04, -.1336392089844E+04, .1031608154297E+04},
 {    6613, .8214813476562E+04, -.1336392089844E+04, .1029013183594E+04},
 {    6614, .8209017578125E+04, -.1323692016602E+04, .1031608154297E+04},
 {    6615, .8214813476562E+04, -.1323692016602E+04, .1029013183594E+04},
 {    6616, .8203829101562E+04, -.1336392089844E+04, .1020016174316E+04},
 {    6617, .8209624023438E+04, -.1336392089844E+04, .1017422119141E+04},
 {    6618, .8203829101562E+04, -.1323692016602E+04, .1020016174316E+04},
 {    6619, .8209624023438E+04, -.1323692016602E+04, .1017422119141E+04},
 {    6620, .8202531250000E+04, -.1336392089844E+04, .1017118347168E+04},
 {    6621, .8208327148438E+04, -.1336392089844E+04, .1014523742676E+04},
 {    6622, .8202531250000E+04, -.1323692016602E+04, .1017118347168E+04},
 {    6623, .8208327148438E+04, -.1323692016602E+04, .1014523742676E+04},
 {    6624, .8197340820312E+04, -.1336392089844E+04, .1005527099609E+04},
 {    6625, .8203137695312E+04, -.1336392089844E+04, .1002932495117E+04},
 {    6626, .8197340820312E+04, -.1323692016602E+04, .1005527099609E+04},
 {    6627, .8203137695312E+04, -.1323692016602E+04, .1002932495117E+04},
 {    6628, .8192153320312E+04, -.1336392089844E+04, .9939351806641E+03},
 {    6629, .8197948242188E+04, -.1336392089844E+04, .9913408813477E+03},
 {    6630, .8192153320312E+04, -.1323692016602E+04, .9939351806641E+03},
 {    6631, .8197948242188E+04, -.1323692016602E+04, .9913408813477E+03},
 {    6632, .8186963867188E+04, -.1336392089844E+04, .9823437500000E+03},
 {    6633, .8192758789062E+04, -.1336392089844E+04, .9797492065430E+03},
 {    6634, .8186963867188E+04, -.1323692016602E+04, .9823437500000E+03},
 {    6635, .8192758789062E+04, -.1323692016602E+04, .9797492065430E+03},
 {    6636, .8254424804688E+04, -.1305911987305E+04, .1133033447266E+04},
 {    6637, .8260219726562E+04, -.1305911987305E+04, .1130439453125E+04},
 {    6638, .8254424804688E+04, -.1288132080078E+04, .1133033447266E+04},
 {    6639, .8260219726562E+04, -.1288132080078E+04, .1130439453125E+04},
 {    6640, .8249233398438E+04, -.1305911987305E+04, .1121441894531E+04},
 {    6641, .8255029296875E+04, -.1305911987305E+04, .1118847412109E+04},
 {    6642, .8249233398438E+04, -.1288132080078E+04, .1121441894531E+04},
 {    6643, .8255029296875E+04, -.1288132080078E+04, .1118847412109E+04},
 {    6644, .8244044921875E+04, -.1305911987305E+04, .1109850585938E+04},
 {    6645, .8249837890625E+04, -.1305911987305E+04, .1107256591797E+04},
 {    6646, .8244044921875E+04, -.1288132080078E+04, .1109850585938E+04},
 {    6647, .8249837890625E+04, -.1288132080078E+04, .1107256591797E+04},
 {    6648, .8238854492188E+04, -.1305911987305E+04, .1098259033203E+04},
 {    6649, .8244649414062E+04, -.1305911987305E+04, .1095664794922E+04},
 {    6650, .8238854492188E+04, -.1288132080078E+04, .1098259033203E+04},
 {    6651, .8244649414062E+04, -.1288132080078E+04, .1095664794922E+04},
 {    6652, .8237558593750E+04, -.1305911987305E+04, .1095361206055E+04},
 {    6653, .8243356445312E+04, -.1305911987305E+04, .1092766601562E+04},
 {    6654, .8237558593750E+04, -.1288132080078E+04, .1095361206055E+04},
 {    6655, .8243356445312E+04, -.1288132080078E+04, .1092766601562E+04},
 {    6656, .8232369140625E+04, -.1305911987305E+04, .1083769287109E+04},
 {    6657, .8238166015625E+04, -.1305911987305E+04, .1081174926758E+04},
 {    6658, .8232369140625E+04, -.1288132080078E+04, .1083769287109E+04},
 {    6659, .8238166015625E+04, -.1288132080078E+04, .1081174926758E+04},
 {    6660, .8227179687500E+04, -.1305911987305E+04, .1072178100586E+04},
 {    6661, .8232976562500E+04, -.1305911987305E+04, .1069583129883E+04},
 {    6662, .8227179687500E+04, -.1288132080078E+04, .1072178100586E+04},
 {    6663, .8232976562500E+04, -.1288132080078E+04, .1069583129883E+04},
 {    6664, .8225882812500E+04, -.1305911987305E+04, .1069280273438E+04},
 {    6665, .8231680664062E+04, -.1305911987305E+04, .1066685424805E+04},
 {    6666, .8225882812500E+04, -.1288132080078E+04, .1069280273438E+04},
 {    6667, .8231680664062E+04, -.1288132080078E+04, .1066685424805E+04},
 {    6668, .8220692382812E+04, -.1305911987305E+04, .1057688842773E+04},
 {    6669, .8226489257812E+04, -.1305911987305E+04, .1055094482422E+04},
 {    6670, .8220692382812E+04, -.1288132080078E+04, .1057688842773E+04},
 {    6671, .8226489257812E+04, -.1288132080078E+04, .1055094482422E+04},
 {    6672, .8215502929688E+04, -.1305911987305E+04, .1046097167969E+04},
 {    6673, .8221301757812E+04, -.1305911987305E+04, .1043502807617E+04},
 {    6674, .8215502929688E+04, -.1288132080078E+04, .1046097167969E+04},
 {    6675, .8221301757812E+04, -.1288132080078E+04, .1043502807617E+04},
 {    6676, .8214208007812E+04, -.1305911987305E+04, .1043198730469E+04},
 {    6677, .8220001953125E+04, -.1305911987305E+04, .1040605102539E+04},
 {    6678, .8214208007812E+04, -.1288132080078E+04, .1043198730469E+04},
 {    6679, .8220001953125E+04, -.1288132080078E+04, .1040605102539E+04},
 {    6680, .8209017578125E+04, -.1305912109375E+04, .1031608154297E+04},
 {    6681, .8214813476562E+04, -.1305912109375E+04, .1029013183594E+04},
 {    6682, .8209017578125E+04, -.1288132080078E+04, .1031608154297E+04},
 {    6683, .8214813476562E+04, -.1288132080078E+04, .1029013183594E+04},
 {    6684, .8203829101562E+04, -.1305912109375E+04, .1020016174316E+04},
 {    6685, .8209624023438E+04, -.1305912109375E+04, .1017422119141E+04},
 {    6686, .8203829101562E+04, -.1288132080078E+04, .1020016174316E+04},
 {    6687, .8209624023438E+04, -.1288132080078E+04, .1017422119141E+04},
 {    6688, .8202531250000E+04, -.1305912109375E+04, .1017118347168E+04},
 {    6689, .8208327148438E+04, -.1305912109375E+04, .1014523742676E+04},
 {    6690, .8202531250000E+04, -.1288132080078E+04, .1017118347168E+04},
 {    6691, .8208327148438E+04, -.1288132080078E+04, .1014523742676E+04},
 {    6692, .8197340820312E+04, -.1305912109375E+04, .1005527099609E+04},
 {    6693, .8203137695312E+04, -.1305912109375E+04, .1002932495117E+04},
 {    6694, .8197340820312E+04, -.1288132080078E+04, .1005527099609E+04},
 {    6695, .8203137695312E+04, -.1288132080078E+04, .1002932495117E+04},
 {    6696, .8192153320312E+04, -.1305912109375E+04, .9939351806641E+03},
 {    6697, .8197948242188E+04, -.1305912109375E+04, .9913408813477E+03},
 {    6698, .8192153320312E+04, -.1288132080078E+04, .9939351806641E+03},
 {    6699, .8197948242188E+04, -.1288132080078E+04, .9913408813477E+03},
 {    6700, .8186963867188E+04, -.1305912109375E+04, .9823437500000E+03},
 {    6701, .8192758789062E+04, -.1305912109375E+04, .9797492065430E+03},
 {    6702, .8186963867188E+04, -.1288132080078E+04, .9823437500000E+03},
 {    6703, .8192758789062E+04, -.1288132080078E+04, .9797492065430E+03},
 {    6704, .8254424804688E+04, -.1283052001953E+04, .1133033447266E+04},
 {    6705, .8260219726562E+04, -.1283052001953E+04, .1130439453125E+04},
 {    6706, .8249233398438E+04, -.1283052001953E+04, .1121441894531E+04},
 {    6707, .8255029296875E+04, -.1283052001953E+04, .1118847412109E+04},
 {    6708, .8244044921875E+04, -.1283052001953E+04, .1109850585938E+04},
 {    6709, .8249837890625E+04, -.1283052001953E+04, .1107256591797E+04},
 {    6710, .8238854492188E+04, -.1283052001953E+04, .1098259033203E+04},
 {    6711, .8244649414062E+04, -.1283052001953E+04, .1095664794922E+04},
 {    6712, .8237558593750E+04, -.1283052001953E+04, .1095361206055E+04},
 {    6713, .8243356445312E+04, -.1283052001953E+04, .1092766601562E+04},
 {    6714, .8232369140625E+04, -.1283052001953E+04, .1083769287109E+04},
 {    6715, .8238166015625E+04, -.1283052001953E+04, .1081174926758E+04},
 {    6716, .8227179687500E+04, -.1283052001953E+04, .1072178100586E+04},
 {    6717, .8232976562500E+04, -.1283052001953E+04, .1069583129883E+04},
 {    6718, .8225882812500E+04, -.1283052001953E+04, .1069280273438E+04},
 {    6719, .8231680664062E+04, -.1283052001953E+04, .1066685424805E+04},
 {    6720, .8220692382812E+04, -.1283052001953E+04, .1057688842773E+04},
 {    6721, .8226489257812E+04, -.1283052001953E+04, .1055094482422E+04},
 {    6722, .8215502929688E+04, -.1283052001953E+04, .1046097167969E+04},
 {    6723, .8221301757812E+04, -.1283052001953E+04, .1043502807617E+04},
 {    6724, .8214208007812E+04, -.1283052001953E+04, .1043198730469E+04},
 {    6725, .8220001953125E+04, -.1283052001953E+04, .1040605102539E+04},
 {    6726, .8209017578125E+04, -.1283052001953E+04, .1031608154297E+04},
 {    6727, .8214813476562E+04, -.1283052001953E+04, .1029013183594E+04},
 {    6728, .8203829101562E+04, -.1283052001953E+04, .1020016174316E+04},
 {    6729, .8209624023438E+04, -.1283052001953E+04, .1017422119141E+04},
 {    6730, .8202531250000E+04, -.1283052001953E+04, .1017118347168E+04},
 {    6731, .8208327148438E+04, -.1283052001953E+04, .1014523742676E+04},
 {    6732, .8197340820312E+04, -.1283052001953E+04, .1005527099609E+04},
 {    6733, .8203137695312E+04, -.1283052001953E+04, .1002932495117E+04},
 {    6734, .8192154296875E+04, -.1283052001953E+04, .9939351196289E+03},
 {    6735, .8197948242188E+04, -.1283052001953E+04, .9913408813477E+03},
 {    6736, .8186963867188E+04, -.1283052001953E+04, .9823437500000E+03},
 {    6737, .8192758789062E+04, -.1283052001953E+04, .9797492065430E+03},
 {    6738, .8254424804688E+04, -.1265272094727E+04, .1133033447266E+04},
 {    6739, .8260219726562E+04, -.1265272094727E+04, .1130439453125E+04},
 {    6740, .8254424804688E+04, -.1247492065430E+04, .1133033447266E+04},
 {    6741, .8260219726562E+04, -.1247492065430E+04, .1130439453125E+04},
 {    6742, .8249233398438E+04, -.1265272094727E+04, .1121441894531E+04},
 {    6743, .8255029296875E+04, -.1265272094727E+04, .1118847412109E+04},
 {    6744, .8249233398438E+04, -.1247492065430E+04, .1121441894531E+04},
 {    6745, .8255029296875E+04, -.1247492065430E+04, .1118847412109E+04},
 {    6746, .8244044921875E+04, -.1265272094727E+04, .1109850585938E+04},
 {    6747, .8249837890625E+04, -.1265272094727E+04, .1107256591797E+04},
 {    6748, .8244044921875E+04, -.1247492065430E+04, .1109850585938E+04},
 {    6749, .8249837890625E+04, -.1247492065430E+04, .1107256591797E+04},
 {    6750, .8238854492188E+04, -.1265272094727E+04, .1098259033203E+04},
 {    6751, .8244649414062E+04, -.1265272094727E+04, .1095664794922E+04},
 {    6752, .8238854492188E+04, -.1247492065430E+04, .1098259033203E+04},
 {    6753, .8244649414062E+04, -.1247492065430E+04, .1095664794922E+04},
 {    6754, .8237558593750E+04, -.1265272094727E+04, .1095361206055E+04},
 {    6755, .8243356445312E+04, -.1265272094727E+04, .1092766601562E+04},
 {    6756, .8237558593750E+04, -.1247492065430E+04, .1095361206055E+04},
 {    6757, .8243356445312E+04, -.1247492065430E+04, .1092766601562E+04},
 {    6758, .8232369140625E+04, -.1265272094727E+04, .1083769287109E+04},
 {    6759, .8238166015625E+04, -.1265272094727E+04, .1081174926758E+04},
 {    6760, .8232369140625E+04, -.1247492065430E+04, .1083769287109E+04},
 {    6761, .8238166015625E+04, -.1247492065430E+04, .1081174926758E+04},
 {    6762, .8227179687500E+04, -.1265272094727E+04, .1072178100586E+04},
 {    6763, .8232976562500E+04, -.1265272094727E+04, .1069583129883E+04},
 {    6764, .8227179687500E+04, -.1247492065430E+04, .1072178100586E+04},
 {    6765, .8232976562500E+04, -.1247492065430E+04, .1069583129883E+04},
 {    6766, .8225882812500E+04, -.1265272094727E+04, .1069280273438E+04},
 {    6767, .8231680664062E+04, -.1265272094727E+04, .1066685424805E+04},
 {    6768, .8225882812500E+04, -.1247492065430E+04, .1069280273438E+04},
 {    6769, .8231680664062E+04, -.1247492065430E+04, .1066685424805E+04},
 {    6770, .8220692382812E+04, -.1265272094727E+04, .1057688842773E+04},
 {    6771, .8226489257812E+04, -.1265272094727E+04, .1055094482422E+04},
 {    6772, .8220692382812E+04, -.1247492065430E+04, .1057688842773E+04},
 {    6773, .8226489257812E+04, -.1247492065430E+04, .1055094482422E+04},
 {    6774, .8215502929688E+04, -.1265272094727E+04, .1046097167969E+04},
 {    6775, .8221301757812E+04, -.1265272094727E+04, .1043502807617E+04},
 {    6776, .8215502929688E+04, -.1247492065430E+04, .1046097167969E+04},
 {    6777, .8221301757812E+04, -.1247492065430E+04, .1043502807617E+04},
 {    6778, .8214208007812E+04, -.1265272094727E+04, .1043198730469E+04},
 {    6779, .8220001953125E+04, -.1265272094727E+04, .1040605102539E+04},
 {    6780, .8214208007812E+04, -.1247492065430E+04, .1043198730469E+04},
 {    6781, .8220001953125E+04, -.1247492065430E+04, .1040605102539E+04},
 {    6782, .8209017578125E+04, -.1265272094727E+04, .1031608154297E+04},
 {    6783, .8214813476562E+04, -.1265272094727E+04, .1029013183594E+04},
 {    6784, .8209017578125E+04, -.1247492065430E+04, .1031608154297E+04},
 {    6785, .8214813476562E+04, -.1247492065430E+04, .1029013183594E+04},
 {    6786, .8203829101562E+04, -.1265272094727E+04, .1020016174316E+04},
 {    6787, .8209624023438E+04, -.1265272094727E+04, .1017422119141E+04},
 {    6788, .8203829101562E+04, -.1247492065430E+04, .1020016174316E+04},
 {    6789, .8209624023438E+04, -.1247492065430E+04, .1017422119141E+04},
 {    6790, .8202531250000E+04, -.1265272094727E+04, .1017118347168E+04},
 {    6791, .8208327148438E+04, -.1265272094727E+04, .1014523742676E+04},
 {    6792, .8202531250000E+04, -.1247492065430E+04, .1017118347168E+04},
 {    6793, .8208327148438E+04, -.1247492065430E+04, .1014523742676E+04},
 {    6794, .8197340820312E+04, -.1265272094727E+04, .1005527099609E+04},
 {    6795, .8203137695312E+04, -.1265272094727E+04, .1002932495117E+04},
 {    6796, .8197340820312E+04, -.1247492065430E+04, .1005527099609E+04},
 {    6797, .8203137695312E+04, -.1247492065430E+04, .1002932495117E+04},
 {    6798, .8192154296875E+04, -.1265272094727E+04, .9939351196289E+03},
 {    6799, .8197948242188E+04, -.1265272094727E+04, .9913408813477E+03},
 {    6800, .8192154296875E+04, -.1247492065430E+04, .9939351196289E+03},
 {    6801, .8197948242188E+04, -.1247492065430E+04, .9913408813477E+03},
 {    6802, .8186963867188E+04, -.1265272094727E+04, .9823437500000E+03},
 {    6803, .8192758789062E+04, -.1265272094727E+04, .9797492065430E+03},
 {    6804, .8186963867188E+04, -.1247492065430E+04, .9823437500000E+03},
 {    6805, .8192758789062E+04, -.1247492065430E+04, .9797492065430E+03},
 {    6806, .8254424804688E+04, -.1234791992188E+04, .1133033447266E+04},
 {    6807, .8260219726562E+04, -.1234791992188E+04, .1130439453125E+04},
 {    6808, .8249233398438E+04, -.1234791992188E+04, .1121441894531E+04},
 {    6809, .8255029296875E+04, -.1234791992188E+04, .1118847412109E+04},
 {    6810, .8244044921875E+04, -.1234791992188E+04, .1109850585938E+04},
 {    6811, .8249837890625E+04, -.1234791992188E+04, .1107256591797E+04},
 {    6812, .8238854492188E+04, -.1234791992188E+04, .1098259033203E+04},
 {    6813, .8244649414062E+04, -.1234791992188E+04, .1095664794922E+04},
 {    6814, .8237558593750E+04, -.1234791992188E+04, .1095361206055E+04},
 {    6815, .8243356445312E+04, -.1234791992188E+04, .1092766601562E+04},
 {    6816, .8232369140625E+04, -.1234791992188E+04, .1083769287109E+04},
 {    6817, .8238166015625E+04, -.1234791992188E+04, .1081174926758E+04},
 {    6818, .8227179687500E+04, -.1234791992188E+04, .1072178100586E+04},
 {    6819, .8232976562500E+04, -.1234791992188E+04, .1069583129883E+04},
 {    6820, .8225882812500E+04, -.1234791992188E+04, .1069280273438E+04},
 {    6821, .8231680664062E+04, -.1234791992188E+04, .1066685424805E+04},
 {    6822, .8220692382812E+04, -.1234791992188E+04, .1057688842773E+04},
 {    6823, .8226489257812E+04, -.1234791992188E+04, .1055094482422E+04},
 {    6824, .8215502929688E+04, -.1234791992188E+04, .1046097167969E+04},
 {    6825, .8221301757812E+04, -.1234791992188E+04, .1043502807617E+04},
 {    6826, .8214208007812E+04, -.1234791992188E+04, .1043198730469E+04},
 {    6827, .8220001953125E+04, -.1234791992188E+04, .1040605102539E+04},
 {    6828, .8209017578125E+04, -.1234791992188E+04, .1031608154297E+04},
 {    6829, .8214813476562E+04, -.1234791992188E+04, .1029013183594E+04},
 {    6830, .8203829101562E+04, -.1234791992188E+04, .1020016174316E+04},
 {    6831, .8209624023438E+04, -.1234791992188E+04, .1017422119141E+04},
 {    6832, .8202531250000E+04, -.1234791992188E+04, .1017118347168E+04},
 {    6833, .8208327148438E+04, -.1234791992188E+04, .1014523742676E+04},
 {    6834, .8197340820312E+04, -.1234791992188E+04, .1005527099609E+04},
 {    6835, .8203137695312E+04, -.1234791992188E+04, .1002932495117E+04},
 {    6836, .8192154296875E+04, -.1234791992188E+04, .9939351196289E+03},
 {    6837, .8197948242188E+04, -.1234791992188E+04, .9913408813477E+03},
 {    6838, .8186963867188E+04, -.1234791992188E+04, .9823437500000E+03},
 {    6839, .8192758789062E+04, -.1234791992188E+04, .9797492065430E+03},
 {    6840, .8426041015625E+04, -.1341724365234E+04, .1371152099609E+04},
 {    6841, .8422049804688E+04, -.1335270507812E+04, .1371400634766E+04},
 {    6842, .8443809570312E+04, -.1327334960938E+04, .1370047119141E+04},
 {    6843, .8437889648438E+04, -.1322570922852E+04, .1370415283203E+04},
 {    6844, .8456097656250E+04, -.1308037963867E+04, .1369283203125E+04},
 {    6845, .8448925781250E+04, -.1305512084961E+04, .1369728515625E+04},
 {    6846, .8460579101562E+04, -.1285592041016E+04, .1369003662109E+04},
 {    6847, .8452972656250E+04, -.1285592041016E+04, .1369477539062E+04},
 {    6848, .8426290039062E+04, -.1340045410156E+04, .1386405517578E+04},
 {    6849, .8422206054688E+04, -.1333810668945E+04, .1384918212891E+04},
 {    6850, .8442291015625E+04, -.1325939086914E+04, .1389075317383E+04},
 {    6851, .8436479492188E+04, -.1321361083984E+04, .1387301025391E+04},
 {    6852, .8453527343750E+04, -.1307054443359E+04, .1390946777344E+04},
 {    6853, .8446577148438E+04, -.1304648437500E+04, .1388968017578E+04},
 {    6854, .8457492187500E+04, -.1285592041016E+04, .1391684814453E+04},
 {    6855, .8450163085938E+04, -.1285592041016E+04, .1389610351562E+04},
 {    6856, .8424702148438E+04, -.1335365112305E+04, .1400936523438E+04},
 {    6857, .8420763671875E+04, -.1329675048828E+04, .1397789794922E+04},
 {    6858, .8435911132812E+04, -.1321664306641E+04, .1406681152344E+04},
 {    6859, .8430772460938E+04, -.1317551025391E+04, .1402878784180E+04},
 {    6860, .8443622070312E+04, -.1304489501953E+04, .1410483642578E+04},
 {    6861, .8437663085938E+04, -.1302357177734E+04, .1406267822266E+04},
 {    6862, .8446400390625E+04, -.1285592041016E+04, .1411709716797E+04},
 {    6863, .8440187500000E+04, -.1285592041016E+04, .1407322753906E+04},
 {    6864, .8421275390625E+04, -.1328045166016E+04, .1413984619141E+04},
 {    6865, .8417715820312E+04, -.1323193847656E+04, .1409345947266E+04},
 {    6866, .8425952148438E+04, -.1315235107422E+04, .1421027954102E+04},
 {    6867, .8421871093750E+04, -.1311849121094E+04, .1415581054688E+04},
 {    6868, .8428724609375E+04, -.1300831542969E+04, .1425544921875E+04},
 {    6869, .8424361328125E+04, -.1299092285156E+04, .1419569580078E+04},
 {    6870, .8429528320312E+04, -.1285592041016E+04, .1427176757812E+04},
 {    6871, .8425131835938E+04, -.1285592041016E+04, .1420984375000E+04},
 {    6872, .8456097656250E+04, -.1263146118164E+04, .1369283203125E+04},
 {    6873, .8448925781250E+04, -.1265671875000E+04, .1369728515625E+04},
 {    6874, .8443809570312E+04, -.1243849121094E+04, .1370047119141E+04},
 {    6875, .8437889648438E+04, -.1248613159180E+04, .1370415283203E+04},
 {    6876, .8426041015625E+04, -.1229459716797E+04, .1371152099609E+04},
 {    6877, .8422049804688E+04, -.1235913452148E+04, .1371400634766E+04},
 {    6878, .8453527343750E+04, -.1264129638672E+04, .1390946777344E+04},
 {    6879, .8446577148438E+04, -.1266535522461E+04, .1388968017578E+04},
 {    6880, .8442291015625E+04, -.1245244995117E+04, .1389075317383E+04},
 {    6881, .8436479492188E+04, -.1249822998047E+04, .1387301025391E+04},
 {    6882, .8426290039062E+04, -.1231138549805E+04, .1386405517578E+04},
 {    6883, .8422206054688E+04, -.1237373413086E+04, .1384918212891E+04},
 {    6884, .8443622070312E+04, -.1266694458008E+04, .1410483642578E+04},
 {    6885, .8437663085938E+04, -.1268826904297E+04, .1406267822266E+04},
 {    6886, .8435911132812E+04, -.1249519775391E+04, .1406681152344E+04},
 {    6887, .8430772460938E+04, -.1253633056641E+04, .1402878784180E+04},
 {    6888, .8424702148438E+04, -.1235818969727E+04, .1400936523438E+04},
 {    6889, .8420763671875E+04, -.1241509033203E+04, .1397789794922E+04},
 {    6890, .8428724609375E+04, -.1270352416992E+04, .1425544921875E+04},
 {    6891, .8424361328125E+04, -.1272091674805E+04, .1419569580078E+04},
 {    6892, .8425950195312E+04, -.1255948974609E+04, .1421027832031E+04},
 {    6893, .8421871093750E+04, -.1259334838867E+04, .1415581054688E+04},
 {    6894, .8421275390625E+04, -.1243138916016E+04, .1413984619141E+04},
 {    6895, .8417715820312E+04, -.1247990112305E+04, .1409345947266E+04},
 {    6896, .8411498046875E+04, -.1348585083008E+04, .1372057617188E+04},
 {    6897, .8395965820312E+04, -.1352731811523E+04, .1373024291992E+04},
 {    6898, .8379957031250E+04, -.1354165283203E+04, .1374020263672E+04},
 {    6899, .8408718750000E+04, -.1341506835938E+04, .1372230590820E+04},
 {    6900, .8394496093750E+04, -.1345259765625E+04, .1373115600586E+04},
 {    6901, .8379839843750E+04, -.1346546997070E+04, .1374027832031E+04},
 {    6902, .8412057617188E+04, -.1346533325195E+04, .1388656372070E+04},
 {    6903, .8396776367188E+04, -.1350473754883E+04, .1390447021484E+04},
 {    6904, .8381058593750E+04, -.1351828613281E+04, .1391734619141E+04},
 {    6905, .8409172851562E+04, -.1339697753906E+04, .1386969970703E+04},
 {    6906, .8395190429688E+04, -.1343256347656E+04, .1388601318359E+04},
 {    6907, .8380813476562E+04, -.1344469970703E+04, .1389774169922E+04},
 {    6908, .8411224609375E+04, -.1340635742188E+04, .1404374511719E+04},
 {    6909, .8396828125000E+04, -.1343846313477E+04, .1406822387695E+04},
 {    6910, .8382024414062E+04, -.1344978759766E+04, .1408245239258E+04},
 {    6911, .8408409179688E+04, -.1334465332031E+04, .1400926513672E+04},
 {    6912, .8395221679688E+04, -.1337365966797E+04, .1403155273438E+04},
 {    6913, .8381664062500E+04, -.1338381103516E+04, .1404450683594E+04},
 {    6914, .8408982421875E+04, -.1331379760742E+04, .1418090087891E+04},
 {    6915, .8396054687500E+04, -.1333391845703E+04, .1420911010742E+04},
 {    6916, .8382775390625E+04, -.1334082397461E+04, .1422429199219E+04},
 {    6917, .8406413085938E+04, -.1326241699219E+04, .1413101806641E+04},
 {    6918, .8394530273438E+04, -.1328073120117E+04, .1415676757812E+04},
 {    6919, .8382329101562E+04, -.1328695434570E+04, .1417059082031E+04},
 {    6920, .8409665039062E+04, -.1314213745117E+04, .1421139404297E+04},
 {    6921, .8396570312500E+04, -.1315619140625E+04, .1424860595703E+04},
 {    6922, .8383052734375E+04, -.1316070434570E+04, .1426718994141E+04},
 {    6923, .8411617187500E+04, -.1300331542969E+04, .1426260742188E+04},
 {    6924, .8397806640625E+04, -.1301091308594E+04, .1430684692383E+04},
 {    6925, .8383490234375E+04, -.1301368896484E+04, .1432792724609E+04},
 {    6926, .8412191406250E+04, -.1285592041016E+04, .1428037475586E+04},
 {    6927, .8398200195312E+04, -.1285592041016E+04, .1432668212891E+04},
 {    6928, .8383625000000E+04, -.1285592041016E+04, .1434864379883E+04},
 {    6929, .8412638671875E+04, -.1317828735352E+04, .1427137451172E+04},
 {    6930, .8398351562500E+04, -.1319378784180E+04, .1431242675781E+04},
 {    6931, .8383582031250E+04, -.1319879516602E+04, .1433295898438E+04},
 {    6932, .8414810546875E+04, -.1302193237305E+04, .1432910644531E+04},
 {    6933, .8399722656250E+04, -.1303031860352E+04, .1437797607422E+04},
 {    6934, .8384069335938E+04, -.1303340698242E+04, .1440129272461E+04},
 {    6935, .8415410156250E+04, -.1285592041016E+04, .1434929443359E+04},
 {    6936, .8400135742188E+04, -.1285592041016E+04, .1440033935547E+04},
 {    6937, .8384215820312E+04, -.1285592041016E+04, .1442461303711E+04},
 {    6938, .8411617187500E+04, -.1270852661133E+04, .1426260742188E+04},
 {    6939, .8397806640625E+04, -.1270092895508E+04, .1430684692383E+04},
 {    6940, .8383490234375E+04, -.1269815307617E+04, .1432792724609E+04},
 {    6941, .8409665039062E+04, -.1256970336914E+04, .1421139404297E+04},
 {    6942, .8396570312500E+04, -.1255565063477E+04, .1424860839844E+04},
 {    6943, .8383052734375E+04, -.1255113769531E+04, .1426718994141E+04},
 {    6944, .8406413085938E+04, -.1244942260742E+04, .1413101806641E+04},
 {    6945, .8394530273438E+04, -.1243110961914E+04, .1415676757812E+04},
 {    6946, .8382329101562E+04, -.1242488647461E+04, .1417059082031E+04},
 {    6947, .8414810546875E+04, -.1268990844727E+04, .1432910644531E+04},
 {    6948, .8399722656250E+04, -.1268152343750E+04, .1437797607422E+04},
 {    6949, .8384069335938E+04, -.1267843627930E+04, .1440129272461E+04},
 {    6950, .8412638671875E+04, -.1253355346680E+04, .1427137451172E+04},
 {    6951, .8398350585938E+04, -.1251805419922E+04, .1431242797852E+04},
 {    6952, .8383582031250E+04, -.1251304687500E+04, .1433296142578E+04},
 {    6953, .8408982421875E+04, -.1239804321289E+04, .1418090087891E+04},
 {    6954, .8396054687500E+04, -.1237792114258E+04, .1420911010742E+04},
 {    6955, .8382775390625E+04, -.1237101562500E+04, .1422429199219E+04},
 {    6956, .8408718750000E+04, -.1229677246094E+04, .1372230590820E+04},
 {    6957, .8394496093750E+04, -.1225924316406E+04, .1373115600586E+04},
 {    6958, .8379839843750E+04, -.1224637084961E+04, .1374027832031E+04},
 {    6959, .8411498046875E+04, -.1222598999023E+04, .1372057617188E+04},
 {    6960, .8395965820312E+04, -.1218452148438E+04, .1373024291992E+04},
 {    6961, .8379957031250E+04, -.1217018798828E+04, .1374020263672E+04},
 {    6962, .8409172851562E+04, -.1231486206055E+04, .1386969970703E+04},
 {    6963, .8395190429688E+04, -.1227927734375E+04, .1388601318359E+04},
 {    6964, .8380813476562E+04, -.1226714111328E+04, .1389774169922E+04},
 {    6965, .8412057617188E+04, -.1224650634766E+04, .1388656372070E+04},
 {    6966, .8396776367188E+04, -.1220710327148E+04, .1390447021484E+04},
 {    6967, .8381058593750E+04, -.1219355468750E+04, .1391734619141E+04},
 {    6968, .8408409179688E+04, -.1236718750000E+04, .1400926513672E+04},
 {    6969, .8395221679688E+04, -.1233817993164E+04, .1403155273438E+04},
 {    6970, .8381664062500E+04, -.1232802856445E+04, .1404450683594E+04},
 {    6971, .8411224609375E+04, -.1230548217773E+04, .1404374511719E+04},
 {    6972, .8396828125000E+04, -.1227337768555E+04, .1406822387695E+04},
 {    6973, .8382024414062E+04, -.1226205322266E+04, .1408245239258E+04},
 {    6974, .8357625000000E+04, -.1352058837891E+04, .1375409912109E+04},
 {    6975, .8336184570312E+04, -.1345451416016E+04, .1376743286133E+04},
 {    6976, .8316801757812E+04, -.1334136718750E+04, .1377949707031E+04},
 {    6977, .8359244140625E+04, -.1344623291016E+04, .1375309082031E+04},
 {    6978, .8339450195312E+04, -.1338615722656E+04, .1376540527344E+04},
 {    6979, .8321532226562E+04, -.1328261474609E+04, .1377655517578E+04},
 {    6980, .8359000000000E+04, -.1349842407227E+04, .1392623291016E+04},
 {    6981, .8337671875000E+04, -.1343536254883E+04, .1392540771484E+04},
 {    6982, .8318395507812E+04, -.1332824462891E+04, .1391391601562E+04},
 {    6983, .8360494140625E+04, -.1342658691406E+04, .1390594726562E+04},
 {    6984, .8340827148438E+04, -.1336937866211E+04, .1390535766602E+04},
 {    6985, .8323023437500E+04, -.1327154785156E+04, .1389492187500E+04},
 {    6986, .8361015625000E+04, -.1343287597656E+04, .1408754150391E+04},
 {    6987, .8340555664062E+04, -.1338070556641E+04, .1407397094727E+04},
 {    6988, .8321794921875E+04, -.1329257934570E+04, .1404066650391E+04},
 {    6989, .8362271484375E+04, -.1336833251953E+04, .1404928833008E+04},
 {    6990, .8343364257812E+04, -.1332077026367E+04, .1403706176758E+04},
 {    6991, .8326014648438E+04, -.1323990112305E+04, .1400654296875E+04},
 {    6992, .8363442382812E+04, -.1332961669922E+04, .1422509033203E+04},
 {    6993, .8344529296875E+04, -.1329494262695E+04, .1420211669922E+04},
 {    6994, .8326718750000E+04, -.1323597045898E+04, .1415410644531E+04},
 {    6995, .8364445312500E+04, -.1327662353516E+04, .1417136962891E+04},
 {    6996, .8346936523438E+04, -.1324490356445E+04, .1415048828125E+04},
 {    6997, .8330444335938E+04, -.1319065917969E+04, .1410639404297E+04},
 {    6998, .8363604492188E+04, -.1315343139648E+04, .1426377929688E+04},
 {    6999, .8344552734375E+04, -.1312948608398E+04, .1422797973633E+04},
 {    7000, .8326957031250E+04, -.1308825317383E+04, .1415790039062E+04},
 {    7001, .8363084960938E+04, -.1300926391602E+04, .1432246459961E+04},
 {    7002, .8343142578125E+04, -.1299645751953E+04, .1427755615234E+04},
 {    7003, .8324864257812E+04, -.1297498779297E+04, .1419091430664E+04},
 {    7004, .8362911132812E+04, -.1285592041016E+04, .1434225830078E+04},
 {    7005, .8342742187500E+04, -.1285592041016E+04, .1429461914062E+04},
 {    7006, .8324180664062E+04, -.1285592041016E+04, .1420242187500E+04},
 {    7007, .8362518554688E+04, -.1319082885742E+04, .1432912597656E+04},
 {    7008, .8341896484375E+04, -.1316457275391E+04, .1428967773438E+04},
 {    7009, .8322871093750E+04, -.1311969360352E+04, .1421322509766E+04},
 {    7010, .8361916992188E+04, -.1302858154297E+04, .1439516723633E+04},
 {    7011, .8340287109375E+04, -.1301451538086E+04, .1434544067383E+04},
 {    7012, .8320489257812E+04, -.1299107177734E+04, .1425043579102E+04},
 {    7013, .8361750976562E+04, -.1285592041016E+04, .1441748046875E+04},
 {    7014, .8339899414062E+04, -.1285592041016E+04, .1436487792969E+04},
 {    7015, .8319815429688E+04, -.1285592041016E+04, .1426400390625E+04},
 {    7016, .8363084960938E+04, -.1270257812500E+04, .1432246948242E+04},
 {    7017, .8343142578125E+04, -.1271538330078E+04, .1427755859375E+04},
 {    7018, .8324864257812E+04, -.1273685302734E+04, .1419091430664E+04},
 {    7019, .8363605468750E+04, -.1255841064453E+04, .1426378173828E+04},
 {    7020, .8344552734375E+04, -.1258235473633E+04, .1422797973633E+04},
 {    7021, .8326957031250E+04, -.1262358642578E+04, .1415790039062E+04},
 {    7022, .8364445312500E+04, -.1243521728516E+04, .1417136962891E+04},
 {    7023, .8346936523438E+04, -.1246693603516E+04, .1415048828125E+04},
 {    7024, .8330444335938E+04, -.1252118041992E+04, .1410639404297E+04},
 {    7025, .8361916992188E+04, -.1268326049805E+04, .1439516723633E+04},
 {    7026, .8340287109375E+04, -.1269732543945E+04, .1434544067383E+04},
 {    7027, .8320489257812E+04, -.1272076782227E+04, .1425043579102E+04},
 {    7028, .8362518554688E+04, -.1252101318359E+04, .1432912597656E+04},
 {    7029, .8341896484375E+04, -.1254726806641E+04, .1428968017578E+04},
 {    7030, .8322871093750E+04, -.1259214721680E+04, .1421322509766E+04},
 {    7031, .8363442382812E+04, -.1238222412109E+04, .1422509033203E+04},
 {    7032, .8344529296875E+04, -.1241689819336E+04, .1420211669922E+04},
 {    7033, .8326718750000E+04, -.1247587036133E+04, .1415410644531E+04},
 {    7034, .8359244140625E+04, -.1226560668945E+04, .1375309082031E+04},
 {    7035, .8339450195312E+04, -.1232568237305E+04, .1376540527344E+04},
 {    7036, .8321532226562E+04, -.1242922485352E+04, .1377655517578E+04},
 {    7037, .8357625000000E+04, -.1219125244141E+04, .1375409912109E+04},
 {    7038, .8336184570312E+04, -.1225732543945E+04, .1376743286133E+04},
 {    7039, .8316801757812E+04, -.1237047363281E+04, .1377949707031E+04},
 {    7040, .8360494140625E+04, -.1228525268555E+04, .1390594726562E+04},
 {    7041, .8340827148438E+04, -.1234246093750E+04, .1390535766602E+04},
 {    7042, .8323023437500E+04, -.1244029296875E+04, .1389492187500E+04},
 {    7043, .8359000000000E+04, -.1221341674805E+04, .1392623291016E+04},
 {    7044, .8337671875000E+04, -.1227647827148E+04, .1392540771484E+04},
 {    7045, .8318395507812E+04, -.1238359619141E+04, .1391391601562E+04},
 {    7046, .8362271484375E+04, -.1234350830078E+04, .1404928833008E+04},
 {    7047, .8343364257812E+04, -.1239106933594E+04, .1403706176758E+04},
 {    7048, .8326014648438E+04, -.1247193969727E+04, .1400654296875E+04},
 {    7049, .8361015625000E+04, -.1227896362305E+04, .1408754150391E+04},
 {    7050, .8340555664062E+04, -.1233113403320E+04, .1407397094727E+04},
 {    7051, .8321794921875E+04, -.1241926025391E+04, .1404066650391E+04},
 {    7052, .8309599609375E+04, -.1316570556641E+04, .1378397949219E+04},
 {    7053, .8303335937500E+04, -.1320806884766E+04, .1378787109375E+04},
 {    7054, .8301347656250E+04, -.1302037353516E+04, .1378911010742E+04},
 {    7055, .8294083007812E+04, -.1304258789062E+04, .1379362548828E+04},
 {    7056, .8298333984375E+04, -.1285592041016E+04, .1379098144531E+04},
 {    7057, .8290727539062E+04, -.1285592041016E+04, .1379572021484E+04},
 {    7058, .8312389648438E+04, -.1315682739258E+04, .1392474121094E+04},
 {    7059, .8306428710938E+04, -.1319756225586E+04, .1394776855469E+04},
 {    7060, .8304990234375E+04, -.1301418334961E+04, .1394541503906E+04},
 {    7061, .8298144531250E+04, -.1303533081055E+04, .1397085937500E+04},
 {    7062, .8302369140625E+04, -.1285592041016E+04, .1395324707031E+04},
 {    7063, .8295243164062E+04, -.1285592041016E+04, .1397993896484E+04},
 {    7064, .8318474609375E+04, -.1312948242188E+04, .1405254638672E+04},
 {    7065, .8313308593750E+04, -.1316689697266E+04, .1409324462891E+04},
 {    7066, .8313375000000E+04, -.1299814819336E+04, .1408311035156E+04},
 {    7067, .8307546875000E+04, -.1301742553711E+04, .1412756591797E+04},
 {    7068, .8311523437500E+04, -.1285592041016E+04, .1409316406250E+04},
 {    7069, .8305521484375E+04, -.1285592041016E+04, .1413933837891E+04},
 {    7070, .8301347656250E+04, -.1269146728516E+04, .1378911010742E+04},
 {    7071, .8294083007812E+04, -.1266925292969E+04, .1379362548828E+04},
 {    7072, .8309599609375E+04, -.1254613403320E+04, .1378397949219E+04},
 {    7073, .8303335937500E+04, -.1250377197266E+04, .1378787109375E+04},
 {    7074, .8304990234375E+04, -.1269765747070E+04, .1394541503906E+04},
 {    7075, .8298144531250E+04, -.1267651000977E+04, .1397085937500E+04},
 {    7076, .8312389648438E+04, -.1255501220703E+04, .1392474121094E+04},
 {    7077, .8306428710938E+04, -.1251427856445E+04, .1394776855469E+04},
 {    7078, .8313375000000E+04, -.1271369262695E+04, .1408311035156E+04},
 {    7079, .8307546875000E+04, -.1269441406250E+04, .1412756591797E+04},
 {    7080, .8318474609375E+04, -.1258235839844E+04, .1405254638672E+04},
 {    7081, .8313308593750E+04, -.1254494384766E+04, .1409324462891E+04},
 {    7082, .8430102539062E+04, -.1348144653320E+04, .1370900146484E+04},
 {    7083, .8449763671875E+04, -.1332063232422E+04, .1369677001953E+04},
 {    7084, .8463281250000E+04, -.1310541137695E+04, .1368836181641E+04},
 {    7085, .8468185546875E+04, -.1285592041016E+04, .1368531005859E+04},
 {    7086, .8430441406250E+04, -.1346246337891E+04, .1387881591797E+04},
 {    7087, .8448137695312E+04, -.1330483642578E+04, .1390833007812E+04},
 {    7088, .8460495117188E+04, -.1309439819336E+04, .1392903076172E+04},
 {    7089, .8464830078125E+04, -.1285592041016E+04, .1393736572266E+04},
 {    7090, .8428708984375E+04, -.1341025512695E+04, .1404062988281E+04},
 {    7091, .8441127929688E+04, -.1325735961914E+04, .1410435424805E+04},
 {    7092, .8449621093750E+04, -.1306606323242E+04, .1414664062500E+04},
 {    7093, .8452648437500E+04, -.1285592041016E+04, .1416058837891E+04},
 {    7094, .8424914062500E+04, -.1332875000000E+04, .1418597167969E+04},
 {    7095, .8430101562500E+04, -.1318604125977E+04, .1426440673828E+04},
 {    7096, .8433154296875E+04, -.1302561645508E+04, .1431481933594E+04},
 {    7097, .8433989257812E+04, -.1285592041016E+04, .1433331909180E+04},
 {    7098, .8463281250000E+04, -.1260642944336E+04, .1368836181641E+04},
 {    7099, .8449763671875E+04, -.1239120727539E+04, .1369677001953E+04},
 {    7100, .8430102539062E+04, -.1223039306641E+04, .1370900146484E+04},
 {    7101, .8460495117188E+04, -.1261744140625E+04, .1392903076172E+04},
 {    7102, .8448137695312E+04, -.1240700439453E+04, .1390833007812E+04},
 {    7103, .8430441406250E+04, -.1224937744141E+04, .1387881591797E+04},
 {    7104, .8449621093750E+04, -.1264577758789E+04, .1414664062500E+04},
 {    7105, .8441127929688E+04, -.1245448120117E+04, .1410435424805E+04},
 {    7106, .8428708984375E+04, -.1230158569336E+04, .1404062988281E+04},
 {    7107, .8433154296875E+04, -.1268622436523E+04, .1431481933594E+04},
 {    7108, .8430101562500E+04, -.1252579833984E+04, .1426440673828E+04},
 {    7109, .8424914062500E+04, -.1238308959961E+04, .1418597167969E+04},
 {    7110, .8414327148438E+04, -.1355647949219E+04, .1371881835938E+04},
 {    7111, .8397466796875E+04, -.1360199829102E+04, .1372930908203E+04},
 {    7112, .8380076171875E+04, -.1361783447266E+04, .1374012939453E+04},
 {    7113, .8414995117188E+04, -.1353353393555E+04, .1390334350586E+04},
 {    7114, .8398394531250E+04, -.1357687011719E+04, .1392289428711E+04},
 {    7115, .8381307617188E+04, -.1359187133789E+04, .1393693969727E+04},
 {    7116, .8414091796875E+04, -.1346793090820E+04, .1407809570312E+04},
 {    7117, .8398463867188E+04, -.1350322753906E+04, .1410485351562E+04},
 {    7118, .8382387695312E+04, -.1351576171875E+04, .1412039062500E+04},
 {    7119, .8411607421875E+04, -.1336508056641E+04, .1423064941406E+04},
 {    7120, .8397609375000E+04, -.1338708129883E+04, .1426140380859E+04},
 {    7121, .8383223632812E+04, -.1339469482422E+04, .1427798706055E+04},
 {    7122, .8415666015625E+04, -.1321436157227E+04, .1433118041992E+04},
 {    7123, .8400151367188E+04, -.1323136108398E+04, .1437619628906E+04},
 {    7124, .8384118164062E+04, -.1323688598633E+04, .1439872558594E+04},
 {    7125, .8418056640625E+04, -.1304050537109E+04, .1439542602539E+04},
 {    7126, .8401666015625E+04, -.1304971069336E+04, .1444904052734E+04},
 {    7127, .8384654296875E+04, -.1305312377930E+04, .1447465332031E+04},
 {    7128, .8418677734375E+04, -.1285592041016E+04, .1441802612305E+04},
 {    7129, .8402099609375E+04, -.1285592041016E+04, .1447394531250E+04},
 {    7130, .8384806640625E+04, -.1285592041016E+04, .1450056762695E+04},
 {    7131, .8418056640625E+04, -.1267133544922E+04, .1439542602539E+04},
 {    7132, .8401666015625E+04, -.1266213134766E+04, .1444904052734E+04},
 {    7133, .8384654296875E+04, -.1265871826172E+04, .1447465576172E+04},
 {    7134, .8415666015625E+04, -.1249747924805E+04, .1433118041992E+04},
 {    7135, .8400151367188E+04, -.1248048095703E+04, .1437619628906E+04},
 {    7136, .8384118164062E+04, -.1247495605469E+04, .1439872558594E+04},
 {    7137, .8411607421875E+04, -.1234676025391E+04, .1423064941406E+04},
 {    7138, .8397609375000E+04, -.1232475952148E+04, .1426140380859E+04},
 {    7139, .8383223632812E+04, -.1231714599609E+04, .1427798706055E+04},
 {    7140, .8414327148438E+04, -.1215536132812E+04, .1371881835938E+04},
 {    7141, .8397466796875E+04, -.1210984252930E+04, .1372930908203E+04},
 {    7142, .8380076171875E+04, -.1209400634766E+04, .1374012939453E+04},
 {    7143, .8414995117188E+04, -.1217830566406E+04, .1390334350586E+04},
 {    7144, .8398394531250E+04, -.1213496948242E+04, .1392289428711E+04},
 {    7145, .8381307617188E+04, -.1211996948242E+04, .1393693969727E+04},
 {    7146, .8414091796875E+04, -.1224390869141E+04, .1407809570312E+04},
 {    7147, .8398463867188E+04, -.1220861328125E+04, .1410485351562E+04},
 {    7148, .8382387695312E+04, -.1219607788086E+04, .1412039062500E+04},
 {    7149, .8355956054688E+04, -.1359487182617E+04, .1375513671875E+04},
 {    7150, .8332826171875E+04, -.1352257202148E+04, .1376952148438E+04},
 {    7151, .8311947265625E+04, -.1339940185547E+04, .1378251953125E+04},
 {    7152, .8357562500000E+04, -.1357033325195E+04, .1394651855469E+04},
 {    7153, .8334590820312E+04, -.1350170410156E+04, .1394563964844E+04},
 {    7154, .8313870117188E+04, -.1338562500000E+04, .1393336914062E+04},
 {    7155, .8359802734375E+04, -.1349743164062E+04, .1412593505859E+04},
 {    7156, .8337814453125E+04, -.1344075927734E+04, .1411134277344E+04},
 {    7157, .8317679687500E+04, -.1334540039062E+04, .1407564086914E+04},
 {    7158, .8362559570312E+04, -.1338277832031E+04, .1427890869141E+04},
 {    7159, .8342341796875E+04, -.1334561645508E+04, .1425425048828E+04},
 {    7160, .8323300781250E+04, -.1328263183594E+04, .1420298339844E+04},
 {    7161, .8361471679688E+04, -.1322839965820E+04, .1439445556641E+04},
 {    7162, .8339308593750E+04, -.1320017822266E+04, .1435146240234E+04},
 {    7163, .8318878906250E+04, -.1315204589844E+04, .1426863159180E+04},
 {    7164, .8360806640625E+04, -.1304793090820E+04, .1446791137695E+04},
 {    7165, .8337501953125E+04, -.1303279296875E+04, .1441364257812E+04},
 {    7166, .8316216796875E+04, -.1300767089844E+04, .1431058349609E+04},
 {    7167, .8360544921875E+04, -.1285592041016E+04, .1449265869141E+04},
 {    7168, .8336965820312E+04, -.1285592041016E+04, .1443489379883E+04},
 {    7169, .8315322265625E+04, -.1285592041016E+04, .1432494873047E+04},
 {    7170, .8360806640625E+04, -.1266391113281E+04, .1446791137695E+04},
 {    7171, .8337501953125E+04, -.1267904785156E+04, .1441364257812E+04},
 {    7172, .8316216796875E+04, -.1270416992188E+04, .1431058349609E+04},
 {    7173, .8361471679688E+04, -.1248344238281E+04, .1439445800781E+04},
 {    7174, .8339308593750E+04, -.1251166259766E+04, .1435146484375E+04},
 {    7175, .8318878906250E+04, -.1255979492188E+04, .1426863159180E+04},
 {    7176, .8362559570312E+04, -.1232906127930E+04, .1427890869141E+04},
 {    7177, .8342341796875E+04, -.1236622436523E+04, .1425425048828E+04},
 {    7178, .8323300781250E+04, -.1242920898438E+04, .1420298339844E+04},
 {    7179, .8355956054688E+04, -.1211696899414E+04, .1375513671875E+04},
 {    7180, .8332826171875E+04, -.1218926757812E+04, .1376952148438E+04},
 {    7181, .8311947265625E+04, -.1231243774414E+04, .1378251953125E+04},
 {    7182, .8357562500000E+04, -.1214150756836E+04, .1394651123047E+04},
 {    7183, .8334590820312E+04, -.1221013671875E+04, .1394563964844E+04},
 {    7184, .8313870117188E+04, -.1232621582031E+04, .1393336914062E+04},
 {    7185, .8359802734375E+04, -.1221440795898E+04, .1412593505859E+04},
 {    7186, .8337814453125E+04, -.1227108154297E+04, .1411134277344E+04},
 {    7187, .8317679687500E+04, -.1236644042969E+04, .1407564086914E+04},
 {    7188, .8297002929688E+04, -.1324975830078E+04, .1379181884766E+04},
 {    7189, .8286802734375E+04, -.1306438110352E+04, .1379815673828E+04},
 {    7190, .8283123046875E+04, -.1285592041016E+04, .1380044921875E+04},
 {    7191, .8300498046875E+04, -.1323879638672E+04, .1397088623047E+04},
 {    7192, .8291315429688E+04, -.1305677490234E+04, .1399659912109E+04},
 {    7193, .8288097656250E+04, -.1285592041016E+04, .1400621582031E+04},
 {    7194, .8308167968750E+04, -.1320447265625E+04, .1413409057617E+04},
 {    7195, .8301751953125E+04, -.1303681274414E+04, .1417247436523E+04},
 {    7196, .8299449218750E+04, -.1285592041016E+04, .1418489257812E+04},
 {    7197, .8286802734375E+04, -.1264745849609E+04, .1379815673828E+04},
 {    7198, .8297002929688E+04, -.1246208251953E+04, .1379181884766E+04},
 {    7199, .8291315429688E+04, -.1265506591797E+04, .1399659912109E+04},
 {    7200, .8300498046875E+04, -.1247304321289E+04, .1397088623047E+04},
 {    7201, .8301751953125E+04, -.1267502807617E+04, .1417247436523E+04},
 {    7202, .8308167968750E+04, -.1250736816406E+04, .1413409057617E+04},
 {    7203, .8417290039062E+04, -.1327818969727E+04, .1320799072266E+04},
 {    7204, .8412272460938E+04, -.1322098754883E+04, .1321111572266E+04},
 {    7205, .8428000976562E+04, -.1315719482422E+04, .1320132568359E+04},
 {    7206, .8421558593750E+04, -.1311670654297E+04, .1320533691406E+04},
 {    7207, .8434862304688E+04, -.1301082031250E+04, .1319705566406E+04},
 {    7208, .8427534179688E+04, -.1299041992188E+04, .1320162353516E+04},
 {    7209, .8437134765625E+04, -.1285072021484E+04, .1319563476562E+04},
 {    7210, .8429532226562E+04, -.1285210205078E+04, .1320037597656E+04},
 {    7211, .8423672851562E+04, -.1338306030273E+04, .1345851074219E+04},
 {    7212, .8419420898438E+04, -.1332012695312E+04, .1346115722656E+04},
 {    7213, .8439601562500E+04, -.1324167358398E+04, .1344859863281E+04},
 {    7214, .8433529296875E+04, -.1319597290039E+04, .1345238159180E+04},
 {    7215, .8450302734375E+04, -.1305735961914E+04, .1344194580078E+04},
 {    7216, .8443067382812E+04, -.1303393310547E+04, .1344644531250E+04},
 {    7217, .8453915039062E+04, -.1284722778320E+04, .1343969970703E+04},
 {    7218, .8446312500000E+04, -.1284863769531E+04, .1344442138672E+04},
 {    7219, .8444029296875E+04, -.1327088500977E+04, .1370033813477E+04},
 {    7220, .8438080078125E+04, -.1322361328125E+04, .1370404052734E+04},
 {    7221, .8456347656250E+04, -.1307424438477E+04, .1369267333984E+04},
 {    7222, .8449145507812E+04, -.1304987304688E+04, .1369715820312E+04},
 {    7223, .8460558593750E+04, -.1284594116211E+04, .1369005615234E+04},
 {    7224, .8452956054688E+04, -.1284735351562E+04, .1369478271484E+04},
 {    7225, .8434657226562E+04, -.1269439819336E+04, .1319718872070E+04},
 {    7226, .8427381835938E+04, -.1271656005859E+04, .1320171142578E+04},
 {    7227, .8427809570312E+04, -.1255175903320E+04, .1320144287109E+04},
 {    7228, .8421418945312E+04, -.1259300903320E+04, .1320542236328E+04},
 {    7229, .8417290039062E+04, -.1243365112305E+04, .1320799072266E+04},
 {    7230, .8412272460938E+04, -.1249085205078E+04, .1321111572266E+04},
 {    7231, .8449885742188E+04, -.1264368164062E+04, .1344219970703E+04},
 {    7232, .8442712890625E+04, -.1266887329102E+04, .1344666503906E+04},
 {    7233, .8439232421875E+04, -.1246573120117E+04, .1344883300781E+04},
 {    7234, .8433221679688E+04, -.1251213745117E+04, .1345257202148E+04},
 {    7235, .8423672851562E+04, -.1232878051758E+04, .1345851074219E+04},
 {    7236, .8419420898438E+04, -.1239171386719E+04, .1346115722656E+04},
 {    7237, .8455847656250E+04, -.1262530639648E+04, .1369298583984E+04},
 {    7238, .8448708007812E+04, -.1265145263672E+04, .1369742919922E+04},
 {    7239, .8443588867188E+04, -.1243601318359E+04, .1370060913086E+04},
 {    7240, .8437699218750E+04, -.1248402343750E+04, .1370427368164E+04},
 {    7241, .8404849609375E+04, -.1336060668945E+04, .1321573242188E+04},
 {    7242, .8390801757812E+04, -.1341091796875E+04, .1322447265625E+04},
 {    7243, .8375976562500E+04, -.1342738525391E+04, .1323369873047E+04},
 {    7244, .8401365234375E+04, -.1329289062500E+04, .1321789306641E+04},
 {    7245, .8389063476562E+04, -.1333676147461E+04, .1322555419922E+04},
 {    7246, .8376084960938E+04, -.1335119873047E+04, .1323362548828E+04},
 {    7247, .8409228515625E+04, -.1345753906250E+04, .1346749755859E+04},
 {    7248, .8393606445312E+04, -.1350203613281E+04, .1347721679688E+04},
 {    7249, .8377422851562E+04, -.1351626220703E+04, .1348728637695E+04},
 {    7250, .8406329101562E+04, -.1338718017578E+04, .1346930419922E+04},
 {    7251, .8392181640625E+04, -.1342721191406E+04, .1347810302734E+04},
 {    7252, .8377537109375E+04, -.1344007812500E+04, .1348721923828E+04},
 {    7253, .8411183593750E+04, -.1348697753906E+04, .1372077270508E+04},
 {    7254, .8395309570312E+04, -.1352840087891E+04, .1373065307617E+04},
 {    7255, .8378955078125E+04, -.1354165283203E+04, .1374082275391E+04},
 {    7256, .8408476562500E+04, -.1341591796875E+04, .1372244873047E+04},
 {    7257, .8393990234375E+04, -.1345341308594E+04, .1373147216797E+04},
 {    7258, .8379074218750E+04, -.1346546997070E+04, .1374074951172E+04},
 {    7259, .8401365234375E+04, -.1241895019531E+04, .1321789306641E+04},
 {    7260, .8389063476562E+04, -.1237507934570E+04, .1322555419922E+04},
 {    7261, .8376084960938E+04, -.1236064208984E+04, .1323362548828E+04},
 {    7262, .8404849609375E+04, -.1235123413086E+04, .1321573242188E+04},
 {    7263, .8390801757812E+04, -.1230092285156E+04, .1322447265625E+04},
 {    7264, .8375976562500E+04, -.1228445434570E+04, .1323369873047E+04},
 {    7265, .8406329101562E+04, -.1232466064453E+04, .1346930419922E+04},
 {    7266, .8392181640625E+04, -.1228462890625E+04, .1347810302734E+04},
 {    7267, .8377537109375E+04, -.1227176269531E+04, .1348721923828E+04},
 {    7268, .8409228515625E+04, -.1225430175781E+04, .1346749755859E+04},
 {    7269, .8393606445312E+04, -.1220980346680E+04, .1347721679688E+04},
 {    7270, .8377422851562E+04, -.1219557861328E+04, .1348728637695E+04},
 {    7271, .8408476562500E+04, -.1229592163086E+04, .1372244873047E+04},
 {    7272, .8393990234375E+04, -.1225842773438E+04, .1373147216797E+04},
 {    7273, .8379074218750E+04, -.1224637084961E+04, .1374074951172E+04},
 {    7274, .8411183593750E+04, -.1222486328125E+04, .1372077270508E+04},
 {    7275, .8395309570312E+04, -.1218343994141E+04, .1373065307617E+04},
 {    7276, .8378955078125E+04, -.1217018798828E+04, .1374082275391E+04},
 {    7277, .8356789062500E+04, -.1341491577148E+04, .1324562988281E+04},
 {    7278, .8337928710938E+04, -.1337738159180E+04, .1325736816406E+04},
 {    7279, .8319833984375E+04, -.1331215454102E+04, .1326862670898E+04},
 {    7280, .8357921875000E+04, -.1333978759766E+04, .1324492187500E+04},
 {    7281, .8340053710938E+04, -.1330520996094E+04, .1325604736328E+04},
 {    7282, .8322888671875E+04, -.1324460693359E+04, .1326672119141E+04},
 {    7283, .8356162109375E+04, -.1349730957031E+04, .1350051025391E+04},
 {    7284, .8335574218750E+04, -.1344100952148E+04, .1351332641602E+04},
 {    7285, .8316517578125E+04, -.1334470581055E+04, .1352518188477E+04},
 {    7286, .8357782226562E+04, -.1342299560547E+04, .1349950683594E+04},
 {    7287, .8338624023438E+04, -.1337180908203E+04, .1351142700195E+04},
 {    7288, .8320872070312E+04, -.1328335205078E+04, .1352247314453E+04},
 {    7289, .8356969726562E+04, -.1351923461914E+04, .1375450195312E+04},
 {    7290, .8335873046875E+04, -.1345319702148E+04, .1376762939453E+04},
 {    7291, .8358743164062E+04, -.1344522216797E+04, .1375340087891E+04},
 {    7292, .8339211914062E+04, -.1338517089844E+04, .1376555419922E+04},
 {    7293, .8357921875000E+04, -.1237205200195E+04, .1324492187500E+04},
 {    7294, .8340053710938E+04, -.1240663085938E+04, .1325604736328E+04},
 {    7295, .8322888671875E+04, -.1246723388672E+04, .1326672119141E+04},
 {    7296, .8356789062500E+04, -.1229692504883E+04, .1324562988281E+04},
 {    7297, .8337928710938E+04, -.1233445922852E+04, .1325736816406E+04},
 {    7298, .8319833984375E+04, -.1239968627930E+04, .1326862670898E+04},
 {    7299, .8357782226562E+04, -.1228884521484E+04, .1349950683594E+04},
 {    7300, .8338624023438E+04, -.1234003173828E+04, .1351142700195E+04},
 {    7301, .8320872070312E+04, -.1242848876953E+04, .1352247314453E+04},
 {    7302, .8356162109375E+04, -.1221453125000E+04, .1350051025391E+04},
 {    7303, .8335573242188E+04, -.1227083129883E+04, .1351333007812E+04},
 {    7304, .8316517578125E+04, -.1236713500977E+04, .1352518188477E+04},
 {    7305, .8358743164062E+04, -.1226661865234E+04, .1375340087891E+04},
 {    7306, .8339211914062E+04, -.1232666992188E+04, .1376555419922E+04},
 {    7307, .8356969726562E+04, -.1219260620117E+04, .1375450195312E+04},
 {    7308, .8335873046875E+04, -.1225864379883E+04, .1376762939453E+04},
 {    7309, .8307448242188E+04, -.1315373046875E+04, .1327632934570E+04},
 {    7310, .8302007812500E+04, -.1320470092773E+04, .1327972045898E+04},
 {    7311, .8295170898438E+04, -.1302306518555E+04, .1328396850586E+04},
 {    7312, .8288076171875E+04, -.1304989013672E+04, .1328838623047E+04},
 {    7313, .8290124023438E+04, -.1285089965820E+04, .1328711181641E+04},
 {    7314, .8282525390625E+04, -.1284899902344E+04, .1329183349609E+04},
 {    7315, .8307425781250E+04, -.1317081665039E+04, .1353083984375E+04},
 {    7316, .8301312500000E+04, -.1321512573242E+04, .1353463623047E+04},
 {    7317, .8297782226562E+04, -.1302428466797E+04, .1353684326172E+04},
 {    7318, .8290527343750E+04, -.1304684692383E+04, .1354134643555E+04},
 {    7319, .8294228515625E+04, -.1285242309570E+04, .1353904907227E+04},
 {    7320, .8286628906250E+04, -.1285070434570E+04, .1354377563477E+04},
 {    7321, .8309544921875E+04, -.1316502319336E+04, .1378401123047E+04},
 {    7322, .8303249023438E+04, -.1320694946289E+04, .1378792602539E+04},
 {    7323, .8301289062500E+04, -.1301871582031E+04, .1378914550781E+04},
 {    7324, .8293988281250E+04, -.1303989257812E+04, .1379369140625E+04},
 {    7325, .8298340820312E+04, -.1285326171875E+04, .1379097900391E+04},
 {    7326, .8290737304688E+04, -.1285161132812E+04, .1379571289062E+04},
 {    7327, .8295525390625E+04, -.1268322753906E+04, .1328375000000E+04},
 {    7328, .8288554687500E+04, -.1265419311523E+04, .1328808349609E+04},
 {    7329, .8307706054688E+04, -.1255612915039E+04, .1327617187500E+04},
 {    7330, .8302361328125E+04, -.1250429443359E+04, .1327949951172E+04},
 {    7331, .8297970703125E+04, -.1268330566406E+04, .1353672119141E+04},
 {    7332, .8290793945312E+04, -.1265862182617E+04, .1354118530273E+04},
 {    7333, .8307584960938E+04, -.1253933471680E+04, .1353073364258E+04},
 {    7334, .8301542968750E+04, -.1249414916992E+04, .1353449584961E+04},
 {    7335, .8301407226562E+04, -.1268979736328E+04, .1378907226562E+04},
 {    7336, .8294177734375E+04, -.1266654296875E+04, .1379357177734E+04},
 {    7337, .8309654296875E+04, -.1254544677734E+04, .1378393920898E+04},
 {    7338, .8303422851562E+04, -.1250264404297E+04, .1378781982422E+04},
 {    7339, .8421827148438E+04, -.1333951660156E+04, .1320517333984E+04},
 {    7340, .8434208007812E+04, -.1320121948242E+04, .1319746337891E+04},
 {    7341, .8442130859375E+04, -.1303327514648E+04, .1319253540039E+04},
 {    7342, .8444737304688E+04, -.1284935058594E+04, .1319091796875E+04},
 {    7343, .8427987304688E+04, -.1344563598633E+04, .1345582519531E+04},
 {    7344, .8445709960938E+04, -.1328701538086E+04, .1344479980469E+04},
 {    7345, .8457543945312E+04, -.1308057250977E+04, .1343742919922E+04},
 {    7346, .8461517578125E+04, -.1284583496094E+04, .1343496337891E+04},
 {    7347, .8450011718750E+04, -.1331780273438E+04, .1369661865234E+04},
 {    7348, .8463561523438E+04, -.1309840087891E+04, .1368818359375E+04},
 {    7349, .8468163085938E+04, -.1284454589844E+04, .1368531738281E+04},
 {    7350, .8441867187500E+04, -.1267022460938E+04, .1319270019531E+04},
 {    7351, .8433960937500E+04, -.1250700927734E+04, .1319761230469E+04},
 {    7352, .8421827148438E+04, -.1237232543945E+04, .1320517333984E+04},
 {    7353, .8457067382812E+04, -.1261871826172E+04, .1343773193359E+04},
 {    7354, .8445280273438E+04, -.1241967651367E+04, .1344506835938E+04},
 {    7355, .8427987304688E+04, -.1226620483398E+04, .1345582519531E+04},
 {    7356, .8462997070312E+04, -.1259939697266E+04, .1368853759766E+04},
 {    7357, .8449513671875E+04, -.1238836303711E+04, .1369692626953E+04},
 {    7358, .8407950195312E+04, -.1343015258789E+04, .1321379882812E+04},
 {    7359, .8392332031250E+04, -.1348553466797E+04, .1322351318359E+04},
 {    7360, .8375864257812E+04, -.1350357177734E+04, .1323376464844E+04},
 {    7361, .8412172851562E+04, -.1352774047852E+04, .1346566406250E+04},
 {    7362, .8395047851562E+04, -.1357680786133E+04, .1347631958008E+04},
 {    7363, .8377304687500E+04, -.1359244506836E+04, .1348736572266E+04},
 {    7364, .8413941406250E+04, -.1355789306641E+04, .1371905761719E+04},
 {    7365, .8396650390625E+04, -.1360335815430E+04, .1372981445312E+04},
 {    7366, .8378835937500E+04, -.1361783447266E+04, .1374089355469E+04},
 {    7367, .8407950195312E+04, -.1228168823242E+04, .1321379882812E+04},
 {    7368, .8392332031250E+04, -.1222630615234E+04, .1322351318359E+04},
 {    7369, .8375864257812E+04, -.1220826904297E+04, .1323376464844E+04},
 {    7370, .8412334960938E+04, -.1218476562500E+04, .1346556762695E+04},
 {    7371, .8395391601562E+04, -.1213566040039E+04, .1347610351562E+04},
 {    7372, .8377831054688E+04, -.1211932617188E+04, .1348703613281E+04},
 {    7373, .8413941406250E+04, -.1215394653320E+04, .1371905761719E+04},
 {    7374, .8396650390625E+04, -.1210848144531E+04, .1372981201172E+04},
 {    7375, .8378835937500E+04, -.1209400634766E+04, .1374089355469E+04},
 {    7376, .8355511718750E+04, -.1348983154297E+04, .1324642822266E+04},
 {    7377, .8335527343750E+04, -.1344889892578E+04, .1325886474609E+04},
 {    7378, .8316380859375E+04, -.1337839477539E+04, .1327077148438E+04},
 {    7379, .8354767578125E+04, -.1357202880859E+04, .1350137695312E+04},
 {    7380, .8332947265625E+04, -.1351193603516E+04, .1351496459961E+04},
 {    7381, .8312741210938E+04, -.1340990600586E+04, .1352753173828E+04},
 {    7382, .8355147460938E+04, -.1359315795898E+04, .1375563598633E+04},
 {    7383, .8332448242188E+04, -.1352090942383E+04, .1376975585938E+04},
 {    7384, .8355511718750E+04, -.1222200927734E+04, .1324642822266E+04},
 {    7385, .8335527343750E+04, -.1226294189453E+04, .1325886474609E+04},
 {    7386, .8316380859375E+04, -.1233344604492E+04, .1327077148438E+04},
 {    7387, .8355114257812E+04, -.1213921630859E+04, .1350116699219E+04},
 {    7388, .8333112304688E+04, -.1219928100586E+04, .1351485595703E+04},
 {    7389, .8312741210938E+04, -.1230193481445E+04, .1352753173828E+04},
 {    7390, .8355147460938E+04, -.1211868286133E+04, .1375563598633E+04},
 {    7391, .8332446289062E+04, -.1219093139648E+04, .1376975585938E+04},
 {    7392, .8296334960938E+04, -.1325418823242E+04, .1328324707031E+04},
 {    7393, .8280916015625E+04, -.1307561279297E+04, .1329284423828E+04},
 {    7394, .8274921875000E+04, -.1284723266602E+04, .1329656494141E+04},
 {    7395, .8295495117188E+04, -.1326303710938E+04, .1353825683594E+04},
 {    7396, .8283361328125E+04, -.1307163085938E+04, .1354580688477E+04},
 {    7397, .8279028320312E+04, -.1284906738281E+04, .1354849853516E+04},
 {    7398, .8296886718750E+04, -.1324821166992E+04, .1379188964844E+04},
 {    7399, .8286677734375E+04, -.1306068115234E+04, .1379823730469E+04},
 {    7400, .8283136718750E+04, -.1285002563477E+04, .1380044433594E+04},
 {    7401, .8281496093750E+04, -.1262635986328E+04, .1329247558594E+04},
 {    7402, .8296778320312E+04, -.1245410766602E+04, .1328297241211E+04},
 {    7403, .8283705078125E+04, -.1263179443359E+04, .1354559448242E+04},
 {    7404, .8295791992188E+04, -.1244538940430E+04, .1353807373047E+04},
 {    7405, .8286936523438E+04, -.1264376342773E+04, .1379807617188E+04},
 {    7406, .8297123046875E+04, -.1246052001953E+04, .1379173583984E+04},
 {    7407, .8396583007812E+04, -.1308452026367E+04, .1322087890625E+04},
 {    7408, .8396583007812E+04, -.1300832031250E+04, .1322087890625E+04},
 {    7409, .8396583007812E+04, -.1293212036133E+04, .1322087890625E+04},
 {    7410, .8396583007812E+04, -.1285592041016E+04, .1322087890625E+04},
 {    7411, .8416537109375E+04, -.1327764526367E+04, .1333570434570E+04},
 {    7412, .8397370117188E+04, -.1308452026367E+04, .1334763183594E+04},
 {    7413, .8428543945312E+04, -.1316176147461E+04, .1332823242188E+04},
 {    7414, .8397370117188E+04, -.1300832031250E+04, .1334762939453E+04},
 {    7415, .8436498046875E+04, -.1301495971680E+04, .1332328125000E+04},
 {    7416, .8397370117188E+04, -.1293212036133E+04, .1334762939453E+04},
 {    7417, .8439189453125E+04, -.1285011352539E+04, .1332161132812E+04},
 {    7418, .8397370117188E+04, -.1285592041016E+04, .1334762939453E+04},
 {    7419, .8396583007812E+04, -.1277972045898E+04, .1322087890625E+04},
 {    7420, .8396583007812E+04, -.1270352050781E+04, .1322087890625E+04},
 {    7421, .8396583007812E+04, -.1262732055664E+04, .1322087890625E+04},
 {    7422, .8436244140625E+04, -.1268958496094E+04, .1332344482422E+04},
 {    7423, .8397370117188E+04, -.1277972045898E+04, .1334762939453E+04},
 {    7424, .8428304687500E+04, -.1254706787109E+04, .1332838134766E+04},
 {    7425, .8397370117188E+04, -.1270352050781E+04, .1334762939453E+04},
 {    7426, .8416537109375E+04, -.1243419555664E+04, .1333570434570E+04},
 {    7427, .8397370117188E+04, -.1262732055664E+04, .1334762939453E+04},
 {    7428, .8389823242188E+04, -.1308452026367E+04, .1322508056641E+04},
 {    7429, .8383062500000E+04, -.1308452026367E+04, .1322928466797E+04},
 {    7430, .8376301757812E+04, -.1308452026367E+04, .1323349365234E+04},
 {    7431, .8404273437500E+04, -.1334692871094E+04, .1334333007812E+04},
 {    7432, .8390820312500E+04, -.1338843261719E+04, .1335170654297E+04},
 {    7433, .8376804687500E+04, -.1340198852539E+04, .1336042480469E+04},
 {    7434, .8390611328125E+04, -.1308452026367E+04, .1335182861328E+04},
 {    7435, .8383849609375E+04, -.1308452026367E+04, .1335604003906E+04},
 {    7436, .8377090820312E+04, -.1308452026367E+04, .1336024536133E+04},
 {    7437, .8389823242188E+04, -.1300832031250E+04, .1322508056641E+04},
 {    7438, .8383062500000E+04, -.1300832031250E+04, .1322928466797E+04},
 {    7439, .8376301757812E+04, -.1300832031250E+04, .1323349365234E+04},
 {    7440, .8389823242188E+04, -.1293212036133E+04, .1322508056641E+04},
 {    7441, .8383062500000E+04, -.1293212036133E+04, .1322928466797E+04},
 {    7442, .8376301757812E+04, -.1293212036133E+04, .1323349365234E+04},
 {    7443, .8389823242188E+04, -.1285592041016E+04, .1322508056641E+04},
 {    7444, .8383062500000E+04, -.1285592041016E+04, .1322928466797E+04},
 {    7445, .8376301757812E+04, -.1285592041016E+04, .1323349365234E+04},
 {    7446, .8390611328125E+04, -.1300832031250E+04, .1335182861328E+04},
 {    7447, .8383849609375E+04, -.1300832031250E+04, .1335604003906E+04},
 {    7448, .8377090820312E+04, -.1300832031250E+04, .1336024536133E+04},
 {    7449, .8390611328125E+04, -.1293212036133E+04, .1335182861328E+04},
 {    7450, .8383849609375E+04, -.1293212036133E+04, .1335604003906E+04},
 {    7451, .8377090820312E+04, -.1293212036133E+04, .1336024536133E+04},
 {    7452, .8390611328125E+04, -.1285592041016E+04, .1335182861328E+04},
 {    7453, .8383849609375E+04, -.1285592041016E+04, .1335604003906E+04},
 {    7454, .8377090820312E+04, -.1285592041016E+04, .1336024536133E+04},
 {    7455, .8389823242188E+04, -.1277972045898E+04, .1322508056641E+04},
 {    7456, .8383062500000E+04, -.1277972045898E+04, .1322928466797E+04},
 {    7457, .8376301757812E+04, -.1277972045898E+04, .1323349365234E+04},
 {    7458, .8389823242188E+04, -.1270352050781E+04, .1322508056641E+04},
 {    7459, .8383062500000E+04, -.1270352050781E+04, .1322928466797E+04},
 {    7460, .8376301757812E+04, -.1270352050781E+04, .1323349365234E+04},
 {    7461, .8389823242188E+04, -.1262732055664E+04, .1322508056641E+04},
 {    7462, .8383062500000E+04, -.1262732055664E+04, .1322928466797E+04},
 {    7463, .8376301757812E+04, -.1262732055664E+04, .1323349365234E+04},
 {    7464, .8390611328125E+04, -.1277972045898E+04, .1335182861328E+04},
 {    7465, .8383849609375E+04, -.1277972045898E+04, .1335604003906E+04},
 {    7466, .8377090820312E+04, -.1277972045898E+04, .1336024536133E+04},
 {    7467, .8390611328125E+04, -.1270352050781E+04, .1335182861328E+04},
 {    7468, .8383849609375E+04, -.1270352050781E+04, .1335604003906E+04},
 {    7469, .8377090820312E+04, -.1270352050781E+04, .1336024536133E+04},
 {    7470, .8390611328125E+04, -.1262732055664E+04, .1335182861328E+04},
 {    7471, .8383849609375E+04, -.1262732055664E+04, .1335604003906E+04},
 {    7472, .8377090820312E+04, -.1262732055664E+04, .1336024536133E+04},
 {    7473, .8404273437500E+04, -.1236491210938E+04, .1334333007812E+04},
 {    7474, .8390820312500E+04, -.1232340820312E+04, .1335170654297E+04},
 {    7475, .8376804687500E+04, -.1230985229492E+04, .1336042480469E+04},
 {    7476, .8361936523438E+04, -.1308452026367E+04, .1324243164062E+04},
 {    7477, .8347570312500E+04, -.1308452026367E+04, .1325136474609E+04},
 {    7478, .8333205078125E+04, -.1308452026367E+04, .1326030151367E+04},
 {    7479, .8357809570312E+04, -.1338767456055E+04, .1337224121094E+04},
 {    7480, .8339248046875E+04, -.1334473999023E+04, .1338379638672E+04},
 {    7481, .8321722656250E+04, -.1326990722656E+04, .1339469970703E+04},
 {    7482, .8362726562500E+04, -.1308452026367E+04, .1336918457031E+04},
 {    7483, .8348358398438E+04, -.1308452026367E+04, .1337812500000E+04},
 {    7484, .8333993164062E+04, -.1308452026367E+04, .1338706054688E+04},
 {    7485, .8361936523438E+04, -.1300832031250E+04, .1324243164062E+04},
 {    7486, .8347570312500E+04, -.1300832031250E+04, .1325136474609E+04},
 {    7487, .8333205078125E+04, -.1300832031250E+04, .1326030151367E+04},
 {    7488, .8361936523438E+04, -.1293212036133E+04, .1324243164062E+04},
 {    7489, .8347570312500E+04, -.1293212036133E+04, .1325136474609E+04},
 {    7490, .8333205078125E+04, -.1293212036133E+04, .1326030151367E+04},
 {    7491, .8361936523438E+04, -.1285592041016E+04, .1324243164062E+04},
 {    7492, .8347570312500E+04, -.1285592041016E+04, .1325136474609E+04},
 {    7493, .8333205078125E+04, -.1285592041016E+04, .1326030151367E+04},
 {    7494, .8362726562500E+04, -.1300832031250E+04, .1336918457031E+04},
 {    7495, .8348358398438E+04, -.1300832031250E+04, .1337812500000E+04},
 {    7496, .8333993164062E+04, -.1300832031250E+04, .1338706054688E+04},
 {    7497, .8362726562500E+04, -.1293212036133E+04, .1336918457031E+04},
 {    7498, .8348358398438E+04, -.1293212036133E+04, .1337812500000E+04},
 {    7499, .8333993164062E+04, -.1293212036133E+04, .1338706054688E+04},
 {    7500, .8362726562500E+04, -.1285592041016E+04, .1336918457031E+04},
 {    7501, .8348358398438E+04, -.1285592041016E+04, .1337812500000E+04},
 {    7502, .8333993164062E+04, -.1285592041016E+04, .1338706054688E+04},
 {    7503, .8361936523438E+04, -.1277972045898E+04, .1324243164062E+04},
 {    7504, .8347570312500E+04, -.1277972045898E+04, .1325136474609E+04},
 {    7505, .8333205078125E+04, -.1277972045898E+04, .1326030151367E+04},
 {    7506, .8361936523438E+04, -.1270352050781E+04, .1324243164062E+04},
 {    7507, .8347570312500E+04, -.1270352050781E+04, .1325136474609E+04},
 {    7508, .8333205078125E+04, -.1270352050781E+04, .1326030151367E+04},
 {    7509, .8361936523438E+04, -.1262732055664E+04, .1324243164062E+04},
 {    7510, .8347570312500E+04, -.1262732055664E+04, .1325136474609E+04},
 {    7511, .8333205078125E+04, -.1262732055664E+04, .1326030151367E+04},
 {    7512, .8362726562500E+04, -.1277972045898E+04, .1336918457031E+04},
 {    7513, .8348358398438E+04, -.1277972045898E+04, .1337812500000E+04},
 {    7514, .8333993164062E+04, -.1277972045898E+04, .1338706054688E+04},
 {    7515, .8362726562500E+04, -.1270352050781E+04, .1336918457031E+04},
 {    7516, .8348358398438E+04, -.1270352050781E+04, .1337812500000E+04},
 {    7517, .8333993164062E+04, -.1270352050781E+04, .1338706054688E+04},
 {    7518, .8362726562500E+04, -.1262732055664E+04, .1336918457031E+04},
 {    7519, .8348358398438E+04, -.1262732055664E+04, .1337812500000E+04},
 {    7520, .8333993164062E+04, -.1262732055664E+04, .1338706054688E+04},
 {    7521, .8357809570312E+04, -.1232416503906E+04, .1337224121094E+04},
 {    7522, .8339248046875E+04, -.1236710083008E+04, .1338379638672E+04},
 {    7523, .8321722656250E+04, -.1244193359375E+04, .1339469970703E+04},
 {    7524, .8307575195312E+04, -.1315470947266E+04, .1327625000000E+04},
 {    7525, .8295340820312E+04, -.1302585083008E+04, .1328386596680E+04},
 {    7526, .8290109375000E+04, -.1285592041016E+04, .1328711914062E+04},
 {    7527, .8307332031250E+04, -.1316724121094E+04, .1340365234375E+04},
 {    7528, .8296475585938E+04, -.1302757812500E+04, .1341040771484E+04},
 {    7529, .8292163085938E+04, -.1285592041016E+04, .1341308959961E+04},
 {    7530, .8295340820312E+04, -.1268598999023E+04, .1328386596680E+04},
 {    7531, .8307575195312E+04, -.1255713134766E+04, .1327625000000E+04},
 {    7532, .8296475585938E+04, -.1268426269531E+04, .1341040771484E+04},
 {    7533, .8307332031250E+04, -.1254459960938E+04, .1340365234375E+04},
 {    7534, .8369994140625E+04, -.1323692016602E+04, .1221945312500E+04},
 {    7535, .8369910156250E+04, -.1328770629883E+04, .1221950439453E+04},
 {    7536, .8360150390625E+04, -.1322393798828E+04, .1222557495117E+04},
 {    7537, .8358428710938E+04, -.1327356079102E+04, .1222663940430E+04},
 {    7538, .8350977539062E+04, -.1318587646484E+04, .1223127807617E+04},
 {    7539, .8347621093750E+04, -.1323220092773E+04, .1223337524414E+04},
 {    7540, .8343103515625E+04, -.1312532836914E+04, .1223618408203E+04},
 {    7541, .8338168945312E+04, -.1316535888672E+04, .1223925537109E+04},
 {    7542, .8337060546875E+04, -.1304642089844E+04, .1223994140625E+04},
 {    7543, .8330760742188E+04, -.1307634399414E+04, .1224385986328E+04},
 {    7544, .8333261718750E+04, -.1295453002930E+04, .1224230468750E+04},
 {    7545, .8326006835938E+04, -.1297067871094E+04, .1224681884766E+04},
 {    7546, .8331967773438E+04, -.1285592041016E+04, .1224311279297E+04},
 {    7547, .8324361328125E+04, -.1285595458984E+04, .1224784179688E+04},
 {    7548, .8333261718750E+04, -.1275731079102E+04, .1224230468750E+04},
 {    7549, .8326004882812E+04, -.1274122802734E+04, .1224682373047E+04},
 {    7550, .8337060546875E+04, -.1266541992188E+04, .1223994140625E+04},
 {    7551, .8330756835938E+04, -.1263555419922E+04, .1224386718750E+04},
 {    7552, .8343103515625E+04, -.1258651245117E+04, .1223618408203E+04},
 {    7553, .8338164062500E+04, -.1254652832031E+04, .1223925659180E+04},
 {    7554, .8350977539062E+04, -.1252596435547E+04, .1223127807617E+04},
 {    7555, .8347615234375E+04, -.1247967163086E+04, .1223338012695E+04},
 {    7556, .8360150390625E+04, -.1248790283203E+04, .1222557495117E+04},
 {    7557, .8358421875000E+04, -.1243829589844E+04, .1222664794922E+04},
 {    7558, .8369994140625E+04, -.1247492065430E+04, .1221945312500E+04},
 {    7559, .8369903320312E+04, -.1242413818359E+04, .1221950683594E+04},
 {    7560, .8371569335938E+04, -.1323692016602E+04, .1247296142578E+04},
 {    7561, .8371481445312E+04, -.1328770507812E+04, .1247301757812E+04},
 {    7562, .8361728515625E+04, -.1322393798828E+04, .1247909057617E+04},
 {    7563, .8360346679688E+04, -.1327275268555E+04, .1247993652344E+04},
 {    7564, .8352555664062E+04, -.1318587646484E+04, .1248478271484E+04},
 {    7565, .8349970703125E+04, -.1322956909180E+04, .1248640014648E+04},
 {    7566, .8344680664062E+04, -.1312532836914E+04, .1248969482422E+04},
 {    7567, .8341070312500E+04, -.1316089599609E+04, .1249193969727E+04},
 {    7568, .8338637695312E+04, -.1304642089844E+04, .1249344970703E+04},
 {    7569, .8334234375000E+04, -.1307157348633E+04, .1249618896484E+04},
 {    7570, .8334838867188E+04, -.1295453002930E+04, .1249581542969E+04},
 {    7571, .8329944335938E+04, -.1296753540039E+04, .1249885864258E+04},
 {    7572, .8333543945312E+04, -.1285592041016E+04, .1249662109375E+04},
 {    7573, .8328473632812E+04, -.1285594238281E+04, .1249978271484E+04},
 {    7574, .8334838867188E+04, -.1275731079102E+04, .1249581542969E+04},
 {    7575, .8329943359375E+04, -.1274434692383E+04, .1249885986328E+04},
 {    7576, .8338637695312E+04, -.1266541992188E+04, .1249344970703E+04},
 {    7577, .8334232421875E+04, -.1264030517578E+04, .1249619140625E+04},
 {    7578, .8344680664062E+04, -.1258651245117E+04, .1248969482422E+04},
 {    7579, .8341067382812E+04, -.1255097534180E+04, .1249194335938E+04},
 {    7580, .8352555664062E+04, -.1252596435547E+04, .1248478271484E+04},
 {    7581, .8349967773438E+04, -.1248229370117E+04, .1248640136719E+04},
 {    7582, .8361728515625E+04, -.1248790283203E+04, .1247909057617E+04},
 {    7583, .8360342773438E+04, -.1243909912109E+04, .1247994873047E+04},
 {    7584, .8371569335938E+04, -.1247492065430E+04, .1247296142578E+04},
 {    7585, .8371477539062E+04, -.1242413818359E+04, .1247302246094E+04},
 {    7586, .8372358398438E+04, -.1323692016602E+04, .1259971923828E+04},
 {    7587, .8372272460938E+04, -.1328770507812E+04, .1259976806641E+04},
 {    7588, .8362516601562E+04, -.1322393798828E+04, .1260583618164E+04},
 {    7589, .8360963867188E+04, -.1327316894531E+04, .1260681396484E+04},
 {    7590, .8353346679688E+04, -.1318587646484E+04, .1261154785156E+04},
 {    7591, .8350372070312E+04, -.1323092651367E+04, .1261339599609E+04},
 {    7592, .8345470703125E+04, -.1312532836914E+04, .1261644653320E+04},
 {    7593, .8341194335938E+04, -.1316316162109E+04, .1261911132812E+04},
 {    7594, .8339426757812E+04, -.1304642089844E+04, .1262020629883E+04},
 {    7595, .8334071289062E+04, -.1307397583008E+04, .1262353515625E+04},
 {    7596, .8335627929688E+04, -.1295453002930E+04, .1262257568359E+04},
 {    7597, .8329552734375E+04, -.1296910400391E+04, .1262635253906E+04},
 {    7598, .8334331054688E+04, -.1285592041016E+04, .1262337646484E+04},
 {    7599, .8327995117188E+04, -.1285594848633E+04, .1262732177734E+04},
 {    7600, .8335627929688E+04, -.1275731079102E+04, .1262257568359E+04},
 {    7601, .8329550781250E+04, -.1274279052734E+04, .1262635009766E+04},
 {    7602, .8339426757812E+04, -.1266541992188E+04, .1262020629883E+04},
 {    7603, .8334069335938E+04, -.1263791259766E+04, .1262354003906E+04},
 {    7604, .8345470703125E+04, -.1258651245117E+04, .1261644653320E+04},
 {    7605, .8341190429688E+04, -.1254871826172E+04, .1261911010742E+04},
 {    7606, .8353346679688E+04, -.1252596435547E+04, .1261154785156E+04},
 {    7607, .8350368164062E+04, -.1248094116211E+04, .1261340209961E+04},
 {    7608, .8362516601562E+04, -.1248790283203E+04, .1260583618164E+04},
 {    7609, .8360958007812E+04, -.1243868530273E+04, .1260681274414E+04},
 {    7610, .8372358398438E+04, -.1247492065430E+04, .1259971923828E+04},
 {    7611, .8372266601562E+04, -.1242413818359E+04, .1259977905273E+04},
 {    7612, .8373148437500E+04, -.1323692016602E+04, .1272647583008E+04},
 {    7613, .8373076171875E+04, -.1328770996094E+04, .1272651367188E+04},
 {    7614, .8363305664062E+04, -.1322393798828E+04, .1273259643555E+04},
 {    7615, .8360541992188E+04, -.1327540771484E+04, .1273431640625E+04},
 {    7616, .8354134765625E+04, -.1318587646484E+04, .1273830200195E+04},
 {    7617, .8348495117188E+04, -.1323859619141E+04, .1274180908203E+04},
 {    7618, .8346257812500E+04, -.1312532836914E+04, .1274320312500E+04},
 {    7619, .8337518554688E+04, -.1317677124023E+04, .1274863647461E+04},
 {    7620, .8340215820312E+04, -.1304641967773E+04, .1274696411133E+04},
 {    7621, .8328399414062E+04, -.1308974975586E+04, .1275430908203E+04},
 {    7622, .8336416992188E+04, -.1295453002930E+04, .1274932617188E+04},
 {    7623, .8322166992188E+04, -.1298011474609E+04, .1275819091797E+04},
 {    7624, .8335121093750E+04, -.1285592041016E+04, .1275013305664E+04},
 {    7625, .8319910156250E+04, -.1285598388672E+04, .1275959716797E+04},
 {    7626, .8336416992188E+04, -.1275731079102E+04, .1274932617188E+04},
 {    7627, .8322162109375E+04, -.1273184448242E+04, .1275818847656E+04},
 {    7628, .8340215820312E+04, -.1266541992188E+04, .1274696411133E+04},
 {    7629, .8328391601562E+04, -.1262218994141E+04, .1275431396484E+04},
 {    7630, .8346257812500E+04, -.1258651245117E+04, .1274320312500E+04},
 {    7631, .8337508789062E+04, -.1253514648438E+04, .1274864501953E+04},
 {    7632, .8354134765625E+04, -.1252596435547E+04, .1273830200195E+04},
 {    7633, .8348483398438E+04, -.1247329345703E+04, .1274182373047E+04},
 {    7634, .8363305664062E+04, -.1248790283203E+04, .1273259643555E+04},
 {    7635, .8360529296875E+04, -.1243645629883E+04, .1273432861328E+04},
 {    7636, .8373148437500E+04, -.1247492065430E+04, .1272647583008E+04},
 {    7637, .8373063476562E+04, -.1242413696289E+04, .1272652587891E+04},
 {    7638, .8373935546875E+04, -.1323692016602E+04, .1285322753906E+04},
 {    7639, .8373880859375E+04, -.1328771484375E+04, .1285326049805E+04},
 {    7640, .8364093750000E+04, -.1322393798828E+04, .1285935424805E+04},
 {    7641, .8358826171875E+04, -.1327779052734E+04, .1286262451172E+04},
 {    7642, .8354922851562E+04, -.1318587646484E+04, .1286505859375E+04},
 {    7643, .8344044921875E+04, -.1324749267578E+04, .1287182373047E+04},
 {    7644, .8347049804688E+04, -.1312532836914E+04, .1286995239258E+04},
 {    7645, .8329909179688E+04, -.1319458007812E+04, .1288061523438E+04},
 {    7646, .8341003906250E+04, -.1304641967773E+04, .1287371826172E+04},
 {    7647, .8317107421875E+04, -.1311455444336E+04, .1288858642578E+04},
 {    7648, .8337206054688E+04, -.1295453002930E+04, .1287608154297E+04},
 {    7649, .8307105468750E+04, -.1300135498047E+04, .1289480712891E+04},
 {    7650, .8335910156250E+04, -.1285592041016E+04, .1287688964844E+04},
 {    7651, .8302954101562E+04, -.1285601684570E+04, .1289739135742E+04},
 {    7652, .8337206054688E+04, -.1275731079102E+04, .1287608154297E+04},
 {    7653, .8307095703125E+04, -.1271064941406E+04, .1289481445312E+04},
 {    7654, .8341003906250E+04, -.1266541992188E+04, .1287371826172E+04},
 {    7655, .8317092773438E+04, -.1259740356445E+04, .1288859497070E+04},
 {    7656, .8347049804688E+04, -.1258651245117E+04, .1286995239258E+04},
 {    7657, .8329892578125E+04, -.1251734375000E+04, .1288062744141E+04},
 {    7658, .8354922851562E+04, -.1252596435547E+04, .1286505859375E+04},
 {    7659, .8344026367188E+04, -.1246439941406E+04, .1287183837891E+04},
 {    7660, .8364093750000E+04, -.1248790283203E+04, .1285935424805E+04},
 {    7661, .8358807617188E+04, -.1243407592773E+04, .1286263916016E+04},
 {    7662, .8373935546875E+04, -.1247492065430E+04, .1285322753906E+04},
 {    7663, .8373863281250E+04, -.1242413330078E+04, .1285327392578E+04},
 {    7664, .8374723632812E+04, -.1323692016602E+04, .1297997802734E+04},
 {    7665, .8374642578125E+04, -.1331311157227E+04, .1298003662109E+04},
 {    7666, .8364881835938E+04, -.1322393798828E+04, .1298610839844E+04},
 {    7667, .8358941406250E+04, -.1330241577148E+04, .1298980224609E+04},
 {    7668, .8355709960938E+04, -.1318587646484E+04, .1299182006836E+04},
 {    7669, .8343544921875E+04, -.1326989135742E+04, .1299938232422E+04},
 {    7670, .8347834960938E+04, -.1312532836914E+04, .1299671875000E+04},
 {    7671, .8328863281250E+04, -.1321317871094E+04, .1300851562500E+04},
 {    7672, .8341792968750E+04, -.1304641967773E+04, .1300046875000E+04},
 {    7673, .8315631835938E+04, -.1312783081055E+04, .1301674804688E+04},
 {    7674, .8337995117188E+04, -.1295453002930E+04, .1300283081055E+04},
 {    7675, .8305403320312E+04, -.1300801879883E+04, .1302311035156E+04},
 {    7676, .8336699218750E+04, -.1285592041016E+04, .1300364257812E+04},
 {    7677, .8301206054688E+04, -.1285606811523E+04, .1302572265625E+04},
 {    7678, .8337995117188E+04, -.1275731079102E+04, .1300283081055E+04},
 {    7679, .8305389648438E+04, -.1270407714844E+04, .1302312011719E+04},
 {    7680, .8341792968750E+04, -.1266541992188E+04, .1300046875000E+04},
 {    7681, .8315609375000E+04, -.1258419433594E+04, .1301676513672E+04},
 {    7682, .8347834960938E+04, -.1258651245117E+04, .1299671875000E+04},
 {    7683, .8328836914062E+04, -.1249879150391E+04, .1300853515625E+04},
 {    7684, .8355709960938E+04, -.1252596435547E+04, .1299182006836E+04},
 {    7685, .8343517578125E+04, -.1244203125000E+04, .1299939941406E+04},
 {    7686, .8364881835938E+04, -.1248790283203E+04, .1298610839844E+04},
 {    7687, .8358913085938E+04, -.1240946533203E+04, .1298982421875E+04},
 {    7688, .8374723632812E+04, -.1247492065430E+04, .1297997802734E+04},
 {    7689, .8374612304688E+04, -.1239874145508E+04, .1298005126953E+04},
 {    7690, .8376301757812E+04, -.1323692016602E+04, .1323349365234E+04},
 {    7691, .8376186523438E+04, -.1335120849609E+04, .1323356689453E+04},
 {    7692, .8366458984375E+04, -.1322393798828E+04, .1323962402344E+04},
 {    7693, .8358232421875E+04, -.1334020263672E+04, .1324473632812E+04},
 {    7694, .8357288085938E+04, -.1318587646484E+04, .1324532470703E+04},
 {    7695, .8340564453125E+04, -.1330657714844E+04, .1325572753906E+04},
 {    7696, .8349412109375E+04, -.1312532836914E+04, .1325021728516E+04},
 {    7697, .8323565429688E+04, -.1324765136719E+04, .1326630737305E+04},
 {    7698, .8343369140625E+04, -.1304641967773E+04, .1325397949219E+04},
 {    7699, .8307975585938E+04, -.1315777221680E+04, .1327599853516E+04},
 {    7700, .8339570312500E+04, -.1295453002930E+04, .1325634277344E+04},
 {    7701, .8295489257812E+04, -.1302806030273E+04, .1328377197266E+04},
 {    7702, .8338275390625E+04, -.1285592041016E+04, .1325714721680E+04},
 {    7703, .8339570312500E+04, -.1275730957031E+04, .1325634277344E+04},
 {    7704, .8295469726562E+04, -.1268407592773E+04, .1328378906250E+04},
 {    7705, .8343369140625E+04, -.1266541992188E+04, .1325397949219E+04},
 {    7706, .8307948242188E+04, -.1255428222656E+04, .1327602294922E+04},
 {    7707, .8349412109375E+04, -.1258651245117E+04, .1325021728516E+04},
 {    7708, .8323534179688E+04, -.1246432983398E+04, .1326632080078E+04},
 {    7709, .8357288085938E+04, -.1252596435547E+04, .1324532470703E+04},
 {    7710, .8340530273438E+04, -.1240535400391E+04, .1325574951172E+04},
 {    7711, .8366458984375E+04, -.1248790283203E+04, .1323962402344E+04},
 {    7712, .8358198242188E+04, -.1237168457031E+04, .1324475585938E+04},
 {    7713, .8376301757812E+04, -.1247492065430E+04, .1323349365234E+04},
 {    7714, .8376151367188E+04, -.1236064453125E+04, .1323358520508E+04},
 {    7715, .8379835937500E+04, -.1248790283203E+04, .1221332153320E+04},
 {    7716, .8380557617188E+04, -.1244006713867E+04, .1221288085938E+04},
 {    7717, .8389005859375E+04, -.1252596435547E+04, .1220762451172E+04},
 {    7718, .8390294921875E+04, -.1248629394531E+04, .1220682006836E+04},
 {    7719, .8396881835938E+04, -.1258651245117E+04, .1220272338867E+04},
 {    7720, .8398389648438E+04, -.1255752075195E+04, .1220178710938E+04},
 {    7721, .8402924804688E+04, -.1266541992188E+04, .1219895629883E+04},
 {    7722, .8404384765625E+04, -.1264721435547E+04, .1219805786133E+04},
 {    7723, .8406723632812E+04, -.1275731079102E+04, .1219660034180E+04},
 {    7724, .8408052734375E+04, -.1274871582031E+04, .1219577270508E+04},
 {    7725, .8408020507812E+04, -.1285592041016E+04, .1219579345703E+04},
 {    7726, .8409285156250E+04, -.1285595336914E+04, .1219500488281E+04},
 {    7727, .8406723632812E+04, -.1295453002930E+04, .1219660034180E+04},
 {    7728, .8408052734375E+04, -.1296318969727E+04, .1219577270508E+04},
 {    7729, .8402924804688E+04, -.1304642089844E+04, .1219895629883E+04},
 {    7730, .8404381835938E+04, -.1306468627930E+04, .1219805664062E+04},
 {    7731, .8396881835938E+04, -.1312532836914E+04, .1220272338867E+04},
 {    7732, .8398385742188E+04, -.1315437133789E+04, .1220178955078E+04},
 {    7733, .8389004882812E+04, -.1318587646484E+04, .1220762451172E+04},
 {    7734, .8390290039062E+04, -.1322558349609E+04, .1220682128906E+04},
 {    7735, .8379835937500E+04, -.1322393798828E+04, .1221332397461E+04},
 {    7736, .8380551757812E+04, -.1327179321289E+04, .1221287963867E+04},
 {    7737, .8381411132812E+04, -.1248790283203E+04, .1246684326172E+04},
 {    7738, .8382133789062E+04, -.1244006103516E+04, .1246639160156E+04},
 {    7739, .8390583007812E+04, -.1252596435547E+04, .1246113159180E+04},
 {    7740, .8391872070312E+04, -.1248628295898E+04, .1246032958984E+04},
 {    7741, .8398461914062E+04, -.1258651245117E+04, .1245623046875E+04},
 {    7742, .8399967773438E+04, -.1255750732422E+04, .1245529296875E+04},
 {    7743, .8404502929688E+04, -.1266541992188E+04, .1245247314453E+04},
 {    7744, .8405961914062E+04, -.1264719970703E+04, .1245156494141E+04},
 {    7745, .8408301757812E+04, -.1275731079102E+04, .1245010498047E+04},
 {    7746, .8409628906250E+04, -.1274870239258E+04, .1244928222656E+04},
 {    7747, .8409595703125E+04, -.1285592041016E+04, .1244930419922E+04},
 {    7748, .8410863281250E+04, -.1285594116211E+04, .1244851806641E+04},
 {    7749, .8408301757812E+04, -.1295453002930E+04, .1245010498047E+04},
 {    7750, .8409627929688E+04, -.1296317993164E+04, .1244928466797E+04},
 {    7751, .8404502929688E+04, -.1304642089844E+04, .1245247314453E+04},
 {    7752, .8405958984375E+04, -.1306467895508E+04, .1245156250000E+04},
 {    7753, .8398461914062E+04, -.1312532836914E+04, .1245623046875E+04},
 {    7754, .8399965820312E+04, -.1315436645508E+04, .1245529418945E+04},
 {    7755, .8390583007812E+04, -.1318587646484E+04, .1246113159180E+04},
 {    7756, .8391869140625E+04, -.1322558105469E+04, .1246033325195E+04},
 {    7757, .8381411132812E+04, -.1322393798828E+04, .1246684326172E+04},
 {    7758, .8382129882812E+04, -.1327179199219E+04, .1246638671875E+04},
 {    7759, .8382202148438E+04, -.1248790283203E+04, .1259359130859E+04},
 {    7760, .8382924804688E+04, -.1244006347656E+04, .1259314453125E+04},
 {    7761, .8391372070312E+04, -.1252596435547E+04, .1258788085938E+04},
 {    7762, .8392662109375E+04, -.1248628906250E+04, .1258708496094E+04},
 {    7763, .8399248046875E+04, -.1258651245117E+04, .1258299316406E+04},
 {    7764, .8400754882812E+04, -.1255751464844E+04, .1258204833984E+04},
 {    7765, .8405288085938E+04, -.1266541992188E+04, .1257923095703E+04},
 {    7766, .8406750000000E+04, -.1264720825195E+04, .1257831542969E+04},
 {    7767, .8409087890625E+04, -.1275731079102E+04, .1257686523438E+04},
 {    7768, .8410417968750E+04, -.1274870971680E+04, .1257603759766E+04},
 {    7769, .8410385742188E+04, -.1285592041016E+04, .1257605712891E+04},
 {    7770, .8411650390625E+04, -.1285594848633E+04, .1257526611328E+04},
 {    7771, .8409087890625E+04, -.1295453002930E+04, .1257686523438E+04},
 {    7772, .8410416992188E+04, -.1296318481445E+04, .1257603759766E+04},
 {    7773, .8405288085938E+04, -.1304642089844E+04, .1257923095703E+04},
 {    7774, .8406748046875E+04, -.1306468261719E+04, .1257832275391E+04},
 {    7775, .8399248046875E+04, -.1312532836914E+04, .1258299316406E+04},
 {    7776, .8400750976562E+04, -.1315436889648E+04, .1258205200195E+04},
 {    7777, .8391372070312E+04, -.1318587646484E+04, .1258788085938E+04},
 {    7778, .8392656250000E+04, -.1322558227539E+04, .1258708740234E+04},
 {    7779, .8382202148438E+04, -.1322393798828E+04, .1259359130859E+04},
 {    7780, .8382920898438E+04, -.1327179199219E+04, .1259314697266E+04},
 {    7781, .8382988281250E+04, -.1248790283203E+04, .1272035156250E+04},
 {    7782, .8383718750000E+04, -.1244008178711E+04, .1271989624023E+04},
 {    7783, .8392161132812E+04, -.1252596435547E+04, .1271464355469E+04},
 {    7784, .8393453125000E+04, -.1248632202148E+04, .1271383544922E+04},
 {    7785, .8400035156250E+04, -.1258651245117E+04, .1270975219727E+04},
 {    7786, .8401545898438E+04, -.1255755493164E+04, .1270881103516E+04},
 {    7787, .8406077148438E+04, -.1266541992188E+04, .1270598754883E+04},
 {    7788, .8407541015625E+04, -.1264725097656E+04, .1270507324219E+04},
 {    7789, .8409875976562E+04, -.1275731079102E+04, .1270362548828E+04},
 {    7790, .8411208007812E+04, -.1274875000000E+04, .1270278808594E+04},
 {    7791, .8411173828125E+04, -.1285592041016E+04, .1270281494141E+04},
 {    7792, .8412440429688E+04, -.1285598388672E+04, .1270202148438E+04},
 {    7793, .8409875976562E+04, -.1295453002930E+04, .1270362548828E+04},
 {    7794, .8411205078125E+04, -.1296321411133E+04, .1270279052734E+04},
 {    7795, .8406077148438E+04, -.1304642089844E+04, .1270598754883E+04},
 {    7796, .8407536132812E+04, -.1306470458984E+04, .1270507568359E+04},
 {    7797, .8400035156250E+04, -.1312532836914E+04, .1270975219727E+04},
 {    7798, .8401538085938E+04, -.1315438232422E+04, .1270881591797E+04},
 {    7799, .8392161132812E+04, -.1318587646484E+04, .1271464355469E+04},
 {    7800, .8393442382812E+04, -.1322558837891E+04, .1271384277344E+04},
 {    7801, .8382988281250E+04, -.1322393798828E+04, .1272035156250E+04},
 {    7802, .8383707031250E+04, -.1327179443359E+04, .1271990722656E+04},
 {    7803, .8383777343750E+04, -.1248790283203E+04, .1284710449219E+04},
 {    7804, .8384860351562E+04, -.1243917114258E+04, .1284642822266E+04},
 {    7805, .8392949218750E+04, -.1252596435547E+04, .1284139892578E+04},
 {    7806, .8395051757812E+04, -.1248320922852E+04, .1284009277344E+04},
 {    7807, .8400823242188E+04, -.1258651245117E+04, .1283649902344E+04},
 {    7808, .8403710937500E+04, -.1255272949219E+04, .1283470458984E+04},
 {    7809, .8406867187500E+04, -.1266541992188E+04, .1283273925781E+04},
 {    7810, .8410284179688E+04, -.1264232299805E+04, .1283060791016E+04},
 {    7811, .8410666015625E+04, -.1275731079102E+04, .1283037231445E+04},
 {    7812, .8414372070312E+04, -.1274569824219E+04, .1282806884766E+04},
 {    7813, .8411963867188E+04, -.1285592041016E+04, .1282956787109E+04},
 {    7814, .8415763671875E+04, -.1285601440430E+04, .1282720703125E+04},
 {    7815, .8410666015625E+04, -.1295453002930E+04, .1283037231445E+04},
 {    7816, .8414367187500E+04, -.1296632568359E+04, .1282807128906E+04},
 {    7817, .8406867187500E+04, -.1304642089844E+04, .1283273925781E+04},
 {    7818, .8410275390625E+04, -.1306968383789E+04, .1283061035156E+04},
 {    7819, .8400823242188E+04, -.1312532836914E+04, .1283649902344E+04},
 {    7820, .8403697265625E+04, -.1315924682617E+04, .1283471679688E+04},
 {    7821, .8392949218750E+04, -.1318587646484E+04, .1284139892578E+04},
 {    7822, .8395036132812E+04, -.1322873046875E+04, .1284010009766E+04},
 {    7823, .8383777343750E+04, -.1322393798828E+04, .1284710449219E+04},
 {    7824, .8384841796875E+04, -.1327271972656E+04, .1284643798828E+04},
 {    7825, .8373843750000E+04, -.1328770385742E+04, .1285328369141E+04},
 {    7826, .8384567382812E+04, -.1248790283203E+04, .1297385253906E+04},
 {    7827, .8386103515625E+04, -.1241500732422E+04, .1297291137695E+04},
 {    7828, .8393738281250E+04, -.1252596435547E+04, .1296815429688E+04},
 {    7829, .8396688476562E+04, -.1246261962891E+04, .1296631591797E+04},
 {    7830, .8401612304688E+04, -.1258651245117E+04, .1296325439453E+04},
 {    7831, .8405602539062E+04, -.1253702514648E+04, .1296076904297E+04},
 {    7832, .8407656250000E+04, -.1266541992188E+04, .1295949707031E+04},
 {    7833, .8412291992188E+04, -.1263201660156E+04, .1295660888672E+04},
 {    7834, .8411454101562E+04, -.1275731079102E+04, .1295713134766E+04},
 {    7835, .8416422851562E+04, -.1274065673828E+04, .1295403808594E+04},
 {    7836, .8412751953125E+04, -.1285592041016E+04, .1295633056641E+04},
 {    7837, .8417820312500E+04, -.1285606567383E+04, .1295317138672E+04},
 {    7838, .8411454101562E+04, -.1295453002930E+04, .1295713134766E+04},
 {    7839, .8416416015625E+04, -.1297146606445E+04, .1295404296875E+04},
 {    7840, .8407656250000E+04, -.1304641967773E+04, .1295949707031E+04},
 {    7841, .8412279296875E+04, -.1308008178711E+04, .1295661621094E+04},
 {    7842, .8401612304688E+04, -.1312532836914E+04, .1296325439453E+04},
 {    7843, .8405582031250E+04, -.1317502807617E+04, .1296078369141E+04},
 {    7844, .8393738281250E+04, -.1318587646484E+04, .1296815429688E+04},
 {    7845, .8396664062500E+04, -.1324937744141E+04, .1296633056641E+04},
 {    7846, .8384567382812E+04, -.1322393798828E+04, .1297385253906E+04},
 {    7847, .8386076171875E+04, -.1329691406250E+04, .1297292480469E+04},
 {    7848, .8374583984375E+04, -.1331309448242E+04, .1298006591797E+04},
 {    7849, .8386143554688E+04, -.1248790283203E+04, .1322736572266E+04},
 {    7850, .8389496093750E+04, -.1237611694336E+04, .1322528808594E+04},
 {    7851, .8395316406250E+04, -.1252596435547E+04, .1322166015625E+04},
 {    7852, .8402095703125E+04, -.1242272705078E+04, .1321744506836E+04},
 {    7853, .8403189453125E+04, -.1258651245117E+04, .1321675903320E+04},
 {    7854, .8413180664062E+04, -.1249873901367E+04, .1321054809570E+04},
 {    7855, .8409233398438E+04, -.1266541992188E+04, .1321300292969E+04},
 {    7856, .8421928710938E+04, -.1260086669922E+04, .1320510253906E+04},
 {    7857, .8413033203125E+04, -.1275731079102E+04, .1321063720703E+04},
 {    7858, .8427578125000E+04, -.1272295776367E+04, .1320159179688E+04},
 {    7859, .8414328125000E+04, -.1285592041016E+04, .1320983398438E+04},
 {    7860, .8429538085938E+04, -.1285609863281E+04, .1320037109375E+04},
 {    7861, .8413033203125E+04, -.1295453002930E+04, .1321063720703E+04},
 {    7862, .8427568359375E+04, -.1298922485352E+04, .1320159912109E+04},
 {    7863, .8409233398438E+04, -.1304641967773E+04, .1321300292969E+04},
 {    7864, .8421909179688E+04, -.1311127197266E+04, .1320512207031E+04},
 {    7865, .8403189453125E+04, -.1312532836914E+04, .1321675903320E+04},
 {    7866, .8413153320312E+04, -.1321333374023E+04, .1321057006836E+04},
 {    7867, .8395316406250E+04, -.1318587646484E+04, .1322166015625E+04},
 {    7868, .8402063476562E+04, -.1328927612305E+04, .1321746337891E+04},
 {    7869, .8386143554688E+04, -.1322393798828E+04, .1322736572266E+04},
 {    7870, .8389460937500E+04, -.1333580932617E+04, .1322531005859E+04},
 {    7871, .8376116210938E+04, -.1335119018555E+04, .1323360839844E+04},
 {    7872, .8369833984375E+04, -.1333849487305E+04, .1221955566406E+04},
 {    7873, .8356708984375E+04, -.1332319824219E+04, .1222771850586E+04},
 {    7874, .8344273437500E+04, -.1327838256836E+04, .1223545654297E+04},
 {    7875, .8333261718750E+04, -.1320519531250E+04, .1224230468750E+04},
 {    7876, .8324483398438E+04, -.1310622680664E+04, .1224776977539E+04},
 {    7877, .8318759765625E+04, -.1298691284180E+04, .1225133300781E+04},
 {    7878, .8316754882812E+04, -.1285604614258E+04, .1225257568359E+04},
 {    7879, .8318751953125E+04, -.1272516967773E+04, .1225132812500E+04},
 {    7880, .8324470703125E+04, -.1260582397461E+04, .1224777832031E+04},
 {    7881, .8333245117188E+04, -.1250680786133E+04, .1224231445312E+04},
 {    7882, .8344250976562E+04, -.1243357055664E+04, .1223546630859E+04},
 {    7883, .8356684570312E+04, -.1238870117188E+04, .1222773681641E+04},
 {    7884, .8369809570312E+04, -.1237335937500E+04, .1221956909180E+04},
 {    7885, .8371426757812E+04, -.1332579711914E+04, .1247305175781E+04},
 {    7886, .8358800781250E+04, -.1331060913086E+04, .1248090454102E+04},
 {    7887, .8346876953125E+04, -.1326624877930E+04, .1248832641602E+04},
 {    7888, .8336383789062E+04, -.1319424438477E+04, .1249484863281E+04},
 {    7889, .8328097656250E+04, -.1309759643555E+04, .1250000732422E+04},
 {    7890, .8322735351562E+04, -.1298205444336E+04, .1250334350586E+04},
 {    7891, .8320869140625E+04, -.1285601074219E+04, .1250449951172E+04},
 {    7892, .8322729492188E+04, -.1272995849609E+04, .1250334960938E+04},
 {    7893, .8328087890625E+04, -.1261439453125E+04, .1250001464844E+04},
 {    7894, .8336370117188E+04, -.1251771362305E+04, .1249486083984E+04},
 {    7895, .8346861328125E+04, -.1244567382812E+04, .1248833984375E+04},
 {    7896, .8358783203125E+04, -.1240127441406E+04, .1248091552734E+04},
 {    7897, .8371408203125E+04, -.1238605224609E+04, .1247306396484E+04},
 {    7898, .8372208007812E+04, -.1332579467773E+04, .1259980468750E+04},
 {    7899, .8359929687500E+04, -.1330983154297E+04, .1260745605469E+04},
 {    7900, .8348434570312E+04, -.1326372436523E+04, .1261460449219E+04},
 {    7901, .8338485351562E+04, -.1318988891602E+04, .1262079711914E+04},
 {    7902, .8330767578125E+04, -.1309286621094E+04, .1262559570312E+04},
 {    7903, .8325877929688E+04, -.1297889282227E+04, .1262863281250E+04},
 {    7904, .8324191406250E+04, -.1285599121094E+04, .1262968994141E+04},
 {    7905, .8325874023438E+04, -.1273308349609E+04, .1262864013672E+04},
 {    7906, .8330759765625E+04, -.1261909423828E+04, .1262560302734E+04},
 {    7907, .8338475585938E+04, -.1252205078125E+04, .1262079956055E+04},
 {    7908, .8348422851562E+04, -.1244818359375E+04, .1261460937500E+04},
 {    7909, .8359916015625E+04, -.1240204589844E+04, .1260746826172E+04},
 {    7910, .8372193359375E+04, -.1238605346680E+04, .1259981933594E+04},
 {    7911, .8373028320312E+04, -.1332580444336E+04, .1272654785156E+04},
 {    7912, .8358991210938E+04, -.1331286743164E+04, .1273528686523E+04},
 {    7913, .8345434570312E+04, -.1327417358398E+04, .1274371337891E+04},
 {    7914, .8332955078125E+04, -.1320859741211E+04, .1275147705078E+04},
 {    7915, .8322397460938E+04, -.1311500854492E+04, .1275804809570E+04},
 {    7916, .8315026367188E+04, -.1299463867188E+04, .1276262695312E+04},
 {    7917, .8312303710938E+04, -.1285607055664E+04, .1276433105469E+04},
 {    7918, .8315015625000E+04, -.1271747924805E+04, .1276263671875E+04},
 {    7919, .8322376953125E+04, -.1259706054688E+04, .1275805664062E+04},
 {    7920, .8332929687500E+04, -.1250340942383E+04, .1275149414062E+04},
 {    7921, .8345407226562E+04, -.1243777587891E+04, .1274373291016E+04},
 {    7922, .8358962890625E+04, -.1239902465820E+04, .1273529785156E+04},
 {    7923, .8372999023438E+04, -.1238604980469E+04, .1272656738281E+04},
 {    7924, .8373847656250E+04, -.1332581176758E+04, .1285328491211E+04},
 {    7925, .8356538085938E+04, -.1331554687500E+04, .1286405273438E+04},
 {    7926, .8339485351562E+04, -.1328408325195E+04, .1287466186523E+04},
 {    7927, .8323044921875E+04, -.1322880615234E+04, .1288488769531E+04},
 {    7928, .8307890625000E+04, -.1314432373047E+04, .1289432373047E+04},
 {    7929, .8295644531250E+04, -.1302133300781E+04, .1290193847656E+04},
 {    7930, .8290279296875E+04, -.1285611572266E+04, .1290527832031E+04},
 {    7931, .8295621093750E+04, -.1269082763672E+04, .1290195312500E+04},
 {    7932, .8307859375000E+04, -.1256774780273E+04, .1289433593750E+04},
 {    7933, .8323009765625E+04, -.1248318481445E+04, .1288491455078E+04},
 {    7934, .8339448242188E+04, -.1242785400391E+04, .1287468505859E+04},
 {    7935, .8356500000000E+04, -.1239634277344E+04, .1286407226562E+04},
 {    7936, .8373810546875E+04, -.1238604370117E+04, .1285330444336E+04},
 {    7937, .8374594726562E+04, -.1336390625000E+04, .1298005615234E+04},
 {    7938, .8356506835938E+04, -.1335241455078E+04, .1299132080078E+04},
 {    7939, .8338721679688E+04, -.1331740478516E+04, .1300237792969E+04},
 {    7940, .8321659179688E+04, -.1325622070312E+04, .1301300537109E+04},
 {    7941, .8306091796875E+04, -.1316313232422E+04, .1302268432617E+04},
 {    7942, .8293757812500E+04, -.1303009887695E+04, .1303036132812E+04},
 {    7943, .8288531250000E+04, -.1285621337891E+04, .1303360961914E+04},
 {    7944, .8293726562500E+04, -.1268223144531E+04, .1303038085938E+04},
 {    7945, .8306046875000E+04, -.1254906494141E+04, .1302271728516E+04},
 {    7946, .8321606445312E+04, -.1245586914062E+04, .1301303466797E+04},
 {    7947, .8338665039062E+04, -.1239458862305E+04, .1300241455078E+04},
 {    7948, .8356449218750E+04, -.1235950439453E+04, .1299135253906E+04},
 {    7949, .8374537109375E+04, -.1234795654297E+04, .1298009277344E+04},
 {    7950, .8376098632812E+04, -.1342739868164E+04, .1323361572266E+04},
 {    7951, .8356219726562E+04, -.1341411499023E+04, .1324598632812E+04},
 {    7952, .8336704101562E+04, -.1337390014648E+04, .1325812866211E+04},
 {    7953, .8318045898438E+04, -.1330377441406E+04, .1326973754883E+04},
 {    7954, .8301160156250E+04, -.1319784545898E+04, .1328023925781E+04},
 {    7955, .8287976562500E+04, -.1304815185547E+04, .1328844604492E+04},
 {    7956, .8282505859375E+04, -.1285619995117E+04, .1329185180664E+04},
 {    7957, .8287950195312E+04, -.1266416870117E+04, .1328846435547E+04},
 {    7958, .8301117187500E+04, -.1251434082031E+04, .1328027832031E+04},
 {    7959, .8317998046875E+04, -.1240830810547E+04, .1326977172852E+04},
 {    7960, .8336650390625E+04, -.1233808959961E+04, .1325816162109E+04},
 {    7961, .8356164062500E+04, -.1229780151367E+04, .1324602050781E+04},
 {    7962, .8376044921875E+04, -.1228446289062E+04, .1323365234375E+04},
 {    7963, .8381285156250E+04, -.1239242187500E+04, .1221243041992E+04},
 {    7964, .8391553710938E+04, -.1244717529297E+04, .1220604003906E+04},
 {    7965, .8399831054688E+04, -.1252909912109E+04, .1220088500977E+04},
 {    7966, .8405796875000E+04, -.1262915771484E+04, .1219718261719E+04},
 {    7967, .8409368164062E+04, -.1274009399414E+04, .1219495361328E+04},
 {    7968, .8410552734375E+04, -.1285604614258E+04, .1219421508789E+04},
 {    7969, .8409364257812E+04, -.1297199462891E+04, .1219495239258E+04},
 {    7970, .8405786132812E+04, -.1308291137695E+04, .1219718627930E+04},
 {    7971, .8399815429688E+04, -.1318294067383E+04, .1220090454102E+04},
 {    7972, .8391534179688E+04, -.1326481933594E+04, .1220604980469E+04},
 {    7973, .8381261718750E+04, -.1331950439453E+04, .1221244750977E+04},
 {    7974, .8369783203125E+04, -.1333847656250E+04, .1221957885742E+04},
 {    7975, .8382723632812E+04, -.1240413940430E+04, .1246602783203E+04},
 {    7976, .8392921875000E+04, -.1245643676758E+04, .1245968139648E+04},
 {    7977, .8401229492188E+04, -.1253549560547E+04, .1245450927734E+04},
 {    7978, .8407273437500E+04, -.1263302124023E+04, .1245074951172E+04},
 {    7979, .8410916015625E+04, -.1274185791016E+04, .1244848266602E+04},
 {    7980, .8412132812500E+04, -.1285600830078E+04, .1244772827148E+04},
 {    7981, .8410913085938E+04, -.1297015502930E+04, .1244849121094E+04},
 {    7982, .8407265625000E+04, -.1307897827148E+04, .1245075683594E+04},
 {    7983, .8401217773438E+04, -.1317648315430E+04, .1245451416016E+04},
 {    7984, .8392907226562E+04, -.1325550537109E+04, .1245969360352E+04},
 {    7985, .8382707031250E+04, -.1330775390625E+04, .1246603027344E+04},
 {    7986, .8371389648438E+04, -.1332578369141E+04, .1247307495117E+04},
 {    7987, .8383507812500E+04, -.1240412963867E+04, .1259278320312E+04},
 {    7988, .8393708984375E+04, -.1245641967773E+04, .1258643554688E+04},
 {    7989, .8402017578125E+04, -.1253547363281E+04, .1258126220703E+04},
 {    7990, .8408061523438E+04, -.1263299926758E+04, .1257750122070E+04},
 {    7991, .8411705078125E+04, -.1274183715820E+04, .1257523559570E+04},
 {    7992, .8412920898438E+04, -.1285598999023E+04, .1257448364258E+04},
 {    7993, .8411701171875E+04, -.1297013916016E+04, .1257523925781E+04},
 {    7994, .8408055664062E+04, -.1307896728516E+04, .1257750488281E+04},
 {    7995, .8402008789062E+04, -.1317647583008E+04, .1258126586914E+04},
 {    7996, .8393697265625E+04, -.1325550170898E+04, .1258643798828E+04},
 {    7997, .8383495117188E+04, -.1330775268555E+04, .1259279174805E+04},
 {    7998, .8372179687500E+04, -.1332578369141E+04, .1259982666016E+04},
 {    7999, .8384311523438E+04, -.1240417236328E+04, .1271952392578E+04},
 {    8000, .8394508789062E+04, -.1245649414062E+04, .1271317626953E+04},
 {    8001, .8402810546875E+04, -.1253556518555E+04, .1270802368164E+04},
 {    8002, .8408853515625E+04, -.1263309326172E+04, .1270425537109E+04},
 {    8003, .8412496093750E+04, -.1274192504883E+04, .1270198486328E+04},
 {    8004, .8413708984375E+04, -.1285606811523E+04, .1270123535156E+04},
 {    8005, .8412489257812E+04, -.1297020263672E+04, .1270199707031E+04},
 {    8006, .8408840820312E+04, -.1307901367188E+04, .1270426025391E+04},
 {    8007, .8402791992188E+04, -.1317650634766E+04, .1270802490234E+04},
 {    8008, .8394484375000E+04, -.1325551635742E+04, .1271319702148E+04},
 {    8009, .8384284179688E+04, -.1330775634766E+04, .1271954345703E+04},
 {    8010, .8372968750000E+04, -.1332578369141E+04, .1272658325195E+04},
 {    8011, .8385632812500E+04, -.1240271118164E+04, .1284594970703E+04},
 {    8012, .8396529296875E+04, -.1245157714844E+04, .1283916992188E+04},
 {    8013, .8405708007812E+04, -.1252801025391E+04, .1283346435547E+04},
 {    8014, .8412600585938E+04, -.1262565307617E+04, .1282917236328E+04},
 {    8015, .8416859375000E+04, -.1273738647461E+04, .1282652099609E+04},
 {    8016, .8418301757812E+04, -.1285611328125E+04, .1282561767578E+04},
 {    8017, .8416849609375E+04, -.1297482788086E+04, .1282652099609E+04},
 {    8018, .8412583007812E+04, -.1308652832031E+04, .1282917968750E+04},
 {    8019, .8405681640625E+04, -.1318411132812E+04, .1283347900391E+04},
 {    8020, .8396496093750E+04, -.1326047119141E+04, .1283918945312E+04},
 {    8021, .8385594726562E+04, -.1330923461914E+04, .1284597900391E+04},
 {    8022, .8373770507812E+04, -.1332578979492E+04, .1285332885742E+04},
 {    8023, .8387013671875E+04, -.1236673828125E+04, .1297234375000E+04},
 {    8024, .8398379882812E+04, -.1242159912109E+04, .1296526855469E+04},
 {    8025, .8407785156250E+04, -.1250585815430E+04, .1295942260742E+04},
 {    8026, .8414717773438E+04, -.1261147583008E+04, .1295510742188E+04},
 {    8027, .8418942382812E+04, -.1273059326172E+04, .1295247314453E+04},
 {    8028, .8420356445312E+04, -.1285620971680E+04, .1295158691406E+04},
 {    8029, .8418928710938E+04, -.1298181030273E+04, .1295247802734E+04},
 {    8030, .8414692382812E+04, -.1310088378906E+04, .1295511718750E+04},
 {    8031, .8407748046875E+04, -.1320642700195E+04, .1295943847656E+04},
 {    8032, .8398331054688E+04, -.1329056396484E+04, .1296529785156E+04},
 {    8033, .8386957031250E+04, -.1334526733398E+04, .1297237060547E+04},
 {    8034, .8374478515625E+04, -.1336387207031E+04, .1298013671875E+04},
 {    8035, .8391379882812E+04, -.1230229614258E+04, .1322410644531E+04},
 {    8036, .8405838867188E+04, -.1235638793945E+04, .1321512207031E+04},
 {    8037, .8418527343750E+04, -.1244447387695E+04, .1320722167969E+04},
 {    8038, .8428501953125E+04, -.1256250244141E+04, .1320101318359E+04},
 {    8039, .8434923828125E+04, -.1270314575195E+04, .1319701904297E+04},
 {    8040, .8437143554688E+04, -.1285619506836E+04, .1319563964844E+04},
 {    8041, .8434908203125E+04, -.1300922241211E+04, .1319702636719E+04},
 {    8042, .8428472656250E+04, -.1314979980469E+04, .1320102783203E+04},
 {    8043, .8418486328125E+04, -.1326772827148E+04, .1320724975586E+04},
 {    8044, .8405791015625E+04, -.1335570678711E+04, .1321514648438E+04},
 {    8045, .8391326171875E+04, -.1340967773438E+04, .1322413574219E+04},
 {    8046, .8369760742188E+04, -.1338928466797E+04, .1221959838867E+04},
 {    8047, .8354990234375E+04, -.1337280883789E+04, .1222878540039E+04},
 {    8048, .8340932617188E+04, -.1332447021484E+04, .1223753417969E+04},
 {    8049, .8328372070312E+04, -.1324487548828E+04, .1224535034180E+04},
 {    8050, .8318222656250E+04, -.1313607788086E+04, .1225166503906E+04},
 {    8051, .8311516601562E+04, -.1300319824219E+04, .1225583740234E+04},
 {    8052, .8309150390625E+04, -.1285618652344E+04, .1225730468750E+04},
 {    8053, .8311500000000E+04, -.1270914794922E+04, .1225584716797E+04},
 {    8054, .8318193359375E+04, -.1257620117188E+04, .1225167968750E+04},
 {    8055, .8328331054688E+04, -.1246729980469E+04, .1224537353516E+04},
 {    8056, .8340884765625E+04, -.1238760131836E+04, .1223756103516E+04},
 {    8057, .8354938476562E+04, -.1233915283203E+04, .1222881835938E+04},
 {    8058, .8369707031250E+04, -.1232258422852E+04, .1221962890625E+04},
 {    8059, .8371369140625E+04, -.1336388793945E+04, .1247308837891E+04},
 {    8060, .8357596679688E+04, -.1334765625000E+04, .1248165283203E+04},
 {    8061, .8344558593750E+04, -.1330028076172E+04, .1248977661133E+04},
 {    8062, .8333030273438E+04, -.1322306640625E+04, .1249694091797E+04},
 {    8063, .8323858398438E+04, -.1311884643555E+04, .1250264892578E+04},
 {    8064, .8317884765625E+04, -.1299345214844E+04, .1250636474609E+04},
 {    8065, .8315798828125E+04, -.1285608886719E+04, .1250765747070E+04},
 {    8066, .8317875000000E+04, -.1271871093750E+04, .1250636718750E+04},
 {    8067, .8323839843750E+04, -.1259327392578E+04, .1250266113281E+04},
 {    8068, .8333003906250E+04, -.1248899169922E+04, .1249696289062E+04},
 {    8069, .8344529296875E+04, -.1241171142578E+04, .1248979125977E+04},
 {    8070, .8357563476562E+04, -.1236426391602E+04, .1248168090820E+04},
 {    8071, .8371333984375E+04, -.1234796997070E+04, .1247311157227E+04},
 {    8072, .8372148437500E+04, -.1336388549805E+04, .1259984619141E+04},
 {    8073, .8358724609375E+04, -.1334690063477E+04, .1260820312500E+04},
 {    8074, .8346112304688E+04, -.1329782836914E+04, .1261604614258E+04},
 {    8075, .8335118164062E+04, -.1321884887695E+04, .1262289306641E+04},
 {    8076, .8326520507812E+04, -.1311415405273E+04, .1262823242188E+04},
 {    8077, .8321024414062E+04, -.1299027954102E+04, .1263165527344E+04},
 {    8078, .8319122070312E+04, -.1285606445312E+04, .1263284301758E+04},
 {    8079, .8321016601562E+04, -.1272183959961E+04, .1263166015625E+04},
 {    8080, .8326504882812E+04, -.1259793090820E+04, .1262824707031E+04},
 {    8081, .8335097656250E+04, -.1249318969727E+04, .1262289794922E+04},
 {    8082, .8346086914062E+04, -.1241414672852E+04, .1261606445312E+04},
 {    8083, .8358695312500E+04, -.1236501098633E+04, .1260822021484E+04},
 {    8084, .8372119140625E+04, -.1234797119141E+04, .1259986083984E+04},
 {    8085, .8372984375000E+04, -.1336389892578E+04, .1272657470703E+04},
 {    8086, .8357440429688E+04, -.1335029296875E+04, .1273624267578E+04},
 {    8087, .8342376953125E+04, -.1330961914062E+04, .1274561767578E+04},
 {    8088, .8328397460938E+04, -.1324022338867E+04, .1275431396484E+04},
 {    8089, .8316416015625E+04, -.1314013427734E+04, .1276177124023E+04},
 {    8090, .8307895507812E+04, -.1300918090820E+04, .1276707519531E+04},
 {    8091, .8304700195312E+04, -.1285618408203E+04, .1276906005859E+04},
 {    8092, .8307875000000E+04, -.1270314453125E+04, .1276708251953E+04},
 {    8093, .8316379882812E+04, -.1257209472656E+04, .1276179199219E+04},
 {    8094, .8328353515625E+04, -.1247189941406E+04, .1275434326172E+04},
 {    8095, .8342329101562E+04, -.1240240600586E+04, .1274564941406E+04},
 {    8096, .8357390625000E+04, -.1236163574219E+04, .1273627563477E+04},
 {    8097, .8372933593750E+04, -.1234796630859E+04, .1272661132812E+04},
 {    8098, .8373818359375E+04, -.1336390991211E+04, .1285330078125E+04},
 {    8099, .8354248046875E+04, -.1335316894531E+04, .1286547851562E+04},
 {    8100, .8334921875000E+04, -.1332037841797E+04, .1287749755859E+04},
 {    8101, .8316192382812E+04, -.1326245361328E+04, .1288915771484E+04},
 {    8102, .8298728515625E+04, -.1317327270508E+04, .1290001953125E+04},
 {    8103, .8284239257812E+04, -.1304102783203E+04, .1290904052734E+04},
 {    8104, .8277602539062E+04, -.1285623657227E+04, .1291316406250E+04},
 {    8105, .8284203125000E+04, -.1267131103516E+04, .1290906005859E+04},
 {    8106, .8298677734375E+04, -.1253890136719E+04, .1290005493164E+04},
 {    8107, .8316136718750E+04, -.1244961303711E+04, .1288919189453E+04},
 {    8108, .8334862304688E+04, -.1239159545898E+04, .1287753906250E+04},
 {    8109, .8354186523438E+04, -.1235873413086E+04, .1286552001953E+04},
 {    8110, .8373756835938E+04, -.1234795410156E+04, .1285333740234E+04},
 {    8111, .8374553710938E+04, -.1341470336914E+04, .1298008178711E+04},
 {    8112, .8353886718750E+04, -.1340248535156E+04, .1299294555664E+04},
 {    8113, .8333523437500E+04, -.1336518066406E+04, .1300561645508E+04},
 {    8114, .8313882812500E+04, -.1329965820312E+04, .1301784545898E+04},
 {    8115, .8295754882812E+04, -.1319949951172E+04, .1302911621094E+04},
 {    8116, .8281062500000E+04, -.1305337890625E+04, .1303825195312E+04},
 {    8117, .8274591796875E+04, -.1285637084961E+04, .1304227783203E+04},
 {    8118, .8281012695312E+04, -.1265919555664E+04, .1303828125000E+04},
 {    8119, .8295681640625E+04, -.1251286743164E+04, .1302915771484E+04},
 {    8120, .8313800781250E+04, -.1241252441406E+04, .1301788574219E+04},
 {    8121, .8333438476562E+04, -.1234688110352E+04, .1300567138672E+04},
 {    8122, .8353798828125E+04, -.1230946777344E+04, .1299300048828E+04},
 {    8123, .8374465820312E+04, -.1229717163086E+04, .1298013427734E+04},
 {    8124, .8376004882812E+04, -.1350358764648E+04, .1323367675781E+04},
 {    8125, .8354192382812E+04, -.1348795532227E+04, .1324725097656E+04},
 {    8126, .8332834960938E+04, -.1344100219727E+04, .1326053222656E+04},
 {    8127, .8312540039062E+04, -.1335942138672E+04, .1327316406250E+04},
 {    8128, .8294367187500E+04, -.1323751098633E+04, .1328447021484E+04},
 {    8129, .8280506835938E+04, -.1306805297852E+04, .1329309692383E+04},
 {    8130, .8274899414062E+04, -.1285627929688E+04, .1329657958984E+04},
 {    8131, .8280472656250E+04, -.1264441650391E+04, .1329311523438E+04},
 {    8132, .8294314453125E+04, -.1247479003906E+04, .1328450927734E+04},
 {    8133, .8312475585938E+04, -.1235274658203E+04, .1327320556641E+04},
 {    8134, .8332765625000E+04, -.1227104248047E+04, .1326058105469E+04},
 {    8135, .8354122070312E+04, -.1222398925781E+04, .1324729003906E+04},
 {    8136, .8375934570312E+04, -.1220828125000E+04, .1323371826172E+04},
 {    8137, .8382005859375E+04, -.1234506713867E+04, .1221198242188E+04},
 {    8138, .8392774414062E+04, -.1240867187500E+04, .1220527832031E+04},
 {    8139, .8401218750000E+04, -.1250104492188E+04, .1220002197266E+04},
 {    8140, .8407154296875E+04, -.1261125732422E+04, .1219632934570E+04},
 {    8141, .8410662109375E+04, -.1273147705078E+04, .1219414672852E+04},
 {    8142, .8411821289062E+04, -.1285618652344E+04, .1219342773438E+04},
 {    8143, .8410652343750E+04, -.1298088745117E+04, .1219415405273E+04},
 {    8144, .8407133789062E+04, -.1310107666016E+04, .1219634277344E+04},
 {    8145, .8401189453125E+04, -.1321123413086E+04, .1220003906250E+04},
 {    8146, .8392733398438E+04, -.1330350708008E+04, .1220530517578E+04},
 {    8147, .8381958007812E+04, -.1336696533203E+04, .1221200927734E+04},
 {    8148, .8369654296875E+04, -.1338924560547E+04, .1221966674805E+04},
 {    8149, .8383307617188E+04, -.1236837402344E+04, .1246565917969E+04},
 {    8150, .8393956054688E+04, -.1242686157227E+04, .1245903442383E+04},
 {    8151, .8402471679688E+04, -.1251362548828E+04, .1245373413086E+04},
 {    8152, .8408564453125E+04, -.1261889160156E+04, .1244994628906E+04},
 {    8153, .8412194335938E+04, -.1273500976562E+04, .1244769287109E+04},
 {    8154, .8413399414062E+04, -.1285609008789E+04, .1244693115234E+04},
 {    8155, .8412187500000E+04, -.1297716430664E+04, .1244769897461E+04},
 {    8156, .8408550781250E+04, -.1309326049805E+04, .1244995361328E+04},
 {    8157, .8402451171875E+04, -.1319848266602E+04, .1245374755859E+04},
 {    8158, .8393932617188E+04, -.1328518798828E+04, .1245904785156E+04},
 {    8159, .8383276367188E+04, -.1334358398438E+04, .1246567626953E+04},
 {    8160, .8371302734375E+04, -.1336386352539E+04, .1247313476562E+04},
 {    8161, .8384092773438E+04, -.1236835815430E+04, .1259241455078E+04},
 {    8162, .8394744140625E+04, -.1242683593750E+04, .1258578735352E+04},
 {    8163, .8403258789062E+04, -.1251359619141E+04, .1258049560547E+04},
 {    8164, .8409353515625E+04, -.1261886108398E+04, .1257669921875E+04},
 {    8165, .8412982421875E+04, -.1273498168945E+04, .1257444702148E+04},
 {    8166, .8414189453125E+04, -.1285606567383E+04, .1257369873047E+04},
 {    8167, .8412978515625E+04, -.1297714477539E+04, .1257444091797E+04},
 {    8168, .8409341796875E+04, -.1309324584961E+04, .1257670654297E+04},
 {    8169, .8403242187500E+04, -.1319847412109E+04, .1258050537109E+04},
 {    8170, .8394720703125E+04, -.1328518310547E+04, .1258580322266E+04},
 {    8171, .8384064453125E+04, -.1334358276367E+04, .1259243652344E+04},
 {    8172, .8372089843750E+04, -.1336386352539E+04, .1259988525391E+04},
 {    8173, .8384900390625E+04, -.1236843383789E+04, .1271916259766E+04},
 {    8174, .8395548828125E+04, -.1242695800781E+04, .1271253417969E+04},
 {    8175, .8404060546875E+04, -.1251373779297E+04, .1270722900391E+04},
 {    8176, .8410145507812E+04, -.1261900756836E+04, .1270345214844E+04},
 {    8177, .8413775390625E+04, -.1273511718750E+04, .1270118896484E+04},
 {    8178, .8414975585938E+04, -.1285618408203E+04, .1270044433594E+04},
 {    8179, .8413765625000E+04, -.1297724121094E+04, .1270119873047E+04},
 {    8180, .8410125000000E+04, -.1309331909180E+04, .1270346435547E+04},
 {    8181, .8404027343750E+04, -.1319852050781E+04, .1270726196289E+04},
 {    8182, .8395506835938E+04, -.1328520874023E+04, .1271255859375E+04},
 {    8183, .8384851562500E+04, -.1334359008789E+04, .1271918945312E+04},
 {    8184, .8372878906250E+04, -.1336386352539E+04, .1272663696289E+04},
 {    8185, .8386403320312E+04, -.1236626831055E+04, .1284546997070E+04},
 {    8186, .8398001953125E+04, -.1241999145508E+04, .1283825073242E+04},
 {    8187, .8407699218750E+04, -.1250334838867E+04, .1283221923828E+04},
 {    8188, .8414915039062E+04, -.1260900512695E+04, .1282772949219E+04},
 {    8189, .8419343750000E+04, -.1272908325195E+04, .1282497314453E+04},
 {    8190, .8420833984375E+04, -.1285622802734E+04, .1282405029297E+04},
 {    8191, .8419330078125E+04, -.1298335693359E+04, .1282498046875E+04},
 {    8192, .8414886718750E+04, -.1310338500977E+04, .1282775146484E+04},
 {    8193, .8407658203125E+04, -.1320895507812E+04, .1283224365234E+04},
 {    8194, .8397951171875E+04, -.1329218505859E+04, .1283828247070E+04},
 {    8195, .8386343750000E+04, -.1334574462891E+04, .1284550781250E+04},
 {    8196, .8373695312500E+04, -.1336387451172E+04, .1285337890625E+04},
 {    8197, .8388100585938E+04, -.1231800781250E+04, .1297165771484E+04},
 {    8198, .8400475585938E+04, -.1237901123047E+04, .1296396484375E+04},
 {    8199, .8410656250000E+04, -.1247225341797E+04, .1295762939453E+04},
 {    8200, .8418120117188E+04, -.1258845825195E+04, .1295298339844E+04},
 {    8201, .8422649414062E+04, -.1271899536133E+04, .1295016235352E+04},
 {    8202, .8424158203125E+04, -.1285636108398E+04, .1294922973633E+04},
 {    8203, .8422628906250E+04, -.1299370483398E+04, .1295017700195E+04},
 {    8204, .8418081054688E+04, -.1312417602539E+04, .1295300537109E+04},
 {    8205, .8410601562500E+04, -.1324026977539E+04, .1295766357422E+04},
 {    8206, .8400402343750E+04, -.1333332641602E+04, .1296400512695E+04},
 {    8207, .8388015625000E+04, -.1339408935547E+04, .1297171508789E+04},
 {    8208, .8374374023438E+04, -.1341465087891E+04, .1298019775391E+04},
 {    8209, .8393257812500E+04, -.1222846801758E+04, .1322294067383E+04},
 {    8210, .8409581054688E+04, -.1229003662109E+04, .1321278564453E+04},
 {    8211, .8423870117188E+04, -.1239020385742E+04, .1320389648438E+04},
 {    8212, .8435078125000E+04, -.1252411987305E+04, .1319692138672E+04},
 {    8213, .8442263671875E+04, -.1268331665039E+04, .1319245605469E+04},
 {    8214, .8444748046875E+04, -.1285628051758E+04, .1319090576172E+04},
 {    8215, .8442245117188E+04, -.1302921508789E+04, .1319246337891E+04},
 {    8216, .8435038085938E+04, -.1318832519531E+04, .1319695068359E+04},
 {    8217, .8423816406250E+04, -.1332211914062E+04, .1320392333984E+04},
 {    8218, .8409516601562E+04, -.1342213745117E+04, .1321282592773E+04},
 {    8219, .8393189453125E+04, -.1348354858398E+04, .1322298828125E+04},
 {    8220, .8442519531250E+04, -.1239348510742E+04, .1048829833984E+04},
 {    8221, .8439310546875E+04, -.1218246704102E+04, .1050265625000E+04},
 {    8222, .8432572265625E+04, -.1198169311523E+04, .1053283203125E+04},
 {    8223, .8422535156250E+04, -.1179819580078E+04, .1057776367188E+04},
 {    8224, .8409551757812E+04, -.1163840087891E+04, .1063587768555E+04},
 {    8225, .8394079101562E+04, -.1150791015625E+04, .1070513916016E+04},
 {    8226, .8376659179688E+04, -.1141129516602E+04, .1078313110352E+04},
 {    8227, .8357899414062E+04, -.1135193969727E+04, .1086710327148E+04},
 {    8228, .8338461914062E+04, -.1133192016602E+04, .1095413208008E+04},
 {    8230, .8442716796875E+04, -.1217321411133E+04, .1048741699219E+04},
 {    8231, .8436559570312E+04, -.1195620605469E+04, .1051496948242E+04},
 {    8232, .8426675781250E+04, -.1175634399414E+04, .1055922485352E+04},
 {    8233, .8413436523438E+04, -.1158124877930E+04, .1061849243164E+04},
 {    8234, .8397342773438E+04, -.1143760498047E+04, .1069054321289E+04},
 {    8235, .8379006835938E+04, -.1133089965820E+04, .1077262207031E+04},
 {    8236, .8359127929688E+04, -.1126520141602E+04, .1086161254883E+04},
 {    8237, .8338461914062E+04, -.1124302001953E+04, .1095413208008E+04},
 {    8239, .8446118164062E+04, -.1216396118164E+04, .1047217773438E+04},
 {    8240, .8440551757812E+04, -.1193072021484E+04, .1049710693359E+04},
 {    8241, .8430818359375E+04, -.1171449340820E+04, .1054067626953E+04},
 {    8242, .8417319335938E+04, -.1152409667969E+04, .1060111083984E+04},
 {    8243, .8400602539062E+04, -.1136730102539E+04, .1067594238281E+04},
 {    8244, .8381351562500E+04, -.1125050415039E+04, .1076211669922E+04},
 {    8245, .8360352539062E+04, -.1117846435547E+04, .1085612426758E+04},
 {    8246, .8338461914062E+04, -.1115411987305E+04, .1095413208008E+04},
 {    8247, .8437331054688E+04, -.1239348510742E+04, .1037238403320E+04},
 {    8248, .8434123046875E+04, -.1218246704102E+04, .1038673706055E+04},
 {    8249, .8427382812500E+04, -.1198169311523E+04, .1041691284180E+04},
 {    8250, .8417343750000E+04, -.1179819580078E+04, .1046184570312E+04},
 {    8251, .8404362304688E+04, -.1163840087891E+04, .1051996582031E+04},
 {    8252, .8388892578125E+04, -.1150791015625E+04, .1058922607422E+04},
 {    8253, .8371469726562E+04, -.1141129516602E+04, .1066721191406E+04},
 {    8254, .8352710937500E+04, -.1135193969727E+04, .1075119140625E+04},
 {    8255, .8333272460938E+04, -.1133192016602E+04, .1083821044922E+04},
 {    8257, .8437526367188E+04, -.1217321411133E+04, .1037150024414E+04},
 {    8258, .8431372070312E+04, -.1195620605469E+04, .1039905761719E+04},
 {    8259, .8421487304688E+04, -.1175634399414E+04, .1044330200195E+04},
 {    8260, .8408247070312E+04, -.1158124877930E+04, .1050258056641E+04},
 {    8261, .8392152343750E+04, -.1143760498047E+04, .1057462890625E+04},
 {    8262, .8373817382812E+04, -.1133089965820E+04, .1065670654297E+04},
 {    8263, .8353939453125E+04, -.1126520141602E+04, .1074569580078E+04},
 {    8264, .8333272460938E+04, -.1124302001953E+04, .1083821044922E+04},
 {    8266, .8440931640625E+04, -.1216396118164E+04, .1035626220703E+04},
 {    8267, .8435361328125E+04, -.1193072021484E+04, .1038119262695E+04},
 {    8268, .8425627929688E+04, -.1171449340820E+04, .1042476196289E+04},
 {    8269, .8412127929688E+04, -.1152409667969E+04, .1048519775391E+04},
 {    8270, .8395413085938E+04, -.1136730102539E+04, .1056002685547E+04},
 {    8271, .8376161132812E+04, -.1125050415039E+04, .1064620727539E+04},
 {    8272, .8355163085938E+04, -.1117846435547E+04, .1074020996094E+04},
 {    8273, .8333272460938E+04, -.1115411987305E+04, .1083821044922E+04},
 {    8274, .8431495117188E+04, -.1239348510742E+04, .1024197265625E+04},
 {    8275, .8428287109375E+04, -.1218246704102E+04, .1025633300781E+04},
 {    8276, .8421544921875E+04, -.1198169311523E+04, .1028651123047E+04},
 {    8277, .8411506835938E+04, -.1179819580078E+04, .1033143920898E+04},
 {    8278, .8398525390625E+04, -.1163840087891E+04, .1038955810547E+04},
 {    8279, .8383054687500E+04, -.1150791015625E+04, .1045882324219E+04},
 {    8280, .8365632812500E+04, -.1141129516602E+04, .1053680908203E+04},
 {    8281, .8346875000000E+04, -.1135193969727E+04, .1062078247070E+04},
 {    8282, .8327435546875E+04, -.1133192016602E+04, .1070780395508E+04},
 {    8284, .8431688476562E+04, -.1217321411133E+04, .1024109863281E+04},
 {    8285, .8425534179688E+04, -.1195620605469E+04, .1026864990234E+04},
 {    8286, .8415651367188E+04, -.1175634399414E+04, .1031290039062E+04},
 {    8287, .8402410156250E+04, -.1158124877930E+04, .1037217529297E+04},
 {    8288, .8386314453125E+04, -.1143760498047E+04, .1044422363281E+04},
 {    8289, .8367979492188E+04, -.1133089965820E+04, .1052630371094E+04},
 {    8290, .8348099609375E+04, -.1126520141602E+04, .1061529663086E+04},
 {    8291, .8327435546875E+04, -.1124302001953E+04, .1070780395508E+04},
 {    8293, .8435093750000E+04, -.1216396118164E+04, .1022585693359E+04},
 {    8294, .8429522460938E+04, -.1193072021484E+04, .1025079345703E+04},
 {    8295, .8419791015625E+04, -.1171449340820E+04, .1029435791016E+04},
 {    8296, .8406290039062E+04, -.1152409667969E+04, .1035479370117E+04},
 {    8297, .8389577148438E+04, -.1136730102539E+04, .1042962158203E+04},
 {    8298, .8370324218750E+04, -.1125050415039E+04, .1051579833984E+04},
 {    8299, .8349326171875E+04, -.1117846435547E+04, .1060980712891E+04},
 {    8300, .8327435546875E+04, -.1115411987305E+04, .1070780395508E+04},
 {    8301, .8425655273438E+04, -.1239348510742E+04, .1011156982422E+04},
 {    8302, .8422446289062E+04, -.1218246704102E+04, .1012593200684E+04},
 {    8303, .8415706054688E+04, -.1198169311523E+04, .1015610839844E+04},
 {    8304, .8405670898438E+04, -.1179819580078E+04, .1020103881836E+04},
 {    8305, .8392688476562E+04, -.1163840087891E+04, .1025915039062E+04},
 {    8306, .8377215820312E+04, -.1150791015625E+04, .1032841796875E+04},
 {    8307, .8359793945312E+04, -.1141129516602E+04, .1040640380859E+04},
 {    8308, .8341037109375E+04, -.1135193969727E+04, .1049037719727E+04},
 {    8309, .8321595703125E+04, -.1133192016602E+04, .1057740356445E+04},
 {    8311, .8425851562500E+04, -.1217321411133E+04, .1011069213867E+04},
 {    8312, .8419696289062E+04, -.1195620605469E+04, .1013824218750E+04},
 {    8313, .8409812500000E+04, -.1175634399414E+04, .1018249267578E+04},
 {    8314, .8396570312500E+04, -.1158124877930E+04, .1024176391602E+04},
 {    8315, .8380475585938E+04, -.1143760498047E+04, .1031381835938E+04},
 {    8316, .8362143554688E+04, -.1133089965820E+04, .1039589355469E+04},
 {    8317, .8342260742188E+04, -.1126520141602E+04, .1048489501953E+04},
 {    8318, .8321595703125E+04, -.1124302001953E+04, .1057740356445E+04},
 {    8320, .8429255859375E+04, -.1216396118164E+04, .1009545288086E+04},
 {    8321, .8423685546875E+04, -.1193072021484E+04, .1012038696289E+04},
 {    8322, .8413953125000E+04, -.1171449340820E+04, .1016395202637E+04},
 {    8323, .8400454101562E+04, -.1152409667969E+04, .1022438354492E+04},
 {    8324, .8383739257812E+04, -.1136730102539E+04, .1029921508789E+04},
 {    8325, .8364488281250E+04, -.1125050415039E+04, .1038539428711E+04},
 {    8326, .8343487304688E+04, -.1117846435547E+04, .1047939819336E+04},
 {    8327, .8321595703125E+04, -.1115411987305E+04, .1057740356445E+04},
 {    8328, .8413980468750E+04, -.1239348510742E+04, .9850762329102E+03},
 {    8329, .8410772460938E+04, -.1218246704102E+04, .9865120849609E+03},
 {    8330, .8404031250000E+04, -.1198169311523E+04, .9895295410156E+03},
 {    8331, .8393994140625E+04, -.1179819580078E+04, .9940228271484E+03},
 {    8332, .8381010742188E+04, -.1163840087891E+04, .9998345947266E+03},
 {    8333, .8365541992188E+04, -.1150791015625E+04, .1006760986328E+04},
 {    8334, .8348119140625E+04, -.1141129516602E+04, .1014559631348E+04},
 {    8335, .8329360351562E+04, -.1135193969727E+04, .1022956909180E+04},
 {    8336, .8309919921875E+04, -.1133192016602E+04, .1031659179688E+04},
 {    8338, .8414174804688E+04, -.1217321411133E+04, .9849887084961E+03},
 {    8339, .8408020507812E+04, -.1195620605469E+04, .9877436523438E+03},
 {    8340, .8398135742188E+04, -.1175634399414E+04, .9921684570312E+03},
 {    8341, .8384894531250E+04, -.1158124877930E+04, .9980961914062E+03},
 {    8342, .8368799804688E+04, -.1143760498047E+04, .1005300781250E+04},
 {    8343, .8350466796875E+04, -.1133089965820E+04, .1013508911133E+04},
 {    8344, .8330586914062E+04, -.1126520141602E+04, .1022408325195E+04},
 {    8345, .8309919921875E+04, -.1124302001953E+04, .1031659179688E+04},
 {    8347, .8417580078125E+04, -.1216396118164E+04, .9834643554688E+03},
 {    8348, .8412010742188E+04, -.1193072021484E+04, .9859576416016E+03},
 {    8349, .8402276367188E+04, -.1171449340820E+04, .9903144531250E+03},
 {    8350, .8388777343750E+04, -.1152409667969E+04, .9963574218750E+03},
 {    8351, .8372062500000E+04, -.1136730102539E+04, .1003840515137E+04},
 {    8352, .8352810546875E+04, -.1125050415039E+04, .1012458618164E+04},
 {    8353, .8331811523438E+04, -.1117846435547E+04, .1021858886719E+04},
 {    8354, .8309919921875E+04, -.1115411987305E+04, .1031659179688E+04},
 {    8355, .8402303710938E+04, -.1239348510742E+04, .9589953613281E+03},
 {    8356, .8399095703125E+04, -.1218246704102E+04, .9604311523438E+03},
 {    8357, .8392356445312E+04, -.1198169311523E+04, .9634486083984E+03},
 {    8358, .8382320312500E+04, -.1179819580078E+04, .9679415893555E+03},
 {    8359, .8369336914062E+04, -.1163840087891E+04, .9737531738281E+03},
 {    8360, .8353864257812E+04, -.1150791015625E+04, .9806799316406E+03},
 {    8361, .8336441406250E+04, -.1141129516602E+04, .9884785766602E+03},
 {    8362, .8317685546875E+04, -.1135193969727E+04, .9968760375977E+03},
 {    8363, .8298244140625E+04, -.1133192016602E+04, .1005578613281E+04},
 {    8365, .8402500000000E+04, -.1217321411133E+04, .9589072265625E+03},
 {    8366, .8396344726562E+04, -.1195620605469E+04, .9616627197266E+03},
 {    8367, .8386460937500E+04, -.1175634521484E+04, .9660875244141E+03},
 {    8368, .8373218750000E+04, -.1158124877930E+04, .9720150146484E+03},
 {    8369, .8357126953125E+04, -.1143760498047E+04, .9792196655273E+03},
 {    8370, .8338791015625E+04, -.1133089965820E+04, .9874279785156E+03},
 {    8371, .8318911132812E+04, -.1126520141602E+04, .9963272705078E+03},
 {    8372, .8298244140625E+04, -.1124302001953E+04, .1005578613281E+04},
 {    8374, .8405904296875E+04, -.1216396118164E+04, .9573835449219E+03},
 {    8375, .8400335937500E+04, -.1193072021484E+04, .9598762207031E+03},
 {    8376, .8390602539062E+04, -.1171449340820E+04, .9642335815430E+03},
 {    8377, .8377102539062E+04, -.1152409667969E+04, .9702769775391E+03},
 {    8378, .8360386718750E+04, -.1136730102539E+04, .9777602539062E+03},
 {    8379, .8341136718750E+04, -.1125050415039E+04, .9863776855469E+03},
 {    8380, .8320136718750E+04, -.1117846435547E+04, .9957780761719E+03},
 {    8381, .8298244140625E+04, -.1115412109375E+04, .1005578613281E+04},
 {    8382, .8390627929688E+04, -.1239348510742E+04, .9329145507812E+03},
 {    8383, .8387419921875E+04, -.1218246704102E+04, .9343503417969E+03},
 {    8384, .8380680664062E+04, -.1198169311523E+04, .9373677978516E+03},
 {    8385, .8370646484375E+04, -.1179819580078E+04, .9418606567383E+03},
 {    8386, .8357660156250E+04, -.1163840087891E+04, .9476726684570E+03},
 {    8387, .8342189453125E+04, -.1150791015625E+04, .9545991821289E+03},
 {    8388, .8324768554688E+04, -.1141129516602E+04, .9623973388672E+03},
 {    8389, .8306009765625E+04, -.1135193969727E+04, .9707949829102E+03},
 {    8390, .8286571289062E+04, -.1133192016602E+04, .9794973144531E+03},
 {    8392, .8390825195312E+04, -.1217321411133E+04, .9328261718750E+03},
 {    8393, .8384670898438E+04, -.1195620605469E+04, .9355817871094E+03},
 {    8394, .8374784179688E+04, -.1175634521484E+04, .9400067138672E+03},
 {    8395, .8361543945312E+04, -.1158124877930E+04, .9459339599609E+03},
 {    8396, .8345449218750E+04, -.1143760498047E+04, .9531389770508E+03},
 {    8397, .8327114257812E+04, -.1133089965820E+04, .9613471679688E+03},
 {    8398, .8307237304688E+04, -.1126520141602E+04, .9702460327148E+03},
 {    8399, .8286571289062E+04, -.1124302001953E+04, .9794973144531E+03},
 {    8401, .8394229492188E+04, -.1216396118164E+04, .9313027343750E+03},
 {    8402, .8388660156250E+04, -.1193072021484E+04, .9337952880859E+03},
 {    8403, .8378925781250E+04, -.1171449340820E+04, .9381528320312E+03},
 {    8404, .8365427734375E+04, -.1152409667969E+04, .9441959838867E+03},
 {    8405, .8348711914062E+04, -.1136730102539E+04, .9516792602539E+03},
 {    8406, .8329461914062E+04, -.1125050415039E+04, .9602965087891E+03},
 {    8407, .8308460937500E+04, -.1117846435547E+04, .9696973266602E+03},
 {    8408, .8286571289062E+04, -.1115412109375E+04, .9794973144531E+03},
 {    8409, .8385440429688E+04, -.1239348510742E+04, .9213228759766E+03},
 {    8410, .8382232421875E+04, -.1218246704102E+04, .9227584228516E+03},
 {    8411, .8375492187500E+04, -.1198169311523E+04, .9257759399414E+03},
 {    8412, .8365455078125E+04, -.1179819580078E+04, .9302692871094E+03},
 {    8413, .8352471679688E+04, -.1163840087891E+04, .9360811767578E+03},
 {    8414, .8337000000000E+04, -.1150791015625E+04, .9430073852539E+03},
 {    8415, .8319578125000E+04, -.1141129516602E+04, .9508060913086E+03},
 {    8416, .8300821289062E+04, -.1135193969727E+04, .9592035522461E+03},
 {    8417, .8281380859375E+04, -.1133192016602E+04, .9679062500000E+03},
 {    8419, .8385636718750E+04, -.1217321411133E+04, .9212347412109E+03},
 {    8420, .8379480468750E+04, -.1195620727539E+04, .9239902954102E+03},
 {    8421, .8369596679688E+04, -.1175634521484E+04, .9284151000977E+03},
 {    8422, .8356356445312E+04, -.1158124877930E+04, .9343424682617E+03},
 {    8423, .8340259765625E+04, -.1143760498047E+04, .9415477294922E+03},
 {    8424, .8321925781250E+04, -.1133089965820E+04, .9497557373047E+03},
 {    8425, .8302046875000E+04, -.1126520141602E+04, .9586547851562E+03},
 {    8426, .8281380859375E+04, -.1124302001953E+04, .9679062500000E+03},
 {    8428, .8389039062500E+04, -.1216396118164E+04, .9197109375000E+03},
 {    8429, .8383468750000E+04, -.1193072021484E+04, .9222044677734E+03},
 {    8430, .8373737304688E+04, -.1171449340820E+04, .9265612792969E+03},
 {    8431, .8360238281250E+04, -.1152409667969E+04, .9326045532227E+03},
 {    8432, .8343522460938E+04, -.1136730102539E+04, .9400874023438E+03},
 {    8433, .8324271484375E+04, -.1125050415039E+04, .9487050781250E+03},
 {    8434, .8303273437500E+04, -.1117846435547E+04, .9581055908203E+03},
 {    8435, .8281380859375E+04, -.1115412109375E+04, .9679062500000E+03},
 {    8436, .8380252929688E+04, -.1239348510742E+04, .9097310180664E+03},
 {    8437, .8377042968750E+04, -.1218246704102E+04, .9111669921875E+03},
 {    8438, .8370301757812E+04, -.1198169311523E+04, .9141845703125E+03},
 {    8439, .8360266601562E+04, -.1179819580078E+04, .9186773681641E+03},
 {    8440, .8347285156250E+04, -.1163840087891E+04, .9244891967773E+03},
 {    8441, .8331810546875E+04, -.1150791015625E+04, .9314160156250E+03},
 {    8442, .8314390625000E+04, -.1141129516602E+04, .9392144775391E+03},
 {    8443, .8295630859375E+04, -.1135193969727E+04, .9476119384766E+03},
 {    8444, .8276192382812E+04, -.1133192016602E+04, .9563143310547E+03},
 {    8446, .8380446289062E+04, -.1217321411133E+04, .9096433105469E+03},
 {    8447, .8374292968750E+04, -.1195620727539E+04, .9123984375000E+03},
 {    8448, .8364407226562E+04, -.1175634521484E+04, .9168236083984E+03},
 {    8449, .8351166992188E+04, -.1158124877930E+04, .9227509765625E+03},
 {    8450, .8335071289062E+04, -.1143760498047E+04, .9299558715820E+03},
 {    8451, .8316737304688E+04, -.1133089965820E+04, .9381639404297E+03},
 {    8452, .8296858398438E+04, -.1126520141602E+04, .9470629882812E+03},
 {    8453, .8276192382812E+04, -.1124302001953E+04, .9563143310547E+03},
 {    8455, .8383851562500E+04, -.1216396118164E+04, .9081196289062E+03},
 {    8456, .8378279296875E+04, -.1193072021484E+04, .9106126708984E+03},
 {    8457, .8368549804688E+04, -.1171449340820E+04, .9149694824219E+03},
 {    8458, .8355051757812E+04, -.1152409667969E+04, .9210127563477E+03},
 {    8459, .8338333007812E+04, -.1136730102539E+04, .9284960937500E+03},
 {    8460, .8319083984375E+04, -.1125050415039E+04, .9371135253906E+03},
 {    8461, .8298083984375E+04, -.1117846435547E+04, .9465142211914E+03},
 {    8462, .8276192382812E+04, -.1115412109375E+04, .9563143310547E+03},
 {    8463, .8375061523438E+04, -.1239348510742E+04, .8981396484375E+03},
 {    8464, .8371854492188E+04, -.1218246704102E+04, .8995756225586E+03},
 {    8465, .8365113281250E+04, -.1198169311523E+04, .9025930175781E+03},
 {    8466, .8355077148438E+04, -.1179819580078E+04, .9070859985352E+03},
 {    8467, .8342094726562E+04, -.1163840087891E+04, .9128980102539E+03},
 {    8468, .8326620117188E+04, -.1150791015625E+04, .9198244628906E+03},
 {    8469, .8309200195312E+04, -.1141129516602E+04, .9276232299805E+03},
 {    8470, .8290443359375E+04, -.1135193969727E+04, .9360202636719E+03},
 {    8471, .8271002929688E+04, -.1133192016602E+04, .9447230224609E+03},
 {    8473, .8375256835938E+04, -.1217321411133E+04, .8980517578125E+03},
 {    8474, .8369101562500E+04, -.1195620727539E+04, .9008072509766E+03},
 {    8475, .8359217773438E+04, -.1175634521484E+04, .9052319335938E+03},
 {    8476, .8345976562500E+04, -.1158124877930E+04, .9111595458984E+03},
 {    8477, .8329882812500E+04, -.1143760498047E+04, .9183644409180E+03},
 {    8478, .8311546875000E+04, -.1133089965820E+04, .9265723876953E+03},
 {    8479, .8291667968750E+04, -.1126520141602E+04, .9354713745117E+03},
 {    8480, .8271002929688E+04, -.1124302001953E+04, .9447230224609E+03},
 {    8482, .8378662109375E+04, -.1216396118164E+04, .8965280151367E+03},
 {    8483, .8373092773438E+04, -.1193072021484E+04, .8990208129883E+03},
 {    8484, .8363361328125E+04, -.1171449340820E+04, .9033778686523E+03},
 {    8485, .8349859375000E+04, -.1152409667969E+04, .9094213867188E+03},
 {    8486, .8333142578125E+04, -.1136730102539E+04, .9169044799805E+03},
 {    8487, .8313894531250E+04, -.1125050415039E+04, .9255219726562E+03},
 {    8488, .8292895507812E+04, -.1117846435547E+04, .9349226074219E+03},
 {    8489, .8271002929688E+04, -.1115412109375E+04, .9447230224609E+03},
 {    8490, .8318960937500E+04, -.1135206542969E+04, .1104142333984E+04},
 {    8491, .8300147460938E+04, -.1141179199219E+04, .1112564086914E+04},
 {    8492, .8282684570312E+04, -.1150899414062E+04, .1120381347656E+04},
 {    8493, .8267190429688E+04, -.1164024902344E+04, .1127318359375E+04},
 {    8494, .8254203125000E+04, -.1180092651367E+04, .1133131225586E+04},
 {    8495, .8244192382812E+04, -.1198536132812E+04, .1137613525391E+04},
 {    8496, .8237499023438E+04, -.1218705200195E+04, .1140609375000E+04},
 {    8497, .8234367187500E+04, -.1239888916016E+04, .1142011596680E+04},
 {    8498, .8317786132812E+04, -.1126521728516E+04, .1104668090820E+04},
 {    8499, .8297896484375E+04, -.1133096191406E+04, .1113572021484E+04},
 {    8500, .8279554687500E+04, -.1143774414062E+04, .1121782592773E+04},
 {    8501, .8263457031250E+04, -.1158148925781E+04, .1128989501953E+04},
 {    8502, .8250209960938E+04, -.1175670898438E+04, .1134919189453E+04},
 {    8503, .8240326171875E+04, -.1195671264648E+04, .1139343994141E+04},
 {    8504, .8234177734375E+04, -.1217387207031E+04, .1142098022461E+04},
 {    8505, .8231989257812E+04, -.1239990722656E+04, .1143076171875E+04},
 {    8506, .8316610351562E+04, -.1117836914062E+04, .1105194458008E+04},
 {    8507, .8295650390625E+04, -.1125013183594E+04, .1114578125000E+04},
 {    8508, .8276424804688E+04, -.1136649291992E+04, .1123183349609E+04},
 {    8509, .8259722656250E+04, -.1152272949219E+04, .1130661132812E+04},
 {    8510, .8246217773438E+04, -.1171249145508E+04, .1136706542969E+04},
 {    8511, .8236459960938E+04, -.1192806396484E+04, .1141074340820E+04},
 {    8512, .8230848632812E+04, -.1216069213867E+04, .1143586425781E+04},
 {    8513, .8229608398438E+04, -.1240092529297E+04, .1144141845703E+04},
 {    8514, .8313771484375E+04, -.1135206542969E+04, .1092551269531E+04},
 {    8515, .8294958984375E+04, -.1141179199219E+04, .1100972656250E+04},
 {    8516, .8277495117188E+04, -.1150899414062E+04, .1108789794922E+04},
 {    8517, .8261999023438E+04, -.1164024902344E+04, .1115727050781E+04},
 {    8518, .8249014648438E+04, -.1180092651367E+04, .1121540161133E+04},
 {    8519, .8239000976562E+04, -.1198536132812E+04, .1126022705078E+04},
 {    8520, .8232313476562E+04, -.1218705200195E+04, .1129017333984E+04},
 {    8521, .8229177734375E+04, -.1239888916016E+04, .1130419677734E+04},
 {    8522, .8312596679688E+04, -.1126521728516E+04, .1093076904297E+04},
 {    8523, .8292709960938E+04, -.1133096191406E+04, .1101979492188E+04},
 {    8524, .8274368164062E+04, -.1143774414062E+04, .1110190551758E+04},
 {    8525, .8258265625000E+04, -.1158148925781E+04, .1117398437500E+04},
 {    8526, .8245022460938E+04, -.1175670898438E+04, .1123327026367E+04},
 {    8527, .8235137695312E+04, -.1195671264648E+04, .1127752929688E+04},
 {    8528, .8228985351562E+04, -.1217387207031E+04, .1130506347656E+04},
 {    8529, .8226797851562E+04, -.1239990722656E+04, .1131485473633E+04},
 {    8530, .8311421875000E+04, -.1117836914062E+04, .1093602539062E+04},
 {    8531, .8290460937500E+04, -.1125013183594E+04, .1102986206055E+04},
 {    8532, .8271234375000E+04, -.1136649291992E+04, .1111591918945E+04},
 {    8533, .8254533203125E+04, -.1152272949219E+04, .1119069580078E+04},
 {    8534, .8241029296875E+04, -.1171249145508E+04, .1125114501953E+04},
 {    8535, .8231269531250E+04, -.1192806396484E+04, .1129482421875E+04},
 {    8536, .8225661132812E+04, -.1216069213867E+04, .1131995117188E+04},
 {    8537, .8224417968750E+04, -.1240092529297E+04, .1132550537109E+04},
 {    8538, .8307932617188E+04, -.1135206542969E+04, .1079510498047E+04},
 {    8539, .8289121093750E+04, -.1141179199219E+04, .1087932128906E+04},
 {    8540, .8271659179688E+04, -.1150899414062E+04, .1095749511719E+04},
 {    8541, .8256161132812E+04, -.1164024902344E+04, .1102686523438E+04},
 {    8542, .8243177734375E+04, -.1180092651367E+04, .1108499145508E+04},
 {    8543, .8233164062500E+04, -.1198536132812E+04, .1112981933594E+04},
 {    8544, .8226472656250E+04, -.1218705200195E+04, .1115976928711E+04},
 {    8545, .8223339843750E+04, -.1239888916016E+04, .1117379638672E+04},
 {    8546, .8306758789062E+04, -.1126521728516E+04, .1080036132812E+04},
 {    8547, .8286873046875E+04, -.1133096191406E+04, .1088939331055E+04},
 {    8548, .8268528320312E+04, -.1143774414062E+04, .1097150634766E+04},
 {    8549, .8252427734375E+04, -.1158148925781E+04, .1104358398438E+04},
 {    8550, .8239181640625E+04, -.1175670898438E+04, .1110286743164E+04},
 {    8551, .8229302734375E+04, -.1195671264648E+04, .1114711303711E+04},
 {    8552, .8223146484375E+04, -.1217387207031E+04, .1117465698242E+04},
 {    8553, .8220958007812E+04, -.1239990722656E+04, .1118444946289E+04},
 {    8554, .8305583984375E+04, -.1117836914062E+04, .1080562133789E+04},
 {    8555, .8284621093750E+04, -.1125013183594E+04, .1089946289062E+04},
 {    8556, .8265397460938E+04, -.1136649291992E+04, .1098551635742E+04},
 {    8557, .8248693359375E+04, -.1152272949219E+04, .1106029052734E+04},
 {    8558, .8235192382812E+04, -.1171249145508E+04, .1112074096680E+04},
 {    8559, .8225434570312E+04, -.1192806396484E+04, .1116442016602E+04},
 {    8560, .8219823242188E+04, -.1216069213867E+04, .1118954833984E+04},
 {    8561, .8218581054688E+04, -.1240092529297E+04, .1119510253906E+04},
 {    8562, .8302099609375E+04, -.1135206542969E+04, .1066469238281E+04},
 {    8563, .8283282226562E+04, -.1141179199219E+04, .1074891845703E+04},
 {    8564, .8265819335938E+04, -.1150899414062E+04, .1082709106445E+04},
 {    8565, .8250323242188E+04, -.1164024902344E+04, .1089646362305E+04},
 {    8566, .8237337890625E+04, -.1180092651367E+04, .1095459106445E+04},
 {    8567, .8227326171875E+04, -.1198536132812E+04, .1099941284180E+04},
 {    8568, .8220633789062E+04, -.1218705200195E+04, .1102936523438E+04},
 {    8569, .8217502929688E+04, -.1239888916016E+04, .1104338867188E+04},
 {    8570, .8300920898438E+04, -.1126521728516E+04, .1066995727539E+04},
 {    8571, .8281034179688E+04, -.1133096191406E+04, .1075898803711E+04},
 {    8572, .8262691406250E+04, -.1143774414062E+04, .1084109863281E+04},
 {    8573, .8246588867188E+04, -.1158148925781E+04, .1091317504883E+04},
 {    8574, .8233345703125E+04, -.1175670898438E+04, .1097246337891E+04},
 {    8575, .8223462890625E+04, -.1195671264648E+04, .1101671386719E+04},
 {    8576, .8217308593750E+04, -.1217387207031E+04, .1104425537109E+04},
 {    8577, .8215123046875E+04, -.1239990722656E+04, .1105403930664E+04},
 {    8578, .8299747070312E+04, -.1117836914062E+04, .1067521484375E+04},
 {    8579, .8278784179688E+04, -.1125013183594E+04, .1076906127930E+04},
 {    8580, .8259560546875E+04, -.1136649291992E+04, .1085510742188E+04},
 {    8581, .8242859375000E+04, -.1152272949219E+04, .1092988281250E+04},
 {    8582, .8229350585938E+04, -.1171249145508E+04, .1099034057617E+04},
 {    8583, .8219596679688E+04, -.1192806396484E+04, .1103401733398E+04},
 {    8584, .8213984375000E+04, -.1216069213867E+04, .1105913818359E+04},
 {    8585, .8212744140625E+04, -.1240092529297E+04, .1106469604492E+04},
 {    8586, .8290421875000E+04, -.1135206542969E+04, .1040389282227E+04},
 {    8587, .8271608398438E+04, -.1141179199219E+04, .1048811157227E+04},
 {    8588, .8254144531250E+04, -.1150899414062E+04, .1056627685547E+04},
 {    8589, .8238650390625E+04, -.1164024902344E+04, .1063564941406E+04},
 {    8590, .8225664062500E+04, -.1180092651367E+04, .1069377685547E+04},
 {    8591, .8215651367188E+04, -.1198536132812E+04, .1073860717773E+04},
 {    8592, .8208959960938E+04, -.1218705200195E+04, .1076855346680E+04},
 {    8593, .8205828125000E+04, -.1239888916016E+04, .1078257568359E+04},
 {    8594, .8289246093750E+04, -.1126521728516E+04, .1040914550781E+04},
 {    8595, .8269361328125E+04, -.1133096191406E+04, .1049817382812E+04},
 {    8596, .8251015625000E+04, -.1143774414062E+04, .1058029052734E+04},
 {    8597, .8234912109375E+04, -.1158148925781E+04, .1065236816406E+04},
 {    8598, .8221671875000E+04, -.1175670898438E+04, .1071165039062E+04},
 {    8599, .8211786132812E+04, -.1195671264648E+04, .1075589965820E+04},
 {    8600, .8205633789062E+04, -.1217387207031E+04, .1078344970703E+04},
 {    8601, .8203447265625E+04, -.1239990722656E+04, .1079322998047E+04},
 {    8602, .8288071289062E+04, -.1117836914062E+04, .1041441040039E+04},
 {    8603, .8267108398438E+04, -.1125013183594E+04, .1050824829102E+04},
 {    8604, .8247885742188E+04, -.1136649291992E+04, .1059430175781E+04},
 {    8605, .8231182617188E+04, -.1152272949219E+04, .1066907714844E+04},
 {    8606, .8217679687500E+04, -.1171249145508E+04, .1072952880859E+04},
 {    8607, .8207920898438E+04, -.1192806396484E+04, .1077320922852E+04},
 {    8608, .8202309570312E+04, -.1216069213867E+04, .1079833251953E+04},
 {    8609, .8201068359375E+04, -.1240092529297E+04, .1080388427734E+04},
 {    8610, .8278745117188E+04, -.1135206542969E+04, .1014307922363E+04},
 {    8611, .8259934570312E+04, -.1141179199219E+04, .1022729980469E+04},
 {    8612, .8242470703125E+04, -.1150899536133E+04, .1030547363281E+04},
 {    8613, .8226974609375E+04, -.1164024902344E+04, .1037484130859E+04},
 {    8614, .8213989257812E+04, -.1180092651367E+04, .1043297119141E+04},
 {    8615, .8203974609375E+04, -.1198536132812E+04, .1047779418945E+04},
 {    8616, .8197284179688E+04, -.1218705200195E+04, .1050774658203E+04},
 {    8617, .8194154296875E+04, -.1239888916016E+04, .1052176879883E+04},
 {    8618, .8277570312500E+04, -.1126521728516E+04, .1014833862305E+04},
 {    8619, .8257681640625E+04, -.1133096191406E+04, .1023736938477E+04},
 {    8620, .8239339843750E+04, -.1143774414062E+04, .1031948364258E+04},
 {    8621, .8223239257812E+04, -.1158148925781E+04, .1039155273438E+04},
 {    8622, .8209996093750E+04, -.1175670898438E+04, .1045084838867E+04},
 {    8623, .8200112304688E+04, -.1195671264648E+04, .1049509643555E+04},
 {    8624, .8193959960938E+04, -.1217387207031E+04, .1052263549805E+04},
 {    8625, .8191771484375E+04, -.1239990722656E+04, .1053242553711E+04},
 {    8626, .8276395507812E+04, -.1117836914062E+04, .1015359985352E+04},
 {    8627, .8255431640625E+04, -.1125013183594E+04, .1024744140625E+04},
 {    8628, .8236209960938E+04, -.1136649291992E+04, .1033349365234E+04},
 {    8629, .8219506835938E+04, -.1152272949219E+04, .1040826416016E+04},
 {    8630, .8206003906250E+04, -.1171249145508E+04, .1046871948242E+04},
 {    8631, .8196246093750E+04, -.1192806396484E+04, .1051239501953E+04},
 {    8632, .8190631347656E+04, -.1216069213867E+04, .1053752441406E+04},
 {    8633, .8189393066406E+04, -.1240092651367E+04, .1054307495117E+04},
 {    8634, .8267069335938E+04, -.1135206542969E+04, .9882270507812E+03},
 {    8635, .8248258789062E+04, -.1141179199219E+04, .9966486816406E+03},
 {    8636, .8230792968750E+04, -.1150899536133E+04, .1004466430664E+04},
 {    8637, .8215297851562E+04, -.1164024902344E+04, .1011403381348E+04},
 {    8638, .8202315429688E+04, -.1180092651367E+04, .1017215454102E+04},
 {    8639, .8192301757812E+04, -.1198536132812E+04, .1021698852539E+04},
 {    8640, .8185608886719E+04, -.1218705200195E+04, .1024694091797E+04},
 {    8641, .8182478515625E+04, -.1239888916016E+04, .1026095458984E+04},
 {    8642, .8265894531250E+04, -.1126521728516E+04, .9887529296875E+03},
 {    8643, .8246007812500E+04, -.1133096191406E+04, .9976555175781E+03},
 {    8644, .8227665039062E+04, -.1143774414062E+04, .1005867187500E+04},
 {    8645, .8211561523438E+04, -.1158148925781E+04, .1013075073242E+04},
 {    8646, .8198318359375E+04, -.1175670898438E+04, .1019003417969E+04},
 {    8647, .8188434570312E+04, -.1195671264648E+04, .1023429077148E+04},
 {    8648, .8182281250000E+04, -.1217387207031E+04, .1026182739258E+04},
 {    8649, .8180096191406E+04, -.1239990722656E+04, .1027161621094E+04},
 {    8650, .8264719726562E+04, -.1117836914062E+04, .9892788085938E+03},
 {    8651, .8243757812500E+04, -.1125013183594E+04, .9986630859375E+03},
 {    8652, .8224534179688E+04, -.1136649291992E+04, .1007268371582E+04},
 {    8653, .8207832031250E+04, -.1152272949219E+04, .1014745727539E+04},
 {    8654, .8194326171875E+04, -.1171249145508E+04, .1020791259766E+04},
 {    8655, .8184569335938E+04, -.1192806396484E+04, .1025158691406E+04},
 {    8656, .8178958496094E+04, -.1216069213867E+04, .1027671264648E+04},
 {    8657, .8177716308594E+04, -.1240092651367E+04, .1028226928711E+04},
 {    8658, .8261879882812E+04, -.1135206542969E+04, .9766353759766E+03},
 {    8659, .8243068359375E+04, -.1141179199219E+04, .9850572509766E+03},
 {    8660, .8225605468750E+04, -.1150899536133E+04, .9928741455078E+03},
 {    8661, .8210108398438E+04, -.1164024902344E+04, .9998118286133E+03},
 {    8662, .8197124023438E+04, -.1180092651367E+04, .1005624633789E+04},
 {    8663, .8187110839844E+04, -.1198536132812E+04, .1010106567383E+04},
 {    8664, .8180420410156E+04, -.1218705200195E+04, .1013102172852E+04},
 {    8665, .8177286132812E+04, -.1239888916016E+04, .1014504577637E+04},
 {    8666, .8260705078125E+04, -.1126521728516E+04, .9771614379883E+03},
 {    8667, .8240818359375E+04, -.1133096191406E+04, .9860640869141E+03},
 {    8668, .8222476562500E+04, -.1143774414062E+04, .9942760009766E+03},
 {    8669, .8206377929688E+04, -.1158148925781E+04, .1001482727051E+04},
 {    8670, .8193130859375E+04, -.1175670898438E+04, .1007411987305E+04},
 {    8671, .8183246093750E+04, -.1195671264648E+04, .1011836914062E+04},
 {    8672, .8177093261719E+04, -.1217387207031E+04, .1014591552734E+04},
 {    8673, .8174909179688E+04, -.1239990722656E+04, .1015569824219E+04},
 {    8674, .8259531250000E+04, -.1117836914062E+04, .9776872558594E+03},
 {    8675, .8238565429688E+04, -.1125013183594E+04, .9870718994141E+03},
 {    8676, .8219344726562E+04, -.1136649291992E+04, .9956768798828E+03},
 {    8677, .8202644531250E+04, -.1152272949219E+04, .1003153686523E+04},
 {    8678, .8189138183594E+04, -.1171249145508E+04, .1009199584961E+04},
 {    8679, .8179381835938E+04, -.1192806396484E+04, .1013567199707E+04},
 {    8680, .8173768554688E+04, -.1216069335938E+04, .1016079589844E+04},
 {    8681, .8172526367188E+04, -.1240092651367E+04, .1016635498047E+04},
 {    8682, .8256692382812E+04, -.1135206542969E+04, .9650437622070E+03},
 {    8683, .8237877929688E+04, -.1141179199219E+04, .9734658813477E+03},
 {    8684, .8220416015625E+04, -.1150899536133E+04, .9812830810547E+03},
 {    8685, .8204917968750E+04, -.1164024902344E+04, .9882203369141E+03},
 {    8686, .8191936523438E+04, -.1180092651367E+04, .9940324096680E+03},
 {    8687, .8181922363281E+04, -.1198536132812E+04, .9985152587891E+03},
 {    8688, .8175230468750E+04, -.1218705200195E+04, .1001510742188E+04},
 {    8689, .8172100585938E+04, -.1239888916016E+04, .1002912353516E+04},
 {    8690, .8255515625000E+04, -.1126521728516E+04, .9655699462891E+03},
 {    8691, .8235626953125E+04, -.1133096191406E+04, .9744730224609E+03},
 {    8692, .8217285156250E+04, -.1143774414062E+04, .9826840820312E+03},
 {    8693, .8201186523438E+04, -.1158148925781E+04, .9898918457031E+03},
 {    8694, .8187939941406E+04, -.1175670898438E+04, .9958207397461E+03},
 {    8695, .8178055664062E+04, -.1195671264648E+04, .1000245971680E+04},
 {    8696, .8171906250000E+04, -.1217387207031E+04, .1002999267578E+04},
 {    8697, .8169720703125E+04, -.1239990722656E+04, .1003978027344E+04},
 {    8698, .8254343750000E+04, -.1117836914062E+04, .9660954589844E+03},
 {    8699, .8233379882812E+04, -.1125013183594E+04, .9754797363281E+03},
 {    8700, .8214157226562E+04, -.1136649291992E+04, .9840851440430E+03},
 {    8701, .8197452148438E+04, -.1152272949219E+04, .9915626220703E+03},
 {    8702, .8183947753906E+04, -.1171249145508E+04, .9976080322266E+03},
 {    8703, .8174192382812E+04, -.1192806396484E+04, .1001975708008E+04},
 {    8704, .8168580078125E+04, -.1216069335938E+04, .1004488098145E+04},
 {    8705, .8167338378906E+04, -.1240092651367E+04, .1005043945312E+04},
 {    8706, .8251500976562E+04, -.1135206542969E+04, .9534526367188E+03},
 {    8707, .8232691406250E+04, -.1141179199219E+04, .9618740844727E+03},
 {    8708, .8215225585938E+04, -.1150899536133E+04, .9696917724609E+03},
 {    8709, .8199730468750E+04, -.1164024902344E+04, .9766282958984E+03},
 {    8710, .8186746582031E+04, -.1180092651367E+04, .9824411621094E+03},
 {    8711, .8176730957031E+04, -.1198536132812E+04, .9869241943359E+03},
 {    8712, .8170043945312E+04, -.1218705200195E+04, .9899189453125E+03},
 {    8713, .8166911132812E+04, -.1239888916016E+04, .9913211669922E+03},
 {    8714, .8250328125000E+04, -.1126521728516E+04, .9539782104492E+03},
 {    8715, .8230437500000E+04, -.1133096191406E+04, .9628814697266E+03},
 {    8716, .8212095703125E+04, -.1143774414062E+04, .9710927124023E+03},
 {    8717, .8195996093750E+04, -.1158148925781E+04, .9782998046875E+03},
 {    8718, .8182752929688E+04, -.1175670898438E+04, .9842291870117E+03},
 {    8719, .8172869140625E+04, -.1195671264648E+04, .9886541748047E+03},
 {    8720, .8166716796875E+04, -.1217387207031E+04, .9914078369141E+03},
 {    8721, .8164530761719E+04, -.1239990722656E+04, .9923865966797E+03},
 {    8722, .8249152343750E+04, -.1117836914062E+04, .9545041503906E+03},
 {    8723, .8228189453125E+04, -.1125013183594E+04, .9638881225586E+03},
 {    8724, .8208966796875E+04, -.1136649291992E+04, .9724938964844E+03},
 {    8725, .8192263671875E+04, -.1152272949219E+04, .9799709472656E+03},
 {    8726, .8178759277344E+04, -.1171249145508E+04, .9860163574219E+03},
 {    8727, .8169003417969E+04, -.1192806396484E+04, .9903842163086E+03},
 {    8728, .8163390625000E+04, -.1216069335938E+04, .9928967285156E+03},
 {    8729, .8162149902344E+04, -.1240092651367E+04, .9934521484375E+03},
 {    8730, .8234250976562E+04, -.1262740478516E+04, .1142062377930E+04},
 {    8731, .8234134765625E+04, -.1285592041016E+04, .1142114868164E+04},
 {    8732, .8231902343750E+04, -.1262791381836E+04, .1143114501953E+04},
 {    8733, .8231818359375E+04, -.1285592041016E+04, .1143152832031E+04},
 {    8734, .8229554687500E+04, -.1262842285156E+04, .1144165893555E+04},
 {    8735, .8229499023438E+04, -.1285592041016E+04, .1144190429688E+04},
 {    8736, .8229063476562E+04, -.1262740478516E+04, .1130471679688E+04},
 {    8737, .8228947265625E+04, -.1285592041016E+04, .1130523315430E+04},
 {    8738, .8226712890625E+04, -.1262791381836E+04, .1131523559570E+04},
 {    8739, .8226626953125E+04, -.1285592041016E+04, .1131561279297E+04},
 {    8740, .8224366210938E+04, -.1262842285156E+04, .1132574584961E+04},
 {    8741, .8224312500000E+04, -.1285592041016E+04, .1132598632812E+04},
 {    8742, .8223227539062E+04, -.1262740478516E+04, .1117430664062E+04},
 {    8743, .8223108398438E+04, -.1285592041016E+04, .1117482788086E+04},
 {    8744, .8220876953125E+04, -.1262791381836E+04, .1118482543945E+04},
 {    8745, .8220790039062E+04, -.1285592041016E+04, .1118520507812E+04},
 {    8746, .8218527343750E+04, -.1262842285156E+04, .1119534423828E+04},
 {    8747, .8218474609375E+04, -.1285592041016E+04, .1119558105469E+04},
 {    8748, .8217389648438E+04, -.1262740478516E+04, .1104390258789E+04},
 {    8749, .8217269531250E+04, -.1285592041016E+04, .1104442504883E+04},
 {    8750, .8215039062500E+04, -.1262791381836E+04, .1105442138672E+04},
 {    8751, .8214955078125E+04, -.1285592041016E+04, .1105479736328E+04},
 {    8752, .8212689453125E+04, -.1262842285156E+04, .1106493286133E+04},
 {    8753, .8212635742188E+04, -.1285592041016E+04, .1106517578125E+04},
 {    8754, .8205712890625E+04, -.1262740478516E+04, .1078309570312E+04},
 {    8755, .8205597656250E+04, -.1285592041016E+04, .1078361328125E+04},
 {    8756, .8203362304688E+04, -.1262791381836E+04, .1079361572266E+04},
 {    8757, .8203278320312E+04, -.1285592041016E+04, .1079398681641E+04},
 {    8758, .8201015625000E+04, -.1262842285156E+04, .1080412353516E+04},
 {    8759, .8200959960938E+04, -.1285592041016E+04, .1080436767578E+04},
 {    8760, .8194037109375E+04, -.1262740478516E+04, .1052228027344E+04},
 {    8761, .8193920898438E+04, -.1285592041016E+04, .1052280273438E+04},
 {    8762, .8191687500000E+04, -.1262791381836E+04, .1053279663086E+04},
 {    8763, .8191603027344E+04, -.1285592041016E+04, .1053317871094E+04},
 {    8764, .8189339843750E+04, -.1262842285156E+04, .1054331542969E+04},
 {    8765, .8189285644531E+04, -.1285592041016E+04, .1054355957031E+04},
 {    8766, .8182361328125E+04, -.1262740478516E+04, .1026147705078E+04},
 {    8767, .8182247558594E+04, -.1285592041016E+04, .1026198974609E+04},
 {    8768, .8180013183594E+04, -.1262791381836E+04, .1027198974609E+04},
 {    8769, .8179927246094E+04, -.1285592041016E+04, .1027237182617E+04},
 {    8770, .8177662109375E+04, -.1262842285156E+04, .1028251098633E+04},
 {    8771, .8177610351562E+04, -.1285592041016E+04, .1028274780273E+04},
 {    8772, .8177172851562E+04, -.1262740478516E+04, .1014555908203E+04},
 {    8773, .8177056152344E+04, -.1285592041016E+04, .1014607910156E+04},
 {    8774, .8174823242188E+04, -.1262791381836E+04, .1015607299805E+04},
 {    8775, .8174738281250E+04, -.1285592041016E+04, .1015645507812E+04},
 {    8776, .8172475585938E+04, -.1262842285156E+04, .1016659057617E+04},
 {    8777, .8172420410156E+04, -.1285592041016E+04, .1016683105469E+04},
 {    8778, .8171982421875E+04, -.1262740478516E+04, .1002964355469E+04},
 {    8779, .8171867675781E+04, -.1285592041016E+04, .1003016174316E+04},
 {    8780, .8169634277344E+04, -.1262791381836E+04, .1004015991211E+04},
 {    8781, .8169549316406E+04, -.1285592041016E+04, .1004053771973E+04},
 {    8782, .8167285156250E+04, -.1262842285156E+04, .1005067687988E+04},
 {    8783, .8167229980469E+04, -.1285592041016E+04, .1005092102051E+04},
 {    8784, .8166795410156E+04, -.1262740478516E+04, .9913728637695E+03},
 {    8785, .8166678222656E+04, -.1285592041016E+04, .9914249267578E+03},
 {    8786, .8164444824219E+04, -.1262791381836E+04, .9924248046875E+03},
 {    8787, .8164359375000E+04, -.1285592041016E+04, .9924627685547E+03},
 {    8788, .8162095214844E+04, -.1262842285156E+04, .9934760742188E+03},
 {    8789, .8162042480469E+04, -.1285592041016E+04, .9935001220703E+03},
 {    8790, .8234250976562E+04, -.1308443603516E+04, .1142062377930E+04},
 {    8791, .8234367187500E+04, -.1331295166016E+04, .1142011596680E+04},
 {    8792, .8231902343750E+04, -.1308392700195E+04, .1143114501953E+04},
 {    8793, .8231989257812E+04, -.1331193359375E+04, .1143076171875E+04},
 {    8794, .8229554687500E+04, -.1308341796875E+04, .1144165893555E+04},
 {    8795, .8229608398438E+04, -.1331091552734E+04, .1144141845703E+04},
 {    8796, .8229063476562E+04, -.1308443603516E+04, .1130471679688E+04},
 {    8797, .8229177734375E+04, -.1331295166016E+04, .1130419677734E+04},
 {    8798, .8226712890625E+04, -.1308392700195E+04, .1131523559570E+04},
 {    8799, .8226797851562E+04, -.1331193359375E+04, .1131485473633E+04},
 {    8800, .8224366210938E+04, -.1308341796875E+04, .1132574584961E+04},
 {    8801, .8224417968750E+04, -.1331091552734E+04, .1132550537109E+04},
 {    8802, .8223227539062E+04, -.1308443603516E+04, .1117430664062E+04},
 {    8803, .8223339843750E+04, -.1331295166016E+04, .1117379638672E+04},
 {    8804, .8220876953125E+04, -.1308392700195E+04, .1118482543945E+04},
 {    8805, .8220958007812E+04, -.1331193359375E+04, .1118444946289E+04},
 {    8806, .8218527343750E+04, -.1308341796875E+04, .1119534423828E+04},
 {    8807, .8218581054688E+04, -.1331091552734E+04, .1119510253906E+04},
 {    8808, .8217389648438E+04, -.1308443603516E+04, .1104390258789E+04},
 {    8809, .8217502929688E+04, -.1331295166016E+04, .1104338867188E+04},
 {    8810, .8215039062500E+04, -.1308392700195E+04, .1105442138672E+04},
 {    8811, .8215123046875E+04, -.1331193359375E+04, .1105403930664E+04},
 {    8812, .8212689453125E+04, -.1308341796875E+04, .1106493286133E+04},
 {    8813, .8212744140625E+04, -.1331091552734E+04, .1106469848633E+04},
 {    8814, .8205712890625E+04, -.1308443603516E+04, .1078309570312E+04},
 {    8815, .8205828125000E+04, -.1331295166016E+04, .1078257568359E+04},
 {    8816, .8203362304688E+04, -.1308392700195E+04, .1079361572266E+04},
 {    8817, .8203447265625E+04, -.1331193359375E+04, .1079322998047E+04},
 {    8818, .8201015625000E+04, -.1308341796875E+04, .1080412353516E+04},
 {    8819, .8201068359375E+04, -.1331091552734E+04, .1080388671875E+04},
 {    8820, .8194037109375E+04, -.1308443603516E+04, .1052228027344E+04},
 {    8821, .8194154296875E+04, -.1331295166016E+04, .1052176879883E+04},
 {    8822, .8191687500000E+04, -.1308392700195E+04, .1053279663086E+04},
 {    8823, .8191771484375E+04, -.1331193359375E+04, .1053242553711E+04},
 {    8824, .8189339843750E+04, -.1308341796875E+04, .1054331542969E+04},
 {    8825, .8189393066406E+04, -.1331091552734E+04, .1054307495117E+04},
 {    8826, .8182361328125E+04, -.1308443603516E+04, .1026147705078E+04},
 {    8827, .8182478515625E+04, -.1331295166016E+04, .1026095458984E+04},
 {    8828, .8180013183594E+04, -.1308392700195E+04, .1027198974609E+04},
 {    8829, .8180096191406E+04, -.1331193359375E+04, .1027161621094E+04},
 {    8830, .8177662109375E+04, -.1308341796875E+04, .1028251098633E+04},
 {    8831, .8177716308594E+04, -.1331091552734E+04, .1028226928711E+04},
 {    8832, .8177172851562E+04, -.1308443603516E+04, .1014555908203E+04},
 {    8833, .8177286132812E+04, -.1331295166016E+04, .1014504577637E+04},
 {    8834, .8174823242188E+04, -.1308392700195E+04, .1015607299805E+04},
 {    8835, .8174909179688E+04, -.1331193359375E+04, .1015569824219E+04},
 {    8836, .8172475585938E+04, -.1308341796875E+04, .1016659057617E+04},
 {    8837, .8172526367188E+04, -.1331091552734E+04, .1016635498047E+04},
 {    8838, .8171982421875E+04, -.1308443603516E+04, .1002964355469E+04},
 {    8839, .8172100585938E+04, -.1331295166016E+04, .1002912353516E+04},
 {    8840, .8169634277344E+04, -.1308392700195E+04, .1004015991211E+04},
 {    8841, .8169720703125E+04, -.1331193359375E+04, .1003978027344E+04},
 {    8842, .8167285156250E+04, -.1308341796875E+04, .1005067687988E+04},
 {    8843, .8167338378906E+04, -.1331091552734E+04, .1005043945312E+04},
 {    8844, .8166795410156E+04, -.1308443603516E+04, .9913728637695E+03},
 {    8845, .8166911132812E+04, -.1331295166016E+04, .9913211669922E+03},
 {    8846, .8164444824219E+04, -.1308392700195E+04, .9924248046875E+03},
 {    8847, .8164530761719E+04, -.1331193359375E+04, .9923865966797E+03},
 {    8848, .8162095214844E+04, -.1308341796875E+04, .9934760742188E+03},
 {    8849, .8162149902344E+04, -.1331091552734E+04, .9934521484375E+03},
 {    8850, .8236053710938E+04, -.1345494140625E+04, .1141256835938E+04},
 {    8851, .8239341796875E+04, -.1359350952148E+04, .1139783935547E+04},
 {    8852, .8244192382812E+04, -.1372648193359E+04, .1137613525391E+04},
 {    8853, .8250514648438E+04, -.1385177246094E+04, .1134782226562E+04},
 {    8854, .8258223632812E+04, -.1396741333008E+04, .1131332275391E+04},
 {    8855, .8267190429688E+04, -.1407159179688E+04, .1127318359375E+04},
 {    8856, .8277272460938E+04, -.1416267211914E+04, .1122805053711E+04},
 {    8857, .8288315429688E+04, -.1423922363281E+04, .1117860595703E+04},
 {    8858, .8300147460938E+04, -.1430004760742E+04, .1112564331055E+04},
 {    8859, .8312578125000E+04, -.1434418823242E+04, .1106999267578E+04},
 {    8860, .8325416992188E+04, -.1437095214844E+04, .1101251464844E+04},
 {    8861, .8338461914062E+04, -.1437992065430E+04, .1095413452148E+04},
 {    8862, .8232999023438E+04, -.1346318969727E+04, .1142624145508E+04},
 {    8863, .8235792968750E+04, -.1361176269531E+04, .1141373168945E+04},
 {    8864, .8240326171875E+04, -.1375513183594E+04, .1139343994141E+04},
 {    8865, .8246522460938E+04, -.1389086059570E+04, .1136570312500E+04},
 {    8866, .8254275390625E+04, -.1401664672852E+04, .1133099365234E+04},
 {    8867, .8263457031250E+04, -.1413035156250E+04, .1128989501953E+04},
 {    8868, .8273909179688E+04, -.1423004394531E+04, .1124310668945E+04},
 {    8869, .8285453125000E+04, -.1431402832031E+04, .1119142089844E+04},
 {    8870, .8297896484375E+04, -.1438087768555E+04, .1113572021484E+04},
 {    8871, .8311033203125E+04, -.1442945556641E+04, .1107692260742E+04},
 {    8872, .8324628906250E+04, -.1445893798828E+04, .1101605224609E+04},
 {    8873, .8338461914062E+04, -.1446882080078E+04, .1095413452148E+04},
 {    8874, .8229945312500E+04, -.1347143798828E+04, .1143991699219E+04},
 {    8875, .8232245117188E+04, -.1363001586914E+04, .1142962402344E+04},
 {    8876, .8236459960938E+04, -.1378378051758E+04, .1141074340820E+04},
 {    8877, .8242527343750E+04, -.1392994995117E+04, .1138358886719E+04},
 {    8878, .8250325195312E+04, -.1406588012695E+04, .1134867187500E+04},
 {    8879, .8259722656250E+04, -.1418911132812E+04, .1130661132812E+04},
 {    8880, .8270543945312E+04, -.1429741577148E+04, .1125817504883E+04},
 {    8881, .8282589843750E+04, -.1438883178711E+04, .1120424682617E+04},
 {    8882, .8295649414062E+04, -.1446170776367E+04, .1114578857422E+04},
 {    8883, .8309482421875E+04, -.1451472412109E+04, .1108385253906E+04},
 {    8884, .8323838867188E+04, -.1454692260742E+04, .1101958618164E+04},
 {    8885, .8338461914062E+04, -.1455772094727E+04, .1095413452148E+04},
 {    8886, .8230862304688E+04, -.1345494140625E+04, .1129665283203E+04},
 {    8887, .8234156250000E+04, -.1359350952148E+04, .1128192138672E+04},
 {    8888, .8239000976562E+04, -.1372648193359E+04, .1126022705078E+04},
 {    8889, .8245328125000E+04, -.1385177246094E+04, .1123190551758E+04},
 {    8890, .8253033203125E+04, -.1396741333008E+04, .1119740356445E+04},
 {    8891, .8261999023438E+04, -.1407159179688E+04, .1115727050781E+04},
 {    8892, .8272083984375E+04, -.1416267211914E+04, .1111212890625E+04},
 {    8893, .8283126953125E+04, -.1423922363281E+04, .1106269531250E+04},
 {    8894, .8294958984375E+04, -.1430004760742E+04, .1100972656250E+04},
 {    8895, .8307389648438E+04, -.1434418823242E+04, .1095407714844E+04},
 {    8896, .8320229492188E+04, -.1437095214844E+04, .1089660400391E+04},
 {    8897, .8333272460938E+04, -.1437992065430E+04, .1083821044922E+04},
 {    8898, .8227806640625E+04, -.1346318969727E+04, .1131032958984E+04},
 {    8899, .8230602539062E+04, -.1361176269531E+04, .1129781738281E+04},
 {    8900, .8235137695312E+04, -.1375513183594E+04, .1127752929688E+04},
 {    8901, .8241333984375E+04, -.1389086059570E+04, .1124978759766E+04},
 {    8902, .8249085937500E+04, -.1401664672852E+04, .1121508300781E+04},
 {    8903, .8258265625000E+04, -.1413035156250E+04, .1117398437500E+04},
 {    8904, .8268716796875E+04, -.1423004394531E+04, .1112719604492E+04},
 {    8905, .8280263671875E+04, -.1431402832031E+04, .1107551025391E+04},
 {    8906, .8292708984375E+04, -.1438087768555E+04, .1101979736328E+04},
 {    8907, .8305841796875E+04, -.1442945556641E+04, .1096101074219E+04},
 {    8908, .8319438476562E+04, -.1445893798828E+04, .1090013305664E+04},
 {    8909, .8333272460938E+04, -.1446882080078E+04, .1083821044922E+04},
 {    8910, .8224752929688E+04, -.1347143798828E+04, .1132400268555E+04},
 {    8911, .8227052734375E+04, -.1363001586914E+04, .1131371093750E+04},
 {    8912, .8231270507812E+04, -.1378378051758E+04, .1129482299805E+04},
 {    8913, .8237336914062E+04, -.1392994995117E+04, .1126767578125E+04},
 {    8914, .8245138671875E+04, -.1406588012695E+04, .1123274902344E+04},
 {    8915, .8254533203125E+04, -.1418911132812E+04, .1119069580078E+04},
 {    8916, .8265353515625E+04, -.1429741577148E+04, .1114225952148E+04},
 {    8917, .8277401367188E+04, -.1438883178711E+04, .1108832275391E+04},
 {    8918, .8290459960938E+04, -.1446170776367E+04, .1102986572266E+04},
 {    8919, .8304292968750E+04, -.1451472412109E+04, .1096794189453E+04},
 {    8920, .8318648437500E+04, -.1454692260742E+04, .1090367187500E+04},
 {    8921, .8333272460938E+04, -.1455772094727E+04, .1083821044922E+04},
 {    8922, .8225024414062E+04, -.1345494140625E+04, .1116625732422E+04},
 {    8923, .8228316406250E+04, -.1359350952148E+04, .1115151489258E+04},
 {    8924, .8233164062500E+04, -.1372648193359E+04, .1112981933594E+04},
 {    8925, .8239490234375E+04, -.1385177246094E+04, .1110149902344E+04},
 {    8926, .8247196289062E+04, -.1396741333008E+04, .1106699707031E+04},
 {    8927, .8256161132812E+04, -.1407159179688E+04, .1102686523438E+04},
 {    8928, .8266246093750E+04, -.1416267211914E+04, .1098172485352E+04},
 {    8929, .8277288085938E+04, -.1423922363281E+04, .1093228759766E+04},
 {    8930, .8289121093750E+04, -.1430004760742E+04, .1087932128906E+04},
 {    8931, .8301552734375E+04, -.1434418823242E+04, .1082366821289E+04},
 {    8932, .8314392578125E+04, -.1437095214844E+04, .1076619262695E+04},
 {    8933, .8327435546875E+04, -.1437992065430E+04, .1070780395508E+04},
 {    8934, .8221971679688E+04, -.1346318969727E+04, .1117992553711E+04},
 {    8935, .8224767578125E+04, -.1361176269531E+04, .1116740844727E+04},
 {    8936, .8229302734375E+04, -.1375513183594E+04, .1114711303711E+04},
 {    8937, .8235492187500E+04, -.1389086059570E+04, .1111938720703E+04},
 {    8938, .8243248046875E+04, -.1401664672852E+04, .1108467407227E+04},
 {    8939, .8252427734375E+04, -.1413035156250E+04, .1104358398438E+04},
 {    8940, .8262880859375E+04, -.1423004394531E+04, .1099679077148E+04},
 {    8941, .8274427734375E+04, -.1431402832031E+04, .1094510131836E+04},
 {    8942, .8286873046875E+04, -.1438087768555E+04, .1088939331055E+04},
 {    8943, .8300003906250E+04, -.1442945556641E+04, .1083060546875E+04},
 {    8944, .8313602539062E+04, -.1445893798828E+04, .1076973022461E+04},
 {    8945, .8327435546875E+04, -.1446882080078E+04, .1070780395508E+04},
 {    8946, .8218916992188E+04, -.1347143798828E+04, .1119359619141E+04},
 {    8947, .8221213867188E+04, -.1363001586914E+04, .1118331176758E+04},
 {    8948, .8225432617188E+04, -.1378378051758E+04, .1116442138672E+04},
 {    8949, .8231498046875E+04, -.1392994995117E+04, .1113727050781E+04},
 {    8950, .8239299804688E+04, -.1406588012695E+04, .1110234619141E+04},
 {    8951, .8248693359375E+04, -.1418911132812E+04, .1106029052734E+04},
 {    8952, .8259514648438E+04, -.1429741577148E+04, .1101185791016E+04},
 {    8953, .8271561523438E+04, -.1438883178711E+04, .1095792236328E+04},
 {    8954, .8284620117188E+04, -.1446170776367E+04, .1089946777344E+04},
 {    8955, .8298454101562E+04, -.1451472412109E+04, .1083753662109E+04},
 {    8956, .8312812500000E+04, -.1454692260742E+04, .1077326538086E+04},
 {    8957, .8327435546875E+04, -.1455772094727E+04, .1070780395508E+04},
 {    8958, .8219188476562E+04, -.1345494140625E+04, .1103584594727E+04},
 {    8959, .8222475585938E+04, -.1359350952148E+04, .1102111450195E+04},
 {    8960, .8227326171875E+04, -.1372648193359E+04, .1099941284180E+04},
 {    8961, .8233650390625E+04, -.1385177246094E+04, .1097109375000E+04},
 {    8962, .8241358398438E+04, -.1396741333008E+04, .1093659423828E+04},
 {    8963, .8250323242188E+04, -.1407159179688E+04, .1089646362305E+04},
 {    8964, .8260410156250E+04, -.1416267211914E+04, .1085131713867E+04},
 {    8965, .8271452148438E+04, -.1423922363281E+04, .1080187988281E+04},
 {    8966, .8283281250000E+04, -.1430004760742E+04, .1074891845703E+04},
 {    8967, .8295714843750E+04, -.1434418823242E+04, .1069326293945E+04},
 {    8968, .8308554687500E+04, -.1437095214844E+04, .1063578979492E+04},
 {    8969, .8321595703125E+04, -.1437992065430E+04, .1057740600586E+04},
 {    8970, .8216134765625E+04, -.1346318969727E+04, .1104951660156E+04},
 {    8971, .8218927734375E+04, -.1361176269531E+04, .1103700683594E+04},
 {    8972, .8223462890625E+04, -.1375513183594E+04, .1101671142578E+04},
 {    8973, .8229655273438E+04, -.1389086059570E+04, .1098897949219E+04},
 {    8974, .8237412109375E+04, -.1401664672852E+04, .1095426513672E+04},
 {    8975, .8246588867188E+04, -.1413035156250E+04, .1091317504883E+04},
 {    8976, .8257044921875E+04, -.1423004394531E+04, .1086638671875E+04},
 {    8977, .8268588867188E+04, -.1431402832031E+04, .1081469726562E+04},
 {    8978, .8281034179688E+04, -.1438087768555E+04, .1075899047852E+04},
 {    8979, .8294166992188E+04, -.1442945556641E+04, .1070019531250E+04},
 {    8980, .8307763671875E+04, -.1445893798828E+04, .1063932495117E+04},
 {    8981, .8321595703125E+04, -.1446882080078E+04, .1057740600586E+04},
 {    8982, .8213080078125E+04, -.1347143798828E+04, .1106319458008E+04},
 {    8983, .8215378906250E+04, -.1363001586914E+04, .1105290527344E+04},
 {    8984, .8219596679688E+04, -.1378378051758E+04, .1103401733398E+04},
 {    8985, .8225662109375E+04, -.1392994995117E+04, .1100686401367E+04},
 {    8986, .8233462890625E+04, -.1406588012695E+04, .1097194458008E+04},
 {    8987, .8242859375000E+04, -.1418911132812E+04, .1092988281250E+04},
 {    8988, .8253676757812E+04, -.1429741577148E+04, .1088145263672E+04},
 {    8989, .8265726562500E+04, -.1438883178711E+04, .1082751342773E+04},
 {    8990, .8278784179688E+04, -.1446170776367E+04, .1076906127930E+04},
 {    8991, .8292618164062E+04, -.1451472412109E+04, .1070713134766E+04},
 {    8992, .8306974609375E+04, -.1454692260742E+04, .1064285888672E+04},
 {    8993, .8321595703125E+04, -.1455772094727E+04, .1057740600586E+04},
 {    8994, .8207511718750E+04, -.1345494140625E+04, .1077503662109E+04},
 {    8995, .8210803710938E+04, -.1359350952148E+04, .1076030151367E+04},
 {    8996, .8215651367188E+04, -.1372648193359E+04, .1073860717773E+04},
 {    8997, .8221978515625E+04, -.1385177246094E+04, .1071028076172E+04},
 {    8998, .8229682617188E+04, -.1396741333008E+04, .1067578857422E+04},
 {    8999, .8238650390625E+04, -.1407159179688E+04, .1063564941406E+04},
 {    9000, .8248732421875E+04, -.1416267211914E+04, .1059051269531E+04},
 {    9001, .8259776367188E+04, -.1423922363281E+04, .1054107177734E+04},
 {    9002, .8271608398438E+04, -.1430004760742E+04, .1048811157227E+04},
 {    9003, .8284040039062E+04, -.1434418823242E+04, .1043245483398E+04},
 {    9004, .8296877929688E+04, -.1437095214844E+04, .1037498046875E+04},
 {    9005, .8309919921875E+04, -.1437992065430E+04, .1031659423828E+04},
 {    9006, .8204457031250E+04, -.1346318969727E+04, .1078871215820E+04},
 {    9007, .8207251953125E+04, -.1361176269531E+04, .1077620117188E+04},
 {    9008, .8211786132812E+04, -.1375513183594E+04, .1075589965820E+04},
 {    9009, .8217983398438E+04, -.1389086059570E+04, .1072816894531E+04},
 {    9010, .8225734375000E+04, -.1401664672852E+04, .1069346191406E+04},
 {    9011, .8234912109375E+04, -.1413035156250E+04, .1065236816406E+04},
 {    9012, .8245370117188E+04, -.1423004394531E+04, .1060557250977E+04},
 {    9013, .8256914062500E+04, -.1431402832031E+04, .1055388793945E+04},
 {    9014, .8269359375000E+04, -.1438087768555E+04, .1049817382812E+04},
 {    9015, .8282492187500E+04, -.1442945556641E+04, .1043938842773E+04},
 {    9016, .8296087890625E+04, -.1445893798828E+04, .1037851562500E+04},
 {    9017, .8309919921875E+04, -.1446882080078E+04, .1031659423828E+04},
 {    9018, .8201403320312E+04, -.1347143798828E+04, .1080238769531E+04},
 {    9019, .8203701171875E+04, -.1363001586914E+04, .1079209594727E+04},
 {    9020, .8207920898438E+04, -.1378378051758E+04, .1077320922852E+04},
 {    9021, .8213987304688E+04, -.1392994995117E+04, .1074605346680E+04},
 {    9022, .8221787109375E+04, -.1406588012695E+04, .1071114013672E+04},
 {    9023, .8231182617188E+04, -.1418911132812E+04, .1066907714844E+04},
 {    9024, .8242001953125E+04, -.1429741577148E+04, .1062063720703E+04},
 {    9025, .8254048828125E+04, -.1438883178711E+04, .1056671264648E+04},
 {    9026, .8267108398438E+04, -.1446170776367E+04, .1050825073242E+04},
 {    9027, .8280941406250E+04, -.1451472412109E+04, .1044632202148E+04},
 {    9028, .8295298828125E+04, -.1454692260742E+04, .1038205322266E+04},
 {    9029, .8309919921875E+04, -.1455772094727E+04, .1031659423828E+04},
 {    9030, .8195837890625E+04, -.1345494140625E+04, .1051422363281E+04},
 {    9031, .8199129882812E+04, -.1359350952148E+04, .1049949218750E+04},
 {    9032, .8203974609375E+04, -.1372648193359E+04, .1047779418945E+04},
 {    9033, .8210300781250E+04, -.1385177246094E+04, .1044948120117E+04},
 {    9034, .8218006835938E+04, -.1396741333008E+04, .1041497680664E+04},
 {    9035, .8226974609375E+04, -.1407159179688E+04, .1037484130859E+04},
 {    9036, .8237057617188E+04, -.1416267211914E+04, .1032970336914E+04},
 {    9037, .8248102539062E+04, -.1423922363281E+04, .1028025878906E+04},
 {    9038, .8259933593750E+04, -.1430004760742E+04, .1022729736328E+04},
 {    9039, .8272363281250E+04, -.1434418823242E+04, .1017164733887E+04},
 {    9040, .8285202148438E+04, -.1437095214844E+04, .1011417480469E+04},
 {    9041, .8298244140625E+04, -.1437992065430E+04, .1005578613281E+04},
 {    9042, .8192784179688E+04, -.1346318969727E+04, .1052789672852E+04},
 {    9043, .8195578125000E+04, -.1361176269531E+04, .1051538452148E+04},
 {    9044, .8200112304688E+04, -.1375513183594E+04, .1049509643555E+04},
 {    9045, .8206305664062E+04, -.1389086059570E+04, .1046736083984E+04},
 {    9046, .8214061523438E+04, -.1401664672852E+04, .1043264892578E+04},
 {    9047, .8223239257812E+04, -.1413035156250E+04, .1039155273438E+04},
 {    9048, .8233690429688E+04, -.1423004394531E+04, .1034476806641E+04},
 {    9049, .8245239257812E+04, -.1431402832031E+04, .1029307739258E+04},
 {    9050, .8257681640625E+04, -.1438087768555E+04, .1023736938477E+04},
 {    9051, .8270815429688E+04, -.1442945556641E+04, .1017858154297E+04},
 {    9052, .8284414062500E+04, -.1445893798828E+04, .1011770996094E+04},
 {    9053, .8298244140625E+04, -.1446882080078E+04, .1005578613281E+04},
 {    9054, .8189727539062E+04, -.1347143798828E+04, .1054157226562E+04},
 {    9055, .8192025390625E+04, -.1363001586914E+04, .1053128906250E+04},
 {    9056, .8196246093750E+04, -.1378378051758E+04, .1051239501953E+04},
 {    9057, .8202312500000E+04, -.1392994995117E+04, .1048524169922E+04},
 {    9058, .8210112304688E+04, -.1406588012695E+04, .1045032226562E+04},
 {    9059, .8219506835938E+04, -.1418911132812E+04, .1040826416016E+04},
 {    9060, .8230326171875E+04, -.1429741577148E+04, .1035983520508E+04},
 {    9061, .8242375000000E+04, -.1438883178711E+04, .1030589599609E+04},
 {    9062, .8255431640625E+04, -.1446170776367E+04, .1024744140625E+04},
 {    9063, .8269264648438E+04, -.1451472412109E+04, .1018551574707E+04},
 {    9064, .8283624023438E+04, -.1454692260742E+04, .1012123901367E+04},
 {    9065, .8298244140625E+04, -.1455772094727E+04, .1005578613281E+04},
 {    9066, .8184162109375E+04, -.1345494140625E+04, .1025341796875E+04},
 {    9067, .8187451660156E+04, -.1359350952148E+04, .1023868286133E+04},
 {    9068, .8192301757812E+04, -.1372648193359E+04, .1021698852539E+04},
 {    9069, .8198627929688E+04, -.1385177246094E+04, .1018866394043E+04},
 {    9070, .8206333007812E+04, -.1396741333008E+04, .1015416748047E+04},
 {    9071, .8215297851562E+04, -.1407159179688E+04, .1011403381348E+04},
 {    9072, .8225379882812E+04, -.1416267211914E+04, .1006889160156E+04},
 {    9073, .8236424804688E+04, -.1423922363281E+04, .1001945373535E+04},
 {    9074, .8248258789062E+04, -.1430004760742E+04, .9966486816406E+03},
 {    9075, .8260687500000E+04, -.1434418823242E+04, .9910838623047E+03},
 {    9076, .8273526367188E+04, -.1437095214844E+04, .9853363647461E+03},
 {    9077, .8286571289062E+04, -.1437992065430E+04, .9794973144531E+03},
 {    9078, .8181107910156E+04, -.1346318969727E+04, .1026709472656E+04},
 {    9079, .8183900390625E+04, -.1361176269531E+04, .1025458251953E+04},
 {    9080, .8188434570312E+04, -.1375513183594E+04, .1023429077148E+04},
 {    9081, .8194631835938E+04, -.1389086059570E+04, .1020655395508E+04},
 {    9082, .8202384765625E+04, -.1401664672852E+04, .1017183593750E+04},
 {    9083, .8211561523438E+04, -.1413035156250E+04, .1013075073242E+04},
 {    9084, .8222015625000E+04, -.1423004394531E+04, .1008395996094E+04},
 {    9085, .8233563476562E+04, -.1431402832031E+04, .1003227172852E+04},
 {    9086, .8246005859375E+04, -.1438087768555E+04, .9976560668945E+03},
 {    9087, .8259139648438E+04, -.1442945556641E+04, .9917767944336E+03},
 {    9088, .8272739257812E+04, -.1445893798828E+04, .9856898803711E+03},
 {    9089, .8286571289062E+04, -.1446882080078E+04, .9794973144531E+03},
 {    9090, .8178052246094E+04, -.1347143798828E+04, .1028076782227E+04},
 {    9091, .8180352539062E+04, -.1363001586914E+04, .1027047973633E+04},
 {    9092, .8184569335938E+04, -.1378378051758E+04, .1025158691406E+04},
 {    9093, .8190636230469E+04, -.1392994995117E+04, .1022443603516E+04},
 {    9094, .8198437500000E+04, -.1406588012695E+04, .1018950988770E+04},
 {    9095, .8207832031250E+04, -.1418911132812E+04, .1014745971680E+04},
 {    9096, .8218648437500E+04, -.1429741577148E+04, .1009902954102E+04},
 {    9097, .8230698242188E+04, -.1438883178711E+04, .1004508483887E+04},
 {    9098, .8243757812500E+04, -.1446170776367E+04, .9986630859375E+03},
 {    9099, .8257590820312E+04, -.1451472412109E+04, .9924700927734E+03},
 {    9100, .8271948242188E+04, -.1454692260742E+04, .9860432128906E+03},
 {    9101, .8286571289062E+04, -.1455772094727E+04, .9794973144531E+03},
 {    9102, .8178971191406E+04, -.1345494140625E+04, .1013750610352E+04},
 {    9103, .8182264648438E+04, -.1359350952148E+04, .1012276367188E+04},
 {    9104, .8187110839844E+04, -.1372648193359E+04, .1010106567383E+04},
 {    9105, .8193436523438E+04, -.1385177246094E+04, .1007275146484E+04},
 {    9106, .8201144531250E+04, -.1396741333008E+04, .1003825317383E+04},
 {    9107, .8210108398438E+04, -.1407159179688E+04, .9998118286133E+03},
 {    9108, .8220194335938E+04, -.1416267211914E+04, .9952976074219E+03},
 {    9109, .8231237304688E+04, -.1423922363281E+04, .9903530273438E+03},
 {    9110, .8243068359375E+04, -.1430004760742E+04, .9850574951172E+03},
 {    9111, .8255500976562E+04, -.1434418823242E+04, .9794918823242E+03},
 {    9112, .8268338867188E+04, -.1437095214844E+04, .9737446289062E+03},
 {    9113, .8281380859375E+04, -.1437992065430E+04, .9679062500000E+03},
 {    9114, .8175918945312E+04, -.1346318969727E+04, .1015117431641E+04},
 {    9115, .8178712402344E+04, -.1361176269531E+04, .1013866821289E+04},
 {    9116, .8183246093750E+04, -.1375513183594E+04, .1011836914062E+04},
 {    9117, .8189440917969E+04, -.1389086059570E+04, .1009064086914E+04},
 {    9118, .8197195312500E+04, -.1401664672852E+04, .1005592407227E+04},
 {    9119, .8206377929688E+04, -.1413035156250E+04, .1001482727051E+04},
 {    9120, .8216827148438E+04, -.1423004394531E+04, .9968044433594E+03},
 {    9121, .8228374023438E+04, -.1431402832031E+04, .9916352539062E+03},
 {    9122, .8240817382812E+04, -.1438087768555E+04, .9860642089844E+03},
 {    9123, .8253950195312E+04, -.1442945556641E+04, .9801857910156E+03},
 {    9124, .8267547851562E+04, -.1445893798828E+04, .9740985717773E+03},
 {    9125, .8281380859375E+04, -.1446882080078E+04, .9679062500000E+03},
 {    9126, .8172863769531E+04, -.1347143798828E+04, .1016484985352E+04},
 {    9127, .8175162597656E+04, -.1363001586914E+04, .1015456298828E+04},
 {    9128, .8179382812500E+04, -.1378378051758E+04, .1013567077637E+04},
 {    9129, .8185448242188E+04, -.1392994995117E+04, .1010851440430E+04},
 {    9130, .8193247070312E+04, -.1406588012695E+04, .1007360046387E+04},
 {    9131, .8202644531250E+04, -.1418911132812E+04, .1003153686523E+04},
 {    9132, .8213463867188E+04, -.1429741577148E+04, .9983107910156E+03},
 {    9133, .8225509765625E+04, -.1438883178711E+04, .9929174804688E+03},
 {    9134, .8238565429688E+04, -.1446170776367E+04, .9870721435547E+03},
 {    9135, .8252401367188E+04, -.1451472412109E+04, .9808787841797E+03},
 {    9136, .8266759765625E+04, -.1454692260742E+04, .9744518432617E+03},
 {    9137, .8281380859375E+04, -.1455772094727E+04, .9679062500000E+03},
 {    9138, .8173783691406E+04, -.1345494140625E+04, .1002158935547E+04},
 {    9139, .8177074707031E+04, -.1359350952148E+04, .1000685058594E+04},
 {    9140, .8181922363281E+04, -.1372648193359E+04, .9985152587891E+03},
 {    9141, .8188246582031E+04, -.1385177246094E+04, .9956837158203E+03},
 {    9142, .8195955078125E+04, -.1396741333008E+04, .9922335205078E+03},
 {    9143, .8204917968750E+04, -.1407159179688E+04, .9882203369141E+03},
 {    9144, .8215002929688E+04, -.1416267211914E+04, .9837064819336E+03},
 {    9145, .8226046875000E+04, -.1423922363281E+04, .9787622680664E+03},
 {    9146, .8237876953125E+04, -.1430004760742E+04, .9734661254883E+03},
 {    9147, .8250309570312E+04, -.1434418823242E+04, .9679006347656E+03},
 {    9148, .8263148437500E+04, -.1437095214844E+04, .9621533203125E+03},
 {    9149, .8276192382812E+04, -.1437992065430E+04, .9563143310547E+03},
 {    9150, .8170728027344E+04, -.1346318969727E+04, .1003526245117E+04},
 {    9151, .8173523925781E+04, -.1361176269531E+04, .1002275146484E+04},
 {    9152, .8178056640625E+04, -.1375513183594E+04, .1000245727539E+04},
 {    9153, .8184254882812E+04, -.1389086059570E+04, .9974719238281E+03},
 {    9154, .8192005859375E+04, -.1401664672852E+04, .9940010375977E+03},
 {    9155, .8201186523438E+04, -.1413035156250E+04, .9898918457031E+03},
 {    9156, .8211635742188E+04, -.1423004394531E+04, .9852130737305E+03},
 {    9157, .8223183593750E+04, -.1431402832031E+04, .9800443725586E+03},
 {    9158, .8235626953125E+04, -.1438087768555E+04, .9744730224609E+03},
 {    9159, .8248761718750E+04, -.1442945556641E+04, .9685936279297E+03},
 {    9160, .8262359375000E+04, -.1445893798828E+04, .9625065917969E+03},
 {    9161, .8276192382812E+04, -.1446882080078E+04, .9563143310547E+03},
 {    9162, .8167674316406E+04, -.1347143798828E+04, .1004893432617E+04},
 {    9163, .8169973632812E+04, -.1363001586914E+04, .1003864746094E+04},
 {    9164, .8174192382812E+04, -.1378378051758E+04, .1001975708008E+04},
 {    9165, .8180256347656E+04, -.1392994995117E+04, .9992603759766E+03},
 {    9166, .8188059082031E+04, -.1406588012695E+04, .9957686767578E+03},
 {    9167, .8197452148438E+04, -.1418911132812E+04, .9915626220703E+03},
 {    9168, .8208273437500E+04, -.1429741577148E+04, .9867194213867E+03},
 {    9169, .8220318359375E+04, -.1438883178711E+04, .9813261718750E+03},
 {    9170, .8233379882812E+04, -.1446170776367E+04, .9754797363281E+03},
 {    9171, .8247211914062E+04, -.1451472412109E+04, .9692873535156E+03},
 {    9172, .8261571289062E+04, -.1454692260742E+04, .9628599853516E+03},
 {    9173, .8276192382812E+04, -.1455772094727E+04, .9563143310547E+03},
 {    9174, .8168592773438E+04, -.1345494140625E+04, .9905673217773E+03},
 {    9175, .8171883789062E+04, -.1359350952148E+04, .9890938720703E+03},
 {    9176, .8176731933594E+04, -.1372648193359E+04, .9869240722656E+03},
 {    9177, .8183058593750E+04, -.1385177246094E+04, .9840921630859E+03},
 {    9178, .8190762695312E+04, -.1396741333008E+04, .9806427612305E+03},
 {    9179, .8199730468750E+04, -.1407159179688E+04, .9766282958984E+03},
 {    9180, .8209815429688E+04, -.1416267211914E+04, .9721143798828E+03},
 {    9181, .8220859375000E+04, -.1423922363281E+04, .9671704101562E+03},
 {    9182, .8232691406250E+04, -.1430004760742E+04, .9618742065430E+03},
 {    9183, .8245122070312E+04, -.1434418823242E+04, .9563088378906E+03},
 {    9184, .8257960937500E+04, -.1437095214844E+04, .9505616455078E+03},
 {    9185, .8271002929688E+04, -.1437992065430E+04, .9447231445312E+03},
 {    9186, .8165538085938E+04, -.1346318969727E+04, .9919350585938E+03},
 {    9187, .8168333496094E+04, -.1361176269531E+04, .9906836547852E+03},
 {    9188, .8172869140625E+04, -.1375513183594E+04, .9886541748047E+03},
 {    9189, .8179063964844E+04, -.1389086059570E+04, .9858805541992E+03},
 {    9190, .8186817382812E+04, -.1401664672852E+04, .9824099121094E+03},
 {    9191, .8195996093750E+04, -.1413035156250E+04, .9782998046875E+03},
 {    9192, .8206447265625E+04, -.1423004394531E+04, .9736213378906E+03},
 {    9193, .8217996093750E+04, -.1431402832031E+04, .9684520263672E+03},
 {    9194, .8230437500000E+04, -.1438087768555E+04, .9628815917969E+03},
 {    9195, .8243571289062E+04, -.1442945556641E+04, .9570025634766E+03},
 {    9196, .8257170898438E+04, -.1445893798828E+04, .9509149780273E+03},
 {    9197, .8271002929688E+04, -.1446882080078E+04, .9447231445312E+03},
 {    9198, .8162482910156E+04, -.1347143798828E+04, .9933021850586E+03},
 {    9199, .8164782226562E+04, -.1363001586914E+04, .9922732543945E+03},
 {    9200, .8169003417969E+04, -.1378378051758E+04, .9903842163086E+03},
 {    9201, .8175068847656E+04, -.1392994995117E+04, .9876690063477E+03},
 {    9202, .8182869628906E+04, -.1406588012695E+04, .9841770019531E+03},
 {    9203, .8192263671875E+04, -.1418911132812E+04, .9799709472656E+03},
 {    9204, .8203082031250E+04, -.1429741577148E+04, .9751278076172E+03},
 {    9205, .8215132812500E+04, -.1438883178711E+04, .9697344970703E+03},
 {    9206, .8228189453125E+04, -.1446170776367E+04, .9638882446289E+03},
 {    9207, .8242023437500E+04, -.1451472412109E+04, .9576959228516E+03},
 {    9208, .8256380859375E+04, -.1454692260742E+04, .9512684936523E+03},
 {    9209, .8271002929688E+04, -.1455772094727E+04, .9447231445312E+03},
 {    9210, .8351462890625E+04, -.1437100830078E+04, .1089592285156E+04},
 {    9211, .8364262695312E+04, -.1434441162109E+04, .1083862670898E+04},
 {    9212, .8376659179688E+04, -.1430054443359E+04, .1078313110352E+04},
 {    9213, .8388461914062E+04, -.1424009033203E+04, .1073029541016E+04},
 {    9214, .8399481445312E+04, -.1416399414062E+04, .1068095703125E+04},
 {    9215, .8409551757812E+04, -.1407343994141E+04, .1063587768555E+04},
 {    9216, .8418514648438E+04, -.1396984130859E+04, .1059575561523E+04},
 {    9217, .8426226562500E+04, -.1385481323242E+04, .1056123291016E+04},
 {    9218, .8432572265625E+04, -.1373015014648E+04, .1053283203125E+04},
 {    9219, .8437447265625E+04, -.1359779541016E+04, .1051099975586E+04},
 {    9220, .8440783203125E+04, -.1345981323242E+04, .1049607666016E+04},
 {    9221, .8442519531250E+04, -.1331835571289E+04, .1048829833984E+04},
 {    9222, .8352287109375E+04, -.1445894409180E+04, .1089223022461E+04},
 {    9223, .8365878906250E+04, -.1442948364258E+04, .1083139038086E+04},
 {    9224, .8379006835938E+04, -.1438093994141E+04, .1077262207031E+04},
 {    9225, .8391444335938E+04, -.1431413818359E+04, .1071693603516E+04},
 {    9226, .8402988281250E+04, -.1423021240234E+04, .1066526733398E+04},
 {    9227, .8413436523438E+04, -.1413059204102E+04, .1061849243164E+04},
 {    9228, .8422613281250E+04, -.1401696777344E+04, .1057740844727E+04},
 {    9229, .8430366210938E+04, -.1389127075195E+04, .1054270385742E+04},
 {    9230, .8436559570312E+04, -.1375563720703E+04, .1051496948242E+04},
 {    9231, .8441096679688E+04, -.1361236938477E+04, .1049466796875E+04},
 {    9232, .8443895507812E+04, -.1346389892578E+04, .1048214233398E+04},
 {    9234, .8353111328125E+04, -.1454687988281E+04, .1088854492188E+04},
 {    9235, .8367495117188E+04, -.1451455566406E+04, .1082414916992E+04},
 {    9236, .8381351562500E+04, -.1446133544922E+04, .1076211669922E+04},
 {    9237, .8394430664062E+04, -.1438818481445E+04, .1070356933594E+04},
 {    9238, .8406491210938E+04, -.1429643188477E+04, .1064958129883E+04},
 {    9239, .8417319335938E+04, -.1418774414062E+04, .1060111328125E+04},
 {    9240, .8426710937500E+04, -.1406409423828E+04, .1055906616211E+04},
 {    9241, .8434502929688E+04, -.1392772827148E+04, .1052417968750E+04},
 {    9242, .8440551757812E+04, -.1378112426758E+04, .1049710693359E+04},
 {    9243, .8444743164062E+04, -.1362694335938E+04, .1047834228516E+04},
 {    9244, .8447007812500E+04, -.1346798461914E+04, .1046820800781E+04},
 {    9246, .8346273437500E+04, -.1437100830078E+04, .1078000732422E+04},
 {    9247, .8359072265625E+04, -.1434441162109E+04, .1072270751953E+04},
 {    9248, .8371469726562E+04, -.1430054443359E+04, .1066721191406E+04},
 {    9249, .8383271484375E+04, -.1424009033203E+04, .1061437988281E+04},
 {    9250, .8394292968750E+04, -.1416399414062E+04, .1056504272461E+04},
 {    9251, .8404362304688E+04, -.1407343994141E+04, .1051996582031E+04},
 {    9252, .8413324218750E+04, -.1396984130859E+04, .1047984497070E+04},
 {    9253, .8421038085938E+04, -.1385481323242E+04, .1044531738281E+04},
 {    9254, .8427382812500E+04, -.1373015014648E+04, .1041691284180E+04},
 {    9255, .8432258789062E+04, -.1359779541016E+04, .1039508544922E+04},
 {    9256, .8435592773438E+04, -.1345981323242E+04, .1038015991211E+04},
 {    9257, .8437331054688E+04, -.1331835571289E+04, .1037238403320E+04},
 {    9258, .8347099609375E+04, -.1445894409180E+04, .1077631835938E+04},
 {    9259, .8360690429688E+04, -.1442948364258E+04, .1071547241211E+04},
 {    9260, .8373817382812E+04, -.1438093994141E+04, .1065670410156E+04},
 {    9261, .8386255859375E+04, -.1431413818359E+04, .1060101928711E+04},
 {    9262, .8397796875000E+04, -.1423021240234E+04, .1054935546875E+04},
 {    9263, .8408247070312E+04, -.1413059204102E+04, .1050258056641E+04},
 {    9264, .8417425781250E+04, -.1401696777344E+04, .1046149658203E+04},
 {    9265, .8425175781250E+04, -.1389127075195E+04, .1042678955078E+04},
 {    9266, .8431372070312E+04, -.1375563720703E+04, .1039905761719E+04},
 {    9267, .8435906250000E+04, -.1361236938477E+04, .1037875488281E+04},
 {    9268, .8438704101562E+04, -.1346389892578E+04, .1036622558594E+04},
 {    9270, .8347920898438E+04, -.1454687988281E+04, .1077263183594E+04},
 {    9271, .8362308593750E+04, -.1451455566406E+04, .1070823242188E+04},
 {    9272, .8376162109375E+04, -.1446133544922E+04, .1064620605469E+04},
 {    9273, .8389241210938E+04, -.1438818481445E+04, .1058765991211E+04},
 {    9274, .8401302734375E+04, -.1429643188477E+04, .1053366455078E+04},
 {    9275, .8412127929688E+04, -.1418774414062E+04, .1048519775391E+04},
 {    9276, .8421524414062E+04, -.1406409423828E+04, .1044314453125E+04},
 {    9277, .8429314453125E+04, -.1392772827148E+04, .1040826660156E+04},
 {    9278, .8435361328125E+04, -.1378112426758E+04, .1038119506836E+04},
 {    9279, .8439553710938E+04, -.1362694335938E+04, .1036242187500E+04},
 {    9280, .8441818359375E+04, -.1346798461914E+04, .1035229003906E+04},
 {    9282, .8340435546875E+04, -.1437100830078E+04, .1064960205078E+04},
 {    9283, .8353233398438E+04, -.1434441162109E+04, .1059230712891E+04},
 {    9284, .8365632812500E+04, -.1430054443359E+04, .1053680908203E+04},
 {    9285, .8377434570312E+04, -.1424009033203E+04, .1048397705078E+04},
 {    9286, .8388456054688E+04, -.1416399414062E+04, .1043463867188E+04},
 {    9287, .8398525390625E+04, -.1407343994141E+04, .1038955810547E+04},
 {    9288, .8407487304688E+04, -.1396984130859E+04, .1034944824219E+04},
 {    9289, .8415199218750E+04, -.1385481323242E+04, .1031491210938E+04},
 {    9290, .8421544921875E+04, -.1373015014648E+04, .1028651123047E+04},
 {    9291, .8426421875000E+04, -.1359779541016E+04, .1026467773438E+04},
 {    9292, .8429753906250E+04, -.1345981323242E+04, .1024976074219E+04},
 {    9293, .8431495117188E+04, -.1331835571289E+04, .1024197265625E+04},
 {    9294, .8341260742188E+04, -.1445894409180E+04, .1064591308594E+04},
 {    9295, .8354852539062E+04, -.1442948364258E+04, .1058506835938E+04},
 {    9296, .8367979492188E+04, -.1438093994141E+04, .1052630371094E+04},
 {    9297, .8380418945312E+04, -.1431413818359E+04, .1047062011719E+04},
 {    9298, .8391962890625E+04, -.1423021240234E+04, .1041894531250E+04},
 {    9299, .8402410156250E+04, -.1413059204102E+04, .1037217529297E+04},
 {    9300, .8411587890625E+04, -.1401696777344E+04, .1033108642578E+04},
 {    9301, .8419339843750E+04, -.1389127075195E+04, .1029638061523E+04},
 {    9302, .8425534179688E+04, -.1375563720703E+04, .1026864990234E+04},
 {    9303, .8430068359375E+04, -.1361236938477E+04, .1024835449219E+04},
 {    9304, .8432869140625E+04, -.1346389892578E+04, .1023582275391E+04},
 {    9306, .8342083984375E+04, -.1454687988281E+04, .1064222045898E+04},
 {    9307, .8356469726562E+04, -.1451455566406E+04, .1057783203125E+04},
 {    9308, .8370324218750E+04, -.1446133544922E+04, .1051579833984E+04},
 {    9309, .8383405273438E+04, -.1438818481445E+04, .1045724609375E+04},
 {    9310, .8395464843750E+04, -.1429643188477E+04, .1040325805664E+04},
 {    9311, .8406290039062E+04, -.1418774414062E+04, .1035479370117E+04},
 {    9312, .8415686523438E+04, -.1406409423828E+04, .1031273437500E+04},
 {    9313, .8423476562500E+04, -.1392772827148E+04, .1027785888672E+04},
 {    9314, .8429523437500E+04, -.1378112426758E+04, .1025079589844E+04},
 {    9315, .8433716796875E+04, -.1362694335938E+04, .1023201904297E+04},
 {    9316, .8435979492188E+04, -.1346798461914E+04, .1022189270020E+04},
 {    9318, .8334597656250E+04, -.1437100830078E+04, .1051919677734E+04},
 {    9319, .8347397460938E+04, -.1434441162109E+04, .1046189697266E+04},
 {    9320, .8359794921875E+04, -.1430054443359E+04, .1040639892578E+04},
 {    9321, .8371597656250E+04, -.1424009033203E+04, .1035356933594E+04},
 {    9322, .8382617187500E+04, -.1416399414062E+04, .1030423217773E+04},
 {    9323, .8392688476562E+04, -.1407343994141E+04, .1025915039062E+04},
 {    9324, .8401648437500E+04, -.1396984130859E+04, .1021903747559E+04},
 {    9325, .8409361328125E+04, -.1385481323242E+04, .1018450866699E+04},
 {    9326, .8415706054688E+04, -.1373015014648E+04, .1015610839844E+04},
 {    9327, .8420583984375E+04, -.1359779541016E+04, .1013427124023E+04},
 {    9328, .8423917968750E+04, -.1345981323242E+04, .1011935180664E+04},
 {    9329, .8425655273438E+04, -.1331835571289E+04, .1011156982422E+04},
 {    9330, .8335421875000E+04, -.1445894409180E+04, .1051550781250E+04},
 {    9331, .8349012695312E+04, -.1442948364258E+04, .1045466430664E+04},
 {    9332, .8362143554688E+04, -.1438093994141E+04, .1039589355469E+04},
 {    9333, .8374580078125E+04, -.1431413818359E+04, .1034020996094E+04},
 {    9334, .8386122070312E+04, -.1423021240234E+04, .1028854003906E+04},
 {    9335, .8396570312500E+04, -.1413059204102E+04, .1024176635742E+04},
 {    9336, .8405749023438E+04, -.1401696777344E+04, .1020068725586E+04},
 {    9337, .8413500976562E+04, -.1389127075195E+04, .1016597839355E+04},
 {    9338, .8419696289062E+04, -.1375563720703E+04, .1013824218750E+04},
 {    9339, .8424233398438E+04, -.1361236938477E+04, .1011794311523E+04},
 {    9340, .8427029296875E+04, -.1346389892578E+04, .1010541625977E+04},
 {    9342, .8336246093750E+04, -.1454687988281E+04, .1051182006836E+04},
 {    9343, .8350630859375E+04, -.1451455566406E+04, .1044742919922E+04},
 {    9344, .8364488281250E+04, -.1446133544922E+04, .1038539184570E+04},
 {    9345, .8377564453125E+04, -.1438818481445E+04, .1032685058594E+04},
 {    9346, .8389626953125E+04, -.1429643188477E+04, .1027285156250E+04},
 {    9347, .8400454101562E+04, -.1418774414062E+04, .1022438598633E+04},
 {    9348, .8409847656250E+04, -.1406409423828E+04, .1018233276367E+04},
 {    9349, .8417638671875E+04, -.1392772827148E+04, .1014745544434E+04},
 {    9350, .8423685546875E+04, -.1378112426758E+04, .1012038696289E+04},
 {    9351, .8427877929688E+04, -.1362694335938E+04, .1010161621094E+04},
 {    9352, .8430143554688E+04, -.1346798461914E+04, .1009148254395E+04},
 {    9354, .8322922851562E+04, -.1437100830078E+04, .1025838623047E+04},
 {    9355, .8335721679688E+04, -.1434441162109E+04, .1020109558105E+04},
 {    9356, .8348119140625E+04, -.1430054443359E+04, .1014559631348E+04},
 {    9357, .8359921875000E+04, -.1424009033203E+04, .1009276367188E+04},
 {    9358, .8370941406250E+04, -.1416399414062E+04, .1004342224121E+04},
 {    9359, .8381010742188E+04, -.1407343994141E+04, .9998345947266E+03},
 {    9360, .8389973632812E+04, -.1396984130859E+04, .9958225708008E+03},
 {    9361, .8397687500000E+04, -.1385481323242E+04, .9923696899414E+03},
 {    9362, .8404031250000E+04, -.1373015014648E+04, .9895295410156E+03},
 {    9363, .8408909179688E+04, -.1359779541016E+04, .9873461914062E+03},
 {    9364, .8412242187500E+04, -.1345981323242E+04, .9858543701172E+03},
 {    9365, .8413980468750E+04, -.1331835571289E+04, .9850762329102E+03},
 {    9366, .8323746093750E+04, -.1445894409180E+04, .1025469970703E+04},
 {    9367, .8337338867188E+04, -.1442948364258E+04, .1019385620117E+04},
 {    9368, .8350466796875E+04, -.1438093994141E+04, .1013508666992E+04},
 {    9369, .8362904296875E+04, -.1431413818359E+04, .1007940185547E+04},
 {    9370, .8374447265625E+04, -.1423021240234E+04, .1002773193359E+04},
 {    9371, .8384894531250E+04, -.1413059204102E+04, .9980961914062E+03},
 {    9372, .8394074218750E+04, -.1401696777344E+04, .9939877319336E+03},
 {    9373, .8401825195312E+04, -.1389127075195E+04, .9905170898438E+03},
 {    9374, .8408020507812E+04, -.1375563720703E+04, .9877436523438E+03},
 {    9375, .8412557617188E+04, -.1361236938477E+04, .9857133789062E+03},
 {    9376, .8415354492188E+04, -.1346389892578E+04, .9844610595703E+03},
 {    9378, .8324572265625E+04, -.1454687988281E+04, .1025100830078E+04},
 {    9379, .8338956054688E+04, -.1451455566406E+04, .1018661499023E+04},
 {    9380, .8352812500000E+04, -.1446133544922E+04, .1012458374023E+04},
 {    9381, .8365890625000E+04, -.1438818481445E+04, .1006603393555E+04},
 {    9382, .8377950195312E+04, -.1429643188477E+04, .1001204223633E+04},
 {    9383, .8388777343750E+04, -.1418774414062E+04, .9963576660156E+03},
 {    9384, .8398172851562E+04, -.1406409423828E+04, .9921522827148E+03},
 {    9385, .8405962890625E+04, -.1392772827148E+04, .9886646728516E+03},
 {    9386, .8412010742188E+04, -.1378112426758E+04, .9859576416016E+03},
 {    9387, .8416203125000E+04, -.1362694335938E+04, .9840812377930E+03},
 {    9388, .8418466796875E+04, -.1346798461914E+04, .9830673217773E+03},
 {    9390, .8311246093750E+04, -.1437100830078E+04, .9997579345703E+03},
 {    9391, .8324047851562E+04, -.1434441162109E+04, .9940277099609E+03},
 {    9392, .8336444335938E+04, -.1430054443359E+04, .9884782104492E+03},
 {    9393, .8348246093750E+04, -.1424009033203E+04, .9831954345703E+03},
 {    9394, .8359264648438E+04, -.1416399414062E+04, .9782612304688E+03},
 {    9395, .8369336914062E+04, -.1407343994141E+04, .9737531738281E+03},
 {    9396, .8378296875000E+04, -.1396984130859E+04, .9697418212891E+03},
 {    9397, .8386011718750E+04, -.1385481323242E+04, .9662888183594E+03},
 {    9398, .8392356445312E+04, -.1373015014648E+04, .9634486083984E+03},
 {    9399, .8397233398438E+04, -.1359779541016E+04, .9612656250000E+03},
 {    9400, .8400567382812E+04, -.1345981323242E+04, .9597734375000E+03},
 {    9401, .8402303710938E+04, -.1331835571289E+04, .9589953613281E+03},
 {    9402, .8312072265625E+04, -.1445894409180E+04, .9993889160156E+03},
 {    9403, .8325664062500E+04, -.1442948364258E+04, .9933045043945E+03},
 {    9404, .8338791015625E+04, -.1438093994141E+04, .9874279785156E+03},
 {    9405, .8351228515625E+04, -.1431413818359E+04, .9818591918945E+03},
 {    9406, .8362770507812E+04, -.1423021240234E+04, .9766923217773E+03},
 {    9407, .8373218750000E+04, -.1413059204102E+04, .9720150146484E+03},
 {    9408, .8382398437500E+04, -.1401696777344E+04, .9679062500000E+03},
 {    9409, .8390149414062E+04, -.1389127075195E+04, .9644364013672E+03},
 {    9410, .8396344726562E+04, -.1375563720703E+04, .9616628417969E+03},
 {    9411, .8400880859375E+04, -.1361236938477E+04, .9596322631836E+03},
 {    9412, .8403679687500E+04, -.1346389892578E+04, .9583799438477E+03},
 {    9414, .8312897460938E+04, -.1454687988281E+04, .9990200195312E+03},
 {    9415, .8327279296875E+04, -.1451455688477E+04, .9925809936523E+03},
 {    9416, .8341136718750E+04, -.1446133544922E+04, .9863776855469E+03},
 {    9417, .8354214843750E+04, -.1438818481445E+04, .9805228881836E+03},
 {    9418, .8366276367188E+04, -.1429643188477E+04, .9751231689453E+03},
 {    9419, .8377102539062E+04, -.1418774414062E+04, .9702769775391E+03},
 {    9420, .8386497070312E+04, -.1406409423828E+04, .9660716552734E+03},
 {    9421, .8394289062500E+04, -.1392772949219E+04, .9625834960938E+03},
 {    9422, .8400333984375E+04, -.1378112426758E+04, .9598768310547E+03},
 {    9423, .8404528320312E+04, -.1362694335938E+04, .9579998779297E+03},
 {    9424, .8406791992188E+04, -.1346798461914E+04, .9569868774414E+03},
 {    9426, .8299573242188E+04, -.1437100830078E+04, .9736770019531E+03},
 {    9427, .8312372070312E+04, -.1434441162109E+04, .9679470825195E+03},
 {    9428, .8324768554688E+04, -.1430054443359E+04, .9623973388672E+03},
 {    9429, .8336569335938E+04, -.1424009033203E+04, .9571144409180E+03},
 {    9430, .8347588867188E+04, -.1416399414062E+04, .9521807250977E+03},
 {    9431, .8357660156250E+04, -.1407343994141E+04, .9476726684570E+03},
 {    9432, .8366622070312E+04, -.1396984130859E+04, .9436605834961E+03},
 {    9433, .8374337890625E+04, -.1385481323242E+04, .9402077026367E+03},
 {    9434, .8380680664062E+04, -.1373015014648E+04, .9373677978516E+03},
 {    9435, .8385557617188E+04, -.1359779541016E+04, .9351845703125E+03},
 {    9436, .8388891601562E+04, -.1345981323242E+04, .9336923828125E+03},
 {    9437, .8390627929688E+04, -.1331835571289E+04, .9329145507812E+03},
 {    9438, .8300396484375E+04, -.1445894409180E+04, .9733081054688E+03},
 {    9439, .8313987304688E+04, -.1442948364258E+04, .9672237548828E+03},
 {    9440, .8327114257812E+04, -.1438093994141E+04, .9613470458984E+03},
 {    9441, .8339556640625E+04, -.1431413818359E+04, .9557781372070E+03},
 {    9442, .8351095703125E+04, -.1423021240234E+04, .9506113281250E+03},
 {    9443, .8361543945312E+04, -.1413059204102E+04, .9459339599609E+03},
 {    9444, .8370722656250E+04, -.1401696777344E+04, .9418255615234E+03},
 {    9445, .8378474609375E+04, -.1389127075195E+04, .9383551635742E+03},
 {    9446, .8384670898438E+04, -.1375563720703E+04, .9355819091797E+03},
 {    9447, .8389206054688E+04, -.1361236938477E+04, .9335515136719E+03},
 {    9448, .8392002929688E+04, -.1346389892578E+04, .9322990722656E+03},
 {    9450, .8301220703125E+04, -.1454687988281E+04, .9729389648438E+03},
 {    9451, .8315604492188E+04, -.1451455688477E+04, .9664996337891E+03},
 {    9452, .8329462890625E+04, -.1446133544922E+04, .9602962646484E+03},
 {    9453, .8342539062500E+04, -.1438818481445E+04, .9544420166016E+03},
 {    9454, .8354600585938E+04, -.1429643188477E+04, .9490423583984E+03},
 {    9455, .8365427734375E+04, -.1418774414062E+04, .9441959838867E+03},
 {    9456, .8374823242188E+04, -.1406409423828E+04, .9399902954102E+03},
 {    9457, .8382613281250E+04, -.1392772949219E+04, .9365028076172E+03},
 {    9458, .8388660156250E+04, -.1378112426758E+04, .9337952880859E+03},
 {    9459, .8392850585938E+04, -.1362694335938E+04, .9319188842773E+03},
 {    9460, .8395116210938E+04, -.1346798461914E+04, .9309057617188E+03},
 {    9462, .8294381835938E+04, -.1437100830078E+04, .9620854492188E+03},
 {    9463, .8307182617188E+04, -.1434441162109E+04, .9563554077148E+03},
 {    9464, .8319578125000E+04, -.1430054443359E+04, .9508060913086E+03},
 {    9465, .8331381835938E+04, -.1424009033203E+04, .9455229492188E+03},
 {    9466, .8342403320312E+04, -.1416399414062E+04, .9405887451172E+03},
 {    9467, .8352471679688E+04, -.1407343994141E+04, .9360811767578E+03},
 {    9468, .8361432617188E+04, -.1396984130859E+04, .9320693359375E+03},
 {    9469, .8369147460938E+04, -.1385481323242E+04, .9286163330078E+03},
 {    9470, .8375492187500E+04, -.1373015014648E+04, .9257759399414E+03},
 {    9471, .8380369140625E+04, -.1359779541016E+04, .9235928955078E+03},
 {    9472, .8383703125000E+04, -.1345981323242E+04, .9221005859375E+03},
 {    9473, .8385440429688E+04, -.1331835571289E+04, .9213228759766E+03},
 {    9474, .8295207031250E+04, -.1445894409180E+04, .9617164306641E+03},
 {    9475, .8308798828125E+04, -.1442948364258E+04, .9556322631836E+03},
 {    9476, .8321926757812E+04, -.1438093994141E+04, .9497553710938E+03},
 {    9477, .8334363281250E+04, -.1431413818359E+04, .9441870117188E+03},
 {    9478, .8345906250000E+04, -.1423021240234E+04, .9390198364258E+03},
 {    9479, .8356356445312E+04, -.1413059204102E+04, .9343424682617E+03},
 {    9480, .8365534179688E+04, -.1401696777344E+04, .9302342529297E+03},
 {    9481, .8373285156250E+04, -.1389127075195E+04, .9267637939453E+03},
 {    9482, .8379480468750E+04, -.1375563720703E+04, .9239905395508E+03},
 {    9483, .8384015625000E+04, -.1361236938477E+04, .9219601440430E+03},
 {    9484, .8386814453125E+04, -.1346389892578E+04, .9207072753906E+03},
 {    9486, .8296032226562E+04, -.1454687988281E+04, .9613473510742E+03},
 {    9487, .8310416015625E+04, -.1451455688477E+04, .9549080200195E+03},
 {    9488, .8324271484375E+04, -.1446133544922E+04, .9487050781250E+03},
 {    9489, .8337351562500E+04, -.1438818481445E+04, .9428501586914E+03},
 {    9490, .8349412109375E+04, -.1429643188477E+04, .9374510498047E+03},
 {    9491, .8360238281250E+04, -.1418774414062E+04, .9326045532227E+03},
 {    9492, .8369631835938E+04, -.1406409423828E+04, .9283994140625E+03},
 {    9493, .8377423828125E+04, -.1392772949219E+04, .9249114379883E+03},
 {    9494, .8383468750000E+04, -.1378112426758E+04, .9222044677734E+03},
 {    9495, .8387664062500E+04, -.1362694335938E+04, .9203271484375E+03},
 {    9496, .8389927734375E+04, -.1346798461914E+04, .9193142089844E+03},
 {    9498, .8289193359375E+04, -.1437100830078E+04, .9504938964844E+03},
 {    9499, .8301991210938E+04, -.1434441162109E+04, .9447644042969E+03},
 {    9500, .8314392578125E+04, -.1430054443359E+04, .9392141113281E+03},
 {    9501, .8326192382812E+04, -.1424009033203E+04, .9339314575195E+03},
 {    9502, .8337213867188E+04, -.1416399414062E+04, .9289973144531E+03},
 {    9503, .8347285156250E+04, -.1407343994141E+04, .9244891967773E+03},
 {    9504, .8356245117188E+04, -.1396984130859E+04, .9204776000977E+03},
 {    9505, .8363958984375E+04, -.1385481323242E+04, .9170246582031E+03},
 {    9506, .8370301757812E+04, -.1373015014648E+04, .9141845703125E+03},
 {    9507, .8375179687500E+04, -.1359779541016E+04, .9120015869141E+03},
 {    9508, .8378514648438E+04, -.1345981323242E+04, .9105090942383E+03},
 {    9509, .8380252929688E+04, -.1331835571289E+04, .9097310180664E+03},
 {    9510, .8290018554688E+04, -.1445894409180E+04, .9501248779297E+03},
 {    9511, .8303610351562E+04, -.1442948364258E+04, .9440401611328E+03},
 {    9512, .8316737304688E+04, -.1438093994141E+04, .9381639404297E+03},
 {    9513, .8329175781250E+04, -.1431413818359E+04, .9325953979492E+03},
 {    9514, .8340718750000E+04, -.1423021240234E+04, .9274283447266E+03},
 {    9515, .8351166992188E+04, -.1413059204102E+04, .9227509765625E+03},
 {    9516, .8360343750000E+04, -.1401696777344E+04, .9186427612305E+03},
 {    9517, .8368096679688E+04, -.1389127075195E+04, .9151721801758E+03},
 {    9518, .8374292968750E+04, -.1375563720703E+04, .9123985595703E+03},
 {    9519, .8378827148438E+04, -.1361236938477E+04, .9103685302734E+03},
 {    9520, .8381623046875E+04, -.1346389892578E+04, .9091159667969E+03},
 {    9522, .8290840820312E+04, -.1454687988281E+04, .9497559814453E+03},
 {    9523, .8305226562500E+04, -.1451455688477E+04, .9433165893555E+03},
 {    9524, .8319083984375E+04, -.1446133544922E+04, .9371134033203E+03},
 {    9525, .8332161132812E+04, -.1438818481445E+04, .9312589111328E+03},
 {    9526, .8344222656250E+04, -.1429643188477E+04, .9258593139648E+03},
 {    9527, .8355051757812E+04, -.1418774414062E+04, .9210127563477E+03},
 {    9528, .8364443359375E+04, -.1406409423828E+04, .9168078002930E+03},
 {    9529, .8372234375000E+04, -.1392772949219E+04, .9133197631836E+03},
 {    9530, .8378279296875E+04, -.1378112426758E+04, .9106125488281E+03},
 {    9531, .8382474609375E+04, -.1362694335938E+04, .9087354125977E+03},
 {    9532, .8384737304688E+04, -.1346798461914E+04, .9077226562500E+03},
 {    9534, .8284005859375E+04, -.1437100830078E+04, .9389019775391E+03},
 {    9535, .8296804687500E+04, -.1434441162109E+04, .9331726074219E+03},
 {    9536, .8309201171875E+04, -.1430054443359E+04, .9276229248047E+03},
 {    9537, .8321003906250E+04, -.1424009033203E+04, .9223397216797E+03},
 {    9538, .8332024414062E+04, -.1416399414062E+04, .9174059448242E+03},
 {    9539, .8342094726562E+04, -.1407343994141E+04, .9128980102539E+03},
 {    9540, .8351055664062E+04, -.1396984130859E+04, .9088862304688E+03},
 {    9541, .8358769531250E+04, -.1385481323242E+04, .9054331665039E+03},
 {    9542, .8365113281250E+04, -.1373015014648E+04, .9025930175781E+03},
 {    9543, .8369991210938E+04, -.1359779541016E+04, .9004100341797E+03},
 {    9544, .8373323242188E+04, -.1345981323242E+04, .8989176025391E+03},
 {    9545, .8375061523438E+04, -.1331835571289E+04, .8981396484375E+03},
 {    9546, .8284829101562E+04, -.1445894409180E+04, .9385336914062E+03},
 {    9547, .8298420898438E+04, -.1442948364258E+04, .9324487915039E+03},
 {    9548, .8311546875000E+04, -.1438093994141E+04, .9265722656250E+03},
 {    9549, .8323988281250E+04, -.1431413818359E+04, .9210035400391E+03},
 {    9550, .8335528320312E+04, -.1423021240234E+04, .9158368530273E+03},
 {    9551, .8345976562500E+04, -.1413059204102E+04, .9111595458984E+03},
 {    9552, .8355155273438E+04, -.1401696777344E+04, .9070510253906E+03},
 {    9553, .8362907226562E+04, -.1389127075195E+04, .9035806884766E+03},
 {    9554, .8369101562500E+04, -.1375563720703E+04, .9008073730469E+03},
 {    9555, .8373636718750E+04, -.1361236938477E+04, .8987772216797E+03},
 {    9556, .8376435546875E+04, -.1346389892578E+04, .8975242919922E+03},
 {    9558, .8285652343750E+04, -.1454687988281E+04, .9381646118164E+03},
 {    9559, .8300037109375E+04, -.1451455688477E+04, .9317251586914E+03},
 {    9560, .8313894531250E+04, -.1446133544922E+04, .9255218505859E+03},
 {    9561, .8326972656250E+04, -.1438818481445E+04, .9196674194336E+03},
 {    9562, .8339037109375E+04, -.1429643188477E+04, .9142673950195E+03},
 {    9563, .8349859375000E+04, -.1418774414062E+04, .9094213867188E+03},
 {    9564, .8359255859375E+04, -.1406409423828E+04, .9052159423828E+03},
 {    9565, .8367047851562E+04, -.1392772949219E+04, .9017278442383E+03},
 {    9566, .8373092773438E+04, -.1378112426758E+04, .8990209350586E+03},
 {    9567, .8377287109375E+04, -.1362694335938E+04, .8971441040039E+03},
 {    9568, .8379548828125E+04, -.1346798461914E+04, .8961311035156E+03},
 {    9570, .8419711914062E+04, -.1202382934570E+04, .1045124511719E+04},
 {    9572, .8422875976562E+04, -.1200798950195E+04, .1043709350586E+04},
 {    9573, .8418489257812E+04, -.1191932617188E+04, .1045672363281E+04},
 {    9574, .8414524414062E+04, -.1202382934570E+04, .1033533081055E+04},
 {    9576, .8417685546875E+04, -.1200798950195E+04, .1032117431641E+04},
 {    9577, .8413298828125E+04, -.1191932617188E+04, .1034080688477E+04},
 {    9578, .8409334960938E+04, -.1202382934570E+04, .1021941223145E+04},
 {    9580, .8412496093750E+04, -.1200798950195E+04, .1020526123047E+04},
 {    9581, .8408112304688E+04, -.1191932617188E+04, .1022489257812E+04},
 {    9582, .8426039062500E+04, -.1199214843750E+04, .1042293090820E+04},
 {    9583, .8421426757812E+04, -.1189894165039E+04, .1044357177734E+04},
 {    9584, .8420850585938E+04, -.1199214843750E+04, .1030701538086E+04},
 {    9585, .8416237304688E+04, -.1189894165039E+04, .1032766357422E+04},
 {    9586, .8415661132812E+04, -.1199214843750E+04, .1019110229492E+04},
 {    9587, .8411046875000E+04, -.1189894165039E+04, .1021174560547E+04},
 {    9591, .8371952148438E+04, -.1154726684570E+04, .1066505126953E+04},
 {    9594, .8408650390625E+04, -.1177040649414E+04, .1050076171875E+04},
 {    9595, .8398184570312E+04, -.1166256225586E+04, .1054761962891E+04},
 {    9596, .8386220703125E+04, -.1157537475586E+04, .1060117797852E+04},
 {    9597, .8373097656250E+04, -.1151129028320E+04, .1065992675781E+04},
 {    9598, .8357102539062E+04, -.1147019042969E+04, .1073153442383E+04},
 {    9599, .8344863281250E+04, -.1145892089844E+04, .1078632080078E+04},
 {    9603, .8366762695312E+04, -.1154726684570E+04, .1054913818359E+04},
 {    9606, .8403463867188E+04, -.1177040649414E+04, .1038485107422E+04},
 {    9607, .8392996093750E+04, -.1166256225586E+04, .1043170410156E+04},
 {    9608, .8381033203125E+04, -.1157537475586E+04, .1048526123047E+04},
 {    9609, .8367907226562E+04, -.1151129028320E+04, .1054401123047E+04},
 {    9610, .8351912109375E+04, -.1147019042969E+04, .1061562011719E+04},
 {    9611, .8339673828125E+04, -.1145892089844E+04, .1067040649414E+04},
 {    9615, .8361575195312E+04, -.1154726684570E+04, .1043322509766E+04},
 {    9618, .8398273437500E+04, -.1177040649414E+04, .1026893554688E+04},
 {    9619, .8387805664062E+04, -.1166256225586E+04, .1031579345703E+04},
 {    9620, .8375843750000E+04, -.1157537475586E+04, .1036934936523E+04},
 {    9621, .8362716796875E+04, -.1151129028320E+04, .1042809814453E+04},
 {    9622, .8346723632812E+04, -.1147019042969E+04, .1049970336914E+04},
 {    9623, .8334484375000E+04, -.1145892089844E+04, .1055449218750E+04},
 {    9624, .8411239257812E+04, -.1174493408203E+04, .1048919067383E+04},
 {    9625, .8400345703125E+04, -.1163271850586E+04, .1053794921875E+04},
 {    9626, .8387897460938E+04, -.1154199584961E+04, .1059367309570E+04},
 {    9627, .8374240234375E+04, -.1147531250000E+04, .1065480712891E+04},
 {    9628, .8357681640625E+04, -.1143262573242E+04, .1072893432617E+04},
 {    9629, .8344863281250E+04, -.1142082031250E+04, .1078632080078E+04},
 {    9630, .8406048828125E+04, -.1174493408203E+04, .1037327148438E+04},
 {    9631, .8395157226562E+04, -.1163271850586E+04, .1042202636719E+04},
 {    9632, .8382708007812E+04, -.1154199584961E+04, .1047775634766E+04},
 {    9633, .8369051757812E+04, -.1147531250000E+04, .1053889038086E+04},
 {    9634, .8352493164062E+04, -.1143262573242E+04, .1061302001953E+04},
 {    9635, .8339673828125E+04, -.1142082031250E+04, .1067040649414E+04},
 {    9636, .8400859375000E+04, -.1174493408203E+04, .1025735229492E+04},
 {    9637, .8389969726562E+04, -.1163271850586E+04, .1030611328125E+04},
 {    9638, .8377520507812E+04, -.1154199584961E+04, .1036184204102E+04},
 {    9639, .8363862304688E+04, -.1147531250000E+04, .1042297729492E+04},
 {    9640, .8347303710938E+04, -.1143262573242E+04, .1049710937500E+04},
 {    9641, .8334484375000E+04, -.1142082031250E+04, .1055449218750E+04},
 {    9644, .8333272460938E+04, -.1145892089844E+04, .1083821044922E+04},
 {    9645, .8321680664062E+04, -.1145892089844E+04, .1089010131836E+04},
 {    9648, .8328083007812E+04, -.1145892089844E+04, .1072229858398E+04},
 {    9649, .8316491210938E+04, -.1145892089844E+04, .1077418945312E+04},
 {    9652, .8322892578125E+04, -.1145892089844E+04, .1060638183594E+04},
 {    9653, .8311302734375E+04, -.1145892089844E+04, .1065827270508E+04},
 {    9654, .8333272460938E+04, -.1142082031250E+04, .1083821044922E+04},
 {    9655, .8321680664062E+04, -.1142082031250E+04, .1089010131836E+04},
 {    9656, .8328083007812E+04, -.1142082031250E+04, .1072229858398E+04},
 {    9657, .8316491210938E+04, -.1142082031250E+04, .1077418945312E+04},
 {    9658, .8322892578125E+04, -.1142082031250E+04, .1060638183594E+04},
 {    9659, .8311302734375E+04, -.1142082031250E+04, .1065827270508E+04},
 {    9665, .8242452148438E+04, -.1215458862305E+04, .1124476562500E+04},
 {    9666, .8309171875000E+04, -.1146974853516E+04, .1094610229492E+04},
 {    9667, .8280581054688E+04, -.1157930908203E+04, .1107408813477E+04},
 {    9668, .8269883789062E+04, -.1164960327148E+04, .1112197265625E+04},
 {    9669, .8256430664062E+04, -.1178871582031E+04, .1118221191406E+04},
 {    9670, .8246790039062E+04, -.1194361083984E+04, .1122535522461E+04},
 {    9671, .8239105468750E+04, -.1214427368164E+04, .1125975463867E+04},
 {    9677, .8237266601562E+04, -.1215458862305E+04, .1112884887695E+04},
 {    9678, .8303983398438E+04, -.1146974853516E+04, .1083018310547E+04},
 {    9679, .8275394531250E+04, -.1157930908203E+04, .1095816894531E+04},
 {    9680, .8264694335938E+04, -.1164960327148E+04, .1100606445312E+04},
 {    9681, .8251239257812E+04, -.1178871582031E+04, .1106629882812E+04},
 {    9682, .8241603515625E+04, -.1194361083984E+04, .1110943847656E+04},
 {    9683, .8233917968750E+04, -.1214427368164E+04, .1114383789062E+04},
 {    9689, .8232075195312E+04, -.1215458862305E+04, .1101293945312E+04},
 {    9690, .8298792968750E+04, -.1146974853516E+04, .1071427246094E+04},
 {    9691, .8270203125000E+04, -.1157930908203E+04, .1084225830078E+04},
 {    9692, .8259505859375E+04, -.1164960327148E+04, .1089014404297E+04},
 {    9693, .8246049804688E+04, -.1178871582031E+04, .1095038452148E+04},
 {    9694, .8236412109375E+04, -.1194361083984E+04, .1099352905273E+04},
 {    9695, .8228728515625E+04, -.1214427368164E+04, .1102792358398E+04},
 {    9696, .8308424804688E+04, -.1143253540039E+04, .1094944213867E+04},
 {    9697, .8279124023438E+04, -.1154471313477E+04, .1108061279297E+04},
 {    9698, .8267785156250E+04, -.1161923339844E+04, .1113137695312E+04},
 {    9699, .8253783203125E+04, -.1176398559570E+04, .1119405517578E+04},
 {    9700, .8243722656250E+04, -.1192567016602E+04, .1123909423828E+04},
 {    9701, .8235757812500E+04, -.1213395874023E+04, .1127474243164E+04},
 {    9702, .8303238281250E+04, -.1143253540039E+04, .1083352416992E+04},
 {    9703, .8273935546875E+04, -.1154471313477E+04, .1096469238281E+04},
 {    9704, .8262595703125E+04, -.1161923339844E+04, .1101545898438E+04},
 {    9705, .8248590820312E+04, -.1176398559570E+04, .1107814697266E+04},
 {    9706, .8238535156250E+04, -.1192567016602E+04, .1112317138672E+04},
 {    9707, .8230570312500E+04, -.1213395874023E+04, .1115882690430E+04},
 {    9708, .8298047851562E+04, -.1143253540039E+04, .1071760742188E+04},
 {    9709, .8268746093750E+04, -.1154471313477E+04, .1084878051758E+04},
 {    9710, .8257405273438E+04, -.1161923339844E+04, .1089954956055E+04},
 {    9711, .8243403320312E+04, -.1176398559570E+04, .1096222167969E+04},
 {    9712, .8233343750000E+04, -.1192567016602E+04, .1100726318359E+04},
 {    9713, .8225382812500E+04, -.1213395874023E+04, .1104290893555E+04},
 {    9714, .8239856445312E+04, -.1230180419922E+04, .1125639648438E+04},
 {    9715, .8236400390625E+04, -.1229770874023E+04, .1127187011719E+04},
 {    9716, .8234666015625E+04, -.1230180419922E+04, .1114048095703E+04},
 {    9717, .8231211914062E+04, -.1229770874023E+04, .1115595214844E+04},
 {    9718, .8229478515625E+04, -.1230180419922E+04, .1102456420898E+04},
 {    9719, .8226023437500E+04, -.1229770874023E+04, .1104003784180E+04},
 {    9720, .8232941406250E+04, -.1229361450195E+04, .1128734863281E+04},
 {    9721, .8227752929688E+04, -.1229361450195E+04, .1117143188477E+04},
 {    9722, .8222564453125E+04, -.1229361450195E+04, .1105552001953E+04},
 {    9723, .8408037109375E+04, -.1202382934570E+04, .1019044189453E+04},
 {    9725, .8411199218750E+04, -.1200798950195E+04, .1017627929688E+04},
 {    9726, .8406813476562E+04, -.1191932617188E+04, .1019591613770E+04},
 {    9727, .8402847656250E+04, -.1202382934570E+04, .1007452026367E+04},
 {    9729, .8406011718750E+04, -.1200798950195E+04, .1006036499023E+04},
 {    9730, .8401625000000E+04, -.1191932617188E+04, .1007999633789E+04},
 {    9731, .8397659179688E+04, -.1202382934570E+04, .9958609008789E+03},
 {    9733, .8400821289062E+04, -.1200798950195E+04, .9944456176758E+03},
 {    9734, .8396436523438E+04, -.1191932617188E+04, .9964083251953E+03},
 {    9735, .8414363281250E+04, -.1199214843750E+04, .1016212158203E+04},
 {    9736, .8409751953125E+04, -.1189894165039E+04, .1018276428223E+04},
 {    9737, .8409173828125E+04, -.1199214843750E+04, .1004620971680E+04},
 {    9738, .8404561523438E+04, -.1189894165039E+04, .1006684326172E+04},
 {    9739, .8403985351562E+04, -.1199214843750E+04, .9930291748047E+03},
 {    9740, .8399372070312E+04, -.1189894165039E+04, .9950935668945E+03},
 {    9744, .8360277343750E+04, -.1154726684570E+04, .1040424072266E+04},
 {    9747, .8396976562500E+04, -.1177040649414E+04, .1023995361328E+04},
 {    9748, .8386509765625E+04, -.1166256225586E+04, .1028680908203E+04},
 {    9749, .8374545898438E+04, -.1157537475586E+04, .1034036621094E+04},
 {    9750, .8361422851562E+04, -.1151129028320E+04, .1039911987305E+04},
 {    9751, .8345427734375E+04, -.1147019042969E+04, .1047071655273E+04},
 {    9752, .8333187500000E+04, -.1145892089844E+04, .1052551025391E+04},
 {    9756, .8355086914062E+04, -.1154726684570E+04, .1028832397461E+04},
 {    9759, .8391787109375E+04, -.1177040649414E+04, .1012403808594E+04},
 {    9760, .8381321289062E+04, -.1166256225586E+04, .1017090332031E+04},
 {    9761, .8369357421875E+04, -.1157537475586E+04, .1022445068359E+04},
 {    9762, .8356234375000E+04, -.1151129028320E+04, .1028320556641E+04},
 {    9763, .8340237304688E+04, -.1147019042969E+04, .1035481201172E+04},
 {    9764, .8327998046875E+04, -.1145892089844E+04, .1040959594727E+04},
 {    9768, .8349899414062E+04, -.1154726684570E+04, .1017241699219E+04},
 {    9771, .8386598632812E+04, -.1177040649414E+04, .1000812438965E+04},
 {    9772, .8376132812500E+04, -.1166256225586E+04, .1005497802734E+04},
 {    9773, .8364167968750E+04, -.1157537475586E+04, .1010854064941E+04},
 {    9774, .8351043945312E+04, -.1151129028320E+04, .1016728881836E+04},
 {    9775, .8335046875000E+04, -.1147019042969E+04, .1023889282227E+04},
 {    9776, .8322810546875E+04, -.1145892089844E+04, .1029367919922E+04},
 {    9777, .8399562500000E+04, -.1174493408203E+04, .1022837768555E+04},
 {    9778, .8388670898438E+04, -.1163271850586E+04, .1027713623047E+04},
 {    9779, .8376223632812E+04, -.1154199584961E+04, .1033285888672E+04},
 {    9780, .8362566406250E+04, -.1147531250000E+04, .1039399536133E+04},
 {    9781, .8346006835938E+04, -.1143262573242E+04, .1046812500000E+04},
 {    9782, .8333187500000E+04, -.1142082031250E+04, .1052551025391E+04},
 {    9783, .8394375000000E+04, -.1174493408203E+04, .1011246093750E+04},
 {    9784, .8383481445312E+04, -.1163271850586E+04, .1016121948242E+04},
 {    9785, .8371033203125E+04, -.1154199584961E+04, .1021694824219E+04},
 {    9786, .8357376953125E+04, -.1147531250000E+04, .1027807617188E+04},
 {    9787, .8340817382812E+04, -.1143262573242E+04, .1035220825195E+04},
 {    9788, .8327998046875E+04, -.1142082031250E+04, .1040959594727E+04},
 {    9789, .8389184570312E+04, -.1174493408203E+04, .9996542968750E+03},
 {    9790, .8378293945312E+04, -.1163271850586E+04, .1004530456543E+04},
 {    9791, .8365845703125E+04, -.1154199584961E+04, .1010103088379E+04},
 {    9792, .8352188476562E+04, -.1147531250000E+04, .1016216796875E+04},
 {    9793, .8335628906250E+04, -.1143262573242E+04, .1023629333496E+04},
 {    9794, .8322810546875E+04, -.1142082031250E+04, .1029367919922E+04},
 {    9797, .8321595703125E+04, -.1145892089844E+04, .1057740356445E+04},
 {    9798, .8310006835938E+04, -.1145892089844E+04, .1062929321289E+04},
 {    9801, .8316409179688E+04, -.1145892089844E+04, .1046148437500E+04},
 {    9802, .8304816406250E+04, -.1145892089844E+04, .1051337646484E+04},
 {    9805, .8311216796875E+04, -.1145892089844E+04, .1034557128906E+04},
 {    9806, .8299626953125E+04, -.1145892089844E+04, .1039746093750E+04},
 {    9807, .8321595703125E+04, -.1142082031250E+04, .1057740356445E+04},
 {    9808, .8310006835938E+04, -.1142082031250E+04, .1062929321289E+04},
 {    9809, .8316409179688E+04, -.1142082031250E+04, .1046148437500E+04},
 {    9810, .8304816406250E+04, -.1142082031250E+04, .1051337646484E+04},
 {    9811, .8311216796875E+04, -.1142082031250E+04, .1034557128906E+04},
 {    9812, .8299626953125E+04, -.1142082031250E+04, .1039746093750E+04},
 {    9818, .8230775390625E+04, -.1215458862305E+04, .1098395996094E+04},
 {    9819, .8297496093750E+04, -.1146974853516E+04, .1068529052734E+04},
 {    9820, .8268904296875E+04, -.1157930908203E+04, .1081328125000E+04},
 {    9821, .8258208984375E+04, -.1164960327148E+04, .1086116455078E+04},
 {    9822, .8244751953125E+04, -.1178871582031E+04, .1092140747070E+04},
 {    9823, .8235114257812E+04, -.1194361083984E+04, .1096454833984E+04},
 {    9824, .8227430664062E+04, -.1214427368164E+04, .1099894531250E+04},
 {    9830, .8225588867188E+04, -.1215458984375E+04, .1086804809570E+04},
 {    9831, .8292307617188E+04, -.1146974853516E+04, .1056937500000E+04},
 {    9832, .8263715820312E+04, -.1157930908203E+04, .1069736572266E+04},
 {    9833, .8253019531250E+04, -.1164960327148E+04, .1074525024414E+04},
 {    9834, .8239565429688E+04, -.1178871582031E+04, .1080548339844E+04},
 {    9835, .8229926757812E+04, -.1194361083984E+04, .1084863403320E+04},
 {    9836, .8222243164062E+04, -.1214427368164E+04, .1088302856445E+04},
 {    9842, .8220398437500E+04, -.1215458984375E+04, .1075213012695E+04},
 {    9843, .8287117187500E+04, -.1146974853516E+04, .1045346069336E+04},
 {    9844, .8258527343750E+04, -.1157930908203E+04, .1058144653320E+04},
 {    9845, .8247830078125E+04, -.1164960327148E+04, .1062933105469E+04},
 {    9846, .8234373046875E+04, -.1178871582031E+04, .1068957519531E+04},
 {    9847, .8224735351562E+04, -.1194361083984E+04, .1073271606445E+04},
 {    9848, .8217052734375E+04, -.1214427368164E+04, .1076711547852E+04},
 {    9849, .8296750000000E+04, -.1143253540039E+04, .1068862792969E+04},
 {    9850, .8267450195312E+04, -.1154471313477E+04, .1081979858398E+04},
 {    9851, .8256109375000E+04, -.1161923339844E+04, .1087056518555E+04},
 {    9852, .8242104492188E+04, -.1176398559570E+04, .1093324829102E+04},
 {    9853, .8232045898438E+04, -.1192567016602E+04, .1097827758789E+04},
 {    9854, .8224082031250E+04, -.1213395874023E+04, .1101393310547E+04},
 {    9855, .8291560546875E+04, -.1143253540039E+04, .1057271240234E+04},
 {    9856, .8262259765625E+04, -.1154471313477E+04, .1070388061523E+04},
 {    9857, .8250919921875E+04, -.1161923339844E+04, .1075464965820E+04},
 {    9858, .8236916015625E+04, -.1176398559570E+04, .1081733520508E+04},
 {    9859, .8226856445312E+04, -.1192567016602E+04, .1086236816406E+04},
 {    9860, .8218893554688E+04, -.1213395874023E+04, .1089802124023E+04},
 {    9861, .8286373046875E+04, -.1143253540039E+04, .1045679931641E+04},
 {    9862, .8257071289062E+04, -.1154471313477E+04, .1058797363281E+04},
 {    9863, .8245730468750E+04, -.1161923339844E+04, .1063873535156E+04},
 {    9864, .8231725585938E+04, -.1176398559570E+04, .1070142089844E+04},
 {    9865, .8221668945312E+04, -.1192567138672E+04, .1074645629883E+04},
 {    9866, .8213703125000E+04, -.1213395874023E+04, .1078210205078E+04},
 {    9867, .8228183593750E+04, -.1230180419922E+04, .1099558349609E+04},
 {    9868, .8224724609375E+04, -.1229770874023E+04, .1101106811523E+04},
 {    9869, .8222994140625E+04, -.1230180419922E+04, .1087966308594E+04},
 {    9870, .8219534179688E+04, -.1229770874023E+04, .1089515014648E+04},
 {    9871, .8217803710938E+04, -.1230180419922E+04, .1076375488281E+04},
 {    9872, .8214348632812E+04, -.1229770874023E+04, .1077922607422E+04},
 {    9873, .8221268554688E+04, -.1229361450195E+04, .1102653930664E+04},
 {    9874, .8216077148438E+04, -.1229361450195E+04, .1091062622070E+04},
 {    9875, .8210889648438E+04, -.1229361450195E+04, .1079470336914E+04},
 {    9876, .8396362304688E+04, -.1202382934570E+04, .9929628906250E+03},
 {    9878, .8399525390625E+04, -.1200798950195E+04, .9915468750000E+03},
 {    9879, .8395138671875E+04, -.1191932617188E+04, .9935108642578E+03},
 {    9880, .8391173828125E+04, -.1202382934570E+04, .9813710327148E+03},
 {    9882, .8394333984375E+04, -.1200798950195E+04, .9799558105469E+03},
 {    9883, .8389948242188E+04, -.1191932617188E+04, .9819192504883E+03},
 {    9884, .8385983398438E+04, -.1202382934570E+04, .9697799072266E+03},
 {    9886, .8389146484375E+04, -.1200798950195E+04, .9683641357422E+03},
 {    9887, .8384760742188E+04, -.1191932617188E+04, .9703276367188E+03},
 {    9888, .8402686523438E+04, -.1199214843750E+04, .9901316528320E+03},
 {    9889, .8398077148438E+04, -.1189894165039E+04, .9921954345703E+03},
 {    9890, .8397497070312E+04, -.1199214843750E+04, .9785399780273E+03},
 {    9891, .8392886718750E+04, -.1189894165039E+04, .9806035766602E+03},
 {    9892, .8392309570312E+04, -.1199214843750E+04, .9669484252930E+03},
 {    9893, .8387697265625E+04, -.1189894165039E+04, .9690126953125E+03},
 {    9897, .8348600585938E+04, -.1154726684570E+04, .1014343322754E+04},
 {    9900, .8385300781250E+04, -.1177040649414E+04, .9979141845703E+03},
 {    9901, .8374835937500E+04, -.1166256225586E+04, .1002599975586E+04},
 {    9902, .8362870117188E+04, -.1157537475586E+04, .1007956176758E+04},
 {    9903, .8349747070312E+04, -.1151129028320E+04, .1013830993652E+04},
 {    9904, .8333752929688E+04, -.1147019042969E+04, .1020991271973E+04},
 {    9905, .8321513671875E+04, -.1145892089844E+04, .1026470092773E+04},
 {    9909, .8343415039062E+04, -.1154726684570E+04, .1002751831055E+04},
 {    9912, .8380111328125E+04, -.1177040649414E+04, .9863228759766E+03},
 {    9913, .8369644531250E+04, -.1166256225586E+04, .9910084838867E+03},
 {    9914, .8357681640625E+04, -.1157537475586E+04, .9963640136719E+03},
 {    9915, .8344557617188E+04, -.1151129028320E+04, .1002239746094E+04},
 {    9916, .8328561523438E+04, -.1147019042969E+04, .1009399780273E+04},
 {    9917, .8316325195312E+04, -.1145892089844E+04, .1014878845215E+04},
 {    9921, .8338223632812E+04, -.1154726684570E+04, .9911601562500E+03},
 {    9924, .8374922851562E+04, -.1177040649414E+04, .9747314453125E+03},
 {    9925, .8364456054688E+04, -.1166256225586E+04, .9794169921875E+03},
 {    9926, .8352493164062E+04, -.1157537475586E+04, .9847727050781E+03},
 {    9927, .8339369140625E+04, -.1151129028320E+04, .9906474609375E+03},
 {    9928, .8323372070312E+04, -.1147019165039E+04, .9978082275391E+03},
 {    9929, .8311132812500E+04, -.1145892089844E+04, .1003287475586E+04},
 {    9930, .8387886718750E+04, -.1174493408203E+04, .9967568359375E+03},
 {    9931, .8376995117188E+04, -.1163271850586E+04, .1001632446289E+04},
 {    9932, .8364545898438E+04, -.1154199584961E+04, .1007205200195E+04},
 {    9933, .8350892578125E+04, -.1147531250000E+04, .1013318420410E+04},
 {    9934, .8334333007812E+04, -.1143262573242E+04, .1020731201172E+04},
 {    9935, .8321513671875E+04, -.1142082031250E+04, .1026470092773E+04},
 {    9936, .8382698242188E+04, -.1174493408203E+04, .9851648559570E+03},
 {    9937, .8371805664062E+04, -.1163271850586E+04, .9900411987305E+03},
 {    9938, .8359357421875E+04, -.1154199584961E+04, .9956135253906E+03},
 {    9939, .8345701171875E+04, -.1147531250000E+04, .1001727416992E+04},
 {    9940, .8329141601562E+04, -.1143262573242E+04, .1009140380859E+04},
 {    9941, .8316325195312E+04, -.1142082031250E+04, .1014878845215E+04},
 {    9942, .8377509765625E+04, -.1174493408203E+04, .9735735473633E+03},
 {    9943, .8366617187500E+04, -.1163271850586E+04, .9784497070312E+03},
 {    9944, .8354169921875E+04, -.1154199584961E+04, .9840219726562E+03},
 {    9945, .8340513671875E+04, -.1147531250000E+04, .9901351928711E+03},
 {    9946, .8323953125000E+04, -.1143262573242E+04, .9975488891602E+03},
 {    9947, .8311132812500E+04, -.1142082031250E+04, .1003287475586E+04},
 {    9950, .8309919921875E+04, -.1145892089844E+04, .1031659179688E+04},
 {    9951, .8298330078125E+04, -.1145892089844E+04, .1036848388672E+04},
 {    9954, .8304731445312E+04, -.1145892089844E+04, .1020067871094E+04},
 {    9955, .8293141601562E+04, -.1145892089844E+04, .1025256958008E+04},
 {    9958, .8299541992188E+04, -.1145892089844E+04, .1008476074219E+04},
 {    9959, .8287951171875E+04, -.1145892089844E+04, .1013665771484E+04},
 {    9960, .8309919921875E+04, -.1142082031250E+04, .1031659179688E+04},
 {    9961, .8298330078125E+04, -.1142082031250E+04, .1036848388672E+04},
 {    9962, .8304731445312E+04, -.1142082031250E+04, .1020067871094E+04},
 {    9963, .8293141601562E+04, -.1142082031250E+04, .1025256958008E+04},
 {    9964, .8299541992188E+04, -.1142082031250E+04, .1008476074219E+04},
 {    9965, .8287951171875E+04, -.1142082031250E+04, .1013665771484E+04},
 {    9971, .8219102539062E+04, -.1215458984375E+04, .1072315063477E+04},
 {    9972, .8285819335938E+04, -.1146974853516E+04, .1042448364258E+04},
 {    9973, .8257229492188E+04, -.1157930908203E+04, .1055246704102E+04},
 {    9974, .8246533203125E+04, -.1164960327148E+04, .1060035644531E+04},
 {    9975, .8233076171875E+04, -.1178871582031E+04, .1066059814453E+04},
 {    9976, .8223436523438E+04, -.1194361083984E+04, .1070374023438E+04},
 {    9977, .8215752929688E+04, -.1214427368164E+04, .1073814208984E+04},
 {    9983, .8213915039062E+04, -.1215458984375E+04, .1060723388672E+04},
 {    9984, .8280632812500E+04, -.1146974853516E+04, .1030856689453E+04},
 {    9985, .8252040039062E+04, -.1157930908203E+04, .1043655395508E+04},
 {    9986, .8241343750000E+04, -.1164960327148E+04, .1048444091797E+04},
 {    9987, .8227887695312E+04, -.1178871582031E+04, .1054468383789E+04},
 {    9988, .8218249023438E+04, -.1194361083984E+04, .1058781982422E+04},
 {    9989, .8210566406250E+04, -.1214427368164E+04, .1062222045898E+04},
 {    9995, .8208722656250E+04, -.1215458984375E+04, .1049131835938E+04},
 {    9996, .8275442382812E+04, -.1146974853516E+04, .1019265014648E+04},
 {    9997, .8246851562500E+04, -.1157930908203E+04, .1032063964844E+04},
 {    9998, .8236155273438E+04, -.1164960327148E+04, .1036852294922E+04},
 {    9999, .8222698242188E+04, -.1178871582031E+04, .1042876586914E+04},
 {   10000, .8213060546875E+04, -.1194361083984E+04, .1047190673828E+04},
 {   10001, .8205376953125E+04, -.1214427368164E+04, .1050630371094E+04},
 {   10002, .8285074218750E+04, -.1143253540039E+04, .1042781982422E+04},
 {   10003, .8255773437500E+04, -.1154471313477E+04, .1055899047852E+04},
 {   10004, .8244435546875E+04, -.1161923339844E+04, .1060974975586E+04},
 {   10005, .8230431640625E+04, -.1176398559570E+04, .1067243774414E+04},
 {   10006, .8220372070312E+04, -.1192567138672E+04, .1071747558594E+04},
 {   10007, .8212409179688E+04, -.1213395874023E+04, .1075311889648E+04},
 {   10008, .8279885742188E+04, -.1143253540039E+04, .1031190307617E+04},
 {   10009, .8250586914062E+04, -.1154471313477E+04, .1044307250977E+04},
 {   10010, .8239244140625E+04, -.1161923339844E+04, .1049384155273E+04},
 {   10011, .8225243164062E+04, -.1176398559570E+04, .1055651977539E+04},
 {   10012, .8215180664062E+04, -.1192567016602E+04, .1060156005859E+04},
 {   10013, .8207219726562E+04, -.1213395874023E+04, .1063720581055E+04},
 {   10014, .8274697265625E+04, -.1143253540039E+04, .1019599121094E+04},
 {   10015, .8245396484375E+04, -.1154471313477E+04, .1032715820312E+04},
 {   10016, .8234056640625E+04, -.1161923339844E+04, .1037792968750E+04},
 {   10017, .8220054687500E+04, -.1176398559570E+04, .1044060791016E+04},
 {   10018, .8209992187500E+04, -.1192567138672E+04, .1048563964844E+04},
 {   10019, .8202030273438E+04, -.1213395874023E+04, .1052129028320E+04},
 {   10020, .8216506835938E+04, -.1230180419922E+04, .1073477661133E+04},
 {   10021, .8213048828125E+04, -.1229770874023E+04, .1075025512695E+04},
 {   10022, .8211317382812E+04, -.1230180419922E+04, .1061886108398E+04},
 {   10023, .8207861328125E+04, -.1229770874023E+04, .1063433471680E+04},
 {   10024, .8206129882812E+04, -.1230180419922E+04, .1050294189453E+04},
 {   10025, .8202668945312E+04, -.1229770874023E+04, .1051842529297E+04},
 {   10026, .8209591796875E+04, -.1229361450195E+04, .1076572753906E+04},
 {   10027, .8204403320312E+04, -.1229361450195E+04, .1064980957031E+04},
 {   10028, .8199211914062E+04, -.1229361450195E+04, .1053390258789E+04},
 {   10029, .8384685546875E+04, -.1202382934570E+04, .9668822021484E+03},
 {   10031, .8387847656250E+04, -.1200798950195E+04, .9654664306641E+03},
 {   10032, .8383462890625E+04, -.1191932617188E+04, .9674299316406E+03},
 {   10033, .8379497070312E+04, -.1202382934570E+04, .9552908325195E+03},
 {   10035, .8382659179688E+04, -.1200798950195E+04, .9538746948242E+03},
 {   10036, .8378274414062E+04, -.1191932617188E+04, .9558381347656E+03},
 {   10037, .8374308593750E+04, -.1202382934570E+04, .9436990966797E+03},
 {   10039, .8377469726562E+04, -.1200798950195E+04, .9422832031250E+03},
 {   10040, .8373084960938E+04, -.1191932617188E+04, .9442466430664E+03},
 {   10041, .8391012695312E+04, -.1199214843750E+04, .9640502929688E+03},
 {   10042, .8386399414062E+04, -.1189894165039E+04, .9661143798828E+03},
 {   10043, .8385822265625E+04, -.1199214843750E+04, .9524589233398E+03},
 {   10044, .8381212890625E+04, -.1189894165039E+04, .9545227050781E+03},
 {   10045, .8380632812500E+04, -.1199214843750E+04, .9408675537109E+03},
 {   10046, .8376022460938E+04, -.1189894165039E+04, .9429315185547E+03},
 {   10050, .8336925781250E+04, -.1154726684570E+04, .9882623291016E+03},
 {   10053, .8373625976562E+04, -.1177040649414E+04, .9718335571289E+03},
 {   10054, .8363158203125E+04, -.1166256225586E+04, .9765191650391E+03},
 {   10055, .8351195312500E+04, -.1157537475586E+04, .9818750000000E+03},
 {   10056, .8338071289062E+04, -.1151129028320E+04, .9877503051758E+03},
 {   10057, .8322076171875E+04, -.1147019165039E+04, .9949102783203E+03},
 {   10058, .8309836914062E+04, -.1145892089844E+04, .1000389099121E+04},
 {   10062, .8331736328125E+04, -.1154726684570E+04, .9766710205078E+03},
 {   10065, .8368436523438E+04, -.1177040649414E+04, .9602418212891E+03},
 {   10066, .8357969726562E+04, -.1166256225586E+04, .9649278564453E+03},
 {   10067, .8346005859375E+04, -.1157537475586E+04, .9702834472656E+03},
 {   10068, .8332882812500E+04, -.1151129028320E+04, .9761584472656E+03},
 {   10069, .8316885742188E+04, -.1147019165039E+04, .9833192138672E+03},
 {   10070, .8304648437500E+04, -.1145892089844E+04, .9887977294922E+03},
 {   10074, .8326548828125E+04, -.1154726684570E+04, .9650794067383E+03},
 {   10077, .8363247070312E+04, -.1177040649414E+04, .9486503295898E+03},
 {   10078, .8352780273438E+04, -.1166256225586E+04, .9533363647461E+03},
 {   10079, .8340816406250E+04, -.1157537475586E+04, .9586919555664E+03},
 {   10080, .8327693359375E+04, -.1151129028320E+04, .9645670776367E+03},
 {   10081, .8311698242188E+04, -.1147019165039E+04, .9717271728516E+03},
 {   10082, .8299458007812E+04, -.1145892089844E+04, .9772065429688E+03},
 {   10083, .8376210937500E+04, -.1174493408203E+04, .9706757812500E+03},
 {   10084, .8365319335938E+04, -.1163271850586E+04, .9755517578125E+03},
 {   10085, .8352872070312E+04, -.1154199584961E+04, .9811243896484E+03},
 {   10086, .8339217773438E+04, -.1147531250000E+04, .9872373046875E+03},
 {   10087, .8322656250000E+04, -.1143262573242E+04, .9946506347656E+03},
 {   10088, .8309836914062E+04, -.1142082031250E+04, .1000389099121E+04},
 {   10089, .8371021484375E+04, -.1174493408203E+04, .9590845947266E+03},
 {   10090, .8360131835938E+04, -.1163271850586E+04, .9639602050781E+03},
 {   10091, .8347682617188E+04, -.1154199584961E+04, .9695328369141E+03},
 {   10092, .8334025390625E+04, -.1147531250000E+04, .9756461791992E+03},
 {   10093, .8317466796875E+04, -.1143262573242E+04, .9830590209961E+03},
 {   10094, .8304648437500E+04, -.1142082031250E+04, .9887977294922E+03},
 {   10095, .8365835937500E+04, -.1174493408203E+04, .9474926147461E+03},
 {   10096, .8354942382812E+04, -.1163271850586E+04, .9523685913086E+03},
 {   10097, .8342493164062E+04, -.1154199584961E+04, .9579416503906E+03},
 {   10098, .8328836914062E+04, -.1147531250000E+04, .9640546264648E+03},
 {   10099, .8312277343750E+04, -.1143262573242E+04, .9714676513672E+03},
 {   10100, .8299458007812E+04, -.1142082031250E+04, .9772065429688E+03},
 {   10103, .8298244140625E+04, -.1145892089844E+04, .1005578613281E+04},
 {   10104, .8286653320312E+04, -.1145892089844E+04, .1010767395020E+04},
 {   10107, .8293055664062E+04, -.1145892089844E+04, .9939868164062E+03},
 {   10108, .8281464843750E+04, -.1145892089844E+04, .9991757812500E+03},
 {   10111, .8287866210938E+04, -.1145892089844E+04, .9823952636719E+03},
 {   10112, .8276275390625E+04, -.1145892089844E+04, .9875843505859E+03},
 {   10113, .8298244140625E+04, -.1142082031250E+04, .1005578613281E+04},
 {   10114, .8286653320312E+04, -.1142082031250E+04, .1010767395020E+04},
 {   10115, .8293055664062E+04, -.1142082031250E+04, .9939868164062E+03},
 {   10116, .8281464843750E+04, -.1142082031250E+04, .9991757812500E+03},
 {   10117, .8287866210938E+04, -.1142082031250E+04, .9823952636719E+03},
 {   10118, .8276275390625E+04, -.1142082031250E+04, .9875843505859E+03},
 {   10124, .8207427734375E+04, -.1215458984375E+04, .1046233642578E+04},
 {   10125, .8274144531250E+04, -.1146974853516E+04, .1016367248535E+04},
 {   10126, .8245555664062E+04, -.1157930908203E+04, .1029165893555E+04},
 {   10127, .8234858398438E+04, -.1164960327148E+04, .1033954467773E+04},
 {   10128, .8221403320312E+04, -.1178871582031E+04, .1039978393555E+04},
 {   10129, .8211764648438E+04, -.1194361083984E+04, .1044292602539E+04},
 {   10130, .8204080078125E+04, -.1214427368164E+04, .1047732788086E+04},
 {   10136, .8202239257812E+04, -.1215458984375E+04, .1034642089844E+04},
 {   10137, .8268955078125E+04, -.1146974853516E+04, .1004775756836E+04},
 {   10138, .8240365234375E+04, -.1157930908203E+04, .1017574218750E+04},
 {   10139, .8229667968750E+04, -.1164960327148E+04, .1022363647461E+04},
 {   10140, .8216210937500E+04, -.1178871582031E+04, .1028387084961E+04},
 {   10141, .8206573242188E+04, -.1194361083984E+04, .1032701660156E+04},
 {   10142, .8198888671875E+04, -.1214427368164E+04, .1036141113281E+04},
 {   10148, .8197049804688E+04, -.1215458984375E+04, .1023051025391E+04},
 {   10149, .8263766601562E+04, -.1146974853516E+04, .9931839599609E+03},
 {   10150, .8235176757812E+04, -.1157930908203E+04, .1005982788086E+04},
 {   10151, .8224478515625E+04, -.1164960327148E+04, .1010771301270E+04},
 {   10152, .8211023437500E+04, -.1178871582031E+04, .1016795654297E+04},
 {   10153, .8201384765625E+04, -.1194361083984E+04, .1021109741211E+04},
 {   10154, .8193701171875E+04, -.1214427368164E+04, .1024549926758E+04},
 {   10155, .8273400390625E+04, -.1143253540039E+04, .1016700622559E+04},
 {   10156, .8244097656250E+04, -.1154471313477E+04, .1029818237305E+04},
 {   10157, .8232757812500E+04, -.1161923339844E+04, .1034894775391E+04},
 {   10158, .8218757812500E+04, -.1176398559570E+04, .1041162841797E+04},
 {   10159, .8208695312500E+04, -.1192567138672E+04, .1045665893555E+04},
 {   10160, .8200732421875E+04, -.1213395874023E+04, .1049231201172E+04},
 {   10161, .8268210937500E+04, -.1143253540039E+04, .1005109863281E+04},
 {   10162, .8238908203125E+04, -.1154471313477E+04, .1018226928711E+04},
 {   10163, .8227570312500E+04, -.1161923339844E+04, .1023303222656E+04},
 {   10164, .8213566406250E+04, -.1176398559570E+04, .1029571289062E+04},
 {   10165, .8203508789062E+04, -.1192567016602E+04, .1034074584961E+04},
 {   10166, .8195544921875E+04, -.1213395874023E+04, .1037639526367E+04},
 {   10167, .8263021484375E+04, -.1143253540039E+04, .9935179443359E+03},
 {   10168, .8233721679688E+04, -.1154471313477E+04, .1006634887695E+04},
 {   10169, .8222380859375E+04, -.1161923339844E+04, .1011711547852E+04},
 {   10170, .8208380859375E+04, -.1176398559570E+04, .1017979980469E+04},
 {   10171, .8198319335938E+04, -.1192567138672E+04, .1022482788086E+04},
 {   10172, .8190354492188E+04, -.1213395874023E+04, .1026048095703E+04},
 {   10173, .8204830078125E+04, -.1230180419922E+04, .1047396484375E+04},
 {   10174, .8201372070312E+04, -.1229770874023E+04, .1048944702148E+04},
 {   10175, .8199640625000E+04, -.1230180419922E+04, .1035805053711E+04},
 {   10176, .8196182617188E+04, -.1229770874023E+04, .1037353149414E+04},
 {   10177, .8194452148438E+04, -.1230180419922E+04, .1024213745117E+04},
 {   10178, .8190997558594E+04, -.1229770874023E+04, .1025761230469E+04},
 {   10179, .8197916992188E+04, -.1229361450195E+04, .1050492309570E+04},
 {   10180, .8192726562500E+04, -.1229361450195E+04, .1038900634766E+04},
 {   10181, .8187536621094E+04, -.1229361450195E+04, .1027308959961E+04},
 {   10182, .8373010742188E+04, -.1202382934570E+04, .9408011474609E+03},
 {   10184, .8376174804688E+04, -.1200798950195E+04, .9393851318359E+03},
 {   10185, .8371789062500E+04, -.1191932617188E+04, .9413486328125E+03},
 {   10186, .8367821289062E+04, -.1202383056641E+04, .9292096557617E+03},
 {   10188, .8370983398438E+04, -.1200798950195E+04, .9277935791016E+03},
 {   10189, .8366597656250E+04, -.1191932617188E+04, .9297575683594E+03},
 {   10190, .8362632812500E+04, -.1202383056641E+04, .9176179199219E+03},
 {   10192, .8365794921875E+04, -.1200798950195E+04, .9162022094727E+03},
 {   10193, .8361410156250E+04, -.1191932617188E+04, .9181655273438E+03},
 {   10194, .8379334960938E+04, -.1199214843750E+04, .9379694213867E+03},
 {   10195, .8374726562500E+04, -.1189894165039E+04, .9400333251953E+03},
 {   10196, .8374147460938E+04, -.1199214843750E+04, .9263778686523E+03},
 {   10197, .8369537109375E+04, -.1189894165039E+04, .9284418945312E+03},
 {   10198, .8368958984375E+04, -.1199214843750E+04, .9147861938477E+03},
 {   10199, .8364347656250E+04, -.1189894165039E+04, .9168504638672E+03},
 {   10203, .8325250976562E+04, -.1154726684570E+04, .9621815795898E+03},
 {   10206, .8361949218750E+04, -.1177040649414E+04, .9457529296875E+03},
 {   10207, .8351482421875E+04, -.1166256225586E+04, .9504384155273E+03},
 {   10208, .8339519531250E+04, -.1157537475586E+04, .9557940063477E+03},
 {   10209, .8326396484375E+04, -.1151129028320E+04, .9616689453125E+03},
 {   10210, .8310400390625E+04, -.1147019165039E+04, .9688295288086E+03},
 {   10211, .8298160156250E+04, -.1145892089844E+04, .9743084106445E+03},
 {   10215, .8320062500000E+04, -.1154726684570E+04, .9505900268555E+03},
 {   10218, .8356760742188E+04, -.1177040649414E+04, .9341608886719E+03},
 {   10219, .8346293945312E+04, -.1166256225586E+04, .9388468017578E+03},
 {   10220, .8334331054688E+04, -.1157537475586E+04, .9442023925781E+03},
 {   10221, .8321206054688E+04, -.1151129028320E+04, .9500776367188E+03},
 {   10222, .8305210937500E+04, -.1147019165039E+04, .9572383422852E+03},
 {   10223, .8292974609375E+04, -.1145892089844E+04, .9627166748047E+03},
 {   10227, .8314873046875E+04, -.1154726684570E+04, .9389984741211E+03},
 {   10230, .8351572265625E+04, -.1177040649414E+04, .9225694580078E+03},
 {   10231, .8341104492188E+04, -.1166256225586E+04, .9272556152344E+03},
 {   10232, .8329143554688E+04, -.1157537475586E+04, .9326107177734E+03},
 {   10233, .8316017578125E+04, -.1151129028320E+04, .9384859619141E+03},
 {   10234, .8300022460938E+04, -.1147019165039E+04, .9456466674805E+03},
 {   10235, .8287781250000E+04, -.1145892089844E+04, .9511254882812E+03},
 {   10236, .8364538085938E+04, -.1174493408203E+04, .9445949707031E+03},
 {   10237, .8353644531250E+04, -.1163271850586E+04, .9494708251953E+03},
 {   10238, .8341197265625E+04, -.1154199584961E+04, .9550435791016E+03},
 {   10239, .8327540039062E+04, -.1147531250000E+04, .9611566162109E+03},
 {   10240, .8310982421875E+04, -.1143262573242E+04, .9685698242188E+03},
 {   10241, .8298160156250E+04, -.1142082031250E+04, .9743084106445E+03},
 {   10242, .8359346679688E+04, -.1174493408203E+04, .9330032958984E+03},
 {   10243, .8348454101562E+04, -.1163271850586E+04, .9378791503906E+03},
 {   10244, .8336008789062E+04, -.1154199584961E+04, .9434520263672E+03},
 {   10245, .8322352539062E+04, -.1147531250000E+04, .9495649414062E+03},
 {   10246, .8305791992188E+04, -.1143262573242E+04, .9569781494141E+03},
 {   10247, .8292974609375E+04, -.1142082031250E+04, .9627166748047E+03},
 {   10248, .8354158203125E+04, -.1174493408203E+04, .9214119873047E+03},
 {   10249, .8343266601562E+04, -.1163271850586E+04, .9262877197266E+03},
 {   10250, .8330817382812E+04, -.1154199584961E+04, .9318605346680E+03},
 {   10251, .8317161132812E+04, -.1147531250000E+04, .9379735717773E+03},
 {   10252, .8300602539062E+04, -.1143262573242E+04, .9453867187500E+03},
 {   10253, .8287781250000E+04, -.1142082031250E+04, .9511254882812E+03},
 {   10256, .8286571289062E+04, -.1145892089844E+04, .9794973144531E+03},
 {   10257, .8274979492188E+04, -.1145892089844E+04, .9846864624023E+03},
 {   10260, .8281380859375E+04, -.1145892089844E+04, .9679062500000E+03},
 {   10261, .8269791015625E+04, -.1145892089844E+04, .9730949707031E+03},
 {   10264, .8276192382812E+04, -.1145892089844E+04, .9563143310547E+03},
 {   10265, .8264600585938E+04, -.1145892089844E+04, .9615036621094E+03},
 {   10266, .8286571289062E+04, -.1142082031250E+04, .9794973144531E+03},
 {   10267, .8274979492188E+04, -.1142082031250E+04, .9846864624023E+03},
 {   10268, .8281380859375E+04, -.1142082031250E+04, .9679062500000E+03},
 {   10269, .8269791015625E+04, -.1142082031250E+04, .9730949707031E+03},
 {   10270, .8276192382812E+04, -.1142082031250E+04, .9563143310547E+03},
 {   10271, .8264600585938E+04, -.1142082031250E+04, .9615036621094E+03},
 {   10277, .8195753906250E+04, -.1215458984375E+04, .1020152832031E+04},
 {   10278, .8262468750000E+04, -.1146974853516E+04, .9902864990234E+03},
 {   10279, .8233879882812E+04, -.1157930908203E+04, .1003084960938E+04},
 {   10280, .8223180664062E+04, -.1164960327148E+04, .1007873840332E+04},
 {   10281, .8209722656250E+04, -.1178871582031E+04, .1013898193359E+04},
 {   10282, .8200087890625E+04, -.1194361083984E+04, .1018211669922E+04},
 {   10283, .8192405273438E+04, -.1214427368164E+04, .1021651489258E+04},
 {   10289, .8190562500000E+04, -.1215458984375E+04, .1008562133789E+04},
 {   10290, .8257280273438E+04, -.1146974853516E+04, .9786949462891E+03},
 {   10291, .8228691406250E+04, -.1157930908203E+04, .9914932861328E+03},
 {   10292, .8217992187500E+04, -.1164960327148E+04, .9962824707031E+03},
 {   10293, .8204539062500E+04, -.1178871582031E+04, .1002305664062E+04},
 {   10294, .8194900390625E+04, -.1194361083984E+04, .1006620117188E+04},
 {   10295, .8187217285156E+04, -.1214427368164E+04, .1010060058594E+04},
 {   10301, .8185374023438E+04, -.1215458984375E+04, .9969697265625E+03},
 {   10302, .8252089843750E+04, -.1146974853516E+04, .9671033325195E+03},
 {   10303, .8223502929688E+04, -.1157930908203E+04, .9799019165039E+03},
 {   10304, .8212803710938E+04, -.1164960327148E+04, .9846909179688E+03},
 {   10305, .8199348632812E+04, -.1178871582031E+04, .9907144165039E+03},
 {   10306, .8189710449219E+04, -.1194361083984E+04, .9950286865234E+03},
 {   10307, .8182025390625E+04, -.1214427368164E+04, .9984688720703E+03},
 {   10308, .8261724609375E+04, -.1143253540039E+04, .9906200561523E+03},
 {   10309, .8232422851562E+04, -.1154471313477E+04, .1003736694336E+04},
 {   10310, .8221083007812E+04, -.1161923339844E+04, .1008813842773E+04},
 {   10311, .8207079101562E+04, -.1176398559570E+04, .1015082153320E+04},
 {   10312, .8197020507812E+04, -.1192567138672E+04, .1019585327148E+04},
 {   10313, .8189055175781E+04, -.1213395874023E+04, .1023150268555E+04},
 {   10314, .8256536132812E+04, -.1143253540039E+04, .9790284423828E+03},
 {   10315, .8227234375000E+04, -.1154471313477E+04, .9921453247070E+03},
 {   10316, .8215892578125E+04, -.1161923339844E+04, .9972222290039E+03},
 {   10317, .8201891601562E+04, -.1176398559570E+04, .1003490234375E+04},
 {   10318, .8191831054688E+04, -.1192567138672E+04, .1007993652344E+04},
 {   10319, .8183867187500E+04, -.1213395874023E+04, .1011558898926E+04},
 {   10320, .8251347656250E+04, -.1143253540039E+04, .9674368896484E+03},
 {   10321, .8222044921875E+04, -.1154471313477E+04, .9805541992188E+03},
 {   10322, .8210705078125E+04, -.1161923339844E+04, .9856306762695E+03},
 {   10323, .8196702148438E+04, -.1176398559570E+04, .9918986816406E+03},
 {   10324, .8186642578125E+04, -.1192567138672E+04, .9964017333984E+03},
 {   10325, .8178680175781E+04, -.1213395874023E+04, .9999671020508E+03},
 {   10326, .8193154296875E+04, -.1230180419922E+04, .1021315612793E+04},
 {   10327, .8189698730469E+04, -.1229770874023E+04, .1022863159180E+04},
 {   10328, .8187967285156E+04, -.1230180419922E+04, .1009723876953E+04},
 {   10329, .8184507324219E+04, -.1229770874023E+04, .1011271972656E+04},
 {   10330, .8182776367188E+04, -.1230180419922E+04, .9981324462891E+03},
 {   10331, .8179319824219E+04, -.1229770874023E+04, .9996801757812E+03},
 {   10332, .8186241699219E+04, -.1229361450195E+04, .1024410888672E+04},
 {   10333, .8181050781250E+04, -.1229361450195E+04, .1012819458008E+04},
 {   10334, .8175862304688E+04, -.1229361450195E+04, .1001227722168E+04},
 {   10335, .8409334960938E+04, -.1368801147461E+04, .1021941223145E+04},
 {   10337, .8412496093750E+04, -.1370385131836E+04, .1020526123047E+04},
 {   10338, .8408718750000E+04, -.1378023315430E+04, .1022217407227E+04},
 {   10339, .8414524414062E+04, -.1368801147461E+04, .1033533081055E+04},
 {   10341, .8417685546875E+04, -.1370385131836E+04, .1032117431641E+04},
 {   10342, .8413908203125E+04, -.1378023315430E+04, .1033809082031E+04},
 {   10343, .8419711914062E+04, -.1368801147461E+04, .1045124511719E+04},
 {   10345, .8422875976562E+04, -.1370385131836E+04, .1043709350586E+04},
 {   10346, .8419100585938E+04, -.1378023315430E+04, .1045400268555E+04},
 {   10347, .8415661132812E+04, -.1371969238281E+04, .1019110229492E+04},
 {   10348, .8411657226562E+04, -.1380061889648E+04, .1020902099609E+04},
 {   10349, .8420850585938E+04, -.1371969238281E+04, .1030701538086E+04},
 {   10350, .8416845703125E+04, -.1380061889648E+04, .1032493652344E+04},
 {   10351, .8426039062500E+04, -.1371969238281E+04, .1042293090820E+04},
 {   10352, .8422036132812E+04, -.1380061889648E+04, .1044085449219E+04},
 {   10356, .8361575195312E+04, -.1416457397461E+04, .1043322509766E+04},
 {   10359, .8398273437500E+04, -.1394143432617E+04, .1026893554688E+04},
 {   10360, .8387805664062E+04, -.1404927734375E+04, .1031579345703E+04},
 {   10361, .8375843750000E+04, -.1413646484375E+04, .1036934936523E+04},
 {   10362, .8362716796875E+04, -.1420055053711E+04, .1042810058594E+04},
 {   10363, .8346954101562E+04, -.1424143676758E+04, .1049866577148E+04},
 {   10364, .8334484375000E+04, -.1425291992188E+04, .1055449218750E+04},
 {   10368, .8366762695312E+04, -.1416457397461E+04, .1054913818359E+04},
 {   10371, .8403463867188E+04, -.1394143432617E+04, .1038485107422E+04},
 {   10372, .8392996093750E+04, -.1404927734375E+04, .1043170410156E+04},
 {   10373, .8381033203125E+04, -.1413646484375E+04, .1048526123047E+04},
 {   10374, .8367907226562E+04, -.1420055053711E+04, .1054401367188E+04},
 {   10375, .8352145507812E+04, -.1424143676758E+04, .1061458251953E+04},
 {   10376, .8339673828125E+04, -.1425291992188E+04, .1067040649414E+04},
 {   10380, .8371952148438E+04, -.1416457397461E+04, .1066505371094E+04},
 {   10383, .8408650390625E+04, -.1394143432617E+04, .1050076416016E+04},
 {   10384, .8398184570312E+04, -.1404927734375E+04, .1054761962891E+04},
 {   10385, .8386220703125E+04, -.1413646484375E+04, .1060117797852E+04},
 {   10386, .8373097656250E+04, -.1420055053711E+04, .1065992675781E+04},
 {   10387, .8357333007812E+04, -.1424143676758E+04, .1073050170898E+04},
 {   10388, .8344863281250E+04, -.1425291992188E+04, .1078632080078E+04},
 {   10389, .8400859375000E+04, -.1396690673828E+04, .1025735473633E+04},
 {   10390, .8389969726562E+04, -.1407912231445E+04, .1030611328125E+04},
 {   10391, .8377520507812E+04, -.1416984375000E+04, .1036184204102E+04},
 {   10392, .8363862304688E+04, -.1423652709961E+04, .1042297729492E+04},
 {   10393, .8347536132812E+04, -.1427900268555E+04, .1049607055664E+04},
 {   10394, .8334484375000E+04, -.1429102050781E+04, .1055449218750E+04},
 {   10395, .8406048828125E+04, -.1396690673828E+04, .1037327148438E+04},
 {   10396, .8395157226562E+04, -.1407912231445E+04, .1042202636719E+04},
 {   10397, .8382708007812E+04, -.1416984375000E+04, .1047775634766E+04},
 {   10398, .8369051757812E+04, -.1423652709961E+04, .1053889038086E+04},
 {   10399, .8352724609375E+04, -.1427900268555E+04, .1061198608398E+04},
 {   10400, .8339673828125E+04, -.1429102050781E+04, .1067040649414E+04},
 {   10401, .8411239257812E+04, -.1396690673828E+04, .1048919067383E+04},
 {   10402, .8400345703125E+04, -.1407912231445E+04, .1053794921875E+04},
 {   10403, .8387897460938E+04, -.1416984375000E+04, .1059367309570E+04},
 {   10404, .8374240234375E+04, -.1423652709961E+04, .1065480712891E+04},
 {   10405, .8357914062500E+04, -.1427900268555E+04, .1072789916992E+04},
 {   10406, .8344863281250E+04, -.1429102050781E+04, .1078632080078E+04},
 {   10409, .8322892578125E+04, -.1425291992188E+04, .1060638427734E+04},
 {   10410, .8311302734375E+04, -.1425291992188E+04, .1065827270508E+04},
 {   10413, .8328083007812E+04, -.1425291992188E+04, .1072229858398E+04},
 {   10414, .8316491210938E+04, -.1425291992188E+04, .1077418945312E+04},
 {   10417, .8333272460938E+04, -.1425291992188E+04, .1083821044922E+04},
 {   10418, .8321680664062E+04, -.1425291992188E+04, .1089010131836E+04},
 {   10419, .8322892578125E+04, -.1429102050781E+04, .1060638427734E+04},
 {   10420, .8311302734375E+04, -.1429102050781E+04, .1065827270508E+04},
 {   10421, .8328083007812E+04, -.1429102050781E+04, .1072229858398E+04},
 {   10422, .8316491210938E+04, -.1429102050781E+04, .1077418945312E+04},
 {   10423, .8333272460938E+04, -.1429102050781E+04, .1083821044922E+04},
 {   10424, .8321680664062E+04, -.1429102050781E+04, .1089010131836E+04},
 {   10430, .8232075195312E+04, -.1355725097656E+04, .1101293945312E+04},
 {   10431, .8298997070312E+04, -.1424336425781E+04, .1071335693359E+04},
 {   10432, .8270402343750E+04, -.1413383789062E+04, .1084136962891E+04},
 {   10433, .8259505859375E+04, -.1406223754883E+04, .1089014404297E+04},
 {   10434, .8246049804688E+04, -.1392312500000E+04, .1095038452148E+04},
 {   10435, .8236412109375E+04, -.1376822998047E+04, .1099352905273E+04},
 {   10436, .8228728515625E+04, -.1356756713867E+04, .1102792358398E+04},
 {   10442, .8237266601562E+04, -.1355725097656E+04, .1112884887695E+04},
 {   10443, .8304185546875E+04, -.1424336425781E+04, .1082927734375E+04},
 {   10444, .8275591796875E+04, -.1413383789062E+04, .1095728149414E+04},
 {   10445, .8264694335938E+04, -.1406223754883E+04, .1100606445312E+04},
 {   10446, .8251239257812E+04, -.1392312500000E+04, .1106629882812E+04},
 {   10447, .8241603515625E+04, -.1376822998047E+04, .1110943847656E+04},
 {   10448, .8233917968750E+04, -.1356756713867E+04, .1114384033203E+04},
 {   10454, .8242452148438E+04, -.1355725097656E+04, .1124476562500E+04},
 {   10455, .8309373046875E+04, -.1424336425781E+04, .1094518920898E+04},
 {   10456, .8280781250000E+04, -.1413383789062E+04, .1107319702148E+04},
 {   10457, .8269883789062E+04, -.1406223754883E+04, .1112197265625E+04},
 {   10458, .8256430664062E+04, -.1392312500000E+04, .1118221191406E+04},
 {   10459, .8246790039062E+04, -.1376822998047E+04, .1122535522461E+04},
 {   10460, .8239105468750E+04, -.1356756713867E+04, .1125975463867E+04},
 {   10461, .8298250000000E+04, -.1428057861328E+04, .1071670410156E+04},
 {   10462, .8268945312500E+04, -.1416843505859E+04, .1084789062500E+04},
 {   10463, .8257405273438E+04, -.1409260742188E+04, .1089954956055E+04},
 {   10464, .8243403320312E+04, -.1394785522461E+04, .1096222167969E+04},
 {   10465, .8233343750000E+04, -.1378616943359E+04, .1100726318359E+04},
 {   10466, .8225382812500E+04, -.1357788208008E+04, .1104290893555E+04},
 {   10467, .8303440429688E+04, -.1428057861328E+04, .1083261352539E+04},
 {   10468, .8274134765625E+04, -.1416843505859E+04, .1096380249023E+04},
 {   10469, .8262595703125E+04, -.1409260742188E+04, .1101545898438E+04},
 {   10470, .8248590820312E+04, -.1394785522461E+04, .1107814697266E+04},
 {   10471, .8238535156250E+04, -.1378617065430E+04, .1112317382812E+04},
 {   10472, .8230570312500E+04, -.1357788208008E+04, .1115882690430E+04},
 {   10473, .8308628906250E+04, -.1428057861328E+04, .1094852783203E+04},
 {   10474, .8279323242188E+04, -.1416843505859E+04, .1107972290039E+04},
 {   10475, .8267785156250E+04, -.1409260742188E+04, .1113137695312E+04},
 {   10476, .8253783203125E+04, -.1394785522461E+04, .1119405517578E+04},
 {   10477, .8243722656250E+04, -.1378616943359E+04, .1123909423828E+04},
 {   10478, .8235757812500E+04, -.1357788208008E+04, .1127474243164E+04},
 {   10479, .8229478515625E+04, -.1341003662109E+04, .1102456420898E+04},
 {   10480, .8226023437500E+04, -.1341413208008E+04, .1104003784180E+04},
 {   10481, .8234666015625E+04, -.1341003662109E+04, .1114048095703E+04},
 {   10482, .8231211914062E+04, -.1341413208008E+04, .1115595214844E+04},
 {   10483, .8239856445312E+04, -.1341003662109E+04, .1125639648438E+04},
 {   10484, .8236400390625E+04, -.1341413208008E+04, .1127187011719E+04},
 {   10485, .8222564453125E+04, -.1341822631836E+04, .1105552001953E+04},
 {   10486, .8227752929688E+04, -.1341822631836E+04, .1117143188477E+04},
 {   10487, .8232941406250E+04, -.1341822631836E+04, .1128734863281E+04},
 {   10488, .8397659179688E+04, -.1368801147461E+04, .9958609008789E+03},
 {   10490, .8400821289062E+04, -.1370385131836E+04, .9944456176758E+03},
 {   10491, .8397044921875E+04, -.1378023315430E+04, .9961365356445E+03},
 {   10492, .8402847656250E+04, -.1368801147461E+04, .1007452026367E+04},
 {   10494, .8406011718750E+04, -.1370385131836E+04, .1006036499023E+04},
 {   10495, .8402232421875E+04, -.1378023315430E+04, .1007727966309E+04},
 {   10496, .8408037109375E+04, -.1368801147461E+04, .1019044189453E+04},
 {   10498, .8411199218750E+04, -.1370385131836E+04, .1017627929688E+04},
 {   10499, .8407421875000E+04, -.1378023315430E+04, .1019319702148E+04},
 {   10500, .8403986328125E+04, -.1371969238281E+04, .9930291748047E+03},
 {   10501, .8399981445312E+04, -.1380061889648E+04, .9948212890625E+03},
 {   10502, .8409173828125E+04, -.1371969238281E+04, .1004620971680E+04},
 {   10503, .8405171875000E+04, -.1380061889648E+04, .1006413146973E+04},
 {   10504, .8414363281250E+04, -.1371969238281E+04, .1016212158203E+04},
 {   10505, .8410359375000E+04, -.1380061889648E+04, .1018004394531E+04},
 {   10509, .8349899414062E+04, -.1416457397461E+04, .1017241699219E+04},
 {   10512, .8386598632812E+04, -.1394143432617E+04, .1000812438965E+04},
 {   10513, .8376132812500E+04, -.1404927734375E+04, .1005497802734E+04},
 {   10514, .8364167968750E+04, -.1413646484375E+04, .1010854064941E+04},
 {   10515, .8351043945312E+04, -.1420055053711E+04, .1016728881836E+04},
 {   10516, .8335279296875E+04, -.1424143676758E+04, .1023785766602E+04},
 {   10517, .8322810546875E+04, -.1425291992188E+04, .1029367919922E+04},
 {   10521, .8355086914062E+04, -.1416457397461E+04, .1028832641602E+04},
 {   10524, .8391787109375E+04, -.1394143432617E+04, .1012403808594E+04},
 {   10525, .8381321289062E+04, -.1404927734375E+04, .1017090332031E+04},
 {   10526, .8369357421875E+04, -.1413646484375E+04, .1022445068359E+04},
 {   10527, .8356234375000E+04, -.1420055053711E+04, .1028320556641E+04},
 {   10528, .8340467773438E+04, -.1424143676758E+04, .1035377563477E+04},
 {   10529, .8327998046875E+04, -.1425291992188E+04, .1040959594727E+04},
 {   10533, .8360277343750E+04, -.1416457397461E+04, .1040424072266E+04},
 {   10536, .8396976562500E+04, -.1394143432617E+04, .1023995361328E+04},
 {   10537, .8386509765625E+04, -.1404927734375E+04, .1028680908203E+04},
 {   10538, .8374545898438E+04, -.1413646484375E+04, .1034036621094E+04},
 {   10539, .8361422851562E+04, -.1420055053711E+04, .1039911987305E+04},
 {   10540, .8345657226562E+04, -.1424143676758E+04, .1046968994141E+04},
 {   10541, .8333187500000E+04, -.1425291992188E+04, .1052551025391E+04},
 {   10542, .8389185546875E+04, -.1396690673828E+04, .9996546020508E+03},
 {   10543, .8378293945312E+04, -.1407912231445E+04, .1004530456543E+04},
 {   10544, .8365845703125E+04, -.1416984375000E+04, .1010103088379E+04},
 {   10545, .8352188476562E+04, -.1423652709961E+04, .1016216796875E+04},
 {   10546, .8335858398438E+04, -.1427900268555E+04, .1023526611328E+04},
 {   10547, .8322810546875E+04, -.1429102050781E+04, .1029367919922E+04},
 {   10548, .8394375000000E+04, -.1396690673828E+04, .1011246093750E+04},
 {   10549, .8383481445312E+04, -.1407912231445E+04, .1016121948242E+04},
 {   10550, .8371033203125E+04, -.1416984375000E+04, .1021694824219E+04},
 {   10551, .8357376953125E+04, -.1423652709961E+04, .1027807861328E+04},
 {   10552, .8341049804688E+04, -.1427900268555E+04, .1035117553711E+04},
 {   10553, .8327998046875E+04, -.1429102050781E+04, .1040959594727E+04},
 {   10554, .8399562500000E+04, -.1396690673828E+04, .1022837768555E+04},
 {   10555, .8388670898438E+04, -.1407912231445E+04, .1027713623047E+04},
 {   10556, .8376223632812E+04, -.1416984375000E+04, .1033285888672E+04},
 {   10557, .8362566406250E+04, -.1423652709961E+04, .1039399780273E+04},
 {   10558, .8346237304688E+04, -.1427900268555E+04, .1046709716797E+04},
 {   10559, .8333187500000E+04, -.1429102050781E+04, .1052551025391E+04},
 {   10562, .8311216796875E+04, -.1425291992188E+04, .1034557128906E+04},
 {   10563, .8299626953125E+04, -.1425291992188E+04, .1039746093750E+04},
 {   10566, .8316409179688E+04, -.1425291992188E+04, .1046148681641E+04},
 {   10567, .8304816406250E+04, -.1425291992188E+04, .1051337890625E+04},
 {   10570, .8321595703125E+04, -.1425291992188E+04, .1057740600586E+04},
 {   10571, .8310006835938E+04, -.1425291992188E+04, .1062929321289E+04},
 {   10572, .8311216796875E+04, -.1429102050781E+04, .1034557128906E+04},
 {   10573, .8299626953125E+04, -.1429102050781E+04, .1039746093750E+04},
 {   10574, .8316409179688E+04, -.1429102050781E+04, .1046148681641E+04},
 {   10575, .8304816406250E+04, -.1429102050781E+04, .1051337890625E+04},
 {   10576, .8321595703125E+04, -.1429102050781E+04, .1057740600586E+04},
 {   10577, .8310006835938E+04, -.1429102050781E+04, .1062929321289E+04},
 {   10583, .8220398437500E+04, -.1355725219727E+04, .1075213012695E+04},
 {   10584, .8287321289062E+04, -.1424336425781E+04, .1045254638672E+04},
 {   10585, .8258725585938E+04, -.1413383789062E+04, .1058056030273E+04},
 {   10586, .8247830078125E+04, -.1406223754883E+04, .1062933349609E+04},
 {   10587, .8234373046875E+04, -.1392312500000E+04, .1068957519531E+04},
 {   10588, .8224735351562E+04, -.1376822998047E+04, .1073271606445E+04},
 {   10589, .8217052734375E+04, -.1356756713867E+04, .1076711547852E+04},
 {   10595, .8225588867188E+04, -.1355725219727E+04, .1086804809570E+04},
 {   10596, .8292508789062E+04, -.1424336425781E+04, .1056846557617E+04},
 {   10597, .8263917968750E+04, -.1413383789062E+04, .1069647216797E+04},
 {   10598, .8253019531250E+04, -.1406223754883E+04, .1074525024414E+04},
 {   10599, .8239565429688E+04, -.1392312500000E+04, .1080548583984E+04},
 {   10600, .8229926757812E+04, -.1376822998047E+04, .1084863403320E+04},
 {   10601, .8222243164062E+04, -.1356756713867E+04, .1088302856445E+04},
 {   10607, .8230775390625E+04, -.1355725097656E+04, .1098395996094E+04},
 {   10608, .8297699218750E+04, -.1424336425781E+04, .1068437622070E+04},
 {   10609, .8269103515625E+04, -.1413383789062E+04, .1081239624023E+04},
 {   10610, .8258208984375E+04, -.1406223754883E+04, .1086116455078E+04},
 {   10611, .8244751953125E+04, -.1392312500000E+04, .1092140747070E+04},
 {   10612, .8235114257812E+04, -.1376822998047E+04, .1096454833984E+04},
 {   10613, .8227430664062E+04, -.1356756713867E+04, .1099894531250E+04},
 {   10614, .8286574218750E+04, -.1428057861328E+04, .1045588745117E+04},
 {   10615, .8257268554688E+04, -.1416843505859E+04, .1058707763672E+04},
 {   10616, .8245730468750E+04, -.1409260742188E+04, .1063873535156E+04},
 {   10617, .8231726562500E+04, -.1394785522461E+04, .1070142333984E+04},
 {   10618, .8221668945312E+04, -.1378617065430E+04, .1074645874023E+04},
 {   10619, .8213703125000E+04, -.1357788208008E+04, .1078210205078E+04},
 {   10620, .8291766601562E+04, -.1428057861328E+04, .1057180175781E+04},
 {   10621, .8262459960938E+04, -.1416843505859E+04, .1070299316406E+04},
 {   10622, .8250919921875E+04, -.1409260742188E+04, .1075464965820E+04},
 {   10623, .8236916015625E+04, -.1394785522461E+04, .1081733520508E+04},
 {   10624, .8226856445312E+04, -.1378617065430E+04, .1086236816406E+04},
 {   10625, .8218893554688E+04, -.1357788208008E+04, .1089802124023E+04},
 {   10626, .8296953125000E+04, -.1428057861328E+04, .1068772216797E+04},
 {   10627, .8267648437500E+04, -.1416843505859E+04, .1081890869141E+04},
 {   10628, .8256109375000E+04, -.1409260742188E+04, .1087056518555E+04},
 {   10629, .8242104492188E+04, -.1394785522461E+04, .1093324829102E+04},
 {   10630, .8232045898438E+04, -.1378616943359E+04, .1097827758789E+04},
 {   10631, .8224082031250E+04, -.1357788208008E+04, .1101393310547E+04},
 {   10632, .8217803710938E+04, -.1341003662109E+04, .1076375488281E+04},
 {   10633, .8214348632812E+04, -.1341413208008E+04, .1077922607422E+04},
 {   10634, .8222994140625E+04, -.1341003662109E+04, .1087966308594E+04},
 {   10635, .8219534179688E+04, -.1341413208008E+04, .1089515014648E+04},
 {   10636, .8228183593750E+04, -.1341003662109E+04, .1099558349609E+04},
 {   10637, .8224724609375E+04, -.1341413208008E+04, .1101106811523E+04},
 {   10638, .8210889648438E+04, -.1341822753906E+04, .1079470336914E+04},
 {   10639, .8216077148438E+04, -.1341822753906E+04, .1091062622070E+04},
 {   10640, .8221268554688E+04, -.1341822631836E+04, .1102653930664E+04},
 {   10641, .8385983398438E+04, -.1368801147461E+04, .9697799072266E+03},
 {   10643, .8389146484375E+04, -.1370385131836E+04, .9683641357422E+03},
 {   10644, .8385367187500E+04, -.1378023315430E+04, .9700554199219E+03},
 {   10645, .8391173828125E+04, -.1368801147461E+04, .9813710327148E+03},
 {   10647, .8394333984375E+04, -.1370385131836E+04, .9799558105469E+03},
 {   10648, .8390558593750E+04, -.1378023315430E+04, .9816467285156E+03},
 {   10649, .8396362304688E+04, -.1368801147461E+04, .9929628906250E+03},
 {   10651, .8399525390625E+04, -.1370385131836E+04, .9915468750000E+03},
 {   10652, .8395747070312E+04, -.1378023315430E+04, .9932383422852E+03},
 {   10653, .8392309570312E+04, -.1371969238281E+04, .9669484252930E+03},
 {   10654, .8388306640625E+04, -.1380061889648E+04, .9687402343750E+03},
 {   10655, .8397497070312E+04, -.1371969238281E+04, .9785399780273E+03},
 {   10656, .8393496093750E+04, -.1380061889648E+04, .9803315429688E+03},
 {   10657, .8402686523438E+04, -.1371969238281E+04, .9901316528320E+03},
 {   10658, .8398684570312E+04, -.1380061889648E+04, .9919235229492E+03},
 {   10662, .8338223632812E+04, -.1416457397461E+04, .9911601562500E+03},
 {   10665, .8374922851562E+04, -.1394143432617E+04, .9747314453125E+03},
 {   10666, .8364456054688E+04, -.1404927856445E+04, .9794169921875E+03},
 {   10667, .8352493164062E+04, -.1413646606445E+04, .9847729492188E+03},
 {   10668, .8339369140625E+04, -.1420055053711E+04, .9906474609375E+03},
 {   10669, .8323603515625E+04, -.1424143676758E+04, .9977050170898E+03},
 {   10670, .8311132812500E+04, -.1425291992188E+04, .1003287475586E+04},
 {   10674, .8343415039062E+04, -.1416457397461E+04, .1002751831055E+04},
 {   10677, .8380111328125E+04, -.1394143432617E+04, .9863228759766E+03},
 {   10678, .8369644531250E+04, -.1404927734375E+04, .9910084838867E+03},
 {   10679, .8357681640625E+04, -.1413646484375E+04, .9963640136719E+03},
 {   10680, .8344557617188E+04, -.1420055053711E+04, .1002239746094E+04},
 {   10681, .8328793945312E+04, -.1424143676758E+04, .1009296752930E+04},
 {   10682, .8316325195312E+04, -.1425291992188E+04, .1014878845215E+04},
 {   10686, .8348600585938E+04, -.1416457397461E+04, .1014343322754E+04},
 {   10689, .8385300781250E+04, -.1394143432617E+04, .9979141845703E+03},
 {   10690, .8374835937500E+04, -.1404927734375E+04, .1002599975586E+04},
 {   10691, .8362870117188E+04, -.1413646484375E+04, .1007956176758E+04},
 {   10692, .8349747070312E+04, -.1420055053711E+04, .1013830993652E+04},
 {   10693, .8333981445312E+04, -.1424143676758E+04, .1020888000488E+04},
 {   10694, .8321513671875E+04, -.1425291992188E+04, .1026470336914E+04},
 {   10695, .8377509765625E+04, -.1396690673828E+04, .9735737915039E+03},
 {   10696, .8366617187500E+04, -.1407912231445E+04, .9784497070312E+03},
 {   10697, .8354169921875E+04, -.1416984497070E+04, .9840219726562E+03},
 {   10698, .8340513671875E+04, -.1423652832031E+04, .9901354370117E+03},
 {   10699, .8324184570312E+04, -.1427900268555E+04, .9974450683594E+03},
 {   10700, .8311132812500E+04, -.1429102050781E+04, .1003287475586E+04},
 {   10701, .8382698242188E+04, -.1396690673828E+04, .9851651000977E+03},
 {   10702, .8371805664062E+04, -.1407912231445E+04, .9900411987305E+03},
 {   10703, .8359357421875E+04, -.1416984375000E+04, .9956135253906E+03},
 {   10704, .8345701171875E+04, -.1423652709961E+04, .1001727416992E+04},
 {   10705, .8329373046875E+04, -.1427900268555E+04, .1009036621094E+04},
 {   10706, .8316325195312E+04, -.1429102050781E+04, .1014878845215E+04},
 {   10707, .8387886718750E+04, -.1396690673828E+04, .9967568359375E+03},
 {   10708, .8376995117188E+04, -.1407912231445E+04, .1001632446289E+04},
 {   10709, .8364545898438E+04, -.1416984375000E+04, .1007205200195E+04},
 {   10710, .8350892578125E+04, -.1423652709961E+04, .1013318664551E+04},
 {   10711, .8334561523438E+04, -.1427900268555E+04, .1020628356934E+04},
 {   10712, .8321513671875E+04, -.1429102050781E+04, .1026470336914E+04},
 {   10715, .8299541992188E+04, -.1425291992188E+04, .1008476074219E+04},
 {   10716, .8287951171875E+04, -.1425291992188E+04, .1013665771484E+04},
 {   10719, .8304731445312E+04, -.1425291992188E+04, .1020067871094E+04},
 {   10720, .8293141601562E+04, -.1425291992188E+04, .1025256958008E+04},
 {   10723, .8309919921875E+04, -.1425291992188E+04, .1031659423828E+04},
 {   10724, .8298330078125E+04, -.1425291992188E+04, .1036848388672E+04},
 {   10725, .8299541992188E+04, -.1429102050781E+04, .1008476074219E+04},
 {   10726, .8287951171875E+04, -.1429102050781E+04, .1013665771484E+04},
 {   10727, .8304731445312E+04, -.1429102050781E+04, .1020067871094E+04},
 {   10728, .8293141601562E+04, -.1429102050781E+04, .1025256958008E+04},
 {   10729, .8309919921875E+04, -.1429102050781E+04, .1031659423828E+04},
 {   10730, .8298330078125E+04, -.1429102050781E+04, .1036848388672E+04},
 {   10736, .8208722656250E+04, -.1355725219727E+04, .1049131835938E+04},
 {   10737, .8275645507812E+04, -.1424336425781E+04, .1019173583984E+04},
 {   10738, .8247053710938E+04, -.1413383789062E+04, .1031974975586E+04},
 {   10739, .8236155273438E+04, -.1406223754883E+04, .1036852294922E+04},
 {   10740, .8222698242188E+04, -.1392312500000E+04, .1042876831055E+04},
 {   10741, .8213060546875E+04, -.1376822998047E+04, .1047190673828E+04},
 {   10742, .8205376953125E+04, -.1356756713867E+04, .1050630371094E+04},
 {   10748, .8213915039062E+04, -.1355725219727E+04, .1060723388672E+04},
 {   10749, .8280835937500E+04, -.1424336425781E+04, .1030765380859E+04},
 {   10750, .8252241210938E+04, -.1413383789062E+04, .1043566772461E+04},
 {   10751, .8241343750000E+04, -.1406223754883E+04, .1048444091797E+04},
 {   10752, .8227887695312E+04, -.1392312500000E+04, .1054468627930E+04},
 {   10753, .8218249023438E+04, -.1376822998047E+04, .1058781982422E+04},
 {   10754, .8210566406250E+04, -.1356756713867E+04, .1062222045898E+04},
 {   10760, .8219102539062E+04, -.1355725219727E+04, .1072315063477E+04},
 {   10761, .8286022460938E+04, -.1424336425781E+04, .1042357299805E+04},
 {   10762, .8257429687500E+04, -.1413383789062E+04, .1055157836914E+04},
 {   10763, .8246533203125E+04, -.1406223754883E+04, .1060035644531E+04},
 {   10764, .8233076171875E+04, -.1392312500000E+04, .1066059814453E+04},
 {   10765, .8223436523438E+04, -.1376822998047E+04, .1070374023438E+04},
 {   10766, .8215752929688E+04, -.1356756713867E+04, .1073814208984E+04},
 {   10767, .8274900390625E+04, -.1428057861328E+04, .1019508056641E+04},
 {   10768, .8245593750000E+04, -.1416843505859E+04, .1032626831055E+04},
 {   10769, .8234056640625E+04, -.1409260742188E+04, .1037792968750E+04},
 {   10770, .8220054687500E+04, -.1394785522461E+04, .1044060791016E+04},
 {   10771, .8209992187500E+04, -.1378617065430E+04, .1048563964844E+04},
 {   10772, .8202030273438E+04, -.1357788208008E+04, .1052129028320E+04},
 {   10773, .8280087890625E+04, -.1428057861328E+04, .1031099365234E+04},
 {   10774, .8250783203125E+04, -.1416843505859E+04, .1044218505859E+04},
 {   10775, .8239244140625E+04, -.1409260742188E+04, .1049384155273E+04},
 {   10776, .8225243164062E+04, -.1394785522461E+04, .1055651977539E+04},
 {   10777, .8215180664062E+04, -.1378617065430E+04, .1060156005859E+04},
 {   10778, .8207219726562E+04, -.1357788208008E+04, .1063720581055E+04},
 {   10779, .8285277343750E+04, -.1428057861328E+04, .1042691284180E+04},
 {   10780, .8255970703125E+04, -.1416843505859E+04, .1055810302734E+04},
 {   10781, .8244435546875E+04, -.1409260742188E+04, .1060974975586E+04},
 {   10782, .8230431640625E+04, -.1394785522461E+04, .1067243774414E+04},
 {   10783, .8220372070312E+04, -.1378617065430E+04, .1071747558594E+04},
 {   10784, .8212409179688E+04, -.1357788208008E+04, .1075311889648E+04},
 {   10785, .8206129882812E+04, -.1341003662109E+04, .1050294189453E+04},
 {   10786, .8202668945312E+04, -.1341413208008E+04, .1051842529297E+04},
 {   10787, .8211317382812E+04, -.1341003662109E+04, .1061886352539E+04},
 {   10788, .8207861328125E+04, -.1341413208008E+04, .1063433471680E+04},
 {   10789, .8216506835938E+04, -.1341003662109E+04, .1073477661133E+04},
 {   10790, .8213048828125E+04, -.1341413208008E+04, .1075025512695E+04},
 {   10791, .8199211914062E+04, -.1341822753906E+04, .1053390258789E+04},
 {   10792, .8204403320312E+04, -.1341822753906E+04, .1064980957031E+04},
 {   10793, .8209591796875E+04, -.1341822753906E+04, .1076572753906E+04},
 {   10794, .8374308593750E+04, -.1368801147461E+04, .9436990966797E+03},
 {   10796, .8377469726562E+04, -.1370385131836E+04, .9422832031250E+03},
 {   10797, .8373692382812E+04, -.1378023315430E+04, .9439748535156E+03},
 {   10798, .8379497070312E+04, -.1368801147461E+04, .9552908325195E+03},
 {   10800, .8382659179688E+04, -.1370385131836E+04, .9538746948242E+03},
 {   10801, .8378882812500E+04, -.1378023315430E+04, .9555660400391E+03},
 {   10802, .8384685546875E+04, -.1368801147461E+04, .9668822021484E+03},
 {   10804, .8387847656250E+04, -.1370385131836E+04, .9654666748047E+03},
 {   10805, .8384070312500E+04, -.1378023315430E+04, .9671577148438E+03},
 {   10806, .8380632812500E+04, -.1371969238281E+04, .9408677978516E+03},
 {   10807, .8376629882812E+04, -.1380061889648E+04, .9426599121094E+03},
 {   10808, .8385822265625E+04, -.1371969238281E+04, .9524589233398E+03},
 {   10809, .8381819335938E+04, -.1380061889648E+04, .9542507324219E+03},
 {   10810, .8391012695312E+04, -.1371969238281E+04, .9640502929688E+03},
 {   10811, .8387008789062E+04, -.1380061889648E+04, .9658424072266E+03},
 {   10815, .8326548828125E+04, -.1416457397461E+04, .9650794067383E+03},
 {   10818, .8363247070312E+04, -.1394143432617E+04, .9486505737305E+03},
 {   10819, .8352780273438E+04, -.1404927856445E+04, .9533363647461E+03},
 {   10820, .8340816406250E+04, -.1413646606445E+04, .9586920776367E+03},
 {   10821, .8327693359375E+04, -.1420055053711E+04, .9645670776367E+03},
 {   10822, .8311927734375E+04, -.1424143676758E+04, .9716242675781E+03},
 {   10823, .8299458007812E+04, -.1425291992188E+04, .9772065429688E+03},
 {   10827, .8331737304688E+04, -.1416457397461E+04, .9766710205078E+03},
 {   10830, .8368436523438E+04, -.1394143432617E+04, .9602418212891E+03},
 {   10831, .8357969726562E+04, -.1404927856445E+04, .9649278564453E+03},
 {   10832, .8346005859375E+04, -.1413646606445E+04, .9702834472656E+03},
 {   10833, .8332882812500E+04, -.1420055053711E+04, .9761585693359E+03},
 {   10834, .8317118164062E+04, -.1424143676758E+04, .9832154541016E+03},
 {   10835, .8304648437500E+04, -.1425291992188E+04, .9887977294922E+03},
 {   10839, .8336925781250E+04, -.1416457397461E+04, .9882623291016E+03},
 {   10842, .8373625976562E+04, -.1394143432617E+04, .9718335571289E+03},
 {   10843, .8363159179688E+04, -.1404927856445E+04, .9765194091797E+03},
 {   10844, .8351195312500E+04, -.1413646606445E+04, .9818750000000E+03},
 {   10845, .8338071289062E+04, -.1420055053711E+04, .9877503051758E+03},
 {   10846, .8322305664062E+04, -.1424143676758E+04, .9948072509766E+03},
 {   10847, .8309836914062E+04, -.1425291992188E+04, .1000389099121E+04},
 {   10848, .8365835937500E+04, -.1396690673828E+04, .9474926147461E+03},
 {   10849, .8354942382812E+04, -.1407912231445E+04, .9523685913086E+03},
 {   10850, .8342493164062E+04, -.1416984497070E+04, .9579417724609E+03},
 {   10851, .8328836914062E+04, -.1423652832031E+04, .9640548706055E+03},
 {   10852, .8312507812500E+04, -.1427900268555E+04, .9713647460938E+03},
 {   10853, .8299458007812E+04, -.1429102050781E+04, .9772065429688E+03},
 {   10854, .8371021484375E+04, -.1396690673828E+04, .9590845947266E+03},
 {   10855, .8360131835938E+04, -.1407912231445E+04, .9639602050781E+03},
 {   10856, .8347682617188E+04, -.1416984497070E+04, .9695328369141E+03},
 {   10857, .8334025390625E+04, -.1423652832031E+04, .9756461791992E+03},
 {   10858, .8317698242188E+04, -.1427900268555E+04, .9829556884766E+03},
 {   10859, .8304648437500E+04, -.1429102050781E+04, .9887977294922E+03},
 {   10860, .8376210937500E+04, -.1396690673828E+04, .9706757812500E+03},
 {   10861, .8365319335938E+04, -.1407912231445E+04, .9755517578125E+03},
 {   10862, .8352872070312E+04, -.1416984497070E+04, .9811243896484E+03},
 {   10863, .8339217773438E+04, -.1423652832031E+04, .9872373046875E+03},
 {   10864, .8322886718750E+04, -.1427900268555E+04, .9945474853516E+03},
 {   10865, .8309836914062E+04, -.1429102050781E+04, .1000389099121E+04},
 {   10868, .8287866210938E+04, -.1425291992188E+04, .9823955078125E+03},
 {   10869, .8276275390625E+04, -.1425291992188E+04, .9875843505859E+03},
 {   10872, .8293055664062E+04, -.1425291992188E+04, .9939868164062E+03},
 {   10873, .8281464843750E+04, -.1425291992188E+04, .9991760253906E+03},
 {   10876, .8298244140625E+04, -.1425291992188E+04, .1005578613281E+04},
 {   10877, .8286653320312E+04, -.1425291992188E+04, .1010767395020E+04},
 {   10878, .8287866210938E+04, -.1429102050781E+04, .9823955078125E+03},
 {   10879, .8276275390625E+04, -.1429102050781E+04, .9875843505859E+03},
 {   10880, .8293055664062E+04, -.1429102050781E+04, .9939868164062E+03},
 {   10881, .8281464843750E+04, -.1429102050781E+04, .9991760253906E+03},
 {   10882, .8298244140625E+04, -.1429102050781E+04, .1005578613281E+04},
 {   10883, .8286653320312E+04, -.1429102050781E+04, .1010767395020E+04},
 {   10889, .8197049804688E+04, -.1355725219727E+04, .1023051025391E+04},
 {   10890, .8263970703125E+04, -.1424336425781E+04, .9930931396484E+03},
 {   10891, .8235375976562E+04, -.1413383789062E+04, .1005894165039E+04},
 {   10892, .8224478515625E+04, -.1406223754883E+04, .1010771545410E+04},
 {   10893, .8211023437500E+04, -.1392312500000E+04, .1016795654297E+04},
 {   10894, .8201384765625E+04, -.1376822998047E+04, .1021109741211E+04},
 {   10895, .8193701171875E+04, -.1356756713867E+04, .1024549926758E+04},
 {   10901, .8202239257812E+04, -.1355725219727E+04, .1034642089844E+04},
 {   10902, .8269160156250E+04, -.1424336425781E+04, .1004684570312E+04},
 {   10903, .8240566406250E+04, -.1413383789062E+04, .1017485229492E+04},
 {   10904, .8229667968750E+04, -.1406223754883E+04, .1022363647461E+04},
 {   10905, .8216210937500E+04, -.1392312500000E+04, .1028387084961E+04},
 {   10906, .8206573242188E+04, -.1376822998047E+04, .1032701660156E+04},
 {   10907, .8198888671875E+04, -.1356756713867E+04, .1036141113281E+04},
 {   10913, .8207427734375E+04, -.1355725219727E+04, .1046233642578E+04},
 {   10914, .8274347656250E+04, -.1424336425781E+04, .1016276306152E+04},
 {   10915, .8245753906250E+04, -.1413383789062E+04, .1029076904297E+04},
 {   10916, .8234858398438E+04, -.1406223754883E+04, .1033954467773E+04},
 {   10917, .8221404296875E+04, -.1392312500000E+04, .1039978515625E+04},
 {   10918, .8211764648438E+04, -.1376822998047E+04, .1044292602539E+04},
 {   10919, .8204080078125E+04, -.1356756713867E+04, .1047732788086E+04},
 {   10920, .8263224609375E+04, -.1428057861328E+04, .9934270019531E+03},
 {   10921, .8233918945312E+04, -.1416843505859E+04, .1006545898438E+04},
 {   10922, .8222380859375E+04, -.1409260742188E+04, .1011711791992E+04},
 {   10923, .8208380859375E+04, -.1394785522461E+04, .1017979980469E+04},
 {   10924, .8198319335938E+04, -.1378617065430E+04, .1022482788086E+04},
 {   10925, .8190354492188E+04, -.1357788208008E+04, .1026048095703E+04},
 {   10926, .8268413085938E+04, -.1428057861328E+04, .1005018432617E+04},
 {   10927, .8239107421875E+04, -.1416843505859E+04, .1018137207031E+04},
 {   10928, .8227570312500E+04, -.1409260742188E+04, .1023303222656E+04},
 {   10929, .8213566406250E+04, -.1394785522461E+04, .1029571289062E+04},
 {   10930, .8203508789062E+04, -.1378617065430E+04, .1034074584961E+04},
 {   10931, .8195544921875E+04, -.1357788208008E+04, .1037639526367E+04},
 {   10932, .8273602539062E+04, -.1428057861328E+04, .1016609985352E+04},
 {   10933, .8244295898438E+04, -.1416843505859E+04, .1029729003906E+04},
 {   10934, .8232757812500E+04, -.1409260742188E+04, .1034894775391E+04},
 {   10935, .8218757812500E+04, -.1394785522461E+04, .1041162841797E+04},
 {   10936, .8208695312500E+04, -.1378617065430E+04, .1045665893555E+04},
 {   10937, .8200732421875E+04, -.1357788208008E+04, .1049231445312E+04},
 {   10938, .8194452148438E+04, -.1341003662109E+04, .1024213745117E+04},
 {   10939, .8190997558594E+04, -.1341413208008E+04, .1025761230469E+04},
 {   10940, .8199640625000E+04, -.1341003662109E+04, .1035805053711E+04},
 {   10941, .8196182617188E+04, -.1341413208008E+04, .1037353149414E+04},
 {   10942, .8204830078125E+04, -.1341003662109E+04, .1047396484375E+04},
 {   10943, .8201372070312E+04, -.1341413208008E+04, .1048944702148E+04},
 {   10944, .8187536621094E+04, -.1341822753906E+04, .1027309204102E+04},
 {   10945, .8192726562500E+04, -.1341822753906E+04, .1038900634766E+04},
 {   10946, .8197916992188E+04, -.1341822753906E+04, .1050492309570E+04},
 {   10947, .8362632812500E+04, -.1368801147461E+04, .9176179199219E+03},
 {   10949, .8365794921875E+04, -.1370385131836E+04, .9162022094727E+03},
 {   10950, .8362016601562E+04, -.1378023315430E+04, .9178937988281E+03},
 {   10951, .8367821289062E+04, -.1368801147461E+04, .9292096557617E+03},
 {   10953, .8370983398438E+04, -.1370385131836E+04, .9277937011719E+03},
 {   10954, .8367207031250E+04, -.1378023315430E+04, .9294855346680E+03},
 {   10955, .8373010742188E+04, -.1368801147461E+04, .9408011474609E+03},
 {   10957, .8376174804688E+04, -.1370385131836E+04, .9393851318359E+03},
 {   10958, .8372394531250E+04, -.1378023315430E+04, .9410769042969E+03},
 {   10959, .8368958984375E+04, -.1371969238281E+04, .9147863159180E+03},
 {   10960, .8364955078125E+04, -.1380061889648E+04, .9165784912109E+03},
 {   10961, .8374147460938E+04, -.1371969238281E+04, .9263778686523E+03},
 {   10962, .8370144531250E+04, -.1380061889648E+04, .9281700439453E+03},
 {   10963, .8379334960938E+04, -.1371969238281E+04, .9379694213867E+03},
 {   10964, .8375333007812E+04, -.1380061889648E+04, .9397619018555E+03},
 {   10968, .8314873046875E+04, -.1416457397461E+04, .9389984741211E+03},
 {   10971, .8351572265625E+04, -.1394143432617E+04, .9225694580078E+03},
 {   10972, .8341104492188E+04, -.1404927856445E+04, .9272556152344E+03},
 {   10973, .8329143554688E+04, -.1413646606445E+04, .9326108398438E+03},
 {   10974, .8316017578125E+04, -.1420055053711E+04, .9384860839844E+03},
 {   10975, .8300252929688E+04, -.1424143676758E+04, .9455427246094E+03},
 {   10976, .8287782226562E+04, -.1425291992188E+04, .9511254882812E+03},
 {   10980, .8320062500000E+04, -.1416457397461E+04, .9505900268555E+03},
 {   10983, .8356759765625E+04, -.1394143432617E+04, .9341611328125E+03},
 {   10984, .8346293945312E+04, -.1404927856445E+04, .9388468017578E+03},
 {   10985, .8334331054688E+04, -.1413646606445E+04, .9442025146484E+03},
 {   10986, .8321206054688E+04, -.1420055053711E+04, .9500776367188E+03},
 {   10987, .8305441406250E+04, -.1424143676758E+04, .9571346435547E+03},
 {   10988, .8292974609375E+04, -.1425291992188E+04, .9627166748047E+03},
 {   10992, .8325250976562E+04, -.1416457397461E+04, .9621814575195E+03},
 {   10995, .8361949218750E+04, -.1394143432617E+04, .9457529296875E+03},
 {   10996, .8351482421875E+04, -.1404927856445E+04, .9504385375977E+03},
 {   10997, .8339519531250E+04, -.1413646606445E+04, .9557941284180E+03},
 {   10998, .8326396484375E+04, -.1420055053711E+04, .9616689453125E+03},
 {   10999, .8310628906250E+04, -.1424143676758E+04, .9687261962891E+03},
 {   11000, .8298160156250E+04, -.1425291992188E+04, .9743084106445E+03},
 {   11001, .8354158203125E+04, -.1396690673828E+04, .9214119873047E+03},
 {   11002, .8343266601562E+04, -.1407912231445E+04, .9262877197266E+03},
 {   11003, .8330817382812E+04, -.1416984497070E+04, .9318605346680E+03},
 {   11004, .8317161132812E+04, -.1423652832031E+04, .9379736938477E+03},
 {   11005, .8300833007812E+04, -.1427900268555E+04, .9452835693359E+03},
 {   11006, .8287782226562E+04, -.1429102050781E+04, .9511254882812E+03},
 {   11007, .8359346679688E+04, -.1396690673828E+04, .9330034179688E+03},
 {   11008, .8348454101562E+04, -.1407912231445E+04, .9378792724609E+03},
 {   11009, .8336008789062E+04, -.1416984497070E+04, .9434520263672E+03},
 {   11010, .8322352539062E+04, -.1423652832031E+04, .9495649414062E+03},
 {   11011, .8306023437500E+04, -.1427900268555E+04, .9568748779297E+03},
 {   11012, .8292974609375E+04, -.1429102050781E+04, .9627166748047E+03},
 {   11013, .8364538085938E+04, -.1396690673828E+04, .9445949707031E+03},
 {   11014, .8353644531250E+04, -.1407912231445E+04, .9494708251953E+03},
 {   11015, .8341197265625E+04, -.1416984497070E+04, .9550435791016E+03},
 {   11016, .8327540039062E+04, -.1423652832031E+04, .9611567382812E+03},
 {   11017, .8311211914062E+04, -.1427900268555E+04, .9684664306641E+03},
 {   11018, .8298160156250E+04, -.1429102050781E+04, .9743084106445E+03},
 {   11021, .8276192382812E+04, -.1425291992188E+04, .9563143310547E+03},
 {   11022, .8264600585938E+04, -.1425291992188E+04, .9615036621094E+03},
 {   11025, .8281380859375E+04, -.1425291992188E+04, .9679062500000E+03},
 {   11026, .8269791015625E+04, -.1425291992188E+04, .9730949707031E+03},
 {   11029, .8286571289062E+04, -.1425291992188E+04, .9794973144531E+03},
 {   11030, .8274979492188E+04, -.1425291992188E+04, .9846864624023E+03},
 {   11031, .8276192382812E+04, -.1429102050781E+04, .9563143310547E+03},
 {   11032, .8264600585938E+04, -.1429102050781E+04, .9615036621094E+03},
 {   11033, .8281380859375E+04, -.1429102050781E+04, .9679062500000E+03},
 {   11034, .8269791015625E+04, -.1429102050781E+04, .9730949707031E+03},
 {   11035, .8286571289062E+04, -.1429102050781E+04, .9794973144531E+03},
 {   11036, .8274979492188E+04, -.1429102050781E+04, .9846864624023E+03},
 {   11042, .8185374023438E+04, -.1355725219727E+04, .9969697265625E+03},
 {   11043, .8252296875000E+04, -.1424336425781E+04, .9670120849609E+03},
 {   11044, .8223698242188E+04, -.1413383789062E+04, .9798132324219E+03},
 {   11045, .8212803710938E+04, -.1406223754883E+04, .9846909179688E+03},
 {   11046, .8199348632812E+04, -.1392312500000E+04, .9907144165039E+03},
 {   11047, .8189710449219E+04, -.1376822998047E+04, .9950286865234E+03},
 {   11048, .8182025390625E+04, -.1356756713867E+04, .9984691162109E+03},
 {   11054, .8190562500000E+04, -.1355725219727E+04, .1008562133789E+04},
 {   11055, .8257483398438E+04, -.1424336425781E+04, .9786036376953E+03},
 {   11056, .8228887695312E+04, -.1413383789062E+04, .9914047851562E+03},
 {   11057, .8217992187500E+04, -.1406223754883E+04, .9962824707031E+03},
 {   11058, .8204539062500E+04, -.1392312500000E+04, .1002305664062E+04},
 {   11059, .8194900390625E+04, -.1376822998047E+04, .1006620361328E+04},
 {   11060, .8187217285156E+04, -.1356756713867E+04, .1010060058594E+04},
 {   11066, .8195753906250E+04, -.1355725219727E+04, .1020152832031E+04},
 {   11067, .8262673828125E+04, -.1424336425781E+04, .9901954956055E+03},
 {   11068, .8234077148438E+04, -.1413383789062E+04, .1002996093750E+04},
 {   11069, .8223180664062E+04, -.1406223754883E+04, .1007873840332E+04},
 {   11070, .8209722656250E+04, -.1392312500000E+04, .1013898193359E+04},
 {   11071, .8200087890625E+04, -.1376822998047E+04, .1018211669922E+04},
 {   11072, .8192405273438E+04, -.1356756713867E+04, .1021651489258E+04},
 {   11073, .8251548828125E+04, -.1428057861328E+04, .9673459472656E+03},
 {   11074, .8222243164062E+04, -.1416843505859E+04, .9804653930664E+03},
 {   11075, .8210705078125E+04, -.1409260742188E+04, .9856306762695E+03},
 {   11076, .8196702148438E+04, -.1394785522461E+04, .9918986816406E+03},
 {   11077, .8186642578125E+04, -.1378617065430E+04, .9964017333984E+03},
 {   11078, .8178680175781E+04, -.1357788208008E+04, .9999671020508E+03},
 {   11079, .8256737304688E+04, -.1428057861328E+04, .9789376220703E+03},
 {   11080, .8227434570312E+04, -.1416843505859E+04, .9920560913086E+03},
 {   11081, .8215892578125E+04, -.1409260742188E+04, .9972222290039E+03},
 {   11082, .8201891601562E+04, -.1394785522461E+04, .1003490234375E+04},
 {   11083, .8191831054688E+04, -.1378617065430E+04, .1007993896484E+04},
 {   11084, .8183867187500E+04, -.1357788208008E+04, .1011558898926E+04},
 {   11085, .8261926757812E+04, -.1428057861328E+04, .9905292358398E+03},
 {   11086, .8232622070312E+04, -.1416843505859E+04, .1003648071289E+04},
 {   11087, .8221083007812E+04, -.1409260742188E+04, .1008813842773E+04},
 {   11088, .8207079101562E+04, -.1394785522461E+04, .1015082153320E+04},
 {   11089, .8197020507812E+04, -.1378617065430E+04, .1019585327148E+04},
 {   11090, .8189055175781E+04, -.1357788208008E+04, .1023150268555E+04},
 {   11091, .8182776367188E+04, -.1341003662109E+04, .9981326904297E+03},
 {   11092, .8179319824219E+04, -.1341413208008E+04, .9996801757812E+03},
 {   11093, .8187967285156E+04, -.1341003662109E+04, .1009723876953E+04},
 {   11094, .8184507324219E+04, -.1341413208008E+04, .1011271972656E+04},
 {   11095, .8193154296875E+04, -.1341003662109E+04, .1021315612793E+04},
 {   11096, .8189698730469E+04, -.1341413208008E+04, .1022863403320E+04},
 {   11097, .8175862304688E+04, -.1341822753906E+04, .1001227722168E+04},
 {   11098, .8181050781250E+04, -.1341822753906E+04, .1012819702148E+04},
 {   11099, .8186241699219E+04, -.1341822753906E+04, .1024410888672E+04},
 {   11100, .8443572265625E+04, -.1387192016602E+04, .1102888671875E+04},
 {   11101, .8417715820312E+04, -.1387192016602E+04, .1109649414062E+04},
 {   11102, .8391859375000E+04, -.1387192016602E+04, .1116410034180E+04},
 {   11103, .8366005859375E+04, -.1387192016602E+04, .1123170776367E+04},
 {   11104, .8443572265625E+04, -.1361791992188E+04, .1102888671875E+04},
 {   11105, .8417715820312E+04, -.1361791992188E+04, .1109649414062E+04},
 {   11106, .8391859375000E+04, -.1361791992188E+04, .1116410034180E+04},
 {   11107, .8366005859375E+04, -.1361791992188E+04, .1123170776367E+04},
 {   11108, .8443572265625E+04, -.1336392089844E+04, .1102888671875E+04},
 {   11109, .8417715820312E+04, -.1336392089844E+04, .1109649414062E+04},
 {   11110, .8391859375000E+04, -.1336392089844E+04, .1116410034180E+04},
 {   11111, .8366005859375E+04, -.1336392089844E+04, .1123170776367E+04},
 {   11112, .8449513671875E+04, -.1387192016602E+04, .1125612792969E+04},
 {   11113, .8424918945312E+04, -.1387192016602E+04, .1132043945312E+04},
 {   11114, .8400327148438E+04, -.1387192016602E+04, .1138474121094E+04},
 {   11115, .8375732421875E+04, -.1387192016602E+04, .1144905151367E+04},
 {   11116, .8449513671875E+04, -.1361791992188E+04, .1125612792969E+04},
 {   11117, .8424918945312E+04, -.1361791992188E+04, .1132043945312E+04},
 {   11118, .8400327148438E+04, -.1361791992188E+04, .1138474121094E+04},
 {   11119, .8375732421875E+04, -.1361791992188E+04, .1144905151367E+04},
 {   11120, .8454465820312E+04, -.1380842041016E+04, .1144550170898E+04},
 {   11121, .8430923828125E+04, -.1382958740234E+04, .1150705444336E+04},
 {   11122, .8407384765625E+04, -.1385075317383E+04, .1156861083984E+04},
 {   11123, .8383843750000E+04, -.1387192016602E+04, .1163016967773E+04},
 {   11124, .8454465820312E+04, -.1358617065430E+04, .1144550170898E+04},
 {   11125, .8430923828125E+04, -.1359675415039E+04, .1150705444336E+04},
 {   11126, .8407384765625E+04, -.1360733642578E+04, .1156861083984E+04},
 {   11127, .8383843750000E+04, -.1361791992188E+04, .1163016967773E+04},
 {   11128, .8454465820312E+04, -.1336392089844E+04, .1144550170898E+04},
 {   11129, .8430923828125E+04, -.1336392089844E+04, .1150705444336E+04},
 {   11130, .8407384765625E+04, -.1336392089844E+04, .1156861083984E+04},
 {   11131, .8383843750000E+04, -.1336392089844E+04, .1163016967773E+04},
 {   11132, .8459418945312E+04, -.1374492065430E+04, .1163487670898E+04},
 {   11133, .8436927734375E+04, -.1378725341797E+04, .1169367431641E+04},
 {   11134, .8414439453125E+04, -.1382958740234E+04, .1175248291016E+04},
 {   11135, .8391950195312E+04, -.1387192016602E+04, .1181129028320E+04},
 {   11136, .8459418945312E+04, -.1355442016602E+04, .1163487670898E+04},
 {   11137, .8436927734375E+04, -.1357558715820E+04, .1169367431641E+04},
 {   11138, .8414439453125E+04, -.1359675415039E+04, .1175248291016E+04},
 {   11139, .8391950195312E+04, -.1361791992188E+04, .1181129028320E+04},
 {   11140, .8459418945312E+04, -.1336392089844E+04, .1163487670898E+04},
 {   11141, .8436927734375E+04, -.1336392089844E+04, .1169367431641E+04},
 {   11142, .8414439453125E+04, -.1336392089844E+04, .1175248291016E+04},
 {   11143, .8391950195312E+04, -.1336392089844E+04, .1181129028320E+04},
 {   11144, .8443572265625E+04, -.1234791992188E+04, .1102888671875E+04},
 {   11145, .8417715820312E+04, -.1234791992188E+04, .1109649414062E+04},
 {   11146, .8391859375000E+04, -.1234791992188E+04, .1116410034180E+04},
 {   11147, .8366005859375E+04, -.1234791992188E+04, .1123170776367E+04},
 {   11148, .8443572265625E+04, -.1209392089844E+04, .1102888671875E+04},
 {   11149, .8417715820312E+04, -.1209392089844E+04, .1109649414062E+04},
 {   11150, .8391859375000E+04, -.1209392089844E+04, .1116410034180E+04},
 {   11151, .8366005859375E+04, -.1209392089844E+04, .1123170776367E+04},
 {   11152, .8443572265625E+04, -.1183992065430E+04, .1102888671875E+04},
 {   11153, .8417715820312E+04, -.1183992065430E+04, .1109649414062E+04},
 {   11154, .8391859375000E+04, -.1183992065430E+04, .1116410034180E+04},
 {   11155, .8366005859375E+04, -.1183992065430E+04, .1123170776367E+04},
 {   11156, .8449513671875E+04, -.1209392089844E+04, .1125612792969E+04},
 {   11157, .8424918945312E+04, -.1209392089844E+04, .1132043945312E+04},
 {   11158, .8400327148438E+04, -.1209392089844E+04, .1138474121094E+04},
 {   11159, .8375732421875E+04, -.1209392089844E+04, .1144905151367E+04},
 {   11160, .8449513671875E+04, -.1183992065430E+04, .1125612792969E+04},
 {   11161, .8424918945312E+04, -.1183992065430E+04, .1132043945312E+04},
 {   11162, .8400327148438E+04, -.1183992065430E+04, .1138474121094E+04},
 {   11163, .8375732421875E+04, -.1183992065430E+04, .1144905151367E+04},
 {   11164, .8454465820312E+04, -.1234791992188E+04, .1144550170898E+04},
 {   11165, .8430923828125E+04, -.1234791992188E+04, .1150705444336E+04},
 {   11166, .8407384765625E+04, -.1234791992188E+04, .1156861083984E+04},
 {   11167, .8383843750000E+04, -.1234791992188E+04, .1163016967773E+04},
 {   11168, .8454465820312E+04, -.1212567016602E+04, .1144550170898E+04},
 {   11169, .8430923828125E+04, -.1211508666992E+04, .1150705444336E+04},
 {   11170, .8407384765625E+04, -.1210450317383E+04, .1156861083984E+04},
 {   11171, .8383843750000E+04, -.1209392089844E+04, .1163016967773E+04},
 {   11172, .8454465820312E+04, -.1190342041016E+04, .1144550170898E+04},
 {   11173, .8430923828125E+04, -.1188225341797E+04, .1150705444336E+04},
 {   11174, .8407384765625E+04, -.1186108642578E+04, .1156861083984E+04},
 {   11175, .8383843750000E+04, -.1183992065430E+04, .1163016967773E+04},
 {   11176, .8459418945312E+04, -.1234791992188E+04, .1163487670898E+04},
 {   11177, .8436927734375E+04, -.1234791992188E+04, .1169367431641E+04},
 {   11178, .8414439453125E+04, -.1234791992188E+04, .1175248291016E+04},
 {   11179, .8391950195312E+04, -.1234791992188E+04, .1181129028320E+04},
 {   11180, .8459418945312E+04, -.1215742065430E+04, .1163487670898E+04},
 {   11181, .8436927734375E+04, -.1213625366211E+04, .1169367431641E+04},
 {   11182, .8414439453125E+04, -.1211508666992E+04, .1175248291016E+04},
 {   11183, .8391950195312E+04, -.1209392089844E+04, .1181129028320E+04},
 {   11184, .8459418945312E+04, -.1196692016602E+04, .1163487670898E+04},
 {   11185, .8436927734375E+04, -.1192458740234E+04, .1169367431641E+04},
 {   11186, .8414439453125E+04, -.1188225341797E+04, .1175248291016E+04},
 {   11187, .8391950195312E+04, -.1183992065430E+04, .1181129028320E+04},
 {   11188, .8338129882812E+04, -.1387192016602E+04, .1130459960938E+04},
 {   11189, .8310255859375E+04, -.1387192016602E+04, .1137748535156E+04},
 {   11190, .8282379882812E+04, -.1387192016602E+04, .1145038208008E+04},
 {   11191, .8338129882812E+04, -.1361791992188E+04, .1130459960938E+04},
 {   11192, .8310255859375E+04, -.1361791992188E+04, .1137748535156E+04},
 {   11193, .8282379882812E+04, -.1361791992188E+04, .1145038208008E+04},
 {   11194, .8338129882812E+04, -.1336392089844E+04, .1130459960938E+04},
 {   11195, .8310255859375E+04, -.1336392089844E+04, .1137748535156E+04},
 {   11196, .8282379882812E+04, -.1336392089844E+04, .1145038208008E+04},
 {   11197, .8347859375000E+04, -.1387192016602E+04, .1152194335938E+04},
 {   11198, .8319983398438E+04, -.1387192016602E+04, .1159483032227E+04},
 {   11199, .8292108398438E+04, -.1387192016602E+04, .1166772827148E+04},
 {   11200, .8347859375000E+04, -.1361791992188E+04, .1152194335938E+04},
 {   11201, .8319983398438E+04, -.1361791992188E+04, .1159483032227E+04},
 {   11202, .8292108398438E+04, -.1361791992188E+04, .1166772827148E+04},
 {   11203, .8347859375000E+04, -.1336392089844E+04, .1152194335938E+04},
 {   11204, .8319983398438E+04, -.1336392089844E+04, .1159483032227E+04},
 {   11205, .8292108398438E+04, -.1336392089844E+04, .1166772827148E+04},
 {   11206, .8357987304688E+04, -.1385075317383E+04, .1169777587891E+04},
 {   11207, .8332132812500E+04, -.1382958740234E+04, .1176538330078E+04},
 {   11208, .8306276367188E+04, -.1380842041016E+04, .1183299072266E+04},
 {   11209, .8356977539062E+04, -.1360733642578E+04, .1170041625977E+04},
 {   11210, .8330111328125E+04, -.1359675415039E+04, .1177066772461E+04},
 {   11211, .8303247070312E+04, -.1358617065430E+04, .1184091552734E+04},
 {   11212, .8355966796875E+04, -.1336392089844E+04, .1170306274414E+04},
 {   11213, .8328090820312E+04, -.1336392089844E+04, .1177595214844E+04},
 {   11214, .8300215820312E+04, -.1336392089844E+04, .1184883789062E+04},
 {   11215, .8368115234375E+04, -.1382958740234E+04, .1187361328125E+04},
 {   11216, .8344278320312E+04, -.1378725341797E+04, .1193593872070E+04},
 {   11217, .8320444335938E+04, -.1374492065430E+04, .1199826660156E+04},
 {   11218, .8366095703125E+04, -.1359675415039E+04, .1187889282227E+04},
 {   11219, .8340242187500E+04, -.1357558715820E+04, .1194650146484E+04},
 {   11220, .8314385742188E+04, -.1355442016602E+04, .1201411376953E+04},
 {   11221, .8364075195312E+04, -.1336392089844E+04, .1188418090820E+04},
 {   11222, .8336200195312E+04, -.1336392089844E+04, .1195706787109E+04},
 {   11223, .8308325195312E+04, -.1336392089844E+04, .1202995727539E+04},
 {   11224, .8366005859375E+04, -.1317342041016E+04, .1123170776367E+04},
 {   11225, .8338129882812E+04, -.1317342041016E+04, .1130459960938E+04},
 {   11226, .8310255859375E+04, -.1317342041016E+04, .1137748535156E+04},
 {   11227, .8282379882812E+04, -.1317342041016E+04, .1145038208008E+04},
 {   11228, .8347859375000E+04, -.1317342041016E+04, .1152194335938E+04},
 {   11229, .8319983398438E+04, -.1317342041016E+04, .1159483032227E+04},
 {   11230, .8292108398438E+04, -.1317342041016E+04, .1166772827148E+04},
 {   11231, .8383843750000E+04, -.1317342041016E+04, .1163016967773E+04},
 {   11232, .8355966796875E+04, -.1317342041016E+04, .1170306274414E+04},
 {   11233, .8328090820312E+04, -.1317342041016E+04, .1177595214844E+04},
 {   11234, .8300215820312E+04, -.1317342041016E+04, .1184883789062E+04},
 {   11235, .8391950195312E+04, -.1317342041016E+04, .1181129028320E+04},
 {   11236, .8364075195312E+04, -.1317342041016E+04, .1188418090820E+04},
 {   11237, .8336200195312E+04, -.1317342041016E+04, .1195706787109E+04},
 {   11238, .8308325195312E+04, -.1317342041016E+04, .1202995727539E+04},
 {   11239, .8366005859375E+04, -.1295117065430E+04, .1123170776367E+04},
 {   11240, .8338129882812E+04, -.1295117065430E+04, .1130459960938E+04},
 {   11241, .8310255859375E+04, -.1295117065430E+04, .1137748535156E+04},
 {   11242, .8282379882812E+04, -.1295117065430E+04, .1145038208008E+04},
 {   11243, .8347859375000E+04, -.1295117065430E+04, .1152194335938E+04},
 {   11244, .8319983398438E+04, -.1295117065430E+04, .1159483032227E+04},
 {   11245, .8292108398438E+04, -.1295117065430E+04, .1166772827148E+04},
 {   11246, .8383843750000E+04, -.1295117065430E+04, .1163016967773E+04},
 {   11247, .8355966796875E+04, -.1295117065430E+04, .1170306274414E+04},
 {   11248, .8328090820312E+04, -.1295117065430E+04, .1177595214844E+04},
 {   11249, .8300215820312E+04, -.1295117065430E+04, .1184883789062E+04},
 {   11250, .8391950195312E+04, -.1295117065430E+04, .1181129028320E+04},
 {   11251, .8364075195312E+04, -.1295117065430E+04, .1188418090820E+04},
 {   11252, .8336200195312E+04, -.1295117065430E+04, .1195706787109E+04},
 {   11253, .8308325195312E+04, -.1295117065430E+04, .1202995727539E+04},
 {   11254, .8366005859375E+04, -.1276067016602E+04, .1123170776367E+04},
 {   11255, .8338129882812E+04, -.1276067016602E+04, .1130459960938E+04},
 {   11256, .8310255859375E+04, -.1276067016602E+04, .1137748535156E+04},
 {   11257, .8282379882812E+04, -.1276067016602E+04, .1145038208008E+04},
 {   11258, .8347859375000E+04, -.1276067016602E+04, .1152194335938E+04},
 {   11259, .8319983398438E+04, -.1276067016602E+04, .1159483032227E+04},
 {   11260, .8292108398438E+04, -.1276067016602E+04, .1166772827148E+04},
 {   11261, .8383843750000E+04, -.1276067016602E+04, .1163016967773E+04},
 {   11262, .8355966796875E+04, -.1276067016602E+04, .1170306274414E+04},
 {   11263, .8328090820312E+04, -.1276067016602E+04, .1177595214844E+04},
 {   11264, .8300215820312E+04, -.1276067016602E+04, .1184883789062E+04},
 {   11265, .8391950195312E+04, -.1276067016602E+04, .1181129028320E+04},
 {   11266, .8364075195312E+04, -.1276067016602E+04, .1188418090820E+04},
 {   11267, .8336200195312E+04, -.1276067016602E+04, .1195706787109E+04},
 {   11268, .8308325195312E+04, -.1276067016602E+04, .1202995727539E+04},
 {   11269, .8366005859375E+04, -.1253842041016E+04, .1123170776367E+04},
 {   11270, .8338129882812E+04, -.1253842041016E+04, .1130459960938E+04},
 {   11271, .8310255859375E+04, -.1253842041016E+04, .1137748535156E+04},
 {   11272, .8282379882812E+04, -.1253842041016E+04, .1145038208008E+04},
 {   11273, .8347859375000E+04, -.1253842041016E+04, .1152194335938E+04},
 {   11274, .8319983398438E+04, -.1253842041016E+04, .1159483032227E+04},
 {   11275, .8292108398438E+04, -.1253842041016E+04, .1166772827148E+04},
 {   11276, .8383843750000E+04, -.1253842041016E+04, .1163016967773E+04},
 {   11277, .8355966796875E+04, -.1253842041016E+04, .1170306274414E+04},
 {   11278, .8328090820312E+04, -.1253842041016E+04, .1177595214844E+04},
 {   11279, .8300215820312E+04, -.1253842041016E+04, .1184883789062E+04},
 {   11280, .8391950195312E+04, -.1253842041016E+04, .1181129028320E+04},
 {   11281, .8364075195312E+04, -.1253842041016E+04, .1188418090820E+04},
 {   11282, .8336200195312E+04, -.1253842041016E+04, .1195706787109E+04},
 {   11283, .8308325195312E+04, -.1253842041016E+04, .1202995727539E+04},
 {   11284, .8338129882812E+04, -.1234791992188E+04, .1130459960938E+04},
 {   11285, .8310255859375E+04, -.1234791992188E+04, .1137748535156E+04},
 {   11286, .8282379882812E+04, -.1234791992188E+04, .1145038208008E+04},
 {   11287, .8347859375000E+04, -.1234791992188E+04, .1152194335938E+04},
 {   11288, .8319983398438E+04, -.1234791992188E+04, .1159483032227E+04},
 {   11289, .8292108398438E+04, -.1234791992188E+04, .1166772827148E+04},
 {   11290, .8355966796875E+04, -.1234791992188E+04, .1170306274414E+04},
 {   11291, .8328090820312E+04, -.1234791992188E+04, .1177595214844E+04},
 {   11292, .8300215820312E+04, -.1234791992188E+04, .1184883789062E+04},
 {   11293, .8364075195312E+04, -.1234791992188E+04, .1188418090820E+04},
 {   11294, .8336200195312E+04, -.1234791992188E+04, .1195706787109E+04},
 {   11295, .8308325195312E+04, -.1234791992188E+04, .1202995727539E+04},
 {   11296, .8338129882812E+04, -.1209392089844E+04, .1130459960938E+04},
 {   11297, .8310255859375E+04, -.1209392089844E+04, .1137748535156E+04},
 {   11298, .8282379882812E+04, -.1209392089844E+04, .1145038208008E+04},
 {   11299, .8338129882812E+04, -.1183992065430E+04, .1130459960938E+04},
 {   11300, .8310255859375E+04, -.1183992065430E+04, .1137748535156E+04},
 {   11301, .8282379882812E+04, -.1183992065430E+04, .1145038208008E+04},
 {   11302, .8347859375000E+04, -.1209392089844E+04, .1152194335938E+04},
 {   11303, .8319983398438E+04, -.1209392089844E+04, .1159483032227E+04},
 {   11304, .8292108398438E+04, -.1209392089844E+04, .1166772827148E+04},
 {   11305, .8347859375000E+04, -.1183992065430E+04, .1152194335938E+04},
 {   11306, .8319983398438E+04, -.1183992065430E+04, .1159483032227E+04},
 {   11307, .8292108398438E+04, -.1183992065430E+04, .1166772827148E+04},
 {   11308, .8356977539062E+04, -.1210450317383E+04, .1170041625977E+04},
 {   11309, .8330111328125E+04, -.1211508666992E+04, .1177066772461E+04},
 {   11310, .8303247070312E+04, -.1212567016602E+04, .1184091552734E+04},
 {   11311, .8357987304688E+04, -.1186108642578E+04, .1169777587891E+04},
 {   11312, .8332132812500E+04, -.1188225341797E+04, .1176538330078E+04},
 {   11313, .8306276367188E+04, -.1190342041016E+04, .1183299072266E+04},
 {   11314, .8366095703125E+04, -.1211508666992E+04, .1187889282227E+04},
 {   11315, .8340242187500E+04, -.1213625366211E+04, .1194650146484E+04},
 {   11316, .8314385742188E+04, -.1215742065430E+04, .1201411376953E+04},
 {   11317, .8368115234375E+04, -.1188225341797E+04, .1187361328125E+04},
 {   11318, .8344278320312E+04, -.1192458740234E+04, .1193593872070E+04},
 {   11319, .8320443359375E+04, -.1196692016602E+04, .1199826416016E+04},
 {   11320, .8404485351562E+04, -.1122758666992E+04, .1121844970703E+04},
 {   11321, .8381972656250E+04, -.1118620239258E+04, .1112112426758E+04},
 {   11322, .8401159179688E+04, -.1100165039062E+04, .1119989013672E+04},
 {   11323, .8382029296875E+04, -.1103046142578E+04, .1112086303711E+04},
 {   11324, .8391325195312E+04, -.1079944824219E+04, .1115582763672E+04},
 {   11325, .8382090820312E+04, -.1087472045898E+04, .1112058715820E+04},
 {   11326, .8409369140625E+04, -.1128391967773E+04, .1094268310547E+04},
 {   11327, .8375225585938E+04, -.1118620239258E+04, .1097043823242E+04},
 {   11328, .8405207031250E+04, -.1098509643555E+04, .1094110473633E+04},
 {   11329, .8375284179688E+04, -.1103046142578E+04, .1097016845703E+04},
 {   11330, .8389226562500E+04, -.1073224121094E+04, .1095620361328E+04},
 {   11331, .8375346679688E+04, -.1087472045898E+04, .1096989501953E+04},
 {   11332, .8402986328125E+04, -.1130790893555E+04, .1066527343750E+04},
 {   11333, .8368478515625E+04, -.1118620239258E+04, .1081974731445E+04},
 {   11334, .8399291015625E+04, -.1098207885742E+04, .1068181640625E+04},
 {   11335, .8368539062500E+04, -.1103046142578E+04, .1081948242188E+04},
 {   11336, .8382629882812E+04, -.1070915771484E+04, .1075640014648E+04},
 {   11337, .8368596679688E+04, -.1087472045898E+04, .1081921508789E+04},
 {   11338, .8387418945312E+04, -.1130790893555E+04, .1031752441406E+04},
 {   11339, .8352910156250E+04, -.1118620239258E+04, .1047200073242E+04},
 {   11340, .8383725585938E+04, -.1098207885742E+04, .1033407104492E+04},
 {   11341, .8352972656250E+04, -.1103046142578E+04, .1047172973633E+04},
 {   11342, .8367062500000E+04, -.1070915771484E+04, .1040865356445E+04},
 {   11343, .8353030273438E+04, -.1087472045898E+04, .1047146728516E+04},
 {   11344, .8371851562500E+04, -.1130790893555E+04, .9969778442383E+03},
 {   11345, .8337343750000E+04, -.1118620239258E+04, .1012425842285E+04},
 {   11346, .8368157226562E+04, -.1098207885742E+04, .9986320800781E+03},
 {   11347, .8337405273438E+04, -.1103046142578E+04, .1012398986816E+04},
 {   11348, .8351494140625E+04, -.1070915771484E+04, .1006090515137E+04},
 {   11349, .8337463867188E+04, -.1087472045898E+04, .1012371765137E+04},
 {   11350, .8353689453125E+04, -.1130791015625E+04, .9564077148438E+03},
 {   11351, .8319183593750E+04, -.1118620239258E+04, .9718551025391E+03},
 {   11352, .8349993164062E+04, -.1098207885742E+04, .9580620727539E+03},
 {   11353, .8319242187500E+04, -.1103046142578E+04, .9718286132812E+03},
 {   11354, .8333335937500E+04, -.1070915771484E+04, .9655201416016E+03},
 {   11355, .8319302734375E+04, -.1087472045898E+04, .9718016357422E+03},
 {   11356, .8335528320312E+04, -.1130791015625E+04, .9158373413086E+03},
 {   11357, .8301020507812E+04, -.1118620239258E+04, .9312851562500E+03},
 {   11358, .8331833984375E+04, -.1098207885742E+04, .9174913330078E+03},
 {   11359, .8301080078125E+04, -.1103046142578E+04, .9312581787109E+03},
 {   11360, .8315170898438E+04, -.1070915771484E+04, .9249499511719E+03},
 {   11361, .8301140625000E+04, -.1087472045898E+04, .9312313232422E+03},
 {   11362, .8319087890625E+04, -.1128391967773E+04, .8925979614258E+03},
 {   11363, .8294276367188E+04, -.1118620239258E+04, .9162157592773E+03},
 {   11364, .8316433593750E+04, -.1098509643555E+04, .8958073730469E+03},
 {   11365, .8294333984375E+04, -.1103046142578E+04, .9161892089844E+03},
 {   11366, .8304663085938E+04, -.1073224121094E+04, .9067205810547E+03},
 {   11367, .8294394531250E+04, -.1087472045898E+04, .9161624145508E+03},
 {   11368, .8295269531250E+04, -.1122758789062E+04, .8778710937500E+03},
 {   11369, .8287529296875E+04, -.1118620239258E+04, .9011470336914E+03},
 {   11370, .8294436523438E+04, -.1100165039062E+04, .8815879516602E+03},
 {   11371, .8287589843750E+04, -.1103046142578E+04, .9011201782227E+03},
 {   11372, .8291172851562E+04, -.1079944824219E+04, .8918577880859E+03},
 {   11373, .8287650390625E+04, -.1087472045898E+04, .9010929565430E+03},
 {   11374, .8389770507812E+04, -.1117541015625E+04, .1144353515625E+04},
 {   11375, .8367351562500E+04, -.1115411987305E+04, .1159951416016E+04},
 {   11376, .8385882812500E+04, -.1093583862305E+04, .1139480834961E+04},
 {   11377, .8364435546875E+04, -.1091147583008E+04, .1153436645508E+04},
 {   11378, .8376462890625E+04, -.1073224121094E+04, .1129073608398E+04},
 {   11379, .8358233398438E+04, -.1070915771484E+04, .1139582275391E+04},
 {   11380, .8367034179688E+04, -.1116215942383E+04, .1118799926758E+04},
 {   11381, .8351951171875E+04, -.1115411987305E+04, .1125551025391E+04},
 {   11382, .8367027343750E+04, -.1101843994141E+04, .1118802490234E+04},
 {   11383, .8351951171875E+04, -.1101442016602E+04, .1125551025391E+04},
 {   11384, .8367021484375E+04, -.1087472045898E+04, .1118804565430E+04},
 {   11385, .8351951171875E+04, -.1087472045898E+04, .1125551025391E+04},
 {   11386, .8360287109375E+04, -.1116215942383E+04, .1103730834961E+04},
 {   11387, .8345205078125E+04, -.1115411987305E+04, .1110482055664E+04},
 {   11388, .8360282226562E+04, -.1101843994141E+04, .1103733520508E+04},
 {   11389, .8345205078125E+04, -.1101442016602E+04, .1110482055664E+04},
 {   11390, .8360277343750E+04, -.1087472045898E+04, .1103736083984E+04},
 {   11391, .8345205078125E+04, -.1087472045898E+04, .1110482055664E+04},
 {   11392, .8353541992188E+04, -.1116215942383E+04, .1088661865234E+04},
 {   11393, .8338461914062E+04, -.1115411987305E+04, .1095413208008E+04},
 {   11394, .8353535156250E+04, -.1101843994141E+04, .1088664550781E+04},
 {   11395, .8338461914062E+04, -.1101442016602E+04, .1095413208008E+04},
 {   11396, .8353531250000E+04, -.1087472045898E+04, .1088666992188E+04},
 {   11397, .8338461914062E+04, -.1087472045898E+04, .1095413208008E+04},
 {   11398, .8337975585938E+04, -.1116215942383E+04, .1053887207031E+04},
 {   11399, .8322892578125E+04, -.1115411987305E+04, .1060638183594E+04},
 {   11400, .8337969726562E+04, -.1101843994141E+04, .1053889648438E+04},
 {   11401, .8322892578125E+04, -.1101442016602E+04, .1060638183594E+04},
 {   11402, .8337963867188E+04, -.1087472045898E+04, .1053892333984E+04},
 {   11403, .8322892578125E+04, -.1087472045898E+04, .1060638183594E+04},
 {   11404, .8322407226562E+04, -.1116215942383E+04, .1019113037109E+04},
 {   11405, .8307327148438E+04, -.1115411987305E+04, .1025863769531E+04},
 {   11406, .8322401367188E+04, -.1101843994141E+04, .1019115295410E+04},
 {   11407, .8307327148438E+04, -.1101442016602E+04, .1025863769531E+04},
 {   11408, .8322395507812E+04, -.1087472045898E+04, .1019117492676E+04},
 {   11409, .8307327148438E+04, -.1087472045898E+04, .1025863769531E+04},
 {   11410, .8304245117188E+04, -.1116216064453E+04, .9785419921875E+03},
 {   11411, .8289163085938E+04, -.1115412109375E+04, .9852933959961E+03},
 {   11412, .8304239257812E+04, -.1101843994141E+04, .9785447998047E+03},
 {   11413, .8289163085938E+04, -.1101442016602E+04, .9852933959961E+03},
 {   11414, .8304234375000E+04, -.1087472045898E+04, .9785473632812E+03},
 {   11415, .8289163085938E+04, -.1087472045898E+04, .9852933959961E+03},
 {   11416, .8286083007812E+04, -.1116216064453E+04, .9379721679688E+03},
 {   11417, .8271002929688E+04, -.1115412109375E+04, .9447230224609E+03},
 {   11418, .8286079101562E+04, -.1101843994141E+04, .9379744873047E+03},
 {   11419, .8271002929688E+04, -.1101442016602E+04, .9447230224609E+03},
 {   11420, .8286072265625E+04, -.1087472045898E+04, .9379769287109E+03},
 {   11421, .8271002929688E+04, -.1087472045898E+04, .9447230224609E+03},
 {   11422, .8279337890625E+04, -.1116216064453E+04, .9229030151367E+03},
 {   11423, .8264257812500E+04, -.1115412109375E+04, .9296538085938E+03},
 {   11424, .8279333007812E+04, -.1101843994141E+04, .9229054565430E+03},
 {   11425, .8264257812500E+04, -.1101442016602E+04, .9296538085938E+03},
 {   11426, .8279328125000E+04, -.1087472045898E+04, .9229077148438E+03},
 {   11427, .8264257812500E+04, -.1087472045898E+04, .9296538085938E+03},
 {   11428, .8272591796875E+04, -.1116216064453E+04, .9078341674805E+03},
 {   11429, .8257511718750E+04, -.1115412109375E+04, .9145847167969E+03},
 {   11430, .8272585937500E+04, -.1101843994141E+04, .9078365478516E+03},
 {   11431, .8257511718750E+04, -.1101442016602E+04, .9145847167969E+03},
 {   11432, .8272580078125E+04, -.1087472045898E+04, .9078391113281E+03},
 {   11433, .8257511718750E+04, -.1087472045898E+04, .9145847167969E+03},
 {   11434, .8268678710938E+04, -.1117541137695E+04, .8738530883789E+03},
 {   11435, .8242110351562E+04, -.1115412109375E+04, .8801839599609E+03},
 {   11436, .8269721679688E+04, -.1093583862305E+04, .8799988403320E+03},
 {   11437, .8245027343750E+04, -.1091147583008E+04, .8866992187500E+03},
 {   11438, .8271210937500E+04, -.1073224121094E+04, .8939567871094E+03},
 {   11439, .8251229492188E+04, -.1070915771484E+04, .9005531616211E+03},
 {   11440, .8371106445312E+04, -.1061086914062E+04, .1107868041992E+04},
 {   11441, .8349082031250E+04, -.1056833251953E+04, .1119139648438E+04},
 {   11442, .8362187500000E+04, -.1056833251953E+04, .1084791381836E+04},
 {   11443, .8338461914062E+04, -.1051912109375E+04, .1095413208008E+04},
 {   11444, .8346622070312E+04, -.1056833251953E+04, .1050016723633E+04},
 {   11445, .8322892578125E+04, -.1051912109375E+04, .1060638183594E+04},
 {   11446, .8331053710938E+04, -.1056833251953E+04, .1015241943359E+04},
 {   11447, .8307327148438E+04, -.1051912109375E+04, .1025863769531E+04},
 {   11448, .8312891601562E+04, -.1056833251953E+04, .9746717529297E+03},
 {   11449, .8289163085938E+04, -.1051912109375E+04, .9852933959961E+03},
 {   11450, .8294728515625E+04, -.1056833251953E+04, .9341013183594E+03},
 {   11451, .8271002929688E+04, -.1051912109375E+04, .9447230224609E+03},
 {   11452, .8283458007812E+04, -.1061086914062E+04, .9120776367188E+03},
 {   11453, .8260381835938E+04, -.1056833251953E+04, .9209960937500E+03},
 {   11454, .8340785156250E+04, -.1117541015625E+04, .1166282348633E+04},
 {   11455, .8314194335938E+04, -.1122758666992E+04, .1162264282227E+04},
 {   11456, .8339740234375E+04, -.1093583862305E+04, .1160136840820E+04},
 {   11457, .8315027343750E+04, -.1100165039062E+04, .1158548095703E+04},
 {   11458, .8338251953125E+04, -.1073224121094E+04, .1146178710938E+04},
 {   11459, .8318291015625E+04, -.1079944824219E+04, .1148277832031E+04},
 {   11460, .8336871093750E+04, -.1116215942383E+04, .1132301513672E+04},
 {   11461, .8321934570312E+04, -.1118620239258E+04, .1138989013672E+04},
 {   11462, .8336878906250E+04, -.1101843994141E+04, .1132299072266E+04},
 {   11463, .8321874023438E+04, -.1103046142578E+04, .1139015258789E+04},
 {   11464, .8336884765625E+04, -.1087472045898E+04, .1132296386719E+04},
 {   11465, .8321814453125E+04, -.1087472045898E+04, .1139042968750E+04},
 {   11466, .8330126953125E+04, -.1116215942383E+04, .1117232666016E+04},
 {   11467, .8315188476562E+04, -.1118620239258E+04, .1123919799805E+04},
 {   11468, .8330132812500E+04, -.1101843994141E+04, .1117230468750E+04},
 {   11469, .8315127929688E+04, -.1103046142578E+04, .1123946655273E+04},
 {   11470, .8330138671875E+04, -.1087472045898E+04, .1117228027344E+04},
 {   11471, .8315067382812E+04, -.1087472045898E+04, .1123973388672E+04},
 {   11472, .8323380859375E+04, -.1116215942383E+04, .1102163574219E+04},
 {   11473, .8308441406250E+04, -.1118620239258E+04, .1108850830078E+04},
 {   11474, .8323384765625E+04, -.1101843994141E+04, .1102160888672E+04},
 {   11475, .8308382812500E+04, -.1103046142578E+04, .1108877441406E+04},
 {   11476, .8323390625000E+04, -.1087472045898E+04, .1102158447266E+04},
 {   11477, .8308323242188E+04, -.1087472045898E+04, .1108904541016E+04},
 {   11478, .8307812500000E+04, -.1116215942383E+04, .1067389404297E+04},
 {   11479, .8292874023438E+04, -.1118620239258E+04, .1074075927734E+04},
 {   11480, .8307818359375E+04, -.1101843994141E+04, .1067386962891E+04},
 {   11481, .8292816406250E+04, -.1103046142578E+04, .1074102783203E+04},
 {   11482, .8307824218750E+04, -.1087472045898E+04, .1067384277344E+04},
 {   11483, .8292755859375E+04, -.1087472045898E+04, .1074129516602E+04},
 {   11484, .8292245117188E+04, -.1116215942383E+04, .1032614257812E+04},
 {   11485, .8277308593750E+04, -.1118620239258E+04, .1039301269531E+04},
 {   11486, .8292250000000E+04, -.1101843994141E+04, .1032612060547E+04},
 {   11487, .8277249023438E+04, -.1103046142578E+04, .1039328247070E+04},
 {   11488, .8292255859375E+04, -.1087472045898E+04, .1032609497070E+04},
 {   11489, .8277189453125E+04, -.1087472045898E+04, .1039355346680E+04},
 {   11490, .8274083984375E+04, -.1116216064453E+04, .9920440673828E+03},
 {   11491, .8259146484375E+04, -.1118620239258E+04, .9987310791016E+03},
 {   11492, .8274088867188E+04, -.1101843994141E+04, .9920412597656E+03},
 {   11493, .8259085937500E+04, -.1103046142578E+04, .9987578125000E+03},
 {   11494, .8274094726562E+04, -.1087472045898E+04, .9920388793945E+03},
 {   11495, .8259026367188E+04, -.1087472045898E+04, .9987851562500E+03},
 {   11496, .8255923828125E+04, -.1116216064453E+04, .9514736328125E+03},
 {   11497, .8240986328125E+04, -.1118620239258E+04, .9581606445312E+03},
 {   11498, .8255927734375E+04, -.1101843994141E+04, .9514712524414E+03},
 {   11499, .8240923828125E+04, -.1103046142578E+04, .9581879882812E+03},
 {   11500, .8255934570312E+04, -.1087472045898E+04, .9514685668945E+03},
 {   11501, .8240865234375E+04, -.1087472045898E+04, .9582145385742E+03},
 {   11502, .8249175781250E+04, -.1116216064453E+04, .9364047851562E+03},
 {   11503, .8234238281250E+04, -.1118620239258E+04, .9430916748047E+03},
 {   11504, .8249181640625E+04, -.1101843994141E+04, .9364023437500E+03},
 {   11505, .8234180664062E+04, -.1103046142578E+04, .9431182861328E+03},
 {   11506, .8249187500000E+04, -.1087472045898E+04, .9363999633789E+03},
 {   11507, .8234119140625E+04, -.1087472045898E+04, .9431454467773E+03},
 {   11508, .8242429687500E+04, -.1116216064453E+04, .9213358764648E+03},
 {   11509, .8227490234375E+04, -.1118620239258E+04, .9280230712891E+03},
 {   11510, .8242436523438E+04, -.1101843994141E+04, .9213330688477E+03},
 {   11511, .8227434570312E+04, -.1103046142578E+04, .9280494384766E+03},
 {   11512, .8242442382812E+04, -.1087472045898E+04, .9213309326172E+03},
 {   11513, .8227373046875E+04, -.1087472045898E+04, .9280765380859E+03},
 {   11514, .8219694335938E+04, -.1117541137695E+04, .8957818603516E+03},
 {   11515, .8204978515625E+04, -.1122758789062E+04, .9182906494141E+03},
 {   11516, .8223579101562E+04, -.1093583862305E+04, .9006544799805E+03},
 {   11517, .8208303710938E+04, -.1100165039062E+04, .9201462402344E+03},
 {   11518, .8233000000000E+04, -.1073224121094E+04, .9110623779297E+03},
 {   11519, .8218137695312E+04, -.1079944824219E+04, .9245528564453E+03},
 {   11520, .8326004882812E+04, -.1061086914062E+04, .1128057983398E+04},
 {   11521, .8304801757812E+04, -.1073224121094E+04, .1133414794922E+04},
 {   11522, .8314734375000E+04, -.1056833251953E+04, .1106033813477E+04},
 {   11523, .8294291015625E+04, -.1070915771484E+04, .1115186157227E+04},
 {   11524, .8299167968750E+04, -.1056833251953E+04, .1071259399414E+04},
 {   11525, .8278723632812E+04, -.1070915771484E+04, .1080411254883E+04},
 {   11526, .8283598632812E+04, -.1056833251953E+04, .1036485351562E+04},
 {   11527, .8263155273438E+04, -.1070915771484E+04, .1045636718750E+04},
 {   11528, .8265437500000E+04, -.1056833251953E+04, .9959149169922E+03},
 {   11529, .8244993164062E+04, -.1070915771484E+04, .1005066650391E+04},
 {   11530, .8247276367188E+04, -.1056833251953E+04, .9553442382812E+03},
 {   11531, .8226833984375E+04, -.1070915771484E+04, .9644958496094E+03},
 {   11532, .8238358398438E+04, -.1061086914062E+04, .9322674560547E+03},
 {   11533, .8220236328125E+04, -.1073224121094E+04, .9445153198242E+03},
 {   11534, .8290374023438E+04, -.1128391967773E+04, .1147537841797E+04},
 {   11535, .8293030273438E+04, -.1098509643555E+04, .1144327880859E+04},
 {   11536, .8273934570312E+04, -.1130790893555E+04, .1124298461914E+04},
 {   11537, .8277630859375E+04, -.1098207885742E+04, .1122643920898E+04},
 {   11538, .8258369140625E+04, -.1130790893555E+04, .1089523437500E+04},
 {   11539, .8262064453125E+04, -.1098207885742E+04, .1087869140625E+04},
 {   11540, .8242800781250E+04, -.1130790893555E+04, .1054749267578E+04},
 {   11541, .8246495117188E+04, -.1098207885742E+04, .1053095214844E+04},
 {   11542, .8224636718750E+04, -.1130791015625E+04, .1014179199219E+04},
 {   11543, .8228334960938E+04, -.1098207885742E+04, .1012524230957E+04},
 {   11544, .8206476562500E+04, -.1130791015625E+04, .9736083984375E+03},
 {   11545, .8210171875000E+04, -.1098207885742E+04, .9719542846680E+03},
 {   11546, .8200092773438E+04, -.1128391967773E+04, .9458676147461E+03},
 {   11547, .8204257812500E+04, -.1098509643555E+04, .9460247802734E+03},
 {   11548, .8314194335938E+04, -.1448425292969E+04, .1162264282227E+04},
 {   11549, .8318018554688E+04, -.1450702758789E+04, .1150647338867E+04},
 {   11550, .8321934570312E+04, -.1452563842773E+04, .1138989013672E+04},
 {   11551, .8315027343750E+04, -.1471019042969E+04, .1158548095703E+04},
 {   11552, .8318426757812E+04, -.1469682495117E+04, .1148792114258E+04},
 {   11553, .8321874023438E+04, -.1468137939453E+04, .1139015258789E+04},
 {   11554, .8318291015625E+04, -.1491239257812E+04, .1148277832031E+04},
 {   11555, .8320052734375E+04, -.1487475585938E+04, .1143660156250E+04},
 {   11556, .8321814453125E+04, -.1483712036133E+04, .1139042968750E+04},
 {   11557, .8297649414062E+04, -.1444463623047E+04, .1153540161133E+04},
 {   11558, .8307375976562E+04, -.1449117065430E+04, .1141315673828E+04},
 {   11559, .8317438476562E+04, -.1452563842773E+04, .1128942749023E+04},
 {   11560, .8299859375000E+04, -.1472313598633E+04, .1150041503906E+04},
 {   11561, .8308534179688E+04, -.1470527343750E+04, .1139542602539E+04},
 {   11562, .8317375976562E+04, -.1468137939453E+04, .1128969482422E+04},
 {   11563, .8309019531250E+04, -.1496188232422E+04, .1138816162109E+04},
 {   11564, .8313169921875E+04, -.1489950073242E+04, .1133906005859E+04},
 {   11565, .8317319335938E+04, -.1483712036133E+04, .1128996337891E+04},
 {   11566, .8283918945312E+04, -.1441495239258E+04, .1140568237305E+04},
 {   11567, .8298151367188E+04, -.1447993286133E+04, .1129856567383E+04},
 {   11568, .8312939453125E+04, -.1452563842773E+04, .1118896362305E+04},
 {   11569, .8286939453125E+04, -.1472885253906E+04, .1137745117188E+04},
 {   11570, .8299772460938E+04, -.1470993408203E+04, .1128396118164E+04},
 {   11571, .8312881835938E+04, -.1468137939453E+04, .1118923217773E+04},
 {   11572, .8300913085938E+04, -.1499238037109E+04, .1127637939453E+04},
 {   11573, .8306866210938E+04, -.1491475097656E+04, .1123294189453E+04},
 {   11574, .8312819335938E+04, -.1483712036133E+04, .1118950683594E+04},
 {   11575, .8273934570312E+04, -.1440393066406E+04, .1124298461914E+04},
 {   11576, .8290863281250E+04, -.1447585205078E+04, .1116720336914E+04},
 {   11577, .8308441406250E+04, -.1452563842773E+04, .1108850830078E+04},
 {   11578, .8277630859375E+04, -.1472976196289E+04, .1122643920898E+04},
 {   11579, .8292843750000E+04, -.1471110351562E+04, .1115833251953E+04},
 {   11580, .8308382812500E+04, -.1468137939453E+04, .1108877441406E+04},
 {   11581, .8294291015625E+04, -.1500268310547E+04, .1115186157227E+04},
 {   11582, .8301306640625E+04, -.1491990112305E+04, .1112044921875E+04},
 {   11583, .8308323242188E+04, -.1483712036133E+04, .1108904541016E+04},
 {   11584, .8263559570312E+04, -.1440393066406E+04, .1101115112305E+04},
 {   11585, .8280483398438E+04, -.1447585205078E+04, .1093537231445E+04},
 {   11586, .8298065429688E+04, -.1452563842773E+04, .1085667236328E+04},
 {   11587, .8267250976562E+04, -.1472976196289E+04, .1099461059570E+04},
 {   11588, .8282466796875E+04, -.1471110351562E+04, .1092650634766E+04},
 {   11589, .8298005859375E+04, -.1468137939453E+04, .1085694213867E+04},
 {   11590, .8283912109375E+04, -.1500268310547E+04, .1092002929688E+04},
 {   11591, .8290930664062E+04, -.1491990112305E+04, .1088862304688E+04},
 {   11592, .8297944335938E+04, -.1483712036133E+04, .1085721435547E+04},
 {   11593, .8253178710938E+04, -.1440393066406E+04, .1077932373047E+04},
 {   11594, .8270107421875E+04, -.1447585205078E+04, .1070353515625E+04},
 {   11595, .8287686523438E+04, -.1452563842773E+04, .1062484252930E+04},
 {   11596, .8256874023438E+04, -.1472976196289E+04, .1076277832031E+04},
 {   11597, .8272086914062E+04, -.1471110351562E+04, .1069467773438E+04},
 {   11598, .8287625976562E+04, -.1468137939453E+04, .1062511474609E+04},
 {   11599, .8273532226562E+04, -.1500268310547E+04, .1068820068359E+04},
 {   11600, .8280550781250E+04, -.1491990112305E+04, .1065679687500E+04},
 {   11601, .8287566406250E+04, -.1483712036133E+04, .1062538330078E+04},
 {   11602, .8242800781250E+04, -.1440393066406E+04, .1054749267578E+04},
 {   11603, .8259726562500E+04, -.1447585205078E+04, .1047171630859E+04},
 {   11604, .8277308593750E+04, -.1452563842773E+04, .1039301269531E+04},
 {   11605, .8246495117188E+04, -.1472976196289E+04, .1053095214844E+04},
 {   11606, .8261708984375E+04, -.1471110351562E+04, .1046284545898E+04},
 {   11607, .8277249023438E+04, -.1468137939453E+04, .1039328247070E+04},
 {   11608, .8263155273438E+04, -.1500268310547E+04, .1045636962891E+04},
 {   11609, .8270171875000E+04, -.1491990112305E+04, .1042495605469E+04},
 {   11610, .8277189453125E+04, -.1483712036133E+04, .1039355346680E+04},
 {   11611, .8233721679688E+04, -.1440393066406E+04, .1034463623047E+04},
 {   11612, .8250650390625E+04, -.1447585205078E+04, .1026885864258E+04},
 {   11613, .8268227539062E+04, -.1452563842773E+04, .1019016235352E+04},
 {   11614, .8237414062500E+04, -.1472976196289E+04, .1032809936523E+04},
 {   11615, .8252629882812E+04, -.1471110351562E+04, .1025998657227E+04},
 {   11616, .8268168945312E+04, -.1468137939453E+04, .1019043273926E+04},
 {   11617, .8254078125000E+04, -.1500268310547E+04, .1025351196289E+04},
 {   11618, .8261091796875E+04, -.1491990112305E+04, .1022210815430E+04},
 {   11619, .8268106445312E+04, -.1483712036133E+04, .1019070129395E+04},
 {   11620, .8224636718750E+04, -.1440393066406E+04, .1014179199219E+04},
 {   11621, .8241568359375E+04, -.1447585205078E+04, .1006600219727E+04},
 {   11622, .8259146484375E+04, -.1452563842773E+04, .9987310791016E+03},
 {   11623, .8228334960938E+04, -.1472976196289E+04, .1012524230957E+04},
 {   11624, .8243547851562E+04, -.1471110351562E+04, .1005714111328E+04},
 {   11625, .8259085937500E+04, -.1468137939453E+04, .9987578125000E+03},
 {   11626, .8244993164062E+04, -.1500268310547E+04, .1005066650391E+04},
 {   11627, .8252010742188E+04, -.1491990112305E+04, .1001925781250E+04},
 {   11628, .8259026367188E+04, -.1483712036133E+04, .9987851562500E+03},
 {   11629, .8215559570312E+04, -.1440393066406E+04, .9938931884766E+03},
 {   11630, .8232486328125E+04, -.1447585205078E+04, .9863156127930E+03},
 {   11631, .8250065429688E+04, -.1452563842773E+04, .9784459228516E+03},
 {   11632, .8219256835938E+04, -.1472976196289E+04, .9922388916016E+03},
 {   11633, .8234468750000E+04, -.1471110351562E+04, .9854283447266E+03},
 {   11634, .8250006835938E+04, -.1468137939453E+04, .9784723510742E+03},
 {   11635, .8235911132812E+04, -.1500268310547E+04, .9847814941406E+03},
 {   11636, .8242930664062E+04, -.1491990112305E+04, .9816403808594E+03},
 {   11637, .8249945312500E+04, -.1483712036133E+04, .9784997558594E+03},
 {   11638, .8206476562500E+04, -.1440393066406E+04, .9736083984375E+03},
 {   11639, .8223406250000E+04, -.1447585205078E+04, .9660299072266E+03},
 {   11640, .8240986328125E+04, -.1452563842773E+04, .9581606445312E+03},
 {   11641, .8210171875000E+04, -.1472976196289E+04, .9719542846680E+03},
 {   11642, .8225387695312E+04, -.1471110351562E+04, .9651434326172E+03},
 {   11643, .8240923828125E+04, -.1468137939453E+04, .9581879882812E+03},
 {   11644, .8226833984375E+04, -.1500268310547E+04, .9644958496094E+03},
 {   11645, .8233849609375E+04, -.1491990112305E+04, .9613551025391E+03},
 {   11646, .8240865234375E+04, -.1483712036133E+04, .9582145385742E+03},
 {   11647, .8200994140625E+04, -.1441495239258E+04, .9553243408203E+03},
 {   11648, .8218462890625E+04, -.1447993286133E+04, .9518438720703E+03},
 {   11649, .8236486328125E+04, -.1452563842773E+04, .9481146240234E+03},
 {   11650, .8205111328125E+04, -.1472885253906E+04, .9549517211914E+03},
 {   11651, .8220630859375E+04, -.1470993408203E+04, .9516088256836E+03},
 {   11652, .8236429687500E+04, -.1468137939453E+04, .9481412963867E+03},
 {   11653, .8221958007812E+04, -.1499238037109E+04, .9512622070312E+03},
 {   11654, .8229163085938E+04, -.1491475097656E+04, .9497152099609E+03},
 {   11655, .8236365234375E+04, -.1483712036133E+04, .9481689453125E+03},
 {   11656, .8200462890625E+04, -.1444463623047E+04, .9364436645508E+03},
 {   11657, .8216060546875E+04, -.1449117065430E+04, .9373301391602E+03},
 {   11658, .8231991210938E+04, -.1452563842773E+04, .9380687255859E+03},
 {   11659, .8204544921875E+04, -.1472313476562E+04, .9371250000000E+03},
 {   11660, .8218157226562E+04, -.1470527343750E+04, .9376472167969E+03},
 {   11661, .8231931640625E+04, -.1468137939453E+04, .9380955810547E+03},
 {   11662, .8219018554688E+04, -.1496188232422E+04, .9377705078125E+03},
 {   11663, .8225442382812E+04, -.1489950195312E+04, .9379465332031E+03},
 {   11664, .8231868164062E+04, -.1483712036133E+04, .9381228637695E+03},
 {   11665, .8204978515625E+04, -.1448425292969E+04, .9182907714844E+03},
 {   11666, .8216186523438E+04, -.1450702758789E+04, .9231781005859E+03},
 {   11667, .8227490234375E+04, -.1452563842773E+04, .9280230712891E+03},
 {   11668, .8208303710938E+04, -.1471019042969E+04, .9201462402344E+03},
 {   11669, .8217843750000E+04, -.1469682495117E+04, .9241086425781E+03},
 {   11670, .8227434570312E+04, -.1468137939453E+04, .9280495605469E+03},
 {   11671, .8218137695312E+04, -.1491239257812E+04, .9245528564453E+03},
 {   11672, .8222755859375E+04, -.1487475585938E+04, .9263146362305E+03},
 {   11673, .8227373046875E+04, -.1483712036133E+04, .9280765380859E+03},
 {   11674, .8340785156250E+04, -.1453642944336E+04, .1166282348633E+04},
 {   11675, .8367351562500E+04, -.1455771972656E+04, .1159951416016E+04},
 {   11676, .8339740234375E+04, -.1477600219727E+04, .1160136840820E+04},
 {   11677, .8364435546875E+04, -.1480036376953E+04, .1153436645508E+04},
 {   11678, .8338251953125E+04, -.1497959838867E+04, .1146178710938E+04},
 {   11679, .8358233398438E+04, -.1500268310547E+04, .1139582275391E+04},
 {   11680, .8338808593750E+04, -.1454382690430E+04, .1149301025391E+04},
 {   11681, .8359652343750E+04, -.1455771972656E+04, .1142751464844E+04},
 {   11682, .8338297851562E+04, -.1473508666992E+04, .1146222900391E+04},
 {   11683, .8358195312500E+04, -.1474889160156E+04, .1139493286133E+04},
 {   11684, .8337568359375E+04, -.1490835937500E+04, .1139237792969E+04},
 {   11685, .8355091796875E+04, -.1491990112305E+04, .1132566406250E+04},
 {   11686, .8336871093750E+04, -.1454968139648E+04, .1132301513672E+04},
 {   11687, .8351951171875E+04, -.1455771972656E+04, .1125551025391E+04},
 {   11688, .8336878906250E+04, -.1469340087891E+04, .1132299072266E+04},
 {   11689, .8351951171875E+04, -.1469741943359E+04, .1125551025391E+04},
 {   11690, .8336884765625E+04, -.1483712036133E+04, .1132296386719E+04},
 {   11691, .8351951171875E+04, -.1483712036133E+04, .1125551025391E+04},
 {   11692, .8332375976562E+04, -.1454968139648E+04, .1122255615234E+04},
 {   11693, .8347455078125E+04, -.1455771972656E+04, .1115504882812E+04},
 {   11694, .8332379882812E+04, -.1469340087891E+04, .1122252563477E+04},
 {   11695, .8347455078125E+04, -.1469741943359E+04, .1115504882812E+04},
 {   11696, .8332385742188E+04, -.1483712036133E+04, .1122250610352E+04},
 {   11697, .8347455078125E+04, -.1483712036133E+04, .1115504882812E+04},
 {   11698, .8327876953125E+04, -.1454968139648E+04, .1112209594727E+04},
 {   11699, .8342958984375E+04, -.1455771972656E+04, .1105459350586E+04},
 {   11700, .8327882812500E+04, -.1469340087891E+04, .1112207031250E+04},
 {   11701, .8342958984375E+04, -.1469741943359E+04, .1105459350586E+04},
 {   11702, .8327888671875E+04, -.1483712036133E+04, .1112204833984E+04},
 {   11703, .8342958984375E+04, -.1483712036133E+04, .1105459350586E+04},
 {   11704, .8323380859375E+04, -.1454968139648E+04, .1102163574219E+04},
 {   11705, .8338461914062E+04, -.1455771972656E+04, .1095413452148E+04},
 {   11706, .8323384765625E+04, -.1469340087891E+04, .1102160888672E+04},
 {   11707, .8338461914062E+04, -.1469741943359E+04, .1095413452148E+04},
 {   11708, .8323389648438E+04, -.1483712036133E+04, .1102158691406E+04},
 {   11709, .8338461914062E+04, -.1483712036133E+04, .1095413452148E+04},
 {   11710, .8313002929688E+04, -.1454968139648E+04, .1078980224609E+04},
 {   11711, .8328083007812E+04, -.1455771972656E+04, .1072229858398E+04},
 {   11712, .8313008789062E+04, -.1469340087891E+04, .1078977783203E+04},
 {   11713, .8328083007812E+04, -.1469741943359E+04, .1072229858398E+04},
 {   11714, .8313014648438E+04, -.1483712036133E+04, .1078975585938E+04},
 {   11715, .8328083007812E+04, -.1483712036133E+04, .1072229858398E+04},
 {   11716, .8302623046875E+04, -.1454968139648E+04, .1055797851562E+04},
 {   11717, .8317705078125E+04, -.1455771972656E+04, .1049046875000E+04},
 {   11718, .8302628906250E+04, -.1469340087891E+04, .1055795410156E+04},
 {   11719, .8317705078125E+04, -.1469741943359E+04, .1049046875000E+04},
 {   11720, .8302634765625E+04, -.1483712036133E+04, .1055793090820E+04},
 {   11721, .8317705078125E+04, -.1483712036133E+04, .1049046875000E+04},
 {   11722, .8292245117188E+04, -.1454968139648E+04, .1032614501953E+04},
 {   11723, .8307327148438E+04, -.1455771972656E+04, .1025863769531E+04},
 {   11724, .8292250000000E+04, -.1469340087891E+04, .1032612060547E+04},
 {   11725, .8307327148438E+04, -.1469741943359E+04, .1025863769531E+04},
 {   11726, .8292255859375E+04, -.1483712036133E+04, .1032609497070E+04},
 {   11727, .8307327148438E+04, -.1483712036133E+04, .1025863769531E+04},
 {   11728, .8283164062500E+04, -.1454968139648E+04, .1012329101562E+04},
 {   11729, .8298244140625E+04, -.1455772094727E+04, .1005578613281E+04},
 {   11730, .8283169921875E+04, -.1469340087891E+04, .1012326660156E+04},
 {   11731, .8298244140625E+04, -.1469742065430E+04, .1005578613281E+04},
 {   11732, .8283175781250E+04, -.1483712036133E+04, .1012324035645E+04},
 {   11733, .8298244140625E+04, -.1483712036133E+04, .1005578613281E+04},
 {   11734, .8274083984375E+04, -.1454968139648E+04, .9920440673828E+03},
 {   11735, .8289163085938E+04, -.1455772094727E+04, .9852933959961E+03},
 {   11736, .8274088867188E+04, -.1469340087891E+04, .9920415039062E+03},
 {   11737, .8289163085938E+04, -.1469742065430E+04, .9852933959961E+03},
 {   11738, .8274094726562E+04, -.1483712036133E+04, .9920388793945E+03},
 {   11739, .8289163085938E+04, -.1483712036133E+04, .9852933959961E+03},
 {   11740, .8265004882812E+04, -.1454968139648E+04, .9717589111328E+03},
 {   11741, .8280083984375E+04, -.1455772094727E+04, .9650081787109E+03},
 {   11742, .8265009765625E+04, -.1469340087891E+04, .9717561645508E+03},
 {   11743, .8280083984375E+04, -.1469742065430E+04, .9650081787109E+03},
 {   11744, .8265014648438E+04, -.1483712036133E+04, .9717538452148E+03},
 {   11745, .8280083984375E+04, -.1483712036133E+04, .9650081787109E+03},
 {   11746, .8255923828125E+04, -.1454968139648E+04, .9514736328125E+03},
 {   11747, .8271002929688E+04, -.1455772094727E+04, .9447231445312E+03},
 {   11748, .8255927734375E+04, -.1469340087891E+04, .9514712524414E+03},
 {   11749, .8271002929688E+04, -.1469742065430E+04, .9447231445312E+03},
 {   11750, .8255934570312E+04, -.1483712036133E+04, .9514685668945E+03},
 {   11751, .8271002929688E+04, -.1483712036133E+04, .9447231445312E+03},
 {   11752, .8251425781250E+04, -.1454968139648E+04, .9414277954102E+03},
 {   11753, .8266504882812E+04, -.1455772094727E+04, .9346771240234E+03},
 {   11754, .8251431640625E+04, -.1469340087891E+04, .9414252319336E+03},
 {   11755, .8266504882812E+04, -.1469742065430E+04, .9346771240234E+03},
 {   11756, .8251438476562E+04, -.1483712036133E+04, .9414225463867E+03},
 {   11757, .8266504882812E+04, -.1483712036133E+04, .9346771240234E+03},
 {   11758, .8246927734375E+04, -.1454968139648E+04, .9313815917969E+03},
 {   11759, .8262008789062E+04, -.1455772094727E+04, .9246307983398E+03},
 {   11760, .8246934570312E+04, -.1469340087891E+04, .9313790893555E+03},
 {   11761, .8262008789062E+04, -.1469742065430E+04, .9246307983398E+03},
 {   11762, .8246939453125E+04, -.1483712036133E+04, .9313767700195E+03},
 {   11763, .8262008789062E+04, -.1483712036133E+04, .9246307983398E+03},
 {   11764, .8242431640625E+04, -.1454968139648E+04, .9213356933594E+03},
 {   11765, .8257511718750E+04, -.1455772094727E+04, .9145848388672E+03},
 {   11766, .8242436523438E+04, -.1469340087891E+04, .9213330688477E+03},
 {   11767, .8257511718750E+04, -.1469742065430E+04, .9145848388672E+03},
 {   11768, .8242442382812E+04, -.1483712036133E+04, .9213309326172E+03},
 {   11769, .8257511718750E+04, -.1483712036133E+04, .9145848388672E+03},
 {   11770, .8231041992188E+04, -.1454382690430E+04, .9085678710938E+03},
 {   11771, .8249811523438E+04, -.1455772094727E+04, .8973844604492E+03},
 {   11772, .8232998046875E+04, -.1473508666992E+04, .9109987792969E+03},
 {   11773, .8251267578125E+04, -.1474889282227E+04, .9006420898438E+03},
 {   11774, .8237721679688E+04, -.1490835937500E+04, .9161964111328E+03},
 {   11775, .8254369140625E+04, -.1491990112305E+04, .9075689697266E+03},
 {   11776, .8219694335938E+04, -.1453643066406E+04, .8957819824219E+03},
 {   11777, .8242110351562E+04, -.1455772094727E+04, .8801839599609E+03},
 {   11778, .8223579101562E+04, -.1477600219727E+04, .9006544799805E+03},
 {   11779, .8245027343750E+04, -.1480036499023E+04, .8866992187500E+03},
 {   11780, .8233000000000E+04, -.1497959960938E+04, .9110623779297E+03},
 {   11781, .8251229492188E+04, -.1500268310547E+04, .9005531616211E+03},
 {   11782, .8331228515625E+04, -.1495278808594E+04, .1128470703125E+04},
 {   11783, .8349909179688E+04, -.1497157226562E+04, .1120988281250E+04},
 {   11784, .8330075195312E+04, -.1506845703125E+04, .1134690795898E+04},
 {   11785, .8352365234375E+04, -.1510602539062E+04, .1126471679688E+04},
 {   11786, .8324966796875E+04, -.1498083496094E+04, .1116592895508E+04},
 {   11787, .8344295898438E+04, -.1500391235352E+04, .1108448486328E+04},
 {   11788, .8322044921875E+04, -.1512454956055E+04, .1120981079102E+04},
 {   11789, .8345634765625E+04, -.1517070556641E+04, .1111438720703E+04},
 {   11790, .8319063476562E+04, -.1499031372070E+04, .1104096923828E+04},
 {   11791, .8338461914062E+04, -.1501491943359E+04, .1095413452148E+04},
 {   11792, .8314734375000E+04, -.1514350830078E+04, .1106033813477E+04},
 {   11793, .8338461914062E+04, -.1519271972656E+04, .1095413452148E+04},
 {   11794, .8308684570312E+04, -.1499031372070E+04, .1080913574219E+04},
 {   11795, .8328083007812E+04, -.1501491943359E+04, .1072229858398E+04},
 {   11796, .8304355468750E+04, -.1514350830078E+04, .1082851196289E+04},
 {   11797, .8328083007812E+04, -.1519271972656E+04, .1072229858398E+04},
 {   11798, .8298308593750E+04, -.1499031372070E+04, .1057729980469E+04},
 {   11799, .8317705078125E+04, -.1501491943359E+04, .1049046875000E+04},
 {   11800, .8293977539062E+04, -.1514350830078E+04, .1059668212891E+04},
 {   11801, .8317705078125E+04, -.1519271972656E+04, .1049046875000E+04},
 {   11802, .8287928710938E+04, -.1499031372070E+04, .1034547607422E+04},
 {   11803, .8307327148438E+04, -.1501491943359E+04, .1025863769531E+04},
 {   11804, .8283598632812E+04, -.1514350830078E+04, .1036485351562E+04},
 {   11805, .8307327148438E+04, -.1519271972656E+04, .1025863769531E+04},
 {   11806, .8278846679688E+04, -.1499031372070E+04, .1014261901855E+04},
 {   11807, .8298244140625E+04, -.1501491943359E+04, .1005578613281E+04},
 {   11808, .8274518554688E+04, -.1514350830078E+04, .1016200317383E+04},
 {   11809, .8298244140625E+04, -.1519271972656E+04, .1005578613281E+04},
 {   11810, .8269765625000E+04, -.1499031372070E+04, .9939766845703E+03},
 {   11811, .8289163085938E+04, -.1501491943359E+04, .9852933959961E+03},
 {   11812, .8265437500000E+04, -.1514350830078E+04, .9959149169922E+03},
 {   11813, .8289163085938E+04, -.1519271972656E+04, .9852933959961E+03},
 {   11814, .8260685546875E+04, -.1499031372070E+04, .9736916503906E+03},
 {   11815, .8280083984375E+04, -.1501491943359E+04, .9650081787109E+03},
 {   11816, .8256356445312E+04, -.1514350830078E+04, .9756295166016E+03},
 {   11817, .8280083984375E+04, -.1519271972656E+04, .9650081787109E+03},
 {   11818, .8251605468750E+04, -.1499031372070E+04, .9534065551758E+03},
 {   11819, .8271002929688E+04, -.1501491943359E+04, .9447231445312E+03},
 {   11820, .8247276367188E+04, -.1514350830078E+04, .9553442382812E+03},
 {   11821, .8271002929688E+04, -.1519271972656E+04, .9447231445312E+03},
 {   11822, .8246217773438E+04, -.1498083496094E+04, .9406787109375E+03},
 {   11823, .8265166992188E+04, -.1500391357422E+04, .9316870117188E+03},
 {   11824, .8240998046875E+04, -.1512455078125E+04, .9399346923828E+03},
 {   11825, .8263829101562E+04, -.1517070556641E+04, .9286972656250E+03},
 {   11826, .8241529296875E+04, -.1495278930664E+04, .9280955810547E+03},
 {   11827, .8259552734375E+04, -.1497157226562E+04, .9191470947266E+03},
 {   11828, .8236119140625E+04, -.1506845703125E+04, .9248145751953E+03},
 {   11829, .8257099609375E+04, -.1510602539062E+04, .9136635742188E+03},
 {   11830, .8389770507812E+04, -.1453642944336E+04, .1144353759766E+04},
 {   11831, .8404485351562E+04, -.1448425292969E+04, .1121844970703E+04},
 {   11832, .8385882812500E+04, -.1477600219727E+04, .1139481079102E+04},
 {   11833, .8401159179688E+04, -.1471019042969E+04, .1119989013672E+04},
 {   11834, .8376462890625E+04, -.1497959838867E+04, .1129073608398E+04},
 {   11835, .8391325195312E+04, -.1491239257812E+04, .1115582763672E+04},
 {   11836, .8378419921875E+04, -.1454382690430E+04, .1131567626953E+04},
 {   11837, .8393274414062E+04, -.1450702758789E+04, .1116958007812E+04},
 {   11838, .8376465820312E+04, -.1473508666992E+04, .1129136962891E+04},
 {   11839, .8391618164062E+04, -.1469682495117E+04, .1116026977539E+04},
 {   11840, .8371743164062E+04, -.1490835937500E+04, .1123939208984E+04},
 {   11841, .8386708984375E+04, -.1487475585938E+04, .1113820800781E+04},
 {   11842, .8367034179688E+04, -.1454968139648E+04, .1118799926758E+04},
 {   11843, .8381972656250E+04, -.1452563842773E+04, .1112112426758E+04},
 {   11844, .8367027343750E+04, -.1469340087891E+04, .1118802490234E+04},
 {   11845, .8382029296875E+04, -.1468137939453E+04, .1112086303711E+04},
 {   11846, .8367021484375E+04, -.1483712036133E+04, .1118804809570E+04},
 {   11847, .8382090820312E+04, -.1483712036133E+04, .1112058959961E+04},
 {   11848, .8362536132812E+04, -.1454968139648E+04, .1108753906250E+04},
 {   11849, .8377472656250E+04, -.1452563842773E+04, .1102066894531E+04},
 {   11850, .8362530273438E+04, -.1469340087891E+04, .1108756347656E+04},
 {   11851, .8377533203125E+04, -.1468137939453E+04, .1102039550781E+04},
 {   11852, .8362524414062E+04, -.1483712036133E+04, .1108758789062E+04},
 {   11853, .8377593750000E+04, -.1483712036133E+04, .1102013793945E+04},
 {   11854, .8358038085938E+04, -.1454968139648E+04, .1098708251953E+04},
 {   11855, .8372975585938E+04, -.1452563842773E+04, .1092020996094E+04},
 {   11856, .8358032226562E+04, -.1469340087891E+04, .1098710571289E+04},
 {   11857, .8373036132812E+04, -.1468137939453E+04, .1091994140625E+04},
 {   11858, .8358026367188E+04, -.1483712036133E+04, .1098713134766E+04},
 {   11859, .8373095703125E+04, -.1483712036133E+04, .1091967773438E+04},
 {   11860, .8353541992188E+04, -.1454968139648E+04, .1088661865234E+04},
 {   11861, .8368478515625E+04, -.1452563842773E+04, .1081974731445E+04},
 {   11862, .8353535156250E+04, -.1469340087891E+04, .1088664550781E+04},
 {   11863, .8368539062500E+04, -.1468137939453E+04, .1081948242188E+04},
 {   11864, .8353531250000E+04, -.1483712036133E+04, .1088666992188E+04},
 {   11865, .8368596679688E+04, -.1483712036133E+04, .1081921508789E+04},
 {   11866, .8343161132812E+04, -.1454968139648E+04, .1065479248047E+04},
 {   11867, .8358099609375E+04, -.1452563842773E+04, .1058792114258E+04},
 {   11868, .8343156250000E+04, -.1469340087891E+04, .1065481323242E+04},
 {   11869, .8358159179688E+04, -.1468137939453E+04, .1058765258789E+04},
 {   11870, .8343151367188E+04, -.1483712036133E+04, .1065483764648E+04},
 {   11871, .8358220703125E+04, -.1483712036133E+04, .1058738159180E+04},
 {   11872, .8332784179688E+04, -.1454968139648E+04, .1042296020508E+04},
 {   11873, .8347723632812E+04, -.1452563842773E+04, .1035608886719E+04},
 {   11874, .8332781250000E+04, -.1469340087891E+04, .1042298217773E+04},
 {   11875, .8347781250000E+04, -.1468137939453E+04, .1035582031250E+04},
 {   11876, .8332774414062E+04, -.1483712036133E+04, .1042301147461E+04},
 {   11877, .8347843750000E+04, -.1483712036133E+04, .1035555297852E+04},
 {   11878, .8322407226562E+04, -.1454968139648E+04, .1019113281250E+04},
 {   11879, .8337343750000E+04, -.1452563842773E+04, .1012425842285E+04},
 {   11880, .8322401367188E+04, -.1469340087891E+04, .1019115295410E+04},
 {   11881, .8337405273438E+04, -.1468137939453E+04, .1012398986816E+04},
 {   11882, .8322395507812E+04, -.1483712036133E+04, .1019117492676E+04},
 {   11883, .8337463867188E+04, -.1483712036133E+04, .1012372009277E+04},
 {   11884, .8313324218750E+04, -.1454968139648E+04, .9988278198242E+03},
 {   11885, .8328265625000E+04, -.1452563842773E+04, .9921403198242E+03},
 {   11886, .8313319335938E+04, -.1469340087891E+04, .9988303222656E+03},
 {   11887, .8328323242188E+04, -.1468137939453E+04, .9921138305664E+03},
 {   11888, .8313313476562E+04, -.1483712036133E+04, .9988328857422E+03},
 {   11889, .8328381835938E+04, -.1483712036133E+04, .9920869140625E+03},
 {   11890, .8304245117188E+04, -.1454968139648E+04, .9785421142578E+03},
 {   11891, .8319183593750E+04, -.1452563842773E+04, .9718551025391E+03},
 {   11892, .8304239257812E+04, -.1469340087891E+04, .9785447998047E+03},
 {   11893, .8319242187500E+04, -.1468137939453E+04, .9718286132812E+03},
 {   11894, .8304234375000E+04, -.1483712036133E+04, .9785473632812E+03},
 {   11895, .8319302734375E+04, -.1483712036133E+04, .9718016357422E+03},
 {   11896, .8295164062500E+04, -.1454968139648E+04, .9582573242188E+03},
 {   11897, .8310101562500E+04, -.1452563842773E+04, .9515703125000E+03},
 {   11898, .8295158203125E+04, -.1469340087891E+04, .9582596435547E+03},
 {   11899, .8310160156250E+04, -.1468137939453E+04, .9515432128906E+03},
 {   11900, .8295152343750E+04, -.1483712036133E+04, .9582623901367E+03},
 {   11901, .8310221679688E+04, -.1483712036133E+04, .9515164184570E+03},
 {   11902, .8286083007812E+04, -.1454968139648E+04, .9379721679688E+03},
 {   11903, .8301020507812E+04, -.1452563842773E+04, .9312851562500E+03},
 {   11904, .8286080078125E+04, -.1469340087891E+04, .9379744262695E+03},
 {   11905, .8301080078125E+04, -.1468137939453E+04, .9312581787109E+03},
 {   11906, .8286072265625E+04, -.1483712036133E+04, .9379769287109E+03},
 {   11907, .8301140625000E+04, -.1483712036133E+04, .9312313232422E+03},
 {   11908, .8281585937500E+04, -.1454968139648E+04, .9279259643555E+03},
 {   11909, .8296522460938E+04, -.1452563842773E+04, .9212391967773E+03},
 {   11910, .8281581054688E+04, -.1469340087891E+04, .9279285888672E+03},
 {   11911, .8296583007812E+04, -.1468137939453E+04, .9212121582031E+03},
 {   11912, .8281574218750E+04, -.1483712036133E+04, .9279310302734E+03},
 {   11913, .8296644531250E+04, -.1483712036133E+04, .9211851196289E+03},
 {   11914, .8277086914062E+04, -.1454968139648E+04, .9178802490234E+03},
 {   11915, .8292025390625E+04, -.1452563842773E+04, .9111931762695E+03},
 {   11916, .8277082031250E+04, -.1469340087891E+04, .9178826904297E+03},
 {   11917, .8292085937500E+04, -.1468137939453E+04, .9111661376953E+03},
 {   11918, .8277076171875E+04, -.1483712036133E+04, .9178851928711E+03},
 {   11919, .8292146484375E+04, -.1483712036133E+04, .9111390991211E+03},
 {   11920, .8272591796875E+04, -.1454968139648E+04, .9078341674805E+03},
 {   11921, .8287529296875E+04, -.1452563842773E+04, .9011470336914E+03},
 {   11922, .8272585937500E+04, -.1469340087891E+04, .9078365478516E+03},
 {   11923, .8287589843750E+04, -.1468137939453E+04, .9011201782227E+03},
 {   11924, .8272580078125E+04, -.1483712036133E+04, .9078392333984E+03},
 {   11925, .8287650390625E+04, -.1483712036133E+04, .9010930786133E+03},
 {   11926, .8270655273438E+04, -.1454382690430E+04, .8908344726562E+03},
 {   11927, .8291445312500E+04, -.1450702758789E+04, .8894880981445E+03},
 {   11928, .8271165039062E+04, -.1473508666992E+04, .8939130859375E+03},
 {   11929, .8291036132812E+04, -.1469682495117E+04, .8913433837891E+03},
 {   11930, .8271893554688E+04, -.1490835937500E+04, .9008980102539E+03},
 {   11931, .8289410156250E+04, -.1487475585938E+04, .8964758300781E+03},
 {   11932, .8268678710938E+04, -.1453643066406E+04, .8738530883789E+03},
 {   11933, .8295269531250E+04, -.1448425292969E+04, .8778710937500E+03},
 {   11934, .8269721679688E+04, -.1477600219727E+04, .8799988403320E+03},
 {   11935, .8294436523438E+04, -.1471019042969E+04, .8815880737305E+03},
 {   11936, .8271210937500E+04, -.1497959960938E+04, .8939567871094E+03},
 {   11937, .8291172851562E+04, -.1491239257812E+04, .8918577880859E+03},
 {   11938, .8367934570312E+04, -.1495278808594E+04, .1112040161133E+04},
 {   11939, .8384018554688E+04, -.1489950073242E+04, .1102189086914E+04},
 {   11940, .8373342773438E+04, -.1506845703125E+04, .1115321289062E+04},
 {   11941, .8390441406250E+04, -.1496188232422E+04, .1102365722656E+04},
 {   11942, .8363247070312E+04, -.1498083496094E+04, .1099457031250E+04},
 {   11943, .8380300781250E+04, -.1491475097656E+04, .1090420288086E+04},
 {   11944, .8368465820312E+04, -.1512454956055E+04, .1100200683594E+04},
 {   11945, .8387504882812E+04, -.1499238037109E+04, .1088873657227E+04},
 {   11946, .8357858398438E+04, -.1499031372070E+04, .1086729125977E+04},
 {   11947, .8375613281250E+04, -.1491990112305E+04, .1078780639648E+04},
 {   11948, .8362187500000E+04, -.1514350830078E+04, .1084791381836E+04},
 {   11949, .8382629882812E+04, -.1500268310547E+04, .1075640258789E+04},
 {   11950, .8347480468750E+04, -.1499031372070E+04, .1063546142578E+04},
 {   11951, .8365235351562E+04, -.1491990112305E+04, .1055597167969E+04},
 {   11952, .8351811523438E+04, -.1514350830078E+04, .1061608398438E+04},
 {   11953, .8372251953125E+04, -.1500268310547E+04, .1052456298828E+04},
 {   11954, .8337102539062E+04, -.1499031372070E+04, .1040363159180E+04},
 {   11955, .8354859375000E+04, -.1491990112305E+04, .1032414184570E+04},
 {   11956, .8341430664062E+04, -.1514350830078E+04, .1038424804688E+04},
 {   11957, .8361874023438E+04, -.1500268310547E+04, .1029273315430E+04},
 {   11958, .8326724609375E+04, -.1499031372070E+04, .1017180297852E+04},
 {   11959, .8344480468750E+04, -.1491990112305E+04, .1009231445312E+04},
 {   11960, .8331053710938E+04, -.1514350830078E+04, .1015241943359E+04},
 {   11961, .8351494140625E+04, -.1500268310547E+04, .1006090759277E+04},
 {   11962, .8317642578125E+04, -.1499031372070E+04, .9968947143555E+03},
 {   11963, .8335399414062E+04, -.1491990112305E+04, .9889459228516E+03},
 {   11964, .8321970703125E+04, -.1514350830078E+04, .9949571533203E+03},
 {   11965, .8342417968750E+04, -.1500268310547E+04, .9858045654297E+03},
 {   11966, .8308562500000E+04, -.1499031372070E+04, .9766095581055E+03},
 {   11967, .8326319335938E+04, -.1491990112305E+04, .9686606445312E+03},
 {   11968, .8312891601562E+04, -.1514350830078E+04, .9746717529297E+03},
 {   11969, .8333335937500E+04, -.1500268310547E+04, .9655201416016E+03},
 {   11970, .8299481445312E+04, -.1499031372070E+04, .9563244628906E+03},
 {   11971, .8317236328125E+04, -.1491990112305E+04, .9483758544922E+03},
 {   11972, .8303812500000E+04, -.1514350830078E+04, .9543862304688E+03},
 {   11973, .8324254882812E+04, -.1500268310547E+04, .9452346801758E+03},
 {   11974, .8290401367188E+04, -.1499031372070E+04, .9360391235352E+03},
 {   11975, .8308159179688E+04, -.1491990112305E+04, .9280902099609E+03},
 {   11976, .8294728515625E+04, -.1514350830078E+04, .9341013183594E+03},
 {   11977, .8315170898438E+04, -.1500268310547E+04, .9249499511719E+03},
 {   11978, .8284496093750E+04, -.1498083496094E+04, .9235427246094E+03},
 {   11979, .8302597656250E+04, -.1491475097656E+04, .9168414306641E+03},
 {   11980, .8287417968750E+04, -.1512455078125E+04, .9191548461914E+03},
 {   11981, .8308548828125E+04, -.1499238037109E+04, .9124978637695E+03},
 {   11982, .8278233398438E+04, -.1495278930664E+04, .9116650390625E+03},
 {   11983, .8296293945312E+04, -.1489950195312E+04, .9062297363281E+03},
 {   11984, .8279388671875E+04, -.1506845703125E+04, .9054448242188E+03},
 {   11985, .8300444335938E+04, -.1496188232422E+04, .9013197021484E+03},
 {   11986, .8393401367188E+04, -.1449117065430E+04, .1102806152344E+04},
 {   11987, .8409002929688E+04, -.1444463623047E+04, .1103692871094E+04},
 {   11988, .8391308593750E+04, -.1470527343750E+04, .1102488281250E+04},
 {   11989, .8404918945312E+04, -.1472313598633E+04, .1103010864258E+04},
 {   11990, .8391002929688E+04, -.1447993286133E+04, .1088291748047E+04},
 {   11991, .8408469726562E+04, -.1441495239258E+04, .1084811279297E+04},
 {   11992, .8388833007812E+04, -.1470993408203E+04, .1088526977539E+04},
 {   11993, .8404352539062E+04, -.1472885253906E+04, .1085183959961E+04},
 {   11994, .8386055664062E+04, -.1447585205078E+04, .1074105468750E+04},
 {   11995, .8402986328125E+04, -.1440393188477E+04, .1066527343750E+04},
 {   11996, .8384078125000E+04, -.1471110351562E+04, .1074991699219E+04},
 {   11997, .8399291015625E+04, -.1472976196289E+04, .1068181640625E+04},
 {   11998, .8375677734375E+04, -.1447585205078E+04, .1050922119141E+04},
 {   11999, .8392608398438E+04, -.1440393188477E+04, .1043344238281E+04},
 {   12000, .8373698242188E+04, -.1471110351562E+04, .1051808837891E+04},
 {   12001, .8388912109375E+04, -.1472976196289E+04, .1044998535156E+04},
 {   12002, .8365301757812E+04, -.1447585205078E+04, .1027739257812E+04},
 {   12003, .8382229492188E+04, -.1440393188477E+04, .1020161010742E+04},
 {   12004, .8363321289062E+04, -.1471110351562E+04, .1028625732422E+04},
 {   12005, .8378534179688E+04, -.1472976196289E+04, .1021815917969E+04},
 {   12006, .8354921875000E+04, -.1447585205078E+04, .1004556396484E+04},
 {   12007, .8371851562500E+04, -.1440393188477E+04, .9969778442383E+03},
 {   12008, .8352943359375E+04, -.1471110351562E+04, .1005442565918E+04},
 {   12009, .8368157226562E+04, -.1472976196289E+04, .9986320800781E+03},
 {   12010, .8345842773438E+04, -.1447585205078E+04, .9842711181641E+03},
 {   12011, .8362769531250E+04, -.1440393188477E+04, .9766927490234E+03},
 {   12012, .8343861328125E+04, -.1471110351562E+04, .9851577148438E+03},
 {   12013, .8359076171875E+04, -.1472976196289E+04, .9783474121094E+03},
 {   12014, .8336760742188E+04, -.1447585205078E+04, .9639862670898E+03},
 {   12015, .8353689453125E+04, -.1440393188477E+04, .9564077148438E+03},
 {   12016, .8334782226562E+04, -.1471110351562E+04, .9648722534180E+03},
 {   12017, .8349993164062E+04, -.1472976196289E+04, .9580620727539E+03},
 {   12018, .8327679687500E+04, -.1447585205078E+04, .9437011718750E+03},
 {   12019, .8344608398438E+04, -.1440393188477E+04, .9361225585938E+03},
 {   12020, .8325699218750E+04, -.1471110351562E+04, .9445873413086E+03},
 {   12021, .8340913085938E+04, -.1472976196289E+04, .9377768554688E+03},
 {   12022, .8318600585938E+04, -.1447585205078E+04, .9234155273438E+03},
 {   12023, .8335528320312E+04, -.1440393188477E+04, .9158373413086E+03},
 {   12024, .8316619140625E+04, -.1471110351562E+04, .9243019409180E+03},
 {   12025, .8331833984375E+04, -.1472976196289E+04, .9174913330078E+03},
 {   12026, .8311312500000E+04, -.1447993286133E+04, .9102791137695E+03},
 {   12027, .8325544921875E+04, -.1441495239258E+04, .8995677490234E+03},
 {   12028, .8309691406250E+04, -.1470993408203E+04, .9117395019531E+03},
 {   12029, .8322523437500E+04, -.1472885253906E+04, .9023908691406E+03},
 {   12030, .8302087890625E+04, -.1449117065430E+04, .8988201904297E+03},
 {   12031, .8311814453125E+04, -.1444463623047E+04, .8865954589844E+03},
 {   12032, .8300927734375E+04, -.1470527343750E+04, .9005933837891E+03},
 {   12033, .8309602539062E+04, -.1472313476562E+04, .8900944213867E+03},
 {   16336, .8426942382812E+04, -.1212568969727E+04, .1041888671875E+04},
 {   16337, .8421750976562E+04, -.1212568969727E+04, .1030297851562E+04},
 {   16338, .8416561523438E+04, -.1212568969727E+04, .1018706237793E+04},
 {   16339, .8415265625000E+04, -.1212568969727E+04, .1015808471680E+04},
 {   16340, .8410077148438E+04, -.1212568969727E+04, .1004216308594E+04},
 {   16341, .8404886718750E+04, -.1212568969727E+04, .9926251220703E+03},
 {   16342, .8403589843750E+04, -.1212568969727E+04, .9897273559570E+03},
 {   16343, .8398401367188E+04, -.1212568969727E+04, .9781357421875E+03},
 {   16344, .8393209960938E+04, -.1212568969727E+04, .9665445556641E+03},
 {   16345, .8391914062500E+04, -.1212568969727E+04, .9636463012695E+03},
 {   16346, .8386726562500E+04, -.1212568969727E+04, .9520548095703E+03},
 {   16347, .8381537109375E+04, -.1212568969727E+04, .9404631347656E+03},
 {   16348, .8380239257812E+04, -.1212568969727E+04, .9375653686523E+03},
 {   16349, .8375050781250E+04, -.1212568969727E+04, .9259735717773E+03},
 {   16350, .8369859375000E+04, -.1212568969727E+04, .9143824462891E+03},
 {   16351, .8416561523438E+04, -.1358615112305E+04, .1018706237793E+04},
 {   16352, .8421750976562E+04, -.1358615112305E+04, .1030297851562E+04},
 {   16353, .8426942382812E+04, -.1358615112305E+04, .1041888671875E+04},
 {   16354, .8404886718750E+04, -.1358615112305E+04, .9926251220703E+03},
 {   16355, .8410077148438E+04, -.1358615112305E+04, .1004216308594E+04},
 {   16356, .8415265625000E+04, -.1358615112305E+04, .1015808471680E+04},
 {   16357, .8393209960938E+04, -.1358615112305E+04, .9665445556641E+03},
 {   16358, .8398401367188E+04, -.1358615112305E+04, .9781357421875E+03},
 {   16359, .8403589843750E+04, -.1358615112305E+04, .9897273559570E+03},
 {   16360, .8381537109375E+04, -.1358615112305E+04, .9404631347656E+03},
 {   16361, .8386725585938E+04, -.1358615112305E+04, .9520546264648E+03},
 {   16362, .8391914062500E+04, -.1358615112305E+04, .9636463012695E+03},
 {   16363, .8369859375000E+04, -.1358615112305E+04, .9143824462891E+03},
 {   16364, .8375050781250E+04, -.1358615112305E+04, .9259735717773E+03},
 {   16365, .8380239257812E+04, -.1358615112305E+04, .9375653686523E+03},
 {   16366, .8403649414062E+04, -.1349092041016E+04, .1241030761719E+04},
 {   16367, .8399874023438E+04, -.1349034057617E+04, .1242720458984E+04},
 {   16368, .8396103515625E+04, -.1348872802734E+04, .1244409790039E+04},
 {   16369, .8392336914062E+04, -.1348608032227E+04, .1246094970703E+04},
 {   16370, .8388576171875E+04, -.1348239257812E+04, .1247778076172E+04},
 {   16371, .8384828125000E+04, -.1347765625000E+04, .1249457031250E+04},
 {   16372, .8381090820312E+04, -.1347183227539E+04, .1251129638672E+04},
 {   16373, .8377371093750E+04, -.1346482177734E+04, .1252793945312E+04},
 {   16374, .8373669921875E+04, -.1345672363281E+04, .1254451416016E+04},
 {   16375, .8393270507812E+04, -.1361791992188E+04, .1217848388672E+04},
 {   16376, .8388418945312E+04, -.1361676025391E+04, .1220020385742E+04},
 {   16377, .8383572265625E+04, -.1361374267578E+04, .1222188964844E+04},
 {   16378, .8378744140625E+04, -.1360839721680E+04, .1224351318359E+04},
 {   16379, .8373938476562E+04, -.1360116821289E+04, .1226503173828E+04},
 {   16380, .8369162109375E+04, -.1359160156250E+04, .1228640136719E+04},
 {   16381, .8364424804688E+04, -.1358009033203E+04, .1230760986328E+04},
 {   16382, .8359740234375E+04, -.1356624389648E+04, .1232858398438E+04},
 {   16383, .8355109375000E+04, -.1355034179688E+04, .1234931640625E+04},
 {   16384, .8388082031250E+04, -.1399891967773E+04, .1206256225586E+04},
 {   16385, .8381340820312E+04, -.1399601928711E+04, .1209274780273E+04},
 {   16386, .8374640625000E+04, -.1398739257812E+04, .1212273681641E+04},
 {   16387, .8368022460938E+04, -.1397311767578E+04, .1215237304688E+04},
 {   16388, .8361521484375E+04, -.1395332031250E+04, .1218146240234E+04},
 {   16389, .8355179687500E+04, -.1392817382812E+04, .1220986450195E+04},
 {   16390, .8349024414062E+04, -.1389788574219E+04, .1223740356445E+04},
 {   16391, .8343091796875E+04, -.1386269775391E+04, .1226396606445E+04},
 {   16392, .8337409179688E+04, -.1382287841797E+04, .1228940795898E+04},
 {   16393, .8385488281250E+04, -.1431641967773E+04, .1200460815430E+04},
 {   16394, .8377363281250E+04, -.1431091186523E+04, .1204098510742E+04},
 {   16395, .8369363281250E+04, -.1429502441406E+04, .1207679809570E+04},
 {   16396, .8361594726562E+04, -.1426941040039E+04, .1211156250000E+04},
 {   16397, .8354151367188E+04, -.1423486938477E+04, .1214488647461E+04},
 {   16398, .8347076171875E+04, -.1419267700195E+04, .1217656738281E+04},
 {   16399, .8340403320312E+04, -.1414377563477E+04, .1220643310547E+04},
 {   16400, .8334158203125E+04, -.1408899780273E+04, .1223438964844E+04},
 {   16401, .8328337890625E+04, -.1402925048828E+04, .1226044433594E+04},
 {   16402, .8382892578125E+04, -.1463391967773E+04, .1194665527344E+04},
 {   16403, .8373588867188E+04, -.1462592285156E+04, .1198830444336E+04},
 {   16404, .8364489257812E+04, -.1460312744141E+04, .1202902954102E+04},
 {   16405, .8355769531250E+04, -.1456673950195E+04, .1206806884766E+04},
 {   16406, .8347555664062E+04, -.1451819580078E+04, .1210483886719E+04},
 {   16407, .8339895507812E+04, -.1445981567383E+04, .1213914184570E+04},
 {   16408, .8332813476562E+04, -.1439320190430E+04, .1217083007812E+04},
 {   16409, .8326329101562E+04, -.1431969848633E+04, .1219987182617E+04},
 {   16410, .8320395507812E+04, -.1424076782227E+04, .1222642700195E+04},
 {   16411, .8377703125000E+04, -.1487521972656E+04, .1183073486328E+04},
 {   16412, .8368129882812E+04, -.1486679199219E+04, .1187359619141E+04},
 {   16413, .8358791992188E+04, -.1484222656250E+04, .1191539794922E+04},
 {   16414, .8349905273438E+04, -.1480249877930E+04, .1195518676758E+04},
 {   16415, .8341652343750E+04, -.1474893066406E+04, .1199212890625E+04},
 {   16416, .8334166992188E+04, -.1468351562500E+04, .1202562622070E+04},
 {   16417, .8327563476562E+04, -.1460810424805E+04, .1205519409180E+04},
 {   16418, .8321926757812E+04, -.1452462036133E+04, .1208042968750E+04},
 {   16419, .8317299804688E+04, -.1443524536133E+04, .1210113525391E+04},
 {   16420, .8372514648438E+04, -.1511651977539E+04, .1171482177734E+04},
 {   16421, .8362908203125E+04, -.1510773681641E+04, .1175782470703E+04},
 {   16422, .8353566406250E+04, -.1508163085938E+04, .1179963989258E+04},
 {   16423, .8344750976562E+04, -.1503892456055E+04, .1183910766602E+04},
 {   16424, .8336703125000E+04, -.1498079833984E+04, .1187513671875E+04},
 {   16425, .8329645507812E+04, -.1490885986328E+04, .1190672851562E+04},
 {   16426, .8323774414062E+04, -.1482510131836E+04, .1193301879883E+04},
 {   16427, .8319249023438E+04, -.1473183715820E+04, .1195327148438E+04},
 {   16428, .8316198242188E+04, -.1463165039062E+04, .1196692138672E+04},
 {   16429, .8362138671875E+04, -.1518002075195E+04, .1148298828125E+04},
 {   16430, .8350589843750E+04, -.1516849243164E+04, .1153472900391E+04},
 {   16431, .8339425781250E+04, -.1513411010742E+04, .1158475463867E+04},
 {   16432, .8329014648438E+04, -.1507801635742E+04, .1163141601562E+04},
 {   16433, .8319702148438E+04, -.1500207153320E+04, .1167314819336E+04},
 {   16434, .8311798828125E+04, -.1490880249023E+04, .1170857788086E+04},
 {   16435, .8305567382812E+04, -.1480131713867E+04, .1173652343750E+04},
 {   16436, .8301219726562E+04, -.1468321289062E+04, .1175602539062E+04},
 {   16437, .8298907226562E+04, -.1455847412109E+04, .1176642822266E+04},
 {   16438, .8351757812500E+04, -.1524352050781E+04, .1125116333008E+04},
 {   16439, .8338183593750E+04, -.1522886962891E+04, .1131192382812E+04},
 {   16440, .8325132812500E+04, -.1518547973633E+04, .1137034545898E+04},
 {   16441, .8313106445312E+04, -.1511502075195E+04, .1142418701172E+04},
 {   16442, .8302565429688E+04, -.1502020019531E+04, .1147137329102E+04},
 {   16443, .8293917968750E+04, -.1490466552734E+04, .1151008666992E+04},
 {   16444, .8287495117188E+04, -.1477285888672E+04, .1153884887695E+04},
 {   16445, .8283540039062E+04, -.1462985107422E+04, .1155654541016E+04},
 {   16446, .8282207031250E+04, -.1448114013672E+04, .1156250976562E+04},
 {   16447, .8341379882812E+04, -.1524352050781E+04, .1101933227539E+04},
 {   16448, .8327807617188E+04, -.1522886962891E+04, .1108008911133E+04},
 {   16449, .8314756835938E+04, -.1518548095703E+04, .1113851196289E+04},
 {   16450, .8302730468750E+04, -.1511502075195E+04, .1119235351562E+04},
 {   16451, .8292189453125E+04, -.1502020019531E+04, .1123953857422E+04},
 {   16452, .8283539062500E+04, -.1490466552734E+04, .1127825683594E+04},
 {   16453, .8277116210938E+04, -.1477285888672E+04, .1130701538086E+04},
 {   16454, .8273162109375E+04, -.1462985107422E+04, .1132472045898E+04},
 {   16455, .8271831054688E+04, -.1448114013672E+04, .1133067260742E+04},
 {   16456, .8331000976562E+04, -.1524352050781E+04, .1078749877930E+04},
 {   16457, .8317427734375E+04, -.1522886962891E+04, .1084826171875E+04},
 {   16458, .8304377929688E+04, -.1518548095703E+04, .1090667968750E+04},
 {   16459, .8292351562500E+04, -.1511502075195E+04, .1096052246094E+04},
 {   16460, .8281810546875E+04, -.1502020019531E+04, .1100770629883E+04},
 {   16461, .8273161132812E+04, -.1490466552734E+04, .1104642944336E+04},
 {   16462, .8266736328125E+04, -.1477285888672E+04, .1107519165039E+04},
 {   16463, .8262781250000E+04, -.1462985107422E+04, .1109289062500E+04},
 {   16464, .8261451171875E+04, -.1448114013672E+04, .1109884277344E+04},
 {   16465, .8320622070312E+04, -.1524352050781E+04, .1055566894531E+04},
 {   16466, .8307050781250E+04, -.1522886962891E+04, .1061642700195E+04},
 {   16467, .8294000000000E+04, -.1518548095703E+04, .1067485473633E+04},
 {   16468, .8281972656250E+04, -.1511502075195E+04, .1072868896484E+04},
 {   16469, .8271431640625E+04, -.1502020019531E+04, .1077587890625E+04},
 {   16470, .8262783203125E+04, -.1490466552734E+04, .1081459716797E+04},
 {   16471, .8256357421875E+04, -.1477285888672E+04, .1084336181641E+04},
 {   16472, .8252406250000E+04, -.1462985107422E+04, .1086105834961E+04},
 {   16473, .8251073242188E+04, -.1448114013672E+04, .1086701293945E+04},
 {   16474, .8310247070312E+04, -.1524352050781E+04, .1032383789062E+04},
 {   16475, .8296671875000E+04, -.1522886962891E+04, .1038459594727E+04},
 {   16476, .8283622070312E+04, -.1518548095703E+04, .1044302246094E+04},
 {   16477, .8271595703125E+04, -.1511502075195E+04, .1049686035156E+04},
 {   16478, .8261054687500E+04, -.1502020019531E+04, .1054405029297E+04},
 {   16479, .8252404296875E+04, -.1490466552734E+04, .1058276855469E+04},
 {   16480, .8245979492188E+04, -.1477285888672E+04, .1061153320312E+04},
 {   16481, .8242026367188E+04, -.1462985107422E+04, .1062922363281E+04},
 {   16482, .8240697265625E+04, -.1448114013672E+04, .1063518188477E+04},
 {   16483, .8299867187500E+04, -.1524352050781E+04, .1009200866699E+04},
 {   16484, .8286293945312E+04, -.1522886962891E+04, .1015276672363E+04},
 {   16485, .8273242187500E+04, -.1518548095703E+04, .1021119262695E+04},
 {   16486, .8261215820312E+04, -.1511502075195E+04, .1026503295898E+04},
 {   16487, .8250675781250E+04, -.1502020019531E+04, .1031221923828E+04},
 {   16488, .8242025390625E+04, -.1490466552734E+04, .1035094116211E+04},
 {   16489, .8235604492188E+04, -.1477285888672E+04, .1037969482422E+04},
 {   16490, .8231646484375E+04, -.1462985107422E+04, .1039739990234E+04},
 {   16491, .8230318359375E+04, -.1448114013672E+04, .1040335571289E+04},
 {   16492, .8289487304688E+04, -.1524352050781E+04, .9860181274414E+03},
 {   16493, .8275916992188E+04, -.1522886962891E+04, .9920937500000E+03},
 {   16494, .8262864257812E+04, -.1518547973633E+04, .9979367675781E+03},
 {   16495, .8250837890625E+04, -.1511502075195E+04, .1003319946289E+04},
 {   16496, .8240298828125E+04, -.1502020019531E+04, .1008038513184E+04},
 {   16497, .8231648437500E+04, -.1490466552734E+04, .1011910766602E+04},
 {   16498, .8225221679688E+04, -.1477285888672E+04, .1014786621094E+04},
 {   16499, .8221269531250E+04, -.1462985107422E+04, .1016556518555E+04},
 {   16500, .8219941406250E+04, -.1448114013672E+04, .1017152099609E+04},
 {   16501, .8279110351562E+04, -.1524352050781E+04, .9628347167969E+03},
 {   16502, .8265537109375E+04, -.1522886962891E+04, .9689109497070E+03},
 {   16503, .8252487304688E+04, -.1518548095703E+04, .9747531127930E+03},
 {   16504, .8240459960938E+04, -.1511502075195E+04, .9801370239258E+03},
 {   16505, .8229919921875E+04, -.1502020019531E+04, .9848557128906E+03},
 {   16506, .8221272460938E+04, -.1490466552734E+04, .9887270507812E+03},
 {   16507, .8214843750000E+04, -.1477285888672E+04, .9916040039062E+03},
 {   16508, .8210893554688E+04, -.1462985107422E+04, .9933732910156E+03},
 {   16509, .8209562500000E+04, -.1448114013672E+04, .9939687500000E+03},
 {   16510, .8268731445312E+04, -.1524352050781E+04, .9396519775391E+03},
 {   16511, .8255161132812E+04, -.1522886962891E+04, .9457275390625E+03},
 {   16512, .8242108398438E+04, -.1518547973633E+04, .9515699462891E+03},
 {   16513, .8230080078125E+04, -.1511502075195E+04, .9569539794922E+03},
 {   16514, .8219542968750E+04, -.1502020019531E+04, .9616723632812E+03},
 {   16515, .8210892578125E+04, -.1490466552734E+04, .9655445556641E+03},
 {   16516, .8204466796875E+04, -.1477285888672E+04, .9684207763672E+03},
 {   16517, .8200512695312E+04, -.1462985107422E+04, .9701904907227E+03},
 {   16518, .8199182617188E+04, -.1448114013672E+04, .9707864379883E+03},
 {   16519, .8258354492188E+04, -.1516732055664E+04, .9164686279297E+03},
 {   16520, .8245090820312E+04, -.1515174804688E+04, .9224058227539E+03},
 {   16521, .8232430664062E+04, -.1510573974609E+04, .9280736694336E+03},
 {   16522, .8220946289062E+04, -.1503138061523E+04, .9332147216797E+03},
 {   16523, .8211161132812E+04, -.1493204956055E+04, .9375949096680E+03},
 {   16524, .8203519531250E+04, -.1481225952148E+04, .9410156860352E+03},
 {   16525, .8198370117188E+04, -.1467745117188E+04, .9433214111328E+03},
 {   16526, .8195941406250E+04, -.1453374389648E+04, .9444084472656E+03},
 {   16527, .8196348632812E+04, -.1438766723633E+04, .9442260131836E+03},
 {   16528, .8247525390625E+04, -.1510950927734E+04, .8934730224609E+03},
 {   16529, .8234710937500E+04, -.1509500488281E+04, .8991841430664E+03},
 {   16530, .8222423828125E+04, -.1505248168945E+04, .9046602783203E+03},
 {   16531, .8211153320312E+04, -.1498400878906E+04, .9096829833984E+03},
 {   16532, .8201331054688E+04, -.1489273193359E+04, .9140604248047E+03},
 {   16533, .8193328125000E+04, -.1478253173828E+04, .9176269531250E+03},
 {   16534, .8187410156250E+04, -.1465832763672E+04, .9202642822266E+03},
 {   16535, .8183779785156E+04, -.1452535766602E+04, .9218817749023E+03},
 {   16536, .8182541015625E+04, -.1438944335938E+04, .9224345703125E+03},
 {   16537, .8236812500000E+04, -.1498565673828E+04, .8704295654297E+03},
 {   16538, .8224815429688E+04, -.1497269531250E+04, .8757648925781E+03},
 {   16539, .8213256835938E+04, -.1493506103516E+04, .8809052124023E+03},
 {   16540, .8202524414062E+04, -.1487477539062E+04, .8856769409180E+03},
 {   16541, .8192954101562E+04, -.1479468627930E+04, .8899342041016E+03},
 {   16542, .8184820800781E+04, -.1469793212891E+04, .8935511474609E+03},
 {   16543, .8178300781250E+04, -.1458874023438E+04, .8964507446289E+03},
 {   16544, .8173550781250E+04, -.1447117553711E+04, .8985638427734E+03},
 {   16545, .8170635742188E+04, -.1434991455078E+04, .8998594970703E+03},
 {   16546, .8225369140625E+04, -.1476274536133E+04, .8476817626953E+03},
 {   16547, .8213412109375E+04, -.1475192871094E+04, .8529566650391E+03},
 {   16548, .8201833984375E+04, -.1472091308594E+04, .8580650634766E+03},
 {   16549, .8190943847656E+04, -.1467160156250E+04, .8628696899414E+03},
 {   16550, .8180984375000E+04, -.1460645751953E+04, .8672635498047E+03},
 {   16551, .8172167480469E+04, -.1452776367188E+04, .8711541137695E+03},
 {   16552, .8164567382812E+04, -.1443887695312E+04, .8745072021484E+03},
 {   16553, .8158286132812E+04, -.1434242187500E+04, .8772778930664E+03},
 {   16554, .8153355957031E+04, -.1424149902344E+04, .8794534912109E+03},
 {   16555, .8213367187500E+04, -.1467190917969E+04, .8251593017578E+03},
 {   16556, .8201268554688E+04, -.1466250732422E+04, .8304489746094E+03},
 {   16557, .8189483886719E+04, -.1463600708008E+04, .8356022338867E+03},
 {   16558, .8178242675781E+04, -.1459435913086E+04, .8405174560547E+03},
 {   16559, .8167693847656E+04, -.1453985107422E+04, .8451309204102E+03},
 {   16560, .8157948730469E+04, -.1447409545898E+04, .8493914794922E+03},
 {   16561, .8148979492188E+04, -.1439981567383E+04, .8533133544922E+03},
 {   16562, .8140811523438E+04, -.1431828369141E+04, .8568847045898E+03},
 {   16563, .8133399414062E+04, -.1423099121094E+04, .8601256103516E+03},
 {   16564, .8362827148438E+04, -.1342605712891E+04, .1259305664062E+04},
 {   16565, .8352286132812E+04, -.1338455322266E+04, .1264023803711E+04},
 {   16566, .8342193359375E+04, -.1333136230469E+04, .1268542480469E+04},
 {   16567, .8332759765625E+04, -.1326525268555E+04, .1272766357422E+04},
 {   16568, .8324321289062E+04, -.1318458618164E+04, .1276543701172E+04},
 {   16569, .8317365234375E+04, -.1308839721680E+04, .1279657226562E+04},
 {   16570, .8312635742188E+04, -.1297716918945E+04, .1281773559570E+04},
 {   16571, .8310916992188E+04, -.1285592041016E+04, .1282543823242E+04},
 {   16572, .8312635742188E+04, -.1273467163086E+04, .1281773559570E+04},
 {   16573, .8317365234375E+04, -.1262344238281E+04, .1279657226562E+04},
 {   16574, .8324321289062E+04, -.1252725463867E+04, .1276543701172E+04},
 {   16575, .8332759765625E+04, -.1244658813477E+04, .1272766357422E+04},
 {   16576, .8342194335938E+04, -.1238047729492E+04, .1268542358398E+04},
 {   16577, .8352286132812E+04, -.1232728759766E+04, .1264023803711E+04},
 {   16578, .8362827148438E+04, -.1228578369141E+04, .1259305664062E+04},
 {   16579, .8373669921875E+04, -.1225511718750E+04, .1254451416016E+04},
 {   16580, .8344752929688E+04, -.1350520507812E+04, .1239568115234E+04},
 {   16581, .8334907226562E+04, -.1344792602539E+04, .1243975585938E+04},
 {   16582, .8325758789062E+04, -.1337815673828E+04, .1248070800781E+04},
 {   16583, .8317546875000E+04, -.1329558227539E+04, .1251746459961E+04},
 {   16584, .8310569335938E+04, -.1320036010742E+04, .1254871093750E+04},
 {   16585, .8305166015625E+04, -.1309353393555E+04, .1257289062500E+04},
 {   16586, .8301735351562E+04, -.1297733520508E+04, .1258824218750E+04},
 {   16587, .8300538085938E+04, -.1285592041016E+04, .1259360839844E+04},
 {   16588, .8301735351562E+04, -.1273450439453E+04, .1258824218750E+04},
 {   16589, .8305166015625E+04, -.1261830688477E+04, .1257289062500E+04},
 {   16590, .8310569335938E+04, -.1251148071289E+04, .1254871093750E+04},
 {   16591, .8317546875000E+04, -.1241625854492E+04, .1251746459961E+04},
 {   16592, .8325758789062E+04, -.1233368408203E+04, .1248070800781E+04},
 {   16593, .8334907226562E+04, -.1226391357422E+04, .1243975585938E+04},
 {   16594, .8344752929688E+04, -.1220663574219E+04, .1239568115234E+04},
 {   16595, .8355109375000E+04, -.1216149780273E+04, .1234931640625E+04},
 {   16596, .8327375976562E+04, -.1373532470703E+04, .1233432861328E+04},
 {   16597, .8318479492188E+04, -.1363407958984E+04, .1237414428711E+04},
 {   16598, .8310815429688E+04, -.1352140014648E+04, .1240846191406E+04},
 {   16599, .8304464843750E+04, -.1339926757812E+04, .1243687500000E+04},
 {   16600, .8299493164062E+04, -.1326972534180E+04, .1245914062500E+04},
 {   16601, .8295905273438E+04, -.1313479248047E+04, .1247519409180E+04},
 {   16602, .8293766601562E+04, -.1299622924805E+04, .1248478759766E+04},
 {   16603, .8293029296875E+04, -.1285592041016E+04, .1248807006836E+04},
 {   16604, .8293766601562E+04, -.1271561157227E+04, .1248478759766E+04},
 {   16605, .8295905273438E+04, -.1257704711914E+04, .1247519409180E+04},
 {   16606, .8299493164062E+04, -.1244211425781E+04, .1245914062500E+04},
 {   16607, .8304464843750E+04, -.1231257324219E+04, .1243687500000E+04},
 {   16608, .8310815429688E+04, -.1219043945312E+04, .1240846191406E+04},
 {   16609, .8318479492188E+04, -.1207776123047E+04, .1237414428711E+04},
 {   16610, .8327375976562E+04, -.1197651489258E+04, .1233432861328E+04},
 {   16611, .8337409179688E+04, -.1188896240234E+04, .1228940795898E+04},
 {   16612, .8318628906250E+04, -.1390868774414E+04, .1230390502930E+04},
 {   16613, .8310359375000E+04, -.1377655639648E+04, .1234093261719E+04},
 {   16614, .8303456054688E+04, -.1363562011719E+04, .1237183837891E+04},
 {   16615, .8297883789062E+04, -.1348774902344E+04, .1239677978516E+04},
 {   16616, .8293596679688E+04, -.1333450683594E+04, .1241596923828E+04},
 {   16617, .8290546875000E+04, -.1317728271484E+04, .1242961669922E+04},
 {   16618, .8288732421875E+04, -.1301729614258E+04, .1243774169922E+04},
 {   16619, .8288116210938E+04, -.1285592041016E+04, .1244049804688E+04},
 {   16620, .8288732421875E+04, -.1269454467773E+04, .1243774169922E+04},
 {   16621, .8290546875000E+04, -.1253455810547E+04, .1242961669922E+04},
 {   16622, .8293596679688E+04, -.1237733398438E+04, .1241596923828E+04},
 {   16623, .8297883789062E+04, -.1222409179688E+04, .1239677978516E+04},
 {   16624, .8303456054688E+04, -.1207622070312E+04, .1237183837891E+04},
 {   16625, .8310359375000E+04, -.1193528442383E+04, .1234093261719E+04},
 {   16626, .8318628906250E+04, -.1180315307617E+04, .1230390502930E+04},
 {   16627, .8328337890625E+04, -.1168259033203E+04, .1226044433594E+04},
 {   16628, .8311053710938E+04, -.1408854125977E+04, .1226824951172E+04},
 {   16629, .8303297851562E+04, -.1392597290039E+04, .1230296508789E+04},
 {   16630, .8296944335938E+04, -.1375627197266E+04, .1233141235352E+04},
 {   16631, .8291883789062E+04, -.1358141601562E+04, .1235406616211E+04},
 {   16632, .8288041015625E+04, -.1340287963867E+04, .1237125732422E+04},
 {   16633, .8285341796875E+04, -.1322184814453E+04, .1238334594727E+04},
 {   16634, .8283736328125E+04, -.1303925903320E+04, .1239052734375E+04},
 {   16635, .8283205078125E+04, -.1285592041016E+04, .1239291748047E+04},
 {   16636, .8283736328125E+04, -.1267258300781E+04, .1239052734375E+04},
 {   16637, .8285341796875E+04, -.1248999389648E+04, .1238334594727E+04},
 {   16638, .8288041015625E+04, -.1230896240234E+04, .1237125732422E+04},
 {   16639, .8291883789062E+04, -.1213042602539E+04, .1235406616211E+04},
 {   16640, .8296944335938E+04, -.1195556884766E+04, .1233141235352E+04},
 {   16641, .8303297851562E+04, -.1178586791992E+04, .1230296508789E+04},
 {   16642, .8311053710938E+04, -.1162330078125E+04, .1226824951172E+04},
 {   16643, .8320395507812E+04, -.1147107299805E+04, .1222642700195E+04},
 {   16644, .8305195312500E+04, -.1426956542969E+04, .1215532958984E+04},
 {   16645, .8295216796875E+04, -.1408743408203E+04, .1219999877930E+04},
 {   16646, .8287119140625E+04, -.1389437500000E+04, .1223624755859E+04},
 {   16647, .8280729492188E+04, -.1369380737305E+04, .1226485839844E+04},
 {   16648, .8275918945312E+04, -.1348808349609E+04, .1228639282227E+04},
 {   16649, .8272547851562E+04, -.1327903930664E+04, .1230146606445E+04},
 {   16650, .8270568359375E+04, -.1306795288086E+04, .1231034057617E+04},
 {   16651, .8269901367188E+04, -.1285592041016E+04, .1231332031250E+04},
 {   16652, .8270568359375E+04, -.1264388916016E+04, .1231034057617E+04},
 {   16653, .8272547851562E+04, -.1243280151367E+04, .1230146606445E+04},
 {   16654, .8275918945312E+04, -.1222375732422E+04, .1228639282227E+04},
 {   16655, .8280729492188E+04, -.1201803344727E+04, .1226485839844E+04},
 {   16656, .8287119140625E+04, -.1181746582031E+04, .1223624755859E+04},
 {   16657, .8295216796875E+04, -.1162440673828E+04, .1219999877930E+04},
 {   16658, .8305195312500E+04, -.1144227661133E+04, .1215532958984E+04},
 {   16659, .8317299804688E+04, -.1127659423828E+04, .1210113525391E+04},
 {   16660, .8300989257812E+04, -.1445488037109E+04, .1203501464844E+04},
 {   16661, .8288444335938E+04, -.1425455444336E+04, .1209116577148E+04},
 {   16662, .8278270507812E+04, -.1403868652344E+04, .1213670898438E+04},
 {   16663, .8270243164062E+04, -.1381224243164E+04, .1217266113281E+04},
 {   16664, .8264192382812E+04, -.1357851196289E+04, .1219973510742E+04},
 {   16665, .8259946289062E+04, -.1334007568359E+04, .1221875122070E+04},
 {   16666, .8257447265625E+04, -.1309868286133E+04, .1222993774414E+04},
 {   16667, .8256601562500E+04, -.1285592041016E+04, .1223373046875E+04},
 {   16668, .8257447265625E+04, -.1261315795898E+04, .1222993774414E+04},
 {   16669, .8259946289062E+04, -.1237176391602E+04, .1221875122070E+04},
 {   16670, .8264192382812E+04, -.1213332763672E+04, .1219973510742E+04},
 {   16671, .8270243164062E+04, -.1189959838867E+04, .1217266113281E+04},
 {   16672, .8278270507812E+04, -.1167315429688E+04, .1213670898438E+04},
 {   16673, .8288444335938E+04, -.1145728515625E+04, .1209116577148E+04},
 {   16674, .8300989257812E+04, -.1125695922852E+04, .1203501464844E+04},
 {   16675, .8316198242188E+04, -.1108019042969E+04, .1196692138672E+04},
 {   16676, .8285581054688E+04, -.1438115478516E+04, .1182607910156E+04},
 {   16677, .8274536132812E+04, -.1418594726562E+04, .1187552001953E+04},
 {   16678, .8265543945312E+04, -.1397847534180E+04, .1191578613281E+04},
 {   16679, .8258422851562E+04, -.1376238159180E+04, .1194765991211E+04},
 {   16680, .8253017578125E+04, -.1354037841797E+04, .1197185791016E+04},
 {   16681, .8249241210938E+04, -.1331429199219E+04, .1198876342773E+04},
 {   16682, .8246992187500E+04, -.1308571289062E+04, .1199883789062E+04},
 {   16683, .8246233398438E+04, -.1285592041016E+04, .1200222290039E+04},
 {   16684, .8246992187500E+04, -.1262612670898E+04, .1199883789062E+04},
 {   16685, .8249241210938E+04, -.1239754882812E+04, .1198876342773E+04},
 {   16686, .8253017578125E+04, -.1217146240234E+04, .1197185791016E+04},
 {   16687, .8258422851562E+04, -.1194945800781E+04, .1194765991211E+04},
 {   16688, .8265543945312E+04, -.1173336547852E+04, .1191578613281E+04},
 {   16689, .8274537109375E+04, -.1152589355469E+04, .1187552368164E+04},
 {   16690, .8285581054688E+04, -.1133068603516E+04, .1182607910156E+04},
 {   16691, .8298907226562E+04, -.1115336669922E+04, .1176642822266E+04},
 {   16692, .8270563476562E+04, -.1430581542969E+04, .1161463989258E+04},
 {   16693, .8260870117188E+04, -.1411676757812E+04, .1165803222656E+04},
 {   16694, .8252953125000E+04, -.1391803222656E+04, .1169347290039E+04},
 {   16695, .8246663085938E+04, -.1371243652344E+04, .1172162719727E+04},
 {   16696, .8241872070312E+04, -.1350204345703E+04, .1174307373047E+04},
 {   16697, .8238505859375E+04, -.1328836547852E+04, .1175814453125E+04},
 {   16698, .8236509765625E+04, -.1307263549805E+04, .1176708984375E+04},
 {   16699, .8235843750000E+04, -.1285592041016E+04, .1177006713867E+04},
 {   16700, .8236509765625E+04, -.1263920410156E+04, .1176708984375E+04},
 {   16701, .8238505859375E+04, -.1242347534180E+04, .1175814453125E+04},
 {   16702, .8241872070312E+04, -.1220979736328E+04, .1174307373047E+04},
 {   16703, .8246663085938E+04, -.1199940307617E+04, .1172162719727E+04},
 {   16704, .8252953125000E+04, -.1179380859375E+04, .1169347290039E+04},
 {   16705, .8260870117188E+04, -.1159507202148E+04, .1165803222656E+04},
 {   16706, .8270563476562E+04, -.1140602539062E+04, .1161463989258E+04},
 {   16707, .8282207031250E+04, -.1123070068359E+04, .1156250976562E+04},
 {   16708, .8260184570312E+04, -.1430581542969E+04, .1138280883789E+04},
 {   16709, .8250491210938E+04, -.1411676757812E+04, .1142620361328E+04},
 {   16710, .8242574218750E+04, -.1391803222656E+04, .1146164794922E+04},
 {   16711, .8236288085938E+04, -.1371243652344E+04, .1148979736328E+04},
 {   16712, .8231496093750E+04, -.1350204345703E+04, .1151124267578E+04},
 {   16713, .8228128906250E+04, -.1328836547852E+04, .1152630493164E+04},
 {   16714, .8226129882812E+04, -.1307263549805E+04, .1153525878906E+04},
 {   16715, .8225464843750E+04, -.1285592041016E+04, .1153824340820E+04},
 {   16716, .8226129882812E+04, -.1263920410156E+04, .1153525878906E+04},
 {   16717, .8228128906250E+04, -.1242347534180E+04, .1152630493164E+04},
 {   16718, .8231496093750E+04, -.1220979736328E+04, .1151124267578E+04},
 {   16719, .8236288085938E+04, -.1199940307617E+04, .1148979736328E+04},
 {   16720, .8242574218750E+04, -.1179380859375E+04, .1146164550781E+04},
 {   16721, .8250491210938E+04, -.1159507202148E+04, .1142620117188E+04},
 {   16722, .8260184570312E+04, -.1140602539062E+04, .1138280883789E+04},
 {   16723, .8271831054688E+04, -.1123070068359E+04, .1133067260742E+04},
 {   16724, .8249805664062E+04, -.1430581542969E+04, .1115098266602E+04},
 {   16725, .8240113281250E+04, -.1411676757812E+04, .1119437011719E+04},
 {   16726, .8232195312500E+04, -.1391803222656E+04, .1122981689453E+04},
 {   16727, .8225906250000E+04, -.1371243652344E+04, .1125797241211E+04},
 {   16728, .8221117187500E+04, -.1350204345703E+04, .1127940917969E+04},
 {   16729, .8217748046875E+04, -.1328836547852E+04, .1129448608398E+04},
 {   16730, .8215751953125E+04, -.1307263549805E+04, .1130343017578E+04},
 {   16731, .8215087890625E+04, -.1285592041016E+04, .1130640869141E+04},
 {   16732, .8215751953125E+04, -.1263920410156E+04, .1130343017578E+04},
 {   16733, .8217748046875E+04, -.1242347534180E+04, .1129448608398E+04},
 {   16734, .8221117187500E+04, -.1220979736328E+04, .1127940917969E+04},
 {   16735, .8225906250000E+04, -.1199940307617E+04, .1125797241211E+04},
 {   16736, .8232195312500E+04, -.1179380859375E+04, .1122981689453E+04},
 {   16737, .8240113281250E+04, -.1159507202148E+04, .1119437011719E+04},
 {   16738, .8249805664062E+04, -.1140602539062E+04, .1115098266602E+04},
 {   16739, .8261451171875E+04, -.1123070068359E+04, .1109884277344E+04},
 {   16740, .8239429687500E+04, -.1430581542969E+04, .1091914794922E+04},
 {   16741, .8229736328125E+04, -.1411676757812E+04, .1096253906250E+04},
 {   16742, .8221820312500E+04, -.1391803222656E+04, .1099797851562E+04},
 {   16743, .8215526367188E+04, -.1371243652344E+04, .1102614013672E+04},
 {   16744, .8210739257812E+04, -.1350204345703E+04, .1104758300781E+04},
 {   16745, .8207372070312E+04, -.1328836547852E+04, .1106264892578E+04},
 {   16746, .8205371093750E+04, -.1307263549805E+04, .1107160644531E+04},
 {   16747, .8204708007812E+04, -.1285592041016E+04, .1107457763672E+04},
 {   16748, .8205371093750E+04, -.1263920410156E+04, .1107160400391E+04},
 {   16749, .8207372070312E+04, -.1242347534180E+04, .1106264892578E+04},
 {   16750, .8210739257812E+04, -.1220979736328E+04, .1104758300781E+04},
 {   16751, .8215526367188E+04, -.1199940307617E+04, .1102614013672E+04},
 {   16752, .8221820312500E+04, -.1179380859375E+04, .1099797851562E+04},
 {   16753, .8229736328125E+04, -.1159507202148E+04, .1096253906250E+04},
 {   16754, .8239429687500E+04, -.1140602539062E+04, .1091914550781E+04},
 {   16755, .8251073242188E+04, -.1123070068359E+04, .1086701293945E+04},
 {   16756, .8229050781250E+04, -.1430581542969E+04, .1068731933594E+04},
 {   16757, .8219357421875E+04, -.1411676757812E+04, .1073071044922E+04},
 {   16758, .8211438476562E+04, -.1391803222656E+04, .1076614990234E+04},
 {   16759, .8205150390625E+04, -.1371243652344E+04, .1079430908203E+04},
 {   16760, .8200360351562E+04, -.1350204345703E+04, .1081574829102E+04},
 {   16761, .8196994140625E+04, -.1328836547852E+04, .1083081787109E+04},
 {   16762, .8194996093750E+04, -.1307263549805E+04, .1083976440430E+04},
 {   16763, .8194328125000E+04, -.1285592041016E+04, .1084275146484E+04},
 {   16764, .8194996093750E+04, -.1263920410156E+04, .1083976440430E+04},
 {   16765, .8196994140625E+04, -.1242347534180E+04, .1083081787109E+04},
 {   16766, .8200360351562E+04, -.1220979736328E+04, .1081574829102E+04},
 {   16767, .8205150390625E+04, -.1199940429688E+04, .1079430908203E+04},
 {   16768, .8211438476562E+04, -.1179380859375E+04, .1076614990234E+04},
 {   16769, .8219357421875E+04, -.1159507202148E+04, .1073070800781E+04},
 {   16770, .8229050781250E+04, -.1140602539062E+04, .1068731689453E+04},
 {   16771, .8240697265625E+04, -.1123070068359E+04, .1063518188477E+04},
 {   16772, .8218670898438E+04, -.1430581542969E+04, .1045548706055E+04},
 {   16773, .8208978515625E+04, -.1411676879883E+04, .1049888183594E+04},
 {   16774, .8201061523438E+04, -.1391803222656E+04, .1053432250977E+04},
 {   16775, .8194773437500E+04, -.1371243652344E+04, .1056247802734E+04},
 {   16776, .8189979980469E+04, -.1350204345703E+04, .1058391845703E+04},
 {   16777, .8186616210938E+04, -.1328836547852E+04, .1059898925781E+04},
 {   16778, .8184617187500E+04, -.1307263549805E+04, .1060793823242E+04},
 {   16779, .8183954101562E+04, -.1285592041016E+04, .1061091064453E+04},
 {   16780, .8184617187500E+04, -.1263920410156E+04, .1060793823242E+04},
 {   16781, .8186616210938E+04, -.1242347534180E+04, .1059898925781E+04},
 {   16782, .8189979980469E+04, -.1220979736328E+04, .1058391845703E+04},
 {   16783, .8194773437500E+04, -.1199940429688E+04, .1056247802734E+04},
 {   16784, .8201061523438E+04, -.1179380859375E+04, .1053432250977E+04},
 {   16785, .8208978515625E+04, -.1159507202148E+04, .1049888183594E+04},
 {   16786, .8218670898438E+04, -.1140602539062E+04, .1045548461914E+04},
 {   16787, .8230318359375E+04, -.1123070068359E+04, .1040335571289E+04},
 {   16788, .8208292968750E+04, -.1430581542969E+04, .1022365722656E+04},
 {   16789, .8198600585938E+04, -.1411676879883E+04, .1026704956055E+04},
 {   16790, .8190685058594E+04, -.1391803222656E+04, .1030248779297E+04},
 {   16791, .8184394042969E+04, -.1371243652344E+04, .1033064697266E+04},
 {   16792, .8179604003906E+04, -.1350204345703E+04, .1035208740234E+04},
 {   16793, .8176237792969E+04, -.1328836547852E+04, .1036715942383E+04},
 {   16794, .8174237792969E+04, -.1307263549805E+04, .1037610595703E+04},
 {   16795, .8173574707031E+04, -.1285592041016E+04, .1037908203125E+04},
 {   16796, .8174237792969E+04, -.1263920410156E+04, .1037610595703E+04},
 {   16797, .8176237792969E+04, -.1242347534180E+04, .1036715942383E+04},
 {   16798, .8179604003906E+04, -.1220979736328E+04, .1035208740234E+04},
 {   16799, .8184394042969E+04, -.1199940429688E+04, .1033064697266E+04},
 {   16800, .8190685058594E+04, -.1179380859375E+04, .1030248535156E+04},
 {   16801, .8198600585938E+04, -.1159507202148E+04, .1026704711914E+04},
 {   16802, .8208292968750E+04, -.1140602539062E+04, .1022365722656E+04},
 {   16803, .8219941406250E+04, -.1123070068359E+04, .1017152099609E+04},
 {   16804, .8197916015625E+04, -.1430581542969E+04, .9991823730469E+03},
 {   16805, .8188223632812E+04, -.1411676879883E+04, .1003521850586E+04},
 {   16806, .8180305175781E+04, -.1391803222656E+04, .1007065979004E+04},
 {   16807, .8174017089844E+04, -.1371243652344E+04, .1009881713867E+04},
 {   16808, .8169226074219E+04, -.1350204345703E+04, .1012025878906E+04},
 {   16809, .8165860839844E+04, -.1328836547852E+04, .1013532958984E+04},
 {   16810, .8163860351562E+04, -.1307263549805E+04, .1014428100586E+04},
 {   16811, .8163196777344E+04, -.1285592041016E+04, .1014725463867E+04},
 {   16812, .8163860351562E+04, -.1263920410156E+04, .1014428100586E+04},
 {   16813, .8165860839844E+04, -.1242347534180E+04, .1013532958984E+04},
 {   16814, .8169226074219E+04, -.1220979736328E+04, .1012025878906E+04},
 {   16815, .8174017089844E+04, -.1199940429688E+04, .1009881713867E+04},
 {   16816, .8180305175781E+04, -.1179380859375E+04, .1007065979004E+04},
 {   16817, .8188223632812E+04, -.1159507202148E+04, .1003521850586E+04},
 {   16818, .8197916015625E+04, -.1140602539062E+04, .9991823730469E+03},
 {   16819, .8209562500000E+04, -.1123070068359E+04, .9939687500000E+03},
 {   16820, .8187537597656E+04, -.1430581542969E+04, .9759996337891E+03},
 {   16821, .8177844726562E+04, -.1411676879883E+04, .9803388671875E+03},
 {   16822, .8169928710938E+04, -.1391803222656E+04, .9838826904297E+03},
 {   16823, .8163638183594E+04, -.1371243652344E+04, .9866986083984E+03},
 {   16824, .8158847656250E+04, -.1350204345703E+04, .9888427124023E+03},
 {   16825, .8155478515625E+04, -.1328836547852E+04, .9903501586914E+03},
 {   16826, .8153480957031E+04, -.1307263549805E+04, .9912449340820E+03},
 {   16827, .8152817871094E+04, -.1285592041016E+04, .9915422363281E+03},
 {   16828, .8153480957031E+04, -.1263920410156E+04, .9912449340820E+03},
 {   16829, .8155478515625E+04, -.1242347534180E+04, .9903501586914E+03},
 {   16830, .8158847656250E+04, -.1220979736328E+04, .9888427124023E+03},
 {   16831, .8163638183594E+04, -.1199940429688E+04, .9866986083984E+03},
 {   16832, .8169928710938E+04, -.1179380859375E+04, .9838826904297E+03},
 {   16833, .8177844726562E+04, -.1159507202148E+04, .9803388671875E+03},
 {   16834, .8187537597656E+04, -.1140602539062E+04, .9759996337891E+03},
 {   16835, .8199182617188E+04, -.1123070068359E+04, .9707864379883E+03},
 {   16836, .8185619140625E+04, -.1422139892578E+04, .9490296020508E+03},
 {   16837, .8176752929688E+04, -.1404242919922E+04, .9529985351562E+03},
 {   16838, .8169532714844E+04, -.1385478637695E+04, .9562304077148E+03},
 {   16839, .8163822753906E+04, -.1366100341797E+04, .9587869873047E+03},
 {   16840, .8159481933594E+04, -.1346299682617E+04, .9607293701172E+03},
 {   16841, .8156437500000E+04, -.1326211547852E+04, .9620924072266E+03},
 {   16842, .8154632324219E+04, -.1305944702148E+04, .9629011840820E+03},
 {   16843, .8154030761719E+04, -.1285592041016E+04, .9631704101562E+03},
 {   16844, .8154632324219E+04, -.1265239379883E+04, .9629011840820E+03},
 {   16845, .8156437500000E+04, -.1244972534180E+04, .9620924072266E+03},
 {   16846, .8159481933594E+04, -.1224884399414E+04, .9607293701172E+03},
 {   16847, .8163822753906E+04, -.1205083740234E+04, .9587869873047E+03},
 {   16848, .8169532714844E+04, -.1185705444336E+04, .9562304077148E+03},
 {   16849, .8176752929688E+04, -.1166941162109E+04, .9529985351562E+03},
 {   16850, .8185619140625E+04, -.1149044189453E+04, .9490296020508E+03},
 {   16851, .8196348632812E+04, -.1132417358398E+04, .9442260131836E+03},
 {   16852, .8172026367188E+04, -.1422170898438E+04, .9271206665039E+03},
 {   16853, .8163328125000E+04, -.1404191772461E+04, .9309960937500E+03},
 {   16854, .8156252441406E+04, -.1385384643555E+04, .9341506958008E+03},
 {   16855, .8150651855469E+04, -.1365993408203E+04, .9366464233398E+03},
 {   16856, .8146402832031E+04, -.1346200561523E+04, .9385404052734E+03},
 {   16857, .8143423339844E+04, -.1326136230469E+04, .9398682250977E+03},
 {   16858, .8141656738281E+04, -.1305904174805E+04, .9406552734375E+03},
 {   16859, .8141066406250E+04, -.1285592041016E+04, .9409178466797E+03},
 {   16860, .8141657714844E+04, -.1265279907227E+04, .9406551513672E+03},
 {   16861, .8143423339844E+04, -.1245047851562E+04, .9398682250977E+03},
 {   16862, .8146402832031E+04, -.1224983520508E+04, .9385402832031E+03},
 {   16863, .8150651855469E+04, -.1205190673828E+04, .9366464233398E+03},
 {   16864, .8156252441406E+04, -.1185799438477E+04, .9341505737305E+03},
 {   16865, .8163328125000E+04, -.1166992309570E+04, .9309960937500E+03},
 {   16866, .8172026367188E+04, -.1149013183594E+04, .9271206665039E+03},
 {   16867, .8182541015625E+04, -.1132239746094E+04, .9224345703125E+03},
 {   16868, .8160607421875E+04, -.1418537109375E+04, .9043188476562E+03},
 {   16869, .8152301269531E+04, -.1400971801758E+04, .9080126953125E+03},
 {   16870, .8145535644531E+04, -.1382636474609E+04, .9110222167969E+03},
 {   16871, .8140174316406E+04, -.1363757812500E+04, .9134057006836E+03},
 {   16872, .8136107910156E+04, -.1344503540039E+04, .9152145996094E+03},
 {   16873, .8133255371094E+04, -.1324995605469E+04, .9164830322266E+03},
 {   16874, .8131563964844E+04, -.1305331298828E+04, .9172351684570E+03},
 {   16875, .8131000000000E+04, -.1285592041016E+04, .9174860229492E+03},
 {   16876, .8131563964844E+04, -.1265852783203E+04, .9172351684570E+03},
 {   16877, .8133255371094E+04, -.1246188354492E+04, .9164830322266E+03},
 {   16878, .8136107910156E+04, -.1226680541992E+04, .9152145996094E+03},
 {   16879, .8140174316406E+04, -.1207426147461E+04, .9134057006836E+03},
 {   16880, .8145535644531E+04, -.1188547607422E+04, .9110222167969E+03},
 {   16881, .8152301269531E+04, -.1170212280273E+04, .9080125732422E+03},
 {   16882, .8160607421875E+04, -.1152646972656E+04, .9043188476562E+03},
 {   16883, .8170635742188E+04, -.1136192626953E+04, .8998594970703E+03},
 {   16884, .8145441894531E+04, -.1408795410156E+04, .8829306030273E+03},
 {   16885, .8141574218750E+04, -.1392469482422E+04, .8846057128906E+03},
 {   16886, .8134637695312E+04, -.1375460449219E+04, .8876669921875E+03},
 {   16887, .8131445800781E+04, -.1357967285156E+04, .8890603637695E+03},
 {   16888, .8129576171875E+04, -.1340134887695E+04, .8898704833984E+03},
 {   16889, .8126636230469E+04, -.1322072021484E+04, .8911676635742E+03},
 {   16890, .8124887207031E+04, -.1303866333008E+04, .8919382934570E+03},
 {   16891, .8124305175781E+04, -.1285592041016E+04, .8921954956055E+03},
 {   16892, .8124887207031E+04, -.1267317749023E+04, .8919382934570E+03},
 {   16893, .8126636230469E+04, -.1249112060547E+04, .8911676635742E+03},
 {   16894, .8129576171875E+04, -.1231049194336E+04, .8898704833984E+03},
 {   16895, .8131445800781E+04, -.1213216796875E+04, .8890603637695E+03},
 {   16896, .8134637695312E+04, -.1195723510742E+04, .8876669921875E+03},
 {   16897, .8141575195312E+04, -.1178714599609E+04, .8846058959961E+03},
 {   16898, .8145441894531E+04, -.1162388671875E+04, .8829304809570E+03},
 {   16899, .8153355957031E+04, -.1147034179688E+04, .8794533691406E+03},
 {   16900, .8129678710938E+04, -.1407779785156E+04, .8616801757812E+03},
 {   16901, .8125414550781E+04, -.1391554565430E+04, .8634978027344E+03},
 {   16902, .8122756347656E+04, -.1374679199219E+04, .8646111450195E+03},
 {   16903, .8119261230469E+04, -.1357337890625E+04, .8661159057617E+03},
 {   16904, .8117145019531E+04, -.1339663085938E+04, .8670166625977E+03},
 {   16905, .8114023925781E+04, -.1321758666992E+04, .8683813476562E+03},
 {   16906, .8112165527344E+04, -.1303710205078E+04, .8691937255859E+03},
 {   16907, .8111544433594E+04, -.1285592041016E+04, .8694658813477E+03},
 {   16908, .8112165527344E+04, -.1267473876953E+04, .8691936035156E+03},
 {   16909, .8114023925781E+04, -.1249425415039E+04, .8683813476562E+03},
 {   16910, .8117145019531E+04, -.1231520996094E+04, .8670166625977E+03},
 {   16911, .8119261230469E+04, -.1213846191406E+04, .8661159057617E+03},
 {   16912, .8122756347656E+04, -.1196504882812E+04, .8646111450195E+03},
 {   16913, .8125414550781E+04, -.1179629394531E+04, .8634978027344E+03},
 {   16914, .8129678710938E+04, -.1163404296875E+04, .8616800537109E+03},
 {   16915, .8133399414062E+04, -.1148084960938E+04, .8601256103516E+03},
 {   16916, .8381090820312E+04, -.1224000854492E+04, .1251129638672E+04},
 {   16917, .8388576171875E+04, -.1222944824219E+04, .1247778076172E+04},
 {   16918, .8396103515625E+04, -.1222311279297E+04, .1244409790039E+04},
 {   16919, .8403649414062E+04, -.1222092041016E+04, .1241030761719E+04},
 {   16920, .8364424804688E+04, -.1213174926758E+04, .1230760986328E+04},
 {   16921, .8373938476562E+04, -.1211067138672E+04, .1226503173828E+04},
 {   16922, .8383572265625E+04, -.1209809814453E+04, .1222188964844E+04},
 {   16923, .8393270507812E+04, -.1209392089844E+04, .1217848388672E+04},
 {   16924, .8349023437500E+04, -.1181395874023E+04, .1223740966797E+04},
 {   16925, .8361521484375E+04, -.1175852294922E+04, .1218146484375E+04},
 {   16926, .8374640625000E+04, -.1172444824219E+04, .1212273925781E+04},
 {   16927, .8388082031250E+04, -.1171291992188E+04, .1206256225586E+04},
 {   16928, .8340397460938E+04, -.1156813110352E+04, .1220645507812E+04},
 {   16929, .8354141601562E+04, -.1147704345703E+04, .1214492919922E+04},
 {   16930, .8369355468750E+04, -.1141684570312E+04, .1207682739258E+04},
 {   16931, .8385488281250E+04, -.1139541992188E+04, .1200460815430E+04},
 {   16932, .8332801757812E+04, -.1131877319336E+04, .1217088623047E+04},
 {   16933, .8347533203125E+04, -.1119379394531E+04, .1210493652344E+04},
 {   16934, .8364474609375E+04, -.1110877319336E+04, .1202910522461E+04},
 {   16935, .8382892578125E+04, -.1107791992188E+04, .1194665527344E+04},
 {   16936, .8327555664062E+04, -.1110380981445E+04, .1205522705078E+04},
 {   16937, .8341638671875E+04, -.1096298828125E+04, .1199218750000E+04},
 {   16938, .8358782226562E+04, -.1086964233398E+04, .1191543823242E+04},
 {   16939, .8377703125000E+04, -.1083661987305E+04, .1183073486328E+04},
 {   16940, .8323773437500E+04, -.1088675170898E+04, .1193301635742E+04},
 {   16941, .8336702148438E+04, -.1073104980469E+04, .1187513793945E+04},
 {   16942, .8353566406250E+04, -.1063020751953E+04, .1179963745117E+04},
 {   16943, .8372514648438E+04, -.1059532104492E+04, .1171481811523E+04},
 {   16944, .8305565429688E+04, -.1091055175781E+04, .1173652587891E+04},
 {   16945, .8319701171875E+04, -.1070978637695E+04, .1167315551758E+04},
 {   16946, .8339427734375E+04, -.1057772827148E+04, .1158475097656E+04},
 {   16947, .8362138671875E+04, -.1053182006836E+04, .1148298828125E+04},
 {   16948, .8287493164062E+04, -.1093902587891E+04, .1153885864258E+04},
 {   16949, .8302562500000E+04, -.1069166992188E+04, .1147138427734E+04},
 {   16950, .8325133789062E+04, -.1052635742188E+04, .1137034179688E+04},
 {   16951, .8351757812500E+04, -.1046832031250E+04, .1125116088867E+04},
 {   16952, .8277114257812E+04, -.1093902587891E+04, .1130702758789E+04},
 {   16953, .8292186523438E+04, -.1069166992188E+04, .1123955200195E+04},
 {   16954, .8314756835938E+04, -.1052635742188E+04, .1113850952148E+04},
 {   16955, .8341379882812E+04, -.1046832031250E+04, .1101933227539E+04},
 {   16956, .8266736328125E+04, -.1093902587891E+04, .1107519653320E+04},
 {   16957, .8281807617188E+04, -.1069166992188E+04, .1100772094727E+04},
 {   16958, .8304378906250E+04, -.1052635742188E+04, .1090667846680E+04},
 {   16959, .8331000976562E+04, -.1046832031250E+04, .1078749877930E+04},
 {   16960, .8256355468750E+04, -.1093902587891E+04, .1084336547852E+04},
 {   16961, .8271429687500E+04, -.1069166992188E+04, .1077588867188E+04},
 {   16962, .8294000000000E+04, -.1052635742188E+04, .1067484985352E+04},
 {   16963, .8320622070312E+04, -.1046832031250E+04, .1055566894531E+04},
 {   16964, .8245978515625E+04, -.1093902587891E+04, .1061153564453E+04},
 {   16965, .8261051757812E+04, -.1069166992188E+04, .1054405761719E+04},
 {   16966, .8283623046875E+04, -.1052635742188E+04, .1044301513672E+04},
 {   16967, .8310247070312E+04, -.1046832031250E+04, .1032383544922E+04},
 {   16968, .8235602539062E+04, -.1093902587891E+04, .1037970458984E+04},
 {   16969, .8250672851562E+04, -.1069166992188E+04, .1031222656250E+04},
 {   16970, .8273243164062E+04, -.1052635742188E+04, .1021119018555E+04},
 {   16971, .8299867187500E+04, -.1046832031250E+04, .1009200866699E+04},
 {   16972, .8225219726562E+04, -.1093902587891E+04, .1014787719727E+04},
 {   16973, .8240295898438E+04, -.1069166992188E+04, .1008039794922E+04},
 {   16974, .8262865234375E+04, -.1052635742188E+04, .9979362182617E+03},
 {   16975, .8289487304688E+04, -.1046832031250E+04, .9860181274414E+03},
 {   16976, .8214842773438E+04, -.1093902587891E+04, .9916044311523E+03},
 {   16977, .8229916992188E+04, -.1069166992188E+04, .9848572998047E+03},
 {   16978, .8252488281250E+04, -.1052635742188E+04, .9747525634766E+03},
 {   16979, .8279110351562E+04, -.1046832031250E+04, .9628347167969E+03},
 {   16980, .8204463867188E+04, -.1093902587891E+04, .9684215087891E+03},
 {   16981, .8219540039062E+04, -.1069166992188E+04, .9616737670898E+03},
 {   16982, .8242108398438E+04, -.1052635742188E+04, .9515697021484E+03},
 {   16983, .8268731445312E+04, -.1046832031250E+04, .9396518554688E+03},
 {   16984, .8198370117188E+04, -.1103439086914E+04, .9433214111328E+03},
 {   16985, .8211161132812E+04, -.1077979125977E+04, .9375949096680E+03},
 {   16986, .8232430664062E+04, -.1060610229492E+04, .9280736694336E+03},
 {   16987, .8258354492188E+04, -.1054452026367E+04, .9164686279297E+03},
 {   16988, .8187424316406E+04, -.1105337524414E+04, .9202577514648E+03},
 {   16989, .8201369140625E+04, -.1081887939453E+04, .9140432739258E+03},
 {   16990, .8222496093750E+04, -.1065914184570E+04, .9046279907227E+03},
 {   16991, .8247638671875E+04, -.1060230590820E+04, .8934229125977E+03},
 {   16992, .8178330566406E+04, -.1112282470703E+04, .8964380493164E+03},
 {   16993, .8193025390625E+04, -.1091669921875E+04, .8899015502930E+03},
 {   16994, .8213393554688E+04, -.1077635009766E+04, .8808440551758E+03},
 {   16995, .8237025390625E+04, -.1072613037109E+04, .8703350830078E+03},
 {   16996, .8164611816406E+04, -.1127257080078E+04, .8744874267578E+03},
 {   16997, .8181096191406E+04, -.1110474121094E+04, .8672141113281E+03},
 {   16998, .8202040039062E+04, -.1099032226562E+04, .8579739990234E+03},
 {   16999, .8225690429688E+04, -.1094901611328E+04, .8475399780273E+03},
 {   17000, .8149040039062E+04, -.1131149047852E+04, .8532868041992E+03},
 {   17001, .8167843261719E+04, -.1117112670898E+04, .8450646362305E+03},
 {   17002, .8189768066406E+04, -.1107502197266E+04, .8354783935547E+03},
 {   17003, .8213804687500E+04, -.1103982055664E+04, .8249686279297E+03},
 {   17004, .8408865234375E+04, -.1223730468750E+04, .1238696044922E+04},
 {   17005, .8413037109375E+04, -.1227533691406E+04, .1236828857422E+04},
 {   17006, .8416256835938E+04, -.1232318115234E+04, .1235386474609E+04},
 {   17007, .8418809570312E+04, -.1237565673828E+04, .1234244262695E+04},
 {   17008, .8400765625000E+04, -.1210404418945E+04, .1214493652344E+04},
 {   17009, .8407849609375E+04, -.1213263305664E+04, .1211321777344E+04},
 {   17010, .8414250976562E+04, -.1217651123047E+04, .1208456787109E+04},
 {   17011, .8419862304688E+04, -.1223183837891E+04, .1205944213867E+04},
 {   17012, .8401522460938E+04, -.1172444824219E+04, .1200239746094E+04},
 {   17013, .8414640625000E+04, -.1175852294922E+04, .1194366821289E+04},
 {   17014, .8427137695312E+04, -.1181395874023E+04, .1188772460938E+04},
 {   17015, .8438753906250E+04, -.1188896240234E+04, .1183572998047E+04},
 {   17016, .8401620117188E+04, -.1141684570312E+04, .1193239135742E+04},
 {   17017, .8416834960938E+04, -.1147704345703E+04, .1186429321289E+04},
 {   17018, .8430576171875E+04, -.1156813110352E+04, .1180276367188E+04},
 {   17019, .8442634765625E+04, -.1168259033203E+04, .1174877929688E+04},
 {   17020, .8401310546875E+04, -.1110877319336E+04, .1186420043945E+04},
 {   17021, .8418250000000E+04, -.1119379394531E+04, .1178837036133E+04},
 {   17022, .8432983398438E+04, -.1131877319336E+04, .1172241943359E+04},
 {   17023, .8445390625000E+04, -.1147107299805E+04, .1166687744141E+04},
 {   17024, .8396625976562E+04, -.1086964233398E+04, .1174603149414E+04},
 {   17025, .8413766601562E+04, -.1096298828125E+04, .1166928710938E+04},
 {   17026, .8427850585938E+04, -.1110381103516E+04, .1160624267578E+04},
 {   17027, .8438106445312E+04, -.1127659423828E+04, .1156033691406E+04},
 {   17028, .8391460937500E+04, -.1063020874023E+04, .1163000732422E+04},
 {   17029, .8408326171875E+04, -.1073104980469E+04, .1155450439453E+04},
 {   17030, .8421256835938E+04, -.1088675170898E+04, .1149662353516E+04},
 {   17031, .8428828125000E+04, -.1108019042969E+04, .1146271972656E+04},
 {   17032, .8384852539062E+04, -.1057772827148E+04, .1138139038086E+04},
 {   17033, .8404585937500E+04, -.1070978637695E+04, .1129315185547E+04},
 {   17034, .8418728515625E+04, -.1091055175781E+04, .1122994140625E+04},
 {   17035, .8425391601562E+04, -.1115336669922E+04, .1120020019531E+04},
 {   17036, .8378379882812E+04, -.1052635742188E+04, .1113197998047E+04},
 {   17037, .8400952148438E+04, -.1069166992188E+04, .1103094238281E+04},
 {   17038, .8416027343750E+04, -.1093902587891E+04, .1096346069336E+04},
 {   17039, .8421307617188E+04, -.1123070068359E+04, .1093981079102E+04},
 {   17040, .8368000000000E+04, -.1052635742188E+04, .1090015258789E+04},
 {   17041, .8390574218750E+04, -.1069166992188E+04, .1079911010742E+04},
 {   17042, .8405646484375E+04, -.1093902587891E+04, .1073163330078E+04},
 {   17043, .8410929687500E+04, -.1123070068359E+04, .1070798217773E+04},
 {   17044, .8357624023438E+04, -.1052635742188E+04, .1066832153320E+04},
 {   17045, .8380195312500E+04, -.1069166992188E+04, .1056727905273E+04},
 {   17046, .8395268554688E+04, -.1093902587891E+04, .1049980102539E+04},
 {   17047, .8400550781250E+04, -.1123070068359E+04, .1047615478516E+04},
 {   17048, .8347246093750E+04, -.1052635742188E+04, .1043649047852E+04},
 {   17049, .8369817382812E+04, -.1069166992188E+04, .1033544677734E+04},
 {   17050, .8384891601562E+04, -.1093902587891E+04, .1026796997070E+04},
 {   17051, .8390172851562E+04, -.1123070068359E+04, .1024432373047E+04},
 {   17052, .8336868164062E+04, -.1052635742188E+04, .1020465759277E+04},
 {   17053, .8359437500000E+04, -.1069166992188E+04, .1010361572266E+04},
 {   17054, .8374511718750E+04, -.1093902587891E+04, .1003614440918E+04},
 {   17055, .8379793945312E+04, -.1123070068359E+04, .1001249267578E+04},
 {   17056, .8326488281250E+04, -.1052635742188E+04, .9972831420898E+03},
 {   17057, .8349060546875E+04, -.1069166992188E+04, .9871785888672E+03},
 {   17058, .8364134765625E+04, -.1093902587891E+04, .9804309082031E+03},
 {   17059, .8369416992188E+04, -.1123070190430E+04, .9780658569336E+03},
 {   17060, .8316110351562E+04, -.1052635742188E+04, .9740998535156E+03},
 {   17061, .8338681640625E+04, -.1069166992188E+04, .9639954833984E+03},
 {   17062, .8353756835938E+04, -.1093902587891E+04, .9572481079102E+03},
 {   17063, .8359039062500E+04, -.1123070190430E+04, .9548834228516E+03},
 {   17064, .8305732421875E+04, -.1052635742188E+04, .9509167480469E+03},
 {   17065, .8328302734375E+04, -.1069166992188E+04, .9408124389648E+03},
 {   17066, .8343376953125E+04, -.1093902587891E+04, .9340648193359E+03},
 {   17067, .8348662109375E+04, -.1123070190430E+04, .9316998901367E+03},
 {   17068, .8295353515625E+04, -.1052635742188E+04, .9277338867188E+03},
 {   17069, .8317926757812E+04, -.1069166992188E+04, .9176291503906E+03},
 {   17070, .8332999023438E+04, -.1093902587891E+04, .9108818359375E+03},
 {   17071, .8338282226562E+04, -.1123070190430E+04, .9085170288086E+03},
 {   17072, .8282773437500E+04, -.1059886230469E+04, .9055366821289E+03},
 {   17073, .8303329101562E+04, -.1075332763672E+04, .8963354492188E+03},
 {   17074, .8316750000000E+04, -.1098337890625E+04, .8903267822266E+03},
 {   17075, .8320916015625E+04, -.1125250976562E+04, .8884621582031E+03},
 {   17076, .8272027343750E+04, -.1065171020508E+04, .8825534057617E+03},
 {   17077, .8293138671875E+04, -.1079242187500E+04, .8731447753906E+03},
 {   17078, .8308204101562E+04, -.1100299316406E+04, .8664310302734E+03},
 {   17079, .8315379882812E+04, -.1125113647461E+04, .8632324829102E+03},
 {   17080, .8260624023438E+04, -.1076878417969E+04, .8598401489258E+03},
 {   17081, .8281646484375E+04, -.1089057861328E+04, .8504903564453E+03},
 {   17082, .8297947265625E+04, -.1107411621094E+04, .8432418212891E+03},
 {   17083, .8308089843750E+04, -.1129340820312E+04, .8387316894531E+03},
 {   17084, .8249996093750E+04, -.1098284301758E+04, .8368159179688E+03},
 {   17085, .8272314453125E+04, -.1107976562500E+04, .8269692382812E+03},
 {   17086, .8291042968750E+04, -.1122736694336E+04, .8187062988281E+03},
 {   17087, .8305157226562E+04, -.1140800781250E+04, .8124784545898E+03},
 {   17088, .8239223632812E+04, -.1106695312500E+04, .8138530273438E+03},
 {   17089, .8263329101562E+04, -.1114508178711E+04, .8033129272461E+03},
 {   17090, .8285178710938E+04, -.1126610107422E+04, .7937590332031E+03},
 {   17091, .8304302734375E+04, -.1142051635742E+04, .7853966674805E+03},
 {   17092, .8420925781250E+04, -.1243257080078E+04, .1233297363281E+04},
 {   17093, .8422620117188E+04, -.1249115478516E+04, .1232539062500E+04},
 {   17094, .8423975585938E+04, -.1255079101562E+04, .1231931884766E+04},
 {   17095, .8425037109375E+04, -.1261113891602E+04, .1231457519531E+04},
 {   17096, .8425833984375E+04, -.1267197143555E+04, .1231099365234E+04},
 {   17097, .8426391601562E+04, -.1273312500000E+04, .1230850708008E+04},
 {   17098, .8426718750000E+04, -.1279447509766E+04, .1230703369141E+04},
 {   17099, .8426830078125E+04, -.1285592041016E+04, .1230653564453E+04},
 {   17100, .8426718750000E+04, -.1291736450195E+04, .1230703369141E+04},
 {   17101, .8426391601562E+04, -.1297871459961E+04, .1230850708008E+04},
 {   17102, .8425833984375E+04, -.1303986816406E+04, .1231099365234E+04},
 {   17103, .8425037109375E+04, -.1310070068359E+04, .1231457519531E+04},
 {   17104, .8423975585938E+04, -.1316104980469E+04, .1231931884766E+04},
 {   17105, .8422620117188E+04, -.1322068481445E+04, .1232539062500E+04},
 {   17106, .8420925781250E+04, -.1327927001953E+04, .1233297363281E+04},
 {   17107, .8418809570312E+04, -.1333618408203E+04, .1234244262695E+04},
 {   17108, .8424770507812E+04, -.1229694458008E+04, .1203746582031E+04},
 {   17109, .8428896484375E+04, -.1236829101562E+04, .1201899658203E+04},
 {   17110, .8432276367188E+04, -.1244418212891E+04, .1200386108398E+04},
 {   17111, .8434983398438E+04, -.1252326049805E+04, .1199175415039E+04},
 {   17112, .8437038085938E+04, -.1260466064453E+04, .1198255615234E+04},
 {   17113, .8438483398438E+04, -.1268760742188E+04, .1197607666016E+04},
 {   17114, .8439347656250E+04, -.1277152832031E+04, .1197221679688E+04},
 {   17115, .8439637695312E+04, -.1285592041016E+04, .1197092163086E+04},
 {   17116, .8439347656250E+04, -.1294031250000E+04, .1197221679688E+04},
 {   17117, .8438483398438E+04, -.1302423339844E+04, .1197607910156E+04},
 {   17118, .8437038085938E+04, -.1310718139648E+04, .1198255615234E+04},
 {   17119, .8434983398438E+04, -.1318858154297E+04, .1199175415039E+04},
 {   17120, .8432276367188E+04, -.1326765869141E+04, .1200386108398E+04},
 {   17121, .8428896484375E+04, -.1334355102539E+04, .1201899658203E+04},
 {   17122, .8424770507812E+04, -.1341489624023E+04, .1203746582031E+04},
 {   17123, .8419862304688E+04, -.1348000244141E+04, .1205944213867E+04},
 {   17124, .8448787109375E+04, -.1197651611328E+04, .1179081176758E+04},
 {   17125, .8457681640625E+04, -.1207776123047E+04, .1175098876953E+04},
 {   17126, .8465349609375E+04, -.1219044067383E+04, .1171666870117E+04},
 {   17127, .8471694335938E+04, -.1231257324219E+04, .1168826171875E+04},
 {   17128, .8476668945312E+04, -.1244211547852E+04, .1166599609375E+04},
 {   17129, .8480254882812E+04, -.1257704833984E+04, .1164994018555E+04},
 {   17130, .8482399414062E+04, -.1271561157227E+04, .1164034667969E+04},
 {   17131, .8483131835938E+04, -.1285592041016E+04, .1163706054688E+04},
 {   17132, .8482399414062E+04, -.1299622924805E+04, .1164034667969E+04},
 {   17133, .8480254882812E+04, -.1313479370117E+04, .1164994018555E+04},
 {   17134, .8476668945312E+04, -.1326972656250E+04, .1166599609375E+04},
 {   17135, .8471694335938E+04, -.1339926757812E+04, .1168826171875E+04},
 {   17136, .8465349609375E+04, -.1352140136719E+04, .1171666870117E+04},
 {   17137, .8457681640625E+04, -.1363407958984E+04, .1175098876953E+04},
 {   17138, .8448787109375E+04, -.1373532592773E+04, .1179081176758E+04},
 {   17139, .8438753906250E+04, -.1382287841797E+04, .1183572998047E+04},
 {   17140, .8452345703125E+04, -.1180315307617E+04, .1170531250000E+04},
 {   17141, .8460615234375E+04, -.1193528442383E+04, .1166829101562E+04},
 {   17142, .8467516601562E+04, -.1207622070312E+04, .1163738769531E+04},
 {   17143, .8473091796875E+04, -.1222409179688E+04, .1161243896484E+04},
 {   17144, .8477378906250E+04, -.1237733398438E+04, .1159325683594E+04},
 {   17145, .8480425781250E+04, -.1253455810547E+04, .1157960571289E+04},
 {   17146, .8482241210938E+04, -.1269454467773E+04, .1157147460938E+04},
 {   17147, .8482855468750E+04, -.1285592041016E+04, .1156872680664E+04},
 {   17148, .8482241210938E+04, -.1301729614258E+04, .1157147705078E+04},
 {   17149, .8480425781250E+04, -.1317728271484E+04, .1157960571289E+04},
 {   17150, .8477378906250E+04, -.1333450683594E+04, .1159325683594E+04},
 {   17151, .8473091796875E+04, -.1348774902344E+04, .1161243896484E+04},
 {   17152, .8467516601562E+04, -.1363562011719E+04, .1163738769531E+04},
 {   17153, .8460615234375E+04, -.1377655639648E+04, .1166829101562E+04},
 {   17154, .8452345703125E+04, -.1390868774414E+04, .1170531250000E+04},
 {   17155, .8442634765625E+04, -.1402925048828E+04, .1174877929688E+04},
 {   17156, .8454732421875E+04, -.1162330078125E+04, .1162505615234E+04},
 {   17157, .8462487304688E+04, -.1178586791992E+04, .1159034423828E+04},
 {   17158, .8468841796875E+04, -.1195556884766E+04, .1156188720703E+04},
 {   17159, .8473901367188E+04, -.1213042602539E+04, .1153924560547E+04},
 {   17160, .8477742187500E+04, -.1230896118164E+04, .1152204589844E+04},
 {   17161, .8480441406250E+04, -.1248999389648E+04, .1150995849609E+04},
 {   17162, .8482045898438E+04, -.1267258178711E+04, .1150278564453E+04},
 {   17163, .8482579101562E+04, -.1285592041016E+04, .1150039306641E+04},
 {   17164, .8482045898438E+04, -.1303925903320E+04, .1150278564453E+04},
 {   17165, .8480441406250E+04, -.1322184692383E+04, .1150995849609E+04},
 {   17166, .8477742187500E+04, -.1340287963867E+04, .1152204589844E+04},
 {   17167, .8473901367188E+04, -.1358141479492E+04, .1153924560547E+04},
 {   17168, .8468841796875E+04, -.1375627197266E+04, .1156188720703E+04},
 {   17169, .8462487304688E+04, -.1392597290039E+04, .1159034423828E+04},
 {   17170, .8454732421875E+04, -.1408854003906E+04, .1162505615234E+04},
 {   17171, .8445390625000E+04, -.1424076782227E+04, .1166687744141E+04},
 {   17172, .8450211914062E+04, -.1144227661133E+04, .1150614257812E+04},
 {   17173, .8460188476562E+04, -.1162440673828E+04, .1146147583008E+04},
 {   17174, .8468289062500E+04, -.1181746582031E+04, .1142522705078E+04},
 {   17175, .8474679687500E+04, -.1201803344727E+04, .1139661254883E+04},
 {   17176, .8479489257812E+04, -.1222375732422E+04, .1137508178711E+04},
 {   17177, .8482857421875E+04, -.1243280151367E+04, .1136000488281E+04},
 {   17178, .8484837890625E+04, -.1264388793945E+04, .1135113891602E+04},
 {   17179, .8485504882812E+04, -.1285592041016E+04, .1134815185547E+04},
 {   17180, .8484837890625E+04, -.1306795288086E+04, .1135113891602E+04},
 {   17181, .8482857421875E+04, -.1327903930664E+04, .1136000488281E+04},
 {   17182, .8479489257812E+04, -.1348808349609E+04, .1137508178711E+04},
 {   17183, .8474679687500E+04, -.1369380737305E+04, .1139661254883E+04},
 {   17184, .8468289062500E+04, -.1389437500000E+04, .1142522705078E+04},
 {   17185, .8460188476562E+04, -.1408743408203E+04, .1146147583008E+04},
 {   17186, .8450211914062E+04, -.1426956420898E+04, .1150614257812E+04},
 {   17187, .8438106445312E+04, -.1443524536133E+04, .1156033691406E+04},
 {   17188, .8444040039062E+04, -.1125696044922E+04, .1139463134766E+04},
 {   17189, .8456584960938E+04, -.1145728637695E+04, .1133847534180E+04},
 {   17190, .8466754882812E+04, -.1167315429688E+04, .1129293701172E+04},
 {   17191, .8474787109375E+04, -.1189959838867E+04, .1125698242188E+04},
 {   17192, .8480834960938E+04, -.1213332885742E+04, .1122991210938E+04},
 {   17193, .8485083984375E+04, -.1237176391602E+04, .1121088867188E+04},
 {   17194, .8487581054688E+04, -.1261315795898E+04, .1119970703125E+04},
 {   17195, .8488428710938E+04, -.1285592041016E+04, .1119591430664E+04},
 {   17196, .8487581054688E+04, -.1309868286133E+04, .1119970703125E+04},
 {   17197, .8485083984375E+04, -.1334007690430E+04, .1121088867188E+04},
 {   17198, .8480834960938E+04, -.1357851318359E+04, .1122991210938E+04},
 {   17199, .8474787109375E+04, -.1381224243164E+04, .1125698486328E+04},
 {   17200, .8466754882812E+04, -.1403868652344E+04, .1129293701172E+04},
 {   17201, .8456584960938E+04, -.1425455566406E+04, .1133847534180E+04},
 {   17202, .8444040039062E+04, -.1445488037109E+04, .1139463134766E+04},
 {   17203, .8428828125000E+04, -.1463165039062E+04, .1146272216797E+04},
 {   17204, .8438717773438E+04, -.1133068603516E+04, .1114055053711E+04},
 {   17205, .8449761718750E+04, -.1152589477539E+04, .1109110839844E+04},
 {   17206, .8458755859375E+04, -.1173336547852E+04, .1105084472656E+04},
 {   17207, .8465876953125E+04, -.1194945922852E+04, .1101896728516E+04},
 {   17208, .8471282226562E+04, -.1217146240234E+04, .1099477905273E+04},
 {   17209, .8475060546875E+04, -.1239754882812E+04, .1097786376953E+04},
 {   17210, .8477307617188E+04, -.1262612792969E+04, .1096779296875E+04},
 {   17211, .8478066406250E+04, -.1285592529297E+04, .1096440429688E+04},
 {   17212, .8477307617188E+04, -.1308571411133E+04, .1096779296875E+04},
 {   17213, .8475060546875E+04, -.1331429199219E+04, .1097786376953E+04},
 {   17214, .8471282226562E+04, -.1354037963867E+04, .1099477905273E+04},
 {   17215, .8465876953125E+04, -.1376238281250E+04, .1101896606445E+04},
 {   17216, .8458755859375E+04, -.1397847656250E+04, .1105084472656E+04},
 {   17217, .8449761718750E+04, -.1418594726562E+04, .1109110839844E+04},
 {   17218, .8438720703125E+04, -.1438115600586E+04, .1114054809570E+04},
 {   17219, .8425391601562E+04, -.1455847412109E+04, .1120020019531E+04},
 {   17220, .8432952148438E+04, -.1140602661133E+04, .1088768066406E+04},
 {   17221, .8442647460938E+04, -.1159507324219E+04, .1084428222656E+04},
 {   17222, .8450563476562E+04, -.1179380859375E+04, .1080884521484E+04},
 {   17223, .8456852539062E+04, -.1199940429688E+04, .1078069213867E+04},
 {   17224, .8461641601562E+04, -.1220979736328E+04, .1075925048828E+04},
 {   17225, .8465008789062E+04, -.1242347534180E+04, .1074417724609E+04},
 {   17226, .8467006835938E+04, -.1263920532227E+04, .1073523315430E+04},
 {   17227, .8467671875000E+04, -.1285592041016E+04, .1073225341797E+04},
 {   17228, .8467006835938E+04, -.1307263671875E+04, .1073523315430E+04},
 {   17229, .8465008789062E+04, -.1328836547852E+04, .1074417724609E+04},
 {   17230, .8461641601562E+04, -.1350204345703E+04, .1075925048828E+04},
 {   17231, .8456851562500E+04, -.1371243774414E+04, .1078069091797E+04},
 {   17232, .8450563476562E+04, -.1391803222656E+04, .1080884521484E+04},
 {   17233, .8442647460938E+04, -.1411676879883E+04, .1084428222656E+04},
 {   17234, .8432952148438E+04, -.1430581542969E+04, .1088767822266E+04},
 {   17235, .8421307617188E+04, -.1448114135742E+04, .1093981079102E+04},
 {   17236, .8422576171875E+04, -.1140602661133E+04, .1065585083008E+04},
 {   17237, .8432268554688E+04, -.1159507324219E+04, .1061245239258E+04},
 {   17238, .8440187500000E+04, -.1179380859375E+04, .1057701904297E+04},
 {   17239, .8446473632812E+04, -.1199940429688E+04, .1054886108398E+04},
 {   17240, .8451265625000E+04, -.1220979736328E+04, .1052741699219E+04},
 {   17241, .8454630859375E+04, -.1242347534180E+04, .1051234741211E+04},
 {   17242, .8456629882812E+04, -.1263920532227E+04, .1050340332031E+04},
 {   17243, .8457294921875E+04, -.1285592041016E+04, .1050042236328E+04},
 {   17244, .8456629882812E+04, -.1307263671875E+04, .1050340332031E+04},
 {   17245, .8454630859375E+04, -.1328836547852E+04, .1051234741211E+04},
 {   17246, .8451265625000E+04, -.1350204467773E+04, .1052741699219E+04},
 {   17247, .8446473632812E+04, -.1371243774414E+04, .1054886108398E+04},
 {   17248, .8440187500000E+04, -.1391803344727E+04, .1057701904297E+04},
 {   17249, .8432266601562E+04, -.1411676879883E+04, .1061245605469E+04},
 {   17250, .8422576171875E+04, -.1430581542969E+04, .1065585083008E+04},
 {   17251, .8410929687500E+04, -.1448114013672E+04, .1070798217773E+04},
 {   17252, .8412195312500E+04, -.1140602661133E+04, .1042401977539E+04},
 {   17253, .8421889648438E+04, -.1159507324219E+04, .1038062744141E+04},
 {   17254, .8429807617188E+04, -.1179380859375E+04, .1034518554688E+04},
 {   17255, .8436095703125E+04, -.1199940429688E+04, .1031703002930E+04},
 {   17256, .8440885742188E+04, -.1220979736328E+04, .1029559082031E+04},
 {   17257, .8444250976562E+04, -.1242347534180E+04, .1028051757812E+04},
 {   17258, .8446253906250E+04, -.1263920532227E+04, .1027156494141E+04},
 {   17259, .8446916992188E+04, -.1285592041016E+04, .1026859497070E+04},
 {   17260, .8446253906250E+04, -.1307263671875E+04, .1027156494141E+04},
 {   17261, .8444250976562E+04, -.1328836547852E+04, .1028051757812E+04},
 {   17262, .8440885742188E+04, -.1350204467773E+04, .1029559082031E+04},
 {   17263, .8436095703125E+04, -.1371243774414E+04, .1031703002930E+04},
 {   17264, .8429807617188E+04, -.1391803344727E+04, .1034518798828E+04},
 {   17265, .8421889648438E+04, -.1411676879883E+04, .1038062744141E+04},
 {   17266, .8412195312500E+04, -.1430581542969E+04, .1042402221680E+04},
 {   17267, .8400550781250E+04, -.1448114013672E+04, .1047615478516E+04},
 {   17268, .8401818359375E+04, -.1140602661133E+04, .1019218994141E+04},
 {   17269, .8411513671875E+04, -.1159507324219E+04, .1014879394531E+04},
 {   17270, .8419429687500E+04, -.1179380859375E+04, .1011335815430E+04},
 {   17271, .8425717773438E+04, -.1199940429688E+04, .1008519714355E+04},
 {   17272, .8430508789062E+04, -.1220979736328E+04, .1006375915527E+04},
 {   17273, .8433873046875E+04, -.1242347534180E+04, .1004868774414E+04},
 {   17274, .8435874023438E+04, -.1263920532227E+04, .1003973754883E+04},
 {   17275, .8436538085938E+04, -.1285592041016E+04, .1003676269531E+04},
 {   17276, .8435874023438E+04, -.1307263671875E+04, .1003973754883E+04},
 {   17277, .8433873046875E+04, -.1328836547852E+04, .1004868774414E+04},
 {   17278, .8430508789062E+04, -.1350204467773E+04, .1006375915527E+04},
 {   17279, .8425717773438E+04, -.1371243774414E+04, .1008519714355E+04},
 {   17280, .8419429687500E+04, -.1391803344727E+04, .1011336059570E+04},
 {   17281, .8411513671875E+04, -.1411676879883E+04, .1014879394531E+04},
 {   17282, .8401818359375E+04, -.1430581542969E+04, .1019218994141E+04},
 {   17283, .8390172851562E+04, -.1448114013672E+04, .1024432373047E+04},
 {   17284, .8391441406250E+04, -.1140602661133E+04, .9960359497070E+03},
 {   17285, .8401133789062E+04, -.1159507324219E+04, .9916964111328E+03},
 {   17286, .8409049804688E+04, -.1179380859375E+04, .9881527099609E+03},
 {   17287, .8415340820312E+04, -.1199940429688E+04, .9853368530273E+03},
 {   17288, .8420129882812E+04, -.1220979736328E+04, .9831928710938E+03},
 {   17289, .8423498046875E+04, -.1242347534180E+04, .9816857299805E+03},
 {   17290, .8425496093750E+04, -.1263920532227E+04, .9807906494141E+03},
 {   17291, .8426162109375E+04, -.1285592041016E+04, .9804927978516E+03},
 {   17292, .8425496093750E+04, -.1307263671875E+04, .9807906494141E+03},
 {   17293, .8423498046875E+04, -.1328836547852E+04, .9816857299805E+03},
 {   17294, .8420129882812E+04, -.1350204467773E+04, .9831928710938E+03},
 {   17295, .8415340820312E+04, -.1371243774414E+04, .9853368530273E+03},
 {   17296, .8409049804688E+04, -.1391803344727E+04, .9881527099609E+03},
 {   17297, .8401133789062E+04, -.1411676879883E+04, .9916964111328E+03},
 {   17298, .8391441406250E+04, -.1430581542969E+04, .9960359497070E+03},
 {   17299, .8379793945312E+04, -.1448114013672E+04, .1001249267578E+04},
 {   17300, .8381061523438E+04, -.1140602661133E+04, .9728529052734E+03},
 {   17301, .8390755859375E+04, -.1159507324219E+04, .9685135498047E+03},
 {   17302, .8398672851562E+04, -.1179380859375E+04, .9649693603516E+03},
 {   17303, .8404961914062E+04, -.1199940429688E+04, .9621536865234E+03},
 {   17304, .8409751953125E+04, -.1220979736328E+04, .9600096435547E+03},
 {   17305, .8413118164062E+04, -.1242347534180E+04, .9585025634766E+03},
 {   17306, .8415116210938E+04, -.1263920532227E+04, .9576077880859E+03},
 {   17307, .8415781250000E+04, -.1285592041016E+04, .9573101806641E+03},
 {   17308, .8415116210938E+04, -.1307263671875E+04, .9576077880859E+03},
 {   17309, .8413118164062E+04, -.1328836547852E+04, .9585025634766E+03},
 {   17310, .8409751953125E+04, -.1350204467773E+04, .9600096435547E+03},
 {   17311, .8404961914062E+04, -.1371243774414E+04, .9621536865234E+03},
 {   17312, .8398672851562E+04, -.1391803344727E+04, .9649693603516E+03},
 {   17313, .8390755859375E+04, -.1411676879883E+04, .9685135498047E+03},
 {   17314, .8381061523438E+04, -.1430581542969E+04, .9728526611328E+03},
 {   17315, .8369416992188E+04, -.1448114013672E+04, .9780658569336E+03},
 {   17316, .8370685546875E+04, -.1140602661133E+04, .9496695556641E+03},
 {   17317, .8380377929688E+04, -.1159507324219E+04, .9453300781250E+03},
 {   17318, .8388293945312E+04, -.1179380859375E+04, .9417861328125E+03},
 {   17319, .8394584960938E+04, -.1199940429688E+04, .9389705200195E+03},
 {   17320, .8399374023438E+04, -.1220979736328E+04, .9368265991211E+03},
 {   17321, .8402740234375E+04, -.1242347534180E+04, .9353197021484E+03},
 {   17322, .8404738281250E+04, -.1263920532227E+04, .9344248046875E+03},
 {   17323, .8405405273438E+04, -.1285592041016E+04, .9341266479492E+03},
 {   17324, .8404738281250E+04, -.1307263671875E+04, .9344248046875E+03},
 {   17325, .8402740234375E+04, -.1328836547852E+04, .9353197021484E+03},
 {   17326, .8399374023438E+04, -.1350204345703E+04, .9368265991211E+03},
 {   17327, .8394584960938E+04, -.1371243774414E+04, .9389705200195E+03},
 {   17328, .8388293945312E+04, -.1391803222656E+04, .9417862548828E+03},
 {   17329, .8380377929688E+04, -.1411676879883E+04, .9453302001953E+03},
 {   17330, .8370685546875E+04, -.1430581542969E+04, .9496695556641E+03},
 {   17331, .8359039062500E+04, -.1448114135742E+04, .9548834228516E+03},
 {   17332, .8360306640625E+04, -.1140602661133E+04, .9264866943359E+03},
 {   17333, .8369998046875E+04, -.1159507324219E+04, .9221472167969E+03},
 {   17334, .8377915039062E+04, -.1179380859375E+04, .9186035156250E+03},
 {   17335, .8384205078125E+04, -.1199940429688E+04, .9157876586914E+03},
 {   17336, .8388995117188E+04, -.1220979736328E+04, .9136434936523E+03},
 {   17337, .8392362304688E+04, -.1242347534180E+04, .9121364135742E+03},
 {   17338, .8394361328125E+04, -.1263920532227E+04, .9112415771484E+03},
 {   17339, .8395026367188E+04, -.1285592041016E+04, .9109437255859E+03},
 {   17340, .8394361328125E+04, -.1307263671875E+04, .9112415771484E+03},
 {   17341, .8392362304688E+04, -.1328836547852E+04, .9121364135742E+03},
 {   17342, .8388995117188E+04, -.1350204467773E+04, .9136434936523E+03},
 {   17343, .8384205078125E+04, -.1371243774414E+04, .9157876586914E+03},
 {   17344, .8377915039062E+04, -.1391803344727E+04, .9186036376953E+03},
 {   17345, .8369998046875E+04, -.1411676879883E+04, .9221472167969E+03},
 {   17346, .8360306640625E+04, -.1430581542969E+04, .9264866943359E+03},
 {   17347, .8348661132812E+04, -.1448114013672E+04, .9317001953125E+03},
 {   17348, .8349926757812E+04, -.1140602661133E+04, .9033034057617E+03},
 {   17349, .8359621093750E+04, -.1159507324219E+04, .8989641723633E+03},
 {   17350, .8367539062500E+04, -.1179380859375E+04, .8954201049805E+03},
 {   17351, .8373826171875E+04, -.1199940429688E+04, .8926044921875E+03},
 {   17352, .8378617187500E+04, -.1220979736328E+04, .8904603271484E+03},
 {   17353, .8381982421875E+04, -.1242347534180E+04, .8889536743164E+03},
 {   17354, .8383982421875E+04, -.1263920532227E+04, .8880585937500E+03},
 {   17355, .8384647460938E+04, -.1285592041016E+04, .8877607421875E+03},
 {   17356, .8383982421875E+04, -.1307263671875E+04, .8880585937500E+03},
 {   17357, .8381982421875E+04, -.1328836547852E+04, .8889536743164E+03},
 {   17358, .8378617187500E+04, -.1350204467773E+04, .8904603271484E+03},
 {   17359, .8373826171875E+04, -.1371243774414E+04, .8926044921875E+03},
 {   17360, .8367539062500E+04, -.1391803222656E+04, .8954202270508E+03},
 {   17361, .8359621093750E+04, -.1411676879883E+04, .8989642944336E+03},
 {   17362, .8349926757812E+04, -.1430581542969E+04, .9033034057617E+03},
 {   17363, .8338282226562E+04, -.1448114013672E+04, .9085170288086E+03},
 {   17364, .8334202148438E+04, -.1141575317383E+04, .8825139770508E+03},
 {   17365, .8345343750000E+04, -.1159725830078E+04, .8775264892578E+03},
 {   17366, .8354493164062E+04, -.1179166992188E+04, .8734310302734E+03},
 {   17367, .8361751953125E+04, -.1199542968750E+04, .8701816406250E+03},
 {   17368, .8367303710938E+04, -.1220552978516E+04, .8676963500977E+03},
 {   17369, .8371176757812E+04, -.1242010009766E+04, .8659615478516E+03},
 {   17370, .8373489257812E+04, -.1263735839844E+04, .8649266357422E+03},
 {   17371, .8374270507812E+04, -.1285592163086E+04, .8645778198242E+03},
 {   17372, .8373489257812E+04, -.1307448364258E+04, .8649266357422E+03},
 {   17373, .8371176757812E+04, -.1329174194336E+04, .8659615478516E+03},
 {   17374, .8367303710938E+04, -.1350631225586E+04, .8676963500977E+03},
 {   17375, .8361751953125E+04, -.1371641235352E+04, .8701816406250E+03},
 {   17376, .8354493164062E+04, -.1392017089844E+04, .8734310302734E+03},
 {   17377, .8345343750000E+04, -.1411458374023E+04, .8775264892578E+03},
 {   17378, .8334202148438E+04, -.1429608886719E+04, .8825139770508E+03},
 {   17379, .8320916015625E+04, -.1445933105469E+04, .8884622802734E+03},
 {   17380, .8328499023438E+04, -.1141583862305E+04, .8573858032227E+03},
 {   17381, .8339523437500E+04, -.1159792968750E+04, .8524726562500E+03},
 {   17382, .8348583984375E+04, -.1179252075195E+04, .8484345703125E+03},
 {   17383, .8355785156250E+04, -.1199618041992E+04, .8452248535156E+03},
 {   17384, .8361292968750E+04, -.1220609863281E+04, .8427706909180E+03},
 {   17385, .8365143554688E+04, -.1242044433594E+04, .8410541992188E+03},
 {   17386, .8367440429688E+04, -.1263751586914E+04, .8400306396484E+03},
 {   17387, .8368213867188E+04, -.1285592163086E+04, .8396860961914E+03},
 {   17388, .8367440429688E+04, -.1307432617188E+04, .8400306396484E+03},
 {   17389, .8365143554688E+04, -.1329139770508E+04, .8410541992188E+03},
 {   17390, .8361292968750E+04, -.1350574340820E+04, .8427706909180E+03},
 {   17391, .8355785156250E+04, -.1371566162109E+04, .8452248535156E+03},
 {   17392, .8348583984375E+04, -.1391932128906E+04, .8484346923828E+03},
 {   17393, .8339523437500E+04, -.1411391235352E+04, .8524726562500E+03},
 {   17394, .8328499023438E+04, -.1429600219727E+04, .8573858032227E+03},
 {   17395, .8315379882812E+04, -.1446070434570E+04, .8632326049805E+03},
 {   17396, .8320719726562E+04, -.1145484985352E+04, .8331144409180E+03},
 {   17397, .8331375000000E+04, -.1163240966797E+04, .8283756103516E+03},
 {   17398, .8340153320312E+04, -.1182181396484E+04, .8244719848633E+03},
 {   17399, .8347153320312E+04, -.1201984863281E+04, .8213587036133E+03},
 {   17400, .8352513671875E+04, -.1222395751953E+04, .8189759521484E+03},
 {   17401, .8356270507812E+04, -.1243237060547E+04, .8173043212891E+03},
 {   17402, .8358512695312E+04, -.1264348266602E+04, .8163077392578E+03},
 {   17403, .8359265625000E+04, -.1285592163086E+04, .8159727783203E+03},
 {   17404, .8358512695312E+04, -.1306835937500E+04, .8163077392578E+03},
 {   17405, .8356270507812E+04, -.1327947143555E+04, .8173043212891E+03},
 {   17406, .8352513671875E+04, -.1348788452148E+04, .8189760742188E+03},
 {   17407, .8347153320312E+04, -.1369199218750E+04, .8213587036133E+03},
 {   17408, .8340153320312E+04, -.1389002685547E+04, .8244721069336E+03},
 {   17409, .8331375000000E+04, -.1407943115234E+04, .8283756103516E+03},
 {   17410, .8320719726562E+04, -.1425699096680E+04, .8331144409180E+03},
 {   17411, .8308089843750E+04, -.1441843261719E+04, .8387316894531E+03},
 {   17412, .8318204101562E+04, -.1155839965820E+04, .8067224731445E+03},
 {   17413, .8325800781250E+04, -.1172303222656E+04, .8033942260742E+03},
 {   17414, .8334955078125E+04, -.1189842163086E+04, .7993547973633E+03},
 {   17415, .8338815429688E+04, -.1208170532227E+04, .7976743164062E+03},
 {   17416, .8342124023438E+04, -.1227066040039E+04, .7962290039062E+03},
 {   17417, .8346089843750E+04, -.1246363159180E+04, .7944794921875E+03},
 {   17418, .8348454101562E+04, -.1265915161133E+04, .7934363403320E+03},
 {   17419, .8349246093750E+04, -.1285592041016E+04, .7930865478516E+03},
 {   17420, .8348454101562E+04, -.1305269165039E+04, .7934363403320E+03},
 {   17421, .8346089843750E+04, -.1324821044922E+04, .7944796142578E+03},
 {   17422, .8342124023438E+04, -.1344118164062E+04, .7962290039062E+03},
 {   17423, .8342291992188E+04, -.1363013671875E+04, .7961175537109E+03},
 {   17424, .8334955078125E+04, -.1381342041016E+04, .7993547973633E+03},
 {   17425, .8325800781250E+04, -.1398880859375E+04, .8033942260742E+03},
 {   17426, .8318204101562E+04, -.1415344116211E+04, .8067225952148E+03},
 {   17427, .8305157226562E+04, -.1430383300781E+04, .8124784545898E+03},
 {   17428, .8318117187500E+04, -.1157016967773E+04, .7793559570312E+03},
 {   17429, .8326455078125E+04, -.1173331420898E+04, .7757457275391E+03},
 {   17430, .8336281250000E+04, -.1190700195312E+04, .7714492187500E+03},
 {   17431, .8338416015625E+04, -.1208850097656E+04, .7705768432617E+03},
 {   17432, .8341037109375E+04, -.1227572998047E+04, .7694661865234E+03},
 {   17433, .8341881835938E+04, -.1246699462891E+04, .7691333618164E+03},
 {   17434, .8344460937500E+04, -.1266083129883E+04, .7680050659180E+03},
 {   17435, .8345323242188E+04, -.1285592163086E+04, .7676281738281E+03},
 {   17436, .8344460937500E+04, -.1305101074219E+04, .7680050659180E+03},
 {   17437, .8341881835938E+04, -.1324484741211E+04, .7691333618164E+03},
 {   17438, .8341037109375E+04, -.1343611328125E+04, .7694661865234E+03},
 {   17439, .8338416015625E+04, -.1362334106445E+04, .7705768432617E+03},
 {   17440, .8336281250000E+04, -.1380484008789E+04, .7714492187500E+03},
 {   17441, .8326455078125E+04, -.1397852783203E+04, .7757458496094E+03},
 {   17442, .8318117187500E+04, -.1414167236328E+04, .7793559570312E+03},
 {   17443, .8304302734375E+04, -.1429132568359E+04, .7853966674805E+03},
 {   17444, .8417601562500E+04, -.1336291137695E+04, .1234785278320E+04},
 {   17445, .8416250000000E+04, -.1338880126953E+04, .1235390136719E+04},
 {   17446, .8414731445312E+04, -.1341355957031E+04, .1236069213867E+04},
 {   17447, .8413016601562E+04, -.1343672485352E+04, .1236837646484E+04},
 {   17448, .8411061523438E+04, -.1345748168945E+04, .1237711791992E+04},
 {   17449, .8408838867188E+04, -.1347469726562E+04, .1238707397461E+04},
 {   17450, .8406341796875E+04, -.1348653564453E+04, .1239826171875E+04},
 {   17451, .8417153320312E+04, -.1350890502930E+04, .1207156250000E+04},
 {   17452, .8414248046875E+04, -.1353535522461E+04, .1208457885742E+04},
 {   17453, .8411138671875E+04, -.1355894653320E+04, .1209848754883E+04},
 {   17454, .8407844726562E+04, -.1357923461914E+04, .1211324707031E+04},
 {   17455, .8404374023438E+04, -.1359570922852E+04, .1212876953125E+04},
 {   17456, .8400760742188E+04, -.1360780395508E+04, .1214494873047E+04},
 {   17457, .8397043945312E+04, -.1361529052734E+04, .1216159179688E+04},
 {   17458, .8433070312500E+04, -.1386269775391E+04, .1186117187500E+04},
 {   17459, .8427135742188E+04, -.1389788574219E+04, .1188772583008E+04},
 {   17460, .8420983398438E+04, -.1392817382812E+04, .1191527709961E+04},
 {   17461, .8414638671875E+04, -.1395332031250E+04, .1194367675781E+04},
 {   17462, .8408141601562E+04, -.1397311767578E+04, .1197277099609E+04},
 {   17463, .8401521484375E+04, -.1398739257812E+04, .1200239990234E+04},
 {   17464, .8394822265625E+04, -.1399602050781E+04, .1203239257812E+04},
 {   17465, .8436816406250E+04, -.1408899780273E+04, .1177482910156E+04},
 {   17466, .8430571289062E+04, -.1414377441406E+04, .1180278442383E+04},
 {   17467, .8423897460938E+04, -.1419267700195E+04, .1183265625000E+04},
 {   17468, .8416824218750E+04, -.1423486938477E+04, .1186432617188E+04},
 {   17469, .8409378906250E+04, -.1426941040039E+04, .1189766113281E+04},
 {   17470, .8401611328125E+04, -.1429502441406E+04, .1193242919922E+04},
 {   17471, .8393610351562E+04, -.1431091186523E+04, .1196823730469E+04},
 {   17472, .8439457031250E+04, -.1431969726562E+04, .1169343261719E+04},
 {   17473, .8432970703125E+04, -.1439320190430E+04, .1172247192383E+04},
 {   17474, .8425890625000E+04, -.1445981567383E+04, .1175416625977E+04},
 {   17475, .8418229492188E+04, -.1451819580078E+04, .1178846923828E+04},
 {   17476, .8410017578125E+04, -.1456673950195E+04, .1182523681641E+04},
 {   17477, .8401292968750E+04, -.1460312744141E+04, .1186427734375E+04},
 {   17478, .8392194335938E+04, -.1462592285156E+04, .1190500732422E+04},
 {   17479, .8433480468750E+04, -.1452462036133E+04, .1158104248047E+04},
 {   17480, .8427842773438E+04, -.1460810424805E+04, .1160627685547E+04},
 {   17481, .8421238281250E+04, -.1468351562500E+04, .1163584106445E+04},
 {   17482, .8413755859375E+04, -.1474893066406E+04, .1166934448242E+04},
 {   17483, .8405501953125E+04, -.1480249877930E+04, .1170628540039E+04},
 {   17484, .8396615234375E+04, -.1484222778320E+04, .1174607910156E+04},
 {   17485, .8387275390625E+04, -.1486679199219E+04, .1178787841797E+04},
 {   17486, .8425777343750E+04, -.1473183837891E+04, .1147638061523E+04},
 {   17487, .8421255859375E+04, -.1482510131836E+04, .1149662719727E+04},
 {   17488, .8415381835938E+04, -.1490885986328E+04, .1152291503906E+04},
 {   17489, .8408325195312E+04, -.1498079833984E+04, .1155450683594E+04},
 {   17490, .8400277343750E+04, -.1503892456055E+04, .1159053833008E+04},
 {   17491, .8391460937500E+04, -.1508163208008E+04, .1163000732422E+04},
 {   17492, .8382119140625E+04, -.1510773681641E+04, .1167182006836E+04},
 {   17493, .8423077148438E+04, -.1468321411133E+04, .1121052368164E+04},
 {   17494, .8418726562500E+04, -.1480131713867E+04, .1122994628906E+04},
 {   17495, .8412492187500E+04, -.1490880249023E+04, .1125780883789E+04},
 {   17496, .8404584960938E+04, -.1500207275391E+04, .1129315795898E+04},
 {   17497, .8395269531250E+04, -.1507801635742E+04, .1133481689453E+04},
 {   17498, .8384853515625E+04, -.1513411132812E+04, .1138138916016E+04},
 {   17499, .8373685546875E+04, -.1516849243164E+04, .1143133422852E+04},
 {   17500, .8419976562500E+04, -.1462985107422E+04, .1094577270508E+04},
 {   17501, .8416025390625E+04, -.1477286010742E+04, .1096346923828E+04},
 {   17502, .8409597656250E+04, -.1490466552734E+04, .1099222900391E+04},
 {   17503, .8400949218750E+04, -.1502020019531E+04, .1103095214844E+04},
 {   17504, .8390408203125E+04, -.1511502075195E+04, .1107813964844E+04},
 {   17505, .8378379882812E+04, -.1518548095703E+04, .1113197998047E+04},
 {   17506, .8365329101562E+04, -.1522886962891E+04, .1119040527344E+04},
 {   17507, .8409597656250E+04, -.1462985229492E+04, .1071394165039E+04},
 {   17508, .8405645507812E+04, -.1477286010742E+04, .1073164306641E+04},
 {   17509, .8399219726562E+04, -.1490466674805E+04, .1076040039062E+04},
 {   17510, .8390572265625E+04, -.1502020019531E+04, .1079912109375E+04},
 {   17511, .8380032226562E+04, -.1511502075195E+04, .1084630615234E+04},
 {   17512, .8368000976562E+04, -.1518548095703E+04, .1090015014648E+04},
 {   17513, .8354951171875E+04, -.1522886962891E+04, .1095857421875E+04},
 {   17514, .8399220703125E+04, -.1462985229492E+04, .1048210937500E+04},
 {   17515, .8395267578125E+04, -.1477286010742E+04, .1049981079102E+04},
 {   17516, .8388840820312E+04, -.1490466674805E+04, .1052857177734E+04},
 {   17517, .8380192382812E+04, -.1502020019531E+04, .1056729003906E+04},
 {   17518, .8369652343750E+04, -.1511502075195E+04, .1061447509766E+04},
 {   17519, .8357624023438E+04, -.1518548095703E+04, .1066831909180E+04},
 {   17520, .8344575195312E+04, -.1522886962891E+04, .1072674072266E+04},
 {   17521, .8388841796875E+04, -.1462985229492E+04, .1025028564453E+04},
 {   17522, .8384889648438E+04, -.1477286010742E+04, .1026797607422E+04},
 {   17523, .8378462890625E+04, -.1490466674805E+04, .1029673828125E+04},
 {   17524, .8369814453125E+04, -.1502020019531E+04, .1033545898438E+04},
 {   17525, .8359274414062E+04, -.1511502075195E+04, .1038264648438E+04},
 {   17526, .8347247070312E+04, -.1518548095703E+04, .1043648193359E+04},
 {   17527, .8334193359375E+04, -.1522886962891E+04, .1049491333008E+04},
 {   17528, .8378463867188E+04, -.1462985229492E+04, .1001844726562E+04},
 {   17529, .8374509765625E+04, -.1477286010742E+04, .1003614746094E+04},
 {   17530, .8368084960938E+04, -.1490466674805E+04, .1006490966797E+04},
 {   17531, .8359435546875E+04, -.1502020019531E+04, .1010363159180E+04},
 {   17532, .8348897460938E+04, -.1511502075195E+04, .1015081665039E+04},
 {   17533, .8336868164062E+04, -.1518548095703E+04, .1020465759277E+04},
 {   17534, .8323815429688E+04, -.1522886962891E+04, .1026307861328E+04},
 {   17535, .8368084960938E+04, -.1462985229492E+04, .9786616210938E+03},
 {   17536, .8364132812500E+04, -.1477286010742E+04, .9804317626953E+03},
 {   17537, .8357706054688E+04, -.1490466674805E+04, .9833081665039E+03},
 {   17538, .8349057617188E+04, -.1502020019531E+04, .9871795654297E+03},
 {   17539, .8338516601562E+04, -.1511502075195E+04, .9918986816406E+03},
 {   17540, .8326488281250E+04, -.1518548095703E+04, .9972828979492E+03},
 {   17541, .8313439453125E+04, -.1522886962891E+04, .1003125061035E+04},
 {   17542, .8357706054688E+04, -.1462985107422E+04, .9554790039062E+03},
 {   17543, .8353755859375E+04, -.1477286010742E+04, .9572486572266E+03},
 {   17544, .8347330078125E+04, -.1490466552734E+04, .9601246337891E+03},
 {   17545, .8338680664062E+04, -.1502020019531E+04, .9639967041016E+03},
 {   17546, .8328138671875E+04, -.1511502075195E+04, .9687159423828E+03},
 {   17547, .8316111328125E+04, -.1518548095703E+04, .9740994873047E+03},
 {   17548, .8303060546875E+04, -.1522886962891E+04, .9799417724609E+03},
 {   17549, .8347329101562E+04, -.1462985229492E+04, .9322956542969E+03},
 {   17550, .8343375000000E+04, -.1477286010742E+04, .9340655517578E+03},
 {   17551, .8336951171875E+04, -.1490466674805E+04, .9369417724609E+03},
 {   17552, .8328301757812E+04, -.1502020019531E+04, .9408137207031E+03},
 {   17553, .8317760742188E+04, -.1511502075195E+04, .9455322265625E+03},
 {   17554, .8305734375000E+04, -.1518548095703E+04, .9509165039062E+03},
 {   17555, .8292682617188E+04, -.1522886962891E+04, .9567587890625E+03},
 {   17556, .8336951171875E+04, -.1462985107422E+04, .9091128540039E+03},
 {   17557, .8332997070312E+04, -.1477286010742E+04, .9108824462891E+03},
 {   17558, .8326572265625E+04, -.1490466552734E+04, .9137589721680E+03},
 {   17559, .8317923828125E+04, -.1502020019531E+04, .9176307983398E+03},
 {   17560, .8307380859375E+04, -.1511502075195E+04, .9223493652344E+03},
 {   17561, .8295355468750E+04, -.1518548095703E+04, .9277332763672E+03},
 {   17562, .8282304687500E+04, -.1522886962891E+04, .9335756835938E+03},
 {   17563, .8320067382812E+04, -.1459621582031E+04, .8888421630859E+03},
 {   17564, .8316748046875E+04, -.1472850952148E+04, .8903279418945E+03},
 {   17565, .8311090820312E+04, -.1485091918945E+04, .8928600463867E+03},
 {   17566, .8303326171875E+04, -.1495854370117E+04, .8963367309570E+03},
 {   17567, .8293760742188E+04, -.1504707641602E+04, .9006191406250E+03},
 {   17568, .8282775390625E+04, -.1511297485352E+04, .9055363769531E+03},
 {   17569, .8270814453125E+04, -.1515359741211E+04, .9108908081055E+03},
 {   17570, .8312857421875E+04, -.1458709716797E+04, .8643573608398E+03},
 {   17571, .8308219726562E+04, -.1470877441406E+04, .8664243164062E+03},
 {   17572, .8301590820312E+04, -.1482096069336E+04, .8693775634766E+03},
 {   17573, .8293175781250E+04, -.1491927124023E+04, .8731282958984E+03},
 {   17574, .8283233398438E+04, -.1499994140625E+04, .8775591430664E+03},
 {   17575, .8272093750000E+04, -.1505998901367E+04, .8825234985352E+03},
 {   17576, .8260124023438E+04, -.1509691162109E+04, .8878583374023E+03},
 {   17577, .8247731445312E+04, -.1510951782227E+04, .8933814697266E+03},
 {   17578, .8303859375000E+04, -.1453055297852E+04, .8406124267578E+03},
 {   17579, .8297976562500E+04, -.1463752685547E+04, .8432283935547E+03},
 {   17580, .8290549804688E+04, -.1473551513672E+04, .8465323486328E+03},
 {   17581, .8281720703125E+04, -.1482093872070E+04, .8504581909180E+03},
 {   17582, .8271702148438E+04, -.1489078369141E+04, .8549136352539E+03},
 {   17583, .8260750000000E+04, -.1494278198242E+04, .8597842407227E+03},
 {   17584, .8249141601562E+04, -.1497464233398E+04, .8649467163086E+03},
 {   17585, .8237202148438E+04, -.1498567260742E+04, .8702567138672E+03},
 {   17586, .8298740234375E+04, -.1439687133789E+04, .8153104248047E+03},
 {   17587, .8291088867188E+04, -.1448416503906E+04, .8186859130859E+03},
 {   17588, .8282284179688E+04, -.1456323242188E+04, .8225702514648E+03},
 {   17589, .8272426757812E+04, -.1463160034180E+04, .8269201660156E+03},
 {   17590, .8261659179688E+04, -.1468720458984E+04, .8316697998047E+03},
 {   17591, .8250185546875E+04, -.1472861206055E+04, .8367323608398E+03},
 {   17592, .8238200195312E+04, -.1475385620117E+04, .8420206298828E+03},
 {   17593, .8225955078125E+04, -.1476276977539E+04, .8474225463867E+03},
 {   17594, .8295124023438E+04, -.1437190429688E+04, .7894098510742E+03},
 {   17595, .8285242187500E+04, -.1444530029297E+04, .7937308959961E+03},
 {   17596, .8274686523438E+04, -.1451049682617E+04, .7983468017578E+03},
 {   17597, .8263483398438E+04, -.1456610839844E+04, .8032453613281E+03},
 {   17598, .8251708007812E+04, -.1461095458984E+04, .8083939819336E+03},
 {   17599, .8239480468750E+04, -.1464437500000E+04, .8137402954102E+03},
 {   17600, .8226909179688E+04, -.1466458618164E+04, .8192377929688E+03},
 {   17601, .8214166015625E+04, -.1467194213867E+04, .8248098754883E+03},
 {   17602, .8318005859375E+04, -.1450828613281E+04, .1150653320312E+04},
 {   17603, .8321814453125E+04, -.1453232055664E+04, .1139042968750E+04},
 {   17604, .8307483398438E+04, -.1448847900391E+04, .1141268066406E+04},
 {   17605, .8317319335938E+04, -.1453232055664E+04, .1128996337891E+04},
 {   17606, .8298369140625E+04, -.1447363647461E+04, .1129759521484E+04},
 {   17607, .8312819335938E+04, -.1453232055664E+04, .1118950683594E+04},
 {   17608, .8291129882812E+04, -.1446812622070E+04, .1116601318359E+04},
 {   17609, .8308323242188E+04, -.1453232055664E+04, .1108904541016E+04},
 {   17610, .8280750000000E+04, -.1446812622070E+04, .1093418334961E+04},
 {   17611, .8297944335938E+04, -.1453232055664E+04, .1085721435547E+04},
 {   17612, .8270373046875E+04, -.1446812622070E+04, .1070235107422E+04},
 {   17613, .8287566406250E+04, -.1453232055664E+04, .1062538330078E+04},
 {   17614, .8259997070312E+04, -.1446812622070E+04, .1047051391602E+04},
 {   17615, .8277189453125E+04, -.1453232055664E+04, .1039355346680E+04},
 {   17616, .8250912109375E+04, -.1446812622070E+04, .1026767211914E+04},
 {   17617, .8268106445312E+04, -.1453232055664E+04, .1019070129395E+04},
 {   17618, .8241833984375E+04, -.1446812622070E+04, .1006481689453E+04},
 {   17619, .8259026367188E+04, -.1453232055664E+04, .9987851562500E+03},
 {   17620, .8232750976562E+04, -.1446812622070E+04, .9861966552734E+03},
 {   17621, .8249945312500E+04, -.1453232055664E+04, .9784997558594E+03},
 {   17622, .8223668945312E+04, -.1446812622070E+04, .9659114379883E+03},
 {   17623, .8240865234375E+04, -.1453232055664E+04, .9582145385742E+03},
 {   17624, .8218678710938E+04, -.1447363647461E+04, .9517468872070E+03},
 {   17625, .8236365234375E+04, -.1453232055664E+04, .9481689453125E+03},
 {   17626, .8216166015625E+04, -.1448847900391E+04, .9372829589844E+03},
 {   17627, .8231868164062E+04, -.1453232055664E+04, .9381228637695E+03},
 {   17628, .8216172851562E+04, -.1450828735352E+04, .9231839599609E+03},
 {   17629, .8227373046875E+04, -.1453232055664E+04, .9280765380859E+03},
 {   17630, .8338834960938E+04, -.1453437500000E+04, .1149289428711E+04},
 {   17631, .8359652343750E+04, -.1454502075195E+04, .1142751464844E+04},
 {   17632, .8336884765625E+04, -.1453232055664E+04, .1132296386719E+04},
 {   17633, .8351951171875E+04, -.1453232055664E+04, .1125551025391E+04},
 {   17634, .8332385742188E+04, -.1453232055664E+04, .1122250610352E+04},
 {   17635, .8347455078125E+04, -.1453232055664E+04, .1115504882812E+04},
 {   17636, .8327888671875E+04, -.1453232055664E+04, .1112204833984E+04},
 {   17637, .8342958984375E+04, -.1453232055664E+04, .1105459350586E+04},
 {   17638, .8323389648438E+04, -.1453232055664E+04, .1102158691406E+04},
 {   17639, .8338461914062E+04, -.1453232055664E+04, .1095413452148E+04},
 {   17640, .8313014648438E+04, -.1453232055664E+04, .1078975585938E+04},
 {   17641, .8328083007812E+04, -.1453232055664E+04, .1072229858398E+04},
 {   17642, .8302634765625E+04, -.1453232055664E+04, .1055793090820E+04},
 {   17643, .8317705078125E+04, -.1453232055664E+04, .1049046875000E+04},
 {   17644, .8292255859375E+04, -.1453232055664E+04, .1032609497070E+04},
 {   17645, .8307327148438E+04, -.1453232055664E+04, .1025863769531E+04},
 {   17646, .8283175781250E+04, -.1453232055664E+04, .1012324035645E+04},
 {   17647, .8298244140625E+04, -.1453232055664E+04, .1005578613281E+04},
 {   17648, .8274094726562E+04, -.1453232055664E+04, .9920388793945E+03},
 {   17649, .8289163085938E+04, -.1453232055664E+04, .9852933959961E+03},
 {   17650, .8265014648438E+04, -.1453232055664E+04, .9717537231445E+03},
 {   17651, .8280083984375E+04, -.1453232055664E+04, .9650081787109E+03},
 {   17652, .8255934570312E+04, -.1453232055664E+04, .9514685668945E+03},
 {   17653, .8271002929688E+04, -.1453232055664E+04, .9447231445312E+03},
 {   17654, .8251438476562E+04, -.1453232055664E+04, .9414225463867E+03},
 {   17655, .8266504882812E+04, -.1453232055664E+04, .9346771240234E+03},
 {   17656, .8246939453125E+04, -.1453232055664E+04, .9313767700195E+03},
 {   17657, .8262008789062E+04, -.1453232055664E+04, .9246307983398E+03},
 {   17658, .8242442382812E+04, -.1453232055664E+04, .9213309326172E+03},
 {   17659, .8257511718750E+04, -.1453232055664E+04, .9145848388672E+03},
 {   17660, .8231068359375E+04, -.1453437500000E+04, .9085563354492E+03},
 {   17661, .8249811523438E+04, -.1454502075195E+04, .8973844604492E+03},
 {   17662, .8378395507812E+04, -.1453437500000E+04, .1131579223633E+04},
 {   17663, .8393288085938E+04, -.1450828613281E+04, .1116952392578E+04},
 {   17664, .8367021484375E+04, -.1453232055664E+04, .1118804809570E+04},
 {   17665, .8382090820312E+04, -.1453232055664E+04, .1112058959961E+04},
 {   17666, .8362524414062E+04, -.1453232055664E+04, .1108758789062E+04},
 {   17667, .8377593750000E+04, -.1453232055664E+04, .1102013793945E+04},
 {   17668, .8358026367188E+04, -.1453232055664E+04, .1098713134766E+04},
 {   17669, .8373095703125E+04, -.1453232055664E+04, .1091967773438E+04},
 {   17670, .8353531250000E+04, -.1453232055664E+04, .1088666992188E+04},
 {   17671, .8368596679688E+04, -.1453232055664E+04, .1081921508789E+04},
 {   17672, .8343151367188E+04, -.1453232055664E+04, .1065483764648E+04},
 {   17673, .8358220703125E+04, -.1453232055664E+04, .1058738159180E+04},
 {   17674, .8332774414062E+04, -.1453232055664E+04, .1042301147461E+04},
 {   17675, .8347843750000E+04, -.1453232055664E+04, .1035555297852E+04},
 {   17676, .8322395507812E+04, -.1453232055664E+04, .1019117492676E+04},
 {   17677, .8337463867188E+04, -.1453232055664E+04, .1012372009277E+04},
 {   17678, .8313313476562E+04, -.1453232055664E+04, .9988328857422E+03},
 {   17679, .8328381835938E+04, -.1453232055664E+04, .9920869140625E+03},
 {   17680, .8304234375000E+04, -.1453232055664E+04, .9785473632812E+03},
 {   17681, .8319302734375E+04, -.1453232055664E+04, .9718016357422E+03},
 {   17682, .8295152343750E+04, -.1453232055664E+04, .9582623901367E+03},
 {   17683, .8310221679688E+04, -.1453232055664E+04, .9515164184570E+03},
 {   17684, .8286072265625E+04, -.1453232055664E+04, .9379769287109E+03},
 {   17685, .8301140625000E+04, -.1453232055664E+04, .9312313232422E+03},
 {   17686, .8281574218750E+04, -.1453232055664E+04, .9279310302734E+03},
 {   17687, .8296644531250E+04, -.1453232055664E+04, .9211851196289E+03},
 {   17688, .8277076171875E+04, -.1453232055664E+04, .9178851928711E+03},
 {   17689, .8292146484375E+04, -.1453232055664E+04, .9111389770508E+03},
 {   17690, .8272580078125E+04, -.1453232055664E+04, .9078392333984E+03},
 {   17691, .8287650390625E+04, -.1453232055664E+04, .9010930786133E+03},
 {   17692, .8270626953125E+04, -.1453437500000E+04, .8908463134766E+03},
 {   17693, .8291458007812E+04, -.1450828735352E+04, .8894819335938E+03},
 {   17694, .8393296875000E+04, -.1448847900391E+04, .1102852539062E+04},
 {   17695, .8390781250000E+04, -.1447363647461E+04, .1088389648438E+04},
 {   17696, .8385791992188E+04, -.1446812622070E+04, .1074223876953E+04},
 {   17697, .8375413085938E+04, -.1446812622070E+04, .1051041381836E+04},
 {   17698, .8365034179688E+04, -.1446812622070E+04, .1027858154297E+04},
 {   17699, .8354658203125E+04, -.1446812622070E+04, .1004675292969E+04},
 {   17700, .8345578125000E+04, -.1446812622070E+04, .9843897705078E+03},
 {   17701, .8336496093750E+04, -.1446812622070E+04, .9641046752930E+03},
 {   17702, .8327414062500E+04, -.1446812622070E+04, .9438195190430E+03},
 {   17703, .8318333984375E+04, -.1446812622070E+04, .9235344848633E+03},
 {   17704, .8311094726562E+04, -.1447363647461E+04, .9103767700195E+03},
 {   17705, .8301980468750E+04, -.1448847900391E+04, .8988674926758E+03},
 {   20000, .8221334960938E+04, -.1114318359375E+04, .6660175170898E+03},
 {   20001, .8229469726562E+04, -.1112345458984E+04, .6923956909180E+03},
 {   20002, .8242388671875E+04, -.1126311401367E+04, .6547593383789E+03},
 {   20003, .8299099609375E+04, -.1177436279297E+04, .6742855224609E+03},
 {   20004, .8285311523438E+04, -.1146860351562E+04, .6791268920898E+03},
 {   20005, .8297224609375E+04, -.1148205322266E+04, .7100858154297E+03},
 {   20006, .8311208984375E+04, -.1177665283203E+04, .7011039428711E+03},
 {   20007, .8264123046875E+04, -.1126217407227E+04, .6953387451172E+03},
 {   20008, .8275026367188E+04, -.1128804321289E+04, .7246196289062E+03},
 {   20009, .8304035156250E+04, -.1152812255859E+04, .7405898437500E+03},
 {   20010, .8306616210938E+04, -.1159820312500E+04, .7722965087891E+03},
 {   20011, .8321920898438E+04, -.1185097290039E+04, .7652559204102E+03},
 {   20012, .8318469726562E+04, -.1180184204102E+04, .7306381835938E+03},
 {   20013, .8281583984375E+04, -.1133493408203E+04, .7528951416016E+03},
 {   20014, .8286395507812E+04, -.1141610351562E+04, .7815134277344E+03},
 {   20015, .8303175781250E+04, -.1206579223633E+04, .6731119384766E+03},
 {   20016, .8315314453125E+04, -.1206553222656E+04, .6989295654297E+03},
 {   20017, .8263010742188E+04, -.1179743286133E+04, .6144863281250E+03},
 {   20018, .8256654296875E+04, -.1153607299805E+04, .6269889526367E+03},
 {   20019, .8268521484375E+04, -.1148243286133E+04, .6489467773438E+03},
 {   20020, .8281695312500E+04, -.1177579223633E+04, .6417368164062E+03},
 {   20021, .8233368164062E+04, -.1137639404297E+04, .6242036743164E+03},
 {   20022, .8232829101562E+04, -.1171894287109E+04, .5965442504883E+03},
 {   20023, .8242522460938E+04, -.1113764404297E+04, .7139424438477E+03},
 {   20024, .8252968750000E+04, -.1119171386719E+04, .7371580200195E+03},
 {   20025, .8262209960938E+04, -.1125983398438E+04, .7648962402344E+03},
 {   20026, .8273609375000E+04, -.1133339355469E+04, .7883507690430E+03},
 {   20027, .8215849609375E+04, -.1120537353516E+04, .6435011596680E+03},
 {   20028, .8184347167969E+04, -.1112431396484E+04, .6478016967773E+03},
 {   20029, .8194059570312E+04, -.1105350463867E+04, .6752220458984E+03},
 {   20030, .8205533203125E+04, -.1104609375000E+04, .7021431274414E+03},
 {   20031, .8150839843750E+04, -.1107752441406E+04, .6574788818359E+03},
 {   20032, .8164010253906E+04, -.1100643432617E+04, .6847281494141E+03},
 {   20033, .8324293945312E+04, -.1291211059570E+04, .7933649902344E+03},
 {   20034, .8328677734375E+04, -.1292647094727E+04, .7941295166016E+03},
 {   20035, .8330552734375E+04, -.1249544189453E+04, .8016476440430E+03},
 {   20036, .8326376953125E+04, -.1257984130859E+04, .7993178100586E+03},
 {   20037, .8325506835938E+04, -.1291203002930E+04, .7721834106445E+03},
 {   20038, .8326670898438E+04, -.1253477172852E+04, .7777946166992E+03},
 {   20039, .8330956054688E+04, -.1230569213867E+04, .8056950073242E+03},
 {   20040, .8324666992188E+04, -.1231558105469E+04, .8057034301758E+03},
 {   20041, .8327803710938E+04, -.1231919189453E+04, .7806319580078E+03},
 {   20042, .8329578125000E+04, -.1211952148438E+04, .8082604980469E+03},
 {   20043, .8320123046875E+04, -.1205905273438E+04, .8118612670898E+03},
 {   20044, .8326210937500E+04, -.1210230224609E+04, .7847802734375E+03},
 {   20045, .8325854492188E+04, -.1191636230469E+04, .8103929443359E+03},
 {   20046, .8310490234375E+04, -.1183226196289E+04, .8164830322266E+03},
 {   20047, .8322251953125E+04, -.1187901245117E+04, .7905787353516E+03},
 {   20048, .8315116210938E+04, -.1168847290039E+04, .8143828125000E+03},
 {   20049, .8296639648438E+04, -.1149485351562E+04, .8216397705078E+03},
 {   20050, .8309751953125E+04, -.1163604248047E+04, .7964669189453E+03},
 {   20051, .8290714843750E+04, -.1144871337891E+04, .8043790893555E+03},
 {   20052, .8284549804688E+04, -.1142659301758E+04, .8269224853516E+03},
 {   20053, .8292534179688E+04, -.1167934326172E+04, .8244078369141E+03},
 {   20054, .8279395507812E+04, -.1139439331055E+04, .8099572753906E+03},
 {   20055, .8263810546875E+04, -.1137565307617E+04, .8353657836914E+03},
 {   20056, .8270958007812E+04, -.1154444335938E+04, .8339989013672E+03},
 {   20057, .8259363281250E+04, -.1134420410156E+04, .8190068359375E+03},
 {   20058, .8249296875000E+04, -.1139257324219E+04, .8422262573242E+03},
 {   20059, .8247178710938E+04, -.1147031372070E+04, .8446380004883E+03},
 {   20060, .8237533203125E+04, -.1135059326172E+04, .8268518066406E+03},
 {   20061, .8218426757812E+04, -.1145035400391E+04, .8566428833008E+03},
 {   20062, .8221731445312E+04, -.1148524291992E+04, .8557681274414E+03},
 {   20063, .8208824218750E+04, -.1142715332031E+04, .8412341308594E+03},
 {   20064, .8186823242188E+04, -.1155191284180E+04, .8524267578125E+03},
 {   20065, .8195674804688E+04, -.1156751342773E+04, .8675089111328E+03},
 {   20066, .8198276367188E+04, -.1158225341797E+04, .8669866943359E+03},
 {   20067, .8168103515625E+04, -.1172313232422E+04, .8601078491211E+03},
 {   20068, .8178054199219E+04, -.1175817260742E+04, .8755500488281E+03},
 {   20069, .8186426757812E+04, -.1169875244141E+04, .8735729980469E+03},
 {   20070, .8155001953125E+04, -.1191598266602E+04, .8648604736328E+03},
 {   20071, .8165515136719E+04, -.1198578247070E+04, .8819425659180E+03},
 {   20072, .8172717773438E+04, -.1190911254883E+04, .8804022827148E+03},
 {   20073, .8144407226562E+04, -.1213425170898E+04, .8686635742188E+03},
 {   20074, .8157070800781E+04, -.1220638183594E+04, .8856065673828E+03},
 {   20075, .8163945312500E+04, -.1213534179688E+04, .8830734863281E+03},
 {   20076, .8133422851562E+04, -.1239191162109E+04, .8694008178711E+03},
 {   20077, .8149476074219E+04, -.1242614135742E+04, .8890161132812E+03},
 {   20078, .8154448242188E+04, -.1236709106445E+04, .8880517578125E+03},
 {   20079, .8146138671875E+04, -.1263442138672E+04, .8910945434570E+03},
 {   20080, .8148759277344E+04, -.1260926147461E+04, .8902681884766E+03},
 {   20081, .8130820800781E+04, -.1261780151367E+04, .8703256225586E+03},
 {   20082, .8144522460938E+04, -.1288301025391E+04, .8918510742188E+03},
 {   20083, .8149380371094E+04, -.1291211059570E+04, .8907600097656E+03},
 {   20084, .8130681640625E+04, -.1288523071289E+04, .8703189086914E+03},
 {   20085, .8111149414062E+04, -.1288110107422E+04, .8493769531250E+03},
 {   20086, .8111573730469E+04, -.1259650146484E+04, .8493336181641E+03},
 {   20087, .8115351562500E+04, -.1234543212891E+04, .8478386230469E+03},
 {   20088, .8130904296875E+04, -.1204896240234E+04, .8478902587891E+03},
 {   20089, .8145407226562E+04, -.1185384277344E+04, .8428988037109E+03},
 {   20090, .8160942382812E+04, -.1168306274414E+04, .8375796508789E+03},
 {   20091, .8176873046875E+04, -.1150780273438E+04, .8299720458984E+03},
 {   20092, .8197159179688E+04, -.1135787353516E+04, .8180693359375E+03},
 {   20093, .8324658203125E+04, -.1254077148438E+04, .7522008666992E+03},
 {   20094, .8327187500000E+04, -.1233545166016E+04, .7535158081055E+03},
 {   20095, .8321255859375E+04, -.1291017089844E+04, .7504875488281E+03},
 {   20096, .8326854492188E+04, -.1209160156250E+04, .7607690429688E+03},
 {   20097, .8252541992188E+04, -.1128946411133E+04, .7955891723633E+03},
 {   20098, .8218214843750E+04, -.1128661376953E+04, .8025316772461E+03},
 {   20099, .8096734863281E+04, -.1257945068359E+04, .8162101440430E+03},
 {   20100, .8092711914062E+04, -.1289028076172E+04, .8066590576172E+03},
 {   20101, .8103931640625E+04, -.1233679199219E+04, .8200980834961E+03},
 {   20102, .8122844238281E+04, -.1200506225586E+04, .8221862792969E+03},
 {   20103, .8139534179688E+04, -.1183268310547E+04, .8191456909180E+03},
 {   20104, .8152487304688E+04, -.1163298339844E+04, .8143187255859E+03},
 {   20105, .8166030273438E+04, -.1144855346680E+04, .8088356323242E+03},
 {   20106, .8185244628906E+04, -.1131825317383E+04, .7927114257812E+03},
 {   20107, .8206840820312E+04, -.1124989379883E+04, .7800773925781E+03},
 {   20108, .8234409179688E+04, -.1122424438477E+04, .7728263549805E+03},
 {   20109, .8315448242188E+04, -.1290112060547E+04, .7311929931641E+03},
 {   20110, .8321006835938E+04, -.1256509155273E+04, .7282465209961E+03},
 {   20111, .8324354492188E+04, -.1234243164062E+04, .7283739013672E+03},
 {   20112, .8323921875000E+04, -.1207326171875E+04, .7286352539062E+03},
 {   20113, .8307087890625E+04, -.1288179077148E+04, .7058951416016E+03},
 {   20114, .8312444335938E+04, -.1259360107422E+04, .7018990478516E+03},
 {   20115, .8293073242188E+04, -.1288069091797E+04, .6754035034180E+03},
 {   20116, .8299047851562E+04, -.1258765136719E+04, .6730797119141E+03},
 {   20117, .8211672851562E+04, -.1286820068359E+04, .6126928710938E+03},
 {   20118, .8171599121094E+04, -.1287780029297E+04, .6262720947266E+03},
 {   20119, .8165477539062E+04, -.1263824096680E+04, .6123370971680E+03},
 {   20120, .8203986328125E+04, -.1264952148438E+04, .5999550781250E+03},
 {   20121, .8144112792969E+04, -.1287884033203E+04, .6410380249023E+03},
 {   20122, .8138145019531E+04, -.1265367065430E+04, .6279558715820E+03},
 {   20123, .8118280273438E+04, -.1288359008789E+04, .6585221557617E+03},
 {   20124, .8112353515625E+04, -.1267026123047E+04, .6448267211914E+03},
 {   20125, .8094773437500E+04, -.1288265014648E+04, .6788886108398E+03},
 {   20126, .8087403808594E+04, -.1267705078125E+04, .6617024536133E+03},
 {   20127, .8088764648438E+04, -.1256013061523E+04, .7835079345703E+03},
 {   20128, .8081342285156E+04, -.1288519042969E+04, .7659102783203E+03},
 {   20129, .8095461425781E+04, -.1232320190430E+04, .7945558471680E+03},
 {   20130, .8195972656250E+04, -.1120557373047E+04, .7605704345703E+03},
 {   20131, .8224050781250E+04, -.1116907348633E+04, .7486752929688E+03},
 {   20132, .8315974609375E+04, -.1234570190430E+04, .7002612304688E+03},
 {   20133, .8304290039062E+04, -.1234104125977E+04, .6735903930664E+03},
 {   20134, .8274801757812E+04, -.1259677124023E+04, .6383092041016E+03},
 {   20135, .8286048828125E+04, -.1234428222656E+04, .6402947387695E+03},
 {   20136, .8286306640625E+04, -.1205487182617E+04, .6385345458984E+03},
 {   20137, .8266165039062E+04, -.1203543212891E+04, .6110313110352E+03},
 {   20138, .8231732421875E+04, -.1202794189453E+04, .5888987426758E+03},
 {   20139, .8200819335938E+04, -.1200527221680E+04, .5887554321289E+03},
 {   20140, .8159269531250E+04, -.1199795288086E+04, .6012469482422E+03},
 {   20141, .8165377929688E+04, -.1161652343750E+04, .6057780761719E+03},
 {   20142, .8196077148438E+04, -.1167313232422E+04, .5963555297852E+03},
 {   20143, .8203892578125E+04, -.1235326171875E+04, .5892828369141E+03},
 {   20144, .8160936523438E+04, -.1232803222656E+04, .6023351440430E+03},
 {   20145, .8132830566406E+04, -.1238016113281E+04, .6170338134766E+03},
 {   20146, .8205527343750E+04, -.1130065429688E+04, .6216671142578E+03},
 {   20147, .8175817382812E+04, -.1127211425781E+04, .6244683837891E+03},
 {   20148, .8129320312500E+04, -.1210305175781E+04, .6145715942383E+03},
 {   20149, .8138049316406E+04, -.1099878417969E+04, .6917573852539E+03},
 {   20150, .8148526367188E+04, -.1101249511719E+04, .7194396972656E+03},
 {   20151, .8175356933594E+04, -.1100600463867E+04, .7118104858398E+03},
 {   20152, .8091757812500E+04, -.1098491455078E+04, .7408794555664E+03},
 {   20153, .8081135742188E+04, -.1097532470703E+04, .7133184814453E+03},
 {   20154, .8054524902344E+04, -.1097082397461E+04, .7250893554688E+03},
 {   20155, .8065468750000E+04, -.1098695434570E+04, .7522424316406E+03},
 {   20156, .8055509277344E+04, -.1141020385742E+04, .7278684692383E+03},
 {   20157, .8065304199219E+04, -.1140851318359E+04, .7519920654297E+03},
 {   20158, .8069968261719E+04, -.1105681396484E+04, .6867026977539E+03},
 {   20159, .8043460449219E+04, -.1105169433594E+04, .6973512573242E+03},
 {   20160, .8046650390625E+04, -.1153991333008E+04, .7060902709961E+03},
 {   20161, .8119084960938E+04, -.1099408447266E+04, .7300857543945E+03},
 {   20162, .8128294921875E+04, -.1107725341797E+04, .7546955566406E+03},
 {   20163, .8158488769531E+04, -.1109361450195E+04, .7437252197266E+03},
 {   20164, .8097947753906E+04, -.1105370483398E+04, .6759517822266E+03},
 {   20165, .8109070312500E+04, -.1098141479492E+04, .7023973388672E+03},
 {   20166, .8129680664062E+04, -.1105771484375E+04, .6646010742188E+03},
 {   20167, .8101479003906E+04, -.1107557373047E+04, .7658840942383E+03},
 {   20168, .8056329101562E+04, -.1246314086914E+04, .6612705078125E+03},
 {   20169, .8052044433594E+04, -.1217843261719E+04, .6512761230469E+03},
 {   20170, .8076268066406E+04, -.1216546264648E+04, .6394502563477E+03},
 {   20171, .8080723632812E+04, -.1244320190430E+04, .6475759277344E+03},
 {   20172, .8101765136719E+04, -.1214186157227E+04, .6271171264648E+03},
 {   20173, .8106033691406E+04, -.1241980224609E+04, .6327735595703E+03},
 {   20174, .8051527343750E+04, -.1186785278320E+04, .6493507690430E+03},
 {   20175, .8075404296875E+04, -.1186200317383E+04, .6384702148438E+03},
 {   20176, .8098152343750E+04, -.1180493286133E+04, .6285466308594E+03},
 {   20177, .8034166992188E+04, -.1127027343750E+04, .6748286132812E+03},
 {   20178, .8060100585938E+04, -.1126598388672E+04, .6648012695312E+03},
 {   20179, .8053860839844E+04, -.1156166259766E+04, .6528675537109E+03},
 {   20180, .8029395019531E+04, -.1156485351562E+04, .6630393066406E+03},
 {   20181, .8027741210938E+04, -.1187414306641E+04, .6596500244141E+03},
 {   20182, .8043501953125E+04, -.1188334228516E+04, .6975463256836E+03},
 {   20183, .8028528320312E+04, -.1218845214844E+04, .6623579711914E+03},
 {   20184, .8045303710938E+04, -.1211430175781E+04, .7024004516602E+03},
 {   20185, .8033191406250E+04, -.1248106201172E+04, .6734692382812E+03},
 {   20186, .8048331054688E+04, -.1227091186523E+04, .7103623046875E+03},
 {   20187, .8062511718750E+04, -.1266017089844E+04, .6768580932617E+03},
 {   20188, .8040392578125E+04, -.1266042114258E+04, .6913593750000E+03},
 {   20189, .8052332519531E+04, -.1243902099609E+04, .7202362060547E+03},
 {   20190, .8086851074219E+04, -.1125932373047E+04, .6537277221680E+03},
 {   20191, .8115416015625E+04, -.1123579345703E+04, .6444545288086E+03},
 {   20192, .8084512695312E+04, -.1155476318359E+04, .6389169311523E+03},
 {   20193, .8086760742188E+04, -.1149685302734E+04, .8060927734375E+03},
 {   20194, .8116174316406E+04, -.1138355346680E+04, .7895264282227E+03},
 {   20195, .8111163085938E+04, -.1124782348633E+04, .7837799072266E+03},
 {   20196, .8083423339844E+04, -.1127811401367E+04, .7967877807617E+03},
 {   20197, .8075661621094E+04, -.1109023437500E+04, .7773757934570E+03},
 {   20198, .8075063964844E+04, -.1148791381836E+04, .7760114135742E+03},
 {   20199, .8128913574219E+04, -.1182853271484E+04, .6151767578125E+03},
 {   20200, .8134738281250E+04, -.1155548339844E+04, .6237980346680E+03},
 {   20201, .8181341308594E+04, -.1110169433594E+04, .7340070190430E+03},
 {   20202, .8174425781250E+04, -.1128941406250E+04, .7710006103516E+03},
 {   20203, .8072874023438E+04, -.1288301025391E+04, .6984991455078E+03},
 {   20204, .8110505371094E+04, -.1154320312500E+04, .7963269042969E+03},
 {   20205, .8108873535156E+04, -.1201780273438E+04, .8005902709961E+03},
 {   20206, .8087224121094E+04, -.1201870239258E+04, .8049542236328E+03},
 {   20207, .8083112304688E+04, -.1231622192383E+04, .7955129394531E+03},
 {   20208, .8074904785156E+04, -.1198142211914E+04, .7757004394531E+03},
 {   20209, .8071464843750E+04, -.1215212158203E+04, .7672632446289E+03},
 {   20210, .8065278320312E+04, -.1276084106445E+04, .7507579345703E+03},
 {   20211, .8061203125000E+04, -.1278615112305E+04, .7414506835938E+03},
 {   20212, .8078462890625E+04, -.1288520019531E+04, .7356887207031E+03},
 {   20213, .8079957031250E+04, -.1288520019531E+04, .7464848022461E+03},
 {   20214, .8066072265625E+04, -.1229176147461E+04, .7540221557617E+03},
 {   20215, .8060829101562E+04, -.1237423217773E+04, .7411442871094E+03},
 {   20216, .8077708496094E+04, -.1251429077148E+04, .7835180053711E+03},
 {   20217, .8071136230469E+04, -.1268609130859E+04, .7646463012695E+03},
 {   20218, .8118741699219E+04, -.1177570312500E+04, .8021271362305E+03},
 {   20219, .8101439941406E+04, -.1177667236328E+04, .8018574829102E+03},
 {   20220, .8088340820312E+04, -.1176907226562E+04, .8077891845703E+03},
 {   20221, .8075062500000E+04, -.1178300292969E+04, .7760585327148E+03},
 {   20222, .8133506835938E+04, -.1154350341797E+04, .7969713134766E+03},
 {   20223, .8056005859375E+04, -.1278842041016E+04, .7291827392578E+03},
 {   20224, .8078709960938E+04, -.1288425048828E+04, .7162497558594E+03},
 {   20225, .8049120117188E+04, -.1276445068359E+04, .7123823242188E+03},
 {   20226, .8118954589844E+04, -.1148256347656E+04, .6276500244141E+03},
 {   20227, .8139469238281E+04, -.1127862426758E+04, .6322988281250E+03},
 {   20228, .8147543945312E+04, -.1132300292969E+04, .7867094726562E+03},
 {   20229, .8212941406250E+04, -.1107603393555E+04, .7264933471680E+03},
 {   20230, .8142623046875E+04, -.1116760375977E+04, .7770410156250E+03},
 {   20231, .8267002929688E+04, -.1235521118164E+04, .6133294677734E+03},
 {   20232, .8232994140625E+04, -.1236648193359E+04, .5914793701172E+03},
 {   20233, .8267757812500E+04, -.1263253173828E+04, .6262641601562E+03},
 {   20234, .8236414062500E+04, -.1266578125000E+04, .6038491821289E+03},
 {   20235, .8258668945312E+04, -.1288181030273E+04, .6293937377930E+03},
 {   20236, .8240596679688E+04, -.1288595092773E+04, .6167203369141E+03},
 {   20237, .8242388671875E+04, -.1450290771484E+04, .6547593383789E+03},
 {   20238, .8229469726562E+04, -.1464256713867E+04, .6923957519531E+03},
 {   20239, .8221334960938E+04, -.1462283691406E+04, .6660175170898E+03},
 {   20240, .8297224609375E+04, -.1428396728516E+04, .7100858154297E+03},
 {   20241, .8285311523438E+04, -.1429741821289E+04, .6791270141602E+03},
 {   20242, .8299099609375E+04, -.1399165771484E+04, .6742855224609E+03},
 {   20243, .8311208984375E+04, -.1398936767578E+04, .7011040649414E+03},
 {   20244, .8275026367188E+04, -.1447797729492E+04, .7246196289062E+03},
 {   20245, .8264123046875E+04, -.1450384765625E+04, .6953387451172E+03},
 {   20246, .8321920898438E+04, -.1391504882812E+04, .7652559204102E+03},
 {   20247, .8306616210938E+04, -.1416781738281E+04, .7722966308594E+03},
 {   20248, .8304035156250E+04, -.1423789794922E+04, .7405898437500E+03},
 {   20249, .8318469726562E+04, -.1396417846680E+04, .7306381835938E+03},
 {   20250, .8286395507812E+04, -.1434991699219E+04, .7815134277344E+03},
 {   20251, .8281583984375E+04, -.1443108764648E+04, .7528951416016E+03},
 {   20252, .8303175781250E+04, -.1370022949219E+04, .6731120605469E+03},
 {   20253, .8315314453125E+04, -.1370048828125E+04, .6989295654297E+03},
 {   20254, .8268521484375E+04, -.1428358764648E+04, .6489467773438E+03},
 {   20255, .8256655273438E+04, -.1422994873047E+04, .6269889526367E+03},
 {   20256, .8263010742188E+04, -.1396858886719E+04, .6144863281250E+03},
 {   20257, .8281695312500E+04, -.1399022827148E+04, .6417368164062E+03},
 {   20258, .8233368164062E+04, -.1438962768555E+04, .6242036743164E+03},
 {   20259, .8232829101562E+04, -.1404707885742E+04, .5965442504883E+03},
 {   20260, .8252968750000E+04, -.1457430664062E+04, .7371580200195E+03},
 {   20261, .8242522460938E+04, -.1462837646484E+04, .7139425048828E+03},
 {   20262, .8273609375000E+04, -.1443262695312E+04, .7883507690430E+03},
 {   20263, .8262209960938E+04, -.1450618774414E+04, .7648962402344E+03},
 {   20264, .8215849609375E+04, -.1456064697266E+04, .6435011596680E+03},
 {   20265, .8194059570312E+04, -.1471251708984E+04, .6752220458984E+03},
 {   20266, .8184347167969E+04, -.1464170654297E+04, .6478016967773E+03},
 {   20267, .8205533203125E+04, -.1471992675781E+04, .7021431884766E+03},
 {   20268, .8164010253906E+04, -.1475958618164E+04, .6847282714844E+03},
 {   20269, .8150839843750E+04, -.1468849731445E+04, .6574788818359E+03},
 {   20270, .8326376953125E+04, -.1324437988281E+04, .7993178100586E+03},
 {   20271, .8330552734375E+04, -.1327057983398E+04, .8016476440430E+03},
 {   20272, .8326670898438E+04, -.1323125000000E+04, .7777946166992E+03},
 {   20273, .8324666992188E+04, -.1345043945312E+04, .8057034301758E+03},
 {   20274, .8330956054688E+04, -.1346032958984E+04, .8056951293945E+03},
 {   20275, .8327803710938E+04, -.1344682983398E+04, .7806320800781E+03},
 {   20276, .8329578125000E+04, -.1364649902344E+04, .8082604980469E+03},
 {   20277, .8320123046875E+04, -.1370696899414E+04, .8118612670898E+03},
 {   20278, .8326210937500E+04, -.1366371826172E+04, .7847802734375E+03},
 {   20279, .8325854492188E+04, -.1384965820312E+04, .8103929443359E+03},
 {   20280, .8310490234375E+04, -.1393375854492E+04, .8164830322266E+03},
 {   20281, .8322251953125E+04, -.1388700805664E+04, .7905787353516E+03},
 {   20282, .8315116210938E+04, -.1407754760742E+04, .8143828125000E+03},
 {   20283, .8296639648438E+04, -.1427116821289E+04, .8216397705078E+03},
 {   20284, .8290714843750E+04, -.1431730712891E+04, .8043790893555E+03},
 {   20285, .8309751953125E+04, -.1412997802734E+04, .7964669189453E+03},
 {   20286, .8284549804688E+04, -.1433942749023E+04, .8269224853516E+03},
 {   20287, .8292534179688E+04, -.1408667846680E+04, .8244079589844E+03},
 {   20288, .8279395507812E+04, -.1437162719727E+04, .8099573974609E+03},
 {   20289, .8263810546875E+04, -.1439036743164E+04, .8353657836914E+03},
 {   20290, .8270958007812E+04, -.1422157836914E+04, .8339989013672E+03},
 {   20291, .8259363281250E+04, -.1442181762695E+04, .8190068359375E+03},
 {   20292, .8249296875000E+04, -.1437344726562E+04, .8422263793945E+03},
 {   20293, .8247178710938E+04, -.1429570800781E+04, .8446380004883E+03},
 {   20294, .8237533203125E+04, -.1441542724609E+04, .8268518066406E+03},
 {   20295, .8218426757812E+04, -.1431566772461E+04, .8566430053711E+03},
 {   20296, .8208824218750E+04, -.1433886718750E+04, .8412341308594E+03},
 {   20297, .8221731445312E+04, -.1428077758789E+04, .8557681274414E+03},
 {   20298, .8198276367188E+04, -.1418376708984E+04, .8669868164062E+03},
 {   20299, .8195674804688E+04, -.1419850830078E+04, .8675089111328E+03},
 {   20300, .8186823242188E+04, -.1421410766602E+04, .8524267578125E+03},
 {   20301, .8186426757812E+04, -.1406726806641E+04, .8735729980469E+03},
 {   20302, .8178054199219E+04, -.1400784790039E+04, .8755500488281E+03},
 {   20303, .8168103515625E+04, -.1404288818359E+04, .8601078491211E+03},
 {   20304, .8172717773438E+04, -.1385690917969E+04, .8804022827148E+03},
 {   20305, .8165515136719E+04, -.1378023925781E+04, .8819425659180E+03},
 {   20306, .8155001953125E+04, -.1385003906250E+04, .8648604736328E+03},
 {   20307, .8163945312500E+04, -.1363067871094E+04, .8830734863281E+03},
 {   20308, .8157070800781E+04, -.1355963867188E+04, .8856065673828E+03},
 {   20309, .8144407226562E+04, -.1363176879883E+04, .8686635742188E+03},
 {   20310, .8154448242188E+04, -.1339892944336E+04, .8880517578125E+03},
 {   20311, .8149476074219E+04, -.1333987915039E+04, .8890161132812E+03},
 {   20312, .8133422851562E+04, -.1337410888672E+04, .8694008178711E+03},
 {   20313, .8146138671875E+04, -.1313160034180E+04, .8910945434570E+03},
 {   20314, .8130820800781E+04, -.1314822021484E+04, .8703256225586E+03},
 {   20315, .8148759277344E+04, -.1315676025391E+04, .8902681884766E+03},
 {   20316, .8111573730469E+04, -.1316952026367E+04, .8493336181641E+03},
 {   20317, .8115351562500E+04, -.1342058959961E+04, .8478386230469E+03},
 {   20318, .8130904296875E+04, -.1371705932617E+04, .8478902587891E+03},
 {   20319, .8145407226562E+04, -.1391217895508E+04, .8428988037109E+03},
 {   20320, .8160942382812E+04, -.1408295776367E+04, .8375796508789E+03},
 {   20321, .8176873046875E+04, -.1425821777344E+04, .8299720458984E+03},
 {   20322, .8197159179688E+04, -.1440814697266E+04, .8180693359375E+03},
 {   20323, .8324658203125E+04, -.1322525024414E+04, .7522008666992E+03},
 {   20324, .8327187500000E+04, -.1343056884766E+04, .7535158081055E+03},
 {   20325, .8326854492188E+04, -.1367441894531E+04, .7607690429688E+03},
 {   20326, .8252541992188E+04, -.1447655761719E+04, .7955891723633E+03},
 {   20327, .8218214843750E+04, -.1447940673828E+04, .8025316772461E+03},
 {   20328, .8096734863281E+04, -.1318656982422E+04, .8162101440430E+03},
 {   20329, .8103931640625E+04, -.1342922973633E+04, .8200980834961E+03},
 {   20330, .8122844238281E+04, -.1376095825195E+04, .8221862792969E+03},
 {   20331, .8139534179688E+04, -.1393333862305E+04, .8191458129883E+03},
 {   20332, .8152487304688E+04, -.1413303833008E+04, .8143187255859E+03},
 {   20333, .8166030273438E+04, -.1431746826172E+04, .8088356323242E+03},
 {   20334, .8185244628906E+04, -.1444776733398E+04, .7927115478516E+03},
 {   20335, .8206840820312E+04, -.1451612792969E+04, .7800773925781E+03},
 {   20336, .8234409179688E+04, -.1454177734375E+04, .7728263549805E+03},
 {   20337, .8321006835938E+04, -.1320093017578E+04, .7282465209961E+03},
 {   20338, .8324354492188E+04, -.1342358886719E+04, .7283739013672E+03},
 {   20339, .8323921875000E+04, -.1369275878906E+04, .7286352539062E+03},
 {   20340, .8312444335938E+04, -.1317241943359E+04, .7018990478516E+03},
 {   20341, .8298801757812E+04, -.1315868041992E+04, .6731243286133E+03},
 {   20342, .8203986328125E+04, -.1311650024414E+04, .5999550781250E+03},
 {   20343, .8165477539062E+04, -.1312778076172E+04, .6123370971680E+03},
 {   20344, .8138145019531E+04, -.1311234985352E+04, .6279558715820E+03},
 {   20345, .8112353515625E+04, -.1309576049805E+04, .6448267211914E+03},
 {   20346, .8087403808594E+04, -.1308896972656E+04, .6617024536133E+03},
 {   20347, .8088764648438E+04, -.1320588989258E+04, .7835079345703E+03},
 {   20348, .8095461425781E+04, -.1344281982422E+04, .7945558471680E+03},
 {   20349, .8195972656250E+04, -.1456044677734E+04, .7605704345703E+03},
 {   20350, .8224050781250E+04, -.1459694702148E+04, .7486752929688E+03},
 {   20351, .8304290039062E+04, -.1342497924805E+04, .6735903930664E+03},
 {   20352, .8315974609375E+04, -.1342031982422E+04, .7002612304688E+03},
 {   20353, .8286048828125E+04, -.1342173950195E+04, .6402947387695E+03},
 {   20354, .8275472656250E+04, -.1316133056641E+04, .6379448242188E+03},
 {   20355, .8286306640625E+04, -.1371114868164E+04, .6385346679688E+03},
 {   20356, .8266165039062E+04, -.1373058837891E+04, .6110313110352E+03},
 {   20357, .8231732421875E+04, -.1373807861328E+04, .5888987426758E+03},
 {   20358, .8165377929688E+04, -.1414949829102E+04, .6057780761719E+03},
 {   20359, .8159269531250E+04, -.1376806884766E+04, .6012469482422E+03},
 {   20360, .8200819335938E+04, -.1376074951172E+04, .5887555541992E+03},
 {   20361, .8196077148438E+04, -.1409288818359E+04, .5963555297852E+03},
 {   20362, .8160936523438E+04, -.1343798950195E+04, .6023351440430E+03},
 {   20363, .8203892578125E+04, -.1341276000977E+04, .5892828369141E+03},
 {   20364, .8132830566406E+04, -.1338585937500E+04, .6170338134766E+03},
 {   20365, .8205527343750E+04, -.1446536743164E+04, .6216671142578E+03},
 {   20366, .8175817382812E+04, -.1449390747070E+04, .6244683837891E+03},
 {   20367, .8129320312500E+04, -.1366296875000E+04, .6145717163086E+03},
 {   20368, .8175356933594E+04, -.1476001586914E+04, .7118104858398E+03},
 {   20369, .8148526367188E+04, -.1475352661133E+04, .7194397583008E+03},
 {   20370, .8138049316406E+04, -.1476723632812E+04, .6917573852539E+03},
 {   20371, .8054524902344E+04, -.1479519653320E+04, .7250893554688E+03},
 {   20372, .8081135742188E+04, -.1479069702148E+04, .7133186035156E+03},
 {   20373, .8091757812500E+04, -.1478110717773E+04, .7408794555664E+03},
 {   20374, .8065468750000E+04, -.1477906616211E+04, .7522425537109E+03},
 {   20375, .8065304199219E+04, -.1435750732422E+04, .7519920654297E+03},
 {   20376, .8055509277344E+04, -.1435581787109E+04, .7278684692383E+03},
 {   20377, .8069968261719E+04, -.1470920654297E+04, .6867026977539E+03},
 {   20378, .8043460449219E+04, -.1471432617188E+04, .6973512573242E+03},
 {   20379, .8046650390625E+04, -.1422610839844E+04, .7060902709961E+03},
 {   20380, .8158488769531E+04, -.1467240722656E+04, .7437252197266E+03},
 {   20381, .8128294921875E+04, -.1468876708984E+04, .7546956176758E+03},
 {   20382, .8119084960938E+04, -.1477193603516E+04, .7300857543945E+03},
 {   20383, .8129680664062E+04, -.1470830688477E+04, .6646010742188E+03},
 {   20384, .8109070312500E+04, -.1478460693359E+04, .7023973388672E+03},
 {   20385, .8097947753906E+04, -.1471231689453E+04, .6759517822266E+03},
 {   20386, .8101479003906E+04, -.1469044677734E+04, .7658840942383E+03},
 {   20387, .8076268066406E+04, -.1360055908203E+04, .6394502563477E+03},
 {   20388, .8052044433594E+04, -.1358758911133E+04, .6512762451172E+03},
 {   20389, .8056329101562E+04, -.1330287963867E+04, .6612705078125E+03},
 {   20390, .8080723632812E+04, -.1332281982422E+04, .6475759277344E+03},
 {   20391, .8101765136719E+04, -.1362415893555E+04, .6271171264648E+03},
 {   20392, .8106033691406E+04, -.1334621948242E+04, .6327735595703E+03},
 {   20393, .8075404296875E+04, -.1390401855469E+04, .6384703369141E+03},
 {   20394, .8051527343750E+04, -.1389816894531E+04, .6493507690430E+03},
 {   20395, .8098152343750E+04, -.1396108886719E+04, .6285466308594E+03},
 {   20396, .8053860839844E+04, -.1420435791016E+04, .6528675537109E+03},
 {   20397, .8060100585938E+04, -.1450003662109E+04, .6648012695312E+03},
 {   20398, .8034166992188E+04, -.1449574707031E+04, .6748286132812E+03},
 {   20399, .8029395019531E+04, -.1420116821289E+04, .6630393066406E+03},
 {   20400, .8027741210938E+04, -.1389187866211E+04, .6596500244141E+03},
 {   20401, .8043501953125E+04, -.1388267822266E+04, .6975463256836E+03},
 {   20402, .8028528320312E+04, -.1357756958008E+04, .6623579711914E+03},
 {   20403, .8045303710938E+04, -.1365171875000E+04, .7024004516602E+03},
 {   20404, .8033191406250E+04, -.1328495971680E+04, .6734692382812E+03},
 {   20405, .8048331054688E+04, -.1349510986328E+04, .7103623046875E+03},
 {   20406, .8062511718750E+04, -.1310584960938E+04, .6768580932617E+03},
 {   20407, .8040392578125E+04, -.1310560058594E+04, .6913593750000E+03},
 {   20408, .8052332519531E+04, -.1332699951172E+04, .7202362060547E+03},
 {   20409, .8084512695312E+04, -.1421125732422E+04, .6389169311523E+03},
 {   20410, .8115416015625E+04, -.1453022705078E+04, .6444545288086E+03},
 {   20411, .8086851074219E+04, -.1450669677734E+04, .6537278442383E+03},
 {   20412, .8111163085938E+04, -.1451819702148E+04, .7837799072266E+03},
 {   20413, .8116174316406E+04, -.1438246826172E+04, .7895264282227E+03},
 {   20414, .8086760742188E+04, -.1426916748047E+04, .8060928955078E+03},
 {   20415, .8083423339844E+04, -.1448790771484E+04, .7967877807617E+03},
 {   20416, .8075063964844E+04, -.1427810791016E+04, .7760114135742E+03},
 {   20417, .8075661621094E+04, -.1467578735352E+04, .7773757934570E+03},
 {   20418, .8128913574219E+04, -.1393748901367E+04, .6151767578125E+03},
 {   20419, .8134738281250E+04, -.1421053833008E+04, .6237980346680E+03},
 {   20420, .8174425781250E+04, -.1447660766602E+04, .7710006103516E+03},
 {   20421, .8181341308594E+04, -.1466432617188E+04, .7340070190430E+03},
 {   20422, .8110505371094E+04, -.1422281738281E+04, .7963269042969E+03},
 {   20423, .8087224121094E+04, -.1374731933594E+04, .8049542236328E+03},
 {   20424, .8108873535156E+04, -.1374821899414E+04, .8005902709961E+03},
 {   20425, .8083112304688E+04, -.1344979980469E+04, .7955129394531E+03},
 {   20426, .8071464843750E+04, -.1361389892578E+04, .7672632446289E+03},
 {   20427, .8074904785156E+04, -.1378459838867E+04, .7757004394531E+03},
 {   20428, .8065278320312E+04, -.1300518066406E+04, .7507579345703E+03},
 {   20429, .8061203125000E+04, -.1297987060547E+04, .7414506835938E+03},
 {   20430, .8060829101562E+04, -.1339178955078E+04, .7411442871094E+03},
 {   20431, .8066072265625E+04, -.1347425903320E+04, .7540221557617E+03},
 {   20432, .8071136230469E+04, -.1307993041992E+04, .7646463012695E+03},
 {   20433, .8077708496094E+04, -.1325172973633E+04, .7835180053711E+03},
 {   20434, .8101439941406E+04, -.1398934814453E+04, .8018574829102E+03},
 {   20435, .8118741699219E+04, -.1399031860352E+04, .8021271362305E+03},
 {   20436, .8088340820312E+04, -.1399694824219E+04, .8077891845703E+03},
 {   20437, .8075062500000E+04, -.1398301879883E+04, .7760585327148E+03},
 {   20438, .8133506835938E+04, -.1422251831055E+04, .7969713134766E+03},
 {   20439, .8056005859375E+04, -.1297760009766E+04, .7291827392578E+03},
 {   20440, .8049120117188E+04, -.1300156982422E+04, .7123823242188E+03},
 {   20441, .8139469238281E+04, -.1448739746094E+04, .6322989501953E+03},
 {   20442, .8118954589844E+04, -.1428345825195E+04, .6276500244141E+03},
 {   20443, .8147543945312E+04, -.1444301757812E+04, .7867094726562E+03},
 {   20444, .8212941406250E+04, -.1468998657227E+04, .7264934692383E+03},
 {   20445, .8142624023438E+04, -.1459841674805E+04, .7770408935547E+03},
 {   20446, .8267002929688E+04, -.1341080932617E+04, .6133294677734E+03},
 {   20447, .8232994140625E+04, -.1339953979492E+04, .5914793701172E+03},
 {   20448, .8267166015625E+04, -.1312300048828E+04, .6254645996094E+03},
 {   20449, .8236414062500E+04, -.1310024047852E+04, .6038491821289E+03},
 {   20450, .8269813476562E+04, -.1288185058594E+04, .6399351806641E+03},
 {   20451, .8229130859375E+04, -.1164320312500E+04, .6860800170898E+03},
 {   20452, .8219431640625E+04, -.1164740356445E+04, .6622189331055E+03},
 {   20453, .8244725585938E+04, -.1167580322266E+04, .6484957275391E+03},
 {   20454, .8266969726562E+04, -.1164662231445E+04, .6920318603516E+03},
 {   20455, .8277658203125E+04, -.1165956298828E+04, .7207186279297E+03},
 {   20456, .8286106445312E+04, -.1167170288086E+04, .7428651733398E+03},
 {   20457, .8296468750000E+04, -.1172900268555E+04, .7716434936523E+03},
 {   20458, .8270948242188E+04, -.1205573242188E+04, .6886805419922E+03},
 {   20459, .8284291015625E+04, -.1206635253906E+04, .7143566284180E+03},
 {   20460, .8241764648438E+04, -.1164019287109E+04, .7060654907227E+03},
 {   20461, .8253408203125E+04, -.1164820312500E+04, .7325335693359E+03},
 {   20462, .8263618164062E+04, -.1152147338867E+04, .7607834472656E+03},
 {   20463, .8276159179688E+04, -.1152757324219E+04, .7850525512695E+03},
 {   20464, .8244424804688E+04, -.1201797241211E+04, .7025409545898E+03},
 {   20465, .8253904296875E+04, -.1197736206055E+04, .7281652221680E+03},
 {   20466, .8298028320312E+04, -.1210683227539E+04, .7380823364258E+03},
 {   20467, .8259788085938E+04, -.1201034179688E+04, .7537557373047E+03},
 {   20468, .8302254882812E+04, -.1202395263672E+04, .7659450683594E+03},
 {   20469, .8274774414062E+04, -.1181143310547E+04, .7802836914062E+03},
 {   20470, .8251607421875E+04, -.1204793212891E+04, .6532927856445E+03},
 {   20471, .8229412109375E+04, -.1201403198242E+04, .6841643066406E+03},
 {   20472, .8208323242188E+04, -.1168660278320E+04, .6339674072266E+03},
 {   20473, .8161774414062E+04, -.1132907348633E+04, .7413535766602E+03},
 {   20474, .8165198730469E+04, -.1180410278320E+04, .7358640136719E+03},
 {   20475, .8184619628906E+04, -.1177901245117E+04, .7266712036133E+03},
 {   20476, .8182754882812E+04, -.1138504394531E+04, .7315166015625E+03},
 {   20477, .8177538085938E+04, -.1145611328125E+04, .7689982299805E+03},
 {   20478, .8176807128906E+04, -.1172380249023E+04, .7616022338867E+03},
 {   20479, .8197355468750E+04, -.1172816284180E+04, .7518117675781E+03},
 {   20480, .8196055664062E+04, -.1149610351562E+04, .7562351684570E+03},
 {   20481, .8234684570312E+04, -.1183770263672E+04, .7639663085938E+03},
 {   20482, .8208016601562E+04, -.1173374267578E+04, .7746589355469E+03},
 {   20483, .8206071289062E+04, -.1152070312500E+04, .7753648071289E+03},
 {   20484, .8233450195312E+04, -.1141547363281E+04, .7688070678711E+03},
 {   20485, .8225425781250E+04, -.1181096313477E+04, .7409253540039E+03},
 {   20486, .8225118164062E+04, -.1150705322266E+04, .7444207763672E+03},
 {   20487, .8210897460938E+04, -.1174790283203E+04, .7188211669922E+03},
 {   20488, .8212396484375E+04, -.1146920288086E+04, .7207428588867E+03},
 {   20489, .8178479003906E+04, -.1161485351562E+04, .6444986572266E+03},
 {   20490, .8192079101562E+04, -.1144780273438E+04, .6734001464844E+03},
 {   20491, .8202250976562E+04, -.1144360351562E+04, .6970751342773E+03},
 {   20492, .8151955078125E+04, -.1144548339844E+04, .6615415649414E+03},
 {   20493, .8162562011719E+04, -.1137438354492E+04, .6864129638672E+03},
 {   20494, .8156502929688E+04, -.1142765380859E+04, .7825332031250E+03},
 {   20495, .8150164062500E+04, -.1174460327148E+04, .7842259521484E+03},
 {   20496, .8143436035156E+04, -.1132990356445E+04, .7480237426758E+03},
 {   20497, .8138465332031E+04, -.1170935302734E+04, .7492154541016E+03},
 {   20498, .8318907226562E+04, -.1291211059570E+04, .7724935302734E+03},
 {   20499, .8318598632812E+04, -.1256347167969E+04, .7762675781250E+03},
 {   20500, .8313353515625E+04, -.1228082153320E+04, .7831387329102E+03},
 {   20501, .8309865234375E+04, -.1200289184570E+04, .7896964111328E+03},
 {   20502, .8302314453125E+04, -.1177020263672E+04, .7967549438477E+03},
 {   20503, .8285027343750E+04, -.1163783325195E+04, .8042858886719E+03},
 {   20504, .8262550781250E+04, -.1153545288086E+04, .8139880371094E+03},
 {   20505, .8238361328125E+04, -.1149315307617E+04, .8238776855469E+03},
 {   20506, .8211397460938E+04, -.1156458251953E+04, .8359410400391E+03},
 {   20507, .8193332031250E+04, -.1165659301758E+04, .8455639648438E+03},
 {   20508, .8176021484375E+04, -.1175414306641E+04, .8532178955078E+03},
 {   20509, .8164821289062E+04, -.1193073242188E+04, .8579597167969E+03},
 {   20510, .8154315429688E+04, -.1212986206055E+04, .8616691894531E+03},
 {   20511, .8146219238281E+04, -.1234395141602E+04, .8625835571289E+03},
 {   20512, .8141961914062E+04, -.1256776123047E+04, .8638906250000E+03},
 {   20513, .8142281250000E+04, -.1291520019531E+04, .8665570068359E+03},
 {   20514, .8139947265625E+04, -.1291520996094E+04, .8364943237305E+03},
 {   20515, .8139616210938E+04, -.1256579101562E+04, .8363707275391E+03},
 {   20516, .8143405273438E+04, -.1233125122070E+04, .8352714843750E+03},
 {   20517, .8150654296875E+04, -.1210551269531E+04, .8368130493164E+03},
 {   20518, .8161256347656E+04, -.1191190307617E+04, .8332422485352E+03},
 {   20519, .8170932617188E+04, -.1173271240234E+04, .8289495239258E+03},
 {   20520, .8186535644531E+04, -.1163630249023E+04, .8212151489258E+03},
 {   20521, .8199300781250E+04, -.1152292358398E+04, .8146037597656E+03},
 {   20522, .8309138671875E+04, -.1257298095703E+04, .7527840576172E+03},
 {   20523, .8305178710938E+04, -.1229749145508E+04, .7583654785156E+03},
 {   20524, .8308779296875E+04, -.1291211059570E+04, .7509728393555E+03},
 {   20525, .8254387695312E+04, -.1142323364258E+04, .7944768066406E+03},
 {   20526, .8218632812500E+04, -.1145060302734E+04, .8028565063477E+03},
 {   20527, .8125747558594E+04, -.1256429077148E+04, .8077823486328E+03},
 {   20528, .8124094238281E+04, -.1291520996094E+04, .8007474975586E+03},
 {   20529, .8133544921875E+04, -.1232938110352E+04, .8083259277344E+03},
 {   20530, .8142414062500E+04, -.1209994262695E+04, .8111502685547E+03},
 {   20531, .8155354003906E+04, -.1189243286133E+04, .8090092163086E+03},
 {   20532, .8162221191406E+04, -.1173395263672E+04, .8057006835938E+03},
 {   20533, .8172309082031E+04, -.1158921264648E+04, .7998519897461E+03},
 {   20534, .8186217773438E+04, -.1153231323242E+04, .7876194458008E+03},
 {   20535, .8219167968750E+04, -.1155871337891E+04, .8009004516602E+03},
 {   20536, .8186321777344E+04, -.1171006225586E+04, .7887932739258E+03},
 {   20537, .8197923828125E+04, -.1161176269531E+04, .8129661254883E+03},
 {   20538, .8253567382812E+04, -.1164830322266E+04, .7897035522461E+03},
 {   20539, .8304916015625E+04, -.1259137084961E+04, .7310228271484E+03},
 {   20540, .8303753906250E+04, -.1291211059570E+04, .7322512207031E+03},
 {   20541, .8301374023438E+04, -.1233126220703E+04, .7342466430664E+03},
 {   20542, .8302243164062E+04, -.1261636108398E+04, .7069771118164E+03},
 {   20543, .8300717773438E+04, -.1290986083984E+04, .7061799926758E+03},
 {   20544, .8283733398438E+04, -.1286886108398E+04, .6775573730469E+03},
 {   20545, .8285406250000E+04, -.1257927124023E+04, .6766157836914E+03},
 {   20546, .8221725585938E+04, -.1291211059570E+04, .6328450927734E+03},
 {   20547, .8217918945312E+04, -.1260253173828E+04, .6258404541016E+03},
 {   20548, .8183022949219E+04, -.1291211059570E+04, .6448064575195E+03},
 {   20549, .8176426757812E+04, -.1253790161133E+04, .6288469848633E+03},
 {   20550, .8155917968750E+04, -.1291211059570E+04, .6644862060547E+03},
 {   20551, .8144722656250E+04, -.1252162109375E+04, .6400581054688E+03},
 {   20552, .8127346679688E+04, -.1291211059570E+04, .6787355346680E+03},
 {   20553, .8123346191406E+04, -.1249490112305E+04, .6601043090820E+03},
 {   20554, .8104884765625E+04, -.1291211059570E+04, .6985787353516E+03},
 {   20555, .8107334472656E+04, -.1248824096680E+04, .6850720825195E+03},
 {   20556, .8106115234375E+04, -.1290320068359E+04, .7623261108398E+03},
 {   20557, .8113750976562E+04, -.1253874145508E+04, .7765959472656E+03},
 {   20558, .8123708007812E+04, -.1232792114258E+04, .7800521240234E+03},
 {   20559, .8288031250000E+04, -.1236701171875E+04, .7098858032227E+03},
 {   20560, .8275713867188E+04, -.1233831176758E+04, .6856658325195E+03},
 {   20561, .8242821289062E+04, -.1259949096680E+04, .6478756713867E+03},
 {   20562, .8252852539062E+04, -.1232455200195E+04, .6539686279297E+03},
 {   20563, .8200662109375E+04, -.1183168334961E+04, .6954318237305E+03},
 {   20564, .8191009765625E+04, -.1199628173828E+04, .6726956176758E+03},
 {   20565, .8175591796875E+04, -.1202662231445E+04, .6368486938477E+03},
 {   20566, .8207239257812E+04, -.1203167236328E+04, .6285049438477E+03},
 {   20567, .8218980468750E+04, -.1206793212891E+04, .6611552734375E+03},
 {   20568, .8227889648438E+04, -.1233180175781E+04, .6590971679688E+03},
 {   20569, .8215752929688E+04, -.1231719116211E+04, .6266950683594E+03},
 {   20570, .8172699218750E+04, -.1226900146484E+04, .6283753051758E+03},
 {   20571, .8142092285156E+04, -.1228361206055E+04, .6362024536133E+03},
 {   20572, .8162013183594E+04, -.1187732299805E+04, .6849189453125E+03},
 {   20573, .8173476562500E+04, -.1184909301758E+04, .7060338745117E+03},
 {   20574, .8169992187500E+04, -.1136291381836E+04, .7081137695312E+03},
 {   20575, .8142997558594E+04, -.1206349243164E+04, .6435178222656E+03},
 {   20576, .8145395019531E+04, -.1131651367188E+04, .6945888671875E+03},
 {   20577, .8153583984375E+04, -.1131778320312E+04, .7179713745117E+03},
 {   20578, .8084936523438E+04, -.1142220336914E+04, .7158573608398E+03},
 {   20579, .8093581054688E+04, -.1138767333984E+04, .7399722290039E+03},
 {   20580, .8075840820312E+04, -.1152681396484E+04, .6933887329102E+03},
 {   20581, .8120189941406E+04, -.1133593383789E+04, .7258135375977E+03},
 {   20582, .8132561035156E+04, -.1133095336914E+04, .7498334960938E+03},
 {   20583, .8102441406250E+04, -.1147200317383E+04, .6767194213867E+03},
 {   20584, .8112231445312E+04, -.1139791381836E+04, .7007926635742E+03},
 {   20585, .8104046875000E+04, -.1142830322266E+04, .7637159423828E+03},
 {   20586, .8133963867188E+04, -.1133030395508E+04, .7203406372070E+03},
 {   20587, .8126419433594E+04, -.1140280395508E+04, .6945307006836E+03},
 {   20588, .8134878906250E+04, -.1139062377930E+04, .6686829833984E+03},
 {   20589, .8069629882812E+04, -.1210764282227E+04, .6923320312500E+03},
 {   20590, .8073770019531E+04, -.1228474121094E+04, .6983433227539E+03},
 {   20591, .8099209960938E+04, -.1229856201172E+04, .6863268432617E+03},
 {   20592, .8093988769531E+04, -.1210489257812E+04, .6822908935547E+03},
 {   20593, .8119687988281E+04, -.1228522216797E+04, .6605174560547E+03},
 {   20594, .8119243164062E+04, -.1207749267578E+04, .6638703002930E+03},
 {   20595, .8067796386719E+04, -.1188221313477E+04, .6891648559570E+03},
 {   20596, .8094521484375E+04, -.1178517333984E+04, .6798299560547E+03},
 {   20597, .8120463378906E+04, -.1164126342773E+04, .6684019775391E+03},
 {   20598, .8078841796875E+04, -.1248837158203E+04, .7049028320312E+03},
 {   20599, .8163018554688E+04, -.1205793212891E+04, .6838976440430E+03},
 {   20600, .8143433105469E+04, -.1173975341797E+04, .6506860351562E+03},
 {   20601, .8105973632812E+04, -.1237955200195E+04, .7411036376953E+03},
 {   20602, .8086023925781E+04, -.1233066162109E+04, .7475635986328E+03},
 {   20603, .8105125488281E+04, -.1290696044922E+04, .7415849609375E+03},
 {   20604, .8083643066406E+04, -.1239868164062E+04, .7319744262695E+03},
 {   20605, .8106452148438E+04, -.1243314208984E+04, .7228034057617E+03},
 {   20606, .8105012207031E+04, -.1291211059570E+04, .7200441284180E+03},
 {   20607, .8101121093750E+04, -.1158870239258E+04, .7661053466797E+03},
 {   20608, .8099274902344E+04, -.1201157226562E+04, .7619778442383E+03},
 {   20609, .8094035644531E+04, -.1218380249023E+04, .7560866699219E+03},
 {   20610, .8093025878906E+04, -.1180880249023E+04, .7688414916992E+03},
 {   20611, .8111271484375E+04, -.1183460205078E+04, .7615294799805E+03},
 {   20612, .8123380371094E+04, -.1160270263672E+04, .7548442993164E+03},
 {   20613, .8141615234375E+04, -.1189377197266E+04, .7844832763672E+03},
 {   20614, .8128741699219E+04, -.1188177246094E+04, .7519396972656E+03},
 {   20615, .8132484863281E+04, -.1206311279297E+04, .7853868408203E+03},
 {   20616, .8123643066406E+04, -.1205172241211E+04, .7482565307617E+03},
 {   20617, .8116605957031E+04, -.1222549194336E+04, .7449093017578E+03},
 {   20618, .8242058593750E+04, -.1291262084961E+04, .6517556762695E+03},
 {   20619, .8229130859375E+04, -.1412281738281E+04, .6860800170898E+03},
 {   20620, .8244725585938E+04, -.1409021850586E+04, .6484957885742E+03},
 {   20621, .8219431640625E+04, -.1411861816406E+04, .6622189331055E+03},
 {   20622, .8277658203125E+04, -.1410645751953E+04, .7207186279297E+03},
 {   20623, .8266969726562E+04, -.1411939819336E+04, .6920318603516E+03},
 {   20624, .8296468750000E+04, -.1403701782227E+04, .7716434936523E+03},
 {   20625, .8286106445312E+04, -.1409431762695E+04, .7428651733398E+03},
 {   20626, .8284291015625E+04, -.1369966918945E+04, .7143566284180E+03},
 {   20627, .8270948242188E+04, -.1371028930664E+04, .6886805419922E+03},
 {   20628, .8253408203125E+04, -.1411781738281E+04, .7325335693359E+03},
 {   20629, .8241764648438E+04, -.1412582763672E+04, .7060654907227E+03},
 {   20630, .8276159179688E+04, -.1423844726562E+04, .7850525512695E+03},
 {   20631, .8263618164062E+04, -.1424454833984E+04, .7607834472656E+03},
 {   20632, .8253904296875E+04, -.1378865844727E+04, .7281652832031E+03},
 {   20633, .8244424804688E+04, -.1374804931641E+04, .7025410156250E+03},
 {   20634, .8298028320312E+04, -.1365918945312E+04, .7380823364258E+03},
 {   20635, .8259788085938E+04, -.1375567871094E+04, .7537557373047E+03},
 {   20636, .8302254882812E+04, -.1374206909180E+04, .7659451904297E+03},
 {   20637, .8274774414062E+04, -.1395458862305E+04, .7802836914062E+03},
 {   20638, .8251607421875E+04, -.1371808837891E+04, .6532927856445E+03},
 {   20639, .8229412109375E+04, -.1375198852539E+04, .6841643066406E+03},
 {   20640, .8208323242188E+04, -.1407941772461E+04, .6339675292969E+03},
 {   20641, .8161774414062E+04, -.1443694702148E+04, .7413535766602E+03},
 {   20642, .8182754882812E+04, -.1438097778320E+04, .7315166015625E+03},
 {   20643, .8184619628906E+04, -.1398700805664E+04, .7266712646484E+03},
 {   20644, .8165198730469E+04, -.1396191772461E+04, .7358640747070E+03},
 {   20645, .8177538085938E+04, -.1430990722656E+04, .7689982299805E+03},
 {   20646, .8196055664062E+04, -.1426991821289E+04, .7562352905273E+03},
 {   20647, .8197355468750E+04, -.1403785888672E+04, .7518117675781E+03},
 {   20648, .8176807128906E+04, -.1404221801758E+04, .7616022338867E+03},
 {   20649, .8234684570312E+04, -.1392831787109E+04, .7639663085938E+03},
 {   20650, .8233450195312E+04, -.1435054809570E+04, .7688070678711E+03},
 {   20651, .8206071289062E+04, -.1424531738281E+04, .7753648071289E+03},
 {   20652, .8208016601562E+04, -.1403227783203E+04, .7746589355469E+03},
 {   20653, .8225425781250E+04, -.1395505859375E+04, .7409253540039E+03},
 {   20654, .8225118164062E+04, -.1425896728516E+04, .7444207763672E+03},
 {   20655, .8210897460938E+04, -.1401811889648E+04, .7188212890625E+03},
 {   20656, .8212396484375E+04, -.1429681762695E+04, .7207428588867E+03},
 {   20657, .8178479003906E+04, -.1415116821289E+04, .6444986572266E+03},
 {   20658, .8192079101562E+04, -.1431821777344E+04, .6734001464844E+03},
 {   20659, .8202250976562E+04, -.1432241821289E+04, .6970751342773E+03},
 {   20660, .8151955078125E+04, -.1432053710938E+04, .6615416870117E+03},
 {   20661, .8162562011719E+04, -.1439163818359E+04, .6864129638672E+03},
 {   20662, .8156502929688E+04, -.1433836791992E+04, .7825332031250E+03},
 {   20663, .8150164062500E+04, -.1402141845703E+04, .7842259521484E+03},
 {   20664, .8143436035156E+04, -.1443611694336E+04, .7480238647461E+03},
 {   20665, .8138465332031E+04, -.1405666870117E+04, .7492154541016E+03},
 {   20666, .8318598632812E+04, -.1320255004883E+04, .7762677001953E+03},
 {   20667, .8313353515625E+04, -.1348519897461E+04, .7831387329102E+03},
 {   20668, .8309865234375E+04, -.1376312866211E+04, .7896964111328E+03},
 {   20669, .8302314453125E+04, -.1399581787109E+04, .7967549438477E+03},
 {   20670, .8285027343750E+04, -.1412818725586E+04, .8042858886719E+03},
 {   20671, .8262550781250E+04, -.1423056762695E+04, .8139880371094E+03},
 {   20672, .8238361328125E+04, -.1427286743164E+04, .8238776855469E+03},
 {   20673, .8211397460938E+04, -.1420143798828E+04, .8359410400391E+03},
 {   20674, .8193332031250E+04, -.1410942749023E+04, .8455639648438E+03},
 {   20675, .8176021484375E+04, -.1401187866211E+04, .8532178955078E+03},
 {   20676, .8164821289062E+04, -.1383528808594E+04, .8579597167969E+03},
 {   20677, .8154315429688E+04, -.1363615844727E+04, .8616691894531E+03},
 {   20678, .8146219238281E+04, -.1342206909180E+04, .8625835571289E+03},
 {   20679, .8141961914062E+04, -.1319825927734E+04, .8638906250000E+03},
 {   20680, .8139616210938E+04, -.1320022949219E+04, .8363707275391E+03},
 {   20681, .8143405273438E+04, -.1343476928711E+04, .8352714843750E+03},
 {   20682, .8150654296875E+04, -.1366050903320E+04, .8368131713867E+03},
 {   20683, .8161256347656E+04, -.1385411865234E+04, .8332422485352E+03},
 {   20684, .8170932617188E+04, -.1403330810547E+04, .8289495239258E+03},
 {   20685, .8186535644531E+04, -.1412971801758E+04, .8212151489258E+03},
 {   20686, .8199300781250E+04, -.1424309814453E+04, .8146037597656E+03},
 {   20687, .8309138671875E+04, -.1319303955078E+04, .7527840576172E+03},
 {   20688, .8305178710938E+04, -.1346852905273E+04, .7583654785156E+03},
 {   20689, .8254387695312E+04, -.1434278808594E+04, .7944768066406E+03},
 {   20690, .8218632812500E+04, -.1431541748047E+04, .8028565063477E+03},
 {   20691, .8125747558594E+04, -.1320172973633E+04, .8077823486328E+03},
 {   20692, .8133544921875E+04, -.1343663940430E+04, .8083259277344E+03},
 {   20693, .8142414062500E+04, -.1366607910156E+04, .8111502685547E+03},
 {   20694, .8155354003906E+04, -.1387358886719E+04, .8090092163086E+03},
 {   20695, .8162221191406E+04, -.1403206787109E+04, .8057008056641E+03},
 {   20696, .8172309082031E+04, -.1417680786133E+04, .7998519897461E+03},
 {   20697, .8186217773438E+04, -.1423370727539E+04, .7876194458008E+03},
 {   20698, .8219167968750E+04, -.1420730834961E+04, .8009004516602E+03},
 {   20699, .8197923828125E+04, -.1415425781250E+04, .8129661254883E+03},
 {   20700, .8186321777344E+04, -.1405595825195E+04, .7887932739258E+03},
 {   20701, .8253567382812E+04, -.1411771850586E+04, .7897035522461E+03},
 {   20702, .8304916015625E+04, -.1317464965820E+04, .7310228271484E+03},
 {   20703, .8301374023438E+04, -.1343475952148E+04, .7342466430664E+03},
 {   20704, .8302243164062E+04, -.1314966064453E+04, .7069771118164E+03},
 {   20705, .8282062500000E+04, -.1315844970703E+04, .6784992675781E+03},
 {   20706, .8217918945312E+04, -.1316348999023E+04, .6258404541016E+03},
 {   20707, .8176426757812E+04, -.1322812011719E+04, .6288469848633E+03},
 {   20708, .8144722656250E+04, -.1324439941406E+04, .6400582275391E+03},
 {   20709, .8123346191406E+04, -.1327111938477E+04, .6601043090820E+03},
 {   20710, .8107334472656E+04, -.1327777954102E+04, .6850720825195E+03},
 {   20711, .8113750976562E+04, -.1322728027344E+04, .7765959472656E+03},
 {   20712, .8123708007812E+04, -.1343809936523E+04, .7800521240234E+03},
 {   20713, .8288031250000E+04, -.1339901000977E+04, .7098858032227E+03},
 {   20714, .8275713867188E+04, -.1342770996094E+04, .6856658325195E+03},
 {   20715, .8245032226562E+04, -.1316964965820E+04, .6499033203125E+03},
 {   20716, .8252852539062E+04, -.1344146972656E+04, .6539686279297E+03},
 {   20717, .8200662109375E+04, -.1393433837891E+04, .6954318237305E+03},
 {   20718, .8191009765625E+04, -.1376973876953E+04, .6726956176758E+03},
 {   20719, .8218980468750E+04, -.1369808837891E+04, .6611552734375E+03},
 {   20720, .8207239257812E+04, -.1373434936523E+04, .6285049438477E+03},
 {   20721, .8175591796875E+04, -.1373939941406E+04, .6368486938477E+03},
 {   20722, .8215752929688E+04, -.1344882934570E+04, .6266950683594E+03},
 {   20723, .8227889648438E+04, -.1343421997070E+04, .6590971679688E+03},
 {   20724, .8172699218750E+04, -.1349701904297E+04, .6283753051758E+03},
 {   20725, .8142092285156E+04, -.1348240966797E+04, .6362024536133E+03},
 {   20726, .8162013183594E+04, -.1388869873047E+04, .6849189453125E+03},
 {   20727, .8173476562500E+04, -.1391692871094E+04, .7060339355469E+03},
 {   20728, .8169992187500E+04, -.1440310791016E+04, .7081137695312E+03},
 {   20729, .8142997558594E+04, -.1370252929688E+04, .6435178222656E+03},
 {   20730, .8153583984375E+04, -.1444823730469E+04, .7179714965820E+03},
 {   20731, .8145395019531E+04, -.1444950683594E+04, .6945888671875E+03},
 {   20732, .8084936523438E+04, -.1434381713867E+04, .7158574218750E+03},
 {   20733, .8093581054688E+04, -.1437834716797E+04, .7399722290039E+03},
 {   20734, .8075840820312E+04, -.1423920776367E+04, .6933887329102E+03},
 {   20735, .8120189941406E+04, -.1443008789062E+04, .7258135375977E+03},
 {   20736, .8132561035156E+04, -.1443506713867E+04, .7498334960938E+03},
 {   20737, .8102441406250E+04, -.1429401733398E+04, .6767194213867E+03},
 {   20738, .8112231445312E+04, -.1436810791016E+04, .7007926635742E+03},
 {   20739, .8104046875000E+04, -.1433771728516E+04, .7637159423828E+03},
 {   20740, .8133963867188E+04, -.1443571777344E+04, .7203406982422E+03},
 {   20741, .8126419433594E+04, -.1436321777344E+04, .6945307006836E+03},
 {   20742, .8134878906250E+04, -.1437539794922E+04, .6686831054688E+03},
 {   20743, .8069629882812E+04, -.1365837890625E+04, .6923321533203E+03},
 {   20744, .8093988769531E+04, -.1366112915039E+04, .6822908935547E+03},
 {   20745, .8099209960938E+04, -.1346745971680E+04, .6863268432617E+03},
 {   20746, .8073770019531E+04, -.1348127929688E+04, .6983433227539E+03},
 {   20747, .8119243164062E+04, -.1368852905273E+04, .6638703002930E+03},
 {   20748, .8119687988281E+04, -.1348079956055E+04, .6605174560547E+03},
 {   20749, .8067796386719E+04, -.1388380859375E+04, .6891648559570E+03},
 {   20750, .8094521484375E+04, -.1398084838867E+04, .6798299560547E+03},
 {   20751, .8120463378906E+04, -.1412475830078E+04, .6684019775391E+03},
 {   20752, .8078841796875E+04, -.1327765014648E+04, .7049028320312E+03},
 {   20753, .8163018554688E+04, -.1370808837891E+04, .6838976440430E+03},
 {   20754, .8143433105469E+04, -.1402626831055E+04, .6506860351562E+03},
 {   20755, .8105973632812E+04, -.1338646972656E+04, .7411036376953E+03},
 {   20756, .8086023925781E+04, -.1343535888672E+04, .7475635986328E+03},
 {   20757, .8106452148438E+04, -.1333287963867E+04, .7228034057617E+03},
 {   20758, .8083643066406E+04, -.1336734008789E+04, .7319744262695E+03},
 {   20759, .8101121093750E+04, -.1417731811523E+04, .7661053466797E+03},
 {   20760, .8099274902344E+04, -.1375444824219E+04, .7619778442383E+03},
 {   20761, .8094035644531E+04, -.1358221923828E+04, .7560866699219E+03},
 {   20762, .8093025878906E+04, -.1395721801758E+04, .7688414916992E+03},
 {   20763, .8111271484375E+04, -.1393141845703E+04, .7615294799805E+03},
 {   20764, .8123380371094E+04, -.1416331787109E+04, .7548442993164E+03},
 {   20765, .8128741699219E+04, -.1388424804688E+04, .7519396972656E+03},
 {   20766, .8141615234375E+04, -.1387224853516E+04, .7844833984375E+03},
 {   20767, .8132484863281E+04, -.1370290893555E+04, .7853868408203E+03},
 {   20768, .8123643066406E+04, -.1371429931641E+04, .7482565307617E+03},
 {   20769, .8116605957031E+04, -.1354052978516E+04, .7449093017578E+03},
 {   20770, .8186063964844E+04, -.1384682861328E+04, .7373969116211E+03},
 {   20771, .8175565917969E+04, -.1380919921875E+04, .7392689819336E+03},
 {   20772, .8175348144531E+04, -.1376617919922E+04, .7319461669922E+03},
 {   20773, .8183397949219E+04, -.1379088867188E+04, .7297929077148E+03},
 {   20774, .8165769042969E+04, -.1376110839844E+04, .7387557983398E+03},
 {   20775, .8167691894531E+04, -.1372699951172E+04, .7312588500977E+03},
 {   20776, .8157501464844E+04, -.1370207885742E+04, .7350997924805E+03},
 {   20777, .8161599121094E+04, -.1367926879883E+04, .7278294677734E+03},
 {   20778, .8206590820312E+04, -.1374015869141E+04, .7023166503906E+03},
 {   20779, .8200710937500E+04, -.1367496826172E+04, .6956728515625E+03},
 {   20780, .8195048828125E+04, -.1366772949219E+04, .6989555053711E+03},
 {   20781, .8198257812500E+04, -.1371542846680E+04, .7046967773438E+03},
 {   20782, .8193001953125E+04, -.1361314941406E+04, .6910818481445E+03},
 {   20783, .8189380859375E+04, -.1362037841797E+04, .6952495117188E+03},
 {   20784, .8150731445312E+04, -.1363774902344E+04, .7293173828125E+03},
 {   20785, .8151365722656E+04, -.1356544921875E+04, .7161989135742E+03},
 {   20786, .8158027343750E+04, -.1357258911133E+04, .7120952148438E+03},
 {   20787, .8157970703125E+04, -.1363015869141E+04, .7221807861328E+03},
 {   20788, .8158225585938E+04, -.1351025878906E+04, .7010650634766E+03},
 {   20789, .8164023437500E+04, -.1354403930664E+04, .7022996826172E+03},
 {   20790, .8172843750000E+04, -.1351426879883E+04, .6900249633789E+03},
 {   20791, .8174180175781E+04, -.1355300903320E+04, .6957087402344E+03},
 {   20792, .8183390136719E+04, -.1355949951172E+04, .6894450073242E+03},
 {   20793, .8182165527344E+04, -.1357877929688E+04, .6937914428711E+03},
 {   20794, .8194929687500E+04, -.1384562866211E+04, .7300856323242E+03},
 {   20795, .8190524902344E+04, -.1379751831055E+04, .7252199707031E+03},
 {   20796, .8201772460938E+04, -.1382842895508E+04, .7215860595703E+03},
 {   20797, .8195831054688E+04, -.1378555908203E+04, .7188729858398E+03},
 {   20798, .8205461914062E+04, -.1379098876953E+04, .7120859375000E+03},
 {   20799, .8198541015625E+04, -.1375679931641E+04, .7116846313477E+03},
 {   20800, .8277112304688E+04, -.1332063964844E+04, .7090800170898E+03},
 {   20801, .8251115234375E+04, -.1335054931641E+04, .7208850708008E+03},
 {   20802, .8263960937500E+04, -.1342686889648E+04, .7365703735352E+03},
 {   20803, .8286263671875E+04, -.1335295898438E+04, .7233074951172E+03},
 {   20804, .8226044921875E+04, -.1342637939453E+04, .7323002929688E+03},
 {   20805, .8239477539062E+04, -.1353475952148E+04, .7476414794922E+03},
 {   20806, .8199932617188E+04, -.1343979003906E+04, .7441720581055E+03},
 {   20807, .8212415039062E+04, -.1357213867188E+04, .7587783813477E+03},
 {   20808, .8173917480469E+04, -.1346528930664E+04, .7560013427734E+03},
 {   20809, .8184838867188E+04, -.1359824951172E+04, .7691708984375E+03},
 {   20810, .8154065429688E+04, -.1363937866211E+04, .7650286865234E+03},
 {   20811, .8162509277344E+04, -.1372399902344E+04, .7770828247070E+03},
 {   20812, .8275943359375E+04, -.1349790893555E+04, .7529920654297E+03},
 {   20813, .8294713867188E+04, -.1338427978516E+04, .7379376220703E+03},
 {   20814, .8251888671875E+04, -.1363356933594E+04, .7640485839844E+03},
 {   20815, .8223711914062E+04, -.1369062866211E+04, .7747803955078E+03},
 {   20816, .8194413085938E+04, -.1371307861328E+04, .7840688476562E+03},
 {   20817, .8169445312500E+04, -.1378619873047E+04, .7911956176758E+03},
 {   20818, .8286773437500E+04, -.1356946899414E+04, .7699930419922E+03},
 {   20819, .8301880859375E+04, -.1342110961914E+04, .7530357666016E+03},
 {   20820, .8263277343750E+04, -.1372790893555E+04, .7811513671875E+03},
 {   20821, .8234124023438E+04, -.1379965820312E+04, .7915908203125E+03},
 {   20822, .8203229492188E+04, -.1381347900391E+04, .7998900756836E+03},
 {   20823, .8175760742188E+04, -.1382896850586E+04, .8063468627930E+03},
 {   20824, .8297159179688E+04, -.1366374877930E+04, .7860485839844E+03},
 {   20825, .8307330078125E+04, -.1347970947266E+04, .7680354003906E+03},
 {   20826, .8275493164062E+04, -.1384594848633E+04, .7964600219727E+03},
 {   20827, .8246632812500E+04, -.1393336791992E+04, .8057575683594E+03},
 {   20828, .8215422851562E+04, -.1393950805664E+04, .8122184448242E+03},
 {   20829, .8186716308594E+04, -.1389834838867E+04, .8171554565430E+03},
 {   20830, .8307139648438E+04, -.1377578857422E+04, .8005953979492E+03},
 {   20831, .8311535156250E+04, -.1355522949219E+04, .7826186523438E+03},
 {   20832, .8288145507812E+04, -.1398256835938E+04, .8091693115234E+03},
 {   20833, .8260419921875E+04, -.1408646850586E+04, .8162360229492E+03},
 {   20834, .8229728515625E+04, -.1408575805664E+04, .8197682495117E+03},
 {   20835, .8200629882812E+04, -.1398877807617E+04, .8220960083008E+03},
 {   20836, .8278287109375E+04, -.1346021972656E+04, .7085454711914E+03},
 {   20837, .8252511718750E+04, -.1355544921875E+04, .7202662353516E+03},
 {   20838, .8263031250000E+04, -.1361259887695E+04, .7347456665039E+03},
 {   20839, .8287476562500E+04, -.1348513916016E+04, .7235904541016E+03},
 {   20840, .8230730468750E+04, -.1372539916992E+04, .7301691894531E+03},
 {   20841, .8240253906250E+04, -.1378505859375E+04, .7443463134766E+03},
 {   20842, .8208225585938E+04, -.1388190795898E+04, .7404027709961E+03},
 {   20843, .8215916015625E+04, -.1392218872070E+04, .7539318847656E+03},
 {   20844, .8273569335938E+04, -.1367000854492E+04, .7492122802734E+03},
 {   20845, .8296783203125E+04, -.1351022949219E+04, .7385662231445E+03},
 {   20846, .8249687500000E+04, -.1384507812500E+04, .7585656127930E+03},
 {   20847, .8223418945312E+04, -.1396291870117E+04, .7675552978516E+03},
 {   20848, .8282231445312E+04, -.1375043945312E+04, .7638911132812E+03},
 {   20849, .8303950195312E+04, -.1356380859375E+04, .7537130737305E+03},
 {   20850, .8257513671875E+04, -.1392264892578E+04, .7730393066406E+03},
 {   20851, .8229577148438E+04, -.1401574829102E+04, .7814743652344E+03},
 {   20852, .8194417968750E+04, -.1394389892578E+04, .7764907226562E+03},
 {   20853, .8199848632812E+04, -.1398720825195E+04, .7887814941406E+03},
 {   20854, .8167017578125E+04, -.1385130859375E+04, .7811198730469E+03},
 {   20855, .8171786132812E+04, -.1389283813477E+04, .7899661865234E+03},
 {   20856, .8286655273438E+04, -.1387282836914E+04, .7772395629883E+03},
 {   20857, .8306030273438E+04, -.1366960937500E+04, .7671146240234E+03},
 {   20858, .8261954101562E+04, -.1403190795898E+04, .7865935058594E+03},
 {   20859, .8233202148438E+04, -.1408999755859E+04, .7948900146484E+03},
 {   20860, .8203588867188E+04, -.1404169799805E+04, .8009845581055E+03},
 {   20861, .8175709472656E+04, -.1393525878906E+04, .7991376342773E+03},
 {   20862, .8286773437500E+04, -.1402466796875E+04, .7872306518555E+03},
 {   20863, .8302833007812E+04, -.1381213867188E+04, .7762851562500E+03},
 {   20864, .8263048828125E+04, -.1416334838867E+04, .7976661987305E+03},
 {   20865, .8234448242188E+04, -.1417912841797E+04, .8067000732422E+03},
 {   20866, .8205921875000E+04, -.1410377807617E+04, .8124531250000E+03},
 {   20867, .8179196777344E+04, -.1397800781250E+04, .8084513549805E+03},
 {   20868, .8172387695312E+04, -.1344694946289E+04, .7514581298828E+03},
 {   20869, .8177559570312E+04, -.1325291015625E+04, .7491054687500E+03},
 {   20870, .8196837890625E+04, -.1323455932617E+04, .7403408813477E+03},
 {   20871, .8192716796875E+04, -.1341984985352E+04, .7422152099609E+03},
 {   20872, .8181235839844E+04, -.1305701049805E+04, .7474347534180E+03},
 {   20873, .8199768554688E+04, -.1304774047852E+04, .7390083618164E+03},
 {   20874, .8180805664062E+04, -.1285593017578E+04, .7476303100586E+03},
 {   20875, .8199407226562E+04, -.1285682006836E+04, .7391718750000E+03},
 {   20876, .8216195312500E+04, -.1322471923828E+04, .7315404052734E+03},
 {   20877, .8213155273438E+04, -.1340554931641E+04, .7329217529297E+03},
 {   20878, .8218333984375E+04, -.1304276977539E+04, .7305649414062E+03},
 {   20879, .8218009765625E+04, -.1285772094727E+04, .7307128906250E+03},
 {   20880, .8235407226562E+04, -.1320542968750E+04, .7228040771484E+03},
 {   20881, .8233381835938E+04, -.1337705932617E+04, .7237234497070E+03},
 {   20882, .8236830078125E+04, -.1303305053711E+04, .7221567382812E+03},
 {   20883, .8236615234375E+04, -.1285861083984E+04, .7222545776367E+03},
 {   20884, .8254508789062E+04, -.1317693969727E+04, .7141187744141E+03},
 {   20885, .8253444335938E+04, -.1333476928711E+04, .7146024780273E+03},
 {   20886, .8255273437500E+04, -.1301874023438E+04, .7137716674805E+03},
 {   20887, .8255216796875E+04, -.1285950073242E+04, .7137970581055E+03},
 {   20888, .8273779296875E+04, -.1315788940430E+04, .7053568115234E+03},
 {   20889, .8273758789062E+04, -.1330661987305E+04, .7053660278320E+03},
 {   20890, .8273801757812E+04, -.1300915039062E+04, .7053471679688E+03},
 {   20891, .8273823242188E+04, -.1286039062500E+04, .7053381958008E+03},
 {   20892, .8172448242188E+04, -.1285775024414E+04, .7357145996094E+03},
 {   20893, .8172753417969E+04, -.1301901977539E+04, .7361098632812E+03},
 {   20894, .8164089843750E+04, -.1285955078125E+04, .7237979736328E+03},
 {   20895, .8164104492188E+04, -.1298718017578E+04, .7242916870117E+03},
 {   20896, .8155731445312E+04, -.1285942016602E+04, .7118833618164E+03},
 {   20897, .8155594238281E+04, -.1295731079102E+04, .7122749633789E+03},
 {   20898, .8147375976562E+04, -.1285761108398E+04, .6999666748047E+03},
 {   20899, .8147248535156E+04, -.1292978027344E+04, .7000240478516E+03},
 {   20900, .8169981445312E+04, -.1317640991211E+04, .7379053955078E+03},
 {   20901, .8162063964844E+04, -.1311223999023E+04, .7257190551758E+03},
 {   20902, .8154425781250E+04, -.1305195068359E+04, .7131364746094E+03},
 {   20903, .8147126953125E+04, -.1299635986328E+04, .7000797119141E+03},
 {   20904, .8166523925781E+04, -.1333856933594E+04, .7410266113281E+03},
 {   20905, .8159710937500E+04, -.1324266967773E+04, .7304432983398E+03},
 {   20906, .8152884765625E+04, -.1314614990234E+04, .7141671752930E+03},
 {   20907, .8147001953125E+04, -.1306295043945E+04, .7001376342773E+03},
 {   20908, .8139208496094E+04, -.1310880004883E+04, .7036795654297E+03},
 {   20909, .8142396484375E+04, -.1317112060547E+04, .7207628173828E+03},
 {   20910, .8132236816406E+04, -.1304368041992E+04, .7068496704102E+03},
 {   20911, .8133008789062E+04, -.1309320068359E+04, .7252288818359E+03},
 {   20912, .8127854980469E+04, -.1295791015625E+04, .7088419799805E+03},
 {   20913, .8126579589844E+04, -.1298701049805E+04, .7283489379883E+03},
 {   20914, .8125875976562E+04, -.1285761108398E+04, .7097424316406E+03},
 {   20915, .8124259277344E+04, -.1286319091797E+04, .7296009521484E+03},
 {   20916, .8144932617188E+04, -.1323343994141E+04, .7377474975586E+03},
 {   20917, .8132478515625E+04, -.1314272949219E+04, .7434105834961E+03},
 {   20918, .8123354492188E+04, -.1301611083984E+04, .7475582275391E+03},
 {   20919, .8120041015625E+04, -.1286319091797E+04, .7490656738281E+03},
 {   20920, .8114130371094E+04, -.1286319091797E+04, .7606467895508E+03},
 {   20921, .8118905273438E+04, -.1303955078125E+04, .7594177856445E+03},
 {   20922, .8129255859375E+04, -.1318276000977E+04, .7534525756836E+03},
 {   20923, .8141380859375E+04, -.1332232910156E+04, .7506162719727E+03},
 {   20924, .8150626464844E+04, -.1341847900391E+04, .7482562866211E+03},
 {   20925, .8152568359375E+04, -.1359360961914E+04, .7604696044922E+03},
 {   20926, .8137552734375E+04, -.1294385009766E+04, .7044328002930E+03},
 {   20927, .8136623535156E+04, -.1285761108398E+04, .7048540649414E+03},
 {   20928, .8139680664062E+04, -.1302000976562E+04, .7034652099609E+03},
 {   20929, .8116321777344E+04, -.1303927001953E+04, .7140856323242E+03},
 {   20930, .8114823730469E+04, -.1305632080078E+04, .7247462768555E+03},
 {   20931, .8126443359375E+04, -.1313671997070E+04, .7191015014648E+03},
 {   20932, .8128666992188E+04, -.1311672973633E+04, .7084731445312E+03},
 {   20933, .8113332031250E+04, -.1307334960938E+04, .7354085693359E+03},
 {   20934, .8124671875000E+04, -.1315876953125E+04, .7297203979492E+03},
 {   20935, .8111832031250E+04, -.1309041015625E+04, .7460688476562E+03},
 {   20936, .8124063476562E+04, -.1318606933594E+04, .7403229980469E+03},
 {   20937, .8110339355469E+04, -.1310744995117E+04, .7567297363281E+03},
 {   20938, .8124317382812E+04, -.1321484008789E+04, .7504877929688E+03},
 {   20939, .8136374023438E+04, -.1323454956055E+04, .7140721435547E+03},
 {   20940, .8140204101562E+04, -.1320593017578E+04, .7032286376953E+03},
 {   20941, .8133442871094E+04, -.1326723022461E+04, .7249006958008E+03},
 {   20942, .8132844726562E+04, -.1331042968750E+04, .7356905517578E+03},
 {   20943, .8133968750000E+04, -.1335656982422E+04, .7456118774414E+03},
 {   20944, .8142815429688E+04, -.1335880981445E+04, .7107568969727E+03},
 {   20945, .8147961425781E+04, -.1332353027344E+04, .6997733154297E+03},
 {   20946, .8139018554688E+04, -.1340019897461E+04, .7217141113281E+03},
 {   20947, .8138722656250E+04, -.1345735961914E+04, .7326141967773E+03},
 {   20948, .8141005859375E+04, -.1351892944336E+04, .7422111816406E+03},
 {   20949, .8146493652344E+04, -.1350245971680E+04, .7091284790039E+03},
 {   20950, .8152287597656E+04, -.1346506958008E+04, .6979512329102E+03},
 {   20951, .8142502441406E+04, -.1354799926758E+04, .7202741088867E+03},
 {   20952, .8143177734375E+04, -.1361455932617E+04, .7313402099609E+03},
 {   20953, .8147291992188E+04, -.1368698852539E+04, .7406677856445E+03},
 {   20954, .8160131347656E+04, -.1346931884766E+04, .6923960571289E+03},
 {   20955, .8156231933594E+04, -.1335475952148E+04, .6937069702148E+03},
 {   20956, .8167973632812E+04, -.1347357910156E+04, .6868418579102E+03},
 {   20957, .8165055664062E+04, -.1339893920898E+04, .6874038696289E+03},
 {   20958, .8176445800781E+04, -.1349394897461E+04, .6835930786133E+03},
 {   20959, .8174377929688E+04, -.1344713989258E+04, .6821699218750E+03},
 {   20960, .8185429687500E+04, -.1352750976562E+04, .6822296142578E+03},
 {   20961, .8184599121094E+04, -.1348797973633E+04, .6775863647461E+03},
 {   20962, .8151944335938E+04, -.1328365966797E+04, .6874110107422E+03},
 {   20963, .8142226562500E+04, -.1321094970703E+04, .6918300781250E+03},
 {   20964, .8161877441406E+04, -.1335324951172E+04, .6828950805664E+03},
 {   20965, .8172180175781E+04, -.1341481933594E+04, .6782106933594E+03},
 {   20966, .8183763183594E+04, -.1344844970703E+04, .6729430541992E+03},
 {   20967, .8172023437500E+04, -.1340784912109E+04, .6689241333008E+03},
 {   20968, .8159151367188E+04, -.1335255981445E+04, .6707423706055E+03},
 {   20969, .8171986816406E+04, -.1340131958008E+04, .6595727539062E+03},
 {   20970, .8156660644531E+04, -.1335281982422E+04, .6584663085938E+03},
 {   20971, .8172648437500E+04, -.1339829956055E+04, .6500723876953E+03},
 {   20972, .8155564941406E+04, -.1336010986328E+04, .6458881835938E+03},
 {   20973, .8148064941406E+04, -.1327916992188E+04, .6774395141602E+03},
 {   20974, .8144539550781E+04, -.1327611938477E+04, .6672811279297E+03},
 {   20975, .8143108886719E+04, -.1328361938477E+04, .6566714477539E+03},
 {   20976, .8138077148438E+04, -.1319814941406E+04, .6844545288086E+03},
 {   20977, .8134402343750E+04, -.1318724975586E+04, .6768302612305E+03},
 {   20978, .8133517578125E+04, -.1319042968750E+04, .6686037597656E+03},
 {   20979, .8117830566406E+04, -.1301681030273E+04, .6923274536133E+03},
 {   20980, .8117564941406E+04, -.1303098999023E+04, .6976835327148E+03},
 {   20981, .8122975097656E+04, -.1308342041016E+04, .6907297973633E+03},
 {   20982, .8122546386719E+04, -.1307702026367E+04, .6842880249023E+03},
 {   20983, .8117297363281E+04, -.1304515014648E+04, .7030380249023E+03},
 {   20984, .8124349609375E+04, -.1309460083008E+04, .6969580688477E+03},
 {   20985, .8116811035156E+04, -.1304242065430E+04, .7085606079102E+03},
 {   20986, .8125653808594E+04, -.1309629028320E+04, .7032150268555E+03},
 {   20987, .8128495605469E+04, -.1313640991211E+04, .6838848876953E+03},
 {   20988, .8127560546875E+04, -.1313586059570E+04, .6763251342773E+03},
 {   20989, .8131334960938E+04, -.1314650024414E+04, .6910197753906E+03},
 {   20990, .8134247558594E+04, -.1315453002930E+04, .6980351562500E+03},
 {   20991, .8204300781250E+04, -.1333596923828E+04, .6682874755859E+03},
 {   20992, .8198187500000E+04, -.1329396972656E+04, .6550922851562E+03},
 {   20993, .8188312500000E+04, -.1337942016602E+04, .6590733032227E+03},
 {   20994, .8198561523438E+04, -.1340467895508E+04, .6691942749023E+03},
 {   20995, .8220354492188E+04, -.1376836914062E+04, .7094531860352E+03},
 {   20996, .8225319335938E+04, -.1369880859375E+04, .7064360961914E+03},
 {   20997, .8220309570312E+04, -.1362240966797E+04, .6946868896484E+03},
 {   20998, .8216541015625E+04, -.1370154907227E+04, .6980163574219E+03},
 {   20999, .8215532226562E+04, -.1350831909180E+04, .6867396850586E+03},
 {   21000, .8211167968750E+04, -.1361208984375E+04, .6892119750977E+03},
 {   21001, .8212079101562E+04, -.1337927978516E+04, .6805864868164E+03},
 {   21002, .8205010742188E+04, -.1351414916992E+04, .6823051147461E+03},
 {   21003, .8195573242188E+04, -.1346071899414E+04, .6735119628906E+03},
 {   21004, .8194292968750E+04, -.1351508911133E+04, .6789687500000E+03},
 {   21005, .8215388671875E+04, -.1383792846680E+04, .7124692382812E+03},
 {   21006, .8215023437500E+04, -.1378025878906E+04, .7025761718750E+03},
 {   21007, .8211306640625E+04, -.1371500854492E+04, .6941458740234E+03},
 {   21008, .8204701171875E+04, -.1364773925781E+04, .6877177734375E+03},
 {   21009, .8195860351562E+04, -.1358374877930E+04, .6836594848633E+03},
 {   21010, .8249481445312E+04, -.1344784912109E+04, .6954486083984E+03},
 {   21011, .8243709960938E+04, -.1329963989258E+04, .6878003540039E+03},
 {   21012, .8236002929688E+04, -.1341763916016E+04, .6923127441406E+03},
 {   21013, .8240119140625E+04, -.1351465942383E+04, .6997070312500E+03},
 {   21014, .8238277343750E+04, -.1328231933594E+04, .6770396728516E+03},
 {   21015, .8231233398438E+04, -.1334716918945E+04, .6835336303711E+03},
 {   21016, .8232844726562E+04, -.1326501953125E+04, .6662781372070E+03},
 {   21017, .8226504882812E+04, -.1329301025391E+04, .6739068603516E+03},
 {   21018, .8227413085938E+04, -.1324771972656E+04, .6555155029297E+03},
 {   21019, .8220340820312E+04, -.1326740966797E+04, .6622328491211E+03},
 {   21020, .8221980468750E+04, -.1323041015625E+04, .6447543334961E+03},
 {   21021, .8214049804688E+04, -.1325160034180E+04, .6482001953125E+03},
 {   21022, .8228232421875E+04, -.1351900878906E+04, .6937235717773E+03},
 {   21023, .8232719726562E+04, -.1360673950195E+04, .7030719604492E+03},
 {   21024, .8223534179688E+04, -.1342569946289E+04, .6855847778320E+03},
 {   21025, .8219515625000E+04, -.1333308959961E+04, .6779205932617E+03},
 {   21026, .8212619140625E+04, -.1329761962891E+04, .6661579589844E+03},
 {   21027, .8206120117188E+04, -.1327277954102E+04, .6516456298828E+03},
 {   21028, .8248219726562E+04, -.1284527099609E+04, .6959982910156E+03},
 {   21029, .8243403320312E+04, -.1284467041016E+04, .6877149658203E+03},
 {   21030, .8243482421875E+04, -.1295842041016E+04, .6877365112305E+03},
 {   21031, .8248850585938E+04, -.1295579101562E+04, .6957232055664E+03},
 {   21032, .8243557617188E+04, -.1307216064453E+04, .6877569580078E+03},
 {   21033, .8248850585938E+04, -.1314656005859E+04, .6957232055664E+03},
 {   21034, .8249166015625E+04, -.1329719970703E+04, .6955878295898E+03},
 {   21035, .8243522460938E+04, -.1315094970703E+04, .6859263916016E+03},
 {   21036, .8238192382812E+04, -.1315534057617E+04, .6761312255859E+03},
 {   21037, .8232863281250E+04, -.1315973022461E+04, .6663343505859E+03},
 {   21038, .8227480468750E+04, -.1314662963867E+04, .6556113281250E+03},
 {   21039, .8222094726562E+04, -.1313354980469E+04, .6448903198242E+03},
 {   21040, .8232883789062E+04, -.1305441040039E+04, .6663911132812E+03},
 {   21041, .8238220703125E+04, -.1306328002930E+04, .6770753784180E+03},
 {   21042, .8227546875000E+04, -.1304555053711E+04, .6557096557617E+03},
 {   21043, .8222208984375E+04, -.1303667968750E+04, .6450253906250E+03},
 {   21044, .8238191406250E+04, -.1295376098633E+04, .6770921020508E+03},
 {   21045, .8232902343750E+04, -.1294911987305E+04, .6664494018555E+03},
 {   21046, .8227613281250E+04, -.1294446044922E+04, .6558059082031E+03},
 {   21047, .8222323242188E+04, -.1293981079102E+04, .6451630249023E+03},
 {   21048, .8238164062500E+04, -.1284423095703E+04, .6771107788086E+03},
 {   21049, .8232919921875E+04, -.1284380004883E+04, .6665061645508E+03},
 {   21050, .8227680664062E+04, -.1284337036133E+04, .6559020385742E+03},
 {   21051, .8222442382812E+04, -.1284294067383E+04, .6452974853516E+03},
 {   21052, .8265291992188E+04, -.1305648071289E+04, .6871866455078E+03},
 {   21053, .8265128906250E+04, -.1296046020508E+04, .6872610473633E+03},
 {   21054, .8264961914062E+04, -.1286445068359E+04, .6873349609375E+03},
 {   21055, .8261511718750E+04, -.1339416015625E+04, .6899806518555E+03},
 {   21056, .8265458984375E+04, -.1315250000000E+04, .6871114501953E+03},
 {   21057, .8220820312500E+04, -.1378845825195E+04, .7211480712891E+03},
 {   21058, .8210798828125E+04, -.1389116821289E+04, .7252733764648E+03},
 {   21059, .8230836914062E+04, -.1368571899414E+04, .7170238037109E+03},
 {   21060, .8248171875000E+04, -.1352317871094E+04, .7091412353516E+03},
 {   21061, .8239505859375E+04, -.1360444946289E+04, .7130820922852E+03},
 {   21062, .8258990234375E+04, -.1347539916992E+04, .7042243652344E+03},
 {   21063, .8269805664062E+04, -.1342760009766E+04, .6993051757812E+03},
 {   21064, .8109145996094E+04, -.1302603027344E+04, .7563250732422E+03},
 {   21065, .8110358398438E+04, -.1301458984375E+04, .7451619262695E+03},
 {   21066, .8106760742188E+04, -.1286319091797E+04, .7555126342773E+03},
 {   21067, .8107390625000E+04, -.1286319091797E+04, .7434874267578E+03},
 {   21068, .8111862792969E+04, -.1300315063477E+04, .7340247802734E+03},
 {   21069, .8108895019531E+04, -.1286319091797E+04, .7315358886719E+03},
 {   21070, .8113694824219E+04, -.1299171997070E+04, .7229324951172E+03},
 {   21071, .8111381347656E+04, -.1286319091797E+04, .7197265014648E+03},
 {   21072, .8115609863281E+04, -.1298069091797E+04, .7118594970703E+03},
 {   21073, .8114142089844E+04, -.1285761108398E+04, .7079754638672E+03},
 {   21074, .8116949707031E+04, -.1298248046875E+04, .7064885864258E+03},
 {   21075, .8118286621094E+04, -.1298425048828E+04, .7011169433594E+03},
 {   21076, .8120243652344E+04, -.1286319091797E+04, .6975540771484E+03},
 {   21077, .8117194335938E+04, -.1286319091797E+04, .7027645263672E+03},
 {   21078, .8130959472656E+04, -.1286975097656E+04, .6914418334961E+03},
 {   21079, .8120246582031E+04, -.1297603027344E+04, .6965760498047E+03},
 {   21080, .8125604003906E+04, -.1286647094727E+04, .6944981689453E+03},
 {   21081, .8206409179688E+04, -.1316785034180E+04, .6520693969727E+03},
 {   21082, .8198567382812E+04, -.1318500976562E+04, .6556586303711E+03},
 {   21083, .8214252929688E+04, -.1315069946289E+04, .6484793090820E+03},
 {   21084, .8206701171875E+04, -.1306292968750E+04, .6524926757812E+03},
 {   21085, .8198944335938E+04, -.1307604980469E+04, .6562263183594E+03},
 {   21086, .8214453125000E+04, -.1304979980469E+04, .6487585449219E+03},
 {   21087, .8206992187500E+04, -.1295799072266E+04, .6529145507812E+03},
 {   21088, .8199321289062E+04, -.1296708984375E+04, .6567919311523E+03},
 {   21089, .8214657226562E+04, -.1294890991211E+04, .6490386962891E+03},
 {   21090, .8207280273438E+04, -.1285307006836E+04, .6533392333984E+03},
 {   21091, .8199702148438E+04, -.1285812011719E+04, .6573579101562E+03},
 {   21092, .8214861328125E+04, -.1284800048828E+04, .6493182983398E+03},
 {   21093, .8176499023438E+04, -.1281461059570E+04, .6655518798828E+03},
 {   21094, .8188103027344E+04, -.1283637084961E+04, .6614547729492E+03},
 {   21095, .8187600097656E+04, -.1296038085938E+04, .6611767578125E+03},
 {   21096, .8175873535156E+04, -.1295366088867E+04, .6655596923828E+03},
 {   21097, .8186604980469E+04, -.1308393066406E+04, .6612413330078E+03},
 {   21098, .8174132812500E+04, -.1310221069336E+04, .6663188476562E+03},
 {   21099, .8185411132812E+04, -.1318297973633E+04, .6558381347656E+03},
 {   21100, .8172253417969E+04, -.1318095947266E+04, .6560182495117E+03},
 {   21101, .8184203125000E+04, -.1328005004883E+04, .6499868164062E+03},
 {   21102, .8170218261719E+04, -.1326611938477E+04, .6448791503906E+03},
 {   21103, .8165724121094E+04, -.1315645019531E+04, .6596673583984E+03},
 {   21104, .8173379882812E+04, -.1302875000000E+04, .6719015502930E+03},
 {   21105, .8165883789062E+04, -.1309399047852E+04, .6753112182617E+03},
 {   21106, .8175257812500E+04, -.1291179077148E+04, .6710494995117E+03},
 {   21107, .8174665039062E+04, -.1279182128906E+04, .6713177490234E+03},
 {   21108, .8175041503906E+04, -.1293229003906E+04, .6763843383789E+03},
 {   21109, .8174713378906E+04, -.1281683105469E+04, .6765330810547E+03},
 {   21110, .8172145507812E+04, -.1304274047852E+04, .6777023925781E+03},
 {   21111, .8162890136719E+04, -.1305486083984E+04, .6819089965820E+03},
 {   21112, .8166196777344E+04, -.1300835083008E+04, .6891705322266E+03},
 {   21113, .8155274414062E+04, -.1304347045898E+04, .6801350097656E+03},
 {   21114, .8153236328125E+04, -.1299486083984E+04, .6862991943359E+03},
 {   21115, .8159240234375E+04, -.1296118041992E+04, .6920459594727E+03},
 {   21116, .8144387695312E+04, -.1292481079102E+04, .6903225097656E+03},
 {   21117, .8152935058594E+04, -.1288765014648E+04, .6946998291016E+03},
 {   21118, .8174981933594E+04, -.1290330078125E+04, .6876486816406E+03},
 {   21119, .8165778320312E+04, -.1294777099609E+04, .6963136596680E+03},
 {   21120, .8159463378906E+04, -.1283169067383E+04, .6991854858398E+03},
 {   21121, .8166646972656E+04, -.1285310058594E+04, .6959179077148E+03},
 {   21122, .8173826660156E+04, -.1287451049805E+04, .6926528320312E+03},
 {   21123, .8159246582031E+04, -.1313307983398E+04, .6626126708984E+03},
 {   21124, .8146236328125E+04, -.1308718017578E+04, .6684387817383E+03},
 {   21125, .8144847167969E+04, -.1298847045898E+04, .6848750000000E+03},
 {   21126, .8132483886719E+04, -.1305202026367E+04, .6734911499023E+03},
 {   21127, .8134997558594E+04, -.1292385009766E+04, .6893530883789E+03},
 {   21128, .8128320312500E+04, -.1297941040039E+04, .6837606201172E+03},
 {   21129, .8157240234375E+04, -.1324741943359E+04, .6437825927734E+03},
 {   21130, .8145136230469E+04, -.1318954956055E+04, .6504072265625E+03},
 {   21131, .8137200683594E+04, -.1312552978516E+04, .6611243286133E+03},
 {   21132, .8149484375000E+04, -.1285242065430E+04, .6951335449219E+03},
 {   21133, .8139930175781E+04, -.1287412109375E+04, .6907876586914E+03},
 {   21134, .8201374023438E+04, -.1391305908203E+04, .7366549072266E+03},
 {   21135, .8188732421875E+04, -.1390265869141E+04, .7450023193359E+03},
 {   21136, .8175354492188E+04, -.1386567871094E+04, .7492952880859E+03},
 {   21137, .8162284179688E+04, -.1380624877930E+04, .7494326782227E+03},
 {   21138, .8154787597656E+04, -.1374662841797E+04, .7450510864258E+03},
 {   21139, .8181108398438E+04, -.1387103881836E+04, .7527299194336E+03},
 {   21140, .8150018554688E+04, -.1371505859375E+04, .7485750122070E+03},
 {   21141, .8152742675781E+04, -.1374312866211E+04, .7564833984375E+03},
 {   21142, .8161440429688E+04, -.1381090820312E+04, .7726504516602E+03},
 {   21143, .8174674804688E+04, -.1385904907227E+04, .7685917358398E+03},
 {   21144, .8168287597656E+04, -.1382110839844E+04, .7585596313477E+03},
 {   21145, .8155469726562E+04, -.1377117919922E+04, .7643893432617E+03},
 {   21146, .8187911132812E+04, -.1390718872070E+04, .7645341186523E+03},
 {   21147, .8194665039062E+04, -.1387647827148E+04, .7465666503906E+03},
 {   21148, .8201910156250E+04, -.1391468872070E+04, .7592327880859E+03},
 {   21149, .8186063964844E+04, -.1187397216797E+04, .7373969116211E+03},
 {   21150, .8183397949219E+04, -.1192990234375E+04, .7297929077148E+03},
 {   21151, .8175348144531E+04, -.1195462280273E+04, .7319461669922E+03},
 {   21152, .8175565917969E+04, -.1191159301758E+04, .7392688598633E+03},
 {   21153, .8167691894531E+04, -.1199380249023E+04, .7312588500977E+03},
 {   21154, .8165769042969E+04, -.1195968261719E+04, .7387557983398E+03},
 {   21155, .8161599121094E+04, -.1204153198242E+04, .7278294677734E+03},
 {   21156, .8157501464844E+04, -.1201871215820E+04, .7350997314453E+03},
 {   21157, .8206590820312E+04, -.1198064208984E+04, .7023166503906E+03},
 {   21158, .8198257812500E+04, -.1200537231445E+04, .7046967773438E+03},
 {   21159, .8195048828125E+04, -.1205307250977E+04, .6989555053711E+03},
 {   21160, .8200710937500E+04, -.1204582275391E+04, .6956728515625E+03},
 {   21161, .8189380859375E+04, -.1210041259766E+04, .6952495117188E+03},
 {   21162, .8193001953125E+04, -.1210765258789E+04, .6910818481445E+03},
 {   21163, .8150731445312E+04, -.1208305175781E+04, .7293173828125E+03},
 {   21164, .8157970703125E+04, -.1209063232422E+04, .7221807861328E+03},
 {   21165, .8158027343750E+04, -.1214820190430E+04, .7120952148438E+03},
 {   21166, .8151365722656E+04, -.1215535156250E+04, .7161989135742E+03},
 {   21167, .8164023437500E+04, -.1217676147461E+04, .7022996826172E+03},
 {   21168, .8158225585938E+04, -.1221054199219E+04, .7010650024414E+03},
 {   21169, .8174180175781E+04, -.1216778198242E+04, .6957087402344E+03},
 {   21170, .8172843750000E+04, -.1220653198242E+04, .6900249633789E+03},
 {   21171, .8182165527344E+04, -.1214201171875E+04, .6937914428711E+03},
 {   21172, .8183390136719E+04, -.1216129150391E+04, .6894450073242E+03},
 {   21173, .8190524902344E+04, -.1192328247070E+04, .7252199707031E+03},
 {   21174, .8194929687500E+04, -.1187517333984E+04, .7300856323242E+03},
 {   21175, .8195831054688E+04, -.1193524291992E+04, .7188729858398E+03},
 {   21176, .8201772460938E+04, -.1189237304688E+04, .7215860595703E+03},
 {   21177, .8198541015625E+04, -.1196400268555E+04, .7116846313477E+03},
 {   21178, .8205461914062E+04, -.1192981201172E+04, .7120859375000E+03},
 {   21179, .8277112304688E+04, -.1240016113281E+04, .7090800170898E+03},
 {   21180, .8286263671875E+04, -.1236784179688E+04, .7233074951172E+03},
 {   21181, .8263960937500E+04, -.1229393188477E+04, .7365703735352E+03},
 {   21182, .8251115234375E+04, -.1237024169922E+04, .7208850708008E+03},
 {   21183, .8239477539062E+04, -.1218603149414E+04, .7476413574219E+03},
 {   21184, .8226044921875E+04, -.1229441162109E+04, .7323002929688E+03},
 {   21185, .8212415039062E+04, -.1214866210938E+04, .7587783813477E+03},
 {   21186, .8199932617188E+04, -.1228099121094E+04, .7441720581055E+03},
 {   21187, .8184838867188E+04, -.1212254150391E+04, .7691708984375E+03},
 {   21188, .8173917480469E+04, -.1225550170898E+04, .7560013427734E+03},
 {   21189, .8162509277344E+04, -.1199680175781E+04, .7770828247070E+03},
 {   21190, .8154065429688E+04, -.1208142211914E+04, .7650286865234E+03},
 {   21191, .8294713867188E+04, -.1233652099609E+04, .7379376220703E+03},
 {   21192, .8275943359375E+04, -.1222289184570E+04, .7529920654297E+03},
 {   21193, .8251888671875E+04, -.1208723266602E+04, .7640485839844E+03},
 {   21194, .8223711914062E+04, -.1203017211914E+04, .7747803955078E+03},
 {   21195, .8194413085938E+04, -.1200771240234E+04, .7840687255859E+03},
 {   21196, .8169445312500E+04, -.1193460205078E+04, .7911956176758E+03},
 {   21197, .8301880859375E+04, -.1229968139648E+04, .7530357666016E+03},
 {   21198, .8286773437500E+04, -.1215133178711E+04, .7699930419922E+03},
 {   21199, .8263277343750E+04, -.1199288208008E+04, .7811513671875E+03},
 {   21200, .8234124023438E+04, -.1192114257812E+04, .7915908203125E+03},
 {   21201, .8203229492188E+04, -.1190732177734E+04, .7998900756836E+03},
 {   21202, .8175760742188E+04, -.1189182250977E+04, .8063468627930E+03},
 {   21203, .8307330078125E+04, -.1224108154297E+04, .7680354003906E+03},
 {   21204, .8297159179688E+04, -.1205704223633E+04, .7860485839844E+03},
 {   21205, .8275493164062E+04, -.1187484252930E+04, .7964598999023E+03},
 {   21206, .8246632812500E+04, -.1178743286133E+04, .8057575683594E+03},
 {   21207, .8215422851562E+04, -.1178128295898E+04, .8122184448242E+03},
 {   21208, .8186716308594E+04, -.1182244262695E+04, .8171554565430E+03},
 {   21209, .8311535156250E+04, -.1216557128906E+04, .7826186523438E+03},
 {   21210, .8307139648438E+04, -.1194500244141E+04, .8005953979492E+03},
 {   21211, .8288145507812E+04, -.1173822265625E+04, .8091693115234E+03},
 {   21212, .8260419921875E+04, -.1163433349609E+04, .8162360229492E+03},
 {   21213, .8229728515625E+04, -.1163503295898E+04, .8197682495117E+03},
 {   21214, .8200629882812E+04, -.1173202270508E+04, .8220958862305E+03},
 {   21215, .8278287109375E+04, -.1226058227539E+04, .7085454711914E+03},
 {   21216, .8287476562500E+04, -.1223564208984E+04, .7235904541016E+03},
 {   21217, .8263031250000E+04, -.1210819213867E+04, .7347456665039E+03},
 {   21218, .8252511718750E+04, -.1216535156250E+04, .7202662353516E+03},
 {   21219, .8240253906250E+04, -.1193573242188E+04, .7443463134766E+03},
 {   21220, .8230730468750E+04, -.1199538208008E+04, .7301691894531E+03},
 {   21221, .8215916015625E+04, -.1179860229492E+04, .7539318847656E+03},
 {   21222, .8208225585938E+04, -.1183888183594E+04, .7404027709961E+03},
 {   21223, .8296783203125E+04, -.1221057128906E+04, .7385662231445E+03},
 {   21224, .8273569335938E+04, -.1205078247070E+04, .7492122802734E+03},
 {   21225, .8249687500000E+04, -.1187570312500E+04, .7585656127930E+03},
 {   21226, .8223418945312E+04, -.1175787231445E+04, .7675552978516E+03},
 {   21227, .8303950195312E+04, -.1215699218750E+04, .7537130737305E+03},
 {   21228, .8282231445312E+04, -.1197035278320E+04, .7638911132812E+03},
 {   21229, .8257513671875E+04, -.1179814208984E+04, .7730393066406E+03},
 {   21230, .8229577148438E+04, -.1170505249023E+04, .7814743652344E+03},
 {   21231, .8199848632812E+04, -.1173359252930E+04, .7887813720703E+03},
 {   21232, .8194417968750E+04, -.1177690307617E+04, .7764907226562E+03},
 {   21233, .8171786132812E+04, -.1182796264648E+04, .7899661865234E+03},
 {   21234, .8167017578125E+04, -.1186948242188E+04, .7811198730469E+03},
 {   21235, .8306030273438E+04, -.1205118164062E+04, .7671146240234E+03},
 {   21236, .8286655273438E+04, -.1184795288086E+04, .7772394409180E+03},
 {   21237, .8261954101562E+04, -.1168888305664E+04, .7865935058594E+03},
 {   21238, .8233202148438E+04, -.1163080322266E+04, .7948898925781E+03},
 {   21239, .8203588867188E+04, -.1167910278320E+04, .8009845581055E+03},
 {   21240, .8175709472656E+04, -.1178554321289E+04, .7991376342773E+03},
 {   21241, .8302833007812E+04, -.1190865234375E+04, .7762851562500E+03},
 {   21242, .8286773437500E+04, -.1169612304688E+04, .7872306518555E+03},
 {   21243, .8263048828125E+04, -.1155745361328E+04, .7976661987305E+03},
 {   21244, .8234448242188E+04, -.1154167358398E+04, .8067000732422E+03},
 {   21245, .8205921875000E+04, -.1161701293945E+04, .8124531250000E+03},
 {   21246, .8179196777344E+04, -.1174278320312E+04, .8084513549805E+03},
 {   21247, .8172387695312E+04, -.1227384155273E+04, .7514581298828E+03},
 {   21248, .8192716796875E+04, -.1230094238281E+04, .7422152099609E+03},
 {   21249, .8196837890625E+04, -.1248624145508E+04, .7403408813477E+03},
 {   21250, .8177559570312E+04, -.1246788085938E+04, .7491054687500E+03},
 {   21251, .8199768554688E+04, -.1267304077148E+04, .7390083618164E+03},
 {   21252, .8181235839844E+04, -.1266379150391E+04, .7474347534180E+03},
 {   21253, .8213155273438E+04, -.1231524169922E+04, .7329216918945E+03},
 {   21254, .8216195312500E+04, -.1249607177734E+04, .7315404052734E+03},
 {   21255, .8218333984375E+04, -.1267803100586E+04, .7305649414062E+03},
 {   21256, .8233381835938E+04, -.1234374145508E+04, .7237234497070E+03},
 {   21257, .8235407226562E+04, -.1251536132812E+04, .7228040771484E+03},
 {   21258, .8236830078125E+04, -.1268774047852E+04, .7221567382812E+03},
 {   21259, .8253444335938E+04, -.1238602172852E+04, .7146024780273E+03},
 {   21260, .8254508789062E+04, -.1254385131836E+04, .7141187744141E+03},
 {   21261, .8255273437500E+04, -.1270205078125E+04, .7137716674805E+03},
 {   21262, .8273758789062E+04, -.1241416137695E+04, .7053660278320E+03},
 {   21263, .8273779296875E+04, -.1256291137695E+04, .7053568115234E+03},
 {   21264, .8273801757812E+04, -.1271165039062E+04, .7053471679688E+03},
 {   21265, .8172753417969E+04, -.1270178100586E+04, .7361098632812E+03},
 {   21266, .8164104492188E+04, -.1273362060547E+04, .7242916870117E+03},
 {   21267, .8155594238281E+04, -.1276348022461E+04, .7122749633789E+03},
 {   21268, .8147248535156E+04, -.1279102050781E+04, .7000240478516E+03},
 {   21269, .8169981445312E+04, -.1254439086914E+04, .7379053955078E+03},
 {   21270, .8162063964844E+04, -.1260856079102E+04, .7257190551758E+03},
 {   21271, .8154425781250E+04, -.1266883056641E+04, .7131364746094E+03},
 {   21272, .8147126953125E+04, -.1272443115234E+04, .7000797119141E+03},
 {   21273, .8166523925781E+04, -.1238222167969E+04, .7410266113281E+03},
 {   21274, .8159710937500E+04, -.1247813110352E+04, .7304432983398E+03},
 {   21275, .8152884765625E+04, -.1257465087891E+04, .7141671752930E+03},
 {   21276, .8147001953125E+04, -.1265785156250E+04, .7001376342773E+03},
 {   21277, .8142396484375E+04, -.1254967163086E+04, .7207628173828E+03},
 {   21278, .8139208496094E+04, -.1261199096680E+04, .7036795654297E+03},
 {   21279, .8133008789062E+04, -.1262760131836E+04, .7252288818359E+03},
 {   21280, .8132236816406E+04, -.1267711059570E+04, .7068496704102E+03},
 {   21281, .8126579589844E+04, -.1273378051758E+04, .7283489379883E+03},
 {   21282, .8127854980469E+04, -.1276289062500E+04, .7088419799805E+03},
 {   21283, .8144932617188E+04, -.1248736083984E+04, .7377474975586E+03},
 {   21284, .8132478515625E+04, -.1257807128906E+04, .7434105834961E+03},
 {   21285, .8123354492188E+04, -.1270469116211E+04, .7475582275391E+03},
 {   21286, .8118905273438E+04, -.1268124145508E+04, .7594177856445E+03},
 {   21287, .8129255859375E+04, -.1253803100586E+04, .7534525756836E+03},
 {   21288, .8141380859375E+04, -.1239847167969E+04, .7506162719727E+03},
 {   21289, .8150626464844E+04, -.1230232177734E+04, .7482562866211E+03},
 {   21290, .8152568359375E+04, -.1212718261719E+04, .7604696044922E+03},
 {   21291, .8137552734375E+04, -.1277695068359E+04, .7044328002930E+03},
 {   21292, .8139680664062E+04, -.1270077148438E+04, .7034652099609E+03},
 {   21293, .8116321777344E+04, -.1268153076172E+04, .7140856323242E+03},
 {   21294, .8128666992188E+04, -.1260407104492E+04, .7084731445312E+03},
 {   21295, .8126443359375E+04, -.1258406127930E+04, .7191015014648E+03},
 {   21296, .8114823730469E+04, -.1266447143555E+04, .7247462768555E+03},
 {   21297, .8124671875000E+04, -.1256203125000E+04, .7297203979492E+03},
 {   21298, .8113332031250E+04, -.1264744140625E+04, .7354085693359E+03},
 {   21299, .8124063476562E+04, -.1253472167969E+04, .7403229980469E+03},
 {   21300, .8111832031250E+04, -.1263039062500E+04, .7460688476562E+03},
 {   21301, .8124317382812E+04, -.1250596191406E+04, .7504877929688E+03},
 {   21302, .8110339355469E+04, -.1261335083008E+04, .7567297363281E+03},
 {   21303, .8140204101562E+04, -.1251487182617E+04, .7032286376953E+03},
 {   21304, .8136374023438E+04, -.1248625122070E+04, .7140721435547E+03},
 {   21305, .8133443847656E+04, -.1245356201172E+04, .7249003906250E+03},
 {   21306, .8132844726562E+04, -.1241036132812E+04, .7356905517578E+03},
 {   21307, .8133968750000E+04, -.1236423217773E+04, .7456118774414E+03},
 {   21308, .8147961425781E+04, -.1239727172852E+04, .6997733154297E+03},
 {   21309, .8142815429688E+04, -.1236198120117E+04, .7107568969727E+03},
 {   21310, .8139018554688E+04, -.1232060180664E+04, .7217141113281E+03},
 {   21311, .8138722656250E+04, -.1226343139648E+04, .7326140747070E+03},
 {   21312, .8141004882812E+04, -.1220186157227E+04, .7422113647461E+03},
 {   21313, .8152287597656E+04, -.1225573242188E+04, .6979512329102E+03},
 {   21314, .8146493652344E+04, -.1221833251953E+04, .7091284179688E+03},
 {   21315, .8142502441406E+04, -.1217280151367E+04, .7202741088867E+03},
 {   21316, .8143177734375E+04, -.1210623168945E+04, .7313402099609E+03},
 {   21317, .8147291992188E+04, -.1203379272461E+04, .7406677856445E+03},
 {   21318, .8156231933594E+04, -.1236602172852E+04, .6937069702148E+03},
 {   21319, .8160131347656E+04, -.1225147216797E+04, .6923960571289E+03},
 {   21320, .8165055664062E+04, -.1232186157227E+04, .6874038696289E+03},
 {   21321, .8167973632812E+04, -.1224720214844E+04, .6868418579102E+03},
 {   21322, .8174377929688E+04, -.1227365234375E+04, .6821699218750E+03},
 {   21323, .8176445800781E+04, -.1222683227539E+04, .6835930786133E+03},
 {   21324, .8184599121094E+04, -.1223282226562E+04, .6775863647461E+03},
 {   21325, .8185429687500E+04, -.1219329223633E+04, .6822296142578E+03},
 {   21326, .8142226562500E+04, -.1250985107422E+04, .6918300781250E+03},
 {   21327, .8151944335938E+04, -.1243714111328E+04, .6874110107422E+03},
 {   21328, .8161877441406E+04, -.1236753173828E+04, .6828950805664E+03},
 {   21329, .8172180175781E+04, -.1230597167969E+04, .6782106933594E+03},
 {   21330, .8183763183594E+04, -.1227234130859E+04, .6729430541992E+03},
 {   21331, .8159151367188E+04, -.1236824218750E+04, .6707423706055E+03},
 {   21332, .8172023437500E+04, -.1231294189453E+04, .6689241333008E+03},
 {   21333, .8156660644531E+04, -.1236797119141E+04, .6584663085938E+03},
 {   21334, .8171986816406E+04, -.1231948242188E+04, .6595726318359E+03},
 {   21335, .8155564941406E+04, -.1236069213867E+04, .6458881835938E+03},
 {   21336, .8172647460938E+04, -.1232249145508E+04, .6500725708008E+03},
 {   21337, .8148064941406E+04, -.1244162109375E+04, .6774395141602E+03},
 {   21338, .8144539550781E+04, -.1244468139648E+04, .6672811279297E+03},
 {   21339, .8143108886719E+04, -.1243718139648E+04, .6566714477539E+03},
 {   21340, .8138077148438E+04, -.1252265136719E+04, .6844545288086E+03},
 {   21341, .8134402343750E+04, -.1253355102539E+04, .6768302612305E+03},
 {   21342, .8133517578125E+04, -.1253037109375E+04, .6686037597656E+03},
 {   21343, .8117830566406E+04, -.1270397094727E+04, .6923274536133E+03},
 {   21344, .8122546386719E+04, -.1264378051758E+04, .6842880249023E+03},
 {   21345, .8122975097656E+04, -.1263737060547E+04, .6907296752930E+03},
 {   21346, .8117564941406E+04, -.1268981079102E+04, .6976835327148E+03},
 {   21347, .8124349609375E+04, -.1262620117188E+04, .6969580688477E+03},
 {   21348, .8117297363281E+04, -.1267565063477E+04, .7030380249023E+03},
 {   21349, .8125653808594E+04, -.1262451171875E+04, .7032150268555E+03},
 {   21350, .8116811035156E+04, -.1267838134766E+04, .7085606079102E+03},
 {   21351, .8127560546875E+04, -.1258494140625E+04, .6763251342773E+03},
 {   21352, .8128495605469E+04, -.1258439086914E+04, .6838848876953E+03},
 {   21353, .8131334960938E+04, -.1257430175781E+04, .6910197753906E+03},
 {   21354, .8134247558594E+04, -.1256627075195E+04, .6980351562500E+03},
 {   21355, .8204300781250E+04, -.1238483154297E+04, .6682874755859E+03},
 {   21356, .8198561523438E+04, -.1231612182617E+04, .6691942749023E+03},
 {   21357, .8188312500000E+04, -.1234138183594E+04, .6590733032227E+03},
 {   21358, .8198187500000E+04, -.1242683105469E+04, .6550922851562E+03},
 {   21359, .8220354492188E+04, -.1195243286133E+04, .7094531860352E+03},
 {   21360, .8216541015625E+04, -.1201924194336E+04, .6980162963867E+03},
 {   21361, .8220309570312E+04, -.1209839233398E+04, .6946867675781E+03},
 {   21362, .8225319335938E+04, -.1202198242188E+04, .7064360961914E+03},
 {   21363, .8211167968750E+04, -.1210870239258E+04, .6892119750977E+03},
 {   21364, .8215532226562E+04, -.1221248168945E+04, .6867396850586E+03},
 {   21365, .8205010742188E+04, -.1220664184570E+04, .6823051147461E+03},
 {   21366, .8212079101562E+04, -.1234150146484E+04, .6805864868164E+03},
 {   21367, .8195573242188E+04, -.1226008178711E+04, .6735119628906E+03},
 {   21368, .8194292968750E+04, -.1220571166992E+04, .6789687500000E+03},
 {   21369, .8215388671875E+04, -.1188286254883E+04, .7124692382812E+03},
 {   21370, .8215023437500E+04, -.1194054199219E+04, .7025761718750E+03},
 {   21371, .8211306640625E+04, -.1200579223633E+04, .6941458740234E+03},
 {   21372, .8204701171875E+04, -.1207306274414E+04, .6877177734375E+03},
 {   21373, .8195860351562E+04, -.1213704223633E+04, .6836594848633E+03},
 {   21374, .8249481445312E+04, -.1227295166016E+04, .6954486083984E+03},
 {   21375, .8240119140625E+04, -.1220614257812E+04, .6997070312500E+03},
 {   21376, .8236002929688E+04, -.1230316162109E+04, .6923127441406E+03},
 {   21377, .8243709960938E+04, -.1242116210938E+04, .6878003540039E+03},
 {   21378, .8231233398438E+04, -.1237362182617E+04, .6835335083008E+03},
 {   21379, .8238277343750E+04, -.1243846191406E+04, .6770396728516E+03},
 {   21380, .8226504882812E+04, -.1242778198242E+04, .6739068603516E+03},
 {   21381, .8232844726562E+04, -.1245577148438E+04, .6662781372070E+03},
 {   21382, .8220340820312E+04, -.1245338134766E+04, .6622328491211E+03},
 {   21383, .8227413085938E+04, -.1247308105469E+04, .6555155029297E+03},
 {   21384, .8214049804688E+04, -.1246919189453E+04, .6482001953125E+03},
 {   21385, .8221980468750E+04, -.1249038085938E+04, .6447542114258E+03},
 {   21386, .8232719726562E+04, -.1211405273438E+04, .7030719604492E+03},
 {   21387, .8228232421875E+04, -.1220179199219E+04, .6937235717773E+03},
 {   21388, .8223534179688E+04, -.1229508178711E+04, .6855847778320E+03},
 {   21389, .8219515625000E+04, -.1238770141602E+04, .6779205932617E+03},
 {   21390, .8212619140625E+04, -.1242317138672E+04, .6661579589844E+03},
 {   21391, .8206120117188E+04, -.1244801147461E+04, .6516456298828E+03},
 {   21392, .8248535156250E+04, -.1272487060547E+04, .6958604736328E+03},
 {   21393, .8243482421875E+04, -.1276238037109E+04, .6877365112305E+03},
 {   21394, .8248850585938E+04, -.1257424072266E+04, .6957232055664E+03},
 {   21395, .8243557617188E+04, -.1264864135742E+04, .6877569580078E+03},
 {   21396, .8249166015625E+04, -.1242359130859E+04, .6955878295898E+03},
 {   21397, .8243899414062E+04, -.1257016113281E+04, .6866207885742E+03},
 {   21398, .8236863281250E+04, -.1256436157227E+04, .6736812744141E+03},
 {   21399, .8232863281250E+04, -.1256107177734E+04, .6663343505859E+03},
 {   21400, .8227480468750E+04, -.1257416137695E+04, .6556113281250E+03},
 {   21401, .8222094726562E+04, -.1258725097656E+04, .6448903198242E+03},
 {   21402, .8238220703125E+04, -.1265751098633E+04, .6770753784180E+03},
 {   21403, .8232883789062E+04, -.1266638061523E+04, .6663911132812E+03},
 {   21404, .8227546875000E+04, -.1267525146484E+04, .6557096557617E+03},
 {   21405, .8222208984375E+04, -.1268412109375E+04, .6450253906250E+03},
 {   21406, .8238191406250E+04, -.1276703125000E+04, .6770921020508E+03},
 {   21407, .8232902343750E+04, -.1277168090820E+04, .6664494018555E+03},
 {   21408, .8227613281250E+04, -.1277633056641E+04, .6558059082031E+03},
 {   21409, .8222323242188E+04, -.1278098022461E+04, .6451630249023E+03},
 {   21410, .8264958984375E+04, -.1263728149414E+04, .6878875732422E+03},
 {   21411, .8264468750000E+04, -.1257639160156E+04, .6875592041016E+03},
 {   21412, .8264974609375E+04, -.1274884033203E+04, .6878821411133E+03},
 {   21413, .8261511718750E+04, -.1232664184570E+04, .6899806518555E+03},
 {   21414, .8210798828125E+04, -.1182963256836E+04, .7252733764648E+03},
 {   21415, .8220820312500E+04, -.1193234252930E+04, .7211480712891E+03},
 {   21416, .8230836914062E+04, -.1203506225586E+04, .7170238037109E+03},
 {   21417, .8248171875000E+04, -.1219761230469E+04, .7091412353516E+03},
 {   21418, .8239505859375E+04, -.1211633178711E+04, .7130820922852E+03},
 {   21419, .8258990234375E+04, -.1224540161133E+04, .7042243652344E+03},
 {   21420, .8269805664062E+04, -.1229319213867E+04, .6993051757812E+03},
 {   21421, .8110358398438E+04, -.1270620117188E+04, .7451618652344E+03},
 {   21422, .8109145996094E+04, -.1269477050781E+04, .7563250732422E+03},
 {   21423, .8111862792969E+04, -.1271765136719E+04, .7340247802734E+03},
 {   21424, .8113694824219E+04, -.1272908081055E+04, .7229324951172E+03},
 {   21425, .8115609863281E+04, -.1274011108398E+04, .7118594970703E+03},
 {   21426, .8118286621094E+04, -.1273653076172E+04, .7011169433594E+03},
 {   21427, .8116949707031E+04, -.1273832031250E+04, .7064885864258E+03},
 {   21428, .8120246582031E+04, -.1274477050781E+04, .6965760498047E+03},
 {   21429, .8198567382812E+04, -.1253579101562E+04, .6556586303711E+03},
 {   21430, .8206409179688E+04, -.1255294067383E+04, .6520693969727E+03},
 {   21431, .8214252929688E+04, -.1257010131836E+04, .6484793090820E+03},
 {   21432, .8198944335938E+04, -.1264474121094E+04, .6562263183594E+03},
 {   21433, .8206701171875E+04, -.1265787109375E+04, .6524926757812E+03},
 {   21434, .8214453125000E+04, -.1267099121094E+04, .6487585449219E+03},
 {   21435, .8199321289062E+04, -.1275370117188E+04, .6567919311523E+03},
 {   21436, .8206992187500E+04, -.1276280029297E+04, .6529145507812E+03},
 {   21437, .8214657226562E+04, -.1277189086914E+04, .6490386962891E+03},
 {   21438, .8175873535156E+04, -.1276712036133E+04, .6655596923828E+03},
 {   21439, .8187600097656E+04, -.1276041137695E+04, .6611767578125E+03},
 {   21440, .8174132812500E+04, -.1261859130859E+04, .6663188476562E+03},
 {   21441, .8186604980469E+04, -.1263686157227E+04, .6612413330078E+03},
 {   21442, .8172253417969E+04, -.1253984130859E+04, .6560182495117E+03},
 {   21443, .8185411132812E+04, -.1253781127930E+04, .6558381347656E+03},
 {   21444, .8170218261719E+04, -.1245467163086E+04, .6448791503906E+03},
 {   21445, .8184203125000E+04, -.1244075195312E+04, .6499868164062E+03},
 {   21446, .8165724121094E+04, -.1256435058594E+04, .6596673583984E+03},
 {   21447, .8165883789062E+04, -.1262681152344E+04, .6753112182617E+03},
 {   21448, .8173379882812E+04, -.1269205078125E+04, .6719015502930E+03},
 {   21449, .8172145507812E+04, -.1267805053711E+04, .6777023925781E+03},
 {   21450, .8166196777344E+04, -.1271245117188E+04, .6891705322266E+03},
 {   21451, .8162890136719E+04, -.1266593139648E+04, .6819089965820E+03},
 {   21452, .8155274414062E+04, -.1267733154297E+04, .6801350097656E+03},
 {   21453, .8159240234375E+04, -.1275961059570E+04, .6920459594727E+03},
 {   21454, .8153236328125E+04, -.1272594116211E+04, .6862991943359E+03},
 {   21455, .8144387695312E+04, -.1279599121094E+04, .6903225097656E+03},
 {   21456, .8174981933594E+04, -.1281749023438E+04, .6876486816406E+03},
 {   21457, .8144847167969E+04, -.1273232055664E+04, .6848750000000E+03},
 {   21458, .8146236328125E+04, -.1263360107422E+04, .6684387817383E+03},
 {   21459, .8159246582031E+04, -.1258771118164E+04, .6626126708984E+03},
 {   21460, .8134997558594E+04, -.1279694091797E+04, .6893530883789E+03},
 {   21461, .8132483886719E+04, -.1266878051758E+04, .6734911499023E+03},
 {   21462, .8128320312500E+04, -.1274139038086E+04, .6837606201172E+03},
 {   21463, .8157240234375E+04, -.1247337158203E+04, .6437825927734E+03},
 {   21464, .8145136230469E+04, -.1253125122070E+04, .6504072265625E+03},
 {   21465, .8137200683594E+04, -.1259527099609E+04, .6611243286133E+03},
 {   21466, .8201374023438E+04, -.1180773193359E+04, .7366549072266E+03},
 {   21467, .8188732421875E+04, -.1181813232422E+04, .7450023193359E+03},
 {   21468, .8175354492188E+04, -.1185511230469E+04, .7492952880859E+03},
 {   21469, .8162284179688E+04, -.1191454223633E+04, .7494325561523E+03},
 {   21470, .8154787597656E+04, -.1197416259766E+04, .7450510864258E+03},
 {   21471, .8181108398438E+04, -.1184976318359E+04, .7527299194336E+03},
 {   21472, .8150018554688E+04, -.1200572265625E+04, .7485750122070E+03},
 {   21473, .8152742675781E+04, -.1197767211914E+04, .7564833984375E+03},
 {   21474, .8161440429688E+04, -.1190988281250E+04, .7726504516602E+03},
 {   21475, .8155469726562E+04, -.1194961181641E+04, .7643893432617E+03},
 {   21476, .8168287597656E+04, -.1189969238281E+04, .7585596313477E+03},
 {   21477, .8174674804688E+04, -.1186174316406E+04, .7685917358398E+03},
 {   21478, .8187911132812E+04, -.1181361328125E+04, .7645341186523E+03},
 {   21479, .8201910156250E+04, -.1180610229492E+04, .7592327880859E+03},
 {   21480, .8194665039062E+04, -.1184432250977E+04, .7465665283203E+03},
 {   21481, .8052436035156E+04, -.1164001342773E+04, .7039116821289E+03},
 {   21482, .8062355468750E+04, -.1164675292969E+04, .7017477416992E+03},
 {   21483, .8065398437500E+04, -.1154073364258E+04, .7113955078125E+03},
 {   21484, .8055514160156E+04, -.1154139282227E+04, .7135186157227E+03},
 {   21485, .8076205078125E+04, -.1165601318359E+04, .6988886718750E+03},
 {   21486, .8079211425781E+04, -.1154260375977E+04, .7085754394531E+03},
 {   21487, .8086124023438E+04, -.1166275268555E+04, .6967251586914E+03},
 {   21488, .8089093261719E+04, -.1154193359375E+04, .7064526977539E+03},
 {   21489, .8096041992188E+04, -.1166948242188E+04, .6945616455078E+03},
 {   21490, .8098975097656E+04, -.1154127319336E+04, .7043301391602E+03},
 {   21491, .8105959960938E+04, -.1167622314453E+04, .6923994140625E+03},
 {   21492, .8108860839844E+04, -.1154061279297E+04, .7022080078125E+03},
 {   21493, .8069591308594E+04, -.1149920288086E+04, .7247914428711E+03},
 {   21494, .8059492675781E+04, -.1150568359375E+04, .7265797119141E+03},
 {   21495, .8083618164062E+04, -.1149523315430E+04, .7223063964844E+03},
 {   21496, .8093716796875E+04, -.1148874267578E+04, .7205175170898E+03},
 {   21497, .8103814941406E+04, -.1148226318359E+04, .7187293701172E+03},
 {   21498, .8113912109375E+04, -.1147577392578E+04, .7169401245117E+03},
 {   21499, .8073680175781E+04, -.1152653320312E+04, .7384290161133E+03},
 {   21500, .8063287109375E+04, -.1154289306641E+04, .7396488037109E+03},
 {   21501, .8088003417969E+04, -.1151270263672E+04, .7365135498047E+03},
 {   21502, .8098396484375E+04, -.1149635253906E+04, .7352937011719E+03},
 {   21503, .8108788574219E+04, -.1148000366211E+04, .7340743408203E+03},
 {   21504, .8119186523438E+04, -.1146364379883E+04, .7328540649414E+03},
 {   21505, .8075711425781E+04, -.1163417358398E+04, .7479197998047E+03},
 {   21506, .8065876464844E+04, -.1164371337891E+04, .7491726074219E+03},
 {   21507, .8089476562500E+04, -.1162716308594E+04, .7459698486328E+03},
 {   21508, .8099311523438E+04, -.1161763305664E+04, .7447159423828E+03},
 {   21509, .8109146972656E+04, -.1160809326172E+04, .7434628295898E+03},
 {   21510, .8118982421875E+04, -.1159856323242E+04, .7422098999023E+03},
 {   21511, .8075870605469E+04, -.1177648315430E+04, .7513947143555E+03},
 {   21512, .8066589355469E+04, -.1178038208008E+04, .7526264648438E+03},
 {   21513, .8089080566406E+04, -.1177510253906E+04, .7494659423828E+03},
 {   21514, .8098364257812E+04, -.1177119262695E+04, .7482339477539E+03},
 {   21515, .8107643066406E+04, -.1176729248047E+04, .7470022583008E+03},
 {   21516, .8116922363281E+04, -.1176338256836E+04, .7457697143555E+03},
 {   21517, .8061389648438E+04, -.1179029296875E+04, .6983147583008E+03},
 {   21518, .8051113769531E+04, -.1178316284180E+04, .7004506835938E+03},
 {   21519, .8075595703125E+04, -.1179996215820E+04, .6954826049805E+03},
 {   21520, .8085875000000E+04, -.1180710205078E+04, .6933463745117E+03},
 {   21521, .8096149902344E+04, -.1181424316406E+04, .6912104492188E+03},
 {   21522, .8106425292969E+04, -.1182138305664E+04, .6890746459961E+03},
 {   21523, .8062117187500E+04, -.1193172241211E+04, .7023731689453E+03},
 {   21524, .8051983398438E+04, -.1192479248047E+04, .7045747680664E+03},
 {   21525, .8076183593750E+04, -.1194118286133E+04, .6994753417969E+03},
 {   21526, .8086320312500E+04, -.1194811279297E+04, .6972740478516E+03},
 {   21527, .8096454589844E+04, -.1195504272461E+04, .6950725097656E+03},
 {   21528, .8106592285156E+04, -.1196198242188E+04, .6928708496094E+03},
 {   21529, .8108224609375E+04, -.1205879272461E+04, .7038764648438E+03},
 {   21530, .8098345214844E+04, -.1205204223633E+04, .7060205078125E+03},
 {   21531, .8111214355469E+04, -.1208468261719E+04, .7181173095703E+03},
 {   21532, .8101553222656E+04, -.1207800170898E+04, .7202037353516E+03},
 {   21533, .8088467285156E+04, -.1204528198242E+04, .7081635742188E+03},
 {   21534, .8091885742188E+04, -.1207132202148E+04, .7222897338867E+03},
 {   21535, .8078586425781E+04, -.1203852172852E+04, .7103065795898E+03},
 {   21536, .8082224609375E+04, -.1206464233398E+04, .7243760986328E+03},
 {   21537, .8064777832031E+04, -.1202924194336E+04, .7131453247070E+03},
 {   21538, .8068631347656E+04, -.1205543212891E+04, .7271575317383E+03},
 {   21539, .8054899414062E+04, -.1202248168945E+04, .7152893676758E+03},
 {   21540, .8058968750000E+04, -.1204875244141E+04, .7292441406250E+03},
 {   21541, .8065222656250E+04, -.1191628295898E+04, .7491599121094E+03},
 {   21542, .8074216308594E+04, -.1192254272461E+04, .7472266235352E+03},
 {   21543, .8072062011719E+04, -.1201403198242E+04, .7386923828125E+03},
 {   21544, .8062552734375E+04, -.1200757202148E+04, .7407286987305E+03},
 {   21545, .8087141601562E+04, -.1193133300781E+04, .7445960083008E+03},
 {   21546, .8085504882812E+04, -.1202302246094E+04, .7359612426758E+03},
 {   21547, .8096136718750E+04, -.1193759277344E+04, .7426611328125E+03},
 {   21548, .8095011230469E+04, -.1202948242188E+04, .7339264526367E+03},
 {   21549, .8105130371094E+04, -.1194385253906E+04, .7407279663086E+03},
 {   21550, .8104521972656E+04, -.1203594238281E+04, .7318911132812E+03},
 {   21551, .8114123535156E+04, -.1195011230469E+04, .7387937622070E+03},
 {   21552, .8114035156250E+04, -.1204240234375E+04, .7298559570312E+03},
 {   21553, .8126690429688E+04, -.1146836303711E+04, .7287947998047E+03},
 {   21554, .8121878906250E+04, -.1146740356445E+04, .7148507690430E+03},
 {   21555, .8134389648438E+04, -.1146855346680E+04, .7240167236328E+03},
 {   21556, .8129837402344E+04, -.1146314331055E+04, .7122255859375E+03},
 {   21557, .8142857421875E+04, -.1147249389648E+04, .7208671875000E+03},
 {   21558, .8138342285156E+04, -.1146199340820E+04, .7105998535156E+03},
 {   21559, .8151808593750E+04, -.1148080322266E+04, .7198990478516E+03},
 {   21560, .8147216796875E+04, -.1146524291992E+04, .7103970947266E+03},
 {   21561, .8160719726562E+04, -.1149552368164E+04, .7206544799805E+03},
 {   21562, .8156119628906E+04, -.1147583374023E+04, .7114542236328E+03},
 {   21563, .8169191894531E+04, -.1152602294922E+04, .7202308959961E+03},
 {   21564, .8164629882812E+04, -.1150841308594E+04, .7118492431641E+03},
 {   21565, .8175173828125E+04, -.1158603271484E+04, .7174332885742E+03},
 {   21566, .8170502929688E+04, -.1157716308594E+04, .7104499511719E+03},
 {   21567, .8177521484375E+04, -.1166624267578E+04, .7140462646484E+03},
 {   21568, .8173439941406E+04, -.1166305297852E+04, .7084673461914E+03},
 {   21569, .8117005859375E+04, -.1152184326172E+04, .7018687133789E+03},
 {   21570, .8125229980469E+04, -.1150780273438E+04, .7013038940430E+03},
 {   21571, .8133772949219E+04, -.1149623291016E+04, .7011102905273E+03},
 {   21572, .8142575683594E+04, -.1148908325195E+04, .7015799560547E+03},
 {   21573, .8151475097656E+04, -.1149021362305E+04, .7028460083008E+03},
 {   21574, .8160032226562E+04, -.1151953369141E+04, .7039680175781E+03},
 {   21575, .8165798339844E+04, -.1159170288086E+04, .7038760375977E+03},
 {   21576, .8114178710938E+04, -.1164467285156E+04, .6935338745117E+03},
 {   21577, .8122441894531E+04, -.1161396240234E+04, .6945385742188E+03},
 {   21578, .8130801757812E+04, -.1158506347656E+04, .6952843017578E+03},
 {   21579, .8139305664062E+04, -.1156063354492E+04, .6959375000000E+03},
 {   21580, .8147982421875E+04, -.1154539306641E+04, .6969221191406E+03},
 {   21581, .8156358886719E+04, -.1156455322266E+04, .6982789306641E+03},
 {   21582, .8161795410156E+04, -.1163324340820E+04, .6990026855469E+03},
 {   21583, .8165699218750E+04, -.1171292236328E+04, .6991533203125E+03},
 {   21584, .8169554687500E+04, -.1179271240234E+04, .6996404418945E+03},
 {   21585, .8177401367188E+04, -.1175416259766E+04, .7119645385742E+03},
 {   21586, .8175953125000E+04, -.1184319213867E+04, .7110737915039E+03},
 {   21587, .8173406738281E+04, -.1187240234375E+04, .7003128051758E+03},
 {   21588, .8119377441406E+04, -.1189806274414E+04, .7344232177734E+03},
 {   21589, .8122353515625E+04, -.1175336303711E+04, .7401005859375E+03},
 {   21590, .8125090820312E+04, -.1184664306641E+04, .7305940551758E+03},
 {   21591, .8128714355469E+04, -.1171703247070E+04, .7352561645508E+03},
 {   21592, .8131647949219E+04, -.1179781250000E+04, .7280189819336E+03},
 {   21593, .8135894042969E+04, -.1168368286133E+04, .7317920532227E+03},
 {   21594, .8139097167969E+04, -.1175615234375E+04, .7272964477539E+03},
 {   21595, .8143831542969E+04, -.1165657226562E+04, .7302881469727E+03},
 {   21596, .8147225585938E+04, -.1172950317383E+04, .7277849121094E+03},
 {   21597, .8152207519531E+04, -.1164160278320E+04, .7301668090820E+03},
 {   21598, .8155749023438E+04, -.1172644287109E+04, .7284355468750E+03},
 {   21599, .8160699218750E+04, -.1164762329102E+04, .7297615966797E+03},
 {   21600, .8163649902344E+04, -.1175765258789E+04, .7284857788086E+03},
 {   21601, .8167950683594E+04, -.1168631225586E+04, .7281629638672E+03},
 {   21602, .8167910644531E+04, -.1182960205078E+04, .7297799682617E+03},
 {   21603, .8171558105469E+04, -.1175891235352E+04, .7271983642578E+03},
 {   21604, .8172681640625E+04, -.1189965209961E+04, .7304796752930E+03},
 {   21605, .8174682128906E+04, -.1183281250000E+04, .7262725830078E+03},
 {   21606, .8179302246094E+04, -.1195383300781E+04, .7297980346680E+03},
 {   21607, .8178098144531E+04, -.1187352294922E+04, .7218286132812E+03},
 {   21608, .8125215820312E+04, -.1159982299805E+04, .7371417846680E+03},
 {   21609, .8132187500000E+04, -.1158172363281E+04, .7320809326172E+03},
 {   21610, .8139954101562E+04, -.1156697265625E+04, .7285238647461E+03},
 {   21611, .8148341308594E+04, -.1155754272461E+04, .7270369873047E+03},
 {   21612, .8156923339844E+04, -.1155738281250E+04, .7271046142578E+03},
 {   21613, .8165349121094E+04, -.1157560302734E+04, .7264391479492E+03},
 {   21614, .8171906738281E+04, -.1162492309570E+04, .7239916381836E+03},
 {   21615, .8174827636719E+04, -.1170129272461E+04, .7215644531250E+03},
 {   21616, .8176271484375E+04, -.1178216308594E+04, .7198106079102E+03},
 {   21617, .8195958007812E+04, -.1207762207031E+04, .7065744628906E+03},
 {   21618, .8193517578125E+04, -.1207664184570E+04, .7001234741211E+03},
 {   21619, .8192962890625E+04, -.1204333251953E+04, .7014368896484E+03},
 {   21620, .8189167480469E+04, -.1207442260742E+04, .6948065795898E+03},
 {   21621, .8189261718750E+04, -.1201006225586E+04, .6967215576172E+03},
 {   21622, .8183333984375E+04, -.1207102172852E+04, .6911610717773E+03},
 {   21623, .8184151367188E+04, -.1198107177734E+04, .6932558593750E+03},
 {   21624, .8176689941406E+04, -.1206670166016E+04, .6894052124023E+03},
 {   21625, .8177736328125E+04, -.1196171264648E+04, .6919624023438E+03},
 {   21626, .8169833007812E+04, -.1206203247070E+04, .6897471923828E+03},
 {   21627, .8170989257812E+04, -.1195248291016E+04, .6925991210938E+03},
 {   21628, .8163441894531E+04, -.1205768188477E+04, .6922641601562E+03},
 {   21629, .8164582519531E+04, -.1195365234375E+04, .6949878540039E+03},
 {   21630, .8158128906250E+04, -.1205388183594E+04, .6966283569336E+03},
 {   21631, .8159130859375E+04, -.1196540283203E+04, .6989499511719E+03},
 {   21632, .8154401367188E+04, -.1205083251953E+04, .7024074707031E+03},
 {   21633, .8155175292969E+04, -.1198664184570E+04, .7041077270508E+03},
 {   21634, .8152691406250E+04, -.1204900268555E+04, .7090676879883E+03},
 {   21635, .8153127929688E+04, -.1201538208008E+04, .7099622192383E+03},
 {   21636, .8153061035156E+04, -.1204864257812E+04, .7159572143555E+03},
 {   21637, .8193445312500E+04, -.1211024169922E+04, .7010962524414E+03},
 {   21638, .8189135742188E+04, -.1213843261719E+04, .6966043090820E+03},
 {   21639, .8183368652344E+04, -.1215921142578E+04, .6935700073242E+03},
 {   21640, .8176759765625E+04, -.1217065185547E+04, .6922428588867E+03},
 {   21641, .8169938476562E+04, -.1217168212891E+04, .6927431640625E+03},
 {   21642, .8163541992188E+04, -.1216247192383E+04, .6949742431641E+03},
 {   21643, .8158240234375E+04, -.1214336181641E+04, .6988291625977E+03},
 {   21644, .8155040527344E+04, -.1211486206055E+04, .7041366577148E+03},
 {   21645, .8153642578125E+04, -.1208223266602E+04, .7100001831055E+03},
 {   21646, .8192888671875E+04, -.1213251220703E+04, .7039238891602E+03},
 {   21647, .8189142578125E+04, -.1218432250977E+04, .7015624389648E+03},
 {   21648, .8184039550781E+04, -.1222620239258E+04, .6999975585938E+03},
 {   21649, .8177639160156E+04, -.1224869140625E+04, .6997720947266E+03},
 {   21650, .8170914062500E+04, -.1225275146484E+04, .7007047729492E+03},
 {   21651, .8164520996094E+04, -.1223978149414E+04, .7026284790039E+03},
 {   21652, .8159138183594E+04, -.1220930175781E+04, .7054223632812E+03},
 {   21653, .8155733398438E+04, -.1216167236328E+04, .7088496093750E+03},
 {   21654, .8154012207031E+04, -.1210620239258E+04, .7124150390625E+03},
 {   21655, .8194472656250E+04, -.1214481201172E+04, .7069899291992E+03},
 {   21656, .8191056640625E+04, -.1220404174805E+04, .7076971435547E+03},
 {   21657, .8185963378906E+04, -.1224912231445E+04, .7087244873047E+03},
 {   21658, .8179768066406E+04, -.1227595214844E+04, .7100441894531E+03},
 {   21659, .8173078125000E+04, -.1228221191406E+04, .7115080566406E+03},
 {   21660, .8166519531250E+04, -.1226701171875E+04, .7129458007812E+03},
 {   21661, .8160733398438E+04, -.1223210205078E+04, .7142226562500E+03},
 {   21662, .8156285156250E+04, -.1218059204102E+04, .7152209472656E+03},
 {   21663, .8153674804688E+04, -.1211729248047E+04, .7158153686523E+03},
 {   21664, .8195221679688E+04, -.1213571166992E+04, .7101920166016E+03},
 {   21665, .8192519531250E+04, -.1218680175781E+04, .7138881225586E+03},
 {   21666, .8188022949219E+04, -.1222564208984E+04, .7173303833008E+03},
 {   21667, .8182229492188E+04, -.1224875244141E+04, .7202001342773E+03},
 {   21668, .8175679199219E+04, -.1225389160156E+04, .7222034912109E+03},
 {   21669, .8169006347656E+04, -.1224060180664E+04, .7231304931641E+03},
 {   21670, .8162910156250E+04, -.1220940185547E+04, .7228522949219E+03},
 {   21671, .8158178710938E+04, -.1216211181641E+04, .7212987060547E+03},
 {   21672, .8154477050781E+04, -.1210828247070E+04, .7191104736328E+03},
 {   21673, .8153794921875E+04, -.1199078247070E+04, .7123549804688E+03},
 {   21674, .8156473632812E+04, -.1193985229492E+04, .7086726684570E+03},
 {   21675, .8160935058594E+04, -.1190102294922E+04, .7052409667969E+03},
 {   21676, .8166692382812E+04, -.1187773315430E+04, .7023713989258E+03},
 {   21677, .8179854492188E+04, -.1188514282227E+04, .6994088745117E+03},
 {   21678, .8185952148438E+04, -.1191574218750E+04, .6996544799805E+03},
 {   21679, .8190535644531E+04, -.1196355224609E+04, .7012877807617E+03},
 {   21680, .8193614257812E+04, -.1201950195312E+04, .7037759399414E+03},
 {   21681, .8195376953125E+04, -.1200878173828E+04, .7067612915039E+03},
 {   21682, .8192784179688E+04, -.1194525268555E+04, .7073616333008E+03},
 {   21683, .8188328613281E+04, -.1189353271484E+04, .7083522338867E+03},
 {   21684, .8182530761719E+04, -.1185847290039E+04, .7096322631836E+03},
 {   21685, .8169251464844E+04, -.1184949218750E+04, .7125238037109E+03},
 {   21686, .8163046875000E+04, -.1187647216797E+04, .7138609008789E+03},
 {   21687, .8157965820312E+04, -.1192176269531E+04, .7149591674805E+03},
 {   21688, .8154562011719E+04, -.1198123291016E+04, .7156793212891E+03},
 {   21689, .8195988281250E+04, -.1201849243164E+04, .7101198730469E+03},
 {   21690, .8193982421875E+04, -.1196366210938E+04, .7137456054688E+03},
 {   21691, .8190018066406E+04, -.1191859252930E+04, .7171055297852E+03},
 {   21692, .8184544921875E+04, -.1188750244141E+04, .7198762817383E+03},
 {   21693, .8171305664062E+04, -.1187854248047E+04, .7227760009766E+03},
 {   21694, .8164818847656E+04, -.1190141235352E+04, .7225996704102E+03},
 {   21695, .8159268554688E+04, -.1193978271484E+04, .7212666015625E+03},
 {   21696, .8155251464844E+04, -.1199050292969E+04, .7189445190430E+03},
 {   21697, .8155152343750E+04, -.1208363159180E+04, .7215274047852E+03},
 {   21698, .8159102539062E+04, -.1211657226562E+04, .7260975952148E+03},
 {   21699, .8164607421875E+04, -.1214415161133E+04, .7291770629883E+03},
 {   21700, .8171070312500E+04, -.1216384155273E+04, .7305156860352E+03},
 {   21701, .8177865722656E+04, -.1217378173828E+04, .7299890747070E+03},
 {   21702, .8184322753906E+04, -.1217270263672E+04, .7276003417969E+03},
 {   21703, .8189833496094E+04, -.1216107177734E+04, .7236385498047E+03},
 {   21704, .8193892578125E+04, -.1214006225586E+04, .7184855957031E+03},
 {   21705, .8195984375000E+04, -.1211124267578E+04, .7126017456055E+03},
 {   21706, .8195332031250E+04, -.1207649169922E+04, .7133690185547E+03},
 {   21707, .8193918945312E+04, -.1207489257812E+04, .7200334472656E+03},
 {   21708, .8190774902344E+04, -.1207230224609E+04, .7260386352539E+03},
 {   21709, .8185554687500E+04, -.1206859252930E+04, .7303607788086E+03},
 {   21710, .8179214843750E+04, -.1206425170898E+04, .7327789306641E+03},
 {   21711, .8172445800781E+04, -.1205966186523E+04, .7332219848633E+03},
 {   21712, .8165904296875E+04, -.1205538208008E+04, .7314699707031E+03},
 {   21713, .8160540527344E+04, -.1205223266602E+04, .7273289184570E+03},
 {   21714, .8156467285156E+04, -.1205018188477E+04, .7218662109375E+03},
 {   21715, .8196394531250E+04, -.1204360229492E+04, .7126006469727E+03},
 {   21716, .8194714843750E+04, -.1201080200195E+04, .7184471435547E+03},
 {   21717, .8190979492188E+04, -.1198300292969E+04, .7235641479492E+03},
 {   21718, .8185623535156E+04, -.1196344238281E+04, .7274799194336E+03},
 {   21719, .8172288574219E+04, -.1195448242188E+04, .7303920898438E+03},
 {   21720, .8165631835938E+04, -.1196635253906E+04, .7289879760742E+03},
 {   21721, .8160062988281E+04, -.1198913208008E+04, .7255737304688E+03},
 {   21722, .8155458496094E+04, -.1201662231445E+04, .7211976318359E+03},
 {   21723, .8052436035156E+04, -.1408130859375E+04, .7039116821289E+03},
 {   21724, .8055514160156E+04, -.1417992797852E+04, .7135186157227E+03},
 {   21725, .8065398437500E+04, -.1418058837891E+04, .7113955078125E+03},
 {   21726, .8062355468750E+04, -.1407457885742E+04, .7017477416992E+03},
 {   21727, .8079211425781E+04, -.1418377807617E+04, .7085754394531E+03},
 {   21728, .8076205078125E+04, -.1407036865234E+04, .6988886718750E+03},
 {   21729, .8089093261719E+04, -.1418443847656E+04, .7064526977539E+03},
 {   21730, .8086124023438E+04, -.1406362792969E+04, .6967251586914E+03},
 {   21731, .8098975097656E+04, -.1418509765625E+04, .7043301391602E+03},
 {   21732, .8096041992188E+04, -.1405689819336E+04, .6945616455078E+03},
 {   21733, .8108860839844E+04, -.1418576782227E+04, .7022080078125E+03},
 {   21734, .8105959960938E+04, -.1405015869141E+04, .6923994140625E+03},
 {   21735, .8059492675781E+04, -.1421563842773E+04, .7265797119141E+03},
 {   21736, .8069591308594E+04, -.1422212768555E+04, .7247914428711E+03},
 {   21737, .8083618164062E+04, -.1423114746094E+04, .7223063964844E+03},
 {   21738, .8093716796875E+04, -.1423762817383E+04, .7205175170898E+03},
 {   21739, .8103814941406E+04, -.1424411743164E+04, .7187293701172E+03},
 {   21740, .8113912109375E+04, -.1425060791016E+04, .7169401245117E+03},
 {   21741, .8063287109375E+04, -.1417843750000E+04, .7396488037109E+03},
 {   21742, .8073680175781E+04, -.1419478759766E+04, .7384290161133E+03},
 {   21743, .8088003417969E+04, -.1421366821289E+04, .7365135498047E+03},
 {   21744, .8098396484375E+04, -.1423002807617E+04, .7352937622070E+03},
 {   21745, .8108788574219E+04, -.1424637817383E+04, .7340743408203E+03},
 {   21746, .8119186523438E+04, -.1426273803711E+04, .7328541870117E+03},
 {   21747, .8065876464844E+04, -.1407761840820E+04, .7491726074219E+03},
 {   21748, .8075711425781E+04, -.1408715820312E+04, .7479199218750E+03},
 {   21749, .8089476562500E+04, -.1409921875000E+04, .7459698486328E+03},
 {   21750, .8099311523438E+04, -.1410874755859E+04, .7447159423828E+03},
 {   21751, .8109146972656E+04, -.1411828857422E+04, .7434628295898E+03},
 {   21752, .8118982421875E+04, -.1412781738281E+04, .7422099609375E+03},
 {   21753, .8066589355469E+04, -.1394093872070E+04, .7526265869141E+03},
 {   21754, .8075870605469E+04, -.1394484863281E+04, .7513947143555E+03},
 {   21755, .8089080566406E+04, -.1395127807617E+04, .7494659423828E+03},
 {   21756, .8098364257812E+04, -.1395517822266E+04, .7482339477539E+03},
 {   21757, .8107643066406E+04, -.1395908813477E+04, .7470022583008E+03},
 {   21758, .8116922363281E+04, -.1396298828125E+04, .7457697143555E+03},
 {   21759, .8051113769531E+04, -.1393816894531E+04, .7004506835938E+03},
 {   21760, .8061389648438E+04, -.1393102905273E+04, .6983147583008E+03},
 {   21761, .8075595703125E+04, -.1392641845703E+04, .6954826049805E+03},
 {   21762, .8085875000000E+04, -.1391927856445E+04, .6933463745117E+03},
 {   21763, .8096149902344E+04, -.1391213867188E+04, .6912104492188E+03},
 {   21764, .8106425292969E+04, -.1390499877930E+04, .6890746459961E+03},
 {   21765, .8051983398438E+04, -.1379653930664E+04, .7045747680664E+03},
 {   21766, .8062117187500E+04, -.1378960937500E+04, .7023731689453E+03},
 {   21767, .8076183593750E+04, -.1378519897461E+04, .6994753417969E+03},
 {   21768, .8086320312500E+04, -.1377826904297E+04, .6972740478516E+03},
 {   21769, .8096454589844E+04, -.1377132934570E+04, .6950726318359E+03},
 {   21770, .8106592285156E+04, -.1376439941406E+04, .6928708496094E+03},
 {   21771, .8098345214844E+04, -.1367433959961E+04, .7060205078125E+03},
 {   21772, .8108224609375E+04, -.1366757934570E+04, .7038764648438E+03},
 {   21773, .8101553222656E+04, -.1364837890625E+04, .7202037353516E+03},
 {   21774, .8111214355469E+04, -.1364168945312E+04, .7181173095703E+03},
 {   21775, .8088467285156E+04, -.1368109863281E+04, .7081635742188E+03},
 {   21776, .8091885742188E+04, -.1365505859375E+04, .7222897338867E+03},
 {   21777, .8078586425781E+04, -.1368785888672E+04, .7103065795898E+03},
 {   21778, .8082224609375E+04, -.1366173950195E+04, .7243760986328E+03},
 {   21779, .8064777832031E+04, -.1369208862305E+04, .7131454467773E+03},
 {   21780, .8068631347656E+04, -.1366589843750E+04, .7271575317383E+03},
 {   21781, .8054899414062E+04, -.1369883911133E+04, .7152893676758E+03},
 {   21782, .8058968750000E+04, -.1367257934570E+04, .7292441406250E+03},
 {   21783, .8065222656250E+04, -.1380504882812E+04, .7491599121094E+03},
 {   21784, .8062552734375E+04, -.1371375854492E+04, .7407286987305E+03},
 {   21785, .8072062011719E+04, -.1370729858398E+04, .7386923828125E+03},
 {   21786, .8074216308594E+04, -.1379878906250E+04, .7472266235352E+03},
 {   21787, .8085504882812E+04, -.1370335937500E+04, .7359612426758E+03},
 {   21788, .8087141601562E+04, -.1379504882812E+04, .7445961303711E+03},
 {   21789, .8095011230469E+04, -.1369689941406E+04, .7339264526367E+03},
 {   21790, .8096136718750E+04, -.1378878906250E+04, .7426611328125E+03},
 {   21791, .8104521972656E+04, -.1369043945312E+04, .7318911132812E+03},
 {   21792, .8105130371094E+04, -.1378252929688E+04, .7407279663086E+03},
 {   21793, .8114035156250E+04, -.1368396850586E+04, .7298559570312E+03},
 {   21794, .8114123535156E+04, -.1377626953125E+04, .7387937622070E+03},
 {   21795, .8121878906250E+04, -.1425897827148E+04, .7148508300781E+03},
 {   21796, .8126690429688E+04, -.1425801757812E+04, .7287947998047E+03},
 {   21797, .8129837402344E+04, -.1426322753906E+04, .7122255859375E+03},
 {   21798, .8134390136719E+04, -.1425782836914E+04, .7240167236328E+03},
 {   21799, .8138342285156E+04, -.1426438720703E+04, .7105998535156E+03},
 {   21800, .8142857421875E+04, -.1425388793945E+04, .7208671875000E+03},
 {   21801, .8147216796875E+04, -.1426113769531E+04, .7103970947266E+03},
 {   21802, .8151808593750E+04, -.1424557861328E+04, .7198991088867E+03},
 {   21803, .8156119628906E+04, -.1425054809570E+04, .7114542236328E+03},
 {   21804, .8160719726562E+04, -.1423085815430E+04, .7206544799805E+03},
 {   21805, .8164629882812E+04, -.1421796752930E+04, .7118492431641E+03},
 {   21806, .8169191894531E+04, -.1420034790039E+04, .7202308959961E+03},
 {   21807, .8170502929688E+04, -.1414921752930E+04, .7104499511719E+03},
 {   21808, .8175173828125E+04, -.1414034790039E+04, .7174332885742E+03},
 {   21809, .8173439941406E+04, -.1406332763672E+04, .7084673461914E+03},
 {   21810, .8177521484375E+04, -.1406013793945E+04, .7140462646484E+03},
 {   21811, .8117005859375E+04, -.1420453857422E+04, .7018687133789E+03},
 {   21812, .8125229980469E+04, -.1421857788086E+04, .7013039550781E+03},
 {   21813, .8133772949219E+04, -.1423014770508E+04, .7011102905273E+03},
 {   21814, .8142575683594E+04, -.1423728759766E+04, .7015799560547E+03},
 {   21815, .8151475097656E+04, -.1423616821289E+04, .7028460083008E+03},
 {   21816, .8160032226562E+04, -.1420684814453E+04, .7039680175781E+03},
 {   21817, .8165798339844E+04, -.1413467773438E+04, .7038760375977E+03},
 {   21818, .8114178710938E+04, -.1408170776367E+04, .6935339965820E+03},
 {   21819, .8122441894531E+04, -.1411241821289E+04, .6945385742188E+03},
 {   21820, .8130801757812E+04, -.1414130859375E+04, .6952843627930E+03},
 {   21821, .8139305664062E+04, -.1416574829102E+04, .6959375000000E+03},
 {   21822, .8147982421875E+04, -.1418098754883E+04, .6969221191406E+03},
 {   21823, .8156358886719E+04, -.1416182861328E+04, .6982789306641E+03},
 {   21824, .8161795410156E+04, -.1409313842773E+04, .6990027465820E+03},
 {   21825, .8165699218750E+04, -.1401345825195E+04, .6991533203125E+03},
 {   21826, .8169554687500E+04, -.1393366821289E+04, .6996404418945E+03},
 {   21827, .8173406738281E+04, -.1385397827148E+04, .7003129272461E+03},
 {   21828, .8175953125000E+04, -.1388318847656E+04, .7110737915039E+03},
 {   21829, .8177401367188E+04, -.1397221801758E+04, .7119645385742E+03},
 {   21830, .8122353515625E+04, -.1397301879883E+04, .7401005859375E+03},
 {   21831, .8119377441406E+04, -.1382831909180E+04, .7344233398438E+03},
 {   21832, .8128714355469E+04, -.1400933837891E+04, .7352561645508E+03},
 {   21833, .8125090820312E+04, -.1387973876953E+04, .7305941772461E+03},
 {   21834, .8135894042969E+04, -.1404268798828E+04, .7317920532227E+03},
 {   21835, .8131647949219E+04, -.1392855834961E+04, .7280189819336E+03},
 {   21836, .8143831542969E+04, -.1406979858398E+04, .7302881469727E+03},
 {   21837, .8139097167969E+04, -.1397021850586E+04, .7272964477539E+03},
 {   21838, .8152207519531E+04, -.1408477783203E+04, .7301668701172E+03},
 {   21839, .8147225585938E+04, -.1399687866211E+04, .7277849121094E+03},
 {   21840, .8160699218750E+04, -.1407875854492E+04, .7297615966797E+03},
 {   21841, .8155749023438E+04, -.1399993774414E+04, .7284355468750E+03},
 {   21842, .8167950683594E+04, -.1404005859375E+04, .7281629638672E+03},
 {   21843, .8163649902344E+04, -.1396872802734E+04, .7284857788086E+03},
 {   21844, .8171558105469E+04, -.1396746826172E+04, .7271983642578E+03},
 {   21845, .8167910644531E+04, -.1389677856445E+04, .7297799682617E+03},
 {   21846, .8174682128906E+04, -.1389356811523E+04, .7262725830078E+03},
 {   21847, .8172681640625E+04, -.1382672851562E+04, .7304796752930E+03},
 {   21848, .8178098144531E+04, -.1385285888672E+04, .7218286132812E+03},
 {   21849, .8179302246094E+04, -.1377253906250E+04, .7297980346680E+03},
 {   21850, .8125215820312E+04, -.1412655761719E+04, .7371417846680E+03},
 {   21851, .8132187500000E+04, -.1414465820312E+04, .7320809326172E+03},
 {   21852, .8139954101562E+04, -.1415939819336E+04, .7285238647461E+03},
 {   21853, .8148341308594E+04, -.1416883789062E+04, .7270369873047E+03},
 {   21854, .8156923339844E+04, -.1416899780273E+04, .7271046142578E+03},
 {   21855, .8165349121094E+04, -.1415076782227E+04, .7264391479492E+03},
 {   21856, .8171906738281E+04, -.1410145751953E+04, .7239916381836E+03},
 {   21857, .8174827636719E+04, -.1402508789062E+04, .7215644531250E+03},
 {   21858, .8176271484375E+04, -.1394421875000E+04, .7198107299805E+03},
 {   21859, .8195958007812E+04, -.1364874877930E+04, .7065745239258E+03},
 {   21860, .8192962890625E+04, -.1368304931641E+04, .7014369506836E+03},
 {   21861, .8193517578125E+04, -.1364973876953E+04, .7001234741211E+03},
 {   21862, .8189261718750E+04, -.1371631835938E+04, .6967215576172E+03},
 {   21863, .8189167480469E+04, -.1365195922852E+04, .6948065795898E+03},
 {   21864, .8184151367188E+04, -.1374530883789E+04, .6932558593750E+03},
 {   21865, .8183333984375E+04, -.1365535888672E+04, .6911610717773E+03},
 {   21866, .8177736328125E+04, -.1376465942383E+04, .6919624023438E+03},
 {   21867, .8176689941406E+04, -.1365967895508E+04, .6894052124023E+03},
 {   21868, .8170989257812E+04, -.1377389892578E+04, .6925991821289E+03},
 {   21869, .8169833007812E+04, -.1366434936523E+04, .6897471923828E+03},
 {   21870, .8164582519531E+04, -.1377271850586E+04, .6949878540039E+03},
 {   21871, .8163441894531E+04, -.1366869873047E+04, .6922641601562E+03},
 {   21872, .8159130859375E+04, -.1376096923828E+04, .6989499511719E+03},
 {   21873, .8158128906250E+04, -.1367248901367E+04, .6966283569336E+03},
 {   21874, .8155175292969E+04, -.1373973876953E+04, .7041077270508E+03},
 {   21875, .8154401367188E+04, -.1367553955078E+04, .7024074707031E+03},
 {   21876, .8153127929688E+04, -.1371099853516E+04, .7099622192383E+03},
 {   21877, .8152691406250E+04, -.1367737915039E+04, .7090676879883E+03},
 {   21878, .8153061035156E+04, -.1367773925781E+04, .7159572143555E+03},
 {   21879, .8193445312500E+04, -.1361613891602E+04, .7010962524414E+03},
 {   21880, .8189135742188E+04, -.1358794921875E+04, .6966043090820E+03},
 {   21881, .8183368652344E+04, -.1356716918945E+04, .6935700683594E+03},
 {   21882, .8176759765625E+04, -.1355571899414E+04, .6922428588867E+03},
 {   21883, .8169938476562E+04, -.1355469970703E+04, .6927431640625E+03},
 {   21884, .8163541992188E+04, -.1356389892578E+04, .6949742431641E+03},
 {   21885, .8158240234375E+04, -.1358301879883E+04, .6988291625977E+03},
 {   21886, .8155040527344E+04, -.1361151855469E+04, .7041366577148E+03},
 {   21887, .8153642578125E+04, -.1364413940430E+04, .7100001831055E+03},
 {   21888, .8192888671875E+04, -.1359386962891E+04, .7039239501953E+03},
 {   21889, .8189142578125E+04, -.1354205932617E+04, .7015624389648E+03},
 {   21890, .8184039550781E+04, -.1350017944336E+04, .6999975585938E+03},
 {   21891, .8177639160156E+04, -.1347768920898E+04, .6997720947266E+03},
 {   21892, .8170914062500E+04, -.1347362915039E+04, .7007047729492E+03},
 {   21893, .8164520996094E+04, -.1348659912109E+04, .7026284790039E+03},
 {   21894, .8159138183594E+04, -.1351707885742E+04, .7054223632812E+03},
 {   21895, .8155733398438E+04, -.1356470947266E+04, .7088496093750E+03},
 {   21896, .8154012207031E+04, -.1362017944336E+04, .7124150390625E+03},
 {   21897, .8194472656250E+04, -.1358155883789E+04, .7069899291992E+03},
 {   21898, .8191056640625E+04, -.1352233886719E+04, .7076971435547E+03},
 {   21899, .8185963378906E+04, -.1347725952148E+04, .7087244873047E+03},
 {   21900, .8179768066406E+04, -.1345041992188E+04, .7100441894531E+03},
 {   21901, .8173078125000E+04, -.1344416992188E+04, .7115080566406E+03},
 {   21902, .8166519531250E+04, -.1345936889648E+04, .7129458007812E+03},
 {   21903, .8160733398438E+04, -.1349427001953E+04, .7142227172852E+03},
 {   21904, .8156285156250E+04, -.1354578979492E+04, .7152210083008E+03},
 {   21905, .8153674804688E+04, -.1360907958984E+04, .7158153686523E+03},
 {   21906, .8195221679688E+04, -.1359066894531E+04, .7101920166016E+03},
 {   21907, .8192519531250E+04, -.1353957885742E+04, .7138881225586E+03},
 {   21908, .8188022949219E+04, -.1350073974609E+04, .7173305053711E+03},
 {   21909, .8182229492188E+04, -.1347761962891E+04, .7202001342773E+03},
 {   21910, .8175679199219E+04, -.1347248901367E+04, .7222034912109E+03},
 {   21911, .8169006347656E+04, -.1348577880859E+04, .7231304931641E+03},
 {   21912, .8162911132812E+04, -.1351697875977E+04, .7228522949219E+03},
 {   21913, .8158178710938E+04, -.1356425903320E+04, .7212987060547E+03},
 {   21914, .8154477050781E+04, -.1361809936523E+04, .7191104736328E+03},
 {   21915, .8153794921875E+04, -.1373558837891E+04, .7123549804688E+03},
 {   21916, .8156473632812E+04, -.1378652832031E+04, .7086726684570E+03},
 {   21917, .8160935058594E+04, -.1382535888672E+04, .7052410278320E+03},
 {   21918, .8166692382812E+04, -.1384864868164E+04, .7023713989258E+03},
 {   21919, .8179854492188E+04, -.1384123901367E+04, .6994088745117E+03},
 {   21920, .8185952148438E+04, -.1381062866211E+04, .6996545410156E+03},
 {   21921, .8190535644531E+04, -.1376282836914E+04, .7012877807617E+03},
 {   21922, .8193614257812E+04, -.1370687866211E+04, .7037759399414E+03},
 {   21923, .8195376953125E+04, -.1371759887695E+04, .7067613525391E+03},
 {   21924, .8192784179688E+04, -.1378112915039E+04, .7073616333008E+03},
 {   21925, .8188328613281E+04, -.1383283813477E+04, .7083522338867E+03},
 {   21926, .8182530761719E+04, -.1386790893555E+04, .7096322631836E+03},
 {   21927, .8169251464844E+04, -.1387688842773E+04, .7125238037109E+03},
 {   21928, .8163046875000E+04, -.1384990844727E+04, .7138609619141E+03},
 {   21929, .8157965820312E+04, -.1380461914062E+04, .7149591674805E+03},
 {   21930, .8154562011719E+04, -.1374514892578E+04, .7156793212891E+03},
 {   21931, .8195988281250E+04, -.1370787841797E+04, .7101198730469E+03},
 {   21932, .8193982421875E+04, -.1376271850586E+04, .7137456054688E+03},
 {   21933, .8190018066406E+04, -.1380778930664E+04, .7171055297852E+03},
 {   21934, .8184544921875E+04, -.1383886840820E+04, .7198762817383E+03},
 {   21935, .8171305664062E+04, -.1384782836914E+04, .7227760009766E+03},
 {   21936, .8164818847656E+04, -.1382496826172E+04, .7225996704102E+03},
 {   21937, .8159268554688E+04, -.1378659912109E+04, .7212666015625E+03},
 {   21938, .8155251464844E+04, -.1373587890625E+04, .7189445190430E+03},
 {   21939, .8155152343750E+04, -.1364274902344E+04, .7215274047852E+03},
 {   21940, .8159102539062E+04, -.1360980957031E+04, .7260975952148E+03},
 {   21941, .8164607421875E+04, -.1358222900391E+04, .7291771850586E+03},
 {   21942, .8171070312500E+04, -.1356252929688E+04, .7305157470703E+03},
 {   21943, .8177865722656E+04, -.1355259887695E+04, .7299890747070E+03},
 {   21944, .8184322753906E+04, -.1355367919922E+04, .7276003417969E+03},
 {   21945, .8189833496094E+04, -.1356530883789E+04, .7236385498047E+03},
 {   21946, .8193892578125E+04, -.1358631958008E+04, .7184855957031E+03},
 {   21947, .8195984375000E+04, -.1361512939453E+04, .7126018066406E+03},
 {   21948, .8195332031250E+04, -.1364988891602E+04, .7133690185547E+03},
 {   21949, .8193918945312E+04, -.1365148925781E+04, .7200334472656E+03},
 {   21950, .8190774902344E+04, -.1365407958984E+04, .7260386352539E+03},
 {   21951, .8185554687500E+04, -.1365778930664E+04, .7303607788086E+03},
 {   21952, .8179214843750E+04, -.1366211914062E+04, .7327789306641E+03},
 {   21953, .8172445800781E+04, -.1366671875000E+04, .7332219848633E+03},
 {   21954, .8165904296875E+04, -.1367099853516E+04, .7314699707031E+03},
 {   21955, .8160540527344E+04, -.1367413940430E+04, .7273289184570E+03},
 {   21956, .8156467285156E+04, -.1367619873047E+04, .7218662109375E+03},
 {   21957, .8196394531250E+04, -.1368277832031E+04, .7126007080078E+03},
 {   21958, .8194714843750E+04, -.1371556884766E+04, .7184471435547E+03},
 {   21959, .8190979492188E+04, -.1374336914062E+04, .7235642089844E+03},
 {   21960, .8185623535156E+04, -.1376293823242E+04, .7274799194336E+03},
 {   21961, .8172288574219E+04, -.1377188842773E+04, .7303920898438E+03},
 {   21962, .8165631835938E+04, -.1376002929688E+04, .7289879760742E+03},
 {   21963, .8160062988281E+04, -.1373724853516E+04, .7255737304688E+03},
 {   21964, .8155458496094E+04, -.1370974853516E+04, .7211976318359E+03},
 {   21965, .8283733398438E+04, -.1286886108398E+04, .6775573730469E+03},
 {   21966, .8242058593750E+04, -.1291262084961E+04, .6517556762695E+03},
 {   21967, .8293073242188E+04, -.1288069091797E+04, .6754035034180E+03},
 {   21968, .8269813476562E+04, -.1288185058594E+04, .6399351806641E+03},
 {   21969, .8227546875000E+04, -.1304554077148E+04, .6557084960938E+03},
 {   22253, .8174509765625E+04, -.1206313232422E+04, .7112658691406E+03},
 {   22254, .8174509765625E+04, -.1206313232422E+04, .7112658691406E+03},
 {   22255, .8174509765625E+04, -.1366324462891E+04, .7112659301758E+03},
 {   22256, .8174509765625E+04, -.1366324462891E+04, .7112659301758E+03},
 {   22257, .8013943359375E+04, -.1128202392578E+04, .7967790527344E+03},
 {   22258, .8016175292969E+04, -.1155288330078E+04, .8085103759766E+03},
 {   22259, .8062068359375E+04, -.1155171386719E+04, .7983302001953E+03},
 {   22260, .8059835937500E+04, -.1128084350586E+04, .7865989990234E+03},
 {   22261, .8053621582031E+04, -.1170226318359E+04, .7595177612305E+03},
 {   22262, .8051879394531E+04, -.1157256347656E+04, .7521077880859E+03},
 {   22263, .8007357910156E+04, -.1170236328125E+04, .7697412719727E+03},
 {   22264, .8005913574219E+04, -.1157226318359E+04, .7624175415039E+03},
 {   22265, .8009550292969E+04, -.1107927368164E+04, .7741533813477E+03},
 {   22266, .8055442871094E+04, -.1107809448242E+04, .7639741210938E+03},
 {   22267, .8049229003906E+04, -.1147586303711E+04, .7408461914062E+03},
 {   22268, .8003436523438E+04, -.1147276367188E+04, .7507937011719E+03},
 {   22269, .8002781738281E+04, -.1097861450195E+04, .7442795410156E+03},
 {   22270, .8048673828125E+04, -.1097744384766E+04, .7341000366211E+03},
 {   22271, .8046224609375E+04, -.1142591308594E+04, .7265540161133E+03},
 {   22272, .8000319335938E+04, -.1142526367188E+04, .7366719970703E+03},
 {   22273, .7995308105469E+04, -.1098976440430E+04, .7144840698242E+03},
 {   22274, .8041201660156E+04, -.1098859497070E+04, .7043046264648E+03},
 {   22275, .8042905273438E+04, -.1142411376953E+04, .7119089965820E+03},
 {   22276, .7997060058594E+04, -.1142356323242E+04, .7222951049805E+03},
 {   22277, .7988666015625E+04, -.1111748413086E+04, .6856539306641E+03},
 {   22278, .8034556152344E+04, -.1111631469727E+04, .6754736328125E+03},
 {   22279, .8039732910156E+04, -.1147980346680E+04, .6983287963867E+03},
 {   22280, .7993872070312E+04, -.1147926391602E+04, .7086503906250E+03},
 {   22281, .7984201171875E+04, -.1132791381836E+04, .6667402954102E+03},
 {   22282, .8030093261719E+04, -.1132674316406E+04, .6565598754883E+03},
 {   22283, .8037016113281E+04, -.1157508300781E+04, .6870425415039E+03},
 {   22284, .7991133300781E+04, -.1157546386719E+04, .6972677612305E+03},
 {   22285, .7981704589844E+04, -.1159501342773E+04, .6562225341797E+03},
 {   22286, .8027595703125E+04, -.1159383300781E+04, .6460433349609E+03},
 {   22287, .8035183593750E+04, -.1170596313477E+04, .6799106445312E+03},
 {   22288, .7989313964844E+04, -.1170576293945E+04, .6901903076172E+03},
 {   22289, .7980782226562E+04, -.1186754272461E+04, .6532086181641E+03},
 {   22290, .8026673828125E+04, -.1186636230469E+04, .6430294189453E+03},
 {   22291, .8034413574219E+04, -.1185408203125E+04, .6775649414062E+03},
 {   22292, .7988546875000E+04, -.1185286254883E+04, .6878453369141E+03},
 {   22293, .8016524414062E+04, -.1183771240234E+04, .8102846679688E+03},
 {   22294, .8062415527344E+04, -.1183654296875E+04, .8001046142578E+03},
 {   22295, .8053572753906E+04, -.1184572265625E+04, .7620023193359E+03},
 {   22296, .8007711425781E+04, -.1184246215820E+04, .7722954101562E+03},
 {   22297, .8015164062500E+04, -.1210531250000E+04, .8053560791016E+03},
 {   22298, .8061055175781E+04, -.1210413208008E+04, .7951768188477E+03},
 {   22299, .8052792968750E+04, -.1199382202148E+04, .7596420288086E+03},
 {   22300, .8006958007812E+04, -.1199023193359E+04, .7700522460938E+03},
 {   22301, .8012124023438E+04, -.1239563110352E+04, .7941373291016E+03},
 {   22302, .8058018066406E+04, -.1239445190430E+04, .7839579467773E+03},
 {   22303, .8050964843750E+04, -.1212464233398E+04, .7524988403320E+03},
 {   22304, .8005070800781E+04, -.1212582275391E+04, .7626783447266E+03},
 {   22305, .8007464355469E+04, -.1261844116211E+04, .7749791870117E+03},
 {   22306, .8053355957031E+04, -.1261726074219E+04, .7647996826172E+03},
 {   22307, .8048270507812E+04, -.1222160156250E+04, .7413513793945E+03},
 {   22308, .8002286621094E+04, -.1222448242188E+04, .7511516723633E+03},
 {   22309, .8002013671875E+04, -.1275001098633E+04, .7475679931641E+03},
 {   22310, .8047903808594E+04, -.1274883056641E+04, .7373876342773E+03},
 {   22311, .8045088378906E+04, -.1227525146484E+04, .7276940917969E+03},
 {   22312, .7999215820312E+04, -.1227506225586E+04, .7379631958008E+03},
 {   22313, .7994621093750E+04, -.1276145141602E+04, .7128544921875E+03},
 {   22314, .8040512207031E+04, -.1276028076172E+04, .7026752929688E+03},
 {   22315, .8041775878906E+04, -.1227361206055E+04, .7130460205078E+03},
 {   22316, .7995907714844E+04, -.1227331176758E+04, .7233336181641E+03},
 {   22317, .7988525878906E+04, -.1265050048828E+04, .6871321411133E+03},
 {   22318, .8034417968750E+04, -.1264933105469E+04, .6769516601562E+03},
 {   22319, .8038694824219E+04, -.1222651245117E+04, .6991224975586E+03},
 {   22320, .7992801757812E+04, -.1222572143555E+04, .7092978515625E+03},
 {   22321, .7983458984375E+04, -.1242049194336E+04, .6655911254883E+03},
 {   22322, .8029351074219E+04, -.1241932128906E+04, .6554116210938E+03},
 {   22323, .8036276855469E+04, -.1213100219727E+04, .6877921752930E+03},
 {   22324, .7990401855469E+04, -.1213034179688E+04, .6980446777344E+03},
 {   22325, .7981023925781E+04, -.1213621215820E+04, .6551363525391E+03},
 {   22326, .8026915527344E+04, -.1213503173828E+04, .6449560546875E+03},
 {   22327, .8034793457031E+04, -.1200169189453E+04, .6803015747070E+03},
 {   22328, .7988929199219E+04, -.1200191284180E+04, .6906033325195E+03},
 {   22329, .8001220703125E+04, -.1163676269531E+04, .7422080688477E+03},
 {   22330, .7999464355469E+04, -.1160766357422E+04, .7342177124023E+03},
 {   22331, .7960611328125E+04, -.1162425292969E+04, .7406535644531E+03},
 {   22332, .7961754882812E+04, -.1165219238281E+04, .7484312133789E+03},
 {   22333, .8002614746094E+04, -.1169176269531E+04, .7487508544922E+03},
 {   22334, .7962594726562E+04, -.1170051269531E+04, .7545281982422E+03},
 {   22335, .8003477050781E+04, -.1176646240234E+04, .7530625000000E+03},
 {   22336, .7963161621094E+04, -.1177509277344E+04, .7592705688477E+03},
 {   22337, .8003686523438E+04, -.1185176269531E+04, .7546102294922E+03},
 {   22338, .7963242187500E+04, -.1186688232422E+04, .7611640625000E+03},
 {   22339, .8003222167969E+04, -.1193724243164E+04, .7532058105469E+03},
 {   22340, .7962863281250E+04, -.1194926269531E+04, .7599011230469E+03},
 {   22341, .8002158203125E+04, -.1201237304688E+04, .7490296630859E+03},
 {   22342, .7962094238281E+04, -.1202274169922E+04, .7560422363281E+03},
 {   22343, .8000607910156E+04, -.1206870239258E+04, .7426120605469E+03},
 {   22344, .7961021484375E+04, -.1207878173828E+04, .7499358520508E+03},
 {   22345, .7998765136719E+04, -.1209879272461E+04, .7346968383789E+03},
 {   22346, .7959524902344E+04, -.1211538208008E+04, .7408673095703E+03},
 {   22347, .7996843261719E+04, -.1209972167969E+04, .7262283935547E+03},
 {   22348, .7958167968750E+04, -.1211493164062E+04, .7321204223633E+03},
 {   22349, .7995077636719E+04, -.1207103271484E+04, .7182316894531E+03},
 {   22350, .7957089843750E+04, -.1208893188477E+04, .7248180541992E+03},
 {   22351, .7993676269531E+04, -.1201598266602E+04, .7116607666016E+03},
 {   22352, .7956108886719E+04, -.1202947265625E+04, .7176415405273E+03},
 {   22353, .7992821777344E+04, -.1194119262695E+04, .7073294677734E+03},
 {   22354, .7955615722656E+04, -.1195242309570E+04, .7133008422852E+03},
 {   22355, .7997538574219E+04, -.1160856323242E+04, .7257708129883E+03},
 {   22356, .7959403320312E+04, -.1162340332031E+04, .7329229125977E+03},
 {   22357, .7995688476562E+04, -.1163876342773E+04, .7178160400391E+03},
 {   22358, .7958051757812E+04, -.1165130371094E+04, .7246262817383E+03},
 {   22359, .7992599609375E+04, -.1185576293945E+04, .7057767333984E+03},
 {   22360, .7955600097656E+04, -.1185975219727E+04, .7119037475586E+03},
 {   22361, .7993063476562E+04, -.1177016235352E+04, .7071831665039E+03},
 {   22362, .7956010742188E+04, -.1178306274414E+04, .7133992919922E+03},
 {   22363, .7994131835938E+04, -.1169486328125E+04, .7113712768555E+03},
 {   22364, .7957062988281E+04, -.1169533325195E+04, .7188812255859E+03},
 {   22365, .7964349609375E+04, -.1096335449219E+04, .7550921020508E+03},
 {   22366, .7959574707031E+04, -.1097557495117E+04, .7245714111328E+03},
 {   22367, .7968705566406E+04, -.1107959350586E+04, .7847775878906E+03},
 {   22368, .7971698242188E+04, -.1128832397461E+04, .8070582275391E+03},
 {   22369, .7972933105469E+04, -.1155769287109E+04, .8189700317383E+03},
 {   22370, .7972387207031E+04, -.1184081298828E+04, .8196209106445E+03},
 {   22371, .7971273437500E+04, -.1210854248047E+04, .8163096923828E+03},
 {   22372, .7968674316406E+04, -.1237309204102E+04, .8034373779297E+03},
 {   22373, .7965165527344E+04, -.1257389160156E+04, .7838224487305E+03},
 {   22374, .7960791992188E+04, -.1270185058594E+04, .7575697021484E+03},
 {   22375, .7955628906250E+04, -.1270906127930E+04, .7244522094727E+03},
 {   22376, .7951982910156E+04, -.1261330078125E+04, .6995729980469E+03},
 {   22377, .7948769531250E+04, -.1241542114258E+04, .6760317993164E+03},
 {   22378, .7947795410156E+04, -.1213408203125E+04, .6647707519531E+03},
 {   22379, .7948020019531E+04, -.1186682250977E+04, .6623287353516E+03},
 {   22380, .7949086914062E+04, -.1158828369141E+04, .6651205444336E+03},
 {   22381, .7950877929688E+04, -.1130820434570E+04, .6734286499023E+03},
 {   22382, .7954491210938E+04, -.1109678466797E+04, .6935974731445E+03},
 {   22383, .7927250976562E+04, -.1127060302734E+04, .8161356811523E+03},
 {   22384, .7925009277344E+04, -.1106217407227E+04, .7937528076172E+03},
 {   22385, .7916959960938E+04, -.1163422363281E+04, .7577639160156E+03},
 {   22386, .7917844726562E+04, -.1168928344727E+04, .7643766479492E+03},
 {   22387, .7915901855469E+04, -.1160511352539E+04, .7496884765625E+03},
 {   22388, .7921392578125E+04, -.1094621459961E+04, .7639677124023E+03},
 {   22389, .7914790039062E+04, -.1160609375000E+04, .7411124877930E+03},
 {   22390, .7917160156250E+04, -.1095865478516E+04, .7333716430664E+03},
 {   22391, .7913770019531E+04, -.1163638305664E+04, .7330751342773E+03},
 {   22392, .7912447265625E+04, -.1108001464844E+04, .7023474731445E+03},
 {   22393, .7912964843750E+04, -.1169256347656E+04, .7265483398438E+03},
 {   22394, .7908737304688E+04, -.1132496337891E+04, .6841181640625E+03},
 {   22395, .7912462402344E+04, -.1176783325195E+04, .7223170166016E+03},
 {   22396, .7906930175781E+04, -.1160689331055E+04, .6789802856445E+03},
 {   22397, .7912331054688E+04, -.1185312255859E+04, .7208858642578E+03},
 {   22398, .7905649414062E+04, -.1185000244141E+04, .6765682983398E+03},
 {   22399, .7912591308594E+04, -.1193819213867E+04, .7224235839844E+03},
 {   22400, .7904980468750E+04, -.1208839233398E+04, .6785016479492E+03},
 {   22401, .7913198730469E+04, -.1201282226562E+04, .7267526855469E+03},
 {   22402, .7904394531250E+04, -.1239794189453E+04, .6830981445312E+03},
 {   22403, .7907373535156E+04, -.1259552124023E+04, .7086740112305E+03},
 {   22404, .7914079101562E+04, -.1206796264648E+04, .7333574829102E+03},
 {   22405, .7915149414062E+04, -.1209693237305E+04, .7414363403320E+03},
 {   22406, .7910411132812E+04, -.1269104125977E+04, .7336357421875E+03},
 {   22407, .7914979492188E+04, -.1268360107422E+04, .7668353881836E+03},
 {   22408, .7916254394531E+04, -.1209623168945E+04, .7500145263672E+03},
 {   22409, .7917273925781E+04, -.1206614257812E+04, .7580581665039E+03},
 {   22410, .7919072265625E+04, -.1255553100586E+04, .7931240844727E+03},
 {   22411, .7918076171875E+04, -.1200978271484E+04, .7645720214844E+03},
 {   22412, .7922541503906E+04, -.1235472167969E+04, .8127456665039E+03},
 {   22413, .7918581542969E+04, -.1193464233398E+04, .7688114624023E+03},
 {   22414, .7925333984375E+04, -.1209024169922E+04, .8255910034180E+03},
 {   22415, .7918715332031E+04, -.1184913208008E+04, .7702534790039E+03},
 {   22416, .7926829101562E+04, -.1182265258789E+04, .8288512573242E+03},
 {   22417, .7918452636719E+04, -.1176383300781E+04, .7687050170898E+03},
 {   22418, .7927838378906E+04, -.1153972290039E+04, .8281369018555E+03},
 {   22419, .7870447265625E+04, -.1161570312500E+04, .7514668579102E+03},
 {   22420, .7872512207031E+04, -.1161626342773E+04, .7591165161133E+03},
 {   22421, .7873615234375E+04, -.1142250366211E+04, .7632010498047E+03},
 {   22422, .7869875976562E+04, -.1142209350586E+04, .7493534545898E+03},
 {   22423, .7876000976562E+04, -.1100374511719E+04, .7720289306641E+03},
 {   22424, .7868697753906E+04, -.1102278442383E+04, .7449927978516E+03},
 {   22425, .7860966796875E+04, -.1113276367188E+04, .7163734130859E+03},
 {   22426, .7865724609375E+04, -.1149007324219E+04, .7339832763672E+03},
 {   22427, .7867958984375E+04, -.1165790283203E+04, .7422549438477E+03},
 {   22428, .7866624023438E+04, -.1170160278320E+04, .7373178100586E+03},
 {   22429, .7863117187500E+04, -.1158114257812E+04, .7243322143555E+03},
 {   22430, .7874493652344E+04, -.1165091308594E+04, .7664457397461E+03},
 {   22431, .7877244628906E+04, -.1147424316406E+04, .7766386108398E+03},
 {   22432, .7883541503906E+04, -.1109237426758E+04, .7999480590820E+03},
 {   22433, .7889531250000E+04, -.1128773437500E+04, .8221182861328E+03},
 {   22434, .7880337890625E+04, -.1156462280273E+04, .7880843505859E+03},
 {   22435, .7876084960938E+04, -.1169274291992E+04, .7723374633789E+03},
 {   22436, .7892831542969E+04, -.1155281372070E+04, .8343345336914E+03},
 {   22437, .7882002441406E+04, -.1171080322266E+04, .7942521972656E+03},
 {   22438, .7877386718750E+04, -.1177818237305E+04, .7771577758789E+03},
 {   22439, .7892935546875E+04, -.1183253295898E+04, .8347235107422E+03},
 {   22440, .7882838867188E+04, -.1184727294922E+04, .7973416137695E+03},
 {   22441, .7877790527344E+04, -.1185464233398E+04, .7786502685547E+03},
 {   22442, .7877506835938E+04, -.1194251220703E+04, .7776018676758E+03},
 {   22443, .7882278808594E+04, -.1198884277344E+04, .7952725830078E+03},
 {   22444, .7892192382812E+04, -.1209634155273E+04, .8319710693359E+03},
 {   22445, .7880404296875E+04, -.1212740234375E+04, .7883312988281E+03},
 {   22446, .7888226074219E+04, -.1234962158203E+04, .8172875366211E+03},
 {   22447, .7876667480469E+04, -.1200772216797E+04, .7744981079102E+03},
 {   22448, .7882797363281E+04, -.1252484130859E+04, .7971937866211E+03},
 {   22449, .7877517089844E+04, -.1222662231445E+04, .7776472778320E+03},
 {   22450, .7874645996094E+04, -.1208286254883E+04, .7670171508789E+03},
 {   22451, .7875952148438E+04, -.1262892089844E+04, .7718508300781E+03},
 {   22452, .7873709960938E+04, -.1227935180664E+04, .7635553588867E+03},
 {   22453, .7872639648438E+04, -.1211238159180E+04, .7595935058594E+03},
 {   22454, .7868654785156E+04, -.1263388061523E+04, .7448305053711E+03},
 {   22455, .7870009765625E+04, -.1227775146484E+04, .7498574218750E+03},
 {   22456, .7870629882812E+04, -.1211540283203E+04, .7521494750977E+03},
 {   22457, .7868393066406E+04, -.1209414184570E+04, .7438631591797E+03},
 {   22458, .7866554199219E+04, -.1223831176758E+04, .7370540771484E+03},
 {   22459, .7862637695312E+04, -.1254526123047E+04, .7225568847656E+03},
 {   22460, .7866644531250E+04, -.1203562255859E+04, .7373930664062E+03},
 {   22461, .7863702636719E+04, -.1214151245117E+04, .7265006713867E+03},
 {   22462, .7857146484375E+04, -.1236203125000E+04, .7022305297852E+03},
 {   22463, .7865489746094E+04, -.1196545288086E+04, .7331162719727E+03},
 {   22464, .7861588867188E+04, -.1202139282227E+04, .7186759643555E+03},
 {   22465, .7864987792969E+04, -.1187919311523E+04, .7312590942383E+03},
 {   22466, .7860907714844E+04, -.1188109252930E+04, .7161544799805E+03},
 {   22467, .7865264160156E+04, -.1178578247070E+04, .7322852783203E+03},
 {   22468, .7861017578125E+04, -.1173437255859E+04, .7165607299805E+03},
 {   22469, .7856977050781E+04, -.1137032348633E+04, .7016057739258E+03},
 {   22470, .7857012207031E+04, -.1168590332031E+04, .7017351074219E+03},
 {   22471, .7856738281250E+04, -.1188303222656E+04, .7007195434570E+03},
 {   22472, .7856882324219E+04, -.1208889160156E+04, .7012543945312E+03},
 {   22473, .8048092285156E+04, -.1178736328125E+04, .7360172119141E+03},
 {   22474, .8047363769531E+04, -.1173316284180E+04, .7329129638672E+03},
 {   22475, .8046277343750E+04, -.1169376342773E+04, .7282926025391E+03},
 {   22476, .8044943847656E+04, -.1167256347656E+04, .7226240844727E+03},
 {   22477, .8043523437500E+04, -.1167236328125E+04, .7165825195312E+03},
 {   22478, .8042170410156E+04, -.1169336303711E+04, .7108406982422E+03},
 {   22479, .8041065429688E+04, -.1173346313477E+04, .7061389770508E+03},
 {   22480, .8040337402344E+04, -.1178776245117E+04, .7030478515625E+03},
 {   22481, .8040082031250E+04, -.1184986328125E+04, .7019578247070E+03},
 {   22482, .8040341796875E+04, -.1191337280273E+04, .7030613403320E+03},
 {   22483, .8048353027344E+04, -.1184926269531E+04, .7371206665039E+03},
 {   22484, .8048119628906E+04, -.1191010253906E+04, .7361320800781E+03},
 {   22485, .8047428222656E+04, -.1196421264648E+04, .7331897583008E+03},
 {   22486, .8046358398438E+04, -.1200496215820E+04, .7286313476562E+03},
 {   22487, .8045018554688E+04, -.1202757202148E+04, .7229426269531E+03},
 {   22488, .8043874511719E+04, -.1202884277344E+04, .7180796508789E+03},
 {   22489, .8042219238281E+04, -.1200850219727E+04, .7110448608398E+03},
 {   22490, .8041097656250E+04, -.1196886230469E+04, .7062694702148E+03},
 {   22491, .8070830566406E+04, -.1169346313477E+04, .7038637084961E+03},
 {   22492, .8072181152344E+04, -.1167236328125E+04, .7096053466797E+03},
 {   22493, .8069725097656E+04, -.1173346313477E+04, .6991625366211E+03},
 {   22494, .8073602050781E+04, -.1167256347656E+04, .7156466064453E+03},
 {   22495, .8068997070312E+04, -.1178776245117E+04, .6960710449219E+03},
 {   22496, .8068741210938E+04, -.1184986328125E+04, .6949803466797E+03},
 {   22497, .8069000000000E+04, -.1191337280273E+04, .6960838012695E+03},
 {   22498, .8069965332031E+04, -.1197431274414E+04, .7001832275391E+03},
 {   22499, .8074935058594E+04, -.1169376342773E+04, .7213171997070E+03},
 {   22500, .8076022460938E+04, -.1173316284180E+04, .7259368286133E+03},
 {   22501, .8076752929688E+04, -.1178736328125E+04, .7290416870117E+03},
 {   22502, .8077011718750E+04, -.1184926269531E+04, .7301449584961E+03},
 {   22503, .8075015136719E+04, -.1200497192383E+04, .7216560668945E+03},
 {   22504, .8073678710938E+04, -.1202758178711E+04, .7159671630859E+03},
 {   22505, .8072877929688E+04, -.1202885253906E+04, .7125628662109E+03},
 {   22506, .8076087402344E+04, -.1196422241211E+04, .7262133789062E+03},
 {   22507, .8071539550781E+04, -.1201523193359E+04, .7068753051758E+03},
 {   22508, .8076780761719E+04, -.1191010253906E+04, .7291552734375E+03},
 {   22509, .8094782714844E+04, -.1185036254883E+04, .7074111328125E+03},
 {   22510, .8092737304688E+04, -.1169346313477E+04, .6987116088867E+03},
 {   22511, .8094085937500E+04, -.1167236328125E+04, .7044540405273E+03},
 {   22512, .8091629882812E+04, -.1173346313477E+04, .6940103759766E+03},
 {   22513, .8095507324219E+04, -.1167256347656E+04, .7104957885742E+03},
 {   22514, .8090902832031E+04, -.1178776245117E+04, .6909189453125E+03},
 {   22515, .8090647949219E+04, -.1185036254883E+04, .6898292236328E+03},
 {   22516, .8090906738281E+04, -.1191338256836E+04, .6909327392578E+03},
 {   22517, .8091661621094E+04, -.1196887207031E+04, .6941409301758E+03},
 {   22518, .8096841796875E+04, -.1169376342773E+04, .7161651611328E+03},
 {   22519, .8097927246094E+04, -.1173316284180E+04, .7207849731445E+03},
 {   22520, .8098658203125E+04, -.1178736328125E+04, .7238899536133E+03},
 {   22521, .8098918457031E+04, -.1185036254883E+04, .7249920654297E+03},
 {   22522, .8096920410156E+04, -.1200497192383E+04, .7165043334961E+03},
 {   22523, .8095584472656E+04, -.1202758178711E+04, .7108154296875E+03},
 {   22524, .8094782714844E+04, -.1202886230469E+04, .7074111328125E+03},
 {   22525, .8097992187500E+04, -.1196423217773E+04, .7210614624023E+03},
 {   22526, .8093222656250E+04, -.1201257202148E+04, .7007848510742E+03},
 {   22527, .8098684570312E+04, -.1191011230469E+04, .7240037231445E+03},
 {   22528, .7915525878906E+04, -.1185103271484E+04, .7455625000000E+03},
 {   22529, .7962889160156E+04, -.1148260375977E+04, .7591493530273E+03},
 {   22530, .7960445312500E+04, -.1142407348633E+04, .7435739135742E+03},
 {   22531, .7964520996094E+04, -.1156572265625E+04, .7707067260742E+03},
 {   22532, .7965526855469E+04, -.1170730224609E+04, .7772508544922E+03},
 {   22533, .7965880859375E+04, -.1185281250000E+04, .7803228759766E+03},
 {   22534, .7965221679688E+04, -.1199980224609E+04, .7782767333984E+03},
 {   22535, .7963635253906E+04, -.1213652221680E+04, .7709398803711E+03},
 {   22536, .7961469238281E+04, -.1223189208984E+04, .7597037353516E+03},
 {   22537, .7958590332031E+04, -.1228972167969E+04, .7440861816406E+03},
 {   22538, .7956375000000E+04, -.1228363159180E+04, .7294537353516E+03},
 {   22539, .7954546386719E+04, -.1224118164062E+04, .7161949462891E+03},
 {   22540, .7953244140625E+04, -.1212414184570E+04, .7045060424805E+03},
 {   22541, .7952984375000E+04, -.1201168212891E+04, .6977984008789E+03},
 {   22542, .7958260742188E+04, -.1143029296875E+04, .7287546997070E+03},
 {   22543, .7956091796875E+04, -.1149151367188E+04, .7153629760742E+03},
 {   22544, .7953199218750E+04, -.1186797241211E+04, .6958916625977E+03},
 {   22545, .7953366210938E+04, -.1173048339844E+04, .6966752929688E+03},
 {   22546, .7954530273438E+04, -.1158220336914E+04, .7050059204102E+03},
 {   22547, .7919912597656E+04, -.1146546386719E+04, .7680270385742E+03},
 {   22548, .7921174804688E+04, -.1154843261719E+04, .7796345825195E+03},
 {   22549, .7918002441406E+04, -.1140714355469E+04, .7523784790039E+03},
 {   22550, .7915950195312E+04, -.1141341308594E+04, .7375405273438E+03},
 {   22551, .7913876953125E+04, -.1147467285156E+04, .7241352539062E+03},
 {   22552, .7912110839844E+04, -.1156528320312E+04, .7138073120117E+03},
 {   22553, .7910402832031E+04, -.1171334228516E+04, .7055510253906E+03},
 {   22554, .7909765136719E+04, -.1185065307617E+04, .7048302612305E+03},
 {   22555, .7909466308594E+04, -.1199433227539E+04, .7067498168945E+03},
 {   22556, .7909937988281E+04, -.1210687255859E+04, .7134268798828E+03},
 {   22557, .7911081542969E+04, -.1222385253906E+04, .7251381835938E+03},
 {   22558, .7912730468750E+04, -.1226623168945E+04, .7384215087891E+03},
 {   22559, .7914710937500E+04, -.1227222167969E+04, .7530855102539E+03},
 {   22560, .7917082519531E+04, -.1221419189453E+04, .7687738647461E+03},
 {   22561, .7918996582031E+04, -.1211872192383E+04, .7800437622070E+03},
 {   22562, .7920537597656E+04, -.1198199218750E+04, .7873880004883E+03},
 {   22563, .7921387207031E+04, -.1183507324219E+04, .7894073486328E+03},
 {   22564, .7921528320312E+04, -.1168976318359E+04, .7862666625977E+03},
 {   22565, .7884202148438E+04, -.1235993164062E+04, .8169952392578E+03},
 {   22566, .7858898925781E+04, -.1232996215820E+04, .8207257080078E+03},
 {   22567, .7852810546875E+04, -.1252617065430E+04, .7981912841797E+03},
 {   22568, .7877756347656E+04, -.1256529174805E+04, .7931362915039E+03},
 {   22569, .7876776367188E+04, -.1207520263672E+04, .7895071411133E+03},
 {   22570, .7873463867188E+04, -.1207520263672E+04, .7772467651367E+03},
 {   22571, .7833601074219E+04, -.1230024169922E+04, .8244803466797E+03},
 {   22572, .7827865234375E+04, -.1248706176758E+04, .8032454833984E+03},
 {   22573, .7808299804688E+04, -.1227027221680E+04, .8282107543945E+03},
 {   22574, .7802927734375E+04, -.1244794189453E+04, .8083244628906E+03},
 {   22575, .7803212402344E+04, -.1205521240234E+04, .8093787231445E+03},
 {   22576, .7799901367188E+04, -.1205521240234E+04, .7971181030273E+03},
 {   22577, .7845629882812E+04, -.1258078125000E+04, .7716101074219E+03},
 {   22578, .7870151367188E+04, -.1262320068359E+04, .7649863891602E+03},
 {   22579, .7870151367188E+04, -.1207520263672E+04, .7649862670898E+03},
 {   22580, .7821108886719E+04, -.1253862182617E+04, .7782338867188E+03},
 {   22581, .7796588867188E+04, -.1249620117188E+04, .7848575439453E+03},
 {   22582, .7796588867188E+04, -.1205521240234E+04, .7848575439453E+03},
 {   22583, .7838450195312E+04, -.1252617187500E+04, .7450288696289E+03},
 {   22584, .7862548339844E+04, -.1256529174805E+04, .7368361816406E+03},
 {   22585, .7866840820312E+04, -.1207520263672E+04, .7527256469727E+03},
 {   22586, .7814354003906E+04, -.1248706176758E+04, .7532221679688E+03},
 {   22587, .7790250000000E+04, -.1244794189453E+04, .7613905639648E+03},
 {   22588, .7793276855469E+04, -.1205521240234E+04, .7725970458984E+03},
 {   22589, .7832364257812E+04, -.1232996215820E+04, .7224943847656E+03},
 {   22590, .7856102539062E+04, -.1235993164062E+04, .7129774780273E+03},
 {   22591, .7863528320312E+04, -.1207520263672E+04, .7404650268555E+03},
 {   22592, .7808617187500E+04, -.1230024169922E+04, .7319873046875E+03},
 {   22593, .7784877929688E+04, -.1227027221680E+04, .7415046386719E+03},
 {   22594, .7789963867188E+04, -.1205521240234E+04, .7603364257812E+03},
 {   22595, .7888513671875E+04, -.1211279174805E+04, .8329589843750E+03},
 {   22596, .7862971679688E+04, -.1209654174805E+04, .8358060302734E+03},
 {   22597, .7876776367188E+04, -.1194820312500E+04, .7895071411133E+03},
 {   22598, .7837429199219E+04, -.1208028198242E+04, .8386542968750E+03},
 {   22599, .7811888671875E+04, -.1204429199219E+04, .8415014038086E+03},
 {   22600, .7890023925781E+04, -.1182120239258E+04, .8385494384766E+03},
 {   22601, .7864395996094E+04, -.1182120239258E+04, .8410782470703E+03},
 {   22602, .7876776367188E+04, -.1182120239258E+04, .7895071411133E+03},
 {   22603, .7838775390625E+04, -.1182120239258E+04, .8436314697266E+03},
 {   22604, .7813146972656E+04, -.1182120239258E+04, .8461602172852E+03},
 {   22605, .7888513671875E+04, -.1152961303711E+04, .8329589843750E+03},
 {   22606, .7862971679688E+04, -.1154586303711E+04, .8358060302734E+03},
 {   22607, .7876776367188E+04, -.1169420288086E+04, .7895071411133E+03},
 {   22608, .7837429199219E+04, -.1156212280273E+04, .8386542968750E+03},
 {   22609, .7811888671875E+04, -.1159811279297E+04, .8415014038086E+03},
 {   22610, .7884202148438E+04, -.1128247436523E+04, .8169952392578E+03},
 {   22611, .7858898925781E+04, -.1131244384766E+04, .8207257080078E+03},
 {   22612, .7876776367188E+04, -.1156720336914E+04, .7895071411133E+03},
 {   22613, .7833601074219E+04, -.1134216308594E+04, .8244803466797E+03},
 {   22614, .7808299804688E+04, -.1137213378906E+04, .8282107543945E+03},
 {   22615, .7873463867188E+04, -.1194820312500E+04, .7772467651367E+03},
 {   22616, .7873463867188E+04, -.1182120239258E+04, .7772467651367E+03},
 {   22617, .7873463867188E+04, -.1169420288086E+04, .7772467651367E+03},
 {   22618, .7873463867188E+04, -.1156720336914E+04, .7772467651367E+03},
 {   22619, .7870151367188E+04, -.1194820312500E+04, .7649862670898E+03},
 {   22620, .7870151367188E+04, -.1169420288086E+04, .7649862670898E+03},
 {   22621, .7870151367188E+04, -.1156720336914E+04, .7649862670898E+03},
 {   22622, .7866840820312E+04, -.1194820312500E+04, .7527256469727E+03},
 {   22623, .7866840820312E+04, -.1169420288086E+04, .7527256469727E+03},
 {   22624, .7866840820312E+04, -.1156720336914E+04, .7527256469727E+03},
 {   22625, .7863528320312E+04, -.1194820312500E+04, .7404650268555E+03},
 {   22626, .7866840820312E+04, -.1182120239258E+04, .7527256469727E+03},
 {   22627, .7863528320312E+04, -.1182120239258E+04, .7404650268555E+03},
 {   22628, .7863528320312E+04, -.1169420288086E+04, .7404650268555E+03},
 {   22629, .7863528320312E+04, -.1156720336914E+04, .7404650268555E+03},
 {   22630, .7853876953125E+04, -.1211279174805E+04, .7047368164062E+03},
 {   22631, .7830375488281E+04, -.1209654174805E+04, .7151370239258E+03},
 {   22632, .7806874023438E+04, -.1208028198242E+04, .7255366821289E+03},
 {   22633, .7783374023438E+04, -.1204429199219E+04, .7359370117188E+03},
 {   22634, .7852889648438E+04, -.1182120239258E+04, .7010767822266E+03},
 {   22635, .7829472656250E+04, -.1182120239258E+04, .7117958374023E+03},
 {   22636, .7806051757812E+04, -.1182120239258E+04, .7224898681641E+03},
 {   22637, .7782636718750E+04, -.1182120239258E+04, .7332086791992E+03},
 {   22638, .7853876953125E+04, -.1152961303711E+04, .7047368164062E+03},
 {   22639, .7830375488281E+04, -.1154586303711E+04, .7151370239258E+03},
 {   22640, .7806873046875E+04, -.1156212280273E+04, .7255366210938E+03},
 {   22641, .7783374023438E+04, -.1159811279297E+04, .7359370117188E+03},
 {   22642, .7856102539062E+04, -.1128247436523E+04, .7129774780273E+03},
 {   22643, .7832364257812E+04, -.1131244384766E+04, .7224943847656E+03},
 {   22644, .7808617187500E+04, -.1134216308594E+04, .7319873046875E+03},
 {   22645, .7784877929688E+04, -.1137213378906E+04, .7415046386719E+03},
 {   22646, .7877756347656E+04, -.1106711425781E+04, .7931362915039E+03},
 {   22647, .7852810546875E+04, -.1110623413086E+04, .7981912841797E+03},
 {   22648, .7827865234375E+04, -.1114534423828E+04, .8032454833984E+03},
 {   22649, .7803212402344E+04, -.1158719360352E+04, .8093787231445E+03},
 {   22650, .7802927734375E+04, -.1118446411133E+04, .8083244628906E+03},
 {   22651, .7799901367188E+04, -.1158719360352E+04, .7971181030273E+03},
 {   22652, .7870151367188E+04, -.1100920410156E+04, .7649862670898E+03},
 {   22653, .7845629882812E+04, -.1105162475586E+04, .7716101074219E+03},
 {   22654, .7821108886719E+04, -.1109378417969E+04, .7782337646484E+03},
 {   22655, .7796588867188E+04, -.1113620361328E+04, .7848575439453E+03},
 {   22656, .7796588867188E+04, -.1158719360352E+04, .7848575439453E+03},
 {   22657, .7862548339844E+04, -.1106711425781E+04, .7368361816406E+03},
 {   22658, .7838450195312E+04, -.1110623413086E+04, .7450288696289E+03},
 {   22659, .7814354003906E+04, -.1114534423828E+04, .7532221679688E+03},
 {   22660, .7790250000000E+04, -.1118446411133E+04, .7613905639648E+03},
 {   22661, .7793276855469E+04, -.1158719360352E+04, .7725970458984E+03},
 {   22662, .7789963867188E+04, -.1158719360352E+04, .7603364257812E+03},
 {   22663, .7788627441406E+04, -.1182120239258E+04, .8527843017578E+03},
 {   22664, .7788275390625E+04, -.1193100219727E+04, .8514846191406E+03},
 {   22665, .7788275390625E+04, -.1171140258789E+04, .8514846191406E+03},
 {   22666, .7755858886719E+04, -.1193100219727E+04, .7314779663086E+03},
 {   22667, .7755507324219E+04, -.1182120239258E+04, .7301785888672E+03},
 {   22668, .7755858886719E+04, -.1171140258789E+04, .7314779663086E+03},
 {   22669, .7764092285156E+04, -.1193151245117E+04, .8531228637695E+03},
 {   22670, .7764585937500E+04, -.1205521240234E+04, .8492017822266E+03},
 {   22671, .7787242675781E+04, -.1205521240234E+04, .8476586914062E+03},
 {   22672, .7741105468750E+04, -.1193100219727E+04, .8453619384766E+03},
 {   22673, .7743073730469E+04, -.1205521240234E+04, .8419358520508E+03},
 {   22674, .7763900390625E+04, -.1182120239258E+04, .8544640502930E+03},
 {   22675, .7740424804688E+04, -.1182120239258E+04, .8465450439453E+03},
 {   22676, .7764092285156E+04, -.1171089233398E+04, .8531228637695E+03},
 {   22677, .7741105468750E+04, -.1171140258789E+04, .8453619384766E+03},
 {   22678, .7787242675781E+04, -.1158719360352E+04, .8476586914062E+03},
 {   22679, .7764585937500E+04, -.1158719360352E+04, .8492017822266E+03},
 {   22680, .7743073730469E+04, -.1158719360352E+04, .8419358520508E+03},
 {   22681, .7756891113281E+04, -.1205521240234E+04, .7353039550781E+03},
 {   22682, .7736540527344E+04, -.1205521240234E+04, .7453790283203E+03},
 {   22683, .7734139648438E+04, -.1193151245117E+04, .7422391357422E+03},
 {   22684, .7721612304688E+04, -.1205521240234E+04, .7624876708984E+03},
 {   22685, .7718185546875E+04, -.1193100219727E+04, .7605189208984E+03},
 {   22686, .7733299316406E+04, -.1182120239258E+04, .7411765136719E+03},
 {   22687, .7717003417969E+04, -.1182120239258E+04, .7598388671875E+03},
 {   22688, .7734139648438E+04, -.1171089355469E+04, .7422391357422E+03},
 {   22689, .7718185546875E+04, -.1171140258789E+04, .7605189208984E+03},
 {   22690, .7736540527344E+04, -.1158719360352E+04, .7453790283203E+03},
 {   22691, .7756891113281E+04, -.1158719360352E+04, .7353039550781E+03},
 {   22692, .7721612304688E+04, -.1158719360352E+04, .7624876708984E+03},
 {   22693, .7723002929688E+04, -.1193253295898E+04, .8295725708008E+03},
 {   22694, .7725965332031E+04, -.1205521240234E+04, .8270089111328E+03},
 {   22695, .7721958007812E+04, -.1182120239258E+04, .8304852294922E+03},
 {   22696, .7723002929688E+04, -.1170987304688E+04, .8295725708008E+03},
 {   22697, .7725965332031E+04, -.1158719360352E+04, .8270089111328E+03},
 {   22698, .7712114746094E+04, -.1193329223633E+04, .8076763916016E+03},
 {   22699, .7715889648438E+04, -.1205521240234E+04, .8066561889648E+03},
 {   22700, .7710764648438E+04, -.1182120239258E+04, .8080407104492E+03},
 {   22701, .7712114746094E+04, -.1170911254883E+04, .8076763916016E+03},
 {   22702, .7715889648438E+04, -.1158719360352E+04, .8066561889648E+03},
 {   22703, .7710495605469E+04, -.1193253295898E+04, .7832760620117E+03},
 {   22704, .7714354003906E+04, -.1205521240234E+04, .7840236206055E+03},
 {   22705, .7709140136719E+04, -.1182120239258E+04, .7830368041992E+03},
 {   22706, .7710495605469E+04, -.1170987304688E+04, .7832760620117E+03},
 {   22707, .7714354003906E+04, -.1158719360352E+04, .7840236206055E+03},
 {   22708, .7795470703125E+04, -.1207520263672E+04, .8068327636719E+03},
 {   22709, .7783324218750E+04, -.1226646240234E+04, .8024558715820E+03},
 {   22710, .7776863769531E+04, -.1234317138672E+04, .7854505615234E+03},
 {   22711, .7798421875000E+04, -.1207520263672E+04, .7793253173828E+03},
 {   22712, .7766922363281E+04, -.1236959106445E+04, .7686232299805E+03},
 {   22713, .7788346191406E+04, -.1207520263672E+04, .7535595703125E+03},
 {   22714, .7749694824219E+04, -.1231498168945E+04, .7496875000000E+03},
 {   22715, .7766873535156E+04, -.1207520263672E+04, .7351658325195E+03},
 {   22716, .7728569335938E+04, -.1233123168945E+04, .7179666748047E+03},
 {   22717, .7742781250000E+04, -.1216385253906E+04, .7079733886719E+03},
 {   22718, .7707419433594E+04, -.1234774169922E+04, .6862610473633E+03},
 {   22719, .7717557617188E+04, -.1225224243164E+04, .6791307983398E+03},
 {   22720, .7681504394531E+04, -.1231498168945E+04, .6527219848633E+03},
 {   22721, .7691000488281E+04, -.1222531250000E+04, .6460441894531E+03},
 {   22722, .7655585449219E+04, -.1228196166992E+04, .6192182006836E+03},
 {   22723, .7664455566406E+04, -.1219839233398E+04, .6129791870117E+03},
 {   22724, .7629667480469E+04, -.1224894165039E+04, .5856789550781E+03},
 {   22725, .7637898437500E+04, -.1217147216797E+04, .5798928222656E+03},
 {   22726, .7603732421875E+04, -.1221617187500E+04, .5521536865234E+03},
 {   22727, .7611337890625E+04, -.1214454223633E+04, .5468068237305E+03},
 {   22728, .7577812988281E+04, -.1218315185547E+04, .5186508789062E+03},
 {   22729, .7584793945312E+04, -.1211762207031E+04, .5137413330078E+03},
 {   22730, .7551896972656E+04, -.1215038208008E+04, .4851116027832E+03},
 {   22731, .7558235351562E+04, -.1209069213867E+04, .4806552429199E+03},
 {   22732, .7538327148438E+04, -.1217223144531E+04, .4636044311523E+03},
 {   22733, .7545080078125E+04, -.1210847167969E+04, .4588556823730E+03},
 {   22734, .7524774902344E+04, -.1219407226562E+04, .4420821228027E+03},
 {   22735, .7531943847656E+04, -.1212651245117E+04, .4370417480469E+03},
 {   22736, .7521511718750E+04, -.1212600219727E+04, .4443758239746E+03},
 {   22737, .7521511718750E+04, -.1199900268555E+04, .4443758239746E+03},
 {   22738, .7770504394531E+04, -.1195176269531E+04, .7326089477539E+03},
 {   22739, .7750011230469E+04, -.1200306274414E+04, .7028883056641E+03},
 {   22740, .7771731445312E+04, -.1182120239258E+04, .7317465209961E+03},
 {   22741, .7752523925781E+04, -.1182120239258E+04, .7011203613281E+03},
 {   22742, .7770504394531E+04, -.1169064331055E+04, .7326089477539E+03},
 {   22743, .7750011230469E+04, -.1163934326172E+04, .7028883056641E+03},
 {   22744, .7766873535156E+04, -.1156720336914E+04, .7351658325195E+03},
 {   22745, .7742781250000E+04, -.1147855346680E+04, .7079733886719E+03},
 {   22746, .7728364746094E+04, -.1205437255859E+04, .6715333251953E+03},
 {   22747, .7732164550781E+04, -.1182120239258E+04, .6688596191406E+03},
 {   22748, .7728364746094E+04, -.1158803344727E+04, .6715333251953E+03},
 {   22749, .7717557617188E+04, -.1139016357422E+04, .6791307983398E+03},
 {   22750, .7701137207031E+04, -.1203989257812E+04, .6389140625000E+03},
 {   22751, .7704691894531E+04, -.1182120239258E+04, .6364162597656E+03},
 {   22752, .7701137207031E+04, -.1160251342773E+04, .6389140625000E+03},
 {   22753, .7691000488281E+04, -.1141709350586E+04, .6460441894531E+03},
 {   22754, .7673909667969E+04, -.1202542236328E+04, .6063314208984E+03},
 {   22755, .7677235351562E+04, -.1182120239258E+04, .6039935302734E+03},
 {   22756, .7673909667969E+04, -.1161698364258E+04, .6063314208984E+03},
 {   22757, .7664455566406E+04, -.1144401367188E+04, .6129791870117E+03},
 {   22758, .7646686523438E+04, -.1201068237305E+04, .5737123413086E+03},
 {   22759, .7649761718750E+04, -.1182120239258E+04, .5715502319336E+03},
 {   22760, .7646686523438E+04, -.1163172363281E+04, .5737123413086E+03},
 {   22761, .7637898437500E+04, -.1147093383789E+04, .5798928222656E+03},
 {   22762, .7619440429688E+04, -.1199621215820E+04, .5411082153320E+03},
 {   22763, .7622286132812E+04, -.1182120239258E+04, .5391066894531E+03},
 {   22764, .7619440429688E+04, -.1164619262695E+04, .5411082153320E+03},
 {   22765, .7611337890625E+04, -.1149786376953E+04, .5468068237305E+03},
 {   22766, .7592211914062E+04, -.1198147216797E+04, .5085249633789E+03},
 {   22767, .7594829101562E+04, -.1182120239258E+04, .5066844177246E+03},
 {   22768, .7592211914062E+04, -.1166093261719E+04, .5085249633789E+03},
 {   22769, .7584793945312E+04, -.1152478393555E+04, .5137413330078E+03},
 {   22770, .7564989257812E+04, -.1196700195312E+04, .4759068603516E+03},
 {   22771, .7567356933594E+04, -.1182120239258E+04, .4742409667969E+03},
 {   22772, .7564989257812E+04, -.1167540283203E+04, .4759068603516E+03},
 {   22773, .7558235351562E+04, -.1155171386719E+04, .4806552429199E+03},
 {   22774, .7552286132812E+04, -.1197665283203E+04, .4537853393555E+03},
 {   22775, .7554821289062E+04, -.1182120239258E+04, .4520028381348E+03},
 {   22776, .7552286132812E+04, -.1166575317383E+04, .4537853393555E+03},
 {   22777, .7545080078125E+04, -.1153393310547E+04, .4588556823730E+03},
 {   22778, .7539608886719E+04, -.1198655273438E+04, .4316502685547E+03},
 {   22779, .7521511718750E+04, -.1191010253906E+04, .4443758239746E+03},
 {   22780, .7542291503906E+04, -.1182120239258E+04, .4297641906738E+03},
 {   22781, .7521511718750E+04, -.1182120239258E+04, .4443758239746E+03},
 {   22782, .7539608886719E+04, -.1165585327148E+04, .4316502685547E+03},
 {   22783, .7521511718750E+04, -.1173230346680E+04, .4443758239746E+03},
 {   22784, .7531943847656E+04, -.1151589355469E+04, .4370417480469E+03},
 {   22785, .7521511718750E+04, -.1164340332031E+04, .4443758239746E+03},
 {   22786, .7783324218750E+04, -.1137594360352E+04, .8024558715820E+03},
 {   22787, .7795470703125E+04, -.1156720336914E+04, .8068327636719E+03},
 {   22788, .7798421875000E+04, -.1156720336914E+04, .7793253173828E+03},
 {   22789, .7776863769531E+04, -.1129923339844E+04, .7854505615234E+03},
 {   22790, .7788346191406E+04, -.1156720336914E+04, .7535595703125E+03},
 {   22791, .7766922363281E+04, -.1127281372070E+04, .7686232299805E+03},
 {   22792, .7749694824219E+04, -.1132742309570E+04, .7496873779297E+03},
 {   22793, .7728569335938E+04, -.1131117431641E+04, .7179666748047E+03},
 {   22794, .7707419433594E+04, -.1129466308594E+04, .6862610473633E+03},
 {   22795, .7681504394531E+04, -.1132742309570E+04, .6527219848633E+03},
 {   22796, .7655585449219E+04, -.1136044311523E+04, .6192182006836E+03},
 {   22797, .7629667480469E+04, -.1139346435547E+04, .5856789550781E+03},
 {   22798, .7603732421875E+04, -.1142623291016E+04, .5521536865234E+03},
 {   22799, .7577812988281E+04, -.1145925292969E+04, .5186508789062E+03},
 {   22800, .7551896972656E+04, -.1149202392578E+04, .4851116027832E+03},
 {   22801, .7538327148438E+04, -.1147017333984E+04, .4636044311523E+03},
 {   22802, .7524774902344E+04, -.1144833374023E+04, .4420821228027E+03},
 {   22803, .7521511718750E+04, -.1151640380859E+04, .4443758239746E+03},
 {   22804, .7774478515625E+04, -.1229821166992E+04, .8119076538086E+03},
 {   22805, .7765858886719E+04, -.1239880126953E+04, .7951460571289E+03},
 {   22806, .7764121093750E+04, -.1230913208008E+04, .8202764892578E+03},
 {   22807, .7753195800781E+04, -.1241810180664E+04, .8047354125977E+03},
 {   22808, .7752747070312E+04, -.1229821166992E+04, .8271905517578E+03},
 {   22809, .7739887695312E+04, -.1239880126953E+04, .8134098510742E+03},
 {   22810, .7740858398438E+04, -.1226646240234E+04, .8323209228516E+03},
 {   22811, .7727044433594E+04, -.1234317138672E+04, .8204884033203E+03},
 {   22812, .7754291992188E+04, -.1243410156250E+04, .7775068359375E+03},
 {   22813, .7740743652344E+04, -.1245620117188E+04, .7870330810547E+03},
 {   22814, .7727198242188E+04, -.1243410156250E+04, .7965601196289E+03},
 {   22815, .7714567871094E+04, -.1236959106445E+04, .8054432373047E+03},
 {   22816, .7738331054688E+04, -.1237289184570E+04, .7576799316406E+03},
 {   22817, .7726132812500E+04, -.1239270141602E+04, .7662562866211E+03},
 {   22818, .7713937011719E+04, -.1237289184570E+04, .7748331298828E+03},
 {   22819, .7702572265625E+04, -.1231498168945E+04, .7828253784180E+03},
 {   22820, .7716828613281E+04, -.1239118164062E+04, .7262217407227E+03},
 {   22821, .7704219238281E+04, -.1241175170898E+04, .7350914916992E+03},
 {   22822, .7691604003906E+04, -.1239118164062E+04, .7439599609375E+03},
 {   22823, .7679888183594E+04, -.1233123168945E+04, .7522005615234E+03},
 {   22824, .7695308105469E+04, -.1240972167969E+04, .6947791748047E+03},
 {   22825, .7682299804688E+04, -.1243080200195E+04, .7039258422852E+03},
 {   22826, .7669293457031E+04, -.1240972167969E+04, .7130719604492E+03},
 {   22827, .7657180664062E+04, -.1234774169922E+04, .7215903320312E+03},
 {   22828, .7670139160156E+04, -.1237289184570E+04, .6607133789062E+03},
 {   22829, .7657942871094E+04, -.1239270141602E+04, .6692908935547E+03},
 {   22830, .7645751953125E+04, -.1237289184570E+04, .6778677978516E+03},
 {   22831, .7634384765625E+04, -.1231498168945E+04, .6858595581055E+03},
 {   22832, .7644987304688E+04, -.1233606201172E+04, .6266702270508E+03},
 {   22833, .7633604003906E+04, -.1235460205078E+04, .6346764526367E+03},
 {   22834, .7622219726562E+04, -.1233606201172E+04, .6426837158203E+03},
 {   22835, .7611620605469E+04, -.1228196166992E+04, .6501350708008E+03},
 {   22836, .7619822265625E+04, -.1229923217773E+04, .5926038818359E+03},
 {   22837, .7609246093750E+04, -.1231650146484E+04, .6000416259766E+03},
 {   22838, .7598670410156E+04, -.1229923217773E+04, .6074786376953E+03},
 {   22839, .7588826660156E+04, -.1224894165039E+04, .6144038696289E+03},
 {   22840, .7594654296875E+04, -.1226265136719E+04, .5585391235352E+03},
 {   22841, .7584890625000E+04, -.1227840209961E+04, .5654064941406E+03},
 {   22842, .7575126464844E+04, -.1226265136719E+04, .5722734375000E+03},
 {   22843, .7566044921875E+04, -.1221617187500E+04, .5786585693359E+03},
 {   22844, .7569501464844E+04, -.1222582153320E+04, .5244956054688E+03},
 {   22845, .7560548339844E+04, -.1224030151367E+04, .5307926025391E+03},
 {   22846, .7551594238281E+04, -.1222582153320E+04, .5370900268555E+03},
 {   22847, .7543280273438E+04, -.1218315185547E+04, .5429334716797E+03},
 {   22848, .7544313476562E+04, -.1218899169922E+04, .4904447937012E+03},
 {   22849, .7536190917969E+04, -.1220220214844E+04, .4961571350098E+03},
 {   22850, .7528066406250E+04, -.1218899169922E+04, .5018705444336E+03},
 {   22851, .7520484375000E+04, -.1215038208008E+04, .5072029418945E+03},
 {   22852, .7530264648438E+04, -.1221363159180E+04, .4692730102539E+03},
 {   22853, .7521579589844E+04, -.1222760131836E+04, .4753799743652E+03},
 {   22854, .7512895996094E+04, -.1221363159180E+04, .4814876098633E+03},
 {   22855, .7504833984375E+04, -.1217223144531E+04, .4871563110352E+03},
 {   22856, .7516195312500E+04, -.1223801147461E+04, .4481160278320E+03},
 {   22857, .7514241699219E+04, -.1212600219727E+04, .4494902038574E+03},
 {   22858, .7506970214844E+04, -.1225300170898E+04, .4546040344238E+03},
 {   22859, .7506970214844E+04, -.1212600219727E+04, .4546040344238E+03},
 {   22860, .7497742675781E+04, -.1223801147461E+04, .4610913696289E+03},
 {   22861, .7499697753906E+04, -.1212600219727E+04, .4597177734375E+03},
 {   22862, .7489162597656E+04, -.1219407226562E+04, .4671251831055E+03},
 {   22863, .7492425781250E+04, -.1212600219727E+04, .4648314819336E+03},
 {   22864, .7786986328125E+04, -.1207520263672E+04, .8223636474609E+03},
 {   22865, .7774218261719E+04, -.1207520263672E+04, .8346323242188E+03},
 {   22866, .7758355957031E+04, -.1207520263672E+04, .8424979858398E+03},
 {   22867, .7740867187500E+04, -.1207520263672E+04, .8452305908203E+03},
 {   22868, .7514241699219E+04, -.1199900268555E+04, .4494902038574E+03},
 {   22869, .7506970214844E+04, -.1199900268555E+04, .4546040344238E+03},
 {   22870, .7499697753906E+04, -.1199900268555E+04, .4597177734375E+03},
 {   22871, .7492425781250E+04, -.1199900268555E+04, .4648314819336E+03},
 {   22872, .7514241699219E+04, -.1191010253906E+04, .4494902038574E+03},
 {   22873, .7506970214844E+04, -.1191010253906E+04, .4546040344238E+03},
 {   22874, .7499697753906E+04, -.1191010253906E+04, .4597177734375E+03},
 {   22875, .7492425781250E+04, -.1191010253906E+04, .4648314819336E+03},
 {   22876, .7514241699219E+04, -.1182120239258E+04, .4494902038574E+03},
 {   22877, .7506970214844E+04, -.1182120239258E+04, .4546040344238E+03},
 {   22878, .7499697753906E+04, -.1182120239258E+04, .4597177734375E+03},
 {   22879, .7492425781250E+04, -.1182120239258E+04, .4648314819336E+03},
 {   22880, .7514241699219E+04, -.1173230346680E+04, .4494902038574E+03},
 {   22881, .7506970214844E+04, -.1173230346680E+04, .4546040344238E+03},
 {   22882, .7499697753906E+04, -.1173230346680E+04, .4597177734375E+03},
 {   22883, .7492425781250E+04, -.1173230346680E+04, .4648314819336E+03},
 {   22884, .7514241699219E+04, -.1164340332031E+04, .4494902038574E+03},
 {   22885, .7506970214844E+04, -.1164340332031E+04, .4546040344238E+03},
 {   22886, .7499697753906E+04, -.1164340332031E+04, .4597177734375E+03},
 {   22887, .7492425781250E+04, -.1164340332031E+04, .4648314819336E+03},
 {   22888, .7774478515625E+04, -.1134419433594E+04, .8119076538086E+03},
 {   22889, .7786986328125E+04, -.1156720336914E+04, .8223636474609E+03},
 {   22890, .7764121093750E+04, -.1133327392578E+04, .8202764892578E+03},
 {   22891, .7774218261719E+04, -.1156720336914E+04, .8346323242188E+03},
 {   22892, .7752747070312E+04, -.1134419433594E+04, .8271905517578E+03},
 {   22893, .7758355957031E+04, -.1156720336914E+04, .8424979858398E+03},
 {   22894, .7740858398438E+04, -.1137594360352E+04, .8323209228516E+03},
 {   22895, .7740867187500E+04, -.1156720336914E+04, .8452305908203E+03},
 {   22896, .7514241699219E+04, -.1151640380859E+04, .4494902038574E+03},
 {   22897, .7506970214844E+04, -.1151640380859E+04, .4546040344238E+03},
 {   22898, .7499697753906E+04, -.1151640380859E+04, .4597177734375E+03},
 {   22899, .7492425781250E+04, -.1151640380859E+04, .4648314819336E+03},
 {   22900, .7765858886719E+04, -.1124360351562E+04, .7951460571289E+03},
 {   22901, .7753195800781E+04, -.1122430419922E+04, .8047354125977E+03},
 {   22902, .7739887695312E+04, -.1124360351562E+04, .8134098510742E+03},
 {   22903, .7727044433594E+04, -.1129923339844E+04, .8204884033203E+03},
 {   22904, .7754291992188E+04, -.1120830444336E+04, .7775068359375E+03},
 {   22905, .7740743652344E+04, -.1118620361328E+04, .7870330810547E+03},
 {   22906, .7727198242188E+04, -.1120830444336E+04, .7965601196289E+03},
 {   22907, .7714567871094E+04, -.1127281372070E+04, .8054432373047E+03},
 {   22908, .7738331054688E+04, -.1126951416016E+04, .7576799316406E+03},
 {   22909, .7726132812500E+04, -.1124970336914E+04, .7662561645508E+03},
 {   22910, .7713937011719E+04, -.1126951416016E+04, .7748330078125E+03},
 {   22911, .7702572265625E+04, -.1132742309570E+04, .7828253784180E+03},
 {   22912, .7716828613281E+04, -.1125122314453E+04, .7262217407227E+03},
 {   22913, .7704219238281E+04, -.1123065429688E+04, .7350914916992E+03},
 {   22914, .7691604003906E+04, -.1125122314453E+04, .7439599609375E+03},
 {   22915, .7679888183594E+04, -.1131117431641E+04, .7522005615234E+03},
 {   22916, .7695308105469E+04, -.1123268432617E+04, .6947791748047E+03},
 {   22917, .7682299804688E+04, -.1121160400391E+04, .7039258422852E+03},
 {   22918, .7669293457031E+04, -.1123268432617E+04, .7130719604492E+03},
 {   22919, .7657180664062E+04, -.1129466308594E+04, .7215903320312E+03},
 {   22920, .7670139160156E+04, -.1126951416016E+04, .6607133789062E+03},
 {   22921, .7657942871094E+04, -.1124970336914E+04, .6692908935547E+03},
 {   22922, .7645751953125E+04, -.1126951416016E+04, .6778677978516E+03},
 {   22923, .7634384765625E+04, -.1132742309570E+04, .6858595581055E+03},
 {   22924, .7644987304688E+04, -.1130634399414E+04, .6266702270508E+03},
 {   22925, .7633604003906E+04, -.1128780395508E+04, .6346764526367E+03},
 {   22926, .7622219726562E+04, -.1130634399414E+04, .6426837158203E+03},
 {   22927, .7611620605469E+04, -.1136044311523E+04, .6501350708008E+03},
 {   22928, .7619822265625E+04, -.1134317382812E+04, .5926038818359E+03},
 {   22929, .7609246093750E+04, -.1132590332031E+04, .6000416259766E+03},
 {   22930, .7598670410156E+04, -.1134317382812E+04, .6074786376953E+03},
 {   22931, .7588826660156E+04, -.1139346435547E+04, .6144038696289E+03},
 {   22932, .7594654296875E+04, -.1137975341797E+04, .5585391235352E+03},
 {   22933, .7584890625000E+04, -.1136400390625E+04, .5654063720703E+03},
 {   22934, .7575126464844E+04, -.1137975341797E+04, .5722734375000E+03},
 {   22935, .7566044921875E+04, -.1142623291016E+04, .5786584472656E+03},
 {   22936, .7569501464844E+04, -.1141658325195E+04, .5244956054688E+03},
 {   22937, .7560548339844E+04, -.1140210327148E+04, .5307926025391E+03},
 {   22938, .7551594238281E+04, -.1141658325195E+04, .5370900268555E+03},
 {   22939, .7543280273438E+04, -.1145925292969E+04, .5429334716797E+03},
 {   22940, .7544313476562E+04, -.1145341308594E+04, .4904447937012E+03},
 {   22941, .7536190917969E+04, -.1144020385742E+04, .4961571350098E+03},
 {   22942, .7528066406250E+04, -.1145341308594E+04, .5018705444336E+03},
 {   22943, .7520484375000E+04, -.1149202392578E+04, .5072029418945E+03},
 {   22944, .7530264648438E+04, -.1142877319336E+04, .4692730102539E+03},
 {   22945, .7521579589844E+04, -.1141480346680E+04, .4753799743652E+03},
 {   22946, .7512895996094E+04, -.1142877319336E+04, .4814876098633E+03},
 {   22947, .7504833984375E+04, -.1147017333984E+04, .4871563110352E+03},
 {   22948, .7516195312500E+04, -.1140439331055E+04, .4481160278320E+03},
 {   22949, .7506970214844E+04, -.1138940307617E+04, .4546040344238E+03},
 {   22950, .7497742675781E+04, -.1140439331055E+04, .4610913696289E+03},
 {   22951, .7489162597656E+04, -.1144833374023E+04, .4671251831055E+03},
 {   22952, .7713985351562E+04, -.1207520263672E+04, .8387052001953E+03},
 {   22953, .7693145507812E+04, -.1207520263672E+04, .8205072631836E+03},
 {   22954, .7683099609375E+04, -.1207520263672E+04, .7940769042969E+03},
 {   22955, .7665656738281E+04, -.1216385253906E+04, .7622090454102E+03},
 {   22956, .7647041992188E+04, -.1225224121094E+04, .7287211303711E+03},
 {   22957, .7624886718750E+04, -.1222531250000E+04, .6925364379883E+03},
 {   22958, .7602747558594E+04, -.1219839233398E+04, .6563741455078E+03},
 {   22959, .7580593750000E+04, -.1217147216797E+04, .6201900634766E+03},
 {   22960, .7558441406250E+04, -.1214454223633E+04, .5840056762695E+03},
 {   22961, .7536299804688E+04, -.1211762207031E+04, .5478440551758E+03},
 {   22962, .7514145996094E+04, -.1209069213867E+04, .5116593933105E+03},
 {   22963, .7498081542969E+04, -.1210847167969E+04, .4919050292969E+03},
 {   22964, .7481993652344E+04, -.1212651245117E+04, .4721661071777E+03},
 {   22965, .7679462402344E+04, -.1195176269531E+04, .7966337890625E+03},
 {   22966, .7658424804688E+04, -.1200306274414E+04, .7672943115234E+03},
 {   22967, .7678238769531E+04, -.1182120239258E+04, .7974963378906E+03},
 {   22968, .7655335449219E+04, -.1182120239258E+04, .7682442016602E+03},
 {   22969, .7679462402344E+04, -.1169064208984E+04, .7966337890625E+03},
 {   22970, .7658424804688E+04, -.1163934326172E+04, .7672943115234E+03},
 {   22971, .7683099609375E+04, -.1156720336914E+04, .7940769042969E+03},
 {   22972, .7665656738281E+04, -.1147855346680E+04, .7622090454102E+03},
 {   22973, .7636239746094E+04, -.1205437255859E+04, .7363193969727E+03},
 {   22974, .7632437988281E+04, -.1182120239258E+04, .7389926147461E+03},
 {   22975, .7636239746094E+04, -.1158803344727E+04, .7363193969727E+03},
 {   22976, .7647041992188E+04, -.1139016357422E+04, .7287211303711E+03},
 {   22977, .7614750000000E+04, -.1203989257812E+04, .6996672363281E+03},
 {   22978, .7611196777344E+04, -.1182120239258E+04, .7021655883789E+03},
 {   22979, .7614750000000E+04, -.1160251342773E+04, .6996672363281E+03},
 {   22980, .7624885742188E+04, -.1141709350586E+04, .6925361328125E+03},
 {   22981, .7593297363281E+04, -.1202542236328E+04, .6630219116211E+03},
 {   22982, .7589971679688E+04, -.1182120239258E+04, .6653596801758E+03},
 {   22983, .7593297363281E+04, -.1161698242188E+04, .6630219116211E+03},
 {   22984, .7602747558594E+04, -.1144401367188E+04, .6563741455078E+03},
 {   22985, .7571806152344E+04, -.1201068237305E+04, .6263706665039E+03},
 {   22986, .7568731933594E+04, -.1182120239258E+04, .6285328369141E+03},
 {   22987, .7571806152344E+04, -.1163172241211E+04, .6263706665039E+03},
 {   22988, .7580593750000E+04, -.1147093261719E+04, .6201900634766E+03},
 {   22989, .7550339355469E+04, -.1199621215820E+04, .5897043457031E+03},
 {   22990, .7547490722656E+04, -.1182120239258E+04, .5917060546875E+03},
 {   22991, .7550339355469E+04, -.1164619262695E+04, .5897043457031E+03},
 {   22992, .7558441406250E+04, -.1149786376953E+04, .5840056762695E+03},
 {   22993, .7528884277344E+04, -.1198147216797E+04, .5530601196289E+03},
 {   22994, .7526264160156E+04, -.1182120239258E+04, .5549006958008E+03},
 {   22995, .7528884277344E+04, -.1166093261719E+04, .5530601196289E+03},
 {   22996, .7536299804688E+04, -.1152478393555E+04, .5478440551758E+03},
 {   22997, .7507394531250E+04, -.1196700195312E+04, .5164081420898E+03},
 {   22998, .7505024414062E+04, -.1182120239258E+04, .5180739135742E+03},
 {   22999, .7507394531250E+04, -.1167540283203E+04, .5164081420898E+03},
 {   23000, .7514145996094E+04, -.1155171386719E+04, .5116593933105E+03},
 {   23001, .7490872070312E+04, -.1197665283203E+04, .4969755859375E+03},
 {   23002, .7488335449219E+04, -.1182120239258E+04, .4987580261230E+03},
 {   23003, .7490872070312E+04, -.1166575317383E+04, .4969755859375E+03},
 {   23004, .7498081542969E+04, -.1153393310547E+04, .4919050292969E+03},
 {   23005, .7474327636719E+04, -.1198655273438E+04, .4775570373535E+03},
 {   23006, .7471646484375E+04, -.1182120239258E+04, .4794427490234E+03},
 {   23007, .7474327636719E+04, -.1165585327148E+04, .4775570373535E+03},
 {   23008, .7481993652344E+04, -.1151589355469E+04, .4721661071777E+03},
 {   23009, .7713985351562E+04, -.1156720336914E+04, .8387052001953E+03},
 {   23010, .7693145507812E+04, -.1156720336914E+04, .8205072631836E+03},
 {   23011, .7769898925781E+04, -.1207520263672E+04, .7975821533203E+03},
 {   23012, .7762593261719E+04, -.1207520263672E+04, .7871940917969E+03},
 {   23013, .7755287597656E+04, -.1207520263672E+04, .7768053588867E+03},
 {   23014, .7739530273438E+04, -.1207520263672E+04, .7543934936523E+03},
 {   23015, .7769898925781E+04, -.1156720336914E+04, .7975821533203E+03},
 {   23016, .7762593261719E+04, -.1156720336914E+04, .7871940917969E+03},
 {   23017, .7755287597656E+04, -.1156720336914E+04, .7768053588867E+03},
 {   23018, .7739530273438E+04, -.1156720336914E+04, .7543934936523E+03},
 {   23019, .7762628417969E+04, -.1207520263672E+04, .8026958007812E+03},
 {   23020, .7755354980469E+04, -.1207520263672E+04, .8078106079102E+03},
 {   23021, .7748083007812E+04, -.1207520263672E+04, .8129242553711E+03},
 {   23022, .7740810546875E+04, -.1207520263672E+04, .8180383300781E+03},
 {   23023, .7755321777344E+04, -.1207520263672E+04, .7923080444336E+03},
 {   23024, .7748051757812E+04, -.1207520263672E+04, .7974216918945E+03},
 {   23025, .7740779296875E+04, -.1207520263672E+04, .8025353393555E+03},
 {   23026, .7733509277344E+04, -.1207520263672E+04, .8076494750977E+03},
 {   23027, .7748017578125E+04, -.1207520263672E+04, .7819193115234E+03},
 {   23028, .7740743652344E+04, -.1207520263672E+04, .7870330810547E+03},
 {   23029, .7733473632812E+04, -.1207520263672E+04, .7921475219727E+03},
 {   23030, .7726203613281E+04, -.1207520263672E+04, .7972613525391E+03},
 {   23031, .7732256347656E+04, -.1207520263672E+04, .7595076904297E+03},
 {   23032, .7724982421875E+04, -.1207520263672E+04, .7646213378906E+03},
 {   23033, .7717713378906E+04, -.1207520263672E+04, .7697358398438E+03},
 {   23034, .7710441894531E+04, -.1207520263672E+04, .7748488769531E+03},
 {   23035, .7762628417969E+04, -.1156720336914E+04, .8026958007812E+03},
 {   23036, .7755354980469E+04, -.1156720336914E+04, .8078106079102E+03},
 {   23037, .7748083007812E+04, -.1156720336914E+04, .8129242553711E+03},
 {   23038, .7740810546875E+04, -.1156720336914E+04, .8180383300781E+03},
 {   23039, .7755321777344E+04, -.1156720336914E+04, .7923080444336E+03},
 {   23040, .7748051757812E+04, -.1156720336914E+04, .7974216918945E+03},
 {   23041, .7740779296875E+04, -.1156720336914E+04, .8025353393555E+03},
 {   23042, .7733509277344E+04, -.1156720336914E+04, .8076494750977E+03},
 {   23043, .7748017578125E+04, -.1156720336914E+04, .7819193115234E+03},
 {   23044, .7740743652344E+04, -.1156720336914E+04, .7870330810547E+03},
 {   23045, .7733473632812E+04, -.1156720336914E+04, .7921475219727E+03},
 {   23046, .7726203613281E+04, -.1156720336914E+04, .7972613525391E+03},
 {   23047, .7732256347656E+04, -.1156720336914E+04, .7595076904297E+03},
 {   23048, .7724982421875E+04, -.1156720336914E+04, .7646213378906E+03},
 {   23049, .7717713378906E+04, -.1156720336914E+04, .7697358398438E+03},
 {   23050, .7710441894531E+04, -.1156720336914E+04, .7748488769531E+03},
 {   23051, .7542116699219E+04, -.1197792236328E+04, .4182130737305E+03},
 {   23052, .7533842773438E+04, -.1211330200195E+04, .4199612731934E+03},
 {   23053, .7531087402344E+04, -.1212321166992E+04, .4143229370117E+03},
 {   23054, .7539392578125E+04, -.1198528198242E+04, .4106275634766E+03},
 {   23055, .7545010742188E+04, -.1182120239258E+04, .4176054687500E+03},
 {   23056, .7542336425781E+04, -.1182120239258E+04, .4092994079590E+03},
 {   23057, .7542116699219E+04, -.1166448242188E+04, .4182130737305E+03},
 {   23058, .7539392578125E+04, -.1165712280273E+04, .4106275634766E+03},
 {   23059, .7533842773438E+04, -.1152910400391E+04, .4199612731934E+03},
 {   23060, .7531087402344E+04, -.1151919311523E+04, .4143229370117E+03},
 {   23061, .7527720703125E+04, -.1212651245117E+04, .4089589843750E+03},
 {   23062, .7535386718750E+04, -.1198655273438E+04, .4035679931641E+03},
 {   23063, .7538066406250E+04, -.1182120239258E+04, .4016825561523E+03},
 {   23064, .7535386718750E+04, -.1165585327148E+04, .4035679931641E+03},
 {   23065, .7527720703125E+04, -.1151589355469E+04, .4089589843750E+03},
 {   23066, .7523810546875E+04, -.1212321166992E+04, .4039750976562E+03},
 {   23067, .7530099609375E+04, -.1198528198242E+04, .3974139709473E+03},
 {   23068, .7532341308594E+04, -.1182120239258E+04, .3950888977051E+03},
 {   23069, .7530099609375E+04, -.1165712280273E+04, .3974139709473E+03},
 {   23070, .7523810546875E+04, -.1151919311523E+04, .4039750976562E+03},
 {   23071, .7519436035156E+04, -.1211330200195E+04, .3994762268066E+03},
 {   23072, .7523879394531E+04, -.1197792236328E+04, .3922838134766E+03},
 {   23073, .7525430664062E+04, -.1182120239258E+04, .3897638244629E+03},
 {   23074, .7523879394531E+04, -.1166448242188E+04, .3922838134766E+03},
 {   23075, .7519436035156E+04, -.1152910400391E+04, .3994762268066E+03},
 {   23076, .7524683105469E+04, -.1219102172852E+04, .4304718017578E+03},
 {   23077, .7520732421875E+04, -.1221287231445E+04, .4237483825684E+03},
 {   23078, .7512539062500E+04, -.1221820190430E+04, .4404415893555E+03},
 {   23079, .7507743652344E+04, -.1224411132812E+04, .4336277770996E+03},
 {   23080, .7516272460938E+04, -.1222023193359E+04, .4170096435547E+03},
 {   23081, .7502745117188E+04, -.1225300170898E+04, .4265207214355E+03},
 {   23082, .7511438476562E+04, -.1221287231445E+04, .4105342407227E+03},
 {   23083, .7497749023438E+04, -.1224411132812E+04, .4194157714844E+03},
 {   23084, .7506448242188E+04, -.1219102172852E+04, .4045416259766E+03},
 {   23085, .7492957519531E+04, -.1221820190430E+04, .4126004638672E+03},
 {   23086, .7537317871094E+04, -.1205107177734E+04, .4384150085449E+03},
 {   23087, .7523478027344E+04, -.1205894287109E+04, .4560035705566E+03},
 {   23088, .7543053222656E+04, -.1182120239258E+04, .4420218505859E+03},
 {   23089, .7527586425781E+04, -.1182120239258E+04, .4618419494629E+03},
 {   23090, .7537318847656E+04, -.1159133300781E+04, .4384149475098E+03},
 {   23091, .7523478027344E+04, -.1158346313477E+04, .4560035705566E+03},
 {   23092, .7524683105469E+04, -.1145138305664E+04, .4304718017578E+03},
 {   23093, .7512539062500E+04, -.1142420410156E+04, .4404415893555E+03},
 {   23094, .7503243164062E+04, -.1205107177734E+04, .3899636840820E+03},
 {   23095, .7482013183594E+04, -.1205894287109E+04, .3970387573242E+03},
 {   23096, .7501792968750E+04, -.1182120239258E+04, .3833477172852E+03},
 {   23097, .7477907226562E+04, -.1182120239258E+04, .3912012023926E+03},
 {   23098, .7503243164062E+04, -.1159133300781E+04, .3899636840820E+03},
 {   23099, .7482013183594E+04, -.1158346313477E+04, .3970386962891E+03},
 {   23100, .7506448242188E+04, -.1145138305664E+04, .4045416259766E+03},
 {   23101, .7492957519531E+04, -.1142420410156E+04, .4126004638672E+03},
 {   23102, .7520732421875E+04, -.1142953369141E+04, .4237483825684E+03},
 {   23103, .7507743652344E+04, -.1139829345703E+04, .4336277770996E+03},
 {   23104, .7516272460938E+04, -.1142217407227E+04, .4170096435547E+03},
 {   23105, .7502745117188E+04, -.1138940307617E+04, .4265207214355E+03},
 {   23106, .7511438476562E+04, -.1142953369141E+04, .4105342407227E+03},
 {   23107, .7497749023438E+04, -.1139829345703E+04, .4194157714844E+03},
 {   23108, .7477614257812E+04, -.1216816162109E+04, .4570184936523E+03},
 {   23109, .7474690917969E+04, -.1217451171875E+04, .4528633422852E+03},
 {   23110, .7471742187500E+04, -.1217680175781E+04, .4486667785645E+03},
 {   23111, .7467562011719E+04, -.1216994140625E+04, .4427245788574E+03},
 {   23112, .7463559570312E+04, -.1214810180664E+04, .4370317077637E+03},
 {   23113, .7492022460938E+04, -.1206174194336E+04, .4775049438477E+03},
 {   23114, .7499340820312E+04, -.1182120239258E+04, .4879143066406E+03},
 {   23115, .7492022460938E+04, -.1158066284180E+04, .4775049438477E+03},
 {   23116, .7477614257812E+04, -.1147424316406E+04, .4570184936523E+03},
 {   23117, .7454516113281E+04, -.1201729248047E+04, .4241713256836E+03},
 {   23118, .7451126953125E+04, -.1182120239258E+04, .4193507995605E+03},
 {   23119, .7454516113281E+04, -.1162511352539E+04, .4241713256836E+03},
 {   23120, .7463559570312E+04, -.1149430297852E+04, .4370317077637E+03},
 {   23121, .7474690917969E+04, -.1146789306641E+04, .4528633422852E+03},
 {   23122, .7471742187500E+04, -.1146560302734E+04, .4486667785645E+03},
 {   23123, .7467562011719E+04, -.1147246337891E+04, .4427245788574E+03},
 {   23124, .7451879394531E+04, -.1212143188477E+04, .4792795715332E+03},
 {   23125, .7448721679688E+04, -.1218137207031E+04, .4747912902832E+03},
 {   23126, .7426105468750E+04, -.1207469238281E+04, .5015337219238E+03},
 {   23127, .7422742187500E+04, -.1218848144531E+04, .4967550964355E+03},
 {   23128, .7400351074219E+04, -.1202770263672E+04, .5237735595703E+03},
 {   23129, .7396769531250E+04, -.1219534179688E+04, .5186824951172E+03},
 {   23130, .7444031250000E+04, -.1221896240234E+04, .4681214294434E+03},
 {   23131, .7416301757812E+04, -.1226138183594E+04, .4875932006836E+03},
 {   23132, .7388605957031E+04, -.1230380126953E+04, .5070692749023E+03},
 {   23133, .7438421875000E+04, -.1220398193359E+04, .4601436157227E+03},
 {   23134, .7409273437500E+04, -.1223801147461E+04, .4775991516113E+03},
 {   23135, .7380130859375E+04, -.1227205200195E+04, .4950188293457E+03},
 {   23136, .7433409179688E+04, -.1216029174805E+04, .4530173034668E+03},
 {   23137, .7403238769531E+04, -.1217223144531E+04, .4690183715820E+03},
 {   23138, .7373087890625E+04, -.1218442138672E+04, .4850047302246E+03},
 {   23139, .7461798828125E+04, -.1201627197266E+04, .4933864746094E+03},
 {   23140, .7431539550781E+04, -.1197055297852E+04, .5092625122070E+03},
 {   23141, .7401315917969E+04, -.1192509277344E+04, .5251455078125E+03},
 {   23142, .7466781250000E+04, -.1182120239258E+04, .5004715270996E+03},
 {   23143, .7434185058594E+04, -.1182120239258E+04, .5130235595703E+03},
 {   23144, .7401624023438E+04, -.1182120239258E+04, .5255809326172E+03},
 {   23145, .7461798828125E+04, -.1162613281250E+04, .4933864746094E+03},
 {   23146, .7431539550781E+04, -.1167185302734E+04, .5092625122070E+03},
 {   23147, .7401315917969E+04, -.1171731323242E+04, .5251455078125E+03},
 {   23148, .7451879394531E+04, -.1152097290039E+04, .4792795715332E+03},
 {   23149, .7426105468750E+04, -.1156771362305E+04, .5015337219238E+03},
 {   23150, .7400351074219E+04, -.1161470336914E+04, .5237735595703E+03},
 {   23151, .7425374511719E+04, -.1201678222656E+04, .4415902709961E+03},
 {   23152, .7396224609375E+04, -.1201653198242E+04, .4590453186035E+03},
 {   23153, .7367082519531E+04, -.1201602172852E+04, .4764639892578E+03},
 {   23154, .7422451660156E+04, -.1182120239258E+04, .4374344482422E+03},
 {   23155, .7393770019531E+04, -.1182120239258E+04, .4555546264648E+03},
 {   23156, .7365095214844E+04, -.1182120239258E+04, .4736387939453E+03},
 {   23157, .7425374511719E+04, -.1162562255859E+04, .4415902709961E+03},
 {   23158, .7396224609375E+04, -.1162587280273E+04, .4590453186035E+03},
 {   23159, .7367082519531E+04, -.1162638305664E+04, .4764639892578E+03},
 {   23160, .7433409179688E+04, -.1148211303711E+04, .4530173034668E+03},
 {   23161, .7403238769531E+04, -.1147017333984E+04, .4690183715820E+03},
 {   23162, .7373087890625E+04, -.1145798339844E+04, .4850047302246E+03},
 {   23163, .7448721679688E+04, -.1146103393555E+04, .4747912902832E+03},
 {   23164, .7422742187500E+04, -.1145417358398E+04, .4967550964355E+03},
 {   23165, .7396769531250E+04, -.1144706298828E+04, .5186824951172E+03},
 {   23166, .7444031250000E+04, -.1142344360352E+04, .4681214294434E+03},
 {   23167, .7416317382812E+04, -.1138102294922E+04, .4876139221191E+03},
 {   23168, .7388619140625E+04, -.1133860351562E+04, .5070896606445E+03},
 {   23169, .7438421875000E+04, -.1143842285156E+04, .4601436157227E+03},
 {   23170, .7409273437500E+04, -.1140439331055E+04, .4775991516113E+03},
 {   23171, .7380145019531E+04, -.1137035400391E+04, .4950389404297E+03},
 {   23172, .7361441406250E+04, -.1208892211914E+04, .5346779785156E+03},
 {   23173, .7356506835938E+04, -.1216969238281E+04, .5276556396484E+03},
 {   23174, .7350003417969E+04, -.1220220214844E+04, .5184105834961E+03},
 {   23175, .7343500488281E+04, -.1216969238281E+04, .5091647338867E+03},
 {   23176, .7338563476562E+04, -.1208892211914E+04, .5021419982910E+03},
 {   23177, .7364951660156E+04, -.1196141235352E+04, .5396646118164E+03},
 {   23178, .7366077148438E+04, -.1182120239258E+04, .5412648315430E+03},
 {   23179, .7364951660156E+04, -.1168099243164E+04, .5396646118164E+03},
 {   23180, .7361441406250E+04, -.1155348388672E+04, .5346779785156E+03},
 {   23181, .7335055664062E+04, -.1196141235352E+04, .4971557006836E+03},
 {   23182, .7333932128906E+04, -.1182120239258E+04, .4955554199219E+03},
 {   23183, .7335055664062E+04, -.1168099243164E+04, .4971557006836E+03},
 {   23184, .7338563476562E+04, -.1155348388672E+04, .5021419982910E+03},
 {   23185, .7356506835938E+04, -.1147271362305E+04, .5276556396484E+03},
 {   23186, .7350003417969E+04, -.1144020263672E+04, .5184105834961E+03},
 {   23187, .7343500488281E+04, -.1147271362305E+04, .5091647338867E+03},
 {   23188, .7351213378906E+04, -.1207825195312E+04, .5389852294922E+03},
 {   23189, .7345333984375E+04, -.1213464233398E+04, .5335855712891E+03},
 {   23190, .7340727539062E+04, -.1204040283203E+04, .5393093872070E+03},
 {   23191, .7335859863281E+04, -.1204955200195E+04, .5357449340820E+03},
 {   23192, .7339241210938E+04, -.1215724243164E+04, .5259782104492E+03},
 {   23193, .7331801757812E+04, -.1205183227539E+04, .5312097778320E+03},
 {   23194, .7334143066406E+04, -.1213464233398E+04, .5176706542969E+03},
 {   23195, .7328905273438E+04, -.1204955200195E+04, .5258557739258E+03},
 {   23196, .7331049804688E+04, -.1207825195312E+04, .5103127136230E+03},
 {   23197, .7327200195312E+04, -.1204040283203E+04, .5200705566406E+03},
 {   23198, .7352291503906E+04, -.1195760253906E+04, .5444063720703E+03},
 {   23199, .7339186523438E+04, -.1193372192383E+04, .5428143920898E+03},
 {   23200, .7352647460938E+04, -.1182145263672E+04, .5462357177734E+03},
 {   23201, .7338679199219E+04, -.1182120239258E+04, .5439477539062E+03},
 {   23202, .7352291503906E+04, -.1168480346680E+04, .5444063720703E+03},
 {   23203, .7339186523438E+04, -.1170868286133E+04, .5428143920898E+03},
 {   23204, .7351213378906E+04, -.1156415283203E+04, .5389852294922E+03},
 {   23205, .7340727539062E+04, -.1160200317383E+04, .5393093872070E+03},
 {   23206, .7326313964844E+04, -.1195760253906E+04, .5074642639160E+03},
 {   23207, .7323378417969E+04, -.1193372192383E+04, .5203342895508E+03},
 {   23208, .7324710449219E+04, -.1182145263672E+04, .5065106201172E+03},
 {   23209, .7322137207031E+04, -.1182120239258E+04, .5204285888672E+03},
 {   23210, .7326313964844E+04, -.1168480346680E+04, .5074642639160E+03},
 {   23211, .7323378417969E+04, -.1170868286133E+04, .5203342895508E+03},
 {   23212, .7331049804688E+04, -.1156415283203E+04, .5103127136230E+03},
 {   23213, .7327200195312E+04, -.1160200317383E+04, .5200705566406E+03},
 {   23214, .7345333984375E+04, -.1150776367188E+04, .5335855712891E+03},
 {   23215, .7335859863281E+04, -.1159285278320E+04, .5357449340820E+03},
 {   23216, .7339241210938E+04, -.1148516357422E+04, .5259782104492E+03},
 {   23217, .7331801757812E+04, -.1159057250977E+04, .5312097778320E+03},
 {   23218, .7334143066406E+04, -.1150776367188E+04, .5176706542969E+03},
 {   23219, .7328905273438E+04, -.1159285278320E+04, .5258557739258E+03},
 {   23220, .7333151367188E+04, -.1193931274414E+04, .5389553833008E+03},
 {   23221, .7332273437500E+04, -.1182120239258E+04, .5400068359375E+03},
 {   23222, .7333151367188E+04, -.1170309326172E+04, .5389553833008E+03},
 {   23223, .7328290527344E+04, -.1194007202148E+04, .5336789550781E+03},
 {   23224, .7327149414062E+04, -.1182120239258E+04, .5344822998047E+03},
 {   23225, .7328290527344E+04, -.1170233276367E+04, .5336789550781E+03},
 {   23226, .7324965820312E+04, -.1193931274414E+04, .5273206787109E+03},
 {   23227, .7323685058594E+04, -.1182120239258E+04, .5277890014648E+03},
 {   23228, .7324965820312E+04, -.1170309326172E+04, .5273206787109E+03},
 {   23229, .8013944335938E+04, -.1443929687500E+04, .7967790527344E+03},
 {   23230, .8059835937500E+04, -.1444047729492E+04, .7865989990234E+03},
 {   23231, .8062068359375E+04, -.1416960815430E+04, .7983303222656E+03},
 {   23232, .8016175292969E+04, -.1416843750000E+04, .8085103759766E+03},
 {   23233, .8051879394531E+04, -.1414875732422E+04, .7521079101562E+03},
 {   23234, .8053621582031E+04, -.1401905761719E+04, .7595177612305E+03},
 {   23235, .8007357910156E+04, -.1401895874023E+04, .7697412719727E+03},
 {   23236, .8005914550781E+04, -.1414905761719E+04, .7624177246094E+03},
 {   23237, .8009550292969E+04, -.1464204711914E+04, .7741535034180E+03},
 {   23238, .8055442871094E+04, -.1464322753906E+04, .7639741210938E+03},
 {   23239, .8049229003906E+04, -.1424545776367E+04, .7408461914062E+03},
 {   23240, .8003436523438E+04, -.1424855834961E+04, .7507937011719E+03},
 {   23241, .8002781738281E+04, -.1474270629883E+04, .7442796630859E+03},
 {   23242, .8048674316406E+04, -.1474387695312E+04, .7341000976562E+03},
 {   23243, .8046224609375E+04, -.1429540771484E+04, .7265540161133E+03},
 {   23244, .8000319335938E+04, -.1429605834961E+04, .7366719970703E+03},
 {   23245, .7995308105469E+04, -.1473155639648E+04, .7144840698242E+03},
 {   23246, .8041201660156E+04, -.1473272705078E+04, .7043046264648E+03},
 {   23247, .8042905273438E+04, -.1429720703125E+04, .7119089965820E+03},
 {   23248, .7997060058594E+04, -.1429775756836E+04, .7222951049805E+03},
 {   23249, .7988666015625E+04, -.1460383666992E+04, .6856539306641E+03},
 {   23250, .8034556152344E+04, -.1460500732422E+04, .6754736328125E+03},
 {   23251, .8039732910156E+04, -.1424151733398E+04, .6983287963867E+03},
 {   23252, .7993872070312E+04, -.1424205810547E+04, .7086503906250E+03},
 {   23253, .7984201171875E+04, -.1439340820312E+04, .6667402954102E+03},
 {   23254, .8030093261719E+04, -.1439457763672E+04, .6565598754883E+03},
 {   23255, .8037016113281E+04, -.1414623779297E+04, .6870425415039E+03},
 {   23256, .7991133300781E+04, -.1414585815430E+04, .6972677612305E+03},
 {   23257, .7981704589844E+04, -.1412630859375E+04, .6562225341797E+03},
 {   23258, .8027595703125E+04, -.1412748779297E+04, .6460433349609E+03},
 {   23259, .8035183593750E+04, -.1401535766602E+04, .6799106445312E+03},
 {   23260, .7989313964844E+04, -.1401555786133E+04, .6901903076172E+03},
 {   23261, .7980782226562E+04, -.1385377807617E+04, .6532086181641E+03},
 {   23262, .8026673828125E+04, -.1385495849609E+04, .6430294189453E+03},
 {   23263, .8034413574219E+04, -.1386723876953E+04, .6775649414062E+03},
 {   23264, .7988546875000E+04, -.1386845825195E+04, .6878453369141E+03},
 {   23265, .8062415527344E+04, -.1388477783203E+04, .8001046142578E+03},
 {   23266, .8016524414062E+04, -.1388360839844E+04, .8102846679688E+03},
 {   23267, .8053572753906E+04, -.1387559814453E+04, .7620023193359E+03},
 {   23268, .8007711425781E+04, -.1387885864258E+04, .7722954101562E+03},
 {   23269, .8061055175781E+04, -.1361718872070E+04, .7951768188477E+03},
 {   23270, .8015164062500E+04, -.1361600952148E+04, .8053560791016E+03},
 {   23271, .8052792968750E+04, -.1372749877930E+04, .7596420288086E+03},
 {   23272, .8006958007812E+04, -.1373108886719E+04, .7700522460938E+03},
 {   23273, .8058018066406E+04, -.1332687011719E+04, .7839579467773E+03},
 {   23274, .8012124023438E+04, -.1332568969727E+04, .7941373291016E+03},
 {   23275, .8050964843750E+04, -.1359667968750E+04, .7524988403320E+03},
 {   23276, .8005070800781E+04, -.1359549926758E+04, .7626783447266E+03},
 {   23277, .8053355957031E+04, -.1310406005859E+04, .7647996826172E+03},
 {   23278, .8007464355469E+04, -.1310287963867E+04, .7749791870117E+03},
 {   23279, .8048270507812E+04, -.1349971923828E+04, .7413513793945E+03},
 {   23280, .8002286621094E+04, -.1349683959961E+04, .7511516723633E+03},
 {   23281, .8047903808594E+04, -.1297249023438E+04, .7373876342773E+03},
 {   23282, .8002013671875E+04, -.1297130981445E+04, .7475679931641E+03},
 {   23283, .8045088378906E+04, -.1344606933594E+04, .7276940917969E+03},
 {   23284, .7999215820312E+04, -.1344625976562E+04, .7379631958008E+03},
 {   23285, .8040512207031E+04, -.1296104003906E+04, .7026752929688E+03},
 {   23286, .7994621093750E+04, -.1295987060547E+04, .7128544921875E+03},
 {   23287, .8041775878906E+04, -.1344770996094E+04, .7130460205078E+03},
 {   23288, .7995907714844E+04, -.1344800903320E+04, .7233336181641E+03},
 {   23289, .8034417968750E+04, -.1307198974609E+04, .6769516601562E+03},
 {   23290, .7988525878906E+04, -.1307082031250E+04, .6871321411133E+03},
 {   23291, .8038694824219E+04, -.1349480957031E+04, .6991224975586E+03},
 {   23292, .7992801757812E+04, -.1349559936523E+04, .7092978515625E+03},
 {   23293, .8029351074219E+04, -.1330199951172E+04, .6554116210938E+03},
 {   23294, .7983458984375E+04, -.1330083007812E+04, .6655911254883E+03},
 {   23295, .8036276855469E+04, -.1359031860352E+04, .6877921752930E+03},
 {   23296, .7990401855469E+04, -.1359097900391E+04, .6980446777344E+03},
 {   23297, .8026915527344E+04, -.1358628906250E+04, .6449560546875E+03},
 {   23298, .7981023925781E+04, -.1358510986328E+04, .6551363525391E+03},
 {   23299, .8034793457031E+04, -.1371962890625E+04, .6803016967773E+03},
 {   23300, .7988929199219E+04, -.1371940917969E+04, .6906033325195E+03},
 {   23301, .8001220703125E+04, -.1408455810547E+04, .7422080688477E+03},
 {   23302, .7999464355469E+04, -.1411365844727E+04, .7342177124023E+03},
 {   23303, .7961754882812E+04, -.1406912841797E+04, .7484312133789E+03},
 {   23304, .7960611328125E+04, -.1409706787109E+04, .7406536865234E+03},
 {   23305, .8002614746094E+04, -.1402955810547E+04, .7487508544922E+03},
 {   23306, .7962594726562E+04, -.1402080810547E+04, .7545281982422E+03},
 {   23307, .8003477050781E+04, -.1395485839844E+04, .7530625000000E+03},
 {   23308, .7963161621094E+04, -.1394622802734E+04, .7592706909180E+03},
 {   23309, .8003686523438E+04, -.1386955810547E+04, .7546102294922E+03},
 {   23310, .7963242187500E+04, -.1385443847656E+04, .7611640625000E+03},
 {   23311, .8003222167969E+04, -.1378407836914E+04, .7532058105469E+03},
 {   23312, .7962863281250E+04, -.1377205932617E+04, .7599011230469E+03},
 {   23313, .8002158203125E+04, -.1370894897461E+04, .7490296630859E+03},
 {   23314, .7962094238281E+04, -.1369857910156E+04, .7560422363281E+03},
 {   23315, .8000607910156E+04, -.1365261962891E+04, .7426120605469E+03},
 {   23316, .7961021484375E+04, -.1364253906250E+04, .7499358520508E+03},
 {   23317, .7998765136719E+04, -.1362252929688E+04, .7346968383789E+03},
 {   23318, .7959524902344E+04, -.1360593872070E+04, .7408673095703E+03},
 {   23319, .7996843261719E+04, -.1362159912109E+04, .7262283935547E+03},
 {   23320, .7958167968750E+04, -.1360638916016E+04, .7321204223633E+03},
 {   23321, .7995077636719E+04, -.1365028930664E+04, .7182316894531E+03},
 {   23322, .7957089843750E+04, -.1363238891602E+04, .7248180541992E+03},
 {   23323, .7993676269531E+04, -.1370533935547E+04, .7116607666016E+03},
 {   23324, .7956108886719E+04, -.1369184936523E+04, .7176415405273E+03},
 {   23325, .7992822753906E+04, -.1378012939453E+04, .7073295288086E+03},
 {   23326, .7955615722656E+04, -.1376889892578E+04, .7133008422852E+03},
 {   23327, .7997538574219E+04, -.1411275756836E+04, .7257708129883E+03},
 {   23328, .7959403320312E+04, -.1409791870117E+04, .7329229125977E+03},
 {   23329, .7995688476562E+04, -.1408255859375E+04, .7178160400391E+03},
 {   23330, .7958051757812E+04, -.1407001831055E+04, .7246262817383E+03},
 {   23331, .7992599609375E+04, -.1386555908203E+04, .7057767333984E+03},
 {   23332, .7955600097656E+04, -.1386156860352E+04, .7119037475586E+03},
 {   23333, .7993063476562E+04, -.1395115844727E+04, .7071831665039E+03},
 {   23334, .7956010742188E+04, -.1393825805664E+04, .7133992919922E+03},
 {   23335, .7994131835938E+04, -.1402645874023E+04, .7113712768555E+03},
 {   23336, .7957062988281E+04, -.1402598876953E+04, .7188812255859E+03},
 {   23337, .7959574707031E+04, -.1474574707031E+04, .7245714111328E+03},
 {   23338, .7964349609375E+04, -.1475796630859E+04, .7550921020508E+03},
 {   23339, .7968705566406E+04, -.1464172729492E+04, .7847775878906E+03},
 {   23340, .7971698242188E+04, -.1443299804688E+04, .8070582275391E+03},
 {   23341, .7972933105469E+04, -.1416362792969E+04, .8189700317383E+03},
 {   23342, .7972387207031E+04, -.1388050903320E+04, .8196209106445E+03},
 {   23343, .7971273437500E+04, -.1361277954102E+04, .8163096923828E+03},
 {   23344, .7968674316406E+04, -.1334822998047E+04, .8034373779297E+03},
 {   23345, .7965165527344E+04, -.1314743041992E+04, .7838224487305E+03},
 {   23346, .7960791992188E+04, -.1301947021484E+04, .7575697021484E+03},
 {   23347, .7955628906250E+04, -.1301226074219E+04, .7244522094727E+03},
 {   23348, .7951982910156E+04, -.1310802001953E+04, .6995729980469E+03},
 {   23349, .7948769531250E+04, -.1330589965820E+04, .6760317993164E+03},
 {   23350, .7947795410156E+04, -.1358723876953E+04, .6647707519531E+03},
 {   23351, .7948020019531E+04, -.1385449829102E+04, .6623287353516E+03},
 {   23352, .7949086914062E+04, -.1413303833008E+04, .6651205444336E+03},
 {   23353, .7950877929688E+04, -.1441311767578E+04, .6734286499023E+03},
 {   23354, .7954491210938E+04, -.1462453735352E+04, .6935974731445E+03},
 {   23355, .7927250976562E+04, -.1445071777344E+04, .8161356811523E+03},
 {   23356, .7925009765625E+04, -.1465914672852E+04, .7937529296875E+03},
 {   23357, .7917844726562E+04, -.1403203857422E+04, .7643766479492E+03},
 {   23358, .7916959960938E+04, -.1408709838867E+04, .7577640380859E+03},
 {   23359, .7915901855469E+04, -.1411620849609E+04, .7496884765625E+03},
 {   23360, .7921392578125E+04, -.1477510620117E+04, .7639677124023E+03},
 {   23361, .7914790039062E+04, -.1411522827148E+04, .7411125488281E+03},
 {   23362, .7917160156250E+04, -.1476266601562E+04, .7333716430664E+03},
 {   23363, .7913770019531E+04, -.1408493774414E+04, .7330751342773E+03},
 {   23364, .7912447265625E+04, -.1464130737305E+04, .7023474731445E+03},
 {   23365, .7912964843750E+04, -.1402875854492E+04, .7265483398438E+03},
 {   23366, .7908737304688E+04, -.1439635742188E+04, .6841181640625E+03},
 {   23367, .7912462402344E+04, -.1395348876953E+04, .7223170166016E+03},
 {   23368, .7906930175781E+04, -.1411442871094E+04, .6789802856445E+03},
 {   23369, .7912331054688E+04, -.1386819824219E+04, .7208859252930E+03},
 {   23370, .7905649414062E+04, -.1387131835938E+04, .6765682983398E+03},
 {   23371, .7912591308594E+04, -.1378312866211E+04, .7224235839844E+03},
 {   23372, .7904980468750E+04, -.1363292968750E+04, .6785016479492E+03},
 {   23373, .7913198730469E+04, -.1370849853516E+04, .7267526855469E+03},
 {   23374, .7904394531250E+04, -.1332338012695E+04, .6830981445312E+03},
 {   23375, .7907373535156E+04, -.1312579956055E+04, .7086740112305E+03},
 {   23376, .7914079101562E+04, -.1365335937500E+04, .7333574829102E+03},
 {   23377, .7915149414062E+04, -.1362438964844E+04, .7414363403320E+03},
 {   23378, .7910411132812E+04, -.1303028076172E+04, .7336357421875E+03},
 {   23379, .7914979492188E+04, -.1303771972656E+04, .7668353881836E+03},
 {   23380, .7916254394531E+04, -.1362508911133E+04, .7500145263672E+03},
 {   23381, .7917273925781E+04, -.1365517944336E+04, .7580581665039E+03},
 {   23382, .7919072265625E+04, -.1316578979492E+04, .7931240844727E+03},
 {   23383, .7918076660156E+04, -.1371153930664E+04, .7645720214844E+03},
 {   23384, .7922541503906E+04, -.1336659912109E+04, .8127456665039E+03},
 {   23385, .7918581542969E+04, -.1378667846680E+04, .7688114624023E+03},
 {   23386, .7925333984375E+04, -.1363107910156E+04, .8255910034180E+03},
 {   23387, .7918715332031E+04, -.1387218872070E+04, .7702534790039E+03},
 {   23388, .7926829101562E+04, -.1389866821289E+04, .8288512573242E+03},
 {   23389, .7918452636719E+04, -.1395748901367E+04, .7687050170898E+03},
 {   23390, .7927838378906E+04, -.1418159790039E+04, .8281369018555E+03},
 {   23391, .7872512207031E+04, -.1410505859375E+04, .7591165161133E+03},
 {   23392, .7870447265625E+04, -.1410561767578E+04, .7514668579102E+03},
 {   23393, .7873615234375E+04, -.1429881835938E+04, .7632010498047E+03},
 {   23394, .7869875976562E+04, -.1429922729492E+04, .7493534545898E+03},
 {   23395, .7868697753906E+04, -.1469853637695E+04, .7449929199219E+03},
 {   23396, .7876000976562E+04, -.1471757690430E+04, .7720289306641E+03},
 {   23397, .7860966796875E+04, -.1458855712891E+04, .7163734130859E+03},
 {   23398, .7865724609375E+04, -.1423124755859E+04, .7339832763672E+03},
 {   23399, .7867958984375E+04, -.1406341796875E+04, .7422549438477E+03},
 {   23400, .7866624023438E+04, -.1401971801758E+04, .7373178100586E+03},
 {   23401, .7863117187500E+04, -.1414017822266E+04, .7243322143555E+03},
 {   23402, .7874493652344E+04, -.1407040771484E+04, .7664457397461E+03},
 {   23403, .7877244628906E+04, -.1424707763672E+04, .7766386108398E+03},
 {   23404, .7883541503906E+04, -.1462894653320E+04, .7999480590820E+03},
 {   23405, .7889531250000E+04, -.1443358764648E+04, .8221184082031E+03},
 {   23406, .7880337890625E+04, -.1415669799805E+04, .7880843505859E+03},
 {   23407, .7876084960938E+04, -.1402857788086E+04, .7723374633789E+03},
 {   23408, .7892831542969E+04, -.1416850830078E+04, .8343345336914E+03},
 {   23409, .7882002441406E+04, -.1401051879883E+04, .7942521972656E+03},
 {   23410, .7877386718750E+04, -.1394313842773E+04, .7771577758789E+03},
 {   23411, .7892935546875E+04, -.1388878906250E+04, .8347235107422E+03},
 {   23412, .7882838867188E+04, -.1387404907227E+04, .7973416137695E+03},
 {   23413, .7877790527344E+04, -.1386667846680E+04, .7786502685547E+03},
 {   23414, .7877506835938E+04, -.1377880859375E+04, .7776018676758E+03},
 {   23415, .7882278808594E+04, -.1373247924805E+04, .7952725830078E+03},
 {   23416, .7892192382812E+04, -.1362497924805E+04, .8319711914062E+03},
 {   23417, .7880404296875E+04, -.1359391845703E+04, .7883312988281E+03},
 {   23418, .7888226074219E+04, -.1337169921875E+04, .8172875366211E+03},
 {   23419, .7876667480469E+04, -.1371359863281E+04, .7744981079102E+03},
 {   23420, .7882797363281E+04, -.1319647949219E+04, .7971937866211E+03},
 {   23421, .7877517089844E+04, -.1349469970703E+04, .7776472778320E+03},
 {   23422, .7874645996094E+04, -.1363845947266E+04, .7670171508789E+03},
 {   23423, .7875952148438E+04, -.1309239990234E+04, .7718508300781E+03},
 {   23424, .7873709960938E+04, -.1344196899414E+04, .7635553588867E+03},
 {   23425, .7872639648438E+04, -.1360893920898E+04, .7595935058594E+03},
 {   23426, .7868654785156E+04, -.1308744018555E+04, .7448305053711E+03},
 {   23427, .7870009765625E+04, -.1344356933594E+04, .7498574218750E+03},
 {   23428, .7870629882812E+04, -.1360591918945E+04, .7521494750977E+03},
 {   23429, .7868393066406E+04, -.1362717895508E+04, .7438631591797E+03},
 {   23430, .7866554199219E+04, -.1348300903320E+04, .7370540771484E+03},
 {   23431, .7862637695312E+04, -.1317605957031E+04, .7225568847656E+03},
 {   23432, .7866644531250E+04, -.1368569946289E+04, .7373930664062E+03},
 {   23433, .7863702636719E+04, -.1357980957031E+04, .7265006713867E+03},
 {   23434, .7857146484375E+04, -.1335928955078E+04, .7022305297852E+03},
 {   23435, .7865489746094E+04, -.1375586914062E+04, .7331162719727E+03},
 {   23436, .7861588867188E+04, -.1369992919922E+04, .7186759643555E+03},
 {   23437, .7864987792969E+04, -.1384212890625E+04, .7312590942383E+03},
 {   23438, .7860907714844E+04, -.1384022827148E+04, .7161544799805E+03},
 {   23439, .7865264160156E+04, -.1393553833008E+04, .7322852783203E+03},
 {   23440, .7861017578125E+04, -.1398694824219E+04, .7165607299805E+03},
 {   23441, .7856977050781E+04, -.1435099853516E+04, .7016057739258E+03},
 {   23442, .7857012207031E+04, -.1403541870117E+04, .7017351074219E+03},
 {   23443, .7856738281250E+04, -.1383828857422E+04, .7007195434570E+03},
 {   23444, .7856882324219E+04, -.1363242919922E+04, .7012543945312E+03},
 {   23445, .8047363769531E+04, -.1398815795898E+04, .7329129638672E+03},
 {   23446, .8048092285156E+04, -.1393395874023E+04, .7360172119141E+03},
 {   23447, .8046277343750E+04, -.1402755859375E+04, .7282926025391E+03},
 {   23448, .8044943847656E+04, -.1404875854492E+04, .7226240844727E+03},
 {   23449, .8043523437500E+04, -.1404895874023E+04, .7165825195312E+03},
 {   23450, .8042170410156E+04, -.1402795898438E+04, .7108406982422E+03},
 {   23451, .8041065429688E+04, -.1398785888672E+04, .7061389770508E+03},
 {   23452, .8040337402344E+04, -.1393355834961E+04, .7030478515625E+03},
 {   23453, .8040082031250E+04, -.1387145874023E+04, .7019578247070E+03},
 {   23454, .8040341796875E+04, -.1380794921875E+04, .7030613403320E+03},
 {   23455, .8048353027344E+04, -.1387205810547E+04, .7371206665039E+03},
 {   23456, .8048119628906E+04, -.1381121826172E+04, .7361320800781E+03},
 {   23457, .8047428710938E+04, -.1375710937500E+04, .7331898803711E+03},
 {   23458, .8046358398438E+04, -.1371635864258E+04, .7286313476562E+03},
 {   23459, .8045018554688E+04, -.1369374877930E+04, .7229426269531E+03},
 {   23460, .8043874511719E+04, -.1369247924805E+04, .7180796508789E+03},
 {   23461, .8042219238281E+04, -.1371281860352E+04, .7110448608398E+03},
 {   23462, .8041097656250E+04, -.1375245849609E+04, .7062694702148E+03},
 {   23463, .8070830566406E+04, -.1402785888672E+04, .7038637084961E+03},
 {   23464, .8072181152344E+04, -.1404895874023E+04, .7096053466797E+03},
 {   23465, .8069725097656E+04, -.1398785888672E+04, .6991625366211E+03},
 {   23466, .8073602050781E+04, -.1404875854492E+04, .7156466064453E+03},
 {   23467, .8068997070312E+04, -.1393355834961E+04, .6960710449219E+03},
 {   23468, .8068741699219E+04, -.1387145874023E+04, .6949804687500E+03},
 {   23469, .8069000000000E+04, -.1380794921875E+04, .6960838012695E+03},
 {   23470, .8069965332031E+04, -.1374700927734E+04, .7001832275391E+03},
 {   23471, .8074935058594E+04, -.1402755859375E+04, .7213171997070E+03},
 {   23472, .8076022460938E+04, -.1398815795898E+04, .7259368286133E+03},
 {   23473, .8076752929688E+04, -.1393395874023E+04, .7290416870117E+03},
 {   23474, .8077011718750E+04, -.1387205810547E+04, .7301449584961E+03},
 {   23475, .8075015136719E+04, -.1371634887695E+04, .7216560668945E+03},
 {   23476, .8073678710938E+04, -.1369373901367E+04, .7159671630859E+03},
 {   23477, .8072878417969E+04, -.1369246948242E+04, .7125631713867E+03},
 {   23478, .8076087402344E+04, -.1375709838867E+04, .7262133789062E+03},
 {   23479, .8071539550781E+04, -.1370608886719E+04, .7068753051758E+03},
 {   23480, .8076780761719E+04, -.1381121826172E+04, .7291552734375E+03},
 {   23481, .8094782714844E+04, -.1387095825195E+04, .7074111328125E+03},
 {   23482, .8094085937500E+04, -.1404895874023E+04, .7044540405273E+03},
 {   23483, .8092737304688E+04, -.1402785888672E+04, .6987117309570E+03},
 {   23484, .8091629882812E+04, -.1398785888672E+04, .6940104980469E+03},
 {   23485, .8095507324219E+04, -.1404875854492E+04, .7104957885742E+03},
 {   23486, .8090902832031E+04, -.1393355834961E+04, .6909189453125E+03},
 {   23487, .8090647949219E+04, -.1387095825195E+04, .6898292236328E+03},
 {   23488, .8090906738281E+04, -.1380793823242E+04, .6909327392578E+03},
 {   23489, .8091661621094E+04, -.1375244873047E+04, .6941409301758E+03},
 {   23490, .8096842285156E+04, -.1402755859375E+04, .7161654052734E+03},
 {   23491, .8097927246094E+04, -.1398815795898E+04, .7207849731445E+03},
 {   23492, .8098658203125E+04, -.1393395874023E+04, .7238899536133E+03},
 {   23493, .8098918457031E+04, -.1387095825195E+04, .7249920654297E+03},
 {   23494, .8095584472656E+04, -.1369373901367E+04, .7108154296875E+03},
 {   23495, .8096920410156E+04, -.1371634887695E+04, .7165043334961E+03},
 {   23496, .8094782714844E+04, -.1369245849609E+04, .7074111328125E+03},
 {   23497, .8097992187500E+04, -.1375708862305E+04, .7210614624023E+03},
 {   23498, .8093222656250E+04, -.1370874877930E+04, .7007848510742E+03},
 {   23499, .8098684570312E+04, -.1381120849609E+04, .7240037231445E+03},
 {   23500, .7915525878906E+04, -.1387028808594E+04, .7455625000000E+03},
 {   23501, .7962889160156E+04, -.1423871826172E+04, .7591493530273E+03},
 {   23502, .7960445312500E+04, -.1429724731445E+04, .7435739746094E+03},
 {   23503, .7964520996094E+04, -.1415559814453E+04, .7707067260742E+03},
 {   23504, .7965526855469E+04, -.1401401855469E+04, .7772509765625E+03},
 {   23505, .7965880859375E+04, -.1386850830078E+04, .7803228759766E+03},
 {   23506, .7965221679688E+04, -.1372151855469E+04, .7782768554688E+03},
 {   23507, .7963635253906E+04, -.1358479858398E+04, .7709398803711E+03},
 {   23508, .7961469238281E+04, -.1348942871094E+04, .7597037353516E+03},
 {   23509, .7958590332031E+04, -.1343159912109E+04, .7440861816406E+03},
 {   23510, .7956375000000E+04, -.1343768920898E+04, .7294537353516E+03},
 {   23511, .7954546386719E+04, -.1348013916016E+04, .7161949462891E+03},
 {   23512, .7953244140625E+04, -.1359717895508E+04, .7045060424805E+03},
 {   23513, .7952984375000E+04, -.1370963867188E+04, .6977984008789E+03},
 {   23514, .7958260742188E+04, -.1429102783203E+04, .7287546997070E+03},
 {   23515, .7956090820312E+04, -.1422980834961E+04, .7153631591797E+03},
 {   23516, .7953199218750E+04, -.1385334838867E+04, .6958916625977E+03},
 {   23517, .7953366210938E+04, -.1399083862305E+04, .6966752929688E+03},
 {   23518, .7954530273438E+04, -.1413911865234E+04, .7050059204102E+03},
 {   23519, .7919912597656E+04, -.1425585815430E+04, .7680271606445E+03},
 {   23520, .7921174804688E+04, -.1417288818359E+04, .7796345825195E+03},
 {   23521, .7918002441406E+04, -.1431417724609E+04, .7523786010742E+03},
 {   23522, .7915950195312E+04, -.1430790771484E+04, .7375405273438E+03},
 {   23523, .7913876953125E+04, -.1424664794922E+04, .7241352539062E+03},
 {   23524, .7912110839844E+04, -.1415603759766E+04, .7138073120117E+03},
 {   23525, .7910402832031E+04, -.1400797851562E+04, .7055510253906E+03},
 {   23526, .7909765136719E+04, -.1387066894531E+04, .7048302612305E+03},
 {   23527, .7909466308594E+04, -.1372698852539E+04, .7067498168945E+03},
 {   23528, .7909937988281E+04, -.1361444946289E+04, .7134268798828E+03},
 {   23529, .7911081542969E+04, -.1349746948242E+04, .7251381835938E+03},
 {   23530, .7912730468750E+04, -.1345508911133E+04, .7384215087891E+03},
 {   23531, .7914710937500E+04, -.1344909912109E+04, .7530855102539E+03},
 {   23532, .7917082519531E+04, -.1350712890625E+04, .7687738647461E+03},
 {   23533, .7918996582031E+04, -.1360259887695E+04, .7800437622070E+03},
 {   23534, .7920537597656E+04, -.1373932861328E+04, .7873880004883E+03},
 {   23535, .7921387207031E+04, -.1388624877930E+04, .7894073486328E+03},
 {   23536, .7921528320312E+04, -.1403155761719E+04, .7862667846680E+03},
 {   23537, .7884202148438E+04, -.1336138916016E+04, .8169952392578E+03},
 {   23538, .7877756347656E+04, -.1315603027344E+04, .7931362915039E+03},
 {   23539, .7852810546875E+04, -.1319515014648E+04, .7981912841797E+03},
 {   23540, .7858898925781E+04, -.1339135986328E+04, .8207257080078E+03},
 {   23541, .7876776855469E+04, -.1364611938477E+04, .7895074462891E+03},
 {   23542, .7873463867188E+04, -.1364611938477E+04, .7772467651367E+03},
 {   23543, .7827865234375E+04, -.1323426025391E+04, .8032454833984E+03},
 {   23544, .7833601074219E+04, -.1342107910156E+04, .8244803466797E+03},
 {   23545, .7802927734375E+04, -.1327338012695E+04, .8083244628906E+03},
 {   23546, .7808299804688E+04, -.1345104980469E+04, .8282107543945E+03},
 {   23547, .7799901367188E+04, -.1366610839844E+04, .7971181030273E+03},
 {   23548, .7803212402344E+04, -.1366610839844E+04, .8093787231445E+03},
 {   23549, .7870151367188E+04, -.1309812011719E+04, .7649863891602E+03},
 {   23550, .7845629882812E+04, -.1314053955078E+04, .7716101074219E+03},
 {   23551, .7870151367188E+04, -.1364611938477E+04, .7649863891602E+03},
 {   23552, .7821108886719E+04, -.1318270019531E+04, .7782338867188E+03},
 {   23553, .7796588867188E+04, -.1322511962891E+04, .7848575439453E+03},
 {   23554, .7796588867188E+04, -.1366610839844E+04, .7848575439453E+03},
 {   23555, .7862548339844E+04, -.1315603027344E+04, .7368361816406E+03},
 {   23556, .7838450195312E+04, -.1319515014648E+04, .7450288696289E+03},
 {   23557, .7866840820312E+04, -.1364611938477E+04, .7527257690430E+03},
 {   23558, .7814354003906E+04, -.1323426025391E+04, .7532221679688E+03},
 {   23559, .7790250000000E+04, -.1327338012695E+04, .7613905639648E+03},
 {   23560, .7793276855469E+04, -.1366610839844E+04, .7725970458984E+03},
 {   23561, .7856102539062E+04, -.1336138916016E+04, .7129774780273E+03},
 {   23562, .7832364257812E+04, -.1339135986328E+04, .7224943847656E+03},
 {   23563, .7863528320312E+04, -.1364611938477E+04, .7404650268555E+03},
 {   23564, .7808617187500E+04, -.1342107910156E+04, .7319873046875E+03},
 {   23565, .7784877929688E+04, -.1345104980469E+04, .7415046386719E+03},
 {   23566, .7789963867188E+04, -.1366610839844E+04, .7603364257812E+03},
 {   23567, .7888513671875E+04, -.1360852905273E+04, .8329589843750E+03},
 {   23568, .7862971679688E+04, -.1362477905273E+04, .8358060302734E+03},
 {   23569, .7876776855469E+04, -.1377311889648E+04, .7895074462891E+03},
 {   23570, .7837429199219E+04, -.1364103881836E+04, .8386542968750E+03},
 {   23571, .7811888671875E+04, -.1367702880859E+04, .8415014038086E+03},
 {   23572, .7890023925781E+04, -.1390011840820E+04, .8385494384766E+03},
 {   23573, .7864395996094E+04, -.1390011840820E+04, .8410782470703E+03},
 {   23574, .7876776855469E+04, -.1390011840820E+04, .7895074462891E+03},
 {   23575, .7838775390625E+04, -.1390011840820E+04, .8436314697266E+03},
 {   23576, .7813146972656E+04, -.1390011840820E+04, .8461602172852E+03},
 {   23577, .7888513671875E+04, -.1419170776367E+04, .8329591064453E+03},
 {   23578, .7862971679688E+04, -.1417545776367E+04, .8358060302734E+03},
 {   23579, .7876776855469E+04, -.1402711791992E+04, .7895074462891E+03},
 {   23580, .7837429199219E+04, -.1415919799805E+04, .8386542968750E+03},
 {   23581, .7811888671875E+04, -.1412320800781E+04, .8415014038086E+03},
 {   23582, .7884202148438E+04, -.1443884765625E+04, .8169952392578E+03},
 {   23583, .7858898925781E+04, -.1440887695312E+04, .8207257080078E+03},
 {   23584, .7876776855469E+04, -.1415411743164E+04, .7895074462891E+03},
 {   23585, .7833601074219E+04, -.1437915771484E+04, .8244803466797E+03},
 {   23586, .7808299804688E+04, -.1434918701172E+04, .8282107543945E+03},
 {   23587, .7873463867188E+04, -.1377311889648E+04, .7772467651367E+03},
 {   23588, .7873463867188E+04, -.1390011840820E+04, .7772467651367E+03},
 {   23589, .7873463867188E+04, -.1402711791992E+04, .7772467651367E+03},
 {   23590, .7873463867188E+04, -.1415411743164E+04, .7772467651367E+03},
 {   23591, .7870151367188E+04, -.1377311889648E+04, .7649863891602E+03},
 {   23592, .7870151367188E+04, -.1415411743164E+04, .7649863891602E+03},
 {   23593, .7870151367188E+04, -.1402711791992E+04, .7649863891602E+03},
 {   23594, .7866840820312E+04, -.1377311889648E+04, .7527257690430E+03},
 {   23595, .7866840820312E+04, -.1415411743164E+04, .7527257690430E+03},
 {   23596, .7866840820312E+04, -.1402711791992E+04, .7527257690430E+03},
 {   23597, .7863528320312E+04, -.1377311889648E+04, .7404650268555E+03},
 {   23598, .7866840820312E+04, -.1390011840820E+04, .7527257690430E+03},
 {   23599, .7863528320312E+04, -.1390011840820E+04, .7404650268555E+03},
 {   23600, .7863528320312E+04, -.1402711791992E+04, .7404650268555E+03},
 {   23601, .7863528320312E+04, -.1415411743164E+04, .7404650268555E+03},
 {   23602, .7853876953125E+04, -.1360852905273E+04, .7047368164062E+03},
 {   23603, .7830375488281E+04, -.1362477905273E+04, .7151370239258E+03},
 {   23604, .7806874023438E+04, -.1364103881836E+04, .7255366821289E+03},
 {   23605, .7783374023438E+04, -.1367702880859E+04, .7359370117188E+03},
 {   23606, .7852889648438E+04, -.1390011840820E+04, .7010767822266E+03},
 {   23607, .7829472656250E+04, -.1390011840820E+04, .7117958374023E+03},
 {   23608, .7806051757812E+04, -.1390011840820E+04, .7224898681641E+03},
 {   23609, .7782636718750E+04, -.1390011840820E+04, .7332086791992E+03},
 {   23610, .7853876953125E+04, -.1419170776367E+04, .7047368164062E+03},
 {   23611, .7830375488281E+04, -.1417545776367E+04, .7151370239258E+03},
 {   23612, .7806874023438E+04, -.1415919799805E+04, .7255366821289E+03},
 {   23613, .7783374023438E+04, -.1412320800781E+04, .7359370117188E+03},
 {   23614, .7856102539062E+04, -.1443884765625E+04, .7129774780273E+03},
 {   23615, .7832364257812E+04, -.1440887695312E+04, .7224943847656E+03},
 {   23616, .7808617187500E+04, -.1437915771484E+04, .7319873046875E+03},
 {   23617, .7784877929688E+04, -.1434918823242E+04, .7415046386719E+03},
 {   23618, .7852810546875E+04, -.1461508666992E+04, .7981912841797E+03},
 {   23619, .7877756347656E+04, -.1465420654297E+04, .7931364135742E+03},
 {   23620, .7827865234375E+04, -.1457597656250E+04, .8032454833984E+03},
 {   23621, .7803212402344E+04, -.1413412841797E+04, .8093787231445E+03},
 {   23622, .7799901367188E+04, -.1413412841797E+04, .7971181030273E+03},
 {   23623, .7802927734375E+04, -.1453685668945E+04, .8083244628906E+03},
 {   23624, .7845629882812E+04, -.1466969726562E+04, .7716101074219E+03},
 {   23625, .7870151367188E+04, -.1471211669922E+04, .7649863891602E+03},
 {   23626, .7821108886719E+04, -.1462753662109E+04, .7782338867188E+03},
 {   23627, .7796588867188E+04, -.1413412841797E+04, .7848575439453E+03},
 {   23628, .7796588867188E+04, -.1458511718750E+04, .7848575439453E+03},
 {   23629, .7838450195312E+04, -.1461508666992E+04, .7450288696289E+03},
 {   23630, .7862548339844E+04, -.1465420654297E+04, .7368361816406E+03},
 {   23631, .7814354003906E+04, -.1457597656250E+04, .7532221679688E+03},
 {   23632, .7793276855469E+04, -.1413412841797E+04, .7725970458984E+03},
 {   23633, .7790250000000E+04, -.1453685668945E+04, .7613907470703E+03},
 {   23634, .7789963867188E+04, -.1413412841797E+04, .7603364257812E+03},
 {   23635, .7788275390625E+04, -.1379031860352E+04, .8514847412109E+03},
 {   23636, .7788627441406E+04, -.1390011840820E+04, .8527843017578E+03},
 {   23637, .7788275390625E+04, -.1400991821289E+04, .8514847412109E+03},
 {   23638, .7755507324219E+04, -.1390011840820E+04, .7301785888672E+03},
 {   23639, .7755858886719E+04, -.1379031860352E+04, .7314779663086E+03},
 {   23640, .7755858886719E+04, -.1400991821289E+04, .7314779663086E+03},
 {   23641, .7787242675781E+04, -.1366610839844E+04, .8476586914062E+03},
 {   23642, .7764585937500E+04, -.1366610839844E+04, .8492017822266E+03},
 {   23643, .7764092285156E+04, -.1378980834961E+04, .8531228637695E+03},
 {   23644, .7743073730469E+04, -.1366610839844E+04, .8419358520508E+03},
 {   23645, .7741105468750E+04, -.1379031860352E+04, .8453620605469E+03},
 {   23646, .7763900390625E+04, -.1390011840820E+04, .8544641723633E+03},
 {   23647, .7740424804688E+04, -.1390011840820E+04, .8465450439453E+03},
 {   23648, .7764092285156E+04, -.1401042846680E+04, .8531228637695E+03},
 {   23649, .7741105468750E+04, -.1400991821289E+04, .8453620605469E+03},
 {   23650, .7787242675781E+04, -.1413412841797E+04, .8476586914062E+03},
 {   23651, .7764585937500E+04, -.1413412841797E+04, .8492017822266E+03},
 {   23652, .7743073730469E+04, -.1413412841797E+04, .8419358520508E+03},
 {   23653, .7756891113281E+04, -.1366610839844E+04, .7353039550781E+03},
 {   23654, .7734139648438E+04, -.1378980834961E+04, .7422391357422E+03},
 {   23655, .7736540527344E+04, -.1366610839844E+04, .7453790283203E+03},
 {   23656, .7718185546875E+04, -.1379031860352E+04, .7605189208984E+03},
 {   23657, .7721612304688E+04, -.1366610839844E+04, .7624876708984E+03},
 {   23658, .7733299316406E+04, -.1390011840820E+04, .7411765136719E+03},
 {   23659, .7717003417969E+04, -.1390011840820E+04, .7598388671875E+03},
 {   23660, .7734139648438E+04, -.1401042846680E+04, .7422391357422E+03},
 {   23661, .7718185546875E+04, -.1400991821289E+04, .7605189208984E+03},
 {   23662, .7756891113281E+04, -.1413412841797E+04, .7353039550781E+03},
 {   23663, .7736540527344E+04, -.1413412841797E+04, .7453790283203E+03},
 {   23664, .7721612304688E+04, -.1413412841797E+04, .7624876708984E+03},
 {   23665, .7725965332031E+04, -.1366610839844E+04, .8270089111328E+03},
 {   23666, .7723002929688E+04, -.1378878906250E+04, .8295725708008E+03},
 {   23667, .7721958007812E+04, -.1390011840820E+04, .8304853515625E+03},
 {   23668, .7723002929688E+04, -.1401144775391E+04, .8295725708008E+03},
 {   23669, .7725965332031E+04, -.1413412841797E+04, .8270089111328E+03},
 {   23670, .7715889648438E+04, -.1366610839844E+04, .8066561889648E+03},
 {   23671, .7712114746094E+04, -.1378802856445E+04, .8076763916016E+03},
 {   23672, .7710764648438E+04, -.1390011840820E+04, .8080407104492E+03},
 {   23673, .7712114746094E+04, -.1401220825195E+04, .8076763916016E+03},
 {   23674, .7715889648438E+04, -.1413412841797E+04, .8066561889648E+03},
 {   23675, .7714354003906E+04, -.1366610839844E+04, .7840236206055E+03},
 {   23676, .7710495605469E+04, -.1378878906250E+04, .7832760620117E+03},
 {   23677, .7709140136719E+04, -.1390011840820E+04, .7830369873047E+03},
 {   23678, .7710495605469E+04, -.1401144775391E+04, .7832760620117E+03},
 {   23679, .7714354003906E+04, -.1413412841797E+04, .7840236206055E+03},
 {   23680, .7795470703125E+04, -.1364611938477E+04, .8068327636719E+03},
 {   23681, .7798421875000E+04, -.1364611938477E+04, .7793250732422E+03},
 {   23682, .7776863769531E+04, -.1337814941406E+04, .7854505615234E+03},
 {   23683, .7783324218750E+04, -.1345485961914E+04, .8024557495117E+03},
 {   23684, .7788346191406E+04, -.1364611938477E+04, .7535595703125E+03},
 {   23685, .7766922363281E+04, -.1335172973633E+04, .7686232299805E+03},
 {   23686, .7766873535156E+04, -.1364611938477E+04, .7351658325195E+03},
 {   23687, .7749694824219E+04, -.1340633911133E+04, .7496875000000E+03},
 {   23688, .7742781250000E+04, -.1355746948242E+04, .7079732666016E+03},
 {   23689, .7728569335938E+04, -.1339008911133E+04, .7179666748047E+03},
 {   23690, .7717557617188E+04, -.1346907958984E+04, .6791307983398E+03},
 {   23691, .7707419433594E+04, -.1337357910156E+04, .6862610473633E+03},
 {   23692, .7691000488281E+04, -.1349600952148E+04, .6460440673828E+03},
 {   23693, .7681504394531E+04, -.1340633911133E+04, .6527217407227E+03},
 {   23694, .7664455566406E+04, -.1352292968750E+04, .6129791870117E+03},
 {   23695, .7655585449219E+04, -.1343935913086E+04, .6192182006836E+03},
 {   23696, .7637898437500E+04, -.1354984985352E+04, .5798928222656E+03},
 {   23697, .7629668457031E+04, -.1347237915039E+04, .5856783447266E+03},
 {   23698, .7611337890625E+04, -.1357677856445E+04, .5468067016602E+03},
 {   23699, .7603732421875E+04, -.1350514892578E+04, .5521538085938E+03},
 {   23700, .7584793945312E+04, -.1360369873047E+04, .5137412109375E+03},
 {   23701, .7577812988281E+04, -.1353816894531E+04, .5186510009766E+03},
 {   23702, .7558235351562E+04, -.1363062866211E+04, .4806552429199E+03},
 {   23703, .7551896972656E+04, -.1357093872070E+04, .4851116027832E+03},
 {   23704, .7545080078125E+04, -.1361284912109E+04, .4588558044434E+03},
 {   23705, .7538326660156E+04, -.1354908935547E+04, .4636043395996E+03},
 {   23706, .7531942871094E+04, -.1359480957031E+04, .4370412292480E+03},
 {   23707, .7524773437500E+04, -.1352724975586E+04, .4420820007324E+03},
 {   23708, .7521510253906E+04, -.1372231933594E+04, .4443758544922E+03},
 {   23709, .7521510253906E+04, -.1359531982422E+04, .4443758544922E+03},
 {   23710, .7770504394531E+04, -.1376955932617E+04, .7326088256836E+03},
 {   23711, .7750011230469E+04, -.1371825927734E+04, .7028883056641E+03},
 {   23712, .7771731445312E+04, -.1390011840820E+04, .7317463989258E+03},
 {   23713, .7752523925781E+04, -.1390011840820E+04, .7011202392578E+03},
 {   23714, .7770504394531E+04, -.1403067871094E+04, .7326088256836E+03},
 {   23715, .7750011230469E+04, -.1408197753906E+04, .7028883056641E+03},
 {   23716, .7766873535156E+04, -.1415411743164E+04, .7351658325195E+03},
 {   23717, .7742781250000E+04, -.1424276733398E+04, .7079732666016E+03},
 {   23718, .7728364746094E+04, -.1366694946289E+04, .6715332031250E+03},
 {   23719, .7732164550781E+04, -.1390011840820E+04, .6688596191406E+03},
 {   23720, .7728364746094E+04, -.1413328857422E+04, .6715332031250E+03},
 {   23721, .7717557617188E+04, -.1433115722656E+04, .6791307983398E+03},
 {   23722, .7701137207031E+04, -.1368142944336E+04, .6389140625000E+03},
 {   23723, .7704691894531E+04, -.1390011840820E+04, .6364161376953E+03},
 {   23724, .7701137207031E+04, -.1411880859375E+04, .6389140625000E+03},
 {   23725, .7691000488281E+04, -.1430422851562E+04, .6460440673828E+03},
 {   23726, .7673909667969E+04, -.1369589843750E+04, .6063314208984E+03},
 {   23727, .7677234375000E+04, -.1390011840820E+04, .6039934082031E+03},
 {   23728, .7673909667969E+04, -.1410433837891E+04, .6063314208984E+03},
 {   23729, .7664455566406E+04, -.1427730834961E+04, .6129791870117E+03},
 {   23730, .7646686523438E+04, -.1371063842773E+04, .5737122192383E+03},
 {   23731, .7649761718750E+04, -.1390011840820E+04, .5715502319336E+03},
 {   23732, .7646686523438E+04, -.1408959838867E+04, .5737122192383E+03},
 {   23733, .7637898437500E+04, -.1425038818359E+04, .5798928222656E+03},
 {   23734, .7619440429688E+04, -.1372510864258E+04, .5411082153320E+03},
 {   23735, .7622286132812E+04, -.1390011840820E+04, .5391065673828E+03},
 {   23736, .7619440429688E+04, -.1407512817383E+04, .5411082153320E+03},
 {   23737, .7611337890625E+04, -.1422345703125E+04, .5468067016602E+03},
 {   23738, .7592211914062E+04, -.1373984863281E+04, .5085250854492E+03},
 {   23739, .7594829101562E+04, -.1390011840820E+04, .5066844177246E+03},
 {   23740, .7592211914062E+04, -.1406038818359E+04, .5085250854492E+03},
 {   23741, .7584793945312E+04, -.1419653808594E+04, .5137412109375E+03},
 {   23742, .7564989257812E+04, -.1375431884766E+04, .4759068603516E+03},
 {   23743, .7567356933594E+04, -.1390011840820E+04, .4742410888672E+03},
 {   23744, .7564989257812E+04, -.1404591796875E+04, .4759068603516E+03},
 {   23745, .7558235351562E+04, -.1416960815430E+04, .4806552429199E+03},
 {   23746, .7552286132812E+04, -.1374466918945E+04, .4537854614258E+03},
 {   23747, .7554821289062E+04, -.1390011840820E+04, .4520028381348E+03},
 {   23748, .7552286132812E+04, -.1405556884766E+04, .4537854614258E+03},
 {   23749, .7545080078125E+04, -.1418738769531E+04, .4588558044434E+03},
 {   23750, .7539609375000E+04, -.1373476928711E+04, .4316498718262E+03},
 {   23751, .7521510253906E+04, -.1381121826172E+04, .4443758544922E+03},
 {   23752, .7542291503906E+04, -.1390011840820E+04, .4297640686035E+03},
 {   23753, .7521510253906E+04, -.1390011840820E+04, .4443758544922E+03},
 {   23754, .7539609375000E+04, -.1406546875000E+04, .4316498718262E+03},
 {   23755, .7521510253906E+04, -.1398901855469E+04, .4443758544922E+03},
 {   23756, .7531942871094E+04, -.1420542724609E+04, .4370412292480E+03},
 {   23757, .7521510253906E+04, -.1407791870117E+04, .4443758544922E+03},
 {   23758, .7783324218750E+04, -.1434537719727E+04, .8024557495117E+03},
 {   23759, .7776863769531E+04, -.1442208740234E+04, .7854505615234E+03},
 {   23760, .7798421875000E+04, -.1415411743164E+04, .7793250732422E+03},
 {   23761, .7795470703125E+04, -.1415411743164E+04, .8068327636719E+03},
 {   23762, .7766922363281E+04, -.1444850708008E+04, .7686232299805E+03},
 {   23763, .7788346191406E+04, -.1415411743164E+04, .7535595703125E+03},
 {   23764, .7749694824219E+04, -.1439389770508E+04, .7496875000000E+03},
 {   23765, .7728569335938E+04, -.1441014770508E+04, .7179666748047E+03},
 {   23766, .7707419433594E+04, -.1442665771484E+04, .6862610473633E+03},
 {   23767, .7681504394531E+04, -.1439389770508E+04, .6527217407227E+03},
 {   23768, .7655585449219E+04, -.1436087768555E+04, .6192182006836E+03},
 {   23769, .7629668457031E+04, -.1432785766602E+04, .5856783447266E+03},
 {   23770, .7603732421875E+04, -.1429508789062E+04, .5521538085938E+03},
 {   23771, .7577812988281E+04, -.1426206787109E+04, .5186510009766E+03},
 {   23772, .7551896972656E+04, -.1422929809570E+04, .4851116027832E+03},
 {   23773, .7538326660156E+04, -.1425114746094E+04, .4636043395996E+03},
 {   23774, .7524773437500E+04, -.1427298828125E+04, .4420820007324E+03},
 {   23775, .7521510253906E+04, -.1420491821289E+04, .4443758544922E+03},
 {   23776, .7765858886719E+04, -.1332251953125E+04, .7951460571289E+03},
 {   23777, .7774478515625E+04, -.1342310913086E+04, .8119076538086E+03},
 {   23778, .7753195800781E+04, -.1330322021484E+04, .8047355346680E+03},
 {   23779, .7764121093750E+04, -.1341218994141E+04, .8202764892578E+03},
 {   23780, .7739887695312E+04, -.1332251953125E+04, .8134098510742E+03},
 {   23781, .7752747070312E+04, -.1342310913086E+04, .8271905517578E+03},
 {   23782, .7727044433594E+04, -.1337814941406E+04, .8204884033203E+03},
 {   23783, .7740857910156E+04, -.1345485961914E+04, .8323207397461E+03},
 {   23784, .7754291992188E+04, -.1328721923828E+04, .7775068359375E+03},
 {   23785, .7740743652344E+04, -.1326511962891E+04, .7870332031250E+03},
 {   23786, .7727198242188E+04, -.1328721923828E+04, .7965603027344E+03},
 {   23787, .7714567871094E+04, -.1335172973633E+04, .8054432373047E+03},
 {   23788, .7738331054688E+04, -.1334843017578E+04, .7576799316406E+03},
 {   23789, .7726132812500E+04, -.1332861938477E+04, .7662562866211E+03},
 {   23790, .7713937011719E+04, -.1334843017578E+04, .7748330078125E+03},
 {   23791, .7702572265625E+04, -.1340633911133E+04, .7828253784180E+03},
 {   23792, .7716828613281E+04, -.1333013916016E+04, .7262217407227E+03},
 {   23793, .7704219238281E+04, -.1330956909180E+04, .7350914916992E+03},
 {   23794, .7691604003906E+04, -.1333013916016E+04, .7439599609375E+03},
 {   23795, .7679888671875E+04, -.1339008911133E+04, .7522006835938E+03},
 {   23796, .7695308105469E+04, -.1331159912109E+04, .6947791748047E+03},
 {   23797, .7682299804688E+04, -.1329052001953E+04, .7039257202148E+03},
 {   23798, .7669293457031E+04, -.1331159912109E+04, .7130719604492E+03},
 {   23799, .7657180664062E+04, -.1337357910156E+04, .7215902099609E+03},
 {   23800, .7670139160156E+04, -.1334843017578E+04, .6607133789062E+03},
 {   23801, .7657943359375E+04, -.1332861938477E+04, .6692905883789E+03},
 {   23802, .7645751464844E+04, -.1334843017578E+04, .6778676757812E+03},
 {   23803, .7634384765625E+04, -.1340633911133E+04, .6858596801758E+03},
 {   23804, .7644987304688E+04, -.1338526000977E+04, .6266702270508E+03},
 {   23805, .7633604003906E+04, -.1336671997070E+04, .6346764526367E+03},
 {   23806, .7622219726562E+04, -.1338526000977E+04, .6426837158203E+03},
 {   23807, .7611620605469E+04, -.1343935913086E+04, .6501353149414E+03},
 {   23808, .7619822265625E+04, -.1342208984375E+04, .5926040039062E+03},
 {   23809, .7609246582031E+04, -.1340481933594E+04, .6000415039062E+03},
 {   23810, .7598670410156E+04, -.1342208984375E+04, .6074786376953E+03},
 {   23811, .7588824707031E+04, -.1347237915039E+04, .6144038085938E+03},
 {   23812, .7594654296875E+04, -.1345866943359E+04, .5585391235352E+03},
 {   23813, .7584890625000E+04, -.1344291992188E+04, .5654066162109E+03},
 {   23814, .7575126464844E+04, -.1345866943359E+04, .5722734375000E+03},
 {   23815, .7566044921875E+04, -.1350514892578E+04, .5786585693359E+03},
 {   23816, .7569501464844E+04, -.1349549926758E+04, .5244954833984E+03},
 {   23817, .7560548339844E+04, -.1348101928711E+04, .5307927246094E+03},
 {   23818, .7551594238281E+04, -.1349549926758E+04, .5370900268555E+03},
 {   23819, .7543280273438E+04, -.1353816894531E+04, .5429337158203E+03},
 {   23820, .7544312500000E+04, -.1353232910156E+04, .4904447631836E+03},
 {   23821, .7536189941406E+04, -.1351911865234E+04, .4961571960449E+03},
 {   23822, .7528066406250E+04, -.1353232910156E+04, .5018704223633E+03},
 {   23823, .7520485351562E+04, -.1357093872070E+04, .5072029113770E+03},
 {   23824, .7530264648438E+04, -.1350768920898E+04, .4692731323242E+03},
 {   23825, .7521579589844E+04, -.1349371948242E+04, .4753799743652E+03},
 {   23826, .7512895996094E+04, -.1350768920898E+04, .4814876098633E+03},
 {   23827, .7504833984375E+04, -.1354908935547E+04, .4871563110352E+03},
 {   23828, .7516195312500E+04, -.1348330932617E+04, .4481160278320E+03},
 {   23829, .7514240722656E+04, -.1359531860352E+04, .4494897155762E+03},
 {   23830, .7506969238281E+04, -.1346831909180E+04, .4546037597656E+03},
 {   23831, .7506969238281E+04, -.1359531860352E+04, .4546037597656E+03},
 {   23832, .7497742675781E+04, -.1348330932617E+04, .4610911560059E+03},
 {   23833, .7499696777344E+04, -.1359531860352E+04, .4597174682617E+03},
 {   23834, .7489162597656E+04, -.1352724975586E+04, .4671249389648E+03},
 {   23835, .7492424804688E+04, -.1359531860352E+04, .4648312072754E+03},
 {   23836, .7786986328125E+04, -.1364611938477E+04, .8223636474609E+03},
 {   23837, .7774218261719E+04, -.1364611938477E+04, .8346323242188E+03},
 {   23838, .7758355957031E+04, -.1364611938477E+04, .8424981079102E+03},
 {   23839, .7740868164062E+04, -.1364611938477E+04, .8452308349609E+03},
 {   23840, .7514240722656E+04, -.1372231933594E+04, .4494897155762E+03},
 {   23841, .7506969238281E+04, -.1372231933594E+04, .4546037597656E+03},
 {   23842, .7499696777344E+04, -.1372231933594E+04, .4597174682617E+03},
 {   23843, .7492424804688E+04, -.1372231933594E+04, .4648312072754E+03},
 {   23844, .7514240722656E+04, -.1381121826172E+04, .4494897155762E+03},
 {   23845, .7506969238281E+04, -.1381121826172E+04, .4546037597656E+03},
 {   23846, .7499696777344E+04, -.1381121826172E+04, .4597174682617E+03},
 {   23847, .7492424804688E+04, -.1381121826172E+04, .4648312072754E+03},
 {   23848, .7514240722656E+04, -.1390011840820E+04, .4494897155762E+03},
 {   23849, .7506969238281E+04, -.1390011840820E+04, .4546037597656E+03},
 {   23850, .7499696777344E+04, -.1390011840820E+04, .4597174682617E+03},
 {   23851, .7492424804688E+04, -.1390011840820E+04, .4648312072754E+03},
 {   23852, .7514240722656E+04, -.1398901855469E+04, .4494897155762E+03},
 {   23853, .7506969238281E+04, -.1398901855469E+04, .4546037597656E+03},
 {   23854, .7499696777344E+04, -.1398901855469E+04, .4597174682617E+03},
 {   23855, .7492424804688E+04, -.1398901855469E+04, .4648312072754E+03},
 {   23856, .7514240722656E+04, -.1407791870117E+04, .4494897155762E+03},
 {   23857, .7506969238281E+04, -.1407791870117E+04, .4546037597656E+03},
 {   23858, .7499696777344E+04, -.1407791870117E+04, .4597174682617E+03},
 {   23859, .7492424804688E+04, -.1407791870117E+04, .4648312072754E+03},
 {   23860, .7786986328125E+04, -.1415411743164E+04, .8223636474609E+03},
 {   23861, .7774478515625E+04, -.1437712768555E+04, .8119076538086E+03},
 {   23862, .7774218261719E+04, -.1415411743164E+04, .8346323242188E+03},
 {   23863, .7764121093750E+04, -.1438804687500E+04, .8202764892578E+03},
 {   23864, .7758355957031E+04, -.1415411743164E+04, .8424981079102E+03},
 {   23865, .7752747070312E+04, -.1437712768555E+04, .8271905517578E+03},
 {   23866, .7740868164062E+04, -.1415411743164E+04, .8452308349609E+03},
 {   23867, .7740857910156E+04, -.1434537719727E+04, .8323207397461E+03},
 {   23868, .7514240722656E+04, -.1420491821289E+04, .4494897155762E+03},
 {   23869, .7506969238281E+04, -.1420491821289E+04, .4546037597656E+03},
 {   23870, .7499696777344E+04, -.1420491821289E+04, .4597174682617E+03},
 {   23871, .7492424804688E+04, -.1420491821289E+04, .4648312072754E+03},
 {   23872, .7765858886719E+04, -.1447771728516E+04, .7951460571289E+03},
 {   23873, .7753195800781E+04, -.1449701660156E+04, .8047355346680E+03},
 {   23874, .7739887695312E+04, -.1447771728516E+04, .8134098510742E+03},
 {   23875, .7727044433594E+04, -.1442208740234E+04, .8204884033203E+03},
 {   23876, .7754291992188E+04, -.1451301757812E+04, .7775068359375E+03},
 {   23877, .7740743652344E+04, -.1453511718750E+04, .7870332031250E+03},
 {   23878, .7727198242188E+04, -.1451301757812E+04, .7965603027344E+03},
 {   23879, .7714567871094E+04, -.1444850708008E+04, .8054434814453E+03},
 {   23880, .7738331054688E+04, -.1445180786133E+04, .7576799316406E+03},
 {   23881, .7726132812500E+04, -.1447161743164E+04, .7662562866211E+03},
 {   23882, .7713937011719E+04, -.1445180786133E+04, .7748330078125E+03},
 {   23883, .7702572265625E+04, -.1439389770508E+04, .7828253784180E+03},
 {   23884, .7716828613281E+04, -.1447009765625E+04, .7262217407227E+03},
 {   23885, .7704219238281E+04, -.1449066772461E+04, .7350914916992E+03},
 {   23886, .7691604003906E+04, -.1447009765625E+04, .7439599609375E+03},
 {   23887, .7679888671875E+04, -.1441014770508E+04, .7522006835938E+03},
 {   23888, .7695308105469E+04, -.1448863769531E+04, .6947791748047E+03},
 {   23889, .7682299804688E+04, -.1450971679688E+04, .7039257202148E+03},
 {   23890, .7669293457031E+04, -.1448863769531E+04, .7130719604492E+03},
 {   23891, .7657180664062E+04, -.1442665771484E+04, .7215902099609E+03},
 {   23892, .7670139160156E+04, -.1445180786133E+04, .6607133789062E+03},
 {   23893, .7657943359375E+04, -.1447161743164E+04, .6692905883789E+03},
 {   23894, .7645751464844E+04, -.1445180786133E+04, .6778676757812E+03},
 {   23895, .7634384765625E+04, -.1439389770508E+04, .6858596801758E+03},
 {   23896, .7644987304688E+04, -.1441497802734E+04, .6266702270508E+03},
 {   23897, .7633604003906E+04, -.1443351806641E+04, .6346764526367E+03},
 {   23898, .7622219726562E+04, -.1441497802734E+04, .6426837158203E+03},
 {   23899, .7611620605469E+04, -.1436087768555E+04, .6501353149414E+03},
 {   23900, .7619822265625E+04, -.1437814697266E+04, .5926040039062E+03},
 {   23901, .7609246582031E+04, -.1439541748047E+04, .6000415039062E+03},
 {   23902, .7598670410156E+04, -.1437814697266E+04, .6074786376953E+03},
 {   23903, .7588824707031E+04, -.1432785766602E+04, .6144038085938E+03},
 {   23904, .7594654296875E+04, -.1434156738281E+04, .5585391235352E+03},
 {   23905, .7584890625000E+04, -.1435731811523E+04, .5654066162109E+03},
 {   23906, .7575126464844E+04, -.1434156738281E+04, .5722734375000E+03},
 {   23907, .7566044921875E+04, -.1429508789062E+04, .5786585693359E+03},
 {   23908, .7569501464844E+04, -.1430473754883E+04, .5244956054688E+03},
 {   23909, .7560548339844E+04, -.1431921752930E+04, .5307927246094E+03},
 {   23910, .7551594238281E+04, -.1430473754883E+04, .5370899047852E+03},
 {   23911, .7543280273438E+04, -.1426206787109E+04, .5429335937500E+03},
 {   23912, .7544312500000E+04, -.1426790771484E+04, .4904447631836E+03},
 {   23913, .7536189941406E+04, -.1428111816406E+04, .4961571960449E+03},
 {   23914, .7528066406250E+04, -.1426790771484E+04, .5018704223633E+03},
 {   23915, .7520485351562E+04, -.1422929809570E+04, .5072029113770E+03},
 {   23916, .7530264648438E+04, -.1429254760742E+04, .4692731323242E+03},
 {   23917, .7521579589844E+04, -.1430651733398E+04, .4753799743652E+03},
 {   23918, .7512895996094E+04, -.1429254760742E+04, .4814876098633E+03},
 {   23919, .7504833984375E+04, -.1425114746094E+04, .4871563110352E+03},
 {   23920, .7516195312500E+04, -.1431692749023E+04, .4481160278320E+03},
 {   23921, .7506969238281E+04, -.1433191772461E+04, .4546037597656E+03},
 {   23922, .7497742675781E+04, -.1431692749023E+04, .4610911560059E+03},
 {   23923, .7489162597656E+04, -.1427298828125E+04, .4671249389648E+03},
 {   23924, .7713985351562E+04, -.1364611938477E+04, .8387052001953E+03},
 {   23925, .7693145507812E+04, -.1364611938477E+04, .8205072631836E+03},
 {   23926, .7683099609375E+04, -.1364611938477E+04, .7940769042969E+03},
 {   23927, .7665657714844E+04, -.1355746948242E+04, .7622087402344E+03},
 {   23928, .7647041992188E+04, -.1346907958984E+04, .7287211303711E+03},
 {   23929, .7624885742188E+04, -.1349600952148E+04, .6925361328125E+03},
 {   23930, .7602748535156E+04, -.1352292968750E+04, .6563742675781E+03},
 {   23931, .7580593750000E+04, -.1354984863281E+04, .6201899414062E+03},
 {   23932, .7558441406250E+04, -.1357677856445E+04, .5840056762695E+03},
 {   23933, .7536299804688E+04, -.1360369873047E+04, .5478439331055E+03},
 {   23934, .7514145996094E+04, -.1363062866211E+04, .5116595153809E+03},
 {   23935, .7498082519531E+04, -.1361284912109E+04, .4919049682617E+03},
 {   23936, .7481993652344E+04, -.1359480957031E+04, .4721661071777E+03},
 {   23937, .7658424804688E+04, -.1371825927734E+04, .7672943115234E+03},
 {   23938, .7679462402344E+04, -.1376955932617E+04, .7966337890625E+03},
 {   23939, .7655912109375E+04, -.1390011840820E+04, .7690620117188E+03},
 {   23940, .7678239257812E+04, -.1390011840820E+04, .7974963378906E+03},
 {   23941, .7658424804688E+04, -.1408197753906E+04, .7672943115234E+03},
 {   23942, .7679462402344E+04, -.1403067871094E+04, .7966337890625E+03},
 {   23943, .7665657714844E+04, -.1424276733398E+04, .7622087402344E+03},
 {   23944, .7683099609375E+04, -.1415411743164E+04, .7940769042969E+03},
 {   23945, .7636239746094E+04, -.1366694946289E+04, .7363193969727E+03},
 {   23946, .7632437988281E+04, -.1390011840820E+04, .7389927368164E+03},
 {   23947, .7636239746094E+04, -.1413328735352E+04, .7363193969727E+03},
 {   23948, .7647041992188E+04, -.1433115722656E+04, .7287211303711E+03},
 {   23949, .7614750000000E+04, -.1368142944336E+04, .6996672363281E+03},
 {   23950, .7611196777344E+04, -.1390011840820E+04, .7021655883789E+03},
 {   23951, .7614750000000E+04, -.1411880859375E+04, .6996672363281E+03},
 {   23952, .7624885742188E+04, -.1430422729492E+04, .6925361328125E+03},
 {   23953, .7593297363281E+04, -.1369589843750E+04, .6630220336914E+03},
 {   23954, .7589971679688E+04, -.1390011840820E+04, .6653595581055E+03},
 {   23955, .7593297363281E+04, -.1410433837891E+04, .6630220336914E+03},
 {   23956, .7602748535156E+04, -.1427730712891E+04, .6563742675781E+03},
 {   23957, .7571806152344E+04, -.1371063842773E+04, .6263706665039E+03},
 {   23958, .7568731933594E+04, -.1390011840820E+04, .6285329589844E+03},
 {   23959, .7571806152344E+04, -.1408959838867E+04, .6263706665039E+03},
 {   23960, .7580593750000E+04, -.1425038818359E+04, .6201898193359E+03},
 {   23961, .7550339843750E+04, -.1372510864258E+04, .5897045898438E+03},
 {   23962, .7547490722656E+04, -.1390011840820E+04, .5917060546875E+03},
 {   23963, .7550339843750E+04, -.1407512817383E+04, .5897045898438E+03},
 {   23964, .7558441406250E+04, -.1422345703125E+04, .5840056762695E+03},
 {   23965, .7528884277344E+04, -.1373984863281E+04, .5530601196289E+03},
 {   23966, .7526264160156E+04, -.1390011840820E+04, .5549005737305E+03},
 {   23967, .7528884277344E+04, -.1406038818359E+04, .5530601196289E+03},
 {   23968, .7536299804688E+04, -.1419653808594E+04, .5478439331055E+03},
 {   23969, .7507395019531E+04, -.1375431884766E+04, .5164082641602E+03},
 {   23970, .7505024414062E+04, -.1390011840820E+04, .5180739135742E+03},
 {   23971, .7507395019531E+04, -.1404591796875E+04, .5164082641602E+03},
 {   23972, .7514145996094E+04, -.1416960815430E+04, .5116595153809E+03},
 {   23973, .7490872070312E+04, -.1374466918945E+04, .4969754638672E+03},
 {   23974, .7488335449219E+04, -.1390011840820E+04, .4987580261230E+03},
 {   23975, .7490872070312E+04, -.1405556762695E+04, .4969754638672E+03},
 {   23976, .7498082519531E+04, -.1418738769531E+04, .4919049682617E+03},
 {   23977, .7474326660156E+04, -.1373476928711E+04, .4775570983887E+03},
 {   23978, .7471646484375E+04, -.1390011840820E+04, .4794421386719E+03},
 {   23979, .7474326660156E+04, -.1406546752930E+04, .4775570983887E+03},
 {   23980, .7481993652344E+04, -.1420542724609E+04, .4721661071777E+03},
 {   23981, .7713985351562E+04, -.1415411743164E+04, .8387052001953E+03},
 {   23982, .7693145507812E+04, -.1415411743164E+04, .8205072631836E+03},
 {   23983, .7769898925781E+04, -.1364611938477E+04, .7975822753906E+03},
 {   23984, .7762593261719E+04, -.1364611938477E+04, .7871940917969E+03},
 {   23985, .7755287597656E+04, -.1364611938477E+04, .7768053588867E+03},
 {   23986, .7739530273438E+04, -.1364611938477E+04, .7543934936523E+03},
 {   23987, .7762593261719E+04, -.1415411743164E+04, .7871942138672E+03},
 {   23988, .7769898925781E+04, -.1415411743164E+04, .7975822753906E+03},
 {   23989, .7755287597656E+04, -.1415411743164E+04, .7768053588867E+03},
 {   23990, .7739530273438E+04, -.1415411743164E+04, .7543934936523E+03},
 {   23991, .7762628417969E+04, -.1364611938477E+04, .8026958007812E+03},
 {   23992, .7755356445312E+04, -.1364611938477E+04, .8078105468750E+03},
 {   23993, .7748083007812E+04, -.1364611938477E+04, .8129243774414E+03},
 {   23994, .7740810546875E+04, -.1364611938477E+04, .8180383300781E+03},
 {   23995, .7755321777344E+04, -.1364611938477E+04, .7923080444336E+03},
 {   23996, .7748051757812E+04, -.1364611938477E+04, .7974216918945E+03},
 {   23997, .7740779296875E+04, -.1364611938477E+04, .8025353393555E+03},
 {   23998, .7733509277344E+04, -.1364611938477E+04, .8076496582031E+03},
 {   23999, .7748017578125E+04, -.1364611938477E+04, .7819191894531E+03},
 {   24000, .7740743652344E+04, -.1364611938477E+04, .7870332031250E+03},
 {   24001, .7733473632812E+04, -.1364611938477E+04, .7921473388672E+03},
 {   24002, .7726203613281E+04, -.1364611938477E+04, .7972613525391E+03},
 {   24003, .7732256347656E+04, -.1364611938477E+04, .7595076904297E+03},
 {   24004, .7724982421875E+04, -.1364611938477E+04, .7646214599609E+03},
 {   24005, .7717713378906E+04, -.1364611938477E+04, .7697359008789E+03},
 {   24006, .7710441894531E+04, -.1364611938477E+04, .7748488769531E+03},
 {   24007, .7762628417969E+04, -.1415411743164E+04, .8026958007812E+03},
 {   24008, .7755356445312E+04, -.1415411743164E+04, .8078105468750E+03},
 {   24009, .7748083007812E+04, -.1415411743164E+04, .8129243774414E+03},
 {   24010, .7740810546875E+04, -.1415411743164E+04, .8180383300781E+03},
 {   24011, .7755321777344E+04, -.1415411743164E+04, .7923080444336E+03},
 {   24012, .7748051269531E+04, -.1415411743164E+04, .7974216918945E+03},
 {   24013, .7740778808594E+04, -.1415411743164E+04, .8025352783203E+03},
 {   24014, .7733509277344E+04, -.1415411743164E+04, .8076495971680E+03},
 {   24015, .7748017578125E+04, -.1415411743164E+04, .7819191894531E+03},
 {   24016, .7740743652344E+04, -.1415411743164E+04, .7870332031250E+03},
 {   24017, .7733473632812E+04, -.1415411743164E+04, .7921473388672E+03},
 {   24018, .7726203613281E+04, -.1415411743164E+04, .7972613525391E+03},
 {   24019, .7732256347656E+04, -.1415411743164E+04, .7595076904297E+03},
 {   24020, .7724982421875E+04, -.1415411743164E+04, .7646214599609E+03},
 {   24021, .7717713378906E+04, -.1415411743164E+04, .7697359008789E+03},
 {   24022, .7710441894531E+04, -.1415411743164E+04, .7748488769531E+03},
 {   24023, .7542116699219E+04, -.1374339843750E+04, .4182130737305E+03},
 {   24024, .7539391601562E+04, -.1373603881836E+04, .4106271667480E+03},
 {   24025, .7531087402344E+04, -.1359810913086E+04, .4143222351074E+03},
 {   24026, .7533841796875E+04, -.1360801879883E+04, .4199608459473E+03},
 {   24027, .7545011718750E+04, -.1390011840820E+04, .4176051330566E+03},
 {   24028, .7542335937500E+04, -.1390011840820E+04, .4092991943359E+03},
 {   24029, .7542116699219E+04, -.1405683837891E+04, .4182130737305E+03},
 {   24030, .7539391601562E+04, -.1406419799805E+04, .4106271667480E+03},
 {   24031, .7533841796875E+04, -.1419221801758E+04, .4199608459473E+03},
 {   24032, .7531087402344E+04, -.1420212768555E+04, .4143222351074E+03},
 {   24033, .7535386718750E+04, -.1373476928711E+04, .4035673522949E+03},
 {   24034, .7527718750000E+04, -.1359480957031E+04, .4089577941895E+03},
 {   24035, .7538065917969E+04, -.1390011840820E+04, .4016818237305E+03},
 {   24036, .7535386718750E+04, -.1406546875000E+04, .4035673522949E+03},
 {   24037, .7527718750000E+04, -.1420542724609E+04, .4089577941895E+03},
 {   24038, .7530099609375E+04, -.1373603881836E+04, .3974136657715E+03},
 {   24039, .7523810546875E+04, -.1359810913086E+04, .4039749755859E+03},
 {   24040, .7532340820312E+04, -.1390011840820E+04, .3950880432129E+03},
 {   24041, .7530099609375E+04, -.1406419799805E+04, .3974136657715E+03},
 {   24042, .7523810546875E+04, -.1420212768555E+04, .4039749755859E+03},
 {   24043, .7523878417969E+04, -.1374339843750E+04, .3922833251953E+03},
 {   24044, .7519435058594E+04, -.1360801879883E+04, .3994758300781E+03},
 {   24045, .7525430664062E+04, -.1390011840820E+04, .3897639465332E+03},
 {   24046, .7523878417969E+04, -.1405683837891E+04, .3922833251953E+03},
 {   24047, .7519435058594E+04, -.1419221801758E+04, .3994758300781E+03},
 {   24048, .7520731933594E+04, -.1350844970703E+04, .4237474670410E+03},
 {   24049, .7524684082031E+04, -.1353029907227E+04, .4304709777832E+03},
 {   24050, .7507743652344E+04, -.1347720947266E+04, .4336275329590E+03},
 {   24051, .7512539062500E+04, -.1350311889648E+04, .4404415893555E+03},
 {   24052, .7516271972656E+04, -.1350108886719E+04, .4170090942383E+03},
 {   24053, .7502745117188E+04, -.1346831909180E+04, .4265203552246E+03},
 {   24054, .7511440429688E+04, -.1350844970703E+04, .4105339355469E+03},
 {   24055, .7497748046875E+04, -.1347720947266E+04, .4194152221680E+03},
 {   24056, .7506446777344E+04, -.1353029907227E+04, .4045408630371E+03},
 {   24057, .7492957519531E+04, -.1350311889648E+04, .4126000671387E+03},
 {   24058, .7537318847656E+04, -.1367024902344E+04, .4384147033691E+03},
 {   24059, .7523478027344E+04, -.1366237915039E+04, .4560034484863E+03},
 {   24060, .7543053222656E+04, -.1390011840820E+04, .4420218505859E+03},
 {   24061, .7527586425781E+04, -.1390011840820E+04, .4618415832520E+03},
 {   24062, .7537318847656E+04, -.1412998779297E+04, .4384147033691E+03},
 {   24063, .7523478027344E+04, -.1413785766602E+04, .4560034484863E+03},
 {   24064, .7524684082031E+04, -.1426993774414E+04, .4304709777832E+03},
 {   24065, .7512539062500E+04, -.1429711791992E+04, .4404415893555E+03},
 {   24066, .7503243164062E+04, -.1367024902344E+04, .3899635009766E+03},
 {   24067, .7482012207031E+04, -.1366237915039E+04, .3970385742188E+03},
 {   24068, .7501791015625E+04, -.1390011840820E+04, .3833474426270E+03},
 {   24069, .7477907226562E+04, -.1390011840820E+04, .3912007141113E+03},
 {   24070, .7503243164062E+04, -.1412998779297E+04, .3899636840820E+03},
 {   24071, .7482012207031E+04, -.1413785766602E+04, .3970385742188E+03},
 {   24072, .7506446777344E+04, -.1426993774414E+04, .4045408630371E+03},
 {   24073, .7492957519531E+04, -.1429711791992E+04, .4126000671387E+03},
 {   24074, .7520731933594E+04, -.1429178833008E+04, .4237474670410E+03},
 {   24075, .7507743652344E+04, -.1432302734375E+04, .4336276550293E+03},
 {   24076, .7516271972656E+04, -.1429914794922E+04, .4170090942383E+03},
 {   24077, .7502745117188E+04, -.1433191772461E+04, .4265203552246E+03},
 {   24078, .7511440429688E+04, -.1429178833008E+04, .4105339355469E+03},
 {   24079, .7497748046875E+04, -.1432302734375E+04, .4194152221680E+03},
 {   24080, .7474690917969E+04, -.1354680908203E+04, .4528630676270E+03},
 {   24081, .7477613281250E+04, -.1355315917969E+04, .4570180969238E+03},
 {   24082, .7471742187500E+04, -.1354451904297E+04, .4486658020020E+03},
 {   24083, .7467561523438E+04, -.1355137939453E+04, .4427242736816E+03},
 {   24084, .7463559570312E+04, -.1357321899414E+04, .4370313415527E+03},
 {   24085, .7492022460938E+04, -.1365957885742E+04, .4775049438477E+03},
 {   24086, .7499340820312E+04, -.1390011840820E+04, .4879140625000E+03},
 {   24087, .7492022460938E+04, -.1414065795898E+04, .4775049438477E+03},
 {   24088, .7477613281250E+04, -.1424707763672E+04, .4570180969238E+03},
 {   24089, .7454515625000E+04, -.1370402832031E+04, .4241705627441E+03},
 {   24090, .7451125976562E+04, -.1390011840820E+04, .4193502502441E+03},
 {   24091, .7454515625000E+04, -.1409620849609E+04, .4241705627441E+03},
 {   24092, .7463559570312E+04, -.1422701782227E+04, .4370313415527E+03},
 {   24093, .7474690917969E+04, -.1425342773438E+04, .4528630676270E+03},
 {   24094, .7471742187500E+04, -.1425571777344E+04, .4486658020020E+03},
 {   24095, .7467561523438E+04, -.1424885742188E+04, .4427242736816E+03},
 {   24096, .7448722167969E+04, -.1353994873047E+04, .4747902221680E+03},
 {   24097, .7451877929688E+04, -.1359988891602E+04, .4792792968750E+03},
 {   24098, .7422742675781E+04, -.1353283935547E+04, .4967547302246E+03},
 {   24099, .7426105957031E+04, -.1364662841797E+04, .5015337219238E+03},
 {   24100, .7396770507812E+04, -.1352597900391E+04, .5186820068359E+03},
 {   24101, .7400351074219E+04, -.1369361938477E+04, .5237731933594E+03},
 {   24102, .7444030273438E+04, -.1350235961914E+04, .4681214904785E+03},
 {   24103, .7416300781250E+04, -.1345993896484E+04, .4875927124023E+03},
 {   24104, .7388605957031E+04, -.1341751953125E+04, .5070693969727E+03},
 {   24105, .7438421875000E+04, -.1351733886719E+04, .4601431274414E+03},
 {   24106, .7409273925781E+04, -.1348330932617E+04, .4775985717773E+03},
 {   24107, .7380129394531E+04, -.1344926879883E+04, .4950183410645E+03},
 {   24108, .7433409179688E+04, -.1356102905273E+04, .4530165710449E+03},
 {   24109, .7403238769531E+04, -.1354908935547E+04, .4690177612305E+03},
 {   24110, .7373089355469E+04, -.1353689941406E+04, .4850039062500E+03},
 {   24111, .7461797851562E+04, -.1370504882812E+04, .4933863830566E+03},
 {   24112, .7431538574219E+04, -.1375076904297E+04, .5092626647949E+03},
 {   24113, .7401315429688E+04, -.1379622802734E+04, .5251448364258E+03},
 {   24114, .7466781250000E+04, -.1390011840820E+04, .5004715270996E+03},
 {   24115, .7434185058594E+04, -.1390011840820E+04, .5130234375000E+03},
 {   24116, .7401625000000E+04, -.1390011840820E+04, .5255809936523E+03},
 {   24117, .7461797851562E+04, -.1409518798828E+04, .4933863830566E+03},
 {   24118, .7431538574219E+04, -.1404946777344E+04, .5092626647949E+03},
 {   24119, .7401315429688E+04, -.1400400878906E+04, .5251448364258E+03},
 {   24120, .7451877929688E+04, -.1420034790039E+04, .4792792968750E+03},
 {   24121, .7426105957031E+04, -.1415360839844E+04, .5015337219238E+03},
 {   24122, .7400351074219E+04, -.1410661743164E+04, .5237731933594E+03},
 {   24123, .7425374023438E+04, -.1370453857422E+04, .4415890808105E+03},
 {   24124, .7396223632812E+04, -.1370478881836E+04, .4590449523926E+03},
 {   24125, .7367081542969E+04, -.1370529907227E+04, .4764638366699E+03},
 {   24126, .7422450683594E+04, -.1390011840820E+04, .4374338989258E+03},
 {   24127, .7393770996094E+04, -.1390011840820E+04, .4555544128418E+03},
 {   24128, .7365095214844E+04, -.1390011840820E+04, .4736381835938E+03},
 {   24129, .7425374023438E+04, -.1409569824219E+04, .4415890808105E+03},
 {   24130, .7396223632812E+04, -.1409544799805E+04, .4590449523926E+03},
 {   24131, .7367081542969E+04, -.1409493774414E+04, .4764638366699E+03},
 {   24132, .7433409179688E+04, -.1423920776367E+04, .4530165710449E+03},
 {   24133, .7403238769531E+04, -.1425114746094E+04, .4690177612305E+03},
 {   24134, .7373089355469E+04, -.1426333740234E+04, .4850039062500E+03},
 {   24135, .7448722167969E+04, -.1426028808594E+04, .4747902221680E+03},
 {   24136, .7422742675781E+04, -.1426714843750E+04, .4967547302246E+03},
 {   24137, .7396769531250E+04, -.1427425781250E+04, .5186822509766E+03},
 {   24138, .7444030273438E+04, -.1429787719727E+04, .4681214904785E+03},
 {   24139, .7416318359375E+04, -.1434029785156E+04, .4876128540039E+03},
 {   24140, .7388619140625E+04, -.1438271728516E+04, .5070895385742E+03},
 {   24141, .7438421875000E+04, -.1428289794922E+04, .4601431274414E+03},
 {   24142, .7409273925781E+04, -.1431692749023E+04, .4775985717773E+03},
 {   24143, .7380146972656E+04, -.1435096801758E+04, .4950384216309E+03},
 {   24144, .7356506835938E+04, -.1355162841797E+04, .5276553955078E+03},
 {   24145, .7361441406250E+04, -.1363239868164E+04, .5346779785156E+03},
 {   24146, .7350003417969E+04, -.1351911865234E+04, .5184100952148E+03},
 {   24147, .7343500488281E+04, -.1355162841797E+04, .5091641235352E+03},
 {   24148, .7338562500000E+04, -.1363239868164E+04, .5021417236328E+03},
 {   24149, .7364950683594E+04, -.1375990844727E+04, .5396645507812E+03},
 {   24150, .7366076171875E+04, -.1390011840820E+04, .5412645263672E+03},
 {   24151, .7364950683594E+04, -.1404032836914E+04, .5396645507812E+03},
 {   24152, .7361441406250E+04, -.1416783813477E+04, .5346779785156E+03},
 {   24153, .7335055664062E+04, -.1375990844727E+04, .4971550903320E+03},
 {   24154, .7333931152344E+04, -.1390011840820E+04, .4955548400879E+03},
 {   24155, .7335055664062E+04, -.1404032836914E+04, .4971550903320E+03},
 {   24156, .7338562500000E+04, -.1416783813477E+04, .5021417236328E+03},
 {   24157, .7356506835938E+04, -.1424860839844E+04, .5276553955078E+03},
 {   24158, .7350003417969E+04, -.1428111816406E+04, .5184100952148E+03},
 {   24159, .7343500488281E+04, -.1424860839844E+04, .5091641235352E+03},
 {   24160, .7345333984375E+04, -.1358667846680E+04, .5335858154297E+03},
 {   24161, .7351212402344E+04, -.1364306884766E+04, .5389848632812E+03},
 {   24162, .7335859375000E+04, -.1367176879883E+04, .5357443847656E+03},
 {   24163, .7340727539062E+04, -.1368091918945E+04, .5393088989258E+03},
 {   24164, .7339240234375E+04, -.1356407958984E+04, .5259779052734E+03},
 {   24165, .7331801757812E+04, -.1366948852539E+04, .5312091674805E+03},
 {   24166, .7334143066406E+04, -.1358667846680E+04, .5176706542969E+03},
 {   24167, .7328905273438E+04, -.1367176879883E+04, .5258552856445E+03},
 {   24168, .7331049804688E+04, -.1364306884766E+04, .5103125915527E+03},
 {   24169, .7327199218750E+04, -.1368091918945E+04, .5200701293945E+03},
 {   24170, .7352291503906E+04, -.1376371826172E+04, .5444057617188E+03},
 {   24171, .7339187500000E+04, -.1378759887695E+04, .5428140869141E+03},
 {   24172, .7352647460938E+04, -.1389986816406E+04, .5462357177734E+03},
 {   24173, .7338679199219E+04, -.1390011840820E+04, .5439473876953E+03},
 {   24174, .7352291503906E+04, -.1403651855469E+04, .5444057617188E+03},
 {   24175, .7339187500000E+04, -.1401263793945E+04, .5428140869141E+03},
 {   24176, .7351213378906E+04, -.1415716796875E+04, .5389849853516E+03},
 {   24177, .7340727539062E+04, -.1411931762695E+04, .5393088989258E+03},
 {   24178, .7326314941406E+04, -.1376371826172E+04, .5074643859863E+03},
 {   24179, .7323378417969E+04, -.1378759887695E+04, .5203339233398E+03},
 {   24180, .7324710449219E+04, -.1389986816406E+04, .5065103759766E+03},
 {   24181, .7322138183594E+04, -.1390011840820E+04, .5204279785156E+03},
 {   24182, .7326314941406E+04, -.1403651855469E+04, .5074643859863E+03},
 {   24183, .7323378417969E+04, -.1401263793945E+04, .5203339233398E+03},
 {   24184, .7331049804688E+04, -.1415716796875E+04, .5103125915527E+03},
 {   24185, .7327199218750E+04, -.1411931762695E+04, .5200701293945E+03},
 {   24186, .7345333984375E+04, -.1421355834961E+04, .5335858154297E+03},
 {   24187, .7335859375000E+04, -.1412846801758E+04, .5357443847656E+03},
 {   24188, .7339240234375E+04, -.1423615722656E+04, .5259779052734E+03},
 {   24189, .7331801757812E+04, -.1413074829102E+04, .5312091674805E+03},
 {   24190, .7334143066406E+04, -.1421355834961E+04, .5176706542969E+03},
 {   24191, .7328905273438E+04, -.1412846801758E+04, .5258552856445E+03},
 {   24192, .7333150390625E+04, -.1378200805664E+04, .5389552612305E+03},
 {   24193, .7332273437500E+04, -.1390011840820E+04, .5400064697266E+03},
 {   24194, .7333150390625E+04, -.1401822875977E+04, .5389552612305E+03},
 {   24195, .7328290527344E+04, -.1378124877930E+04, .5336785888672E+03},
 {   24196, .7327148437500E+04, -.1390011840820E+04, .5344820556641E+03},
 {   24197, .7328290527344E+04, -.1401898803711E+04, .5336785888672E+03},
 {   24198, .7324966308594E+04, -.1378200805664E+04, .5273203125000E+03},
 {   24199, .7323684570312E+04, -.1390011840820E+04, .5277888793945E+03},
 {   24200, .7324966308594E+04, -.1401822875977E+04, .5273203125000E+03},
 {   24201, .8127561523438E+04, -.1366324462891E+04, .7284619750977E+03},
 {   24202, .8127561523438E+04, -.1206313232422E+04, .7284619750977E+03},
 {   24203, .8191705566406E+04, -.1366324462891E+04, .7582155761719E+03},
 {   24204, .8191705566406E+04, -.1206313232422E+04, .7582155761719E+03},
 {   24205, .8125099609375E+04, -.1363167114258E+04, .7182486572266E+03},
 {   24206, .8181508789062E+04, -.1366324462891E+04, .7607738647461E+03},
 {   24207, .8125099609375E+04, -.1203155517578E+04, .7182486572266E+03},
 {   24208, .8181508789062E+04, -.1206312744141E+04, .7607738647461E+03},
 {   24209, .7870151367188E+04, -.1182120239258E+04, .7649862670898E+03},
 {   24210, .7796588867188E+04, -.1182120239258E+04, .7848575439453E+03},
 {   24211, .7870151367188E+04, -.1390011840820E+04, .7649863891602E+03},
 {   24212, .7796588867188E+04, -.1390011840820E+04, .7848575439453E+03},
 {   24213, .7704219238281E+04, -.1182120239258E+04, .7350914916992E+03},
 {   24214, .7536190917969E+04, -.1182120239258E+04, .4961571350098E+03},
 {   24215, .7704219238281E+04, -.1390011840820E+04, .7350914916992E+03},
 {   24216, .7536189941406E+04, -.1390011840820E+04, .4961571960449E+03},
 {   27000, .8226993164062E+04, -.1336620727539E+04, .7976733398438E+03},
 {   27001, .8218631835938E+04, -.1337331909180E+04, .7993851928711E+03},
 {   27002, .8226340820312E+04, -.1341218017578E+04, .7980206909180E+03},
 {   27003, .8235999023438E+04, -.1337331909180E+04, .8138170166016E+03},
 {   27004, .8226375000000E+04, -.1337331909180E+04, .8166793823242E+03},
 {   27005, .8235304687500E+04, -.1342030883789E+04, .8141282348633E+03},
 {   27006, .8296393554688E+04, -.1341929199219E+04, .9510841674805E+03},
 {   27007, .8288351562500E+04, -.1337331787109E+04, .9551288452148E+03},
 {   27008, .8294467773438E+04, -.1345815429688E+04, .9519457397461E+03},
 {   27009, .8301472656250E+04, -.1340786254883E+04, .9678442382812E+03},
 {   27010, .8296103515625E+04, -.1337331787109E+04, .9724464721680E+03},
 {   27011, .8300013671875E+04, -.1343732666016E+04, .9684979248047E+03},
 {   27012, .8261293945312E+04, -.1343707275391E+04, .8641632080078E+03},
 {   27013, .8249620117188E+04, -.1337331909180E+04, .8686099243164E+03},
 {   27014, .8258625976562E+04, -.1349092041016E+04, .8653566894531E+03},
 {   27015, .8269662109375E+04, -.1344037475586E+04, .8811774902344E+03},
 {   27016, .8257364257812E+04, -.1337331909180E+04, .8859042968750E+03},
 {   27017, .8266831054688E+04, -.1349752441406E+04, .8824439086914E+03},
 {   27018, .8252650390625E+04, -.1343148437500E+04, .8474666748047E+03},
 {   27019, .8241867187500E+04, -.1337331909180E+04, .8512919311523E+03},
 {   27020, .8250214843750E+04, -.1348101440430E+04, .8485565185547E+03},
 {   27021, .8224715820312E+04, -.1344494628906E+04, .7987474365234E+03},
 {   27022, .8233333984375E+04, -.1345993286133E+04, .8150104370117E+03},
 {   27023, .8288285156250E+04, -.1347491821289E+04, .9348429565430E+03},
 {   27024, .8280608398438E+04, -.1337331787109E+04, .9378344116211E+03},
 {   27025, .8284833007812E+04, -.1350590698242E+04, .9363892211914E+03},
 {   27026, .8291593750000E+04, -.1348431640625E+04, .9532325439453E+03},
 {   27027, .8246553710938E+04, -.1351378051758E+04, .8501962890625E+03},
 {   27028, .8254616210938E+04, -.1352698852539E+04, .8671520385742E+03},
 {   27029, .8241808593750E+04, -.1347136230469E+04, .8316995239258E+03},
 {   27030, .8234552734375E+04, -.1337331909180E+04, .8349478759766E+03},
 {   27031, .8238468750000E+04, -.1350133422852E+04, .8331937866211E+03},
 {   27032, .8222283203125E+04, -.1346704467773E+04, .7998370361328E+03},
 {   27033, .8230389648438E+04, -.1348660278320E+04, .8163284301758E+03},
 {   27034, .8280775390625E+04, -.1351682861328E+04, .9382050781250E+03},
 {   27035, .8288185546875E+04, -.1349346069336E+04, .9547577514648E+03},
 {   27036, .8277778320312E+04, -.1352267089844E+04, .9192036132812E+03},
 {   27037, .8272856445312E+04, -.1337331787109E+04, .9205166625977E+03},
 {   27038, .8273188476562E+04, -.1353511596680E+04, .9212584838867E+03},
 {   27039, .8270339843750E+04, -.1353283081055E+04, .9017739868164E+03},
 {   27040, .8265114257812E+04, -.1337331787109E+04, .9032221679688E+03},
 {   27041, .8265448242188E+04, -.1354603881836E+04, .9039638061523E+03},
 {   27042, .8242241210938E+04, -.1352546508789E+04, .8521263427734E+03},
 {   27043, .8249911132812E+04, -.1353968872070E+04, .8692589721680E+03},
 {   27044, .8219406250000E+04, -.1347466430664E+04, .8011240844727E+03},
 {   27045, .8226912109375E+04, -.1349574707031E+04, .8178853759766E+03},
 {   27046, .8284777343750E+04, -.1348431640625E+04, .9562835693359E+03},
 {   27047, .8281879882812E+04, -.1345815429688E+04, .9575806274414E+03},
 {   27048, .8292709960938E+04, -.1345688476562E+04, .9717672729492E+03},
 {   27049, .8290530273438E+04, -.1343732666016E+04, .9727429199219E+03},
 {   27050, .8268596679688E+04, -.1352267089844E+04, .9233133544922E+03},
 {   27051, .8264705078125E+04, -.1348761840820E+04, .9250568237305E+03},
 {   27052, .8276693359375E+04, -.1350590698242E+04, .9400320434570E+03},
 {   27053, .8273239257812E+04, -.1347491821289E+04, .9415782470703E+03},
 {   27054, .8260531250000E+04, -.1353283081055E+04, .9061640625000E+03},
 {   27055, .8256380859375E+04, -.1349549194336E+04, .9080220336914E+03},
 {   27056, .8252667968750E+04, -.1353537109375E+04, .8887849121094E+03},
 {   27057, .8248448242188E+04, -.1349752441406E+04, .8906737060547E+03},
 {   27058, .8245180664062E+04, -.1352698852539E+04, .8713761596680E+03},
 {   27059, .8241169921875E+04, -.1349092041016E+04, .8731716308594E+03},
 {   27060, .8230610351562E+04, -.1350133422852E+04, .8367122802734E+03},
 {   27061, .8227273437500E+04, -.1347136230469E+04, .8382065429688E+03},
 {   27062, .8237905273438E+04, -.1351378051758E+04, .8540671386719E+03},
 {   27063, .8234244140625E+04, -.1348101440430E+04, .8557070922852E+03},
 {   27064, .8223435546875E+04, -.1348660278320E+04, .8194417724609E+03},
 {   27065, .8220466796875E+04, -.1345993286133E+04, .8207703247070E+03},
 {   27066, .8279955078125E+04, -.1341929199219E+04, .9584421386719E+03},
 {   27067, .8289070312500E+04, -.1340786254883E+04, .9733968505859E+03},
 {   27068, .8262106445312E+04, -.1343529418945E+04, .9262193603516E+03},
 {   27069, .8270921875000E+04, -.1342818237305E+04, .9426162109375E+03},
 {   27070, .8253599609375E+04, -.1343935791016E+04, .9092671508789E+03},
 {   27071, .8238504882812E+04, -.1343707275391E+04, .8743649902344E+03},
 {   27072, .8245620117188E+04, -.1344037475586E+04, .8919396972656E+03},
 {   27073, .8218496093750E+04, -.1342030883789E+04, .8216526489258E+03},
 {   27074, .8225045898438E+04, -.1342640502930E+04, .8392028198242E+03},
 {   27075, .8270110351562E+04, -.1337331787109E+04, .9429790649414E+03},
 {   27076, .8279260742188E+04, -.1337331787109E+04, .9587534790039E+03},
 {   27077, .8261179687500E+04, -.1337331787109E+04, .9266342773438E+03},
 {   27078, .8252626953125E+04, -.1337331787109E+04, .9097031250000E+03},
 {   27079, .8244624023438E+04, -.1337331909180E+04, .8923859863281E+03},
 {   27080, .8224255859375E+04, -.1337331909180E+04, .8395559082031E+03},
 {   27081, .8231809570312E+04, -.1343148437500E+04, .8567966918945E+03},
 {   27082, .8230948242188E+04, -.1337331909180E+04, .8571810913086E+03},
 {   27083, .8319657226562E+04, -.1337331787109E+04, .9531367187500E+03},
 {   27084, .8313778320312E+04, -.1337331787109E+04, .9579672241211E+03},
 {   27085, .8314540039062E+04, -.1337331787109E+04, .9420141601562E+03},
 {   27086, .8323265625000E+04, -.1337331787109E+04, .9376350708008E+03},
 {   27087, .8317617187500E+04, -.1351174804688E+04, .9540495605469E+03},
 {   27088, .8312247070312E+04, -.1347720458984E+04, .9586520385742E+03},
 {   27089, .8312499023438E+04, -.1351124023438E+04, .9429274902344E+03},
 {   27090, .8320552734375E+04, -.1355721435547E+04, .9388489990234E+03},
 {   27091, .8307863281250E+04, -.1337331787109E+04, .9627850341797E+03},
 {   27092, .8305812500000E+04, -.1337331787109E+04, .9463936767578E+03},
 {   27093, .8306842773438E+04, -.1344265991211E+04, .9632418212891E+03},
 {   27094, .8304443359375E+04, -.1346526611328E+04, .9470060424805E+03},
 {   27095, .8301983398438E+04, -.1337331787109E+04, .9676157836914E+03},
 {   27096, .8297085937500E+04, -.1337331787109E+04, .9507729492188E+03},
 {   27097, .8331836914062E+04, -.1337331787109E+04, .9336582031250E+03},
 {   27098, .8323827148438E+04, -.1337331787109E+04, .9202406005859E+03},
 {   27099, .8334635742188E+04, -.1337331787109E+04, .9158478393555E+03},
 {   27100, .8328617187500E+04, -.1360318847656E+04, .9347939453125E+03},
 {   27101, .8320581054688E+04, -.1359302856445E+04, .9216936035156E+03},
 {   27102, .8330577148438E+04, -.1364789306641E+04, .9176640014648E+03},
 {   27103, .8313020507812E+04, -.1337331787109E+04, .9246332397461E+03},
 {   27104, .8310584960938E+04, -.1353816406250E+04, .9257229614258E+03},
 {   27105, .8302213867188E+04, -.1337331787109E+04, .9290258789062E+03},
 {   27106, .8300589843750E+04, -.1348304687500E+04, .9297525024414E+03},
 {   27107, .8291416992188E+04, -.1337331787109E+04, .9334415893555E+03},
 {   27108, .8290605468750E+04, -.1342818237305E+04, .9338049926758E+03},
 {   27109, .8322119140625E+04, -.1337331787109E+04, .9019421386719E+03},
 {   27110, .8334459960938E+04, -.1337331787109E+04, .8973081665039E+03},
 {   27111, .8318479492188E+04, -.1362096801758E+04, .9035715332031E+03},
 {   27112, .8329893554688E+04, -.1368294433594E+04, .8993526000977E+03},
 {   27113, .8309812500000E+04, -.1337331787109E+04, .9065890502930E+03},
 {   27114, .8307077148438E+04, -.1355899291992E+04, .9078137207031E+03},
 {   27115, .8297481445312E+04, -.1337331787109E+04, .9112462158203E+03},
 {   27116, .8295674804688E+04, -.1349701660156E+04, .9120556030273E+03},
 {   27117, .8285173828125E+04, -.1337331787109E+04, .9158930664062E+03},
 {   27118, .8284267578125E+04, -.1343529418945E+04, .9162978515625E+03},
 {   27119, .8317699218750E+04, -.1337331787109E+04, .8833001098633E+03},
 {   27120, .8330861328125E+04, -.1337331787109E+04, .8783261108398E+03},
 {   27121, .8313802734375E+04, -.1363773193359E+04, .8850435791016E+03},
 {   27122, .8325967773438E+04, -.1370402587891E+04, .8805161132812E+03},
 {   27123, .8304546875000E+04, -.1337331787109E+04, .8882976684570E+03},
 {   27124, .8301624023438E+04, -.1357169189453E+04, .8896051635742E+03},
 {   27125, .8291406250000E+04, -.1337331787109E+04, .8932611083984E+03},
 {   27126, .8289459960938E+04, -.1350565307617E+04, .8941326904297E+03},
 {   27127, .8278243164062E+04, -.1337331787109E+04, .8982354736328E+03},
 {   27128, .8277269531250E+04, -.1343935791016E+04, .8986711425781E+03},
 {   27129, .8310549804688E+04, -.1337331909180E+04, .8651554565430E+03},
 {   27130, .8323682617188E+04, -.1338093872070E+04, .8600550537109E+03},
 {   27131, .8306584960938E+04, -.1364205078125E+04, .8669301147461E+03},
 {   27132, .8318907226562E+04, -.1370910644531E+04, .8621929931641E+03},
 {   27133, .8297263671875E+04, -.1337331909180E+04, .8703517456055E+03},
 {   27134, .8294272460938E+04, -.1357473999023E+04, .8716906738281E+03},
 {   27135, .8283943359375E+04, -.1337331909180E+04, .8755354003906E+03},
 {   27136, .8281973632812E+04, -.1350768432617E+04, .8764172973633E+03},
 {   27137, .8270659179688E+04, -.1337331909180E+04, .8807312622070E+03},
 {   27138, .8300107421875E+04, -.1337331909180E+04, .8490981445312E+03},
 {   27139, .8312752929688E+04, -.1337331909180E+04, .8442156982422E+03},
 {   27140, .8296350585938E+04, -.1362808105469E+04, .8507796630859E+03},
 {   27141, .8306543945312E+04, -.1368523071289E+04, .8460766601562E+03},
 {   27142, .8287481445312E+04, -.1337331909180E+04, .8539703369141E+03},
 {   27143, .8284675781250E+04, -.1356432617188E+04, .8552261962891E+03},
 {   27144, .8274857421875E+04, -.1337331909180E+04, .8588424682617E+03},
 {   27145, .8272980468750E+04, -.1350057250977E+04, .8596832885742E+03},
 {   27146, .8262245117188E+04, -.1337331909180E+04, .8637375488281E+03},
 {   27147, .8288465820312E+04, -.1337331909180E+04, .8344114990234E+03},
 {   27148, .8301894531250E+04, -.1336366699219E+04, .8301533813477E+03},
 {   27149, .8285033203125E+04, -.1360623657227E+04, .8359473876953E+03},
 {   27150, .8295840820312E+04, -.1366440307617E+04, .8321116943359E+03},
 {   27151, .8276834960938E+04, -.1337331909180E+04, .8386437377930E+03},
 {   27152, .8274264648438E+04, -.1354807006836E+04, .8397954101562E+03},
 {   27153, .8265170898438E+04, -.1337331909180E+04, .8428633422852E+03},
 {   27154, .8263456054688E+04, -.1348965087891E+04, .8436311645508E+03},
 {   27155, .8253508789062E+04, -.1337331909180E+04, .8470826416016E+03},
 {   27156, .8276917968750E+04, -.1337331909180E+04, .8194324951172E+03},
 {   27157, .8287624023438E+04, -.1337331909180E+04, .8158088378906E+03},
 {   27158, .8273788085938E+04, -.1358540893555E+04, .8208335571289E+03},
 {   27159, .8283729492188E+04, -.1363849487305E+04, .8175523681641E+03},
 {   27160, .8266223632812E+04, -.1337331909180E+04, .8230795288086E+03},
 {   27161, .8263880859375E+04, -.1353232299805E+04, .8241277465820E+03},
 {   27162, .8255527343750E+04, -.1337331909180E+04, .8267264404297E+03},
 {   27163, .8253951171875E+04, -.1347923706055E+04, .8274321289062E+03},
 {   27164, .8244819335938E+04, -.1337331909180E+04, .8303505859375E+03},
 {   27165, .8244032226562E+04, -.1342640502930E+04, .8307033081055E+03},
 {   27166, .8264861328125E+04, -.1337331909180E+04, .8051825561523E+03},
 {   27167, .8274489257812E+04, -.1337331909180E+04, .8023200073242E+03},
 {   27168, .8262078125000E+04, -.1356102416992E+04, .8064279174805E+03},
 {   27169, .8271011718750E+04, -.1360776000977E+04, .8038763427734E+03},
 {   27170, .8255244140625E+04, -.1337331909180E+04, .8080684814453E+03},
 {   27171, .8253158203125E+04, -.1351403442383E+04, .8090021972656E+03},
 {   27172, .8245617187500E+04, -.1337331909180E+04, .8109310302734E+03},
 {   27173, .8244226562500E+04, -.1346704467773E+04, .8115536499023E+03},
 {   27174, .8252373046875E+04, -.1337331909180E+04, .7928238525391E+03},
 {   27175, .8260131835938E+04, -.1337331909180E+04, .7913255004883E+03},
 {   27176, .8249536132812E+04, -.1352851318359E+04, .7939534301758E+03},
 {   27177, .8257257812500E+04, -.1356737426758E+04, .7926122436523E+03},
 {   27178, .8243520507812E+04, -.1337331909180E+04, .7945599975586E+03},
 {   27179, .8241803710938E+04, -.1348965087891E+04, .7953282470703E+03},
 {   27180, .8235220703125E+04, -.1337331909180E+04, .7961607666016E+03},
 {   27181, .8234083007812E+04, -.1345078857422E+04, .7966693725586E+03},
 {   27182, .8311797851562E+04, -.1362935058594E+04, .9566548461914E+03},
 {   27183, .8307865234375E+04, -.1356534301758E+04, .9606137084961E+03},
 {   27184, .8306704101562E+04, -.1362808105469E+04, .9455220336914E+03},
 {   27185, .8312831054688E+04, -.1371291625977E+04, .9423051147461E+03},
 {   27186, .8303945312500E+04, -.1350133422852E+04, .9645391235352E+03},
 {   27187, .8300597656250E+04, -.1354324462891E+04, .9487283325195E+03},
 {   27188, .8318948242188E+04, -.1379800659180E+04, .9391217041016E+03},
 {   27189, .8311354492188E+04, -.1377946411133E+04, .9258240966797E+03},
 {   27190, .8319054687500E+04, -.1388081054688E+04, .9228218383789E+03},
 {   27191, .8303675781250E+04, -.1367786499023E+04, .9288158569336E+03},
 {   27192, .8295976562500E+04, -.1357626464844E+04, .9318176879883E+03},
 {   27193, .8308071289062E+04, -.1383102661133E+04, .9082312622070E+03},
 {   27194, .8316886718750E+04, -.1394532714844E+04, .9051749877930E+03},
 {   27195, .8299264648438E+04, -.1371647216797E+04, .9113107299805E+03},
 {   27196, .8290457031250E+04, -.1360217285156E+04, .9143906860352E+03},
 {   27197, .8281649414062E+04, -.1348761840820E+04, .9174707641602E+03},
 {   27198, .8302695312500E+04, -.1386201416016E+04, .8900148925781E+03},
 {   27199, .8312082031250E+04, -.1398418823242E+04, .8867324218750E+03},
 {   27200, .8293299804688E+04, -.1373984008789E+04, .8933311767578E+03},
 {   27201, .8283896484375E+04, -.1361766601562E+04, .8966235351562E+03},
 {   27202, .8274486328125E+04, -.1349549194336E+04, .8999165039062E+03},
 {   27203, .8295294921875E+04, -.1386988891602E+04, .8719844360352E+03},
 {   27204, .8304788085938E+04, -.1399384033203E+04, .8685134277344E+03},
 {   27205, .8285811523438E+04, -.1374568237305E+04, .8754784545898E+03},
 {   27206, .8276316406250E+04, -.1362147705078E+04, .8789494628906E+03},
 {   27207, .8285662109375E+04, -.1384398071289E+04, .8555639648438E+03},
 {   27208, .8292644531250E+04, -.1394659667969E+04, .8507687988281E+03},
 {   27209, .8276656250000E+04, -.1372637817383E+04, .8588170166016E+03},
 {   27210, .8267624023438E+04, -.1360852294922E+04, .8620805664062E+03},
 {   27211, .8275251953125E+04, -.1380384887695E+04, .8403267211914E+03},
 {   27212, .8283599609375E+04, -.1391129028320E+04, .8375914306641E+03},
 {   27213, .8266913085938E+04, -.1369615234375E+04, .8430853271484E+03},
 {   27214, .8258564453125E+04, -.1358845703125E+04, .8458209838867E+03},
 {   27215, .8264884765625E+04, -.1376524047852E+04, .8248187866211E+03},
 {   27216, .8272577148438E+04, -.1386328491211E+04, .8225441894531E+03},
 {   27217, .8257203125000E+04, -.1366719604492E+04, .8271166992188E+03},
 {   27218, .8249498046875E+04, -.1356915283203E+04, .8294251098633E+03},
 {   27219, .8254197265625E+04, -.1372002807617E+04, .8099566650391E+03},
 {   27220, .8261181640625E+04, -.1380664306641E+04, .8082770385742E+03},
 {   27221, .8247246093750E+04, -.1363341430664E+04, .8116488647461E+03},
 {   27222, .8240284179688E+04, -.1354654663086E+04, .8133181152344E+03},
 {   27223, .8243024414062E+04, -.1366008422852E+04, .7968698730469E+03},
 {   27224, .8249121093750E+04, -.1373171264648E+04, .7962548828125E+03},
 {   27225, .8236912109375E+04, -.1358845703125E+04, .7975180053711E+03},
 {   27226, .8230815429688E+04, -.1351657470703E+04, .7981326293945E+03},
 {   27227, .8303105468750E+04, -.1370783691406E+04, .9605462646484E+03},
 {   27228, .8301349609375E+04, -.1362401611328E+04, .9635301513672E+03},
 {   27229, .8298033203125E+04, -.1370605834961E+04, .9494031982422E+03},
 {   27230, .8301262695312E+04, -.1381705688477E+04, .9474836425781E+03},
 {   27231, .8299588867188E+04, -.1354045043945E+04, .9664901733398E+03},
 {   27232, .8294801757812E+04, -.1359531494141E+04, .9513229980469E+03},
 {   27233, .8297834960938E+04, -.1345688476562E+04, .9694736938477E+03},
 {   27234, .8304505859375E+04, -.1392805419922E+04, .9455875244141E+03},
 {   27235, .8297537109375E+04, -.1390392456055E+04, .9320095214844E+03},
 {   27236, .8301784179688E+04, -.1403651245117E+04, .9305539550781E+03},
 {   27237, .8293314453125E+04, -.1377133666992E+04, .9334546508789E+03},
 {   27238, .8289066406250E+04, -.1363849487305E+04, .9349104614258E+03},
 {   27239, .8292515625000E+04, -.1397123413086E+04, .9151950683594E+03},
 {   27240, .8297437500000E+04, -.1412058593750E+04, .9138819580078E+03},
 {   27241, .8287602539062E+04, -.1382162841797E+04, .9165312500000E+03},
 {   27242, .8282692382812E+04, -.1367227661133E+04, .9178674316406E+03},
 {   27243, .8286053710938E+04, -.1401187500000E+04, .8974665527344E+03},
 {   27244, .8291310546875E+04, -.1417164062500E+04, .8960311279297E+03},
 {   27245, .8280827148438E+04, -.1385236206055E+04, .8989143676758E+03},
 {   27246, .8275594726562E+04, -.1369259643555E+04, .9003392333984E+03},
 {   27247, .8278417968750E+04, -.1402203491211E+04, .8795394287109E+03},
 {   27248, .8283692382812E+04, -.1418408691406E+04, .8779573974609E+03},
 {   27249, .8273152343750E+04, -.1385972900391E+04, .8811450195312E+03},
 {   27250, .8267879882812E+04, -.1369767700195E+04, .8827270507812E+03},
 {   27251, .8262614257812E+04, -.1353537109375E+04, .8843326416016E+03},
 {   27252, .8269643554688E+04, -.1398825195312E+04, .8627353515625E+03},
 {   27253, .8273019531250E+04, -.1412160278320E+04, .8592754516602E+03},
 {   27254, .8264645507812E+04, -.1383458251953E+04, .8641928710938E+03},
 {   27255, .8259625000000E+04, -.1368091308594E+04, .8656610107422E+03},
 {   27256, .8260622070312E+04, -.1393567504883E+04, .8468756103516E+03},
 {   27257, .8265308593750E+04, -.1407639038086E+04, .8457797241211E+03},
 {   27258, .8255922851562E+04, -.1379521240234E+04, .8480046997070E+03},
 {   27259, .8251237304688E+04, -.1365449707031E+04, .8491005859375E+03},
 {   27260, .8251554687500E+04, -.1388538208008E+04, .8307864379883E+03},
 {   27261, .8255910156250E+04, -.1401339843750E+04, .8300059814453E+03},
 {   27262, .8247188476562E+04, -.1375736694336E+04, .8316001586914E+03},
 {   27263, .8242844726562E+04, -.1362935058594E+04, .8324035034180E+03},
 {   27264, .8242420898438E+04, -.1382620117188E+04, .8152286987305E+03},
 {   27265, .8246437500000E+04, -.1393948486328E+04, .8148775024414E+03},
 {   27266, .8238415039062E+04, -.1371317016602E+04, .8156026611328E+03},
 {   27267, .8234395507812E+04, -.1359988647461E+04, .8159543457031E+03},
 {   27268, .8233262695312E+04, -.1374796875000E+04, .8012390136719E+03},
 {   27269, .8236926757812E+04, -.1384169433594E+04, .8017138671875E+03},
 {   27270, .8229609375000E+04, -.1365424316406E+04, .8007870483398E+03},
 {   27271, .8225947265625E+04, -.1356077026367E+04, .8003120727539E+03},
 {   27272, .8292835937500E+04, -.1373526855469E+04, .9651440429688E+03},
 {   27273, .8293656250000E+04, -.1364484497070E+04, .9669757080078E+03},
 {   27274, .8287833007812E+04, -.1373348999023E+04, .9539696044922E+03},
 {   27275, .8287657226562E+04, -.1385363281250E+04, .9535756835938E+03},
 {   27276, .8294463867188E+04, -.1355416625977E+04, .9687838745117E+03},
 {   27277, .8288006835938E+04, -.1361360229492E+04, .9543640747070E+03},
 {   27278, .8295285156250E+04, -.1346374267578E+04, .9706152343750E+03},
 {   27279, .8287490234375E+04, -.1397377441406E+04, .9532045288086E+03},
 {   27280, .8281261718750E+04, -.1394761230469E+04, .9392952880859E+03},
 {   27281, .8281427734375E+04, -.1409112304688E+04, .9396658325195E+03},
 {   27282, .8281096679688E+04, -.1380410278320E+04, .9389240722656E+03},
 {   27283, .8280930664062E+04, -.1366033813477E+04, .9385535278320E+03},
 {   27284, .8274155273438E+04, -.1402051025391E+04, .9234141845703E+03},
 {   27285, .8274485351562E+04, -.1418230834961E+04, .9241564331055E+03},
 {   27286, .8273832031250E+04, -.1385871215820E+04, .9226959228516E+03},
 {   27287, .8273512695312E+04, -.1369691406250E+04, .9219766845703E+03},
 {   27288, .8266462890625E+04, -.1406445312500E+04, .9062358398438E+03},
 {   27289, .8266805664062E+04, -.1423742675781E+04, .9070011596680E+03},
 {   27290, .8266130859375E+04, -.1389173217773E+04, .9054941406250E+03},
 {   27291, .8265789062500E+04, -.1371901245117E+04, .9047287597656E+03},
 {   27292, .8258502929688E+04, -.1407537475586E+04, .8884545288086E+03},
 {   27293, .8258793945312E+04, -.1425114257812E+04, .8891035156250E+03},
 {   27294, .8258223632812E+04, -.1389986083984E+04, .8878286132812E+03},
 {   27295, .8257931640625E+04, -.1372434692383E+04, .8871795654297E+03},
 {   27296, .8257654296875E+04, -.1354883300781E+04, .8865532226562E+03},
 {   27297, .8250772460938E+04, -.1403905273438E+04, .8711831054688E+03},
 {   27298, .8250617187500E+04, -.1418764282227E+04, .8708352050781E+03},
 {   27299, .8250479492188E+04, -.1387242797852E+04, .8705341796875E+03},
 {   27300, .8250189453125E+04, -.1370605834961E+04, .8698851318359E+03},
 {   27301, .8243350585938E+04, -.1398215698242E+04, .8546072387695E+03},
 {   27302, .8243722656250E+04, -.1413430297852E+04, .8554419555664E+03},
 {   27303, .8242986328125E+04, -.1383001098633E+04, .8537957763672E+03},
 {   27304, .8242613281250E+04, -.1367761108398E+04, .8529613647461E+03},
 {   27305, .8235836914062E+04, -.1392754638672E+04, .8378227539062E+03},
 {   27306, .8236274414062E+04, -.1406623046875E+04, .8387962646484E+03},
 {   27307, .8235412109375E+04, -.1378911621094E+04, .8368720703125E+03},
 {   27308, .8234987304688E+04, -.1365043212891E+04, .8359213867188E+03},
 {   27309, .8234552734375E+04, -.1351174804688E+04, .8349478759766E+03},
 {   27310, .8228512695312E+04, -.1386353881836E+04, .8214550781250E+03},
 {   27311, .8229051757812E+04, -.1398622070312E+04, .8226607055664E+03},
 {   27312, .8227981445312E+04, -.1374111083984E+04, .8202731933594E+03},
 {   27313, .8227441406250E+04, -.1361842895508E+04, .8190677490234E+03},
 {   27314, .8221766601562E+04, -.1377895629883E+04, .8063860473633E+03},
 {   27315, .8222553710938E+04, -.1388030273438E+04, .8081483154297E+03},
 {   27316, .8220987304688E+04, -.1367761108398E+04, .8046475830078E+03},
 {   27317, .8220196289062E+04, -.1357601074219E+04, .8028858642578E+03},
 {   27318, .8282563476562E+04, -.1370783691406E+04, .9697417602539E+03},
 {   27319, .8285933593750E+04, -.1362401611328E+04, .9704313354492E+03},
 {   27320, .8277609375000E+04, -.1370605834961E+04, .9585465087891E+03},
 {   27321, .8274024414062E+04, -.1381705688477E+04, .9596782226562E+03},
 {   27322, .8289317382812E+04, -.1354045043945E+04, .9710875244141E+03},
 {   27323, .8281193359375E+04, -.1359531494141E+04, .9574148559570E+03},
 {   27324, .8270451171875E+04, -.1392805419922E+04, .9608327026367E+03},
 {   27325, .8264966796875E+04, -.1390392456055E+04, .9465905761719E+03},
 {   27326, .8261049804688E+04, -.1403651245117E+04, .9487883300781E+03},
 {   27327, .8268857421875E+04, -.1377133666992E+04, .9444035644531E+03},
 {   27328, .8272771484375E+04, -.1363849487305E+04, .9422060546875E+03},
 {   27329, .8255793945312E+04, -.1397123413086E+04, .9316337890625E+03},
 {   27330, .8251535156250E+04, -.1412058593750E+04, .9344305419922E+03},
 {   27331, .8260061523438E+04, -.1382162841797E+04, .9288604736328E+03},
 {   27332, .8264329101562E+04, -.1367227661133E+04, .9260870361328E+03},
 {   27333, .8246852539062E+04, -.1401187500000E+04, .9150158081055E+03},
 {   27334, .8242277343750E+04, -.1417164062500E+04, .9179812011719E+03},
 {   27335, .8251411132812E+04, -.1385236206055E+04, .9120841064453E+03},
 {   27336, .8255981445312E+04, -.1369259643555E+04, .9091189575195E+03},
 {   27337, .8238566406250E+04, -.1402203491211E+04, .8973796386719E+03},
 {   27338, .8233871093750E+04, -.1418408691406E+04, .9002604370117E+03},
 {   27339, .8243269531250E+04, -.1385972900391E+04, .8945223388672E+03},
 {   27340, .8247965820312E+04, -.1369767700195E+04, .8916418457031E+03},
 {   27341, .8231877929688E+04, -.1398825195312E+04, .8796412353516E+03},
 {   27342, .8227785156250E+04, -.1413455688477E+04, .8825587158203E+03},
 {   27343, .8236315429688E+04, -.1383458251953E+04, .8768750000000E+03},
 {   27344, .8240731445312E+04, -.1368091308594E+04, .8741190795898E+03},
 {   27345, .8226055664062E+04, -.1393567504883E+04, .8623492431641E+03},
 {   27346, .8221863281250E+04, -.1408604248047E+04, .8652850952148E+03},
 {   27347, .8230029296875E+04, -.1379521240234E+04, .8595973510742E+03},
 {   27348, .8233965820312E+04, -.1365449707031E+04, .8568322753906E+03},
 {   27349, .8220098632812E+04, -.1388538208008E+04, .8448690185547E+03},
 {   27350, .8216178710938E+04, -.1402940063477E+04, .8479311523438E+03},
 {   27351, .8223613281250E+04, -.1375736694336E+04, .8421545410156E+03},
 {   27352, .8227105468750E+04, -.1362935058594E+04, .8394500732422E+03},
 {   27353, .8214578125000E+04, -.1382620117188E+04, .8276926879883E+03},
 {   27354, .8210952148438E+04, -.1396590087891E+04, .8310966186523E+03},
 {   27355, .8217525390625E+04, -.1371317016602E+04, .8249536743164E+03},
 {   27356, .8220463867188E+04, -.1359988647461E+04, .8221913452148E+03},
 {   27357, .8210243164062E+04, -.1374796875000E+04, .8115443725586E+03},
 {   27358, .8207259765625E+04, -.1388157226562E+04, .8158851318359E+03},
 {   27359, .8212340820312E+04, -.1365424316406E+04, .8085184936523E+03},
 {   27360, .8214423828125E+04, -.1356077026367E+04, .8054699707031E+03},
 {   27361, .8216511718750E+04, -.1346704467773E+04, .8024212646484E+03},
 {   27362, .8273847656250E+04, -.1362935058594E+04, .9736436767578E+03},
 {   27363, .8279419921875E+04, -.1356534301758E+04, .9733477172852E+03},
 {   27364, .8268936523438E+04, -.1362808105469E+04, .9624278564453E+03},
 {   27365, .8262456054688E+04, -.1371291625977E+04, .9648565673828E+03},
 {   27366, .8284958007812E+04, -.1350133422852E+04, .9730386962891E+03},
 {   27367, .8275395507812E+04, -.1354324462891E+04, .9600101318359E+03},
 {   27368, .8256004882812E+04, -.1379800659180E+04, .9672984619141E+03},
 {   27369, .8251149414062E+04, -.1377946411133E+04, .9527762451172E+03},
 {   27370, .8243779296875E+04, -.1388081054688E+04, .9565200805664E+03},
 {   27371, .8258515625000E+04, -.1367786499023E+04, .9490324096680E+03},
 {   27372, .8265862304688E+04, -.1357626464844E+04, .9452988891602E+03},
 {   27373, .8240213867188E+04, -.1383102661133E+04, .9386080932617E+03},
 {   27374, .8232062500000E+04, -.1394532714844E+04, .9431480102539E+03},
 {   27375, .8248377929688E+04, -.1371647216797E+04, .9340910644531E+03},
 {   27376, .8256541992188E+04, -.1360217285156E+04, .9295738525391E+03},
 {   27377, .8230229492188E+04, -.1386201416016E+04, .9224565429688E+03},
 {   27378, .8221505859375E+04, -.1398418823242E+04, .9272800292969E+03},
 {   27379, .8238938476562E+04, -.1373984008789E+04, .9176677246094E+03},
 {   27380, .8247659179688E+04, -.1361766601562E+04, .9128448486328E+03},
 {   27381, .8221687500000E+04, -.1386988891602E+04, .9049349975586E+03},
 {   27382, .8212774414062E+04, -.1399384033203E+04, .9097045288086E+03},
 {   27383, .8230611328125E+04, -.1374568237305E+04, .9001893310547E+03},
 {   27384, .8239525390625E+04, -.1362147705078E+04, .8954196166992E+03},
 {   27385, .8215859375000E+04, -.1384398071289E+04, .8868126831055E+03},
 {   27386, .8207232421875E+04, -.1396437622070E+04, .8914805908203E+03},
 {   27387, .8224306640625E+04, -.1372637817383E+04, .8822510986328E+03},
 {   27388, .8232732421875E+04, -.1360852294922E+04, .8776998901367E+03},
 {   27389, .8211426757812E+04, -.1380384887695E+04, .8688981933594E+03},
 {   27390, .8202790039062E+04, -.1392602294922E+04, .8739062500000E+03},
 {   27391, .8219040039062E+04, -.1369615234375E+04, .8645164794922E+03},
 {   27392, .8226640625000E+04, -.1358845703125E+04, .8601118774414E+03},
 {   27393, .8206765625000E+04, -.1376524047852E+04, .8508369750977E+03},
 {   27394, .8198243164062E+04, -.1388741455078E+04, .8560999145508E+03},
 {   27395, .8213621093750E+04, -.1366719604492E+04, .8466275634766E+03},
 {   27396, .8220450195312E+04, -.1356915283203E+04, .8424287719727E+03},
 {   27397, .8202799804688E+04, -.1372002807617E+04, .8329647827148E+03},
 {   27398, .8194153320312E+04, -.1384728271484E+04, .8389505004883E+03},
 {   27399, .8208691406250E+04, -.1363341430664E+04, .8289078369141E+03},
 {   27400, .8214576171875E+04, -.1354654663086E+04, .8248271484375E+03},
 {   27401, .8200482421875E+04, -.1366008422852E+04, .8159137573242E+03},
 {   27402, .8192092773438E+04, -.1379292602539E+04, .8235931396484E+03},
 {   27403, .8205037109375E+04, -.1358845703125E+04, .8117878417969E+03},
 {   27404, .8209554687500E+04, -.1351657470703E+04, .8076497802734E+03},
 {   27405, .8214078125000E+04, -.1344494628906E+04, .8035107421875E+03},
 {   27406, .8268029296875E+04, -.1351174804688E+04, .9762485351562E+03},
 {   27407, .8275036132812E+04, -.1347720458984E+04, .9753095703125E+03},
 {   27408, .8263142578125E+04, -.1351124023438E+04, .9650224609375E+03},
 {   27409, .8254735351562E+04, -.1355721435547E+04, .9683128662109E+03},
 {   27410, .8282060546875E+04, -.1344265991211E+04, .9743361816406E+03},
 {   27411, .8271548828125E+04, -.1346526611328E+04, .9617325439453E+03},
 {   27412, .8246340820312E+04, -.1360318847656E+04, .9716257324219E+03},
 {   27413, .8241920898438E+04, -.1359302856445E+04, .9569063720703E+03},
 {   27414, .8232257812500E+04, -.1364789306641E+04, .9616779785156E+03},
 {   27415, .8251584960938E+04, -.1353816406250E+04, .9521351928711E+03},
 {   27416, .8261248046875E+04, -.1348304687500E+04, .9473640747070E+03},
 {   27417, .8229805664062E+04, -.1362096801758E+04, .9432680053711E+03},
 {   27418, .8219055664062E+04, -.1368294433594E+04, .9489707641602E+03},
 {   27419, .8240587890625E+04, -.1355899291992E+04, .9375780029297E+03},
 {   27420, .8251347656250E+04, -.1349701660156E+04, .9318988037109E+03},
 {   27421, .8219101562500E+04, -.1363773193359E+04, .9274382324219E+03},
 {   27422, .8207619140625E+04, -.1370402587891E+04, .9334963378906E+03},
 {   27423, .8230614257812E+04, -.1357169189453E+04, .9213933715820E+03},
 {   27424, .8242096679688E+04, -.1350565307617E+04, .9153352661133E+03},
 {   27425, .8210399414062E+04, -.1364205078125E+04, .9099890136719E+03},
 {   27426, .8198657226562E+04, -.1370910644531E+04, .9160246582031E+03},
 {   27427, .8222150390625E+04, -.1357473999023E+04, .9039768066406E+03},
 {   27428, .8233869140625E+04, -.1350768432617E+04, .8979519653320E+03},
 {   27429, .8205168945312E+04, -.1362808105469E+04, .8915972900391E+03},
 {   27430, .8193670898438E+04, -.1369412109375E+04, .8975519409180E+03},
 {   27431, .8216262695312E+04, -.1356432617188E+04, .8858524780273E+03},
 {   27432, .8227377929688E+04, -.1350057250977E+04, .8800968627930E+03},
 {   27433, .8201645507812E+04, -.1360623657227E+04, .8732776489258E+03},
 {   27434, .8189549804688E+04, -.1367634033203E+04, .8798889160156E+03},
 {   27435, .8211689453125E+04, -.1354807006836E+04, .8678062744141E+03},
 {   27436, .8221750976562E+04, -.1348965087891E+04, .8623016967773E+03},
 {   27437, .8197863281250E+04, -.1358540893555E+04, .8548222656250E+03},
 {   27438, .8185425781250E+04, -.1365805297852E+04, .8619762573242E+03},
 {   27439, .8206919921875E+04, -.1353232299805E+04, .8496271362305E+03},
 {   27440, .8216000000000E+04, -.1347923706055E+04, .8444213867188E+03},
 {   27441, .8194917968750E+04, -.1356102416992E+04, .8364934692383E+03},
 {   27442, .8181551269531E+04, -.1364078002930E+04, .8449262695312E+03},
 {   27443, .8202780273438E+04, -.1351403442383E+04, .8315543212891E+03},
 {   27444, .8210632812500E+04, -.1346704467773E+04, .8265919799805E+03},
 {   27445, .8193967773438E+04, -.1352851318359E+04, .8188298339844E+03},
 {   27446, .8179223144531E+04, -.1362122314453E+04, .8304671630859E+03},
 {   27447, .8200144531250E+04, -.1348965087891E+04, .8139777221680E+03},
 {   27448, .8206290039062E+04, -.1345078857422E+04, .8091124877930E+03},
 {   27449, .8212455078125E+04, -.1341218017578E+04, .8042374877930E+03},
 {   27450, .8265989257812E+04, -.1337331787109E+04, .9771616210938E+03},
 {   27451, .8273507812500E+04, -.1337331787109E+04, .9759941406250E+03},
 {   27452, .8261101562500E+04, -.1337331787109E+04, .9659357299805E+03},
 {   27453, .8252025390625E+04, -.1337331787109E+04, .9695270996094E+03},
 {   27454, .8281041015625E+04, -.1337331787109E+04, .9747925415039E+03},
 {   27455, .8270179687500E+04, -.1337331787109E+04, .9623446655273E+03},
 {   27456, .8288560546875E+04, -.1337331787109E+04, .9736248168945E+03},
 {   27457, .8242957031250E+04, -.1337331787109E+04, .9731406250000E+03},
 {   27458, .8238676757812E+04, -.1337331787109E+04, .9583593139648E+03},
 {   27459, .8228200195312E+04, -.1337331787109E+04, .9634939575195E+03},
 {   27460, .8249150390625E+04, -.1337331787109E+04, .9532250976562E+03},
 {   27461, .8259626953125E+04, -.1337331787109E+04, .9480907592773E+03},
 {   27462, .8226164062500E+04, -.1337331787109E+04, .9448974609375E+03},
 {   27463, .8214510742188E+04, -.1337331787109E+04, .9510046386719E+03},
 {   27464, .8237828125000E+04, -.1337331787109E+04, .9388132324219E+03},
 {   27465, .8249514648438E+04, -.1337331787109E+04, .9327188110352E+03},
 {   27466, .8215204101562E+04, -.1337331787109E+04, .9291823730469E+03},
 {   27467, .8202726562500E+04, -.1337331787109E+04, .9356864624023E+03},
 {   27468, .8227693359375E+04, -.1337331787109E+04, .9227011718750E+03},
 {   27469, .8240147460938E+04, -.1337331787109E+04, .9162069702148E+03},
 {   27470, .8206434570312E+04, -.1337331909180E+04, .9117637939453E+03},
 {   27471, .8193721679688E+04, -.1337331909180E+04, .9182346801758E+03},
 {   27472, .8219160156250E+04, -.1337331909180E+04, .9053159790039E+03},
 {   27473, .8231900390625E+04, -.1337331909180E+04, .8988336791992E+03},
 {   27474, .8201414062500E+04, -.1337331909180E+04, .8932781372070E+03},
 {   27475, .8188780273438E+04, -.1337331909180E+04, .8997414550781E+03},
 {   27476, .8213457031250E+04, -.1337331909180E+04, .8871081542969E+03},
 {   27477, .8225500976562E+04, -.1337331909180E+04, .8809375000000E+03},
 {   27478, .8237553710938E+04, -.1337331909180E+04, .8747904052734E+03},
 {   27479, .8198213867188E+04, -.1337331909180E+04, .8748133544922E+03},
 {   27480, .8184340332031E+04, -.1337331909180E+04, .8823043212891E+03},
 {   27481, .8209118164062E+04, -.1337331909180E+04, .8689582519531E+03},
 {   27482, .8220036132812E+04, -.1337331909180E+04, .8630693969727E+03},
 {   27483, .8194736328125E+04, -.1337331909180E+04, .8562226562500E+03},
 {   27484, .8180007324219E+04, -.1337331909180E+04, .8645412597656E+03},
 {   27485, .8204579101562E+04, -.1337331909180E+04, .8506751098633E+03},
 {   27486, .8214421875000E+04, -.1337331909180E+04, .8451269531250E+03},
 {   27487, .8192136718750E+04, -.1337331909180E+04, .8377387695312E+03},
 {   27488, .8175604003906E+04, -.1337331909180E+04, .8479232177734E+03},
 {   27489, .8200695312500E+04, -.1337331909180E+04, .8324879760742E+03},
 {   27490, .8209242187500E+04, -.1337331909180E+04, .8272145385742E+03},
 {   27491, .8217823242188E+04, -.1337331909180E+04, .8219534912109E+03},
 {   27492, .8191673828125E+04, -.1337331909180E+04, .8198571166992E+03},
 {   27493, .8171235839844E+04, -.1337331909180E+04, .8356853637695E+03},
 {   27494, .8198428710938E+04, -.1337331909180E+04, .8147458496094E+03},
 {   27495, .8205153320312E+04, -.1337331909180E+04, .8096209716797E+03},
 {   27496, .8211873046875E+04, -.1337331909180E+04, .8044968872070E+03},
 {   27497, .8211873046875E+04, -.1324403320312E+04, .8044968872070E+03},
 {   27498, .8218631835938E+04, -.1324403320312E+04, .7993851928711E+03},
 {   27499, .8209442382812E+04, -.1337331909180E+04, .7788684082031E+03},
 {   27500, .8201913085938E+04, -.1337331909180E+04, .7822408447266E+03},
 {   27501, .8201913085938E+04, -.1324403320312E+04, .7822408447266E+03},
 {   27502, .8209442382812E+04, -.1324403320312E+04, .7788684082031E+03},
 {   27503, .8171235839844E+04, -.1324377807617E+04, .8356853637695E+03},
 {   27504, .8191673828125E+04, -.1324403320312E+04, .8198571166992E+03},
 {   27505, .8179353515625E+04, -.1337331909180E+04, .7923390502930E+03},
 {   27506, .8159024902344E+04, -.1337331909180E+04, .8014402465820E+03},
 {   27507, .8159024902344E+04, -.1324403320312E+04, .8014402465820E+03},
 {   27508, .8179353515625E+04, -.1324403320312E+04, .7923390502930E+03},
 {   27509, .8198428710938E+04, -.1324403320312E+04, .8147458496094E+03},
 {   27510, .8186889160156E+04, -.1337331909180E+04, .7889658813477E+03},
 {   27511, .8186889160156E+04, -.1324403320312E+04, .7889658813477E+03},
 {   27512, .8205153320312E+04, -.1324403320312E+04, .8096209716797E+03},
 {   27513, .8194401367188E+04, -.1337331909180E+04, .7856035156250E+03},
 {   27514, .8194401367188E+04, -.1324403320312E+04, .7856035156250E+03},
 {   27515, .8211873046875E+04, -.1311449218750E+04, .8044968872070E+03},
 {   27516, .8218631835938E+04, -.1311449218750E+04, .7993850708008E+03},
 {   27517, .8201913085938E+04, -.1311449218750E+04, .7822408447266E+03},
 {   27518, .8209442382812E+04, -.1311449218750E+04, .7788684082031E+03},
 {   27519, .8171235839844E+04, -.1311449218750E+04, .8356853637695E+03},
 {   27520, .8191673828125E+04, -.1311449218750E+04, .8198571166992E+03},
 {   27521, .8159024902344E+04, -.1311449218750E+04, .8014402465820E+03},
 {   27522, .8179353515625E+04, -.1311449218750E+04, .7923390502930E+03},
 {   27523, .8198428710938E+04, -.1311449218750E+04, .8147458496094E+03},
 {   27524, .8186889160156E+04, -.1311449218750E+04, .7889658813477E+03},
 {   27525, .8205153320312E+04, -.1311449218750E+04, .8096209716797E+03},
 {   27526, .8194401367188E+04, -.1311449218750E+04, .7856035156250E+03},
 {   27527, .8211873046875E+04, -.1298520629883E+04, .8044968872070E+03},
 {   27528, .8218631835938E+04, -.1298520629883E+04, .7993850708008E+03},
 {   27529, .8201913085938E+04, -.1298520629883E+04, .7822408447266E+03},
 {   27530, .8209442382812E+04, -.1298520629883E+04, .7788684082031E+03},
 {   27531, .8171235839844E+04, -.1298520629883E+04, .8356853637695E+03},
 {   27532, .8191673828125E+04, -.1298520629883E+04, .8198571166992E+03},
 {   27533, .8159024902344E+04, -.1298520629883E+04, .8014402465820E+03},
 {   27534, .8179353515625E+04, -.1298520629883E+04, .7923390502930E+03},
 {   27535, .8198428710938E+04, -.1298520629883E+04, .8147458496094E+03},
 {   27536, .8186889160156E+04, -.1298520629883E+04, .7889658813477E+03},
 {   27537, .8205153320312E+04, -.1298520629883E+04, .8096209716797E+03},
 {   27538, .8194401367188E+04, -.1298520629883E+04, .7856035156250E+03},
 {   27539, .8211873046875E+04, -.1285592041016E+04, .8044968872070E+03},
 {   27540, .8218631835938E+04, -.1285592041016E+04, .7993850708008E+03},
 {   27541, .8201913085938E+04, -.1285592041016E+04, .7822408447266E+03},
 {   27542, .8209442382812E+04, -.1285592041016E+04, .7788684082031E+03},
 {   27543, .8171235839844E+04, -.1285592041016E+04, .8356853637695E+03},
 {   27544, .8191673828125E+04, -.1285592041016E+04, .8198571166992E+03},
 {   27545, .8159024902344E+04, -.1285592041016E+04, .8014402465820E+03},
 {   27546, .8179353515625E+04, -.1285592041016E+04, .7923390502930E+03},
 {   27547, .8198428710938E+04, -.1285592041016E+04, .8147458496094E+03},
 {   27548, .8186889160156E+04, -.1285592041016E+04, .7889658813477E+03},
 {   27549, .8205153320312E+04, -.1285592041016E+04, .8096209716797E+03},
 {   27550, .8194401367188E+04, -.1285592041016E+04, .7856035156250E+03},
 {   27551, .8217823242188E+04, -.1324403320312E+04, .8219534912109E+03},
 {   27552, .8226375000000E+04, -.1324403320312E+04, .8166793823242E+03},
 {   27553, .8224255859375E+04, -.1324403320312E+04, .8395559082031E+03},
 {   27554, .8234124023438E+04, -.1324403320312E+04, .8339976806641E+03},
 {   27555, .8230948242188E+04, -.1324403320312E+04, .8571810913086E+03},
 {   27556, .8241867187500E+04, -.1324403320312E+04, .8512919311523E+03},
 {   27557, .8237553710938E+04, -.1324403320312E+04, .8747904052734E+03},
 {   27558, .8249620117188E+04, -.1324403320312E+04, .8686099243164E+03},
 {   27559, .8244624023438E+04, -.1324403198242E+04, .8923859863281E+03},
 {   27560, .8257364257812E+04, -.1324403198242E+04, .8859042968750E+03},
 {   27561, .8252626953125E+04, -.1324403198242E+04, .9097031250000E+03},
 {   27562, .8265114257812E+04, -.1324403198242E+04, .9032221679688E+03},
 {   27563, .8261179687500E+04, -.1324403198242E+04, .9266342773438E+03},
 {   27564, .8272856445312E+04, -.1324403198242E+04, .9205166625977E+03},
 {   27565, .8270110351562E+04, -.1324403198242E+04, .9429790649414E+03},
 {   27566, .8280608398438E+04, -.1324403198242E+04, .9378344116211E+03},
 {   27567, .8279260742188E+04, -.1324403198242E+04, .9587534790039E+03},
 {   27568, .8288351562500E+04, -.1324403198242E+04, .9551288452148E+03},
 {   27569, .8288560546875E+04, -.1324403198242E+04, .9736248168945E+03},
 {   27570, .8296103515625E+04, -.1324403198242E+04, .9724464721680E+03},
 {   27571, .8228200195312E+04, -.1324403198242E+04, .9634939575195E+03},
 {   27572, .8242957031250E+04, -.1324403198242E+04, .9731406250000E+03},
 {   27573, .8252025390625E+04, -.1324403198242E+04, .9695270996094E+03},
 {   27574, .8238676757812E+04, -.1324403198242E+04, .9583593139648E+03},
 {   27575, .8214510742188E+04, -.1324403198242E+04, .9510046386719E+03},
 {   27576, .8226164062500E+04, -.1324403198242E+04, .9448974609375E+03},
 {   27577, .8202726562500E+04, -.1324403198242E+04, .9356864624023E+03},
 {   27578, .8215204101562E+04, -.1324403198242E+04, .9291823730469E+03},
 {   27579, .8193721679688E+04, -.1324403198242E+04, .9182346801758E+03},
 {   27580, .8206434570312E+04, -.1324403198242E+04, .9117637939453E+03},
 {   27581, .8188780273438E+04, -.1324403320312E+04, .8997414550781E+03},
 {   27582, .8201414062500E+04, -.1324403320312E+04, .8932781372070E+03},
 {   27583, .8184340332031E+04, -.1324403320312E+04, .8823043212891E+03},
 {   27584, .8198213867188E+04, -.1324403320312E+04, .8748133544922E+03},
 {   27585, .8180007324219E+04, -.1324403320312E+04, .8645412597656E+03},
 {   27586, .8194736328125E+04, -.1324403320312E+04, .8562226562500E+03},
 {   27587, .8175604003906E+04, -.1324403320312E+04, .8479232177734E+03},
 {   27588, .8192136718750E+04, -.1324403320312E+04, .8377387695312E+03},
 {   27589, .8265989257812E+04, -.1324403198242E+04, .9771616210938E+03},
 {   27590, .8273507812500E+04, -.1324403198242E+04, .9759941406250E+03},
 {   27591, .8261101562500E+04, -.1324403198242E+04, .9659357299805E+03},
 {   27592, .8281041015625E+04, -.1324403198242E+04, .9747925415039E+03},
 {   27593, .8270179687500E+04, -.1324403198242E+04, .9623446655273E+03},
 {   27594, .8249150390625E+04, -.1324403198242E+04, .9532250976562E+03},
 {   27595, .8259626953125E+04, -.1324403198242E+04, .9480907592773E+03},
 {   27596, .8237828125000E+04, -.1324403198242E+04, .9388132324219E+03},
 {   27597, .8249514648438E+04, -.1324403198242E+04, .9327188110352E+03},
 {   27598, .8227693359375E+04, -.1324403198242E+04, .9227011718750E+03},
 {   27599, .8240147460938E+04, -.1324403198242E+04, .9162069702148E+03},
 {   27600, .8219160156250E+04, -.1324403198242E+04, .9053159790039E+03},
 {   27601, .8231900390625E+04, -.1324403198242E+04, .8988336791992E+03},
 {   27602, .8213457031250E+04, -.1324403320312E+04, .8871081542969E+03},
 {   27603, .8225500976562E+04, -.1324403320312E+04, .8809375000000E+03},
 {   27604, .8209118164062E+04, -.1324403320312E+04, .8689582519531E+03},
 {   27605, .8220036132812E+04, -.1324403320312E+04, .8630693969727E+03},
 {   27606, .8204579101562E+04, -.1324403320312E+04, .8506751098633E+03},
 {   27607, .8214421875000E+04, -.1324403320312E+04, .8451269531250E+03},
 {   27608, .8200695312500E+04, -.1324403320312E+04, .8324879760742E+03},
 {   27609, .8209242187500E+04, -.1324403320312E+04, .8272145385742E+03},
 {   27610, .8217823242188E+04, -.1311449218750E+04, .8219534912109E+03},
 {   27611, .8226375000000E+04, -.1311449218750E+04, .8166793823242E+03},
 {   27612, .8224255859375E+04, -.1311449218750E+04, .8395559082031E+03},
 {   27613, .8234124023438E+04, -.1311449218750E+04, .8339975585938E+03},
 {   27614, .8230948242188E+04, -.1311449218750E+04, .8571810913086E+03},
 {   27615, .8241867187500E+04, -.1311449218750E+04, .8512919311523E+03},
 {   27616, .8237553710938E+04, -.1311449218750E+04, .8747904052734E+03},
 {   27617, .8249620117188E+04, -.1311449218750E+04, .8686098022461E+03},
 {   27618, .8244624023438E+04, -.1311449218750E+04, .8923859863281E+03},
 {   27619, .8257364257812E+04, -.1311449218750E+04, .8859041748047E+03},
 {   27620, .8252626953125E+04, -.1311449218750E+04, .9097031250000E+03},
 {   27621, .8265114257812E+04, -.1311449218750E+04, .9032221679688E+03},
 {   27622, .8261179687500E+04, -.1311449218750E+04, .9266342773438E+03},
 {   27623, .8272856445312E+04, -.1311449218750E+04, .9205166625977E+03},
 {   27624, .8270110351562E+04, -.1311449218750E+04, .9429790649414E+03},
 {   27625, .8280608398438E+04, -.1311449218750E+04, .9378344116211E+03},
 {   27626, .8279260742188E+04, -.1311449218750E+04, .9587534790039E+03},
 {   27627, .8288351562500E+04, -.1311449218750E+04, .9551288452148E+03},
 {   27628, .8288560546875E+04, -.1311449218750E+04, .9736248168945E+03},
 {   27629, .8296103515625E+04, -.1311449218750E+04, .9724464721680E+03},
 {   27630, .8228200195312E+04, -.1311449218750E+04, .9634939575195E+03},
 {   27631, .8242957031250E+04, -.1311449218750E+04, .9731406250000E+03},
 {   27632, .8252025390625E+04, -.1311449218750E+04, .9695270996094E+03},
 {   27633, .8238676757812E+04, -.1311449218750E+04, .9583593139648E+03},
 {   27634, .8214510742188E+04, -.1311449218750E+04, .9510046386719E+03},
 {   27635, .8226164062500E+04, -.1311449218750E+04, .9448974609375E+03},
 {   27636, .8202726562500E+04, -.1311449218750E+04, .9356864624023E+03},
 {   27637, .8215204101562E+04, -.1311449218750E+04, .9291823730469E+03},
 {   27638, .8193721679688E+04, -.1311449218750E+04, .9182346801758E+03},
 {   27639, .8206434570312E+04, -.1311449218750E+04, .9117637939453E+03},
 {   27640, .8188780273438E+04, -.1311449218750E+04, .8997414550781E+03},
 {   27641, .8201414062500E+04, -.1311449218750E+04, .8932781372070E+03},
 {   27642, .8184340332031E+04, -.1311449218750E+04, .8823043212891E+03},
 {   27643, .8198213867188E+04, -.1311449218750E+04, .8748133544922E+03},
 {   27644, .8180007324219E+04, -.1311449218750E+04, .8645412597656E+03},
 {   27645, .8194736328125E+04, -.1311449218750E+04, .8562226562500E+03},
 {   27646, .8175604003906E+04, -.1311449218750E+04, .8479232177734E+03},
 {   27647, .8192136718750E+04, -.1311449218750E+04, .8377387695312E+03},
 {   27648, .8265989257812E+04, -.1311449218750E+04, .9771616210938E+03},
 {   27649, .8273507812500E+04, -.1311449218750E+04, .9759941406250E+03},
 {   27650, .8261101562500E+04, -.1311449218750E+04, .9659357299805E+03},
 {   27651, .8281041015625E+04, -.1311449218750E+04, .9747925415039E+03},
 {   27652, .8270179687500E+04, -.1311449218750E+04, .9623446655273E+03},
 {   27653, .8249150390625E+04, -.1311449218750E+04, .9532250976562E+03},
 {   27654, .8259626953125E+04, -.1311449218750E+04, .9480907592773E+03},
 {   27655, .8237828125000E+04, -.1311449218750E+04, .9388132324219E+03},
 {   27656, .8249514648438E+04, -.1311449218750E+04, .9327188110352E+03},
 {   27657, .8227693359375E+04, -.1311449218750E+04, .9227011718750E+03},
 {   27658, .8240147460938E+04, -.1311449218750E+04, .9162069702148E+03},
 {   27659, .8219160156250E+04, -.1311449218750E+04, .9053159790039E+03},
 {   27660, .8231900390625E+04, -.1311449218750E+04, .8988336791992E+03},
 {   27661, .8213457031250E+04, -.1311449218750E+04, .8871081542969E+03},
 {   27662, .8225500976562E+04, -.1311449218750E+04, .8809375000000E+03},
 {   27663, .8209118164062E+04, -.1311449218750E+04, .8689582519531E+03},
 {   27664, .8220036132812E+04, -.1311449218750E+04, .8630693969727E+03},
 {   27665, .8204579101562E+04, -.1311449218750E+04, .8506751098633E+03},
 {   27666, .8214421875000E+04, -.1311449218750E+04, .8451269531250E+03},
 {   27667, .8200695312500E+04, -.1311449218750E+04, .8324879760742E+03},
 {   27668, .8209242187500E+04, -.1311449218750E+04, .8272145385742E+03},
 {   27669, .8217823242188E+04, -.1298520629883E+04, .8219534912109E+03},
 {   27670, .8226375000000E+04, -.1298520629883E+04, .8166793823242E+03},
 {   27671, .8224255859375E+04, -.1298520629883E+04, .8395559082031E+03},
 {   27672, .8234124023438E+04, -.1298520629883E+04, .8339975585938E+03},
 {   27673, .8230948242188E+04, -.1298520629883E+04, .8571810913086E+03},
 {   27674, .8241867187500E+04, -.1298520629883E+04, .8512919311523E+03},
 {   27675, .8237553710938E+04, -.1298520629883E+04, .8747904052734E+03},
 {   27676, .8249620117188E+04, -.1298520629883E+04, .8686098022461E+03},
 {   27677, .8244624023438E+04, -.1298520629883E+04, .8923859863281E+03},
 {   27678, .8257364257812E+04, -.1298520629883E+04, .8859041748047E+03},
 {   27679, .8252626953125E+04, -.1298520629883E+04, .9097031250000E+03},
 {   27680, .8265114257812E+04, -.1298520629883E+04, .9032221679688E+03},
 {   27681, .8261179687500E+04, -.1298520629883E+04, .9266342773438E+03},
 {   27682, .8272856445312E+04, -.1298520629883E+04, .9205166625977E+03},
 {   27683, .8270110351562E+04, -.1298520629883E+04, .9429790649414E+03},
 {   27684, .8280608398438E+04, -.1298520629883E+04, .9378344116211E+03},
 {   27685, .8279260742188E+04, -.1298520629883E+04, .9587534790039E+03},
 {   27686, .8288351562500E+04, -.1298520629883E+04, .9551288452148E+03},
 {   27687, .8288560546875E+04, -.1298520629883E+04, .9736248168945E+03},
 {   27688, .8296103515625E+04, -.1298520629883E+04, .9724464721680E+03},
 {   27689, .8228200195312E+04, -.1298520629883E+04, .9634939575195E+03},
 {   27690, .8242957031250E+04, -.1298520629883E+04, .9731406250000E+03},
 {   27691, .8252025390625E+04, -.1298520629883E+04, .9695270996094E+03},
 {   27692, .8238676757812E+04, -.1298520629883E+04, .9583593139648E+03},
 {   27693, .8214510742188E+04, -.1298520629883E+04, .9510046386719E+03},
 {   27694, .8226164062500E+04, -.1298520629883E+04, .9448974609375E+03},
 {   27695, .8202726562500E+04, -.1298520629883E+04, .9356864624023E+03},
 {   27696, .8215204101562E+04, -.1298520629883E+04, .9291823730469E+03},
 {   27697, .8193721679688E+04, -.1298520629883E+04, .9182346801758E+03},
 {   27698, .8206434570312E+04, -.1298520629883E+04, .9117637939453E+03},
 {   27699, .8188780273438E+04, -.1298520629883E+04, .8997414550781E+03},
 {   27700, .8201414062500E+04, -.1298520629883E+04, .8932781372070E+03},
 {   27701, .8184340332031E+04, -.1298520629883E+04, .8823043212891E+03},
 {   27702, .8198213867188E+04, -.1298520629883E+04, .8748133544922E+03},
 {   27703, .8180007324219E+04, -.1298520629883E+04, .8645412597656E+03},
 {   27704, .8194736328125E+04, -.1298520629883E+04, .8562226562500E+03},
 {   27705, .8175604003906E+04, -.1298520629883E+04, .8479232177734E+03},
 {   27706, .8192136718750E+04, -.1298520629883E+04, .8377387695312E+03},
 {   27707, .8265989257812E+04, -.1298520629883E+04, .9771616210938E+03},
 {   27708, .8273507812500E+04, -.1298520629883E+04, .9759941406250E+03},
 {   27709, .8261101562500E+04, -.1298520629883E+04, .9659357299805E+03},
 {   27710, .8281041015625E+04, -.1298520629883E+04, .9747925415039E+03},
 {   27711, .8270179687500E+04, -.1298520629883E+04, .9623446655273E+03},
 {   27712, .8249150390625E+04, -.1298520629883E+04, .9532250976562E+03},
 {   27713, .8259626953125E+04, -.1298520629883E+04, .9480907592773E+03},
 {   27714, .8237828125000E+04, -.1298520629883E+04, .9388132324219E+03},
 {   27715, .8249514648438E+04, -.1298520629883E+04, .9327188110352E+03},
 {   27716, .8227693359375E+04, -.1298520629883E+04, .9227011718750E+03},
 {   27717, .8240147460938E+04, -.1298520629883E+04, .9162069702148E+03},
 {   27718, .8219160156250E+04, -.1298520629883E+04, .9053159790039E+03},
 {   27719, .8231900390625E+04, -.1298520629883E+04, .8988336791992E+03},
 {   27720, .8213457031250E+04, -.1298520629883E+04, .8871081542969E+03},
 {   27721, .8225500976562E+04, -.1298520629883E+04, .8809375000000E+03},
 {   27722, .8209118164062E+04, -.1298520629883E+04, .8689582519531E+03},
 {   27723, .8220036132812E+04, -.1298520629883E+04, .8630693969727E+03},
 {   27724, .8204579101562E+04, -.1298520629883E+04, .8506751098633E+03},
 {   27725, .8214421875000E+04, -.1298520629883E+04, .8451269531250E+03},
 {   27726, .8200695312500E+04, -.1298520629883E+04, .8324879760742E+03},
 {   27727, .8209242187500E+04, -.1298520629883E+04, .8272145385742E+03},
 {   27728, .8217823242188E+04, -.1285592041016E+04, .8219534912109E+03},
 {   27729, .8226375000000E+04, -.1285592041016E+04, .8166793823242E+03},
 {   27730, .8224255859375E+04, -.1285592041016E+04, .8395559082031E+03},
 {   27731, .8234124023438E+04, -.1285592041016E+04, .8339975585938E+03},
 {   27732, .8230948242188E+04, -.1285592041016E+04, .8571810913086E+03},
 {   27733, .8241867187500E+04, -.1285592041016E+04, .8512919311523E+03},
 {   27734, .8237553710938E+04, -.1285592041016E+04, .8747904052734E+03},
 {   27735, .8249620117188E+04, -.1285592041016E+04, .8686098022461E+03},
 {   27736, .8244624023438E+04, -.1285592041016E+04, .8923859863281E+03},
 {   27737, .8257364257812E+04, -.1285592041016E+04, .8859041748047E+03},
 {   27738, .8252626953125E+04, -.1285592041016E+04, .9097031250000E+03},
 {   27739, .8265114257812E+04, -.1285592041016E+04, .9032221679688E+03},
 {   27740, .8261179687500E+04, -.1285592041016E+04, .9266342773438E+03},
 {   27741, .8272856445312E+04, -.1285592041016E+04, .9205166625977E+03},
 {   27742, .8270110351562E+04, -.1285592041016E+04, .9429790649414E+03},
 {   27743, .8280608398438E+04, -.1285592041016E+04, .9378344116211E+03},
 {   27744, .8279260742188E+04, -.1285592041016E+04, .9587534790039E+03},
 {   27745, .8288351562500E+04, -.1285592041016E+04, .9551288452148E+03},
 {   27746, .8288560546875E+04, -.1285592041016E+04, .9736248168945E+03},
 {   27747, .8296103515625E+04, -.1285592041016E+04, .9724464721680E+03},
 {   27748, .8228200195312E+04, -.1285592041016E+04, .9634939575195E+03},
 {   27749, .8242957031250E+04, -.1285592041016E+04, .9731406250000E+03},
 {   27750, .8252025390625E+04, -.1285592041016E+04, .9695270996094E+03},
 {   27751, .8238676757812E+04, -.1285592041016E+04, .9583593139648E+03},
 {   27752, .8214510742188E+04, -.1285592041016E+04, .9510046386719E+03},
 {   27753, .8226164062500E+04, -.1285592041016E+04, .9448974609375E+03},
 {   27754, .8202726562500E+04, -.1285592041016E+04, .9356864624023E+03},
 {   27755, .8215204101562E+04, -.1285592041016E+04, .9291823730469E+03},
 {   27756, .8193721679688E+04, -.1285592041016E+04, .9182346801758E+03},
 {   27757, .8206434570312E+04, -.1285592041016E+04, .9117637939453E+03},
 {   27758, .8188780273438E+04, -.1285592041016E+04, .8997414550781E+03},
 {   27759, .8201414062500E+04, -.1285592041016E+04, .8932781372070E+03},
 {   27760, .8184340332031E+04, -.1285592041016E+04, .8823043212891E+03},
 {   27761, .8198213867188E+04, -.1285592041016E+04, .8748133544922E+03},
 {   27762, .8180007324219E+04, -.1285592041016E+04, .8645412597656E+03},
 {   27763, .8194736328125E+04, -.1285592041016E+04, .8562226562500E+03},
 {   27764, .8175604003906E+04, -.1285592041016E+04, .8479232177734E+03},
 {   27765, .8192136718750E+04, -.1285592041016E+04, .8377387695312E+03},
 {   27766, .8265989257812E+04, -.1285592041016E+04, .9771616210938E+03},
 {   27767, .8273507812500E+04, -.1285592041016E+04, .9759941406250E+03},
 {   27768, .8261101562500E+04, -.1285592041016E+04, .9659357299805E+03},
 {   27769, .8281041015625E+04, -.1285592041016E+04, .9747925415039E+03},
 {   27770, .8270179687500E+04, -.1285592041016E+04, .9623446655273E+03},
 {   27771, .8249150390625E+04, -.1285592041016E+04, .9532250976562E+03},
 {   27772, .8259626953125E+04, -.1285592041016E+04, .9480907592773E+03},
 {   27773, .8237828125000E+04, -.1285592041016E+04, .9388132324219E+03},
 {   27774, .8249514648438E+04, -.1285592041016E+04, .9327188110352E+03},
 {   27775, .8227693359375E+04, -.1285592041016E+04, .9227011718750E+03},
 {   27776, .8240147460938E+04, -.1285592041016E+04, .9162069702148E+03},
 {   27777, .8219160156250E+04, -.1285592041016E+04, .9053159790039E+03},
 {   27778, .8231900390625E+04, -.1285592041016E+04, .8988336791992E+03},
 {   27779, .8213457031250E+04, -.1285592041016E+04, .8871081542969E+03},
 {   27780, .8225500976562E+04, -.1285592041016E+04, .8809375000000E+03},
 {   27781, .8209118164062E+04, -.1285592041016E+04, .8689582519531E+03},
 {   27782, .8220036132812E+04, -.1285592041016E+04, .8630693969727E+03},
 {   27783, .8204579101562E+04, -.1285592041016E+04, .8506751098633E+03},
 {   27784, .8214421875000E+04, -.1285592041016E+04, .8451269531250E+03},
 {   27785, .8200695312500E+04, -.1285592041016E+04, .8324879760742E+03},
 {   27786, .8209242187500E+04, -.1285592041016E+04, .8272145385742E+03},
 {   27787, .8258444335938E+04, -.1298520629883E+04, .9783404541016E+03},
 {   27788, .8258444335938E+04, -.1285592041016E+04, .9783404541016E+03},
 {   27789, .8258444335938E+04, -.1311449218750E+04, .9783404541016E+03},
 {   27790, .8258444335938E+04, -.1324403198242E+04, .9783404541016E+03},
 {   27791, .8258444335938E+04, -.1337331787109E+04, .9783404541016E+03},
 {   27792, .8261017578125E+04, -.1354654663086E+04, .9771882324219E+03},
 {   27793, .8268274414062E+04, -.1369335815430E+04, .9739399414062E+03},
 {   27794, .8279170898438E+04, -.1379140258789E+04, .9690620117188E+03},
 {   27795, .8292014648438E+04, -.1382569213867E+04, .9633126220703E+03},
 {   27796, .8304833007812E+04, -.1379140258789E+04, .9575733032227E+03},
 {   27797, .8315731445312E+04, -.1369335815430E+04, .9526955566406E+03},
 {   27798, .8322986328125E+04, -.1354654663086E+04, .9494475097656E+03},
 {   27799, .8325802734375E+04, -.1337331787109E+04, .9477143554688E+03},
 {   27800, .8226993164062E+04, -.1234563476562E+04, .7976733398438E+03},
 {   27801, .8226340820312E+04, -.1229966064453E+04, .7980206909180E+03},
 {   27802, .8218631835938E+04, -.1233852294922E+04, .7993850708008E+03},
 {   27803, .8235999023438E+04, -.1233852294922E+04, .8138170166016E+03},
 {   27804, .8235304687500E+04, -.1229153320312E+04, .8141282348633E+03},
 {   27805, .8226375000000E+04, -.1233852294922E+04, .8166793823242E+03},
 {   27806, .8296393554688E+04, -.1229254882812E+04, .9510841674805E+03},
 {   27807, .8294467773438E+04, -.1225368652344E+04, .9519457397461E+03},
 {   27808, .8288351562500E+04, -.1233852294922E+04, .9551288452148E+03},
 {   27809, .8301472656250E+04, -.1230397827148E+04, .9678442382812E+03},
 {   27810, .8300013671875E+04, -.1227451416016E+04, .9684979248047E+03},
 {   27811, .8296103515625E+04, -.1233852294922E+04, .9724464721680E+03},
 {   27812, .8261293945312E+04, -.1227476806641E+04, .8641632080078E+03},
 {   27813, .8258625976562E+04, -.1222092041016E+04, .8653566894531E+03},
 {   27814, .8249620117188E+04, -.1233852294922E+04, .8686098022461E+03},
 {   27815, .8269662109375E+04, -.1227146606445E+04, .8811774902344E+03},
 {   27816, .8266831054688E+04, -.1221431640625E+04, .8824437866211E+03},
 {   27817, .8257364257812E+04, -.1233852294922E+04, .8859041748047E+03},
 {   27818, .8252650390625E+04, -.1228035644531E+04, .8474666748047E+03},
 {   27819, .8250214843750E+04, -.1223082641602E+04, .8485565185547E+03},
 {   27820, .8241867187500E+04, -.1233852294922E+04, .8512919311523E+03},
 {   27821, .8224715820312E+04, -.1226689453125E+04, .7987474365234E+03},
 {   27822, .8233333984375E+04, -.1225190917969E+04, .8150104370117E+03},
 {   27823, .8288285156250E+04, -.1223692260742E+04, .9348429565430E+03},
 {   27824, .8284833007812E+04, -.1220593505859E+04, .9363892211914E+03},
 {   27825, .8280608398438E+04, -.1233852294922E+04, .9378344116211E+03},
 {   27826, .8291593750000E+04, -.1222752441406E+04, .9532325439453E+03},
 {   27827, .8246553710938E+04, -.1219806030273E+04, .8501962890625E+03},
 {   27828, .8254616210938E+04, -.1218485229492E+04, .8671519165039E+03},
 {   27829, .8241808593750E+04, -.1224047851562E+04, .8316995239258E+03},
 {   27830, .8238468750000E+04, -.1221050659180E+04, .8331937866211E+03},
 {   27831, .8234552734375E+04, -.1233852294922E+04, .8349477539062E+03},
 {   27832, .8222283203125E+04, -.1224479614258E+04, .7998370361328E+03},
 {   27833, .8230389648438E+04, -.1222523925781E+04, .8163284301758E+03},
 {   27834, .8280775390625E+04, -.1219501220703E+04, .9382050781250E+03},
 {   27835, .8288185546875E+04, -.1221838012695E+04, .9547577514648E+03},
 {   27836, .8277778320312E+04, -.1218916992188E+04, .9192036132812E+03},
 {   27837, .8273188476562E+04, -.1217672485352E+04, .9212583618164E+03},
 {   27838, .8272856445312E+04, -.1233852294922E+04, .9205166625977E+03},
 {   27839, .8270339843750E+04, -.1217901000977E+04, .9017739868164E+03},
 {   27840, .8265448242188E+04, -.1216580200195E+04, .9039638061523E+03},
 {   27841, .8265114257812E+04, -.1233852294922E+04, .9032221679688E+03},
 {   27842, .8242241210938E+04, -.1218637695312E+04, .8521263427734E+03},
 {   27843, .8249911132812E+04, -.1217215209961E+04, .8692589721680E+03},
 {   27844, .8219406250000E+04, -.1223717651367E+04, .8011240844727E+03},
 {   27845, .8226912109375E+04, -.1221609497070E+04, .8178853759766E+03},
 {   27846, .8284777343750E+04, -.1222752441406E+04, .9562835693359E+03},
 {   27847, .8281879882812E+04, -.1225368652344E+04, .9575806274414E+03},
 {   27848, .8292709960938E+04, -.1225495605469E+04, .9717671508789E+03},
 {   27849, .8290530273438E+04, -.1227451416016E+04, .9727429199219E+03},
 {   27850, .8268596679688E+04, -.1218916992188E+04, .9233133544922E+03},
 {   27851, .8264705078125E+04, -.1222422241211E+04, .9250568237305E+03},
 {   27852, .8276693359375E+04, -.1220593505859E+04, .9400320434570E+03},
 {   27853, .8273239257812E+04, -.1223692260742E+04, .9415782470703E+03},
 {   27854, .8260531250000E+04, -.1217901000977E+04, .9061640625000E+03},
 {   27855, .8256380859375E+04, -.1221634887695E+04, .9080220336914E+03},
 {   27856, .8252667968750E+04, -.1217647094727E+04, .8887849121094E+03},
 {   27857, .8248448242188E+04, -.1221431640625E+04, .8906737060547E+03},
 {   27858, .8245180664062E+04, -.1218485229492E+04, .8713761596680E+03},
 {   27859, .8241169921875E+04, -.1222092041016E+04, .8731716308594E+03},
 {   27860, .8230610351562E+04, -.1221050659180E+04, .8367122802734E+03},
 {   27861, .8227273437500E+04, -.1224047851562E+04, .8382065429688E+03},
 {   27862, .8237905273438E+04, -.1219806030273E+04, .8540671386719E+03},
 {   27863, .8234244140625E+04, -.1223082641602E+04, .8557070922852E+03},
 {   27864, .8223435546875E+04, -.1222523925781E+04, .8194417724609E+03},
 {   27865, .8220466796875E+04, -.1225190917969E+04, .8207703247070E+03},
 {   27866, .8279955078125E+04, -.1229254882812E+04, .9584421386719E+03},
 {   27867, .8289070312500E+04, -.1230397827148E+04, .9733968505859E+03},
 {   27868, .8262106445312E+04, -.1227654663086E+04, .9262193603516E+03},
 {   27869, .8270921875000E+04, -.1228365844727E+04, .9426162109375E+03},
 {   27870, .8253599609375E+04, -.1227248291016E+04, .9092671508789E+03},
 {   27871, .8238504882812E+04, -.1227476806641E+04, .8743649902344E+03},
 {   27872, .8245620117188E+04, -.1227146606445E+04, .8919396972656E+03},
 {   27873, .8218496093750E+04, -.1229153320312E+04, .8216526489258E+03},
 {   27874, .8225045898438E+04, -.1228543701172E+04, .8392028198242E+03},
 {   27875, .8270110351562E+04, -.1233852294922E+04, .9429790649414E+03},
 {   27876, .8279260742188E+04, -.1233852294922E+04, .9587534790039E+03},
 {   27877, .8261179687500E+04, -.1233852294922E+04, .9266342773438E+03},
 {   27878, .8252626953125E+04, -.1233852294922E+04, .9097031250000E+03},
 {   27879, .8244624023438E+04, -.1233852294922E+04, .8923859863281E+03},
 {   27880, .8224255859375E+04, -.1233852294922E+04, .8395559082031E+03},
 {   27881, .8231809570312E+04, -.1228035644531E+04, .8567966918945E+03},
 {   27882, .8230948242188E+04, -.1233852294922E+04, .8571810913086E+03},
 {   27883, .8319657226562E+04, -.1233852294922E+04, .9531364746094E+03},
 {   27884, .8323265625000E+04, -.1233852294922E+04, .9376350708008E+03},
 {   27885, .8314540039062E+04, -.1233852294922E+04, .9420141601562E+03},
 {   27886, .8313777343750E+04, -.1233852294922E+04, .9579672851562E+03},
 {   27887, .8317617187500E+04, -.1220009277344E+04, .9540495605469E+03},
 {   27888, .8320552734375E+04, -.1215462646484E+04, .9388489990234E+03},
 {   27889, .8312499023438E+04, -.1220060058594E+04, .9429274902344E+03},
 {   27890, .8312247070312E+04, -.1223463623047E+04, .9586520385742E+03},
 {   27891, .8305812500000E+04, -.1233852294922E+04, .9463936767578E+03},
 {   27892, .8307863281250E+04, -.1233852294922E+04, .9627850341797E+03},
 {   27893, .8304443359375E+04, -.1224657470703E+04, .9470060424805E+03},
 {   27894, .8306842773438E+04, -.1226918090820E+04, .9632418212891E+03},
 {   27895, .8297085937500E+04, -.1233852294922E+04, .9507729492188E+03},
 {   27896, .8301983398438E+04, -.1233852294922E+04, .9676156616211E+03},
 {   27897, .8331836914062E+04, -.1233852294922E+04, .9336582031250E+03},
 {   27898, .8334635742188E+04, -.1233852294922E+04, .9158478393555E+03},
 {   27899, .8323827148438E+04, -.1233852294922E+04, .9202406005859E+03},
 {   27900, .8328617187500E+04, -.1210865234375E+04, .9347939453125E+03},
 {   27901, .8330577148438E+04, -.1206394897461E+04, .9176640014648E+03},
 {   27902, .8320581054688E+04, -.1211881225586E+04, .9216936035156E+03},
 {   27903, .8313020507812E+04, -.1233852294922E+04, .9246332397461E+03},
 {   27904, .8310584960938E+04, -.1217367675781E+04, .9257229614258E+03},
 {   27905, .8302213867188E+04, -.1233852294922E+04, .9290258789062E+03},
 {   27906, .8300589843750E+04, -.1222879394531E+04, .9297525024414E+03},
 {   27907, .8291416992188E+04, -.1233852294922E+04, .9334415893555E+03},
 {   27908, .8290605468750E+04, -.1228365844727E+04, .9338049926758E+03},
 {   27909, .8334459960938E+04, -.1233852294922E+04, .8973080444336E+03},
 {   27910, .8322119140625E+04, -.1233852294922E+04, .9019421386719E+03},
 {   27911, .8329893554688E+04, -.1202889648438E+04, .8993526000977E+03},
 {   27912, .8318479492188E+04, -.1209087280273E+04, .9035715332031E+03},
 {   27913, .8309812500000E+04, -.1233852294922E+04, .9065889282227E+03},
 {   27914, .8307077148438E+04, -.1215284912109E+04, .9078137207031E+03},
 {   27915, .8297481445312E+04, -.1233852294922E+04, .9112462158203E+03},
 {   27916, .8295674804688E+04, -.1221482421875E+04, .9120556030273E+03},
 {   27917, .8285173828125E+04, -.1233852294922E+04, .9158930664062E+03},
 {   27918, .8284267578125E+04, -.1227654663086E+04, .9162978515625E+03},
 {   27919, .8330861328125E+04, -.1233852294922E+04, .8783261108398E+03},
 {   27920, .8317699218750E+04, -.1233852294922E+04, .8833001098633E+03},
 {   27921, .8325967773438E+04, -.1200781494141E+04, .8805161132812E+03},
 {   27922, .8313802734375E+04, -.1207410888672E+04, .8850435791016E+03},
 {   27923, .8304546875000E+04, -.1233852294922E+04, .8882976684570E+03},
 {   27924, .8301624023438E+04, -.1214014892578E+04, .8896051635742E+03},
 {   27925, .8291406250000E+04, -.1233852294922E+04, .8932611083984E+03},
 {   27926, .8289459960938E+04, -.1220618896484E+04, .8941326904297E+03},
 {   27927, .8278243164062E+04, -.1233852294922E+04, .8982354736328E+03},
 {   27928, .8277269531250E+04, -.1227248291016E+04, .8986711425781E+03},
 {   27929, .8323682617188E+04, -.1233090209961E+04, .8600550537109E+03},
 {   27930, .8310549804688E+04, -.1233852294922E+04, .8651554565430E+03},
 {   27931, .8318907226562E+04, -.1200273437500E+04, .8621929931641E+03},
 {   27932, .8306584960938E+04, -.1206979003906E+04, .8669301147461E+03},
 {   27933, .8297263671875E+04, -.1233852294922E+04, .8703516235352E+03},
 {   27934, .8294272460938E+04, -.1213710083008E+04, .8716905517578E+03},
 {   27935, .8283943359375E+04, -.1233852294922E+04, .8755354003906E+03},
 {   27936, .8281973632812E+04, -.1220415649414E+04, .8764172973633E+03},
 {   27937, .8270659179688E+04, -.1233852294922E+04, .8807312622070E+03},
 {   27938, .8312752929688E+04, -.1233852294922E+04, .8442156982422E+03},
 {   27939, .8300107421875E+04, -.1233852294922E+04, .8490981445312E+03},
 {   27940, .8306543945312E+04, -.1202661010742E+04, .8460766601562E+03},
 {   27941, .8296350585938E+04, -.1208376098633E+04, .8507796630859E+03},
 {   27942, .8287481445312E+04, -.1233852294922E+04, .8539703369141E+03},
 {   27943, .8284675781250E+04, -.1214751464844E+04, .8552261962891E+03},
 {   27944, .8274857421875E+04, -.1233852294922E+04, .8588424682617E+03},
 {   27945, .8272980468750E+04, -.1221126831055E+04, .8596832885742E+03},
 {   27946, .8262245117188E+04, -.1233852294922E+04, .8637375488281E+03},
 {   27947, .8301894531250E+04, -.1234817504883E+04, .8301533813477E+03},
 {   27948, .8288465820312E+04, -.1233852294922E+04, .8344114990234E+03},
 {   27949, .8295840820312E+04, -.1204743896484E+04, .8321116943359E+03},
 {   27950, .8285033203125E+04, -.1210560424805E+04, .8359473876953E+03},
 {   27951, .8276834960938E+04, -.1233852294922E+04, .8386437377930E+03},
 {   27952, .8274264648438E+04, -.1216377075195E+04, .8397954101562E+03},
 {   27953, .8265170898438E+04, -.1233852294922E+04, .8428633422852E+03},
 {   27954, .8263456054688E+04, -.1222219116211E+04, .8436311645508E+03},
 {   27955, .8253508789062E+04, -.1233852294922E+04, .8470826416016E+03},
 {   27956, .8287624023438E+04, -.1233852294922E+04, .8158088378906E+03},
 {   27957, .8276917968750E+04, -.1233852294922E+04, .8194324951172E+03},
 {   27958, .8283729492188E+04, -.1207334716797E+04, .8175523681641E+03},
 {   27959, .8273788085938E+04, -.1212643310547E+04, .8208335571289E+03},
 {   27960, .8266223632812E+04, -.1233852294922E+04, .8230795288086E+03},
 {   27961, .8263880859375E+04, -.1217951904297E+04, .8241277465820E+03},
 {   27962, .8255527343750E+04, -.1233852294922E+04, .8267264404297E+03},
 {   27963, .8253951171875E+04, -.1223260498047E+04, .8274321289062E+03},
 {   27964, .8244819335938E+04, -.1233852294922E+04, .8303505859375E+03},
 {   27965, .8244032226562E+04, -.1228543701172E+04, .8307033081055E+03},
 {   27966, .8274489257812E+04, -.1233852294922E+04, .8023200073242E+03},
 {   27967, .8264861328125E+04, -.1233852294922E+04, .8051824340820E+03},
 {   27968, .8271011718750E+04, -.1210408081055E+04, .8038763427734E+03},
 {   27969, .8262078125000E+04, -.1215081665039E+04, .8064279174805E+03},
 {   27970, .8255244140625E+04, -.1233852294922E+04, .8080683593750E+03},
 {   27971, .8253158203125E+04, -.1219780639648E+04, .8090021972656E+03},
 {   27972, .8245617187500E+04, -.1233852294922E+04, .8109310302734E+03},
 {   27973, .8244226562500E+04, -.1224479614258E+04, .8115536499023E+03},
 {   27974, .8260131835938E+04, -.1233852294922E+04, .7913255004883E+03},
 {   27975, .8252373046875E+04, -.1233852294922E+04, .7928238525391E+03},
 {   27976, .8257257812500E+04, -.1214446655273E+04, .7926122436523E+03},
 {   27977, .8249536132812E+04, -.1218332885742E+04, .7939534301758E+03},
 {   27978, .8243520507812E+04, -.1233852294922E+04, .7945599975586E+03},
 {   27979, .8241803710938E+04, -.1222219116211E+04, .7953282470703E+03},
 {   27980, .8235220703125E+04, -.1233852294922E+04, .7961607666016E+03},
 {   27981, .8234083007812E+04, -.1226105224609E+04, .7966694946289E+03},
 {   27982, .8311797851562E+04, -.1208249023438E+04, .9566548461914E+03},
 {   27983, .8312831054688E+04, -.1199892456055E+04, .9423051147461E+03},
 {   27984, .8306704101562E+04, -.1208376098633E+04, .9455220336914E+03},
 {   27985, .8307865234375E+04, -.1214649902344E+04, .9606137084961E+03},
 {   27986, .8300597656250E+04, -.1216859619141E+04, .9487282104492E+03},
 {   27987, .8303945312500E+04, -.1221050659180E+04, .9645391235352E+03},
 {   27988, .8318948242188E+04, -.1191383422852E+04, .9391217041016E+03},
 {   27989, .8319054687500E+04, -.1183103027344E+04, .9228218383789E+03},
 {   27990, .8311354492188E+04, -.1193237670898E+04, .9258240966797E+03},
 {   27991, .8303675781250E+04, -.1203397705078E+04, .9288158569336E+03},
 {   27992, .8295976562500E+04, -.1213557617188E+04, .9318176879883E+03},
 {   27993, .8316885742188E+04, -.1176651489258E+04, .9051748657227E+03},
 {   27994, .8308071289062E+04, -.1188081420898E+04, .9082312622070E+03},
 {   27995, .8299264648438E+04, -.1199536865234E+04, .9113107299805E+03},
 {   27996, .8290457031250E+04, -.1210966796875E+04, .9143906860352E+03},
 {   27997, .8281649414062E+04, -.1222422241211E+04, .9174707641602E+03},
 {   27998, .8312082031250E+04, -.1172765258789E+04, .8867324218750E+03},
 {   27999, .8302695312500E+04, -.1184982666016E+04, .8900148925781E+03},
 {   28000, .8293299804688E+04, -.1197200073242E+04, .8933311767578E+03},
 {   28001, .8283896484375E+04, -.1209417480469E+04, .8966235351562E+03},
 {   28002, .8274486328125E+04, -.1221634887695E+04, .8999165039062E+03},
 {   28003, .8304788085938E+04, -.1171800048828E+04, .8685134277344E+03},
 {   28004, .8295294921875E+04, -.1184195312500E+04, .8719844360352E+03},
 {   28005, .8285811523438E+04, -.1196615844727E+04, .8754784545898E+03},
 {   28006, .8276316406250E+04, -.1209036499023E+04, .8789494628906E+03},
 {   28007, .8292644531250E+04, -.1176524414062E+04, .8507686767578E+03},
 {   28008, .8285662109375E+04, -.1186786010742E+04, .8555639648438E+03},
 {   28009, .8276656250000E+04, -.1198546264648E+04, .8588170166016E+03},
 {   28010, .8267624023438E+04, -.1210331909180E+04, .8620805664062E+03},
 {   28011, .8283599609375E+04, -.1180055053711E+04, .8375914306641E+03},
 {   28012, .8275251953125E+04, -.1190799316406E+04, .8403267211914E+03},
 {   28013, .8266913085938E+04, -.1201568847656E+04, .8430853271484E+03},
 {   28014, .8258564453125E+04, -.1212338500977E+04, .8458209838867E+03},
 {   28015, .8272577148438E+04, -.1184855712891E+04, .8225441894531E+03},
 {   28016, .8264884765625E+04, -.1194660034180E+04, .8248186645508E+03},
 {   28017, .8257203125000E+04, -.1204464477539E+04, .8271166992188E+03},
 {   28018, .8249498046875E+04, -.1214268798828E+04, .8294251098633E+03},
 {   28019, .8261181640625E+04, -.1190519897461E+04, .8082770385742E+03},
 {   28020, .8254197265625E+04, -.1199181274414E+04, .8099566650391E+03},
 {   28021, .8247246093750E+04, -.1207842651367E+04, .8116488647461E+03},
 {   28022, .8240284179688E+04, -.1216529418945E+04, .8133181152344E+03},
 {   28023, .8249121093750E+04, -.1198012817383E+04, .7962548828125E+03},
 {   28024, .8243024414062E+04, -.1205175659180E+04, .7968697509766E+03},
 {   28025, .8236912109375E+04, -.1212338500977E+04, .7975180053711E+03},
 {   28026, .8230815429688E+04, -.1219526611328E+04, .7981326293945E+03},
 {   28027, .8303105468750E+04, -.1200400390625E+04, .9605462646484E+03},
 {   28028, .8301262695312E+04, -.1189478515625E+04, .9474836425781E+03},
 {   28029, .8298033203125E+04, -.1200578247070E+04, .9494031982422E+03},
 {   28030, .8301349609375E+04, -.1208782470703E+04, .9635301513672E+03},
 {   28031, .8294801757812E+04, -.1211652709961E+04, .9513229980469E+03},
 {   28032, .8299588867188E+04, -.1217139038086E+04, .9664901733398E+03},
 {   28033, .8297834960938E+04, -.1225495605469E+04, .9694736938477E+03},
 {   28034, .8304505859375E+04, -.1178378662109E+04, .9455875244141E+03},
 {   28035, .8301784179688E+04, -.1167532836914E+04, .9305539550781E+03},
 {   28036, .8297537109375E+04, -.1180791625977E+04, .9320095214844E+03},
 {   28037, .8293314453125E+04, -.1194050415039E+04, .9334546508789E+03},
 {   28038, .8289066406250E+04, -.1207334716797E+04, .9349104614258E+03},
 {   28039, .8297437500000E+04, -.1159125488281E+04, .9138818359375E+03},
 {   28040, .8292515625000E+04, -.1174060668945E+04, .9151949462891E+03},
 {   28041, .8287602539062E+04, -.1189021240234E+04, .9165312500000E+03},
 {   28042, .8282692382812E+04, -.1203956420898E+04, .9178674316406E+03},
 {   28043, .8291310546875E+04, -.1154020019531E+04, .8960311279297E+03},
 {   28044, .8286053710938E+04, -.1169996704102E+04, .8974665527344E+03},
 {   28045, .8280827148438E+04, -.1185947875977E+04, .8989143676758E+03},
 {   28046, .8275594726562E+04, -.1201924438477E+04, .9003392333984E+03},
 {   28047, .8283692382812E+04, -.1152775390625E+04, .8779573974609E+03},
 {   28048, .8278417968750E+04, -.1168980712891E+04, .8795394287109E+03},
 {   28049, .8273152343750E+04, -.1185211303711E+04, .8811448974609E+03},
 {   28050, .8267879882812E+04, -.1201416503906E+04, .8827270507812E+03},
 {   28051, .8262614257812E+04, -.1217647094727E+04, .8843326416016E+03},
 {   28052, .8273019531250E+04, -.1159023803711E+04, .8592753295898E+03},
 {   28053, .8269643554688E+04, -.1172358886719E+04, .8627353515625E+03},
 {   28054, .8264645507812E+04, -.1187725830078E+04, .8641928710938E+03},
 {   28055, .8259625000000E+04, -.1203092895508E+04, .8656610107422E+03},
 {   28056, .8265308593750E+04, -.1163545043945E+04, .8457797241211E+03},
 {   28057, .8260622070312E+04, -.1177616699219E+04, .8468756103516E+03},
 {   28058, .8255922851562E+04, -.1191662841797E+04, .8480046997070E+03},
 {   28059, .8251237304688E+04, -.1205734497070E+04, .8491005859375E+03},
 {   28060, .8255910156250E+04, -.1169844238281E+04, .8300059814453E+03},
 {   28061, .8251554687500E+04, -.1182645874023E+04, .8307864379883E+03},
 {   28062, .8247188476562E+04, -.1195447509766E+04, .8316001586914E+03},
 {   28063, .8242844726562E+04, -.1208249023438E+04, .8324035034180E+03},
 {   28064, .8246437500000E+04, -.1177235717773E+04, .8148775024414E+03},
 {   28065, .8242420898438E+04, -.1188564086914E+04, .8152285766602E+03},
 {   28066, .8238415039062E+04, -.1199867065430E+04, .8156026611328E+03},
 {   28067, .8234395507812E+04, -.1211195434570E+04, .8159543457031E+03},
 {   28068, .8236926757812E+04, -.1187014648438E+04, .8017138671875E+03},
 {   28069, .8233262695312E+04, -.1196387207031E+04, .8012390136719E+03},
 {   28070, .8229609375000E+04, -.1205759887695E+04, .8007870483398E+03},
 {   28071, .8225947265625E+04, -.1215107055664E+04, .8003120727539E+03},
 {   28072, .8292835937500E+04, -.1197657226562E+04, .9651439208984E+03},
 {   28073, .8287657226562E+04, -.1185820800781E+04, .9535756835938E+03},
 {   28074, .8287833007812E+04, -.1197835083008E+04, .9539694824219E+03},
 {   28075, .8293655273438E+04, -.1206699707031E+04, .9669754638672E+03},
 {   28076, .8288006835938E+04, -.1209823852539E+04, .9543640747070E+03},
 {   28077, .8294463867188E+04, -.1215767456055E+04, .9687836303711E+03},
 {   28078, .8295285156250E+04, -.1224809814453E+04, .9706152343750E+03},
 {   28079, .8287490234375E+04, -.1173806640625E+04, .9532045288086E+03},
 {   28080, .8281427734375E+04, -.1162071899414E+04, .9396658325195E+03},
 {   28081, .8281261718750E+04, -.1176422851562E+04, .9392952880859E+03},
 {   28082, .8281096679688E+04, -.1190773803711E+04, .9389240722656E+03},
 {   28083, .8280930664062E+04, -.1205150268555E+04, .9385535278320E+03},
 {   28084, .8274485351562E+04, -.1152953247070E+04, .9241564331055E+03},
 {   28085, .8274155273438E+04, -.1169133056641E+04, .9234141845703E+03},
 {   28086, .8273832031250E+04, -.1185312866211E+04, .9226959228516E+03},
 {   28087, .8273512695312E+04, -.1201492675781E+04, .9219766845703E+03},
 {   28088, .8266805664062E+04, -.1147441406250E+04, .9070011596680E+03},
 {   28089, .8266462890625E+04, -.1164738891602E+04, .9062358398438E+03},
 {   28090, .8266130859375E+04, -.1182010864258E+04, .9054941406250E+03},
 {   28091, .8265789062500E+04, -.1199282836914E+04, .9047287597656E+03},
 {   28092, .8258793945312E+04, -.1146069824219E+04, .8891033935547E+03},
 {   28093, .8258502929688E+04, -.1163646606445E+04, .8884545288086E+03},
 {   28094, .8258223632812E+04, -.1181197998047E+04, .8878286132812E+03},
 {   28095, .8257931640625E+04, -.1198749511719E+04, .8871795654297E+03},
 {   28096, .8257654296875E+04, -.1216300903320E+04, .8865532226562E+03},
 {   28097, .8250617187500E+04, -.1152419799805E+04, .8708352050781E+03},
 {   28098, .8250772460938E+04, -.1167278808594E+04, .8711829833984E+03},
 {   28099, .8250479492188E+04, -.1183941284180E+04, .8705341796875E+03},
 {   28100, .8250189453125E+04, -.1200578247070E+04, .8698851318359E+03},
 {   28101, .8243722656250E+04, -.1157753906250E+04, .8554419555664E+03},
 {   28102, .8243350585938E+04, -.1172968505859E+04, .8546072387695E+03},
 {   28103, .8242986328125E+04, -.1188183105469E+04, .8537956542969E+03},
 {   28104, .8242613281250E+04, -.1203423095703E+04, .8529613647461E+03},
 {   28105, .8236274414062E+04, -.1164561035156E+04, .8387962646484E+03},
 {   28106, .8235836914062E+04, -.1178429443359E+04, .8378227539062E+03},
 {   28107, .8235412109375E+04, -.1192272460938E+04, .8368720703125E+03},
 {   28108, .8234987304688E+04, -.1206140869141E+04, .8359213867188E+03},
 {   28109, .8234552734375E+04, -.1220009277344E+04, .8349477539062E+03},
 {   28110, .8229051757812E+04, -.1172562011719E+04, .8226607055664E+03},
 {   28111, .8228512695312E+04, -.1184830322266E+04, .8214550781250E+03},
 {   28112, .8227981445312E+04, -.1197073120117E+04, .8202730712891E+03},
 {   28113, .8227441406250E+04, -.1209341308594E+04, .8190677490234E+03},
 {   28114, .8222553710938E+04, -.1183153808594E+04, .8081483154297E+03},
 {   28115, .8221766601562E+04, -.1193288452148E+04, .8063859252930E+03},
 {   28116, .8220987304688E+04, -.1203423095703E+04, .8046475830078E+03},
 {   28117, .8220196289062E+04, -.1213583007812E+04, .8028858642578E+03},
 {   28118, .8282563476562E+04, -.1200400390625E+04, .9697417602539E+03},
 {   28119, .8274024414062E+04, -.1189478515625E+04, .9596782226562E+03},
 {   28120, .8277609375000E+04, -.1200578247070E+04, .9585463867188E+03},
 {   28121, .8285933593750E+04, -.1208782470703E+04, .9704313354492E+03},
 {   28122, .8281193359375E+04, -.1211652709961E+04, .9574148559570E+03},
 {   28123, .8289317382812E+04, -.1217139038086E+04, .9710875244141E+03},
 {   28124, .8270451171875E+04, -.1178378662109E+04, .9608327026367E+03},
 {   28125, .8261049804688E+04, -.1167532836914E+04, .9487883300781E+03},
 {   28126, .8264966796875E+04, -.1180791625977E+04, .9465905761719E+03},
 {   28127, .8268857421875E+04, -.1194050415039E+04, .9444035644531E+03},
 {   28128, .8272771484375E+04, -.1207334716797E+04, .9422060546875E+03},
 {   28129, .8251535156250E+04, -.1159125488281E+04, .9344305419922E+03},
 {   28130, .8255793945312E+04, -.1174060668945E+04, .9316337890625E+03},
 {   28131, .8260061523438E+04, -.1189021240234E+04, .9288604736328E+03},
 {   28132, .8264329101562E+04, -.1203956420898E+04, .9260870361328E+03},
 {   28133, .8242277343750E+04, -.1154020019531E+04, .9179812011719E+03},
 {   28134, .8246852539062E+04, -.1169996704102E+04, .9150158081055E+03},
 {   28135, .8251411132812E+04, -.1185947875977E+04, .9120841064453E+03},
 {   28136, .8255981445312E+04, -.1201924438477E+04, .9091189575195E+03},
 {   28137, .8233871093750E+04, -.1152775390625E+04, .9002604370117E+03},
 {   28138, .8238566406250E+04, -.1168980712891E+04, .8973796386719E+03},
 {   28139, .8243269531250E+04, -.1185211303711E+04, .8945223388672E+03},
 {   28140, .8247965820312E+04, -.1201416503906E+04, .8916418457031E+03},
 {   28141, .8227785156250E+04, -.1157728515625E+04, .8825587158203E+03},
 {   28142, .8231877929688E+04, -.1172358886719E+04, .8796411132812E+03},
 {   28143, .8236315429688E+04, -.1187725830078E+04, .8768750000000E+03},
 {   28144, .8240731445312E+04, -.1203092895508E+04, .8741190795898E+03},
 {   28145, .8221863281250E+04, -.1162579833984E+04, .8652849731445E+03},
 {   28146, .8226055664062E+04, -.1177616699219E+04, .8623492431641E+03},
 {   28147, .8230029296875E+04, -.1191662841797E+04, .8595973510742E+03},
 {   28148, .8233965820312E+04, -.1205734497070E+04, .8568322753906E+03},
 {   28149, .8216178710938E+04, -.1168244018555E+04, .8479311523438E+03},
 {   28150, .8220098632812E+04, -.1182645874023E+04, .8448688964844E+03},
 {   28151, .8223613281250E+04, -.1195447509766E+04, .8421545410156E+03},
 {   28152, .8227105468750E+04, -.1208249023438E+04, .8394500732422E+03},
 {   28153, .8210952148438E+04, -.1174594116211E+04, .8310966186523E+03},
 {   28154, .8214578125000E+04, -.1188564086914E+04, .8276926879883E+03},
 {   28155, .8217525390625E+04, -.1199867065430E+04, .8249536132812E+03},
 {   28156, .8220463867188E+04, -.1211195434570E+04, .8221913452148E+03},
 {   28157, .8207259765625E+04, -.1183026855469E+04, .8158851318359E+03},
 {   28158, .8210243164062E+04, -.1196387207031E+04, .8115443725586E+03},
 {   28159, .8212340820312E+04, -.1205759887695E+04, .8085184936523E+03},
 {   28160, .8214423828125E+04, -.1215107055664E+04, .8054699707031E+03},
 {   28161, .8216511718750E+04, -.1224479614258E+04, .8024212646484E+03},
 {   28162, .8273847656250E+04, -.1208249023438E+04, .9736436767578E+03},
 {   28163, .8262456054688E+04, -.1199892456055E+04, .9648565673828E+03},
 {   28164, .8268936523438E+04, -.1208376098633E+04, .9624278564453E+03},
 {   28165, .8279419921875E+04, -.1214649902344E+04, .9733477172852E+03},
 {   28166, .8275395507812E+04, -.1216859619141E+04, .9600101318359E+03},
 {   28167, .8284958007812E+04, -.1221050659180E+04, .9730386962891E+03},
 {   28168, .8256004882812E+04, -.1191383422852E+04, .9672983398438E+03},
 {   28169, .8243779296875E+04, -.1183103027344E+04, .9565200805664E+03},
 {   28170, .8251149414062E+04, -.1193237670898E+04, .9527761230469E+03},
 {   28171, .8258515625000E+04, -.1203397705078E+04, .9490324096680E+03},
 {   28172, .8265862304688E+04, -.1213557617188E+04, .9452988891602E+03},
 {   28173, .8232062500000E+04, -.1176651489258E+04, .9431480102539E+03},
 {   28174, .8240213867188E+04, -.1188081420898E+04, .9386080932617E+03},
 {   28175, .8248377929688E+04, -.1199536865234E+04, .9340910644531E+03},
 {   28176, .8256541992188E+04, -.1210966796875E+04, .9295738525391E+03},
 {   28177, .8221505859375E+04, -.1172765258789E+04, .9272800292969E+03},
 {   28178, .8230229492188E+04, -.1184982666016E+04, .9224565429688E+03},
 {   28179, .8238938476562E+04, -.1197200073242E+04, .9176677246094E+03},
 {   28180, .8247659179688E+04, -.1209417480469E+04, .9128448486328E+03},
 {   28181, .8212774414062E+04, -.1171800048828E+04, .9097045288086E+03},
 {   28182, .8221687500000E+04, -.1184195312500E+04, .9049349975586E+03},
 {   28183, .8230611328125E+04, -.1196615844727E+04, .9001893310547E+03},
 {   28184, .8239525390625E+04, -.1209036499023E+04, .8954196166992E+03},
 {   28185, .8207232421875E+04, -.1174746459961E+04, .8914805908203E+03},
 {   28186, .8215859375000E+04, -.1186786010742E+04, .8868126831055E+03},
 {   28187, .8224306640625E+04, -.1198546264648E+04, .8822510986328E+03},
 {   28188, .8232732421875E+04, -.1210331909180E+04, .8776997680664E+03},
 {   28189, .8202790039062E+04, -.1178581909180E+04, .8739061279297E+03},
 {   28190, .8211426757812E+04, -.1190799316406E+04, .8688981933594E+03},
 {   28191, .8219040039062E+04, -.1201568847656E+04, .8645164794922E+03},
 {   28192, .8226640625000E+04, -.1212338500977E+04, .8601118774414E+03},
 {   28193, .8198243164062E+04, -.1182442626953E+04, .8560999145508E+03},
 {   28194, .8206765625000E+04, -.1194660034180E+04, .8508369750977E+03},
 {   28195, .8213621093750E+04, -.1204464477539E+04, .8466275634766E+03},
 {   28196, .8220450195312E+04, -.1214268798828E+04, .8424287719727E+03},
 {   28197, .8194153320312E+04, -.1186455810547E+04, .8389505004883E+03},
 {   28198, .8202799804688E+04, -.1199181274414E+04, .8329647827148E+03},
 {   28199, .8208691406250E+04, -.1207842651367E+04, .8289078369141E+03},
 {   28200, .8214576171875E+04, -.1216529418945E+04, .8248271484375E+03},
 {   28201, .8192092773438E+04, -.1191891479492E+04, .8235931396484E+03},
 {   28202, .8200482421875E+04, -.1205175659180E+04, .8159137573242E+03},
 {   28203, .8205037109375E+04, -.1212338500977E+04, .8117878417969E+03},
 {   28204, .8209554687500E+04, -.1219526611328E+04, .8076497802734E+03},
 {   28205, .8214078125000E+04, -.1226689453125E+04, .8035107421875E+03},
 {   28206, .8268029296875E+04, -.1220009277344E+04, .9762485351562E+03},
 {   28207, .8254735351562E+04, -.1215462646484E+04, .9683128662109E+03},
 {   28208, .8263142578125E+04, -.1220060058594E+04, .9650224609375E+03},
 {   28209, .8275036132812E+04, -.1223463623047E+04, .9753095703125E+03},
 {   28210, .8271548828125E+04, -.1224657470703E+04, .9617325439453E+03},
 {   28211, .8282060546875E+04, -.1226918090820E+04, .9743361816406E+03},
 {   28212, .8246340820312E+04, -.1210865234375E+04, .9716257324219E+03},
 {   28213, .8232257812500E+04, -.1206394897461E+04, .9616779785156E+03},
 {   28214, .8241920898438E+04, -.1211881225586E+04, .9569063720703E+03},
 {   28215, .8251584960938E+04, -.1217367675781E+04, .9521351928711E+03},
 {   28216, .8261248046875E+04, -.1222879394531E+04, .9473640747070E+03},
 {   28217, .8219055664062E+04, -.1202889648438E+04, .9489707641602E+03},
 {   28218, .8229805664062E+04, -.1209087280273E+04, .9432680053711E+03},
 {   28219, .8240587890625E+04, -.1215284912109E+04, .9375780029297E+03},
 {   28220, .8251347656250E+04, -.1221482421875E+04, .9318988037109E+03},
 {   28221, .8207619140625E+04, -.1200781494141E+04, .9334962158203E+03},
 {   28222, .8219101562500E+04, -.1207410888672E+04, .9274382324219E+03},
 {   28223, .8230614257812E+04, -.1214014892578E+04, .9213933715820E+03},
 {   28224, .8242096679688E+04, -.1220618896484E+04, .9153352661133E+03},
 {   28225, .8198657226562E+04, -.1200273437500E+04, .9160245361328E+03},
 {   28226, .8210399414062E+04, -.1206979003906E+04, .9099890136719E+03},
 {   28227, .8222150390625E+04, -.1213710083008E+04, .9039768066406E+03},
 {   28228, .8233869140625E+04, -.1220415649414E+04, .8979519653320E+03},
 {   28229, .8193670898438E+04, -.1201772094727E+04, .8975519409180E+03},
 {   28230, .8205168945312E+04, -.1208376098633E+04, .8915972900391E+03},
 {   28231, .8216262695312E+04, -.1214751464844E+04, .8858524780273E+03},
 {   28232, .8227377929688E+04, -.1221126831055E+04, .8800968627930E+03},
 {   28233, .8189549804688E+04, -.1203550048828E+04, .8798889160156E+03},
 {   28234, .8201645507812E+04, -.1210560424805E+04, .8732775878906E+03},
 {   28235, .8211689453125E+04, -.1216377075195E+04, .8678062744141E+03},
 {   28236, .8221750976562E+04, -.1222219116211E+04, .8623015747070E+03},
 {   28237, .8185425781250E+04, -.1205378906250E+04, .8619762573242E+03},
 {   28238, .8197863281250E+04, -.1212643310547E+04, .8548222656250E+03},
 {   28239, .8206919921875E+04, -.1217951904297E+04, .8496271362305E+03},
 {   28240, .8216000000000E+04, -.1223260498047E+04, .8444213867188E+03},
 {   28241, .8181551269531E+04, -.1207106079102E+04, .8449262695312E+03},
 {   28242, .8194917968750E+04, -.1215081665039E+04, .8364933471680E+03},
 {   28243, .8202780273438E+04, -.1219780639648E+04, .8315543212891E+03},
 {   28244, .8210632812500E+04, -.1224479614258E+04, .8265918579102E+03},
 {   28245, .8179223144531E+04, -.1209061889648E+04, .8304671630859E+03},
 {   28246, .8193967773438E+04, -.1218332885742E+04, .8188298339844E+03},
 {   28247, .8200144531250E+04, -.1222219116211E+04, .8139776000977E+03},
 {   28248, .8206290039062E+04, -.1226105224609E+04, .8091124877930E+03},
 {   28249, .8212455078125E+04, -.1229966064453E+04, .8042374877930E+03},
 {   28250, .8265989257812E+04, -.1233852294922E+04, .9771616210938E+03},
 {   28251, .8252025390625E+04, -.1233852294922E+04, .9695270996094E+03},
 {   28252, .8261101562500E+04, -.1233852294922E+04, .9659357299805E+03},
 {   28253, .8273507812500E+04, -.1233852294922E+04, .9759941406250E+03},
 {   28254, .8270179687500E+04, -.1233852294922E+04, .9623446655273E+03},
 {   28255, .8281041015625E+04, -.1233852294922E+04, .9747925415039E+03},
 {   28256, .8288560546875E+04, -.1233852294922E+04, .9736248168945E+03},
 {   28257, .8242957031250E+04, -.1233852294922E+04, .9731406250000E+03},
 {   28258, .8228200195312E+04, -.1233852294922E+04, .9634939575195E+03},
 {   28259, .8238676757812E+04, -.1233852294922E+04, .9583593139648E+03},
 {   28260, .8249150390625E+04, -.1233852294922E+04, .9532250976562E+03},
 {   28261, .8259626953125E+04, -.1233852294922E+04, .9480907592773E+03},
 {   28262, .8214510742188E+04, -.1233852294922E+04, .9510046386719E+03},
 {   28263, .8226164062500E+04, -.1233852294922E+04, .9448974609375E+03},
 {   28264, .8237828125000E+04, -.1233852294922E+04, .9388132324219E+03},
 {   28265, .8249514648438E+04, -.1233852294922E+04, .9327188110352E+03},
 {   28266, .8202726562500E+04, -.1233852294922E+04, .9356864624023E+03},
 {   28267, .8215204101562E+04, -.1233852294922E+04, .9291823730469E+03},
 {   28268, .8227693359375E+04, -.1233852294922E+04, .9227011718750E+03},
 {   28269, .8240147460938E+04, -.1233852294922E+04, .9162069702148E+03},
 {   28270, .8193721679688E+04, -.1233852294922E+04, .9182346801758E+03},
 {   28271, .8206434570312E+04, -.1233852294922E+04, .9117637939453E+03},
 {   28272, .8219160156250E+04, -.1233852294922E+04, .9053159790039E+03},
 {   28273, .8231900390625E+04, -.1233852294922E+04, .8988336791992E+03},
 {   28274, .8188780273438E+04, -.1233852294922E+04, .8997414550781E+03},
 {   28275, .8201414062500E+04, -.1233852294922E+04, .8932781372070E+03},
 {   28276, .8213457031250E+04, -.1233852294922E+04, .8871081542969E+03},
 {   28277, .8225500976562E+04, -.1233852294922E+04, .8809375000000E+03},
 {   28278, .8237553710938E+04, -.1233852294922E+04, .8747904052734E+03},
 {   28279, .8184340332031E+04, -.1233852294922E+04, .8823043212891E+03},
 {   28280, .8198213867188E+04, -.1233852294922E+04, .8748133544922E+03},
 {   28281, .8209118164062E+04, -.1233852294922E+04, .8689582519531E+03},
 {   28282, .8220036132812E+04, -.1233852294922E+04, .8630693969727E+03},
 {   28283, .8180007324219E+04, -.1233852294922E+04, .8645412597656E+03},
 {   28284, .8194736328125E+04, -.1233852294922E+04, .8562226562500E+03},
 {   28285, .8204579101562E+04, -.1233852294922E+04, .8506751098633E+03},
 {   28286, .8214421875000E+04, -.1233852294922E+04, .8451269531250E+03},
 {   28287, .8175604003906E+04, -.1233852294922E+04, .8479232177734E+03},
 {   28288, .8192136718750E+04, -.1233852294922E+04, .8377387695312E+03},
 {   28289, .8200695312500E+04, -.1233852294922E+04, .8324879760742E+03},
 {   28290, .8209242187500E+04, -.1233852294922E+04, .8272145385742E+03},
 {   28291, .8217823242188E+04, -.1233852294922E+04, .8219534912109E+03},
 {   28292, .8171235839844E+04, -.1233852294922E+04, .8356853637695E+03},
 {   28293, .8191673828125E+04, -.1233852294922E+04, .8198571166992E+03},
 {   28294, .8198428710938E+04, -.1233852294922E+04, .8147458496094E+03},
 {   28295, .8205153320312E+04, -.1233852294922E+04, .8096209716797E+03},
 {   28296, .8211873046875E+04, -.1233852294922E+04, .8044968872070E+03},
 {   28297, .8218631835938E+04, -.1246780883789E+04, .7993850708008E+03},
 {   28298, .8211873046875E+04, -.1246780883789E+04, .8044968872070E+03},
 {   28299, .8209442382812E+04, -.1233852294922E+04, .7788684082031E+03},
 {   28300, .8209442382812E+04, -.1246780883789E+04, .7788684082031E+03},
 {   28301, .8201913085938E+04, -.1246780883789E+04, .7822408447266E+03},
 {   28302, .8201913085938E+04, -.1233852294922E+04, .7822408447266E+03},
 {   28303, .8191673828125E+04, -.1246780883789E+04, .8198571166992E+03},
 {   28304, .8171235839844E+04, -.1246806274414E+04, .8356853637695E+03},
 {   28305, .8179353515625E+04, -.1233852294922E+04, .7923390502930E+03},
 {   28306, .8179353515625E+04, -.1246780883789E+04, .7923390502930E+03},
 {   28307, .8159024902344E+04, -.1246780883789E+04, .8014402465820E+03},
 {   28308, .8159024902344E+04, -.1233852294922E+04, .8014402465820E+03},
 {   28309, .8198428710938E+04, -.1246780883789E+04, .8147458496094E+03},
 {   28310, .8186889160156E+04, -.1233852294922E+04, .7889658813477E+03},
 {   28311, .8186889160156E+04, -.1246780883789E+04, .7889658813477E+03},
 {   28312, .8205153320312E+04, -.1246780883789E+04, .8096209716797E+03},
 {   28313, .8194401367188E+04, -.1233852294922E+04, .7856035156250E+03},
 {   28314, .8194401367188E+04, -.1246780883789E+04, .7856035156250E+03},
 {   28315, .8218631835938E+04, -.1259734863281E+04, .7993850708008E+03},
 {   28316, .8211873046875E+04, -.1259734863281E+04, .8044968872070E+03},
 {   28317, .8209442382812E+04, -.1259734863281E+04, .7788684082031E+03},
 {   28318, .8201913085938E+04, -.1259734863281E+04, .7822408447266E+03},
 {   28319, .8191673828125E+04, -.1259734863281E+04, .8198571166992E+03},
 {   28320, .8171235839844E+04, -.1259734863281E+04, .8356853637695E+03},
 {   28321, .8179353515625E+04, -.1259734863281E+04, .7923390502930E+03},
 {   28322, .8159024902344E+04, -.1259734863281E+04, .8014402465820E+03},
 {   28323, .8198428710938E+04, -.1259734863281E+04, .8147458496094E+03},
 {   28324, .8186889160156E+04, -.1259734863281E+04, .7889658813477E+03},
 {   28325, .8205153320312E+04, -.1259734863281E+04, .8096209716797E+03},
 {   28326, .8194401367188E+04, -.1259734863281E+04, .7856035156250E+03},
 {   28327, .8218631835938E+04, -.1272663452148E+04, .7993850708008E+03},
 {   28328, .8211873046875E+04, -.1272663452148E+04, .8044968872070E+03},
 {   28329, .8209442382812E+04, -.1272663452148E+04, .7788684082031E+03},
 {   28330, .8201913085938E+04, -.1272663452148E+04, .7822408447266E+03},
 {   28331, .8191673828125E+04, -.1272663452148E+04, .8198571166992E+03},
 {   28332, .8171235839844E+04, -.1272663452148E+04, .8356853637695E+03},
 {   28333, .8179353515625E+04, -.1272663452148E+04, .7923390502930E+03},
 {   28334, .8159024902344E+04, -.1272663452148E+04, .8014402465820E+03},
 {   28335, .8198428710938E+04, -.1272663452148E+04, .8147458496094E+03},
 {   28336, .8186889160156E+04, -.1272663452148E+04, .7889658813477E+03},
 {   28337, .8205153320312E+04, -.1272663452148E+04, .8096209716797E+03},
 {   28338, .8194401367188E+04, -.1272663452148E+04, .7856035156250E+03},
 {   28339, .8226375000000E+04, -.1246780883789E+04, .8166793823242E+03},
 {   28340, .8217823242188E+04, -.1246780883789E+04, .8219534912109E+03},
 {   28341, .8234124023438E+04, -.1246780883789E+04, .8339975585938E+03},
 {   28342, .8224255859375E+04, -.1246780883789E+04, .8395559082031E+03},
 {   28343, .8241867187500E+04, -.1246780883789E+04, .8512919311523E+03},
 {   28344, .8230948242188E+04, -.1246780883789E+04, .8571810913086E+03},
 {   28345, .8249620117188E+04, -.1246780883789E+04, .8686098022461E+03},
 {   28346, .8237553710938E+04, -.1246780883789E+04, .8747904052734E+03},
 {   28347, .8257364257812E+04, -.1246780883789E+04, .8859041748047E+03},
 {   28348, .8244624023438E+04, -.1246780883789E+04, .8923859863281E+03},
 {   28349, .8265114257812E+04, -.1246780883789E+04, .9032221679688E+03},
 {   28350, .8252626953125E+04, -.1246780883789E+04, .9097031250000E+03},
 {   28351, .8272856445312E+04, -.1246780883789E+04, .9205166625977E+03},
 {   28352, .8261179687500E+04, -.1246780883789E+04, .9266342773438E+03},
 {   28353, .8280608398438E+04, -.1246780883789E+04, .9378344116211E+03},
 {   28354, .8270110351562E+04, -.1246780883789E+04, .9429790649414E+03},
 {   28355, .8288351562500E+04, -.1246780883789E+04, .9551288452148E+03},
 {   28356, .8279260742188E+04, -.1246780883789E+04, .9587534790039E+03},
 {   28357, .8296103515625E+04, -.1246780883789E+04, .9724464721680E+03},
 {   28358, .8288560546875E+04, -.1246780883789E+04, .9736248168945E+03},
 {   28359, .8228200195312E+04, -.1246780883789E+04, .9634939575195E+03},
 {   28360, .8238676757812E+04, -.1246780883789E+04, .9583593139648E+03},
 {   28361, .8252025390625E+04, -.1246780883789E+04, .9695270996094E+03},
 {   28362, .8242957031250E+04, -.1246780883789E+04, .9731406250000E+03},
 {   28363, .8214510742188E+04, -.1246780883789E+04, .9510046386719E+03},
 {   28364, .8226164062500E+04, -.1246780883789E+04, .9448974609375E+03},
 {   28365, .8202726562500E+04, -.1246780883789E+04, .9356864624023E+03},
 {   28366, .8215204101562E+04, -.1246780883789E+04, .9291823730469E+03},
 {   28367, .8193721679688E+04, -.1246780883789E+04, .9182346801758E+03},
 {   28368, .8206434570312E+04, -.1246780883789E+04, .9117637939453E+03},
 {   28369, .8188780273438E+04, -.1246780883789E+04, .8997414550781E+03},
 {   28370, .8201414062500E+04, -.1246780883789E+04, .8932781372070E+03},
 {   28371, .8184340332031E+04, -.1246780883789E+04, .8823043212891E+03},
 {   28372, .8198213867188E+04, -.1246780883789E+04, .8748133544922E+03},
 {   28373, .8180007324219E+04, -.1246780883789E+04, .8645412597656E+03},
 {   28374, .8194736328125E+04, -.1246780883789E+04, .8562226562500E+03},
 {   28375, .8175604003906E+04, -.1246780883789E+04, .8479232177734E+03},
 {   28376, .8192136718750E+04, -.1246780883789E+04, .8377387695312E+03},
 {   28377, .8261101562500E+04, -.1246780883789E+04, .9659357299805E+03},
 {   28378, .8273507812500E+04, -.1246780883789E+04, .9759941406250E+03},
 {   28379, .8265989257812E+04, -.1246780883789E+04, .9771616210938E+03},
 {   28380, .8270179687500E+04, -.1246780883789E+04, .9623446655273E+03},
 {   28381, .8281041015625E+04, -.1246780883789E+04, .9747925415039E+03},
 {   28382, .8249150390625E+04, -.1246780883789E+04, .9532250976562E+03},
 {   28383, .8259626953125E+04, -.1246780883789E+04, .9480907592773E+03},
 {   28384, .8237828125000E+04, -.1246780883789E+04, .9388132324219E+03},
 {   28385, .8249514648438E+04, -.1246780883789E+04, .9327188110352E+03},
 {   28386, .8227693359375E+04, -.1246780883789E+04, .9227011718750E+03},
 {   28387, .8240147460938E+04, -.1246780883789E+04, .9162069702148E+03},
 {   28388, .8219160156250E+04, -.1246780883789E+04, .9053159790039E+03},
 {   28389, .8231900390625E+04, -.1246780883789E+04, .8988336791992E+03},
 {   28390, .8213457031250E+04, -.1246780883789E+04, .8871081542969E+03},
 {   28391, .8225500976562E+04, -.1246780883789E+04, .8809375000000E+03},
 {   28392, .8209118164062E+04, -.1246780883789E+04, .8689582519531E+03},
 {   28393, .8220036132812E+04, -.1246780883789E+04, .8630693969727E+03},
 {   28394, .8204579101562E+04, -.1246780883789E+04, .8506751098633E+03},
 {   28395, .8214421875000E+04, -.1246780883789E+04, .8451269531250E+03},
 {   28396, .8200695312500E+04, -.1246780883789E+04, .8324879760742E+03},
 {   28397, .8209242187500E+04, -.1246780883789E+04, .8272145385742E+03},
 {   28398, .8226375000000E+04, -.1259734863281E+04, .8166793823242E+03},
 {   28399, .8217823242188E+04, -.1259734863281E+04, .8219534912109E+03},
 {   28400, .8234124023438E+04, -.1259734863281E+04, .8339975585938E+03},
 {   28401, .8224255859375E+04, -.1259734863281E+04, .8395559082031E+03},
 {   28402, .8241867187500E+04, -.1259734863281E+04, .8512919311523E+03},
 {   28403, .8230948242188E+04, -.1259734863281E+04, .8571810913086E+03},
 {   28404, .8249620117188E+04, -.1259734863281E+04, .8686098022461E+03},
 {   28405, .8237553710938E+04, -.1259734863281E+04, .8747904052734E+03},
 {   28406, .8257364257812E+04, -.1259734863281E+04, .8859041748047E+03},
 {   28407, .8244624023438E+04, -.1259734863281E+04, .8923859863281E+03},
 {   28408, .8265114257812E+04, -.1259734863281E+04, .9032221679688E+03},
 {   28409, .8252626953125E+04, -.1259734863281E+04, .9097031250000E+03},
 {   28410, .8272856445312E+04, -.1259734863281E+04, .9205166625977E+03},
 {   28411, .8261179687500E+04, -.1259734863281E+04, .9266342773438E+03},
 {   28412, .8280608398438E+04, -.1259734863281E+04, .9378344116211E+03},
 {   28413, .8270110351562E+04, -.1259734863281E+04, .9429790649414E+03},
 {   28414, .8288351562500E+04, -.1259734863281E+04, .9551288452148E+03},
 {   28415, .8279260742188E+04, -.1259734863281E+04, .9587534790039E+03},
 {   28416, .8296103515625E+04, -.1259734863281E+04, .9724464721680E+03},
 {   28417, .8288560546875E+04, -.1259734863281E+04, .9736248168945E+03},
 {   28418, .8228200195312E+04, -.1259734863281E+04, .9634939575195E+03},
 {   28419, .8238676757812E+04, -.1259734863281E+04, .9583593139648E+03},
 {   28420, .8252025390625E+04, -.1259734863281E+04, .9695270996094E+03},
 {   28421, .8242957031250E+04, -.1259734863281E+04, .9731406250000E+03},
 {   28422, .8214510742188E+04, -.1259734863281E+04, .9510046386719E+03},
 {   28423, .8226164062500E+04, -.1259734863281E+04, .9448974609375E+03},
 {   28424, .8202726562500E+04, -.1259734863281E+04, .9356864624023E+03},
 {   28425, .8215204101562E+04, -.1259734863281E+04, .9291823730469E+03},
 {   28426, .8193721679688E+04, -.1259734863281E+04, .9182346801758E+03},
 {   28427, .8206434570312E+04, -.1259734863281E+04, .9117637939453E+03},
 {   28428, .8188780273438E+04, -.1259734863281E+04, .8997414550781E+03},
 {   28429, .8201414062500E+04, -.1259734863281E+04, .8932781372070E+03},
 {   28430, .8184340332031E+04, -.1259734863281E+04, .8823043212891E+03},
 {   28431, .8198213867188E+04, -.1259734863281E+04, .8748133544922E+03},
 {   28432, .8180007324219E+04, -.1259734863281E+04, .8645412597656E+03},
 {   28433, .8194736328125E+04, -.1259734863281E+04, .8562226562500E+03},
 {   28434, .8175604003906E+04, -.1259734863281E+04, .8479232177734E+03},
 {   28435, .8192136718750E+04, -.1259734863281E+04, .8377387695312E+03},
 {   28436, .8261101562500E+04, -.1259734863281E+04, .9659357299805E+03},
 {   28437, .8273507812500E+04, -.1259734863281E+04, .9759941406250E+03},
 {   28438, .8265989257812E+04, -.1259734863281E+04, .9771616210938E+03},
 {   28439, .8270179687500E+04, -.1259734863281E+04, .9623446655273E+03},
 {   28440, .8281041015625E+04, -.1259734863281E+04, .9747925415039E+03},
 {   28441, .8249150390625E+04, -.1259734863281E+04, .9532250976562E+03},
 {   28442, .8259626953125E+04, -.1259734863281E+04, .9480907592773E+03},
 {   28443, .8237828125000E+04, -.1259734863281E+04, .9388132324219E+03},
 {   28444, .8249514648438E+04, -.1259734863281E+04, .9327188110352E+03},
 {   28445, .8227693359375E+04, -.1259734863281E+04, .9227011718750E+03},
 {   28446, .8240147460938E+04, -.1259734863281E+04, .9162069702148E+03},
 {   28447, .8219160156250E+04, -.1259734863281E+04, .9053159790039E+03},
 {   28448, .8231900390625E+04, -.1259734863281E+04, .8988336791992E+03},
 {   28449, .8213457031250E+04, -.1259734863281E+04, .8871081542969E+03},
 {   28450, .8225500976562E+04, -.1259734863281E+04, .8809375000000E+03},
 {   28451, .8209118164062E+04, -.1259734863281E+04, .8689582519531E+03},
 {   28452, .8220036132812E+04, -.1259734863281E+04, .8630693969727E+03},
 {   28453, .8204579101562E+04, -.1259734863281E+04, .8506751098633E+03},
 {   28454, .8214421875000E+04, -.1259734863281E+04, .8451269531250E+03},
 {   28455, .8200695312500E+04, -.1259734863281E+04, .8324879760742E+03},
 {   28456, .8209242187500E+04, -.1259734863281E+04, .8272145385742E+03},
 {   28457, .8226375000000E+04, -.1272663452148E+04, .8166793823242E+03},
 {   28458, .8217823242188E+04, -.1272663452148E+04, .8219534912109E+03},
 {   28459, .8234124023438E+04, -.1272663452148E+04, .8339975585938E+03},
 {   28460, .8224255859375E+04, -.1272663452148E+04, .8395559082031E+03},
 {   28461, .8241867187500E+04, -.1272663452148E+04, .8512919311523E+03},
 {   28462, .8230948242188E+04, -.1272663452148E+04, .8571810913086E+03},
 {   28463, .8249620117188E+04, -.1272663452148E+04, .8686098022461E+03},
 {   28464, .8237553710938E+04, -.1272663452148E+04, .8747904052734E+03},
 {   28465, .8257364257812E+04, -.1272663452148E+04, .8859041748047E+03},
 {   28466, .8244624023438E+04, -.1272663452148E+04, .8923859863281E+03},
 {   28467, .8265114257812E+04, -.1272663452148E+04, .9032221679688E+03},
 {   28468, .8252626953125E+04, -.1272663452148E+04, .9097031250000E+03},
 {   28469, .8272856445312E+04, -.1272663452148E+04, .9205166625977E+03},
 {   28470, .8261179687500E+04, -.1272663452148E+04, .9266342773438E+03},
 {   28471, .8280608398438E+04, -.1272663452148E+04, .9378344116211E+03},
 {   28472, .8270110351562E+04, -.1272663452148E+04, .9429790649414E+03},
 {   28473, .8288351562500E+04, -.1272663452148E+04, .9551288452148E+03},
 {   28474, .8279260742188E+04, -.1272663452148E+04, .9587534790039E+03},
 {   28475, .8296103515625E+04, -.1272663452148E+04, .9724464721680E+03},
 {   28476, .8288560546875E+04, -.1272663452148E+04, .9736248168945E+03},
 {   28477, .8228200195312E+04, -.1272663452148E+04, .9634939575195E+03},
 {   28478, .8238676757812E+04, -.1272663452148E+04, .9583593139648E+03},
 {   28479, .8252025390625E+04, -.1272663452148E+04, .9695270996094E+03},
 {   28480, .8242957031250E+04, -.1272663452148E+04, .9731406250000E+03},
 {   28481, .8214510742188E+04, -.1272663452148E+04, .9510046386719E+03},
 {   28482, .8226164062500E+04, -.1272663452148E+04, .9448974609375E+03},
 {   28483, .8202726562500E+04, -.1272663452148E+04, .9356864624023E+03},
 {   28484, .8215204101562E+04, -.1272663452148E+04, .9291823730469E+03},
 {   28485, .8193721679688E+04, -.1272663452148E+04, .9182346801758E+03},
 {   28486, .8206434570312E+04, -.1272663452148E+04, .9117637939453E+03},
 {   28487, .8188780273438E+04, -.1272663452148E+04, .8997414550781E+03},
 {   28488, .8201414062500E+04, -.1272663452148E+04, .8932781372070E+03},
 {   28489, .8184340332031E+04, -.1272663452148E+04, .8823043212891E+03},
 {   28490, .8198213867188E+04, -.1272663452148E+04, .8748133544922E+03},
 {   28491, .8180007324219E+04, -.1272663452148E+04, .8645412597656E+03},
 {   28492, .8194736328125E+04, -.1272663452148E+04, .8562226562500E+03},
 {   28493, .8175604003906E+04, -.1272663452148E+04, .8479232177734E+03},
 {   28494, .8192136718750E+04, -.1272663452148E+04, .8377387695312E+03},
 {   28495, .8261101562500E+04, -.1272663452148E+04, .9659357299805E+03},
 {   28496, .8273507812500E+04, -.1272663452148E+04, .9759941406250E+03},
 {   28497, .8265989257812E+04, -.1272663452148E+04, .9771616210938E+03},
 {   28498, .8270179687500E+04, -.1272663452148E+04, .9623446655273E+03},
 {   28499, .8281041015625E+04, -.1272663452148E+04, .9747925415039E+03},
 {   28500, .8249150390625E+04, -.1272663452148E+04, .9532250976562E+03},
 {   28501, .8259626953125E+04, -.1272663452148E+04, .9480907592773E+03},
 {   28502, .8237828125000E+04, -.1272663452148E+04, .9388132324219E+03},
 {   28503, .8249514648438E+04, -.1272663452148E+04, .9327188110352E+03},
 {   28504, .8227693359375E+04, -.1272663452148E+04, .9227011718750E+03},
 {   28505, .8240147460938E+04, -.1272663452148E+04, .9162069702148E+03},
 {   28506, .8219160156250E+04, -.1272663452148E+04, .9053159790039E+03},
 {   28507, .8231900390625E+04, -.1272663452148E+04, .8988336791992E+03},
 {   28508, .8213457031250E+04, -.1272663452148E+04, .8871081542969E+03},
 {   28509, .8225500976562E+04, -.1272663452148E+04, .8809375000000E+03},
 {   28510, .8209118164062E+04, -.1272663452148E+04, .8689582519531E+03},
 {   28511, .8220036132812E+04, -.1272663452148E+04, .8630693969727E+03},
 {   28512, .8204579101562E+04, -.1272663452148E+04, .8506751098633E+03},
 {   28513, .8214421875000E+04, -.1272663452148E+04, .8451269531250E+03},
 {   28514, .8200695312500E+04, -.1272663452148E+04, .8324879760742E+03},
 {   28515, .8209242187500E+04, -.1272663452148E+04, .8272145385742E+03},
 {   28516, .8258444335938E+04, -.1272663452148E+04, .9783404541016E+03},
 {   28517, .8258444335938E+04, -.1259734863281E+04, .9783404541016E+03},
 {   28518, .8258444335938E+04, -.1246780883789E+04, .9783404541016E+03},
 {   28519, .8258444335938E+04, -.1233852294922E+04, .9783404541016E+03},
 {   28520, .8261017578125E+04, -.1216529418945E+04, .9771882324219E+03},
 {   28521, .8268274414062E+04, -.1201848266602E+04, .9739399414062E+03},
 {   28522, .8279170898438E+04, -.1192043823242E+04, .9690620117188E+03},
 {   28523, .8292014648438E+04, -.1188614868164E+04, .9633126220703E+03},
 {   28524, .8304833007812E+04, -.1192043823242E+04, .9575733032227E+03},
 {   28525, .8315731445312E+04, -.1201848266602E+04, .9526955566406E+03},
 {   28526, .8322986328125E+04, -.1216529418945E+04, .9494475097656E+03},
 {   28527, .8325802734375E+04, -.1233852294922E+04, .9477143554688E+03},
 {   28528, .8309548828125E+04, -.1285592041016E+04, .8940711669922E+03},
 {   28529, .8255583984375E+04, -.1285592041016E+04, .7735197753906E+03},
 {   28530, .7748051757812E+04, -.1377311889648E+04, .7974216918945E+03},
 {   28531, .7748051269531E+04, -.1402711791992E+04, .7974216918945E+03},
 {   28532, .7748051757812E+04, -.1377311889648E+04, .7974216918945E+03},
 {   28533, .7748051269531E+04, -.1402711791992E+04, .7974216918945E+03},
 {   28534, .7724982421875E+04, -.1390011840820E+04, .7646214599609E+03},
 {   28535, .7748051757812E+04, -.1390011840820E+04, .7974216918945E+03},
 {   28536, .7733509277344E+04, -.1390011840820E+04, .8076496582031E+03},
 {   28537, .7733909179688E+04, -.1390011596680E+04, .8115634765625E+03},
 {   28538, .7733908691406E+04, -.1390011962891E+04, .7832792968750E+03},
 {   28539, .7748051757812E+04, -.1169420288086E+04, .7974216918945E+03},
 {   28540, .7748051757812E+04, -.1182120361328E+04, .7974216918945E+03},
 {   28541, .7748051757812E+04, -.1194820312500E+04, .7974216918945E+03},
 {   28542, .7748051757812E+04, -.1169420288086E+04, .7974216918945E+03},
 {   28543, .7748051757812E+04, -.1194820312500E+04, .7974216918945E+03},
 {   28544, .7740980468750E+04, -.1182120117188E+04, .8044931640625E+03},
 {   28545, .7740979980469E+04, -.1182120117188E+04, .7903510742188E+03},
 {   28546, .7724982421875E+04, -.1182120361328E+04, .7646213378906E+03},
 {   28547, .7733509277344E+04, -.1182120361328E+04, .8076494750977E+03},
 {  101529, .8247464843750E+04, -.1285592041016E+04, .1063097167969E+04},
 {  101530, .8247464843750E+04, -.1285592041016E+04, .1063097167969E+04},
 {  101531, .8237796875000E+04, -.1152452880859E+04, .9735024414062E+03},
 {  101532, .8242986328125E+04, -.1152452880859E+04, .9850939941406E+03},
 {  101533, .8248174804688E+04, -.1152452880859E+04, .9966860351562E+03},
 {  101535, .8249468750000E+04, -.1152452880859E+04, .9995839843750E+03},
 {  101536, .8254661132812E+04, -.1152452880859E+04, .1011175048828E+04},
 {  101537, .8259849609375E+04, -.1152452880859E+04, .1022766296387E+04},
 {  101539, .8261147460938E+04, -.1152452880859E+04, .1025664306641E+04},
 {  101540, .8266334960938E+04, -.1152452880859E+04, .1037256347656E+04},
 {  101541, .8271526367188E+04, -.1152452880859E+04, .1048847534180E+04},
 {  101543, .8272822265625E+04, -.1152452880859E+04, .1051745727539E+04},
 {  101544, .8278010742188E+04, -.1152452880859E+04, .1063336669922E+04},
 {  101545, .8283201171875E+04, -.1152452880859E+04, .1074928222656E+04},
 {  101547, .8284498046875E+04, -.1152452880859E+04, .1077826904297E+04},
 {  101548, .8289686523438E+04, -.1152452880859E+04, .1089417846680E+04},
 {  101549, .8294876953125E+04, -.1152452880859E+04, .1101009277344E+04},
 {  101550, .8295978515625E+04, -.1156043457031E+04, .1100516235352E+04},
 {  101551, .8290789062500E+04, -.1156043457031E+04, .1088925415039E+04},
 {  101552, .8285598632812E+04, -.1156043457031E+04, .1077333251953E+04},
 {  101553, .8284300781250E+04, -.1156043457031E+04, .1074435424805E+04},
 {  101554, .8279113281250E+04, -.1156043457031E+04, .1062843750000E+04},
 {  101555, .8273923828125E+04, -.1156043457031E+04, .1051252319336E+04},
 {  101556, .8272626953125E+04, -.1156043457031E+04, .1048354614258E+04},
 {  101557, .8267438476562E+04, -.1156043457031E+04, .1036762695312E+04},
 {  101558, .8262250000000E+04, -.1156043457031E+04, .1025171142578E+04},
 {  101559, .8260950195312E+04, -.1156043457031E+04, .1022273925781E+04},
 {  101560, .8255762695312E+04, -.1156043457031E+04, .1010681945801E+04},
 {  101561, .8250573242188E+04, -.1156043457031E+04, .9990906982422E+03},
 {  101562, .8249275390625E+04, -.1156043457031E+04, .9961928710938E+03},
 {  101563, .8244086914062E+04, -.1156043457031E+04, .9846012573242E+03},
 {  101564, .8238898437500E+04, -.1156043457031E+04, .9730093994141E+03},
 {  101565, .8236693359375E+04, -.1148862426758E+04, .9739956665039E+03},
 {  101566, .8241884765625E+04, -.1148862426758E+04, .9855869750977E+03},
 {  101567, .8247073242188E+04, -.1148862426758E+04, .9971785888672E+03},
 {  101568, .8248370117188E+04, -.1148862426758E+04, .1000076538086E+04},
 {  101569, .8253558593750E+04, -.1148862426758E+04, .1011668334961E+04},
 {  101570, .8258750000000E+04, -.1148862426758E+04, .1023259460449E+04},
 {  101571, .8260046875000E+04, -.1148862426758E+04, .1026156982422E+04},
 {  101572, .8265234375000E+04, -.1148862426758E+04, .1037749511719E+04},
 {  101573, .8270422851562E+04, -.1148862426758E+04, .1049340698242E+04},
 {  101574, .8271720703125E+04, -.1148862426758E+04, .1052238769531E+04},
 {  101575, .8276911132812E+04, -.1148862426758E+04, .1063830078125E+04},
 {  101576, .8282100585938E+04, -.1148862426758E+04, .1075421386719E+04},
 {  101577, .8283397460938E+04, -.1148862426758E+04, .1078319335938E+04},
 {  101578, .8288585937500E+04, -.1148862426758E+04, .1089911010742E+04},
 {  101579, .8293775390625E+04, -.1148862426758E+04, .1101502197266E+04},
 {  101580, .8237997070312E+04, -.1418860107422E+04, .9734124755859E+03},
 {  101581, .8243186523438E+04, -.1418860107422E+04, .9850040283203E+03},
 {  101582, .8248374023438E+04, -.1418860107422E+04, .9965958251953E+03},
 {  101584, .8249673828125E+04, -.1418860107422E+04, .9994937133789E+03},
 {  101585, .8254862304688E+04, -.1418860107422E+04, .1011085449219E+04},
 {  101586, .8260049804688E+04, -.1418860107422E+04, .1022676879883E+04},
 {  101588, .8261349609375E+04, -.1418860107422E+04, .1025574340820E+04},
 {  101589, .8266537109375E+04, -.1418860107422E+04, .1037165771484E+04},
 {  101590, .8271726562500E+04, -.1418860107422E+04, .1048757324219E+04},
 {  101592, .8273024414062E+04, -.1418860107422E+04, .1051655273438E+04},
 {  101593, .8278212890625E+04, -.1418860107422E+04, .1063246826172E+04},
 {  101594, .8283398437500E+04, -.1418860107422E+04, .1074839111328E+04},
 {  101596, .8284699218750E+04, -.1418860107422E+04, .1077736816406E+04},
 {  101597, .8289887695312E+04, -.1418860107422E+04, .1089327758789E+04},
 {  101598, .8295075195312E+04, -.1418860107422E+04, .1100919799805E+04},
 {  101599, .8296178710938E+04, -.1415269653320E+04, .1100426513672E+04},
 {  101600, .8293976562500E+04, -.1422450683594E+04, .1101411987305E+04},
 {  101601, .8290987304688E+04, -.1415269653320E+04, .1088835205078E+04},
 {  101602, .8288788085938E+04, -.1422450683594E+04, .1089820800781E+04},
 {  101603, .8285800781250E+04, -.1415269653320E+04, .1077243408203E+04},
 {  101604, .8283596679688E+04, -.1422450683594E+04, .1078229492188E+04},
 {  101605, .8284503906250E+04, -.1415269653320E+04, .1074345336914E+04},
 {  101606, .8279313476562E+04, -.1415269653320E+04, .1062754028320E+04},
 {  101607, .8274124023438E+04, -.1415269653320E+04, .1051162719727E+04},
 {  101608, .8282299804688E+04, -.1422450683594E+04, .1075331787109E+04},
 {  101609, .8277112304688E+04, -.1422450683594E+04, .1063739746094E+04},
 {  101610, .8271921875000E+04, -.1422450683594E+04, .1052147949219E+04},
 {  101611, .8272829101562E+04, -.1415269653320E+04, .1048264526367E+04},
 {  101612, .8267637695312E+04, -.1415269653320E+04, .1036673095703E+04},
 {  101613, .8262451171875E+04, -.1415269653320E+04, .1025082153320E+04},
 {  101614, .8270625000000E+04, -.1422450683594E+04, .1049250244141E+04},
 {  101615, .8265438476562E+04, -.1422450683594E+04, .1037659057617E+04},
 {  101616, .8260248046875E+04, -.1422450683594E+04, .1026067504883E+04},
 {  101617, .8258949218750E+04, -.1422450683594E+04, .1023169433594E+04},
 {  101618, .8253759765625E+04, -.1422450683594E+04, .1011578063965E+04},
 {  101619, .8248573242188E+04, -.1422450683594E+04, .9999860839844E+03},
 {  101620, .8261153320312E+04, -.1415269653320E+04, .1022183898926E+04},
 {  101621, .8255961914062E+04, -.1415269653320E+04, .1010592163086E+04},
 {  101622, .8250773437500E+04, -.1415269653320E+04, .9990004882812E+03},
 {  101623, .8247272460938E+04, -.1422450683594E+04, .9970887451172E+03},
 {  101624, .8242085937500E+04, -.1422450683594E+04, .9854969482422E+03},
 {  101625, .8236895507812E+04, -.1422450683594E+04, .9739055175781E+03},
 {  101626, .8249478515625E+04, -.1415269653320E+04, .9961022338867E+03},
 {  101627, .8244285156250E+04, -.1415269653320E+04, .9845111083984E+03},
 {  101628, .8239096679688E+04, -.1415269653320E+04, .9729197387695E+03},
 {  101629, .8375061523438E+04, -.1250778564453E+04, .8981396484375E+03},
 {  101630, .8442519531250E+04, -.1250778442383E+04, .1048829833984E+04},
 {  101631, .8375061523438E+04, -.1320405639648E+04, .8981396484375E+03},
 {  101632, .8442519531250E+04, -.1320405517578E+04, .1048829833984E+04},
 {  101633, .8437331054688E+04, -.1320405517578E+04, .1037238403320E+04},
 {  101634, .8431495117188E+04, -.1320405517578E+04, .1024197265625E+04},
 {  101635, .8425655273438E+04, -.1320405639648E+04, .1011156982422E+04},
 {  101636, .8413980468750E+04, -.1320405639648E+04, .9850762329102E+03},
 {  101637, .8402303710938E+04, -.1320405639648E+04, .9589953613281E+03},
 {  101638, .8390627929688E+04, -.1320405639648E+04, .9329145507812E+03},
 {  101639, .8385440429688E+04, -.1320405639648E+04, .9213228759766E+03},
 {  101640, .8380252929688E+04, -.1320405639648E+04, .9097310180664E+03},
 {  101641, .8437331054688E+04, -.1250778442383E+04, .1037238403320E+04},
 {  101642, .8431495117188E+04, -.1250778442383E+04, .1024197265625E+04},
 {  101643, .8425655273438E+04, -.1250778442383E+04, .1011156982422E+04},
 {  101644, .8413980468750E+04, -.1250778442383E+04, .9850762329102E+03},
 {  101645, .8402303710938E+04, -.1250778442383E+04, .9589953613281E+03},
 {  101646, .8390627929688E+04, -.1250778564453E+04, .9329145507812E+03},
 {  101647, .8385440429688E+04, -.1250778564453E+04, .9213228759766E+03},
 {  101648, .8380252929688E+04, -.1250778564453E+04, .9097310180664E+03}
};

struct hex hexes[NHEXES] = {
 {     1,   1, {     1,      2,      4,      3,      5,      6,      8,      7}},
 {     2,   1, {     5,      6,      8,      7,      9,     10,     12,     11}},
 {     3,   1, {     9,     10,     12,     11,     13,     14,     16,     15}},
 {     4,   1, {    13,     14,     16,     15,     17,     18,     20,     19}},
 {     5,   1, {     3,      4,     22,     21,      7,      8,     24,     23}},
 {     6,   1, {     7,      8,     24,     23,     11,     12,     26,     25}},
 {     7,   1, {    11,     12,     26,     25,     15,     16,     28,     27}},
 {     8,   1, {    15,     16,     28,     27,     19,     20,     30,     29}},
 {     9,   1, {    21,     22,     32,     31,     23,     24,     40,     39}},
 {    10,   1, {    31,     32,     34,     33,     39,     40,     42,     41}},
 {    11,   1, {    33,     34,     36,     35,     41,     42,     44,     43}},
 {    12,   1, {    35,     36,     38,     37,     43,     44,     46,     45}},
 {    13,   1, {    23,     24,     40,     39,     25,     26,     48,     47}},
 {    14,   1, {    39,     40,     42,     41,     47,     48,     50,     49}},
 {    15,   1, {    41,     42,     44,     43,     49,     50,     52,     51}},
 {    16,   1, {    43,     44,     46,     45,     51,     52,     54,     53}},
 {    17,   1, {    25,     26,     48,     47,     27,     28,     56,     55}},
 {    18,   1, {    47,     48,     50,     49,     55,     56,     58,     57}},
 {    19,   1, {    49,     50,     52,     51,     57,     58,     60,     59}},
 {    20,   1, {    51,     52,     54,     53,     59,     60,     62,     61}},
 {    21,   1, {    27,     28,     56,     55,     29,     30,     64,     63}},
 {    22,   1, {    55,     56,     58,     57,     63,     64,     66,     65}},
 {    23,   1, {    57,     58,     60,     59,     65,     66,     68,     67}},
 {    24,   1, {    59,     60,     62,     61,     67,     68,     70,     69}},
 {    25,   1, {    37,     38,     72,     71,     45,     46,     74,     73}},
 {    26,   1, {    45,     46,     74,     73,     53,     54,     76,     75}},
 {    27,   1, {    53,     54,     76,     75,     61,     62,     78,     77}},
 {    28,   1, {    61,     62,     78,     77,     69,     70,     80,     79}},
 {    29,   1, {    71,     72,     82,     81,     73,     74,     84,     83}},
 {    30,   1, {    73,     74,     84,     83,     75,     76,     86,     85}},
 {    31,   1, {    75,     76,     86,     85,     77,     78,     88,     87}},
 {    32,   1, {    77,     78,     88,     87,     79,     80,     90,     89}},
 {    33,   1, {     6,     91,     92,      8,     10,     93,     94,     12}},
 {    34,   1, {     8,     92,     95,     24,     12,     94,     96,     26}},
 {    35,   1, {    12,     94,     96,     26,     16,     97,     98,     28}},
 {    36,   1, {    16,     97,     98,     28,     20,     99,    100,     30}},
 {    37,   1, {    20,     99,    100,     30,    101,    102,    104,    103}},
 {    38,   1, {   101,    102,    104,    103,    105,    106,    108,    107}},
 {    39,   1, {   105,    106,    108,    107,    109,    110,    112,    111}},
 {    40,   1, {   109,    110,    112,    111,    113,    114,    116,    115}},
 {    41,   1, {   113,    114,    116,    115,    117,    118,    120,    119}},
 {    42,   1, {   117,    118,    120,    119,    121,    122,    124,    123}},
 {    43,   1, {   121,    122,    124,    123,    125,    126,    128,    127}},
 {    44,   1, {   125,    126,    128,    127,    129,    130,    132,    131}},
 {    45,   1, {   129,    130,    132,    131,    133,    134,    136,    135}},
 {    46,   1, {   133,    134,    136,    135,    137,    138,    140,    139}},
 {    47,   1, {   137,    138,    140,    139,    141,    142,    144,    143}},
 {    48,   1, {   141,    142,    144,    143,    145,    146,    148,    147}},
 {    49,   1, {   145,    146,    148,    147,    149,    150,    152,    151}},
 {    50,   1, {   149,    150,    152,    151,    153,    154,    156,    155}},
 {    51,   1, {   153,    154,    156,    155,    157,    158,    160,    159}},
 {    52,   1, {   157,    158,    160,    159,    161,    162,    164,    163}},
 {    53,   1, {   161,    162,    164,    163,    165,    166,    168,    167}},
 {    54,   1, {    24,     95,    169,     40,     26,     96,    173,     48}},
 {    55,   1, {    40,    169,    170,     42,     48,    173,    174,     50}},
 {    56,   1, {    42,    170,    171,     44,     50,    174,    175,     52}},
 {    57,   1, {    44,    171,    172,     46,     52,    175,    176,     54}},
 {    58,   1, {    30,    100,    177,     64,    103,    104,    182,    181}},
 {    59,   1, {    64,    177,    178,     66,    181,    182,    184,    183}},
 {    60,   1, {    66,    178,    179,     68,    183,    184,    186,    185}},
 {    61,   1, {    68,    179,    180,     70,    185,    186,    188,    187}},
 {    62,   1, {   103,    104,    182,    181,    107,    108,    190,    189}},
 {    63,   1, {   181,    182,    184,    183,    189,    190,    192,    191}},
 {    64,   1, {   183,    184,    186,    185,    191,    192,    194,    193}},
 {    65,   1, {   185,    186,    188,    187,    193,    194,    196,    195}},
 {    66,   1, {   107,    108,    190,    189,    111,    112,    198,    197}},
 {    67,   1, {   189,    190,    192,    191,    197,    198,    200,    199}},
 {    68,   1, {   191,    192,    194,    193,    199,    200,    202,    201}},
 {    69,   1, {   193,    194,    196,    195,    201,    202,    204,    203}},
 {    70,   1, {   111,    112,    198,    197,    115,    116,    206,    205}},
 {    71,   1, {   197,    198,    200,    199,    205,    206,    208,    207}},
 {    72,   1, {   199,    200,    202,    201,    207,    208,    210,    209}},
 {    73,   1, {   201,    202,    204,    203,    209,    210,    212,    211}},
 {    74,   1, {   115,    116,    206,    205,    119,    120,    214,    213}},
 {    75,   1, {   205,    206,    208,    207,    213,    214,    216,    215}},
 {    76,   1, {   207,    208,    210,    209,    215,    216,    218,    217}},
 {    77,   1, {   209,    210,    212,    211,    217,    218,    220,    219}},
 {    78,   1, {   119,    120,    214,    213,    123,    124,    222,    221}},
 {    79,   1, {   213,    214,    216,    215,    221,    222,    224,    223}},
 {    80,   1, {   215,    216,    218,    217,    223,    224,    226,    225}},
 {    81,   1, {   217,    218,    220,    219,    225,    226,    228,    227}},
 {    82,   1, {   123,    124,    222,    221,    127,    128,    230,    229}},
 {    83,   1, {   221,    222,    224,    223,    229,    230,    232,    231}},
 {    84,   1, {   223,    224,    226,    225,    231,    232,    234,    233}},
 {    85,   1, {   225,    226,    228,    227,    233,    234,    236,    235}},
 {    86,   1, {   127,    128,    230,    229,    131,    132,    238,    237}},
 {    87,   1, {   229,    230,    232,    231,    237,    238,    240,    239}},
 {    88,   1, {   231,    232,    234,    233,    239,    240,    242,    241}},
 {    89,   1, {   233,    234,    236,    235,    241,    242,    244,    243}},
 {    90,   1, {   131,    132,    238,    237,    135,    136,    246,    245}},
 {    91,   1, {   237,    238,    240,    239,    245,    246,    248,    247}},
 {    92,   1, {   239,    240,    242,    241,    247,    248,    250,    249}},
 {    93,   1, {   241,    242,    244,    243,    249,    250,    252,    251}},
 {    94,   1, {   135,    136,    246,    245,    139,    140,    254,    253}},
 {    95,   1, {   245,    246,    248,    247,    253,    254,    256,    255}},
 {    96,   1, {   247,    248,    250,    249,    255,    256,    258,    257}},
 {    97,   1, {   249,    250,    252,    251,    257,    258,    260,    259}},
 {    98,   1, {   139,    140,    254,    253,    143,    144,    262,    261}},
 {    99,   1, {   253,    254,    256,    255,    261,    262,    264,    263}},
 {   100,   1, {   255,    256,    258,    257,    263,    264,    266,    265}},
 {   101,   1, {   257,    258,    260,    259,    265,    266,    268,    267}},
 {   102,   1, {   143,    144,    262,    261,    147,    148,    270,    269}},
 {   103,   1, {   261,    262,    264,    263,    269,    270,    272,    271}},
 {   104,   1, {   263,    264,    266,    265,    271,    272,    274,    273}},
 {   105,   1, {   265,    266,    268,    267,    273,    274,    276,    275}},
 {   106,   1, {   147,    148,    270,    269,    151,    152,    278,    277}},
 {   107,   1, {   269,    270,    272,    271,    277,    278,    280,    279}},
 {   108,   1, {   271,    272,    274,    273,    279,    280,    282,    281}},
 {   109,   1, {   273,    274,    276,    275,    281,    282,    284,    283}},
 {   110,   1, {   151,    152,    278,    277,    155,    156,    286,    285}},
 {   111,   1, {   277,    278,    280,    279,    285,    286,    288,    287}},
 {   112,   1, {   279,    280,    282,    281,    287,    288,    290,    289}},
 {   113,   1, {   281,    282,    284,    283,    289,    290,    292,    291}},
 {   114,   1, {   155,    156,    286,    285,    159,    160,    294,    293}},
 {   115,   1, {   285,    286,    288,    287,    293,    294,    296,    295}},
 {   116,   1, {   287,    288,    290,    289,    295,    296,    298,    297}},
 {   117,   1, {   289,    290,    292,    291,    297,    298,    300,    299}},
 {   118,   1, {   159,    160,    294,    293,    163,    164,    302,    301}},
 {   119,   1, {   293,    294,    296,    295,    301,    302,    304,    303}},
 {   120,   1, {   295,    296,    298,    297,    303,    304,    306,    305}},
 {   121,   1, {   297,    298,    300,    299,    305,    306,    308,    307}},
 {   122,   1, {   163,    164,    302,    301,    167,    168,    310,    309}},
 {   123,   1, {   301,    302,    304,    303,    309,    310,    312,    311}},
 {   124,   1, {   303,    304,    306,    305,    311,    312,    314,    313}},
 {   125,   1, {   305,    306,    308,    307,    313,    314,    316,    315}},
 {   126,   1, {    46,    172,    317,     74,     54,    176,    318,     76}},
 {   127,   1, {    54,    176,    318,     76,     62,    319,    320,     78}},
 {   128,   1, {    62,    319,    320,     78,     70,    180,    321,     80}},
 {   129,   1, {    70,    180,    321,     80,    187,    188,    323,    322}},
 {   130,   1, {   187,    188,    323,    322,    195,    196,    325,    324}},
 {   131,   1, {   195,    196,    325,    324,    203,    204,    327,    326}},
 {   132,   1, {   203,    204,    327,    326,    211,    212,    329,    328}},
 {   133,   1, {   211,    212,    329,    328,    219,    220,    331,    330}},
 {   134,   1, {   219,    220,    331,    330,    227,    228,    333,    332}},
 {   135,   1, {   227,    228,    333,    332,    235,    236,    335,    334}},
 {   136,   1, {   235,    236,    335,    334,    243,    244,    337,    336}},
 {   137,   1, {   243,    244,    337,    336,    251,    252,    339,    338}},
 {   138,   1, {   251,    252,    339,    338,    259,    260,    341,    340}},
 {   139,   1, {   259,    260,    341,    340,    267,    268,    343,    342}},
 {   140,   1, {   267,    268,    343,    342,    275,    276,    345,    344}},
 {   141,   1, {   275,    276,    345,    344,    283,    284,    347,    346}},
 {   142,   1, {   283,    284,    347,    346,    291,    292,    349,    348}},
 {   143,   1, {   291,    292,    349,    348,    299,    300,    351,    350}},
 {   144,   1, {   299,    300,    351,    350,    307,    308,    353,    352}},
 {   145,   1, {   307,    308,    353,    352,    315,    316,    355,    354}},
 {   146,   1, {    74,    317,    356,     84,     76,    318,    357,     86}},
 {   147,   1, {    91,    358,    360,     92,     93,    362,    364,     94}},
 {   148,   1, {   358,    359,    361,    360,    362,    363,    365,    364}},
 {   149,   1, {    92,    360,    366,     95,     94,    364,    368,     96}},
 {   150,   1, {   360,    361,    367,    366,    364,    365,    369,    368}},
 {   151,   1, {    94,    364,    368,     96,     97,    370,    372,     98}},
 {   152,   1, {   364,    365,    369,    368,    370,    371,    373,    372}},
 {   153,   1, {    97,    370,    372,     98,     99,    374,    376,    100}},
 {   154,   1, {   370,    371,    373,    372,    374,    375,    377,    376}},
 {   155,   1, {    99,    374,    376,    100,    102,    378,    380,    104}},
 {   156,   1, {   374,    375,    377,    376,    378,    379,    381,    380}},
 {   157,   1, {   102,    378,    380,    104,    106,    382,    384,    108}},
 {   158,   1, {   378,    379,    381,    380,    382,    383,    385,    384}},
 {   159,   1, {   106,    382,    384,    108,    110,    386,    388,    112}},
 {   160,   1, {   382,    383,    385,    384,    386,    387,    389,    388}},
 {   161,   1, {   110,    386,    388,    112,    114,    390,    392,    116}},
 {   162,   1, {   386,    387,    389,    388,    390,    391,    393,    392}},
 {   163,   1, {   114,    390,    392,    116,    118,    394,    396,    120}},
 {   164,   1, {   390,    391,    393,    392,    394,    395,    397,    396}},
 {   165,   1, {   118,    394,    396,    120,    122,    398,    400,    124}},
 {   166,   1, {   394,    395,    397,    396,    398,    399,    401,    400}},
 {   167,   1, {   122,    398,    400,    124,    126,    402,    404,    128}},
 {   168,   1, {   398,    399,    401,    400,    402,    403,    405,    404}},
 {   169,   1, {   126,    402,    404,    128,    130,    406,    408,    132}},
 {   170,   1, {   402,    403,    405,    404,    406,    407,    409,    408}},
 {   171,   1, {   130,    406,    408,    132,    134,    410,    412,    136}},
 {   172,   1, {   406,    407,    409,    408,    410,    411,    413,    412}},
 {   173,   1, {   134,    410,    412,    136,    138,    414,    416,    140}},
 {   174,   1, {   410,    411,    413,    412,    414,    415,    417,    416}},
 {   175,   1, {   138,    414,    416,    140,    142,    418,    420,    144}},
 {   176,   1, {   414,    415,    417,    416,    418,    419,    421,    420}},
 {   177,   1, {   142,    418,    420,    144,    146,    422,    424,    148}},
 {   178,   1, {   418,    419,    421,    420,    422,    423,    425,    424}},
 {   179,   1, {   146,    422,    424,    148,    150,    426,    428,    152}},
 {   180,   1, {   422,    423,    425,    424,    426,    427,    429,    428}},
 {   181,   1, {   150,    426,    428,    152,    154,    430,    432,    156}},
 {   182,   1, {   426,    427,    429,    428,    430,    431,    433,    432}},
 {   183,   1, {   154,    430,    432,    156,    158,    434,    436,    160}},
 {   184,   1, {   430,    431,    433,    432,    434,    435,    437,    436}},
 {   185,   1, {   158,    434,    436,    160,    162,    438,    440,    164}},
 {   186,   1, {   434,    435,    437,    436,    438,    439,    441,    440}},
 {   187,   1, {   162,    438,    440,    164,    166,    442,    444,    168}},
 {   188,   1, {   438,    439,    441,    440,    442,    443,    445,    444}},
 {   189,   1, {    95,    366,    446,    169,     96,    368,    454,    173}},
 {   190,   1, {   366,    367,    447,    446,    368,    369,    455,    454}},
 {   191,   1, {   169,    446,    448,    170,    173,    454,    456,    174}},
 {   192,   1, {   446,    447,    449,    448,    454,    455,    457,    456}},
 {   193,   1, {   170,    448,    450,    171,    174,    456,    458,    175}},
 {   194,   1, {   448,    449,    451,    450,    456,    457,    459,    458}},
 {   195,   1, {   171,    450,    452,    172,    175,    458,    460,    176}},
 {   196,   1, {   450,    451,    453,    452,    458,    459,    461,    460}},
 {   197,   1, {   164,    440,    462,    302,    168,    444,    470,    310}},
 {   198,   1, {   440,    441,    463,    462,    444,    445,    471,    470}},
 {   199,   1, {   302,    462,    464,    304,    310,    470,    472,    312}},
 {   200,   1, {   462,    463,    465,    464,    470,    471,    473,    472}},
 {   201,   1, {   304,    464,    466,    306,    312,    472,    474,    314}},
 {   202,   1, {   464,    465,    467,    466,    472,    473,    475,    474}},
 {   203,   1, {   306,    466,    468,    308,    314,    474,    476,    316}},
 {   204,   1, {   466,    467,    469,    468,    474,    475,    477,    476}},
 {   205,   1, {   172,    452,    478,    317,    176,    460,    480,    318}},
 {   206,   1, {   452,    453,    479,    478,    460,    461,    481,    480}},
 {   207,   1, {   176,    460,    480,    318,    319,    482,    484,    320}},
 {   208,   1, {   460,    461,    481,    480,    482,    483,    485,    484}},
 {   209,   1, {   319,    482,    484,    320,    180,    486,    488,    321}},
 {   210,   1, {   482,    483,    485,    484,    486,    487,    489,    488}},
 {   211,   1, {   180,    486,    488,    321,    188,    490,    492,    323}},
 {   212,   1, {   486,    487,    489,    488,    490,    491,    493,    492}},
 {   213,   1, {   188,    490,    492,    323,    196,    494,    496,    325}},
 {   214,   1, {   490,    491,    493,    492,    494,    495,    497,    496}},
 {   215,   1, {   196,    494,    496,    325,    204,    498,    500,    327}},
 {   216,   1, {   494,    495,    497,    496,    498,    499,    501,    500}},
 {   217,   1, {   204,    498,    500,    327,    212,    502,    504,    329}},
 {   218,   1, {   498,    499,    501,    500,    502,    503,    505,    504}},
 {   219,   1, {   212,    502,    504,    329,    220,    506,    508,    331}},
 {   220,   1, {   502,    503,    505,    504,    506,    507,    509,    508}},
 {   221,   1, {   220,    506,    508,    331,    228,    510,    512,    333}},
 {   222,   1, {   506,    507,    509,    508,    510,    511,    513,    512}},
 {   223,   1, {   228,    510,    512,    333,    236,    514,    516,    335}},
 {   224,   1, {   510,    511,    513,    512,    514,    515,    517,    516}},
 {   225,   1, {   236,    514,    516,    335,    244,    518,    520,    337}},
 {   226,   1, {   514,    515,    517,    516,    518,    519,    521,    520}},
 {   227,   1, {   244,    518,    520,    337,    252,    522,    524,    339}},
 {   228,   1, {   518,    519,    521,    520,    522,    523,    525,    524}},
 {   229,   1, {   252,    522,    524,    339,    260,    526,    528,    341}},
 {   230,   1, {   522,    523,    525,    524,    526,    527,    529,    528}},
 {   231,   1, {   260,    526,    528,    341,    268,    530,    532,    343}},
 {   232,   1, {   526,    527,    529,    528,    530,    531,    533,    532}},
 {   233,   1, {   268,    530,    532,    343,    276,    534,    536,    345}},
 {   234,   1, {   530,    531,    533,    532,    534,    535,    537,    536}},
 {   235,   1, {   276,    534,    536,    345,    284,    538,    540,    347}},
 {   236,   1, {   534,    535,    537,    536,    538,    539,    541,    540}},
 {   237,   1, {   284,    538,    540,    347,    292,    542,    544,    349}},
 {   238,   1, {   538,    539,    541,    540,    542,    543,    545,    544}},
 {   239,   1, {   292,    542,    544,    349,    300,    546,    548,    351}},
 {   240,   1, {   542,    543,    545,    544,    546,    547,    549,    548}},
 {   241,   1, {   300,    546,    548,    351,    308,    468,    550,    353}},
 {   242,   1, {   546,    547,    549,    548,    468,    469,    551,    550}},
 {   243,   1, {   308,    468,    550,    353,    316,    476,    552,    355}},
 {   244,   1, {   468,    469,    551,    550,    476,    477,    553,    552}},
 {   245,   1, {   317,    478,    554,    356,    318,    480,    556,    357}},
 {   246,   1, {   478,    479,    555,    554,    480,    481,    557,    556}},
 {   247,   1, {   359,    558,    560,    361,    363,    562,    564,    365}},
 {   248,   1, {   558,    559,    561,    560,    562,    563,    565,    564}},
 {   249,   1, {   361,    560,    566,    367,    365,    564,    568,    369}},
 {   250,   1, {   560,    561,    567,    566,    564,    565,    569,    568}},
 {   251,   1, {   365,    564,    568,    369,    371,    570,    572,    373}},
 {   252,   1, {   564,    565,    569,    568,    570,    571,    573,    572}},
 {   253,   1, {   371,    570,    572,    373,    375,    574,    576,    377}},
 {   254,   1, {   570,    571,    573,    572,    574,    575,    577,    576}},
 {   255,   1, {   375,    574,    576,    377,    379,    578,    580,    381}},
 {   256,   1, {   574,    575,    577,    576,    578,    579,    581,    580}},
 {   257,   1, {   379,    578,    580,    381,    383,    582,    584,    385}},
 {   258,   1, {   578,    579,    581,    580,    582,    583,    585,    584}},
 {   259,   1, {   383,    582,    584,    385,    387,    586,    588,    389}},
 {   260,   1, {   582,    583,    585,    584,    586,    587,    589,    588}},
 {   261,   1, {   387,    586,    588,    389,    391,    590,    592,    393}},
 {   262,   1, {   586,    587,    589,    588,    590,    591,    593,    592}},
 {   263,   1, {   391,    590,    592,    393,    395,    594,    596,    397}},
 {   264,   1, {   590,    591,    593,    592,    594,    595,    597,    596}},
 {   265,   1, {   395,    594,    596,    397,    399,    598,    600,    401}},
 {   266,   1, {   594,    595,    597,    596,    598,    599,    601,    600}},
 {   267,   1, {   399,    598,    600,    401,    403,    602,    604,    405}},
 {   268,   1, {   598,    599,    601,    600,    602,    603,    605,    604}},
 {   269,   1, {   403,    602,    604,    405,    407,    606,    608,    409}},
 {   270,   1, {   602,    603,    605,    604,    606,    607,    609,    608}},
 {   271,   1, {   407,    606,    608,    409,    411,    610,    612,    413}},
 {   272,   1, {   606,    607,    609,    608,    610,    611,    613,    612}},
 {   273,   1, {   411,    610,    612,    413,    415,    614,    616,    417}},
 {   274,   1, {   610,    611,    613,    612,    614,    615,    617,    616}},
 {   275,   1, {   415,    614,    616,    417,    419,    618,    620,    421}},
 {   276,   1, {   614,    615,    617,    616,    618,    619,    621,    620}},
 {   277,   1, {   419,    618,    620,    421,    423,    622,    624,    425}},
 {   278,   1, {   618,    619,    621,    620,    622,    623,    625,    624}},
 {   279,   1, {   423,    622,    624,    425,    427,    626,    628,    429}},
 {   280,   1, {   622,    623,    625,    624,    626,    627,    629,    628}},
 {   281,   1, {   427,    626,    628,    429,    431,    630,    632,    433}},
 {   282,   1, {   626,    627,    629,    628,    630,    631,    633,    632}},
 {   283,   1, {   431,    630,    632,    433,    435,    634,    636,    437}},
 {   284,   1, {   630,    631,    633,    632,    634,    635,    637,    636}},
 {   285,   1, {   435,    634,    636,    437,    439,    638,    640,    441}},
 {   286,   1, {   634,    635,    637,    636,    638,    639,    641,    640}},
 {   287,   1, {   439,    638,    640,    441,    443,    642,    644,    445}},
 {   288,   1, {   638,    639,    641,    640,    642,    643,    645,    644}},
 {   289,   1, {   367,    566,    646,    447,    369,    568,    654,    455}},
 {   290,   1, {   566,    567,    647,    646,    568,    569,    655,    654}},
 {   291,   1, {   447,    646,    648,    449,    455,    654,    656,    457}},
 {   292,   1, {   646,    647,    649,    648,    654,    655,    657,    656}},
 {   293,   1, {   449,    648,    650,    451,    457,    656,    658,    459}},
 {   294,   1, {   648,    649,    651,    650,    656,    657,    659,    658}},
 {   295,   1, {   451,    650,    652,    453,    459,    658,    660,    461}},
 {   296,   1, {   650,    651,    653,    652,    658,    659,    661,    660}},
 {   297,   1, {   369,    568,    654,    455,    373,    572,    663,    662}},
 {   298,   1, {   568,    569,    655,    654,    572,    573,    664,    663}},
 {   299,   1, {   455,    654,    656,    457,    662,    663,    666,    665}},
 {   300,   1, {   654,    655,    657,    656,    663,    664,    667,    666}},
 {   301,   1, {   457,    656,    658,    459,    665,    666,    669,    668}},
 {   302,   1, {   656,    657,    659,    658,    666,    667,    670,    669}},
 {   303,   1, {   459,    658,    660,    461,    668,    669,    671,    483}},
 {   304,   1, {   658,    659,    661,    660,    669,    670,    672,    671}},
 {   305,   1, {   373,    572,    663,    662,    377,    576,    674,    673}},
 {   306,   1, {   572,    573,    664,    663,    576,    577,    675,    674}},
 {   307,   1, {   662,    663,    666,    665,    673,    674,    677,    676}},
 {   308,   1, {   663,    664,    667,    666,    674,    675,    678,    677}},
 {   309,   1, {   665,    666,    669,    668,    676,    677,    680,    679}},
 {   310,   1, {   666,    667,    670,    669,    677,    678,    681,    680}},
 {   311,   1, {   668,    669,    671,    483,    679,    680,    682,    487}},
 {   312,   1, {   669,    670,    672,    671,    680,    681,    683,    682}},
 {   313,   1, {   377,    576,    674,    673,    381,    580,    685,    684}},
 {   314,   1, {   576,    577,    675,    674,    580,    581,    686,    685}},
 {   315,   1, {   673,    674,    677,    676,    684,    685,    688,    687}},
 {   316,   1, {   674,    675,    678,    677,    685,    686,    689,    688}},
 {   317,   1, {   676,    677,    680,    679,    687,    688,    691,    690}},
 {   318,   1, {   677,    678,    681,    680,    688,    689,    692,    691}},
 {   319,   1, {   679,    680,    682,    487,    690,    691,    693,    491}},
 {   320,   1, {   680,    681,    683,    682,    691,    692,    694,    693}},
 {   321,   1, {   381,    580,    685,    684,    385,    584,    696,    695}},
 {   322,   1, {   580,    581,    686,    685,    584,    585,    697,    696}},
 {   323,   1, {   684,    685,    688,    687,    695,    696,    699,    698}},
 {   324,   1, {   685,    686,    689,    688,    696,    697,    700,    699}},
 {   325,   1, {   687,    688,    691,    690,    698,    699,    702,    701}},
 {   326,   1, {   688,    689,    692,    691,    699,    700,    703,    702}},
 {   327,   1, {   690,    691,    693,    491,    701,    702,    704,    495}},
 {   328,   1, {   691,    692,    694,    693,    702,    703,    705,    704}},
 {   329,   1, {   385,    584,    696,    695,    389,    588,    707,    706}},
 {   330,   1, {   584,    585,    697,    696,    588,    589,    708,    707}},
 {   331,   1, {   695,    696,    699,    698,    706,    707,    710,    709}},
 {   332,   1, {   696,    697,    700,    699,    707,    708,    711,    710}},
 {   333,   1, {   698,    699,    702,    701,    709,    710,    713,    712}},
 {   334,   1, {   699,    700,    703,    702,    710,    711,    714,    713}},
 {   335,   1, {   701,    702,    704,    495,    712,    713,    715,    499}},
 {   336,   1, {   702,    703,    705,    704,    713,    714,    716,    715}},
 {   337,   1, {   389,    588,    707,    706,    393,    592,    718,    717}},
 {   338,   1, {   588,    589,    708,    707,    592,    593,    719,    718}},
 {   339,   1, {   706,    707,    710,    709,    717,    718,    721,    720}},
 {   340,   1, {   707,    708,    711,    710,    718,    719,    722,    721}},
 {   341,   1, {   709,    710,    713,    712,    720,    721,    724,    723}},
 {   342,   1, {   710,    711,    714,    713,    721,    722,    725,    724}},
 {   343,   1, {   712,    713,    715,    499,    723,    724,    726,    503}},
 {   344,   1, {   713,    714,    716,    715,    724,    725,    727,    726}},
 {   345,   1, {   393,    592,    718,    717,    397,    596,    729,    728}},
 {   346,   1, {   592,    593,    719,    718,    596,    597,    730,    729}},
 {   347,   1, {   717,    718,    721,    720,    728,    729,    732,    731}},
 {   348,   1, {   718,    719,    722,    721,    729,    730,    733,    732}},
 {   349,   1, {   720,    721,    724,    723,    731,    732,    735,    734}},
 {   350,   1, {   721,    722,    725,    724,    732,    733,    736,    735}},
 {   351,   1, {   723,    724,    726,    503,    734,    735,    737,    507}},
 {   352,   1, {   724,    725,    727,    726,    735,    736,    738,    737}},
 {   353,   1, {   397,    596,    729,    728,    401,    600,    740,    739}},
 {   354,   1, {   596,    597,    730,    729,    600,    601,    741,    740}},
 {   355,   1, {   728,    729,    732,    731,    739,    740,    743,    742}},
 {   356,   1, {   729,    730,    733,    732,    740,    741,    744,    743}},
 {   357,   1, {   731,    732,    735,    734,    742,    743,    746,    745}},
 {   358,   1, {   732,    733,    736,    735,    743,    744,    747,    746}},
 {   359,   1, {   734,    735,    737,    507,    745,    746,    748,    511}},
 {   360,   1, {   735,    736,    738,    737,    746,    747,    749,    748}},
 {   361,   1, {   401,    600,    740,    739,    405,    604,    751,    750}},
 {   362,   1, {   600,    601,    741,    740,    604,    605,    752,    751}},
 {   363,   1, {   739,    740,    743,    742,    750,    751,    754,    753}},
 {   364,   1, {   740,    741,    744,    743,    751,    752,    755,    754}},
 {   365,   1, {   742,    743,    746,    745,    753,    754,    757,    756}},
 {   366,   1, {   743,    744,    747,    746,    754,    755,    758,    757}},
 {   367,   1, {   745,    746,    748,    511,    756,    757,    759,    515}},
 {   368,   1, {   746,    747,    749,    748,    757,    758,    760,    759}},
 {   369,   1, {   405,    604,    751,    750,    409,    608,    762,    761}},
 {   370,   1, {   604,    605,    752,    751,    608,    609,    763,    762}},
 {   371,   1, {   750,    751,    754,    753,    761,    762,    765,    764}},
 {   372,   1, {   751,    752,    755,    754,    762,    763,    766,    765}},
 {   373,   1, {   753,    754,    757,    756,    764,    765,    768,    767}},
 {   374,   1, {   754,    755,    758,    757,    765,    766,    769,    768}},
 {   375,   1, {   756,    757,    759,    515,    767,    768,    770,    519}},
 {   376,   1, {   757,    758,    760,    759,    768,    769,    771,    770}},
 {   377,   1, {   409,    608,    762,    761,    413,    612,    773,    772}},
 {   378,   1, {   608,    609,    763,    762,    612,    613,    774,    773}},
 {   379,   1, {   761,    762,    765,    764,    772,    773,    776,    775}},
 {   380,   1, {   762,    763,    766,    765,    773,    774,    777,    776}},
 {   381,   1, {   764,    765,    768,    767,    775,    776,    779,    778}},
 {   382,   1, {   765,    766,    769,    768,    776,    777,    780,    779}},
 {   383,   1, {   767,    768,    770,    519,    778,    779,    781,    523}},
 {   384,   1, {   768,    769,    771,    770,    779,    780,    782,    781}},
 {   385,   1, {   413,    612,    773,    772,    417,    616,    784,    783}},
 {   386,   1, {   612,    613,    774,    773,    616,    617,    785,    784}},
 {   387,   1, {   772,    773,    776,    775,    783,    784,    787,    786}},
 {   388,   1, {   773,    774,    777,    776,    784,    785,    788,    787}},
 {   389,   1, {   775,    776,    779,    778,    786,    787,    790,    789}},
 {   390,   1, {   776,    777,    780,    779,    787,    788,    791,    790}},
 {   391,   1, {   778,    779,    781,    523,    789,    790,    792,    527}},
 {   392,   1, {   779,    780,    782,    781,    790,    791,    793,    792}},
 {   393,   1, {   417,    616,    784,    783,    421,    620,    795,    794}},
 {   394,   1, {   616,    617,    785,    784,    620,    621,    796,    795}},
 {   395,   1, {   783,    784,    787,    786,    794,    795,    798,    797}},
 {   396,   1, {   784,    785,    788,    787,    795,    796,    799,    798}},
 {   397,   1, {   786,    787,    790,    789,    797,    798,    801,    800}},
 {   398,   1, {   787,    788,    791,    790,    798,    799,    802,    801}},
 {   399,   1, {   789,    790,    792,    527,    800,    801,    803,    531}},
 {   400,   1, {   790,    791,    793,    792,    801,    802,    804,    803}},
 {   401,   1, {   421,    620,    795,    794,    425,    624,    806,    805}},
 {   402,   1, {   620,    621,    796,    795,    624,    625,    807,    806}},
 {   403,   1, {   794,    795,    798,    797,    805,    806,    809,    808}},
 {   404,   1, {   795,    796,    799,    798,    806,    807,    810,    809}},
 {   405,   1, {   797,    798,    801,    800,    808,    809,    812,    811}},
 {   406,   1, {   798,    799,    802,    801,    809,    810,    813,    812}},
 {   407,   1, {   800,    801,    803,    531,    811,    812,    814,    535}},
 {   408,   1, {   801,    802,    804,    803,    812,    813,    815,    814}},
 {   409,   1, {   425,    624,    806,    805,    429,    628,    817,    816}},
 {   410,   1, {   624,    625,    807,    806,    628,    629,    818,    817}},
 {   411,   1, {   805,    806,    809,    808,    816,    817,    820,    819}},
 {   412,   1, {   806,    807,    810,    809,    817,    818,    821,    820}},
 {   413,   1, {   808,    809,    812,    811,    819,    820,    823,    822}},
 {   414,   1, {   809,    810,    813,    812,    820,    821,    824,    823}},
 {   415,   1, {   811,    812,    814,    535,    822,    823,    825,    539}},
 {   416,   1, {   812,    813,    815,    814,    823,    824,    826,    825}},
 {   417,   1, {   429,    628,    817,    816,    433,    632,    828,    827}},
 {   418,   1, {   628,    629,    818,    817,    632,    633,    829,    828}},
 {   419,   1, {   816,    817,    820,    819,    827,    828,    831,    830}},
 {   420,   1, {   817,    818,    821,    820,    828,    829,    832,    831}},
 {   421,   1, {   819,    820,    823,    822,    830,    831,    834,    833}},
 {   422,   1, {   820,    821,    824,    823,    831,    832,    835,    834}},
 {   423,   1, {   822,    823,    825,    539,    833,    834,    836,    543}},
 {   424,   1, {   823,    824,    826,    825,    834,    835,    837,    836}},
 {   425,   1, {   433,    632,    828,    827,    437,    636,    839,    838}},
 {   426,   1, {   632,    633,    829,    828,    636,    637,    840,    839}},
 {   427,   1, {   827,    828,    831,    830,    838,    839,    842,    841}},
 {   428,   1, {   828,    829,    832,    831,    839,    840,    843,    842}},
 {   429,   1, {   830,    831,    834,    833,    841,    842,    845,    844}},
 {   430,   1, {   831,    832,    835,    834,    842,    843,    846,    845}},
 {   431,   1, {   833,    834,    836,    543,    844,    845,    847,    547}},
 {   432,   1, {   834,    835,    837,    836,    845,    846,    848,    847}},
 {   433,   1, {   437,    636,    839,    838,    441,    640,    849,    463}},
 {   434,   1, {   636,    637,    840,    839,    640,    641,    850,    849}},
 {   435,   1, {   838,    839,    842,    841,    463,    849,    851,    465}},
 {   436,   1, {   839,    840,    843,    842,    849,    850,    852,    851}},
 {   437,   1, {   841,    842,    845,    844,    465,    851,    853,    467}},
 {   438,   1, {   842,    843,    846,    845,    851,    852,    854,    853}},
 {   439,   1, {   844,    845,    847,    547,    467,    853,    855,    469}},
 {   440,   1, {   845,    846,    848,    847,    853,    854,    856,    855}},
 {   441,   1, {   441,    640,    849,    463,    445,    644,    857,    471}},
 {   442,   1, {   640,    641,    850,    849,    644,    645,    858,    857}},
 {   443,   1, {   463,    849,    851,    465,    471,    857,    859,    473}},
 {   444,   1, {   849,    850,    852,    851,    857,    858,    860,    859}},
 {   445,   1, {   465,    851,    853,    467,    473,    859,    861,    475}},
 {   446,   1, {   851,    852,    854,    853,    859,    860,    862,    861}},
 {   447,   1, {   467,    853,    855,    469,    475,    861,    863,    477}},
 {   448,   1, {   853,    854,    856,    855,    861,    862,    864,    863}},
 {   449,   1, {   453,    652,    865,    479,    461,    660,    867,    481}},
 {   450,   1, {   652,    653,    866,    865,    660,    661,    868,    867}},
 {   451,   1, {   461,    660,    867,    481,    483,    671,    869,    485}},
 {   452,   1, {   660,    661,    868,    867,    671,    672,    870,    869}},
 {   453,   1, {   483,    671,    869,    485,    487,    682,    871,    489}},
 {   454,   1, {   671,    672,    870,    869,    682,    683,    872,    871}},
 {   455,   1, {   487,    682,    871,    489,    491,    693,    873,    493}},
 {   456,   1, {   682,    683,    872,    871,    693,    694,    874,    873}},
 {   457,   1, {   491,    693,    873,    493,    495,    704,    875,    497}},
 {   458,   1, {   693,    694,    874,    873,    704,    705,    876,    875}},
 {   459,   1, {   495,    704,    875,    497,    499,    715,    877,    501}},
 {   460,   1, {   704,    705,    876,    875,    715,    716,    878,    877}},
 {   461,   1, {   499,    715,    877,    501,    503,    726,    879,    505}},
 {   462,   1, {   715,    716,    878,    877,    726,    727,    880,    879}},
 {   463,   1, {   503,    726,    879,    505,    507,    737,    881,    509}},
 {   464,   1, {   726,    727,    880,    879,    737,    738,    882,    881}},
 {   465,   1, {   507,    737,    881,    509,    511,    748,    883,    513}},
 {   466,   1, {   737,    738,    882,    881,    748,    749,    884,    883}},
 {   467,   1, {   511,    748,    883,    513,    515,    759,    885,    517}},
 {   468,   1, {   748,    749,    884,    883,    759,    760,    886,    885}},
 {   469,   1, {   515,    759,    885,    517,    519,    770,    887,    521}},
 {   470,   1, {   759,    760,    886,    885,    770,    771,    888,    887}},
 {   471,   1, {   519,    770,    887,    521,    523,    781,    889,    525}},
 {   472,   1, {   770,    771,    888,    887,    781,    782,    890,    889}},
 {   473,   1, {   523,    781,    889,    525,    527,    792,    891,    529}},
 {   474,   1, {   781,    782,    890,    889,    792,    793,    892,    891}},
 {   475,   1, {   527,    792,    891,    529,    531,    803,    893,    533}},
 {   476,   1, {   792,    793,    892,    891,    803,    804,    894,    893}},
 {   477,   1, {   531,    803,    893,    533,    535,    814,    895,    537}},
 {   478,   1, {   803,    804,    894,    893,    814,    815,    896,    895}},
 {   479,   1, {   535,    814,    895,    537,    539,    825,    897,    541}},
 {   480,   1, {   814,    815,    896,    895,    825,    826,    898,    897}},
 {   481,   1, {   539,    825,    897,    541,    543,    836,    899,    545}},
 {   482,   1, {   825,    826,    898,    897,    836,    837,    900,    899}},
 {   483,   1, {   543,    836,    899,    545,    547,    847,    901,    549}},
 {   484,   1, {   836,    837,    900,    899,    847,    848,    902,    901}},
 {   485,   1, {   547,    847,    901,    549,    469,    855,    903,    551}},
 {   486,   1, {   847,    848,    902,    901,    855,    856,    904,    903}},
 {   487,   1, {   469,    855,    903,    551,    477,    863,    905,    553}},
 {   488,   1, {   855,    856,    904,    903,    863,    864,    906,    905}},
 {   489,   1, {   479,    865,    907,    555,    481,    867,    909,    557}},
 {   490,   1, {   865,    866,    908,    907,    867,    868,    910,    909}},
 {   491,   1, {   911,    912,    914,    913,    915,    916,    918,    917}},
 {   492,   1, {   915,    916,    918,    917,    919,    920,    922,    921}},
 {   493,   1, {   919,    920,    922,    921,    923,    924,    926,    925}},
 {   494,   1, {   923,    924,    926,    925,    927,    928,    930,    929}},
 {   495,   1, {   913,    914,    932,    931,    917,    918,    936,    935}},
 {   496,   1, {   931,    932,    934,    933,    935,    936,    938,    937}},
 {   497,   1, {   925,    926,    940,    939,    929,    930,    944,    943}},
 {   498,   1, {   939,    940,    942,    941,    943,    944,    946,    945}},
 {   499,   1, {   912,    947,    949,    914,    916,    951,    953,    918}},
 {   500,   1, {   947,    948,    950,    949,    951,    952,    954,    953}},
 {   501,   1, {   916,    951,    953,    918,    920,    955,    957,    922}},
 {   502,   1, {   951,    952,    954,    953,    955,    956,    958,    957}},
 {   503,   1, {   920,    955,    957,    922,    924,    959,    961,    926}},
 {   504,   1, {   955,    956,    958,    957,    959,    960,    962,    961}},
 {   505,   1, {   924,    959,    961,    926,    928,    963,    965,    930}},
 {   506,   1, {   959,    960,    962,    961,    963,    964,    966,    965}},
 {   507,   1, {   914,    949,    967,    932,    918,    953,    971,    936}},
 {   508,   1, {   949,    950,    968,    967,    953,    954,    972,    971}},
 {   509,   1, {   932,    967,    969,    934,    936,    971,    973,    938}},
 {   510,   1, {   967,    968,    970,    969,    971,    972,    974,    973}},
 {   511,   1, {   926,    961,    975,    940,    930,    965,    979,    944}},
 {   512,   1, {   961,    962,    976,    975,    965,    966,    980,    979}},
 {   513,   1, {   940,    975,    977,    942,    944,    979,    981,    946}},
 {   514,   1, {   975,    976,    978,    977,    979,    980,    982,    981}},
 {   515,   1, {   934,    969,    983,    933,    938,    973,    985,    937}},
 {   516,   1, {   969,    970,    984,    983,    973,    974,    986,    985}},
 {   517,   1, {   942,    977,    987,    941,    946,    981,    989,    945}},
 {   518,   1, {   977,    978,    988,    987,    981,    982,    990,    989}},
 {   519,   1, {   948,    991,    992,    950,    952,    993,    994,    954}},
 {   520,   1, {   952,    993,    994,    954,    956,    995,    996,    958}},
 {   521,   1, {   956,    995,    996,    958,    960,    997,    998,    962}},
 {   522,   1, {   960,    997,    998,    962,    964,    999,   1000,    966}},
 {   523,   1, {   950,    992,   1001,    968,    954,    994,   1003,    972}},
 {   524,   1, {   968,   1001,   1002,    970,    972,   1003,   1004,    974}},
 {   525,   1, {   954,    994,   1003,    972,    958,    996,   1006,   1005}},
 {   526,   1, {   972,   1003,   1004,    974,   1005,   1006,   1008,   1007}},
 {   527,   1, {   958,    996,   1006,   1005,    962,    998,   1009,    976}},
 {   528,   1, {  1005,   1006,   1008,   1007,    976,   1009,   1010,    978}},
 {   529,   1, {   962,    998,   1009,    976,    966,   1000,   1011,    980}},
 {   530,   1, {   976,   1009,   1010,    978,    980,   1011,   1012,    982}},
 {   531,   1, {   970,   1002,   1013,    984,    974,   1004,   1014,    986}},
 {   532,   1, {   974,   1004,   1014,    986,   1007,   1008,   1016,   1015}},
 {   533,   1, {  1007,   1008,   1016,   1015,    978,   1010,   1017,    988}},
 {   534,   1, {   978,   1010,   1017,    988,    982,   1012,   1018,    990}},
 {   535,   1, {  1019,   1020,   1022,   1021,   1033,   1034,   1036,   1035}},
 {   536,   1, {  1021,   1022,   1024,   1023,   1035,   1036,   1038,   1037}},
 {   537,   1, {  1023,   1024,   1026,   1025,   1037,   1038,   1040,   1039}},
 {   538,   1, {  1025,   1026,   1028,   1027,   1039,   1040,   1042,   1041}},
 {   539,   1, {  1027,   1028,   1030,   1029,   1041,   1042,   1044,   1043}},
 {   540,   1, {  1029,   1030,   1032,   1031,   1043,   1044,   1046,   1045}},
 {   541,   1, {  1031,   1032,   1020,   1019,   1045,   1046,   1034,   1033}},
 {   542,   1, {  1033,   1034,   1036,   1035,   1047,   1048,   1050,   1049}},
 {   543,   1, {  1035,   1036,   1038,   1037,   1049,   1050,   1052,   1051}},
 {   544,   1, {  1037,   1038,   1040,   1039,   1051,   1052,   1054,   1053}},
 {   545,   1, {  1039,   1040,   1042,   1041,   1053,   1054,   1056,   1055}},
 {   546,   1, {  1041,   1042,   1044,   1043,   1055,   1056,   1058,   1057}},
 {   547,   1, {  1043,   1044,   1046,   1045,   1057,   1058,   1060,   1059}},
 {   548,   1, {  1045,   1046,   1034,   1033,   1059,   1060,   1048,   1047}},
 {   549,   1, {  1047,   1048,   1050,   1049,   1061,   1062,   1064,   1063}},
 {   550,   1, {  1049,   1050,   1052,   1051,   1063,   1064,   1066,   1065}},
 {   551,   1, {  1051,   1052,   1054,   1053,   1065,   1066,   1068,   1067}},
 {   552,   1, {  1053,   1054,   1056,   1055,   1067,   1068,   1070,   1069}},
 {   553,   1, {  1055,   1056,   1058,   1057,   1069,   1070,   1072,   1071}},
 {   554,   1, {  1057,   1058,   1060,   1059,   1071,   1072,   1074,   1073}},
 {   555,   1, {  1059,   1060,   1048,   1047,   1073,   1074,   1062,   1061}},
 {   556,   1, {  1075,   1076,   1078,   1077,   1081,   1082,   1084,   1083}},
 {   557,   1, {  1077,   1078,   1080,   1079,   1083,   1084,   1086,   1085}},
 {   558,   1, {  1081,   1082,   1084,   1083,   1087,   1088,   1090,   1089}},
 {   559,   1, {  1083,   1084,   1086,   1085,   1089,   1090,   1092,   1091}},
 {   560,   1, {  1087,   1088,   1090,   1089,   1093,   1094,   1096,   1095}},
 {   561,   1, {  1089,   1090,   1092,   1091,   1095,   1096,   1098,   1097}},
 {   562,   1, {  1093,   1094,   1096,   1095,   1099,   1100,   1102,   1101}},
 {   563,   1, {  1095,   1096,   1098,   1097,   1101,   1102,   1104,   1103}},
 {   564,   1, {  1079,   1080,   1106,   1105,   1085,   1086,   1108,   1107}},
 {   565,   1, {  1085,   1086,   1108,   1107,   1091,   1092,   1110,   1109}},
 {   566,   1, {  1091,   1092,   1110,   1109,   1097,   1098,   1112,   1111}},
 {   567,   1, {  1097,   1098,   1112,   1111,   1103,   1104,   1114,   1113}},
 {   568,   1, {  1105,   1106,   1116,   1115,   1107,   1108,   1118,   1117}},
 {   569,   1, {  1107,   1108,   1118,   1117,   1109,   1110,   1120,   1119}},
 {   570,   1, {  1109,   1110,   1120,   1119,   1111,   1112,   1122,   1121}},
 {   571,   1, {  1111,   1112,   1122,   1121,   1113,   1114,   1124,   1123}},
 {   572,   1, {  1076,   1125,   1127,   1078,   1082,   1131,   1133,   1084}},
 {   573,   1, {  1125,   1126,   1128,   1127,   1131,   1132,   1134,   1133}},
 {   574,   1, {  1078,   1127,   1129,   1080,   1084,   1133,   1135,   1086}},
 {   575,   1, {  1127,   1128,   1130,   1129,   1133,   1134,   1136,   1135}},
 {   576,   1, {  1094,   1137,   1139,   1096,   1100,   1143,   1145,   1102}},
 {   577,   1, {  1137,   1138,   1140,   1139,   1143,   1144,   1146,   1145}},
 {   578,   1, {  1096,   1139,   1141,   1098,   1102,   1145,   1147,   1104}},
 {   579,   1, {  1139,   1140,   1142,   1141,   1145,   1146,   1148,   1147}},
 {   580,   1, {  1080,   1129,   1149,   1106,   1086,   1135,   1151,   1108}},
 {   581,   1, {  1129,   1130,   1150,   1149,   1135,   1136,   1152,   1151}},
 {   582,   1, {  1098,   1141,   1153,   1112,   1104,   1147,   1155,   1114}},
 {   583,   1, {  1141,   1142,   1154,   1153,   1147,   1148,   1156,   1155}},
 {   584,   1, {  1106,   1149,   1157,   1116,   1108,   1151,   1159,   1118}},
 {   585,   1, {  1149,   1150,   1158,   1157,   1151,   1152,   1160,   1159}},
 {   586,   1, {  1112,   1153,   1161,   1122,   1114,   1155,   1163,   1124}},
 {   587,   1, {  1153,   1154,   1162,   1161,   1155,   1156,   1164,   1163}},
 {   588,   1, {  1126,   1165,   1166,   1128,   1132,   1168,   1169,   1134}},
 {   589,   1, {  1128,   1166,   1167,   1130,   1134,   1169,   1170,   1136}},
 {   590,   1, {  1138,   1171,   1172,   1140,   1144,   1174,   1175,   1146}},
 {   591,   1, {  1140,   1172,   1173,   1142,   1146,   1175,   1176,   1148}},
 {   592,   1, {  1130,   1167,   1177,   1150,   1136,   1170,   1178,   1152}},
 {   593,   1, {  1142,   1173,   1179,   1154,   1148,   1176,   1180,   1156}},
 {   594,   1, {  1150,   1177,   1181,   1158,   1152,   1178,   1182,   1160}},
 {   595,   1, {  1154,   1179,   1183,   1162,   1156,   1180,   1184,   1164}},
 {   596,   1, {  1191,   1192,   1195,   1194,   1185,   1186,   1189,   1188}},
 {   597,   1, {  1192,   1193,   1196,   1195,   1186,   1187,   1190,   1189}},
 {   598,   1, {  1197,   1198,   1201,   1200,   1191,   1192,   1195,   1194}},
 {   599,   1, {  1198,   1199,   1202,   1201,   1192,   1193,   1196,   1195}},
 {   600,   1, {  1203,   1204,   1207,   1206,   1197,   1198,   1201,   1200}},
 {   601,   1, {  1204,   1205,   1208,   1207,   1198,   1199,   1202,   1201}},
 {   602,   1, {  1209,   1210,   1213,   1212,   1203,   1204,   1207,   1206}},
 {   603,   1, {  1210,   1211,   1214,   1213,   1204,   1205,   1208,   1207}},
 {   604,   1, {  1215,   1216,   1219,   1218,   1251,   1252,   1255,   1254}},
 {   605,   1, {  1216,   1217,   1220,   1219,   1252,   1253,   1256,   1255}},
 {   606,   1, {  1218,   1219,   1222,   1221,   1254,   1255,   1258,   1257}},
 {   607,   1, {  1219,   1220,   1223,   1222,   1255,   1256,   1259,   1258}},
 {   608,   1, {  1221,   1222,   1225,   1224,   1257,   1258,   1261,   1260}},
 {   609,   1, {  1222,   1223,   1226,   1225,   1258,   1259,   1262,   1261}},
 {   610,   1, {  1224,   1225,   1228,   1227,   1260,   1261,   1264,   1263}},
 {   611,   1, {  1225,   1226,   1229,   1228,   1261,   1262,   1265,   1264}},
 {   612,   1, {  1227,   1228,   1231,   1230,   1263,   1264,   1267,   1266}},
 {   613,   1, {  1228,   1229,   1232,   1231,   1264,   1265,   1268,   1267}},
 {   614,   1, {  1230,   1231,   1234,   1233,   1266,   1267,   1270,   1269}},
 {   615,   1, {  1231,   1232,   1235,   1234,   1267,   1268,   1271,   1270}},
 {   616,   1, {  1233,   1234,   1237,   1236,   1269,   1270,   1273,   1272}},
 {   617,   1, {  1234,   1235,   1238,   1237,   1270,   1271,   1274,   1273}},
 {   618,   1, {  1236,   1237,   1240,   1239,   1272,   1273,   1276,   1275}},
 {   619,   1, {  1237,   1238,   1241,   1240,   1273,   1274,   1277,   1276}},
 {   620,   1, {  1239,   1240,   1243,   1242,   1275,   1276,   1279,   1278}},
 {   621,   1, {  1240,   1241,   1244,   1243,   1276,   1277,   1280,   1279}},
 {   622,   1, {  1242,   1243,   1246,   1245,   1278,   1279,   1282,   1281}},
 {   623,   1, {  1243,   1244,   1247,   1246,   1279,   1280,   1283,   1282}},
 {   624,   1, {  1245,   1246,   1249,   1248,   1281,   1282,   1285,   1284}},
 {   625,   1, {  1246,   1247,   1250,   1249,   1282,   1283,   1286,   1285}},
 {   626,   1, {  1248,   1249,   1216,   1215,   1284,   1285,   1252,   1251}},
 {   627,   1, {  1249,   1250,   1217,   1216,   1285,   1286,   1253,   1252}},
 {   628,   1, {  1287,   1288,   1291,   1290,   1323,   1324,   1327,   1326}},
 {   629,   1, {  1288,   1289,   1292,   1291,   1324,   1325,   1328,   1327}},
 {   630,   1, {  1290,   1291,   1294,   1293,   1326,   1327,   1330,   1329}},
 {   631,   1, {  1291,   1292,   1295,   1294,   1327,   1328,   1331,   1330}},
 {   632,   1, {  1293,   1294,   1297,   1296,   1329,   1330,   1333,   1332}},
 {   633,   1, {  1294,   1295,   1298,   1297,   1330,   1331,   1334,   1333}},
 {   634,   1, {  1296,   1297,   1300,   1299,   1332,   1333,   1336,   1335}},
 {   635,   1, {  1297,   1298,   1301,   1300,   1333,   1334,   1337,   1336}},
 {   636,   1, {  1299,   1300,   1303,   1302,   1335,   1336,   1339,   1338}},
 {   637,   1, {  1300,   1301,   1304,   1303,   1336,   1337,   1340,   1339}},
 {   638,   1, {  1302,   1303,   1306,   1305,   1338,   1339,   1342,   1341}},
 {   639,   1, {  1303,   1304,   1307,   1306,   1339,   1340,   1343,   1342}},
 {   640,   1, {  1305,   1306,   1309,   1308,   1341,   1342,   1345,   1344}},
 {   641,   1, {  1306,   1307,   1310,   1309,   1342,   1343,   1346,   1345}},
 {   642,   1, {  1308,   1309,   1312,   1311,   1344,   1345,   1348,   1347}},
 {   643,   1, {  1309,   1310,   1313,   1312,   1345,   1346,   1349,   1348}},
 {   644,   1, {  1311,   1312,   1315,   1314,   1347,   1348,   1351,   1350}},
 {   645,   1, {  1312,   1313,   1316,   1315,   1348,   1349,   1352,   1351}},
 {   646,   1, {  1314,   1315,   1318,   1317,   1350,   1351,   1354,   1353}},
 {   647,   1, {  1315,   1316,   1319,   1318,   1351,   1352,   1355,   1354}},
 {   648,   1, {  1317,   1318,   1321,   1320,   1353,   1354,   1357,   1356}},
 {   649,   1, {  1318,   1319,   1322,   1321,   1354,   1355,   1358,   1357}},
 {   650,   1, {  1320,   1321,   1288,   1287,   1356,   1357,   1324,   1323}},
 {   651,   1, {  1321,   1322,   1289,   1288,   1357,   1358,   1325,   1324}},
 {   652,   1, {  1217,   1359,   1360,   1220,   1253,   1371,   1372,   1256}},
 {   653,   1, {  1220,   1360,   1361,   1223,   1256,   1372,   1373,   1259}},
 {   654,   1, {  1223,   1361,   1362,   1226,   1259,   1373,   1374,   1262}},
 {   655,   1, {  1226,   1362,   1363,   1229,   1262,   1374,   1375,   1265}},
 {   656,   1, {  1229,   1363,   1364,   1232,   1265,   1375,   1376,   1268}},
 {   657,   1, {  1232,   1364,   1365,   1235,   1268,   1376,   1377,   1271}},
 {   658,   1, {  1235,   1365,   1366,   1238,   1271,   1377,   1378,   1274}},
 {   659,   1, {  1238,   1366,   1367,   1241,   1274,   1378,   1379,   1277}},
 {   660,   1, {  1241,   1367,   1368,   1244,   1277,   1379,   1380,   1280}},
 {   661,   1, {  1244,   1368,   1369,   1247,   1280,   1380,   1381,   1283}},
 {   662,   1, {  1247,   1369,   1370,   1250,   1283,   1381,   1382,   1286}},
 {   663,   1, {  1250,   1370,   1359,   1217,   1286,   1382,   1371,   1253}},
 {   664,   1, {  1253,   1371,   1372,   1256,   1289,   1383,   1384,   1292}},
 {   665,   1, {  1256,   1372,   1373,   1259,   1292,   1384,   1385,   1295}},
 {   666,   1, {  1259,   1373,   1374,   1262,   1295,   1385,   1386,   1298}},
 {   667,   1, {  1262,   1374,   1375,   1265,   1298,   1386,   1387,   1301}},
 {   668,   1, {  1265,   1375,   1376,   1268,   1301,   1387,   1388,   1304}},
 {   669,   1, {  1268,   1376,   1377,   1271,   1304,   1388,   1389,   1307}},
 {   670,   1, {  1271,   1377,   1378,   1274,   1307,   1389,   1390,   1310}},
 {   671,   1, {  1274,   1378,   1379,   1277,   1310,   1390,   1391,   1313}},
 {   672,   1, {  1277,   1379,   1380,   1280,   1313,   1391,   1392,   1316}},
 {   673,   1, {  1280,   1380,   1381,   1283,   1316,   1392,   1393,   1319}},
 {   674,   1, {  1283,   1381,   1382,   1286,   1319,   1393,   1394,   1322}},
 {   675,   1, {  1286,   1382,   1371,   1253,   1322,   1394,   1383,   1289}},
 {   676,   1, {  1289,   1383,   1384,   1292,   1325,   1395,   1396,   1328}},
 {   677,   1, {  1292,   1384,   1385,   1295,   1328,   1396,   1397,   1331}},
 {   678,   1, {  1295,   1385,   1386,   1298,   1331,   1397,   1398,   1334}},
 {   679,   1, {  1298,   1386,   1387,   1301,   1334,   1398,   1399,   1337}},
 {   680,   1, {  1301,   1387,   1388,   1304,   1337,   1399,   1400,   1340}},
 {   681,   1, {  1304,   1388,   1389,   1307,   1340,   1400,   1401,   1343}},
 {   682,   1, {  1307,   1389,   1390,   1310,   1343,   1401,   1402,   1346}},
 {   683,   1, {  1310,   1390,   1391,   1313,   1346,   1402,   1403,   1349}},
 {   684,   1, {  1313,   1391,   1392,   1316,   1349,   1403,   1404,   1352}},
 {   685,   1, {  1316,   1392,   1393,   1319,   1352,   1404,   1405,   1355}},
 {   686,   1, {  1319,   1393,   1394,   1322,   1355,   1405,   1406,   1358}},
 {   687,   1, {  1322,   1394,   1383,   1289,   1358,   1406,   1395,   1325}},
 {   688,   1, {  1414,   1415,   1419,   1418,      6,   1407,   1411,   1410}},
 {   689,   1, {  1415,   1416,   1420,   1419,   1407,   1408,   1412,   1411}},
 {   690,   1, {  1416,   1417,   1421,   1420,   1408,   1409,   1413,   1412}},
 {   691,   1, {  1418,   1419,   1427,   1426,   1410,   1411,   1423,   1422}},
 {   692,   1, {  1419,   1420,   1428,   1427,   1411,   1412,   1424,   1423}},
 {   693,   1, {  1420,   1421,   1429,   1428,   1412,   1413,   1425,   1424}},
 {   694,   1, {  1426,   1427,   1435,   1434,   1422,   1423,   1431,   1430}},
 {   695,   1, {  1427,   1428,   1436,   1435,   1423,   1424,   1432,   1431}},
 {   696,   1, {  1428,   1429,   1437,   1436,   1424,   1425,   1433,   1432}},
 {   697,   1, {  1438,   1439,   1443,   1442,   1426,   1427,   1435,   1434}},
 {   698,   1, {  1439,   1440,   1444,   1443,   1427,   1428,   1436,   1435}},
 {   699,   1, {  1440,   1441,   1445,   1444,   1428,   1429,   1437,   1436}},
 {   700,   1, {  1446,   1447,   1451,   1450,   1438,   1439,   1443,   1442}},
 {   701,   1, {  1447,   1448,   1452,   1451,   1439,   1440,   1444,   1443}},
 {   702,   1, {  1448,   1449,   1453,   1452,   1440,   1441,   1445,   1444}},
 {   703,   1, {  1454,   1455,   1459,   1458,   1446,   1447,   1451,   1450}},
 {   704,   1, {  1455,   1456,   1460,   1459,   1447,   1448,   1452,   1451}},
 {   705,   1, {  1456,   1457,   1461,   1460,   1448,   1449,   1453,   1452}},
 {   706,   1, {  1462,   1463,   1467,   1466,   1454,   1455,   1459,   1458}},
 {   707,   1, {  1463,   1464,   1468,   1467,   1455,   1456,   1460,   1459}},
 {   708,   1, {  1464,   1465,   1469,   1468,   1456,   1457,   1461,   1460}},
 {   709,   1, {  1470,   1471,   1475,   1474,   1462,   1463,   1467,   1466}},
 {   710,   1, {  1471,   1472,   1476,   1475,   1463,   1464,   1468,   1467}},
 {   711,   1, {  1472,   1473,   1477,   1476,   1464,   1465,   1469,   1468}},
 {   712,   1, {  1434,   1435,   1483,   1482,   1430,   1431,   1479,   1478}},
 {   713,   1, {  1435,   1436,   1484,   1483,   1431,   1432,   1480,   1479}},
 {   714,   1, {  1436,   1437,   1485,   1484,   1432,   1433,   1481,   1480}},
 {   715,   1, {  1482,   1483,   1491,   1490,   1478,   1479,   1487,   1486}},
 {   716,   1, {  1483,   1484,   1492,   1491,   1479,   1480,   1488,   1487}},
 {   717,   1, {  1484,   1485,   1493,   1492,   1480,   1481,   1489,   1488}},
 {   718,   1, {  1494,   1495,   1499,   1498,   1482,   1483,   1491,   1490}},
 {   719,   1, {  1495,   1496,   1500,   1499,   1483,   1484,   1492,   1491}},
 {   720,   1, {  1496,   1497,   1501,   1500,   1484,   1485,   1493,   1492}},
 {   721,   1, {  1502,   1503,   1507,   1506,   1494,   1495,   1499,   1498}},
 {   722,   1, {  1503,   1504,   1508,   1507,   1495,   1496,   1500,   1499}},
 {   723,   1, {  1504,   1505,   1509,   1508,   1496,   1497,   1501,   1500}},
 {   724,   1, {  1510,   1511,   1515,   1514,   1502,   1503,   1507,   1506}},
 {   725,   1, {  1511,   1512,   1516,   1515,   1503,   1504,   1508,   1507}},
 {   726,   1, {  1512,   1513,   1517,   1516,   1504,   1505,   1509,   1508}},
 {   727,   1, {  1518,   1519,   1523,   1522,   1510,   1511,   1515,   1514}},
 {   728,   1, {  1519,   1520,   1524,   1523,   1511,   1512,   1516,   1515}},
 {   729,   1, {  1520,   1521,   1525,   1524,   1512,   1513,   1517,   1516}},
 {   730,   1, {  1526,   1527,   1531,   1530,   1518,   1519,   1523,   1522}},
 {   731,   1, {  1527,   1528,   1532,   1531,   1519,   1520,   1524,   1523}},
 {   732,   1, {  1528,   1529,   1533,   1532,   1520,   1521,   1525,   1524}},
 {   733,   1, {  1490,   1491,   1539,   1538,   1486,   1487,   1535,   1534}},
 {   734,   1, {  1491,   1492,   1540,   1539,   1487,   1488,   1536,   1535}},
 {   735,   1, {  1492,   1493,   1541,   1540,   1488,   1489,   1537,   1536}},
 {   736,   1, {  1538,   1539,   1547,   1546,   1534,   1535,   1543,   1542}},
 {   737,   1, {  1539,   1540,   1548,   1547,   1535,   1536,   1544,   1543}},
 {   738,   1, {  1540,   1541,   1549,   1548,   1536,   1537,   1545,   1544}},
 {   739,   1, {  1550,   1551,   1555,   1554,   1538,   1539,   1547,   1546}},
 {   740,   1, {  1551,   1552,   1556,   1555,   1539,   1540,   1548,   1547}},
 {   741,   1, {  1552,   1553,   1557,   1556,   1540,   1541,   1549,   1548}},
 {   742,   1, {  1558,   1559,   1563,   1562,   1550,   1551,   1555,   1554}},
 {   743,   1, {  1559,   1560,   1564,   1563,   1551,   1552,   1556,   1555}},
 {   744,   1, {  1560,   1561,   1565,   1564,   1552,   1553,   1557,   1556}},
 {   745,   1, {  1566,   1567,   1571,   1570,   1558,   1559,   1563,   1562}},
 {   746,   1, {  1567,   1568,   1572,   1571,   1559,   1560,   1564,   1563}},
 {   747,   1, {  1568,   1569,   1573,   1572,   1560,   1561,   1565,   1564}},
 {   748,   1, {  1574,   1575,   1579,   1578,   1566,   1567,   1571,   1570}},
 {   749,   1, {  1575,   1576,   1580,   1579,   1567,   1568,   1572,   1571}},
 {   750,   1, {  1576,   1577,   1581,   1580,   1568,   1569,   1573,   1572}},
 {   751,   1, {  1582,   1583,   1587,   1586,   1574,   1575,   1579,   1578}},
 {   752,   1, {  1583,   1584,   1588,   1587,   1575,   1576,   1580,   1579}},
 {   753,   1, {  1584,   1585,   1589,   1588,   1576,   1577,   1581,   1580}},
 {   754,   1, {  1546,   1547,   1595,   1594,   1542,   1543,   1591,   1590}},
 {   755,   1, {  1547,   1548,   1596,   1595,   1543,   1544,   1592,   1591}},
 {   756,   1, {  1548,   1549,   1597,   1596,   1544,   1545,   1593,   1592}},
 {   757,   1, {  1594,   1595,   1602,   1601,   1590,   1591,   1598,     84}},
 {   758,   1, {  1595,   1596,   1603,   1602,   1591,   1592,   1599,   1598}},
 {   759,   1, {  1596,   1597,   1604,   1603,   1592,   1593,   1600,   1599}},
 {   760,   1, {  1417,   1607,   1608,   1421,   1409,   1605,   1606,   1413}},
 {   761,   1, {  1421,   1608,   1610,   1429,   1413,   1606,   1609,   1425}},
 {   762,   1, {  1614,   1615,   1617,   1616,   1611,   1612,   1613,   1473}},
 {   763,   1, {  1429,   1610,   1619,   1437,   1425,   1609,   1618,   1433}},
 {   764,   1, {  1441,   1620,   1621,   1445,   1429,   1610,   1619,   1437}},
 {   765,   1, {  1449,   1622,   1623,   1453,   1441,   1620,   1621,   1445}},
 {   766,   1, {  1457,   1624,   1625,   1461,   1449,   1622,   1623,   1453}},
 {   767,   1, {  1465,   1626,   1627,   1469,   1457,   1624,   1625,   1461}},
 {   768,   1, {  1473,   1613,   1628,   1477,   1465,   1626,   1627,   1469}},
 {   769,   1, {  1616,   1617,   1630,   1629,   1473,   1613,   1628,   1477}},
 {   770,   1, {  1437,   1619,   1632,   1485,   1433,   1618,   1631,   1481}},
 {   771,   1, {  1629,   1630,   1635,   1634,   1477,   1628,   1633,   1529}},
 {   772,   1, {  1485,   1632,   1637,   1493,   1481,   1631,   1636,   1489}},
 {   773,   1, {  1497,   1638,   1639,   1501,   1485,   1632,   1637,   1493}},
 {   774,   1, {  1505,   1640,   1641,   1509,   1497,   1638,   1639,   1501}},
 {   775,   1, {  1513,   1642,   1643,   1517,   1505,   1640,   1641,   1509}},
 {   776,   1, {  1521,   1644,   1645,   1525,   1513,   1642,   1643,   1517}},
 {   777,   1, {  1529,   1633,   1646,   1533,   1521,   1644,   1645,   1525}},
 {   778,   1, {  1634,   1635,   1648,   1647,   1529,   1633,   1646,   1533}},
 {   779,   1, {  1493,   1637,   1650,   1541,   1489,   1636,   1649,   1537}},
 {   780,   1, {  1647,   1648,   1653,   1652,   1533,   1646,   1651,   1585}},
 {   781,   1, {  1541,   1650,   1655,   1549,   1537,   1649,   1654,   1545}},
 {   782,   1, {  1553,   1656,   1657,   1557,   1541,   1650,   1655,   1549}},
 {   783,   1, {  1561,   1658,   1659,   1565,   1553,   1656,   1657,   1557}},
 {   784,   1, {  1569,   1660,   1661,   1573,   1561,   1658,   1659,   1565}},
 {   785,   1, {  1577,   1662,   1663,   1581,   1569,   1660,   1661,   1573}},
 {   786,   1, {  1585,   1651,   1664,   1589,   1577,   1662,   1663,   1581}},
 {   787,   1, {  1652,   1653,   1666,   1665,   1585,   1651,   1664,   1589}},
 {   788,   1, {  1549,   1655,   1668,   1597,   1545,   1654,   1667,   1593}},
 {   789,   1, {  1665,   1666,   1672,   1671,   1589,   1664,   1670,   1669}},
 {   790,   1, {  1597,   1668,   1674,   1604,   1593,   1667,   1673,   1600}},
 {   791,   1, {  1615,   1677,   1678,   1617,   1612,   1675,   1676,   1613}},
 {   792,   1, {  1626,   1681,   1682,   1627,   1624,   1679,   1680,   1625}},
 {   793,   1, {  1613,   1676,   1683,   1628,   1626,   1681,   1682,   1627}},
 {   794,   1, {  1617,   1678,   1684,   1630,   1613,   1676,   1683,   1628}},
 {   795,   1, {  1630,   1684,   1686,   1635,   1628,   1683,   1685,   1633}},
 {   796,   1, {  1644,   1689,   1690,   1645,   1642,   1687,   1688,   1643}},
 {   797,   1, {  1633,   1685,   1691,   1646,   1644,   1689,   1690,   1645}},
 {   798,   1, {  1635,   1686,   1692,   1648,   1633,   1685,   1691,   1646}},
 {   799,   1, {  1648,   1692,   1694,   1653,   1646,   1691,   1693,   1651}},
 {   800,   1, {  1662,   1697,   1698,   1663,   1660,   1695,   1696,   1661}},
 {   801,   1, {  1651,   1693,   1699,   1664,   1662,   1697,   1698,   1663}},
 {   802,   1, {  1653,   1694,   1700,   1666,   1651,   1693,   1699,   1664}},
 {   803,   1, {  1666,   1700,   1702,   1672,   1664,   1699,   1701,   1670}},
 {   804,   1, {  1677,   1709,   1712,   1678,   1675,   1703,   1706,   1676}},
 {   805,   1, {  1709,   1710,   1713,   1712,   1703,   1704,   1707,   1706}},
 {   806,   1, {  1710,   1711,   1714,   1713,   1704,   1705,   1708,   1707}},
 {   807,   1, {  1678,   1712,   1718,   1684,   1676,   1706,   1715,   1683}},
 {   808,   1, {  1712,   1713,   1719,   1718,   1706,   1707,   1716,   1715}},
 {   809,   1, {  1713,   1714,   1720,   1719,   1707,   1708,   1717,   1716}},
 {   810,   1, {  1684,   1718,   1724,   1686,   1683,   1715,   1721,   1685}},
 {   811,   1, {  1718,   1719,   1725,   1724,   1715,   1716,   1722,   1721}},
 {   812,   1, {  1719,   1720,   1726,   1725,   1716,   1717,   1723,   1722}},
 {   813,   1, {  1689,   1733,   1736,   1690,   1687,   1727,   1730,   1688}},
 {   814,   1, {  1733,   1734,   1737,   1736,   1727,   1728,   1731,   1730}},
 {   815,   1, {  1734,   1735,   1738,   1737,   1728,   1729,   1732,   1731}},
 {   816,   1, {  1685,   1721,   1739,   1691,   1689,   1733,   1736,   1690}},
 {   817,   1, {  1721,   1722,   1740,   1739,   1733,   1734,   1737,   1736}},
 {   818,   1, {  1722,   1723,   1741,   1740,   1734,   1735,   1738,   1737}},
 {   819,   1, {  1686,   1724,   1742,   1692,   1685,   1721,   1739,   1691}},
 {   820,   1, {  1724,   1725,   1743,   1742,   1721,   1722,   1740,   1739}},
 {   821,   1, {  1725,   1726,   1744,   1743,   1722,   1723,   1741,   1740}},
 {   822,   1, {  1692,   1742,   1748,   1694,   1691,   1739,   1745,   1693}},
 {   823,   1, {  1742,   1743,   1749,   1748,   1739,   1740,   1746,   1745}},
 {   824,   1, {  1743,   1744,   1750,   1749,   1740,   1741,   1747,   1746}},
 {   825,   1, {  1694,   1748,   1754,   1700,   1693,   1745,   1751,   1699}},
 {   826,   1, {  1748,   1749,   1755,   1754,   1745,   1746,   1752,   1751}},
 {   827,   1, {  1749,   1750,   1756,   1755,   1746,   1747,   1753,   1752}},
 {   828,   1, {  1700,   1754,   1760,   1702,   1699,   1751,   1757,   1701}},
 {   829,   1, {  1754,   1755,   1761,   1760,   1751,   1752,   1758,   1757}},
 {   830,   1, {  1755,   1756,   1762,   1761,   1752,   1753,   1759,   1758}},
 {   831,   2, {  1763,   1764,   1766,   1765,   1771,   1772,   1774,   1773}},
 {   832,   2, {  1765,   1766,   1768,   1767,   1773,   1774,   1776,   1775}},
 {   833,   2, {  1767,   1768,   1770,   1769,   1775,   1776,   1778,   1777}},
 {   834,   2, {  1771,   1772,   1774,   1773,   1779,   1780,   1782,   1781}},
 {   835,   2, {  1773,   1774,   1776,   1775,   1781,   1782,   1784,   1783}},
 {   836,   2, {  1775,   1776,   1778,   1777,   1783,   1784,   1786,   1785}},
 {   837,   2, {  1779,   1780,   1782,   1781,   1787,   1788,   1790,   1789}},
 {   838,   2, {  1781,   1782,   1784,   1783,   1789,   1790,   1792,   1791}},
 {   839,   2, {  1783,   1784,   1786,   1785,   1791,   1792,   1794,   1793}},
 {   840,   2, {  1787,   1788,   1790,   1789,   1795,   1796,   1798,   1797}},
 {   841,   2, {  1789,   1790,   1792,   1791,   1797,   1798,   1800,   1799}},
 {   842,   2, {  1791,   1792,   1794,   1793,   1799,   1800,   1802,   1801}},
 {   843,   2, {  1795,   1796,   1798,   1797,   1803,   1804,   1806,   1805}},
 {   844,   2, {  1797,   1798,   1800,   1799,   1805,   1806,   1808,   1807}},
 {   845,   2, {  1799,   1800,   1802,   1801,   1807,   1808,   1810,   1809}},
 {   846,   2, {  1803,   1804,   1806,   1805,   1811,   1812,   1814,   1813}},
 {   847,   2, {  1805,   1806,   1808,   1807,   1813,   1814,   1816,   1815}},
 {   848,   2, {  1807,   1808,   1810,   1809,   1815,   1816,   1818,   1817}},
 {   849,   2, {  1769,   1770,   1820,   1819,   1777,   1778,   1832,   1831}},
 {   850,   2, {  1819,   1820,   1822,   1821,   1831,   1832,   1834,   1833}},
 {   851,   2, {  1821,   1822,   1824,   1823,   1833,   1834,   1836,   1835}},
 {   852,   2, {  1823,   1824,   1826,   1825,   1835,   1836,   1838,   1837}},
 {   853,   2, {  1825,   1826,   1828,   1827,   1837,   1838,   1840,   1839}},
 {   854,   2, {  1827,   1828,   1830,   1829,   1839,   1840,   1842,   1841}},
 {   855,   2, {  1777,   1778,   1832,   1831,   1785,   1786,   1844,   1843}},
 {   856,   2, {  1831,   1832,   1834,   1833,   1843,   1844,   1846,   1845}},
 {   857,   2, {  1833,   1834,   1836,   1835,   1845,   1846,   1848,   1847}},
 {   858,   2, {  1835,   1836,   1838,   1837,   1847,   1848,   1850,   1849}},
 {   859,   2, {  1837,   1838,   1840,   1839,   1849,   1850,   1852,   1851}},
 {   860,   2, {  1839,   1840,   1842,   1841,   1851,   1852,   1854,   1853}},
 {   861,   2, {  1785,   1786,   1844,   1843,   1793,   1794,   1856,   1855}},
 {   862,   2, {  1843,   1844,   1846,   1845,   1855,   1856,   1858,   1857}},
 {   863,   2, {  1845,   1846,   1848,   1847,   1857,   1858,   1860,   1859}},
 {   864,   2, {  1847,   1848,   1850,   1849,   1859,   1860,   1862,   1861}},
 {   865,   2, {  1849,   1850,   1852,   1851,   1861,   1862,   1864,   1863}},
 {   866,   2, {  1851,   1852,   1854,   1853,   1863,   1864,   1866,   1865}},
 {   867,   2, {  1793,   1794,   1856,   1855,   1801,   1802,   1868,   1867}},
 {   868,   2, {  1855,   1856,   1858,   1857,   1867,   1868,   1870,   1869}},
 {   869,   2, {  1857,   1858,   1860,   1859,   1869,   1870,   1872,   1871}},
 {   870,   2, {  1859,   1860,   1862,   1861,   1871,   1872,   1874,   1873}},
 {   871,   2, {  1861,   1862,   1864,   1863,   1873,   1874,   1876,   1875}},
 {   872,   2, {  1863,   1864,   1866,   1865,   1875,   1876,   1878,   1877}},
 {   873,   2, {  1801,   1802,   1868,   1867,   1809,   1810,   1880,   1879}},
 {   874,   2, {  1867,   1868,   1870,   1869,   1879,   1880,   1882,   1881}},
 {   875,   2, {  1869,   1870,   1872,   1871,   1881,   1882,   1884,   1883}},
 {   876,   2, {  1871,   1872,   1874,   1873,   1883,   1884,   1886,   1885}},
 {   877,   2, {  1873,   1874,   1876,   1875,   1885,   1886,   1888,   1887}},
 {   878,   2, {  1875,   1876,   1878,   1877,   1887,   1888,   1890,   1889}},
 {   879,   2, {  1809,   1810,   1880,   1879,   1817,   1818,   1892,   1891}},
 {   880,   2, {  1879,   1880,   1882,   1881,   1891,   1892,   1894,   1893}},
 {   881,   2, {  1881,   1882,   1884,   1883,   1893,   1894,   1896,   1895}},
 {   882,   2, {  1883,   1884,   1886,   1885,   1895,   1896,   1898,   1897}},
 {   883,   2, {  1885,   1886,   1888,   1887,   1897,   1898,   1900,   1899}},
 {   884,   2, {  1887,   1888,   1890,   1889,   1899,   1900,   1902,   1901}},
 {   885,   2, {  1829,   1830,   1904,   1903,   1841,   1842,   1908,   1907}},
 {   886,   2, {  1903,   1904,   1906,   1905,   1907,   1908,   1910,   1909}},
 {   887,   2, {  1905,   1906,   1764,   1763,   1909,   1910,   1772,   1771}},
 {   888,   2, {  1841,   1842,   1908,   1907,   1853,   1854,   1912,   1911}},
 {   889,   2, {  1907,   1908,   1910,   1909,   1911,   1912,   1914,   1913}},
 {   890,   2, {  1909,   1910,   1772,   1771,   1913,   1914,   1780,   1779}},
 {   891,   2, {  1853,   1854,   1912,   1911,   1865,   1866,   1916,   1915}},
 {   892,   2, {  1911,   1912,   1914,   1913,   1915,   1916,   1918,   1917}},
 {   893,   2, {  1913,   1914,   1780,   1779,   1917,   1918,   1788,   1787}},
 {   894,   2, {  1865,   1866,   1916,   1915,   1877,   1878,   1920,   1919}},
 {   895,   2, {  1915,   1916,   1918,   1917,   1919,   1920,   1922,   1921}},
 {   896,   2, {  1917,   1918,   1788,   1787,   1921,   1922,   1796,   1795}},
 {   897,   2, {  1877,   1878,   1920,   1919,   1889,   1890,   1924,   1923}},
 {   898,   2, {  1919,   1920,   1922,   1921,   1923,   1924,   1926,   1925}},
 {   899,   2, {  1921,   1922,   1796,   1795,   1925,   1926,   1804,   1803}},
 {   900,   2, {  1889,   1890,   1924,   1923,   1901,   1902,   1928,   1927}},
 {   901,   2, {  1923,   1924,   1926,   1925,   1927,   1928,   1930,   1929}},
 {   902,   2, {  1925,   1926,   1804,   1803,   1929,   1930,   1812,   1811}},
 {   903,   2, {  1772,   1931,   1932,   1774,   1780,   1935,   1936,   1782}},
 {   904,   2, {  1774,   1932,   1933,   1776,   1782,   1936,   1937,   1784}},
 {   905,   2, {  1776,   1933,   1934,   1778,   1784,   1937,   1938,   1786}},
 {   906,   2, {  1778,   1934,   1939,   1832,   1786,   1938,   1945,   1844}},
 {   907,   2, {  1832,   1939,   1940,   1834,   1844,   1945,   1946,   1846}},
 {   908,   2, {  1834,   1940,   1941,   1836,   1846,   1946,   1947,   1848}},
 {   909,   2, {  1836,   1941,   1942,   1838,   1848,   1947,   1948,   1850}},
 {   910,   2, {  1838,   1942,   1943,   1840,   1850,   1948,   1949,   1852}},
 {   911,   2, {  1840,   1943,   1944,   1842,   1852,   1949,   1950,   1854}},
 {   912,   2, {  1842,   1944,   1951,   1908,   1854,   1950,   1953,   1912}},
 {   913,   2, {  1908,   1951,   1952,   1910,   1912,   1953,   1954,   1914}},
 {   914,   2, {  1910,   1952,   1931,   1772,   1914,   1954,   1935,   1780}},
 {   915,   3, {  1957,   1958,   1960,   1959,   1965,   1966,   1968,   1967}},
 {   916,   3, {  1959,   1960,   1962,   1961,   1967,   1968,   1970,   1969}},
 {   917,   3, {  1961,   1962,   1964,   1963,   1969,   1970,   1972,   1971}},
 {   918,   3, {  1965,   1966,   1968,   1967,   1973,   1974,   1976,   1975}},
 {   919,   3, {  1967,   1968,   1970,   1969,   1975,   1976,   1978,   1977}},
 {   920,   3, {  1969,   1970,   1972,   1971,   1977,   1978,   1980,   1979}},
 {   921,   3, {  1973,   1974,   1976,   1975,   1981,   1982,   1984,   1983}},
 {   922,   3, {  1975,   1976,   1978,   1977,   1983,   1984,   1986,   1985}},
 {   923,   3, {  1977,   1978,   1980,   1979,   1985,   1986,   1988,   1987}},
 {   924,   3, {  1981,   1982,   1984,   1983,   1989,   1990,   1992,   1991}},
 {   925,   3, {  1983,   1984,   1986,   1985,   1991,   1992,   1994,   1993}},
 {   926,   3, {  1985,   1986,   1988,   1987,   1993,   1994,   1996,   1995}},
 {   927,   3, {  1989,   1990,   1992,   1991,   1997,   1998,   2000,   1999}},
 {   928,   3, {  1991,   1992,   1994,   1993,   1999,   2000,   2002,   2001}},
 {   929,   3, {  1993,   1994,   1996,   1995,   2001,   2002,   2004,   2003}},
 {   930,   3, {  1997,   1998,   2000,   1999,   2005,   2006,   2008,   2007}},
 {   931,   3, {  1999,   2000,   2002,   2001,   2007,   2008,   2010,   2009}},
 {   932,   3, {  2001,   2002,   2004,   2003,   2009,   2010,   2012,   2011}},
 {   933,   3, {  2005,   2006,   2008,   2007,   2013,   2014,   2016,   2015}},
 {   934,   3, {  2007,   2008,   2010,   2009,   2015,   2016,   2018,   2017}},
 {   935,   3, {  2009,   2010,   2012,   2011,   2017,   2018,   2020,   2019}},
 {   936,   3, {  1963,   1964,   2022,   2021,   1971,   1972,   2034,   2033}},
 {   937,   3, {  2021,   2022,   2024,   2023,   2033,   2034,   2036,   2035}},
 {   938,   3, {  2023,   2024,   2026,   2025,   2035,   2036,   2038,   2037}},
 {   939,   3, {  2025,   2026,   2028,   2027,   2037,   2038,   2040,   2039}},
 {   940,   3, {  2027,   2028,   2030,   2029,   2039,   2040,   2042,   2041}},
 {   941,   3, {  2029,   2030,   2032,   2031,   2041,   2042,   2044,   2043}},
 {   942,   3, {  1971,   1972,   2034,   2033,   1979,   1980,   2046,   2045}},
 {   943,   3, {  2033,   2034,   2036,   2035,   2045,   2046,   2048,   2047}},
 {   944,   3, {  2035,   2036,   2038,   2037,   2047,   2048,   2050,   2049}},
 {   945,   3, {  2037,   2038,   2040,   2039,   2049,   2050,   2052,   2051}},
 {   946,   3, {  2039,   2040,   2042,   2041,   2051,   2052,   2054,   2053}},
 {   947,   3, {  2041,   2042,   2044,   2043,   2053,   2054,   2056,   2055}},
 {   948,   3, {  1979,   1980,   2046,   2045,   1987,   1988,   2058,   2057}},
 {   949,   3, {  2045,   2046,   2048,   2047,   2057,   2058,   2060,   2059}},
 {   950,   3, {  2047,   2048,   2050,   2049,   2059,   2060,   2062,   2061}},
 {   951,   3, {  2049,   2050,   2052,   2051,   2061,   2062,   2064,   2063}},
 {   952,   3, {  2051,   2052,   2054,   2053,   2063,   2064,   2066,   2065}},
 {   953,   3, {  2053,   2054,   2056,   2055,   2065,   2066,   2068,   2067}},
 {   954,   3, {  1987,   1988,   2058,   2057,   1995,   1996,   2070,   2069}},
 {   955,   3, {  2057,   2058,   2060,   2059,   2069,   2070,   2072,   2071}},
 {   956,   3, {  2059,   2060,   2062,   2061,   2071,   2072,   2074,   2073}},
 {   957,   3, {  2061,   2062,   2064,   2063,   2073,   2074,   2076,   2075}},
 {   958,   3, {  2063,   2064,   2066,   2065,   2075,   2076,   2078,   2077}},
 {   959,   3, {  2065,   2066,   2068,   2067,   2077,   2078,   2080,   2079}},
 {   960,   3, {  1995,   1996,   2070,   2069,   2003,   2004,   2082,   2081}},
 {   961,   3, {  2069,   2070,   2072,   2071,   2081,   2082,   2084,   2083}},
 {   962,   3, {  2071,   2072,   2074,   2073,   2083,   2084,   2086,   2085}},
 {   963,   3, {  2073,   2074,   2076,   2075,   2085,   2086,   2088,   2087}},
 {   964,   3, {  2075,   2076,   2078,   2077,   2087,   2088,   2090,   2089}},
 {   965,   3, {  2077,   2078,   2080,   2079,   2089,   2090,   2092,   2091}},
 {   966,   3, {  2003,   2004,   2082,   2081,   2011,   2012,   2094,   2093}},
 {   967,   3, {  2081,   2082,   2084,   2083,   2093,   2094,   2096,   2095}},
 {   968,   3, {  2083,   2084,   2086,   2085,   2095,   2096,   2098,   2097}},
 {   969,   3, {  2085,   2086,   2088,   2087,   2097,   2098,   2100,   2099}},
 {   970,   3, {  2087,   2088,   2090,   2089,   2099,   2100,   2102,   2101}},
 {   971,   3, {  2089,   2090,   2092,   2091,   2101,   2102,   2104,   2103}},
 {   972,   3, {  2011,   2012,   2094,   2093,   2019,   2020,   2106,   2105}},
 {   973,   3, {  2093,   2094,   2096,   2095,   2105,   2106,   2108,   2107}},
 {   974,   3, {  2095,   2096,   2098,   2097,   2107,   2108,   2110,   2109}},
 {   975,   3, {  2097,   2098,   2100,   2099,   2109,   2110,   2112,   2111}},
 {   976,   3, {  2099,   2100,   2102,   2101,   2111,   2112,   2114,   2113}},
 {   977,   3, {  2101,   2102,   2104,   2103,   2113,   2114,   2116,   2115}},
 {   978,   3, {  2031,   2032,   2118,   2117,   2043,   2044,   2122,   2121}},
 {   979,   3, {  2117,   2118,   2120,   2119,   2121,   2122,   2124,   2123}},
 {   980,   3, {  2119,   2120,   1958,   1957,   2123,   2124,   1966,   1965}},
 {   981,   3, {  2043,   2044,   2122,   2121,   2055,   2056,   2126,   2125}},
 {   982,   3, {  2121,   2122,   2124,   2123,   2125,   2126,   2128,   2127}},
 {   983,   3, {  2123,   2124,   1966,   1965,   2127,   2128,   1974,   1973}},
 {   984,   3, {  2055,   2056,   2126,   2125,   2067,   2068,   2130,   2129}},
 {   985,   3, {  2125,   2126,   2128,   2127,   2129,   2130,   2132,   2131}},
 {   986,   3, {  2127,   2128,   1974,   1973,   2131,   2132,   1982,   1981}},
 {   987,   3, {  2067,   2068,   2130,   2129,   2079,   2080,   2134,   2133}},
 {   988,   3, {  2129,   2130,   2132,   2131,   2133,   2134,   2136,   2135}},
 {   989,   3, {  2131,   2132,   1982,   1981,   2135,   2136,   1990,   1989}},
 {   990,   3, {  2079,   2080,   2134,   2133,   2091,   2092,   2138,   2137}},
 {   991,   3, {  2133,   2134,   2136,   2135,   2137,   2138,   2140,   2139}},
 {   992,   3, {  2135,   2136,   1990,   1989,   2139,   2140,   1998,   1997}},
 {   993,   3, {  2091,   2092,   2138,   2137,   2103,   2104,   2142,   2141}},
 {   994,   3, {  2137,   2138,   2140,   2139,   2141,   2142,   2144,   2143}},
 {   995,   3, {  2139,   2140,   1998,   1997,   2143,   2144,   2006,   2005}},
 {   996,   3, {  2103,   2104,   2142,   2141,   2115,   2116,   2146,   2145}},
 {   997,   3, {  2141,   2142,   2144,   2143,   2145,   2146,   2148,   2147}},
 {   998,   3, {  2143,   2144,   2006,   2005,   2147,   2148,   2014,   2013}},
 {   999,   3, {  1990,   2149,   2150,   1992,   1998,   2153,   2154,   2000}},
 {  1000,   3, {  1992,   2150,   2151,   1994,   2000,   2154,   2155,   2002}},
 {  1001,   3, {  1994,   2151,   2152,   1996,   2002,   2155,   2156,   2004}},
 {  1002,   3, {  1998,   2153,   2154,   2000,   2006,   2157,   2158,   2008}},
 {  1003,   3, {  2000,   2154,   2155,   2002,   2008,   2158,   2159,   2010}},
 {  1004,   3, {  2002,   2155,   2156,   2004,   2010,   2159,   2160,   2012}},
 {  1005,   3, {  2006,   2157,   2158,   2008,   2014,   2161,   2162,   2016}},
 {  1006,   3, {  2008,   2158,   2159,   2010,   2016,   2162,   2163,   2018}},
 {  1007,   3, {  2010,   2159,   2160,   2012,   2018,   2163,   2164,   2020}},
 {  1008,   3, {  1996,   2152,   2165,   2070,   2004,   2156,   2171,   2082}},
 {  1009,   3, {  2070,   2165,   2166,   2072,   2082,   2171,   2172,   2084}},
 {  1010,   3, {  2072,   2166,   2167,   2074,   2084,   2172,   2173,   2086}},
 {  1011,   3, {  2074,   2167,   2168,   2076,   2086,   2173,   2174,   2088}},
 {  1012,   3, {  2076,   2168,   2169,   2078,   2088,   2174,   2175,   2090}},
 {  1013,   3, {  2078,   2169,   2170,   2080,   2090,   2175,   2176,   2092}},
 {  1014,   3, {  2004,   2156,   2171,   2082,   2012,   2160,   2177,   2094}},
 {  1015,   3, {  2082,   2171,   2172,   2084,   2094,   2177,   2178,   2096}},
 {  1016,   3, {  2084,   2172,   2173,   2086,   2096,   2178,   2179,   2098}},
 {  1017,   3, {  2086,   2173,   2174,   2088,   2098,   2179,   2180,   2100}},
 {  1018,   3, {  2088,   2174,   2175,   2090,   2100,   2180,   2181,   2102}},
 {  1019,   3, {  2090,   2175,   2176,   2092,   2102,   2181,   2182,   2104}},
 {  1020,   3, {  2012,   2160,   2177,   2094,   2020,   2164,   2183,   2106}},
 {  1021,   3, {  2094,   2177,   2178,   2096,   2106,   2183,   2184,   2108}},
 {  1022,   3, {  2096,   2178,   2179,   2098,   2108,   2184,   2185,   2110}},
 {  1023,   3, {  2098,   2179,   2180,   2100,   2110,   2185,   2186,   2112}},
 {  1024,   3, {  2100,   2180,   2181,   2102,   2112,   2186,   2187,   2114}},
 {  1025,   3, {  2102,   2181,   2182,   2104,   2114,   2187,   2188,   2116}},
 {  1026,   3, {  2080,   2170,   2189,   2134,   2092,   2176,   2191,   2138}},
 {  1027,   3, {  2134,   2189,   2190,   2136,   2138,   2191,   2192,   2140}},
 {  1028,   3, {  2136,   2190,   2149,   1990,   2140,   2192,   2153,   1998}},
 {  1029,   3, {  2092,   2176,   2191,   2138,   2104,   2182,   2193,   2142}},
 {  1030,   3, {  2138,   2191,   2192,   2140,   2142,   2193,   2194,   2144}},
 {  1031,   3, {  2140,   2192,   2153,   1998,   2144,   2194,   2157,   2006}},
 {  1032,   3, {  2104,   2182,   2193,   2142,   2116,   2188,   2195,   2146}},
 {  1033,   3, {  2142,   2193,   2194,   2144,   2146,   2195,   2196,   2148}},
 {  1034,   3, {  2144,   2194,   2157,   2006,   2148,   2196,   2161,   2014}},
 {  1515,  31, {  1194,   1195,   2883,   2882,   1188,   1189,   2877,   2876}},
 {  1516,  31, {  1195,   1196,   2884,   2883,   1189,   1190,   2878,   2877}},
 {  1517,  31, {  2882,   2883,   2886,   2885,   2876,   2877,   2880,   2879}},
 {  1518,  31, {  2883,   2884,   2887,   2886,   2877,   2878,   2881,   2880}},
 {  1519,  31, {  1200,   1201,   2889,   2888,   1194,   1195,   2883,   2882}},
 {  1520,  31, {  1201,   1202,   2890,   2889,   1195,   1196,   2884,   2883}},
 {  1521,  31, {  2888,   2889,   2892,   2891,   2882,   2883,   2886,   2885}},
 {  1522,  31, {  2889,   2890,   2893,   2892,   2883,   2884,   2887,   2886}},
 {  1523,  31, {  1206,   1207,   2895,   2894,   1200,   1201,   2889,   2888}},
 {  1524,  31, {  1207,   1208,   2896,   2895,   1201,   1202,   2890,   2889}},
 {  1525,  31, {  2894,   2895,   2898,   2897,   2888,   2889,   2892,   2891}},
 {  1526,  31, {  2895,   2896,   2899,   2898,   2889,   2890,   2893,   2892}},
 {  1527,  31, {  1212,   1213,   2901,   2900,   1206,   1207,   2895,   2894}},
 {  1528,  31, {  1213,   1214,   2902,   2901,   1207,   1208,   2896,   2895}},
 {  1529,  31, {  2900,   2901,   2904,   2903,   2894,   2895,   2898,   2897}},
 {  1530,  31, {  2901,   2902,   2905,   2904,   2895,   2896,   2899,   2898}},
 {  1531,   5, {  2906,   2907,   2909,   2908,   2910,   2911,   2913,   2912}},
 {  1532,   5, {  2910,   2911,   2913,   2912,   2914,   2915,   2917,   2916}},
 {  1534,   5, {  2918,   2919,   2921,   2920,   2922,   2923,   2925,   2924}},
 {  1535,   5, {  2922,   2923,   2925,   2924,   2926,   2927,   2929,   2928}},
 {  1537,   5, {  2930,   2931,   2933,   2932,   2934,   2935,   2937,   2936}},
 {  1538,   5, {  2934,   2935,   2937,   2936,   2938,   2939,   2941,   2940}},
 {  1540,   5, {  2942,   2943,   2945,   2944,   2946,   2947,   2949,   2948}},
 {  1541,   5, {  2946,   2947,   2949,   2948,   2950,   2951,   2953,   2952}},
 {  1543,   5, {  2954,   2955,   2957,   2956,   2958,   2959,   2961,   2960}},
 {  1544,   5, {  2958,   2959,   2961,   2960,   2962,   2963,   2965,   2964}},
 {  1545,   5, {  2907,   2966,   2968,   2909,   2911,   2970,   2972,   2913}},
 {  1546,   5, {  2966,   2967,   2969,   2968,   2970,   2971,   2973,   2972}},
 {  1547,   5, {  2911,   2970,   2972,   2913,   2915,   2974,   2976,   2917}},
 {  1548,   5, {  2970,   2971,   2973,   2972,   2974,   2975,   2977,   2976}},
 {  1551,   5, {  2919,   2978,   2980,   2921,   2923,   2982,   2984,   2925}},
 {  1552,   5, {  2978,   2979,   2981,   2980,   2982,   2983,   2985,   2984}},
 {  1553,   5, {  2923,   2982,   2984,   2925,   2927,   2986,   2988,   2929}},
 {  1554,   5, {  2982,   2983,   2985,   2984,   2986,   2987,   2989,   2988}},
 {  1557,   5, {  2931,   2990,   2992,   2933,   2935,   2994,   2996,   2937}},
 {  1558,   5, {  2990,   2991,   2993,   2992,   2994,   2995,   2997,   2996}},
 {  1559,   5, {  2935,   2994,   2996,   2937,   2939,   2998,   3000,   2941}},
 {  1560,   5, {  2994,   2995,   2997,   2996,   2998,   2999,   3001,   3000}},
 {  1563,   5, {  2943,   3002,   3004,   2945,   2947,   3006,   3008,   2949}},
 {  1564,   5, {  3002,   3003,   3005,   3004,   3006,   3007,   3009,   3008}},
 {  1565,   5, {  2947,   3006,   3008,   2949,   2951,   3010,   3012,   2953}},
 {  1566,   5, {  3006,   3007,   3009,   3008,   3010,   3011,   3013,   3012}},
 {  1569,   5, {  2955,   3014,   3016,   2957,   2959,   3018,   3020,   2961}},
 {  1570,   5, {  3014,   3015,   3017,   3016,   3018,   3019,   3021,   3020}},
 {  1571,   5, {  2959,   3018,   3020,   2961,   2963,   3022,   3024,   2965}},
 {  1572,   5, {  3018,   3019,   3021,   3020,   3022,   3023,   3025,   3024}},
 {  1573,   5, {  2967,   3026,   3027,   2969,   2971,   3028,   3029,   2973}},
 {  1574,   5, {  2971,   3028,   3029,   2973,   2975,   3030,   3031,   2977}},
 {  1576,   5, {  2979,   3032,   3033,   2981,   2983,   3034,   3035,   2985}},
 {  1577,   5, {  2983,   3034,   3035,   2985,   2987,   3036,   3037,   2989}},
 {  1579,   5, {  2991,   3038,   3039,   2993,   2995,   3040,   3041,   2997}},
 {  1580,   5, {  2995,   3040,   3041,   2997,   2999,   3042,   3043,   3001}},
 {  1582,   5, {  3003,   3044,   3045,   3005,   3007,   3046,   3047,   3009}},
 {  1583,   5, {  3007,   3046,   3047,   3009,   3011,   3048,   3049,   3013}},
 {  1585,   5, {  3015,   3050,   3051,   3017,   3019,   3052,   3053,   3021}},
 {  1586,   5, {  3019,   3052,   3053,   3021,   3023,   3054,   3055,   3025}},
 {  1587,   5, {  3056,   3057,   3059,   3058,   3060,   3061,   3063,   3062}},
 {  1588,   5, {  3060,   3061,   3063,   3062,   3064,   3065,   3067,   3066}},
 {  1590,   5, {  3068,   3069,   3071,   3070,   3072,   3073,   3075,   3074}},
 {  1591,   5, {  3072,   3073,   3075,   3074,   3076,   3077,   3079,   3078}},
 {  1593,   5, {  3080,   3081,   3083,   3082,   3084,   3085,   3087,   3086}},
 {  1594,   5, {  3084,   3085,   3087,   3086,   3088,   3089,   3091,   3090}},
 {  1596,   5, {  3092,   3093,   3095,   3094,   3096,   3097,   3099,   3098}},
 {  1597,   5, {  3096,   3097,   3099,   3098,   3100,   3101,   3103,   3102}},
 {  1599,   5, {  3104,   3105,   3107,   3106,   3108,   3109,   3111,   3110}},
 {  1600,   5, {  3108,   3109,   3111,   3110,   3112,   3113,   3115,   3114}},
 {  1601,   5, {  3057,   3116,   3118,   3059,   3061,   3120,   3122,   3063}},
 {  1602,   5, {  3116,   3117,   3119,   3118,   3120,   3121,   3123,   3122}},
 {  1603,   5, {  3061,   3120,   3122,   3063,   3065,   3124,   3126,   3067}},
 {  1604,   5, {  3120,   3121,   3123,   3122,   3124,   3125,   3127,   3126}},
 {  1607,   5, {  3069,   3128,   3130,   3071,   3073,   3132,   3134,   3075}},
 {  1608,   5, {  3128,   3129,   3131,   3130,   3132,   3133,   3135,   3134}},
 {  1609,   5, {  3073,   3132,   3134,   3075,   3077,   3136,   3138,   3079}},
 {  1610,   5, {  3132,   3133,   3135,   3134,   3136,   3137,   3139,   3138}},
 {  1613,   5, {  3081,   3140,   3142,   3083,   3085,   3144,   3146,   3087}},
 {  1614,   5, {  3140,   3141,   3143,   3142,   3144,   3145,   3147,   3146}},
 {  1615,   5, {  3085,   3144,   3146,   3087,   3089,   3148,   3150,   3091}},
 {  1616,   5, {  3144,   3145,   3147,   3146,   3148,   3149,   3151,   3150}},
 {  1619,   5, {  3093,   3152,   3154,   3095,   3097,   3156,   3158,   3099}},
 {  1620,   5, {  3152,   3153,   3155,   3154,   3156,   3157,   3159,   3158}},
 {  1621,   5, {  3097,   3156,   3158,   3099,   3101,   3160,   3162,   3103}},
 {  1622,   5, {  3156,   3157,   3159,   3158,   3160,   3161,   3163,   3162}},
 {  1625,   5, {  3105,   3164,   3166,   3107,   3109,   3168,   3170,   3111}},
 {  1626,   5, {  3164,   3165,   3167,   3166,   3168,   3169,   3171,   3170}},
 {  1627,   5, {  3109,   3168,   3170,   3111,   3113,   3172,   3174,   3115}},
 {  1628,   5, {  3168,   3169,   3171,   3170,   3172,   3173,   3175,   3174}},
 {  1629,   5, {  3117,   3176,   3177,   3119,   3121,   3178,   3179,   3123}},
 {  1630,   5, {  3121,   3178,   3179,   3123,   3125,   3180,   3181,   3127}},
 {  1632,   5, {  3129,   3182,   3183,   3131,   3133,   3184,   3185,   3135}},
 {  1633,   5, {  3133,   3184,   3185,   3135,   3137,   3186,   3187,   3139}},
 {  1635,   5, {  3141,   3188,   3189,   3143,   3145,   3190,   3191,   3147}},
 {  1636,   5, {  3145,   3190,   3191,   3147,   3149,   3192,   3193,   3151}},
 {  1638,   5, {  3153,   3194,   3195,   3155,   3157,   3196,   3197,   3159}},
 {  1639,   5, {  3157,   3196,   3197,   3159,   3161,   3198,   3199,   3163}},
 {  1641,   5, {  3165,   3200,   3201,   3167,   3169,   3202,   3203,   3171}},
 {  1642,   5, {  3169,   3202,   3203,   3171,   3173,   3204,   3205,   3175}},
 {  1643,   5, {  3206,   3207,   3211,   3210,   3214,   3215,   3219,   3218}},
 {  1644,   5, {  3207,   3208,   3212,   3211,   3215,   3216,   3220,   3219}},
 {  1645,   5, {  3208,   3209,   3213,   3212,   3216,   3217,   3221,   3220}},
 {  1646,   5, {  3214,   3215,   3219,   3218,   3222,   3223,   3227,   3226}},
 {  1647,   5, {  3215,   3216,   3220,   3219,   3223,   3224,   3228,   3227}},
 {  1648,   5, {  3216,   3217,   3221,   3220,   3224,   3225,   3229,   3228}},
 {  1652,   5, {  3230,   3231,   3235,   3234,   3238,   3239,   3243,   3242}},
 {  1653,   5, {  3231,   3232,   3236,   3235,   3239,   3240,   3244,   3243}},
 {  1654,   5, {  3232,   3233,   3237,   3236,   3240,   3241,   3245,   3244}},
 {  1655,   5, {  3238,   3239,   3243,   3242,   3246,   3247,   3251,   3250}},
 {  1656,   5, {  3239,   3240,   3244,   3243,   3247,   3248,   3252,   3251}},
 {  1657,   5, {  3240,   3241,   3245,   3244,   3248,   3249,   3253,   3252}},
 {  1661,   5, {  3254,   3255,   3259,   3258,   3262,   3263,   3267,   3266}},
 {  1662,   5, {  3255,   3256,   3260,   3259,   3263,   3264,   3268,   3267}},
 {  1663,   5, {  3256,   3257,   3261,   3260,   3264,   3265,   3269,   3268}},
 {  1664,   5, {  3262,   3263,   3267,   3266,   3270,   3271,   3275,   3274}},
 {  1665,   5, {  3263,   3264,   3268,   3267,   3271,   3272,   3276,   3275}},
 {  1666,   5, {  3264,   3265,   3269,   3268,   3272,   3273,   3277,   3276}},
 {  1670,   5, {  3278,   3279,   3283,   3282,   3286,   3287,   3291,   3290}},
 {  1671,   5, {  3279,   3280,   3284,   3283,   3287,   3288,   3292,   3291}},
 {  1672,   5, {  3280,   3281,   3285,   3284,   3288,   3289,   3293,   3292}},
 {  1673,   5, {  3286,   3287,   3291,   3290,   3294,   3295,   3299,   3298}},
 {  1674,   5, {  3287,   3288,   3292,   3291,   3295,   3296,   3300,   3299}},
 {  1675,   5, {  3288,   3289,   3293,   3292,   3296,   3297,   3301,   3300}},
 {  1679,   5, {  3302,   3303,   3307,   3306,   3310,   3311,   3315,   3314}},
 {  1680,   5, {  3303,   3304,   3308,   3307,   3311,   3312,   3316,   3315}},
 {  1681,   5, {  3304,   3305,   3309,   3308,   3312,   3313,   3317,   3316}},
 {  1682,   5, {  3310,   3311,   3315,   3314,   3318,   3319,   3323,   3322}},
 {  1683,   5, {  3311,   3312,   3316,   3315,   3319,   3320,   3324,   3323}},
 {  1684,   5, {  3312,   3313,   3317,   3316,   3320,   3321,   3325,   3324}},
 {  1685,   5, {  3326,   3327,   3331,   3330,   3334,   3335,   3339,   3338}},
 {  1686,   5, {  3327,   3328,   3332,   3331,   3335,   3336,   3340,   3339}},
 {  1687,   5, {  3328,   3329,   3333,   3332,   3336,   3337,   3341,   3340}},
 {  1688,   5, {  3334,   3335,   3339,   3338,   3342,   3343,   3347,   3346}},
 {  1689,   5, {  3335,   3336,   3340,   3339,   3343,   3344,   3348,   3347}},
 {  1690,   5, {  3336,   3337,   3341,   3340,   3344,   3345,   3349,   3348}},
 {  1694,   5, {  3350,   3351,   3355,   3354,   3358,   3359,   3363,   3362}},
 {  1695,   5, {  3351,   3352,   3356,   3355,   3359,   3360,   3364,   3363}},
 {  1696,   5, {  3352,   3353,   3357,   3356,   3360,   3361,   3365,   3364}},
 {  1697,   5, {  3358,   3359,   3363,   3362,   3366,   3367,   3371,   3370}},
 {  1698,   5, {  3359,   3360,   3364,   3363,   3367,   3368,   3372,   3371}},
 {  1699,   5, {  3360,   3361,   3365,   3364,   3368,   3369,   3373,   3372}},
 {  1703,   5, {  3374,   3375,   3379,   3378,   3382,   3383,   3387,   3386}},
 {  1704,   5, {  3375,   3376,   3380,   3379,   3383,   3384,   3388,   3387}},
 {  1705,   5, {  3376,   3377,   3381,   3380,   3384,   3385,   3389,   3388}},
 {  1706,   5, {  3382,   3383,   3387,   3386,   3390,   3391,   3395,   3394}},
 {  1707,   5, {  3383,   3384,   3388,   3387,   3391,   3392,   3396,   3395}},
 {  1708,   5, {  3384,   3385,   3389,   3388,   3392,   3393,   3397,   3396}},
 {  1712,   5, {  3398,   3399,   3403,   3402,   3406,   3407,   3411,   3410}},
 {  1713,   5, {  3399,   3400,   3404,   3403,   3407,   3408,   3412,   3411}},
 {  1714,   5, {  3400,   3401,   3405,   3404,   3408,   3409,   3413,   3412}},
 {  1715,   5, {  3406,   3407,   3411,   3410,   3414,   3415,   3419,   3418}},
 {  1716,   5, {  3407,   3408,   3412,   3411,   3415,   3416,   3420,   3419}},
 {  1717,   5, {  3408,   3409,   3413,   3412,   3416,   3417,   3421,   3420}},
 {  1721,   5, {  3422,   3423,   3427,   3426,   3430,   3431,   3435,   3434}},
 {  1722,   5, {  3423,   3424,   3428,   3427,   3431,   3432,   3436,   3435}},
 {  1723,   5, {  3424,   3425,   3429,   3428,   3432,   3433,   3437,   3436}},
 {  1724,   5, {  3430,   3431,   3435,   3434,   3438,   3439,   3443,   3442}},
 {  1725,   5, {  3431,   3432,   3436,   3435,   3439,   3440,   3444,   3443}},
 {  1726,   5, {  3432,   3433,   3437,   3436,   3440,   3441,   3445,   3444}},
 {  1727,   5, {  3446,   3447,   3451,   3450,   3454,   3455,   3459,   3458}},
 {  1728,   5, {  3447,   3448,   3452,   3451,   3455,   3456,   3460,   3459}},
 {  1729,   5, {  3448,   3449,   3453,   3452,   3456,   3457,   3461,   3460}},
 {  1730,   5, {  3454,   3455,   3459,   3458,   3462,   3463,   3467,   3466}},
 {  1731,   5, {  3455,   3456,   3460,   3459,   3463,   3464,   3468,   3467}},
 {  1732,   5, {  3456,   3457,   3461,   3460,   3464,   3465,   3469,   3468}},
 {  1736,   5, {  3470,   3471,   3475,   3474,   3478,   3479,   3483,   3482}},
 {  1737,   5, {  3471,   3472,   3476,   3475,   3479,   3480,   3484,   3483}},
 {  1738,   5, {  3472,   3473,   3477,   3476,   3480,   3481,   3485,   3484}},
 {  1739,   5, {  3478,   3479,   3483,   3482,   3486,   3487,   3491,   3490}},
 {  1740,   5, {  3479,   3480,   3484,   3483,   3487,   3488,   3492,   3491}},
 {  1741,   5, {  3480,   3481,   3485,   3484,   3488,   3489,   3493,   3492}},
 {  1745,   5, {  3494,   3495,   3499,   3498,   3502,   3503,   3507,   3506}},
 {  1746,   5, {  3495,   3496,   3500,   3499,   3503,   3504,   3508,   3507}},
 {  1747,   5, {  3496,   3497,   3501,   3500,   3504,   3505,   3509,   3508}},
 {  1748,   5, {  3502,   3503,   3507,   3506,   3510,   3511,   3515,   3514}},
 {  1749,   5, {  3503,   3504,   3508,   3507,   3511,   3512,   3516,   3515}},
 {  1750,   5, {  3504,   3505,   3509,   3508,   3512,   3513,   3517,   3516}},
 {  1754,   5, {  3518,   3519,   3523,   3522,   3526,   3527,   3531,   3530}},
 {  1755,   5, {  3519,   3520,   3524,   3523,   3527,   3528,   3532,   3531}},
 {  1756,   5, {  3520,   3521,   3525,   3524,   3528,   3529,   3533,   3532}},
 {  1757,   5, {  3526,   3527,   3531,   3530,   3534,   3535,   3539,   3538}},
 {  1758,   5, {  3527,   3528,   3532,   3531,   3535,   3536,   3540,   3539}},
 {  1759,   5, {  3528,   3529,   3533,   3532,   3536,   3537,   3541,   3540}},
 {  1763,   5, {  3542,   3543,   3547,   3546,   3550,   3551,   3555,   3554}},
 {  1764,   5, {  3543,   3544,   3548,   3547,   3551,   3552,   3556,   3555}},
 {  1765,   5, {  3544,   3545,   3549,   3548,   3552,   3553,   3557,   3556}},
 {  1766,   5, {  3550,   3551,   3555,   3554,   3558,   3559,   3563,   3562}},
 {  1767,   5, {  3551,   3552,   3556,   3555,   3559,   3560,   3564,   3563}},
 {  1768,   5, {  3552,   3553,   3557,   3556,   3560,   3561,   3565,   3564}},
 {  1769,   5, {  3566,   3567,   3571,   3570,   3574,   3575,   3579,   3578}},
 {  1770,   5, {  3567,   3568,   3572,   3571,   3575,   3576,   3580,   3579}},
 {  1771,   5, {  3568,   3569,   3573,   3572,   3576,   3577,   3581,   3580}},
 {  1772,   5, {  3574,   3575,   3579,   3578,   3582,   3583,   3587,   3586}},
 {  1773,   5, {  3575,   3576,   3580,   3579,   3583,   3584,   3588,   3587}},
 {  1774,   5, {  3576,   3577,   3581,   3580,   3584,   3585,   3589,   3588}},
 {  1778,   5, {  3590,   3591,   3595,   3594,   3598,   3599,   3603,   3602}},
 {  1779,   5, {  3591,   3592,   3596,   3595,   3599,   3600,   3604,   3603}},
 {  1780,   5, {  3592,   3593,   3597,   3596,   3600,   3601,   3605,   3604}},
 {  1781,   5, {  3598,   3599,   3603,   3602,   3606,   3607,   3611,   3610}},
 {  1782,   5, {  3599,   3600,   3604,   3603,   3607,   3608,   3612,   3611}},
 {  1783,   5, {  3600,   3601,   3605,   3604,   3608,   3609,   3613,   3612}},
 {  1787,   5, {  3614,   3615,   3619,   3618,   3622,   3623,   3627,   3626}},
 {  1788,   5, {  3615,   3616,   3620,   3619,   3623,   3624,   3628,   3627}},
 {  1789,   5, {  3616,   3617,   3621,   3620,   3624,   3625,   3629,   3628}},
 {  1790,   5, {  3622,   3623,   3627,   3626,   3630,   3631,   3635,   3634}},
 {  1791,   5, {  3623,   3624,   3628,   3627,   3631,   3632,   3636,   3635}},
 {  1792,   5, {  3624,   3625,   3629,   3628,   3632,   3633,   3637,   3636}},
 {  1796,   5, {  3638,   3639,   3643,   3642,   3646,   3647,   3651,   3650}},
 {  1797,   5, {  3639,   3640,   3644,   3643,   3647,   3648,   3652,   3651}},
 {  1798,   5, {  3640,   3641,   3645,   3644,   3648,   3649,   3653,   3652}},
 {  1799,   5, {  3646,   3647,   3651,   3650,   3654,   3655,   3659,   3658}},
 {  1800,   5, {  3647,   3648,   3652,   3651,   3655,   3656,   3660,   3659}},
 {  1801,   5, {  3648,   3649,   3653,   3652,   3656,   3657,   3661,   3660}},
 {  1805,   5, {  3662,   3663,   3667,   3666,   3670,   3671,   3675,   3674}},
 {  1806,   5, {  3663,   3664,   3668,   3667,   3671,   3672,   3676,   3675}},
 {  1807,   5, {  3664,   3665,   3669,   3668,   3672,   3673,   3677,   3676}},
 {  1808,   5, {  3670,   3671,   3675,   3674,   3678,   3679,   3683,   3682}},
 {  1809,   5, {  3671,   3672,   3676,   3675,   3679,   3680,   3684,   3683}},
 {  1810,   5, {  3672,   3673,   3677,   3676,   3680,   3681,   3685,   3684}},
 {  1811,   5, {  3690,   3691,   3693,   3692,   3686,   3687,   3689,   3688}},
 {  1812,   5, {  3694,   3695,   3697,   3696,   3690,   3691,   3693,   3692}},
 {  1813,   5, {  3698,   3699,   3701,   3700,   3694,   3695,   3697,   3696}},
 {  1814,   5, {  3702,   3703,   3705,   3704,   3698,   3699,   3701,   3700}},
 {  1815,   5, {  3706,   3707,   3709,   3708,   3702,   3703,   3705,   3704}},
 {  1816,   5, {  3710,   3711,   3713,   3712,   3706,   3707,   3709,   3708}},
 {  1817,   5, {  3714,   3715,   3717,   3716,   3710,   3711,   3713,   3712}},
 {  1818,   5, {  3718,   3719,   3721,   3720,   3714,   3715,   3717,   3716}},
 {  1819,   5, {  3722,   3723,   3725,   3724,   3718,   3719,   3721,   3720}},
 {  1820,   5, {  3726,   3727,   3729,   3728,   3722,   3723,   3725,   3724}},
 {  1821,   5, {  3730,   3731,   3733,   3732,   3726,   3727,   3729,   3728}},
 {  1822,   5, {  3734,   3735,   3737,   3736,   3730,   3731,   3733,   3732}},
 {  1823,   5, {  3738,   3739,   3741,   3740,   3734,   3735,   3737,   3736}},
 {  1824,   5, {  3742,   3743,   3745,   3744,   3738,   3739,   3741,   3740}},
 {  1825,   5, {  3746,   3747,   3749,   3748,   3742,   3743,   3745,   3744}},
 {  1826,   5, {  3750,   3751,   3753,   3752,   3746,   3747,   3749,   3748}},
 {  1827,   5, {  3692,   3693,   3759,   3758,   3688,   3689,   3755,   3754}},
 {  1828,   5, {  3758,   3759,   3761,   3760,   3754,   3755,   3757,   3756}},
 {  1829,   5, {  3696,   3697,   3763,   3762,   3692,   3693,   3759,   3758}},
 {  1830,   5, {  3762,   3763,   3765,   3764,   3758,   3759,   3761,   3760}},
 {  1831,   5, {  3700,   3701,   3767,   3766,   3696,   3697,   3763,   3762}},
 {  1832,   5, {  3766,   3767,   3769,   3768,   3762,   3763,   3765,   3764}},
 {  1833,   5, {  3704,   3705,   3771,   3770,   3700,   3701,   3767,   3766}},
 {  1834,   5, {  3770,   3771,   3773,   3772,   3766,   3767,   3769,   3768}},
 {  1835,   5, {  3708,   3709,   3775,   3774,   3704,   3705,   3771,   3770}},
 {  1836,   5, {  3774,   3775,   3777,   3776,   3770,   3771,   3773,   3772}},
 {  1837,   5, {  3712,   3713,   3779,   3778,   3708,   3709,   3775,   3774}},
 {  1838,   5, {  3778,   3779,   3781,   3780,   3774,   3775,   3777,   3776}},
 {  1839,   5, {  3716,   3717,   3783,   3782,   3712,   3713,   3779,   3778}},
 {  1840,   5, {  3782,   3783,   3785,   3784,   3778,   3779,   3781,   3780}},
 {  1841,   5, {  3720,   3721,   3787,   3786,   3716,   3717,   3783,   3782}},
 {  1842,   5, {  3786,   3787,   3789,   3788,   3782,   3783,   3785,   3784}},
 {  1843,   5, {  3724,   3725,   3791,   3790,   3720,   3721,   3787,   3786}},
 {  1844,   5, {  3790,   3791,   3793,   3792,   3786,   3787,   3789,   3788}},
 {  1845,   5, {  3728,   3729,   3795,   3794,   3724,   3725,   3791,   3790}},
 {  1846,   5, {  3794,   3795,   3797,   3796,   3790,   3791,   3793,   3792}},
 {  1847,   5, {  3732,   3733,   3799,   3798,   3728,   3729,   3795,   3794}},
 {  1848,   5, {  3798,   3799,   3801,   3800,   3794,   3795,   3797,   3796}},
 {  1849,   5, {  3736,   3737,   3803,   3802,   3732,   3733,   3799,   3798}},
 {  1850,   5, {  3802,   3803,   3805,   3804,   3798,   3799,   3801,   3800}},
 {  1851,   5, {  3740,   3741,   3807,   3806,   3736,   3737,   3803,   3802}},
 {  1852,   5, {  3806,   3807,   3809,   3808,   3802,   3803,   3805,   3804}},
 {  1853,   5, {  3744,   3745,   3811,   3810,   3740,   3741,   3807,   3806}},
 {  1854,   5, {  3810,   3811,   3813,   3812,   3806,   3807,   3809,   3808}},
 {  1855,   5, {  3748,   3749,   3815,   3814,   3744,   3745,   3811,   3810}},
 {  1856,   5, {  3814,   3815,   3817,   3816,   3810,   3811,   3813,   3812}},
 {  1857,   5, {  3752,   3753,   3819,   3818,   3748,   3749,   3815,   3814}},
 {  1858,   5, {  3818,   3819,   3821,   3820,   3814,   3815,   3817,   3816}},
 {  1859,   5, {  3760,   3761,   3825,   3824,   3756,   3757,   3823,   3822}},
 {  1860,   5, {  3764,   3765,   3827,   3826,   3760,   3761,   3825,   3824}},
 {  1861,   5, {  3768,   3769,   3829,   3828,   3764,   3765,   3827,   3826}},
 {  1862,   5, {  3772,   3773,   3831,   3830,   3768,   3769,   3829,   3828}},
 {  1863,   5, {  3776,   3777,   3833,   3832,   3772,   3773,   3831,   3830}},
 {  1864,   5, {  3780,   3781,   3835,   3834,   3776,   3777,   3833,   3832}},
 {  1865,   5, {  3784,   3785,   3837,   3836,   3780,   3781,   3835,   3834}},
 {  1866,   5, {  3788,   3789,   3839,   3838,   3784,   3785,   3837,   3836}},
 {  1867,   5, {  3792,   3793,   3841,   3840,   3788,   3789,   3839,   3838}},
 {  1868,   5, {  3796,   3797,   3843,   3842,   3792,   3793,   3841,   3840}},
 {  1869,   5, {  3800,   3801,   3845,   3844,   3796,   3797,   3843,   3842}},
 {  1870,   5, {  3804,   3805,   3847,   3846,   3800,   3801,   3845,   3844}},
 {  1871,   5, {  3808,   3809,   3849,   3848,   3804,   3805,   3847,   3846}},
 {  1872,   5, {  3812,   3813,   3851,   3850,   3808,   3809,   3849,   3848}},
 {  1873,   5, {  3816,   3817,   3853,   3852,   3812,   3813,   3851,   3850}},
 {  1874,   5, {  3820,   3821,   3855,   3854,   3816,   3817,   3853,   3852}},
 {  1875,   5, {  3824,   3825,   3861,   3860,   3822,   3823,   3857,   3856}},
 {  1876,   5, {  3860,   3861,   3863,   3862,   3856,   3857,   3859,   3858}},
 {  1877,   5, {  3826,   3827,   3865,   3864,   3824,   3825,   3861,   3860}},
 {  1878,   5, {  3864,   3865,   3867,   3866,   3860,   3861,   3863,   3862}},
 {  1879,   5, {  3828,   3829,   3869,   3868,   3826,   3827,   3865,   3864}},
 {  1880,   5, {  3868,   3869,   3871,   3870,   3864,   3865,   3867,   3866}},
 {  1881,   5, {  3830,   3831,   3873,   3872,   3828,   3829,   3869,   3868}},
 {  1882,   5, {  3872,   3873,   3875,   3874,   3868,   3869,   3871,   3870}},
 {  1883,   5, {  3832,   3833,   3877,   3876,   3830,   3831,   3873,   3872}},
 {  1884,   5, {  3876,   3877,   3879,   3878,   3872,   3873,   3875,   3874}},
 {  1885,   5, {  3834,   3835,   3881,   3880,   3832,   3833,   3877,   3876}},
 {  1886,   5, {  3880,   3881,   3883,   3882,   3876,   3877,   3879,   3878}},
 {  1887,   5, {  3836,   3837,   3885,   3884,   3834,   3835,   3881,   3880}},
 {  1888,   5, {  3884,   3885,   3887,   3886,   3880,   3881,   3883,   3882}},
 {  1889,   5, {  3838,   3839,   3889,   3888,   3836,   3837,   3885,   3884}},
 {  1890,   5, {  3888,   3889,   3891,   3890,   3884,   3885,   3887,   3886}},
 {  1891,   5, {  3840,   3841,   3893,   3892,   3838,   3839,   3889,   3888}},
 {  1892,   5, {  3892,   3893,   3895,   3894,   3888,   3889,   3891,   3890}},
 {  1893,   5, {  3842,   3843,   3897,   3896,   3840,   3841,   3893,   3892}},
 {  1894,   5, {  3896,   3897,   3899,   3898,   3892,   3893,   3895,   3894}},
 {  1895,   5, {  3844,   3845,   3901,   3900,   3842,   3843,   3897,   3896}},
 {  1896,   5, {  3900,   3901,   3903,   3902,   3896,   3897,   3899,   3898}},
 {  1897,   5, {  3846,   3847,   3905,   3904,   3844,   3845,   3901,   3900}},
 {  1898,   5, {  3904,   3905,   3907,   3906,   3900,   3901,   3903,   3902}},
 {  1899,   5, {  3848,   3849,   3909,   3908,   3846,   3847,   3905,   3904}},
 {  1900,   5, {  3908,   3909,   3911,   3910,   3904,   3905,   3907,   3906}},
 {  1901,   5, {  3850,   3851,   3913,   3912,   3848,   3849,   3909,   3908}},
 {  1902,   5, {  3912,   3913,   3915,   3914,   3908,   3909,   3911,   3910}},
 {  1903,   5, {  3852,   3853,   3917,   3916,   3850,   3851,   3913,   3912}},
 {  1904,   5, {  3916,   3917,   3919,   3918,   3912,   3913,   3915,   3914}},
 {  1905,   5, {  3854,   3855,   3921,   3920,   3852,   3853,   3917,   3916}},
 {  1906,   5, {  3920,   3921,   3923,   3922,   3916,   3917,   3919,   3918}},
 {  1907,   5, {  3862,   3863,   3927,   3926,   3858,   3859,   3925,   3924}},
 {  1908,   5, {  3866,   3867,   3929,   3928,   3862,   3863,   3927,   3926}},
 {  1909,   5, {  3870,   3871,   3931,   3930,   3866,   3867,   3929,   3928}},
 {  1910,   5, {  3874,   3875,   3933,   3932,   3870,   3871,   3931,   3930}},
 {  1911,   5, {  3878,   3879,   3935,   3934,   3874,   3875,   3933,   3932}},
 {  1912,   5, {  3882,   3883,   3937,   3936,   3878,   3879,   3935,   3934}},
 {  1913,   5, {  3886,   3887,   3939,   3938,   3882,   3883,   3937,   3936}},
 {  1914,   5, {  3890,   3891,   3941,   3940,   3886,   3887,   3939,   3938}},
 {  1915,   5, {  3894,   3895,   3943,   3942,   3890,   3891,   3941,   3940}},
 {  1916,   5, {  3898,   3899,   3945,   3944,   3894,   3895,   3943,   3942}},
 {  1917,   5, {  3902,   3903,   3947,   3946,   3898,   3899,   3945,   3944}},
 {  1918,   5, {  3906,   3907,   3949,   3948,   3902,   3903,   3947,   3946}},
 {  1919,   5, {  3910,   3911,   3951,   3950,   3906,   3907,   3949,   3948}},
 {  1920,   5, {  3914,   3915,   3953,   3952,   3910,   3911,   3951,   3950}},
 {  1921,   5, {  3918,   3919,   3955,   3954,   3914,   3915,   3953,   3952}},
 {  1922,   5, {  3922,   3923,   3957,   3956,   3918,   3919,   3955,   3954}},
 {  1923,  30, {  1323,   1324,   1327,   1326,   3958,   3959,   3962,   3961}},
 {  1924,  30, {  1324,   1325,   1328,   1327,   3959,   3960,   3963,   3962}},
 {  1925,  30, {  1326,   1327,   1330,   1329,   3961,   3962,   3965,   3964}},
 {  1926,  30, {  1327,   1328,   1331,   1330,   3962,   3963,   3966,   3965}},
 {  1927,  30, {  1329,   1330,   1333,   1332,   3964,   3965,   3968,   3967}},
 {  1928,  30, {  1330,   1331,   1334,   1333,   3965,   3966,   3969,   3968}},
 {  1929,  30, {  1332,   1333,   1336,   1335,   3967,   3968,   3971,   3970}},
 {  1930,  30, {  1333,   1334,   1337,   1336,   3968,   3969,   3972,   3971}},
 {  1931,  30, {  1335,   1336,   1339,   1338,   3970,   3971,   3974,   3973}},
 {  1932,  30, {  1336,   1337,   1340,   1339,   3971,   3972,   3975,   3974}},
 {  1933,  30, {  1338,   1339,   1342,   1341,   3973,   3974,   3977,   3976}},
 {  1934,  30, {  1339,   1340,   1343,   1342,   3974,   3975,   3978,   3977}},
 {  1935,  30, {  1341,   1342,   1345,   1344,   3976,   3977,   3980,   3979}},
 {  1936,  30, {  1342,   1343,   1346,   1345,   3977,   3978,   3981,   3980}},
 {  1937,  30, {  1344,   1345,   1348,   1347,   3979,   3980,   3983,   3982}},
 {  1938,  30, {  1345,   1346,   1349,   1348,   3980,   3981,   3984,   3983}},
 {  1939,  30, {  1347,   1348,   1351,   1350,   3982,   3983,   3986,   3985}},
 {  1940,  30, {  1348,   1349,   1352,   1351,   3983,   3984,   3987,   3986}},
 {  1941,  30, {  1350,   1351,   1354,   1353,   3985,   3986,   3989,   3988}},
 {  1942,  30, {  1351,   1352,   1355,   1354,   3986,   3987,   3990,   3989}},
 {  1943,  30, {  1353,   1354,   1357,   1356,   3988,   3989,   3992,   3991}},
 {  1944,  30, {  1354,   1355,   1358,   1357,   3989,   3990,   3993,   3992}},
 {  1945,  30, {  1356,   1357,   1324,   1323,   3991,   3992,   3959,   3958}},
 {  1946,  30, {  1357,   1358,   1325,   1324,   3992,   3993,   3960,   3959}},
 {  1947,  30, {  3958,   3959,   3962,   3961,   3994,   3995,   3998,   3997}},
 {  1948,  30, {  3959,   3960,   3963,   3962,   3995,   3996,   3999,   3998}},
 {  1949,  30, {  3961,   3962,   3965,   3964,   3997,   3998,   4001,   4000}},
 {  1950,  30, {  3962,   3963,   3966,   3965,   3998,   3999,   4002,   4001}},
 {  1951,  30, {  3964,   3965,   3968,   3967,   4000,   4001,   4004,   4003}},
 {  1952,  30, {  3965,   3966,   3969,   3968,   4001,   4002,   4005,   4004}},
 {  1953,  30, {  3967,   3968,   3971,   3970,   4003,   4004,   4007,   4006}},
 {  1954,  30, {  3968,   3969,   3972,   3971,   4004,   4005,   4008,   4007}},
 {  1955,  30, {  3970,   3971,   3974,   3973,   4006,   4007,   4010,   4009}},
 {  1956,  30, {  3971,   3972,   3975,   3974,   4007,   4008,   4011,   4010}},
 {  1957,  30, {  3973,   3974,   3977,   3976,   4009,   4010,   4013,   4012}},
 {  1958,  30, {  3974,   3975,   3978,   3977,   4010,   4011,   4014,   4013}},
 {  1959,  30, {  3976,   3977,   3980,   3979,   4012,   4013,   4016,   4015}},
 {  1960,  30, {  3977,   3978,   3981,   3980,   4013,   4014,   4017,   4016}},
 {  1961,  30, {  3979,   3980,   3983,   3982,   4015,   4016,   4019,   4018}},
 {  1962,  30, {  3980,   3981,   3984,   3983,   4016,   4017,   4020,   4019}},
 {  1963,  30, {  3982,   3983,   3986,   3985,   4018,   4019,   4022,   4021}},
 {  1964,  30, {  3983,   3984,   3987,   3986,   4019,   4020,   4023,   4022}},
 {  1965,  30, {  3985,   3986,   3989,   3988,   4021,   4022,   4025,   4024}},
 {  1966,  30, {  3986,   3987,   3990,   3989,   4022,   4023,   4026,   4025}},
 {  1967,  30, {  3988,   3989,   3992,   3991,   4024,   4025,   4028,   4027}},
 {  1968,  30, {  3989,   3990,   3993,   3992,   4025,   4026,   4029,   4028}},
 {  1969,  30, {  3991,   3992,   3959,   3958,   4027,   4028,   3995,   3994}},
 {  1970,  30, {  3992,   3993,   3960,   3959,   4028,   4029,   3996,   3995}},
 {  1971,  30, {  3994,   3995,   3998,   3997,   4030,   4031,   4034,   4033}},
 {  1972,  30, {  3995,   3996,   3999,   3998,   4031,   4032,   4035,   4034}},
 {  1973,  30, {  3997,   3998,   4001,   4000,   4033,   4034,   4037,   4036}},
 {  1974,  30, {  3998,   3999,   4002,   4001,   4034,   4035,   4038,   4037}},
 {  1975,  30, {  4000,   4001,   4004,   4003,   4036,   4037,   4040,   4039}},
 {  1976,  30, {  4001,   4002,   4005,   4004,   4037,   4038,   4041,   4040}},
 {  1977,  30, {  4003,   4004,   4007,   4006,   4039,   4040,   4043,   4042}},
 {  1978,  30, {  4004,   4005,   4008,   4007,   4040,   4041,   4044,   4043}},
 {  1979,  30, {  4006,   4007,   4010,   4009,   4042,   4043,   4046,   4045}},
 {  1980,  30, {  4007,   4008,   4011,   4010,   4043,   4044,   4047,   4046}},
 {  1981,  30, {  4009,   4010,   4013,   4012,   4045,   4046,   4049,   4048}},
 {  1982,  30, {  4010,   4011,   4014,   4013,   4046,   4047,   4050,   4049}},
 {  1983,  30, {  4012,   4013,   4016,   4015,   4048,   4049,   4052,   4051}},
 {  1984,  30, {  4013,   4014,   4017,   4016,   4049,   4050,   4053,   4052}},
 {  1985,  30, {  4015,   4016,   4019,   4018,   4051,   4052,   4055,   4054}},
 {  1986,  30, {  4016,   4017,   4020,   4019,   4052,   4053,   4056,   4055}},
 {  1987,  30, {  4018,   4019,   4022,   4021,   4054,   4055,   4058,   4057}},
 {  1988,  30, {  4019,   4020,   4023,   4022,   4055,   4056,   4059,   4058}},
 {  1989,  30, {  4021,   4022,   4025,   4024,   4057,   4058,   4061,   4060}},
 {  1990,  30, {  4022,   4023,   4026,   4025,   4058,   4059,   4062,   4061}},
 {  1991,  30, {  4024,   4025,   4028,   4027,   4060,   4061,   4064,   4063}},
 {  1992,  30, {  4025,   4026,   4029,   4028,   4061,   4062,   4065,   4064}},
 {  1993,  30, {  4027,   4028,   3995,   3994,   4063,   4064,   4031,   4030}},
 {  1994,  30, {  4028,   4029,   3996,   3995,   4064,   4065,   4032,   4031}},
 {  1995,  30, {  4030,   4031,   4034,   4033,   4066,   4067,   4070,   4069}},
 {  1996,  30, {  4031,   4032,   4035,   4034,   4067,   4068,   4071,   4070}},
 {  1997,  30, {  4033,   4034,   4037,   4036,   4069,   4070,   4073,   4072}},
 {  1998,  30, {  4034,   4035,   4038,   4037,   4070,   4071,   4074,   4073}},
 {  1999,  30, {  4036,   4037,   4040,   4039,   4072,   4073,   4076,   4075}},
 {  2000,  30, {  4037,   4038,   4041,   4040,   4073,   4074,   4077,   4076}},
 {  2001,  30, {  4039,   4040,   4043,   4042,   4075,   4076,   4079,   4078}},
 {  2002,  30, {  4040,   4041,   4044,   4043,   4076,   4077,   4080,   4079}},
 {  2003,  30, {  4042,   4043,   4046,   4045,   4078,   4079,   4082,   4081}},
 {  2004,  30, {  4043,   4044,   4047,   4046,   4079,   4080,   4083,   4082}},
 {  2005,  30, {  4045,   4046,   4049,   4048,   4081,   4082,   4085,   4084}},
 {  2006,  30, {  4046,   4047,   4050,   4049,   4082,   4083,   4086,   4085}},
 {  2007,  30, {  4048,   4049,   4052,   4051,   4084,   4085,   4088,   4087}},
 {  2008,  30, {  4049,   4050,   4053,   4052,   4085,   4086,   4089,   4088}},
 {  2009,  30, {  4051,   4052,   4055,   4054,   4087,   4088,   4091,   4090}},
 {  2010,  30, {  4052,   4053,   4056,   4055,   4088,   4089,   4092,   4091}},
 {  2011,  30, {  4054,   4055,   4058,   4057,   4090,   4091,   4094,   4093}},
 {  2012,  30, {  4055,   4056,   4059,   4058,   4091,   4092,   4095,   4094}},
 {  2013,  30, {  4057,   4058,   4061,   4060,   4093,   4094,   4097,   4096}},
 {  2014,  30, {  4058,   4059,   4062,   4061,   4094,   4095,   4098,   4097}},
 {  2015,  30, {  4060,   4061,   4064,   4063,   4096,   4097,   4100,   4099}},
 {  2016,  30, {  4061,   4062,   4065,   4064,   4097,   4098,   4101,   4100}},
 {  2017,  30, {  4063,   4064,   4031,   4030,   4099,   4100,   4067,   4066}},
 {  2018,  30, {  4064,   4065,   4032,   4031,   4100,   4101,   4068,   4067}},
 {  2019,  30, {  4066,   4067,   4070,   4069,   4102,   4103,   4106,   4105}},
 {  2020,  30, {  4067,   4068,   4071,   4070,   4103,   4104,   4107,   4106}},
 {  2021,  30, {  4069,   4070,   4073,   4072,   4105,   4106,   4109,   4108}},
 {  2022,  30, {  4070,   4071,   4074,   4073,   4106,   4107,   4110,   4109}},
 {  2023,  30, {  4072,   4073,   4076,   4075,   4108,   4109,   4112,   4111}},
 {  2024,  30, {  4073,   4074,   4077,   4076,   4109,   4110,   4113,   4112}},
 {  2025,  30, {  4075,   4076,   4079,   4078,   4111,   4112,   4115,   4114}},
 {  2026,  30, {  4076,   4077,   4080,   4079,   4112,   4113,   4116,   4115}},
 {  2027,  30, {  4078,   4079,   4082,   4081,   4114,   4115,   4118,   4117}},
 {  2028,  30, {  4079,   4080,   4083,   4082,   4115,   4116,   4119,   4118}},
 {  2029,  30, {  4081,   4082,   4085,   4084,   4117,   4118,   4121,   4120}},
 {  2030,  30, {  4082,   4083,   4086,   4085,   4118,   4119,   4122,   4121}},
 {  2031,  30, {  4084,   4085,   4088,   4087,   4120,   4121,   4124,   4123}},
 {  2032,  30, {  4085,   4086,   4089,   4088,   4121,   4122,   4125,   4124}},
 {  2033,  30, {  4087,   4088,   4091,   4090,   4123,   4124,   4127,   4126}},
 {  2034,  30, {  4088,   4089,   4092,   4091,   4124,   4125,   4128,   4127}},
 {  2035,  30, {  4090,   4091,   4094,   4093,   4126,   4127,   4130,   4129}},
 {  2036,  30, {  4091,   4092,   4095,   4094,   4127,   4128,   4131,   4130}},
 {  2037,  30, {  4093,   4094,   4097,   4096,   4129,   4130,   4133,   4132}},
 {  2038,  30, {  4094,   4095,   4098,   4097,   4130,   4131,   4134,   4133}},
 {  2039,  30, {  4096,   4097,   4100,   4099,   4132,   4133,   4136,   4135}},
 {  2040,  30, {  4097,   4098,   4101,   4100,   4133,   4134,   4137,   4136}},
 {  2041,  30, {  4099,   4100,   4067,   4066,   4135,   4136,   4103,   4102}},
 {  2042,  30, {  4100,   4101,   4068,   4067,   4136,   4137,   4104,   4103}},
 {  2043,  29, {  4174,   4175,   4178,   4177,   4138,   4139,   4142,   4141}},
 {  2044,  29, {  4175,   4176,   4179,   4178,   4139,   4140,   4143,   4142}},
 {  2045,  29, {  4177,   4178,   4181,   4180,   4141,   4142,   4145,   4144}},
 {  2046,  29, {  4178,   4179,   4182,   4181,   4142,   4143,   4146,   4145}},
 {  2047,  29, {  4180,   4181,   4184,   4183,   4144,   4145,   4148,   4147}},
 {  2048,  29, {  4181,   4182,   4185,   4184,   4145,   4146,   4149,   4148}},
 {  2049,  29, {  4183,   4184,   4187,   4186,   4147,   4148,   4151,   4150}},
 {  2050,  29, {  4184,   4185,   4188,   4187,   4148,   4149,   4152,   4151}},
 {  2051,  29, {  4186,   4187,   4190,   4189,   4150,   4151,   4154,   4153}},
 {  2052,  29, {  4187,   4188,   4191,   4190,   4151,   4152,   4155,   4154}},
 {  2053,  29, {  4189,   4190,   4193,   4192,   4153,   4154,   4157,   4156}},
 {  2054,  29, {  4190,   4191,   4194,   4193,   4154,   4155,   4158,   4157}},
 {  2055,  29, {  4192,   4193,   4196,   4195,   4156,   4157,   4160,   4159}},
 {  2056,  29, {  4193,   4194,   4197,   4196,   4157,   4158,   4161,   4160}},
 {  2057,  29, {  4195,   4196,   4199,   4198,   4159,   4160,   4163,   4162}},
 {  2058,  29, {  4196,   4197,   4200,   4199,   4160,   4161,   4164,   4163}},
 {  2059,  29, {  4198,   4199,   4202,   4201,   4162,   4163,   4166,   4165}},
 {  2060,  29, {  4199,   4200,   4203,   4202,   4163,   4164,   4167,   4166}},
 {  2061,  29, {  4201,   4202,   4205,   4204,   4165,   4166,   4169,   4168}},
 {  2062,  29, {  4202,   4203,   4206,   4205,   4166,   4167,   4170,   4169}},
 {  2063,  29, {  4204,   4205,   4208,   4207,   4168,   4169,   4172,   4171}},
 {  2064,  29, {  4205,   4206,   4209,   4208,   4169,   4170,   4173,   4172}},
 {  2065,  29, {  4207,   4208,   4175,   4174,   4171,   4172,   4139,   4138}},
 {  2066,  29, {  4208,   4209,   4176,   4175,   4172,   4173,   4140,   4139}},
 {  2067,  29, {  4210,   4211,   4214,   4213,   4174,   4175,   4178,   4177}},
 {  2068,  29, {  4211,   4212,   4215,   4214,   4175,   4176,   4179,   4178}},
 {  2069,  29, {  4213,   4214,   4217,   4216,   4177,   4178,   4181,   4180}},
 {  2070,  29, {  4214,   4215,   4218,   4217,   4178,   4179,   4182,   4181}},
 {  2071,  29, {  4216,   4217,   4220,   4219,   4180,   4181,   4184,   4183}},
 {  2072,  29, {  4217,   4218,   4221,   4220,   4181,   4182,   4185,   4184}},
 {  2073,  29, {  4219,   4220,   4223,   4222,   4183,   4184,   4187,   4186}},
 {  2074,  29, {  4220,   4221,   4224,   4223,   4184,   4185,   4188,   4187}},
 {  2075,  29, {  4222,   4223,   4226,   4225,   4186,   4187,   4190,   4189}},
 {  2076,  29, {  4223,   4224,   4227,   4226,   4187,   4188,   4191,   4190}},
 {  2077,  29, {  4225,   4226,   4229,   4228,   4189,   4190,   4193,   4192}},
 {  2078,  29, {  4226,   4227,   4230,   4229,   4190,   4191,   4194,   4193}},
 {  2079,  29, {  4228,   4229,   4232,   4231,   4192,   4193,   4196,   4195}},
 {  2080,  29, {  4229,   4230,   4233,   4232,   4193,   4194,   4197,   4196}},
 {  2081,  29, {  4231,   4232,   4235,   4234,   4195,   4196,   4199,   4198}},
 {  2082,  29, {  4232,   4233,   4236,   4235,   4196,   4197,   4200,   4199}},
 {  2083,  29, {  4234,   4235,   4238,   4237,   4198,   4199,   4202,   4201}},
 {  2084,  29, {  4235,   4236,   4239,   4238,   4199,   4200,   4203,   4202}},
 {  2085,  29, {  4237,   4238,   4241,   4240,   4201,   4202,   4205,   4204}},
 {  2086,  29, {  4238,   4239,   4242,   4241,   4202,   4203,   4206,   4205}},
 {  2087,  29, {  4240,   4241,   4244,   4243,   4204,   4205,   4208,   4207}},
 {  2088,  29, {  4241,   4242,   4245,   4244,   4205,   4206,   4209,   4208}},
 {  2089,  29, {  4243,   4244,   4211,   4210,   4207,   4208,   4175,   4174}},
 {  2090,  29, {  4244,   4245,   4212,   4211,   4208,   4209,   4176,   4175}},
 {  2091,  29, {  4246,   4247,   4250,   4249,   4210,   4211,   4214,   4213}},
 {  2092,  29, {  4247,   4248,   4251,   4250,   4211,   4212,   4215,   4214}},
 {  2093,  29, {  4249,   4250,   4253,   4252,   4213,   4214,   4217,   4216}},
 {  2094,  29, {  4250,   4251,   4254,   4253,   4214,   4215,   4218,   4217}},
 {  2095,  29, {  4252,   4253,   4256,   4255,   4216,   4217,   4220,   4219}},
 {  2096,  29, {  4253,   4254,   4257,   4256,   4217,   4218,   4221,   4220}},
 {  2097,  29, {  4255,   4256,   4259,   4258,   4219,   4220,   4223,   4222}},
 {  2098,  29, {  4256,   4257,   4260,   4259,   4220,   4221,   4224,   4223}},
 {  2099,  29, {  4258,   4259,   4262,   4261,   4222,   4223,   4226,   4225}},
 {  2100,  29, {  4259,   4260,   4263,   4262,   4223,   4224,   4227,   4226}},
 {  2101,  29, {  4261,   4262,   4265,   4264,   4225,   4226,   4229,   4228}},
 {  2102,  29, {  4262,   4263,   4266,   4265,   4226,   4227,   4230,   4229}},
 {  2103,  29, {  4264,   4265,   4268,   4267,   4228,   4229,   4232,   4231}},
 {  2104,  29, {  4265,   4266,   4269,   4268,   4229,   4230,   4233,   4232}},
 {  2105,  29, {  4267,   4268,   4271,   4270,   4231,   4232,   4235,   4234}},
 {  2106,  29, {  4268,   4269,   4272,   4271,   4232,   4233,   4236,   4235}},
 {  2107,  29, {  4270,   4271,   4274,   4273,   4234,   4235,   4238,   4237}},
 {  2108,  29, {  4271,   4272,   4275,   4274,   4235,   4236,   4239,   4238}},
 {  2109,  29, {  4273,   4274,   4277,   4276,   4237,   4238,   4241,   4240}},
 {  2110,  29, {  4274,   4275,   4278,   4277,   4238,   4239,   4242,   4241}},
 {  2111,  29, {  4276,   4277,   4280,   4279,   4240,   4241,   4244,   4243}},
 {  2112,  29, {  4277,   4278,   4281,   4280,   4241,   4242,   4245,   4244}},
 {  2113,  29, {  4279,   4280,   4247,   4246,   4243,   4244,   4211,   4210}},
 {  2114,  29, {  4280,   4281,   4248,   4247,   4244,   4245,   4212,   4211}},
 {  2115,  29, {  4282,   4283,   4286,   4285,   4246,   4247,   4250,   4249}},
 {  2116,  29, {  4283,   4284,   4287,   4286,   4247,   4248,   4251,   4250}},
 {  2117,  29, {  4285,   4286,   4289,   4288,   4249,   4250,   4253,   4252}},
 {  2118,  29, {  4286,   4287,   4290,   4289,   4250,   4251,   4254,   4253}},
 {  2119,  29, {  4288,   4289,   4292,   4291,   4252,   4253,   4256,   4255}},
 {  2120,  29, {  4289,   4290,   4293,   4292,   4253,   4254,   4257,   4256}},
 {  2121,  29, {  4291,   4292,   4295,   4294,   4255,   4256,   4259,   4258}},
 {  2122,  29, {  4292,   4293,   4296,   4295,   4256,   4257,   4260,   4259}},
 {  2123,  29, {  4294,   4295,   4298,   4297,   4258,   4259,   4262,   4261}},
 {  2124,  29, {  4295,   4296,   4299,   4298,   4259,   4260,   4263,   4262}},
 {  2125,  29, {  4297,   4298,   4301,   4300,   4261,   4262,   4265,   4264}},
 {  2126,  29, {  4298,   4299,   4302,   4301,   4262,   4263,   4266,   4265}},
 {  2127,  29, {  4300,   4301,   4304,   4303,   4264,   4265,   4268,   4267}},
 {  2128,  29, {  4301,   4302,   4305,   4304,   4265,   4266,   4269,   4268}},
 {  2129,  29, {  4303,   4304,   4307,   4306,   4267,   4268,   4271,   4270}},
 {  2130,  29, {  4304,   4305,   4308,   4307,   4268,   4269,   4272,   4271}},
 {  2131,  29, {  4306,   4307,   4310,   4309,   4270,   4271,   4274,   4273}},
 {  2132,  29, {  4307,   4308,   4311,   4310,   4271,   4272,   4275,   4274}},
 {  2133,  29, {  4309,   4310,   4313,   4312,   4273,   4274,   4277,   4276}},
 {  2134,  29, {  4310,   4311,   4314,   4313,   4274,   4275,   4278,   4277}},
 {  2135,  29, {  4312,   4313,   4316,   4315,   4276,   4277,   4280,   4279}},
 {  2136,  29, {  4313,   4314,   4317,   4316,   4277,   4278,   4281,   4280}},
 {  2137,  29, {  4315,   4316,   4283,   4282,   4279,   4280,   4247,   4246}},
 {  2138,  29, {  4316,   4317,   4284,   4283,   4280,   4281,   4248,   4247}},
 {  2139,  72, {  4318,   4319,   4322,   4321,   4282,   4283,   4286,   4285}},
 {  2140,  72, {  4319,   4320,   4323,   4322,   4283,   4284,   4287,   4286}},
 {  2141,  72, {  4321,   4322,   4325,   4324,   4285,   4286,   4289,   4288}},
 {  2142,  72, {  4322,   4323,   4326,   4325,   4286,   4287,   4290,   4289}},
 {  2143,  72, {  4324,   4325,   4328,   4327,   4288,   4289,   4292,   4291}},
 {  2144,  72, {  4325,   4326,   4329,   4328,   4289,   4290,   4293,   4292}},
 {  2145,  72, {  4327,   4328,   4331,   4330,   4291,   4292,   4295,   4294}},
 {  2146,  72, {  4328,   4329,   4332,   4331,   4292,   4293,   4296,   4295}},
 {  2147,  72, {  4330,   4331,   4334,   4333,   4294,   4295,   4298,   4297}},
 {  2148,  72, {  4331,   4332,   4335,   4334,   4295,   4296,   4299,   4298}},
 {  2149,  72, {  4333,   4334,   4337,   4336,   4297,   4298,   4301,   4300}},
 {  2150,  72, {  4334,   4335,   4338,   4337,   4298,   4299,   4302,   4301}},
 {  2151,  72, {  4336,   4337,   4340,   4339,   4300,   4301,   4304,   4303}},
 {  2152,  72, {  4337,   4338,   4341,   4340,   4301,   4302,   4305,   4304}},
 {  2153,  72, {  4339,   4340,   4343,   4342,   4303,   4304,   4307,   4306}},
 {  2154,  72, {  4340,   4341,   4344,   4343,   4304,   4305,   4308,   4307}},
 {  2155,  72, {  4342,   4343,   4346,   4345,   4306,   4307,   4310,   4309}},
 {  2156,  72, {  4343,   4344,   4347,   4346,   4307,   4308,   4311,   4310}},
 {  2157,  72, {  4345,   4346,   4349,   4348,   4309,   4310,   4313,   4312}},
 {  2158,  72, {  4346,   4347,   4350,   4349,   4310,   4311,   4314,   4313}},
 {  2159,  72, {  4348,   4349,   4352,   4351,   4312,   4313,   4316,   4315}},
 {  2160,  72, {  4349,   4350,   4353,   4352,   4313,   4314,   4317,   4316}},
 {  2161,  72, {  4351,   4352,   4319,   4318,   4315,   4316,   4283,   4282}},
 {  2162,  72, {  4352,   4353,   4320,   4319,   4316,   4317,   4284,   4283}},
 {  2163,   6, {  4354,   4355,   4357,   4356,   4360,   4361,   4363,   4362}},
 {  2164,   6, {  4356,   4357,   4359,   4358,   4362,   4363,   4365,   4364}},
 {  2165,   6, {  4358,   4359,   4367,   4366,   4364,   4365,   4369,   4368}},
 {  2166,   6, {  4366,   4367,   4371,   4370,   4368,   4369,   4375,   4374}},
 {  2167,   6, {  4370,   4371,   4373,   4372,   4374,   4375,   4377,   4376}},
 {  2168,   6, {  4378,   4379,   4383,   4382,   4355,   4390,   4393,   4357}},
 {  2169,   6, {  4379,   4380,   4384,   4383,   4390,   4391,   4394,   4393}},
 {  2170,   6, {  4380,   4381,   4385,   4384,   4391,   4392,   4395,   4394}},
 {  2171,   6, {  4382,   4383,   4387,   4386,   4357,   4393,   4396,   4359}},
 {  2172,   6, {  4383,   4384,   4388,   4387,   4393,   4394,   4397,   4396}},
 {  2173,   6, {  4384,   4385,   4389,   4388,   4394,   4395,   4398,   4397}},
 {  2174,   6, {  4355,   4390,   4393,   4357,   4361,   4399,   4402,   4363}},
 {  2175,   6, {  4390,   4391,   4394,   4393,   4399,   4400,   4403,   4402}},
 {  2176,   6, {  4391,   4392,   4395,   4394,   4400,   4401,   4404,   4403}},
 {  2177,   6, {  4357,   4393,   4396,   4359,   4363,   4402,   4405,   4365}},
 {  2178,   6, {  4393,   4394,   4397,   4396,   4402,   4403,   4406,   4405}},
 {  2179,   6, {  4394,   4395,   4398,   4397,   4403,   4404,   4407,   4406}},
 {  2180,   6, {  4386,   4387,   4409,   4408,   4359,   4396,   4412,   4367}},
 {  2181,   6, {  4387,   4388,   4410,   4409,   4396,   4397,   4413,   4412}},
 {  2182,   6, {  4388,   4389,   4411,   4410,   4397,   4398,   4414,   4413}},
 {  2183,   6, {  4359,   4396,   4412,   4367,   4365,   4405,   4415,   4369}},
 {  2184,   6, {  4396,   4397,   4413,   4412,   4405,   4406,   4416,   4415}},
 {  2185,   6, {  4397,   4398,   4414,   4413,   4406,   4407,   4417,   4416}},
 {  2186,   6, {  4408,   4409,   4419,   4418,   4367,   4412,   4426,   4371}},
 {  2187,   6, {  4409,   4410,   4420,   4419,   4412,   4413,   4427,   4426}},
 {  2188,   6, {  4410,   4411,   4421,   4420,   4413,   4414,   4428,   4427}},
 {  2189,   6, {  4418,   4419,   4423,   4422,   4371,   4426,   4429,   4373}},
 {  2190,   6, {  4419,   4420,   4424,   4423,   4426,   4427,   4430,   4429}},
 {  2191,   6, {  4420,   4421,   4425,   4424,   4427,   4428,   4431,   4430}},
 {  2192,   6, {  4367,   4412,   4426,   4371,   4369,   4415,   4432,   4375}},
 {  2193,   6, {  4412,   4413,   4427,   4426,   4415,   4416,   4433,   4432}},
 {  2194,   6, {  4413,   4414,   4428,   4427,   4416,   4417,   4434,   4433}},
 {  2195,   6, {  4371,   4426,   4429,   4373,   4375,   4432,   4435,   4377}},
 {  2196,   6, {  4426,   4427,   4430,   4429,   4432,   4433,   4436,   4435}},
 {  2197,   6, {  4427,   4428,   4431,   4430,   4433,   4434,   4437,   4436}},
 {  2198,   6, {  4392,   4438,   4439,   4395,   4401,   4441,   4442,   4404}},
 {  2199,   6, {  4395,   4439,   4440,   4398,   4404,   4442,   4443,   4407}},
 {  2200,   6, {  4398,   4440,   4444,   4414,   4407,   4443,   4445,   4417}},
 {  2201,   6, {  4414,   4444,   4446,   4428,   4417,   4445,   4448,   4434}},
 {  2202,   6, {  4428,   4446,   4447,   4431,   4434,   4448,   4449,   4437}},
 {  2203,   7, {  4465,   4466,   4469,   4468,   4450,   4451,   4454,   4453}},
 {  2204,   7, {  4466,   4467,   4470,   4469,   4451,   4452,   4455,   4454}},
 {  2205,   7, {  4468,   4469,   4472,   4471,   4453,   4454,   4457,   4456}},
 {  2206,   7, {  4469,   4470,   4473,   4472,   4454,   4455,   4458,   4457}},
 {  2207,   7, {  4471,   4472,   4475,   4474,   4456,   4457,   4460,   4459}},
 {  2208,   7, {  4472,   4473,   4476,   4475,   4457,   4458,   4461,   4460}},
 {  2209,   7, {  4474,   4475,   4478,   4477,   4459,   4460,   4463,   4462}},
 {  2210,   7, {  4475,   4476,   4479,   4478,   4460,   4461,   4464,   4463}},
 {  2211,   7, {  4480,   4481,   4484,   4483,   4465,   4466,   4469,   4468}},
 {  2212,   7, {  4481,   4482,   4485,   4484,   4466,   4467,   4470,   4469}},
 {  2213,   7, {  4483,   4484,   4487,   4486,   4468,   4469,   4472,   4471}},
 {  2214,   7, {  4484,   4485,   4488,   4487,   4469,   4470,   4473,   4472}},
 {  2215,   7, {  4486,   4487,   4490,   4489,   4471,   4472,   4475,   4474}},
 {  2216,   7, {  4487,   4488,   4491,   4490,   4472,   4473,   4476,   4475}},
 {  2217,   7, {  4489,   4490,   4493,   4492,   4474,   4475,   4478,   4477}},
 {  2218,   7, {  4490,   4491,   4494,   4493,   4475,   4476,   4479,   4478}},
 {  2219,   7, {  4495,   4496,   4499,   4498,   4480,   4481,   4484,   4483}},
 {  2220,   7, {  4496,   4497,   4500,   4499,   4481,   4482,   4485,   4484}},
 {  2221,   7, {  4498,   4499,   4502,   4501,   4483,   4484,   4487,   4486}},
 {  2222,   7, {  4499,   4500,   4503,   4502,   4484,   4485,   4488,   4487}},
 {  2223,   7, {  4501,   4502,   4505,   4504,   4486,   4487,   4490,   4489}},
 {  2224,   7, {  4502,   4503,   4506,   4505,   4487,   4488,   4491,   4490}},
 {  2225,   7, {  4504,   4505,   4508,   4507,   4489,   4490,   4493,   4492}},
 {  2226,   7, {  4505,   4506,   4509,   4508,   4490,   4491,   4494,   4493}},
 {  2227,   7, {  4510,   4511,   4514,   4513,   4495,   4496,   4499,   4498}},
 {  2228,   7, {  4511,   4512,   4515,   4514,   4496,   4497,   4500,   4499}},
 {  2229,   7, {  4513,   4514,   4517,   4516,   4498,   4499,   4502,   4501}},
 {  2230,   7, {  4514,   4515,   4518,   4517,   4499,   4500,   4503,   4502}},
 {  2231,   7, {  4516,   4517,   4520,   4519,   4501,   4502,   4505,   4504}},
 {  2232,   7, {  4517,   4518,   4521,   4520,   4502,   4503,   4506,   4505}},
 {  2233,   7, {  4519,   4520,   4523,   4522,   4504,   4505,   4508,   4507}},
 {  2234,   7, {  4520,   4521,   4524,   4523,   4505,   4506,   4509,   4508}},
 {  2235,   7, {  4525,   4526,   4529,   4528,   4510,   4511,   4514,   4513}},
 {  2236,   7, {  4526,   4527,   4530,   4529,   4511,   4512,   4515,   4514}},
 {  2237,   7, {  4528,   4529,   4532,   4531,   4513,   4514,   4517,   4516}},
 {  2238,   7, {  4529,   4530,   4533,   4532,   4514,   4515,   4518,   4517}},
 {  2239,   7, {  4531,   4532,   4535,   4534,   4516,   4517,   4520,   4519}},
 {  2240,   7, {  4532,   4533,   4536,   4535,   4517,   4518,   4521,   4520}},
 {  2241,   7, {  4534,   4535,   4538,   4537,   4519,   4520,   4523,   4522}},
 {  2242,   7, {  4535,   4536,   4539,   4538,   4520,   4521,   4524,   4523}},
 {  2243,   7, {  4540,   4541,   4544,   4543,   4525,   4526,   4529,   4528}},
 {  2244,   7, {  4541,   4542,   4545,   4544,   4526,   4527,   4530,   4529}},
 {  2245,   7, {  4543,   4544,   4547,   4546,   4528,   4529,   4532,   4531}},
 {  2246,   7, {  4544,   4545,   4548,   4547,   4529,   4530,   4533,   4532}},
 {  2247,   7, {  4546,   4547,   4550,   4549,   4531,   4532,   4535,   4534}},
 {  2248,   7, {  4547,   4548,   4551,   4550,   4532,   4533,   4536,   4535}},
 {  2249,   7, {  4549,   4550,   4553,   4552,   4534,   4535,   4538,   4537}},
 {  2250,   7, {  4550,   4551,   4554,   4553,   4535,   4536,   4539,   4538}},
 {  2251,   7, {  4555,   4556,   4559,   4558,   4540,   4541,   4544,   4543}},
 {  2252,   7, {  4556,   4557,   4560,   4559,   4541,   4542,   4545,   4544}},
 {  2253,   7, {  4558,   4559,   4562,   4561,   4543,   4544,   4547,   4546}},
 {  2254,   7, {  4559,   4560,   4563,   4562,   4544,   4545,   4548,   4547}},
 {  2255,   7, {  4561,   4562,   4565,   4564,   4546,   4547,   4550,   4549}},
 {  2256,   7, {  4562,   4563,   4566,   4565,   4547,   4548,   4551,   4550}},
 {  2257,   7, {  4564,   4565,   4568,   4567,   4549,   4550,   4553,   4552}},
 {  2258,   7, {  4565,   4566,   4569,   4568,   4550,   4551,   4554,   4553}},
 {  2259,   7, {  4570,   4571,   4574,   4573,   4555,   4556,   4559,   4558}},
 {  2260,   7, {  4571,   4572,   4575,   4574,   4556,   4557,   4560,   4559}},
 {  2261,   7, {  4573,   4574,   4577,   4576,   4558,   4559,   4562,   4561}},
 {  2262,   7, {  4574,   4575,   4578,   4577,   4559,   4560,   4563,   4562}},
 {  2263,   7, {  4576,   4577,   4580,   4579,   4561,   4562,   4565,   4564}},
 {  2264,   7, {  4577,   4578,   4581,   4580,   4562,   4563,   4566,   4565}},
 {  2265,   7, {  4579,   4580,   4583,   4582,   4564,   4565,   4568,   4567}},
 {  2266,   7, {  4580,   4581,   4584,   4583,   4565,   4566,   4569,   4568}},
 {  2267,   7, {  4585,   4586,   4589,   4588,   4570,   4571,   4574,   4573}},
 {  2268,   7, {  4586,   4587,   4590,   4589,   4571,   4572,   4575,   4574}},
 {  2269,   7, {  4588,   4589,   4592,   4591,   4573,   4574,   4577,   4576}},
 {  2270,   7, {  4589,   4590,   4593,   4592,   4574,   4575,   4578,   4577}},
 {  2271,   7, {  4591,   4592,   4595,   4594,   4576,   4577,   4580,   4579}},
 {  2272,   7, {  4592,   4593,   4596,   4595,   4577,   4578,   4581,   4580}},
 {  2273,   7, {  4594,   4595,   4598,   4597,   4579,   4580,   4583,   4582}},
 {  2274,   7, {  4595,   4596,   4599,   4598,   4580,   4581,   4584,   4583}},
 {  2275,   7, {  4600,   4601,   4604,   4603,   4585,   4586,   4589,   4588}},
 {  2276,   7, {  4601,   4602,   4605,   4604,   4586,   4587,   4590,   4589}},
 {  2277,   7, {  4603,   4604,   4607,   4606,   4588,   4589,   4592,   4591}},
 {  2278,   7, {  4604,   4605,   4608,   4607,   4589,   4590,   4593,   4592}},
 {  2279,   7, {  4606,   4607,   4610,   4609,   4591,   4592,   4595,   4594}},
 {  2280,   7, {  4607,   4608,   4611,   4610,   4592,   4593,   4596,   4595}},
 {  2281,   7, {  4609,   4610,   4613,   4612,   4594,   4595,   4598,   4597}},
 {  2282,   7, {  4610,   4611,   4614,   4613,   4595,   4596,   4599,   4598}},
 {  2283,   7, {  4615,   4616,   4619,   4618,   4600,   4601,   4604,   4603}},
 {  2284,   7, {  4616,   4617,   4620,   4619,   4601,   4602,   4605,   4604}},
 {  2285,   7, {  4618,   4619,   4622,   4621,   4603,   4604,   4607,   4606}},
 {  2286,   7, {  4619,   4620,   4623,   4622,   4604,   4605,   4608,   4607}},
 {  2287,   7, {  4621,   4622,   4625,   4624,   4606,   4607,   4610,   4609}},
 {  2288,   7, {  4622,   4623,   4626,   4625,   4607,   4608,   4611,   4610}},
 {  2289,   7, {  4624,   4625,   4628,   4627,   4609,   4610,   4613,   4612}},
 {  2290,   7, {  4625,   4626,   4629,   4628,   4610,   4611,   4614,   4613}},
 {  2291,   7, {  4630,   4631,   4634,   4633,   4615,   4616,   4619,   4618}},
 {  2292,   7, {  4631,   4632,   4635,   4634,   4616,   4617,   4620,   4619}},
 {  2293,   7, {  4633,   4634,   4637,   4636,   4618,   4619,   4622,   4621}},
 {  2294,   7, {  4634,   4635,   4638,   4637,   4619,   4620,   4623,   4622}},
 {  2295,   7, {  4636,   4637,   4640,   4639,   4621,   4622,   4625,   4624}},
 {  2296,   7, {  4637,   4638,   4641,   4640,   4622,   4623,   4626,   4625}},
 {  2297,   7, {  4639,   4640,   4643,   4642,   4624,   4625,   4628,   4627}},
 {  2298,   7, {  4640,   4641,   4644,   4643,   4625,   4626,   4629,   4628}},
 {  2299,   7, {  4645,   4646,   4649,   4648,   4630,   4631,   4634,   4633}},
 {  2300,   7, {  4646,   4647,   4650,   4649,   4631,   4632,   4635,   4634}},
 {  2301,   7, {  4648,   4649,   4652,   4651,   4633,   4634,   4637,   4636}},
 {  2302,   7, {  4649,   4650,   4653,   4652,   4634,   4635,   4638,   4637}},
 {  2303,   7, {  4651,   4652,   4655,   4654,   4636,   4637,   4640,   4639}},
 {  2304,   7, {  4652,   4653,   4656,   4655,   4637,   4638,   4641,   4640}},
 {  2305,   7, {  4654,   4655,   4658,   4657,   4639,   4640,   4643,   4642}},
 {  2306,   7, {  4655,   4656,   4659,   4658,   4640,   4641,   4644,   4643}},
 {  2307,   7, {  4660,   4661,   4664,   4663,   4645,   4646,   4649,   4648}},
 {  2308,   7, {  4661,   4662,   4665,   4664,   4646,   4647,   4650,   4649}},
 {  2309,   7, {  4663,   4664,   4667,   4666,   4648,   4649,   4652,   4651}},
 {  2310,   7, {  4664,   4665,   4668,   4667,   4649,   4650,   4653,   4652}},
 {  2311,   7, {  4666,   4667,   4670,   4669,   4651,   4652,   4655,   4654}},
 {  2312,   7, {  4667,   4668,   4671,   4670,   4652,   4653,   4656,   4655}},
 {  2313,   7, {  4669,   4670,   4673,   4672,   4654,   4655,   4658,   4657}},
 {  2314,   7, {  4670,   4671,   4674,   4673,   4655,   4656,   4659,   4658}},
 {  2315,   7, {  4675,   4676,   4679,   4678,   4660,   4661,   4664,   4663}},
 {  2316,   7, {  4676,   4677,   4680,   4679,   4661,   4662,   4665,   4664}},
 {  2317,   7, {  4678,   4679,   4682,   4681,   4663,   4664,   4667,   4666}},
 {  2318,   7, {  4679,   4680,   4683,   4682,   4664,   4665,   4668,   4667}},
 {  2319,   7, {  4681,   4682,   4685,   4684,   4666,   4667,   4670,   4669}},
 {  2320,   7, {  4682,   4683,   4686,   4685,   4667,   4668,   4671,   4670}},
 {  2321,   7, {  4684,   4685,   4688,   4687,   4669,   4670,   4673,   4672}},
 {  2322,   7, {  4685,   4686,   4689,   4688,   4670,   4671,   4674,   4673}},
 {  2323,   7, {  4690,   4691,   4694,   4693,   4675,   4676,   4679,   4678}},
 {  2324,   7, {  4691,   4692,   4695,   4694,   4676,   4677,   4680,   4679}},
 {  2325,   7, {  4693,   4694,   4697,   4696,   4678,   4679,   4682,   4681}},
 {  2326,   7, {  4694,   4695,   4698,   4697,   4679,   4680,   4683,   4682}},
 {  2327,   7, {  4696,   4697,   4700,   4699,   4681,   4682,   4685,   4684}},
 {  2328,   7, {  4697,   4698,   4701,   4700,   4682,   4683,   4686,   4685}},
 {  2329,   7, {  4699,   4700,   4703,   4702,   4684,   4685,   4688,   4687}},
 {  2330,   7, {  4700,   4701,   4704,   4703,   4685,   4686,   4689,   4688}},
 {  2331,   8, {  4711,   4712,   4715,   4714,   4705,   4706,   4709,   4708}},
 {  2332,   8, {  4712,   4713,   4716,   4715,   4706,   4707,   4710,   4709}},
 {  2333,   8, {  4717,   4718,   4721,   4720,   4711,   4712,   4715,   4714}},
 {  2334,   8, {  4718,   4719,   4722,   4721,   4712,   4713,   4716,   4715}},
 {  2335,   8, {  4723,   4724,   4727,   4726,   4717,   4718,   4721,   4720}},
 {  2336,   8, {  4724,   4725,   4728,   4727,   4718,   4719,   4722,   4721}},
 {  2337,   8, {  4729,   4730,   4733,   4732,   4723,   4724,   4727,   4726}},
 {  2338,   8, {  4730,   4731,   4734,   4733,   4724,   4725,   4728,   4727}},
 {  2339,   8, {  4735,   4736,   4739,   4738,   4729,   4730,   4733,   4732}},
 {  2340,   8, {  4736,   4737,   4740,   4739,   4730,   4731,   4734,   4733}},
 {  2341,   8, {  4741,   4742,   4745,   4744,   4735,   4736,   4739,   4738}},
 {  2342,   8, {  4742,   4743,   4746,   4745,   4736,   4737,   4740,   4739}},
 {  2343,   8, {  4747,   4748,   4751,   4750,   4741,   4742,   4745,   4744}},
 {  2344,   8, {  4748,   4749,   4752,   4751,   4742,   4743,   4746,   4745}},
 {  2345,   8, {  4753,   4754,   4757,   4756,   4747,   4748,   4751,   4750}},
 {  2346,   8, {  4754,   4755,   4758,   4757,   4748,   4749,   4752,   4751}},
 {  2347,   8, {  4759,   4760,   4763,   4762,   4753,   4754,   4757,   4756}},
 {  2348,   8, {  4760,   4761,   4764,   4763,   4754,   4755,   4758,   4757}},
 {  2349,   8, {  4765,   4766,   4769,   4768,   4759,   4760,   4763,   4762}},
 {  2350,   8, {  4766,   4767,   4770,   4769,   4760,   4761,   4764,   4763}},
 {  2351,   8, {  4771,   4772,   4775,   4774,   4765,   4766,   4769,   4768}},
 {  2352,   8, {  4772,   4773,   4776,   4775,   4766,   4767,   4770,   4769}},
 {  2353,   8, {  4777,   4778,   4781,   4780,   4771,   4772,   4775,   4774}},
 {  2354,   8, {  4778,   4779,   4782,   4781,   4772,   4773,   4776,   4775}},
 {  2355,   8, {  4783,   4784,   4787,   4786,   4777,   4778,   4781,   4780}},
 {  2356,   8, {  4784,   4785,   4788,   4787,   4778,   4779,   4782,   4781}},
 {  2357,   8, {  4789,   4790,   4793,   4792,   4783,   4784,   4787,   4786}},
 {  2358,   8, {  4790,   4791,   4794,   4793,   4784,   4785,   4788,   4787}},
 {  2359,   8, {  4795,   4796,   4799,   4798,   4789,   4790,   4793,   4792}},
 {  2360,   8, {  4796,   4797,   4800,   4799,   4790,   4791,   4794,   4793}},
 {  2361,   8, {  4801,   4802,   4805,   4804,   4795,   4796,   4799,   4798}},
 {  2362,   8, {  4802,   4803,   4806,   4805,   4796,   4797,   4800,   4799}},
 {  2363,   8, {  4807,   4808,   4811,   4810,   4801,   4802,   4805,   4804}},
 {  2364,   8, {  4808,   4809,   4812,   4811,   4802,   4803,   4806,   4805}},
 {  2365,   8, {  4714,   4715,   4821,   4820,   4708,   4709,   4814,   4813}},
 {  2366,   8, {  4715,   4716,   4822,   4821,   4709,   4710,   4815,   4814}},
 {  2367,   8, {  4820,   4821,   4824,   4823,   4813,   4814,   4817,   4816}},
 {  2368,   8, {  4821,   4822,   4825,   4824,   4814,   4815,   4818,   4817}},
 {  2369,   8, {  4823,   4824,   4826,    103,   4816,   4817,   4819,     30}},
 {  2370,   8, {  4824,   4825,   4465,   4826,   4817,   4818,   4450,   4819}},
 {  2371,   8, {  4720,   4721,   4828,   4827,   4714,   4715,   4821,   4820}},
 {  2372,   8, {  4721,   4722,   4829,   4828,   4715,   4716,   4822,   4821}},
 {  2373,   8, {  4827,   4828,   4831,   4830,   4820,   4821,   4824,   4823}},
 {  2374,   8, {  4828,   4829,   4832,   4831,   4821,   4822,   4825,   4824}},
 {  2375,   8, {  4830,   4831,   4833,    107,   4823,   4824,   4826,    103}},
 {  2376,   8, {  4831,   4832,   4480,   4833,   4824,   4825,   4465,   4826}},
 {  2377,   8, {  4726,   4727,   4835,   4834,   4720,   4721,   4828,   4827}},
 {  2378,   8, {  4727,   4728,   4836,   4835,   4721,   4722,   4829,   4828}},
 {  2379,   8, {  4834,   4835,   4838,   4837,   4827,   4828,   4831,   4830}},
 {  2380,   8, {  4835,   4836,   4839,   4838,   4828,   4829,   4832,   4831}},
 {  2381,   8, {  4837,   4838,   4840,    111,   4830,   4831,   4833,    107}},
 {  2382,   8, {  4838,   4839,   4495,   4840,   4831,   4832,   4480,   4833}},
 {  2383,   8, {  4732,   4733,   4842,   4841,   4726,   4727,   4835,   4834}},
 {  2384,   8, {  4733,   4734,   4843,   4842,   4727,   4728,   4836,   4835}},
 {  2385,   8, {  4841,   4842,   4845,   4844,   4834,   4835,   4838,   4837}},
 {  2386,   8, {  4842,   4843,   4846,   4845,   4835,   4836,   4839,   4838}},
 {  2387,   8, {  4844,   4845,   4847,    115,   4837,   4838,   4840,    111}},
 {  2388,   8, {  4845,   4846,   4510,   4847,   4838,   4839,   4495,   4840}},
 {  2389,   8, {  4738,   4739,   4849,   4848,   4732,   4733,   4842,   4841}},
 {  2390,   8, {  4739,   4740,   4850,   4849,   4733,   4734,   4843,   4842}},
 {  2391,   8, {  4848,   4849,   4852,   4851,   4841,   4842,   4845,   4844}},
 {  2392,   8, {  4849,   4850,   4853,   4852,   4842,   4843,   4846,   4845}},
 {  2393,   8, {  4851,   4852,   4854,    119,   4844,   4845,   4847,    115}},
 {  2394,   8, {  4852,   4853,   4525,   4854,   4845,   4846,   4510,   4847}},
 {  2395,   8, {  4744,   4745,   4856,   4855,   4738,   4739,   4849,   4848}},
 {  2396,   8, {  4745,   4746,   4857,   4856,   4739,   4740,   4850,   4849}},
 {  2397,   8, {  4855,   4856,   4859,   4858,   4848,   4849,   4852,   4851}},
 {  2398,   8, {  4856,   4857,   4860,   4859,   4849,   4850,   4853,   4852}},
 {  2399,   8, {  4858,   4859,   4861,    123,   4851,   4852,   4854,    119}},
 {  2400,   8, {  4859,   4860,   4540,   4861,   4852,   4853,   4525,   4854}},
 {  2401,   8, {  4750,   4751,   4863,   4862,   4744,   4745,   4856,   4855}},
 {  2402,   8, {  4751,   4752,   4864,   4863,   4745,   4746,   4857,   4856}},
 {  2403,   8, {  4862,   4863,   4866,   4865,   4855,   4856,   4859,   4858}},
 {  2404,   8, {  4863,   4864,   4867,   4866,   4856,   4857,   4860,   4859}},
 {  2405,   8, {  4865,   4866,   4868,    127,   4858,   4859,   4861,    123}},
 {  2406,   8, {  4866,   4867,   4555,   4868,   4859,   4860,   4540,   4861}},
 {  2407,   8, {  4756,   4757,   4870,   4869,   4750,   4751,   4863,   4862}},
 {  2408,   8, {  4757,   4758,   4871,   4870,   4751,   4752,   4864,   4863}},
 {  2409,   8, {  4869,   4870,   4873,   4872,   4862,   4863,   4866,   4865}},
 {  2410,   8, {  4870,   4871,   4874,   4873,   4863,   4864,   4867,   4866}},
 {  2411,   8, {  4872,   4873,   4875,    131,   4865,   4866,   4868,    127}},
 {  2412,   8, {  4873,   4874,   4570,   4875,   4866,   4867,   4555,   4868}},
 {  2413,   8, {  4762,   4763,   4877,   4876,   4756,   4757,   4870,   4869}},
 {  2414,   8, {  4763,   4764,   4878,   4877,   4757,   4758,   4871,   4870}},
 {  2415,   8, {  4876,   4877,   4880,   4879,   4869,   4870,   4873,   4872}},
 {  2416,   8, {  4877,   4878,   4881,   4880,   4870,   4871,   4874,   4873}},
 {  2417,   8, {  4879,   4880,   4882,    135,   4872,   4873,   4875,    131}},
 {  2418,   8, {  4880,   4881,   4585,   4882,   4873,   4874,   4570,   4875}},
 {  2419,   8, {  4768,   4769,   4884,   4883,   4762,   4763,   4877,   4876}},
 {  2420,   8, {  4769,   4770,   4885,   4884,   4763,   4764,   4878,   4877}},
 {  2421,   8, {  4883,   4884,   4887,   4886,   4876,   4877,   4880,   4879}},
 {  2422,   8, {  4884,   4885,   4888,   4887,   4877,   4878,   4881,   4880}},
 {  2423,   8, {  4886,   4887,   4889,    139,   4879,   4880,   4882,    135}},
 {  2424,   8, {  4887,   4888,   4600,   4889,   4880,   4881,   4585,   4882}},
 {  2425,   8, {  4774,   4775,   4891,   4890,   4768,   4769,   4884,   4883}},
 {  2426,   8, {  4775,   4776,   4892,   4891,   4769,   4770,   4885,   4884}},
 {  2427,   8, {  4890,   4891,   4894,   4893,   4883,   4884,   4887,   4886}},
 {  2428,   8, {  4891,   4892,   4895,   4894,   4884,   4885,   4888,   4887}},
 {  2429,   8, {  4893,   4894,   4896,    143,   4886,   4887,   4889,    139}},
 {  2430,   8, {  4894,   4895,   4615,   4896,   4887,   4888,   4600,   4889}},
 {  2431,   8, {  4780,   4781,   4898,   4897,   4774,   4775,   4891,   4890}},
 {  2432,   8, {  4781,   4782,   4899,   4898,   4775,   4776,   4892,   4891}},
 {  2433,   8, {  4897,   4898,   4901,   4900,   4890,   4891,   4894,   4893}},
 {  2434,   8, {  4898,   4899,   4902,   4901,   4891,   4892,   4895,   4894}},
 {  2435,   8, {  4900,   4901,   4903,    147,   4893,   4894,   4896,    143}},
 {  2436,   8, {  4901,   4902,   4630,   4903,   4894,   4895,   4615,   4896}},
 {  2437,   8, {  4786,   4787,   4905,   4904,   4780,   4781,   4898,   4897}},
 {  2438,   8, {  4787,   4788,   4906,   4905,   4781,   4782,   4899,   4898}},
 {  2439,   8, {  4904,   4905,   4908,   4907,   4897,   4898,   4901,   4900}},
 {  2440,   8, {  4905,   4906,   4909,   4908,   4898,   4899,   4902,   4901}},
 {  2441,   8, {  4907,   4908,   4910,    151,   4900,   4901,   4903,    147}},
 {  2442,   8, {  4908,   4909,   4645,   4910,   4901,   4902,   4630,   4903}},
 {  2443,   8, {  4792,   4793,   4912,   4911,   4786,   4787,   4905,   4904}},
 {  2444,   8, {  4793,   4794,   4913,   4912,   4787,   4788,   4906,   4905}},
 {  2445,   8, {  4911,   4912,   4915,   4914,   4904,   4905,   4908,   4907}},
 {  2446,   8, {  4912,   4913,   4916,   4915,   4905,   4906,   4909,   4908}},
 {  2447,   8, {  4914,   4915,   4917,    155,   4907,   4908,   4910,    151}},
 {  2448,   8, {  4915,   4916,   4660,   4917,   4908,   4909,   4645,   4910}},
 {  2449,   8, {  4798,   4799,   4919,   4918,   4792,   4793,   4912,   4911}},
 {  2450,   8, {  4799,   4800,   4920,   4919,   4793,   4794,   4913,   4912}},
 {  2451,   8, {  4918,   4919,   4922,   4921,   4911,   4912,   4915,   4914}},
 {  2452,   8, {  4919,   4920,   4923,   4922,   4912,   4913,   4916,   4915}},
 {  2453,   8, {  4921,   4922,   4924,    159,   4914,   4915,   4917,    155}},
 {  2454,   8, {  4922,   4923,   4675,   4924,   4915,   4916,   4660,   4917}},
 {  2455,   8, {  4804,   4805,   4926,   4925,   4798,   4799,   4919,   4918}},
 {  2456,   8, {  4805,   4806,   4927,   4926,   4799,   4800,   4920,   4919}},
 {  2457,   8, {  4925,   4926,   4929,   4928,   4918,   4919,   4922,   4921}},
 {  2458,   8, {  4926,   4927,   4930,   4929,   4919,   4920,   4923,   4922}},
 {  2459,   8, {  4928,   4929,   4931,    163,   4921,   4922,   4924,    159}},
 {  2460,   8, {  4929,   4930,   4690,   4931,   4922,   4923,   4675,   4924}},
 {  2461,   8, {   103,   4826,   4936,    181,     30,   4819,   4932,     64}},
 {  2462,   8, {  4826,   4465,   4468,   4936,   4819,   4450,   4453,   4932}},
 {  2463,   8, {   181,   4936,   4937,    183,     64,   4932,   4933,     66}},
 {  2464,   8, {  4936,   4468,   4471,   4937,   4932,   4453,   4456,   4933}},
 {  2465,   8, {   183,   4937,   4938,    185,     66,   4933,   4934,     68}},
 {  2466,   8, {  4937,   4471,   4474,   4938,   4933,   4456,   4459,   4934}},
 {  2467,   8, {   185,   4938,   4939,    187,     68,   4934,   4935,     70}},
 {  2468,   8, {  4938,   4474,   4477,   4939,   4934,   4459,   4462,   4935}},
 {  2469,   8, {   107,   4833,   4940,    189,    103,   4826,   4936,    181}},
 {  2470,   8, {  4833,   4480,   4483,   4940,   4826,   4465,   4468,   4936}},
 {  2471,   8, {   189,   4940,   4941,    191,    181,   4936,   4937,    183}},
 {  2472,   8, {  4940,   4483,   4486,   4941,   4936,   4468,   4471,   4937}},
 {  2473,   8, {   191,   4941,   4942,    193,    183,   4937,   4938,    185}},
 {  2474,   8, {  4941,   4486,   4489,   4942,   4937,   4471,   4474,   4938}},
 {  2475,   8, {   193,   4942,   4943,    195,    185,   4938,   4939,    187}},
 {  2476,   8, {  4942,   4489,   4492,   4943,   4938,   4474,   4477,   4939}},
 {  2477,   8, {   111,   4840,   4944,    197,    107,   4833,   4940,    189}},
 {  2478,   8, {  4840,   4495,   4498,   4944,   4833,   4480,   4483,   4940}},
 {  2479,   8, {   197,   4944,   4945,    199,    189,   4940,   4941,    191}},
 {  2480,   8, {  4944,   4498,   4501,   4945,   4940,   4483,   4486,   4941}},
 {  2481,   8, {   199,   4945,   4946,    201,    191,   4941,   4942,    193}},
 {  2482,   8, {  4945,   4501,   4504,   4946,   4941,   4486,   4489,   4942}},
 {  2483,   8, {   201,   4946,   4947,    203,    193,   4942,   4943,    195}},
 {  2484,   8, {  4946,   4504,   4507,   4947,   4942,   4489,   4492,   4943}},
 {  2485,   8, {   115,   4847,   4948,    205,    111,   4840,   4944,    197}},
 {  2486,   8, {  4847,   4510,   4513,   4948,   4840,   4495,   4498,   4944}},
 {  2487,   8, {   205,   4948,   4949,    207,    197,   4944,   4945,    199}},
 {  2488,   8, {  4948,   4513,   4516,   4949,   4944,   4498,   4501,   4945}},
 {  2489,   8, {   207,   4949,   4950,    209,    199,   4945,   4946,    201}},
 {  2490,   8, {  4949,   4516,   4519,   4950,   4945,   4501,   4504,   4946}},
 {  2491,   8, {   209,   4950,   4951,    211,    201,   4946,   4947,    203}},
 {  2492,   8, {  4950,   4519,   4522,   4951,   4946,   4504,   4507,   4947}},
 {  2493,   8, {   119,   4854,   4952,    213,    115,   4847,   4948,    205}},
 {  2494,   8, {  4854,   4525,   4528,   4952,   4847,   4510,   4513,   4948}},
 {  2495,   8, {   213,   4952,   4953,    215,    205,   4948,   4949,    207}},
 {  2496,   8, {  4952,   4528,   4531,   4953,   4948,   4513,   4516,   4949}},
 {  2497,   8, {   215,   4953,   4954,    217,    207,   4949,   4950,    209}},
 {  2498,   8, {  4953,   4531,   4534,   4954,   4949,   4516,   4519,   4950}},
 {  2499,   8, {   217,   4954,   4955,    219,    209,   4950,   4951,    211}},
 {  2500,   8, {  4954,   4534,   4537,   4955,   4950,   4519,   4522,   4951}},
 {  2501,   8, {   123,   4861,   4956,    221,    119,   4854,   4952,    213}},
 {  2502,   8, {  4861,   4540,   4543,   4956,   4854,   4525,   4528,   4952}},
 {  2503,   8, {   221,   4956,   4957,    223,    213,   4952,   4953,    215}},
 {  2504,   8, {  4956,   4543,   4546,   4957,   4952,   4528,   4531,   4953}},
 {  2505,   8, {   223,   4957,   4958,    225,    215,   4953,   4954,    217}},
 {  2506,   8, {  4957,   4546,   4549,   4958,   4953,   4531,   4534,   4954}},
 {  2507,   8, {   225,   4958,   4959,    227,    217,   4954,   4955,    219}},
 {  2508,   8, {  4958,   4549,   4552,   4959,   4954,   4534,   4537,   4955}},
 {  2509,   8, {   127,   4868,   4960,    229,    123,   4861,   4956,    221}},
 {  2510,   8, {  4868,   4555,   4558,   4960,   4861,   4540,   4543,   4956}},
 {  2511,   8, {   229,   4960,   4961,    231,    221,   4956,   4957,    223}},
 {  2512,   8, {  4960,   4558,   4561,   4961,   4956,   4543,   4546,   4957}},
 {  2513,   8, {   231,   4961,   4962,    233,    223,   4957,   4958,    225}},
 {  2514,   8, {  4961,   4561,   4564,   4962,   4957,   4546,   4549,   4958}},
 {  2515,   8, {   233,   4962,   4963,    235,    225,   4958,   4959,    227}},
 {  2516,   8, {  4962,   4564,   4567,   4963,   4958,   4549,   4552,   4959}},
 {  2517,   8, {   131,   4875,   4964,    237,    127,   4868,   4960,    229}},
 {  2518,   8, {  4875,   4570,   4573,   4964,   4868,   4555,   4558,   4960}},
 {  2519,   8, {   237,   4964,   4965,    239,    229,   4960,   4961,    231}},
 {  2520,   8, {  4964,   4573,   4576,   4965,   4960,   4558,   4561,   4961}},
 {  2521,   8, {   239,   4965,   4966,    241,    231,   4961,   4962,    233}},
 {  2522,   8, {  4965,   4576,   4579,   4966,   4961,   4561,   4564,   4962}},
 {  2523,   8, {   241,   4966,   4967,    243,    233,   4962,   4963,    235}},
 {  2524,   8, {  4966,   4579,   4582,   4967,   4962,   4564,   4567,   4963}},
 {  2525,   8, {   135,   4882,   4968,    245,    131,   4875,   4964,    237}},
 {  2526,   8, {  4882,   4585,   4588,   4968,   4875,   4570,   4573,   4964}},
 {  2527,   8, {   245,   4968,   4969,    247,    237,   4964,   4965,    239}},
 {  2528,   8, {  4968,   4588,   4591,   4969,   4964,   4573,   4576,   4965}},
 {  2529,   8, {   247,   4969,   4970,    249,    239,   4965,   4966,    241}},
 {  2530,   8, {  4969,   4591,   4594,   4970,   4965,   4576,   4579,   4966}},
 {  2531,   8, {   249,   4970,   4971,    251,    241,   4966,   4967,    243}},
 {  2532,   8, {  4970,   4594,   4597,   4971,   4966,   4579,   4582,   4967}},
 {  2533,   8, {   139,   4889,   4972,    253,    135,   4882,   4968,    245}},
 {  2534,   8, {  4889,   4600,   4603,   4972,   4882,   4585,   4588,   4968}},
 {  2535,   8, {   253,   4972,   4973,    255,    245,   4968,   4969,    247}},
 {  2536,   8, {  4972,   4603,   4606,   4973,   4968,   4588,   4591,   4969}},
 {  2537,   8, {   255,   4973,   4974,    257,    247,   4969,   4970,    249}},
 {  2538,   8, {  4973,   4606,   4609,   4974,   4969,   4591,   4594,   4970}},
 {  2539,   8, {   257,   4974,   4975,    259,    249,   4970,   4971,    251}},
 {  2540,   8, {  4974,   4609,   4612,   4975,   4970,   4594,   4597,   4971}},
 {  2541,   8, {   143,   4896,   4976,    261,    139,   4889,   4972,    253}},
 {  2542,   8, {  4896,   4615,   4618,   4976,   4889,   4600,   4603,   4972}},
 {  2543,   8, {   261,   4976,   4977,    263,    253,   4972,   4973,    255}},
 {  2544,   8, {  4976,   4618,   4621,   4977,   4972,   4603,   4606,   4973}},
 {  2545,   8, {   263,   4977,   4978,    265,    255,   4973,   4974,    257}},
 {  2546,   8, {  4977,   4621,   4624,   4978,   4973,   4606,   4609,   4974}},
 {  2547,   8, {   265,   4978,   4979,    267,    257,   4974,   4975,    259}},
 {  2548,   8, {  4978,   4624,   4627,   4979,   4974,   4609,   4612,   4975}},
 {  2549,   8, {   147,   4903,   4980,    269,    143,   4896,   4976,    261}},
 {  2550,   8, {  4903,   4630,   4633,   4980,   4896,   4615,   4618,   4976}},
 {  2551,   8, {   269,   4980,   4981,    271,    261,   4976,   4977,    263}},
 {  2552,   8, {  4980,   4633,   4636,   4981,   4976,   4618,   4621,   4977}},
 {  2553,   8, {   271,   4981,   4982,    273,    263,   4977,   4978,    265}},
 {  2554,   8, {  4981,   4636,   4639,   4982,   4977,   4621,   4624,   4978}},
 {  2555,   8, {   273,   4982,   4983,    275,    265,   4978,   4979,    267}},
 {  2556,   8, {  4982,   4639,   4642,   4983,   4978,   4624,   4627,   4979}},
 {  2557,   8, {   151,   4910,   4984,    277,    147,   4903,   4980,    269}},
 {  2558,   8, {  4910,   4645,   4648,   4984,   4903,   4630,   4633,   4980}},
 {  2559,   8, {   277,   4984,   4985,    279,    269,   4980,   4981,    271}},
 {  2560,   8, {  4984,   4648,   4651,   4985,   4980,   4633,   4636,   4981}},
 {  2561,   8, {   279,   4985,   4986,    281,    271,   4981,   4982,    273}},
 {  2562,   8, {  4985,   4651,   4654,   4986,   4981,   4636,   4639,   4982}},
 {  2563,   8, {   281,   4986,   4987,    283,    273,   4982,   4983,    275}},
 {  2564,   8, {  4986,   4654,   4657,   4987,   4982,   4639,   4642,   4983}},
 {  2565,   8, {   155,   4917,   4988,    285,    151,   4910,   4984,    277}},
 {  2566,   8, {  4917,   4660,   4663,   4988,   4910,   4645,   4648,   4984}},
 {  2567,   8, {   285,   4988,   4989,    287,    277,   4984,   4985,    279}},
 {  2568,   8, {  4988,   4663,   4666,   4989,   4984,   4648,   4651,   4985}},
 {  2569,   8, {   287,   4989,   4990,    289,    279,   4985,   4986,    281}},
 {  2570,   8, {  4989,   4666,   4669,   4990,   4985,   4651,   4654,   4986}},
 {  2571,   8, {   289,   4990,   4991,    291,    281,   4986,   4987,    283}},
 {  2572,   8, {  4990,   4669,   4672,   4991,   4986,   4654,   4657,   4987}},
 {  2573,   8, {   159,   4924,   4992,    293,    155,   4917,   4988,    285}},
 {  2574,   8, {  4924,   4675,   4678,   4992,   4917,   4660,   4663,   4988}},
 {  2575,   8, {   293,   4992,   4993,    295,    285,   4988,   4989,    287}},
 {  2576,   8, {  4992,   4678,   4681,   4993,   4988,   4663,   4666,   4989}},
 {  2577,   8, {   295,   4993,   4994,    297,    287,   4989,   4990,    289}},
 {  2578,   8, {  4993,   4681,   4684,   4994,   4989,   4666,   4669,   4990}},
 {  2579,   8, {   297,   4994,   4995,    299,    289,   4990,   4991,    291}},
 {  2580,   8, {  4994,   4684,   4687,   4995,   4990,   4669,   4672,   4991}},
 {  2581,   8, {   163,   4931,   4996,    301,    159,   4924,   4992,    293}},
 {  2582,   8, {  4931,   4690,   4693,   4996,   4924,   4675,   4678,   4992}},
 {  2583,   8, {   301,   4996,   4997,    303,    293,   4992,   4993,    295}},
 {  2584,   8, {  4996,   4693,   4696,   4997,   4992,   4678,   4681,   4993}},
 {  2585,   8, {   303,   4997,   4998,    305,    295,   4993,   4994,    297}},
 {  2586,   8, {  4997,   4696,   4699,   4998,   4993,   4681,   4684,   4994}},
 {  2587,   8, {   305,   4998,   4999,    307,    297,   4994,   4995,    299}},
 {  2588,   8, {  4998,   4699,   4702,   4999,   4994,   4684,   4687,   4995}},
 {  2589,   8, {   187,   4939,   5010,   5009,     70,   4935,   5001,   5000}},
 {  2590,   8, {  4939,   4477,   5011,   5010,   4935,   4462,   5002,   5001}},
 {  2591,   8, {  5009,   5010,   5013,   5012,   5000,   5001,   5004,   5003}},
 {  2592,   8, {  5010,   5011,   5014,   5013,   5001,   5002,   5005,   5004}},
 {  2593,   8, {  5012,   5013,   5016,   5015,   5003,   5004,   5007,   5006}},
 {  2594,   8, {  5013,   5014,   5017,   5016,   5004,   5005,   5008,   5007}},
 {  2595,   8, {   195,   4943,   5019,   5018,    187,   4939,   5010,   5009}},
 {  2596,   8, {  4943,   4492,   5020,   5019,   4939,   4477,   5011,   5010}},
 {  2597,   8, {  5018,   5019,   5022,   5021,   5009,   5010,   5013,   5012}},
 {  2598,   8, {  5019,   5020,   5023,   5022,   5010,   5011,   5014,   5013}},
 {  2599,   8, {  5021,   5022,   5025,   5024,   5012,   5013,   5016,   5015}},
 {  2600,   8, {  5022,   5023,   5026,   5025,   5013,   5014,   5017,   5016}},
 {  2601,   8, {   203,   4947,   5028,   5027,    195,   4943,   5019,   5018}},
 {  2602,   8, {  4947,   4507,   5029,   5028,   4943,   4492,   5020,   5019}},
 {  2603,   8, {  5027,   5028,   5031,   5030,   5018,   5019,   5022,   5021}},
 {  2604,   8, {  5028,   5029,   5032,   5031,   5019,   5020,   5023,   5022}},
 {  2605,   8, {  5030,   5031,   5034,   5033,   5021,   5022,   5025,   5024}},
 {  2606,   8, {  5031,   5032,   5035,   5034,   5022,   5023,   5026,   5025}},
 {  2607,   8, {   211,   4951,   5037,   5036,    203,   4947,   5028,   5027}},
 {  2608,   8, {  4951,   4522,   5038,   5037,   4947,   4507,   5029,   5028}},
 {  2609,   8, {  5036,   5037,   5040,   5039,   5027,   5028,   5031,   5030}},
 {  2610,   8, {  5037,   5038,   5041,   5040,   5028,   5029,   5032,   5031}},
 {  2611,   8, {  5039,   5040,   5043,   5042,   5030,   5031,   5034,   5033}},
 {  2612,   8, {  5040,   5041,   5044,   5043,   5031,   5032,   5035,   5034}},
 {  2613,   8, {   219,   4955,   5046,   5045,    211,   4951,   5037,   5036}},
 {  2614,   8, {  4955,   4537,   5047,   5046,   4951,   4522,   5038,   5037}},
 {  2615,   8, {  5045,   5046,   5049,   5048,   5036,   5037,   5040,   5039}},
 {  2616,   8, {  5046,   5047,   5050,   5049,   5037,   5038,   5041,   5040}},
 {  2617,   8, {  5048,   5049,   5052,   5051,   5039,   5040,   5043,   5042}},
 {  2618,   8, {  5049,   5050,   5053,   5052,   5040,   5041,   5044,   5043}},
 {  2619,   8, {   227,   4959,   5055,   5054,    219,   4955,   5046,   5045}},
 {  2620,   8, {  4959,   4552,   5056,   5055,   4955,   4537,   5047,   5046}},
 {  2621,   8, {  5054,   5055,   5058,   5057,   5045,   5046,   5049,   5048}},
 {  2622,   8, {  5055,   5056,   5059,   5058,   5046,   5047,   5050,   5049}},
 {  2623,   8, {  5057,   5058,   5061,   5060,   5048,   5049,   5052,   5051}},
 {  2624,   8, {  5058,   5059,   5062,   5061,   5049,   5050,   5053,   5052}},
 {  2625,   8, {   235,   4963,   5064,   5063,    227,   4959,   5055,   5054}},
 {  2626,   8, {  4963,   4567,   5065,   5064,   4959,   4552,   5056,   5055}},
 {  2627,   8, {  5063,   5064,   5067,   5066,   5054,   5055,   5058,   5057}},
 {  2628,   8, {  5064,   5065,   5068,   5067,   5055,   5056,   5059,   5058}},
 {  2629,   8, {  5066,   5067,   5070,   5069,   5057,   5058,   5061,   5060}},
 {  2630,   8, {  5067,   5068,   5071,   5070,   5058,   5059,   5062,   5061}},
 {  2631,   8, {   243,   4967,   5073,   5072,    235,   4963,   5064,   5063}},
 {  2632,   8, {  4967,   4582,   5074,   5073,   4963,   4567,   5065,   5064}},
 {  2633,   8, {  5072,   5073,   5076,   5075,   5063,   5064,   5067,   5066}},
 {  2634,   8, {  5073,   5074,   5077,   5076,   5064,   5065,   5068,   5067}},
 {  2635,   8, {  5075,   5076,   5079,   5078,   5066,   5067,   5070,   5069}},
 {  2636,   8, {  5076,   5077,   5080,   5079,   5067,   5068,   5071,   5070}},
 {  2637,   8, {   251,   4971,   5082,   5081,    243,   4967,   5073,   5072}},
 {  2638,   8, {  4971,   4597,   5083,   5082,   4967,   4582,   5074,   5073}},
 {  2639,   8, {  5081,   5082,   5085,   5084,   5072,   5073,   5076,   5075}},
 {  2640,   8, {  5082,   5083,   5086,   5085,   5073,   5074,   5077,   5076}},
 {  2641,   8, {  5084,   5085,   5088,   5087,   5075,   5076,   5079,   5078}},
 {  2642,   8, {  5085,   5086,   5089,   5088,   5076,   5077,   5080,   5079}},
 {  2643,   8, {   259,   4975,   5091,   5090,    251,   4971,   5082,   5081}},
 {  2644,   8, {  4975,   4612,   5092,   5091,   4971,   4597,   5083,   5082}},
 {  2645,   8, {  5090,   5091,   5094,   5093,   5081,   5082,   5085,   5084}},
 {  2646,   8, {  5091,   5092,   5095,   5094,   5082,   5083,   5086,   5085}},
 {  2647,   8, {  5093,   5094,   5097,   5096,   5084,   5085,   5088,   5087}},
 {  2648,   8, {  5094,   5095,   5098,   5097,   5085,   5086,   5089,   5088}},
 {  2649,   8, {   267,   4979,   5100,   5099,    259,   4975,   5091,   5090}},
 {  2650,   8, {  4979,   4627,   5101,   5100,   4975,   4612,   5092,   5091}},
 {  2651,   8, {  5099,   5100,   5103,   5102,   5090,   5091,   5094,   5093}},
 {  2652,   8, {  5100,   5101,   5104,   5103,   5091,   5092,   5095,   5094}},
 {  2653,   8, {  5102,   5103,   5106,   5105,   5093,   5094,   5097,   5096}},
 {  2654,   8, {  5103,   5104,   5107,   5106,   5094,   5095,   5098,   5097}},
 {  2655,   8, {   275,   4983,   5109,   5108,    267,   4979,   5100,   5099}},
 {  2656,   8, {  4983,   4642,   5110,   5109,   4979,   4627,   5101,   5100}},
 {  2657,   8, {  5108,   5109,   5112,   5111,   5099,   5100,   5103,   5102}},
 {  2658,   8, {  5109,   5110,   5113,   5112,   5100,   5101,   5104,   5103}},
 {  2659,   8, {  5111,   5112,   5115,   5114,   5102,   5103,   5106,   5105}},
 {  2660,   8, {  5112,   5113,   5116,   5115,   5103,   5104,   5107,   5106}},
 {  2661,   8, {   283,   4987,   5118,   5117,    275,   4983,   5109,   5108}},
 {  2662,   8, {  4987,   4657,   5119,   5118,   4983,   4642,   5110,   5109}},
 {  2663,   8, {  5117,   5118,   5121,   5120,   5108,   5109,   5112,   5111}},
 {  2664,   8, {  5118,   5119,   5122,   5121,   5109,   5110,   5113,   5112}},
 {  2665,   8, {  5120,   5121,   5124,   5123,   5111,   5112,   5115,   5114}},
 {  2666,   8, {  5121,   5122,   5125,   5124,   5112,   5113,   5116,   5115}},
 {  2667,   8, {   291,   4991,   5127,   5126,    283,   4987,   5118,   5117}},
 {  2668,   8, {  4991,   4672,   5128,   5127,   4987,   4657,   5119,   5118}},
 {  2669,   8, {  5126,   5127,   5130,   5129,   5117,   5118,   5121,   5120}},
 {  2670,   8, {  5127,   5128,   5131,   5130,   5118,   5119,   5122,   5121}},
 {  2671,   8, {  5129,   5130,   5133,   5132,   5120,   5121,   5124,   5123}},
 {  2672,   8, {  5130,   5131,   5134,   5133,   5121,   5122,   5125,   5124}},
 {  2673,   8, {   299,   4995,   5136,   5135,    291,   4991,   5127,   5126}},
 {  2674,   8, {  4995,   4687,   5137,   5136,   4991,   4672,   5128,   5127}},
 {  2675,   8, {  5135,   5136,   5139,   5138,   5126,   5127,   5130,   5129}},
 {  2676,   8, {  5136,   5137,   5140,   5139,   5127,   5128,   5131,   5130}},
 {  2677,   8, {  5138,   5139,   5142,   5141,   5129,   5130,   5133,   5132}},
 {  2678,   8, {  5139,   5140,   5143,   5142,   5130,   5131,   5134,   5133}},
 {  2679,   8, {   307,   4999,   5145,   5144,    299,   4995,   5136,   5135}},
 {  2680,   8, {  4999,   4702,   5146,   5145,   4995,   4687,   5137,   5136}},
 {  2681,   8, {  5144,   5145,   5148,   5147,   5135,   5136,   5139,   5138}},
 {  2682,   8, {  5145,   5146,   5149,   5148,   5136,   5137,   5140,   5139}},
 {  2683,   8, {  5147,   5148,   5151,   5150,   5138,   5139,   5142,   5141}},
 {  2684,   8, {  5148,   5149,   5152,   5151,   5139,   5140,   5143,   5142}},
 {  2685,   8, {  5015,   5016,   5157,   5156,   5006,   5007,   5154,   5153}},
 {  2686,   8, {  5016,   5017,   5158,   5157,   5007,   5008,   5155,   5154}},
 {  2687,   8, {  5024,   5025,   5160,   5159,   5015,   5016,   5157,   5156}},
 {  2688,   8, {  5025,   5026,   5161,   5160,   5016,   5017,   5158,   5157}},
 {  2689,   8, {  5033,   5034,   5163,   5162,   5024,   5025,   5160,   5159}},
 {  2690,   8, {  5034,   5035,   5164,   5163,   5025,   5026,   5161,   5160}},
 {  2691,   8, {  5042,   5043,   5166,   5165,   5033,   5034,   5163,   5162}},
 {  2692,   8, {  5043,   5044,   5167,   5166,   5034,   5035,   5164,   5163}},
 {  2693,   8, {  5051,   5052,   5169,   5168,   5042,   5043,   5166,   5165}},
 {  2694,   8, {  5052,   5053,   5170,   5169,   5043,   5044,   5167,   5166}},
 {  2695,   8, {  5060,   5061,   5172,   5171,   5051,   5052,   5169,   5168}},
 {  2696,   8, {  5061,   5062,   5173,   5172,   5052,   5053,   5170,   5169}},
 {  2697,   8, {  5069,   5070,   5175,   5174,   5060,   5061,   5172,   5171}},
 {  2698,   8, {  5070,   5071,   5176,   5175,   5061,   5062,   5173,   5172}},
 {  2699,   8, {  5078,   5079,   5178,   5177,   5069,   5070,   5175,   5174}},
 {  2700,   8, {  5079,   5080,   5179,   5178,   5070,   5071,   5176,   5175}},
 {  2701,   8, {  5087,   5088,   5181,   5180,   5078,   5079,   5178,   5177}},
 {  2702,   8, {  5088,   5089,   5182,   5181,   5079,   5080,   5179,   5178}},
 {  2703,   8, {  5096,   5097,   5184,   5183,   5087,   5088,   5181,   5180}},
 {  2704,   8, {  5097,   5098,   5185,   5184,   5088,   5089,   5182,   5181}},
 {  2705,   8, {  5105,   5106,   5187,   5186,   5096,   5097,   5184,   5183}},
 {  2706,   8, {  5106,   5107,   5188,   5187,   5097,   5098,   5185,   5184}},
 {  2707,   8, {  5114,   5115,   5190,   5189,   5105,   5106,   5187,   5186}},
 {  2708,   8, {  5115,   5116,   5191,   5190,   5106,   5107,   5188,   5187}},
 {  2709,   8, {  5123,   5124,   5193,   5192,   5114,   5115,   5190,   5189}},
 {  2710,   8, {  5124,   5125,   5194,   5193,   5115,   5116,   5191,   5190}},
 {  2711,   8, {  5132,   5133,   5196,   5195,   5123,   5124,   5193,   5192}},
 {  2712,   8, {  5133,   5134,   5197,   5196,   5124,   5125,   5194,   5193}},
 {  2713,   8, {  5141,   5142,   5199,   5198,   5132,   5133,   5196,   5195}},
 {  2714,   8, {  5142,   5143,   5200,   5199,   5133,   5134,   5197,   5196}},
 {  2715,   8, {  5150,   5151,   5202,   5201,   5141,   5142,   5199,   5198}},
 {  2716,   8, {  5151,   5152,   5203,   5202,   5142,   5143,   5200,   5199}},
 {  2717,   8, {  5204,   5205,   5208,   5207,   5150,   5151,   5202,   5201}},
 {  2718,   8, {  5205,   5206,   5209,   5208,   5151,   5152,   5203,   5202}},
 {  2719,   9, {  5212,   5033,   5162,   5213,   5210,   5024,   5159,   5211}},
 {  2720,   9, {  5214,   5042,   5165,   5215,   5212,   5033,   5162,   5213}},
 {  2721,   9, {  5216,   5051,   5168,   5217,   5214,   5042,   5165,   5215}},
 {  2722,   9, {  5218,   5060,   5171,   5219,   5216,   5051,   5168,   5217}},
 {  2723,   9, {  5220,   5069,   5174,   5221,   5218,   5060,   5171,   5219}},
 {  2724,   9, {  5222,   5078,   5177,   5223,   5220,   5069,   5174,   5221}},
 {  2725,   9, {  5224,   5087,   5180,   5225,   5222,   5078,   5177,   5223}},
 {  2726,   9, {  5226,   5096,   5183,   5227,   5224,   5087,   5180,   5225}},
 {  2727,   9, {  5228,   5105,   5186,   5229,   5226,   5096,   5183,   5227}},
 {  2728,   9, {  5230,   5114,   5189,   5231,   5228,   5105,   5186,   5229}},
 {  2729,   9, {  5232,   5123,   5192,   5233,   5230,   5114,   5189,   5231}},
 {  2730,   9, {  5234,   5132,   5195,   5235,   5232,   5123,   5192,   5233}},
 {  2731,   9, {  5236,   5141,   5198,   5237,   5234,   5132,   5195,   5235}},
 {  2732,   9, {  5238,   5150,   5201,   5239,   5236,   5141,   5198,   5237}},
 {  2733,  10, {  5240,   4720,   4717,   5241,   5242,   4726,   4723,   5243}},
 {  2734,  10, {  5242,   4726,   4723,   5243,   5244,   4732,   4729,   5245}},
 {  2735,  10, {  5244,   4732,   4729,   5245,   5246,   4738,   4735,   5247}},
 {  2736,  10, {  5246,   4738,   4735,   5247,   5248,   4744,   4741,   5249}},
 {  2737,  10, {  5248,   4744,   4741,   5249,   5250,   4750,   4747,   5251}},
 {  2738,  10, {  5250,   4750,   4747,   5251,   5252,   4756,   4753,   5253}},
 {  2739,  10, {  5252,   4756,   4753,   5253,   5254,   4762,   4759,   5255}},
 {  2740,  10, {  5254,   4762,   4759,   5255,   5256,   4768,   4765,   5257}},
 {  2741,  10, {  5256,   4768,   4765,   5257,   5258,   4774,   4771,   5259}},
 {  2742,  10, {  5258,   4774,   4771,   5259,   5260,   4780,   4777,   5261}},
 {  2743,  10, {  5260,   4780,   4777,   5261,   5262,   4786,   4783,   5263}},
 {  2744,  10, {  5262,   4786,   4783,   5263,   5264,   4792,   4789,   5265}},
 {  2745,  10, {  5264,   4792,   4789,   5265,   5266,   4798,   4795,   5267}},
 {  2746,  10, {  5266,   4798,   4795,   5267,   5268,   4804,   4801,   5269}},
 {  2747,  11, {  5270,   5271,   5280,   5279,   5288,   5289,   5297,   2906}},
 {  2748,  11, {  5271,   5272,   5281,   5280,   5289,   5290,   2907,   5297}},
 {  2749,  11, {  5272,   5273,   5282,   5281,   5290,   5291,   5298,   2907}},
 {  2750,  11, {  5273,   5274,   5283,   5282,   5291,   5292,   2966,   5298}},
 {  2751,  11, {  5274,   5275,   5284,   5283,   5292,   5293,   5299,   2966}},
 {  2752,  11, {  5275,   5276,   5285,   5284,   5293,   5294,   2967,   5299}},
 {  2753,  11, {  5276,   5277,   5286,   5285,   5294,   5295,   5300,   2967}},
 {  2754,  11, {  5277,   5278,   5287,   5286,   5295,   5296,   3026,   5300}},
 {  2755,  11, {  5288,   5289,   5297,   2906,   5301,   5302,   5310,   2910}},
 {  2756,  11, {  5289,   5290,   2907,   5297,   5302,   5303,   2911,   5310}},
 {  2757,  11, {  5290,   5291,   5298,   2907,   5303,   5304,   5311,   2911}},
 {  2758,  11, {  5291,   5292,   2966,   5298,   5304,   5305,   2970,   5311}},
 {  2759,  11, {  5292,   5293,   5299,   2966,   5305,   5306,   5312,   2970}},
 {  2760,  11, {  5293,   5294,   2967,   5299,   5306,   5307,   2971,   5312}},
 {  2761,  11, {  5294,   5295,   5300,   2967,   5307,   5308,   5313,   2971}},
 {  2762,  11, {  5295,   5296,   3026,   5300,   5308,   5309,   3028,   5313}},
 {  2763,  11, {  5301,   5302,   5310,   2910,   5314,   5315,   5323,   2914}},
 {  2764,  11, {  5302,   5303,   2911,   5310,   5315,   5316,   2915,   5323}},
 {  2765,  11, {  5303,   5304,   5311,   2911,   5316,   5317,   5324,   2915}},
 {  2766,  11, {  5304,   5305,   2970,   5311,   5317,   5318,   2974,   5324}},
 {  2767,  11, {  5305,   5306,   5312,   2970,   5318,   5319,   5325,   2974}},
 {  2768,  11, {  5306,   5307,   2971,   5312,   5319,   5320,   2975,   5325}},
 {  2769,  11, {  5307,   5308,   5313,   2971,   5320,   5321,   5326,   2975}},
 {  2770,  11, {  5308,   5309,   3028,   5313,   5321,   5322,   3030,   5326}},
 {  2771,  11, {  5314,   5315,   5323,   2914,   5327,   5328,   5336,   2918}},
 {  2772,  11, {  5315,   5316,   2915,   5323,   5328,   5329,   2919,   5336}},
 {  2773,  11, {  5316,   5317,   5324,   2915,   5329,   5330,   5337,   2919}},
 {  2774,  11, {  5317,   5318,   2974,   5324,   5330,   5331,   2978,   5337}},
 {  2775,  11, {  5318,   5319,   5325,   2974,   5331,   5332,   5338,   2978}},
 {  2776,  11, {  5319,   5320,   2975,   5325,   5332,   5333,   2979,   5338}},
 {  2777,  11, {  5320,   5321,   5326,   2975,   5333,   5334,   5339,   2979}},
 {  2778,  11, {  5321,   5322,   3030,   5326,   5334,   5335,   3032,   5339}},
 {  2779,  11, {  5327,   5328,   5336,   2918,   5340,   5341,   5349,   2922}},
 {  2780,  11, {  5328,   5329,   2919,   5336,   5341,   5342,   2923,   5349}},
 {  2781,  11, {  5329,   5330,   5337,   2919,   5342,   5343,   5350,   2923}},
 {  2782,  11, {  5330,   5331,   2978,   5337,   5343,   5344,   2982,   5350}},
 {  2783,  11, {  5331,   5332,   5338,   2978,   5344,   5345,   5351,   2982}},
 {  2784,  11, {  5332,   5333,   2979,   5338,   5345,   5346,   2983,   5351}},
 {  2785,  11, {  5333,   5334,   5339,   2979,   5346,   5347,   5352,   2983}},
 {  2786,  11, {  5334,   5335,   3032,   5339,   5347,   5348,   3034,   5352}},
 {  2787,  11, {  5340,   5341,   5349,   2922,   5353,   5354,   5362,   2926}},
 {  2788,  11, {  5341,   5342,   2923,   5349,   5354,   5355,   2927,   5362}},
 {  2789,  11, {  5342,   5343,   5350,   2923,   5355,   5356,   5363,   2927}},
 {  2790,  11, {  5343,   5344,   2982,   5350,   5356,   5357,   2986,   5363}},
 {  2791,  11, {  5344,   5345,   5351,   2982,   5357,   5358,   5364,   2986}},
 {  2792,  11, {  5345,   5346,   2983,   5351,   5358,   5359,   2987,   5364}},
 {  2793,  11, {  5346,   5347,   5352,   2983,   5359,   5360,   5365,   2987}},
 {  2794,  11, {  5347,   5348,   3034,   5352,   5360,   5361,   3036,   5365}},
 {  2795,  11, {  5353,   5354,   5362,   2926,   5366,   5367,   5375,   2930}},
 {  2796,  11, {  5354,   5355,   2927,   5362,   5367,   5368,   2931,   5375}},
 {  2797,  11, {  5355,   5356,   5363,   2927,   5368,   5369,   5376,   2931}},
 {  2798,  11, {  5356,   5357,   2986,   5363,   5369,   5370,   2990,   5376}},
 {  2799,  11, {  5357,   5358,   5364,   2986,   5370,   5371,   5377,   2990}},
 {  2800,  11, {  5358,   5359,   2987,   5364,   5371,   5372,   2991,   5377}},
 {  2801,  11, {  5359,   5360,   5365,   2987,   5372,   5373,   5378,   2991}},
 {  2802,  11, {  5360,   5361,   3036,   5365,   5373,   5374,   3038,   5378}},
 {  2803,  11, {  5366,   5367,   5375,   2930,   5379,   5380,   5388,   2934}},
 {  2804,  11, {  5367,   5368,   2931,   5375,   5380,   5381,   2935,   5388}},
 {  2805,  11, {  5368,   5369,   5376,   2931,   5381,   5382,   5389,   2935}},
 {  2806,  11, {  5369,   5370,   2990,   5376,   5382,   5383,   2994,   5389}},
 {  2807,  11, {  5370,   5371,   5377,   2990,   5383,   5384,   5390,   2994}},
 {  2808,  11, {  5371,   5372,   2991,   5377,   5384,   5385,   2995,   5390}},
 {  2809,  11, {  5372,   5373,   5378,   2991,   5385,   5386,   5391,   2995}},
 {  2810,  11, {  5373,   5374,   3038,   5378,   5386,   5387,   3040,   5391}},
 {  2811,  11, {  5379,   5380,   5388,   2934,   5392,   5393,   5401,   2938}},
 {  2812,  11, {  5380,   5381,   2935,   5388,   5393,   5394,   2939,   5401}},
 {  2813,  11, {  5381,   5382,   5389,   2935,   5394,   5395,   5402,   2939}},
 {  2814,  11, {  5382,   5383,   2994,   5389,   5395,   5396,   2998,   5402}},
 {  2815,  11, {  5383,   5384,   5390,   2994,   5396,   5397,   5403,   2998}},
 {  2816,  11, {  5384,   5385,   2995,   5390,   5397,   5398,   2999,   5403}},
 {  2817,  11, {  5385,   5386,   5391,   2995,   5398,   5399,   5404,   2999}},
 {  2818,  11, {  5386,   5387,   3040,   5391,   5399,   5400,   3042,   5404}},
 {  2819,  11, {  5392,   5393,   5401,   2938,   5405,   5406,   5414,   2942}},
 {  2820,  11, {  5393,   5394,   2939,   5401,   5406,   5407,   2943,   5414}},
 {  2821,  11, {  5394,   5395,   5402,   2939,   5407,   5408,   5415,   2943}},
 {  2822,  11, {  5395,   5396,   2998,   5402,   5408,   5409,   3002,   5415}},
 {  2823,  11, {  5396,   5397,   5403,   2998,   5409,   5410,   5416,   3002}},
 {  2824,  11, {  5397,   5398,   2999,   5403,   5410,   5411,   3003,   5416}},
 {  2825,  11, {  5398,   5399,   5404,   2999,   5411,   5412,   5417,   3003}},
 {  2826,  11, {  5399,   5400,   3042,   5404,   5412,   5413,   3044,   5417}},
 {  2827,  11, {  5405,   5406,   5414,   2942,   5418,   5419,   5427,   2946}},
 {  2828,  11, {  5406,   5407,   2943,   5414,   5419,   5420,   2947,   5427}},
 {  2829,  11, {  5407,   5408,   5415,   2943,   5420,   5421,   5428,   2947}},
 {  2830,  11, {  5408,   5409,   3002,   5415,   5421,   5422,   3006,   5428}},
 {  2831,  11, {  5409,   5410,   5416,   3002,   5422,   5423,   5429,   3006}},
 {  2832,  11, {  5410,   5411,   3003,   5416,   5423,   5424,   3007,   5429}},
 {  2833,  11, {  5411,   5412,   5417,   3003,   5424,   5425,   5430,   3007}},
 {  2834,  11, {  5412,   5413,   3044,   5417,   5425,   5426,   3046,   5430}},
 {  2835,  11, {  5418,   5419,   5427,   2946,   5431,   5432,   5440,   2950}},
 {  2836,  11, {  5419,   5420,   2947,   5427,   5432,   5433,   2951,   5440}},
 {  2837,  11, {  5420,   5421,   5428,   2947,   5433,   5434,   5441,   2951}},
 {  2838,  11, {  5421,   5422,   3006,   5428,   5434,   5435,   3010,   5441}},
 {  2839,  11, {  5422,   5423,   5429,   3006,   5435,   5436,   5442,   3010}},
 {  2840,  11, {  5423,   5424,   3007,   5429,   5436,   5437,   3011,   5442}},
 {  2841,  11, {  5424,   5425,   5430,   3007,   5437,   5438,   5443,   3011}},
 {  2842,  11, {  5425,   5426,   3046,   5430,   5438,   5439,   3048,   5443}},
 {  2843,  11, {  5431,   5432,   5440,   2950,   5444,   5445,   5453,   2954}},
 {  2844,  11, {  5432,   5433,   2951,   5440,   5445,   5446,   2955,   5453}},
 {  2845,  11, {  5433,   5434,   5441,   2951,   5446,   5447,   5454,   2955}},
 {  2846,  11, {  5434,   5435,   3010,   5441,   5447,   5448,   3014,   5454}},
 {  2847,  11, {  5435,   5436,   5442,   3010,   5448,   5449,   5455,   3014}},
 {  2848,  11, {  5436,   5437,   3011,   5442,   5449,   5450,   3015,   5455}},
 {  2849,  11, {  5437,   5438,   5443,   3011,   5450,   5451,   5456,   3015}},
 {  2850,  11, {  5438,   5439,   3048,   5443,   5451,   5452,   3050,   5456}},
 {  2851,  11, {  5444,   5445,   5453,   2954,   5457,   5458,   5466,   2958}},
 {  2852,  11, {  5445,   5446,   2955,   5453,   5458,   5459,   2959,   5466}},
 {  2853,  11, {  5446,   5447,   5454,   2955,   5459,   5460,   5467,   2959}},
 {  2854,  11, {  5447,   5448,   3014,   5454,   5460,   5461,   3018,   5467}},
 {  2855,  11, {  5448,   5449,   5455,   3014,   5461,   5462,   5468,   3018}},
 {  2856,  11, {  5449,   5450,   3015,   5455,   5462,   5463,   3019,   5468}},
 {  2857,  11, {  5450,   5451,   5456,   3015,   5463,   5464,   5469,   3019}},
 {  2858,  11, {  5451,   5452,   3050,   5456,   5464,   5465,   3052,   5469}},
 {  2859,  11, {  5457,   5458,   5466,   2958,   5470,   5471,   5479,   2962}},
 {  2860,  11, {  5458,   5459,   2959,   5466,   5471,   5472,   2963,   5479}},
 {  2861,  11, {  5459,   5460,   5467,   2959,   5472,   5473,   5480,   2963}},
 {  2862,  11, {  5460,   5461,   3018,   5467,   5473,   5474,   3022,   5480}},
 {  2863,  11, {  5461,   5462,   5468,   3018,   5474,   5475,   5481,   3022}},
 {  2864,  11, {  5462,   5463,   3019,   5468,   5475,   5476,   3023,   5481}},
 {  2865,  11, {  5463,   5464,   5469,   3019,   5476,   5477,   5482,   3023}},
 {  2866,  11, {  5464,   5465,   3052,   5469,   5477,   5478,   3054,   5482}},
 {  2867,  11, {  5470,   5471,   5479,   2962,   5483,   5484,   5493,   5492}},
 {  2868,  11, {  5471,   5472,   2963,   5479,   5484,   5485,   5494,   5493}},
 {  2869,  11, {  5472,   5473,   5480,   2963,   5485,   5486,   5495,   5494}},
 {  2870,  11, {  5473,   5474,   3022,   5480,   5486,   5487,   5496,   5495}},
 {  2871,  11, {  5474,   5475,   5481,   3022,   5487,   5488,   5497,   5496}},
 {  2872,  11, {  5475,   5476,   3023,   5481,   5488,   5489,   5498,   5497}},
 {  2873,  11, {  5476,   5477,   5482,   3023,   5489,   5490,   5499,   5498}},
 {  2874,  11, {  5477,   5478,   3054,   5482,   5490,   5491,   5500,   5499}},
 {  2875,  11, {  5501,   5502,   5316,   5314,   5504,   5505,   5329,   5327}},
 {  2876,  11, {  5502,   5503,   5318,   5316,   5505,   5506,   5331,   5329}},
 {  2877,  11, {  5504,   5505,   5329,   5327,   5507,   5508,   5342,   5340}},
 {  2878,  11, {  5505,   5506,   5331,   5329,   5508,   5509,   5344,   5342}},
 {  2879,  11, {  5507,   5508,   5342,   5340,   5510,   5511,   5355,   5353}},
 {  2880,  11, {  5508,   5509,   5344,   5342,   5511,   5512,   5357,   5355}},
 {  2881,  11, {  5510,   5511,   5355,   5353,   5513,   5514,   5368,   5366}},
 {  2882,  11, {  5511,   5512,   5357,   5355,   5514,   5515,   5370,   5368}},
 {  2883,  11, {  5513,   5514,   5368,   5366,   5516,   5517,   5381,   5379}},
 {  2884,  11, {  5514,   5515,   5370,   5368,   5517,   5518,   5383,   5381}},
 {  2885,  11, {  5516,   5517,   5381,   5379,   5519,   5520,   5394,   5392}},
 {  2886,  11, {  5517,   5518,   5383,   5381,   5520,   5521,   5396,   5394}},
 {  2887,  11, {  5519,   5520,   5394,   5392,   5522,   5523,   5407,   5405}},
 {  2888,  11, {  5520,   5521,   5396,   5394,   5523,   5524,   5409,   5407}},
 {  2889,  11, {  5522,   5523,   5407,   5405,   5525,   5526,   5420,   5418}},
 {  2890,  11, {  5523,   5524,   5409,   5407,   5526,   5527,   5422,   5420}},
 {  2891,  11, {  5525,   5526,   5420,   5418,   5528,   5529,   5433,   5431}},
 {  2892,  11, {  5526,   5527,   5422,   5420,   5529,   5530,   5435,   5433}},
 {  2893,  11, {  5528,   5529,   5433,   5431,   5531,   5532,   5446,   5444}},
 {  2894,  11, {  5529,   5530,   5435,   5433,   5532,   5533,   5448,   5446}},
 {  2895,  11, {  5503,   5534,   5320,   5318,   5506,   5536,   5333,   5331}},
 {  2896,  11, {  5534,   5535,   5322,   5320,   5536,   5537,   5335,   5333}},
 {  2897,  11, {  5506,   5536,   5333,   5331,   5509,   5538,   5346,   5344}},
 {  2898,  11, {  5536,   5537,   5335,   5333,   5538,   5539,   5348,   5346}},
 {  2899,  11, {  5509,   5538,   5346,   5344,   5512,   5540,   5359,   5357}},
 {  2900,  11, {  5538,   5539,   5348,   5346,   5540,   5541,   5361,   5359}},
 {  2901,  11, {  5524,   5542,   5411,   5409,   5527,   5544,   5424,   5422}},
 {  2902,  11, {  5542,   5543,   5413,   5411,   5544,   5545,   5426,   5424}},
 {  2903,  11, {  5527,   5544,   5424,   5422,   5530,   5546,   5437,   5435}},
 {  2904,  11, {  5544,   5545,   5426,   5424,   5546,   5547,   5439,   5437}},
 {  2905,  11, {  5530,   5546,   5437,   5435,   5533,   5548,   5450,   5448}},
 {  2906,  11, {  5546,   5547,   5439,   5437,   5548,   5549,   5452,   5450}},
 {  2907,  12, {  5568,   5569,   5577,   3176,   5550,   5551,   5560,   5559}},
 {  2908,  12, {  5569,   5570,   3117,   5577,   5551,   5552,   5561,   5560}},
 {  2909,  12, {  5570,   5571,   5578,   3117,   5552,   5553,   5562,   5561}},
 {  2910,  12, {  5571,   5572,   3116,   5578,   5553,   5554,   5563,   5562}},
 {  2911,  12, {  5572,   5573,   5579,   3116,   5554,   5555,   5564,   5563}},
 {  2912,  12, {  5573,   5574,   3057,   5579,   5555,   5556,   5565,   5564}},
 {  2913,  12, {  5574,   5575,   5580,   3057,   5556,   5557,   5566,   5565}},
 {  2914,  12, {  5575,   5576,   3056,   5580,   5557,   5558,   5567,   5566}},
 {  2915,  12, {  5581,   5582,   5590,   3178,   5568,   5569,   5577,   3176}},
 {  2916,  12, {  5582,   5583,   3121,   5590,   5569,   5570,   3117,   5577}},
 {  2917,  12, {  5583,   5584,   5591,   3121,   5570,   5571,   5578,   3117}},
 {  2918,  12, {  5584,   5585,   3120,   5591,   5571,   5572,   3116,   5578}},
 {  2919,  12, {  5585,   5586,   5592,   3120,   5572,   5573,   5579,   3116}},
 {  2920,  12, {  5586,   5587,   3061,   5592,   5573,   5574,   3057,   5579}},
 {  2921,  12, {  5587,   5588,   5593,   3061,   5574,   5575,   5580,   3057}},
 {  2922,  12, {  5588,   5589,   3060,   5593,   5575,   5576,   3056,   5580}},
 {  2923,  12, {  5594,   5595,   5603,   3180,   5581,   5582,   5590,   3178}},
 {  2924,  12, {  5595,   5596,   3125,   5603,   5582,   5583,   3121,   5590}},
 {  2925,  12, {  5596,   5597,   5604,   3125,   5583,   5584,   5591,   3121}},
 {  2926,  12, {  5597,   5598,   3124,   5604,   5584,   5585,   3120,   5591}},
 {  2927,  12, {  5598,   5599,   5605,   3124,   5585,   5586,   5592,   3120}},
 {  2928,  12, {  5599,   5600,   3065,   5605,   5586,   5587,   3061,   5592}},
 {  2929,  12, {  5600,   5601,   5606,   3065,   5587,   5588,   5593,   3061}},
 {  2930,  12, {  5601,   5602,   3064,   5606,   5588,   5589,   3060,   5593}},
 {  2931,  12, {  5607,   5608,   5616,   3182,   5594,   5595,   5603,   3180}},
 {  2932,  12, {  5608,   5609,   3129,   5616,   5595,   5596,   3125,   5603}},
 {  2933,  12, {  5609,   5610,   5617,   3129,   5596,   5597,   5604,   3125}},
 {  2934,  12, {  5610,   5611,   3128,   5617,   5597,   5598,   3124,   5604}},
 {  2935,  12, {  5611,   5612,   5618,   3128,   5598,   5599,   5605,   3124}},
 {  2936,  12, {  5612,   5613,   3069,   5618,   5599,   5600,   3065,   5605}},
 {  2937,  12, {  5613,   5614,   5619,   3069,   5600,   5601,   5606,   3065}},
 {  2938,  12, {  5614,   5615,   3068,   5619,   5601,   5602,   3064,   5606}},
 {  2939,  12, {  5620,   5621,   5629,   3184,   5607,   5608,   5616,   3182}},
 {  2940,  12, {  5621,   5622,   3133,   5629,   5608,   5609,   3129,   5616}},
 {  2941,  12, {  5622,   5623,   5630,   3133,   5609,   5610,   5617,   3129}},
 {  2942,  12, {  5623,   5624,   3132,   5630,   5610,   5611,   3128,   5617}},
 {  2943,  12, {  5624,   5625,   5631,   3132,   5611,   5612,   5618,   3128}},
 {  2944,  12, {  5625,   5626,   3073,   5631,   5612,   5613,   3069,   5618}},
 {  2945,  12, {  5626,   5627,   5632,   3073,   5613,   5614,   5619,   3069}},
 {  2946,  12, {  5627,   5628,   3072,   5632,   5614,   5615,   3068,   5619}},
 {  2947,  12, {  5633,   5634,   5642,   3186,   5620,   5621,   5629,   3184}},
 {  2948,  12, {  5634,   5635,   3137,   5642,   5621,   5622,   3133,   5629}},
 {  2949,  12, {  5635,   5636,   5643,   3137,   5622,   5623,   5630,   3133}},
 {  2950,  12, {  5636,   5637,   3136,   5643,   5623,   5624,   3132,   5630}},
 {  2951,  12, {  5637,   5638,   5644,   3136,   5624,   5625,   5631,   3132}},
 {  2952,  12, {  5638,   5639,   3077,   5644,   5625,   5626,   3073,   5631}},
 {  2953,  12, {  5639,   5640,   5645,   3077,   5626,   5627,   5632,   3073}},
 {  2954,  12, {  5640,   5641,   3076,   5645,   5627,   5628,   3072,   5632}},
 {  2955,  12, {  5646,   5647,   5655,   3188,   5633,   5634,   5642,   3186}},
 {  2956,  12, {  5647,   5648,   3141,   5655,   5634,   5635,   3137,   5642}},
 {  2957,  12, {  5648,   5649,   5656,   3141,   5635,   5636,   5643,   3137}},
 {  2958,  12, {  5649,   5650,   3140,   5656,   5636,   5637,   3136,   5643}},
 {  2959,  12, {  5650,   5651,   5657,   3140,   5637,   5638,   5644,   3136}},
 {  2960,  12, {  5651,   5652,   3081,   5657,   5638,   5639,   3077,   5644}},
 {  2961,  12, {  5652,   5653,   5658,   3081,   5639,   5640,   5645,   3077}},
 {  2962,  12, {  5653,   5654,   3080,   5658,   5640,   5641,   3076,   5645}},
 {  2963,  12, {  5659,   5660,   5668,   3190,   5646,   5647,   5655,   3188}},
 {  2964,  12, {  5660,   5661,   3145,   5668,   5647,   5648,   3141,   5655}},
 {  2965,  12, {  5661,   5662,   5669,   3145,   5648,   5649,   5656,   3141}},
 {  2966,  12, {  5662,   5663,   3144,   5669,   5649,   5650,   3140,   5656}},
 {  2967,  12, {  5663,   5664,   5670,   3144,   5650,   5651,   5657,   3140}},
 {  2968,  12, {  5664,   5665,   3085,   5670,   5651,   5652,   3081,   5657}},
 {  2969,  12, {  5665,   5666,   5671,   3085,   5652,   5653,   5658,   3081}},
 {  2970,  12, {  5666,   5667,   3084,   5671,   5653,   5654,   3080,   5658}},
 {  2971,  12, {  5672,   5673,   5681,   3192,   5659,   5660,   5668,   3190}},
 {  2972,  12, {  5673,   5674,   3149,   5681,   5660,   5661,   3145,   5668}},
 {  2973,  12, {  5674,   5675,   5682,   3149,   5661,   5662,   5669,   3145}},
 {  2974,  12, {  5675,   5676,   3148,   5682,   5662,   5663,   3144,   5669}},
 {  2975,  12, {  5676,   5677,   5683,   3148,   5663,   5664,   5670,   3144}},
 {  2976,  12, {  5677,   5678,   3089,   5683,   5664,   5665,   3085,   5670}},
 {  2977,  12, {  5678,   5679,   5684,   3089,   5665,   5666,   5671,   3085}},
 {  2978,  12, {  5679,   5680,   3088,   5684,   5666,   5667,   3084,   5671}},
 {  2979,  12, {  5685,   5686,   5694,   3194,   5672,   5673,   5681,   3192}},
 {  2980,  12, {  5686,   5687,   3153,   5694,   5673,   5674,   3149,   5681}},
 {  2981,  12, {  5687,   5688,   5695,   3153,   5674,   5675,   5682,   3149}},
 {  2982,  12, {  5688,   5689,   3152,   5695,   5675,   5676,   3148,   5682}},
 {  2983,  12, {  5689,   5690,   5696,   3152,   5676,   5677,   5683,   3148}},
 {  2984,  12, {  5690,   5691,   3093,   5696,   5677,   5678,   3089,   5683}},
 {  2985,  12, {  5691,   5692,   5697,   3093,   5678,   5679,   5684,   3089}},
 {  2986,  12, {  5692,   5693,   3092,   5697,   5679,   5680,   3088,   5684}},
 {  2987,  12, {  5698,   5699,   5707,   3196,   5685,   5686,   5694,   3194}},
 {  2988,  12, {  5699,   5700,   3157,   5707,   5686,   5687,   3153,   5694}},
 {  2989,  12, {  5700,   5701,   5708,   3157,   5687,   5688,   5695,   3153}},
 {  2990,  12, {  5701,   5702,   3156,   5708,   5688,   5689,   3152,   5695}},
 {  2991,  12, {  5702,   5703,   5709,   3156,   5689,   5690,   5696,   3152}},
 {  2992,  12, {  5703,   5704,   3097,   5709,   5690,   5691,   3093,   5696}},
 {  2993,  12, {  5704,   5705,   5710,   3097,   5691,   5692,   5697,   3093}},
 {  2994,  12, {  5705,   5706,   3096,   5710,   5692,   5693,   3092,   5697}},
 {  2995,  12, {  5711,   5712,   5720,   3198,   5698,   5699,   5707,   3196}},
 {  2996,  12, {  5712,   5713,   3161,   5720,   5699,   5700,   3157,   5707}},
 {  2997,  12, {  5713,   5714,   5721,   3161,   5700,   5701,   5708,   3157}},
 {  2998,  12, {  5714,   5715,   3160,   5721,   5701,   5702,   3156,   5708}},
 {  2999,  12, {  5715,   5716,   5722,   3160,   5702,   5703,   5709,   3156}},
 {  3000,  12, {  5716,   5717,   3101,   5722,   5703,   5704,   3097,   5709}},
 {  3001,  12, {  5717,   5718,   5723,   3101,   5704,   5705,   5710,   3097}},
 {  3002,  12, {  5718,   5719,   3100,   5723,   5705,   5706,   3096,   5710}},
 {  3003,  12, {  5724,   5725,   5733,   3200,   5711,   5712,   5720,   3198}},
 {  3004,  12, {  5725,   5726,   3165,   5733,   5712,   5713,   3161,   5720}},
 {  3005,  12, {  5726,   5727,   5734,   3165,   5713,   5714,   5721,   3161}},
 {  3006,  12, {  5727,   5728,   3164,   5734,   5714,   5715,   3160,   5721}},
 {  3007,  12, {  5728,   5729,   5735,   3164,   5715,   5716,   5722,   3160}},
 {  3008,  12, {  5729,   5730,   3105,   5735,   5716,   5717,   3101,   5722}},
 {  3009,  12, {  5730,   5731,   5736,   3105,   5717,   5718,   5723,   3101}},
 {  3010,  12, {  5731,   5732,   3104,   5736,   5718,   5719,   3100,   5723}},
 {  3011,  12, {  5737,   5738,   5746,   3202,   5724,   5725,   5733,   3200}},
 {  3012,  12, {  5738,   5739,   3169,   5746,   5725,   5726,   3165,   5733}},
 {  3013,  12, {  5739,   5740,   5747,   3169,   5726,   5727,   5734,   3165}},
 {  3014,  12, {  5740,   5741,   3168,   5747,   5727,   5728,   3164,   5734}},
 {  3015,  12, {  5741,   5742,   5748,   3168,   5728,   5729,   5735,   3164}},
 {  3016,  12, {  5742,   5743,   3109,   5748,   5729,   5730,   3105,   5735}},
 {  3017,  12, {  5743,   5744,   5749,   3109,   5730,   5731,   5736,   3105}},
 {  3018,  12, {  5744,   5745,   3108,   5749,   5731,   5732,   3104,   5736}},
 {  3019,  12, {  5750,   5751,   5759,   3204,   5737,   5738,   5746,   3202}},
 {  3020,  12, {  5751,   5752,   3173,   5759,   5738,   5739,   3169,   5746}},
 {  3021,  12, {  5752,   5753,   5760,   3173,   5739,   5740,   5747,   3169}},
 {  3022,  12, {  5753,   5754,   3172,   5760,   5740,   5741,   3168,   5747}},
 {  3023,  12, {  5754,   5755,   5761,   3172,   5741,   5742,   5748,   3168}},
 {  3024,  12, {  5755,   5756,   3113,   5761,   5742,   5743,   3109,   5748}},
 {  3025,  12, {  5756,   5757,   5762,   3113,   5743,   5744,   5749,   3109}},
 {  3026,  12, {  5757,   5758,   3112,   5762,   5744,   5745,   3108,   5749}},
 {  3027,  12, {  5763,   5764,   5773,   5772,   5750,   5751,   5759,   3204}},
 {  3028,  12, {  5764,   5765,   5774,   5773,   5751,   5752,   3173,   5759}},
 {  3029,  12, {  5765,   5766,   5775,   5774,   5752,   5753,   5760,   3173}},
 {  3030,  12, {  5766,   5767,   5776,   5775,   5753,   5754,   3172,   5760}},
 {  3031,  12, {  5767,   5768,   5777,   5776,   5754,   5755,   5761,   3172}},
 {  3032,  12, {  5768,   5769,   5778,   5777,   5755,   5756,   3113,   5761}},
 {  3033,  12, {  5769,   5770,   5779,   5778,   5756,   5757,   5762,   3113}},
 {  3034,  12, {  5770,   5771,   5780,   5779,   5757,   5758,   3112,   5762}},
 {  3035,  12, {  5784,   5785,   5609,   5607,   5781,   5782,   5596,   5594}},
 {  3036,  12, {  5785,   5786,   5611,   5609,   5782,   5783,   5598,   5596}},
 {  3037,  12, {  5787,   5788,   5622,   5620,   5784,   5785,   5609,   5607}},
 {  3038,  12, {  5788,   5789,   5624,   5622,   5785,   5786,   5611,   5609}},
 {  3039,  12, {  5790,   5791,   5635,   5633,   5787,   5788,   5622,   5620}},
 {  3040,  12, {  5791,   5792,   5637,   5635,   5788,   5789,   5624,   5622}},
 {  3041,  12, {  5793,   5794,   5648,   5646,   5790,   5791,   5635,   5633}},
 {  3042,  12, {  5794,   5795,   5650,   5648,   5791,   5792,   5637,   5635}},
 {  3043,  12, {  5796,   5797,   5661,   5659,   5793,   5794,   5648,   5646}},
 {  3044,  12, {  5797,   5798,   5663,   5661,   5794,   5795,   5650,   5648}},
 {  3045,  12, {  5799,   5800,   5674,   5672,   5796,   5797,   5661,   5659}},
 {  3046,  12, {  5800,   5801,   5676,   5674,   5797,   5798,   5663,   5661}},
 {  3047,  12, {  5802,   5803,   5687,   5685,   5799,   5800,   5674,   5672}},
 {  3048,  12, {  5803,   5804,   5689,   5687,   5800,   5801,   5676,   5674}},
 {  3049,  12, {  5805,   5806,   5700,   5698,   5802,   5803,   5687,   5685}},
 {  3050,  12, {  5806,   5807,   5702,   5700,   5803,   5804,   5689,   5687}},
 {  3051,  12, {  5808,   5809,   5713,   5711,   5805,   5806,   5700,   5698}},
 {  3052,  12, {  5809,   5810,   5715,   5713,   5806,   5807,   5702,   5700}},
 {  3053,  12, {  5811,   5812,   5726,   5724,   5808,   5809,   5713,   5711}},
 {  3054,  12, {  5812,   5813,   5728,   5726,   5809,   5810,   5715,   5713}},
 {  3055,  12, {  5786,   5816,   5613,   5611,   5783,   5814,   5600,   5598}},
 {  3056,  12, {  5816,   5817,   5615,   5613,   5814,   5815,   5602,   5600}},
 {  3057,  12, {  5789,   5818,   5626,   5624,   5786,   5816,   5613,   5611}},
 {  3058,  12, {  5818,   5819,   5628,   5626,   5816,   5817,   5615,   5613}},
 {  3059,  12, {  5792,   5820,   5639,   5637,   5789,   5818,   5626,   5624}},
 {  3060,  12, {  5820,   5821,   5641,   5639,   5818,   5819,   5628,   5626}},
 {  3061,  12, {  5807,   5824,   5704,   5702,   5804,   5822,   5691,   5689}},
 {  3062,  12, {  5824,   5825,   5706,   5704,   5822,   5823,   5693,   5691}},
 {  3063,  12, {  5810,   5826,   5717,   5715,   5807,   5824,   5704,   5702}},
 {  3064,  12, {  5826,   5827,   5719,   5717,   5824,   5825,   5706,   5704}},
 {  3065,  12, {  5813,   5828,   5730,   5728,   5810,   5826,   5717,   5715}},
 {  3066,  12, {  5828,   5829,   5732,   5730,   5826,   5827,   5719,   5717}},
 {  3067,  12, {  5830,   5831,   5833,   5832,   5834,   5835,   5837,   5836}},
 {  3068,  12, {  5834,   5835,   5837,   5836,   5838,   5839,   5841,   5840}},
 {  3069,  12, {  5838,   5839,   5841,   5840,   5842,   5843,   5845,   5844}},
 {  3070,  12, {  5842,   5843,   5845,   5844,   5846,   5847,   5849,   5848}},
 {  3071,  12, {  5832,   5833,   5851,   5850,   5836,   5837,   5855,   5854}},
 {  3072,  12, {  5850,   5851,   5853,   5852,   5854,   5855,   5857,   5856}},
 {  3073,  12, {  5844,   5845,   5859,   5858,   5848,   5849,   5863,   5862}},
 {  3074,  12, {  5858,   5859,   5861,   5860,   5862,   5863,   5865,   5864}},
 {  3075,  12, {  5852,   5853,   5867,   5866,   5856,   5857,   5869,   5868}},
 {  3076,  12, {  5856,   5857,   5869,   5868,   5870,   5871,   5873,   5872}},
 {  3077,  12, {  5870,   5871,   5873,   5872,   5860,   5861,   5875,   5874}},
 {  3078,  12, {  5860,   5861,   5875,   5874,   5864,   5865,   5877,   5876}},
 {  3079,  12, {  5831,   5878,   5879,   5833,   5835,   5880,   5881,   5837}},
 {  3080,  12, {  5835,   5880,   5881,   5837,   5839,   5882,   5883,   5841}},
 {  3081,  12, {  5839,   5882,   5883,   5841,   5843,   5884,   5885,   5845}},
 {  3082,  12, {  5843,   5884,   5885,   5845,   5847,   5886,   5887,   5849}},
 {  3083,  12, {  5833,   5879,   5888,   5851,   5837,   5881,   5890,   5855}},
 {  3084,  12, {  5851,   5888,   5889,   5853,   5855,   5890,   5891,   5857}},
 {  3085,  12, {  5845,   5885,   5892,   5859,   5849,   5887,   5894,   5863}},
 {  3086,  12, {  5859,   5892,   5893,   5861,   5863,   5894,   5895,   5865}},
 {  3087,  12, {  5853,   5889,   5896,   5867,   5857,   5891,   5897,   5869}},
 {  3088,  12, {  5857,   5891,   5897,   5869,   5871,   5898,   5899,   5873}},
 {  3089,  12, {  5871,   5898,   5899,   5873,   5861,   5893,   5900,   5875}},
 {  3090,  12, {  5861,   5893,   5900,   5875,   5865,   5895,   5901,   5877}},
 {  3091,  13, {  5902,   5903,   5907,   5906,   5914,   5915,   5919,   5918}},
 {  3092,  13, {  5903,   5904,   5908,   5907,   5915,   5916,   5920,   5919}},
 {  3093,  13, {  5904,   5905,   5909,   5908,   5916,   5917,   5921,   5920}},
 {  3094,  13, {  5906,   5907,   5911,   5910,   5918,   5919,   3207,   3206}},
 {  3095,  13, {  5907,   5908,   5912,   5911,   5919,   5920,   3208,   3207}},
 {  3096,  13, {  5908,   5909,   5913,   5912,   5920,   5921,   3209,   3208}},
 {  3097,  13, {  5914,   5915,   5919,   5918,   5922,   5923,   5927,   5926}},
 {  3098,  13, {  5915,   5916,   5920,   5919,   5923,   5924,   5928,   5927}},
 {  3099,  13, {  5916,   5917,   5921,   5920,   5924,   5925,   5929,   5928}},
 {  3100,  13, {  5918,   5919,   3207,   3206,   5926,   5927,   3215,   3214}},
 {  3101,  13, {  5919,   5920,   3208,   3207,   5927,   5928,   3216,   3215}},
 {  3102,  13, {  5920,   5921,   3209,   3208,   5928,   5929,   3217,   3216}},
 {  3103,  13, {  5922,   5923,   5927,   5926,   5930,   5931,   5935,   5934}},
 {  3104,  13, {  5923,   5924,   5928,   5927,   5931,   5932,   5936,   5935}},
 {  3105,  13, {  5924,   5925,   5929,   5928,   5932,   5933,   5937,   5936}},
 {  3106,  13, {  5926,   5927,   3215,   3214,   5934,   5935,   3223,   3222}},
 {  3107,  13, {  5927,   5928,   3216,   3215,   5935,   5936,   3224,   3223}},
 {  3108,  13, {  5928,   5929,   3217,   3216,   5936,   5937,   3225,   3224}},
 {  3109,  13, {  5930,   5931,   5935,   5934,   5938,   5939,   5943,   5942}},
 {  3110,  13, {  5931,   5932,   5936,   5935,   5939,   5940,   5944,   5943}},
 {  3111,  13, {  5932,   5933,   5937,   5936,   5940,   5941,   5945,   5944}},
 {  3112,  13, {  5934,   5935,   3223,   3222,   5942,   5943,   3231,   3230}},
 {  3113,  13, {  5935,   5936,   3224,   3223,   5943,   5944,   3232,   3231}},
 {  3114,  13, {  5936,   5937,   3225,   3224,   5944,   5945,   3233,   3232}},
 {  3115,  13, {  5938,   5939,   5943,   5942,   5946,   5947,   5951,   5950}},
 {  3116,  13, {  5939,   5940,   5944,   5943,   5947,   5948,   5952,   5951}},
 {  3117,  13, {  5940,   5941,   5945,   5944,   5948,   5949,   5953,   5952}},
 {  3118,  13, {  5942,   5943,   3231,   3230,   5950,   5951,   3239,   3238}},
 {  3119,  13, {  5943,   5944,   3232,   3231,   5951,   5952,   3240,   3239}},
 {  3120,  13, {  5944,   5945,   3233,   3232,   5952,   5953,   3241,   3240}},
 {  3121,  13, {  5946,   5947,   5951,   5950,   5954,   5955,   5959,   5958}},
 {  3122,  13, {  5947,   5948,   5952,   5951,   5955,   5956,   5960,   5959}},
 {  3123,  13, {  5948,   5949,   5953,   5952,   5956,   5957,   5961,   5960}},
 {  3124,  13, {  5950,   5951,   3239,   3238,   5958,   5959,   3247,   3246}},
 {  3125,  13, {  5951,   5952,   3240,   3239,   5959,   5960,   3248,   3247}},
 {  3126,  13, {  5952,   5953,   3241,   3240,   5960,   5961,   3249,   3248}},
 {  3127,  13, {  5954,   5955,   5959,   5958,   5962,   5963,   5967,   5966}},
 {  3128,  13, {  5955,   5956,   5960,   5959,   5963,   5964,   5968,   5967}},
 {  3129,  13, {  5956,   5957,   5961,   5960,   5964,   5965,   5969,   5968}},
 {  3130,  13, {  5958,   5959,   3247,   3246,   5966,   5967,   3255,   3254}},
 {  3131,  13, {  5959,   5960,   3248,   3247,   5967,   5968,   3256,   3255}},
 {  3132,  13, {  5960,   5961,   3249,   3248,   5968,   5969,   3257,   3256}},
 {  3133,  13, {  5962,   5963,   5967,   5966,   5970,   5971,   5975,   5974}},
 {  3134,  13, {  5963,   5964,   5968,   5967,   5971,   5972,   5976,   5975}},
 {  3135,  13, {  5964,   5965,   5969,   5968,   5972,   5973,   5977,   5976}},
 {  3136,  13, {  5966,   5967,   3255,   3254,   5974,   5975,   3263,   3262}},
 {  3137,  13, {  5967,   5968,   3256,   3255,   5975,   5976,   3264,   3263}},
 {  3138,  13, {  5968,   5969,   3257,   3256,   5976,   5977,   3265,   3264}},
 {  3139,  13, {  5970,   5971,   5975,   5974,   5978,   5979,   5983,   5982}},
 {  3140,  13, {  5971,   5972,   5976,   5975,   5979,   5980,   5984,   5983}},
 {  3141,  13, {  5972,   5973,   5977,   5976,   5980,   5981,   5985,   5984}},
 {  3142,  13, {  5974,   5975,   3263,   3262,   5982,   5983,   3271,   3270}},
 {  3143,  13, {  5975,   5976,   3264,   3263,   5983,   5984,   3272,   3271}},
 {  3144,  13, {  5976,   5977,   3265,   3264,   5984,   5985,   3273,   3272}},
 {  3145,  13, {  5978,   5979,   5983,   5982,   5986,   5987,   5991,   5990}},
 {  3146,  13, {  5979,   5980,   5984,   5983,   5987,   5988,   5992,   5991}},
 {  3147,  13, {  5980,   5981,   5985,   5984,   5988,   5989,   5993,   5992}},
 {  3148,  13, {  5982,   5983,   3271,   3270,   5990,   5991,   3279,   3278}},
 {  3149,  13, {  5983,   5984,   3272,   3271,   5991,   5992,   3280,   3279}},
 {  3150,  13, {  5984,   5985,   3273,   3272,   5992,   5993,   3281,   3280}},
 {  3151,  13, {  5986,   5987,   5991,   5990,   5994,   5995,   5999,   5998}},
 {  3152,  13, {  5987,   5988,   5992,   5991,   5995,   5996,   6000,   5999}},
 {  3153,  13, {  5988,   5989,   5993,   5992,   5996,   5997,   6001,   6000}},
 {  3154,  13, {  5990,   5991,   3279,   3278,   5998,   5999,   3287,   3286}},
 {  3155,  13, {  5991,   5992,   3280,   3279,   5999,   6000,   3288,   3287}},
 {  3156,  13, {  5992,   5993,   3281,   3280,   6000,   6001,   3289,   3288}},
 {  3157,  13, {  5994,   5995,   5999,   5998,   6002,   6003,   6007,   6006}},
 {  3158,  13, {  5995,   5996,   6000,   5999,   6003,   6004,   6008,   6007}},
 {  3159,  13, {  5996,   5997,   6001,   6000,   6004,   6005,   6009,   6008}},
 {  3160,  13, {  5998,   5999,   3287,   3286,   6006,   6007,   3295,   3294}},
 {  3161,  13, {  5999,   6000,   3288,   3287,   6007,   6008,   3296,   3295}},
 {  3162,  13, {  6000,   6001,   3289,   3288,   6008,   6009,   3297,   3296}},
 {  3163,  13, {  6002,   6003,   6007,   6006,   6010,   6011,   6015,   6014}},
 {  3164,  13, {  6003,   6004,   6008,   6007,   6011,   6012,   6016,   6015}},
 {  3165,  13, {  6004,   6005,   6009,   6008,   6012,   6013,   6017,   6016}},
 {  3166,  13, {  6006,   6007,   3295,   3294,   6014,   6015,   3303,   3302}},
 {  3167,  13, {  6007,   6008,   3296,   3295,   6015,   6016,   3304,   3303}},
 {  3168,  13, {  6008,   6009,   3297,   3296,   6016,   6017,   3305,   3304}},
 {  3169,  13, {  6010,   6011,   6015,   6014,   6018,   6019,   6023,   6022}},
 {  3170,  13, {  6011,   6012,   6016,   6015,   6019,   6020,   6024,   6023}},
 {  3171,  13, {  6012,   6013,   6017,   6016,   6020,   6021,   6025,   6024}},
 {  3172,  13, {  6014,   6015,   3303,   3302,   6022,   6023,   3311,   3310}},
 {  3173,  13, {  6015,   6016,   3304,   3303,   6023,   6024,   3312,   3311}},
 {  3174,  13, {  6016,   6017,   3305,   3304,   6024,   6025,   3313,   3312}},
 {  3175,  13, {  6018,   6019,   6023,   6022,   6026,   6027,   6031,   6030}},
 {  3176,  13, {  6019,   6020,   6024,   6023,   6027,   6028,   6032,   6031}},
 {  3177,  13, {  6020,   6021,   6025,   6024,   6028,   6029,   6033,   6032}},
 {  3178,  13, {  6022,   6023,   3311,   3310,   6030,   6031,   3319,   3318}},
 {  3179,  13, {  6023,   6024,   3312,   3311,   6031,   6032,   3320,   3319}},
 {  3180,  13, {  6024,   6025,   3313,   3312,   6032,   6033,   3321,   3320}},
 {  3181,  13, {  6026,   6027,   6031,   6030,   6034,   6035,   6039,   6038}},
 {  3182,  13, {  6027,   6028,   6032,   6031,   6035,   6036,   6040,   6039}},
 {  3183,  13, {  6028,   6029,   6033,   6032,   6036,   6037,   6041,   6040}},
 {  3184,  13, {  6030,   6031,   3319,   3318,   6038,   6039,   6043,   6042}},
 {  3185,  13, {  6031,   6032,   3320,   3319,   6039,   6040,   6044,   6043}},
 {  3186,  13, {  6032,   6033,   3321,   3320,   6040,   6041,   6045,   6044}},
 {  3187,  14, {  6046,   6047,   6051,   6050,   6058,   6059,   6063,   6062}},
 {  3188,  14, {  6047,   6048,   6052,   6051,   6059,   6060,   6064,   6063}},
 {  3189,  14, {  6048,   6049,   6053,   6052,   6060,   6061,   6065,   6064}},
 {  3190,  14, {  6050,   6051,   6055,   6054,   6062,   6063,   3327,   3326}},
 {  3191,  14, {  6051,   6052,   6056,   6055,   6063,   6064,   3328,   3327}},
 {  3192,  14, {  6052,   6053,   6057,   6056,   6064,   6065,   3329,   3328}},
 {  3193,  14, {  6058,   6059,   6063,   6062,   6066,   6067,   6071,   6070}},
 {  3194,  14, {  6059,   6060,   6064,   6063,   6067,   6068,   6072,   6071}},
 {  3195,  14, {  6060,   6061,   6065,   6064,   6068,   6069,   6073,   6072}},
 {  3196,  14, {  6062,   6063,   3327,   3326,   6070,   6071,   3335,   3334}},
 {  3197,  14, {  6063,   6064,   3328,   3327,   6071,   6072,   3336,   3335}},
 {  3198,  14, {  6064,   6065,   3329,   3328,   6072,   6073,   3337,   3336}},
 {  3199,  14, {  6066,   6067,   6071,   6070,   6074,   6075,   6079,   6078}},
 {  3200,  14, {  6067,   6068,   6072,   6071,   6075,   6076,   6080,   6079}},
 {  3201,  14, {  6068,   6069,   6073,   6072,   6076,   6077,   6081,   6080}},
 {  3202,  14, {  6070,   6071,   3335,   3334,   6078,   6079,   3343,   3342}},
 {  3203,  14, {  6071,   6072,   3336,   3335,   6079,   6080,   3344,   3343}},
 {  3204,  14, {  6072,   6073,   3337,   3336,   6080,   6081,   3345,   3344}},
 {  3205,  14, {  6074,   6075,   6079,   6078,   6082,   6083,   6087,   6086}},
 {  3206,  14, {  6075,   6076,   6080,   6079,   6083,   6084,   6088,   6087}},
 {  3207,  14, {  6076,   6077,   6081,   6080,   6084,   6085,   6089,   6088}},
 {  3208,  14, {  6078,   6079,   3343,   3342,   6086,   6087,   3351,   3350}},
 {  3209,  14, {  6079,   6080,   3344,   3343,   6087,   6088,   3352,   3351}},
 {  3210,  14, {  6080,   6081,   3345,   3344,   6088,   6089,   3353,   3352}},
 {  3211,  14, {  6082,   6083,   6087,   6086,   6090,   6091,   6095,   6094}},
 {  3212,  14, {  6083,   6084,   6088,   6087,   6091,   6092,   6096,   6095}},
 {  3213,  14, {  6084,   6085,   6089,   6088,   6092,   6093,   6097,   6096}},
 {  3214,  14, {  6086,   6087,   3351,   3350,   6094,   6095,   3359,   3358}},
 {  3215,  14, {  6087,   6088,   3352,   3351,   6095,   6096,   3360,   3359}},
 {  3216,  14, {  6088,   6089,   3353,   3352,   6096,   6097,   3361,   3360}},
 {  3217,  14, {  6090,   6091,   6095,   6094,   6098,   6099,   6103,   6102}},
 {  3218,  14, {  6091,   6092,   6096,   6095,   6099,   6100,   6104,   6103}},
 {  3219,  14, {  6092,   6093,   6097,   6096,   6100,   6101,   6105,   6104}},
 {  3220,  14, {  6094,   6095,   3359,   3358,   6102,   6103,   3367,   3366}},
 {  3221,  14, {  6095,   6096,   3360,   3359,   6103,   6104,   3368,   3367}},
 {  3222,  14, {  6096,   6097,   3361,   3360,   6104,   6105,   3369,   3368}},
 {  3223,  14, {  6098,   6099,   6103,   6102,   6106,   6107,   6111,   6110}},
 {  3224,  14, {  6099,   6100,   6104,   6103,   6107,   6108,   6112,   6111}},
 {  3225,  14, {  6100,   6101,   6105,   6104,   6108,   6109,   6113,   6112}},
 {  3226,  14, {  6102,   6103,   3367,   3366,   6110,   6111,   3375,   3374}},
 {  3227,  14, {  6103,   6104,   3368,   3367,   6111,   6112,   3376,   3375}},
 {  3228,  14, {  6104,   6105,   3369,   3368,   6112,   6113,   3377,   3376}},
 {  3229,  14, {  6106,   6107,   6111,   6110,   6114,   6115,   6119,   6118}},
 {  3230,  14, {  6107,   6108,   6112,   6111,   6115,   6116,   6120,   6119}},
 {  3231,  14, {  6108,   6109,   6113,   6112,   6116,   6117,   6121,   6120}},
 {  3232,  14, {  6110,   6111,   3375,   3374,   6118,   6119,   3383,   3382}},
 {  3233,  14, {  6111,   6112,   3376,   3375,   6119,   6120,   3384,   3383}},
 {  3234,  14, {  6112,   6113,   3377,   3376,   6120,   6121,   3385,   3384}},
 {  3235,  14, {  6114,   6115,   6119,   6118,   6122,   6123,   6127,   6126}},
 {  3236,  14, {  6115,   6116,   6120,   6119,   6123,   6124,   6128,   6127}},
 {  3237,  14, {  6116,   6117,   6121,   6120,   6124,   6125,   6129,   6128}},
 {  3238,  14, {  6118,   6119,   3383,   3382,   6126,   6127,   3391,   3390}},
 {  3239,  14, {  6119,   6120,   3384,   3383,   6127,   6128,   3392,   3391}},
 {  3240,  14, {  6120,   6121,   3385,   3384,   6128,   6129,   3393,   3392}},
 {  3241,  14, {  6122,   6123,   6127,   6126,   6130,   6131,   6135,   6134}},
 {  3242,  14, {  6123,   6124,   6128,   6127,   6131,   6132,   6136,   6135}},
 {  3243,  14, {  6124,   6125,   6129,   6128,   6132,   6133,   6137,   6136}},
 {  3244,  14, {  6126,   6127,   3391,   3390,   6134,   6135,   3399,   3398}},
 {  3245,  14, {  6127,   6128,   3392,   3391,   6135,   6136,   3400,   3399}},
 {  3246,  14, {  6128,   6129,   3393,   3392,   6136,   6137,   3401,   3400}},
 {  3247,  14, {  6130,   6131,   6135,   6134,   6138,   6139,   6143,   6142}},
 {  3248,  14, {  6131,   6132,   6136,   6135,   6139,   6140,   6144,   6143}},
 {  3249,  14, {  6132,   6133,   6137,   6136,   6140,   6141,   6145,   6144}},
 {  3250,  14, {  6134,   6135,   3399,   3398,   6142,   6143,   3407,   3406}},
 {  3251,  14, {  6135,   6136,   3400,   3399,   6143,   6144,   3408,   3407}},
 {  3252,  14, {  6136,   6137,   3401,   3400,   6144,   6145,   3409,   3408}},
 {  3253,  14, {  6138,   6139,   6143,   6142,   6146,   6147,   6151,   6150}},
 {  3254,  14, {  6139,   6140,   6144,   6143,   6147,   6148,   6152,   6151}},
 {  3255,  14, {  6140,   6141,   6145,   6144,   6148,   6149,   6153,   6152}},
 {  3256,  14, {  6142,   6143,   3407,   3406,   6150,   6151,   3415,   3414}},
 {  3257,  14, {  6143,   6144,   3408,   3407,   6151,   6152,   3416,   3415}},
 {  3258,  14, {  6144,   6145,   3409,   3408,   6152,   6153,   3417,   3416}},
 {  3259,  14, {  6146,   6147,   6151,   6150,   6154,   6155,   6159,   6158}},
 {  3260,  14, {  6147,   6148,   6152,   6151,   6155,   6156,   6160,   6159}},
 {  3261,  14, {  6148,   6149,   6153,   6152,   6156,   6157,   6161,   6160}},
 {  3262,  14, {  6150,   6151,   3415,   3414,   6158,   6159,   3423,   3422}},
 {  3263,  14, {  6151,   6152,   3416,   3415,   6159,   6160,   3424,   3423}},
 {  3264,  14, {  6152,   6153,   3417,   3416,   6160,   6161,   3425,   3424}},
 {  3265,  14, {  6154,   6155,   6159,   6158,   6162,   6163,   6167,   6166}},
 {  3266,  14, {  6155,   6156,   6160,   6159,   6163,   6164,   6168,   6167}},
 {  3267,  14, {  6156,   6157,   6161,   6160,   6164,   6165,   6169,   6168}},
 {  3268,  14, {  6158,   6159,   3423,   3422,   6166,   6167,   3431,   3430}},
 {  3269,  14, {  6159,   6160,   3424,   3423,   6167,   6168,   3432,   3431}},
 {  3270,  14, {  6160,   6161,   3425,   3424,   6168,   6169,   3433,   3432}},
 {  3271,  14, {  6162,   6163,   6167,   6166,   6170,   6171,   6175,   6174}},
 {  3272,  14, {  6163,   6164,   6168,   6167,   6171,   6172,   6176,   6175}},
 {  3273,  14, {  6164,   6165,   6169,   6168,   6172,   6173,   6177,   6176}},
 {  3274,  14, {  6166,   6167,   3431,   3430,   6174,   6175,   3439,   3438}},
 {  3275,  14, {  6167,   6168,   3432,   3431,   6175,   6176,   3440,   3439}},
 {  3276,  14, {  6168,   6169,   3433,   3432,   6176,   6177,   3441,   3440}},
 {  3277,  14, {  6170,   6171,   6175,   6174,   6178,   6179,   6183,   6182}},
 {  3278,  14, {  6171,   6172,   6176,   6175,   6179,   6180,   6184,   6183}},
 {  3279,  14, {  6172,   6173,   6177,   6176,   6180,   6181,   6185,   6184}},
 {  3280,  14, {  6174,   6175,   3439,   3438,   6182,   6183,   6187,   6186}},
 {  3281,  14, {  6175,   6176,   3440,   3439,   6183,   6184,   6188,   6187}},
 {  3282,  14, {  6176,   6177,   3441,   3440,   6184,   6185,   6189,   6188}},
 {  3283,  15, {  6190,   6191,   6195,   6194,   6202,   6203,   6207,   6206}},
 {  3284,  15, {  6191,   6192,   6196,   6195,   6203,   6204,   6208,   6207}},
 {  3285,  15, {  6192,   6193,   6197,   6196,   6204,   6205,   6209,   6208}},
 {  3286,  15, {  6194,   6195,   6199,   6198,   6206,   6207,   3447,   3446}},
 {  3287,  15, {  6195,   6196,   6200,   6199,   6207,   6208,   3448,   3447}},
 {  3288,  15, {  6196,   6197,   6201,   6200,   6208,   6209,   3449,   3448}},
 {  3289,  15, {  6202,   6203,   6207,   6206,   6210,   6211,   6215,   6214}},
 {  3290,  15, {  6203,   6204,   6208,   6207,   6211,   6212,   6216,   6215}},
 {  3291,  15, {  6204,   6205,   6209,   6208,   6212,   6213,   6217,   6216}},
 {  3292,  15, {  6206,   6207,   3447,   3446,   6214,   6215,   3455,   3454}},
 {  3293,  15, {  6207,   6208,   3448,   3447,   6215,   6216,   3456,   3455}},
 {  3294,  15, {  6208,   6209,   3449,   3448,   6216,   6217,   3457,   3456}},
 {  3295,  15, {  6210,   6211,   6215,   6214,   6218,   6219,   6223,   6222}},
 {  3296,  15, {  6211,   6212,   6216,   6215,   6219,   6220,   6224,   6223}},
 {  3297,  15, {  6212,   6213,   6217,   6216,   6220,   6221,   6225,   6224}},
 {  3298,  15, {  6214,   6215,   3455,   3454,   6222,   6223,   3463,   3462}},
 {  3299,  15, {  6215,   6216,   3456,   3455,   6223,   6224,   3464,   3463}},
 {  3300,  15, {  6216,   6217,   3457,   3456,   6224,   6225,   3465,   3464}},
 {  3301,  15, {  6218,   6219,   6223,   6222,   6226,   6227,   6231,   6230}},
 {  3302,  15, {  6219,   6220,   6224,   6223,   6227,   6228,   6232,   6231}},
 {  3303,  15, {  6220,   6221,   6225,   6224,   6228,   6229,   6233,   6232}},
 {  3304,  15, {  6222,   6223,   3463,   3462,   6230,   6231,   3471,   3470}},
 {  3305,  15, {  6223,   6224,   3464,   3463,   6231,   6232,   3472,   3471}},
 {  3306,  15, {  6224,   6225,   3465,   3464,   6232,   6233,   3473,   3472}},
 {  3307,  15, {  6226,   6227,   6231,   6230,   6234,   6235,   6239,   6238}},
 {  3308,  15, {  6227,   6228,   6232,   6231,   6235,   6236,   6240,   6239}},
 {  3309,  15, {  6228,   6229,   6233,   6232,   6236,   6237,   6241,   6240}},
 {  3310,  15, {  6230,   6231,   3471,   3470,   6238,   6239,   3479,   3478}},
 {  3311,  15, {  6231,   6232,   3472,   3471,   6239,   6240,   3480,   3479}},
 {  3312,  15, {  6232,   6233,   3473,   3472,   6240,   6241,   3481,   3480}},
 {  3313,  15, {  6234,   6235,   6239,   6238,   6242,   6243,   6247,   6246}},
 {  3314,  15, {  6235,   6236,   6240,   6239,   6243,   6244,   6248,   6247}},
 {  3315,  15, {  6236,   6237,   6241,   6240,   6244,   6245,   6249,   6248}},
 {  3316,  15, {  6238,   6239,   3479,   3478,   6246,   6247,   3487,   3486}},
 {  3317,  15, {  6239,   6240,   3480,   3479,   6247,   6248,   3488,   3487}},
 {  3318,  15, {  6240,   6241,   3481,   3480,   6248,   6249,   3489,   3488}},
 {  3319,  15, {  6242,   6243,   6247,   6246,   6250,   6251,   6255,   6254}},
 {  3320,  15, {  6243,   6244,   6248,   6247,   6251,   6252,   6256,   6255}},
 {  3321,  15, {  6244,   6245,   6249,   6248,   6252,   6253,   6257,   6256}},
 {  3322,  15, {  6246,   6247,   3487,   3486,   6254,   6255,   3495,   3494}},
 {  3323,  15, {  6247,   6248,   3488,   3487,   6255,   6256,   3496,   3495}},
 {  3324,  15, {  6248,   6249,   3489,   3488,   6256,   6257,   3497,   3496}},
 {  3325,  15, {  6250,   6251,   6255,   6254,   6258,   6259,   6263,   6262}},
 {  3326,  15, {  6251,   6252,   6256,   6255,   6259,   6260,   6264,   6263}},
 {  3327,  15, {  6252,   6253,   6257,   6256,   6260,   6261,   6265,   6264}},
 {  3328,  15, {  6254,   6255,   3495,   3494,   6262,   6263,   3503,   3502}},
 {  3329,  15, {  6255,   6256,   3496,   3495,   6263,   6264,   3504,   3503}},
 {  3330,  15, {  6256,   6257,   3497,   3496,   6264,   6265,   3505,   3504}},
 {  3331,  15, {  6258,   6259,   6263,   6262,   6266,   6267,   6271,   6270}},
 {  3332,  15, {  6259,   6260,   6264,   6263,   6267,   6268,   6272,   6271}},
 {  3333,  15, {  6260,   6261,   6265,   6264,   6268,   6269,   6273,   6272}},
 {  3334,  15, {  6262,   6263,   3503,   3502,   6270,   6271,   3511,   3510}},
 {  3335,  15, {  6263,   6264,   3504,   3503,   6271,   6272,   3512,   3511}},
 {  3336,  15, {  6264,   6265,   3505,   3504,   6272,   6273,   3513,   3512}},
 {  3337,  15, {  6266,   6267,   6271,   6270,   6274,   6275,   6279,   6278}},
 {  3338,  15, {  6267,   6268,   6272,   6271,   6275,   6276,   6280,   6279}},
 {  3339,  15, {  6268,   6269,   6273,   6272,   6276,   6277,   6281,   6280}},
 {  3340,  15, {  6270,   6271,   3511,   3510,   6278,   6279,   3519,   3518}},
 {  3341,  15, {  6271,   6272,   3512,   3511,   6279,   6280,   3520,   3519}},
 {  3342,  15, {  6272,   6273,   3513,   3512,   6280,   6281,   3521,   3520}},
 {  3343,  15, {  6274,   6275,   6279,   6278,   6282,   6283,   6287,   6286}},
 {  3344,  15, {  6275,   6276,   6280,   6279,   6283,   6284,   6288,   6287}},
 {  3345,  15, {  6276,   6277,   6281,   6280,   6284,   6285,   6289,   6288}},
 {  3346,  15, {  6278,   6279,   3519,   3518,   6286,   6287,   3527,   3526}},
 {  3347,  15, {  6279,   6280,   3520,   3519,   6287,   6288,   3528,   3527}},
 {  3348,  15, {  6280,   6281,   3521,   3520,   6288,   6289,   3529,   3528}},
 {  3349,  15, {  6282,   6283,   6287,   6286,   6290,   6291,   6295,   6294}},
 {  3350,  15, {  6283,   6284,   6288,   6287,   6291,   6292,   6296,   6295}},
 {  3351,  15, {  6284,   6285,   6289,   6288,   6292,   6293,   6297,   6296}},
 {  3352,  15, {  6286,   6287,   3527,   3526,   6294,   6295,   3535,   3534}},
 {  3353,  15, {  6287,   6288,   3528,   3527,   6295,   6296,   3536,   3535}},
 {  3354,  15, {  6288,   6289,   3529,   3528,   6296,   6297,   3537,   3536}},
 {  3355,  15, {  6290,   6291,   6295,   6294,   6298,   6299,   6303,   6302}},
 {  3356,  15, {  6291,   6292,   6296,   6295,   6299,   6300,   6304,   6303}},
 {  3357,  15, {  6292,   6293,   6297,   6296,   6300,   6301,   6305,   6304}},
 {  3358,  15, {  6294,   6295,   3535,   3534,   6302,   6303,   3543,   3542}},
 {  3359,  15, {  6295,   6296,   3536,   3535,   6303,   6304,   3544,   3543}},
 {  3360,  15, {  6296,   6297,   3537,   3536,   6304,   6305,   3545,   3544}},
 {  3361,  15, {  6298,   6299,   6303,   6302,   6306,   6307,   6311,   6310}},
 {  3362,  15, {  6299,   6300,   6304,   6303,   6307,   6308,   6312,   6311}},
 {  3363,  15, {  6300,   6301,   6305,   6304,   6308,   6309,   6313,   6312}},
 {  3364,  15, {  6302,   6303,   3543,   3542,   6310,   6311,   3551,   3550}},
 {  3365,  15, {  6303,   6304,   3544,   3543,   6311,   6312,   3552,   3551}},
 {  3366,  15, {  6304,   6305,   3545,   3544,   6312,   6313,   3553,   3552}},
 {  3367,  15, {  6306,   6307,   6311,   6310,   6314,   6315,   6319,   6318}},
 {  3368,  15, {  6307,   6308,   6312,   6311,   6315,   6316,   6320,   6319}},
 {  3369,  15, {  6308,   6309,   6313,   6312,   6316,   6317,   6321,   6320}},
 {  3370,  15, {  6310,   6311,   3551,   3550,   6318,   6319,   3559,   3558}},
 {  3371,  15, {  6311,   6312,   3552,   3551,   6319,   6320,   3560,   3559}},
 {  3372,  15, {  6312,   6313,   3553,   3552,   6320,   6321,   3561,   3560}},
 {  3373,  15, {  6314,   6315,   6319,   6318,   6322,   6323,   6327,   6326}},
 {  3374,  15, {  6315,   6316,   6320,   6319,   6323,   6324,   6328,   6327}},
 {  3375,  15, {  6316,   6317,   6321,   6320,   6324,   6325,   6329,   6328}},
 {  3376,  15, {  6318,   6319,   3559,   3558,   6326,   6327,   6331,   6330}},
 {  3377,  15, {  6319,   6320,   3560,   3559,   6327,   6328,   6332,   6331}},
 {  3378,  15, {  6320,   6321,   3561,   3560,   6328,   6329,   6333,   6332}},
 {  3379,  16, {  6334,   6335,   6339,   6338,   6346,   6347,   6351,   6350}},
 {  3380,  16, {  6335,   6336,   6340,   6339,   6347,   6348,   6352,   6351}},
 {  3381,  16, {  6336,   6337,   6341,   6340,   6348,   6349,   6353,   6352}},
 {  3382,  16, {  6338,   6339,   6343,   6342,   6350,   6351,   3567,   3566}},
 {  3383,  16, {  6339,   6340,   6344,   6343,   6351,   6352,   3568,   3567}},
 {  3384,  16, {  6340,   6341,   6345,   6344,   6352,   6353,   3569,   3568}},
 {  3385,  16, {  6346,   6347,   6351,   6350,   6354,   6355,   6359,   6358}},
 {  3386,  16, {  6347,   6348,   6352,   6351,   6355,   6356,   6360,   6359}},
 {  3387,  16, {  6348,   6349,   6353,   6352,   6356,   6357,   6361,   6360}},
 {  3388,  16, {  6350,   6351,   3567,   3566,   6358,   6359,   3575,   3574}},
 {  3389,  16, {  6351,   6352,   3568,   3567,   6359,   6360,   3576,   3575}},
 {  3390,  16, {  6352,   6353,   3569,   3568,   6360,   6361,   3577,   3576}},
 {  3391,  16, {  6354,   6355,   6359,   6358,   6362,   6363,   6367,   6366}},
 {  3392,  16, {  6355,   6356,   6360,   6359,   6363,   6364,   6368,   6367}},
 {  3393,  16, {  6356,   6357,   6361,   6360,   6364,   6365,   6369,   6368}},
 {  3394,  16, {  6358,   6359,   3575,   3574,   6366,   6367,   3583,   3582}},
 {  3395,  16, {  6359,   6360,   3576,   3575,   6367,   6368,   3584,   3583}},
 {  3396,  16, {  6360,   6361,   3577,   3576,   6368,   6369,   3585,   3584}},
 {  3397,  16, {  6362,   6363,   6367,   6366,   6370,   6371,   6375,   6374}},
 {  3398,  16, {  6363,   6364,   6368,   6367,   6371,   6372,   6376,   6375}},
 {  3399,  16, {  6364,   6365,   6369,   6368,   6372,   6373,   6377,   6376}},
 {  3400,  16, {  6366,   6367,   3583,   3582,   6374,   6375,   3591,   3590}},
 {  3401,  16, {  6367,   6368,   3584,   3583,   6375,   6376,   3592,   3591}},
 {  3402,  16, {  6368,   6369,   3585,   3584,   6376,   6377,   3593,   3592}},
 {  3403,  16, {  6370,   6371,   6375,   6374,   6378,   6379,   6383,   6382}},
 {  3404,  16, {  6371,   6372,   6376,   6375,   6379,   6380,   6384,   6383}},
 {  3405,  16, {  6372,   6373,   6377,   6376,   6380,   6381,   6385,   6384}},
 {  3406,  16, {  6374,   6375,   3591,   3590,   6382,   6383,   3599,   3598}},
 {  3407,  16, {  6375,   6376,   3592,   3591,   6383,   6384,   3600,   3599}},
 {  3408,  16, {  6376,   6377,   3593,   3592,   6384,   6385,   3601,   3600}},
 {  3409,  16, {  6378,   6379,   6383,   6382,   6386,   6387,   6391,   6390}},
 {  3410,  16, {  6379,   6380,   6384,   6383,   6387,   6388,   6392,   6391}},
 {  3411,  16, {  6380,   6381,   6385,   6384,   6388,   6389,   6393,   6392}},
 {  3412,  16, {  6382,   6383,   3599,   3598,   6390,   6391,   3607,   3606}},
 {  3413,  16, {  6383,   6384,   3600,   3599,   6391,   6392,   3608,   3607}},
 {  3414,  16, {  6384,   6385,   3601,   3600,   6392,   6393,   3609,   3608}},
 {  3415,  16, {  6386,   6387,   6391,   6390,   6394,   6395,   6399,   6398}},
 {  3416,  16, {  6387,   6388,   6392,   6391,   6395,   6396,   6400,   6399}},
 {  3417,  16, {  6388,   6389,   6393,   6392,   6396,   6397,   6401,   6400}},
 {  3418,  16, {  6390,   6391,   3607,   3606,   6398,   6399,   3615,   3614}},
 {  3419,  16, {  6391,   6392,   3608,   3607,   6399,   6400,   3616,   3615}},
 {  3420,  16, {  6392,   6393,   3609,   3608,   6400,   6401,   3617,   3616}},
 {  3421,  16, {  6394,   6395,   6399,   6398,   6402,   6403,   6407,   6406}},
 {  3422,  16, {  6395,   6396,   6400,   6399,   6403,   6404,   6408,   6407}},
 {  3423,  16, {  6396,   6397,   6401,   6400,   6404,   6405,   6409,   6408}},
 {  3424,  16, {  6398,   6399,   3615,   3614,   6406,   6407,   3623,   3622}},
 {  3425,  16, {  6399,   6400,   3616,   3615,   6407,   6408,   3624,   3623}},
 {  3426,  16, {  6400,   6401,   3617,   3616,   6408,   6409,   3625,   3624}},
 {  3427,  16, {  6402,   6403,   6407,   6406,   6410,   6411,   6415,   6414}},
 {  3428,  16, {  6403,   6404,   6408,   6407,   6411,   6412,   6416,   6415}},
 {  3429,  16, {  6404,   6405,   6409,   6408,   6412,   6413,   6417,   6416}},
 {  3430,  16, {  6406,   6407,   3623,   3622,   6414,   6415,   3631,   3630}},
 {  3431,  16, {  6407,   6408,   3624,   3623,   6415,   6416,   3632,   3631}},
 {  3432,  16, {  6408,   6409,   3625,   3624,   6416,   6417,   3633,   3632}},
 {  3433,  16, {  6410,   6411,   6415,   6414,   6418,   6419,   6423,   6422}},
 {  3434,  16, {  6411,   6412,   6416,   6415,   6419,   6420,   6424,   6423}},
 {  3435,  16, {  6412,   6413,   6417,   6416,   6420,   6421,   6425,   6424}},
 {  3436,  16, {  6414,   6415,   3631,   3630,   6422,   6423,   3639,   3638}},
 {  3437,  16, {  6415,   6416,   3632,   3631,   6423,   6424,   3640,   3639}},
 {  3438,  16, {  6416,   6417,   3633,   3632,   6424,   6425,   3641,   3640}},
 {  3439,  16, {  6418,   6419,   6423,   6422,   6426,   6427,   6431,   6430}},
 {  3440,  16, {  6419,   6420,   6424,   6423,   6427,   6428,   6432,   6431}},
 {  3441,  16, {  6420,   6421,   6425,   6424,   6428,   6429,   6433,   6432}},
 {  3442,  16, {  6422,   6423,   3639,   3638,   6430,   6431,   3647,   3646}},
 {  3443,  16, {  6423,   6424,   3640,   3639,   6431,   6432,   3648,   3647}},
 {  3444,  16, {  6424,   6425,   3641,   3640,   6432,   6433,   3649,   3648}},
 {  3445,  16, {  6426,   6427,   6431,   6430,   6434,   6435,   6439,   6438}},
 {  3446,  16, {  6427,   6428,   6432,   6431,   6435,   6436,   6440,   6439}},
 {  3447,  16, {  6428,   6429,   6433,   6432,   6436,   6437,   6441,   6440}},
 {  3448,  16, {  6430,   6431,   3647,   3646,   6438,   6439,   3655,   3654}},
 {  3449,  16, {  6431,   6432,   3648,   3647,   6439,   6440,   3656,   3655}},
 {  3450,  16, {  6432,   6433,   3649,   3648,   6440,   6441,   3657,   3656}},
 {  3451,  16, {  6434,   6435,   6439,   6438,   6442,   6443,   6447,   6446}},
 {  3452,  16, {  6435,   6436,   6440,   6439,   6443,   6444,   6448,   6447}},
 {  3453,  16, {  6436,   6437,   6441,   6440,   6444,   6445,   6449,   6448}},
 {  3454,  16, {  6438,   6439,   3655,   3654,   6446,   6447,   3663,   3662}},
 {  3455,  16, {  6439,   6440,   3656,   3655,   6447,   6448,   3664,   3663}},
 {  3456,  16, {  6440,   6441,   3657,   3656,   6448,   6449,   3665,   3664}},
 {  3457,  16, {  6442,   6443,   6447,   6446,   6450,   6451,   6455,   6454}},
 {  3458,  16, {  6443,   6444,   6448,   6447,   6451,   6452,   6456,   6455}},
 {  3459,  16, {  6444,   6445,   6449,   6448,   6452,   6453,   6457,   6456}},
 {  3460,  16, {  6446,   6447,   3663,   3662,   6454,   6455,   3671,   3670}},
 {  3461,  16, {  6447,   6448,   3664,   3663,   6455,   6456,   3672,   3671}},
 {  3462,  16, {  6448,   6449,   3665,   3664,   6456,   6457,   3673,   3672}},
 {  3463,  16, {  6450,   6451,   6455,   6454,   6458,   6459,   6463,   6462}},
 {  3464,  16, {  6451,   6452,   6456,   6455,   6459,   6460,   6464,   6463}},
 {  3465,  16, {  6452,   6453,   6457,   6456,   6460,   6461,   6465,   6464}},
 {  3466,  16, {  6454,   6455,   3671,   3670,   6462,   6463,   3679,   3678}},
 {  3467,  16, {  6455,   6456,   3672,   3671,   6463,   6464,   3680,   3679}},
 {  3468,  16, {  6456,   6457,   3673,   3672,   6464,   6465,   3681,   3680}},
 {  3469,  16, {  6458,   6459,   6463,   6462,   6466,   6467,   6471,   6470}},
 {  3470,  16, {  6459,   6460,   6464,   6463,   6467,   6468,   6472,   6471}},
 {  3471,  16, {  6460,   6461,   6465,   6464,   6468,   6469,   6473,   6472}},
 {  3472,  16, {  6462,   6463,   3679,   3678,   6470,   6471,   6475,   6474}},
 {  3473,  16, {  6463,   6464,   3680,   3679,   6471,   6472,   6476,   6475}},
 {  3474,  16, {  6464,   6465,   3681,   3680,   6472,   6473,   6477,   6476}},
 {  3475,  17, {  6481,   3696,   3762,   6482,   6478,   3692,   3758,   6479}},
 {  3476,  17, {  6482,   3762,   3764,   6483,   6479,   3758,   3760,   6480}},
 {  3477,  17, {  6484,   3700,   3766,   6485,   6481,   3696,   3762,   6482}},
 {  3478,  17, {  6485,   3766,   3768,   6486,   6482,   3762,   3764,   6483}},
 {  3479,  17, {  6490,   3708,   3774,   6491,   6487,   3704,   3770,   6488}},
 {  3480,  17, {  6491,   3774,   3776,   6492,   6488,   3770,   3772,   6489}},
 {  3481,  17, {  6493,   3712,   3778,   6494,   6490,   3708,   3774,   6491}},
 {  3482,  17, {  6494,   3778,   3780,   6495,   6491,   3774,   3776,   6492}},
 {  3483,  17, {  6496,   3716,   3782,   6497,   6493,   3712,   3778,   6494}},
 {  3484,  17, {  6497,   3782,   3784,   6498,   6494,   3778,   3780,   6495}},
 {  3485,  17, {  6499,   3720,   3786,   6500,   6496,   3716,   3782,   6497}},
 {  3486,  17, {  6500,   3786,   3788,   6501,   6497,   3782,   3784,   6498}},
 {  3487,  17, {  6502,   3724,   3790,   6503,   6499,   3720,   3786,   6500}},
 {  3488,  17, {  6503,   3790,   3792,   6504,   6500,   3786,   3788,   6501}},
 {  3489,  17, {  6505,   3728,   3794,   6506,   6502,   3724,   3790,   6503}},
 {  3490,  17, {  6506,   3794,   3796,   6507,   6503,   3790,   3792,   6504}},
 {  3491,  17, {  6508,   3732,   3798,   6509,   6505,   3728,   3794,   6506}},
 {  3492,  17, {  6509,   3798,   3800,   6510,   6506,   3794,   3796,   6507}},
 {  3493,  17, {  6511,   3736,   3802,   6512,   6508,   3732,   3798,   6509}},
 {  3494,  17, {  6512,   3802,   3804,   6513,   6509,   3798,   3800,   6510}},
 {  3495,  17, {  6517,   3744,   3810,   6518,   6514,   3740,   3806,   6515}},
 {  3496,  17, {  6518,   3810,   3812,   6519,   6515,   3806,   3808,   6516}},
 {  3497,  17, {  6520,   3748,   3814,   6521,   6517,   3744,   3810,   6518}},
 {  3498,  17, {  6521,   3814,   3816,   6522,   6518,   3810,   3812,   6519}},
 {  3499,  17, {  6483,   3764,   3826,   6524,   6480,   3760,   3824,   6523}},
 {  3500,  17, {  6486,   3768,   3828,   6525,   6483,   3764,   3826,   6524}},
 {  3501,  17, {  6492,   3776,   3832,   6527,   6489,   3772,   3830,   6526}},
 {  3502,  17, {  6495,   3780,   3834,   6528,   6492,   3776,   3832,   6527}},
 {  3503,  17, {  6498,   3784,   3836,   6529,   6495,   3780,   3834,   6528}},
 {  3504,  17, {  6501,   3788,   3838,   6530,   6498,   3784,   3836,   6529}},
 {  3505,  17, {  6504,   3792,   3840,   6531,   6501,   3788,   3838,   6530}},
 {  3506,  17, {  6507,   3796,   3842,   6532,   6504,   3792,   3840,   6531}},
 {  3507,  17, {  6510,   3800,   3844,   6533,   6507,   3796,   3842,   6532}},
 {  3508,  17, {  6513,   3804,   3846,   6534,   6510,   3800,   3844,   6533}},
 {  3509,  17, {  6519,   3812,   3850,   6536,   6516,   3808,   3848,   6535}},
 {  3510,  17, {  6522,   3816,   3852,   6537,   6519,   3812,   3850,   6536}},
 {  3511,  17, {  6524,   3826,   3864,   6540,   6523,   3824,   3860,   6538}},
 {  3512,  17, {  6540,   3864,   3866,   6541,   6538,   3860,   3862,   6539}},
 {  3513,  17, {  6525,   3828,   3868,   6542,   6524,   3826,   3864,   6540}},
 {  3514,  17, {  6542,   3868,   3870,   6543,   6540,   3864,   3866,   6541}},
 {  3515,  17, {  6527,   3832,   3876,   6546,   6526,   3830,   3872,   6544}},
 {  3516,  17, {  6546,   3876,   3878,   6547,   6544,   3872,   3874,   6545}},
 {  3517,  17, {  6528,   3834,   3880,   6548,   6527,   3832,   3876,   6546}},
 {  3518,  17, {  6548,   3880,   3882,   6549,   6546,   3876,   3878,   6547}},
 {  3519,  17, {  6529,   3836,   3884,   6550,   6528,   3834,   3880,   6548}},
 {  3520,  17, {  6550,   3884,   3886,   6551,   6548,   3880,   3882,   6549}},
 {  3521,  17, {  6530,   3838,   3888,   6552,   6529,   3836,   3884,   6550}},
 {  3522,  17, {  6552,   3888,   3890,   6553,   6550,   3884,   3886,   6551}},
 {  3523,  17, {  6531,   3840,   3892,   6554,   6530,   3838,   3888,   6552}},
 {  3524,  17, {  6554,   3892,   3894,   6555,   6552,   3888,   3890,   6553}},
 {  3525,  17, {  6532,   3842,   3896,   6556,   6531,   3840,   3892,   6554}},
 {  3526,  17, {  6556,   3896,   3898,   6557,   6554,   3892,   3894,   6555}},
 {  3527,  17, {  6533,   3844,   3900,   6558,   6532,   3842,   3896,   6556}},
 {  3528,  17, {  6558,   3900,   3902,   6559,   6556,   3896,   3898,   6557}},
 {  3529,  17, {  6534,   3846,   3904,   6560,   6533,   3844,   3900,   6558}},
 {  3530,  17, {  6560,   3904,   3906,   6561,   6558,   3900,   3902,   6559}},
 {  3531,  17, {  6536,   3850,   3912,   6564,   6535,   3848,   3908,   6562}},
 {  3532,  17, {  6564,   3912,   3914,   6565,   6562,   3908,   3910,   6563}},
 {  3533,  17, {  6537,   3852,   3916,   6566,   6536,   3850,   3912,   6564}},
 {  3534,  17, {  6566,   3916,   3918,   6567,   6564,   3912,   3914,   6565}},
 {  3535,  18, {  3691,   6572,   6574,   3693,   3687,   6568,   6570,   3689}},
 {  3536,  18, {  6572,   6573,   6575,   6574,   6568,   6569,   6571,   6570}},
 {  3537,  18, {  3695,   6576,   6578,   3697,   3691,   6572,   6574,   3693}},
 {  3538,  18, {  6576,   6577,   6579,   6578,   6572,   6573,   6575,   6574}},
 {  3539,  18, {  3699,   6580,   6582,   3701,   3695,   6576,   6578,   3697}},
 {  3540,  18, {  6580,   6581,   6583,   6582,   6576,   6577,   6579,   6578}},
 {  3541,  18, {  3703,   6584,   6586,   3705,   3699,   6580,   6582,   3701}},
 {  3542,  18, {  6584,   6585,   6587,   6586,   6580,   6581,   6583,   6582}},
 {  3543,  18, {  3707,   6588,   6590,   3709,   3703,   6584,   6586,   3705}},
 {  3544,  18, {  6588,   6589,   6591,   6590,   6584,   6585,   6587,   6586}},
 {  3545,  18, {  3711,   6592,   6594,   3713,   3707,   6588,   6590,   3709}},
 {  3546,  18, {  6592,   6593,   6595,   6594,   6588,   6589,   6591,   6590}},
 {  3547,  18, {  3715,   6596,   6598,   3717,   3711,   6592,   6594,   3713}},
 {  3548,  18, {  6596,   6597,   6599,   6598,   6592,   6593,   6595,   6594}},
 {  3549,  18, {  3719,   6600,   6602,   3721,   3715,   6596,   6598,   3717}},
 {  3550,  18, {  6600,   6601,   6603,   6602,   6596,   6597,   6599,   6598}},
 {  3551,  18, {  3723,   6604,   6606,   3725,   3719,   6600,   6602,   3721}},
 {  3552,  18, {  6604,   6605,   6607,   6606,   6600,   6601,   6603,   6602}},
 {  3553,  18, {  3727,   6608,   6610,   3729,   3723,   6604,   6606,   3725}},
 {  3554,  18, {  6608,   6609,   6611,   6610,   6604,   6605,   6607,   6606}},
 {  3555,  18, {  3731,   6612,   6614,   3733,   3727,   6608,   6610,   3729}},
 {  3556,  18, {  6612,   6613,   6615,   6614,   6608,   6609,   6611,   6610}},
 {  3557,  18, {  3735,   6616,   6618,   3737,   3731,   6612,   6614,   3733}},
 {  3558,  18, {  6616,   6617,   6619,   6618,   6612,   6613,   6615,   6614}},
 {  3559,  18, {  3739,   6620,   6622,   3741,   3735,   6616,   6618,   3737}},
 {  3560,  18, {  6620,   6621,   6623,   6622,   6616,   6617,   6619,   6618}},
 {  3561,  18, {  3743,   6624,   6626,   3745,   3739,   6620,   6622,   3741}},
 {  3562,  18, {  6624,   6625,   6627,   6626,   6620,   6621,   6623,   6622}},
 {  3563,  18, {  3747,   6628,   6630,   3749,   3743,   6624,   6626,   3745}},
 {  3564,  18, {  6628,   6629,   6631,   6630,   6624,   6625,   6627,   6626}},
 {  3565,  18, {  3751,   6632,   6634,   3753,   3747,   6628,   6630,   3749}},
 {  3566,  18, {  6632,   6633,   6635,   6634,   6628,   6629,   6631,   6630}},
 {  3567,  18, {  3693,   6574,   6640,   3759,   3689,   6570,   6636,   3755}},
 {  3568,  18, {  6574,   6575,   6641,   6640,   6570,   6571,   6637,   6636}},
 {  3569,  18, {  3759,   6640,   6642,   3761,   3755,   6636,   6638,   3757}},
 {  3570,  18, {  6640,   6641,   6643,   6642,   6636,   6637,   6639,   6638}},
 {  3571,  18, {  3697,   6578,   6644,   3763,   3693,   6574,   6640,   3759}},
 {  3572,  18, {  6578,   6579,   6645,   6644,   6574,   6575,   6641,   6640}},
 {  3573,  18, {  3763,   6644,   6646,   3765,   3759,   6640,   6642,   3761}},
 {  3574,  18, {  6644,   6645,   6647,   6646,   6640,   6641,   6643,   6642}},
 {  3575,  18, {  3701,   6582,   6648,   3767,   3697,   6578,   6644,   3763}},
 {  3576,  18, {  6582,   6583,   6649,   6648,   6578,   6579,   6645,   6644}},
 {  3577,  18, {  3767,   6648,   6650,   3769,   3763,   6644,   6646,   3765}},
 {  3578,  18, {  6648,   6649,   6651,   6650,   6644,   6645,   6647,   6646}},
 {  3579,  18, {  3705,   6586,   6652,   3771,   3701,   6582,   6648,   3767}},
 {  3580,  18, {  6586,   6587,   6653,   6652,   6582,   6583,   6649,   6648}},
 {  3581,  18, {  3771,   6652,   6654,   3773,   3767,   6648,   6650,   3769}},
 {  3582,  18, {  6652,   6653,   6655,   6654,   6648,   6649,   6651,   6650}},
 {  3583,  18, {  3709,   6590,   6656,   3775,   3705,   6586,   6652,   3771}},
 {  3584,  18, {  6590,   6591,   6657,   6656,   6586,   6587,   6653,   6652}},
 {  3585,  18, {  3775,   6656,   6658,   3777,   3771,   6652,   6654,   3773}},
 {  3586,  18, {  6656,   6657,   6659,   6658,   6652,   6653,   6655,   6654}},
 {  3587,  18, {  3713,   6594,   6660,   3779,   3709,   6590,   6656,   3775}},
 {  3588,  18, {  6594,   6595,   6661,   6660,   6590,   6591,   6657,   6656}},
 {  3589,  18, {  3779,   6660,   6662,   3781,   3775,   6656,   6658,   3777}},
 {  3590,  18, {  6660,   6661,   6663,   6662,   6656,   6657,   6659,   6658}},
 {  3591,  18, {  3717,   6598,   6664,   3783,   3713,   6594,   6660,   3779}},
 {  3592,  18, {  6598,   6599,   6665,   6664,   6594,   6595,   6661,   6660}},
 {  3593,  18, {  3783,   6664,   6666,   3785,   3779,   6660,   6662,   3781}},
 {  3594,  18, {  6664,   6665,   6667,   6666,   6660,   6661,   6663,   6662}},
 {  3595,  18, {  3721,   6602,   6668,   3787,   3717,   6598,   6664,   3783}},
 {  3596,  18, {  6602,   6603,   6669,   6668,   6598,   6599,   6665,   6664}},
 {  3597,  18, {  3787,   6668,   6670,   3789,   3783,   6664,   6666,   3785}},
 {  3598,  18, {  6668,   6669,   6671,   6670,   6664,   6665,   6667,   6666}},
 {  3599,  18, {  3725,   6606,   6672,   3791,   3721,   6602,   6668,   3787}},
 {  3600,  18, {  6606,   6607,   6673,   6672,   6602,   6603,   6669,   6668}},
 {  3601,  18, {  3791,   6672,   6674,   3793,   3787,   6668,   6670,   3789}},
 {  3602,  18, {  6672,   6673,   6675,   6674,   6668,   6669,   6671,   6670}},
 {  3603,  18, {  3729,   6610,   6676,   3795,   3725,   6606,   6672,   3791}},
 {  3604,  18, {  6610,   6611,   6677,   6676,   6606,   6607,   6673,   6672}},
 {  3605,  18, {  3795,   6676,   6678,   3797,   3791,   6672,   6674,   3793}},
 {  3606,  18, {  6676,   6677,   6679,   6678,   6672,   6673,   6675,   6674}},
 {  3607,  18, {  3733,   6614,   6680,   3799,   3729,   6610,   6676,   3795}},
 {  3608,  18, {  6614,   6615,   6681,   6680,   6610,   6611,   6677,   6676}},
 {  3609,  18, {  3799,   6680,   6682,   3801,   3795,   6676,   6678,   3797}},
 {  3610,  18, {  6680,   6681,   6683,   6682,   6676,   6677,   6679,   6678}},
 {  3611,  18, {  3737,   6618,   6684,   3803,   3733,   6614,   6680,   3799}},
 {  3612,  18, {  6618,   6619,   6685,   6684,   6614,   6615,   6681,   6680}},
 {  3613,  18, {  3803,   6684,   6686,   3805,   3799,   6680,   6682,   3801}},
 {  3614,  18, {  6684,   6685,   6687,   6686,   6680,   6681,   6683,   6682}},
 {  3615,  18, {  3741,   6622,   6688,   3807,   3737,   6618,   6684,   3803}},
 {  3616,  18, {  6622,   6623,   6689,   6688,   6618,   6619,   6685,   6684}},
 {  3617,  18, {  3807,   6688,   6690,   3809,   3803,   6684,   6686,   3805}},
 {  3618,  18, {  6688,   6689,   6691,   6690,   6684,   6685,   6687,   6686}},
 {  3619,  18, {  3745,   6626,   6692,   3811,   3741,   6622,   6688,   3807}},
 {  3620,  18, {  6626,   6627,   6693,   6692,   6622,   6623,   6689,   6688}},
 {  3621,  18, {  3811,   6692,   6694,   3813,   3807,   6688,   6690,   3809}},
 {  3622,  18, {  6692,   6693,   6695,   6694,   6688,   6689,   6691,   6690}},
 {  3623,  18, {  3749,   6630,   6696,   3815,   3745,   6626,   6692,   3811}},
 {  3624,  18, {  6630,   6631,   6697,   6696,   6626,   6627,   6693,   6692}},
 {  3625,  18, {  3815,   6696,   6698,   3817,   3811,   6692,   6694,   3813}},
 {  3626,  18, {  6696,   6697,   6699,   6698,   6692,   6693,   6695,   6694}},
 {  3627,  18, {  3753,   6634,   6700,   3819,   3749,   6630,   6696,   3815}},
 {  3628,  18, {  6634,   6635,   6701,   6700,   6630,   6631,   6697,   6696}},
 {  3629,  18, {  3819,   6700,   6702,   3821,   3815,   6696,   6698,   3817}},
 {  3630,  18, {  6700,   6701,   6703,   6702,   6696,   6697,   6699,   6698}},
 {  3631,  18, {  3761,   6642,   6706,   3825,   3757,   6638,   6704,   3823}},
 {  3632,  18, {  6642,   6643,   6707,   6706,   6638,   6639,   6705,   6704}},
 {  3633,  18, {  3765,   6646,   6708,   3827,   3761,   6642,   6706,   3825}},
 {  3634,  18, {  6646,   6647,   6709,   6708,   6642,   6643,   6707,   6706}},
 {  3635,  18, {  3769,   6650,   6710,   3829,   3765,   6646,   6708,   3827}},
 {  3636,  18, {  6650,   6651,   6711,   6710,   6646,   6647,   6709,   6708}},
 {  3637,  18, {  3773,   6654,   6712,   3831,   3769,   6650,   6710,   3829}},
 {  3638,  18, {  6654,   6655,   6713,   6712,   6650,   6651,   6711,   6710}},
 {  3639,  18, {  3777,   6658,   6714,   3833,   3773,   6654,   6712,   3831}},
 {  3640,  18, {  6658,   6659,   6715,   6714,   6654,   6655,   6713,   6712}},
 {  3641,  18, {  3781,   6662,   6716,   3835,   3777,   6658,   6714,   3833}},
 {  3642,  18, {  6662,   6663,   6717,   6716,   6658,   6659,   6715,   6714}},
 {  3643,  18, {  3785,   6666,   6718,   3837,   3781,   6662,   6716,   3835}},
 {  3644,  18, {  6666,   6667,   6719,   6718,   6662,   6663,   6717,   6716}},
 {  3645,  18, {  3789,   6670,   6720,   3839,   3785,   6666,   6718,   3837}},
 {  3646,  18, {  6670,   6671,   6721,   6720,   6666,   6667,   6719,   6718}},
 {  3647,  18, {  3793,   6674,   6722,   3841,   3789,   6670,   6720,   3839}},
 {  3648,  18, {  6674,   6675,   6723,   6722,   6670,   6671,   6721,   6720}},
 {  3649,  18, {  3797,   6678,   6724,   3843,   3793,   6674,   6722,   3841}},
 {  3650,  18, {  6678,   6679,   6725,   6724,   6674,   6675,   6723,   6722}},
 {  3651,  18, {  3801,   6682,   6726,   3845,   3797,   6678,   6724,   3843}},
 {  3652,  18, {  6682,   6683,   6727,   6726,   6678,   6679,   6725,   6724}},
 {  3653,  18, {  3805,   6686,   6728,   3847,   3801,   6682,   6726,   3845}},
 {  3654,  18, {  6686,   6687,   6729,   6728,   6682,   6683,   6727,   6726}},
 {  3655,  18, {  3809,   6690,   6730,   3849,   3805,   6686,   6728,   3847}},
 {  3656,  18, {  6690,   6691,   6731,   6730,   6686,   6687,   6729,   6728}},
 {  3657,  18, {  3813,   6694,   6732,   3851,   3809,   6690,   6730,   3849}},
 {  3658,  18, {  6694,   6695,   6733,   6732,   6690,   6691,   6731,   6730}},
 {  3659,  18, {  3817,   6698,   6734,   3853,   3813,   6694,   6732,   3851}},
 {  3660,  18, {  6698,   6699,   6735,   6734,   6694,   6695,   6733,   6732}},
 {  3661,  18, {  3821,   6702,   6736,   3855,   3817,   6698,   6734,   3853}},
 {  3662,  18, {  6702,   6703,   6737,   6736,   6698,   6699,   6735,   6734}},
 {  3663,  18, {  3825,   6706,   6742,   3861,   3823,   6704,   6738,   3857}},
 {  3664,  18, {  6706,   6707,   6743,   6742,   6704,   6705,   6739,   6738}},
 {  3665,  18, {  3861,   6742,   6744,   3863,   3857,   6738,   6740,   3859}},
 {  3666,  18, {  6742,   6743,   6745,   6744,   6738,   6739,   6741,   6740}},
 {  3667,  18, {  3827,   6708,   6746,   3865,   3825,   6706,   6742,   3861}},
 {  3668,  18, {  6708,   6709,   6747,   6746,   6706,   6707,   6743,   6742}},
 {  3669,  18, {  3865,   6746,   6748,   3867,   3861,   6742,   6744,   3863}},
 {  3670,  18, {  6746,   6747,   6749,   6748,   6742,   6743,   6745,   6744}},
 {  3671,  18, {  3829,   6710,   6750,   3869,   3827,   6708,   6746,   3865}},
 {  3672,  18, {  6710,   6711,   6751,   6750,   6708,   6709,   6747,   6746}},
 {  3673,  18, {  3869,   6750,   6752,   3871,   3865,   6746,   6748,   3867}},
 {  3674,  18, {  6750,   6751,   6753,   6752,   6746,   6747,   6749,   6748}},
 {  3675,  18, {  3831,   6712,   6754,   3873,   3829,   6710,   6750,   3869}},
 {  3676,  18, {  6712,   6713,   6755,   6754,   6710,   6711,   6751,   6750}},
 {  3677,  18, {  3873,   6754,   6756,   3875,   3869,   6750,   6752,   3871}},
 {  3678,  18, {  6754,   6755,   6757,   6756,   6750,   6751,   6753,   6752}},
 {  3679,  18, {  3833,   6714,   6758,   3877,   3831,   6712,   6754,   3873}},
 {  3680,  18, {  6714,   6715,   6759,   6758,   6712,   6713,   6755,   6754}},
 {  3681,  18, {  3877,   6758,   6760,   3879,   3873,   6754,   6756,   3875}},
 {  3682,  18, {  6758,   6759,   6761,   6760,   6754,   6755,   6757,   6756}},
 {  3683,  18, {  3835,   6716,   6762,   3881,   3833,   6714,   6758,   3877}},
 {  3684,  18, {  6716,   6717,   6763,   6762,   6714,   6715,   6759,   6758}},
 {  3685,  18, {  3881,   6762,   6764,   3883,   3877,   6758,   6760,   3879}},
 {  3686,  18, {  6762,   6763,   6765,   6764,   6758,   6759,   6761,   6760}},
 {  3687,  18, {  3837,   6718,   6766,   3885,   3835,   6716,   6762,   3881}},
 {  3688,  18, {  6718,   6719,   6767,   6766,   6716,   6717,   6763,   6762}},
 {  3689,  18, {  3885,   6766,   6768,   3887,   3881,   6762,   6764,   3883}},
 {  3690,  18, {  6766,   6767,   6769,   6768,   6762,   6763,   6765,   6764}},
 {  3691,  18, {  3839,   6720,   6770,   3889,   3837,   6718,   6766,   3885}},
 {  3692,  18, {  6720,   6721,   6771,   6770,   6718,   6719,   6767,   6766}},
 {  3693,  18, {  3889,   6770,   6772,   3891,   3885,   6766,   6768,   3887}},
 {  3694,  18, {  6770,   6771,   6773,   6772,   6766,   6767,   6769,   6768}},
 {  3695,  18, {  3841,   6722,   6774,   3893,   3839,   6720,   6770,   3889}},
 {  3696,  18, {  6722,   6723,   6775,   6774,   6720,   6721,   6771,   6770}},
 {  3697,  18, {  3893,   6774,   6776,   3895,   3889,   6770,   6772,   3891}},
 {  3698,  18, {  6774,   6775,   6777,   6776,   6770,   6771,   6773,   6772}},
 {  3699,  18, {  3843,   6724,   6778,   3897,   3841,   6722,   6774,   3893}},
 {  3700,  18, {  6724,   6725,   6779,   6778,   6722,   6723,   6775,   6774}},
 {  3701,  18, {  3897,   6778,   6780,   3899,   3893,   6774,   6776,   3895}},
 {  3702,  18, {  6778,   6779,   6781,   6780,   6774,   6775,   6777,   6776}},
 {  3703,  18, {  3845,   6726,   6782,   3901,   3843,   6724,   6778,   3897}},
 {  3704,  18, {  6726,   6727,   6783,   6782,   6724,   6725,   6779,   6778}},
 {  3705,  18, {  3901,   6782,   6784,   3903,   3897,   6778,   6780,   3899}},
 {  3706,  18, {  6782,   6783,   6785,   6784,   6778,   6779,   6781,   6780}},
 {  3707,  18, {  3847,   6728,   6786,   3905,   3845,   6726,   6782,   3901}},
 {  3708,  18, {  6728,   6729,   6787,   6786,   6726,   6727,   6783,   6782}},
 {  3709,  18, {  3905,   6786,   6788,   3907,   3901,   6782,   6784,   3903}},
 {  3710,  18, {  6786,   6787,   6789,   6788,   6782,   6783,   6785,   6784}},
 {  3711,  18, {  3849,   6730,   6790,   3909,   3847,   6728,   6786,   3905}},
 {  3712,  18, {  6730,   6731,   6791,   6790,   6728,   6729,   6787,   6786}},
 {  3713,  18, {  3909,   6790,   6792,   3911,   3905,   6786,   6788,   3907}},
 {  3714,  18, {  6790,   6791,   6793,   6792,   6786,   6787,   6789,   6788}},
 {  3715,  18, {  3851,   6732,   6794,   3913,   3849,   6730,   6790,   3909}},
 {  3716,  18, {  6732,   6733,   6795,   6794,   6730,   6731,   6791,   6790}},
 {  3717,  18, {  3913,   6794,   6796,   3915,   3909,   6790,   6792,   3911}},
 {  3718,  18, {  6794,   6795,   6797,   6796,   6790,   6791,   6793,   6792}},
 {  3719,  18, {  3853,   6734,   6798,   3917,   3851,   6732,   6794,   3913}},
 {  3720,  18, {  6734,   6735,   6799,   6798,   6732,   6733,   6795,   6794}},
 {  3721,  18, {  3917,   6798,   6800,   3919,   3913,   6794,   6796,   3915}},
 {  3722,  18, {  6798,   6799,   6801,   6800,   6794,   6795,   6797,   6796}},
 {  3723,  18, {  3855,   6736,   6802,   3921,   3853,   6734,   6798,   3917}},
 {  3724,  18, {  6736,   6737,   6803,   6802,   6734,   6735,   6799,   6798}},
 {  3725,  18, {  3921,   6802,   6804,   3923,   3917,   6798,   6800,   3919}},
 {  3726,  18, {  6802,   6803,   6805,   6804,   6798,   6799,   6801,   6800}},
 {  3727,  18, {  3863,   6744,   6808,   3927,   3859,   6740,   6806,   3925}},
 {  3728,  18, {  6744,   6745,   6809,   6808,   6740,   6741,   6807,   6806}},
 {  3729,  18, {  3867,   6748,   6810,   3929,   3863,   6744,   6808,   3927}},
 {  3730,  18, {  6748,   6749,   6811,   6810,   6744,   6745,   6809,   6808}},
 {  3731,  18, {  3871,   6752,   6812,   3931,   3867,   6748,   6810,   3929}},
 {  3732,  18, {  6752,   6753,   6813,   6812,   6748,   6749,   6811,   6810}},
 {  3733,  18, {  3875,   6756,   6814,   3933,   3871,   6752,   6812,   3931}},
 {  3734,  18, {  6756,   6757,   6815,   6814,   6752,   6753,   6813,   6812}},
 {  3735,  18, {  3879,   6760,   6816,   3935,   3875,   6756,   6814,   3933}},
 {  3736,  18, {  6760,   6761,   6817,   6816,   6756,   6757,   6815,   6814}},
 {  3737,  18, {  3883,   6764,   6818,   3937,   3879,   6760,   6816,   3935}},
 {  3738,  18, {  6764,   6765,   6819,   6818,   6760,   6761,   6817,   6816}},
 {  3739,  18, {  3887,   6768,   6820,   3939,   3883,   6764,   6818,   3937}},
 {  3740,  18, {  6768,   6769,   6821,   6820,   6764,   6765,   6819,   6818}},
 {  3741,  18, {  3891,   6772,   6822,   3941,   3887,   6768,   6820,   3939}},
 {  3742,  18, {  6772,   6773,   6823,   6822,   6768,   6769,   6821,   6820}},
 {  3743,  18, {  3895,   6776,   6824,   3943,   3891,   6772,   6822,   3941}},
 {  3744,  18, {  6776,   6777,   6825,   6824,   6772,   6773,   6823,   6822}},
 {  3745,  18, {  3899,   6780,   6826,   3945,   3895,   6776,   6824,   3943}},
 {  3746,  18, {  6780,   6781,   6827,   6826,   6776,   6777,   6825,   6824}},
 {  3747,  18, {  3903,   6784,   6828,   3947,   3899,   6780,   6826,   3945}},
 {  3748,  18, {  6784,   6785,   6829,   6828,   6780,   6781,   6827,   6826}},
 {  3749,  18, {  3907,   6788,   6830,   3949,   3903,   6784,   6828,   3947}},
 {  3750,  18, {  6788,   6789,   6831,   6830,   6784,   6785,   6829,   6828}},
 {  3751,  18, {  3911,   6792,   6832,   3951,   3907,   6788,   6830,   3949}},
 {  3752,  18, {  6792,   6793,   6833,   6832,   6788,   6789,   6831,   6830}},
 {  3753,  18, {  3915,   6796,   6834,   3953,   3911,   6792,   6832,   3951}},
 {  3754,  18, {  6796,   6797,   6835,   6834,   6792,   6793,   6833,   6832}},
 {  3755,  18, {  3919,   6800,   6836,   3955,   3915,   6796,   6834,   3953}},
 {  3756,  18, {  6800,   6801,   6837,   6836,   6796,   6797,   6835,   6834}},
 {  3757,  18, {  3923,   6804,   6838,   3957,   3919,   6800,   6836,   3955}},
 {  3758,  18, {  6804,   6805,   6839,   6838,   6800,   6801,   6837,   6836}},
 {  3759,  19, {  6848,   6849,   6851,   6850,   6840,   6841,   6843,   6842}},
 {  3760,  19, {  6850,   6851,   6853,   6852,   6842,   6843,   6845,   6844}},
 {  3761,  19, {  6852,   6853,   6855,   6854,   6844,   6845,   6847,   6846}},
 {  3762,  19, {  6856,   6857,   6859,   6858,   6848,   6849,   6851,   6850}},
 {  3763,  19, {  6858,   6859,   6861,   6860,   6850,   6851,   6853,   6852}},
 {  3764,  19, {  6860,   6861,   6863,   6862,   6852,   6853,   6855,   6854}},
 {  3765,  19, {  6864,   6865,   6867,   6866,   6856,   6857,   6859,   6858}},
 {  3766,  19, {  6866,   6867,   6869,   6868,   6858,   6859,   6861,   6860}},
 {  3767,  19, {  6868,   6869,   6871,   6870,   6860,   6861,   6863,   6862}},
 {  3768,  19, {  6854,   6855,   6879,   6878,   6846,   6847,   6873,   6872}},
 {  3769,  19, {  6878,   6879,   6881,   6880,   6872,   6873,   6875,   6874}},
 {  3770,  19, {  6880,   6881,   6883,   6882,   6874,   6875,   6877,   6876}},
 {  3771,  19, {  6862,   6863,   6885,   6884,   6854,   6855,   6879,   6878}},
 {  3772,  19, {  6884,   6885,   6887,   6886,   6878,   6879,   6881,   6880}},
 {  3773,  19, {  6886,   6887,   6889,   6888,   6880,   6881,   6883,   6882}},
 {  3774,  19, {  6870,   6871,   6891,   6890,   6862,   6863,   6885,   6884}},
 {  3775,  19, {  6890,   6891,   6893,   6892,   6884,   6885,   6887,   6886}},
 {  3776,  19, {  6892,   6893,   6895,   6894,   6886,   6887,   6889,   6888}},
 {  3777,  19, {  6848,   6902,   6905,   6849,   6840,   6896,   6899,   6841}},
 {  3778,  19, {  6902,   6903,   6906,   6905,   6896,   6897,   6900,   6899}},
 {  3779,  19, {  6903,   6904,   6907,   6906,   6897,   6898,   6901,   6900}},
 {  3780,  19, {  6856,   6908,   6911,   6857,   6848,   6902,   6905,   6849}},
 {  3781,  19, {  6908,   6909,   6912,   6911,   6902,   6903,   6906,   6905}},
 {  3782,  19, {  6909,   6910,   6913,   6912,   6903,   6904,   6907,   6906}},
 {  3783,  19, {  6864,   6914,   6917,   6865,   6856,   6908,   6911,   6857}},
 {  3784,  19, {  6914,   6915,   6918,   6917,   6908,   6909,   6912,   6911}},
 {  3785,  19, {  6915,   6916,   6919,   6918,   6909,   6910,   6913,   6912}},
 {  3786,  19, {  6864,   6914,   6929,   6866,   6865,   6917,   6920,   6867}},
 {  3787,  19, {  6914,   6915,   6930,   6929,   6917,   6918,   6921,   6920}},
 {  3788,  19, {  6915,   6916,   6931,   6930,   6918,   6919,   6922,   6921}},
 {  3789,  19, {  6866,   6929,   6932,   6868,   6867,   6920,   6923,   6869}},
 {  3790,  19, {  6929,   6930,   6933,   6932,   6920,   6921,   6924,   6923}},
 {  3791,  19, {  6930,   6931,   6934,   6933,   6921,   6922,   6925,   6924}},
 {  3792,  19, {  6868,   6932,   6935,   6870,   6869,   6923,   6926,   6871}},
 {  3793,  19, {  6932,   6933,   6936,   6935,   6923,   6924,   6927,   6926}},
 {  3794,  19, {  6933,   6934,   6937,   6936,   6924,   6925,   6928,   6927}},
 {  3795,  19, {  6870,   6935,   6947,   6890,   6871,   6926,   6938,   6891}},
 {  3796,  19, {  6935,   6936,   6948,   6947,   6926,   6927,   6939,   6938}},
 {  3797,  19, {  6936,   6937,   6949,   6948,   6927,   6928,   6940,   6939}},
 {  3798,  19, {  6890,   6947,   6950,   6892,   6891,   6938,   6941,   6893}},
 {  3799,  19, {  6947,   6948,   6951,   6950,   6938,   6939,   6942,   6941}},
 {  3800,  19, {  6948,   6949,   6952,   6951,   6939,   6940,   6943,   6942}},
 {  3801,  19, {  6892,   6950,   6953,   6894,   6893,   6941,   6944,   6895}},
 {  3802,  19, {  6950,   6951,   6954,   6953,   6941,   6942,   6945,   6944}},
 {  3803,  19, {  6951,   6952,   6955,   6954,   6942,   6943,   6946,   6945}},
 {  3804,  19, {  6883,   6962,   6965,   6882,   6877,   6956,   6959,   6876}},
 {  3805,  19, {  6962,   6963,   6966,   6965,   6956,   6957,   6960,   6959}},
 {  3806,  19, {  6963,   6964,   6967,   6966,   6957,   6958,   6961,   6960}},
 {  3807,  19, {  6889,   6968,   6971,   6888,   6883,   6962,   6965,   6882}},
 {  3808,  19, {  6968,   6969,   6972,   6971,   6962,   6963,   6966,   6965}},
 {  3809,  19, {  6969,   6970,   6973,   6972,   6963,   6964,   6967,   6966}},
 {  3810,  19, {  6895,   6944,   6953,   6894,   6889,   6968,   6971,   6888}},
 {  3811,  19, {  6944,   6945,   6954,   6953,   6968,   6969,   6972,   6971}},
 {  3812,  19, {  6945,   6946,   6955,   6954,   6969,   6970,   6973,   6972}},
 {  3813,  19, {  6904,   6980,   6983,   6907,   6898,   6974,   6977,   6901}},
 {  3814,  19, {  6980,   6981,   6984,   6983,   6974,   6975,   6978,   6977}},
 {  3815,  19, {  6981,   6982,   6985,   6984,   6975,   6976,   6979,   6978}},
 {  3816,  19, {  6910,   6986,   6989,   6913,   6904,   6980,   6983,   6907}},
 {  3817,  19, {  6986,   6987,   6990,   6989,   6980,   6981,   6984,   6983}},
 {  3818,  19, {  6987,   6988,   6991,   6990,   6981,   6982,   6985,   6984}},
 {  3819,  19, {  6916,   6992,   6995,   6919,   6910,   6986,   6989,   6913}},
 {  3820,  19, {  6992,   6993,   6996,   6995,   6986,   6987,   6990,   6989}},
 {  3821,  19, {  6993,   6994,   6997,   6996,   6987,   6988,   6991,   6990}},
 {  3822,  19, {  6916,   6992,   7007,   6931,   6919,   6995,   6998,   6922}},
 {  3823,  19, {  6992,   6993,   7008,   7007,   6995,   6996,   6999,   6998}},
 {  3824,  19, {  6993,   6994,   7009,   7008,   6996,   6997,   7000,   6999}},
 {  3825,  19, {  6931,   7007,   7010,   6934,   6922,   6998,   7001,   6925}},
 {  3826,  19, {  7007,   7008,   7011,   7010,   6998,   6999,   7002,   7001}},
 {  3827,  19, {  7008,   7009,   7012,   7011,   6999,   7000,   7003,   7002}},
 {  3828,  19, {  6934,   7010,   7013,   6937,   6925,   7001,   7004,   6928}},
 {  3829,  19, {  7010,   7011,   7014,   7013,   7001,   7002,   7005,   7004}},
 {  3830,  19, {  7011,   7012,   7015,   7014,   7002,   7003,   7006,   7005}},
 {  3831,  19, {  6937,   7013,   7025,   6949,   6928,   7004,   7016,   6940}},
 {  3832,  19, {  7013,   7014,   7026,   7025,   7004,   7005,   7017,   7016}},
 {  3833,  19, {  7014,   7015,   7027,   7026,   7005,   7006,   7018,   7017}},
 {  3834,  19, {  6949,   7025,   7028,   6952,   6940,   7016,   7019,   6943}},
 {  3835,  19, {  7025,   7026,   7029,   7028,   7016,   7017,   7020,   7019}},
 {  3836,  19, {  7026,   7027,   7030,   7029,   7017,   7018,   7021,   7020}},
 {  3837,  19, {  6952,   7028,   7031,   6955,   6943,   7019,   7022,   6946}},
 {  3838,  19, {  7028,   7029,   7032,   7031,   7019,   7020,   7023,   7022}},
 {  3839,  19, {  7029,   7030,   7033,   7032,   7020,   7021,   7024,   7023}},
 {  3840,  19, {  6964,   7040,   7043,   6967,   6958,   7034,   7037,   6961}},
 {  3841,  19, {  7040,   7041,   7044,   7043,   7034,   7035,   7038,   7037}},
 {  3842,  19, {  7041,   7042,   7045,   7044,   7035,   7036,   7039,   7038}},
 {  3843,  19, {  6970,   7046,   7049,   6973,   6964,   7040,   7043,   6967}},
 {  3844,  19, {  7046,   7047,   7050,   7049,   7040,   7041,   7044,   7043}},
 {  3845,  19, {  7047,   7048,   7051,   7050,   7041,   7042,   7045,   7044}},
 {  3846,  19, {  6946,   7022,   7031,   6955,   6970,   7046,   7049,   6973}},
 {  3847,  19, {  7022,   7023,   7032,   7031,   7046,   7047,   7050,   7049}},
 {  3848,  19, {  7023,   7024,   7033,   7032,   7047,   7048,   7051,   7050}},
 {  3849,  19, {  6985,   6982,   7059,   7058,   6979,   6976,   7053,   7052}},
 {  3850,  19, {  7058,   7059,   7061,   7060,   7052,   7053,   7055,   7054}},
 {  3851,  19, {  7060,   7061,   7063,   7062,   7054,   7055,   7057,   7056}},
 {  3852,  19, {  6991,   6988,   7065,   7064,   6985,   6982,   7059,   7058}},
 {  3853,  19, {  7064,   7065,   7067,   7066,   7058,   7059,   7061,   7060}},
 {  3854,  19, {  7066,   7067,   7069,   7068,   7060,   7061,   7063,   7062}},
 {  3855,  19, {  6997,   6994,   7009,   7000,   6991,   6988,   7065,   7064}},
 {  3856,  19, {  7000,   7009,   7012,   7003,   7064,   7065,   7067,   7066}},
 {  3857,  19, {  7003,   7012,   7015,   7006,   7066,   7067,   7069,   7068}},
 {  3858,  19, {  7062,   7063,   7075,   7074,   7056,   7057,   7071,   7070}},
 {  3859,  19, {  7074,   7075,   7077,   7076,   7070,   7071,   7073,   7072}},
 {  3860,  19, {  7076,   7077,   7045,   7042,   7072,   7073,   7039,   7036}},
 {  3861,  19, {  7068,   7069,   7079,   7078,   7062,   7063,   7075,   7074}},
 {  3862,  19, {  7078,   7079,   7081,   7080,   7074,   7075,   7077,   7076}},
 {  3863,  19, {  7080,   7081,   7051,   7048,   7076,   7077,   7045,   7042}},
 {  3864,  19, {  7006,   7015,   7027,   7018,   7068,   7069,   7079,   7078}},
 {  3865,  19, {  7018,   7027,   7030,   7021,   7078,   7079,   7081,   7080}},
 {  3866,  19, {  7021,   7030,   7033,   7024,   7080,   7081,   7051,   7048}},
 {  3867,  19, {  7086,   6848,   6850,   7087,   7082,   6840,   6842,   7083}},
 {  3868,  19, {  7087,   6850,   6852,   7088,   7083,   6842,   6844,   7084}},
 {  3869,  19, {  7088,   6852,   6854,   7089,   7084,   6844,   6846,   7085}},
 {  3870,  19, {  7090,   6856,   6858,   7091,   7086,   6848,   6850,   7087}},
 {  3871,  19, {  7091,   6858,   6860,   7092,   7087,   6850,   6852,   7088}},
 {  3872,  19, {  7092,   6860,   6862,   7093,   7088,   6852,   6854,   7089}},
 {  3873,  19, {  7094,   6864,   6866,   7095,   7090,   6856,   6858,   7091}},
 {  3874,  19, {  7095,   6866,   6868,   7096,   7091,   6858,   6860,   7092}},
 {  3875,  19, {  7096,   6868,   6870,   7097,   7092,   6860,   6862,   7093}},
 {  3876,  19, {  7089,   6854,   6878,   7101,   7085,   6846,   6872,   7098}},
 {  3877,  19, {  7101,   6878,   6880,   7102,   7098,   6872,   6874,   7099}},
 {  3878,  19, {  7102,   6880,   6882,   7103,   7099,   6874,   6876,   7100}},
 {  3879,  19, {  7093,   6862,   6884,   7104,   7089,   6854,   6878,   7101}},
 {  3880,  19, {  7104,   6884,   6886,   7105,   7101,   6878,   6880,   7102}},
 {  3881,  19, {  7105,   6886,   6888,   7106,   7102,   6880,   6882,   7103}},
 {  3882,  19, {  7097,   6870,   6890,   7107,   7093,   6862,   6884,   7104}},
 {  3883,  19, {  7107,   6890,   6892,   7108,   7104,   6884,   6886,   7105}},
 {  3884,  19, {  7108,   6892,   6894,   7109,   7105,   6886,   6888,   7106}},
 {  3885,  19, {  7086,   7113,   6902,   6848,   7082,   7110,   6896,   6840}},
 {  3886,  19, {  7113,   7114,   6903,   6902,   7110,   7111,   6897,   6896}},
 {  3887,  19, {  7114,   7115,   6904,   6903,   7111,   7112,   6898,   6897}},
 {  3888,  19, {  7090,   7116,   6908,   6856,   7086,   7113,   6902,   6848}},
 {  3889,  19, {  7116,   7117,   6909,   6908,   7113,   7114,   6903,   6902}},
 {  3890,  19, {  7117,   7118,   6910,   6909,   7114,   7115,   6904,   6903}},
 {  3891,  19, {  7094,   7119,   6914,   6864,   7090,   7116,   6908,   6856}},
 {  3892,  19, {  7119,   7120,   6915,   6914,   7116,   7117,   6909,   6908}},
 {  3893,  19, {  7120,   7121,   6916,   6915,   7117,   7118,   6910,   6909}},
 {  3894,  19, {  7094,   7119,   7122,   7095,   6864,   6914,   6929,   6866}},
 {  3895,  19, {  7119,   7120,   7123,   7122,   6914,   6915,   6930,   6929}},
 {  3896,  19, {  7120,   7121,   7124,   7123,   6915,   6916,   6931,   6930}},
 {  3897,  19, {  7095,   7122,   7125,   7096,   6866,   6929,   6932,   6868}},
 {  3898,  19, {  7122,   7123,   7126,   7125,   6929,   6930,   6933,   6932}},
 {  3899,  19, {  7123,   7124,   7127,   7126,   6930,   6931,   6934,   6933}},
 {  3900,  19, {  7096,   7125,   7128,   7097,   6868,   6932,   6935,   6870}},
 {  3901,  19, {  7125,   7126,   7129,   7128,   6932,   6933,   6936,   6935}},
 {  3902,  19, {  7126,   7127,   7130,   7129,   6933,   6934,   6937,   6936}},
 {  3903,  19, {  7097,   7128,   7131,   7107,   6870,   6935,   6947,   6890}},
 {  3904,  19, {  7128,   7129,   7132,   7131,   6935,   6936,   6948,   6947}},
 {  3905,  19, {  7129,   7130,   7133,   7132,   6936,   6937,   6949,   6948}},
 {  3906,  19, {  7107,   7131,   7134,   7108,   6890,   6947,   6950,   6892}},
 {  3907,  19, {  7131,   7132,   7135,   7134,   6947,   6948,   6951,   6950}},
 {  3908,  19, {  7132,   7133,   7136,   7135,   6948,   6949,   6952,   6951}},
 {  3909,  19, {  7108,   7134,   7137,   7109,   6892,   6950,   6953,   6894}},
 {  3910,  19, {  7134,   7135,   7138,   7137,   6950,   6951,   6954,   6953}},
 {  3911,  19, {  7135,   7136,   7139,   7138,   6951,   6952,   6955,   6954}},
 {  3912,  19, {  6882,   6965,   7143,   7103,   6876,   6959,   7140,   7100}},
 {  3913,  19, {  6965,   6966,   7144,   7143,   6959,   6960,   7141,   7140}},
 {  3914,  19, {  6966,   6967,   7145,   7144,   6960,   6961,   7142,   7141}},
 {  3915,  19, {  6888,   6971,   7146,   7106,   6882,   6965,   7143,   7103}},
 {  3916,  19, {  6971,   6972,   7147,   7146,   6965,   6966,   7144,   7143}},
 {  3917,  19, {  6972,   6973,   7148,   7147,   6966,   6967,   7145,   7144}},
 {  3918,  19, {  6894,   6953,   7137,   7109,   6888,   6971,   7146,   7106}},
 {  3919,  19, {  6953,   6954,   7138,   7137,   6971,   6972,   7147,   7146}},
 {  3920,  19, {  6954,   6955,   7139,   7138,   6972,   6973,   7148,   7147}},
 {  3921,  19, {  7115,   7152,   6980,   6904,   7112,   7149,   6974,   6898}},
 {  3922,  19, {  7152,   7153,   6981,   6980,   7149,   7150,   6975,   6974}},
 {  3923,  19, {  7153,   7154,   6982,   6981,   7150,   7151,   6976,   6975}},
 {  3924,  19, {  7118,   7155,   6986,   6910,   7115,   7152,   6980,   6904}},
 {  3925,  19, {  7155,   7156,   6987,   6986,   7152,   7153,   6981,   6980}},
 {  3926,  19, {  7156,   7157,   6988,   6987,   7153,   7154,   6982,   6981}},
 {  3927,  19, {  7121,   7158,   6992,   6916,   7118,   7155,   6986,   6910}},
 {  3928,  19, {  7158,   7159,   6993,   6992,   7155,   7156,   6987,   6986}},
 {  3929,  19, {  7159,   7160,   6994,   6993,   7156,   7157,   6988,   6987}},
 {  3930,  19, {  7121,   7158,   7161,   7124,   6916,   6992,   7007,   6931}},
 {  3931,  19, {  7158,   7159,   7162,   7161,   6992,   6993,   7008,   7007}},
 {  3932,  19, {  7159,   7160,   7163,   7162,   6993,   6994,   7009,   7008}},
 {  3933,  19, {  7124,   7161,   7164,   7127,   6931,   7007,   7010,   6934}},
 {  3934,  19, {  7161,   7162,   7165,   7164,   7007,   7008,   7011,   7010}},
 {  3935,  19, {  7162,   7163,   7166,   7165,   7008,   7009,   7012,   7011}},
 {  3936,  19, {  7127,   7164,   7167,   7130,   6934,   7010,   7013,   6937}},
 {  3937,  19, {  7164,   7165,   7168,   7167,   7010,   7011,   7014,   7013}},
 {  3938,  19, {  7165,   7166,   7169,   7168,   7011,   7012,   7015,   7014}},
 {  3939,  19, {  7130,   7167,   7170,   7133,   6937,   7013,   7025,   6949}},
 {  3940,  19, {  7167,   7168,   7171,   7170,   7013,   7014,   7026,   7025}},
 {  3941,  19, {  7168,   7169,   7172,   7171,   7014,   7015,   7027,   7026}},
 {  3942,  19, {  7133,   7170,   7173,   7136,   6949,   7025,   7028,   6952}},
 {  3943,  19, {  7170,   7171,   7174,   7173,   7025,   7026,   7029,   7028}},
 {  3944,  19, {  7171,   7172,   7175,   7174,   7026,   7027,   7030,   7029}},
 {  3945,  19, {  7136,   7173,   7176,   7139,   6952,   7028,   7031,   6955}},
 {  3946,  19, {  7173,   7174,   7177,   7176,   7028,   7029,   7032,   7031}},
 {  3947,  19, {  7174,   7175,   7178,   7177,   7029,   7030,   7033,   7032}},
 {  3948,  19, {  6967,   7043,   7182,   7145,   6961,   7037,   7179,   7142}},
 {  3949,  19, {  7043,   7044,   7183,   7182,   7037,   7038,   7180,   7179}},
 {  3950,  19, {  7044,   7045,   7184,   7183,   7038,   7039,   7181,   7180}},
 {  3951,  19, {  6973,   7049,   7185,   7148,   6967,   7043,   7182,   7145}},
 {  3952,  19, {  7049,   7050,   7186,   7185,   7043,   7044,   7183,   7182}},
 {  3953,  19, {  7050,   7051,   7187,   7186,   7044,   7045,   7184,   7183}},
 {  3954,  19, {  6955,   7031,   7176,   7139,   6973,   7049,   7185,   7148}},
 {  3955,  19, {  7031,   7032,   7177,   7176,   7049,   7050,   7186,   7185}},
 {  3956,  19, {  7032,   7033,   7178,   7177,   7050,   7051,   7187,   7186}},
 {  3957,  19, {  6982,   7154,   7191,   7059,   6976,   7151,   7188,   7053}},
 {  3958,  19, {  7059,   7191,   7192,   7061,   7053,   7188,   7189,   7055}},
 {  3959,  19, {  7061,   7192,   7193,   7063,   7055,   7189,   7190,   7057}},
 {  3960,  19, {  6988,   7157,   7194,   7065,   6982,   7154,   7191,   7059}},
 {  3961,  19, {  7065,   7194,   7195,   7067,   7059,   7191,   7192,   7061}},
 {  3962,  19, {  7067,   7195,   7196,   7069,   7061,   7192,   7193,   7063}},
 {  3963,  19, {  6994,   7160,   7163,   7009,   6988,   7157,   7194,   7065}},
 {  3964,  19, {  7009,   7163,   7166,   7012,   7065,   7194,   7195,   7067}},
 {  3965,  19, {  7012,   7166,   7169,   7015,   7067,   7195,   7196,   7069}},
 {  3966,  19, {  7063,   7193,   7199,   7075,   7057,   7190,   7197,   7071}},
 {  3967,  19, {  7075,   7199,   7200,   7077,   7071,   7197,   7198,   7073}},
 {  3968,  19, {  7077,   7200,   7184,   7045,   7073,   7198,   7181,   7039}},
 {  3969,  19, {  7069,   7196,   7201,   7079,   7063,   7193,   7199,   7075}},
 {  3970,  19, {  7079,   7201,   7202,   7081,   7075,   7199,   7200,   7077}},
 {  3971,  19, {  7081,   7202,   7187,   7051,   7077,   7200,   7184,   7045}},
 {  3972,  19, {  7015,   7169,   7172,   7027,   7069,   7196,   7201,   7079}},
 {  3973,  19, {  7027,   7172,   7175,   7030,   7079,   7201,   7202,   7081}},
 {  3974,  19, {  7030,   7175,   7178,   7033,   7081,   7202,   7187,   7051}},
 {  3975,  19, {  7211,   7212,   7214,   7213,   7203,   7204,   7206,   7205}},
 {  3976,  19, {  7213,   7214,   7216,   7215,   7205,   7206,   7208,   7207}},
 {  3977,  19, {  7215,   7216,   7218,   7217,   7207,   7208,   7210,   7209}},
 {  3978,  19, {  6840,   6841,   7220,   7219,   7211,   7212,   7214,   7213}},
 {  3979,  19, {  7219,   7220,   7222,   7221,   7213,   7214,   7216,   7215}},
 {  3980,  19, {  7221,   7222,   7224,   7223,   7215,   7216,   7218,   7217}},
 {  3981,  19, {  7217,   7218,   7232,   7231,   7209,   7210,   7226,   7225}},
 {  3982,  19, {  7231,   7232,   7234,   7233,   7225,   7226,   7228,   7227}},
 {  3983,  19, {  7233,   7234,   7236,   7235,   7227,   7228,   7230,   7229}},
 {  3984,  19, {  7223,   7224,   7238,   7237,   7217,   7218,   7232,   7231}},
 {  3985,  19, {  7237,   7238,   7240,   7239,   7231,   7232,   7234,   7233}},
 {  3986,  19, {  7239,   7240,   6877,   6876,   7233,   7234,   7236,   7235}},
 {  3987,  19, {  7211,   7247,   7250,   7212,   7203,   7241,   7244,   7204}},
 {  3988,  19, {  7247,   7248,   7251,   7250,   7241,   7242,   7245,   7244}},
 {  3989,  19, {  7248,   7249,   7252,   7251,   7242,   7243,   7246,   7245}},
 {  3990,  19, {  6840,   7253,   7256,   6841,   7211,   7247,   7250,   7212}},
 {  3991,  19, {  7253,   7254,   7257,   7256,   7247,   7248,   7251,   7250}},
 {  3992,  19, {  7254,   7255,   7258,   7257,   7248,   7249,   7252,   7251}},
 {  3993,  19, {  7236,   7265,   7268,   7235,   7230,   7259,   7262,   7229}},
 {  3994,  19, {  7265,   7266,   7269,   7268,   7259,   7260,   7263,   7262}},
 {  3995,  19, {  7266,   7267,   7270,   7269,   7260,   7261,   7264,   7263}},
 {  3996,  19, {  6877,   7271,   7274,   6876,   7236,   7265,   7268,   7235}},
 {  3997,  19, {  7271,   7272,   7275,   7274,   7265,   7266,   7269,   7268}},
 {  3998,  19, {  7272,   7273,   7276,   7275,   7266,   7267,   7270,   7269}},
 {  3999,  19, {  7249,   7283,   7286,   7252,   7243,   7277,   7280,   7246}},
 {  4000,  19, {  7283,   7284,   7287,   7286,   7277,   7278,   7281,   7280}},
 {  4001,  19, {  7284,   7285,   7288,   7287,   7278,   7279,   7282,   7281}},
 {  4002,  19, {  7255,   7289,   7291,   7258,   7249,   7283,   7286,   7252}},
 {  4003,  19, {  7289,   7290,   7292,   7291,   7283,   7284,   7287,   7286}},
 {  4004,  19, {  7290,   6976,   6979,   7292,   7284,   7285,   7288,   7287}},
 {  4005,  19, {  7267,   7299,   7302,   7270,   7261,   7293,   7296,   7264}},
 {  4006,  19, {  7299,   7300,   7303,   7302,   7293,   7294,   7297,   7296}},
 {  4007,  19, {  7300,   7301,   7304,   7303,   7294,   7295,   7298,   7297}},
 {  4008,  19, {  7273,   7305,   7307,   7276,   7267,   7299,   7302,   7270}},
 {  4009,  19, {  7305,   7306,   7308,   7307,   7299,   7300,   7303,   7302}},
 {  4010,  19, {  7306,   7036,   7039,   7308,   7300,   7301,   7304,   7303}},
 {  4011,  19, {  7288,   7285,   7316,   7315,   7282,   7279,   7310,   7309}},
 {  4012,  19, {  7315,   7316,   7318,   7317,   7309,   7310,   7312,   7311}},
 {  4013,  19, {  7317,   7318,   7320,   7319,   7311,   7312,   7314,   7313}},
 {  4014,  19, {  6979,   6976,   7322,   7321,   7288,   7285,   7316,   7315}},
 {  4015,  19, {  7321,   7322,   7324,   7323,   7315,   7316,   7318,   7317}},
 {  4016,  19, {  7323,   7324,   7326,   7325,   7317,   7318,   7320,   7319}},
 {  4017,  19, {  7319,   7320,   7332,   7331,   7313,   7314,   7328,   7327}},
 {  4018,  19, {  7331,   7332,   7334,   7333,   7327,   7328,   7330,   7329}},
 {  4019,  19, {  7333,   7334,   7304,   7301,   7329,   7330,   7298,   7295}},
 {  4020,  19, {  7325,   7326,   7336,   7335,   7319,   7320,   7332,   7331}},
 {  4021,  19, {  7335,   7336,   7338,   7337,   7331,   7332,   7334,   7333}},
 {  4022,  19, {  7337,   7338,   7039,   7036,   7333,   7334,   7304,   7301}},
 {  4023,  19, {  7343,   7211,   7213,   7344,   7339,   7203,   7205,   7340}},
 {  4024,  19, {  7344,   7213,   7215,   7345,   7340,   7205,   7207,   7341}},
 {  4025,  19, {  7345,   7215,   7217,   7346,   7341,   7207,   7209,   7342}},
 {  4026,  19, {  7082,   6840,   7219,   7347,   7343,   7211,   7213,   7344}},
 {  4027,  19, {  7347,   7219,   7221,   7348,   7344,   7213,   7215,   7345}},
 {  4028,  19, {  7348,   7221,   7223,   7349,   7345,   7215,   7217,   7346}},
 {  4029,  19, {  7346,   7217,   7231,   7353,   7342,   7209,   7225,   7350}},
 {  4030,  19, {  7353,   7231,   7233,   7354,   7350,   7225,   7227,   7351}},
 {  4031,  19, {  7354,   7233,   7235,   7355,   7351,   7227,   7229,   7352}},
 {  4032,  19, {  7349,   7223,   7237,   7356,   7346,   7217,   7231,   7353}},
 {  4033,  19, {  7356,   7237,   7239,   7357,   7353,   7231,   7233,   7354}},
 {  4034,  19, {  7357,   7239,   6876,   7100,   7354,   7233,   7235,   7355}},
 {  4035,  19, {  7343,   7361,   7247,   7211,   7339,   7358,   7241,   7203}},
 {  4036,  19, {  7361,   7362,   7248,   7247,   7358,   7359,   7242,   7241}},
 {  4037,  19, {  7362,   7363,   7249,   7248,   7359,   7360,   7243,   7242}},
 {  4038,  19, {  7082,   7364,   7253,   6840,   7343,   7361,   7247,   7211}},
 {  4039,  19, {  7364,   7365,   7254,   7253,   7361,   7362,   7248,   7247}},
 {  4040,  19, {  7365,   7366,   7255,   7254,   7362,   7363,   7249,   7248}},
 {  4041,  19, {  7235,   7268,   7370,   7355,   7229,   7262,   7367,   7352}},
 {  4042,  19, {  7268,   7269,   7371,   7370,   7262,   7263,   7368,   7367}},
 {  4043,  19, {  7269,   7270,   7372,   7371,   7263,   7264,   7369,   7368}},
 {  4044,  19, {  6876,   7274,   7373,   7100,   7235,   7268,   7370,   7355}},
 {  4045,  19, {  7274,   7275,   7374,   7373,   7268,   7269,   7371,   7370}},
 {  4046,  19, {  7275,   7276,   7375,   7374,   7269,   7270,   7372,   7371}},
 {  4047,  19, {  7363,   7379,   7283,   7249,   7360,   7376,   7277,   7243}},
 {  4048,  19, {  7379,   7380,   7284,   7283,   7376,   7377,   7278,   7277}},
 {  4049,  19, {  7380,   7381,   7285,   7284,   7377,   7378,   7279,   7278}},
 {  4050,  19, {  7366,   7382,   7289,   7255,   7363,   7379,   7283,   7249}},
 {  4051,  19, {  7382,   7383,   7290,   7289,   7379,   7380,   7284,   7283}},
 {  4052,  19, {  7383,   7151,   6976,   7290,   7380,   7381,   7285,   7284}},
 {  4053,  19, {  7270,   7302,   7387,   7372,   7264,   7296,   7384,   7369}},
 {  4054,  19, {  7302,   7303,   7388,   7387,   7296,   7297,   7385,   7384}},
 {  4055,  19, {  7303,   7304,   7389,   7388,   7297,   7298,   7386,   7385}},
 {  4056,  19, {  7276,   7307,   7390,   7375,   7270,   7302,   7387,   7372}},
 {  4057,  19, {  7307,   7308,   7391,   7390,   7302,   7303,   7388,   7387}},
 {  4058,  19, {  7308,   7039,   7181,   7391,   7303,   7304,   7389,   7388}},
 {  4059,  19, {  7285,   7381,   7395,   7316,   7279,   7378,   7392,   7310}},
 {  4060,  19, {  7316,   7395,   7396,   7318,   7310,   7392,   7393,   7312}},
 {  4061,  19, {  7318,   7396,   7397,   7320,   7312,   7393,   7394,   7314}},
 {  4062,  19, {  6976,   7151,   7398,   7322,   7285,   7381,   7395,   7316}},
 {  4063,  19, {  7322,   7398,   7399,   7324,   7316,   7395,   7396,   7318}},
 {  4064,  19, {  7324,   7399,   7400,   7326,   7318,   7396,   7397,   7320}},
 {  4065,  19, {  7320,   7397,   7403,   7332,   7314,   7394,   7401,   7328}},
 {  4066,  19, {  7332,   7403,   7404,   7334,   7328,   7401,   7402,   7330}},
 {  4067,  19, {  7334,   7404,   7389,   7304,   7330,   7402,   7386,   7298}},
 {  4068,  19, {  7326,   7400,   7405,   7336,   7320,   7397,   7403,   7332}},
 {  4069,  19, {  7336,   7405,   7406,   7338,   7332,   7403,   7404,   7334}},
 {  4070,  19, {  7338,   7406,   7181,   7039,   7334,   7404,   7389,   7304}},
 {  4071,  19, {  7411,   7412,   7414,   7413,   7204,   7407,   7408,   7206}},
 {  4072,  19, {  7413,   7414,   7416,   7415,   7206,   7408,   7409,   7208}},
 {  4073,  19, {  7415,   7416,   7418,   7417,   7208,   7409,   7410,   7210}},
 {  4074,  19, {  7417,   7418,   7423,   7422,   7210,   7410,   7419,   7226}},
 {  4075,  19, {  7422,   7423,   7425,   7424,   7226,   7419,   7420,   7228}},
 {  4076,  19, {  7424,   7425,   7427,   7426,   7228,   7420,   7421,   7230}},
 {  4077,  19, {  7411,   7431,   7434,   7412,   7204,   7244,   7428,   7407}},
 {  4078,  19, {  7431,   7432,   7435,   7434,   7244,   7245,   7429,   7428}},
 {  4079,  19, {  7432,   7433,   7436,   7435,   7245,   7246,   7430,   7429}},
 {  4080,  19, {  7412,   7434,   7446,   7414,   7407,   7428,   7437,   7408}},
 {  4081,  19, {  7434,   7435,   7447,   7446,   7428,   7429,   7438,   7437}},
 {  4082,  19, {  7435,   7436,   7448,   7447,   7429,   7430,   7439,   7438}},
 {  4083,  19, {  7414,   7446,   7449,   7416,   7408,   7437,   7440,   7409}},
 {  4084,  19, {  7446,   7447,   7450,   7449,   7437,   7438,   7441,   7440}},
 {  4085,  19, {  7447,   7448,   7451,   7450,   7438,   7439,   7442,   7441}},
 {  4086,  19, {  7416,   7449,   7452,   7418,   7409,   7440,   7443,   7410}},
 {  4087,  19, {  7449,   7450,   7453,   7452,   7440,   7441,   7444,   7443}},
 {  4088,  19, {  7450,   7451,   7454,   7453,   7441,   7442,   7445,   7444}},
 {  4089,  19, {  7418,   7452,   7464,   7423,   7410,   7443,   7455,   7419}},
 {  4090,  19, {  7452,   7453,   7465,   7464,   7443,   7444,   7456,   7455}},
 {  4091,  19, {  7453,   7454,   7466,   7465,   7444,   7445,   7457,   7456}},
 {  4092,  19, {  7423,   7464,   7467,   7425,   7419,   7455,   7458,   7420}},
 {  4093,  19, {  7464,   7465,   7468,   7467,   7455,   7456,   7459,   7458}},
 {  4094,  19, {  7465,   7466,   7469,   7468,   7456,   7457,   7460,   7459}},
 {  4095,  19, {  7425,   7467,   7470,   7427,   7420,   7458,   7461,   7421}},
 {  4096,  19, {  7467,   7468,   7471,   7470,   7458,   7459,   7462,   7461}},
 {  4097,  19, {  7468,   7469,   7472,   7471,   7459,   7460,   7463,   7462}},
 {  4098,  19, {  7427,   7470,   7473,   7426,   7421,   7461,   7259,   7230}},
 {  4099,  19, {  7470,   7471,   7474,   7473,   7461,   7462,   7260,   7259}},
 {  4100,  19, {  7471,   7472,   7475,   7474,   7462,   7463,   7261,   7260}},
 {  4101,  19, {  7433,   7479,   7482,   7436,   7246,   7280,   7476,   7430}},
 {  4102,  19, {  7479,   7480,   7483,   7482,   7280,   7281,   7477,   7476}},
 {  4103,  19, {  7480,   7481,   7484,   7483,   7281,   7282,   7478,   7477}},
 {  4104,  19, {  7436,   7482,   7494,   7448,   7430,   7476,   7485,   7439}},
 {  4105,  19, {  7482,   7483,   7495,   7494,   7476,   7477,   7486,   7485}},
 {  4106,  19, {  7483,   7484,   7496,   7495,   7477,   7478,   7487,   7486}},
 {  4107,  19, {  7448,   7494,   7497,   7451,   7439,   7485,   7488,   7442}},
 {  4108,  19, {  7494,   7495,   7498,   7497,   7485,   7486,   7489,   7488}},
 {  4109,  19, {  7495,   7496,   7499,   7498,   7486,   7487,   7490,   7489}},
 {  4110,  19, {  7451,   7497,   7500,   7454,   7442,   7488,   7491,   7445}},
 {  4111,  19, {  7497,   7498,   7501,   7500,   7488,   7489,   7492,   7491}},
 {  4112,  19, {  7498,   7499,   7502,   7501,   7489,   7490,   7493,   7492}},
 {  4113,  19, {  7454,   7500,   7512,   7466,   7445,   7491,   7503,   7457}},
 {  4114,  19, {  7500,   7501,   7513,   7512,   7491,   7492,   7504,   7503}},
 {  4115,  19, {  7501,   7502,   7514,   7513,   7492,   7493,   7505,   7504}},
 {  4116,  19, {  7466,   7512,   7515,   7469,   7457,   7503,   7506,   7460}},
 {  4117,  19, {  7512,   7513,   7516,   7515,   7503,   7504,   7507,   7506}},
 {  4118,  19, {  7513,   7514,   7517,   7516,   7504,   7505,   7508,   7507}},
 {  4119,  19, {  7469,   7515,   7518,   7472,   7460,   7506,   7509,   7463}},
 {  4120,  19, {  7515,   7516,   7519,   7518,   7506,   7507,   7510,   7509}},
 {  4121,  19, {  7516,   7517,   7520,   7519,   7507,   7508,   7511,   7510}},
 {  4122,  19, {  7472,   7518,   7521,   7475,   7463,   7509,   7293,   7261}},
 {  4123,  19, {  7518,   7519,   7522,   7521,   7509,   7510,   7294,   7293}},
 {  4124,  19, {  7519,   7520,   7523,   7522,   7510,   7511,   7295,   7294}},
 {  4125,  19, {  7484,   7481,   7527,   7496,   7478,   7282,   7524,   7487}},
 {  4126,  19, {  7496,   7527,   7528,   7499,   7487,   7524,   7525,   7490}},
 {  4127,  19, {  7499,   7528,   7529,   7502,   7490,   7525,   7526,   7493}},
 {  4128,  19, {  7502,   7529,   7532,   7514,   7493,   7526,   7530,   7505}},
 {  4129,  19, {  7514,   7532,   7533,   7517,   7505,   7530,   7531,   7508}},
 {  4130,  19, {  7517,   7533,   7523,   7520,   7508,   7531,   7295,   7511}},
 {  4131,  19, {  7560,   7561,   7563,   7562,   7534,   7535,   7537,   7536}},
 {  4132,  19, {  7562,   7563,   7565,   7564,   7536,   7537,   7539,   7538}},
 {  4133,  19, {  7564,   7565,   7567,   7566,   7538,   7539,   7541,   7540}},
 {  4134,  19, {  7566,   7567,   7569,   7568,   7540,   7541,   7543,   7542}},
 {  4135,  19, {  7568,   7569,   7571,   7570,   7542,   7543,   7545,   7544}},
 {  4136,  19, {  7570,   7571,   7573,   7572,   7544,   7545,   7547,   7546}},
 {  4137,  19, {  7572,   7573,   7575,   7574,   7546,   7547,   7549,   7548}},
 {  4138,  19, {  7574,   7575,   7577,   7576,   7548,   7549,   7551,   7550}},
 {  4139,  19, {  7576,   7577,   7579,   7578,   7550,   7551,   7553,   7552}},
 {  4140,  19, {  7578,   7579,   7581,   7580,   7552,   7553,   7555,   7554}},
 {  4141,  19, {  7580,   7581,   7583,   7582,   7554,   7555,   7557,   7556}},
 {  4142,  19, {  7582,   7583,   7585,   7584,   7556,   7557,   7559,   7558}},
 {  4143,  19, {  7586,   7587,   7589,   7588,   7560,   7561,   7563,   7562}},
 {  4144,  19, {  7588,   7589,   7591,   7590,   7562,   7563,   7565,   7564}},
 {  4145,  19, {  7590,   7591,   7593,   7592,   7564,   7565,   7567,   7566}},
 {  4146,  19, {  7592,   7593,   7595,   7594,   7566,   7567,   7569,   7568}},
 {  4147,  19, {  7594,   7595,   7597,   7596,   7568,   7569,   7571,   7570}},
 {  4148,  19, {  7596,   7597,   7599,   7598,   7570,   7571,   7573,   7572}},
 {  4149,  19, {  7598,   7599,   7601,   7600,   7572,   7573,   7575,   7574}},
 {  4150,  19, {  7600,   7601,   7603,   7602,   7574,   7575,   7577,   7576}},
 {  4151,  19, {  7602,   7603,   7605,   7604,   7576,   7577,   7579,   7578}},
 {  4152,  19, {  7604,   7605,   7607,   7606,   7578,   7579,   7581,   7580}},
 {  4153,  19, {  7606,   7607,   7609,   7608,   7580,   7581,   7583,   7582}},
 {  4154,  19, {  7608,   7609,   7611,   7610,   7582,   7583,   7585,   7584}},
 {  4155,  19, {  7612,   7613,   7615,   7614,   7586,   7587,   7589,   7588}},
 {  4156,  19, {  7614,   7615,   7617,   7616,   7588,   7589,   7591,   7590}},
 {  4157,  19, {  7616,   7617,   7619,   7618,   7590,   7591,   7593,   7592}},
 {  4158,  19, {  7618,   7619,   7621,   7620,   7592,   7593,   7595,   7594}},
 {  4159,  19, {  7620,   7621,   7623,   7622,   7594,   7595,   7597,   7596}},
 {  4160,  19, {  7622,   7623,   7625,   7624,   7596,   7597,   7599,   7598}},
 {  4161,  19, {  7624,   7625,   7627,   7626,   7598,   7599,   7601,   7600}},
 {  4162,  19, {  7626,   7627,   7629,   7628,   7600,   7601,   7603,   7602}},
 {  4163,  19, {  7628,   7629,   7631,   7630,   7602,   7603,   7605,   7604}},
 {  4164,  19, {  7630,   7631,   7633,   7632,   7604,   7605,   7607,   7606}},
 {  4165,  19, {  7632,   7633,   7635,   7634,   7606,   7607,   7609,   7608}},
 {  4166,  19, {  7634,   7635,   7637,   7636,   7608,   7609,   7611,   7610}},
 {  4167,  19, {  7638,   7639,   7641,   7640,   7612,   7613,   7615,   7614}},
 {  4168,  19, {  7640,   7641,   7643,   7642,   7614,   7615,   7617,   7616}},
 {  4169,  19, {  7642,   7643,   7645,   7644,   7616,   7617,   7619,   7618}},
 {  4170,  19, {  7644,   7645,   7647,   7646,   7618,   7619,   7621,   7620}},
 {  4171,  19, {  7646,   7647,   7649,   7648,   7620,   7621,   7623,   7622}},
 {  4172,  19, {  7648,   7649,   7651,   7650,   7622,   7623,   7625,   7624}},
 {  4173,  19, {  7650,   7651,   7653,   7652,   7624,   7625,   7627,   7626}},
 {  4174,  19, {  7652,   7653,   7655,   7654,   7626,   7627,   7629,   7628}},
 {  4175,  19, {  7654,   7655,   7657,   7656,   7628,   7629,   7631,   7630}},
 {  4176,  19, {  7656,   7657,   7659,   7658,   7630,   7631,   7633,   7632}},
 {  4177,  19, {  7658,   7659,   7661,   7660,   7632,   7633,   7635,   7634}},
 {  4178,  19, {  7660,   7661,   7663,   7662,   7634,   7635,   7637,   7636}},
 {  4179,  19, {  7664,   7665,   7667,   7666,   7638,   7639,   7641,   7640}},
 {  4180,  19, {  7666,   7667,   7669,   7668,   7640,   7641,   7643,   7642}},
 {  4181,  19, {  7668,   7669,   7671,   7670,   7642,   7643,   7645,   7644}},
 {  4182,  19, {  7670,   7671,   7673,   7672,   7644,   7645,   7647,   7646}},
 {  4183,  19, {  7672,   7673,   7675,   7674,   7646,   7647,   7649,   7648}},
 {  4184,  19, {  7674,   7675,   7677,   7676,   7648,   7649,   7651,   7650}},
 {  4185,  19, {  7676,   7677,   7679,   7678,   7650,   7651,   7653,   7652}},
 {  4186,  19, {  7678,   7679,   7681,   7680,   7652,   7653,   7655,   7654}},
 {  4187,  19, {  7680,   7681,   7683,   7682,   7654,   7655,   7657,   7656}},
 {  4188,  19, {  7682,   7683,   7685,   7684,   7656,   7657,   7659,   7658}},
 {  4189,  19, {  7684,   7685,   7687,   7686,   7658,   7659,   7661,   7660}},
 {  4190,  19, {  7686,   7687,   7689,   7688,   7660,   7661,   7663,   7662}},
 {  4191,  19, {  7690,   7691,   7693,   7692,   7664,   7665,   7667,   7666}},
 {  4192,  19, {  7692,   7693,   7695,   7694,   7666,   7667,   7669,   7668}},
 {  4193,  19, {  7694,   7695,   7697,   7696,   7668,   7669,   7671,   7670}},
 {  4194,  19, {  7696,   7697,   7699,   7698,   7670,   7671,   7673,   7672}},
 {  4195,  19, {  7698,   7699,   7701,   7700,   7672,   7673,   7675,   7674}},
 {  4196,  19, {  7700,   7701,   7526,   7702,   7674,   7675,   7677,   7676}},
 {  4197,  19, {  7702,   7526,   7704,   7703,   7676,   7677,   7679,   7678}},
 {  4198,  19, {  7703,   7704,   7706,   7705,   7678,   7679,   7681,   7680}},
 {  4199,  19, {  7705,   7706,   7708,   7707,   7680,   7681,   7683,   7682}},
 {  4200,  19, {  7707,   7708,   7710,   7709,   7682,   7683,   7685,   7684}},
 {  4201,  19, {  7709,   7710,   7712,   7711,   7684,   7685,   7687,   7686}},
 {  4202,  19, {  7711,   7712,   7714,   7713,   7686,   7687,   7689,   7688}},
 {  4203,  19, {  7584,   7585,   7738,   7737,   7558,   7559,   7716,   7715}},
 {  4204,  19, {  7737,   7738,   7740,   7739,   7715,   7716,   7718,   7717}},
 {  4205,  19, {  7739,   7740,   7742,   7741,   7717,   7718,   7720,   7719}},
 {  4206,  19, {  7741,   7742,   7744,   7743,   7719,   7720,   7722,   7721}},
 {  4207,  19, {  7743,   7744,   7746,   7745,   7721,   7722,   7724,   7723}},
 {  4208,  19, {  7745,   7746,   7748,   7747,   7723,   7724,   7726,   7725}},
 {  4209,  19, {  7747,   7748,   7750,   7749,   7725,   7726,   7728,   7727}},
 {  4210,  19, {  7749,   7750,   7752,   7751,   7727,   7728,   7730,   7729}},
 {  4211,  19, {  7751,   7752,   7754,   7753,   7729,   7730,   7732,   7731}},
 {  4212,  19, {  7753,   7754,   7756,   7755,   7731,   7732,   7734,   7733}},
 {  4213,  19, {  7755,   7756,   7758,   7757,   7733,   7734,   7736,   7735}},
 {  4214,  19, {  7757,   7758,   7561,   7560,   7735,   7736,   7535,   7534}},
 {  4215,  19, {  7610,   7611,   7760,   7759,   7584,   7585,   7738,   7737}},
 {  4216,  19, {  7759,   7760,   7762,   7761,   7737,   7738,   7740,   7739}},
 {  4217,  19, {  7761,   7762,   7764,   7763,   7739,   7740,   7742,   7741}},
 {  4218,  19, {  7763,   7764,   7766,   7765,   7741,   7742,   7744,   7743}},
 {  4219,  19, {  7765,   7766,   7768,   7767,   7743,   7744,   7746,   7745}},
 {  4220,  19, {  7767,   7768,   7770,   7769,   7745,   7746,   7748,   7747}},
 {  4221,  19, {  7769,   7770,   7772,   7771,   7747,   7748,   7750,   7749}},
 {  4222,  19, {  7771,   7772,   7774,   7773,   7749,   7750,   7752,   7751}},
 {  4223,  19, {  7773,   7774,   7776,   7775,   7751,   7752,   7754,   7753}},
 {  4224,  19, {  7775,   7776,   7778,   7777,   7753,   7754,   7756,   7755}},
 {  4225,  19, {  7777,   7778,   7780,   7779,   7755,   7756,   7758,   7757}},
 {  4226,  19, {  7779,   7780,   7587,   7586,   7757,   7758,   7561,   7560}},
 {  4227,  19, {  7636,   7637,   7782,   7781,   7610,   7611,   7760,   7759}},
 {  4228,  19, {  7781,   7782,   7784,   7783,   7759,   7760,   7762,   7761}},
 {  4229,  19, {  7783,   7784,   7786,   7785,   7761,   7762,   7764,   7763}},
 {  4230,  19, {  7785,   7786,   7788,   7787,   7763,   7764,   7766,   7765}},
 {  4231,  19, {  7787,   7788,   7790,   7789,   7765,   7766,   7768,   7767}},
 {  4232,  19, {  7789,   7790,   7792,   7791,   7767,   7768,   7770,   7769}},
 {  4233,  19, {  7791,   7792,   7794,   7793,   7769,   7770,   7772,   7771}},
 {  4234,  19, {  7793,   7794,   7796,   7795,   7771,   7772,   7774,   7773}},
 {  4235,  19, {  7795,   7796,   7798,   7797,   7773,   7774,   7776,   7775}},
 {  4236,  19, {  7797,   7798,   7800,   7799,   7775,   7776,   7778,   7777}},
 {  4237,  19, {  7799,   7800,   7802,   7801,   7777,   7778,   7780,   7779}},
 {  4238,  19, {  7801,   7802,   7613,   7612,   7779,   7780,   7587,   7586}},
 {  4239,  19, {  7662,   7663,   7804,   7803,   7636,   7637,   7782,   7781}},
 {  4240,  19, {  7803,   7804,   7806,   7805,   7781,   7782,   7784,   7783}},
 {  4241,  19, {  7805,   7806,   7808,   7807,   7783,   7784,   7786,   7785}},
 {  4242,  19, {  7807,   7808,   7810,   7809,   7785,   7786,   7788,   7787}},
 {  4243,  19, {  7809,   7810,   7812,   7811,   7787,   7788,   7790,   7789}},
 {  4244,  19, {  7811,   7812,   7814,   7813,   7789,   7790,   7792,   7791}},
 {  4245,  19, {  7813,   7814,   7816,   7815,   7791,   7792,   7794,   7793}},
 {  4246,  19, {  7815,   7816,   7818,   7817,   7793,   7794,   7796,   7795}},
 {  4247,  19, {  7817,   7818,   7820,   7819,   7795,   7796,   7798,   7797}},
 {  4248,  19, {  7819,   7820,   7822,   7821,   7797,   7798,   7800,   7799}},
 {  4249,  19, {  7821,   7822,   7824,   7823,   7799,   7800,   7802,   7801}},
 {  4250,  19, {  7823,   7824,   7825,   7638,   7801,   7802,   7613,   7612}},
 {  4251,  19, {  7688,   7689,   7827,   7826,   7662,   7663,   7804,   7803}},
 {  4252,  19, {  7826,   7827,   7829,   7828,   7803,   7804,   7806,   7805}},
 {  4253,  19, {  7828,   7829,   7831,   7830,   7805,   7806,   7808,   7807}},
 {  4254,  19, {  7830,   7831,   7833,   7832,   7807,   7808,   7810,   7809}},
 {  4255,  19, {  7832,   7833,   7835,   7834,   7809,   7810,   7812,   7811}},
 {  4256,  19, {  7834,   7835,   7837,   7836,   7811,   7812,   7814,   7813}},
 {  4257,  19, {  7836,   7837,   7839,   7838,   7813,   7814,   7816,   7815}},
 {  4258,  19, {  7838,   7839,   7841,   7840,   7815,   7816,   7818,   7817}},
 {  4259,  19, {  7840,   7841,   7843,   7842,   7817,   7818,   7820,   7819}},
 {  4260,  19, {  7842,   7843,   7845,   7844,   7819,   7820,   7822,   7821}},
 {  4261,  19, {  7844,   7845,   7847,   7846,   7821,   7822,   7824,   7823}},
 {  4262,  19, {  7846,   7847,   7848,   7664,   7823,   7824,   7825,   7638}},
 {  4263,  19, {  7713,   7714,   7850,   7849,   7688,   7689,   7827,   7826}},
 {  4264,  19, {  7849,   7850,   7852,   7851,   7826,   7827,   7829,   7828}},
 {  4265,  19, {  7851,   7852,   7854,   7853,   7828,   7829,   7831,   7830}},
 {  4266,  19, {  7853,   7854,   7856,   7855,   7830,   7831,   7833,   7832}},
 {  4267,  19, {  7855,   7856,   7858,   7857,   7832,   7833,   7835,   7834}},
 {  4268,  19, {  7857,   7858,   7860,   7859,   7834,   7835,   7837,   7836}},
 {  4269,  19, {  7859,   7860,   7862,   7861,   7836,   7837,   7839,   7838}},
 {  4270,  19, {  7861,   7862,   7864,   7863,   7838,   7839,   7841,   7840}},
 {  4271,  19, {  7863,   7864,   7866,   7865,   7840,   7841,   7843,   7842}},
 {  4272,  19, {  7865,   7866,   7868,   7867,   7842,   7843,   7845,   7844}},
 {  4273,  19, {  7867,   7868,   7870,   7869,   7844,   7845,   7847,   7846}},
 {  4274,  19, {  7869,   7870,   7871,   7690,   7846,   7847,   7848,   7664}},
 {  4275,  19, {  7561,   7885,   7886,   7563,   7535,   7872,   7873,   7537}},
 {  4276,  19, {  7563,   7886,   7887,   7565,   7537,   7873,   7874,   7539}},
 {  4277,  19, {  7565,   7887,   7888,   7567,   7539,   7874,   7875,   7541}},
 {  4278,  19, {  7567,   7888,   7889,   7569,   7541,   7875,   7876,   7543}},
 {  4279,  19, {  7569,   7889,   7890,   7571,   7543,   7876,   7877,   7545}},
 {  4280,  19, {  7571,   7890,   7891,   7573,   7545,   7877,   7878,   7547}},
 {  4281,  19, {  7573,   7891,   7892,   7575,   7547,   7878,   7879,   7549}},
 {  4282,  19, {  7575,   7892,   7893,   7577,   7549,   7879,   7880,   7551}},
 {  4283,  19, {  7577,   7893,   7894,   7579,   7551,   7880,   7881,   7553}},
 {  4284,  19, {  7579,   7894,   7895,   7581,   7553,   7881,   7882,   7555}},
 {  4285,  19, {  7581,   7895,   7896,   7583,   7555,   7882,   7883,   7557}},
 {  4286,  19, {  7583,   7896,   7897,   7585,   7557,   7883,   7884,   7559}},
 {  4287,  19, {  7587,   7898,   7899,   7589,   7561,   7885,   7886,   7563}},
 {  4288,  19, {  7589,   7899,   7900,   7591,   7563,   7886,   7887,   7565}},
 {  4289,  19, {  7591,   7900,   7901,   7593,   7565,   7887,   7888,   7567}},
 {  4290,  19, {  7593,   7901,   7902,   7595,   7567,   7888,   7889,   7569}},
 {  4291,  19, {  7595,   7902,   7903,   7597,   7569,   7889,   7890,   7571}},
 {  4292,  19, {  7597,   7903,   7904,   7599,   7571,   7890,   7891,   7573}},
 {  4293,  19, {  7599,   7904,   7905,   7601,   7573,   7891,   7892,   7575}},
 {  4294,  19, {  7601,   7905,   7906,   7603,   7575,   7892,   7893,   7577}},
 {  4295,  19, {  7603,   7906,   7907,   7605,   7577,   7893,   7894,   7579}},
 {  4296,  19, {  7605,   7907,   7908,   7607,   7579,   7894,   7895,   7581}},
 {  4297,  19, {  7607,   7908,   7909,   7609,   7581,   7895,   7896,   7583}},
 {  4298,  19, {  7609,   7909,   7910,   7611,   7583,   7896,   7897,   7585}},
 {  4299,  19, {  7613,   7911,   7912,   7615,   7587,   7898,   7899,   7589}},
 {  4300,  19, {  7615,   7912,   7913,   7617,   7589,   7899,   7900,   7591}},
 {  4301,  19, {  7617,   7913,   7914,   7619,   7591,   7900,   7901,   7593}},
 {  4302,  19, {  7619,   7914,   7915,   7621,   7593,   7901,   7902,   7595}},
 {  4303,  19, {  7621,   7915,   7916,   7623,   7595,   7902,   7903,   7597}},
 {  4304,  19, {  7623,   7916,   7917,   7625,   7597,   7903,   7904,   7599}},
 {  4305,  19, {  7625,   7917,   7918,   7627,   7599,   7904,   7905,   7601}},
 {  4306,  19, {  7627,   7918,   7919,   7629,   7601,   7905,   7906,   7603}},
 {  4307,  19, {  7629,   7919,   7920,   7631,   7603,   7906,   7907,   7605}},
 {  4308,  19, {  7631,   7920,   7921,   7633,   7605,   7907,   7908,   7607}},
 {  4309,  19, {  7633,   7921,   7922,   7635,   7607,   7908,   7909,   7609}},
 {  4310,  19, {  7635,   7922,   7923,   7637,   7609,   7909,   7910,   7611}},
 {  4311,  19, {  7639,   7924,   7925,   7641,   7613,   7911,   7912,   7615}},
 {  4312,  19, {  7641,   7925,   7926,   7643,   7615,   7912,   7913,   7617}},
 {  4313,  19, {  7643,   7926,   7927,   7645,   7617,   7913,   7914,   7619}},
 {  4314,  19, {  7645,   7927,   7928,   7647,   7619,   7914,   7915,   7621}},
 {  4315,  19, {  7647,   7928,   7929,   7649,   7621,   7915,   7916,   7623}},
 {  4316,  19, {  7649,   7929,   7930,   7651,   7623,   7916,   7917,   7625}},
 {  4317,  19, {  7651,   7930,   7931,   7653,   7625,   7917,   7918,   7627}},
 {  4318,  19, {  7653,   7931,   7932,   7655,   7627,   7918,   7919,   7629}},
 {  4319,  19, {  7655,   7932,   7933,   7657,   7629,   7919,   7920,   7631}},
 {  4320,  19, {  7657,   7933,   7934,   7659,   7631,   7920,   7921,   7633}},
 {  4321,  19, {  7659,   7934,   7935,   7661,   7633,   7921,   7922,   7635}},
 {  4322,  19, {  7661,   7935,   7936,   7663,   7635,   7922,   7923,   7637}},
 {  4323,  19, {  7665,   7937,   7938,   7667,   7639,   7924,   7925,   7641}},
 {  4324,  19, {  7667,   7938,   7939,   7669,   7641,   7925,   7926,   7643}},
 {  4325,  19, {  7669,   7939,   7940,   7671,   7643,   7926,   7927,   7645}},
 {  4326,  19, {  7671,   7940,   7941,   7673,   7645,   7927,   7928,   7647}},
 {  4327,  19, {  7673,   7941,   7942,   7675,   7647,   7928,   7929,   7649}},
 {  4328,  19, {  7675,   7942,   7943,   7677,   7649,   7929,   7930,   7651}},
 {  4329,  19, {  7677,   7943,   7944,   7679,   7651,   7930,   7931,   7653}},
 {  4330,  19, {  7679,   7944,   7945,   7681,   7653,   7931,   7932,   7655}},
 {  4331,  19, {  7681,   7945,   7946,   7683,   7655,   7932,   7933,   7657}},
 {  4332,  19, {  7683,   7946,   7947,   7685,   7657,   7933,   7934,   7659}},
 {  4333,  19, {  7685,   7947,   7948,   7687,   7659,   7934,   7935,   7661}},
 {  4334,  19, {  7687,   7948,   7949,   7689,   7661,   7935,   7936,   7663}},
 {  4335,  19, {  7691,   7950,   7951,   7693,   7665,   7937,   7938,   7667}},
 {  4336,  19, {  7693,   7951,   7952,   7695,   7667,   7938,   7939,   7669}},
 {  4337,  19, {  7695,   7952,   7953,   7697,   7669,   7939,   7940,   7671}},
 {  4338,  19, {  7697,   7953,   7954,   7699,   7671,   7940,   7941,   7673}},
 {  4339,  19, {  7699,   7954,   7955,   7701,   7673,   7941,   7942,   7675}},
 {  4340,  19, {  7701,   7955,   7956,   7526,   7675,   7942,   7943,   7677}},
 {  4341,  19, {  7526,   7956,   7957,   7704,   7677,   7943,   7944,   7679}},
 {  4342,  19, {  7704,   7957,   7958,   7706,   7679,   7944,   7945,   7681}},
 {  4343,  19, {  7706,   7958,   7959,   7708,   7681,   7945,   7946,   7683}},
 {  4344,  19, {  7708,   7959,   7960,   7710,   7683,   7946,   7947,   7685}},
 {  4345,  19, {  7710,   7960,   7961,   7712,   7685,   7947,   7948,   7687}},
 {  4346,  19, {  7712,   7961,   7962,   7714,   7687,   7948,   7949,   7689}},
 {  4347,  19, {  7585,   7897,   7975,   7738,   7559,   7884,   7963,   7716}},
 {  4348,  19, {  7738,   7975,   7976,   7740,   7716,   7963,   7964,   7718}},
 {  4349,  19, {  7740,   7976,   7977,   7742,   7718,   7964,   7965,   7720}},
 {  4350,  19, {  7742,   7977,   7978,   7744,   7720,   7965,   7966,   7722}},
 {  4351,  19, {  7744,   7978,   7979,   7746,   7722,   7966,   7967,   7724}},
 {  4352,  19, {  7746,   7979,   7980,   7748,   7724,   7967,   7968,   7726}},
 {  4353,  19, {  7748,   7980,   7981,   7750,   7726,   7968,   7969,   7728}},
 {  4354,  19, {  7750,   7981,   7982,   7752,   7728,   7969,   7970,   7730}},
 {  4355,  19, {  7752,   7982,   7983,   7754,   7730,   7970,   7971,   7732}},
 {  4356,  19, {  7754,   7983,   7984,   7756,   7732,   7971,   7972,   7734}},
 {  4357,  19, {  7756,   7984,   7985,   7758,   7734,   7972,   7973,   7736}},
 {  4358,  19, {  7758,   7985,   7986,   7561,   7736,   7973,   7974,   7535}},
 {  4359,  19, {  7611,   7910,   7987,   7760,   7585,   7897,   7975,   7738}},
 {  4360,  19, {  7760,   7987,   7988,   7762,   7738,   7975,   7976,   7740}},
 {  4361,  19, {  7762,   7988,   7989,   7764,   7740,   7976,   7977,   7742}},
 {  4362,  19, {  7764,   7989,   7990,   7766,   7742,   7977,   7978,   7744}},
 {  4363,  19, {  7766,   7990,   7991,   7768,   7744,   7978,   7979,   7746}},
 {  4364,  19, {  7768,   7991,   7992,   7770,   7746,   7979,   7980,   7748}},
 {  4365,  19, {  7770,   7992,   7993,   7772,   7748,   7980,   7981,   7750}},
 {  4366,  19, {  7772,   7993,   7994,   7774,   7750,   7981,   7982,   7752}},
 {  4367,  19, {  7774,   7994,   7995,   7776,   7752,   7982,   7983,   7754}},
 {  4368,  19, {  7776,   7995,   7996,   7778,   7754,   7983,   7984,   7756}},
 {  4369,  19, {  7778,   7996,   7997,   7780,   7756,   7984,   7985,   7758}},
 {  4370,  19, {  7780,   7997,   7998,   7587,   7758,   7985,   7986,   7561}},
 {  4371,  19, {  7637,   7923,   7999,   7782,   7611,   7910,   7987,   7760}},
 {  4372,  19, {  7782,   7999,   8000,   7784,   7760,   7987,   7988,   7762}},
 {  4373,  19, {  7784,   8000,   8001,   7786,   7762,   7988,   7989,   7764}},
 {  4374,  19, {  7786,   8001,   8002,   7788,   7764,   7989,   7990,   7766}},
 {  4375,  19, {  7788,   8002,   8003,   7790,   7766,   7990,   7991,   7768}},
 {  4376,  19, {  7790,   8003,   8004,   7792,   7768,   7991,   7992,   7770}},
 {  4377,  19, {  7792,   8004,   8005,   7794,   7770,   7992,   7993,   7772}},
 {  4378,  19, {  7794,   8005,   8006,   7796,   7772,   7993,   7994,   7774}},
 {  4379,  19, {  7796,   8006,   8007,   7798,   7774,   7994,   7995,   7776}},
 {  4380,  19, {  7798,   8007,   8008,   7800,   7776,   7995,   7996,   7778}},
 {  4381,  19, {  7800,   8008,   8009,   7802,   7778,   7996,   7997,   7780}},
 {  4382,  19, {  7802,   8009,   8010,   7613,   7780,   7997,   7998,   7587}},
 {  4383,  19, {  7663,   7936,   8011,   7804,   7637,   7923,   7999,   7782}},
 {  4384,  19, {  7804,   8011,   8012,   7806,   7782,   7999,   8000,   7784}},
 {  4385,  19, {  7806,   8012,   8013,   7808,   7784,   8000,   8001,   7786}},
 {  4386,  19, {  7808,   8013,   8014,   7810,   7786,   8001,   8002,   7788}},
 {  4387,  19, {  7810,   8014,   8015,   7812,   7788,   8002,   8003,   7790}},
 {  4388,  19, {  7812,   8015,   8016,   7814,   7790,   8003,   8004,   7792}},
 {  4389,  19, {  7814,   8016,   8017,   7816,   7792,   8004,   8005,   7794}},
 {  4390,  19, {  7816,   8017,   8018,   7818,   7794,   8005,   8006,   7796}},
 {  4391,  19, {  7818,   8018,   8019,   7820,   7796,   8006,   8007,   7798}},
 {  4392,  19, {  7820,   8019,   8020,   7822,   7798,   8007,   8008,   7800}},
 {  4393,  19, {  7822,   8020,   8021,   7824,   7800,   8008,   8009,   7802}},
 {  4394,  19, {  7824,   8021,   8022,   7825,   7802,   8009,   8010,   7613}},
 {  4395,  19, {  7689,   7949,   8023,   7827,   7663,   7936,   8011,   7804}},
 {  4396,  19, {  7827,   8023,   8024,   7829,   7804,   8011,   8012,   7806}},
 {  4397,  19, {  7829,   8024,   8025,   7831,   7806,   8012,   8013,   7808}},
 {  4398,  19, {  7831,   8025,   8026,   7833,   7808,   8013,   8014,   7810}},
 {  4399,  19, {  7833,   8026,   8027,   7835,   7810,   8014,   8015,   7812}},
 {  4400,  19, {  7835,   8027,   8028,   7837,   7812,   8015,   8016,   7814}},
 {  4401,  19, {  7837,   8028,   8029,   7839,   7814,   8016,   8017,   7816}},
 {  4402,  19, {  7839,   8029,   8030,   7841,   7816,   8017,   8018,   7818}},
 {  4403,  19, {  7841,   8030,   8031,   7843,   7818,   8018,   8019,   7820}},
 {  4404,  19, {  7843,   8031,   8032,   7845,   7820,   8019,   8020,   7822}},
 {  4405,  19, {  7845,   8032,   8033,   7847,   7822,   8020,   8021,   7824}},
 {  4406,  19, {  7847,   8033,   8034,   7848,   7824,   8021,   8022,   7825}},
 {  4407,  19, {  7714,   7962,   8035,   7850,   7689,   7949,   8023,   7827}},
 {  4408,  19, {  7850,   8035,   8036,   7852,   7827,   8023,   8024,   7829}},
 {  4409,  19, {  7852,   8036,   8037,   7854,   7829,   8024,   8025,   7831}},
 {  4410,  19, {  7854,   8037,   8038,   7856,   7831,   8025,   8026,   7833}},
 {  4411,  19, {  7856,   8038,   8039,   7858,   7833,   8026,   8027,   7835}},
 {  4412,  19, {  7858,   8039,   8040,   7860,   7835,   8027,   8028,   7837}},
 {  4413,  19, {  7860,   8040,   8041,   7862,   7837,   8028,   8029,   7839}},
 {  4414,  19, {  7862,   8041,   8042,   7864,   7839,   8029,   8030,   7841}},
 {  4415,  19, {  7864,   8042,   8043,   7866,   7841,   8030,   8031,   7843}},
 {  4416,  19, {  7866,   8043,   8044,   7868,   7843,   8031,   8032,   7845}},
 {  4417,  19, {  7868,   8044,   8045,   7870,   7845,   8032,   8033,   7847}},
 {  4418,  19, {  7870,   8045,   7243,   7871,   7847,   8033,   8034,   7848}},
 {  4419,  19, {  7885,   8059,   8060,   7886,   7872,   8046,   8047,   7873}},
 {  4420,  19, {  7886,   8060,   8061,   7887,   7873,   8047,   8048,   7874}},
 {  4421,  19, {  7887,   8061,   8062,   7888,   7874,   8048,   8049,   7875}},
 {  4422,  19, {  7888,   8062,   8063,   7889,   7875,   8049,   8050,   7876}},
 {  4423,  19, {  7889,   8063,   8064,   7890,   7876,   8050,   8051,   7877}},
 {  4424,  19, {  7890,   8064,   8065,   7891,   7877,   8051,   8052,   7878}},
 {  4425,  19, {  7891,   8065,   8066,   7892,   7878,   8052,   8053,   7879}},
 {  4426,  19, {  7892,   8066,   8067,   7893,   7879,   8053,   8054,   7880}},
 {  4427,  19, {  7893,   8067,   8068,   7894,   7880,   8054,   8055,   7881}},
 {  4428,  19, {  7894,   8068,   8069,   7895,   7881,   8055,   8056,   7882}},
 {  4429,  19, {  7895,   8069,   8070,   7896,   7882,   8056,   8057,   7883}},
 {  4430,  19, {  7896,   8070,   8071,   7897,   7883,   8057,   8058,   7884}},
 {  4431,  19, {  7898,   8072,   8073,   7899,   7885,   8059,   8060,   7886}},
 {  4432,  19, {  7899,   8073,   8074,   7900,   7886,   8060,   8061,   7887}},
 {  4433,  19, {  7900,   8074,   8075,   7901,   7887,   8061,   8062,   7888}},
 {  4434,  19, {  7901,   8075,   8076,   7902,   7888,   8062,   8063,   7889}},
 {  4435,  19, {  7902,   8076,   8077,   7903,   7889,   8063,   8064,   7890}},
 {  4436,  19, {  7903,   8077,   8078,   7904,   7890,   8064,   8065,   7891}},
 {  4437,  19, {  7904,   8078,   8079,   7905,   7891,   8065,   8066,   7892}},
 {  4438,  19, {  7905,   8079,   8080,   7906,   7892,   8066,   8067,   7893}},
 {  4439,  19, {  7906,   8080,   8081,   7907,   7893,   8067,   8068,   7894}},
 {  4440,  19, {  7907,   8081,   8082,   7908,   7894,   8068,   8069,   7895}},
 {  4441,  19, {  7908,   8082,   8083,   7909,   7895,   8069,   8070,   7896}},
 {  4442,  19, {  7909,   8083,   8084,   7910,   7896,   8070,   8071,   7897}},
 {  4443,  19, {  7911,   8085,   8086,   7912,   7898,   8072,   8073,   7899}},
 {  4444,  19, {  7912,   8086,   8087,   7913,   7899,   8073,   8074,   7900}},
 {  4445,  19, {  7913,   8087,   8088,   7914,   7900,   8074,   8075,   7901}},
 {  4446,  19, {  7914,   8088,   8089,   7915,   7901,   8075,   8076,   7902}},
 {  4447,  19, {  7915,   8089,   8090,   7916,   7902,   8076,   8077,   7903}},
 {  4448,  19, {  7916,   8090,   8091,   7917,   7903,   8077,   8078,   7904}},
 {  4449,  19, {  7917,   8091,   8092,   7918,   7904,   8078,   8079,   7905}},
 {  4450,  19, {  7918,   8092,   8093,   7919,   7905,   8079,   8080,   7906}},
 {  4451,  19, {  7919,   8093,   8094,   7920,   7906,   8080,   8081,   7907}},
 {  4452,  19, {  7920,   8094,   8095,   7921,   7907,   8081,   8082,   7908}},
 {  4453,  19, {  7921,   8095,   8096,   7922,   7908,   8082,   8083,   7909}},
 {  4454,  19, {  7922,   8096,   8097,   7923,   7909,   8083,   8084,   7910}},
 {  4455,  19, {  7924,   8098,   8099,   7925,   7911,   8085,   8086,   7912}},
 {  4456,  19, {  7925,   8099,   8100,   7926,   7912,   8086,   8087,   7913}},
 {  4457,  19, {  7926,   8100,   8101,   7927,   7913,   8087,   8088,   7914}},
 {  4458,  19, {  7927,   8101,   8102,   7928,   7914,   8088,   8089,   7915}},
 {  4459,  19, {  7928,   8102,   8103,   7929,   7915,   8089,   8090,   7916}},
 {  4460,  19, {  7929,   8103,   8104,   7930,   7916,   8090,   8091,   7917}},
 {  4461,  19, {  7930,   8104,   8105,   7931,   7917,   8091,   8092,   7918}},
 {  4462,  19, {  7931,   8105,   8106,   7932,   7918,   8092,   8093,   7919}},
 {  4463,  19, {  7932,   8106,   8107,   7933,   7919,   8093,   8094,   7920}},
 {  4464,  19, {  7933,   8107,   8108,   7934,   7920,   8094,   8095,   7921}},
 {  4465,  19, {  7934,   8108,   8109,   7935,   7921,   8095,   8096,   7922}},
 {  4466,  19, {  7935,   8109,   8110,   7936,   7922,   8096,   8097,   7923}},
 {  4467,  19, {  7937,   8111,   8112,   7938,   7924,   8098,   8099,   7925}},
 {  4468,  19, {  7938,   8112,   8113,   7939,   7925,   8099,   8100,   7926}},
 {  4469,  19, {  7939,   8113,   8114,   7940,   7926,   8100,   8101,   7927}},
 {  4470,  19, {  7940,   8114,   8115,   7941,   7927,   8101,   8102,   7928}},
 {  4471,  19, {  7941,   8115,   8116,   7942,   7928,   8102,   8103,   7929}},
 {  4472,  19, {  7942,   8116,   8117,   7943,   7929,   8103,   8104,   7930}},
 {  4473,  19, {  7943,   8117,   8118,   7944,   7930,   8104,   8105,   7931}},
 {  4474,  19, {  7944,   8118,   8119,   7945,   7931,   8105,   8106,   7932}},
 {  4475,  19, {  7945,   8119,   8120,   7946,   7932,   8106,   8107,   7933}},
 {  4476,  19, {  7946,   8120,   8121,   7947,   7933,   8107,   8108,   7934}},
 {  4477,  19, {  7947,   8121,   8122,   7948,   7934,   8108,   8109,   7935}},
 {  4478,  19, {  7948,   8122,   8123,   7949,   7935,   8109,   8110,   7936}},
 {  4479,  19, {  7950,   8124,   8125,   7951,   7937,   8111,   8112,   7938}},
 {  4480,  19, {  7951,   8125,   8126,   7952,   7938,   8112,   8113,   7939}},
 {  4481,  19, {  7952,   8126,   8127,   7953,   7939,   8113,   8114,   7940}},
 {  4482,  19, {  7953,   8127,   8128,   7954,   7940,   8114,   8115,   7941}},
 {  4483,  19, {  7954,   8128,   8129,   7955,   7941,   8115,   8116,   7942}},
 {  4484,  19, {  7955,   8129,   8130,   7956,   7942,   8116,   8117,   7943}},
 {  4485,  19, {  7956,   8130,   8131,   7957,   7943,   8117,   8118,   7944}},
 {  4486,  19, {  7957,   8131,   8132,   7958,   7944,   8118,   8119,   7945}},
 {  4487,  19, {  7958,   8132,   8133,   7959,   7945,   8119,   8120,   7946}},
 {  4488,  19, {  7959,   8133,   8134,   7960,   7946,   8120,   8121,   7947}},
 {  4489,  19, {  7960,   8134,   8135,   7961,   7947,   8121,   8122,   7948}},
 {  4490,  19, {  7961,   8135,   8136,   7962,   7948,   8122,   8123,   7949}},
 {  4491,  19, {  7897,   8071,   8149,   7975,   7884,   8058,   8137,   7963}},
 {  4492,  19, {  7975,   8149,   8150,   7976,   7963,   8137,   8138,   7964}},
 {  4493,  19, {  7976,   8150,   8151,   7977,   7964,   8138,   8139,   7965}},
 {  4494,  19, {  7977,   8151,   8152,   7978,   7965,   8139,   8140,   7966}},
 {  4495,  19, {  7978,   8152,   8153,   7979,   7966,   8140,   8141,   7967}},
 {  4496,  19, {  7979,   8153,   8154,   7980,   7967,   8141,   8142,   7968}},
 {  4497,  19, {  7980,   8154,   8155,   7981,   7968,   8142,   8143,   7969}},
 {  4498,  19, {  7981,   8155,   8156,   7982,   7969,   8143,   8144,   7970}},
 {  4499,  19, {  7982,   8156,   8157,   7983,   7970,   8144,   8145,   7971}},
 {  4500,  19, {  7983,   8157,   8158,   7984,   7971,   8145,   8146,   7972}},
 {  4501,  19, {  7984,   8158,   8159,   7985,   7972,   8146,   8147,   7973}},
 {  4502,  19, {  7985,   8159,   8160,   7986,   7973,   8147,   8148,   7974}},
 {  4503,  19, {  7910,   8084,   8161,   7987,   7897,   8071,   8149,   7975}},
 {  4504,  19, {  7987,   8161,   8162,   7988,   7975,   8149,   8150,   7976}},
 {  4505,  19, {  7988,   8162,   8163,   7989,   7976,   8150,   8151,   7977}},
 {  4506,  19, {  7989,   8163,   8164,   7990,   7977,   8151,   8152,   7978}},
 {  4507,  19, {  7990,   8164,   8165,   7991,   7978,   8152,   8153,   7979}},
 {  4508,  19, {  7991,   8165,   8166,   7992,   7979,   8153,   8154,   7980}},
 {  4509,  19, {  7992,   8166,   8167,   7993,   7980,   8154,   8155,   7981}},
 {  4510,  19, {  7993,   8167,   8168,   7994,   7981,   8155,   8156,   7982}},
 {  4511,  19, {  7994,   8168,   8169,   7995,   7982,   8156,   8157,   7983}},
 {  4512,  19, {  7995,   8169,   8170,   7996,   7983,   8157,   8158,   7984}},
 {  4513,  19, {  7996,   8170,   8171,   7997,   7984,   8158,   8159,   7985}},
 {  4514,  19, {  7997,   8171,   8172,   7998,   7985,   8159,   8160,   7986}},
 {  4515,  19, {  7923,   8097,   8173,   7999,   7910,   8084,   8161,   7987}},
 {  4516,  19, {  7999,   8173,   8174,   8000,   7987,   8161,   8162,   7988}},
 {  4517,  19, {  8000,   8174,   8175,   8001,   7988,   8162,   8163,   7989}},
 {  4518,  19, {  8001,   8175,   8176,   8002,   7989,   8163,   8164,   7990}},
 {  4519,  19, {  8002,   8176,   8177,   8003,   7990,   8164,   8165,   7991}},
 {  4520,  19, {  8003,   8177,   8178,   8004,   7991,   8165,   8166,   7992}},
 {  4521,  19, {  8004,   8178,   8179,   8005,   7992,   8166,   8167,   7993}},
 {  4522,  19, {  8005,   8179,   8180,   8006,   7993,   8167,   8168,   7994}},
 {  4523,  19, {  8006,   8180,   8181,   8007,   7994,   8168,   8169,   7995}},
 {  4524,  19, {  8007,   8181,   8182,   8008,   7995,   8169,   8170,   7996}},
 {  4525,  19, {  8008,   8182,   8183,   8009,   7996,   8170,   8171,   7997}},
 {  4526,  19, {  8009,   8183,   8184,   8010,   7997,   8171,   8172,   7998}},
 {  4527,  19, {  7936,   8110,   8185,   8011,   7923,   8097,   8173,   7999}},
 {  4528,  19, {  8011,   8185,   8186,   8012,   7999,   8173,   8174,   8000}},
 {  4529,  19, {  8012,   8186,   8187,   8013,   8000,   8174,   8175,   8001}},
 {  4530,  19, {  8013,   8187,   8188,   8014,   8001,   8175,   8176,   8002}},
 {  4531,  19, {  8014,   8188,   8189,   8015,   8002,   8176,   8177,   8003}},
 {  4532,  19, {  8015,   8189,   8190,   8016,   8003,   8177,   8178,   8004}},
 {  4533,  19, {  8016,   8190,   8191,   8017,   8004,   8178,   8179,   8005}},
 {  4534,  19, {  8017,   8191,   8192,   8018,   8005,   8179,   8180,   8006}},
 {  4535,  19, {  8018,   8192,   8193,   8019,   8006,   8180,   8181,   8007}},
 {  4536,  19, {  8019,   8193,   8194,   8020,   8007,   8181,   8182,   8008}},
 {  4537,  19, {  8020,   8194,   8195,   8021,   8008,   8182,   8183,   8009}},
 {  4538,  19, {  8021,   8195,   8196,   8022,   8009,   8183,   8184,   8010}},
 {  4539,  19, {  7949,   8123,   8197,   8023,   7936,   8110,   8185,   8011}},
 {  4540,  19, {  8023,   8197,   8198,   8024,   8011,   8185,   8186,   8012}},
 {  4541,  19, {  8024,   8198,   8199,   8025,   8012,   8186,   8187,   8013}},
 {  4542,  19, {  8025,   8199,   8200,   8026,   8013,   8187,   8188,   8014}},
 {  4543,  19, {  8026,   8200,   8201,   8027,   8014,   8188,   8189,   8015}},
 {  4544,  19, {  8027,   8201,   8202,   8028,   8015,   8189,   8190,   8016}},
 {  4545,  19, {  8028,   8202,   8203,   8029,   8016,   8190,   8191,   8017}},
 {  4546,  19, {  8029,   8203,   8204,   8030,   8017,   8191,   8192,   8018}},
 {  4547,  19, {  8030,   8204,   8205,   8031,   8018,   8192,   8193,   8019}},
 {  4548,  19, {  8031,   8205,   8206,   8032,   8019,   8193,   8194,   8020}},
 {  4549,  19, {  8032,   8206,   8207,   8033,   8020,   8194,   8195,   8021}},
 {  4550,  19, {  8033,   8207,   8208,   8034,   8021,   8195,   8196,   8022}},
 {  4551,  19, {  7962,   8136,   8209,   8035,   7949,   8123,   8197,   8023}},
 {  4552,  19, {  8035,   8209,   8210,   8036,   8023,   8197,   8198,   8024}},
 {  4553,  19, {  8036,   8210,   8211,   8037,   8024,   8198,   8199,   8025}},
 {  4554,  19, {  8037,   8211,   8212,   8038,   8025,   8199,   8200,   8026}},
 {  4555,  19, {  8038,   8212,   8213,   8039,   8026,   8200,   8201,   8027}},
 {  4556,  19, {  8039,   8213,   8214,   8040,   8027,   8201,   8202,   8028}},
 {  4557,  19, {  8040,   8214,   8215,   8041,   8028,   8202,   8203,   8029}},
 {  4558,  19, {  8041,   8215,   8216,   8042,   8029,   8203,   8204,   8030}},
 {  4559,  19, {  8042,   8216,   8217,   8043,   8030,   8204,   8205,   8031}},
 {  4560,  19, {  8043,   8217,   8218,   8044,   8031,   8205,   8206,   8032}},
 {  4561,  19, {  8044,   8218,   8219,   8045,   8032,   8206,   8207,   8033}},
 {  4562,  19, {  8045,   8219,   7360,   7243,   8033,   8207,   8208,   8034}},
 {  4564,  20, {  8221,   8222,   8231,   8230,   8248,   8249,   8258,   8257}},
 {  4565,  20, {  8222,   8223,   8232,   8231,   8249,   8250,   8259,   8258}},
 {  4566,  20, {  8223,   8224,   8233,   8232,   8250,   8251,   8260,   8259}},
 {  4567,  20, {  8224,   8225,   8234,   8233,   8251,   8252,   8261,   8260}},
 {  4568,  20, {  8225,   8226,   8235,   8234,   8252,   8253,   8262,   8261}},
 {  4569,  20, {  8226,   8227,   8236,   8235,   8253,   8254,   8263,   8262}},
 {  4570,  20, {  8227,   8228,   8237,   8236,   8254,   8255,   8264,   8263}},
 {  4572,  20, {  8230,   8231,   8240,   8239,   8257,   8258,   8267,   8266}},
 {  4573,  20, {  8231,   8232,   8241,   8240,   8258,   8259,   8268,   8267}},
 {  4574,  20, {  8232,   8233,   8242,   8241,   8259,   8260,   8269,   8268}},
 {  4575,  20, {  8233,   8234,   8243,   8242,   8260,   8261,   8270,   8269}},
 {  4576,  20, {  8234,   8235,   8244,   8243,   8261,   8262,   8271,   8270}},
 {  4577,  20, {  8235,   8236,   8245,   8244,   8262,   8263,   8272,   8271}},
 {  4578,  20, {  8236,   8237,   8246,   8245,   8263,   8264,   8273,   8272}},
 {  4580,  20, {  8248,   8249,   8258,   8257,   8275,   8276,   8285,   8284}},
 {  4581,  20, {  8249,   8250,   8259,   8258,   8276,   8277,   8286,   8285}},
 {  4582,  20, {  8250,   8251,   8260,   8259,   8277,   8278,   8287,   8286}},
 {  4583,  20, {  8251,   8252,   8261,   8260,   8278,   8279,   8288,   8287}},
 {  4584,  20, {  8252,   8253,   8262,   8261,   8279,   8280,   8289,   8288}},
 {  4585,  20, {  8253,   8254,   8263,   8262,   8280,   8281,   8290,   8289}},
 {  4586,  20, {  8254,   8255,   8264,   8263,   8281,   8282,   8291,   8290}},
 {  4588,  20, {  8257,   8258,   8267,   8266,   8284,   8285,   8294,   8293}},
 {  4589,  20, {  8258,   8259,   8268,   8267,   8285,   8286,   8295,   8294}},
 {  4590,  20, {  8259,   8260,   8269,   8268,   8286,   8287,   8296,   8295}},
 {  4591,  20, {  8260,   8261,   8270,   8269,   8287,   8288,   8297,   8296}},
 {  4592,  20, {  8261,   8262,   8271,   8270,   8288,   8289,   8298,   8297}},
 {  4593,  20, {  8262,   8263,   8272,   8271,   8289,   8290,   8299,   8298}},
 {  4594,  20, {  8263,   8264,   8273,   8272,   8290,   8291,   8300,   8299}},
 {  4596,  20, {  8275,   8276,   8285,   8284,   8302,   8303,   8312,   8311}},
 {  4597,  20, {  8276,   8277,   8286,   8285,   8303,   8304,   8313,   8312}},
 {  4598,  20, {  8277,   8278,   8287,   8286,   8304,   8305,   8314,   8313}},
 {  4599,  20, {  8278,   8279,   8288,   8287,   8305,   8306,   8315,   8314}},
 {  4600,  20, {  8279,   8280,   8289,   8288,   8306,   8307,   8316,   8315}},
 {  4601,  20, {  8280,   8281,   8290,   8289,   8307,   8308,   8317,   8316}},
 {  4602,  20, {  8281,   8282,   8291,   8290,   8308,   8309,   8318,   8317}},
 {  4604,  20, {  8284,   8285,   8294,   8293,   8311,   8312,   8321,   8320}},
 {  4605,  20, {  8285,   8286,   8295,   8294,   8312,   8313,   8322,   8321}},
 {  4606,  20, {  8286,   8287,   8296,   8295,   8313,   8314,   8323,   8322}},
 {  4607,  20, {  8287,   8288,   8297,   8296,   8314,   8315,   8324,   8323}},
 {  4608,  20, {  8288,   8289,   8298,   8297,   8315,   8316,   8325,   8324}},
 {  4609,  20, {  8289,   8290,   8299,   8298,   8316,   8317,   8326,   8325}},
 {  4610,  20, {  8290,   8291,   8300,   8299,   8317,   8318,   8327,   8326}},
 {  4612,  20, {  8302,   8303,   8312,   8311,   8329,   8330,   8339,   8338}},
 {  4613,  20, {  8303,   8304,   8313,   8312,   8330,   8331,   8340,   8339}},
 {  4614,  20, {  8304,   8305,   8314,   8313,   8331,   8332,   8341,   8340}},
 {  4615,  20, {  8305,   8306,   8315,   8314,   8332,   8333,   8342,   8341}},
 {  4616,  20, {  8306,   8307,   8316,   8315,   8333,   8334,   8343,   8342}},
 {  4617,  20, {  8307,   8308,   8317,   8316,   8334,   8335,   8344,   8343}},
 {  4618,  20, {  8308,   8309,   8318,   8317,   8335,   8336,   8345,   8344}},
 {  4620,  20, {  8311,   8312,   8321,   8320,   8338,   8339,   8348,   8347}},
 {  4621,  20, {  8312,   8313,   8322,   8321,   8339,   8340,   8349,   8348}},
 {  4622,  20, {  8313,   8314,   8323,   8322,   8340,   8341,   8350,   8349}},
 {  4623,  20, {  8314,   8315,   8324,   8323,   8341,   8342,   8351,   8350}},
 {  4624,  20, {  8315,   8316,   8325,   8324,   8342,   8343,   8352,   8351}},
 {  4625,  20, {  8316,   8317,   8326,   8325,   8343,   8344,   8353,   8352}},
 {  4626,  20, {  8317,   8318,   8327,   8326,   8344,   8345,   8354,   8353}},
 {  4628,  20, {  8329,   8330,   8339,   8338,   8356,   8357,   8366,   8365}},
 {  4629,  20, {  8330,   8331,   8340,   8339,   8357,   8358,   8367,   8366}},
 {  4630,  20, {  8331,   8332,   8341,   8340,   8358,   8359,   8368,   8367}},
 {  4631,  20, {  8332,   8333,   8342,   8341,   8359,   8360,   8369,   8368}},
 {  4632,  20, {  8333,   8334,   8343,   8342,   8360,   8361,   8370,   8369}},
 {  4633,  20, {  8334,   8335,   8344,   8343,   8361,   8362,   8371,   8370}},
 {  4634,  20, {  8335,   8336,   8345,   8344,   8362,   8363,   8372,   8371}},
 {  4636,  20, {  8338,   8339,   8348,   8347,   8365,   8366,   8375,   8374}},
 {  4637,  20, {  8339,   8340,   8349,   8348,   8366,   8367,   8376,   8375}},
 {  4638,  20, {  8340,   8341,   8350,   8349,   8367,   8368,   8377,   8376}},
 {  4639,  20, {  8341,   8342,   8351,   8350,   8368,   8369,   8378,   8377}},
 {  4640,  20, {  8342,   8343,   8352,   8351,   8369,   8370,   8379,   8378}},
 {  4641,  20, {  8343,   8344,   8353,   8352,   8370,   8371,   8380,   8379}},
 {  4642,  20, {  8344,   8345,   8354,   8353,   8371,   8372,   8381,   8380}},
 {  4644,  20, {  8356,   8357,   8366,   8365,   8383,   8384,   8393,   8392}},
 {  4645,  20, {  8357,   8358,   8367,   8366,   8384,   8385,   8394,   8393}},
 {  4646,  20, {  8358,   8359,   8368,   8367,   8385,   8386,   8395,   8394}},
 {  4647,  20, {  8359,   8360,   8369,   8368,   8386,   8387,   8396,   8395}},
 {  4648,  20, {  8360,   8361,   8370,   8369,   8387,   8388,   8397,   8396}},
 {  4649,  20, {  8361,   8362,   8371,   8370,   8388,   8389,   8398,   8397}},
 {  4650,  20, {  8362,   8363,   8372,   8371,   8389,   8390,   8399,   8398}},
 {  4652,  20, {  8365,   8366,   8375,   8374,   8392,   8393,   8402,   8401}},
 {  4653,  20, {  8366,   8367,   8376,   8375,   8393,   8394,   8403,   8402}},
 {  4654,  20, {  8367,   8368,   8377,   8376,   8394,   8395,   8404,   8403}},
 {  4655,  20, {  8368,   8369,   8378,   8377,   8395,   8396,   8405,   8404}},
 {  4656,  20, {  8369,   8370,   8379,   8378,   8396,   8397,   8406,   8405}},
 {  4657,  20, {  8370,   8371,   8380,   8379,   8397,   8398,   8407,   8406}},
 {  4658,  20, {  8371,   8372,   8381,   8380,   8398,   8399,   8408,   8407}},
 {  4660,  20, {  8383,   8384,   8393,   8392,   8410,   8411,   8420,   8419}},
 {  4661,  20, {  8384,   8385,   8394,   8393,   8411,   8412,   8421,   8420}},
 {  4662,  20, {  8385,   8386,   8395,   8394,   8412,   8413,   8422,   8421}},
 {  4663,  20, {  8386,   8387,   8396,   8395,   8413,   8414,   8423,   8422}},
 {  4664,  20, {  8387,   8388,   8397,   8396,   8414,   8415,   8424,   8423}},
 {  4665,  20, {  8388,   8389,   8398,   8397,   8415,   8416,   8425,   8424}},
 {  4666,  20, {  8389,   8390,   8399,   8398,   8416,   8417,   8426,   8425}},
 {  4668,  20, {  8392,   8393,   8402,   8401,   8419,   8420,   8429,   8428}},
 {  4669,  20, {  8393,   8394,   8403,   8402,   8420,   8421,   8430,   8429}},
 {  4670,  20, {  8394,   8395,   8404,   8403,   8421,   8422,   8431,   8430}},
 {  4671,  20, {  8395,   8396,   8405,   8404,   8422,   8423,   8432,   8431}},
 {  4672,  20, {  8396,   8397,   8406,   8405,   8423,   8424,   8433,   8432}},
 {  4673,  20, {  8397,   8398,   8407,   8406,   8424,   8425,   8434,   8433}},
 {  4674,  20, {  8398,   8399,   8408,   8407,   8425,   8426,   8435,   8434}},
 {  4676,  20, {  8410,   8411,   8420,   8419,   8437,   8438,   8447,   8446}},
 {  4677,  20, {  8411,   8412,   8421,   8420,   8438,   8439,   8448,   8447}},
 {  4678,  20, {  8412,   8413,   8422,   8421,   8439,   8440,   8449,   8448}},
 {  4679,  20, {  8413,   8414,   8423,   8422,   8440,   8441,   8450,   8449}},
 {  4680,  20, {  8414,   8415,   8424,   8423,   8441,   8442,   8451,   8450}},
 {  4681,  20, {  8415,   8416,   8425,   8424,   8442,   8443,   8452,   8451}},
 {  4682,  20, {  8416,   8417,   8426,   8425,   8443,   8444,   8453,   8452}},
 {  4684,  20, {  8419,   8420,   8429,   8428,   8446,   8447,   8456,   8455}},
 {  4685,  20, {  8420,   8421,   8430,   8429,   8447,   8448,   8457,   8456}},
 {  4686,  20, {  8421,   8422,   8431,   8430,   8448,   8449,   8458,   8457}},
 {  4687,  20, {  8422,   8423,   8432,   8431,   8449,   8450,   8459,   8458}},
 {  4688,  20, {  8423,   8424,   8433,   8432,   8450,   8451,   8460,   8459}},
 {  4689,  20, {  8424,   8425,   8434,   8433,   8451,   8452,   8461,   8460}},
 {  4690,  20, {  8425,   8426,   8435,   8434,   8452,   8453,   8462,   8461}},
 {  4692,  20, {  8437,   8438,   8447,   8446,   8464,   8465,   8474,   8473}},
 {  4693,  20, {  8438,   8439,   8448,   8447,   8465,   8466,   8475,   8474}},
 {  4694,  20, {  8439,   8440,   8449,   8448,   8466,   8467,   8476,   8475}},
 {  4695,  20, {  8440,   8441,   8450,   8449,   8467,   8468,   8477,   8476}},
 {  4696,  20, {  8441,   8442,   8451,   8450,   8468,   8469,   8478,   8477}},
 {  4697,  20, {  8442,   8443,   8452,   8451,   8469,   8470,   8479,   8478}},
 {  4698,  20, {  8443,   8444,   8453,   8452,   8470,   8471,   8480,   8479}},
 {  4700,  20, {  8446,   8447,   8456,   8455,   8473,   8474,   8483,   8482}},
 {  4701,  20, {  8447,   8448,   8457,   8456,   8474,   8475,   8484,   8483}},
 {  4702,  20, {  8448,   8449,   8458,   8457,   8475,   8476,   8485,   8484}},
 {  4703,  20, {  8449,   8450,   8459,   8458,   8476,   8477,   8486,   8485}},
 {  4704,  20, {  8450,   8451,   8460,   8459,   8477,   8478,   8487,   8486}},
 {  4705,  20, {  8451,   8452,   8461,   8460,   8478,   8479,   8488,   8487}},
 {  4706,  20, {  8452,   8453,   8462,   8461,   8479,   8480,   8489,   8488}},
 {  4707,  20, {  8228,   8490,   8498,   8237,   8255,   8514,   8522,   8264}},
 {  4708,  20, {  8490,   8491,   8499,   8498,   8514,   8515,   8523,   8522}},
 {  4709,  20, {  8491,   8492,   8500,   8499,   8515,   8516,   8524,   8523}},
 {  4710,  20, {  8492,   8493,   8501,   8500,   8516,   8517,   8525,   8524}},
 {  4711,  20, {  8493,   8494,   8502,   8501,   8517,   8518,   8526,   8525}},
 {  4712,  20, {  8494,   8495,   8503,   8502,   8518,   8519,   8527,   8526}},
 {  4713,  20, {  8495,   8496,   8504,   8503,   8519,   8520,   8528,   8527}},
 {  4714,  20, {  8496,   8497,   8505,   8504,   8520,   8521,   8529,   8528}},
 {  4715,  20, {  8237,   8498,   8506,   8246,   8264,   8522,   8530,   8273}},
 {  4716,  20, {  8498,   8499,   8507,   8506,   8522,   8523,   8531,   8530}},
 {  4717,  20, {  8499,   8500,   8508,   8507,   8523,   8524,   8532,   8531}},
 {  4718,  20, {  8500,   8501,   8509,   8508,   8524,   8525,   8533,   8532}},
 {  4719,  20, {  8501,   8502,   8510,   8509,   8525,   8526,   8534,   8533}},
 {  4720,  20, {  8502,   8503,   8511,   8510,   8526,   8527,   8535,   8534}},
 {  4721,  20, {  8503,   8504,   8512,   8511,   8527,   8528,   8536,   8535}},
 {  4722,  20, {  8504,   8505,   8513,   8512,   8528,   8529,   8537,   8536}},
 {  4723,  20, {  8255,   8514,   8522,   8264,   8282,   8538,   8546,   8291}},
 {  4724,  20, {  8514,   8515,   8523,   8522,   8538,   8539,   8547,   8546}},
 {  4725,  20, {  8515,   8516,   8524,   8523,   8539,   8540,   8548,   8547}},
 {  4726,  20, {  8516,   8517,   8525,   8524,   8540,   8541,   8549,   8548}},
 {  4727,  20, {  8517,   8518,   8526,   8525,   8541,   8542,   8550,   8549}},
 {  4728,  20, {  8518,   8519,   8527,   8526,   8542,   8543,   8551,   8550}},
 {  4729,  20, {  8519,   8520,   8528,   8527,   8543,   8544,   8552,   8551}},
 {  4730,  20, {  8520,   8521,   8529,   8528,   8544,   8545,   8553,   8552}},
 {  4731,  20, {  8264,   8522,   8530,   8273,   8291,   8546,   8554,   8300}},
 {  4732,  20, {  8522,   8523,   8531,   8530,   8546,   8547,   8555,   8554}},
 {  4733,  20, {  8523,   8524,   8532,   8531,   8547,   8548,   8556,   8555}},
 {  4734,  20, {  8524,   8525,   8533,   8532,   8548,   8549,   8557,   8556}},
 {  4735,  20, {  8525,   8526,   8534,   8533,   8549,   8550,   8558,   8557}},
 {  4736,  20, {  8526,   8527,   8535,   8534,   8550,   8551,   8559,   8558}},
 {  4737,  20, {  8527,   8528,   8536,   8535,   8551,   8552,   8560,   8559}},
 {  4738,  20, {  8528,   8529,   8537,   8536,   8552,   8553,   8561,   8560}},
 {  4739,  20, {  8282,   8538,   8546,   8291,   8309,   8562,   8570,   8318}},
 {  4740,  20, {  8538,   8539,   8547,   8546,   8562,   8563,   8571,   8570}},
 {  4741,  20, {  8539,   8540,   8548,   8547,   8563,   8564,   8572,   8571}},
 {  4742,  20, {  8540,   8541,   8549,   8548,   8564,   8565,   8573,   8572}},
 {  4743,  20, {  8541,   8542,   8550,   8549,   8565,   8566,   8574,   8573}},
 {  4744,  20, {  8542,   8543,   8551,   8550,   8566,   8567,   8575,   8574}},
 {  4745,  20, {  8543,   8544,   8552,   8551,   8567,   8568,   8576,   8575}},
 {  4746,  20, {  8544,   8545,   8553,   8552,   8568,   8569,   8577,   8576}},
 {  4747,  20, {  8291,   8546,   8554,   8300,   8318,   8570,   8578,   8327}},
 {  4748,  20, {  8546,   8547,   8555,   8554,   8570,   8571,   8579,   8578}},
 {  4749,  20, {  8547,   8548,   8556,   8555,   8571,   8572,   8580,   8579}},
 {  4750,  20, {  8548,   8549,   8557,   8556,   8572,   8573,   8581,   8580}},
 {  4751,  20, {  8549,   8550,   8558,   8557,   8573,   8574,   8582,   8581}},
 {  4752,  20, {  8550,   8551,   8559,   8558,   8574,   8575,   8583,   8582}},
 {  4753,  20, {  8551,   8552,   8560,   8559,   8575,   8576,   8584,   8583}},
 {  4754,  20, {  8552,   8553,   8561,   8560,   8576,   8577,   8585,   8584}},
 {  4755,  20, {  8309,   8562,   8570,   8318,   8336,   8586,   8594,   8345}},
 {  4756,  20, {  8562,   8563,   8571,   8570,   8586,   8587,   8595,   8594}},
 {  4757,  20, {  8563,   8564,   8572,   8571,   8587,   8588,   8596,   8595}},
 {  4758,  20, {  8564,   8565,   8573,   8572,   8588,   8589,   8597,   8596}},
 {  4759,  20, {  8565,   8566,   8574,   8573,   8589,   8590,   8598,   8597}},
 {  4760,  20, {  8566,   8567,   8575,   8574,   8590,   8591,   8599,   8598}},
 {  4761,  20, {  8567,   8568,   8576,   8575,   8591,   8592,   8600,   8599}},
 {  4762,  20, {  8568,   8569,   8577,   8576,   8592,   8593,   8601,   8600}},
 {  4763,  20, {  8318,   8570,   8578,   8327,   8345,   8594,   8602,   8354}},
 {  4764,  20, {  8570,   8571,   8579,   8578,   8594,   8595,   8603,   8602}},
 {  4765,  20, {  8571,   8572,   8580,   8579,   8595,   8596,   8604,   8603}},
 {  4766,  20, {  8572,   8573,   8581,   8580,   8596,   8597,   8605,   8604}},
 {  4767,  20, {  8573,   8574,   8582,   8581,   8597,   8598,   8606,   8605}},
 {  4768,  20, {  8574,   8575,   8583,   8582,   8598,   8599,   8607,   8606}},
 {  4769,  20, {  8575,   8576,   8584,   8583,   8599,   8600,   8608,   8607}},
 {  4770,  20, {  8576,   8577,   8585,   8584,   8600,   8601,   8609,   8608}},
 {  4771,  20, {  8336,   8586,   8594,   8345,   8363,   8610,   8618,   8372}},
 {  4772,  20, {  8586,   8587,   8595,   8594,   8610,   8611,   8619,   8618}},
 {  4773,  20, {  8587,   8588,   8596,   8595,   8611,   8612,   8620,   8619}},
 {  4774,  20, {  8588,   8589,   8597,   8596,   8612,   8613,   8621,   8620}},
 {  4775,  20, {  8589,   8590,   8598,   8597,   8613,   8614,   8622,   8621}},
 {  4776,  20, {  8590,   8591,   8599,   8598,   8614,   8615,   8623,   8622}},
 {  4777,  20, {  8591,   8592,   8600,   8599,   8615,   8616,   8624,   8623}},
 {  4778,  20, {  8592,   8593,   8601,   8600,   8616,   8617,   8625,   8624}},
 {  4779,  20, {  8345,   8594,   8602,   8354,   8372,   8618,   8626,   8381}},
 {  4780,  20, {  8594,   8595,   8603,   8602,   8618,   8619,   8627,   8626}},
 {  4781,  20, {  8595,   8596,   8604,   8603,   8619,   8620,   8628,   8627}},
 {  4782,  20, {  8596,   8597,   8605,   8604,   8620,   8621,   8629,   8628}},
 {  4783,  20, {  8597,   8598,   8606,   8605,   8621,   8622,   8630,   8629}},
 {  4784,  20, {  8598,   8599,   8607,   8606,   8622,   8623,   8631,   8630}},
 {  4785,  20, {  8599,   8600,   8608,   8607,   8623,   8624,   8632,   8631}},
 {  4786,  20, {  8600,   8601,   8609,   8608,   8624,   8625,   8633,   8632}},
 {  4787,  20, {  8363,   8610,   8618,   8372,   8390,   8634,   8642,   8399}},
 {  4788,  20, {  8610,   8611,   8619,   8618,   8634,   8635,   8643,   8642}},
 {  4789,  20, {  8611,   8612,   8620,   8619,   8635,   8636,   8644,   8643}},
 {  4790,  20, {  8612,   8613,   8621,   8620,   8636,   8637,   8645,   8644}},
 {  4791,  20, {  8613,   8614,   8622,   8621,   8637,   8638,   8646,   8645}},
 {  4792,  20, {  8614,   8615,   8623,   8622,   8638,   8639,   8647,   8646}},
 {  4793,  20, {  8615,   8616,   8624,   8623,   8639,   8640,   8648,   8647}},
 {  4794,  20, {  8616,   8617,   8625,   8624,   8640,   8641,   8649,   8648}},
 {  4795,  20, {  8372,   8618,   8626,   8381,   8399,   8642,   8650,   8408}},
 {  4796,  20, {  8618,   8619,   8627,   8626,   8642,   8643,   8651,   8650}},
 {  4797,  20, {  8619,   8620,   8628,   8627,   8643,   8644,   8652,   8651}},
 {  4798,  20, {  8620,   8621,   8629,   8628,   8644,   8645,   8653,   8652}},
 {  4799,  20, {  8621,   8622,   8630,   8629,   8645,   8646,   8654,   8653}},
 {  4800,  20, {  8622,   8623,   8631,   8630,   8646,   8647,   8655,   8654}},
 {  4801,  20, {  8623,   8624,   8632,   8631,   8647,   8648,   8656,   8655}},
 {  4802,  20, {  8624,   8625,   8633,   8632,   8648,   8649,   8657,   8656}},
 {  4803,  20, {  8390,   8634,   8642,   8399,   8417,   8658,   8666,   8426}},
 {  4804,  20, {  8634,   8635,   8643,   8642,   8658,   8659,   8667,   8666}},
 {  4805,  20, {  8635,   8636,   8644,   8643,   8659,   8660,   8668,   8667}},
 {  4806,  20, {  8636,   8637,   8645,   8644,   8660,   8661,   8669,   8668}},
 {  4807,  20, {  8637,   8638,   8646,   8645,   8661,   8662,   8670,   8669}},
 {  4808,  20, {  8638,   8639,   8647,   8646,   8662,   8663,   8671,   8670}},
 {  4809,  20, {  8639,   8640,   8648,   8647,   8663,   8664,   8672,   8671}},
 {  4810,  20, {  8640,   8641,   8649,   8648,   8664,   8665,   8673,   8672}},
 {  4811,  20, {  8399,   8642,   8650,   8408,   8426,   8666,   8674,   8435}},
 {  4812,  20, {  8642,   8643,   8651,   8650,   8666,   8667,   8675,   8674}},
 {  4813,  20, {  8643,   8644,   8652,   8651,   8667,   8668,   8676,   8675}},
 {  4814,  20, {  8644,   8645,   8653,   8652,   8668,   8669,   8677,   8676}},
 {  4815,  20, {  8645,   8646,   8654,   8653,   8669,   8670,   8678,   8677}},
 {  4816,  20, {  8646,   8647,   8655,   8654,   8670,   8671,   8679,   8678}},
 {  4817,  20, {  8647,   8648,   8656,   8655,   8671,   8672,   8680,   8679}},
 {  4818,  20, {  8648,   8649,   8657,   8656,   8672,   8673,   8681,   8680}},
 {  4819,  20, {  8417,   8658,   8666,   8426,   8444,   8682,   8690,   8453}},
 {  4820,  20, {  8658,   8659,   8667,   8666,   8682,   8683,   8691,   8690}},
 {  4821,  20, {  8659,   8660,   8668,   8667,   8683,   8684,   8692,   8691}},
 {  4822,  20, {  8660,   8661,   8669,   8668,   8684,   8685,   8693,   8692}},
 {  4823,  20, {  8661,   8662,   8670,   8669,   8685,   8686,   8694,   8693}},
 {  4824,  20, {  8662,   8663,   8671,   8670,   8686,   8687,   8695,   8694}},
 {  4825,  20, {  8663,   8664,   8672,   8671,   8687,   8688,   8696,   8695}},
 {  4826,  20, {  8664,   8665,   8673,   8672,   8688,   8689,   8697,   8696}},
 {  4827,  20, {  8426,   8666,   8674,   8435,   8453,   8690,   8698,   8462}},
 {  4828,  20, {  8666,   8667,   8675,   8674,   8690,   8691,   8699,   8698}},
 {  4829,  20, {  8667,   8668,   8676,   8675,   8691,   8692,   8700,   8699}},
 {  4830,  20, {  8668,   8669,   8677,   8676,   8692,   8693,   8701,   8700}},
 {  4831,  20, {  8669,   8670,   8678,   8677,   8693,   8694,   8702,   8701}},
 {  4832,  20, {  8670,   8671,   8679,   8678,   8694,   8695,   8703,   8702}},
 {  4833,  20, {  8671,   8672,   8680,   8679,   8695,   8696,   8704,   8703}},
 {  4834,  20, {  8672,   8673,   8681,   8680,   8696,   8697,   8705,   8704}},
 {  4835,  20, {  8444,   8682,   8690,   8453,   8471,   8706,   8714,   8480}},
 {  4836,  20, {  8682,   8683,   8691,   8690,   8706,   8707,   8715,   8714}},
 {  4837,  20, {  8683,   8684,   8692,   8691,   8707,   8708,   8716,   8715}},
 {  4838,  20, {  8684,   8685,   8693,   8692,   8708,   8709,   8717,   8716}},
 {  4839,  20, {  8685,   8686,   8694,   8693,   8709,   8710,   8718,   8717}},
 {  4840,  20, {  8686,   8687,   8695,   8694,   8710,   8711,   8719,   8718}},
 {  4841,  20, {  8687,   8688,   8696,   8695,   8711,   8712,   8720,   8719}},
 {  4842,  20, {  8688,   8689,   8697,   8696,   8712,   8713,   8721,   8720}},
 {  4843,  20, {  8453,   8690,   8698,   8462,   8480,   8714,   8722,   8489}},
 {  4844,  20, {  8690,   8691,   8699,   8698,   8714,   8715,   8723,   8722}},
 {  4845,  20, {  8691,   8692,   8700,   8699,   8715,   8716,   8724,   8723}},
 {  4846,  20, {  8692,   8693,   8701,   8700,   8716,   8717,   8725,   8724}},
 {  4847,  20, {  8693,   8694,   8702,   8701,   8717,   8718,   8726,   8725}},
 {  4848,  20, {  8694,   8695,   8703,   8702,   8718,   8719,   8727,   8726}},
 {  4849,  20, {  8695,   8696,   8704,   8703,   8719,   8720,   8728,   8727}},
 {  4850,  20, {  8696,   8697,   8705,   8704,   8720,   8721,   8729,   8728}},
 {  4851,  20, {  8497,   8730,   8732,   8505,   8521,   8736,   8738,   8529}},
 {  4852,  20, {  8730,   8731,   8733,   8732,   8736,   8737,   8739,   8738}},
 {  4853,  20, {  8505,   8732,   8734,   8513,   8529,   8738,   8740,   8537}},
 {  4854,  20, {  8732,   8733,   8735,   8734,   8738,   8739,   8741,   8740}},
 {  4855,  20, {  8521,   8736,   8738,   8529,   8545,   8742,   8744,   8553}},
 {  4856,  20, {  8736,   8737,   8739,   8738,   8742,   8743,   8745,   8744}},
 {  4857,  20, {  8529,   8738,   8740,   8537,   8553,   8744,   8746,   8561}},
 {  4858,  20, {  8738,   8739,   8741,   8740,   8744,   8745,   8747,   8746}},
 {  4859,  20, {  8545,   8742,   8744,   8553,   8569,   8748,   8750,   8577}},
 {  4860,  20, {  8742,   8743,   8745,   8744,   8748,   8749,   8751,   8750}},
 {  4861,  20, {  8553,   8744,   8746,   8561,   8577,   8750,   8752,   8585}},
 {  4862,  20, {  8744,   8745,   8747,   8746,   8750,   8751,   8753,   8752}},
 {  4863,  20, {  8569,   8748,   8750,   8577,   8593,   8754,   8756,   8601}},
 {  4864,  20, {  8748,   8749,   8751,   8750,   8754,   8755,   8757,   8756}},
 {  4865,  20, {  8577,   8750,   8752,   8585,   8601,   8756,   8758,   8609}},
 {  4866,  20, {  8750,   8751,   8753,   8752,   8756,   8757,   8759,   8758}},
 {  4867,  20, {  8593,   8754,   8756,   8601,   8617,   8760,   8762,   8625}},
 {  4868,  20, {  8754,   8755,   8757,   8756,   8760,   8761,   8763,   8762}},
 {  4869,  20, {  8601,   8756,   8758,   8609,   8625,   8762,   8764,   8633}},
 {  4870,  20, {  8756,   8757,   8759,   8758,   8762,   8763,   8765,   8764}},
 {  4871,  20, {  8617,   8760,   8762,   8625,   8641,   8766,   8768,   8649}},
 {  4872,  20, {  8760,   8761,   8763,   8762,   8766,   8767,   8769,   8768}},
 {  4873,  20, {  8625,   8762,   8764,   8633,   8649,   8768,   8770,   8657}},
 {  4874,  20, {  8762,   8763,   8765,   8764,   8768,   8769,   8771,   8770}},
 {  4875,  20, {  8641,   8766,   8768,   8649,   8665,   8772,   8774,   8673}},
 {  4876,  20, {  8766,   8767,   8769,   8768,   8772,   8773,   8775,   8774}},
 {  4877,  20, {  8649,   8768,   8770,   8657,   8673,   8774,   8776,   8681}},
 {  4878,  20, {  8768,   8769,   8771,   8770,   8774,   8775,   8777,   8776}},
 {  4879,  20, {  8665,   8772,   8774,   8673,   8689,   8778,   8780,   8697}},
 {  4880,  20, {  8772,   8773,   8775,   8774,   8778,   8779,   8781,   8780}},
 {  4881,  20, {  8673,   8774,   8776,   8681,   8697,   8780,   8782,   8705}},
 {  4882,  20, {  8774,   8775,   8777,   8776,   8780,   8781,   8783,   8782}},
 {  4883,  20, {  8689,   8778,   8780,   8697,   8713,   8784,   8786,   8721}},
 {  4884,  20, {  8778,   8779,   8781,   8780,   8784,   8785,   8787,   8786}},
 {  4885,  20, {  8697,   8780,   8782,   8705,   8721,   8786,   8788,   8729}},
 {  4886,  20, {  8780,   8781,   8783,   8782,   8786,   8787,   8789,   8788}},
 {  4887,  20, {  8731,   8790,   8792,   8733,   8737,   8796,   8798,   8739}},
 {  4888,  20, {  8790,   8791,   8793,   8792,   8796,   8797,   8799,   8798}},
 {  4889,  20, {  8733,   8792,   8794,   8735,   8739,   8798,   8800,   8741}},
 {  4890,  20, {  8792,   8793,   8795,   8794,   8798,   8799,   8801,   8800}},
 {  4891,  20, {  8737,   8796,   8798,   8739,   8743,   8802,   8804,   8745}},
 {  4892,  20, {  8796,   8797,   8799,   8798,   8802,   8803,   8805,   8804}},
 {  4893,  20, {  8739,   8798,   8800,   8741,   8745,   8804,   8806,   8747}},
 {  4894,  20, {  8798,   8799,   8801,   8800,   8804,   8805,   8807,   8806}},
 {  4895,  20, {  8743,   8802,   8804,   8745,   8749,   8808,   8810,   8751}},
 {  4896,  20, {  8802,   8803,   8805,   8804,   8808,   8809,   8811,   8810}},
 {  4897,  20, {  8745,   8804,   8806,   8747,   8751,   8810,   8812,   8753}},
 {  4898,  20, {  8804,   8805,   8807,   8806,   8810,   8811,   8813,   8812}},
 {  4899,  20, {  8749,   8808,   8810,   8751,   8755,   8814,   8816,   8757}},
 {  4900,  20, {  8808,   8809,   8811,   8810,   8814,   8815,   8817,   8816}},
 {  4901,  20, {  8751,   8810,   8812,   8753,   8757,   8816,   8818,   8759}},
 {  4902,  20, {  8810,   8811,   8813,   8812,   8816,   8817,   8819,   8818}},
 {  4903,  20, {  8755,   8814,   8816,   8757,   8761,   8820,   8822,   8763}},
 {  4904,  20, {  8814,   8815,   8817,   8816,   8820,   8821,   8823,   8822}},
 {  4905,  20, {  8757,   8816,   8818,   8759,   8763,   8822,   8824,   8765}},
 {  4906,  20, {  8816,   8817,   8819,   8818,   8822,   8823,   8825,   8824}},
 {  4907,  20, {  8761,   8820,   8822,   8763,   8767,   8826,   8828,   8769}},
 {  4908,  20, {  8820,   8821,   8823,   8822,   8826,   8827,   8829,   8828}},
 {  4909,  20, {  8763,   8822,   8824,   8765,   8769,   8828,   8830,   8771}},
 {  4910,  20, {  8822,   8823,   8825,   8824,   8828,   8829,   8831,   8830}},
 {  4911,  20, {  8767,   8826,   8828,   8769,   8773,   8832,   8834,   8775}},
 {  4912,  20, {  8826,   8827,   8829,   8828,   8832,   8833,   8835,   8834}},
 {  4913,  20, {  8769,   8828,   8830,   8771,   8775,   8834,   8836,   8777}},
 {  4914,  20, {  8828,   8829,   8831,   8830,   8834,   8835,   8837,   8836}},
 {  4915,  20, {  8773,   8832,   8834,   8775,   8779,   8838,   8840,   8781}},
 {  4916,  20, {  8832,   8833,   8835,   8834,   8838,   8839,   8841,   8840}},
 {  4917,  20, {  8775,   8834,   8836,   8777,   8781,   8840,   8842,   8783}},
 {  4918,  20, {  8834,   8835,   8837,   8836,   8840,   8841,   8843,   8842}},
 {  4919,  20, {  8779,   8838,   8840,   8781,   8785,   8844,   8846,   8787}},
 {  4920,  20, {  8838,   8839,   8841,   8840,   8844,   8845,   8847,   8846}},
 {  4921,  20, {  8781,   8840,   8842,   8783,   8787,   8846,   8848,   8789}},
 {  4922,  20, {  8840,   8841,   8843,   8842,   8846,   8847,   8849,   8848}},
 {  4923,  20, {  8791,   8850,   8862,   8793,   8797,   8886,   8898,   8799}},
 {  4924,  20, {  8850,   8851,   8863,   8862,   8886,   8887,   8899,   8898}},
 {  4925,  20, {  8851,   8852,   8864,   8863,   8887,   8888,   8900,   8899}},
 {  4926,  20, {  8852,   8853,   8865,   8864,   8888,   8889,   8901,   8900}},
 {  4927,  20, {  8853,   8854,   8866,   8865,   8889,   8890,   8902,   8901}},
 {  4928,  20, {  8854,   8855,   8867,   8866,   8890,   8891,   8903,   8902}},
 {  4929,  20, {  8855,   8856,   8868,   8867,   8891,   8892,   8904,   8903}},
 {  4930,  20, {  8856,   8857,   8869,   8868,   8892,   8893,   8905,   8904}},
 {  4931,  20, {  8857,   8858,   8870,   8869,   8893,   8894,   8906,   8905}},
 {  4932,  20, {  8858,   8859,   8871,   8870,   8894,   8895,   8907,   8906}},
 {  4933,  20, {  8859,   8860,   8872,   8871,   8895,   8896,   8908,   8907}},
 {  4934,  20, {  8860,   8861,   8873,   8872,   8896,   8897,   8909,   8908}},
 {  4935,  20, {  8793,   8862,   8874,   8795,   8799,   8898,   8910,   8801}},
 {  4936,  20, {  8862,   8863,   8875,   8874,   8898,   8899,   8911,   8910}},
 {  4937,  20, {  8863,   8864,   8876,   8875,   8899,   8900,   8912,   8911}},
 {  4938,  20, {  8864,   8865,   8877,   8876,   8900,   8901,   8913,   8912}},
 {  4939,  20, {  8865,   8866,   8878,   8877,   8901,   8902,   8914,   8913}},
 {  4940,  20, {  8866,   8867,   8879,   8878,   8902,   8903,   8915,   8914}},
 {  4941,  20, {  8867,   8868,   8880,   8879,   8903,   8904,   8916,   8915}},
 {  4942,  20, {  8868,   8869,   8881,   8880,   8904,   8905,   8917,   8916}},
 {  4943,  20, {  8869,   8870,   8882,   8881,   8905,   8906,   8918,   8917}},
 {  4944,  20, {  8870,   8871,   8883,   8882,   8906,   8907,   8919,   8918}},
 {  4945,  20, {  8871,   8872,   8884,   8883,   8907,   8908,   8920,   8919}},
 {  4946,  20, {  8872,   8873,   8885,   8884,   8908,   8909,   8921,   8920}},
 {  4947,  20, {  8797,   8886,   8898,   8799,   8803,   8922,   8934,   8805}},
 {  4948,  20, {  8886,   8887,   8899,   8898,   8922,   8923,   8935,   8934}},
 {  4949,  20, {  8887,   8888,   8900,   8899,   8923,   8924,   8936,   8935}},
 {  4950,  20, {  8888,   8889,   8901,   8900,   8924,   8925,   8937,   8936}},
 {  4951,  20, {  8889,   8890,   8902,   8901,   8925,   8926,   8938,   8937}},
 {  4952,  20, {  8890,   8891,   8903,   8902,   8926,   8927,   8939,   8938}},
 {  4953,  20, {  8891,   8892,   8904,   8903,   8927,   8928,   8940,   8939}},
 {  4954,  20, {  8892,   8893,   8905,   8904,   8928,   8929,   8941,   8940}},
 {  4955,  20, {  8893,   8894,   8906,   8905,   8929,   8930,   8942,   8941}},
 {  4956,  20, {  8894,   8895,   8907,   8906,   8930,   8931,   8943,   8942}},
 {  4957,  20, {  8895,   8896,   8908,   8907,   8931,   8932,   8944,   8943}},
 {  4958,  20, {  8896,   8897,   8909,   8908,   8932,   8933,   8945,   8944}},
 {  4959,  20, {  8799,   8898,   8910,   8801,   8805,   8934,   8946,   8807}},
 {  4960,  20, {  8898,   8899,   8911,   8910,   8934,   8935,   8947,   8946}},
 {  4961,  20, {  8899,   8900,   8912,   8911,   8935,   8936,   8948,   8947}},
 {  4962,  20, {  8900,   8901,   8913,   8912,   8936,   8937,   8949,   8948}},
 {  4963,  20, {  8901,   8902,   8914,   8913,   8937,   8938,   8950,   8949}},
 {  4964,  20, {  8902,   8903,   8915,   8914,   8938,   8939,   8951,   8950}},
 {  4965,  20, {  8903,   8904,   8916,   8915,   8939,   8940,   8952,   8951}},
 {  4966,  20, {  8904,   8905,   8917,   8916,   8940,   8941,   8953,   8952}},
 {  4967,  20, {  8905,   8906,   8918,   8917,   8941,   8942,   8954,   8953}},
 {  4968,  20, {  8906,   8907,   8919,   8918,   8942,   8943,   8955,   8954}},
 {  4969,  20, {  8907,   8908,   8920,   8919,   8943,   8944,   8956,   8955}},
 {  4970,  20, {  8908,   8909,   8921,   8920,   8944,   8945,   8957,   8956}},
 {  4971,  20, {  8803,   8922,   8934,   8805,   8809,   8958,   8970,   8811}},
 {  4972,  20, {  8922,   8923,   8935,   8934,   8958,   8959,   8971,   8970}},
 {  4973,  20, {  8923,   8924,   8936,   8935,   8959,   8960,   8972,   8971}},
 {  4974,  20, {  8924,   8925,   8937,   8936,   8960,   8961,   8973,   8972}},
 {  4975,  20, {  8925,   8926,   8938,   8937,   8961,   8962,   8974,   8973}},
 {  4976,  20, {  8926,   8927,   8939,   8938,   8962,   8963,   8975,   8974}},
 {  4977,  20, {  8927,   8928,   8940,   8939,   8963,   8964,   8976,   8975}},
 {  4978,  20, {  8928,   8929,   8941,   8940,   8964,   8965,   8977,   8976}},
 {  4979,  20, {  8929,   8930,   8942,   8941,   8965,   8966,   8978,   8977}},
 {  4980,  20, {  8930,   8931,   8943,   8942,   8966,   8967,   8979,   8978}},
 {  4981,  20, {  8931,   8932,   8944,   8943,   8967,   8968,   8980,   8979}},
 {  4982,  20, {  8932,   8933,   8945,   8944,   8968,   8969,   8981,   8980}},
 {  4983,  20, {  8805,   8934,   8946,   8807,   8811,   8970,   8982,   8813}},
 {  4984,  20, {  8934,   8935,   8947,   8946,   8970,   8971,   8983,   8982}},
 {  4985,  20, {  8935,   8936,   8948,   8947,   8971,   8972,   8984,   8983}},
 {  4986,  20, {  8936,   8937,   8949,   8948,   8972,   8973,   8985,   8984}},
 {  4987,  20, {  8937,   8938,   8950,   8949,   8973,   8974,   8986,   8985}},
 {  4988,  20, {  8938,   8939,   8951,   8950,   8974,   8975,   8987,   8986}},
 {  4989,  20, {  8939,   8940,   8952,   8951,   8975,   8976,   8988,   8987}},
 {  4990,  20, {  8940,   8941,   8953,   8952,   8976,   8977,   8989,   8988}},
 {  4991,  20, {  8941,   8942,   8954,   8953,   8977,   8978,   8990,   8989}},
 {  4992,  20, {  8942,   8943,   8955,   8954,   8978,   8979,   8991,   8990}},
 {  4993,  20, {  8943,   8944,   8956,   8955,   8979,   8980,   8992,   8991}},
 {  4994,  20, {  8944,   8945,   8957,   8956,   8980,   8981,   8993,   8992}},
 {  4995,  20, {  8809,   8958,   8970,   8811,   8815,   8994,   9006,   8817}},
 {  4996,  20, {  8958,   8959,   8971,   8970,   8994,   8995,   9007,   9006}},
 {  4997,  20, {  8959,   8960,   8972,   8971,   8995,   8996,   9008,   9007}},
 {  4998,  20, {  8960,   8961,   8973,   8972,   8996,   8997,   9009,   9008}},
 {  4999,  20, {  8961,   8962,   8974,   8973,   8997,   8998,   9010,   9009}},
 {  5000,  20, {  8962,   8963,   8975,   8974,   8998,   8999,   9011,   9010}},
 {  5001,  20, {  8963,   8964,   8976,   8975,   8999,   9000,   9012,   9011}},
 {  5002,  20, {  8964,   8965,   8977,   8976,   9000,   9001,   9013,   9012}},
 {  5003,  20, {  8965,   8966,   8978,   8977,   9001,   9002,   9014,   9013}},
 {  5004,  20, {  8966,   8967,   8979,   8978,   9002,   9003,   9015,   9014}},
 {  5005,  20, {  8967,   8968,   8980,   8979,   9003,   9004,   9016,   9015}},
 {  5006,  20, {  8968,   8969,   8981,   8980,   9004,   9005,   9017,   9016}},
 {  5007,  20, {  8811,   8970,   8982,   8813,   8817,   9006,   9018,   8819}},
 {  5008,  20, {  8970,   8971,   8983,   8982,   9006,   9007,   9019,   9018}},
 {  5009,  20, {  8971,   8972,   8984,   8983,   9007,   9008,   9020,   9019}},
 {  5010,  20, {  8972,   8973,   8985,   8984,   9008,   9009,   9021,   9020}},
 {  5011,  20, {  8973,   8974,   8986,   8985,   9009,   9010,   9022,   9021}},
 {  5012,  20, {  8974,   8975,   8987,   8986,   9010,   9011,   9023,   9022}},
 {  5013,  20, {  8975,   8976,   8988,   8987,   9011,   9012,   9024,   9023}},
 {  5014,  20, {  8976,   8977,   8989,   8988,   9012,   9013,   9025,   9024}},
 {  5015,  20, {  8977,   8978,   8990,   8989,   9013,   9014,   9026,   9025}},
 {  5016,  20, {  8978,   8979,   8991,   8990,   9014,   9015,   9027,   9026}},
 {  5017,  20, {  8979,   8980,   8992,   8991,   9015,   9016,   9028,   9027}},
 {  5018,  20, {  8980,   8981,   8993,   8992,   9016,   9017,   9029,   9028}},
 {  5019,  20, {  8815,   8994,   9006,   8817,   8821,   9030,   9042,   8823}},
 {  5020,  20, {  8994,   8995,   9007,   9006,   9030,   9031,   9043,   9042}},
 {  5021,  20, {  8995,   8996,   9008,   9007,   9031,   9032,   9044,   9043}},
 {  5022,  20, {  8996,   8997,   9009,   9008,   9032,   9033,   9045,   9044}},
 {  5023,  20, {  8997,   8998,   9010,   9009,   9033,   9034,   9046,   9045}},
 {  5024,  20, {  8998,   8999,   9011,   9010,   9034,   9035,   9047,   9046}},
 {  5025,  20, {  8999,   9000,   9012,   9011,   9035,   9036,   9048,   9047}},
 {  5026,  20, {  9000,   9001,   9013,   9012,   9036,   9037,   9049,   9048}},
 {  5027,  20, {  9001,   9002,   9014,   9013,   9037,   9038,   9050,   9049}},
 {  5028,  20, {  9002,   9003,   9015,   9014,   9038,   9039,   9051,   9050}},
 {  5029,  20, {  9003,   9004,   9016,   9015,   9039,   9040,   9052,   9051}},
 {  5030,  20, {  9004,   9005,   9017,   9016,   9040,   9041,   9053,   9052}},
 {  5031,  20, {  8817,   9006,   9018,   8819,   8823,   9042,   9054,   8825}},
 {  5032,  20, {  9006,   9007,   9019,   9018,   9042,   9043,   9055,   9054}},
 {  5033,  20, {  9007,   9008,   9020,   9019,   9043,   9044,   9056,   9055}},
 {  5034,  20, {  9008,   9009,   9021,   9020,   9044,   9045,   9057,   9056}},
 {  5035,  20, {  9009,   9010,   9022,   9021,   9045,   9046,   9058,   9057}},
 {  5036,  20, {  9010,   9011,   9023,   9022,   9046,   9047,   9059,   9058}},
 {  5037,  20, {  9011,   9012,   9024,   9023,   9047,   9048,   9060,   9059}},
 {  5038,  20, {  9012,   9013,   9025,   9024,   9048,   9049,   9061,   9060}},
 {  5039,  20, {  9013,   9014,   9026,   9025,   9049,   9050,   9062,   9061}},
 {  5040,  20, {  9014,   9015,   9027,   9026,   9050,   9051,   9063,   9062}},
 {  5041,  20, {  9015,   9016,   9028,   9027,   9051,   9052,   9064,   9063}},
 {  5042,  20, {  9016,   9017,   9029,   9028,   9052,   9053,   9065,   9064}},
 {  5043,  20, {  8821,   9030,   9042,   8823,   8827,   9066,   9078,   8829}},
 {  5044,  20, {  9030,   9031,   9043,   9042,   9066,   9067,   9079,   9078}},
 {  5045,  20, {  9031,   9032,   9044,   9043,   9067,   9068,   9080,   9079}},
 {  5046,  20, {  9032,   9033,   9045,   9044,   9068,   9069,   9081,   9080}},
 {  5047,  20, {  9033,   9034,   9046,   9045,   9069,   9070,   9082,   9081}},
 {  5048,  20, {  9034,   9035,   9047,   9046,   9070,   9071,   9083,   9082}},
 {  5049,  20, {  9035,   9036,   9048,   9047,   9071,   9072,   9084,   9083}},
 {  5050,  20, {  9036,   9037,   9049,   9048,   9072,   9073,   9085,   9084}},
 {  5051,  20, {  9037,   9038,   9050,   9049,   9073,   9074,   9086,   9085}},
 {  5052,  20, {  9038,   9039,   9051,   9050,   9074,   9075,   9087,   9086}},
 {  5053,  20, {  9039,   9040,   9052,   9051,   9075,   9076,   9088,   9087}},
 {  5054,  20, {  9040,   9041,   9053,   9052,   9076,   9077,   9089,   9088}},
 {  5055,  20, {  8823,   9042,   9054,   8825,   8829,   9078,   9090,   8831}},
 {  5056,  20, {  9042,   9043,   9055,   9054,   9078,   9079,   9091,   9090}},
 {  5057,  20, {  9043,   9044,   9056,   9055,   9079,   9080,   9092,   9091}},
 {  5058,  20, {  9044,   9045,   9057,   9056,   9080,   9081,   9093,   9092}},
 {  5059,  20, {  9045,   9046,   9058,   9057,   9081,   9082,   9094,   9093}},
 {  5060,  20, {  9046,   9047,   9059,   9058,   9082,   9083,   9095,   9094}},
 {  5061,  20, {  9047,   9048,   9060,   9059,   9083,   9084,   9096,   9095}},
 {  5062,  20, {  9048,   9049,   9061,   9060,   9084,   9085,   9097,   9096}},
 {  5063,  20, {  9049,   9050,   9062,   9061,   9085,   9086,   9098,   9097}},
 {  5064,  20, {  9050,   9051,   9063,   9062,   9086,   9087,   9099,   9098}},
 {  5065,  20, {  9051,   9052,   9064,   9063,   9087,   9088,   9100,   9099}},
 {  5066,  20, {  9052,   9053,   9065,   9064,   9088,   9089,   9101,   9100}},
 {  5067,  20, {  8827,   9066,   9078,   8829,   8833,   9102,   9114,   8835}},
 {  5068,  20, {  9066,   9067,   9079,   9078,   9102,   9103,   9115,   9114}},
 {  5069,  20, {  9067,   9068,   9080,   9079,   9103,   9104,   9116,   9115}},
 {  5070,  20, {  9068,   9069,   9081,   9080,   9104,   9105,   9117,   9116}},
 {  5071,  20, {  9069,   9070,   9082,   9081,   9105,   9106,   9118,   9117}},
 {  5072,  20, {  9070,   9071,   9083,   9082,   9106,   9107,   9119,   9118}},
 {  5073,  20, {  9071,   9072,   9084,   9083,   9107,   9108,   9120,   9119}},
 {  5074,  20, {  9072,   9073,   9085,   9084,   9108,   9109,   9121,   9120}},
 {  5075,  20, {  9073,   9074,   9086,   9085,   9109,   9110,   9122,   9121}},
 {  5076,  20, {  9074,   9075,   9087,   9086,   9110,   9111,   9123,   9122}},
 {  5077,  20, {  9075,   9076,   9088,   9087,   9111,   9112,   9124,   9123}},
 {  5078,  20, {  9076,   9077,   9089,   9088,   9112,   9113,   9125,   9124}},
 {  5079,  20, {  8829,   9078,   9090,   8831,   8835,   9114,   9126,   8837}},
 {  5080,  20, {  9078,   9079,   9091,   9090,   9114,   9115,   9127,   9126}},
 {  5081,  20, {  9079,   9080,   9092,   9091,   9115,   9116,   9128,   9127}},
 {  5082,  20, {  9080,   9081,   9093,   9092,   9116,   9117,   9129,   9128}},
 {  5083,  20, {  9081,   9082,   9094,   9093,   9117,   9118,   9130,   9129}},
 {  5084,  20, {  9082,   9083,   9095,   9094,   9118,   9119,   9131,   9130}},
 {  5085,  20, {  9083,   9084,   9096,   9095,   9119,   9120,   9132,   9131}},
 {  5086,  20, {  9084,   9085,   9097,   9096,   9120,   9121,   9133,   9132}},
 {  5087,  20, {  9085,   9086,   9098,   9097,   9121,   9122,   9134,   9133}},
 {  5088,  20, {  9086,   9087,   9099,   9098,   9122,   9123,   9135,   9134}},
 {  5089,  20, {  9087,   9088,   9100,   9099,   9123,   9124,   9136,   9135}},
 {  5090,  20, {  9088,   9089,   9101,   9100,   9124,   9125,   9137,   9136}},
 {  5091,  20, {  8833,   9102,   9114,   8835,   8839,   9138,   9150,   8841}},
 {  5092,  20, {  9102,   9103,   9115,   9114,   9138,   9139,   9151,   9150}},
 {  5093,  20, {  9103,   9104,   9116,   9115,   9139,   9140,   9152,   9151}},
 {  5094,  20, {  9104,   9105,   9117,   9116,   9140,   9141,   9153,   9152}},
 {  5095,  20, {  9105,   9106,   9118,   9117,   9141,   9142,   9154,   9153}},
 {  5096,  20, {  9106,   9107,   9119,   9118,   9142,   9143,   9155,   9154}},
 {  5097,  20, {  9107,   9108,   9120,   9119,   9143,   9144,   9156,   9155}},
 {  5098,  20, {  9108,   9109,   9121,   9120,   9144,   9145,   9157,   9156}},
 {  5099,  20, {  9109,   9110,   9122,   9121,   9145,   9146,   9158,   9157}},
 {  5100,  20, {  9110,   9111,   9123,   9122,   9146,   9147,   9159,   9158}},
 {  5101,  20, {  9111,   9112,   9124,   9123,   9147,   9148,   9160,   9159}},
 {  5102,  20, {  9112,   9113,   9125,   9124,   9148,   9149,   9161,   9160}},
 {  5103,  20, {  8835,   9114,   9126,   8837,   8841,   9150,   9162,   8843}},
 {  5104,  20, {  9114,   9115,   9127,   9126,   9150,   9151,   9163,   9162}},
 {  5105,  20, {  9115,   9116,   9128,   9127,   9151,   9152,   9164,   9163}},
 {  5106,  20, {  9116,   9117,   9129,   9128,   9152,   9153,   9165,   9164}},
 {  5107,  20, {  9117,   9118,   9130,   9129,   9153,   9154,   9166,   9165}},
 {  5108,  20, {  9118,   9119,   9131,   9130,   9154,   9155,   9167,   9166}},
 {  5109,  20, {  9119,   9120,   9132,   9131,   9155,   9156,   9168,   9167}},
 {  5110,  20, {  9120,   9121,   9133,   9132,   9156,   9157,   9169,   9168}},
 {  5111,  20, {  9121,   9122,   9134,   9133,   9157,   9158,   9170,   9169}},
 {  5112,  20, {  9122,   9123,   9135,   9134,   9158,   9159,   9171,   9170}},
 {  5113,  20, {  9123,   9124,   9136,   9135,   9159,   9160,   9172,   9171}},
 {  5114,  20, {  9124,   9125,   9137,   9136,   9160,   9161,   9173,   9172}},
 {  5115,  20, {  8839,   9138,   9150,   8841,   8845,   9174,   9186,   8847}},
 {  5116,  20, {  9138,   9139,   9151,   9150,   9174,   9175,   9187,   9186}},
 {  5117,  20, {  9139,   9140,   9152,   9151,   9175,   9176,   9188,   9187}},
 {  5118,  20, {  9140,   9141,   9153,   9152,   9176,   9177,   9189,   9188}},
 {  5119,  20, {  9141,   9142,   9154,   9153,   9177,   9178,   9190,   9189}},
 {  5120,  20, {  9142,   9143,   9155,   9154,   9178,   9179,   9191,   9190}},
 {  5121,  20, {  9143,   9144,   9156,   9155,   9179,   9180,   9192,   9191}},
 {  5122,  20, {  9144,   9145,   9157,   9156,   9180,   9181,   9193,   9192}},
 {  5123,  20, {  9145,   9146,   9158,   9157,   9181,   9182,   9194,   9193}},
 {  5124,  20, {  9146,   9147,   9159,   9158,   9182,   9183,   9195,   9194}},
 {  5125,  20, {  9147,   9148,   9160,   9159,   9183,   9184,   9196,   9195}},
 {  5126,  20, {  9148,   9149,   9161,   9160,   9184,   9185,   9197,   9196}},
 {  5127,  20, {  8841,   9150,   9162,   8843,   8847,   9186,   9198,   8849}},
 {  5128,  20, {  9150,   9151,   9163,   9162,   9186,   9187,   9199,   9198}},
 {  5129,  20, {  9151,   9152,   9164,   9163,   9187,   9188,   9200,   9199}},
 {  5130,  20, {  9152,   9153,   9165,   9164,   9188,   9189,   9201,   9200}},
 {  5131,  20, {  9153,   9154,   9166,   9165,   9189,   9190,   9202,   9201}},
 {  5132,  20, {  9154,   9155,   9167,   9166,   9190,   9191,   9203,   9202}},
 {  5133,  20, {  9155,   9156,   9168,   9167,   9191,   9192,   9204,   9203}},
 {  5134,  20, {  9156,   9157,   9169,   9168,   9192,   9193,   9205,   9204}},
 {  5135,  20, {  9157,   9158,   9170,   9169,   9193,   9194,   9206,   9205}},
 {  5136,  20, {  9158,   9159,   9171,   9170,   9194,   9195,   9207,   9206}},
 {  5137,  20, {  9159,   9160,   9172,   9171,   9195,   9196,   9208,   9207}},
 {  5138,  20, {  9160,   9161,   9173,   9172,   9196,   9197,   9209,   9208}},
 {  5139,  20, {  8861,   9210,   9222,   8873,   8897,   9246,   9258,   8909}},
 {  5140,  20, {  9210,   9211,   9223,   9222,   9246,   9247,   9259,   9258}},
 {  5141,  20, {  9211,   9212,   9224,   9223,   9247,   9248,   9260,   9259}},
 {  5142,  20, {  9212,   9213,   9225,   9224,   9248,   9249,   9261,   9260}},
 {  5143,  20, {  9213,   9214,   9226,   9225,   9249,   9250,   9262,   9261}},
 {  5144,  20, {  9214,   9215,   9227,   9226,   9250,   9251,   9263,   9262}},
 {  5145,  20, {  9215,   9216,   9228,   9227,   9251,   9252,   9264,   9263}},
 {  5146,  20, {  9216,   9217,   9229,   9228,   9252,   9253,   9265,   9264}},
 {  5147,  20, {  9217,   9218,   9230,   9229,   9253,   9254,   9266,   9265}},
 {  5148,  20, {  9218,   9219,   9231,   9230,   9254,   9255,   9267,   9266}},
 {  5149,  20, {  9219,   9220,   9232,   9231,   9255,   9256,   9268,   9267}},
 {  5151,  20, {  8873,   9222,   9234,   8885,   8909,   9258,   9270,   8921}},
 {  5152,  20, {  9222,   9223,   9235,   9234,   9258,   9259,   9271,   9270}},
 {  5153,  20, {  9223,   9224,   9236,   9235,   9259,   9260,   9272,   9271}},
 {  5154,  20, {  9224,   9225,   9237,   9236,   9260,   9261,   9273,   9272}},
 {  5155,  20, {  9225,   9226,   9238,   9237,   9261,   9262,   9274,   9273}},
 {  5156,  20, {  9226,   9227,   9239,   9238,   9262,   9263,   9275,   9274}},
 {  5157,  20, {  9227,   9228,   9240,   9239,   9263,   9264,   9276,   9275}},
 {  5158,  20, {  9228,   9229,   9241,   9240,   9264,   9265,   9277,   9276}},
 {  5159,  20, {  9229,   9230,   9242,   9241,   9265,   9266,   9278,   9277}},
 {  5160,  20, {  9230,   9231,   9243,   9242,   9266,   9267,   9279,   9278}},
 {  5161,  20, {  9231,   9232,   9244,   9243,   9267,   9268,   9280,   9279}},
 {  5163,  20, {  8897,   9246,   9258,   8909,   8933,   9282,   9294,   8945}},
 {  5164,  20, {  9246,   9247,   9259,   9258,   9282,   9283,   9295,   9294}},
 {  5165,  20, {  9247,   9248,   9260,   9259,   9283,   9284,   9296,   9295}},
 {  5166,  20, {  9248,   9249,   9261,   9260,   9284,   9285,   9297,   9296}},
 {  5167,  20, {  9249,   9250,   9262,   9261,   9285,   9286,   9298,   9297}},
 {  5168,  20, {  9250,   9251,   9263,   9262,   9286,   9287,   9299,   9298}},
 {  5169,  20, {  9251,   9252,   9264,   9263,   9287,   9288,   9300,   9299}},
 {  5170,  20, {  9252,   9253,   9265,   9264,   9288,   9289,   9301,   9300}},
 {  5171,  20, {  9253,   9254,   9266,   9265,   9289,   9290,   9302,   9301}},
 {  5172,  20, {  9254,   9255,   9267,   9266,   9290,   9291,   9303,   9302}},
 {  5173,  20, {  9255,   9256,   9268,   9267,   9291,   9292,   9304,   9303}},
 {  5175,  20, {  8909,   9258,   9270,   8921,   8945,   9294,   9306,   8957}},
 {  5176,  20, {  9258,   9259,   9271,   9270,   9294,   9295,   9307,   9306}},
 {  5177,  20, {  9259,   9260,   9272,   9271,   9295,   9296,   9308,   9307}},
 {  5178,  20, {  9260,   9261,   9273,   9272,   9296,   9297,   9309,   9308}},
 {  5179,  20, {  9261,   9262,   9274,   9273,   9297,   9298,   9310,   9309}},
 {  5180,  20, {  9262,   9263,   9275,   9274,   9298,   9299,   9311,   9310}},
 {  5181,  20, {  9263,   9264,   9276,   9275,   9299,   9300,   9312,   9311}},
 {  5182,  20, {  9264,   9265,   9277,   9276,   9300,   9301,   9313,   9312}},
 {  5183,  20, {  9265,   9266,   9278,   9277,   9301,   9302,   9314,   9313}},
 {  5184,  20, {  9266,   9267,   9279,   9278,   9302,   9303,   9315,   9314}},
 {  5185,  20, {  9267,   9268,   9280,   9279,   9303,   9304,   9316,   9315}},
 {  5187,  20, {  8933,   9282,   9294,   8945,   8969,   9318,   9330,   8981}},
 {  5188,  20, {  9282,   9283,   9295,   9294,   9318,   9319,   9331,   9330}},
 {  5189,  20, {  9283,   9284,   9296,   9295,   9319,   9320,   9332,   9331}},
 {  5190,  20, {  9284,   9285,   9297,   9296,   9320,   9321,   9333,   9332}},
 {  5191,  20, {  9285,   9286,   9298,   9297,   9321,   9322,   9334,   9333}},
 {  5192,  20, {  9286,   9287,   9299,   9298,   9322,   9323,   9335,   9334}},
 {  5193,  20, {  9287,   9288,   9300,   9299,   9323,   9324,   9336,   9335}},
 {  5194,  20, {  9288,   9289,   9301,   9300,   9324,   9325,   9337,   9336}},
 {  5195,  20, {  9289,   9290,   9302,   9301,   9325,   9326,   9338,   9337}},
 {  5196,  20, {  9290,   9291,   9303,   9302,   9326,   9327,   9339,   9338}},
 {  5197,  20, {  9291,   9292,   9304,   9303,   9327,   9328,   9340,   9339}},
 {  5199,  20, {  8945,   9294,   9306,   8957,   8981,   9330,   9342,   8993}},
 {  5200,  20, {  9294,   9295,   9307,   9306,   9330,   9331,   9343,   9342}},
 {  5201,  20, {  9295,   9296,   9308,   9307,   9331,   9332,   9344,   9343}},
 {  5202,  20, {  9296,   9297,   9309,   9308,   9332,   9333,   9345,   9344}},
 {  5203,  20, {  9297,   9298,   9310,   9309,   9333,   9334,   9346,   9345}},
 {  5204,  20, {  9298,   9299,   9311,   9310,   9334,   9335,   9347,   9346}},
 {  5205,  20, {  9299,   9300,   9312,   9311,   9335,   9336,   9348,   9347}},
 {  5206,  20, {  9300,   9301,   9313,   9312,   9336,   9337,   9349,   9348}},
 {  5207,  20, {  9301,   9302,   9314,   9313,   9337,   9338,   9350,   9349}},
 {  5208,  20, {  9302,   9303,   9315,   9314,   9338,   9339,   9351,   9350}},
 {  5209,  20, {  9303,   9304,   9316,   9315,   9339,   9340,   9352,   9351}},
 {  5211,  20, {  8969,   9318,   9330,   8981,   9005,   9354,   9366,   9017}},
 {  5212,  20, {  9318,   9319,   9331,   9330,   9354,   9355,   9367,   9366}},
 {  5213,  20, {  9319,   9320,   9332,   9331,   9355,   9356,   9368,   9367}},
 {  5214,  20, {  9320,   9321,   9333,   9332,   9356,   9357,   9369,   9368}},
 {  5215,  20, {  9321,   9322,   9334,   9333,   9357,   9358,   9370,   9369}},
 {  5216,  20, {  9322,   9323,   9335,   9334,   9358,   9359,   9371,   9370}},
 {  5217,  20, {  9323,   9324,   9336,   9335,   9359,   9360,   9372,   9371}},
 {  5218,  20, {  9324,   9325,   9337,   9336,   9360,   9361,   9373,   9372}},
 {  5219,  20, {  9325,   9326,   9338,   9337,   9361,   9362,   9374,   9373}},
 {  5220,  20, {  9326,   9327,   9339,   9338,   9362,   9363,   9375,   9374}},
 {  5221,  20, {  9327,   9328,   9340,   9339,   9363,   9364,   9376,   9375}},
 {  5223,  20, {  8981,   9330,   9342,   8993,   9017,   9366,   9378,   9029}},
 {  5224,  20, {  9330,   9331,   9343,   9342,   9366,   9367,   9379,   9378}},
 {  5225,  20, {  9331,   9332,   9344,   9343,   9367,   9368,   9380,   9379}},
 {  5226,  20, {  9332,   9333,   9345,   9344,   9368,   9369,   9381,   9380}},
 {  5227,  20, {  9333,   9334,   9346,   9345,   9369,   9370,   9382,   9381}},
 {  5228,  20, {  9334,   9335,   9347,   9346,   9370,   9371,   9383,   9382}},
 {  5229,  20, {  9335,   9336,   9348,   9347,   9371,   9372,   9384,   9383}},
 {  5230,  20, {  9336,   9337,   9349,   9348,   9372,   9373,   9385,   9384}},
 {  5231,  20, {  9337,   9338,   9350,   9349,   9373,   9374,   9386,   9385}},
 {  5232,  20, {  9338,   9339,   9351,   9350,   9374,   9375,   9387,   9386}},
 {  5233,  20, {  9339,   9340,   9352,   9351,   9375,   9376,   9388,   9387}},
 {  5235,  20, {  9005,   9354,   9366,   9017,   9041,   9390,   9402,   9053}},
 {  5236,  20, {  9354,   9355,   9367,   9366,   9390,   9391,   9403,   9402}},
 {  5237,  20, {  9355,   9356,   9368,   9367,   9391,   9392,   9404,   9403}},
 {  5238,  20, {  9356,   9357,   9369,   9368,   9392,   9393,   9405,   9404}},
 {  5239,  20, {  9357,   9358,   9370,   9369,   9393,   9394,   9406,   9405}},
 {  5240,  20, {  9358,   9359,   9371,   9370,   9394,   9395,   9407,   9406}},
 {  5241,  20, {  9359,   9360,   9372,   9371,   9395,   9396,   9408,   9407}},
 {  5242,  20, {  9360,   9361,   9373,   9372,   9396,   9397,   9409,   9408}},
 {  5243,  20, {  9361,   9362,   9374,   9373,   9397,   9398,   9410,   9409}},
 {  5244,  20, {  9362,   9363,   9375,   9374,   9398,   9399,   9411,   9410}},
 {  5245,  20, {  9363,   9364,   9376,   9375,   9399,   9400,   9412,   9411}},
 {  5247,  20, {  9017,   9366,   9378,   9029,   9053,   9402,   9414,   9065}},
 {  5248,  20, {  9366,   9367,   9379,   9378,   9402,   9403,   9415,   9414}},
 {  5249,  20, {  9367,   9368,   9380,   9379,   9403,   9404,   9416,   9415}},
 {  5250,  20, {  9368,   9369,   9381,   9380,   9404,   9405,   9417,   9416}},
 {  5251,  20, {  9369,   9370,   9382,   9381,   9405,   9406,   9418,   9417}},
 {  5252,  20, {  9370,   9371,   9383,   9382,   9406,   9407,   9419,   9418}},
 {  5253,  20, {  9371,   9372,   9384,   9383,   9407,   9408,   9420,   9419}},
 {  5254,  20, {  9372,   9373,   9385,   9384,   9408,   9409,   9421,   9420}},
 {  5255,  20, {  9373,   9374,   9386,   9385,   9409,   9410,   9422,   9421}},
 {  5256,  20, {  9374,   9375,   9387,   9386,   9410,   9411,   9423,   9422}},
 {  5257,  20, {  9375,   9376,   9388,   9387,   9411,   9412,   9424,   9423}},
 {  5259,  20, {  9041,   9390,   9402,   9053,   9077,   9426,   9438,   9089}},
 {  5260,  20, {  9390,   9391,   9403,   9402,   9426,   9427,   9439,   9438}},
 {  5261,  20, {  9391,   9392,   9404,   9403,   9427,   9428,   9440,   9439}},
 {  5262,  20, {  9392,   9393,   9405,   9404,   9428,   9429,   9441,   9440}},
 {  5263,  20, {  9393,   9394,   9406,   9405,   9429,   9430,   9442,   9441}},
 {  5264,  20, {  9394,   9395,   9407,   9406,   9430,   9431,   9443,   9442}},
 {  5265,  20, {  9395,   9396,   9408,   9407,   9431,   9432,   9444,   9443}},
 {  5266,  20, {  9396,   9397,   9409,   9408,   9432,   9433,   9445,   9444}},
 {  5267,  20, {  9397,   9398,   9410,   9409,   9433,   9434,   9446,   9445}},
 {  5268,  20, {  9398,   9399,   9411,   9410,   9434,   9435,   9447,   9446}},
 {  5269,  20, {  9399,   9400,   9412,   9411,   9435,   9436,   9448,   9447}},
 {  5271,  20, {  9053,   9402,   9414,   9065,   9089,   9438,   9450,   9101}},
 {  5272,  20, {  9402,   9403,   9415,   9414,   9438,   9439,   9451,   9450}},
 {  5273,  20, {  9403,   9404,   9416,   9415,   9439,   9440,   9452,   9451}},
 {  5274,  20, {  9404,   9405,   9417,   9416,   9440,   9441,   9453,   9452}},
 {  5275,  20, {  9405,   9406,   9418,   9417,   9441,   9442,   9454,   9453}},
 {  5276,  20, {  9406,   9407,   9419,   9418,   9442,   9443,   9455,   9454}},
 {  5277,  20, {  9407,   9408,   9420,   9419,   9443,   9444,   9456,   9455}},
 {  5278,  20, {  9408,   9409,   9421,   9420,   9444,   9445,   9457,   9456}},
 {  5279,  20, {  9409,   9410,   9422,   9421,   9445,   9446,   9458,   9457}},
 {  5280,  20, {  9410,   9411,   9423,   9422,   9446,   9447,   9459,   9458}},
 {  5281,  20, {  9411,   9412,   9424,   9423,   9447,   9448,   9460,   9459}},
 {  5283,  20, {  9077,   9426,   9438,   9089,   9113,   9462,   9474,   9125}},
 {  5284,  20, {  9426,   9427,   9439,   9438,   9462,   9463,   9475,   9474}},
 {  5285,  20, {  9427,   9428,   9440,   9439,   9463,   9464,   9476,   9475}},
 {  5286,  20, {  9428,   9429,   9441,   9440,   9464,   9465,   9477,   9476}},
 {  5287,  20, {  9429,   9430,   9442,   9441,   9465,   9466,   9478,   9477}},
 {  5288,  20, {  9430,   9431,   9443,   9442,   9466,   9467,   9479,   9478}},
 {  5289,  20, {  9431,   9432,   9444,   9443,   9467,   9468,   9480,   9479}},
 {  5290,  20, {  9432,   9433,   9445,   9444,   9468,   9469,   9481,   9480}},
 {  5291,  20, {  9433,   9434,   9446,   9445,   9469,   9470,   9482,   9481}},
 {  5292,  20, {  9434,   9435,   9447,   9446,   9470,   9471,   9483,   9482}},
 {  5293,  20, {  9435,   9436,   9448,   9447,   9471,   9472,   9484,   9483}},
 {  5295,  20, {  9089,   9438,   9450,   9101,   9125,   9474,   9486,   9137}},
 {  5296,  20, {  9438,   9439,   9451,   9450,   9474,   9475,   9487,   9486}},
 {  5297,  20, {  9439,   9440,   9452,   9451,   9475,   9476,   9488,   9487}},
 {  5298,  20, {  9440,   9441,   9453,   9452,   9476,   9477,   9489,   9488}},
 {  5299,  20, {  9441,   9442,   9454,   9453,   9477,   9478,   9490,   9489}},
 {  5300,  20, {  9442,   9443,   9455,   9454,   9478,   9479,   9491,   9490}},
 {  5301,  20, {  9443,   9444,   9456,   9455,   9479,   9480,   9492,   9491}},
 {  5302,  20, {  9444,   9445,   9457,   9456,   9480,   9481,   9493,   9492}},
 {  5303,  20, {  9445,   9446,   9458,   9457,   9481,   9482,   9494,   9493}},
 {  5304,  20, {  9446,   9447,   9459,   9458,   9482,   9483,   9495,   9494}},
 {  5305,  20, {  9447,   9448,   9460,   9459,   9483,   9484,   9496,   9495}},
 {  5307,  20, {  9113,   9462,   9474,   9125,   9149,   9498,   9510,   9161}},
 {  5308,  20, {  9462,   9463,   9475,   9474,   9498,   9499,   9511,   9510}},
 {  5309,  20, {  9463,   9464,   9476,   9475,   9499,   9500,   9512,   9511}},
 {  5310,  20, {  9464,   9465,   9477,   9476,   9500,   9501,   9513,   9512}},
 {  5311,  20, {  9465,   9466,   9478,   9477,   9501,   9502,   9514,   9513}},
 {  5312,  20, {  9466,   9467,   9479,   9478,   9502,   9503,   9515,   9514}},
 {  5313,  20, {  9467,   9468,   9480,   9479,   9503,   9504,   9516,   9515}},
 {  5314,  20, {  9468,   9469,   9481,   9480,   9504,   9505,   9517,   9516}},
 {  5315,  20, {  9469,   9470,   9482,   9481,   9505,   9506,   9518,   9517}},
 {  5316,  20, {  9470,   9471,   9483,   9482,   9506,   9507,   9519,   9518}},
 {  5317,  20, {  9471,   9472,   9484,   9483,   9507,   9508,   9520,   9519}},
 {  5319,  20, {  9125,   9474,   9486,   9137,   9161,   9510,   9522,   9173}},
 {  5320,  20, {  9474,   9475,   9487,   9486,   9510,   9511,   9523,   9522}},
 {  5321,  20, {  9475,   9476,   9488,   9487,   9511,   9512,   9524,   9523}},
 {  5322,  20, {  9476,   9477,   9489,   9488,   9512,   9513,   9525,   9524}},
 {  5323,  20, {  9477,   9478,   9490,   9489,   9513,   9514,   9526,   9525}},
 {  5324,  20, {  9478,   9479,   9491,   9490,   9514,   9515,   9527,   9526}},
 {  5325,  20, {  9479,   9480,   9492,   9491,   9515,   9516,   9528,   9527}},
 {  5326,  20, {  9480,   9481,   9493,   9492,   9516,   9517,   9529,   9528}},
 {  5327,  20, {  9481,   9482,   9494,   9493,   9517,   9518,   9530,   9529}},
 {  5328,  20, {  9482,   9483,   9495,   9494,   9518,   9519,   9531,   9530}},
 {  5329,  20, {  9483,   9484,   9496,   9495,   9519,   9520,   9532,   9531}},
 {  5331,  20, {  9149,   9498,   9510,   9161,   9185,   9534,   9546,   9197}},
 {  5332,  20, {  9498,   9499,   9511,   9510,   9534,   9535,   9547,   9546}},
 {  5333,  20, {  9499,   9500,   9512,   9511,   9535,   9536,   9548,   9547}},
 {  5334,  20, {  9500,   9501,   9513,   9512,   9536,   9537,   9549,   9548}},
 {  5335,  20, {  9501,   9502,   9514,   9513,   9537,   9538,   9550,   9549}},
 {  5336,  20, {  9502,   9503,   9515,   9514,   9538,   9539,   9551,   9550}},
 {  5337,  20, {  9503,   9504,   9516,   9515,   9539,   9540,   9552,   9551}},
 {  5338,  20, {  9504,   9505,   9517,   9516,   9540,   9541,   9553,   9552}},
 {  5339,  20, {  9505,   9506,   9518,   9517,   9541,   9542,   9554,   9553}},
 {  5340,  20, {  9506,   9507,   9519,   9518,   9542,   9543,   9555,   9554}},
 {  5341,  20, {  9507,   9508,   9520,   9519,   9543,   9544,   9556,   9555}},
 {  5343,  20, {  9161,   9510,   9522,   9173,   9197,   9546,   9558,   9209}},
 {  5344,  20, {  9510,   9511,   9523,   9522,   9546,   9547,   9559,   9558}},
 {  5345,  20, {  9511,   9512,   9524,   9523,   9547,   9548,   9560,   9559}},
 {  5346,  20, {  9512,   9513,   9525,   9524,   9548,   9549,   9561,   9560}},
 {  5347,  20, {  9513,   9514,   9526,   9525,   9549,   9550,   9562,   9561}},
 {  5348,  20, {  9514,   9515,   9527,   9526,   9550,   9551,   9563,   9562}},
 {  5349,  20, {  9515,   9516,   9528,   9527,   9551,   9552,   9564,   9563}},
 {  5350,  20, {  9516,   9517,   9529,   9528,   9552,   9553,   9565,   9564}},
 {  5351,  20, {  9517,   9518,   9530,   9529,   9553,   9554,   9566,   9565}},
 {  5352,  20, {  9518,   9519,   9531,   9530,   9554,   9555,   9567,   9566}},
 {  5353,  20, {  9519,   9520,   9532,   9531,   9555,   9556,   9568,   9567}},
 {  5355,  21, {  9570,   3210,   9573,   9572,   9574,   3218,   9577,   9576}},
 {  5356,  21, {  9574,   3218,   9577,   9576,   9578,   3226,   9581,   9580}},
 {  5357,  21, {  9572,   9573,   9583,   9582,   9576,   9577,   9585,   9584}},
 {  5358,  21, {  9576,   9577,   9585,   9584,   9580,   9581,   9587,   9586}},
 {  5359,  21, {  3210,   3211,   9594,   9573,   3218,   3219,   9606,   9577}},
 {  5360,  21, {  3211,   3212,   9595,   9594,   3219,   3220,   9607,   9606}},
 {  5361,  21, {  3212,   3213,   9596,   9595,   3220,   3221,   9608,   9607}},
 {  5362,  21, {  3213,   9591,   9597,   9596,   3221,   9603,   9609,   9608}},
 {  5363,  21, {  9591,   2908,   9598,   9597,   9603,   2912,   9610,   9609}},
 {  5364,  21, {  2908,   2909,   9599,   9598,   2912,   2913,   9611,   9610}},
 {  5365,  21, {  3218,   3219,   9606,   9577,   3226,   3227,   9618,   9581}},
 {  5366,  21, {  3219,   3220,   9607,   9606,   3227,   3228,   9619,   9618}},
 {  5367,  21, {  3220,   3221,   9608,   9607,   3228,   3229,   9620,   9619}},
 {  5368,  21, {  3221,   9603,   9609,   9608,   3229,   9615,   9621,   9620}},
 {  5369,  21, {  9603,   2912,   9610,   9609,   9615,   2916,   9622,   9621}},
 {  5370,  21, {  2912,   2913,   9611,   9610,   2916,   2917,   9623,   9622}},
 {  5371,  21, {  9573,   9594,   9624,   9583,   9577,   9606,   9630,   9585}},
 {  5372,  21, {  9594,   9595,   9625,   9624,   9606,   9607,   9631,   9630}},
 {  5373,  21, {  9595,   9596,   9626,   9625,   9607,   9608,   9632,   9631}},
 {  5374,  21, {  9596,   9597,   9627,   9626,   9608,   9609,   9633,   9632}},
 {  5375,  21, {  9597,   9598,   9628,   9627,   9609,   9610,   9634,   9633}},
 {  5376,  21, {  9598,   9599,   9629,   9628,   9610,   9611,   9635,   9634}},
 {  5377,  21, {  9577,   9606,   9630,   9585,   9581,   9618,   9636,   9587}},
 {  5378,  21, {  9606,   9607,   9631,   9630,   9618,   9619,   9637,   9636}},
 {  5379,  21, {  9607,   9608,   9632,   9631,   9619,   9620,   9638,   9637}},
 {  5380,  21, {  9608,   9609,   9633,   9632,   9620,   9621,   9639,   9638}},
 {  5381,  21, {  9609,   9610,   9634,   9633,   9621,   9622,   9640,   9639}},
 {  5382,  21, {  9610,   9611,   9635,   9634,   9622,   9623,   9641,   9640}},
 {  5383,  21, {  2909,   2968,   9644,   9599,   2913,   2972,   9648,   9611}},
 {  5384,  21, {  2968,   2969,   9645,   9644,   2972,   2973,   9649,   9648}},
 {  5385,  21, {  2913,   2972,   9648,   9611,   2917,   2976,   9652,   9623}},
 {  5386,  21, {  2972,   2973,   9649,   9648,   2976,   2977,   9653,   9652}},
 {  5387,  21, {  9599,   9644,   9654,   9629,   9611,   9648,   9656,   9635}},
 {  5388,  21, {  9644,   9645,   9655,   9654,   9648,   9649,   9657,   9656}},
 {  5389,  21, {  9611,   9648,   9656,   9635,   9623,   9652,   9658,   9641}},
 {  5390,  21, {  9648,   9649,   9657,   9656,   9652,   9653,   9659,   9658}},
 {  5391,  21, {  2969,   3027,   9666,   9645,   2973,   3029,   9678,   9649}},
 {  5393,  21, {  3570,   3571,   9668,   9667,   3578,   3579,   9680,   9679}},
 {  5394,  21, {  3571,   3572,   9669,   9668,   3579,   3580,   9681,   9680}},
 {  5395,  21, {  3572,   3573,   9670,   9669,   3580,   3581,   9682,   9681}},
 {  5396,  21, {  3573,   9665,   9671,   9670,   3581,   9677,   9683,   9682}},
 {  5397,  21, {  2973,   3029,   9678,   9649,   2977,   3031,   9690,   9653}},
 {  5399,  21, {  3578,   3579,   9680,   9679,   3586,   3587,   9692,   9691}},
 {  5400,  21, {  3579,   3580,   9681,   9680,   3587,   3588,   9693,   9692}},
 {  5401,  21, {  3580,   3581,   9682,   9681,   3588,   3589,   9694,   9693}},
 {  5402,  21, {  3581,   9677,   9683,   9682,   3589,   9689,   9695,   9694}},
 {  5403,  21, {  9645,   9666,   9696,   9655,   9649,   9678,   9702,   9657}},
 {  5405,  21, {  9667,   9668,   9698,   9697,   9679,   9680,   9704,   9703}},
 {  5406,  21, {  9668,   9669,   9699,   9698,   9680,   9681,   9705,   9704}},
 {  5407,  21, {  9669,   9670,   9700,   9699,   9681,   9682,   9706,   9705}},
 {  5408,  21, {  9670,   9671,   9701,   9700,   9682,   9683,   9707,   9706}},
 {  5409,  21, {  9649,   9678,   9702,   9657,   9653,   9690,   9708,   9659}},
 {  5411,  21, {  9679,   9680,   9704,   9703,   9691,   9692,   9710,   9709}},
 {  5412,  21, {  9680,   9681,   9705,   9704,   9692,   9693,   9711,   9710}},
 {  5413,  21, {  9681,   9682,   9706,   9705,   9693,   9694,   9712,   9711}},
 {  5414,  21, {  9682,   9683,   9707,   9706,   9694,   9695,   9713,   9712}},
 {  5415,  21, {  9665,   9714,   9715,   9671,   9677,   9716,   9717,   9683}},
 {  5416,  21, {  9677,   9716,   9717,   9683,   9689,   9718,   9719,   9695}},
 {  5417,  21, {  9671,   9715,   9720,   9701,   9683,   9717,   9721,   9707}},
 {  5418,  21, {  9683,   9717,   9721,   9707,   9695,   9719,   9722,   9713}},
 {  5419,  21, {  9723,   3234,   9726,   9725,   9727,   3242,   9730,   9729}},
 {  5420,  21, {  9727,   3242,   9730,   9729,   9731,   3250,   9734,   9733}},
 {  5421,  21, {  9725,   9726,   9736,   9735,   9729,   9730,   9738,   9737}},
 {  5422,  21, {  9729,   9730,   9738,   9737,   9733,   9734,   9740,   9739}},
 {  5423,  21, {  3234,   3235,   9747,   9726,   3242,   3243,   9759,   9730}},
 {  5424,  21, {  3235,   3236,   9748,   9747,   3243,   3244,   9760,   9759}},
 {  5425,  21, {  3236,   3237,   9749,   9748,   3244,   3245,   9761,   9760}},
 {  5426,  21, {  3237,   9744,   9750,   9749,   3245,   9756,   9762,   9761}},
 {  5427,  21, {  9744,   2920,   9751,   9750,   9756,   2924,   9763,   9762}},
 {  5428,  21, {  2920,   2921,   9752,   9751,   2924,   2925,   9764,   9763}},
 {  5429,  21, {  3242,   3243,   9759,   9730,   3250,   3251,   9771,   9734}},
 {  5430,  21, {  3243,   3244,   9760,   9759,   3251,   3252,   9772,   9771}},
 {  5431,  21, {  3244,   3245,   9761,   9760,   3252,   3253,   9773,   9772}},
 {  5432,  21, {  3245,   9756,   9762,   9761,   3253,   9768,   9774,   9773}},
 {  5433,  21, {  9756,   2924,   9763,   9762,   9768,   2928,   9775,   9774}},
 {  5434,  21, {  2924,   2925,   9764,   9763,   2928,   2929,   9776,   9775}},
 {  5435,  21, {  9726,   9747,   9777,   9736,   9730,   9759,   9783,   9738}},
 {  5436,  21, {  9747,   9748,   9778,   9777,   9759,   9760,   9784,   9783}},
 {  5437,  21, {  9748,   9749,   9779,   9778,   9760,   9761,   9785,   9784}},
 {  5438,  21, {  9749,   9750,   9780,   9779,   9761,   9762,   9786,   9785}},
 {  5439,  21, {  9750,   9751,   9781,   9780,   9762,   9763,   9787,   9786}},
 {  5440,  21, {  9751,   9752,   9782,   9781,   9763,   9764,   9788,   9787}},
 {  5441,  21, {  9730,   9759,   9783,   9738,   9734,   9771,   9789,   9740}},
 {  5442,  21, {  9759,   9760,   9784,   9783,   9771,   9772,   9790,   9789}},
 {  5443,  21, {  9760,   9761,   9785,   9784,   9772,   9773,   9791,   9790}},
 {  5444,  21, {  9761,   9762,   9786,   9785,   9773,   9774,   9792,   9791}},
 {  5445,  21, {  9762,   9763,   9787,   9786,   9774,   9775,   9793,   9792}},
 {  5446,  21, {  9763,   9764,   9788,   9787,   9775,   9776,   9794,   9793}},
 {  5447,  21, {  2921,   2980,   9797,   9752,   2925,   2984,   9801,   9764}},
 {  5448,  21, {  2980,   2981,   9798,   9797,   2984,   2985,   9802,   9801}},
 {  5449,  21, {  2925,   2984,   9801,   9764,   2929,   2988,   9805,   9776}},
 {  5450,  21, {  2984,   2985,   9802,   9801,   2988,   2989,   9806,   9805}},
 {  5451,  21, {  9752,   9797,   9807,   9782,   9764,   9801,   9809,   9788}},
 {  5452,  21, {  9797,   9798,   9808,   9807,   9801,   9802,   9810,   9809}},
 {  5453,  21, {  9764,   9801,   9809,   9788,   9776,   9805,   9811,   9794}},
 {  5454,  21, {  9801,   9802,   9810,   9809,   9805,   9806,   9812,   9811}},
 {  5455,  21, {  2981,   3033,   9819,   9798,   2985,   3035,   9831,   9802}},
 {  5457,  21, {  3594,   3595,   9821,   9820,   3602,   3603,   9833,   9832}},
 {  5458,  21, {  3595,   3596,   9822,   9821,   3603,   3604,   9834,   9833}},
 {  5459,  21, {  3596,   3597,   9823,   9822,   3604,   3605,   9835,   9834}},
 {  5460,  21, {  3597,   9818,   9824,   9823,   3605,   9830,   9836,   9835}},
 {  5461,  21, {  2985,   3035,   9831,   9802,   2989,   3037,   9843,   9806}},
 {  5463,  21, {  3602,   3603,   9833,   9832,   3610,   3611,   9845,   9844}},
 {  5464,  21, {  3603,   3604,   9834,   9833,   3611,   3612,   9846,   9845}},
 {  5465,  21, {  3604,   3605,   9835,   9834,   3612,   3613,   9847,   9846}},
 {  5466,  21, {  3605,   9830,   9836,   9835,   3613,   9842,   9848,   9847}},
 {  5467,  21, {  9798,   9819,   9849,   9808,   9802,   9831,   9855,   9810}},
 {  5469,  21, {  9820,   9821,   9851,   9850,   9832,   9833,   9857,   9856}},
 {  5470,  21, {  9821,   9822,   9852,   9851,   9833,   9834,   9858,   9857}},
 {  5471,  21, {  9822,   9823,   9853,   9852,   9834,   9835,   9859,   9858}},
 {  5472,  21, {  9823,   9824,   9854,   9853,   9835,   9836,   9860,   9859}},
 {  5473,  21, {  9802,   9831,   9855,   9810,   9806,   9843,   9861,   9812}},
 {  5475,  21, {  9832,   9833,   9857,   9856,   9844,   9845,   9863,   9862}},
 {  5476,  21, {  9833,   9834,   9858,   9857,   9845,   9846,   9864,   9863}},
 {  5477,  21, {  9834,   9835,   9859,   9858,   9846,   9847,   9865,   9864}},
 {  5478,  21, {  9835,   9836,   9860,   9859,   9847,   9848,   9866,   9865}},
 {  5479,  21, {  9818,   9867,   9868,   9824,   9830,   9869,   9870,   9836}},
 {  5480,  21, {  9830,   9869,   9870,   9836,   9842,   9871,   9872,   9848}},
 {  5481,  21, {  9824,   9868,   9873,   9854,   9836,   9870,   9874,   9860}},
 {  5482,  21, {  9836,   9870,   9874,   9860,   9848,   9872,   9875,   9866}},
 {  5483,  21, {  9876,   3258,   9879,   9878,   9880,   3266,   9883,   9882}},
 {  5484,  21, {  9880,   3266,   9883,   9882,   9884,   3274,   9887,   9886}},
 {  5485,  21, {  9878,   9879,   9889,   9888,   9882,   9883,   9891,   9890}},
 {  5486,  21, {  9882,   9883,   9891,   9890,   9886,   9887,   9893,   9892}},
 {  5487,  21, {  3258,   3259,   9900,   9879,   3266,   3267,   9912,   9883}},
 {  5488,  21, {  3259,   3260,   9901,   9900,   3267,   3268,   9913,   9912}},
 {  5489,  21, {  3260,   3261,   9902,   9901,   3268,   3269,   9914,   9913}},
 {  5490,  21, {  3261,   9897,   9903,   9902,   3269,   9909,   9915,   9914}},
 {  5491,  21, {  9897,   2932,   9904,   9903,   9909,   2936,   9916,   9915}},
 {  5492,  21, {  2932,   2933,   9905,   9904,   2936,   2937,   9917,   9916}},
 {  5493,  21, {  3266,   3267,   9912,   9883,   3274,   3275,   9924,   9887}},
 {  5494,  21, {  3267,   3268,   9913,   9912,   3275,   3276,   9925,   9924}},
 {  5495,  21, {  3268,   3269,   9914,   9913,   3276,   3277,   9926,   9925}},
 {  5496,  21, {  3269,   9909,   9915,   9914,   3277,   9921,   9927,   9926}},
 {  5497,  21, {  9909,   2936,   9916,   9915,   9921,   2940,   9928,   9927}},
 {  5498,  21, {  2936,   2937,   9917,   9916,   2940,   2941,   9929,   9928}},
 {  5499,  21, {  9879,   9900,   9930,   9889,   9883,   9912,   9936,   9891}},
 {  5500,  21, {  9900,   9901,   9931,   9930,   9912,   9913,   9937,   9936}},
 {  5501,  21, {  9901,   9902,   9932,   9931,   9913,   9914,   9938,   9937}},
 {  5502,  21, {  9902,   9903,   9933,   9932,   9914,   9915,   9939,   9938}},
 {  5503,  21, {  9903,   9904,   9934,   9933,   9915,   9916,   9940,   9939}},
 {  5504,  21, {  9904,   9905,   9935,   9934,   9916,   9917,   9941,   9940}},
 {  5505,  21, {  9883,   9912,   9936,   9891,   9887,   9924,   9942,   9893}},
 {  5506,  21, {  9912,   9913,   9937,   9936,   9924,   9925,   9943,   9942}},
 {  5507,  21, {  9913,   9914,   9938,   9937,   9925,   9926,   9944,   9943}},
 {  5508,  21, {  9914,   9915,   9939,   9938,   9926,   9927,   9945,   9944}},
 {  5509,  21, {  9915,   9916,   9940,   9939,   9927,   9928,   9946,   9945}},
 {  5510,  21, {  9916,   9917,   9941,   9940,   9928,   9929,   9947,   9946}},
 {  5511,  21, {  2933,   2992,   9950,   9905,   2937,   2996,   9954,   9917}},
 {  5512,  21, {  2992,   2993,   9951,   9950,   2996,   2997,   9955,   9954}},
 {  5513,  21, {  2937,   2996,   9954,   9917,   2941,   3000,   9958,   9929}},
 {  5514,  21, {  2996,   2997,   9955,   9954,   3000,   3001,   9959,   9958}},
 {  5515,  21, {  9905,   9950,   9960,   9935,   9917,   9954,   9962,   9941}},
 {  5516,  21, {  9950,   9951,   9961,   9960,   9954,   9955,   9963,   9962}},
 {  5517,  21, {  9917,   9954,   9962,   9941,   9929,   9958,   9964,   9947}},
 {  5518,  21, {  9954,   9955,   9963,   9962,   9958,   9959,   9965,   9964}},
 {  5519,  21, {  2993,   3039,   9972,   9951,   2997,   3041,   9984,   9955}},
 {  5521,  21, {  3618,   3619,   9974,   9973,   3626,   3627,   9986,   9985}},
 {  5522,  21, {  3619,   3620,   9975,   9974,   3627,   3628,   9987,   9986}},
 {  5523,  21, {  3620,   3621,   9976,   9975,   3628,   3629,   9988,   9987}},
 {  5524,  21, {  3621,   9971,   9977,   9976,   3629,   9983,   9989,   9988}},
 {  5525,  21, {  2997,   3041,   9984,   9955,   3001,   3043,   9996,   9959}},
 {  5527,  21, {  3626,   3627,   9986,   9985,   3634,   3635,   9998,   9997}},
 {  5528,  21, {  3627,   3628,   9987,   9986,   3635,   3636,   9999,   9998}},
 {  5529,  21, {  3628,   3629,   9988,   9987,   3636,   3637,  10000,   9999}},
 {  5530,  21, {  3629,   9983,   9989,   9988,   3637,   9995,  10001,  10000}},
 {  5531,  21, {  9951,   9972,  10002,   9961,   9955,   9984,  10008,   9963}},
 {  5533,  21, {  9973,   9974,  10004,  10003,   9985,   9986,  10010,  10009}},
 {  5534,  21, {  9974,   9975,  10005,  10004,   9986,   9987,  10011,  10010}},
 {  5535,  21, {  9975,   9976,  10006,  10005,   9987,   9988,  10012,  10011}},
 {  5536,  21, {  9976,   9977,  10007,  10006,   9988,   9989,  10013,  10012}},
 {  5537,  21, {  9955,   9984,  10008,   9963,   9959,   9996,  10014,   9965}},
 {  5539,  21, {  9985,   9986,  10010,  10009,   9997,   9998,  10016,  10015}},
 {  5540,  21, {  9986,   9987,  10011,  10010,   9998,   9999,  10017,  10016}},
 {  5541,  21, {  9987,   9988,  10012,  10011,   9999,  10000,  10018,  10017}},
 {  5542,  21, {  9988,   9989,  10013,  10012,  10000,  10001,  10019,  10018}},
 {  5543,  21, {  9971,  10020,  10021,   9977,   9983,  10022,  10023,   9989}},
 {  5544,  21, {  9983,  10022,  10023,   9989,   9995,  10024,  10025,  10001}},
 {  5545,  21, {  9977,  10021,  10026,  10007,   9989,  10023,  10027,  10013}},
 {  5546,  21, {  9989,  10023,  10027,  10013,  10001,  10025,  10028,  10019}},
 {  5547,  21, { 10029,   3282,  10032,  10031,  10033,   3290,  10036,  10035}},
 {  5548,  21, { 10033,   3290,  10036,  10035,  10037,   3298,  10040,  10039}},
 {  5549,  21, { 10031,  10032,  10042,  10041,  10035,  10036,  10044,  10043}},
 {  5550,  21, { 10035,  10036,  10044,  10043,  10039,  10040,  10046,  10045}},
 {  5551,  21, {  3282,   3283,  10053,  10032,   3290,   3291,  10065,  10036}},
 {  5552,  21, {  3283,   3284,  10054,  10053,   3291,   3292,  10066,  10065}},
 {  5553,  21, {  3284,   3285,  10055,  10054,   3292,   3293,  10067,  10066}},
 {  5554,  21, {  3285,  10050,  10056,  10055,   3293,  10062,  10068,  10067}},
 {  5555,  21, { 10050,   2944,  10057,  10056,  10062,   2948,  10069,  10068}},
 {  5556,  21, {  2944,   2945,  10058,  10057,   2948,   2949,  10070,  10069}},
 {  5557,  21, {  3290,   3291,  10065,  10036,   3298,   3299,  10077,  10040}},
 {  5558,  21, {  3291,   3292,  10066,  10065,   3299,   3300,  10078,  10077}},
 {  5559,  21, {  3292,   3293,  10067,  10066,   3300,   3301,  10079,  10078}},
 {  5560,  21, {  3293,  10062,  10068,  10067,   3301,  10074,  10080,  10079}},
 {  5561,  21, { 10062,   2948,  10069,  10068,  10074,   2952,  10081,  10080}},
 {  5562,  21, {  2948,   2949,  10070,  10069,   2952,   2953,  10082,  10081}},
 {  5563,  21, { 10032,  10053,  10083,  10042,  10036,  10065,  10089,  10044}},
 {  5564,  21, { 10053,  10054,  10084,  10083,  10065,  10066,  10090,  10089}},
 {  5565,  21, { 10054,  10055,  10085,  10084,  10066,  10067,  10091,  10090}},
 {  5566,  21, { 10055,  10056,  10086,  10085,  10067,  10068,  10092,  10091}},
 {  5567,  21, { 10056,  10057,  10087,  10086,  10068,  10069,  10093,  10092}},
 {  5568,  21, { 10057,  10058,  10088,  10087,  10069,  10070,  10094,  10093}},
 {  5569,  21, { 10036,  10065,  10089,  10044,  10040,  10077,  10095,  10046}},
 {  5570,  21, { 10065,  10066,  10090,  10089,  10077,  10078,  10096,  10095}},
 {  5571,  21, { 10066,  10067,  10091,  10090,  10078,  10079,  10097,  10096}},
 {  5572,  21, { 10067,  10068,  10092,  10091,  10079,  10080,  10098,  10097}},
 {  5573,  21, { 10068,  10069,  10093,  10092,  10080,  10081,  10099,  10098}},
 {  5574,  21, { 10069,  10070,  10094,  10093,  10081,  10082,  10100,  10099}},
 {  5575,  21, {  2945,   3004,  10103,  10058,   2949,   3008,  10107,  10070}},
 {  5576,  21, {  3004,   3005,  10104,  10103,   3008,   3009,  10108,  10107}},
 {  5577,  21, {  2949,   3008,  10107,  10070,   2953,   3012,  10111,  10082}},
 {  5578,  21, {  3008,   3009,  10108,  10107,   3012,   3013,  10112,  10111}},
 {  5579,  21, { 10058,  10103,  10113,  10088,  10070,  10107,  10115,  10094}},
 {  5580,  21, { 10103,  10104,  10114,  10113,  10107,  10108,  10116,  10115}},
 {  5581,  21, { 10070,  10107,  10115,  10094,  10082,  10111,  10117,  10100}},
 {  5582,  21, { 10107,  10108,  10116,  10115,  10111,  10112,  10118,  10117}},
 {  5583,  21, {  3005,   3045,  10125,  10104,   3009,   3047,  10137,  10108}},
 {  5585,  21, {  3642,   3643,  10127,  10126,   3650,   3651,  10139,  10138}},
 {  5586,  21, {  3643,   3644,  10128,  10127,   3651,   3652,  10140,  10139}},
 {  5587,  21, {  3644,   3645,  10129,  10128,   3652,   3653,  10141,  10140}},
 {  5588,  21, {  3645,  10124,  10130,  10129,   3653,  10136,  10142,  10141}},
 {  5589,  21, {  3009,   3047,  10137,  10108,   3013,   3049,  10149,  10112}},
 {  5591,  21, {  3650,   3651,  10139,  10138,   3658,   3659,  10151,  10150}},
 {  5592,  21, {  3651,   3652,  10140,  10139,   3659,   3660,  10152,  10151}},
 {  5593,  21, {  3652,   3653,  10141,  10140,   3660,   3661,  10153,  10152}},
 {  5594,  21, {  3653,  10136,  10142,  10141,   3661,  10148,  10154,  10153}},
 {  5595,  21, { 10104,  10125,  10155,  10114,  10108,  10137,  10161,  10116}},
 {  5597,  21, { 10126,  10127,  10157,  10156,  10138,  10139,  10163,  10162}},
 {  5598,  21, { 10127,  10128,  10158,  10157,  10139,  10140,  10164,  10163}},
 {  5599,  21, { 10128,  10129,  10159,  10158,  10140,  10141,  10165,  10164}},
 {  5600,  21, { 10129,  10130,  10160,  10159,  10141,  10142,  10166,  10165}},
 {  5601,  21, { 10108,  10137,  10161,  10116,  10112,  10149,  10167,  10118}},
 {  5603,  21, { 10138,  10139,  10163,  10162,  10150,  10151,  10169,  10168}},
 {  5604,  21, { 10139,  10140,  10164,  10163,  10151,  10152,  10170,  10169}},
 {  5605,  21, { 10140,  10141,  10165,  10164,  10152,  10153,  10171,  10170}},
 {  5606,  21, { 10141,  10142,  10166,  10165,  10153,  10154,  10172,  10171}},
 {  5607,  21, { 10124,  10173,  10174,  10130,  10136,  10175,  10176,  10142}},
 {  5608,  21, { 10136,  10175,  10176,  10142,  10148,  10177,  10178,  10154}},
 {  5609,  21, { 10130,  10174,  10179,  10160,  10142,  10176,  10180,  10166}},
 {  5610,  21, { 10142,  10176,  10180,  10166,  10154,  10178,  10181,  10172}},
 {  5611,  21, { 10182,   3306,  10185,  10184,  10186,   3314,  10189,  10188}},
 {  5612,  21, { 10186,   3314,  10189,  10188,  10190,   3322,  10193,  10192}},
 {  5613,  21, { 10184,  10185,  10195,  10194,  10188,  10189,  10197,  10196}},
 {  5614,  21, { 10188,  10189,  10197,  10196,  10192,  10193,  10199,  10198}},
 {  5615,  21, {  3306,   3307,  10206,  10185,   3314,   3315,  10218,  10189}},
 {  5616,  21, {  3307,   3308,  10207,  10206,   3315,   3316,  10219,  10218}},
 {  5617,  21, {  3308,   3309,  10208,  10207,   3316,   3317,  10220,  10219}},
 {  5618,  21, {  3309,  10203,  10209,  10208,   3317,  10215,  10221,  10220}},
 {  5619,  21, { 10203,   2956,  10210,  10209,  10215,   2960,  10222,  10221}},
 {  5620,  21, {  2956,   2957,  10211,  10210,   2960,   2961,  10223,  10222}},
 {  5621,  21, {  3314,   3315,  10218,  10189,   3322,   3323,  10230,  10193}},
 {  5622,  21, {  3315,   3316,  10219,  10218,   3323,   3324,  10231,  10230}},
 {  5623,  21, {  3316,   3317,  10220,  10219,   3324,   3325,  10232,  10231}},
 {  5624,  21, {  3317,  10215,  10221,  10220,   3325,  10227,  10233,  10232}},
 {  5625,  21, { 10215,   2960,  10222,  10221,  10227,   2964,  10234,  10233}},
 {  5626,  21, {  2960,   2961,  10223,  10222,   2964,   2965,  10235,  10234}},
 {  5627,  21, { 10185,  10206,  10236,  10195,  10189,  10218,  10242,  10197}},
 {  5628,  21, { 10206,  10207,  10237,  10236,  10218,  10219,  10243,  10242}},
 {  5629,  21, { 10207,  10208,  10238,  10237,  10219,  10220,  10244,  10243}},
 {  5630,  21, { 10208,  10209,  10239,  10238,  10220,  10221,  10245,  10244}},
 {  5631,  21, { 10209,  10210,  10240,  10239,  10221,  10222,  10246,  10245}},
 {  5632,  21, { 10210,  10211,  10241,  10240,  10222,  10223,  10247,  10246}},
 {  5633,  21, { 10189,  10218,  10242,  10197,  10193,  10230,  10248,  10199}},
 {  5634,  21, { 10218,  10219,  10243,  10242,  10230,  10231,  10249,  10248}},
 {  5635,  21, { 10219,  10220,  10244,  10243,  10231,  10232,  10250,  10249}},
 {  5636,  21, { 10220,  10221,  10245,  10244,  10232,  10233,  10251,  10250}},
 {  5637,  21, { 10221,  10222,  10246,  10245,  10233,  10234,  10252,  10251}},
 {  5638,  21, { 10222,  10223,  10247,  10246,  10234,  10235,  10253,  10252}},
 {  5639,  21, {  2957,   3016,  10256,  10211,   2961,   3020,  10260,  10223}},
 {  5640,  21, {  3016,   3017,  10257,  10256,   3020,   3021,  10261,  10260}},
 {  5641,  21, {  2961,   3020,  10260,  10223,   2965,   3024,  10264,  10235}},
 {  5642,  21, {  3020,   3021,  10261,  10260,   3024,   3025,  10265,  10264}},
 {  5643,  21, { 10211,  10256,  10266,  10241,  10223,  10260,  10268,  10247}},
 {  5644,  21, { 10256,  10257,  10267,  10266,  10260,  10261,  10269,  10268}},
 {  5645,  21, { 10223,  10260,  10268,  10247,  10235,  10264,  10270,  10253}},
 {  5646,  21, { 10260,  10261,  10269,  10268,  10264,  10265,  10271,  10270}},
 {  5647,  21, {  3017,   3051,  10278,  10257,   3021,   3053,  10290,  10261}},
 {  5649,  21, {  3666,   3667,  10280,  10279,   3674,   3675,  10292,  10291}},
 {  5650,  21, {  3667,   3668,  10281,  10280,   3675,   3676,  10293,  10292}},
 {  5651,  21, {  3668,   3669,  10282,  10281,   3676,   3677,  10294,  10293}},
 {  5652,  21, {  3669,  10277,  10283,  10282,   3677,  10289,  10295,  10294}},
 {  5653,  21, {  3021,   3053,  10290,  10261,   3025,   3055,  10302,  10265}},
 {  5655,  21, {  3674,   3675,  10292,  10291,   3682,   3683,  10304,  10303}},
 {  5656,  21, {  3675,   3676,  10293,  10292,   3683,   3684,  10305,  10304}},
 {  5657,  21, {  3676,   3677,  10294,  10293,   3684,   3685,  10306,  10305}},
 {  5658,  21, {  3677,  10289,  10295,  10294,   3685,  10301,  10307,  10306}},
 {  5659,  21, { 10257,  10278,  10308,  10267,  10261,  10290,  10314,  10269}},
 {  5661,  21, { 10279,  10280,  10310,  10309,  10291,  10292,  10316,  10315}},
 {  5662,  21, { 10280,  10281,  10311,  10310,  10292,  10293,  10317,  10316}},
 {  5663,  21, { 10281,  10282,  10312,  10311,  10293,  10294,  10318,  10317}},
 {  5664,  21, { 10282,  10283,  10313,  10312,  10294,  10295,  10319,  10318}},
 {  5665,  21, { 10261,  10290,  10314,  10269,  10265,  10302,  10320,  10271}},
 {  5667,  21, { 10291,  10292,  10316,  10315,  10303,  10304,  10322,  10321}},
 {  5668,  21, { 10292,  10293,  10317,  10316,  10304,  10305,  10323,  10322}},
 {  5669,  21, { 10293,  10294,  10318,  10317,  10305,  10306,  10324,  10323}},
 {  5670,  21, { 10294,  10295,  10319,  10318,  10306,  10307,  10325,  10324}},
 {  5671,  21, { 10277,  10326,  10327,  10283,  10289,  10328,  10329,  10295}},
 {  5672,  21, { 10289,  10328,  10329,  10295,  10301,  10330,  10331,  10307}},
 {  5673,  21, { 10283,  10327,  10332,  10313,  10295,  10329,  10333,  10319}},
 {  5674,  21, { 10295,  10329,  10333,  10319,  10307,  10331,  10334,  10325}},
 {  5675,  21, { 10335,   3349,  10338,  10337,  10339,   3341,  10342,  10341}},
 {  5676,  21, { 10339,   3341,  10342,  10341,  10343,   3333,  10346,  10345}},
 {  5677,  21, { 10337,  10338,  10348,  10347,  10341,  10342,  10350,  10349}},
 {  5678,  21, { 10341,  10342,  10350,  10349,  10345,  10346,  10352,  10351}},
 {  5679,  21, {  3349,   3348,  10359,  10338,   3341,   3340,  10371,  10342}},
 {  5680,  21, {  3348,   3347,  10360,  10359,   3340,   3339,  10372,  10371}},
 {  5681,  21, {  3347,   3346,  10361,  10360,   3339,   3338,  10373,  10372}},
 {  5682,  21, {  3346,  10356,  10362,  10361,   3338,  10368,  10374,  10373}},
 {  5683,  21, { 10356,   3181,  10363,  10362,  10368,   3179,  10375,  10374}},
 {  5684,  21, {  3181,   3127,  10364,  10363,   3179,   3123,  10376,  10375}},
 {  5685,  21, {  3341,   3340,  10371,  10342,   3333,   3332,  10383,  10346}},
 {  5686,  21, {  3340,   3339,  10372,  10371,   3332,   3331,  10384,  10383}},
 {  5687,  21, {  3339,   3338,  10373,  10372,   3331,   3330,  10385,  10384}},
 {  5688,  21, {  3338,  10368,  10374,  10373,   3330,  10380,  10386,  10385}},
 {  5689,  21, { 10368,   3179,  10375,  10374,  10380,   3177,  10387,  10386}},
 {  5690,  21, {  3179,   3123,  10376,  10375,   3177,   3119,  10388,  10387}},
 {  5691,  21, { 10338,  10359,  10389,  10348,  10342,  10371,  10395,  10350}},
 {  5692,  21, { 10359,  10360,  10390,  10389,  10371,  10372,  10396,  10395}},
 {  5693,  21, { 10360,  10361,  10391,  10390,  10372,  10373,  10397,  10396}},
 {  5694,  21, { 10361,  10362,  10392,  10391,  10373,  10374,  10398,  10397}},
 {  5695,  21, { 10362,  10363,  10393,  10392,  10374,  10375,  10399,  10398}},
 {  5696,  21, { 10363,  10364,  10394,  10393,  10375,  10376,  10400,  10399}},
 {  5697,  21, { 10342,  10371,  10395,  10350,  10346,  10383,  10401,  10352}},
 {  5698,  21, { 10371,  10372,  10396,  10395,  10383,  10384,  10402,  10401}},
 {  5699,  21, { 10372,  10373,  10397,  10396,  10384,  10385,  10403,  10402}},
 {  5700,  21, { 10373,  10374,  10398,  10397,  10385,  10386,  10404,  10403}},
 {  5701,  21, { 10374,  10375,  10399,  10398,  10386,  10387,  10405,  10404}},
 {  5702,  21, { 10375,  10376,  10400,  10399,  10387,  10388,  10406,  10405}},
 {  5703,  21, {  3127,   3126,  10409,  10364,   3123,   3122,  10413,  10376}},
 {  5704,  21, {  3126,   3067,  10410,  10409,   3122,   3063,  10414,  10413}},
 {  5705,  21, {  3123,   3122,  10413,  10376,   3119,   3118,  10417,  10388}},
 {  5706,  21, {  3122,   3063,  10414,  10413,   3118,   3059,  10418,  10417}},
 {  5707,  21, { 10364,  10409,  10419,  10394,  10376,  10413,  10421,  10400}},
 {  5708,  21, { 10409,  10410,  10420,  10419,  10413,  10414,  10422,  10421}},
 {  5709,  21, { 10376,  10413,  10421,  10400,  10388,  10417,  10423,  10406}},
 {  5710,  21, { 10413,  10414,  10422,  10421,  10417,  10418,  10424,  10423}},
 {  5711,  21, {  3067,   3066,  10431,  10410,   3063,   3062,  10443,  10414}},
 {  5713,  21, {  3469,   3468,  10433,  10432,   3461,   3460,  10445,  10444}},
 {  5714,  21, {  3468,   3467,  10434,  10433,   3460,   3459,  10446,  10445}},
 {  5715,  21, {  3467,   3466,  10435,  10434,   3459,   3458,  10447,  10446}},
 {  5716,  21, {  3466,  10430,  10436,  10435,   3458,  10442,  10448,  10447}},
 {  5717,  21, {  3063,   3062,  10443,  10414,   3059,   3058,  10455,  10418}},
 {  5719,  21, {  3461,   3460,  10445,  10444,   3453,   3452,  10457,  10456}},
 {  5720,  21, {  3460,   3459,  10446,  10445,   3452,   3451,  10458,  10457}},
 {  5721,  21, {  3459,   3458,  10447,  10446,   3451,   3450,  10459,  10458}},
 {  5722,  21, {  3458,  10442,  10448,  10447,   3450,  10454,  10460,  10459}},
 {  5723,  21, { 10410,  10431,  10461,  10420,  10414,  10443,  10467,  10422}},
 {  5725,  21, { 10432,  10433,  10463,  10462,  10444,  10445,  10469,  10468}},
 {  5726,  21, { 10433,  10434,  10464,  10463,  10445,  10446,  10470,  10469}},
 {  5727,  21, { 10434,  10435,  10465,  10464,  10446,  10447,  10471,  10470}},
 {  5728,  21, { 10435,  10436,  10466,  10465,  10447,  10448,  10472,  10471}},
 {  5729,  21, { 10414,  10443,  10467,  10422,  10418,  10455,  10473,  10424}},
 {  5731,  21, { 10444,  10445,  10469,  10468,  10456,  10457,  10475,  10474}},
 {  5732,  21, { 10445,  10446,  10470,  10469,  10457,  10458,  10476,  10475}},
 {  5733,  21, { 10446,  10447,  10471,  10470,  10458,  10459,  10477,  10476}},
 {  5734,  21, { 10447,  10448,  10472,  10471,  10459,  10460,  10478,  10477}},
 {  5735,  21, { 10430,  10479,  10480,  10436,  10442,  10481,  10482,  10448}},
 {  5736,  21, { 10442,  10481,  10482,  10448,  10454,  10483,  10484,  10460}},
 {  5737,  21, { 10436,  10480,  10485,  10466,  10448,  10482,  10486,  10472}},
 {  5738,  21, { 10448,  10482,  10486,  10472,  10460,  10484,  10487,  10478}},
 {  5739,  21, { 10488,   3373,  10491,  10490,  10492,   3365,  10495,  10494}},
 {  5740,  21, { 10492,   3365,  10495,  10494,  10496,   3357,  10499,  10498}},
 {  5741,  21, { 10490,  10491,  10501,  10500,  10494,  10495,  10503,  10502}},
 {  5742,  21, { 10494,  10495,  10503,  10502,  10498,  10499,  10505,  10504}},
 {  5743,  21, {  3373,   3372,  10512,  10491,   3365,   3364,  10524,  10495}},
 {  5744,  21, {  3372,   3371,  10513,  10512,   3364,   3363,  10525,  10524}},
 {  5745,  21, {  3371,   3370,  10514,  10513,   3363,   3362,  10526,  10525}},
 {  5746,  21, {  3370,  10509,  10515,  10514,   3362,  10521,  10527,  10526}},
 {  5747,  21, { 10509,   3187,  10516,  10515,  10521,   3185,  10528,  10527}},
 {  5748,  21, {  3187,   3139,  10517,  10516,   3185,   3135,  10529,  10528}},
 {  5749,  21, {  3365,   3364,  10524,  10495,   3357,   3356,  10536,  10499}},
 {  5750,  21, {  3364,   3363,  10525,  10524,   3356,   3355,  10537,  10536}},
 {  5751,  21, {  3363,   3362,  10526,  10525,   3355,   3354,  10538,  10537}},
 {  5752,  21, {  3362,  10521,  10527,  10526,   3354,  10533,  10539,  10538}},
 {  5753,  21, { 10521,   3185,  10528,  10527,  10533,   3183,  10540,  10539}},
 {  5754,  21, {  3185,   3135,  10529,  10528,   3183,   3131,  10541,  10540}},
 {  5755,  21, { 10491,  10512,  10542,  10501,  10495,  10524,  10548,  10503}},
 {  5756,  21, { 10512,  10513,  10543,  10542,  10524,  10525,  10549,  10548}},
 {  5757,  21, { 10513,  10514,  10544,  10543,  10525,  10526,  10550,  10549}},
 {  5758,  21, { 10514,  10515,  10545,  10544,  10526,  10527,  10551,  10550}},
 {  5759,  21, { 10515,  10516,  10546,  10545,  10527,  10528,  10552,  10551}},
 {  5760,  21, { 10516,  10517,  10547,  10546,  10528,  10529,  10553,  10552}},
 {  5761,  21, { 10495,  10524,  10548,  10503,  10499,  10536,  10554,  10505}},
 {  5762,  21, { 10524,  10525,  10549,  10548,  10536,  10537,  10555,  10554}},
 {  5763,  21, { 10525,  10526,  10550,  10549,  10537,  10538,  10556,  10555}},
 {  5764,  21, { 10526,  10527,  10551,  10550,  10538,  10539,  10557,  10556}},
 {  5765,  21, { 10527,  10528,  10552,  10551,  10539,  10540,  10558,  10557}},
 {  5766,  21, { 10528,  10529,  10553,  10552,  10540,  10541,  10559,  10558}},
 {  5767,  21, {  3139,   3138,  10562,  10517,   3135,   3134,  10566,  10529}},
 {  5768,  21, {  3138,   3079,  10563,  10562,   3134,   3075,  10567,  10566}},
 {  5769,  21, {  3135,   3134,  10566,  10529,   3131,   3130,  10570,  10541}},
 {  5770,  21, {  3134,   3075,  10567,  10566,   3130,   3071,  10571,  10570}},
 {  5771,  21, { 10517,  10562,  10572,  10547,  10529,  10566,  10574,  10553}},
 {  5772,  21, { 10562,  10563,  10573,  10572,  10566,  10567,  10575,  10574}},
 {  5773,  21, { 10529,  10566,  10574,  10553,  10541,  10570,  10576,  10559}},
 {  5774,  21, { 10566,  10567,  10575,  10574,  10570,  10571,  10577,  10576}},
 {  5775,  21, {  3079,   3078,  10584,  10563,   3075,   3074,  10596,  10567}},
 {  5777,  21, {  3493,   3492,  10586,  10585,   3485,   3484,  10598,  10597}},
 {  5778,  21, {  3492,   3491,  10587,  10586,   3484,   3483,  10599,  10598}},
 {  5779,  21, {  3491,   3490,  10588,  10587,   3483,   3482,  10600,  10599}},
 {  5780,  21, {  3490,  10583,  10589,  10588,   3482,  10595,  10601,  10600}},
 {  5781,  21, {  3075,   3074,  10596,  10567,   3071,   3070,  10608,  10571}},
 {  5783,  21, {  3485,   3484,  10598,  10597,   3477,   3476,  10610,  10609}},
 {  5784,  21, {  3484,   3483,  10599,  10598,   3476,   3475,  10611,  10610}},
 {  5785,  21, {  3483,   3482,  10600,  10599,   3475,   3474,  10612,  10611}},
 {  5786,  21, {  3482,  10595,  10601,  10600,   3474,  10607,  10613,  10612}},
 {  5787,  21, { 10563,  10584,  10614,  10573,  10567,  10596,  10620,  10575}},
 {  5789,  21, { 10585,  10586,  10616,  10615,  10597,  10598,  10622,  10621}},
 {  5790,  21, { 10586,  10587,  10617,  10616,  10598,  10599,  10623,  10622}},
 {  5791,  21, { 10587,  10588,  10618,  10617,  10599,  10600,  10624,  10623}},
 {  5792,  21, { 10588,  10589,  10619,  10618,  10600,  10601,  10625,  10624}},
 {  5793,  21, { 10567,  10596,  10620,  10575,  10571,  10608,  10626,  10577}},
 {  5795,  21, { 10597,  10598,  10622,  10621,  10609,  10610,  10628,  10627}},
 {  5796,  21, { 10598,  10599,  10623,  10622,  10610,  10611,  10629,  10628}},
 {  5797,  21, { 10599,  10600,  10624,  10623,  10611,  10612,  10630,  10629}},
 {  5798,  21, { 10600,  10601,  10625,  10624,  10612,  10613,  10631,  10630}},
 {  5799,  21, { 10583,  10632,  10633,  10589,  10595,  10634,  10635,  10601}},
 {  5800,  21, { 10595,  10634,  10635,  10601,  10607,  10636,  10637,  10613}},
 {  5801,  21, { 10589,  10633,  10638,  10619,  10601,  10635,  10639,  10625}},
 {  5802,  21, { 10601,  10635,  10639,  10625,  10613,  10637,  10640,  10631}},
 {  5803,  21, { 10641,   3397,  10644,  10643,  10645,   3389,  10648,  10647}},
 {  5804,  21, { 10645,   3389,  10648,  10647,  10649,   3381,  10652,  10651}},
 {  5805,  21, { 10643,  10644,  10654,  10653,  10647,  10648,  10656,  10655}},
 {  5806,  21, { 10647,  10648,  10656,  10655,  10651,  10652,  10658,  10657}},
 {  5807,  21, {  3397,   3396,  10665,  10644,   3389,   3388,  10677,  10648}},
 {  5808,  21, {  3396,   3395,  10666,  10665,   3388,   3387,  10678,  10677}},
 {  5809,  21, {  3395,   3394,  10667,  10666,   3387,   3386,  10679,  10678}},
 {  5810,  21, {  3394,  10662,  10668,  10667,   3386,  10674,  10680,  10679}},
 {  5811,  21, { 10662,   3193,  10669,  10668,  10674,   3191,  10681,  10680}},
 {  5812,  21, {  3193,   3151,  10670,  10669,   3191,   3147,  10682,  10681}},
 {  5813,  21, {  3389,   3388,  10677,  10648,   3381,   3380,  10689,  10652}},
 {  5814,  21, {  3388,   3387,  10678,  10677,   3380,   3379,  10690,  10689}},
 {  5815,  21, {  3387,   3386,  10679,  10678,   3379,   3378,  10691,  10690}},
 {  5816,  21, {  3386,  10674,  10680,  10679,   3378,  10686,  10692,  10691}},
 {  5817,  21, { 10674,   3191,  10681,  10680,  10686,   3189,  10693,  10692}},
 {  5818,  21, {  3191,   3147,  10682,  10681,   3189,   3143,  10694,  10693}},
 {  5819,  21, { 10644,  10665,  10695,  10654,  10648,  10677,  10701,  10656}},
 {  5820,  21, { 10665,  10666,  10696,  10695,  10677,  10678,  10702,  10701}},
 {  5821,  21, { 10666,  10667,  10697,  10696,  10678,  10679,  10703,  10702}},
 {  5822,  21, { 10667,  10668,  10698,  10697,  10679,  10680,  10704,  10703}},
 {  5823,  21, { 10668,  10669,  10699,  10698,  10680,  10681,  10705,  10704}},
 {  5824,  21, { 10669,  10670,  10700,  10699,  10681,  10682,  10706,  10705}},
 {  5825,  21, { 10648,  10677,  10701,  10656,  10652,  10689,  10707,  10658}},
 {  5826,  21, { 10677,  10678,  10702,  10701,  10689,  10690,  10708,  10707}},
 {  5827,  21, { 10678,  10679,  10703,  10702,  10690,  10691,  10709,  10708}},
 {  5828,  21, { 10679,  10680,  10704,  10703,  10691,  10692,  10710,  10709}},
 {  5829,  21, { 10680,  10681,  10705,  10704,  10692,  10693,  10711,  10710}},
 {  5830,  21, { 10681,  10682,  10706,  10705,  10693,  10694,  10712,  10711}},
 {  5831,  21, {  3151,   3150,  10715,  10670,   3147,   3146,  10719,  10682}},
 {  5832,  21, {  3150,   3091,  10716,  10715,   3146,   3087,  10720,  10719}},
 {  5833,  21, {  3147,   3146,  10719,  10682,   3143,   3142,  10723,  10694}},
 {  5834,  21, {  3146,   3087,  10720,  10719,   3142,   3083,  10724,  10723}},
 {  5835,  21, { 10670,  10715,  10725,  10700,  10682,  10719,  10727,  10706}},
 {  5836,  21, { 10715,  10716,  10726,  10725,  10719,  10720,  10728,  10727}},
 {  5837,  21, { 10682,  10719,  10727,  10706,  10694,  10723,  10729,  10712}},
 {  5838,  21, { 10719,  10720,  10728,  10727,  10723,  10724,  10730,  10729}},
 {  5839,  21, {  3091,   3090,  10737,  10716,   3087,   3086,  10749,  10720}},
 {  5841,  21, {  3517,   3516,  10739,  10738,   3509,   3508,  10751,  10750}},
 {  5842,  21, {  3516,   3515,  10740,  10739,   3508,   3507,  10752,  10751}},
 {  5843,  21, {  3515,   3514,  10741,  10740,   3507,   3506,  10753,  10752}},
 {  5844,  21, {  3514,  10736,  10742,  10741,   3506,  10748,  10754,  10753}},
 {  5845,  21, {  3087,   3086,  10749,  10720,   3083,   3082,  10761,  10724}},
 {  5847,  21, {  3509,   3508,  10751,  10750,   3501,   3500,  10763,  10762}},
 {  5848,  21, {  3508,   3507,  10752,  10751,   3500,   3499,  10764,  10763}},
 {  5849,  21, {  3507,   3506,  10753,  10752,   3499,   3498,  10765,  10764}},
 {  5850,  21, {  3506,  10748,  10754,  10753,   3498,  10760,  10766,  10765}},
 {  5851,  21, { 10716,  10737,  10767,  10726,  10720,  10749,  10773,  10728}},
 {  5853,  21, { 10738,  10739,  10769,  10768,  10750,  10751,  10775,  10774}},
 {  5854,  21, { 10739,  10740,  10770,  10769,  10751,  10752,  10776,  10775}},
 {  5855,  21, { 10740,  10741,  10771,  10770,  10752,  10753,  10777,  10776}},
 {  5856,  21, { 10741,  10742,  10772,  10771,  10753,  10754,  10778,  10777}},
 {  5857,  21, { 10720,  10749,  10773,  10728,  10724,  10761,  10779,  10730}},
 {  5859,  21, { 10750,  10751,  10775,  10774,  10762,  10763,  10781,  10780}},
 {  5860,  21, { 10751,  10752,  10776,  10775,  10763,  10764,  10782,  10781}},
 {  5861,  21, { 10752,  10753,  10777,  10776,  10764,  10765,  10783,  10782}},
 {  5862,  21, { 10753,  10754,  10778,  10777,  10765,  10766,  10784,  10783}},
 {  5863,  21, { 10736,  10785,  10786,  10742,  10748,  10787,  10788,  10754}},
 {  5864,  21, { 10748,  10787,  10788,  10754,  10760,  10789,  10790,  10766}},
 {  5865,  21, { 10742,  10786,  10791,  10772,  10754,  10788,  10792,  10778}},
 {  5866,  21, { 10754,  10788,  10792,  10778,  10766,  10790,  10793,  10784}},
 {  5867,  21, { 10794,   3421,  10797,  10796,  10798,   3413,  10801,  10800}},
 {  5868,  21, { 10798,   3413,  10801,  10800,  10802,   3405,  10805,  10804}},
 {  5869,  21, { 10796,  10797,  10807,  10806,  10800,  10801,  10809,  10808}},
 {  5870,  21, { 10800,  10801,  10809,  10808,  10804,  10805,  10811,  10810}},
 {  5871,  21, {  3421,   3420,  10818,  10797,   3413,   3412,  10830,  10801}},
 {  5872,  21, {  3420,   3419,  10819,  10818,   3412,   3411,  10831,  10830}},
 {  5873,  21, {  3419,   3418,  10820,  10819,   3411,   3410,  10832,  10831}},
 {  5874,  21, {  3418,  10815,  10821,  10820,   3410,  10827,  10833,  10832}},
 {  5875,  21, { 10815,   3199,  10822,  10821,  10827,   3197,  10834,  10833}},
 {  5876,  21, {  3199,   3163,  10823,  10822,   3197,   3159,  10835,  10834}},
 {  5877,  21, {  3413,   3412,  10830,  10801,   3405,   3404,  10842,  10805}},
 {  5878,  21, {  3412,   3411,  10831,  10830,   3404,   3403,  10843,  10842}},
 {  5879,  21, {  3411,   3410,  10832,  10831,   3403,   3402,  10844,  10843}},
 {  5880,  21, {  3410,  10827,  10833,  10832,   3402,  10839,  10845,  10844}},
 {  5881,  21, { 10827,   3197,  10834,  10833,  10839,   3195,  10846,  10845}},
 {  5882,  21, {  3197,   3159,  10835,  10834,   3195,   3155,  10847,  10846}},
 {  5883,  21, { 10797,  10818,  10848,  10807,  10801,  10830,  10854,  10809}},
 {  5884,  21, { 10818,  10819,  10849,  10848,  10830,  10831,  10855,  10854}},
 {  5885,  21, { 10819,  10820,  10850,  10849,  10831,  10832,  10856,  10855}},
 {  5886,  21, { 10820,  10821,  10851,  10850,  10832,  10833,  10857,  10856}},
 {  5887,  21, { 10821,  10822,  10852,  10851,  10833,  10834,  10858,  10857}},
 {  5888,  21, { 10822,  10823,  10853,  10852,  10834,  10835,  10859,  10858}},
 {  5889,  21, { 10801,  10830,  10854,  10809,  10805,  10842,  10860,  10811}},
 {  5890,  21, { 10830,  10831,  10855,  10854,  10842,  10843,  10861,  10860}},
 {  5891,  21, { 10831,  10832,  10856,  10855,  10843,  10844,  10862,  10861}},
 {  5892,  21, { 10832,  10833,  10857,  10856,  10844,  10845,  10863,  10862}},
 {  5893,  21, { 10833,  10834,  10858,  10857,  10845,  10846,  10864,  10863}},
 {  5894,  21, { 10834,  10835,  10859,  10858,  10846,  10847,  10865,  10864}},
 {  5895,  21, {  3163,   3162,  10868,  10823,   3159,   3158,  10872,  10835}},
 {  5896,  21, {  3162,   3103,  10869,  10868,   3158,   3099,  10873,  10872}},
 {  5897,  21, {  3159,   3158,  10872,  10835,   3155,   3154,  10876,  10847}},
 {  5898,  21, {  3158,   3099,  10873,  10872,   3154,   3095,  10877,  10876}},
 {  5899,  21, { 10823,  10868,  10878,  10853,  10835,  10872,  10880,  10859}},
 {  5900,  21, { 10868,  10869,  10879,  10878,  10872,  10873,  10881,  10880}},
 {  5901,  21, { 10835,  10872,  10880,  10859,  10847,  10876,  10882,  10865}},
 {  5902,  21, { 10872,  10873,  10881,  10880,  10876,  10877,  10883,  10882}},
 {  5903,  21, {  3103,   3102,  10890,  10869,   3099,   3098,  10902,  10873}},
 {  5905,  21, {  3541,   3540,  10892,  10891,   3533,   3532,  10904,  10903}},
 {  5906,  21, {  3540,   3539,  10893,  10892,   3532,   3531,  10905,  10904}},
 {  5907,  21, {  3539,   3538,  10894,  10893,   3531,   3530,  10906,  10905}},
 {  5908,  21, {  3538,  10889,  10895,  10894,   3530,  10901,  10907,  10906}},
 {  5909,  21, {  3099,   3098,  10902,  10873,   3095,   3094,  10914,  10877}},
 {  5911,  21, {  3533,   3532,  10904,  10903,   3525,   3524,  10916,  10915}},
 {  5912,  21, {  3532,   3531,  10905,  10904,   3524,   3523,  10917,  10916}},
 {  5913,  21, {  3531,   3530,  10906,  10905,   3523,   3522,  10918,  10917}},
 {  5914,  21, {  3530,  10901,  10907,  10906,   3522,  10913,  10919,  10918}},
 {  5915,  21, { 10869,  10890,  10920,  10879,  10873,  10902,  10926,  10881}},
 {  5917,  21, { 10891,  10892,  10922,  10921,  10903,  10904,  10928,  10927}},
 {  5918,  21, { 10892,  10893,  10923,  10922,  10904,  10905,  10929,  10928}},
 {  5919,  21, { 10893,  10894,  10924,  10923,  10905,  10906,  10930,  10929}},
 {  5920,  21, { 10894,  10895,  10925,  10924,  10906,  10907,  10931,  10930}},
 {  5921,  21, { 10873,  10902,  10926,  10881,  10877,  10914,  10932,  10883}},
 {  5923,  21, { 10903,  10904,  10928,  10927,  10915,  10916,  10934,  10933}},
 {  5924,  21, { 10904,  10905,  10929,  10928,  10916,  10917,  10935,  10934}},
 {  5925,  21, { 10905,  10906,  10930,  10929,  10917,  10918,  10936,  10935}},
 {  5926,  21, { 10906,  10907,  10931,  10930,  10918,  10919,  10937,  10936}},
 {  5927,  21, { 10889,  10938,  10939,  10895,  10901,  10940,  10941,  10907}},
 {  5928,  21, { 10901,  10940,  10941,  10907,  10913,  10942,  10943,  10919}},
 {  5929,  21, { 10895,  10939,  10944,  10925,  10907,  10941,  10945,  10931}},
 {  5930,  21, { 10907,  10941,  10945,  10931,  10919,  10943,  10946,  10937}},
 {  5931,  21, { 10947,   3445,  10950,  10949,  10951,   3437,  10954,  10953}},
 {  5932,  21, { 10951,   3437,  10954,  10953,  10955,   3429,  10958,  10957}},
 {  5933,  21, { 10949,  10950,  10960,  10959,  10953,  10954,  10962,  10961}},
 {  5934,  21, { 10953,  10954,  10962,  10961,  10957,  10958,  10964,  10963}},
 {  5935,  21, {  3445,   3444,  10971,  10950,   3437,   3436,  10983,  10954}},
 {  5936,  21, {  3444,   3443,  10972,  10971,   3436,   3435,  10984,  10983}},
 {  5937,  21, {  3443,   3442,  10973,  10972,   3435,   3434,  10985,  10984}},
 {  5938,  21, {  3442,  10968,  10974,  10973,   3434,  10980,  10986,  10985}},
 {  5939,  21, { 10968,   3205,  10975,  10974,  10980,   3203,  10987,  10986}},
 {  5940,  21, {  3205,   3175,  10976,  10975,   3203,   3171,  10988,  10987}},
 {  5941,  21, {  3437,   3436,  10983,  10954,   3429,   3428,  10995,  10958}},
 {  5942,  21, {  3436,   3435,  10984,  10983,   3428,   3427,  10996,  10995}},
 {  5943,  21, {  3435,   3434,  10985,  10984,   3427,   3426,  10997,  10996}},
 {  5944,  21, {  3434,  10980,  10986,  10985,   3426,  10992,  10998,  10997}},
 {  5945,  21, { 10980,   3203,  10987,  10986,  10992,   3201,  10999,  10998}},
 {  5946,  21, {  3203,   3171,  10988,  10987,   3201,   3167,  11000,  10999}},
 {  5947,  21, { 10950,  10971,  11001,  10960,  10954,  10983,  11007,  10962}},
 {  5948,  21, { 10971,  10972,  11002,  11001,  10983,  10984,  11008,  11007}},
 {  5949,  21, { 10972,  10973,  11003,  11002,  10984,  10985,  11009,  11008}},
 {  5950,  21, { 10973,  10974,  11004,  11003,  10985,  10986,  11010,  11009}},
 {  5951,  21, { 10974,  10975,  11005,  11004,  10986,  10987,  11011,  11010}},
 {  5952,  21, { 10975,  10976,  11006,  11005,  10987,  10988,  11012,  11011}},
 {  5953,  21, { 10954,  10983,  11007,  10962,  10958,  10995,  11013,  10964}},
 {  5954,  21, { 10983,  10984,  11008,  11007,  10995,  10996,  11014,  11013}},
 {  5955,  21, { 10984,  10985,  11009,  11008,  10996,  10997,  11015,  11014}},
 {  5956,  21, { 10985,  10986,  11010,  11009,  10997,  10998,  11016,  11015}},
 {  5957,  21, { 10986,  10987,  11011,  11010,  10998,  10999,  11017,  11016}},
 {  5958,  21, { 10987,  10988,  11012,  11011,  10999,  11000,  11018,  11017}},
 {  5959,  21, {  3175,   3174,  11021,  10976,   3171,   3170,  11025,  10988}},
 {  5960,  21, {  3174,   3115,  11022,  11021,   3170,   3111,  11026,  11025}},
 {  5961,  21, {  3171,   3170,  11025,  10988,   3167,   3166,  11029,  11000}},
 {  5962,  21, {  3170,   3111,  11026,  11025,   3166,   3107,  11030,  11029}},
 {  5963,  21, { 10976,  11021,  11031,  11006,  10988,  11025,  11033,  11012}},
 {  5964,  21, { 11021,  11022,  11032,  11031,  11025,  11026,  11034,  11033}},
 {  5965,  21, { 10988,  11025,  11033,  11012,  11000,  11029,  11035,  11018}},
 {  5966,  21, { 11025,  11026,  11034,  11033,  11029,  11030,  11036,  11035}},
 {  5967,  21, {  3115,   3114,  11043,  11022,   3111,   3110,  11055,  11026}},
 {  5969,  21, {  3565,   3564,  11045,  11044,   3557,   3556,  11057,  11056}},
 {  5970,  21, {  3564,   3563,  11046,  11045,   3556,   3555,  11058,  11057}},
 {  5971,  21, {  3563,   3562,  11047,  11046,   3555,   3554,  11059,  11058}},
 {  5972,  21, {  3562,  11042,  11048,  11047,   3554,  11054,  11060,  11059}},
 {  5973,  21, {  3111,   3110,  11055,  11026,   3107,   3106,  11067,  11030}},
 {  5975,  21, {  3557,   3556,  11057,  11056,   3549,   3548,  11069,  11068}},
 {  5976,  21, {  3556,   3555,  11058,  11057,   3548,   3547,  11070,  11069}},
 {  5977,  21, {  3555,   3554,  11059,  11058,   3547,   3546,  11071,  11070}},
 {  5978,  21, {  3554,  11054,  11060,  11059,   3546,  11066,  11072,  11071}},
 {  5979,  21, { 11022,  11043,  11073,  11032,  11026,  11055,  11079,  11034}},
 {  5981,  21, { 11044,  11045,  11075,  11074,  11056,  11057,  11081,  11080}},
 {  5982,  21, { 11045,  11046,  11076,  11075,  11057,  11058,  11082,  11081}},
 {  5983,  21, { 11046,  11047,  11077,  11076,  11058,  11059,  11083,  11082}},
 {  5984,  21, { 11047,  11048,  11078,  11077,  11059,  11060,  11084,  11083}},
 {  5985,  21, { 11026,  11055,  11079,  11034,  11030,  11067,  11085,  11036}},
 {  5987,  21, { 11056,  11057,  11081,  11080,  11068,  11069,  11087,  11086}},
 {  5988,  21, { 11057,  11058,  11082,  11081,  11069,  11070,  11088,  11087}},
 {  5989,  21, { 11058,  11059,  11083,  11082,  11070,  11071,  11089,  11088}},
 {  5990,  21, { 11059,  11060,  11084,  11083,  11071,  11072,  11090,  11089}},
 {  5991,  21, { 11042,  11091,  11092,  11048,  11054,  11093,  11094,  11060}},
 {  5992,  21, { 11054,  11093,  11094,  11060,  11066,  11095,  11096,  11072}},
 {  5993,  21, { 11048,  11092,  11097,  11078,  11060,  11094,  11098,  11084}},
 {  5994,  21, { 11060,  11094,  11098,  11084,  11072,  11096,  11099,  11090}},
 {  5995,  22, { 11112,  11113,  11117,  11116,  11100,  11101,  11105,  11104}},
 {  5996,  22, { 11113,  11114,  11118,  11117,  11101,  11102,  11106,  11105}},
 {  5997,  22, { 11114,  11115,  11119,  11118,  11102,  11103,  11107,  11106}},
 {  5998,  22, { 11116,  11117,   1407,      6,  11104,  11105,  11109,  11108}},
 {  5999,  22, { 11117,  11118,   1408,   1407,  11105,  11106,  11110,  11109}},
 {  6000,  22, { 11118,  11119,   1409,   1408,  11106,  11107,  11111,  11110}},
 {  6001,  22, { 11120,  11121,  11125,  11124,  11112,  11113,  11117,  11116}},
 {  6002,  22, { 11121,  11122,  11126,  11125,  11113,  11114,  11118,  11117}},
 {  6003,  22, { 11122,  11123,  11127,  11126,  11114,  11115,  11119,  11118}},
 {  6004,  22, { 11124,  11125,  11129,  11128,  11116,  11117,   1407,      6}},
 {  6005,  22, { 11125,  11126,  11130,  11129,  11117,  11118,   1408,   1407}},
 {  6006,  22, { 11126,  11127,  11131,  11130,  11118,  11119,   1409,   1408}},
 {  6007,  22, { 11132,  11133,  11137,  11136,  11120,  11121,  11125,  11124}},
 {  6008,  22, { 11133,  11134,  11138,  11137,  11121,  11122,  11126,  11125}},
 {  6009,  22, { 11134,  11135,  11139,  11138,  11122,  11123,  11127,  11126}},
 {  6010,  22, { 11136,  11137,  11141,  11140,  11124,  11125,  11129,  11128}},
 {  6011,  22, { 11137,  11138,  11142,  11141,  11125,  11126,  11130,  11129}},
 {  6012,  22, { 11138,  11139,  11143,  11142,  11126,  11127,  11131,  11130}},
 {  6013,  22, {    84,   1598,  11157,  11156,  11144,  11145,  11149,  11148}},
 {  6014,  22, {  1598,   1599,  11158,  11157,  11145,  11146,  11150,  11149}},
 {  6015,  22, {  1599,   1600,  11159,  11158,  11146,  11147,  11151,  11150}},
 {  6016,  22, { 11156,  11157,  11161,  11160,  11148,  11149,  11153,  11152}},
 {  6017,  22, { 11157,  11158,  11162,  11161,  11149,  11150,  11154,  11153}},
 {  6018,  22, { 11158,  11159,  11163,  11162,  11150,  11151,  11155,  11154}},
 {  6019,  22, { 11164,  11165,  11169,  11168,     84,   1598,  11157,  11156}},
 {  6020,  22, { 11165,  11166,  11170,  11169,   1598,   1599,  11158,  11157}},
 {  6021,  22, { 11166,  11167,  11171,  11170,   1599,   1600,  11159,  11158}},
 {  6022,  22, { 11168,  11169,  11173,  11172,  11156,  11157,  11161,  11160}},
 {  6023,  22, { 11169,  11170,  11174,  11173,  11157,  11158,  11162,  11161}},
 {  6024,  22, { 11170,  11171,  11175,  11174,  11158,  11159,  11163,  11162}},
 {  6025,  22, { 11176,  11177,  11181,  11180,  11164,  11165,  11169,  11168}},
 {  6026,  22, { 11177,  11178,  11182,  11181,  11165,  11166,  11170,  11169}},
 {  6027,  22, { 11178,  11179,  11183,  11182,  11166,  11167,  11171,  11170}},
 {  6028,  22, { 11180,  11181,  11185,  11184,  11168,  11169,  11173,  11172}},
 {  6029,  22, { 11181,  11182,  11186,  11185,  11169,  11170,  11174,  11173}},
 {  6030,  22, { 11182,  11183,  11187,  11186,  11170,  11171,  11175,  11174}},
 {  6031,  22, { 11115,  11197,  11200,  11119,  11103,  11188,  11191,  11107}},
 {  6032,  22, { 11197,  11198,  11201,  11200,  11188,  11189,  11192,  11191}},
 {  6033,  22, { 11198,  11199,  11202,  11201,  11189,  11190,  11193,  11192}},
 {  6034,  22, { 11119,  11200,  11203,   1409,  11107,  11191,  11194,  11111}},
 {  6035,  22, { 11200,  11201,  11204,  11203,  11191,  11192,  11195,  11194}},
 {  6036,  22, { 11201,  11202,  11205,  11204,  11192,  11193,  11196,  11195}},
 {  6037,  22, { 11123,  11206,  11209,  11127,  11115,  11197,  11200,  11119}},
 {  6038,  22, { 11206,  11207,  11210,  11209,  11197,  11198,  11201,  11200}},
 {  6039,  22, { 11207,  11208,  11211,  11210,  11198,  11199,  11202,  11201}},
 {  6040,  22, { 11127,  11209,  11212,  11131,  11119,  11200,  11203,   1409}},
 {  6041,  22, { 11209,  11210,  11213,  11212,  11200,  11201,  11204,  11203}},
 {  6042,  22, { 11210,  11211,  11214,  11213,  11201,  11202,  11205,  11204}},
 {  6043,  22, { 11135,  11215,  11218,  11139,  11123,  11206,  11209,  11127}},
 {  6044,  22, { 11215,  11216,  11219,  11218,  11206,  11207,  11210,  11209}},
 {  6045,  22, { 11216,  11217,  11220,  11219,  11207,  11208,  11211,  11210}},
 {  6046,  22, { 11139,  11218,  11221,  11143,  11127,  11209,  11212,  11131}},
 {  6047,  22, { 11218,  11219,  11222,  11221,  11209,  11210,  11213,  11212}},
 {  6048,  22, { 11219,  11220,  11223,  11222,  11210,  11211,  11214,  11213}},
 {  6049,  22, {  1409,  11203,  11228,   1425,  11111,  11194,  11225,  11224}},
 {  6050,  22, { 11203,  11204,  11229,  11228,  11194,  11195,  11226,  11225}},
 {  6051,  22, { 11204,  11205,  11230,  11229,  11195,  11196,  11227,  11226}},
 {  6052,  22, { 11131,  11212,  11232,  11231,   1409,  11203,  11228,   1425}},
 {  6053,  22, { 11212,  11213,  11233,  11232,  11203,  11204,  11229,  11228}},
 {  6054,  22, { 11213,  11214,  11234,  11233,  11204,  11205,  11230,  11229}},
 {  6055,  22, { 11143,  11221,  11236,  11235,  11131,  11212,  11232,  11231}},
 {  6056,  22, { 11221,  11222,  11237,  11236,  11212,  11213,  11233,  11232}},
 {  6057,  22, { 11222,  11223,  11238,  11237,  11213,  11214,  11234,  11233}},
 {  6058,  22, {  1425,  11228,  11243,   1481,  11224,  11225,  11240,  11239}},
 {  6059,  22, { 11228,  11229,  11244,  11243,  11225,  11226,  11241,  11240}},
 {  6060,  22, { 11229,  11230,  11245,  11244,  11226,  11227,  11242,  11241}},
 {  6061,  22, { 11231,  11232,  11247,  11246,   1425,  11228,  11243,   1481}},
 {  6062,  22, { 11232,  11233,  11248,  11247,  11228,  11229,  11244,  11243}},
 {  6063,  22, { 11233,  11234,  11249,  11248,  11229,  11230,  11245,  11244}},
 {  6064,  22, { 11235,  11236,  11251,  11250,  11231,  11232,  11247,  11246}},
 {  6065,  22, { 11236,  11237,  11252,  11251,  11232,  11233,  11248,  11247}},
 {  6066,  22, { 11237,  11238,  11253,  11252,  11233,  11234,  11249,  11248}},
 {  6067,  22, {  1481,  11243,  11258,   1489,  11239,  11240,  11255,  11254}},
 {  6068,  22, { 11243,  11244,  11259,  11258,  11240,  11241,  11256,  11255}},
 {  6069,  22, { 11244,  11245,  11260,  11259,  11241,  11242,  11257,  11256}},
 {  6070,  22, { 11246,  11247,  11262,  11261,   1481,  11243,  11258,   1489}},
 {  6071,  22, { 11247,  11248,  11263,  11262,  11243,  11244,  11259,  11258}},
 {  6072,  22, { 11248,  11249,  11264,  11263,  11244,  11245,  11260,  11259}},
 {  6073,  22, { 11250,  11251,  11266,  11265,  11246,  11247,  11262,  11261}},
 {  6074,  22, { 11251,  11252,  11267,  11266,  11247,  11248,  11263,  11262}},
 {  6075,  22, { 11252,  11253,  11268,  11267,  11248,  11249,  11264,  11263}},
 {  6076,  22, {  1489,  11258,  11273,   1545,  11254,  11255,  11270,  11269}},
 {  6077,  22, { 11258,  11259,  11274,  11273,  11255,  11256,  11271,  11270}},
 {  6078,  22, { 11259,  11260,  11275,  11274,  11256,  11257,  11272,  11271}},
 {  6079,  22, { 11261,  11262,  11277,  11276,   1489,  11258,  11273,   1545}},
 {  6080,  22, { 11262,  11263,  11278,  11277,  11258,  11259,  11274,  11273}},
 {  6081,  22, { 11263,  11264,  11279,  11278,  11259,  11260,  11275,  11274}},
 {  6082,  22, { 11265,  11266,  11281,  11280,  11261,  11262,  11277,  11276}},
 {  6083,  22, { 11266,  11267,  11282,  11281,  11262,  11263,  11278,  11277}},
 {  6084,  22, { 11267,  11268,  11283,  11282,  11263,  11264,  11279,  11278}},
 {  6085,  22, {  1545,  11273,  11287,   1600,  11269,  11270,  11284,  11147}},
 {  6086,  22, { 11273,  11274,  11288,  11287,  11270,  11271,  11285,  11284}},
 {  6087,  22, { 11274,  11275,  11289,  11288,  11271,  11272,  11286,  11285}},
 {  6088,  22, { 11276,  11277,  11290,  11167,   1545,  11273,  11287,   1600}},
 {  6089,  22, { 11277,  11278,  11291,  11290,  11273,  11274,  11288,  11287}},
 {  6090,  22, { 11278,  11279,  11292,  11291,  11274,  11275,  11289,  11288}},
 {  6091,  22, { 11280,  11281,  11293,  11179,  11276,  11277,  11290,  11167}},
 {  6092,  22, { 11281,  11282,  11294,  11293,  11277,  11278,  11291,  11290}},
 {  6093,  22, { 11282,  11283,  11295,  11294,  11278,  11279,  11292,  11291}},
 {  6094,  22, {  1600,  11287,  11302,  11159,  11147,  11284,  11296,  11151}},
 {  6095,  22, { 11287,  11288,  11303,  11302,  11284,  11285,  11297,  11296}},
 {  6096,  22, { 11288,  11289,  11304,  11303,  11285,  11286,  11298,  11297}},
 {  6097,  22, { 11159,  11302,  11305,  11163,  11151,  11296,  11299,  11155}},
 {  6098,  22, { 11302,  11303,  11306,  11305,  11296,  11297,  11300,  11299}},
 {  6099,  22, { 11303,  11304,  11307,  11306,  11297,  11298,  11301,  11300}},
 {  6100,  22, { 11167,  11290,  11308,  11171,   1600,  11287,  11302,  11159}},
 {  6101,  22, { 11290,  11291,  11309,  11308,  11287,  11288,  11303,  11302}},
 {  6102,  22, { 11291,  11292,  11310,  11309,  11288,  11289,  11304,  11303}},
 {  6103,  22, { 11171,  11308,  11311,  11175,  11159,  11302,  11305,  11163}},
 {  6104,  22, { 11308,  11309,  11312,  11311,  11302,  11303,  11306,  11305}},
 {  6105,  22, { 11309,  11310,  11313,  11312,  11303,  11304,  11307,  11306}},
 {  6106,  22, { 11179,  11293,  11314,  11183,  11167,  11290,  11308,  11171}},
 {  6107,  22, { 11293,  11294,  11315,  11314,  11290,  11291,  11309,  11308}},
 {  6108,  22, { 11294,  11295,  11316,  11315,  11291,  11292,  11310,  11309}},
 {  6109,  22, { 11183,  11314,  11317,  11187,  11171,  11308,  11311,  11175}},
 {  6110,  22, { 11314,  11315,  11318,  11317,  11308,  11309,  11312,  11311}},
 {  6111,  22, { 11315,  11316,  11319,  11318,  11309,  11310,  11313,  11312}},
 {  6112,  24, { 11320,  11321,  11323,  11322,  11326,  11327,  11329,  11328}},
 {  6113,  24, { 11322,  11323,  11325,  11324,  11328,  11329,  11331,  11330}},
 {  6114,  24, { 11326,  11327,  11329,  11328,  11332,  11333,  11335,  11334}},
 {  6115,  24, { 11328,  11329,  11331,  11330,  11334,  11335,  11337,  11336}},
 {  6116,  24, { 11332,  11333,  11335,  11334,  11338,  11339,  11341,  11340}},
 {  6117,  24, { 11334,  11335,  11337,  11336,  11340,  11341,  11343,  11342}},
 {  6118,  24, { 11338,  11339,  11341,  11340,  11344,  11345,  11347,  11346}},
 {  6119,  24, { 11340,  11341,  11343,  11342,  11346,  11347,  11349,  11348}},
 {  6120,  24, { 11344,  11345,  11347,  11346,  11350,  11351,  11353,  11352}},
 {  6121,  24, { 11346,  11347,  11349,  11348,  11352,  11353,  11355,  11354}},
 {  6122,  24, { 11350,  11351,  11353,  11352,  11356,  11357,  11359,  11358}},
 {  6123,  24, { 11352,  11353,  11355,  11354,  11358,  11359,  11361,  11360}},
 {  6124,  24, { 11356,  11357,  11359,  11358,  11362,  11363,  11365,  11364}},
 {  6125,  24, { 11358,  11359,  11361,  11360,  11364,  11365,  11367,  11366}},
 {  6126,  24, { 11362,  11363,  11365,  11364,  11368,  11369,  11371,  11370}},
 {  6127,  24, { 11364,  11365,  11367,  11366,  11370,  11371,  11373,  11372}},
 {  6128,  24, { 11320,  11374,  11376,  11322,  11321,  11380,  11382,  11323}},
 {  6129,  24, { 11374,  11375,  11377,  11376,  11380,  11381,  11383,  11382}},
 {  6130,  24, { 11322,  11376,  11378,  11324,  11323,  11382,  11384,  11325}},
 {  6131,  24, { 11376,  11377,  11379,  11378,  11382,  11383,  11385,  11384}},
 {  6132,  24, { 11321,  11380,  11382,  11323,  11327,  11386,  11388,  11329}},
 {  6133,  24, { 11380,  11381,  11383,  11382,  11386,  11387,  11389,  11388}},
 {  6134,  24, { 11323,  11382,  11384,  11325,  11329,  11388,  11390,  11331}},
 {  6135,  24, { 11382,  11383,  11385,  11384,  11388,  11389,  11391,  11390}},
 {  6136,  24, { 11327,  11386,  11388,  11329,  11333,  11392,  11394,  11335}},
 {  6137,  24, { 11386,  11387,  11389,  11388,  11392,  11393,  11395,  11394}},
 {  6138,  24, { 11329,  11388,  11390,  11331,  11335,  11394,  11396,  11337}},
 {  6139,  24, { 11388,  11389,  11391,  11390,  11394,  11395,  11397,  11396}},
 {  6140,  24, { 11333,  11392,  11394,  11335,  11339,  11398,  11400,  11341}},
 {  6141,  24, { 11392,  11393,  11395,  11394,  11398,  11399,  11401,  11400}},
 {  6142,  24, { 11335,  11394,  11396,  11337,  11341,  11400,  11402,  11343}},
 {  6143,  24, { 11394,  11395,  11397,  11396,  11400,  11401,  11403,  11402}},
 {  6144,  24, { 11339,  11398,  11400,  11341,  11345,  11404,  11406,  11347}},
 {  6145,  24, { 11398,  11399,  11401,  11400,  11404,  11405,  11407,  11406}},
 {  6146,  24, { 11341,  11400,  11402,  11343,  11347,  11406,  11408,  11349}},
 {  6147,  24, { 11400,  11401,  11403,  11402,  11406,  11407,  11409,  11408}},
 {  6148,  24, { 11345,  11404,  11406,  11347,  11351,  11410,  11412,  11353}},
 {  6149,  24, { 11404,  11405,  11407,  11406,  11410,  11411,  11413,  11412}},
 {  6150,  24, { 11347,  11406,  11408,  11349,  11353,  11412,  11414,  11355}},
 {  6151,  24, { 11406,  11407,  11409,  11408,  11412,  11413,  11415,  11414}},
 {  6152,  24, { 11351,  11410,  11412,  11353,  11357,  11416,  11418,  11359}},
 {  6153,  24, { 11410,  11411,  11413,  11412,  11416,  11417,  11419,  11418}},
 {  6154,  24, { 11353,  11412,  11414,  11355,  11359,  11418,  11420,  11361}},
 {  6155,  24, { 11412,  11413,  11415,  11414,  11418,  11419,  11421,  11420}},
 {  6156,  24, { 11357,  11416,  11418,  11359,  11363,  11422,  11424,  11365}},
 {  6157,  24, { 11416,  11417,  11419,  11418,  11422,  11423,  11425,  11424}},
 {  6158,  24, { 11359,  11418,  11420,  11361,  11365,  11424,  11426,  11367}},
 {  6159,  24, { 11418,  11419,  11421,  11420,  11424,  11425,  11427,  11426}},
 {  6160,  24, { 11363,  11422,  11424,  11365,  11369,  11428,  11430,  11371}},
 {  6161,  24, { 11422,  11423,  11425,  11424,  11428,  11429,  11431,  11430}},
 {  6162,  24, { 11365,  11424,  11426,  11367,  11371,  11430,  11432,  11373}},
 {  6163,  24, { 11424,  11425,  11427,  11426,  11430,  11431,  11433,  11432}},
 {  6164,  24, { 11369,  11428,  11430,  11371,  11368,  11434,  11436,  11370}},
 {  6165,  24, { 11428,  11429,  11431,  11430,  11434,  11435,  11437,  11436}},
 {  6166,  24, { 11371,  11430,  11432,  11373,  11370,  11436,  11438,  11372}},
 {  6167,  24, { 11430,  11431,  11433,  11432,  11436,  11437,  11439,  11438}},
 {  6168,  24, { 11325,  11384,  11378,  11324,  11331,  11390,  11440,  11330}},
 {  6169,  24, { 11384,  11385,  11379,  11378,  11390,  11391,  11441,  11440}},
 {  6170,  24, { 11331,  11390,  11440,  11330,  11337,  11396,  11442,  11336}},
 {  6171,  24, { 11390,  11391,  11441,  11440,  11396,  11397,  11443,  11442}},
 {  6172,  24, { 11337,  11396,  11442,  11336,  11343,  11402,  11444,  11342}},
 {  6173,  24, { 11396,  11397,  11443,  11442,  11402,  11403,  11445,  11444}},
 {  6174,  24, { 11343,  11402,  11444,  11342,  11349,  11408,  11446,  11348}},
 {  6175,  24, { 11402,  11403,  11445,  11444,  11408,  11409,  11447,  11446}},
 {  6176,  24, { 11349,  11408,  11446,  11348,  11355,  11414,  11448,  11354}},
 {  6177,  24, { 11408,  11409,  11447,  11446,  11414,  11415,  11449,  11448}},
 {  6178,  24, { 11355,  11414,  11448,  11354,  11361,  11420,  11450,  11360}},
 {  6179,  24, { 11414,  11415,  11449,  11448,  11420,  11421,  11451,  11450}},
 {  6180,  24, { 11361,  11420,  11450,  11360,  11367,  11426,  11452,  11366}},
 {  6181,  24, { 11420,  11421,  11451,  11450,  11426,  11427,  11453,  11452}},
 {  6182,  24, { 11367,  11426,  11452,  11366,  11373,  11432,  11438,  11372}},
 {  6183,  24, { 11426,  11427,  11453,  11452,  11432,  11433,  11439,  11438}},
 {  6184,  24, { 11375,  11454,  11456,  11377,  11381,  11460,  11462,  11383}},
 {  6185,  24, { 11454,  11455,  11457,  11456,  11460,  11461,  11463,  11462}},
 {  6186,  24, { 11377,  11456,  11458,  11379,  11383,  11462,  11464,  11385}},
 {  6187,  24, { 11456,  11457,  11459,  11458,  11462,  11463,  11465,  11464}},
 {  6188,  24, { 11381,  11460,  11462,  11383,  11387,  11466,  11468,  11389}},
 {  6189,  24, { 11460,  11461,  11463,  11462,  11466,  11467,  11469,  11468}},
 {  6190,  24, { 11383,  11462,  11464,  11385,  11389,  11468,  11470,  11391}},
 {  6191,  24, { 11462,  11463,  11465,  11464,  11468,  11469,  11471,  11470}},
 {  6192,  24, { 11387,  11466,  11468,  11389,  11393,  11472,  11474,  11395}},
 {  6193,  24, { 11466,  11467,  11469,  11468,  11472,  11473,  11475,  11474}},
 {  6194,  24, { 11389,  11468,  11470,  11391,  11395,  11474,  11476,  11397}},
 {  6195,  24, { 11468,  11469,  11471,  11470,  11474,  11475,  11477,  11476}},
 {  6196,  24, { 11393,  11472,  11474,  11395,  11399,  11478,  11480,  11401}},
 {  6197,  24, { 11472,  11473,  11475,  11474,  11478,  11479,  11481,  11480}},
 {  6198,  24, { 11395,  11474,  11476,  11397,  11401,  11480,  11482,  11403}},
 {  6199,  24, { 11474,  11475,  11477,  11476,  11480,  11481,  11483,  11482}},
 {  6200,  24, { 11399,  11478,  11480,  11401,  11405,  11484,  11486,  11407}},
 {  6201,  24, { 11478,  11479,  11481,  11480,  11484,  11485,  11487,  11486}},
 {  6202,  24, { 11401,  11480,  11482,  11403,  11407,  11486,  11488,  11409}},
 {  6203,  24, { 11480,  11481,  11483,  11482,  11486,  11487,  11489,  11488}},
 {  6204,  24, { 11405,  11484,  11486,  11407,  11411,  11490,  11492,  11413}},
 {  6205,  24, { 11484,  11485,  11487,  11486,  11490,  11491,  11493,  11492}},
 {  6206,  24, { 11407,  11486,  11488,  11409,  11413,  11492,  11494,  11415}},
 {  6207,  24, { 11486,  11487,  11489,  11488,  11492,  11493,  11495,  11494}},
 {  6208,  24, { 11411,  11490,  11492,  11413,  11417,  11496,  11498,  11419}},
 {  6209,  24, { 11490,  11491,  11493,  11492,  11496,  11497,  11499,  11498}},
 {  6210,  24, { 11413,  11492,  11494,  11415,  11419,  11498,  11500,  11421}},
 {  6211,  24, { 11492,  11493,  11495,  11494,  11498,  11499,  11501,  11500}},
 {  6212,  24, { 11417,  11496,  11498,  11419,  11423,  11502,  11504,  11425}},
 {  6213,  24, { 11496,  11497,  11499,  11498,  11502,  11503,  11505,  11504}},
 {  6214,  24, { 11419,  11498,  11500,  11421,  11425,  11504,  11506,  11427}},
 {  6215,  24, { 11498,  11499,  11501,  11500,  11504,  11505,  11507,  11506}},
 {  6216,  24, { 11423,  11502,  11504,  11425,  11429,  11508,  11510,  11431}},
 {  6217,  24, { 11502,  11503,  11505,  11504,  11508,  11509,  11511,  11510}},
 {  6218,  24, { 11425,  11504,  11506,  11427,  11431,  11510,  11512,  11433}},
 {  6219,  24, { 11504,  11505,  11507,  11506,  11510,  11511,  11513,  11512}},
 {  6220,  24, { 11429,  11508,  11510,  11431,  11435,  11514,  11516,  11437}},
 {  6221,  24, { 11508,  11509,  11511,  11510,  11514,  11515,  11517,  11516}},
 {  6222,  24, { 11431,  11510,  11512,  11433,  11437,  11516,  11518,  11439}},
 {  6223,  24, { 11510,  11511,  11513,  11512,  11516,  11517,  11519,  11518}},
 {  6224,  24, { 11385,  11464,  11458,  11379,  11391,  11470,  11520,  11441}},
 {  6225,  24, { 11464,  11465,  11459,  11458,  11470,  11471,  11521,  11520}},
 {  6226,  24, { 11391,  11470,  11520,  11441,  11397,  11476,  11522,  11443}},
 {  6227,  24, { 11470,  11471,  11521,  11520,  11476,  11477,  11523,  11522}},
 {  6228,  24, { 11397,  11476,  11522,  11443,  11403,  11482,  11524,  11445}},
 {  6229,  24, { 11476,  11477,  11523,  11522,  11482,  11483,  11525,  11524}},
 {  6230,  24, { 11403,  11482,  11524,  11445,  11409,  11488,  11526,  11447}},
 {  6231,  24, { 11482,  11483,  11525,  11524,  11488,  11489,  11527,  11526}},
 {  6232,  24, { 11409,  11488,  11526,  11447,  11415,  11494,  11528,  11449}},
 {  6233,  24, { 11488,  11489,  11527,  11526,  11494,  11495,  11529,  11528}},
 {  6234,  24, { 11415,  11494,  11528,  11449,  11421,  11500,  11530,  11451}},
 {  6235,  24, { 11494,  11495,  11529,  11528,  11500,  11501,  11531,  11530}},
 {  6236,  24, { 11421,  11500,  11530,  11451,  11427,  11506,  11532,  11453}},
 {  6237,  24, { 11500,  11501,  11531,  11530,  11506,  11507,  11533,  11532}},
 {  6238,  24, { 11427,  11506,  11532,  11453,  11433,  11512,  11518,  11439}},
 {  6239,  24, { 11506,  11507,  11533,  11532,  11512,  11513,  11519,  11518}},
 {  6240,  24, { 11461,  11455,  11457,  11463,  11467,  11534,  11535,  11469}},
 {  6241,  24, { 11463,  11457,  11459,  11465,  11469,  11535,  11521,  11471}},
 {  6242,  24, { 11467,  11534,  11535,  11469,  11473,  11536,  11537,  11475}},
 {  6243,  24, { 11469,  11535,  11521,  11471,  11475,  11537,  11523,  11477}},
 {  6244,  24, { 11473,  11536,  11537,  11475,  11479,  11538,  11539,  11481}},
 {  6245,  24, { 11475,  11537,  11523,  11477,  11481,  11539,  11525,  11483}},
 {  6246,  24, { 11479,  11538,  11539,  11481,  11485,  11540,  11541,  11487}},
 {  6247,  24, { 11481,  11539,  11525,  11483,  11487,  11541,  11527,  11489}},
 {  6248,  24, { 11485,  11540,  11541,  11487,  11491,  11542,  11543,  11493}},
 {  6249,  24, { 11487,  11541,  11527,  11489,  11493,  11543,  11529,  11495}},
 {  6250,  24, { 11491,  11542,  11543,  11493,  11497,  11544,  11545,  11499}},
 {  6251,  24, { 11493,  11543,  11529,  11495,  11499,  11545,  11531,  11501}},
 {  6252,  24, { 11497,  11544,  11545,  11499,  11503,  11546,  11547,  11505}},
 {  6253,  24, { 11499,  11545,  11531,  11501,  11505,  11547,  11533,  11507}},
 {  6254,  24, { 11503,  11546,  11547,  11505,  11509,  11515,  11517,  11511}},
 {  6255,  24, { 11505,  11547,  11533,  11507,  11511,  11517,  11519,  11513}},
 {  6256,  23, { 11548,  11549,  11552,  11551,  11557,  11558,  11561,  11560}},
 {  6257,  23, { 11549,  11550,  11553,  11552,  11558,  11559,  11562,  11561}},
 {  6258,  23, { 11551,  11552,  11555,  11554,  11560,  11561,  11564,  11563}},
 {  6259,  23, { 11552,  11553,  11556,  11555,  11561,  11562,  11565,  11564}},
 {  6260,  23, { 11557,  11558,  11561,  11560,  11566,  11567,  11570,  11569}},
 {  6261,  23, { 11558,  11559,  11562,  11561,  11567,  11568,  11571,  11570}},
 {  6262,  23, { 11560,  11561,  11564,  11563,  11569,  11570,  11573,  11572}},
 {  6263,  23, { 11561,  11562,  11565,  11564,  11570,  11571,  11574,  11573}},
 {  6264,  23, { 11566,  11567,  11570,  11569,  11575,  11576,  11579,  11578}},
 {  6265,  23, { 11567,  11568,  11571,  11570,  11576,  11577,  11580,  11579}},
 {  6266,  23, { 11569,  11570,  11573,  11572,  11578,  11579,  11582,  11581}},
 {  6267,  23, { 11570,  11571,  11574,  11573,  11579,  11580,  11583,  11582}},
 {  6268,  23, { 11575,  11576,  11579,  11578,  11584,  11585,  11588,  11587}},
 {  6269,  23, { 11576,  11577,  11580,  11579,  11585,  11586,  11589,  11588}},
 {  6270,  23, { 11578,  11579,  11582,  11581,  11587,  11588,  11591,  11590}},
 {  6271,  23, { 11579,  11580,  11583,  11582,  11588,  11589,  11592,  11591}},
 {  6272,  23, { 11584,  11585,  11588,  11587,  11593,  11594,  11597,  11596}},
 {  6273,  23, { 11585,  11586,  11589,  11588,  11594,  11595,  11598,  11597}},
 {  6274,  23, { 11587,  11588,  11591,  11590,  11596,  11597,  11600,  11599}},
 {  6275,  23, { 11588,  11589,  11592,  11591,  11597,  11598,  11601,  11600}},
 {  6276,  23, { 11593,  11594,  11597,  11596,  11602,  11603,  11606,  11605}},
 {  6277,  23, { 11594,  11595,  11598,  11597,  11603,  11604,  11607,  11606}},
 {  6278,  23, { 11596,  11597,  11600,  11599,  11605,  11606,  11609,  11608}},
 {  6279,  23, { 11597,  11598,  11601,  11600,  11606,  11607,  11610,  11609}},
 {  6280,  23, { 11602,  11603,  11606,  11605,  11611,  11612,  11615,  11614}},
 {  6281,  23, { 11603,  11604,  11607,  11606,  11612,  11613,  11616,  11615}},
 {  6282,  23, { 11605,  11606,  11609,  11608,  11614,  11615,  11618,  11617}},
 {  6283,  23, { 11606,  11607,  11610,  11609,  11615,  11616,  11619,  11618}},
 {  6284,  23, { 11611,  11612,  11615,  11614,  11620,  11621,  11624,  11623}},
 {  6285,  23, { 11612,  11613,  11616,  11615,  11621,  11622,  11625,  11624}},
 {  6286,  23, { 11614,  11615,  11618,  11617,  11623,  11624,  11627,  11626}},
 {  6287,  23, { 11615,  11616,  11619,  11618,  11624,  11625,  11628,  11627}},
 {  6288,  23, { 11620,  11621,  11624,  11623,  11629,  11630,  11633,  11632}},
 {  6289,  23, { 11621,  11622,  11625,  11624,  11630,  11631,  11634,  11633}},
 {  6290,  23, { 11623,  11624,  11627,  11626,  11632,  11633,  11636,  11635}},
 {  6291,  23, { 11624,  11625,  11628,  11627,  11633,  11634,  11637,  11636}},
 {  6292,  23, { 11629,  11630,  11633,  11632,  11638,  11639,  11642,  11641}},
 {  6293,  23, { 11630,  11631,  11634,  11633,  11639,  11640,  11643,  11642}},
 {  6294,  23, { 11632,  11633,  11636,  11635,  11641,  11642,  11645,  11644}},
 {  6295,  23, { 11633,  11634,  11637,  11636,  11642,  11643,  11646,  11645}},
 {  6296,  23, { 11638,  11639,  11642,  11641,  11647,  11648,  11651,  11650}},
 {  6297,  23, { 11639,  11640,  11643,  11642,  11648,  11649,  11652,  11651}},
 {  6298,  23, { 11641,  11642,  11645,  11644,  11650,  11651,  11654,  11653}},
 {  6299,  23, { 11642,  11643,  11646,  11645,  11651,  11652,  11655,  11654}},
 {  6300,  23, { 11647,  11648,  11651,  11650,  11656,  11657,  11660,  11659}},
 {  6301,  23, { 11648,  11649,  11652,  11651,  11657,  11658,  11661,  11660}},
 {  6302,  23, { 11650,  11651,  11654,  11653,  11659,  11660,  11663,  11662}},
 {  6303,  23, { 11651,  11652,  11655,  11654,  11660,  11661,  11664,  11663}},
 {  6304,  23, { 11656,  11657,  11660,  11659,  11665,  11666,  11669,  11668}},
 {  6305,  23, { 11657,  11658,  11661,  11660,  11666,  11667,  11670,  11669}},
 {  6306,  23, { 11659,  11660,  11663,  11662,  11668,  11669,  11672,  11671}},
 {  6307,  23, { 11660,  11661,  11664,  11663,  11669,  11670,  11673,  11672}},
 {  6308,  23, { 11548,  11674,  11676,  11551,  11549,  11680,  11682,  11552}},
 {  6309,  23, { 11674,  11675,  11677,  11676,  11680,  11681,  11683,  11682}},
 {  6310,  23, { 11551,  11676,  11678,  11554,  11552,  11682,  11684,  11555}},
 {  6311,  23, { 11676,  11677,  11679,  11678,  11682,  11683,  11685,  11684}},
 {  6312,  23, { 11549,  11680,  11682,  11552,  11550,  11686,  11688,  11553}},
 {  6313,  23, { 11680,  11681,  11683,  11682,  11686,  11687,  11689,  11688}},
 {  6314,  23, { 11552,  11682,  11684,  11555,  11553,  11688,  11690,  11556}},
 {  6315,  23, { 11682,  11683,  11685,  11684,  11688,  11689,  11691,  11690}},
 {  6316,  23, { 11550,  11686,  11688,  11553,  11559,  11692,  11694,  11562}},
 {  6317,  23, { 11686,  11687,  11689,  11688,  11692,  11693,  11695,  11694}},
 {  6318,  23, { 11553,  11688,  11690,  11556,  11562,  11694,  11696,  11565}},
 {  6319,  23, { 11688,  11689,  11691,  11690,  11694,  11695,  11697,  11696}},
 {  6320,  23, { 11559,  11692,  11694,  11562,  11568,  11698,  11700,  11571}},
 {  6321,  23, { 11692,  11693,  11695,  11694,  11698,  11699,  11701,  11700}},
 {  6322,  23, { 11562,  11694,  11696,  11565,  11571,  11700,  11702,  11574}},
 {  6323,  23, { 11694,  11695,  11697,  11696,  11700,  11701,  11703,  11702}},
 {  6324,  23, { 11568,  11698,  11700,  11571,  11577,  11704,  11706,  11580}},
 {  6325,  23, { 11698,  11699,  11701,  11700,  11704,  11705,  11707,  11706}},
 {  6326,  23, { 11571,  11700,  11702,  11574,  11580,  11706,  11708,  11583}},
 {  6327,  23, { 11700,  11701,  11703,  11702,  11706,  11707,  11709,  11708}},
 {  6328,  23, { 11577,  11704,  11706,  11580,  11586,  11710,  11712,  11589}},
 {  6329,  23, { 11704,  11705,  11707,  11706,  11710,  11711,  11713,  11712}},
 {  6330,  23, { 11580,  11706,  11708,  11583,  11589,  11712,  11714,  11592}},
 {  6331,  23, { 11706,  11707,  11709,  11708,  11712,  11713,  11715,  11714}},
 {  6332,  23, { 11586,  11710,  11712,  11589,  11595,  11716,  11718,  11598}},
 {  6333,  23, { 11710,  11711,  11713,  11712,  11716,  11717,  11719,  11718}},
 {  6334,  23, { 11589,  11712,  11714,  11592,  11598,  11718,  11720,  11601}},
 {  6335,  23, { 11712,  11713,  11715,  11714,  11718,  11719,  11721,  11720}},
 {  6336,  23, { 11595,  11716,  11718,  11598,  11604,  11722,  11724,  11607}},
 {  6337,  23, { 11716,  11717,  11719,  11718,  11722,  11723,  11725,  11724}},
 {  6338,  23, { 11598,  11718,  11720,  11601,  11607,  11724,  11726,  11610}},
 {  6339,  23, { 11718,  11719,  11721,  11720,  11724,  11725,  11727,  11726}},
 {  6340,  23, { 11604,  11722,  11724,  11607,  11613,  11728,  11730,  11616}},
 {  6341,  23, { 11722,  11723,  11725,  11724,  11728,  11729,  11731,  11730}},
 {  6342,  23, { 11607,  11724,  11726,  11610,  11616,  11730,  11732,  11619}},
 {  6343,  23, { 11724,  11725,  11727,  11726,  11730,  11731,  11733,  11732}},
 {  6344,  23, { 11613,  11728,  11730,  11616,  11622,  11734,  11736,  11625}},
 {  6345,  23, { 11728,  11729,  11731,  11730,  11734,  11735,  11737,  11736}},
 {  6346,  23, { 11616,  11730,  11732,  11619,  11625,  11736,  11738,  11628}},
 {  6347,  23, { 11730,  11731,  11733,  11732,  11736,  11737,  11739,  11738}},
 {  6348,  23, { 11622,  11734,  11736,  11625,  11631,  11740,  11742,  11634}},
 {  6349,  23, { 11734,  11735,  11737,  11736,  11740,  11741,  11743,  11742}},
 {  6350,  23, { 11625,  11736,  11738,  11628,  11634,  11742,  11744,  11637}},
 {  6351,  23, { 11736,  11737,  11739,  11738,  11742,  11743,  11745,  11744}},
 {  6352,  23, { 11631,  11740,  11742,  11634,  11640,  11746,  11748,  11643}},
 {  6353,  23, { 11740,  11741,  11743,  11742,  11746,  11747,  11749,  11748}},
 {  6354,  23, { 11634,  11742,  11744,  11637,  11643,  11748,  11750,  11646}},
 {  6355,  23, { 11742,  11743,  11745,  11744,  11748,  11749,  11751,  11750}},
 {  6356,  23, { 11640,  11746,  11748,  11643,  11649,  11752,  11754,  11652}},
 {  6357,  23, { 11746,  11747,  11749,  11748,  11752,  11753,  11755,  11754}},
 {  6358,  23, { 11643,  11748,  11750,  11646,  11652,  11754,  11756,  11655}},
 {  6359,  23, { 11748,  11749,  11751,  11750,  11754,  11755,  11757,  11756}},
 {  6360,  23, { 11649,  11752,  11754,  11652,  11658,  11758,  11760,  11661}},
 {  6361,  23, { 11752,  11753,  11755,  11754,  11758,  11759,  11761,  11760}},
 {  6362,  23, { 11652,  11754,  11756,  11655,  11661,  11760,  11762,  11664}},
 {  6363,  23, { 11754,  11755,  11757,  11756,  11760,  11761,  11763,  11762}},
 {  6364,  23, { 11658,  11758,  11760,  11661,  11667,  11764,  11766,  11670}},
 {  6365,  23, { 11758,  11759,  11761,  11760,  11764,  11765,  11767,  11766}},
 {  6366,  23, { 11661,  11760,  11762,  11664,  11670,  11766,  11768,  11673}},
 {  6367,  23, { 11760,  11761,  11763,  11762,  11766,  11767,  11769,  11768}},
 {  6368,  23, { 11667,  11764,  11766,  11670,  11666,  11770,  11772,  11669}},
 {  6369,  23, { 11764,  11765,  11767,  11766,  11770,  11771,  11773,  11772}},
 {  6370,  23, { 11670,  11766,  11768,  11673,  11669,  11772,  11774,  11672}},
 {  6371,  23, { 11766,  11767,  11769,  11768,  11772,  11773,  11775,  11774}},
 {  6372,  23, { 11666,  11770,  11772,  11669,  11665,  11776,  11778,  11668}},
 {  6373,  23, { 11770,  11771,  11773,  11772,  11776,  11777,  11779,  11778}},
 {  6374,  23, { 11669,  11772,  11774,  11672,  11668,  11778,  11780,  11671}},
 {  6375,  23, { 11772,  11773,  11775,  11774,  11778,  11779,  11781,  11780}},
 {  6376,  23, { 11556,  11690,  11684,  11555,  11565,  11696,  11782,  11564}},
 {  6377,  23, { 11690,  11691,  11685,  11684,  11696,  11697,  11783,  11782}},
 {  6378,  23, { 11555,  11684,  11678,  11554,  11564,  11782,  11784,  11563}},
 {  6379,  23, { 11684,  11685,  11679,  11678,  11782,  11783,  11785,  11784}},
 {  6380,  23, { 11565,  11696,  11782,  11564,  11574,  11702,  11786,  11573}},
 {  6381,  23, { 11696,  11697,  11783,  11782,  11702,  11703,  11787,  11786}},
 {  6382,  23, { 11564,  11782,  11784,  11563,  11573,  11786,  11788,  11572}},
 {  6383,  23, { 11782,  11783,  11785,  11784,  11786,  11787,  11789,  11788}},
 {  6384,  23, { 11574,  11702,  11786,  11573,  11583,  11708,  11790,  11582}},
 {  6385,  23, { 11702,  11703,  11787,  11786,  11708,  11709,  11791,  11790}},
 {  6386,  23, { 11573,  11786,  11788,  11572,  11582,  11790,  11792,  11581}},
 {  6387,  23, { 11786,  11787,  11789,  11788,  11790,  11791,  11793,  11792}},
 {  6388,  23, { 11583,  11708,  11790,  11582,  11592,  11714,  11794,  11591}},
 {  6389,  23, { 11708,  11709,  11791,  11790,  11714,  11715,  11795,  11794}},
 {  6390,  23, { 11582,  11790,  11792,  11581,  11591,  11794,  11796,  11590}},
 {  6391,  23, { 11790,  11791,  11793,  11792,  11794,  11795,  11797,  11796}},
 {  6392,  23, { 11592,  11714,  11794,  11591,  11601,  11720,  11798,  11600}},
 {  6393,  23, { 11714,  11715,  11795,  11794,  11720,  11721,  11799,  11798}},
 {  6394,  23, { 11591,  11794,  11796,  11590,  11600,  11798,  11800,  11599}},
 {  6395,  23, { 11794,  11795,  11797,  11796,  11798,  11799,  11801,  11800}},
 {  6396,  23, { 11601,  11720,  11798,  11600,  11610,  11726,  11802,  11609}},
 {  6397,  23, { 11720,  11721,  11799,  11798,  11726,  11727,  11803,  11802}},
 {  6398,  23, { 11600,  11798,  11800,  11599,  11609,  11802,  11804,  11608}},
 {  6399,  23, { 11798,  11799,  11801,  11800,  11802,  11803,  11805,  11804}},
 {  6400,  23, { 11610,  11726,  11802,  11609,  11619,  11732,  11806,  11618}},
 {  6401,  23, { 11726,  11727,  11803,  11802,  11732,  11733,  11807,  11806}},
 {  6402,  23, { 11609,  11802,  11804,  11608,  11618,  11806,  11808,  11617}},
 {  6403,  23, { 11802,  11803,  11805,  11804,  11806,  11807,  11809,  11808}},
 {  6404,  23, { 11619,  11732,  11806,  11618,  11628,  11738,  11810,  11627}},
 {  6405,  23, { 11732,  11733,  11807,  11806,  11738,  11739,  11811,  11810}},
 {  6406,  23, { 11618,  11806,  11808,  11617,  11627,  11810,  11812,  11626}},
 {  6407,  23, { 11806,  11807,  11809,  11808,  11810,  11811,  11813,  11812}},
 {  6408,  23, { 11628,  11738,  11810,  11627,  11637,  11744,  11814,  11636}},
 {  6409,  23, { 11738,  11739,  11811,  11810,  11744,  11745,  11815,  11814}},
 {  6410,  23, { 11627,  11810,  11812,  11626,  11636,  11814,  11816,  11635}},
 {  6411,  23, { 11810,  11811,  11813,  11812,  11814,  11815,  11817,  11816}},
 {  6412,  23, { 11637,  11744,  11814,  11636,  11646,  11750,  11818,  11645}},
 {  6413,  23, { 11744,  11745,  11815,  11814,  11750,  11751,  11819,  11818}},
 {  6414,  23, { 11636,  11814,  11816,  11635,  11645,  11818,  11820,  11644}},
 {  6415,  23, { 11814,  11815,  11817,  11816,  11818,  11819,  11821,  11820}},
 {  6416,  23, { 11646,  11750,  11818,  11645,  11655,  11756,  11822,  11654}},
 {  6417,  23, { 11750,  11751,  11819,  11818,  11756,  11757,  11823,  11822}},
 {  6418,  23, { 11645,  11818,  11820,  11644,  11654,  11822,  11824,  11653}},
 {  6419,  23, { 11818,  11819,  11821,  11820,  11822,  11823,  11825,  11824}},
 {  6420,  23, { 11655,  11756,  11822,  11654,  11664,  11762,  11826,  11663}},
 {  6421,  23, { 11756,  11757,  11823,  11822,  11762,  11763,  11827,  11826}},
 {  6422,  23, { 11654,  11822,  11824,  11653,  11663,  11826,  11828,  11662}},
 {  6423,  23, { 11822,  11823,  11825,  11824,  11826,  11827,  11829,  11828}},
 {  6424,  23, { 11664,  11762,  11826,  11663,  11673,  11768,  11774,  11672}},
 {  6425,  23, { 11762,  11763,  11827,  11826,  11768,  11769,  11775,  11774}},
 {  6426,  23, { 11663,  11826,  11828,  11662,  11672,  11774,  11780,  11671}},
 {  6427,  23, { 11826,  11827,  11829,  11828,  11774,  11775,  11781,  11780}},
 {  6428,  23, { 11675,  11830,  11832,  11677,  11681,  11836,  11838,  11683}},
 {  6429,  23, { 11830,  11831,  11833,  11832,  11836,  11837,  11839,  11838}},
 {  6430,  23, { 11677,  11832,  11834,  11679,  11683,  11838,  11840,  11685}},
 {  6431,  23, { 11832,  11833,  11835,  11834,  11838,  11839,  11841,  11840}},
 {  6432,  23, { 11681,  11836,  11838,  11683,  11687,  11842,  11844,  11689}},
 {  6433,  23, { 11836,  11837,  11839,  11838,  11842,  11843,  11845,  11844}},
 {  6434,  23, { 11683,  11838,  11840,  11685,  11689,  11844,  11846,  11691}},
 {  6435,  23, { 11838,  11839,  11841,  11840,  11844,  11845,  11847,  11846}},
 {  6436,  23, { 11687,  11842,  11844,  11689,  11693,  11848,  11850,  11695}},
 {  6437,  23, { 11842,  11843,  11845,  11844,  11848,  11849,  11851,  11850}},
 {  6438,  23, { 11689,  11844,  11846,  11691,  11695,  11850,  11852,  11697}},
 {  6439,  23, { 11844,  11845,  11847,  11846,  11850,  11851,  11853,  11852}},
 {  6440,  23, { 11693,  11848,  11850,  11695,  11699,  11854,  11856,  11701}},
 {  6441,  23, { 11848,  11849,  11851,  11850,  11854,  11855,  11857,  11856}},
 {  6442,  23, { 11695,  11850,  11852,  11697,  11701,  11856,  11858,  11703}},
 {  6443,  23, { 11850,  11851,  11853,  11852,  11856,  11857,  11859,  11858}},
 {  6444,  23, { 11699,  11854,  11856,  11701,  11705,  11860,  11862,  11707}},
 {  6445,  23, { 11854,  11855,  11857,  11856,  11860,  11861,  11863,  11862}},
 {  6446,  23, { 11701,  11856,  11858,  11703,  11707,  11862,  11864,  11709}},
 {  6447,  23, { 11856,  11857,  11859,  11858,  11862,  11863,  11865,  11864}},
 {  6448,  23, { 11705,  11860,  11862,  11707,  11711,  11866,  11868,  11713}},
 {  6449,  23, { 11860,  11861,  11863,  11862,  11866,  11867,  11869,  11868}},
 {  6450,  23, { 11707,  11862,  11864,  11709,  11713,  11868,  11870,  11715}},
 {  6451,  23, { 11862,  11863,  11865,  11864,  11868,  11869,  11871,  11870}},
 {  6452,  23, { 11711,  11866,  11868,  11713,  11717,  11872,  11874,  11719}},
 {  6453,  23, { 11866,  11867,  11869,  11868,  11872,  11873,  11875,  11874}},
 {  6454,  23, { 11713,  11868,  11870,  11715,  11719,  11874,  11876,  11721}},
 {  6455,  23, { 11868,  11869,  11871,  11870,  11874,  11875,  11877,  11876}},
 {  6456,  23, { 11717,  11872,  11874,  11719,  11723,  11878,  11880,  11725}},
 {  6457,  23, { 11872,  11873,  11875,  11874,  11878,  11879,  11881,  11880}},
 {  6458,  23, { 11719,  11874,  11876,  11721,  11725,  11880,  11882,  11727}},
 {  6459,  23, { 11874,  11875,  11877,  11876,  11880,  11881,  11883,  11882}},
 {  6460,  23, { 11723,  11878,  11880,  11725,  11729,  11884,  11886,  11731}},
 {  6461,  23, { 11878,  11879,  11881,  11880,  11884,  11885,  11887,  11886}},
 {  6462,  23, { 11725,  11880,  11882,  11727,  11731,  11886,  11888,  11733}},
 {  6463,  23, { 11880,  11881,  11883,  11882,  11886,  11887,  11889,  11888}},
 {  6464,  23, { 11729,  11884,  11886,  11731,  11735,  11890,  11892,  11737}},
 {  6465,  23, { 11884,  11885,  11887,  11886,  11890,  11891,  11893,  11892}},
 {  6466,  23, { 11731,  11886,  11888,  11733,  11737,  11892,  11894,  11739}},
 {  6467,  23, { 11886,  11887,  11889,  11888,  11892,  11893,  11895,  11894}},
 {  6468,  23, { 11735,  11890,  11892,  11737,  11741,  11896,  11898,  11743}},
 {  6469,  23, { 11890,  11891,  11893,  11892,  11896,  11897,  11899,  11898}},
 {  6470,  23, { 11737,  11892,  11894,  11739,  11743,  11898,  11900,  11745}},
 {  6471,  23, { 11892,  11893,  11895,  11894,  11898,  11899,  11901,  11900}},
 {  6472,  23, { 11741,  11896,  11898,  11743,  11747,  11902,  11904,  11749}},
 {  6473,  23, { 11896,  11897,  11899,  11898,  11902,  11903,  11905,  11904}},
 {  6474,  23, { 11743,  11898,  11900,  11745,  11749,  11904,  11906,  11751}},
 {  6475,  23, { 11898,  11899,  11901,  11900,  11904,  11905,  11907,  11906}},
 {  6476,  23, { 11747,  11902,  11904,  11749,  11753,  11908,  11910,  11755}},
 {  6477,  23, { 11902,  11903,  11905,  11904,  11908,  11909,  11911,  11910}},
 {  6478,  23, { 11749,  11904,  11906,  11751,  11755,  11910,  11912,  11757}},
 {  6479,  23, { 11904,  11905,  11907,  11906,  11910,  11911,  11913,  11912}},
 {  6480,  23, { 11753,  11908,  11910,  11755,  11759,  11914,  11916,  11761}},
 {  6481,  23, { 11908,  11909,  11911,  11910,  11914,  11915,  11917,  11916}},
 {  6482,  23, { 11755,  11910,  11912,  11757,  11761,  11916,  11918,  11763}},
 {  6483,  23, { 11910,  11911,  11913,  11912,  11916,  11917,  11919,  11918}},
 {  6484,  23, { 11759,  11914,  11916,  11761,  11765,  11920,  11922,  11767}},
 {  6485,  23, { 11914,  11915,  11917,  11916,  11920,  11921,  11923,  11922}},
 {  6486,  23, { 11761,  11916,  11918,  11763,  11767,  11922,  11924,  11769}},
 {  6487,  23, { 11916,  11917,  11919,  11918,  11922,  11923,  11925,  11924}},
 {  6488,  23, { 11765,  11920,  11922,  11767,  11771,  11926,  11928,  11773}},
 {  6489,  23, { 11920,  11921,  11923,  11922,  11926,  11927,  11929,  11928}},
 {  6490,  23, { 11767,  11922,  11924,  11769,  11773,  11928,  11930,  11775}},
 {  6491,  23, { 11922,  11923,  11925,  11924,  11928,  11929,  11931,  11930}},
 {  6492,  23, { 11771,  11926,  11928,  11773,  11777,  11932,  11934,  11779}},
 {  6493,  23, { 11926,  11927,  11929,  11928,  11932,  11933,  11935,  11934}},
 {  6494,  23, { 11773,  11928,  11930,  11775,  11779,  11934,  11936,  11781}},
 {  6495,  23, { 11928,  11929,  11931,  11930,  11934,  11935,  11937,  11936}},
 {  6496,  23, { 11691,  11846,  11840,  11685,  11697,  11852,  11938,  11783}},
 {  6497,  23, { 11846,  11847,  11841,  11840,  11852,  11853,  11939,  11938}},
 {  6498,  23, { 11685,  11840,  11834,  11679,  11783,  11938,  11940,  11785}},
 {  6499,  23, { 11840,  11841,  11835,  11834,  11938,  11939,  11941,  11940}},
 {  6500,  23, { 11697,  11852,  11938,  11783,  11703,  11858,  11942,  11787}},
 {  6501,  23, { 11852,  11853,  11939,  11938,  11858,  11859,  11943,  11942}},
 {  6502,  23, { 11783,  11938,  11940,  11785,  11787,  11942,  11944,  11789}},
 {  6503,  23, { 11938,  11939,  11941,  11940,  11942,  11943,  11945,  11944}},
 {  6504,  23, { 11703,  11858,  11942,  11787,  11709,  11864,  11946,  11791}},
 {  6505,  23, { 11858,  11859,  11943,  11942,  11864,  11865,  11947,  11946}},
 {  6506,  23, { 11787,  11942,  11944,  11789,  11791,  11946,  11948,  11793}},
 {  6507,  23, { 11942,  11943,  11945,  11944,  11946,  11947,  11949,  11948}},
 {  6508,  23, { 11709,  11864,  11946,  11791,  11715,  11870,  11950,  11795}},
 {  6509,  23, { 11864,  11865,  11947,  11946,  11870,  11871,  11951,  11950}},
 {  6510,  23, { 11791,  11946,  11948,  11793,  11795,  11950,  11952,  11797}},
 {  6511,  23, { 11946,  11947,  11949,  11948,  11950,  11951,  11953,  11952}},
 {  6512,  23, { 11715,  11870,  11950,  11795,  11721,  11876,  11954,  11799}},
 {  6513,  23, { 11870,  11871,  11951,  11950,  11876,  11877,  11955,  11954}},
 {  6514,  23, { 11795,  11950,  11952,  11797,  11799,  11954,  11956,  11801}},
 {  6515,  23, { 11950,  11951,  11953,  11952,  11954,  11955,  11957,  11956}},
 {  6516,  23, { 11721,  11876,  11954,  11799,  11727,  11882,  11958,  11803}},
 {  6517,  23, { 11876,  11877,  11955,  11954,  11882,  11883,  11959,  11958}},
 {  6518,  23, { 11799,  11954,  11956,  11801,  11803,  11958,  11960,  11805}},
 {  6519,  23, { 11954,  11955,  11957,  11956,  11958,  11959,  11961,  11960}},
 {  6520,  23, { 11727,  11882,  11958,  11803,  11733,  11888,  11962,  11807}},
 {  6521,  23, { 11882,  11883,  11959,  11958,  11888,  11889,  11963,  11962}},
 {  6522,  23, { 11803,  11958,  11960,  11805,  11807,  11962,  11964,  11809}},
 {  6523,  23, { 11958,  11959,  11961,  11960,  11962,  11963,  11965,  11964}},
 {  6524,  23, { 11733,  11888,  11962,  11807,  11739,  11894,  11966,  11811}},
 {  6525,  23, { 11888,  11889,  11963,  11962,  11894,  11895,  11967,  11966}},
 {  6526,  23, { 11807,  11962,  11964,  11809,  11811,  11966,  11968,  11813}},
 {  6527,  23, { 11962,  11963,  11965,  11964,  11966,  11967,  11969,  11968}},
 {  6528,  23, { 11739,  11894,  11966,  11811,  11745,  11900,  11970,  11815}},
 {  6529,  23, { 11894,  11895,  11967,  11966,  11900,  11901,  11971,  11970}},
 {  6530,  23, { 11811,  11966,  11968,  11813,  11815,  11970,  11972,  11817}},
 {  6531,  23, { 11966,  11967,  11969,  11968,  11970,  11971,  11973,  11972}},
 {  6532,  23, { 11745,  11900,  11970,  11815,  11751,  11906,  11974,  11819}},
 {  6533,  23, { 11900,  11901,  11971,  11970,  11906,  11907,  11975,  11974}},
 {  6534,  23, { 11815,  11970,  11972,  11817,  11819,  11974,  11976,  11821}},
 {  6535,  23, { 11970,  11971,  11973,  11972,  11974,  11975,  11977,  11976}},
 {  6536,  23, { 11751,  11906,  11974,  11819,  11757,  11912,  11978,  11823}},
 {  6537,  23, { 11906,  11907,  11975,  11974,  11912,  11913,  11979,  11978}},
 {  6538,  23, { 11819,  11974,  11976,  11821,  11823,  11978,  11980,  11825}},
 {  6539,  23, { 11974,  11975,  11977,  11976,  11978,  11979,  11981,  11980}},
 {  6540,  23, { 11757,  11912,  11978,  11823,  11763,  11918,  11982,  11827}},
 {  6541,  23, { 11912,  11913,  11979,  11978,  11918,  11919,  11983,  11982}},
 {  6542,  23, { 11823,  11978,  11980,  11825,  11827,  11982,  11984,  11829}},
 {  6543,  23, { 11978,  11979,  11981,  11980,  11982,  11983,  11985,  11984}},
 {  6544,  23, { 11763,  11918,  11982,  11827,  11769,  11924,  11930,  11775}},
 {  6545,  23, { 11918,  11919,  11983,  11982,  11924,  11925,  11931,  11930}},
 {  6546,  23, { 11827,  11982,  11984,  11829,  11775,  11930,  11936,  11781}},
 {  6547,  23, { 11982,  11983,  11985,  11984,  11930,  11931,  11937,  11936}},
 {  6548,  23, { 11843,  11837,  11839,  11845,  11849,  11986,  11988,  11851}},
 {  6549,  23, { 11837,  11831,  11833,  11839,  11986,  11987,  11989,  11988}},
 {  6550,  23, { 11845,  11839,  11841,  11847,  11851,  11988,  11939,  11853}},
 {  6551,  23, { 11839,  11833,  11835,  11841,  11988,  11989,  11941,  11939}},
 {  6552,  23, { 11849,  11986,  11988,  11851,  11855,  11990,  11992,  11857}},
 {  6553,  23, { 11986,  11987,  11989,  11988,  11990,  11991,  11993,  11992}},
 {  6554,  23, { 11851,  11988,  11939,  11853,  11857,  11992,  11943,  11859}},
 {  6555,  23, { 11988,  11989,  11941,  11939,  11992,  11993,  11945,  11943}},
 {  6556,  23, { 11855,  11990,  11992,  11857,  11861,  11994,  11996,  11863}},
 {  6557,  23, { 11990,  11991,  11993,  11992,  11994,  11995,  11997,  11996}},
 {  6558,  23, { 11857,  11992,  11943,  11859,  11863,  11996,  11947,  11865}},
 {  6559,  23, { 11992,  11993,  11945,  11943,  11996,  11997,  11949,  11947}},
 {  6560,  23, { 11861,  11994,  11996,  11863,  11867,  11998,  12000,  11869}},
 {  6561,  23, { 11994,  11995,  11997,  11996,  11998,  11999,  12001,  12000}},
 {  6562,  23, { 11863,  11996,  11947,  11865,  11869,  12000,  11951,  11871}},
 {  6563,  23, { 11996,  11997,  11949,  11947,  12000,  12001,  11953,  11951}},
 {  6564,  23, { 11867,  11998,  12000,  11869,  11873,  12002,  12004,  11875}},
 {  6565,  23, { 11998,  11999,  12001,  12000,  12002,  12003,  12005,  12004}},
 {  6566,  23, { 11869,  12000,  11951,  11871,  11875,  12004,  11955,  11877}},
 {  6567,  23, { 12000,  12001,  11953,  11951,  12004,  12005,  11957,  11955}},
 {  6568,  23, { 11873,  12002,  12004,  11875,  11879,  12006,  12008,  11881}},
 {  6569,  23, { 12002,  12003,  12005,  12004,  12006,  12007,  12009,  12008}},
 {  6570,  23, { 11875,  12004,  11955,  11877,  11881,  12008,  11959,  11883}},
 {  6571,  23, { 12004,  12005,  11957,  11955,  12008,  12009,  11961,  11959}},
 {  6572,  23, { 11879,  12006,  12008,  11881,  11885,  12010,  12012,  11887}},
 {  6573,  23, { 12006,  12007,  12009,  12008,  12010,  12011,  12013,  12012}},
 {  6574,  23, { 11881,  12008,  11959,  11883,  11887,  12012,  11963,  11889}},
 {  6575,  23, { 12008,  12009,  11961,  11959,  12012,  12013,  11965,  11963}},
 {  6576,  23, { 11885,  12010,  12012,  11887,  11891,  12014,  12016,  11893}},
 {  6577,  23, { 12010,  12011,  12013,  12012,  12014,  12015,  12017,  12016}},
 {  6578,  23, { 11887,  12012,  11963,  11889,  11893,  12016,  11967,  11895}},
 {  6579,  23, { 12012,  12013,  11965,  11963,  12016,  12017,  11969,  11967}},
 {  6580,  23, { 11891,  12014,  12016,  11893,  11897,  12018,  12020,  11899}},
 {  6581,  23, { 12014,  12015,  12017,  12016,  12018,  12019,  12021,  12020}},
 {  6582,  23, { 11893,  12016,  11967,  11895,  11899,  12020,  11971,  11901}},
 {  6583,  23, { 12016,  12017,  11969,  11967,  12020,  12021,  11973,  11971}},
 {  6584,  23, { 11897,  12018,  12020,  11899,  11903,  12022,  12024,  11905}},
 {  6585,  23, { 12018,  12019,  12021,  12020,  12022,  12023,  12025,  12024}},
 {  6586,  23, { 11899,  12020,  11971,  11901,  11905,  12024,  11975,  11907}},
 {  6587,  23, { 12020,  12021,  11973,  11971,  12024,  12025,  11977,  11975}},
 {  6588,  23, { 11903,  12022,  12024,  11905,  11909,  12026,  12028,  11911}},
 {  6589,  23, { 12022,  12023,  12025,  12024,  12026,  12027,  12029,  12028}},
 {  6590,  23, { 11905,  12024,  11975,  11907,  11911,  12028,  11979,  11913}},
 {  6591,  23, { 12024,  12025,  11977,  11975,  12028,  12029,  11981,  11979}},
 {  6592,  23, { 11909,  12026,  12028,  11911,  11915,  12030,  12032,  11917}},
 {  6593,  23, { 12026,  12027,  12029,  12028,  12030,  12031,  12033,  12032}},
 {  6594,  23, { 11911,  12028,  11979,  11913,  11917,  12032,  11983,  11919}},
 {  6595,  23, { 12028,  12029,  11981,  11979,  12032,  12033,  11985,  11983}},
 {  6596,  23, { 11915,  12030,  12032,  11917,  11921,  11927,  11929,  11923}},
 {  6597,  23, { 12030,  12031,  12033,  12032,  11927,  11933,  11935,  11929}},
 {  6598,  23, { 11917,  12032,  11983,  11919,  11923,  11929,  11931,  11925}},
 {  6599,  23, { 12032,  12033,  11985,  11983,  11929,  11935,  11937,  11931}},
 { 20434,  33, { 20000,  20001,  20451,  20452,  20002,  20002,  20453,  20453}},
 { 20435,  33, { 20004,  20003,  20006,  20005,  20007,  20454,  20455,  20008}},
 { 20436,  33, { 20010,  20009,  20012,  20011,  20014,  20013,  20456,  20457}},
 { 20437,  33, { 20003,  20015,  20016,  20006,  20454,  20458,  20459,  20455}},
 { 20438,  33, { 20018,  20017,  20020,  20019,  20021,  20022,  20453,  20002}},
 { 20439,  33, { 20007,  20454,  20455,  20008,  20023,  20460,  20461,  20024}},
 { 20440,  33, { 20014,  20013,  20456,  20457,  20026,  20025,  20462,  20463}},
 { 20441,  33, { 20454,  20458,  20459,  20455,  20460,  20464,  20465,  20461}},
 { 20442,  33, { 20455,  20459,  20466,  20456,  20461,  20465,  20467,  20462}},
 { 20443,  33, { 20456,  20466,  20468,  20457,  20462,  20467,  20469,  20463}},
 { 20444,  33, { 20453,  20454,  20460,  20451,  20470,  20458,  20464,  20471}},
 { 20445,  33, { 20452,  20472,  20027,  20000,  20453,  20453,  20002,  20002}},
 { 20446,  33, { 20473,  20474,  20475,  20476,  20477,  20478,  20479,  20480}},
 { 20447,  33, { 20481,  20482,  20483,  20484,  20485,  20479,  20480,  20486}},
 { 20448,  33, { 20485,  20479,  20480,  20486,  20487,  20475,  20476,  20488}},
 { 20449,  33, { 20472,  20452,  20000,  20027,  20489,  20490,  20029,  20028}},
 { 20450,  33, { 20460,  20464,  20465,  20461,  20488,  20487,  20485,  20486}},
 { 20451,  33, { 20461,  20465,  20467,  20462,  20486,  20485,  20481,  20484}},
 { 20452,  33, { 20001,  20000,  20452,  20451,  20030,  20029,  20490,  20491}},
 { 20453,  33, { 20489,  20490,  20029,  20028,  20492,  20493,  20032,  20031}},
 { 20454,  33, { 20494,  20477,  20478,  20495,  20496,  20473,  20474,  20497}},
 { 20455,  33, { 20033,  20498,  20499,  20036,  20034,  20037,  20038,  20035}},
 { 20456,  33, { 20036,  20499,  20500,  20040,  20035,  20038,  20041,  20039}},
 { 20457,  33, { 20039,  20040,  20043,  20042,  20041,  20500,  20501,  20044}},
 { 20458,  33, { 20042,  20043,  20046,  20045,  20044,  20501,  20502,  20047}},
 { 20459,  33, { 20045,  20046,  20049,  20048,  20047,  20502,  20051,  20050}},
 { 20460,  33, { 20049,  20046,  20053,  20052,  20051,  20502,  20503,  20054}},
 { 20461,  33, { 20052,  20053,  20056,  20055,  20054,  20503,  20504,  20057}},
 { 20462,  33, { 20055,  20056,  20059,  20058,  20057,  20504,  20505,  20060}},
 { 20463,  33, { 20058,  20060,  20505,  20059,  20061,  20063,  20506,  20062}},
 { 20464,  33, { 20062,  20061,  20063,  20506,  20066,  20065,  20064,  20507}},
 { 20465,  33, { 20066,  20065,  20064,  20507,  20069,  20068,  20067,  20508}},
 { 20466,  33, { 20069,  20068,  20067,  20508,  20072,  20071,  20070,  20509}},
 { 20467,  33, { 20072,  20071,  20070,  20509,  20075,  20074,  20073,  20510}},
 { 20468,  33, { 20075,  20074,  20073,  20510,  20078,  20077,  20076,  20511}},
 { 20469,  33, { 20077,  20076,  20511,  20078,  20079,  20081,  20512,  20080}},
 { 20470,  33, { 20079,  20081,  20512,  20080,  20082,  20084,  20513,  20083}},
 { 20471,  33, { 20514,  20513,  20512,  20515,  20085,  20084,  20081,  20086}},
 { 20472,  33, { 20516,  20511,  20510,  20517,  20087,  20076,  20073,  20088}},
 { 20473,  33, { 20515,  20512,  20511,  20516,  20086,  20081,  20076,  20087}},
 { 20474,  33, { 20517,  20510,  20509,  20518,  20088,  20073,  20070,  20089}},
 { 20475,  33, { 20518,  20509,  20508,  20519,  20089,  20070,  20067,  20090}},
 { 20476,  33, { 20519,  20508,  20507,  20520,  20090,  20067,  20064,  20091}},
 { 20477,  33, { 20520,  20507,  20506,  20521,  20091,  20064,  20063,  20092}},
 { 20478,  33, { 20522,  20523,  20500,  20499,  20093,  20094,  20041,  20038}},
 { 20479,  33, { 20524,  20522,  20499,  20498,  20095,  20093,  20038,  20037}},
 { 20480,  33, { 20041,  20500,  20501,  20044,  20094,  20523,  20468,  20096}},
 { 20481,  33, { 20044,  20501,  20502,  20047,  20096,  20468,  20457,  20011}},
 { 20482,  33, { 20047,  20502,  20051,  20050,  20011,  20457,  20014,  20010}},
 { 20483,  33, { 20502,  20503,  20054,  20051,  20457,  20463,  20026,  20014}},
 { 20484,  33, { 20503,  20504,  20057,  20054,  20463,  20525,  20097,  20026}},
 { 20485,  33, { 20060,  20505,  20506,  20063,  20098,  20526,  20521,  20092}},
 { 20486,  33, { 20057,  20504,  20505,  20060,  20097,  20525,  20526,  20098}},
 { 20487,  33, { 20086,  20515,  20514,  20085,  20099,  20527,  20528,  20100}},
 { 20488,  33, { 20087,  20516,  20515,  20086,  20101,  20529,  20527,  20099}},
 { 20489,  33, { 20088,  20517,  20516,  20087,  20102,  20530,  20529,  20101}},
 { 20490,  33, { 20089,  20518,  20517,  20088,  20103,  20531,  20530,  20102}},
 { 20491,  33, { 20090,  20519,  20518,  20089,  20104,  20532,  20531,  20103}},
 { 20492,  33, { 20091,  20520,  20519,  20090,  20105,  20533,  20532,  20104}},
 { 20493,  33, { 20091,  20092,  20521,  20520,  20105,  20106,  20534,  20533}},
 { 20494,  33, { 20535,  20482,  20536,  20537,  20526,  20483,  20534,  20521}},
 { 20495,  33, { 20092,  20106,  20107,  20098,  20521,  20534,  20483,  20526}},
 { 20496,  33, { 20482,  20483,  20526,  20535,  20481,  20484,  20525,  20538}},
 { 20497,  33, { 20483,  20107,  20098,  20526,  20484,  20108,  20097,  20525}},
 { 20498,  33, { 20538,  20525,  20463,  20469,  20481,  20484,  20462,  20467}},
 { 20499,  33, { 20097,  20525,  20484,  20108,  20026,  20463,  20462,  20025}},
 { 20500,  33, { 20095,  20093,  20110,  20109,  20524,  20522,  20539,  20540}},
 { 20501,  33, { 20093,  20094,  20111,  20110,  20522,  20523,  20541,  20539}},
 { 20502,  33, { 20094,  20096,  20112,  20111,  20523,  20468,  20466,  20541}},
 { 20503,  33, { 20109,  20110,  20114,  20113,  20540,  20539,  20542,  20543}},
 { 20504,  33, { 20543,  20544,  20545,  20542,  20113,  20115,  20116,  20114}},
 { 20505,  33, { 20117,  20546,  20547,  20120,  20118,  20548,  20549,  20119}},
 { 20506,  33, { 20118,  20548,  20549,  20119,  20121,  20550,  20551,  20122}},
 { 20507,  33, { 20121,  20550,  20551,  20122,  20123,  20552,  20553,  20124}},
 { 20508,  33, { 20123,  20552,  20553,  20124,  20125,  20554,  20555,  20126}},
 { 20509,  33, { 20128,  20127,  20099,  20100,  20556,  20557,  20527,  20528}},
 { 20510,  33, { 20127,  20129,  20101,  20099,  20557,  20558,  20529,  20527}},
 { 20511,  33, { 20107,  20130,  20131,  20108,  20483,  20480,  20486,  20484}},
 { 20512,  33, { 20108,  20131,  20024,  20025,  20484,  20486,  20461,  20462}},
 { 20513,  33, { 20013,  20009,  20012,  20456,  20008,  20005,  20006,  20455}},
 { 20514,  33, { 20559,  20459,  20016,  20132,  20560,  20458,  20015,  20133}},
 { 20515,  33, { 20539,  20541,  20111,  20110,  20542,  20559,  20132,  20114}},
 { 20516,  33, { 20545,  20560,  20133,  20116,  20561,  20562,  20135,  20134}},
 { 20517,  33, { 20560,  20458,  20015,  20133,  20562,  20470,  20136,  20135}},
 { 20518,  33, { 20458,  20454,  20003,  20015,  20470,  20453,  20020,  20136}},
 { 20519,  33, { 20007,  20004,  20003,  20454,  20002,  20019,  20020,  20453}},
 { 20520,  33, { 20464,  20471,  20563,  20487,  20460,  20451,  20491,  20488}},
 { 20521,  33, { 20490,  20489,  20472,  20452,  20564,  20565,  20566,  20567}},
 { 20522,  33, { 20472,  20452,  20567,  20566,  20453,  20453,  20470,  20470}},
 { 20523,  33, { 20136,  20020,  20017,  20137,  20470,  20453,  20022,  20138}},
 { 20524,  33, { 20567,  20470,  20562,  20568,  20566,  20566,  20569,  20569}},
 { 20525,  33, { 20140,  20139,  20142,  20141,  20565,  20566,  20472,  20489}},
 { 20526,  33, { 20143,  20139,  20140,  20144,  20569,  20566,  20565,  20570}},
 { 20527,  33, { 20120,  20143,  20144,  20119,  20547,  20569,  20570,  20549}},
 { 20528,  33, { 20122,  20119,  20144,  20145,  20551,  20549,  20570,  20571}},
 { 20529,  33, { 20466,  20459,  20455,  20456,  20112,  20016,  20006,  20012}},
 { 20530,  33, { 20027,  20028,  20147,  20146,  20472,  20489,  20141,  20142}},
 { 20531,  33, { 20572,  20573,  20563,  20564,  20493,  20574,  20491,  20490}},
 { 20532,  33, { 20574,  20476,  20475,  20573,  20491,  20488,  20487,  20563}},
 { 20533,  33, { 20144,  20140,  20148,  20145,  20570,  20565,  20575,  20571}},
 { 20534,  33, { 20151,  20032,  20493,  20574,  20150,  20149,  20576,  20577}},
 { 20535,  33, { 20153,  20152,  20155,  20154,  20578,  20579,  20157,  20156}},
 { 20536,  33, { 20153,  20154,  20159,  20158,  20578,  20156,  20160,  20580}},
 { 20537,  33, { 20161,  20150,  20163,  20162,  20581,  20581,  20582,  20582}},
 { 20538,  33, { 20164,  20166,  20149,  20165,  20583,  20583,  20584,  20584}},
 { 20539,  33, { 20585,  20579,  20152,  20167,  20582,  20581,  20161,  20162}},
 { 20540,  33, { 20152,  20579,  20578,  20153,  20161,  20581,  20584,  20165}},
 { 20541,  33, { 20153,  20578,  20580,  20158,  20165,  20584,  20583,  20164}},
 { 20542,  33, { 20581,  20150,  20577,  20586,  20584,  20149,  20576,  20587}},
 { 20543,  33, { 20492,  20493,  20032,  20031,  20588,  20576,  20149,  20166}},
 { 20544,  33, { 20169,  20168,  20171,  20170,  20589,  20590,  20591,  20592}},
 { 20545,  33, { 20170,  20171,  20173,  20172,  20592,  20591,  20593,  20594}},
 { 20546,  33, { 20174,  20169,  20170,  20175,  20595,  20589,  20592,  20596}},
 { 20547,  33, { 20175,  20170,  20172,  20176,  20596,  20592,  20594,  20597}},
 { 20548,  33, { 20178,  20177,  20180,  20179,  20158,  20159,  20160,  20580}},
 { 20549,  33, { 20179,  20180,  20181,  20174,  20580,  20160,  20182,  20595}},
 { 20550,  33, { 20174,  20181,  20183,  20169,  20595,  20182,  20184,  20589}},
 { 20551,  33, { 20169,  20183,  20185,  20168,  20589,  20184,  20186,  20590}},
 { 20552,  33, { 20168,  20185,  20188,  20187,  20590,  20186,  20189,  20598}},
 { 20553,  33, { 20190,  20191,  20166,  20164,  20192,  20192,  20583,  20583}},
 { 20554,  33, { 20179,  20580,  20595,  20174,  20192,  20583,  20596,  20175}},
 { 20555,  33, { 20584,  20149,  20576,  20587,  20583,  20166,  20588,  20597}},
 { 20556,  33, { 20194,  20193,  20196,  20195,  20585,  20198,  20197,  20167}},
 { 20557,  33, { 20197,  20155,  20152,  20167,  20198,  20157,  20579,  20585}},
 { 20558,  33, { 20572,  20599,  20575,  20600,  20564,  20564,  20565,  20565}},
 { 20559,  33, { 20600,  20575,  20148,  20199,  20565,  20565,  20140,  20140}},
 { 20560,  33, { 20565,  20600,  20492,  20489,  20140,  20199,  20200,  20141}},
 { 20561,  33, { 20201,  20476,  20480,  20130,  20163,  20473,  20477,  20202}},
 { 20562,  33, { 20574,  20476,  20201,  20151,  20577,  20473,  20163,  20150}},
 { 20563,  33, { 20187,  20203,  20125,  20126,  20598,  20224,  20554,  20555}},
 { 20564,  33, { 20557,  20556,  20128,  20127,  20601,  20601,  20602,  20602}},
 { 20565,  33, { 20556,  20603,  20213,  20128,  20601,  20601,  20602,  20602}},
 { 20566,  33, { 20213,  20602,  20601,  20603,  20212,  20604,  20605,  20606}},
 { 20567,  33, { 20212,  20604,  20605,  20606,  20224,  20598,  20555,  20554}},
 { 20568,  33, { 20204,  20194,  20585,  20607,  20193,  20193,  20198,  20198}},
 { 20569,  33, { 20205,  20129,  20207,  20206,  20608,  20609,  20209,  20208}},
 { 20570,  33, { 20210,  20214,  20602,  20213,  20211,  20215,  20604,  20212}},
 { 20571,  33, { 20127,  20128,  20217,  20216,  20602,  20602,  20214,  20214}},
 { 20572,  33, { 20128,  20213,  20210,  20217,  20602,  20602,  20214,  20214}},
 { 20573,  33, { 20218,  20219,  20610,  20611,  20205,  20205,  20608,  20608}},
 { 20574,  33, { 20205,  20206,  20220,  20219,  20608,  20208,  20221,  20610}},
 { 20575,  33, { 20610,  20607,  20198,  20221,  20219,  20204,  20193,  20220}},
 { 20576,  33, { 20612,  20222,  20218,  20611,  20497,  20495,  20613,  20614}},
 { 20577,  33, { 20582,  20163,  20473,  20496,  20581,  20150,  20577,  20586}},
 { 20578,  33, { 20530,  20102,  20103,  20531,  20615,  20205,  20218,  20613}},
 { 20579,  33, { 20575,  20148,  20145,  20571,  20594,  20172,  20173,  20593}},
 { 20580,  33, { 20571,  20145,  20122,  20551,  20593,  20173,  20124,  20553}},
 { 20581,  33, { 20103,  20218,  20222,  20104,  20531,  20613,  20495,  20532}},
 { 20582,  33, { 20491,  20574,  20151,  20030,  20490,  20493,  20032,  20029}},
 { 20583,  33, { 20454,  20460,  20023,  20007,  20453,  20451,  20001,  20002}},
 { 20584,  33, { 20223,  20189,  20215,  20211,  20224,  20598,  20604,  20212}},
 { 20585,  33, { 20102,  20530,  20529,  20101,  20205,  20615,  20558,  20129}},
 { 20586,  33, { 20127,  20216,  20207,  20129,  20602,  20214,  20209,  20609}},
 { 20587,  33, { 20189,  20223,  20225,  20188,  20598,  20224,  20203,  20187}},
 { 20588,  33, { 20031,  20227,  20200,  20492,  20166,  20191,  20226,  20588}},
 { 20589,  33, { 20031,  20492,  20200,  20227,  20028,  20489,  20141,  20147}},
 { 20590,  33, { 20176,  20172,  20148,  20199,  20597,  20594,  20575,  20600}},
 { 20591,  33, { 20226,  20176,  20199,  20200,  20588,  20597,  20600,  20492}},
 { 20592,  33, { 20564,  20572,  20493,  20490,  20565,  20600,  20492,  20489}},
 { 20593,  33, { 20171,  20126,  20124,  20173,  20591,  20555,  20553,  20593}},
 { 20594,  33, { 20165,  20149,  20150,  20161,  20584,  20584,  20581,  20581}},
 { 20595,  33, { 20582,  20163,  20202,  20228,  20496,  20473,  20477,  20494}},
 { 20596,  33, { 20191,  20192,  20176,  20226,  20166,  20583,  20597,  20588}},
 { 20597,  33, { 20583,  20596,  20175,  20192,  20597,  20597,  20176,  20176}},
 { 20598,  33, { 20477,  20478,  20479,  20480,  20534,  20536,  20482,  20483}},
 { 20599,  33, { 20104,  20222,  20228,  20105,  20532,  20495,  20494,  20533}},
 { 20600,  33, { 20228,  20202,  20106,  20105,  20494,  20477,  20534,  20533}},
 { 20601,  33, { 20534,  20536,  20532,  20533,  20477,  20478,  20495,  20494}},
 { 20602,  33, { 20534,  20106,  20107,  20483,  20477,  20202,  20130,  20480}},
 { 20603,  33, { 20480,  20130,  20131,  20486,  20476,  20201,  20229,  20488}},
 { 20604,  33, { 20488,  20460,  20023,  20229,  20486,  20461,  20024,  20131}},
 { 20605,  33, { 20112,  20016,  20132,  20111,  20466,  20459,  20559,  20541}},
 { 20606,  33, { 20559,  20132,  20114,  20542,  20560,  20133,  20116,  20545}},
 { 20607,  33, { 20011,  20096,  20468,  20457,  20012,  20112,  20466,  20456}},
 { 20608,  33, { 20201,  20151,  20030,  20229,  20476,  20574,  20491,  20488}},
 { 20609,  33, { 20229,  20030,  20001,  20023,  20488,  20491,  20451,  20460}},
 { 20610,  33, { 20218,  20219,  20204,  20222,  20611,  20610,  20607,  20612}},
 { 20611,  33, { 20222,  20204,  20194,  20228,  20612,  20607,  20585,  20582}},
 { 20612,  33, { 20495,  20222,  20228,  20494,  20497,  20612,  20582,  20496}},
 { 20613,  33, { 20025,  20024,  20008,  20013,  20462,  20461,  20455,  20456}},
 { 20614,  33, { 20598,  20187,  20168,  20590,  20555,  20126,  20171,  20591}},
 { 20615,  33, { 20194,  20585,  20167,  20195,  20228,  20582,  20162,  20230}},
 { 20616,  33, { 20230,  20228,  20582,  20162,  20202,  20202,  20163,  20163}},
 { 20617,  33, { 20218,  20613,  20615,  20205,  20611,  20614,  20616,  20608}},
 { 20618,  33, { 20205,  20615,  20558,  20129,  20608,  20616,  20617,  20609}},
 { 20619,  33, { 20129,  20558,  20557,  20127,  20609,  20617,  20601,  20602}},
 { 20620,  33, { 20164,  20190,  20192,  20583,  20158,  20178,  20179,  20580}},
 { 20621,  33, { 20451,  20453,  20470,  20471,  20452,  20452,  20567,  20567}},
 { 20622,  33, { 20451,  20471,  20563,  20491,  20452,  20567,  20564,  20490}},
 { 20623,  33, { 20137,  20231,  20135,  20136,  20138,  20232,  20562,  20470}},
 { 20624,  33, { 20231,  20233,  20134,  20135,  20232,  20234,  20561,  20562}},
 { 20625,  33, { 20618,  20546,  20547,  20561,  20236,  20236,  20234,  20234}},
 { 20626,  33, { 20561,  20547,  20569,  20562,  20234,  20234,  20232,  20232}},
 { 20627,  33, { 20562,  20569,  20566,  20470,  20232,  20232,  20138,  20138}},
 { 20628,  33, { 20470,  20566,  20472,  20453,  20138,  20138,  20022,  20022}},
 { 20629,  33, { 20566,  20139,  20142,  20472,  20138,  20138,  20022,  20022}},
 { 20630,  33, { 20453,  20472,  20027,  20002,  20022,  20022,  20021,  20021}},
 { 20631,  33, { 20472,  20142,  20146,  20027,  20022,  20022,  20021,  20021}},
 { 20632,  33, { 20546,  20117,  20120,  20547,  20236,  20236,  20234,  20234}},
 { 20633,  33, { 20547,  20120,  20143,  20569,  20234,  20234,  20232,  20232}},
 { 20634,  33, { 20569,  20143,  20139,  20566,  20232,  20232,  20138,  20138}},
 { 20635,  33, { 20237,  20238,  20619,  20620,  20239,  20239,  20621,  20621}},
 { 20636,  33, { 20241,  20240,  20243,  20242,  20245,  20244,  20622,  20623}},
 { 20637,  33, { 20247,  20246,  20249,  20248,  20250,  20624,  20625,  20251}},
 { 20638,  33, { 20242,  20243,  20253,  20252,  20623,  20622,  20626,  20627}},
 { 20639,  33, { 20255,  20254,  20257,  20256,  20258,  20237,  20620,  20259}},
 { 20640,  33, { 20245,  20244,  20622,  20623,  20261,  20260,  20628,  20629}},
 { 20641,  33, { 20250,  20624,  20625,  20251,  20262,  20630,  20631,  20263}},
 { 20642,  33, { 20623,  20622,  20626,  20627,  20629,  20628,  20632,  20633}},
 { 20643,  33, { 20622,  20625,  20634,  20626,  20628,  20631,  20635,  20632}},
 { 20644,  33, { 20625,  20624,  20636,  20634,  20631,  20630,  20637,  20635}},
 { 20645,  33, { 20620,  20619,  20629,  20623,  20638,  20639,  20633,  20627}},
 { 20646,  33, { 20620,  20640,  20264,  20237,  20621,  20621,  20239,  20239}},
 { 20647,  33, { 20641,  20642,  20643,  20644,  20645,  20646,  20647,  20648}},
 { 20648,  33, { 20649,  20650,  20651,  20652,  20653,  20654,  20646,  20647}},
 { 20649,  33, { 20653,  20654,  20646,  20647,  20655,  20656,  20642,  20643}},
 { 20650,  33, { 20640,  20264,  20239,  20621,  20657,  20266,  20265,  20658}},
 { 20651,  33, { 20629,  20628,  20632,  20633,  20656,  20654,  20653,  20655}},
 { 20652,  33, { 20628,  20631,  20635,  20632,  20654,  20650,  20649,  20653}},
 { 20653,  33, { 20238,  20619,  20621,  20239,  20267,  20659,  20658,  20265}},
 { 20654,  33, { 20657,  20266,  20265,  20658,  20660,  20269,  20268,  20661}},
 { 20655,  33, { 20662,  20663,  20648,  20645,  20664,  20665,  20644,  20641}},
 { 20656,  33, { 20033,  20270,  20666,  20498,  20034,  20271,  20272,  20037}},
 { 20657,  33, { 20270,  20273,  20667,  20666,  20271,  20274,  20275,  20272}},
 { 20658,  33, { 20274,  20276,  20277,  20273,  20275,  20278,  20668,  20667}},
 { 20659,  33, { 20276,  20279,  20280,  20277,  20278,  20281,  20669,  20668}},
 { 20660,  33, { 20279,  20282,  20283,  20280,  20281,  20285,  20284,  20669}},
 { 20661,  33, { 20283,  20286,  20287,  20280,  20284,  20288,  20670,  20669}},
 { 20662,  33, { 20286,  20289,  20290,  20287,  20288,  20291,  20671,  20670}},
 { 20663,  33, { 20289,  20292,  20293,  20290,  20291,  20294,  20672,  20671}},
 { 20664,  33, { 20292,  20293,  20672,  20294,  20295,  20297,  20673,  20296}},
 { 20665,  33, { 20297,  20673,  20296,  20295,  20298,  20674,  20300,  20299}},
 { 20666,  33, { 20298,  20674,  20300,  20299,  20301,  20675,  20303,  20302}},
 { 20667,  33, { 20301,  20675,  20303,  20302,  20304,  20676,  20306,  20305}},
 { 20668,  33, { 20304,  20676,  20306,  20305,  20307,  20677,  20309,  20308}},
 { 20669,  33, { 20307,  20677,  20309,  20308,  20310,  20678,  20312,  20311}},
 { 20670,  33, { 20311,  20310,  20678,  20312,  20313,  20315,  20679,  20314}},
 { 20671,  33, { 20313,  20315,  20679,  20314,  20082,  20083,  20513,  20084}},
 { 20672,  33, { 20514,  20680,  20679,  20513,  20085,  20316,  20314,  20084}},
 { 20673,  33, { 20681,  20682,  20677,  20678,  20317,  20318,  20309,  20312}},
 { 20674,  33, { 20680,  20681,  20678,  20679,  20316,  20317,  20312,  20314}},
 { 20675,  33, { 20682,  20683,  20676,  20677,  20318,  20319,  20306,  20309}},
 { 20676,  33, { 20683,  20684,  20675,  20676,  20319,  20320,  20303,  20306}},
 { 20677,  33, { 20684,  20685,  20674,  20675,  20320,  20321,  20300,  20303}},
 { 20678,  33, { 20685,  20686,  20673,  20674,  20321,  20322,  20296,  20300}},
 { 20679,  33, { 20687,  20666,  20667,  20688,  20323,  20272,  20275,  20324}},
 { 20680,  33, { 20524,  20498,  20666,  20687,  20095,  20037,  20272,  20323}},
 { 20681,  33, { 20275,  20278,  20668,  20667,  20324,  20325,  20636,  20688}},
 { 20682,  33, { 20278,  20281,  20669,  20668,  20325,  20246,  20624,  20636}},
 { 20683,  33, { 20281,  20285,  20284,  20669,  20246,  20247,  20250,  20624}},
 { 20684,  33, { 20669,  20284,  20288,  20670,  20624,  20250,  20262,  20630}},
 { 20685,  33, { 20670,  20288,  20291,  20671,  20630,  20262,  20326,  20689}},
 { 20686,  33, { 20294,  20296,  20673,  20672,  20327,  20322,  20686,  20690}},
 { 20687,  33, { 20291,  20294,  20672,  20671,  20326,  20327,  20690,  20689}},
 { 20688,  33, { 20316,  20085,  20514,  20680,  20328,  20100,  20528,  20691}},
 { 20689,  33, { 20317,  20316,  20680,  20681,  20329,  20328,  20691,  20692}},
 { 20690,  33, { 20318,  20317,  20681,  20682,  20330,  20329,  20692,  20693}},
 { 20691,  33, { 20319,  20318,  20682,  20683,  20331,  20330,  20693,  20694}},
 { 20692,  33, { 20320,  20319,  20683,  20684,  20332,  20331,  20694,  20695}},
 { 20693,  33, { 20321,  20320,  20684,  20685,  20333,  20332,  20695,  20696}},
 { 20694,  33, { 20321,  20685,  20686,  20322,  20333,  20696,  20697,  20334}},
 { 20695,  33, { 20698,  20699,  20700,  20652,  20690,  20686,  20697,  20651}},
 { 20696,  33, { 20322,  20327,  20335,  20334,  20686,  20690,  20651,  20697}},
 { 20697,  33, { 20652,  20698,  20690,  20651,  20649,  20701,  20689,  20650}},
 { 20698,  33, { 20651,  20690,  20327,  20335,  20650,  20689,  20326,  20336}},
 { 20699,  33, { 20701,  20637,  20630,  20689,  20649,  20635,  20631,  20650}},
 { 20700,  33, { 20326,  20336,  20650,  20689,  20262,  20263,  20631,  20630}},
 { 20701,  33, { 20095,  20109,  20337,  20323,  20524,  20540,  20702,  20687}},
 { 20702,  33, { 20323,  20337,  20338,  20324,  20687,  20702,  20703,  20688}},
 { 20703,  33, { 20324,  20338,  20339,  20325,  20688,  20703,  20634,  20636}},
 { 20704,  33, { 20109,  20113,  20340,  20337,  20540,  20543,  20704,  20702}},
 { 20705,  33, { 20543,  20704,  20705,  20544,  20113,  20340,  20341,  20115}},
 { 20706,  33, { 20117,  20342,  20706,  20546,  20118,  20343,  20707,  20548}},
 { 20707,  33, { 20118,  20343,  20707,  20548,  20121,  20344,  20708,  20550}},
 { 20708,  33, { 20121,  20344,  20708,  20550,  20123,  20345,  20709,  20552}},
 { 20709,  33, { 20123,  20345,  20709,  20552,  20125,  20346,  20710,  20554}},
 { 20710,  33, { 20128,  20100,  20328,  20347,  20556,  20528,  20691,  20711}},
 { 20711,  33, { 20347,  20328,  20329,  20348,  20711,  20691,  20692,  20712}},
 { 20712,  33, { 20335,  20336,  20350,  20349,  20651,  20650,  20654,  20646}},
 { 20713,  33, { 20336,  20263,  20260,  20350,  20650,  20631,  20628,  20654}},
 { 20714,  33, { 20251,  20625,  20249,  20248,  20244,  20622,  20243,  20240}},
 { 20715,  33, { 20713,  20352,  20253,  20626,  20714,  20351,  20252,  20627}},
 { 20716,  33, { 20702,  20337,  20338,  20703,  20704,  20340,  20352,  20713}},
 { 20717,  33, { 20705,  20341,  20351,  20714,  20715,  20354,  20353,  20716}},
 { 20718,  33, { 20714,  20351,  20252,  20627,  20716,  20353,  20355,  20638}},
 { 20719,  33, { 20627,  20252,  20242,  20623,  20638,  20355,  20257,  20620}},
 { 20720,  33, { 20245,  20623,  20242,  20241,  20237,  20620,  20257,  20254}},
 { 20721,  33, { 20633,  20655,  20717,  20639,  20629,  20656,  20659,  20619}},
 { 20722,  33, { 20658,  20621,  20640,  20657,  20718,  20719,  20720,  20721}},
 { 20723,  33, { 20620,  20621,  20719,  20638,  20640,  20640,  20720,  20720}},
 { 20724,  33, { 20355,  20356,  20256,  20257,  20638,  20357,  20259,  20620}},
 { 20725,  33, { 20720,  20638,  20716,  20722,  20719,  20719,  20723,  20723}},
 { 20726,  33, { 20359,  20358,  20361,  20360,  20721,  20657,  20640,  20720}},
 { 20727,  33, { 20363,  20362,  20359,  20360,  20722,  20724,  20721,  20720}},
 { 20728,  33, { 20342,  20343,  20362,  20363,  20706,  20707,  20724,  20722}},
 { 20729,  33, { 20344,  20364,  20362,  20343,  20708,  20725,  20724,  20707}},
 { 20730,  33, { 20634,  20625,  20622,  20626,  20339,  20249,  20243,  20253}},
 { 20731,  33, { 20264,  20365,  20366,  20266,  20640,  20361,  20358,  20657}},
 { 20732,  33, { 20726,  20718,  20717,  20727,  20661,  20658,  20659,  20728}},
 { 20733,  33, { 20728,  20727,  20643,  20642,  20659,  20717,  20655,  20656}},
 { 20734,  33, { 20362,  20364,  20367,  20359,  20724,  20725,  20729,  20721}},
 { 20735,  33, { 20368,  20728,  20661,  20268,  20369,  20730,  20731,  20370}},
 { 20736,  33, { 20372,  20371,  20374,  20373,  20732,  20376,  20375,  20733}},
 { 20737,  33, { 20372,  20377,  20378,  20371,  20732,  20734,  20379,  20376}},
 { 20738,  33, { 20735,  20369,  20380,  20736,  20382,  20382,  20381,  20381}},
 { 20739,  33, { 20737,  20383,  20370,  20738,  20385,  20385,  20384,  20384}},
 { 20740,  33, { 20739,  20386,  20373,  20733,  20736,  20381,  20382,  20735}},
 { 20741,  33, { 20373,  20372,  20732,  20733,  20382,  20384,  20738,  20735}},
 { 20742,  33, { 20372,  20377,  20734,  20732,  20384,  20385,  20737,  20738}},
 { 20743,  33, { 20735,  20740,  20730,  20369,  20738,  20741,  20731,  20370}},
 { 20744,  33, { 20660,  20269,  20268,  20661,  20742,  20383,  20370,  20731}},
 { 20745,  33, { 20388,  20387,  20390,  20389,  20743,  20744,  20745,  20746}},
 { 20746,  33, { 20387,  20391,  20392,  20390,  20744,  20747,  20748,  20745}},
 { 20747,  33, { 20394,  20393,  20387,  20388,  20749,  20750,  20744,  20743}},
 { 20748,  33, { 20393,  20395,  20391,  20387,  20750,  20751,  20747,  20744}},
 { 20749,  33, { 20397,  20396,  20399,  20398,  20377,  20734,  20379,  20378}},
 { 20750,  33, { 20396,  20394,  20400,  20399,  20734,  20749,  20401,  20379}},
 { 20751,  33, { 20394,  20388,  20402,  20400,  20749,  20743,  20403,  20401}},
 { 20752,  33, { 20388,  20389,  20404,  20402,  20743,  20746,  20405,  20403}},
 { 20753,  33, { 20389,  20406,  20407,  20404,  20746,  20752,  20408,  20405}},
 { 20754,  33, { 20409,  20410,  20383,  20737,  20411,  20411,  20385,  20385}},
 { 20755,  33, { 20396,  20394,  20749,  20734,  20409,  20393,  20750,  20737}},
 { 20756,  33, { 20738,  20741,  20731,  20370,  20737,  20751,  20742,  20383}},
 { 20757,  33, { 20413,  20412,  20415,  20414,  20739,  20386,  20417,  20416}},
 { 20758,  33, { 20417,  20386,  20373,  20374,  20416,  20739,  20733,  20375}},
 { 20759,  33, { 20718,  20753,  20729,  20721,  20726,  20726,  20754,  20754}},
 { 20760,  33, { 20721,  20729,  20367,  20359,  20754,  20754,  20418,  20418}},
 { 20761,  33, { 20721,  20657,  20660,  20754,  20359,  20358,  20419,  20418}},
 { 20762,  33, { 20421,  20349,  20646,  20642,  20380,  20420,  20645,  20641}},
 { 20763,  33, { 20728,  20368,  20421,  20642,  20730,  20369,  20380,  20641}},
 { 20764,  33, { 20406,  20346,  20125,  20203,  20752,  20710,  20554,  20224}},
 { 20765,  33, { 20755,  20556,  20128,  20756,  20711,  20711,  20347,  20347}},
 { 20766,  33, { 20755,  20603,  20213,  20756,  20556,  20556,  20128,  20128}},
 { 20767,  33, { 20213,  20603,  20755,  20756,  20212,  20606,  20757,  20758}},
 { 20768,  33, { 20212,  20606,  20757,  20758,  20224,  20554,  20710,  20752}},
 { 20769,  33, { 20414,  20413,  20739,  20416,  20422,  20422,  20759,  20759}},
 { 20770,  33, { 20424,  20423,  20425,  20348,  20760,  20427,  20426,  20761}},
 { 20771,  33, { 20428,  20213,  20756,  20431,  20429,  20212,  20758,  20430}},
 { 20772,  33, { 20756,  20128,  20432,  20431,  20347,  20347,  20433,  20433}},
 { 20773,  33, { 20756,  20213,  20428,  20431,  20128,  20128,  20432,  20432}},
 { 20774,  33, { 20424,  20434,  20762,  20760,  20435,  20435,  20763,  20763}},
 { 20775,  33, { 20424,  20434,  20436,  20423,  20760,  20762,  20437,  20427}},
 { 20776,  33, { 20762,  20437,  20416,  20759,  20434,  20436,  20414,  20422}},
 { 20777,  33, { 20764,  20763,  20435,  20438,  20665,  20765,  20766,  20663}},
 { 20778,  33, { 20736,  20664,  20641,  20380,  20735,  20740,  20730,  20369}},
 { 20779,  33, { 20693,  20694,  20331,  20330,  20767,  20766,  20435,  20424}},
 { 20780,  33, { 20729,  20725,  20364,  20367,  20747,  20748,  20392,  20391}},
 { 20781,  33, { 20725,  20708,  20344,  20364,  20748,  20709,  20345,  20392}},
 { 20782,  33, { 20331,  20332,  20438,  20435,  20694,  20695,  20663,  20766}},
 { 20783,  33, { 20659,  20267,  20368,  20728,  20658,  20265,  20268,  20661}},
 { 20784,  33, { 20623,  20245,  20261,  20629,  20620,  20237,  20238,  20619}},
 { 20785,  33, { 20439,  20429,  20430,  20408,  20224,  20212,  20758,  20752}},
 { 20786,  33, { 20330,  20329,  20692,  20693,  20424,  20348,  20712,  20767}},
 { 20787,  33, { 20347,  20348,  20425,  20433,  20756,  20761,  20426,  20431}},
 { 20788,  33, { 20408,  20407,  20440,  20439,  20752,  20406,  20203,  20224}},
 { 20789,  33, { 20269,  20660,  20419,  20441,  20383,  20742,  20442,  20410}},
 { 20790,  33, { 20269,  20441,  20419,  20660,  20266,  20366,  20358,  20657}},
 { 20791,  33, { 20395,  20418,  20367,  20391,  20751,  20754,  20729,  20747}},
 { 20792,  33, { 20442,  20419,  20418,  20395,  20742,  20660,  20754,  20751}},
 { 20793,  33, { 20718,  20658,  20661,  20726,  20721,  20657,  20660,  20754}},
 { 20794,  33, { 20390,  20392,  20345,  20346,  20745,  20748,  20709,  20710}},
 { 20795,  33, { 20738,  20370,  20369,  20735,  20384,  20384,  20382,  20382}},
 { 20796,  33, { 20736,  20443,  20420,  20380,  20664,  20662,  20645,  20641}},
 { 20797,  33, { 20410,  20442,  20395,  20409,  20383,  20742,  20751,  20737}},
 { 20798,  33, { 20751,  20750,  20393,  20395,  20737,  20737,  20409,  20409}},
 { 20799,  33, { 20645,  20646,  20647,  20648,  20697,  20651,  20652,  20700}},
 { 20800,  33, { 20332,  20333,  20443,  20438,  20695,  20696,  20662,  20663}},
 { 20801,  33, { 20443,  20333,  20334,  20420,  20662,  20696,  20697,  20645}},
 { 20802,  33, { 20697,  20696,  20695,  20700,  20645,  20662,  20663,  20648}},
 { 20803,  33, { 20697,  20651,  20335,  20334,  20645,  20646,  20349,  20420}},
 { 20804,  33, { 20646,  20654,  20350,  20349,  20642,  20656,  20444,  20421}},
 { 20805,  33, { 20656,  20444,  20261,  20629,  20654,  20350,  20260,  20628}},
 { 20806,  33, { 20339,  20338,  20352,  20253,  20634,  20703,  20713,  20626}},
 { 20807,  33, { 20713,  20704,  20340,  20352,  20714,  20705,  20341,  20351}},
 { 20808,  33, { 20246,  20624,  20636,  20325,  20249,  20625,  20634,  20339}},
 { 20809,  33, { 20421,  20444,  20267,  20368,  20642,  20656,  20659,  20728}},
 { 20810,  33, { 20444,  20261,  20238,  20267,  20656,  20629,  20619,  20659}},
 { 20811,  33, { 20435,  20438,  20422,  20434,  20763,  20764,  20759,  20762}},
 { 20812,  33, { 20438,  20443,  20413,  20422,  20764,  20736,  20739,  20759}},
 { 20813,  33, { 20663,  20662,  20443,  20438,  20665,  20664,  20736,  20764}},
 { 20814,  33, { 20263,  20251,  20244,  20260,  20631,  20625,  20622,  20628}},
 { 20815,  33, { 20752,  20746,  20389,  20406,  20710,  20745,  20390,  20346}},
 { 20816,  33, { 20413,  20412,  20386,  20739,  20443,  20445,  20381,  20736}},
 { 20817,  33, { 20420,  20443,  20736,  20380,  20445,  20445,  20381,  20381}},
 { 20818,  33, { 20435,  20424,  20767,  20766,  20763,  20760,  20768,  20765}},
 { 20819,  33, { 20424,  20348,  20712,  20767,  20760,  20761,  20769,  20768}},
 { 20820,  33, { 20348,  20347,  20711,  20712,  20761,  20756,  20755,  20769}},
 { 20821,  33, { 20385,  20737,  20409,  20411,  20377,  20734,  20396,  20397}},
 { 20822,  33, { 20621,  20620,  20638,  20719,  20619,  20619,  20639,  20639}},
 { 20823,  33, { 20619,  20659,  20717,  20639,  20621,  20658,  20718,  20719}},
 { 20824,  33, { 20356,  20355,  20353,  20446,  20357,  20638,  20716,  20447}},
 { 20825,  33, { 20446,  20353,  20354,  20448,  20447,  20716,  20715,  20449}},
 { 20826,  33, { 20236,  20546,  20706,  20449,  20618,  20618,  20715,  20715}},
 { 20827,  33, { 20449,  20706,  20722,  20447,  20715,  20715,  20716,  20716}},
 { 20828,  33, { 20447,  20722,  20720,  20357,  20716,  20716,  20638,  20638}},
 { 20829,  33, { 20357,  20720,  20640,  20259,  20638,  20638,  20620,  20620}},
 { 20830,  33, { 20357,  20360,  20361,  20259,  20720,  20720,  20640,  20640}},
 { 20831,  33, { 20259,  20640,  20264,  20258,  20620,  20620,  20237,  20237}},
 { 20832,  33, { 20259,  20361,  20365,  20258,  20640,  20640,  20264,  20264}},
 { 20833,  33, { 20236,  20117,  20342,  20449,  20546,  20546,  20706,  20706}},
 { 20834,  33, { 20449,  20342,  20363,  20447,  20706,  20706,  20722,  20722}},
 { 20835,  33, { 20447,  20363,  20360,  20357,  20722,  20722,  20720,  20720}},
 { 20836,  33, { 20618,  20236,  20235,  20450,  20715,  20449,  20448,  20354}},
 { 20837,  33, { 20618,  20450,  20235,  20236,  20561,  20134,  20233,  20234}},
 { 23080,  40, { 22258,  22257,  22260,  22259,  22263,  22264,  22262,  22261}},
 { 23081,  40, { 22257,  22265,  22266,  22260,  22264,  22268,  22267,  22262}},
 { 23082,  40, { 22265,  22269,  22270,  22266,  22268,  22272,  22271,  22267}},
 { 23083,  40, { 22269,  22273,  22274,  22270,  22272,  22276,  22275,  22271}},
 { 23084,  40, { 22273,  22277,  22278,  22274,  22276,  22280,  22279,  22275}},
 { 23085,  40, { 22277,  22281,  22282,  22278,  22280,  22284,  22283,  22279}},
 { 23086,  40, { 22281,  22285,  22286,  22282,  22284,  22288,  22287,  22283}},
 { 23087,  40, { 22285,  22289,  22290,  22286,  22288,  22292,  22291,  22287}},
 { 23088,  40, { 22293,  22258,  22259,  22294,  22296,  22263,  22261,  22295}},
 { 23089,  40, { 22297,  22293,  22294,  22298,  22300,  22296,  22295,  22299}},
 { 23090,  40, { 22301,  22297,  22298,  22302,  22304,  22300,  22299,  22303}},
 { 23091,  40, { 22305,  22301,  22302,  22306,  22308,  22304,  22303,  22307}},
 { 23092,  40, { 22309,  22305,  22306,  22310,  22312,  22308,  22307,  22311}},
 { 23093,  40, { 22313,  22309,  22310,  22314,  22316,  22312,  22311,  22315}},
 { 23094,  40, { 22317,  22313,  22314,  22318,  22320,  22316,  22315,  22319}},
 { 23095,  40, { 22321,  22317,  22318,  22322,  22324,  22320,  22319,  22323}},
 { 23096,  40, { 22325,  22321,  22322,  22326,  22328,  22324,  22323,  22327}},
 { 23097,  40, { 22289,  22325,  22326,  22290,  22292,  22328,  22327,  22291}},
 { 23098,  40, { 22268,  22272,  22330,  22329,  22529,  22530,  22331,  22332}},
 { 23099,  40, { 22264,  22268,  22329,  22333,  22531,  22529,  22332,  22334}},
 { 23100,  40, { 22263,  22264,  22333,  22335,  22532,  22531,  22334,  22336}},
 { 23101,  40, { 22296,  22263,  22335,  22337,  22533,  22532,  22336,  22338}},
 { 23102,  40, { 22300,  22296,  22337,  22339,  22534,  22533,  22338,  22340}},
 { 23103,  40, { 22304,  22300,  22339,  22341,  22535,  22534,  22340,  22342}},
 { 23104,  40, { 22308,  22304,  22341,  22343,  22536,  22535,  22342,  22344}},
 { 23105,  40, { 22308,  22343,  22345,  22312,  22536,  22344,  22346,  22537}},
 { 23106,  40, { 22312,  22345,  22347,  22316,  22537,  22346,  22348,  22538}},
 { 23107,  40, { 22347,  22349,  22320,  22316,  22348,  22350,  22539,  22538}},
 { 23108,  40, { 22320,  22349,  22351,  22324,  22539,  22350,  22352,  22540}},
 { 23109,  40, { 22351,  22353,  22328,  22324,  22352,  22354,  22541,  22540}},
 { 23110,  40, { 22330,  22272,  22276,  22355,  22331,  22530,  22542,  22356}},
 { 23111,  40, { 22355,  22276,  22280,  22357,  22356,  22542,  22543,  22358}},
 { 23112,  40, { 22359,  22292,  22328,  22353,  22360,  22544,  22541,  22354}},
 { 23113,  40, { 22292,  22359,  22361,  22288,  22544,  22360,  22362,  22545}},
 { 23114,  40, { 22361,  22363,  22284,  22288,  22362,  22364,  22546,  22545}},
 { 23115,  40, { 22280,  22284,  22363,  22357,  22543,  22546,  22364,  22358}},
 { 23116,  40, { 22276,  22272,  22269,  22273,  22542,  22530,  22365,  22366}},
 { 23117,  40, { 22272,  22268,  22265,  22269,  22530,  22529,  22367,  22365}},
 { 23118,  40, { 22268,  22264,  22257,  22265,  22529,  22531,  22368,  22367}},
 { 23119,  40, { 22264,  22263,  22258,  22257,  22531,  22532,  22369,  22368}},
 { 23120,  40, { 22263,  22296,  22293,  22258,  22532,  22533,  22370,  22369}},
 { 23121,  40, { 22296,  22300,  22297,  22293,  22533,  22534,  22371,  22370}},
 { 23122,  40, { 22300,  22304,  22301,  22297,  22534,  22535,  22372,  22371}},
 { 23123,  40, { 22304,  22308,  22305,  22301,  22535,  22536,  22373,  22372}},
 { 23124,  40, { 22308,  22312,  22309,  22305,  22536,  22537,  22374,  22373}},
 { 23125,  40, { 22312,  22316,  22313,  22309,  22537,  22538,  22375,  22374}},
 { 23126,  40, { 22316,  22320,  22317,  22313,  22538,  22539,  22376,  22375}},
 { 23127,  40, { 22320,  22324,  22321,  22317,  22539,  22540,  22377,  22376}},
 { 23128,  40, { 22324,  22328,  22325,  22321,  22540,  22541,  22378,  22377}},
 { 23129,  40, { 22328,  22292,  22289,  22325,  22541,  22544,  22379,  22378}},
 { 23130,  40, { 22292,  22288,  22285,  22289,  22544,  22545,  22380,  22379}},
 { 23131,  40, { 22288,  22284,  22281,  22285,  22545,  22546,  22381,  22380}},
 { 23132,  40, { 22284,  22280,  22277,  22281,  22546,  22543,  22382,  22381}},
 { 23133,  40, { 22280,  22276,  22273,  22277,  22543,  22542,  22366,  22382}},
 { 23134,  40, { 22547,  22384,  22383,  22548,  22529,  22367,  22368,  22531}},
 { 23135,  40, { 22529,  22332,  22334,  22531,  22547,  22385,  22386,  22548}},
 { 23136,  40, { 22331,  22332,  22529,  22530,  22387,  22385,  22547,  22549}},
 { 23137,  40, { 22530,  22529,  22367,  22365,  22549,  22547,  22384,  22388}},
 { 23138,  40, { 22356,  22331,  22530,  22542,  22389,  22387,  22549,  22550}},
 { 23139,  40, { 22542,  22530,  22365,  22366,  22550,  22549,  22388,  22390}},
 { 23140,  40, { 22358,  22356,  22542,  22543,  22391,  22389,  22550,  22551}},
 { 23141,  40, { 22543,  22542,  22366,  22382,  22551,  22550,  22390,  22392}},
 { 23142,  40, { 22364,  22358,  22543,  22546,  22393,  22391,  22551,  22552}},
 { 23143,  40, { 22546,  22543,  22382,  22381,  22552,  22551,  22392,  22394}},
 { 23144,  40, { 22362,  22364,  22546,  22545,  22395,  22393,  22552,  22553}},
 { 23145,  40, { 22545,  22546,  22381,  22380,  22553,  22552,  22394,  22396}},
 { 23146,  40, { 22360,  22362,  22545,  22544,  22397,  22395,  22553,  22554}},
 { 23147,  40, { 22544,  22545,  22380,  22379,  22554,  22553,  22396,  22398}},
 { 23148,  40, { 22354,  22360,  22544,  22541,  22399,  22397,  22554,  22555}},
 { 23149,  40, { 22541,  22544,  22379,  22378,  22555,  22554,  22398,  22400}},
 { 23150,  40, { 22352,  22354,  22541,  22540,  22401,  22399,  22555,  22556}},
 { 23151,  40, { 22540,  22541,  22378,  22377,  22556,  22555,  22400,  22402}},
 { 23152,  40, { 22539,  22540,  22377,  22376,  22557,  22556,  22402,  22403}},
 { 23153,  40, { 22350,  22352,  22540,  22539,  22404,  22401,  22556,  22557}},
 { 23154,  40, { 22348,  22350,  22539,  22538,  22405,  22404,  22557,  22558}},
 { 23155,  40, { 22538,  22539,  22376,  22375,  22558,  22557,  22403,  22406}},
 { 23156,  40, { 22537,  22538,  22375,  22374,  22559,  22558,  22406,  22407}},
 { 23157,  40, { 22346,  22348,  22538,  22537,  22408,  22405,  22558,  22559}},
 { 23158,  40, { 22344,  22346,  22537,  22536,  22409,  22408,  22559,  22560}},
 { 23159,  40, { 22536,  22537,  22374,  22373,  22560,  22559,  22407,  22410}},
 { 23160,  40, { 22344,  22536,  22535,  22342,  22409,  22560,  22561,  22411}},
 { 23161,  40, { 22372,  22535,  22536,  22373,  22412,  22561,  22560,  22410}},
 { 23162,  40, { 22535,  22534,  22340,  22342,  22561,  22562,  22413,  22411}},
 { 23163,  40, { 22372,  22371,  22534,  22535,  22412,  22414,  22562,  22561}},
 { 23164,  40, { 22534,  22533,  22338,  22340,  22562,  22563,  22415,  22413}},
 { 23165,  40, { 22371,  22370,  22533,  22534,  22414,  22416,  22563,  22562}},
 { 23166,  40, { 22533,  22532,  22336,  22338,  22563,  22564,  22417,  22415}},
 { 23167,  40, { 22533,  22370,  22369,  22532,  22563,  22416,  22418,  22564}},
 { 23168,  40, { 22417,  22386,  22548,  22564,  22336,  22334,  22531,  22532}},
 { 23169,  40, { 22564,  22548,  22383,  22418,  22532,  22531,  22368,  22369}},
 { 23170,  40, { 22549,  22550,  22389,  22387,  22421,  22422,  22419,  22420}},
 { 23171,  40, { 22549,  22388,  22390,  22550,  22421,  22423,  22424,  22422}},
 { 23172,  40, { 22550,  22390,  22392,  22551,  22422,  22424,  22425,  22426}},
 { 23173,  40, { 22389,  22550,  22551,  22391,  22419,  22422,  22426,  22427}},
 { 23174,  40, { 22391,  22551,  22552,  22393,  22427,  22426,  22429,  22428}},
 { 23175,  40, { 22385,  22547,  22549,  22387,  22430,  22431,  22421,  22420}},
 { 23176,  40, { 22547,  22384,  22388,  22549,  22431,  22432,  22423,  22421}},
 { 23177,  40, { 22548,  22383,  22384,  22547,  22434,  22433,  22432,  22431}},
 { 23178,  40, { 22386,  22548,  22547,  22385,  22435,  22434,  22431,  22430}},
 { 23179,  40, { 22564,  22418,  22383,  22548,  22437,  22436,  22433,  22434}},
 { 23180,  40, { 22417,  22564,  22548,  22386,  22438,  22437,  22434,  22435}},
 { 23181,  40, { 22563,  22416,  22418,  22564,  22440,  22439,  22436,  22437}},
 { 23182,  40, { 22563,  22564,  22417,  22415,  22440,  22437,  22438,  22441}},
 { 23183,  40, { 22562,  22563,  22415,  22413,  22443,  22440,  22441,  22442}},
 { 23184,  40, { 22414,  22416,  22563,  22562,  22444,  22439,  22440,  22443}},
 { 23185,  40, { 22446,  22445,  22443,  22444,  22412,  22561,  22562,  22414}},
 { 23186,  40, { 22445,  22447,  22442,  22443,  22561,  22411,  22413,  22562}},
 { 23187,  40, { 22412,  22561,  22560,  22410,  22446,  22445,  22449,  22448}},
 { 23188,  40, { 22561,  22411,  22409,  22560,  22445,  22447,  22450,  22449}},
 { 23189,  40, { 22410,  22560,  22559,  22407,  22448,  22449,  22452,  22451}},
 { 23190,  40, { 22560,  22409,  22408,  22559,  22449,  22450,  22453,  22452}},
 { 23191,  40, { 22559,  22558,  22406,  22407,  22452,  22455,  22454,  22451}},
 { 23192,  40, { 22559,  22408,  22405,  22558,  22452,  22453,  22456,  22455}},
 { 23193,  40, { 22558,  22405,  22404,  22557,  22455,  22456,  22457,  22458}},
 { 23194,  40, { 22406,  22558,  22557,  22403,  22454,  22455,  22458,  22459}},
 { 23195,  40, { 22557,  22404,  22401,  22556,  22458,  22457,  22460,  22461}},
 { 23196,  40, { 22403,  22557,  22556,  22402,  22459,  22458,  22461,  22462}},
 { 23197,  40, { 22401,  22399,  22555,  22556,  22460,  22463,  22464,  22461}},
 { 23198,  40, { 22399,  22397,  22554,  22555,  22463,  22465,  22466,  22464}},
 { 23199,  40, { 22397,  22395,  22553,  22554,  22465,  22467,  22468,  22466}},
 { 23200,  40, { 22551,  22392,  22394,  22552,  22426,  22425,  22469,  22429}},
 { 23201,  40, { 22393,  22552,  22553,  22395,  22428,  22429,  22468,  22467}},
 { 23202,  40, { 22553,  22552,  22394,  22396,  22468,  22429,  22469,  22470}},
 { 23203,  40, { 22554,  22553,  22396,  22398,  22466,  22468,  22470,  22471}},
 { 23204,  40, { 22555,  22554,  22398,  22400,  22464,  22466,  22471,  22472}},
 { 23205,  40, { 22556,  22555,  22400,  22402,  22461,  22464,  22472,  22462}},
 { 24220,  43, { 23232,  23231,  23230,  23229,  23235,  23234,  23233,  23236}},
 { 24221,  43, { 23229,  23230,  23238,  23237,  23236,  23233,  23239,  23240}},
 { 24222,  43, { 23237,  23238,  23242,  23241,  23240,  23239,  23243,  23244}},
 { 24223,  43, { 23241,  23242,  23246,  23245,  23244,  23243,  23247,  23248}},
 { 24224,  43, { 23245,  23246,  23250,  23249,  23248,  23247,  23251,  23252}},
 { 24225,  43, { 23249,  23250,  23254,  23253,  23252,  23251,  23255,  23256}},
 { 24226,  43, { 23253,  23254,  23258,  23257,  23256,  23255,  23259,  23260}},
 { 24227,  43, { 23257,  23258,  23262,  23261,  23260,  23259,  23263,  23264}},
 { 24228,  43, { 23266,  23265,  23231,  23232,  23268,  23267,  23234,  23235}},
 { 24229,  43, { 23270,  23269,  23265,  23266,  23272,  23271,  23267,  23268}},
 { 24230,  43, { 23274,  23273,  23269,  23270,  23276,  23275,  23271,  23272}},
 { 24231,  43, { 23278,  23277,  23273,  23274,  23280,  23279,  23275,  23276}},
 { 24232,  43, { 23282,  23281,  23277,  23278,  23284,  23283,  23279,  23280}},
 { 24233,  43, { 23286,  23285,  23281,  23282,  23288,  23287,  23283,  23284}},
 { 24234,  43, { 23290,  23289,  23285,  23286,  23292,  23291,  23287,  23288}},
 { 24235,  43, { 23294,  23293,  23289,  23290,  23296,  23295,  23291,  23292}},
 { 24236,  43, { 23298,  23297,  23293,  23294,  23300,  23299,  23295,  23296}},
 { 24237,  43, { 23261,  23262,  23297,  23298,  23264,  23263,  23299,  23300}},
 { 24238,  43, { 23240,  23301,  23302,  23244,  23501,  23303,  23304,  23502}},
 { 24239,  43, { 23236,  23305,  23301,  23240,  23503,  23306,  23303,  23501}},
 { 24240,  43, { 23235,  23307,  23305,  23236,  23504,  23308,  23306,  23503}},
 { 24241,  43, { 23268,  23309,  23307,  23235,  23505,  23310,  23308,  23504}},
 { 24242,  43, { 23272,  23311,  23309,  23268,  23506,  23312,  23310,  23505}},
 { 24243,  43, { 23276,  23313,  23311,  23272,  23507,  23314,  23312,  23506}},
 { 24244,  43, { 23280,  23315,  23313,  23276,  23508,  23316,  23314,  23507}},
 { 24245,  43, { 23280,  23284,  23317,  23315,  23508,  23509,  23318,  23316}},
 { 24246,  43, { 23284,  23288,  23319,  23317,  23509,  23510,  23320,  23318}},
 { 24247,  43, { 23319,  23288,  23292,  23321,  23320,  23510,  23511,  23322}},
 { 24248,  43, { 23292,  23296,  23323,  23321,  23511,  23512,  23324,  23322}},
 { 24249,  43, { 23323,  23296,  23300,  23325,  23324,  23512,  23513,  23326}},
 { 24250,  43, { 23302,  23327,  23248,  23244,  23304,  23328,  23514,  23502}},
 { 24251,  43, { 23327,  23329,  23252,  23248,  23328,  23330,  23515,  23514}},
 { 24252,  43, { 23331,  23325,  23300,  23264,  23332,  23326,  23513,  23516}},
 { 24253,  43, { 23264,  23260,  23333,  23331,  23516,  23517,  23334,  23332}},
 { 24254,  43, { 23333,  23260,  23256,  23335,  23334,  23517,  23518,  23336}},
 { 24255,  43, { 23252,  23329,  23335,  23256,  23515,  23330,  23336,  23518}},
 { 24256,  43, { 23248,  23245,  23241,  23244,  23514,  23337,  23338,  23502}},
 { 24257,  43, { 23244,  23241,  23237,  23240,  23502,  23338,  23339,  23501}},
 { 24258,  43, { 23240,  23237,  23229,  23236,  23501,  23339,  23340,  23503}},
 { 24259,  43, { 23236,  23229,  23232,  23235,  23503,  23340,  23341,  23504}},
 { 24260,  43, { 23235,  23232,  23266,  23268,  23504,  23341,  23342,  23505}},
 { 24261,  43, { 23268,  23266,  23270,  23272,  23505,  23342,  23343,  23506}},
 { 24262,  43, { 23272,  23270,  23274,  23276,  23506,  23343,  23344,  23507}},
 { 24263,  43, { 23276,  23274,  23278,  23280,  23507,  23344,  23345,  23508}},
 { 24264,  43, { 23280,  23278,  23282,  23284,  23508,  23345,  23346,  23509}},
 { 24265,  43, { 23284,  23282,  23286,  23288,  23509,  23346,  23347,  23510}},
 { 24266,  43, { 23288,  23286,  23290,  23292,  23510,  23347,  23348,  23511}},
 { 24267,  43, { 23292,  23290,  23294,  23296,  23511,  23348,  23349,  23512}},
 { 24268,  43, { 23296,  23294,  23298,  23300,  23512,  23349,  23350,  23513}},
 { 24269,  43, { 23300,  23298,  23261,  23264,  23513,  23350,  23351,  23516}},
 { 24270,  43, { 23264,  23261,  23257,  23260,  23516,  23351,  23352,  23517}},
 { 24271,  43, { 23260,  23257,  23253,  23256,  23517,  23352,  23353,  23518}},
 { 24272,  43, { 23256,  23253,  23249,  23252,  23518,  23353,  23354,  23515}},
 { 24273,  43, { 23252,  23249,  23245,  23248,  23515,  23354,  23337,  23514}},
 { 24274,  43, { 23519,  23520,  23355,  23356,  23501,  23503,  23340,  23339}},
 { 24275,  43, { 23501,  23503,  23306,  23303,  23519,  23520,  23357,  23358}},
 { 24276,  43, { 23304,  23502,  23501,  23303,  23359,  23521,  23519,  23358}},
 { 24277,  43, { 23502,  23338,  23339,  23501,  23521,  23360,  23356,  23519}},
 { 24278,  43, { 23328,  23514,  23502,  23304,  23361,  23522,  23521,  23359}},
 { 24279,  43, { 23514,  23337,  23338,  23502,  23522,  23362,  23360,  23521}},
 { 24280,  43, { 23330,  23515,  23514,  23328,  23363,  23523,  23522,  23361}},
 { 24281,  43, { 23515,  23354,  23337,  23514,  23523,  23364,  23362,  23522}},
 { 24282,  43, { 23336,  23518,  23515,  23330,  23365,  23524,  23523,  23363}},
 { 24283,  43, { 23518,  23353,  23354,  23515,  23524,  23366,  23364,  23523}},
 { 24284,  43, { 23334,  23517,  23518,  23336,  23367,  23525,  23524,  23365}},
 { 24285,  43, { 23517,  23352,  23353,  23518,  23525,  23368,  23366,  23524}},
 { 24286,  43, { 23332,  23516,  23517,  23334,  23369,  23526,  23525,  23367}},
 { 24287,  43, { 23516,  23351,  23352,  23517,  23526,  23370,  23368,  23525}},
 { 24288,  43, { 23326,  23513,  23516,  23332,  23371,  23527,  23526,  23369}},
 { 24289,  43, { 23513,  23350,  23351,  23516,  23527,  23372,  23370,  23526}},
 { 24290,  43, { 23324,  23512,  23513,  23326,  23373,  23528,  23527,  23371}},
 { 24291,  43, { 23512,  23349,  23350,  23513,  23528,  23374,  23372,  23527}},
 { 24292,  43, { 23511,  23348,  23349,  23512,  23529,  23375,  23374,  23528}},
 { 24293,  43, { 23322,  23511,  23512,  23324,  23376,  23529,  23528,  23373}},
 { 24294,  43, { 23320,  23510,  23511,  23322,  23377,  23530,  23529,  23376}},
 { 24295,  43, { 23510,  23347,  23348,  23511,  23530,  23378,  23375,  23529}},
 { 24296,  43, { 23509,  23346,  23347,  23510,  23531,  23379,  23378,  23530}},
 { 24297,  43, { 23318,  23509,  23510,  23320,  23380,  23531,  23530,  23377}},
 { 24298,  43, { 23316,  23508,  23509,  23318,  23381,  23532,  23531,  23380}},
 { 24299,  43, { 23508,  23345,  23346,  23509,  23532,  23382,  23379,  23531}},
 { 24300,  43, { 23316,  23314,  23507,  23508,  23381,  23383,  23533,  23532}},
 { 24301,  43, { 23344,  23345,  23508,  23507,  23384,  23382,  23532,  23533}},
 { 24302,  43, { 23507,  23314,  23312,  23506,  23533,  23383,  23385,  23534}},
 { 24303,  43, { 23344,  23507,  23506,  23343,  23384,  23533,  23534,  23386}},
 { 24304,  43, { 23506,  23312,  23310,  23505,  23534,  23385,  23387,  23535}},
 { 24305,  43, { 23343,  23506,  23505,  23342,  23386,  23534,  23535,  23388}},
 { 24306,  43, { 23505,  23310,  23308,  23504,  23535,  23387,  23389,  23536}},
 { 24307,  43, { 23505,  23504,  23341,  23342,  23535,  23536,  23390,  23388}},
 { 24308,  43, { 23389,  23536,  23520,  23357,  23308,  23504,  23503,  23306}},
 { 24309,  43, { 23536,  23390,  23355,  23520,  23504,  23341,  23340,  23503}},
 { 24310,  43, { 23521,  23359,  23361,  23522,  23393,  23391,  23392,  23394}},
 { 24311,  43, { 23521,  23522,  23362,  23360,  23393,  23394,  23395,  23396}},
 { 24312,  43, { 23522,  23523,  23364,  23362,  23394,  23398,  23397,  23395}},
 { 24313,  43, { 23361,  23363,  23523,  23522,  23392,  23399,  23398,  23394}},
 { 24314,  43, { 23363,  23365,  23524,  23523,  23399,  23400,  23401,  23398}},
 { 24315,  43, { 23358,  23359,  23521,  23519,  23402,  23391,  23393,  23403}},
 { 24316,  43, { 23519,  23521,  23360,  23356,  23403,  23393,  23396,  23404}},
 { 24317,  43, { 23520,  23519,  23356,  23355,  23406,  23403,  23404,  23405}},
 { 24318,  43, { 23357,  23358,  23519,  23520,  23407,  23402,  23403,  23406}},
 { 24319,  43, { 23536,  23520,  23355,  23390,  23409,  23406,  23405,  23408}},
 { 24320,  43, { 23389,  23357,  23520,  23536,  23410,  23407,  23406,  23409}},
 { 24321,  43, { 23535,  23536,  23390,  23388,  23412,  23409,  23408,  23411}},
 { 24322,  43, { 23535,  23387,  23389,  23536,  23412,  23413,  23410,  23409}},
 { 24323,  43, { 23534,  23385,  23387,  23535,  23415,  23414,  23413,  23412}},
 { 24324,  43, { 23386,  23534,  23535,  23388,  23416,  23415,  23412,  23411}},
 { 24325,  43, { 23418,  23416,  23415,  23417,  23384,  23386,  23534,  23533}},
 { 24326,  43, { 23417,  23415,  23414,  23419,  23533,  23534,  23385,  23383}},
 { 24327,  43, { 23384,  23382,  23532,  23533,  23418,  23420,  23421,  23417}},
 { 24328,  43, { 23533,  23532,  23381,  23383,  23417,  23421,  23422,  23419}},
 { 24329,  43, { 23382,  23379,  23531,  23532,  23420,  23423,  23424,  23421}},
 { 24330,  43, { 23532,  23531,  23380,  23381,  23421,  23424,  23425,  23422}},
 { 24331,  43, { 23531,  23379,  23378,  23530,  23424,  23423,  23426,  23427}},
 { 24332,  43, { 23531,  23530,  23377,  23380,  23424,  23427,  23428,  23425}},
 { 24333,  43, { 23530,  23529,  23376,  23377,  23427,  23430,  23429,  23428}},
 { 24334,  43, { 23378,  23375,  23529,  23530,  23426,  23431,  23430,  23427}},
 { 24335,  43, { 23529,  23528,  23373,  23376,  23430,  23433,  23432,  23429}},
 { 24336,  43, { 23375,  23374,  23528,  23529,  23431,  23434,  23433,  23430}},
 { 24337,  43, { 23373,  23528,  23527,  23371,  23432,  23433,  23436,  23435}},
 { 24338,  43, { 23371,  23527,  23526,  23369,  23435,  23436,  23438,  23437}},
 { 24339,  43, { 23369,  23526,  23525,  23367,  23437,  23438,  23440,  23439}},
 { 24340,  43, { 23523,  23524,  23366,  23364,  23398,  23401,  23441,  23397}},
 { 24341,  43, { 23365,  23367,  23525,  23524,  23400,  23439,  23440,  23401}},
 { 24342,  43, { 23525,  23368,  23366,  23524,  23440,  23442,  23441,  23401}},
 { 24343,  43, { 23526,  23370,  23368,  23525,  23438,  23443,  23442,  23440}},
 { 24344,  43, { 23527,  23372,  23370,  23526,  23436,  23444,  23443,  23438}},
 { 24345,  43, { 23528,  23374,  23372,  23527,  23433,  23434,  23444,  23436}},
 { 27000,  51, { 27001,  27000,  27003,  27004,  27002,  27002,  27005,  27005}},
 { 27001,  51, { 27007,  27006,  27009,  27010,  27008,  27008,  27011,  27011}},
 { 27002,  51, { 27013,  27012,  27015,  27016,  27014,  27014,  27017,  27017}},
 { 27003,  51, { 27019,  27018,  27012,  27013,  27020,  27020,  27014,  27014}},
 { 27004,  51, { 27001,  27002,  27005,  27004,  27021,  27021,  27022,  27022}},
 { 27005,  51, { 27024,  27023,  27008,  27007,  27025,  27025,  27026,  27026}},
 { 27006,  51, { 27019,  27020,  27014,  27013,  27027,  27027,  27028,  27028}},
 { 27007,  51, { 27030,  27029,  27020,  27019,  27031,  27031,  27027,  27027}},
 { 27008,  51, { 27001,  27021,  27022,  27004,  27032,  27032,  27033,  27033}},
 { 27009,  51, { 27024,  27025,  27026,  27007,  27034,  27034,  27035,  27035}},
 { 27010,  51, { 27037,  27036,  27025,  27024,  27038,  27038,  27034,  27034}},
 { 27011,  51, { 27040,  27039,  27036,  27037,  27041,  27041,  27038,  27038}},
 { 27012,  51, { 27019,  27027,  27028,  27013,  27042,  27042,  27043,  27043}},
 { 27013,  51, { 27001,  27032,  27033,  27004,  27044,  27044,  27045,  27045}},
 { 27014,  51, { 27007,  27046,  27048,  27010,  27047,  27047,  27049,  27049}},
 { 27015,  51, { 27037,  27050,  27052,  27024,  27051,  27051,  27053,  27053}},
 { 27016,  51, { 27040,  27054,  27050,  27037,  27055,  27055,  27051,  27051}},
 { 27017,  51, { 27016,  27056,  27054,  27040,  27057,  27057,  27055,  27055}},
 { 27018,  51, { 27013,  27058,  27056,  27016,  27059,  27059,  27057,  27057}},
 { 27019,  51, { 27030,  27060,  27062,  27019,  27061,  27061,  27063,  27063}},
 { 27020,  51, { 27004,  27064,  27060,  27030,  27065,  27065,  27061,  27061}},
 { 27021,  51, { 27007,  27047,  27049,  27010,  27066,  27066,  27067,  27067}},
 { 27022,  51, { 27037,  27051,  27053,  27024,  27068,  27068,  27069,  27069}},
 { 27023,  51, { 27040,  27055,  27051,  27037,  27070,  27070,  27068,  27068}},
 { 27024,  51, { 27013,  27059,  27057,  27016,  27071,  27071,  27072,  27072}},
 { 27025,  51, { 27004,  27065,  27061,  27030,  27073,  27073,  27074,  27074}},
 { 27026,  51, { 27024,  27069,  27066,  27007,  27075,  27075,  27076,  27076}},
 { 27027,  51, { 27037,  27068,  27069,  27024,  27077,  27077,  27075,  27075}},
 { 27028,  51, { 27040,  27070,  27068,  27037,  27078,  27078,  27077,  27077}},
 { 27029,  51, { 27016,  27072,  27070,  27040,  27079,  27079,  27078,  27078}},
 { 27030,  51, { 27030,  27074,  27081,  27019,  27080,  27080,  27082,  27082}},
 { 27031,  51, { 27083,  27084,  27085,  27086,  27087,  27088,  27089,  27090}},
 { 27032,  51, { 27084,  27091,  27092,  27085,  27088,  27093,  27094,  27089}},
 { 27033,  51, { 27091,  27095,  27096,  27092,  27093,  27009,  27006,  27094}},
 { 27034,  51, { 27097,  27086,  27098,  27099,  27100,  27090,  27101,  27102}},
 { 27035,  51, { 27086,  27085,  27103,  27098,  27090,  27089,  27104,  27101}},
 { 27036,  51, { 27085,  27092,  27105,  27103,  27089,  27094,  27106,  27104}},
 { 27037,  51, { 27092,  27096,  27107,  27105,  27094,  27006,  27108,  27106}},
 { 27038,  51, { 27099,  27098,  27109,  27110,  27102,  27101,  27111,  27112}},
 { 27039,  51, { 27098,  27103,  27113,  27109,  27101,  27104,  27114,  27111}},
 { 27040,  51, { 27103,  27105,  27115,  27113,  27104,  27106,  27116,  27114}},
 { 27041,  51, { 27105,  27107,  27117,  27115,  27106,  27108,  27118,  27116}},
 { 27042,  51, { 27110,  27109,  27119,  27120,  27112,  27111,  27121,  27122}},
 { 27043,  51, { 27109,  27113,  27123,  27119,  27111,  27114,  27124,  27121}},
 { 27044,  51, { 27113,  27115,  27125,  27123,  27114,  27116,  27126,  27124}},
 { 27045,  51, { 27115,  27117,  27127,  27125,  27116,  27118,  27128,  27126}},
 { 27046,  51, { 27120,  27119,  27129,  27130,  27122,  27121,  27131,  27132}},
 { 27047,  51, { 27119,  27123,  27133,  27129,  27121,  27124,  27134,  27131}},
 { 27048,  51, { 27123,  27125,  27135,  27133,  27124,  27126,  27136,  27134}},
 { 27049,  51, { 27125,  27127,  27137,  27135,  27126,  27128,  27015,  27136}},
 { 27050,  51, { 27130,  27129,  27138,  27139,  27132,  27131,  27140,  27141}},
 { 27051,  51, { 27129,  27133,  27142,  27138,  27131,  27134,  27143,  27140}},
 { 27052,  51, { 27133,  27135,  27144,  27142,  27134,  27136,  27145,  27143}},
 { 27053,  51, { 27135,  27137,  27146,  27144,  27136,  27015,  27012,  27145}},
 { 27054,  51, { 27139,  27138,  27147,  27148,  27141,  27140,  27149,  27150}},
 { 27055,  51, { 27138,  27142,  27151,  27147,  27140,  27143,  27152,  27149}},
 { 27056,  51, { 27142,  27144,  27153,  27151,  27143,  27145,  27154,  27152}},
 { 27057,  51, { 27144,  27146,  27155,  27153,  27145,  27012,  27018,  27154}},
 { 27058,  51, { 27148,  27147,  27156,  27157,  27150,  27149,  27158,  27159}},
 { 27059,  51, { 27147,  27151,  27160,  27156,  27149,  27152,  27161,  27158}},
 { 27060,  51, { 27151,  27153,  27162,  27160,  27152,  27154,  27163,  27161}},
 { 27061,  51, { 27153,  27155,  27164,  27162,  27154,  27018,  27165,  27163}},
 { 27062,  51, { 27157,  27156,  27166,  27167,  27159,  27158,  27168,  27169}},
 { 27063,  51, { 27156,  27160,  27170,  27166,  27158,  27161,  27171,  27168}},
 { 27064,  51, { 27160,  27162,  27172,  27170,  27161,  27163,  27173,  27171}},
 { 27065,  51, { 27162,  27164,  27003,  27172,  27163,  27165,  27005,  27173}},
 { 27066,  51, { 27167,  27166,  27174,  27175,  27169,  27168,  27176,  27177}},
 { 27067,  51, { 27166,  27170,  27178,  27174,  27168,  27171,  27179,  27176}},
 { 27068,  51, { 27170,  27172,  27180,  27178,  27171,  27173,  27181,  27179}},
 { 27069,  51, { 27172,  27003,  27000,  27180,  27173,  27005,  27002,  27181}},
 { 27070,  51, { 27087,  27088,  27089,  27090,  27182,  27183,  27184,  27185}},
 { 27071,  51, { 27088,  27093,  27094,  27089,  27183,  27186,  27187,  27184}},
 { 27072,  51, { 27093,  27009,  27006,  27094,  27186,  27011,  27008,  27187}},
 { 27073,  51, { 27100,  27090,  27101,  27102,  27188,  27185,  27189,  27190}},
 { 27074,  51, { 27090,  27089,  27104,  27101,  27185,  27184,  27191,  27189}},
 { 27075,  51, { 27089,  27094,  27106,  27104,  27184,  27187,  27192,  27191}},
 { 27076,  51, { 27094,  27006,  27108,  27106,  27187,  27008,  27023,  27192}},
 { 27077,  51, { 27102,  27101,  27111,  27112,  27190,  27189,  27193,  27194}},
 { 27078,  51, { 27101,  27104,  27114,  27111,  27189,  27191,  27195,  27193}},
 { 27079,  51, { 27104,  27106,  27116,  27114,  27191,  27192,  27196,  27195}},
 { 27080,  51, { 27106,  27108,  27118,  27116,  27192,  27023,  27197,  27196}},
 { 27081,  51, { 27112,  27111,  27121,  27122,  27194,  27193,  27198,  27199}},
 { 27082,  51, { 27111,  27114,  27124,  27121,  27193,  27195,  27200,  27198}},
 { 27083,  51, { 27114,  27116,  27126,  27124,  27195,  27196,  27201,  27200}},
 { 27084,  51, { 27116,  27118,  27128,  27126,  27196,  27197,  27202,  27201}},
 { 27085,  51, { 27122,  27121,  27131,  27132,  27199,  27198,  27203,  27204}},
 { 27086,  51, { 27121,  27124,  27134,  27131,  27198,  27200,  27205,  27203}},
 { 27087,  51, { 27124,  27126,  27136,  27134,  27200,  27201,  27206,  27205}},
 { 27088,  51, { 27126,  27128,  27015,  27136,  27201,  27202,  27017,  27206}},
 { 27089,  51, { 27132,  27131,  27140,  27141,  27204,  27203,  27207,  27208}},
 { 27090,  51, { 27131,  27134,  27143,  27140,  27203,  27205,  27209,  27207}},
 { 27091,  51, { 27134,  27136,  27145,  27143,  27205,  27206,  27210,  27209}},
 { 27092,  51, { 27136,  27015,  27012,  27145,  27206,  27017,  27014,  27210}},
 { 27093,  51, { 27141,  27140,  27149,  27150,  27208,  27207,  27211,  27212}},
 { 27094,  51, { 27140,  27143,  27152,  27149,  27207,  27209,  27213,  27211}},
 { 27095,  51, { 27143,  27145,  27154,  27152,  27209,  27210,  27214,  27213}},
 { 27096,  51, { 27145,  27012,  27018,  27154,  27210,  27014,  27020,  27214}},
 { 27097,  51, { 27150,  27149,  27158,  27159,  27212,  27211,  27215,  27216}},
 { 27098,  51, { 27149,  27152,  27161,  27158,  27211,  27213,  27217,  27215}},
 { 27099,  51, { 27152,  27154,  27163,  27161,  27213,  27214,  27218,  27217}},
 { 27100,  51, { 27154,  27018,  27165,  27163,  27214,  27020,  27029,  27218}},
 { 27101,  51, { 27159,  27158,  27168,  27169,  27216,  27215,  27219,  27220}},
 { 27102,  51, { 27158,  27161,  27171,  27168,  27215,  27217,  27221,  27219}},
 { 27103,  51, { 27161,  27163,  27173,  27171,  27217,  27218,  27222,  27221}},
 { 27104,  51, { 27163,  27165,  27005,  27173,  27218,  27029,  27022,  27222}},
 { 27105,  51, { 27169,  27168,  27176,  27177,  27220,  27219,  27223,  27224}},
 { 27106,  51, { 27168,  27171,  27179,  27176,  27219,  27221,  27225,  27223}},
 { 27107,  51, { 27171,  27173,  27181,  27179,  27221,  27222,  27226,  27225}},
 { 27108,  51, { 27173,  27005,  27002,  27181,  27222,  27022,  27021,  27226}},
 { 27109,  51, { 27182,  27183,  27184,  27185,  27227,  27228,  27229,  27230}},
 { 27110,  51, { 27183,  27186,  27187,  27184,  27228,  27231,  27232,  27229}},
 { 27111,  51, { 27186,  27011,  27008,  27187,  27231,  27233,  27026,  27232}},
 { 27112,  51, { 27188,  27185,  27189,  27190,  27234,  27230,  27235,  27236}},
 { 27113,  51, { 27185,  27184,  27191,  27189,  27230,  27229,  27237,  27235}},
 { 27114,  51, { 27184,  27187,  27192,  27191,  27229,  27232,  27238,  27237}},
 { 27115,  51, { 27187,  27008,  27023,  27192,  27232,  27026,  27025,  27238}},
 { 27116,  51, { 27190,  27189,  27193,  27194,  27236,  27235,  27239,  27240}},
 { 27117,  51, { 27189,  27191,  27195,  27193,  27235,  27237,  27241,  27239}},
 { 27118,  51, { 27191,  27192,  27196,  27195,  27237,  27238,  27242,  27241}},
 { 27119,  51, { 27192,  27023,  27197,  27196,  27238,  27025,  27036,  27242}},
 { 27120,  51, { 27194,  27193,  27198,  27199,  27240,  27239,  27243,  27244}},
 { 27121,  51, { 27193,  27195,  27200,  27198,  27239,  27241,  27245,  27243}},
 { 27122,  51, { 27195,  27196,  27201,  27200,  27241,  27242,  27246,  27245}},
 { 27123,  51, { 27196,  27197,  27202,  27201,  27242,  27036,  27039,  27246}},
 { 27124,  51, { 27199,  27198,  27203,  27204,  27244,  27243,  27247,  27248}},
 { 27125,  51, { 27198,  27200,  27205,  27203,  27243,  27245,  27249,  27247}},
 { 27126,  51, { 27200,  27201,  27206,  27205,  27245,  27246,  27250,  27249}},
 { 27127,  51, { 27201,  27202,  27017,  27206,  27246,  27039,  27251,  27250}},
 { 27128,  51, { 27204,  27203,  27207,  27208,  27248,  27247,  27252,  27253}},
 { 27129,  51, { 27203,  27205,  27209,  27207,  27247,  27249,  27254,  27252}},
 { 27130,  51, { 27205,  27206,  27210,  27209,  27249,  27250,  27255,  27254}},
 { 27131,  51, { 27206,  27017,  27014,  27210,  27250,  27251,  27028,  27255}},
 { 27132,  51, { 27208,  27207,  27211,  27212,  27253,  27252,  27256,  27257}},
 { 27133,  51, { 27207,  27209,  27213,  27211,  27252,  27254,  27258,  27256}},
 { 27134,  51, { 27209,  27210,  27214,  27213,  27254,  27255,  27259,  27258}},
 { 27135,  51, { 27210,  27014,  27020,  27214,  27255,  27028,  27027,  27259}},
 { 27136,  51, { 27212,  27211,  27215,  27216,  27257,  27256,  27260,  27261}},
 { 27137,  51, { 27211,  27213,  27217,  27215,  27256,  27258,  27262,  27260}},
 { 27138,  51, { 27213,  27214,  27218,  27217,  27258,  27259,  27263,  27262}},
 { 27139,  51, { 27214,  27020,  27029,  27218,  27259,  27027,  27031,  27263}},
 { 27140,  51, { 27216,  27215,  27219,  27220,  27261,  27260,  27264,  27265}},
 { 27141,  51, { 27215,  27217,  27221,  27219,  27260,  27262,  27266,  27264}},
 { 27142,  51, { 27217,  27218,  27222,  27221,  27262,  27263,  27267,  27266}},
 { 27143,  51, { 27218,  27029,  27022,  27222,  27263,  27031,  27033,  27267}},
 { 27144,  51, { 27220,  27219,  27223,  27224,  27265,  27264,  27268,  27269}},
 { 27145,  51, { 27219,  27221,  27225,  27223,  27264,  27266,  27270,  27268}},
 { 27146,  51, { 27221,  27222,  27226,  27225,  27266,  27267,  27271,  27270}},
 { 27147,  51, { 27222,  27022,  27021,  27226,  27267,  27033,  27032,  27271}},
 { 27148,  51, { 27227,  27228,  27229,  27230,  27272,  27273,  27274,  27275}},
 { 27149,  51, { 27228,  27231,  27232,  27229,  27273,  27276,  27277,  27274}},
 { 27150,  51, { 27231,  27233,  27026,  27232,  27276,  27278,  27035,  27277}},
 { 27151,  51, { 27234,  27230,  27235,  27236,  27279,  27275,  27280,  27281}},
 { 27152,  51, { 27230,  27229,  27237,  27235,  27275,  27274,  27282,  27280}},
 { 27153,  51, { 27229,  27232,  27238,  27237,  27274,  27277,  27283,  27282}},
 { 27154,  51, { 27232,  27026,  27025,  27238,  27277,  27035,  27034,  27283}},
 { 27155,  51, { 27236,  27235,  27239,  27240,  27281,  27280,  27284,  27285}},
 { 27156,  51, { 27235,  27237,  27241,  27239,  27280,  27282,  27286,  27284}},
 { 27157,  51, { 27237,  27238,  27242,  27241,  27282,  27283,  27287,  27286}},
 { 27158,  51, { 27238,  27025,  27036,  27242,  27283,  27034,  27038,  27287}},
 { 27159,  51, { 27240,  27239,  27243,  27244,  27285,  27284,  27288,  27289}},
 { 27160,  51, { 27239,  27241,  27245,  27243,  27284,  27286,  27290,  27288}},
 { 27161,  51, { 27241,  27242,  27246,  27245,  27286,  27287,  27291,  27290}},
 { 27162,  51, { 27242,  27036,  27039,  27246,  27287,  27038,  27041,  27291}},
 { 27163,  51, { 27244,  27243,  27247,  27248,  27289,  27288,  27292,  27293}},
 { 27164,  51, { 27243,  27245,  27249,  27247,  27288,  27290,  27294,  27292}},
 { 27165,  51, { 27245,  27246,  27250,  27249,  27290,  27291,  27295,  27294}},
 { 27166,  51, { 27246,  27039,  27251,  27250,  27291,  27041,  27296,  27295}},
 { 27167,  51, { 27248,  27247,  27252,  27253,  27293,  27292,  27297,  27298}},
 { 27168,  51, { 27247,  27249,  27254,  27252,  27292,  27294,  27299,  27297}},
 { 27169,  51, { 27249,  27250,  27255,  27254,  27294,  27295,  27300,  27299}},
 { 27170,  51, { 27250,  27251,  27028,  27255,  27295,  27296,  27043,  27300}},
 { 27171,  51, { 27253,  27252,  27256,  27257,  27298,  27297,  27301,  27302}},
 { 27172,  51, { 27252,  27254,  27258,  27256,  27297,  27299,  27303,  27301}},
 { 27173,  51, { 27254,  27255,  27259,  27258,  27299,  27300,  27304,  27303}},
 { 27174,  51, { 27255,  27028,  27027,  27259,  27300,  27043,  27042,  27304}},
 { 27175,  51, { 27257,  27256,  27260,  27261,  27302,  27301,  27305,  27306}},
 { 27176,  51, { 27256,  27258,  27262,  27260,  27301,  27303,  27307,  27305}},
 { 27177,  51, { 27258,  27259,  27263,  27262,  27303,  27304,  27308,  27307}},
 { 27178,  51, { 27259,  27027,  27031,  27263,  27304,  27042,  27309,  27308}},
 { 27179,  51, { 27261,  27260,  27264,  27265,  27306,  27305,  27310,  27311}},
 { 27180,  51, { 27260,  27262,  27266,  27264,  27305,  27307,  27312,  27310}},
 { 27181,  51, { 27262,  27263,  27267,  27266,  27307,  27308,  27313,  27312}},
 { 27182,  51, { 27263,  27031,  27033,  27267,  27308,  27309,  27045,  27313}},
 { 27183,  51, { 27265,  27264,  27268,  27269,  27311,  27310,  27314,  27315}},
 { 27184,  51, { 27264,  27266,  27270,  27268,  27310,  27312,  27316,  27314}},
 { 27185,  51, { 27266,  27267,  27271,  27270,  27312,  27313,  27317,  27316}},
 { 27186,  51, { 27267,  27033,  27032,  27271,  27313,  27045,  27044,  27317}},
 { 27187,  51, { 27272,  27273,  27274,  27275,  27318,  27319,  27320,  27321}},
 { 27188,  51, { 27273,  27276,  27277,  27274,  27319,  27322,  27323,  27320}},
 { 27189,  51, { 27276,  27278,  27035,  27277,  27322,  27048,  27046,  27323}},
 { 27190,  51, { 27279,  27275,  27280,  27281,  27324,  27321,  27325,  27326}},
 { 27191,  51, { 27275,  27274,  27282,  27280,  27321,  27320,  27327,  27325}},
 { 27192,  51, { 27274,  27277,  27283,  27282,  27320,  27323,  27328,  27327}},
 { 27193,  51, { 27277,  27035,  27034,  27283,  27323,  27046,  27052,  27328}},
 { 27194,  51, { 27281,  27280,  27284,  27285,  27326,  27325,  27329,  27330}},
 { 27195,  51, { 27280,  27282,  27286,  27284,  27325,  27327,  27331,  27329}},
 { 27196,  51, { 27282,  27283,  27287,  27286,  27327,  27328,  27332,  27331}},
 { 27197,  51, { 27283,  27034,  27038,  27287,  27328,  27052,  27050,  27332}},
 { 27198,  51, { 27285,  27284,  27288,  27289,  27330,  27329,  27333,  27334}},
 { 27199,  51, { 27284,  27286,  27290,  27288,  27329,  27331,  27335,  27333}},
 { 27200,  51, { 27286,  27287,  27291,  27290,  27331,  27332,  27336,  27335}},
 { 27201,  51, { 27287,  27038,  27041,  27291,  27332,  27050,  27054,  27336}},
 { 27202,  51, { 27289,  27288,  27292,  27293,  27334,  27333,  27337,  27338}},
 { 27203,  51, { 27288,  27290,  27294,  27292,  27333,  27335,  27339,  27337}},
 { 27204,  51, { 27290,  27291,  27295,  27294,  27335,  27336,  27340,  27339}},
 { 27205,  51, { 27291,  27041,  27296,  27295,  27336,  27054,  27056,  27340}},
 { 27206,  51, { 27293,  27292,  27297,  27298,  27338,  27337,  27341,  27342}},
 { 27207,  51, { 27292,  27294,  27299,  27297,  27337,  27339,  27343,  27341}},
 { 27208,  51, { 27294,  27295,  27300,  27299,  27339,  27340,  27344,  27343}},
 { 27209,  51, { 27295,  27296,  27043,  27300,  27340,  27056,  27058,  27344}},
 { 27210,  51, { 27298,  27297,  27301,  27302,  27342,  27341,  27345,  27346}},
 { 27211,  51, { 27297,  27299,  27303,  27301,  27341,  27343,  27347,  27345}},
 { 27212,  51, { 27299,  27300,  27304,  27303,  27343,  27344,  27348,  27347}},
 { 27213,  51, { 27300,  27043,  27042,  27304,  27344,  27058,  27062,  27348}},
 { 27214,  51, { 27302,  27301,  27305,  27306,  27346,  27345,  27349,  27350}},
 { 27215,  51, { 27301,  27303,  27307,  27305,  27345,  27347,  27351,  27349}},
 { 27216,  51, { 27303,  27304,  27308,  27307,  27347,  27348,  27352,  27351}},
 { 27217,  51, { 27304,  27042,  27309,  27308,  27348,  27062,  27060,  27352}},
 { 27218,  51, { 27306,  27305,  27310,  27311,  27350,  27349,  27353,  27354}},
 { 27219,  51, { 27305,  27307,  27312,  27310,  27349,  27351,  27355,  27353}},
 { 27220,  51, { 27307,  27308,  27313,  27312,  27351,  27352,  27356,  27355}},
 { 27221,  51, { 27308,  27309,  27045,  27313,  27352,  27060,  27064,  27356}},
 { 27222,  51, { 27311,  27310,  27314,  27315,  27354,  27353,  27357,  27358}},
 { 27223,  51, { 27310,  27312,  27316,  27314,  27353,  27355,  27359,  27357}},
 { 27224,  51, { 27312,  27313,  27317,  27316,  27355,  27356,  27360,  27359}},
 { 27225,  51, { 27313,  27045,  27044,  27317,  27356,  27064,  27361,  27360}},
 { 27226,  51, { 27318,  27319,  27320,  27321,  27362,  27363,  27364,  27365}},
 { 27227,  51, { 27319,  27322,  27323,  27320,  27363,  27366,  27367,  27364}},
 { 27228,  51, { 27322,  27048,  27046,  27323,  27366,  27049,  27047,  27367}},
 { 27229,  51, { 27324,  27321,  27325,  27326,  27368,  27365,  27369,  27370}},
 { 27230,  51, { 27321,  27320,  27327,  27325,  27365,  27364,  27371,  27369}},
 { 27231,  51, { 27320,  27323,  27328,  27327,  27364,  27367,  27372,  27371}},
 { 27232,  51, { 27323,  27046,  27052,  27328,  27367,  27047,  27053,  27372}},
 { 27233,  51, { 27326,  27325,  27329,  27330,  27370,  27369,  27373,  27374}},
 { 27234,  51, { 27325,  27327,  27331,  27329,  27369,  27371,  27375,  27373}},
 { 27235,  51, { 27327,  27328,  27332,  27331,  27371,  27372,  27376,  27375}},
 { 27236,  51, { 27328,  27052,  27050,  27332,  27372,  27053,  27051,  27376}},
 { 27237,  51, { 27330,  27329,  27333,  27334,  27374,  27373,  27377,  27378}},
 { 27238,  51, { 27329,  27331,  27335,  27333,  27373,  27375,  27379,  27377}},
 { 27239,  51, { 27331,  27332,  27336,  27335,  27375,  27376,  27380,  27379}},
 { 27240,  51, { 27332,  27050,  27054,  27336,  27376,  27051,  27055,  27380}},
 { 27241,  51, { 27334,  27333,  27337,  27338,  27378,  27377,  27381,  27382}},
 { 27242,  51, { 27333,  27335,  27339,  27337,  27377,  27379,  27383,  27381}},
 { 27243,  51, { 27335,  27336,  27340,  27339,  27379,  27380,  27384,  27383}},
 { 27244,  51, { 27336,  27054,  27056,  27340,  27380,  27055,  27057,  27384}},
 { 27245,  51, { 27338,  27337,  27341,  27342,  27382,  27381,  27385,  27386}},
 { 27246,  51, { 27337,  27339,  27343,  27341,  27381,  27383,  27387,  27385}},
 { 27247,  51, { 27339,  27340,  27344,  27343,  27383,  27384,  27388,  27387}},
 { 27248,  51, { 27340,  27056,  27058,  27344,  27384,  27057,  27059,  27388}},
 { 27249,  51, { 27342,  27341,  27345,  27346,  27386,  27385,  27389,  27390}},
 { 27250,  51, { 27341,  27343,  27347,  27345,  27385,  27387,  27391,  27389}},
 { 27251,  51, { 27343,  27344,  27348,  27347,  27387,  27388,  27392,  27391}},
 { 27252,  51, { 27344,  27058,  27062,  27348,  27388,  27059,  27063,  27392}},
 { 27253,  51, { 27346,  27345,  27349,  27350,  27390,  27389,  27393,  27394}},
 { 27254,  51, { 27345,  27347,  27351,  27349,  27389,  27391,  27395,  27393}},
 { 27255,  51, { 27347,  27348,  27352,  27351,  27391,  27392,  27396,  27395}},
 { 27256,  51, { 27348,  27062,  27060,  27352,  27392,  27063,  27061,  27396}},
 { 27257,  51, { 27350,  27349,  27353,  27354,  27394,  27393,  27397,  27398}},
 { 27258,  51, { 27349,  27351,  27355,  27353,  27393,  27395,  27399,  27397}},
 { 27259,  51, { 27351,  27352,  27356,  27355,  27395,  27396,  27400,  27399}},
 { 27260,  51, { 27352,  27060,  27064,  27356,  27396,  27061,  27065,  27400}},
 { 27261,  51, { 27354,  27353,  27357,  27358,  27398,  27397,  27401,  27402}},
 { 27262,  51, { 27353,  27355,  27359,  27357,  27397,  27399,  27403,  27401}},
 { 27263,  51, { 27355,  27356,  27360,  27359,  27399,  27400,  27404,  27403}},
 { 27264,  51, { 27356,  27064,  27361,  27360,  27400,  27065,  27405,  27404}},
 { 27265,  51, { 27362,  27363,  27364,  27365,  27406,  27407,  27408,  27409}},
 { 27266,  51, { 27363,  27366,  27367,  27364,  27407,  27410,  27411,  27408}},
 { 27267,  51, { 27366,  27049,  27047,  27367,  27410,  27067,  27066,  27411}},
 { 27268,  51, { 27368,  27365,  27369,  27370,  27412,  27409,  27413,  27414}},
 { 27269,  51, { 27365,  27364,  27371,  27369,  27409,  27408,  27415,  27413}},
 { 27270,  51, { 27364,  27367,  27372,  27371,  27408,  27411,  27416,  27415}},
 { 27271,  51, { 27367,  27047,  27053,  27372,  27411,  27066,  27069,  27416}},
 { 27272,  51, { 27370,  27369,  27373,  27374,  27414,  27413,  27417,  27418}},
 { 27273,  51, { 27369,  27371,  27375,  27373,  27413,  27415,  27419,  27417}},
 { 27274,  51, { 27371,  27372,  27376,  27375,  27415,  27416,  27420,  27419}},
 { 27275,  51, { 27372,  27053,  27051,  27376,  27416,  27069,  27068,  27420}},
 { 27276,  51, { 27374,  27373,  27377,  27378,  27418,  27417,  27421,  27422}},
 { 27277,  51, { 27373,  27375,  27379,  27377,  27417,  27419,  27423,  27421}},
 { 27278,  51, { 27375,  27376,  27380,  27379,  27419,  27420,  27424,  27423}},
 { 27279,  51, { 27376,  27051,  27055,  27380,  27420,  27068,  27070,  27424}},
 { 27280,  51, { 27378,  27377,  27381,  27382,  27422,  27421,  27425,  27426}},
 { 27281,  51, { 27377,  27379,  27383,  27381,  27421,  27423,  27427,  27425}},
 { 27282,  51, { 27379,  27380,  27384,  27383,  27423,  27424,  27428,  27427}},
 { 27283,  51, { 27380,  27055,  27057,  27384,  27424,  27070,  27072,  27428}},
 { 27284,  51, { 27382,  27381,  27385,  27386,  27426,  27425,  27429,  27430}},
 { 27285,  51, { 27381,  27383,  27387,  27385,  27425,  27427,  27431,  27429}},
 { 27286,  51, { 27383,  27384,  27388,  27387,  27427,  27428,  27432,  27431}},
 { 27287,  51, { 27384,  27057,  27059,  27388,  27428,  27072,  27071,  27432}},
 { 27288,  51, { 27386,  27385,  27389,  27390,  27430,  27429,  27433,  27434}},
 { 27289,  51, { 27385,  27387,  27391,  27389,  27429,  27431,  27435,  27433}},
 { 27290,  51, { 27387,  27388,  27392,  27391,  27431,  27432,  27436,  27435}},
 { 27291,  51, { 27388,  27059,  27063,  27392,  27432,  27071,  27081,  27436}},
 { 27292,  51, { 27390,  27389,  27393,  27394,  27434,  27433,  27437,  27438}},
 { 27293,  51, { 27389,  27391,  27395,  27393,  27433,  27435,  27439,  27437}},
 { 27294,  51, { 27391,  27392,  27396,  27395,  27435,  27436,  27440,  27439}},
 { 27295,  51, { 27392,  27063,  27061,  27396,  27436,  27081,  27074,  27440}},
 { 27296,  51, { 27394,  27393,  27397,  27398,  27438,  27437,  27441,  27442}},
 { 27297,  51, { 27393,  27395,  27399,  27397,  27437,  27439,  27443,  27441}},
 { 27298,  51, { 27395,  27396,  27400,  27399,  27439,  27440,  27444,  27443}},
 { 27299,  51, { 27396,  27061,  27065,  27400,  27440,  27074,  27073,  27444}},
 { 27300,  51, { 27398,  27397,  27401,  27402,  27442,  27441,  27445,  27446}},
 { 27301,  51, { 27397,  27399,  27403,  27401,  27441,  27443,  27447,  27445}},
 { 27302,  51, { 27399,  27400,  27404,  27403,  27443,  27444,  27448,  27447}},
 { 27303,  51, { 27400,  27065,  27405,  27404,  27444,  27073,  27449,  27448}},
 { 27304,  51, { 27406,  27407,  27408,  27409,  27450,  27451,  27452,  27453}},
 { 27305,  51, { 27407,  27410,  27411,  27408,  27451,  27454,  27455,  27452}},
 { 27306,  51, { 27410,  27067,  27066,  27411,  27454,  27456,  27076,  27455}},
 { 27307,  51, { 27412,  27409,  27413,  27414,  27457,  27453,  27458,  27459}},
 { 27308,  51, { 27409,  27408,  27415,  27413,  27453,  27452,  27460,  27458}},
 { 27309,  51, { 27408,  27411,  27416,  27415,  27452,  27455,  27461,  27460}},
 { 27310,  51, { 27411,  27066,  27069,  27416,  27455,  27076,  27075,  27461}},
 { 27311,  51, { 27414,  27413,  27417,  27418,  27459,  27458,  27462,  27463}},
 { 27312,  51, { 27413,  27415,  27419,  27417,  27458,  27460,  27464,  27462}},
 { 27313,  51, { 27415,  27416,  27420,  27419,  27460,  27461,  27465,  27464}},
 { 27314,  51, { 27416,  27069,  27068,  27420,  27461,  27075,  27077,  27465}},
 { 27315,  51, { 27418,  27417,  27421,  27422,  27463,  27462,  27466,  27467}},
 { 27316,  51, { 27417,  27419,  27423,  27421,  27462,  27464,  27468,  27466}},
 { 27317,  51, { 27419,  27420,  27424,  27423,  27464,  27465,  27469,  27468}},
 { 27318,  51, { 27420,  27068,  27070,  27424,  27465,  27077,  27078,  27469}},
 { 27319,  51, { 27422,  27421,  27425,  27426,  27467,  27466,  27470,  27471}},
 { 27320,  51, { 27421,  27423,  27427,  27425,  27466,  27468,  27472,  27470}},
 { 27321,  51, { 27423,  27424,  27428,  27427,  27468,  27469,  27473,  27472}},
 { 27322,  51, { 27424,  27070,  27072,  27428,  27469,  27078,  27079,  27473}},
 { 27323,  51, { 27426,  27425,  27429,  27430,  27471,  27470,  27474,  27475}},
 { 27324,  51, { 27425,  27427,  27431,  27429,  27470,  27472,  27476,  27474}},
 { 27325,  51, { 27427,  27428,  27432,  27431,  27472,  27473,  27477,  27476}},
 { 27326,  51, { 27428,  27072,  27071,  27432,  27473,  27079,  27478,  27477}},
 { 27327,  51, { 27430,  27429,  27433,  27434,  27475,  27474,  27479,  27480}},
 { 27328,  51, { 27429,  27431,  27435,  27433,  27474,  27476,  27481,  27479}},
 { 27329,  51, { 27431,  27432,  27436,  27435,  27476,  27477,  27482,  27481}},
 { 27330,  51, { 27432,  27071,  27081,  27436,  27477,  27478,  27082,  27482}},
 { 27331,  51, { 27434,  27433,  27437,  27438,  27480,  27479,  27483,  27484}},
 { 27332,  51, { 27433,  27435,  27439,  27437,  27479,  27481,  27485,  27483}},
 { 27333,  51, { 27435,  27436,  27440,  27439,  27481,  27482,  27486,  27485}},
 { 27334,  51, { 27436,  27081,  27074,  27440,  27482,  27082,  27080,  27486}},
 { 27335,  51, { 27438,  27437,  27441,  27442,  27484,  27483,  27487,  27488}},
 { 27336,  51, { 27437,  27439,  27443,  27441,  27483,  27485,  27489,  27487}},
 { 27337,  51, { 27439,  27440,  27444,  27443,  27485,  27486,  27490,  27489}},
 { 27338,  51, { 27440,  27074,  27073,  27444,  27486,  27080,  27491,  27490}},
 { 27339,  51, { 27442,  27441,  27445,  27446,  27488,  27487,  27492,  27493}},
 { 27340,  51, { 27441,  27443,  27447,  27445,  27487,  27489,  27494,  27492}},
 { 27341,  51, { 27443,  27444,  27448,  27447,  27489,  27490,  27495,  27494}},
 { 27342,  51, { 27444,  27073,  27449,  27448,  27490,  27491,  27496,  27495}},
 { 27343,  51, { 27001,  27496,  27497,  27498,  27499,  27500,  27501,  27502}},
 { 27344,  51, { 27492,  27493,  27503,  27504,  27505,  27506,  27507,  27508}},
 { 27345,  51, { 27494,  27492,  27504,  27509,  27510,  27505,  27508,  27511}},
 { 27346,  51, { 27495,  27494,  27509,  27512,  27513,  27510,  27511,  27514}},
 { 27347,  51, { 27496,  27495,  27512,  27497,  27500,  27513,  27514,  27501}},
 { 27348,  51, { 27498,  27497,  27515,  27516,  27502,  27501,  27517,  27518}},
 { 27349,  51, { 27504,  27503,  27519,  27520,  27508,  27507,  27521,  27522}},
 { 27350,  51, { 27509,  27504,  27520,  27523,  27511,  27508,  27522,  27524}},
 { 27351,  51, { 27512,  27509,  27523,  27525,  27514,  27511,  27524,  27526}},
 { 27352,  51, { 27497,  27512,  27525,  27515,  27501,  27514,  27526,  27517}},
 { 27353,  51, { 27516,  27515,  27527,  27528,  27518,  27517,  27529,  27530}},
 { 27354,  51, { 27520,  27519,  27531,  27532,  27522,  27521,  27533,  27534}},
 { 27355,  51, { 27523,  27520,  27532,  27535,  27524,  27522,  27534,  27536}},
 { 27356,  51, { 27525,  27523,  27535,  27537,  27526,  27524,  27536,  27538}},
 { 27357,  51, { 27515,  27525,  27537,  27527,  27517,  27526,  27538,  27529}},
 { 27358,  51, { 27528,  27527,  27539,  27540,  27530,  27529,  27541,  27542}},
 { 27359,  51, { 27532,  27531,  27543,  27544,  27534,  27533,  27545,  27546}},
 { 27360,  51, { 27535,  27532,  27544,  27547,  27536,  27534,  27546,  27548}},
 { 27361,  51, { 27537,  27535,  27547,  27549,  27538,  27536,  27548,  27550}},
 { 27362,  51, { 27527,  27537,  27549,  27539,  27529,  27538,  27550,  27541}},
 { 27363,  51, { 27001,  27496,  27491,  27004,  27498,  27497,  27551,  27552}},
 { 27364,  51, { 27004,  27491,  27080,  27030,  27552,  27551,  27553,  27554}},
 { 27365,  51, { 27030,  27080,  27082,  27019,  27554,  27553,  27555,  27556}},
 { 27366,  51, { 27019,  27082,  27478,  27013,  27556,  27555,  27557,  27558}},
 { 27367,  51, { 27013,  27478,  27079,  27016,  27558,  27557,  27559,  27560}},
 { 27368,  51, { 27016,  27079,  27078,  27040,  27560,  27559,  27561,  27562}},
 { 27369,  51, { 27040,  27078,  27077,  27037,  27562,  27561,  27563,  27564}},
 { 27370,  51, { 27037,  27077,  27075,  27024,  27564,  27563,  27565,  27566}},
 { 27371,  51, { 27024,  27075,  27076,  27007,  27566,  27565,  27567,  27568}},
 { 27372,  51, { 27007,  27076,  27456,  27010,  27568,  27567,  27569,  27570}},
 { 27373,  51, { 27459,  27457,  27453,  27458,  27571,  27572,  27573,  27574}},
 { 27374,  51, { 27463,  27459,  27458,  27462,  27575,  27571,  27574,  27576}},
 { 27375,  51, { 27467,  27463,  27462,  27466,  27577,  27575,  27576,  27578}},
 { 27376,  51, { 27471,  27467,  27466,  27470,  27579,  27577,  27578,  27580}},
 { 27377,  51, { 27475,  27471,  27470,  27474,  27581,  27579,  27580,  27582}},
 { 27378,  51, { 27480,  27475,  27474,  27479,  27583,  27581,  27582,  27584}},
 { 27379,  51, { 27484,  27480,  27479,  27483,  27585,  27583,  27584,  27586}},
 { 27380,  51, { 27488,  27484,  27483,  27487,  27587,  27585,  27586,  27588}},
 { 27381,  51, { 27493,  27488,  27487,  27492,  27503,  27587,  27588,  27504}},
 { 27382,  51, { 27453,  27450,  27451,  27452,  27573,  27589,  27590,  27591}},
 { 27383,  51, { 27452,  27451,  27454,  27455,  27591,  27590,  27592,  27593}},
 { 27384,  51, { 27455,  27454,  27456,  27076,  27593,  27592,  27569,  27567}},
 { 27385,  51, { 27458,  27453,  27452,  27460,  27574,  27573,  27591,  27594}},
 { 27386,  51, { 27460,  27452,  27455,  27461,  27594,  27591,  27593,  27595}},
 { 27387,  51, { 27461,  27455,  27076,  27075,  27595,  27593,  27567,  27565}},
 { 27388,  51, { 27462,  27458,  27460,  27464,  27576,  27574,  27594,  27596}},
 { 27389,  51, { 27464,  27460,  27461,  27465,  27596,  27594,  27595,  27597}},
 { 27390,  51, { 27465,  27461,  27075,  27077,  27597,  27595,  27565,  27563}},
 { 27391,  51, { 27466,  27462,  27464,  27468,  27578,  27576,  27596,  27598}},
 { 27392,  51, { 27468,  27464,  27465,  27469,  27598,  27596,  27597,  27599}},
 { 27393,  51, { 27469,  27465,  27077,  27078,  27599,  27597,  27563,  27561}},
 { 27394,  51, { 27470,  27466,  27468,  27472,  27580,  27578,  27598,  27600}},
 { 27395,  51, { 27472,  27468,  27469,  27473,  27600,  27598,  27599,  27601}},
 { 27396,  51, { 27473,  27469,  27078,  27079,  27601,  27599,  27561,  27559}},
 { 27397,  51, { 27474,  27470,  27472,  27476,  27582,  27580,  27600,  27602}},
 { 27398,  51, { 27476,  27472,  27473,  27477,  27602,  27600,  27601,  27603}},
 { 27399,  51, { 27477,  27473,  27079,  27478,  27603,  27601,  27559,  27557}},
 { 27400,  51, { 27479,  27474,  27476,  27481,  27584,  27582,  27602,  27604}},
 { 27401,  51, { 27481,  27476,  27477,  27482,  27604,  27602,  27603,  27605}},
 { 27402,  51, { 27482,  27477,  27478,  27082,  27605,  27603,  27557,  27555}},
 { 27403,  51, { 27483,  27479,  27481,  27485,  27586,  27584,  27604,  27606}},
 { 27404,  51, { 27485,  27481,  27482,  27486,  27606,  27604,  27605,  27607}},
 { 27405,  51, { 27486,  27482,  27082,  27080,  27607,  27605,  27555,  27553}},
 { 27406,  51, { 27487,  27483,  27485,  27489,  27588,  27586,  27606,  27608}},
 { 27407,  51, { 27489,  27485,  27486,  27490,  27608,  27606,  27607,  27609}},
 { 27408,  51, { 27490,  27486,  27080,  27491,  27609,  27607,  27553,  27551}},
 { 27409,  51, { 27492,  27487,  27489,  27494,  27504,  27588,  27608,  27509}},
 { 27410,  51, { 27494,  27489,  27490,  27495,  27509,  27608,  27609,  27512}},
 { 27411,  51, { 27495,  27490,  27491,  27496,  27512,  27609,  27551,  27497}},
 { 27412,  51, { 27498,  27497,  27551,  27552,  27516,  27515,  27610,  27611}},
 { 27413,  51, { 27552,  27551,  27553,  27554,  27611,  27610,  27612,  27613}},
 { 27414,  51, { 27554,  27553,  27555,  27556,  27613,  27612,  27614,  27615}},
 { 27415,  51, { 27556,  27555,  27557,  27558,  27615,  27614,  27616,  27617}},
 { 27416,  51, { 27558,  27557,  27559,  27560,  27617,  27616,  27618,  27619}},
 { 27417,  51, { 27560,  27559,  27561,  27562,  27619,  27618,  27620,  27621}},
 { 27418,  51, { 27562,  27561,  27563,  27564,  27621,  27620,  27622,  27623}},
 { 27419,  51, { 27564,  27563,  27565,  27566,  27623,  27622,  27624,  27625}},
 { 27420,  51, { 27566,  27565,  27567,  27568,  27625,  27624,  27626,  27627}},
 { 27421,  51, { 27568,  27567,  27569,  27570,  27627,  27626,  27628,  27629}},
 { 27422,  51, { 27571,  27572,  27573,  27574,  27630,  27631,  27632,  27633}},
 { 27423,  51, { 27575,  27571,  27574,  27576,  27634,  27630,  27633,  27635}},
 { 27424,  51, { 27577,  27575,  27576,  27578,  27636,  27634,  27635,  27637}},
 { 27425,  51, { 27579,  27577,  27578,  27580,  27638,  27636,  27637,  27639}},
 { 27426,  51, { 27581,  27579,  27580,  27582,  27640,  27638,  27639,  27641}},
 { 27427,  51, { 27583,  27581,  27582,  27584,  27642,  27640,  27641,  27643}},
 { 27428,  51, { 27585,  27583,  27584,  27586,  27644,  27642,  27643,  27645}},
 { 27429,  51, { 27587,  27585,  27586,  27588,  27646,  27644,  27645,  27647}},
 { 27430,  51, { 27503,  27587,  27588,  27504,  27519,  27646,  27647,  27520}},
 { 27431,  51, { 27573,  27589,  27590,  27591,  27632,  27648,  27649,  27650}},
 { 27432,  51, { 27591,  27590,  27592,  27593,  27650,  27649,  27651,  27652}},
 { 27433,  51, { 27593,  27592,  27569,  27567,  27652,  27651,  27628,  27626}},
 { 27434,  51, { 27574,  27573,  27591,  27594,  27633,  27632,  27650,  27653}},
 { 27435,  51, { 27594,  27591,  27593,  27595,  27653,  27650,  27652,  27654}},
 { 27436,  51, { 27595,  27593,  27567,  27565,  27654,  27652,  27626,  27624}},
 { 27437,  51, { 27576,  27574,  27594,  27596,  27635,  27633,  27653,  27655}},
 { 27438,  51, { 27596,  27594,  27595,  27597,  27655,  27653,  27654,  27656}},
 { 27439,  51, { 27597,  27595,  27565,  27563,  27656,  27654,  27624,  27622}},
 { 27440,  51, { 27578,  27576,  27596,  27598,  27637,  27635,  27655,  27657}},
 { 27441,  51, { 27598,  27596,  27597,  27599,  27657,  27655,  27656,  27658}},
 { 27442,  51, { 27599,  27597,  27563,  27561,  27658,  27656,  27622,  27620}},
 { 27443,  51, { 27580,  27578,  27598,  27600,  27639,  27637,  27657,  27659}},
 { 27444,  51, { 27600,  27598,  27599,  27601,  27659,  27657,  27658,  27660}},
 { 27445,  51, { 27601,  27599,  27561,  27559,  27660,  27658,  27620,  27618}},
 { 27446,  51, { 27582,  27580,  27600,  27602,  27641,  27639,  27659,  27661}},
 { 27447,  51, { 27602,  27600,  27601,  27603,  27661,  27659,  27660,  27662}},
 { 27448,  51, { 27603,  27601,  27559,  27557,  27662,  27660,  27618,  27616}},
 { 27449,  51, { 27584,  27582,  27602,  27604,  27643,  27641,  27661,  27663}},
 { 27450,  51, { 27604,  27602,  27603,  27605,  27663,  27661,  27662,  27664}},
 { 27451,  51, { 27605,  27603,  27557,  27555,  27664,  27662,  27616,  27614}},
 { 27452,  51, { 27586,  27584,  27604,  27606,  27645,  27643,  27663,  27665}},
 { 27453,  51, { 27606,  27604,  27605,  27607,  27665,  27663,  27664,  27666}},
 { 27454,  51, { 27607,  27605,  27555,  27553,  27666,  27664,  27614,  27612}},
 { 27455,  51, { 27588,  27586,  27606,  27608,  27647,  27645,  27665,  27667}},
 { 27456,  51, { 27608,  27606,  27607,  27609,  27667,  27665,  27666,  27668}},
 { 27457,  51, { 27609,  27607,  27553,  27551,  27668,  27666,  27612,  27610}},
 { 27458,  51, { 27504,  27588,  27608,  27509,  27520,  27647,  27667,  27523}},
 { 27459,  51, { 27509,  27608,  27609,  27512,  27523,  27667,  27668,  27525}},
 { 27460,  51, { 27512,  27609,  27551,  27497,  27525,  27668,  27610,  27515}},
 { 27461,  51, { 27516,  27515,  27610,  27611,  27528,  27527,  27669,  27670}},
 { 27462,  51, { 27611,  27610,  27612,  27613,  27670,  27669,  27671,  27672}},
 { 27463,  51, { 27613,  27612,  27614,  27615,  27672,  27671,  27673,  27674}},
 { 27464,  51, { 27615,  27614,  27616,  27617,  27674,  27673,  27675,  27676}},
 { 27465,  51, { 27617,  27616,  27618,  27619,  27676,  27675,  27677,  27678}},
 { 27466,  51, { 27619,  27618,  27620,  27621,  27678,  27677,  27679,  27680}},
 { 27467,  51, { 27621,  27620,  27622,  27623,  27680,  27679,  27681,  27682}},
 { 27468,  51, { 27623,  27622,  27624,  27625,  27682,  27681,  27683,  27684}},
 { 27469,  51, { 27625,  27624,  27626,  27627,  27684,  27683,  27685,  27686}},
 { 27470,  51, { 27627,  27626,  27628,  27629,  27686,  27685,  27687,  27688}},
 { 27471,  51, { 27630,  27631,  27632,  27633,  27689,  27690,  27691,  27692}},
 { 27472,  51, { 27634,  27630,  27633,  27635,  27693,  27689,  27692,  27694}},
 { 27473,  51, { 27636,  27634,  27635,  27637,  27695,  27693,  27694,  27696}},
 { 27474,  51, { 27638,  27636,  27637,  27639,  27697,  27695,  27696,  27698}},
 { 27475,  51, { 27640,  27638,  27639,  27641,  27699,  27697,  27698,  27700}},
 { 27476,  51, { 27642,  27640,  27641,  27643,  27701,  27699,  27700,  27702}},
 { 27477,  51, { 27644,  27642,  27643,  27645,  27703,  27701,  27702,  27704}},
 { 27478,  51, { 27646,  27644,  27645,  27647,  27705,  27703,  27704,  27706}},
 { 27479,  51, { 27519,  27646,  27647,  27520,  27531,  27705,  27706,  27532}},
 { 27480,  51, { 27632,  27648,  27649,  27650,  27691,  27707,  27708,  27709}},
 { 27481,  51, { 27650,  27649,  27651,  27652,  27709,  27708,  27710,  27711}},
 { 27482,  51, { 27652,  27651,  27628,  27626,  27711,  27710,  27687,  27685}},
 { 27483,  51, { 27633,  27632,  27650,  27653,  27692,  27691,  27709,  27712}},
 { 27484,  51, { 27653,  27650,  27652,  27654,  27712,  27709,  27711,  27713}},
 { 27485,  51, { 27654,  27652,  27626,  27624,  27713,  27711,  27685,  27683}},
 { 27486,  51, { 27635,  27633,  27653,  27655,  27694,  27692,  27712,  27714}},
 { 27487,  51, { 27655,  27653,  27654,  27656,  27714,  27712,  27713,  27715}},
 { 27488,  51, { 27656,  27654,  27624,  27622,  27715,  27713,  27683,  27681}},
 { 27489,  51, { 27637,  27635,  27655,  27657,  27696,  27694,  27714,  27716}},
 { 27490,  51, { 27657,  27655,  27656,  27658,  27716,  27714,  27715,  27717}},
 { 27491,  51, { 27658,  27656,  27622,  27620,  27717,  27715,  27681,  27679}},
 { 27492,  51, { 27639,  27637,  27657,  27659,  27698,  27696,  27716,  27718}},
 { 27493,  51, { 27659,  27657,  27658,  27660,  27718,  27716,  27717,  27719}},
 { 27494,  51, { 27660,  27658,  27620,  27618,  27719,  27717,  27679,  27677}},
 { 27495,  51, { 27641,  27639,  27659,  27661,  27700,  27698,  27718,  27720}},
 { 27496,  51, { 27661,  27659,  27660,  27662,  27720,  27718,  27719,  27721}},
 { 27497,  51, { 27662,  27660,  27618,  27616,  27721,  27719,  27677,  27675}},
 { 27498,  51, { 27643,  27641,  27661,  27663,  27702,  27700,  27720,  27722}},
 { 27499,  51, { 27663,  27661,  27662,  27664,  27722,  27720,  27721,  27723}},
 { 27500,  51, { 27664,  27662,  27616,  27614,  27723,  27721,  27675,  27673}},
 { 27501,  51, { 27645,  27643,  27663,  27665,  27704,  27702,  27722,  27724}},
 { 27502,  51, { 27665,  27663,  27664,  27666,  27724,  27722,  27723,  27725}},
 { 27503,  51, { 27666,  27664,  27614,  27612,  27725,  27723,  27673,  27671}},
 { 27504,  51, { 27647,  27645,  27665,  27667,  27706,  27704,  27724,  27726}},
 { 27505,  51, { 27667,  27665,  27666,  27668,  27726,  27724,  27725,  27727}},
 { 27506,  51, { 27668,  27666,  27612,  27610,  27727,  27725,  27671,  27669}},
 { 27507,  51, { 27520,  27647,  27667,  27523,  27532,  27706,  27726,  27535}},
 { 27508,  51, { 27523,  27667,  27668,  27525,  27535,  27726,  27727,  27537}},
 { 27509,  51, { 27525,  27668,  27610,  27515,  27537,  27727,  27669,  27527}},
 { 27510,  51, { 27528,  27527,  27669,  27670,  27540,  27539,  27728,  27729}},
 { 27511,  51, { 27670,  27669,  27671,  27672,  27729,  27728,  27730,  27731}},
 { 27512,  51, { 27672,  27671,  27673,  27674,  27731,  27730,  27732,  27733}},
 { 27513,  51, { 27674,  27673,  27675,  27676,  27733,  27732,  27734,  27735}},
 { 27514,  51, { 27676,  27675,  27677,  27678,  27735,  27734,  27736,  27737}},
 { 27515,  51, { 27678,  27677,  27679,  27680,  27737,  27736,  27738,  27739}},
 { 27516,  51, { 27680,  27679,  27681,  27682,  27739,  27738,  27740,  27741}},
 { 27517,  51, { 27682,  27681,  27683,  27684,  27741,  27740,  27742,  27743}},
 { 27518,  51, { 27684,  27683,  27685,  27686,  27743,  27742,  27744,  27745}},
 { 27519,  51, { 27686,  27685,  27687,  27688,  27745,  27744,  27746,  27747}},
 { 27520,  51, { 27689,  27690,  27691,  27692,  27748,  27749,  27750,  27751}},
 { 27521,  51, { 27693,  27689,  27692,  27694,  27752,  27748,  27751,  27753}},
 { 27522,  51, { 27695,  27693,  27694,  27696,  27754,  27752,  27753,  27755}},
 { 27523,  51, { 27697,  27695,  27696,  27698,  27756,  27754,  27755,  27757}},
 { 27524,  51, { 27699,  27697,  27698,  27700,  27758,  27756,  27757,  27759}},
 { 27525,  51, { 27701,  27699,  27700,  27702,  27760,  27758,  27759,  27761}},
 { 27526,  51, { 27703,  27701,  27702,  27704,  27762,  27760,  27761,  27763}},
 { 27527,  51, { 27705,  27703,  27704,  27706,  27764,  27762,  27763,  27765}},
 { 27528,  51, { 27531,  27705,  27706,  27532,  27543,  27764,  27765,  27544}},
 { 27529,  51, { 27691,  27707,  27708,  27709,  27750,  27766,  27767,  27768}},
 { 27530,  51, { 27709,  27708,  27710,  27711,  27768,  27767,  27769,  27770}},
 { 27531,  51, { 27711,  27710,  27687,  27685,  27770,  27769,  27746,  27744}},
 { 27532,  51, { 27692,  27691,  27709,  27712,  27751,  27750,  27768,  27771}},
 { 27533,  51, { 27712,  27709,  27711,  27713,  27771,  27768,  27770,  27772}},
 { 27534,  51, { 27713,  27711,  27685,  27683,  27772,  27770,  27744,  27742}},
 { 27535,  51, { 27694,  27692,  27712,  27714,  27753,  27751,  27771,  27773}},
 { 27536,  51, { 27714,  27712,  27713,  27715,  27773,  27771,  27772,  27774}},
 { 27537,  51, { 27715,  27713,  27683,  27681,  27774,  27772,  27742,  27740}},
 { 27538,  51, { 27696,  27694,  27714,  27716,  27755,  27753,  27773,  27775}},
 { 27539,  51, { 27716,  27714,  27715,  27717,  27775,  27773,  27774,  27776}},
 { 27540,  51, { 27717,  27715,  27681,  27679,  27776,  27774,  27740,  27738}},
 { 27541,  51, { 27698,  27696,  27716,  27718,  27757,  27755,  27775,  27777}},
 { 27542,  51, { 27718,  27716,  27717,  27719,  27777,  27775,  27776,  27778}},
 { 27543,  51, { 27719,  27717,  27679,  27677,  27778,  27776,  27738,  27736}},
 { 27544,  51, { 27700,  27698,  27718,  27720,  27759,  27757,  27777,  27779}},
 { 27545,  51, { 27720,  27718,  27719,  27721,  27779,  27777,  27778,  27780}},
 { 27546,  51, { 27721,  27719,  27677,  27675,  27780,  27778,  27736,  27734}},
 { 27547,  51, { 27702,  27700,  27720,  27722,  27761,  27759,  27779,  27781}},
 { 27548,  51, { 27722,  27720,  27721,  27723,  27781,  27779,  27780,  27782}},
 { 27549,  51, { 27723,  27721,  27675,  27673,  27782,  27780,  27734,  27732}},
 { 27550,  51, { 27704,  27702,  27722,  27724,  27763,  27761,  27781,  27783}},
 { 27551,  51, { 27724,  27722,  27723,  27725,  27783,  27781,  27782,  27784}},
 { 27552,  51, { 27725,  27723,  27673,  27671,  27784,  27782,  27732,  27730}},
 { 27553,  51, { 27706,  27704,  27724,  27726,  27765,  27763,  27783,  27785}},
 { 27554,  51, { 27726,  27724,  27725,  27727,  27785,  27783,  27784,  27786}},
 { 27555,  51, { 27727,  27725,  27671,  27669,  27786,  27784,  27730,  27728}},
 { 27556,  51, { 27532,  27706,  27726,  27535,  27544,  27765,  27785,  27547}},
 { 27557,  51, { 27535,  27726,  27727,  27537,  27547,  27785,  27786,  27549}},
 { 27558,  51, { 27537,  27727,  27669,  27527,  27549,  27786,  27728,  27539}},
 { 27559,  51, { 27095,  27010,  27007,  27096,  27009,  27009,  27006,  27006}},
 { 27560,  51, { 27011,  27010,  27007,  27008,  27233,  27233,  27026,  27026}},
 { 27561,  51, { 27278,  27233,  27026,  27035,  27010,  27010,  27007,  27007}},
 { 27562,  51, { 27278,  27010,  27007,  27035,  27048,  27048,  27046,  27046}},
 { 27563,  51, { 27010,  27456,  27076,  27007,  27067,  27067,  27066,  27066}},
 { 27564,  51, { 27007,  27066,  27069,  27024,  27047,  27047,  27053,  27053}},
 { 27565,  51, { 27007,  27047,  27053,  27024,  27046,  27046,  27052,  27052}},
 { 27566,  51, { 27035,  27007,  27024,  27034,  27046,  27046,  27052,  27052}},
 { 27567,  51, { 27108,  27023,  27008,  27006,  27024,  27024,  27007,  27007}},
 { 27568,  51, { 27007,  27006,  27108,  27024,  27096,  27096,  27107,  27107}},
 { 27569,  51, { 27024,  27052,  27050,  27037,  27034,  27034,  27038,  27038}},
 { 27570,  51, { 27197,  27036,  27025,  27023,  27037,  27037,  27024,  27024}},
 { 27571,  51, { 27024,  27023,  27197,  27037,  27108,  27108,  27118,  27118}},
 { 27572,  51, { 27024,  27108,  27118,  27037,  27107,  27107,  27117,  27117}},
 { 27573,  51, { 27037,  27050,  27054,  27040,  27038,  27038,  27041,  27041}},
 { 27574,  51, { 27037,  27036,  27039,  27040,  27197,  27197,  27202,  27202}},
 { 27575,  51, { 27202,  27040,  27037,  27197,  27128,  27128,  27118,  27118}},
 { 27576,  51, { 27128,  27040,  27037,  27118,  27127,  27127,  27117,  27117}},
 { 27577,  51, { 27057,  27072,  27070,  27055,  27016,  27016,  27040,  27040}},
 { 27578,  51, { 27040,  27054,  27056,  27016,  27041,  27041,  27296,  27296}},
 { 27579,  51, { 27128,  27040,  27016,  27015,  27202,  27202,  27017,  27017}},
 { 27580,  51, { 27015,  27016,  27040,  27128,  27137,  27137,  27127,  27127}},
 { 27581,  51, { 27296,  27016,  27040,  27041,  27251,  27251,  27039,  27039}},
 { 27582,  51, { 27017,  27251,  27039,  27202,  27016,  27016,  27040,  27040}},
 { 27583,  51, { 27079,  27072,  27071,  27478,  27016,  27016,  27013,  27013}},
 { 27584,  51, { 27013,  27043,  27296,  27016,  27058,  27058,  27056,  27056}},
 { 27585,  51, { 27016,  27296,  27043,  27013,  27251,  27251,  27028,  27028}},
 { 27586,  51, { 27017,  27016,  27013,  27014,  27251,  27251,  27028,  27028}},
 { 27587,  51, { 27013,  27146,  27137,  27016,  27012,  27012,  27015,  27015}},
 { 27588,  51, { 27082,  27019,  27013,  27478,  27081,  27081,  27071,  27071}},
 { 27589,  51, { 27071,  27059,  27063,  27081,  27013,  27013,  27019,  27019}},
 { 27590,  51, { 27058,  27043,  27042,  27062,  27013,  27013,  27019,  27019}},
 { 27591,  51, { 27059,  27058,  27062,  27063,  27013,  27013,  27019,  27019}},
 { 27592,  51, { 27155,  27018,  27012,  27146,  27019,  27019,  27013,  27013}},
 { 27593,  51, { 27787,  27707,  27691,  27690,  27788,  27766,  27750,  27749}},
 { 27594,  51, { 27789,  27648,  27632,  27631,  27787,  27707,  27691,  27690}},
 { 27595,  51, { 27572,  27790,  27589,  27573,  27631,  27789,  27648,  27632}},
 { 27596,  51, { 27791,  27450,  27453,  27457,  27790,  27589,  27573,  27572}},
 { 27597,  51, { 27792,  27406,  27409,  27412,  27791,  27450,  27453,  27457}},
 { 27598,  51, { 27368,  27793,  27362,  27365,  27412,  27792,  27406,  27409}},
 { 27599,  51, { 27794,  27793,  27362,  27318,  27324,  27368,  27365,  27321}},
 { 27600,  51, { 27272,  27275,  27279,  27795,  27318,  27321,  27324,  27794}},
 { 27601,  51, { 27227,  27796,  27795,  27272,  27230,  27234,  27279,  27275}},
 { 27602,  51, { 27234,  27188,  27185,  27230,  27796,  27797,  27182,  27227}},
 { 27603,  51, { 27182,  27797,  27188,  27185,  27087,  27798,  27100,  27090}},
 { 27604,  51, { 27086,  27090,  27100,  27097,  27083,  27087,  27798,  27799}},
 { 27605,  51, { 27155,  27019,  27030,  27164,  27018,  27018,  27165,  27165}},
 { 27606,  51, { 27074,  27030,  27004,  27073,  27080,  27080,  27491,  27491}},
 { 27607,  51, { 27061,  27074,  27081,  27063,  27030,  27030,  27019,  27019}},
 { 27608,  51, { 27004,  27073,  27449,  27001,  27065,  27065,  27405,  27405}},
 { 27609,  51, { 27496,  27001,  27004,  27491,  27449,  27449,  27073,  27073}},
 { 27610,  51, { 27030,  27309,  27042,  27019,  27060,  27060,  27062,  27062}},
 { 27611,  51, { 27030,  27031,  27027,  27019,  27309,  27309,  27042,  27042}},
 { 27612,  51, { 27030,  27165,  27018,  27019,  27029,  27029,  27020,  27020}},
 { 27613,  51, { 27030,  27165,  27005,  27004,  27164,  27164,  27003,  27003}},
 { 27614,  51, { 27030,  27060,  27064,  27004,  27309,  27309,  27045,  27045}},
 { 27615,  51, { 27309,  27031,  27033,  27045,  27030,  27030,  27004,  27004}},
 { 27616,  51, { 27030,  27031,  27033,  27004,  27029,  27029,  27022,  27022}},
 { 27617,  51, { 27030,  27029,  27022,  27004,  27165,  27165,  27005,  27005}},
 { 27618,  51, { 27004,  27065,  27405,  27001,  27064,  27064,  27361,  27361}},
 { 27619,  51, { 27004,  27064,  27361,  27001,  27045,  27045,  27044,  27044}},
 { 27620,  51, { 27801,  27800,  27803,  27804,  27802,  27802,  27805,  27805}},
 { 27621,  51, { 27807,  27806,  27809,  27810,  27808,  27808,  27811,  27811}},
 { 27622,  51, { 27813,  27812,  27815,  27816,  27814,  27814,  27817,  27817}},
 { 27623,  51, { 27819,  27818,  27812,  27813,  27820,  27820,  27814,  27814}},
 { 27624,  51, { 27821,  27801,  27804,  27822,  27802,  27802,  27805,  27805}},
 { 27625,  51, { 27824,  27823,  27807,  27826,  27825,  27825,  27808,  27808}},
 { 27626,  51, { 27827,  27819,  27813,  27828,  27820,  27820,  27814,  27814}},
 { 27627,  51, { 27830,  27829,  27819,  27827,  27831,  27831,  27820,  27820}},
 { 27628,  51, { 27832,  27821,  27822,  27833,  27802,  27802,  27805,  27805}},
 { 27629,  51, { 27834,  27824,  27826,  27835,  27825,  27825,  27808,  27808}},
 { 27630,  51, { 27837,  27836,  27824,  27834,  27838,  27838,  27825,  27825}},
 { 27631,  51, { 27840,  27839,  27836,  27837,  27841,  27841,  27838,  27838}},
 { 27632,  51, { 27842,  27827,  27828,  27843,  27820,  27820,  27814,  27814}},
 { 27633,  51, { 27844,  27832,  27833,  27845,  27802,  27802,  27805,  27805}},
 { 27634,  51, { 27847,  27846,  27848,  27849,  27808,  27808,  27811,  27811}},
 { 27635,  51, { 27851,  27850,  27852,  27853,  27838,  27838,  27825,  27825}},
 { 27636,  51, { 27855,  27854,  27850,  27851,  27841,  27841,  27838,  27838}},
 { 27637,  51, { 27857,  27856,  27854,  27855,  27817,  27817,  27841,  27841}},
 { 27638,  51, { 27859,  27858,  27856,  27857,  27814,  27814,  27817,  27817}},
 { 27639,  51, { 27861,  27860,  27862,  27863,  27831,  27831,  27820,  27820}},
 { 27640,  51, { 27865,  27864,  27860,  27861,  27805,  27805,  27831,  27831}},
 { 27641,  51, { 27866,  27847,  27849,  27867,  27808,  27808,  27811,  27811}},
 { 27642,  51, { 27868,  27851,  27853,  27869,  27838,  27838,  27825,  27825}},
 { 27643,  51, { 27870,  27855,  27851,  27868,  27841,  27841,  27838,  27838}},
 { 27644,  51, { 27871,  27859,  27857,  27872,  27814,  27814,  27817,  27817}},
 { 27645,  51, { 27873,  27865,  27861,  27874,  27805,  27805,  27831,  27831}},
 { 27646,  51, { 27875,  27869,  27866,  27876,  27825,  27825,  27808,  27808}},
 { 27647,  51, { 27877,  27868,  27869,  27875,  27838,  27838,  27825,  27825}},
 { 27648,  51, { 27878,  27870,  27868,  27877,  27841,  27841,  27838,  27838}},
 { 27649,  51, { 27879,  27872,  27870,  27878,  27817,  27817,  27841,  27841}},
 { 27650,  51, { 27880,  27874,  27881,  27882,  27831,  27831,  27820,  27820}},
 { 27651,  51, { 27883,  27884,  27885,  27886,  27887,  27888,  27889,  27890}},
 { 27652,  51, { 27886,  27885,  27891,  27892,  27890,  27889,  27893,  27894}},
 { 27653,  51, { 27892,  27891,  27895,  27896,  27894,  27893,  27806,  27809}},
 { 27654,  51, { 27897,  27898,  27899,  27884,  27900,  27901,  27902,  27888}},
 { 27655,  51, { 27884,  27899,  27903,  27885,  27888,  27902,  27904,  27889}},
 { 27656,  51, { 27885,  27903,  27905,  27891,  27889,  27904,  27906,  27893}},
 { 27657,  51, { 27891,  27905,  27907,  27895,  27893,  27906,  27908,  27806}},
 { 27658,  51, { 27898,  27909,  27910,  27899,  27901,  27911,  27912,  27902}},
 { 27659,  51, { 27899,  27910,  27913,  27903,  27902,  27912,  27914,  27904}},
 { 27660,  51, { 27903,  27913,  27915,  27905,  27904,  27914,  27916,  27906}},
 { 27661,  51, { 27905,  27915,  27917,  27907,  27906,  27916,  27918,  27908}},
 { 27662,  51, { 27909,  27919,  27920,  27910,  27911,  27921,  27922,  27912}},
 { 27663,  51, { 27910,  27920,  27923,  27913,  27912,  27922,  27924,  27914}},
 { 27664,  51, { 27913,  27923,  27925,  27915,  27914,  27924,  27926,  27916}},
 { 27665,  51, { 27915,  27925,  27927,  27917,  27916,  27926,  27928,  27918}},
 { 27666,  51, { 27919,  27929,  27930,  27920,  27921,  27931,  27932,  27922}},
 { 27667,  51, { 27920,  27930,  27933,  27923,  27922,  27932,  27934,  27924}},
 { 27668,  51, { 27923,  27933,  27935,  27925,  27924,  27934,  27936,  27926}},
 { 27669,  51, { 27925,  27935,  27937,  27927,  27926,  27936,  27815,  27928}},
 { 27670,  51, { 27929,  27938,  27939,  27930,  27931,  27940,  27941,  27932}},
 { 27671,  51, { 27930,  27939,  27942,  27933,  27932,  27941,  27943,  27934}},
 { 27672,  51, { 27933,  27942,  27944,  27935,  27934,  27943,  27945,  27936}},
 { 27673,  51, { 27935,  27944,  27946,  27937,  27936,  27945,  27812,  27815}},
 { 27674,  51, { 27938,  27947,  27948,  27939,  27940,  27949,  27950,  27941}},
 { 27675,  51, { 27939,  27948,  27951,  27942,  27941,  27950,  27952,  27943}},
 { 27676,  51, { 27942,  27951,  27953,  27944,  27943,  27952,  27954,  27945}},
 { 27677,  51, { 27944,  27953,  27955,  27946,  27945,  27954,  27818,  27812}},
 { 27678,  51, { 27947,  27956,  27957,  27948,  27949,  27958,  27959,  27950}},
 { 27679,  51, { 27948,  27957,  27960,  27951,  27950,  27959,  27961,  27952}},
 { 27680,  51, { 27951,  27960,  27962,  27953,  27952,  27961,  27963,  27954}},
 { 27681,  51, { 27953,  27962,  27964,  27955,  27954,  27963,  27965,  27818}},
 { 27682,  51, { 27956,  27966,  27967,  27957,  27958,  27968,  27969,  27959}},
 { 27683,  51, { 27957,  27967,  27970,  27960,  27959,  27969,  27971,  27961}},
 { 27684,  51, { 27960,  27970,  27972,  27962,  27961,  27971,  27973,  27963}},
 { 27685,  51, { 27962,  27972,  27803,  27964,  27963,  27973,  27804,  27965}},
 { 27686,  51, { 27966,  27974,  27975,  27967,  27968,  27976,  27977,  27969}},
 { 27687,  51, { 27967,  27975,  27978,  27970,  27969,  27977,  27979,  27971}},
 { 27688,  51, { 27970,  27978,  27980,  27972,  27971,  27979,  27981,  27973}},
 { 27689,  51, { 27972,  27980,  27800,  27803,  27973,  27981,  27801,  27804}},
 { 27690,  51, { 27887,  27888,  27889,  27890,  27982,  27983,  27984,  27985}},
 { 27691,  51, { 27890,  27889,  27893,  27894,  27985,  27984,  27986,  27987}},
 { 27692,  51, { 27894,  27893,  27806,  27809,  27987,  27986,  27807,  27810}},
 { 27693,  51, { 27900,  27901,  27902,  27888,  27988,  27989,  27990,  27983}},
 { 27694,  51, { 27888,  27902,  27904,  27889,  27983,  27990,  27991,  27984}},
 { 27695,  51, { 27889,  27904,  27906,  27893,  27984,  27991,  27992,  27986}},
 { 27696,  51, { 27893,  27906,  27908,  27806,  27986,  27992,  27823,  27807}},
 { 27697,  51, { 27901,  27911,  27912,  27902,  27989,  27993,  27994,  27990}},
 { 27698,  51, { 27902,  27912,  27914,  27904,  27990,  27994,  27995,  27991}},
 { 27699,  51, { 27904,  27914,  27916,  27906,  27991,  27995,  27996,  27992}},
 { 27700,  51, { 27906,  27916,  27918,  27908,  27992,  27996,  27997,  27823}},
 { 27701,  51, { 27911,  27921,  27922,  27912,  27993,  27998,  27999,  27994}},
 { 27702,  51, { 27912,  27922,  27924,  27914,  27994,  27999,  28000,  27995}},
 { 27703,  51, { 27914,  27924,  27926,  27916,  27995,  28000,  28001,  27996}},
 { 27704,  51, { 27916,  27926,  27928,  27918,  27996,  28001,  28002,  27997}},
 { 27705,  51, { 27921,  27931,  27932,  27922,  27998,  28003,  28004,  27999}},
 { 27706,  51, { 27922,  27932,  27934,  27924,  27999,  28004,  28005,  28000}},
 { 27707,  51, { 27924,  27934,  27936,  27926,  28000,  28005,  28006,  28001}},
 { 27708,  51, { 27926,  27936,  27815,  27928,  28001,  28006,  27816,  28002}},
 { 27709,  51, { 27931,  27940,  27941,  27932,  28003,  28007,  28008,  28004}},
 { 27710,  51, { 27932,  27941,  27943,  27934,  28004,  28008,  28009,  28005}},
 { 27711,  51, { 27934,  27943,  27945,  27936,  28005,  28009,  28010,  28006}},
 { 27712,  51, { 27936,  27945,  27812,  27815,  28006,  28010,  27813,  27816}},
 { 27713,  51, { 27940,  27949,  27950,  27941,  28007,  28011,  28012,  28008}},
 { 27714,  51, { 27941,  27950,  27952,  27943,  28008,  28012,  28013,  28009}},
 { 27715,  51, { 27943,  27952,  27954,  27945,  28009,  28013,  28014,  28010}},
 { 27716,  51, { 27945,  27954,  27818,  27812,  28010,  28014,  27819,  27813}},
 { 27717,  51, { 27949,  27958,  27959,  27950,  28011,  28015,  28016,  28012}},
 { 27718,  51, { 27950,  27959,  27961,  27952,  28012,  28016,  28017,  28013}},
 { 27719,  51, { 27952,  27961,  27963,  27954,  28013,  28017,  28018,  28014}},
 { 27720,  51, { 27954,  27963,  27965,  27818,  28014,  28018,  27829,  27819}},
 { 27721,  51, { 27958,  27968,  27969,  27959,  28015,  28019,  28020,  28016}},
 { 27722,  51, { 27959,  27969,  27971,  27961,  28016,  28020,  28021,  28017}},
 { 27723,  51, { 27961,  27971,  27973,  27963,  28017,  28021,  28022,  28018}},
 { 27724,  51, { 27963,  27973,  27804,  27965,  28018,  28022,  27822,  27829}},
 { 27725,  51, { 27968,  27976,  27977,  27969,  28019,  28023,  28024,  28020}},
 { 27726,  51, { 27969,  27977,  27979,  27971,  28020,  28024,  28025,  28021}},
 { 27727,  51, { 27971,  27979,  27981,  27973,  28021,  28025,  28026,  28022}},
 { 27728,  51, { 27973,  27981,  27801,  27804,  28022,  28026,  27821,  27822}},
 { 27729,  51, { 27982,  27983,  27984,  27985,  28027,  28028,  28029,  28030}},
 { 27730,  51, { 27985,  27984,  27986,  27987,  28030,  28029,  28031,  28032}},
 { 27731,  51, { 27987,  27986,  27807,  27810,  28032,  28031,  27826,  28033}},
 { 27732,  51, { 27988,  27989,  27990,  27983,  28034,  28035,  28036,  28028}},
 { 27733,  51, { 27983,  27990,  27991,  27984,  28028,  28036,  28037,  28029}},
 { 27734,  51, { 27984,  27991,  27992,  27986,  28029,  28037,  28038,  28031}},
 { 27735,  51, { 27986,  27992,  27823,  27807,  28031,  28038,  27824,  27826}},
 { 27736,  51, { 27989,  27993,  27994,  27990,  28035,  28039,  28040,  28036}},
 { 27737,  51, { 27990,  27994,  27995,  27991,  28036,  28040,  28041,  28037}},
 { 27738,  51, { 27991,  27995,  27996,  27992,  28037,  28041,  28042,  28038}},
 { 27739,  51, { 27992,  27996,  27997,  27823,  28038,  28042,  27836,  27824}},
 { 27740,  51, { 27993,  27998,  27999,  27994,  28039,  28043,  28044,  28040}},
 { 27741,  51, { 27994,  27999,  28000,  27995,  28040,  28044,  28045,  28041}},
 { 27742,  51, { 27995,  28000,  28001,  27996,  28041,  28045,  28046,  28042}},
 { 27743,  51, { 27996,  28001,  28002,  27997,  28042,  28046,  27839,  27836}},
 { 27744,  51, { 27998,  28003,  28004,  27999,  28043,  28047,  28048,  28044}},
 { 27745,  51, { 27999,  28004,  28005,  28000,  28044,  28048,  28049,  28045}},
 { 27746,  51, { 28000,  28005,  28006,  28001,  28045,  28049,  28050,  28046}},
 { 27747,  51, { 28001,  28006,  27816,  28002,  28046,  28050,  28051,  27839}},
 { 27748,  51, { 28003,  28007,  28008,  28004,  28047,  28052,  28053,  28048}},
 { 27749,  51, { 28004,  28008,  28009,  28005,  28048,  28053,  28054,  28049}},
 { 27750,  51, { 28005,  28009,  28010,  28006,  28049,  28054,  28055,  28050}},
 { 27751,  51, { 28006,  28010,  27813,  27816,  28050,  28055,  27828,  28051}},
 { 27752,  51, { 28007,  28011,  28012,  28008,  28052,  28056,  28057,  28053}},
 { 27753,  51, { 28008,  28012,  28013,  28009,  28053,  28057,  28058,  28054}},
 { 27754,  51, { 28009,  28013,  28014,  28010,  28054,  28058,  28059,  28055}},
 { 27755,  51, { 28010,  28014,  27819,  27813,  28055,  28059,  27827,  27828}},
 { 27756,  51, { 28011,  28015,  28016,  28012,  28056,  28060,  28061,  28057}},
 { 27757,  51, { 28012,  28016,  28017,  28013,  28057,  28061,  28062,  28058}},
 { 27758,  51, { 28013,  28017,  28018,  28014,  28058,  28062,  28063,  28059}},
 { 27759,  51, { 28014,  28018,  27829,  27819,  28059,  28063,  27830,  27827}},
 { 27760,  51, { 28015,  28019,  28020,  28016,  28060,  28064,  28065,  28061}},
 { 27761,  51, { 28016,  28020,  28021,  28017,  28061,  28065,  28066,  28062}},
 { 27762,  51, { 28017,  28021,  28022,  28018,  28062,  28066,  28067,  28063}},
 { 27763,  51, { 28018,  28022,  27822,  27829,  28063,  28067,  27833,  27830}},
 { 27764,  51, { 28019,  28023,  28024,  28020,  28064,  28068,  28069,  28065}},
 { 27765,  51, { 28020,  28024,  28025,  28021,  28065,  28069,  28070,  28066}},
 { 27766,  51, { 28021,  28025,  28026,  28022,  28066,  28070,  28071,  28067}},
 { 27767,  51, { 28022,  28026,  27821,  27822,  28067,  28071,  27832,  27833}},
 { 27768,  51, { 28027,  28028,  28029,  28030,  28072,  28073,  28074,  28075}},
 { 27769,  51, { 28030,  28029,  28031,  28032,  28075,  28074,  28076,  28077}},
 { 27770,  51, { 28032,  28031,  27826,  28033,  28077,  28076,  27835,  28078}},
 { 27771,  51, { 28034,  28035,  28036,  28028,  28079,  28080,  28081,  28073}},
 { 27772,  51, { 28028,  28036,  28037,  28029,  28073,  28081,  28082,  28074}},
 { 27773,  51, { 28029,  28037,  28038,  28031,  28074,  28082,  28083,  28076}},
 { 27774,  51, { 28031,  28038,  27824,  27826,  28076,  28083,  27834,  27835}},
 { 27775,  51, { 28035,  28039,  28040,  28036,  28080,  28084,  28085,  28081}},
 { 27776,  51, { 28036,  28040,  28041,  28037,  28081,  28085,  28086,  28082}},
 { 27777,  51, { 28037,  28041,  28042,  28038,  28082,  28086,  28087,  28083}},
 { 27778,  51, { 28038,  28042,  27836,  27824,  28083,  28087,  27837,  27834}},
 { 27779,  51, { 28039,  28043,  28044,  28040,  28084,  28088,  28089,  28085}},
 { 27780,  51, { 28040,  28044,  28045,  28041,  28085,  28089,  28090,  28086}},
 { 27781,  51, { 28041,  28045,  28046,  28042,  28086,  28090,  28091,  28087}},
 { 27782,  51, { 28042,  28046,  27839,  27836,  28087,  28091,  27840,  27837}},
 { 27783,  51, { 28043,  28047,  28048,  28044,  28088,  28092,  28093,  28089}},
 { 27784,  51, { 28044,  28048,  28049,  28045,  28089,  28093,  28094,  28090}},
 { 27785,  51, { 28045,  28049,  28050,  28046,  28090,  28094,  28095,  28091}},
 { 27786,  51, { 28046,  28050,  28051,  27839,  28091,  28095,  28096,  27840}},
 { 27787,  51, { 28047,  28052,  28053,  28048,  28092,  28097,  28098,  28093}},
 { 27788,  51, { 28048,  28053,  28054,  28049,  28093,  28098,  28099,  28094}},
 { 27789,  51, { 28049,  28054,  28055,  28050,  28094,  28099,  28100,  28095}},
 { 27790,  51, { 28050,  28055,  27828,  28051,  28095,  28100,  27843,  28096}},
 { 27791,  51, { 28052,  28056,  28057,  28053,  28097,  28101,  28102,  28098}},
 { 27792,  51, { 28053,  28057,  28058,  28054,  28098,  28102,  28103,  28099}},
 { 27793,  51, { 28054,  28058,  28059,  28055,  28099,  28103,  28104,  28100}},
 { 27794,  51, { 28055,  28059,  27827,  27828,  28100,  28104,  27842,  27843}},
 { 27795,  51, { 28056,  28060,  28061,  28057,  28101,  28105,  28106,  28102}},
 { 27796,  51, { 28057,  28061,  28062,  28058,  28102,  28106,  28107,  28103}},
 { 27797,  51, { 28058,  28062,  28063,  28059,  28103,  28107,  28108,  28104}},
 { 27798,  51, { 28059,  28063,  27830,  27827,  28104,  28108,  28109,  27842}},
 { 27799,  51, { 28060,  28064,  28065,  28061,  28105,  28110,  28111,  28106}},
 { 27800,  51, { 28061,  28065,  28066,  28062,  28106,  28111,  28112,  28107}},
 { 27801,  51, { 28062,  28066,  28067,  28063,  28107,  28112,  28113,  28108}},
 { 27802,  51, { 28063,  28067,  27833,  27830,  28108,  28113,  27845,  28109}},
 { 27803,  51, { 28064,  28068,  28069,  28065,  28110,  28114,  28115,  28111}},
 { 27804,  51, { 28065,  28069,  28070,  28066,  28111,  28115,  28116,  28112}},
 { 27805,  51, { 28066,  28070,  28071,  28067,  28112,  28116,  28117,  28113}},
 { 27806,  51, { 28067,  28071,  27832,  27833,  28113,  28117,  27844,  27845}},
 { 27807,  51, { 28072,  28073,  28074,  28075,  28118,  28119,  28120,  28121}},
 { 27808,  51, { 28075,  28074,  28076,  28077,  28121,  28120,  28122,  28123}},
 { 27809,  51, { 28077,  28076,  27835,  28078,  28123,  28122,  27846,  27848}},
 { 27810,  51, { 28079,  28080,  28081,  28073,  28124,  28125,  28126,  28119}},
 { 27811,  51, { 28073,  28081,  28082,  28074,  28119,  28126,  28127,  28120}},
 { 27812,  51, { 28074,  28082,  28083,  28076,  28120,  28127,  28128,  28122}},
 { 27813,  51, { 28076,  28083,  27834,  27835,  28122,  28128,  27852,  27846}},
 { 27814,  51, { 28080,  28084,  28085,  28081,  28125,  28129,  28130,  28126}},
 { 27815,  51, { 28081,  28085,  28086,  28082,  28126,  28130,  28131,  28127}},
 { 27816,  51, { 28082,  28086,  28087,  28083,  28127,  28131,  28132,  28128}},
 { 27817,  51, { 28083,  28087,  27837,  27834,  28128,  28132,  27850,  27852}},
 { 27818,  51, { 28084,  28088,  28089,  28085,  28129,  28133,  28134,  28130}},
 { 27819,  51, { 28085,  28089,  28090,  28086,  28130,  28134,  28135,  28131}},
 { 27820,  51, { 28086,  28090,  28091,  28087,  28131,  28135,  28136,  28132}},
 { 27821,  51, { 28087,  28091,  27840,  27837,  28132,  28136,  27854,  27850}},
 { 27822,  51, { 28088,  28092,  28093,  28089,  28133,  28137,  28138,  28134}},
 { 27823,  51, { 28089,  28093,  28094,  28090,  28134,  28138,  28139,  28135}},
 { 27824,  51, { 28090,  28094,  28095,  28091,  28135,  28139,  28140,  28136}},
 { 27825,  51, { 28091,  28095,  28096,  27840,  28136,  28140,  27856,  27854}},
 { 27826,  51, { 28092,  28097,  28098,  28093,  28137,  28141,  28142,  28138}},
 { 27827,  51, { 28093,  28098,  28099,  28094,  28138,  28142,  28143,  28139}},
 { 27828,  51, { 28094,  28099,  28100,  28095,  28139,  28143,  28144,  28140}},
 { 27829,  51, { 28095,  28100,  27843,  28096,  28140,  28144,  27858,  27856}},
 { 27830,  51, { 28097,  28101,  28102,  28098,  28141,  28145,  28146,  28142}},
 { 27831,  51, { 28098,  28102,  28103,  28099,  28142,  28146,  28147,  28143}},
 { 27832,  51, { 28099,  28103,  28104,  28100,  28143,  28147,  28148,  28144}},
 { 27833,  51, { 28100,  28104,  27842,  27843,  28144,  28148,  27862,  27858}},
 { 27834,  51, { 28101,  28105,  28106,  28102,  28145,  28149,  28150,  28146}},
 { 27835,  51, { 28102,  28106,  28107,  28103,  28146,  28150,  28151,  28147}},
 { 27836,  51, { 28103,  28107,  28108,  28104,  28147,  28151,  28152,  28148}},
 { 27837,  51, { 28104,  28108,  28109,  27842,  28148,  28152,  27860,  27862}},
 { 27838,  51, { 28105,  28110,  28111,  28106,  28149,  28153,  28154,  28150}},
 { 27839,  51, { 28106,  28111,  28112,  28107,  28150,  28154,  28155,  28151}},
 { 27840,  51, { 28107,  28112,  28113,  28108,  28151,  28155,  28156,  28152}},
 { 27841,  51, { 28108,  28113,  27845,  28109,  28152,  28156,  27864,  27860}},
 { 27842,  51, { 28110,  28114,  28115,  28111,  28153,  28157,  28158,  28154}},
 { 27843,  51, { 28111,  28115,  28116,  28112,  28154,  28158,  28159,  28155}},
 { 27844,  51, { 28112,  28116,  28117,  28113,  28155,  28159,  28160,  28156}},
 { 27845,  51, { 28113,  28117,  27844,  27845,  28156,  28160,  28161,  27864}},
 { 27846,  51, { 28118,  28119,  28120,  28121,  28162,  28163,  28164,  28165}},
 { 27847,  51, { 28121,  28120,  28122,  28123,  28165,  28164,  28166,  28167}},
 { 27848,  51, { 28123,  28122,  27846,  27848,  28167,  28166,  27847,  27849}},
 { 27849,  51, { 28124,  28125,  28126,  28119,  28168,  28169,  28170,  28163}},
 { 27850,  51, { 28119,  28126,  28127,  28120,  28163,  28170,  28171,  28164}},
 { 27851,  51, { 28120,  28127,  28128,  28122,  28164,  28171,  28172,  28166}},
 { 27852,  51, { 28122,  28128,  27852,  27846,  28166,  28172,  27853,  27847}},
 { 27853,  51, { 28125,  28129,  28130,  28126,  28169,  28173,  28174,  28170}},
 { 27854,  51, { 28126,  28130,  28131,  28127,  28170,  28174,  28175,  28171}},
 { 27855,  51, { 28127,  28131,  28132,  28128,  28171,  28175,  28176,  28172}},
 { 27856,  51, { 28128,  28132,  27850,  27852,  28172,  28176,  27851,  27853}},
 { 27857,  51, { 28129,  28133,  28134,  28130,  28173,  28177,  28178,  28174}},
 { 27858,  51, { 28130,  28134,  28135,  28131,  28174,  28178,  28179,  28175}},
 { 27859,  51, { 28131,  28135,  28136,  28132,  28175,  28179,  28180,  28176}},
 { 27860,  51, { 28132,  28136,  27854,  27850,  28176,  28180,  27855,  27851}},
 { 27861,  51, { 28133,  28137,  28138,  28134,  28177,  28181,  28182,  28178}},
 { 27862,  51, { 28134,  28138,  28139,  28135,  28178,  28182,  28183,  28179}},
 { 27863,  51, { 28135,  28139,  28140,  28136,  28179,  28183,  28184,  28180}},
 { 27864,  51, { 28136,  28140,  27856,  27854,  28180,  28184,  27857,  27855}},
 { 27865,  51, { 28137,  28141,  28142,  28138,  28181,  28185,  28186,  28182}},
 { 27866,  51, { 28138,  28142,  28143,  28139,  28182,  28186,  28187,  28183}},
 { 27867,  51, { 28139,  28143,  28144,  28140,  28183,  28187,  28188,  28184}},
 { 27868,  51, { 28140,  28144,  27858,  27856,  28184,  28188,  27859,  27857}},
 { 27869,  51, { 28141,  28145,  28146,  28142,  28185,  28189,  28190,  28186}},
 { 27870,  51, { 28142,  28146,  28147,  28143,  28186,  28190,  28191,  28187}},
 { 27871,  51, { 28143,  28147,  28148,  28144,  28187,  28191,  28192,  28188}},
 { 27872,  51, { 28144,  28148,  27862,  27858,  28188,  28192,  27863,  27859}},
 { 27873,  51, { 28145,  28149,  28150,  28146,  28189,  28193,  28194,  28190}},
 { 27874,  51, { 28146,  28150,  28151,  28147,  28190,  28194,  28195,  28191}},
 { 27875,  51, { 28147,  28151,  28152,  28148,  28191,  28195,  28196,  28192}},
 { 27876,  51, { 28148,  28152,  27860,  27862,  28192,  28196,  27861,  27863}},
 { 27877,  51, { 28149,  28153,  28154,  28150,  28193,  28197,  28198,  28194}},
 { 27878,  51, { 28150,  28154,  28155,  28151,  28194,  28198,  28199,  28195}},
 { 27879,  51, { 28151,  28155,  28156,  28152,  28195,  28199,  28200,  28196}},
 { 27880,  51, { 28152,  28156,  27864,  27860,  28196,  28200,  27865,  27861}},
 { 27881,  51, { 28153,  28157,  28158,  28154,  28197,  28201,  28202,  28198}},
 { 27882,  51, { 28154,  28158,  28159,  28155,  28198,  28202,  28203,  28199}},
 { 27883,  51, { 28155,  28159,  28160,  28156,  28199,  28203,  28204,  28200}},
 { 27884,  51, { 28156,  28160,  28161,  27864,  28200,  28204,  28205,  27865}},
 { 27885,  51, { 28162,  28163,  28164,  28165,  28206,  28207,  28208,  28209}},
 { 27886,  51, { 28165,  28164,  28166,  28167,  28209,  28208,  28210,  28211}},
 { 27887,  51, { 28167,  28166,  27847,  27849,  28211,  28210,  27866,  27867}},
 { 27888,  51, { 28168,  28169,  28170,  28163,  28212,  28213,  28214,  28207}},
 { 27889,  51, { 28163,  28170,  28171,  28164,  28207,  28214,  28215,  28208}},
 { 27890,  51, { 28164,  28171,  28172,  28166,  28208,  28215,  28216,  28210}},
 { 27891,  51, { 28166,  28172,  27853,  27847,  28210,  28216,  27869,  27866}},
 { 27892,  51, { 28169,  28173,  28174,  28170,  28213,  28217,  28218,  28214}},
 { 27893,  51, { 28170,  28174,  28175,  28171,  28214,  28218,  28219,  28215}},
 { 27894,  51, { 28171,  28175,  28176,  28172,  28215,  28219,  28220,  28216}},
 { 27895,  51, { 28172,  28176,  27851,  27853,  28216,  28220,  27868,  27869}},
 { 27896,  51, { 28173,  28177,  28178,  28174,  28217,  28221,  28222,  28218}},
 { 27897,  51, { 28174,  28178,  28179,  28175,  28218,  28222,  28223,  28219}},
 { 27898,  51, { 28175,  28179,  28180,  28176,  28219,  28223,  28224,  28220}},
 { 27899,  51, { 28176,  28180,  27855,  27851,  28220,  28224,  27870,  27868}},
 { 27900,  51, { 28177,  28181,  28182,  28178,  28221,  28225,  28226,  28222}},
 { 27901,  51, { 28178,  28182,  28183,  28179,  28222,  28226,  28227,  28223}},
 { 27902,  51, { 28179,  28183,  28184,  28180,  28223,  28227,  28228,  28224}},
 { 27903,  51, { 28180,  28184,  27857,  27855,  28224,  28228,  27872,  27870}},
 { 27904,  51, { 28181,  28185,  28186,  28182,  28225,  28229,  28230,  28226}},
 { 27905,  51, { 28182,  28186,  28187,  28183,  28226,  28230,  28231,  28227}},
 { 27906,  51, { 28183,  28187,  28188,  28184,  28227,  28231,  28232,  28228}},
 { 27907,  51, { 28184,  28188,  27859,  27857,  28228,  28232,  27871,  27872}},
 { 27908,  51, { 28185,  28189,  28190,  28186,  28229,  28233,  28234,  28230}},
 { 27909,  51, { 28186,  28190,  28191,  28187,  28230,  28234,  28235,  28231}},
 { 27910,  51, { 28187,  28191,  28192,  28188,  28231,  28235,  28236,  28232}},
 { 27911,  51, { 28188,  28192,  27863,  27859,  28232,  28236,  27881,  27871}},
 { 27912,  51, { 28189,  28193,  28194,  28190,  28233,  28237,  28238,  28234}},
 { 27913,  51, { 28190,  28194,  28195,  28191,  28234,  28238,  28239,  28235}},
 { 27914,  51, { 28191,  28195,  28196,  28192,  28235,  28239,  28240,  28236}},
 { 27915,  51, { 28192,  28196,  27861,  27863,  28236,  28240,  27874,  27881}},
 { 27916,  51, { 28193,  28197,  28198,  28194,  28237,  28241,  28242,  28238}},
 { 27917,  51, { 28194,  28198,  28199,  28195,  28238,  28242,  28243,  28239}},
 { 27918,  51, { 28195,  28199,  28200,  28196,  28239,  28243,  28244,  28240}},
 { 27919,  51, { 28196,  28200,  27865,  27861,  28240,  28244,  27873,  27874}},
 { 27920,  51, { 28197,  28201,  28202,  28198,  28241,  28245,  28246,  28242}},
 { 27921,  51, { 28198,  28202,  28203,  28199,  28242,  28246,  28247,  28243}},
 { 27922,  51, { 28199,  28203,  28204,  28200,  28243,  28247,  28248,  28244}},
 { 27923,  51, { 28200,  28204,  28205,  27865,  28244,  28248,  28249,  27873}},
 { 27924,  51, { 28206,  28207,  28208,  28209,  28250,  28251,  28252,  28253}},
 { 27925,  51, { 28209,  28208,  28210,  28211,  28253,  28252,  28254,  28255}},
 { 27926,  51, { 28211,  28210,  27866,  27867,  28255,  28254,  27876,  28256}},
 { 27927,  51, { 28212,  28213,  28214,  28207,  28257,  28258,  28259,  28251}},
 { 27928,  51, { 28207,  28214,  28215,  28208,  28251,  28259,  28260,  28252}},
 { 27929,  51, { 28208,  28215,  28216,  28210,  28252,  28260,  28261,  28254}},
 { 27930,  51, { 28210,  28216,  27869,  27866,  28254,  28261,  27875,  27876}},
 { 27931,  51, { 28213,  28217,  28218,  28214,  28258,  28262,  28263,  28259}},
 { 27932,  51, { 28214,  28218,  28219,  28215,  28259,  28263,  28264,  28260}},
 { 27933,  51, { 28215,  28219,  28220,  28216,  28260,  28264,  28265,  28261}},
 { 27934,  51, { 28216,  28220,  27868,  27869,  28261,  28265,  27877,  27875}},
 { 27935,  51, { 28217,  28221,  28222,  28218,  28262,  28266,  28267,  28263}},
 { 27936,  51, { 28218,  28222,  28223,  28219,  28263,  28267,  28268,  28264}},
 { 27937,  51, { 28219,  28223,  28224,  28220,  28264,  28268,  28269,  28265}},
 { 27938,  51, { 28220,  28224,  27870,  27868,  28265,  28269,  27878,  27877}},
 { 27939,  51, { 28221,  28225,  28226,  28222,  28266,  28270,  28271,  28267}},
 { 27940,  51, { 28222,  28226,  28227,  28223,  28267,  28271,  28272,  28268}},
 { 27941,  51, { 28223,  28227,  28228,  28224,  28268,  28272,  28273,  28269}},
 { 27942,  51, { 28224,  28228,  27872,  27870,  28269,  28273,  27879,  27878}},
 { 27943,  51, { 28225,  28229,  28230,  28226,  28270,  28274,  28275,  28271}},
 { 27944,  51, { 28226,  28230,  28231,  28227,  28271,  28275,  28276,  28272}},
 { 27945,  51, { 28227,  28231,  28232,  28228,  28272,  28276,  28277,  28273}},
 { 27946,  51, { 28228,  28232,  27871,  27872,  28273,  28277,  28278,  27879}},
 { 27947,  51, { 28229,  28233,  28234,  28230,  28274,  28279,  28280,  28275}},
 { 27948,  51, { 28230,  28234,  28235,  28231,  28275,  28280,  28281,  28276}},
 { 27949,  51, { 28231,  28235,  28236,  28232,  28276,  28281,  28282,  28277}},
 { 27950,  51, { 28232,  28236,  27881,  27871,  28277,  28282,  27882,  28278}},
 { 27951,  51, { 28233,  28237,  28238,  28234,  28279,  28283,  28284,  28280}},
 { 27952,  51, { 28234,  28238,  28239,  28235,  28280,  28284,  28285,  28281}},
 { 27953,  51, { 28235,  28239,  28240,  28236,  28281,  28285,  28286,  28282}},
 { 27954,  51, { 28236,  28240,  27874,  27881,  28282,  28286,  27880,  27882}},
 { 27955,  51, { 28237,  28241,  28242,  28238,  28283,  28287,  28288,  28284}},
 { 27956,  51, { 28238,  28242,  28243,  28239,  28284,  28288,  28289,  28285}},
 { 27957,  51, { 28239,  28243,  28244,  28240,  28285,  28289,  28290,  28286}},
 { 27958,  51, { 28240,  28244,  27873,  27874,  28286,  28290,  28291,  27880}},
 { 27959,  51, { 28241,  28245,  28246,  28242,  28287,  28292,  28293,  28288}},
 { 27960,  51, { 28242,  28246,  28247,  28243,  28288,  28293,  28294,  28289}},
 { 27961,  51, { 28243,  28247,  28248,  28244,  28289,  28294,  28295,  28290}},
 { 27962,  51, { 28244,  28248,  28249,  27873,  28290,  28295,  28296,  28291}},
 { 27963,  51, { 27802,  28297,  28298,  28296,  28299,  28300,  28301,  28302}},
 { 27964,  51, { 28293,  28303,  28304,  28292,  28305,  28306,  28307,  28308}},
 { 27965,  51, { 28294,  28309,  28303,  28293,  28310,  28311,  28306,  28305}},
 { 27966,  51, { 28295,  28312,  28309,  28294,  28313,  28314,  28311,  28310}},
 { 27967,  51, { 28296,  28298,  28312,  28295,  28302,  28301,  28314,  28313}},
 { 27968,  51, { 28297,  28315,  28316,  28298,  28300,  28317,  28318,  28301}},
 { 27969,  51, { 28303,  28319,  28320,  28304,  28306,  28321,  28322,  28307}},
 { 27970,  51, { 28309,  28323,  28319,  28303,  28311,  28324,  28321,  28306}},
 { 27971,  51, { 28312,  28325,  28323,  28309,  28314,  28326,  28324,  28311}},
 { 27972,  51, { 28298,  28316,  28325,  28312,  28301,  28318,  28326,  28314}},
 { 27973,  51, { 28315,  28327,  28328,  28316,  28317,  28329,  28330,  28318}},
 { 27974,  51, { 28319,  28331,  28332,  28320,  28321,  28333,  28334,  28322}},
 { 27975,  51, { 28323,  28335,  28331,  28319,  28324,  28336,  28333,  28321}},
 { 27976,  51, { 28325,  28337,  28335,  28323,  28326,  28338,  28336,  28324}},
 { 27977,  51, { 28316,  28328,  28337,  28325,  28318,  28330,  28338,  28326}},
 { 27978,  51, { 28327,  27540,  27539,  28328,  28329,  27542,  27541,  28330}},
 { 27979,  51, { 28331,  27544,  27543,  28332,  28333,  27546,  27545,  28334}},
 { 27980,  51, { 28335,  27547,  27544,  28331,  28336,  27548,  27546,  28333}},
 { 27981,  51, { 28337,  27549,  27547,  28335,  28338,  27550,  27548,  28336}},
 { 27982,  51, { 28328,  27539,  27549,  28337,  28330,  27541,  27550,  28338}},
 { 27983,  51, { 27802,  27805,  28291,  28296,  28297,  28339,  28340,  28298}},
 { 27984,  51, { 27805,  27831,  27880,  28291,  28339,  28341,  28342,  28340}},
 { 27985,  51, { 27831,  27820,  27882,  27880,  28341,  28343,  28344,  28342}},
 { 27986,  51, { 27820,  27814,  28278,  27882,  28343,  28345,  28346,  28344}},
 { 27987,  51, { 27814,  27817,  27879,  28278,  28345,  28347,  28348,  28346}},
 { 27988,  51, { 27817,  27841,  27878,  27879,  28347,  28349,  28350,  28348}},
 { 27989,  51, { 27841,  27838,  27877,  27878,  28349,  28351,  28352,  28350}},
 { 27990,  51, { 27838,  27825,  27875,  27877,  28351,  28353,  28354,  28352}},
 { 27991,  51, { 27825,  27808,  27876,  27875,  28353,  28355,  28356,  28354}},
 { 27992,  51, { 27808,  27811,  28256,  27876,  28355,  28357,  28358,  28356}},
 { 27993,  51, { 28258,  28259,  28251,  28257,  28359,  28360,  28361,  28362}},
 { 27994,  51, { 28262,  28263,  28259,  28258,  28363,  28364,  28360,  28359}},
 { 27995,  51, { 28266,  28267,  28263,  28262,  28365,  28366,  28364,  28363}},
 { 27996,  51, { 28270,  28271,  28267,  28266,  28367,  28368,  28366,  28365}},
 { 27997,  51, { 28274,  28275,  28271,  28270,  28369,  28370,  28368,  28367}},
 { 27998,  51, { 28279,  28280,  28275,  28274,  28371,  28372,  28370,  28369}},
 { 27999,  51, { 28283,  28284,  28280,  28279,  28373,  28374,  28372,  28371}},
 { 28000,  51, { 28287,  28288,  28284,  28283,  28375,  28376,  28374,  28373}},
 { 28001,  51, { 28292,  28293,  28288,  28287,  28304,  28303,  28376,  28375}},
 { 28002,  51, { 28251,  28252,  28253,  28250,  28361,  28377,  28378,  28379}},
 { 28003,  51, { 28252,  28254,  28255,  28253,  28377,  28380,  28381,  28378}},
 { 28004,  51, { 28254,  27876,  28256,  28255,  28380,  28356,  28358,  28381}},
 { 28005,  51, { 28259,  28260,  28252,  28251,  28360,  28382,  28377,  28361}},
 { 28006,  51, { 28260,  28261,  28254,  28252,  28382,  28383,  28380,  28377}},
 { 28007,  51, { 28261,  27875,  27876,  28254,  28383,  28354,  28356,  28380}},
 { 28008,  51, { 28263,  28264,  28260,  28259,  28364,  28384,  28382,  28360}},
 { 28009,  51, { 28264,  28265,  28261,  28260,  28384,  28385,  28383,  28382}},
 { 28010,  51, { 28265,  27877,  27875,  28261,  28385,  28352,  28354,  28383}},
 { 28011,  51, { 28267,  28268,  28264,  28263,  28366,  28386,  28384,  28364}},
 { 28012,  51, { 28268,  28269,  28265,  28264,  28386,  28387,  28385,  28384}},
 { 28013,  51, { 28269,  27878,  27877,  28265,  28387,  28350,  28352,  28385}},
 { 28014,  51, { 28271,  28272,  28268,  28267,  28368,  28388,  28386,  28366}},
 { 28015,  51, { 28272,  28273,  28269,  28268,  28388,  28389,  28387,  28386}},
 { 28016,  51, { 28273,  27879,  27878,  28269,  28389,  28348,  28350,  28387}},
 { 28017,  51, { 28275,  28276,  28272,  28271,  28370,  28390,  28388,  28368}},
 { 28018,  51, { 28276,  28277,  28273,  28272,  28390,  28391,  28389,  28388}},
 { 28019,  51, { 28277,  28278,  27879,  28273,  28391,  28346,  28348,  28389}},
 { 28020,  51, { 28280,  28281,  28276,  28275,  28372,  28392,  28390,  28370}},
 { 28021,  51, { 28281,  28282,  28277,  28276,  28392,  28393,  28391,  28390}},
 { 28022,  51, { 28282,  27882,  28278,  28277,  28393,  28344,  28346,  28391}},
 { 28023,  51, { 28284,  28285,  28281,  28280,  28374,  28394,  28392,  28372}},
 { 28024,  51, { 28285,  28286,  28282,  28281,  28394,  28395,  28393,  28392}},
 { 28025,  51, { 28286,  27880,  27882,  28282,  28395,  28342,  28344,  28393}},
 { 28026,  51, { 28288,  28289,  28285,  28284,  28376,  28396,  28394,  28374}},
 { 28027,  51, { 28289,  28290,  28286,  28285,  28396,  28397,  28395,  28394}},
 { 28028,  51, { 28290,  28291,  27880,  28286,  28397,  28340,  28342,  28395}},
 { 28029,  51, { 28293,  28294,  28289,  28288,  28303,  28309,  28396,  28376}},
 { 28030,  51, { 28294,  28295,  28290,  28289,  28309,  28312,  28397,  28396}},
 { 28031,  51, { 28295,  28296,  28291,  28290,  28312,  28298,  28340,  28397}},
 { 28032,  51, { 28297,  28339,  28340,  28298,  28315,  28398,  28399,  28316}},
 { 28033,  51, { 28339,  28341,  28342,  28340,  28398,  28400,  28401,  28399}},
 { 28034,  51, { 28341,  28343,  28344,  28342,  28400,  28402,  28403,  28401}},
 { 28035,  51, { 28343,  28345,  28346,  28344,  28402,  28404,  28405,  28403}},
 { 28036,  51, { 28345,  28347,  28348,  28346,  28404,  28406,  28407,  28405}},
 { 28037,  51, { 28347,  28349,  28350,  28348,  28406,  28408,  28409,  28407}},
 { 28038,  51, { 28349,  28351,  28352,  28350,  28408,  28410,  28411,  28409}},
 { 28039,  51, { 28351,  28353,  28354,  28352,  28410,  28412,  28413,  28411}},
 { 28040,  51, { 28353,  28355,  28356,  28354,  28412,  28414,  28415,  28413}},
 { 28041,  51, { 28355,  28357,  28358,  28356,  28414,  28416,  28417,  28415}},
 { 28042,  51, { 28359,  28360,  28361,  28362,  28418,  28419,  28420,  28421}},
 { 28043,  51, { 28363,  28364,  28360,  28359,  28422,  28423,  28419,  28418}},
 { 28044,  51, { 28365,  28366,  28364,  28363,  28424,  28425,  28423,  28422}},
 { 28045,  51, { 28367,  28368,  28366,  28365,  28426,  28427,  28425,  28424}},
 { 28046,  51, { 28369,  28370,  28368,  28367,  28428,  28429,  28427,  28426}},
 { 28047,  51, { 28371,  28372,  28370,  28369,  28430,  28431,  28429,  28428}},
 { 28048,  51, { 28373,  28374,  28372,  28371,  28432,  28433,  28431,  28430}},
 { 28049,  51, { 28375,  28376,  28374,  28373,  28434,  28435,  28433,  28432}},
 { 28050,  51, { 28304,  28303,  28376,  28375,  28320,  28319,  28435,  28434}},
 { 28051,  51, { 28361,  28377,  28378,  28379,  28420,  28436,  28437,  28438}},
 { 28052,  51, { 28377,  28380,  28381,  28378,  28436,  28439,  28440,  28437}},
 { 28053,  51, { 28380,  28356,  28358,  28381,  28439,  28415,  28417,  28440}},
 { 28054,  51, { 28360,  28382,  28377,  28361,  28419,  28441,  28436,  28420}},
 { 28055,  51, { 28382,  28383,  28380,  28377,  28441,  28442,  28439,  28436}},
 { 28056,  51, { 28383,  28354,  28356,  28380,  28442,  28413,  28415,  28439}},
 { 28057,  51, { 28364,  28384,  28382,  28360,  28423,  28443,  28441,  28419}},
 { 28058,  51, { 28384,  28385,  28383,  28382,  28443,  28444,  28442,  28441}},
 { 28059,  51, { 28385,  28352,  28354,  28383,  28444,  28411,  28413,  28442}},
 { 28060,  51, { 28366,  28386,  28384,  28364,  28425,  28445,  28443,  28423}},
 { 28061,  51, { 28386,  28387,  28385,  28384,  28445,  28446,  28444,  28443}},
 { 28062,  51, { 28387,  28350,  28352,  28385,  28446,  28409,  28411,  28444}},
 { 28063,  51, { 28368,  28388,  28386,  28366,  28427,  28447,  28445,  28425}},
 { 28064,  51, { 28388,  28389,  28387,  28386,  28447,  28448,  28446,  28445}},
 { 28065,  51, { 28389,  28348,  28350,  28387,  28448,  28407,  28409,  28446}},
 { 28066,  51, { 28370,  28390,  28388,  28368,  28429,  28449,  28447,  28427}},
 { 28067,  51, { 28390,  28391,  28389,  28388,  28449,  28450,  28448,  28447}},
 { 28068,  51, { 28391,  28346,  28348,  28389,  28450,  28405,  28407,  28448}},
 { 28069,  51, { 28372,  28392,  28390,  28370,  28431,  28451,  28449,  28429}},
 { 28070,  51, { 28392,  28393,  28391,  28390,  28451,  28452,  28450,  28449}},
 { 28071,  51, { 28393,  28344,  28346,  28391,  28452,  28403,  28405,  28450}},
 { 28072,  51, { 28374,  28394,  28392,  28372,  28433,  28453,  28451,  28431}},
 { 28073,  51, { 28394,  28395,  28393,  28392,  28453,  28454,  28452,  28451}},
 { 28074,  51, { 28395,  28342,  28344,  28393,  28454,  28401,  28403,  28452}},
 { 28075,  51, { 28376,  28396,  28394,  28374,  28435,  28455,  28453,  28433}},
 { 28076,  51, { 28396,  28397,  28395,  28394,  28455,  28456,  28454,  28453}},
 { 28077,  51, { 28397,  28340,  28342,  28395,  28456,  28399,  28401,  28454}},
 { 28078,  51, { 28303,  28309,  28396,  28376,  28319,  28323,  28455,  28435}},
 { 28079,  51, { 28309,  28312,  28397,  28396,  28323,  28325,  28456,  28455}},
 { 28080,  51, { 28312,  28298,  28340,  28397,  28325,  28316,  28399,  28456}},
 { 28081,  51, { 28315,  28398,  28399,  28316,  28327,  28457,  28458,  28328}},
 { 28082,  51, { 28398,  28400,  28401,  28399,  28457,  28459,  28460,  28458}},
 { 28083,  51, { 28400,  28402,  28403,  28401,  28459,  28461,  28462,  28460}},
 { 28084,  51, { 28402,  28404,  28405,  28403,  28461,  28463,  28464,  28462}},
 { 28085,  51, { 28404,  28406,  28407,  28405,  28463,  28465,  28466,  28464}},
 { 28086,  51, { 28406,  28408,  28409,  28407,  28465,  28467,  28468,  28466}},
 { 28087,  51, { 28408,  28410,  28411,  28409,  28467,  28469,  28470,  28468}},
 { 28088,  51, { 28410,  28412,  28413,  28411,  28469,  28471,  28472,  28470}},
 { 28089,  51, { 28412,  28414,  28415,  28413,  28471,  28473,  28474,  28472}},
 { 28090,  51, { 28414,  28416,  28417,  28415,  28473,  28475,  28476,  28474}},
 { 28091,  51, { 28418,  28419,  28420,  28421,  28477,  28478,  28479,  28480}},
 { 28092,  51, { 28422,  28423,  28419,  28418,  28481,  28482,  28478,  28477}},
 { 28093,  51, { 28424,  28425,  28423,  28422,  28483,  28484,  28482,  28481}},
 { 28094,  51, { 28426,  28427,  28425,  28424,  28485,  28486,  28484,  28483}},
 { 28095,  51, { 28428,  28429,  28427,  28426,  28487,  28488,  28486,  28485}},
 { 28096,  51, { 28430,  28431,  28429,  28428,  28489,  28490,  28488,  28487}},
 { 28097,  51, { 28432,  28433,  28431,  28430,  28491,  28492,  28490,  28489}},
 { 28098,  51, { 28434,  28435,  28433,  28432,  28493,  28494,  28492,  28491}},
 { 28099,  51, { 28320,  28319,  28435,  28434,  28332,  28331,  28494,  28493}},
 { 28100,  51, { 28420,  28436,  28437,  28438,  28479,  28495,  28496,  28497}},
 { 28101,  51, { 28436,  28439,  28440,  28437,  28495,  28498,  28499,  28496}},
 { 28102,  51, { 28439,  28415,  28417,  28440,  28498,  28474,  28476,  28499}},
 { 28103,  51, { 28419,  28441,  28436,  28420,  28478,  28500,  28495,  28479}},
 { 28104,  51, { 28441,  28442,  28439,  28436,  28500,  28501,  28498,  28495}},
 { 28105,  51, { 28442,  28413,  28415,  28439,  28501,  28472,  28474,  28498}},
 { 28106,  51, { 28423,  28443,  28441,  28419,  28482,  28502,  28500,  28478}},
 { 28107,  51, { 28443,  28444,  28442,  28441,  28502,  28503,  28501,  28500}},
 { 28108,  51, { 28444,  28411,  28413,  28442,  28503,  28470,  28472,  28501}},
 { 28109,  51, { 28425,  28445,  28443,  28423,  28484,  28504,  28502,  28482}},
 { 28110,  51, { 28445,  28446,  28444,  28443,  28504,  28505,  28503,  28502}},
 { 28111,  51, { 28446,  28409,  28411,  28444,  28505,  28468,  28470,  28503}},
 { 28112,  51, { 28427,  28447,  28445,  28425,  28486,  28506,  28504,  28484}},
 { 28113,  51, { 28447,  28448,  28446,  28445,  28506,  28507,  28505,  28504}},
 { 28114,  51, { 28448,  28407,  28409,  28446,  28507,  28466,  28468,  28505}},
 { 28115,  51, { 28429,  28449,  28447,  28427,  28488,  28508,  28506,  28486}},
 { 28116,  51, { 28449,  28450,  28448,  28447,  28508,  28509,  28507,  28506}},
 { 28117,  51, { 28450,  28405,  28407,  28448,  28509,  28464,  28466,  28507}},
 { 28118,  51, { 28431,  28451,  28449,  28429,  28490,  28510,  28508,  28488}},
 { 28119,  51, { 28451,  28452,  28450,  28449,  28510,  28511,  28509,  28508}},
 { 28120,  51, { 28452,  28403,  28405,  28450,  28511,  28462,  28464,  28509}},
 { 28121,  51, { 28433,  28453,  28451,  28431,  28492,  28512,  28510,  28490}},
 { 28122,  51, { 28453,  28454,  28452,  28451,  28512,  28513,  28511,  28510}},
 { 28123,  51, { 28454,  28401,  28403,  28452,  28513,  28460,  28462,  28511}},
 { 28124,  51, { 28435,  28455,  28453,  28433,  28494,  28514,  28512,  28492}},
 { 28125,  51, { 28455,  28456,  28454,  28453,  28514,  28515,  28513,  28512}},
 { 28126,  51, { 28456,  28399,  28401,  28454,  28515,  28458,  28460,  28513}},
 { 28127,  51, { 28319,  28323,  28455,  28435,  28331,  28335,  28514,  28494}},
 { 28128,  51, { 28323,  28325,  28456,  28455,  28335,  28337,  28515,  28514}},
 { 28129,  51, { 28325,  28316,  28399,  28456,  28337,  28328,  28458,  28515}},
 { 28130,  51, { 28327,  28457,  28458,  28328,  27540,  27729,  27728,  27539}},
 { 28131,  51, { 28457,  28459,  28460,  28458,  27729,  27731,  27730,  27728}},
 { 28132,  51, { 28459,  28461,  28462,  28460,  27731,  27733,  27732,  27730}},
 { 28133,  51, { 28461,  28463,  28464,  28462,  27733,  27735,  27734,  27732}},
 { 28134,  51, { 28463,  28465,  28466,  28464,  27735,  27737,  27736,  27734}},
 { 28135,  51, { 28465,  28467,  28468,  28466,  27737,  27739,  27738,  27736}},
 { 28136,  51, { 28467,  28469,  28470,  28468,  27739,  27741,  27740,  27738}},
 { 28137,  51, { 28469,  28471,  28472,  28470,  27741,  27743,  27742,  27740}},
 { 28138,  51, { 28471,  28473,  28474,  28472,  27743,  27745,  27744,  27742}},
 { 28139,  51, { 28473,  28475,  28476,  28474,  27745,  27747,  27746,  27744}},
 { 28140,  51, { 28477,  28478,  28479,  28480,  27748,  27751,  27750,  27749}},
 { 28141,  51, { 28481,  28482,  28478,  28477,  27752,  27753,  27751,  27748}},
 { 28142,  51, { 28483,  28484,  28482,  28481,  27754,  27755,  27753,  27752}},
 { 28143,  51, { 28485,  28486,  28484,  28483,  27756,  27757,  27755,  27754}},
 { 28144,  51, { 28487,  28488,  28486,  28485,  27758,  27759,  27757,  27756}},
 { 28145,  51, { 28489,  28490,  28488,  28487,  27760,  27761,  27759,  27758}},
 { 28146,  51, { 28491,  28492,  28490,  28489,  27762,  27763,  27761,  27760}},
 { 28147,  51, { 28493,  28494,  28492,  28491,  27764,  27765,  27763,  27762}},
 { 28148,  51, { 28332,  28331,  28494,  28493,  27543,  27544,  27765,  27764}},
 { 28149,  51, { 28479,  28495,  28496,  28497,  27750,  27768,  27767,  27766}},
 { 28150,  51, { 28495,  28498,  28499,  28496,  27768,  27770,  27769,  27767}},
 { 28151,  51, { 28498,  28474,  28476,  28499,  27770,  27744,  27746,  27769}},
 { 28152,  51, { 28478,  28500,  28495,  28479,  27751,  27771,  27768,  27750}},
 { 28153,  51, { 28500,  28501,  28498,  28495,  27771,  27772,  27770,  27768}},
 { 28154,  51, { 28501,  28472,  28474,  28498,  27772,  27742,  27744,  27770}},
 { 28155,  51, { 28482,  28502,  28500,  28478,  27753,  27773,  27771,  27751}},
 { 28156,  51, { 28502,  28503,  28501,  28500,  27773,  27774,  27772,  27771}},
 { 28157,  51, { 28503,  28470,  28472,  28501,  27774,  27740,  27742,  27772}},
 { 28158,  51, { 28484,  28504,  28502,  28482,  27755,  27775,  27773,  27753}},
 { 28159,  51, { 28504,  28505,  28503,  28502,  27775,  27776,  27774,  27773}},
 { 28160,  51, { 28505,  28468,  28470,  28503,  27776,  27738,  27740,  27774}},
 { 28161,  51, { 28486,  28506,  28504,  28484,  27757,  27777,  27775,  27755}},
 { 28162,  51, { 28506,  28507,  28505,  28504,  27777,  27778,  27776,  27775}},
 { 28163,  51, { 28507,  28466,  28468,  28505,  27778,  27736,  27738,  27776}},
 { 28164,  51, { 28488,  28508,  28506,  28486,  27759,  27779,  27777,  27757}},
 { 28165,  51, { 28508,  28509,  28507,  28506,  27779,  27780,  27778,  27777}},
 { 28166,  51, { 28509,  28464,  28466,  28507,  27780,  27734,  27736,  27778}},
 { 28167,  51, { 28490,  28510,  28508,  28488,  27761,  27781,  27779,  27759}},
 { 28168,  51, { 28510,  28511,  28509,  28508,  27781,  27782,  27780,  27779}},
 { 28169,  51, { 28511,  28462,  28464,  28509,  27782,  27732,  27734,  27780}},
 { 28170,  51, { 28492,  28512,  28510,  28490,  27763,  27783,  27781,  27761}},
 { 28171,  51, { 28512,  28513,  28511,  28510,  27783,  27784,  27782,  27781}},
 { 28172,  51, { 28513,  28460,  28462,  28511,  27784,  27730,  27732,  27782}},
 { 28173,  51, { 28494,  28514,  28512,  28492,  27765,  27785,  27783,  27763}},
 { 28174,  51, { 28514,  28515,  28513,  28512,  27785,  27786,  27784,  27783}},
 { 28175,  51, { 28515,  28458,  28460,  28513,  27786,  27728,  27730,  27784}},
 { 28176,  51, { 28331,  28335,  28514,  28494,  27544,  27547,  27785,  27765}},
 { 28177,  51, { 28335,  28337,  28515,  28514,  27547,  27549,  27786,  27785}},
 { 28178,  51, { 28337,  28328,  28458,  28515,  27549,  27539,  27728,  27786}},
 { 28179,  51, { 27809,  27811,  27808,  27806,  27896,  27896,  27895,  27895}},
 { 28180,  51, { 28033,  27811,  27808,  27826,  27810,  27810,  27807,  27807}},
 { 28181,  51, { 27811,  28033,  27826,  27808,  28078,  28078,  27835,  27835}},
 { 28182,  51, { 27848,  27811,  27808,  27846,  28078,  28078,  27835,  27835}},
 { 28183,  51, { 27867,  28256,  27876,  27866,  27811,  27811,  27808,  27808}},
 { 28184,  51, { 27847,  27866,  27869,  27853,  27808,  27808,  27825,  27825}},
 { 28185,  51, { 27846,  27847,  27853,  27852,  27808,  27808,  27825,  27825}},
 { 28186,  51, { 27846,  27808,  27825,  27852,  27835,  27835,  27834,  27834}},
 { 28187,  51, { 27825,  27823,  27807,  27808,  27908,  27908,  27806,  27806}},
 { 28188,  51, { 27895,  27806,  27908,  27907,  27808,  27808,  27825,  27825}},
 { 28189,  51, { 27834,  27852,  27850,  27837,  27825,  27825,  27838,  27838}},
 { 28190,  51, { 27838,  27836,  27824,  27825,  27997,  27997,  27823,  27823}},
 { 28191,  51, { 27908,  27823,  27997,  27918,  27825,  27825,  27838,  27838}},
 { 28192,  51, { 27907,  27908,  27918,  27917,  27825,  27825,  27838,  27838}},
 { 28193,  51, { 27837,  27850,  27854,  27840,  27838,  27838,  27841,  27841}},
 { 28194,  51, { 27997,  27836,  27839,  28002,  27838,  27838,  27841,  27841}},
 { 28195,  51, { 27928,  27841,  27838,  27918,  28002,  28002,  27997,  27997}},
 { 28196,  51, { 27927,  27841,  27838,  27917,  27928,  27928,  27918,  27918}},
 { 28197,  51, { 27817,  27872,  27870,  27841,  27857,  27857,  27855,  27855}},
 { 28198,  51, { 27840,  27854,  27856,  28096,  27841,  27841,  27817,  27817}},
 { 28199,  51, { 28002,  27841,  27817,  27816,  27928,  27928,  27815,  27815}},
 { 28200,  51, { 27937,  27817,  27841,  27927,  27815,  27815,  27928,  27928}},
 { 28201,  51, { 28051,  27817,  27841,  27839,  28096,  28096,  27840,  27840}},
 { 28202,  51, { 27817,  28051,  27839,  27841,  27816,  27816,  28002,  28002}},
 { 28203,  51, { 27817,  27872,  27871,  27814,  27879,  27879,  28278,  28278}},
 { 28204,  51, { 27858,  27843,  28096,  27856,  27814,  27814,  27817,  27817}},
 { 28205,  51, { 28051,  28096,  27843,  27828,  27817,  27817,  27814,  27814}},
 { 28206,  51, { 28051,  27817,  27814,  27828,  27816,  27816,  27813,  27813}},
 { 28207,  51, { 27812,  27946,  27937,  27815,  27814,  27814,  27817,  27817}},
 { 28208,  51, { 27881,  27820,  27814,  27871,  27882,  27882,  28278,  28278}},
 { 28209,  51, { 27814,  27859,  27863,  27820,  27871,  27871,  27881,  27881}},
 { 28210,  51, { 27814,  27843,  27842,  27820,  27858,  27858,  27862,  27862}},
 { 28211,  51, { 27814,  27858,  27862,  27820,  27859,  27859,  27863,  27863}},
 { 28212,  51, { 27820,  27818,  27812,  27814,  27955,  27955,  27946,  27946}},
 { 28213,  51, { 28516,  28480,  28479,  28497,  27788,  27749,  27750,  27766}},
 { 28214,  51, { 28517,  28421,  28420,  28438,  28516,  28480,  28479,  28497}},
 { 28215,  51, { 28362,  28361,  28379,  28518,  28421,  28420,  28438,  28517}},
 { 28216,  51, { 28519,  28257,  28251,  28250,  28518,  28362,  28361,  28379}},
 { 28217,  51, { 28520,  28212,  28207,  28206,  28519,  28257,  28251,  28250}},
 { 28218,  51, { 28168,  28163,  28162,  28521,  28212,  28207,  28206,  28520}},
 { 28219,  51, { 28522,  28118,  28162,  28521,  28124,  28119,  28163,  28168}},
 { 28220,  51, { 28072,  28523,  28079,  28073,  28118,  28522,  28124,  28119}},
 { 28221,  51, { 28027,  28072,  28523,  28524,  28028,  28073,  28079,  28034}},
 { 28222,  51, { 28034,  28028,  27983,  27988,  28524,  28027,  27982,  28525}},
 { 28223,  51, { 27982,  27983,  27988,  28525,  27887,  27888,  27900,  28526}},
 { 28224,  51, { 27884,  27897,  27900,  27888,  27883,  28527,  28526,  27887}},
 { 28225,  51, { 27818,  27820,  27831,  27965,  27955,  27955,  27964,  27964}},
 { 28226,  51, { 27880,  27831,  27805,  28291,  27874,  27874,  27873,  27873}},
 { 28227,  51, { 27831,  27874,  27881,  27820,  27861,  27861,  27863,  27863}},
 { 28228,  51, { 27865,  27873,  28249,  28205,  27805,  27805,  27802,  27802}},
 { 28229,  51, { 28249,  27802,  27805,  27873,  28296,  28296,  28291,  28291}},
 { 28230,  51, { 27860,  28109,  27842,  27862,  27831,  27831,  27820,  27820}},
 { 28231,  51, { 28109,  27830,  27827,  27842,  27831,  27831,  27820,  27820}},
 { 28232,  51, { 27829,  27965,  27818,  27819,  27831,  27831,  27820,  27820}},
 { 28233,  51, { 27964,  27965,  27804,  27803,  27831,  27831,  27805,  27805}},
 { 28234,  51, { 28109,  27860,  27864,  27845,  27831,  27831,  27805,  27805}},
 { 28235,  51, { 27831,  27830,  27833,  27805,  28109,  28109,  27845,  27845}},
 { 28236,  51, { 27829,  27830,  27833,  27822,  27831,  27831,  27805,  27805}},
 { 28237,  51, { 27965,  27829,  27822,  27804,  27831,  27831,  27805,  27805}},
 { 28238,  51, { 27864,  27865,  28205,  28161,  27805,  27805,  27802,  27802}},
 { 28239,  51, { 27845,  27864,  28161,  27844,  27805,  27805,  27802,  27802}},
 {101404,  21, {101551, 101550,   3027,   3029, 101548, 101549,   9666,   9678}},
 {101405,  21, {  9678, 101548, 101549,   9666,   9702, 101578, 101579,   9696}},
 {101406,  21, {  3570, 101550, 101551,   3578,   9667, 101549, 101548,   9679}},
 {101407,  21, {  9667, 101549, 101548,   9679,   9697, 101579, 101578,   9703}},
 {101408,  21, {  9690,   9678, 101548, 101547,   3031,   3029, 101551, 101552}},
 {101409,  21, {  9690, 101547, 101548,   9678,   9708, 101577, 101578,   9702}},
 {101410,  21, {101552,   3586,   3578, 101551, 101547,   9691,   9679, 101548}},
 {101411,  21, {101547,   9691,   9679, 101548, 101577,   9709,   9703, 101578}},
 {101412,  21, {  3035, 101554, 101553,   3033,   9831, 101544, 101545,   9819}},
 {101413,  21, {  9831, 101544, 101545,   9819,   9855, 101575, 101576,   9849}},
 {101414,  21, {101554,   3602,   3594, 101553, 101544,   9832,   9820, 101545}},
 {101415,  21, {101544,   9832,   9820, 101545, 101575,   9856,   9850, 101576}},
 {101416,  21, {  3037, 101555, 101554,   3035,   9843, 101543, 101544,   9831}},
 {101417,  21, {  9843, 101543, 101544,   9831,   9861, 101574, 101575,   9855}},
 {101418,  21, {101555,   3610,   3602, 101554, 101543,   9844,   9832, 101544}},
 {101419,  21, {101543,   9844,   9832, 101544, 101574,   9862,   9856, 101575}},
 {101420,  21, {  3041, 101557, 101556,   3039,   9984, 101540, 101541,   9972}},
 {101421,  21, {  9984, 101540, 101541,   9972,  10008, 101572, 101573,  10002}},
 {101422,  21, {101557,   3626,   3618, 101556, 101540,   9985,   9973, 101541}},
 {101423,  21, {101540,   9985,   9973, 101541, 101572,  10009,  10003, 101573}},
 {101424,  21, {  3043, 101558, 101557,   3041,   9996, 101539, 101540,   9984}},
 {101425,  21, {  9996, 101539, 101540,   9984,  10014, 101571, 101572,  10008}},
 {101426,  21, {101558,   3634,   3626, 101557, 101539,   9997,   9985, 101540}},
 {101427,  21, {101539,   9997,   9985, 101540, 101571,  10015,  10009, 101572}},
 {101428,  21, {101560, 101559,   3045,   3047, 101536, 101537,  10125,  10137}},
 {101429,  21, { 10137, 101536, 101537,  10125,  10161, 101569, 101570,  10155}},
 {101430,  21, {101560,   3650,   3642, 101559, 101536,  10138,  10126, 101537}},
 {101431,  21, {101536,  10138,  10126, 101537, 101569,  10162,  10156, 101570}},
 {101432,  21, {  3049, 101561, 101560,   3047,  10149, 101535, 101536,  10137}},
 {101433,  21, { 10149, 101535, 101536,  10137,  10167, 101568, 101569,  10161}},
 {101434,  21, {101561,   3658,   3650, 101560, 101535,  10150,  10138, 101536}},
 {101435,  21, {101535,  10150,  10138, 101536, 101568,  10168,  10162, 101569}},
 {101436,  21, {  3053, 101563, 101562,   3051,  10290, 101532, 101533,  10278}},
 {101437,  21, { 10290, 101532, 101533,  10278,  10314, 101566, 101567,  10308}},
 {101438,  21, {101563,   3674,   3666, 101562, 101532,  10291,  10279, 101533}},
 {101439,  21, {101532,  10291,  10279, 101533, 101566,  10315,  10309, 101567}},
 {101440,  21, {  3055, 101564, 101563,   3053,  10302, 101531, 101532,  10290}},
 {101441,  21, { 10302, 101531, 101532,  10290,  10320, 101565, 101566,  10314}},
 {101442,  21, {101564,   3682,   3674, 101563, 101531,  10303,  10291, 101532}},
 {101443,  21, {101531,  10303,  10291, 101532, 101565,  10321,  10315, 101566}},
 {101464,  21, {  3058, 101599, 101601,   3062,  10455, 101598, 101597,  10443}},
 {101465,  21, { 10455, 101598, 101597,  10443,  10473, 101600, 101602,  10467}},
 {101466,  21, {101599,   3453,   3461, 101601, 101598,  10456,  10444, 101597}},
 {101467,  21, {101598,  10456,  10444, 101597, 101600,  10474,  10468, 101602}},
 {101468,  21, {  3062, 101601, 101603,   3066,  10443, 101597, 101596,  10431}},
 {101469,  21, { 10443, 101597, 101596,  10431,  10467, 101602, 101604,  10461}},
 {101470,  21, {101601,   3461,   3469, 101603, 101597,  10444,  10432, 101596}},
 {101471,  21, {101597,  10444,  10432, 101596, 101602,  10468,  10462, 101604}},
 {101472,  21, {  3070, 101605, 101606,   3074,  10608, 101594, 101593,  10596}},
 {101473,  21, { 10608, 101594, 101593,  10596,  10626, 101608, 101609,  10620}},
 {101474,  21, {101605,   3477,   3485, 101606, 101594,  10609,  10597, 101593}},
 {101475,  21, {101594,  10609,  10597, 101593, 101608,  10627,  10621, 101609}},
 {101476,  21, {  3074, 101606, 101607,   3078,  10596, 101593, 101592,  10584}},
 {101477,  21, { 10596, 101593, 101592,  10584,  10620, 101609, 101610,  10614}},
 {101478,  21, {101606,   3485,   3493, 101607, 101593,  10597,  10585, 101592}},
 {101479,  21, {101593,  10597,  10585, 101592, 101609,  10621,  10615, 101610}},
 {101480,  21, {  3082, 101611, 101612,   3086,  10761, 101590, 101589,  10749}},
 {101481,  21, { 10761, 101590, 101589,  10749,  10779, 101614, 101615,  10773}},
 {101482,  21, {101611,   3501,   3509, 101612, 101590,  10762,  10750, 101589}},
 {101483,  21, {101590,  10762,  10750, 101589, 101614,  10780,  10774, 101615}},
 {101484,  21, {  3086, 101612, 101613,   3090,  10749, 101589, 101588,  10737}},
 {101485,  21, { 10749, 101589, 101588,  10737,  10773, 101615, 101616,  10767}},
 {101486,  21, {101612,   3509,   3517, 101613, 101589,  10750,  10738, 101588}},
 {101487,  21, {101589,  10750,  10738, 101588, 101615,  10774,  10768, 101616}},
 {101488,  21, {  3094, 101620, 101621,   3098,  10914, 101586, 101585,  10902}},
 {101489,  21, { 10914, 101586, 101585,  10902,  10932, 101617, 101618,  10926}},
 {101490,  21, {101620,   3525,   3533, 101621, 101586,  10915,  10903, 101585}},
 {101491,  21, {101586,  10915,  10903, 101585, 101617,  10933,  10927, 101618}},
 {101492,  21, {  3098, 101621, 101622,   3102,  10902, 101585, 101584,  10890}},
 {101493,  21, { 10902, 101585, 101584,  10890,  10926, 101618, 101619,  10920}},
 {101494,  21, {101621,   3533,   3541, 101622, 101585,  10903,  10891, 101584}},
 {101495,  21, {101585,  10903,  10891, 101584, 101618,  10927,  10921, 101619}},
 {101496,  21, {  3106, 101626, 101627,   3110,  11067, 101582, 101581,  11055}},
 {101497,  21, { 11067, 101582, 101581,  11055,  11085, 101623, 101624,  11079}},
 {101498,  21, {  3110, 101627, 101628,   3114,  11055, 101581, 101580,  11043}},
 {101499,  21, {101627,   3557,   3565, 101628, 101581,  11056,  11044, 101580}},
 {101500,  21, {101581,  11056,  11044, 101580, 101624,  11080,  11074, 101625}},
 {101501,  21, {101580,  11043,  11055, 101581, 101625,  11073,  11079, 101624}},
 {101502,  21, {  3557, 101627, 101626,   3549,  11056, 101581, 101582,  11068}},
 {101503,  21, { 11056, 101581, 101582,  11068,  11080, 101624, 101623,  11086}}
};

struct beam beams[NBEAMS] = {
 {24982,  57,  {24209,  24210,  22656}},
 {24983,  58,  {24211,  24212,  23554}},
 {24984,  59,  {24213,  24214,  22968}},
 {24985,  60,  {24215,  24216,  23793}}
};

struct shell shells[NSHELLS] = {
 {     1,  25, {  5026,   5035,   5164,   5161}},
 {     2,  25, {  5035,   5044,   5167,   5164}},
 {     3,  25, {  5161,   5164,  16337,  16336}},
 {     4,  25, { 16336,  16337,   9576,   9572}},
 {     5,  25, {  5164,   5167,  16338,  16337}},
 {     6,  25, { 16337,  16338,   9580,   9576}},
 {     7,  25, {  9572,   9576,   9577,   9573}},
 {     8,  25, {  9576,   9580,   9581,   9577}},
 {     9,  25, {  9573,   9577,   9606,   9594}},
 {    10,  25, {  9594,   9606,   9607,   9595}},
 {    11,  25, {  9595,   9607,   9608,   9596}},
 {    12,  25, {  9596,   9608,   9609,   9597}},
 {    13,  25, {  9597,   9609,   9610,   9598}},
 {    14,  25, {  9598,   9610,   9611,   9599}},
 {    15,  25, {  9577,   9581,   9618,   9606}},
 {    16,  25, {  9606,   9618,   9619,   9607}},
 {    17,  25, {  9607,   9619,   9620,   9608}},
 {    18,  25, {  9608,   9620,   9621,   9609}},
 {    19,  25, {  9609,   9621,   9622,   9610}},
 {    20,  25, {  9610,   9622,   9623,   9611}},
 {    21,  25, {  9599,   9611,   9648,   9644}},
 {    22,  25, {  9644,   9648,   9649,   9645}},
 {    23,  25, {  9611,   9623,   9652,   9648}},
 {    24,  25, {  9648,   9652,   9653,   9649}},
 {    25,  25, {  9645,   9649,   9678,   9666}},
 {    27,  25, {  9667,   9679,   9680,   9668}},
 {    28,  25, {  9668,   9680,   9681,   9669}},
 {    29,  25, {  9669,   9681,   9682,   9670}},
 {    30,  25, {  9670,   9682,   9683,   9671}},
 {    31,  25, {  9649,   9653,   9690,   9678}},
 {    33,  25, {  9679,   9691,   9692,   9680}},
 {    34,  25, {  9680,   9692,   9693,   9681}},
 {    35,  25, {  9681,   9693,   9694,   9682}},
 {    36,  25, {  9682,   9694,   9695,   9683}},
 {    37,  25, {  9671,   9683,   9717,   9715}},
 {    38,  25, {  9683,   9695,   9719,   9717}},
 {    39,  25, {  9715,   9717,   6541,   6539}},
 {    40,  25, {  9717,   9719,   6543,   6541}},
 {    41,  25, {  6539,   6541,   6540,   6538}},
 {    42,  25, {  6538,   6540,   6524,   6523}},
 {    43,  25, {  6541,   6543,   6542,   6540}},
 {    44,  25, {  6540,   6542,   6525,   6524}},
 {    45,  25, {  5053,   5062,   5173,   5170}},
 {    46,  25, {  5062,   5071,   5176,   5173}},
 {    47,  25, {  5170,   5173,  16340,  16339}},
 {    48,  25, { 16339,  16340,   9729,   9725}},
 {    49,  25, {  5173,   5176,  16341,  16340}},
 {    50,  25, { 16340,  16341,   9733,   9729}},
 {    51,  25, {  9725,   9729,   9730,   9726}},
 {    52,  25, {  9729,   9733,   9734,   9730}},
 {    53,  25, {  9726,   9730,   9759,   9747}},
 {    54,  25, {  9747,   9759,   9760,   9748}},
 {    55,  25, {  9748,   9760,   9761,   9749}},
 {    56,  25, {  9749,   9761,   9762,   9750}},
 {    57,  25, {  9750,   9762,   9763,   9751}},
 {    58,  25, {  9751,   9763,   9764,   9752}},
 {    59,  25, {  9730,   9734,   9771,   9759}},
 {    60,  25, {  9759,   9771,   9772,   9760}},
 {    61,  25, {  9760,   9772,   9773,   9761}},
 {    62,  25, {  9761,   9773,   9774,   9762}},
 {    63,  25, {  9762,   9774,   9775,   9763}},
 {    64,  25, {  9763,   9775,   9776,   9764}},
 {    65,  25, {  9752,   9764,   9801,   9797}},
 {    66,  25, {  9797,   9801,   9802,   9798}},
 {    67,  25, {  9764,   9776,   9805,   9801}},
 {    68,  25, {  9801,   9805,   9806,   9802}},
 {    69,  25, {  9798,   9802,   9831,   9819}},
 {    71,  25, {  9820,   9832,   9833,   9821}},
 {    72,  25, {  9821,   9833,   9834,   9822}},
 {    73,  25, {  9822,   9834,   9835,   9823}},
 {    74,  25, {  9823,   9835,   9836,   9824}},
 {    75,  25, {  9802,   9806,   9843,   9831}},
 {    77,  25, {  9832,   9844,   9845,   9833}},
 {    78,  25, {  9833,   9845,   9846,   9834}},
 {    79,  25, {  9834,   9846,   9847,   9835}},
 {    80,  25, {  9835,   9847,   9848,   9836}},
 {    81,  25, {  9824,   9836,   9870,   9868}},
 {    82,  25, {  9836,   9848,   9872,   9870}},
 {    83,  25, {  9868,   9870,   6547,   6545}},
 {    84,  25, {  9870,   9872,   6549,   6547}},
 {    85,  25, {  6545,   6547,   6546,   6544}},
 {    86,  25, {  6544,   6546,   6527,   6526}},
 {    87,  25, {  6547,   6549,   6548,   6546}},
 {    88,  25, {  6546,   6548,   6528,   6527}},
 {    89,  25, {  5080,   5089,   5182,   5179}},
 {    90,  25, {  5089,   5098,   5185,   5182}},
 {    91,  25, {  5179,   5182,  16343,  16342}},
 {    92,  25, { 16342,  16343,   9882,   9878}},
 {    93,  25, {  5182,   5185,  16344,  16343}},
 {    94,  25, { 16343,  16344,   9886,   9882}},
 {    95,  25, {  9878,   9882,   9883,   9879}},
 {    96,  25, {  9882,   9886,   9887,   9883}},
 {    97,  25, {  9879,   9883,   9912,   9900}},
 {    98,  25, {  9900,   9912,   9913,   9901}},
 {    99,  25, {  9901,   9913,   9914,   9902}},
 {   100,  25, {  9902,   9914,   9915,   9903}},
 {   101,  25, {  9903,   9915,   9916,   9904}},
 {   102,  25, {  9904,   9916,   9917,   9905}},
 {   103,  25, {  9883,   9887,   9924,   9912}},
 {   104,  25, {  9912,   9924,   9925,   9913}},
 {   105,  25, {  9913,   9925,   9926,   9914}},
 {   106,  25, {  9914,   9926,   9927,   9915}},
 {   107,  25, {  9915,   9927,   9928,   9916}},
 {   108,  25, {  9916,   9928,   9929,   9917}},
 {   109,  25, {  9905,   9917,   9954,   9950}},
 {   110,  25, {  9950,   9954,   9955,   9951}},
 {   111,  25, {  9917,   9929,   9958,   9954}},
 {   112,  25, {  9954,   9958,   9959,   9955}},
 {   113,  25, {  9951,   9955,   9984,   9972}},
 {   115,  25, {  9973,   9985,   9986,   9974}},
 {   116,  25, {  9974,   9986,   9987,   9975}},
 {   117,  25, {  9975,   9987,   9988,   9976}},
 {   118,  25, {  9976,   9988,   9989,   9977}},
 {   119,  25, {  9955,   9959,   9996,   9984}},
 {   121,  25, {  9985,   9997,   9998,   9986}},
 {   122,  25, {  9986,   9998,   9999,   9987}},
 {   123,  25, {  9987,   9999,  10000,   9988}},
 {   124,  25, {  9988,  10000,  10001,   9989}},
 {   125,  25, {  9977,   9989,  10023,  10021}},
 {   126,  25, {  9989,  10001,  10025,  10023}},
 {   127,  25, { 10021,  10023,   6553,   6551}},
 {   128,  25, { 10023,  10025,   6555,   6553}},
 {   129,  25, {  6551,   6553,   6552,   6550}},
 {   130,  25, {  6550,   6552,   6530,   6529}},
 {   131,  25, {  6553,   6555,   6554,   6552}},
 {   132,  25, {  6552,   6554,   6531,   6530}},
 {   133,  25, {  5107,   5116,   5191,   5188}},
 {   134,  25, {  5116,   5125,   5194,   5191}},
 {   135,  25, {  5188,   5191,  16346,  16345}},
 {   136,  25, { 16345,  16346,  10035,  10031}},
 {   137,  25, {  5191,   5194,  16347,  16346}},
 {   138,  25, { 16346,  16347,  10039,  10035}},
 {   139,  25, { 10031,  10035,  10036,  10032}},
 {   140,  25, { 10035,  10039,  10040,  10036}},
 {   141,  25, { 10032,  10036,  10065,  10053}},
 {   142,  25, { 10053,  10065,  10066,  10054}},
 {   143,  25, { 10054,  10066,  10067,  10055}},
 {   144,  25, { 10055,  10067,  10068,  10056}},
 {   145,  25, { 10056,  10068,  10069,  10057}},
 {   146,  25, { 10057,  10069,  10070,  10058}},
 {   147,  25, { 10036,  10040,  10077,  10065}},
 {   148,  25, { 10065,  10077,  10078,  10066}},
 {   149,  25, { 10066,  10078,  10079,  10067}},
 {   150,  25, { 10067,  10079,  10080,  10068}},
 {   151,  25, { 10068,  10080,  10081,  10069}},
 {   152,  25, { 10069,  10081,  10082,  10070}},
 {   153,  25, { 10058,  10070,  10107,  10103}},
 {   154,  25, { 10103,  10107,  10108,  10104}},
 {   155,  25, { 10070,  10082,  10111,  10107}},
 {   156,  25, { 10107,  10111,  10112,  10108}},
 {   157,  25, { 10104,  10108,  10137,  10125}},
 {   159,  25, { 10126,  10138,  10139,  10127}},
 {   160,  25, { 10127,  10139,  10140,  10128}},
 {   161,  25, { 10128,  10140,  10141,  10129}},
 {   162,  25, { 10129,  10141,  10142,  10130}},
 {   163,  25, { 10108,  10112,  10149,  10137}},
 {   165,  25, { 10138,  10150,  10151,  10139}},
 {   166,  25, { 10139,  10151,  10152,  10140}},
 {   167,  25, { 10140,  10152,  10153,  10141}},
 {   168,  25, { 10141,  10153,  10154,  10142}},
 {   169,  25, { 10130,  10142,  10176,  10174}},
 {   170,  25, { 10142,  10154,  10178,  10176}},
 {   171,  25, { 10174,  10176,   6559,   6557}},
 {   172,  25, { 10176,  10178,   6561,   6559}},
 {   173,  25, {  6557,   6559,   6558,   6556}},
 {   174,  25, {  6556,   6558,   6533,   6532}},
 {   175,  25, {  6559,   6561,   6560,   6558}},
 {   176,  25, {  6558,   6560,   6534,   6533}},
 {   177,  25, {  5134,   5143,   5200,   5197}},
 {   178,  25, {  5143,   5152,   5203,   5200}},
 {   179,  25, {  5197,   5200,  16349,  16348}},
 {   180,  25, { 16348,  16349,  10188,  10184}},
 {   181,  25, {  5200,   5203,  16350,  16349}},
 {   182,  25, { 16349,  16350,  10192,  10188}},
 {   183,  25, { 10184,  10188,  10189,  10185}},
 {   184,  25, { 10188,  10192,  10193,  10189}},
 {   185,  25, { 10185,  10189,  10218,  10206}},
 {   186,  25, { 10206,  10218,  10219,  10207}},
 {   187,  25, { 10207,  10219,  10220,  10208}},
 {   188,  25, { 10208,  10220,  10221,  10209}},
 {   189,  25, { 10209,  10221,  10222,  10210}},
 {   190,  25, { 10210,  10222,  10223,  10211}},
 {   191,  25, { 10189,  10193,  10230,  10218}},
 {   192,  25, { 10218,  10230,  10231,  10219}},
 {   193,  25, { 10219,  10231,  10232,  10220}},
 {   194,  25, { 10220,  10232,  10233,  10221}},
 {   195,  25, { 10221,  10233,  10234,  10222}},
 {   196,  25, { 10222,  10234,  10235,  10223}},
 {   197,  25, { 10211,  10223,  10260,  10256}},
 {   198,  25, { 10256,  10260,  10261,  10257}},
 {   199,  25, { 10223,  10235,  10264,  10260}},
 {   200,  25, { 10260,  10264,  10265,  10261}},
 {   201,  25, { 10257,  10261,  10290,  10278}},
 {   203,  25, { 10279,  10291,  10292,  10280}},
 {   204,  25, { 10280,  10292,  10293,  10281}},
 {   205,  25, { 10281,  10293,  10294,  10282}},
 {   206,  25, { 10282,  10294,  10295,  10283}},
 {   207,  25, { 10261,  10265,  10302,  10290}},
 {   209,  25, { 10291,  10303,  10304,  10292}},
 {   210,  25, { 10292,  10304,  10305,  10293}},
 {   211,  25, { 10293,  10305,  10306,  10294}},
 {   212,  25, { 10294,  10306,  10307,  10295}},
 {   213,  25, { 10283,  10295,  10329,  10327}},
 {   214,  25, { 10295,  10307,  10331,  10329}},
 {   215,  25, { 10327,  10329,   6565,   6563}},
 {   216,  25, { 10329,  10331,   6567,   6565}},
 {   217,  25, {  6563,   6565,   6564,   6562}},
 {   218,  25, {  6562,   6564,   6536,   6535}},
 {   219,  25, {  6565,   6567,   6566,   6564}},
 {   220,  25, {  6564,   6566,   6537,   6536}},
 {   221,  25, {  4734,   4728,   4725,   4731}},
 {   222,  25, {  4728,   4722,   4719,   4725}},
 {   223,  25, {  4731,   4725,  16352,  16351}},
 {   224,  25, { 16351,  16352,  10341,  10337}},
 {   225,  25, {  4725,   4719,  16353,  16352}},
 {   226,  25, { 16352,  16353,  10345,  10341}},
 {   227,  25, { 10337,  10341,  10342,  10338}},
 {   228,  25, { 10341,  10345,  10346,  10342}},
 {   229,  25, { 10338,  10342,  10371,  10359}},
 {   230,  25, { 10359,  10371,  10372,  10360}},
 {   231,  25, { 10360,  10372,  10373,  10361}},
 {   232,  25, { 10361,  10373,  10374,  10362}},
 {   233,  25, { 10362,  10374,  10375,  10363}},
 {   234,  25, { 10363,  10375,  10376,  10364}},
 {   235,  25, { 10342,  10346,  10383,  10371}},
 {   236,  25, { 10371,  10383,  10384,  10372}},
 {   237,  25, { 10372,  10384,  10385,  10373}},
 {   238,  25, { 10373,  10385,  10386,  10374}},
 {   239,  25, { 10374,  10386,  10387,  10375}},
 {   240,  25, { 10375,  10387,  10388,  10376}},
 {   241,  25, { 10364,  10376,  10413,  10409}},
 {   242,  25, { 10409,  10413,  10414,  10410}},
 {   243,  25, { 10376,  10388,  10417,  10413}},
 {   244,  25, { 10413,  10417,  10418,  10414}},
 {   245,  25, { 10410,  10414,  10443,  10431}},
 {   247,  25, { 10432,  10444,  10445,  10433}},
 {   248,  25, { 10433,  10445,  10446,  10434}},
 {   249,  25, { 10434,  10446,  10447,  10435}},
 {   250,  25, { 10435,  10447,  10448,  10436}},
 {   251,  25, { 10414,  10418,  10455,  10443}},
 {   253,  25, { 10444,  10456,  10457,  10445}},
 {   254,  25, { 10445,  10457,  10458,  10446}},
 {   255,  25, { 10446,  10458,  10459,  10447}},
 {   256,  25, { 10447,  10459,  10460,  10448}},
 {   257,  25, { 10436,  10448,  10482,  10480}},
 {   258,  25, { 10448,  10460,  10484,  10482}},
 {   259,  25, { 10480,  10482,   6481,   6484}},
 {   260,  25, { 10482,  10484,   6478,   6481}},
 {   261,  25, {  6484,   6481,   6482,   6485}},
 {   262,  25, {  6485,   6482,   6483,   6486}},
 {   263,  25, {  6481,   6478,   6479,   6482}},
 {   264,  25, {  6482,   6479,   6480,   6483}},
 {   265,  25, {  4752,   4746,   4743,   4749}},
 {   266,  25, {  4746,   4740,   4737,   4743}},
 {   267,  25, {  4749,   4743,  16355,  16354}},
 {   268,  25, { 16354,  16355,  10494,  10490}},
 {   269,  25, {  4743,   4737,  16356,  16355}},
 {   270,  25, { 16355,  16356,  10498,  10494}},
 {   271,  25, { 10490,  10494,  10495,  10491}},
 {   272,  25, { 10494,  10498,  10499,  10495}},
 {   273,  25, { 10491,  10495,  10524,  10512}},
 {   274,  25, { 10512,  10524,  10525,  10513}},
 {   275,  25, { 10513,  10525,  10526,  10514}},
 {   276,  25, { 10514,  10526,  10527,  10515}},
 {   277,  25, { 10515,  10527,  10528,  10516}},
 {   278,  25, { 10516,  10528,  10529,  10517}},
 {   279,  25, { 10495,  10499,  10536,  10524}},
 {   280,  25, { 10524,  10536,  10537,  10525}},
 {   281,  25, { 10525,  10537,  10538,  10526}},
 {   282,  25, { 10526,  10538,  10539,  10527}},
 {   283,  25, { 10527,  10539,  10540,  10528}},
 {   284,  25, { 10528,  10540,  10541,  10529}},
 {   285,  25, { 10517,  10529,  10566,  10562}},
 {   286,  25, { 10562,  10566,  10567,  10563}},
 {   287,  25, { 10529,  10541,  10570,  10566}},
 {   288,  25, { 10566,  10570,  10571,  10567}},
 {   289,  25, { 10563,  10567,  10596,  10584}},
 {   291,  25, { 10585,  10597,  10598,  10586}},
 {   292,  25, { 10586,  10598,  10599,  10587}},
 {   293,  25, { 10587,  10599,  10600,  10588}},
 {   294,  25, { 10588,  10600,  10601,  10589}},
 {   295,  25, { 10567,  10571,  10608,  10596}},
 {   297,  25, { 10597,  10609,  10610,  10598}},
 {   298,  25, { 10598,  10610,  10611,  10599}},
 {   299,  25, { 10599,  10611,  10612,  10600}},
 {   300,  25, { 10600,  10612,  10613,  10601}},
 {   301,  25, { 10589,  10601,  10635,  10633}},
 {   302,  25, { 10601,  10613,  10637,  10635}},
 {   303,  25, { 10633,  10635,   6490,   6493}},
 {   304,  25, { 10635,  10637,   6487,   6490}},
 {   305,  25, {  6493,   6490,   6491,   6494}},
 {   306,  25, {  6494,   6491,   6492,   6495}},
 {   307,  25, {  6490,   6487,   6488,   6491}},
 {   308,  25, {  6491,   6488,   6489,   6492}},
 {   309,  25, {  4770,   4764,   4761,   4767}},
 {   310,  25, {  4764,   4758,   4755,   4761}},
 {   311,  25, {  4767,   4761,  16358,  16357}},
 {   312,  25, { 16357,  16358,  10647,  10643}},
 {   313,  25, {  4761,   4755,  16359,  16358}},
 {   314,  25, { 16358,  16359,  10651,  10647}},
 {   315,  25, { 10643,  10647,  10648,  10644}},
 {   316,  25, { 10647,  10651,  10652,  10648}},
 {   317,  25, { 10644,  10648,  10677,  10665}},
 {   318,  25, { 10665,  10677,  10678,  10666}},
 {   319,  25, { 10666,  10678,  10679,  10667}},
 {   320,  25, { 10667,  10679,  10680,  10668}},
 {   321,  25, { 10668,  10680,  10681,  10669}},
 {   322,  25, { 10669,  10681,  10682,  10670}},
 {   323,  25, { 10648,  10652,  10689,  10677}},
 {   324,  25, { 10677,  10689,  10690,  10678}},
 {   325,  25, { 10678,  10690,  10691,  10679}},
 {   326,  25, { 10679,  10691,  10692,  10680}},
 {   327,  25, { 10680,  10692,  10693,  10681}},
 {   328,  25, { 10681,  10693,  10694,  10682}},
 {   329,  25, { 10670,  10682,  10719,  10715}},
 {   330,  25, { 10715,  10719,  10720,  10716}},
 {   331,  25, { 10682,  10694,  10723,  10719}},
 {   332,  25, { 10719,  10723,  10724,  10720}},
 {   333,  25, { 10716,  10720,  10749,  10737}},
 {   335,  25, { 10738,  10750,  10751,  10739}},
 {   336,  25, { 10739,  10751,  10752,  10740}},
 {   337,  25, { 10740,  10752,  10753,  10741}},
 {   338,  25, { 10741,  10753,  10754,  10742}},
 {   339,  25, { 10720,  10724,  10761,  10749}},
 {   341,  25, { 10750,  10762,  10763,  10751}},
 {   342,  25, { 10751,  10763,  10764,  10752}},
 {   343,  25, { 10752,  10764,  10765,  10753}},
 {   344,  25, { 10753,  10765,  10766,  10754}},
 {   345,  25, { 10742,  10754,  10788,  10786}},
 {   346,  25, { 10754,  10766,  10790,  10788}},
 {   347,  25, { 10786,  10788,   6499,   6502}},
 {   348,  25, { 10788,  10790,   6496,   6499}},
 {   349,  25, {  6502,   6499,   6500,   6503}},
 {   350,  25, {  6503,   6500,   6501,   6504}},
 {   351,  25, {  6499,   6496,   6497,   6500}},
 {   352,  25, {  6500,   6497,   6498,   6501}},
 {   353,  25, {  4788,   4782,   4779,   4785}},
 {   354,  25, {  4782,   4776,   4773,   4779}},
 {   355,  25, {  4785,   4779,  16361,  16360}},
 {   356,  25, { 16360,  16361,  10800,  10796}},
 {   357,  25, {  4779,   4773,  16362,  16361}},
 {   358,  25, { 16361,  16362,  10804,  10800}},
 {   359,  25, { 10796,  10800,  10801,  10797}},
 {   360,  25, { 10800,  10804,  10805,  10801}},
 {   361,  25, { 10797,  10801,  10830,  10818}},
 {   362,  25, { 10818,  10830,  10831,  10819}},
 {   363,  25, { 10819,  10831,  10832,  10820}},
 {   364,  25, { 10820,  10832,  10833,  10821}},
 {   365,  25, { 10821,  10833,  10834,  10822}},
 {   366,  25, { 10822,  10834,  10835,  10823}},
 {   367,  25, { 10801,  10805,  10842,  10830}},
 {   368,  25, { 10830,  10842,  10843,  10831}},
 {   369,  25, { 10831,  10843,  10844,  10832}},
 {   370,  25, { 10832,  10844,  10845,  10833}},
 {   371,  25, { 10833,  10845,  10846,  10834}},
 {   372,  25, { 10834,  10846,  10847,  10835}},
 {   373,  25, { 10823,  10835,  10872,  10868}},
 {   374,  25, { 10868,  10872,  10873,  10869}},
 {   375,  25, { 10835,  10847,  10876,  10872}},
 {   376,  25, { 10872,  10876,  10877,  10873}},
 {   377,  25, { 10869,  10873,  10902,  10890}},
 {   379,  25, { 10891,  10903,  10904,  10892}},
 {   380,  25, { 10892,  10904,  10905,  10893}},
 {   381,  25, { 10893,  10905,  10906,  10894}},
 {   382,  25, { 10894,  10906,  10907,  10895}},
 {   383,  25, { 10873,  10877,  10914,  10902}},
 {   385,  25, { 10903,  10915,  10916,  10904}},
 {   386,  25, { 10904,  10916,  10917,  10905}},
 {   387,  25, { 10905,  10917,  10918,  10906}},
 {   388,  25, { 10906,  10918,  10919,  10907}},
 {   389,  25, { 10895,  10907,  10941,  10939}},
 {   390,  25, { 10907,  10919,  10943,  10941}},
 {   391,  25, { 10939,  10941,   6508,   6511}},
 {   392,  25, { 10941,  10943,   6505,   6508}},
 {   393,  25, {  6511,   6508,   6509,   6512}},
 {   394,  25, {  6512,   6509,   6510,   6513}},
 {   395,  25, {  6508,   6505,   6506,   6509}},
 {   396,  25, {  6509,   6506,   6507,   6510}},
 {   397,  25, {  4806,   4800,   4797,   4803}},
 {   398,  25, {  4800,   4794,   4791,   4797}},
 {   399,  25, {  4803,   4797,  16364,  16363}},
 {   400,  25, { 16363,  16364,  10953,  10949}},
 {   401,  25, {  4797,   4791,  16365,  16364}},
 {   402,  25, { 16364,  16365,  10957,  10953}},
 {   403,  25, { 10949,  10953,  10954,  10950}},
 {   404,  25, { 10953,  10957,  10958,  10954}},
 {   405,  25, { 10950,  10954,  10983,  10971}},
 {   406,  25, { 10971,  10983,  10984,  10972}},
 {   407,  25, { 10972,  10984,  10985,  10973}},
 {   408,  25, { 10973,  10985,  10986,  10974}},
 {   409,  25, { 10974,  10986,  10987,  10975}},
 {   410,  25, { 10975,  10987,  10988,  10976}},
 {   411,  25, { 10954,  10958,  10995,  10983}},
 {   412,  25, { 10983,  10995,  10996,  10984}},
 {   413,  25, { 10984,  10996,  10997,  10985}},
 {   414,  25, { 10985,  10997,  10998,  10986}},
 {   415,  25, { 10986,  10998,  10999,  10987}},
 {   416,  25, { 10987,  10999,  11000,  10988}},
 {   417,  25, { 10976,  10988,  11025,  11021}},
 {   418,  25, { 11021,  11025,  11026,  11022}},
 {   419,  25, { 10988,  11000,  11029,  11025}},
 {   420,  25, { 11025,  11029,  11030,  11026}},
 {   421,  25, { 11022,  11026,  11055,  11043}},
 {   423,  25, { 11044,  11056,  11057,  11045}},
 {   424,  25, { 11045,  11057,  11058,  11046}},
 {   425,  25, { 11046,  11058,  11059,  11047}},
 {   426,  25, { 11047,  11059,  11060,  11048}},
 {   427,  25, { 11026,  11030,  11067,  11055}},
 {   429,  25, { 11056,  11068,  11069,  11057}},
 {   430,  25, { 11057,  11069,  11070,  11058}},
 {   431,  25, { 11058,  11070,  11071,  11059}},
 {   432,  25, { 11059,  11071,  11072,  11060}},
 {   433,  25, { 11048,  11060,  11094,  11092}},
 {   434,  25, { 11060,  11072,  11096,  11094}},
 {   435,  25, { 11092,  11094,   6517,   6520}},
 {   436,  25, { 11094,  11096,   6514,   6517}},
 {   437,  25, {  6520,   6517,   6518,   6521}},
 {   438,  25, {  6521,   6518,   6519,   6522}},
 {   439,  25, {  6517,   6514,   6515,   6518}},
 {   440,  25, {  6518,   6515,   6516,   6519}},
 {   441,  26, { 11112,  11116,  11117,  11113}},
 {   442,  26, { 11113,  11117,  11118,  11114}},
 {   443,  26, { 11114,  11118,  11119,  11115}},
 {   444,  26, { 11116,      6,   1407,  11117}},
 {   445,  26, { 11117,   1407,   1408,  11118}},
 {   446,  26, { 11118,   1408,   1409,  11119}},
 {   447,  26, {     6,   1422,   1423,   1407}},
 {   448,  26, {  1407,   1423,   1424,   1408}},
 {   449,  26, {  1408,   1424,   1425,   1409}},
 {   450,  26, {  1422,   1478,   1479,   1423}},
 {   451,  26, {  1423,   1479,   1480,   1424}},
 {   452,  26, {  1424,   1480,   1481,   1425}},
 {   453,  26, {  1478,   1486,   1487,   1479}},
 {   454,  26, {  1479,   1487,   1488,   1480}},
 {   455,  26, {  1480,   1488,   1489,   1481}},
 {   456,  26, {  1486,   1542,   1543,   1487}},
 {   457,  26, {  1487,   1543,   1544,   1488}},
 {   458,  26, {  1488,   1544,   1545,   1489}},
 {   459,  26, {  1542,     84,   1598,   1543}},
 {   460,  26, {  1543,   1598,   1599,   1544}},
 {   461,  26, {  1544,   1599,   1600,   1545}},
 {   462,  26, {    84,  11156,  11157,   1598}},
 {   463,  26, {  1598,  11157,  11158,   1599}},
 {   464,  26, {  1599,  11158,  11159,   1600}},
 {   465,  26, { 11156,  11160,  11161,  11157}},
 {   466,  26, { 11157,  11161,  11162,  11158}},
 {   467,  26, { 11158,  11162,  11163,  11159}},
 {   468,  26, { 11115,  11119,  11200,  11197}},
 {   469,  26, { 11197,  11200,  11201,  11198}},
 {   470,  26, { 11198,  11201,  11202,  11199}},
 {   471,  26, { 11119,   1409,  11203,  11200}},
 {   472,  26, { 11200,  11203,  11204,  11201}},
 {   473,  26, { 11201,  11204,  11205,  11202}},
 {   474,  26, {  1409,   1425,  11228,  11203}},
 {   475,  26, { 11203,  11228,  11229,  11204}},
 {   476,  26, { 11204,  11229,  11230,  11205}},
 {   477,  26, {  1425,   1481,  11243,  11228}},
 {   478,  26, { 11228,  11243,  11244,  11229}},
 {   479,  26, { 11229,  11244,  11245,  11230}},
 {   480,  26, {  1481,   1489,  11258,  11243}},
 {   481,  26, { 11243,  11258,  11259,  11244}},
 {   482,  26, { 11244,  11259,  11260,  11245}},
 {   483,  26, {  1489,   1545,  11273,  11258}},
 {   484,  26, { 11258,  11273,  11274,  11259}},
 {   485,  26, { 11259,  11274,  11275,  11260}},
 {   486,  26, {  1545,   1600,  11287,  11273}},
 {   487,  26, { 11273,  11287,  11288,  11274}},
 {   488,  26, { 11274,  11288,  11289,  11275}},
 {   489,  26, {  1600,  11159,  11302,  11287}},
 {   490,  26, { 11287,  11302,  11303,  11288}},
 {   491,  26, { 11288,  11303,  11304,  11289}},
 {   492,  26, { 11159,  11163,  11305,  11302}},
 {   493,  26, { 11302,  11305,  11306,  11303}},
 {   494,  26, { 11303,  11306,  11307,  11304}},
 {   495,  27, { 16366,  16375,  16376,  16367}},
 {   496,  27, { 16367,  16376,  16377,  16368}},
 {   497,  27, { 16368,  16377,  16378,  16369}},
 {   498,  27, { 16369,  16378,  16379,  16370}},
 {   499,  27, { 16370,  16379,  16380,  16371}},
 {   500,  27, { 16371,  16380,  16381,  16372}},
 {   501,  27, { 16372,  16381,  16382,  16373}},
 {   502,  27, { 16373,  16382,  16383,  16374}},
 {   503,  27, { 16375,  16384,  16385,  16376}},
 {   504,  27, { 16376,  16385,  16386,  16377}},
 {   505,  27, { 16377,  16386,  16387,  16378}},
 {   506,  27, { 16378,  16387,  16388,  16379}},
 {   507,  27, { 16379,  16388,  16389,  16380}},
 {   508,  27, { 16380,  16389,  16390,  16381}},
 {   509,  27, { 16381,  16390,  16391,  16382}},
 {   510,  27, { 16382,  16391,  16392,  16383}},
 {   511,  27, { 16384,  16393,  16394,  16385}},
 {   512,  27, { 16385,  16394,  16395,  16386}},
 {   513,  27, { 16386,  16395,  16396,  16387}},
 {   514,  27, { 16387,  16396,  16397,  16388}},
 {   515,  27, { 16388,  16397,  16398,  16389}},
 {   516,  27, { 16389,  16398,  16399,  16390}},
 {   517,  27, { 16390,  16399,  16400,  16391}},
 {   518,  27, { 16391,  16400,  16401,  16392}},
 {   519,  27, { 16393,  16402,  16403,  16394}},
 {   520,  27, { 16394,  16403,  16404,  16395}},
 {   521,  27, { 16395,  16404,  16405,  16396}},
 {   522,  27, { 16396,  16405,  16406,  16397}},
 {   523,  27, { 16397,  16406,  16407,  16398}},
 {   524,  27, { 16398,  16407,  16408,  16399}},
 {   525,  27, { 16399,  16408,  16409,  16400}},
 {   526,  27, { 16400,  16409,  16410,  16401}},
 {   527,  27, { 16402,  16411,  16412,  16403}},
 {   528,  27, { 16403,  16412,  16413,  16404}},
 {   529,  27, { 16404,  16413,  16414,  16405}},
 {   530,  27, { 16405,  16414,  16415,  16406}},
 {   531,  27, { 16406,  16415,  16416,  16407}},
 {   532,  27, { 16407,  16416,  16417,  16408}},
 {   533,  27, { 16408,  16417,  16418,  16409}},
 {   534,  27, { 16409,  16418,  16419,  16410}},
 {   535,  27, { 16411,  16420,  16421,  16412}},
 {   536,  27, { 16412,  16421,  16422,  16413}},
 {   537,  27, { 16413,  16422,  16423,  16414}},
 {   538,  27, { 16414,  16423,  16424,  16415}},
 {   539,  27, { 16415,  16424,  16425,  16416}},
 {   540,  27, { 16416,  16425,  16426,  16417}},
 {   541,  27, { 16417,  16426,  16427,  16418}},
 {   542,  27, { 16418,  16427,  16428,  16419}},
 {   543,  27, { 16420,  16429,  16430,  16421}},
 {   544,  27, { 16421,  16430,  16431,  16422}},
 {   545,  27, { 16422,  16431,  16432,  16423}},
 {   546,  27, { 16423,  16432,  16433,  16424}},
 {   547,  27, { 16424,  16433,  16434,  16425}},
 {   548,  27, { 16425,  16434,  16435,  16426}},
 {   549,  27, { 16426,  16435,  16436,  16427}},
 {   550,  27, { 16427,  16436,  16437,  16428}},
 {   551,  27, { 16429,  16438,  16439,  16430}},
 {   552,  27, { 16430,  16439,  16440,  16431}},
 {   553,  27, { 16431,  16440,  16441,  16432}},
 {   554,  27, { 16432,  16441,  16442,  16433}},
 {   555,  27, { 16433,  16442,  16443,  16434}},
 {   556,  27, { 16434,  16443,  16444,  16435}},
 {   557,  27, { 16435,  16444,  16445,  16436}},
 {   558,  27, { 16436,  16445,  16446,  16437}},
 {   559,  27, { 16438,  16447,  16448,  16439}},
 {   560,  27, { 16439,  16448,  16449,  16440}},
 {   561,  27, { 16440,  16449,  16450,  16441}},
 {   562,  27, { 16441,  16450,  16451,  16442}},
 {   563,  27, { 16442,  16451,  16452,  16443}},
 {   564,  27, { 16443,  16452,  16453,  16444}},
 {   565,  27, { 16444,  16453,  16454,  16445}},
 {   566,  27, { 16445,  16454,  16455,  16446}},
 {   567,  27, { 16447,  16456,  16457,  16448}},
 {   568,  27, { 16448,  16457,  16458,  16449}},
 {   569,  27, { 16449,  16458,  16459,  16450}},
 {   570,  27, { 16450,  16459,  16460,  16451}},
 {   571,  27, { 16451,  16460,  16461,  16452}},
 {   572,  27, { 16452,  16461,  16462,  16453}},
 {   573,  27, { 16453,  16462,  16463,  16454}},
 {   574,  27, { 16454,  16463,  16464,  16455}},
 {   575,  27, { 16456,  16465,  16466,  16457}},
 {   576,  27, { 16457,  16466,  16467,  16458}},
 {   577,  27, { 16458,  16467,  16468,  16459}},
 {   578,  27, { 16459,  16468,  16469,  16460}},
 {   579,  27, { 16460,  16469,  16470,  16461}},
 {   580,  27, { 16461,  16470,  16471,  16462}},
 {   581,  27, { 16462,  16471,  16472,  16463}},
 {   582,  27, { 16463,  16472,  16473,  16464}},
 {   583,  27, { 16465,  16474,  16475,  16466}},
 {   584,  27, { 16466,  16475,  16476,  16467}},
 {   585,  27, { 16467,  16476,  16477,  16468}},
 {   586,  27, { 16468,  16477,  16478,  16469}},
 {   587,  27, { 16469,  16478,  16479,  16470}},
 {   588,  27, { 16470,  16479,  16480,  16471}},
 {   589,  27, { 16471,  16480,  16481,  16472}},
 {   590,  27, { 16472,  16481,  16482,  16473}},
 {   591,  27, { 16474,  16483,  16484,  16475}},
 {   592,  27, { 16475,  16484,  16485,  16476}},
 {   593,  27, { 16476,  16485,  16486,  16477}},
 {   594,  27, { 16477,  16486,  16487,  16478}},
 {   595,  27, { 16478,  16487,  16488,  16479}},
 {   596,  27, { 16479,  16488,  16489,  16480}},
 {   597,  27, { 16480,  16489,  16490,  16481}},
 {   598,  27, { 16481,  16490,  16491,  16482}},
 {   599,  27, { 16483,  16492,  16493,  16484}},
 {   600,  27, { 16484,  16493,  16494,  16485}},
 {   601,  27, { 16485,  16494,  16495,  16486}},
 {   602,  27, { 16486,  16495,  16496,  16487}},
 {   603,  27, { 16487,  16496,  16497,  16488}},
 {   604,  27, { 16488,  16497,  16498,  16489}},
 {   605,  27, { 16489,  16498,  16499,  16490}},
 {   606,  27, { 16490,  16499,  16500,  16491}},
 {   607,  27, { 16492,  16501,  16502,  16493}},
 {   608,  27, { 16493,  16502,  16503,  16494}},
 {   609,  27, { 16494,  16503,  16504,  16495}},
 {   610,  27, { 16495,  16504,  16505,  16496}},
 {   611,  27, { 16496,  16505,  16506,  16497}},
 {   612,  27, { 16497,  16506,  16507,  16498}},
 {   613,  27, { 16498,  16507,  16508,  16499}},
 {   614,  27, { 16499,  16508,  16509,  16500}},
 {   615,  27, { 16501,  16510,  16511,  16502}},
 {   616,  27, { 16502,  16511,  16512,  16503}},
 {   617,  27, { 16503,  16512,  16513,  16504}},
 {   618,  27, { 16504,  16513,  16514,  16505}},
 {   619,  27, { 16505,  16514,  16515,  16506}},
 {   620,  27, { 16506,  16515,  16516,  16507}},
 {   621,  27, { 16507,  16516,  16517,  16508}},
 {   622,  27, { 16508,  16517,  16518,  16509}},
 {   623,  27, { 16510,  16519,  16520,  16511}},
 {   624,  27, { 16511,  16520,  16521,  16512}},
 {   625,  27, { 16512,  16521,  16522,  16513}},
 {   626,  27, { 16513,  16522,  16523,  16514}},
 {   627,  27, { 16514,  16523,  16524,  16515}},
 {   628,  27, { 16515,  16524,  16525,  16516}},
 {   629,  27, { 16516,  16525,  16526,  16517}},
 {   630,  27, { 16517,  16526,  16527,  16518}},
 {   631,  27, { 16519,  16528,  16529,  16520}},
 {   632,  27, { 16520,  16529,  16530,  16521}},
 {   633,  27, { 16521,  16530,  16531,  16522}},
 {   634,  27, { 16522,  16531,  16532,  16523}},
 {   635,  27, { 16523,  16532,  16533,  16524}},
 {   636,  27, { 16524,  16533,  16534,  16525}},
 {   637,  27, { 16525,  16534,  16535,  16526}},
 {   638,  27, { 16526,  16535,  16536,  16527}},
 {   639,  27, { 16528,  16537,  16538,  16529}},
 {   640,  27, { 16529,  16538,  16539,  16530}},
 {   641,  27, { 16530,  16539,  16540,  16531}},
 {   642,  27, { 16531,  16540,  16541,  16532}},
 {   643,  27, { 16532,  16541,  16542,  16533}},
 {   644,  27, { 16533,  16542,  16543,  16534}},
 {   645,  27, { 16534,  16543,  16544,  16535}},
 {   646,  27, { 16535,  16544,  16545,  16536}},
 {   647,  27, { 16537,  16546,  16547,  16538}},
 {   648,  27, { 16538,  16547,  16548,  16539}},
 {   649,  27, { 16539,  16548,  16549,  16540}},
 {   650,  27, { 16540,  16549,  16550,  16541}},
 {   651,  27, { 16541,  16550,  16551,  16542}},
 {   652,  27, { 16542,  16551,  16552,  16543}},
 {   653,  27, { 16543,  16552,  16553,  16544}},
 {   654,  27, { 16544,  16553,  16554,  16545}},
 {   655,  27, { 16546,  16555,  16556,  16547}},
 {   656,  27, { 16547,  16556,  16557,  16548}},
 {   657,  27, { 16548,  16557,  16558,  16549}},
 {   658,  27, { 16549,  16558,  16559,  16550}},
 {   659,  27, { 16550,  16559,  16560,  16551}},
 {   660,  27, { 16551,  16560,  16561,  16552}},
 {   661,  27, { 16552,  16561,  16562,  16553}},
 {   662,  27, { 16553,  16562,  16563,  16554}},
 {   663,  27, { 16374,  16383,  16580,  16564}},
 {   664,  27, { 16564,  16580,  16581,  16565}},
 {   665,  27, { 16565,  16581,  16582,  16566}},
 {   666,  27, { 16566,  16582,  16583,  16567}},
 {   667,  27, { 16567,  16583,  16584,  16568}},
 {   668,  27, { 16568,  16584,  16585,  16569}},
 {   669,  27, { 16569,  16585,  16586,  16570}},
 {   670,  27, { 16570,  16586,  16587,  16571}},
 {   671,  27, { 16571,  16587,  16588,  16572}},
 {   672,  27, { 16572,  16588,  16589,  16573}},
 {   673,  27, { 16573,  16589,  16590,  16574}},
 {   674,  27, { 16574,  16590,  16591,  16575}},
 {   675,  27, { 16575,  16591,  16592,  16576}},
 {   676,  27, { 16576,  16592,  16593,  16577}},
 {   677,  27, { 16577,  16593,  16594,  16578}},
 {   678,  27, { 16578,  16594,  16595,  16579}},
 {   679,  27, { 16383,  16392,  16596,  16580}},
 {   680,  27, { 16580,  16596,  16597,  16581}},
 {   681,  27, { 16581,  16597,  16598,  16582}},
 {   682,  27, { 16582,  16598,  16599,  16583}},
 {   683,  27, { 16583,  16599,  16600,  16584}},
 {   684,  27, { 16584,  16600,  16601,  16585}},
 {   685,  27, { 16585,  16601,  16602,  16586}},
 {   686,  27, { 16586,  16602,  16603,  16587}},
 {   687,  27, { 16587,  16603,  16604,  16588}},
 {   688,  27, { 16588,  16604,  16605,  16589}},
 {   689,  27, { 16589,  16605,  16606,  16590}},
 {   690,  27, { 16590,  16606,  16607,  16591}},
 {   691,  27, { 16591,  16607,  16608,  16592}},
 {   692,  27, { 16592,  16608,  16609,  16593}},
 {   693,  27, { 16593,  16609,  16610,  16594}},
 {   694,  27, { 16594,  16610,  16611,  16595}},
 {   695,  27, { 16392,  16401,  16612,  16596}},
 {   696,  27, { 16596,  16612,  16613,  16597}},
 {   697,  27, { 16597,  16613,  16614,  16598}},
 {   698,  27, { 16598,  16614,  16615,  16599}},
 {   699,  27, { 16599,  16615,  16616,  16600}},
 {   700,  27, { 16600,  16616,  16617,  16601}},
 {   701,  27, { 16601,  16617,  16618,  16602}},
 {   702,  27, { 16602,  16618,  16619,  16603}},
 {   703,  27, { 16603,  16619,  16620,  16604}},
 {   704,  27, { 16604,  16620,  16621,  16605}},
 {   705,  27, { 16605,  16621,  16622,  16606}},
 {   706,  27, { 16606,  16622,  16623,  16607}},
 {   707,  27, { 16607,  16623,  16624,  16608}},
 {   708,  27, { 16608,  16624,  16625,  16609}},
 {   709,  27, { 16609,  16625,  16626,  16610}},
 {   710,  27, { 16610,  16626,  16627,  16611}},
 {   711,  27, { 16401,  16410,  16628,  16612}},
 {   712,  27, { 16612,  16628,  16629,  16613}},
 {   713,  27, { 16613,  16629,  16630,  16614}},
 {   714,  27, { 16614,  16630,  16631,  16615}},
 {   715,  27, { 16615,  16631,  16632,  16616}},
 {   716,  27, { 16616,  16632,  16633,  16617}},
 {   717,  27, { 16617,  16633,  16634,  16618}},
 {   718,  27, { 16618,  16634,  16635,  16619}},
 {   719,  27, { 16619,  16635,  16636,  16620}},
 {   720,  27, { 16620,  16636,  16637,  16621}},
 {   721,  27, { 16621,  16637,  16638,  16622}},
 {   722,  27, { 16622,  16638,  16639,  16623}},
 {   723,  27, { 16623,  16639,  16640,  16624}},
 {   724,  27, { 16624,  16640,  16641,  16625}},
 {   725,  27, { 16625,  16641,  16642,  16626}},
 {   726,  27, { 16626,  16642,  16643,  16627}},
 {   727,  27, { 16410,  16419,  16644,  16628}},
 {   728,  27, { 16628,  16644,  16645,  16629}},
 {   729,  27, { 16629,  16645,  16646,  16630}},
 {   730,  27, { 16630,  16646,  16647,  16631}},
 {   731,  27, { 16631,  16647,  16648,  16632}},
 {   732,  27, { 16632,  16648,  16649,  16633}},
 {   733,  27, { 16633,  16649,  16650,  16634}},
 {   734,  27, { 16634,  16650,  16651,  16635}},
 {   735,  27, { 16635,  16651,  16652,  16636}},
 {   736,  27, { 16636,  16652,  16653,  16637}},
 {   737,  27, { 16637,  16653,  16654,  16638}},
 {   738,  27, { 16638,  16654,  16655,  16639}},
 {   739,  27, { 16639,  16655,  16656,  16640}},
 {   740,  27, { 16640,  16656,  16657,  16641}},
 {   741,  27, { 16641,  16657,  16658,  16642}},
 {   742,  27, { 16642,  16658,  16659,  16643}},
 {   743,  27, { 16419,  16428,  16660,  16644}},
 {   744,  27, { 16644,  16660,  16661,  16645}},
 {   745,  27, { 16645,  16661,  16662,  16646}},
 {   746,  27, { 16646,  16662,  16663,  16647}},
 {   747,  27, { 16647,  16663,  16664,  16648}},
 {   748,  27, { 16648,  16664,  16665,  16649}},
 {   749,  27, { 16649,  16665,  16666,  16650}},
 {   750,  27, { 16650,  16666,  16667,  16651}},
 {   751,  27, { 16651,  16667,  16668,  16652}},
 {   752,  27, { 16652,  16668,  16669,  16653}},
 {   753,  27, { 16653,  16669,  16670,  16654}},
 {   754,  27, { 16654,  16670,  16671,  16655}},
 {   755,  27, { 16655,  16671,  16672,  16656}},
 {   756,  27, { 16656,  16672,  16673,  16657}},
 {   757,  27, { 16657,  16673,  16674,  16658}},
 {   758,  27, { 16658,  16674,  16675,  16659}},
 {   759,  27, { 16428,  16437,  16676,  16660}},
 {   760,  27, { 16660,  16676,  16677,  16661}},
 {   761,  27, { 16661,  16677,  16678,  16662}},
 {   762,  27, { 16662,  16678,  16679,  16663}},
 {   763,  27, { 16663,  16679,  16680,  16664}},
 {   764,  27, { 16664,  16680,  16681,  16665}},
 {   765,  27, { 16665,  16681,  16682,  16666}},
 {   766,  27, { 16666,  16682,  16683,  16667}},
 {   767,  27, { 16667,  16683,  16684,  16668}},
 {   768,  27, { 16668,  16684,  16685,  16669}},
 {   769,  27, { 16669,  16685,  16686,  16670}},
 {   770,  27, { 16670,  16686,  16687,  16671}},
 {   771,  27, { 16671,  16687,  16688,  16672}},
 {   772,  27, { 16672,  16688,  16689,  16673}},
 {   773,  27, { 16673,  16689,  16690,  16674}},
 {   774,  27, { 16674,  16690,  16691,  16675}},
 {   775,  27, { 16437,  16446,  16692,  16676}},
 {   776,  27, { 16676,  16692,  16693,  16677}},
 {   777,  27, { 16677,  16693,  16694,  16678}},
 {   778,  27, { 16678,  16694,  16695,  16679}},
 {   779,  27, { 16679,  16695,  16696,  16680}},
 {   780,  27, { 16680,  16696,  16697,  16681}},
 {   781,  27, { 16681,  16697,  16698,  16682}},
 {   782,  27, { 16682,  16698,  16699,  16683}},
 {   783,  27, { 16683,  16699,  16700,  16684}},
 {   784,  27, { 16684,  16700,  16701,  16685}},
 {   785,  27, { 16685,  16701,  16702,  16686}},
 {   786,  27, { 16686,  16702,  16703,  16687}},
 {   787,  27, { 16687,  16703,  16704,  16688}},
 {   788,  27, { 16688,  16704,  16705,  16689}},
 {   789,  27, { 16689,  16705,  16706,  16690}},
 {   790,  27, { 16690,  16706,  16707,  16691}},
 {   791,  27, { 16446,  16455,  16708,  16692}},
 {   792,  27, { 16692,  16708,  16709,  16693}},
 {   793,  27, { 16693,  16709,  16710,  16694}},
 {   794,  27, { 16694,  16710,  16711,  16695}},
 {   795,  27, { 16695,  16711,  16712,  16696}},
 {   796,  27, { 16696,  16712,  16713,  16697}},
 {   797,  27, { 16697,  16713,  16714,  16698}},
 {   798,  27, { 16698,  16714,  16715,  16699}},
 {   799,  27, { 16699,  16715,  16716,  16700}},
 {   800,  27, { 16700,  16716,  16717,  16701}},
 {   801,  27, { 16701,  16717,  16718,  16702}},
 {   802,  27, { 16702,  16718,  16719,  16703}},
 {   803,  27, { 16703,  16719,  16720,  16704}},
 {   804,  27, { 16704,  16720,  16721,  16705}},
 {   805,  27, { 16705,  16721,  16722,  16706}},
 {   806,  27, { 16706,  16722,  16723,  16707}},
 {   807,  27, { 16455,  16464,  16724,  16708}},
 {   808,  27, { 16708,  16724,  16725,  16709}},
 {   809,  27, { 16709,  16725,  16726,  16710}},
 {   810,  27, { 16710,  16726,  16727,  16711}},
 {   811,  27, { 16711,  16727,  16728,  16712}},
 {   812,  27, { 16712,  16728,  16729,  16713}},
 {   813,  27, { 16713,  16729,  16730,  16714}},
 {   814,  27, { 16714,  16730,  16731,  16715}},
 {   815,  27, { 16715,  16731,  16732,  16716}},
 {   816,  27, { 16716,  16732,  16733,  16717}},
 {   817,  27, { 16717,  16733,  16734,  16718}},
 {   818,  27, { 16718,  16734,  16735,  16719}},
 {   819,  27, { 16719,  16735,  16736,  16720}},
 {   820,  27, { 16720,  16736,  16737,  16721}},
 {   821,  27, { 16721,  16737,  16738,  16722}},
 {   822,  27, { 16722,  16738,  16739,  16723}},
 {   823,  27, { 16464,  16473,  16740,  16724}},
 {   824,  27, { 16724,  16740,  16741,  16725}},
 {   825,  27, { 16725,  16741,  16742,  16726}},
 {   826,  27, { 16726,  16742,  16743,  16727}},
 {   827,  27, { 16727,  16743,  16744,  16728}},
 {   828,  27, { 16728,  16744,  16745,  16729}},
 {   829,  27, { 16729,  16745,  16746,  16730}},
 {   830,  27, { 16730,  16746,  16747,  16731}},
 {   831,  27, { 16731,  16747,  16748,  16732}},
 {   832,  27, { 16732,  16748,  16749,  16733}},
 {   833,  27, { 16733,  16749,  16750,  16734}},
 {   834,  27, { 16734,  16750,  16751,  16735}},
 {   835,  27, { 16735,  16751,  16752,  16736}},
 {   836,  27, { 16736,  16752,  16753,  16737}},
 {   837,  27, { 16737,  16753,  16754,  16738}},
 {   838,  27, { 16738,  16754,  16755,  16739}},
 {   839,  27, { 16473,  16482,  16756,  16740}},
 {   840,  27, { 16740,  16756,  16757,  16741}},
 {   841,  27, { 16741,  16757,  16758,  16742}},
 {   842,  27, { 16742,  16758,  16759,  16743}},
 {   843,  27, { 16743,  16759,  16760,  16744}},
 {   844,  27, { 16744,  16760,  16761,  16745}},
 {   845,  27, { 16745,  16761,  16762,  16746}},
 {   846,  27, { 16746,  16762,  16763,  16747}},
 {   847,  27, { 16747,  16763,  16764,  16748}},
 {   848,  27, { 16748,  16764,  16765,  16749}},
 {   849,  27, { 16749,  16765,  16766,  16750}},
 {   850,  27, { 16750,  16766,  16767,  16751}},
 {   851,  27, { 16751,  16767,  16768,  16752}},
 {   852,  27, { 16752,  16768,  16769,  16753}},
 {   853,  27, { 16753,  16769,  16770,  16754}},
 {   854,  27, { 16754,  16770,  16771,  16755}},
 {   855,  27, { 16482,  16491,  16772,  16756}},
 {   856,  27, { 16756,  16772,  16773,  16757}},
 {   857,  27, { 16757,  16773,  16774,  16758}},
 {   858,  27, { 16758,  16774,  16775,  16759}},
 {   859,  27, { 16759,  16775,  16776,  16760}},
 {   860,  27, { 16760,  16776,  16777,  16761}},
 {   861,  27, { 16761,  16777,  16778,  16762}},
 {   862,  27, { 16762,  16778,  16779,  16763}},
 {   863,  27, { 16763,  16779,  16780,  16764}},
 {   864,  27, { 16764,  16780,  16781,  16765}},
 {   865,  27, { 16765,  16781,  16782,  16766}},
 {   866,  27, { 16766,  16782,  16783,  16767}},
 {   867,  27, { 16767,  16783,  16784,  16768}},
 {   868,  27, { 16768,  16784,  16785,  16769}},
 {   869,  27, { 16769,  16785,  16786,  16770}},
 {   870,  27, { 16770,  16786,  16787,  16771}},
 {   871,  27, { 16491,  16500,  16788,  16772}},
 {   872,  27, { 16772,  16788,  16789,  16773}},
 {   873,  27, { 16773,  16789,  16790,  16774}},
 {   874,  27, { 16774,  16790,  16791,  16775}},
 {   875,  27, { 16775,  16791,  16792,  16776}},
 {   876,  27, { 16776,  16792,  16793,  16777}},
 {   877,  27, { 16777,  16793,  16794,  16778}},
 {   878,  27, { 16778,  16794,  16795,  16779}},
 {   879,  27, { 16779,  16795,  16796,  16780}},
 {   880,  27, { 16780,  16796,  16797,  16781}},
 {   881,  27, { 16781,  16797,  16798,  16782}},
 {   882,  27, { 16782,  16798,  16799,  16783}},
 {   883,  27, { 16783,  16799,  16800,  16784}},
 {   884,  27, { 16784,  16800,  16801,  16785}},
 {   885,  27, { 16785,  16801,  16802,  16786}},
 {   886,  27, { 16786,  16802,  16803,  16787}},
 {   887,  27, { 16500,  16509,  16804,  16788}},
 {   888,  27, { 16788,  16804,  16805,  16789}},
 {   889,  27, { 16789,  16805,  16806,  16790}},
 {   890,  27, { 16790,  16806,  16807,  16791}},
 {   891,  27, { 16791,  16807,  16808,  16792}},
 {   892,  27, { 16792,  16808,  16809,  16793}},
 {   893,  27, { 16793,  16809,  16810,  16794}},
 {   894,  27, { 16794,  16810,  16811,  16795}},
 {   895,  27, { 16795,  16811,  16812,  16796}},
 {   896,  27, { 16796,  16812,  16813,  16797}},
 {   897,  27, { 16797,  16813,  16814,  16798}},
 {   898,  27, { 16798,  16814,  16815,  16799}},
 {   899,  27, { 16799,  16815,  16816,  16800}},
 {   900,  27, { 16800,  16816,  16817,  16801}},
 {   901,  27, { 16801,  16817,  16818,  16802}},
 {   902,  27, { 16802,  16818,  16819,  16803}},
 {   903,  27, { 16509,  16518,  16820,  16804}},
 {   904,  27, { 16804,  16820,  16821,  16805}},
 {   905,  27, { 16805,  16821,  16822,  16806}},
 {   906,  27, { 16806,  16822,  16823,  16807}},
 {   907,  27, { 16807,  16823,  16824,  16808}},
 {   908,  27, { 16808,  16824,  16825,  16809}},
 {   909,  27, { 16809,  16825,  16826,  16810}},
 {   910,  27, { 16810,  16826,  16827,  16811}},
 {   911,  27, { 16811,  16827,  16828,  16812}},
 {   912,  27, { 16812,  16828,  16829,  16813}},
 {   913,  27, { 16813,  16829,  16830,  16814}},
 {   914,  27, { 16814,  16830,  16831,  16815}},
 {   915,  27, { 16815,  16831,  16832,  16816}},
 {   916,  27, { 16816,  16832,  16833,  16817}},
 {   917,  27, { 16817,  16833,  16834,  16818}},
 {   918,  27, { 16818,  16834,  16835,  16819}},
 {   919,  27, { 16518,  16527,  16836,  16820}},
 {   920,  27, { 16820,  16836,  16837,  16821}},
 {   921,  27, { 16821,  16837,  16838,  16822}},
 {   922,  27, { 16822,  16838,  16839,  16823}},
 {   923,  27, { 16823,  16839,  16840,  16824}},
 {   924,  27, { 16824,  16840,  16841,  16825}},
 {   925,  27, { 16825,  16841,  16842,  16826}},
 {   926,  27, { 16826,  16842,  16843,  16827}},
 {   927,  27, { 16827,  16843,  16844,  16828}},
 {   928,  27, { 16828,  16844,  16845,  16829}},
 {   929,  27, { 16829,  16845,  16846,  16830}},
 {   930,  27, { 16830,  16846,  16847,  16831}},
 {   931,  27, { 16831,  16847,  16848,  16832}},
 {   932,  27, { 16832,  16848,  16849,  16833}},
 {   933,  27, { 16833,  16849,  16850,  16834}},
 {   934,  27, { 16834,  16850,  16851,  16835}},
 {   935,  27, { 16527,  16536,  16852,  16836}},
 {   936,  27, { 16836,  16852,  16853,  16837}},
 {   937,  27, { 16837,  16853,  16854,  16838}},
 {   938,  27, { 16838,  16854,  16855,  16839}},
 {   939,  27, { 16839,  16855,  16856,  16840}},
 {   940,  27, { 16840,  16856,  16857,  16841}},
 {   941,  27, { 16841,  16857,  16858,  16842}},
 {   942,  27, { 16842,  16858,  16859,  16843}},
 {   943,  27, { 16843,  16859,  16860,  16844}},
 {   944,  27, { 16844,  16860,  16861,  16845}},
 {   945,  27, { 16845,  16861,  16862,  16846}},
 {   946,  27, { 16846,  16862,  16863,  16847}},
 {   947,  27, { 16847,  16863,  16864,  16848}},
 {   948,  27, { 16848,  16864,  16865,  16849}},
 {   949,  27, { 16849,  16865,  16866,  16850}},
 {   950,  27, { 16850,  16866,  16867,  16851}},
 {   951,  27, { 16536,  16545,  16868,  16852}},
 {   952,  27, { 16852,  16868,  16869,  16853}},
 {   953,  27, { 16853,  16869,  16870,  16854}},
 {   954,  27, { 16854,  16870,  16871,  16855}},
 {   955,  27, { 16855,  16871,  16872,  16856}},
 {   956,  27, { 16856,  16872,  16873,  16857}},
 {   957,  27, { 16857,  16873,  16874,  16858}},
 {   958,  27, { 16858,  16874,  16875,  16859}},
 {   959,  27, { 16859,  16875,  16876,  16860}},
 {   960,  27, { 16860,  16876,  16877,  16861}},
 {   961,  27, { 16861,  16877,  16878,  16862}},
 {   962,  27, { 16862,  16878,  16879,  16863}},
 {   963,  27, { 16863,  16879,  16880,  16864}},
 {   964,  27, { 16864,  16880,  16881,  16865}},
 {   965,  27, { 16865,  16881,  16882,  16866}},
 {   966,  27, { 16866,  16882,  16883,  16867}},
 {   967,  27, { 16545,  16554,  16884,  16868}},
 {   968,  27, { 16868,  16884,  16885,  16869}},
 {   969,  27, { 16869,  16885,  16886,  16870}},
 {   970,  27, { 16870,  16886,  16887,  16871}},
 {   971,  27, { 16871,  16887,  16888,  16872}},
 {   972,  27, { 16872,  16888,  16889,  16873}},
 {   973,  27, { 16873,  16889,  16890,  16874}},
 {   974,  27, { 16874,  16890,  16891,  16875}},
 {   975,  27, { 16875,  16891,  16892,  16876}},
 {   976,  27, { 16876,  16892,  16893,  16877}},
 {   977,  27, { 16877,  16893,  16894,  16878}},
 {   978,  27, { 16878,  16894,  16895,  16879}},
 {   979,  27, { 16879,  16895,  16896,  16880}},
 {   980,  27, { 16880,  16896,  16897,  16881}},
 {   981,  27, { 16881,  16897,  16898,  16882}},
 {   982,  27, { 16882,  16898,  16899,  16883}},
 {   983,  27, { 16554,  16563,  16900,  16884}},
 {   984,  27, { 16884,  16900,  16901,  16885}},
 {   985,  27, { 16885,  16901,  16902,  16886}},
 {   986,  27, { 16886,  16902,  16903,  16887}},
 {   987,  27, { 16887,  16903,  16904,  16888}},
 {   988,  27, { 16888,  16904,  16905,  16889}},
 {   989,  27, { 16889,  16905,  16906,  16890}},
 {   990,  27, { 16890,  16906,  16907,  16891}},
 {   991,  27, { 16891,  16907,  16908,  16892}},
 {   992,  27, { 16892,  16908,  16909,  16893}},
 {   993,  27, { 16893,  16909,  16910,  16894}},
 {   994,  27, { 16894,  16910,  16911,  16895}},
 {   995,  27, { 16895,  16911,  16912,  16896}},
 {   996,  27, { 16896,  16912,  16913,  16897}},
 {   997,  27, { 16897,  16913,  16914,  16898}},
 {   998,  27, { 16898,  16914,  16915,  16899}},
 {   999,  27, { 16579,  16595,  16920,  16916}},
 {  1000,  27, { 16916,  16920,  16921,  16917}},
 {  1001,  27, { 16917,  16921,  16922,  16918}},
 {  1002,  27, { 16918,  16922,  16923,  16919}},
 {  1003,  27, { 16595,  16611,  16924,  16920}},
 {  1004,  27, { 16920,  16924,  16925,  16921}},
 {  1005,  27, { 16921,  16925,  16926,  16922}},
 {  1006,  27, { 16922,  16926,  16927,  16923}},
 {  1007,  27, { 16611,  16627,  16928,  16924}},
 {  1008,  27, { 16924,  16928,  16929,  16925}},
 {  1009,  27, { 16925,  16929,  16930,  16926}},
 {  1010,  27, { 16926,  16930,  16931,  16927}},
 {  1011,  27, { 16627,  16643,  16932,  16928}},
 {  1012,  27, { 16928,  16932,  16933,  16929}},
 {  1013,  27, { 16929,  16933,  16934,  16930}},
 {  1014,  27, { 16930,  16934,  16935,  16931}},
 {  1015,  27, { 16643,  16659,  16936,  16932}},
 {  1016,  27, { 16932,  16936,  16937,  16933}},
 {  1017,  27, { 16933,  16937,  16938,  16934}},
 {  1018,  27, { 16934,  16938,  16939,  16935}},
 {  1019,  27, { 16659,  16675,  16940,  16936}},
 {  1020,  27, { 16936,  16940,  16941,  16937}},
 {  1021,  27, { 16937,  16941,  16942,  16938}},
 {  1022,  27, { 16938,  16942,  16943,  16939}},
 {  1023,  27, { 16675,  16691,  16944,  16940}},
 {  1024,  27, { 16940,  16944,  16945,  16941}},
 {  1025,  27, { 16941,  16945,  16946,  16942}},
 {  1026,  27, { 16942,  16946,  16947,  16943}},
 {  1027,  27, { 16691,  16707,  16948,  16944}},
 {  1028,  27, { 16944,  16948,  16949,  16945}},
 {  1029,  27, { 16945,  16949,  16950,  16946}},
 {  1030,  27, { 16946,  16950,  16951,  16947}},
 {  1031,  27, { 16707,  16723,  16952,  16948}},
 {  1032,  27, { 16948,  16952,  16953,  16949}},
 {  1033,  27, { 16949,  16953,  16954,  16950}},
 {  1034,  27, { 16950,  16954,  16955,  16951}},
 {  1035,  27, { 16723,  16739,  16956,  16952}},
 {  1036,  27, { 16952,  16956,  16957,  16953}},
 {  1037,  27, { 16953,  16957,  16958,  16954}},
 {  1038,  27, { 16954,  16958,  16959,  16955}},
 {  1039,  27, { 16739,  16755,  16960,  16956}},
 {  1040,  27, { 16956,  16960,  16961,  16957}},
 {  1041,  27, { 16957,  16961,  16962,  16958}},
 {  1042,  27, { 16958,  16962,  16963,  16959}},
 {  1043,  27, { 16755,  16771,  16964,  16960}},
 {  1044,  27, { 16960,  16964,  16965,  16961}},
 {  1045,  27, { 16961,  16965,  16966,  16962}},
 {  1046,  27, { 16962,  16966,  16967,  16963}},
 {  1047,  27, { 16771,  16787,  16968,  16964}},
 {  1048,  27, { 16964,  16968,  16969,  16965}},
 {  1049,  27, { 16965,  16969,  16970,  16966}},
 {  1050,  27, { 16966,  16970,  16971,  16967}},
 {  1051,  27, { 16787,  16803,  16972,  16968}},
 {  1052,  27, { 16968,  16972,  16973,  16969}},
 {  1053,  27, { 16969,  16973,  16974,  16970}},
 {  1054,  27, { 16970,  16974,  16975,  16971}},
 {  1055,  27, { 16803,  16819,  16976,  16972}},
 {  1056,  27, { 16972,  16976,  16977,  16973}},
 {  1057,  27, { 16973,  16977,  16978,  16974}},
 {  1058,  27, { 16974,  16978,  16979,  16975}},
 {  1059,  27, { 16819,  16835,  16980,  16976}},
 {  1060,  27, { 16976,  16980,  16981,  16977}},
 {  1061,  27, { 16977,  16981,  16982,  16978}},
 {  1062,  27, { 16978,  16982,  16983,  16979}},
 {  1063,  27, { 16835,  16851,  16984,  16980}},
 {  1064,  27, { 16980,  16984,  16985,  16981}},
 {  1065,  27, { 16981,  16985,  16986,  16982}},
 {  1066,  27, { 16982,  16986,  16987,  16983}},
 {  1067,  27, { 16851,  16867,  16988,  16984}},
 {  1068,  27, { 16984,  16988,  16989,  16985}},
 {  1069,  27, { 16985,  16989,  16990,  16986}},
 {  1070,  27, { 16986,  16990,  16991,  16987}},
 {  1071,  27, { 16867,  16883,  16992,  16988}},
 {  1072,  27, { 16988,  16992,  16993,  16989}},
 {  1073,  27, { 16989,  16993,  16994,  16990}},
 {  1074,  27, { 16990,  16994,  16995,  16991}},
 {  1075,  27, { 16883,  16899,  16996,  16992}},
 {  1076,  27, { 16992,  16996,  16997,  16993}},
 {  1077,  27, { 16993,  16997,  16998,  16994}},
 {  1078,  27, { 16994,  16998,  16999,  16995}},
 {  1079,  27, { 16899,  16915,  17000,  16996}},
 {  1080,  27, { 16996,  17000,  17001,  16997}},
 {  1081,  27, { 16997,  17001,  17002,  16998}},
 {  1082,  27, { 16998,  17002,  17003,  16999}},
 {  1083,  27, { 16919,  16923,  17008,  17004}},
 {  1084,  27, { 17004,  17008,  17009,  17005}},
 {  1085,  27, { 17005,  17009,  17010,  17006}},
 {  1086,  27, { 17006,  17010,  17011,  17007}},
 {  1087,  27, { 16923,  16927,  17012,  17008}},
 {  1088,  27, { 17008,  17012,  17013,  17009}},
 {  1089,  27, { 17009,  17013,  17014,  17010}},
 {  1090,  27, { 17010,  17014,  17015,  17011}},
 {  1091,  27, { 16927,  16931,  17016,  17012}},
 {  1092,  27, { 17012,  17016,  17017,  17013}},
 {  1093,  27, { 17013,  17017,  17018,  17014}},
 {  1094,  27, { 17014,  17018,  17019,  17015}},
 {  1095,  27, { 16931,  16935,  17020,  17016}},
 {  1096,  27, { 17016,  17020,  17021,  17017}},
 {  1097,  27, { 17017,  17021,  17022,  17018}},
 {  1098,  27, { 17018,  17022,  17023,  17019}},
 {  1099,  27, { 16935,  16939,  17024,  17020}},
 {  1100,  27, { 17020,  17024,  17025,  17021}},
 {  1101,  27, { 17021,  17025,  17026,  17022}},
 {  1102,  27, { 17022,  17026,  17027,  17023}},
 {  1103,  27, { 16939,  16943,  17028,  17024}},
 {  1104,  27, { 17024,  17028,  17029,  17025}},
 {  1105,  27, { 17025,  17029,  17030,  17026}},
 {  1106,  27, { 17026,  17030,  17031,  17027}},
 {  1107,  27, { 16943,  16947,  17032,  17028}},
 {  1108,  27, { 17028,  17032,  17033,  17029}},
 {  1109,  27, { 17029,  17033,  17034,  17030}},
 {  1110,  27, { 17030,  17034,  17035,  17031}},
 {  1111,  27, { 16947,  16951,  17036,  17032}},
 {  1112,  27, { 17032,  17036,  17037,  17033}},
 {  1113,  27, { 17033,  17037,  17038,  17034}},
 {  1114,  27, { 17034,  17038,  17039,  17035}},
 {  1115,  27, { 16951,  16955,  17040,  17036}},
 {  1116,  27, { 17036,  17040,  17041,  17037}},
 {  1117,  27, { 17037,  17041,  17042,  17038}},
 {  1118,  27, { 17038,  17042,  17043,  17039}},
 {  1119,  27, { 16955,  16959,  17044,  17040}},
 {  1120,  27, { 17040,  17044,  17045,  17041}},
 {  1121,  27, { 17041,  17045,  17046,  17042}},
 {  1122,  27, { 17042,  17046,  17047,  17043}},
 {  1123,  27, { 16959,  16963,  17048,  17044}},
 {  1124,  27, { 17044,  17048,  17049,  17045}},
 {  1125,  27, { 17045,  17049,  17050,  17046}},
 {  1126,  27, { 17046,  17050,  17051,  17047}},
 {  1127,  27, { 16963,  16967,  17052,  17048}},
 {  1128,  27, { 17048,  17052,  17053,  17049}},
 {  1129,  27, { 17049,  17053,  17054,  17050}},
 {  1130,  27, { 17050,  17054,  17055,  17051}},
 {  1131,  27, { 16967,  16971,  17056,  17052}},
 {  1132,  27, { 17052,  17056,  17057,  17053}},
 {  1133,  27, { 17053,  17057,  17058,  17054}},
 {  1134,  27, { 17054,  17058,  17059,  17055}},
 {  1135,  27, { 16971,  16975,  17060,  17056}},
 {  1136,  27, { 17056,  17060,  17061,  17057}},
 {  1137,  27, { 17057,  17061,  17062,  17058}},
 {  1138,  27, { 17058,  17062,  17063,  17059}},
 {  1139,  27, { 16975,  16979,  17064,  17060}},
 {  1140,  27, { 17060,  17064,  17065,  17061}},
 {  1141,  27, { 17061,  17065,  17066,  17062}},
 {  1142,  27, { 17062,  17066,  17067,  17063}},
 {  1143,  27, { 16979,  16983,  17068,  17064}},
 {  1144,  27, { 17064,  17068,  17069,  17065}},
 {  1145,  27, { 17065,  17069,  17070,  17066}},
 {  1146,  27, { 17066,  17070,  17071,  17067}},
 {  1147,  27, { 16983,  16987,  17072,  17068}},
 {  1148,  27, { 17068,  17072,  17073,  17069}},
 {  1149,  27, { 17069,  17073,  17074,  17070}},
 {  1150,  27, { 17070,  17074,  17075,  17071}},
 {  1151,  27, { 16987,  16991,  17076,  17072}},
 {  1152,  27, { 17072,  17076,  17077,  17073}},
 {  1153,  27, { 17073,  17077,  17078,  17074}},
 {  1154,  27, { 17074,  17078,  17079,  17075}},
 {  1155,  27, { 16991,  16995,  17080,  17076}},
 {  1156,  27, { 17076,  17080,  17081,  17077}},
 {  1157,  27, { 17077,  17081,  17082,  17078}},
 {  1158,  27, { 17078,  17082,  17083,  17079}},
 {  1159,  27, { 16995,  16999,  17084,  17080}},
 {  1160,  27, { 17080,  17084,  17085,  17081}},
 {  1161,  27, { 17081,  17085,  17086,  17082}},
 {  1162,  27, { 17082,  17086,  17087,  17083}},
 {  1163,  27, { 16999,  17003,  17088,  17084}},
 {  1164,  27, { 17084,  17088,  17089,  17085}},
 {  1165,  27, { 17085,  17089,  17090,  17086}},
 {  1166,  27, { 17086,  17090,  17091,  17087}},
 {  1167,  27, { 17007,  17011,  17108,  17092}},
 {  1168,  27, { 17092,  17108,  17109,  17093}},
 {  1169,  27, { 17093,  17109,  17110,  17094}},
 {  1170,  27, { 17094,  17110,  17111,  17095}},
 {  1171,  27, { 17095,  17111,  17112,  17096}},
 {  1172,  27, { 17096,  17112,  17113,  17097}},
 {  1173,  27, { 17097,  17113,  17114,  17098}},
 {  1174,  27, { 17098,  17114,  17115,  17099}},
 {  1175,  27, { 17099,  17115,  17116,  17100}},
 {  1176,  27, { 17100,  17116,  17117,  17101}},
 {  1177,  27, { 17101,  17117,  17118,  17102}},
 {  1178,  27, { 17102,  17118,  17119,  17103}},
 {  1179,  27, { 17103,  17119,  17120,  17104}},
 {  1180,  27, { 17104,  17120,  17121,  17105}},
 {  1181,  27, { 17105,  17121,  17122,  17106}},
 {  1182,  27, { 17106,  17122,  17123,  17107}},
 {  1183,  27, { 17011,  17015,  17124,  17108}},
 {  1184,  27, { 17108,  17124,  17125,  17109}},
 {  1185,  27, { 17109,  17125,  17126,  17110}},
 {  1186,  27, { 17110,  17126,  17127,  17111}},
 {  1187,  27, { 17111,  17127,  17128,  17112}},
 {  1188,  27, { 17112,  17128,  17129,  17113}},
 {  1189,  27, { 17113,  17129,  17130,  17114}},
 {  1190,  27, { 17114,  17130,  17131,  17115}},
 {  1191,  27, { 17115,  17131,  17132,  17116}},
 {  1192,  27, { 17116,  17132,  17133,  17117}},
 {  1193,  27, { 17117,  17133,  17134,  17118}},
 {  1194,  27, { 17118,  17134,  17135,  17119}},
 {  1195,  27, { 17119,  17135,  17136,  17120}},
 {  1196,  27, { 17120,  17136,  17137,  17121}},
 {  1197,  27, { 17121,  17137,  17138,  17122}},
 {  1198,  27, { 17122,  17138,  17139,  17123}},
 {  1199,  27, { 17015,  17019,  17140,  17124}},
 {  1200,  27, { 17124,  17140,  17141,  17125}},
 {  1201,  27, { 17125,  17141,  17142,  17126}},
 {  1202,  27, { 17126,  17142,  17143,  17127}},
 {  1203,  27, { 17127,  17143,  17144,  17128}},
 {  1204,  27, { 17128,  17144,  17145,  17129}},
 {  1205,  27, { 17129,  17145,  17146,  17130}},
 {  1206,  27, { 17130,  17146,  17147,  17131}},
 {  1207,  27, { 17131,  17147,  17148,  17132}},
 {  1208,  27, { 17132,  17148,  17149,  17133}},
 {  1209,  27, { 17133,  17149,  17150,  17134}},
 {  1210,  27, { 17134,  17150,  17151,  17135}},
 {  1211,  27, { 17135,  17151,  17152,  17136}},
 {  1212,  27, { 17136,  17152,  17153,  17137}},
 {  1213,  27, { 17137,  17153,  17154,  17138}},
 {  1214,  27, { 17138,  17154,  17155,  17139}},
 {  1215,  27, { 17019,  17023,  17156,  17140}},
 {  1216,  27, { 17140,  17156,  17157,  17141}},
 {  1217,  27, { 17141,  17157,  17158,  17142}},
 {  1218,  27, { 17142,  17158,  17159,  17143}},
 {  1219,  27, { 17143,  17159,  17160,  17144}},
 {  1220,  27, { 17144,  17160,  17161,  17145}},
 {  1221,  27, { 17145,  17161,  17162,  17146}},
 {  1222,  27, { 17146,  17162,  17163,  17147}},
 {  1223,  27, { 17147,  17163,  17164,  17148}},
 {  1224,  27, { 17148,  17164,  17165,  17149}},
 {  1225,  27, { 17149,  17165,  17166,  17150}},
 {  1226,  27, { 17150,  17166,  17167,  17151}},
 {  1227,  27, { 17151,  17167,  17168,  17152}},
 {  1228,  27, { 17152,  17168,  17169,  17153}},
 {  1229,  27, { 17153,  17169,  17170,  17154}},
 {  1230,  27, { 17154,  17170,  17171,  17155}},
 {  1231,  27, { 17023,  17027,  17172,  17156}},
 {  1232,  27, { 17156,  17172,  17173,  17157}},
 {  1233,  27, { 17157,  17173,  17174,  17158}},
 {  1234,  27, { 17158,  17174,  17175,  17159}},
 {  1235,  27, { 17159,  17175,  17176,  17160}},
 {  1236,  27, { 17160,  17176,  17177,  17161}},
 {  1237,  27, { 17161,  17177,  17178,  17162}},
 {  1238,  27, { 17162,  17178,  17179,  17163}},
 {  1239,  27, { 17163,  17179,  17180,  17164}},
 {  1240,  27, { 17164,  17180,  17181,  17165}},
 {  1241,  27, { 17165,  17181,  17182,  17166}},
 {  1242,  27, { 17166,  17182,  17183,  17167}},
 {  1243,  27, { 17167,  17183,  17184,  17168}},
 {  1244,  27, { 17168,  17184,  17185,  17169}},
 {  1245,  27, { 17169,  17185,  17186,  17170}},
 {  1246,  27, { 17170,  17186,  17187,  17171}},
 {  1247,  27, { 17027,  17031,  17188,  17172}},
 {  1248,  27, { 17172,  17188,  17189,  17173}},
 {  1249,  27, { 17173,  17189,  17190,  17174}},
 {  1250,  27, { 17174,  17190,  17191,  17175}},
 {  1251,  27, { 17175,  17191,  17192,  17176}},
 {  1252,  27, { 17176,  17192,  17193,  17177}},
 {  1253,  27, { 17177,  17193,  17194,  17178}},
 {  1254,  27, { 17178,  17194,  17195,  17179}},
 {  1255,  27, { 17179,  17195,  17196,  17180}},
 {  1256,  27, { 17180,  17196,  17197,  17181}},
 {  1257,  27, { 17181,  17197,  17198,  17182}},
 {  1258,  27, { 17182,  17198,  17199,  17183}},
 {  1259,  27, { 17183,  17199,  17200,  17184}},
 {  1260,  27, { 17184,  17200,  17201,  17185}},
 {  1261,  27, { 17185,  17201,  17202,  17186}},
 {  1262,  27, { 17186,  17202,  17203,  17187}},
 {  1263,  27, { 17031,  17035,  17204,  17188}},
 {  1264,  27, { 17188,  17204,  17205,  17189}},
 {  1265,  27, { 17189,  17205,  17206,  17190}},
 {  1266,  27, { 17190,  17206,  17207,  17191}},
 {  1267,  27, { 17191,  17207,  17208,  17192}},
 {  1268,  27, { 17192,  17208,  17209,  17193}},
 {  1269,  27, { 17193,  17209,  17210,  17194}},
 {  1270,  27, { 17194,  17210,  17211,  17195}},
 {  1271,  27, { 17195,  17211,  17212,  17196}},
 {  1272,  27, { 17196,  17212,  17213,  17197}},
 {  1273,  27, { 17197,  17213,  17214,  17198}},
 {  1274,  27, { 17198,  17214,  17215,  17199}},
 {  1275,  27, { 17199,  17215,  17216,  17200}},
 {  1276,  27, { 17200,  17216,  17217,  17201}},
 {  1277,  27, { 17201,  17217,  17218,  17202}},
 {  1278,  27, { 17202,  17218,  17219,  17203}},
 {  1279,  27, { 17035,  17039,  17220,  17204}},
 {  1280,  27, { 17204,  17220,  17221,  17205}},
 {  1281,  27, { 17205,  17221,  17222,  17206}},
 {  1282,  27, { 17206,  17222,  17223,  17207}},
 {  1283,  27, { 17207,  17223,  17224,  17208}},
 {  1284,  27, { 17208,  17224,  17225,  17209}},
 {  1285,  27, { 17209,  17225,  17226,  17210}},
 {  1286,  27, { 17210,  17226,  17227,  17211}},
 {  1287,  27, { 17211,  17227,  17228,  17212}},
 {  1288,  27, { 17212,  17228,  17229,  17213}},
 {  1289,  27, { 17213,  17229,  17230,  17214}},
 {  1290,  27, { 17214,  17230,  17231,  17215}},
 {  1291,  27, { 17215,  17231,  17232,  17216}},
 {  1292,  27, { 17216,  17232,  17233,  17217}},
 {  1293,  27, { 17217,  17233,  17234,  17218}},
 {  1294,  27, { 17218,  17234,  17235,  17219}},
 {  1295,  27, { 17039,  17043,  17236,  17220}},
 {  1296,  27, { 17220,  17236,  17237,  17221}},
 {  1297,  27, { 17221,  17237,  17238,  17222}},
 {  1298,  27, { 17222,  17238,  17239,  17223}},
 {  1299,  27, { 17223,  17239,  17240,  17224}},
 {  1300,  27, { 17224,  17240,  17241,  17225}},
 {  1301,  27, { 17225,  17241,  17242,  17226}},
 {  1302,  27, { 17226,  17242,  17243,  17227}},
 {  1303,  27, { 17227,  17243,  17244,  17228}},
 {  1304,  27, { 17228,  17244,  17245,  17229}},
 {  1305,  27, { 17229,  17245,  17246,  17230}},
 {  1306,  27, { 17230,  17246,  17247,  17231}},
 {  1307,  27, { 17231,  17247,  17248,  17232}},
 {  1308,  27, { 17232,  17248,  17249,  17233}},
 {  1309,  27, { 17233,  17249,  17250,  17234}},
 {  1310,  27, { 17234,  17250,  17251,  17235}},
 {  1311,  27, { 17043,  17047,  17252,  17236}},
 {  1312,  27, { 17236,  17252,  17253,  17237}},
 {  1313,  27, { 17237,  17253,  17254,  17238}},
 {  1314,  27, { 17238,  17254,  17255,  17239}},
 {  1315,  27, { 17239,  17255,  17256,  17240}},
 {  1316,  27, { 17240,  17256,  17257,  17241}},
 {  1317,  27, { 17241,  17257,  17258,  17242}},
 {  1318,  27, { 17242,  17258,  17259,  17243}},
 {  1319,  27, { 17243,  17259,  17260,  17244}},
 {  1320,  27, { 17244,  17260,  17261,  17245}},
 {  1321,  27, { 17245,  17261,  17262,  17246}},
 {  1322,  27, { 17246,  17262,  17263,  17247}},
 {  1323,  27, { 17247,  17263,  17264,  17248}},
 {  1324,  27, { 17248,  17264,  17265,  17249}},
 {  1325,  27, { 17249,  17265,  17266,  17250}},
 {  1326,  27, { 17250,  17266,  17267,  17251}},
 {  1327,  27, { 17047,  17051,  17268,  17252}},
 {  1328,  27, { 17252,  17268,  17269,  17253}},
 {  1329,  27, { 17253,  17269,  17270,  17254}},
 {  1330,  27, { 17254,  17270,  17271,  17255}},
 {  1331,  27, { 17255,  17271,  17272,  17256}},
 {  1332,  27, { 17256,  17272,  17273,  17257}},
 {  1333,  27, { 17257,  17273,  17274,  17258}},
 {  1334,  27, { 17258,  17274,  17275,  17259}},
 {  1335,  27, { 17259,  17275,  17276,  17260}},
 {  1336,  27, { 17260,  17276,  17277,  17261}},
 {  1337,  27, { 17261,  17277,  17278,  17262}},
 {  1338,  27, { 17262,  17278,  17279,  17263}},
 {  1339,  27, { 17263,  17279,  17280,  17264}},
 {  1340,  27, { 17264,  17280,  17281,  17265}},
 {  1341,  27, { 17265,  17281,  17282,  17266}},
 {  1342,  27, { 17266,  17282,  17283,  17267}},
 {  1343,  27, { 17051,  17055,  17284,  17268}},
 {  1344,  27, { 17268,  17284,  17285,  17269}},
 {  1345,  27, { 17269,  17285,  17286,  17270}},
 {  1346,  27, { 17270,  17286,  17287,  17271}},
 {  1347,  27, { 17271,  17287,  17288,  17272}},
 {  1348,  27, { 17272,  17288,  17289,  17273}},
 {  1349,  27, { 17273,  17289,  17290,  17274}},
 {  1350,  27, { 17274,  17290,  17291,  17275}},
 {  1351,  27, { 17275,  17291,  17292,  17276}},
 {  1352,  27, { 17276,  17292,  17293,  17277}},
 {  1353,  27, { 17277,  17293,  17294,  17278}},
 {  1354,  27, { 17278,  17294,  17295,  17279}},
 {  1355,  27, { 17279,  17295,  17296,  17280}},
 {  1356,  27, { 17280,  17296,  17297,  17281}},
 {  1357,  27, { 17281,  17297,  17298,  17282}},
 {  1358,  27, { 17282,  17298,  17299,  17283}},
 {  1359,  27, { 17055,  17059,  17300,  17284}},
 {  1360,  27, { 17284,  17300,  17301,  17285}},
 {  1361,  27, { 17285,  17301,  17302,  17286}},
 {  1362,  27, { 17286,  17302,  17303,  17287}},
 {  1363,  27, { 17287,  17303,  17304,  17288}},
 {  1364,  27, { 17288,  17304,  17305,  17289}},
 {  1365,  27, { 17289,  17305,  17306,  17290}},
 {  1366,  27, { 17290,  17306,  17307,  17291}},
 {  1367,  27, { 17291,  17307,  17308,  17292}},
 {  1368,  27, { 17292,  17308,  17309,  17293}},
 {  1369,  27, { 17293,  17309,  17310,  17294}},
 {  1370,  27, { 17294,  17310,  17311,  17295}},
 {  1371,  27, { 17295,  17311,  17312,  17296}},
 {  1372,  27, { 17296,  17312,  17313,  17297}},
 {  1373,  27, { 17297,  17313,  17314,  17298}},
 {  1374,  27, { 17298,  17314,  17315,  17299}},
 {  1375,  27, { 17059,  17063,  17316,  17300}},
 {  1376,  27, { 17300,  17316,  17317,  17301}},
 {  1377,  27, { 17301,  17317,  17318,  17302}},
 {  1378,  27, { 17302,  17318,  17319,  17303}},
 {  1379,  27, { 17303,  17319,  17320,  17304}},
 {  1380,  27, { 17304,  17320,  17321,  17305}},
 {  1381,  27, { 17305,  17321,  17322,  17306}},
 {  1382,  27, { 17306,  17322,  17323,  17307}},
 {  1383,  27, { 17307,  17323,  17324,  17308}},
 {  1384,  27, { 17308,  17324,  17325,  17309}},
 {  1385,  27, { 17309,  17325,  17326,  17310}},
 {  1386,  27, { 17310,  17326,  17327,  17311}},
 {  1387,  27, { 17311,  17327,  17328,  17312}},
 {  1388,  27, { 17312,  17328,  17329,  17313}},
 {  1389,  27, { 17313,  17329,  17330,  17314}},
 {  1390,  27, { 17314,  17330,  17331,  17315}},
 {  1391,  27, { 17063,  17067,  17332,  17316}},
 {  1392,  27, { 17316,  17332,  17333,  17317}},
 {  1393,  27, { 17317,  17333,  17334,  17318}},
 {  1394,  27, { 17318,  17334,  17335,  17319}},
 {  1395,  27, { 17319,  17335,  17336,  17320}},
 {  1396,  27, { 17320,  17336,  17337,  17321}},
 {  1397,  27, { 17321,  17337,  17338,  17322}},
 {  1398,  27, { 17322,  17338,  17339,  17323}},
 {  1399,  27, { 17323,  17339,  17340,  17324}},
 {  1400,  27, { 17324,  17340,  17341,  17325}},
 {  1401,  27, { 17325,  17341,  17342,  17326}},
 {  1402,  27, { 17326,  17342,  17343,  17327}},
 {  1403,  27, { 17327,  17343,  17344,  17328}},
 {  1404,  27, { 17328,  17344,  17345,  17329}},
 {  1405,  27, { 17329,  17345,  17346,  17330}},
 {  1406,  27, { 17330,  17346,  17347,  17331}},
 {  1407,  27, { 17067,  17071,  17348,  17332}},
 {  1408,  27, { 17332,  17348,  17349,  17333}},
 {  1409,  27, { 17333,  17349,  17350,  17334}},
 {  1410,  27, { 17334,  17350,  17351,  17335}},
 {  1411,  27, { 17335,  17351,  17352,  17336}},
 {  1412,  27, { 17336,  17352,  17353,  17337}},
 {  1413,  27, { 17337,  17353,  17354,  17338}},
 {  1414,  27, { 17338,  17354,  17355,  17339}},
 {  1415,  27, { 17339,  17355,  17356,  17340}},
 {  1416,  27, { 17340,  17356,  17357,  17341}},
 {  1417,  27, { 17341,  17357,  17358,  17342}},
 {  1418,  27, { 17342,  17358,  17359,  17343}},
 {  1419,  27, { 17343,  17359,  17360,  17344}},
 {  1420,  27, { 17344,  17360,  17361,  17345}},
 {  1421,  27, { 17345,  17361,  17362,  17346}},
 {  1422,  27, { 17346,  17362,  17363,  17347}},
 {  1423,  27, { 17071,  17075,  17364,  17348}},
 {  1424,  27, { 17348,  17364,  17365,  17349}},
 {  1425,  27, { 17349,  17365,  17366,  17350}},
 {  1426,  27, { 17350,  17366,  17367,  17351}},
 {  1427,  27, { 17351,  17367,  17368,  17352}},
 {  1428,  27, { 17352,  17368,  17369,  17353}},
 {  1429,  27, { 17353,  17369,  17370,  17354}},
 {  1430,  27, { 17354,  17370,  17371,  17355}},
 {  1431,  27, { 17355,  17371,  17372,  17356}},
 {  1432,  27, { 17356,  17372,  17373,  17357}},
 {  1433,  27, { 17357,  17373,  17374,  17358}},
 {  1434,  27, { 17358,  17374,  17375,  17359}},
 {  1435,  27, { 17359,  17375,  17376,  17360}},
 {  1436,  27, { 17360,  17376,  17377,  17361}},
 {  1437,  27, { 17361,  17377,  17378,  17362}},
 {  1438,  27, { 17362,  17378,  17379,  17363}},
 {  1439,  27, { 17075,  17079,  17380,  17364}},
 {  1440,  27, { 17364,  17380,  17381,  17365}},
 {  1441,  27, { 17365,  17381,  17382,  17366}},
 {  1442,  27, { 17366,  17382,  17383,  17367}},
 {  1443,  27, { 17367,  17383,  17384,  17368}},
 {  1444,  27, { 17368,  17384,  17385,  17369}},
 {  1445,  27, { 17369,  17385,  17386,  17370}},
 {  1446,  27, { 17370,  17386,  17387,  17371}},
 {  1447,  27, { 17371,  17387,  17388,  17372}},
 {  1448,  27, { 17372,  17388,  17389,  17373}},
 {  1449,  27, { 17373,  17389,  17390,  17374}},
 {  1450,  27, { 17374,  17390,  17391,  17375}},
 {  1451,  27, { 17375,  17391,  17392,  17376}},
 {  1452,  27, { 17376,  17392,  17393,  17377}},
 {  1453,  27, { 17377,  17393,  17394,  17378}},
 {  1454,  27, { 17378,  17394,  17395,  17379}},
 {  1455,  27, { 17079,  17083,  17396,  17380}},
 {  1456,  27, { 17380,  17396,  17397,  17381}},
 {  1457,  27, { 17381,  17397,  17398,  17382}},
 {  1458,  27, { 17382,  17398,  17399,  17383}},
 {  1459,  27, { 17383,  17399,  17400,  17384}},
 {  1460,  27, { 17384,  17400,  17401,  17385}},
 {  1461,  27, { 17385,  17401,  17402,  17386}},
 {  1462,  27, { 17386,  17402,  17403,  17387}},
 {  1463,  27, { 17387,  17403,  17404,  17388}},
 {  1464,  27, { 17388,  17404,  17405,  17389}},
 {  1465,  27, { 17389,  17405,  17406,  17390}},
 {  1466,  27, { 17390,  17406,  17407,  17391}},
 {  1467,  27, { 17391,  17407,  17408,  17392}},
 {  1468,  27, { 17392,  17408,  17409,  17393}},
 {  1469,  27, { 17393,  17409,  17410,  17394}},
 {  1470,  27, { 17394,  17410,  17411,  17395}},
 {  1471,  27, { 17083,  17087,  17412,  17396}},
 {  1472,  27, { 17396,  17412,  17413,  17397}},
 {  1473,  27, { 17397,  17413,  17414,  17398}},
 {  1474,  27, { 17398,  17414,  17415,  17399}},
 {  1475,  27, { 17399,  17415,  17416,  17400}},
 {  1476,  27, { 17400,  17416,  17417,  17401}},
 {  1477,  27, { 17401,  17417,  17418,  17402}},
 {  1478,  27, { 17402,  17418,  17419,  17403}},
 {  1479,  27, { 17403,  17419,  17420,  17404}},
 {  1480,  27, { 17404,  17420,  17421,  17405}},
 {  1481,  27, { 17405,  17421,  17422,  17406}},
 {  1482,  27, { 17406,  17422,  17423,  17407}},
 {  1483,  27, { 17407,  17423,  17424,  17408}},
 {  1484,  27, { 17408,  17424,  17425,  17409}},
 {  1485,  27, { 17409,  17425,  17426,  17410}},
 {  1486,  27, { 17410,  17426,  17427,  17411}},
 {  1487,  27, { 17087,  17091,  17428,  17412}},
 {  1488,  27, { 17412,  17428,  17429,  17413}},
 {  1489,  27, { 17413,  17429,  17430,  17414}},
 {  1490,  27, { 17414,  17430,  17431,  17415}},
 {  1491,  27, { 17415,  17431,  17432,  17416}},
 {  1492,  27, { 17416,  17432,  17433,  17417}},
 {  1493,  27, { 17417,  17433,  17434,  17418}},
 {  1494,  27, { 17418,  17434,  17435,  17419}},
 {  1495,  27, { 17419,  17435,  17436,  17420}},
 {  1496,  27, { 17420,  17436,  17437,  17421}},
 {  1497,  27, { 17421,  17437,  17438,  17422}},
 {  1498,  27, { 17422,  17438,  17439,  17423}},
 {  1499,  27, { 17423,  17439,  17440,  17424}},
 {  1500,  27, { 17424,  17440,  17441,  17425}},
 {  1501,  27, { 17425,  17441,  17442,  17426}},
 {  1502,  27, { 17426,  17442,  17443,  17427}},
 {  1503,  27, { 17107,  17123,  17451,  17444}},
 {  1504,  27, { 17444,  17451,  17452,  17445}},
 {  1505,  27, { 17445,  17452,  17453,  17446}},
 {  1506,  27, { 17446,  17453,  17454,  17447}},
 {  1507,  27, { 17447,  17454,  17455,  17448}},
 {  1508,  27, { 17448,  17455,  17456,  17449}},
 {  1509,  27, { 17449,  17456,  17457,  17450}},
 {  1510,  27, { 17450,  17457,  16375,  16366}},
 {  1511,  27, { 17123,  17139,  17458,  17451}},
 {  1512,  27, { 17451,  17458,  17459,  17452}},
 {  1513,  27, { 17452,  17459,  17460,  17453}},
 {  1514,  27, { 17453,  17460,  17461,  17454}},
 {  1515,  27, { 17454,  17461,  17462,  17455}},
 {  1516,  27, { 17455,  17462,  17463,  17456}},
 {  1517,  27, { 17456,  17463,  17464,  17457}},
 {  1518,  27, { 17457,  17464,  16384,  16375}},
 {  1519,  27, { 17139,  17155,  17465,  17458}},
 {  1520,  27, { 17458,  17465,  17466,  17459}},
 {  1521,  27, { 17459,  17466,  17467,  17460}},
 {  1522,  27, { 17460,  17467,  17468,  17461}},
 {  1523,  27, { 17461,  17468,  17469,  17462}},
 {  1524,  27, { 17462,  17469,  17470,  17463}},
 {  1525,  27, { 17463,  17470,  17471,  17464}},
 {  1526,  27, { 17464,  17471,  16393,  16384}},
 {  1527,  27, { 17155,  17171,  17472,  17465}},
 {  1528,  27, { 17465,  17472,  17473,  17466}},
 {  1529,  27, { 17466,  17473,  17474,  17467}},
 {  1530,  27, { 17467,  17474,  17475,  17468}},
 {  1531,  27, { 17468,  17475,  17476,  17469}},
 {  1532,  27, { 17469,  17476,  17477,  17470}},
 {  1533,  27, { 17470,  17477,  17478,  17471}},
 {  1534,  27, { 17471,  17478,  16402,  16393}},
 {  1535,  27, { 17171,  17187,  17479,  17472}},
 {  1536,  27, { 17472,  17479,  17480,  17473}},
 {  1537,  27, { 17473,  17480,  17481,  17474}},
 {  1538,  27, { 17474,  17481,  17482,  17475}},
 {  1539,  27, { 17475,  17482,  17483,  17476}},
 {  1540,  27, { 17476,  17483,  17484,  17477}},
 {  1541,  27, { 17477,  17484,  17485,  17478}},
 {  1542,  27, { 17478,  17485,  16411,  16402}},
 {  1543,  27, { 17187,  17203,  17486,  17479}},
 {  1544,  27, { 17479,  17486,  17487,  17480}},
 {  1545,  27, { 17480,  17487,  17488,  17481}},
 {  1546,  27, { 17481,  17488,  17489,  17482}},
 {  1547,  27, { 17482,  17489,  17490,  17483}},
 {  1548,  27, { 17483,  17490,  17491,  17484}},
 {  1549,  27, { 17484,  17491,  17492,  17485}},
 {  1550,  27, { 17485,  17492,  16420,  16411}},
 {  1551,  27, { 17203,  17219,  17493,  17486}},
 {  1552,  27, { 17486,  17493,  17494,  17487}},
 {  1553,  27, { 17487,  17494,  17495,  17488}},
 {  1554,  27, { 17488,  17495,  17496,  17489}},
 {  1555,  27, { 17489,  17496,  17497,  17490}},
 {  1556,  27, { 17490,  17497,  17498,  17491}},
 {  1557,  27, { 17491,  17498,  17499,  17492}},
 {  1558,  27, { 17492,  17499,  16429,  16420}},
 {  1559,  27, { 17219,  17235,  17500,  17493}},
 {  1560,  27, { 17493,  17500,  17501,  17494}},
 {  1561,  27, { 17494,  17501,  17502,  17495}},
 {  1562,  27, { 17495,  17502,  17503,  17496}},
 {  1563,  27, { 17496,  17503,  17504,  17497}},
 {  1564,  27, { 17497,  17504,  17505,  17498}},
 {  1565,  27, { 17498,  17505,  17506,  17499}},
 {  1566,  27, { 17499,  17506,  16438,  16429}},
 {  1567,  27, { 17235,  17251,  17507,  17500}},
 {  1568,  27, { 17500,  17507,  17508,  17501}},
 {  1569,  27, { 17501,  17508,  17509,  17502}},
 {  1570,  27, { 17502,  17509,  17510,  17503}},
 {  1571,  27, { 17503,  17510,  17511,  17504}},
 {  1572,  27, { 17504,  17511,  17512,  17505}},
 {  1573,  27, { 17505,  17512,  17513,  17506}},
 {  1574,  27, { 17506,  17513,  16447,  16438}},
 {  1575,  27, { 17251,  17267,  17514,  17507}},
 {  1576,  27, { 17507,  17514,  17515,  17508}},
 {  1577,  27, { 17508,  17515,  17516,  17509}},
 {  1578,  27, { 17509,  17516,  17517,  17510}},
 {  1579,  27, { 17510,  17517,  17518,  17511}},
 {  1580,  27, { 17511,  17518,  17519,  17512}},
 {  1581,  27, { 17512,  17519,  17520,  17513}},
 {  1582,  27, { 17513,  17520,  16456,  16447}},
 {  1583,  27, { 17267,  17283,  17521,  17514}},
 {  1584,  27, { 17514,  17521,  17522,  17515}},
 {  1585,  27, { 17515,  17522,  17523,  17516}},
 {  1586,  27, { 17516,  17523,  17524,  17517}},
 {  1587,  27, { 17517,  17524,  17525,  17518}},
 {  1588,  27, { 17518,  17525,  17526,  17519}},
 {  1589,  27, { 17519,  17526,  17527,  17520}},
 {  1590,  27, { 17520,  17527,  16465,  16456}},
 {  1591,  27, { 17283,  17299,  17528,  17521}},
 {  1592,  27, { 17521,  17528,  17529,  17522}},
 {  1593,  27, { 17522,  17529,  17530,  17523}},
 {  1594,  27, { 17523,  17530,  17531,  17524}},
 {  1595,  27, { 17524,  17531,  17532,  17525}},
 {  1596,  27, { 17525,  17532,  17533,  17526}},
 {  1597,  27, { 17526,  17533,  17534,  17527}},
 {  1598,  27, { 17527,  17534,  16474,  16465}},
 {  1599,  27, { 17299,  17315,  17535,  17528}},
 {  1600,  27, { 17528,  17535,  17536,  17529}},
 {  1601,  27, { 17529,  17536,  17537,  17530}},
 {  1602,  27, { 17530,  17537,  17538,  17531}},
 {  1603,  27, { 17531,  17538,  17539,  17532}},
 {  1604,  27, { 17532,  17539,  17540,  17533}},
 {  1605,  27, { 17533,  17540,  17541,  17534}},
 {  1606,  27, { 17534,  17541,  16483,  16474}},
 {  1607,  27, { 17315,  17331,  17542,  17535}},
 {  1608,  27, { 17535,  17542,  17543,  17536}},
 {  1609,  27, { 17536,  17543,  17544,  17537}},
 {  1610,  27, { 17537,  17544,  17545,  17538}},
 {  1611,  27, { 17538,  17545,  17546,  17539}},
 {  1612,  27, { 17539,  17546,  17547,  17540}},
 {  1613,  27, { 17540,  17547,  17548,  17541}},
 {  1614,  27, { 17541,  17548,  16492,  16483}},
 {  1615,  27, { 17331,  17347,  17549,  17542}},
 {  1616,  27, { 17542,  17549,  17550,  17543}},
 {  1617,  27, { 17543,  17550,  17551,  17544}},
 {  1618,  27, { 17544,  17551,  17552,  17545}},
 {  1619,  27, { 17545,  17552,  17553,  17546}},
 {  1620,  27, { 17546,  17553,  17554,  17547}},
 {  1621,  27, { 17547,  17554,  17555,  17548}},
 {  1622,  27, { 17548,  17555,  16501,  16492}},
 {  1623,  27, { 17347,  17363,  17556,  17549}},
 {  1624,  27, { 17549,  17556,  17557,  17550}},
 {  1625,  27, { 17550,  17557,  17558,  17551}},
 {  1626,  27, { 17551,  17558,  17559,  17552}},
 {  1627,  27, { 17552,  17559,  17560,  17553}},
 {  1628,  27, { 17553,  17560,  17561,  17554}},
 {  1629,  27, { 17554,  17561,  17562,  17555}},
 {  1630,  27, { 17555,  17562,  16510,  16501}},
 {  1631,  27, { 17363,  17379,  17563,  17556}},
 {  1632,  27, { 17556,  17563,  17564,  17557}},
 {  1633,  27, { 17557,  17564,  17565,  17558}},
 {  1634,  27, { 17558,  17565,  17566,  17559}},
 {  1635,  27, { 17559,  17566,  17567,  17560}},
 {  1636,  27, { 17560,  17567,  17568,  17561}},
 {  1637,  27, { 17561,  17568,  17569,  17562}},
 {  1638,  27, { 17562,  17569,  16519,  16510}},
 {  1639,  27, { 17379,  17395,  17570,  17563}},
 {  1640,  27, { 17563,  17570,  17571,  17564}},
 {  1641,  27, { 17564,  17571,  17572,  17565}},
 {  1642,  27, { 17565,  17572,  17573,  17566}},
 {  1643,  27, { 17566,  17573,  17574,  17567}},
 {  1644,  27, { 17567,  17574,  17575,  17568}},
 {  1645,  27, { 17568,  17575,  17576,  17569}},
 {  1646,  27, { 17569,  17576,  17577,  16519}},
 {  1647,  27, { 17395,  17411,  17578,  17570}},
 {  1648,  27, { 17570,  17578,  17579,  17571}},
 {  1649,  27, { 17571,  17579,  17580,  17572}},
 {  1650,  27, { 17572,  17580,  17581,  17573}},
 {  1651,  27, { 17573,  17581,  17582,  17574}},
 {  1652,  27, { 17574,  17582,  17583,  17575}},
 {  1653,  27, { 17575,  17583,  17584,  17576}},
 {  1654,  27, { 17576,  17584,  17585,  17577}},
 {  1655,  27, { 17411,  17427,  17586,  17578}},
 {  1656,  27, { 17578,  17586,  17587,  17579}},
 {  1657,  27, { 17579,  17587,  17588,  17580}},
 {  1658,  27, { 17580,  17588,  17589,  17581}},
 {  1659,  27, { 17581,  17589,  17590,  17582}},
 {  1660,  27, { 17582,  17590,  17591,  17583}},
 {  1661,  27, { 17583,  17591,  17592,  17584}},
 {  1662,  27, { 17584,  17592,  17593,  17585}},
 {  1663,  27, { 17427,  17443,  17594,  17586}},
 {  1664,  27, { 17586,  17594,  17595,  17587}},
 {  1665,  27, { 17587,  17595,  17596,  17588}},
 {  1666,  27, { 17588,  17596,  17597,  17589}},
 {  1667,  27, { 17589,  17597,  17598,  17590}},
 {  1668,  27, { 17590,  17598,  17599,  17591}},
 {  1669,  27, { 17591,  17599,  17600,  17592}},
 {  1670,  27, { 17592,  17600,  17601,  17593}},
 {  1671,  28, { 11548,  11557,  11560,  11551}},
 {  1672,  28, { 11551,  11560,  11563,  11554}},
 {  1673,  28, { 11557,  11566,  11569,  11560}},
 {  1674,  28, { 11560,  11569,  11572,  11563}},
 {  1675,  28, { 11566,  11575,  11578,  11569}},
 {  1676,  28, { 11569,  11578,  11581,  11572}},
 {  1677,  28, { 11548,  11557,  17604,  17602}},
 {  1678,  28, { 17602,  17604,  17605,  17603}},
 {  1679,  28, { 11557,  11566,  17606,  17604}},
 {  1680,  28, { 17604,  17606,  17607,  17605}},
 {  1681,  28, { 11566,  11575,  17608,  17606}},
 {  1682,  28, { 17606,  17608,  17609,  17607}},
 {  1683,  28, { 11575,  11584,  11587,  11578}},
 {  1684,  28, { 11578,  11587,  11590,  11581}},
 {  1685,  28, { 11584,  11593,  11596,  11587}},
 {  1686,  28, { 11587,  11596,  11599,  11590}},
 {  1687,  28, { 11593,  11602,  11605,  11596}},
 {  1688,  28, { 11596,  11605,  11608,  11599}},
 {  1689,  28, { 11575,  11584,  17610,  17608}},
 {  1690,  28, { 17608,  17610,  17611,  17609}},
 {  1691,  28, { 11584,  11593,  17612,  17610}},
 {  1692,  28, { 17610,  17612,  17613,  17611}},
 {  1693,  28, { 11593,  11602,  17614,  17612}},
 {  1694,  28, { 17612,  17614,  17615,  17613}},
 {  1695,  28, { 11602,  11611,  11614,  11605}},
 {  1696,  28, { 11605,  11614,  11617,  11608}},
 {  1697,  28, { 11611,  11620,  11623,  11614}},
 {  1698,  28, { 11614,  11623,  11626,  11617}},
 {  1699,  28, { 11620,  11629,  11632,  11623}},
 {  1700,  28, { 11623,  11632,  11635,  11626}},
 {  1701,  28, { 11629,  11638,  11641,  11632}},
 {  1702,  28, { 11632,  11641,  11644,  11635}},
 {  1703,  28, { 11602,  11611,  17616,  17614}},
 {  1704,  28, { 17614,  17616,  17617,  17615}},
 {  1705,  28, { 11611,  11620,  17618,  17616}},
 {  1706,  28, { 17616,  17618,  17619,  17617}},
 {  1707,  28, { 11620,  11629,  17620,  17618}},
 {  1708,  28, { 17618,  17620,  17621,  17619}},
 {  1709,  28, { 11629,  11638,  17622,  17620}},
 {  1710,  28, { 17620,  17622,  17623,  17621}},
 {  1711,  28, { 11638,  11647,  11650,  11641}},
 {  1712,  28, { 11641,  11650,  11653,  11644}},
 {  1713,  28, { 11647,  11656,  11659,  11650}},
 {  1714,  28, { 11650,  11659,  11662,  11653}},
 {  1715,  28, { 11656,  11665,  11668,  11659}},
 {  1716,  28, { 11659,  11668,  11671,  11662}},
 {  1717,  28, { 11638,  11647,  17624,  17622}},
 {  1718,  28, { 17622,  17624,  17625,  17623}},
 {  1719,  28, { 11647,  11656,  17626,  17624}},
 {  1720,  28, { 17624,  17626,  17627,  17625}},
 {  1721,  28, { 11656,  11665,  17628,  17626}},
 {  1722,  28, { 17626,  17628,  17629,  17627}},
 {  1723,  28, { 11548,  17602,  17630,  11674}},
 {  1724,  28, { 11674,  17630,  17631,  11675}},
 {  1725,  28, { 17602,  17603,  17632,  17630}},
 {  1726,  28, { 17630,  17632,  17633,  17631}},
 {  1727,  28, { 11548,  11551,  11676,  11674}},
 {  1728,  28, { 11674,  11676,  11677,  11675}},
 {  1729,  28, { 11551,  11554,  11678,  11676}},
 {  1730,  28, { 11676,  11678,  11679,  11677}},
 {  1731,  28, { 17603,  17605,  17634,  17632}},
 {  1732,  28, { 17632,  17634,  17635,  17633}},
 {  1733,  28, { 17605,  17607,  17636,  17634}},
 {  1734,  28, { 17634,  17636,  17637,  17635}},
 {  1735,  28, { 17607,  17609,  17638,  17636}},
 {  1736,  28, { 17636,  17638,  17639,  17637}},
 {  1737,  28, { 17609,  17611,  17640,  17638}},
 {  1738,  28, { 17638,  17640,  17641,  17639}},
 {  1739,  28, { 17611,  17613,  17642,  17640}},
 {  1740,  28, { 17640,  17642,  17643,  17641}},
 {  1741,  28, { 17613,  17615,  17644,  17642}},
 {  1742,  28, { 17642,  17644,  17645,  17643}},
 {  1743,  28, { 17615,  17617,  17646,  17644}},
 {  1744,  28, { 17644,  17646,  17647,  17645}},
 {  1745,  28, { 17617,  17619,  17648,  17646}},
 {  1746,  28, { 17646,  17648,  17649,  17647}},
 {  1747,  28, { 17619,  17621,  17650,  17648}},
 {  1748,  28, { 17648,  17650,  17651,  17649}},
 {  1749,  28, { 17621,  17623,  17652,  17650}},
 {  1750,  28, { 17650,  17652,  17653,  17651}},
 {  1751,  28, { 17623,  17625,  17654,  17652}},
 {  1752,  28, { 17652,  17654,  17655,  17653}},
 {  1753,  28, { 17625,  17627,  17656,  17654}},
 {  1754,  28, { 17654,  17656,  17657,  17655}},
 {  1755,  28, { 17627,  17629,  17658,  17656}},
 {  1756,  28, { 17656,  17658,  17659,  17657}},
 {  1757,  28, { 17629,  17628,  17660,  17658}},
 {  1758,  28, { 17658,  17660,  17661,  17659}},
 {  1759,  28, { 17628,  11665,  11776,  17660}},
 {  1760,  28, { 17660,  11776,  11777,  17661}},
 {  1761,  28, { 11665,  11668,  11778,  11776}},
 {  1762,  28, { 11776,  11778,  11779,  11777}},
 {  1763,  28, { 11668,  11671,  11780,  11778}},
 {  1764,  28, { 11778,  11780,  11781,  11779}},
 {  1765,  28, { 11554,  11563,  11784,  11678}},
 {  1766,  28, { 11678,  11784,  11785,  11679}},
 {  1767,  28, { 11563,  11572,  11788,  11784}},
 {  1768,  28, { 11784,  11788,  11789,  11785}},
 {  1769,  28, { 11572,  11581,  11792,  11788}},
 {  1770,  28, { 11788,  11792,  11793,  11789}},
 {  1771,  28, { 11581,  11590,  11796,  11792}},
 {  1772,  28, { 11792,  11796,  11797,  11793}},
 {  1773,  28, { 11590,  11599,  11800,  11796}},
 {  1774,  28, { 11796,  11800,  11801,  11797}},
 {  1775,  28, { 11599,  11608,  11804,  11800}},
 {  1776,  28, { 11800,  11804,  11805,  11801}},
 {  1777,  28, { 11608,  11617,  11808,  11804}},
 {  1778,  28, { 11804,  11808,  11809,  11805}},
 {  1779,  28, { 11617,  11626,  11812,  11808}},
 {  1780,  28, { 11808,  11812,  11813,  11809}},
 {  1781,  28, { 11626,  11635,  11816,  11812}},
 {  1782,  28, { 11812,  11816,  11817,  11813}},
 {  1783,  28, { 11635,  11644,  11820,  11816}},
 {  1784,  28, { 11816,  11820,  11821,  11817}},
 {  1785,  28, { 11644,  11653,  11824,  11820}},
 {  1786,  28, { 11820,  11824,  11825,  11821}},
 {  1787,  28, { 11653,  11662,  11828,  11824}},
 {  1788,  28, { 11824,  11828,  11829,  11825}},
 {  1789,  28, { 11662,  11671,  11780,  11828}},
 {  1790,  28, { 11828,  11780,  11781,  11829}},
 {  1791,  28, { 11675,  17631,  17662,  11830}},
 {  1792,  28, { 11830,  17662,  17663,  11831}},
 {  1793,  28, { 17631,  17633,  17664,  17662}},
 {  1794,  28, { 17662,  17664,  17665,  17663}},
 {  1795,  28, { 11675,  11677,  11832,  11830}},
 {  1796,  28, { 11830,  11832,  11833,  11831}},
 {  1797,  28, { 11677,  11679,  11834,  11832}},
 {  1798,  28, { 11832,  11834,  11835,  11833}},
 {  1799,  28, { 17633,  17635,  17666,  17664}},
 {  1800,  28, { 17664,  17666,  17667,  17665}},
 {  1801,  28, { 17635,  17637,  17668,  17666}},
 {  1802,  28, { 17666,  17668,  17669,  17667}},
 {  1803,  28, { 17637,  17639,  17670,  17668}},
 {  1804,  28, { 17668,  17670,  17671,  17669}},
 {  1805,  28, { 17639,  17641,  17672,  17670}},
 {  1806,  28, { 17670,  17672,  17673,  17671}},
 {  1807,  28, { 17641,  17643,  17674,  17672}},
 {  1808,  28, { 17672,  17674,  17675,  17673}},
 {  1809,  28, { 17643,  17645,  17676,  17674}},
 {  1810,  28, { 17674,  17676,  17677,  17675}},
 {  1811,  28, { 17645,  17647,  17678,  17676}},
 {  1812,  28, { 17676,  17678,  17679,  17677}},
 {  1813,  28, { 17647,  17649,  17680,  17678}},
 {  1814,  28, { 17678,  17680,  17681,  17679}},
 {  1815,  28, { 17649,  17651,  17682,  17680}},
 {  1816,  28, { 17680,  17682,  17683,  17681}},
 {  1817,  28, { 17651,  17653,  17684,  17682}},
 {  1818,  28, { 17682,  17684,  17685,  17683}},
 {  1819,  28, { 17653,  17655,  17686,  17684}},
 {  1820,  28, { 17684,  17686,  17687,  17685}},
 {  1821,  28, { 17655,  17657,  17688,  17686}},
 {  1822,  28, { 17686,  17688,  17689,  17687}},
 {  1823,  28, { 17657,  17659,  17690,  17688}},
 {  1824,  28, { 17688,  17690,  17691,  17689}},
 {  1825,  28, { 17659,  17661,  17692,  17690}},
 {  1826,  28, { 17690,  17692,  17693,  17691}},
 {  1827,  28, { 17661,  11777,  11932,  17692}},
 {  1828,  28, { 17692,  11932,  11933,  17693}},
 {  1829,  28, { 11777,  11779,  11934,  11932}},
 {  1830,  28, { 11932,  11934,  11935,  11933}},
 {  1831,  28, { 11779,  11781,  11936,  11934}},
 {  1832,  28, { 11934,  11936,  11937,  11935}},
 {  1833,  28, { 11679,  11785,  11940,  11834}},
 {  1834,  28, { 11834,  11940,  11941,  11835}},
 {  1835,  28, { 11785,  11789,  11944,  11940}},
 {  1836,  28, { 11940,  11944,  11945,  11941}},
 {  1837,  28, { 11789,  11793,  11948,  11944}},
 {  1838,  28, { 11944,  11948,  11949,  11945}},
 {  1839,  28, { 11793,  11797,  11952,  11948}},
 {  1840,  28, { 11948,  11952,  11953,  11949}},
 {  1841,  28, { 11797,  11801,  11956,  11952}},
 {  1842,  28, { 11952,  11956,  11957,  11953}},
 {  1843,  28, { 11801,  11805,  11960,  11956}},
 {  1844,  28, { 11956,  11960,  11961,  11957}},
 {  1845,  28, { 11805,  11809,  11964,  11960}},
 {  1846,  28, { 11960,  11964,  11965,  11961}},
 {  1847,  28, { 11809,  11813,  11968,  11964}},
 {  1848,  28, { 11964,  11968,  11969,  11965}},
 {  1849,  28, { 11813,  11817,  11972,  11968}},
 {  1850,  28, { 11968,  11972,  11973,  11969}},
 {  1851,  28, { 11817,  11821,  11976,  11972}},
 {  1852,  28, { 11972,  11976,  11977,  11973}},
 {  1853,  28, { 11821,  11825,  11980,  11976}},
 {  1854,  28, { 11976,  11980,  11981,  11977}},
 {  1855,  28, { 11825,  11829,  11984,  11980}},
 {  1856,  28, { 11980,  11984,  11985,  11981}},
 {  1857,  28, { 11829,  11781,  11936,  11984}},
 {  1858,  28, { 11984,  11936,  11937,  11985}},
 {  1859,  28, { 17665,  17667,  17694,  17663}},
 {  1860,  28, { 17663,  17694,  11987,  11831}},
 {  1861,  28, { 17667,  17669,  17695,  17694}},
 {  1862,  28, { 17694,  17695,  11991,  11987}},
 {  1863,  28, { 17669,  17671,  17696,  17695}},
 {  1864,  28, { 17695,  17696,  11995,  11991}},
 {  1865,  28, { 17671,  17673,  17697,  17696}},
 {  1866,  28, { 17696,  17697,  11999,  11995}},
 {  1867,  28, { 17673,  17675,  17698,  17697}},
 {  1868,  28, { 17697,  17698,  12003,  11999}},
 {  1869,  28, { 17675,  17677,  17699,  17698}},
 {  1870,  28, { 17698,  17699,  12007,  12003}},
 {  1871,  28, { 17677,  17679,  17700,  17699}},
 {  1872,  28, { 17699,  17700,  12011,  12007}},
 {  1873,  28, { 17679,  17681,  17701,  17700}},
 {  1874,  28, { 17700,  17701,  12015,  12011}},
 {  1875,  28, { 17681,  17683,  17702,  17701}},
 {  1876,  28, { 17701,  17702,  12019,  12015}},
 {  1877,  28, { 17683,  17685,  17703,  17702}},
 {  1878,  28, { 17702,  17703,  12023,  12019}},
 {  1879,  28, { 17685,  17687,  17704,  17703}},
 {  1880,  28, { 17703,  17704,  12027,  12023}},
 {  1881,  28, { 17687,  17689,  17705,  17704}},
 {  1882,  28, { 17704,  17705,  12031,  12027}},
 {  1883,  28, { 17689,  17691,  17693,  17705}},
 {  1884,  28, { 17705,  17693,  11933,  12031}},
 {  1885,  28, { 11831,  11987,  11989,  11833}},
 {  1886,  28, { 11833,  11989,  11941,  11835}},
 {  1887,  28, { 11987,  11991,  11993,  11989}},
 {  1888,  28, { 11989,  11993,  11945,  11941}},
 {  1889,  28, { 11991,  11995,  11997,  11993}},
 {  1890,  28, { 11993,  11997,  11949,  11945}},
 {  1891,  28, { 11995,  11999,  12001,  11997}},
 {  1892,  28, { 11997,  12001,  11953,  11949}},
 {  1893,  28, { 11999,  12003,  12005,  12001}},
 {  1894,  28, { 12001,  12005,  11957,  11953}},
 {  1895,  28, { 12003,  12007,  12009,  12005}},
 {  1896,  28, { 12005,  12009,  11961,  11957}},
 {  1897,  28, { 12007,  12011,  12013,  12009}},
 {  1898,  28, { 12009,  12013,  11965,  11961}},
 {  1899,  28, { 12011,  12015,  12017,  12013}},
 {  1900,  28, { 12013,  12017,  11969,  11965}},
 {  1901,  28, { 12015,  12019,  12021,  12017}},
 {  1902,  28, { 12017,  12021,  11973,  11969}},
 {  1903,  28, { 12019,  12023,  12025,  12021}},
 {  1904,  28, { 12021,  12025,  11977,  11973}},
 {  1905,  28, { 12023,  12027,  12029,  12025}},
 {  1906,  28, { 12025,  12029,  11981,  11977}},
 {  1907,  28, { 12027,  12031,  12033,  12029}},
 {  1908,  28, { 12029,  12033,  11985,  11981}},
 {  1909,  28, { 12031,  11933,  11935,  12033}},
 {  1910,  28, { 12033,  11935,  11937,  11985}},
 { 20000,  32, { 20000,  20001,  20002,  20002}},
 { 20001,  32, { 20003,  20004,  20005,  20006}},
 { 20002,  32, { 20004,  20007,  20008,  20005}},
 { 20003,  32, { 20009,  20010,  20011,  20012}},
 { 20004,  32, { 20009,  20013,  20014,  20010}},
 { 20005,  32, { 20015,  20003,  20006,  20016}},
 { 20006,  32, { 20017,  20018,  20019,  20020}},
 { 20007,  32, { 20018,  20021,  20002,  20019}},
 { 20008,  32, { 20017,  20022,  20021,  20018}},
 { 20009,  32, { 20007,  20023,  20024,  20008}},
 { 20010,  32, { 20013,  20025,  20026,  20014}},
 { 20011,  32, { 20027,  20000,  20002,  20002}},
 { 20012,  32, { 20027,  20028,  20029,  20000}},
 { 20013,  32, { 20000,  20029,  20030,  20001}},
 { 20014,  32, { 20028,  20031,  20032,  20029}},
 { 20015,  32, { 20033,  20034,  20035,  20036}},
 { 20016,  32, { 20034,  20037,  20038,  20035}},
 { 20017,  32, { 20036,  20035,  20039,  20040}},
 { 20018,  32, { 20035,  20038,  20041,  20039}},
 { 20019,  32, { 20040,  20039,  20042,  20043}},
 { 20020,  32, { 20039,  20041,  20044,  20042}},
 { 20021,  32, { 20043,  20042,  20045,  20046}},
 { 20022,  32, { 20042,  20044,  20047,  20045}},
 { 20023,  32, { 20046,  20045,  20048,  20049}},
 { 20024,  32, { 20045,  20047,  20050,  20048}},
 { 20025,  32, { 20048,  20050,  20051,  20049}},
 { 20026,  32, { 20046,  20049,  20052,  20053}},
 { 20027,  32, { 20049,  20051,  20054,  20052}},
 { 20028,  32, { 20053,  20052,  20055,  20056}},
 { 20029,  32, { 20052,  20054,  20057,  20055}},
 { 20030,  32, { 20056,  20055,  20058,  20059}},
 { 20031,  32, { 20055,  20057,  20060,  20058}},
 { 20032,  32, { 20058,  20061,  20062,  20059}},
 { 20033,  32, { 20060,  20063,  20061,  20058}},
 { 20034,  32, { 20063,  20064,  20065,  20061}},
 { 20035,  32, { 20061,  20065,  20066,  20062}},
 { 20036,  32, { 20064,  20067,  20068,  20065}},
 { 20037,  32, { 20065,  20068,  20069,  20066}},
 { 20038,  32, { 20067,  20070,  20071,  20068}},
 { 20039,  32, { 20068,  20071,  20072,  20069}},
 { 20040,  32, { 20070,  20073,  20074,  20071}},
 { 20041,  32, { 20071,  20074,  20075,  20072}},
 { 20042,  32, { 20073,  20076,  20077,  20074}},
 { 20043,  32, { 20074,  20077,  20078,  20075}},
 { 20044,  32, { 20077,  20079,  20080,  20078}},
 { 20045,  32, { 20076,  20081,  20079,  20077}},
 { 20046,  32, { 20079,  20082,  20083,  20080}},
 { 20047,  32, { 20081,  20084,  20082,  20079}},
 { 20048,  32, { 20085,  20084,  20081,  20086}},
 { 20049,  32, { 20087,  20076,  20073,  20088}},
 { 20050,  32, { 20086,  20081,  20076,  20087}},
 { 20051,  32, { 20088,  20073,  20070,  20089}},
 { 20052,  32, { 20089,  20070,  20067,  20090}},
 { 20053,  32, { 20090,  20067,  20064,  20091}},
 { 20054,  32, { 20091,  20064,  20063,  20092}},
 { 20055,  32, { 20093,  20094,  20041,  20038}},
 { 20056,  32, { 20095,  20093,  20038,  20037}},
 { 20057,  32, { 20041,  20094,  20096,  20044}},
 { 20058,  32, { 20044,  20096,  20011,  20047}},
 { 20059,  32, { 20047,  20011,  20010,  20050}},
 { 20060,  32, { 20050,  20010,  20014,  20051}},
 { 20061,  32, { 20051,  20014,  20026,  20054}},
 { 20062,  32, { 20054,  20026,  20097,  20057}},
 { 20063,  32, { 20060,  20098,  20092,  20063}},
 { 20064,  32, { 20057,  20097,  20098,  20060}},
 { 20065,  32, { 20086,  20099,  20100,  20085}},
 { 20066,  32, { 20087,  20101,  20099,  20086}},
 { 20067,  32, { 20088,  20102,  20101,  20087}},
 { 20068,  32, { 20089,  20103,  20102,  20088}},
 { 20069,  32, { 20090,  20104,  20103,  20089}},
 { 20070,  32, { 20091,  20105,  20104,  20090}},
 { 20071,  32, { 20092,  20106,  20105,  20091}},
 { 20072,  32, { 20106,  20092,  20098,  20107}},
 { 20073,  32, { 20098,  20097,  20108,  20107}},
 { 20074,  32, { 20097,  20026,  20025,  20108}},
 { 20075,  32, { 20093,  20095,  20109,  20110}},
 { 20076,  32, { 20094,  20093,  20110,  20111}},
 { 20077,  32, { 20096,  20094,  20111,  20112}},
 { 20078,  32, { 20110,  20109,  20113,  20114}},
 { 20079,  32, { 20113,  20115,  20116,  20114}},
 { 20080,  32, { 20117,  20118,  20119,  20120}},
 { 20081,  32, { 20118,  20121,  20122,  20119}},
 { 20082,  32, { 20121,  20123,  20124,  20122}},
 { 20083,  32, { 20123,  20125,  20126,  20124}},
 { 20084,  32, { 20127,  20128,  20100,  20099}},
 { 20085,  32, { 20129,  20127,  20099,  20101}},
 { 20086,  32, { 20130,  20107,  20108,  20131}},
 { 20087,  32, { 20131,  20108,  20025,  20024}},
 { 20088,  32, { 20012,  20006,  20005,  20009}},
 { 20089,  32, { 20009,  20005,  20008,  20013}},
 { 20090,  32, { 20132,  20133,  20015,  20016}},
 { 20091,  32, { 20110,  20114,  20132,  20111}},
 { 20092,  32, { 20116,  20134,  20135,  20133}},
 { 20093,  32, { 20133,  20135,  20136,  20015}},
 { 20094,  32, { 20015,  20136,  20020,  20003}},
 { 20095,  32, { 20003,  20020,  20019,  20004}},
 { 20096,  32, { 20004,  20019,  20002,  20007}},
 { 20097,  32, { 20020,  20136,  20137,  20017}},
 { 20098,  32, { 20137,  20138,  20022,  20017}},
 { 20099,  32, { 20139,  20140,  20141,  20142}},
 { 20100,  32, { 20139,  20143,  20144,  20140}},
 { 20101,  32, { 20143,  20120,  20119,  20144}},
 { 20102,  32, { 20119,  20122,  20145,  20144}},
 { 20103,  32, { 20112,  20016,  20006,  20012}},
 { 20104,  32, { 20028,  20027,  20146,  20147}},
 { 20105,  32, { 20146,  20142,  20141,  20147}},
 { 20106,  32, { 20140,  20144,  20145,  20148}},
 { 20107,  32, { 20032,  20149,  20150,  20151}},
 { 20108,  32, { 20152,  20153,  20154,  20155}},
 { 20109,  32, { 20154,  20156,  20157,  20155}},
 { 20110,  32, { 20154,  20153,  20158,  20159}},
 { 20111,  32, { 20159,  20160,  20156,  20154}},
 { 20112,  32, { 20161,  20162,  20163,  20150}},
 { 20113,  32, { 20164,  20165,  20149,  20166}},
 { 20114,  32, { 20167,  20162,  20161,  20152}},
 { 20115,  32, { 20152,  20161,  20165,  20153}},
 { 20116,  32, { 20153,  20165,  20164,  20158}},
 { 20117,  32, { 20031,  20166,  20149,  20032}},
 { 20118,  32, { 20168,  20169,  20170,  20171}},
 { 20119,  32, { 20171,  20170,  20172,  20173}},
 { 20120,  32, { 20169,  20174,  20175,  20170}},
 { 20121,  32, { 20170,  20175,  20176,  20172}},
 { 20122,  32, { 20177,  20178,  20179,  20180}},
 { 20123,  32, { 20180,  20160,  20159,  20177}},
 { 20124,  32, { 20177,  20159,  20158,  20178}},
 { 20125,  32, { 20180,  20179,  20174,  20181}},
 { 20126,  32, { 20181,  20182,  20160,  20180}},
 { 20127,  32, { 20181,  20174,  20169,  20183}},
 { 20128,  32, { 20183,  20184,  20182,  20181}},
 { 20129,  32, { 20183,  20169,  20168,  20185}},
 { 20130,  32, { 20185,  20186,  20184,  20183}},
 { 20131,  32, { 20185,  20168,  20187,  20188}},
 { 20132,  32, { 20188,  20189,  20186,  20185}},
 { 20133,  32, { 20190,  20191,  20192,  20192}},
 { 20134,  32, { 20190,  20164,  20166,  20191}},
 { 20135,  32, { 20179,  20192,  20175,  20174}},
 { 20136,  32, { 20193,  20194,  20195,  20196}},
 { 20137,  32, { 20195,  20167,  20197,  20196}},
 { 20138,  32, { 20196,  20197,  20198,  20193}},
 { 20139,  32, { 20155,  20197,  20167,  20152}},
 { 20140,  32, { 20155,  20157,  20198,  20197}},
 { 20141,  32, { 20148,  20199,  20140,  20140}},
 { 20142,  32, { 20140,  20199,  20200,  20141}},
 { 20143,  32, { 20201,  20163,  20202,  20130}},
 { 20144,  32, { 20151,  20150,  20163,  20201}},
 { 20145,  32, { 20203,  20187,  20126,  20125}},
 { 20146,  32, { 20204,  20194,  20193,  20193}},
 { 20147,  32, { 20129,  20205,  20206,  20207}},
 { 20148,  32, { 20206,  20208,  20209,  20207}},
 { 20149,  32, { 20210,  20211,  20212,  20213}},
 { 20150,  32, { 20214,  20215,  20211,  20210}},
 { 20151,  32, { 20127,  20216,  20217,  20128}},
 { 20152,  32, { 20217,  20216,  20214,  20214}},
 { 20153,  32, { 20128,  20217,  20210,  20213}},
 { 20154,  32, { 20210,  20217,  20214,  20214}},
 { 20155,  32, { 20218,  20219,  20205,  20205}},
 { 20156,  32, { 20206,  20205,  20219,  20220}},
 { 20157,  32, { 20220,  20221,  20208,  20206}},
 { 20158,  32, { 20221,  20220,  20193,  20198}},
 { 20159,  32, { 20219,  20204,  20193,  20220}},
 { 20160,  32, { 20103,  20218,  20205,  20102}},
 { 20161,  32, { 20145,  20173,  20172,  20148}},
 { 20162,  32, { 20122,  20124,  20173,  20145}},
 { 20163,  32, { 20218,  20103,  20104,  20222}},
 { 20164,  32, { 20030,  20029,  20032,  20151}},
 { 20165,  32, { 20007,  20002,  20001,  20023}},
 { 20166,  32, { 20189,  20223,  20211,  20215}},
 { 20167,  32, { 20223,  20224,  20212,  20211}},
 { 20168,  32, { 20102,  20205,  20129,  20101}},
 { 20169,  32, { 20216,  20127,  20129,  20207}},
 { 20170,  32, { 20207,  20209,  20214,  20216}},
 { 20171,  32, { 20223,  20189,  20188,  20225}},
 { 20172,  32, { 20188,  20187,  20203,  20225}},
 { 20173,  32, { 20225,  20203,  20224,  20223}},
 { 20174,  32, { 20200,  20226,  20191,  20227}},
 { 20175,  32, { 20227,  20191,  20166,  20031}},
 { 20176,  32, { 20031,  20028,  20147,  20227}},
 { 20177,  32, { 20227,  20147,  20141,  20200}},
 { 20178,  32, { 20172,  20176,  20199,  20148}},
 { 20179,  32, { 20176,  20226,  20200,  20199}},
 { 20180,  32, { 20126,  20171,  20173,  20124}},
 { 20181,  32, { 20165,  20161,  20150,  20149}},
 { 20182,  32, { 20222,  20104,  20105,  20228}},
 { 20183,  32, { 20202,  20228,  20105,  20106}},
 { 20184,  32, { 20107,  20130,  20202,  20106}},
 { 20185,  32, { 20131,  20229,  20201,  20130}},
 { 20186,  32, { 20229,  20131,  20024,  20023}},
 { 20187,  32, { 20016,  20112,  20111,  20132}},
 { 20188,  32, { 20114,  20116,  20133,  20132}},
 { 20189,  32, { 20096,  20112,  20012,  20011}},
 { 20190,  32, { 20151,  20201,  20229,  20030}},
 { 20191,  32, { 20030,  20229,  20023,  20001}},
 { 20192,  32, { 20219,  20218,  20222,  20204}},
 { 20193,  32, { 20204,  20222,  20228,  20194}},
 { 20194,  32, { 20024,  20025,  20013,  20008}},
 { 20195,  32, { 20168,  20171,  20126,  20187}},
 { 20196,  32, { 20194,  20228,  20230,  20195}},
 { 20197,  32, { 20195,  20230,  20162,  20167}},
 { 20198,  32, { 20230,  20228,  20202,  20202}},
 { 20199,  32, { 20202,  20163,  20162,  20230}},
 { 20200,  32, { 20192,  20179,  20178,  20190}},
 { 20201,  32, { 20190,  20178,  20158,  20164}},
 { 20202,  32, { 20231,  20137,  20136,  20135}},
 { 20203,  32, { 20231,  20232,  20138,  20137}},
 { 20204,  32, { 20233,  20231,  20135,  20134}},
 { 20205,  32, { 20233,  20234,  20232,  20231}},
 { 20206,  32, { 20235,  20236,  20234,  20233}},
 { 20207,  32, { 20139,  20142,  20022,  20138}},
 { 20208,  32, { 20027,  20002,  20021,  20021}},
 { 20209,  32, { 20142,  20146,  20021,  20022}},
 { 20210,  32, { 20146,  20027,  20021,  20021}},
 { 20211,  32, { 20117,  20120,  20234,  20236}},
 { 20212,  32, { 20120,  20143,  20232,  20234}},
 { 20213,  32, { 20143,  20139,  20138,  20232}},
 { 20214,  32, { 20237,  20238,  20239,  20239}},
 { 20215,  32, { 20240,  20241,  20242,  20243}},
 { 20216,  32, { 20240,  20244,  20245,  20241}},
 { 20217,  32, { 20246,  20247,  20248,  20249}},
 { 20218,  32, { 20247,  20250,  20251,  20248}},
 { 20219,  32, { 20243,  20242,  20252,  20253}},
 { 20220,  32, { 20254,  20255,  20256,  20257}},
 { 20221,  32, { 20255,  20258,  20259,  20256}},
 { 20222,  32, { 20254,  20237,  20258,  20255}},
 { 20223,  32, { 20244,  20260,  20261,  20245}},
 { 20224,  32, { 20250,  20262,  20263,  20251}},
 { 20225,  32, { 20264,  20237,  20239,  20239}},
 { 20226,  32, { 20239,  20265,  20266,  20264}},
 { 20227,  32, { 20238,  20267,  20265,  20239}},
 { 20228,  32, { 20265,  20268,  20269,  20266}},
 { 20229,  32, { 20270,  20271,  20034,  20033}},
 { 20230,  32, { 20034,  20271,  20272,  20037}},
 { 20231,  32, { 20273,  20274,  20271,  20270}},
 { 20232,  32, { 20271,  20274,  20275,  20272}},
 { 20233,  32, { 20276,  20274,  20273,  20277}},
 { 20234,  32, { 20276,  20278,  20275,  20274}},
 { 20235,  32, { 20279,  20276,  20277,  20280}},
 { 20236,  32, { 20279,  20281,  20278,  20276}},
 { 20237,  32, { 20282,  20279,  20280,  20283}},
 { 20238,  32, { 20283,  20284,  20285,  20282}},
 { 20239,  32, { 20282,  20285,  20281,  20279}},
 { 20240,  32, { 20286,  20283,  20280,  20287}},
 { 20241,  32, { 20286,  20288,  20284,  20283}},
 { 20242,  32, { 20289,  20286,  20287,  20290}},
 { 20243,  32, { 20289,  20291,  20288,  20286}},
 { 20244,  32, { 20292,  20289,  20290,  20293}},
 { 20245,  32, { 20292,  20294,  20291,  20289}},
 { 20246,  32, { 20292,  20295,  20296,  20294}},
 { 20247,  32, { 20293,  20297,  20295,  20292}},
 { 20248,  32, { 20297,  20298,  20299,  20295}},
 { 20249,  32, { 20295,  20299,  20300,  20296}},
 { 20250,  32, { 20298,  20301,  20302,  20299}},
 { 20251,  32, { 20299,  20302,  20303,  20300}},
 { 20252,  32, { 20301,  20304,  20305,  20302}},
 { 20253,  32, { 20302,  20305,  20306,  20303}},
 { 20254,  32, { 20304,  20307,  20308,  20305}},
 { 20255,  32, { 20305,  20308,  20309,  20306}},
 { 20256,  32, { 20307,  20310,  20311,  20308}},
 { 20257,  32, { 20308,  20311,  20312,  20309}},
 { 20258,  32, { 20311,  20313,  20314,  20312}},
 { 20259,  32, { 20310,  20315,  20313,  20311}},
 { 20260,  32, { 20313,  20082,  20084,  20314}},
 { 20261,  32, { 20315,  20083,  20082,  20313}},
 { 20262,  32, { 20085,  20316,  20314,  20084}},
 { 20263,  32, { 20317,  20318,  20309,  20312}},
 { 20264,  32, { 20316,  20317,  20312,  20314}},
 { 20265,  32, { 20318,  20319,  20306,  20309}},
 { 20266,  32, { 20319,  20320,  20303,  20306}},
 { 20267,  32, { 20320,  20321,  20300,  20303}},
 { 20268,  32, { 20321,  20322,  20296,  20300}},
 { 20269,  32, { 20323,  20272,  20275,  20324}},
 { 20270,  32, { 20095,  20037,  20272,  20323}},
 { 20271,  32, { 20278,  20325,  20324,  20275}},
 { 20272,  32, { 20281,  20246,  20325,  20278}},
 { 20273,  32, { 20284,  20250,  20247,  20285}},
 { 20274,  32, { 20285,  20247,  20246,  20281}},
 { 20275,  32, { 20288,  20262,  20250,  20284}},
 { 20276,  32, { 20291,  20326,  20262,  20288}},
 { 20277,  32, { 20296,  20322,  20327,  20294}},
 { 20278,  32, { 20294,  20327,  20326,  20291}},
 { 20279,  32, { 20085,  20100,  20328,  20316}},
 { 20280,  32, { 20316,  20328,  20329,  20317}},
 { 20281,  32, { 20317,  20329,  20330,  20318}},
 { 20282,  32, { 20318,  20330,  20331,  20319}},
 { 20283,  32, { 20319,  20331,  20332,  20320}},
 { 20284,  32, { 20320,  20332,  20333,  20321}},
 { 20285,  32, { 20321,  20333,  20334,  20322}},
 { 20286,  32, { 20327,  20322,  20334,  20335}},
 { 20287,  32, { 20335,  20336,  20326,  20327}},
 { 20288,  32, { 20336,  20263,  20262,  20326}},
 { 20289,  32, { 20109,  20095,  20323,  20337}},
 { 20290,  32, { 20337,  20323,  20324,  20338}},
 { 20291,  32, { 20338,  20324,  20325,  20339}},
 { 20292,  32, { 20113,  20109,  20337,  20340}},
 { 20293,  32, { 20113,  20340,  20341,  20115}},
 { 20294,  32, { 20342,  20343,  20118,  20117}},
 { 20295,  32, { 20343,  20344,  20121,  20118}},
 { 20296,  32, { 20344,  20345,  20123,  20121}},
 { 20297,  32, { 20345,  20346,  20125,  20123}},
 { 20298,  32, { 20100,  20128,  20347,  20328}},
 { 20299,  32, { 20328,  20347,  20348,  20329}},
 { 20300,  32, { 20336,  20335,  20349,  20350}},
 { 20301,  32, { 20263,  20336,  20350,  20260}},
 { 20302,  32, { 20251,  20244,  20240,  20248}},
 { 20303,  32, { 20248,  20240,  20243,  20249}},
 { 20304,  32, { 20253,  20252,  20351,  20352}},
 { 20305,  32, { 20338,  20352,  20340,  20337}},
 { 20306,  32, { 20351,  20353,  20354,  20341}},
 { 20307,  32, { 20252,  20355,  20353,  20351}},
 { 20308,  32, { 20242,  20257,  20355,  20252}},
 { 20309,  32, { 20245,  20237,  20254,  20241}},
 { 20310,  32, { 20241,  20254,  20257,  20242}},
 { 20311,  32, { 20356,  20355,  20257,  20256}},
 { 20312,  32, { 20256,  20259,  20357,  20356}},
 { 20313,  32, { 20358,  20359,  20360,  20361}},
 { 20314,  32, { 20362,  20363,  20360,  20359}},
 { 20315,  32, { 20343,  20342,  20363,  20362}},
 { 20316,  32, { 20364,  20344,  20343,  20362}},
 { 20317,  32, { 20339,  20249,  20243,  20253}},
 { 20318,  32, { 20365,  20264,  20266,  20366}},
 { 20319,  32, { 20366,  20358,  20361,  20365}},
 { 20320,  32, { 20364,  20362,  20359,  20367}},
 { 20321,  32, { 20368,  20369,  20370,  20268}},
 { 20322,  32, { 20371,  20372,  20373,  20374}},
 { 20323,  32, { 20374,  20375,  20376,  20371}},
 { 20324,  32, { 20377,  20372,  20371,  20378}},
 { 20325,  32, { 20371,  20376,  20379,  20378}},
 { 20326,  32, { 20369,  20380,  20381,  20382}},
 { 20327,  32, { 20383,  20370,  20384,  20385}},
 { 20328,  32, { 20373,  20382,  20381,  20386}},
 { 20329,  32, { 20372,  20384,  20382,  20373}},
 { 20330,  32, { 20377,  20385,  20384,  20372}},
 { 20331,  32, { 20268,  20370,  20383,  20269}},
 { 20332,  32, { 20387,  20388,  20389,  20390}},
 { 20333,  32, { 20391,  20387,  20390,  20392}},
 { 20334,  32, { 20393,  20394,  20388,  20387}},
 { 20335,  32, { 20395,  20393,  20387,  20391}},
 { 20336,  32, { 20396,  20397,  20398,  20399}},
 { 20337,  32, { 20397,  20377,  20378,  20398}},
 { 20338,  32, { 20398,  20378,  20379,  20399}},
 { 20339,  32, { 20394,  20396,  20399,  20400}},
 { 20340,  32, { 20399,  20379,  20401,  20400}},
 { 20341,  32, { 20388,  20394,  20400,  20402}},
 { 20342,  32, { 20400,  20401,  20403,  20402}},
 { 20343,  32, { 20389,  20388,  20402,  20404}},
 { 20344,  32, { 20402,  20403,  20405,  20404}},
 { 20345,  32, { 20406,  20389,  20404,  20407}},
 { 20346,  32, { 20404,  20405,  20408,  20407}},
 { 20347,  32, { 20409,  20410,  20411,  20411}},
 { 20348,  32, { 20410,  20383,  20385,  20411}},
 { 20349,  32, { 20394,  20393,  20409,  20396}},
 { 20350,  32, { 20412,  20413,  20414,  20415}},
 { 20351,  32, { 20414,  20416,  20417,  20415}},
 { 20352,  32, { 20415,  20417,  20386,  20412}},
 { 20353,  32, { 20386,  20417,  20374,  20373}},
 { 20354,  32, { 20417,  20416,  20375,  20374}},
 { 20355,  32, { 20367,  20359,  20418,  20418}},
 { 20356,  32, { 20359,  20358,  20419,  20418}},
 { 20357,  32, { 20349,  20420,  20380,  20421}},
 { 20358,  32, { 20421,  20380,  20369,  20368}},
 { 20359,  32, { 20346,  20406,  20203,  20125}},
 { 20360,  32, { 20414,  20413,  20422,  20422}},
 { 20361,  32, { 20423,  20424,  20348,  20425}},
 { 20362,  32, { 20425,  20426,  20427,  20423}},
 { 20363,  32, { 20428,  20429,  20430,  20431}},
 { 20364,  32, { 20213,  20212,  20429,  20428}},
 { 20365,  32, { 20128,  20432,  20433,  20347}},
 { 20366,  32, { 20432,  20431,  20433,  20433}},
 { 20367,  32, { 20213,  20428,  20432,  20128}},
 { 20368,  32, { 20428,  20431,  20432,  20432}},
 { 20369,  32, { 20424,  20434,  20435,  20435}},
 { 20370,  32, { 20434,  20424,  20423,  20436}},
 { 20371,  32, { 20423,  20427,  20437,  20436}},
 { 20372,  32, { 20416,  20414,  20436,  20437}},
 { 20373,  32, { 20434,  20436,  20414,  20422}},
 { 20374,  32, { 20330,  20424,  20435,  20331}},
 { 20375,  32, { 20367,  20391,  20392,  20364}},
 { 20376,  32, { 20364,  20392,  20345,  20344}},
 { 20377,  32, { 20332,  20331,  20435,  20438}},
 { 20378,  32, { 20368,  20268,  20265,  20267}},
 { 20379,  32, { 20261,  20238,  20237,  20245}},
 { 20380,  32, { 20429,  20439,  20408,  20430}},
 { 20381,  32, { 20429,  20212,  20224,  20439}},
 { 20382,  32, { 20329,  20348,  20424,  20330}},
 { 20383,  32, { 20348,  20347,  20433,  20425}},
 { 20384,  32, { 20433,  20431,  20426,  20425}},
 { 20385,  32, { 20407,  20408,  20439,  20440}},
 { 20386,  32, { 20439,  20224,  20203,  20440}},
 { 20387,  32, { 20440,  20203,  20406,  20407}},
 { 20388,  32, { 20269,  20383,  20410,  20441}},
 { 20389,  32, { 20441,  20410,  20442,  20419}},
 { 20390,  32, { 20419,  20358,  20366,  20441}},
 { 20391,  32, { 20441,  20366,  20266,  20269}},
 { 20392,  32, { 20418,  20395,  20391,  20367}},
 { 20393,  32, { 20419,  20442,  20395,  20418}},
 { 20394,  32, { 20392,  20390,  20346,  20345}},
 { 20395,  32, { 20370,  20369,  20382,  20384}},
 { 20396,  32, { 20442,  20410,  20409,  20395}},
 { 20397,  32, { 20393,  20395,  20409,  20409}},
 { 20398,  32, { 20333,  20332,  20438,  20443}},
 { 20399,  32, { 20333,  20443,  20420,  20334}},
 { 20400,  32, { 20334,  20420,  20349,  20335}},
 { 20401,  32, { 20349,  20421,  20444,  20350}},
 { 20402,  32, { 20261,  20260,  20350,  20444}},
 { 20403,  32, { 20338,  20339,  20253,  20352}},
 { 20404,  32, { 20352,  20351,  20341,  20340}},
 { 20405,  32, { 20246,  20249,  20339,  20325}},
 { 20406,  32, { 20444,  20421,  20368,  20267}},
 { 20407,  32, { 20261,  20444,  20267,  20238}},
 { 20408,  32, { 20438,  20435,  20434,  20422}},
 { 20409,  32, { 20443,  20438,  20422,  20413}},
 { 20410,  32, { 20251,  20263,  20260,  20244}},
 { 20411,  32, { 20406,  20346,  20390,  20389}},
 { 20412,  32, { 20386,  20381,  20445,  20412}},
 { 20413,  32, { 20412,  20445,  20443,  20413}},
 { 20414,  32, { 20420,  20443,  20445,  20445}},
 { 20415,  32, { 20445,  20381,  20380,  20420}},
 { 20416,  32, { 20385,  20377,  20397,  20411}},
 { 20417,  32, { 20411,  20397,  20396,  20409}},
 { 20418,  32, { 20355,  20356,  20446,  20353}},
 { 20419,  32, { 20356,  20357,  20447,  20446}},
 { 20420,  32, { 20353,  20446,  20448,  20354}},
 { 20421,  32, { 20446,  20447,  20449,  20448}},
 { 20422,  32, { 20448,  20449,  20236,  20235}},
 { 20423,  32, { 20357,  20259,  20361,  20360}},
 { 20424,  32, { 20264,  20258,  20237,  20237}},
 { 20425,  32, { 20259,  20258,  20365,  20361}},
 { 20426,  32, { 20365,  20258,  20264,  20264}},
 { 20427,  32, { 20236,  20449,  20342,  20117}},
 { 20428,  32, { 20449,  20447,  20363,  20342}},
 { 20429,  32, { 20447,  20357,  20360,  20363}},
 { 20430,  32, { 20176,  20192,  20175,  20175}},
 { 20431,  32, { 20176,  20192,  20191,  20226}},
 { 20432,  32, { 20450,  20354,  20448,  20235}},
 { 20433,  32, { 20450,  20134,  20233,  20235}},
 { 20838,  34, { 20043,  20501,  20500,  20040}},
 { 20839,  34, { 20501,  20468,  20523,  20500}},
 { 20840,  34, { 20468,  20466,  20541,  20523}},
 { 20841,  34, { 20040,  20500,  20499,  20036}},
 { 20842,  34, { 20500,  20523,  20522,  20499}},
 { 20843,  34, { 20523,  20541,  20539,  20522}},
 { 20844,  34, { 20036,  20499,  20498,  20033}},
 { 20845,  34, { 20499,  20522,  20524,  20498}},
 { 20846,  34, { 20522,  20539,  20540,  20524}},
 { 20847,  34, { 20544,  20543,  20542,  20545}},
 { 20848,  34, { 20545,  20542,  20559,  20560}},
 { 20849,  34, { 20560,  20559,  20459,  20458}},
 { 20850,  34, { 20543,  20540,  20539,  20542}},
 { 20851,  34, { 20542,  20539,  20541,  20559}},
 { 20852,  34, { 20559,  20541,  20466,  20459}},
 { 20853,  34, { 20561,  20545,  20560,  20562}},
 { 20854,  34, { 20562,  20560,  20458,  20470}},
 { 20855,  34, { 20562,  20569,  20547,  20561}},
 { 20856,  34, { 20561,  20547,  20546,  20618}},
 { 20857,  34, { 20562,  20568,  20569,  20569}},
 { 20858,  34, { 20562,  20470,  20567,  20568}},
 { 20859,  34, { 20568,  20567,  20566,  20569}},
 { 20860,  34, { 20550,  20548,  20549,  20551}},
 { 20861,  34, { 20548,  20546,  20547,  20549}},
 { 20862,  34, { 20547,  20569,  20570,  20549}},
 { 20863,  34, { 20569,  20566,  20565,  20570}},
 { 20864,  34, { 20549,  20570,  20571,  20551}},
 { 20865,  34, { 20570,  20565,  20575,  20571}},
 { 20866,  34, { 20550,  20551,  20553,  20552}},
 { 20867,  34, { 20552,  20553,  20555,  20554}},
 { 20868,  34, { 20575,  20594,  20593,  20571}},
 { 20869,  34, { 20594,  20592,  20591,  20593}},
 { 20870,  34, { 20571,  20593,  20553,  20551}},
 { 20871,  34, { 20593,  20591,  20555,  20553}},
 { 20872,  34, { 20575,  20600,  20597,  20594}},
 { 20873,  34, { 20594,  20597,  20596,  20592}},
 { 20874,  34, { 20600,  20492,  20588,  20597}},
 { 20875,  34, { 20062,  20506,  20505,  20059}},
 { 20876,  34, { 20506,  20521,  20526,  20505}},
 { 20877,  34, { 20059,  20505,  20504,  20056}},
 { 20878,  34, { 20505,  20526,  20525,  20504}},
 { 20879,  34, { 20056,  20504,  20503,  20053}},
 { 20880,  34, { 20504,  20525,  20463,  20503}},
 { 20881,  34, { 20053,  20503,  20502,  20046}},
 { 20882,  34, { 20503,  20463,  20457,  20502}},
 { 20883,  34, { 20046,  20502,  20501,  20043}},
 { 20884,  34, { 20502,  20457,  20468,  20501}},
 { 20885,  34, { 20599,  20572,  20600,  20575}},
 { 20886,  34, { 20492,  20493,  20576,  20588}},
 { 20887,  34, { 20536,  20482,  20535,  20537}},
 { 20888,  34, { 20482,  20481,  20538,  20535}},
 { 20889,  34, { 20481,  20467,  20469,  20538}},
 { 20890,  34, { 20467,  20466,  20468,  20469}},
 { 20891,  34, { 20468,  20457,  20463,  20469}},
 { 20892,  34, { 20463,  20525,  20538,  20469}},
 { 20893,  34, { 20525,  20526,  20535,  20538}},
 { 20894,  34, { 20526,  20521,  20537,  20535}},
 { 20895,  34, { 20536,  20537,  20521,  20534}},
 { 20896,  34, { 20582,  20585,  20579,  20581}},
 { 20897,  34, { 20585,  20198,  20157,  20579}},
 { 20898,  34, { 20581,  20579,  20578,  20584}},
 { 20899,  34, { 20579,  20157,  20156,  20578}},
 { 20900,  34, { 20584,  20578,  20580,  20583}},
 { 20901,  34, { 20578,  20156,  20160,  20580}},
 { 20902,  34, { 20069,  20072,  20509,  20508}},
 { 20903,  34, { 20072,  20075,  20510,  20509}},
 { 20904,  34, { 20075,  20078,  20511,  20510}},
 { 20905,  34, { 20078,  20080,  20512,  20511}},
 { 20906,  34, { 20508,  20509,  20518,  20519}},
 { 20907,  34, { 20509,  20510,  20517,  20518}},
 { 20908,  34, { 20510,  20511,  20516,  20517}},
 { 20909,  34, { 20511,  20512,  20515,  20516}},
 { 20910,  34, { 20080,  20083,  20513,  20512}},
 { 20911,  34, { 20512,  20513,  20514,  20515}},
 { 20912,  34, { 20514,  20528,  20527,  20515}},
 { 20913,  34, { 20527,  20529,  20516,  20515}},
 { 20914,  34, { 20529,  20530,  20517,  20516}},
 { 20915,  34, { 20530,  20531,  20518,  20517}},
 { 20916,  34, { 20531,  20532,  20519,  20518}},
 { 20917,  34, { 20508,  20507,  20066,  20069}},
 { 20918,  34, { 20066,  20507,  20506,  20062}},
 { 20919,  34, { 20506,  20507,  20520,  20521}},
 { 20920,  34, { 20507,  20508,  20519,  20520}},
 { 20921,  34, { 20520,  20519,  20532,  20533}},
 { 20922,  34, { 20520,  20533,  20534,  20521}},
 { 20923,  34, { 20527,  20557,  20558,  20529}},
 { 20924,  34, { 20529,  20558,  20615,  20530}},
 { 20925,  34, { 20530,  20615,  20613,  20531}},
 { 20926,  34, { 20531,  20613,  20495,  20532}},
 { 20927,  34, { 20557,  20601,  20617,  20558}},
 { 20928,  34, { 20558,  20617,  20616,  20615}},
 { 20929,  34, { 20615,  20616,  20614,  20613}},
 { 20930,  34, { 20613,  20614,  20497,  20495}},
 { 20931,  34, { 20474,  20497,  20496,  20473}},
 { 20932,  34, { 20474,  20473,  20476,  20475}},
 { 20933,  34, { 20599,  20575,  20565,  20564}},
 { 20934,  34, { 20564,  20565,  20566,  20567}},
 { 20935,  34, { 20532,  20495,  20478,  20536}},
 { 20936,  34, { 20495,  20497,  20474,  20478}},
 { 20937,  34, { 20536,  20478,  20479,  20482}},
 { 20938,  34, { 20478,  20474,  20475,  20479}},
 { 20939,  34, { 20482,  20479,  20485,  20481}},
 { 20940,  34, { 20479,  20475,  20487,  20485}},
 { 20941,  34, { 20481,  20485,  20465,  20467}},
 { 20942,  34, { 20485,  20487,  20464,  20465}},
 { 20943,  34, { 20467,  20465,  20459,  20466}},
 { 20944,  34, { 20465,  20464,  20458,  20459}},
 { 20945,  34, { 20557,  20556,  20601,  20601}},
 { 20946,  34, { 20556,  20557,  20527,  20528}},
 { 20947,  34, { 20464,  20487,  20563,  20471}},
 { 20948,  34, { 20471,  20563,  20564,  20567}},
 { 20949,  34, { 20458,  20464,  20471,  20470}},
 { 20950,  34, { 20471,  20567,  20470,  20470}},
 { 20951,  34, { 20564,  20572,  20599,  20599}},
 { 20952,  34, { 20564,  20490,  20493,  20572}},
 { 20953,  34, { 20563,  20491,  20490,  20564}},
 { 20954,  34, { 20487,  20488,  20491,  20563}},
 { 20955,  34, { 20182,  20595,  20580,  20160}},
 { 20956,  34, { 20583,  20580,  20595,  20596}},
 { 20957,  34, { 20596,  20595,  20589,  20592}},
 { 20958,  34, { 20184,  20589,  20595,  20182}},
 { 20959,  34, { 20592,  20589,  20590,  20591}},
 { 20960,  34, { 20186,  20590,  20589,  20184}},
 { 20961,  34, { 20591,  20590,  20598,  20555}},
 { 20962,  34, { 20189,  20598,  20590,  20186}},
 { 20963,  34, { 20603,  20601,  20556,  20556}},
 { 20964,  34, { 20603,  20606,  20605,  20601}},
 { 20965,  34, { 20606,  20554,  20555,  20605}},
 { 20966,  34, { 20614,  20611,  20612,  20497}},
 { 20967,  34, { 20582,  20496,  20497,  20612}},
 { 20968,  34, { 20585,  20582,  20612,  20607}},
 { 20969,  34, { 20611,  20610,  20607,  20612}},
 { 20970,  34, { 20198,  20585,  20607,  20607}},
 { 20971,  34, { 20610,  20221,  20198,  20607}},
 { 20972,  34, { 20215,  20604,  20598,  20189}},
 { 20973,  34, { 20555,  20598,  20604,  20605}},
 { 20974,  34, { 20214,  20602,  20604,  20215}},
 { 20975,  34, { 20605,  20604,  20602,  20601}},
 { 20976,  34, { 20214,  20209,  20609,  20602}},
 { 20977,  34, { 20617,  20601,  20602,  20609}},
 { 20978,  34, { 20209,  20208,  20608,  20609}},
 { 20979,  34, { 20616,  20617,  20609,  20608}},
 { 20980,  34, { 20616,  20608,  20611,  20614}},
 { 20981,  34, { 20608,  20610,  20611,  20611}},
 { 20982,  34, { 20608,  20208,  20221,  20610}},
 { 20983,  34, { 20493,  20490,  20491,  20574}},
 { 20984,  34, { 20488,  20476,  20574,  20491}},
 { 20985,  34, { 20476,  20473,  20577,  20574}},
 { 20986,  34, { 20576,  20493,  20574,  20577}},
 { 20987,  34, { 20577,  20473,  20496,  20586}},
 { 20988,  34, { 20576,  20577,  20586,  20587}},
 { 20989,  34, { 20588,  20576,  20587,  20597}},
 { 20990,  34, { 20586,  20496,  20582,  20581}},
 { 20991,  34, { 20587,  20586,  20581,  20584}},
 { 20992,  34, { 20597,  20587,  20584,  20583}},
 { 20993,  34, { 20487,  20475,  20476,  20488}},
 { 20994,  34, { 20534,  20533,  20532,  20536}},
 { 20995,  34, { 20493,  20492,  20600,  20572}},
 { 20996,  34, { 20583,  20596,  20597,  20597}},
 { 20997,  34, { 20277,  20273,  20667,  20668}},
 { 20998,  34, { 20668,  20667,  20688,  20636}},
 { 20999,  34, { 20636,  20688,  20703,  20634}},
 { 21000,  34, { 20273,  20270,  20666,  20667}},
 { 21001,  34, { 20667,  20666,  20687,  20688}},
 { 21002,  34, { 20688,  20687,  20702,  20703}},
 { 21003,  34, { 20270,  20033,  20498,  20666}},
 { 21004,  34, { 20666,  20498,  20524,  20687}},
 { 21005,  34, { 20687,  20524,  20540,  20702}},
 { 21006,  34, { 20544,  20705,  20704,  20543}},
 { 21007,  34, { 20705,  20714,  20713,  20704}},
 { 21008,  34, { 20714,  20627,  20626,  20713}},
 { 21009,  34, { 20543,  20704,  20702,  20540}},
 { 21010,  34, { 20704,  20713,  20703,  20702}},
 { 21011,  34, { 20713,  20626,  20634,  20703}},
 { 21012,  34, { 20715,  20716,  20714,  20705}},
 { 21013,  34, { 20716,  20638,  20627,  20714}},
 { 21014,  34, { 20716,  20715,  20706,  20722}},
 { 21015,  34, { 20715,  20618,  20546,  20706}},
 { 21016,  34, { 20716,  20722,  20723,  20723}},
 { 21017,  34, { 20716,  20723,  20719,  20638}},
 { 21018,  34, { 20723,  20722,  20720,  20719}},
 { 21019,  34, { 20550,  20708,  20707,  20548}},
 { 21020,  34, { 20548,  20707,  20706,  20546}},
 { 21021,  34, { 20706,  20707,  20724,  20722}},
 { 21022,  34, { 20722,  20724,  20721,  20720}},
 { 21023,  34, { 20707,  20708,  20725,  20724}},
 { 21024,  34, { 20724,  20725,  20729,  20721}},
 { 21025,  34, { 20550,  20552,  20709,  20708}},
 { 21026,  34, { 20552,  20554,  20710,  20709}},
 { 21027,  34, { 20729,  20725,  20748,  20747}},
 { 21028,  34, { 20747,  20748,  20745,  20744}},
 { 21029,  34, { 20725,  20708,  20709,  20748}},
 { 21030,  34, { 20748,  20709,  20710,  20745}},
 { 21031,  34, { 20729,  20747,  20751,  20754}},
 { 21032,  34, { 20747,  20744,  20750,  20751}},
 { 21033,  34, { 20754,  20751,  20742,  20660}},
 { 21034,  34, { 20297,  20293,  20672,  20673}},
 { 21035,  34, { 20673,  20672,  20690,  20686}},
 { 21036,  34, { 20293,  20290,  20671,  20672}},
 { 21037,  34, { 20672,  20671,  20689,  20690}},
 { 21038,  34, { 20290,  20287,  20670,  20671}},
 { 21039,  34, { 20671,  20670,  20630,  20689}},
 { 21040,  34, { 20287,  20280,  20669,  20670}},
 { 21041,  34, { 20670,  20669,  20624,  20630}},
 { 21042,  34, { 20280,  20277,  20668,  20669}},
 { 21043,  34, { 20669,  20668,  20636,  20624}},
 { 21044,  34, { 20753,  20729,  20754,  20726}},
 { 21045,  34, { 20660,  20742,  20731,  20661}},
 { 21046,  34, { 20700,  20699,  20698,  20652}},
 { 21047,  34, { 20652,  20698,  20701,  20649}},
 { 21048,  34, { 20649,  20701,  20637,  20635}},
 { 21049,  34, { 20635,  20637,  20636,  20634}},
 { 21050,  34, { 20636,  20637,  20630,  20624}},
 { 21051,  34, { 20630,  20637,  20701,  20689}},
 { 21052,  34, { 20689,  20701,  20698,  20690}},
 { 21053,  34, { 20690,  20698,  20699,  20686}},
 { 21054,  34, { 20700,  20697,  20686,  20699}},
 { 21055,  34, { 20736,  20735,  20733,  20739}},
 { 21056,  34, { 20739,  20733,  20375,  20416}},
 { 21057,  34, { 20735,  20738,  20732,  20733}},
 { 21058,  34, { 20733,  20732,  20376,  20375}},
 { 21059,  34, { 20738,  20737,  20734,  20732}},
 { 21060,  34, { 20732,  20734,  20379,  20376}},
 { 21061,  34, { 20301,  20675,  20676,  20304}},
 { 21062,  34, { 20304,  20676,  20677,  20307}},
 { 21063,  34, { 20307,  20677,  20678,  20310}},
 { 21064,  34, { 20310,  20678,  20679,  20315}},
 { 21065,  34, { 20675,  20684,  20683,  20676}},
 { 21066,  34, { 20676,  20683,  20682,  20677}},
 { 21067,  34, { 20677,  20682,  20681,  20678}},
 { 21068,  34, { 20678,  20681,  20680,  20679}},
 { 21069,  34, { 20315,  20679,  20513,  20083}},
 { 21070,  34, { 20679,  20680,  20514,  20513}},
 { 21071,  34, { 20514,  20680,  20691,  20528}},
 { 21072,  34, { 20691,  20680,  20681,  20692}},
 { 21073,  34, { 20692,  20681,  20682,  20693}},
 { 21074,  34, { 20693,  20682,  20683,  20694}},
 { 21075,  34, { 20694,  20683,  20684,  20695}},
 { 21076,  34, { 20675,  20301,  20298,  20674}},
 { 21077,  34, { 20298,  20297,  20673,  20674}},
 { 21078,  34, { 20673,  20686,  20685,  20674}},
 { 21079,  34, { 20674,  20685,  20684,  20675}},
 { 21080,  34, { 20685,  20696,  20695,  20684}},
 { 21081,  34, { 20685,  20686,  20697,  20696}},
 { 21082,  34, { 20691,  20692,  20712,  20711}},
 { 21083,  34, { 20692,  20693,  20767,  20712}},
 { 21084,  34, { 20693,  20694,  20766,  20767}},
 { 21085,  34, { 20694,  20695,  20663,  20766}},
 { 21086,  34, { 20711,  20712,  20769,  20755}},
 { 21087,  34, { 20712,  20767,  20768,  20769}},
 { 21088,  34, { 20767,  20766,  20765,  20768}},
 { 21089,  34, { 20766,  20663,  20665,  20765}},
 { 21090,  34, { 20644,  20641,  20664,  20665}},
 { 21091,  34, { 20644,  20643,  20642,  20641}},
 { 21092,  34, { 20753,  20718,  20721,  20729}},
 { 21093,  34, { 20718,  20719,  20720,  20721}},
 { 21094,  34, { 20695,  20700,  20648,  20663}},
 { 21095,  34, { 20663,  20648,  20644,  20665}},
 { 21096,  34, { 20700,  20652,  20647,  20648}},
 { 21097,  34, { 20648,  20647,  20643,  20644}},
 { 21098,  34, { 20652,  20649,  20653,  20647}},
 { 21099,  34, { 20647,  20653,  20655,  20643}},
 { 21100,  34, { 20649,  20635,  20632,  20653}},
 { 21101,  34, { 20653,  20632,  20633,  20655}},
 { 21102,  34, { 20635,  20634,  20626,  20632}},
 { 21103,  34, { 20632,  20626,  20627,  20633}},
 { 21104,  34, { 20711,  20755,  20556,  20556}},
 { 21105,  34, { 20556,  20528,  20691,  20711}},
 { 21106,  34, { 20633,  20639,  20717,  20655}},
 { 21107,  34, { 20639,  20719,  20718,  20717}},
 { 21108,  34, { 20627,  20638,  20639,  20633}},
 { 21109,  34, { 20639,  20638,  20719,  20719}},
 { 21110,  34, { 20718,  20753,  20726,  20726}},
 { 21111,  34, { 20718,  20726,  20661,  20658}},
 { 21112,  34, { 20717,  20718,  20658,  20659}},
 { 21113,  34, { 20655,  20717,  20659,  20656}},
 { 21114,  34, { 20401,  20379,  20734,  20749}},
 { 21115,  34, { 20737,  20750,  20749,  20734}},
 { 21116,  34, { 20750,  20744,  20743,  20749}},
 { 21117,  34, { 20403,  20401,  20749,  20743}},
 { 21118,  34, { 20744,  20745,  20746,  20743}},
 { 21119,  34, { 20405,  20403,  20743,  20746}},
 { 21120,  34, { 20745,  20710,  20752,  20746}},
 { 21121,  34, { 20408,  20405,  20746,  20752}},
 { 21122,  34, { 20603,  20556,  20755,  20755}},
 { 21123,  34, { 20603,  20755,  20757,  20606}},
 { 21124,  34, { 20606,  20757,  20710,  20554}},
 { 21125,  34, { 20765,  20665,  20764,  20763}},
 { 21126,  34, { 20736,  20764,  20665,  20664}},
 { 21127,  34, { 20739,  20759,  20764,  20736}},
 { 21128,  34, { 20763,  20764,  20759,  20762}},
 { 21129,  34, { 20416,  20759,  20739,  20739}},
 { 21130,  34, { 20762,  20759,  20416,  20437}},
 { 21131,  34, { 20430,  20408,  20752,  20758}},
 { 21132,  34, { 20710,  20757,  20758,  20752}},
 { 21133,  34, { 20431,  20430,  20758,  20756}},
 { 21134,  34, { 20757,  20755,  20756,  20758}},
 { 21135,  34, { 20431,  20756,  20761,  20426}},
 { 21136,  34, { 20769,  20761,  20756,  20755}},
 { 21137,  34, { 20426,  20761,  20760,  20427}},
 { 21138,  34, { 20768,  20760,  20761,  20769}},
 { 21139,  34, { 20768,  20765,  20763,  20760}},
 { 21140,  34, { 20760,  20763,  20762,  20762}},
 { 21141,  34, { 20760,  20762,  20437,  20427}},
 { 21142,  34, { 20661,  20728,  20659,  20658}},
 { 21143,  34, { 20656,  20659,  20728,  20642}},
 { 21144,  34, { 20642,  20728,  20730,  20641}},
 { 21145,  34, { 20731,  20730,  20728,  20661}},
 { 21146,  34, { 20730,  20740,  20664,  20641}},
 { 21147,  34, { 20731,  20741,  20740,  20730}},
 { 21148,  34, { 20742,  20751,  20741,  20731}},
 { 21149,  34, { 20740,  20735,  20736,  20664}},
 { 21150,  34, { 20741,  20738,  20735,  20740}},
 { 21151,  34, { 20751,  20737,  20738,  20741}},
 { 21152,  34, { 20655,  20656,  20642,  20643}},
 { 21153,  34, { 20697,  20700,  20695,  20696}},
 { 21154,  34, { 20661,  20726,  20754,  20660}},
 { 21155,  34, { 20737,  20751,  20750,  20750}},
 { 21156,  35, { 20770,  20771,  20772,  20773}},
 { 21157,  35, { 20771,  20774,  20775,  20772}},
 { 21158,  35, { 20774,  20776,  20777,  20775}},
 { 21159,  35, { 20778,  20779,  20780,  20781}},
 { 21160,  35, { 20779,  20782,  20783,  20780}},
 { 21161,  35, { 20784,  20785,  20786,  20787}},
 { 21162,  35, { 20785,  20788,  20789,  20786}},
 { 21163,  35, { 20788,  20790,  20791,  20789}},
 { 21164,  35, { 20782,  20792,  20793,  20783}},
 { 21165,  35, { 20792,  20790,  20791,  20793}},
 { 21166,  35, { 20770,  20794,  20795,  20773}},
 { 21167,  35, { 20794,  20796,  20797,  20795}},
 { 21168,  35, { 20796,  20798,  20799,  20797}},
 { 21169,  35, { 20798,  20778,  20781,  20799}},
 { 21170,  35, { 20800,  20801,  20802,  20803}},
 { 21171,  35, { 20801,  20804,  20805,  20802}},
 { 21172,  35, { 20804,  20806,  20807,  20805}},
 { 21173,  35, { 20806,  20808,  20809,  20807}},
 { 21174,  35, { 20808,  20810,  20811,  20809}},
 { 21175,  35, { 20803,  20802,  20812,  20813}},
 { 21176,  35, { 20802,  20805,  20814,  20812}},
 { 21177,  35, { 20805,  20807,  20815,  20814}},
 { 21178,  35, { 20807,  20809,  20816,  20815}},
 { 21179,  35, { 20809,  20811,  20817,  20816}},
 { 21180,  35, { 20813,  20812,  20818,  20819}},
 { 21181,  35, { 20812,  20814,  20820,  20818}},
 { 21182,  35, { 20814,  20815,  20821,  20820}},
 { 21183,  35, { 20815,  20816,  20822,  20821}},
 { 21184,  35, { 20816,  20817,  20823,  20822}},
 { 21185,  35, { 20819,  20818,  20824,  20825}},
 { 21186,  35, { 20818,  20820,  20826,  20824}},
 { 21187,  35, { 20820,  20821,  20827,  20826}},
 { 21188,  35, { 20821,  20822,  20828,  20827}},
 { 21189,  35, { 20822,  20823,  20829,  20828}},
 { 21190,  35, { 20825,  20824,  20830,  20831}},
 { 21191,  35, { 20824,  20826,  20832,  20830}},
 { 21192,  35, { 20826,  20827,  20833,  20832}},
 { 21193,  35, { 20827,  20828,  20834,  20833}},
 { 21194,  35, { 20828,  20829,  20835,  20834}},
 { 21195,  35, { 20836,  20837,  20838,  20839}},
 { 21196,  35, { 20837,  20840,  20841,  20838}},
 { 21197,  35, { 20840,  20842,  20843,  20841}},
 { 21198,  35, { 20839,  20838,  20844,  20845}},
 { 21199,  35, { 20838,  20841,  20846,  20844}},
 { 21200,  35, { 20841,  20843,  20847,  20846}},
 { 21201,  35, { 20845,  20844,  20848,  20849}},
 { 21202,  35, { 20844,  20846,  20850,  20848}},
 { 21203,  35, { 20846,  20847,  20851,  20850}},
 { 21204,  35, { 20847,  20852,  20853,  20851}},
 { 21205,  35, { 20852,  20854,  20855,  20853}},
 { 21206,  35, { 20849,  20848,  20856,  20857}},
 { 21207,  35, { 20848,  20850,  20858,  20856}},
 { 21208,  35, { 20850,  20851,  20859,  20858}},
 { 21209,  35, { 20851,  20853,  20860,  20859}},
 { 21210,  35, { 20853,  20855,  20861,  20860}},
 { 21211,  35, { 20857,  20856,  20862,  20863}},
 { 21212,  35, { 20856,  20858,  20864,  20862}},
 { 21213,  35, { 20858,  20859,  20865,  20864}},
 { 21214,  35, { 20859,  20860,  20866,  20865}},
 { 21215,  35, { 20860,  20861,  20867,  20866}},
 { 21216,  35, { 20868,  20869,  20870,  20871}},
 { 21217,  35, { 20869,  20872,  20873,  20870}},
 { 21218,  35, { 20872,  20874,  20875,  20873}},
 { 21219,  35, { 20871,  20870,  20876,  20877}},
 { 21220,  35, { 20870,  20873,  20878,  20876}},
 { 21221,  35, { 20873,  20875,  20879,  20878}},
 { 21222,  35, { 20877,  20876,  20880,  20881}},
 { 21223,  35, { 20876,  20878,  20882,  20880}},
 { 21224,  35, { 20878,  20879,  20883,  20882}},
 { 21225,  35, { 20881,  20880,  20884,  20885}},
 { 21226,  35, { 20880,  20882,  20886,  20884}},
 { 21227,  35, { 20882,  20883,  20887,  20886}},
 { 21228,  35, { 20885,  20884,  20888,  20889}},
 { 21229,  35, { 20884,  20886,  20890,  20888}},
 { 21230,  35, { 20886,  20887,  20891,  20890}},
 { 21231,  35, { 20874,  20892,  20893,  20872}},
 { 21232,  35, { 20892,  20894,  20895,  20893}},
 { 21233,  35, { 20894,  20896,  20897,  20895}},
 { 21234,  35, { 20896,  20898,  20899,  20897}},
 { 21235,  35, { 20872,  20893,  20900,  20869}},
 { 21236,  35, { 20893,  20895,  20901,  20900}},
 { 21237,  35, { 20895,  20897,  20902,  20901}},
 { 21238,  35, { 20897,  20899,  20903,  20902}},
 { 21239,  35, { 20869,  20900,  20904,  20868}},
 { 21240,  35, { 20900,  20901,  20905,  20904}},
 { 21241,  35, { 20901,  20902,  20906,  20905}},
 { 21242,  35, { 20902,  20903,  20907,  20906}},
 { 21243,  35, { 20907,  20908,  20909,  20906}},
 { 21244,  35, { 20908,  20910,  20911,  20909}},
 { 21245,  35, { 20910,  20912,  20913,  20911}},
 { 21246,  35, { 20912,  20914,  20915,  20913}},
 { 21247,  35, { 20906,  20909,  20916,  20905}},
 { 21248,  35, { 20909,  20911,  20917,  20916}},
 { 21249,  35, { 20911,  20913,  20918,  20917}},
 { 21250,  35, { 20913,  20915,  20919,  20918}},
 { 21251,  35, { 20920,  20919,  20918,  20921}},
 { 21252,  35, { 20921,  20918,  20917,  20922}},
 { 21253,  35, { 20922,  20917,  20916,  20923}},
 { 21254,  35, { 20923,  20916,  20924,  20924}},
 { 21255,  35, { 20916,  20924,  20904,  20905}},
 { 21256,  35, { 20904,  20868,  20925,  20924}},
 { 21257,  35, { 20899,  20926,  20927,  20898}},
 { 21258,  35, { 20914,  20927,  20926,  20912}},
 { 21259,  35, { 20903,  20928,  20926,  20899}},
 { 21260,  35, { 20912,  20926,  20928,  20910}},
 { 21261,  35, { 20907,  20908,  20928,  20903}},
 { 21262,  35, { 20910,  20908,  20928,  20928}},
 { 21263,  35, { 20929,  20930,  20931,  20932}},
 { 21264,  35, { 20930,  20933,  20934,  20931}},
 { 21265,  35, { 20933,  20935,  20936,  20934}},
 { 21266,  35, { 20935,  20937,  20938,  20936}},
 { 21267,  35, { 20932,  20931,  20939,  20940}},
 { 21268,  35, { 20931,  20934,  20941,  20939}},
 { 21269,  35, { 20934,  20936,  20942,  20941}},
 { 21270,  35, { 20936,  20938,  20943,  20942}},
 { 21271,  35, { 20940,  20939,  20944,  20945}},
 { 21272,  35, { 20939,  20941,  20946,  20944}},
 { 21273,  35, { 20941,  20942,  20947,  20946}},
 { 21274,  35, { 20942,  20943,  20948,  20947}},
 { 21275,  35, { 20945,  20944,  20949,  20950}},
 { 21276,  35, { 20944,  20946,  20951,  20949}},
 { 21277,  35, { 20946,  20947,  20952,  20951}},
 { 21278,  35, { 20947,  20948,  20953,  20952}},
 { 21279,  35, { 20950,  20954,  20955,  20945}},
 { 21280,  35, { 20954,  20956,  20957,  20955}},
 { 21281,  35, { 20956,  20958,  20959,  20957}},
 { 21282,  35, { 20958,  20960,  20961,  20959}},
 { 21283,  35, { 20945,  20955,  20962,  20963}},
 { 21284,  35, { 20955,  20957,  20964,  20962}},
 { 21285,  35, { 20957,  20959,  20965,  20964}},
 { 21286,  35, { 20959,  20961,  20966,  20965}},
 { 21287,  35, { 20965,  20967,  20968,  20964}},
 { 21288,  35, { 20967,  20969,  20970,  20968}},
 { 21289,  35, { 20969,  20971,  20972,  20970}},
 { 21290,  35, { 20964,  20968,  20973,  20962}},
 { 21291,  35, { 20968,  20970,  20974,  20973}},
 { 21292,  35, { 20970,  20972,  20975,  20974}},
 { 21293,  35, { 20962,  20973,  20976,  20963}},
 { 21294,  35, { 20973,  20974,  20977,  20976}},
 { 21295,  35, { 20974,  20975,  20978,  20977}},
 { 21296,  35, { 20979,  20980,  20981,  20982}},
 { 21297,  35, { 20980,  20983,  20984,  20981}},
 { 21298,  35, { 20983,  20985,  20986,  20984}},
 { 21299,  35, { 20985,  20929,  20932,  20986}},
 { 21300,  35, { 20982,  20981,  20987,  20988}},
 { 21301,  35, { 20981,  20984,  20989,  20987}},
 { 21302,  35, { 20984,  20986,  20990,  20989}},
 { 21303,  35, { 20986,  20932,  20940,  20990}},
 { 21304,  35, { 20988,  20987,  20977,  20978}},
 { 21305,  35, { 20987,  20989,  20976,  20977}},
 { 21306,  35, { 20989,  20990,  20963,  20976}},
 { 21307,  35, { 20990,  20940,  20945,  20963}},
 { 21308,  35, { 20991,  20992,  20993,  20994}},
 { 21309,  35, { 20995,  20996,  20997,  20998}},
 { 21310,  35, { 20998,  20997,  20999,  21000}},
 { 21311,  35, { 21000,  20999,  21001,  21002}},
 { 21312,  35, { 20994,  20993,  20966,  21003}},
 { 21313,  35, { 21003,  20966,  20961,  21004}},
 { 21314,  35, { 21005,  20995,  20998,  21006}},
 { 21315,  35, { 21006,  20998,  21000,  21007}},
 { 21316,  35, { 21007,  21000,  21002,  21008}},
 { 21317,  35, { 21004,  20961,  20960,  21009}},
 { 21318,  35, { 20991,  21001,  21002,  20994}},
 { 21319,  35, { 20994,  21002,  21003,  21003}},
 { 21320,  35, { 21002,  21003,  21004,  21004}},
 { 21321,  35, { 21004,  21009,  21008,  21002}},
 { 21322,  35, { 20971,  20969,  20993,  20993}},
 { 21323,  35, { 20967,  20966,  20965,  20965}},
 { 21324,  35, { 20966,  20967,  20969,  20993}},
 { 21325,  35, { 21010,  21011,  21012,  21013}},
 { 21326,  35, { 21011,  21014,  21015,  21012}},
 { 21327,  35, { 21014,  21016,  21017,  21015}},
 { 21328,  35, { 21016,  21018,  21019,  21017}},
 { 21329,  35, { 21018,  21020,  21021,  21019}},
 { 21330,  35, { 21013,  21012,  21022,  21023}},
 { 21331,  35, { 21012,  21015,  21024,  21022}},
 { 21332,  35, { 21015,  21017,  21025,  21024}},
 { 21333,  35, { 21017,  21019,  21026,  21025}},
 { 21334,  35, { 21019,  21021,  21027,  21026}},
 { 21335,  35, { 21023,  21022,  20997,  20996}},
 { 21336,  35, { 21022,  21024,  20999,  20997}},
 { 21337,  35, { 21024,  21025,  21001,  20999}},
 { 21338,  35, { 21025,  21026,  20991,  21001}},
 { 21339,  35, { 21026,  21027,  20992,  20991}},
 { 21340,  35, { 21028,  21029,  21030,  21031}},
 { 21341,  35, { 21031,  21030,  21032,  21033}},
 { 21342,  35, { 21034,  21035,  21011,  21010}},
 { 21343,  35, { 21011,  21014,  21036,  21035}},
 { 21344,  35, { 21014,  21016,  21037,  21036}},
 { 21345,  35, { 21016,  21018,  21038,  21037}},
 { 21346,  35, { 21018,  21020,  21039,  21038}},
 { 21347,  35, { 21036,  21037,  21040,  21041}},
 { 21348,  35, { 21037,  21038,  21042,  21040}},
 { 21349,  35, { 21038,  21039,  21043,  21042}},
 { 21350,  35, { 21032,  21041,  21044,  21030}},
 { 21351,  35, { 21041,  21040,  21045,  21044}},
 { 21352,  35, { 21040,  21042,  21046,  21045}},
 { 21353,  35, { 21042,  21043,  21047,  21046}},
 { 21354,  35, { 21030,  21044,  21048,  21029}},
 { 21355,  35, { 21044,  21045,  21049,  21048}},
 { 21356,  35, { 21045,  21046,  21050,  21049}},
 { 21357,  35, { 21046,  21047,  21051,  21050}},
 { 21358,  35, { 21033,  21052,  21053,  21031}},
 { 21359,  35, { 21031,  21053,  21054,  21028}},
 { 21360,  35, { 21034,  21055,  21010,  21010}},
 { 21361,  35, { 21034,  21056,  21055,  21055}},
 { 21362,  35, { 21005,  20995,  21057,  21058}},
 { 21363,  35, { 21059,  21057,  20995,  20996}},
 { 21364,  35, { 21060,  21061,  21023,  21013}},
 { 21365,  35, { 20996,  21023,  21061,  21059}},
 { 21366,  35, { 21013,  21010,  21062,  21060}},
 { 21367,  35, { 21063,  21062,  21010,  21055}},
 { 21368,  35, { 21060,  21062,  20837,  20837}},
 { 21369,  35, { 21062,  20837,  20836,  21063}},
 { 21370,  35, { 21060,  20837,  21061,  21061}},
 { 21371,  35, { 21061,  20837,  20840,  21059}},
 { 21372,  35, { 21059,  20840,  21057,  21057}},
 { 21373,  35, { 21057,  20840,  20842,  21058}},
 { 21374,  35, { 20937,  21064,  21065,  20935}},
 { 21375,  35, { 21064,  21066,  21067,  21065}},
 { 21376,  35, { 20935,  21065,  21068,  20933}},
 { 21377,  35, { 21065,  21067,  21069,  21068}},
 { 21378,  35, { 20933,  21068,  21070,  20930}},
 { 21379,  35, { 21071,  21070,  21068,  21069}},
 { 21380,  35, { 20930,  21070,  21072,  20929}},
 { 21381,  35, { 21070,  21071,  21073,  21072}},
 { 21382,  35, { 20983,  20985,  21074,  21075}},
 { 21383,  35, { 21072,  21074,  20985,  20929}},
 { 21384,  35, { 21076,  21075,  21074,  21077}},
 { 21385,  35, { 21073,  21072,  21074,  21077}},
 { 21386,  35, { 21078,  20979,  21079,  21080}},
 { 21387,  35, { 21076,  21075,  21079,  21080}},
 { 21388,  35, { 21075,  21079,  20980,  20983}},
 { 21389,  35, { 20979,  20980,  21079,  21079}},
 { 21390,  35, { 20992,  21027,  21081,  21082}},
 { 21391,  35, { 21027,  21021,  21083,  21081}},
 { 21392,  35, { 21021,  21020,  21039,  21083}},
 { 21393,  35, { 21082,  21081,  21084,  21085}},
 { 21394,  35, { 21081,  21083,  21086,  21084}},
 { 21395,  35, { 21083,  21039,  21043,  21086}},
 { 21396,  35, { 21085,  21084,  21087,  21088}},
 { 21397,  35, { 21084,  21086,  21089,  21087}},
 { 21398,  35, { 21086,  21043,  21047,  21089}},
 { 21399,  35, { 21088,  21087,  21090,  21091}},
 { 21400,  35, { 21087,  21089,  21092,  21090}},
 { 21401,  35, { 21089,  21047,  21051,  21092}},
 { 21402,  35, { 21093,  21094,  21095,  21096}},
 { 21403,  35, { 21094,  21091,  21088,  21095}},
 { 21404,  35, { 21096,  21095,  21097,  21098}},
 { 21405,  35, { 21095,  21088,  21085,  21097}},
 { 21406,  35, { 21098,  21097,  21099,  21100}},
 { 21407,  35, { 21097,  21085,  21082,  21099}},
 { 21408,  35, { 21100,  21099,  21101,  21102}},
 { 21409,  35, { 21099,  21082,  20992,  21101}},
 { 21410,  35, { 21102,  21103,  21100,  21100}},
 { 21411,  35, { 21103,  21100,  21098,  21098}},
 { 21412,  35, { 21103,  21098,  21104,  21105}},
 { 21413,  35, { 21098,  21096,  21106,  21104}},
 { 21414,  35, { 21106,  21096,  21093,  21107}},
 { 21415,  35, { 21108,  21109,  21107,  21106}},
 { 21416,  35, { 21108,  21106,  21104,  21110}},
 { 21417,  35, { 21110,  21111,  21112,  21108}},
 { 21418,  35, { 21111,  21110,  21104,  21105}},
 { 21419,  35, { 21105,  21113,  21111,  21111}},
 { 21420,  35, { 21111,  21114,  21115,  21112}},
 { 21421,  35, { 21115,  21114,  21116,  21117}},
 { 21422,  35, { 21109,  21118,  21108,  21108}},
 { 21423,  35, { 21118,  21108,  21112,  21112}},
 { 21424,  35, { 21118,  21112,  21115,  21119}},
 { 21425,  35, { 21119,  21115,  21117,  21120}},
 { 21426,  35, { 21120,  21121,  21119,  21119}},
 { 21427,  35, { 21119,  21121,  21122,  21118}},
 { 21428,  35, { 21113,  21123,  21124,  21125}},
 { 21429,  35, { 21124,  21126,  21127,  21125}},
 { 21430,  35, { 21126,  21128,  21078,  21127}},
 { 21431,  35, { 21123,  21129,  21102,  21103}},
 { 21432,  35, { 21123,  21124,  21130,  21129}},
 { 21433,  35, { 21130,  21131,  21124,  21124}},
 { 21434,  35, { 21126,  21124,  21131,  21131}},
 { 21435,  35, { 20776,  20784,  20787,  20777}},
 { 21436,  35, { 21117,  21132,  21120,  21120}},
 { 21437,  35, { 21117,  21116,  21133,  21132}},
 { 21438,  35, { 21116,  21133,  21078,  21127}},
 { 21439,  35, { 21114,  21125,  21127,  21116}},
 { 21440,  35, { 21114,  21111,  21113,  21125}},
 { 21441,  35, { 20975,  21130,  21131,  20978}},
 { 21442,  35, { 21130,  21129,  20972,  20975}},
 { 21443,  35, { 21129,  21102,  20971,  20972}},
 { 21444,  35, { 21102,  21101,  20993,  20971}},
 { 21445,  35, { 21101,  20992,  20993,  20993}},
 { 21446,  35, { 21131,  21126,  20988,  20978}},
 { 21447,  35, { 21126,  20988,  20982,  21128}},
 { 21448,  35, { 20982,  21128,  21078,  20979}},
 { 21449,  35, { 20952,  20951,  20785,  20784}},
 { 21450,  35, { 20785,  20951,  20949,  20949}},
 { 21451,  35, { 20949,  20785,  20788,  20950}},
 { 21452,  35, { 20788,  20954,  20950,  20950}},
 { 21453,  35, { 20788,  20790,  20956,  20954}},
 { 21454,  35, { 20790,  20958,  20956,  20956}},
 { 21455,  35, { 20790,  20792,  20960,  20958}},
 { 21456,  35, { 20792,  20782,  21009,  20960}},
 { 21457,  35, { 20782,  20779,  21008,  21009}},
 { 21458,  35, { 20779,  20778,  21007,  21008}},
 { 21459,  35, { 20778,  21006,  21007,  21007}},
 { 21460,  35, { 21006,  20778,  20798,  21005}},
 { 21461,  35, { 21005,  20798,  20796,  21058}},
 { 21462,  35, { 20794,  21134,  21058,  20796}},
 { 21463,  35, { 21134,  21135,  20770,  20794}},
 { 21464,  35, { 21135,  21136,  20771,  20770}},
 { 21465,  35, { 21136,  21137,  20774,  20771}},
 { 21466,  35, { 20774,  20776,  21138,  21137}},
 { 21467,  35, { 20953,  21138,  20776,  20776}},
 { 21468,  35, { 20776,  20784,  20952,  20953}},
 { 21469,  35, { 21058,  20842,  21134,  21134}},
 { 21470,  35, { 21135,  21136,  21139,  21139}},
 { 21471,  35, { 20953,  21140,  21138,  21138}},
 { 21472,  35, { 21140,  21141,  21137,  21138}},
 { 21473,  35, { 21142,  21143,  21144,  21145}},
 { 21474,  35, { 21139,  21144,  21143,  21146}},
 { 21475,  35, { 21144,  21145,  21141,  21137}},
 { 21476,  35, { 21144,  21139,  21136,  21137}},
 { 21477,  35, { 20842,  21147,  21148,  20843}},
 { 21478,  35, { 21146,  21148,  21147,  21139}},
 { 21479,  35, { 21139,  21147,  21135,  21135}},
 { 21480,  35, { 21147,  20842,  21134,  21135}},
 { 21481,  35, { 21142,  21143,  20854,  20854}},
 { 21482,  35, { 21143,  20854,  20852,  21146}},
 { 21483,  35, { 21146,  20852,  21148,  21148}},
 { 21484,  35, { 21148,  20852,  20847,  20843}},
 { 21485,  35, { 20810,  20808,  20868,  20925}},
 { 21486,  35, { 20808,  20806,  20871,  20868}},
 { 21487,  35, { 20806,  20804,  20877,  20871}},
 { 21488,  35, { 20804,  20881,  20877,  20877}},
 { 21489,  35, { 20881,  20885,  20801,  20804}},
 { 21490,  35, { 20885,  20889,  20800,  20801}},
 { 21491,  35, { 21123,  21113,  21105,  21103}},
 { 21492,  35, { 20829,  20835,  20866,  20867}},
 { 21493,  35, { 20866,  20835,  20834,  20865}},
 { 21494,  35, { 20865,  20834,  20833,  20864}},
 { 21495,  35, { 20864,  20833,  20832,  20862}},
 { 21496,  35, { 20862,  20832,  20830,  20863}},
 { 21497,  35, { 20830,  20831,  20857,  20863}},
 { 21498,  35, { 20831,  20825,  20849,  20857}},
 { 21499,  35, { 20825,  20819,  20845,  20849}},
 { 21500,  35, { 20819,  20813,  20839,  20845}},
 { 21501,  35, { 20813,  20803,  20836,  20839}},
 { 21502,  35, { 20803,  20800,  21063,  20836}},
 { 21503,  35, { 21063,  20800,  20889,  21055}},
 { 21504,  35, { 20889,  20888,  21056,  21055}},
 { 21505,  35, { 20888,  20890,  21052,  21056}},
 { 21506,  35, { 20890,  20891,  21053,  21052}},
 { 21507,  35, { 20891,  21054,  21053,  21053}},
 { 21508,  35, { 20829,  20823,  20867,  20867}},
 { 21509,  35, { 20823,  20817,  20861,  20867}},
 { 21510,  35, { 20817,  20855,  20861,  20861}},
 { 21511,  35, { 20817,  20811,  20854,  20855}},
 { 21512,  35, { 20811,  21142,  20854,  20854}},
 { 21513,  35, { 20811,  20810,  21145,  21142}},
 { 21514,  35, { 21145,  21141,  20925,  20810}},
 { 21515,  35, { 21141,  21140,  20924,  20925}},
 { 21516,  35, { 21140,  20924,  20953,  20953}},
 { 21517,  35, { 20924,  20953,  20948,  20923}},
 { 21518,  35, { 20923,  20948,  20943,  20922}},
 { 21519,  35, { 20922,  20943,  20938,  20938}},
 { 21520,  35, { 20938,  20937,  20921,  20922}},
 { 21521,  35, { 20937,  21064,  20921,  20921}},
 { 21522,  35, { 20921,  20920,  21066,  21064}},
 { 21523,  35, { 21149,  21150,  21151,  21152}},
 { 21524,  35, { 21152,  21151,  21153,  21154}},
 { 21525,  35, { 21154,  21153,  21155,  21156}},
 { 21526,  35, { 21157,  21158,  21159,  21160}},
 { 21527,  35, { 21160,  21159,  21161,  21162}},
 { 21528,  35, { 21163,  21164,  21165,  21166}},
 { 21529,  35, { 21166,  21165,  21167,  21168}},
 { 21530,  35, { 21168,  21167,  21169,  21170}},
 { 21531,  35, { 21162,  21161,  21171,  21172}},
 { 21532,  35, { 21172,  21171,  21169,  21170}},
 { 21533,  35, { 21149,  21150,  21173,  21174}},
 { 21534,  35, { 21174,  21173,  21175,  21176}},
 { 21535,  35, { 21176,  21175,  21177,  21178}},
 { 21536,  35, { 21178,  21177,  21158,  21157}},
 { 21537,  35, { 21179,  21180,  21181,  21182}},
 { 21538,  35, { 21182,  21181,  21183,  21184}},
 { 21539,  35, { 21184,  21183,  21185,  21186}},
 { 21540,  35, { 21186,  21185,  21187,  21188}},
 { 21541,  35, { 21188,  21187,  21189,  21190}},
 { 21542,  35, { 21180,  21191,  21192,  21181}},
 { 21543,  35, { 21181,  21192,  21193,  21183}},
 { 21544,  35, { 21183,  21193,  21194,  21185}},
 { 21545,  35, { 21185,  21194,  21195,  21187}},
 { 21546,  35, { 21187,  21195,  21196,  21189}},
 { 21547,  35, { 21191,  21197,  21198,  21192}},
 { 21548,  35, { 21192,  21198,  21199,  21193}},
 { 21549,  35, { 21193,  21199,  21200,  21194}},
 { 21550,  35, { 21194,  21200,  21201,  21195}},
 { 21551,  35, { 21195,  21201,  21202,  21196}},
 { 21552,  35, { 21197,  21203,  21204,  21198}},
 { 21553,  35, { 21198,  21204,  21205,  21199}},
 { 21554,  35, { 21199,  21205,  21206,  21200}},
 { 21555,  35, { 21200,  21206,  21207,  21201}},
 { 21556,  35, { 21201,  21207,  21208,  21202}},
 { 21557,  35, { 21203,  21209,  21210,  21204}},
 { 21558,  35, { 21204,  21210,  21211,  21205}},
 { 21559,  35, { 21205,  21211,  21212,  21206}},
 { 21560,  35, { 21206,  21212,  21213,  21207}},
 { 21561,  35, { 21207,  21213,  21214,  21208}},
 { 21562,  35, { 21215,  21216,  21217,  21218}},
 { 21563,  35, { 21218,  21217,  21219,  21220}},
 { 21564,  35, { 21220,  21219,  21221,  21222}},
 { 21565,  35, { 21216,  21223,  21224,  21217}},
 { 21566,  35, { 21217,  21224,  21225,  21219}},
 { 21567,  35, { 21219,  21225,  21226,  21221}},
 { 21568,  35, { 21223,  21227,  21228,  21224}},
 { 21569,  35, { 21224,  21228,  21229,  21225}},
 { 21570,  35, { 21225,  21229,  21230,  21226}},
 { 21571,  35, { 21226,  21230,  21231,  21232}},
 { 21572,  35, { 21232,  21231,  21233,  21234}},
 { 21573,  35, { 21227,  21235,  21236,  21228}},
 { 21574,  35, { 21228,  21236,  21237,  21229}},
 { 21575,  35, { 21229,  21237,  21238,  21230}},
 { 21576,  35, { 21230,  21238,  21239,  21231}},
 { 21577,  35, { 21231,  21239,  21240,  21233}},
 { 21578,  35, { 21235,  21241,  21242,  21236}},
 { 21579,  35, { 21236,  21242,  21243,  21237}},
 { 21580,  35, { 21237,  21243,  21244,  21238}},
 { 21581,  35, { 21238,  21244,  21245,  21239}},
 { 21582,  35, { 21239,  21245,  21246,  21240}},
 { 21583,  35, { 21247,  21248,  21249,  21250}},
 { 21584,  35, { 21250,  21249,  21251,  21252}},
 { 21585,  35, { 21252,  21251,  20875,  20874}},
 { 21586,  35, { 21248,  21253,  21254,  21249}},
 { 21587,  35, { 21249,  21254,  21255,  21251}},
 { 21588,  35, { 21251,  21255,  20879,  20875}},
 { 21589,  35, { 21253,  21256,  21257,  21254}},
 { 21590,  35, { 21254,  21257,  21258,  21255}},
 { 21591,  35, { 21255,  21258,  20883,  20879}},
 { 21592,  35, { 21256,  21259,  21260,  21257}},
 { 21593,  35, { 21257,  21260,  21261,  21258}},
 { 21594,  35, { 21258,  21261,  20887,  20883}},
 { 21595,  35, { 21259,  21262,  21263,  21260}},
 { 21596,  35, { 21260,  21263,  21264,  21261}},
 { 21597,  35, { 21261,  21264,  20891,  20887}},
 { 21598,  35, { 20874,  21252,  21265,  20892}},
 { 21599,  35, { 20892,  21265,  21266,  20894}},
 { 21600,  35, { 20894,  21266,  21267,  20896}},
 { 21601,  35, { 20896,  21267,  21268,  20898}},
 { 21602,  35, { 21252,  21250,  21269,  21265}},
 { 21603,  35, { 21265,  21269,  21270,  21266}},
 { 21604,  35, { 21266,  21270,  21271,  21267}},
 { 21605,  35, { 21267,  21271,  21272,  21268}},
 { 21606,  35, { 21250,  21247,  21273,  21269}},
 { 21607,  35, { 21269,  21273,  21274,  21270}},
 { 21608,  35, { 21270,  21274,  21275,  21271}},
 { 21609,  35, { 21271,  21275,  21276,  21272}},
 { 21610,  35, { 21276,  21275,  21277,  21278}},
 { 21611,  35, { 21278,  21277,  21279,  21280}},
 { 21612,  35, { 21280,  21279,  21281,  21282}},
 { 21613,  35, { 21282,  21281,  20915,  20914}},
 { 21614,  35, { 21275,  21274,  21283,  21277}},
 { 21615,  35, { 21277,  21283,  21284,  21279}},
 { 21616,  35, { 21279,  21284,  21285,  21281}},
 { 21617,  35, { 21281,  21285,  20919,  20915}},
 { 21618,  35, { 20920,  21286,  21285,  20919}},
 { 21619,  35, { 21286,  21287,  21284,  21285}},
 { 21620,  35, { 21287,  21288,  21283,  21284}},
 { 21621,  35, { 21288,  21289,  21283,  21283}},
 { 21622,  35, { 21283,  21274,  21273,  21289}},
 { 21623,  35, { 21273,  21289,  21290,  21247}},
 { 21624,  35, { 21268,  20898,  20927,  21291}},
 { 21625,  35, { 20914,  21282,  21291,  20927}},
 { 21626,  35, { 21272,  21268,  21291,  21292}},
 { 21627,  35, { 21282,  21280,  21292,  21291}},
 { 21628,  35, { 21276,  21272,  21292,  21278}},
 { 21629,  35, { 21280,  21292,  21278,  21278}},
 { 21630,  35, { 21293,  21294,  21295,  21296}},
 { 21631,  35, { 21296,  21295,  21297,  21298}},
 { 21632,  35, { 21298,  21297,  21299,  21300}},
 { 21633,  35, { 21300,  21299,  21301,  21302}},
 { 21634,  35, { 21294,  21303,  21304,  21295}},
 { 21635,  35, { 21295,  21304,  21305,  21297}},
 { 21636,  35, { 21297,  21305,  21306,  21299}},
 { 21637,  35, { 21299,  21306,  21307,  21301}},
 { 21638,  35, { 21303,  21308,  21309,  21304}},
 { 21639,  35, { 21304,  21309,  21310,  21305}},
 { 21640,  35, { 21305,  21310,  21311,  21306}},
 { 21641,  35, { 21306,  21311,  21312,  21307}},
 { 21642,  35, { 21308,  21313,  21314,  21309}},
 { 21643,  35, { 21309,  21314,  21315,  21310}},
 { 21644,  35, { 21310,  21315,  21316,  21311}},
 { 21645,  35, { 21311,  21316,  21317,  21312}},
 { 21646,  35, { 21313,  21308,  21318,  21319}},
 { 21647,  35, { 21319,  21318,  21320,  21321}},
 { 21648,  35, { 21321,  21320,  21322,  21323}},
 { 21649,  35, { 21323,  21322,  21324,  21325}},
 { 21650,  35, { 21308,  21326,  21327,  21318}},
 { 21651,  35, { 21318,  21327,  21328,  21320}},
 { 21652,  35, { 21320,  21328,  21329,  21322}},
 { 21653,  35, { 21322,  21329,  21330,  21324}},
 { 21654,  35, { 21329,  21328,  21331,  21332}},
 { 21655,  35, { 21332,  21331,  21333,  21334}},
 { 21656,  35, { 21334,  21333,  21335,  21336}},
 { 21657,  35, { 21328,  21327,  21337,  21331}},
 { 21658,  35, { 21331,  21337,  21338,  21333}},
 { 21659,  35, { 21333,  21338,  21339,  21335}},
 { 21660,  35, { 21327,  21326,  21340,  21337}},
 { 21661,  35, { 21337,  21340,  21341,  21338}},
 { 21662,  35, { 21338,  21341,  21342,  21339}},
 { 21663,  35, { 21343,  21344,  21345,  21346}},
 { 21664,  35, { 21346,  21345,  21347,  21348}},
 { 21665,  35, { 21348,  21347,  21349,  21350}},
 { 21666,  35, { 21350,  21349,  21294,  21293}},
 { 21667,  35, { 21344,  21351,  21352,  21345}},
 { 21668,  35, { 21345,  21352,  21353,  21347}},
 { 21669,  35, { 21347,  21353,  21354,  21349}},
 { 21670,  35, { 21349,  21354,  21303,  21294}},
 { 21671,  35, { 21351,  21342,  21341,  21352}},
 { 21672,  35, { 21352,  21341,  21340,  21353}},
 { 21673,  35, { 21353,  21340,  21326,  21354}},
 { 21674,  35, { 21354,  21326,  21308,  21303}},
 { 21675,  35, { 21355,  21356,  21357,  21358}},
 { 21676,  35, { 21359,  21360,  21361,  21362}},
 { 21677,  35, { 21360,  21363,  21364,  21361}},
 { 21678,  35, { 21363,  21365,  21366,  21364}},
 { 21679,  35, { 21356,  21367,  21330,  21357}},
 { 21680,  35, { 21367,  21368,  21324,  21330}},
 { 21681,  35, { 21369,  21370,  21360,  21359}},
 { 21682,  35, { 21370,  21371,  21363,  21360}},
 { 21683,  35, { 21371,  21372,  21365,  21363}},
 { 21684,  35, { 21368,  21373,  21325,  21324}},
 { 21685,  35, { 21355,  21356,  21365,  21366}},
 { 21686,  35, { 21356,  21367,  21365,  21365}},
 { 21687,  35, { 21365,  21368,  21367,  21367}},
 { 21688,  35, { 21368,  21365,  21372,  21373}},
 { 21689,  35, { 21336,  21357,  21334,  21334}},
 { 21690,  35, { 21332,  21329,  21330,  21330}},
 { 21691,  35, { 21330,  21357,  21334,  21332}},
 { 21692,  35, { 21374,  21375,  21376,  21377}},
 { 21693,  35, { 21377,  21376,  21378,  21379}},
 { 21694,  35, { 21379,  21378,  21380,  21381}},
 { 21695,  35, { 21381,  21380,  21382,  21383}},
 { 21696,  35, { 21383,  21382,  21384,  21385}},
 { 21697,  35, { 21375,  21386,  21387,  21376}},
 { 21698,  35, { 21376,  21387,  21388,  21378}},
 { 21699,  35, { 21378,  21388,  21389,  21380}},
 { 21700,  35, { 21380,  21389,  21390,  21382}},
 { 21701,  35, { 21382,  21390,  21391,  21384}},
 { 21702,  35, { 21386,  21362,  21361,  21387}},
 { 21703,  35, { 21387,  21361,  21364,  21388}},
 { 21704,  35, { 21388,  21364,  21366,  21389}},
 { 21705,  35, { 21389,  21366,  21355,  21390}},
 { 21706,  35, { 21390,  21355,  21358,  21391}},
 { 21707,  35, { 21028,  21392,  21393,  21029}},
 { 21708,  35, { 21392,  21394,  21395,  21393}},
 { 21709,  35, { 21396,  21374,  21377,  21397}},
 { 21710,  35, { 21377,  21397,  21398,  21379}},
 { 21711,  35, { 21379,  21398,  21399,  21381}},
 { 21712,  35, { 21381,  21399,  21400,  21383}},
 { 21713,  35, { 21383,  21400,  21401,  21385}},
 { 21714,  35, { 21397,  21395,  21402,  21398}},
 { 21715,  35, { 21398,  21402,  21403,  21399}},
 { 21716,  35, { 21399,  21403,  21404,  21400}},
 { 21717,  35, { 21400,  21404,  21405,  21401}},
 { 21718,  35, { 21395,  21393,  21406,  21402}},
 { 21719,  35, { 21402,  21406,  21407,  21403}},
 { 21720,  35, { 21403,  21407,  21408,  21404}},
 { 21721,  35, { 21404,  21408,  21409,  21405}},
 { 21722,  35, { 21393,  21029,  21048,  21406}},
 { 21723,  35, { 21406,  21048,  21049,  21407}},
 { 21724,  35, { 21407,  21049,  21050,  21408}},
 { 21725,  35, { 21408,  21050,  21051,  21409}},
 { 21726,  35, { 21396,  21394,  21410,  21411}},
 { 21727,  35, { 21394,  21392,  21412,  21410}},
 { 21728,  35, { 21392,  21028,  21054,  21412}},
 { 21729,  35, { 21396,  21374,  21413,  21413}},
 { 21730,  35, { 21396,  21413,  21411,  21411}},
 { 21731,  35, { 21369,  21414,  21415,  21359}},
 { 21732,  35, { 21416,  21362,  21359,  21415}},
 { 21733,  35, { 21417,  21375,  21386,  21418}},
 { 21734,  35, { 21362,  21416,  21418,  21386}},
 { 21735,  35, { 21375,  21417,  21419,  21374}},
 { 21736,  35, { 21420,  21413,  21374,  21419}},
 { 21737,  35, { 21417,  21218,  21419,  21419}},
 { 21738,  35, { 21419,  21420,  21215,  21218}},
 { 21739,  35, { 21417,  21418,  21218,  21218}},
 { 21740,  35, { 21418,  21416,  21220,  21218}},
 { 21741,  35, { 21416,  21415,  21220,  21220}},
 { 21742,  35, { 21415,  21414,  21222,  21220}},
 { 21743,  35, { 21302,  21300,  21421,  21422}},
 { 21744,  35, { 21422,  21421,  21067,  21066}},
 { 21745,  35, { 21300,  21298,  21423,  21421}},
 { 21746,  35, { 21421,  21423,  21069,  21067}},
 { 21747,  35, { 21298,  21296,  21424,  21423}},
 { 21748,  35, { 21071,  21069,  21423,  21424}},
 { 21749,  35, { 21296,  21293,  21425,  21424}},
 { 21750,  35, { 21424,  21425,  21073,  21071}},
 { 21751,  35, { 21348,  21426,  21427,  21350}},
 { 21752,  35, { 21425,  21293,  21350,  21427}},
 { 21753,  35, { 21076,  21077,  21427,  21426}},
 { 21754,  35, { 21073,  21077,  21427,  21425}},
 { 21755,  35, { 21078,  21080,  21428,  21343}},
 { 21756,  35, { 21076,  21080,  21428,  21426}},
 { 21757,  35, { 21426,  21348,  21346,  21428}},
 { 21758,  35, { 21343,  21428,  21346,  21346}},
 { 21759,  35, { 21358,  21429,  21430,  21391}},
 { 21760,  35, { 21391,  21430,  21431,  21384}},
 { 21761,  35, { 21384,  21431,  21401,  21385}},
 { 21762,  35, { 21429,  21432,  21433,  21430}},
 { 21763,  35, { 21430,  21433,  21434,  21431}},
 { 21764,  35, { 21431,  21434,  21405,  21401}},
 { 21765,  35, { 21432,  21435,  21436,  21433}},
 { 21766,  35, { 21433,  21436,  21437,  21434}},
 { 21767,  35, { 21434,  21437,  21409,  21405}},
 { 21768,  35, { 21435,  21091,  21090,  21436}},
 { 21769,  35, { 21436,  21090,  21092,  21437}},
 { 21770,  35, { 21437,  21092,  21051,  21409}},
 { 21771,  35, { 21093,  21438,  21439,  21094}},
 { 21772,  35, { 21094,  21439,  21435,  21091}},
 { 21773,  35, { 21438,  21440,  21441,  21439}},
 { 21774,  35, { 21439,  21441,  21432,  21435}},
 { 21775,  35, { 21440,  21442,  21443,  21441}},
 { 21776,  35, { 21441,  21443,  21429,  21432}},
 { 21777,  35, { 21442,  21444,  21445,  21443}},
 { 21778,  35, { 21443,  21445,  21358,  21429}},
 { 21779,  35, { 21444,  21442,  21446,  21446}},
 { 21780,  35, { 21446,  21440,  21442,  21442}},
 { 21781,  35, { 21446,  21447,  21448,  21440}},
 { 21782,  35, { 21440,  21448,  21107,  21438}},
 { 21783,  35, { 21107,  21093,  21438,  21438}},
 { 21784,  35, { 21109,  21449,  21448,  21107}},
 { 21785,  35, { 21449,  21109,  21450,  21451}},
 { 21786,  35, { 21451,  21447,  21448,  21449}},
 { 21787,  35, { 21447,  21451,  21452,  21452}},
 { 21788,  35, { 21451,  21450,  21453,  21454}},
 { 21789,  35, { 21453,  21120,  21455,  21454}},
 { 21790,  35, { 21456,  21450,  21109,  21109}},
 { 21791,  35, { 21452,  21457,  21458,  21459}},
 { 21792,  35, { 21458,  21457,  21460,  21461}},
 { 21793,  35, { 21461,  21460,  21078,  21462}},
 { 21794,  35, { 21459,  21446,  21444,  21463}},
 { 21795,  35, { 21459,  21463,  21464,  21458}},
 { 21796,  35, { 21464,  21458,  21465,  21465}},
 { 21797,  35, { 21461,  21465,  21458,  21458}},
 { 21798,  35, { 21156,  21155,  21164,  21163}},
 { 21799,  35, { 21120,  21132,  21133,  21455}},
 { 21800,  35, { 21455,  21460,  21078,  21133}},
 { 21801,  35, { 21454,  21455,  21460,  21457}},
 { 21802,  35, { 21454,  21457,  21452,  21451}},
 { 21803,  35, { 21339,  21342,  21465,  21464}},
 { 21804,  35, { 21464,  21339,  21335,  21463}},
 { 21805,  35, { 21463,  21335,  21336,  21444}},
 { 21806,  35, { 21444,  21336,  21357,  21445}},
 { 21807,  35, { 21445,  21357,  21358,  21358}},
 { 21808,  35, { 21465,  21342,  21351,  21461}},
 { 21809,  35, { 21461,  21462,  21344,  21351}},
 { 21810,  35, { 21344,  21343,  21078,  21462}},
 { 21811,  35, { 21316,  21163,  21166,  21315}},
 { 21812,  35, { 21166,  21314,  21315,  21315}},
 { 21813,  35, { 21314,  21313,  21168,  21166}},
 { 21814,  35, { 21168,  21313,  21319,  21319}},
 { 21815,  35, { 21168,  21319,  21321,  21170}},
 { 21816,  35, { 21170,  21321,  21323,  21323}},
 { 21817,  35, { 21170,  21323,  21325,  21172}},
 { 21818,  35, { 21172,  21325,  21373,  21162}},
 { 21819,  35, { 21162,  21373,  21372,  21160}},
 { 21820,  35, { 21160,  21372,  21371,  21157}},
 { 21821,  35, { 21157,  21371,  21370,  21370}},
 { 21822,  35, { 21370,  21369,  21178,  21157}},
 { 21823,  35, { 21369,  21414,  21176,  21178}},
 { 21824,  35, { 21174,  21176,  21414,  21466}},
 { 21825,  35, { 21466,  21174,  21149,  21467}},
 { 21826,  35, { 21467,  21149,  21152,  21468}},
 { 21827,  35, { 21468,  21152,  21154,  21469}},
 { 21828,  35, { 21154,  21469,  21470,  21156}},
 { 21829,  35, { 21317,  21156,  21470,  21470}},
 { 21830,  35, { 21156,  21317,  21316,  21163}},
 { 21831,  35, { 21414,  21466,  21222,  21222}},
 { 21832,  35, { 21467,  21471,  21468,  21468}},
 { 21833,  35, { 21317,  21470,  21472,  21472}},
 { 21834,  35, { 21472,  21470,  21469,  21473}},
 { 21835,  35, { 21474,  21475,  21476,  21477}},
 { 21836,  35, { 21471,  21478,  21477,  21476}},
 { 21837,  35, { 21476,  21469,  21473,  21475}},
 { 21838,  35, { 21476,  21469,  21468,  21471}},
 { 21839,  35, { 21222,  21221,  21479,  21480}},
 { 21840,  35, { 21478,  21471,  21480,  21479}},
 { 21841,  35, { 21471,  21467,  21480,  21480}},
 { 21842,  35, { 21480,  21467,  21466,  21222}},
 { 21843,  35, { 21474,  21234,  21477,  21477}},
 { 21844,  35, { 21477,  21478,  21232,  21234}},
 { 21845,  35, { 21478,  21479,  21232,  21232}},
 { 21846,  35, { 21479,  21221,  21226,  21232}},
 { 21847,  35, { 21190,  21290,  21247,  21188}},
 { 21848,  35, { 21188,  21247,  21248,  21186}},
 { 21849,  35, { 21186,  21248,  21253,  21184}},
 { 21850,  35, { 21184,  21253,  21256,  21256}},
 { 21851,  35, { 21256,  21184,  21182,  21259}},
 { 21852,  35, { 21259,  21182,  21179,  21262}},
 { 21853,  35, { 21459,  21446,  21447,  21452}},
 { 21854,  35, { 21208,  21246,  21245,  21214}},
 { 21855,  35, { 21245,  21244,  21213,  21214}},
 { 21856,  35, { 21244,  21243,  21212,  21213}},
 { 21857,  35, { 21243,  21242,  21211,  21212}},
 { 21858,  35, { 21242,  21241,  21210,  21211}},
 { 21859,  35, { 21210,  21241,  21235,  21209}},
 { 21860,  35, { 21209,  21235,  21227,  21203}},
 { 21861,  35, { 21203,  21227,  21223,  21197}},
 { 21862,  35, { 21197,  21223,  21216,  21191}},
 { 21863,  35, { 21191,  21216,  21215,  21180}},
 { 21864,  35, { 21180,  21215,  21420,  21179}},
 { 21865,  35, { 21420,  21413,  21262,  21179}},
 { 21866,  35, { 21262,  21413,  21411,  21263}},
 { 21867,  35, { 21263,  21411,  21410,  21264}},
 { 21868,  35, { 21264,  21410,  21412,  20891}},
 { 21869,  35, { 20891,  21412,  21054,  21054}},
 { 21870,  35, { 21208,  21246,  21202,  21202}},
 { 21871,  35, { 21202,  21246,  21240,  21196}},
 { 21872,  35, { 21196,  21240,  21233,  21233}},
 { 21873,  35, { 21196,  21233,  21234,  21189}},
 { 21874,  35, { 21189,  21234,  21474,  21474}},
 { 21875,  35, { 21189,  21474,  21475,  21190}},
 { 21876,  35, { 21475,  21190,  21290,  21473}},
 { 21877,  35, { 21473,  21290,  21289,  21472}},
 { 21878,  35, { 21472,  21317,  21289,  21289}},
 { 21879,  35, { 21289,  21288,  21312,  21317}},
 { 21880,  35, { 21288,  21287,  21307,  21312}},
 { 21881,  35, { 21287,  21301,  21307,  21307}},
 { 21882,  35, { 21301,  21287,  21286,  21302}},
 { 21883,  35, { 21302,  21286,  21422,  21422}},
 { 21884,  35, { 21286,  21422,  21066,  20920}},
 { 21885,  35, { 21456,  21122,  21118,  21109}},
 { 21886,  35, { 21122,  21456,  21453,  21121}},
 { 21887,  35, { 21121,  21453,  21120,  21120}},
 { 21888,  35, { 21450,  21456,  21453,  21453}},
 { 21889,  35, { 21394,  21397,  21396,  21396}},
 { 21890,  35, { 21394,  21395,  21397,  21397}},
 { 21891,  35, { 21035,  21036,  21041,  21032}},
 { 21892,  35, { 21033,  21035,  21032,  21032}},
 { 21893,  35, { 21035,  21033,  21034,  21034}},
 { 21894,  35, { 21056,  21033,  21052,  21052}},
 { 21895,  35, { 21056,  21034,  21033,  21033}},
 { 21896,  36, { 21481,  21482,  21483,  21484}},
 { 21897,  36, { 21482,  21485,  21486,  21483}},
 { 21898,  36, { 21485,  21487,  21488,  21486}},
 { 21899,  36, { 21487,  21489,  21490,  21488}},
 { 21900,  36, { 21489,  21491,  21492,  21490}},
 { 21901,  36, { 21484,  21483,  21493,  21494}},
 { 21902,  36, { 21483,  21486,  21495,  21493}},
 { 21903,  36, { 21486,  21488,  21496,  21495}},
 { 21904,  36, { 21488,  21490,  21497,  21496}},
 { 21905,  36, { 21490,  21492,  21498,  21497}},
 { 21906,  36, { 21494,  21493,  21499,  21500}},
 { 21907,  36, { 21493,  21495,  21501,  21499}},
 { 21908,  36, { 21495,  21496,  21502,  21501}},
 { 21909,  36, { 21496,  21497,  21503,  21502}},
 { 21910,  36, { 21497,  21498,  21504,  21503}},
 { 21911,  36, { 21500,  21499,  21505,  21506}},
 { 21912,  36, { 21499,  21501,  21507,  21505}},
 { 21913,  36, { 21501,  21502,  21508,  21507}},
 { 21914,  36, { 21502,  21503,  21509,  21508}},
 { 21915,  36, { 21503,  21504,  21510,  21509}},
 { 21916,  36, { 21506,  21505,  21511,  21512}},
 { 21917,  36, { 21505,  21507,  21513,  21511}},
 { 21918,  36, { 21507,  21508,  21514,  21513}},
 { 21919,  36, { 21508,  21509,  21515,  21514}},
 { 21920,  36, { 21509,  21510,  21516,  21515}},
 { 21921,  36, { 21481,  21482,  21517,  21518}},
 { 21922,  36, { 21482,  21485,  21519,  21517}},
 { 21923,  36, { 21485,  21487,  21520,  21519}},
 { 21924,  36, { 21487,  21489,  21521,  21520}},
 { 21925,  36, { 21489,  21491,  21522,  21521}},
 { 21926,  36, { 21518,  21517,  21523,  21524}},
 { 21927,  36, { 21517,  21519,  21525,  21523}},
 { 21928,  36, { 21519,  21520,  21526,  21525}},
 { 21929,  36, { 21520,  21521,  21527,  21526}},
 { 21930,  36, { 21521,  21522,  21528,  21527}},
 { 21931,  36, { 21528,  21529,  21530,  21527}},
 { 21932,  36, { 21529,  21531,  21532,  21530}},
 { 21933,  36, { 21527,  21530,  21533,  21526}},
 { 21934,  36, { 21530,  21532,  21534,  21533}},
 { 21935,  36, { 21526,  21533,  21535,  21525}},
 { 21936,  36, { 21533,  21534,  21536,  21535}},
 { 21937,  36, { 21525,  21535,  21537,  21523}},
 { 21938,  36, { 21535,  21536,  21538,  21537}},
 { 21939,  36, { 21523,  21537,  21539,  21524}},
 { 21940,  36, { 21537,  21538,  21540,  21539}},
 { 21941,  36, { 21541,  21542,  21543,  21544}},
 { 21942,  36, { 21542,  21545,  21546,  21543}},
 { 21943,  36, { 21545,  21547,  21548,  21546}},
 { 21944,  36, { 21547,  21549,  21550,  21548}},
 { 21945,  36, { 21549,  21551,  21552,  21550}},
 { 21946,  36, { 21544,  21543,  21538,  21540}},
 { 21947,  36, { 21543,  21546,  21536,  21538}},
 { 21948,  36, { 21546,  21548,  21534,  21536}},
 { 21949,  36, { 21548,  21550,  21532,  21534}},
 { 21950,  36, { 21550,  21552,  21531,  21532}},
 { 21951,  36, { 21504,  21553,  21554,  21498}},
 { 21952,  36, { 21553,  21555,  21556,  21554}},
 { 21953,  36, { 21555,  21557,  21558,  21556}},
 { 21954,  36, { 21557,  21559,  21560,  21558}},
 { 21955,  36, { 21559,  21561,  21562,  21560}},
 { 21956,  36, { 21561,  21563,  21564,  21562}},
 { 21957,  36, { 21563,  21565,  21566,  21564}},
 { 21958,  36, { 21565,  21567,  21568,  21566}},
 { 21959,  36, { 21498,  21554,  21569,  21492}},
 { 21960,  36, { 21554,  21556,  21570,  21569}},
 { 21961,  36, { 21556,  21558,  21571,  21570}},
 { 21962,  36, { 21558,  21560,  21572,  21571}},
 { 21963,  36, { 21560,  21562,  21573,  21572}},
 { 21964,  36, { 21562,  21564,  21574,  21573}},
 { 21965,  36, { 21564,  21566,  21575,  21574}},
 { 21966,  36, { 21492,  21569,  21576,  21491}},
 { 21967,  36, { 21569,  21570,  21577,  21576}},
 { 21968,  36, { 21570,  21571,  21578,  21577}},
 { 21969,  36, { 21571,  21572,  21579,  21578}},
 { 21970,  36, { 21572,  21573,  21580,  21579}},
 { 21971,  36, { 21573,  21574,  21581,  21580}},
 { 21972,  36, { 21574,  21575,  21582,  21581}},
 { 21973,  36, { 21575,  21568,  21583,  21582}},
 { 21974,  36, { 21584,  21585,  21586,  21587}},
 { 21975,  36, { 21551,  21588,  21589,  21516}},
 { 21976,  36, { 21588,  21590,  21591,  21589}},
 { 21977,  36, { 21590,  21592,  21593,  21591}},
 { 21978,  36, { 21592,  21594,  21595,  21593}},
 { 21979,  36, { 21594,  21596,  21597,  21595}},
 { 21980,  36, { 21596,  21598,  21599,  21597}},
 { 21981,  36, { 21598,  21600,  21601,  21599}},
 { 21982,  36, { 21600,  21602,  21603,  21601}},
 { 21983,  36, { 21602,  21604,  21605,  21603}},
 { 21984,  36, { 21604,  21606,  21607,  21605}},
 { 21985,  36, { 21516,  21589,  21608,  21510}},
 { 21986,  36, { 21589,  21591,  21609,  21608}},
 { 21987,  36, { 21591,  21593,  21610,  21609}},
 { 21988,  36, { 21593,  21595,  21611,  21610}},
 { 21989,  36, { 21595,  21597,  21612,  21611}},
 { 21990,  36, { 21597,  21599,  21613,  21612}},
 { 21991,  36, { 21599,  21601,  21614,  21613}},
 { 21992,  36, { 21601,  21603,  21615,  21614}},
 { 21993,  36, { 21603,  21605,  21616,  21615}},
 { 21994,  36, { 21605,  21607,  21616,  21616}},
 { 21995,  36, { 21510,  21608,  21553,  21504}},
 { 21996,  36, { 21608,  21609,  21555,  21553}},
 { 21997,  36, { 21609,  21610,  21557,  21555}},
 { 21998,  36, { 21610,  21611,  21559,  21557}},
 { 21999,  36, { 21611,  21612,  21561,  21559}},
 { 22000,  36, { 21612,  21613,  21563,  21561}},
 { 22001,  36, { 21613,  21614,  21565,  21563}},
 { 22002,  36, { 21614,  21615,  21567,  21565}},
 { 22003,  36, { 21615,  21616,  21585,  21567}},
 { 22004,  36, { 21616,  21607,  21586,  21585}},
 { 22005,  36, { 21551,  21516,  21515,  21549}},
 { 22006,  36, { 21515,  21514,  21547,  21549}},
 { 22007,  36, { 21514,  21513,  21545,  21547}},
 { 22008,  36, { 21513,  21511,  21542,  21545}},
 { 22009,  36, { 21511,  21512,  21541,  21542}},
 { 22010,  36, { 21617,  21618,  21619,  21619}},
 { 22011,  36, { 21619,  21618,  21620,  21621}},
 { 22012,  36, { 21621,  21620,  21622,  21623}},
 { 22013,  36, { 21623,  21622,  21624,  21625}},
 { 22014,  36, { 21625,  21624,  21626,  21627}},
 { 22015,  36, { 21627,  21626,  21628,  21629}},
 { 22016,  36, { 21629,  21628,  21630,  21631}},
 { 22017,  36, { 21631,  21630,  21632,  21633}},
 { 22018,  36, { 21633,  21632,  21634,  21635}},
 { 22019,  36, { 21635,  21634,  21636,  21636}},
 { 22020,  36, { 21617,  21637,  21618,  21618}},
 { 22021,  36, { 21618,  21637,  21638,  21620}},
 { 22022,  36, { 21620,  21638,  21639,  21622}},
 { 22023,  36, { 21622,  21639,  21640,  21624}},
 { 22024,  36, { 21624,  21640,  21641,  21626}},
 { 22025,  36, { 21626,  21641,  21642,  21628}},
 { 22026,  36, { 21628,  21642,  21643,  21630}},
 { 22027,  36, { 21630,  21643,  21644,  21632}},
 { 22028,  36, { 21632,  21644,  21645,  21634}},
 { 22029,  36, { 21634,  21645,  21636,  21636}},
 { 22030,  36, { 21617,  21646,  21637,  21637}},
 { 22031,  36, { 21637,  21646,  21647,  21638}},
 { 22032,  36, { 21638,  21647,  21648,  21639}},
 { 22033,  36, { 21639,  21648,  21649,  21640}},
 { 22034,  36, { 21640,  21649,  21650,  21641}},
 { 22035,  36, { 21641,  21650,  21651,  21642}},
 { 22036,  36, { 21642,  21651,  21652,  21643}},
 { 22037,  36, { 21643,  21652,  21653,  21644}},
 { 22038,  36, { 21644,  21653,  21654,  21645}},
 { 22039,  36, { 21645,  21654,  21636,  21636}},
 { 22040,  36, { 21617,  21655,  21646,  21646}},
 { 22041,  36, { 21646,  21655,  21656,  21647}},
 { 22042,  36, { 21647,  21656,  21657,  21648}},
 { 22043,  36, { 21648,  21657,  21658,  21649}},
 { 22044,  36, { 21649,  21658,  21659,  21650}},
 { 22045,  36, { 21650,  21659,  21660,  21651}},
 { 22046,  36, { 21651,  21660,  21661,  21652}},
 { 22047,  36, { 21652,  21661,  21662,  21653}},
 { 22048,  36, { 21653,  21662,  21663,  21654}},
 { 22049,  36, { 21654,  21663,  21636,  21636}},
 { 22050,  36, { 21617,  21664,  21655,  21655}},
 { 22051,  36, { 21655,  21664,  21665,  21656}},
 { 22052,  36, { 21656,  21665,  21666,  21657}},
 { 22053,  36, { 21657,  21666,  21667,  21658}},
 { 22054,  36, { 21658,  21667,  21668,  21659}},
 { 22055,  36, { 21659,  21668,  21669,  21660}},
 { 22056,  36, { 21660,  21669,  21670,  21661}},
 { 22057,  36, { 21661,  21670,  21671,  21662}},
 { 22058,  36, { 21662,  21671,  21672,  21663}},
 { 22059,  36, { 21663,  21672,  21636,  21636}},
 { 22060,  36, { 21636,  21635,  21673,  21673}},
 { 22061,  36, { 21673,  21635,  21633,  21674}},
 { 22062,  36, { 21674,  21633,  21631,  21675}},
 { 22063,  36, { 21675,  21631,  21629,  21676}},
 { 22064,  36, { 21676,  21629,  21627,  21587}},
 { 22065,  36, { 21587,  21627,  21625,  21677}},
 { 22066,  36, { 21677,  21625,  21623,  21678}},
 { 22067,  36, { 21678,  21623,  21621,  21679}},
 { 22068,  36, { 21679,  21621,  21619,  21680}},
 { 22069,  36, { 21680,  21619,  21617,  21617}},
 { 22070,  36, { 21617,  21680,  21681,  21681}},
 { 22071,  36, { 21681,  21680,  21679,  21682}},
 { 22072,  36, { 21682,  21679,  21678,  21683}},
 { 22073,  36, { 21683,  21678,  21677,  21684}},
 { 22074,  36, { 21684,  21677,  21587,  21586}},
 { 22075,  36, { 21586,  21587,  21676,  21685}},
 { 22076,  36, { 21685,  21676,  21675,  21686}},
 { 22077,  36, { 21686,  21675,  21674,  21687}},
 { 22078,  36, { 21687,  21674,  21673,  21688}},
 { 22079,  36, { 21688,  21673,  21636,  21636}},
 { 22080,  36, { 21617,  21681,  21689,  21689}},
 { 22081,  36, { 21689,  21681,  21682,  21690}},
 { 22082,  36, { 21690,  21682,  21683,  21691}},
 { 22083,  36, { 21691,  21683,  21684,  21692}},
 { 22084,  36, { 21692,  21684,  21586,  21607}},
 { 22085,  36, { 21607,  21586,  21685,  21693}},
 { 22086,  36, { 21693,  21685,  21686,  21694}},
 { 22087,  36, { 21694,  21686,  21687,  21695}},
 { 22088,  36, { 21695,  21687,  21688,  21696}},
 { 22089,  36, { 21696,  21688,  21636,  21636}},
 { 22090,  36, { 21636,  21697,  21672,  21672}},
 { 22091,  36, { 21672,  21697,  21698,  21671}},
 { 22092,  36, { 21671,  21698,  21699,  21670}},
 { 22093,  36, { 21670,  21699,  21700,  21669}},
 { 22094,  36, { 21669,  21700,  21701,  21668}},
 { 22095,  36, { 21668,  21701,  21702,  21667}},
 { 22096,  36, { 21667,  21702,  21703,  21666}},
 { 22097,  36, { 21666,  21703,  21704,  21665}},
 { 22098,  36, { 21665,  21704,  21705,  21664}},
 { 22099,  36, { 21664,  21705,  21617,  21617}},
 { 22100,  36, { 21617,  21706,  21705,  21705}},
 { 22101,  36, { 21705,  21706,  21707,  21704}},
 { 22102,  36, { 21704,  21707,  21708,  21703}},
 { 22103,  36, { 21703,  21708,  21709,  21702}},
 { 22104,  36, { 21702,  21709,  21710,  21701}},
 { 22105,  36, { 21701,  21710,  21711,  21700}},
 { 22106,  36, { 21700,  21711,  21712,  21699}},
 { 22107,  36, { 21699,  21712,  21713,  21698}},
 { 22108,  36, { 21698,  21713,  21714,  21697}},
 { 22109,  36, { 21697,  21714,  21636,  21636}},
 { 22110,  36, { 21617,  21715,  21706,  21706}},
 { 22111,  36, { 21706,  21715,  21716,  21707}},
 { 22112,  36, { 21707,  21716,  21717,  21708}},
 { 22113,  36, { 21708,  21717,  21718,  21709}},
 { 22114,  36, { 21709,  21718,  21606,  21710}},
 { 22115,  36, { 21710,  21606,  21719,  21711}},
 { 22116,  36, { 21711,  21719,  21720,  21712}},
 { 22117,  36, { 21712,  21720,  21721,  21713}},
 { 22118,  36, { 21713,  21721,  21722,  21714}},
 { 22119,  36, { 21714,  21722,  21636,  21636}},
 { 22120,  36, { 21636,  21696,  21722,  21722}},
 { 22121,  36, { 21722,  21696,  21695,  21721}},
 { 22122,  36, { 21721,  21695,  21694,  21720}},
 { 22123,  36, { 21720,  21694,  21693,  21719}},
 { 22124,  36, { 21719,  21693,  21607,  21606}},
 { 22125,  36, { 21606,  21607,  21692,  21718}},
 { 22126,  36, { 21718,  21692,  21691,  21717}},
 { 22127,  36, { 21717,  21691,  21690,  21716}},
 { 22128,  36, { 21716,  21690,  21689,  21715}},
 { 22129,  36, { 21715,  21689,  21617,  21617}},
 { 22130,  36, { 21583,  21568,  21585,  21584}},
 { 22131,  36, { 21568,  21567,  21585,  21585}},
 { 22132,  36, { 21575,  21566,  21568,  21568}},
 { 22133,  37, { 21723,  21726,  21725,  21724}},
 { 22134,  37, { 21726,  21728,  21727,  21725}},
 { 22135,  37, { 21728,  21730,  21729,  21727}},
 { 22136,  37, { 21730,  21732,  21731,  21729}},
 { 22137,  37, { 21732,  21734,  21733,  21731}},
 { 22138,  37, { 21724,  21725,  21736,  21735}},
 { 22139,  37, { 21725,  21727,  21737,  21736}},
 { 22140,  37, { 21727,  21729,  21738,  21737}},
 { 22141,  37, { 21729,  21731,  21739,  21738}},
 { 22142,  37, { 21731,  21733,  21740,  21739}},
 { 22143,  37, { 21735,  21736,  21742,  21741}},
 { 22144,  37, { 21736,  21737,  21743,  21742}},
 { 22145,  37, { 21737,  21738,  21744,  21743}},
 { 22146,  37, { 21738,  21739,  21745,  21744}},
 { 22147,  37, { 21739,  21740,  21746,  21745}},
 { 22148,  37, { 21741,  21742,  21748,  21747}},
 { 22149,  37, { 21742,  21743,  21749,  21748}},
 { 22150,  37, { 21743,  21744,  21750,  21749}},
 { 22151,  37, { 21744,  21745,  21751,  21750}},
 { 22152,  37, { 21745,  21746,  21752,  21751}},
 { 22153,  37, { 21747,  21748,  21754,  21753}},
 { 22154,  37, { 21748,  21749,  21755,  21754}},
 { 22155,  37, { 21749,  21750,  21756,  21755}},
 { 22156,  37, { 21750,  21751,  21757,  21756}},
 { 22157,  37, { 21751,  21752,  21758,  21757}},
 { 22158,  37, { 21723,  21759,  21760,  21726}},
 { 22159,  37, { 21726,  21760,  21761,  21728}},
 { 22160,  37, { 21728,  21761,  21762,  21730}},
 { 22161,  37, { 21730,  21762,  21763,  21732}},
 { 22162,  37, { 21732,  21763,  21764,  21734}},
 { 22163,  37, { 21759,  21765,  21766,  21760}},
 { 22164,  37, { 21760,  21766,  21767,  21761}},
 { 22165,  37, { 21761,  21767,  21768,  21762}},
 { 22166,  37, { 21762,  21768,  21769,  21763}},
 { 22167,  37, { 21763,  21769,  21770,  21764}},
 { 22168,  37, { 21770,  21769,  21771,  21772}},
 { 22169,  37, { 21772,  21771,  21773,  21774}},
 { 22170,  37, { 21769,  21768,  21775,  21771}},
 { 22171,  37, { 21771,  21775,  21776,  21773}},
 { 22172,  37, { 21768,  21767,  21777,  21775}},
 { 22173,  37, { 21775,  21777,  21778,  21776}},
 { 22174,  37, { 21767,  21766,  21779,  21777}},
 { 22175,  37, { 21777,  21779,  21780,  21778}},
 { 22176,  37, { 21766,  21765,  21781,  21779}},
 { 22177,  37, { 21779,  21781,  21782,  21780}},
 { 22178,  37, { 21783,  21786,  21785,  21784}},
 { 22179,  37, { 21786,  21788,  21787,  21785}},
 { 22180,  37, { 21788,  21790,  21789,  21787}},
 { 22181,  37, { 21790,  21792,  21791,  21789}},
 { 22182,  37, { 21792,  21794,  21793,  21791}},
 { 22183,  37, { 21784,  21785,  21780,  21782}},
 { 22184,  37, { 21785,  21787,  21778,  21780}},
 { 22185,  37, { 21787,  21789,  21776,  21778}},
 { 22186,  37, { 21789,  21791,  21773,  21776}},
 { 22187,  37, { 21791,  21793,  21774,  21773}},
 { 22188,  37, { 21746,  21740,  21795,  21796}},
 { 22189,  37, { 21796,  21795,  21797,  21798}},
 { 22190,  37, { 21798,  21797,  21799,  21800}},
 { 22191,  37, { 21800,  21799,  21801,  21802}},
 { 22192,  37, { 21802,  21801,  21803,  21804}},
 { 22193,  37, { 21804,  21803,  21805,  21806}},
 { 22194,  37, { 21806,  21805,  21807,  21808}},
 { 22195,  37, { 21808,  21807,  21809,  21810}},
 { 22196,  37, { 21740,  21733,  21811,  21795}},
 { 22197,  37, { 21795,  21811,  21812,  21797}},
 { 22198,  37, { 21797,  21812,  21813,  21799}},
 { 22199,  37, { 21799,  21813,  21814,  21801}},
 { 22200,  37, { 21801,  21814,  21815,  21803}},
 { 22201,  37, { 21803,  21815,  21816,  21805}},
 { 22202,  37, { 21805,  21816,  21817,  21807}},
 { 22203,  37, { 21733,  21734,  21818,  21811}},
 { 22204,  37, { 21811,  21818,  21819,  21812}},
 { 22205,  37, { 21812,  21819,  21820,  21813}},
 { 22206,  37, { 21813,  21820,  21821,  21814}},
 { 22207,  37, { 21814,  21821,  21822,  21815}},
 { 22208,  37, { 21815,  21822,  21823,  21816}},
 { 22209,  37, { 21816,  21823,  21824,  21817}},
 { 22210,  37, { 21817,  21824,  21825,  21809}},
 { 22211,  37, { 21826,  21827,  21828,  21829}},
 { 22212,  37, { 21794,  21758,  21830,  21831}},
 { 22213,  37, { 21831,  21830,  21832,  21833}},
 { 22214,  37, { 21833,  21832,  21834,  21835}},
 { 22215,  37, { 21835,  21834,  21836,  21837}},
 { 22216,  37, { 21837,  21836,  21838,  21839}},
 { 22217,  37, { 21839,  21838,  21840,  21841}},
 { 22218,  37, { 21841,  21840,  21842,  21843}},
 { 22219,  37, { 21843,  21842,  21844,  21845}},
 { 22220,  37, { 21845,  21844,  21846,  21847}},
 { 22221,  37, { 21847,  21846,  21848,  21849}},
 { 22222,  37, { 21758,  21752,  21850,  21830}},
 { 22223,  37, { 21830,  21850,  21851,  21832}},
 { 22224,  37, { 21832,  21851,  21852,  21834}},
 { 22225,  37, { 21834,  21852,  21853,  21836}},
 { 22226,  37, { 21836,  21853,  21854,  21838}},
 { 22227,  37, { 21838,  21854,  21855,  21840}},
 { 22228,  37, { 21840,  21855,  21856,  21842}},
 { 22229,  37, { 21842,  21856,  21857,  21844}},
 { 22230,  37, { 21844,  21857,  21858,  21846}},
 { 22231,  37, { 21846,  21858,  21848,  21848}},
 { 22232,  37, { 21752,  21746,  21796,  21850}},
 { 22233,  37, { 21850,  21796,  21798,  21851}},
 { 22234,  37, { 21851,  21798,  21800,  21852}},
 { 22235,  37, { 21852,  21800,  21802,  21853}},
 { 22236,  37, { 21853,  21802,  21804,  21854}},
 { 22237,  37, { 21854,  21804,  21806,  21855}},
 { 22238,  37, { 21855,  21806,  21808,  21856}},
 { 22239,  37, { 21856,  21808,  21810,  21857}},
 { 22240,  37, { 21857,  21810,  21829,  21858}},
 { 22241,  37, { 21858,  21829,  21828,  21848}},
 { 22242,  37, { 21794,  21792,  21757,  21758}},
 { 22243,  37, { 21757,  21792,  21790,  21756}},
 { 22244,  37, { 21756,  21790,  21788,  21755}},
 { 22245,  37, { 21755,  21788,  21786,  21754}},
 { 22246,  37, { 21754,  21786,  21783,  21753}},
 { 22247,  37, { 21859,  21860,  21861,  21861}},
 { 22248,  37, { 21860,  21862,  21863,  21861}},
 { 22249,  37, { 21862,  21864,  21865,  21863}},
 { 22250,  37, { 21864,  21866,  21867,  21865}},
 { 22251,  37, { 21866,  21868,  21869,  21867}},
 { 22252,  37, { 21868,  21870,  21871,  21869}},
 { 22253,  37, { 21870,  21872,  21873,  21871}},
 { 22254,  37, { 21872,  21874,  21875,  21873}},
 { 22255,  37, { 21874,  21876,  21877,  21875}},
 { 22256,  37, { 21876,  21878,  21877,  21877}},
 { 22257,  37, { 21859,  21861,  21879,  21879}},
 { 22258,  37, { 21861,  21863,  21880,  21879}},
 { 22259,  37, { 21863,  21865,  21881,  21880}},
 { 22260,  37, { 21865,  21867,  21882,  21881}},
 { 22261,  37, { 21867,  21869,  21883,  21882}},
 { 22262,  37, { 21869,  21871,  21884,  21883}},
 { 22263,  37, { 21871,  21873,  21885,  21884}},
 { 22264,  37, { 21873,  21875,  21886,  21885}},
 { 22265,  37, { 21875,  21877,  21887,  21886}},
 { 22266,  37, { 21877,  21878,  21887,  21887}},
 { 22267,  37, { 21859,  21879,  21888,  21888}},
 { 22268,  37, { 21879,  21880,  21889,  21888}},
 { 22269,  37, { 21880,  21881,  21890,  21889}},
 { 22270,  37, { 21881,  21882,  21891,  21890}},
 { 22271,  37, { 21882,  21883,  21892,  21891}},
 { 22272,  37, { 21883,  21884,  21893,  21892}},
 { 22273,  37, { 21884,  21885,  21894,  21893}},
 { 22274,  37, { 21885,  21886,  21895,  21894}},
 { 22275,  37, { 21886,  21887,  21896,  21895}},
 { 22276,  37, { 21887,  21878,  21896,  21896}},
 { 22277,  37, { 21859,  21888,  21897,  21897}},
 { 22278,  37, { 21888,  21889,  21898,  21897}},
 { 22279,  37, { 21889,  21890,  21899,  21898}},
 { 22280,  37, { 21890,  21891,  21900,  21899}},
 { 22281,  37, { 21891,  21892,  21901,  21900}},
 { 22282,  37, { 21892,  21893,  21902,  21901}},
 { 22283,  37, { 21893,  21894,  21903,  21902}},
 { 22284,  37, { 21894,  21895,  21904,  21903}},
 { 22285,  37, { 21895,  21896,  21905,  21904}},
 { 22286,  37, { 21896,  21878,  21905,  21905}},
 { 22287,  37, { 21859,  21897,  21906,  21906}},
 { 22288,  37, { 21897,  21898,  21907,  21906}},
 { 22289,  37, { 21898,  21899,  21908,  21907}},
 { 22290,  37, { 21899,  21900,  21909,  21908}},
 { 22291,  37, { 21900,  21901,  21910,  21909}},
 { 22292,  37, { 21901,  21902,  21911,  21910}},
 { 22293,  37, { 21902,  21903,  21912,  21911}},
 { 22294,  37, { 21903,  21904,  21913,  21912}},
 { 22295,  37, { 21904,  21905,  21914,  21913}},
 { 22296,  37, { 21905,  21878,  21914,  21914}},
 { 22297,  37, { 21878,  21876,  21915,  21915}},
 { 22298,  37, { 21915,  21876,  21874,  21916}},
 { 22299,  37, { 21916,  21874,  21872,  21917}},
 { 22300,  37, { 21917,  21872,  21870,  21918}},
 { 22301,  37, { 21918,  21870,  21868,  21827}},
 { 22302,  37, { 21827,  21868,  21866,  21919}},
 { 22303,  37, { 21919,  21866,  21864,  21920}},
 { 22304,  37, { 21920,  21864,  21862,  21921}},
 { 22305,  37, { 21921,  21862,  21860,  21922}},
 { 22306,  37, { 21922,  21860,  21859,  21859}},
 { 22307,  37, { 21859,  21923,  21922,  21922}},
 { 22308,  37, { 21923,  21924,  21921,  21922}},
 { 22309,  37, { 21924,  21925,  21920,  21921}},
 { 22310,  37, { 21925,  21926,  21919,  21920}},
 { 22311,  37, { 21926,  21828,  21827,  21919}},
 { 22312,  37, { 21828,  21927,  21918,  21827}},
 { 22313,  37, { 21927,  21928,  21917,  21918}},
 { 22314,  37, { 21928,  21929,  21916,  21917}},
 { 22315,  37, { 21929,  21930,  21915,  21916}},
 { 22316,  37, { 21930,  21878,  21915,  21915}},
 { 22317,  37, { 21859,  21931,  21923,  21923}},
 { 22318,  37, { 21931,  21932,  21924,  21923}},
 { 22319,  37, { 21932,  21933,  21925,  21924}},
 { 22320,  37, { 21933,  21934,  21926,  21925}},
 { 22321,  37, { 21934,  21848,  21828,  21926}},
 { 22322,  37, { 21848,  21935,  21927,  21828}},
 { 22323,  37, { 21935,  21936,  21928,  21927}},
 { 22324,  37, { 21936,  21937,  21929,  21928}},
 { 22325,  37, { 21937,  21938,  21930,  21929}},
 { 22326,  37, { 21938,  21878,  21930,  21930}},
 { 22327,  37, { 21878,  21939,  21914,  21914}},
 { 22328,  37, { 21914,  21939,  21940,  21913}},
 { 22329,  37, { 21913,  21940,  21941,  21912}},
 { 22330,  37, { 21912,  21941,  21942,  21911}},
 { 22331,  37, { 21911,  21942,  21943,  21910}},
 { 22332,  37, { 21910,  21943,  21944,  21909}},
 { 22333,  37, { 21909,  21944,  21945,  21908}},
 { 22334,  37, { 21908,  21945,  21946,  21907}},
 { 22335,  37, { 21907,  21946,  21947,  21906}},
 { 22336,  37, { 21906,  21947,  21859,  21859}},
 { 22337,  37, { 21859,  21947,  21948,  21948}},
 { 22338,  37, { 21947,  21946,  21949,  21948}},
 { 22339,  37, { 21946,  21945,  21950,  21949}},
 { 22340,  37, { 21945,  21944,  21951,  21950}},
 { 22341,  37, { 21944,  21943,  21952,  21951}},
 { 22342,  37, { 21943,  21942,  21953,  21952}},
 { 22343,  37, { 21942,  21941,  21954,  21953}},
 { 22344,  37, { 21941,  21940,  21955,  21954}},
 { 22345,  37, { 21940,  21939,  21956,  21955}},
 { 22346,  37, { 21939,  21878,  21956,  21956}},
 { 22347,  37, { 21859,  21948,  21957,  21957}},
 { 22348,  37, { 21948,  21949,  21958,  21957}},
 { 22349,  37, { 21949,  21950,  21959,  21958}},
 { 22350,  37, { 21950,  21951,  21960,  21959}},
 { 22351,  37, { 21951,  21952,  21849,  21960}},
 { 22352,  37, { 21952,  21953,  21961,  21849}},
 { 22353,  37, { 21953,  21954,  21962,  21961}},
 { 22354,  37, { 21954,  21955,  21963,  21962}},
 { 22355,  37, { 21955,  21956,  21964,  21963}},
 { 22356,  37, { 21956,  21878,  21964,  21964}},
 { 22357,  37, { 21878,  21938,  21964,  21964}},
 { 22358,  37, { 21964,  21938,  21937,  21963}},
 { 22359,  37, { 21963,  21937,  21936,  21962}},
 { 22360,  37, { 21962,  21936,  21935,  21961}},
 { 22361,  37, { 21961,  21935,  21848,  21849}},
 { 22362,  37, { 21849,  21848,  21934,  21960}},
 { 22363,  37, { 21960,  21934,  21933,  21959}},
 { 22364,  37, { 21959,  21933,  21932,  21958}},
 { 22365,  37, { 21958,  21932,  21931,  21957}},
 { 22366,  37, { 21957,  21931,  21859,  21859}},
 { 22367,  37, { 21825,  21826,  21829,  21809}},
 { 22368,  37, { 21809,  21829,  21810,  21810}},
 { 22369,  37, { 21817,  21809,  21807,  21807}},
 { 22370,  50, { 21054,  21028,  21392,  21412}},
 { 22371,  50, { 21411,  21394,  21410,  21410}},
 { 22372,  50, { 21394,  21392,  21412,  21410}},
 { 22373,  50, { 21410,  20545,  21411,  21411}},
 { 22374,  50, { 21410,  20545,  21965,  21412}},
 { 22375,  50, { 21054,  21965,  21412,  21412}},
 { 22376,  50, { 21050,  21966,  20561,  21408}},
 { 22377,  50, { 20561,  21408,  21404,  21404}},
 { 22378,  50, { 21404,  20561,  21400,  21400}},
 { 22379,  50, { 20116,  20545,  21965,  21967}},
 { 22380,  50, { 21965,  20705,  20341,  21967}},
 { 22381,  50, { 21050,  21046,  21966,  21966}},
 { 22382,  50, { 21028,  21054,  21053,  21031}},
 { 22383,  50, { 21033,  21052,  21053,  21031}},
 { 22384,  50, { 21052,  21056,  21033,  21033}},
 { 22385,  50, { 21053,  21965,  21054,  21054}},
 { 22386,  50, { 21056,  20705,  21052,  21052}},
 { 22387,  50, { 20705,  21052,  21053,  21965}},
 { 22388,  50, { 21968,  21966,  20715,  20354}},
 { 22389,  50, { 20715,  21969,  21038,  21038}},
 { 22390,  50, { 20715,  21966,  21046,  21969}},
 { 22391,  50, { 21968,  20134,  20561,  21966}},
 { 22392,  50, { 21033,  21035,  21056,  21056}},
 { 22393,  50, { 21035,  21036,  21056,  21056}},
 { 22394,  50, { 21038,  21037,  20715,  20715}},
 { 22395,  50, { 21036,  20715,  21037,  21037}},
 { 22396,  50, { 21036,  20715,  21056,  21056}},
 { 22397,  50, { 21056,  20715,  20705,  20705}},
 { 22398,  50, { 20705,  20341,  20354,  20715}},
 { 22399,  50, { 21394,  21397,  21411,  21411}},
 { 22400,  50, { 21397,  21411,  21398,  21398}},
 { 22401,  50, { 21399,  20561,  21400,  21400}},
 { 22402,  50, { 21398,  20561,  21399,  21399}},
 { 22403,  50, { 21411,  20561,  21398,  21398}},
 { 22404,  50, { 20545,  20561,  21411,  21411}},
 { 22405,  50, { 20545,  20561,  20134,  20116}},
 { 22702,  38, { 22257,  22258,  22259,  22260}},
 { 22703,  38, { 22259,  22261,  22262,  22260}},
 { 22704,  38, { 22263,  22264,  22262,  22261}},
 { 22705,  38, { 22265,  22257,  22260,  22266}},
 { 22706,  38, { 22260,  22262,  22267,  22266}},
 { 22707,  38, { 22264,  22268,  22267,  22262}},
 { 22708,  38, { 22269,  22265,  22266,  22270}},
 { 22709,  38, { 22266,  22267,  22271,  22270}},
 { 22710,  38, { 22268,  22272,  22271,  22267}},
 { 22711,  38, { 22273,  22269,  22270,  22274}},
 { 22712,  38, { 22270,  22271,  22275,  22274}},
 { 22713,  38, { 22272,  22276,  22275,  22271}},
 { 22714,  38, { 22277,  22273,  22274,  22278}},
 { 22715,  38, { 22274,  22275,  22279,  22278}},
 { 22716,  38, { 22276,  22280,  22279,  22275}},
 { 22717,  38, { 22281,  22277,  22278,  22282}},
 { 22718,  38, { 22278,  22279,  22283,  22282}},
 { 22719,  38, { 22280,  22284,  22283,  22279}},
 { 22720,  38, { 22285,  22281,  22282,  22286}},
 { 22721,  38, { 22282,  22283,  22287,  22286}},
 { 22722,  38, { 22284,  22288,  22287,  22283}},
 { 22723,  38, { 22289,  22285,  22286,  22290}},
 { 22724,  38, { 22286,  22287,  22291,  22290}},
 { 22725,  38, { 22288,  22292,  22291,  22287}},
 { 22726,  38, { 22258,  22293,  22294,  22259}},
 { 22727,  38, { 22294,  22295,  22261,  22259}},
 { 22728,  38, { 22296,  22263,  22261,  22295}},
 { 22729,  38, { 22293,  22297,  22298,  22294}},
 { 22730,  38, { 22298,  22299,  22295,  22294}},
 { 22731,  38, { 22300,  22296,  22295,  22299}},
 { 22732,  38, { 22297,  22301,  22302,  22298}},
 { 22733,  38, { 22302,  22303,  22299,  22298}},
 { 22734,  38, { 22304,  22300,  22299,  22303}},
 { 22735,  38, { 22301,  22305,  22306,  22302}},
 { 22736,  38, { 22306,  22307,  22303,  22302}},
 { 22737,  38, { 22308,  22304,  22303,  22307}},
 { 22738,  38, { 22305,  22309,  22310,  22306}},
 { 22739,  38, { 22310,  22311,  22307,  22306}},
 { 22740,  38, { 22312,  22308,  22307,  22311}},
 { 22741,  38, { 22309,  22313,  22314,  22310}},
 { 22742,  38, { 22314,  22315,  22311,  22310}},
 { 22743,  38, { 22316,  22312,  22311,  22315}},
 { 22744,  38, { 22313,  22317,  22318,  22314}},
 { 22745,  38, { 22318,  22319,  22315,  22314}},
 { 22746,  38, { 22320,  22316,  22315,  22319}},
 { 22747,  38, { 22317,  22321,  22322,  22318}},
 { 22748,  38, { 22322,  22323,  22319,  22318}},
 { 22749,  38, { 22324,  22320,  22319,  22323}},
 { 22750,  38, { 22321,  22325,  22326,  22322}},
 { 22751,  38, { 22326,  22327,  22323,  22322}},
 { 22752,  38, { 22328,  22324,  22323,  22327}},
 { 22753,  38, { 22325,  22289,  22290,  22326}},
 { 22754,  38, { 22290,  22291,  22327,  22326}},
 { 22755,  38, { 22292,  22328,  22327,  22291}},
 { 22756,  38, { 22329,  22268,  22272,  22330}},
 { 22757,  38, { 22330,  22331,  22332,  22329}},
 { 22758,  38, { 22333,  22264,  22268,  22329}},
 { 22759,  38, { 22329,  22332,  22334,  22333}},
 { 22760,  38, { 22335,  22263,  22264,  22333}},
 { 22761,  38, { 22333,  22334,  22336,  22335}},
 { 22762,  38, { 22337,  22296,  22263,  22335}},
 { 22763,  38, { 22335,  22336,  22338,  22337}},
 { 22764,  38, { 22339,  22300,  22296,  22337}},
 { 22765,  38, { 22337,  22338,  22340,  22339}},
 { 22766,  38, { 22341,  22304,  22300,  22339}},
 { 22767,  38, { 22339,  22340,  22342,  22341}},
 { 22768,  38, { 22343,  22308,  22304,  22341}},
 { 22769,  38, { 22341,  22342,  22344,  22343}},
 { 22770,  38, { 22312,  22308,  22343,  22345}},
 { 22771,  38, { 22343,  22344,  22346,  22345}},
 { 22772,  38, { 22316,  22312,  22345,  22347}},
 { 22773,  38, { 22345,  22346,  22348,  22347}},
 { 22774,  38, { 22316,  22347,  22349,  22320}},
 { 22775,  38, { 22347,  22348,  22350,  22349}},
 { 22776,  38, { 22324,  22320,  22349,  22351}},
 { 22777,  38, { 22349,  22350,  22352,  22351}},
 { 22778,  38, { 22353,  22351,  22324,  22328}},
 { 22779,  38, { 22353,  22354,  22352,  22351}},
 { 22780,  38, { 22355,  22330,  22272,  22276}},
 { 22781,  38, { 22355,  22356,  22331,  22330}},
 { 22782,  38, { 22357,  22355,  22276,  22280}},
 { 22783,  38, { 22357,  22358,  22356,  22355}},
 { 22784,  38, { 22292,  22359,  22353,  22328}},
 { 22785,  38, { 22359,  22360,  22354,  22353}},
 { 22786,  38, { 22288,  22292,  22359,  22361}},
 { 22787,  38, { 22359,  22360,  22362,  22361}},
 { 22788,  38, { 22288,  22361,  22363,  22284}},
 { 22789,  38, { 22361,  22362,  22364,  22363}},
 { 22790,  38, { 22357,  22280,  22284,  22363}},
 { 22791,  38, { 22363,  22364,  22358,  22357}},
 { 22792,  38, { 22269,  22365,  22366,  22273}},
 { 22793,  38, { 22265,  22367,  22365,  22269}},
 { 22794,  38, { 22257,  22368,  22367,  22265}},
 { 22795,  38, { 22258,  22369,  22368,  22257}},
 { 22796,  38, { 22293,  22370,  22369,  22258}},
 { 22797,  38, { 22297,  22371,  22370,  22293}},
 { 22798,  38, { 22301,  22372,  22371,  22297}},
 { 22799,  38, { 22305,  22373,  22372,  22301}},
 { 22800,  38, { 22309,  22374,  22373,  22305}},
 { 22801,  38, { 22313,  22375,  22374,  22309}},
 { 22802,  38, { 22317,  22376,  22375,  22313}},
 { 22803,  38, { 22321,  22377,  22376,  22317}},
 { 22804,  38, { 22325,  22378,  22377,  22321}},
 { 22805,  38, { 22289,  22379,  22378,  22325}},
 { 22806,  38, { 22285,  22380,  22379,  22289}},
 { 22807,  38, { 22281,  22381,  22380,  22285}},
 { 22808,  38, { 22277,  22382,  22381,  22281}},
 { 22809,  38, { 22273,  22366,  22382,  22277}},
 { 22810,  38, { 22383,  22368,  22367,  22384}},
 { 22811,  38, { 22332,  22385,  22386,  22334}},
 { 22812,  38, { 22331,  22387,  22385,  22332}},
 { 22813,  38, { 22367,  22384,  22388,  22365}},
 { 22814,  38, { 22356,  22389,  22387,  22331}},
 { 22815,  38, { 22365,  22388,  22390,  22366}},
 { 22816,  38, { 22358,  22391,  22389,  22356}},
 { 22817,  38, { 22366,  22390,  22392,  22382}},
 { 22818,  38, { 22364,  22393,  22391,  22358}},
 { 22819,  38, { 22382,  22392,  22394,  22381}},
 { 22820,  38, { 22362,  22395,  22393,  22364}},
 { 22821,  38, { 22381,  22394,  22396,  22380}},
 { 22822,  38, { 22360,  22397,  22395,  22362}},
 { 22823,  38, { 22380,  22396,  22398,  22379}},
 { 22824,  38, { 22354,  22399,  22397,  22360}},
 { 22825,  38, { 22379,  22398,  22400,  22378}},
 { 22826,  38, { 22352,  22401,  22399,  22354}},
 { 22827,  38, { 22378,  22400,  22402,  22377}},
 { 22828,  38, { 22377,  22402,  22403,  22376}},
 { 22829,  38, { 22350,  22404,  22401,  22352}},
 { 22830,  38, { 22348,  22405,  22404,  22350}},
 { 22831,  38, { 22376,  22403,  22406,  22375}},
 { 22832,  38, { 22375,  22406,  22407,  22374}},
 { 22833,  38, { 22346,  22408,  22405,  22348}},
 { 22834,  38, { 22344,  22409,  22408,  22346}},
 { 22835,  38, { 22374,  22407,  22410,  22373}},
 { 22836,  38, { 22342,  22411,  22409,  22344}},
 { 22837,  38, { 22372,  22412,  22410,  22373}},
 { 22838,  38, { 22342,  22411,  22413,  22340}},
 { 22839,  38, { 22371,  22414,  22412,  22372}},
 { 22840,  38, { 22340,  22413,  22415,  22338}},
 { 22841,  38, { 22370,  22416,  22414,  22371}},
 { 22842,  38, { 22338,  22415,  22417,  22336}},
 { 22843,  38, { 22369,  22418,  22416,  22370}},
 { 22844,  38, { 22417,  22336,  22334,  22386}},
 { 22845,  38, { 22383,  22368,  22369,  22418}},
 { 22846,  38, { 22389,  22419,  22420,  22387}},
 { 22847,  38, { 22421,  22420,  22419,  22422}},
 { 22848,  38, { 22388,  22423,  22424,  22390}},
 { 22849,  38, { 22421,  22422,  22424,  22423}},
 { 22850,  38, { 22390,  22424,  22425,  22392}},
 { 22851,  38, { 22422,  22426,  22425,  22424}},
 { 22852,  38, { 22391,  22427,  22419,  22389}},
 { 22853,  38, { 22419,  22427,  22426,  22422}},
 { 22854,  38, { 22393,  22428,  22427,  22391}},
 { 22855,  38, { 22427,  22428,  22429,  22426}},
 { 22856,  38, { 22387,  22420,  22430,  22385}},
 { 22857,  38, { 22430,  22420,  22421,  22431}},
 { 22858,  38, { 22384,  22432,  22423,  22388}},
 { 22859,  38, { 22431,  22421,  22423,  22432}},
 { 22860,  38, { 22383,  22433,  22432,  22384}},
 { 22861,  38, { 22434,  22431,  22432,  22433}},
 { 22862,  38, { 22385,  22430,  22435,  22386}},
 { 22863,  38, { 22435,  22430,  22431,  22434}},
 { 22864,  38, { 22418,  22436,  22433,  22383}},
 { 22865,  38, { 22437,  22434,  22433,  22436}},
 { 22866,  38, { 22386,  22435,  22438,  22417}},
 { 22867,  38, { 22438,  22435,  22434,  22437}},
 { 22868,  38, { 22416,  22439,  22436,  22418}},
 { 22869,  38, { 22440,  22437,  22436,  22439}},
 { 22870,  38, { 22417,  22438,  22441,  22415}},
 { 22871,  38, { 22440,  22441,  22438,  22437}},
 { 22872,  38, { 22415,  22441,  22442,  22413}},
 { 22873,  38, { 22443,  22442,  22441,  22440}},
 { 22874,  38, { 22414,  22444,  22439,  22416}},
 { 22875,  38, { 22444,  22443,  22440,  22439}},
 { 22876,  38, { 22445,  22446,  22444,  22443}},
 { 22877,  38, { 22446,  22412,  22414,  22444}},
 { 22878,  38, { 22447,  22445,  22443,  22442}},
 { 22879,  38, { 22442,  22413,  22411,  22447}},
 { 22880,  38, { 22410,  22448,  22446,  22412}},
 { 22881,  38, { 22446,  22448,  22449,  22445}},
 { 22882,  38, { 22411,  22447,  22450,  22409}},
 { 22883,  38, { 22445,  22449,  22450,  22447}},
 { 22884,  38, { 22407,  22451,  22448,  22410}},
 { 22885,  38, { 22448,  22451,  22452,  22449}},
 { 22886,  38, { 22409,  22450,  22453,  22408}},
 { 22887,  38, { 22449,  22452,  22453,  22450}},
 { 22888,  38, { 22406,  22454,  22451,  22407}},
 { 22889,  38, { 22452,  22451,  22454,  22455}},
 { 22890,  38, { 22408,  22453,  22456,  22405}},
 { 22891,  38, { 22452,  22455,  22456,  22453}},
 { 22892,  38, { 22405,  22456,  22457,  22404}},
 { 22893,  38, { 22455,  22458,  22457,  22456}},
 { 22894,  38, { 22403,  22459,  22454,  22406}},
 { 22895,  38, { 22454,  22459,  22458,  22455}},
 { 22896,  38, { 22404,  22457,  22460,  22401}},
 { 22897,  38, { 22458,  22461,  22460,  22457}},
 { 22898,  38, { 22402,  22462,  22459,  22403}},
 { 22899,  38, { 22459,  22462,  22461,  22458}},
 { 22900,  38, { 22399,  22463,  22460,  22401}},
 { 22901,  38, { 22460,  22463,  22464,  22461}},
 { 22902,  38, { 22397,  22465,  22463,  22399}},
 { 22903,  38, { 22463,  22465,  22466,  22464}},
 { 22904,  38, { 22395,  22467,  22465,  22397}},
 { 22905,  38, { 22465,  22467,  22468,  22466}},
 { 22906,  38, { 22392,  22425,  22469,  22394}},
 { 22907,  38, { 22426,  22429,  22469,  22425}},
 { 22908,  38, { 22395,  22467,  22428,  22393}},
 { 22909,  38, { 22428,  22467,  22468,  22429}},
 { 22910,  38, { 22394,  22469,  22470,  22396}},
 { 22911,  38, { 22468,  22470,  22469,  22429}},
 { 22912,  38, { 22396,  22470,  22471,  22398}},
 { 22913,  38, { 22466,  22471,  22470,  22468}},
 { 22914,  38, { 22398,  22471,  22472,  22400}},
 { 22915,  38, { 22464,  22472,  22471,  22466}},
 { 22916,  38, { 22400,  22472,  22462,  22402}},
 { 22917,  38, { 22461,  22462,  22472,  22464}},
 { 22918,  39, { 22263,  22264,  22333,  22335}},
 { 22919,  39, { 22263,  22261,  22262,  22264}},
 { 22920,  39, { 22264,  22268,  22329,  22333}},
 { 22921,  39, { 22264,  22262,  22267,  22268}},
 { 22922,  39, { 22268,  22272,  22330,  22329}},
 { 22923,  39, { 22268,  22267,  22271,  22272}},
 { 22924,  39, { 22272,  22276,  22355,  22330}},
 { 22925,  39, { 22272,  22271,  22275,  22276}},
 { 22926,  39, { 22276,  22280,  22357,  22355}},
 { 22927,  39, { 22276,  22275,  22279,  22280}},
 { 22928,  39, { 22280,  22284,  22363,  22357}},
 { 22929,  39, { 22280,  22279,  22283,  22284}},
 { 22930,  39, { 22284,  22288,  22361,  22363}},
 { 22931,  39, { 22284,  22283,  22287,  22288}},
 { 22932,  39, { 22361,  22288,  22292,  22359}},
 { 22933,  39, { 22288,  22287,  22291,  22292}},
 { 22934,  39, { 22359,  22292,  22328,  22353}},
 { 22935,  39, { 22292,  22291,  22327,  22328}},
 { 22936,  39, { 22262,  22261,  22473,  22474}},
 { 22937,  39, { 22267,  22262,  22474,  22475}},
 { 22938,  39, { 22271,  22267,  22475,  22476}},
 { 22939,  39, { 22275,  22271,  22476,  22477}},
 { 22940,  39, { 22279,  22275,  22477,  22478}},
 { 22941,  39, { 22283,  22279,  22478,  22479}},
 { 22942,  39, { 22287,  22283,  22479,  22480}},
 { 22943,  39, { 22287,  22480,  22481,  22291}},
 { 22944,  39, { 22291,  22481,  22482,  22327}},
 { 22945,  39, { 22263,  22335,  22337,  22296}},
 { 22946,  39, { 22296,  22295,  22261,  22263}},
 { 22947,  39, { 22296,  22337,  22339,  22300}},
 { 22948,  39, { 22300,  22299,  22295,  22296}},
 { 22949,  39, { 22300,  22339,  22341,  22304}},
 { 22950,  39, { 22304,  22303,  22299,  22300}},
 { 22951,  39, { 22304,  22341,  22343,  22308}},
 { 22952,  39, { 22349,  22351,  22324,  22320}},
 { 22953,  39, { 22351,  22353,  22328,  22324}},
 { 22954,  39, { 22328,  22327,  22323,  22324}},
 { 22955,  39, { 22473,  22261,  22295,  22483}},
 { 22956,  39, { 22483,  22295,  22299,  22484}},
 { 22957,  39, { 22484,  22299,  22303,  22485}},
 { 22958,  39, { 22303,  22304,  22308,  22307}},
 { 22959,  39, { 22485,  22303,  22307,  22486}},
 { 22960,  39, { 22307,  22308,  22312,  22311}},
 { 22961,  39, { 22486,  22307,  22311,  22487}},
 { 22962,  39, { 22311,  22312,  22316,  22315}},
 { 22963,  39, { 22487,  22311,  22315,  22488}},
 { 22964,  39, { 22315,  22316,  22320,  22319}},
 { 22965,  39, { 22488,  22315,  22319,  22489}},
 { 22966,  39, { 22319,  22320,  22324,  22323}},
 { 22967,  39, { 22490,  22489,  22319,  22323}},
 { 22968,  39, { 22482,  22490,  22323,  22327}},
 { 22969,  39, { 22491,  22478,  22477,  22492}},
 { 22970,  39, { 22493,  22479,  22478,  22491}},
 { 22971,  39, { 22492,  22477,  22476,  22494}},
 { 22972,  39, { 22495,  22480,  22479,  22493}},
 { 22973,  39, { 22496,  22481,  22480,  22495}},
 { 22974,  39, { 22497,  22482,  22481,  22496}},
 { 22975,  39, { 22498,  22490,  22482,  22497}},
 { 22976,  39, { 22494,  22476,  22475,  22499}},
 { 22977,  39, { 22499,  22475,  22474,  22500}},
 { 22978,  39, { 22500,  22474,  22473,  22501}},
 { 22979,  39, { 22501,  22473,  22483,  22502}},
 { 22980,  39, { 22503,  22486,  22487,  22504}},
 { 22981,  39, { 22504,  22487,  22488,  22505}},
 { 22982,  39, { 22506,  22485,  22486,  22503}},
 { 22983,  39, { 22505,  22488,  22489,  22507}},
 { 22984,  39, { 22507,  22489,  22490,  22498}},
 { 22985,  39, { 22508,  22484,  22485,  22506}},
 { 22986,  39, { 22502,  22483,  22484,  22508}},
 { 22987,  39, { 22509,  22510,  22511,  22511}},
 { 22988,  39, { 22510,  22491,  22492,  22511}},
 { 22989,  39, { 22510,  22509,  22512,  22512}},
 { 22990,  39, { 22512,  22493,  22491,  22510}},
 { 22991,  39, { 22513,  22509,  22511,  22511}},
 { 22992,  39, { 22511,  22492,  22494,  22513}},
 { 22993,  39, { 22512,  22509,  22514,  22514}},
 { 22994,  39, { 22514,  22495,  22493,  22512}},
 { 22995,  39, { 22514,  22509,  22515,  22515}},
 { 22996,  39, { 22515,  22496,  22495,  22514}},
 { 22997,  39, { 22515,  22509,  22516,  22516}},
 { 22998,  39, { 22516,  22497,  22496,  22515}},
 { 22999,  39, { 22517,  22516,  22509,  22509}},
 { 23000,  39, { 22517,  22498,  22497,  22516}},
 { 23001,  39, { 22518,  22509,  22513,  22513}},
 { 23002,  39, { 22513,  22494,  22499,  22518}},
 { 23003,  39, { 22519,  22509,  22518,  22518}},
 { 23004,  39, { 22518,  22499,  22500,  22519}},
 { 23005,  39, { 22520,  22509,  22519,  22519}},
 { 23006,  39, { 22519,  22500,  22501,  22520}},
 { 23007,  39, { 22520,  22521,  22509,  22509}},
 { 23008,  39, { 22520,  22501,  22502,  22521}},
 { 23009,  39, { 22509,  22522,  22523,  22523}},
 { 23010,  39, { 22522,  22503,  22504,  22523}},
 { 23011,  39, { 22523,  22524,  22509,  22509}},
 { 23012,  39, { 22523,  22504,  22505,  22524}},
 { 23013,  39, { 22525,  22522,  22509,  22509}},
 { 23014,  39, { 22525,  22506,  22503,  22522}},
 { 23015,  39, { 22524,  22526,  22509,  22509}},
 { 23016,  39, { 22524,  22505,  22507,  22526}},
 { 23017,  39, { 22526,  22517,  22509,  22509}},
 { 23018,  39, { 22526,  22507,  22498,  22517}},
 { 23019,  39, { 22527,  22525,  22509,  22509}},
 { 23020,  39, { 22527,  22508,  22506,  22525}},
 { 23021,  39, { 22521,  22527,  22509,  22509}},
 { 23022,  39, { 22521,  22502,  22508,  22527}},
 { 23023,  39, { 22345,  22312,  22308,  22343}},
 { 23024,  39, { 22347,  22316,  22312,  22345}},
 { 23025,  39, { 22349,  22320,  22316,  22347}},
 { 23026,  39, { 22413,  22415,  22528,  22528}},
 { 23027,  39, { 22415,  22417,  22528,  22528}},
 { 23028,  39, { 22417,  22386,  22528,  22528}},
 { 23029,  39, { 22386,  22385,  22528,  22528}},
 { 23030,  39, { 22385,  22387,  22528,  22528}},
 { 23031,  39, { 22387,  22389,  22528,  22528}},
 { 23032,  39, { 22389,  22391,  22528,  22528}},
 { 23033,  39, { 22391,  22528,  22393,  22393}},
 { 23034,  39, { 22528,  22395,  22393,  22393}},
 { 23035,  39, { 22528,  22397,  22395,  22395}},
 { 23036,  39, { 22528,  22399,  22397,  22397}},
 { 23037,  39, { 22528,  22401,  22399,  22399}},
 { 23038,  39, { 22528,  22404,  22401,  22401}},
 { 23039,  39, { 22528,  22405,  22404,  22404}},
 { 23040,  39, { 22528,  22408,  22405,  22405}},
 { 23041,  39, { 22528,  22409,  22408,  22408}},
 { 23042,  39, { 22528,  22411,  22409,  22409}},
 { 23043,  39, { 22528,  22413,  22411,  22411}},
 { 23044,  39, { 22409,  22344,  22342,  22411}},
 { 23045,  39, { 22341,  22342,  22344,  22343}},
 { 23046,  39, { 22408,  22346,  22344,  22409}},
 { 23047,  39, { 22343,  22344,  22346,  22345}},
 { 23048,  39, { 22405,  22348,  22346,  22408}},
 { 23049,  39, { 22345,  22346,  22348,  22347}},
 { 23050,  39, { 22404,  22350,  22348,  22405}},
 { 23051,  39, { 22347,  22348,  22350,  22349}},
 { 23052,  39, { 22401,  22352,  22350,  22404}},
 { 23053,  39, { 22349,  22350,  22352,  22351}},
 { 23054,  39, { 22399,  22354,  22352,  22401}},
 { 23055,  39, { 22351,  22352,  22354,  22353}},
 { 23056,  39, { 22353,  22354,  22360,  22359}},
 { 23057,  39, { 22397,  22360,  22354,  22399}},
 { 23058,  39, { 22359,  22360,  22362,  22361}},
 { 23059,  39, { 22395,  22362,  22360,  22397}},
 { 23060,  39, { 22361,  22362,  22364,  22363}},
 { 23061,  39, { 22393,  22364,  22362,  22395}},
 { 23062,  39, { 22363,  22364,  22358,  22357}},
 { 23063,  39, { 22391,  22358,  22364,  22393}},
 { 23064,  39, { 22357,  22358,  22356,  22355}},
 { 23065,  39, { 22389,  22356,  22358,  22391}},
 { 23066,  39, { 22355,  22356,  22331,  22330}},
 { 23067,  39, { 22387,  22331,  22356,  22389}},
 { 23068,  39, { 22330,  22331,  22332,  22329}},
 { 23069,  39, { 22385,  22332,  22331,  22387}},
 { 23070,  39, { 22329,  22332,  22334,  22333}},
 { 23071,  39, { 22386,  22334,  22332,  22385}},
 { 23072,  39, { 22333,  22334,  22336,  22335}},
 { 23073,  39, { 22417,  22336,  22334,  22386}},
 { 23074,  39, { 22415,  22338,  22336,  22417}},
 { 23075,  39, { 22335,  22336,  22338,  22337}},
 { 23076,  39, { 22413,  22340,  22338,  22415}},
 { 23077,  39, { 22337,  22338,  22340,  22339}},
 { 23078,  39, { 22411,  22342,  22340,  22413}},
 { 23079,  39, { 22339,  22340,  22342,  22341}},
 { 23206,  44, { 22565,  22566,  22567,  22568}},
 { 23207,  44, { 22569,  22565,  22568,  22570}},
 { 23208,  44, { 22566,  22571,  22572,  22567}},
 { 23209,  44, { 22571,  22573,  22574,  22572}},
 { 23210,  44, { 22573,  22575,  22576,  22574}},
 { 23211,  44, { 22568,  22567,  22577,  22578}},
 { 23212,  44, { 22570,  22568,  22578,  22579}},
 { 23213,  44, { 22567,  22572,  22580,  22577}},
 { 23214,  44, { 22572,  22574,  22581,  22580}},
 { 23215,  44, { 22574,  22576,  22582,  22581}},
 { 23216,  44, { 22578,  22577,  22583,  22584}},
 { 23217,  44, { 22579,  22578,  22584,  22585}},
 { 23218,  44, { 22577,  22580,  22586,  22583}},
 { 23219,  44, { 22580,  22581,  22587,  22586}},
 { 23220,  44, { 22581,  22582,  22588,  22587}},
 { 23221,  44, { 22584,  22583,  22589,  22590}},
 { 23222,  44, { 22585,  22584,  22590,  22591}},
 { 23223,  44, { 22583,  22586,  22592,  22589}},
 { 23224,  44, { 22586,  22587,  22593,  22592}},
 { 23225,  44, { 22587,  22588,  22594,  22593}},
 { 23226,  44, { 22595,  22596,  22566,  22565}},
 { 23227,  44, { 22595,  22565,  22569,  22597}},
 { 23228,  44, { 22596,  22598,  22571,  22566}},
 { 23229,  44, { 22598,  22599,  22573,  22571}},
 { 23230,  44, { 22600,  22601,  22596,  22595}},
 { 23231,  44, { 22600,  22595,  22597,  22602}},
 { 23232,  44, { 22601,  22603,  22598,  22596}},
 { 23233,  44, { 22603,  22604,  22599,  22598}},
 { 23234,  44, { 22605,  22606,  22601,  22600}},
 { 23235,  44, { 22605,  22600,  22602,  22607}},
 { 23236,  44, { 22606,  22608,  22603,  22601}},
 { 23237,  44, { 22608,  22609,  22604,  22603}},
 { 23238,  44, { 22610,  22611,  22606,  22605}},
 { 23239,  44, { 22610,  22605,  22607,  22612}},
 { 23240,  44, { 22611,  22613,  22608,  22606}},
 { 23241,  44, { 22613,  22614,  22609,  22608}},
 { 23242,  44, { 22597,  22569,  22570,  22615}},
 { 23243,  44, { 22602,  22597,  22615,  22616}},
 { 23244,  44, { 22607,  22602,  22616,  22617}},
 { 23245,  44, { 22612,  22607,  22617,  22618}},
 { 23246,  44, { 22615,  22570,  22579,  22619}},
 { 23247,  44, { 22618,  22617,  22620,  22621}},
 { 23248,  44, { 22619,  22579,  22585,  22622}},
 { 23249,  44, { 22621,  22620,  22623,  22624}},
 { 23250,  44, { 22622,  22585,  22591,  22625}},
 { 23251,  44, { 22626,  22622,  22625,  22627}},
 { 23252,  44, { 22623,  22626,  22627,  22628}},
 { 23253,  44, { 22624,  22623,  22628,  22629}},
 { 23254,  44, { 22625,  22591,  22590,  22630}},
 { 23255,  44, { 22590,  22589,  22631,  22630}},
 { 23256,  44, { 22589,  22592,  22632,  22631}},
 { 23257,  44, { 22592,  22593,  22633,  22632}},
 { 23258,  44, { 22627,  22625,  22630,  22634}},
 { 23259,  44, { 22630,  22631,  22635,  22634}},
 { 23260,  44, { 22631,  22632,  22636,  22635}},
 { 23261,  44, { 22632,  22633,  22637,  22636}},
 { 23262,  44, { 22628,  22627,  22634,  22638}},
 { 23263,  44, { 22634,  22635,  22639,  22638}},
 { 23264,  44, { 22635,  22636,  22640,  22639}},
 { 23265,  44, { 22636,  22637,  22641,  22640}},
 { 23266,  44, { 22629,  22628,  22638,  22642}},
 { 23267,  44, { 22638,  22639,  22643,  22642}},
 { 23268,  44, { 22639,  22640,  22644,  22643}},
 { 23269,  44, { 22640,  22641,  22645,  22644}},
 { 23270,  44, { 22611,  22610,  22646,  22647}},
 { 23271,  44, { 22610,  22612,  22618,  22646}},
 { 23272,  44, { 22613,  22611,  22647,  22648}},
 { 23273,  44, { 22649,  22614,  22650,  22651}},
 { 23274,  44, { 22614,  22613,  22648,  22650}},
 { 23275,  44, { 22647,  22646,  22652,  22653}},
 { 23276,  44, { 22646,  22618,  22621,  22652}},
 { 23277,  44, { 22648,  22647,  22653,  22654}},
 { 23278,  44, { 22651,  22650,  22655,  22656}},
 { 23279,  44, { 22650,  22648,  22654,  22655}},
 { 23280,  44, { 22653,  22652,  22657,  22658}},
 { 23281,  44, { 22652,  22621,  22624,  22657}},
 { 23282,  44, { 22654,  22653,  22658,  22659}},
 { 23283,  44, { 22656,  22655,  22660,  22661}},
 { 23284,  44, { 22655,  22654,  22659,  22660}},
 { 23285,  44, { 22658,  22657,  22642,  22643}},
 { 23286,  44, { 22657,  22624,  22629,  22642}},
 { 23287,  44, { 22659,  22658,  22643,  22644}},
 { 23288,  44, { 22661,  22660,  22645,  22662}},
 { 23289,  44, { 22660,  22659,  22644,  22645}},
 { 23290,  53, { 22604,  22663,  22664,  22599}},
 { 23291,  53, { 22609,  22665,  22663,  22604}},
 { 23292,  53, { 22633,  22666,  22667,  22637}},
 { 23293,  53, { 22637,  22667,  22668,  22641}},
 { 23294,  53, { 22664,  22669,  22670,  22671}},
 { 23295,  53, { 22669,  22672,  22673,  22670}},
 { 23296,  53, { 22663,  22674,  22669,  22664}},
 { 23297,  53, { 22674,  22675,  22672,  22669}},
 { 23298,  53, { 22665,  22676,  22674,  22663}},
 { 23299,  53, { 22676,  22677,  22675,  22674}},
 { 23300,  53, { 22678,  22679,  22676,  22665}},
 { 23301,  53, { 22679,  22680,  22677,  22676}},
 { 23302,  53, { 22681,  22682,  22683,  22666}},
 { 23303,  53, { 22682,  22684,  22685,  22683}},
 { 23304,  53, { 22666,  22683,  22686,  22667}},
 { 23305,  53, { 22683,  22685,  22687,  22686}},
 { 23306,  53, { 22667,  22686,  22688,  22668}},
 { 23307,  53, { 22686,  22687,  22689,  22688}},
 { 23308,  53, { 22668,  22688,  22690,  22691}},
 { 23309,  53, { 22688,  22689,  22692,  22690}},
 { 23310,  53, { 22673,  22672,  22693,  22694}},
 { 23311,  53, { 22672,  22675,  22695,  22693}},
 { 23312,  53, { 22675,  22677,  22696,  22695}},
 { 23313,  53, { 22677,  22680,  22697,  22696}},
 { 23314,  53, { 22694,  22693,  22698,  22699}},
 { 23315,  53, { 22693,  22695,  22700,  22698}},
 { 23316,  53, { 22695,  22696,  22701,  22700}},
 { 23317,  53, { 22696,  22697,  22702,  22701}},
 { 23318,  53, { 22699,  22698,  22703,  22704}},
 { 23319,  53, { 22698,  22700,  22705,  22703}},
 { 23320,  53, { 22700,  22701,  22706,  22705}},
 { 23321,  53, { 22701,  22702,  22707,  22706}},
 { 23322,  53, { 22704,  22703,  22685,  22684}},
 { 23323,  53, { 22703,  22705,  22687,  22685}},
 { 23324,  53, { 22705,  22706,  22689,  22687}},
 { 23325,  53, { 22706,  22707,  22692,  22689}},
 { 23326,  53, { 22691,  22668,  22641,  22641}},
 { 23327,  44, { 22645,  22662,  22641,  22641}},
 { 23328,  44, { 22609,  22614,  22649,  22649}},
 { 23329,  53, { 22678,  22665,  22609,  22609}},
 { 23330,  53, { 22633,  22681,  22666,  22666}},
 { 23331,  44, { 22594,  22593,  22633,  22633}},
 { 23332,  53, { 22671,  22664,  22599,  22599}},
 { 23333,  44, { 22599,  22573,  22575,  22575}},
 { 23334,  46, { 22708,  22709,  22710,  22711}},
 { 23335,  46, { 22711,  22710,  22712,  22713}},
 { 23336,  46, { 22713,  22712,  22714,  22715}},
 { 23337,  46, { 22715,  22714,  22716,  22717}},
 { 23338,  46, { 22717,  22716,  22718,  22719}},
 { 23339,  46, { 22719,  22718,  22720,  22721}},
 { 23340,  46, { 22721,  22720,  22722,  22723}},
 { 23341,  46, { 22723,  22722,  22724,  22725}},
 { 23342,  46, { 22725,  22724,  22726,  22727}},
 { 23343,  46, { 22727,  22726,  22728,  22729}},
 { 23344,  46, { 22729,  22728,  22730,  22731}},
 { 23345,  46, { 22731,  22730,  22732,  22733}},
 { 23346,  46, { 22733,  22732,  22734,  22735}},
 { 23347,  46, { 22734,  22736,  22737,  22735}},
 { 23348,  46, { 22738,  22715,  22717,  22739}},
 { 23349,  46, { 22740,  22738,  22739,  22741}},
 { 23350,  46, { 22742,  22740,  22741,  22743}},
 { 23351,  46, { 22744,  22742,  22743,  22745}},
 { 23352,  46, { 22739,  22717,  22719,  22746}},
 { 23353,  46, { 22741,  22739,  22746,  22747}},
 { 23354,  46, { 22743,  22741,  22747,  22748}},
 { 23355,  46, { 22745,  22743,  22748,  22749}},
 { 23356,  46, { 22746,  22719,  22721,  22750}},
 { 23357,  46, { 22747,  22746,  22750,  22751}},
 { 23358,  46, { 22748,  22747,  22751,  22752}},
 { 23359,  46, { 22749,  22748,  22752,  22753}},
 { 23360,  46, { 22750,  22721,  22723,  22754}},
 { 23361,  46, { 22751,  22750,  22754,  22755}},
 { 23362,  46, { 22752,  22751,  22755,  22756}},
 { 23363,  46, { 22753,  22752,  22756,  22757}},
 { 23364,  46, { 22754,  22723,  22725,  22758}},
 { 23365,  46, { 22755,  22754,  22758,  22759}},
 { 23366,  46, { 22756,  22755,  22759,  22760}},
 { 23367,  46, { 22757,  22756,  22760,  22761}},
 { 23368,  46, { 22758,  22725,  22727,  22762}},
 { 23369,  46, { 22759,  22758,  22762,  22763}},
 { 23370,  46, { 22760,  22759,  22763,  22764}},
 { 23371,  46, { 22761,  22760,  22764,  22765}},
 { 23372,  46, { 22762,  22727,  22729,  22766}},
 { 23373,  46, { 22763,  22762,  22766,  22767}},
 { 23374,  46, { 22764,  22763,  22767,  22768}},
 { 23375,  46, { 22765,  22764,  22768,  22769}},
 { 23376,  46, { 22766,  22729,  22731,  22770}},
 { 23377,  46, { 22767,  22766,  22770,  22771}},
 { 23378,  46, { 22768,  22767,  22771,  22772}},
 { 23379,  46, { 22769,  22768,  22772,  22773}},
 { 23380,  46, { 22770,  22731,  22733,  22774}},
 { 23381,  46, { 22771,  22770,  22774,  22775}},
 { 23382,  46, { 22772,  22771,  22775,  22776}},
 { 23383,  46, { 22773,  22772,  22776,  22777}},
 { 23384,  46, { 22774,  22733,  22735,  22778}},
 { 23385,  46, { 22735,  22737,  22779,  22778}},
 { 23386,  46, { 22775,  22774,  22778,  22780}},
 { 23387,  46, { 22778,  22779,  22781,  22780}},
 { 23388,  46, { 22776,  22775,  22780,  22782}},
 { 23389,  46, { 22780,  22781,  22783,  22782}},
 { 23390,  46, { 22777,  22776,  22782,  22784}},
 { 23391,  46, { 22782,  22783,  22785,  22784}},
 { 23392,  46, { 22786,  22787,  22788,  22789}},
 { 23393,  46, { 22789,  22788,  22790,  22791}},
 { 23394,  46, { 22791,  22790,  22744,  22792}},
 { 23395,  46, { 22792,  22744,  22745,  22793}},
 { 23396,  46, { 22793,  22745,  22749,  22794}},
 { 23397,  46, { 22794,  22749,  22753,  22795}},
 { 23398,  46, { 22795,  22753,  22757,  22796}},
 { 23399,  46, { 22796,  22757,  22761,  22797}},
 { 23400,  46, { 22797,  22761,  22765,  22798}},
 { 23401,  46, { 22798,  22765,  22769,  22799}},
 { 23402,  46, { 22799,  22769,  22773,  22800}},
 { 23403,  46, { 22800,  22773,  22777,  22801}},
 { 23404,  46, { 22801,  22777,  22784,  22802}},
 { 23405,  46, { 22784,  22785,  22803,  22802}},
 { 23406,  46, { 22709,  22804,  22805,  22710}},
 { 23407,  46, { 22804,  22806,  22807,  22805}},
 { 23408,  46, { 22806,  22808,  22809,  22807}},
 { 23409,  46, { 22808,  22810,  22811,  22809}},
 { 23410,  46, { 22710,  22805,  22812,  22712}},
 { 23411,  46, { 22805,  22807,  22813,  22812}},
 { 23412,  46, { 22807,  22809,  22814,  22813}},
 { 23413,  46, { 22809,  22811,  22815,  22814}},
 { 23414,  46, { 22712,  22812,  22816,  22714}},
 { 23415,  46, { 22812,  22813,  22817,  22816}},
 { 23416,  46, { 22813,  22814,  22818,  22817}},
 { 23417,  46, { 22814,  22815,  22819,  22818}},
 { 23418,  46, { 22714,  22816,  22820,  22716}},
 { 23419,  46, { 22816,  22817,  22821,  22820}},
 { 23420,  46, { 22817,  22818,  22822,  22821}},
 { 23421,  46, { 22818,  22819,  22823,  22822}},
 { 23422,  46, { 22716,  22820,  22824,  22718}},
 { 23423,  46, { 22820,  22821,  22825,  22824}},
 { 23424,  46, { 22821,  22822,  22826,  22825}},
 { 23425,  46, { 22822,  22823,  22827,  22826}},
 { 23426,  46, { 22718,  22824,  22828,  22720}},
 { 23427,  46, { 22824,  22825,  22829,  22828}},
 { 23428,  46, { 22825,  22826,  22830,  22829}},
 { 23429,  46, { 22826,  22827,  22831,  22830}},
 { 23430,  46, { 22720,  22828,  22832,  22722}},
 { 23431,  46, { 22828,  22829,  22833,  22832}},
 { 23432,  46, { 22829,  22830,  22834,  22833}},
 { 23433,  46, { 22830,  22831,  22835,  22834}},
 { 23434,  46, { 22722,  22832,  22836,  22724}},
 { 23435,  46, { 22832,  22833,  22837,  22836}},
 { 23436,  46, { 22833,  22834,  22838,  22837}},
 { 23437,  46, { 22834,  22835,  22839,  22838}},
 { 23438,  46, { 22724,  22836,  22840,  22726}},
 { 23439,  46, { 22836,  22837,  22841,  22840}},
 { 23440,  46, { 22837,  22838,  22842,  22841}},
 { 23441,  46, { 22838,  22839,  22843,  22842}},
 { 23442,  46, { 22726,  22840,  22844,  22728}},
 { 23443,  46, { 22840,  22841,  22845,  22844}},
 { 23444,  46, { 22841,  22842,  22846,  22845}},
 { 23445,  46, { 22842,  22843,  22847,  22846}},
 { 23446,  46, { 22728,  22844,  22848,  22730}},
 { 23447,  46, { 22844,  22845,  22849,  22848}},
 { 23448,  46, { 22845,  22846,  22850,  22849}},
 { 23449,  46, { 22846,  22847,  22851,  22850}},
 { 23450,  46, { 22730,  22848,  22852,  22732}},
 { 23451,  46, { 22848,  22849,  22853,  22852}},
 { 23452,  46, { 22849,  22850,  22854,  22853}},
 { 23453,  46, { 22850,  22851,  22855,  22854}},
 { 23454,  46, { 22732,  22852,  22856,  22734}},
 { 23455,  46, { 22734,  22856,  22857,  22736}},
 { 23456,  46, { 22852,  22853,  22858,  22856}},
 { 23457,  46, { 22856,  22858,  22859,  22857}},
 { 23458,  46, { 22853,  22854,  22860,  22858}},
 { 23459,  46, { 22858,  22860,  22861,  22859}},
 { 23460,  46, { 22854,  22855,  22862,  22860}},
 { 23461,  46, { 22860,  22862,  22863,  22861}},
 { 23462,  46, { 22708,  22864,  22804,  22709}},
 { 23463,  46, { 22864,  22865,  22806,  22804}},
 { 23464,  46, { 22865,  22866,  22808,  22806}},
 { 23465,  46, { 22866,  22867,  22810,  22808}},
 { 23466,  46, { 22736,  22857,  22868,  22737}},
 { 23467,  46, { 22857,  22859,  22869,  22868}},
 { 23468,  46, { 22859,  22861,  22870,  22869}},
 { 23469,  46, { 22861,  22863,  22871,  22870}},
 { 23470,  46, { 22737,  22868,  22872,  22779}},
 { 23471,  46, { 22868,  22869,  22873,  22872}},
 { 23472,  46, { 22869,  22870,  22874,  22873}},
 { 23473,  46, { 22870,  22871,  22875,  22874}},
 { 23474,  46, { 22779,  22872,  22876,  22781}},
 { 23475,  46, { 22872,  22873,  22877,  22876}},
 { 23476,  46, { 22873,  22874,  22878,  22877}},
 { 23477,  46, { 22874,  22875,  22879,  22878}},
 { 23478,  46, { 22781,  22876,  22880,  22783}},
 { 23479,  46, { 22876,  22877,  22881,  22880}},
 { 23480,  46, { 22877,  22878,  22882,  22881}},
 { 23481,  46, { 22878,  22879,  22883,  22882}},
 { 23482,  46, { 22783,  22880,  22884,  22785}},
 { 23483,  46, { 22880,  22881,  22885,  22884}},
 { 23484,  46, { 22881,  22882,  22886,  22885}},
 { 23485,  46, { 22882,  22883,  22887,  22886}},
 { 23486,  46, { 22786,  22888,  22889,  22787}},
 { 23487,  46, { 22888,  22890,  22891,  22889}},
 { 23488,  46, { 22890,  22892,  22893,  22891}},
 { 23489,  46, { 22892,  22894,  22895,  22893}},
 { 23490,  46, { 22785,  22884,  22896,  22803}},
 { 23491,  46, { 22884,  22885,  22897,  22896}},
 { 23492,  46, { 22885,  22886,  22898,  22897}},
 { 23493,  46, { 22886,  22887,  22899,  22898}},
 { 23494,  46, { 22888,  22786,  22789,  22900}},
 { 23495,  46, { 22890,  22888,  22900,  22901}},
 { 23496,  46, { 22892,  22890,  22901,  22902}},
 { 23497,  46, { 22894,  22892,  22902,  22903}},
 { 23498,  46, { 22900,  22789,  22791,  22904}},
 { 23499,  46, { 22901,  22900,  22904,  22905}},
 { 23500,  46, { 22902,  22901,  22905,  22906}},
 { 23501,  46, { 22903,  22902,  22906,  22907}},
 { 23502,  46, { 22904,  22791,  22792,  22908}},
 { 23503,  46, { 22905,  22904,  22908,  22909}},
 { 23504,  46, { 22906,  22905,  22909,  22910}},
 { 23505,  46, { 22907,  22906,  22910,  22911}},
 { 23506,  46, { 22908,  22792,  22793,  22912}},
 { 23507,  46, { 22909,  22908,  22912,  22913}},
 { 23508,  46, { 22910,  22909,  22913,  22914}},
 { 23509,  46, { 22911,  22910,  22914,  22915}},
 { 23510,  46, { 22912,  22793,  22794,  22916}},
 { 23511,  46, { 22913,  22912,  22916,  22917}},
 { 23512,  46, { 22914,  22913,  22917,  22918}},
 { 23513,  46, { 22915,  22914,  22918,  22919}},
 { 23514,  46, { 22916,  22794,  22795,  22920}},
 { 23515,  46, { 22917,  22916,  22920,  22921}},
 { 23516,  46, { 22918,  22917,  22921,  22922}},
 { 23517,  46, { 22919,  22918,  22922,  22923}},
 { 23518,  46, { 22920,  22795,  22796,  22924}},
 { 23519,  46, { 22921,  22920,  22924,  22925}},
 { 23520,  46, { 22922,  22921,  22925,  22926}},
 { 23521,  46, { 22923,  22922,  22926,  22927}},
 { 23522,  46, { 22924,  22796,  22797,  22928}},
 { 23523,  46, { 22925,  22924,  22928,  22929}},
 { 23524,  46, { 22926,  22925,  22929,  22930}},
 { 23525,  46, { 22927,  22926,  22930,  22931}},
 { 23526,  46, { 22928,  22797,  22798,  22932}},
 { 23527,  46, { 22929,  22928,  22932,  22933}},
 { 23528,  46, { 22930,  22929,  22933,  22934}},
 { 23529,  46, { 22931,  22930,  22934,  22935}},
 { 23530,  46, { 22932,  22798,  22799,  22936}},
 { 23531,  46, { 22933,  22932,  22936,  22937}},
 { 23532,  46, { 22934,  22933,  22937,  22938}},
 { 23533,  46, { 22935,  22934,  22938,  22939}},
 { 23534,  46, { 22936,  22799,  22800,  22940}},
 { 23535,  46, { 22937,  22936,  22940,  22941}},
 { 23536,  46, { 22938,  22937,  22941,  22942}},
 { 23537,  46, { 22939,  22938,  22942,  22943}},
 { 23538,  46, { 22940,  22800,  22801,  22944}},
 { 23539,  46, { 22941,  22940,  22944,  22945}},
 { 23540,  46, { 22942,  22941,  22945,  22946}},
 { 23541,  46, { 22943,  22942,  22946,  22947}},
 { 23542,  46, { 22944,  22801,  22802,  22948}},
 { 23543,  46, { 22803,  22896,  22948,  22802}},
 { 23544,  46, { 22945,  22944,  22948,  22949}},
 { 23545,  46, { 22896,  22897,  22949,  22948}},
 { 23546,  46, { 22946,  22945,  22949,  22950}},
 { 23547,  46, { 22897,  22898,  22950,  22949}},
 { 23548,  46, { 22947,  22946,  22950,  22951}},
 { 23549,  46, { 22898,  22899,  22951,  22950}},
 { 23550,  46, { 22810,  22867,  22952,  22811}},
 { 23551,  46, { 22811,  22952,  22953,  22815}},
 { 23552,  46, { 22815,  22953,  22954,  22819}},
 { 23553,  46, { 22819,  22954,  22955,  22823}},
 { 23554,  46, { 22823,  22955,  22956,  22827}},
 { 23555,  46, { 22827,  22956,  22957,  22831}},
 { 23556,  46, { 22831,  22957,  22958,  22835}},
 { 23557,  46, { 22835,  22958,  22959,  22839}},
 { 23558,  46, { 22839,  22959,  22960,  22843}},
 { 23559,  46, { 22843,  22960,  22961,  22847}},
 { 23560,  46, { 22847,  22961,  22962,  22851}},
 { 23561,  46, { 22851,  22962,  22963,  22855}},
 { 23562,  46, { 22855,  22963,  22964,  22862}},
 { 23563,  46, { 22863,  22862,  22964,  22871}},
 { 23564,  46, { 22954,  22965,  22966,  22955}},
 { 23565,  46, { 22965,  22967,  22968,  22966}},
 { 23566,  46, { 22967,  22969,  22970,  22968}},
 { 23567,  46, { 22969,  22971,  22972,  22970}},
 { 23568,  46, { 22955,  22966,  22973,  22956}},
 { 23569,  46, { 22966,  22968,  22974,  22973}},
 { 23570,  46, { 22968,  22970,  22975,  22974}},
 { 23571,  46, { 22970,  22972,  22976,  22975}},
 { 23572,  46, { 22956,  22973,  22977,  22957}},
 { 23573,  46, { 22973,  22974,  22978,  22977}},
 { 23574,  46, { 22974,  22975,  22979,  22978}},
 { 23575,  46, { 22975,  22976,  22980,  22979}},
 { 23576,  46, { 22957,  22977,  22981,  22958}},
 { 23577,  46, { 22977,  22978,  22982,  22981}},
 { 23578,  46, { 22978,  22979,  22983,  22982}},
 { 23579,  46, { 22979,  22980,  22984,  22983}},
 { 23580,  46, { 22958,  22981,  22985,  22959}},
 { 23581,  46, { 22981,  22982,  22986,  22985}},
 { 23582,  46, { 22982,  22983,  22987,  22986}},
 { 23583,  46, { 22983,  22984,  22988,  22987}},
 { 23584,  46, { 22959,  22985,  22989,  22960}},
 { 23585,  46, { 22985,  22986,  22990,  22989}},
 { 23586,  46, { 22986,  22987,  22991,  22990}},
 { 23587,  46, { 22987,  22988,  22992,  22991}},
 { 23588,  46, { 22960,  22989,  22993,  22961}},
 { 23589,  46, { 22989,  22990,  22994,  22993}},
 { 23590,  46, { 22990,  22991,  22995,  22994}},
 { 23591,  46, { 22991,  22992,  22996,  22995}},
 { 23592,  46, { 22961,  22993,  22997,  22962}},
 { 23593,  46, { 22993,  22994,  22998,  22997}},
 { 23594,  46, { 22994,  22995,  22999,  22998}},
 { 23595,  46, { 22995,  22996,  23000,  22999}},
 { 23596,  46, { 22962,  22997,  23001,  22963}},
 { 23597,  46, { 22997,  22998,  23002,  23001}},
 { 23598,  46, { 22998,  22999,  23003,  23002}},
 { 23599,  46, { 22999,  23000,  23004,  23003}},
 { 23600,  46, { 22963,  23001,  23005,  22964}},
 { 23601,  46, { 22871,  22964,  23005,  22875}},
 { 23602,  46, { 23001,  23002,  23006,  23005}},
 { 23603,  46, { 22875,  23005,  23006,  22879}},
 { 23604,  46, { 23002,  23003,  23007,  23006}},
 { 23605,  46, { 22879,  23006,  23007,  22883}},
 { 23606,  46, { 23003,  23004,  23008,  23007}},
 { 23607,  46, { 22883,  23007,  23008,  22887}},
 { 23608,  46, { 22895,  22894,  22903,  23009}},
 { 23609,  46, { 23009,  22903,  22907,  23010}},
 { 23610,  46, { 23010,  22907,  22911,  22971}},
 { 23611,  46, { 22971,  22911,  22915,  22972}},
 { 23612,  46, { 22972,  22915,  22919,  22976}},
 { 23613,  46, { 22976,  22919,  22923,  22980}},
 { 23614,  46, { 22980,  22923,  22927,  22984}},
 { 23615,  46, { 22984,  22927,  22931,  22988}},
 { 23616,  46, { 22988,  22931,  22935,  22992}},
 { 23617,  46, { 22992,  22935,  22939,  22996}},
 { 23618,  46, { 22996,  22939,  22943,  23000}},
 { 23619,  46, { 23000,  22943,  22947,  23004}},
 { 23620,  46, { 23004,  22947,  22951,  23008}},
 { 23621,  46, { 22887,  23008,  22951,  22899}},
 { 23622,  46, { 23011,  22708,  22711,  23012}},
 { 23623,  46, { 23012,  22711,  22713,  23013}},
 { 23624,  46, { 23013,  22713,  22715,  23014}},
 { 23625,  46, { 22787,  23015,  23016,  22788}},
 { 23626,  46, { 22788,  23016,  23017,  22790}},
 { 23627,  46, { 22790,  23017,  23018,  22744}},
 { 23628,  46, { 22864,  22708,  23011,  23019}},
 { 23629,  46, { 22865,  22864,  23019,  23020}},
 { 23630,  46, { 22866,  22865,  23020,  23021}},
 { 23631,  46, { 22867,  22866,  23021,  23022}},
 { 23632,  46, { 23019,  23011,  23012,  23023}},
 { 23633,  46, { 23020,  23019,  23023,  23024}},
 { 23634,  46, { 23021,  23020,  23024,  23025}},
 { 23635,  46, { 23022,  23021,  23025,  23026}},
 { 23636,  46, { 23023,  23012,  23013,  23027}},
 { 23637,  46, { 23024,  23023,  23027,  23028}},
 { 23638,  46, { 23025,  23024,  23028,  23029}},
 { 23639,  46, { 23026,  23025,  23029,  23030}},
 { 23640,  46, { 23027,  23013,  23014,  23031}},
 { 23641,  46, { 23028,  23027,  23031,  23032}},
 { 23642,  46, { 23029,  23028,  23032,  23033}},
 { 23643,  46, { 23030,  23029,  23033,  23034}},
 { 23644,  46, { 22787,  22889,  23035,  23015}},
 { 23645,  46, { 22889,  22891,  23036,  23035}},
 { 23646,  46, { 22891,  22893,  23037,  23036}},
 { 23647,  46, { 22893,  22895,  23038,  23037}},
 { 23648,  46, { 23015,  23035,  23039,  23016}},
 { 23649,  46, { 23035,  23036,  23040,  23039}},
 { 23650,  46, { 23036,  23037,  23041,  23040}},
 { 23651,  46, { 23037,  23038,  23042,  23041}},
 { 23652,  46, { 23016,  23039,  23043,  23017}},
 { 23653,  46, { 23039,  23040,  23044,  23043}},
 { 23654,  46, { 23040,  23041,  23045,  23044}},
 { 23655,  46, { 23041,  23042,  23046,  23045}},
 { 23656,  46, { 23017,  23043,  23047,  23018}},
 { 23657,  46, { 23043,  23044,  23048,  23047}},
 { 23658,  46, { 23044,  23045,  23049,  23048}},
 { 23659,  46, { 23045,  23046,  23050,  23049}},
 { 23660,  46, { 22867,  23022,  23026,  22952}},
 { 23661,  46, { 22952,  23026,  23030,  22953}},
 { 23662,  46, { 22953,  23030,  23034,  22954}},
 { 23663,  46, { 23038,  22895,  23009,  23042}},
 { 23664,  46, { 23042,  23009,  23010,  23046}},
 { 23665,  46, { 23046,  23010,  22971,  23050}},
 { 23666,  48, { 23051,  23052,  23053,  23054}},
 { 23667,  48, { 23055,  23051,  23054,  23056}},
 { 23668,  48, { 23057,  23055,  23056,  23058}},
 { 23669,  48, { 23059,  23057,  23058,  23060}},
 { 23670,  48, { 23054,  23053,  23061,  23062}},
 { 23671,  48, { 23056,  23054,  23062,  23063}},
 { 23672,  48, { 23058,  23056,  23063,  23064}},
 { 23673,  48, { 23060,  23058,  23064,  23065}},
 { 23674,  48, { 23062,  23061,  23066,  23067}},
 { 23675,  48, { 23063,  23062,  23067,  23068}},
 { 23676,  48, { 23064,  23063,  23068,  23069}},
 { 23677,  48, { 23065,  23064,  23069,  23070}},
 { 23678,  48, { 23067,  23066,  23071,  23072}},
 { 23679,  48, { 23068,  23067,  23072,  23073}},
 { 23680,  48, { 23069,  23068,  23073,  23074}},
 { 23681,  48, { 23070,  23069,  23074,  23075}},
 { 23682,  48, { 23052,  23076,  23077,  23053}},
 { 23683,  48, { 23076,  23078,  23079,  23077}},
 { 23684,  48, { 23053,  23077,  23080,  23061}},
 { 23685,  48, { 23077,  23079,  23081,  23080}},
 { 23686,  48, { 23061,  23080,  23082,  23066}},
 { 23687,  48, { 23080,  23081,  23083,  23082}},
 { 23688,  48, { 23066,  23082,  23084,  23071}},
 { 23689,  48, { 23082,  23083,  23085,  23084}},
 { 23690,  48, { 23051,  23086,  23076,  23052}},
 { 23691,  48, { 23086,  23087,  23078,  23076}},
 { 23692,  48, { 23055,  23088,  23086,  23051}},
 { 23693,  48, { 23088,  23089,  23087,  23086}},
 { 23694,  48, { 23057,  23090,  23088,  23055}},
 { 23695,  48, { 23090,  23091,  23089,  23088}},
 { 23696,  48, { 23059,  23092,  23090,  23057}},
 { 23697,  48, { 23092,  23093,  23091,  23090}},
 { 23698,  48, { 23071,  23084,  23094,  23072}},
 { 23699,  48, { 23084,  23085,  23095,  23094}},
 { 23700,  48, { 23072,  23094,  23096,  23073}},
 { 23701,  48, { 23094,  23095,  23097,  23096}},
 { 23702,  48, { 23073,  23096,  23098,  23074}},
 { 23703,  48, { 23096,  23097,  23099,  23098}},
 { 23704,  48, { 23074,  23098,  23100,  23075}},
 { 23705,  48, { 23098,  23099,  23101,  23100}},
 { 23706,  48, { 23092,  23059,  23060,  23102}},
 { 23707,  48, { 23093,  23092,  23102,  23103}},
 { 23708,  48, { 23102,  23060,  23065,  23104}},
 { 23709,  48, { 23103,  23102,  23104,  23105}},
 { 23710,  48, { 23104,  23065,  23070,  23106}},
 { 23711,  48, { 23105,  23104,  23106,  23107}},
 { 23712,  48, { 23106,  23070,  23075,  23100}},
 { 23713,  48, { 23107,  23106,  23100,  23101}},
 { 23714,  48, { 23078,  23108,  23109,  23079}},
 { 23715,  48, { 23079,  23109,  23110,  23081}},
 { 23716,  48, { 23081,  23110,  23111,  23083}},
 { 23717,  48, { 23083,  23111,  23112,  23085}},
 { 23718,  48, { 23087,  23113,  23108,  23078}},
 { 23719,  48, { 23089,  23114,  23113,  23087}},
 { 23720,  48, { 23091,  23115,  23114,  23089}},
 { 23721,  48, { 23093,  23116,  23115,  23091}},
 { 23722,  48, { 23085,  23112,  23117,  23095}},
 { 23723,  48, { 23095,  23117,  23118,  23097}},
 { 23724,  48, { 23097,  23118,  23119,  23099}},
 { 23725,  48, { 23099,  23119,  23120,  23101}},
 { 23726,  48, { 23116,  23093,  23103,  23121}},
 { 23727,  48, { 23121,  23103,  23105,  23122}},
 { 23728,  48, { 23122,  23105,  23107,  23123}},
 { 23729,  48, { 23123,  23107,  23101,  23120}},
 { 23730,  48, { 23108,  23124,  23125,  23109}},
 { 23731,  48, { 23124,  23126,  23127,  23125}},
 { 23732,  48, { 23126,  23128,  23129,  23127}},
 { 23733,  48, { 23109,  23125,  23130,  23110}},
 { 23734,  48, { 23125,  23127,  23131,  23130}},
 { 23735,  48, { 23127,  23129,  23132,  23131}},
 { 23736,  48, { 23110,  23130,  23133,  23111}},
 { 23737,  48, { 23130,  23131,  23134,  23133}},
 { 23738,  48, { 23131,  23132,  23135,  23134}},
 { 23739,  48, { 23111,  23133,  23136,  23112}},
 { 23740,  48, { 23133,  23134,  23137,  23136}},
 { 23741,  48, { 23134,  23135,  23138,  23137}},
 { 23742,  48, { 23113,  23139,  23124,  23108}},
 { 23743,  48, { 23139,  23140,  23126,  23124}},
 { 23744,  48, { 23140,  23141,  23128,  23126}},
 { 23745,  48, { 23114,  23142,  23139,  23113}},
 { 23746,  48, { 23142,  23143,  23140,  23139}},
 { 23747,  48, { 23143,  23144,  23141,  23140}},
 { 23748,  48, { 23115,  23145,  23142,  23114}},
 { 23749,  48, { 23145,  23146,  23143,  23142}},
 { 23750,  48, { 23146,  23147,  23144,  23143}},
 { 23751,  48, { 23116,  23148,  23145,  23115}},
 { 23752,  48, { 23148,  23149,  23146,  23145}},
 { 23753,  48, { 23149,  23150,  23147,  23146}},
 { 23754,  48, { 23112,  23136,  23151,  23117}},
 { 23755,  48, { 23136,  23137,  23152,  23151}},
 { 23756,  48, { 23137,  23138,  23153,  23152}},
 { 23757,  48, { 23117,  23151,  23154,  23118}},
 { 23758,  48, { 23151,  23152,  23155,  23154}},
 { 23759,  48, { 23152,  23153,  23156,  23155}},
 { 23760,  48, { 23118,  23154,  23157,  23119}},
 { 23761,  48, { 23154,  23155,  23158,  23157}},
 { 23762,  48, { 23155,  23156,  23159,  23158}},
 { 23763,  48, { 23119,  23157,  23160,  23120}},
 { 23764,  48, { 23157,  23158,  23161,  23160}},
 { 23765,  48, { 23158,  23159,  23162,  23161}},
 { 23766,  48, { 23148,  23116,  23121,  23163}},
 { 23767,  48, { 23149,  23148,  23163,  23164}},
 { 23768,  48, { 23150,  23149,  23164,  23165}},
 { 23769,  48, { 23163,  23121,  23122,  23166}},
 { 23770,  48, { 23164,  23163,  23166,  23167}},
 { 23771,  48, { 23165,  23164,  23167,  23168}},
 { 23772,  48, { 23166,  23122,  23123,  23169}},
 { 23773,  48, { 23167,  23166,  23169,  23170}},
 { 23774,  48, { 23168,  23167,  23170,  23171}},
 { 23775,  48, { 23169,  23123,  23120,  23160}},
 { 23776,  48, { 23170,  23169,  23160,  23161}},
 { 23777,  48, { 23171,  23170,  23161,  23162}},
 { 23778,  48, { 23128,  23172,  23173,  23129}},
 { 23779,  48, { 23129,  23173,  23174,  23132}},
 { 23780,  48, { 23132,  23174,  23175,  23135}},
 { 23781,  48, { 23135,  23175,  23176,  23138}},
 { 23782,  48, { 23141,  23177,  23172,  23128}},
 { 23783,  48, { 23144,  23178,  23177,  23141}},
 { 23784,  48, { 23147,  23179,  23178,  23144}},
 { 23785,  48, { 23150,  23180,  23179,  23147}},
 { 23786,  48, { 23138,  23176,  23181,  23153}},
 { 23787,  48, { 23153,  23181,  23182,  23156}},
 { 23788,  48, { 23156,  23182,  23183,  23159}},
 { 23789,  48, { 23159,  23183,  23184,  23162}},
 { 23790,  48, { 23180,  23150,  23165,  23185}},
 { 23791,  48, { 23185,  23165,  23168,  23186}},
 { 23792,  48, { 23186,  23168,  23171,  23187}},
 { 23793,  48, { 23187,  23171,  23162,  23184}},
 { 23794,  48, { 23172,  23188,  23189,  23173}},
 { 23795,  48, { 23188,  23190,  23191,  23189}},
 { 23796,  48, { 23173,  23189,  23192,  23174}},
 { 23797,  48, { 23189,  23191,  23193,  23192}},
 { 23798,  48, { 23174,  23192,  23194,  23175}},
 { 23799,  48, { 23192,  23193,  23195,  23194}},
 { 23800,  48, { 23175,  23194,  23196,  23176}},
 { 23801,  48, { 23194,  23195,  23197,  23196}},
 { 23802,  48, { 23177,  23198,  23188,  23172}},
 { 23803,  48, { 23198,  23199,  23190,  23188}},
 { 23804,  48, { 23178,  23200,  23198,  23177}},
 { 23805,  48, { 23200,  23201,  23199,  23198}},
 { 23806,  48, { 23179,  23202,  23200,  23178}},
 { 23807,  48, { 23202,  23203,  23201,  23200}},
 { 23808,  48, { 23180,  23204,  23202,  23179}},
 { 23809,  48, { 23204,  23205,  23203,  23202}},
 { 23810,  48, { 23176,  23196,  23206,  23181}},
 { 23811,  48, { 23196,  23197,  23207,  23206}},
 { 23812,  48, { 23181,  23206,  23208,  23182}},
 { 23813,  48, { 23206,  23207,  23209,  23208}},
 { 23814,  48, { 23182,  23208,  23210,  23183}},
 { 23815,  48, { 23208,  23209,  23211,  23210}},
 { 23816,  48, { 23183,  23210,  23212,  23184}},
 { 23817,  48, { 23210,  23211,  23213,  23212}},
 { 23818,  48, { 23204,  23180,  23185,  23214}},
 { 23819,  48, { 23205,  23204,  23214,  23215}},
 { 23820,  48, { 23214,  23185,  23186,  23216}},
 { 23821,  48, { 23215,  23214,  23216,  23217}},
 { 23822,  48, { 23216,  23186,  23187,  23218}},
 { 23823,  48, { 23217,  23216,  23218,  23219}},
 { 23824,  48, { 23218,  23187,  23184,  23212}},
 { 23825,  48, { 23219,  23218,  23212,  23213}},
 { 23826,  48, { 23190,  23199,  23220,  23191}},
 { 23827,  48, { 23199,  23201,  23221,  23220}},
 { 23828,  48, { 23201,  23203,  23222,  23221}},
 { 23829,  48, { 23203,  23205,  23215,  23222}},
 { 23830,  48, { 23191,  23220,  23223,  23193}},
 { 23831,  48, { 23220,  23221,  23224,  23223}},
 { 23832,  48, { 23221,  23222,  23225,  23224}},
 { 23833,  48, { 23222,  23215,  23217,  23225}},
 { 23834,  48, { 23193,  23223,  23226,  23195}},
 { 23835,  48, { 23223,  23224,  23227,  23226}},
 { 23836,  48, { 23224,  23225,  23228,  23227}},
 { 23837,  48, { 23225,  23217,  23219,  23228}},
 { 23838,  48, { 23195,  23226,  23207,  23197}},
 { 23839,  48, { 23226,  23227,  23209,  23207}},
 { 23840,  48, { 23227,  23228,  23211,  23209}},
 { 23841,  48, { 23228,  23219,  23213,  23211}},
 { 23842,  41, { 23229,  23230,  23231,  23232}},
 { 23843,  41, { 23231,  23230,  23233,  23234}},
 { 23844,  41, { 23235,  23234,  23233,  23236}},
 { 23845,  41, { 23237,  23238,  23230,  23229}},
 { 23846,  41, { 23230,  23238,  23239,  23233}},
 { 23847,  41, { 23236,  23233,  23239,  23240}},
 { 23848,  41, { 23241,  23242,  23238,  23237}},
 { 23849,  41, { 23238,  23242,  23243,  23239}},
 { 23850,  41, { 23240,  23239,  23243,  23244}},
 { 23851,  41, { 23245,  23246,  23242,  23241}},
 { 23852,  41, { 23242,  23246,  23247,  23243}},
 { 23853,  41, { 23244,  23243,  23247,  23248}},
 { 23854,  41, { 23249,  23250,  23246,  23245}},
 { 23855,  41, { 23246,  23250,  23251,  23247}},
 { 23856,  41, { 23248,  23247,  23251,  23252}},
 { 23857,  41, { 23253,  23254,  23250,  23249}},
 { 23858,  41, { 23250,  23254,  23255,  23251}},
 { 23859,  41, { 23252,  23251,  23255,  23256}},
 { 23860,  41, { 23257,  23258,  23254,  23253}},
 { 23861,  41, { 23254,  23258,  23259,  23255}},
 { 23862,  41, { 23256,  23255,  23259,  23260}},
 { 23863,  41, { 23261,  23262,  23258,  23257}},
 { 23864,  41, { 23258,  23262,  23263,  23259}},
 { 23865,  41, { 23260,  23259,  23263,  23264}},
 { 23866,  41, { 23232,  23231,  23265,  23266}},
 { 23867,  41, { 23265,  23231,  23234,  23267}},
 { 23868,  41, { 23268,  23267,  23234,  23235}},
 { 23869,  41, { 23266,  23265,  23269,  23270}},
 { 23870,  41, { 23269,  23265,  23267,  23271}},
 { 23871,  41, { 23272,  23271,  23267,  23268}},
 { 23872,  41, { 23270,  23269,  23273,  23274}},
 { 23873,  41, { 23273,  23269,  23271,  23275}},
 { 23874,  41, { 23276,  23275,  23271,  23272}},
 { 23875,  41, { 23274,  23273,  23277,  23278}},
 { 23876,  41, { 23277,  23273,  23275,  23279}},
 { 23877,  41, { 23280,  23279,  23275,  23276}},
 { 23878,  41, { 23278,  23277,  23281,  23282}},
 { 23879,  41, { 23281,  23277,  23279,  23283}},
 { 23880,  41, { 23284,  23283,  23279,  23280}},
 { 23881,  41, { 23282,  23281,  23285,  23286}},
 { 23882,  41, { 23285,  23281,  23283,  23287}},
 { 23883,  41, { 23288,  23287,  23283,  23284}},
 { 23884,  41, { 23286,  23285,  23289,  23290}},
 { 23885,  41, { 23289,  23285,  23287,  23291}},
 { 23886,  41, { 23292,  23291,  23287,  23288}},
 { 23887,  41, { 23290,  23289,  23293,  23294}},
 { 23888,  41, { 23293,  23289,  23291,  23295}},
 { 23889,  41, { 23296,  23295,  23291,  23292}},
 { 23890,  41, { 23294,  23293,  23297,  23298}},
 { 23891,  41, { 23297,  23293,  23295,  23299}},
 { 23892,  41, { 23300,  23299,  23295,  23296}},
 { 23893,  41, { 23298,  23297,  23262,  23261}},
 { 23894,  41, { 23262,  23297,  23299,  23263}},
 { 23895,  41, { 23264,  23263,  23299,  23300}},
 { 23896,  41, { 23301,  23302,  23244,  23240}},
 { 23897,  41, { 23302,  23301,  23303,  23304}},
 { 23898,  41, { 23305,  23301,  23240,  23236}},
 { 23899,  41, { 23301,  23305,  23306,  23303}},
 { 23900,  41, { 23307,  23305,  23236,  23235}},
 { 23901,  41, { 23305,  23307,  23308,  23306}},
 { 23902,  41, { 23309,  23307,  23235,  23268}},
 { 23903,  41, { 23307,  23309,  23310,  23308}},
 { 23904,  41, { 23311,  23309,  23268,  23272}},
 { 23905,  41, { 23309,  23311,  23312,  23310}},
 { 23906,  41, { 23313,  23311,  23272,  23276}},
 { 23907,  41, { 23311,  23313,  23314,  23312}},
 { 23908,  41, { 23315,  23313,  23276,  23280}},
 { 23909,  41, { 23313,  23315,  23316,  23314}},
 { 23910,  41, { 23284,  23317,  23315,  23280}},
 { 23911,  41, { 23315,  23317,  23318,  23316}},
 { 23912,  41, { 23288,  23319,  23317,  23284}},
 { 23913,  41, { 23317,  23319,  23320,  23318}},
 { 23914,  41, { 23288,  23292,  23321,  23319}},
 { 23915,  41, { 23319,  23321,  23322,  23320}},
 { 23916,  41, { 23296,  23323,  23321,  23292}},
 { 23917,  41, { 23321,  23323,  23324,  23322}},
 { 23918,  41, { 23325,  23300,  23296,  23323}},
 { 23919,  41, { 23325,  23323,  23324,  23326}},
 { 23920,  41, { 23327,  23248,  23244,  23302}},
 { 23921,  41, { 23327,  23302,  23304,  23328}},
 { 23922,  41, { 23329,  23252,  23248,  23327}},
 { 23923,  41, { 23329,  23327,  23328,  23330}},
 { 23924,  41, { 23264,  23300,  23325,  23331}},
 { 23925,  41, { 23331,  23325,  23326,  23332}},
 { 23926,  41, { 23260,  23333,  23331,  23264}},
 { 23927,  41, { 23331,  23333,  23334,  23332}},
 { 23928,  41, { 23260,  23256,  23335,  23333}},
 { 23929,  41, { 23333,  23335,  23336,  23334}},
 { 23930,  41, { 23329,  23335,  23256,  23252}},
 { 23931,  41, { 23335,  23329,  23330,  23336}},
 { 23932,  41, { 23241,  23245,  23337,  23338}},
 { 23933,  41, { 23237,  23241,  23338,  23339}},
 { 23934,  41, { 23229,  23237,  23339,  23340}},
 { 23935,  41, { 23232,  23229,  23340,  23341}},
 { 23936,  41, { 23266,  23232,  23341,  23342}},
 { 23937,  41, { 23270,  23266,  23342,  23343}},
 { 23938,  41, { 23274,  23270,  23343,  23344}},
 { 23939,  41, { 23278,  23274,  23344,  23345}},
 { 23940,  41, { 23282,  23278,  23345,  23346}},
 { 23941,  41, { 23286,  23282,  23346,  23347}},
 { 23942,  41, { 23290,  23286,  23347,  23348}},
 { 23943,  41, { 23294,  23290,  23348,  23349}},
 { 23944,  41, { 23298,  23294,  23349,  23350}},
 { 23945,  41, { 23261,  23298,  23350,  23351}},
 { 23946,  41, { 23257,  23261,  23351,  23352}},
 { 23947,  41, { 23253,  23257,  23352,  23353}},
 { 23948,  41, { 23249,  23253,  23353,  23354}},
 { 23949,  41, { 23245,  23249,  23354,  23337}},
 { 23950,  41, { 23355,  23356,  23339,  23340}},
 { 23951,  41, { 23303,  23306,  23357,  23358}},
 { 23952,  41, { 23304,  23303,  23358,  23359}},
 { 23953,  41, { 23339,  23338,  23360,  23356}},
 { 23954,  41, { 23328,  23304,  23359,  23361}},
 { 23955,  41, { 23338,  23337,  23362,  23360}},
 { 23956,  41, { 23330,  23328,  23361,  23363}},
 { 23957,  41, { 23337,  23354,  23364,  23362}},
 { 23958,  41, { 23336,  23330,  23363,  23365}},
 { 23959,  41, { 23354,  23353,  23366,  23364}},
 { 23960,  41, { 23334,  23336,  23365,  23367}},
 { 23961,  41, { 23353,  23352,  23368,  23366}},
 { 23962,  41, { 23332,  23334,  23367,  23369}},
 { 23963,  41, { 23352,  23351,  23370,  23368}},
 { 23964,  41, { 23326,  23332,  23369,  23371}},
 { 23965,  41, { 23351,  23350,  23372,  23370}},
 { 23966,  41, { 23324,  23326,  23371,  23373}},
 { 23967,  41, { 23350,  23349,  23374,  23372}},
 { 23968,  41, { 23349,  23348,  23375,  23374}},
 { 23969,  41, { 23322,  23324,  23373,  23376}},
 { 23970,  41, { 23320,  23322,  23376,  23377}},
 { 23971,  41, { 23348,  23347,  23378,  23375}},
 { 23972,  41, { 23347,  23346,  23379,  23378}},
 { 23973,  41, { 23318,  23320,  23377,  23380}},
 { 23974,  41, { 23316,  23318,  23380,  23381}},
 { 23975,  41, { 23346,  23345,  23382,  23379}},
 { 23976,  41, { 23314,  23316,  23381,  23383}},
 { 23977,  41, { 23344,  23345,  23382,  23384}},
 { 23978,  41, { 23314,  23312,  23385,  23383}},
 { 23979,  41, { 23343,  23344,  23384,  23386}},
 { 23980,  41, { 23312,  23310,  23387,  23385}},
 { 23981,  41, { 23342,  23343,  23386,  23388}},
 { 23982,  41, { 23310,  23308,  23389,  23387}},
 { 23983,  41, { 23341,  23342,  23388,  23390}},
 { 23984,  41, { 23389,  23357,  23306,  23308}},
 { 23985,  41, { 23355,  23390,  23341,  23340}},
 { 23986,  41, { 23361,  23359,  23391,  23392}},
 { 23987,  41, { 23393,  23394,  23392,  23391}},
 { 23988,  41, { 23360,  23362,  23395,  23396}},
 { 23989,  41, { 23393,  23396,  23395,  23394}},
 { 23990,  41, { 23362,  23364,  23397,  23395}},
 { 23991,  41, { 23394,  23395,  23397,  23398}},
 { 23992,  41, { 23363,  23361,  23392,  23399}},
 { 23993,  41, { 23392,  23394,  23398,  23399}},
 { 23994,  41, { 23365,  23363,  23399,  23400}},
 { 23995,  41, { 23399,  23398,  23401,  23400}},
 { 23996,  41, { 23359,  23358,  23402,  23391}},
 { 23997,  41, { 23402,  23403,  23393,  23391}},
 { 23998,  41, { 23356,  23360,  23396,  23404}},
 { 23999,  41, { 23403,  23404,  23396,  23393}},
 { 24000,  41, { 23355,  23356,  23404,  23405}},
 { 24001,  41, { 23406,  23405,  23404,  23403}},
 { 24002,  41, { 23358,  23357,  23407,  23402}},
 { 24003,  41, { 23407,  23406,  23403,  23402}},
 { 24004,  41, { 23390,  23355,  23405,  23408}},
 { 24005,  41, { 23409,  23408,  23405,  23406}},
 { 24006,  41, { 23357,  23389,  23410,  23407}},
 { 24007,  41, { 23410,  23409,  23406,  23407}},
 { 24008,  41, { 23388,  23390,  23408,  23411}},
 { 24009,  41, { 23412,  23411,  23408,  23409}},
 { 24010,  41, { 23389,  23387,  23413,  23410}},
 { 24011,  41, { 23412,  23409,  23410,  23413}},
 { 24012,  41, { 23387,  23385,  23414,  23413}},
 { 24013,  41, { 23415,  23412,  23413,  23414}},
 { 24014,  41, { 23386,  23388,  23411,  23416}},
 { 24015,  41, { 23416,  23411,  23412,  23415}},
 { 24016,  41, { 23417,  23415,  23416,  23418}},
 { 24017,  41, { 23418,  23416,  23386,  23384}},
 { 24018,  41, { 23419,  23414,  23415,  23417}},
 { 24019,  41, { 23414,  23419,  23383,  23385}},
 { 24020,  41, { 23382,  23384,  23418,  23420}},
 { 24021,  41, { 23418,  23417,  23421,  23420}},
 { 24022,  41, { 23383,  23381,  23422,  23419}},
 { 24023,  41, { 23417,  23419,  23422,  23421}},
 { 24024,  41, { 23379,  23382,  23420,  23423}},
 { 24025,  41, { 23420,  23421,  23424,  23423}},
 { 24026,  41, { 23381,  23380,  23425,  23422}},
 { 24027,  41, { 23421,  23422,  23425,  23424}},
 { 24028,  41, { 23378,  23379,  23423,  23426}},
 { 24029,  41, { 23424,  23427,  23426,  23423}},
 { 24030,  41, { 23380,  23377,  23428,  23425}},
 { 24031,  41, { 23424,  23425,  23428,  23427}},
 { 24032,  41, { 23377,  23376,  23429,  23428}},
 { 24033,  41, { 23427,  23428,  23429,  23430}},
 { 24034,  41, { 23375,  23378,  23426,  23431}},
 { 24035,  41, { 23426,  23427,  23430,  23431}},
 { 24036,  41, { 23376,  23373,  23432,  23429}},
 { 24037,  41, { 23430,  23429,  23432,  23433}},
 { 24038,  41, { 23374,  23375,  23431,  23434}},
 { 24039,  41, { 23431,  23430,  23433,  23434}},
 { 24040,  41, { 23371,  23373,  23432,  23435}},
 { 24041,  41, { 23432,  23433,  23436,  23435}},
 { 24042,  41, { 23369,  23371,  23435,  23437}},
 { 24043,  41, { 23435,  23436,  23438,  23437}},
 { 24044,  41, { 23367,  23369,  23437,  23439}},
 { 24045,  41, { 23437,  23438,  23440,  23439}},
 { 24046,  41, { 23364,  23366,  23441,  23397}},
 { 24047,  41, { 23398,  23397,  23441,  23401}},
 { 24048,  41, { 23367,  23365,  23400,  23439}},
 { 24049,  41, { 23400,  23401,  23440,  23439}},
 { 24050,  41, { 23366,  23368,  23442,  23441}},
 { 24051,  41, { 23440,  23401,  23441,  23442}},
 { 24052,  41, { 23368,  23370,  23443,  23442}},
 { 24053,  41, { 23438,  23440,  23442,  23443}},
 { 24054,  41, { 23370,  23372,  23444,  23443}},
 { 24055,  41, { 23436,  23438,  23443,  23444}},
 { 24056,  41, { 23372,  23374,  23434,  23444}},
 { 24057,  41, { 23433,  23436,  23444,  23434}},
 { 24058,  42, { 23235,  23307,  23305,  23236}},
 { 24059,  42, { 23235,  23236,  23233,  23234}},
 { 24060,  42, { 23236,  23305,  23301,  23240}},
 { 24061,  42, { 23236,  23240,  23239,  23233}},
 { 24062,  42, { 23240,  23301,  23302,  23244}},
 { 24063,  42, { 23240,  23244,  23243,  23239}},
 { 24064,  42, { 23244,  23302,  23327,  23248}},
 { 24065,  42, { 23244,  23248,  23247,  23243}},
 { 24066,  42, { 23248,  23327,  23329,  23252}},
 { 24067,  42, { 23248,  23252,  23251,  23247}},
 { 24068,  42, { 23252,  23329,  23335,  23256}},
 { 24069,  42, { 23252,  23256,  23255,  23251}},
 { 24070,  42, { 23256,  23335,  23333,  23260}},
 { 24071,  42, { 23256,  23260,  23259,  23255}},
 { 24072,  42, { 23333,  23331,  23264,  23260}},
 { 24073,  42, { 23260,  23264,  23263,  23259}},
 { 24074,  42, { 23331,  23325,  23300,  23264}},
 { 24075,  42, { 23264,  23300,  23299,  23263}},
 { 24076,  42, { 23233,  23445,  23446,  23234}},
 { 24077,  42, { 23239,  23447,  23445,  23233}},
 { 24078,  42, { 23243,  23448,  23447,  23239}},
 { 24079,  42, { 23247,  23449,  23448,  23243}},
 { 24080,  42, { 23251,  23450,  23449,  23247}},
 { 24081,  42, { 23255,  23451,  23450,  23251}},
 { 24082,  42, { 23259,  23452,  23451,  23255}},
 { 24083,  42, { 23259,  23263,  23453,  23452}},
 { 24084,  42, { 23263,  23299,  23454,  23453}},
 { 24085,  42, { 23235,  23268,  23309,  23307}},
 { 24086,  42, { 23268,  23235,  23234,  23267}},
 { 24087,  42, { 23268,  23272,  23311,  23309}},
 { 24088,  42, { 23272,  23268,  23267,  23271}},
 { 24089,  42, { 23272,  23276,  23313,  23311}},
 { 24090,  42, { 23276,  23272,  23271,  23275}},
 { 24091,  42, { 23276,  23280,  23315,  23313}},
 { 24092,  42, { 23321,  23292,  23296,  23323}},
 { 24093,  42, { 23323,  23296,  23300,  23325}},
 { 24094,  42, { 23300,  23296,  23295,  23299}},
 { 24095,  42, { 23446,  23455,  23267,  23234}},
 { 24096,  42, { 23455,  23456,  23271,  23267}},
 { 24097,  42, { 23456,  23457,  23275,  23271}},
 { 24098,  42, { 23275,  23279,  23280,  23276}},
 { 24099,  42, { 23457,  23458,  23279,  23275}},
 { 24100,  42, { 23279,  23283,  23284,  23280}},
 { 24101,  42, { 23458,  23459,  23283,  23279}},
 { 24102,  42, { 23283,  23287,  23288,  23284}},
 { 24103,  42, { 23459,  23460,  23287,  23283}},
 { 24104,  42, { 23287,  23291,  23292,  23288}},
 { 24105,  42, { 23460,  23461,  23291,  23287}},
 { 24106,  42, { 23291,  23295,  23296,  23292}},
 { 24107,  42, { 23462,  23295,  23291,  23461}},
 { 24108,  42, { 23454,  23299,  23295,  23462}},
 { 24109,  42, { 23463,  23464,  23449,  23450}},
 { 24110,  42, { 23465,  23463,  23450,  23451}},
 { 24111,  42, { 23464,  23466,  23448,  23449}},
 { 24112,  42, { 23467,  23465,  23451,  23452}},
 { 24113,  42, { 23468,  23467,  23452,  23453}},
 { 24114,  42, { 23469,  23468,  23453,  23454}},
 { 24115,  42, { 23470,  23469,  23454,  23462}},
 { 24116,  42, { 23466,  23471,  23447,  23448}},
 { 24117,  42, { 23471,  23472,  23445,  23447}},
 { 24118,  42, { 23472,  23473,  23446,  23445}},
 { 24119,  42, { 23473,  23474,  23455,  23446}},
 { 24120,  42, { 23475,  23476,  23459,  23458}},
 { 24121,  42, { 23476,  23477,  23460,  23459}},
 { 24122,  42, { 23478,  23475,  23458,  23457}},
 { 24123,  42, { 23477,  23479,  23461,  23460}},
 { 24124,  42, { 23479,  23470,  23462,  23461}},
 { 24125,  42, { 23480,  23478,  23457,  23456}},
 { 24126,  42, { 23474,  23480,  23456,  23455}},
 { 24127,  42, { 23481,  23482,  23483,  23483}},
 { 24128,  42, { 23483,  23482,  23464,  23463}},
 { 24129,  42, { 23483,  23484,  23481,  23481}},
 { 24130,  42, { 23484,  23483,  23463,  23465}},
 { 24131,  42, { 23485,  23482,  23481,  23481}},
 { 24132,  42, { 23482,  23485,  23466,  23464}},
 { 24133,  42, { 23484,  23486,  23481,  23481}},
 { 24134,  42, { 23486,  23484,  23465,  23467}},
 { 24135,  42, { 23486,  23487,  23481,  23481}},
 { 24136,  42, { 23487,  23486,  23467,  23468}},
 { 24137,  42, { 23487,  23488,  23481,  23481}},
 { 24138,  42, { 23488,  23487,  23468,  23469}},
 { 24139,  42, { 23489,  23481,  23488,  23488}},
 { 24140,  42, { 23489,  23488,  23469,  23470}},
 { 24141,  42, { 23490,  23485,  23481,  23481}},
 { 24142,  42, { 23485,  23490,  23471,  23466}},
 { 24143,  42, { 23491,  23490,  23481,  23481}},
 { 24144,  42, { 23490,  23491,  23472,  23471}},
 { 24145,  42, { 23492,  23491,  23481,  23481}},
 { 24146,  42, { 23491,  23492,  23473,  23472}},
 { 24147,  42, { 23492,  23481,  23493,  23493}},
 { 24148,  42, { 23492,  23493,  23474,  23473}},
 { 24149,  42, { 23481,  23494,  23495,  23495}},
 { 24150,  42, { 23495,  23494,  23476,  23475}},
 { 24151,  42, { 23494,  23481,  23496,  23496}},
 { 24152,  42, { 23494,  23496,  23477,  23476}},
 { 24153,  42, { 23497,  23481,  23495,  23495}},
 { 24154,  42, { 23497,  23495,  23475,  23478}},
 { 24155,  42, { 23496,  23481,  23498,  23498}},
 { 24156,  42, { 23496,  23498,  23479,  23477}},
 { 24157,  42, { 23498,  23481,  23489,  23489}},
 { 24158,  42, { 23498,  23489,  23470,  23479}},
 { 24159,  42, { 23499,  23481,  23497,  23497}},
 { 24160,  42, { 23499,  23497,  23478,  23480}},
 { 24161,  42, { 23493,  23481,  23499,  23499}},
 { 24162,  42, { 23493,  23499,  23480,  23474}},
 { 24163,  42, { 23317,  23315,  23280,  23284}},
 { 24164,  42, { 23319,  23317,  23284,  23288}},
 { 24165,  42, { 23321,  23319,  23288,  23292}},
 { 24166,  42, { 23385,  23500,  23387,  23387}},
 { 24167,  42, { 23387,  23500,  23389,  23389}},
 { 24168,  42, { 23389,  23500,  23357,  23357}},
 { 24169,  42, { 23357,  23500,  23358,  23358}},
 { 24170,  42, { 23358,  23500,  23359,  23359}},
 { 24171,  42, { 23359,  23500,  23361,  23361}},
 { 24172,  42, { 23361,  23500,  23363,  23363}},
 { 24173,  42, { 23363,  23365,  23500,  23500}},
 { 24174,  42, { 23500,  23365,  23367,  23367}},
 { 24175,  42, { 23500,  23367,  23369,  23369}},
 { 24176,  42, { 23500,  23369,  23371,  23371}},
 { 24177,  42, { 23500,  23371,  23373,  23373}},
 { 24178,  42, { 23500,  23373,  23376,  23376}},
 { 24179,  42, { 23500,  23376,  23377,  23377}},
 { 24180,  42, { 23500,  23377,  23380,  23380}},
 { 24181,  42, { 23500,  23380,  23381,  23381}},
 { 24182,  42, { 23500,  23381,  23383,  23383}},
 { 24183,  42, { 23500,  23383,  23385,  23385}},
 { 24184,  42, { 23381,  23383,  23314,  23316}},
 { 24185,  42, { 23313,  23315,  23316,  23314}},
 { 24186,  42, { 23380,  23381,  23316,  23318}},
 { 24187,  42, { 23315,  23317,  23318,  23316}},
 { 24188,  42, { 23377,  23380,  23318,  23320}},
 { 24189,  42, { 23317,  23319,  23320,  23318}},
 { 24190,  42, { 23376,  23377,  23320,  23322}},
 { 24191,  42, { 23319,  23321,  23322,  23320}},
 { 24192,  42, { 23373,  23376,  23322,  23324}},
 { 24193,  42, { 23321,  23323,  23324,  23322}},
 { 24194,  42, { 23371,  23373,  23324,  23326}},
 { 24195,  42, { 23323,  23325,  23326,  23324}},
 { 24196,  42, { 23325,  23331,  23332,  23326}},
 { 24197,  42, { 23369,  23371,  23326,  23332}},
 { 24198,  42, { 23331,  23333,  23334,  23332}},
 { 24199,  42, { 23367,  23369,  23332,  23334}},
 { 24200,  42, { 23333,  23335,  23336,  23334}},
 { 24201,  42, { 23365,  23367,  23334,  23336}},
 { 24202,  42, { 23335,  23329,  23330,  23336}},
 { 24203,  42, { 23363,  23365,  23336,  23330}},
 { 24204,  42, { 23329,  23327,  23328,  23330}},
 { 24205,  42, { 23361,  23363,  23330,  23328}},
 { 24206,  42, { 23327,  23302,  23304,  23328}},
 { 24207,  42, { 23359,  23361,  23328,  23304}},
 { 24208,  42, { 23302,  23301,  23303,  23304}},
 { 24209,  42, { 23358,  23359,  23304,  23303}},
 { 24210,  42, { 23301,  23305,  23306,  23303}},
 { 24211,  42, { 23357,  23358,  23303,  23306}},
 { 24212,  42, { 23305,  23307,  23308,  23306}},
 { 24213,  42, { 23389,  23357,  23306,  23308}},
 { 24214,  42, { 23387,  23389,  23308,  23310}},
 { 24215,  42, { 23307,  23309,  23310,  23308}},
 { 24216,  42, { 23385,  23387,  23310,  23312}},
 { 24217,  42, { 23309,  23311,  23312,  23310}},
 { 24218,  42, { 23383,  23385,  23312,  23314}},
 { 24219,  42, { 23311,  23313,  23314,  23312}},
 { 24346,  45, { 23537,  23538,  23539,  23540}},
 { 24347,  45, { 23541,  23542,  23538,  23537}},
 { 24348,  45, { 23540,  23539,  23543,  23544}},
 { 24349,  45, { 23544,  23543,  23545,  23546}},
 { 24350,  45, { 23546,  23545,  23547,  23548}},
 { 24351,  45, { 23538,  23549,  23550,  23539}},
 { 24352,  45, { 23542,  23551,  23549,  23538}},
 { 24353,  45, { 23539,  23550,  23552,  23543}},
 { 24354,  45, { 23543,  23552,  23553,  23545}},
 { 24355,  45, { 23545,  23553,  23554,  23547}},
 { 24356,  45, { 23549,  23555,  23556,  23550}},
 { 24357,  45, { 23551,  23557,  23555,  23549}},
 { 24358,  45, { 23550,  23556,  23558,  23552}},
 { 24359,  45, { 23552,  23558,  23559,  23553}},
 { 24360,  45, { 23553,  23559,  23560,  23554}},
 { 24361,  45, { 23555,  23561,  23562,  23556}},
 { 24362,  45, { 23557,  23563,  23561,  23555}},
 { 24363,  45, { 23556,  23562,  23564,  23558}},
 { 24364,  45, { 23558,  23564,  23565,  23559}},
 { 24365,  45, { 23559,  23565,  23566,  23560}},
 { 24366,  45, { 23567,  23537,  23540,  23568}},
 { 24367,  45, { 23567,  23569,  23541,  23537}},
 { 24368,  45, { 23568,  23540,  23544,  23570}},
 { 24369,  45, { 23570,  23544,  23546,  23571}},
 { 24370,  45, { 23572,  23567,  23568,  23573}},
 { 24371,  45, { 23572,  23574,  23569,  23567}},
 { 24372,  45, { 23573,  23568,  23570,  23575}},
 { 24373,  45, { 23575,  23570,  23571,  23576}},
 { 24374,  45, { 23577,  23572,  23573,  23578}},
 { 24375,  45, { 23577,  23579,  23574,  23572}},
 { 24376,  45, { 23578,  23573,  23575,  23580}},
 { 24377,  45, { 23580,  23575,  23576,  23581}},
 { 24378,  45, { 23582,  23577,  23578,  23583}},
 { 24379,  45, { 23582,  23584,  23579,  23577}},
 { 24380,  45, { 23583,  23578,  23580,  23585}},
 { 24381,  45, { 23585,  23580,  23581,  23586}},
 { 24382,  45, { 23569,  23587,  23542,  23541}},
 { 24383,  45, { 23574,  23588,  23587,  23569}},
 { 24384,  45, { 23579,  23589,  23588,  23574}},
 { 24385,  45, { 23584,  23590,  23589,  23579}},
 { 24386,  45, { 23587,  23591,  23551,  23542}},
 { 24387,  45, { 23590,  23592,  23593,  23589}},
 { 24388,  45, { 23591,  23594,  23557,  23551}},
 { 24389,  45, { 23592,  23595,  23596,  23593}},
 { 24390,  45, { 23594,  23597,  23563,  23557}},
 { 24391,  45, { 23598,  23599,  23597,  23594}},
 { 24392,  45, { 23596,  23600,  23599,  23598}},
 { 24393,  45, { 23595,  23601,  23600,  23596}},
 { 24394,  45, { 23597,  23602,  23561,  23563}},
 { 24395,  45, { 23561,  23602,  23603,  23562}},
 { 24396,  45, { 23562,  23603,  23604,  23564}},
 { 24397,  45, { 23564,  23604,  23605,  23565}},
 { 24398,  45, { 23599,  23606,  23602,  23597}},
 { 24399,  45, { 23602,  23606,  23607,  23603}},
 { 24400,  45, { 23603,  23607,  23608,  23604}},
 { 24401,  45, { 23604,  23608,  23609,  23605}},
 { 24402,  45, { 23600,  23610,  23606,  23599}},
 { 24403,  45, { 23606,  23610,  23611,  23607}},
 { 24404,  45, { 23607,  23611,  23612,  23608}},
 { 24405,  45, { 23608,  23612,  23613,  23609}},
 { 24406,  45, { 23601,  23614,  23610,  23600}},
 { 24407,  45, { 23610,  23614,  23615,  23611}},
 { 24408,  45, { 23611,  23615,  23616,  23612}},
 { 24409,  45, { 23612,  23616,  23617,  23613}},
 { 24410,  45, { 23583,  23618,  23619,  23582}},
 { 24411,  45, { 23582,  23619,  23590,  23584}},
 { 24412,  45, { 23585,  23620,  23618,  23583}},
 { 24413,  45, { 23621,  23622,  23623,  23586}},
 { 24414,  45, { 23586,  23623,  23620,  23585}},
 { 24415,  45, { 23618,  23624,  23625,  23619}},
 { 24416,  45, { 23619,  23625,  23592,  23590}},
 { 24417,  45, { 23620,  23626,  23624,  23618}},
 { 24418,  45, { 23622,  23627,  23628,  23623}},
 { 24419,  45, { 23623,  23628,  23626,  23620}},
 { 24420,  45, { 23624,  23629,  23630,  23625}},
 { 24421,  45, { 23625,  23630,  23595,  23592}},
 { 24422,  45, { 23626,  23631,  23629,  23624}},
 { 24423,  45, { 23627,  23632,  23633,  23628}},
 { 24424,  45, { 23628,  23633,  23631,  23626}},
 { 24425,  45, { 23629,  23615,  23614,  23630}},
 { 24426,  45, { 23630,  23614,  23601,  23595}},
 { 24427,  45, { 23631,  23616,  23615,  23629}},
 { 24428,  45, { 23632,  23634,  23617,  23633}},
 { 24429,  45, { 23633,  23617,  23616,  23631}},
 { 24430,  54, { 23576,  23571,  23635,  23636}},
 { 24431,  54, { 23581,  23576,  23636,  23637}},
 { 24432,  54, { 23605,  23609,  23638,  23639}},
 { 24433,  54, { 23609,  23613,  23640,  23638}},
 { 24434,  54, { 23635,  23641,  23642,  23643}},
 { 24435,  54, { 23643,  23642,  23644,  23645}},
 { 24436,  54, { 23636,  23635,  23643,  23646}},
 { 24437,  54, { 23646,  23643,  23645,  23647}},
 { 24438,  54, { 23637,  23636,  23646,  23648}},
 { 24439,  54, { 23648,  23646,  23647,  23649}},
 { 24440,  54, { 23650,  23637,  23648,  23651}},
 { 24441,  54, { 23651,  23648,  23649,  23652}},
 { 24442,  54, { 23653,  23639,  23654,  23655}},
 { 24443,  54, { 23655,  23654,  23656,  23657}},
 { 24444,  54, { 23639,  23638,  23658,  23654}},
 { 24445,  54, { 23654,  23658,  23659,  23656}},
 { 24446,  54, { 23638,  23640,  23660,  23658}},
 { 24447,  54, { 23658,  23660,  23661,  23659}},
 { 24448,  54, { 23640,  23662,  23663,  23660}},
 { 24449,  54, { 23660,  23663,  23664,  23661}},
 { 24450,  54, { 23644,  23665,  23666,  23645}},
 { 24451,  54, { 23645,  23666,  23667,  23647}},
 { 24452,  54, { 23647,  23667,  23668,  23649}},
 { 24453,  54, { 23649,  23668,  23669,  23652}},
 { 24454,  54, { 23665,  23670,  23671,  23666}},
 { 24455,  54, { 23666,  23671,  23672,  23667}},
 { 24456,  54, { 23667,  23672,  23673,  23668}},
 { 24457,  54, { 23668,  23673,  23674,  23669}},
 { 24458,  54, { 23670,  23675,  23676,  23671}},
 { 24459,  54, { 23671,  23676,  23677,  23672}},
 { 24460,  54, { 23672,  23677,  23678,  23673}},
 { 24461,  54, { 23673,  23678,  23679,  23674}},
 { 24462,  54, { 23675,  23657,  23656,  23676}},
 { 24463,  54, { 23676,  23656,  23659,  23677}},
 { 24464,  54, { 23677,  23659,  23661,  23678}},
 { 24465,  54, { 23678,  23661,  23664,  23679}},
 { 24466,  54, { 23662,  23613,  23640,  23640}},
 { 24467,  45, { 23617,  23613,  23634,  23634}},
 { 24468,  45, { 23581,  23621,  23586,  23586}},
 { 24469,  54, { 23650,  23581,  23637,  23637}},
 { 24470,  54, { 23605,  23639,  23653,  23653}},
 { 24471,  45, { 23566,  23605,  23565,  23565}},
 { 24472,  54, { 23641,  23571,  23635,  23635}},
 { 24473,  45, { 23571,  23548,  23546,  23546}},
 { 24474,  47, { 23680,  23681,  23682,  23683}},
 { 24475,  47, { 23681,  23684,  23685,  23682}},
 { 24476,  47, { 23684,  23686,  23687,  23685}},
 { 24477,  47, { 23686,  23688,  23689,  23687}},
 { 24478,  47, { 23688,  23690,  23691,  23689}},
 { 24479,  47, { 23690,  23692,  23693,  23691}},
 { 24480,  47, { 23692,  23694,  23695,  23693}},
 { 24481,  47, { 23694,  23696,  23697,  23695}},
 { 24482,  47, { 23696,  23698,  23699,  23697}},
 { 24483,  47, { 23698,  23700,  23701,  23699}},
 { 24484,  47, { 23700,  23702,  23703,  23701}},
 { 24485,  47, { 23702,  23704,  23705,  23703}},
 { 24486,  47, { 23704,  23706,  23707,  23705}},
 { 24487,  47, { 23707,  23706,  23708,  23709}},
 { 24488,  47, { 23710,  23711,  23688,  23686}},
 { 24489,  47, { 23712,  23713,  23711,  23710}},
 { 24490,  47, { 23714,  23715,  23713,  23712}},
 { 24491,  47, { 23716,  23717,  23715,  23714}},
 { 24492,  47, { 23711,  23718,  23690,  23688}},
 { 24493,  47, { 23713,  23719,  23718,  23711}},
 { 24494,  47, { 23715,  23720,  23719,  23713}},
 { 24495,  47, { 23717,  23721,  23720,  23715}},
 { 24496,  47, { 23718,  23722,  23692,  23690}},
 { 24497,  47, { 23719,  23723,  23722,  23718}},
 { 24498,  47, { 23720,  23724,  23723,  23719}},
 { 24499,  47, { 23721,  23725,  23724,  23720}},
 { 24500,  47, { 23722,  23726,  23694,  23692}},
 { 24501,  47, { 23723,  23727,  23726,  23722}},
 { 24502,  47, { 23724,  23728,  23727,  23723}},
 { 24503,  47, { 23725,  23729,  23728,  23724}},
 { 24504,  47, { 23726,  23730,  23696,  23694}},
 { 24505,  47, { 23727,  23731,  23730,  23726}},
 { 24506,  47, { 23728,  23732,  23731,  23727}},
 { 24507,  47, { 23729,  23733,  23732,  23728}},
 { 24508,  47, { 23730,  23734,  23698,  23696}},
 { 24509,  47, { 23731,  23735,  23734,  23730}},
 { 24510,  47, { 23732,  23736,  23735,  23731}},
 { 24511,  47, { 23733,  23737,  23736,  23732}},
 { 24512,  47, { 23734,  23738,  23700,  23698}},
 { 24513,  47, { 23735,  23739,  23738,  23734}},
 { 24514,  47, { 23736,  23740,  23739,  23735}},
 { 24515,  47, { 23737,  23741,  23740,  23736}},
 { 24516,  47, { 23738,  23742,  23702,  23700}},
 { 24517,  47, { 23739,  23743,  23742,  23738}},
 { 24518,  47, { 23740,  23744,  23743,  23739}},
 { 24519,  47, { 23741,  23745,  23744,  23740}},
 { 24520,  47, { 23742,  23746,  23704,  23702}},
 { 24521,  47, { 23743,  23747,  23746,  23742}},
 { 24522,  47, { 23744,  23748,  23747,  23743}},
 { 24523,  47, { 23745,  23749,  23748,  23744}},
 { 24524,  47, { 23746,  23750,  23706,  23704}},
 { 24525,  47, { 23706,  23750,  23751,  23708}},
 { 24526,  47, { 23747,  23752,  23750,  23746}},
 { 24527,  47, { 23750,  23752,  23753,  23751}},
 { 24528,  47, { 23748,  23754,  23752,  23747}},
 { 24529,  47, { 23752,  23754,  23755,  23753}},
 { 24530,  47, { 23749,  23756,  23754,  23748}},
 { 24531,  47, { 23754,  23756,  23757,  23755}},
 { 24532,  47, { 23758,  23759,  23760,  23761}},
 { 24533,  47, { 23759,  23762,  23763,  23760}},
 { 24534,  47, { 23762,  23764,  23716,  23763}},
 { 24535,  47, { 23764,  23765,  23717,  23716}},
 { 24536,  47, { 23765,  23766,  23721,  23717}},
 { 24537,  47, { 23766,  23767,  23725,  23721}},
 { 24538,  47, { 23767,  23768,  23729,  23725}},
 { 24539,  47, { 23768,  23769,  23733,  23729}},
 { 24540,  47, { 23769,  23770,  23737,  23733}},
 { 24541,  47, { 23770,  23771,  23741,  23737}},
 { 24542,  47, { 23771,  23772,  23745,  23741}},
 { 24543,  47, { 23772,  23773,  23749,  23745}},
 { 24544,  47, { 23773,  23774,  23756,  23749}},
 { 24545,  47, { 23756,  23774,  23775,  23757}},
 { 24546,  47, { 23683,  23682,  23776,  23777}},
 { 24547,  47, { 23777,  23776,  23778,  23779}},
 { 24548,  47, { 23779,  23778,  23780,  23781}},
 { 24549,  47, { 23781,  23780,  23782,  23783}},
 { 24550,  47, { 23682,  23685,  23784,  23776}},
 { 24551,  47, { 23776,  23784,  23785,  23778}},
 { 24552,  47, { 23778,  23785,  23786,  23780}},
 { 24553,  47, { 23780,  23786,  23787,  23782}},
 { 24554,  47, { 23685,  23687,  23788,  23784}},
 { 24555,  47, { 23784,  23788,  23789,  23785}},
 { 24556,  47, { 23785,  23789,  23790,  23786}},
 { 24557,  47, { 23786,  23790,  23791,  23787}},
 { 24558,  47, { 23687,  23689,  23792,  23788}},
 { 24559,  47, { 23788,  23792,  23793,  23789}},
 { 24560,  47, { 23789,  23793,  23794,  23790}},
 { 24561,  47, { 23790,  23794,  23795,  23791}},
 { 24562,  47, { 23689,  23691,  23796,  23792}},
 { 24563,  47, { 23792,  23796,  23797,  23793}},
 { 24564,  47, { 23793,  23797,  23798,  23794}},
 { 24565,  47, { 23794,  23798,  23799,  23795}},
 { 24566,  47, { 23691,  23693,  23800,  23796}},
 { 24567,  47, { 23796,  23800,  23801,  23797}},
 { 24568,  47, { 23797,  23801,  23802,  23798}},
 { 24569,  47, { 23798,  23802,  23803,  23799}},
 { 24570,  47, { 23693,  23695,  23804,  23800}},
 { 24571,  47, { 23800,  23804,  23805,  23801}},
 { 24572,  47, { 23801,  23805,  23806,  23802}},
 { 24573,  47, { 23802,  23806,  23807,  23803}},
 { 24574,  47, { 23695,  23697,  23808,  23804}},
 { 24575,  47, { 23804,  23808,  23809,  23805}},
 { 24576,  47, { 23805,  23809,  23810,  23806}},
 { 24577,  47, { 23806,  23810,  23811,  23807}},
 { 24578,  47, { 23697,  23699,  23812,  23808}},
 { 24579,  47, { 23808,  23812,  23813,  23809}},
 { 24580,  47, { 23809,  23813,  23814,  23810}},
 { 24581,  47, { 23810,  23814,  23815,  23811}},
 { 24582,  47, { 23699,  23701,  23816,  23812}},
 { 24583,  47, { 23812,  23816,  23817,  23813}},
 { 24584,  47, { 23813,  23817,  23818,  23814}},
 { 24585,  47, { 23814,  23818,  23819,  23815}},
 { 24586,  47, { 23701,  23703,  23820,  23816}},
 { 24587,  47, { 23816,  23820,  23821,  23817}},
 { 24588,  47, { 23817,  23821,  23822,  23818}},
 { 24589,  47, { 23818,  23822,  23823,  23819}},
 { 24590,  47, { 23703,  23705,  23824,  23820}},
 { 24591,  47, { 23820,  23824,  23825,  23821}},
 { 24592,  47, { 23821,  23825,  23826,  23822}},
 { 24593,  47, { 23822,  23826,  23827,  23823}},
 { 24594,  47, { 23705,  23707,  23828,  23824}},
 { 24595,  47, { 23707,  23709,  23829,  23828}},
 { 24596,  47, { 23824,  23828,  23830,  23825}},
 { 24597,  47, { 23828,  23829,  23831,  23830}},
 { 24598,  47, { 23825,  23830,  23832,  23826}},
 { 24599,  47, { 23830,  23831,  23833,  23832}},
 { 24600,  47, { 23826,  23832,  23834,  23827}},
 { 24601,  47, { 23832,  23833,  23835,  23834}},
 { 24602,  47, { 23680,  23683,  23777,  23836}},
 { 24603,  47, { 23836,  23777,  23779,  23837}},
 { 24604,  47, { 23837,  23779,  23781,  23838}},
 { 24605,  47, { 23838,  23781,  23783,  23839}},
 { 24606,  47, { 23709,  23708,  23840,  23829}},
 { 24607,  47, { 23829,  23840,  23841,  23831}},
 { 24608,  47, { 23831,  23841,  23842,  23833}},
 { 24609,  47, { 23833,  23842,  23843,  23835}},
 { 24610,  47, { 23708,  23751,  23844,  23840}},
 { 24611,  47, { 23840,  23844,  23845,  23841}},
 { 24612,  47, { 23841,  23845,  23846,  23842}},
 { 24613,  47, { 23842,  23846,  23847,  23843}},
 { 24614,  47, { 23751,  23753,  23848,  23844}},
 { 24615,  47, { 23844,  23848,  23849,  23845}},
 { 24616,  47, { 23845,  23849,  23850,  23846}},
 { 24617,  47, { 23846,  23850,  23851,  23847}},
 { 24618,  47, { 23753,  23755,  23852,  23848}},
 { 24619,  47, { 23848,  23852,  23853,  23849}},
 { 24620,  47, { 23849,  23853,  23854,  23850}},
 { 24621,  47, { 23850,  23854,  23855,  23851}},
 { 24622,  47, { 23755,  23757,  23856,  23852}},
 { 24623,  47, { 23852,  23856,  23857,  23853}},
 { 24624,  47, { 23853,  23857,  23858,  23854}},
 { 24625,  47, { 23854,  23858,  23859,  23855}},
 { 24626,  47, { 23758,  23761,  23860,  23861}},
 { 24627,  47, { 23861,  23860,  23862,  23863}},
 { 24628,  47, { 23863,  23862,  23864,  23865}},
 { 24629,  47, { 23865,  23864,  23866,  23867}},
 { 24630,  47, { 23757,  23775,  23868,  23856}},
 { 24631,  47, { 23856,  23868,  23869,  23857}},
 { 24632,  47, { 23857,  23869,  23870,  23858}},
 { 24633,  47, { 23858,  23870,  23871,  23859}},
 { 24634,  47, { 23861,  23872,  23759,  23758}},
 { 24635,  47, { 23863,  23873,  23872,  23861}},
 { 24636,  47, { 23865,  23874,  23873,  23863}},
 { 24637,  47, { 23867,  23875,  23874,  23865}},
 { 24638,  47, { 23872,  23876,  23762,  23759}},
 { 24639,  47, { 23873,  23877,  23876,  23872}},
 { 24640,  47, { 23874,  23878,  23877,  23873}},
 { 24641,  47, { 23875,  23879,  23878,  23874}},
 { 24642,  47, { 23876,  23880,  23764,  23762}},
 { 24643,  47, { 23877,  23881,  23880,  23876}},
 { 24644,  47, { 23878,  23882,  23881,  23877}},
 { 24645,  47, { 23879,  23883,  23882,  23878}},
 { 24646,  47, { 23880,  23884,  23765,  23764}},
 { 24647,  47, { 23881,  23885,  23884,  23880}},
 { 24648,  47, { 23882,  23886,  23885,  23881}},
 { 24649,  47, { 23883,  23887,  23886,  23882}},
 { 24650,  47, { 23884,  23888,  23766,  23765}},
 { 24651,  47, { 23885,  23889,  23888,  23884}},
 { 24652,  47, { 23886,  23890,  23889,  23885}},
 { 24653,  47, { 23887,  23891,  23890,  23886}},
 { 24654,  47, { 23888,  23892,  23767,  23766}},
 { 24655,  47, { 23889,  23893,  23892,  23888}},
 { 24656,  47, { 23890,  23894,  23893,  23889}},
 { 24657,  47, { 23891,  23895,  23894,  23890}},
 { 24658,  47, { 23892,  23896,  23768,  23767}},
 { 24659,  47, { 23893,  23897,  23896,  23892}},
 { 24660,  47, { 23894,  23898,  23897,  23893}},
 { 24661,  47, { 23895,  23899,  23898,  23894}},
 { 24662,  47, { 23896,  23900,  23769,  23768}},
 { 24663,  47, { 23897,  23901,  23900,  23896}},
 { 24664,  47, { 23898,  23902,  23901,  23897}},
 { 24665,  47, { 23899,  23903,  23902,  23898}},
 { 24666,  47, { 23900,  23904,  23770,  23769}},
 { 24667,  47, { 23901,  23905,  23904,  23900}},
 { 24668,  47, { 23902,  23906,  23905,  23901}},
 { 24669,  47, { 23903,  23907,  23906,  23902}},
 { 24670,  47, { 23904,  23908,  23771,  23770}},
 { 24671,  47, { 23905,  23909,  23908,  23904}},
 { 24672,  47, { 23906,  23910,  23909,  23905}},
 { 24673,  47, { 23907,  23911,  23910,  23906}},
 { 24674,  47, { 23908,  23912,  23772,  23771}},
 { 24675,  47, { 23909,  23913,  23912,  23908}},
 { 24676,  47, { 23910,  23914,  23913,  23909}},
 { 24677,  47, { 23911,  23915,  23914,  23910}},
 { 24678,  47, { 23912,  23916,  23773,  23772}},
 { 24679,  47, { 23913,  23917,  23916,  23912}},
 { 24680,  47, { 23914,  23918,  23917,  23913}},
 { 24681,  47, { 23915,  23919,  23918,  23914}},
 { 24682,  47, { 23916,  23920,  23774,  23773}},
 { 24683,  47, { 23775,  23774,  23920,  23868}},
 { 24684,  47, { 23917,  23921,  23920,  23916}},
 { 24685,  47, { 23868,  23920,  23921,  23869}},
 { 24686,  47, { 23918,  23922,  23921,  23917}},
 { 24687,  47, { 23869,  23921,  23922,  23870}},
 { 24688,  47, { 23919,  23923,  23922,  23918}},
 { 24689,  47, { 23870,  23922,  23923,  23871}},
 { 24690,  47, { 23783,  23782,  23924,  23839}},
 { 24691,  47, { 23782,  23787,  23925,  23924}},
 { 24692,  47, { 23787,  23791,  23926,  23925}},
 { 24693,  47, { 23791,  23795,  23927,  23926}},
 { 24694,  47, { 23795,  23799,  23928,  23927}},
 { 24695,  47, { 23799,  23803,  23929,  23928}},
 { 24696,  47, { 23803,  23807,  23930,  23929}},
 { 24697,  47, { 23807,  23811,  23931,  23930}},
 { 24698,  47, { 23811,  23815,  23932,  23931}},
 { 24699,  47, { 23815,  23819,  23933,  23932}},
 { 24700,  47, { 23819,  23823,  23934,  23933}},
 { 24701,  47, { 23823,  23827,  23935,  23934}},
 { 24702,  47, { 23827,  23834,  23936,  23935}},
 { 24703,  47, { 23835,  23843,  23936,  23834}},
 { 24704,  47, { 23926,  23927,  23937,  23938}},
 { 24705,  47, { 23938,  23937,  23939,  23940}},
 { 24706,  47, { 23940,  23939,  23941,  23942}},
 { 24707,  47, { 23942,  23941,  23943,  23944}},
 { 24708,  47, { 23927,  23928,  23945,  23937}},
 { 24709,  47, { 23937,  23945,  23946,  23939}},
 { 24710,  47, { 23939,  23946,  23947,  23941}},
 { 24711,  47, { 23941,  23947,  23948,  23943}},
 { 24712,  47, { 23928,  23929,  23949,  23945}},
 { 24713,  47, { 23945,  23949,  23950,  23946}},
 { 24714,  47, { 23946,  23950,  23951,  23947}},
 { 24715,  47, { 23947,  23951,  23952,  23948}},
 { 24716,  47, { 23929,  23930,  23953,  23949}},
 { 24717,  47, { 23949,  23953,  23954,  23950}},
 { 24718,  47, { 23950,  23954,  23955,  23951}},
 { 24719,  47, { 23951,  23955,  23956,  23952}},
 { 24720,  47, { 23930,  23931,  23957,  23953}},
 { 24721,  47, { 23953,  23957,  23958,  23954}},
 { 24722,  47, { 23954,  23958,  23959,  23955}},
 { 24723,  47, { 23955,  23959,  23960,  23956}},
 { 24724,  47, { 23931,  23932,  23961,  23957}},
 { 24725,  47, { 23957,  23961,  23962,  23958}},
 { 24726,  47, { 23958,  23962,  23963,  23959}},
 { 24727,  47, { 23959,  23963,  23964,  23960}},
 { 24728,  47, { 23932,  23933,  23965,  23961}},
 { 24729,  47, { 23961,  23965,  23966,  23962}},
 { 24730,  47, { 23962,  23966,  23967,  23963}},
 { 24731,  47, { 23963,  23967,  23968,  23964}},
 { 24732,  47, { 23933,  23934,  23969,  23965}},
 { 24733,  47, { 23965,  23969,  23970,  23966}},
 { 24734,  47, { 23966,  23970,  23971,  23967}},
 { 24735,  47, { 23967,  23971,  23972,  23968}},
 { 24736,  47, { 23934,  23935,  23973,  23969}},
 { 24737,  47, { 23969,  23973,  23974,  23970}},
 { 24738,  47, { 23970,  23974,  23975,  23971}},
 { 24739,  47, { 23971,  23975,  23976,  23972}},
 { 24740,  47, { 23935,  23936,  23977,  23973}},
 { 24741,  47, { 23843,  23847,  23977,  23936}},
 { 24742,  47, { 23973,  23977,  23978,  23974}},
 { 24743,  47, { 23847,  23851,  23978,  23977}},
 { 24744,  47, { 23974,  23978,  23979,  23975}},
 { 24745,  47, { 23851,  23855,  23979,  23978}},
 { 24746,  47, { 23975,  23979,  23980,  23976}},
 { 24747,  47, { 23855,  23859,  23980,  23979}},
 { 24748,  47, { 23866,  23981,  23875,  23867}},
 { 24749,  47, { 23981,  23982,  23879,  23875}},
 { 24750,  47, { 23982,  23944,  23883,  23879}},
 { 24751,  47, { 23944,  23943,  23887,  23883}},
 { 24752,  47, { 23943,  23948,  23891,  23887}},
 { 24753,  47, { 23948,  23952,  23895,  23891}},
 { 24754,  47, { 23952,  23956,  23899,  23895}},
 { 24755,  47, { 23956,  23960,  23903,  23899}},
 { 24756,  47, { 23960,  23964,  23907,  23903}},
 { 24757,  47, { 23964,  23968,  23911,  23907}},
 { 24758,  47, { 23968,  23972,  23915,  23911}},
 { 24759,  47, { 23972,  23976,  23919,  23915}},
 { 24760,  47, { 23976,  23980,  23923,  23919}},
 { 24761,  47, { 23859,  23871,  23923,  23980}},
 { 24762,  47, { 23983,  23984,  23681,  23680}},
 { 24763,  47, { 23984,  23985,  23684,  23681}},
 { 24764,  47, { 23985,  23986,  23686,  23684}},
 { 24765,  47, { 23761,  23760,  23987,  23988}},
 { 24766,  47, { 23760,  23763,  23989,  23987}},
 { 24767,  47, { 23763,  23716,  23990,  23989}},
 { 24768,  47, { 23836,  23991,  23983,  23680}},
 { 24769,  47, { 23837,  23992,  23991,  23836}},
 { 24770,  47, { 23838,  23993,  23992,  23837}},
 { 24771,  47, { 23839,  23994,  23993,  23838}},
 { 24772,  47, { 23991,  23995,  23984,  23983}},
 { 24773,  47, { 23992,  23996,  23995,  23991}},
 { 24774,  47, { 23993,  23997,  23996,  23992}},
 { 24775,  47, { 23994,  23998,  23997,  23993}},
 { 24776,  47, { 23995,  23999,  23985,  23984}},
 { 24777,  47, { 23996,  24000,  23999,  23995}},
 { 24778,  47, { 23997,  24001,  24000,  23996}},
 { 24779,  47, { 23998,  24002,  24001,  23997}},
 { 24780,  47, { 23999,  24003,  23986,  23985}},
 { 24781,  47, { 24000,  24004,  24003,  23999}},
 { 24782,  47, { 24001,  24005,  24004,  24000}},
 { 24783,  47, { 24002,  24006,  24005,  24001}},
 { 24784,  47, { 23761,  23988,  24007,  23860}},
 { 24785,  47, { 23860,  24007,  24008,  23862}},
 { 24786,  47, { 23862,  24008,  24009,  23864}},
 { 24787,  47, { 23864,  24009,  24010,  23866}},
 { 24788,  47, { 23988,  23987,  24011,  24007}},
 { 24789,  47, { 24007,  24011,  24012,  24008}},
 { 24790,  47, { 24008,  24012,  24013,  24009}},
 { 24791,  47, { 24009,  24013,  24014,  24010}},
 { 24792,  47, { 23987,  23989,  24015,  24011}},
 { 24793,  47, { 24011,  24015,  24016,  24012}},
 { 24794,  47, { 24012,  24016,  24017,  24013}},
 { 24795,  47, { 24013,  24017,  24018,  24014}},
 { 24796,  47, { 23989,  23990,  24019,  24015}},
 { 24797,  47, { 24015,  24019,  24020,  24016}},
 { 24798,  47, { 24016,  24020,  24021,  24017}},
 { 24799,  47, { 24017,  24021,  24022,  24018}},
 { 24800,  47, { 23839,  23924,  23998,  23994}},
 { 24801,  47, { 23924,  23925,  24002,  23998}},
 { 24802,  47, { 23925,  23926,  24006,  24002}},
 { 24803,  47, { 24010,  24014,  23981,  23866}},
 { 24804,  47, { 24014,  24018,  23982,  23981}},
 { 24805,  47, { 24018,  24022,  23944,  23982}},
 { 24806,  49, { 24023,  24024,  24025,  24026}},
 { 24807,  49, { 24027,  24028,  24024,  24023}},
 { 24808,  49, { 24029,  24030,  24028,  24027}},
 { 24809,  49, { 24031,  24032,  24030,  24029}},
 { 24810,  49, { 24024,  24033,  24034,  24025}},
 { 24811,  49, { 24028,  24035,  24033,  24024}},
 { 24812,  49, { 24030,  24036,  24035,  24028}},
 { 24813,  49, { 24032,  24037,  24036,  24030}},
 { 24814,  49, { 24033,  24038,  24039,  24034}},
 { 24815,  49, { 24035,  24040,  24038,  24033}},
 { 24816,  49, { 24036,  24041,  24040,  24035}},
 { 24817,  49, { 24037,  24042,  24041,  24036}},
 { 24818,  49, { 24038,  24043,  24044,  24039}},
 { 24819,  49, { 24040,  24045,  24043,  24038}},
 { 24820,  49, { 24041,  24046,  24045,  24040}},
 { 24821,  49, { 24042,  24047,  24046,  24041}},
 { 24822,  49, { 24026,  24025,  24048,  24049}},
 { 24823,  49, { 24049,  24048,  24050,  24051}},
 { 24824,  49, { 24025,  24034,  24052,  24048}},
 { 24825,  49, { 24048,  24052,  24053,  24050}},
 { 24826,  49, { 24034,  24039,  24054,  24052}},
 { 24827,  49, { 24052,  24054,  24055,  24053}},
 { 24828,  49, { 24039,  24044,  24056,  24054}},
 { 24829,  49, { 24054,  24056,  24057,  24055}},
 { 24830,  49, { 24023,  24026,  24049,  24058}},
 { 24831,  49, { 24058,  24049,  24051,  24059}},
 { 24832,  49, { 24027,  24023,  24058,  24060}},
 { 24833,  49, { 24060,  24058,  24059,  24061}},
 { 24834,  49, { 24029,  24027,  24060,  24062}},
 { 24835,  49, { 24062,  24060,  24061,  24063}},
 { 24836,  49, { 24031,  24029,  24062,  24064}},
 { 24837,  49, { 24064,  24062,  24063,  24065}},
 { 24838,  49, { 24044,  24043,  24066,  24056}},
 { 24839,  49, { 24056,  24066,  24067,  24057}},
 { 24840,  49, { 24043,  24045,  24068,  24066}},
 { 24841,  49, { 24066,  24068,  24069,  24067}},
 { 24842,  49, { 24045,  24046,  24070,  24068}},
 { 24843,  49, { 24068,  24070,  24071,  24069}},
 { 24844,  49, { 24046,  24047,  24072,  24070}},
 { 24845,  49, { 24070,  24072,  24073,  24071}},
 { 24846,  49, { 24064,  24074,  24032,  24031}},
 { 24847,  49, { 24065,  24075,  24074,  24064}},
 { 24848,  49, { 24074,  24076,  24037,  24032}},
 { 24849,  49, { 24075,  24077,  24076,  24074}},
 { 24850,  49, { 24076,  24078,  24042,  24037}},
 { 24851,  49, { 24077,  24079,  24078,  24076}},
 { 24852,  49, { 24078,  24072,  24047,  24042}},
 { 24853,  49, { 24079,  24073,  24072,  24078}},
 { 24854,  49, { 24051,  24050,  24080,  24081}},
 { 24855,  49, { 24050,  24053,  24082,  24080}},
 { 24856,  49, { 24053,  24055,  24083,  24082}},
 { 24857,  49, { 24055,  24057,  24084,  24083}},
 { 24858,  49, { 24059,  24051,  24081,  24085}},
 { 24859,  49, { 24061,  24059,  24085,  24086}},
 { 24860,  49, { 24063,  24061,  24086,  24087}},
 { 24861,  49, { 24065,  24063,  24087,  24088}},
 { 24862,  49, { 24057,  24067,  24089,  24084}},
 { 24863,  49, { 24067,  24069,  24090,  24089}},
 { 24864,  49, { 24069,  24071,  24091,  24090}},
 { 24865,  49, { 24071,  24073,  24092,  24091}},
 { 24866,  49, { 24088,  24093,  24075,  24065}},
 { 24867,  49, { 24093,  24094,  24077,  24075}},
 { 24868,  49, { 24094,  24095,  24079,  24077}},
 { 24869,  49, { 24095,  24092,  24073,  24079}},
 { 24870,  49, { 24081,  24080,  24096,  24097}},
 { 24871,  49, { 24097,  24096,  24098,  24099}},
 { 24872,  49, { 24099,  24098,  24100,  24101}},
 { 24873,  49, { 24080,  24082,  24102,  24096}},
 { 24874,  49, { 24096,  24102,  24103,  24098}},
 { 24875,  49, { 24098,  24103,  24104,  24100}},
 { 24876,  49, { 24082,  24083,  24105,  24102}},
 { 24877,  49, { 24102,  24105,  24106,  24103}},
 { 24878,  49, { 24103,  24106,  24107,  24104}},
 { 24879,  49, { 24083,  24084,  24108,  24105}},
 { 24880,  49, { 24105,  24108,  24109,  24106}},
 { 24881,  49, { 24106,  24109,  24110,  24107}},
 { 24882,  49, { 24085,  24081,  24097,  24111}},
 { 24883,  49, { 24111,  24097,  24099,  24112}},
 { 24884,  49, { 24112,  24099,  24101,  24113}},
 { 24885,  49, { 24086,  24085,  24111,  24114}},
 { 24886,  49, { 24114,  24111,  24112,  24115}},
 { 24887,  49, { 24115,  24112,  24113,  24116}},
 { 24888,  49, { 24087,  24086,  24114,  24117}},
 { 24889,  49, { 24117,  24114,  24115,  24118}},
 { 24890,  49, { 24118,  24115,  24116,  24119}},
 { 24891,  49, { 24088,  24087,  24117,  24120}},
 { 24892,  49, { 24120,  24117,  24118,  24121}},
 { 24893,  49, { 24121,  24118,  24119,  24122}},
 { 24894,  49, { 24084,  24089,  24123,  24108}},
 { 24895,  49, { 24108,  24123,  24124,  24109}},
 { 24896,  49, { 24109,  24124,  24125,  24110}},
 { 24897,  49, { 24089,  24090,  24126,  24123}},
 { 24898,  49, { 24123,  24126,  24127,  24124}},
 { 24899,  49, { 24124,  24127,  24128,  24125}},
 { 24900,  49, { 24090,  24091,  24129,  24126}},
 { 24901,  49, { 24126,  24129,  24130,  24127}},
 { 24902,  49, { 24127,  24130,  24131,  24128}},
 { 24903,  49, { 24091,  24092,  24132,  24129}},
 { 24904,  49, { 24129,  24132,  24133,  24130}},
 { 24905,  49, { 24130,  24133,  24134,  24131}},
 { 24906,  49, { 24120,  24135,  24093,  24088}},
 { 24907,  49, { 24121,  24136,  24135,  24120}},
 { 24908,  49, { 24122,  24137,  24136,  24121}},
 { 24909,  49, { 24135,  24138,  24094,  24093}},
 { 24910,  49, { 24136,  24139,  24138,  24135}},
 { 24911,  49, { 24137,  24140,  24139,  24136}},
 { 24912,  49, { 24138,  24141,  24095,  24094}},
 { 24913,  49, { 24139,  24142,  24141,  24138}},
 { 24914,  49, { 24140,  24143,  24142,  24139}},
 { 24915,  49, { 24141,  24132,  24092,  24095}},
 { 24916,  49, { 24142,  24133,  24132,  24141}},
 { 24917,  49, { 24143,  24134,  24133,  24142}},
 { 24918,  49, { 24101,  24100,  24144,  24145}},
 { 24919,  49, { 24100,  24104,  24146,  24144}},
 { 24920,  49, { 24104,  24107,  24147,  24146}},
 { 24921,  49, { 24107,  24110,  24148,  24147}},
 { 24922,  49, { 24113,  24101,  24145,  24149}},
 { 24923,  49, { 24116,  24113,  24149,  24150}},
 { 24924,  49, { 24119,  24116,  24150,  24151}},
 { 24925,  49, { 24122,  24119,  24151,  24152}},
 { 24926,  49, { 24110,  24125,  24153,  24148}},
 { 24927,  49, { 24125,  24128,  24154,  24153}},
 { 24928,  49, { 24128,  24131,  24155,  24154}},
 { 24929,  49, { 24131,  24134,  24156,  24155}},
 { 24930,  49, { 24152,  24157,  24137,  24122}},
 { 24931,  49, { 24157,  24158,  24140,  24137}},
 { 24932,  49, { 24158,  24159,  24143,  24140}},
 { 24933,  49, { 24159,  24156,  24134,  24143}},
 { 24934,  49, { 24145,  24144,  24160,  24161}},
 { 24935,  49, { 24161,  24160,  24162,  24163}},
 { 24936,  49, { 24144,  24146,  24164,  24160}},
 { 24937,  49, { 24160,  24164,  24165,  24162}},
 { 24938,  49, { 24146,  24147,  24166,  24164}},
 { 24939,  49, { 24164,  24166,  24167,  24165}},
 { 24940,  49, { 24147,  24148,  24168,  24166}},
 { 24941,  49, { 24166,  24168,  24169,  24167}},
 { 24942,  49, { 24149,  24145,  24161,  24170}},
 { 24943,  49, { 24170,  24161,  24163,  24171}},
 { 24944,  49, { 24150,  24149,  24170,  24172}},
 { 24945,  49, { 24172,  24170,  24171,  24173}},
 { 24946,  49, { 24151,  24150,  24172,  24174}},
 { 24947,  49, { 24174,  24172,  24173,  24175}},
 { 24948,  49, { 24152,  24151,  24174,  24176}},
 { 24949,  49, { 24176,  24174,  24175,  24177}},
 { 24950,  49, { 24148,  24153,  24178,  24168}},
 { 24951,  49, { 24168,  24178,  24179,  24169}},
 { 24952,  49, { 24153,  24154,  24180,  24178}},
 { 24953,  49, { 24178,  24180,  24181,  24179}},
 { 24954,  49, { 24154,  24155,  24182,  24180}},
 { 24955,  49, { 24180,  24182,  24183,  24181}},
 { 24956,  49, { 24155,  24156,  24184,  24182}},
 { 24957,  49, { 24182,  24184,  24185,  24183}},
 { 24958,  49, { 24176,  24186,  24157,  24152}},
 { 24959,  49, { 24177,  24187,  24186,  24176}},
 { 24960,  49, { 24186,  24188,  24158,  24157}},
 { 24961,  49, { 24187,  24189,  24188,  24186}},
 { 24962,  49, { 24188,  24190,  24159,  24158}},
 { 24963,  49, { 24189,  24191,  24190,  24188}},
 { 24964,  49, { 24190,  24184,  24156,  24159}},
 { 24965,  49, { 24191,  24185,  24184,  24190}},
 { 24966,  49, { 24163,  24162,  24192,  24171}},
 { 24967,  49, { 24171,  24192,  24193,  24173}},
 { 24968,  49, { 24173,  24193,  24194,  24175}},
 { 24969,  49, { 24175,  24194,  24187,  24177}},
 { 24970,  49, { 24162,  24165,  24195,  24192}},
 { 24971,  49, { 24192,  24195,  24196,  24193}},
 { 24972,  49, { 24193,  24196,  24197,  24194}},
 { 24973,  49, { 24194,  24197,  24189,  24187}},
 { 24974,  49, { 24165,  24167,  24198,  24195}},
 { 24975,  49, { 24195,  24198,  24199,  24196}},
 { 24976,  49, { 24196,  24199,  24200,  24197}},
 { 24977,  49, { 24197,  24200,  24191,  24189}},
 { 24978,  49, { 24167,  24169,  24179,  24198}},
 { 24979,  49, { 24198,  24179,  24181,  24199}},
 { 24980,  49, { 24199,  24181,  24183,  24200}},
 { 24981,  49, { 24200,  24183,  24185,  24191}},
 { 28240,  52, { 27002,  27001,  27000,  27000}},
 { 28241,  52, { 27000,  27001,  27004,  27003}},
 { 28242,  52, { 27009,  27010,  27011,  27011}},
 { 28243,  52, { 27021,  27001,  27002,  27002}},
 { 28244,  52, { 27032,  27001,  27021,  27021}},
 { 28245,  52, { 27044,  27001,  27032,  27032}},
 { 28246,  52, { 27048,  27010,  27049,  27049}},
 { 28247,  52, { 27049,  27010,  27067,  27067}},
 { 28248,  52, { 27086,  27085,  27084,  27083}},
 { 28249,  52, { 27083,  27084,  27088,  27087}},
 { 28250,  52, { 27085,  27092,  27091,  27084}},
 { 28251,  52, { 27084,  27091,  27093,  27088}},
 { 28252,  52, { 27092,  27096,  27095,  27091}},
 { 28253,  52, { 27091,  27095,  27009,  27093}},
 { 28254,  52, { 27099,  27098,  27086,  27097}},
 { 28255,  52, { 27099,  27097,  27100,  27102}},
 { 28256,  52, { 27098,  27103,  27085,  27086}},
 { 28257,  52, { 27103,  27105,  27092,  27085}},
 { 28258,  52, { 27105,  27107,  27096,  27092}},
 { 28259,  52, { 27110,  27109,  27098,  27099}},
 { 28260,  52, { 27110,  27099,  27102,  27112}},
 { 28261,  52, { 27109,  27113,  27103,  27098}},
 { 28262,  52, { 27113,  27115,  27105,  27103}},
 { 28263,  52, { 27115,  27117,  27107,  27105}},
 { 28264,  52, { 27120,  27119,  27109,  27110}},
 { 28265,  52, { 27120,  27110,  27112,  27122}},
 { 28266,  52, { 27119,  27123,  27113,  27109}},
 { 28267,  52, { 27123,  27125,  27115,  27113}},
 { 28268,  52, { 27125,  27127,  27117,  27115}},
 { 28269,  52, { 27130,  27129,  27119,  27120}},
 { 28270,  52, { 27130,  27120,  27122,  27132}},
 { 28271,  52, { 27129,  27133,  27123,  27119}},
 { 28272,  52, { 27133,  27135,  27125,  27123}},
 { 28273,  52, { 27135,  27137,  27127,  27125}},
 { 28274,  52, { 27139,  27138,  27129,  27130}},
 { 28275,  52, { 27139,  27130,  27132,  27141}},
 { 28276,  52, { 27138,  27142,  27133,  27129}},
 { 28277,  52, { 27142,  27144,  27135,  27133}},
 { 28278,  52, { 27144,  27146,  27137,  27135}},
 { 28279,  52, { 27148,  27147,  27138,  27139}},
 { 28280,  52, { 27148,  27139,  27141,  27150}},
 { 28281,  52, { 27147,  27151,  27142,  27138}},
 { 28282,  52, { 27151,  27153,  27144,  27142}},
 { 28283,  52, { 27153,  27155,  27146,  27144}},
 { 28284,  52, { 27157,  27156,  27147,  27148}},
 { 28285,  52, { 27157,  27148,  27150,  27159}},
 { 28286,  52, { 27156,  27160,  27151,  27147}},
 { 28287,  52, { 27160,  27162,  27153,  27151}},
 { 28288,  52, { 27162,  27164,  27155,  27153}},
 { 28289,  52, { 27167,  27166,  27156,  27157}},
 { 28290,  52, { 27167,  27157,  27159,  27169}},
 { 28291,  52, { 27166,  27170,  27160,  27156}},
 { 28292,  52, { 27170,  27172,  27162,  27160}},
 { 28293,  52, { 27172,  27003,  27164,  27162}},
 { 28294,  52, { 27175,  27174,  27166,  27167}},
 { 28295,  52, { 27174,  27175,  27177,  27176}},
 { 28296,  52, { 27175,  27167,  27169,  27177}},
 { 28297,  52, { 27174,  27178,  27170,  27166}},
 { 28298,  52, { 27178,  27174,  27176,  27179}},
 { 28299,  52, { 27178,  27180,  27172,  27170}},
 { 28300,  52, { 27180,  27178,  27179,  27181}},
 { 28301,  52, { 27180,  27000,  27003,  27172}},
 { 28302,  52, { 27000,  27180,  27181,  27002}},
 { 28303,  52, { 27087,  27088,  27183,  27182}},
 { 28304,  52, { 27088,  27093,  27186,  27183}},
 { 28305,  52, { 27093,  27009,  27011,  27186}},
 { 28306,  52, { 27102,  27100,  27188,  27190}},
 { 28307,  52, { 27112,  27102,  27190,  27194}},
 { 28308,  52, { 27122,  27112,  27194,  27199}},
 { 28309,  52, { 27132,  27122,  27199,  27204}},
 { 28310,  52, { 27141,  27132,  27204,  27208}},
 { 28311,  52, { 27150,  27141,  27208,  27212}},
 { 28312,  52, { 27159,  27150,  27212,  27216}},
 { 28313,  52, { 27169,  27159,  27216,  27220}},
 { 28314,  52, { 27176,  27177,  27224,  27223}},
 { 28315,  52, { 27177,  27169,  27220,  27224}},
 { 28316,  52, { 27179,  27176,  27223,  27225}},
 { 28317,  52, { 27181,  27179,  27225,  27226}},
 { 28318,  52, { 27002,  27181,  27226,  27021}},
 { 28319,  52, { 27182,  27183,  27228,  27227}},
 { 28320,  52, { 27183,  27186,  27231,  27228}},
 { 28321,  52, { 27186,  27011,  27233,  27231}},
 { 28322,  52, { 27190,  27188,  27234,  27236}},
 { 28323,  52, { 27194,  27190,  27236,  27240}},
 { 28324,  52, { 27199,  27194,  27240,  27244}},
 { 28325,  52, { 27204,  27199,  27244,  27248}},
 { 28326,  52, { 27208,  27204,  27248,  27253}},
 { 28327,  52, { 27212,  27208,  27253,  27257}},
 { 28328,  52, { 27216,  27212,  27257,  27261}},
 { 28329,  52, { 27220,  27216,  27261,  27265}},
 { 28330,  52, { 27223,  27224,  27269,  27268}},
 { 28331,  52, { 27224,  27220,  27265,  27269}},
 { 28332,  52, { 27225,  27223,  27268,  27270}},
 { 28333,  52, { 27226,  27225,  27270,  27271}},
 { 28334,  52, { 27021,  27226,  27271,  27032}},
 { 28335,  52, { 27227,  27228,  27273,  27272}},
 { 28336,  52, { 27228,  27231,  27276,  27273}},
 { 28337,  52, { 27231,  27233,  27278,  27276}},
 { 28338,  52, { 27236,  27234,  27279,  27281}},
 { 28339,  52, { 27240,  27236,  27281,  27285}},
 { 28340,  52, { 27244,  27240,  27285,  27289}},
 { 28341,  52, { 27248,  27244,  27289,  27293}},
 { 28342,  52, { 27253,  27248,  27293,  27298}},
 { 28343,  52, { 27257,  27253,  27298,  27302}},
 { 28344,  52, { 27261,  27257,  27302,  27306}},
 { 28345,  52, { 27265,  27261,  27306,  27311}},
 { 28346,  52, { 27268,  27269,  27315,  27314}},
 { 28347,  52, { 27269,  27265,  27311,  27315}},
 { 28348,  52, { 27270,  27268,  27314,  27316}},
 { 28349,  52, { 27271,  27270,  27316,  27317}},
 { 28350,  52, { 27032,  27271,  27317,  27044}},
 { 28351,  52, { 27272,  27273,  27319,  27318}},
 { 28352,  52, { 27273,  27276,  27322,  27319}},
 { 28353,  52, { 27276,  27278,  27048,  27322}},
 { 28354,  52, { 27281,  27279,  27324,  27326}},
 { 28355,  52, { 27285,  27281,  27326,  27330}},
 { 28356,  52, { 27289,  27285,  27330,  27334}},
 { 28357,  52, { 27293,  27289,  27334,  27338}},
 { 28358,  52, { 27298,  27293,  27338,  27342}},
 { 28359,  52, { 27302,  27298,  27342,  27346}},
 { 28360,  52, { 27306,  27302,  27346,  27350}},
 { 28361,  52, { 27311,  27306,  27350,  27354}},
 { 28362,  52, { 27314,  27315,  27358,  27357}},
 { 28363,  52, { 27315,  27311,  27354,  27358}},
 { 28364,  52, { 27316,  27314,  27357,  27359}},
 { 28365,  52, { 27317,  27316,  27359,  27360}},
 { 28366,  52, { 27044,  27317,  27360,  27361}},
 { 28367,  52, { 27318,  27319,  27363,  27362}},
 { 28368,  52, { 27319,  27322,  27366,  27363}},
 { 28369,  52, { 27322,  27048,  27049,  27366}},
 { 28370,  52, { 27326,  27324,  27368,  27370}},
 { 28371,  52, { 27330,  27326,  27370,  27374}},
 { 28372,  52, { 27334,  27330,  27374,  27378}},
 { 28373,  52, { 27338,  27334,  27378,  27382}},
 { 28374,  52, { 27342,  27338,  27382,  27386}},
 { 28375,  52, { 27346,  27342,  27386,  27390}},
 { 28376,  52, { 27350,  27346,  27390,  27394}},
 { 28377,  52, { 27354,  27350,  27394,  27398}},
 { 28378,  52, { 27357,  27358,  27402,  27401}},
 { 28379,  52, { 27358,  27354,  27398,  27402}},
 { 28380,  52, { 27359,  27357,  27401,  27403}},
 { 28381,  52, { 27360,  27359,  27403,  27404}},
 { 28382,  52, { 27361,  27360,  27404,  27405}},
 { 28383,  52, { 27362,  27363,  27407,  27406}},
 { 28384,  52, { 27363,  27366,  27410,  27407}},
 { 28385,  52, { 27366,  27049,  27067,  27410}},
 { 28386,  52, { 27370,  27368,  27412,  27414}},
 { 28387,  52, { 27374,  27370,  27414,  27418}},
 { 28388,  52, { 27378,  27374,  27418,  27422}},
 { 28389,  52, { 27382,  27378,  27422,  27426}},
 { 28390,  52, { 27386,  27382,  27426,  27430}},
 { 28391,  52, { 27390,  27386,  27430,  27434}},
 { 28392,  52, { 27394,  27390,  27434,  27438}},
 { 28393,  52, { 27398,  27394,  27438,  27442}},
 { 28394,  52, { 27401,  27402,  27446,  27445}},
 { 28395,  52, { 27402,  27398,  27442,  27446}},
 { 28396,  52, { 27403,  27401,  27445,  27447}},
 { 28397,  52, { 27404,  27403,  27447,  27448}},
 { 28398,  52, { 27405,  27404,  27448,  27449}},
 { 28399,  52, { 27406,  27407,  27451,  27450}},
 { 28400,  52, { 27407,  27410,  27454,  27451}},
 { 28401,  52, { 27410,  27067,  27456,  27454}},
 { 28402,  52, { 27414,  27412,  27457,  27459}},
 { 28403,  52, { 27418,  27414,  27459,  27463}},
 { 28404,  52, { 27422,  27418,  27463,  27467}},
 { 28405,  52, { 27426,  27422,  27467,  27471}},
 { 28406,  52, { 27430,  27426,  27471,  27475}},
 { 28407,  52, { 27434,  27430,  27475,  27480}},
 { 28408,  52, { 27438,  27434,  27480,  27484}},
 { 28409,  52, { 27442,  27438,  27484,  27488}},
 { 28410,  52, { 27445,  27446,  27493,  27492}},
 { 28411,  52, { 27446,  27442,  27488,  27493}},
 { 28412,  52, { 27447,  27445,  27492,  27494}},
 { 28413,  52, { 27448,  27447,  27494,  27495}},
 { 28414,  52, { 27449,  27448,  27495,  27496}},
 { 28415,  52, { 27001,  27496,  27500,  27499}},
 { 28416,  52, { 27498,  27001,  27499,  27502}},
 { 28417,  52, { 27499,  27500,  27501,  27502}},
 { 28418,  52, { 27492,  27493,  27506,  27505}},
 { 28419,  52, { 27493,  27503,  27507,  27506}},
 { 28420,  52, { 27505,  27506,  27507,  27508}},
 { 28421,  52, { 27494,  27492,  27505,  27510}},
 { 28422,  52, { 27510,  27505,  27508,  27511}},
 { 28423,  52, { 27495,  27494,  27510,  27513}},
 { 28424,  52, { 27513,  27510,  27511,  27514}},
 { 28425,  52, { 27496,  27495,  27513,  27500}},
 { 28426,  52, { 27500,  27513,  27514,  27501}},
 { 28427,  52, { 27516,  27498,  27502,  27518}},
 { 28428,  52, { 27502,  27501,  27517,  27518}},
 { 28429,  52, { 27503,  27519,  27521,  27507}},
 { 28430,  52, { 27508,  27507,  27521,  27522}},
 { 28431,  52, { 27511,  27508,  27522,  27524}},
 { 28432,  52, { 27514,  27511,  27524,  27526}},
 { 28433,  52, { 27501,  27514,  27526,  27517}},
 { 28434,  52, { 27528,  27516,  27518,  27530}},
 { 28435,  52, { 27518,  27517,  27529,  27530}},
 { 28436,  52, { 27519,  27531,  27533,  27521}},
 { 28437,  52, { 27522,  27521,  27533,  27534}},
 { 28438,  52, { 27524,  27522,  27534,  27536}},
 { 28439,  52, { 27526,  27524,  27536,  27538}},
 { 28440,  52, { 27517,  27526,  27538,  27529}},
 { 28441,  52, { 27540,  27528,  27530,  27542}},
 { 28442,  52, { 27530,  27529,  27541,  27542}},
 { 28443,  52, { 27531,  27543,  27545,  27533}},
 { 28444,  52, { 27534,  27533,  27545,  27546}},
 { 28445,  52, { 27536,  27534,  27546,  27548}},
 { 28446,  52, { 27538,  27536,  27548,  27550}},
 { 28447,  52, { 27529,  27538,  27550,  27541}},
 { 28448,  52, { 27004,  27001,  27498,  27552}},
 { 28449,  52, { 27030,  27004,  27552,  27554}},
 { 28450,  52, { 27019,  27030,  27554,  27556}},
 { 28451,  52, { 27013,  27019,  27556,  27558}},
 { 28452,  52, { 27016,  27013,  27558,  27560}},
 { 28453,  52, { 27040,  27016,  27560,  27562}},
 { 28454,  52, { 27037,  27040,  27562,  27564}},
 { 28455,  52, { 27024,  27037,  27564,  27566}},
 { 28456,  52, { 27007,  27024,  27566,  27568}},
 { 28457,  52, { 27456,  27010,  27570,  27569}},
 { 28458,  52, { 27010,  27007,  27568,  27570}},
 { 28459,  52, { 27459,  27457,  27572,  27571}},
 { 28460,  52, { 27463,  27459,  27571,  27575}},
 { 28461,  52, { 27467,  27463,  27575,  27577}},
 { 28462,  52, { 27471,  27467,  27577,  27579}},
 { 28463,  52, { 27475,  27471,  27579,  27581}},
 { 28464,  52, { 27480,  27475,  27581,  27583}},
 { 28465,  52, { 27484,  27480,  27583,  27585}},
 { 28466,  52, { 27488,  27484,  27585,  27587}},
 { 28467,  52, { 27493,  27488,  27587,  27503}},
 { 28468,  52, { 27450,  27451,  27590,  27589}},
 { 28469,  52, { 27451,  27454,  27592,  27590}},
 { 28470,  52, { 27454,  27456,  27569,  27592}},
 { 28471,  52, { 27552,  27498,  27516,  27611}},
 { 28472,  52, { 27554,  27552,  27611,  27613}},
 { 28473,  52, { 27556,  27554,  27613,  27615}},
 { 28474,  52, { 27558,  27556,  27615,  27617}},
 { 28475,  52, { 27560,  27558,  27617,  27619}},
 { 28476,  52, { 27562,  27560,  27619,  27621}},
 { 28477,  52, { 27564,  27562,  27621,  27623}},
 { 28478,  52, { 27566,  27564,  27623,  27625}},
 { 28479,  52, { 27568,  27566,  27625,  27627}},
 { 28480,  52, { 27569,  27570,  27629,  27628}},
 { 28481,  52, { 27570,  27568,  27627,  27629}},
 { 28482,  52, { 27571,  27572,  27631,  27630}},
 { 28483,  52, { 27575,  27571,  27630,  27634}},
 { 28484,  52, { 27577,  27575,  27634,  27636}},
 { 28485,  52, { 27579,  27577,  27636,  27638}},
 { 28486,  52, { 27581,  27579,  27638,  27640}},
 { 28487,  52, { 27583,  27581,  27640,  27642}},
 { 28488,  52, { 27585,  27583,  27642,  27644}},
 { 28489,  52, { 27587,  27585,  27644,  27646}},
 { 28490,  52, { 27503,  27587,  27646,  27519}},
 { 28491,  52, { 27589,  27590,  27649,  27648}},
 { 28492,  52, { 27590,  27592,  27651,  27649}},
 { 28493,  52, { 27592,  27569,  27628,  27651}},
 { 28494,  52, { 27611,  27516,  27528,  27670}},
 { 28495,  52, { 27613,  27611,  27670,  27672}},
 { 28496,  52, { 27615,  27613,  27672,  27674}},
 { 28497,  52, { 27617,  27615,  27674,  27676}},
 { 28498,  52, { 27619,  27617,  27676,  27678}},
 { 28499,  52, { 27621,  27619,  27678,  27680}},
 { 28500,  52, { 27623,  27621,  27680,  27682}},
 { 28501,  52, { 27625,  27623,  27682,  27684}},
 { 28502,  52, { 27627,  27625,  27684,  27686}},
 { 28503,  52, { 27628,  27629,  27688,  27687}},
 { 28504,  52, { 27629,  27627,  27686,  27688}},
 { 28505,  52, { 27630,  27631,  27690,  27689}},
 { 28506,  52, { 27634,  27630,  27689,  27693}},
 { 28507,  52, { 27636,  27634,  27693,  27695}},
 { 28508,  52, { 27638,  27636,  27695,  27697}},
 { 28509,  52, { 27640,  27638,  27697,  27699}},
 { 28510,  52, { 27642,  27640,  27699,  27701}},
 { 28511,  52, { 27644,  27642,  27701,  27703}},
 { 28512,  52, { 27646,  27644,  27703,  27705}},
 { 28513,  52, { 27519,  27646,  27705,  27531}},
 { 28514,  52, { 27648,  27649,  27708,  27707}},
 { 28515,  52, { 27649,  27651,  27710,  27708}},
 { 28516,  52, { 27651,  27628,  27687,  27710}},
 { 28517,  52, { 27670,  27528,  27540,  27729}},
 { 28518,  52, { 27672,  27670,  27729,  27731}},
 { 28519,  52, { 27674,  27672,  27731,  27733}},
 { 28520,  52, { 27676,  27674,  27733,  27735}},
 { 28521,  52, { 27678,  27676,  27735,  27737}},
 { 28522,  52, { 27680,  27678,  27737,  27739}},
 { 28523,  52, { 27682,  27680,  27739,  27741}},
 { 28524,  52, { 27684,  27682,  27741,  27743}},
 { 28525,  52, { 27686,  27684,  27743,  27745}},
 { 28526,  52, { 27687,  27688,  27747,  27746}},
 { 28527,  52, { 27688,  27686,  27745,  27747}},
 { 28528,  52, { 27689,  27690,  27749,  27748}},
 { 28529,  52, { 27693,  27689,  27748,  27752}},
 { 28530,  52, { 27695,  27693,  27752,  27754}},
 { 28531,  52, { 27697,  27695,  27754,  27756}},
 { 28532,  52, { 27699,  27697,  27756,  27758}},
 { 28533,  52, { 27701,  27699,  27758,  27760}},
 { 28534,  52, { 27703,  27701,  27760,  27762}},
 { 28535,  52, { 27705,  27703,  27762,  27764}},
 { 28536,  52, { 27531,  27705,  27764,  27543}},
 { 28537,  52, { 27707,  27708,  27767,  27766}},
 { 28538,  52, { 27708,  27710,  27769,  27767}},
 { 28539,  52, { 27710,  27687,  27746,  27769}},
 { 28540,  52, { 27009,  27095,  27010,  27010}},
 { 28541,  52, { 27010,  27095,  27096,  27007}},
 { 28542,  52, { 27233,  27011,  27010,  27010}},
 { 28543,  52, { 27010,  27278,  27233,  27233}},
 { 28544,  52, { 27048,  27278,  27010,  27010}},
 { 28545,  52, { 27067,  27010,  27456,  27456}},
 { 28546,  52, { 27007,  27096,  27107,  27024}},
 { 28547,  52, { 27024,  27107,  27117,  27037}},
 { 28548,  52, { 27127,  27040,  27037,  27117}},
 { 28549,  52, { 27137,  27016,  27040,  27127}},
 { 28550,  52, { 27146,  27013,  27016,  27137}},
 { 28551,  52, { 27155,  27019,  27013,  27146}},
 { 28552,  52, { 27787,  27707,  27766,  27788}},
 { 28553,  52, { 27690,  27787,  27788,  27749}},
 { 28554,  52, { 27789,  27648,  27707,  27787}},
 { 28555,  52, { 27631,  27789,  27787,  27690}},
 { 28556,  52, { 27572,  27790,  27789,  27631}},
 { 28557,  52, { 27790,  27589,  27648,  27789}},
 { 28558,  52, { 27791,  27450,  27589,  27790}},
 { 28559,  52, { 27457,  27791,  27790,  27572}},
 { 28560,  52, { 27792,  27406,  27450,  27791}},
 { 28561,  52, { 27412,  27792,  27791,  27457}},
 { 28562,  52, { 27368,  27793,  27792,  27412}},
 { 28563,  52, { 27793,  27362,  27406,  27792}},
 { 28564,  52, { 27318,  27362,  27793,  27794}},
 { 28565,  52, { 27794,  27793,  27368,  27324}},
 { 28566,  52, { 27279,  27795,  27794,  27324}},
 { 28567,  52, { 27795,  27272,  27318,  27794}},
 { 28568,  52, { 27272,  27795,  27796,  27227}},
 { 28569,  52, { 27796,  27795,  27279,  27234}},
 { 28570,  52, { 27234,  27188,  27797,  27796}},
 { 28571,  52, { 27796,  27797,  27182,  27227}},
 { 28572,  52, { 27182,  27797,  27798,  27087}},
 { 28573,  52, { 27797,  27188,  27100,  27798}},
 { 28574,  52, { 27100,  27097,  27799,  27798}},
 { 28575,  52, { 27097,  27086,  27083,  27799}},
 { 28576,  52, { 27083,  27087,  27798,  27799}},
 { 28577,  52, { 27019,  27155,  27164,  27030}},
 { 28578,  52, { 27449,  27001,  27405,  27405}},
 { 28579,  52, { 27449,  27496,  27001,  27001}},
 { 28580,  52, { 27030,  27164,  27003,  27004}},
 { 28581,  52, { 27405,  27001,  27361,  27361}},
 { 28582,  52, { 27361,  27001,  27044,  27044}},
 { 28583,  52, { 27802,  27801,  27800,  27800}},
 { 28584,  52, { 27802,  27800,  27803,  27805}},
 { 28585,  52, { 27809,  27810,  27811,  27811}},
 { 28586,  52, { 27802,  27821,  27801,  27801}},
 { 28587,  52, { 27802,  27832,  27821,  27821}},
 { 28588,  52, { 27802,  27844,  27832,  27832}},
 { 28589,  52, { 27848,  27849,  27811,  27811}},
 { 28590,  52, { 27849,  27867,  27811,  27811}},
 { 28591,  52, { 27886,  27885,  27884,  27883}},
 { 28592,  52, { 27886,  27883,  27887,  27890}},
 { 28593,  52, { 27892,  27891,  27885,  27886}},
 { 28594,  52, { 27892,  27886,  27890,  27894}},
 { 28595,  52, { 27896,  27895,  27891,  27892}},
 { 28596,  52, { 27896,  27892,  27894,  27809}},
 { 28597,  52, { 27884,  27899,  27898,  27897}},
 { 28598,  52, { 27897,  27898,  27901,  27900}},
 { 28599,  52, { 27885,  27903,  27899,  27884}},
 { 28600,  52, { 27891,  27905,  27903,  27885}},
 { 28601,  52, { 27895,  27907,  27905,  27891}},
 { 28602,  52, { 27899,  27910,  27909,  27898}},
 { 28603,  52, { 27898,  27909,  27911,  27901}},
 { 28604,  52, { 27903,  27913,  27910,  27899}},
 { 28605,  52, { 27905,  27915,  27913,  27903}},
 { 28606,  52, { 27907,  27917,  27915,  27905}},
 { 28607,  52, { 27910,  27920,  27919,  27909}},
 { 28608,  52, { 27909,  27919,  27921,  27911}},
 { 28609,  52, { 27913,  27923,  27920,  27910}},
 { 28610,  52, { 27915,  27925,  27923,  27913}},
 { 28611,  52, { 27917,  27927,  27925,  27915}},
 { 28612,  52, { 27920,  27930,  27929,  27919}},
 { 28613,  52, { 27919,  27929,  27931,  27921}},
 { 28614,  52, { 27923,  27933,  27930,  27920}},
 { 28615,  52, { 27925,  27935,  27933,  27923}},
 { 28616,  52, { 27927,  27937,  27935,  27925}},
 { 28617,  52, { 27930,  27939,  27938,  27929}},
 { 28618,  52, { 27929,  27938,  27940,  27931}},
 { 28619,  52, { 27933,  27942,  27939,  27930}},
 { 28620,  52, { 27935,  27944,  27942,  27933}},
 { 28621,  52, { 27937,  27946,  27944,  27935}},
 { 28622,  52, { 27939,  27948,  27947,  27938}},
 { 28623,  52, { 27938,  27947,  27949,  27940}},
 { 28624,  52, { 27942,  27951,  27948,  27939}},
 { 28625,  52, { 27944,  27953,  27951,  27942}},
 { 28626,  52, { 27946,  27955,  27953,  27944}},
 { 28627,  52, { 27948,  27957,  27956,  27947}},
 { 28628,  52, { 27947,  27956,  27958,  27949}},
 { 28629,  52, { 27951,  27960,  27957,  27948}},
 { 28630,  52, { 27953,  27962,  27960,  27951}},
 { 28631,  52, { 27955,  27964,  27962,  27953}},
 { 28632,  52, { 27957,  27967,  27966,  27956}},
 { 28633,  52, { 27956,  27966,  27968,  27958}},
 { 28634,  52, { 27960,  27970,  27967,  27957}},
 { 28635,  52, { 27962,  27972,  27970,  27960}},
 { 28636,  52, { 27964,  27803,  27972,  27962}},
 { 28637,  52, { 27967,  27975,  27974,  27966}},
 { 28638,  52, { 27966,  27974,  27976,  27968}},
 { 28639,  52, { 27974,  27975,  27977,  27976}},
 { 28640,  52, { 27970,  27978,  27975,  27967}},
 { 28641,  52, { 27975,  27978,  27979,  27977}},
 { 28642,  52, { 27972,  27980,  27978,  27970}},
 { 28643,  52, { 27978,  27980,  27981,  27979}},
 { 28644,  52, { 27803,  27800,  27980,  27972}},
 { 28645,  52, { 27980,  27800,  27801,  27981}},
 { 28646,  52, { 27890,  27887,  27982,  27985}},
 { 28647,  52, { 27894,  27890,  27985,  27987}},
 { 28648,  52, { 27809,  27894,  27987,  27810}},
 { 28649,  52, { 27900,  27901,  27989,  27988}},
 { 28650,  52, { 27901,  27911,  27993,  27989}},
 { 28651,  52, { 27911,  27921,  27998,  27993}},
 { 28652,  52, { 27921,  27931,  28003,  27998}},
 { 28653,  52, { 27931,  27940,  28007,  28003}},
 { 28654,  52, { 27940,  27949,  28011,  28007}},
 { 28655,  52, { 27949,  27958,  28015,  28011}},
 { 28656,  52, { 27958,  27968,  28019,  28015}},
 { 28657,  52, { 27968,  27976,  28023,  28019}},
 { 28658,  52, { 27976,  27977,  28024,  28023}},
 { 28659,  52, { 27977,  27979,  28025,  28024}},
 { 28660,  52, { 27979,  27981,  28026,  28025}},
 { 28661,  52, { 27981,  27801,  27821,  28026}},
 { 28662,  52, { 27985,  27982,  28027,  28030}},
 { 28663,  52, { 27987,  27985,  28030,  28032}},
 { 28664,  52, { 27810,  27987,  28032,  28033}},
 { 28665,  52, { 27988,  27989,  28035,  28034}},
 { 28666,  52, { 27989,  27993,  28039,  28035}},
 { 28667,  52, { 27993,  27998,  28043,  28039}},
 { 28668,  52, { 27998,  28003,  28047,  28043}},
 { 28669,  52, { 28003,  28007,  28052,  28047}},
 { 28670,  52, { 28007,  28011,  28056,  28052}},
 { 28671,  52, { 28011,  28015,  28060,  28056}},
 { 28672,  52, { 28015,  28019,  28064,  28060}},
 { 28673,  52, { 28019,  28023,  28068,  28064}},
 { 28674,  52, { 28023,  28024,  28069,  28068}},
 { 28675,  52, { 28024,  28025,  28070,  28069}},
 { 28676,  52, { 28025,  28026,  28071,  28070}},
 { 28677,  52, { 28026,  27821,  27832,  28071}},
 { 28678,  52, { 28030,  28027,  28072,  28075}},
 { 28679,  52, { 28032,  28030,  28075,  28077}},
 { 28680,  52, { 28033,  28032,  28077,  28078}},
 { 28681,  52, { 28034,  28035,  28080,  28079}},
 { 28682,  52, { 28035,  28039,  28084,  28080}},
 { 28683,  52, { 28039,  28043,  28088,  28084}},
 { 28684,  52, { 28043,  28047,  28092,  28088}},
 { 28685,  52, { 28047,  28052,  28097,  28092}},
 { 28686,  52, { 28052,  28056,  28101,  28097}},
 { 28687,  52, { 28056,  28060,  28105,  28101}},
 { 28688,  52, { 28060,  28064,  28110,  28105}},
 { 28689,  52, { 28064,  28068,  28114,  28110}},
 { 28690,  52, { 28068,  28069,  28115,  28114}},
 { 28691,  52, { 28069,  28070,  28116,  28115}},
 { 28692,  52, { 28070,  28071,  28117,  28116}},
 { 28693,  52, { 28071,  27832,  27844,  28117}},
 { 28694,  52, { 28075,  28072,  28118,  28121}},
 { 28695,  52, { 28077,  28075,  28121,  28123}},
 { 28696,  52, { 28078,  28077,  28123,  27848}},
 { 28697,  52, { 28079,  28080,  28125,  28124}},
 { 28698,  52, { 28080,  28084,  28129,  28125}},
 { 28699,  52, { 28084,  28088,  28133,  28129}},
 { 28700,  52, { 28088,  28092,  28137,  28133}},
 { 28701,  52, { 28092,  28097,  28141,  28137}},
 { 28702,  52, { 28097,  28101,  28145,  28141}},
 { 28703,  52, { 28101,  28105,  28149,  28145}},
 { 28704,  52, { 28105,  28110,  28153,  28149}},
 { 28705,  52, { 28110,  28114,  28157,  28153}},
 { 28706,  52, { 28114,  28115,  28158,  28157}},
 { 28707,  52, { 28115,  28116,  28159,  28158}},
 { 28708,  52, { 28116,  28117,  28160,  28159}},
 { 28709,  52, { 28117,  27844,  28161,  28160}},
 { 28710,  52, { 28121,  28118,  28162,  28165}},
 { 28711,  52, { 28123,  28121,  28165,  28167}},
 { 28712,  52, { 27848,  28123,  28167,  27849}},
 { 28713,  52, { 28124,  28125,  28169,  28168}},
 { 28714,  52, { 28125,  28129,  28173,  28169}},
 { 28715,  52, { 28129,  28133,  28177,  28173}},
 { 28716,  52, { 28133,  28137,  28181,  28177}},
 { 28717,  52, { 28137,  28141,  28185,  28181}},
 { 28718,  52, { 28141,  28145,  28189,  28185}},
 { 28719,  52, { 28145,  28149,  28193,  28189}},
 { 28720,  52, { 28149,  28153,  28197,  28193}},
 { 28721,  52, { 28153,  28157,  28201,  28197}},
 { 28722,  52, { 28157,  28158,  28202,  28201}},
 { 28723,  52, { 28158,  28159,  28203,  28202}},
 { 28724,  52, { 28159,  28160,  28204,  28203}},
 { 28725,  52, { 28160,  28161,  28205,  28204}},
 { 28726,  52, { 28165,  28162,  28206,  28209}},
 { 28727,  52, { 28167,  28165,  28209,  28211}},
 { 28728,  52, { 27849,  28167,  28211,  27867}},
 { 28729,  52, { 28168,  28169,  28213,  28212}},
 { 28730,  52, { 28169,  28173,  28217,  28213}},
 { 28731,  52, { 28173,  28177,  28221,  28217}},
 { 28732,  52, { 28177,  28181,  28225,  28221}},
 { 28733,  52, { 28181,  28185,  28229,  28225}},
 { 28734,  52, { 28185,  28189,  28233,  28229}},
 { 28735,  52, { 28189,  28193,  28237,  28233}},
 { 28736,  52, { 28193,  28197,  28241,  28237}},
 { 28737,  52, { 28197,  28201,  28245,  28241}},
 { 28738,  52, { 28201,  28202,  28246,  28245}},
 { 28739,  52, { 28202,  28203,  28247,  28246}},
 { 28740,  52, { 28203,  28204,  28248,  28247}},
 { 28741,  52, { 28204,  28205,  28249,  28248}},
 { 28742,  52, { 28209,  28206,  28250,  28253}},
 { 28743,  52, { 28211,  28209,  28253,  28255}},
 { 28744,  52, { 27867,  28211,  28255,  28256}},
 { 28745,  52, { 28212,  28213,  28258,  28257}},
 { 28746,  52, { 28213,  28217,  28262,  28258}},
 { 28747,  52, { 28217,  28221,  28266,  28262}},
 { 28748,  52, { 28221,  28225,  28270,  28266}},
 { 28749,  52, { 28225,  28229,  28274,  28270}},
 { 28750,  52, { 28229,  28233,  28279,  28274}},
 { 28751,  52, { 28233,  28237,  28283,  28279}},
 { 28752,  52, { 28237,  28241,  28287,  28283}},
 { 28753,  52, { 28241,  28245,  28292,  28287}},
 { 28754,  52, { 28245,  28246,  28293,  28292}},
 { 28755,  52, { 28246,  28247,  28294,  28293}},
 { 28756,  52, { 28247,  28248,  28295,  28294}},
 { 28757,  52, { 28248,  28249,  28296,  28295}},
 { 28758,  52, { 27802,  28297,  28300,  28299}},
 { 28759,  52, { 28296,  27802,  28299,  28302}},
 { 28760,  52, { 28299,  28300,  28301,  28302}},
 { 28761,  52, { 28304,  28292,  28308,  28307}},
 { 28762,  52, { 28292,  28293,  28305,  28308}},
 { 28763,  52, { 28305,  28306,  28307,  28308}},
 { 28764,  52, { 28293,  28294,  28310,  28305}},
 { 28765,  52, { 28310,  28311,  28306,  28305}},
 { 28766,  52, { 28294,  28295,  28313,  28310}},
 { 28767,  52, { 28313,  28314,  28311,  28310}},
 { 28768,  52, { 28295,  28296,  28302,  28313}},
 { 28769,  52, { 28302,  28301,  28314,  28313}},
 { 28770,  52, { 28297,  28315,  28317,  28300}},
 { 28771,  52, { 28300,  28317,  28318,  28301}},
 { 28772,  52, { 28320,  28304,  28307,  28322}},
 { 28773,  52, { 28306,  28321,  28322,  28307}},
 { 28774,  52, { 28311,  28324,  28321,  28306}},
 { 28775,  52, { 28314,  28326,  28324,  28311}},
 { 28776,  52, { 28301,  28318,  28326,  28314}},
 { 28777,  52, { 28315,  28327,  28329,  28317}},
 { 28778,  52, { 28317,  28329,  28330,  28318}},
 { 28779,  52, { 28332,  28320,  28322,  28334}},
 { 28780,  52, { 28321,  28333,  28334,  28322}},
 { 28781,  52, { 28324,  28336,  28333,  28321}},
 { 28782,  52, { 28326,  28338,  28336,  28324}},
 { 28783,  52, { 28318,  28330,  28338,  28326}},
 { 28784,  52, { 28327,  27540,  27542,  28329}},
 { 28785,  52, { 28329,  27542,  27541,  28330}},
 { 28786,  52, { 27543,  28332,  28334,  27545}},
 { 28787,  52, { 28333,  27546,  27545,  28334}},
 { 28788,  52, { 28336,  27548,  27546,  28333}},
 { 28789,  52, { 28338,  27550,  27548,  28336}},
 { 28790,  52, { 28330,  27541,  27550,  28338}},
 { 28791,  52, { 27802,  27805,  28339,  28297}},
 { 28792,  52, { 27805,  27831,  28341,  28339}},
 { 28793,  52, { 27831,  27820,  28343,  28341}},
 { 28794,  52, { 27820,  27814,  28345,  28343}},
 { 28795,  52, { 27814,  27817,  28347,  28345}},
 { 28796,  52, { 27817,  27841,  28349,  28347}},
 { 28797,  52, { 27841,  27838,  28351,  28349}},
 { 28798,  52, { 27838,  27825,  28353,  28351}},
 { 28799,  52, { 27825,  27808,  28355,  28353}},
 { 28800,  52, { 27808,  27811,  28357,  28355}},
 { 28801,  52, { 27811,  28256,  28358,  28357}},
 { 28802,  52, { 28257,  28258,  28359,  28362}},
 { 28803,  52, { 28258,  28262,  28363,  28359}},
 { 28804,  52, { 28262,  28266,  28365,  28363}},
 { 28805,  52, { 28266,  28270,  28367,  28365}},
 { 28806,  52, { 28270,  28274,  28369,  28367}},
 { 28807,  52, { 28274,  28279,  28371,  28369}},
 { 28808,  52, { 28279,  28283,  28373,  28371}},
 { 28809,  52, { 28283,  28287,  28375,  28373}},
 { 28810,  52, { 28287,  28292,  28304,  28375}},
 { 28811,  52, { 28253,  28250,  28379,  28378}},
 { 28812,  52, { 28255,  28253,  28378,  28381}},
 { 28813,  52, { 28256,  28255,  28381,  28358}},
 { 28814,  52, { 28297,  28339,  28398,  28315}},
 { 28815,  52, { 28339,  28341,  28400,  28398}},
 { 28816,  52, { 28341,  28343,  28402,  28400}},
 { 28817,  52, { 28343,  28345,  28404,  28402}},
 { 28818,  52, { 28345,  28347,  28406,  28404}},
 { 28819,  52, { 28347,  28349,  28408,  28406}},
 { 28820,  52, { 28349,  28351,  28410,  28408}},
 { 28821,  52, { 28351,  28353,  28412,  28410}},
 { 28822,  52, { 28353,  28355,  28414,  28412}},
 { 28823,  52, { 28355,  28357,  28416,  28414}},
 { 28824,  52, { 28357,  28358,  28417,  28416}},
 { 28825,  52, { 28362,  28359,  28418,  28421}},
 { 28826,  52, { 28359,  28363,  28422,  28418}},
 { 28827,  52, { 28363,  28365,  28424,  28422}},
 { 28828,  52, { 28365,  28367,  28426,  28424}},
 { 28829,  52, { 28367,  28369,  28428,  28426}},
 { 28830,  52, { 28369,  28371,  28430,  28428}},
 { 28831,  52, { 28371,  28373,  28432,  28430}},
 { 28832,  52, { 28373,  28375,  28434,  28432}},
 { 28833,  52, { 28375,  28304,  28320,  28434}},
 { 28834,  52, { 28378,  28379,  28438,  28437}},
 { 28835,  52, { 28381,  28378,  28437,  28440}},
 { 28836,  52, { 28358,  28381,  28440,  28417}},
 { 28837,  52, { 28315,  28398,  28457,  28327}},
 { 28838,  52, { 28398,  28400,  28459,  28457}},
 { 28839,  52, { 28400,  28402,  28461,  28459}},
 { 28840,  52, { 28402,  28404,  28463,  28461}},
 { 28841,  52, { 28404,  28406,  28465,  28463}},
 { 28842,  52, { 28406,  28408,  28467,  28465}},
 { 28843,  52, { 28408,  28410,  28469,  28467}},
 { 28844,  52, { 28410,  28412,  28471,  28469}},
 { 28845,  52, { 28412,  28414,  28473,  28471}},
 { 28846,  52, { 28414,  28416,  28475,  28473}},
 { 28847,  52, { 28416,  28417,  28476,  28475}},
 { 28848,  52, { 28421,  28418,  28477,  28480}},
 { 28849,  52, { 28418,  28422,  28481,  28477}},
 { 28850,  52, { 28422,  28424,  28483,  28481}},
 { 28851,  52, { 28424,  28426,  28485,  28483}},
 { 28852,  52, { 28426,  28428,  28487,  28485}},
 { 28853,  52, { 28428,  28430,  28489,  28487}},
 { 28854,  52, { 28430,  28432,  28491,  28489}},
 { 28855,  52, { 28432,  28434,  28493,  28491}},
 { 28856,  52, { 28434,  28320,  28332,  28493}},
 { 28857,  52, { 28437,  28438,  28497,  28496}},
 { 28858,  52, { 28440,  28437,  28496,  28499}},
 { 28859,  52, { 28417,  28440,  28499,  28476}},
 { 28860,  52, { 28327,  28457,  27729,  27540}},
 { 28861,  52, { 28457,  28459,  27731,  27729}},
 { 28862,  52, { 28459,  28461,  27733,  27731}},
 { 28863,  52, { 28461,  28463,  27735,  27733}},
 { 28864,  52, { 28463,  28465,  27737,  27735}},
 { 28865,  52, { 28465,  28467,  27739,  27737}},
 { 28866,  52, { 28467,  28469,  27741,  27739}},
 { 28867,  52, { 28469,  28471,  27743,  27741}},
 { 28868,  52, { 28471,  28473,  27745,  27743}},
 { 28869,  52, { 28473,  28475,  27747,  27745}},
 { 28870,  52, { 28475,  28476,  27746,  27747}},
 { 28871,  52, { 28480,  28477,  27748,  27749}},
 { 28872,  52, { 28477,  28481,  27752,  27748}},
 { 28873,  52, { 28481,  28483,  27754,  27752}},
 { 28874,  52, { 28483,  28485,  27756,  27754}},
 { 28875,  52, { 28485,  28487,  27758,  27756}},
 { 28876,  52, { 28487,  28489,  27760,  27758}},
 { 28877,  52, { 28489,  28491,  27762,  27760}},
 { 28878,  52, { 28491,  28493,  27764,  27762}},
 { 28879,  52, { 28493,  28332,  27543,  27764}},
 { 28880,  52, { 28496,  28497,  27766,  27767}},
 { 28881,  52, { 28499,  28496,  27767,  27769}},
 { 28882,  52, { 28476,  28499,  27769,  27746}},
 { 28883,  52, { 27896,  27809,  27811,  27811}},
 { 28884,  52, { 27896,  27811,  27808,  27895}},
 { 28885,  52, { 27810,  28033,  27811,  27811}},
 { 28886,  52, { 28078,  27811,  28033,  28033}},
 { 28887,  52, { 28078,  27848,  27811,  27811}},
 { 28888,  52, { 27811,  27867,  28256,  28256}},
 { 28889,  52, { 27895,  27808,  27825,  27907}},
 { 28890,  52, { 27907,  27825,  27838,  27917}},
 { 28891,  52, { 27841,  27927,  27917,  27838}},
 { 28892,  52, { 27817,  27937,  27927,  27841}},
 { 28893,  52, { 27814,  27946,  27937,  27817}},
 { 28894,  52, { 27820,  27955,  27946,  27814}},
 { 28895,  52, { 28516,  28480,  27749,  27788}},
 { 28896,  52, { 28497,  28516,  27788,  27766}},
 { 28897,  52, { 28517,  28421,  28480,  28516}},
 { 28898,  52, { 28438,  28517,  28516,  28497}},
 { 28899,  52, { 28379,  28518,  28517,  28438}},
 { 28900,  52, { 28518,  28362,  28421,  28517}},
 { 28901,  52, { 28519,  28257,  28362,  28518}},
 { 28902,  52, { 28250,  28519,  28518,  28379}},
 { 28903,  52, { 28520,  28212,  28257,  28519}},
 { 28904,  52, { 28206,  28520,  28519,  28250}},
 { 28905,  52, { 28162,  28521,  28520,  28206}},
 { 28906,  52, { 28521,  28168,  28212,  28520}},
 { 28907,  52, { 28521,  28162,  28118,  28522}},
 { 28908,  52, { 28521,  28522,  28124,  28168}},
 { 28909,  52, { 28072,  28523,  28522,  28118}},
 { 28910,  52, { 28523,  28079,  28124,  28522}},
 { 28911,  52, { 28524,  28523,  28072,  28027}},
 { 28912,  52, { 28523,  28524,  28034,  28079}},
 { 28913,  52, { 27988,  28034,  28524,  28525}},
 { 28914,  52, { 28524,  28027,  27982,  28525}},
 { 28915,  52, { 27988,  28525,  28526,  27900}},
 { 28916,  52, { 28525,  27982,  27887,  28526}},
 { 28917,  52, { 27884,  27897,  28527,  27883}},
 { 28918,  52, { 27897,  27900,  28526,  28527}},
 { 28919,  52, { 27883,  28527,  28526,  27887}},
 { 28920,  52, { 27955,  27820,  27831,  27964}},
 { 28921,  52, { 28249,  28205,  27802,  27802}},
 { 28922,  52, { 28296,  28249,  27802,  27802}},
 { 28923,  52, { 27964,  27831,  27805,  27803}},
 { 28924,  52, { 28205,  28161,  27802,  27802}},
 { 28925,  52, { 28161,  27844,  27802,  27802}},
 {101384,  25, { 10290,  10302, 101531, 101532}},
 {101385,  25, { 10303,  10291, 101532, 101531}},
 {101386,  25, { 10291,  10279, 101533, 101532}},
 {101387,  25, { 10278,  10290, 101532, 101533}},
 {101388,  25, { 10137,  10149, 101535, 101536}},
 {101389,  25, { 10150,  10138, 101536, 101535}},
 {101390,  25, { 10138,  10126, 101537, 101536}},
 {101391,  25, { 10125,  10137, 101536, 101537}},
 {101392,  25, {  9984,   9996, 101539, 101540}},
 {101393,  25, {  9997,   9985, 101540, 101539}},
 {101394,  25, {  9985,   9973, 101541, 101540}},
 {101395,  25, {  9972,   9984, 101540, 101541}},
 {101396,  25, {  9831,   9843, 101543, 101544}},
 {101397,  25, {  9844,   9832, 101544, 101543}},
 {101398,  25, {  9832,   9820, 101545, 101544}},
 {101399,  25, {  9819,   9831, 101544, 101545}},
 {101400,  25, {  9678,   9690, 101547, 101548}},
 {101401,  25, {  9691,   9679, 101548, 101547}},
 {101402,  25, {  9679,   9667, 101549, 101548}},
 {101403,  25, {  9666,   9678, 101548, 101549}},
 {101444,  25, { 11056,  11044, 101580, 101581}},
 {101445,  25, { 11043,  11055, 101581, 101580}},
 {101446,  25, { 11055,  11067, 101582, 101581}},
 {101447,  25, { 11068,  11056, 101581, 101582}},
 {101448,  25, { 10903,  10891, 101584, 101585}},
 {101449,  25, { 10890,  10902, 101585, 101584}},
 {101450,  25, { 10902,  10914, 101586, 101585}},
 {101451,  25, { 10915,  10903, 101585, 101586}},
 {101452,  25, { 10750,  10738, 101588, 101589}},
 {101453,  25, { 10737,  10749, 101589, 101588}},
 {101454,  25, { 10749,  10761, 101590, 101589}},
 {101455,  25, { 10762,  10750, 101589, 101590}},
 {101456,  25, { 10597,  10585, 101592, 101593}},
 {101457,  25, { 10584,  10596, 101593, 101592}},
 {101458,  25, { 10596,  10608, 101594, 101593}},
 {101459,  25, { 10609,  10597, 101593, 101594}},
 {101460,  25, { 10444,  10432, 101596, 101597}},
 {101461,  25, { 10431,  10443, 101597, 101596}},
 {101462,  25, { 10443,  10455, 101598, 101597}},
 {101463,  25, { 10456,  10444, 101597, 101598}},
 {101505,  69, {  8220,   8221,   8248,   8247}},
 {101508,  69, {  8221,   8222,   8249,   8248}},
 {101510,  68, {  8222,   8223,   8250,   8249}},
 {101512,  68, {  8223,   8224,   8251,   8250}},
 {101514,  68, {  8224,   8225,   8252,   8251}},
 {101516,  68, {  8225,   8226,   8253,   8252}},
 {101518,  68, {  8226,   8227,   8254,   8253}},
 {101520,  68, {  8227,   8228,   8255,   8254}},
 {101538,  69, {  8247,   8248,   8275,   8274}},
 {101540,  69, {  8248,   8249,   8276,   8275}},
 {101541,  68, {  8249,   8250,   8277,   8276}},
 {101542,  68, {  8250,   8251,   8278,   8277}},
 {101543,  68, {  8251,   8252,   8279,   8278}},
 {101544,  68, {  8252,   8253,   8280,   8279}},
 {101545,  68, {  8253,   8254,   8281,   8280}},
 {101546,  68, {  8254,   8255,   8282,   8281}},
 {101556,  69, {  8274,   8275,   8302,   8301}},
 {101558,  69, {  8275,   8276,   8303,   8302}},
 {101559,  68, {  8276,   8277,   8304,   8303}},
 {101560,  68, {  8277,   8278,   8305,   8304}},
 {101561,  68, {  8278,   8279,   8306,   8305}},
 {101562,  68, {  8279,   8280,   8307,   8306}},
 {101563,  68, {  8280,   8281,   8308,   8307}},
 {101564,  68, {  8281,   8282,   8309,   8308}},
 {101574,  69, {  8301,   8302,   8329,   8328}},
 {101576,  69, {  8302,   8303,   8330,   8329}},
 {101577,  68, {  8303,   8304,   8331,   8330}},
 {101578,  68, {  8304,   8305,   8332,   8331}},
 {101579,  68, {  8305,   8306,   8333,   8332}},
 {101580,  68, {  8306,   8307,   8334,   8333}},
 {101581,  68, {  8307,   8308,   8335,   8334}},
 {101582,  68, {  8308,   8309,   8336,   8335}},
 {101592,  69, {  8328,   8329,   8356,   8355}},
 {101594,  69, {  8329,   8330,   8357,   8356}},
 {101595,  68, {  8330,   8331,   8358,   8357}},
 {101596,  68, {  8331,   8332,   8359,   8358}},
 {101597,  68, {  8332,   8333,   8360,   8359}},
 {101598,  68, {  8333,   8334,   8361,   8360}},
 {101599,  68, {  8334,   8335,   8362,   8361}},
 {101600,  68, {  8335,   8336,   8363,   8362}},
 {101610,  69, {  8355,   8356,   8383,   8382}},
 {101612,  69, {  8356,   8357,   8384,   8383}},
 {101613,  68, {  8357,   8358,   8385,   8384}},
 {101614,  68, {  8358,   8359,   8386,   8385}},
 {101615,  68, {  8359,   8360,   8387,   8386}},
 {101616,  68, {  8360,   8361,   8388,   8387}},
 {101617,  68, {  8361,   8362,   8389,   8388}},
 {101618,  68, {  8362,   8363,   8390,   8389}},
 {101628,  69, {  8382,   8383,   8410,   8409}},
 {101630,  69, {  8383,   8384,   8411,   8410}},
 {101631,  68, {  8384,   8385,   8412,   8411}},
 {101632,  68, {  8385,   8386,   8413,   8412}},
 {101633,  68, {  8386,   8387,   8414,   8413}},
 {101634,  68, {  8387,   8388,   8415,   8414}},
 {101635,  68, {  8388,   8389,   8416,   8415}},
 {101636,  68, {  8389,   8390,   8417,   8416}},
 {101646,  69, {  8409,   8410,   8437,   8436}},
 {101648,  69, {  8410,   8411,   8438,   8437}},
 {101649,  68, {  8411,   8412,   8439,   8438}},
 {101650,  68, {  8412,   8413,   8440,   8439}},
 {101651,  68, {  8413,   8414,   8441,   8440}},
 {101652,  68, {  8414,   8415,   8442,   8441}},
 {101653,  68, {  8415,   8416,   8443,   8442}},
 {101654,  68, {  8416,   8417,   8444,   8443}},
 {101664,  69, {  8436,   8437,   8464,   8463}},
 {101667,  69, {  8437,   8438,   8465,   8464}},
 {101669,  68, {  8438,   8439,   8466,   8465}},
 {101671,  68, {  8439,   8440,   8467,   8466}},
 {101673,  68, {  8440,   8441,   8468,   8467}},
 {101675,  68, {  8441,   8442,   8469,   8468}},
 {101677,  68, {  8442,   8443,   8470,   8469}},
 {101679,  68, {  8443,   8444,   8471,   8470}},
 {101699,  68, {  8228,   8490,   8514,   8255}},
 {101701,  68, {  8490,   8491,   8515,   8514}},
 {101703,  68, {  8491,   8492,   8516,   8515}},
 {101705,  68, {  8492,   8493,   8517,   8516}},
 {101707,  68, {  8493,   8494,   8518,   8517}},
 {101709,  68, {  8494,   8495,   8519,   8518}},
 {101711,  68, {  8495,   8496,   8520,   8519}},
 {101713,  68, {  8496,   8497,   8521,   8520}},
 {101730,  68, {  8255,   8514,   8538,   8282}},
 {101731,  68, {  8514,   8515,   8539,   8538}},
 {101732,  68, {  8515,   8516,   8540,   8539}},
 {101733,  68, {  8516,   8517,   8541,   8540}},
 {101734,  68, {  8517,   8518,   8542,   8541}},
 {101735,  68, {  8518,   8519,   8543,   8542}},
 {101736,  68, {  8519,   8520,   8544,   8543}},
 {101737,  68, {  8520,   8521,   8545,   8544}},
 {101746,  68, {  8282,   8538,   8562,   8309}},
 {101747,  68, {  8538,   8539,   8563,   8562}},
 {101748,  68, {  8539,   8540,   8564,   8563}},
 {101749,  68, {  8540,   8541,   8565,   8564}},
 {101750,  68, {  8541,   8542,   8566,   8565}},
 {101751,  68, {  8542,   8543,   8567,   8566}},
 {101752,  68, {  8543,   8544,   8568,   8567}},
 {101753,  68, {  8544,   8545,   8569,   8568}},
 {101762,  68, {  8309,   8562,   8586,   8336}},
 {101763,  68, {  8562,   8563,   8587,   8586}},
 {101764,  68, {  8563,   8564,   8588,   8587}},
 {101765,  68, {  8564,   8565,   8589,   8588}},
 {101766,  68, {  8565,   8566,   8590,   8589}},
 {101767,  68, {  8566,   8567,   8591,   8590}},
 {101768,  68, {  8567,   8568,   8592,   8591}},
 {101769,  68, {  8568,   8569,   8593,   8592}},
 {101778,  68, {  8336,   8586,   8610,   8363}},
 {101779,  68, {  8586,   8587,   8611,   8610}},
 {101780,  68, {  8587,   8588,   8612,   8611}},
 {101781,  68, {  8588,   8589,   8613,   8612}},
 {101782,  68, {  8589,   8590,   8614,   8613}},
 {101783,  68, {  8590,   8591,   8615,   8614}},
 {101784,  68, {  8591,   8592,   8616,   8615}},
 {101785,  68, {  8592,   8593,   8617,   8616}},
 {101794,  68, {  8363,   8610,   8634,   8390}},
 {101795,  68, {  8610,   8611,   8635,   8634}},
 {101796,  68, {  8611,   8612,   8636,   8635}},
 {101797,  68, {  8612,   8613,   8637,   8636}},
 {101798,  68, {  8613,   8614,   8638,   8637}},
 {101799,  68, {  8614,   8615,   8639,   8638}},
 {101800,  68, {  8615,   8616,   8640,   8639}},
 {101801,  68, {  8616,   8617,   8641,   8640}},
 {101810,  68, {  8390,   8634,   8658,   8417}},
 {101811,  68, {  8634,   8635,   8659,   8658}},
 {101812,  68, {  8635,   8636,   8660,   8659}},
 {101813,  68, {  8636,   8637,   8661,   8660}},
 {101814,  68, {  8637,   8638,   8662,   8661}},
 {101815,  68, {  8638,   8639,   8663,   8662}},
 {101816,  68, {  8639,   8640,   8664,   8663}},
 {101817,  68, {  8640,   8641,   8665,   8664}},
 {101826,  68, {  8417,   8658,   8682,   8444}},
 {101827,  68, {  8658,   8659,   8683,   8682}},
 {101828,  68, {  8659,   8660,   8684,   8683}},
 {101829,  68, {  8660,   8661,   8685,   8684}},
 {101830,  68, {  8661,   8662,   8686,   8685}},
 {101831,  68, {  8662,   8663,   8687,   8686}},
 {101832,  68, {  8663,   8664,   8688,   8687}},
 {101833,  68, {  8664,   8665,   8689,   8688}},
 {101842,  68, {  8444,   8682,   8706,   8471}},
 {101844,  68, {  8682,   8683,   8707,   8706}},
 {101846,  68, {  8683,   8684,   8708,   8707}},
 {101848,  68, {  8684,   8685,   8709,   8708}},
 {101850,  68, {  8685,   8686,   8710,   8709}},
 {101852,  68, {  8686,   8687,   8711,   8710}},
 {101854,  68, {  8687,   8688,   8712,   8711}},
 {101856,  68, {  8688,   8689,   8713,   8712}},
 {101875,  70, {  8497,   8730,   8736,   8521}},
 {101877,  70, {  8730,   8731,   8737,   8736}},
 {101882,  70, {  8521,   8736,   8742,   8545}},
 {101883,  70, {  8736,   8737,   8743,   8742}},
 {101886,  70, {  8545,   8742,   8748,   8569}},
 {101887,  70, {  8742,   8743,   8749,   8748}},
 {101890,  70, {  8569,   8748,   8754,   8593}},
 {101891,  70, {  8748,   8749,   8755,   8754}},
 {101894,  70, {  8593,   8754,   8760,   8617}},
 {101895,  70, {  8754,   8755,   8761,   8760}},
 {101898,  70, {  8617,   8760,   8766,   8641}},
 {101899,  70, {  8760,   8761,   8767,   8766}},
 {101902,  70, {  8641,   8766,   8772,   8665}},
 {101903,  70, {  8766,   8767,   8773,   8772}},
 {101906,  70, {  8665,   8772,   8778,   8689}},
 {101907,  70, {  8772,   8773,   8779,   8778}},
 {101910,  70, {  8689,   8778,   8784,   8713}},
 {101912,  70, {  8778,   8779,   8785,   8784}},
 {101919,  70, {  8731,   8790,   8796,   8737}},
 {101921,  70, {  8790,   8791,   8797,   8796}},
 {101926,  70, {  8737,   8796,   8802,   8743}},
 {101927,  70, {  8796,   8797,   8803,   8802}},
 {101930,  70, {  8743,   8802,   8808,   8749}},
 {101931,  70, {  8802,   8803,   8809,   8808}},
 {101934,  70, {  8749,   8808,   8814,   8755}},
 {101935,  70, {  8808,   8809,   8815,   8814}},
 {101938,  70, {  8755,   8814,   8820,   8761}},
 {101939,  70, {  8814,   8815,   8821,   8820}},
 {101942,  70, {  8761,   8820,   8826,   8767}},
 {101943,  70, {  8820,   8821,   8827,   8826}},
 {101946,  70, {  8767,   8826,   8832,   8773}},
 {101947,  70, {  8826,   8827,   8833,   8832}},
 {101950,  70, {  8773,   8832,   8838,   8779}},
 {101951,  70, {  8832,   8833,   8839,   8838}},
 {101954,  70, {  8779,   8838,   8844,   8785}},
 {101956,  70, {  8838,   8839,   8845,   8844}},
 {101963,  68, {  8791,   8850,   8886,   8797}},
 {101965,  68, {  8850,   8851,   8887,   8886}},
 {101967,  68, {  8851,   8852,   8888,   8887}},
 {101969,  68, {  8852,   8853,   8889,   8888}},
 {101971,  68, {  8853,   8854,   8890,   8889}},
 {101973,  68, {  8854,   8855,   8891,   8890}},
 {101975,  68, {  8855,   8856,   8892,   8891}},
 {101977,  68, {  8856,   8857,   8893,   8892}},
 {101979,  68, {  8857,   8858,   8894,   8893}},
 {101981,  68, {  8858,   8859,   8895,   8894}},
 {101983,  68, {  8859,   8860,   8896,   8895}},
 {101985,  68, {  8860,   8861,   8897,   8896}},
 {102010,  68, {  8797,   8886,   8922,   8803}},
 {102011,  68, {  8886,   8887,   8923,   8922}},
 {102012,  68, {  8887,   8888,   8924,   8923}},
 {102013,  68, {  8888,   8889,   8925,   8924}},
 {102014,  68, {  8889,   8890,   8926,   8925}},
 {102015,  68, {  8890,   8891,   8927,   8926}},
 {102016,  68, {  8891,   8892,   8928,   8927}},
 {102017,  68, {  8892,   8893,   8929,   8928}},
 {102018,  68, {  8893,   8894,   8930,   8929}},
 {102019,  68, {  8894,   8895,   8931,   8930}},
 {102020,  68, {  8895,   8896,   8932,   8931}},
 {102021,  68, {  8896,   8897,   8933,   8932}},
 {102034,  68, {  8803,   8922,   8958,   8809}},
 {102035,  68, {  8922,   8923,   8959,   8958}},
 {102036,  68, {  8923,   8924,   8960,   8959}},
 {102037,  68, {  8924,   8925,   8961,   8960}},
 {102038,  68, {  8925,   8926,   8962,   8961}},
 {102039,  68, {  8926,   8927,   8963,   8962}},
 {102040,  68, {  8927,   8928,   8964,   8963}},
 {102041,  68, {  8928,   8929,   8965,   8964}},
 {102042,  68, {  8929,   8930,   8966,   8965}},
 {102043,  68, {  8930,   8931,   8967,   8966}},
 {102044,  68, {  8931,   8932,   8968,   8967}},
 {102045,  68, {  8932,   8933,   8969,   8968}},
 {102058,  68, {  8809,   8958,   8994,   8815}},
 {102059,  68, {  8958,   8959,   8995,   8994}},
 {102060,  68, {  8959,   8960,   8996,   8995}},
 {102061,  68, {  8960,   8961,   8997,   8996}},
 {102062,  68, {  8961,   8962,   8998,   8997}},
 {102063,  68, {  8962,   8963,   8999,   8998}},
 {102064,  68, {  8963,   8964,   9000,   8999}},
 {102065,  68, {  8964,   8965,   9001,   9000}},
 {102066,  68, {  8965,   8966,   9002,   9001}},
 {102067,  68, {  8966,   8967,   9003,   9002}},
 {102068,  68, {  8967,   8968,   9004,   9003}},
 {102069,  68, {  8968,   8969,   9005,   9004}},
 {102082,  68, {  8815,   8994,   9030,   8821}},
 {102083,  68, {  8994,   8995,   9031,   9030}},
 {102084,  68, {  8995,   8996,   9032,   9031}},
 {102085,  68, {  8996,   8997,   9033,   9032}},
 {102086,  68, {  8997,   8998,   9034,   9033}},
 {102087,  68, {  8998,   8999,   9035,   9034}},
 {102088,  68, {  8999,   9000,   9036,   9035}},
 {102089,  68, {  9000,   9001,   9037,   9036}},
 {102090,  68, {  9001,   9002,   9038,   9037}},
 {102091,  68, {  9002,   9003,   9039,   9038}},
 {102092,  68, {  9003,   9004,   9040,   9039}},
 {102093,  68, {  9004,   9005,   9041,   9040}},
 {102106,  68, {  8821,   9030,   9066,   8827}},
 {102107,  68, {  9030,   9031,   9067,   9066}},
 {102108,  68, {  9031,   9032,   9068,   9067}},
 {102109,  68, {  9032,   9033,   9069,   9068}},
 {102110,  68, {  9033,   9034,   9070,   9069}},
 {102111,  68, {  9034,   9035,   9071,   9070}},
 {102112,  68, {  9035,   9036,   9072,   9071}},
 {102113,  68, {  9036,   9037,   9073,   9072}},
 {102114,  68, {  9037,   9038,   9074,   9073}},
 {102115,  68, {  9038,   9039,   9075,   9074}},
 {102116,  68, {  9039,   9040,   9076,   9075}},
 {102117,  68, {  9040,   9041,   9077,   9076}},
 {102130,  68, {  8827,   9066,   9102,   8833}},
 {102131,  68, {  9066,   9067,   9103,   9102}},
 {102132,  68, {  9067,   9068,   9104,   9103}},
 {102133,  68, {  9068,   9069,   9105,   9104}},
 {102134,  68, {  9069,   9070,   9106,   9105}},
 {102135,  68, {  9070,   9071,   9107,   9106}},
 {102136,  68, {  9071,   9072,   9108,   9107}},
 {102137,  68, {  9072,   9073,   9109,   9108}},
 {102138,  68, {  9073,   9074,   9110,   9109}},
 {102139,  68, {  9074,   9075,   9111,   9110}},
 {102140,  68, {  9075,   9076,   9112,   9111}},
 {102141,  68, {  9076,   9077,   9113,   9112}},
 {102154,  68, {  8833,   9102,   9138,   8839}},
 {102155,  68, {  9102,   9103,   9139,   9138}},
 {102156,  68, {  9103,   9104,   9140,   9139}},
 {102157,  68, {  9104,   9105,   9141,   9140}},
 {102158,  68, {  9105,   9106,   9142,   9141}},
 {102159,  68, {  9106,   9107,   9143,   9142}},
 {102160,  68, {  9107,   9108,   9144,   9143}},
 {102161,  68, {  9108,   9109,   9145,   9144}},
 {102162,  68, {  9109,   9110,   9146,   9145}},
 {102163,  68, {  9110,   9111,   9147,   9146}},
 {102164,  68, {  9111,   9112,   9148,   9147}},
 {102165,  68, {  9112,   9113,   9149,   9148}},
 {102178,  68, {  8839,   9138,   9174,   8845}},
 {102180,  68, {  9138,   9139,   9175,   9174}},
 {102182,  68, {  9139,   9140,   9176,   9175}},
 {102184,  68, {  9140,   9141,   9177,   9176}},
 {102186,  68, {  9141,   9142,   9178,   9177}},
 {102188,  68, {  9142,   9143,   9179,   9178}},
 {102190,  68, {  9143,   9144,   9180,   9179}},
 {102192,  68, {  9144,   9145,   9181,   9180}},
 {102194,  68, {  9145,   9146,   9182,   9181}},
 {102196,  68, {  9146,   9147,   9183,   9182}},
 {102198,  68, {  9147,   9148,   9184,   9183}},
 {102200,  68, {  9148,   9149,   9185,   9184}},
 {102227,  68, {  8861,   9210,   9246,   8897}},
 {102229,  68, {  9210,   9211,   9247,   9246}},
 {102231,  68, {  9211,   9212,   9248,   9247}},
 {102233,  68, {  9212,   9213,   9249,   9248}},
 {102235,  68, {  9213,   9214,   9250,   9249}},
 {102237,  68, {  9214,   9215,   9251,   9250}},
 {102239,  68, {  9215,   9216,   9252,   9251}},
 {102241,  68, {  9216,   9217,   9253,   9252}},
 {102243,  68, {  9217,   9218,   9254,   9253}},
 {102245,  69, {  9218,   9219,   9255,   9254}},
 {102247,  69, {  9219,   9220,   9256,   9255}},
 {102249,  69, {  9220,   9221,   9257,   9256}},
 {102276,  68, {  8897,   9246,   9282,   8933}},
 {102277,  68, {  9246,   9247,   9283,   9282}},
 {102278,  68, {  9247,   9248,   9284,   9283}},
 {102279,  68, {  9248,   9249,   9285,   9284}},
 {102280,  68, {  9249,   9250,   9286,   9285}},
 {102281,  68, {  9250,   9251,   9287,   9286}},
 {102282,  68, {  9251,   9252,   9288,   9287}},
 {102283,  68, {  9252,   9253,   9289,   9288}},
 {102284,  68, {  9253,   9254,   9290,   9289}},
 {102285,  69, {  9254,   9255,   9291,   9290}},
 {102286,  69, {  9255,   9256,   9292,   9291}},
 {102287,  69, {  9256,   9257,   9293,   9292}},
 {102302,  68, {  8933,   9282,   9318,   8969}},
 {102303,  68, {  9282,   9283,   9319,   9318}},
 {102304,  68, {  9283,   9284,   9320,   9319}},
 {102305,  68, {  9284,   9285,   9321,   9320}},
 {102306,  68, {  9285,   9286,   9322,   9321}},
 {102307,  68, {  9286,   9287,   9323,   9322}},
 {102308,  68, {  9287,   9288,   9324,   9323}},
 {102309,  68, {  9288,   9289,   9325,   9324}},
 {102310,  68, {  9289,   9290,   9326,   9325}},
 {102311,  69, {  9290,   9291,   9327,   9326}},
 {102312,  69, {  9291,   9292,   9328,   9327}},
 {102313,  69, {  9292,   9293,   9329,   9328}},
 {102328,  68, {  8969,   9318,   9354,   9005}},
 {102329,  68, {  9318,   9319,   9355,   9354}},
 {102330,  68, {  9319,   9320,   9356,   9355}},
 {102331,  68, {  9320,   9321,   9357,   9356}},
 {102332,  68, {  9321,   9322,   9358,   9357}},
 {102333,  68, {  9322,   9323,   9359,   9358}},
 {102334,  68, {  9323,   9324,   9360,   9359}},
 {102335,  68, {  9324,   9325,   9361,   9360}},
 {102336,  68, {  9325,   9326,   9362,   9361}},
 {102337,  69, {  9326,   9327,   9363,   9362}},
 {102338,  69, {  9327,   9328,   9364,   9363}},
 {102339,  69, {  9328,   9329,   9365,   9364}},
 {102354,  68, {  9005,   9354,   9390,   9041}},
 {102355,  68, {  9354,   9355,   9391,   9390}},
 {102356,  68, {  9355,   9356,   9392,   9391}},
 {102357,  68, {  9356,   9357,   9393,   9392}},
 {102358,  68, {  9357,   9358,   9394,   9393}},
 {102359,  68, {  9358,   9359,   9395,   9394}},
 {102360,  68, {  9359,   9360,   9396,   9395}},
 {102361,  68, {  9360,   9361,   9397,   9396}},
 {102362,  68, {  9361,   9362,   9398,   9397}},
 {102363,  69, {  9362,   9363,   9399,   9398}},
 {102364,  69, {  9363,   9364,   9400,   9399}},
 {102365,  69, {  9364,   9365,   9401,   9400}},
 {102380,  68, {  9041,   9390,   9426,   9077}},
 {102381,  68, {  9390,   9391,   9427,   9426}},
 {102382,  68, {  9391,   9392,   9428,   9427}},
 {102383,  68, {  9392,   9393,   9429,   9428}},
 {102384,  68, {  9393,   9394,   9430,   9429}},
 {102385,  68, {  9394,   9395,   9431,   9430}},
 {102386,  68, {  9395,   9396,   9432,   9431}},
 {102387,  68, {  9396,   9397,   9433,   9432}},
 {102388,  68, {  9397,   9398,   9434,   9433}},
 {102389,  69, {  9398,   9399,   9435,   9434}},
 {102390,  69, {  9399,   9400,   9436,   9435}},
 {102391,  69, {  9400,   9401,   9437,   9436}},
 {102406,  68, {  9077,   9426,   9462,   9113}},
 {102407,  68, {  9426,   9427,   9463,   9462}},
 {102408,  68, {  9427,   9428,   9464,   9463}},
 {102409,  68, {  9428,   9429,   9465,   9464}},
 {102410,  68, {  9429,   9430,   9466,   9465}},
 {102411,  68, {  9430,   9431,   9467,   9466}},
 {102412,  68, {  9431,   9432,   9468,   9467}},
 {102413,  68, {  9432,   9433,   9469,   9468}},
 {102414,  68, {  9433,   9434,   9470,   9469}},
 {102415,  69, {  9434,   9435,   9471,   9470}},
 {102416,  69, {  9435,   9436,   9472,   9471}},
 {102417,  69, {  9436,   9437,   9473,   9472}},
 {102432,  68, {  9113,   9462,   9498,   9149}},
 {102433,  68, {  9462,   9463,   9499,   9498}},
 {102434,  68, {  9463,   9464,   9500,   9499}},
 {102435,  68, {  9464,   9465,   9501,   9500}},
 {102436,  68, {  9465,   9466,   9502,   9501}},
 {102437,  68, {  9466,   9467,   9503,   9502}},
 {102438,  68, {  9467,   9468,   9504,   9503}},
 {102439,  68, {  9468,   9469,   9505,   9504}},
 {102440,  68, {  9469,   9470,   9506,   9505}},
 {102441,  69, {  9470,   9471,   9507,   9506}},
 {102442,  69, {  9471,   9472,   9508,   9507}},
 {102443,  69, {  9472,   9473,   9509,   9508}},
 {102458,  68, {  9149,   9498,   9534,   9185}},
 {102460,  68, {  9498,   9499,   9535,   9534}},
 {102462,  68, {  9499,   9500,   9536,   9535}},
 {102464,  68, {  9500,   9501,   9537,   9536}},
 {102466,  68, {  9501,   9502,   9538,   9537}},
 {102468,  68, {  9502,   9503,   9539,   9538}},
 {102470,  68, {  9503,   9504,   9540,   9539}},
 {102472,  68, {  9504,   9505,   9541,   9540}},
 {102474,  68, {  9505,   9506,   9542,   9541}},
 {102476,  69, {  9506,   9507,   9543,   9542}},
 {102478,  69, {  9507,   9508,   9544,   9543}},
 {102480,  69, {  9508,   9509,   9545,   9544}},
 {102481,  68, {  9545,   9509, 101640, 101631}},
 {102482,  68, {  9509,   9473, 101639, 101640}},
 {102483,  68, {  9473,   9437, 101638, 101639}},
 {102484,  68, {  9437,   9401, 101637, 101638}},
 {102485,  68, {  9401,   9365, 101636, 101637}},
 {102486,  68, {  9365,   9329, 101635, 101636}},
 {102487,  68, {  9329,   9293, 101634, 101635}},
 {102488,  68, {  9293,   9257, 101633, 101634}},
 {102489,  68, {  9257,   9221, 101632, 101633}},
 {102490,  68, {101629, 101648,   8436,   8463}},
 {102491,  68, {101648, 101647,   8409,   8436}},
 {102492,  68, {101647, 101646,   8382,   8409}},
 {102493,  68, {101646, 101645,   8355,   8382}},
 {102494,  68, {101645, 101644,   8328,   8355}},
 {102495,  68, {101644, 101643,   8301,   8328}},
 {102496,  68, {101643, 101642,   8274,   8301}},
 {102497,  68, {101642, 101641,   8247,   8274}},
 {102498,  68, {101641, 101630,   8220,   8247}}
};

#define NMATS 61

struct materialInfo {
   int    materialNumber;
   char  *materialName;
};

struct materialInfo materials[NMATS] = {
 { 1, "rigid body"},
 { 2, "rigid body"},
 { 3, "rigid body"},
 { 5, "Mooney-Rivlin Rubber"},
 { 6, "rigid body"},
 { 7, "rigid body"},
 { 8, "elastic"},
 { 9, "rigid body"},
 {10, "rigid body"},
 {11, "rigid body"},
 {12, "rigid body"},
 {13, "rigid body"},
 {14, "rigid body"},
 {15, "rigid body"},
 {16, "rigid body"},
 {17, "rigid body"},
 {18, "rigid body"},
 {19, "rigid body"},
 {20, "Low Density Urethane Foam"},
 {21, "elastic"},
 {22, "Low Density Urethane Foam"},
 {23, "Low Density Urethane Foam"},
 {24, "Low Density Urethane Foam"},
 {25, "elastic"},
 {26, "elastic"},
 {27, "elastic"},
 {28, "elastic"},
 {29, "Mooney-Rivlin Rubber"},
 {30, "Mooney-Rivlin Rubber"},
 {31, "Mooney-Rivlin Rubber"},
 {32, "elastic"},
 {33, "Low Density Urethane Foam"},
 {34, "elastic"},
 {35, "rigid body"},
 {36, "rigid body"},
 {37, "rigid body"},
 {38, "elastic"},
 {39, "rigid body"},
 {40, "Low Density Urethane Foam"},
 {41, "elastic"},
 {42, "rigid body"},
 {43, "Low Density Urethane Foam"},
 {44, "elastic"},
 {45, "elastic"},
 {46, "rigid body"},
 {47, "rigid body"},
 {48, "rigid body"},
 {49, "rigid body"},
 {50, "rigid body"},
 {51, "Low Density Urethane Foam"},
 {52, "elastic"},
 {53, "rigid body"},
 {54, "rigid body"},
 {57, "elastic"},
 {58, "elastic"},
 {59, "lkbone"},
 {60, "lkbone"},
 {68, "elastic"},
 {69, "elastic"},
 {70, "elastic"},
 {72, "Mooney-Rivlin Rubber"}
};

int main (int argc, char **argv)
{
    int       i, j, k;
    int       driver = DB_PDB;

    for (i = 0; i < argc; i++)
    {
        if (strcmp(argv[i], "-driver") == 0)
        {
            i++;

            if (strcmp(argv[i], "DB_HDF5") == 0)
            {
                driver = DB_HDF5;
            }
            else if (strcmp(argv[i], "DB_PDB") == 0)
            {
                driver = DB_PDB;
            }
            else
            {
               fprintf(stderr,"Uncrecognized driver name \"%s\"\n",
                   argv[i]);
            }
        }
    }

    //
    // Create the coordinate arrays.
    //
    int       nnodes = NNODES;
    double    coord0[NNODES], coord1[NNODES], coord2[NNODES];
    double   *coords[3];
    char     *coordnames[3] = {"xcoords", "ycoords", "zcoords"};

    for (i = 0; i < NNODES; i++)
    {
         coord0[i] = nodes[i].x;
         coord1[i] = nodes[i].y;
         coord2[i] = nodes[i].z;
    }
    coords[0] = coord0;
    coords[1] = coord1;
    coords[2] = coord2;

    //
    // Create the connectivity and material information.
    //
    int       ndims = 3;
    int       nzones = NHEXES + NBEAMS + NSHELLS;
    int       lznodelist = NHEXES * 8 + NBEAMS * 2 + NSHELLS * 4;
    int       znodelist[NHEXES * 8 + NBEAMS * 2 + NSHELLS * 4];
    int       origin = 0;
    int       zshapetype[3] = {DB_ZONETYPE_HEX, DB_ZONETYPE_BEAM,
                               DB_ZONETYPE_QUAD};
    int       zshapesize[3] = {8, 2, 4};
    int       zshapecnt[3] = {NHEXES, NBEAMS, NSHELLS};
    int       nzshapes = 3;
    int       matlist[NHEXES + NBEAMS + NSHELLS];
    int       matnos[NMATS];

    j = 0;
    k = 0;
    for (i = 0; i < NHEXES; i++)
    {
        znodelist[j++] = hexes[i].nodes[0];
        znodelist[j++] = hexes[i].nodes[1];
        znodelist[j++] = hexes[i].nodes[2];
        znodelist[j++] = hexes[i].nodes[3];
        znodelist[j++] = hexes[i].nodes[4];
        znodelist[j++] = hexes[i].nodes[5];
        znodelist[j++] = hexes[i].nodes[6];
        znodelist[j++] = hexes[i].nodes[7];
        matlist[k++] = hexes[i].materialNumber;
    }
    for (i = 0; i < NBEAMS; i++)
    {
        znodelist[j++] = beams[i].nodes[0];
        znodelist[j++] = beams[i].nodes[1];
        matlist[k++] = beams[i].materialNumber;
    }
    for (i = 0; i < NSHELLS; i++)
    {
        znodelist[j++] = shells[i].nodes[0];
        znodelist[j++] = shells[i].nodes[1];
        znodelist[j++] = shells[i].nodes[2];
        znodelist[j++] = shells[i].nodes[3];
        matlist[k++] = shells[i].materialNumber;
    }

    //
    // Remap the zonelist.
    //
    for (i = 0; i < lznodelist; i++)
    {
        int low  = 0;
        int high = NNODES - 1;
        int mid;
        while (low <= high)
        {
            mid = (low + high) / 2;
            if (znodelist[i] < nodes[mid].nodeNumber)
                high = mid - 1;
            else if (znodelist[i] > nodes[mid].nodeNumber)
                low = mid + 1;
            else
                break;
        }
        if (znodelist[i] == nodes[mid].nodeNumber)
            znodelist[i] = mid;
        else
            cerr << "Didn't find node " << znodelist[i] << endl;
    }

    //
    // Extract the material numbers.
    //
    for (i = 0; i < NMATS; i++)
    {
        matnos[i] = materials[i].materialNumber;
    }

    //
    // Create the file.
    //
    DBfile    *dbfile;

    dbfile = DBCreate("sid97.silo", DB_CLOBBER, DB_LOCAL,
                      "unstructured mesh of globe", driver);

    //
    // Create a zonelist and then call DBCalcExternalFacelist to calculate
    // the face list that goes with it.
    //
    DBfacelist *fl;

    DBPutZonelist2(dbfile, "zl", nzones, ndims, znodelist, lznodelist, origin,
                   0, 0, zshapetype, zshapesize, zshapecnt, nzshapes, NULL);
 
    fl = DBCalcExternalFacelist2(znodelist, nnodes, 0, 0, origin, zshapetype,
                                 zshapesize, zshapecnt, nzshapes, matlist, 1);
 
    DBPutFacelist(dbfile, "fl", fl->nfaces, fl->ndims, fl->nodelist,
                  fl->lnodelist, fl->origin, fl->zoneno, fl->shapesize,
                  fl->shapecnt, fl->nshapes, fl->types, fl->typelist,
                  fl->ntypes);
 
    DBFreeFacelist(fl);
 
    int        cycle = 0;
    float      time = 0.;
    DBoptlist *optlist = DBMakeOptlist(10);
    DBAddOption(optlist, DBOPT_CYCLE, &cycle);
    DBAddOption(optlist, DBOPT_TIME, &time);
 
    DBPutUcdmesh(dbfile, "mesh1", ndims, coordnames, (float **)coords, nnodes,
                 nzones, "zl", "fl", DB_DOUBLE, optlist);
 
    int       nmat = NMATS;
    int       dims[3];

    dims[0] = nzones;
    dims[1] = 1;
    dims[2] = 1;
    DBPutMaterial(dbfile, "mat1", "mesh1", nmat, matnos, matlist, dims, ndims,
                  NULL, NULL, NULL, NULL, 0, DB_FLOAT, optlist);

    DBClose(dbfile);
 
    return(0);
}
